import os
import sys
import pycifstar
from cryspy.scripts.cl_rhochi import RhoChi


def template_for_single():
    s_out = """

data_mono
_setup_wavelength      1.40
_setup_field           1.00

_diffrn_radiation_polarization -0.87
_diffrn_radiation_efficiency    1.00

_extinction_mosaicity 0.5523
_extinction_radius    77.7983
_extinction_model     gauss

_diffrn_orient_matrix_UB_11 6.59783
_diffrn_orient_matrix_UB_12 -6.99807
_diffrn_orient_matrix_UB_13 3.3663
_diffrn_orient_matrix_UB_21 2.18396
_diffrn_orient_matrix_UB_22 -2.60871
_diffrn_orient_matrix_UB_23 -9.5302
_diffrn_orient_matrix_UB_31 7.4657
_diffrn_orient_matrix_UB_32 6.94702
_diffrn_orient_matrix_UB_33 -0.18685

_phase_label  Fe3O4

loop_
_diffrn_refln_index_h
_diffrn_refln_index_k
_diffrn_refln_index_l
_diffrn_refln_fr
_diffrn_refln_fr_sigma
_diffrn_refln_fr_calc
0 0 8  0.64545 0.01329 0.6712661351345524
2 0 6  1.75682 0.04540 1.7645623312806638
0 2 6  1.67974 0.03711 1.6365392780135470
2 0 10 0.90510 0.02316 0.9305458781468400
0 2 10 0.92428 0.01791 0.9390707747959662
4 -4 0 0.71857 0.01026 0.6835720140623864
    """
    
    return s_out

def template_for_pd():
    s_out = """
data_pd
_setup_wavelength      0.84
_setup_field           1.00
_setup_offset_2theta -0.385404

_chi2_sum True
_chi2_diff False
_chi2_up False
_chi2_down False

_range_2theta_min     4.000
_range_2theta_max    80.000

loop_
_pd_background_2theta
_pd_background_intensity
 4.5 256.0
40.0 158.0
80.0  65.0

loop_
_exclude_2theta_min
_exclude_2theta_max
0.0 1.0

_diffrn_radiation_polarization -0.87
_diffrn_radiation_efficiency    1.00

_extinction_mosaicity 0.5523
_extinction_radius    77.7983
_extinction_model     gauss

_diffrn_radiation_polarization 0.0
_diffrn_radiation_efficiency   1.0

_pd_instr_resolution_u 16.9776
_pd_instr_resolution_v -2.8357
_pd_instr_resolution_w  0.5763
_pd_instr_resolution_x  0.0
_pd_instr_resolution_y  0.0

loop_
_phase_label
_phase_scale
_phase_igsize
Fe3O4 0.02381 0.0

loop_
_pd_meas_2theta
_pd_meas_intensity_up
_pd_meas_intensity_up_sigma
_pd_meas_intensity_down
_pd_meas_intensity_down_sigma
4.0 465.8 128.97 301.88 129.3
4.2 323.78 118.22 206.06 120.0
4.4 307.14 115.9 230.47 116.53
4.6 172.13 115.84 236.52 113.74
4.8 164.31 110.75 291.6 110.73
5.0 166.47 106.51 426.73 109.08
5.2 217.67 104.66 183.24 107.79
5.4 84.01 99.91 179.87 99.98
5.6 220.87 98.72 297.76 99.83
5.8 223.46 98.77 149.78 98.18
6.0 408.25 96.89 356.86 96.41
6.2 381.21 94.03 287.26 96.94
6.4 239.85 94.06 101.64 95.3
6.6 58.16 90.93 194.35 89.9
6.8 127.79 88.33 273.48 89.35
7.0 148.69 87.5 165.7 89.52
7.2 34.61 87.11 223.81 88.08
    """
    return s_out

def template_for_pd2d():
    s_out = """

data_powder2d
_setup_wavelength     0.840
_setup_field          1.000
_setup_offset_2theta -0.385404
_setup_offset_phi     0.000

_chi2_sum  True
_chi2_diff False
_chi2_up   False
_chi2_down False

_range_2theta_min     4.000
_range_2theta_max    80.000
_range_phi_min       -2.000
_range_phi_max       40.000

_pd2d_background_2theta_phi_intensity
;
     2   4.50        40.00        80.00      
 -3.00 -350.0        -369(1)     -400.0      
 41.00 -350.0        -404(2)     -400.0  
;

loop_
_exclude_2theta_min
_exclude_2theta_max
_exclude_phi_min
_exclude_phi_max
0.0 1.0 -5.0 -4.0

_pd2d_instr_reflex_asymmetry_p1 0.0
_pd2d_instr_reflex_asymmetry_p2 0.0
_pd2d_instr_reflex_asymmetry_p3 0.0
_pd2d_instr_reflex_asymmetry_p4 0.0

_diffrn_radiation_polarization 1.0
_diffrn_radiation_efficiency   1.0

_pd2d_instr_resolution_u 16.9776
_pd2d_instr_resolution_v -2.8357
_pd2d_instr_resolution_w  0.5763
_pd2d_instr_resolution_x  0.0
_pd2d_instr_resolution_y  0.0

loop_
_phase_label
_phase_scale
_phase_igsize
Fe3O4 0.02381 0.0

_pd2d_meas_2theta_phi_intensity_up
;
         241   4.00         4.20         4.40         4.60         4.80         5.00         5.20         5.40         5.60         5.80         6.00         6.20         6.40         6.60         6.80         7.00         7.20         7.40         7.60         7.80         8.00         8.20         8.40         8.60         8.80         9.00         9.20         9.40         9.60         9.80        10.00        10.20        10.40        10.60        10.80        11.00        11.20        11.40        11.60        11.80        12.00        12.20        12.40        12.60        12.80        13.00        13.20        13.40        13.60        13.80        14.00        14.20        14.40        14.60        14.80        15.00        15.20        15.40        15.60        15.80        16.00        16.20        16.40        16.60        16.80        17.00        17.20        17.40        17.60        17.80        18.00        18.20        18.40        18.60        18.80        19.00        19.20        19.40        19.60        19.80        20.00        20.20        20.40        20.60        20.80        21.00        21.20        21.40        21.60        21.80        22.00        22.20        22.40        22.60        22.80        23.00        23.20        23.40        23.60        23.80        24.00        24.20        24.40        24.60        24.80        25.00        25.20        25.40        25.60        25.80        26.00        26.20        26.40        26.60        26.80        27.00        27.20        27.40        27.60        27.80        28.00        28.20        28.40        28.60        28.80        29.00        29.20        29.40        29.60        29.80        30.00        30.20        30.40        30.60        30.80        31.00        31.20        31.40        31.60        31.80        32.00        32.20        32.40        32.60        32.80        33.00        33.20        33.40        33.60        33.80        34.00        34.20        34.40        34.60        34.80        35.00        35.20        35.40        35.60        35.80        36.00        36.20        36.40        36.60        36.80        37.00        37.20        37.40        37.60        37.80        38.00        38.20        38.40        38.60        38.80        39.00        39.20        39.40        39.60        39.80        40.00        40.20        40.40        40.60        40.80        41.00        41.20        41.40        41.60        41.80        42.00        42.20        42.40        42.60        42.80        43.00        43.20        43.40        43.60        43.80        44.00        44.20        44.40        44.60        44.80        45.00        45.20        45.40        45.60        45.80        46.00        46.20        46.40        46.60        46.80        47.00        47.20        47.40        47.60        47.80        48.00        48.20        48.40        48.60        48.80        49.00        49.20        49.40        49.60        49.80        50.00        50.20        50.40        50.60        50.80        51.00        51.20        51.40        51.60        51.80        52.00        52.20        52.40        52.60        52.80        53.00        53.20        53.40        53.60        53.80        54.00        54.20        54.40        54.60        54.80        55.00        55.20        55.40        55.60        55.80        56.00        56.20        56.40        56.60        56.80        57.00        57.20        57.40        57.60        57.80        58.00        58.20        58.40        58.60        58.80        59.00        59.20        59.40        59.60        59.80        60.00        60.20        60.40        60.60        60.80        61.00        61.20        61.40        61.60        61.80        62.00        62.20        62.40        62.60        62.80        63.00        63.20        63.40        63.60        63.80        64.00        64.20        64.40        64.60        64.80        65.00        65.20        65.40        65.60        65.80        66.00        66.20        66.40        66.60        66.80        67.00        67.20        67.40        67.60        67.80        68.00        68.20        68.40        68.60        68.80        69.00        69.20        69.40        69.60        69.80        70.00        70.20        70.40        70.60        70.80        71.00        71.20        71.40        71.60        71.80        72.00        72.20        72.40        72.60        72.80        73.00        73.20        73.40        73.60        73.80        74.00        74.20        74.40        74.60        74.80        75.00        75.20        75.40        75.60        75.80        76.00        76.20        76.40        76.60        76.80        77.00        77.20        77.40        77.60        77.80        78.00        78.20        78.40        78.60        78.80        79.00        79.20        79.40        79.60        79.80        80.00        80.20        80.40        80.60        80.80        81.00        81.20        81.40        81.60        81.80        82.00        82.20        82.40        82.60        82.80        83.00        83.20        83.40        83.60        83.80        84.00        84.20        84.40        84.60        84.80        85.00        85.20        85.40        85.60        85.80        86.00        86.20        86.40        86.60        86.80        87.00        87.20        87.40        87.60        87.80        88.00        88.20        88.40        88.60        88.80        89.00        89.20        89.40        89.60        89.80        90.00        90.20        90.40        90.60        90.80        91.00        91.20        91.40        91.60        91.80        92.00        92.20        92.40        92.60        92.80        93.00        93.20        93.40        93.60        93.80        94.00        94.20        94.40        94.60        94.80        95.00        95.20        95.40        95.60        95.80        96.00        96.20        96.40        96.60        96.80        97.00        97.20        97.40        97.60        97.80        98.00        98.20        98.40        98.60        98.80        99.00        99.20        99.40        99.60        99.80       100.00       100.20       100.40       100.60       100.80       101.00       101.20       101.40       101.60       101.80       102.00       102.20       102.40       102.60       102.80       103.00       103.20       103.40       103.60       103.80       104.00       104.20       104.40       104.60       104.80       105.00       105.20       105.40       105.60       105.80       106.00       106.20       106.40       106.60       106.80       107.00       107.20       107.40       107.60       107.80       108.00       108.20       108.40       108.60       108.80       109.00       109.20       109.40       109.60       109.80       110.00       110.20       110.40       110.60       110.80       111.00       111.20       111.40       111.60       111.80       112.00       112.20       112.40       112.60       112.80       113.00       113.20       113.40       113.60       113.80       114.00       114.20       114.40       114.60       114.80       115.00       115.20       115.40       115.60       115.80       116.00       116.20       116.40       116.60       116.80       117.00       117.20       117.40       117.60       117.80       118.00       118.20       118.40       118.60       118.80       119.00       119.20       119.40       119.60       119.80       120.00       120.20       120.40       120.60       120.80       121.00       121.20       121.40       121.60       121.80       122.00       122.20       122.40       122.60       122.80       123.00       123.20       123.40       123.60       123.80       124.00       124.20       124.40       124.60       124.80       125.00       125.20       125.40       125.60       125.80       126.00       126.20       126.40       126.60       126.80       127.00       127.20       127.40       127.60       127.80       128.00       128.20       128.40       128.60       128.80       129.00       129.20       129.40       129.60       129.80       130.00       130.20       130.40       130.60       130.80       131.00       131.20       131.40       131.60       131.80       132.00       132.20       132.40       132.60       132.80       133.00       133.20       133.40       133.60       133.80       134.00       134.20       134.40       134.60       134.80       135.00       135.20       135.40       135.60       135.80       136.00       136.20       136.40       136.60       136.80       137.00       137.20       137.40       137.60       137.80       138.00       138.20       138.40       138.60       138.80       139.00       139.20       139.40       139.60       139.80       140.00       140.20       140.40       140.60       140.80       141.00       141.20       141.40       141.60       141.80       142.00       142.20       142.40       142.60       142.80       143.00       143.20       143.40       143.60       143.80       144.00       144.20       144.40       144.60       144.80       145.00       145.20       145.40       145.60       145.80       146.00       146.20       146.40       146.60       146.80       147.00       147.20       147.40       147.60       147.80       148.00       148.20       148.40       148.60       148.80       149.00       149.20       149.40       149.60       149.80       150.00      
      -40.00    -331.2243    -441.7184    -165.4574   -255.04425   -266.21728   -345.16936   -495.28545   -445.75888   -551.80308   -360.60401   -499.51811   -371.54139   -543.69325   -260.78826   -559.88329   -376.08355    -586.5902          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -39.50   -367.06001   -334.89957    -19.03981   -368.40767   -388.36626   -387.11145   -460.28206   -257.13416   -504.58813   -293.77682   -538.43688   -351.89403   -553.04223    -348.1433   -620.61851   -525.51391   -465.59408          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -39.00   -468.35805   -361.54946   -227.15401   -493.43758    -402.6348   -366.44569   -407.45424     -222.144    -347.5662   -408.85764   -551.88214   -282.41867   -610.12649   -427.75663   -568.01653   -710.07063   -528.54547   -663.15661          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -38.50   -606.87762   -452.83686   -464.09166   -396.85362   -332.25873   -373.29971   -287.82815   -416.85128   -235.26827   -526.47234   -567.25814   -261.42718    -596.6799   -309.53023   -507.12002   -586.46382    -613.4283   -397.89457          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -38.00     -694.919   -444.04782   -690.42553    -399.0268   -351.31291   -411.50137   -321.51628   -591.22956   -469.20376   -552.73062   -384.39595   -318.94882   -485.05304   -370.98635   -586.14907    -313.8733   -579.58312   -405.05731   -361.05863          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -37.50   -776.64279    -389.6432   -592.42334   -500.83156   -443.00434   -445.60401    -433.7251   -625.85655   -650.49811   -533.33019   -383.43967   -313.73056   -427.70581   -405.10723    -673.1846   -251.04336   -568.76724   -350.39637   -437.89349          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -37.00    -759.9152   -403.08861   -406.31955   -595.27785   -580.63246   -426.02261    -461.6027   -503.81581   -677.59363   -574.19638    -581.5985   -212.35447   -560.65991    -313.5657   -718.18155    -276.7793    -586.0278   -397.06087   -597.00431          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -36.50   -622.32881   -468.05825    -294.5157   -571.29413   -648.17915   -446.46198   -408.01001    -393.2001   -638.52494   -640.76154   -627.90651   -153.34645    -752.2095   -432.56585   -532.28515   -268.03385   -536.83857   -454.56064   -563.62727   -611.19243          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -36.00   -408.22453   -433.81354   -287.31635   -566.13651   -611.57294   -403.37151   -385.10632    -338.8391   -637.83769   -488.58669   -442.06497    -387.6944   -747.82186    -573.8694   -371.46455   -401.37807   -498.82525   -527.58375   -478.61673    -564.5507          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -35.50   -195.69738   -458.10344   -343.98894   -508.44441   -464.27254   -259.97047   -315.34288   -406.54311   -522.92514   -432.05187   -327.82603   -538.89756   -623.01659    -613.1353   -424.10743    -487.4463   -605.57891   -500.91466   -560.62472   -649.01162   -552.29683          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -35.00    -51.61986    -545.7686   -356.82895   -445.88495   -477.19686   -301.56418    -239.9977   -453.38839   -404.87399   -473.11957   -560.21834   -513.21903   -596.29412   -633.56666   -436.05303   -458.23075   -536.97248   -561.02371   -609.25462   -712.55607   -342.57897          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -34.50   -141.07709   -652.32446   -323.16753     -339.882   -648.70799   -441.07129   -318.93268    -422.2607   -386.15297   -523.84226   -842.91082   -434.17602   -444.81724   -610.84596   -602.57222   -422.62862   -489.37714   -666.73081   -530.29763   -719.86539   -457.87854   -571.90117          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -34.00   -220.03948   -602.56132   -312.47363   -321.67966   -773.98057   -629.28022   -384.45695   -353.41355   -417.79793   -488.56292   -705.97128    -399.2673   -416.61658   -572.56335   -664.72808   -417.44663   -643.53203    -554.5594   -530.28122   -704.24267   -552.33726   -506.20309          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -33.50   -288.59472   -558.98594   -417.74055   -321.58814   -651.64213   -627.14768   -410.52984   -462.57485    -447.3001   -480.63344   -491.92209   -443.78678   -594.81614   -461.56083   -550.68127   -472.47918   -626.95133    -439.9089   -355.50723   -615.55997   -393.78178   -638.19608   -443.00848          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -33.00   -336.06024   -572.50214   -544.39902   -298.79376   -584.42852   -521.08092   -451.36269   -397.50536   -530.45097   -507.24479    -507.4618   -472.55293   -741.07073   -298.42998   -570.68737   -549.43261   -444.61918   -419.80141    -554.1978    -645.9341   -447.49782   -613.55254   -552.64233   -518.12843          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -32.50   -331.86914   -497.88154   -528.03735   -349.04939   -540.72379   -407.28995   -427.26277    -93.83066   -589.25829   -438.41163   -486.32543    -535.8371   -460.59374   -492.17678   -611.88387   -469.85735   -472.74598   -452.14986   -591.14929   -816.25275   -622.17885   -443.27293   -692.63625   -320.31898          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -32.00   -270.72193   -389.31544   -462.84953   -395.17792   -506.33182   -283.37453   -414.74294   -184.80903   -642.02256   -349.61173   -421.29349   -630.42046   -353.00543   -659.69204   -492.93884    -423.9467   -605.00916   -388.49291   -431.57343   -672.98854   -365.63105   -446.04473   -553.87593   -387.53381     -3.96384          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -31.50   -223.16184   -272.48133   -349.44672   -329.66483    -478.5868   -238.99251   -390.40347   -368.54314   -676.06483   -351.24934   -419.18783   -547.83406   -691.47091   -492.00603   -490.69783   -488.40325   -508.18989   -435.75403   -443.29438   -542.16394   -370.35205   -395.76928    -524.0925   -557.33743   -146.47015          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -31.00    -205.5983   -185.11152   -285.08914   -164.21877   -412.88074   -290.62387   -385.73567   -405.81849   -600.64954   -433.14762   -417.98895   -411.38821   -886.81628   -546.76445   -434.05341   -613.15566   -444.09018    -484.8501   -494.99523   -562.73795   -439.36367   -275.75726    -622.2061   -484.85684   -336.71388    307.80236          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -30.50   -188.40366    -153.4209   -223.79855     25.05848   -316.80252   -339.39147   -493.03484   -451.01281   -408.61022   -473.97068    -390.6445   -369.45991   -576.58755   -493.26698   -635.21881   -544.76023   -425.43296   -558.35591   -441.17032   -473.29966   -376.91358   -466.16387    -601.9044    -452.8139   -328.86306    271.92318    650.09466          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -30.00   -172.31842   -176.60273   -102.03358     69.61031   -215.39666   -338.60142   -562.14617   -534.76451   -197.55878   -511.36645   -362.26233   -431.43702   -496.88139   -520.69946   -792.45839   -498.38863   -399.01671   -593.43736   -411.07168   -498.45045   -187.13588    -612.5478    -494.2902   -469.97985   -158.13735    429.80731    836.76616          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -29.50   -233.23526   -267.61512    -46.57541    -93.31525   -316.13216   -271.06385   -515.17719   -442.74206   -265.36314   -356.61416   -406.65881   -579.87562   -530.25425   -494.21783   -508.85248      -455.37   -422.73093   -595.81462   -447.94958   -344.54629   -232.86313   -510.41903   -412.34874   -302.05751      12.2394    277.33651    689.99443     738.9267          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -29.00   -300.58291   -322.67892    -60.04166   -244.02947   -425.08041   -244.25011   -556.66338    -309.2195   -461.74312    -47.94922   -422.31716   -654.77153   -571.28936   -365.60515   -302.40438   -464.19834    -395.2134   -670.38188   -459.14392   -415.62785   -398.56996   -484.12775    -412.8805   -333.45816    -56.69861    311.24953    763.19721    845.60518     68.66308          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -28.50   -392.50715   -309.37017   -139.88248   -386.57886   -465.42732   -301.23442   -588.29631    -231.5878   -536.48998   -232.08162   -403.80988   -487.85322   -531.22732    -382.7279    -247.0571   -555.30849     -373.122   -383.11052   -494.28892   -548.70794   -501.99908   -511.37095   -495.86247   -583.82477   -102.51601    565.87924   1011.31753    845.25418    136.41228   -230.96445          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -28.00   -481.03301   -256.68149   -242.19558   -566.85254   -447.92252   -378.43821   -470.84401   -274.71514   -575.10674   -517.20175   -404.98917   -335.93778   -497.62829   -385.43935    -272.5263   -551.86565   -356.72928   -122.51841   -461.64744   -572.63679   -500.40013   -551.73643   -561.26322   -481.10714   -213.40482    317.87994   1086.13502   1236.26115    369.24809   -137.90658   -554.39932          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -27.50    -539.2661   -199.80133   -308.76114   -675.71086    -452.1373   -419.72488   -451.60622    -381.9162   -640.30276   -615.32105   -365.18411   -349.00219   -406.27529   -347.77394    -323.8505     -438.439   -487.46448    -216.3031   -551.79068    -628.7448   -389.93588    -524.7648   -469.60438   -490.19501    -215.6707    429.08306   1242.37708   1350.57577     325.0766   -256.33944   -506.98616          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -27.00   -570.45735   -147.47918   -336.13876   -640.00318   -501.15556    -458.6669    -507.4006   -457.92189   -579.48865   -503.28948   -271.83559   -393.47215   -377.93062   -261.26031   -269.11304   -491.09559   -581.69652    -449.5632   -669.07264   -470.12111   -510.61155   -418.74734   -325.75296   -308.78052   -230.16213    606.76961   1171.57101   1344.94729    365.23042   -188.54079    -564.3297   -514.04892          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -26.50   -457.21714   -169.14712   -301.76638     -459.476   -517.25996   -518.89258   -523.69824   -415.59614   -541.61359   -394.72301   -246.68958   -404.73742   -496.42293   -344.17138   -477.72003   -277.66956   -468.73696   -510.82461   -686.33304   -410.98215   -569.77094   -440.40881   -487.71767   -419.54657   -218.72788    298.11232   1161.44916   1492.03168    687.51374    -36.02519   -522.11864   -494.05792   -558.22482          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -26.00     -288.442   -237.00917   -248.28262   -309.59265    -445.8851    -515.8947   -481.68359    -336.5975   -459.97202    -488.9034   -407.45568   -446.22153   -625.58943   -383.77743   -453.98168   -287.37043   -445.91151    -540.6656   -538.49156   -539.08749   -494.41053   -529.19069   -575.24633   -596.79772    -99.85317    524.78461   1288.11576   1552.32418    792.30936     48.74909    -447.4424   -554.78279   -575.63018   -507.75648          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -25.50   -186.96627   -372.66701   -175.83428   -276.14775   -302.04117   -376.06108   -389.00321   -409.11446   -472.95195   -608.97781   -462.01755   -422.22953   -473.01798    -493.5833   -270.61913   -396.19649   -379.02224   -529.06775   -436.85632   -549.30543   -403.90206   -486.42989   -394.51002   -447.99212    -86.98998    787.48848   1402.61974   1507.16231    746.91627    -54.34565   -613.27709   -391.94952   -574.74673   -639.59741   -621.25733          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -25.00   -175.74198   -421.76979   -195.31584   -284.72185   -226.83398   -261.23424   -329.56312   -483.76355   -418.04186    -467.8067   -280.02224   -444.52593   -332.38941   -561.93739   -237.92484   -418.06963   -340.70157   -285.08475   -396.42622   -568.37168   -425.91301   -589.01445   -362.92377   -387.72491    -33.27346    861.90515   1356.33189   1359.19083    789.51717    -24.69598   -439.72445   -799.54465   -655.01098   -763.05254   -462.90125    -465.6649          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -24.50    -200.9337   -401.53785   -229.16139   -266.82776   -224.01494    -225.5067   -326.16491   -497.31296   -281.82176   -265.73159    -98.85155   -504.50887    -331.9552   -360.17886   -421.80279   -465.23813     -466.932   -240.10275    -459.6983   -548.63923   -595.43167   -662.27076    -373.9681   -277.67562   -114.76749    659.95401   1596.12972   1370.96482     871.7427    117.97017   -385.13543   -511.46633   -773.83797   -736.71442   -536.41256   -471.55833   -589.74948          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -24.00   -190.34467   -334.24455   -256.16447   -290.39668   -223.67861   -255.22946   -371.90943   -508.62961   -217.99218   -371.76789   -200.95111   -485.57954   -169.68764   -270.04439   -436.34686   -531.03651   -571.85854   -399.05572   -543.16342   -436.17984    -519.0454   -433.12325   -268.14248   -378.42724   -108.28579    677.73215   1759.82359   1745.60201    968.87832    100.53082   -272.42048    -432.3906   -528.55675   -489.35545   -691.69319   -632.58673   -607.13396   -608.95604          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -23.50   -175.81527   -357.41343   -215.21876   -328.40331   -245.45954   -316.66473   -462.34119   -447.91033   -240.45526   -509.43625    -297.6193   -401.66817   -181.93735   -470.67871   -408.41368   -456.19306   -571.81424    -536.3981   -359.70114   -322.31158   -538.43065   -363.93537   -311.06854   -418.67003      4.85538    682.91645   1824.06833   1788.62468   1277.04134    233.17623   -178.28546   -539.50015   -442.23576   -460.48499   -506.38873   -486.77463   -539.21959   -689.36002   -466.72615   -616.75097          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -23.00   -259.55455   -434.96223   -125.35733   -368.40736   -273.04909   -414.01436   -470.34942   -357.47267   -370.33984   -476.27406   -336.87682   -412.89039   -299.40723   -578.08023    -480.0293   -226.10696   -487.39175    -544.5686   -204.91323   -302.12731   -465.45047   -503.76247   -429.08615   -290.34916     83.58453   1000.85907   1599.36705   1832.92097   1306.92783    195.48641   -453.54763   -641.75788   -453.86456   -418.08198   -284.42126   -467.79044   -607.50297   -690.62441    -416.0345   -712.38749   -547.02699          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -22.50   -346.84322     -466.137   -100.48248   -399.09159   -322.25478   -461.27882   -379.69999   -510.65724   -611.10581   -404.89819   -358.89349   -453.91717     -318.285    -384.3474   -545.60681   -122.43557   -476.12008   -533.98625   -170.85152   -318.12125   -362.81627   -504.83893   -372.64521   -261.22398    -40.16326    649.20285   1754.85001   2113.67365   1253.68916    181.47305   -403.43226   -456.50973   -469.65844   -521.95558   -368.78734   -495.46736   -626.40785   -651.25883   -455.10761   -427.32499   -629.80658   -430.40916          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -22.00   -413.06127   -503.95266   -193.90906   -454.98544   -392.92598   -452.23975   -331.38825   -608.61683   -669.92698   -479.40077   -358.39777   -379.84521   -313.50731   -357.21592   -647.64524   -330.40767   -566.85421   -588.84922    -327.4453   -450.90041   -292.98445   -421.97976   -513.30663   -210.72087   -157.02546    543.20814   1861.40761   2020.23523    1379.2288    155.04957    -329.8937   -608.47981   -255.67662   -478.89678   -382.27699   -440.28727   -601.50347    -536.0187   -553.20677   -646.81036   -494.99086   -456.55646   -462.68018          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -21.50   -404.34894   -553.41708   -316.42263   -482.26019   -319.14311    -438.2397   -357.51417   -539.05101   -446.48443    -639.9822   -387.62201   -318.80535   -285.76055   -443.84743   -679.11424    -534.5184   -570.58091   -434.81436   -429.54108   -453.66677   -338.67733   -350.14957   -479.87063   -301.56481     20.91238    713.09076   1994.47139   2051.67694   1540.81279    242.46206   -218.72212   -459.84605   -534.57448   -372.72457   -280.56205   -499.51436   -410.96821   -482.34096   -521.49816   -652.13009    -622.7677   -512.77694   -679.60547    -604.3037   -648.32974          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -21.00    -453.7302   -607.49127   -321.95929   -459.48349   -206.60313   -388.98206   -360.07321   -431.28525   -333.07713   -674.74127   -440.90016   -336.41948   -312.87517   -393.61631   -621.80586   -674.93756   -529.04106   -263.27978   -379.93836    -288.9744   -314.56607   -280.61333    -194.0241   -321.03524    -17.63987    627.34263   2031.98585   2335.63548   1518.01929    132.28718   -172.55163    -396.1334   -468.75417   -522.24367   -275.76625   -373.63397   -552.96149    -465.7595   -529.51111   -475.96793    -465.6473   -576.95096    -596.3065   -527.53801   -761.40344   -561.98354          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -20.50   -553.21942   -495.21845   -255.12402   -457.96448   -179.66189   -250.80841   -393.08168   -385.97112    -316.7391   -580.12173   -468.97905   -345.47585   -388.11258   -254.23278   -578.62797   -525.07456   -490.87092    -268.9452   -392.53379   -349.43488   -304.16918   -331.85035   -371.02059     -35.1782   -133.86877    502.41622   1941.29684   2558.68592   1266.94554    298.50582    -43.74669   -318.82502   -402.67195   -328.63083   -262.78336   -538.05292   -503.40832   -561.65637   -537.69355   -581.37581   -542.74819   -565.64012   -619.24982   -662.19076   -495.83439   -635.50675   -540.18317   -776.79732          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -20.00   -606.79421   -438.72108   -157.16875   -434.53714    -226.6931   -172.97728   -384.80177   -373.96956   -269.33908   -490.03615   -436.22778   -316.92729   -361.18153   -303.55977   -447.12106   -453.92164   -443.99342   -247.53571   -462.69582   -402.00064   -275.53877   -483.03754   -571.24412   -236.74744    -10.97578    557.02392    2002.7707   2467.89234   1178.40407    443.34834   -158.47648    -300.6645   -527.03618    -327.6897   -411.04948   -595.89165   -590.77682   -488.89375   -490.68093   -603.51512   -616.81918   -647.57329   -784.89674   -555.73327   -354.14997   -448.38799   -669.84628   -609.04252    -604.2092   -515.60185          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -19.50   -631.49847   -445.44888   -304.91009   -385.79853   -308.19238   -264.97152   -327.53682   -368.67216   -463.88129   -295.36973   -404.58978   -273.63699   -202.78679   -411.18313   -404.44784   -381.84595   -311.04687   -261.02966   -358.65176   -461.67227   -355.86783   -439.38585   -521.08555   -337.30847     -36.2368   1069.21651   2149.62273   2798.28849   1568.72538    375.33954   -229.78422   -283.04486   -463.26346   -442.31273   -585.08936   -544.36635   -482.67895   -571.81505   -436.63397   -578.61462    -549.4122   -483.04283   -678.88997   -431.57137   -564.86616   -469.21482   -730.61569   -671.48063   -694.19688    -593.3181   -669.78705          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -19.00   -578.32609    -474.8397    -484.7779    -344.9233   -385.27866   -420.95309   -277.00639    -436.2346   -691.17358   -241.90853   -285.44412    -140.1459   -177.40798   -351.80646   -369.04489   -357.43426   -185.68529   -449.12671   -278.82508   -517.66999   -709.20928   -441.54199   -520.40398   -210.35837    -54.88782    847.27556   2004.19748   2696.46814   1622.76837    317.80971   -224.58099   -493.73641   -254.93522   -473.28348   -423.48404   -467.02686   -464.80924   -350.75373   -412.26235   -552.03335   -658.74428   -489.28649   -558.45036   -659.43409   -593.18697   -670.08869   -586.43546   -681.20913    -596.7934   -682.45305   -770.83896   -435.44266   -622.01713          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -18.50   -432.51851   -486.20597   -583.23069   -405.53508   -402.80431   -542.35238   -300.29136   -401.72937   -615.95032   -369.65502   -167.89986   -108.11762   -588.44025   -301.27971   -452.91694   -432.88453   -196.47196   -431.09736   -233.29082   -459.20477     -526.074   -526.28468   -525.94767    -209.6761   -146.40976    709.46112   2300.80071   2704.24462   1828.95896    411.24806   -197.18082   -464.90652   -229.90793   -386.14666    -413.6424   -400.37207    -487.4396   -589.26029   -450.81429   -396.55362   -328.06205   -600.67109   -552.02019   -781.04819    -522.5856    -520.5826   -496.19344   -588.26939   -574.77095   -656.23634   -752.74232   -604.89561   -416.31278   -418.69106   -420.29739          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -18.00   -332.86202   -533.65358   -309.55158   -419.05725   -322.29201     -510.974   -312.96313   -292.42019   -436.78269   -406.76861   -282.41373   -158.83606    -674.8684   -427.04704   -625.34623   -449.39994   -500.05576   -367.82449   -355.53035   -374.01409   -404.49704   -503.07361   -402.00075   -283.49993   -216.65054    668.03204   2173.92683   3079.65504   1984.20775    409.66757     -96.2877   -384.99095   -309.62455   -410.48017   -421.85059   -429.65695   -424.81998   -459.23604   -392.30007    -479.4045   -493.31267   -594.10727   -610.71949   -471.56036   -506.43559   -616.25152   -471.01188   -790.06921   -583.64538   -660.01623   -462.78113   -484.53146   -552.11925   -646.30681    -557.2254   -753.99983   -384.31349          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -17.50   -290.36587   -536.42345    -79.68767   -380.14235   -205.88939   -396.15386   -309.27973     -201.229   -355.72711   -333.49536   -421.06313   -264.09238   -387.41181   -496.28377   -494.73323   -372.55456   -616.77883   -373.33541   -493.54812    -331.6429   -441.62122   -421.64958   -301.92812   -275.04414   -164.15944    908.39029   2222.69971   3202.76807   2045.54924    607.09756   -205.55561   -229.61036   -324.90902   -197.90688   -468.64865    -451.1599   -330.32431    -293.3827   -376.84239   -463.30543   -378.87886   -410.33955   -541.31334   -536.25346   -411.95866    -539.2182   -504.56395   -590.36193   -603.62945   -546.11522   -673.35682   -507.68546   -569.56135   -603.86347   -518.07806    -566.5664   -473.11166   -673.69077    -528.5936          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -17.00   -359.82652    -503.7039    -48.79794   -366.61365   -129.50547   -288.80768    -367.4624    -56.97612   -264.46477   -268.36546   -482.31643   -341.75912   -275.19979   -444.88832   -381.06445   -402.49438   -539.91808   -357.76077   -424.85638   -245.15137   -187.46616   -369.24525   -358.69321   -279.15168    -67.37581    933.86348   2477.52871   3157.75158   1714.40147    682.02458   -147.64086   -312.12561   -317.50388    -308.7061   -471.45014   -521.59861   -290.77207   -461.35879   -413.48838   -481.68377    -367.8263   -321.44286   -549.24456   -346.29982   -488.45709   -689.61131    -459.2413    -539.6499   -543.83537   -311.94094   -537.83337   -506.56981   -542.20025   -582.93972   -550.19619   -687.71778   -455.81168   -546.46462   -567.70941   -657.26478   -574.83304   -596.92529          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -16.50   -337.86934   -450.63983   -148.04354   -370.27593   -307.45004   -244.04189   -368.16572     89.16002    -199.3542   -259.20908    -522.9325   -450.34478   -132.20728   -301.03861   -501.95784   -357.43954   -219.56922   -337.50352   -310.52809   -263.29764   -144.89873   -237.41557   -314.55654   -289.53502     76.63247    910.99344   2358.76659   3057.00956   2134.63114    721.37631   -174.47567   -352.69878   -288.36762   -368.50331    -447.5537   -470.80382     -444.204   -366.69785   -418.27273   -467.61008   -348.10066   -484.19938   -414.72684   -467.59951    -494.8764   -533.68745   -457.35388   -653.04993   -603.02488   -558.06666   -611.79851   -583.46981   -476.94104   -607.54007   -596.27015   -552.95235   -437.40421   -483.34856   -438.09497   -576.83007   -689.61565   -529.40954   -444.93529   -559.71497          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -16.00   -233.71267   -430.04205   -207.05003    -282.1046   -441.90308   -331.97029   -333.90759     24.57569   -381.80778   -216.12986   -498.12452   -607.86116    -66.25713    -42.19653   -450.72746   -437.02777   -135.11912   -385.10642   -301.37786   -257.52429   -478.44425   -259.92187   -305.91908   -373.94402     104.7832    898.76087   2633.06103   3123.74023   2228.56043    948.13385   -140.12397   -430.85671   -317.54334   -190.84113   -494.62556   -420.54377     -490.176   -239.82219   -519.12653   -332.03227   -404.29989   -510.56043   -291.28519   -500.11054   -504.80145   -457.97686   -536.75654   -554.83227   -634.80928   -545.16813   -596.42589   -583.40872   -476.08494   -609.04363   -494.85942   -514.92431   -557.23465   -525.94608    -483.1442   -458.82912   -648.57901   -555.37727   -636.64168    -638.3438   -597.15592   -571.85381   -545.79891          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -15.50   -171.32104   -436.26043   -223.73203   -246.88328   -489.66577   -445.42093   -335.79988   -310.35079     -514.855   -221.21668   -446.93713    -492.8456   -113.01503   -150.20473    -99.64436   -533.40565   -322.65944   -361.55272   -361.88171   -359.83274   -273.19282   -354.55276   -475.97715   -369.67604       4.7018    982.47027   2678.37777   3266.65718    2291.6341    566.37109    -33.31655   -306.39845   -310.33022   -413.11994   -422.91657   -448.99664   -459.06435   -270.12316   -343.42019   -209.74463   -708.39005   -548.11877   -626.53815   -603.31996   -377.36307   -539.73569   -600.94667   -597.79803   -370.07306   -587.51268   -616.79193   -592.64854   -317.61832   -558.42966   -499.73019   -650.07743   -465.14713   -476.70517   -515.30822   -549.60812   -503.67528    -580.8382   -519.91026   -612.67942   -574.78188   -552.92123   -532.01759   -646.01084   -539.26865          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -15.00   -167.47619   -479.67958   -260.64123   -309.13451   -532.79931   -462.72838   -426.98727   -532.05635   -485.83145   -249.27365   -245.76512   -351.36499   -214.74394   -497.71618   -143.51769   -537.58294   -216.76291   -227.98188   -333.78889   -448.79897   -280.31436   -300.96928   -458.09705   -170.90991   -109.40009    977.04213   2451.86462   3413.42734   2291.82443    710.73075    -52.46202   -305.20901    -341.2244   -402.17814   -427.09547   -392.14517   -457.49551   -364.95763   -420.49827   -329.61381     -405.754   -342.95779   -435.27818   -568.86643    -451.4652    -574.0811   -578.34642   -578.71659   -393.08868    -486.4415    -639.3593   -595.09478    -446.9715   -506.87148   -655.58498   -589.97712   -568.75558   -599.57107   -548.11428   -487.99608   -334.74145   -732.59468    -247.6186   -582.01423   -464.24977   -641.55332   -562.80451   -481.75872   -528.33301   -303.17199   -281.63725    146.63378          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -14.50   -238.10164   -515.79521   -328.15266   -463.04229   -594.90294   -283.87655    -501.6268   -448.46157   -383.59519   -331.03933   -123.29186   -405.80546   -322.00975   -399.86546   -360.14898   -520.86153     68.20692   -372.64801   -362.72873   -402.53892   -449.05882   -411.44599   -188.43722    -244.5341    -39.57791    912.85009   2466.50312   3398.66018   2443.65288   1011.72891   -123.00878   -337.82402   -172.89826   -333.78517   -517.78918   -382.16698   -429.84351   -421.21564   -322.58519   -406.04976    -265.7888   -411.66864   -425.69128   -385.19699   -392.21196   -451.17321   -374.77144   -487.94222    -453.6501   -564.54834   -490.32535   -485.56783    -738.9919   -446.27962   -442.89492   -618.42295   -502.21853   -478.67949    -534.8055   -376.51486   -523.64489   -510.58231   -525.23985   -591.81168   -531.63307   -425.55027   -468.44865   -629.95461   -592.02366   -523.29054   -208.06101    412.04896    328.22991     32.30775   -269.18657   -632.36718          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -14.00   -437.36299   -517.95195   -446.02187   -630.62602   -383.16011    -99.64912   -264.58969   -412.76449   -256.98307   -366.80586   -275.28838   -477.36689   -447.26763   -360.17474   -414.16256    -457.5366   -200.31788   -475.22584   -276.59553   -356.22913   -420.29494   -461.99348   -314.60256   -325.72239     68.25439    971.02042   2471.48349   3456.04518    2607.3286   1069.77936    162.28084   -406.42524   -291.90852   -271.07344   -601.66115   -458.91414   -469.34279   -324.64711   -467.75807   -363.15178   -258.28499   -577.16551   -481.18187   -480.74186   -360.76244   -482.97306   -388.01516   -599.00785   -579.22202   -578.11516    -693.3071   -505.02563   -564.68349   -549.77622    -429.4909   -695.37661   -435.96879   -541.92199   -490.01385   -568.64394   -632.42687   -559.48828   -391.35684   -487.26688   -428.43017   -460.24688   -437.89166   -574.33426   -425.99542   -251.66739    142.82656     389.5409    435.27255    184.34523    -86.82842   -268.31996   -539.87096   -535.93936   -506.11438          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -13.50   -596.43674   -453.53953   -545.17944   -512.86892   -114.92379   -251.03458      -4.6103   -456.85959   -213.65185   -257.30074   -450.00022   -450.58899   -472.24855   -306.97139   -376.61123   -450.53788   -353.87696   -452.84141   -262.85785   -401.90376   -478.23406   -229.20174   -463.32344   -299.45602     12.03956   1146.14844   2528.40705   3539.21215   2550.76333   1022.17851      5.43162   -286.20854   -464.45822   -282.38344   -211.94208   -426.81341    -256.7966   -353.19394   -512.31438   -284.33611   -350.33651   -252.94297   -383.51386   -415.11858   -366.85019   -298.30269    -478.7926   -540.35685   -354.03383     -586.078   -416.39306    -436.2969   -490.70235   -497.56061   -367.58329   -461.23004   -438.97014   -419.57704   -259.57529   -501.84364   -547.82091   -561.49658   -614.42225   -487.73842   -651.38678    -494.4505   -629.93019      -460.03   -375.04437   -201.91573    148.74229    303.00359    643.81543    333.46978     -74.1432    -300.2303   -442.10476   -385.14937    -564.0679   -465.77821   -507.26639   -536.35205   -557.08704          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -13.00   -662.50193   -386.59749   -543.80671   -286.29934    -88.86235   -464.97127      5.27337   -488.58583   -372.45623   -176.39393   -472.92755   -353.22719   -354.20989   -297.70353   -293.02842   -366.76654   -389.10447   -448.15526   -350.61199   -407.79644   -527.27066     -244.106   -486.41751   -369.97243     11.57213   1070.69011   2672.70242   3557.84951   2744.27467   1008.32375     36.71359   -190.13019     -293.467   -367.25915    -229.9391   -367.82175   -477.33284   -398.71128   -346.90602   -415.48782   -391.56293   -298.49673   -563.11977   -399.53711   -405.14932   -354.29921   -396.00998   -478.40945   -307.48761   -298.90686   -427.31622   -355.51954     -686.633   -452.27223   -478.02811   -452.54477    -462.9564   -312.38565   -366.56663   -595.31662   -358.39281   -403.71785   -559.56322   -519.16378   -478.99002   -640.04016   -482.84765   -519.68945   -247.60649   -376.38978    352.45894    696.19405    691.06841    492.87578     58.49953    -192.7039   -389.15706    -343.6286   -466.83107   -732.23528   -550.51738     -409.376   -520.22852   -548.27994   -522.95282   -621.24169   -596.72286          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -12.50   -539.64641   -334.93681   -447.11596   -197.24121   -187.50821   -487.61357    -29.68517   -425.44907   -551.09749   -236.86076   -408.40611   -107.80357   -240.16683   -374.83886    -304.3637    -151.3321   -244.47574   -491.55747   -430.68745   -405.94426   -417.45437    -326.1382   -292.19215   -349.29226    157.65922   1017.46518    2801.1657   3697.91768    2936.0999   1089.68736    -13.98981   -129.47775   -267.99609   -414.29462   -370.49168   -354.31697   -434.57069   -265.27266   -409.47259   -412.46809   -624.63746   -433.71963   -591.05331   -369.05481    -418.0611   -589.03176   -455.81941   -414.80248   -368.86731   -294.14157   -465.83002   -332.82739   -637.22358   -520.36586   -523.83859   -560.82231   -456.72406   -429.59265   -379.49971   -193.45922   -379.43063   -461.28844   -447.97093   -366.59278   -366.48042   -383.06674   -492.72093   -420.35065   -464.48184   -157.02573    137.17638     617.0299    851.78784    553.67068    152.48224      0.95608   -294.56885   -376.26679   -452.24164   -502.67779   -578.56132   -435.53169   -545.18463    -441.4674   -547.96857   -554.98619   -640.58342   -471.03331   -410.06519   -202.52825    239.54014          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -12.00   -439.22327   -308.77364   -359.70109   -338.42674   -242.26283    -357.2031   -145.17081   -365.41669   -397.60269   -272.79324   -257.76272    -14.17524    -65.84964   -347.74563   -288.90531   -329.84708    -84.09927   -337.21403   -405.25171    -406.8227   -438.76929    -392.3246   -470.90008   -280.94304     105.4198    879.78477   2930.99347   3735.01159   2861.86831   1266.74911   -130.58409    -89.65722   -303.17623   -413.32416   -279.64649   -454.80274   -270.16933   -181.90817   -531.17824   -449.43783   -346.25517   -431.01157   -460.62074   -334.35062   -344.88639   -442.22528   -499.55072    -345.1012   -400.53746   -422.17195   -407.36632   -487.45104   -448.57976   -397.79545    -374.2395   -442.36475   -500.23276   -657.62799   -271.86589   -195.14333   -352.85855   -324.35559   -533.05845   -348.74619   -541.83334   -288.53551   -504.71993   -497.12232    -404.7127   -165.98299    130.78695    865.94101   1011.15766      787.964    263.38025     67.60801   -219.09963   -272.29603     -416.183   -516.93877   -431.50221   -524.42474   -463.68752   -405.09283   -490.40729   -408.60438   -587.30696   -399.42427   -306.87916    -56.12847    329.58158    618.14582    509.96618      3.07877   -208.28753   -373.05218          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -11.50   -398.76696   -299.58898    -363.6089   -470.10054   -256.45623   -260.49724   -355.48957    -335.1682   -252.44361   -423.07466   -182.31668   -262.86551    -36.34253   -244.04842   -432.83592   -527.98454   -257.31886   -218.46792   -140.94826    -386.4223   -525.46591   -403.12597   -502.69631   -172.57412    135.02958    732.52766   2688.45672   3609.00873   2958.93962   1407.17143     73.71632   -102.86782   -295.52485   -596.48704   -398.61402   -560.03726   -196.30798   -149.38925   -571.63795   -560.64695   -456.76116   -507.53588   -488.05383    -368.3284   -379.20335   -437.02624     -411.824   -310.76248   -257.85993   -427.56496   -423.27028   -340.87292   -477.33112   -414.99641   -496.08298   -568.08476   -170.87099   -528.24779   -386.08678   -304.77316   -423.30813   -386.53694   -403.33722   -563.77815   -318.27713   -580.33689    -332.3915   -523.11454   -406.00716   -222.60384    262.03106   1033.42111   1285.69381   1003.21813    288.68069     34.40845   -210.53632   -396.99034   -493.20124   -375.18008    -584.6177   -532.00287   -496.83243   -529.62739   -524.50623   -449.78968   -460.40935   -423.96347   -322.91488    -57.63618    594.32068    882.89419    762.27864    285.00464    -13.75007   -344.10105   -623.27954   -570.68863   -515.93863   -510.09458   -551.89684          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -11.00   -462.37265   -308.62189   -418.94806   -441.00608   -280.88942   -316.38209    -448.0153   -297.25747   -310.72808   -571.51033   -316.53172   -401.16693   -175.56855   -310.60135   -587.33078   -272.38924   -318.03274   -248.68797   -144.30533   -330.03232   -569.21067   -383.56814   -188.96478   -365.09898    194.85148    988.23079   2864.81136   3540.48135   2973.36162   1395.80874    234.99801     -166.158   -326.31061   -294.54984   -423.40953   -433.60676   -274.75619   -223.44417   -353.56978    -367.6942   -216.12518   -339.12582   -596.51419   -585.74715   -451.21776   -382.64692   -568.76703   -419.40187   -432.01203   -476.74743    -602.9242     -798.577   -451.99157   -479.06387    -396.2399   -483.21107   -343.44359   -413.01429   -387.80993   -340.58285   -384.46741   -350.65264   -315.48358    -363.9829   -451.56021   -562.15299   -528.42179   -434.94558   -422.55531    -233.3422    465.95496   1170.79631   1170.56276    999.72213     427.6404    205.63316    -20.80913   -142.49715   -459.37668   -560.14741   -564.50167   -634.28505   -387.41427   -406.18323   -478.50157   -414.53053   -451.98231   -403.52387   -285.65101    173.95017    583.86548    985.55193    735.43066    584.11923     66.91394   -213.81998   -483.02509   -419.74917   -385.80905   -394.04995   -304.01216   -325.26563    -390.3391   -396.93582   -488.76923    -530.9485   -595.83239          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -10.50   -541.63432   -337.72594   -490.66413   -355.59599   -336.65758   -392.79606   -332.63531   -261.60285    -389.3182     -450.283   -375.30397    -397.2283   -307.55362   -364.41052   -430.48335   -336.67082   -165.42702   -276.61665   -140.72457   -250.55206   -522.95784   -351.67349   -229.38071   -391.12608    239.75103    810.85899   3135.50582   3801.33619   3052.87668   1310.53093     75.10188   -207.86697   -284.30076   -396.23531   -314.64331   -526.23277   -330.27946   -256.54404   -428.29205   -407.73255   -322.91728   -581.35566   -527.73098   -575.00112   -426.18523   -421.26065   -640.83489    -287.3244   -377.29585   -424.34171   -553.78856   -510.58827   -488.76502   -353.52403   -299.69916   -354.65018   -455.47912   -325.38108   -217.73685   -378.65844     -344.778   -381.27506   -290.41206   -366.47943   -391.01734   -445.69906   -473.92104   -495.08711      -369.19    -62.22253    390.24963   1179.61594   1335.33727   1065.16663    569.67914    170.36553    -67.02344    -279.9228   -472.49663   -609.46636   -622.31303   -612.55156   -456.31892   -442.78245   -486.00086   -364.13622   -564.80435   -413.72506   -281.78088    121.60011    882.75586   1245.96314   1189.84072    610.07903    146.16584   -276.16612   -386.27177   -478.70176   -441.82384   -518.57199   -298.43316   -232.94172   -247.26981   -288.48734   -363.30384   -425.02474   -451.53397   -509.18171   -598.10657   -616.05994   -570.39926   -676.53154    -585.8089          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -10.00   -637.66038   -370.78613   -477.15502   -275.95943   -387.92935   -348.07827   -279.45811   -242.92229   -325.20496   -293.31874   -217.70263   -432.19986   -435.86209   -473.98885   -368.31183   -584.50662   -506.82396   -270.16934   -152.20903   -288.72612   -441.74044   -463.20146   -342.15009   -156.49202    277.12597     843.3279    2989.5484   4147.23531   3151.39701   1227.33807     160.9589   -178.26835   -163.05487   -426.20029   -207.33987   -282.33875   -405.20949   -386.29892   -220.23662   -405.42897   -392.53286   -495.78292   -411.52037   -457.96269   -223.54878   -406.53762    -478.4923   -270.13598   -423.05937   -392.69331   -327.13544   -318.62912   -507.20666   -433.14745   -406.50838   -497.28685   -533.34001   -167.08046   -311.10923   -403.66001   -456.59547   -422.25535   -290.81968   -407.89275   -541.70666   -530.55609   -468.26979   -313.66089   -249.95871    -60.49743    352.00559    957.70256   1694.38619   1268.11363    707.77105    218.06381   -105.06466   -233.82031   -456.39959   -469.60052   -545.66548   -319.49922   -448.92457   -407.78724   -380.43041   -369.56443     -474.254   -332.30443   -136.37969    288.89801    945.48409   1400.04614   1326.35691    913.90159    100.43401   -273.97884   -356.13419   -404.12509   -528.42327   -432.74589   -344.84952   -336.47992    -98.49879   -152.46604    -306.4479   -348.20291   -561.20261   -542.81455   -512.73562   -516.54831   -582.89399   -475.66001   -452.69381    -382.8401   -533.61748   -559.29617   -541.76275   -572.61012   -461.70105   -478.02462   -484.07772          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -9.50   -724.22385   -406.19165   -431.16843   -244.20856   -383.75221   -271.43685   -231.00503   -273.55148   -204.50808    -272.9182   -146.54319   -506.40319   -465.71113   -506.61051   -401.12947   -555.44011   -623.52146   -211.58086   -108.86104   -338.49928    -313.5772   -377.64217   -322.09596    -52.19344    339.06784   1088.12623   2584.80983    4102.1832    3320.6924   1125.67772    182.48785    -99.64406   -274.70195   -337.95587   -304.37527   -225.84767   -421.99733   -353.37993   -459.84124   -313.37732   -434.00296   -628.89607   -425.93787   -454.60877   -467.74904   -457.41711   -371.76783   -472.08434   -417.30389   -316.87895   -385.94525   -227.00339   -487.69165    -355.9449   -585.20577   -409.54383    -291.1575   -269.41128    -270.7004   -251.23238   -383.91934   -381.89801   -307.18066   -300.34179   -353.43589    -443.6969   -331.57833   -392.07914   -190.48539   -113.33318    454.26232   1212.21051   1615.58739   1338.86416    727.91218     60.62674    -71.77775   -284.07356   -414.38354   -531.40148   -441.96833   -542.76764   -428.12997   -307.75846   -265.71085   -292.27827   -479.98904    -353.2771   -212.12039    215.13528    966.45503   1642.03347   1463.86126   1036.99139    438.92071    -26.13971   -278.11459   -419.55217   -396.65672    -458.5914    -405.6838   -244.29823    -73.92649    -177.2597   -433.24885   -245.32282   -523.58923   -498.93611   -289.03714   -519.41088   -632.21711   -476.12122   -471.90442   -632.51218    -615.9222   -564.05519   -355.52177   -485.67906   -400.36291   -492.21908   -451.52042    -501.7416   -620.06338   -415.01601   -473.69568   -123.05216      8.47486   -236.44078   -445.69018          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -9.00   -788.38547   -424.40378   -404.69359   -223.32704   -302.44936   -276.04554   -112.80563   -367.50251   -161.79656   -262.11213   -227.04044   -353.40019    -447.7693   -370.51923   -357.38559   -507.50874   -361.78815   -289.71998    -93.68197   -325.12845   -397.46304   -350.83388   -166.19645   -185.53481    256.57182   1087.32654   2952.15103   4056.46883   3313.10252   1508.53182     54.15918   -356.78884   -243.00071   -242.58872   -294.35241    -92.82553   -257.74672   -499.05544    -308.1674   -436.42812   -357.03803   -195.96081   -431.63914   -323.15523   -548.88853   -491.81675   -441.06986    -524.3356   -422.80946   -380.56727   -500.90233   -274.65035   -363.35676   -298.42696   -486.67011   -355.07378   -393.43622   -349.53356    -372.8348    -244.1862    -345.7099   -441.04982   -274.28017   -314.57075   -354.70905   -466.64349   -423.32486   -439.66503   -490.31988   -148.04559    444.66941   1080.44071   1563.10553   1264.32934    746.33339    132.97131      2.89301    -54.42772    -85.85059   -430.35003   -474.85736   -483.37722    -183.3647    -314.3902   -563.58031   -422.56616   -516.84959   -495.54472    -92.98081    389.07358   1050.18067   1513.49052   1882.85712    1234.5303       447.13     39.86232   -301.28291   -434.95531   -360.93423   -359.25266    -171.0187   -138.52612   -221.42931   -195.36746   -303.79664   -386.93482     -480.011   -509.74216   -456.72443   -448.90262   -553.28479   -544.64574   -607.03466   -495.32023   -500.76715   -414.05082   -264.17645   -494.37822   -414.37702   -593.58418   -453.61674   -566.85553   -288.20841   -311.15634    -215.6862    -57.37254     49.24781    -38.27203   -164.52636   -271.74223   -305.34954   -611.91504   -542.04889    -464.5921   -517.15587   -311.47218   -334.66353    272.23775          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -8.50   -664.72189   -420.94875    -422.1859     -89.8534   -214.69064   -346.08966   -159.45178   -410.36121   -159.91363   -252.10636   -271.53825   -113.96943    -434.3519   -317.30154   -312.53693   -353.98443   -332.36437   -332.39066    -62.70839   -277.34539   -442.18883   -471.81606      -138.98   -224.93942     19.33526   1092.15899   2996.04948   4218.36855   3179.77168   1405.33605    -42.74927    -291.8933   -384.83857   -308.21389    -12.73515   -350.27912   -306.39346    -395.0095   -365.49983   -472.29174   -319.57907   -220.21891    -660.3791   -285.57181   -609.01384   -545.28465   -444.60931   -473.98889   -416.75878   -278.03846   -486.63066   -560.84685   -504.15906   -402.16708   -360.32336   -355.41977   -309.37377   -165.91152   -316.85108   -349.32985   -250.22381   -307.60441   -419.63615   -280.44406   -467.47176   -282.26372   -281.49703   -218.89509   -331.30078   -154.52927    332.74219   1193.35745   1728.15226   1276.33806    745.37171    236.92673     -3.44484   -128.62201   -212.61215   -474.81964   -377.23829   -377.17669   -248.34111   -293.24725    -420.5336   -349.02191   -452.26685   -117.91414   -180.09393    419.92527   1319.10678   1836.79613   2076.56183   1352.59025    604.51999     152.0467   -282.50844   -251.86409    -262.6361    -330.2043   -226.37109    -27.07259   -236.92791    -33.62316   -401.27621    -318.9621   -367.67762   -294.81074   -494.25983   -600.34216   -564.14321   -480.26161   -428.68653   -528.73121   -425.76355   -508.89419   -410.58437   -481.75722   -465.88432   -377.67946   -461.01505   -534.32185   -420.95898   -405.83395      -3.7218     14.43596     84.52004    -23.75832    -26.73785   -223.77783   -378.25129   -481.82365   -470.40289   -408.49624   -486.17355   -315.14139    -54.45409    190.79174    789.49686    738.78611    774.92299    507.78008    162.31171   -269.84435    -195.7805   -301.18978   -454.61743   -463.25684          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -8.00   -383.35512   -365.51641   -450.00434    123.86341   -286.75136   -402.31696   -258.44724   -310.27726   -176.86667   -262.36442   -317.60873   -314.52799    -444.2116   -347.64011   -341.05656   -401.86612   -320.18285   -168.13568   -139.46613   -297.11856   -393.53782   -346.97927    -147.0587   -205.12939    116.34503    986.53802   3030.17478   4285.34145   3476.59342   1395.91875      -18.722   -103.53266   -258.03322   -479.94334   -110.28573   -116.18398   -441.44327   -138.00977   -523.42215   -373.72832   -538.49727   -274.93104    -459.4781   -275.18616   -195.61281   -361.36175   -432.24348   -389.72506   -520.05453   -240.81305   -498.67051   -501.28342   -327.69579   -327.26893   -480.65069   -551.99736   -387.39042   -258.65474   -329.39865   -198.55718   -442.68221   -279.06916   -497.78731   -354.73649   -161.93928   -524.90682   -488.93209   -305.08955   -261.99045   -136.82965    626.89724   1363.22528   1661.30366   1429.93352    914.53197    268.74581      -54.413   -104.78057   -190.41482   -403.79362   -291.86755   -382.84198   -363.39233   -160.43617   -305.36066   -320.38232   -336.99164   -415.23203    -63.12649    428.94979   1321.05403   2007.59987   2079.86846   1456.89114    631.93331    -53.60518   -360.20121   -291.67645   -353.92494   -401.33984   -247.81723    -45.97147   -262.97495   -138.83083   -277.33406   -356.52431   -383.26365   -496.37649   -506.80022    -634.4546   -461.69389   -422.18935   -383.14725   -563.29876     -412.724   -386.42946   -448.61618     -524.759   -491.22969   -482.41249   -537.51483   -521.53276   -246.50771    -249.0557    -16.21068     94.38845     66.90528     37.54835   -180.97278   -310.05879   -331.50322   -408.25684   -602.14857   -500.54163   -380.35575   -369.75909     -1.47019    318.16966    859.12751   1248.40937   1009.66239     829.4369    342.33076    118.56919    -184.9825   -298.81479   -302.41655   -293.24299   -370.98414   -399.29304   -496.52553   -498.61333   -572.55645   -458.22123   -507.71173   -640.56262   -563.73675   -532.83713   -580.01428   -521.43772          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -7.50    -99.98236   -307.66524   -371.22912    297.77142   -404.39309   -411.42404   -301.08144   -196.52247   -203.43397   -331.83079   -403.16385   -624.97467   -437.21014   -383.09539   -362.49105   -533.83129   -179.09797   -129.60489   -196.14157    -432.8911   -257.37009   -356.96919   -205.80165   -198.24009    -39.75391    944.12314   3207.99368   4334.71951    3521.4352   1537.49315    314.99059   -315.30826   -252.40149   -501.28287    -238.7404   -238.11259   -358.30019   -311.66525   -418.12298   -443.16636   -484.60654   -286.67049   -352.96807   -344.68171    -283.9049   -369.58264   -450.13603     -450.169   -419.31151   -372.26437   -273.57736   -501.44076   -371.71969   -347.46519   -454.73595   -476.72801   -286.14111   -347.23476   -527.51111   -113.69226   -385.56162   -357.18876   -403.16107   -479.15412   -398.90209   -380.73535   -232.87464   -525.69992   -295.02716    -54.93213    456.48876    1369.6552   1711.46256   1591.68787    921.71927    246.11828     -38.6554    -68.14739   -164.86234    -214.7112   -322.00593   -417.65786   -294.20363   -365.94664   -320.79238   -388.01595   -245.74426   -299.13849   -168.97941    356.72614   1182.43764   1895.70903   2054.61429   1348.30821    869.45473     85.29073   -239.15391   -321.27817   -271.03316   -428.51282     -215.071    -55.21834    -78.98101   -113.94685    -234.7267   -284.87409   -313.53941   -510.49784   -489.58613   -542.17555   -549.05618    -471.4043   -456.43651   -377.98057   -526.79231   -509.52988   -329.56819   -406.08744   -367.85945   -350.72816   -399.50418   -477.71319   -319.40519   -481.01902     77.44582     96.57639    129.10805    176.73347     67.55609   -233.90571   -320.32896   -319.16641   -473.20225   -342.48941   -488.72379   -307.82236     42.46883    460.14796    864.00043   1289.46655   1121.13959   1129.48028    392.19841    203.59352    -80.88281   -119.05365   -300.59833   -370.51579   -382.13878   -488.58996   -455.38468   -427.85331   -666.32549   -555.92253   -757.59017   -489.48097   -471.18553   -592.19872   -418.51471   -645.98856   -484.82707   -528.49568   -533.67849   -517.67925   -303.68255   -542.59308   -437.57966   -349.29711   -405.16787    -537.2779   -649.71195   -619.22065   -711.59899   -527.04638          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -7.00   -173.52587   -266.45646   -277.15682    207.86617   -412.23387   -412.31958   -307.39869   -158.34428   -278.88292   -417.99121   -531.72101   -303.44379   -431.05751   -449.98359   -396.13707   -426.81953   -115.28836   -219.46576   -246.49513   -238.14252   -166.38329   -428.01579   -329.32473   -189.90637   -108.30635    986.67385    3302.6944   4258.44642   3594.66021   1643.42212    219.92107   -283.91866   -331.57195   -374.02979    -271.8581   -416.49235   -216.81278   -254.09503   -388.86046   -353.96065   -423.83427   -348.38715   -134.30752   -373.98681   -365.57726   -260.09017   -468.57391   -344.34712    -316.7928   -479.62911   -392.31926   -392.43116   -361.98991   -468.31039   -424.25802   -491.43563   -252.89846   -356.98034   -306.30183   -404.45411   -291.66093   -378.44798   -285.01164   -468.33102   -454.56299   -404.65012   -360.89613   -426.38282    -455.0396    166.07589    606.84317   1390.83447   1717.89191   1817.32086    998.86791    404.24149     70.58971    -45.75089      20.0149    -85.85726   -426.23775   -263.39943   -437.19989   -307.26294   -403.32501   -664.13903   -296.34746   -128.13062     81.71712    444.25074    1589.7925    2134.4295    2344.4452   1681.60945    776.88988      96.5539   -300.77373   -329.97558   -347.69773   -285.49165   -106.85538    -197.8469     71.45255   -179.49717   -261.25802   -383.39015    -407.1198   -452.54228   -392.50429    -509.6053   -352.72956   -355.78515   -535.80932   -474.38169   -460.60708   -432.38538   -445.03124   -353.82593   -354.38701   -415.77695    -460.5871   -362.15134   -413.91223   -232.23955     89.65409     197.1335    263.58967    228.48958    122.79289   -187.63394   -375.92056   -332.97099   -505.32363   -455.71667   -495.89647   -277.52812     21.56164    433.39118   1048.35726   1423.86373   1456.99051    1068.1494     425.4528     133.4382   -252.65791   -316.53514   -350.95673   -237.64955   -353.72868   -173.18017   -519.77112   -591.34822   -415.23841   -399.22398   -407.07625   -609.70782   -581.68679   -485.66591   -601.20651   -493.17281   -553.49329   -507.18477   -505.84873   -401.17289   -488.51427   -404.72886   -369.09941   -300.57089   -547.52241   -365.26979   -398.86147   -474.57717   -629.22722   -557.59124   -309.97943   -346.67516   -235.99697    -311.8246   -473.57004   -300.91344   -475.62161   -530.77229   -518.08341   -595.33845     -501.029   -447.72946   -505.70217   -648.87648   -660.06098   -637.40379   -601.80479          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -6.50   -444.64928   -290.38244   -233.45391     98.08723   -297.62068   -396.34988   -266.14775   -131.25122   -397.70801   -391.21623   -506.97839     -1.20085   -459.35009    -361.7116   -447.88582    -297.9878    -28.01511   -337.26006   -285.79014    -81.37505    -90.38714   -272.59581   -414.56628   -174.89849     69.66816   1347.26354   3225.83348   4055.92319   3741.94779    1493.8164    198.21538   -211.11798   -353.80709   -317.48761   -318.56892   -323.85158    -293.5385    -357.9456    -155.2939   -397.59389   -391.86369   -349.08606   -343.16204   -319.09009   -318.89948   -370.42122   -333.62996   -475.31355   -394.55079   -382.94615   -363.85367   -274.40976   -342.06282   -452.98856   -365.38902   -409.26098   -303.94368   -345.39625   -260.02181   -276.29057   -257.97649   -634.47576   -425.63662   -492.29939   -329.70706   -519.76111   -335.12294   -396.74802   -250.01876   -112.47343    539.28651   1332.73371   1833.48594   1590.57012   1017.35384    492.50715    128.73861    -15.74629   -142.10234    -49.64739   -306.83689   -426.18926   -176.29993   -500.55248   -287.11382   -455.34799   -305.68388   -126.96122    167.29887    566.77932   1561.68399   2428.76053    2551.5355   1642.41199    889.69292    271.00075    -177.7313   -415.75147   -441.06025   -305.17392   -293.74655     70.86961    -28.82058    -33.86609   -238.56869   -272.74643   -333.75854   -475.17137   -369.30451   -465.60337   -389.00649   -525.71231   -591.11881   -529.34632   -450.77634    -359.0127   -341.92964   -509.81475   -336.24244   -456.90273   -398.48498   -443.41903   -392.47908   -316.30995      4.87224     209.1711     261.8997    396.63093     70.79923    -49.55472   -224.37368   -256.19857   -418.42181    -511.3099    -301.5525   -205.81439      2.01304     510.2234    887.09887   1298.29196   1361.41238   1201.69404    504.05644    203.18171     36.35426   -153.33625   -358.04487   -402.56623    -211.3478   -268.76471   -340.83914   -475.68429   -554.14637   -431.33688   -546.64752   -580.70046   -477.33795   -538.88679    -594.9162   -617.16844   -477.15256   -492.97842   -455.96682   -353.56908    -320.9057   -347.23656   -347.89731   -330.84685   -295.42346   -392.54254   -553.85781   -475.80914   -515.19826   -394.49295   -422.58824   -143.33526   -222.42958   -204.99187   -390.48179   -405.10743   -328.12075     -363.682   -519.01061   -442.30749   -465.01312   -534.71362   -612.46672   -480.57736   -389.91586   -351.68838     -618.901   -563.21394   -572.72674   -467.32473   -608.54186   -576.76888   -567.98458   -558.78407   -470.28308   -284.64572    -457.7702    -468.5868   -426.95867   -465.29347   -497.75312   -654.94834    -631.2763   -614.09574   -584.40648   -444.06179   -457.35625          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -6.00   -690.80901   -306.57217   -317.16018      85.3239   -213.64982   -398.30271   -172.99896   -300.58002   -354.75745   -334.34475   -370.80676   -106.90562   -482.69388   -330.99286    -471.4313   -177.48104   -240.01806   -377.72437   -302.74151   -126.37407   -331.73344      -257.36   -502.68531   -112.83968     18.01099   1198.82764   3264.08201   4147.96742   3858.59314   1521.18667    381.17698   -180.31878   -246.96207   -157.38229    -297.3421    -199.9148   -395.77778   -200.56677   -567.13845   -466.47523   -349.20406   -397.68725   -451.47521    -200.2418   -339.08095   -333.28335   -403.18365   -407.41942    -283.4303   -363.16466   -346.51511   -374.23288   -501.42584   -395.41587   -361.38435   -402.21265   -265.88401   -156.27174   -288.17306   -252.74149   -126.76859   -317.12931   -294.10063   -337.32681   -260.52366    -401.1203   -229.56717   -324.79741   -353.34773    -19.54646    475.73531   1398.10973   2000.96871   1900.12999    948.93976    394.02183    352.49465    179.67833   -294.03993   -365.86327    -371.1031   -313.68734   -317.91605   -390.98131   -233.59019   -349.28842   -220.76057   -229.66611     80.61388    609.09383   1465.31734   2492.54091   2414.13316   1626.24696    666.99168    154.48673    -234.9686   -440.67187   -168.79488   -215.81514   -131.12378   -117.24863    111.26232   -112.83873   -336.26458    -136.0444    -392.1322   -481.73654   -406.81993   -508.58888   -440.06482    -451.5551   -446.39078   -392.09536   -588.55052   -441.30416     -352.469   -342.26635   -435.94432   -397.12757   -435.33361   -426.43487   -255.19448   -152.37653    -37.61491     54.32275    489.27611    331.18155    156.86106   -142.66939   -262.79573   -471.48166   -316.79982   -227.03569   -377.01454   -229.84377    179.64449    763.77043    1063.0224   1433.24866   1453.43454   1119.97311    565.56221     91.09337   -179.84245   -308.34676    -70.67112    -70.20987   -288.44047   -241.38172   -453.48747   -262.10695   -532.02217   -413.28984   -463.10171   -544.25877   -497.18168   -602.93275   -409.14444   -527.36064    -418.1799   -415.03979     -446.688   -527.20173   -340.17359   -338.93326   -226.18493   -297.02867   -337.92725   -445.37183    -523.1664   -383.40872   -572.43463   -427.94829    -337.8322   -276.50484   -261.74501   -129.00488   -207.07836   -265.78201   -291.01595   -440.40052   -169.20024   -449.27077   -388.86085   -456.94746   -451.84729   -570.69811   -490.45523    -566.6862   -510.00845   -612.29715   -580.89872   -619.87069   -505.35135   -364.09586   -447.06711   -330.08233   -240.79821   -241.84341   -258.17065   -498.14462   -313.81943   -522.97624    -356.8337   -527.63767   -395.12266   -460.00287   -461.70971   -428.79594   -361.81387   -427.98596   -587.47735   -493.10844   -595.02335    -604.4385   -473.85841   -618.94628   -708.81842   -516.97858   -638.88163   -676.62022   -477.00093   -725.58052   -514.15365    -529.4784   -516.95767   -455.40827   -472.25708   -197.27508   -324.97298   -335.29878   -336.67063   -541.20605   -485.86734   -277.17172   -341.37757          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -5.50   -693.02989   -325.89354   -402.00735      97.6481   -353.51217    -372.9289   -146.26569   -484.09415   -143.70315   -298.78362   -357.91232   -193.02002   -511.45562   -519.64705   -402.84667   -146.38708   -480.69102   -335.67734   -312.68179   -283.69337   -431.55621   -252.09357   -597.12846   -280.64096    -71.68373   1008.09794   3282.23519   4723.97822   3910.60137    1665.2509    325.03904    -207.3788   -299.42239   -453.79264    -331.1273   -237.57779   -347.44774   -311.86083   -418.73311    -449.9655   -379.28521   -353.13217   -336.49722   -442.18735    -278.7925   -350.90991   -324.51999   -263.36406   -221.61682   -408.41255   -413.50707   -542.94826   -410.04581   -356.85246   -442.05516   -452.06721   -262.20649   -299.70635   -183.08512   -232.39617   -325.38549     -428.302   -232.33071   -309.41621   -372.96024   -303.74663   -261.09829   -278.32506   -278.46973    114.93448    783.87241   1212.18659    1934.1591   2029.79288   1049.78781    341.59829      96.3508    -165.4236   -184.36817    -209.7711   -246.38988   -471.58004   -372.28751   -369.00066   -332.88664   -359.88358   -241.55123   -404.88957    -65.73382    490.71928    1683.1587   2680.32704   2946.69506   1936.81207     644.9237    297.97511   -204.58592   -316.19173   -379.25296   -408.64943   -107.98309   -111.12272    -65.64046    -37.94058   -188.43674   -177.25691   -458.82752   -418.94688   -349.25022   -434.29432   -394.52276   -475.15841   -376.66855   -391.06411   -364.09263    -400.8773   -388.01416   -302.84171   -284.29925   -360.97878   -473.94969   -252.07328   -358.25383    -53.62302    193.98424    361.56162    409.89798    191.10147    279.81163     93.40165   -168.28564   -331.98871   -352.08052   -274.33463   -353.98607    -81.71361    206.02544    827.53453   1151.08835   1448.77525   1760.01673   1259.52927    767.18072    308.87989   -162.14681    -35.55583   -185.83113   -203.93735   -193.16232   -233.13525   -364.09993   -400.40223   -317.83871   -396.65336   -493.37206   -423.28798   -361.84123   -422.85275   -509.22914   -414.30642   -485.19081   -424.35152   -463.85556   -532.63839   -420.51932   -398.64663   -303.10488   -373.89001   -348.32217   -459.90484   -701.76245   -491.88014    -522.7157    -405.2599   -335.27834   -326.84967   -201.34348     31.88653    -90.32705     -39.1968   -234.30818   -326.45681   -341.57569   -329.64636   -371.18157    -502.7968   -363.64551   -391.50019   -518.93451   -404.65727   -577.50933    -526.0167    -619.9802   -457.68107   -401.82291   -530.49415   -352.47119   -313.62236   -450.85997   -318.27072   -166.58084   -334.87098   -301.74206   -390.76352   -242.43404    -313.0378   -244.99842   -450.57553   -140.60349    -317.6902   -368.67941   -140.62311   -101.01319   -508.59462   -348.45911   -672.41425   -407.47312   -497.73004     -554.679   -409.25716    -481.6245   -548.46204   -580.04391   -658.81567   -563.17983   -406.56817   -460.17714   -464.61211   -414.95392   -413.42554   -384.53689   -137.35813   -148.48242   -311.58442   -267.41604   -312.74905   -332.19569   -151.20313    -319.3937     -3.58674   -174.81728   -176.87636   -387.04194   -538.61529   -546.96523   -577.48668    -758.8419   -744.49438   -644.87094   -568.45335   -506.55082   -466.28787   -581.51209   -439.54222   -410.30557    -403.4499   -536.91737   -461.38876    -421.3569   -338.57048   -537.63878   -411.69826   -439.45853   -108.19175    -349.4975   -260.78947   -320.84589   -427.00346   -404.97387   -435.10978   -384.16587   -594.38942          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -5.00   -646.42732   -341.24329   -393.50366     65.75465   -488.93456    -359.9344   -279.47134   -522.13745   -103.62681   -278.45087   -378.33813   -240.82162   -488.90169   -489.67841   -376.22352   -174.57547   -657.19569   -408.52733   -173.85423    -317.3681   -504.73693   -107.58912   -486.61609   -575.55424    -73.13803    813.21755   3138.09936   4304.21726   3184.13779    1638.1983    220.55833   -293.98124   -186.08904   -282.27639    -337.8892   -208.03885    -217.9411   -357.96056   -329.18427   -484.34036   -311.08823   -319.15791   -437.93622   -350.44664   -199.36924   -389.63707   -330.99744    -508.0869   -349.89858   -450.30347   -371.48947   -481.37087   -409.18181   -320.52352    -449.8424    -386.0871   -296.14965   -370.98259   -369.86985    -68.80335    -237.3715   -390.26408   -435.48115   -344.79725   -381.44972   -593.87492   -481.20422   -291.16635   -312.01907    -94.07345    687.20558   1439.26355   2092.25816   1851.91786   1202.12953    442.99773     103.9272     38.45882    -35.41219   -156.23315   -327.31486   -483.96696   -413.07318   -484.43568   -248.85985   -365.18622   -371.61084    -350.7603    -56.27274    556.77617   1563.29812   2406.72395   2798.40646    1761.6526    929.20528    233.61038   -162.67963   -177.51055   -424.77016   -272.05433     13.33062    111.47049    163.67406     41.60578   -256.30895   -328.85922   -352.82327   -552.68049   -249.90059   -232.62965   -499.14644   -388.59516   -285.89057   -333.67748   -595.94547   -366.47331   -220.34338   -373.64214   -281.82882   -288.08642   -419.96828   -345.65298   -166.59157   -235.98224    228.73314    281.72241    442.80005    327.42347    147.79172    -48.55077      -3.3748   -327.04739   -296.90418   -302.34719   -171.93601    -76.35535    -20.21751    682.67078   1576.68873   1791.10322   1796.94336   1503.87065    830.67822    306.53262    -59.97897   -141.19115    -106.8708    -73.61305    -86.55543   -118.07527   -283.46962   -469.17636    -373.7325   -409.32874   -521.72611   -490.03844   -557.43846   -374.52736    -403.9028   -387.86686   -438.26783   -464.74407   -348.28921   -354.25002    -75.08929   -229.27325    -60.39475   -195.45272   -330.07957   -367.62727   -380.45842   -562.26877   -484.33462   -258.64362   -336.69495    -67.93272     33.66916   -118.21738    -10.48512   -151.24169   -201.00702   -145.50889   -310.44834   -287.15551   -332.58005   -471.18876   -573.39851   -519.83141   -418.75404   -523.30743   -443.22199   -425.94654   -418.47505   -343.56598    -630.6216   -368.78622   -193.52486   -108.89421   -123.28863    -65.51706    -72.70084     37.18261   -263.84231   -392.17388   -444.71095   -253.06002   -330.29115   -361.53147    -55.57782   -287.02044   -140.34387   -205.41634   -324.77265   -400.48675   -366.96778   -458.33247   -591.62957   -358.64392   -499.42098   -497.70181   -374.73519   -596.93767   -458.27585   -561.26186   -503.98353    -400.0452   -209.94228    -33.23996   -223.90763   -187.94694   -267.32644   -314.01115   -226.69819   -172.15619   -205.79782    -57.24781    -22.35769   -184.33276    -79.96111    -16.47274   -262.11018    -212.2311   -175.75937   -358.63426   -482.27807   -562.37483   -348.21478   -571.87653   -556.88013   -668.29157   -609.61666   -615.97245   -478.95883   -637.18415   -629.60212   -391.75702   -422.27319   -414.09518   -429.06302   -364.32618   -275.84454   -347.87451   -130.69897   -106.94727    -44.15551   -129.71208    -34.33457     -1.31608    -101.7761   -360.33266   -242.46876   -348.01475   -451.04757   -440.28404   -629.07299      -611.69   -462.76953   -593.52304    -762.6836   -542.51919   -478.11806   -585.56376   -548.92924   -547.09503   -551.49689   -559.23653   -556.94602   -651.47636   -682.93438   -701.23946   -754.04448   -698.44984   -680.48438   -765.41234   -971.80429   -656.51386   -704.08365   -940.11847   -765.72266   -851.44249   -838.93125   -933.97189   -625.06392   -747.02452   -873.99636   -674.32835    -651.2257   -696.44441   -645.66973   -823.44144   -766.34547   -713.88717   -675.46923    -716.1789   -785.54294    -660.2767   -626.87404   -641.73399   -717.92456   -727.86837   -806.54369   -727.65123   -717.25662   -762.41263   -739.96941   -811.52147   -590.09764   -613.46985          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -4.50   -615.81542   -354.84687   -339.35665    -85.02776   -360.26085   -404.78689   -393.48666   -532.83417    -238.3169   -251.92891   -324.29387   -318.62112   -458.10599   -294.75638   -488.19047   -305.55904    -389.8627   -459.59165    -14.94812   -250.42359   -593.64598   -161.26758   -389.10279   -533.24822     -4.69755    790.22228   3072.28545   4500.95634   3580.65523   1636.28361    179.03206   -248.85708   -248.82526   -475.29219   -289.56988   -196.00715    -39.16719   -251.62692   -339.67747   -436.33996   -245.14209   -342.50789   -357.56102     -504.089   -444.05636   -415.66787   -298.58911   -372.25679   -457.05521   -294.79349   -261.10473   -578.99973   -369.07752   -167.38009   -156.91254   -325.36299   -354.61908   -304.16849     -81.1012   -388.35552   -186.16388   -394.23898   -186.67454   -319.36462   -387.17134   -258.57187   -355.81582   -374.23683    -260.2539     26.67385    513.23809   1486.20792   2268.43872   1880.28804   1096.19802    410.31861    186.97155    -53.48332   -151.57235   -326.82733   -369.24121   -376.13185   -411.13437   -413.59124   -359.64357   -287.69189   -231.95495   -211.85253    -32.49903    587.93277   1396.76024   2492.65761    2577.0624   1747.15139    970.18047    350.33398   -222.94414   -402.09656   -325.82288   -314.19907   -219.68826    -55.85285      82.3338     -14.7732    -205.4925    -85.97598   -253.77313   -355.12994   -482.30987   -422.04884   -439.21257   -360.82458   -418.12197   -356.46644   -423.78594   -329.00404   -385.38342   -366.52531   -267.20699   -439.44421   -246.44769   -371.52525   -289.03899    -68.40705     76.30357    680.83509    592.87057    617.01316    219.23958    100.88765   -155.75322   -145.05878   -315.40514   -426.81916   -254.50466     -4.77607    273.41075    998.30682   1334.41179   1846.93644   1987.36023   1616.83408    996.01628    272.69408     27.98075   -192.21865   -229.68878       -69.57   -144.46196   -248.38446   -226.18558   -395.12773   -341.02322   -407.81329    -466.8064   -408.13452    -331.0147   -361.45422   -415.25438   -397.32428   -467.71121   -406.14722   -351.04894   -446.01469   -301.35939    -93.93327   -274.66964   -181.35801   -226.83137   -352.76846   -384.09273   -431.52285   -398.22801   -388.47077   -145.00344     -6.07373     -2.87487     -6.16084    -95.73903     46.59926     -7.25564   -268.52948   -238.21856   -280.99756   -397.95679   -279.15254   -434.66388    -372.1111   -511.08279   -430.10651   -396.69927   -466.55244   -498.17201   -341.87252   -406.29335   -257.42507   -238.07078    -50.85167   -208.70687      28.1973    -57.50162    -32.47723   -139.28855   -226.85789   -226.00572   -178.50294   -272.99273    -88.10595    -68.60038    -97.02635    -58.03235   -165.66045   -173.17524    -439.5921   -272.50477   -357.23175   -506.35625   -589.25453   -615.91326   -452.70673   -525.81266    -544.9358   -444.58393   -486.24769   -507.98503   -416.14116   -281.69787   -205.15207   -171.71637   -230.83178   -117.62884   -216.83886   -326.58735   -176.53535    -190.4828   -127.86836   -257.35793     80.97939     83.88854     58.35704      9.25719   -143.26162   -231.60008   -325.97973   -343.35953     -324.854   -529.00699     -541.122   -462.88448   -456.75824   -382.62045   -410.27778   -617.66276   -449.06893   -453.06504   -509.62602   -440.97447   -245.33663   -400.32412   -325.31751    -140.6203   -176.98663     47.69827      2.23832    -73.67334    -51.81453       53.334   -174.92325   -191.50167   -225.49633   -210.05791    -292.6613   -388.08329   -389.65917   -437.44836   -548.76158   -576.97369   -538.95576   -585.51695   -403.54241   -444.21989   -394.73969   -576.68625   -471.23037   -502.14213   -548.86697   -489.77956   -620.05708   -617.06137   -549.09049    -657.2876   -687.18808   -876.60303   -666.34484   -649.31844   -585.95579   -595.72666   -709.70102   -677.88523   -692.79259   -737.68833   -550.98749   -723.31101   -633.83985   -713.58402   -582.26758   -696.38304   -630.36235   -690.81636    -699.5209   -627.16078   -646.17262   -518.34476   -660.42307    -601.8315   -579.60502   -549.18634    -524.5329   -645.15703   -716.66606   -704.07502   -376.30557   -580.52696   -502.15534   -506.79686    -674.4895   -736.69807   -750.52178   -559.93179   -614.31943   -627.18329   -446.43583   -660.14302   -628.21078   -563.92975   -617.56444   -397.52101   -535.29731   -574.33404   -415.54608   -772.70961    -637.1318   -559.03426   -647.94246   -622.60042   -591.59393   -574.45905   -712.70501   -728.38439   -564.77364   -498.14036   -473.33321   -414.52073   -387.66316   -511.94609   -522.84493   -456.77181   -567.35643   -372.95423   -220.28455   -480.97295   -575.71457   -488.43595   -215.64348   -292.80543   -384.14713   -383.68359   -698.08057   -705.71604   -392.33154   -479.88547   -673.77436   -564.80378   -643.98694   -595.85609   -681.21457   -909.68955   -643.71324   -506.70868   -694.50579   -651.31292    -264.4382   -975.22709   -823.92283   -753.10848   -921.02206   -716.23655   -713.11688   -776.19729   -759.12602   -721.65204   -769.78064   -837.12042   -892.85768   -832.33126   -874.39505   -891.04046    -484.9546   -799.89793   -807.74858   -936.88162   -653.51692   -809.02991   -923.86351   -908.11624   -861.58121  -1030.64325   -770.96168   -743.26025   -783.63716  -1049.19956   -649.14377   -817.62155   -966.39252  -1084.92945   -815.15343  -1002.11205  -1174.51411   -971.92816    -797.3829   -1105.9816  -1138.75831  -1041.66081  -1223.38618  -1035.35848  -1170.40858   -957.95838  -1041.21317  -1121.12859  -1306.72161   -996.30283  -1218.64689  -1158.90095  -1038.23174  -1242.73796  -1062.20447  -1410.47048  -1062.47618  -1205.35752    -1232.964  -1014.59642  -1504.83835  -1286.69022  -1290.20451  -1259.27744  -1393.26273  -1102.07346   -984.21422  -1355.55314  -1254.85539  -1403.32974  -1554.06026  -1581.02309  -1064.43894  -1193.95444  -1633.63822   -1433.0084  -1961.79169  -1549.89967  -1290.31054  -1609.89534  -1315.77968  -1524.33631  -1732.04665  -1324.02189  -1581.91894  -1818.18005  -1421.34307  -1124.30414  -1444.99812   -1653.6598   -1529.8281  -1543.55248  -1666.42103  -1840.31439  -1494.67141  -1253.84009  -1479.70529  -1223.51092  -1528.43394  -1440.16053   -925.40929  -1751.35604  -1655.00796  -1374.86439   -596.39664  -1567.38263  -1724.94212  -1219.43618  -1886.82106  -1176.31887          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -4.00   -397.82097    -348.1117   -297.07028   -221.44477    -173.7025   -499.19348   -384.07064   -539.56729   -341.82115   -233.50933   -253.91012   -437.51304    -419.3442   -320.51111   -483.44138   -355.21874     -92.0644   -509.82846      7.44784   -195.14847   -392.51696   -271.78577   -319.27474   -158.71872    223.59727    906.93677   3251.70652   4396.60902   3941.15094   1652.54895    -72.83012   -230.60741   -213.14746   -482.14843   -215.98253   -281.94641   -214.58774     29.64403   -231.07066    -439.1662   -332.46853   -389.93764   -268.35173   -327.60319   -432.52124    -398.1022   -495.25336   -490.33473   -355.85023   -356.37333    -338.5175   -416.66445   -471.89451   -423.02206   -130.62495   -300.93811   -207.88048   -262.73366   -320.25571   -304.15895   -377.74887    -33.74576   -285.48982   -488.74275     -310.609   -373.03894   -384.01311   -392.73569   -130.50732     -47.7527    659.26424   1712.51473   2117.48908   1979.53519   1100.28504     397.9552     245.7665    -25.29764   -213.42323   -218.64542    -312.9377   -406.61709   -354.72818   -349.37075   -246.53136   -215.37499   -271.87501    -82.57427     -8.40027    375.73351   1213.38296   2190.87974   2342.50726   1930.27381   1091.27664    313.77048    -91.72214    -332.4113   -105.58463   -445.12076    -220.1409     131.4715    140.09282     67.96899    -31.87317   -271.56414   -321.47709   -344.99202    -543.9777   -317.20043    -368.8544   -379.91826   -412.87133   -385.63409   -389.79809   -391.82272   -424.42026   -331.85223   -359.39357   -267.14597   -284.94508   -314.32572   -188.19769   -200.52749    263.11818    401.61796    629.79646    481.38941    347.12029      -10.099    -84.01726   -117.24832    -436.6763   -332.12247   -284.75705   -118.38289     276.9886    812.94178   1460.40067   1914.79602   1911.20062   1562.71169    889.61952    408.75098     76.75093   -120.16416   -100.65549    -23.92589      6.80433   -112.17429   -285.14643    -300.9949   -389.33371   -437.48688   -417.27188   -355.20215   -211.66228   -341.18295   -340.39864   -374.16558   -418.93722    -295.1926   -388.67798   -318.27211   -214.16138   -307.12147   -162.20868   -256.88722   -220.33909   -383.92456   -231.67544    -213.3264   -475.61357   -246.04478   -115.05798   -191.43517     28.53675      117.636    -74.36708    -48.51035    -23.22724    -86.98014    -319.5847   -445.05392   -319.63951   -294.49292   -267.27404   -468.35368   -404.51525   -447.85073    -478.2609   -375.39085   -205.36761   -424.05822   -243.35692   -223.48192   -305.16671     29.90242     -23.3139    160.06632    -61.74253    -16.25425     -137.967   -160.78205   -270.81381    -176.8308    -72.49066   -137.48773     39.18449    142.49184    110.36367    179.77991    -77.90731   -201.80352    -307.7168   -501.20607   -579.64958   -450.06601    -446.5204   -551.65963   -457.32987   -621.17166   -352.37607   -524.77004   -282.62377   -402.51831   -307.94344   -282.73507    -24.03811   -156.06646   -104.61606   -153.58824   -232.59125   -292.03743    -41.94973     -82.2644   -120.35556    135.05475     66.34347     -8.30838    -28.23608   -170.99962   -181.92529   -251.64098   -395.87314   -276.92526   -569.88712   -347.86834   -447.47911   -332.67866   -592.70203    -389.3992   -451.46316   -360.73874   -271.38131   -258.38639   -337.87776   -355.62952   -309.69534    -65.00176   -116.27383    -61.72221    -32.28145     10.94749    -61.78541    105.04528    165.84137    -101.6501   -113.90762    -253.7717   -378.06239   -261.94053   -483.25292   -365.94046     -573.878   -562.91582   -450.42855   -477.03599   -425.28445   -428.96444   -382.67271   -480.90138   -207.30485   -316.90598   -453.85248   -420.66905    -472.5552   -421.70536   -548.02023   -579.20284   -569.31531   -466.91389   -589.03828   -583.41833   -630.58059   -754.91033    -676.5267   -459.69267   -577.12668   -663.58345   -583.72437   -651.36985   -600.34696   -766.10673   -735.07661   -465.61134   -588.79255   -632.16464   -456.57365   -465.95232   -574.17094   -565.18374   -485.87354   -651.72523   -587.25537   -457.65205   -483.18156   -569.40801   -504.15563   -610.96519   -573.32322    -542.3965   -600.33887   -606.38227   -470.67016   -552.20178   -603.79145   -542.82497   -374.64773   -488.20413   -478.98057   -581.24729   -730.21602   -449.98321   -404.07125    -397.3083   -511.96188    -393.0354   -579.01842   -474.67414   -346.28591   -477.64407   -473.07454   -593.38726   -505.57255   -671.91071   -527.04689   -515.12361   -448.21039    -671.7173   -364.25761   -380.01013   -420.47148   -503.26483   -298.15173   -502.15865    -113.0846   -463.56799   -195.16858   -157.65007   -316.65637    -93.67722   -273.12841   -354.86283   -255.20476   -195.82467   -214.67917   -377.55837     -540.222   -394.52814   -281.54371   -473.12039   -309.38365   -531.45563   -366.08531   -428.84059   -325.78667   -306.24568    -290.3876   -464.35215   -732.61015   -528.50281   -357.90281   -688.81837    -643.0174    -692.0711   -633.93277   -576.84537   -568.34204   -906.80506   -709.12487   -641.93306   -748.53727   -719.30483   -659.74486   -806.17869   -810.23425   -251.16116    -504.7047   -778.13675   -618.38807   -866.33558   -787.38555   -522.54777   -673.57343   -861.38689   -672.49488   -644.77044   -192.89741   -522.82705   -759.99812   -551.67971   -982.00313   -780.41892   -530.15174   -604.91693   -559.93397   -830.20465   -745.62565   -750.78771   -942.78883   -844.50883   -709.98134   -829.60025    -875.7632   -817.92439   -800.20398   -687.43972  -1052.33311  -1049.24106   -868.99545  -1053.17361  -1054.33978   -831.51092  -1045.78452  -1125.38171  -1055.88153  -1328.16834  -1159.27249  -1150.14898  -1357.95248  -1216.23304  -1504.75313   -1229.9496  -1048.30062  -1294.15664  -1721.36585  -1591.02061  -1290.30231  -1351.57426  -1179.41205  -1354.97568  -1374.43914  -1477.19553   -1304.5938  -1193.77739  -1238.83243  -1383.90392  -1281.40788  -1719.69314  -1403.33386  -1678.39471  -1409.25474   -1313.7943  -1510.76165  -1529.18002  -1382.92046  -1623.05171  -1410.17815  -1343.87615  -1277.71693  -1702.43152  -1690.17746   -995.54075  -1965.77371  -1385.85093  -1600.46228  -1647.96094  -1202.33228  -2135.96438  -1795.05324  -1315.34337  -1382.42228  -1417.30577  -1638.68414  -1639.59268  -1643.22569   -422.79037  -1106.57832  -1021.18201  -1325.40099          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -3.50   -169.45775    -321.5222   -412.55023   -386.78812    -32.34399    -577.4976   -379.91296   -542.29283   -423.32569   -309.45244   -264.98382   -501.22108   -413.34097   -360.16084   -396.08416     -300.715   -307.44349   -262.27021   -236.38921   -230.71074   -277.35744   -374.76163   -286.36714   -263.74536    166.37517    938.03063    3195.2693   4293.47621   3781.91849    1528.2786    198.88861    207.46533   -259.27384   -440.31023   -284.29358   -332.59092   -219.13731    -69.26328   -257.52275   -470.38606   -275.40812   -303.14588    -345.4215   -241.96022   -385.04716   -403.63526   -269.96643   -542.80664   -438.77313   -441.39324   -291.98769    -401.0542   -332.96888    -357.2602    -51.75829   -171.89788   -462.99233   -203.94665   -101.92111   -187.22624    -84.76747   -277.32262   -411.78904   -351.98934   -364.99345   -383.04042    -184.1458   -520.66419   -251.67786   -157.35627    687.40195   1550.00823   2396.37348   2000.16462   1117.97242    783.31895    223.21743     41.08143   -156.86477   -137.53208   -270.48575   -416.28269   -327.76183   -293.19455   -235.51648   -396.14729   -315.02069    -95.08841    -13.55583    300.49972   1472.22084    2273.4009     2426.152   1862.80006    874.77234     95.90607   -128.47038   -217.09979   -272.35086   -147.77734   -195.07264     52.89989    272.41895     62.69482    -20.85257    -60.98598   -244.45028   -630.07123   -313.60505   -292.76652   -245.90246   -472.57909   -267.28827   -415.00427   -418.98378   -283.18781   -303.76049    -337.2663   -325.22912   -241.12344   -263.34021   -319.44378   -255.05948     43.84043    171.21159    576.80955     663.0993    460.17077    319.62092    237.22895   -137.05161   -282.46086   -174.90717   -316.12437    -88.00005     44.19462    200.57732    904.33269   1571.85526   2060.56718   2024.41299   1787.35298   1107.37266      646.919    143.50445     60.59116    -47.87927    -50.73889     -118.679   -245.37775   -244.19243   -412.51083   -240.02343   -381.72384    -328.2832   -362.95128   -421.67438   -427.81658   -363.38614   -320.34601   -320.15209   -397.33086    -246.8058   -281.47306   -112.13311    -15.03464    -52.47892   -208.76074   -256.11079   -218.13662   -394.24525   -381.02735   -135.41863   -170.25674   -240.75252    -72.17735     64.74589     11.85696    109.47865    148.68939    -57.95532      -188.64   -169.77567   -308.25978   -286.15548   -277.72247   -295.04153   -380.74244    -463.1622    -480.6862   -392.47922   -480.53468   -293.78598   -237.86724   -279.94283   -337.75463   -287.30289    -25.88365     56.61408    156.21673     96.84741     62.99225     25.52323    -24.07313   -161.97383   -240.74038    -15.33111     81.56139     -51.1779    338.06925     60.15491    -77.89925   -113.32735   -128.62603   -316.90001   -355.42165   -495.01599   -551.33233   -328.97386   -506.53464   -398.49233   -327.62377   -416.52431   -307.55797   -343.68617   -258.57864   -329.48181   -131.02437   -148.11476   -284.98656   -213.50267   -201.27834     65.60043   -138.45826    -87.05621    126.11172     59.50813    137.60309    -79.31083    -65.21577      -2.9889   -108.51676   -131.38676   -151.98455   -525.95041    -61.57549   -239.49197   -499.79479   -360.22801   -479.81744   -435.32023   -465.79543   -444.64873   -371.09244   -419.15172   -250.18568   -281.87125     -93.8544   -193.74481     55.31288      3.92822     63.62433    179.73169     -6.32695    436.90115    123.57962    290.79542    171.30435     64.20017     -6.12552    -88.46247    -67.40532     -330.946    -405.2625    -374.8411   -322.41866   -532.30741   -459.33359   -251.88219   -361.37743   -274.74653   -310.23383   -263.49241    -316.1791    -350.3101   -444.54193   -692.31695    -543.6122   -581.94012   -595.94575    -826.8963   -472.46316   -490.86645    -585.1457   -710.76345   -451.30944   -726.09089   -594.85282   -732.92506   -612.88429   -593.62169   -596.34802   -668.91563   -447.34457   -717.92432   -624.04824   -436.98137    -633.6284   -316.30437   -363.54632   -404.83657   -534.91661    -501.9593    -403.7506   -290.34196   -491.56048    -476.4686   -495.80915   -371.54438   -368.94443    -379.9497   -484.90529   -551.07146   -450.25639   -661.62182    -527.9661   -635.54796   -557.47741   -707.59358   -455.38467   -137.22934   -636.01787   -357.39102   -622.12973    -293.5734   -443.76358   -352.91077   -422.02643   -256.80742   -466.92296    -563.8595   -316.19019   -407.66108   -539.22086   -429.88606   -357.37537   -408.13033   -289.32142   -338.66135   -194.51013    -540.4001   -525.88763   -246.04936   -516.41957   -407.12031   -366.10503    -50.71148   -174.39467     -31.4176   -175.57748   -279.32384   -339.44695   -173.93863    -115.3974     -1.30071    -84.89066   -260.50869   -338.66921   -361.56184    -292.4099   -197.99642   -372.43681   -469.44097   -483.83638   -334.15884   -519.86088   -305.28889   -338.49944    -411.6063   -380.06385    -402.2972   -586.30262    -442.8394   -383.47833   -706.38121   -291.83144   -744.51621   -309.07836   -318.41085   -674.48551   -482.63128   -461.10755   -375.91928   -550.55753   -335.63349   -703.52263   -729.98786   -794.02667   -453.40782   -484.51426   -450.02273   -361.42629   -699.20985   -739.23786   -770.08102   -362.21559    -474.4792   -511.92507   -678.48668    -637.1924   -495.59281   -582.83927    -734.0834   -448.47102    -454.2021   -405.99933   -554.73422   -811.42902   -675.52909    -541.0072   -794.58606  -1037.76065   -702.96448   -407.40892   -637.64204   -663.45315    -817.1585   -714.85922    -785.0431   -862.07934    -990.2017   -821.08713   -879.06912  -1095.74465  -1032.42426   -893.93791   -975.13043  -1028.45877  -1061.68316    -1145.009  -1369.40139  -1192.50073  -1200.68816   -1119.3678  -1500.02866  -1356.73074  -1339.32278   -1202.6552   -987.34984  -1310.05242  -1517.54592  -1277.06888  -1286.24334   -1261.9906  -1397.05229  -1070.29856  -1001.99531  -1475.17959  -1193.72707  -1333.49465  -1047.66743  -1318.25435   -956.39182  -1392.52826  -1582.03208    -1164.924  -1066.54748  -1364.39429  -1268.93326   -521.45999  -1528.38434  -1175.10793  -1404.63962  -1424.85394  -1330.40376  -1499.39353  -1890.88553  -1723.67733  -1450.44308  -1661.55038  -1385.91751  -1480.74847   -983.14259  -1536.21065  -1274.66708  -1428.30953  -1456.80215  -2026.81878   -930.99477  -1750.72634   -970.04541          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -3.00     27.31091   -294.13809   -538.34659   -552.51542     48.81398    -596.6634   -480.14688   -469.41116   -453.35515   -366.05923   -293.04226   -532.86943   -422.25628   -313.41645   -289.44088   -291.81053   -474.54653   -157.71844   -471.10945   -362.70299   -454.27916   -276.89484   -153.27605   -176.79145     67.67814    805.35171   3095.15628   4404.11162   3855.67233   1651.41453    199.29006   -251.83955   -191.96207   -335.07858   -356.99798   -313.44561   -131.32751   -438.67781   -280.18766   -240.47518   -518.76572   -317.05882   -296.85781   -336.99114   -342.23443   -530.19558   -341.65625   -510.97249    -528.5246   -273.65773   -499.41472     -473.145   -398.73702   -389.68542   -385.15245   -332.19498    -63.08429   -240.09146     -8.09986     -2.78078   -261.15332   -298.35529   -317.28737   -253.51688   -411.45521   -499.52735    -259.9305   -374.21796   -213.78795    -54.94209    507.97879   1402.11721     2326.953   2091.05649   1337.53939    447.43825    227.42454    -73.86674   -140.78929   -341.28993   -401.39205    -230.5646    -337.1475   -180.71412   -390.85568   -410.05781   -282.82223   -137.21792   -134.93442    493.22095    1429.6818   2373.67544   2825.13867   2016.25703    939.93407     213.7967    -45.44376   -179.38409   -214.01066    159.97843    -87.33539    244.58955    233.05429     61.64924     83.05392   -246.39827   -154.88927   -261.97346   -383.48677   -292.07846   -333.01415   -464.18678   -347.31998   -343.73033   -314.74284   -416.39447   -395.93996   -252.97998    -259.7075    -409.9074   -313.82587   -173.62723   -261.77851     -2.49742    370.15638    722.97216     689.7593    438.85279    352.35415     17.64984     60.30738   -274.88713   -201.07657   -301.83568    -311.2598   -185.00496    175.29938    953.11977   1527.85298   2081.38659   2282.85847   1966.52082   1190.32948    745.20907    278.60804      9.76079    -81.84498     47.08148    -92.58394   -193.04326   -150.99899    -224.4338   -377.77923    -405.0595   -396.09891   -396.25399   -461.09426   -165.71993   -247.24925   -333.76246   -256.92531    -388.6403   -328.70786   -216.01209   -195.96503    -91.98758    153.37328   -148.02097   -122.64951    -19.96706   -323.01252    -383.9914   -174.37478   -355.04733   -123.02718    133.79384      45.4678    -48.99263     44.00933    -42.71187     85.02225   -229.54028   -160.49817   -153.09032   -313.83799    -213.6994   -325.19871   -278.10522   -254.34776   -283.07228   -531.62407   -403.29669   -406.08314   -213.50968   -275.64494   -154.15161    -97.22151     -40.0992     78.04303    -10.68397    -13.46579     73.72393    -39.92039   -182.28439    -77.22057     71.54475    -47.24656     43.07856    -47.76358     77.46848    119.75869    -137.8894    -34.95257   -155.17021   -278.08583   -352.60264   -369.94687    -260.3606   -374.65523   -332.88621   -442.05586   -240.25986   -495.71383    -324.9465   -334.98829   -465.72341   -229.14949   -200.51093    -61.34874   -303.40554   -101.57415     15.56508   -227.48582    -52.12636    -67.15165     62.38973     11.74666     29.03881    -32.35985     45.24126   -124.00475   -161.34576   -224.19856   -186.05958   -249.79423   -348.47542   -594.55398   -301.27221   -517.03824   -342.17057   -372.00601   -394.24914   -354.98544   -325.24717   -220.94848   -227.32828    -237.3784   -106.07868     20.62048   -122.39584    244.54814     22.33358    395.34427    495.94938    283.93799    193.94583    305.62996    143.05419    165.80426    203.04384   -246.49453   -104.14604   -217.36332     -269.324   -298.48591   -519.02226   -481.15701   -227.14604   -240.37097   -486.29961   -293.92911    -454.2138   -245.23575     -195.411   -530.27846   -299.87935   -402.38317   -379.81797    -565.0178   -423.84101   -346.13113   -570.20205   -551.75678   -414.85862   -479.55199   -465.63808   -580.19578   -588.50086   -605.20161   -517.34605   -555.12199    -394.1107   -647.67424   -654.95709   -326.56197   -536.58004   -469.99134   -643.32274   -483.09245   -525.88192    -234.9611   -383.29554    -418.5827   -357.25206   -320.30103   -262.05793   -370.42579   -274.52884   -407.60423   -452.40928   -324.50466   -414.04615   -686.11706   -635.25068   -688.21496   -454.37498   -378.37035   -595.78822   -348.09384   -360.88054   -504.93009   -312.06137   -422.54316     -384.123   -432.82496   -399.61426   -317.56398   -425.77596   -392.32101   -421.94375    -385.5778   -427.91704    -401.3817   -501.27975    -566.6629   -442.13438   -392.06182   -574.49471   -277.48292   -276.06928    -335.9901   -182.46177   -238.29404    -396.0872   -497.54072     26.64036   -169.72353    -396.3363   -288.52192    -27.05054   -111.21626   -188.64694     -2.33175   -224.52264   -137.03452   -268.67203    -52.35573   -381.98702   -249.04592   -208.88287   -249.02376    -412.8105   -315.01904   -359.44479   -358.42745   -365.70193   -298.10252   -349.81416   -368.81298   -397.18115   -127.19273   -501.72804   -296.88887    -183.6802   -439.34903   -478.82357    -416.1187   -443.66422    -424.6248   -517.54183   -604.20538   -468.49245   -609.27661   -517.64032    -493.9391   -581.51417   -399.85847   -582.82558   -680.75359   -434.12059   -653.89484   -556.98776   -849.07616   -611.40181   -582.84177   -526.28114   -485.51599    -675.8333   -201.34009    -379.0391   -738.12182   -285.57718   -719.89059    -691.1199   -527.36449   -517.28707   -700.66809    -84.58439   -489.56421   -418.53173   -787.55882    -536.9712   -594.07215   -472.23662   -847.59355   -731.75479   -529.88318   -635.02813    -993.7823  -1013.67568   -828.67567  -1026.96446  -1025.19033   -992.69409    -943.7685  -1048.68041   -931.21556  -1018.69831   -1129.3923    -853.2988  -1239.54195  -1117.46424  -1106.69018    -982.2456  -1059.99792  -1138.86848   -1198.1555  -1293.54743  -1333.84561  -1084.59751  -1068.52376  -1295.46417  -1357.88135  -1112.79145  -1202.49465  -1022.25793  -1396.45177  -1415.92015  -1319.20674  -1324.43564  -1541.46426  -1175.72395  -1387.87513   -1188.7434  -1322.01664  -1421.33977  -1319.64983    -922.1077  -1512.88301  -1451.38271   -1335.0997  -1213.99431  -1787.07163  -1107.16307  -1394.45032  -1665.60004  -1419.62005  -1214.47553  -1377.47404  -1595.15492    -727.6599   -540.70324  -1095.63746  -1153.85003  -1519.66402  -1480.25482   -505.47681  -1311.46073   -828.21457  -1116.60901  -1513.62003          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -2.50   -124.26159    -378.0484   -540.68711    -629.6652    108.55281   -603.34714   -600.84038   -358.61232   -383.30551   -354.63053   -240.15016   -409.88324   -382.75153   -308.82741   -220.64902   -297.71721    -597.5064    -296.2907   -394.73666   -439.82442   -291.48307   -179.17399   -108.22181    -84.55118     35.44998     724.8443   3176.93109   4580.90235    4076.7805   1512.17692    330.45426   -313.62641   -274.26824   -350.65377   -358.51246   -180.37815   -225.66111   -215.80759   -417.16258   -372.98491   -201.92715   -228.52594   -334.03378   -357.77051   -293.50972   -397.02033   -294.98076   -314.24324   -237.96803   -435.65692   -368.21341   -360.50922   -556.15312   -421.25101   -472.76766    -230.2185   -269.73203   -343.74857   -154.01713   -169.90993   -183.08132    -72.71543   -207.62992   -219.33099   -282.59717   -278.02408   -222.30978   -251.04412   -201.50163   -125.62522    793.85168   1614.84912    2204.0108   2152.86352   1173.50251    553.79987    258.95556        -3.66   -328.86863     -2.38192    -234.8838   -137.77237   -334.09504   -184.66715   -307.70291    -314.6134   -180.96734    119.95419     20.23711    341.86205   1429.32783   2597.35896   2321.81427   2216.87073   1063.39745    323.64265    -89.03617   -107.22774   -358.04313   -220.20721    -81.74987    -36.76298    115.24114      -28.622     76.92957   -180.64242   -360.57419    -337.6913   -326.48164    -502.7057   -413.74474   -424.72696   -362.55526   -371.30775   -278.37083   -339.55523   -353.53899   -365.91679   -304.59242   -250.47088   -236.84227   -391.43927   -133.27327     18.79964    190.44112    527.59294    466.56276    609.64935    441.28781    102.22376    -62.90664   -208.00593   -344.29264   -334.21761   -326.61487   -113.01709    272.63401    942.44049   1572.99273   2169.99264   2388.94571   2146.77516    1453.1156    986.61921    306.93939    244.43777     37.13964     52.00066    178.85016    -92.68041   -178.59415   -201.06637   -353.09716   -389.48759   -364.21814   -393.48296    -314.4181   -368.52345   -340.50837    -188.8178   -231.94018   -345.56562   -231.41317   -397.67005    159.92823    -26.64539    -97.77143    -37.87832    -52.32639   -323.86284   -247.08751   -182.20708    -418.6248   -292.43441    -43.83064     49.77687    115.09007    301.47144    253.70644     77.34168     66.40896    -76.96098   -164.24251   -158.52208   -314.81535   -338.47734    -77.62759   -228.11115    -355.3272   -357.98312    -413.7825   -323.34127   -322.16272   -150.16673     -76.9345   -192.22999   -113.35729    -39.12074    105.03735     167.1855    144.88894    199.81547     26.03356    -11.02423    -90.42926   -211.54264    -73.95911    -55.30279    -92.41523     10.48307   -282.24671   -203.65178   -235.89995    -137.1946   -107.28521   -251.10326    -339.5073    -289.8636   -344.29037   -368.40055    -343.3262   -323.75247   -553.72116   -319.40359   -185.44133   -265.43826    -389.9957   -410.05212   -102.54458   -143.41076    -39.93699     14.26998    -28.24729   -183.06853    -33.82975   -111.35794    162.78781     15.57152     16.13816    337.31013    199.75584     -4.32434    -49.85178   -202.98068   -188.84669   -196.13884    -64.95472   -181.49801   -297.28716   -409.63962   -386.03249   -479.91195    -319.6726   -189.24682   -206.56455     -245.983   -160.87434    -32.39832    -30.98132    -84.00496     98.62062    135.51156    443.39745    443.02587     456.7251    346.50805    500.37729    372.26267     47.10414    174.31737     56.07238   -242.19153   -129.99878    -332.7712   -255.15032   -285.33665   -409.38332   -287.25998   -295.73138   -345.21484   -340.06108   -267.69078   -107.99461   -480.19773   -189.03415   -156.03578   -349.68291   -125.47203   -342.81186    -574.6878   -349.29913   -470.28846   -336.88301    -530.0039   -557.44116   -639.73423   -433.75205   -684.25968   -518.67038   -620.03311   -435.93091   -460.76483   -474.03511   -370.59719   -411.97653   -376.52398   -361.69578   -465.70593   -446.45994   -506.72044   -497.19962   -383.72037   -208.27345   -180.12963   -296.23328    -370.3413   -159.56489    -182.5371    -431.2189    -69.76756   -261.67376   -322.51256   -258.28104   -233.13917   -502.78435   -392.24693   -367.58171   -363.87777   -440.83203   -541.06865   -383.12691   -348.75233   -300.77144   -385.02842   -375.24213   -391.55046   -333.94682   -393.96793   -311.49708   -340.39277   -311.76616   -237.23791   -309.32136   -386.34559   -282.85139   -382.19964   -490.05632   -499.29876   -231.52535   -213.07438   -441.30318   -414.51658   -290.08156   -286.71756      -34.569    -180.7378   -113.92955    -45.31742     151.6432    -89.49764      19.5422     59.95682     42.64523    -96.23055    -225.8116   -191.79668     -27.5752   -150.59926   -134.74071   -137.03345   -115.11381   -164.53985   -144.81915   -419.57795   -392.45247   -268.41522   -188.19508   -391.71582   -224.42954   -112.22678   -180.38284   -297.43542   -245.24328    -169.4427   -392.82832   -298.56624    -229.5916   -267.67919   -110.18999   -582.17432   -557.16783    -418.4106   -384.80495   -508.16704   -410.31961   -366.21291   -626.50345   -399.58855   -337.94559   -377.89197   -426.92901   -390.68693   -529.93951   -310.90867   -313.08854   -566.47032   -589.09843   -612.67562   -555.81733    -309.2085    -385.7071   -509.68624   -212.73593   -449.90427   -476.20987   -494.92691   -220.82012   -649.82248   -428.87174   -205.27006   -625.21087   -491.87234   -859.30983   -793.87991   -641.84257   -826.50465    -625.1482   -644.31015   -457.40559    -708.4456   -892.15819   -849.06018   -954.56098   -871.05798   -852.10078  -1123.22769  -1051.38593  -1106.66159   -807.02054   -706.61001  -1001.04427  -1295.47618  -1139.91282  -1151.22372   -852.29599  -1325.11735  -1255.25881  -1141.64576  -1115.73502  -1317.04246  -1230.93587   -831.55142  -1231.52336  -1279.72719   -884.37105  -1391.91254  -1355.08465  -1296.33858  -1316.67497  -1194.49459  -1385.42907   -960.41117  -1064.41661  -1531.54462  -1318.11803  -1079.16788  -1270.01847   -957.63808   -831.27447  -1179.20864  -1570.45159  -1419.94312  -1461.34884   -843.81804   -917.97017   -1011.0548  -1261.97749  -1558.37788   -997.18507  -1533.70202  -1052.29718  -1195.40952  -1025.37822  -1203.27723  -1258.74449   -948.18804  -1389.31718  -1362.12306  -2085.81525  -1876.67477  -1138.32338          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -2.00   -272.16486   -493.35681   -420.94706   -508.88955   -115.39689   -547.41333   -543.32768   -294.15707   -303.83189   -353.26661   -187.25442   -246.99443   -313.83963   -358.15839   -249.39964    -379.2866    -535.8358   -374.17833   -359.35933   -413.70721   -195.18715   -210.62547    -123.2168    -46.39396      94.2178   1112.19316    3553.3468   4615.67319   3634.16394   1538.00521    237.53429   -265.62365    -201.4635   -346.92971   -363.62338   -108.83398   -430.57342   -289.44539   -101.95138   -136.91669   -617.97986   -338.21809   -462.96326   -356.22252   -344.02155   -346.95878   -189.00496   -347.71376   -300.65632   -341.17093   -205.69449   -476.35719   -380.89188   -262.89608   -277.44694   -218.39168   -183.13103   -253.05011   -215.87777   -173.56542    -297.6937   -165.31118   -278.78518   -352.35653   -323.28026   -277.88518   -268.23347    -299.6159   -118.43221     35.03139    720.30217   1438.10895   2410.89127   2095.48739   1207.12902    495.46023    134.63771    -95.05859    -112.0435   -292.95367   -171.15189   -169.20455   -314.48305   -193.61421   -131.05267     -98.6054   -281.95996   -230.92041   -116.64843     663.6182   1532.92043   2472.64203   2626.05728   2167.60844   1509.04211    321.86047   -159.28665   -125.53415    -457.7554    -50.78325   -202.01135    106.33581     248.5697     55.40424    -88.95449    -68.66921   -188.41809   -255.84937   -410.37859   -370.89288   -382.62248   -383.94148   -404.87868   -238.93864   -326.44342   -164.41166    -218.9248   -255.25401   -219.29549   -228.78755   -305.02021   -256.26598   -194.59949   -184.46608    167.91951     703.5396    846.00818    632.70671    410.50698    -65.97574     53.27538   -367.91443   -267.08751   -223.71647    -374.3459   -112.48102    337.25938   1046.74616   1758.76435      2195.37   2408.96612   1817.44994    1328.6708    840.60104    191.98789     50.25805     70.20536    -34.65063    117.14101     94.73597    -198.6971   -139.56892   -312.23351   -205.32533   -299.98401   -268.89638   -370.43438    -373.5083   -397.57738   -303.13807   -322.25445   -470.67367   -227.97827    -62.37637   -119.75694   -250.32267    113.52226   -180.40878   -114.58439   -131.74102   -275.53599   -382.08631   -358.46232   -125.98401      12.6252     80.59163     95.20879    172.09845    239.46293     15.01303     125.6076    -153.8583   -353.67526   -266.51137   -139.39575   -393.89711   -248.65436   -449.38857   -368.62495   -476.68694   -272.16291   -364.83119   -293.03686   -258.86435   -300.36698   -176.35444    -13.66037    -83.04228    214.17105     14.05178    166.86827    240.92721    341.73975     35.50426    -72.99448    -36.32557    -20.91178   -110.43415     37.86418    119.22411     44.20315     62.71463    -140.1135      3.41837   -276.37001   -327.45646   -263.11644   -295.89237   -517.86492   -526.85684   -272.16513   -396.11698   -133.02539   -363.52182   -321.07608   -164.26028   -123.70209    -55.25596    -40.26877    -12.79259    -11.83158   -150.98866     21.31111    -22.36791     68.12698     83.16247    -11.77029    133.96576     292.2111    120.23498     54.79577    -22.59654    105.88008    180.73593   -118.62764   -147.95029   -360.66003   -134.21887   -442.47979   -388.38888   -220.82357   -129.25696   -350.52451   -313.32914    -93.36223   -236.93404   -154.16947    -46.12021   -110.67359     -21.9409    184.10547    307.19256    281.35112    536.34373    449.19249    667.82607    537.21686    497.03241    478.09323    111.41306     23.46469    101.46645    -196.2517    -95.53499   -294.85748   -274.05055   -243.60843   -429.51504   -415.63011    -212.7038   -282.43578   -140.20373    -45.80205   -192.88802   -109.37714   -209.58217   -251.54856   -280.20627   -279.47322   -206.60466   -340.86694   -382.95064   -448.68432   -541.84281   -457.45977   -421.09538   -542.42043   -447.72642   -326.23392   -513.56604   -273.96925   -475.29297    -446.9816   -362.39567   -452.01069   -532.56116   -429.00991   -528.49382   -324.12754   -447.47696   -186.35823   -267.80481   -267.80287   -274.22227   -172.37446   -271.22691   -243.16403   -189.44263   -264.67476   -265.88132   -179.04311   -269.38407   -457.18265   -373.20281   -384.45506   -284.72409   -421.07087   -380.78704    -441.2126   -252.41365   -288.27488   -488.60679   -334.46764   -234.81816    -15.24866   -357.43985   -134.40506   -322.58829   -356.98128   -170.29978   -464.85736    -84.36673   -331.98178   -366.33289   -455.94849   -334.08946   -499.86866   -287.51158    -350.5826   -404.04931   -269.55768   -394.85259   -222.85047   -205.14173   -282.17072     -25.8767    -74.57781    -96.75438    -21.38269     41.27203   -191.08452    -52.70855     62.08955     69.50585    -73.74848   -187.35209     57.27548   -115.94565     -49.7424   -219.83459   -382.80281   -195.07811   -346.08123     -53.5594    -152.7383    -299.7981    -42.66288     -57.4735   -291.29747    -218.5622    -87.74719   -397.69368    -402.4456   -332.23013   -281.93523   -251.62671   -352.04038   -273.11229   -537.05815   -238.91877   -260.61369    -70.80553   -444.08361   -357.02953    -123.2507   -423.12198   -523.56449   -544.22506   -487.62655   -566.45429    -319.8353   -181.33181   -543.92618    -67.83962   -197.93556   -497.13975   -241.56177   -435.61831   -468.68189   -518.88523   -280.20139   -394.10001    -431.3471   -520.21859   -287.35719   -559.90251   -657.01626   -366.95738   -409.54459   -733.25044   -480.00393   -714.03961   -525.20348   -388.84348    -617.6525   -602.27341   -373.02172   -775.64274   -326.53051   -633.21972   -741.79134   -699.46449  -1023.83241   -397.69085   -841.37898  -1062.12936   -833.91743   -798.60866   -579.04263   -666.25948  -1207.54375  -1344.87306   -842.89708  -1151.96274   -975.28033  -1112.92092  -1218.15605  -1189.87738  -1494.26699  -1008.52244  -1460.31277   -833.31178  -1346.55382  -1113.92899  -1449.69654   -1258.2361  -1152.55347   -827.36235  -1246.77962  -1101.69622  -1478.82032   -1373.0453  -1220.58007  -1136.60643  -1300.31327  -1286.87319  -1556.98734   -889.98932  -1523.38143  -1466.73114  -1333.07811  -1176.34151  -1505.24767  -1206.38567  -1432.36156  -1244.21087    -761.7343  -1554.71858  -1346.58935  -1640.80537  -1357.74396  -1505.09363   -846.10159  -1176.72081  -1206.92054   -627.75034  -1100.01678  -1795.30809   -1061.8495  -1370.87044   -850.40467          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -1.50   -365.50074   -589.79663   -298.42114   -395.80958   -336.77826   -489.61178   -394.05707   -274.03742   -217.94847    -361.1698   -159.12329   -194.83807   -336.96917   -271.82677   -247.27831   -392.40088   -371.14985   -385.49116   -356.46599   -344.92494   -219.65523   -422.74758   -218.46015   -195.87757    -62.59538   1161.00164   3419.51188   4429.93526   3665.86136   1616.43584    209.24425   -221.00292   -101.00707   -285.34267   -399.18853   -108.26233   -408.04762   -393.16871   -338.37624   -457.48969    -282.6061   -336.55591   -304.85335   -486.59083   -473.15257   -247.05028   -342.27817   -450.73971    -354.0662   -354.07391   -374.57447   -453.80688    -421.0606   -377.75104   -318.90389   -348.78384   -409.95543   -313.78634   -391.12493   -261.33002   -277.35169   -165.22824   -345.92574   -206.69158   -378.02988   -307.76421   -307.10407   -301.17804   -295.98599     19.21913    565.03464   1769.92024   2507.86073   2051.12186   1370.51375    514.78972    142.61704     157.8414    -65.20154   -173.28576   -212.75991   -115.50401   -236.88342   -209.94692   -303.82966   -282.38918   -180.08776    -116.6351    -93.28256    630.84392   1461.21181    2561.4596   2771.97428   2230.90374   1173.51669    439.89774     72.31787   -112.37884   -378.10817   -213.89738   -171.66712    427.28227    267.26278    257.39307    -79.63592   -144.16346   -271.44736   -372.54444   -185.68739    -374.8233   -468.82162   -416.55554   -272.52091   -404.80245   -355.29682   -271.04795   -242.84171   -263.22201   -369.27563    -131.1197   -295.47036   -160.75606   -235.29463     -3.05314    279.39539    584.02027    820.82208    668.35328    325.74862    263.33408      2.01971   -195.20443   -167.85167   -305.67361    -11.25317    -21.49978    423.71392    1049.8861   1851.75517   2725.92619   2646.20017   2232.10486   1479.71571    741.01153    394.27099    157.30009      4.80626    109.62367   -111.82421    -57.01851     -106.574   -256.69007   -198.03322   -331.48668   -304.22805   -412.15241   -387.05088   -348.82189    -362.2531   -386.23407   -235.57269   -233.57482   -229.94299    -276.7538     50.25157    117.08919    147.68452      2.61209    -92.86146    -319.8066   -315.18948   -335.44536   -109.18678   -333.36743    -49.13123     12.65488     55.01904    203.84075      238.294    170.37215    -37.99797    -13.05754    -57.84247   -166.11557    -94.42012   -174.74817    -59.52905    -284.6354   -386.45179   -338.90387    -426.6806   -198.78685   -359.14313   -302.77177   -291.16264   -112.49405    -92.05375      2.26396    160.74682    103.03556    179.09412     208.7551     66.21945      6.16284     26.78669     50.86396    -66.33593     -9.22163    -44.95909     43.20236    179.29675     11.66016      8.44796    -70.03055    -49.73799   -367.43079    -368.4226   -476.67084   -291.63661   -405.46956    -341.3273   -301.71871   -169.00982   -406.25144   -255.46095   -222.34318   -129.19996   -126.29104     31.61292     77.58363     -18.8402      33.6001     76.20403    -90.69407    -52.88644    225.48486     216.9212     77.51292    368.17042    280.26557    176.48051     58.35978    181.08155   -123.31845   -144.88086    -231.3087   -396.15526   -262.15217   -235.00763   -397.52475   -202.86566   -343.34014   -376.38816   -182.91576    -242.9528   -315.28108   -144.96056   -267.75058    -112.1124     -8.56658    162.35976    210.73964    211.17225    494.44967    521.58294    497.77244    480.14656    507.88878    486.99856    247.38634       9.0789     60.83324     -3.49581   -192.72764   -229.35354   -261.93925   -300.75161   -399.90387   -186.93367    -226.9459   -109.90305    -290.4734   -225.36691    -72.46861   -124.94905   -362.54858   -260.96713   -112.10952   -232.35085   -203.69291   -348.37504   -431.33771   -232.02387   -355.97879   -357.66194    -450.5347   -412.77107   -400.77336   -608.38408   -432.82839   -377.39262   -341.91723   -420.16131   -468.68392   -310.31863    -396.9846   -328.38026   -221.58242   -322.10363   -263.42696   -329.23423   -184.79837   -191.87113    -70.75435    -29.18755   -337.17511   -123.88767   -178.06028   -173.19987   -227.76899   -110.14154    -34.21296   -388.55519   -257.83236   -351.25198   -254.65619   -487.56498   -378.03453   -429.12856   -354.03562   -318.83959   -292.15695   -210.00088   -133.97423   -118.31005   -226.79431    -11.49879   -132.95164   -308.30191   -168.58433   -157.41536   -291.66141    -264.4004    -293.9042   -428.64375   -278.50935   -173.44744   -411.28822   -477.12158   -183.99496   -245.60841    -23.95947    -55.42201    -91.86441   -118.49977   -194.24384    -32.63412     64.87737     218.7698     54.70257       5.2484     -5.03683     -31.6408    -80.69353     164.6106     91.77647    -57.03145   -128.21653    -65.97534    -45.72746    -27.86276   -206.58855    -47.10824    -23.48694    -134.6604   -291.80548   -238.62498    -353.9629   -185.01221   -223.30107    -194.8648   -246.28359   -263.12283     -314.303   -297.87772   -495.14111    -10.89382   -121.32437    -243.0058   -162.29269   -461.95566   -258.71184   -443.96494   -499.95819   -462.94646   -322.94855   -250.26528   -476.51093   -387.34579   -342.41377    -370.3381   -231.87366   -340.94041   -550.07818   -401.62237   -326.65207   -344.00419   -318.37407   -407.76018   -414.08617    -346.6291   -268.51563   -301.95841   -411.84316   -602.58095   -244.11829   -612.20973   -501.93608   -502.23116   -432.14536   -443.83033   -590.91671    -392.5919   -810.85723   -323.97441   -527.89085   -575.37451   -891.99137   -328.22943   -507.85659   -640.69683   -761.50931   -773.56168   -617.50937   -324.39712  -1056.31235   -757.23261   -662.18842   -999.32495  -1003.25799   -554.35866   -920.18154  -1093.02622   -1387.1896  -1116.00895  -1050.96671    -769.2854   -1226.8233  -1142.96502  -1180.84274  -1141.10457  -1299.60178   -892.51902  -1128.34082  -1163.06453  -1216.56554   -1104.1635  -1170.76844  -1338.29805  -1437.69907   -755.09721   -989.71602  -1234.73431  -1223.47492  -1206.44248   -905.35267  -1108.87732  -1059.18956  -1167.05212  -1186.87377  -1072.21722  -1548.04065  -1157.86514  -1041.76677   -1392.3177  -1533.88141   -755.25638  -1450.62079   -918.28125  -1508.30054  -1299.05233   -529.24974   -659.89337    -1415.104  -1460.30589  -1720.89876  -1349.73262   -1098.7281   -831.46517  -1467.56792   -1418.3975          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -1.00   -275.88118   -427.91241   -211.05558   -349.57116   -325.35499   -358.32529    -312.9737   -282.23702   -188.43223   -365.26191   -140.39857   -168.00678     -380.839   -130.58997    -227.9045    -333.5464    -45.19707   -228.43048   -417.60469   -382.58274   -268.70418   -290.40054   -294.07743   -225.09364      3.28538   1307.34918   3569.37304   4389.09881   3573.18458   1551.46232    218.58226   -190.16573   -158.68257   -393.89428   -378.23144   -156.47108   -451.89527   -409.83111   -252.05857     -304.327   -452.00603   -550.76221    -327.7924   -365.94889   -359.05278   -296.17791   -308.68146   -369.77258   -286.45196    -149.0679   -421.95769    -381.5794    -298.5119   -110.59963   -316.91498   -349.93885   -327.93527   -310.43275   -128.58959   -222.38661   -336.99904   -199.32403   -410.48353   -382.06722   -352.44266   -397.56237   -225.17164   -334.85079   -187.67541     26.10805    796.73465   1735.99366   2450.31108   2120.33407   1405.58054    633.36668    363.36841     89.29045   -195.56506    -97.61091   -203.65356   -162.96112     15.39368   -187.28436   -243.12647   -301.09328   -205.66123   -199.50044    -54.48295    493.71221   1448.69888   2647.51677   3029.73319   2311.17244   1294.68125    416.35927   -129.77471   -491.67542   -475.85639   -322.52966     37.87818    393.82323    292.40272    158.93237     54.17525      40.7602   -296.18191   -283.65137   -373.91502   -133.93872   -251.97967   -445.28607   -385.67775   -381.18146   -364.68108   -271.50009    -246.7074    -58.44699   -241.67445   -304.11293   -187.66527   -302.30164   -143.92281     82.01899    306.07409    679.62297    854.99998    826.93601    595.80317    267.08003      200.779   -204.53479    -37.26692    -199.6173   -127.52105    -55.58596    531.58065   1383.38175   1870.36331   2642.75016   2775.82001   2313.14614   1662.97507    855.87122    385.75818    266.70762     38.88366     26.30547      42.6424     21.57953     -6.05553   -299.60327   -289.32993   -362.49447   -412.68268   -224.13598    -324.8298   -326.46496   -160.66258   -234.08534   -238.86292   -232.53085   -202.05344   -224.18913   -129.07854     -4.49272    217.48046     31.56401   -162.91246    -66.21122   -154.85882   -152.07818   -323.60416   -152.01875      3.96981    -25.81982    221.03224    293.58566     71.10058    122.01806     18.18989    -82.03223   -154.60194    -60.60936   -145.83407   -192.26506    -60.61531   -295.10686   -418.73497   -401.76307   -253.89928   -397.02296   -197.58651   -181.87509   -210.80987    -96.87483    124.85048    274.17097    184.14026    179.95254    229.63887    229.64857     359.0514   -115.77689     -0.93733     39.52164    151.82454     66.79106    -38.02708     36.31823     148.3885    152.83749    130.27937    -90.80573     -36.5953   -313.34398   -312.87063   -208.46345    -230.3341   -401.33393   -262.96445   -487.82247   -359.83553   -255.10866    -93.87575   -114.90883   -219.38485    -70.07272    -18.46087    101.45236     67.12025    118.65199     76.71949     68.98295    -15.23284     65.24746    198.73851    271.54276    211.78888    102.36377    112.16131    130.42901     228.6458    263.31734   -206.03359   -179.50545   -348.52589   -221.04632   -297.97627   -242.63709   -509.97339    -296.5014   -314.44389   -256.07526   -156.64741    -52.70902     -30.8461    -63.94475     22.62988     65.08599    220.81327    288.70168    277.32333    562.58875     575.8098    487.55717    547.81028    611.52479    248.33994    338.22752    152.13412      20.9431    -34.94921   -137.87725   -122.22584   -169.58268   -338.46669   -127.83811    -68.45383   -329.69936    -72.30088     33.38468   -158.77316   -209.98983      3.88738   -133.54633   -159.59047   -241.62714   -252.63593   -412.28825   -127.31236   -289.55609   -328.53884   -384.07422   -360.14196   -588.24773   -377.58034   -345.98309   -406.06631   -365.84643   -413.35772   -366.87128   -345.54052   -376.04547   -412.44115   -159.81637    -312.5923   -236.05476   -350.29245   -366.90645    -258.4656   -109.06468    -39.12235   -111.21528   -228.31635    -94.66492   -101.07741   -155.77523    -25.18223   -214.97001   -198.76695   -242.44246   -316.59544   -502.20143   -354.51589   -223.35397   -314.87725   -348.81057   -287.93579    -174.0812   -111.46138    -16.59472    -60.29278   -168.55243   -226.12319      1.25776   -183.47436   -157.48335   -190.16095   -188.10667   -161.89772   -294.42943   -206.89091    -194.6196     19.24377   -215.95504    -98.84239    -63.73304   -151.47041   -167.98521   -126.18531     29.91427   -140.18384   -233.99249     -3.97056     98.88053    -96.22725    138.05623      57.5786     33.26005    373.14848    -34.53737     75.01967     118.2092    -39.20963     -8.73857    -80.66258    -18.41878      4.59933   -211.03273     88.49201   -148.83469     13.03355    -415.5051    -26.95011   -151.72516     48.11921   -133.36532   -271.93225   -284.78718    -33.93891   -202.01846   -204.88219     -38.8935   -269.75377   -183.49168   -511.49722   -325.52955   -136.64398   -210.95773   -335.67017   -452.94454   -242.10063   -367.10614   -200.10412   -476.86968    -15.82242   -181.29912   -415.83204   -406.29447    -450.4825   -207.72937   -189.16937   -621.36692   -517.57286   -229.01469   -528.96058    -154.4713   -132.59903     34.68446   -238.70081   -428.30874   -335.62879   -348.60333   -267.59726   -725.48923   -399.25247   -364.66542   -442.73096   -553.28165   -314.91417   -333.18568   -329.90178   -515.10511    -492.0484   -490.67186   -407.73463   -260.91837   -320.42749    -583.0267   -744.97455   -725.34432   -795.68053   -801.58127   -659.72229   -934.73905   -966.76675  -1132.17187   -857.32175  -1204.47004   -696.49638  -1046.05976   -706.38786  -1126.04266  -1074.89757   -770.58626   -771.59927  -1375.69906   -907.57634  -1073.92712  -1259.09669  -1415.27612  -1526.01993  -1625.68202  -1067.21968   -1299.2859   -826.95298   -923.81748  -1502.41181  -1259.57162  -1278.28677   -984.21174  -1172.99972  -1098.94953   -1025.4389   -910.72917  -1497.39576  -1054.58396  -1234.86319  -1024.83219  -1384.85507  -1549.22237  -1105.44219   -948.15904  -1632.54758   -732.09382  -1327.25512  -1316.42423  -1294.02299  -1136.51264   -935.91342  -1430.39147  -1040.82807   -951.84054  -1424.96535   -958.98853   -998.51695   -754.70535   -362.83413  -1137.31469   -392.52038          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -0.50   -185.04851    -265.0281   -127.92038   -319.29108   -208.92209   -173.13495    -308.5591   -326.09367   -279.70412   -365.68653   -193.57811   -277.14752   -245.50738   -318.54857   -362.94451   -394.33306   -169.06453   -383.27847   -352.43698   -323.68184   -350.28079   -278.41179   -261.72574   -185.59266    -43.02225   1328.68406   3659.05278   4444.43398    3497.0109   1334.42046    212.41862   -171.42435   -152.25041   -466.17783   -290.92927    -87.47273   -309.46224    -219.9801   -281.35598   -220.49705   -282.57868   -236.67261   -306.57372   -265.50859   -333.77058   -473.44416    -599.7105   -136.32561   -319.54852   -422.19617   -254.27577   -401.70343   -346.02617   -315.89995   -309.44821   -367.16104    -240.4972   -304.39054   -382.39634   -251.87925   -238.55425   -302.16252   -281.38135   -440.97519   -452.05859   -245.02913    -324.1976   -420.55304   -303.43954     36.71878    726.32311   1761.12229   2534.56616   2143.44902   1350.52021    597.71673       71.845     51.17399     24.50251   -131.76836   -323.22096   -254.87969     -243.005   -224.10032   -228.96206   -216.44708   -306.23109    -97.56589     51.03753    604.49024   1454.86705   2650.96876   2946.53875   2344.81526   1279.17587    265.17052   -152.24658   -387.32247   -321.79537   -150.36811    -40.75447    141.03479    354.32208    157.02873    204.59271    -90.97652   -324.03783   -229.71923    -184.4659   -294.73104   -319.95215   -183.50621   -403.96807   -339.83162   -120.39594   -405.82525   -214.93723   -176.79682   -217.33429   -232.63031   -170.13597   -276.44045   -136.93171      35.5975      390.069     667.4127    938.65351    729.16852    609.88928    190.80141    -53.81666   -121.55269    -333.9885   -112.41327    -31.95955     35.28527    538.25943   1332.07768   2096.04314   2838.44533   2759.34104   2470.14195   1605.74279    963.41831    507.13183    354.97571    114.41549    144.68374     -0.72243    -64.73252   -126.93914   -333.86369   -282.88901   -334.87015   -309.09017   -275.35596   -300.47263   -375.43723   -106.36589   -285.68802   -397.04138   -147.97928   -133.33744   -303.58501    -48.13049     -10.8491    132.08187     55.70945     89.16463   -219.34595    -235.1162   -218.71097   -277.25956   -258.96021    -42.30497    121.37915    260.21522    382.36029    262.31405    302.01893    137.08033      41.1675   -125.57936    -73.36622   -131.31538   -130.75869   -245.82685   -218.86401   -263.04262   -431.67085   -300.14898   -308.19364    -387.3269   -204.91256   -166.79618   -236.87121    187.28051    118.54199    200.94935    244.59629    185.54669    215.18815     138.0707     24.91681     22.00689    -79.42626    -21.62091    108.90499    -26.25658    125.56343    213.82179    144.66113      30.4983     36.98498    -32.97543   -162.78979   -345.17841   -270.64321   -316.37743   -328.09817   -318.59167   -309.92595    -440.4831   -256.36739   -127.65897    -169.9589    -74.97173    -70.44557     55.75682     -73.3983     58.73953    180.22697     72.55519    127.97729    179.86203     76.95032    111.50995    302.86344    221.97159    425.93711    230.90477    303.06715     56.52591    -53.00133    -23.17363   -242.23962   -139.21471   -208.25273   -180.47915   -291.82921     25.59596   -231.86459   -222.82493   -300.98024     20.25456     94.92927     -4.27425    -23.30492     292.1488    144.50301     199.1884    548.37646    326.02467    665.36317     378.2761    758.92278    502.61655    524.34798    280.85402    236.95487    113.82934    102.54489    -67.70574    -78.15037   -221.92444   -136.16139   -223.26621    -93.10652   -178.86311    -50.05952    -161.1324      -87.462     -0.90873      -63.158   -101.43775   -115.82592    -204.5906   -198.85214   -187.75946   -277.17003   -377.17001   -342.14723   -535.03821   -246.11405   -422.32544   -335.97539   -414.51869    -371.5156   -360.24257   -357.66367   -442.57077   -359.13764   -388.61955   -530.17601    -379.2479   -424.60657   -315.83451   -398.38877   -229.90043   -209.77371   -178.46802   -203.46466   -143.33367    -87.10185   -151.75782    -179.8757    -59.42266    -48.98288   -135.38875   -152.64882   -148.76111   -207.71278   -120.52628   -361.34703   -272.81628   -269.15847   -168.53927   -187.81503   -217.99114   -185.94331   -165.39583   -231.91403   -191.07586   -111.15916    -81.42683    -91.20108   -186.56442    -72.11534   -286.14559    -95.42026    -65.96795    -36.41666   -164.90608   -109.38439   -154.91237    -307.3118   -176.13196   -274.07824    -73.13506   -360.40241   -114.73019   -234.72835   -115.53063     15.26787    -53.16587      7.17885    -41.36927      3.19476      2.78412     87.43766    121.74214     28.37224     23.90629    143.46607     90.87747     59.93759    -36.69913     -4.33982    -38.01596    -46.02503   -143.47807   -263.95882     -26.3233    -46.20069   -211.61669    -64.50096   -132.91417   -114.62729   -184.60397   -136.70566   -146.30935    -403.3447   -150.88446   -167.47024   -132.17072       7.2711    -78.11661   -242.12983   -282.51273   -488.67022   -387.96614   -265.51152     -61.3871   -178.44073    -156.0162   -363.71926   -225.08164   -448.87007   -562.15771   -428.57958   -341.99588   -266.00631   -196.82388     28.48356   -502.95093   -272.48685    -15.17729   -107.48817   -476.99965    -205.0812   -450.34398   -421.89958   -361.20315   -377.83993   -409.54263   -304.86716   -488.43881   -220.96424   -625.85777   -431.67041   -417.31917   -572.54141   -308.34922   -449.36306   -487.13728   -520.70005   -696.26088   -398.67324   -467.45104   -651.01702   -739.88116   -787.79881   -827.30671   -655.26226   -591.97758   -492.33868   -676.22731   -869.67479  -1066.24946   -874.61975   -838.01959   -932.51515  -1318.01321   -752.08607  -1207.66959   -901.36434   -992.36919   -927.45446  -1109.27879  -1244.73047   -876.31001   -915.99041  -1181.21246  -1004.27708  -1186.33607  -1233.13283  -1143.37672   -978.16946   -945.80984   -823.47389   -1510.6512  -1088.21506  -1088.12077  -1448.16134  -1114.74215   -814.41957  -1239.75127  -1231.30908  -1515.42081   -627.28991  -1152.34302  -1035.19643   -1205.9403   -905.01609  -1167.62275  -1283.06322  -1197.90717  -1147.09436   -737.06299   -1505.8402  -1179.71395  -1644.27692  -1127.02592  -1188.25474   -854.01656   -547.32073   -700.51398   -963.77147   -872.18354   -210.69891     -588.816          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        0.00   -181.81366   -186.03683   -111.89463   -323.41287   -168.36268   -101.33639   -310.66994   -357.57937    -353.2843   -342.46357   -234.01086   -347.56363   -214.06063   -455.28626   -435.02022   -435.92646   -358.37536   -586.62894   -239.69887   -245.41324   -428.19062   -323.32801   -162.18518    -89.57789   -161.95834   1148.26874   3442.17064   4476.76731   3853.07549   1235.47597    332.51538   -108.14073   -227.10929   -281.85776   -351.38684   -400.48486   -361.95304   -403.49045   -493.17021   -422.01689   -232.83857    -479.9557   -286.82534   -224.00706   -355.43573   -344.93285   -303.35797   -282.40994    -473.7121   -243.83432     -326.312   -271.25571   -462.89042   -457.71068   -513.10737    -201.2595   -385.03908   -269.41488   -239.30423   -151.16682   -246.84734   -240.87973    -293.6492   -200.83028   -266.53598   -242.96942   -271.72804   -307.95688   -101.98423    -53.09309    613.77114   1622.90301   2384.39955   2214.30618   1371.87053     510.8246    483.26855     20.60033     -153.546   -335.37851   -324.48404   -374.30622   -134.77598   -456.48733   -267.05218    -131.9575   -140.16455   -158.80439      82.3483    586.45079   1608.57685   2707.87219   3183.60589   2383.22612   1502.31391    418.20954   -120.77738   -308.07036   -148.98971   -417.07076   -210.44399    165.64498    334.08877    195.93613     60.92905   -137.52814   -253.82424   -338.43939   -284.67904   -248.21072   -181.60645   -167.89859   -177.69859   -327.89173   -409.44385   -229.45446   -286.93869   -222.11357   -182.54811   -250.24409   -311.09907   -454.22776      -125.97     58.89515    496.64571    831.32351    959.63958    779.50777    590.58127    355.62284    -18.65418     -60.7671   -131.28084   -200.49189    -73.37587     -0.78904    487.01215   1199.44938   2202.43617   2754.03738   2850.18282   2389.02736   1649.09742    919.13535    497.22468    230.64575     28.43224     47.48823    169.98552    170.82118      7.64978     -6.43374   -169.32518   -262.10618   -316.14519    -276.3891   -283.80053   -204.81378   -191.20614   -247.84069   -166.97732   -220.85981   -185.30924   -213.09367   -152.60273    162.25911    -30.96358     19.31719    -66.20969   -182.47299   -216.51063   -247.33611   -422.85788   -241.79104   -204.48918    103.82803    221.56913     331.4732     276.4028     58.67237    105.33027    -150.6673   -196.52619   -180.68011   -228.01343   -333.87563   -201.41397    -184.4474   -360.85073    -367.7039   -266.66227    -331.0691   -334.96744   -257.95103   -280.18034    -82.05698     92.37378    -69.61794    202.91394    228.19627    205.67648    258.84669    271.89053     12.16278     -78.5393     82.51963     55.89458    -57.35043     41.85867     213.5864    -79.19119    150.96249      -54.931    -12.81392   -197.85639   -238.68398   -236.40126   -356.78708   -277.45274   -245.20273   -420.21284   -379.56231   -410.56919   -119.71614    -58.89682    -82.23761    -19.18989    -88.88153    134.52579    -75.79538     18.55547    -68.25975    114.56108   -146.57813     36.17231    202.39987    308.78548    355.82378    428.89707    275.70761    214.07199    146.56564    108.86753   -110.48536   -171.12976     -51.2298   -189.22902   -234.73221   -153.75472   -255.49946   -208.25174   -408.84389   -190.25494   -184.70642   -132.67325    -37.15522    -69.47447    -34.86007     39.05546     63.75384     244.7792    654.70203    515.11353    510.31211    424.69152    584.71093    786.75946       640.39    629.50429     433.3996    196.26005    162.39232      98.3238     -5.51026    -84.84684     50.62692   -144.61619   -141.59046    -28.84302    -65.64652     11.48181    -37.75803   -180.88946     78.35299     52.90865    -66.67362   -120.70418   -181.01952   -120.34201   -218.19779   -409.51201   -250.78344   -230.88556   -385.40957   -451.78994   -331.28999    -483.7353   -428.63118   -513.41934   -574.37983   -293.49398   -434.54353   -450.89232   -197.29214   -277.69591   -157.83739   -216.75883    -273.4341    -209.5109    -67.38457   -226.92449    -79.77582    -52.54872    -91.81763   -103.86401   -250.87749    -85.33355   -129.27617     -1.74738    -41.35941   -164.11564   -113.25517   -200.63062    125.24665   -159.80688   -205.85461   -284.42724   -190.51024   -126.65516    -164.4606   -219.30537   -345.61216    -10.26071   -128.35971   -222.16176   -157.46334     -37.5561   -410.31157      2.39151    -67.70306     -4.68643   -199.78804   -124.91372     -4.52411   -162.31304   -192.71708    -194.6141    -243.3963   -118.29806   -160.18392    -53.26642    105.12134   -105.23164     -9.61797   -311.81127     74.94587     -9.96631     26.12606     73.05362    224.70249    254.09935     80.05281    205.18228    182.83819       -9.818    -38.44585    -30.78184    -30.19513     28.68884     70.72149   -183.19309    -26.47958   -161.19947   -301.62527   -169.75603    -73.16756     49.40132   -141.32741   -242.51497   -250.94662     50.22254   -207.15878   -209.77771   -151.82689   -123.64556   -208.16218   -233.09231    -115.4272   -244.80424   -271.08439   -286.43176   -200.61069   -420.24598   -260.56216   -265.01759   -402.71111   -289.32962   -434.35864   -435.67173   -139.13405   -198.21113   -348.24787   -229.88434   -360.64041   -424.54175   -105.78054   -602.51251   -415.12761   -366.52071   -297.51391   -236.99588   -204.66741   -273.98077   -224.28042   -197.02988   -156.28137    -469.6874   -265.73923    -25.08245   -358.97711   -371.76716   -138.38424   -626.77991   -335.29992   -657.20391    -252.9697    -705.6214   -930.17151   -692.39396   -404.40738   -979.44916   -494.16861   -731.98733   -602.77639   -856.28988   -723.70448   -835.96076  -1048.25338   -734.10903   -794.04525   -849.36082  -1100.78166   -942.31192   -796.60492  -1131.87724    -736.2588  -1165.06913    -748.0727   -936.04523   -993.34711   -752.37447  -1091.00719   -1090.1662  -1352.44976  -1189.26923  -1020.64934    -1283.284   -968.95305   -902.57765  -1038.79749   -930.14486   -816.92813   -1297.0029   -1094.2767  -1107.40094   -1078.1568   -807.18708  -1021.60455   -1213.8578  -1283.73279   -764.94825   -1320.5639  -1744.06937  -1159.62479  -1294.86926  -1166.01557   -932.43533  -1311.73498  -1095.54453  -1276.22638  -1028.25051  -1688.29712   -615.73359   -1462.5673  -1529.55214  -1329.21872  -1037.80766  -1448.81332   -1479.9963   -1288.8543  -1026.80976          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        0.50   -325.12151   -258.55938   -225.18556   -399.11666   -293.64479   -290.42273   -329.11688   -363.70398   -383.59031   -246.81534   -235.06511   -305.41227    -516.2964   -383.44952   -234.12864   -336.02801   -361.00107   -561.23509   -155.61895   -283.91793   -388.05574    -207.6387    -171.8573     82.51731     342.0198   1107.94097   3396.73776   4761.05094   3658.09997   1453.60798    260.97914   -281.45232   -113.81721   -135.82648    -330.5935   -244.84264    -53.71775   -313.08443   -434.12338   -373.82999   -157.79927   -305.60716   -311.26549   -384.89243     -383.191   -533.45827   -259.84749   -230.65309   -381.58279   -303.84782   -404.81901   -289.25492   -385.28303   -402.83734   -460.87077   -383.84437   -386.97132   -222.37638   -155.87548   -225.25207   -261.69113   -221.82826   -351.31608   -360.33512   -312.07579   -302.30631   -363.79832    -208.4811   -186.08754      67.2184    735.84324   1603.07863   2560.00717   2043.79733   1478.01192    750.98378    333.68663    188.80075    -96.37916   -224.48811   -308.77188   -289.54428    -59.45945    -391.5976   -388.39576   -135.95602   -303.66076   -347.09183    129.06536    479.02805   1768.21107   2765.44407   3154.68466   2294.22999   1272.09204    551.92018     25.26827   -331.24414   -382.55192   -212.30087     98.31673     44.65866    293.05487    206.96442     80.24152   -193.52919   -224.68333   -247.48704   -390.38415   -416.76587   -438.19891   -290.76497   -450.23244   -467.37927   -252.66197   -223.74377   -258.57656   -292.90191   -160.31238    -253.4177   -195.73468   -185.53923    -73.36673    131.65768    457.16423    730.24165    718.88724    755.57707    520.74615    429.50984      7.29257   -116.87805   -220.48026   -274.84678     -90.1625    -55.07065     467.4153   1236.05877   2195.82446   2918.88078   3030.08606   2401.01132   1722.71279    810.88354    271.40827    123.94042    128.41412    448.67922     130.3853    159.35057    -159.3689   -168.18849   -142.13349   -445.36536   -229.23281   -356.34994   -428.86595   -344.65685   -378.07025   -119.97741    -60.86082   -301.41416    -54.96219   -200.05318   -184.06624    -87.63658    113.05486    -26.69577    -213.7294     -109.467   -202.68508   -266.18507   -211.14982    -239.8126    -92.54443    243.02381     87.21322    250.52269     119.5874    237.60755    138.97646     45.13097   -158.45132    -76.40316   -129.21312   -223.28811   -172.15726    -254.5506    -296.9005   -267.45636   -236.51251    -244.5125   -309.48065   -228.65037   -214.70735   -196.24473     -8.18159    119.55922     116.7813    275.47717    148.42095     242.9238     61.77127     72.75421    -77.57212    -58.03853   -118.58743    -42.55357     90.13752    191.59432    234.30497    -43.48298     -5.77866     173.4103   -151.47552   -294.88698   -230.07119   -298.98647   -154.04883   -464.06997     -366.259   -360.41629   -350.41574   -344.18781   -118.72973   -103.44875    -71.79538   -101.88759   -103.10133    220.80933    149.23656    125.67625    -10.64354    127.84226      52.1755     131.4076    299.27283    257.01182    312.77117    390.56115    259.58044    154.33033     94.85729    145.40683    -37.18537   -254.90927    -190.8982   -329.78048    -321.3766   -337.61028   -244.61027   -256.84128   -102.95104    -123.4469   -281.73051      44.2655     68.77093    143.67395    184.50819    170.53551    318.24514    469.10905    519.47611    677.38778    646.06212    672.39566    615.23698    579.29057    477.56799    414.48514    238.79686     277.4738    -106.0561    -24.74544   -212.86926   -188.09172    -16.04721   -141.97711    -12.62046   -122.29405     76.72388     20.83876    -58.69837    -26.74764    -55.24091   -104.34031   -107.54576    -68.20622   -235.51614   -227.54977   -194.11905   -235.12257   -248.96869   -331.64651   -305.14652   -443.17214   -517.39779   -419.40333   -468.46058    -326.8523   -411.40292   -284.22156   -285.73592   -456.96018   -215.71928   -178.12297   -322.78564   -185.94659    -97.79375   -220.92915   -204.22636     -28.3857   -334.74117    -81.32157   -156.71239      -99.871   -129.82044    -41.62771    -54.04211    -56.77464   -131.77574    -80.79032    -36.57791   -107.99993   -285.68925   -135.86326    -50.22108   -149.72761   -211.63612   -217.76821   -218.03438    -88.91149     41.12365    -180.5806      3.86715    -97.20403   -155.49201    -60.09574   -260.59615    -41.95778   -269.39509   -230.00867    -49.02457    -69.15429     19.22205   -308.16964   -289.64796   -167.97794   -256.19626    -88.17998    -98.94959    115.39481    -70.65036     54.65181    -63.64937      9.33474    209.12522     39.19317     448.1265     59.12761    441.14655     233.5769    105.44158      9.53685    127.34474     66.76152   -151.13414     53.43447    -97.37112    238.29509    -77.15932    -56.10506   -341.25971    -23.59959   -227.15861    -42.50887    -75.42123    -24.32086   -154.27928   -121.11273   -198.62284    -119.4129    -17.12506    -79.45771   -125.73591   -442.63391   -627.60584     -1.65097    -170.1804   -184.26062   -213.67044   -340.46821   -305.55053   -403.48074   -178.57778   -442.26253   -452.88098   -125.11008   -380.10421    -58.88783   -162.62337   -157.89017   -137.30161   -309.44545   -338.87265   -141.88739   -315.24784   -360.55048   -220.07907   -349.93806   -204.95439   -405.10366    -361.4504     68.99553   -419.83638   -391.94561   -250.64186   -232.19344   -449.60269   -120.73152   -444.39323   -259.32303   -380.85549   -770.49155   -729.87866   -501.78881   -472.27999   -618.46072    -339.8666   -567.44866   -615.96952   -566.66964   -711.00882   -776.81784   -859.46767   -931.65544  -1038.12954   -777.50578   -864.62901    -579.6147   -896.24585   -806.03126   -739.99585   -746.96224   -695.33651  -1059.20514  -1032.93536   -538.35618   -952.28325  -1126.16221  -1262.41329  -1087.23347  -1144.68572   -973.32785  -1207.93516  -1162.85802     -894.586   -950.49994  -1211.76926  -1079.90431    -892.7802  -1122.48615  -1314.44325   -1039.9176   -891.18198  -1433.80476   -622.27045  -1115.53439   -883.49656  -1169.39182   -778.32391   -981.22165  -1196.32315  -1180.42744  -1207.84024  -1557.50506   -994.89819  -1072.99213  -1208.61903   -902.97682  -1167.22626  -1271.50551  -1015.68057   -922.77163  -1037.05934   -608.99806   -963.94137   -648.52975    -342.0032  -1116.64619   -293.85918          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        1.00   -468.04991   -330.97256   -339.96856   -478.33738   -419.52344   -474.62113   -338.91999   -365.62828   -392.77007    -251.3234    -271.8932   -296.71147   -582.39288   -333.00622   -175.75043   -270.73953   -346.31644    -338.8432   -142.69848   -265.33189   -209.02058   -134.46482   -231.26153    107.22271    149.33245   1309.42874   3242.79482   4935.25058   3785.46184   1631.04421    369.34838   -245.53392   -155.85516    -67.19484   -193.99428   -134.75141    -221.5859   -203.16027   -279.15411   -226.35767   -283.81643   -513.92096   -349.90643   -245.37021    -509.0392   -369.93008   -331.05117   -372.99098   -352.60341   -228.08431   -326.98954   -391.80057   -368.08535   -227.34563   -391.86859   -165.19597   -394.41935   -433.28559    -43.68951    -143.9968   -212.57766   -365.13421   -276.69182   -299.15024   -313.97445    -341.5279   -365.79001    -158.0064   -243.26179    -32.27366    784.03831    1619.2712   2411.48468   2186.63764   1384.52319    530.77836    246.38955   -144.75797   -349.66618   -383.82011   -428.39625   -519.32154   -356.45363   -302.20719   -296.60926   -292.95334   -191.51314   -116.57823      96.9129    515.86333   1704.52526   2767.47598    2860.8087   2614.51074    1420.6385    705.98738   -287.94556   -197.17936   -454.67331   -208.50787   -144.05217    134.50049    379.15815    120.23826       49.765   -234.22646   -256.21402   -196.38001   -281.67236    -302.3118   -247.02433   -286.87373   -335.59065   -347.34662   -358.73473   -306.57917   -192.22432    -113.1147   -228.45028    -129.8746   -272.94889   -226.97781   -218.18828     79.28835    433.65645    594.18424    842.43665    944.32708    664.90892    517.34562    122.74136     35.68458   -317.92154    -56.01387     -107.093    111.13018    391.17366    1354.3962   1969.87013   2673.61723   2972.86005   2535.28527   1519.10799    866.29329    285.67802    191.46089    116.28177    101.15442      82.9733    174.41557   -238.18316   -169.13656   -253.69962   -432.21024   -287.66848   -207.74232   -147.66433   -248.06396   -272.12244   -212.57101   -300.91135   -199.97467    -78.99349   -170.73907   -154.62876     77.84604    -33.40021      36.5381      5.99171   -146.89977   -256.99057   -304.75436   -283.20732   -172.43942   -104.38136    183.97643    271.57452    259.57063     243.0067    218.50857    164.75869     25.00177    -58.54098   -188.08574   -208.02082   -150.26706   -245.91405   -245.50098     -104.498   -280.62501   -339.52667    -256.7623   -322.03426   -293.02169   -297.55005    -157.7277     97.71947     -10.6471    234.47887    342.15788    304.50888    121.80226    213.38567    187.90708     35.21239     95.85424       1.9194     86.65121     87.15164    212.30851    -61.98746    185.01148    110.68363     54.28267    -96.73262   -194.57755   -439.94214   -141.13342   -333.34675   -397.58511   -277.70297   -341.64182   -317.31693   -239.13376   -232.61148   -239.29283   -100.09887     19.43443    -66.83828     30.24265    102.39432     32.39613    191.70012     55.42401      5.23604    -32.61625    248.71627    316.53399     299.8954    420.58497    251.75281    141.60335    -17.39327     10.33809   -144.49127   -370.92514   -215.27568   -298.78369   -331.30623   -214.86051   -135.37011   -274.15928   -174.03418   -162.56455    -80.27716   -184.58873     28.10264    164.45759     34.13581    164.22492    251.88013    330.85038    554.54838    588.02567    595.72491    718.99665    675.12376    680.57238    541.08762    426.93133    332.43317    182.19081     70.45761     -73.3918   -244.19779   -206.02897    -49.65439   -200.96184      22.4011   -127.32261     -8.36606     41.72648   -118.63024    -50.79832     -32.9846     39.51914      3.13299    -40.91784     -8.16499    -117.3585   -364.11458   -360.03811   -146.53237    -283.8064    -344.5297    -343.4503   -357.62123   -450.49896    -478.3436   -222.83402   -224.45435   -265.38265   -339.65747   -215.45223    -183.6381   -275.28239   -217.50704   -220.25064   -277.22973   -168.66158    -81.96659     27.02364     -73.4954    -18.65631    -85.93061     57.03139    -12.50786     28.11317    -88.97852     -42.3294   -221.02151   -166.05082   -241.46923   -230.93209   -154.01291   -193.67971    -222.9143   -158.45187   -206.72305   -136.59433   -119.40097   -131.75135   -129.09252    -96.10688     88.54639   -115.61858   -138.43331   -239.40739    -25.00492    -28.57926    -84.80817      9.06164   -249.06154    -160.8392    -204.6972   -245.35401   -143.49491    -95.09258   -362.42399   -197.12241    -84.12583    108.93553   -141.41611   -207.79844    100.25526    -44.92573      91.8854      196.042     13.92054    241.27294     114.8074     64.39264    168.04754     308.4121    -86.20579    -50.90083   -142.07078     89.78511    -18.65161   -179.46589    -318.0858     44.01614    175.22915   -113.19022    137.86233   -119.74256    100.31791   -168.22042    -73.05274   -102.46402    -75.75387    -40.68133      50.5265   -236.43602   -162.89392   -120.77309   -233.38565    -29.80515   -181.97356   -241.38847   -331.47484    -99.45781   -393.21259   -173.86889   -166.68603   -251.27495   -150.48283   -354.48025   -173.86647   -318.37145   -303.79238   -156.70723   -403.47382    -140.5192   -332.17385   -163.03573   -245.07793   -377.72288   -288.02805   -271.97274   -264.52763   -300.28328    -180.8877   -528.22939   -174.32767   -229.92872   -322.96872   -252.54703   -182.27664   -226.53709   -353.96131     -320.468   -362.04315   -469.26768   -664.45783   -339.29783   -583.08984   -661.50093   -596.81072    -529.2589    -625.7682   -805.54122   -789.45907   -646.97156   -777.93975   -824.11951   -852.52721   -804.95828   -498.34773   -816.79366   -728.38122  -1096.27172   -837.89325   -914.94843   -880.99631  -1194.16218   -995.98985   -861.98623   -979.78328  -1241.91533     -976.098  -1080.43748   -897.89843   -750.27634  -1031.14854  -1084.72224   -937.65977  -1308.71331   -744.02158   -794.51124  -1045.74466  -1048.56399  -1231.17208   -1036.2377  -1521.61826  -1141.82117    -489.5142   -821.48482   -992.42147   -865.56484  -1423.07894   -941.25184   -820.93631  -1100.28306   -810.41104  -1169.98252   -830.27091   -883.56274    -1299.215  -1058.17032  -1712.32739   -994.81526  -1033.01653   -809.41544   -634.23927   -807.31941  -1288.50996   -171.51976  -1261.20192  -1405.21052   -1448.8061          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        1.50   -406.21967   -334.87912   -394.97295   -379.33985   -306.37986   -446.42861    -245.4133   -347.70642   -333.75593   -496.48367   -378.58542   -341.46629   -362.72052   -307.41613   -242.48514   -225.16959   -329.63968    -70.24805   -147.59475   -232.44639   -142.55184   -176.50628   -285.54607      1.43019     83.59396   1354.99447   3355.64997   4720.49666   3876.60311   1573.54287    232.07554   -106.40031   -245.67068   -173.71991   -406.83051   -346.42365    -166.2698   -142.96816   -495.15235   -447.41075   -304.38251   -395.16799   -429.27994   -315.89358   -276.60823   -502.96309   -376.80953   -192.19835   -122.51397   -503.05504   -390.25637   -433.06713   -281.09565   -357.65737    -350.4987   -437.21155     -265.842   -211.90879   -211.01927    -99.96587   -169.41794   -263.55307   -272.10746   -333.58596   -288.06852   -357.34176   -194.51272   -233.16447   -115.65161    -56.42821    557.23852   1771.78262   2565.11605   1994.32547   1191.82707    445.34918    206.74948    230.89363   -202.02051   -494.90557   -489.49433   -649.36206    -256.9483    -205.8725   -320.10522   -325.19745   -318.51299     -84.3437    101.15256    738.68853   1697.12433   2975.00105   2958.96599    2389.0912   1041.20605    489.29718   -131.69836   -115.12601   -266.85009   -165.85932   -142.20331    235.35625    418.03157    222.17494    193.69864    -89.61871   -422.00487   -288.13226   -319.11067   -347.22631   -303.99213   -532.43282   -512.81669   -299.17768   -360.61406   -259.46711    -186.9418    -94.80833   -133.87278   -332.13544     -51.7391   -128.57487   -135.10613    112.79903     309.7182    599.46126    1028.9204   1017.12882    604.55003    365.84944    219.21703   -221.79291   -151.54873   -221.51272   -219.11214     41.40108    497.62592   1176.52838   2236.74158   2468.87614   2788.93389    2538.4625   1909.06735    917.61002    484.41934    298.88903    215.44531    180.88362     63.03497    -133.4256    -40.27661   -158.91707   -202.56877   -191.54354   -334.15386   -319.98944   -109.34461   -432.15306   -205.06309   -378.43373   -142.75476   -419.68143   -229.77016   -151.83664   -154.96638    -84.14463     17.09495    119.54886     62.23291   -176.17174   -180.26813   -101.70912   -129.33079   -131.70447    -18.26504     15.57336    185.99397    419.55113     144.0255    226.21904    -98.39816     -3.47522     -51.2098      8.32328    -110.9169   -154.25695   -191.51182   -188.59984   -456.48277   -337.55426   -350.25671   -346.44635   -247.72347   -200.16723   -186.41856    -61.80234    130.11162     14.90751       76.573    231.83676    228.42543    339.40251    177.47885      83.6458     -42.5874     72.16533    -20.45939     98.81553    154.31241     219.0849    258.46208      34.3771     44.02374   -123.93504   -286.79734   -223.24218   -296.72992   -311.78784   -252.71907   -394.93768    -403.4944   -302.71083   -227.80445   -193.88589   -234.76056   -159.51892    -84.97634    -22.13519    104.70437     72.08402    160.40207     140.7809    167.11367     43.37178    117.78765    114.43026    142.49694    188.95455    290.89917      249.634    333.28726    236.26398    151.95814    -17.88699    127.04555   -199.73055    -193.2826   -209.72701     15.85754   -223.99805   -215.46832   -294.76382   -221.64589   -146.71007     -18.4097    -67.95907    -11.43971     14.05977      71.4858     31.85474    192.38172    484.16575    423.02723    698.55443    558.91084     684.4103    639.41057    634.82672    598.27453    454.09584     343.4532     35.54101    287.36038     12.89618    127.80597   -136.78296     43.83939     26.94324    -69.74841    -35.46497     130.8537     74.56112    -86.00842    -17.21056    -57.55894   -173.71229     74.19593   -179.30037    -79.20496    -96.55334   -333.81065    -240.3807   -421.60456    -179.4838   -343.48701   -421.47645   -391.25862   -251.66281   -410.82031   -393.09999   -262.67552    -316.0288   -327.10609   -149.21161   -334.00096   -315.14165   -189.40355   -143.92829   -284.86643   -259.84324   -184.08907    -76.61984     61.59418    -22.40231    151.21662   -161.90782     68.87441    -10.33555   -144.85363   -257.60037    110.82599   -148.00266   -241.07218   -139.35702   -177.24414    -321.8018    -93.18284   -209.53826   -200.85242   -204.11459   -221.09326    -153.9391   -117.76932   -164.86654    -50.65599   -166.92082   -205.65646    -29.32645     44.02028   -103.11077     -7.15946   -247.04216   -108.77474   -216.47183    -60.05127   -177.67998   -124.93413   -188.87284     34.26075   -101.34469     -82.2045    -190.9475     -14.2529    109.48235    -67.28765    104.41952    282.86366    232.72505    206.21261    250.69697    219.75645    226.25464    218.15695    172.75108    223.60003     166.0553    -20.36686   -214.26855     19.11196     -2.06278   -127.77488    -122.2423    188.66189    -13.16874    -83.11859     45.51118   -261.00322   -157.60326   -241.68765   -210.62355     -8.15502   -102.20824    -206.6048      -19.821   -144.00085   -155.54845     78.20483   -226.43662    -85.43289   -207.01404   -513.69229   -215.13824   -254.94816   -353.37693   -344.06015   -484.76499   -311.62666   -297.51917   -469.95213   -489.40483   -500.12328   -389.83962   -466.65193    -244.2622   -287.25946   -181.57836   -262.82444    -398.6915   -252.99852   -292.00464    -97.35726   -282.94643   -340.20406     42.76324   -183.51643   -431.15838   -200.34574   -191.96798   -472.03605   -186.35738   -347.62694   -407.46052   -434.81417   -317.09406    -247.9477   -538.04833   -518.91405   -454.22462   -445.18446   -326.00262   -567.84824    -647.8474   -713.91414   -651.81709   -715.41294   -774.20342   -535.67957  -1016.72468   -734.79067   -868.17625   -683.12639   -798.20512   -846.43519   -765.30579   -725.97523  -1138.46371  -1045.68392  -1021.78012  -1153.10447  -1008.14966   -721.79436   -1112.2859   -832.59102  -1043.15042   -742.61104  -1110.04221   -1028.1273  -1055.20403   -916.06104  -1155.63207   -836.45132  -1071.12238  -1108.47539   -942.83708   -939.98145    -1286.134  -1167.57671  -1153.54966  -1049.24363   -944.61422   -947.23877  -1128.07976   -668.67533   -1253.5822    -844.2663   -993.32845  -1435.67589  -1189.75617  -1434.16963  -1870.79002  -1092.43565   -998.50487     -818.236  -1438.89918   -813.29737    -455.5648   -1037.1879  -1186.10212  -1302.93096   -852.75098   -877.77102          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        2.00   -158.19075   -299.75646   -426.27261   -241.97828    -179.3015   -425.47347    -140.5034   -328.47413   -292.71961   -630.63766   -364.70868   -389.19556   -254.20998   -291.20197   -248.76368   -129.27712   -387.69143   -233.99054    -90.35465   -218.01869   -395.95115   -383.34911    -312.5056   -162.63518     127.7333   1253.41182    3526.7923   4349.73332   3779.97873   1617.82562    213.23946   -108.89836   -274.21932   -217.74259    -292.7108   -182.53233   -237.48173    -52.31106   -490.36804   -450.21234   -417.61378   -282.89187   -302.17878   -341.00999   -271.64267   -475.08709   -486.62619   -342.45589   -410.83959   -405.21484   -304.89777   -382.81038   -327.39199   -303.29603   -258.39534   -177.15535   -199.77588    -301.8527   -180.58799    -74.61521   -188.12968   -271.15981   -444.77807   -264.91688   -223.38132    -88.93626   -354.94068   -228.18612   -300.86799    -61.61211    682.20214   1754.45152   2381.50872   2095.43028   1412.35716    619.17644    -29.96125    102.51193    -80.21851   -358.90429   -362.36923   -275.22197   -492.69123   -209.77122   -313.82951    -86.10202   -188.30577     53.58367     28.95749    535.50006   1685.08679   2791.85451   2760.53172   2147.00561    825.70602      140.105     61.23036    -176.4562   -311.29977   -265.92709   -173.79742    202.08651    402.56729     104.8656    -33.54666   -202.91286   -198.86801   -369.97982   -357.99247   -359.54265   -257.65968   -469.29757   -320.18599   -325.37958   -289.10112   -303.09525   -251.38383   -153.26697   -170.15848   -208.46844   -234.62002   -304.25532     -96.9419     55.55093    351.42635    782.66223    793.84116    835.74122    604.49098    360.12155     74.39721     -8.28732   -170.89413   -157.44211    -98.14726     99.77855    428.68267   1245.89722   2135.32417   2774.03363   2868.90309   2504.15868   1551.64649   1137.10075    578.95482     331.5358    144.60765    130.06943      90.8761     73.29218     -9.13162   -241.64239   -317.26034   -283.02714   -286.71644    -228.2053   -359.30754    -343.0704   -395.25885   -177.88897   -384.98903   -246.35212    -51.27235      -190.99    -17.61964     65.75553    116.62073    145.64201   -130.52911    -110.8845   -170.01481   -287.74411   -232.86443   -198.49077   -116.80498     49.17895    199.67741     98.87303    323.52411    165.93282    268.80382     81.39218   -129.08418   -187.59239   -151.77558   -156.42528   -210.12805   -261.54042   -266.58332    -231.9076   -215.65105   -191.48147   -167.93874   -211.62195    -231.1515    -34.99584    -36.46506     95.18562      47.7224    220.10262    166.55561    217.93789     -31.8498    122.99803    -43.38477    -200.1576    -46.44776      7.27841    145.56476    221.00113    152.84799     78.55136    -15.59262     -90.7993    -32.75739    -200.6735   -388.28513   -480.43235     -6.99322   -314.09801    -189.2851   -239.23261   -311.94797   -302.10495   -307.40513   -290.86893    -11.61567     29.40685    -43.73544     -38.2551     67.32985    133.23482     141.5469     83.60914    145.88076    217.89168    226.39184    452.67924    355.13533     367.7157    238.57532    209.20831    243.32169    -40.30534     -44.9977    -62.65376    -256.3645   -150.27392   -141.09684   -184.75243   -174.24074     -187.258    -77.14361   -159.32697     -9.44283    -60.90069     38.33241     19.91101    381.68506    100.11977     34.70697    489.12315    478.15001     744.7094    730.71873    852.27137   1003.83057    901.89897    562.62763    340.18275     420.6006     36.44077     86.39297     -4.75611    -94.37335   -123.60816      4.85176    -33.16258   -114.19173     76.19388    107.15505    142.26553     72.68512     39.68005     35.96332   -136.93121     82.65284   -123.84635    -131.5805    -155.2029   -339.00882   -278.88845   -384.39134    -81.90325   -250.41203   -248.06456   -335.55137   -444.81396   -334.45628    -400.9394    -352.2958     -184.291   -146.83979   -407.90416   -329.84292   -197.11011   -125.51836   -275.97605   -236.75278    -71.48271     -69.5782   -108.15934    -59.48629    -68.66108     92.22803     -48.2693     16.63946     83.82134     31.75784   -118.71221     23.32455   -224.94964   -311.74944   -304.52381   -208.80519   -253.12393   -142.53905   -359.62701   -387.04472   -369.04252   -107.49958   -168.69968     -61.6819    102.74288    -37.20475    192.07039     23.01304   -129.65189   -152.69589    -92.22855    -173.7902     64.31623   -106.43386    -29.52484    -86.77162   -218.83198    -85.83488   -248.28986     -98.4873   -295.09645    -77.97483     88.18643    -69.77509     73.54251    -61.35452    -12.49439     -5.51414      -4.0297    266.32025    184.14218    308.41062    202.56041    156.74712    155.89803      58.3736     61.68368     14.98164     39.07389     96.47793     43.94106    -97.76851     137.1711     26.52878     68.44558     50.86187    101.10033   -325.37378      83.9929     -63.6181    216.50705   -221.19973     98.73997   -162.06343   -213.96775     -9.69375     17.73252     114.3167      53.2597   -355.30967   -354.68951   -172.54509   -235.12928   -156.84315   -103.44117   -108.40655   -272.76058   -389.71948   -292.50096   -186.62311   -188.19752   -170.63709      -321.77   -290.67075   -261.38505   -212.48538   -343.01664   -339.38089   -241.89181     -76.3359   -229.74666    -92.95273    -87.25185   -258.70069   -130.44545   -212.76615    -269.1956   -112.30612   -380.80602     -90.2575    -278.3486   -290.06952   -331.49799   -233.69795   -361.31485   -209.20758   -596.16676   -616.21612   -506.77527   -427.91945   -708.44265   -462.60382   -635.14219   -753.90606   -631.93283   -982.97498   -820.91224   -744.54371   -815.49279   -472.89436   -869.98702   -737.52434   -891.23785  -1079.59731   -944.66225   -815.72636  -1099.51104  -1100.06731  -1173.75716   -990.73489   -939.66932   -913.34136    -783.2398  -1012.81115  -1018.81759   -944.88445  -1122.72623   -754.89686   -825.37717   -882.29395  -1131.05146  -1193.85806  -1183.71195  -1177.83525   -785.61246   -928.44317   -724.92257   -794.08583   -1074.1995    -914.7615   -771.75072  -1060.45547  -1149.60549  -1102.28278   -839.78963  -1035.56988  -1304.20283  -1029.36082   -643.47709   -653.89971  -1142.72618   -179.77176  -1055.50445   -978.94497   -421.76439   -983.06969   -994.95485  -1048.15405  -1624.75653   -368.30038  -1052.31745    -617.2519          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        2.50     96.96423   -261.92267   -443.50997   -166.59813   -206.26045   -443.00874    -365.1154   -260.96217   -324.87889   -531.96584   -193.78057   -432.60782    -192.0143   -277.77764    -277.2263    -54.39788    -396.7762   -348.00769   -118.50805   -312.49755   -347.48792   -347.40793   -316.77174    -128.1382    -26.43776   1090.02825   3589.03703   4235.79719    3664.7291   1549.52977    199.12477    -85.73128   -219.73331    -91.44447   -557.24546   -203.30283   -411.11829   -372.96094    -298.8241   -330.12957   -307.13335   -250.28434   -247.97261   -345.11504   -341.76523   -358.54875   -461.43522   -304.76096   -217.02087    -432.0459   -332.45247   -356.28597   -469.15745   -381.77745    -315.6373   -263.17153   -248.68903   -212.37807   -292.19163   -262.65179   -417.54413   -179.64332   -197.08369   -301.22736   -285.40133   -307.99307   -331.04782   -320.58147   -168.72271   -108.67309    876.02167   1808.81535   2310.97397   2123.20968   1463.91603    666.32842    204.30496     10.23553   -127.47214   -379.17494   -370.39292   -434.51876    -151.8625   -177.87731   -315.95735   -261.54795     -97.9642   -162.91614    152.13348    645.83828   1499.53984   2594.96171   3012.37813   2194.78468   1084.02807    323.86054    105.06972   -261.85805     31.31428   -368.93953     -43.1467    276.80386    220.60356    312.13703     22.61014   -115.87104   -200.47003   -312.91523   -164.85527   -486.97484   -300.17036   -439.00432   -327.46923   -239.65764   -288.46332   -255.63166   -166.51348   -256.85328    -74.00068    -191.3624   -331.03765   -573.90472    -81.01456     58.55414    446.26205    786.54117    864.82774    737.66046    589.13822    321.88875     55.92302    -50.00802   -226.82867   -289.28503     -25.3637    -84.17986     564.2806   1241.42834   2166.21433   2627.28092   2918.35254   2365.85436   1602.38237   1112.27956    407.35237    226.95833    107.75444    198.12403    284.65988    140.06882    -162.4388   -103.41723   -332.86014   -248.90056   -254.94893   -256.63398   -397.80392   -303.64089   -183.13255   -299.86055    -31.23471   -256.32959     -79.0616     32.77095   -262.67604    -22.20317     150.9404    -45.59306     63.12297     26.12417   -309.57243    -304.4126   -223.44984    -38.86282   -199.70015     90.34611    214.82339    294.35826    116.72505    353.78733     70.89743      2.33532   -138.60407    -236.7396   -191.92238   -116.47322   -256.61094   -123.33671    -290.2175   -574.67202   -314.20631   -376.96825   -263.53346   -278.63094   -308.73777   -149.32856    -99.15198    -26.56792    111.98931    302.34387    138.02174    361.50754     10.41676     36.87092     -7.91583    -23.55789   -125.75166    102.43038    161.01597    239.18263    102.53643    115.43112     18.26868     12.84085      0.71575    -30.94604   -324.71895   -261.72012   -341.75935    -373.8419   -380.07087   -373.39282   -491.18074   -378.12367   -235.77753   -192.60035      0.69147     12.56873      8.66677    289.08427     31.58802     26.31095    250.44445    115.71698    308.85491    176.61115    333.50658    282.43541    292.44876    282.61991    167.87958    198.54591     98.71924     72.46494     -91.0192   -128.93206   -336.41011   -179.86221   -397.36789   -290.40397   -336.33137   -263.44531     -85.9582     23.37648      0.65217      9.86055    168.25278    121.34952     206.3034    288.94323    220.48921    244.80057      550.216    634.06726    766.88204    737.10481    635.58369    862.32559    445.69521    475.81139    393.79383    203.35233    123.44771     -9.94636    -42.54853   -110.93126    113.26794     40.10221    -49.06489     66.74024    -83.48149    327.05177     80.73526    147.83268    -41.06154    125.77524    201.32253    -19.95508   -128.70949    -190.7484   -210.77273   -116.26667   -190.03797   -397.17418   -427.89082   -337.33058   -171.05515   -213.67922   -434.30291   -331.24806   -424.46359   -405.79966   -375.09554   -237.32142   -338.83715   -375.41662    -269.9364   -184.04317   -208.93522   -163.16574      82.1931   -243.65997    -74.79076     48.88039    -47.17674    -35.16792    223.11481      4.04735    -36.59648    134.74297    100.56612   -211.47741    -246.0954   -136.52673    -90.21181   -123.00127   -123.38056    -28.58661   -265.36213     66.03817    -31.03365   -201.24454     -0.06118    -46.85776    -200.2701    -78.26757   -148.55026     33.75804    -27.24792    -73.48609    -133.5182    -64.21137   -339.37927    -61.26706   -288.52835   -361.10295    -124.1248   -201.96652     -65.5506   -115.27388    -87.16821    170.48998      64.1425     19.61696     13.76951     34.93111     58.45503     16.06135    150.91997    267.38299    209.94357      95.8921    225.44814    176.47788     -11.0899     41.16308     98.25488     33.94617     -111.905   -227.47702   -128.77915    -14.45599      7.47527     17.79717      6.71626    -17.07856   -141.33479   -288.01637    -81.16593   -161.73889    -41.24654    -41.68963    104.37594    184.09745   -174.67916   -241.85192    -115.5631    -47.21641   -121.54427   -295.00292   -309.61687   -119.63856   -433.60436    -49.62828   -294.27783   -159.49106     -0.33741    -56.80269   -508.99395   -227.06834   -189.13505   -238.87887   -215.54327    -101.0958   -247.35917   -351.04235   -331.09836    -85.29238   -348.63112    -48.99757    -302.6689   -452.97737   -215.90792   -287.41876   -223.34471    -316.1938   -395.34562   -391.38528   -147.29808   -186.98129   -411.29537     -298.333   -455.89154   -489.74421    -276.9565   -378.79372   -604.21787   -469.25731   -386.68663   -661.46447   -426.24773   -383.79802   -709.16219   -712.76995    -810.1415   -612.73644   -802.62797   -785.39849   -581.30049   -808.34197   -620.70584  -1151.05739   -626.85912  -1070.51321   -970.16825   -867.82949   -847.89539   -893.11228   -789.35358  -1229.51475   -889.92694  -1034.20562  -1161.85128  -1049.23289   -1062.4468   -997.33401   -711.92221   -755.79961    -767.2078   -990.57593   -807.72338   -721.65285  -1184.09411  -1169.42105   -891.98195  -1469.02345   -865.22872   -853.73522  -1163.99079  -1018.19235  -1074.62615  -1387.14141   -847.53962  -1609.95347  -1056.85741  -1395.36186   -668.96744   -659.20966  -1145.83506   -597.02606   -727.49411  -1117.15674  -1285.65402  -1460.91112   -780.05381   -1309.1499   -174.23984  -1130.52751     -35.7439   -400.52416   -909.05011          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        3.00    146.28065   -214.98188   -413.50725   -211.42506    -352.9257   -464.73545   -640.90555    -196.4997   -346.47352   -450.45779   -198.05491    -220.0988   -121.84855   -392.75903   -431.34802     -3.58072   -328.94508   -421.47504   -160.17251   -408.66238   -307.10999   -285.29831   -237.21533     -16.2153    203.53824    947.59423   3606.57604   4764.00575   3676.00384   1412.75351    245.99993   -229.17915   -248.02365     -293.464   -487.17866    -163.6488   -269.35073   -595.79584   -191.56521   -215.85632   -291.89735   -257.51457   -466.78983   -312.14209   -281.31547   -366.55218   -311.36053   -616.23471   -340.09233   -476.33783   -407.21939   -252.95824   -251.11439   -317.69069   -231.28475   -205.25256   -310.34534    -140.4891   -216.69096    -71.58915      -360.06   -258.57095   -168.29846   -245.32185   -365.48411   -259.26603   -351.30388   -101.45287    -150.8092     28.27831    685.38791   1794.46985   2541.51542   2238.20143    992.81871    762.21341    222.30763    -12.68262   -339.49516   -225.20039    -292.3383   -323.15399   -275.88516   -249.96408   -338.79822   -120.05559   -146.53578    -167.6552    135.40472    558.02209   1832.59221   2844.27269   3193.14713   2407.48139   1389.38487    350.36093    -12.48923    -129.9323    -91.35893   -235.78015    -16.93093     280.3289    416.44344     79.02029    177.34329    -77.22356   -254.46469   -370.15732   -416.34954   -270.48383   -265.67827   -288.38106   -361.63585   -315.52603   -190.65776   -358.78485   -186.80209   -140.11489    -79.99916   -301.43432    -302.3734   -226.69126    -78.57481     97.62921    437.97838    672.61607    579.34761    809.39432    420.73539     355.3309    149.71553   -215.26778   -239.36556   -104.91462   -157.04233     25.83368      617.652   1108.03496   1901.06508   2752.90962   2674.90305   2385.20578   1738.28191   1218.88845    740.35124    172.22166    209.71242       8.2292    217.53284     107.5583    -44.02836   -176.60523   -398.69008    -236.8717   -320.64966   -330.00861   -223.26826   -274.31396   -333.12441   -232.87898   -323.36007    -293.5117   -206.88448    -13.66821     -6.30133     33.38557     73.82452     -19.6478      -16.998   -168.99395   -156.10974   -268.71222   -373.35694   -106.50114   -113.88431    262.03681    347.86516    261.90093     38.56929    139.86536    -57.84372      9.54687   -100.63064   -145.38678   -207.23943   -238.55146   -229.31812   -175.87067   -304.81211   -274.81327   -251.04666   -306.76988   -230.86934    -84.54558   -162.89988   -258.38252    155.07611     51.07579    364.89285    459.98743    424.42233    190.49149     57.97184      5.81329    -52.97654     60.76772    -54.72228    197.21102    297.14263    207.55161    -31.70685     286.1681      1.08676    -235.2911   -211.66222   -395.16246   -216.08353   -299.79426   -173.21357   -354.25086   -308.19557   -270.47346   -376.44306   -211.31883   -271.64591    -99.15406   -178.00092   -105.96246     139.9207     74.05562     65.11225     91.10547     97.40588     72.10967     71.07635    366.12507    268.94192    430.60435    323.24037    478.33475    317.41519     323.6328    119.84016     54.91496    -42.52664    -147.5385   -114.19387   -303.00803    -143.5692   -312.83428   -232.16755   -209.43926   -237.49539    -77.69224    -35.58589    -90.58718   -315.50693    -10.86153      89.4341    160.76453     372.6057    437.16676    535.29572    819.32602    644.83593    659.53341    706.76679    708.06379    682.46686    616.00369    366.06016    217.25275    106.99348   -227.58546   -194.65718     60.38519    -61.75974    -79.95116     99.13076     -2.90808    -29.41333     40.39113    156.52719    102.90226    -66.50581    -43.24622    -44.19689   -228.56435    -45.90588   -136.88974   -129.07724    -77.18378   -385.20891   -496.75759   -283.32724   -323.58993   -392.20738   -192.03819   -304.00212   -406.90688   -289.06873   -314.99184   -323.88445   -270.55346   -294.65768    -91.84838   -326.07827   -166.62503    -34.20181   -169.40478   -106.55049   -122.70634    -80.25913     67.43407     136.5483    334.00264    176.13953    -60.37192     -45.8923   -217.83835   -190.34669   -109.80337   -126.65189   -142.06188   -394.86423   -129.32001   -123.56057   -444.88397   -355.60566    -94.43113    -95.53633    -56.33373    -13.06348    -51.92113    -79.35425   -130.45142    -119.1939     105.6573    -86.40231      7.07751    -287.9946     60.21595     -3.29936   -152.76015   -201.27287      -17.584   -189.88266     161.4023     16.36994   -183.25917    -36.35145   -124.67667   -170.17336    -81.90127     188.1069     96.89217    150.75124    120.41477    267.10741    151.75202    122.40709    132.41527      295.432    237.85862      3.05681      17.8415    103.37866    106.59089     26.18779    -12.46763    -50.51994    207.21069   -224.11534    182.01001   -234.90823   -107.91785    -34.81354   -168.41998     58.70248    -15.05209    138.93732   -235.56203     47.69749    -31.00451    -114.0956   -148.06477      1.80621   -164.96931   -220.10736   -421.41536   -338.97245   -483.22275    -486.4007   -456.73138   -184.48158    -107.8758   -194.89121   -102.58352   -509.14789   -156.94607   -187.80922   -102.28246     36.62553   -324.97977    -185.5032   -139.59894    -236.1955     -206.623   -263.33289   -209.77219   -259.05591   -341.41923    -41.07763   -324.38787   -400.48567   -350.38972   -340.15041   -342.48933   -401.90382   -183.31635   -154.65841   -368.18921   -123.56631   -484.03004    -452.5636   -426.15298   -575.30003   -531.74951   -298.90815   -461.41771   -754.37054   -624.75569   -617.71082   -704.95602   -565.29297   -715.50638   -675.60372   -879.93547   -454.34275   -864.43649   -738.62034   -898.15906  -1092.08497   -861.99974   -821.12048  -1023.40633   -804.66722   -1109.7488   -842.54699   -968.16145  -1081.61526  -1067.45279   -854.80409  -1082.37423   -1100.4902  -1133.63296   -719.11377    -1108.012   -879.00173   -824.62628  -1223.38849  -1113.48871  -1129.22637   -897.89484    -772.5596   -734.08198   -738.59659  -1004.55025  -1057.34824    -943.2613   -822.07152    -860.6087   -729.11409  -1185.30727  -1208.36352  -1017.70029  -1540.62959   -1259.9268   -486.52163   -840.65079   -730.71352   -768.41107  -1137.61111   -996.97655   -736.38179   -891.68574   -131.84921  -1064.62427   -821.77972  -1004.82082   -742.96815          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        3.50     96.82127   -169.47014   -385.68956   -260.21815   -446.60862   -410.07554   -532.21851    -349.8257   -291.47598   -404.82943   -353.39375    116.13477    -65.23372   -500.78849   -501.23012    -58.77362   -355.06513   -351.01106    -79.14501   -437.58789   -337.49169    -254.6599   -244.65333    -37.02943    190.39711   1159.35316    3690.5602    4786.0659   3394.90167   1593.73996    404.62425   -314.68382    -155.5841   -385.57106   -315.21794   -395.78431   -271.13034   -347.35208   -390.33288   -457.01622   -303.01624   -342.00158   -540.63808   -197.77628   -431.09135   -452.91531   -432.21941   -167.69901   -341.53577   -400.53989   -447.51941   -494.81812   -208.79661   -453.87501   -258.97313   -377.24816   -393.57666   -238.52973    -88.52575    -33.05702   -533.20159   -455.34951   -353.19852    -431.0128   -375.37474   -429.94366   -262.14636   -400.10317    -269.9782     67.82043    558.97762    1812.4899    2503.7042   2076.57468   1288.81212    689.01936    371.60136    208.02576   -174.72119    -128.8373   -248.56475   -215.91522   -347.88442   -190.50705   -327.08716    -45.88042   -479.22052   -199.72903    193.95207     731.6569   1605.01288   2664.39679   2865.92148   2414.57313   1324.13858    182.63624    -59.92966   -180.80467   -418.74669   -196.85074    -20.36696    245.96511    391.42167    146.65221    170.62488    -33.26488   -181.64952   -197.81887   -161.36109    -452.5616   -319.72733   -339.20909   -339.72797     -351.595    -29.74755   -274.52317    -97.09481   -199.97682   -315.65214   -159.95712   -236.34375   -135.75882   -237.03518   -166.69143    405.70186    804.08272    994.85642    776.11367    683.67173    332.82572    151.40186   -140.05904    -81.86744   -190.55483   -105.29636    -18.76156    481.16665   1260.73634   2136.61628   2526.94889   2746.77968     2343.127   1597.27326    986.64654    294.58552    171.31241      99.7264    144.78709     90.95878    149.38511     33.65784    -97.91179    -173.6097   -309.25583   -352.89218   -300.09076   -251.74804   -460.32552   -205.56423   -303.61146   -259.02703    -213.2582   -153.42293   -179.46579   -100.75116     81.37204     33.26022    -26.84629    -22.43071   -132.75525   -181.00118    -80.15181   -221.80577   -172.72015   -129.21982    127.71877    245.07795    283.30517    304.49544    220.61803    174.98342   -144.10208    -17.95001    -91.60247    -72.59469   -171.42752   -188.53831   -282.42857   -257.12002   -320.82076   -184.61422   -198.81356    -233.9538   -179.11196   -150.91306   -113.47219     -56.8702    185.08103     323.7205    264.69517    335.74398    158.92743    175.44591     15.90841    -98.42175     79.14982    -58.05243     -12.8954    102.41175    205.36608    165.64276     230.8064    -48.22735       9.9927   -147.71523   -203.97354   -151.17292   -270.01586   -317.55963   -377.23975   -323.22437    -201.0894   -270.56012   -240.21836   -110.91484   -143.80804   -160.77156    -88.58529     74.28389    -54.29969     82.45753     45.50619     52.20372    176.39682    155.18309    257.83421    396.57184    530.82455    350.50303    267.77668    185.69682    171.15062    147.31168    115.10252     27.47954   -134.67342   -211.61952    -196.6238    -92.41333   -206.02984   -254.29888   -150.11359   -237.74964   -145.36293   -127.69464     79.83198   -147.77759    -11.07085     166.1471    299.52691    225.76844    411.09094    481.11192    674.36707     830.8913    796.63104    779.11858    682.78239    679.29686    581.44971    102.92327    248.55376    206.86061     96.64935     18.79747   -123.52806   -149.84238   -213.78675    -97.94728    -53.79062   -140.79206    129.17703     48.66965    -67.74307    167.06964    102.67277    112.63003   -248.46215    -127.5663   -233.59355     36.08701   -254.46017   -295.27132   -186.61511   -387.26175   -240.81293   -311.34076   -277.79925    -229.2458   -181.17172   -259.91679    -84.95272   -109.85558   -356.38563   -225.69624   -221.35232   -292.87667   -158.02842    -107.6852   -151.52478   -206.30792   -156.93486    -51.57856     73.07699     30.43833    128.23548   -112.32863   -101.68715     84.88335    -82.56262   -148.31112   -242.70245   -156.52036   -282.17777   -258.24366   -169.52902     79.79738   -145.03777   -210.36674   -135.79001   -352.29399   -136.49077    -24.80257   -172.23995     22.39324    -80.51817    121.18707     -23.7777     -47.1882       8.4703    -11.60003     -17.3482   -100.24919     92.22202    -97.67928     -118.013   -266.44904     -92.4616   -195.98644    -20.45699   -137.59868    -69.81876   -174.05147   -193.47362    207.16044    185.14594    -86.55966     -2.30065     92.87012    280.36374    171.10575    288.27992    162.11147     81.60983     95.07202    -29.14048     -5.93367     -36.8122     67.11645    112.56181    -85.18036   -109.35808    114.00076    224.04994   -185.27005     -244.222    212.14056     -8.44709   -252.85456     76.22153     21.92919   -155.37893   -260.21786     66.66992    -82.28442   -227.15935   -104.53105   -301.48104   -177.51601   -311.04316      8.96076   -131.83776   -235.59606   -236.89664   -160.61159   -211.82921   -295.26117    -277.6086   -156.71296   -154.53739     -60.0753   -166.25016    -51.62285   -344.34254   -340.04674     32.32729    -11.60761    -76.11439    -241.5899   -259.60208   -238.32464   -273.08875   -377.97305   -300.50025    -97.25254   -252.12027   -215.94223   -201.16873   -285.37256   -384.82484   -297.75578   -548.28665   -121.55376   -314.86662   -446.49473   -362.81024   -549.97057   -521.52586   -676.09009   -602.77219   -491.31954    -598.6421   -552.27159   -701.07192   -604.54535    -608.5405   -694.13776   -844.16605     -750.659   -554.36373   -729.39568    -664.3215   -828.31769    -710.1279   -975.70406   -731.11473   -911.63094   -664.06134   -759.07189  -1086.07003  -1020.67075  -1196.67354  -1152.87227  -1119.99675   -900.71352   -847.00298  -1009.53145   -794.42374    -813.5904   -705.76976   -1166.0696    -749.1053  -1023.33156   -895.53097   -963.52851  -1147.05571   -1047.0542   -873.95259  -1060.16265   -672.21209  -1096.89668  -1312.38406  -1071.47751  -1014.50488  -1396.23887  -1165.68367   -512.76439  -1095.85458  -1582.98349   -945.09548   -764.32681   -896.97447   -813.98797    -803.1055   -880.73936  -1138.66359    -349.0634   -225.37066   -598.96022   -907.28261   -856.32378          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        4.00     48.44132   -163.45622   -291.57821   -249.98544   -296.96316   -271.77964   -260.68822   -517.10447   -228.82737   -344.92446   -273.75903   -147.35015   -165.04586   -619.66201   -494.84944   -169.00858   -415.57291   -287.59855    -16.08924   -424.37803   -578.29659    -467.1549   -471.12164    -153.9207    122.17537   1215.63855   3466.03953   4848.01904   3548.08557   1489.00808    404.29212   -237.04833   -133.69414   -305.91665    -10.71711   -212.69998   -174.57611   -267.18539   -262.29716   -269.59366   -283.57725    -158.6659   -206.99715   -268.58825   -227.46293     -308.918   -502.61488   -230.39787   -482.01648   -472.69726   -259.31467   -270.43683   -379.84411   -363.03759   -399.22197   -341.49435   -258.33894     -16.3248   -144.14234      6.34293   -271.78016   -259.31496   -257.46778   -392.44414   -373.62688   -234.66747     -308.218   -206.64797   -227.76366    240.68636    745.58999   1683.78971   2332.83565   2217.30877   1358.55425    645.50662    533.31401     33.19253     26.18175   -253.86448    -145.9288   -284.92511   -178.71069   -335.71331   -276.82628   -290.88334   -334.58321   -273.30158     28.96451     631.5946   1650.96075   2749.33783   3273.82466   2457.83483   1320.96145    518.75319     82.31821   -218.23773   -131.74063   -234.38114   -110.38648    251.08994    388.13659    207.72513    133.18285    -141.3713   -243.76065    -212.3271    -244.9447   -424.11797   -315.33262   -237.52207    -344.9672   -399.18436   -334.01755   -249.73414   -215.32137   -103.91232   -277.33054   -173.20872   -309.75785    -185.6345   -101.13759     38.62228    466.06572    836.73353    922.24334    690.15296    432.26305    391.54905     49.99898    -42.68454   -143.08576   -176.60264   -117.09851    110.36253    492.68558   1264.50467   1977.82065   2518.18698   2911.03084   2429.66331   1616.25041    998.28429    508.18366    296.54912    137.08133    259.69727     93.32021     40.09169   -162.38435   -114.54805   -317.63623   -277.81642   -416.56377   -208.76477   -224.39197   -277.89542   -260.20442   -204.12188   -253.40198    -235.8472   -210.04709   -161.31681     59.63662     -38.9922    159.00392     40.45441   -120.71411     -113.185   -295.40829   -123.52127   -104.67526   -225.48421    -46.14701    180.59267    329.46664    126.83375    275.30124    230.66925    320.22298   -139.46361     35.58953     19.50002   -239.48249    -73.32996   -222.34195   -308.58523   -323.65467   -302.08271   -281.06496   -321.07815   -299.95067   -215.75324    -94.30394    -92.92876    -32.95697    129.28191    234.74394    332.08262    253.26158    274.83558    112.13686     74.05683    -57.65768     -97.6281    -80.35717    105.21966    211.63009    178.28524    212.17171     65.92597     88.73406   -205.74318   -186.71888   -169.17423   -250.16355   -350.06013   -265.26402   -329.12292   -292.01026   -246.61947   -334.44846   -202.20673   -185.60735   -106.71312   -113.94123    -33.44403    100.54999     29.84029    175.17384     68.79116    174.14174     41.67428    200.53577    215.57597    325.34217    325.26532    479.76276    426.99663     278.8275    388.79785    132.47955     -5.44931     88.13615    -123.3661   -222.26508   -184.28995   -289.56939   -298.52569   -271.41364    -33.55211   -144.39689    -73.72354      -80.198     -73.8737    182.65186    104.71919    171.25181    194.44739     109.3968    398.43652    523.08625     518.3783    889.28918    830.34443     912.6278    594.05221    415.46551    498.98358    351.22049    200.67227     61.83803    -50.27868   -100.54152   -251.37839   -202.65653    -93.55666   -101.44998   -104.47921     60.42353     61.93531    108.20835     87.50578     54.05994     88.73258      -0.7993    -53.76641   -223.37116   -157.99115   -198.70391   -208.75237   -348.65594   -127.42101   -263.11299   -197.68092   -338.10474   -273.93358   -303.58097   -492.28686    -372.4584   -326.84679   -190.19221   -171.30924    -92.90047   -311.17937    -49.18094   -218.89688    -36.55185   -196.52962      7.00214     47.83024     29.90492    -79.91131    -16.46032    -26.52246    -30.77973    -32.33146    -59.70585     72.58253     113.8675     -57.9197   -176.11132     24.31383   -121.03764    -88.05008   -154.02587   -308.50195     -257.252   -257.57272   -104.67006   -199.94601   -163.93412    119.18009    -17.35696   -126.41638    108.93686   -161.92608   -201.80989    -17.95689    157.65024   -218.87528   -262.97816    -93.96225    -81.85808    -87.22753   -134.60204    -81.54419   -203.08351   -187.83836     95.69922      29.6089    -35.48915    141.35907      6.94219     67.53887    125.52779      373.681    150.86992   -186.52095    128.95111     45.03637    504.52914     64.31175    269.01219    151.68658   -141.78892    -33.21618    -37.78319    -33.83821      0.99737   -259.84432    -33.97278    -24.83566    -40.75417    114.42051   -204.42791   -216.66632   -131.13172     36.21298    -21.69097   -258.19589    132.34581   -134.29709      135.117     86.19063    -111.7473     13.40954   -158.13155    -389.9255   -341.40198    -99.12303   -233.95404   -290.65543   -144.37815   -198.00224   -586.19421   -148.87585   -315.44448   -498.79118   -244.47689   -353.78821   -193.75083    -36.33624    -225.3993   -434.96527    -39.63511   -122.70384   -500.59598   -242.60566   -367.98501    -481.5321   -142.61657    -90.29428    -92.63133   -156.31009     -71.7503   -202.47873   -316.44007    -77.83541    -411.9689   -181.85839   -358.44542   -400.15639   -329.49851   -227.91363   -256.87467   -345.66372   -450.78571   -522.47377   -645.02515   -667.98728   -647.95084   -642.88096   -747.60742   -563.94702   -723.26317   -764.02774   -612.73038   -587.00224   -957.86115   -613.39411   -690.97068    -765.2952   -864.96042  -1008.10787  -1044.69474   -849.96441   -941.13039   -895.18389   -972.88638  -1012.24203  -1165.01883   -801.82039  -1048.12361  -1041.75852   -873.10992   -925.77504   -866.52863   -951.29803   -802.70232   -1021.1865  -1040.58895  -1099.81271   -601.89855  -1102.03872   -908.52732  -1410.96027   -871.09925  -1109.93413   -761.08238   -846.36027  -1069.12954  -1052.21618   -812.88193    -811.5263    -986.4543  -1234.34337   -760.05067   -736.51975   -581.27367  -1184.61785   -955.34342   -570.34264  -1274.21185   -660.31642   -991.34144   -793.95923   -635.83078   -280.08978   -876.29473          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        4.50   -110.35964   -291.47432   -142.00513   -226.14715   -141.47291   -155.88746   -191.31435   -536.46316   -202.38835   -241.14548    -66.07708   -509.85573   -244.70294   -514.09633   -425.94287   -201.96661   -545.18036    -253.8155    -10.60368   -339.70476   -527.40337   -293.20355   -166.90397    -63.10402    177.30722   1118.70278   3317.17417   4534.00605   3758.73859   1457.76985    159.04889   -435.24468   -291.11267   -254.14082   -122.01954   -160.40704   -280.67378    -48.28626   -276.18563   -233.52808   -413.69785   -377.38656   -274.96394   -331.59928   -492.21913   -303.05311   -451.37238   -249.18845   -471.89348   -153.07835   -420.57123   -355.69688   -470.29507   -307.26447   -294.96494   -301.26692   -278.99007   -406.44082   -176.87628    -187.8383   -208.82664   -269.12998    -340.0689   -279.29629   -254.21396   -324.49284     -318.245     -234.967    -51.57884     151.1469    611.51318   1712.88736   2570.41233   2053.19136   1323.68688    293.80305    284.36058     24.48718   -178.63029    -193.8088   -166.80019   -178.47058   -110.28033   -263.76422   -396.82014   -297.85532   -181.44082   -293.58877    -95.57192    512.60982     1704.245   2781.71988    3287.9229   2589.45879   1262.72128    541.18703    -47.78305   -256.60558   -311.04438   -198.50037     37.04584    285.04925    341.26268     49.54498     74.68632    -58.45908   -262.33454   -286.61528   -270.95066   -344.26454   -429.13924   -320.85203   -287.82199   -263.62656   -298.34221   -307.89464   -252.13659   -281.67184   -328.96732    -304.3536   -355.88473   -194.99092   -214.41641     85.94305    453.70106     594.6568    954.20846    905.83579    630.11374    248.15579     54.23837    -48.75187   -271.12786   -233.11647    -83.71284     94.55831    466.07809   1277.47922   2069.95704   2545.95404   2924.32644   2444.67873   1544.49971    911.43443     457.0805     230.0317    141.06573    299.35855    257.75169      20.0521   -131.93082    -97.18578   -155.33943   -324.48653   -323.41376   -170.46677   -170.26835   -244.59216   -292.02892   -311.14322   -288.59518   -170.56301    -54.72397   -189.70281   -148.35284     84.22385     -16.0214     41.93398    116.19444   -109.75955   -256.43887   -139.87485   -144.93323   -103.62032    -91.27176    114.53907    284.45438    432.83636    190.22479    399.27461     82.03657    -59.36235    -53.28308   -189.59357   -124.95287   -292.47554   -122.37739   -306.46797     -392.061   -317.54152   -224.51193   -184.87888   -381.10588   -283.16939    -80.43639    100.84591     18.35624    -109.9782    247.80185    182.19063    281.02153    137.40833    133.61878    133.13302     34.07902    111.81262     -9.18633    123.85683     94.40769    152.27963    235.66888     69.03432     15.04658   -108.77061   -124.36871   -216.42814   -183.62531   -305.27739   -217.69128   -321.02692   -264.17417    -302.5832   -344.18682   -139.59532    -59.98886    -68.57845    -81.84155     88.80743     -39.7065     158.0178     52.17674      41.3449     37.86992     74.20284    217.95117    198.11134    228.59533    365.81805    295.11713    391.30813    332.57139    255.03984   -128.93875    316.18818    -22.25663   -161.59361   -234.64259   -310.97054   -215.11589   -281.66056   -306.83191   -310.50089   -192.24785   -139.30833     -10.6433    -85.29408     72.96677     68.26264    140.30196    202.79696    409.22559    400.26131    399.23216    599.17728    772.14753    908.05816    883.96182    648.81264    533.49028    471.77097    332.24765    294.07762     32.82449   -122.49597    -88.83295   -176.54968     -35.9731    -54.46893   -133.23988     33.39189      39.6584     62.42061    105.63263    128.56999     99.75664     -1.60205    -91.07262     -83.7912   -162.12056    -28.99628   -176.83214    -28.40434   -341.99328   -233.35316   -344.44384   -365.89489   -336.99316   -362.09392   -275.48918   -238.99077   -281.46514   -324.02733   -137.92158   -105.20311    -65.83674   -249.60855   -232.25364   -175.03123    -70.58658    -80.38004   -106.39624     16.75811    -29.92076     98.11137      34.9986     69.31139    202.52751      75.1299      7.70709     25.34697    -82.00443     21.31819   -167.44075   -132.73437   -119.80411   -273.52244    -50.41763   -116.88629   -136.15863   -191.73326    -82.23126   -195.74997   -261.47291      1.60257   -137.50906    -10.56276    -30.68024   -125.86814    -79.94876      53.8513    -18.32541      15.4642   -147.44212    -66.33613    -98.08608   -258.19026    -99.63096   -125.14867    -77.20359   -123.47226    -53.17878     57.48566    -91.50068      -7.4767    229.59758     86.60517    -17.02761    339.13345    215.13208    369.31703    151.86128    133.74943    441.01244    288.52251    226.31668     56.46447      255.753    -36.93016     139.0203    -63.38388     39.56481     44.61651   -250.19474    -29.87192      8.29228     88.74133   -173.40792     30.24121    -86.07023     40.72032    -69.01919    200.49586     60.60586   -197.97305    -253.5174   -144.29926   -206.77479    -80.12902   -438.09991   -243.36542   -227.97034    -348.2735   -223.29743   -213.35945    -78.35874   -243.32286   -252.51028   -332.43083   -132.67766   -496.94224    -21.92534   -375.17802   -349.38443   -268.22311   -122.70725   -193.70841   -375.15547   -103.29705   -377.86337   -428.96392    -72.88425   -139.35343   -131.10444   -448.33142   -306.48869    -55.03452   -378.44577   -110.75589   -195.92879   -205.12694   -318.16244   -297.40464   -365.94239   -359.44649   -635.28149   -322.85759   -568.63675   -461.68519   -458.55226    -348.3679     -473.946   -544.94932   -770.74186   -550.90635   -651.85678   -551.92034   -801.90642   -615.43942    -678.9123   -894.12028   -717.62861   -633.17502   -848.26973  -1028.33826   -961.90951   -802.44087   -892.93558   -692.40828   -833.66203   -928.89502   -961.41096  -1122.09754   -890.59326   -895.31649   -896.47788   -879.44415   -1062.7413  -1175.25559   -1015.8912   -780.60838  -1143.80463   -938.69854   -956.15727   -983.99028   -590.70663   -966.82472   -755.07905   -577.27967    -792.7409  -1061.77462   -938.88867   -698.32536  -1038.07962  -1316.79542  -1315.76426   -1112.5321  -1101.91007   -996.84355  -1036.54372  -1173.14559    -519.0958   -402.43577   -764.93689   -959.32673   -1324.2412    -71.77629    -673.9487  -1455.43198   -745.95586   -839.19471   -954.94578          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        5.00   -298.19594   -418.83773     13.40871   -241.24341   -120.57349   -174.51784   -314.14669   -523.13499   -197.14092   -230.80433     -97.0439   -291.04845   -220.27037   -203.06923   -332.21811   -214.51567   -370.51408   -231.57554   -171.54883    -143.2373   -353.51924   -201.79503    -90.75596   -257.93223     205.1896   1378.59185   3322.59747   4324.85341   3624.72167   1454.27926    107.93253    -78.96112   -335.82354   -181.02003   -292.89191   -246.84975   -187.75894   -515.79996   -278.52917   -458.94501   -289.68054   -355.92339   -382.01566   -237.26357   -237.04875   -268.45728   -382.92414   -326.44755   -106.88673   -301.38465   -484.71013   -211.17004   -331.90282   -310.99595   -397.36663   -321.37187    -246.0442   -200.16421   -197.89118   -296.48173   -217.60614   -174.58246   -368.39321   -257.96241   -197.24083   -370.33856   -380.96588   -106.36343   -183.69963     98.99069    782.38735   1782.90118   2336.69017   2292.77745   1355.22649    492.87096    175.91776      70.8751    -85.05004   -234.70269   -292.65157   -211.60369   -231.55818   -338.02243   -250.58028   -311.69462   -241.57696   -118.72286     61.04742    675.49099   1886.99691   3025.72935   3473.25604   2544.68945   1575.05682    413.06122     24.52711   -134.08961      -88.436   -173.20055    -16.05339     71.37349    432.31228     267.9088    167.76139   -152.83815   -324.93615   -196.63458   -292.64398   -377.75653   -343.14275   -178.97151   -384.52788   -326.50194   -318.43962   -234.46569   -108.81977    -95.51408   -162.40708   -370.68249   -117.70942   -158.62854    -50.56263     31.89039     306.4009    671.28284    886.64749     899.7178     662.3829    365.86526     -28.3497    -79.69851   -153.96081   -218.91337    -62.69595     97.62106    561.36938   1176.66134   2014.83653   2722.59593   3020.35034   2447.46225   1831.10424    927.98259    454.49054    251.20741     93.94421     93.54365    247.13889     14.89677     41.15795   -142.88527   -431.50162   -209.02778    -198.6584   -244.67128   -189.15654   -250.71607   -177.68587    -243.6235   -151.67163   -218.38299   -257.41488   -147.68699      2.56567    -71.95673    -42.77278    110.97583   -172.63559   -102.08173    -332.6463    -181.5822   -248.11108    -56.74958     -6.25296     95.12231    294.04687    175.45865    242.16407    247.21592    -39.74123      5.51651    145.31022     33.61113   -281.23365   -134.51297   -188.20285   -211.20085   -248.52444   -282.95332   -324.59188   -314.20074   -106.75042   -302.66164   -166.00818    -64.30941     90.00863     43.88843    161.64379    113.14502    500.68787    130.64151    189.10024     107.1807    104.91746    -75.05913     28.85322    221.45591    176.39513    307.96106    144.82886    169.08942     -0.22016    -34.06252    -90.36608   -222.98921   -279.21508   -355.71466   -157.82684   -247.08734   -254.17423    -341.8283   -327.69863   -221.85089    -158.6899    -97.81642   -209.41969     -7.83157    182.97851    156.17501     47.87016    -46.50498     78.52223     47.51939    124.16612    300.87231    212.31876    360.14261    380.92978     198.4305     387.3724    182.74802    175.91238     27.69045     -7.23306   -172.72606    -66.93871   -201.31844   -361.53653   -313.20613   -257.83144   -118.52844   -170.14869     16.64383     97.97947   -172.99654    -114.8386      37.8098     53.80847     34.81264    505.88158    504.18791     695.9244    644.11463     846.8165    849.05929    813.49357    710.50179    608.64502     555.2717    295.10017     90.32939     62.67573     23.39368   -130.96283    -30.37518   -101.26957     -86.5711    -87.49065     -12.3922     12.03419      14.2691    172.23677    156.13918   -102.75253     234.1112     23.58427    -51.06483      -63.034   -214.81265   -299.78065   -192.73383   -330.18555   -296.28566   -203.78261     -269.877   -454.73064   -350.39723   -197.87684   -302.38117   -209.80429   -138.85516   -304.48681   -210.92489   -278.83364    -297.5334   -153.27716   -149.58941   -277.62993    -99.60615     -249.474    -61.78494     -9.95632       3.5189    133.35293     78.21819     87.49056     98.15335    -11.33801   -133.51819    -34.21142   -199.91262   -118.54827     -68.2691   -123.79041    -62.06062    -23.81582    -175.4057    -30.06975   -258.02085     60.99227    -19.12657   -130.98931   -122.20649    -58.39825      0.73165    -32.22927     55.15743    -99.29713     -90.6083    -99.67603     73.31386   -221.36945   -155.79837   -109.26758    -118.6877   -199.24463    -76.23146    -35.27854   -206.45749   -130.57599   -142.72094   -214.25784   -170.89768    129.71475     -78.9668    109.35665    224.74509      35.2743    240.78516     80.82195    354.14157    167.22177    167.80832    195.76017    172.17802     54.28522     -2.10419     103.0778     36.70073   -127.64625    -99.67574     37.21428    -64.09788    -18.89646     32.21555      8.28106     63.50558   -397.90572    -91.98914    100.12686    -42.01428    -89.43767    -29.71611     29.95813     15.19866       18.835       6.4558   -119.76865   -189.76566   -316.83914   -404.43457    -43.89569   -179.17334   -324.67114   -378.02182     14.75603   -321.90713   -365.68672   -244.98005   -218.79094   -298.53143   -199.50902   -134.72893   -243.57303       0.7656   -387.68061    -340.4257   -355.65672   -159.02664   -135.63926   -427.15421   -207.28678     -82.4926   -115.04237   -439.80118   -334.99195    -202.7007   -369.49721   -309.61675    -472.2885   -446.11491   -623.94752   -416.53781   -523.78583   -279.08095   -323.20989   -431.67028   -379.03761    -528.3504   -533.67598   -541.15836   -507.64005   -772.58119     -742.001   -417.34267   -498.92316  -1084.80016   -726.22841   -758.09102   -566.42953   -809.07292   -732.99882   -841.47715   -765.43063    -796.4136   -952.70631  -1169.77258  -1139.79637   -990.90677  -1135.45175   -946.55414   -938.98635  -1085.99265  -1041.86279   -785.24151   -789.85125   -765.51819   -1185.6879   -857.21853   -920.55038  -1073.70142  -1247.60718   -931.87062   -968.97251  -1254.54834   -1344.1512   -824.25276    -960.8387   -873.42173  -1182.78624  -1376.41633  -1242.73763   -668.53846   -637.76275   -857.94891  -1079.98017    -537.5393   -680.38156   -902.95529   -1152.3281   -1117.5357   -707.23666  -1053.86326   -1176.8976   -418.78058  -1087.34458   -876.02773   -802.00161  -1094.35924   -411.49892          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        5.50   -493.76279   -421.54067   -175.28937   -322.63211   -120.05204   -207.14206    -391.4627   -483.77648   -318.14995    -389.8355    -187.5503   -149.14357   -245.26966     -264.361   -263.89304    -229.6239    -95.52909   -264.90804   -227.15371    -281.8091   -363.59402   -349.10104   -153.55076   -392.07895     29.11335   1010.21738   3208.31428   4561.12468    3477.9798   1644.14797     274.3587   -105.29841   -482.93048    -181.7414   -268.36022   -260.15105   -330.05966   -195.45344   -408.55113   -341.00132   -303.66062   -140.05758    -429.8377   -327.11305   -175.26927   -247.31367   -503.59349   -291.18626   -209.66431   -366.28166   -387.17207   -169.38097   -344.95364   -395.66557   -196.75582   -185.37645   -336.78512   -326.15839    -66.74287   -230.43436   -159.74406    -319.0566   -361.33424    -162.2388   -140.11585   -428.05696   -281.30956    -258.9425   -228.82565     68.45934    625.13357    1783.6724   2390.92923   2250.99798   1169.86617    505.08393    194.92005    143.07476     21.73144   -149.05594    -85.77339   -117.41345   -341.30188   -354.22204   -206.26159   -255.54825   -307.98081    -120.9141     32.62075    659.95126   1576.63218   2772.98503    3304.4426    2662.1335   1489.85085    679.01497      83.4878   -264.13377   -292.50243    -36.14397     -32.2722    260.75144    404.19486    450.15281     -0.54272   -128.13609   -142.13444   -229.50846   -172.34516   -183.15562   -427.70202   -308.74531   -287.02036   -505.31374   -220.46265    -77.90551   -328.29858   -155.64447   -186.34485   -131.68962   -160.12779   -369.01701   -148.72734     -6.30437    239.11975    706.75472    778.58023    980.85003     500.5212    267.30045     68.67737   -130.35075   -105.61832    -205.6589    -78.72165    100.50943     385.1529   1224.62667    2413.1482   2774.86153   2880.70044   2392.48043   1601.92737     755.0248     302.4934    226.41907     135.5932    184.85462    210.90378    261.92465     37.51217   -115.08997   -106.96281   -233.34763   -277.94904    -349.0035   -241.06784   -219.57497   -243.13749    -215.8532   -310.20982   -189.62794   -164.86193   -214.41017   -112.74992     -5.46352    103.46194    -51.20102    -49.57286   -142.77794   -140.08873   -283.37081    -333.1614   -132.41811   -209.27869     95.42746    231.71907    206.28908    -22.68179     65.77432     59.54113    -94.57025      5.19852   -139.51996   -273.33837   -342.53429   -226.80516   -173.89471   -329.04656   -266.28732   -357.42331   -280.07592   -240.75533   -173.31482    -62.39972     92.98014     79.35671    163.48992     28.42657    231.85039    322.94253    218.50101    201.86662    164.86458     74.90634    109.62374     93.68908    -33.10442     252.8566     142.9224    282.44767     129.6276     83.80214     95.17334   -156.71761   -260.65559    -93.32041   -366.63058   -120.81074   -221.51502   -345.91244   -234.94492   -273.95529    -96.53744   -145.81107    -115.7644    -11.29364     -2.45595     -4.10254     60.19009      214.829    111.44562     33.98523    126.73709    231.04559    155.69002    250.79739    249.47898    271.06479    207.01257    376.50145     28.46098    472.99857    -17.58392    -120.6916    -46.61577   -209.05007   -405.14647   -402.36921   -356.89094   -264.66958   -258.94253     -192.824   -156.17406      33.1303     68.64669     49.29946    162.93524      81.1933    171.37101    362.86584    330.79436    506.66109    881.64936    631.25196    635.06446    526.60872    838.42384    531.00909    444.19541    404.75973     32.76346     229.4689   -110.02753      -99.947    -162.5317     43.35336    -47.19068     -24.5546   -138.32974    138.09782    179.10825     57.02851     129.2866   -170.34052     39.65307     -17.1329    -57.50056   -275.98351    104.85123   -225.74966    -343.5061   -130.08617    -416.6207   -377.14417   -297.48964   -335.03029   -240.37777   -282.85151    -248.9051   -310.17844   -215.48112   -261.27002   -108.97583   -279.93063   -172.41304    -268.1778    -74.90509   -217.60453   -391.05854   -177.98335   -110.91375    -85.56311     126.8483     61.93064    -83.27353     23.60732      48.2492    -85.12809    -34.74085    -74.96887   -121.17425     61.27402   -197.58818   -134.01576   -101.62166   -127.31339    -94.57387   -159.96481    -78.14877    -94.70956   -141.10088    -83.96394    -60.50147     27.61798     10.29561     -6.01215    -96.43493    126.51446    -29.10423    -58.54449    -91.54668    -95.80693     24.01718   -140.61868   -227.18501   -114.24691   -130.37448     -8.80701   -174.39844      -67.845    -21.23766     46.86708     25.52538     99.75437     79.85197    199.52722    117.84337    377.39711    317.45008    241.14518    104.79615    187.54139    156.92299    126.87555     73.18826     27.92288   -127.89762    128.08131     66.82143    -73.21395    121.60601    -87.28336   -156.69082   -117.23467     -49.0443     35.37122    125.91234     45.48592     -0.04394     -62.0243   -161.18927    -45.17419   -211.59565    227.49221   -441.96347    -37.39049   -133.50809   -189.91716    -97.01625    -72.62166    -334.1841     67.17683   -175.12577   -297.66462    -87.99472   -203.63202    -33.30537    -79.11245   -312.15561   -428.73151   -194.66816   -316.54532   -205.22763   -152.81687    -70.63963   -240.96107   -238.01087   -150.25443   -393.67729   -278.08348   -270.37349   -425.48225    -97.59122   -236.31157   -449.93809   -301.28701   -373.51736     16.84986   -344.15539   -221.92397     -7.69835   -425.02397   -228.10623   -363.81308   -209.41527   -335.63759   -396.12576   -604.97618   -675.69163   -457.04938   -677.63156   -614.13503   -574.69673   -645.58494   -764.83248   -788.80737   -631.57328   -755.23321   -713.25312   -722.36183   -722.52955   -907.06155  -1084.03557   -827.38884   -863.21958   -813.24571   -944.53797  -1103.49588    -835.3887   -775.34412   -919.58084   -730.51655   -878.45575  -1134.46904   -766.18758    -935.0351    -713.6444   -883.64269   -957.53872  -1145.92538  -1108.57569   -988.36087    -921.8912  -1009.57403   -997.86262  -1318.07178  -1179.80559  -1118.38664  -1160.95009   -974.48844   -913.39283   -707.25969   -861.91578  -1172.46092  -1098.13305   -928.54268   -924.04584   -959.37933  -1195.46984  -1510.22199   -767.50322  -1322.45699   -895.69139  -1056.05759    -910.2401   -796.41419   -380.75104  -1077.22739   -323.02344  -1014.98176          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        6.00   -458.95534   -299.85417   -389.99925   -406.83595   -106.45377   -225.05082   -364.68034   -421.17676   -436.33422   -423.96735    -236.2346   -341.60027   -328.34113   -396.47945   -291.63159   -268.80177   -171.92195   -351.80401   -230.14515   -357.69578   -372.72075   -184.14311   -240.67181   -414.80173    -32.17959    991.62559   3327.37532   4444.06678   3516.43827   1709.14713    257.82789   -121.95179    -312.3474    -368.0777   -278.40805   -364.69628   -321.94312   -184.89143   -352.28747   -378.40038   -251.21893    -299.2457    -89.31709   -295.77095   -343.49269   -444.95069   -266.72492   -466.39341   -269.71579   -338.18853   -308.80746   -363.93426   -412.95409   -224.75625   -317.03104   -345.45359    -241.6031   -405.11493    -103.4949    -77.66987   -144.09843   -327.95106   -206.54387    -42.05213    -278.2338   -308.82575   -178.27393    -268.6866   -414.39208     66.31534    639.11125   1807.89004   2424.56183   2252.39752   1137.36309    583.87386    118.77142    188.95345    100.24253    -67.88208    -68.20942     38.18663   -202.52452   -164.21659   -305.32933   -140.58978   -125.08167    -61.62842     45.41104    652.79364   1792.90648   2774.84261   3296.85988   2589.72255   1384.38416    416.35846   -157.36429   -310.57446   -136.38033   -310.21934    -15.64573    174.30784    341.72907    334.53575    201.98447   -135.86628    -43.54948   -126.65186   -296.29187   -264.31864   -286.38985   -217.51987   -487.51273   -239.91434   -130.05424   -142.87585   -255.52064   -176.94116   -322.58462   -157.78384   -224.20803   -236.41347   -102.75619    124.53112    391.70742    659.55647    991.69282    911.38701    585.97622    409.76637    156.13531    -47.80116    -93.27229   -196.08045   -154.95596     55.64711     535.6427   1131.75524   2266.63201   2664.57601   2514.97807   2272.78114   1686.37706    1019.0099    405.14276     181.2411    144.85491    173.31175    224.22886     32.29933    -18.67059   -146.22877   -246.46591   -246.15914   -234.77492   -245.92682   -270.83256   -269.98099   -328.93709    -362.4826   -243.31119   -279.14191   -281.67694      -1.2295    -49.37773    -87.33229   -132.45265    174.01927    -77.01162   -165.76633    -199.4465   -351.23615   -307.33092   -196.17474   -189.10045     11.20511     94.57813    223.32544    205.04396      268.115    159.79255    -89.05327    -62.57046   -250.61537    -81.88432   -156.53577   -175.61915   -365.68061   -337.04754   -161.02084   -339.26412   -176.46262   -274.68874   -209.64708   -184.25905    -68.15651     80.20349     57.94383     90.24238    241.70557    391.21079    224.78614    139.80769     73.31209     50.72667    -120.4638    247.08405       18.939    224.83573    168.74463     83.93266    201.68949      71.1248    -76.12628     -204.002   -222.52475   -308.60347   -462.50611    -250.4942   -463.69203   -425.28144   -220.84329   -162.53437   -285.63104   -218.52272   -183.17572    119.02263    -156.9656     43.93835     89.98247     85.00645     84.92299      0.92618    -95.28815    -67.98225     38.07176     284.7149    423.85553     341.7178    405.16478    207.47517      200.098     48.32232    107.71375     46.55256   -124.66731   -231.32159   -253.89524   -412.01072   -278.10054   -139.71288    -221.1814   -194.88268    -87.01451    -43.35186     21.89852    -140.8024   -105.85075     45.60543    373.07168    114.89054    634.69926     692.1168    623.85447    726.08715    836.55783    830.96906    691.17005    230.55102    658.96671    557.63586    185.30397    232.23226     62.22537    -10.13441   -136.21329   -175.57025    -81.19683      4.12332   -121.74307     49.23344    -42.83028     33.69105     21.34214     30.92622      50.9693     72.48285   -175.22043      -189.67   -152.01337   -171.13053   -144.57077   -204.39104   -183.49035   -743.98004    -240.3131   -440.50895   -453.09228   -239.62214   -133.95613    -427.5507   -246.81278   -262.80029   -359.14765   -152.89177   -217.16048   -238.24707   -159.56032    -72.79609    -76.86935   -113.79438    -53.63365   -112.19683   -143.26688    -17.91865     54.50712    135.18319    -174.6005     73.13342     81.89419     83.69107    -18.78603   -182.31741   -174.05848    -66.71128   -101.92718   -125.39208    -78.28273    -47.06972     -8.66382    -12.01595     46.62581    -32.02523     97.14764     35.40152     -7.61658   -121.14546       2.5009    -14.77028     27.97582    -16.93884   -281.44533   -211.02164    -42.56801      -3.5335   -153.06983   -294.45662    -31.30049   -139.49954   -177.07067    -21.78455     35.20744    -120.6355    -16.58168    126.51298     63.72411    287.33585     98.12963    298.08882    326.61229    304.91581    117.51619    261.44128     53.69076     63.82436     42.58646     166.5005     92.47305    124.70757     16.90293   -106.28357     -1.96503    116.71872       15.356    -179.0635    107.42374     86.84038    -13.91155    -55.71189    -42.58324     -5.48279     -21.8042    -86.58349     41.69236     64.86172   -172.49822    112.74692    -79.65087    -152.7906     13.33722   -378.01098   -394.74312   -259.72454    -185.3199      -263.46   -220.83528   -494.79918      77.1375   -160.58522   -240.59684   -471.55923   -373.04259   -240.83213   -375.37241   -163.40576   -266.63504   -403.18678    -77.33031   -335.98635   -368.01853    -415.1825   -233.25109   -220.39545   -120.49076   -481.06003   -173.31952    -15.70534   -267.28833   -322.19858   -383.30682   -317.39073   -393.08144   -304.80583   -463.84906   -454.43795   -493.24291   -318.34018   -384.51002   -426.70269   -495.18312    -642.5371    -632.4194   -551.16476   -518.21271   -693.86677   -654.33175    -544.1368   -659.06695   -785.18942   -443.25937   -684.28814   -899.12565   -861.75391   -717.30367    -683.6122   -569.80288  -1041.02808  -1116.77145  -1150.82047  -1131.11778   -835.76716   -927.74165     -948.924  -1046.22773  -1124.33773  -1124.32308    -956.4517   -1033.2266   -923.02561  -1127.99778   -793.02336     -672.013   -843.76802  -1322.73239  -1190.89908  -1365.59127   -836.55498   -975.98118  -1070.06818  -1086.58803   -756.82881   -771.62194  -1075.43075    -754.5185  -1005.45342  -1332.57071   -872.96296   -978.08882  -1170.33482   -568.42858  -1149.08552   -811.91907  -1234.78525   -718.73719  -1153.44374   -120.21896   -903.55337   -821.09031   -754.14482   -583.06902   -1120.2142          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        6.50   -424.04167   -180.46693   -441.16379   -340.64511   -118.83865   -228.19171   -319.19217   -361.15056   -450.98121   -271.69026   -282.21991   -367.55202   -258.20747   -413.88296   -438.47031    -329.9103   -235.89351   -303.19842   -247.76839   -372.55525   -349.54447   -135.30595   -104.44491   -308.29516     59.45953   1028.88472   3368.86902   4593.61596   3564.53926   1527.24748    197.96424   -136.65907   -484.84784   -204.75295   -365.59461   -306.32159    -265.7994   -212.71728   -268.19802   -423.44802   -353.47654    -537.3523   -351.28879   -291.99412   -395.51504   -182.14526   -422.65145   -336.18284   -247.53503   -385.33028   -234.60686   -295.15392   -367.44659   -425.29295   -355.98063   -275.43537   -176.70157   -271.58976   -135.88238   -201.40125   -244.29579   -218.42897   -290.16261     -2.04352    -340.9683   -342.48938   -413.93447   -238.95097   -419.12798     244.6573    760.84018   1736.90884   2452.21245   1974.04747   1177.53473    525.79919    312.23553     52.15192     80.73064   -169.35321   -200.27617   -234.53363   -174.16043    -294.5851   -166.40056   -219.96861   -378.08133    -50.36295    175.47118    801.06458   1748.00703    2833.0993   2726.73822   2404.86705   1352.76714    496.68297    -19.39011   -344.06469   -157.53773   -114.72411    -23.03271    581.32734    569.56273    129.76878    101.75291   -137.90668   -216.65291   -350.27247   -261.16662   -332.99909     -240.973   -287.85078   -372.39625   -341.35386   -362.36063   -291.25481    -240.2644   -103.10438   -134.04402   -151.32024   -249.07946   -254.96195      -72.889     38.42787    398.88567      584.719   1225.39616    946.39763    516.59751    340.77818     95.40004     22.06385   -131.84547   -203.84479    -79.29187    -13.27476    419.96908   1193.21726   2124.79058   2636.69016   3097.04043   2369.35848   1748.79501    836.16879    548.26077    449.54718    146.04266    211.02783     73.04389      67.5293    -54.40436    -66.11224   -331.77244   -341.98753   -208.10741   -271.10084   -243.21222    -230.9161   -219.64942   -325.29535   -142.99051   -210.56744   -154.35724    -112.4503    -93.24482     61.47036     75.74683     69.25625    100.20097    -91.21926   -251.76017   -312.79341   -241.64458   -258.38622     75.52961    236.97382    222.49803    274.43622    118.83202    263.19645      6.43608    -54.43999    -52.18865   -206.17172   -168.00628   -228.49656   -215.18321   -227.20259   -246.00828   -208.87868    -227.9886   -107.22571   -216.88335   -170.64624    -42.73677   -111.70224      6.08579     60.36134    206.05799    247.59613      263.218    152.81082    400.92835    147.49898    -56.45402   -100.31792     30.10363     33.67953    131.46884    233.50379    191.04277     94.91385      3.92108   -155.60022   -182.84851    -93.70284      -305.25   -237.78753   -277.74092   -354.00774   -264.39842   -130.29577   -398.62689   -154.79261   -262.20883   -131.87135   -130.24862     45.65562     12.91778      96.8543    212.03846     92.50527     70.05346     97.12666    267.17985    232.93639    390.46516    402.21113    180.90516    454.39522    308.66493    237.82919     82.09154    193.14399    -61.86609   -144.25603   -186.07519   -117.15898   -213.20962   -243.56271   -281.47379   -329.64775   -124.39216   -117.79821    -87.46929    -13.82208    134.72953    132.21913    -32.35883     169.2778    288.23562    345.70923    519.37204     586.2068    805.07991    749.18021    787.68788    678.76414    519.21707    475.77089    267.53121    313.47859    237.36856    -42.44477    -91.23586     -19.9376   -213.33638    -35.35354    -47.28983     39.26483     40.31001     -4.60967    152.50975     46.37783    186.27153     84.21543    -54.91482    -16.22501    -56.03221   -283.19653   -200.71614   -160.07126    -185.4607   -225.04694   -180.86094   -298.38579   -332.48105   -299.37049   -236.44437   -287.11776   -311.75771   -161.54701   -339.29951   -483.35418   -230.81832    -83.49276   -239.62448   -171.88722   -169.83187   -190.92759    -10.43501   -177.43587       85.607     80.32517      4.88592     19.91139     27.86726    -35.09972     31.53134    175.09721    -57.88606   -107.68858   -155.80473    -61.08542   -201.08214    -91.90203    -64.24166    -31.23219    -83.96593   -223.34224    -32.06438   -201.26795     -5.51427    -18.27822     -179.564     64.17417   -245.32659   -108.69666    -65.43258     52.64841    -45.89777     -49.9194    -32.15682    -74.87375   -181.05882   -158.81712   -200.54466     37.85407   -140.74342   -192.06199   -148.99881     17.59959    -59.52594    117.49502    -14.11571    151.51919    161.26487     149.2456    265.67702    114.72201    368.28487    183.64967    247.13109    209.14013     78.22065    161.04556     75.99273      0.15714     264.3897    -51.42797    -82.66238     93.59158     90.65436     27.18712     22.03726    -52.26856    -49.93918    -11.95022     -12.5117   -222.50549    -49.71537     -12.8266    -90.78134     62.75854   -128.75178   -266.17809   -163.73319   -102.78507   -315.14632   -214.88929   -113.38274   -325.51736   -422.31389   -290.80509   -682.73068    -46.45736    -17.35987   -162.66168   -243.04601    -23.17759     64.83731    -67.35973   -162.22823   -357.63502   -184.13774   -503.95341     23.59669   -326.92026   -184.62632   -144.39044   -320.53181   -155.50758    -392.9852    -32.51212   -299.62992   -397.14544    -36.64016   -206.43137   -526.32377   -466.24001    -83.16058   -489.31563   -252.00936   -352.03489   -250.04071   -576.80226   -349.69122   -548.31797   -584.15719   -516.08453   -395.13107   -531.08427   -604.91818   -752.30938   -595.86938   -718.14611   -635.63983   -696.48987   -596.53809   -965.24502   -836.23455  -1071.63542    -900.4279   -617.46622   -897.38879   -938.30469  -1029.27006   -803.18495   -980.96734   -771.99038  -1131.67814   -954.06138   -961.12042   -967.32451  -1065.11232   -913.74569   -934.22694   -1021.5381    -770.3313   -702.36041  -1166.19485  -1043.28696   -759.38584   -997.06757  -1481.17158   -808.21759   -555.77132   -885.52903   -906.62398   -944.14631   -1313.6833   -740.78759   -696.45699    -915.0623   -824.10457   -443.74502   -724.81171   -775.90297  -1368.44628   -962.72879   -847.93425  -1446.21742  -1226.38735   -808.99225   -761.19335  -1241.51495   -401.65551   -620.48988   -694.20261   -252.26707          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        7.00    -402.7282   -143.93025   -194.67306   -266.80024   -148.31299   -207.98436   -195.39964   -320.37917   -442.73628    -209.0288   -266.75851   -185.01357   -139.03608   -383.09622   -390.21948   -431.13331   -304.51075   -249.65127   -313.77219   -358.24063   -198.49903   -118.88431   -217.32253   -243.54176    103.52156    989.85833   3277.15345    4468.8008    3331.5473   1241.52908     145.1761   -136.36418   -135.62256     -111.688   -331.41144   -301.45447    -63.25167   -236.61903    -373.6211   -352.03122   -242.71799   -119.77495   -340.90168    -284.2886   -316.66103    -348.5677   -472.70324   -257.68593   -362.88464   -186.52837   -206.23771   -415.62864   -351.87424   -556.32504   -224.83743    -390.6889   -149.47869   -296.36495   -307.71271   -206.14503    -36.97376   -286.37877   -523.22452    -63.98244   -120.94039   -334.79624   -484.30871   -214.43137     -95.9729     84.50898    651.44543   1985.31759   2504.01174    2329.3318   1118.41844    537.92775    295.34723    205.97696   -119.50384   -246.55765    -73.78877   -311.51875    -64.88534    -251.5465   -432.33205   -180.02531   -274.20543   -200.10296    -26.47958    733.51844   1897.70241    2829.3156    3203.0629   2714.98543   1270.31075    465.70909     124.8649    -170.0547   -181.04319    -89.96992     -57.7951    280.72162    260.58833    214.10636     11.61307     -48.7909    -53.50885   -291.85531   -327.39692   -277.08525   -335.61732   -253.93389   -229.20059   -273.17848   -329.09807   -352.21055   -407.26939   -166.20845   -175.10341   -243.38851   -268.82537   -168.16862    -69.45552    -31.26577    437.43146    818.24954    945.93654    719.74604     675.9242    163.52868     -0.50805    -39.25402     -90.6351   -234.40707    -31.19377     86.50439    481.82302   1137.48034   2221.01132    2789.3311   2793.04877   2192.10417   1533.32874    984.74333    494.38731    223.28881    155.32299     42.94481    276.58054      5.10796   -117.86908   -206.98695    -271.6624   -353.93011   -230.93273   -307.90359   -391.17888   -294.95121   -330.19695   -275.90927   -109.45925   -137.18392   -297.78092    -56.09554    -52.15862     -27.1047    -20.06069      22.9493    121.62489     -234.996    -189.8835   -157.69793   -245.60229   -165.72393   -178.86132     54.10281    282.24825    286.27852    204.60355     67.92163      1.58886    -72.22768    -23.16874     67.89914    -21.01847   -250.24927   -159.18412   -373.95129   -227.54313   -251.79232   -231.17646   -284.01322   -351.30313    -60.51814   -126.89208     20.40141     96.39228     86.83138    142.57084    322.80832    334.09241    255.92798      56.9373      -3.9914     96.85099     10.69472    -21.26985     24.21306    140.79002    221.18596    320.92471    259.98663     65.71811    -70.15022    -51.28325   -218.76467   -447.92101   -231.38742   -365.80116   -411.59625   -229.31986   -350.78082   -278.08292   -176.13819   -236.80634   -128.41806    -68.18094    -81.39131    136.46291    227.93984    270.38321    113.09573     74.94974      14.7986     129.0258    263.52152     239.1271    278.88542    271.39996     75.52091     220.1174    200.38373    196.94627     41.12536   -211.23682   -121.36051   -238.67074   -290.99316   -349.07215   -256.40315   -243.67059    -85.69111    -137.9977   -187.53917     92.37228   -118.57663    188.61132    222.23736     52.69146    214.62153    292.20229    555.50347    473.58226    733.97966    795.49449    662.87603    677.15511    695.04987    660.10204    412.41155    490.58136    244.09823      6.46206   -165.01525   -104.13721     -42.6073    -109.2203     -3.68704    116.05736      5.67403     13.43437    200.68905     82.03129     20.93114     25.69291    -36.47321    103.10806   -175.90688   -234.77899    -25.32146   -183.53148   -209.96568   -322.75701   -239.19262   -162.26874   -294.66152   -243.45992   -346.77684   -247.47589    -209.6383   -305.78046   -305.32761   -295.95383   -311.32043   -401.05034   -125.23678   -224.64157    -38.46779   -114.93657   -137.45383     -11.2968     -2.43494    -37.35908    -55.17238    -39.76408    139.41494    116.05592     -9.69109   -112.81866   -159.20167    -81.20484    -23.93451    -15.43201    -69.50763   -130.89065   -173.83974   -157.72267     41.67544   -211.18884    -82.26085   -174.39291    -66.42669   -277.73114   -173.81854   -175.39614    109.43764    -72.34991    174.72177      43.2355      34.7259   -222.55066    -71.99802    -35.65241     -180.379   -119.87545    -78.21689    -21.03301    -15.40556   -189.78623   -273.65921   -464.84533    -17.76813   -119.73824   -281.11885    169.89219    -80.12953     78.67721    283.68073    120.60576    199.33367    267.87257    312.87806    263.79467    321.35087      2.80152     70.97035    -23.43576     -0.99495     17.83362    233.74123    221.77807      12.7786     17.98463     27.30425     -6.41802   -104.65949     71.29827     86.46203     -69.6607    117.45749     -55.7815     -9.25315    -21.45142      16.5722     -8.72566   -227.62736   -110.55044   -236.87616   -477.29352     62.47621   -174.54768   -282.36875   -376.97115   -259.08959   -122.86662     -8.50714   -298.16056    -73.34057   -259.64308   -284.76076   -306.04975    -297.0032   -383.45356   -358.91169   -240.61393   -244.86461    -488.7611   -124.03183   -326.41035   -253.55478   -226.21676    -44.95402    105.53548   -251.70624   -119.38041    -215.6936   -262.55584   -215.43872    -67.34214   -334.59716   -255.85573   -296.95272     -38.4196   -328.12518   -268.11577   -521.76386   -593.11552   -389.39004   -498.83253     -497.865   -767.55582   -692.86937   -656.43158   -623.60244   -697.77257   -575.39362   -762.73522   -633.68616   -645.03577   -681.27632  -1145.92892  -1009.39234    -950.5289   -942.88233   -711.83284   -967.06179   -854.87794   -798.79396   -657.34348   -845.94686   -768.24674   -876.56563   -837.98808  -1066.85767  -1084.81608   -928.22649  -1134.73303  -1129.61753   -747.19211   -719.26929  -1121.66708   -852.54563  -1026.62888   -608.51872   -1068.3423   -943.86937  -1458.15648   -713.10638   -961.67981  -1004.64143  -1332.81814  -1055.31337   -288.18363    -830.1539  -1023.09474  -1148.59615   -799.44519  -1138.36661  -1210.57649  -1705.30803  -1418.31447   -839.00296  -1169.68455  -1100.57408  -1018.74503   -743.41685   -524.91044   -248.22407   -400.35391   -986.64657          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        7.50   -500.25141   -119.36656     24.07533   -210.32386   -390.47298   -179.37069   -100.62502   -303.19043    -471.6029   -249.06122   -265.80716    -92.44788   -181.77387   -355.21662   -305.93079   -330.22101   -365.23583   -165.64674   -373.49994   -291.66081   -290.47465   -156.79942   -376.83538   -156.38518     -34.3832      1076.79    3172.5913   4482.46326   3547.52263   1500.90318    150.63952    -57.02934   -225.02412   -384.23649   -207.79712   -369.64941   -223.21777   -171.18728   -368.94725   -403.06701   -320.42859   -354.72243   -347.08408   -359.01395   -209.04955   -391.25128   -413.82579    -313.5022   -350.80318   -373.83254   -402.34235   -396.92319    -452.0262   -308.55336   -354.48461   -388.94091    -255.0536   -470.23427    -79.76468   -267.82423   -284.86554   -285.69365   -190.48385   -169.06713   -115.47834   -311.07729   -347.58987   -283.46558    -174.2556    139.64515    879.54705   1725.50665   2363.18011   2313.27465   1312.99507    650.57584    146.87666    148.85385   -146.29405    -66.96463    -468.5245   -111.08276   -276.61444   -219.57982   -220.20652   -343.83383    -241.7328   -217.65834    249.25365    854.11051   1709.30617   2906.08937   3214.16352   2537.79889   1481.44434     424.1133    -60.21126   -183.44816   -275.71129   -183.93842    202.93996    209.77703    269.75803    170.68236     98.16809    -90.66355   -402.48677   -251.19351   -273.11799   -263.03323   -254.70101   -333.15557   -397.39222   -323.37724   -133.11695   -148.09306   -196.60871   -125.89511   -167.69102   -181.13891   -335.72581   -201.80575     80.08389     26.75856     398.8479    845.87564    921.94086    903.81611     686.6844    182.08826     59.80068     41.04675   -248.86225   -170.50404   -206.26248     21.92186    467.51161   1202.81408   2299.63291   2808.88465   2603.93551   2487.37906   1578.47223   1049.68055    483.23764    362.60806     48.61936     59.70721    128.42333    -68.66903     77.46892   -201.70086    -256.9704   -186.27717    -263.8165   -267.10274   -281.86577   -226.90737   -241.35674   -334.81084    -172.3469   -321.06217   -313.73864   -267.83416    -59.46446     24.91829     14.56276   -197.90515      -69.739   -125.16166     -50.4506   -312.99876    -242.9006   -312.13194    -59.13011    -53.46727    279.48301    155.44107    139.72573     115.7205    125.64785    260.54973    -67.62227      3.13789   -205.76305   -175.95055   -333.48998   -299.07385   -366.83456   -263.57516   -393.64887   -331.43387   -198.67438   -345.05078    -97.62924    -93.47569     55.69214     35.77912    120.73478    177.92529    430.62791    288.53473    295.57485     100.0601     -7.10591     62.65785    -32.40995    121.48422    271.04338    128.75159    194.51603    112.80522     31.06433    -94.25136   -232.20641   -259.51556   -286.00243   -307.04362   -271.12242   -295.62117   -251.92389   -249.06808   -267.71523   -324.66207    -48.96658     -1.63886    -56.65709     -1.15895     90.30193    -31.25572    153.51268      6.23396    173.33424      1.97772    135.73359     -18.5505    345.53113    213.52043    433.16405     234.8408    150.64827    154.52048    160.58855     76.54713   -104.38665   -217.98579   -191.95365   -102.66688   -160.49629   -110.54358   -350.92233   -265.85269   -152.90767     22.75424    206.71043    161.40166    124.31814    224.33423    -61.51934    320.34018    285.95889    187.84576    660.91333    644.64629    653.60136    819.27803    733.16676     632.5879    538.36828    559.15447    243.35752    268.32675     18.61516     42.77213     10.68871    -94.68601   -144.85736   -193.98385   -162.91084    -47.01638   -137.01534    -80.72534      53.1979    224.63732    -29.53939     66.32443    -93.35492     -44.2134   -159.65684      0.85059    -86.53328   -280.54034   -484.63008   -126.20989   -291.83348    -346.1234   -384.35115   -448.61886   -196.91276   -309.53199   -215.12689   -391.64872   -409.58571   -212.47071   -413.28799   -124.75515   -165.17422   -119.43744   -193.17797   -278.37768    -30.26486   -115.37934     -2.94523    -78.45667    -17.22299    -71.46592    -133.6955   -120.49223    -77.85186    109.72709      -2.9576   -129.90414   -221.63759   -229.78226   -157.79504   -119.96184    -64.17171   -294.63293    -52.95135   -159.50255    -47.23714    -13.24362    -138.7256   -101.50524    -48.79838     -71.5103     -0.36617   -103.13686    -14.81195     68.56816     82.10195    -53.46473   -199.71165   -229.62568   -156.50217   -200.52707   -267.37285   -146.87788   -214.43289    -85.87834    -62.96191   -203.15667    -123.0196     59.68784    -40.81306    199.65335    237.13474      155.721    235.88105    248.62563    279.78429    275.42893     96.17095     81.90065      2.62637     91.28571    179.76796     92.98526    -23.55805    -35.09029    -84.89546    -54.23345   -248.88905      1.56801    -93.36981     37.90826   -108.69897   -168.04665        42.84   -141.97423    -109.5418    -83.64588     13.28505    -66.79526    -82.93435   -195.85441     36.13471   -182.91135   -314.54961   -210.18226   -223.62574   -115.00569   -377.56987   -354.58203   -204.16135   -163.49725   -226.34931   -378.63784   -206.08166   -264.49709   -196.62278   -227.92922   -266.06817   -144.59472     -65.4875    -32.46508   -319.57075   -331.89455   -333.46284   -187.23883   -399.47136   -353.05175     -267.776    -87.37479   -129.02652   -387.69941    -348.8085   -334.21266   -575.50393   -118.91226    -348.7362   -254.88102    -365.7289   -353.86229   -674.70953   -546.78204   -435.92765   -673.27802   -598.60873   -515.17794   -626.35156   -641.87529   -752.21815   -431.91374   -951.37505   -781.95998   -603.49757   -671.31177   -805.01945   -833.79956   -863.26538   -754.79132   -788.96513   -938.82692   -590.74191  -1339.91069   -944.37722   -980.12466   -955.57595   -982.61326   -801.90103  -1033.65582   -955.17689  -1063.41323  -1012.70598   -907.31023   -609.67794   -981.52559   -806.28615   -590.05815    -807.7559   -468.46556  -1351.93732   -917.05942   -936.14269  -1358.99165   -898.79412   -764.88632   -754.37403   -851.99818   -703.85151  -1217.46868   -780.00812   -834.44387   -763.68174  -1315.91393   -809.03074   -1170.5556   -941.44002   -742.88142  -1037.73768  -1130.09635    -912.4337   -998.82543  -1059.14731   -674.57201   -526.95261   -773.36491   -922.35421    -795.3163          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        8.00   -587.93382    -90.77862   -146.68641   -187.52501   -586.82829   -183.02223   -327.13853   -273.07475   -514.25078    -275.6044   -307.53015    -77.11564   -242.02881   -227.21362   -561.45911    -53.69854   -357.80081   -205.78495    -299.7628   -195.43747   -379.23592   -165.83498   -101.70284   -171.50583    -13.87445   1235.25077   3188.47686   4641.53173   3450.60638   1242.90632    380.23157      3.17009   -458.36167   -227.63667   -260.72406   -419.48811    -435.2235   -367.12932   -342.88273   -371.42621   -263.86977   -398.25493   -297.70381   -540.18135   -359.02775    -316.1626    -473.5026   -289.61796   -380.89394   -472.00348   -375.87715   -453.30212   -474.15115   -381.18301   -326.18505   -384.29133   -139.27987   -145.57485   -181.11097   -293.96614   -284.67136   -405.25168   -345.87992   -249.23227   -366.87566   -235.40835   -422.00916   -173.21025   -183.81692    240.83143    843.45274   1869.75523   2248.51834   2288.01345   1264.99821    440.29432    368.98099     78.83371    162.94217    -19.97277   -279.36675   -146.10219    -281.3015    -92.10755   -244.27202   -385.09273   -298.10959   -159.60359     142.8619    730.33519   1760.22795   2803.00776   3243.19869   2505.45698   1746.80772    421.64211      2.02121    -95.39734   -278.86631   -284.64101    -22.57079     273.7938     304.4225     186.8871    178.20033    -98.57739   -221.42651   -243.17238   -394.52409   -251.39076   -339.76126   -450.50957   -374.73373   -326.12013   -454.62249    -279.8647   -272.07187   -276.51325   -124.47009   -140.03473    -260.4204   -342.73634   -140.18585    125.59436    409.71501    851.58188    850.96548    753.08463    728.07146    226.69692    264.98197   -110.27011   -234.00816   -101.89161   -123.72885    -62.29588    490.48778   1088.68065    2187.8212   2747.83186   2701.37484   2615.52724    1891.7977    938.88637    474.88185    408.44102    212.22737    255.12754     84.61614     22.42669   -158.44234   -287.17544   -146.80064   -250.70348   -219.25507   -357.17355   -149.30339    -229.9465   -248.95371   -179.29235    -258.6309   -298.94003   -307.33808   -126.68045   -121.05196    -46.29581     24.89074     50.03212     35.40729   -133.97613   -231.48881   -244.60581   -315.44547   -105.25792       61.638     -74.4544     111.7156     201.3848    194.19543     77.89137    207.40657    101.77648    -93.18195   -107.57245   -229.24361    -187.5123   -123.76151   -294.33709   -235.02563   -211.46931   -360.71312   -249.91257   -327.14627   -300.62663   -160.70568    -42.06572    -38.94131      -93.696    110.26025    161.79501       7.1408    321.34362    222.07105     70.42046     26.35282    -17.08092    -30.86739     -2.49144     30.67564    170.64568    149.75928    133.56052    -14.12545    -69.59298   -109.22489    -242.4574   -340.54113   -333.99283   -294.23352   -243.67219   -350.94564   -172.83196   -217.75146   -226.78276   -178.28446    -65.88522     13.17093     31.78899    -34.55786     74.92268     61.64956     55.48059     34.81572    103.29223    218.56967    129.38221    106.62942    311.67533    382.62153    276.80159    295.03625     256.1645    150.44239    -18.37145   -104.12753    -12.01711   -139.21898   -144.44661   -175.76844   -139.93392   -214.45655    -126.7045   -189.17004    -96.98208    -60.26397   -154.24801     22.55674      76.7612     27.29799     64.88654    364.71879    467.26044    550.57626    659.49117     855.4938    850.42668    709.79052     701.3748    754.38396    486.79268     514.4824    177.22155    176.29038     99.40821    -87.76986   -217.24971    -41.73993   -164.67465   -104.47349     79.36515      4.63005     56.11043    -21.56688     236.1892     47.30162    114.73566    -28.52614      7.29805    -66.07429   -345.43209   -259.88336   -265.75381   -164.87673   -438.75592    -75.71039    -406.0112   -380.70754   -138.26524   -472.68439   -331.37734   -164.79791   -197.99561   -223.31767   -132.76543    -79.61678    -209.5233   -231.91805   -208.97538    -96.33518   -148.08416    -26.30069    -42.27723     67.10028     31.20905     -30.1679    -25.87311      3.14663    -16.90959     54.57132     -69.3207    -73.40548   -243.66058    -84.88978    -71.22166   -161.84813   -169.41187   -202.27899   -229.19064   -115.91042   -123.58666   -206.79777   -168.33728    -76.42224     34.71561    -61.15061    -79.80691     46.42271   -112.92772       4.0278    -58.79466     31.14458     -12.3845   -252.18664    211.46362   -170.07532   -148.63264    -96.04579    -134.0337   -161.45561    -37.57268    -14.22067    -75.97157     -18.6954    -86.60865     57.14216    281.83339    233.01464     97.17987    263.55895    250.94172    626.36646     81.97621    231.68666    172.78782    187.55797     53.38133    167.48504    132.75521     10.52207   -197.29075     -88.5018   -108.74206    213.45033    218.56345    -61.27764     56.69402    -51.92371     -81.2449   -165.34054   -144.88149    119.06915    -71.79607    -124.0028    -90.25601   -111.51828   -101.33088   -228.65865   -231.29294    -74.93107    -86.14293   -178.13274    -17.76277    -157.1999   -294.12983   -233.23634    -30.35074   -398.00104   -167.39766   -259.26735   -154.74689   -309.90928   -480.00924   -103.22984    -124.5691   -199.93735   -320.55953   -252.79494   -161.25717   -185.35274   -256.46338   -277.87571   -230.59979   -368.98362   -500.21229   -304.50701   -397.24964   -345.33552   -264.13014   -393.36509   -447.31329   -103.49743   -337.74849   -316.30055   -209.39651   -713.73646   -318.39227   -403.37751   -353.20782   -429.73104   -375.24558   -294.18139   -525.18483   -566.49745   -594.05281   -768.33121    -465.5807   -779.17378   -674.49415   -812.90635   -889.54824   -942.07388   -784.61468   -850.78537   -848.59398   -887.20747  -1076.28552   -837.41741   -683.11684  -1112.69138   -813.45703   -791.19274   -772.30798  -1012.70925    -792.4472   -830.89445  -1082.74098  -1078.63792   -936.86061   -853.65777   -991.87795   -934.66942   -941.50753   -1078.3569   -974.50893  -1068.32061   -891.47191   -843.14338   -787.38032  -1120.67411   -502.65596   -987.63616  -1218.12697   -854.21212  -1320.10684   -770.61832   -793.59552    -938.5761  -1035.92202  -1347.20995   -960.71978   -963.85239   -985.46135   -614.22162  -1233.67923  -1205.40466   -398.50472    -791.3183   -787.36054   -449.51977   -945.61778          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        8.50       -597.1   -295.87163   -413.53669   -190.30031   -538.81569   -286.05207   -562.90885   -225.47984   -410.00697   -335.63393   -303.73155    -56.18739   -189.98199    -40.55108   -609.19592    -37.31691   -225.53223   -269.00853   -276.66448   -242.74635   -379.37026   -187.33772   -162.08955   -324.73568    168.60604   1259.31999   3177.89963   4532.48446   3276.50355   1315.99719    152.52537   -218.94264   -441.07924   -333.72129   -503.09634   -431.33948   -416.04107   -347.81232   -314.89747   -339.78983   -591.53809    -424.1639   -291.52766   -427.68459   -504.62619   -394.40899   -438.09652   -342.29591   -336.09799   -471.67653   -503.20552   -595.58995   -337.12129   -506.46561   -335.38309   -274.55122   -334.94569   -288.30414   -143.68095   -106.83365   -262.27446   -450.49364   -377.65335   -341.67723    -367.5336   -314.78767    -337.3202    -236.1397   -245.68627      68.0621     696.4566   1831.86687   2540.08322   2128.15445   1161.60488    489.23889    268.64591    -32.74781     18.32908    -336.2383   -228.17147   -131.55512    -168.9661   -353.39458   -296.57273   -379.22416   -126.96807     50.01931     23.77704    897.63643   2003.59499   2922.37409   3523.96737   2699.19057   1492.03019    427.58055    -88.53215    -74.27712   -160.01765   -240.60964    -12.63701    349.34048    295.89647    333.15593    -54.64115   -164.16977    -304.5569    -337.6084   -310.91124    -201.4944   -302.15788   -372.58904   -199.43079   -372.77879   -226.40036   -268.07549   -193.74791   -106.36033    -94.44136   -282.93702    -224.4509   -315.26638    -98.84536     15.72728    426.61565    752.91366    981.37978    897.24152    548.90947    338.51537       9.4959     31.12054   -166.27465     -192.619   -105.70699   -193.24808    466.13945   1193.38949   1974.78335   2784.20725   2878.72009   2256.93139   1723.03175    859.84907    556.75121    344.65383    173.36769    139.59554    228.55742     -5.97604    112.40064   -240.51031   -215.58688   -259.32527   -308.97121   -212.29998   -147.71788   -218.90528   -301.40342   -306.43481    -264.8628   -114.29027   -196.60007    -85.67766    -65.35897    -98.40538     18.00563      1.65506   -109.00933   -200.32211    -91.45113   -163.24013   -138.49632   -163.95989   -113.22635    203.28571     187.3849    106.92671    211.03956    199.31051    -68.32718     68.42056     20.00632    -67.00757    -43.91988   -215.13146   -215.28431   -247.74442   -471.07808   -253.45034   -194.85372   -394.60789   -155.69815   -191.22748   -132.99524      -81.899     74.14507     234.6769     -30.4503    293.41893    235.38231     98.51817    143.37569    184.97244     66.89585    -60.81842     -8.07412     -0.17461    129.19662    213.01145    102.72605     81.80427    -36.44424   -296.54901    -47.04146   -236.45789   -274.04612   -281.06374   -339.71686   -383.51428   -351.68168     -340.011   -146.74535   -406.26596   -196.87491   -195.02078   -209.19596    147.54764     55.11633   -153.81332    136.95426    142.14128    173.92985    125.33054    201.71718    152.07741    220.75285    428.46147    421.06519    444.81219    222.77313    283.24175    -38.72715     52.39625   -149.08692    -31.08351    -212.5887   -209.72211   -232.91805   -360.46987   -237.90054   -160.27638   -123.75793    -89.29272      32.8475    105.20408    125.49412    182.01987    111.66847    306.75634    453.84205    298.44174    668.24352    630.78978    757.12634    603.95479    551.43396    660.73952    544.13144    449.55144    246.74847    152.72763     145.3553    -33.91883    -47.63251   -104.61845    -45.89074     -0.59565    -70.91279    -75.61191    -74.11309     45.29528    118.55986     62.22631     29.07478     11.93788    -71.35235     11.74164     17.91007   -129.56822   -129.16285    -71.31036   -199.27764   -294.96429   -237.06187   -364.24723    -326.0742   -335.90016   -199.61315   -241.91017   -260.08694    -307.0955   -170.51666   -219.30136   -288.18012    -64.30394   -158.22716   -388.57304   -260.90009   -178.83543   -215.80196    -46.71845    -85.46475     90.06219     64.06387     74.53161     89.65271     181.2336    -99.09025    -86.79616   -124.11187     -4.84405   -260.90089    -187.8664   -165.72698   -427.74031   -260.25424     30.94432    -15.57455   -164.35394   -107.68047      5.86457    -47.62616   -139.62142    -43.70611    121.42217   -167.44011     94.41591    -121.8003     -46.7093     13.87751    -53.13635    -28.87723   -163.14099    -58.82553    -97.85249    -78.87927    -69.92742   -224.14352   -134.80354    -34.66896     12.01943     125.9174    138.66501     70.70158     34.85805    180.84364    189.01123    171.17055     97.67106     63.14171    216.76218    235.90362    257.35604     67.98754     90.58287     138.3426    341.88528    275.94539    -29.24492    -20.61139     -0.97878    -17.13309    140.32588    135.55449   -199.27815    -60.87979    -12.40319   -177.69855     52.03713   -172.11996     -81.7259     98.03717    135.98675   -168.04135   -296.34194    -72.76126    -21.78051    -62.70181   -178.15324     30.55028   -121.66544    -198.6208   -427.01497    -214.3963   -170.26872   -251.80052   -387.92387   -253.30732   -203.74589    -13.34853   -461.68138   -103.22602    -165.1897   -323.62423   -278.50137    -84.60241   -148.57276   -332.62445     -265.432    -197.2118   -259.04662   -409.59912   -252.82698    -190.5699   -188.65373    -66.00835     -444.476   -114.25335   -169.55616   -620.59832   -246.51117   -238.68623   -418.32789   -443.58277   -483.84571   -398.89021   -304.96286   -541.79692   -447.40636   -738.33299   -587.10182   -431.75546    -705.5383   -729.88998   -520.18709   -535.87736   -766.98503   -585.58797   -660.76484   -765.82643   -739.44864   -733.16033   -754.99553   -980.11089   -724.13926   -802.13136   -994.17253   -971.02484   -630.92732   -962.60375   -690.37148  -1136.96272   -957.66208   -922.61955  -1118.14591  -1027.47329   -699.81633  -1017.41239   -854.60393  -1021.19516   -738.70364   -671.10585  -1104.35926  -1473.78458  -1168.64133  -1001.29987   -751.84666   -257.86656  -1019.04367  -1103.85605   -1227.6159  -1042.48496  -1106.31773   -664.73838   -868.49188  -1036.92026   -750.73026  -1164.56058   -687.42972   -897.42224   -872.64445   -1246.8326  -1091.85167   -696.75299  -1112.26201   -572.10541   -697.22954  -1111.82358   -924.56041          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        9.00    -401.4474   -472.38057   -600.62235   -241.18606   -398.60923   -429.09755   -555.35529   -231.93852   -318.94381   -351.93288   -242.69558     -49.8945   -171.78944   -179.64807   -358.82043   -108.57254   -152.23378   -180.96794   -269.24183   -381.83663   -341.14463   -295.78977   -311.90341   -194.80706    310.36413   1241.49164   3015.52172    4529.3105   3310.47226   1444.38838    320.26922    -231.8257   -267.02778   -194.35731   -430.80757    -375.5817   -337.85771   -243.09843   -271.12794   -302.08512   -359.27561   -297.98851   -238.98056   -340.56701   -489.51989   -291.89764   -486.85616   -507.10395   -216.91342     -449.806   -268.54223   -289.58677   -359.40601   -184.69539   -417.31056   -234.38059   -251.81483   -265.08246   -154.60003    -170.2278   -186.43641   -366.02059   -400.89699   -269.72338   -318.86253   -290.63197    -430.2675   -297.70653   -359.15813    -33.87531    579.30218   1742.89021    2352.7397   1866.42526   1144.33342    590.96114    173.80892     19.54121   -138.33146   -330.37966   -168.90789   -243.00248    -183.5738      1.94761   -524.75057   -328.11426   -228.88032   -113.16611     86.87788    643.35403   1621.25059   3018.15371   3290.67783   2371.53281   1202.44929    352.65007    113.76785   -161.49539   -275.45428    -83.97518     87.09216    117.45167    250.51919    262.17952     12.64607   -236.66983   -203.12037   -283.32851    -459.4134   -340.38878   -339.35687    -302.3042   -328.68687   -326.32003   -328.83471   -231.35369   -311.35072    -214.7905   -199.95729   -178.12542   -218.04359   -241.74484   -186.37401     127.2649    425.99486    891.35496    926.49596    836.83676    665.93992    287.60499    -30.17665     -4.34493   -243.02173    -74.57303   -230.69321     23.51631    417.33201   1280.66076   2001.30582   2743.41657   2757.40767   2426.32974    1875.9177    856.37794    376.83353    136.46228    139.37787    113.21436     62.87328    -75.66991   -107.32432   -185.56673    -73.49639   -301.68188   -264.08617   -202.68975   -336.47443   -385.74009   -330.31927   -204.77009   -366.89479   -245.08417   -275.85433    -43.98742    -45.08458   -172.57401     66.85276      1.64292      -87.688   -100.67412    -96.91696   -327.51342   -327.77822    -55.19712    -29.78803     31.20842    246.47114    223.10562    224.89748     28.53267    -16.90745    -49.10083   -148.67928   -125.73478   -207.26716    -78.16722   -281.91102   -244.15504   -246.51407   -207.35179   -218.27897   -330.38697   -298.04169   -170.39583   -186.66078     13.68095     89.77177     24.84972    238.46662     235.9316    387.44311    361.61497    149.95014     17.51945    140.47162   -102.94041    121.79768     39.09716    148.39371      5.30906     52.70475     131.3557    113.23547     42.74888   -147.79568   -341.49386   -200.38136   -213.66834   -377.21605   -382.36406   -276.22266   -343.17041   -319.21492    -327.1727   -253.66742    -53.89804    -88.71781     69.03641    145.06231     -0.21133    109.23973     50.64232   -101.75904     152.5254    184.03037     91.12991    258.95034    304.32101    540.49183    322.79219    382.49785     262.8773    137.42547     29.39622    131.61428    -288.6805   -125.32004   -313.26452    -37.65995    -283.6717   -204.91332   -158.04992    -255.4457   -194.55252    164.61777    -23.63276    116.44559    264.19759    -19.00678     23.33415    427.09111    354.69295    655.26589    673.09342    622.99639    742.06589    772.28816    717.21793    645.50527    394.92326    138.76651    155.44151    123.40048     78.69993    -77.60131   -175.22645   -177.56157    -10.59921    241.25948    -42.42575     -0.28516    -17.47527     50.73857    160.96989     47.32296    -50.70692     -9.24027    -67.27091   -102.97435   -206.08627    -91.29433   -114.19223   -182.70709   -331.92532   -362.26725   -348.03399   -398.72086   -425.00601    -318.2241   -210.69271    -219.7998   -183.20858   -189.51885   -194.56597   -204.28868    -91.89562   -194.06791   -152.86163    -82.46428      5.92441    -211.2607    102.00782      65.3882   -101.99878     58.38401      1.09035      6.43143     24.17233     64.47427     41.83796     -81.0126    -64.72025   -121.53746   -233.14062   -257.99263    -148.9395   -159.16596    -96.49871     57.72604   -174.67048    -78.09992   -161.74033      44.2164    145.41226   -165.77783     -37.3212     22.29168    -50.55462   -142.17006    -74.36618    -13.95364    -35.30177   -111.05906     20.53147   -137.74027   -107.51217   -213.83447   -187.54384     -6.75656     11.91225    -73.89728    -246.5911    -21.31441    111.31265    114.05662     77.29415    115.56113    229.65239    290.05546     51.98245    379.34674    283.91398    261.79977     72.66954    140.02163    133.52604    105.57697     55.64816     70.45386     53.81031    174.71012      74.6859     35.49009     14.46166    -54.30056     10.99662     -6.97811     43.47647   -146.79319      7.45212   -185.15152   -176.94861   -141.55065    -37.08834    -94.10285   -198.91483    -324.3132   -151.18594   -189.19687   -181.07293   -158.05393      6.69033   -238.58954   -215.27104     58.09198   -315.15763    -64.95877   -277.80057   -195.35957   -185.59474   -135.14341   -443.01675     -77.3525   -263.42444   -276.07888   -164.06711   -278.06046    -200.8057   -288.09431   -306.97433    -55.47883    -254.8122   -370.39609   -212.39281   -309.35089   -210.53023   -370.60636   -222.53973     78.19348   -139.20812    -163.2123     -1.15939   -343.99675   -381.54023   -187.31682   -402.37809   -160.99989   -159.92223   -468.27124   -423.13417   -475.68767   -662.21374   -556.53077   -841.70957   -639.35778   -809.93493   -796.97276   -751.38259   -620.44424   -695.15252   -630.10617   -714.66204   -763.29118  -1023.35833   -885.35016   -968.29775  -1001.11182   -580.43078  -1102.71077   -769.76048   -918.35609  -1025.53732  -1060.45393   -1237.9229   -876.52228    -734.9818  -1131.51743  -1134.32953   -648.43242   -826.25406  -1243.79657   -811.20811   -908.47549  -1141.47933   -951.94305   -559.08021   -678.64488  -1451.24386  -1098.90561   -830.88837   -817.62626   -813.95824   -970.41622   -1072.5898   -886.90583   -783.85049  -1049.08385   -604.23398   -833.30035   -776.09691   -319.43322   -914.28268   -484.00443   -686.70696   -679.39644   -829.48505   -497.72332   -781.95352    -837.5836   -900.14956          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        9.50   -242.71322   -532.79096   -453.71921   -277.32867   -298.47993    -423.9231   -460.08784   -300.87322    -291.0424   -279.34718     -246.701   -217.00433   -182.98472   -350.27499   -402.46226    -409.0935   -128.76576   -207.91398   -331.24999   -493.44962    -325.1963   -190.26234   -475.96031   -104.30166    323.03888   1244.37474   3038.78835   4482.38476    3488.3668   1487.91459    321.13965   -209.64478   -189.80922   -295.85082   -250.97682   -251.53609   -304.99301   -450.37093    -189.9767   -309.34374   -425.30432   -406.15832   -284.17876   -358.04456   -487.92128    -349.1688   -344.23897   -280.68373   -223.55375   -230.23839   -257.81757   -349.51854   -387.05848   -254.38262   -415.95229     -94.6962   -307.24799    -90.31026   -258.38825   -221.00821   -427.02146   -392.23724   -137.80084   -240.04004   -324.94163   -233.10109   -338.47092   -121.10034   -233.37442     29.00353    668.92786   1685.59055   2162.69857   2167.69543   1326.94842    624.61545    268.25174    111.44041      41.2536    -215.8832   -314.45487   -174.07988   -147.52488   -335.05044   -224.61977   -234.15386   -219.90319    -58.96085     85.29841     668.1717   1936.62162   2857.13767   3175.97391   2583.55652   1205.45835    360.17724     39.79868    -162.4721   -122.32975    -120.3961    136.53151    231.23607    255.55803    193.30777    110.34299   -207.20505    -213.0809   -310.98364   -254.89046   -464.85863   -448.33985   -366.88797    -401.1303   -284.74228    -329.1718   -190.36306    -245.0835   -123.72547     39.10081   -230.76898    -232.7048   -295.22094    -58.47014     70.90791    365.89668    634.94316    882.38029    640.74873    562.35178    230.33132      23.7057     23.56409   -164.65955    -66.36813    -60.78816     46.67156    499.84113   1428.18289   1996.30904    2908.5491   2872.54481   2394.77778    1589.4709    971.99948     472.5712    292.26303    179.45796    178.27911    153.60421    -89.73875    -72.89945    -64.87669    -268.0577   -311.84062   -209.08742   -227.96622   -224.42153   -221.46129   -277.61301   -157.70034   -197.55364   -173.24382   -172.17946   -191.17245   -109.26061     50.42474     51.24669    117.06998    -62.39192   -198.15749   -135.93018   -207.12324   -295.89521   -254.04163     -6.12057    131.93608    196.76344    281.64497     172.5394    269.99605    182.00089    -42.05753     -0.66439   -120.70271    -97.34239   -233.72618    -194.5268   -317.82466   -361.38167   -281.59784   -218.07076   -237.48033   -263.99563   -238.36851     40.65666    -84.47265    -72.08894     44.32922    172.00488     222.3199    181.97056    245.88951    188.10585     17.46203    -83.43339    -47.43708    -12.58546    110.83081     91.07448    282.66034    208.64563    249.35604     99.65292     36.97313   -208.38394   -261.85273    -290.2661   -345.10795    -361.9028   -373.72548   -271.23114   -382.85962   -282.64276   -235.18318   -150.81594   -169.25591    -11.72585    -43.68897    111.73093     65.41826    205.89659    213.67874     60.15784     96.58004    131.04495    270.83079    158.45877    203.69868    358.62045    245.10191    226.86639     222.2994    214.87294    -27.86869    -13.39262    -73.83734   -144.20847   -133.37966   -154.56357   -224.00425   -218.40768   -282.12918   -158.61673    -236.9934     50.50243     32.48019     15.59373    153.05164     46.64858    157.47424    353.21495    491.24644    636.02438    569.29441    595.59226    740.56418    690.41814     787.2442    554.93289     466.4576    243.45435    290.36481     92.07743    -53.55017    -54.59147   -184.92391    -16.01318    -25.42386   -165.96586    -58.81669     95.51787     16.56205    -32.57953    186.50526     67.33709    181.56377     13.54391    -65.14807   -275.30754    -208.0116   -134.17506   -148.80603    -74.64179   -393.95826   -463.11856   -307.55077   -247.81292   -238.45739    -294.6636   -133.72442   -476.50035   -249.57069   -347.71821    -90.35455    -323.0274   -206.09485   -135.08708   -119.87008   -237.27311   -184.26024    -23.48408     18.58838       60.102     150.9699     48.60283      53.1574     -7.71416     52.52933     83.18528    -60.76013   -145.91496   -181.94017     34.71404   -137.79948   -187.93219    -97.87502   -107.93176   -260.24886    -62.29726     72.01099   -244.83583    -64.55016    -75.97405    107.52473     -1.28912     -6.97618    -52.76443    -56.67347     84.77049    185.01645   -150.78731   -181.86456   -137.45694    -43.40862   -119.35412   -125.91652   -216.40949    -15.46935    -65.23051     28.12414     -65.0938    -75.79069    -32.40304    -75.11171     64.41983     76.63167    175.32242    261.56437     111.2294    226.50348    319.93363    209.72525     151.2828      319.115     52.03024    232.55574    169.14019    157.18118    -15.14049    -40.17032     80.83369     71.67988    -39.73678   -227.07941    162.05996     -2.49505      5.95102    -32.76646     24.02071     -0.05974    -237.6283    -92.08644      79.1618   -170.13818   -282.65836   -300.63756    -98.06349    -92.54027    -20.66248     -219.116   -124.13557   -185.59288   -276.12594   -233.96733   -245.65838     -304.074    -232.8967   -230.91715    -34.35319   -304.68041   -380.13313    -32.78028    -81.74675   -226.75116    -13.77814    -60.59502   -175.68415    -225.8895    -343.2856   -132.99117   -158.38577     -184.585   -131.49894   -275.09745    -96.33708   -248.39952   -193.21931   -178.64257    -33.49683    -76.18627    -289.0105   -401.59597   -109.48658   -346.15766   -440.93146   -675.76169   -504.97529    -417.6229   -343.80689   -669.71601   -581.54037   -469.10567   -765.36379   -531.34208   -546.16454   -606.92988   -724.25917   -866.92424   -708.68315   -765.92507   -764.83117   -858.24432   -811.43039   -881.92946    -616.7728   -736.04735    -846.3265   -835.77156   -826.57878   -1326.3177   -985.97356   -649.02415  -1031.54061   -549.44243   -959.74368  -1058.14775   -980.37551  -1045.08894    -819.8359   -976.06765   -771.18265    -907.3494   -800.01415  -1027.03866  -1212.39939   -870.78423   -816.84978  -1102.31934   -820.02214   -973.04438    -1361.748  -1036.09383  -1044.44455  -1360.09179   -887.04083   -834.95843  -1160.54811   -843.80125  -1096.73912   -828.44646   -695.51223   -798.17438  -1199.74921    -777.3263   -730.24565   -345.81041   -614.86759   -373.31971   -823.96946   -486.44227          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       10.00   -173.87323   -395.07866   -316.26338   -290.29988   -250.39292   -388.88719    -437.0974    -456.5442   -272.94137   -257.80138   -385.24766   -412.05909   -161.65596   -268.49359   -557.57216   -398.58274    -39.35268    -325.0771   -287.40293   -272.42737   -240.76221   -149.62208   -435.77485   -136.15584    187.10376   1145.52805   3313.98884   4411.79218   3406.29263   1470.50881    160.86497   -340.01277   -170.34005   -385.13487   -401.12825   -118.44941   -225.72676   -178.74957   -255.13453   -346.05589   -178.63347   -261.59051   -241.84868   -350.14657   -328.40216   -292.57339   -511.25161   -280.58156   -264.57193   -303.16983   -310.54507   -231.99885   -208.73792   -402.72349   -467.67028    -550.4247   -336.67146    -66.94635    -51.84683   -174.39738   -357.63591   -301.52274   -277.04515   -316.32087   -350.60421   -327.65023   -369.70388   -356.88489    -95.05303    -99.02556    817.86237    1908.2347   2525.15765   2089.85204   1208.45939    484.16024    146.27847    114.51614   -208.82154    -176.7548    -280.1946   -299.14034   -183.03803   -230.38479    -357.6193    -66.10295   -264.88529   -123.47197    -68.40549    654.30924    1794.4761     2878.776   3237.43895   2340.20494   1356.54679    282.72893    -65.11672     14.60494    -45.39154   -251.34913    137.07138     148.9293    367.74223    163.31881   -114.81679   -180.06466    -143.7375   -386.65025   -330.52631   -160.31657   -225.07734   -278.29924   -170.26709   -469.14611   -243.71152   -284.52691    -334.9908   -159.83484   -224.81168   -344.95991   -330.56666   -330.40615   -102.99542    121.85211    420.04889    616.51962     977.5892     960.2708    595.65189    282.38816    -35.85491    -32.96835    -212.2379   -192.59434    -79.96889       26.401    560.75533   1299.20057   2045.30088   2887.51005   2838.30939   2213.76172   1719.01462     792.2315    451.11434    263.85914    192.69794    121.92409     161.1024     52.35614    -25.33414   -120.93621   -336.13109   -282.28263   -385.89435   -190.91132   -391.18027   -286.90522    -379.2978   -162.78655   -301.49039   -260.19044   -163.81007    -97.77537     23.50866     13.82143   -201.60771   -193.53737   -138.12431   -315.25054     -94.5677   -298.61761    -203.5695   -229.56713   -151.48125     97.68346    228.22262    104.84877     270.1212     91.56401     13.48851    -39.32378   -180.08268   -147.19066    -68.22965   -151.34384   -240.11215   -227.12435   -419.31552   -151.49935   -175.93801   -251.58118   -348.02342   -183.39745    -31.16214    -64.70514    105.60466      6.54514    172.34505    230.73344    250.92586    218.95285    177.54869    193.59002    144.35852     -49.2652     53.52583    107.74164    203.29382    219.27498     44.46217    -30.85009    -37.41348    -93.66192   -122.39277   -181.88974   -243.24737   -201.31575    -305.8527   -336.49162   -406.93333   -410.10559   -228.85759     -250.485   -155.82919   -275.56647    -48.05601     96.92703     -42.5739    201.60346    205.86932    113.58533     66.66542     -7.04781    239.07527    346.30649     336.3587    416.91305    306.99954    378.98356    257.81365    517.96057      3.44888     -83.3974   -127.22301    -63.19892   -233.26185   -442.89791   -215.15525   -353.70655   -154.73954       6.6251    -82.65486    170.74216     27.14411    130.21359     207.6157    -39.63915    259.10539    220.97695     288.9371    525.21075    513.90103    537.32208    709.56391    771.26619    766.57175    696.71032    527.11359    411.20354    398.66466    285.66212     46.36453    187.03936      3.57099     29.03439    -35.99853   -187.54978    -60.02919     32.32887      0.34839     53.63828    147.56198     92.54104     79.94974     43.42329    -58.59207     -42.4233   -146.52898    -39.29758    -173.4484   -340.21564   -352.56393   -174.76296   -358.30603   -158.05541   -370.34937   -279.70739   -315.99671   -234.86828   -492.94498   -294.50155   -210.11693   -202.19151   -141.41004   -109.58183    -96.34256   -200.40112    -57.69411     42.76399   -132.83576   -130.32482     92.58331     75.96942      77.2126    126.25856    -80.95773   -104.16005    -18.76952    196.01208    -30.25561    -45.31039   -151.39146     26.83527   -248.21206   -252.15387   -164.24959    -54.45806    -34.96819     -5.81704    -51.59168       8.1338    -82.29863     -19.1989     -3.81059     84.22003   -118.85708     75.76702    -39.73954     24.51426    -34.85295    -171.8506   -133.40552    -72.41804     -5.75795    -16.32529     -9.94188     -8.70529   -146.44215   -240.07091    167.15796   -243.33736   -266.99252    -88.37334     15.63655    163.75573     40.81874    203.16827     32.62291    132.86013    314.65224    202.84674    103.21591    292.43262    118.22523     89.77428     82.90191     44.19791       66.945     159.2328    -52.90566     44.92733    175.79904     31.33095    -55.64592   -160.44183     66.28228     40.29476   -210.26623    -69.12036    -19.84271   -291.45888     -224.403     56.53529    -16.35234    -194.4334    -99.57429    -55.20761   -125.32651   -508.35717    -61.81012   -373.97178   -482.56649   -217.47985     21.93328   -226.48721    -181.0971   -344.39741   -241.13544   -428.92418   -322.63588   -171.79557   -194.08563    -73.09593   -255.34389   -232.92911    -61.43719    -89.68433   -106.59104   -249.54162   -332.26039   -271.13399   -232.66915    -42.30504    -18.37188    -90.67815   -300.71072   -541.29678   -324.79825    -596.3658   -151.50768   -414.37923   -484.37923   -217.50853   -379.06972   -419.79641   -417.19941   -204.80738   -330.05416   -559.99002   -550.28668   -471.89616    -634.2922   -540.32136   -515.47552   -456.47629   -666.74869   -671.37108   -647.05209   -624.06276   -801.40663   -593.01409   -768.78505   -898.69959   -980.12786    -784.2179   -879.50901   -970.90274   -820.92839    -931.0315  -1006.42081   -977.16064    -757.7304  -1042.14057   -896.34239   -761.01064  -1199.78445   -839.05876   -1117.1294   -783.53068   -945.41731  -1257.40889   -757.47581     -522.527    -940.1548   -844.52061   -753.43609    -780.3527   -924.55044   -618.06098  -1249.59984   -837.13012   -916.77456  -1094.77327  -1022.99279   -680.38088  -1102.02409    -830.4523  -1021.95479  -1149.26922  -1009.83519   -783.94191  -1288.99271  -1265.11012   -876.06415  -1047.97434   -642.46853   -648.82508   -489.24762   -311.56577          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       10.50   -221.75523   -287.14725   -325.81437   -291.48832   -224.63331   -335.64564   -472.22187   -611.57289   -205.54203   -316.20969   -459.26841   -440.78743   -119.27053   -247.80686   -330.73538     -87.0865    -20.42611   -176.54577    -88.97677   -260.93713   -394.54373   -233.24938   -294.92685   -260.67135     92.68538   1185.05073   3189.08005   4457.54408   3415.73499   1351.08069     73.61506   -353.84004   -297.54016   -302.32992   -291.62747   -409.17399    -247.1541   -287.32916   -166.22533   -418.35694   -434.59317   -260.94146   -188.73232   -213.07418    -282.0501   -420.57228   -384.64216   -368.55454   -417.49846   -405.16534   -133.59899   -390.40114    -255.5857   -475.40133   -252.25755   -378.64943   -296.54291   -165.46297   -154.12181   -294.84824   -329.78432   -222.02791   -355.67976   -169.44651   -398.76071   -368.31589   -445.95649   -317.82864   -196.35836     68.86302    732.33957   1626.00693   2298.69471   1945.60408   1035.49084    387.18826    248.67098     21.98189    -93.78355   -329.67521   -450.54122   -375.97729   -395.87761   -215.97564    -114.9026   -358.38561    -50.06906   -152.89914     280.8562    803.69739   2029.12755   2865.36793   3081.13304   2373.25998   1182.94472    555.17871     17.85059   -271.78637   -242.38913   -146.40803    -134.0201    364.60391    216.21165    262.72367     50.95085    -91.21583   -149.77391   -327.59886   -288.97556   -164.73643   -270.65941   -336.98287   -438.71132   -227.47805   -252.68884   -159.01624    -224.2294   -140.81179   -184.34403   -162.52149    -208.9046   -144.04692    -61.68321     66.17528    444.47639    729.73456     949.9631    783.17201    660.16061    274.67549    -66.98028   -147.79345   -215.52328   -139.55256    -41.08044     15.70486    510.60984   1465.26974   1915.30704   2691.63026   2751.99092   2199.69932   1624.37396    921.50265    355.97221    118.91946    247.93142    169.76587    250.22718    234.27059    -42.13316   -119.86975   -134.64166   -216.55862   -453.79404   -165.89949   -387.56685    -162.9705   -215.10617   -169.06921   -340.01051    -141.2437   -261.74472    -25.14989   -140.48076    -244.3115    -55.79601     -6.46329     21.86136   -109.56891   -279.34671   -295.89734   -369.07852    -310.2467   -244.04436     80.98845    314.57262    136.61286    195.70852    221.63131     20.23577    -70.30206    -89.69197   -192.93491   -121.23068   -283.89361   -328.49797     -334.966   -315.35428   -252.21894   -301.84175    -345.1203   -283.97994   -226.74497   -148.18725   -152.57226    -74.83587     64.18128    115.22901    350.65361      333.723    348.64561    121.85256    165.29401    -18.68765      -2.2856      0.59777     44.15261    117.80437    181.75955     96.46821    125.77458    -64.55915     -1.05442   -138.27325   -233.15542   -302.51011   -337.02556   -180.89353   -298.71258   -182.70136   -337.68432   -400.91391   -324.36991   -131.87428    -98.38132   -185.54089    -37.49105     60.57919    -15.43199    -74.89802    187.21186     30.72998     87.15297    256.78973    230.97003    229.89097     302.0692    298.67575    390.51919    237.03892    191.93848    171.64227    116.76322     71.83303   -122.79213   -209.83722     43.17561   -279.93823   -285.18217   -218.86159    -168.2843   -194.32014    -66.25425      6.51093     34.16247    -32.59813     79.05061    138.89603    201.15689    386.10285    554.64309    609.31735    630.44589    855.60761    861.40785    767.06498    625.37254    446.32909    484.85393    353.54128    420.84518    212.08344    -66.63383     18.33766   -199.96109   -184.87412   -122.63166   -131.90706     38.27675    169.39684     73.22527     -7.34887    160.93368     80.02946     14.21854       -84.24   -142.36999   -144.03718   -197.04101   -225.33683   -282.14659   -192.29042   -357.15821   -458.22628   -429.49025   -145.81126   -329.84332   -194.79006   -331.40286   -394.28568   -215.62371   -179.16064   -268.85394   -280.90066    -44.18932    -54.38585   -107.45976   -255.11099   -158.70393    -24.43464    -54.87972    -283.7909     27.24193      39.9232    -33.43423    -42.43849    -35.48065      7.49225     39.66023   -127.65938    -41.11791   -265.34104    -52.27065   -133.81759    -19.85854   -249.52068    -168.7326    -85.49432     63.85855    -32.38846    -76.10337   -116.56343     42.16776   -181.06768    131.81317    -65.11031     91.93745   -138.71541    -77.41953   -193.95888     31.86887   -285.30669    -78.01035    -53.78863   -221.51765   -139.89444    -120.7473     58.45177   -197.31886   -101.00997     35.85781     12.46143     -6.14627     -7.77393     95.47709    -43.54947    311.63033     67.99502    195.46736    112.82606    232.43426    205.88113    283.39315    117.68662     42.24806     32.27831     18.29798     32.77594      6.63092     23.89813    -73.57866    -111.4071    -83.14216   -164.29064      5.84367     -7.81734     95.81654   -191.27781      0.25166     37.30077   -329.58879       4.6284     29.47711      4.60534   -181.33867     10.12714   -161.27441   -247.29779   -164.37624   -129.33001   -112.56853   -465.26884   -192.91193      5.73242   -315.59048   -123.82258    -91.99038    -61.74285   -154.29906      48.6249    -54.81269   -238.58115   -212.97902   -129.29645   -183.85394   -188.06425   -193.32919   -264.11719   -297.62662   -128.01904    -92.98889   -174.42217   -310.72273   -336.07688   -265.89964   -250.08713   -223.40126   -284.68809   -278.57317   -375.91028   -194.11025    -408.9336   -367.01736   -520.08567   -563.53083   -333.76546   -485.49578    -473.3186   -764.42582   -543.05061   -621.76001   -643.54027   -435.68909   -841.41172   -814.09712   -801.96881   -691.77998   -791.17021   -553.00339   -749.44343    -841.4246   -624.36162   -684.32994   -872.16604   -828.30488   -870.43701   -996.41547   -923.30594   -915.12084  -1179.44661   -709.38584   -758.20897   -817.09934   -726.87264   -989.86572   -720.73134   -695.04288   -855.79273   -927.96021  -1038.71656   -482.61434   -918.35456  -1207.26442    -632.7516  -1015.66581  -1095.17992   -826.12056  -1284.11435   -1157.2529  -1169.01161  -1025.57854   -617.40474   -734.95134  -1153.00178  -1112.00261  -1078.40329  -1306.24031    -998.3754   -712.40196   -856.53565  -1197.78619  -1169.69618  -1523.41372   -568.70049   -562.01212  -1034.34418   -546.87343   -185.40589   -577.43768          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       11.00   -286.50909   -235.13555   -443.35499   -291.47846    -277.1096   -311.33067    -480.1424   -498.15066   -102.55561   -260.88123   -351.73615    -469.2172   -177.44814   -324.89075    -95.59989   -242.12047   -257.50177    -44.33669   -141.44155   -361.07804   -430.35607   -223.81734   -217.46703   -277.15743    200.65304   1171.61864    3352.9553   4644.32524   3456.52648   1348.88664     44.16161   -308.97697   -301.59935   -199.85188    -316.5503   -395.43753   -316.80618   -292.52566   -363.71081   -291.41081   -407.14274   -462.91873   -476.06303   -340.17012   -496.29862    -248.5565   -581.83192   -349.74331    -385.4566   -369.85472   -346.11011   -384.04627   -291.27663   -415.74092   -245.34998   -364.85053   -221.13859   -198.01038    -93.25333    -42.51167      -255.32   -340.11406   -124.89182   -367.06587   -260.03528   -374.36578   -292.95871   -187.05667   -160.66841      6.29092     734.4008   1741.64009   2401.53953    2150.4676   1124.86278    419.74945    309.64583    325.64281      0.12667   -503.27163     -435.873   -367.27869   -425.97453   -302.58845   -258.89588   -320.76638   -501.20694   -260.50578    200.42998    623.79572   1885.84311   2783.33097    2942.1498   2203.13677   1167.13825     397.8862     19.14913   -239.84526   -380.62635   -125.81917    153.94747    203.41222    279.56449     196.1232     11.24341   -108.94512   -210.14917    -183.8643   -281.84625   -362.18487   -259.11269    -166.1907   -207.18592   -221.37871   -322.19091   -194.55836   -200.03391   -108.99597   -348.11808   -242.37995   -379.87816   -112.26978    -194.3033    128.51927    358.49474    772.30639    826.17008    830.60696    700.09876    196.66678    -14.21455    -132.6801   -234.17376   -223.83174   -139.53149    107.41379    677.54026   1288.32832   2271.61223   2633.06068    2758.3464   2216.07394   1653.76201    887.18715    497.67731    346.25737    223.98882    146.72577     27.47635    -61.46062    -57.63176   -135.90193   -212.88788   -290.46859   -365.76984   -154.63272    -289.9181    -243.1404   -286.45751   -357.88672   -201.00894   -212.50249   -160.58537   -222.54043     21.83623    -13.22226    129.86199    109.45081     -52.9944    -44.95546   -103.21975    -340.2368   -244.49254    -82.81018    -85.43943     48.46824    226.63992    285.41274    309.40694    221.84945     161.7151   -102.88311     76.92239    155.95746   -286.86777   -208.68896   -435.35624    -249.9333    -360.0341     -318.393   -289.46766   -267.24811   -105.92315   -284.65387    -15.62548     -121.307   -168.59685    230.07418     124.7798    223.94443     14.61768    393.95921    193.95461     96.79181    -60.11108   -136.09965    131.19864   -238.97122    146.84261    131.74071     82.03377     90.93574    121.25954     -3.98906   -124.23471   -204.69553   -400.46243   -335.49047   -449.68637   -224.12024   -237.74619   -281.30842   -212.79919     -98.0499   -347.36057   -223.25256    -70.35458     41.65391     39.63506     11.00531    142.12465     56.83999    125.67042     -9.57142    -50.56807      311.066    264.15819    250.23199    248.77684    243.96581    292.73343     249.9003     148.5425     97.34612     96.07263    -16.41318   -200.19871   -189.91644   -240.92156    -259.4097   -197.69033   -349.15979     -31.3972    -107.4426    -56.60639    -74.99307    197.38262    -82.25034    188.60741    244.37591    186.58639    398.84967    658.96924    643.77717     550.0535    720.83758    773.13887    707.12887    672.48497    429.83148    306.77225    309.91455    141.88216    -46.07869   -126.02832     -22.6459   -137.69846    -44.64522     73.94404     69.19614     34.26821    151.79458    112.38423     98.22047    112.72866   -113.90152    -92.13714     59.82371   -218.78314   -166.06263    -99.54299   -312.15379   -284.39458   -218.08506   -179.80637   -185.51421   -204.23491   -285.10631    -289.7944   -257.30568   -151.19433    -231.4187    -245.5981   -319.45284   -203.81058   -146.27706   -202.30212   -212.90548    -25.18893   -236.67846     14.34254    -18.18252     -3.77733    -61.43498    -70.44258    120.47512     28.74613     22.34056      4.20353   -164.51873     -76.3358    -33.46341     48.49472   -216.93488    -74.30597    -92.99542    -39.14226   -132.52848    -69.17937   -304.09639     61.19365    -53.75745    -35.69224   -112.46763   -101.64258   -136.71751   -100.20666     96.71042    172.98278    -138.2736     73.65117    -62.50528    -77.30164   -229.67356   -232.96174    -69.90345    -82.45252     -8.64813    -156.0742     -6.67572    -153.3247   -164.81381   -111.19768     90.21301     64.80287     23.61923     123.1224     191.7847    127.27518    365.31559     100.8716     13.49091     100.0048    206.00742    158.25121    209.00305      9.71942    -23.04355     129.5719     21.90372     76.12081     42.49797     76.52522   -141.50427    138.02195   -245.80477   -256.64584    -37.27607     18.33101      8.77333    -57.84792   -130.80555   -183.60685    -78.35153     -1.84386   -117.61984     40.72519   -256.22109     -83.5885    -89.48846   -244.46521   -248.49837    -14.72856   -239.51339   -620.30272   -253.00785   -302.11184   -502.28525   -245.38582    -146.6683   -240.95557   -282.51411   -251.87414    -19.52886   -247.35022   -505.64343   -369.14586   -562.58916   -221.98029   -336.29788   -284.11209   -137.76036   -116.37375   -145.52696   -133.80363    -93.18009   -389.70779   -321.40735   -200.07114   -492.80208   -437.76089    -90.62929   -321.28242   -496.61816   -409.33267   -289.04656    -576.2171   -474.24702   -834.71069   -462.18846   -542.77128    -452.9733   -700.46181   -871.71519   -971.20975   -560.73411    -566.9918   -512.89376   -819.07065   -592.11483   -634.15671   -915.79995   -828.81093   -720.40589   -953.25893   -838.98122   -965.99661   -701.99757   -970.15353   -910.81783  -1007.03327   -712.73843   -950.99657   -804.04227   -805.02575  -1061.02508   -994.44288   -779.18875   -831.10387   -907.88295  -1049.44775   -693.44712  -1128.93257  -1081.53042   -632.64273  -1236.20162   -695.19156  -1259.86008   -988.83833   -921.98449   -915.84291  -1106.83494  -1134.71073  -1200.07333   -913.71153   -912.93609   -1186.7301   -893.60832   -864.58161  -1353.33898  -1029.87843   -526.51582   -988.55277   -610.52072   -627.00463  -1057.69404  -1053.27129  -1177.89794   -652.53712   -890.62075          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       11.50   -389.77225     -218.358   -535.17584   -333.53465   -338.38457   -300.96008    -433.1486   -377.77749   -169.31549    -29.88632   -296.94817   -429.91586   -363.19564   -377.77702   -215.47495   -333.78047   -312.55416     13.80413   -279.25061   -425.69342   -403.63841   -324.06135   -264.00175   -310.73554    171.46227   1020.05442    3299.5504   4207.50174    3420.6709   1513.27394     119.2358   -202.36587   -102.30409   -219.75624   -248.89964   -125.19267   -307.34379    -27.26797   -194.70997    -207.1701   -359.69097   -337.84994   -264.35993   -382.64881    -478.6734   -253.18156   -408.64856   -268.47567   -268.92265   -405.99543   -494.64255   -459.19404   -371.55108   -425.36249   -341.68407   -342.15434   -481.23291    -165.6373    -87.50164   -139.33705   -255.89279   -292.63036   -360.84993   -241.53045   -195.68652    -119.0599   -377.86629   -388.32058   -298.01953     72.36299    597.90274   1984.89824   2535.41095    1875.8356   1282.01239    474.75079    281.99968    -20.51682   -343.06795   -587.90053   -421.36056   -305.45407   -205.55687   -408.66176   -412.34148   -148.60017   -166.33793    -84.84358     46.25275    705.80125   1761.85619   2772.03961   2979.32168    2164.1103    980.19069    472.57608    -60.01247   -158.05333   -296.17866   -128.54237     60.26363    167.34545    207.20676    118.36363    -11.53746   -161.20112   -267.11169   -313.79275   -195.54471   -358.77424   -277.75243   -229.04898   -363.73727   -252.91397   -300.83536   -285.41924   -214.09984    -253.1936   -190.56581   -121.67352   -201.12542   -294.65969    -58.64777      3.71051    451.48684    808.64836    931.93253    852.34845    630.57044    245.50564    -47.13155   -193.28947   -132.23629   -106.44812    -74.85909     52.30625    449.23995   1391.02968   2149.69295   2907.68519   2912.79994   2334.16805   1682.66137    906.37339    475.30266     179.7508    174.93476    201.25854     33.38375   -125.94558    -25.52169    -85.28783   -205.15898   -266.25255   -374.02043   -233.96948   -341.54562   -233.14525   -352.48819   -250.42672   -178.35262   -360.29865   -294.60811   -158.02713    -79.89815      -5.8001    127.01473     13.67651    -27.15639   -192.15494   -280.62089   -145.72186   -159.76819   -194.61461     -69.5388    -57.70195    356.67662    240.88104     139.6458    180.75734      35.1487    -85.93276    -25.35761   -128.48606   -245.98665   -242.65531   -251.82302    -376.5718   -277.53884   -345.91039   -266.10773   -239.12116   -237.95041   -193.91417   -197.34387      3.68781   -142.79706     39.82553    315.21804    407.09393     82.32103    250.02602    351.57701     31.12582    -51.30988     79.04674    -17.38608    100.11712    157.01894    187.82812     225.6166    251.95729     -9.46851   -100.51042   -118.49789    -211.8871   -319.39385   -290.77029   -378.19419   -266.43282   -149.06078   -344.48551   -235.37353    -99.18294   -117.43779   -148.35352    -35.80079     61.16136    143.47871     130.5943     57.27415     29.46499    151.65783     77.95938     96.92765    316.16458    192.04143    316.02045     410.1983     97.65141    272.42437     63.13307    254.43677    138.39398    -11.17754   -223.98244     17.98155    -97.22352   -327.01911   -338.82453   -336.73148   -316.55665   -233.93862    -47.34465     -1.12877    131.90215     -6.04904    123.05722    132.66423    166.35857    380.75703    349.69865    476.73913    488.75836    655.35576    822.94933    833.77478    726.20516    491.38428    196.98225    210.98472     179.4083    176.85877    144.85321     35.79061   -183.24413   -203.85456    -66.35923     23.91766    111.66566     55.29964   -136.16424    225.32383    129.84253     -32.3075     50.33979    -15.11681     56.83539    -30.59391   -152.01779    -189.1247   -218.59521   -134.27794    -311.1417   -327.29758   -361.35072   -225.72606   -273.77156   -143.44933   -274.54396   -345.61789   -223.11424   -300.69807   -297.93956   -274.78272   -210.75342    -89.43694     -80.6583   -116.65413   -200.26989    -12.85825   -185.29393     10.72555    -13.21105    210.58596    220.16386   -143.36607   -154.78101    -87.63215    -16.95647   -102.75401    -125.2248   -156.74603    -45.50586   -162.57539   -268.53437   -109.61722    -79.77123   -192.05948    117.72917   -265.24503   -173.19917   -135.42266    -88.26484   -119.28029     -0.50927   -126.53991    -86.99275    -78.68335     106.6922   -150.54335     -9.59332    -59.82287   -113.56715   -301.46927      -32.163     57.20197     -10.7974    -35.91721   -111.45299   -164.28465     74.49741     21.36368     13.59273    -89.87952     34.95574    172.01544    133.55929     59.72712     275.5457    249.90176    304.91986     224.8467    314.02233    195.33732     -0.25138    -49.05691    -37.79309     12.85519    -20.52996   -198.59434    -95.69901    -38.71216     -17.5848   -182.43918    -122.5559     12.80578    -16.55984    -62.98531   -305.86616     32.41528    -271.5528     -89.9078    -129.2151    -58.54027   -106.51609   -158.87215   -188.77573    -87.83215   -337.07804   -174.36167   -174.44771    -17.51642   -152.84537     -77.6077   -235.07193   -472.42576    -176.2564     24.28039    -42.86558   -202.49528   -187.74633   -295.10461    -74.94942   -229.37062   -194.41523   -318.34531    -86.83077    -86.65058    -140.2343   -163.06281   -244.30345   -218.63754   -280.27876   -244.86435   -162.46828    -457.2776   -171.61457      0.03314   -293.18743   -524.73525    -29.40577   -230.78849   -155.98083   -453.87879   -279.47151   -481.84575   -142.47008   -510.97007   -369.27911   -544.00807   -374.76445   -499.43213   -532.97729   -525.89025   -580.70954   -721.75091   -601.58416   -533.55013   -809.45354   -655.11129   -839.71631   -831.90353   -401.47586   -832.95041    -839.8408   -723.16142  -1011.15955  -1077.98467   -821.10618   -888.31018   -866.45231   -664.18483   -979.90223   -967.29622  -1047.70461   -753.18683   -800.39787   -752.19671   -906.01916   -837.63713  -1092.12784  -1134.55018  -1057.85462  -1073.01128   -921.97108  -1060.15313  -1139.50263   -636.95748   -849.66658   -760.78743  -1127.50388  -1502.04678  -1309.89732  -1147.18842   -918.89048   -951.76373    -778.5591  -1073.21145    -502.0334    -913.5374   -630.28998  -1640.82616   -648.17252   -334.98793   -677.51984   -1033.3279   -948.99896  -1520.93264   -678.82764          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       12.00   -495.34743   -211.57305   -455.86781   -392.26312   -361.39719   -219.50607   -377.86308   -387.93312   -297.20991     77.39386   -385.55914    -366.7775   -360.01438   -404.48669   -161.75745   -153.67383   -268.29346   -228.23005   -378.78138   -307.22851   -347.44389   -320.24446   -185.05375   -250.91539     92.55776   1265.45923   3185.58924   4158.92733   3383.10376   1488.79772     66.61592    -84.25077   -244.72982   -335.07293   -215.59489   -471.19314   -249.50106   -400.10491   -181.99895   -387.70651   -191.39741   -488.28585   -271.67806   -352.04767   -252.55843   -195.41226   -432.47924   -462.59217   -401.58188   -494.41925   -373.40306   -427.84904   -227.80245   -384.82903   -259.98369   -401.86732   -319.77017   -202.86073    -60.44134   -245.45055   -378.95498   -244.57355   -398.99584   -244.91551    -258.1531   -419.64221   -367.50415   -135.85027   -242.62666     65.03999    798.68822   1699.41881   2255.54836   1894.11322   1092.26103     406.4952    125.93604    -81.07051   -274.84829   -519.51173   -570.86714   -521.76662   -248.40333   -254.18366    -265.4117   -289.83354   -296.60924   -117.03407    174.49488    717.44421   1710.59771   2815.94185   3069.39994   2319.00138    1210.4662    277.24864     56.81087   -182.25035   -231.67064    -22.57096     173.2314      71.3371    224.22559     252.8272     -32.6893     -209.901   -244.50425   -280.44214   -263.19442   -327.19636   -275.33526   -353.49073   -239.61974   -207.81081    -271.6495   -202.34282     -89.4581   -196.29001   -207.79341   -315.05402   -243.38886   -372.12868    -76.08763    146.16714    363.37214    792.35813    766.42273    789.40742    641.11482    327.38229    118.78547   -108.92644   -307.13851   -275.32196    -62.99342    -56.09182     361.5019   1195.42821   2033.67758   2729.72898   2647.87047   2310.23882   1586.96105    730.81644    514.94434      314.132     84.62452      37.0556    105.45416   -111.31441   -136.75899   -199.04843    -240.5326    -288.7649   -267.47832   -342.99148   -217.29377   -281.32912   -197.46328   -219.76353   -223.31821   -101.69544   -124.51478   -147.99598    -13.03135    -27.48687    -93.93206    -96.52717   -132.34496   -121.60658   -112.31062   -325.15138   -297.80521    -98.78898    -21.28705    129.87614    128.32881    304.70614    296.25072    112.23993     98.92372    241.06859     -43.7337    -108.5835   -410.53155      9.62021   -171.56714   -116.33462   -238.60088   -180.27835   -174.93907   -376.84166   -295.75306   -223.36689   -248.62748   -119.48463    -84.08901    171.11187    277.00702    202.02096    202.89089    199.90846    123.81328    -15.55946    -44.69117    -68.86378     99.39017     71.73134     83.32276    144.52775     91.15295   -107.20144     69.10729     89.41672    -67.99101   -209.13555   -225.98592   -250.40602   -417.65092   -414.58126   -104.11991   -172.17767   -431.28501   -105.24293   -187.79791   -214.59565   -174.77913     62.82776     32.22033    134.37759    177.15516    162.92022    168.78441     13.48179    153.67801    181.32929    354.39053     335.1499    206.01174     90.03443    308.12355    308.98965    105.01504      6.55718   -143.81944    -36.88091   -145.55093   -302.71108   -186.92153   -184.40843   -230.96294   -184.22967   -261.54141   -128.32159   -130.04057     24.85815    -31.72888    108.43184    197.65696    212.11589      272.438    470.92692    499.31935    561.74936    551.90331    849.41053    705.29028    760.84991    535.26901    669.02441    349.11354     65.54292    208.71079      -71.995     92.57219    -76.28727     77.77813     -60.5637    -63.57097      5.83156    209.14578    -43.60332     97.23966      3.88473      93.2884    166.75806    -15.98505    276.88846    -60.70888   -221.05893   -242.46842   -204.75302   -361.86013   -382.95828   -263.81705   -300.93264   -292.31454   -284.74535   -299.06418   -317.81997    -158.5258   -218.06782   -215.72075   -188.98806   -237.28941   -246.74062     -5.11549   -221.27745   -114.14372   -220.47354    -46.80237   -114.42954    -90.69151    -59.39474     -95.3077    -88.65466     -8.54695    -73.07283     -73.3057    -59.19377   -223.11472   -170.45967   -232.19081   -126.81064     -32.3158    -106.7806   -119.29233   -278.07645   -132.13731    -74.39132    -46.47634    -67.37425    120.83318     -29.7512   -168.92857    -51.76422   -175.59082    -94.87668    -35.29118      56.3535    -15.24046    -116.6954    -19.19897   -121.88126   -170.08637   -152.17688    -46.45795     29.75695   -444.78291    -18.33372   -106.01551    -54.98181    -48.32058     -10.8043   -124.39088    301.16475    236.10605    261.01366    -41.75628     70.07755    277.63214     220.2077    179.35439    355.60878     -4.52113     93.56901    135.69998    255.56241      8.10308   -126.32642     -63.5079     46.80778   -181.48935    -58.18414   -119.12029     56.66844   -187.92026     -9.00637      82.0674     100.0679     -9.16501   -223.84237   -146.53315   -124.46836    -40.44204    104.02313     -68.8677     -59.3377    -228.3535   -192.93474   -252.74976     69.27178    -62.63379   -189.29851   -431.78281   -527.02269   -274.47811    126.21891    -123.4038      71.2126   -355.88412   -202.81215    -93.63408   -104.80087   -231.40528    -88.39812   -177.89731   -305.77145    -364.0574   -244.20093   -303.16718   -409.01929   -303.13958   -434.62935   -407.49281   -253.79732    -341.9193   -519.33757   -291.14386   -570.99007   -781.70225   -165.53058   -106.02432    -616.3811   -286.86282   -648.67119   -637.72309   -639.81485   -505.47029   -352.54149   -397.50959   -500.84468   -633.02865   -241.34701   -647.56312   -541.38239   -723.78058   -896.94768   -561.82459   -726.01968   -812.38447   -969.95876   -904.98772   -934.06814   -750.56689   -913.95915   -964.32301   -778.58602   -707.10739  -1034.48518    -889.6129  -1118.71654   -820.59873  -1059.68067  -1054.92486   -752.96101  -1018.82629  -1212.84713  -1020.52355   -873.65856   -760.07719   -842.30745   -873.04172   -583.18872   -849.71069   -931.53422  -1095.41362  -1119.13202   -984.33389   -438.88966   -489.92562   -965.69924    -325.7923    -815.6153   -761.59857  -1072.40061   -1266.6458   -758.12868  -1050.94968   -962.09275    -838.4366  -1611.91037   -873.07646    -893.2011   -475.38088   -756.99124   -485.28719   -948.44462    -626.0433     25.06557          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       12.50   -561.37323   -276.93366   -373.63316   -420.47399   -329.63258    -78.25845   -460.39028   -395.03869   -411.48692     -27.6686   -376.61869   -236.31989   -359.84808   -250.53154    -47.86129   -232.31384   -197.40924   -362.84193   -193.04357   -198.44086   -430.95216   -244.22148   -248.86838   -388.56424      21.4016   1194.89609   3189.24815   4075.51716   3068.44003   1466.54279    215.73419     27.00882   -193.99751   -230.88278   -201.27293   -318.13252    -268.9138   -237.22646   -273.61736   -404.71847   -308.67766   -527.42194   -293.91593   -271.25742   -297.41142   -471.58159   -410.78782   -308.36581    -437.3914   -383.54507   -273.34461   -349.01011    -84.55134   -211.79562   -315.81475     -418.096   -394.75757     -106.021   -248.58524    -195.9404   -377.23634   -244.55041   -197.07896   -248.51295   -250.65338   -367.14097    -325.1278   -217.29462    -62.11228     76.46444     783.9403   1939.83045   2380.03535   1993.55388   1292.75508     393.7627    450.21135   -311.03472   -207.41814   -464.77137   -497.46107   -356.31801   -325.72799   -541.18454   -260.71338   -187.39422   -103.10009   -219.07671     -2.61607     839.1555   1914.95874   2949.27612    3231.5838   2172.78406   1252.49789    380.31662     -3.40945   -215.35842   -217.84976   -190.13508     -2.98021    159.48085    320.74806     113.5334    -51.67758    -33.06472   -280.42515   -251.02466   -253.40923   -408.11228   -403.85001   -235.92068    -292.9556   -282.50483   -262.63555   -347.79859   -161.74845    -84.10353   -241.26825   -148.98179    -91.85923   -190.89192   -180.57188     48.65274    269.88751    806.66829    971.17257    835.46805    472.62677    297.50443     33.52153   -161.59234   -180.44669   -230.00735   -230.68302    117.12189    518.72807   1285.53515   1960.39548   2819.16666   2817.67235   2412.85939   1881.03276    731.92573    209.73926    309.39994     217.1068     99.75473    105.82765    115.87024   -140.30078   -151.93472   -190.85441   -141.97805   -394.92905   -343.59955   -328.69255   -242.93301    -269.9666   -220.61821   -232.32732   -247.33774   -173.57786    -45.86225   -117.46518     46.06365     52.07641    -24.67281    -63.16258    -150.2265    -89.23786   -171.21708   -142.72515   -208.16547   -107.85501    -94.88383    159.65869    192.93393    237.77559     49.69671     81.79927    -90.59386   -111.32799   -245.58469   -176.11992   -225.96518   -142.55239   -323.65094   -195.40142   -251.52327   -256.75827   -348.29975   -305.68268   -226.50078   -222.34654     -3.42817     91.99563      29.7517    198.26271    151.97239     71.19329    290.55504    161.56646     46.50127    -54.19052   -105.13175     79.65555    156.29575    211.50398    274.01215    142.46914    121.90271    146.34929    -97.37806    -88.93021   -151.85605   -159.72387   -267.57745   -235.33542   -281.64331   -348.60264   -198.81007   -412.28622   -371.04784   -211.92593   -124.92866     -11.6407   -143.02911    226.47082     16.78464    145.68159    159.83212     181.5293    267.65062    178.88742     139.2378    278.82064    172.95547    174.68136    200.39181    240.39966    223.81476     254.5099    -81.14841     74.02504    -86.22393   -227.97746   -165.45115   -339.32365   -278.17917   -157.44273   -302.51551     21.42084   -196.18121   -163.53409     79.10026     77.80531     29.81442     49.38589   -105.47174     202.4152    358.89675    676.73976     570.3874    798.97263    842.13582    890.45312    610.44446    648.94323    587.65745    215.00218     104.3678    144.96864   -143.02722   -108.96787    -67.57706   -147.21529   -191.93539     34.87704     83.68063    -50.50371     34.68513    105.95465     29.93542     57.40914    117.90762    -29.12158    120.15585   -117.93765   -205.35621    -20.10445   -214.96655   -229.69934   -299.64614   -301.03067   -333.38146   -443.09836   -217.10977   -357.54424   -289.62169    -260.5494   -361.03123    -401.9038   -269.20742     92.11203   -171.11862    -98.78317    -66.22799   -280.62234   -107.39345   -152.78767    -42.25195     15.62548     42.76387    137.49143      39.2193     71.35334    106.96812    -33.67881     78.58607    -94.25886     -93.4858   -114.91058    -99.17714   -104.43421    -60.40893   -163.95202    -170.6657    -78.09332    -85.29078   -235.09112   -137.60733     73.13768      4.81199    215.02475   -111.74389    -76.97434     -8.45082    -64.01506   -260.57185   -208.82444   -200.26481    100.83937    -93.97817   -179.96289    -23.18106    -61.16837   -241.65809   -108.22026   -114.13843   -217.57006    -55.79145    235.12415    212.63097     81.52061     29.85459    354.85313    222.58262      72.5162    194.63799    222.03046    276.06529     91.53371     25.69486     68.63128     72.96061    107.56918     86.34984     226.2449    -97.56003      -22.117     29.81635    -92.51768     23.06962     54.87709    -46.05274   -132.18675     54.81364   -233.39957    -44.45837     30.70848     71.36339     -8.19362    -67.93388   -157.25402    -246.9978   -113.18852    -68.86826   -163.52347   -209.13536   -183.43031   -161.48761   -157.11845    -56.76186   -242.69092    -69.58437    -29.61151   -116.29631    -234.9768    -145.8529   -296.37883   -144.07349   -160.51075   -363.04386    -22.21779   -391.24303   -196.01962   -147.36724   -228.14486    -47.53168   -181.33693   -159.11819   -316.18225   -209.82689   -316.17054   -179.54745   -287.52862   -194.37537   -425.14426   -258.48273   -384.01709    -89.89764   -246.31996   -374.59933     -722.198   -239.11495   -190.05964   -497.41126   -480.88071   -521.13455   -440.95196   -700.14132    -656.0857   -294.73293   -465.30117   -356.03279    -875.2955   -682.44278   -345.44506   -807.23559   -753.30818     -613.317  -1128.86041   -884.77882   -642.06558  -1104.00883   -868.65276   -986.37959  -1015.77581    -932.4954   -950.69897   -682.07886   -764.91899   -917.29428   -791.67277  -1194.91092   -1021.4238   -940.11433  -1057.65477  -1047.48775   -720.14197   -754.02216   -956.55767  -1159.95311  -1103.05068   -818.82159  -1108.13699  -1079.03751   -915.47056  -1073.85113   -670.53656    -937.5237    -961.4202   -781.00805   -432.80422    -491.2514   -572.86781  -1032.97349   -581.18677    -959.4279   -880.42747  -1234.86596  -1139.36862   -199.86239  -1046.81393   -998.73103   -530.37141   -310.81035   -713.50005          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       13.00   -592.37277   -368.84435   -322.53392   -355.08895   -288.81873    -179.3763   -558.00522   -450.15885   -475.63531   -133.26378   -234.51989   -171.95418   -407.25765   -204.72454   -161.27414   -367.84366   -149.47821   -389.83556   -201.01533    -241.8971   -429.03994   -122.24657   -457.34278    -397.6847    170.67936   1168.67525   3419.42099   4092.28957   3266.87002   1467.70024    235.84535   -133.67639   -263.96954   -382.26316      -203.71   -261.53779   -313.08672   -216.53343   -316.27951   -325.11675   -272.72569   -418.34195   -277.96034   -123.38249   -403.36945   -415.48971   -394.14593   -405.97277   -420.68759   -168.69839   -368.98662   -184.99551    -246.8965   -275.14827   -428.51349   -314.06795   -299.83411   -156.36943   -231.54823   -155.68488   -145.58985   -372.60782   -157.67624   -311.28859   -342.90877   -323.32023   -285.49072   -252.49193   -142.15131   -166.74621    957.57785    1886.1229   2114.43233   2062.57908    975.51016    405.73204     97.56072   -156.05013   -322.79819    -583.2118   -391.75984   -306.21523   -324.48283   -347.86912   -189.47051   -163.50635   -215.77026    -259.7574     56.01233    720.91811   1998.09997   3107.50064   3227.21589   2165.38134   1393.15527    385.65695   -146.04331   -309.30636   -284.08706   -257.14602    -64.39793    289.94678    272.54123    176.52148     45.62586      0.94862   -195.68258   -200.56835   -255.22375   -312.02477   -421.96069   -416.79031    -265.1264   -324.63303    -340.3191   -296.49413   -261.70415   -240.18707   -208.09992   -248.87997      -270.78   -331.84879   -226.44537     62.56297    453.64171    688.87453    951.39583    917.30511    403.87648    465.12907     15.98802    -36.45729   -106.60293   -113.60704   -205.52375    -32.77533    448.98567   1127.13562   2093.42707   2682.27125   2707.85716   2379.28111   1634.31857     969.6377    376.42661    241.16979    233.06368    170.88303    112.25453     87.49152     -5.13304   -128.86171   -335.30799     -352.501   -368.70867   -320.50982   -294.25487   -295.64212   -260.31475   -168.54474   -225.46162   -241.80752   -230.23708   -132.82162    -257.5894     17.99867    -10.21411     87.81143      66.4272   -129.85806   -108.83103    -176.8759   -198.15035   -157.70798      2.25973    -89.97955    146.33248    239.46097    186.35602      8.31923     42.83035   -194.26159     19.31674    -155.4213    -51.02239   -218.37201   -178.33572   -315.05984   -430.16122     24.79138   -267.48809   -249.67857   -324.24474   -219.99211   -148.62852     21.60305    114.03163    -66.93723     213.0164    109.13412    240.43207     30.59376     223.1515     -4.23044    -172.4799     94.53402     88.49397      94.9969    199.17989    134.74755    -31.91687    203.70024     21.01402     -9.90698    -48.56573   -206.40666   -352.86996   -223.68937   -334.75769   -432.20433   -554.69831   -234.99877   -216.50184   -264.68578   -242.51495   -142.74297   -144.38064     12.04013    140.35292     85.40366     19.54039    103.56268     44.75519    136.87189    208.37456    -95.59239     66.96649    299.51405    352.77145     460.8471    212.56423    251.46319    152.70599    153.11712    -35.37674    -52.75522    -71.98854      36.5258     46.74863    -302.3612    -46.48224   -366.26098    -44.92423    -253.8503   -135.28793    -48.71267    -14.81042     75.07352    208.39604      39.6581    405.19316    397.48535    440.99038    655.66187     627.7206    795.45636    806.24592    610.80614    748.27681    555.99417    409.06181    238.50444    140.88586    -15.53776   -183.71098       2.4802     13.26522   -247.28419    172.48881    -41.46584     46.22773     23.03205     160.4314     98.09258    196.12887     28.81175     43.48913    -166.1399    -36.31122     -96.6209   -107.24671   -254.19876    -228.2234   -263.09047   -419.59594   -367.12261   -437.82703   -286.59487   -222.45811   -326.58434   -189.00288   -335.40436   -390.92146   -236.63859   -149.43053   -232.82017   -115.76871   -230.99096    -82.59291    -77.77246    -23.86652    -26.14806     46.58101    141.40378     -8.58689     37.46538      -68.077    120.59474     63.34943   -129.13132      18.7347    -43.03346    -94.90121    -312.5817   -117.72122    -75.52248   -185.80305    -86.79248   -264.99043   -152.95721    -144.0219    -40.57364   -114.54241    -176.9443    -94.49929     19.73139     67.76944     93.25926    136.88087     81.50071    -18.74633      1.31396   -196.04273    -77.20983   -201.42499   -176.90617   -277.28588   -118.73844   -344.57402    -35.83478    -86.10503    -40.64225    -96.68097    -69.93607     57.00154    143.84267      126.325    192.35882    206.80032    186.73209    427.17065     21.76727    213.60904    127.81665    297.46659    161.37873    -20.05112    157.99278     105.7156    -19.98132    108.18846      2.29965    103.86985   -110.07736       1.9992    -92.04818     20.23851    149.31972    -93.91172   -163.64273      -36.844   -203.04879   -230.32977    -30.71637   -177.22886   -261.11386    -73.46134    -104.0268   -446.91444   -192.30208   -102.56759   -291.19147   -410.32462   -266.97628   -354.32125     23.78105     43.38748   -239.21038    -274.3725   -134.14717   -281.94789    -271.5023   -263.87597   -267.34145   -193.99906   -298.42478   -288.57342   -268.53908    -283.1945    -371.6818   -395.89345   -442.23869   -340.72368   -289.73131   -317.82727     15.11675   -250.84889   -174.29794   -331.21162   -129.99647   -135.10772   -369.62361      0.27326   -112.85095   -379.83418   -103.80916   -442.60454   -494.55462   -249.23308   -448.87171   -444.95169   -648.90876   -481.72503   -738.43256   -629.83327   -676.90844   -757.76178   -702.23134  -1042.21851   -569.24909   -845.92598   -631.79338   -744.96366   -799.72284   -463.34189   -961.68936    -782.0979   -873.94276   -930.09571  -1009.39108  -1004.67431   -840.03087   -903.55444  -1031.35886   -936.47384   -902.68591   -882.36246   -916.83318   -778.07824  -1142.86444   -804.94344  -1061.33612  -1079.15408  -1171.87829   -850.58162   -650.94547  -1098.05213   -503.54482    -661.7204   -827.04327   -830.95338   -820.90912  -1158.10475  -1127.89652   -956.64563  -1157.56605    -894.9583   -900.32879  -1195.97212   -960.03163   -122.98708   -883.33316   -884.70054   -585.82104   -653.15304   -813.39815    -10.10221   -257.22553   -614.99733          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       13.50   -631.09211    -469.4643   -310.46944   -251.84843   -273.85369   -302.72211   -467.69667    -527.2713   -414.17091   -293.34472   -137.90262   -198.18947   -444.96222   -291.19652   -256.66639    -331.8684    -77.94501    -259.3153   -480.97557   -252.20947   -324.34309   -174.08169   -237.31162   -281.44627    236.33944   1224.16525   3438.53596   4197.44242   3524.77267   1199.21935    149.95074   -148.63579   -308.79823    -337.3192   -297.55676   -274.33984     -316.339   -205.86966    -378.2816   -379.29929   -360.06573   -443.01847   -237.16425   -264.85925   -499.76945   -449.87728   -378.81301   -307.55168   -228.57662   -480.79167   -414.00682    -65.42017   -269.35437   -353.39073   -397.99862   -302.62081   -190.14042   -106.92403   -112.41867   -396.64512   -198.96985   -347.59922   -377.41745   -447.07139   -184.02181   -171.43929   -366.49141    -301.5857    -71.07061    -44.12014    666.14269   1795.59535   2356.57864   1964.79983   1052.90024    454.15315     29.01722   -115.10388   -308.31408   -375.35665   -146.62181   -156.81051   -144.58037   -175.89994   -146.54483   -221.84179   -206.50594   -173.06513    -52.05361    846.86876   1813.55405   3097.54818   3117.61505   2168.47846   1148.69501    191.15687    -77.31645   -156.25993   -148.17756    -75.86593      77.2774    296.75859    420.89263    227.77349     39.65336    -48.11229   -128.02483   -286.11899   -300.88789   -315.60571   -333.23799   -389.84153   -343.12528   -358.56446   -129.20678   -268.34388   -313.09725   -267.12804   -122.56013   -426.15859   -210.06379   -403.31515     10.00952     81.22909    399.55751    888.51967   1015.33992    743.30637     565.7672    138.51029     39.85049    -56.21744   -187.30898   -202.20422    -53.19996    -70.17457     603.6434   1256.34878   2149.61734   2709.96862   3009.50801   2469.94817   1617.99936    947.20101    452.78759    246.98892    152.64885    158.42532    173.10769     48.39672     25.52062   -138.58973   -326.66156   -341.50852   -274.45396   -298.87706   -353.73302   -306.13997   -261.98402   -419.52849   -194.17472   -182.22181   -249.42586   -108.30139     -50.0884    -56.98963    175.49712    -46.43303    -84.76526   -290.37333   -173.14693   -282.11907   -254.65712   -169.70789    -15.26987    -87.77295     96.13358    237.45488    245.22036      175.381    -47.56048      53.3367    -94.42221     39.10889   -223.70917   -135.32083   -229.22198   -222.43296   -370.71032   -284.54961   -223.60272   -341.05329   -398.48551    -87.29379   -350.26563   -127.25028   -265.55154      15.8555    174.09846      76.6796    436.91035    120.23886     45.43899   -107.27093     56.85491     40.32897       6.8394    -19.89518     93.53649    280.46154    223.21639    154.58562   -119.57532    -24.95826   -161.65469    -220.4884    -257.1106   -255.93063    -280.5527   -144.86066   -255.85671   -436.33003    -292.6927   -147.01814   -188.69869   -180.74463    -19.32329     114.8281    -70.48708     45.36447    -57.58771    108.53657    220.52704    185.50104    241.55067    167.84559    286.41793    390.70897    335.88383    303.92719    300.59233    261.14255    -105.2398     11.12076    118.76946   -135.92173    -227.6679    -94.00001   -174.89167   -118.25423   -328.83696   -259.42329    -177.8411     -35.8972   -121.74687      5.88091     80.33453    131.04769    119.11848     54.18328    164.69221    265.83348    411.59522    633.98705    638.18457    703.67826    644.93676    619.31695    827.36964    653.22463    402.52446    356.44978     -49.1774     81.38598    -65.08385   -126.59428    -149.4733   -161.22471   -195.44882    -43.87517     -53.4779     32.64934    290.65434    200.86036     51.68196    146.43234      7.92086    -99.24098   -146.86869   -220.64357   -198.83674    -179.5278   -155.48808   -416.98961   -296.73212   -235.72094   -311.44444   -196.86772   -268.56854      -429.47   -335.83592    -303.7844   -401.94775   -115.20756   -101.26115      10.3745   -283.91714   -320.34916   -275.56645   -149.03225   -179.99794     13.07445     -1.61046     95.43294    156.65103     92.54608    -12.10655     -0.05669     41.15848     67.81348      1.03903     -58.3537    -65.96192   -119.47341   -414.96395   -132.55345   -257.13536    -71.84016   -111.01466   -280.83556     66.17936   -155.97486   -139.70015      5.86887    -37.06636    -22.35301     37.57651      -150.12    -27.17436   -218.74192     12.00133   -177.53537   -183.52672      2.15531     54.43712      3.36915   -223.24749   -138.38117   -115.92309   -168.37299   -220.65401   -209.67557   -113.55461    130.97478    -12.49819    226.61673     13.17482    123.40961    156.11821    108.73909     51.88614     318.9582    131.00195    169.07439     43.08964    157.46392    135.49414    -73.80792     96.02105     89.80586    127.62058    101.91489    -55.83579     45.29768    -95.05579     10.42768    -45.86246    109.31854    -80.26613     30.05566    156.70633    186.68198   -156.26905   -251.00964    -78.26807   -117.60829     -93.3323   -290.29465   -189.44382   -412.78554   -121.23819   -106.86256   -396.05636   -330.39636   -262.25221   -111.11928   -484.02352   -195.48804   -265.80604   -160.58022   -184.16343   -186.59293   -265.00334   -183.25171   -301.77577   -216.82002   -159.49883   -233.62573   -117.36495   -230.40013   -227.34199   -197.24122   -242.63627   -488.91527   -227.68867   -419.19256    -89.03746   -205.60774   -426.18596   -272.58849     55.27627   -372.31528    -178.8683   -465.54584   -673.78106   -520.81563    -301.9875   -212.88957   -248.91145   -338.38955   -620.91731   -559.36575   -402.32386   -708.37901   -727.05485   -682.82293   -615.87297   -726.86447    -816.7886   -601.94015   -651.03966   -800.98013   -836.00894  -1162.60992   -476.99802   -931.73468   -928.84445   -747.81826   -863.53252   -966.57403   -849.81004  -1010.62702   -903.43867   -955.52338    -841.5761   -982.44193   -869.81619   -765.32511   -982.53325   -805.38045   -908.86054   -994.09902   -903.49106   -994.27801   -888.47014   -443.47009   -1083.5101   -698.12328  -1180.71807  -1315.59218  -1024.83824   -902.76772  -1034.32588  -1436.94335   -600.31689   -613.34993   -719.11917   -762.56314  -1349.51512  -1427.05794   -749.78675  -1537.30445   -817.41232  -1060.97742    -885.2155  -1512.77321   -1059.5281  -1300.58454   -606.35499          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       14.00   -685.72887   -475.86577   -290.11475   -314.04642   -245.53953   -421.64155   -322.42474   -506.71941   -351.54922   -393.58397    -95.53387   -193.58301   -377.51752   -331.62154   -176.29877   -333.22242   -304.98357    -54.73219   -550.27441    -307.4644    -296.2224   -469.25704   -166.08341    -83.82674    163.60539   1237.58741   3465.84499   4214.41833   3505.87498   1419.17916     99.90518   -153.70949   -204.35939   -238.67832   -277.63855   -203.49392   -256.24305   -347.90081   -287.51526   -353.14815   -261.60884   -463.06477   -406.55694   -274.55682     -412.938   -376.82082   -214.58286   -347.28751   -330.61494   -376.03825   -496.56274   -224.50739   -224.45995   -311.64586   -383.29174   -142.90129   -138.46096   -231.34231    -27.87846   -216.58881   -107.37873   -394.43172   -378.08418   -241.85254   -269.60675   -259.48124   -324.36272   -161.07862   -149.43947     -6.16941    856.78466   1712.21214   2346.55368   1877.54237   1053.23516    349.38201    126.78332   -162.88534    -347.2982   -457.15763   -232.13403   -399.58652   -192.97954   -225.51316   -203.19972   -151.74091   -183.20018   -119.50652     22.48201    567.96953   1891.10767   3065.13848   3177.28102   2191.15159   1236.07788    358.49305     13.19724   -159.57051    -348.2476    -51.71283      9.71963    241.04926    238.22668    190.28342    -68.20512   -119.00698   -299.07166   -338.25523    -303.3217   -191.54614   -381.67048   -290.38395   -288.28768   -306.69815   -301.31842   -133.96607   -240.92215   -194.44453    -110.8798   -127.15434   -232.05601   -200.92149   -150.72374       115.32    465.10508    659.57738     778.4407    761.81441    448.96827    289.59745    104.18833    -11.84841    -72.96711    -82.04063   -186.10441     -9.78155    515.34097   1126.56764   1873.31105   2554.53325   2701.38267   2275.52632     1564.217    978.21727    412.20636    333.02238     96.17288     291.6788    222.82198     101.7515    -27.74054   -320.22406   -122.35916   -287.38503   -370.48452   -227.14182   -346.53835   -288.18054   -396.15943   -223.68611   -261.22702   -435.97427    -210.1742   -210.42033    -71.69247   -104.24243     96.18125    117.19912    -30.84676     74.61734   -101.66498   -178.77278   -319.67819    -86.03658    -169.8212     80.87514     56.82813     99.96711    132.49529    317.11526    139.90215    -28.81997    181.13734    -81.07836   -195.08589   -132.08006    -70.14993   -164.43739   -200.16868   -167.03435   -156.60632   -265.06738   -152.80005   -181.83064   -127.53435     -3.33656     -9.59541     121.6974    137.30727    403.91581    270.07338    209.26498      -0.9812     52.73043   -141.26301      6.85927    -80.28398     51.48868     -2.49628    149.69394     215.6372     122.5157      3.78101    206.84618   -139.09753   -145.69377    -158.5045   -269.30145   -194.83914   -245.20039   -354.42169   -217.66772   -140.62997   -332.51696   -122.95453    -40.03135    -34.71536      16.7267     -5.97635    125.08234    198.96526     68.55937     138.6948    189.14715    105.11169    118.79896     73.32627    437.52356    276.47263    230.15486    151.91783    250.51958    158.10542    -56.49098   -190.75559    -19.69613   -301.04476   -143.37896   -251.77038    -264.8167   -180.94978   -172.04077   -330.79687    -25.60283    -84.45132   -195.85175     17.95117    212.47621    130.17698    197.61452     255.4169    410.41444    485.92639    500.65933    742.75858    687.36841    797.37036    751.76853    660.94349    449.53076    388.19131    222.50143    134.29309     -7.88465     11.70985     -8.28258   -162.32055   -213.70748     -5.47719    -60.56028      0.52351    -27.50657     42.12529     81.29146   -112.90758    164.37053    -19.40321    -10.19684   -221.27939   -110.02847    -23.12355   -231.81321   -278.19249   -147.81483    -134.0264   -255.44686   -373.08397   -287.75356   -165.82114   -237.42287   -117.31678   -400.00676   -277.87825   -254.95638   -198.16145   -197.59596   -257.00397    -86.40735    -186.2163    -16.38421    -65.29987   -205.52352    -28.89071    -50.32222     84.05198    258.68402     75.95163     57.69414    -26.37083    -70.05806   -128.86141   -230.22633    141.10177     -75.5614    -76.88983   -194.80871    -72.47006   -115.84416   -316.52725   -134.65406    -180.0431    -78.35909   -101.57833   -107.24606    -24.57988    -22.86956    124.39438    -74.43464   -172.72972    -94.58029     42.17863      18.0678    -237.4166      -67.624   -111.22331   -194.62212   -157.42616     69.88079   -119.12217    -27.15035    -132.1455    -158.3706   -213.65727    102.42479    178.61401    113.70683    131.17802     57.72251     71.57668    269.98388    -98.58968     510.7812    -75.00774    110.20277    178.76941    156.10132     25.67789    109.24157      71.5835    131.58626    -88.67753   -267.18628    -48.52348      7.80749     57.85286     70.76493    -59.81738   -131.79974    -16.29402   -221.58722    -79.65462   -206.71704   -158.38907   -177.84809     -71.9588   -174.97933    -49.80274   -319.39287      38.7295    119.29392   -131.88606   -163.66591   -228.11704   -451.70429    -95.32534   -295.48164    -215.4152    -272.0074   -409.17897   -333.30429   -223.78942   -226.70773   -149.73089   -157.80258   -246.91539   -213.29582   -241.41236    -87.60677   -198.72155   -119.03925   -407.65637   -367.90996   -153.54722   -167.11633   -126.55733   -303.96324   -241.13169      9.36564   -379.86779   -195.88231   -321.73605   -443.24948   -277.96403   -372.63823   -221.97901   -497.84008   -346.04956      56.6329   -218.06568   -439.45385   -473.97352    -372.7971   -697.54747   -640.79441   -621.07197   -578.26985   -529.96758   -506.61492    -614.1518   -819.14265   -591.04596   -534.56904   -947.02317   -774.74523   -792.32312   -560.90253   -917.27251    -853.0206   -674.27084   -907.94558   -962.34118    -793.7607   -960.46062   -912.92453  -1057.58024  -1006.18086  -1072.55496  -1200.16281  -1179.26893  -1054.97874   -951.32348  -1153.55039   -907.27671   -985.15555   -690.05868   -720.42584   -954.83348  -1195.47647   -673.71189  -1123.00809  -1004.27793   -749.94003   -499.24709  -1046.08086   -571.11151  -1208.73335  -1157.62813   -600.13856   -951.69406   -672.97258   -867.37058  -1222.66427  -1441.75617   -680.32742   -576.08439   -876.99254  -1080.41102     43.85673   -784.50428          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       14.50   -656.43218   -452.05344   -279.36036   -386.64209   -285.90297   -530.08164   -270.47806   -420.83424   -352.22059   -394.29164   -167.72323   -147.22652   -232.56552   -433.97921   -105.45853   -400.15053   -411.90555   -318.24573   -456.24034   -309.63842   -273.87495   -324.66778    -178.9266    -49.66682     70.26093   1364.31197    3538.2952   4449.27729   3175.90194   1391.94268    143.73726   -188.28047   -144.73932   -374.65371   -223.03953   -300.11783   -429.42047   -234.67493   -491.24742   -307.44808   -507.69386   -357.19584   -392.91023   -411.56405   -398.12916   -266.43649   -441.28602   -529.07029   -259.24621   -239.25623   -170.56574   -353.90807   -351.51861   -297.76939   -348.28241   -345.56704   -246.96227    -90.25441    -237.1578   -245.57119   -227.56759   -202.09484   -467.98914   -370.39181   -490.24512   -225.88321   -208.95576   -437.58554   -220.48143    104.77119    848.15793   1832.57294   2332.95897   2101.37979   1046.64521    549.78593     49.10076       4.8151   -241.59562   -178.46268   -149.83752   -340.13153   -263.42448    -197.1821   -314.74628   -314.87164   -343.04533    -80.53431      3.58903     889.8742   1840.82212   2852.43205   3073.56339   2285.22474   1132.15478    321.33188     79.43405   -171.11572   -275.49774   -126.60329    -68.27266    195.84902    307.94757    161.04541    -90.12677    -93.89915   -214.62939    -73.19363    -193.3061   -397.06736   -532.62212   -449.03944    -67.38915   -492.28095   -263.48827    -303.0297   -263.70034   -111.82934   -150.61638   -235.21995   -366.23625   -225.05276    -178.6656    -40.91468    428.66131    747.58034    761.05875    853.85046    552.66996     48.56584    -46.47213    -22.56045   -182.68935   -104.40028   -130.46909    183.59723    432.94306   1187.10902   1811.75667   2780.89941   2697.45041   2138.38531   1665.73234      819.862    460.59896    259.61387     52.87092    203.42368    -57.36874    105.46611      6.70484   -162.75416   -243.50088   -199.32596   -347.69021   -297.21238   -246.96594    -249.3487   -171.99356   -330.63201   -214.62764   -104.82942    -132.2861   -215.19404     -87.6368     12.69962    -26.89886    -42.89114     -46.9868    -40.55317   -419.99966   -190.54273   -197.87869   -165.12679    -94.16754      6.57194    116.22593    227.10918    264.96207     71.76999     52.35725     38.26934   -103.53481   -159.67069     -1.54898   -274.89234    -94.25858   -416.88756   -329.00551   -391.53686   -113.20442   -362.88018   -202.51651   -279.15603   -300.47833   -172.80097      -52.069    232.90835     64.52643     258.1737    113.45466    220.51759    126.52203      5.92369     -5.77816    -77.03402    151.79887      6.72187      9.88206     97.90492     92.63908    207.23943    -220.7603    -23.72881   -183.57035   -142.28307   -343.67393   -489.83697    -322.2633   -396.26473   -314.20273   -387.03453   -167.32013   -319.19274   -213.98563   -162.34224   -167.41964    -36.36616     57.89197    146.14039    213.47441     57.15217   -110.40046    178.95079     34.48588     146.8882    254.82014    279.60773    460.25411    293.79449    251.05187    143.36481     93.57427      54.7397   -183.56836    -50.78547   -118.98791   -291.69722   -254.46683   -283.85771    -90.54442   -195.55597    -139.1825    -60.46845    -27.23265    -71.03944    -37.23207      75.9203    120.94467    155.67631    204.84711    237.33965    437.59768    649.53443    564.03497    563.69637    687.17603    809.30103    445.18057    306.77375    340.20932     29.90846    134.22071    -21.29636     34.75675    -13.09504     25.11427   -162.05227   -144.98961     33.29183      36.6239    100.53916    111.62795    281.43251    -45.55656    200.25455     42.21334   -217.91667   -107.02972   -179.35471   -211.80876   -177.76645   -212.61762   -222.43221   -406.53119   -453.37601   -297.62107   -112.34549   -286.13718    -243.5615   -281.95817   -130.34173   -268.12007    -142.6518   -394.25289   -172.41731   -228.59897   -251.08034   -230.51303     53.09041     -60.1758   -157.31255    -64.78213      14.5056     34.79425     -82.6735     -44.6455    -32.54917    -92.81035    -34.79838   -156.54027   -197.74641   -153.76814    -55.53843   -209.00358   -148.58422   -179.32484     -74.2702   -176.28248    -61.39243    -42.07409    -74.34343    -80.97201   -172.08291   -123.98329    -49.80844    -67.18707     75.21612    -72.72309     85.23065   -123.20519   -107.77301    -75.00539    -18.62252   -137.74111   -116.45286    -71.82786   -405.66727   -152.61882      -195.78   -106.75172      6.10673     -2.68623    -48.01943      232.799      0.88684     67.29548    171.72844     32.60862    280.36462     46.20391    167.87825    227.58797     84.76426      231.506     15.38624    155.09469    119.11655    180.13041   -129.34351    -20.81558   -153.49575     85.01314     53.62461   -210.74938   -193.15689   -109.03691     62.06713   -164.51575    -20.40552    -62.49156      2.40986    -83.92371    -41.92755   -167.63623    -31.69285   -122.90954   -218.37325   -180.43864   -365.32686   -323.68244   -282.08339   -297.03201   -485.98708   -291.14583   -115.51449   -120.66482   -329.23317   -603.80659   -140.97402   -248.76521   -359.62914   -174.93434   -208.83282   -200.11367   -326.83951   -266.42314   -266.13631   -158.77025   -263.83217   -461.02167    -66.52901   -417.43765   -302.79757   -518.71367    -310.2745   -116.42775   -327.22309    -212.3677   -326.83129   -249.63625   -246.31995   -444.21962    -11.15686   -320.50765   -538.34155    -540.6333   -520.63129   -648.01336   -558.21274   -546.37677     -824.465   -574.49873   -414.98563   -781.49455     -679.889   -604.89997   -641.63211   -328.80884    -889.9926   -687.47951   -419.59189   -810.27495   -664.43783   -894.65391   -1100.7532   -709.29206   -947.79988   -855.78903   -665.10859   -986.14334   -713.18715  -1319.45002   -999.44871  -1137.75695   -988.09776    -719.2501  -1101.27645  -1270.28976   -864.29804   -810.25951   -721.44768  -1207.27078  -1226.97519  -1111.64683  -1023.39019    -911.9872   -1323.0422  -1213.09534  -1027.83334  -1084.71113   -853.03389   -1009.7762  -1016.80143  -1255.44015  -1353.00677   -804.49638   -951.33037    -954.8703   -925.55888  -1155.75343  -1002.53026  -1016.43486     25.69465   -878.84709   -682.13246   -931.30988    314.82118    -458.1108          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       15.00   -524.15408   -383.29575   -268.11541   -429.65959   -408.36674   -572.07665   -338.24023   -377.02281   -280.74311   -312.14383   -253.25299   -161.56531    -95.04256   -524.79751    -261.5735   -441.02474   -210.58711   -355.93161   -271.36806   -321.61777   -314.52118   -235.75636    -55.31168    -92.40958     39.10662   1325.04349   3311.13915   4157.23275   2911.71168   1480.07387    247.18756   -149.26244   -324.50486   -202.62332   -436.19609   -339.34449    -184.0178   -261.68635   -329.48716   -252.16417   -384.49657   -267.97331   -404.82613    -429.3083    -95.51595    -224.3852    -516.5481   -343.19917   -200.52477    -341.1218   -312.11826   -386.47889   -280.61227   -366.27915    -330.5935   -354.63897   -242.28289   -185.50083      7.09011    -84.19032   -387.46117   -403.72829   -329.87626   -428.48593   -235.62896   -185.97067   -363.77392   -341.91494   -253.71832      -2.8612    897.01933   1868.67232   2416.53705    1926.9176   1081.06318    447.76667    172.39629     73.76828   -621.33119   -186.30457   -619.20583   -520.27477   -289.15098   -223.14362   -159.15324    -89.29589   -258.07516    -252.2468    264.33743    752.19494   1882.67499   2838.81913   3050.81647   2193.47288   1165.78776      371.083   -110.15949     -0.37179    -166.9918   -211.19038   -158.31463    217.65722    468.82384     87.96219     -52.2272   -239.92829   -185.81814   -255.43951   -334.23163   -382.75562   -483.88645   -440.40663   -295.42914   -320.28654    -398.1374    -161.6824     -249.444   -277.71383   -173.94538   -111.24371   -227.04081    -82.98393   -188.33048    360.13204    407.50179    743.41467    778.76485    944.15867    447.83002    271.39854   -211.69367   -225.51186    -69.12976    -82.02683   -138.26394     98.62214    508.04341   1227.47488   2007.15919   2760.91692   2665.17303   2202.31851   1569.82772    808.87368    425.61329     139.9273     49.99805    230.45278    261.17797     22.01061     97.63991   -238.88546   -356.72835   -440.83455   -265.62052   -305.25133   -270.45324   -357.81234   -226.62942   -124.22232    -77.29925   -215.30913   -292.65623    -93.47058   -140.09142    116.86785     81.80073     39.04281    -15.31699   -148.28416   -175.18743   -237.17094   -227.65627   -272.35946    -71.87343   -134.05419    346.72876    127.79885    305.09817    166.62148     97.73386     -79.7363   -160.15937    -39.25222   -195.08792    -141.0279   -261.04543   -237.40464   -293.08337    -338.1565   -328.22039   -216.24956   -260.69275   -224.79009     -45.0575    -70.04147    -55.69089       74.125    165.55494    146.72521    312.41531    256.31793    171.31651     10.26027    -15.82404      26.8766    -16.63666     93.56516     36.12508     42.84686      82.8435     -56.5541   -119.19767    -38.77773   -212.59921    -444.0348   -252.41257   -285.59468   -542.69707   -338.64847   -378.70872   -301.13874   -280.33771   -310.82653   -219.40224   -121.30173     16.45992    133.93147    111.80006     91.27656     23.51207    -44.91067    319.80595     40.88825    141.33687    340.69881    361.88438    218.99977    367.81498    262.45233    272.10591     366.8902     277.6705      7.62748    -72.21925   -294.27942   -122.46602   -229.22789    -19.80636   -278.72963   -178.16869      3.95639    -76.82551   -149.06902   -218.83478    -21.35696     10.75702    175.47448    119.06807    196.57859     38.73329    501.99538     580.0585    523.39651    822.27843    683.69722    575.49799    820.37181    558.70617     342.6859    335.87932     316.6885    291.07905    -38.25119   -116.93272     -5.40713    -17.37626    -54.68387   -177.61555    -12.57669    -17.07548     51.44823    137.73712    164.84456    109.04051    137.44729    114.81524    -22.39813   -167.42635   -159.72086   -226.45397     45.96466   -225.11287   -305.39298   -224.36183   -229.92997   -264.32124   -357.09749   -311.60016   -277.53943   -217.80641   -158.40683   -176.26548   -168.74943   -238.15631    -44.26401   -162.89225     -94.6293   -134.16163   -219.10146    -24.56801   -205.76095   -101.12131   -106.83541     16.53759      3.01455     -201.294    -39.01194    -59.26912    -25.88333    -18.48509    -99.27179   -134.38105   -185.44018   -152.18673    -209.8261     60.34149   -237.57827   -182.36068   -180.87749   -119.85773     -0.78081    -22.77302     -50.7234   -171.47329      5.16733   -188.51054    -59.12539     -27.8759    -69.51181     12.39597   -285.87984   -254.04261   -155.83664   -118.63317   -188.91861   -177.65196   -239.86253    -130.7452     32.36456     55.27551    -29.20675   -148.61822    -66.97457     94.02389     53.42932    148.87071    157.11605    144.87074     97.05323     133.7547    198.29346    154.07961    236.75367    153.60377     13.05459     109.0941      1.68583     17.94396     48.46534     56.45317    -70.61481    152.27641    184.45895   -203.26978     -6.45466    -35.66034   -130.12686     22.23007   -161.95277    -45.11039   -334.83938    -12.97763   -304.76267   -134.15487   -304.66425     -77.8496   -138.40094   -233.70052   -232.65301   -175.82976   -228.98428   -455.13517   -254.28905   -446.09364   -360.78746   -467.65826   -431.22355   -326.93586   -202.93958    -54.61779   -155.35854   -197.19477   -398.12196   -166.65131   -253.11772   -125.51457   -126.89578    -31.30637   -271.43939   -396.91438   -609.99622   -474.16374   -467.00432    -42.65292   -342.13713    -181.6439   -306.10654    -192.5015    -208.8681   -178.92918   -418.70769   -308.41563   -620.53292   -631.42992   -575.68449   -566.69387    -65.66261    -621.4586   -565.76362   -540.78257   -768.39792   -430.68728   -453.99888   -571.75729   -643.17437   -792.57084   -543.00145   -961.78158   -544.53305  -1019.54989   -642.06368  -1176.02181   -725.98464   -706.73341   -973.06062   -999.05877  -1185.08358  -1000.62355   -957.96407  -1354.83733   -918.22692   -825.95769   -571.73943   -953.52953  -1121.24425   -901.17382   -998.95659   -872.69401   -873.38572  -1132.48937  -1056.23068  -1072.45228   -826.17643  -1131.64051   -963.39868  -1042.10698  -1717.92398   -824.31709   -690.08062   -829.64071   -567.70298  -1245.73935   -843.82639   -916.58987   -985.22207  -1057.69554  -1078.76484   -891.04802  -1070.05156   -743.85797   -776.81791   -864.56245   -1578.9725    -348.5211   -205.21705   -588.66978   -365.87049   -680.31335          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       15.50   -379.78204   -363.04977   -249.53056    -455.1023   -537.21515   -505.29294    -391.5731   -447.51255   -184.04545   -132.15406   -293.36463    -296.4266   -144.71708   -606.26464   -299.71996   -416.97297   -291.03684   -349.24822   -186.06393   -294.04998   -250.28765   -134.71088   -284.76765   -205.78598    144.57922   1236.92119   3327.98115   4065.35538   3016.62595   1436.55344    237.15473   -229.09952    -61.70944   -274.26596   -174.96315   -323.93693   -393.35851   -230.73592   -192.37924   -184.95172   -218.77955    -439.6357    -404.0436   -352.90258   -339.06217   -260.17112   -421.67521   -318.22091   -371.65615   -376.39072    -384.0806   -438.28482   -425.77904   -308.32001   -320.27064    -216.1499    -268.8184   -109.16806   -142.71469   -227.52275   -263.14657   -336.01169    -406.4903   -227.76309   -326.03108   -339.36099   -233.22132   -418.10214   -141.79445     45.70565    662.06287   1702.21122   2317.02357   1991.81362    1146.1396     469.3839    285.29596    -80.51859   -273.74609   -475.67908   -395.97985   -364.64032   -231.87806   -203.70191   -372.84196   -271.17803   -291.17021     -86.5569    196.84956    685.27165   1971.49838   2886.94151   3031.52988    2289.6207   1105.89851    280.69442     70.02324    -99.25533    -13.31079   -126.58313   -103.75598    189.36957    251.72946    190.87168    161.76674   -120.50285   -321.28224   -290.20522   -340.87153   -224.11969   -277.47142   -406.00105   -337.51459   -388.38952   -376.77021    -326.0507     -92.5885   -169.25604   -230.15843   -176.58514   -281.92847   -226.19432   -224.59838     38.84761    455.43105    848.08598    927.97802    701.45995    531.69305    252.58669     19.69884    -18.29263   -143.73889   -126.84011   -127.29026     96.63691    493.27984   1184.63341   2024.86011   2916.07296   2835.34468   2455.68342   1577.79194    738.29457     371.9194    111.14557     130.6971    123.61612     78.11277   -112.77354     10.15101   -134.95857   -165.36161   -196.35014   -385.87702   -196.73752   -279.39973    -209.2482   -271.82171   -285.03639   -134.42301    -224.5305   -267.24524   -177.19242   -127.68729     -74.8408     80.74336    -130.1409    -79.95124   -190.58229   -115.46938   -243.32374   -447.63798   -314.66859     -2.98138     -53.3645    140.00186    214.05454     -31.4569    181.59162     22.49031    -31.46132    -86.66645   -197.18145   -167.48772   -241.81997   -251.11141   -275.00691   -213.69454   -323.48746   -331.88908   -231.85422   -373.03337   -204.11292   -210.09233   -148.50123      5.91353     32.17896    104.63382    330.15785    120.28842    224.80084     63.62759    102.68171     -2.16724     70.74497    131.84426    -51.43682      158.751    294.90777      37.9285    -12.99157     -3.37227    -233.0935    -225.6985   -198.18012   -265.37122   -256.47163   -340.83885   -357.18471    -315.9468   -264.97733   -321.95448   -265.25431   -184.49719     -41.3383    -85.06657     -1.59303   -110.20673    142.32525    162.97315     90.07274     73.87933     78.23879     52.74241    252.13802    179.32699     71.36781    252.27762    190.30395    241.62234    210.45626     70.57635    -12.85485    -83.46502   -147.84163   -179.23452   -363.13688    -238.0252   -306.03182   -135.17126   -195.93503    -167.7403   -156.38637   -140.07376    209.64756     64.32369     57.10445      94.3337    305.55519    315.13106    368.42029    379.04627    504.78942    635.56295    628.99075    823.06244    618.53092    536.15085    491.19591    287.33754    271.94717    145.13933     51.92669     21.59763   -246.58592      14.7204   -171.50064     146.3554     86.51558    -43.71105    256.15822     24.58791     90.80663   -116.17478    116.25483     65.29673    -34.40336   -290.95356   -196.91449   -156.40137   -225.10804   -305.73671   -313.60211   -363.70787    -244.5675   -256.49395   -374.43907   -298.78004   -148.31254   -160.64136   -360.59181   -112.94362   -214.22641   -340.87059   -263.25418   -213.08667   -226.39716   -119.49231   -166.99509    -20.27337    -26.95294     -4.91157    -57.93603      3.40966      9.83444     98.72214    -34.18576   -153.14935     23.14051    -83.97173    -38.61982    -82.20064     -64.8427   -220.89776    -84.26252   -229.93604   -131.27391    -83.63523   -189.32204   -209.82236    -125.0909    -67.62114   -162.05973     -81.0477    -151.9856    -92.77626     -3.15639   -100.59772      2.89659     -111.474    -81.22996   -180.14167     14.58015   -183.50051    -188.3167    -272.1287    -217.1412    -85.15584     49.55259      2.20952      -8.2327   -113.69884     67.13456    -91.28952    -33.39852     -6.92167    114.99992     68.38471     75.78385     31.49876     157.1575     46.70398    290.42679    183.21117      8.77217    -40.05046     95.83284    -56.09314    -97.97274   -120.15537    108.06755   -156.54107     62.57573   -104.00602    -44.34582     -9.37486     15.64571    -99.14406    -12.34513   -175.23592   -123.83828   -249.68241   -141.59399   -331.41843    128.91927   -260.89442     -116.373    -252.9072   -105.36756   -231.63872   -219.79728   -326.96041   -313.25301   -255.02343   -226.13002    -96.47538   -288.43052   -358.68477   -394.31122   -384.84592   -410.61152   -254.27931   -180.64069   -333.89685   -431.58394   -229.91337   -440.32568   -273.41992   -242.27501   -422.88352   -307.00075   -145.44108   -297.48556   -354.63526   -581.88809    -97.37838   -276.88884   -142.69693   -191.85284   -389.25959   -632.48339    -268.7943   -670.26132   -609.99919   -293.55424   -458.48004   -920.09359   -523.02408   -493.87878   -664.65374   -650.99952   -523.00581   -520.67486    -670.5356   -788.50653   -556.02609   -676.83996   -643.16762   -544.83435   -631.66946   -622.71205   -570.50132   -1043.5191   -870.10315   -677.51767  -1033.01123  -1035.12655   -736.83495   -843.21889  -1274.16757   -766.41927  -1088.89091   -531.31077   -980.30873    -938.0344  -1050.20161   -851.25779  -1182.99559    -879.1522  -1208.75588  -1118.11462  -1147.90413   -744.26079   -1032.8207  -1062.09676  -1116.12659   -719.77226    -828.6497   -862.73307   -961.28278  -1011.86139     -1308.43  -1057.31996  -1174.13152  -1344.55615   -607.70589   -1120.3016   -599.06023   -921.04595  -1490.96085   -896.34352   -789.99736   -446.53848   -785.12678   -203.94492    -867.8074   -813.50457   -723.12202          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       16.00   -377.35964   -398.78181   -199.05001   -486.15155   -540.28664    -415.4335   -250.04826   -566.04666   -262.44695       7.5075   -440.50015   -306.47323   -298.48632   -565.50281    -141.2671   -512.63872   -484.76588   -276.85413   -120.73997   -283.63815   -201.09269   -193.60473   -287.50106   -358.08923    105.47358   1427.55688   3257.77547   4485.19316    3124.4752   1443.75451    258.29093   -223.61491   -258.29312    -272.9239   -390.16797    -422.1829   -435.61376   -350.84462   -297.36745   -224.64696   -377.49525   -509.96122   -313.88674   -439.81172   -421.25169   -432.90876   -200.27087   -404.26949    -398.3848   -419.01949   -340.77054   -366.49783   -314.91891   -256.42658   -309.18127   -266.35747   -272.20016   -169.54329   -130.86586   -266.08122   -345.44903   -307.14264   -172.21566   -254.96891    -431.9887   -257.18861   -139.02507   -463.75074   -138.14844    -86.16662    775.58492   1813.66865   2334.13028   2125.10182   1048.23045    324.86367     35.32762    -322.6483  -1032.59736  -1165.51092   -831.98258   -434.15677   -236.64555    -349.7517   -158.93355   -100.18448     -313.241   -197.36139    120.16419    818.15839   1824.01549   2863.34608    3094.6602     2094.867   1016.22793    288.08512    -45.26132   -116.39623    200.94217    -20.11435    -86.30804     96.16827    231.15116    140.73537     29.07339   -176.89524   -145.61208   -209.78363   -238.72831   -378.53701   -334.01243   -287.81045   -256.54037   -273.71409    -354.8237   -210.64517   -191.18319   -268.16833    -98.43693    -46.02408    -10.27561   -146.10918   -115.06735    -86.73365    431.72666    862.37273     802.1447     857.8639    562.77682    327.01056     -3.34671    -76.69745    -235.9828   -211.71534   -259.93261     81.68599    294.31314   1259.37811   2033.07696   2567.66061    2640.3296   2142.29912   1323.76338     795.3779     644.0877    243.28809     30.51303     83.94552     33.70712     17.97966    -83.35012   -228.30823   -164.84007   -282.65987    -319.6612   -161.09341   -280.64329   -235.78436   -355.81434   -286.27387   -170.56245   -163.55407   -189.24492   -118.63275     16.31397     32.75409    -99.37237     30.68183    -70.82608   -263.01085   -331.56863   -277.08596   -277.42497   -230.78953    -46.56635      75.8856    163.58097    282.27495    137.39161     39.40318    167.45008   -192.83203   -189.30229   -194.88435   -319.92601    -286.6801   -268.54445   -286.93976   -311.18897   -368.82574   -275.68442   -274.53091     34.96651   -280.56705   -282.45057    -75.98164     -35.8711     14.77595     96.80818    321.12831    207.25258    153.11561    -31.76394     95.89707   -119.27574    140.46157    112.00263    112.07889     63.16194     -56.6505     66.52014    166.40497     83.42005    -88.29729   -252.98636   -264.02995   -211.33062   -295.98843   -272.36144   -318.00212   -411.58114   -226.37348    -143.8247    -381.6858    -98.41674      70.9351   -155.32143   -169.22007      64.6783    175.66151     86.91888     31.90801     76.49683    219.89292     252.6794    195.41829    343.47458    268.27126    241.98653    218.09511    400.22848    279.64437    325.50745    -16.75034    -202.7261   -107.32562   -216.13241   -248.98709   -157.71926   -261.56112   -158.71656   -109.74287   -230.13651   -129.42754    -87.00154      48.4864    135.06659     -1.91539    101.39624    124.53029    289.27985    279.63971    610.77082    503.66002    726.28619    670.43717    714.85429    654.78237    476.63255    441.09321    311.23127     118.5469     23.66185     22.53703    -10.37769   -164.80652    126.16332    -27.16093     90.80238    177.93337     39.91874    131.56188     52.36462     29.90662     53.07295     57.45493     32.27038    106.87108    -82.89313   -222.20142     -242.751   -213.45952   -164.98761   -139.19597   -205.89389   -265.81021   -212.37617   -287.67437    -347.5237   -354.36366   -362.59253   -247.91422   -331.31513    -201.7119   -205.66455   -183.88849   -274.86179   -228.03281   -129.29846   -165.47773      -44.681    -80.20492    -68.34947    -96.79159    -19.40933    -44.50831    -38.14442     78.42958    -64.67927    -29.56118   -237.40862    -14.36256    -88.81651   -156.54761   -225.09426   -184.93363   -114.10797    -275.2387   -175.88086   -249.75051   -129.05891   -122.36796   -262.87098   -132.82435     -9.33558   -160.71859   -139.01649      -1.5037     15.35905    -54.77146   -121.64873   -167.05091    -178.0194   -107.06062   -227.90115    -54.26865     93.55311   -151.65623   -185.13469    -108.5194   -288.07285   -169.97342   -113.95123   -110.42703     70.83592     49.82865     63.03182    162.89858    271.05627    234.69995    149.28058     65.06443    147.30138     69.37983    153.12466     88.80422    107.23029    101.14474     51.36157   -149.38774    -89.53884   -141.65687    -44.14993   -313.70049    162.03813   -181.61255     68.66923    -97.94035    -74.24739    -39.54011    -45.68769   -170.81268   -169.45418   -114.44568    -23.82197    -89.21921   -227.01897   -302.30464   -134.43337   -263.07831   -161.83025   -264.13525   -415.66931   -362.43488   -161.57867   -378.17943   -304.05946   -300.67538   -430.72376   -320.22299   -335.69509   -365.59423   -154.99613   -300.65488    -148.6179   -341.70004   -306.11856   -497.98555   -381.43523    -75.07359   -186.54364    -214.8288   -356.10965   -362.42407     -62.2908   -710.94958   -363.45815    -580.8667   -263.23006    -270.3764   -354.20516   -311.29509   -195.82214      -360.33   -609.60227   -372.52863   -189.49395   -408.68642   -407.78116   -571.45771   -716.66673   -549.73699   -707.50183   -740.82539   -742.83996   -801.67139   -767.84441   -822.56657    -978.8852   -655.39517   -887.53566  -1146.14037     -606.093  -1357.67285   -866.95255  -1002.84305  -1031.30852  -1251.90288     -959.958   -927.38507   -538.83559  -1115.04363   -813.23109  -1036.24528   -782.70662    -944.4136  -1040.61452  -1016.54589  -1086.71508   -893.02419   -970.71011   -845.20069   -923.31639   -996.64527  -1415.49628  -1145.78555  -1478.27803   -769.61566  -1041.58635   -1219.1016  -1048.33892   -985.31659  -1107.87615   -849.58759  -1089.66203  -1759.84916  -1365.29757   -551.52756   -680.09215  -1318.25044  -1502.91969  -1049.80254   -911.91944   -1424.6514   -725.43848   -884.01416    -205.1413    -26.36646   -1134.3425          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       16.50   -470.74107   -438.14693    -185.2314   -450.01487   -551.45212   -428.26619    -52.89097   -569.74427   -386.25308    -36.12306   -444.04041   -225.94827   -289.56967   -491.27152    -243.3553   -594.42968   -303.20261   -201.37191   -203.52564   -149.24007   -421.47567   -339.98158    -38.23221   -103.53205    126.51733   1535.47569   3149.07118   4050.55432   3159.30441   1393.07496    146.38323   -141.44274   -164.28161   -281.76115    -285.6331   -404.72712   -532.55893   -308.33225   -284.57235    -297.6123   -329.69338   -403.72378   -434.48943    -412.3458   -490.13627   -240.63127   -356.09051   -535.27577   -347.46877   -265.80943   -231.30376   -394.33961   -360.83478   -367.66555   -327.99856   -258.72169   -244.46437   -313.45487   -182.14558   -258.05126   -375.46059   -364.79713   -273.39056   -320.36169    -298.6874   -278.73509   -206.97035   -270.12002    -49.20519     95.94707    782.99558   1771.19795    2206.1513   2035.88713   1109.89973     177.5809   -514.69626  -1396.73364  -2564.75821  -1978.52935  -1248.27503   -561.52709   -332.80247   -245.29268   -291.75556   -302.48192   -283.24381    -12.29418    121.01978    740.61339   1850.11734   2816.15221   3068.79186    2020.7486    935.11949    231.55492       5.6352   -226.70248   -137.94374     -4.54899     69.35983    165.95747    385.27905    197.32202     85.52822   -299.54334   -312.31191   -302.34592   -352.98604   -295.34934   -333.55139   -353.06771    -318.1524   -282.09462   -426.29175    -186.2003   -155.43034   -260.00617   -146.58434   -144.60428   -239.31854   -295.60121   -174.27938    217.00497    538.76383    782.21633    808.72148    853.26724     542.9523    265.26137     18.39501   -136.51441   -304.59149    -48.09127    -92.94554     56.72543    590.24708   1206.03254   2210.38846   2723.80638   2812.48868   2367.55271   1571.94331     961.5503    342.70606    284.56669     -5.68603    203.00384      2.36927    -80.00366   -240.66734    -227.2856   -291.52966   -204.98222   -146.33339   -367.87783   -460.00672   -252.53437    -218.5503   -116.91154   -346.38604   -286.65598    -52.92418    -86.81339     -21.6438      97.4596     15.79681    -142.7441   -132.46596   -270.04877   -402.77296   -548.48584    -458.6292   -305.02203   -236.03752     49.95149     36.56372    273.99726    243.30587    268.18923    -17.05226     54.74391    -19.86178   -117.45662   -168.44056   -382.05423   -184.44845   -393.76097   -314.22368   -369.85029   -328.52157   -311.34174   -227.94055   -162.95054   -326.10203   -101.99241    -47.38403     142.4213    182.95472    285.17249    168.59011    182.55997    167.75882     20.14744     30.09485    -27.18588     49.51503    143.50948    145.65747    298.02295    117.02173    180.51722     15.71957    -111.9598   -168.59302   -252.41464    -357.4167   -334.77442   -248.21907    -444.9298   -311.01625   -276.28316   -309.27481   -120.28021    -90.91533   -187.35654   -139.10471     79.09058     13.97472     41.83034      92.5648    202.44055    165.60601    110.90226     94.60507    138.02126    195.85423    321.74176     287.3788    468.83367    277.90144    110.92977    265.39881       4.3263   -234.56478     -8.47617   -384.28106    -121.2208   -268.35082   -205.74645   -249.37835   -228.72628    -83.43331     -47.8944   -139.40349      5.33421     96.46966     -48.8042    112.92235    167.41367    178.11831    413.84018    449.07808    600.96653    501.74855    669.73379    616.59289     705.9086    506.51319    533.15359     381.5275    176.20139    -154.1405   -110.55546    -30.72544    -236.6084    -216.1701   -118.28976   -269.99104    122.21654    -94.88602     56.68798   -175.81648     87.60853    281.03698     47.49246   -270.42862     65.25503    -53.68741   -237.20256    -95.11979    -149.0296   -245.54767   -240.73167   -294.94201   -324.68703   -204.40554   -311.90754   -367.74171   -151.86799   -153.36992    -372.2782   -159.33022   -183.82397   -213.55283   -181.81654    -42.31996    -401.8352   -190.29524    -39.23306   -142.04982    -49.29034     -9.25977    -50.65021   -119.76473   -111.29629   -136.52301    -13.97767     -36.8458     15.85766   -210.43007    -46.44849   -374.96702   -339.25445   -134.12319   -186.94023   -173.30702   -194.23355    -35.91544   -142.36723      0.89288   -123.40386    -78.99044    -44.80104    -40.79674    -21.57454    -68.46644    -11.02582     21.89313    -68.51679   -110.62518     55.35327   -100.43025    -87.71693   -167.26143   -336.78799   -188.60123   -267.98166   -155.85949    -157.4967   -198.46503     12.56682   -162.11075      2.23679     64.98846      30.0389    127.92327    178.52988     62.24687    148.37352    -45.47499     63.93576    195.42641    148.29454    159.33364   -137.21503     86.40702   -142.75917    -188.1701   -128.31962     78.46813   -115.54497   -179.98716    -94.47537   -156.40589   -132.57626   -129.61149   -158.24848   -104.95408   -103.48494    -53.51071    -67.65349   -138.37408   -215.75731   -284.51505   -159.77948   -103.93432    -341.7128   -300.09889    -168.1134   -362.64082   -195.23738   -262.34022   -374.89092   -345.31502   -264.56729   -145.31179   -270.98993   -331.68078   -346.97907   -192.19409   -341.95097   -293.37632   -273.82503   -421.30919   -300.07069   -104.49841   -201.80473    -278.6447   -397.69328    -169.2668   -153.52492   -192.69998   -460.78542   -351.15633   -611.07775   -367.76294   -203.39805   -289.27297    -450.6865   -326.03775   -397.51918    -368.1789   -401.85438   -776.34733   -340.06809   -438.34626   -591.71237   -438.84686   -674.72895    -639.6974   -558.63262    -741.8802   -437.62641    -694.1011   -727.94447   -730.62725  -1096.34731   -865.09461   -810.35742   -742.08913   -982.35253   -624.54291   -882.43677  -1059.97115   -909.31907  -1068.23189  -1197.19535   -927.59483  -1077.23528   -892.19783  -1111.72157  -1006.49973  -1190.07813   -956.43288   -998.98203  -1338.54488  -1172.28713   -958.67088  -1220.71304  -1173.93378  -1060.18381   -979.91029   -748.60738  -1534.80752   -807.52241   -962.37488    -1435.898   -997.45152  -1196.12598    -789.0319   -756.72997   -976.86367  -1240.13336  -2122.00101  -1381.46567   -940.26589   -729.71036  -1368.02067  -1008.63599  -1658.25089  -1117.37366  -1200.44428  -1072.02824   -846.35278  -1755.37727  -1118.60673  -1152.46988          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       17.00   -592.80293   -386.67753   -216.64665   -390.24568   -523.40858     -530.618   -138.90235   -523.72926   -345.56876    -114.4823    -381.1659   -272.68888   -119.07424    -237.5035   -406.53152   -607.91742   -144.35915    -34.80057   -391.14672    -80.92631   -188.20794   -395.35791   -173.12607      -18.956    178.78669   1544.99017   3224.82095   3992.55942   2959.49423   1334.87759     14.53382   -272.65242   -167.73473   -160.68612   -341.10893   -144.95239   -288.31087   -221.27001   -343.63269   -282.12441   -275.36122   -300.86277   -396.32147   -308.27603   -422.43565   -514.21069   -333.35655   -303.77886   -478.38495   -317.74017     -442.232   -330.47353   -420.75747   -409.39958   -383.95912   -515.00962    -364.7464   -400.15042   -145.87728    -239.7318   -163.56195   -403.98657   -348.08158   -366.19401   -289.75551   -406.89257   -142.65835   -102.27439   -423.61251    213.47378     869.8936   1740.65053   2230.36884   1837.72769    960.28104     14.25587    -818.4403  -1849.18392   -2946.1359  -2698.55308  -1474.59574   -550.94765   -536.43495   -330.61362   -100.42693   -249.05523    -45.94482   -249.76705     60.60026     638.8144   1809.22361   2821.54025   2829.57541   2118.73155    974.10003    165.95492    -13.50918   -346.83767   -116.14563      2.44548     51.91685    160.51555    228.35047    139.34141   -144.87502   -177.68533   -247.14815   -208.55488   -247.25141   -337.43905    -322.4525   -308.68535   -255.29391   -264.52838   -225.41435   -178.41996   -121.84452     -437.289   -379.62934   -189.29916   -223.33196    -49.74486   -103.83914     78.80408    331.01233    740.38694    952.79519    831.94926      515.199    241.84502    -86.65206    -76.65385   -280.89411   -206.55345    -75.26674     17.13546    464.92623   1225.23706   2201.03934    2819.5696   2863.12315   2320.37506   1637.47059    947.15242    440.46045    166.76827     49.46566    116.00544     37.35038     46.07888    -98.06535   -264.90206    -224.9017   -346.16534   -184.54461    -343.0896    -288.5302    -322.5642   -145.69411   -236.77411   -355.26436   -356.07153   -244.12806   -195.57958    -91.26158     -9.77443     97.26624     80.51619    -24.56336   -286.04584   -321.67705   -415.86199   -401.23762   -255.49411    -73.19468    -57.41097     31.59979     243.6487    259.99942    118.96455     -1.43099    -84.63577   -153.43586   -168.25232    -296.9369   -266.04206   -260.67513   -106.09866    -275.1144   -272.37062   -217.43911   -185.61485   -272.69428   -193.98643   -216.25608   -206.72528     54.28504     99.83056    175.94087      267.634    236.52254    207.91728     29.30806     16.53564    -117.4233     -6.20734    -43.39124     15.83006     102.2969    192.82811    127.69274    110.63265     33.14733    -75.87114   -141.85349   -302.44929   -269.76195   -264.32122   -320.36588   -370.77617   -372.80093   -426.40551   -198.22053   -110.38439   -286.40314   -254.18944   -216.93898     -85.8348   -115.90934     60.64603     29.22984    -64.27895     19.46739     91.23743    105.16628    193.40599    129.84467    294.60871    410.60732    495.46717    229.05925    375.45517    221.45812     -6.08437   -115.13996   -153.73937   -215.16996    -146.2881   -146.66356   -398.53343    -54.49577    -207.6266   -462.72626   -109.57235    -39.59665     -1.43588    -19.36621     38.22692     31.98704    250.39829    262.67795     611.4248    385.11288     690.1921    630.39504    653.55731    664.97855    549.81464    460.08129    493.66266     327.1004     48.54444    -48.94536     97.93274   -191.75795   -198.59144   -219.39354   -257.71572    -47.98292   -137.24113     51.54172     11.24381     81.73124   -123.49675    162.88117    -27.99277     76.08617   -102.62047      19.5768   -119.16589   -160.29462   -150.05979   -412.86778    -260.3943    -177.6394   -268.90186    -197.3126   -378.26196   -204.30205    -279.2812   -417.91494   -408.44159   -309.43628   -200.06433     -258.146   -322.41202   -140.54679   -196.33287   -142.14287   -283.61533     10.08084     13.02238    -93.29493    -83.93511     11.42358   -140.97795    -83.79261      41.4181    -98.90847    -69.63603    -73.14763    -52.52242   -127.56556   -121.90801    -222.2712    -135.6802   -255.20078   -185.28077    -73.02986   -293.63466    -84.86538    -86.35441   -179.09298   -298.08767     28.74732   -274.57274    -55.78354   -197.96759      9.97615    -57.60884    -62.52179   -170.03085   -192.76141    -83.52778   -114.08676   -218.17932   -228.78512    -39.03591   -322.94133    -128.3213   -162.84531   -204.99563   -113.39326   -160.14781     60.33266    -30.57016     21.35935     108.2506     56.18268    123.68583     76.37967    152.40271     47.42677     92.93398    139.99038    -13.83125   -208.10345      0.88434   -213.61357    -48.69248    -77.83429   -240.86012   -185.98458   -186.16641    -179.2692   -126.57431    -327.0512   -127.14772   -136.70082   -258.61344   -197.19128     35.21247   -182.94052   -714.76608    -65.36077   -262.19026   -379.69628    -78.69256   -359.53085   -353.07833   -311.82656    -19.28261    -497.1664    -217.7913   -143.06011   -416.52411   -541.15933   -369.83634   -389.53245   -323.83657   -352.07931   -306.97935   -439.58063   -360.57172   -264.18743   -266.05748    -193.8992   -344.73456   -448.06028   -432.40937   -399.86919    -300.2502   -470.45212   -788.81223   -359.28499   -411.63328   -608.93688   -563.18754   -386.84552   -626.02167   -508.29632   -353.42677   -693.80377   -543.82323   -509.46972   -453.28168   -675.17839   -519.94731   -437.44209   -344.27572   -599.84858   -425.17249   -767.25459   -738.52094   -604.63894   -815.89991   -910.36922   -603.94719   -759.70718   -602.36055  -1041.92048   -885.98975  -1059.69857   -853.97373   -959.30646   -969.56835  -1049.81386   -925.51838  -1145.35458   -913.18341   -876.33452  -1157.08035  -1020.06257  -1237.69339  -1154.44441  -1179.90479  -1248.86476  -1091.77301  -1636.80333  -1279.05188   -825.81929   -882.12225   -1000.1984  -1270.24769  -1098.94071  -1111.63923   -1176.7621  -1089.45418   -1005.1294  -1432.25707  -1345.10329   -1471.4579  -1219.42021  -1177.27203  -1605.19681  -1684.80269  -1107.99021  -1493.81151  -1721.55544   -617.81855  -1373.93012  -1226.57033   -463.95519   -1047.8503   -632.15055  -1087.20364   -263.67538  -1009.22091          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       17.50   -484.57887   -304.45389   -242.35842   -330.38365   -448.37884   -450.76072   -255.27784      -462.15    -314.9259   -125.47543   -347.42506   -323.83143   -125.95967    -20.75337   -400.75523   -543.61506   -152.23838   -113.50539   -510.03656   -363.23627   -226.16286    -287.6796   -128.79442   -171.76457    229.53553   1430.61436   3248.27923   4001.98808   3041.92586    1097.5149     135.0214   -210.49805   -159.52668    -79.80716   -386.96601    -56.65353   -290.29512    -286.0371   -207.35807   -298.49699   -289.87073   -389.77725   -213.56105    -255.7918   -457.16757   -437.66238   -309.95772    -321.4592   -472.34945   -367.45453   -174.22412   -429.32228   -363.94137   -370.11205   -289.60038   -288.96373   -304.72947   -283.96197   -111.17096   -181.42841   -298.29866    -417.2941   -545.53593    -429.3609   -382.03966    -328.7559   -236.85567   -268.13276   -104.11506    210.92911    823.07532   1763.18347   2177.53432   1921.38072    1075.5983    248.38148   -548.36014  -2406.02818  -2839.85676  -2623.20771   -1460.6078   -483.08198   -221.36058   -279.04741   -320.15273    -76.24325   -162.23836   -120.97568    -81.11017    700.33956   1616.56636    2551.3817   2601.34693   1890.47501   1056.80334     90.60423    -74.80337   -235.39836   -180.73549    -60.04354     40.52167    288.77882    131.18853      89.1941    -89.60368    -70.09947   -259.18231     -269.223   -262.95535   -307.80726   -244.97133   -336.77815    -250.7107   -243.17156   -420.54208   -256.87893    -191.4604   -129.40295   -281.55512    -88.59985   -192.69145   -240.50303    -58.79811       20.563    517.14058    798.55685    1021.8323    635.05288      620.439    254.56567     64.38051     -231.065   -115.33248   -224.46348   -104.54751     164.2419    498.12417   1424.62435    2001.5899   2635.45905   2563.66009   2086.18742   1496.05267    875.54634    306.74589    323.85659    209.62469     59.19291     52.60863     22.85931   -100.41764   -108.95087   -127.84955    -84.26765   -354.52444   -399.04655   -239.21876   -275.17469   -203.42086   -207.89648   -194.69531   -331.63744   -129.68772   -234.61405     -6.55872    -55.94102      8.87989      16.1615     -94.4296   -167.13321   -278.19582   -351.50668   -302.55622   -142.36191    -71.88356     21.88378     16.06104    140.04603    197.78431      1.58381    -55.41271    159.53161   -104.85459    -46.67606   -214.64209    -258.3017   -232.38719   -318.54196   -172.85803   -351.01373    -12.26432   -250.98374   -217.59757   -238.43127   -265.37947   -190.88754   -154.95251     16.68021     78.64397    256.78446    144.02493     138.0711    179.75708    120.14043      6.56417    -22.00706    -71.99216      4.44484     93.51162    174.61233    227.86606    247.39291     60.24364     40.18439   -131.69382    -24.12458   -121.97116   -292.22384   -231.69467   -269.23419   -201.90272   -287.43594   -473.97913     -330.226   -110.81699     -53.1842      0.59352    -80.65399   -138.22633    217.95488      6.22801     12.39786    -42.48531      4.34625     409.4089       37.203    133.21536    118.87324     83.38361    168.78053    168.38661    248.76422    -10.14989     74.65462     -49.0173    -14.95557    -87.97711   -314.25983   -233.86294   -170.13511   -262.64771   -217.00247   -145.28119      56.5282     -63.0383    -87.63794     -16.9069     66.19203     84.41392    167.20142     323.9035    372.69896    534.95751    594.66677    670.13136     669.9444    678.03091    345.37741    648.29549    429.78682    230.31603      98.3593     173.0762    209.06413   -116.00961    -196.8213    -65.50479   -139.29937   -174.23261    -40.05066    -28.85224     77.98287    -48.46579     62.06732      52.3505   -118.16618    -70.23678   -161.65021     -78.1356   -178.55399   -216.32736   -257.31676   -301.81048   -343.84344   -440.77807   -323.44652   -351.64028   -447.91293   -246.62353   -243.99697   -294.05407   -220.14196   -236.44843   -189.98158   -270.78625   -171.53158   -317.27524   -151.17981   -357.52091    -235.8595   -144.48453    -238.5406     11.68475    117.27334    -87.70547    -50.84199     -31.0182    -67.09458   -119.39645   -182.12398   -129.69069   -207.52506   -126.43298   -127.14081   -228.28154   -211.64925   -210.89546   -314.66381   -192.87345   -129.98919   -125.15306    -91.12167   -160.99942    -36.27038   -169.41583   -168.36533   -210.41412     -8.98722     69.73328     42.38533    -80.73237   -189.48768   -314.02791      10.9889    -201.9986   -135.39001    -120.9353   -301.91671   -137.61929   -283.35542    -88.02249    -49.36646     62.62348   -219.30857     15.55406     -2.61675     52.43104      1.28256    -38.98268    250.62087     84.48836    442.62151    163.12434     -50.1996    111.58686    -24.07207     29.10935   -154.32895    -59.89085   -190.38103   -278.49083    -24.60999    -104.1659   -375.72703   -112.71417   -152.77217    -87.57875   -175.87288   -356.05001    -248.2201   -294.70861   -139.40373   -268.42955   -424.99276    126.59932   -179.31317   -243.73343   -312.61085   -315.47315    -167.4395    -405.7511   -273.07993   -114.56043   -413.37176   -558.83814   -474.61179   -817.05252   -344.89285   -459.57005   -432.97797   -160.09036   -327.20351   -572.04998   -405.51235   -229.32948   -331.31451    -515.7617   -279.55828   -232.38014   -294.82785   -309.12165   -479.70343   -350.18733   -221.23318    -443.6858    -231.7822   -669.41135   -212.44433   -261.75204   -426.75667   -308.96103   -663.23322   -496.80831   -571.06768   -604.63559   -619.19974   -658.88478   -963.91164   -475.12327   -550.30095   -506.00363   -787.58547   -700.40653    -808.1677   -951.48541   -908.18424   -723.07681  -1125.89697  -1025.46015  -1278.19378   -934.83005  -1021.32667  -1141.68395   -946.46859  -1165.17829  -1090.35696   -1309.3366  -1256.35106   -980.71717  -1141.40189   -973.85533  -1070.82006   -987.60121  -1053.73367  -1293.94122   -1101.7559  -1218.80639  -1175.49206  -1231.63991  -1551.10445   -945.75094  -1153.88603  -1592.84699  -1288.19482  -1583.44844   -878.05392  -1514.09649  -1610.71705  -1522.32757  -1763.45274   -695.16702  -1371.91132  -1145.90522  -1014.47079  -1112.50213  -1556.86035   -1338.2858  -1753.92592    -964.5706  -1242.31247  -1020.70603  -1294.15433  -1863.51159  -1313.02521   -898.51997  -1530.49206  -1264.54368   -934.28384          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       18.00   -241.29844   -353.91965   -216.64894   -214.69462   -358.55992   -208.19766   -299.04912   -361.64592    -285.4468   -204.94376   -319.16694   -349.40666   -265.00677   -268.64082   -413.87518   -297.83529   -258.26983   -243.22198   -360.33619   -332.22722   -374.15237   -217.10432    -184.3583   -139.78556    289.58927   1256.66219   3112.84317   3976.70745   2905.27871   1096.17366    158.58015   -303.74867   -210.34826   -149.21152   -285.07176    -256.3121   -298.62514   -362.71557   -345.22493   -317.34569   -214.74722    -362.7914   -447.55007   -293.83401    -433.1374   -237.16334   -455.29552   -315.80457   -434.81131   -501.55329   -242.71176   -532.79351   -364.40028   -228.02284   -290.42174   -346.34396    -238.5487   -290.40259     84.14459   -168.62696   -206.45296   -457.94217   -322.08067   -275.06313   -328.33973   -264.64068   -306.41733    -353.2532   -266.02144    132.40191    690.01574   1640.52172   2336.49553   1999.87901   1058.99469    274.09961   -467.16185  -1266.11928  -1659.39572  -1510.53392  -1011.42507    -534.9198   -339.95143   -392.13326   -256.06383   -203.91446   -238.32091   -179.15479     28.34721    692.30692   1548.25073   2704.19289   2845.16695   1955.03296   1091.42977    306.94952   -107.60052   -204.52427   -219.25175    -21.32128    -48.87223    291.10107    241.13011     62.20873     -8.32162    -76.89475   -207.28026   -298.14944   -301.67825   -273.19991   -325.22331     -251.492   -330.14868   -162.06251   -188.01559   -284.65537   -216.17174   -216.75431   -273.09265   -303.03653   -158.90636   -279.95413   -153.90885     117.9646    390.65702    776.65077    985.84418    740.20634     729.9564    330.92211    -27.33187   -180.22824     -163.382    -21.82896      55.4482     77.66269    637.02366   1349.61848   1886.70852   2646.54521   2739.87024   2246.78436   1432.94404    807.62819    486.01091    132.33442      9.97654     102.0078    238.60619     -2.55699     -49.5153   -319.69882   -284.03888   -149.72195    -99.52334   -319.14634   -263.40696    -93.65421   -277.34858   -161.00042   -216.28525   -331.61594   -192.40988   -235.62418    -51.03888    165.98426    -38.37773   -145.93042    -173.8265   -205.07667   -265.53984   -165.55252    -275.6759   -176.93604   -301.54873    -35.21676    100.61269     179.4327    356.74372     92.93097   -101.92211     26.29332   -371.67084   -117.74805    -142.7182   -221.15164   -197.77719    -45.69251   -347.00925     -209.387   -263.41134   -198.67949   -147.72609    -87.96759   -136.76196   -203.83416    -19.90981     97.66816     31.14482    341.00924    248.23122    -56.06905     67.24035    -43.07613     275.8077     96.18374     24.05902    186.69153    219.19387     79.76595    -16.99477    134.91191    271.19746     15.77556   -163.55628   -267.24932   -342.72963   -266.28088   -194.07846   -361.80061   -109.31697   -295.31496   -214.00275    -316.3389    -47.30292   -116.49306   -118.64186     50.87422     49.05224     74.49772     88.81946     86.69677       82.056    240.19524     90.93683     331.1363    150.33709    228.91292    215.62058    230.97311    184.39675     69.18283    270.43314      90.7319     71.55947   -211.26624   -120.22515   -309.05861   -378.92545   -248.76099   -251.99457   -133.96552   -172.54428   -178.65949    -59.42993     50.69378     46.79315     14.57893     -5.32262     264.5871    157.19983    354.68878    510.69022    269.29704    639.45014    728.77474    817.44217    726.81839    422.34144    390.19831    179.37681    114.37421    122.23336    101.53604    -100.4317    -39.30218    -41.62214    -87.70788    -124.4764    -62.24523    -40.01479     19.50492    101.79194     -4.09306    122.79731    -26.92556   -162.93698    -88.62349    -149.5323   -247.67697   -109.68349   -206.59593   -281.82777    -431.1113    -364.1129   -367.47103     -253.131   -329.53892   -313.56382   -321.21622   -309.71526    -33.83262   -271.27772   -305.87548   -210.02522   -240.22382   -303.03184    -88.68632   -230.17152    -45.70556   -186.13768   -140.50886    -44.35734   -212.01045     -70.7155    -73.93212    -46.35904    -54.58213      5.28057    -145.4892     -96.5024   -113.73538    -86.37127    -229.2598   -227.58207   -310.03362   -243.90802   -160.19118   -214.30341   -166.44232   -199.95684   -257.14934   -375.20729     -7.76094   -120.81641    -80.01205    -54.15336   -168.04262   -236.45003       4.2034    -70.18304   -134.96806   -267.68977   -163.10134   -301.31287   -329.39315   -277.37554   -345.84371   -352.33487   -187.86767    -69.86239   -247.45871   -199.62899   -216.50143    -94.33245    208.10926    -43.74012     51.87088     -1.99716    -12.92274    -33.85888     63.43407     63.49822    -16.73162    200.60187    -85.41552     70.56119     44.00249   -116.65904   -179.93656    -11.34212   -281.70894   -115.23858    -93.62333   -306.04108   -204.95931    -81.94785   -277.00208   -293.01619   -280.59875   -135.24589   -145.70135   -252.29831   -351.74618   -110.48511    -391.2589   -424.61428   -584.24698   -279.22973   -323.85504    -261.0205   -485.49736   -303.83724    -665.0061   -544.27714   -529.90807   -599.56658   -301.37427   -386.36131   -458.39499   -540.70664   -365.98107   -408.90396   -432.95549   -236.36356   -308.31112   -269.11545   -717.43624   -597.05927   -192.97191   -460.72242   -580.80191   -458.28106   -554.38551   -424.44141   -363.23337  -1000.87994   -550.49945   -445.07847   -739.37541   -498.60627   -509.29028   -829.69801   -551.91722   -655.25418   -671.45002   -807.63459   -654.53441   -850.60296   -777.35449   -933.58297  -1020.81322   -950.02546   -890.47608     -838.792  -1052.22441   -734.95577  -1286.15387   -975.37674   -1004.0849  -1063.60325  -1016.00632  -1146.50556  -1335.83601  -1561.14638  -1409.06668   -906.35559  -1365.67244  -1208.16265  -1495.52733  -1170.88252  -1297.05365  -1471.79064  -1596.51442  -1264.89001  -1602.55299  -1150.07085  -1146.81578   -1204.0401  -1193.78157  -1381.11559  -1529.74515  -1362.01328  -1366.76741  -1205.11638  -1029.59199  -1194.61908  -1391.26264  -1167.69759  -1365.27955  -1219.42777  -1324.84557  -1771.07898  -1563.34891  -1497.92097  -1587.73837   -2148.0578  -1769.49034  -2459.91028  -1394.80347  -1592.43414  -1925.49147  -2089.46791  -1170.64673  -1204.69577  -1660.50325   -184.69664  -1582.22666          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       18.50     38.99571   -563.21782   -199.28916   -112.36219   -370.11258    -88.77214   -301.86668   -324.22769   -321.90603   -359.69071   -281.33358   -450.42297    -84.53641   -425.64202   -351.02438   -346.85122   -310.96744    -260.8111    -354.5682    -43.19972   -114.88016    -216.9504   -278.55458   -155.31554    232.01079   1377.99768   2915.83843   3917.79484   3260.95315   1194.82237    -51.00987   -297.85579   -199.34978    -45.40567   -187.57346   -251.36922   -185.66835   -292.61318   -221.22044   -272.84975   -238.61695    -353.1862   -495.28769   -347.16932   -341.90955    -259.4214   -318.32649   -215.42578   -395.11528   -499.46094   -409.50998   -439.88846   -347.29656     -397.545   -314.29209   -467.86083   -106.45383   -156.69714    -93.30146   -467.55706     11.17288   -308.34464   -338.16387   -351.33828   -379.87871   -294.44103   -409.43565   -423.84835   -344.07881     16.69584    701.46299   1784.57399   2361.80386   2002.78084    950.75259    453.34158    -43.18098   -546.59561    -854.4589   -907.03691   -583.42762   -355.43889    -345.1406   -148.13947   -187.58131   -281.08658   -267.91344     32.62393     -9.34124    562.26784   1813.25119   2702.40416   2960.47166   1906.06961    938.79968    177.17229     17.77177   -125.09245   -118.18848    -23.60607   -160.42954    -26.25197    301.45623    231.19794    -21.74625   -152.88407   -314.86896   -409.48976   -300.58359   -321.73454   -272.06908   -460.01113   -384.30163   -329.53365    -295.6003   -418.53469   -216.92862   -164.46611   -214.39662   -193.38186    -133.0964    -366.4607    -47.97801     45.56318    303.85786    726.83562    857.08242    759.62888    439.94361    173.53233     89.70364   -161.74876   -228.57157   -225.39306    -115.3732    -29.63887    469.53971   1346.48254   1907.84557   2375.60046   2728.93124   2036.38514   1399.41185    820.71971    445.75471    215.40449     224.6998    135.23514     -8.76639     11.17245   -165.14695   -178.57595   -269.66246    -130.8692   -256.27491   -421.46618   -202.37822   -158.68799   -203.68876   -166.84969   -232.36862   -257.44303   -192.03416   -201.12066   -175.32388   -145.18994      1.43545   -144.83181   -279.62049   -339.61252    -290.3362   -230.25414   -298.58549   -213.70153     12.68227     52.86224    129.49166    176.39283    223.82543    108.30248      58.2744    -57.00396   -135.75174   -196.80005   -176.53026     -114.268   -217.83851    -359.0367   -310.95005   -361.44772   -352.58719   -318.51562   -316.17433   -257.27088   -106.54437    -97.09794    -10.08791      22.4259    157.77746    234.94812    317.58579     164.5035    211.12032     69.57993    -36.79395    -23.75905      5.89862    133.66221     82.73058    268.77454    215.79032      -1.8784      5.14513    -20.77912   -149.39657   -224.56769   -232.97024   -194.99282    -352.6602   -178.84932   -267.43383   -280.89678     -211.034   -241.55421   -168.12871   -137.88096    -45.78627    146.38026    116.91833    170.96086    107.85866      64.2018     105.1487     33.49664       -6.842    301.28618    153.24155    177.99807    291.94478    318.88172    423.07552    129.72727    121.08553     49.43027    -23.84502   -247.23756   -183.51908    -77.23994   -290.89907   -208.10676   -149.92034   -143.71837    -68.32621    -99.31762   -143.40279      2.34386     75.36084     21.45257     30.09349     97.61994    260.37572    406.46244    399.60996    494.27898    641.91514    817.36282    597.35049    637.55494    751.80174    455.70551    273.78589     209.8146    131.40161    -42.71133     -9.00141   -326.77749    -55.37228     31.59296    -27.00961     -65.8333      53.5956     63.55089    -48.03098     56.90731     79.60185    -61.83736   -102.80161   -122.06039     -9.51308   -192.71834    -79.82161   -256.65362   -277.40639   -382.16923   -387.52053   -243.97653   -452.04263   -483.25728    -450.0126   -307.06194   -408.78308   -200.10599   -256.45707   -167.72024   -238.90337   -285.43813   -245.36105   -195.53349   -337.75503   -166.75297   -137.63383     30.17589   -109.15044    -99.54106    -42.19504    -125.1406   -149.70759   -131.47116    -54.90874   -162.31087   -190.29813   -156.77233   -188.83013   -205.63822   -203.21567   -202.76596   -239.96234    -220.3246   -191.08917   -340.16167   -266.56086   -152.60881   -201.13034   -165.53676   -131.61586   -209.67527   -120.62071     45.08646     -214.123   -202.45896    -83.63462   -153.75101   -223.85057   -267.22974   -261.20894   -195.29591   -411.87076   -327.92117   -350.46502   -567.97268   -281.89591   -197.86628    -100.5584   -215.65943    -56.74924    -58.92287   -224.74174     34.84512    139.55676     57.45551    -39.37257     -50.0059     -6.33367    107.41455     13.56736     14.14827     64.61105   -257.09348   -157.01483   -183.79989      1.28063   -341.24391   -147.93148   -216.41052   -135.17975   -287.80628   -301.17515   -112.10714   -339.06608   -279.16348   -271.95711   -387.50594   -357.27927   -595.55572    -296.1759   -547.94254   -516.33812   -206.89735   -320.24695   -209.59255   -495.43109   -538.42223    -440.8851   -473.53345   -548.95224    -481.7887   -589.63856   -531.95418   -711.85505   -577.57633   -398.05643   -311.21845   -346.80383   -498.31456   -476.29486   -618.97603   -288.75545   -806.98743   -401.82819   -291.78682    -480.1612   -646.01586   -603.75651   -590.44394   -606.82346    -690.9852   -556.86408   -541.38889   -520.77004   -400.72632   -701.45598   -506.72387   -663.95077   -482.45115    -716.3267    -606.4966  -1020.19351    -919.1101   -633.18199   -933.88753   -778.97409  -1104.41158   -926.94589  -1161.50122   -880.15423  -1127.98751   -965.66706  -1462.20387  -1205.38403  -1384.95434  -1284.99926  -1008.14912  -1125.75769  -1220.65946  -1097.19701  -1367.24983  -1515.17108  -1146.70481  -1074.51352  -1208.48309   -946.74258  -1156.61362   -1476.9372  -1358.66049   -1242.8251  -1123.84038  -1620.99795   -1717.4296  -1567.70496   -927.09946  -1577.27168   -1429.9568  -1458.01788    -1302.356  -1734.57782  -1453.28077  -1459.79446   -1752.8773  -1322.53163  -1735.33481  -1090.03983  -1474.53676  -1841.47016  -1640.65319  -1820.84466  -1997.51778  -1653.87774  -1701.73758  -2027.27528  -1512.72685  -1532.69418  -2030.17365  -1751.14854   -2439.8088  -2889.36833  -2830.35567  -1528.87921  -2019.14861          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       19.00      4.49215   -809.52318   -300.93997   -124.10689   -360.96285   -231.64387   -300.45087   -308.36772   -324.96696   -404.39788   -100.08144   -367.08662     -1.28288   -336.48377   -331.34845   -413.69993   -312.25606   -236.24633   -464.08964   -121.24749      5.32722   -326.93182   -239.70051   -161.24936    -33.24362   1433.79695    2748.3556   3770.58306   3046.67653   1265.74668   -110.99008   -323.74269   -160.18774   -331.46896   -197.59511   -509.25097   -168.67205   -268.05976   -350.00151   -330.31312   -371.18054   -302.71096   -422.85615   -335.08646   -380.34128   -328.70562    -365.9803   -372.24748   -301.15324   -196.15342   -390.74349    -282.9676   -322.64922   -406.71334   -380.69017   -305.67101   -234.54102    -87.01029   -337.74646   -220.28543   -214.53665   -163.03976   -224.93789   -331.26139   -446.47818   -364.01319   -330.26662   -237.47378   -196.86962     81.01152     638.5685   1762.60423    2416.6974   1818.97307   1046.52385    402.68735    105.93523    -23.93244   -232.53905   -413.98649   -302.61246   -423.35609   -164.64335     -198.459   -470.23862   -224.23131   -190.56888      -38.107     43.83893    959.49095   1840.25866   2779.44297   2946.75502   2028.31083    625.16266    347.19658   -153.38727   -206.38903    -34.52466   -165.59718   -165.42373    186.48461    140.90199    127.03308   -115.82479    -61.35822   -214.26376   -559.09458   -447.62239   -408.19908   -376.99041   -285.07902   -371.94437   -531.22532   -193.83172   -258.88024   -154.21433   -236.88641   -229.79061   -123.82539   -234.66384   -141.66285   -187.83952      62.5732    253.07944    733.23693    929.39781    908.66422    512.97268    238.08025    -44.00003    -93.28075   -108.67786   -262.64478    -49.13479     16.34263    515.47336    1364.5867   2023.10587   2511.10193    2695.9087   2020.46212   1453.86059    733.40174    338.65552    150.65129    328.45275    160.62332     51.19249    110.98882   -118.09694   -295.44677   -205.73932   -224.69878   -218.53485   -386.60212   -282.21208   -290.56009   -233.91769   -275.47434   -206.42958   -328.80442   -208.21899   -259.87414   -261.79676   -105.04282     82.93385    -74.65445   -138.67459   -172.36027   -199.93648   -112.43088   -241.33586   -282.48227   -221.76571    159.31853     78.47982    328.71846    337.05101     92.95299    137.70772   -124.99215   -141.88452   -186.19097   -241.88094    -195.6083   -239.58937   -276.16533   -114.54626   -383.28944   -226.56233   -344.77891   -176.39161   -233.64363   -195.61851      0.18277     30.13439    213.02633    175.55587    187.88716     37.82146     96.74051     84.71866   -122.75073    -64.43677     24.46432      29.8846      0.26934    167.15909     81.04025    227.11907     16.34619     72.90428    -55.06554   -142.78467    -267.1676   -340.27287   -302.96479   -222.37347   -401.87098   -253.91301    -367.8419    -336.1879   -106.84909   -264.59063    -73.41235    -35.72804    -18.01233     90.63731    -55.05121     50.98163    239.54984     -13.8036     28.69454     38.10783     167.1811    178.33758    280.85174    236.02204    284.50227    224.55253       67.508    147.19311    -27.26009    -87.18563    -46.88046   -207.20788   -209.53568    -225.5606   -190.61984   -226.15653   -192.58156    -29.08513   -106.82218   -275.47049   -106.67989   -268.57346    101.12498     63.45722    232.35067    147.22857    501.68273    426.34981    656.36989    831.36404    760.32209    906.38402    731.96987    628.03374    423.64764    412.08072    126.23146     276.7269     10.73022   -142.61572   -141.07944   -105.10592   -188.83369     -7.04953   -124.85174      32.1693    -22.95886     49.38942   -151.42369    -91.87649      6.07572     23.79792   -173.67224   -271.26346      6.82819   -210.92136   -383.82415   -201.96224   -139.38083   -320.24734   -383.96106   -363.15468   -214.84993   -551.05468   -302.93922   -279.65309   -285.47869   -363.09862    -286.4842   -258.60762   -172.46069   -270.25975   -164.36302    -108.3831     -310.416   -157.21206     -8.48749    -301.7782   -277.09098    -60.99698    -95.59245   -140.16968   -112.89147   -145.46844   -109.64546    -142.4718   -211.80256   -280.87064   -489.53335   -244.68837    -204.5787    -227.4197   -311.15727    -81.40045   -184.86481   -128.65146   -348.25531   -157.21389     -38.9258    -309.9697    -95.78327   -124.91199   -124.64137   -288.26516    -23.87384   -302.33364   -315.33116   -244.17123    -237.5112   -407.96052    -53.75242   -256.53785   -296.34254   -210.09353   -331.82298    -86.94883     65.31599   -151.08316   -147.91012   -224.38393    -211.9269    -87.66877    -19.04707    -89.98294    -25.05222   -133.07212    211.70588   -112.56766     38.89269    -78.45705   -157.16198   -181.36829   -197.06254   -370.58592   -295.38689   -364.50928   -378.28093   -301.51887   -363.58953   -144.33829   -295.32348   -301.35681   -478.51585   -256.57401   -159.14943   -278.95634   -282.63793   -341.45149   -380.29658   -438.27053   -547.58738   -307.51416   -472.62375   -508.76199   -570.36098   -594.18777    -454.3076   -492.60327   -529.72673   -528.39996   -521.44844    -654.6903    -581.7383   -457.72731   -510.26364   -516.57132   -510.74096   -706.02485   -589.22321   -458.01448   -536.11225   -591.48562   -451.30304   -618.52254   -560.82968   -461.94124   -554.74834    -584.7051   -384.84167   -547.79957   -725.36863   -631.67995   -703.34058   -712.79492   -804.89031   -838.26262   -769.40748   -658.23527   -863.81918   -635.33309   -667.65049    -639.7379   -739.84505   -956.07125   -728.45909   -940.56827   -804.94667    -791.2809   -841.42478  -1298.73207  -1129.41041    -980.9456    -976.9374  -1097.35286  -1076.62421  -1029.97487  -1389.29208  -1415.81804  -1230.46244  -1190.39466  -1695.86769  -1363.07177    -1262.512  -1067.39801  -1324.03648  -1136.21198  -1443.14783  -1704.33493  -1693.22371  -1500.91383  -1678.46527  -1547.45694  -1448.74035  -1690.39616  -1460.99755  -1376.96684  -1945.97631  -1507.67813  -1105.42945  -1814.98299  -2352.50402  -2110.76073  -1558.02061  -1910.83566  -1756.88799  -2234.82941  -2061.20666  -1728.28398  -1992.24784   -1300.1831  -2321.73397  -2766.52943  -1315.08408  -1607.24583  -2420.26196  -1640.31306  -1816.45549  -2426.70176  -1436.39157  -2713.84913  -1694.61056  -2033.21383  -2476.93625          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       19.50   -208.52004   -792.77477   -402.79882   -182.03432   -343.36824   -338.05961   -267.09173   -262.87453   -328.71773   -280.08499   -235.73382   -203.93833   -284.88553   -309.06834   -393.57379   -348.24856   -324.35998   -357.43051   -310.21736   -147.00252   -114.05033    -285.2817   -218.35767   -148.25737    166.12069   1289.32177   2891.52357   3804.60934   2681.92592    1194.8103    -14.37061   -350.22533   -388.52497   -357.25786   -276.13981   -401.01946     -8.85764   -414.38649   -179.99228    -384.7011   -465.00038   -325.18713   -410.31796   -301.59743   -312.54931   -388.02616   -490.07976   -463.09732   -403.51478   -404.74462   -347.38077   -251.46325   -338.87063   -305.06726   -317.07876   -187.67657   -257.83231   -238.84931   -188.07684    -256.5859   -326.30256   -270.34237   -230.37613   -333.88948   -231.12572   -326.36881   -219.54083   -298.41752    -219.6998    237.92559    919.38093   1779.08203   2512.62052   2079.87194   1136.69447    418.22468     25.56796    -68.45756    -70.73308   -168.65126   -326.99705   -318.14345   -214.18994   -286.93303   -168.83532   -141.98033   -140.85266   -132.57328     61.16877     937.0241   1884.20432   2753.02993    2733.1025   1888.64711   1111.39833     241.6085    -52.50972   -111.41968   -310.99705     23.17872     42.36057    178.62669    289.66224    123.82539   -182.06953   -126.90943   -115.99551    -198.9161   -387.90947   -347.60555   -342.89664   -388.35195   -282.31911   -369.25151   -376.05511   -329.31905    -262.5013   -122.73161   -202.19438    -171.5305   -213.84423    -78.46616    -90.26062    139.00548    474.68608     497.3676    795.13291    907.77385    690.26253    273.39428      1.69137   -158.09118   -263.77134   -118.43886    -21.20424    -39.94854    669.53773    1274.9194   2085.16965   2489.27935   2674.54389    2263.2982   1537.46878    658.11225    392.91274     289.0642    259.88681    161.90079     41.50694     -33.1972    -29.90345   -177.43983   -293.85726   -305.31734   -323.38688   -189.92606   -341.71243   -170.69957   -418.00657   -347.37028   -321.03007   -324.56731    -47.09566   -145.98016   -139.45349   -102.56614      22.3936    104.55827     58.70352    -34.67018   -264.00105   -227.18494   -140.90607   -226.28515    -62.20765     69.40645     92.10893    127.80742    183.07905    148.35275     84.69623   -122.46703    -23.02219     -134.021   -235.05823   -258.59905   -173.99094   -232.60009   -440.26069   -368.51293   -231.16152   -228.11546   -220.29856    -160.3805   -102.07238   -174.09566   -114.28314     63.23586    101.02404    220.59853    242.77187    175.73104    143.35404     55.67249    -26.84475     15.50617     -7.09442    136.63786    136.06593    191.35902     71.88588    261.68896    -25.53117   -148.01518    -46.88769   -366.84009    -235.8851   -222.56484   -232.38524   -314.76615    -329.3668   -272.04185   -250.36212   -222.34779   -250.47837   -131.97387   -101.35122   -110.92025     65.13756      0.42881    117.77413    105.38653     23.98117    119.63006    125.17661     32.93122    338.56629    293.22185    323.84987     248.4883    245.84639    302.65861     31.48552    281.47014   -191.81451    -88.91616   -146.62224   -218.75085   -123.47082   -236.78784   -285.62467   -178.58265   -288.24012    -16.45541   -163.38769    -16.07235      7.33994     90.98769      67.2777    141.88517    216.62806    310.86725    482.85092    515.84737    657.18147    691.76841    659.87409    667.94235    378.93606    468.90747    521.84067    273.58919    133.44234   -101.87566    -28.28103   -149.96358   -158.92133   -210.43226     15.30359    -59.27708     20.38924    -36.27772     54.76233      42.7443     78.39184   -143.87702    -11.68384     -62.6871    -197.1347    -24.31622   -413.26999   -185.38691   -104.78381     -332.959   -468.76993   -409.07156   -363.39644   -338.60484   -286.82003   -259.93355   -317.17039   -398.65146   -237.29699   -246.25547   -228.99767    -287.6955   -226.46708   -324.00363   -243.25142   -289.87658    -43.46575   -200.95183    -85.94001    -75.78475    171.85481    -83.77818   -160.63054   -129.70967     -76.4974   -122.33065   -175.84015   -174.55382   -326.25528   -302.96434   -388.21584   -221.57961   -251.58599   -199.06852   -352.98082   -141.84618   -404.77513   -217.30445   -178.45841    -196.7518   -373.51111   -229.60483   -315.52063    -13.39903    -355.2245   -226.93116   -416.58516    -213.7019   -361.24131   -257.03477   -267.76479   -196.50212   -315.07792   -292.33407   -270.32069   -290.16358   -200.59274   -307.41974   -370.88912   -164.18375   -258.02719   -104.60452    -114.1805   -109.16549   -136.07136   -135.37215     18.75872   -160.63897   -157.81209   -108.25142   -201.65209   -257.30535   -157.52494   -271.56387   -274.47562   -609.14074   -165.60398   -292.69539   -339.82609      -399.14   -417.27398   -333.93384   -351.89496   -465.18262   -384.89063   -358.04778    -441.0153   -327.18335   -340.26539   -341.80793   -548.14311   -400.30885   -464.01419   -402.82488   -483.99829   -369.80818   -705.86755   -429.76733   -471.76403    -806.7593   -626.75223   -560.98898   -547.28025   -465.89988   -455.17381   -575.15706   -609.69088   -608.29207   -647.45637   -539.06228   -734.01201   -503.73742   -810.12707   -547.91284   -631.45331   -596.14961   -591.12387   -535.23851   -530.71776   -624.79292   -770.75075   -842.91102    -767.6839   -867.01815   -712.83042   -911.89225   -899.93884   -560.32808  -1013.75775   -927.98379   -736.69681   -681.77014   -757.21282   -1031.1103  -1193.49882   -919.98601   -924.90863   -946.41926   -888.81188  -1413.02866  -1182.57675  -1173.71912  -1332.70667  -1413.11653  -1165.30783  -1001.96782  -1426.13831  -1269.47563  -1277.96217  -1387.31633  -1295.01897  -1559.14794  -1440.36451     -1657.63  -1759.77773  -1734.06442  -1503.51144  -1603.17925   -1243.6264  -1370.63896  -1372.42598  -1819.82389  -1359.65891   -1554.3424  -1511.47387  -1995.14071  -1460.30049  -1420.15329   -1983.7185  -1987.42915  -1681.21867  -1960.11644  -2077.66179  -1392.96725  -1755.29304  -2015.94389  -1672.12082  -1726.90354  -2042.93721  -1506.21151  -1691.59501  -1620.81224  -2331.01163  -2227.82535  -1978.77972  -1800.30267  -2270.03616  -2367.13554   -2286.5591  -2452.80613  -2169.70699  -2321.98304  -3132.46901          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       20.00   -495.82978   -708.58845   -521.23647   -198.81026   -372.27838   -365.85116   -181.91977   -188.73579   -346.25585    -138.8294   -496.87515    -172.2023   -457.79831   -211.73805    -383.6422   -275.55925    -355.9531   -132.05189   -237.65588    -60.12725   -344.81665   -308.97216   -232.07112   -248.19429     355.3637   1157.05825   3042.36928     3844.426   2791.08618   1182.30768     73.55694   -338.69405   -180.76195    -225.0767   -349.50583   -246.95722   -127.66412   -257.65283   -302.03314   -306.57304   -333.41566   -261.75306   -319.27486   -380.80465   -309.48635   -527.85845   -353.59306    -351.7583   -515.64928   -483.81823   -305.32929     -413.207   -218.45522   -347.03998   -302.67087    -309.6188   -265.09477   -177.42362   -182.71865      89.6588   -234.10811   -315.93504   -244.63852   -327.44434    -75.27247    -367.3251   -354.80892   -226.17875    -85.40784    471.93131    763.42896   1806.76172   2269.49111   1786.96512    959.23235    408.89271     51.63837   -175.43938    -232.8807   -344.73327   -113.81837   -109.75923   -407.61367   -210.82234   -223.55565   -380.97399   -335.23561   -275.65719     10.17852    818.29993   1914.50166   2890.56924   2801.51644   1819.98662    766.88806    142.72938     32.72136   -244.81939   -171.60792   -190.08667    107.05426    123.16015    136.59205       8.6414    150.77352   -338.11856   -341.22718   -335.46573   -349.43243    -310.5326   -300.90127   -244.01804   -308.51374   -440.70733   -315.53163   -363.77363   -340.64237   -199.54298    -53.33993   -155.19772   -208.45597   -229.12705   -205.58832      0.11921    297.54922     730.8146   1005.56178    794.95753    660.36993    241.46783     72.13614   -104.14558   -195.35801   -198.90711      6.59163     -12.2629    365.23824   1290.76494   2179.08985   2714.06239   2800.69908   2449.51623   1478.74543    871.01885     476.4904    166.59187    161.05749    168.91711     45.67677    -14.61193   -181.89964   -218.92232   -263.92951   -276.66384   -330.89006   -197.99504   -331.25802   -307.59871   -349.06962   -254.34953   -347.35938   -270.42653    -67.27903    -99.02699   -111.07201     -66.3403   -124.26582      49.5725     35.65299    -80.97106   -358.38922   -361.25811   -257.86993   -146.60402     44.95139   -121.25154    -54.02507    330.11412     65.58065     268.8904    264.37098    -25.91324    -48.01411   -128.63612    -54.46601   -376.91426   -220.46884    -290.2175   -250.63301   -206.81027   -248.21226    -278.3503   -253.62839   -192.82028   -187.15647     -143.151    -55.60596     -73.3411    235.79198     58.06395     286.5788    233.37952    -53.77823     25.26387    -34.99268   -111.68076     -2.09329    -86.42034     -9.32969    -24.61899     48.38895     25.56915    -60.38093   -110.19797    -71.25132   -254.74106   -191.83486    -62.37526    -201.7252   -337.34725   -372.28204   -346.77744   -231.84857   -239.28484   -196.32057   -145.91416   -244.42692     16.04272     73.70931     27.43769    -28.88295    114.66658    102.02382     78.08087    130.83473     145.2726    181.06181    143.96305    313.99555    464.96586    285.71455    306.89514     37.42416   -122.38826    -57.92772    -95.31502    -231.9183   -277.16418   -132.65486   -291.31608   -228.92409    -134.8148    -76.13648   -108.76984   -171.08541     -63.6301     32.16544    -66.53926    112.11283    223.19422    249.48115    130.66053    277.82822    429.41951    633.93461    775.63151    814.16065    503.36402    501.61632    371.63481    588.84673    313.77489     -2.02562   -101.49994    -79.41233   -324.44366   -311.41148    -186.4628   -322.62265     -9.60307     51.90028    217.20062    -49.68237    -17.09204     31.62492       9.9999    -85.28894    -55.68104   -223.59496    -137.2549    -97.99646   -171.26079   -298.58651    -382.0738    -392.0248   -312.37755   -351.05595   -445.53228    -431.7828   -459.69954    -274.8734   -335.01424   -128.78936   -313.57926    -410.9013   -218.71258   -169.99418   -136.39982   -365.53533   -276.43051   -172.19055    -54.47671   -189.49391     83.17985    -67.17035    -98.94841    -213.3873    -52.16001   -270.82143   -223.16814    -185.7528   -198.63761   -245.65124     -296.878   -231.07025   -256.90382   -326.66936   -297.31817   -376.80309   -268.72789   -252.93012   -172.49173   -436.59401   -311.38397   -134.80927   -251.77925   -140.02721   -338.26526   -333.74133   -153.68854   -253.45913   -190.80084   -237.29799   -406.89643   -509.45315   -355.65727   -328.51174   -445.53793   -486.58446   -404.29627   -386.82617   -373.91927   -168.30886   -275.77239   -380.92407   -125.68764    -400.4879   -208.03154   -117.84202   -168.97985    -68.50099   -206.06863   -296.80309    -268.3943   -110.21198   -282.99867   -363.01729    -368.5301   -244.64194   -452.49776   -258.31865   -434.48739   -432.47994   -451.04955   -726.82059   -401.01969   -578.53755   -475.82458   -408.78881   -380.75753   -406.00144   -466.98986   -606.72257    -552.3387   -513.58631   -668.04649    -698.8199   -785.35257   -690.08134   -704.28708   -575.27995   -886.30943   -848.51156   -713.57373   -572.77291   -865.53403   -708.77692   -838.21459   -893.28725    -913.6882   -786.15769   -862.73932   -777.73089   -742.91281    -855.0102  -1099.12807   -922.10075  -1030.93063   -731.88754   -992.86026          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan  -1801.73109  -1650.46389  -1709.98777  -1666.33724  -2017.98055  -1799.84766  -1942.57894  -2257.40987  -2139.38867  -1771.49874  -1614.77768  -1938.48923  -2096.37927  -2273.32961   -2319.4738  -1799.08368  -1957.38504  -1519.30898  -2162.36569  -2310.57265  -2441.33877  -2619.70733   -2332.0078  -2762.30346  -2949.10869  -2472.09412  -1665.79914  -2931.56647   -1790.7944  -2293.29215  -2384.88732  -2923.84884  -2514.15688  -3355.04524   -1974.2483    -2823.397          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       20.50   -486.51302   -650.51951   -576.03661   -214.93172    -435.7111   -370.42632   -221.32457   -178.89665   -343.12228   -221.17276   -488.31025   -122.27491   -542.05913   -273.24279   -400.46967   -338.01858   -325.37399      5.79637   -173.24994    -90.54588   -408.37545   -567.71895   -449.75127   -170.92531    340.90848   1088.00071   3027.63853   3753.48967   2807.12081   1231.95383    152.80896   -446.03663   -245.70584   -138.69581   -302.87703   -280.10186   -266.37782     -44.7496   -326.52858   -173.97476   -247.26919   -347.21733   -265.36986   -271.87717   -382.28546   -378.18166   -465.98934   -421.23512   -387.41993   -404.81236   -255.19669   -402.70542   -327.76895   -286.75161   -359.76053   -299.07971   -269.49556    -53.99322   -174.62604   -148.51045   -387.57908   -279.48723   -265.95727   -262.46694   -234.22993   -376.03502     -70.3527   -270.79502   -132.10894    178.09759    865.30281   1757.22271    2124.9883   1968.30826   1024.96582    352.80103    141.43399   -103.14609   -367.95827   -260.07659    -125.6451   -245.67039   -324.06055   -345.24794    -328.3999   -229.85513   -229.29901    -12.89823    245.23131    835.64963   1807.47907   2637.60353   2645.90843   2071.05318    876.52805     55.99812     -12.0613   -255.01583   -107.70039   -304.13675    -40.20764    164.47542    274.65207     165.2535    -98.40995   -384.40099   -256.81231   -194.10996   -343.62804   -215.50328   -179.80453   -410.42104   -271.88294   -262.42444   -256.78732   -181.66353   -209.94103   -160.36457   -204.33287   -205.96832   -376.80002   -225.06992    -10.30257    129.67545    560.64594     693.5867    944.69107    893.44482    592.92361    349.73574     75.95173   -156.80692   -177.66427   -196.09996    -90.75911    -99.18312    447.88801   1209.52057   2018.95267   2705.38511   2709.21139    2124.3411   1326.42925    835.75397    380.00746    247.89382    343.07658    108.03258    -55.78439     42.30933    -91.32244   -253.68085   -180.91679     -326.551   -245.54979    -70.23468   -274.36144   -344.03154   -225.18663   -283.04724   -277.85867   -218.99829    -256.2258   -135.49977    -62.75943    -23.26855     84.84513     26.46337   -126.16671    -23.70486   -236.49815    -59.35493   -125.56218   -193.16063   -274.33855    -81.31614    275.68067     288.9807     71.51365    198.99092      19.5198   -319.30003    -122.5204    -95.64945    -78.51272   -179.60013    -129.1241   -283.00633   -316.86548   -219.37832   -269.60981   -303.86928   -193.42255   -152.55602    -87.08068    -91.66723     -34.6317     81.24922    257.98137    192.91511    237.48185     255.7446     67.64844     38.90755    -94.60179    -40.86549    -145.0131    -33.19518     95.15657     32.67433    -21.35988     71.06001    116.62647    -47.06967   -133.20321   -342.70531   -261.44689   -270.19712   -324.64865   -257.69695   -273.09955   -245.34188   -357.45659   -346.10981   -214.22704   -177.14309   -118.66072   -103.59107    -129.3087     79.41976     55.32748     78.18125      36.2753    121.24098     78.16867     10.72919     -2.27932    186.00155    327.02835     32.88757    335.00831    -17.36536     61.96051     24.97715    -55.49735   -155.75875     73.92066     13.88046   -257.11325   -274.70994   -192.91104   -272.81216    -251.6616    -33.57887    -66.09076       5.4178     -5.72318      11.0384     49.30769    -37.63098    225.06361    159.52489    217.12891    559.91384     564.9093    707.15989    604.48333    612.01951    454.57221    355.86789    339.86386    169.79941    177.28867   -176.60394    -26.45965   -273.96463   -126.32547   -143.07141   -232.87883    -90.24997     31.09369    -72.63102    -41.69628   -343.22581     74.65469    -11.61868     -98.4632   -170.87226   -137.64319   -164.92759   -214.62207   -201.06472   -186.09291   -416.36033   -413.64269   -299.53363   -393.13326    -82.52588   -420.71739   -415.74155   -456.27602   -280.72118   -317.89932   -463.42073   -293.23289    -295.4478   -296.58089   -405.16263   -168.92594   -277.85144   -260.55523   -248.06957   -219.55806   -284.50062   -187.36826   -120.44806   -116.62187   -171.79238   -287.56554   -315.12344   -310.48495    -97.53715   -244.75996   -391.80675   -471.74668   -258.33718    -231.4562   -382.84433     -360.889   -561.37997   -345.45567   -511.27705   -426.99322   -343.88807   -316.99007   -412.29831   -272.85145   -270.05909   -386.38507   -538.65459    -189.8768   -359.76472   -389.98831   -317.01691   -296.58816   -313.38739   -378.23174   -469.96434   -671.71381   -492.73909   -618.34374   -500.05947   -420.41716   -370.54106   -420.07984   -349.64036   -522.66422   -289.89861   -688.34273   -326.39292   -496.05339   -558.84248   -453.24894   -456.87741   -726.66693          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       21.00   -321.90699   -520.97063   -548.79113   -292.62975    -369.6547   -326.65811   -337.11864   -293.68402   -271.68336   -311.81305   -519.51902   -219.34835   -630.46721    -525.3451   -195.27225   -349.93959   -286.27771   -116.51264    -129.7856   -155.15657   -351.09215   -459.36087   -431.53323    -22.36046    146.32167   1277.74744   2945.33475   3720.53305   2735.29515   1259.57337    359.24512   -478.35345   -224.29244   -148.31535   -301.80912   -163.91152   -459.59707   -246.64176   -456.26058    -205.0661   -229.62945   -378.24126   -242.43687   -529.13405   -360.65441    -367.5072   -527.29175   -357.68754   -460.74424   -325.24798   -487.46297   -386.68495   -299.33899    -245.6894   -379.83812   -231.22825   -142.30506    -26.30305   -359.32372   -158.48346   -295.00597   -298.17961    -250.3062   -342.39054   -412.76467   -270.52028   -189.26243   -209.29554    -83.65481    132.58608    961.81944   1834.08037   2103.11701   1678.14784    824.97236    285.70991     -7.40748     -4.44676   -387.21883    -238.4224   -395.00458   -265.40656   -303.83546   -196.33795   -285.47107   -192.28815   -172.45367   -114.68576    218.85199    740.26033   1679.63045   2663.74438   2679.40033   1918.20846    946.61024    118.04341   -169.47873   -232.28092   -285.18533   -154.91624     20.19048    186.73305    215.72874    158.62217     -8.58774    -69.84264   -266.88586   -266.41406   -371.66864   -239.45353   -288.42191   -495.07965   -391.45837   -345.29192   -268.78948   -221.06442   -327.00967    -69.66576   -166.70558   -176.19561   -239.95272   -346.33578     -5.57108    112.36279    405.03788    939.09332   1009.04542    772.48908    716.19053    286.82875     52.95902   -101.79472   -147.82709    -108.8065    -55.78193     30.70412    517.91995   1126.43235   2180.90214   2445.43847   2757.27104   2096.75535   1390.67769    878.92156    442.54188    240.93125    190.51532    295.28291    140.44083    246.77454    -21.71095   -260.90819   -173.59012   -316.33526    -440.0329   -111.55835    -75.44782   -200.01089   -190.19114   -323.00212   -193.78418   -452.03066   -364.45493    -199.5071   -105.35428    -29.24698     -65.2414      12.2748   -149.16687   -161.83383   -106.68977   -188.37235   -201.74148   -229.30276   -113.59341   -139.36832    228.04686     34.41599    313.70656    247.40382     72.39458   -123.56242     43.05931    -74.25682   -296.95841     -8.78991    -371.4711   -171.88719   -340.87197   -304.42936   -262.62408   -171.74843   -236.23892   -145.82645   -130.05878   -140.67325    151.40309     -27.1993    170.89978    238.51379     94.45289    184.14708    176.71292     33.06733       0.7365     58.02678     52.57731     73.92086    121.12074    101.59687    144.41721    216.38432     160.7582   -113.17572   -105.59519   -154.44499   -245.34841   -276.73451   -321.85395   -271.60105   -369.29052   -146.16668   -265.52036   -257.63588   -128.44671   -105.75299    -30.46457    -70.05206     47.60705     14.04925      3.71544    174.99235     98.69846     20.48465    135.67541    133.15202     96.13098    213.57012     197.7777    215.48919    158.81781     64.59201     23.85188    -17.15048   -110.48275   -386.50659    -368.5081    -242.4708   -352.47563    -207.0718   -301.33677   -236.97912   -130.99363   -121.55129    -97.21405   -127.94792    -44.68378    236.94172    166.12344    187.92593     238.6375     250.0586    257.77954    390.85301    530.89138    530.40616    496.96468    547.18856    548.28886      527.169    252.43154    144.92471    174.59595     27.96946      -61.854   -186.55291   -136.23185      7.16402   -185.65335      71.7787   -127.14936     51.80974     92.07303    -99.30357     112.6546    123.06281    -99.65908     11.10238     -172.054   -253.75484   -166.17106   -276.79941    -282.1923   -362.40728   -388.61496   -282.82728   -268.22838   -269.97038   -380.09078   -499.60733   -413.42544   -330.69793    -305.7707   -392.52145   -262.29941   -286.18682   -321.41458   -347.85302   -149.51357   -278.97772   -284.56013   -109.52592    -207.4724   -134.24375    -76.51906   -216.17812   -192.48294   -395.64877   -180.49231    -232.1162   -190.17503   -228.36227   -285.63459   -442.45833   -396.75791   -583.14924   -510.86179    -484.5306   -474.57485   -364.36231   -477.59432   -469.12762   -342.68526   -457.69608   -385.42924   -578.77403   -491.15014   -413.96805   -550.94247   -610.68856   -589.83602          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       21.50   -132.91596   -413.37211    -490.9023   -425.56894     -313.356   -201.78987   -351.97145   -426.32921     -57.6056   -258.75763     -552.043   -379.53113    -454.0665   -424.50053   -122.74039   -364.04078   -304.70885   -265.83329    -38.87487   -468.76395   -362.11379   -338.47427    -294.2061    -68.98605    325.28017   1317.26884   2600.46229   4008.93072    2630.2376    1081.4542     36.05139   -281.04741   -222.46797    -276.8983   -264.46307   -225.47305   -347.28816   -437.87963   -293.42874   -402.79114   -369.26017   -404.92186   -315.58279   -476.27104   -144.64842   -437.67694   -472.75249   -430.96689   -559.93494   -404.65922   -434.45305   -483.27346   -285.17171   -285.37692   -379.84857    -98.70386   -349.14054    -76.22042     10.50586   -158.75921   -164.94886    -375.6989   -396.55538   -209.22077   -328.22022   -309.52256    -132.6537   -246.53534    -203.5636    105.34586    807.35026   1914.76865   2198.67677   1813.87776    970.11372    385.97686    274.62147    -53.47052   -192.30362   -148.61634   -283.63566    -62.58997   -107.20437   -105.07282    -324.6904   -142.01256    -342.7752   -129.66165    102.82298    538.18805   1723.08568   2771.79404   2600.14435   1845.14459    865.82705    271.27327    -167.0188   -140.39887   -263.38424   -136.94934     -3.79535      97.4645    163.22833    251.22261   -142.81172   -119.88843    -312.8844   -360.27805   -372.60684   -341.88932   -272.17029   -284.56803   -309.94322     -455.424   -223.70045   -236.18488    -96.59262   -169.88798   -231.36872   -181.36169   -132.94507   -265.64366    -77.04647    236.96912      642.146    916.29028   1027.65278    955.94871    639.50467    112.62795    -88.55087    -65.82645   -268.35758    -137.8381    -95.92878    118.52481    543.79365   1128.49319   1808.98193   2382.98473   2268.11651   2321.17648   1764.14251    691.80532    478.16165    200.71317    199.33383     56.47483    143.06749    -28.52714    -63.09329   -189.25364   -341.53719    -192.6908   -285.15863   -410.11016   -343.54443   -219.76717   -225.65394    -93.11107   -213.26798   -166.13466   -181.46552     23.12015     26.15623    -23.82744     40.70737     19.48906     28.46307     -141.424   -237.74434   -373.62731   -192.97939   -263.59225   -184.78801     16.11472    316.50622    185.70784     257.2585    219.99182    151.47606    -22.58751    -55.57327     -79.9605   -169.12624   -204.99915   -431.66874   -307.61879    -131.5361   -318.64894   -262.65515   -227.09678   -205.06168    -259.2432   -136.82082   -124.17198    -53.09955    359.39108     19.74145    269.59365    198.23566    188.68866     51.45389      9.15076     78.15497    -23.77198    111.61842     21.92989     11.65318        3.877     98.07232    155.02191    -27.63822     -5.88911   -237.24797   -270.01211   -254.11921   -377.69913   -381.32584   -339.00223   -243.31852   -308.84758   -315.49661   -295.33129   -152.99115    -74.85003    -92.80667   -165.20626    117.22193    112.97421       7.2432     -2.61223     20.93555     78.74197     64.33563     90.05785    153.75056     91.44481    119.77937    319.57705    186.48322     268.9054     111.8068    127.11755    -32.97056   -136.40523   -290.63328   -231.33423      9.39834   -358.14889   -226.46212   -342.68368   -242.53736   -169.29446    -113.4601     18.46117   -122.66368     -4.61155     46.12707    122.44766     156.0761    284.21261    561.23276    402.61243    421.14076    711.66714    600.01934    512.49022    578.86418    337.96353    363.66468    185.27546     85.90741   -153.35785   -176.25813   -214.78164   -219.73824   -242.58884    -83.83572   -324.69068    -97.23863    -49.75901     19.78693    -72.89301    -95.24074    -90.75232    -65.23641    -97.46372   -213.96143   -277.98438    -400.9365   -313.32143   -421.47629   -342.65532   -472.32774   -381.45707   -407.58323   -360.63286   -259.48137   -408.70337   -479.53352    -434.7372   -306.39355   -486.42414    -281.5316   -239.35574   -368.44602    -229.1774   -296.81474   -242.69849    -326.5669   -376.58389     -40.5926     -248.979   -254.55481   -105.65021    -198.0122   -353.39868   -451.83356   -280.38918      -492.08    -473.8731   -396.75282          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       22.00   -150.67136   -354.58366   -517.22213   -478.09021   -428.34735    -58.17715    -263.5646   -411.05442     -3.62854   -197.01057   -371.97334    -401.1158   -331.32986   -250.06365   -189.80096   -402.01706   -385.68756    -515.4685    -38.89366   -292.14486   -322.45529   -145.42541   -145.27742   -121.41789    505.45592   1243.92627   2896.55249   3821.86049   2701.20382   1067.33383     95.73008   -170.37231    -301.1022   -305.64025   -257.69823   -140.45211   -275.90107   -401.10084   -390.52501   -190.37545   -196.45306   -271.33524   -253.18728   -423.83924   -474.57017   -388.73604   -591.83531    -349.0025   -425.27829   -422.30384   -354.88989   -362.62508    -348.0167   -485.01002   -460.23341   -296.19343    -350.0277   -147.90116   -109.95924   -152.73104   -135.23552   -189.28492   -221.96912   -382.81622   -409.59046   -310.09401   -198.93179   -174.95212   -299.87853     90.87438    886.76967   2012.27297    2198.2005   1781.76456   1033.28058    470.94602     51.43909   -188.39085    -41.71518    -62.90099   -152.79499    -85.61423     50.28034    -88.85205   -290.38338   -212.05307    -62.97125   -209.30827    231.36872    797.76485   1708.37994   2501.42041   2527.28619    1943.1302    989.12574     325.3743    -173.7937   -320.17289   -305.24454   -198.98334     -67.0718    141.80021    145.05765     94.91942   -162.75365   -260.25878   -198.40486   -228.72326   -297.29868   -361.71767   -317.74877   -483.34571   -324.00828   -286.23413   -346.89424   -199.26812   -131.84152   -380.71317   -162.99128   -177.06205   -244.82547   -228.48838     13.49385     228.2695    515.83586    977.07614    923.95405    785.72226    554.95505    233.47464    196.80062    -13.46293    -97.15586    -111.5543   -122.29262      1.05334    588.91761   1038.52665   1777.76113   2318.40786   2577.67941   2092.23125   1383.99125    791.19872    377.82706    166.92082     75.73251     49.91567     24.80741    -81.68742    -48.88969    -312.7062   -244.56165    -292.1811   -309.51753    -216.9385   -216.13505   -278.54535   -165.07826   -227.12361   -428.36338   -158.98417   -257.17052   -227.21819    -85.48893     13.45004     16.53789     78.44444     51.56846   -249.06594   -112.27692   -262.64546   -319.81195   -241.98253   -208.19975     58.55266    268.84856    312.15601     334.5489    185.71492     157.4609   -232.22668    -56.11854    -31.56606     81.90818   -301.07585   -350.11095   -262.33367   -317.16177   -303.53422    -71.09474   -225.73263   -104.31128   -121.07522   -197.51971    -121.0291     61.30477    177.69512    208.81132    247.01895     135.7799    235.38653    153.66422     55.61955      5.69932      8.28089    -16.26485    201.24567     82.65649    -39.58043     10.01644     83.89656    -75.44747   -178.50877   -125.29672    -34.70291   -291.05933   -180.70701   -222.38327   -231.73562   -271.19013   -338.30632   -260.91999   -212.88651    -99.26482   -152.11095   -145.34381    -93.00193     18.93211     131.6746     30.67472    -65.03469     51.63572      54.8402    122.79789     169.1305    185.30642    140.92201    109.89127    310.99086    418.84285    159.34446    -13.62826    163.81491    -77.95287   -127.97272    -90.47415    -228.1997   -223.59077   -345.39006   -257.50768   -199.22378   -229.71537    -62.35092     -26.0389    -99.20622    -41.72652    -37.72615     76.96531    138.57771    128.56382    151.49057     236.2029    316.58714    343.31962     659.8367    448.03708    708.31537     391.1004    326.80189    435.02183    267.91885     40.88107      7.83963   -197.11071   -202.34048    -214.3311    -78.84714    -64.89797     -41.1231    -307.6018    -83.87952     -77.7262     20.56358     -27.6839   -249.07709   -152.71874    -181.2759   -170.86181   -330.83722   -326.36161   -412.87848   -326.20753    -218.7997   -397.17995   -391.05537   -347.74381   -427.00135   -454.78984   -275.32932   -616.86945     -415.875    -266.1983   -501.53693   -461.18074   -495.00992   -338.48128   -493.58805          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       22.50   -258.54129   -369.38371    -579.3408   -446.24672   -570.40831     23.49998   -146.21102   -373.15707   -343.98324   -246.58427   -192.75477   -377.67276   -334.80459   -315.06617   -252.50976    -225.8522   -465.31027   -463.20485   -205.21076   -145.02656   -216.30822   -259.65854     -205.257   -220.28207    280.17596   1415.58339   2972.61622   3697.25237   3208.71387   1121.64589    182.43032   -149.69421   -207.22167   -187.23931   -345.88566   -144.84489   -299.07077   -102.67475   -361.19058   -241.72116   -378.05867   -231.03471    -208.1771   -355.59338   -446.10954   -406.19817   -427.62837   -440.43527   -334.70594   -506.96473   -234.51046   -411.98423   -287.46107   -269.77033   -303.19681   -381.69723   -273.46722    122.20512    -85.40592    -70.05718   -343.95851   -377.09009   -394.81805   -377.04768   -409.23155   -271.62308    -226.3464   -245.76108   -267.98473    136.25964    718.57625   1878.07332   2256.54954   1764.86121   1086.32867    503.87584     13.19878   -162.49388   -317.35467   -178.63367    -415.5661    -293.5839      12.1265     -19.9211    -103.5209   -191.74121   -328.69257   -105.73003    -26.87161    710.94798   1744.14925   2507.84852   2842.57071     1623.363     778.3808    301.45314      -35.607   -182.74253   -161.99497    -277.6726     64.15962      151.387     223.4647     165.2551    -32.47812   -199.08891   -226.42708   -218.95838   -351.87146   -607.96685    -328.7401   -190.71579   -393.45532   -323.11961   -354.53204     -190.417   -179.82098   -137.17831    -61.26314    -97.13123   -211.24119   -270.52237    -126.8917    134.74118    645.82595    930.99856    1132.9105    745.31233    557.16504    267.43461     53.41276     208.7399      -189.85   -193.23185   -183.84816     51.35494    369.68364   1150.12679   1864.43607   2415.62319   2437.68015   2095.37489   1214.22874    812.84825    207.61732      62.4099    286.51804    134.56919     31.04248     85.97076    -96.86694   -156.33472   -229.79508   -201.09598   -347.51207   -291.05611    -323.7407   -317.55522   -244.29939    -131.6157   -165.83471   -165.05934   -283.35636    -48.28055    -64.10694     34.17019     214.7388     122.9067    -29.86232     47.81038   -271.52386   -351.51736   -385.30399   -284.69815    -49.07071     76.62728     -39.3976    211.98843    268.43266    170.74108     -35.2661   -218.04186   -126.18953    -66.16391   -299.83646   -191.61628    -51.22664   -180.82392    -283.4705   -208.26319   -357.63819   -242.82359   -263.83467   -161.14894    -319.1688       9.1213    137.99914    -49.21507     61.65956    114.23504     312.3226    246.09861    158.53632    148.05508    -34.15866    -98.24144       14.458     15.48736    132.16987    171.56353    122.46963     63.40968    -25.56431   -207.23307   -220.06764   -215.44783   -297.67331   -357.54719   -376.80046   -413.85645   -282.36923    -203.6544    -329.4548   -194.69563   -263.64199     52.38879   -164.04426   -146.60789      6.46489    -84.75906     21.41854      50.1881    104.11063    103.10646    107.24591     86.87247    101.19866    205.78435    187.01269    254.19808     72.39931     82.99611    292.08258    -16.70959   -100.55505   -290.95877   -248.03023   -347.73834   -269.83427   -266.13764   -381.75174   -198.24371   -254.61063   -260.65962   -149.67889   -170.95734   -107.37571    -44.60509     38.93505    -76.14915    171.10375    209.00378    346.62781    152.18298    393.53062    619.29989    498.45214    605.28374    499.82673    448.10301    318.02772    222.33576    -17.93301     73.65152   -234.60435    -144.4938   -219.41186    -218.9888   -146.16848    -66.37571    -67.13208     -1.63045   -100.76586   -207.16612      3.32441     -82.3862    -79.08571   -309.86654   -251.37291    -294.2196   -320.74264   -289.44935   -343.52847   -404.91963   -575.84003   -472.26291          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       23.00   -328.31746   -338.42774    -500.0522   -354.48917     -628.622     22.60237    -200.4604   -327.23126   -543.99367   -284.49033   -208.09638   -215.17057   -428.68866   -426.66564   -294.44396    -192.0364     -528.152   -417.36458   -371.60763    -65.01894   -305.00032   -287.43217   -298.82343   -250.95629     60.41002   1536.35904   3032.28158   3800.31982   2879.56096   1009.58844    211.83872   -120.07989   -434.42896   -174.97894   -289.14618   -315.10449   -272.52741   -288.68793   -335.77301   -270.31659    -43.76736   -428.13375   -335.34122   -294.65804   -355.97589   -340.29389   -280.35879   -450.30069   -301.29848    -212.0774   -335.67949   -403.54717   -377.08086   -429.05833   -348.85526   -412.31697   -311.23997   -113.10581    -47.33791   -224.33034   -338.90298   -225.76177   -411.47293   -389.98794    -280.2752   -294.54807   -324.80905   -262.37173   -130.04114    -78.92847    847.36042   1822.25462   2396.36888   1773.74803   1035.39099    268.11602   -126.28682   -275.67688   -375.99612   -239.24618   -250.53679   -261.51329   -261.60994   -372.78368   -129.72017   -185.36382   -165.84645   -282.78718     78.63386    637.53828   1793.15548   2646.76687   2552.77153   2192.83928    886.79199     213.4719   -153.66197    -73.26802   -218.72459   -150.67205    110.59963     62.39442    272.77928    113.46827     77.99183    -97.57836   -211.94294   -309.01833   -195.63378   -588.29959   -406.96391   -268.10638   -239.51849   -318.91727   -349.95253   -276.56668   -212.87563   -183.70519   -169.69431   -146.27576   -227.83054   -169.91454       2.1122     72.03228    362.29266    856.75956   1080.88444    876.90964    420.11723    193.36116     75.06125     12.53303   -139.62454   -161.03333   -210.85684      54.4469    526.54745   1021.36948   1925.18164    2439.5605   2449.18165   2207.94674    1411.2106     771.8157    246.01081    170.23811     41.27412     80.87659     40.55563     39.21324     -98.2396    -154.7337   -176.43468   -294.23379   -189.33471   -336.40402   -430.14043   -340.05874   -161.85148    -88.97043   -263.29657   -324.59064    -212.1473   -144.89708     76.28681     25.26704     43.14122     -1.10814     43.37832   -157.78725   -318.18512   -393.28617   -249.64505   -196.11049    -198.2371    -95.05052     140.2236    189.39782    219.81069     -3.84282     73.79377     95.35056   -276.38479    -51.07518   -112.56487   -170.84624   -251.26608   -410.95746   -268.87578   -229.92261    -141.1989   -356.99355   -216.95641   -248.21401   -173.04564    -26.19287   -128.64258    -99.23832    107.08843     81.27829    255.98275    205.42687    135.16718    -14.61298     36.18296    109.82793    -63.03433    -12.83977     38.23497    220.87466      74.6335     62.80899      0.61382      35.8579   -286.38294   -110.30431   -126.41961   -196.97248   -341.74452   -359.07106   -342.09547   -281.67837   -315.82692   -106.91234   -231.83432   -256.16303    -10.17467    -24.22203     70.47081     82.31554     60.49999    -51.29899     58.15918     -3.29054      45.2113     68.93326    110.52833    158.75627    250.06041     214.7031    220.63508    261.32002    112.31053     13.77585   -157.71326   -208.16867   -182.72285   -226.74353    -242.0546   -246.63456   -163.32068   -256.77276   -239.42251   -270.21899   -154.13796   -174.77805    -23.93864    -90.32198    103.26772     95.59455     88.54934    270.46479    372.52841    234.00155    553.72199    615.04175     388.3487    429.95668    430.51645    281.40095    285.21692    110.51932     11.39498    -24.28252   -108.23153   -147.58753   -249.25323   -369.82901   -323.19198   -119.98907   -189.18555   -247.36864   -391.24676   -177.03645   -372.68099          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       23.50   -286.00562   -309.73115   -384.84396   -301.01403   -593.96511    -59.71864   -381.61931   -285.27876   -396.65875   -227.31738   -274.78785   -141.74273   -581.52517   -327.02398   -351.82305   -117.16078   -512.14175   -462.66145    -183.2493    -84.68471   -366.83741    -72.82731    -429.8735   -142.58775     86.24867   1419.22772   2779.62593   3519.10234   2867.49631   1173.25013     20.84693   -156.81146   -400.02359   -342.06767   -190.47101    -54.56942   -227.53204   -457.61113   -272.84112   -296.06353   -392.05251   -249.80704   -419.09696   -434.47424   -321.75358   -368.14334   -189.59236   -426.27238    -386.1941   -316.04367   -376.90266   -349.80763   -369.16089   -397.50869   -271.48506    -282.6047   -181.28678   -264.92645   -209.96011    -89.54857   -243.81338   -281.61268    -386.2397   -243.29358   -504.97712   -415.36759   -193.77926    -223.8451   -108.18806     36.08075    948.38936   1575.18718   2445.91461   1814.06066    1041.0402     287.8083     207.1953   -257.26694   -314.19148   -392.32395    -69.79285   -173.57276   -278.19371   -317.48671   -124.15214   -126.39299   -120.52449    -58.50988    101.93469     719.4362   1937.99734   2517.86724   2624.40198    1853.0879    820.41284     71.45792   -149.74701    -59.22953   -191.64812   -137.60563    -15.80446    195.80948    232.54759    146.58073     10.89476   -375.25011   -245.73775   -290.85332   -330.80714   -328.53885   -334.30241    -228.6886   -514.20829   -264.60551   -402.72479   -285.96151   -187.99993   -102.18327   -173.91054   -432.75757   -357.60829   -281.99346   -122.23268     71.95121    504.69211     736.8071    870.11155    530.80298    409.62733     222.7073     32.56955    -89.02608    -43.52356   -165.31486   -145.89916    112.51767    371.94952    887.72119   2010.88347   2463.67821   2566.59713   2262.73672   1437.53477    687.86285    417.33576    208.65647    239.37992    265.51729     45.77692    124.90352    -84.38277   -147.82155   -174.91366   -268.49042   -230.82409   -160.92686   -237.42752   -355.39092   -325.66154    -99.15617   -228.35575   -255.41386     10.82729   -131.53605    -50.86419   -125.84622    -51.09887     66.05147   -297.18026   -118.96105   -155.26773   -233.10838   -125.39254   -154.39276    -14.28704     79.67754    162.31565    120.02731    291.24481     50.71622     98.06509    -57.48843    -55.46387   -194.95132    -29.61787   -203.49002   -308.44136    -305.2323   -328.97478   -273.63298   -266.01919   -318.34053   -147.09511   -117.94187   -159.15715   -264.27921      -6.4326      60.9764    -42.34691    144.94793    281.90351    266.63029     55.15957     22.18196     28.25021      12.4391    -47.68477     77.22593     48.35873    206.74634     102.3805    198.39962    -33.38072    -41.62317    -195.8223   -232.09821   -231.65955   -306.25556   -279.83481   -287.23579   -405.13707   -244.29155   -230.76306     -328.847   -223.09621   -181.50088    -19.11668      3.26699     31.24755    -37.68264    149.11509      3.44055     30.68749     35.24726     40.05721     37.60601    109.33076     166.0768    157.65999    266.78162    299.07822    253.61674     92.77435   -105.59448   -120.40252    -213.8255   -261.69784   -365.95333     -324.829   -211.47534   -315.84056   -321.19611   -212.73783   -212.26698   -306.01517   -151.98023    -38.94909     -7.42827     12.44632    -53.21567     80.52596    -51.65696    450.75095    282.86411    338.49413    519.25228    430.98384    313.84032    262.03034     161.5234    131.96824    -78.67176   -102.12603   -295.69999     -408.225          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       24.00   -171.68916   -322.80464   -296.93128   -335.93433   -566.85155   -139.17935   -402.19227   -348.69144   -431.85069   -265.08755   -266.25414   -213.95286   -510.75363   -262.46895    -417.4978   -167.94762   -279.28401    -367.0901   -103.95366   -256.32887   -219.73164   -175.58231   -503.84094     -26.0971    365.12618   1252.66486   3051.25503   3495.95548   2877.74274   1330.53181    -41.98958    -302.8148    -283.9493   -226.57621   -514.10069   -384.53673   -204.40893   -330.50966   -293.62473   -345.39882   -218.11588   -398.13004   -291.55188   -368.25939   -362.39043   -377.55883   -343.03298   -469.49935   -404.69885   -195.28612    -153.8261   -265.96935   -338.03766   -382.25811   -229.26396   -417.12585   -304.38251    -135.0159   -100.22885   -122.05878   -343.57154    -37.40415   -389.05987   -295.22283   -271.25075   -232.12451   -386.97794    -71.98152   -242.37135    120.28439    939.42282   1918.70388   2307.25455   1694.38601    962.32592    276.61454     68.29615    -62.70752   -129.80813     24.24034   -266.17293   -112.49061   -280.24633   -180.16208     -19.3013   -258.79109   -342.95868     19.83773    205.80693    868.23863   1837.43046     2593.673    2566.4894   1786.10363    891.55131    229.29311   -162.96664   -221.43157   -223.61741   -131.26617     71.53694     66.10841     14.88178    149.49383   -167.00528   -187.36899   -274.27468   -282.36198   -366.31224   -337.76399   -258.83919   -267.93892   -250.74215    -429.1182   -291.18641   -269.98757    -321.8584    -108.2202   -171.61223   -203.89705   -152.15205   -145.62436   -243.40367    100.53865     467.3548    687.23501    842.09947    605.51184    561.38771    227.17057      7.94056   -170.94569   -281.57381   -197.61202    -83.07901      16.4929    499.07901    1052.5332   1734.91213   2515.45207    2558.2904    2132.9449   1628.52626    751.90421    385.63464    306.49034    -13.72472    199.94247      57.2691   -136.34474   -121.62664   -197.45717     -324.428   -300.24305   -309.33478    -84.07126   -443.85806   -279.86769   -358.05006   -292.23601   -158.18542   -208.54736   -229.04695    -53.82835    -80.34335    -51.33626     81.01797    -53.80163     72.73975   -169.41913   -179.61652   -240.89147   -235.75486   -120.39517      88.5552     19.72096    243.14668    307.01542    205.57552     54.28483    -10.23926    153.60783     -71.0595   -166.07216    -37.58209     42.93534   -167.72423   -107.16151   -170.08052   -242.30985   -382.16843   -305.27741   -252.67408   -250.98613   -182.89644   -124.27172    -91.31835     86.80865    252.17168    180.01281    236.12936    307.46215    201.05399     48.76048     59.26637     -9.76164    -40.82039       0.9013     69.33562     178.1739     64.90451     57.21154     12.37464     -59.7067    -75.04385   -164.01491   -257.07155   -207.45076   -337.76336   -230.36252   -212.31205    -293.0493   -232.36208   -363.19851   -341.45016   -199.43354   -174.12328    -55.84247    -91.64131     80.94344    228.14603      2.99053    135.83809    119.54703     -72.6007    169.16679    231.46307    256.44109    223.64751    244.94539    149.36381    157.42648    101.06516    -33.12937    -88.24652     -44.2958   -160.88527   -299.96238    -271.4875   -233.08559   -350.73789   -282.53972   -265.47191   -256.51021   -317.92893    -32.90856   -135.26379   -209.19268   -125.39074     80.71256      8.96391     51.78602    149.09031    -15.53448     63.59705     75.75726          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       24.50    -66.72831   -310.09481    -324.9887   -502.78147   -472.98964   -172.35521   -382.26684   -371.99528   -566.03881   -346.61221   -300.06522   -282.58337   -522.62898   -283.17324   -252.77296   -244.23544   -114.87856    -247.2955    -248.1558   -350.45412   -151.24557   -237.86447   -381.22853   -200.88458    297.01503   1297.27287   2975.32104   3608.91308   2669.88157   1129.44799    122.51259   -365.42197   -238.28421   -110.99838   -241.16757   -389.26367   -151.19174   -223.27875   -335.55005   -591.43279    -284.3784   -352.89051   -247.22917   -342.15406   -355.42818   -387.87254   -505.88358    -447.2575   -427.82139   -383.25245   -485.03765   -217.09354   -337.42013   -442.84362   -245.73172   -381.09392   -351.87089   -160.03383   -222.43051   -168.52489   -350.10052   -198.35001   -337.30992   -369.24979   -271.01146   -430.64277   -210.37496   -315.09675   -272.46749      3.81757    858.82603   1930.87567   2054.30332   1898.09814    953.14802    332.34815    219.81098    -30.87177   -194.83675   -139.80928   -310.65207   -355.35712   -106.03726   -287.56355   -157.99201   -244.37766   -214.18301   -170.20497    175.48514    889.97723   1913.72592    2827.4463   2520.10703   1743.99193     903.8105     170.0793    -31.33973    -90.10612   -239.49894   -168.62135    -74.14133    101.74901    331.45998    160.81272    -12.41942    -10.30719    -235.3679   -221.90808   -249.92595   -342.90507   -198.80067   -270.51582   -394.53647   -267.83355   -408.52406   -436.62342   -227.33025   -173.12797    -251.8369   -213.84918   -232.03882   -121.38045   -179.45263     92.85366    369.81455    529.36447    969.65537    651.95706    514.48441      102.875   -118.01485   -190.43204   -128.83277   -213.29384    -65.76407    167.54491    516.25938   1168.89113   1945.89218   2383.55216   2558.38653   2121.87875   1396.37596    868.00646    536.65871     90.53406    247.90223    -66.76766     88.35225    -55.72495   -108.96314   -225.16851   -329.98619   -267.19992   -293.87267   -296.98566   -322.43698   -298.10209   -138.90051    -138.9573   -306.52754   -230.94222    -19.35158   -149.63283   -167.01178     -5.61033     46.00064    -99.33682      6.23768   -127.16082    -200.2288    -84.05016   -207.09743   -231.92448       9.2753     56.00675     70.17142    170.87536    370.72319      4.49032    -36.45941     -6.06006   -108.47464     -55.8329   -186.95663       -193.8   -193.26631   -191.96733   -241.53912   -362.20479   -254.66044   -358.19262   -187.21174   -320.25759   -261.98842    -89.18384     19.08189    196.88796     79.98177    349.92105     20.98412    245.95139    147.38122     35.19768      31.3089      4.00634    -15.22164    -50.15196      43.4548     87.99916     151.7332    -56.79834     93.99969    -55.04842   -193.83631   -135.61242   -322.15584    -238.0233   -264.56228   -390.31517   -433.11464   -277.22666   -355.59663   -338.29789   -326.39093   -298.43079   -263.19334   -148.31887   -143.11899     64.20114    156.09357     22.72888     65.02419    -64.04042     76.05439    172.38901    112.22823    132.30587     19.99507    217.39268    131.38482    -70.69849    -33.36163     -127.076     -9.52743   -167.60349   -264.45856   -304.10409   -214.49211   -251.58108    -377.0642   -354.90839   -280.39858   -260.85138   -262.16521   -381.06449    -388.0663          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       25.00    -24.69421   -268.26327   -396.58818   -653.39143   -283.46106   -278.24913   -313.33548   -302.73493   -506.08053   -323.53494   -393.07324   -283.98568   -523.90566   -236.86737    -35.50806   -240.01775   -132.16824   -254.90251   -371.71852   -322.01285    -68.87086   -158.01354    -205.8479   -283.24376    200.02675   1202.41905   2753.11677   3903.20175   2439.91471   1057.03545    -19.58063   -283.13018   -152.83992   -391.75244   -414.62126   -330.25149   -215.80508   -182.78285   -371.04117   -412.97046   -553.06199   -421.53782   -324.95718   -510.16736   -330.92273   -483.80947   -382.11152    -377.3425   -354.80552   -354.92588   -281.81154    -344.8394    -407.5988   -459.39498   -328.31346   -305.54989    -423.3352     53.98703     23.46258   -252.03123   -359.62161    -409.7859   -512.73224   -415.75446   -325.17023   -429.70733   -295.62029   -190.71896    -59.69969    113.14038    986.27878     1804.337   1965.56269   1626.79292    915.91545    319.18229     35.68455   -176.22858   -227.72881    -72.26155   -269.24795   -203.52212   -151.57827   -187.14241   -341.77881   -134.14183   -269.05052   -148.51341    -35.10715    894.89929   1904.62679   2394.85041   2521.17549    2001.7146    734.34341    144.01456   -112.28578   -279.79073   -221.66253    -23.67258   -152.02277    142.51349    110.69815     51.24975    -38.97515   -270.42415   -305.65331   -145.08028   -293.47314   -316.47755   -477.66308   -352.16731   -326.56698   -261.45264   -329.56134   -219.30425   -266.10877   -184.33627   -191.62571   -261.84474   -197.13583   -248.79805   -152.86592    245.43444     456.7814    768.61354    682.85597    745.22692    575.56446     50.20945     34.67398    -13.99208   -151.84099    -64.50877    -18.18106     22.89952    444.62272   1137.24849   1833.63507    2401.9619    2402.5994    2009.2419   1379.17587    884.19861    345.05596     30.67485     312.5519    121.06566     65.58459   -110.09881   -180.84613   -146.42678   -287.75949   -363.51162    -409.4629   -190.04667    -301.1325   -179.39201   -287.34901   -358.17926   -351.62513   -183.78365   -324.54995   -184.38699   -134.32507    -34.60031    -29.43246     95.36477   -118.75382     27.28044   -247.54665   -175.28482   -182.13312   -217.24498   -222.90776      32.4366      1.81349    105.91564    104.44305     72.48271     51.77944     -58.1416   -121.85985    -90.26725   -170.21755   -210.90277   -300.90863   -203.70635   -270.40274   -199.92343   -184.06031    -290.9929   -228.54793   -196.98265   -140.44943   -156.20308     -90.3895    126.40037    139.21158     221.8424    177.98625    398.46011    163.51647     42.05208    -63.25535    -25.94748   -185.21233     28.22409    105.47894     76.34052     134.2061    -14.33189     32.37277    -27.97479   -287.18582   -212.87883   -347.27966   -274.07946   -237.71201   -521.57132    -309.7686   -255.49769   -370.56449   -258.08713   -350.74731   -111.73032   -219.65794   -100.91024     42.26959   -158.68605    107.34318     -3.67221    -44.89129    -43.13655     81.70052      -4.9242     31.59313    125.17214    105.92026    234.89768     67.57372     81.82249     32.13752   -199.71989   -203.19704   -223.26367   -298.10919   -352.76065   -489.22132   -491.31313          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       25.50    -67.50737   -224.61266   -416.52889   -607.72833    -79.03636   -388.35315   -263.12853   -283.38062   -385.00046   -341.16641   -387.29632   -234.85799   -307.62296   -261.23929   -229.10599    -237.4036    -64.03859   -165.43574   -369.72656     -2.81126   -242.37547   -104.64506   -241.17148   -256.28964    268.80886   1099.83884   2831.74411   3211.51869   2389.38892    907.38519    -71.92194   -362.20467    -194.0272   -254.61593   -346.62795   -289.28281   -343.87379   -190.00302   -381.09535   -445.24771   -623.29933   -452.03969   -479.10342   -584.17071   -229.93328   -437.75456   -350.34369   -517.90998   -294.06554   -270.81014   -309.32272   -308.60265   -346.06115   -453.06149   -339.49084   -504.25272   -284.08065   -224.38476    -351.2308   -253.77429   -178.98142   -280.73254   -406.19532   -272.31775   -145.14581   -221.26438   -363.00077   -219.27501   -255.79028    147.23053    913.71517   1616.85208   2010.77066   1618.27245    954.79434    235.56736     99.22862   -335.82023   -236.34651   -239.58411   -279.80195    -87.03788    -68.37276   -166.13214   -186.35111   -252.85339    -345.5998   -204.79657    161.07355    887.73624    1593.9237   2507.25167   2379.41063   1861.34869    716.85213    345.32257   -150.60721   -218.58698   -218.76565   -136.67038     10.11891    177.64536     153.7047    270.37009   -152.33967    -98.08383   -258.15979   -230.86231    -361.7855   -258.51733   -291.81714   -313.99404    -380.3847   -161.99468   -300.22624   -145.72965   -286.68021    -94.54784   -291.55526    -163.0759   -275.98171   -193.81616   -311.08461     36.17213    415.76576    827.15106    918.05157    719.49282    477.09853    159.77005     -54.1466   -173.98735   -167.10441   -107.29089     42.21928     62.07182    355.82106    857.73196   1749.90008    2309.7395   2562.53657   2127.65535   1439.30504    950.71859    295.96651    505.43934    157.77081      101.133     46.53579     53.52552   -141.36498    -67.49368   -228.99687   -287.11568   -233.45564   -349.35367   -229.24746    -258.7419   -196.81132   -175.89974   -103.63552   -327.22641   -233.65711   -233.21046    -27.31629      16.3654     97.46579    133.31527     13.16721   -149.58509    -226.9691   -175.11509   -281.16408   -303.96467   -192.13227    -30.28077    147.05476    163.70217     140.6793    108.09717    114.90373   -120.77545   -102.08717   -135.64971    -67.34927   -206.57893   -274.75981    -254.8226   -365.85571   -342.24741   -368.97064   -262.99672   -296.32191   -214.39026   -127.93869   -217.58153       8.4143     79.23848    113.68865    229.25754    274.15435    155.30349     97.37959    -53.69473    103.04185   -120.45406     59.46211    -92.91141   -115.50896    -51.93406    -24.01851     75.82079    -63.51509    -39.94781    -31.72351   -225.11994   -263.92958   -346.46141   -313.15501   -341.71515    -425.4062   -267.95961   -468.38758     -184.839   -281.57368    -305.8287   -117.83047    -77.62141   -123.69081    -49.13016     34.31951     47.91119     31.11687   -143.25462      0.45632    126.23811     26.22919     59.34691    -55.63324     -43.2554    -50.36868   -173.21334   -334.31464          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       26.00   -135.95685   -170.22088   -341.49759   -403.57655   -112.12821   -388.06423   -273.99807   -327.82302   -335.74437   -376.30057   -434.46168   -110.75734   -279.31914   -286.14484   -314.77737   -312.19603   -135.38535   -196.79145   -319.98762   -183.39869   -171.42785    -60.93331   -331.42782    -86.01125    198.20179   1258.62144   2891.19349   3234.80054   2308.98894    852.49674   -192.14313   -525.61015   -355.33572   -302.76225   -431.62644   -288.67681   -343.85015   -431.41045   -522.21148   -393.81779   -203.55128   -377.84678   -420.10082    -468.5537   -442.86512   -352.35924   -174.78487   -356.02799   -322.46103   -274.80735   -320.94446   -467.31094   -521.74622   -222.36085   -539.60148   -395.13753   -348.45054   -226.14032   -184.81264   -254.62508    -93.61357   -246.24697   -309.87716   -384.80099   -408.99404   -351.63127   -341.01998   -347.09438    -22.42154     14.35319    796.44148    1815.8209   2305.93451   1748.45707    890.88274    320.10915    -89.88189    -104.8307   -169.25371   -330.66303   -319.60008   -248.57566    -24.37771    -380.3158   -294.47391   -180.29617     -167.776   -199.65756     -2.42901    623.72823   1551.08696   2675.94247   2542.01479   1662.02633    835.22951    250.53485     -60.2731   -324.25347    -219.8347   -146.05508    -51.61107    104.09919    180.79382   -131.16091   -104.95156   -266.74346   -258.61123   -368.71978   -387.75126    -431.2098   -383.44599   -367.13293   -442.10717   -267.78291   -163.72407   -297.15602   -143.98484   -143.31369    -170.7589   -189.57769   -277.92679    -48.78123   -178.24998     98.45384    548.48565    765.40109    732.79392    800.85565    544.96733    258.14736     18.71477   -144.34991   -146.32502   -299.10845     57.66844     31.33599    237.36689   1117.86167   1841.57661   2452.43839   2527.19293   2101.32857   1528.73779    791.23108    506.54733    171.27444     70.82196    239.33266     24.31944     22.44902   -185.16157   -250.81065   -158.64583   -255.36602   -256.80008   -196.37173   -363.69498   -291.20415   -144.66286   -280.17786    -174.5021   -151.87052   -259.59104   -149.01638    -81.27576   -104.61633   -114.82689     27.53413     -10.5934   -168.67246     11.54739    -248.0279   -164.13791   -351.31839    181.81652     74.08576     76.42757    152.70335     47.66688    125.34117    -59.42853    -54.51045   -134.96585   -173.93535    -148.0752   -191.30549   -150.56736    -54.10659   -252.45226   -282.70301    -170.2974   -361.62253   -154.47163   -135.66921   -294.15746   -218.44708    -100.3777    178.21695    163.32409     141.1901    169.71537      137.618    107.62561     98.20666   -124.65377    -53.12765    -79.42433     43.12926     -1.92835      75.5513    128.13842     45.52795     20.82208     37.40209   -232.70494   -177.59145   -423.27581   -405.65641   -508.09071   -342.28333   -416.55613   -351.95073   -185.86201   -276.91642   -239.82084   -272.07403   -119.51859    -75.98724   -158.62601    -51.03991    -95.31908    -84.89477    -49.92483    -33.83914   -269.44867   -364.37392          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       26.50   -211.27376   -145.80087   -285.67585   -293.45173   -269.70414   -275.76088   -387.33939   -350.82478   -305.70106    -365.1294   -480.90005     -126.176   -293.12898   -205.35178   -409.42292   -194.61076   -191.54052    -298.2857    -233.9995    -423.4733   -191.10568   -200.87945   -188.66829    -29.35431    193.69211   1356.59025   2780.99341   3475.68122   2234.94608    556.70795     63.76391   -225.44419   -247.01975   -280.05514   -421.82776   -253.53806   -296.95269   -300.97507   -451.29689   -541.92797   -383.80077   -331.47453   -191.76644   -287.19656   -395.23631    -459.7545   -323.00927   -314.78607   -367.40027   -428.50029   -318.03871   -321.64004   -361.22641   -331.79492   -313.11692   -494.07602   -421.17664    -184.4647   -226.56758   -293.65563   -446.66246   -215.00192    -66.66304   -323.00848   -250.60718   -262.79184   -157.00585   -320.63295   -276.84011    268.95805    830.99804   1768.06871   2112.86904   1808.96807   1015.37187    285.38251     92.96674     -27.7843    -32.59897   -197.51932   -303.08485   -198.67728   -399.81868   -404.06061   -386.32175    -232.3996   -168.87146   -207.27066    242.64353    793.78514   1731.23652   2651.06307   2403.65508   1747.77384    755.85046    153.88075    -18.20596    -24.92639   -225.19584    -58.00113   -110.96839    -72.93076     197.3442     78.24204     14.90422   -199.15882   -174.69205   -250.10958   -268.34208   -214.78076   -369.25045   -386.42363   -280.27155   -260.59326    -275.9947   -269.98131   -200.55547   -225.33191   -208.04448   -147.31957   -279.57264   -105.80323   -235.03222     53.90968    421.02007    504.15357    889.15885     898.8981    500.66908    135.68084      6.44088   -175.10287   -156.41542    -76.99786   -184.74602    149.57605    273.02421    995.52547   1797.77982   2413.08784   2496.27288   2051.07323   1302.28384    892.59305    440.87915    207.15651    187.51333     154.3735    -85.31235   -142.50239   -138.27961    -81.93583   -177.69402     -190.733   -119.20218   -218.42722    -222.1182   -267.39331   -174.70447   -174.40082   -130.47222   -154.84934   -209.93896    -96.94524   -148.55032    -45.92947    -69.24651     31.51644     22.64017   -276.02368    -272.0797   -168.53993   -195.60011   -155.75534    -50.23594     -2.27804    -56.64442    248.66637     81.16625     33.23258    -10.44198     92.63237   -175.70612   -241.16564   -135.45388   -241.11988   -261.87927   -154.56172   -210.35378   -268.04764   -228.12985   -271.01555   -275.48716   -151.32926   -218.21384   -186.69341    -77.75193    -36.65079    -76.83828    150.23006    119.62142    136.16356    147.65963     -48.4956      34.1327   -116.12751    -74.89062     -0.29353     79.20586      1.92277    -30.68532     47.21881    -67.45736     -91.3813   -126.07499   -149.87615    -372.8659   -347.45891   -445.09163   -284.78167   -293.91522   -369.89077   -403.71934   -352.49914   -331.85156   -325.85263   -240.57299   -217.28705   -171.91621   -319.90198          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       27.00   -269.53412   -174.04692    -338.9219   -301.56868   -447.31309   -187.91086   -531.31602   -372.35803   -195.73469   -477.82501   -228.99627   -255.21266   -291.61537   -169.86446   -474.70242    -84.15933   -297.42341   -277.62836   -245.24392    -472.9673   -326.03442   -224.53264    -79.01807    -29.12471    397.18034   1421.75566   2783.21377   3154.25688   2036.31331   1248.22667    104.68194   -194.68156   -251.84573   -174.71212   -276.87763   -182.06148   -311.27204   -191.34535   -510.61353   -285.60192   -333.08884   -351.23827   -339.12308   -207.81693   -299.08681   -190.09407   -427.32294   -373.36551   -354.10342   -473.68774   -263.64802   -262.55623   -248.78615   -204.94302   -326.85501   -377.79047     -52.5332    -80.46061    -180.9532   -184.85045   -381.22557   -221.26076   -377.60475   -232.27614   -485.70894   -348.95331   -361.35823   -354.46139   -261.17191    215.24441    903.36868   1494.26761   2038.20789   1775.71306    885.48102     359.4188    154.92337      -100.97    -88.61553   -163.36048   -122.58834   -204.61845   -310.23397   -469.24884   -251.85014   -222.61055   -312.16942     -0.13515     42.13042    886.14315   1877.58594   2649.17896   2449.85309   1887.93731     800.1331    207.30955   -100.64714   -247.24969   -319.54845     -94.1432    -96.91177    186.40518    253.24156   -110.76099    -33.64783   -231.39148   -202.76793   -353.76692    -335.1113   -321.72142   -354.89337   -187.87387   -441.58042    -303.2129   -333.21128   -281.79327    -159.0769   -156.05953   -236.44372    -96.89321   -182.83302   -189.29732   -177.68507    135.18997    344.05257    769.23923    976.23938    645.89084    575.80612    231.17357     53.83893    123.97307   -211.02389   -153.16531   -177.87848     96.22754     385.3881    1003.2153   1769.48463   2393.95781   2421.81867   2166.42433   1610.30145     909.7836    447.82133    238.77047    153.11189     46.13637    204.18581       4.6261    -86.34544   -315.29458   -303.17718   -276.01138   -435.92634   -402.27119   -125.53513   -307.55663   -232.87723   -205.19585   -292.44133   -238.91915   -151.15255   -132.30617   -136.54484   -141.91685     -0.92015      18.9217      0.09394    -27.45778   -237.58577   -246.24275   -338.06616   -124.69822   -109.29093    -76.35321     13.09884    121.62037    107.64006    185.15975    118.46903    -72.85354   -133.11827   -179.26849   -194.49777   -311.63568   -238.69133   -205.64766   -343.70611   -376.96821   -237.86808   -311.37799   -272.95868   -226.65387   -186.46127   -214.14505   -109.73947     30.45522    253.32656     91.64642     143.8572    122.61826      77.1546     45.82238    -15.91215    -52.77403    -96.22743     51.70893     -32.3774     99.48782      5.06354    -45.67327    -94.03415   -302.58319   -196.64479   -234.09136   -390.40358   -383.44565   -328.67258   -421.17098   -380.97859   -354.81497   -372.26599   -490.98789          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       27.50   -320.38472   -202.81442   -505.90895   -398.53527   -286.71029   -193.79328   -524.61611   -348.65055   -122.73139    -478.7179   -197.22799    -143.8113    -402.2773   -172.92222   -382.32163    -94.93879   -321.15501   -400.63984   -283.11877   -345.46626   -263.72937   -155.69925   -158.21991   -191.54645    278.82952   1302.49905   2773.11624   3239.56219   2273.45369   1201.16023      69.8678   -343.35861   -213.61763   -249.98359   -153.51065   -416.26103   -237.34661   -124.38149   -264.76964   -220.34134   -217.42183   -500.87561   -427.15105   -299.16594   -186.71557   -246.60606     -410.921    -493.4011    -343.9424   -370.04539   -296.57033   -320.00448   -302.25465   -188.57184   -253.07632   -369.38462   -340.93403   -149.70257    -33.98662   -136.77788   -253.13827   -299.39643   -386.57861   -459.17063   -452.66659   -198.07532   -362.34269    -73.25719   -154.67742    308.00603    903.10807   1854.93194   2144.03559   1732.89325    743.22573    583.02941    143.89149      -29.983    -71.08875   -133.09722   -354.24104   -245.72483   -200.68439   -214.39654   -158.20514   -160.63196    -77.31903   -255.87014     46.57095    819.04106   1798.26227   2394.07584   2387.23667   1673.17157    825.94922    -41.82687    -30.77468   -114.95826   -136.60275   -209.53567    114.96507    125.87289      70.2631    120.59009    -96.88506   -302.80064   -349.43585   -254.86895   -330.19693   -299.34932   -311.75366   -275.29458   -389.55905   -275.22182   -321.12628   -293.49366   -282.53257   -375.38717    -242.4632   -225.61798    -94.39005    -11.62432   -119.57057      -4.3866    305.57694    725.37252    868.78109    667.56839    556.96889    183.78984    -12.71531    -77.15185   -135.10216   -282.76287    -34.49178    -51.39693    391.74419    939.86424   1779.80787   2577.03176   2375.28395   2148.39945   1521.67872    892.07392    458.49008    299.88937    243.32246     86.85823     65.93777     72.43977    -112.8466   -302.25343   -198.82141    -145.5571   -433.50435   -330.91384   -234.52245   -307.21938   -253.82278   -166.12967   -208.41185   -269.43205   -305.79419   -168.63634   -209.85466    -92.67676     -20.4483    -12.83309     -7.79118   -148.54603   -187.44146   -242.60293   -248.12808   -162.03639     -55.6726    -58.99608    309.72112    297.87975      0.86922     95.40719    241.14255     -33.0082    -72.16152   -213.17834   -138.06491   -204.80126   -303.80606   -248.60208    -199.8053    -260.1308   -363.95324    -294.4315   -249.87218   -247.60137   -172.61432    -250.2043   -117.59624     31.51658    138.10705    104.70299     224.1162    148.14507    133.30148     42.59045   -142.31614    -250.6866   -347.39562   -173.49172     25.53461     77.91195     30.44598    -52.90881    196.04174   -231.69653   -146.01921   -334.78209   -428.05213   -495.51572   -392.19605          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       28.00   -351.30605   -230.97757   -755.20422   -494.67694   -107.82533   -183.41795   -315.25429   -303.40644   -267.75277   -433.67835   -326.26961   -264.51086   -395.72024   -214.95723   -233.69689     -9.94002   -215.14587   -465.67957   -293.42466   -184.58131    -20.11493   -354.96638   -296.76412   -162.90987    211.59329   1473.53725   2672.62362   3204.83668   2421.74517    1077.8729    -87.73597   -310.03708   -397.53554   -376.20932   -105.50366   -433.76284   -242.78052   -210.36555   -123.45585   -308.63244    -389.4014   -369.35851    -259.7534   -236.08093    -531.7721   -375.38559   -519.12707   -505.43464   -454.59031   -381.31873   -248.83291   -465.30578   -196.80491   -330.37526   -186.79009   -225.18346   -305.08013    -148.8216   -190.32712   -104.91626   -438.54786   -512.75606   -332.55747     -252.527   -528.13147   -328.40666   -460.96745   -336.21137   -238.96244    113.23008      951.378   1583.43035   2068.63926   1589.06055    852.29207     478.3986    173.55725     24.29235       6.9638   -267.38669   -209.97985   -201.48397   -361.52155    -220.8379   -285.60546   -267.00185   -261.57636     -95.7147    161.64022    642.05917   1695.80721   2377.72085   2486.98969   1675.03574    603.02549     196.6007   -107.07448   -336.49364   -301.55388   -347.94197    -20.58072     99.58542    165.91608    -11.82137   -146.94788   -133.48007   -162.81932   -282.07938   -211.56722    -393.4138   -445.19429   -315.11629   -234.16913     -91.3452   -342.89912   -130.52088   -201.62235    -208.3446      1.28768   -246.63309   -226.98903   -202.08784    -54.93093     49.87411    290.60455    662.27399    864.31895    834.28681    464.30996    201.20148     -6.14477   -130.89583   -302.00043   -231.32953    -96.18028       0.8955    368.48334    938.19119   1826.88553   2276.79311      2499.33   1859.52162   1316.55425    973.99891    424.69011    202.59088     13.83685    124.06397   -112.48589     44.51069   -229.93607   -183.77332   -283.16856   -400.12548   -165.99564   -279.19029    -194.5097    -57.13927   -361.23819   -195.88614   -231.73794    -289.9596   -165.91535   -173.32616    -70.12944     22.26021    -18.17205     34.60096   -163.90626    -126.1782   -166.71821   -141.70474   -355.49053   -269.68829   -155.99627   -126.34356    145.52384    145.96972    185.08577    118.02567    101.91037    -29.85857   -144.05921   -184.97942   -127.92952    -244.8773   -269.78294   -250.37926    -282.4003   -345.33608   -351.83371   -138.20136   -202.06401   -275.54029   -336.54409   -130.46676   -113.58817   -111.21754    -19.82496    147.01417    116.28826     89.85922    111.95522    -35.19772    -24.60922   -179.17928   -111.29762   -102.40351    -96.47402    -138.9823   -167.06883    -64.56564   -145.74183   -421.48482          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       28.50   -353.74683   -246.72438   -673.28524   -470.78121    -30.77008    -170.7837    -23.83721   -319.47186   -291.55805   -504.82604   -355.45547   -530.49173   -337.66752   -336.26927   -377.41392    -41.02474   -285.29302   -345.89221   -348.57702   -209.93222   -122.90703   -347.12497   -380.72798    -178.7556    293.04803    1492.4986   2664.48739   3273.98016    2202.7657   1084.13159   -268.02879   -398.91655   -270.48734   -157.44617   -143.85536   -330.96419   -298.56641   -278.91294   -200.94183   -215.39653   -317.99023   -570.58887   -373.95782   -178.95994   -406.78219   -345.25675   -396.10557   -360.07546   -566.88855   -435.87146   -403.38621   -377.04772   -323.52338   -290.37069   -348.97992   -288.07951   -271.86695    -91.07703   -310.04416   -274.97455   -152.21641   -365.64075   -408.79768   -479.62738   -377.41924   -324.56665   -291.08083   -305.51702   -300.60413    204.95382    776.20686   1781.02367   2116.48683   1596.57684    922.01171    346.99112     91.11597     26.23747   -137.72558    -56.92222   -318.04904   -147.63696   -121.86965   -462.97712   -403.46526   -317.25923   -196.77958   -145.58082    -58.01361    729.32451   1737.31677   2398.53459   2191.86933   1596.62134    771.79029     147.0673   -247.50664   -370.77687   -216.37625       102.01    105.49153    151.63935    109.78808    -66.93492   -147.46603     -92.2212   -297.53853   -236.25459    -342.2797   -253.30419   -300.75323   -340.60779   -218.34241   -281.65118   -411.36444   -320.85466   -153.64425    -18.46113   -259.87227   -198.97007   -182.85831   -214.63463   -107.65911     32.80282    134.52894    769.93125    722.44228    803.57073    520.17041     305.0317      -7.6895     21.09242   -196.14128    -77.03736   -109.75132      -1.3037    451.11491   1028.30222   1852.60586   2228.20899   2589.14779   2281.71548   1580.72839    836.98274    586.25944    163.54128     97.01919    103.40014    109.86816    -67.85971   -222.66962   -147.53071   -291.02403   -343.51489   -435.64488   -356.30955   -374.36584   -215.83002   -154.82603   -265.32738   -140.76749   -319.80926   -136.32334   -230.88302      3.71024   -112.86377   -118.33041    -16.05076   -139.73418    -29.39265    -137.0994    -92.13873   -252.33673   -293.05305   -205.28916    -51.31631     31.73311     46.99652    128.13957    137.40805     33.41265    164.15842    -28.80096    -94.10252    -37.01585    -64.88202    -386.2862    -212.5263   -302.69775   -279.70023   -224.99078   -294.24584   -376.50234   -168.14855   -238.44505     -97.8533    -65.65836   -117.67535       50.258      84.7212     93.23053     74.76984     74.22326     10.14365   -135.02305   -243.24624     -202.894   -249.23637   -305.19916          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       29.00   -323.59651   -217.13759   -458.01308   -397.10165   -128.15828   -189.86463   -137.87757   -330.54327   -165.41027   -360.59422   -362.23938   -379.02256   -388.99849   -269.78847   -396.07624   -228.04692   -346.29106   -391.62089   -348.47209   -237.81349   -314.17407   -407.45423   -241.98095    -24.38512     71.26385   1298.99438   2436.95997   3012.35128   2163.03906   1012.66207    124.52226   -247.46849   -270.25025    -264.3005   -369.97896   -353.84487   -343.35046   -378.07359   -392.50852   -253.06377   -342.35296   -214.67755   -498.26104   -214.87486   -378.36824   -258.97652    -339.7763   -289.20532   -402.35818   -251.53784   -432.17872   -215.91299   -314.74756   -542.06461   -219.77965   -270.54787   -148.56166   -124.70801    -73.34857   -245.87655    -275.2545   -208.28602   -413.93348   -389.29836   -384.46716   -245.23784   -272.17848   -176.19021   -245.22809      347.847    894.99726   1677.61919    2070.3506   1584.91463   1034.01024    442.85344     30.67177    -61.91446   -253.08027   -120.13494   -362.29885     46.59484   -304.54265   -179.85908   -507.74437   -115.17995   -180.27999    -53.94299    220.60112    850.99757   1459.63068   2233.14852   2320.12145   1840.67507    797.56385    107.06636   -146.75317   -317.09036   -387.51332   -381.53572    -31.43244     -0.59332     27.80898     86.32111   -155.09206   -283.09277   -380.02652   -362.98687   -308.72471   -396.56682   -278.72159   -420.13824   -353.37217   -138.53072   -311.58816   -258.12897   -262.81795   -224.21745   -184.49174   -113.41287   -296.43763   -160.07082    -74.71309     25.48163     212.0396    736.32938    776.59766    767.17343    641.72324    247.30094     71.88504    -56.72773    -131.5473   -185.27693   -120.00285     99.08846    424.71026   1100.17867    1961.8277   2666.79768   2935.89618   2236.33554   1593.18346    841.14146    380.35806    -51.38686     33.65172   -162.28582     67.86381   -169.93559   -152.37538   -106.62577   -198.33199   -354.49584   -410.75143   -177.65416   -101.46645   -230.87967   -390.02266    -52.63112   -293.39672   -269.49422   -252.79017    -157.8092    -69.62693     -8.20102    -25.72089    -82.93726   -138.03145   -328.37649   -156.42782   -323.55732   -293.77135   -322.98552   -258.58573    -75.32428    151.32862     142.7541    158.02631    189.20026    -10.02781    -33.68876    -81.48136    -33.76694   -215.32408   -301.67207   -224.04027   -301.27275   -374.11469   -238.29961   -325.99098   -368.61792   -231.10835    -274.6296   -239.55649   -263.05049    -77.14205    -141.7903     18.99629    -66.92189     60.05763     23.42556     -3.51631   -326.98716          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       29.50   -260.32217   -225.53142   -394.80293   -330.50752   -335.83104   -237.97996   -231.94938   -326.45463     -5.80985   -228.31539   -450.50965   -260.89115   -346.55482   -165.11168   -304.52756   -341.09071   -545.19755    -188.0456   -393.59938   -263.01486   -494.94706   -301.88973   -188.40509   -136.48928    343.64902   1420.06454   2260.28417   3148.03524   2219.54351   1145.23434    233.93122   -274.65862   -279.06067   -311.39558   -283.54349   -244.14709   -300.61837   -313.98976   -431.79768   -316.37679   -324.50091   -247.94304   -439.55145   -254.17616    -552.0199   -508.62976   -428.30144   -442.00766   -419.67488   -210.25253   -214.45423   -328.89361   -337.33295   -259.17314   -358.20279   -310.92207   -152.59767    -92.50486      2.82136   -171.85607   -210.23216   -219.42433   -469.53042   -411.18543   -345.13652   -479.56261   -359.66145   -302.60941    -67.74122    451.86243    799.14828   1821.76256   2163.06766    1762.1683    870.58607     84.63781   -120.50882   -349.80217   -133.42749   -161.80639    -85.43028   -224.87174   -447.07018   -101.15473   -312.05419   -470.74397   -117.82304    -41.75583    223.97515     873.0116   1480.14868    2482.4348    2437.4909   1756.40584    672.13093    199.87063   -133.11937   -268.51639   -192.48114   -299.64213   -163.18173    282.83302     24.23174     52.45354     -66.2292    -98.51391   -216.67068   -436.84575   -277.19335   -318.12071   -314.27273   -285.81774   -261.11754   -322.07854   -298.40804   -235.22055    -69.95513   -132.58632   -141.28892   -312.49037   -231.42665   -141.95469    -99.77412   -156.98751    320.94621    720.15641    923.03273    804.54867     482.5579    402.89245    -58.22836     -23.9462   -211.88121   -180.87513      7.04883    156.94374    526.52298   1135.88589   1741.15422   2445.98244   2542.23046   2280.48943   1553.68816    790.32366    487.49072    183.86654    -32.41639     32.91396     27.85639     59.55022    -111.4521   -148.29055   -227.32471   -167.80585   -278.34505   -349.87795   -221.36609   -341.14899   -317.05887   -243.71122   -329.95124   -129.26533   -255.65903   -214.24364     -58.8215     -67.7001   -103.42331   -101.69079    -11.91651   -104.20252   -221.22984   -230.73625   -261.95221   -130.20723   -299.78453     66.11876    174.22215    196.52453     125.1387    104.13103   -147.66823     -44.4253   -142.72913   -234.80548   -280.83313   -295.08588   -288.76709   -228.34203   -256.91681   -432.75882   -303.05275   -282.42823   -263.66679   -299.54098   -316.33577   -309.56312   -284.70842   -152.55219    -97.23803   -187.35011          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       30.00   -312.51325   -219.62627   -541.49509   -289.69805   -329.25169    -241.5374   -211.12628   -333.99418   -133.68562   -290.48061   -527.89663   -320.42184   -310.40799   -254.18366   -326.13888   -456.10997   -496.16375   -247.87242   -441.10418   -396.84225    -434.0849   -256.12392   -321.69002   -118.22323    456.21881   1332.87543   2269.37955    3044.3877   2233.51424   1064.66575    256.51674    -265.6904    -253.8273    -376.3249   -333.80789   -168.24295   -258.00205   -270.31756   -526.95362   -290.84065   -243.91406   -428.06818   -353.99924   -384.94575   -432.89659   -332.72429   -152.69071   -285.61413    -188.9319   -174.11564   -288.67007   -290.07869   -305.88973   -256.94393   -326.82631    -223.3452   -213.45061   -338.87311   -118.27713   -149.93009   -269.03399   -210.50117   -281.81174   -285.93217   -422.39817   -343.06418   -358.37147   -234.15885   -115.02223    302.63777     904.9677   1915.08962   2085.86438   1749.76199    856.01656    289.41644     51.50416     -80.4842    -163.7437   -253.83081   -550.56159   -118.62043   -224.18297   -333.81474   -221.53902    -86.86601   -227.48108   -205.45513    356.70833    899.00125   1813.81059   2507.52203   2368.92614   1771.07827    938.27174    405.91498     -50.2159   -106.45769   -390.67431    -98.39097   -107.55712    229.20774      73.6663     98.51846    -39.07252   -299.36591   -225.57565   -329.57576   -404.46681   -203.62873   -308.29054    -351.8184   -299.40005   -320.75659   -293.59188   -108.65937    -87.16966    -59.42598    -31.74888    -77.07502   -202.65557   -138.13907   -159.94057    -24.24621    363.36697     618.6506    866.24782    673.48761    520.66825    101.69633   -116.17995    -49.03324   -119.84133    -204.2794   -164.82209    105.21194     298.9595    770.47103   1450.57094   2494.50627   2481.24166   2232.07925   1548.56011     959.6123    398.08168    269.06712    147.46726      81.7779     40.10043    122.26574   -154.24177    -241.8913   -324.03028   -300.72653   -336.78354   -218.12118   -265.18817   -268.47936   -298.78865   -322.63501   -186.69112    -63.85453    -264.1706   -300.05552   -106.39485   -132.42793     42.19637     23.78259   -152.98527     -9.51482    -100.4247   -287.13656   -327.99688   -179.66252    -99.94331     19.81142      71.5181    154.83679    218.32254    -87.16906     -6.56343    -59.88448   -178.12675   -156.71164   -253.60838   -445.45456   -166.78306   -306.88326   -216.31425   -287.95574   -326.55697   -377.63767   -301.95326   -272.85453   -313.78997   -284.87849          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       30.50   -455.59412   -219.91768   -656.63019   -271.57067   -261.91761   -179.14699   -122.34602   -303.30754   -201.08295    -359.2788   -550.79723   -421.66235   -299.57955   -325.05109   -321.69312   -444.51512   -265.12034   -379.95816   -271.75446   -428.98966    -199.1724   -283.62102   -521.76146   -173.63707    217.07574   1154.15156   2479.95051   2874.52433   2385.73444   1193.03524    -11.42424    -80.93391   -267.18864   -433.03309   -425.81096   -261.48976     -320.701   -345.94125   -511.17845   -403.90825   -112.71266   -291.69569   -367.97114   -334.57491   -267.33081   -331.79651   -370.02265   -410.01299   -517.45362   -518.15593   -307.62527   -408.02845    -361.1566   -312.92795   -482.33122   -329.51193    -75.59915    -174.1787    -37.90785   -286.32002   -234.19364   -273.42817    -447.2754   -326.29021   -252.69536   -297.29927   -414.52866   -259.96803   -147.71565    416.81814    972.67911   1769.40138   2096.19505   1535.53519    962.65804    456.60847    -64.45461    -81.02989   -223.52147   -345.01529   -396.72758    -430.7758   -352.47219   -266.26586   -283.29782   -216.51803   -267.61176   -228.43726    146.07859    927.16679   1793.44454   2486.66857   2422.81243   1858.52694   1073.30192    351.99296     13.51805   -229.10516   -210.97813   -272.52179   -180.18434     79.20967     43.91088     54.17787   -181.16866   -150.78823   -258.43465   -283.47147   -235.81862   -386.23187   -245.29636   -210.19331   -332.90816   -301.81969   -144.60263   -263.01998   -126.22786   -201.78835   -281.11176   -141.83983   -123.45342   -162.78665   -122.62788    -52.59212     97.75761    543.96967    831.32579      744.134     609.7966    360.52069    161.27317    -17.70403   -260.57417   -199.04841   -211.33948    -55.40857    436.87666    919.62944    1609.6524   2173.74869   2529.49503   2298.39315    1772.2534    885.40091    688.83197    369.18707    148.81351    -11.22829     45.91537   -111.18367   -214.67438   -143.57935   -151.22184   -237.21266    -346.3878   -302.91451   -290.04806   -287.96233    -61.97998   -277.64522   -261.35487   -221.47061   -233.26517   -145.47678    -57.69201    -53.91947     43.17799     -5.86413     41.07672    -66.79024    -69.98188   -285.00738   -110.25692   -190.55766   -162.88362   -207.98876    -36.50954    158.76755    217.00883    242.02657     67.83452    -46.74049   -233.64406   -271.47607    -73.70366   -237.47096   -206.93677   -318.98247   -424.17888   -290.08063   -466.89187   -447.54133          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       31.00    -633.9937   -230.63234   -705.06717   -193.19734   -259.50152   -101.48153   -112.19891   -330.90542   -123.37752    -284.9118   -486.71265   -431.38483    -318.8527   -353.24008   -273.27173   -245.20418    -196.5859   -288.15478   -155.63467   -253.56049   -133.92083   -295.69891   -285.67501   -152.13835    263.61958    945.87974   2432.64664   2715.66156   2322.54071    936.76393     22.60448    -21.26105   -402.68633   -422.92383   -415.16376   -524.61303   -237.38226   -450.19895   -286.02726   -367.98356   -294.87604   -511.89596    -419.6216   -334.06279    -345.6255   -449.55643   -383.47935   -295.13245   -392.72692   -439.09973   -369.90485   -403.37685   -260.63487   -348.21066   -318.47073   -277.90672    -38.05317   -236.69761     -14.6743   -116.56967   -399.69727   -215.94066   -405.57399   -337.70058   -322.61082   -397.24215   -267.47671   -208.76896   -107.17281    176.35999    931.64423   1765.49994   1896.10194   1579.43331     873.9592    271.14175    284.24575   -161.02454   -275.37482   -367.67455   -358.00528   -463.05788   -276.72363   -254.85672   -435.25464   -233.82118   -302.91767   -262.04903    163.71028    808.61313   1751.95517   2422.10377   2373.22169    1556.8034   1015.01459     -29.1711    -65.15379   -225.05273   -241.85171   -218.60756   -265.57948    161.64732    144.17349    -61.77241    101.83727    -94.52231   -302.53665   -313.22332   -242.30263   -328.37739   -352.84166   -180.79147   -170.75216   -249.41576   -354.96872   -306.53706   -194.47369   -295.16697    -58.51077   -116.80169   -260.42602   -244.13273   -170.50238    -59.57205    259.11378     495.6626    790.81153    731.93373    532.22658    340.21938     75.78197    -125.6584    -72.56852    -223.5723    -75.84117   -100.39209    448.54727    852.55492   1668.76887   2266.40561    2616.4282   2129.18327   1440.04559    825.36875    358.83617    147.23309    166.13743     87.56559     -7.31082    -79.93458   -165.38413   -243.79704   -171.00425   -269.50098   -311.54696    -241.0374   -279.20098   -399.84449   -332.32013   -227.51159   -320.03244   -285.22549   -201.50693   -245.59033   -150.91346    -67.34202     57.67806    -62.68091    -31.88355   -136.31539   -106.94677   -310.25673   -134.88957   -308.86199     66.14928   -189.46698   -192.40852     69.71872     11.11106     59.82166     98.14917      -8.7694   -121.94203   -163.82565   -245.23748   -281.87563   -359.31789   -404.59361          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       31.50   -685.94305   -272.48953   -523.78596   -225.22811    -300.6513   -208.17255    -226.3866   -399.44477   -101.96879   -113.28795   -534.85539   -501.81606   -449.24115   -355.77263   -344.53742    -42.84159    -257.6311   -159.22156    -74.79304   -162.26175   -234.95869   -232.73501    -69.12395    -68.61902    400.93979   1036.67677   2338.90941   2711.14259   2051.58944    695.97901     12.25965    -62.49848   -193.92889   -310.13536   -396.25486   -212.36517   -191.50064   -413.26726   -214.52941   -344.48194   -226.49955   -295.87769   -275.54179    -471.4223   -352.91986   -232.99614   -565.24804    -87.17919   -345.56482   -453.54166   -365.64131   -471.26879   -438.35334   -436.15718   -371.54483   -356.42816   -294.14875    -35.88515   -213.85094   -230.78385   -242.32744   -322.34468   -428.06625   -327.29626   -277.99198   -304.43441   -214.33481   -268.40969   -182.66239    150.15562    781.51732    1460.3827   2033.17028    1572.3287    786.24861     94.02226    145.73975    -19.20737    -94.08076   -207.50703   -325.48852   -169.07451   -452.88105   -237.73121     71.38966   -316.38297   -258.17512   -164.98131    249.65656    707.34442   1411.90768   2288.56387   2268.94377   1639.82848    758.63364    207.68784     -8.36644   -176.09508   -125.85056   -109.87098    -14.85062     39.40068     95.60971     92.79834    -148.2141   -121.59315   -324.04649   -327.41192   -279.19101   -336.42746   -294.66283   -325.66925   -271.30584   -226.64634   -262.47228    -79.59066   -131.18824   -231.47763   -145.05525   -294.32894   -339.19257    -59.11208   -104.85778   -144.58097    237.20822     785.6485    810.79742    703.55609    407.81629    203.03002     -28.3106   -173.55644   -254.57097   -128.58772   -106.32577     -62.1445    361.98281    917.46519   1798.65543   2388.93219   2484.79484   2267.99821   1629.92426    954.68908     471.4048     320.6012     -4.07864    -42.26203    145.64057    -65.02002   -213.29469   -277.36887   -270.88971    -354.1939   -192.19366   -157.74739    -355.7964   -295.64946   -164.59665    -99.05862   -267.96396   -308.83144    -191.7508   -133.10196   -158.44852     24.49906   -107.33113     21.58729   -112.04007   -104.33642   -179.98933    -178.9724   -247.54843   -309.16948   -250.10609    -86.26572     -4.77302    -21.58775     60.86114    264.77874         9.18   -123.76632   -223.89593   -269.59131          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       32.00   -596.83016   -262.74594   -341.96768   -378.24314   -328.07536   -285.93026   -209.49156   -472.29296   -181.72211     13.79858   -659.55563   -454.48964   -406.88343   -292.27649   -380.56588   -289.70707   -231.61441   -138.13052   -303.00107    -356.7172   -384.98871    -34.74384   -275.50368    147.90915    589.99049   1325.94296   2369.18205   2737.01601   1685.79015    877.76289     112.6865   -109.25506   -205.43382   -289.55976   -321.00539   -141.48555   -315.95373   -331.28966   -398.97493    -357.9164   -534.31524   -305.86552    -225.8809   -281.51958   -498.63126   -465.16142   -489.32397   -294.35902   -217.15638   -583.20295   -289.31291   -358.04036   -421.80761   -290.50506   -425.43112    -339.6914   -399.63959   -180.70561   -194.91854    -24.64781   -203.25877   -241.70604   -341.84339   -151.09271   -271.89883   -374.88076   -232.89495   -271.22006   -158.18319    180.60252   1033.24334   1853.67063   2235.44976    1710.0133    980.63882    310.40261    125.24162     42.67019   -109.38045    -362.6633   -233.77942   -204.69195   -282.24972    -109.7342   -268.14676   -180.21734   -286.81208    -46.02805     74.07632    607.09837   1572.60383   2123.30618   2403.83004   1420.30349      849.554    269.73759     16.47968     70.16981     -4.59488   -313.60469    -56.76346    221.98319      6.39806    107.51845    -89.15812     -224.607    -371.5317   -276.29362   -342.52757    -263.6254   -383.79836   -293.81005   -315.26343   -144.41616   -169.70394   -252.32094   -315.05841   -113.69158   -194.75385   -203.62372   -186.69339    -89.54137    -134.0028    -76.40863    315.28169    678.44961    917.25066    874.61215    639.69767    344.44963     97.74075   -150.83479    -64.40782   -250.77068    -61.68412    -62.19005    389.20925    795.76196    1614.1205   2212.78252   2401.23073   2048.86716   1618.61369    970.92649    621.84517    277.77155    268.59738     50.56323     71.31934    109.58451   -109.10843   -226.96569   -312.59127   -216.74075   -269.54247   -141.61328   -372.02552   -471.27204   -206.09321   -179.51809   -302.79455   -223.09831   -212.87635   -197.67472    -93.44686     17.77837   -125.78175     12.80929      2.52598     71.47589   -112.18796    -209.4606   -278.91064   -313.50499   -215.38752   -230.24052    122.52816    -104.9325     -6.81336      3.11389   -294.80277          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       32.50   -481.10432   -199.32892    -184.7046   -642.26719   -276.44484   -287.02283   -168.26895    -333.5444   -209.73694    -125.5287   -475.45919   -219.20926   -196.95752     -275.469   -226.71639   -436.06644     18.09862   -239.19661   -463.47827   -347.62712   -379.19394    -67.07211   -307.70625    -30.90287    363.93805   1246.06256   2195.82832   2653.54014   1760.98754    933.71181    105.81514   -216.59389   -301.15443   -312.43196   -228.78984   -290.71388   -234.56359   -341.76523   -367.59612   -420.99071   -277.84535   -333.69435   -431.31341   -356.44189   -431.03744   -252.14344    -258.6228     -166.435   -385.87578   -363.49867   -312.96933   -329.24474   -395.83337   -252.43564   -487.03752   -231.66223   -325.70383     -244.223    -67.66632    -81.73603   -188.14446    -216.0873   -336.18431    -323.2264   -335.95403   -246.03133   -207.34266   -252.17806   -205.28396    267.56213     826.0819   1547.19584   1854.33277   1451.37396    861.26447    354.68577   -166.00881    -36.22985   -127.26505   -100.16878    -161.1838   -250.10561   -273.24027   -107.30856   -461.78936   -224.06649   -261.74342    -83.71222     14.31382    666.29262   1476.91588   2229.24577   2078.61317    1575.7521    782.73923    339.75163    -18.99607    -38.42406    -72.77156    -57.53962    -94.18505     12.99794    161.49303      4.31265    -96.01834   -169.37108   -284.69202    -304.6748   -264.43911   -217.16272   -255.96748   -296.04135   -292.43296   -359.75946   -325.88755   -295.50091    -201.9339   -112.84422    -84.96447   -164.68087   -130.47614   -255.56612   -208.27558    -49.50175    121.55723     594.4311    959.29068    744.73169     555.3897    328.57479     48.07483   -131.98009    -180.2645   -211.22417    -88.67089     63.42692    276.10328    695.67891   1638.83833   2165.86454   2422.13762    2046.4634   1431.12568    957.04134    506.01471    292.35778     -3.77308      4.00756     69.77945   -108.12966    -45.72118   -133.83047   -265.88764   -239.85745   -284.49885   -343.57134   -236.40444    -285.1323   -359.19037   -289.02265   -373.42183   -279.67153   -250.40644   -325.29303    -85.11812    -32.28957    -43.65634      -3.3003   -125.02823     -67.1024   -242.19393   -195.79514   -360.51032   -269.29371   -215.56898   -213.33308   -328.44118   -307.22478          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       33.00   -368.19711   -149.46176   -142.86677    -561.4129   -102.11991   -320.77543   -186.09847   -191.20577   -217.14914   -218.56604   -361.88089   -324.73854    -77.74021   -377.63981   -181.10539   -389.59324   -129.82083   -204.59862    -281.2456   -194.22353   -185.38903   -233.61656   -190.05138   -238.24249    375.58171   1137.93404   2231.46571   2521.02352    1969.1684    936.26773    150.20778   -119.40657    -310.9186   -155.67325   -179.88427   -483.34722   -177.41619   -360.25796   -525.34212   -317.83438   -404.32822   -416.50109   -280.45523   -529.91455   -550.99259   -450.20384   -265.58815   -352.96888   -233.77473   -383.38349   -388.30494   -508.88005   -454.93863   -329.78895   -429.55558      7.52791   -416.92909   -364.48646   -205.87598    -15.96438   -244.47769     26.46413   -297.22356   -198.17853   -301.23018   -359.29861   -352.56824   -206.49946   -271.52948    112.95228    867.16924   1671.33403   2251.52128   1374.92532    975.82896    364.37379     -3.23335    -12.14299     -12.1066    -153.8802   -151.47018   -248.81823    -269.5016   -339.89013   -107.16833   -200.83983   -332.18921   -196.26903    264.82726    628.06139   1343.41596   2082.23002   2196.38628   1479.43271    773.79612     39.05892     75.87715    -33.38673   -185.29162   -123.71963   -290.02216     57.08025    -20.88438    -94.64023    -52.44646   -182.09455   -257.74183   -238.37576   -320.98829   -322.69349   -259.61075   -322.14686   -385.34676   -325.06828   -416.80422   -290.98699   -234.94464   -209.60462   -141.73062   -191.45946   -219.77448   -100.97085    -86.70779    -65.39821    253.46428    446.72517    690.03173    817.09582    696.69236    236.68701    -14.77641   -126.26189   -321.06466   -275.82493   -249.35918     25.14272    305.73201    942.28622   1423.90669   2009.91334   2280.14292   2099.34785   1544.10437   1004.12264    426.41825     82.28324     92.54421    106.25039    112.60345    -23.66285     60.07116   -122.24926    -266.4271   -171.56465   -252.29378   -253.27159   -401.28107   -315.92647   -216.15142   -181.59914   -284.55147   -247.46378    -324.6616   -206.39026    -300.2534   -116.40505    -38.53643    -40.35516    -83.64074   -199.14096   -398.56134   -199.89709     -235.067   -333.10054          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       33.50   -396.71753   -210.97604    -120.4811   -420.30104     154.7266   -418.80735   -252.44495   -192.15971   -279.98119   -317.75004   -445.30306   -405.97028   -337.32223   -391.80791    -276.8869   -405.40469   -194.74008     39.96294   -224.96946   -255.05547   -236.36189   -229.43861   -284.65127    -39.18516    486.11095   1023.72975   2232.46136   2487.28639   1862.95078    914.67942    184.27411   -274.52972   -317.26185   -423.81973   -303.98524   -441.08295   -291.13661   -429.92893    -389.8956   -406.38733   -502.65304   -241.11882    -247.4731   -586.12665   -553.53028   -396.66471   -335.01166   -363.63164   -353.19181   -469.52659   -417.98166   -423.53919    -237.2577   -173.30428   -328.70006   -132.63348   -283.96594   -273.61769    -211.3291    -53.39684   -343.23332   -367.43898   -346.60603    -269.2487   -297.71852   -244.03367   -192.03485   -194.75243    -50.31544    250.79157    855.42733   1569.59458   1969.28686   1379.77778    839.99925    389.80475      2.43519   -290.69878   -265.17378   -400.83462   -378.26832    -54.49274   -281.34456    -86.47214   -261.01175   -288.64362   -463.84084   -176.18386      187.761     698.2298   1713.16035   2174.62887   1949.42905   1499.55316     631.2822     86.06999   -208.74916    -22.97998   -180.43635   -135.02039   -116.18486    -107.1237    105.13182     31.71093     30.16622    -278.0891   -184.03694   -357.23437   -478.82073   -351.54007   -334.38817   -340.41089   -239.21462   -166.34771     -222.161   -250.42418   -296.33636   -194.09488    -55.31276    -17.27756   -165.33982   -158.23958    -60.10595     64.78556    303.13054    493.72969    859.89072    805.64987    582.06571    309.69954    154.86697   -145.31052    -288.8095   -179.01373   -159.85196     13.03845    306.64395     737.5222   1429.47104   2061.90631   2514.01654   1977.40042   1647.60181   1098.75208    498.95241    241.66524    119.27402     87.97166    126.55106     20.40741   -127.16482    -80.63713   -335.96019   -244.34091   -340.66709   -244.62714   -189.49971   -251.98491    -219.4858   -215.77748   -151.62772    -269.8861   -191.70397   -273.87926   -109.73077    -42.96827   -103.89707     -142.855   -134.29769   -238.09271    -212.2983          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       34.00   -424.86447   -376.26715   -172.10475   -292.92587    296.47393   -428.37261   -385.43683   -289.86262   -374.27903   -355.54594   -378.02862   -370.55501   -466.69348   -380.13044   -288.90886   -339.96431   -258.57048   -266.23778   -222.53133   -357.53053   -244.06971   -276.87154   -318.84875    -31.93375    299.66622   1214.66971   2251.30322    2622.9408   1942.11295    717.34893    108.12799   -364.70314   -430.76332   -377.97447   -372.76042   -422.58734   -256.89567   -251.54665   -410.19063   -242.64131     -398.727   -202.01783   -435.99123    -492.9723   -375.85039   -129.12838   -465.14277   -305.65054   -451.50108   -209.04586   -277.58344   -379.17506   -280.64597   -357.27262   -341.50972   -362.02609   -315.53115   -137.05961    -31.46626   -300.78469   -300.28888   -425.85716   -286.82251   -189.57909   -137.94268   -358.26306    -91.63691   -201.65029    -58.77042    226.73631    793.09532   1593.64404    1998.6403   1517.44543    976.47525    202.35711    213.07962   -105.52195   -360.90908   -471.96949   -136.50937   -318.83995   -250.53402   -249.11964   -181.23053   -351.07034   -197.45686   -160.91638      122.333     693.6568   1334.90454   1887.24356   2143.39241   1434.95834    471.81948      15.3298      -147.58   -174.94053   -164.98873   -132.60136     -4.82208     55.02624     -5.70601    128.37235     -60.1913   -160.76438   -314.26643    -390.9852   -228.82061   -271.09673   -444.62018   -292.21357   -367.30041    -300.7082    -253.8287   -247.87853    -96.31217    -62.47619   -136.17191   -102.75212   -259.21774    -118.9173   -143.24678      2.23305    132.04175    531.03864    780.41539    715.07053    473.38209    410.68809    131.10551    -50.65738    -92.43083   -231.95017   -245.25332   -100.34696    110.32397    868.61974   1579.35344   2038.42312   2452.19439   2195.95256   1468.01825   1114.96606    566.66788     113.5499    114.06861    126.31947    -43.23764      2.97342   -262.17619   -168.93969   -154.76668   -374.92478   -312.15962   -281.83316   -293.84884   -321.38166    -220.7765    -156.6719   -271.83303   -244.31835   -365.23502   -135.96767   -197.77667   -126.28494    -95.03444   -291.92362          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       34.50   -452.63494    -465.5871   -269.72482   -286.07889     10.92244   -441.60036   -474.22363   -320.01213   -264.07466   -341.29831   -293.93414   -456.19407   -473.97485   -407.85677   -287.20519   -199.43325   -301.81077   -340.77467   -240.64076   -453.81652   -152.42513   -390.92415    -347.2827   -109.34886    306.07033   1120.39506   2237.26952   2559.73029   1825.43313    801.73499   -161.01733   -224.51046    -365.4395   -389.75428   -445.90452   -473.23586   -434.39794   -173.25591   -286.38881   -500.89274   -278.63361   -398.05367   -302.14684    -402.0298   -400.96432   -448.83876   -525.62208   -550.89462   -467.03058   -315.06753   -453.37754   -492.62074   -223.01756   -437.12743   -357.52861    151.32789   -257.35912   -103.40142    -51.98538   -188.57314    -336.3368   -291.24609   -245.70873   -450.63974   -277.65009   -181.86122   -427.49676      -210.66    -86.47834    302.53123    864.66828   1525.50375   1870.08722   1213.89065    769.88438    356.13797      2.46574   -239.36118   -282.42844   -300.51771   -207.45809   -414.24346   -334.53026   -168.47345   -404.35196   -294.91047   -108.59356   -177.08826     57.26913    633.29363   1407.64402   2134.01168   2015.92715   1528.13559    527.45677   -154.52056   -238.69466   -337.83133   -256.59885    -54.79913    -39.12905     14.56949     63.34255     96.12965     27.36668   -267.44673   -237.11424   -252.38756   -361.07044   -295.84776   -305.46592    -358.2075   -235.82141    -375.7063   -425.26398   -402.83486   -185.14181   -248.38668   -195.89177   -175.31499   -163.20282   -106.23317   -159.77284     30.82622    260.25448    570.17345    668.85017    769.12936    528.04603    235.54007    148.95564    -48.10124    -81.11849   -247.82968    -37.22589     -7.77091    228.96537    802.43867   1402.62475   2193.35034   2189.11591   2285.99243     1585.605   1191.87624    641.91339    112.59083    116.10623    -12.75054    107.68049   -164.35516   -119.72206    -85.24431   -137.61596   -281.53654   -449.64502   -288.17483   -347.01707   -291.79023   -342.81893   -245.07309   -200.41381   -262.26843   -468.01956   -319.62033   -198.12723   -315.82741          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       35.00   -406.32595   -285.05384   -356.01061   -348.40516   -235.12069   -488.61908   -430.63483   -301.03365   -134.30585   -353.95495   -302.58123   -597.74917   -406.49111   -344.85881   -322.20334    -101.7118   -282.05173   -348.52164   -296.66108   -452.64857   -120.34542    -291.4093   -532.97143   -295.08795    519.61061    1211.0862   2211.46543   2492.41474   1743.38968    954.85606     20.93651   -340.77948   -128.42592   -430.18902   -356.66324   -454.82916   -389.80876    -342.9005   -272.61611   -289.87928   -356.48571   -444.17885   -331.33274   -415.04052   -329.62988   -372.88771   -531.28626    -273.3714   -417.81501   -211.35772   -470.55014     -340.436    -167.2271   -374.37087   -329.37336    -84.37777    -81.30543    -31.12976   -225.49584    -192.2908   -189.17026    -262.7789   -403.70595   -321.25326   -447.39546   -296.86694   -250.95707    -324.9937       11.709    147.19326   1041.11433   1703.40704   2025.76625   1658.38269    775.18314     87.87357    -27.07398      -165.79   -200.53874   -124.60473   -380.43983   -206.09555   -225.27208    -146.6123   -145.10372     -221.049   -140.03139     -85.1978    247.17896    789.56286   1242.79736   1991.46571   2074.05116   1268.96935    617.93774    191.68998   -185.56435   -271.56162    -120.9171   -128.68647   -253.22113      6.20336    192.79066    -45.31796    -93.61353    -205.3226   -146.92717   -426.09228   -314.28222   -432.78028   -369.94432   -382.32288   -266.46844    -299.7571   -250.01649    -220.6839   -275.00375    -117.4118    -93.44753   -184.15456   -184.78783   -303.74391   -176.08026    136.20757    332.00445    479.87252    636.28726    935.62812    572.26368    328.62858     73.54684      2.06979   -159.27491   -126.20485     -65.5121     88.42159    211.47533    689.68432   1241.98666   1883.04959   2397.43094   2196.68537    1688.5816    950.03961    657.18772     318.1843      8.91231    150.68552    -30.83675    -59.97544    -38.30969   -104.39527   -235.39281   -223.97268   -355.58718   -316.02853   -362.10004   -310.59822   -322.88094   -236.21041   -342.02146   -329.10196   -360.67611          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       35.50   -327.74103   -160.27891   -421.99052   -381.17847   -210.40445   -417.86495   -302.59185   -256.15613    -49.59141   -395.37375   -328.33353   -591.00263   -293.16902   -243.87976   -356.56527      8.39798    -321.8444   -400.43922   -265.02791   -416.33575   -253.94611   -106.37317   -561.21746   -220.72836    355.73001   1141.22811   2057.58086   2474.82226   1848.64545    967.59302   -120.83143   -247.25725   -382.87368   -538.55977   -341.75376   -334.07172   -301.40139   -258.27011    -256.4003   -269.71084   -455.95846   -249.31739   -288.74867   -490.68526    -268.2989   -387.11465   -432.43161   -449.18966   -351.22745   -378.87255   -195.92199   -383.13922   -309.58345   -686.02102   -427.96131   -352.47962   -271.79745    -82.17275   -163.64122   -170.93072   -166.06582   -273.14431   -168.81623   -279.05466   -325.81897   -315.60967   -428.44513   -194.91673   -123.13413     278.4535    887.74153   1416.70458   1774.67364   1587.23388    735.99009    303.95297     -5.85349     90.26357   -419.18606   -234.34078   -176.39729    -245.7914   -280.53821   -174.02674   -349.67602   -360.75141   -189.02893    -26.24672    266.47533    649.96463   1413.49884   1687.86914   2045.54083   1632.58425     920.3079    258.04503    -84.10349   -200.25909     -61.1057   -135.00365    -42.96945     54.45566   -216.89182    217.51039    -119.2028   -160.58437   -278.83079   -409.48498   -217.62798   -348.67056    -375.7715   -298.49452   -265.23593   -266.95869   -257.23916   -305.44078   -160.52648    -218.5908    -48.94971    -67.84248    -198.5628   -238.02754    -45.95663    167.88194    194.79902    424.60891     743.0513    716.07977    715.43647    274.74303    -99.92298   -175.83106   -325.74734    -79.05372   -246.97048     -8.93513    284.71005     601.8208   1178.21054   1762.98932   2254.45538   2003.59766   1572.68401    973.73058    721.31182    337.61751    183.42449      40.2927    -65.95471    -65.72449    -19.09456   -123.31372   -127.68034   -313.09892   -402.39622    -338.0384   -276.67076   -403.10697   -383.88782    -446.9556          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       36.00   -302.36976    -91.80205   -373.07436   -406.22569   -107.52445   -258.46012   -286.16238   -195.07448   -224.20178   -469.69439   -308.15147   -504.12363   -187.57159   -249.99738   -406.77435     -82.4204   -408.79906    -441.1305   -189.28328   -275.98684   -322.05061   -208.37631   -320.45011   -106.06982    335.10115   1291.79401   1870.67599   2268.45783   1606.52264    812.76871      0.03403   -222.91477   -350.83635   -345.06862   -366.77192   -303.59327   -458.71981   -240.35606   -344.49512   -298.17129   -122.47065   -350.59447    -471.0358   -513.22205   -376.42629   -362.42208   -187.12719   -324.12565   -309.62809   -395.45874   -306.88736   -245.93461   -435.84842   -361.48539   -433.13347   -194.24022   -272.83948     -73.1918   -147.80518   -186.54451    -234.7306   -365.21112   -315.71375   -456.79241   -347.55011   -362.97227   -442.62168   -292.90099   -103.08416    229.05176    784.17279   1646.95452   1726.65481   1526.61752    880.45878    350.64844    142.35885   -236.04556   -344.55905   -335.35778     -1.78795    -54.35647    -205.7405   -372.42585    -317.5046   -345.52616   -361.70239    152.30557    203.75388    769.83932   1239.94926    1992.9451   2075.62515    1604.9274    735.75097    269.59922   -106.46391     -219.015   -450.28859   -124.47163    -12.46281    -68.89351   -152.09281    -60.38743    -98.39066    -98.11922   -181.45733   -217.92903   -313.72461   -214.52554   -348.68757   -322.67916   -238.39218   -256.12823   -275.66312   -256.96366   -294.59853     -96.9261     -4.02786   -149.81275   -214.87363   -240.87345   -163.17582      36.5288     98.96489    474.84003    752.55277    678.50497     342.3024     427.7582    120.06928   -137.69671   -150.80522   -214.56455    -61.60506   -153.22184    261.19705    661.52256   1417.91004   1808.25125   2385.34476   2156.84528   1368.80786   1153.80705    553.20749    162.11625     -3.26254    -40.33349    -133.3559      -4.3269   -168.88776   -137.76759   -356.55911   -274.17831   -385.76909   -277.72475   -371.71732   -407.57621          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       36.50   -311.02967   -207.57382   -221.26172   -436.63389    -83.19703   -213.04047   -360.63286   -224.32044   -318.22363    -581.2637   -477.71577    -381.6043   -260.93761   -386.24073   -390.21867   -305.82248   -566.42221   -331.96243   -288.11432   -296.58205   -367.50327   -276.29214   -283.93216   -214.66293    567.00062   1104.91871   1884.66824   2334.75437   1728.71834    711.22071    140.89965   -385.61168   -257.88356   -445.38602   -234.13717   -157.56236    -251.3936   -385.20355   -407.23165   -354.46733   -264.52843   -318.52621   -413.45909   -410.68257    -561.9553   -378.86394   -369.06841   -260.11256   -523.85328   -440.33846   -319.58198   -313.66236   -521.70189   -242.41145   -393.06465    -246.3624   -341.78492   -164.90645     48.56802     40.47844   -187.28728   -423.55205   -327.21188   -188.68876      -474.42   -318.17021   -218.85797    -297.8021    -18.12424     344.5845     932.4431   1562.29493    1743.7001   1412.83899     777.6854    315.26596    101.34154   -312.31427   -237.49275   -465.29257    -323.1238   -170.11823   -250.53932   -318.87542    -14.97037   -407.43682   -163.54578    -88.23184      59.0043    666.34194   1318.17799    1973.0654    2226.5853   1527.96868    723.33482    205.87063     -60.4885    -267.1848    -13.22193    -69.28595    -83.23121    -69.61228     64.48605     64.91857    -29.19991   -308.34186    -290.8864   -271.60578   -351.02577   -391.48271   -376.18875   -349.90475   -414.59646   -278.99662   -430.55439   -195.49727   -284.36145    -42.00989   -213.26444   -188.22608   -148.17926   -269.28206   -183.69453    -89.75205    214.33321    455.01446    635.10545    823.28566    513.17551    284.80195    177.89305    -48.52535   -237.20019   -147.50482   -142.14548     146.5018    193.78624    643.90893   1071.65531   1616.74608   2201.58758   2006.48572   1625.24013   1095.52177    533.54738    206.42253    196.38565     63.60704      3.33788   -209.90276    -85.58268   -265.20087   -300.37401   -397.04842   -308.51482          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       37.00   -195.81857   -389.88016    -54.01905   -472.45162   -104.24527   -245.72769   -367.63596   -297.96785   -247.35416   -446.27562   -468.17618   -250.83247   -175.05965   -505.43117   -230.44575   -352.83591   -692.65571   -273.38738   -337.37719   -303.99199   -396.32464   -297.44596   -487.96064   -338.08044    527.89859   1213.05594   2007.32654   2256.43073   1843.38355    653.10176    121.21321   -268.89916   -173.77993   -361.21229   -188.52879   -199.42877   -412.40992   -463.60056   -307.86257   -274.84632   -455.03669   -602.71349   -485.03038   -418.42521   -388.14302   -403.32038   -295.29793   -436.32792    -367.9717   -378.71844    -312.8071   -310.61474   -448.55424   -549.40683   -319.35009   -414.84649   -301.72487    -55.63283   -240.91692    -92.99667    -227.6709   -398.57578   -381.04308   -265.22463   -131.91909   -380.19759   -183.10308   -212.82708   -276.10066    319.13824    791.18142   1507.45948   1672.74067   1415.72476    722.52844    544.44353     11.33424    -71.93687   -199.25181   -230.38187   -292.95987   -301.72678    -80.41612   -239.24197   -207.21702   -485.24495   -209.34471    -56.85258     38.31293    509.32118   1382.74917   2199.30244   1912.25328     1345.423    859.16155    158.07649    -47.46925    -18.55762   -154.78599   -162.03243   -130.10733     19.53392     59.91664   -191.10938    -78.12714   -117.92653   -280.86211   -396.85817   -261.87508    -279.7827   -231.41834   -317.26877   -286.72639   -398.11856   -270.02133   -153.97021   -172.51231    -66.82552   -234.75932    -207.5514   -189.26083   -274.68621   -195.17645     -48.5921    156.64135     390.2634    559.69984    696.64528    450.98434    290.63503     129.8214    -122.2052   -136.70382   -250.69718   -109.87353    -32.68448    165.52641    523.93288    981.82713   1677.29852   2079.42686   2185.70358   1720.99474   1106.00131    655.32904    206.23328    242.80657     82.51786   -109.87124   -151.13001   -152.79306   -121.93841   -302.36213          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       37.50    -13.49567   -597.37111     13.70355   -422.53261   -146.41145    -284.8834   -336.70061   -361.43399   -246.31475   -423.63167   -278.50048     -200.919    -35.19378   -352.99471   -330.35484   -220.00074   -492.21687   -473.28729   -280.19114   -346.88116   -386.29469   -332.94111   -417.48728   -265.41709    490.17083   1220.29198   2130.85612   2403.90035   1809.34171    712.47928    112.73286     11.90549   -172.76481   -248.22469   -444.43329   -172.02883   -376.37312   -141.79384   -429.39174   -527.04845   -322.10689   -362.51152   -228.66304   -315.86538   -316.05979    -421.6047   -339.60283   -259.30784   -461.17344   -445.20577   -322.29727   -195.71792   -448.54759   -444.26337   -256.56359   -278.83247   -279.58716    -65.09859   -279.07047     56.85485   -123.81901   -494.46756     -393.183     53.99736   -413.05088   -379.31002   -256.44416   -275.14375   -257.47309    137.11366    867.06659   1626.16851   1507.03459    1477.0951    981.66817     308.9207     79.88407    -74.58081    -84.08527      2.13851   -130.80939   -297.06582   -290.32214   -399.77438      119.586   -318.14187   -319.35397   -187.06856     62.02467    570.17462    1054.7234   2012.14352   2091.40287   1796.76066     670.5373    283.65415    -93.07232   -133.89748    -51.74814   -232.69915    -31.38255     12.98599     -6.49374   -157.45442    -79.30764   -358.48354   -345.64056    -248.7428    -420.3946   -355.95544   -292.09041   -276.96228   -421.80621   -312.03822   -259.55056   -178.11066     -225.323   -162.62445    -91.68758   -250.11078     -13.3588   -143.75564   -313.78188    -59.90341    197.28593    305.68372    555.93986    724.27388    644.94373    345.86822     77.22722   -106.92811   -321.07667   -121.91377   -139.83468   -157.39714     67.75298     455.9536   1013.18608   1527.05222   2186.36669     2102.647   1643.26305   1180.91401    567.82458    227.51104    -77.06759    -37.71414     -9.38289   -328.97236   -305.81984          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       38.00      63.8911   -623.55748   -181.85091   -350.52717   -163.88228   -435.37183   -277.50818   -382.81848   -509.63253   -604.04568   -230.85178   -208.29661   -136.21597   -332.68416   -479.00449   -360.43281   -329.95311   -601.73205   -274.93352   -161.81307   -410.85205   -150.53754   -343.40284   -147.84795     201.0603   1328.38326   2201.21474   2151.08238   1789.55279    662.47196   -110.01934    205.39074   -300.41358   -475.26642   -507.01021   -273.78391   -473.95586   -291.81776   -269.14275   -463.85284   -364.03505   -363.38374   -315.91291   -275.77021   -344.83761   -267.80081   -452.84811   -295.32824   -348.56853   -355.61964   -339.05795   -491.71081   -370.56265   -348.16927     -397.047   -263.52259   -276.13659   -376.65801     48.28991   -133.54978   -294.32046    -264.3338   -378.96099   -414.90456   -377.73113   -301.98657   -228.51894   -202.85304   -243.16686    224.99798    879.19186   1473.44076   1664.63424   1244.57883    772.96975    272.01535     26.14797    -47.43195     41.25583    147.43526   -136.20372     -5.44865   -184.18446   -264.97298   -161.04368   -183.16026    -181.7317   -297.00178     77.69276    666.83754    1339.2628   2002.78244   1993.78432   1592.14043    938.20191    418.07554    128.93551     37.67974   -128.70194   -253.83227   -145.03085     52.62395     -8.48788     46.45266   -100.19464    -96.84874   -105.07718   -277.64684   -344.51554   -285.13315   -377.69658   -313.71387    -346.9681   -290.55761   -376.04442   -229.48731   -119.01825   -271.50465   -261.33186   -166.79276   -335.62334   -227.37343    -163.5976     18.25301    236.28308    503.70251    473.80171    840.92496    590.20924    530.11045     207.8889     -45.5147    -134.3087   -243.32856   -184.23923   -158.24117     59.47039    586.20243   1060.09034   1743.07864   2131.99291   2233.36441   1692.34014    991.45873    648.59364     93.71393    -74.29358     -47.7349   -367.45237          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       38.50    -78.06576    -481.6831   -282.87018   -255.59996   -150.79775   -540.20951   -169.12549   -400.31565   -694.69335   -455.21375   -373.09599   -234.14633   -240.91167   -452.09188   -379.56839   -609.16701   -396.64405   -516.27322   -416.55284    -18.85869   -469.13311   -239.17038   -360.91916     11.19482     74.04634    1178.4512   1750.45676   2304.56441    1822.3712    952.29504   -129.09606   -171.24041    -405.0018   -352.34474   -327.28305   -330.96834   -488.97209   -407.26544   -309.62046   -319.47139   -320.38098   -328.04515   -407.12628   -275.97047   -178.88535   -303.73043   -315.20898   -193.41563   -477.17507   -229.81656   -354.37779   -349.23161   -404.31105   -255.97943   -427.02154   -376.37107   -309.62805   -191.05444   -221.73529    -31.05246   -173.87095   -250.24769   -444.30308   -257.52489   -326.15534   -317.20482   -449.14151   -338.11113   -138.21175    329.39223    784.62993   1484.91069   1688.94541   1227.01694    649.25927     273.5294     25.53409     97.06422    108.99851    226.23306    -27.92699   -128.96717   -364.93103   -432.01946   -444.64312    -185.8407   -243.61843   -131.24448     78.96255    433.46592   1125.06603   1850.73698   1911.35729   1750.72506     909.7818    476.12321    205.73911   -201.94812   -132.20845   -197.87742    -89.93034    -29.30839    -45.55817    -73.75193   -142.07148   -354.87041   -265.05246   -267.09193   -299.78623   -387.39978   -370.36771   -169.92845   -394.95293   -398.90772   -449.03924   -176.09079   -235.71167   -126.66108   -207.30896    -97.14253   -255.37027    -97.10181   -201.42671   -193.91555     54.26835    339.92474    572.81109    667.38527    374.42231    206.40184    205.70958     -1.32997   -114.69746   -252.27484    -243.6247   -144.03573      2.34339    386.37147   1071.90667   1744.85183   2019.82971   1845.53555   1431.03779    732.03133    414.57624     57.77481   -237.52669          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       39.00   -175.14019   -327.63851   -301.88279   -264.67553   -193.11879   -586.19149     -23.1181   -479.77192   -388.74332   -142.55155   -487.85491   -361.90232   -208.31642   -439.87238   -403.36576   -450.15548   -463.31259   -711.33556    -392.4472   -274.73529    -108.5218   -348.66534   -265.97805     23.39442     57.84759   1034.57645   1595.66433   2235.95984    1863.0727    1050.4153    -76.76495   -313.76065   -291.60913   -288.95174   -377.12615   -329.91225   -348.12016   -131.50913    -293.7828   -316.21669   -397.95068   -293.73993   -407.12142   -407.99379   -253.66963   -395.66459   -425.56879   -370.31333   -297.96597   -284.75394   -432.20445   -375.99008   -304.15946   -242.98737   -395.28585   -294.65093   -231.49402      -7.7283     92.97391   -120.99173    -79.94146   -256.32424   -311.09334   -270.41927   -281.72922   -418.10778   -275.19833    -278.8859   -120.38205    281.52009    637.41885   1393.53346   1810.09458   1479.18574    725.22067     463.6212      75.1035    -95.37918    182.32421      0.49293    -19.18726     -90.0155   -170.87359   -159.72287   -126.35387   -205.62364   -310.10126    -122.0408    104.03617    511.09241   1287.72666   1911.12566    2120.7671   1505.24816   1052.65315    311.99223     99.82972    -46.04144   -339.88914   -234.68021    -97.22576    -61.19207   -144.91275    -81.44811    -64.93809   -207.86336   -333.30438   -309.52495   -189.08651   -253.11747   -400.06674   -371.38685   -332.03982   -244.26721   -341.74454   -236.90454   -141.85054    -82.68821      0.86275   -185.61252   -232.22604   -201.81316   -272.46218    -82.84921    288.80034    347.76756    601.80461    692.55908    622.59874    284.75096    -34.30789     -39.8286   -273.52539   -312.23764   -237.51588   -141.76871    -74.66479    380.19117    945.44962   1294.30314   1641.46248   1671.23694   1229.32953    533.49621    -31.26972          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       39.50   -154.22092   -247.47495   -273.98342    -317.9549   -322.41768   -506.26487   -114.18033   -615.24372    -106.7956     31.73972   -387.72741   -426.93921     -85.7803   -408.40259   -347.23927   -498.01765   -519.31132   -542.45169   -171.11397   -415.29541     -34.4873   -205.16717   -218.91611   -224.79695    401.22111    988.02913    1875.7586   2253.75641   1620.58785   1019.46439    174.54103   -392.21677   -352.70143   -277.17788   -293.69515   -380.41596   -220.48344   -166.09437   -384.73237   -356.19647   -355.18348    -83.92951   -390.07466   -387.67652   -340.65669   -357.17042   -465.63763   -324.07387   -436.25671   -512.27109   -484.69948   -392.47305   -312.48602    -319.5087   -411.36107   -358.86094   -321.75107    -80.33053      16.1194   -255.74171   -198.08833   -212.17411   -313.24615   -403.44884   -393.05161   -380.23167   -299.52435   -352.53297    113.80957    319.03864    863.70315    1332.2078   1670.76559   1118.99914    996.46197    233.13616    135.53703    -66.28168   -131.58368   -246.11832   -255.16573   -478.15807   -157.52311     -5.91468   -163.26224    -193.6886   -223.32332    -197.0462     58.41161    487.16859   1154.30976      1950.48   2109.10143   1811.21867    944.78858     442.7007     16.43174   -100.46615   -322.24205   -242.10715    -95.22267     -4.13084    -64.17325   -101.10915   -160.09715   -147.58128   -257.45986   -206.59088   -485.82705   -297.00075   -285.21018   -454.53625   -361.60339   -454.00461   -301.12681   -145.52182   -145.04471   -179.06674   -189.23208   -204.52683    -79.02161   -116.14714   -121.70887    -72.59672     59.90294    305.58209    569.36864    574.08975    421.91212      393.846    148.80528    128.21006   -191.19654   -211.97219   -249.74265   -112.65856   -122.18573     317.8175     843.8683   1271.06757    1489.3313   1032.51869    298.14343          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       40.00     -62.6512   -289.81099   -382.38977   -339.39722   -403.87104   -384.83728    -286.7513   -632.56935   -227.50791   -199.87582   -387.07743   -453.30025   -156.50633   -494.48715   -461.47395    -562.9571   -467.28604   -444.57027   -131.91117   -302.97726   -203.32533   -108.35387   -232.15786   -122.14007    628.70441   1026.74458   1965.65382   2009.60996   1502.85091     894.6183    127.79784   -262.12796   -328.45052   -368.05706   -459.96419   -451.42785   -241.58043   -156.71976   -170.94034   -484.72701   -396.23879    -103.1572   -389.86061   -449.39877   -301.08682   -339.16388   -320.90451   -410.33741    -398.4542   -410.51222   -372.52715   -410.70891   -264.05953   -380.29082    -415.4353   -329.64882    -12.90823    -210.4315    -158.3769   -250.76773   -268.78967   -206.85864   -199.16296   -376.12954   -388.95238   -330.93326   -359.36281   -178.05125   -138.35634    344.10522    751.08935   1403.50299   1580.50573    1520.4249     886.2366    330.03844    210.15838    -23.12267      47.1967   -167.04246    -56.14719   -161.66334   -305.91888   -249.76757   -167.21296   -208.95253   -364.83733   -202.59764    225.42227     441.8195    1087.0704   1996.03363   1877.70872   2017.92708   1181.03582    494.90337   -152.12248   -248.54957   -161.26068   -255.62934    -17.74234     -9.34577      65.2756    -68.51726    -162.6361   -234.51841   -182.90298   -233.24061   -239.87184   -283.96179   -311.52598   -210.57793   -333.25799   -360.89755   -174.24845   -355.43527   -165.61497   -173.76806   -149.68062   -110.32393   -245.91148   -198.73455   -154.52933     -25.4625    236.95072    434.08573    460.60709    682.43822    606.55722     436.1921      70.0024      4.86408   -246.58361   -219.46542    -234.6575   -156.27497   -216.86422    183.81221    714.71896    759.57021    388.61947          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       40.50   -267.66341   -292.05717   -570.98497   -283.18878   -325.38581   -381.94144   -451.14769   -577.74444   -353.49182   -348.11796   -336.15382   -447.71926   -411.16595   -448.22536   -590.79923   -239.68796   -374.93223   -389.34643   -207.87555    -163.3714   -201.67345   -239.62969   -322.91359   -148.42986    370.56358   1140.02001   2062.13173   1964.69071   1552.58687    776.25148     98.48664   -229.09211   -186.76577   -402.64891   -439.05697   -314.66704   -485.82636   -345.74755     -5.91432    -359.9604   -477.43993   -220.02685   -296.61352   -352.96593   -324.53861   -349.68711   -378.33819   -416.94846   -320.80741   -510.27886   -367.91746   -289.85121   -445.37024    -424.4463   -384.48298   -306.32001   -263.08381    -339.1494   -191.80379   -282.27335   -417.69486   -330.14398   -257.31756   -214.43164   -447.04533    -339.0881   -265.55924   -391.52034    -24.30422    312.42316    871.19406   1580.59002   1731.76923   1199.90301    724.64801    161.02375    -91.71442   -147.35624   -141.27303   -219.89073   -260.00034   -380.60217   -176.67885   -113.10714   -294.51954   -573.84169   -164.48597    -186.5681     77.93376    539.16971    964.20027   1711.45774   1878.35421   1771.78182    912.82273    328.59266   -223.87751   -215.95232   -437.54608   -370.11562   -156.04499    -91.22168     17.86402      -5.8377   -177.79671    -293.7141   -253.72145   -356.36603   -279.69683   -475.13621    -167.1216    -207.1674   -501.90741   -201.26786   -316.87062   -290.62457   -223.36937   -292.49341    -28.83711   -109.14586   -260.31496   -323.89743   -147.96909    -76.05716    126.59693    427.29577    724.75293    656.32151    547.14872    411.45156    -12.42452    -48.55298   -180.36524   -189.65146    -289.1207   -292.23046   -153.29409    -44.72655     81.95991          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       41.00   -452.38251   -253.90294   -601.98359   -160.30011    -180.8408   -452.84878   -495.97039   -609.78083   -389.20033   -433.45635   -227.31091    -415.5716   -335.28724   -549.85163   -509.27422    -203.4386   -465.64492   -276.84954   -282.09016   -230.33036    -55.52618   -424.67335   -229.07317     80.47751    372.37771   1064.01233   2031.64489   1833.16201   1796.17269     745.8035    341.30049   -212.88102   -225.21478   -584.17465   -293.62124   -418.80032   -357.12327   -247.26737   -294.41208   -462.71946   -477.34988   -349.32652   -457.87756   -399.53875    -311.8087   -280.86929   -339.42649   -304.19858   -403.70938   -542.90225   -553.81358   -432.96821   -332.86014   -163.76688    -404.3535   -197.56562    -201.8183   -257.91491      -91.169     38.25296   -231.32901   -147.71701   -395.01858   -336.08874   -323.08512   -219.68748    -357.0044   -502.58712     -73.3194    182.10286   1000.04455   1563.62383   1735.27889   1172.10685    664.86076    384.76818   -106.97519     -25.3461   -242.74347    -214.1198    -155.9991    -267.8116    -522.5118   -378.92782   -452.78895   -316.33313   -218.88644   -130.10569     53.53498    556.26333   1350.63219   1750.48862   1885.59873   1442.88134    880.46122    271.13734   -227.12452   -120.85536   -418.63471   -367.10928    -78.47116   -125.32293    -19.57138   -271.60758    137.83411   -311.85601   -173.10092   -322.22934   -290.45866   -368.32972   -365.57587   -309.93018   -378.52593   -328.43193   -268.79078   -233.48772   -267.69417   -138.80184   -161.20605   -138.67687   -270.85662   -214.87715    -218.9281   -239.46505    166.43906    310.19508    458.37521    625.04207    570.32321    312.28367    183.12683    -40.48806    -14.10414   -225.36917   -249.04617   -206.15706   -305.16942          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       41.50   -616.71724   -254.71631   -540.73036   -146.37772    -71.68249   -431.01229   -597.67548   -609.31155   -303.62701   -514.18484   -256.07099   -318.89522   -392.68415   -578.57119   -378.90871   -387.16706   -419.06642   -251.23832   -349.30447   -329.64072   -191.32351   -428.01813   -251.93433     21.03789     354.9938    833.13238    1716.7538   1670.56995   1631.71933    837.00835    156.22955   -388.33999   -404.39889   -566.69168   -189.90159   -260.84491   -227.17911   -116.38013   -358.42361   -445.36576   -336.07748    -212.8007   -536.86589   -359.76005   -546.62859     -272.287   -453.86166   -374.04442   -288.92931   -451.89839   -308.12473   -387.72703   -224.12773   -343.60703   -368.73731   -305.51215   -288.21496    -56.65657    -20.53611   -154.30211   -198.35903   -114.50957   -240.18299   -289.37132   -331.25717   -229.39635   -348.93384   -279.72063    -72.79657    114.99173    911.86387   1406.74475   1702.79424    1316.1599    963.65884    336.88533     -0.52767   -192.68786     -8.03973   -431.02125   -237.85105   -188.84753   -190.41328   -264.63018   -380.70429   -204.08956   -388.80038   -268.73904     97.22831    581.86851    939.54393   1562.52573   1877.75882   1380.29197     884.4265    369.46725   -193.08232   -369.87212    -443.2807   -154.52322   -228.83058    -33.74922   -185.26703    -43.95598   -201.95336   -269.40758   -153.85293   -322.84166   -276.35855   -258.61342   -407.34372   -388.77887   -323.85349   -298.86484   -322.91037   -358.37248    -126.4719   -256.55199    -261.7423   -163.44123   -132.21086   -192.93142   -297.07464   -225.34413     14.23538    249.08089    437.70368    405.19269    560.36448    379.68811      87.2615   -129.48414   -242.28311   -359.83627    -276.3022          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       42.00   -693.31338   -337.12448   -479.52258   -188.00936    -36.89527   -349.20972   -618.67683   -589.23884   -271.77416   -423.60038   -275.79679   -425.45886   -453.66715   -294.77589   -193.76934   -462.05281   -230.45715    -329.4148   -331.87324   -358.29393   -257.12962   -275.82423   -225.53532    -33.49839    350.35084    887.06073   1655.56289   1560.46046   1389.94303    602.80289    -34.67009   -291.15392   -439.99379   -379.42514   -353.11026    -68.49977   -389.03708   -352.32522   -154.90973   -457.26642   -243.27042   -347.89395   -391.64277   -403.12067   -571.81038   -383.99602   -160.50323   -450.52854   -593.18657   -555.01062   -383.41567   -248.59073     -317.044   -139.27709   -334.68652   -176.60541   -222.06675     -87.6544     -93.3242     -6.75601   -244.92775   -283.54807   -310.39511      1.63241   -289.93515   -363.73823   -495.88466   -240.93367   -167.19286    248.40774    760.37531    1507.8215   1289.85699   1419.26872    884.07399     380.4351    -66.30282   -160.30606   -228.65584      -437.21   -223.40503   -219.49371   -310.19317   -293.38622   -213.08779   -210.45362   -364.14347    -55.07638     83.43273    383.54599   1011.78989   1631.30103   1865.71584   1593.49702    723.54633    294.80347   -239.56097   -315.74626   -385.01695   -315.13514   -184.08566   -132.05434     67.57214     74.87079     14.26468   -172.57589   -374.30223   -265.51781   -345.68659   -413.51873   -342.32803   -241.60902   -271.41529   -446.11364   -510.35499   -192.00755   -319.43916   -253.90746   -170.34686   -236.04271    -91.57782   -205.88869   -116.62951     -8.71383     40.98325    213.64098    423.81357    521.97315    644.32709    318.21036    105.20285   -243.41028   -429.07229   -451.78663          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       42.50   -510.10477   -441.75004   -331.92755   -316.96048   -195.65722   -233.26703   -444.40392    -605.7371   -282.21812   -294.48046   -384.04907   -590.30854   -427.45505   -174.10231   -323.64777   -273.10993   -217.72671   -286.03857   -103.68204   -468.38201   -205.09984   -186.00093   -295.51106    134.20143     108.4394    789.15488   1628.85502   1666.24373   1243.00711    798.70447   -104.83187   -285.37672   -332.78478    -467.1216   -323.82326   -457.00142   -501.75381   -205.97889   -305.05307   -316.68153   -337.83738   -530.85019   -354.00819   -406.95116   -489.16668   -317.27565   -370.10029    -600.9282   -288.84008   -430.37152   -272.22493   -408.88025   -237.85253   -445.67778   -425.08887   -251.93638   -219.31176   -102.88451   -104.39551   -170.60781    -259.7906    -132.4518   -139.11642   -266.76579   -331.12896   -255.57261   -318.61163   -290.11455     21.13457    182.59105    762.02237   1320.56909   1562.94257   1481.25603    849.12273    273.14174      20.7003    -28.92585   -453.92075   -178.75531   -322.83213   -208.81865   -316.42245   -356.99981   -322.84084    -91.89619   -240.47864   -144.50734     -98.6983    457.60617    949.51308   1486.42688   1693.08442   1430.85733    681.15306     53.53661     75.44087   -257.81496   -545.36244    -67.68684   -232.29378   -132.55923     30.18504   -149.15499   -164.29288    -210.2538   -365.75284   -267.35136   -238.96519    -283.7131   -573.25475   -282.28935   -307.22618   -365.87121   -278.01817   -412.41125   -279.21176   -277.48723   -185.44171   -284.94141   -178.23482    -86.20001   -219.92745    -90.56041    -34.75964      288.229     286.6703    422.58983    501.84537    281.93466   -192.71075   -257.75652          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       43.00   -362.57869   -568.64391    -350.9529   -503.90072   -464.33084   -184.82618   -468.32535   -572.83894   -317.21441   -251.94301   -496.48813   -496.17585   -520.46695   -233.56314   -403.83552   -149.57883    -298.0302   -189.15131    -64.08001   -357.43459   -226.04865   -348.72297    -495.9519     47.95676    119.80105    797.13834   1630.37456    1570.8263    1026.7613    587.64862     50.88144   -188.82578   -210.69601   -633.62601   -271.07474   -364.94813   -130.09931   -403.87962   -352.79647   -542.37169    -198.1035   -291.73715   -395.23228   -574.22127   -346.09625   -441.29875   -425.68733   -220.38687   -232.21085   -327.37581   -452.46449   -536.84176   -222.13831   -308.27298   -355.66918   -269.83474   -221.56995    -46.06318     -75.0541     27.25627   -318.49129   -334.72778   -358.11568   -433.82825   -402.23041   -441.87604   -313.35055   -265.58634    127.23267     50.99504    559.80462   1531.03285   1685.44234   1369.82865   1023.44777     338.9853     45.92537   -117.75687   -166.30207   -318.48208   -295.61531   -240.23469   -313.05623   -333.62628   -302.80727      -212.91   -254.21457    -47.23032     94.51324    261.21232   1069.57073   1401.45287   1897.60316   1527.30463    695.08008    128.93041   -161.44043   -275.85488   -353.51202   -276.88407   -306.22131   -262.53114     -3.05569   -188.87493   -313.85244   -311.30272    -99.24205   -167.51355   -232.77367   -479.88678   -313.42254   -310.45066   -332.12679   -295.33069   -381.93964   -380.11411    -250.1854   -132.86046   -186.91031   -187.69604   -253.18609   -226.88846   -168.66518   -238.17524    -63.07558    309.90102    410.57048    348.96013     354.5847     43.36385   -200.20428          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       43.50    -261.4729   -482.38193    -426.7339   -585.43606   -642.62275   -175.91741   -577.81003   -492.17602   -355.85642   -327.68591   -443.23529   -337.66326   -439.55823   -201.86589   -376.42003   -140.35387   -204.24289   -233.02969   -485.01663   -443.86858   -404.41722   -340.64779   -375.03472      5.71257    281.18816    912.50625   1368.84354   1759.07751   1283.31308    801.88235    148.14534    -82.72763   -119.10435   -545.49317   -439.09972   -335.55789   -111.78163   -607.73667   -413.75153   -381.00365   -328.70514   -472.34165   -594.60811    -462.4435   -397.97829   -400.72866   -314.79331   -260.06421   -287.85598    -267.2552   -351.35415   -346.93737   -350.59139   -245.73412   -179.59683   -414.72336   -274.92542    -238.8249     23.19546   -116.64736     -77.1571    -323.7594   -312.53369    -406.0763   -285.89197   -256.01169   -237.10327   -331.35283   -205.78105    123.24678   1011.27136   1309.37608   1467.65911   1570.16383    980.55422    452.67288   -103.25477   -151.14138   -295.00831   -459.18944   -344.24555   -364.19369   -307.08351   -265.63914   -401.06966   -157.45867    -208.1604   -236.17115     161.9585    635.97381   1011.76022   1582.82021   1754.52649   1379.40171    689.86921     390.4692   -217.13678   -327.47867   -242.45279   -309.00232   -335.62706   -142.49561      -87.874     98.50552   -168.99864   -227.15502   -201.36542   -268.28269   -422.72579   -390.22191   -391.70095   -270.69077   -328.43189   -301.94793   -228.66162    -432.2968   -212.61371   -305.55222   -299.60943   -203.20454   -213.51667   -201.53353   -250.72025   -109.01365     27.20895    181.00345    275.17668    399.70199     61.33224          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       44.00   -215.19134   -393.91883   -535.56538    -476.0131   -612.01223   -169.69948   -423.96563   -427.76601    -287.4734   -525.38313   -428.09961    -491.0174   -406.17367   -175.46255   -360.50355   -185.21344   -295.31656   -249.33454   -429.44783   -402.92725   -573.70068   -375.40277   -347.92952      21.0385     244.1809    839.38931   1310.65822   1690.29329   1380.63666    686.42653     38.46415    -86.46835   -158.33316   -418.64135   -355.26494   -515.87653   -270.25389   -497.96944   -777.17254    -354.2658    -426.3126   -339.54147   -397.11718   -458.28882   -372.88972   -328.24634   -464.11277   -483.81276   -356.91805    -363.6964   -373.83952   -444.36793   -335.57482   -330.31628   -560.74745   -390.65719   -142.86121    -12.59157    -40.31334   -319.06936   -194.76647   -184.43477    -200.7221   -364.85184   -231.45459    -322.6032     -170.266   -329.12268   -263.63852    303.27037     912.1496    1266.2675   1664.65088    1549.7451    977.22187    271.97665    -90.23317   -106.49654   -203.77506   -281.05412   -433.42391   -191.70311   -600.16421   -277.71051   -171.91372   -231.04149   -177.82198   -409.07767     79.69496    338.46128     826.7059   1332.21742   1589.90207   1459.26757    868.06657    389.66932     10.43425    -92.70794   -303.68735   -197.66052    -300.3471   -149.70757    -60.67467    -32.49438    -28.86216    -22.26593   -238.28297   -315.72008   -199.23209   -447.89982   -330.17366   -351.23012   -367.55167   -345.65356   -444.90431   -340.99468   -282.54085    -73.20084   -204.61027    -64.66929   -208.85748   -357.09561   -218.80144   -156.56114   -159.18015    145.51279    -81.01648     -5.57852          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       44.50   -251.72515   -317.49712   -335.35444   -338.48666   -426.09308   -225.48288    -253.8673   -322.36374   -249.91144   -585.75993    -284.4608   -624.82151   -434.36861   -335.38735   -321.57623   -199.03215   -444.54336   -214.28477   -276.47623   -301.63742   -469.74224   -432.67895   -173.92713   -140.37756    228.26963    819.38162   1439.77144   1640.86274   1127.65121    521.86247   -124.08054   -370.89076   -253.80208   -349.14758   -495.98911   -400.25196   -407.87084   -495.55402   -333.45601   -443.44469   -471.01054     -587.063   -512.47669   -370.37132   -540.46314   -270.64778   -422.62834   -349.51185   -308.52622   -309.04197   -336.94392   -504.09668   -410.86062   -419.60389   -236.51678   -356.86764    232.13207    -45.45283    -85.51096    -50.35474   -181.06726   -144.96615   -245.75959   -325.71947    -297.1572   -422.06922   -230.15275   -156.25058     36.69371    147.85814    559.39977   1402.37288   1536.13873   1432.89396   1017.24754    518.22368     88.64912    -68.33161   -142.01853   -247.41902   -155.39683   -337.60775    -408.4727   -358.83144   -149.47853   -219.23495   -286.73731   -306.03032     60.64595    450.76798    785.86822   1372.71854   1508.40261   1347.83917    922.67368    270.38101     38.14679   -140.54859   -167.96469   -313.66653   -187.70716   -125.80507     15.45709   -172.45235    -53.77127     -38.2158   -126.38803   -313.24012   -223.63735    -300.6008   -199.24287   -343.71304   -162.13176   -402.00428   -444.62153   -260.59152   -222.30529   -165.82054   -214.06919   -162.49417   -141.78275   -333.83071   -418.79745    -47.10165   -176.99948   -229.27643          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       45.00   -258.36682    -245.8317   -113.71709   -178.78344   -264.58203   -309.64298   -118.37043   -259.02892   -279.79634   -488.42387    -39.29128   -447.24502   -242.08757   -568.75114   -131.99578   -197.68336   -443.49783    -251.9471   -304.32162    -411.9851   -293.54602   -306.26506    -201.0304   -181.67162    141.55529     977.5484    1389.2879   1511.13536   1173.11381    595.53978    -56.68402     -386.363   -315.68149   -256.16704   -423.57477   -499.28811   -179.38122   -535.49744   -559.53325   -491.40574   -397.76044    -398.8827   -406.75159   -284.25405   -462.05236   -293.81242   -462.78014   -500.21099    -499.1618   -413.44408   -385.98932   -472.04539   -376.34149   -414.27684   -479.12161   -230.98165   -258.20422    -54.79167    -34.53473     -4.09763    -66.25398   -265.71135   -322.75619   -233.20264   -230.57656   -338.61201   -328.88907   -291.09163   -266.61182    418.79413    684.61595   1218.28718   1488.42934   1544.74155    918.88299    449.33581    184.43531    -32.46602   -380.04375   -205.13307   -100.04436    -49.89144   -153.84111   -316.91195   -354.38798   -284.74125   -273.22461   -242.06086    126.28803    362.16665   1058.98889     1636.006    1568.9449   1561.28449    881.24794    343.94507     49.18032   -238.55426   -213.77708   -300.56334    -286.7313   -146.42797     26.12298     -78.9225   -233.54417   -179.07308   -316.44342   -262.45299   -295.77276   -363.53025   -340.32776   -300.54911   -392.16742   -430.13423   -358.21148   -337.69561   -316.36917   -185.77227   -253.07027   -223.79552   -170.18265    -400.4025   -238.85953   -393.55699   -303.98265          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       45.50   -218.29923   -285.65591     24.04393   -167.52815   -155.37495   -301.68129    -71.39864   -269.39595   -378.04549   -608.56772    -14.63814    -325.1651   -235.24812   -444.53255      3.25525   -395.58244   -448.28094   -342.95416   -447.82032   -554.99468   -275.31504   -205.41271   -204.45396   -210.42664    208.23427    969.95525    1490.5496   1501.42645    1191.5677    496.35788     -34.0453   -356.70642   -266.82707   -144.43154   -462.82295   -421.59988    -80.47804   -404.94781   -485.62534   -483.86308   -507.13882   -548.50195   -482.87048   -310.15632   -324.98431   -417.09788   -406.56635    -572.7527   -429.29325   -304.41062    -382.8694   -408.80126   -368.71975   -368.13968   -454.59341   -257.32044   -171.82418   -181.06228    -47.23127   -155.34436    -36.39974   -281.98106   -368.16726   -362.63789    -167.8553   -272.86488   -305.09793   -139.64566    -125.1284    409.19452     752.5035   1465.46695   1521.78538   1473.15194    836.12876    520.11727    -49.11343   -137.24076   -146.46483   -247.75707   -393.62872     -163.783   -390.60743   -201.17089   -344.96606   -391.30183   -240.12124   -120.30898     25.04285    361.46264    799.07309   1549.72614   1495.75103   1349.35505     923.5699      664.438     46.69725    140.83637   -256.84553   -262.03812   -206.48668   -154.56307   -111.85661   -174.89224   -247.71768   -246.33716   -314.67534   -275.55481   -284.30562   -291.61347    -457.7453   -345.51513   -322.00977    -293.1292     -353.537   -285.61648   -311.98535   -187.01818   -235.48422   -187.18373   -322.23842   -341.01086   -451.06937          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       46.00   -161.82277   -334.18739     45.95922   -246.76716   -208.62323   -261.48822   -179.88743   -272.60776   -373.00971   -499.62525      3.91782   -312.14807   -337.72579   -574.41887   -219.30945   -536.45894   -353.58511   -468.04242   -262.75681   -379.67285   -102.71297   -344.83974   -209.43578    -154.4451    261.08967    699.86972    1360.2866   1557.38789   1270.81783    416.72244     69.81265   -298.05672   -143.39168   -339.02517   -401.06417   -314.81828    -450.0724   -268.90074   -479.36368   -429.57891   -482.69656   -598.63536   -391.16996    -408.7909   -258.92098   -225.01849   -422.08741   -249.88237   -442.06099   -465.27162   -388.06198   -296.51068   -385.42416    -465.2432   -318.77689   -397.17627    -307.4626    -82.79334     59.66541   -286.42974   -326.66513    -189.1126   -343.67899   -347.47016   -327.15287   -187.56109   -238.93368   -127.11953      -8.3521    152.02952     712.7882     961.4327   1596.45025   1602.66481    747.23264    215.94819   -169.02427    -76.87631   -293.11701   -244.95525   -448.34517   -509.71934   -136.05889   -222.26477    -27.55757   -349.48177   -326.42068   -182.77218     97.48197    307.39795    925.20063   1362.75901   1616.66507   1487.12834   1014.15563    357.48288    258.49104    -55.34183   -258.23932   -175.81466   -243.47874   -186.99381    -28.08829   -124.23058     29.55679   -124.96839   -346.65502   -286.39252   -352.39112   -471.46662   -273.45086   -304.28335   -330.01118   -301.35985   -404.04253   -475.19204   -323.49012    -294.0896   -179.82178   -304.00564   -109.35055   -339.84962          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       46.50    -193.4129   -391.41699   -123.27387   -334.27528    -276.4245   -247.21335   -410.87007   -266.40876   -278.53487   -341.10327    -19.31031   -361.85012   -354.45461   -572.75589   -242.65966    -430.7398   -288.14941   -479.64577   -221.89476   -339.87156   -289.03916   -425.33481    -231.4093    -73.55399    325.94567    665.42281   1169.71328   1273.87998   1150.93931     720.0184    154.53852    -362.1431   -297.66109    -317.7367   -261.78219   -215.83039   -336.26562     -1.91598   -365.99267    -341.8485   -523.20342   -408.90124   -535.45619   -566.25972   -515.62398   -426.18435   -333.73733   -216.71904   -302.50834   -399.47191   -264.00549   -573.19246    -424.0645   -376.58638   -162.94148    -251.8403   -216.03082   -236.19861    -50.32712   -132.38746    -45.23765   -341.09992   -125.71504   -318.09573   -374.22254   -261.06901   -338.58738   -196.73673     -93.2013     90.42604    802.88172    1237.3442   1367.09098   1481.80469    922.88058    469.04108     82.67489   -147.46591   -209.15726   -364.74482   -176.43672   -125.88735    -27.46628   -136.34202   -211.39815   -468.13274   -221.37993   -179.42657    104.49926    270.90527    848.66612   1468.06647   1513.06168   1446.11822   1132.51613      364.646    -12.92902   -328.68195   -252.44136   -268.74135   -346.21205    -52.48914    -146.4049   -318.77161   -263.83422   -178.51849   -252.07636   -307.06074   -368.49427   -356.31712   -212.14739   -339.03593   -344.98711   -360.39879   -346.77003   -342.61329   -446.26493   -292.11235   -231.69823   -331.53241   -442.82996          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       47.00   -273.11067   -439.43384   -257.10135   -399.15156   -421.10123   -290.52928   -455.87724   -328.79001   -323.43895   -319.98186   -147.96537   -411.76758   -334.38554   -255.08534   -194.92906   -449.32739   -332.82139   -370.32933   -219.49334   -367.53021   -243.55785   -376.19035   -286.78495   -289.36583    261.57961    722.23318   1172.04329   1256.66952    1394.5122    570.19984    147.03084   -168.98224   -262.26868   -244.22603   -304.84863   -273.39631   -453.79196   -331.73215   -167.22195   -397.61394   -475.93208   -238.50908   -462.65263   -351.52344   -483.88004   -368.18169   -315.10831   -326.27514    -95.75964   -356.88696   -544.15305   -241.71108    -330.7193   -566.34417   -324.49457   -462.18057   -226.14605   -254.02189   -225.20391   -116.50327    -79.47947   -232.88563   -121.53429   -143.60219   -312.74612    -35.19357   -321.49325   -365.57805    -83.40665    136.69109    796.98693   1173.29509   1619.54617   1209.42721    879.44056    396.59789     78.14891    -80.93137   -242.56343   -119.43012    -282.0775   -141.16119   -210.22536   -223.87586   -284.94695    -223.3308    -300.1322   -267.12367    135.11239    506.65289     854.3391    1244.8777   1556.06237   1357.03346    962.98779    603.70848    -213.2238   -183.05814   -300.54201   -317.98243   -151.79419   -177.19243    -52.33417    -90.23668   -119.42505    -96.97128   -224.57205   -337.41637   -346.22761   -311.98167   -369.83665   -239.83071   -400.67191   -363.34844    -285.3599   -233.18914   -279.60515    -133.3274   -303.39719   -484.51921          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       47.50    -383.7116   -407.54547   -350.49442   -433.83593   -588.13324   -375.14707   -390.09356   -273.06342   -534.23501   -309.49805   -296.76754   -507.21679   -338.47677     -7.53978   -319.87789   -474.06046   -287.43187   -377.23045    -207.4853   -511.32349   -273.64106   -349.67207   -270.95033   -240.47382    148.56142    893.93036   1225.36978   1177.17449   1121.09238    427.15119     -38.8973   -138.76078   -341.92344   -387.81193   -240.69871   -281.77411   -321.04879   -333.01781   -502.13225    -439.0984   -428.62534   -227.72712   -323.04546   -368.72387   -360.96461   -509.76929   -489.29091   -383.24402   -488.20539   -283.11718   -571.20637   -374.94927   -336.41614   -547.42823   -347.55171   -444.18625   -248.33877   -156.04954      66.6702   -146.09168   -355.51044   -291.39167   -257.85239   -241.69404   -406.83926   -154.39006   -291.77485     -229.361    -26.41572     85.64417    870.24278   1290.94084    1544.8653   1299.39136    899.36056     427.1283    -41.76281    -96.01674   -425.49635    -80.57433   -266.56054   -255.08759   -487.76573   -275.51559   -465.62998   -212.72071   -428.19376     -174.735     45.40942    399.08361    872.39179    1386.8026   1403.00227   1421.39723    830.32223     498.5929    -97.41724   -125.92193   -276.27448    -103.7093   -149.29774   -214.61051   -218.06147   -141.19541    -85.55906    -40.46148   -215.93568   -208.30873   -324.97797   -302.54131   -306.95597    -375.5869   -364.03586   -423.47475   -364.25889   -274.20357   -250.77859   -358.11772          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       48.00    -486.4594   -347.26535   -397.22042   -510.59819   -734.28797   -443.11586   -304.37124   -151.15879   -658.50696   -150.96696   -426.30356   -456.25842    -345.0068   -299.33809   -456.76966   -362.61955   -201.91638   -363.59153   -203.43457   -458.40372   -367.80399    -291.6647   -166.51885    -41.67251    169.93122    784.57452    1342.9363   1298.55618   1037.71176    480.82738    -22.84813   -162.06546   -314.33643   -279.45012   -392.13692   -410.37857   -253.30732    -46.60029   -444.45489   -424.27011   -205.71725   -439.51021   -446.78689   -451.55396   -394.08071   -601.21373   -441.10287   -331.21454   -457.14002   -431.62107   -462.17623   -285.24755   -366.38705   -544.08544   -273.14805   -362.05303   -247.73382   -146.45905    -96.65036   -171.44049   -142.58972   -224.71502   -417.57969   -135.89986   -357.71322    -246.0449   -357.37197   -259.52478    -19.10892    159.12831    477.54538   1209.58615   1471.12641   1198.83146    907.29127    514.84009     43.12479   -308.11304   -219.36447   -193.17481   -316.54788   -131.66395   -157.51499   -523.51113   -220.69719   -340.68784   -286.52457   -165.40759    -64.67746    321.25085    565.36123   1337.96909   1534.87204   1226.70026    955.57932    379.22735    -50.82458   -338.31942   -346.04912    -66.65198   -421.51305   -286.78852   -106.97985    -197.0204    144.77552   -255.66828   -421.96225   -297.84635   -167.82516   -326.85974   -397.27739   -310.70634   -424.60815   -264.53928   -311.65034   -320.60512   -398.92257          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       48.50   -457.33036   -258.66733   -385.31543   -482.94461   -687.30758   -530.13762   -231.95553   -102.42206   -395.53772    -56.04053     -366.091   -374.71352   -275.36427   -411.56214   -522.61169   -283.51931    -316.3644   -306.06844   -238.06896   -469.11577   -405.79306   -214.68697   -306.76983   -252.23181     126.4084     580.3207   1430.11433   1215.32551   1022.55678    510.51618      1.59592   -184.25379   -311.28337   -360.55566   -344.01976   -383.55573   -271.80238   -381.19022   -460.53677   -349.83318   -334.72476   -286.01239    -613.0987   -419.13506   -387.08426   -412.56048   -448.20978   -625.50036   -328.74103   -405.89315   -453.25983   -514.81684   -273.83334   -479.54791   -230.60854   -346.64341    -219.4032    -275.2232     40.62228    -81.54969   -127.79035   -246.64866   -408.70722   -308.74021   -279.87252   -285.13065   -296.17022   -304.24921   -205.59778     49.53081    675.67457   1080.08378   1378.12211   1472.96707    995.59297     307.6495    120.13872   -284.91693   -120.26029   -283.40834   -325.06312   -624.90298   -430.81213   -216.70239   -172.25446   -303.81841   -321.58443   -379.63304    -95.24201    545.88955    700.77539   1358.97709   1503.66269   1264.28261    851.75243    264.86206      38.8416   -242.42803   -356.76773   -229.24681   -307.27784   -127.35993   -180.41743    -74.76346   -169.58017     32.46581   -220.99429   -222.00083   -382.11359   -244.71416   -371.52138   -353.41876   -365.32586   -383.79291    -348.5363   -350.86632          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       49.00   -425.88971   -142.20925   -389.19489   -298.71523   -530.08919   -571.88853   -237.97749   -174.31243    -204.2174   -217.30228   -327.54429   -371.60083   -371.00614   -398.76336   -677.74631   -332.13559   -340.49796   -304.54373   -513.30938    -493.4654   -253.88383   -204.11118   -394.36106   -202.27013    152.71668    620.86491   1480.53152   1138.91524    989.09383    581.75404    -157.6633    -155.3978   -119.65853   -381.49836   -386.25806   -495.56317   -409.93508   -404.25464    -408.3772   -373.59898    -474.6896   -366.60459   -550.92181   -407.16105   -255.67288   -347.23825   -496.71193   -506.52927   -503.36416   -352.66893   -412.29573   -351.44678   -460.63941   -345.10641   -315.85383   -198.12452   -128.91705     -1.73589    131.69271   -257.39144   -168.78182   -190.30188   -407.11386     -128.976   -254.54726    -244.6642   -310.99101   -216.02067   -333.20628    -22.31311    765.68212    880.89354   1398.87284   1226.72228    969.14879    495.12829   -183.16842    -53.41632   -142.42507   -498.05252   -452.97024   -229.64194   -370.60551   -490.74883    -95.44459   -383.58968    -460.7504   -150.14447     68.04623     68.90149    812.89558    853.30273   1503.04278   1546.81556    993.01725    447.93611     31.32131   -320.88836   -192.27237   -284.19656   -276.28222   -256.99609   -211.34189   -218.98122     52.56099   -183.67735   -390.69677   -265.71374   -393.08246   -340.50264   -356.68126   -249.88164     -353.469   -444.40542   -382.00843          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       49.50   -394.07591   -123.52522   -352.76612   -173.68092   -392.42756   -464.13402   -321.26061   -177.29717       -124.0   -256.33279   -410.38306   -298.09923   -533.50753   -326.60686   -540.17612   -463.72034   -307.32417   -317.16932   -556.68153   -720.64131   -205.88468   -114.14791    -306.5607    -154.3053    273.57204    526.62684   1339.78098   1242.01773    990.34075    264.59641   -223.48208   -155.08945   -315.00038   -446.07715   -469.81425   -409.10214   -404.89055   -134.74392   -324.29339     -377.011   -379.43467   -378.51598   -443.59515   -420.35876    -348.2198   -461.52831   -235.85163    -384.6794   -473.02325   -224.18191   -409.98794    -345.4654   -150.90142   -349.05926   -532.83975   -113.15872   -162.49131   -210.55601    -49.53744   -173.70047   -194.46659    -189.0947   -279.06238   -284.44058   -186.71624   -360.41457   -388.67575   -117.78887   -107.32549    331.34932    836.44584   1033.05415   1675.52459   1309.40905    793.96583    436.29356    -34.48249   -251.89305   -118.60206   -267.25089   -374.07515   -178.89124   -457.46608   -377.83502   -420.47078   -256.80977    -265.0831   -250.50217   -181.44175    459.32258    716.39613    1381.4238   1586.51233   1468.50583    1077.2373    546.18941     74.71029   -430.45228   -482.87489   -334.59365   -397.89106     58.53061   -139.89191    -78.93317   -146.72087   -124.72357   -224.00745   -360.49573   -347.47429   -321.59786     -364.362   -438.23639   -257.70107   -435.31395          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       50.00   -361.89768   -129.60721   -187.14195   -229.46058   -326.18084    -443.1088   -223.37894    -39.49411   -247.04361   -225.34366   -494.41188   -362.79821   -543.76106   -262.47843   -279.78353   -361.58239    -409.3572   -339.47442   -338.60517   -535.52838   -248.92738   -362.19968   -188.09705   -342.89995    288.42412    584.97543   1230.41795    1246.6317    700.25618    505.64749    -55.38188   -331.74235   -300.04774   -379.46655   -427.71284   -427.77018   -292.11145   -544.03966   -344.85365   -227.12643    -458.1793   -283.22938      -392.65    -186.5526   -421.66017   -525.35555   -430.51757    -298.6119   -332.08382   -512.02626   -447.58569   -339.78207   -485.43649   -363.39483   -484.33658   -247.47804   -112.25682    -48.38529   -373.07401     20.99552   -373.46305   -102.02082   -330.15018   -349.02189   -230.77037   -300.01288   -317.52287   -123.77503     64.17444     154.8598    650.18832   1263.34989   1175.60609    1346.7654    737.37836    413.69202    198.70444    -71.37525   -275.48635   -211.89634   -207.14797    -411.1136   -364.16239   -596.18586   -353.97066   -457.89451   -215.52845    -221.8071      7.85911    286.29428    708.45662    1070.7309    1579.9154   1592.76773   1076.33208    649.41049    -41.92031   -208.00456   -127.90705   -275.25552   -173.03258   -153.98334     -110.856   -152.30893   -131.43947   -286.37717   -296.79617    -243.4331   -348.74265   -230.36595   -475.38498   -501.33955   -381.41995          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       50.50   -313.12684   -160.34308   -109.44315   -377.67924   -286.39652   -559.93476   -160.11576    -81.87028   -394.52259   -294.28006   -442.68012   -358.78037   -599.19091   -373.56939   -306.55461   -295.01362   -296.74986   -396.87057   -359.14247    -240.1284   -318.19112   -328.26707   -107.79384   -284.48471    106.16783    738.31461   1231.03818    962.84174    699.11132    529.64467     65.37425    -156.4678   -262.92124   -249.86776   -361.10966   -437.26643    -338.9347    -545.3121   -396.93442   -385.53209   -294.72697   -349.91468   -433.42049   -474.52133   -413.82758   -430.40796    -325.8368   -215.90106   -226.27469   -478.07387   -435.82451   -179.11364    -518.2153   -350.61603   -222.01408   -477.42829    -59.74845    -45.22578    149.81546    -59.66883   -153.55655   -294.39462   -272.12145   -344.51793    -233.4373   -325.93284   -454.41771    -92.10657   -137.80197    152.34488    696.28878    950.98846   1322.72732   1288.28462    615.63974    451.16615     -78.1461     -8.83465   -397.16317   -369.25296    -536.7118    -69.69019   -620.71253   -164.03319   -491.17094    -230.3776   -170.99506   -193.80393   -186.17909    459.86991    683.70335   1166.13145   1691.27309   1516.03994   1260.90159    853.20426    106.40331     73.86891   -121.18409   -217.55795   -184.49379   -223.37068    -155.8041    -63.15243   -351.07163    -127.7848    -300.2397   -305.52244   -373.98304   -480.06025   -401.57578   -416.04962          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       51.00   -340.25678   -379.84552    -139.4288   -443.65462   -268.72901    -610.6205   -225.87719   -286.59975   -397.78089   -314.08004   -431.40538   -348.37545   -533.18072   -491.05664   -562.00954   -309.15018   -302.77839   -340.33213   -361.18435    -348.2135   -350.78325   -253.80201   -258.27089   -152.55004    -203.3429    778.14911   1028.39088    983.72016    813.54466     408.8108    -81.43092   -182.70758   -260.29305   -477.40501   -279.82627   -461.28932   -509.86804   -549.82006   -383.35835   -489.03861   -326.51965   -391.90132   -406.64709   -347.84383   -348.57312   -389.76344   -523.26707   -219.33553   -146.35878   -435.75021   -417.04575   -337.57478   -430.74159   -448.63218   -213.74841    -36.30741   -215.56069    -97.57058    -43.12701    -88.06527   -117.50794   -320.22956   -245.82969   -342.86366   -163.95437   -270.32894   -270.89538   -110.99829   -167.43141    240.01585    700.06482   1058.62409   1332.04314   1407.64652    789.41363    246.75932     28.74747   -270.25123   -372.87595   -797.26188   -688.17875   -584.48482   -547.64666   -204.25829    -334.8995   -253.73182   -318.01431   -193.40824   -197.67705    166.65754    430.93013    916.74498   1485.83995    1568.7903   1256.29922    672.94503    194.52383      65.3456   -403.02148    -269.3527   -215.84614   -326.39825   -157.95279    -75.94007    -15.18673   -285.19446   -310.21974   -333.35793   -248.66449   -366.22495   -358.25708          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       51.50   -442.75901   -536.23003   -341.25867    -406.0381      -230.36   -587.96439   -246.73546   -395.54453   -346.85393   -261.99099   -365.28302   -465.22588   -389.46415   -481.23777   -574.04218   -241.16895   -230.12207    -359.6351   -406.60731   -322.67987   -354.09255   -165.76001   -242.42398   -318.08512    270.27015    668.80384   1095.84061    904.83956   1004.25807    304.78775   -181.71303   -322.49538   -310.84928   -411.42232   -334.68951   -503.77384   -453.93953   -482.85613   -252.29674   -420.37737   -369.31983   -314.35782   -505.42496   -448.12761   -425.64477    -57.23497   -256.93029   -293.93917    -445.2252   -421.60973   -651.68374   -350.24512   -354.11146   -325.32535   -311.94694   -389.83041    -85.60628   -242.96929    -122.8613   -205.49241   -158.54244   -185.96469   -167.75113   -420.14717    -317.5194   -327.29707   -234.14793   -170.08177   -228.95293    163.95196    367.96773   1079.29613   1327.29307   1443.19124    910.19014    273.79961     11.37935    -38.32109   -369.77988   -535.47492   -808.05317   -456.61828   -417.42822   -507.08107   -248.40234   -482.83877   -117.70252   -150.35599    -40.86231    189.76355    513.11802   1086.72829   1387.21399   1269.65748   1070.44354    692.19614     71.52754    -73.39859   -305.28511   -275.99617    -323.1619   -208.61113   -325.93735   -177.59391   -271.97503   -251.73638    -335.8115   -371.83467   -494.79788   -443.51464          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       52.00   -541.72149   -610.04025   -407.48641   -469.58825   -214.50143   -541.19165   -142.13748   -234.22032   -446.37205   -211.79122   -393.27593   -420.89892   -382.26898   -261.74916   -416.96422   -273.04178   -266.59015   -454.27231   -500.78191   -282.33647   -322.77501   -134.43836   -205.92595   -302.87979    288.71224    773.80988   1143.55716   1142.99296    972.27288    246.32135    -15.55934    -383.8328   -325.30032   -305.40585   -492.21889   -569.54845   -423.21606   -495.81197   -333.20953     -304.258   -195.90237   -380.12907    -235.7911   -503.89872    -263.4498   -332.53844   -260.65432   -312.71963   -538.27594   -403.96968   -305.87176   -460.13713   -201.45846   -274.08102   -373.53737    -93.35869    -239.0891    -35.97421    -31.58445   -261.14743   -143.50653   -323.97217   -364.44514   -444.99751   -261.02554   -349.50792   -164.49767   -276.44779    -64.84104    192.25581    606.09112    1132.1597   1329.63082    1446.3938   1125.91212    334.97515    -155.8105   -367.30983   -345.12975   -655.14115   -679.55329   -625.70341   -349.92747   -395.76431   -278.17911   -318.94073   -154.63358   -172.55821    123.96979     56.43386    432.78356   1001.66533   1428.75646   1409.01832     930.5105    686.07243    156.20746    -97.77398   -238.33037   -367.36757   -124.79576   -142.71565   -163.57029   -136.30404   -123.68674   -228.30374   -366.30906    -299.1882   -385.31237          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       52.50   -373.42114   -497.63141   -270.88583   -636.96397   -240.12586   -507.54525      3.93988   -134.46591    -513.2695   -199.11901   -511.47781   -354.39874    -542.8242     -53.1974   -370.11343    -51.78714    -375.1661   -295.71505   -407.02771   -429.67617   -296.63441   -270.63114   -169.28016   -319.24661    348.98395    491.09367    907.09497   1255.66081   1024.58727    382.46896    199.06825    -76.92752   -526.00379   -403.83747   -557.19675   -314.84695   -390.01372   -364.48842   -164.35603   -522.36281    -48.18593   -279.19101    -302.4629    -445.2952   -267.99591   -366.51933   -369.64029   -325.94876    -402.5868   -370.03174   -288.31427   -408.87553   -267.30666   -377.07468   -104.17075   -382.81343   -150.09795    -57.79514    -98.80891    -68.65855   -235.11907    -170.8093   -257.70892   -341.74334   -466.56746   -450.08752   -216.38256   -319.68994   -155.64692    225.45688    588.32979   1126.45091   1569.87953    1221.5278    957.50162    649.55343    -23.41494   -129.46216   -368.82105   -277.26361   -809.10017   -286.50056    -149.0686   -147.12051   -361.76456   -411.87803   -148.99099   -115.87043   -210.25916    206.94402    490.48738    941.77622   1326.37989   1362.53315    957.86475    494.07877    152.84238     -9.52245    -253.6198   -259.11784   -339.85723   -122.42824   -235.99697   -106.01894   -285.14696   -337.36246   -354.56616   -493.21183          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       53.00   -192.89681   -181.46072   -141.48519   -815.71912   -281.84959   -475.20508   -198.05178   -237.66203   -406.06803   -122.11842   -545.34307   -328.33749   -582.15837    -33.30232   -349.15339   -349.63116   -276.56895   -174.69116   -269.06016   -217.82078   -346.86659    -27.13121   -144.51371     20.29805    276.37954    449.42124    872.89254   1112.47027    845.04619    453.92931    173.91037   -179.74291   -264.82884   -371.91598   -534.48952   -399.12609   -219.26047   -333.67439     -255.068   -355.07153     -248.107   -416.69856   -369.11829    -516.2061   -537.61231   -504.14909   -505.87676   -394.85694   -414.02011   -529.52948   -390.55244   -487.21403   -293.08103   -347.96567   -545.01787   -434.91728   -456.32748     36.59349    -74.16391   -135.48506   -232.94113   -273.78533   -223.45318   -295.41175   -320.90027    -240.4052   -386.44048   -195.70565    -47.92105    280.70967    563.88887   1028.06647   1427.94273   1408.48158    892.73197    365.44239    100.56756   -261.04555    -403.3951   -187.99374   -591.27786   -528.89719   -341.98297   -454.11603   -388.93964    -323.4534   -232.29787   -213.46049   -208.60494    261.23156    633.91085    850.68083   1341.90086   1435.20437   1238.19527    545.01685       9.0771    -113.2665   -303.97899   -302.77389   -433.85541   -274.49877   -314.55369   -266.86078   -161.56085   -216.63081   -400.45586          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       53.50    -86.20924     89.35705   -220.25569    -746.7281   -304.24864   -354.63798    -454.2167   -281.31502   -240.81357    -42.34049   -665.50022   -327.00927   -669.88781   -245.48567   -301.51474   -567.10657   -226.02294   -163.71374   -274.21178   -115.49192     -322.102   -221.34077   -111.43047    -65.35941    213.28253     512.2511    989.37164   1100.07904    588.63311    664.49862    154.12271   -243.03145   -252.37363   -364.57071   -276.30385   -424.73883   -399.40083   -210.98694   -444.56355   -344.76574   -310.06775   -610.68902   -430.98272   -508.32571   -402.63781   -513.60624   -284.90423    -486.7456   -653.88554   -413.19134   -476.91618   -352.77926   -390.11097   -439.98344   -305.26758   -189.45518   -262.67428    -84.60226    -47.07405     -3.66223     57.26554   -269.58397   -126.91994   -264.04572   -311.37587   -261.01709   -306.25873   -240.87996    -233.9446    302.30627    440.49085   1004.68148   1295.70498   1364.39007    769.54387    410.52178    146.39369   -236.29047   -339.17214   -625.61443   -263.92286     -370.712    -780.8402   -450.70901   -164.56724   -325.50745   -264.79287   -207.97835   -230.27578    225.88987    520.98098    1028.3743   1375.04781   1477.08364   1258.31706    559.49271     399.2702   -192.52532   -521.31654   -220.65359   -300.68506   -177.84018   -226.53441    -41.84986   -278.47261   -390.39577          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       54.00   -119.18571    135.78964   -503.63715   -606.60384   -325.07266   -171.86875   -462.37079   -129.94227    -203.2637   -152.17176   -605.86757   -284.42209   -626.11369   -305.43854   -308.90129    -426.6304   -274.69987   -170.74964   -364.18643   -179.80883    -62.59154   -266.65657   -182.58397    -297.7455      88.1393    393.39775   1105.28655   1063.90795    602.29318    470.61303      55.3596    -64.04035   -203.35292   -351.20759   -599.61884   -274.02118   -389.62729   -608.79435   -350.57936   -436.49597   -405.54698   -455.98596   -317.61669   -453.73608   -294.36794   -271.64071   -271.96356   -144.99763   -455.42658   -353.99376    -486.9332   -457.66519   -313.32226   -376.49146   -216.16926   -174.99383   -132.00081   -148.79903     37.52891   -191.75283   -115.51493    -62.66318   -241.30991   -313.74414   -336.91424   -237.35073   -424.72779   -167.04482    -197.5776    336.86428    447.62851    1009.6378   1684.28382   1425.30824     827.0421    606.15387      68.9516   -135.42804   -107.91646   -319.29603    -561.5219   -482.96463   -523.23475   -651.06073   -401.04889   -510.55782    -114.0615   -189.24944   -198.25755      82.5929    389.40058    896.07338   1389.41248   1372.64447   1155.43454    679.37307    294.48027   -119.88095   -426.29686   -456.98809   -406.37569   -221.77621   -385.20531   -351.13742   -367.87747          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       54.50   -148.49289   -119.13105   -567.60206   -395.94804   -368.74686    -60.18337   -298.40014   -193.35705    -308.5027    -267.4799   -409.55772   -183.14092   -249.67553   -345.12204   -318.90477   -220.66896   -425.23867    -281.8314    -438.7746     -97.8459     -7.53951   -255.35836   -250.66774    -59.19432     39.39136    306.53853    830.45277     938.5721    783.94293    421.21317    -55.03146   -138.33935   -194.69683     -185.664   -345.13405   -373.65545   -409.95658   -332.41202   -354.84461   -312.43854   -442.18172   -316.38358   -302.59574   -377.64854   -365.23411   -315.68542   -302.51579   -556.48587   -458.98971    -338.8636   -339.42179   -364.75824   -175.99035   -270.91251    -195.5806    -213.8197   -262.12598     -5.84757    -99.92379   -170.82585   -194.10013   -200.83084   -284.03337    -361.4083   -193.34303   -295.80437   -291.08114   -298.68511   -174.73576    116.86551     727.1731   1006.54425   1245.04369   1335.19034    813.80492    521.98287    198.40014    -95.38465    -49.40956   -377.57844   -259.66009    -694.2443   -235.19591   -567.07806    -312.5115   -377.18212   -204.47865    -20.41097   -187.41935    290.85982    527.38455    832.65418   1129.57488   1396.83861   1208.46151    483.12528     52.94828   -397.04063   -382.16587    -332.5537   -407.57546   -352.23172   -318.12934   -405.72161          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       55.00   -170.84868   -279.25691   -405.34731   -330.84652   -391.08976   -137.66256   -170.94476   -179.31012   -416.91777   -384.46283   -303.73595   -271.39486   -134.21319   -439.52645   -342.38605   -377.21035   -319.47917   -399.02309   -223.72714   -295.74749   -212.44555   -285.55415   -171.80138   -105.56594     36.42821    456.24805    624.73005    795.84743    757.89314      259.727     44.77157   -350.10083   -387.85896   -380.96315   -326.35522   -612.89563   -509.24491   -505.17603    -409.0136   -401.93354   -332.11568   -361.73407   -343.87354   -565.88397   -368.25267   -428.82306   -415.83087   -362.93059   -283.88523    -512.2459   -431.49566   -305.37546   -346.75042   -327.84525   -255.89865   -174.88766     25.89704     46.26012    -67.97281     92.85365   -170.69613   -376.20127   -342.64715   -465.10914    -97.31762   -208.89608   -178.72681   -284.05313   -141.27786     58.23647    371.96197   1046.79044     1146.887   1267.60331    804.05393    794.15984     153.2294    -87.30795   -190.36647   -251.04111   -434.29793   -408.34765   -507.52258   -287.83516   -454.21343   -432.28215   -467.80719   -244.52259   -160.06891    164.84981    443.60918    991.80178   1251.40636   1322.31029   1110.02789    888.80391    132.59177    -363.6157   -829.15815   -526.30183   -232.29009   -376.99731   -363.80093   -367.37984          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       55.50   -243.28499    -345.4348   -270.79689   -422.20802   -337.65162   -297.62241    -219.5391    -62.27845   -511.33239   -407.81214    -220.6545   -330.49443   -218.16077   -418.12213   -284.45382   -565.82263   -289.42596   -536.57243    -193.2925   -414.01192   -376.90265   -367.30257    -20.32756   -169.68566     281.8467    552.12953    565.62149    590.68387    766.15612    244.29318    189.09365   -291.57792   -377.38543   -282.77028   -494.33438   -340.88089   -265.65446   -490.74141   -396.35231   -412.16505   -364.61687   -330.71619   -325.77535   -538.95923   -501.07904   -410.47258   -405.83371   -193.17099    -266.3798   -398.41654   -356.03967   -404.40631   -316.66492   -475.59138   -347.49247   -337.80032   -218.52228   -142.79259     68.78788      3.38309   -104.51671   -278.28222    -189.7081   -374.58821    -185.4213   -201.45019    -142.8495   -254.86651   -156.04313    208.15232    538.35546    811.11592   1121.49005   1211.61463    1131.3248    425.15013     19.77822    -44.67904   -437.35754   -173.30821   -359.92468   -692.28491   -137.36981   -243.62939   -508.55846     20.20734   -333.37609   -208.93661   -304.69167    -72.78296    273.42508    876.40917    1130.1832   1315.39854   1150.28227    702.84435    131.56661   -240.42216   -361.21929   -297.73552   -315.31431   -128.03295   -370.95334          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       56.00   -334.38566   -357.96741   -177.61557   -507.92048   -188.27872   -335.94394   -282.20371   -199.12572   -442.52325   -367.38057   -185.81933   -281.05484    -207.4812   -299.81163   -219.17396   -352.36056     -324.295   -741.84689   -363.04254   -315.76267   -343.44987   -338.71486    -15.76913    -56.71964    291.76101    433.72074     778.6512    818.78271    669.33345    402.65604    -67.19461    -179.7794   -312.07652    -234.7928   -329.86859   -259.32499   -498.99344   -659.64428   -220.49808   -374.52919   -511.23722   -247.66065   -415.39658   -664.85606   -302.49452   -336.88664   -423.47155   -340.69775   -447.34613   -402.20193   -364.30649   -214.17566   -466.59328   -387.21859   -360.97013    -272.8855   -130.22217     63.29025    -21.29517     -49.6578   -232.26271   -261.11466   -389.58989   -250.79019    -88.01108   -222.26384    -238.3979   -218.27124    -84.30888     -34.9803    554.27426     877.8468   1525.99925   1442.68923   1163.49336    316.09598    375.65866   -162.36297   -389.62253   -486.90469   -431.65497   -286.88326   -432.67471   -554.03383   -326.54129   -315.09091   -406.26841   -150.46804    -398.1501      7.68046    305.07992    880.98764    866.01395   1098.51582    949.06952    531.26984    234.86192   -128.58021   -304.91223   -458.78663   -262.92304   -424.81859          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       56.50   -424.52455   -371.05396   -176.49843   -494.42968   -112.13257   -298.51851   -303.33462   -296.82646   -440.26781   -246.50883   -278.84779   -209.44281   -176.17384   -255.34881   -206.62548   -105.08431   -374.27811   -613.99009   -347.95552   -309.19968   -245.61211   -349.72798   -124.47518    -122.3125     274.0618    505.71206    747.06167    859.67823     629.7079     362.8557   -136.09851   -143.02714   -326.54441    -284.6902   -265.49455   -209.44932   -648.08028   -546.41562    -56.46019   -436.78321   -192.52188   -228.91243   -398.98655   -399.40635   -443.69848   -549.48296   -366.68185   -421.50841   -446.53575   -397.28217   -486.65386   -445.70037   -373.95372   -437.38108   -442.09177   -364.42985    -35.10603    -37.33465    -51.74371   -201.05456    -37.29885   -168.28673   -301.22628   -292.72862   -346.84051   -256.58957   -190.29563   -199.09186   -214.40106     14.04891    613.06278   1147.69265    1200.0245   1237.20996   1141.03031     504.3703    263.35672   -137.94197   -411.21096    -418.8795   -393.50285   -455.63258   -526.92129   -400.23113   -452.85357   -284.97776   -257.98943    -189.3411   -281.88446    -85.34269     391.5893    667.43983   1105.15186    980.05424   1020.83652    590.25683     181.8418    -55.61365   -316.34135   -407.18692   -390.98842          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       57.00   -513.69405   -358.14264   -202.49511    -360.7072    -203.3145   -361.99823   -348.07492   -249.27433   -507.40588   -153.21676   -267.81214   -226.46839   -336.08694    -73.42289   -181.01456   -255.44436   -506.46126   -414.60422   -383.65863   -243.69335   -284.72799    -56.03777   -318.49706    -93.10318    181.63265    545.40022    753.37994    746.42648    830.31458    244.06194      45.6727   -133.45091   -493.81571   -334.88825   -323.43743   -157.90301   -403.73511   -388.40855   -323.05372    -289.4724   -630.27914   -363.34383   -347.62635   -525.23361   -111.77288   -466.59381   -558.12345   -342.20006    -412.0634    -280.2669   -402.44788   -403.43628   -383.35102   -365.11566   -239.47096   -352.97368   -192.40136    -96.83675   -206.03685   -108.38572     -155.873     -65.3835   -198.66063   -411.13079   -435.51809   -183.78033   -238.25312   -107.76449   -174.31569    132.76283    714.88116    811.76578    906.45546   1331.27395   1100.32509    536.14815    190.69255     135.8401   -348.83292   -393.68902   -268.61095     -331.607    -592.8349   -391.50167   -423.92966   -369.02557   -339.52451   -210.76744   -217.39139   -224.96399    294.96307    540.39396    979.96975   1069.30391   1094.15999    644.81471    166.34356    -96.49495   -226.45695   -428.05802   -395.99614          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       57.50   -587.31378   -233.49432   -191.14996   -246.52456   -397.25366     -420.367   -388.01618     -155.767      -608.22   -287.11031   -175.28724   -422.32589   -313.89406      8.52569    -52.94745   -289.20119   -490.75788   -376.47221   -397.71222     -404.868   -283.58537   -195.47567   -405.43604     59.41101     39.41014     619.8581    685.14182    787.95123    662.49805    299.05231    117.35189   -162.92333   -351.24833   -390.27553   -194.99906   -315.72808   -730.15318   -286.83561   -346.33719    -80.02807   -482.96917   -423.50553   -241.03199   -159.58934   -262.58318   -426.29086   -243.95204   -482.53631   -223.08553    -152.0485   -334.51109   -338.09166    -594.6278   -243.72255   -368.14789   -247.22491   -100.99779     -98.6058    -42.47762    -20.75101    -76.10836   -131.22669   -229.31535   -312.14212   -387.90477   -252.37256   -354.64308   -239.26668     65.60564     71.68339    489.42327    1117.0501   1221.86441   1219.58084    815.54414    358.99569     264.7878   -127.73596   -251.35462   -377.67734   -296.14418   -289.95888   -482.51062   -478.55152   -379.90696   -379.72428    -364.2422   -439.78181   -285.17504     43.19215    297.30612     666.2972   1069.51323   1056.82552   1085.63949     952.6282     64.84567   -140.67169   -145.96351   -444.94505          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       58.00   -547.19427     63.24677   -405.34972   -221.13586   -444.03677   -479.76176   -408.87839   -132.91828   -760.30328   -394.16953   -292.96315   -601.52808   -224.47321   -199.38361   -132.74703   -381.97025   -547.32185   -339.29241   -314.33299   -441.38936   -309.75033   -388.69497   -442.96002     41.13923    -57.76091    480.20547    627.30707    766.40481    387.72469    199.14171      80.4344    -256.5308   -423.91519   -302.37018   -620.03915   -485.39704   -653.24333   -391.66079   -273.24709   -281.64672   -384.55372   -526.72216   -313.92964   -476.29712   -379.52428   -309.24104   -459.21535    -300.3578    -485.2418   -335.18659   -264.52766   -447.33193   -257.02911   -337.67866   -279.34874     -23.4935   -369.65823     27.79771    106.40424    -57.86675   -222.45435   -217.06698   -224.49932   -273.04827   -348.51592   -199.99438   -271.71765   -352.19457   -257.04921     54.23665    462.34645    898.51607   1222.90873   1290.76391    856.91985    732.88762    151.38833   -102.08059    -66.13792   -333.19585   -362.47713   -556.69041   -400.10601    -289.4747   -505.15938    -409.7038   -474.25399   -141.15435   -181.27419     -295.061     216.1316    518.31632    896.47613   1212.05524    951.60521    591.18798    249.07703   -177.51546   -372.22963          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       58.50   -401.47903    146.91223    -623.2406   -117.66951   -335.95736    -566.1222   -498.33917   -304.69307   -746.76025   -479.87395   -409.38964   -491.29433   -266.75164   -201.08304   -169.67528   -536.05927   -562.76872   -349.88004   -430.95757   -156.76276   -414.96307   -336.74167   -290.65888    -73.04347    115.68474    308.40669    541.66441    664.51371    526.98434    274.53129     41.21752   -364.75515   -317.23217   -507.69912   -308.40923   -480.35168   -629.42581   -339.17028   -308.47201   -324.60415   -564.52106   -400.54582   -399.81767    -388.6955   -368.43329   -453.85367   -285.44245   -539.63933   -367.34692   -450.01428   -281.74683   -470.01103   -335.86981   -409.88015   -328.97794   -172.81127    -98.72674     13.72059   -130.87648    -93.87273   -250.65054   -268.50259   -281.24797   -266.05871   -273.28641   -274.75942   -340.04495    -85.90738   -207.48003      66.2302    467.17855    957.90413   1431.59916    925.95731    967.82452     749.4644     28.42765     39.82744   -493.78282   -306.54989   -609.41561    -188.0409   -487.45678   -441.04069   -438.13495   -289.62248   -310.47199   -137.65115   -195.51038   -148.30753    298.73821     517.0035    702.80553   1043.32796    833.51966    487.59496      3.04386   -275.15861          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       59.00   -220.00551     117.9731   -713.72059     71.17421   -181.55029   -586.42085   -615.30135   -407.62109   -603.70597   -483.55927   -339.02492   -381.92558   -436.74895   -115.02259   -121.63303   -459.45334   -443.45434   -247.77843   -353.14231   -246.53618   -479.64205    -381.6547   -441.81185   -178.99937      59.6073    412.09457    451.58015     465.4221    179.37943     420.5326     -8.98878   -265.44762    -478.8052   -317.11034   -290.90667   -639.23801   -491.29266   -362.59415   -296.82291   -145.69228   -501.61179   -385.30219   -268.25965   -372.95389   -227.65334   -456.36099   -411.67883   -260.98505   -333.58131   -436.45511   -388.88128   -499.10882   -306.74296   -279.62402   -413.18416   -518.67221   -119.86938   -182.37612    -41.73383    -97.26423   -109.01438   -309.93676   -208.15483    -239.7476    -44.64576   -191.93578   -339.54555   -135.89717    -47.57595    289.74281    435.45135    815.75981   1197.27955   1438.83703    988.29872    470.92123     19.31582   -310.84718   -391.60874   -220.23064   -327.79279   -398.70207   -225.04638    -386.1618   -403.72942   -216.37536   -221.68987   -227.61373   -155.09935   -112.46157     89.33401    546.06584    769.21428    925.30462    800.95798    468.81609     77.69709   -326.72057          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       59.50   -195.47218   -197.44273   -438.06933    -79.96601   -214.86705   -525.26248   -461.76166   -222.69186   -418.14248   -451.55286   -294.89578   -361.20357   -438.48473   -175.92686   -129.28234   -452.92357   -328.56615   -199.07866     38.29041     -29.1673   -454.21445   -676.95981   -606.39754   -152.76294     230.0409    500.88144    678.54762    645.07896    461.63848    314.85607    -81.17914   -339.34232   -459.55432   -262.08778   -320.98525   -599.84984   -492.86757   -360.49952   -241.20958   -182.18614   -147.81949   -441.91567   -546.40353   -461.36055   -369.99492   -395.13642   -458.47813   -157.87276   -385.40393   -278.64512   -253.54506   -491.99716   -610.03197   -357.29132   -365.13132     -81.2136    -26.12242    -88.33106       66.081    132.60958   -277.55269   -305.24698   -373.21342   -204.60827    -192.8706   -256.32185   -435.31805   -287.07055   -124.91813    -60.90234    298.34021    883.16041    1023.3013    1265.3731    972.63826     629.0918    217.04265    -62.79465   -270.14927   -335.83445    -86.21638    -62.97602   -360.22819   -633.15949   -419.01212   -388.63232   -321.28991   -273.67894   -149.52681   -140.05133       27.189    417.44546    731.67542     796.0868    785.94618     439.6593   -106.61934          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       60.00   -259.71675    -484.7465    -301.2367   -409.56706   -283.40548   -452.56569   -398.19965    -86.08976   -359.27633   -456.52803    -370.4229   -373.47236   -388.15601   -328.67653   -174.19824   -400.69291     -343.122   -238.10747   -233.43762     35.94123   -424.19012    -357.6909   -363.79266   -403.87826    325.45006    340.17978    568.17855    641.23095     501.5768    269.46957   -260.30197   -275.03822   -299.19666   -308.91351   -307.00552   -715.24324   -316.46493   -436.69801   -224.55651   -545.67903   -342.86709   -381.76193   -558.75221     -385.512   -471.17941   -337.16928    -405.9387   -412.41504    -371.4095   -322.26571   -556.15918   -335.38373   -361.04109   -284.85153   -355.31095   -311.50116   -152.85364    -91.53576   -225.90656   -203.19881     -2.14159   -127.73594   -321.50531   -425.21551   -291.17174    -261.2144    -247.0753   -391.10736   -103.27813     -1.50732    487.50074    878.75064   1221.10266   1309.66049      833.496    635.68292    239.78517     -6.30556   -159.60883   -413.22154   -193.24623   -242.02167   -295.49248    -412.1244   -430.28408   -252.30057   -422.31453   -235.61939   -266.06988   -209.90317    190.53756    484.33473    697.10831   1010.69154    539.26536    348.32902          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       60.50   -429.91238   -737.49839   -304.98851   -636.78292     -390.877    -363.8043   -448.50515   -156.29466   -483.75315   -495.41986   -350.80897   -415.26489   -326.17775   -342.72084   -344.57508   -396.98006   -389.59797   -149.49556   -284.56318   -116.46047   -430.72601   -230.91909   -360.37647   -379.83521     32.00856      33.9038    506.93773    485.62834     345.6218    437.85911    -120.7529   -251.85162    -397.3591    -24.04628   -273.21593   -472.58569   -402.26418   -181.02762    -383.4193   -195.68725    -504.9266   -543.25358   -688.26331   -341.51085   -577.38501   -553.23758   -226.67591   -510.13264   -441.69336   -417.18465   -350.10444   -368.97293   -543.13896   -349.52793    -351.9129   -372.47715    -90.93167    -100.6929    -90.94694    -74.21297   -131.29833   -375.80796   -279.05846   -543.63252   -362.37883   -487.18129    -344.2637     43.85191   -266.14345     -3.74924    398.40283    833.80575   1092.80685   1152.06367    762.98225    654.17693    341.99136     38.77663    -78.23189   -247.03152   -188.97528   -528.95241   -497.18539   -484.72163   -332.94415   -244.14788   -485.97474   -352.41703   -328.50601   -135.70271      10.9436    395.25852    623.68586    808.51125    608.26819      23.1796          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       61.00    -374.7142   -727.80592   -395.20164   -622.19902   -306.52912   -244.36032   -404.97215   -357.17535   -547.88076   -443.87804   -150.39751   -383.98691   -226.47617   -278.36164   -410.52331   -528.98993   -236.64887   -223.24223   -263.49266   -358.57201   -110.06238   -267.30234    -397.0779   -351.23447     -3.09732     86.26555    977.67711    498.04326    295.96981    107.70943       32.061   -324.78957   -563.37422   -451.71597   -235.99358   -529.20202   -356.48042   -232.71279   -374.37631   -314.74068   -292.88999   -402.82266   -489.67311   -664.71127   -444.10361   -397.96298   -298.39248    -351.3302   -341.89723   -414.08287   -180.60053   -379.77959   -542.57896    -341.8345   -209.90597   -375.52187     -224.658   -135.10757    -44.06538    -73.15409   -102.64321   -184.76344   -514.86081   -298.49534   -286.15179   -377.63883   -487.75145     -249.325    -273.1319     -6.21796    297.74295    927.52158   1125.97375    978.56886    1206.5777    511.72206     327.4724      7.52705   -257.28553   -360.92683   -178.94299   -370.22122   -307.05525   -484.33328   -404.30762   -492.62001   -466.58129   -372.40102   -298.50312    -97.41408    190.90996    334.65283    566.10046    756.59717     496.7573          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       61.50   -297.24654   -602.67266   -452.59661   -515.36258   -242.70664   -206.21435   -295.13526   -507.72585   -477.23382   -361.63654    -26.58933   -393.61457   -266.10744   -361.14543   -368.56584   -584.24591   -123.01453   -233.90582   -242.40685   -345.75336   -107.28165   -187.86879    -432.0884    -288.1081    229.90876     166.8724    688.45571    414.02936    500.30676     200.7688    -29.78021   -170.87163   -307.39786   -415.45078    -473.7042   -339.69377    -469.6068   -248.72086   -396.87228   -502.44464   -478.37239   -327.19472   -397.71241   -218.31735    -397.0578   -284.31735   -321.68192   -380.55669    -558.2449   -534.54115   -384.65338   -355.90523   -432.57388   -337.53686   -104.04684   -340.10677   -282.54189      6.38995     19.34485    -94.98382    -46.69393     -3.23045   -289.72771   -265.28052   -385.26425   -228.21256   -237.18966   -400.83994   -223.03019      1.85256    460.37715    892.43882   1361.04659   1248.63887   1064.42506    704.14238    349.49445    -32.28678   -245.66631   -146.80403   -488.23168   -549.44144   -289.69019   -385.15513   -139.75003   -417.53406   -345.35436   -418.41868    -42.65079    -69.26436    294.09789    462.85705    442.60922    399.73257    235.81479          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       62.00   -197.87462   -417.17749   -484.70752   -351.14047    -186.3686   -236.69108   -262.34989   -506.74478   -373.35738   -276.94585   -224.79034   -376.15893   -187.39922   -451.17499   -430.17735   -216.61272    -288.1879   -437.92754   -141.25285   -340.52059   -525.67405     84.13074   -541.99199   -125.65773    166.53209    396.42502    426.07887    423.97327    398.82259    222.31023   -156.35776   -130.77918   -174.12142   -236.17585   -458.10494   -304.45715   -428.67652   -193.79304   -454.55361   -493.66971   -373.04601   -316.12308   -416.75663   -421.66535   -306.67031   -300.84572   -286.11506   -409.39964   -464.97541   -420.74667   -542.15542   -191.96421   -281.29645   -379.49042   -298.85175    -348.4734      50.3542   -117.21539     118.5441     68.63326   -301.07397   -157.29442   -190.70247    -439.3048   -214.36948   -301.21647   -196.99544   -398.65661   -185.14584   -139.00552    261.39359    837.57561    1128.5271    906.35722    1087.5495    494.91741     267.7911   -186.76399      6.02052   -178.34256   -163.60816   -387.30756   -362.45896   -368.59684    -409.1186   -356.89575   -335.47686   -403.93861   -228.68025   -108.86479    -82.12137    375.42941    377.19455    304.46205          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       62.50   -197.69573   -343.45471   -431.95517    -324.0813    -42.43567   -190.30321   -396.96182   -546.18956   -332.71156   -383.03599   -292.92049   -390.59977   -104.65318   -377.84259    -427.4226   -223.79332   -195.68044   -530.76315   -300.82893   -484.47633   -260.54931    -24.95435   -389.13349   -211.78934    155.47688    495.72932    403.65188     509.4863    171.62448    129.98917    -36.67648   -177.44112   -204.40668   -232.14151   -431.65344   -427.06032   -588.86936   -196.61221   -344.60946   -444.32047   -390.64853    -375.4483    -582.2724   -450.73294   -233.24642   -157.31039    -259.8876   -209.83278   -338.05719   -497.92688   -353.14461   -473.10371   -335.67285   -333.73393   -316.98181      30.8629   -291.60881    -98.45301   -130.04939   -152.06839    -63.01915   -145.46469   -184.72657   -150.37754   -322.09037     -382.288   -307.76846   -464.17601   -137.13498    204.15366     307.7918    711.46308   1054.84812   1097.37642   1031.13315     745.9294    347.60456     98.19913   -164.70821   -277.38317   -176.10868   -480.54899   -355.60001    -313.4189   -158.91802   -184.83022   -300.63174   -310.55163   -227.93517    -105.0871    124.98084    310.93943    419.52098    235.46256          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       63.00   -261.12465   -376.18851   -321.21398   -357.26698    108.16142    -32.30534   -581.59902   -590.79123   -405.48753   -459.01598   -218.27158   -517.86644   -175.61711   -342.82874   -387.53668   -580.55064    -12.42604   -384.09679   -565.46872   -546.12563   -211.38791   -212.32594   -358.91066   -242.16726    235.74334    421.66229    509.45983    598.59688    303.39309    321.23695   -218.80368     92.74191   -302.95196   -310.36514   -485.98673    -363.9724   -405.41828   -325.17422   -147.80385   -515.48894   -352.06947   -377.07911   -376.87661   -310.10403   -265.63453   -367.01548    -568.4528    -399.8481    -26.61017   -373.32815   -432.46618   -365.42869   -254.72966    -342.0532   -220.16434   -220.57151    -349.7006   -105.38383   -101.38948   -136.13351    -64.44959   -270.61237   -406.27007   -539.00926    -352.3862   -533.22405   -120.61241   -269.29615   -149.62603    -78.10127    332.22806    718.12624   1046.14758   1194.79772    790.91447     937.2369    206.56188     85.25813    -226.2607     -99.2256   -349.65622   -503.35287   -524.93324   -569.47479    -354.7509   -149.11994   -315.71029   -316.89253   -107.29512   -117.82228     76.32245     165.5026     50.32839          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       63.50   -353.57462   -507.64626    -189.2838   -417.59157    -23.14952   -244.87123   -553.25926     -379.823   -352.38415   -442.70012   -208.86674   -600.86236   -214.43133   -390.58043   -369.87029   -498.36996   -164.31902   -420.42153   -478.39487   -150.60695   -419.19189   -144.17237   -295.86901   -289.41698    180.13013    347.15894    515.11463    506.86998    191.95698     316.8275   -145.49722   -170.21786    -77.54594   -429.96392   -297.80814   -176.11043   -484.79339   -307.96532   -446.53765   -531.92611   -317.51118   -287.23406   -205.76221   -320.31276   -338.39535   -303.21431   -368.62832     -573.717   -383.96498   -579.15499    -75.83365   -241.63179   -374.29917   -228.05075   -262.35318   -234.96592   -208.55134   -151.65205   -163.17627    -19.12767    -62.33365   -165.09267    -79.68609   -378.08382   -514.29801   -425.18499   -354.78013   -269.59316   -145.01817   -138.22624    341.15605    588.87531    961.38798   1158.93604    750.40627    645.98926    379.50345     26.59011   -183.20773    -258.9401    -359.9161   -431.23984   -245.21636   -267.42985   -155.65674   -283.81018   -555.40358   -340.70751     -380.173    -99.98764    -79.80644     36.55594   -222.27735          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       64.00   -432.08121   -347.35679   -221.66168   -369.48616   -303.15337   -442.88568   -492.26808    -38.39537   -237.32094   -344.26221   -342.31132   -606.25237   -307.10576   -481.95366    -73.22433   -216.82264   -317.54104   -522.89329   -486.19205   -231.72248   -510.89477    -245.0636    -280.4542   -379.32777    219.09947    388.65976     323.2176    617.94758    313.71029    110.64547   -122.76781   -428.51078     -93.5986   -229.08764   -414.42633    -20.55955    -290.9899   -198.89638   -463.49365    -395.8375   -283.67144    -483.0264    -190.1672   -526.22455   -180.55378   -229.45093   -530.04677   -268.99544   -228.76277   -336.39774   -358.77466    -278.5881    -330.2647    -34.38605   -135.29752   -203.16083   -186.78662    -217.9921     30.71815    148.40693    -28.94437    -97.48039   -461.21686   -112.35149   -475.32506   -412.18505   -410.61563   -218.62259    -44.01714    -24.00472    310.70424    669.97924   1022.34579    959.25941   1335.72787    646.65999     295.0803   -176.19814     51.50814   -143.74333   -467.78342   -253.33004    -299.9891   -140.86066   -325.96946   -415.97776   -241.11224   -380.52791    -182.0535    -256.0565   -126.13004    -41.66686          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       64.50   -443.80065   -195.92888   -284.19897   -204.37206   -591.38711   -557.14578   -395.12394   -175.01755   -348.94728   -209.23759   -542.68188   -610.27742   -517.07741   -403.86222     47.73039   -394.28865   -289.56101   -255.01135    -445.9824   -311.01755   -493.33884   -243.52099   -290.83958   -181.46796     -1.33638    244.58361    395.18783    576.30726     595.8187     77.36506   -296.17374   -351.97767   -166.31293   -233.50769   -306.61684    -163.0802   -296.36773   -163.44489   -265.84018   -331.92241   -294.44773   -239.85092   -439.14115   -320.14526   -261.49791     -2.45832    -191.7019   -320.93999   -426.31247   -319.65569   -461.92985   -207.96132   -369.28541   -310.06211   -150.13084     14.16174    -38.10373    -70.03449    -18.58725   -179.23862     32.57529   -210.60771   -194.59052   -321.65678    -399.0137   -470.51096    -235.2423    -279.0018   -353.28459   -140.77012    269.06522    498.03534   1040.35491   1146.19097     982.0519    653.22654    243.69177     24.39295   -204.74846   -290.76711   -477.20221   -395.06386   -419.90859   -252.46673   -245.61599   -313.75016   -412.83832   -414.39489   -322.07825   -352.38944    -24.83629   -315.60466          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       65.00   -354.98725   -105.24728   -280.65343   -204.35147   -520.56167   -536.12929   -223.04573   -201.43709   -440.76684   -150.48634   -541.27883   -490.73706    -531.2325   -233.52238   -239.80475   -713.03274   -306.07476   -267.06853   -478.29235   -439.11684   -235.88767   -163.15331   -194.22223   -159.62518   -143.88571     95.12816     93.47899    454.50316    509.31323     106.8337    201.06444   -152.42689   -184.11088   -255.46388   -398.29093   -365.16889   -419.02938   -275.81769   -241.44458   -323.97974   -502.74009   -283.54647   -303.90093   -307.84306   -445.92773   -339.64273   -438.07912   -466.83876   -629.31569   -155.47061    -430.9505   -346.62207   -348.07876   -293.46199   -212.16434    -81.94096   -205.65092   -232.51886    -44.80605   -279.31418   -203.27454   -234.93725    -253.2643   -372.38356   -217.03288   -411.44961   -271.78294   -281.63357   -167.96197    -17.48679    253.06079    630.70401    781.32807    984.60559    919.07364    600.43162    265.08825     93.03309   -104.50762    -64.07649   -343.51393   -404.74587   -407.16313   -606.00097   -519.02761   -402.23062   -234.60651   -256.36696   -233.71798   -185.06646     -173.407          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       65.50    -208.0355    -194.7684   -275.87022   -289.05878   -393.07194   -435.96285      -69.219   -252.56637   -364.22808    -231.4527   -325.79958    -487.2932   -398.56149   -304.76914   -325.03936   -383.85494   -370.25265   -282.86229   -493.75056   -357.65023   -273.74793   -304.63417    -35.95344   -242.78299   -240.85011    112.68284     37.81697    393.07029    310.44632    247.61091    -31.10769     -97.0074   -255.22319   -431.14998   -474.19099   -442.64618   -277.41353   -328.10737   -489.67358   -358.57601   -372.02733   -574.23975   -353.56568   -412.05688   -324.42228   -421.88439    -567.8177   -255.11228    -336.8353   -183.87405   -323.71836   -200.64198   -268.65123   -122.11861   -329.63614   -346.45404   -121.31646   -156.68816   -210.86058   -213.06355    -41.44254    -122.6847    -31.75696   -256.77961   -443.76322   -256.11878    -65.72308   -250.61662     67.24978     -9.89881    456.98941    769.46215   1001.47559    894.99257    843.18159    479.88027    398.68469    122.89487    -47.81339   -222.92475   -424.75587   -270.47937   -280.92535   -458.22561   -379.10284    -368.7802   -348.03953   -492.24729   -380.94117   -277.86637   -304.87203          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       66.00   -182.89845   -293.51475   -312.63353   -338.51862   -282.40206   -526.58121   -125.78712   -444.37206    -275.9534   -340.72254   -248.89046   -498.52434   -455.88951   -314.24841   -296.48787   -322.03029   -357.32109   -267.17103   -464.33218   -225.66395   -232.07667   -364.15631   -225.73507   -237.56792    -80.18626     23.08402    145.08816     351.3311    223.40066    222.51221     15.94595    -21.71116    -199.2501   -377.36929   -436.62893   -381.01513   -303.16528   -340.19163     -198.928   -396.56887   -177.20593   -549.86816   -162.00127    -430.2466   -147.77864   -499.34064   -379.54957   -455.88927   -235.23039   -541.93261    -352.2198   -584.14806   -400.92453   -219.32702   -284.90994   -274.61954   -123.00356    -74.21899    -99.20113    -55.80864     68.87997   -124.75332   -159.88747   -272.34236   -402.13095   -528.64862    -296.2143   -198.61143     -83.7201    -16.95882    439.84445    671.41439    923.50539   1159.93616    957.06163    563.33413    448.96761    152.41402   -115.93939   -178.07826    -321.1936   -311.05131   -440.03546   -450.58742   -238.90486   -229.23514   -216.67506   -225.75835   -295.25846    -460.2942          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       66.50   -266.49973   -374.93317   -396.11438   -375.22407   -368.09489   -606.74393   -301.70747   -596.48242   -292.25022   -428.93174   -241.89659   -409.86604    -489.3057   -392.07857    -283.3164   -441.75487   -549.25681   -332.68837   -344.20347   -319.87579   -455.26428   -551.25247   -244.89515    -134.1967    -33.98236    -50.79014     24.42564    408.53761    411.88675     179.2342   -244.03861   -243.58249   -213.64223   -328.19477   -373.16013   -194.10176    -407.1674   -205.72683   -330.21534      8.90508   -192.55886   -309.52515   -236.44554   -386.86693   -639.17934   -463.09168   -243.75085   -363.87211   -261.29178   -346.58305    -360.8775   -264.51814   -232.63326   -370.87043   -258.44298   -462.53223    -53.74064    -262.2817    271.44762   -173.04938    -98.48135   -166.58647   -253.66681   -222.92226   -301.23452   -193.94521   -263.87373   -238.90002   -176.46143      6.21801    552.05354    684.21484    954.81216    893.78292   1057.43251    717.17755    381.78434    201.66209   -197.63388   -246.58727   -197.45174   -535.29826   -309.72171   -486.57287   -428.94895   -385.19057   -325.40124   -377.65209   -123.85813   -362.89555          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       67.00   -335.11055   -430.68375   -501.78034   -379.42395   -531.96951   -563.36266   -265.68321    -350.9631   -462.95379   -457.42607   -251.74126   -325.37024   -406.09855   -529.48372    -416.1493   -347.40993   -496.17157   -200.57911   -351.48689   -449.67189   -549.87178   -448.30947   -210.39947   -215.28062    -60.47514     -2.45971    213.14452    373.15797    301.31696     313.4464   -227.39403   -285.38531   -485.55448   -413.50415   -248.41314    -142.9829   -475.83039   -203.82777     -401.425   -268.91688    -16.45183    -87.38709   -396.20933   -225.93362    -239.0206   -518.64453   -402.09556   -320.82854   -388.19216   -173.86818   -538.09068   -305.48748   -386.43294   -348.12663   -391.16663    -38.07552   -439.35641     89.20924    161.71672    -40.74752   -228.06313   -278.88573   -311.89882   -123.04201   -373.05721   -205.78395   -382.28861   -254.39096   -251.55655    107.76767    182.11003    756.59457    806.94996   1139.64451    1329.5417    875.97026    450.82447    133.74064   -238.30173   -221.49747   -311.19509    -277.2747   -312.17231   -499.66032   -400.42729   -424.78342   -410.91275   -311.66373   -330.34656          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       67.50   -368.02378   -450.79696   -579.10867   -336.91812   -455.73669   -458.84191   -207.44559   -130.92332   -470.55524   -365.64795   -295.40518   -249.47361   -353.27525   -557.39015   -463.97412   -102.08442   -230.51294   -318.13715   -397.33504    -357.8305   -455.18272   -175.84078     -128.281   -438.34007      5.59236    -48.36547    486.61765     166.2524    289.67537    368.10839     28.61095   -168.13644   -186.59179   -282.60482   -259.87863   -385.49389    -419.8571    -383.9975   -470.14959   -359.59347   -141.50869   -161.55989   -419.62991   -381.80957   -177.73105   -401.49382   -296.71262   -328.17035   -316.60279   -222.41116   -300.95941   -341.33324   -274.34823   -591.13119   -317.65235   -373.13595   -238.48263      0.84869   -317.43776    -67.95784    -71.33966      3.58238    -76.33529   -239.14833   -166.09793   -209.23571   -295.91084   -219.38266   -234.58523     -37.0768    165.44552    480.89706     903.1386    893.92827   1026.94718    690.99743    556.20532     182.3647     -49.3063    -172.0533   -294.74741   -172.59302    -360.2712   -334.44936   -411.17393    -329.4489   -196.39458   -253.17075   -406.91461          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       68.00   -382.25564    -436.6514   -458.43342   -285.25862   -296.52203   -418.49497   -272.14712    -62.00673   -282.76306   -222.06484   -273.34932   -257.11545   -357.82873   -523.68979    -483.8364     -228.724   -287.59959   -752.50568   -352.23504   -430.75037   -361.10441   -162.71383   -127.47286   -466.35012    -87.22558    124.84146    263.54962    -76.94905    190.32128     26.57637   -152.38587   -152.21475    -43.36087   -361.03092   -240.37524   -427.35475   -519.86768   -319.99462   -488.94502   -414.63902   -276.26707   -474.15102    -485.2757   -403.18076    -260.5937   -310.12047   -284.40984   -317.46757   -397.20706   -364.27775   -534.20354   -304.60278   -452.97215   -369.99891   -133.00003    -198.7352   -120.62669    -73.64243   -162.86741      1.81216    -42.65195     20.13656     -187.243   -397.09446   -325.33205   -279.52958   -413.14757   -254.97353   -232.93157    -432.4079    295.84777    712.41101    949.12417    979.80382   1022.81933   1053.51669    475.71825     64.21434   -165.35235   -166.99728   -223.40578   -325.88985   -426.13497   -265.97205   -228.21323    -262.5031   -281.24186   -285.06725          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       68.50   -391.53835   -283.11197   -296.99363   -278.07512   -112.49573   -369.22111   -428.58273   -129.45023   -180.39307   -251.65443    -199.0707   -274.32125   -402.28094    -360.9913   -349.89117   -331.94214    -345.8504   -546.20342   -487.70141   -570.05945   -354.66962   -224.81746   -183.48408   -315.10028   -203.95714    227.01011     225.2061    -82.42667    325.83153    -69.67658   -120.81835   -311.11158   -286.07917   -399.96341    -384.7732    -164.8184    -468.8581    -403.5733   -572.25916   -441.75524   -436.52592   -470.30667    -558.2688   -550.72731   -199.46269   -330.51404   -375.72723    -380.7724   -222.03297   -275.70639   -305.42294   -443.79201   -464.71578   -312.05954   -272.78349    -96.69734   -119.70003   -131.28102     65.45307      61.1391    -36.68088    -35.99579   -201.26716   -268.08727   -231.51704   -345.18224    -172.3374   -391.29958   -250.59198   -156.47446    322.39127    700.17594    952.50667     1078.079   1085.09992    764.80606    537.87335    270.93281   -210.87551   -125.74761   -397.22627   -261.42451    -284.2957   -171.75552   -373.11678   -310.56647   -591.27305   -488.74474          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       69.00   -464.86739   -112.77962    -182.6004    -287.0234    -67.51536   -338.31796   -534.71642   -254.06506   -253.76635   -398.65457   -238.05625   -229.90787   -439.44033   -240.78029   -250.54964   -344.15644   -384.76117   -347.00336   -480.30357    -298.7255    -343.3197   -314.71141    -242.0138   -335.31085   -125.36096       90.961    202.46327     73.29108    255.21808    110.85116   -129.64785   -295.63558   -242.24412    -387.0867   -170.42521   -324.13193   -447.00919   -380.00501    -442.4934   -610.90392   -582.75505   -435.50761   -400.68467   -430.85668   -450.94709   -292.37447   -460.17472   -634.73061   -351.87659   -362.00336   -405.95215   -327.53714   -514.23691   -489.11589   -296.40967   -234.26481   -166.98178     90.75244   -103.87015   -175.61216   -155.09626   -264.14707    -91.27148   -547.61181    -46.92113   -314.25166   -389.63686   -300.51223   -268.23952    -25.48673    336.47372    589.35646    956.27859   1018.30961   1293.57463    852.96469    400.72249    191.99245    -187.4857   -266.91347   -361.49367   -327.81784   -222.60634   -172.05574   -324.55941    -553.7117   -380.15176   -489.46181          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       69.50   -522.91322     62.15732   -237.48177   -225.03843   -139.11759   -330.20291   -532.15959   -257.02552   -316.40093   -385.79608   -362.88093   -330.78678   -427.93973    -269.7245   -298.84158    -408.0057   -479.85019   -582.13755   -502.28779    -217.7612    -268.3039   -364.63568   -233.70535   -314.30706   -247.10586     51.30827    190.24675    176.46617    171.15462    244.56132   -198.42726   -403.20265    -91.95283     -452.764   -267.41717   -532.37047     -589.902   -313.03375   -415.31416   -106.31908    -311.3327   -398.71795   -488.10594   -217.47084   -351.20033    -286.3634   -490.22371   -191.83079   -388.13152   -479.74926   -344.34375   -324.21636   -348.98581   -551.44156    -392.6078   -129.31417    -24.19741      3.11364    124.81949    -84.02369   -111.13608   -194.77541   -368.37583   -182.75876    -227.6431   -315.38975   -286.90251   -297.64357    -43.34155   -145.62494     171.7738    475.84482     919.6164   1206.63241   1086.69363   1327.49472    648.39521    210.67498     -139.671   -265.18074   -399.57464   -306.74194   -300.72946   -432.66558   -321.10486   -316.37037   -498.88526          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       70.00    -553.7621   -164.59999    -362.3136    -77.11279   -263.05464    -343.4301   -531.69994   -302.58476   -428.75332   -209.69922   -432.47643   -338.48973   -365.21472   -320.36438   -269.33752   -394.48867   -536.49606   -495.98716   -657.08221   -359.95433    -239.3211   -271.35307   -209.90051   -295.91143   -210.38159   -158.87414    180.77698    289.83511    293.75764     47.12042    -72.43493     -203.385   -185.31587   -260.25024   -433.30872   -373.01323   -161.79092   -288.41522   -356.06391   -423.79098   -244.76616   -395.25185   -270.44032   -449.85064   -308.24888   -275.47604   -512.63441   -416.96126   -269.36091   -352.29368    -326.0984   -350.47919   -238.35486   -135.10396   -487.14933   -280.12689    -72.16693    -13.87955   -115.67137    -98.06113     65.58838   -274.19917   -264.92689   -187.97273    -384.0749   -596.38283   -199.06287   -284.25818   -315.57753    -71.14499     63.46667    463.46509    663.27049   1008.71274     939.4261    747.51061    502.74374     64.93271   -154.20728   -162.27985   -337.09161   -258.07347   -288.23468   -419.81837   -379.44336   -315.30678   -385.45657          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       70.50   -547.90082   -396.00107   -519.53878   -153.76383   -368.18489   -362.86983    -458.6619   -430.69938   -528.30238    -222.4863   -519.97799   -178.69943   -234.93645   -336.90376    -83.98006   -439.64157    -376.5248   -269.21778    -556.7583   -382.46857   -198.95596   -198.30197   -310.73705   -201.04258   -226.70001   -124.02137    121.09076    290.94536    254.95434    103.44314   -188.47286   -252.42112   -208.84958   -204.40279   -440.08387    -331.6167   -491.25906   -277.28964    -298.8405   -390.86614   -142.74268   -410.32755   -355.65981   -431.74623   -305.15279   -477.79695   -511.64006   -563.45879   -407.27519    -441.5662   -305.13213   -383.53822   -276.01451   -368.14808   -555.11992   -184.06621     -29.5962    -280.4986     10.40528   -177.65935    -16.73692   -176.36543     -303.166   -180.99085   -206.58719   -252.57628   -168.62871   -149.58697   -311.89565   -150.50412      63.1023    345.17308    921.97709    996.14323    931.69414    736.18767    345.79662     140.7053   -281.34676   -343.14054   -337.38088    -302.0899   -412.31026   -281.46519   -266.81246    -389.0006   -303.07043          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       71.00    -557.5207   -552.29149    -510.3926   -336.00268   -424.69858   -613.00647   -381.52505   -528.77217    -549.8548   -222.61442   -494.80434   -217.02081    -57.66734   -378.00224    -92.25239   -439.46147   -350.90348   -416.53891   -212.61937   -328.15769   -241.36348    -88.61933   -423.82444   -107.46788   -305.06547    -37.69163    247.24179    432.11321     201.5276    124.15917    -18.53431   -416.48228   -299.81338   -379.79526   -323.67017   -399.91591   -350.55078   -267.83287   -248.81511   -442.91409   -256.90491   -473.33712   -322.47962   -211.93157   -314.85803   -625.21291   -683.85338   -270.44619   -356.88074   -473.35178   -196.14161   -375.14812    -378.6271   -278.17883   -444.43973   -111.19352   -315.55652     104.3271     23.13001     62.02264    -36.54644     24.24833   -309.28949   -262.99983   -329.95684   -191.75745   -305.98287   -191.85751   -396.88843     -121.186    303.08249    667.36783    987.55362   1220.66195   1150.61237     797.8584    442.59357      2.22165   -191.55171   -356.45025   -349.90631   -348.62368   -237.67686   -251.64771   -326.68513    -454.6665          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       71.50   -567.43305   -400.51505   -463.00723   -452.25381   -242.40176   -910.42436   -360.90565   -598.19586   -488.96243   -197.30362   -373.70073   -295.24037    -203.1469   -607.83747   -181.82587   -432.77258   -314.57675    -599.9633   -330.32895   -525.26241   -333.30446   -128.39433   -429.01983   -185.27099   -182.26018    -26.41289    344.71461    283.27081     59.82219       3.0424   -192.26509   -354.16429   -298.77545   -319.10939   -258.63317   -423.69196   -118.65789   -197.25961   -315.81164   -258.72623    -615.6801   -217.68162   -231.92678   -141.13609    -341.9189   -433.53681   -616.84073   -181.37976   -371.58638   -303.71268   -259.05704   -470.65886   -406.70086   -308.33608   -145.40417   -278.61053   -196.28509    -33.69736   -123.41353    -83.38495     -74.2463   -129.14213   -111.01773    -308.3886   -182.92741   -351.49526   -392.46295   -185.27159   -114.85256     16.84182     205.2222    501.71118    731.43793   1064.51613   1197.23825    549.41772    581.95525     99.88206   -124.35464   -107.70425   -212.64817   -493.71099   -467.12871    -254.2635   -383.77372   -400.88769          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       72.00   -632.14194   -365.92666   -461.05199   -332.88056    -17.62577   -975.96106   -413.19807   -686.04284   -295.76171   -235.41634   -343.21758   -189.86103   -311.49904   -538.08643   -322.76679   -406.02423   -448.29505   -721.52198   -578.55248   -484.38235   -409.84523    -125.7212   -435.05276   -257.91765   -104.02123    -34.45694   -187.19349    156.17305   -158.15039      12.8885    -32.18945   -209.61735   -434.62683   -399.07486   -441.20021   -333.16153    -65.71631   -335.95118   -414.56534   -175.15019    -203.9509   -305.51929   -217.98273    -518.1313   -397.02624   -337.35317   -407.07297   -343.87893   -249.46234   -513.07957    -52.98946   -764.74457   -275.25695   -233.55286   -417.05085   -358.15001   -232.26728    -95.27003    -146.6854    -130.1108     57.69081   -303.33428   -384.29483   -563.54994   -427.30488   -341.88545   -270.71344   -365.49064     28.53026   -190.13688    118.25507    304.69911    478.90355   1119.00819    989.45787    784.17521    699.35684    112.27131    -87.72604   -336.55631   -352.27925    -456.2536   -357.79412   -460.26153   -411.10028   -557.72614          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       72.50   -672.96949   -463.18688   -512.46343   -333.48293    -64.45975   -764.14823   -464.08562   -578.64466   -178.13395    -239.2836   -387.32522    -23.20847   -328.08046   -451.19089   -235.34753   -384.34537   -581.26939   -479.01619   -201.09287   -343.76581   -534.09913   -366.79549   -410.22046   -386.86436    -48.77272     22.14112   -158.50057    -67.59353    235.76395     103.3168     35.83434   -393.89048   -389.89725    -448.9112   -501.32804   -391.37936    -348.0736   -442.08263   -377.12552   -251.55595   -314.71233   -368.80528   -679.12048   -566.53353   -584.90796   -474.42052   -361.48854    -321.0128   -298.81649   -344.77896    -263.0268   -352.21913   -241.71752   -345.89036   -307.29456    -466.6441    168.38461    -58.19188    -83.75754     23.80345      6.71677    -40.82583   -397.12716   -395.16461   -325.49315   -250.71083   -292.50903   -371.62229   -310.01831     59.75338    302.57688    264.49994    943.68766     937.7052    763.32328    820.85119    516.92506    -11.77728    150.49939   -276.50889   -408.04065   -310.04074   -226.80832   -468.14197   -370.85455          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       73.00   -692.24755    -618.0288    -511.1739    -335.7118   -234.39816   -635.38859   -448.46758   -466.51722   -243.60559   -122.77027   -408.24978     84.52597   -348.78169   -427.01242    -62.18492   -434.36962   -406.83705     -143.458   -164.33539   -391.78163   -489.85805   -286.02833   -430.59443   -380.21463   -419.31882     58.60361     59.64738     -80.2956    372.59566     46.12731   -209.55459   -238.62503   -480.96411   -522.96053   -446.76269    -671.7983   -280.97357   -440.56076   -304.74098    -378.7529   -411.07714   -380.10295    -571.1036    -295.0589   -428.02882   -505.84231     -369.736   -334.36809   -412.05917   -437.42363    -405.3006   -423.19966   -176.67704   -362.40717   -488.22684   -456.96056   -185.63689      4.26227    -97.64605    145.70251    159.94725     23.16553   -350.46384   -237.99833   -404.83546   -293.87786   -462.76883   -210.05408    -232.4879   -168.69116    132.00216    458.57781    844.17757    909.55289    751.89075    573.26866    437.76761    314.31525     -61.3141   -159.92678   -249.89745   -339.63381   -366.15507   -139.51591   -359.62906          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       73.50   -741.63943   -688.16627   -484.28487   -253.88236   -368.04105   -491.01165    -419.8027   -389.02917    -320.9965   -159.04158   -323.89777    195.15245   -296.51175   -379.43508    -43.11045   -218.21319   -272.93441   -159.89856   -389.65031   -322.12804    -258.5428   -194.60315   -180.15798   -318.35788    -362.6554   -224.78695      79.9722    176.46108    220.26423    -73.91071    -293.4723    -160.6021   -322.95809   -464.45194   -435.38713   -400.36881   -358.65019   -325.93254   -402.75534   -358.52532   -359.65499   -311.58986   -327.30595   -266.94722   -455.45492   -469.68404   -398.61523   -456.98583   -420.39508   -522.61091   -353.55126   -397.65056   -332.87425   -482.98711   -210.70992   -426.75948    133.74275   -296.72598   -173.54563      70.4768    -97.71762    -142.8404   -318.51215   -101.28408   -253.10041    -273.0605   -412.19615   -390.72252   -204.56749   -286.07856    372.46581     449.8257    856.61517    868.04266    989.64755    752.00257    644.60757     73.39757    -90.44759    -96.33723     -300.095   -516.29289   -313.87798   -358.97094   -378.97295          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       74.00   -804.21893   -664.88777   -520.82027    -137.1432   -281.51609   -340.34327   -294.04174   -357.29638   -374.49874    -385.8816   -315.84394   -130.76402   -237.89174   -269.88853    -99.13963   -146.50832   -178.48926   -192.61033   -344.43733   -212.79034   -117.79791   -367.04968   -249.59535   -175.73239   -196.41511       14.261    -115.9644    177.37911    304.69616    -75.32653    -322.8933   -357.32703   -249.86404   -250.52386    -500.9204   -292.48703   -279.86331   -312.00969   -372.51229    -394.8761   -489.85398   -372.00796   -128.02105   -303.92033   -537.15057   -310.73551   -138.77019   -544.36515   -424.93076   -332.93833   -320.65868   -367.81767   -325.24778   -376.65385    -223.4616   -280.62099    -36.85248   -337.04252    316.57292     15.83996   -119.24628   -358.11737   -130.33642   -268.27257   -395.22888   -496.60035    -91.04628    -398.2031    -286.3819    120.19414    257.04618    820.82328    689.68127    987.16598   1243.25479    852.47613    671.44146    341.55763     19.19418   -215.82023   -230.68989   -323.07041   -353.34133   -425.52071          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       74.50   -877.44967   -595.08882    -625.0886    -268.0984    -85.03784   -307.57328   -127.11738    -339.0239   -430.64322   -449.27931   -365.89888   -301.97264   -218.40741   -222.25288   -112.54959   -355.31968   -142.11722    -217.2568   -372.92405    -426.3543   -227.19231   -375.51252   -387.38407   -238.28975   -196.58739     11.05166   -100.98848      42.6766    107.87855     73.74582   -165.90607   -582.88975   -244.80065   -126.27365    -439.5619    -25.92549   -333.16292   -443.73365   -374.53322   -351.55045   -401.15454   -231.07786   -223.23813   -368.21919   -350.85704   -434.96706   -353.19793   -449.53577   -673.68751    -364.3294   -470.27149   -198.04229   -503.70864   -351.43042    -156.5942   -365.98753   -119.97945     44.67164    -85.73206   -179.45074     22.97503   -235.80775     83.08754   -386.08377   -152.81856   -241.27528    -389.6798   -213.76226   -239.72272    -47.37739    389.87556    528.66486    686.56399    743.56225    961.73427    934.75928    707.11576    343.33318   -185.82679    -16.24356   -315.68351   -316.35209   -292.93118   -450.99933          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       75.00   -917.65163   -523.85186   -591.68153   -418.64026    -42.95261   -334.85201    -49.98264    -296.5917   -453.50845   -373.57375   -301.59353   -161.62363   -211.93347    -364.3885    -153.7091   -304.63137    -189.6216   -331.64909   -578.12331   -654.71619   -636.32777   -475.65696   -424.71605   -282.07076    -20.74464   -188.38938     67.95931     22.81113     46.31469     71.93338    -84.70441   -364.19355    -198.4068   -447.38038    -369.7238   -283.03673   -392.04184   -351.36342   -424.05043   -447.04242   -331.71551   -304.11224   -388.70186   -262.24975   -241.89098   -538.79411   -205.05513   -409.57076   -326.88779   -358.18735   -321.98842   -179.01492   -333.47044    -507.1653    -393.3135     -87.3846   -105.45035    -53.09535    -70.40454     -2.67591   -243.29252   -180.24839   -279.03753   -328.92693   -436.08797   -400.31238   -179.04161    -400.3466   -260.82038   -267.71551    125.90128    919.95843    745.28356    815.93611   1080.23421    928.26849    515.90409    199.38567     -53.0865    -151.1977    -309.5115   -360.64039   -312.31966   -694.84333          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       75.50   -936.89579   -483.84246   -482.08446   -474.45335   -184.84314   -408.40306    -82.46921   -331.76626   -479.06449   -376.77455   -194.60054   -149.66189   -242.89272   -357.13027   -389.73254   -185.40948     -318.292   -333.65253   -522.78039   -721.48199   -453.05237    -655.8753   -227.28019     47.07587    -47.06382    -80.41969     20.92538    193.53172     15.56593   -168.84796    -85.18743   -494.84207    -266.7682   -432.40008   -370.13706   -404.57825   -240.90768   -514.21907   -472.96668   -526.17453   -208.60639   -236.15504   -503.59428   -334.77797   -332.01599   -546.44067   -492.18204   -391.80587   -176.43968    -439.5361   -552.71286   -296.63714    -354.3576   -402.36565   -419.30544   -157.50349   -179.77886    -22.61259    -83.97835   -150.22241     -1.50235   -168.45692   -188.73929   -475.42759    -239.7976   -342.23717   -285.00447   -395.17782   -254.42349   -157.79195     -52.6234    377.47592    557.27832    886.84844    843.08054    812.21629    628.22927    221.93454     65.32688   -363.33084   -341.39391   -303.51543   -427.35686   -288.61917          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       76.00   -930.87832   -439.48826   -412.30553   -360.13694   -360.67162    -519.7157   -160.11893   -509.20806   -504.64517    -422.6685   -242.91508   -397.40253   -277.31939   -230.90518   -411.65847    -440.9808   -299.62552    -78.46324   -372.68799   -545.16508   -315.05418   -478.35144   -162.07587   -238.78094   -225.17005    -89.65817    -60.44071    133.87727     20.26155    -61.37029    -81.70897    -305.7886   -229.52631   -360.01802   -347.01148   -527.83324   -523.66392   -352.72553   -379.60786   -381.77775   -399.46844   -395.17668   -325.58216   -320.47656   -435.97549   -427.55683   -259.90228   -315.18531   -201.17133   -166.40904    -304.5031   -210.97803   -433.97597   -213.22897     14.21792    -98.32471   -157.40174   -246.15567     17.65049   -134.79008   -285.88707   -160.70356   -295.72983   -142.86979   -187.10615   -269.43315   -526.69241    -314.9787   -324.87595     -92.0174    255.15867    597.64725    660.69208    915.18871    969.34682   1009.97909    390.58639    327.90148    103.52319   -210.54281   -275.73045   -371.99513   -253.50037          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       76.50   -725.82528    -399.1408   -526.71221   -267.66509     -396.038   -602.45354   -322.61854   -646.82274   -414.29768   -448.81744   -443.26607   -544.17159   -442.74821   -206.03931   -219.25988   -544.12886   -378.21322   -169.50619   -266.89663   -376.29792   -309.37692   -389.75158   -223.84771   -220.24032   -489.46404   -246.93076    -120.4756     12.90617     96.33813     -74.5958     67.34661   -378.95669   -123.01435   -207.80783    -427.6557   -388.75339   -431.23405   -374.48807   -424.17803   -368.81734   -371.65456   -330.21487    -338.2834    -372.6279   -356.06074   -365.81655   -321.30092   -422.18342    -314.5627   -379.70769   -362.82236   -230.78511   -461.79665      78.7701   -137.38765   -222.83847    -82.26832   -157.84324     74.93015    145.03043   -134.72807     -20.0976   -236.16893   -265.70815   -245.05677   -313.16449   -494.09617   -444.87239   -158.07398   -139.29981    267.17397    466.23259    858.23104    837.39277    1251.8966    766.88071    553.89975    249.86579   -154.72633   -113.77141   -246.65996   -329.31339   -290.97087          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       77.00   -515.12676   -397.98605   -594.81059   -217.76716   -456.25993   -481.89652   -475.16412   -542.80103   -304.25352    -413.2262   -458.35672   -496.01388    -564.6346   -212.53688   -136.96301   -283.85703   -380.92905   -556.39418   -273.22148   -271.35761    -65.77051   -471.50095   -193.47268   -311.04472   -332.61923   -295.21432     88.76331   -101.95538   -207.78033   -106.01989   -248.40756   -136.20229    -86.32198   -302.62061   -245.49897   -367.20915   -362.26069   -267.43022   -391.80726   -203.42875   -308.26191   -490.33592   -523.24318   -291.18104   -316.21694   -328.42159   -533.09739   -159.34062   -345.99676    -381.5499   -421.42794    -449.6464   -214.91154   -324.34979   -376.15582   -378.58869    -58.27677     -3.92974    -147.0096   -216.56182    -58.67255    -43.06076    -79.11368    -375.2018   -281.23031   -176.96127   -488.79189   -375.04431   -194.13734     37.90267    -37.27658    351.45449     541.4543   1000.99273   1009.46588    850.17823    917.32372    364.98406    129.55828   -187.55308   -408.86474   -351.61667   -340.47213          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       77.50    -334.2441   -409.75561    -551.1645    -168.2224   -570.22873   -364.59122   -587.37147   -426.17715   -209.54526   -358.11559   -176.77866   -397.61555   -619.74704   -291.15453       5.4694   -385.08821   -358.09848   -428.25081   -247.72964   -165.46114    -24.06894   -487.77734   -298.93579     -543.788   -363.16775    -42.09772     -3.04054   -142.17319     12.16941    -43.81747   -213.53599   -141.59757   -277.65215   -283.11238   -432.21898   -450.88361   -288.72655   -165.44433   -280.13974    -479.6948    -383.4967     -620.942   -315.95484   -374.50144   -361.29093    -380.8727   -291.43876   -284.01973   -276.48535   -260.02926    -478.7531   -530.44327   -295.52096   -327.34428   -318.31993   -256.20291    -28.21979     35.47875   -108.45627   -241.31466   -118.35644     30.10514     31.09574    -215.8805   -250.02749   -375.66382   -459.04511   -435.15004   -119.89656   -222.62833    -41.91697    322.29027    838.53897   1247.69442    952.33795    878.86382    711.18014    308.57442   -117.56271     -77.5296   -291.88122   -276.61167   -351.56685          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       78.00   -309.56391   -421.03707   -457.37246   -211.66927   -670.12773   -378.26133   -687.56331   -307.66444   -164.45204   -347.60281    -86.63967   -324.97189   -503.51897   -423.98437   -138.27229   -546.58934   -348.08236   -281.61815    -17.55803   -131.02073   -345.06192   -384.34368   -461.57709   -302.90259    -343.6483   -212.42039     15.00274    216.77035    175.03058     66.12538   -276.22865   -273.19075    -333.8964   -247.85276    -431.2905   -381.58298   -372.31539   -285.47879   -292.91558   -437.10734   -355.92592   -259.31169   -275.05483   -299.30463   -391.24978   -397.48665   -376.20059   -327.19406   -230.09996   -275.74753   -154.38824   -257.98817   -328.12916    -226.5322   -228.55856   -158.01755   -296.16912    -57.05818     11.27189    -19.35779    -79.15302    -57.70963   -265.41366    -323.3719   -308.60668   -346.42888   -435.13479    -399.3036    -407.4026   -218.25978     -148.131    221.88183    768.21446    878.95012    908.51527     950.0809     740.1029    220.86771    195.64623    -81.74679   -468.51124   -526.53101   -255.33061          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       78.50   -247.26455   -471.46889   -402.38674   -409.24662   -684.05764   -446.50385   -606.40109   -190.75305   -274.00825   -350.81567   -128.86375   -357.61736   -436.38921   -342.22675   -274.73705    -396.9667   -435.01837   -216.13161   -257.24802   -238.30376   -294.33669     -377.336   -318.43067   -163.68855   -295.33421   -153.19276    178.24017     49.92766      66.6684    -74.75978   -136.16926   -267.30389   -276.18503   -159.25347   -234.41026   -247.94501   -198.72732   -313.76561   -302.38435   -547.36995   -202.69388   -309.67881   -335.25969   -410.04144   -295.77927   -407.89518   -448.88303   -468.29849   -236.53711   -445.49677    -251.9726   -296.38658   -255.96938   -297.71716    -355.9598     -214.784   -110.81053    -169.3383    168.18248    180.11888    -64.75621    -72.30836   -137.11418    -209.9982   -389.60454   -222.93886   -223.35969   -175.02868   -254.95579   -424.93135     32.04447    179.52504    830.13161    907.20746    912.76698   1022.86153    852.28908    427.56427    -29.02477    -128.7628   -150.63338   -398.56963          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       79.00     -191.357   -524.51637   -572.39208   -590.41662   -546.22614   -449.75845   -500.01361   -152.41273   -373.08394   -255.64513   -165.86675   -456.90786   -407.85958   -215.30791   -325.55623   -235.35478   -584.20133   -269.32694   -514.19817   -472.67253   -246.21548   -611.63829   -489.25736   -205.91261    -69.90812   -210.38449   -109.02782    -31.36231   -205.00369   -143.05408   -232.27544    -318.8844   -295.25442    -20.57332    -361.8228   -341.48489    -88.50543   -265.48104   -418.97712   -424.22767   -252.07574   -345.10148   -108.94924   -379.37943   -153.46535   -395.67865   -385.87816   -341.12435   -155.62421   -335.85201   -413.59229   -187.36234   -408.37919   -292.54087   -268.63938   -270.39465   -513.24897   -119.31189   -159.51316    -174.6724    -44.60244   -323.71799   -220.28369    -228.4638    -234.9735   -166.14119   -165.21003   -196.25417    -36.79237   -186.48601    199.93134     281.1105    713.13698    891.19289    897.35726    846.12577    631.74321    310.95221     27.32342   -143.20084   -205.25042   -285.76042          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       79.50   -282.40753   -595.91552   -805.25262   -546.25535   -373.95952   -335.11133   -492.34976   -126.32746   -533.59705   -141.58377   -224.11725   -419.56019   -314.22936   -294.02978   -364.37225   -134.61536   -451.10216   -307.40595   -425.25108   -520.77895   -185.46495   -277.74698   -501.93026   -265.87469    -62.23796   -303.80804     67.61872   -195.11059    -52.90405   -161.34423   -368.43778    -99.94786   -296.07065   -173.92183   -248.38403   -273.95974   -332.79156   -291.08784   -401.55294   -223.00023    -239.1733   -405.97148   -363.77542   -107.53591   -320.76637   -153.40802   -412.35691   -428.80017   -107.37033   -172.21611   -245.46953   -345.16505   -219.07865   -163.08848   -345.61928   -113.63792    -162.7367    -53.38856   -151.83578      5.74595      7.42165   -106.04105   -284.42647    -335.7154   -210.15982   -329.31149   -283.80665   -411.63274   -168.73653   -280.50969    149.09487    507.77267    806.65422     760.1184    752.43362    699.29001    660.40729    263.00841     63.94198   -278.09674   -147.99749   -368.48408          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       80.00   -415.60603   -756.04936   -974.24548    -486.2735   -373.27947     -222.746   -524.12208   -161.53505   -703.41996    -43.28048   -189.46099   -339.55578   -345.71543   -399.58403   -283.56471   -219.31739   -375.05698   -271.09664   -373.65473   -502.68124   -412.51639    -142.7559   -286.60872   -402.03877   -112.15137   -314.86726    -59.38157   -290.17011     95.80588   -176.99834   -234.27267    -367.0441   -317.67251    -36.25947   -333.71054   -168.50386   -272.97435   -334.08341   -307.62095   -420.44603   -395.34939   -551.74624   -183.72217   -263.31379   -533.58368   -316.30312   -565.74334   -482.30078     -402.616   -487.56466    -327.5556   -247.92572    -201.1052   -122.09788   -340.92013   -334.55363    -95.02816   -132.81941   -106.84172   -240.57189    -43.99911     -5.89024   -144.95535   -358.75122   -391.41211   -215.43739   -558.37677   -310.35342   -275.60068   -127.04337    -64.77177    387.60237    505.14945    854.01978   1034.36811    904.25589    728.81356    337.63569    124.66343    145.20652   -231.26874   -181.88248          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
;


_pd2d_meas_2theta_phi_intensity_up_sigma
;
         241   4.00         4.20         4.40         4.60         4.80         5.00         5.20         5.40         5.60         5.80         6.00         6.20         6.40         6.60         6.80         7.00         7.20         7.40         7.60         7.80         8.00         8.20         8.40         8.60         8.80         9.00         9.20         9.40         9.60         9.80        10.00        10.20        10.40        10.60        10.80        11.00        11.20        11.40        11.60        11.80        12.00        12.20        12.40        12.60        12.80        13.00        13.20        13.40        13.60        13.80        14.00        14.20        14.40        14.60        14.80        15.00        15.20        15.40        15.60        15.80        16.00        16.20        16.40        16.60        16.80        17.00        17.20        17.40        17.60        17.80        18.00        18.20        18.40        18.60        18.80        19.00        19.20        19.40        19.60        19.80        20.00        20.20        20.40        20.60        20.80        21.00        21.20        21.40        21.60        21.80        22.00        22.20        22.40        22.60        22.80        23.00        23.20        23.40        23.60        23.80        24.00        24.20        24.40        24.60        24.80        25.00        25.20        25.40        25.60        25.80        26.00        26.20        26.40        26.60        26.80        27.00        27.20        27.40        27.60        27.80        28.00        28.20        28.40        28.60        28.80        29.00        29.20        29.40        29.60        29.80        30.00        30.20        30.40        30.60        30.80        31.00        31.20        31.40        31.60        31.80        32.00        32.20        32.40        32.60        32.80        33.00        33.20        33.40        33.60        33.80        34.00        34.20        34.40        34.60        34.80        35.00        35.20        35.40        35.60        35.80        36.00        36.20        36.40        36.60        36.80        37.00        37.20        37.40        37.60        37.80        38.00        38.20        38.40        38.60        38.80        39.00        39.20        39.40        39.60        39.80        40.00        40.20        40.40        40.60        40.80        41.00        41.20        41.40        41.60        41.80        42.00        42.20        42.40        42.60        42.80        43.00        43.20        43.40        43.60        43.80        44.00        44.20        44.40        44.60        44.80        45.00        45.20        45.40        45.60        45.80        46.00        46.20        46.40        46.60        46.80        47.00        47.20        47.40        47.60        47.80        48.00        48.20        48.40        48.60        48.80        49.00        49.20        49.40        49.60        49.80        50.00        50.20        50.40        50.60        50.80        51.00        51.20        51.40        51.60        51.80        52.00        52.20        52.40        52.60        52.80        53.00        53.20        53.40        53.60        53.80        54.00        54.20        54.40        54.60        54.80        55.00        55.20        55.40        55.60        55.80        56.00        56.20        56.40        56.60        56.80        57.00        57.20        57.40        57.60        57.80        58.00        58.20        58.40        58.60        58.80        59.00        59.20        59.40        59.60        59.80        60.00        60.20        60.40        60.60        60.80        61.00        61.20        61.40        61.60        61.80        62.00        62.20        62.40        62.60        62.80        63.00        63.20        63.40        63.60        63.80        64.00        64.20        64.40        64.60        64.80        65.00        65.20        65.40        65.60        65.80        66.00        66.20        66.40        66.60        66.80        67.00        67.20        67.40        67.60        67.80        68.00        68.20        68.40        68.60        68.80        69.00        69.20        69.40        69.60        69.80        70.00        70.20        70.40        70.60        70.80        71.00        71.20        71.40        71.60        71.80        72.00        72.20        72.40        72.60        72.80        73.00        73.20        73.40        73.60        73.80        74.00        74.20        74.40        74.60        74.80        75.00        75.20        75.40        75.60        75.80        76.00        76.20        76.40        76.60        76.80        77.00        77.20        77.40        77.60        77.80        78.00        78.20        78.40        78.60        78.80        79.00        79.20        79.40        79.60        79.80        80.00        80.20        80.40        80.60        80.80        81.00        81.20        81.40        81.60        81.80        82.00        82.20        82.40        82.60        82.80        83.00        83.20        83.40        83.60        83.80        84.00        84.20        84.40        84.60        84.80        85.00        85.20        85.40        85.60        85.80        86.00        86.20        86.40        86.60        86.80        87.00        87.20        87.40        87.60        87.80        88.00        88.20        88.40        88.60        88.80        89.00        89.20        89.40        89.60        89.80        90.00        90.20        90.40        90.60        90.80        91.00        91.20        91.40        91.60        91.80        92.00        92.20        92.40        92.60        92.80        93.00        93.20        93.40        93.60        93.80        94.00        94.20        94.40        94.60        94.80        95.00        95.20        95.40        95.60        95.80        96.00        96.20        96.40        96.60        96.80        97.00        97.20        97.40        97.60        97.80        98.00        98.20        98.40        98.60        98.80        99.00        99.20        99.40        99.60        99.80       100.00       100.20       100.40       100.60       100.80       101.00       101.20       101.40       101.60       101.80       102.00       102.20       102.40       102.60       102.80       103.00       103.20       103.40       103.60       103.80       104.00       104.20       104.40       104.60       104.80       105.00       105.20       105.40       105.60       105.80       106.00       106.20       106.40       106.60       106.80       107.00       107.20       107.40       107.60       107.80       108.00       108.20       108.40       108.60       108.80       109.00       109.20       109.40       109.60       109.80       110.00       110.20       110.40       110.60       110.80       111.00       111.20       111.40       111.60       111.80       112.00       112.20       112.40       112.60       112.80       113.00       113.20       113.40       113.60       113.80       114.00       114.20       114.40       114.60       114.80       115.00       115.20       115.40       115.60       115.80       116.00       116.20       116.40       116.60       116.80       117.00       117.20       117.40       117.60       117.80       118.00       118.20       118.40       118.60       118.80       119.00       119.20       119.40       119.60       119.80       120.00       120.20       120.40       120.60       120.80       121.00       121.20       121.40       121.60       121.80       122.00       122.20       122.40       122.60       122.80       123.00       123.20       123.40       123.60       123.80       124.00       124.20       124.40       124.60       124.80       125.00       125.20       125.40       125.60       125.80       126.00       126.20       126.40       126.60       126.80       127.00       127.20       127.40       127.60       127.80       128.00       128.20       128.40       128.60       128.80       129.00       129.20       129.40       129.60       129.80       130.00       130.20       130.40       130.60       130.80       131.00       131.20       131.40       131.60       131.80       132.00       132.20       132.40       132.60       132.80       133.00       133.20       133.40       133.60       133.80       134.00       134.20       134.40       134.60       134.80       135.00       135.20       135.40       135.60       135.80       136.00       136.20       136.40       136.60       136.80       137.00       137.20       137.40       137.60       137.80       138.00       138.20       138.40       138.60       138.80       139.00       139.20       139.40       139.60       139.80       140.00       140.20       140.40       140.60       140.80       141.00       141.20       141.40       141.60       141.80       142.00       142.20       142.40       142.60       142.80       143.00       143.20       143.40       143.60       143.80       144.00       144.20       144.40       144.60       144.80       145.00       145.20       145.40       145.60       145.80       146.00       146.20       146.40       146.60       146.80       147.00       147.20       147.40       147.60       147.80       148.00       148.20       148.40       148.60       148.80       149.00       149.20       149.40       149.60       149.80       150.00      
      -40.00    233.54349    224.10645    208.27566    216.41348    205.63116    216.02289    204.60888    197.05559    186.23034    183.14186    184.47784    179.19186    186.02513    181.23491    182.75614    176.69782     167.5986          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -39.50    232.66974    223.49483    209.67221    217.60858      209.059     220.4587    201.25281    198.89958    188.39998       177.37    185.73035    182.52026    184.40737    179.30679    180.05557    173.97157    169.35174          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -39.00    232.51012    223.97423    215.90039    218.09426    211.72563    216.21584    199.31511    196.81975    188.71256    184.65146    186.43862    183.56351    181.07582    181.70731    175.49167    170.83482    169.69325    167.02211          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -38.50    232.61476      224.171    221.85087     217.1526    213.16458    211.96458    196.77904    193.95383    189.21666    186.24361    186.57205    183.98342    182.44787    183.83478    174.68287    169.67856     170.3779    162.26836          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -38.00     232.5503      222.426    226.86797    215.82756    212.43038    211.02957    195.80521    192.92192    192.75947     186.2519     184.0049    186.04187    185.59195    183.75307    172.77978    168.97817    172.75163     166.7277    158.95614          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -37.50    231.53965    219.79472    224.41141    214.12238    212.57219    210.51038    198.28605    192.71909    197.71459    189.18966    186.22118    187.75275    185.64275     182.6954    176.61038    167.88444    169.78491    166.51928    164.54764          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -37.00      229.571    216.42607    221.95407     215.9899    213.95626     209.4996    199.56638    192.70378    198.87677    190.69984    190.58618    185.44325    185.54426    181.28515     175.2209    167.26843    169.81113    169.83847     171.9999          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -36.50     228.9676    213.77403    223.14889    220.70069    218.80729    207.53389    198.57968    193.94826    193.17537    191.35845    189.09932    181.56635    191.54335    180.06618    171.49689     167.0191    173.53623    168.27729    172.63965    166.19627          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -36.00    229.55962    221.57707    226.93724    224.78615    220.78543    205.26777     196.1312    195.32487    191.23812    190.67976    187.77314    182.69407    190.49643    178.88234    170.51658    166.65738    175.54164    168.48932    169.04799    162.96702          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -35.50    230.14804    227.74761    227.54743    223.19286    219.69472    203.09549    196.79081    194.88663    191.48823    188.74749    188.57533    186.73938    185.22285    178.49354    174.94622    168.69647     184.4311    170.18429     169.7373    166.87068    159.44535          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -35.00    231.61355    231.95199    227.46889    217.83406    217.54551    202.18693    199.37194    194.94057    192.73565    183.60377    188.09477    188.45244    188.54325    175.69517    181.23807     173.8582    178.27102    173.71431    171.24715     169.4364     149.7518          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -34.50    233.54579    231.51579    228.42689    213.80791    216.75867    202.53424     199.2307    197.71553    194.73277    179.89582    185.90347    191.76891     194.7271    175.63607    180.40006     177.4566    175.12981    176.41758    171.07216     172.8724    159.17606      163.769          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -34.00    234.15662    230.23873    227.96721    213.94891     216.9032    203.32533    196.42059    201.25748    194.96653    185.97888    185.96423    196.00146    193.38419    180.12136    177.61477    174.53958    174.14777    177.09429    168.46979    174.32987    161.97393    159.06323          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -33.50    233.45952    229.90289    225.94966    213.85822    215.39985    199.50408    194.56079    200.09114    192.32872    193.46326    187.27721    196.37323    189.48046    179.87566    173.75785    173.19434    173.78964    179.79806    167.53465    165.93554    160.18497     156.0124    158.34144          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -33.00     232.5815    230.38959     224.0867     212.2275    212.01099    198.06087    196.93047    197.10465    193.11121    194.03963    188.95663    192.35958    186.12041    180.03179    172.64717    173.00156    173.46054    174.02931    170.75086    165.90399    162.28039    153.99236     157.2169    161.34156          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -32.50    232.08944    232.00153    224.04091    214.14215     207.8074    200.17296    199.66181    196.18518    192.65742    190.50944    191.90336    190.50384    181.01795    183.74525    174.03373      171.442    174.22647    168.24722    172.00799    169.76026    162.33039    162.53122    160.50452    164.50957          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -32.00    232.43748    232.84804    222.68387     215.7015    204.82007     204.4343    200.57901    193.24762    193.71793    189.36333    190.99623    190.39985    179.50834    183.49193    173.07973    174.77602    173.05347    169.58739    168.09207    168.96356    166.49064    164.21576    161.16237    162.24206      164.209          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -31.50    233.20043    233.41327    220.03223    214.03079    206.23132    208.26652    203.76374    189.02624    195.62395    189.49545    187.98087    186.34037    183.15687    178.93398    174.47648    176.91042     174.2266    173.05467    168.71601    164.13334    165.04325    158.58495     165.5952    164.77718    163.75794          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -31.00    235.98503    235.85247     220.5463    214.27747    208.80569    206.65895    204.07904    186.77924    195.74107    189.09082    187.93687    183.84539    184.38295     176.2044    169.87662    177.31281    176.81652    177.39906     174.1401    161.95498    164.12537    162.71519    166.43415    168.96919    163.17753    178.98754          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -30.50    239.33531     237.0788    223.83775    217.24501    209.22601    204.82666    198.47462    191.49853    193.26759    190.78892    187.32854    184.51547    187.31941    181.20782    167.44095    176.10253     175.8525    174.73224    170.62832     166.0402    165.46979    162.46885    165.09006    166.44147     166.4721    181.63966    186.84213          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -30.00    243.21946    237.11014    227.60561    221.34298    206.83248     205.8839    194.24344    197.11067    190.41321    191.70217    188.88163    187.32349    194.20871    182.45652    170.50843    171.56062    174.40144    168.74951    168.05467     167.1883    165.63956    161.42911    160.96785     164.8078    174.23471    178.42379    188.30295          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -29.50    243.40133    235.44319    227.56324    220.61423    208.46059    201.59715    197.21746     199.6107    189.40839    191.36483    192.04378    185.54255    191.83766    176.11275    172.80577    178.13214     175.7923    170.24678    166.90306    169.25854    165.69296    164.43194    163.09852    163.01879    179.17993     181.8132    186.33295    183.16734          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -29.00    243.82729    235.21875    227.22972    219.27445    210.42889    195.60814    199.77843    202.57469    191.84808    189.98513    190.72391    186.00202    185.37077    170.48855    172.15467    179.62911    178.25192    170.83367    167.22958    171.77163    165.11354    159.83783     161.5944    164.61477    174.46448    181.35728    192.73142    187.65507    170.84962          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -28.50    245.08425    237.46535    226.72542    217.49595    210.03597    196.08517    199.87594    202.72367    195.60369     192.7612    189.33213    185.33361    182.00845    175.48739    176.47177     178.8169    175.86684    169.69032    169.49252    169.28579     164.3143    157.03288    162.79376    169.12456    173.75232    180.35669    192.39583    191.33381    168.04477    158.02699          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -28.00    246.14331     239.4078     227.4318    215.73704    206.88314    201.77573    198.82566    200.02246    200.08569    193.19164    188.60345       183.08    181.48071    175.97408     179.2278    180.97122    176.29043    169.42885    172.36052    168.30154    163.93211    160.63995    165.80467    167.26754    171.08894    181.03138    191.86986    194.35653    173.13075    158.87674    154.00351          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -27.50    245.84467    240.29214     229.1474    214.75778    206.29931    204.79152    198.70605    197.19665    202.70704    190.35737    186.42607    183.88906    180.10882    177.29362    177.62828    178.08066     180.1197    175.51583    173.44275    165.61323    163.90761    167.80445    167.54861     166.3628    169.65351    182.25799    192.71894    198.94616    178.20244    162.95326    152.08699          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -27.00    244.28748    240.20459    231.89528    215.33918    207.15103    204.01488     196.8715    195.66528    197.90935    191.37353     185.3371    183.05151    178.20739    181.39671    177.42884    175.85637    177.86191    175.97448    174.32367    167.56149    166.15683    167.01698    168.80464    165.29289    167.42812     183.8482    195.84256    204.21073    179.39893    161.64134    154.46508    148.92121          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -26.50    242.77155    239.31112    232.26334    215.95505    207.34053    202.17981    193.53553    195.31286    194.03469    194.01398    183.25411    181.21011    177.69431    180.88123    181.17803    174.56199    174.56224    172.54515    172.96222    170.01099    169.00832    168.91698    167.50018    166.81124    168.22599    179.91731    201.50996    207.13862    178.35834     162.1297    157.40266    147.62346    146.60191          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -26.00    242.82875    239.20604    231.12436    216.93657    207.38293     201.6649    194.10118    199.70828    193.93669    193.64864     178.6771    180.01377    174.21383    177.38346     181.7425    174.33065    178.38681    169.13423    168.67988    167.38856    170.41952      166.224    168.89045    165.12941    168.37987    181.93009    203.04402    202.37436     189.4892    159.97957     156.9357    151.18417    145.77304    141.72179          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -25.50    243.85024    240.85807    229.20191    215.22165    208.19874    202.54507     199.5958    204.65618    194.12843    193.51469    177.29895    179.37834    173.11082    184.17255    174.22307    177.44291     179.7482    167.65854     165.6906    170.46845    166.64155     164.5241    165.63476    162.34483    167.33045    186.19795    205.66403    204.78893    189.57938    164.95141    157.71886    153.52712    147.91234    145.01272    144.00611          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -25.00    246.21702    241.90342    227.56438    209.65992    210.66341    203.24767    201.45312    204.80182     195.1887     192.3679    179.51051    179.47295    175.96234    189.86046    170.95357    182.79567    173.00189    171.20112     163.0409    171.85975    167.15036    163.72281    164.28804    161.92667    175.72115    176.40122    206.31309    204.85825    188.04631    166.54934    157.88557    152.08779    147.73714    149.13253    143.97865     144.1241          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -24.50    248.11533    241.50812    225.50123    204.77119      214.844    203.70678    204.32205    204.22258    195.46776    191.72182     182.5658    181.20161    178.36502    185.64218    177.22429     181.4936    168.99241    173.66028    164.95193    169.07764    166.88383    167.26005    163.10943    161.81016    172.88911    182.77815    206.91868    204.65472    186.58564     169.3127    157.91394    151.54992    149.42652    148.02697    144.48511    142.98306    144.54101          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -24.00    247.95725    239.90413    224.45557    209.12315    216.15906    204.34281      209.154    206.33752    196.05643    192.69479    188.72891    185.28727    178.21524    181.28116    179.06712    179.95378    170.45974    173.27692    169.56219    166.80235    166.27278    170.88392     174.3184    161.89866    168.93284    187.96252    209.90674    215.14153    194.80099    173.44581    159.29053    150.05472    149.13426     144.7015    148.77357    141.64847     144.5635    141.76487          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -23.50    246.41304    239.77021    227.31079    214.23495    215.67951    205.82773    207.62047     205.9087    199.12772    191.99115    191.16445    186.73598    180.74502    179.21299    177.79945    178.17901    170.96697    170.85707    171.22562    167.29319    167.46067    172.47821    172.02497    162.79912    165.40826    188.67693    209.42984    211.76627    197.25652     176.8594    165.15244      150.411    150.49608    145.09619    144.91263    142.35009    146.43038    140.90244    139.81504    143.16569          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -23.00    248.14208    239.86302    232.15509    217.68836    212.03076    207.78897    201.39412     204.8044    201.50863    189.29982    188.97262    188.26432    181.76048    181.24266    176.95813    175.84779    171.82991    171.19991    168.93499    170.33234    168.36599    166.34582    169.90223    164.17388    171.18034    186.55149    207.76238    212.15891    195.86466    171.16322    158.05886     152.5738    152.02776    144.13144    144.50838     145.2578    142.34172    138.88438    139.27277     137.5551    143.23554          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -22.50     250.3811    240.34041    232.29355    214.75603    208.39835    209.35616    199.52335     208.1258    197.90177    187.61152     190.0838    187.36184    178.60891    183.57569    180.49187     177.7957     172.4952    171.81398    167.41329    171.83298    167.98798    167.07452    168.60827     162.5218     172.7921    187.51565    203.92205    221.28732    194.67042    172.90998    155.97752    153.49475    150.14833    148.55324    145.43956    148.11059    139.15162    140.15837    140.33997    139.95549    140.25093    133.33294          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -22.00    252.88123    240.48508    225.00485      211.359    206.36269    211.00017    202.24857     211.2369    195.97151    189.86341    187.46665    182.90035    174.33819    184.25647    181.13516     176.8606    174.01292    175.26295    170.67357    168.48174    167.90351    167.86932    165.44105    161.37599    165.78768    189.07387    207.00477    218.71181    200.57331    176.07545    156.15768    154.26482    149.60387    150.22102    149.36946    150.01006     143.1258    140.96028     143.0086    136.90173    141.74611     136.3726    140.25134          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -21.50    256.40264    239.63712    218.13872    210.02492    206.69297    211.85572    202.43351    208.62198    198.46581    191.97239    184.80335    182.46787    178.02353    187.65708    178.55496    176.91793    172.07074    177.54339    172.47544    166.86162     167.3583    166.63621    165.73957    161.14737    168.37295    192.83558     212.0475    217.06838    203.88627    174.31456    159.49624    151.09349    152.12375     152.1392    149.53891    151.35537    140.71952    143.06209    145.73536    141.97285    138.97087    140.02925    139.66354    130.78569    137.32584          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -21.00    259.13324    237.87023    218.72318    210.55855    208.36745    211.14921    204.35677    202.87878    200.11586    190.23515     187.6298    186.39019    184.35733    187.60514    178.94943    181.03869    171.16083    172.08655    170.72514     164.9959    164.77554    169.34698    165.29973    162.09953    168.43481    192.83985    213.26815    218.31031    208.20707    173.12761    162.91478    150.71105    151.54578    149.58044    146.93239    145.68864    145.04784    143.02976    148.12802    141.26385    143.11599    141.05553    138.55345     132.7028     133.1454    130.90836          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -20.50    260.63629    241.20435    221.85267    210.24693    210.77492    208.90967    209.76877    200.38697     200.0156    188.73716    187.94044    188.02354    184.08158    183.29198    179.63515    181.70376    175.40957    171.72171    170.70283     169.9776    164.58298      173.389    166.05379     165.3515    171.30702    189.12486    209.79911    221.04282    206.62245    171.29182    156.97345    149.73844    149.39215    153.06797    145.23768    145.36399     141.3105    145.67877    144.06197    141.20841    143.52139    141.20485    139.55667    139.56792     132.4428    133.58657    127.59268    135.83639          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -20.00    258.27857    246.00262    225.20907    213.11397    212.64973      208.555    211.34576    203.98255    199.91496    188.04906    184.40258    189.12189    184.20018    180.46861    179.78925     181.4778    176.54495    171.37453    167.86498    172.92482    160.33258    171.92146    165.98303    163.77853    168.06681    188.59323    218.36618    226.73889    201.88906    172.47893    152.39562    153.01606    155.23958    150.57399    146.81593    145.76401    148.76864    145.77766    141.64712    142.37106    145.38248    143.18774    141.34898    143.09421    137.20363    136.17567    137.18362    131.84898    131.99931    131.07508          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -19.50    256.68564    251.45864    226.64458      216.211    213.12479    208.23483    210.25889    206.34168    201.16344    185.82391    179.71309    189.86896    183.74907    177.89756    181.23205    173.21506    173.37267    170.57729    172.15784     169.0875    162.52131    166.52732    162.19314     163.3294     165.1065    190.61642    223.80381    232.22756    203.64691    174.14597    154.64983    150.65205    155.76382    154.31727    145.97123    147.29653    149.04973    144.27998    142.29533    142.77339    147.45782    143.61265    144.87939    137.84358    138.71363    134.11198    135.55486    134.88134     137.0291    131.20497    129.70039          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -19.00    255.01276     249.7635    230.43644    218.37468     213.6439    206.36005    208.40806    206.26663    203.41576    186.60956    180.47673    189.44173    182.26624    178.48772    182.40823    171.85567     170.7184    173.67788    175.20796    169.07155    173.42738    165.03076    163.17872    164.25671    167.20426    191.95076     219.4629    231.13896    207.10648    174.19102    160.09678    156.23531    152.37931    154.52302     150.6784    146.59849    148.60724    148.93945    145.40511    139.91247    142.60338     139.4379    137.86364    140.18568    137.94522    135.88539    140.86933    136.28287    137.26343    130.73766     131.6058    131.79947    127.64132          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -18.50    257.30884    246.02068    233.95358    219.85218    214.97816    205.65347    205.31304    205.91307    203.09041    189.78478    184.02763    189.57569    181.09412    180.16501    180.92074    176.65731    169.45867    174.57862    172.66347    172.75108    170.88736     170.8519    164.60084    165.12353    169.92714    192.24455    221.13012    230.35103    209.37893    175.22609    162.53991    156.35751    153.68987    151.43037    153.46181    148.16479    146.23267    140.25512     141.8014    143.37776    144.29441    144.60103    136.20339    139.85854    136.65235      137.143    136.26686    138.89286    136.18249    131.64017    129.88987    129.41538    129.22598    121.30417    127.99557          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -18.00    259.84686    244.39202    227.39395    222.11078    215.91816    209.71374    202.63001    207.69842    200.22072    190.60135    186.59741    188.77082    181.69376    185.23331    177.19192    178.91613    168.89021    175.90285    168.59459    177.73058    170.51639    171.13086    161.42722    169.76067    168.18368    192.05567    217.27477    232.28069    210.11686    177.58767    159.11138    154.26749    148.94931    148.66955    146.99108    148.77641    144.64579    146.62174    146.38026      143.315    139.43407    149.19819    138.37735    137.14153    137.26919    138.40822    134.31272    138.70411    135.40214    133.24236    130.57788     133.3436    129.99275    125.81633    130.93111    135.43889    131.61971          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -17.50    262.09543    243.59481    222.01458    223.48312    215.65048    215.82832    202.42146    212.10837    198.37395    188.09908    186.30813    188.61537    184.33769    187.13589     178.7374    178.00806    170.64069    176.96861    167.26523    175.73719    167.99394    167.04067    162.36223    170.29586    169.81603    196.39017    215.53362     232.7021    211.24222    181.32204    153.69149    151.95609    152.90855    150.64427     150.2613    148.92706     144.7288    144.24378    144.32842    147.63423    148.09366    144.96009    139.07068    140.77111    139.03872    136.08913    137.44719    140.56026    135.31435    129.89803    129.66243    134.43745    132.53738    128.49218    132.39491    132.21442    128.75376    135.46819    133.78183          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -17.00    258.98516    242.93454    221.90764    223.11788    215.53953    217.16794    203.14399    217.06891    197.45428    185.35525    187.80862    189.09631    184.05933    186.14464     180.2757    180.58749    174.64811    175.95665    174.43029    172.98953    169.50134     164.0308    162.34559    159.89957    172.01725    195.68863    219.07606    234.04288    204.06927    181.47513    157.82443     151.7972    151.07352    149.77279    152.01081    151.09956    144.87025    147.23379    142.37326    147.01465     143.0227    139.81808    136.12558    134.45648    140.97269    139.54106     135.3095    136.74199    136.90196    130.96989    133.39963    131.79546    134.64095    130.96511    130.99928    128.62012    134.23116    132.43696    136.59431    137.38539    133.48417    132.81511          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -16.50     256.6933    242.05041    225.48823    221.52798     216.6941    217.43989    206.98261    220.03735    197.39545    187.33308    190.59223    187.75606    181.61596    185.75954    179.38037    179.45763     171.0889    173.37115    175.36395    170.44385    169.96387    167.57971    159.83177    164.70026    173.29533    192.92029     222.5976    235.74883     210.5637    183.54214    158.12889    150.27462    146.12062     147.8632    150.15646    151.96665    146.20848    145.95811    147.34001    144.96913    141.49414    145.57387    136.89185    138.45091    140.17179    140.17562    131.44815    142.21229    139.34312    136.27366    134.47022     134.4154    133.71344    132.17242    132.66487    130.01228    130.51797    135.94241    134.17727    135.87514    133.64157     127.3378    125.88803    122.77985          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -16.00    255.73236    240.21249    228.32784    221.51203    217.24787    217.16911    211.21608    217.60964    199.93751    189.16956     192.7259    185.87364    180.31997    183.95782    176.72357    176.18247    168.79998    172.44034    174.57498    168.32011    169.38107    166.15654    157.85708    165.79106    166.14261    190.80411    225.22481    237.05817    209.55971    185.88262    160.17337    157.64837    149.19466    150.01277    147.24115    148.90038    148.60487    143.08126    146.35505    147.88871    145.47047    145.21489    141.67427    140.59139    137.73723    140.43036    137.84612    134.79626    133.82649    132.24496    134.07827    134.07219    133.07882    132.77409    134.63736    132.70467    134.90428    134.07376    138.97982    135.33008     132.7712     129.2023    123.90955    127.14676    121.87484    124.45584    119.01244          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -15.50    259.65791    237.84884    228.31223    221.82349    215.43209    217.58402    213.11754    211.90027    200.59755    189.44214    195.19783    184.50276    178.61882    183.39902    175.98235    175.89919    173.24622    170.85548    173.98086     168.4751    169.14042    163.05537    160.15703    161.78641       164.41    193.83313    224.52028    232.67105    210.58997    182.36025     159.4667    153.36775     153.2324    151.87192    144.51468    147.43036    150.99469    146.18595    144.35069    143.12711    140.72522    141.33423    140.74912    139.65638    137.03115    143.45757    137.96596    141.26168    136.28234    136.67138     136.0382    136.10696    133.36387     135.2228    130.50837    133.33727    135.01048    139.95781    142.09054    139.65586    139.04414    130.90255    126.84564    124.75249    127.21392    126.44674    122.39169    122.43161    122.54871          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -15.00    263.66818    236.15982    228.97136    221.63941     212.8998    217.83439    212.22578    208.35813    199.26104     188.7717    192.09917    182.70541     179.7687    185.47275    176.70299     174.4958    174.34547    172.12438    174.80285    170.34794    170.98312    167.32138     161.3263    161.65132    166.92538    197.29579     221.2646     238.4372     214.0593    179.90433    162.00487    154.01628    155.45197     156.3637    147.36236    148.80968    151.08746    150.14522    140.01303    141.73813    139.78584     139.3786    140.44246    140.26666    141.52474    140.89451    132.13491    143.81947     130.2266    137.62485    134.96935    135.23414    132.41572    131.35665    135.59187    135.58267    136.04387    141.39191      143.475    140.30211    136.64699    134.83788    130.98692    128.21671     123.5571    123.20271    126.79858    125.26691     120.8009     127.9338    130.24118      137.761          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -14.50    267.17615    234.95216    230.98716     220.2853    211.74759    215.19448     211.6415    205.05953    197.26758    189.86494    189.26335    183.52898    182.19795    185.80322    178.30291    174.48429    173.39155    176.84514    177.90437    171.69206    174.60925    170.18977    161.03753    163.71455    172.49809    194.09084    224.82939    238.03621    218.12181    183.23583    164.15478    156.05143    156.33205     155.4431    152.32804    145.28128    145.24284     150.1271    143.77116    142.73558     146.5678    141.97902    140.85214    142.40946    133.37335    140.30081    139.39603    140.90903    138.56878    133.72831    136.44431     138.2987    131.97822    133.17023    131.67726    134.36619    135.83357    139.98228    138.12892    139.54881    138.88789     131.0785    131.17361    130.64294    127.72167    125.33831     128.1477    126.91518    125.29974    125.13582    134.12071    141.24349    139.12555    138.86123    127.10558    122.51604          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -14.00    266.51569    233.86874    233.49572    218.79657    213.61778     212.3764    213.11412    202.13981    194.19987    191.89602    186.49323    185.70876    185.97534    185.60251    177.35553    176.28833    174.00068    179.91681    177.16144     172.9985    172.84573    170.62845     169.9024    166.17697    174.68778     191.3106    224.47291    236.36507    223.57186    184.16088     163.3359    154.71741    152.24458    149.33911    146.79676    144.40873    144.91866    144.23147    156.78104    144.72065    144.46082    143.62164    143.53835      142.568    138.24035    140.20938    137.68059     137.8611    140.12566    134.97813    138.11258    135.08092    134.68539    136.28092    134.37043    134.23506    141.40913    141.17309    140.44622    136.04862    141.52289    132.60967    131.05097    129.06832    127.84328    124.55712    126.70489    124.93419    125.85157    127.22628     135.6357    144.98267    143.46245    135.98676    132.16861    124.49411    126.47464     126.5358    122.08744          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -13.50    266.19041    237.98705    234.81993    219.01513    214.24642    214.12272    214.58168    205.19743    192.21714    190.41054    184.18612    183.65826    186.68823    185.09799     176.8019    178.10239    173.49606    179.19586    174.67529    171.39893    169.44139    164.38624    171.23776    168.87753    170.57637    193.48729    224.23187    239.38017     222.1914    183.23441    163.16772    153.61939    154.63535    148.96652      146.914    146.97468    145.93914    143.33562    148.15876    146.89214    143.20429    139.16764    143.55981    138.79968    140.91164    138.08031    134.66059    135.84197    136.99206     131.2025    137.05922     129.7958    136.31797    133.60894    133.69727    136.17024    135.07476    139.63807      145.117    141.28731    136.17918    132.27197    129.81952    130.46673    129.65398    127.07765    126.49701    126.70036    127.71196    128.77881    139.91112    146.16745    148.14854     143.2415    130.63638    127.54277    125.81205    129.22771    127.54697    126.66041    123.10806    116.72992    115.94644          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -13.00    265.30553    242.71666    234.05984    220.69668    216.17161    216.86276    211.15582    207.26466    196.76847    189.11362    187.00596    182.12801    188.94754     184.3087    176.09896    177.97824    171.41172    176.45223    172.83726    172.11826    166.93251    163.53482    167.07088    167.79534     166.7496    190.51558    224.89817     236.0279    225.24109    186.28743     163.0686    154.01725    154.44164    147.64041    147.76965     146.7504    150.28989    146.73914    145.78806    146.79256    141.29431    140.31389    141.54917    134.73664    138.69779    146.50818    133.13295    139.22156    135.41962    134.94561     138.2157    131.53721    138.00436    136.45366    133.90187    134.24854    136.13548    140.61366     142.6534    141.16613    137.23135    132.07349    132.51724    130.28476    129.34525    130.94865     125.4211     126.0098    127.01733    129.47632    144.05531    146.60815    151.36183    146.42518    133.40881    128.96767    129.00303    128.60461    129.53354    128.31355    123.16228    124.49837    120.78677    119.15538    118.24705    113.68156    116.85549          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -12.50    261.29689    246.61267    232.12867    222.39023    218.50665    217.75886    206.74488    208.93002    203.47794    191.24244      188.352    183.51443    190.85554    178.86218     175.6854    174.25234    170.46034    171.86486    171.47475    174.43984    170.55158    162.77639    161.54567    165.13219    172.42894    191.75778    227.42855    234.63396     227.4626     189.2688    161.12398    151.75316    152.25923    152.34153    149.18349     149.8339     153.7363    146.41211    144.83616    147.99681    145.84138    148.32971     144.1626    141.01134    137.18564    134.86202    139.07072    140.71092    132.15859    136.90368    133.08487    136.81141    138.09356    134.66012    130.99867    134.25802    133.82182    140.58223    144.91782    139.35379    138.62388    132.00119    134.30471    128.16726    129.66764    123.24018    127.70969    122.84876    125.29215    127.86007    142.07153    150.80831    155.64384    151.92669    136.37109    132.11773    129.56107    130.45495    130.42803    128.39059    124.81351    118.01576    122.81907    117.97354    117.79078     115.4757    114.91846    114.95538    118.85527    122.36911    129.10897          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -12.00    256.95664    244.54673    230.55938    226.35517    219.34386    216.60054    207.40048    210.59989    203.38922    193.51746    188.76691    185.00183    186.96779    177.61591    176.44337    173.20065     170.2942    170.79715    171.06883    176.01899    169.09604    163.01844    161.21853    164.44982    171.06575    193.29663    230.87983     238.1479    225.34926    196.51572    161.92501    154.27289    152.05195     152.5467    150.07397    154.01449    149.74434    146.03073    145.32128    148.95026    141.64748    143.55098     142.4413    138.09828    140.79429    138.96055    139.25021    136.37904    138.61501     137.5712    138.01621    138.72275    134.64537     137.4595    132.37732    135.73593     137.2199    143.70544    139.97232    141.59421    133.64508    134.58695    127.01277    130.73916    129.92915    127.30589    127.51364    123.78236    129.46396    135.26196    144.67064    155.32869    160.59748    152.27669    136.25145    131.37435    127.51304    129.72362    128.81831     127.3539    125.78066    124.72129    122.84836    123.29019    124.20643    120.08492    117.29912    118.26724    117.98859    125.88427    134.75882    138.70569    137.94783    129.12307     118.5167    116.00083          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -11.50    254.18907    243.06525    228.97744    229.32279    219.37257    215.68338    210.20603    209.48499    201.72162    192.92233    189.62295    187.57553    185.27974    179.76384     179.7272    173.35933    171.79988    171.25696    174.52616    174.03644    165.68952    163.39365    162.08964    167.93525     166.9799    193.63288    229.43254    239.31121    226.71084    197.17023    164.62526    154.62621     152.9347    150.97983    146.75486    146.32618    149.34628    151.46879    150.56178    150.37948    141.68517    140.11836    139.01841    140.72402    142.01787    140.62136    139.95555    133.68693    134.68136     135.2089    139.27057    134.75928    131.22909    131.40224    134.83176    137.47676    136.90428     143.6313    142.60149    145.15881    137.33177    133.14679    131.99491    132.45505    130.81103    129.39221    128.42922    121.64529    125.59498    132.69628     145.4069    154.34095    163.42102    153.67522    136.59196    134.32997    130.94059    133.73276    132.17933    129.81582    127.42682    126.72305      124.421    120.33853    119.34647    117.25835    115.35951    115.80115    120.67141    126.10253    137.41118    143.90357    142.38692    130.46513    123.20034    119.47158    111.95015    115.23349    115.16251    110.10639    114.91118          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -11.00    255.70304    242.44943     227.0888    225.83228    219.87213    215.37193    211.35974    207.35203    199.28983     192.4999    188.57311    189.43671    187.36643    181.45369     182.5556    172.72399    172.64506    172.90219    168.94554    173.80028    167.01453    163.16623    163.88274      164.534    166.80863    192.71765    232.04482    240.43601    225.03921    194.07868    164.05073    158.00702    154.58032    150.72932    146.00349    149.65521    150.38002    149.79284    142.28077    146.59723    140.13622    142.25148    144.96428    145.45185    137.60754    144.28397    135.05485    139.60352    133.35686    134.04868    139.61855    136.29627    134.69331    132.57472    129.84411    136.67852    137.85257    138.17877    142.21243    142.88912    134.62448    135.48375    131.61872    130.31984    129.05542     129.1554    124.41332    126.57697    132.29123    130.89026    142.99797    158.39776    164.63165    157.33766    141.21832    134.55007    135.04585    131.58797    129.87154    133.98509    129.80031     130.7605    125.82721    122.78274    117.97486    117.72267    115.07062    117.63506    122.72032    131.20152    139.12945    148.39947    144.11457    137.31986    125.33749    119.04975    115.79172    112.77654    114.37871    113.58537    116.27162    114.32502    115.23457    113.57924    113.66521    110.87688    105.47562          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -10.50    257.50682    242.36815    225.58307    222.36879    219.16476    214.63673    209.84818    205.80852    196.88726    194.61875    189.74166    189.34128    186.98941    183.21462     179.8737    175.00165    172.41848    172.63388    163.37226    174.32717    169.75492    161.88751    163.03722    161.80655    174.19438    192.19534      235.852    243.13747    224.62552    191.35482    165.21909    157.43687    157.17887    151.07614    149.84998    147.39696    148.62392    147.83126    148.56996    144.09455    140.26384    142.77292    144.96825    144.40073    141.35416    142.49681    142.59712    138.13817     138.1351    137.94643    139.28975    133.29189    133.65689    133.07722    133.62522    133.44408    138.31642    141.41442    143.39409    140.64202    136.30349    134.69243    132.88884    132.79762    128.86714    132.12454    128.19516    127.35154    128.94721    133.36384    143.89373    160.27834    163.05448    151.00477    144.77017    135.98827    130.54932    133.52653    136.41274    133.40403    130.18416    131.93167    125.66424    123.55375    124.07381    119.35233    117.81237    115.32777    121.14335    127.46602    140.48481    151.04837    151.51074    138.98445    131.10339     120.7077    117.74807    115.90163    115.06965    112.88601    117.12937    118.35989     118.0369      114.561    116.57602    111.44617    108.04973    111.90727    112.04235    114.82549    111.58772     110.4086    109.91735          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -10.00    258.91526     242.1914    227.44786    221.70193    218.19731    217.31128    208.24239    203.61247    195.63346    196.89288    192.61485     190.3907    187.05028    187.42011     178.3567    178.14667    173.45909    172.40109    168.13698    171.20505     170.6848    156.91649    160.47567    162.47835    173.34293    193.94583    237.14329    248.94123    224.41055    192.35161    164.98961    157.06568    151.12142    153.04173    150.20953    149.43755    144.82144    148.85813    150.11225    145.33041    140.24349    144.30259    144.07485    140.25509    136.62227    142.01594    136.07394    139.53751    136.87712    136.96233     140.6955    134.14922     134.5051    133.03325     131.6259    135.74154    139.74071    145.67325    144.80289     138.6431    133.58161    134.26584     135.8717    129.14818    133.26597    133.69703    128.63742    128.00035    130.35931    133.31336    143.44865    158.14167     165.9497    160.87599    145.94425    135.51274    133.79156    136.07715    137.48439    133.67832    134.16935    131.03058    126.31351    126.15466     121.3813    119.14277     121.7556    119.07425    120.51713    134.34231    144.21345    156.62152    153.39542    144.96574     130.3729    125.45268    118.33455    117.29167    113.37657    114.12721    117.26555    117.33526    117.84091    117.43984    115.96152     113.4382    113.27769    112.18405    114.11856    113.38477    112.51268    112.37566    111.13095    112.15993    109.87416    112.94944    107.65607     108.3818    109.86694    107.91125    109.83026          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -9.50    259.68107    242.03433    229.79974    224.30198    218.05198    220.14965    211.74285    202.62613    194.83931     194.2766      192.591    192.80897    186.34677    187.57542    179.24234    177.09399    174.31164    175.63617    173.72817    170.39597    166.43175    159.85064    159.60962    166.38091    173.96448     195.5453    233.25503    248.80761    227.78265     193.5096    167.09307    156.32498     156.8026    152.33314    148.55576    146.77923    148.32301    147.70804    144.93143    147.25476    142.54089    146.68722    146.09768    142.90678    144.46131    142.81538    138.22331    136.53138    136.00298    136.35128    136.44107    132.72605    138.86778    134.35511    133.39578    135.14741    135.30021    139.23389    142.52247    141.45794    136.39141    132.72206    136.56972    131.03052    126.41359    134.50549    120.96185    127.15862    127.32808    135.74082    145.40992    160.63818     166.4142    157.94552    145.92369    135.32862    133.35105    133.86331     135.2108    131.83304    130.91855    133.12138    132.08952     127.1524    125.70028    118.34412    119.40581    118.62969    123.45406    131.66762    146.20863    160.89773    157.12148    148.41162    136.59426    126.97204    121.29059    119.76616    115.56938    119.05107    120.47221    119.69849    119.94852    119.36593    114.50425    111.34302    113.43176    114.42423    113.63482    114.21071    112.65698    112.22973    109.95795    113.76186    113.54501    111.57036    109.22482    112.30352    112.27657    107.97395    109.42368    111.75955    111.14652    117.66686    118.83137    118.24758    120.81414     120.1551    115.96656          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -9.00    260.76468    242.12653    231.06677    226.93433    218.68775    221.08724    215.38242    205.16002    196.03946    191.07719    190.78613    192.58959    184.69785    180.72581    177.30798     175.2094    172.12216    174.58489    176.26085    173.56792    167.65845    164.48551    160.98823    168.50783     174.8829    195.30048    228.88332    246.96335    235.04386    195.96378    170.14005    153.61615    155.85945     150.2091    148.65467    142.81607    148.48181    148.40817    141.36178    144.15536    147.38584    140.74396    148.40192    141.13566    145.40617    143.83805    140.44475    142.68043    132.77883    137.06399    133.78484    133.08432    134.08955    137.17479     133.1045    137.44528    138.96192    142.80453    145.48023    144.95458     136.4394    135.80264    133.42938    129.72034    130.46495    127.48822    125.91972    126.10231    130.20151    132.70591    147.33073    158.65346    165.89388    157.56537    148.57898    140.45747    136.41608    134.51753    132.51698    127.74875    129.74256    131.66279    127.00991    127.32674    123.47979     118.3451    120.63933    124.26887    125.25427     136.2465    151.18197    164.06558    161.89568    153.30258    139.97664    128.11304    123.59259    118.52628    118.50576    118.59538    120.99323    123.55593    120.77487    122.01773    119.45043    114.26683    115.25285    115.46339     114.6716    113.35276    117.00727    114.89913     112.0653     113.4688    111.87875    111.57383    111.97977    112.51908    109.35192    116.16959    113.46473    111.36412     112.0959    115.77205    119.24415    118.17061    125.83012    122.91659    119.16077    118.23188     113.4064    111.87848      113.894     115.8078    111.44863    115.35068    119.85499    126.10835          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -8.50    261.10065     242.3896    230.80177    228.24223    218.83399    220.89205    216.02495    206.41102    197.90513    193.83852    188.10418    192.51474    186.30366    180.39734    174.30098    175.86322    171.08093    173.69691    172.34079    172.85418    169.23199    170.77594     163.7324    166.81623    178.33758    196.44756    230.38016    248.70564    228.35059    195.02232    170.41246    163.96585    151.58658    149.31373    150.05371    150.62728    152.09382    151.16771    143.04832    152.92498    141.95572    145.43994    150.83956    141.93998     142.7159    139.26394    140.73753    141.85678    140.65358    134.65212    136.21032    136.69794    137.94162    133.32779    134.77396     135.7439    139.72824    140.33122    142.57078    137.36085    134.19555     135.1193    131.90605    130.68189    130.67196    131.38891    127.88546    127.61384    130.75116    133.79784     147.5686    160.70775    167.09752    160.96797    151.40664    141.63246    136.13259     137.4538    136.38376    130.66373    132.44156    133.45182    129.10807    130.75873    123.45372    122.21861    120.03316    120.61445    124.65602    134.45932    150.56161    163.54175    163.29576     155.6538    137.68139    127.59887    122.60262    119.52615    112.21376    116.51021    118.20607    123.14133    125.02166     122.1277    120.27491    111.14576     118.6031     116.0038    115.30146    116.32661    113.83714    111.87787    112.30565    115.88258    111.29626    112.09057     115.0096     113.3634    109.84468    111.41306    108.75042    109.47499    115.24451    119.19766    120.36711    123.67585    126.29327    123.58265    120.68117    119.53413    113.69977    110.32386    112.84698    114.05888    116.09966    116.69493    118.98842    126.81058    134.83718    141.24166      140.403    136.84901    130.30466    121.75693    118.37603    115.39544    116.57969    114.95035          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -8.00    260.12611    240.32644    230.13955    228.75961    222.45529    220.07058    216.41524    202.29458    198.72295    196.04371     184.4988    192.05679    187.85798    185.53228    173.32072    176.08554    171.21674     176.8237    170.53451    171.81721    172.06944    167.80982    165.32246    163.75595    171.82721    198.37771    233.93076    252.04607    234.40791    196.85493    168.64064    163.54566     155.8553    151.68361    152.93501    151.56544     150.5627    154.76052    147.03354    146.65428    141.20375    142.20082    144.55764    141.09353    139.18358    142.01564    137.01709    138.63529    138.95138    138.38317    136.37173    140.68634    131.47811    129.97473    133.94417    133.51083    136.78629    141.46983    142.51856    141.87269    136.96525    132.53109    134.77913    130.39704    130.47077    131.65862    129.39654    129.85222    131.99683    134.97767     146.4543    163.45087    168.02185    164.91005     148.7675    141.94161    136.12892    139.34718    136.83735    133.98893    133.37562      136.305    133.65362    129.09742     125.4047    123.85374    119.62332    122.10043    126.30559    134.23625    151.70704    164.15017    165.59256    155.09273    140.89657    130.18025    124.97165    122.98601    117.42071    122.52823    123.59467    126.24232    122.18953    122.64475    117.66259    113.80635    114.75196    117.70835    114.23661    118.91017    117.57297    113.13711    115.95735    115.99514    113.70042    111.37387    111.60262    111.70407     112.6589    115.04711    110.53073    114.74674    113.20196    114.88239    118.02104    124.68821    123.24944    123.98449    125.69048    117.44481    115.60761    110.61582    112.87726    118.17332    115.33159    116.48329    121.11614    126.82951    138.65735    144.10684    143.84801    139.14844    133.85054    122.31794    122.63209    119.29113    119.54189    116.81847    116.78304    116.86268    115.14446    118.11502    110.59426    109.88908    110.94035    112.38194    111.43328    109.43294     111.1067    108.12989          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -7.50    258.62471    236.52705    228.53717    228.81945    226.54272    217.83943     215.3177    198.09758     198.4239    195.46439    184.11724    191.28677    189.28771    184.05305    174.17006    176.07572    176.87526    174.62035    170.61159    169.18391    165.12831    166.66272     168.2779    165.45291     170.5762    199.76719    233.07998    250.26846    234.93721     197.2294    168.10279    158.75471    157.41594    155.36961    147.87372    150.97932    148.82292    150.84386    149.33979    147.86517    145.14203    146.27267    139.71866    141.65127    143.42866    139.84223    138.75025    143.02747    135.81768    137.53893    135.31211    138.23723    136.98051    133.32822    130.06695    133.15646    137.84373     140.1929    144.16365     142.4593    137.64308    133.86353    130.83522    130.10036    130.66703    127.81434    121.58083    131.14066    130.51726     134.6383    147.96813    164.69996    171.14008    162.33411    148.64833     143.3715    135.68572    135.03684    137.54533    135.40735    137.80565    134.39816    133.65149    130.48989    126.25196    123.99096    118.43447    121.38244    126.78523    136.73989    149.22078     163.8656    164.50959    156.74534    142.62132    131.74117    127.74983    119.99181    119.41499    124.97798     120.8875    124.85374    122.11944    122.59854    118.13428    115.39457    115.73805    116.81816    118.66204    119.47644    117.84593    115.07795    116.58762    113.62036    114.58397    113.30806    113.03196     112.5076    113.99791    115.90876    115.42828    112.47826    113.87545    117.96742    118.28237    124.46552    125.69453    123.60576     126.7129    119.08931    115.89746    110.95728    113.52285    112.29655      115.494     119.3769    124.12234    129.41339     140.8025    145.90905     146.9124    140.80896    138.56398    124.11411    123.39777    127.08633    120.04495    118.36863    119.99598     118.0201    114.56385    117.38489    114.11823    116.48044     111.7154    108.47509    109.28445    112.03612    111.23419    115.31254    114.06723    114.81703    110.64018    111.73583    113.28403    119.89931    115.13378    119.00307    115.08824    111.72265    115.00714     114.2305    117.10388    115.49002          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -7.00     257.5902    232.05261     226.7733    228.05696     227.6555    215.72282    213.63141    201.69264    197.50118     196.3468    186.09565    194.22451    188.33802    181.61698    179.80104    176.82392    178.43499    169.53483    172.98279    172.19051    161.73723     166.7473    171.39615     165.2578     172.5349    197.77858    235.68707    246.90021    235.19527    196.49654    168.07828      161.881    159.28539    148.63262    149.50275    152.61849    150.12294    153.94706    151.98098    148.97437    143.56991    143.34774    139.10853    143.78261    141.49242    142.14104    137.37916     140.1398    139.00696    139.00593    135.09074    139.95599    137.84083    131.17078    134.92267    135.63722    137.56481    141.13415      144.181    138.08517    137.59739    137.10491    137.56487    133.28963    132.55578    128.42161    130.19495    131.49924    128.77073      135.072    148.91249    163.46395    166.58341    163.06347    153.26596    140.42731    135.32093    137.93206    134.68977    136.24194    140.05373    135.47096    134.80477    129.21451    126.32542     126.2938    119.46906    122.57381    124.09088    132.53413    154.53095    165.90558    167.91179    164.18103    147.48328    131.35696    127.84101    126.75879    120.33297    123.17783    123.40374    123.43245    122.70553    123.10633    120.92851     119.1163    117.16101    116.99262    117.56486    117.01529    114.89467    113.91581    114.05677    114.87084    115.11665    113.09462    113.68072    112.07713    112.19627    113.08474    113.36036    114.72141    117.70795     118.6445    120.12067    126.02824    130.11589    126.27397    122.30076    122.23311    116.77517    116.72066    115.35971    114.46127    115.89092    117.63275    124.52855    130.58394    141.59868    145.37141    153.70404    143.73978    136.19809    127.57658     127.4202    121.20373    122.42211     121.7252    121.16349    116.80547    117.31619    116.27787    117.14939    115.35695    109.36615    113.02403    109.22703    112.46216    114.10749    112.92985    108.93413    114.32977    111.75767    112.25642     116.4907    116.60094    115.34019     116.8688    118.86606    117.94231    115.70513    118.39537    116.01498    118.49048     118.0271    120.88659    119.21332    123.37183    125.86012    123.70424     121.9464    118.34752    118.45194    117.02403    114.56341    115.38402    111.36561     113.3878    113.77329    111.41569    112.98616          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -6.50    257.25607    233.94242    226.12229    226.94584     226.9124     216.0856     213.6257    206.40898    197.60056    198.01053    188.32571     196.5997    185.21345    178.87241     178.3678    176.91433     176.7062    168.94965    173.76769    172.78552    165.33211    164.80633    170.85158    165.06083    174.81115     196.7261    233.01866    244.60705    238.33951    200.19031    164.94254    163.23058    161.78238    152.39355    146.71194    153.43844    149.23875    154.11723    149.11408    148.61709    143.18698    141.43785    143.83941    141.36802    141.65872    140.72565    137.97824    137.09884    137.19687    139.93284    136.15046    140.84935     133.6418    132.61619    134.57498    137.78929    138.50617    138.95226    146.37709     137.1429    138.96433     138.2927    137.03914     134.6285    129.15229    132.28058     126.4516    126.73838    132.17526    137.05958    147.77637    162.39935    169.14766    166.46498    152.04566    142.60549    138.42501     139.3232    136.08507    133.90845     141.0699    140.37599    136.10862    130.10996    126.90821     122.6489    122.61019    117.25279    126.79521    135.41599    161.27106    172.54712    173.75731    160.49266    147.48427    134.59402    131.23593    124.85898    120.74488    124.73984    124.26158    128.15829    126.15499    122.91472    118.95297    118.91665    113.24484    118.43181    117.55787    121.08754    115.78109    115.97484    116.41133     115.3631    117.15884    112.79876     116.2313    114.57199     116.1608    112.60464    115.87463    112.53165    116.89779    114.60136    120.59464    127.13292    128.58455    127.99983    123.95597    122.26501    116.85703    115.10047    114.65455    114.68931     117.5294    120.84591    125.41298    134.55677    144.18864    150.70863    149.17654    144.69172    139.11768    129.42004    126.01336    125.30309    124.40738     119.6898    119.08238     116.4941    114.18034    115.93309    111.26841    112.69924    111.40817    114.52323    111.46895    114.13864    113.80525    112.49647    114.30626    116.47521    113.15599    115.00564    119.06104    119.85452    118.69433    117.69912    115.98921    117.28087    117.29378     118.8025    116.72344    118.41494    118.26001    117.45487    121.71509    127.55278    127.86979    122.66676    122.99668    119.91138     118.1598    117.64415    116.01744    117.09115    114.87077    113.71464     112.2809    113.01986     116.4945    115.62517    112.75167    114.20097    115.09902    116.19547    119.67286    120.55162    122.03013    117.80873    118.13371    114.48256    117.20654    115.47692    116.51693    116.47852    114.71797    116.05527    117.68712    118.50805    118.91084          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -6.00    257.05253     236.7499    228.61294     226.5072    226.09584    216.49868    215.47805    205.48722    198.43815    199.19769    190.81616    192.38663    185.42768    179.58621    172.48893    175.67206    177.81889    168.31542    175.41545    166.81091    169.69024     164.8594    172.83248    164.91829    172.63744    194.48739    231.95457    246.05493    240.43083    203.60222     165.1566    162.29639    155.11529    152.29624    151.94571     152.7107    150.68196    149.21362    146.79413    144.21563    144.65954     144.2605    147.37154    138.51941    145.21723    142.17153    136.16158    140.22878    132.51309    137.72869    135.93484    138.31866    136.18727    134.71675    132.63112    135.48221    136.61392    143.51166    143.75718    139.33632    139.20266    136.67765    133.82864    132.31163    129.76711    127.13153      126.245     127.3179    130.26653    135.51702    147.63438    164.63955      171.736    169.69767    152.75283     143.0114    140.93892    140.96974    137.12481    136.07294    137.19174    138.61821    137.29169    133.03463    125.87154    122.48152    120.60004     120.2037    125.78305    137.47055    158.87897     168.9106    173.24884    164.14057    151.50857    140.25822    131.00603    129.69799    127.92948    125.25048    125.27221    126.67742    127.47182    122.68881    121.76316    115.86576    114.63417     115.5175    113.86098    117.94737    114.51946    116.38979    118.14662     112.6562    117.91188    112.75335    119.25163    113.69311    116.26545    117.73202     114.2719     115.5755    115.46059    119.46407    121.82963    128.56331    130.28144    128.63334     125.3622    123.22088    116.83136    115.64644    115.03751    116.73745    118.39406     118.4537    123.02854    136.24104    145.75971    152.14123    153.57534    147.77697    137.43557    129.73334    123.32423     123.6918    124.46715    121.25339    121.54114    120.46145    116.92822    116.20257     117.1471    114.50336    111.12661    113.48391    109.37897    112.81726    115.29202    116.09302    113.05485     113.0762    113.74222    119.99106    118.24247    117.37526    118.30856    118.55118    119.71816     114.6689    115.54767    118.69156    120.26376    119.17157    118.51603    123.39028     123.4231    127.77263    127.86897    128.89484    123.75727    122.12537    115.84101    116.70354    118.11014    114.13364    117.76301    115.89739    113.29797    116.25952    116.44409    114.64061    116.57796    119.43158    114.25174     116.2026    117.53487     119.9158    120.68198    124.68858    120.59649    117.55239    120.33184    118.13372    120.50826    118.82933    118.29188    117.23528    121.30791    118.03668    119.55184    121.00897    119.55607    116.16356    116.88016    117.04579    114.83967    117.33262    118.31817     117.0009    116.66288    118.30844    117.75747    119.91763    118.61981    121.78419    120.04996    118.66609    121.68317    122.45453    122.56752    125.69661    124.37428     124.9186    127.18191    126.22831    128.81271          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -5.50    258.25715    238.31219    230.97697    226.42598    223.68631    218.16604    216.74076    204.48717    198.64068    195.78582    193.22682    188.14443    186.92088    186.95091    177.85878    175.94002    177.45128    168.46699    179.16165    172.31742    169.59666    166.10209    177.06379    170.91195    167.77228     193.0019    231.04666    252.10603    241.80012     199.8879    171.04026    162.64682    155.15721    151.50209    151.95662    150.26697     148.6045    147.69471    144.14031    148.02287    145.67876    144.44712    139.49533    146.53611    141.22837    137.88573    134.36612    137.68742    139.22544    142.27615    137.83686    138.99593    135.87516     136.0184     133.8571    139.60957    137.98163    143.96391    145.70491    141.95791    135.27715    133.44858    137.06757    133.64412    127.73183    128.63647    129.88387    127.61245    130.15495    137.79089    149.70819    164.53126    169.88492    167.35652    153.01274    141.77717    139.54002    138.03413      136.906     135.5361    135.41778    140.94898    137.53892     135.8699    126.45213    118.22142    122.33298    124.80572    126.76833    141.36457    160.97484    176.11832    182.85765    172.52501    160.69827    145.07687     137.8861    131.50577     128.0866    128.76962    128.53227     127.3443    124.51971    124.54128    121.39395    117.63921    116.13446    115.93474    115.43924    120.09523    119.65551    115.15709    115.49039    115.56977    115.56886    114.96701    111.76825    113.32832    114.48657     112.2385    112.86845    114.80427    118.61116    118.67524    121.10233    126.93656    132.98001    131.36691    125.26432    123.65268    115.57828    116.19352    115.92598    109.74654    117.25238    120.85246    123.35285    136.08348    149.93746    154.33297    156.98638      148.422    143.02209    128.28525    129.44051    128.04409    129.96614    124.95666    123.17634     120.4454    116.67897    114.64333    114.17712    111.80629    111.61181    112.67521    111.41764    114.24022    112.59474    113.27884     118.7401    115.40469    115.14646    118.20589    114.24371    121.11849    120.84683    121.28071    120.60649    121.49235    120.64388    118.79737    118.81507    118.99788    122.71841    122.44349    125.39537    134.66923    130.58229    130.45541    123.96195     123.7502    118.31446    118.90243    117.60509     115.6858    117.11943    116.07601    113.02136     113.6032    116.18954    117.49803    114.91521    116.15695    116.36025    120.97627    122.91938    118.76272    120.53507    122.33123    120.96781    120.17517    125.46336    119.79165    123.23717     118.1957    115.70215    118.94209    119.99534    125.15667    121.13583       122.98    119.02714    121.40238    115.86449    120.31829    120.75388    116.37838    120.70918    118.22405    118.95018    118.79214    121.12184    122.66105    120.77304    121.04743    124.39603    127.56262    129.12413    125.66927    127.72364     127.5199    124.68757    124.04902    129.45305    132.21401    128.67846    138.14307     130.8127    133.43816    131.95528    131.35476    129.05334    129.22332    125.56802    127.83359    127.73053    126.02458    123.24879    127.91644    126.35731    122.20524    127.60898    123.52982    125.88154    125.36593    128.40065    127.30527    128.59936    127.16121    133.72298    131.21335    132.76243    133.25731    138.22899    132.30015    134.64379    133.86502    133.66167     130.1072    126.18587    129.54118          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -5.00    259.81096    236.79416    232.83343    226.21454    221.43637    220.36334    216.50339    203.69904    201.32405    192.59617     196.0575    186.89867    193.34805    187.83625    181.84621     177.2733    174.46155    172.94223     181.8516     173.6283    168.49434    168.20755    168.28811    166.59487    169.44299    193.41873    230.26098    251.63999    238.10867    197.66503     172.3223     163.5218    155.12531    152.68834    152.53066    148.38225      149.245    150.67665    145.11287    143.97333    142.55295    146.71528    140.03388    142.16495    142.01545    143.67758    138.14385    138.41974    137.16217    138.44354    136.82484    141.84985     130.5694    136.10875    134.96966    136.78131    137.93045    141.40717    147.65323    140.89738    140.08739    136.22753    127.62535    134.31744     131.5614    130.32889    126.76709    128.23779    134.84612    135.17701    149.17857     166.7164    174.33017    163.94674    152.55249    145.45162    138.46039    138.82684     133.5271      134.527    133.34075    142.41077    135.48082    132.72376    126.53938    127.11723    118.29889    123.69764    128.70455    137.16592      158.964    176.80399    188.10477    184.16367    173.51914    158.55769    146.67858    140.55348    137.32098    135.68282    131.93178    129.82312    129.11077     124.0653    119.80599    121.29663    117.24044    118.22007    115.52882     117.1656    118.95706    116.27548    119.37924    115.88327    114.64212    113.13483    115.17315    113.61169    116.23307    115.25529    114.45351    114.56785    113.37923    116.33147    120.63146      128.648     129.7231    129.92147    127.75817     122.8994     115.9949    115.79134    116.85493    114.06772     118.4845    119.27066    123.44928    135.20071    152.60109    159.14618    159.61279    156.01253     142.1705    138.36301    131.59336    133.75116    133.56319    126.40451    121.90472    120.95188    120.61346    116.90172    113.96209    113.66181     113.4276     113.9838    113.23448    115.72763    114.33533    116.21698    115.77748    114.66021    114.56059    118.36711    118.90569    121.13163    121.03402    123.15589    119.10579    118.99602    118.73945    117.71544    121.59271    121.42921    123.17817    122.76311    126.29926    127.70361    127.27906    127.73436      125.496    120.91039    121.04538    123.61038    117.26099    121.65484     116.6566    116.09594    115.02208    117.36016    116.13838     115.0544    117.04633    118.46279    116.08047    119.18108    122.71575    125.00859    125.07404    124.42185    123.86199    122.47734    122.54593    121.56208    120.42582    120.45414    120.96935    119.63354    123.02694    122.09649    123.86004    124.49683    124.56372    122.09343    117.02614    119.56935    118.29984    115.56816    116.66899    117.16087    118.64862    118.06319    121.03864    119.98877    121.02786     122.6481    125.05464    122.43707    128.70023    126.75613    122.86449    128.37718    128.77004    126.56002    127.47892    132.81526    130.07529    134.92408    131.18952    132.23886    130.45904    131.75518    130.64654    130.79673    127.83539    130.40025    123.16154    127.70402    125.05563    126.20927    127.49681    128.14683    125.82641    129.23572     124.8031    126.64075    131.47878    126.02037    129.95891    129.79223    131.08223    136.18217    136.13829    135.95231     136.3843     137.5037    135.88424    136.93311    133.75533    135.85039    133.48654    130.91362    129.93149    131.90309     130.7888    124.60772    132.05096     132.3056    126.24206    130.98256     131.2779    131.69368    130.19542      129.788    127.44966     127.9125     130.1788    128.67377    129.22469    127.42699    130.65156    126.82286    127.64164    128.11514    129.64609    126.92982    129.63384    129.25863    127.87818    126.93284    127.94594    130.73456    126.19766    125.22705    127.08665    126.70368    127.47777    127.79794    128.13859     128.2842    128.39926    128.93469    128.05282    132.91816    129.88379    132.37987    129.50103    127.92306    130.28619    132.63773    132.84112    132.70696    130.28901    128.75409    132.18076     133.3804    131.96677    130.39628          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -4.50    261.33076    235.44893    234.64281    225.51322     221.0393    221.98697     216.1693    203.15016    207.05345    193.87624    196.84337      188.087    197.90949    184.49113    182.64699    177.85052    176.47743     174.7884    182.09569    168.63077    171.62044    169.25555    163.05465    163.68926    175.02457    195.86912    236.80833    246.69759    241.50302    200.38183    166.24227    163.33317    153.97717    150.09656    152.47631     149.0967    153.43104     151.3913    149.32894    141.62705    142.88501    142.47995    140.73163    141.70784    142.12339    145.04027    139.09358    141.48394    137.39092    140.33595    138.06944    138.76279    138.69021    136.51029    134.82083    137.25482     132.3573    142.13931    144.02132    139.90602    139.05082    136.07963    131.89971     132.9008    136.89281    127.20605    126.31251    128.73342    131.58135    135.51702    149.21868    167.33676     171.8855    171.28882    156.25473     141.9249    140.03715    137.59189     134.1735    131.35351    137.41767    139.75105    140.47706    134.76927    127.12157    123.61177    123.80622    122.21318    127.98115    139.07706    158.09307    179.15943    188.84485    191.50731    179.77508    165.99962    152.65863    142.56779    145.30536    143.31771    139.92573     135.1769    130.23006    126.68465    116.25758    118.41883    114.44551    118.22422    119.27304    119.95239    118.19964    117.50234    112.46145    115.37074    115.42668    113.71251    116.40588    114.18878    115.38846    112.87813    113.54073    115.20559    114.52192    116.44966    124.53535    128.95651    132.82561     132.0169    126.98189    122.32103    118.20411    117.06973    115.55387     116.0008    120.05811    116.58667    126.02115    139.97204    151.89283    158.87866    160.28155     153.4484    143.12122    134.83455    133.28713    131.28616    129.32156    126.24682    123.48828    118.33103    120.92904    117.94925    118.27922    115.48282    116.35137    114.83872    113.22181    113.47634    115.73597    116.50565    114.17126      114.166    112.82666     119.1507    120.41495    118.62277    121.43123    121.02897     119.0913    115.98446    120.32948    118.54612     122.3702    123.06992    125.48237    126.74495    127.00723    130.61675    128.70741    128.27074    126.45181    121.65289    119.97782    120.45143    119.02986    118.59643    117.82573    120.60886    115.95572    120.25395     117.4312    119.58677    117.81488    114.61728    118.31396    122.36768    121.24169    124.76341    120.67534    127.38672    126.02856    122.52321    124.37766     123.1567    120.73694    123.11177    121.55045     123.4962    126.68759    126.98654    124.57297    122.51357    126.26036    122.37268    117.85503    119.28828    121.21834     120.9855    121.34866    118.69173    118.11289    121.24932    121.47388    121.68373    121.54957    122.39941    123.19651     122.5505    126.77224    129.90916    128.05443    129.27265    129.95254    127.29428    130.17147    133.39611    136.92049    134.07508    134.84864    133.36915    135.20229    134.61793    132.84535    131.53007    130.81881    129.25959    123.72921    125.75954    127.06643    126.31402     126.7979    130.20033    128.00705    127.25426    131.66998    130.86106    132.13287    130.88164    130.60136    129.43364    132.70787    136.34744     136.3075    139.87914    138.27597    138.57403    138.49376    140.25852    138.08542    141.30683    141.11431     146.6978    141.95506    137.36633    133.74024    131.21567    129.50731      130.629     127.4673    132.74681    125.72447    130.02068    130.30103    131.58716    130.07313    131.73646    132.94219      130.682    127.96168    126.07262    130.32136    127.10333    128.73037    127.04999    129.18695    129.14783    127.63973    128.15369    126.70689    128.51855    126.63476    127.48835    129.95831     130.1089    125.13473     132.7554    129.19122    133.22108    130.10116    130.22008    127.70723     128.3733    129.71514    130.78935    132.27175    131.17157    131.21019    130.94905     133.8106    131.01446    134.57895    128.58827    135.48619    131.58005    135.68664    135.00049    133.43771    138.35735     133.4487    134.02075    138.40707    138.05559    141.60301    140.91539    140.30426    142.89311    140.31716    146.72973    143.79704    148.76682    144.71501    144.33787    143.24171    147.75716    148.91581    157.73756     150.0387     147.1774    149.59684    151.15708    151.55849    156.54009    154.18724    156.79122    152.83888    157.89888    159.47955    162.50187    163.04392    165.48018    168.23861    158.56574    159.73167    166.09648    172.36395    162.93092    167.99746    160.67904    160.89536    166.50542    164.05481     166.2434    165.79416    162.81717    168.21741    175.16042    173.15434    168.78938     173.4634     176.0589     172.8406    177.82414    174.82014    185.41018    186.38826    178.51791    174.87635    169.03912    182.17481    189.38773    182.58891    178.01176    177.19886    185.75438    182.20047    187.98698    195.75361    183.92435    185.94605    188.66529    189.11203    192.85568     200.6322    194.26955    207.03012    191.35029    206.28936    195.52997     203.5723    204.76391    208.08609    211.86945    207.75829    203.88306      210.979    219.68483    213.34541    225.30836    207.95635    214.46132    223.58481    232.77447    227.36509    221.24169    231.03306    231.94813    230.32847     239.5916    234.93959    243.78894     245.2955    240.24285    226.75596    246.70857    250.57986    247.70256    246.68831    244.88026    251.91889    249.49728    269.13027    268.83497    264.79094     248.7314    260.03124    268.84036    277.80122    286.13845    266.37739     282.0277    292.72456    283.20978    292.62328     280.7707    296.52874     305.2915    297.21943    331.57588    311.18893    338.66144    336.97193    328.28703    337.56056    334.59876    331.89606    355.37069    379.29578    370.78136    381.98076    384.09947    394.28717    383.23196     430.4004     426.1049    480.09058    445.18948    426.13911    442.57805     492.8044    583.08991     556.1249    537.18426    548.55861    576.84167    673.23177    770.23122    896.93648    826.26717    782.42367    777.52232    841.11197          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -4.00    260.19052    237.41516    236.16865    225.00433    220.98665    222.85602    214.96717    200.40788    208.97549    195.33769    196.96122    191.59987    192.40838    180.98416     183.0071    179.19661    179.09656    173.94577     179.2491    171.00936    167.00421    172.15112    163.44714    165.92766    174.25057    195.11608    231.96158    245.38078     244.5003    201.77183    168.51556    156.05002    157.67648    150.18866     151.8303    147.96494    149.47026    147.16504    147.14771    143.84433     145.6354     141.5014    143.41848    145.21061    139.99675    145.09129     138.4579    142.06704    135.66207    137.52487    138.86814    135.65382    131.99372    137.43839    139.25711    141.31546    140.74927    142.07705    144.07529     138.4181    139.96687    134.19964     135.0406    133.41059     130.9176    128.78554    131.40729    125.40149    131.76742    139.16052    148.12426    166.72301    173.54205    169.37461    154.96143    139.30877    139.47584     137.4725    133.97395    131.21339    137.76531    144.24747    137.18641    133.77909    125.44293    123.41378    122.75633     123.8654    128.40438    137.91777    159.23156    180.57304    191.62218    192.17225    179.58698    163.08623    152.86155    145.81574    149.02338    151.13761    145.58367    138.35036    131.10811    127.25671    120.94712    121.24322    117.35356    114.13243     117.2093    119.36168    116.14973    116.50145    115.62959    112.88455     114.8549    116.24709     113.1659    110.75213    118.62552    116.39637     114.6534    114.76783    115.87055    118.39183     121.9033    130.48735    132.64931    133.81144    130.19592    123.51088    121.36747     116.4878    114.38468    115.83248    116.02288     122.3553    129.06515    139.51289    152.34356    161.24329    161.42155    154.84606    141.69021    133.69303    129.28613    129.67716    127.14725    127.38361    124.91487     122.0454    120.30101    119.78562    115.73992    117.03705    114.66485    110.97286      112.797     115.0849    114.19569    116.04093    117.57989    116.05595    112.63446    118.28398     120.1893    122.14237    120.57764    122.69031    121.18832    120.74618    117.61136    119.17335    122.90431    123.42668    125.36218    125.53121    128.61203    129.07202    133.30558    130.44333    127.60667    122.44947    121.58924    125.51727    121.93911    117.05945    118.64495    118.75699    115.76307    117.64346    117.84807    116.55421    118.77469    120.65644     117.8961    121.37169    122.90254      125.912    125.71729    128.38863    127.35667    126.50082    123.35548    123.36711    125.39718    121.35914    123.40788    122.81796    127.85968    128.78899    129.11021    127.96677    127.06685    124.73398    125.15306    122.75546    122.72833    119.68288    118.97178    120.64156    116.35738    124.02759    121.36249    122.52364    123.67743    121.99212    125.64593    126.49279     127.5674    130.27434    129.32063    129.78948    129.05266    125.09403    132.38987    133.75467    138.58461    136.52382    136.52022    135.05959    134.26244    136.41549    134.29036    134.01572    132.98345    128.10378    127.08199    128.27493    131.64899    129.57424     129.8872    129.15722     127.9714    131.01699    134.34461    128.45827    132.92935     135.8021    133.32048    134.19907    132.93509    136.49029    136.07564    136.07448     139.9782    138.03685    141.68032    139.72999    135.78885    135.69893    140.32387     139.9996    139.70551     137.4133    131.21962    132.33856    128.39566    131.25339    126.68804    133.94795      133.234    129.88927    134.29856    129.75164    130.98363    133.41053    131.39356    131.92261    127.33493    130.88296    131.06817    127.23497    129.37884    127.63417    127.89731    128.17272    127.57058    125.22358    127.87473     127.3753    126.02534    129.56635    129.18114    130.98211    129.09008    131.49759     126.6616    130.40899    132.60963    129.83793    129.95941    132.87879    130.84476    131.36737    134.95982    134.12494    129.20646    131.46937    137.86055    136.39467    135.35026    134.10655    136.90708    133.19707    131.22808    135.33317    135.31292    135.90039    135.02252    133.87345    134.04184    140.38326    140.47074    141.10134    142.38518    140.29808    135.99775    139.62892    144.24092    146.87602    149.03069    146.18911    146.63923    144.57245    151.28115    154.10331    154.27003    144.76513    147.50653      154.984    157.52063    156.09395    154.70303    157.18871    156.43757    158.62276    162.28617     163.0065    156.91715    164.27593    166.26526      161.811    164.10548    169.39041    162.91693    170.07892    163.19262     170.4347     163.6546     171.6663    172.50367    164.91659    173.31774    173.50168     166.3679    174.95261     169.3059    175.28113    174.63577    176.99089    181.94021    177.26352    178.32948    179.98548    183.93545    184.23536    174.46422    182.07939    188.91224    187.97492    181.24103     186.8724    182.26263    183.24292     189.1336     187.9701    192.45487    198.13239    192.64522    192.68957    188.51425    198.33233    207.56054     203.7754    205.07791    202.06007     204.1156    198.01443    201.96532    199.93391    215.55366    208.29718    211.79832    204.58198    212.07289    215.00674    217.20025    208.52182    219.97667    226.04583     220.2411    230.02095    233.90185    226.97992     229.9316    230.97827    226.62541    236.85721    247.73055    234.49288    240.39835    240.58635    236.87871    247.77766    249.17571    249.44497    241.13643    249.08177    247.01335    247.38838    265.73482    256.06557    272.90416    258.40006    253.00357    265.10456    281.58256    267.29963    264.15709     271.7086    272.66051    281.08601    281.76896    296.14311    299.74803    282.52938    294.97515    305.55843    287.56661    339.11983    329.86599    346.78508    323.00729    319.69424    333.80281    360.31689    374.24911     376.8229    357.33006    367.51512    342.30813    383.69297    427.36409    408.67357    458.00912    436.53437    430.77407    436.90457    499.90497     556.6507    555.04355     514.9277    501.74875    561.96988    627.61425    764.02268     779.4736    661.74288    775.22912    796.65462    793.07861          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -3.50    258.91051    242.44721    234.83349    225.46916    221.71164    223.40183    213.65583    196.72195     206.1394    198.05525    194.39764    193.72028    188.77964    178.53277    182.84192    181.86024    176.94566    176.49248    176.79301    171.23054    164.28784     174.9125     169.6732    170.14738     173.5232    194.86912      230.622    249.18562    242.95732    196.17297    167.96982    156.11538    161.51933    149.24123    151.37986    148.07145    149.20058    148.84747    152.64342    141.91303    140.51576     144.2177      143.082     143.9752    141.23672    141.56758    136.42416    138.26515    140.75663    138.31066    140.47344    139.52481    132.90229    129.88093    133.57352    134.40904    136.42862    140.75431    140.11155    143.46016     136.5092    136.87328    136.21802    132.49821     130.0872    128.49227    130.78675     131.7274    132.82878    136.20131    150.47079    169.38212    176.82791    168.26833    154.28307     144.6585    137.86904    136.02868    131.73215    132.72349    135.40117    141.84058    137.64509    134.53694    128.20971    123.87424    124.76695    121.30867     130.1073    141.97161     160.8706     178.8881     189.6406    187.35871    176.56175    165.31146    158.79235    150.52521    150.96873    150.79663    143.11166    137.39535    132.61135    124.47912    119.96324    118.53538    116.72197    117.93047    120.25842    121.56975    118.32324    121.42489    117.58722    115.57554    116.32405    115.93265    113.26191    113.62795     117.2547    115.73323    113.45228    118.68601    117.69432    119.05727     124.7963    134.57267     134.1006    132.69382    130.77035    126.71571    118.92364    118.41891    117.85734    114.87545    118.94429    123.33935    124.42247    140.82476    150.82005    162.08895    161.87571    156.98944    144.34421    135.03108    129.70176    130.15183     124.6239    126.86296    125.74749    124.87915    122.01786    119.41153    116.38822    115.08236    116.35565    114.15524    115.25718    117.29498    113.07606    113.60252    112.26117     117.1513    117.66081    117.67253    120.45644    123.00071    124.43954    121.67277    119.98597    119.62221    121.45393    122.38389    120.12623    125.46358    126.50219    122.65419    133.38098    128.98349    131.38885     130.8027    128.04871    125.99586    124.59363    122.80838    119.21423     119.6352    118.75985    115.41746    117.70147    119.44773    116.21715    121.73344    117.75818    118.00482    120.04041    122.34119    125.28463    124.38096     125.4467    128.36135    128.60273    127.56138    126.63946    122.51517     124.9897    121.65973    121.46431    123.30952    127.51167    132.58718    128.91857    128.24467    126.72212    123.82422    125.57304    124.69958    123.26508    121.84627    118.51679    119.51434     118.4751    120.48521    123.87888    124.03766    123.25841    124.95377    123.81873    127.94754    129.52309     129.3827    127.77191    130.34185    129.19988    128.57223    131.12439    137.06692    138.14557    141.60605    139.10295    139.12692    141.87729    142.04325    140.78402    138.58014    134.62918    130.35648    126.13414    129.16266    129.50257    130.76392    128.54581    130.78175    130.63669    135.33483    132.76873    133.89579    135.57281    134.98703    132.07166     132.4028    136.27128    137.46089    139.92326    140.64028    144.75332    140.57206    141.20813    143.13608    141.22513    140.27257    137.21887    133.75068    131.23628    129.23801    130.60448     128.2151    130.87612     128.5825    129.48396    129.45075    131.34204    136.03786    130.36762    132.46747    129.11274    132.59024    135.05817    130.84381    132.59038    130.16352    129.10772    123.92569    131.50931     128.8087    127.12367    127.72728    130.40058     127.0881    128.32023    128.49329    131.80464    128.22693    128.00705    130.92475    127.00588    131.03791    128.52871    131.35344    132.03877    130.08443    126.20886    127.83502    131.39396    133.69997    134.56058    135.90593     134.1069    128.84444    134.16351    133.10659    133.05114    133.46823    135.95597    133.75114    136.13353     138.1337    138.77522    135.81539    134.35658    135.46386    135.39524    140.59508    139.49244    143.61514     146.2559    142.50395    142.15892    147.24028    147.40551    146.35918    144.99254    143.28366     146.6027    154.21661    151.31721    153.18425    149.74922    147.87617    150.55255    150.40489    151.00301    155.38885    160.26386    159.63953     162.9891    156.54311    159.71163    161.42924     159.9364    164.56777    161.26384    160.44045    164.40963    164.31532    166.56621    164.58668    167.95245    169.05415    167.39684    169.42394    166.83068    170.40808    172.41295    173.73514    170.33857    173.56017    172.70003    177.04244    182.10405    174.70529    179.57465    179.07175    182.07708    183.84291    183.69132     182.5362     184.3929     180.9821    186.62306    182.40956    190.82595    184.73943    182.89752    181.95521    186.38516    187.30554    192.96168    197.57249     195.2988    196.60765    194.00239    201.53307    196.35977    202.70819    201.71195    204.03958    196.79376    205.19289    205.07066    215.55947    202.96756    196.09577    216.97297    212.32104    211.10971    206.51432    208.31488     223.4344    221.92514     220.1356    208.76638    232.83998    226.06776    223.85027    237.31823    246.43487     240.2533    229.82823    232.78747     244.1544    233.30293    228.37749    220.83511    250.41695    248.08751    243.87182    240.01644    238.99759     245.7533    243.21344    254.08587    254.38406    262.95047    257.36842    255.06646    257.54894    281.39594     273.5375    277.09644    260.61144    275.25523    267.27746    293.05279    298.81497    283.18796    280.45568    286.20455    289.82001    301.91064    334.00959     313.9878    343.59093    332.37676    327.90336    330.83614    329.77948     355.6788    381.39705    380.17492    372.15074     382.4607    375.46784    443.20301    443.68566    438.00678    453.46056    419.58472     426.1156    490.04555    527.22213    599.79538    547.68639    532.96339    515.55322    584.57862     792.2995    839.08418    741.45522    802.69835    870.59546    761.70753          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -3.00     257.8204    247.33133    233.66384    226.16079    223.51242    221.93634    217.47449     198.1891    204.03142    200.16783    191.43034    194.97428    188.45296    179.28081    181.45987    180.47887    175.95006    177.56999    173.54018     170.6892    168.52193    167.66629    170.14046    171.10613    171.66584    194.80192    230.94821    254.53763    241.00081    200.35936    170.74203    154.01334    158.93351    150.33781    152.90365    149.39499    149.68387    157.32782    146.55839    150.10766    141.14686     143.8289    145.43747    143.58507    145.52628    144.27316    134.69352    142.05817     139.9324    138.44994    138.57716    137.73231    133.99499    133.63891    131.07911    137.67412    134.94408    138.79755    145.36831    142.93542    137.66838    135.30452    134.73221    134.93957    131.22011    127.04205    126.47079    130.07414    135.52993    137.66358    154.75121    164.26146    175.13443    174.71036    157.23644    141.32543    138.14156    140.58024    133.29658    133.12102    135.13003    138.51159    136.82802    130.24822    127.97948    125.24894     120.2573    126.40894    126.80688    138.78906    157.97697    175.21052      189.798    188.87631    178.72919    173.03104    165.08428    154.10067    151.72392    146.27417    141.79413    138.04614    134.32653    127.16079    123.52452     117.4016    120.42533    117.92001    121.30893    122.24732    119.73918    119.55638    118.40694    116.38476    116.77681     119.6134    117.33799    111.77594    116.13988    114.78434    115.06436     116.8659    116.23205    120.34055    129.62761    136.30669    136.12148    130.45017    130.70159    125.35544    117.95886    116.86583    115.24769    115.58505    120.72796    121.63947     126.8401    139.64912    153.59537    162.02197    162.82998    159.50875    147.41735    135.45866    130.82272    129.37497    128.38114    127.58144    128.86701    123.65321     120.7247    120.61216    118.67275     115.8184    111.39676    112.83147    116.10948    113.56535    112.20419    114.41793    116.59678    113.58916    113.34029    118.49355    117.67529     123.5862    120.98049    124.37879    120.27171    120.91446    122.46201    121.45362    122.72912    126.99706    126.35421    126.30792    128.39162    132.68726    129.80842     129.3351    131.18121    125.70631    125.00783    124.97808    123.49142    121.83346     116.4733    115.13972    115.10643    118.81768     117.9723    119.63347    117.15093    121.38782    120.61007    122.60181    124.37113    126.99881    127.42829     130.1023    127.09719    125.13641    130.37316     126.9365    124.80617     119.3708    126.10755    123.84782     124.4157    126.43486    127.10696     129.3665     127.3561    122.17242    121.66158    119.38645    121.41674     124.1179    119.66193    120.74005    119.81034    121.04223    122.10061    122.37657    126.09226    123.34267    126.66078    128.72124    130.21855     129.6701    128.57946    127.35247    127.14136    132.59685    136.26295    138.26565    141.32118    138.72498    143.89363    143.86643    147.06855    147.61423    142.42087    142.64739    132.51371    135.82026    128.48463    133.68104    131.12157    135.22055    134.66137    136.70612    136.42855    138.65418    136.06978    137.55443    140.62406    136.54469    135.07787     135.8949    135.21611    133.68284     139.4312    140.51933    139.43036    136.89721    140.66593    145.35112    139.90757    137.93994    137.61007    133.96285    132.54384     132.3582    128.82858    130.01007     128.6668    127.91713    129.41523    134.48886    128.86515    137.19516      132.737    129.22871    134.34558    133.22432    134.54479    132.71051    134.67737    127.42411    131.55077    131.16059    132.27626    130.92714    128.26009    130.65157    126.75685    128.03701    127.37494    132.17644    130.75744    125.72054    133.44098    130.31846    124.29418     127.7394    127.51747    128.94865    128.35299    129.27415    131.59279    128.15799    133.97468    130.75671    131.95451    133.46267    132.61476    132.23521    136.22112    133.57203    134.45483    130.79881     135.1551    135.70893    135.87127    137.39327    129.53203    133.78266    137.92323    137.50207    137.18248    139.06329    140.80926    141.84295    141.79618    146.44095    144.06875    149.31925    146.32967    145.45036    146.49488    145.09594    147.47029    150.48886    154.93707    155.95127    150.38012    146.43846    144.47786    151.71001    154.53282     153.7917    149.76386    154.94737    157.90243    160.68208    161.63107    166.27607    164.29507    162.12376    162.45809    157.79405    165.93929    166.53787    170.40817     167.7285    166.00839    165.53854    166.59896    173.93175    167.92656    169.80229     171.6271    169.31874    174.10304    175.05659    179.08583    177.99177    177.72243    179.26841    177.71571    175.86525    185.04539     178.4978    178.36349    186.03589     184.0705    179.11571    194.57583    188.53832    181.70876    190.22626    183.23033    186.68121    185.62881    188.15652    197.37537    201.12977     190.3662    194.49518    195.08974     201.4634    208.62362    199.98086    192.39885    194.97265    199.91687    202.76423    206.77994    202.51584    207.46341      198.824    208.93262    210.47244    213.11943    214.60053     213.9577    219.34141    213.76427    220.93086    223.44858    233.88025    229.77361    233.31595    230.90451    225.81279     229.6528    242.49922    236.97165    230.60879    240.99724    237.73511    234.18744    249.70411    238.16097    252.01862    245.06148     248.8723     235.8119    234.09787    258.17019    267.98822    265.99869    242.70768    258.97472    254.95844    268.39114    275.52153    244.85289    260.55726    264.02302    264.59465     273.9784    294.18143    289.80483    279.55599     302.1219    299.12249     281.8126    302.78193     322.3859    324.81954     328.5243    312.79934    340.66014    335.70302    366.04643    348.43674    368.38182    376.51137    372.44554      357.108    458.10909    478.29147    394.58648    447.39396     432.8937    434.56071    472.63717    551.72899     484.9583    471.81085    521.43556    500.09549    608.98899    798.64434    800.40405    750.74037    714.00776    775.19507    744.05061          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -2.50    258.27143    248.81951    231.64264    226.12325    225.06174    220.67872    222.01009     202.9355    202.57231    199.29605    189.07063    192.11864    189.84163    179.48992     179.2271    177.95449    175.90393    176.47007    169.01961    170.51521    168.45833    165.23021    169.51629    167.39221    171.11165     194.5827    230.98042    254.65506    241.68991    196.54536    173.57732    155.89658    162.30456    151.47014    151.40825    147.56261    152.41822    148.46414    149.21756    140.58839    145.46648    143.02851    141.11042    140.89253    143.14562    142.88836    138.75181    137.16297    140.00042    141.00957    141.74988     136.2639    135.62546    131.46664    132.90639    135.83893    137.79608    141.96423    144.78837    138.07622    140.36332    136.67806    129.03429    131.36169    131.53664    127.06353     131.7973    130.04257    134.00126    134.85242    152.83681    164.89972    175.21748    174.12065    155.73226    146.70574    139.53541    137.95034    132.75473    130.43353    132.39326    133.88862      135.329    135.36237    126.96932    122.21711    123.25017    123.29385    130.42836    142.44549    157.83432    176.73319    186.22693    188.74908    180.86999    174.49843    168.66723     156.5468     160.0154    148.09268    141.92196    136.33795    133.26904    128.16319    120.50636    122.73223    120.85724    122.10354    121.16348     123.4066    119.74702    121.64838    118.14755     117.8981    117.00176    115.76375    111.95814    114.02407    118.21872    113.18629    114.06684    113.40743    116.32651    115.78654    123.17623    128.62524     133.4207      132.987    128.65972    123.73431    118.37381      119.539    113.56362    117.30463    120.16385     118.0612     126.1884    142.52337    154.00683    161.95837    167.51898    164.72511     149.2615     141.3882    131.20808    129.01544     129.7498    126.97847    127.35149    122.83911    122.68446    120.48803    115.86499    115.79742    113.61124    113.91953    113.56069    117.18588    116.43087    117.62022    115.15414    116.95129    113.95455    119.56588    119.00884    119.89178    121.44323    121.98305    124.71387    123.91909    123.48259     127.6426    121.91025    126.32637    128.60064    128.42021    129.85768    131.01789     134.5497    130.66547    131.07735    125.61725    122.07826    123.29719    121.70553    120.97244    118.56368    118.47738    118.06827    119.19836    118.04212    118.40181     115.7028    118.16325    120.81394    122.18183    126.86537    127.89554    127.14846    126.83528    127.28471    127.09851    127.46265    127.37605     123.6377    121.20943    124.46239    121.99391    130.15528    125.42425    128.95286    127.94589    125.50962     124.6941    122.53193    122.00371    116.60941    121.23557    120.23066    123.51621    117.53996    123.24391    121.82524     125.2564    124.52457    125.75724    127.10418    131.17463    128.32886    131.90674     130.3585    125.23511    128.33018    132.17495     138.2142    137.92572    140.70803    144.53691    144.37185    144.19893    143.43805    141.75963    136.61582    136.34059    140.38157    134.23985     128.8213     133.2837    131.77155    136.30956    141.22943    144.36216    140.59924    141.17131    144.69617    139.43455    137.59992    133.08773    133.42686    133.97011    138.48332    140.90352    139.43616    139.84095    143.41625    141.41803    141.36849    139.91835    140.48434    140.40375    136.75921    135.01946    134.60831    135.25431    129.63537    133.17905    129.96113    129.96758    127.74493     133.6743    132.41403    133.19731    131.82662    133.16679    130.68239    131.67462    131.08745    130.96091     130.6272    130.03137    129.19192    129.75248    134.08442    131.97825     129.2699     134.0447    132.24769    126.18387    128.95325    129.13557    129.68322    132.94501     130.4513    128.38229    126.08789    129.92038     129.8434    129.66374     125.6786    131.89932    131.29426    130.85562    137.59601    126.97567     136.2002    136.58354    133.87463    132.69394    134.56476     130.7607    132.00264    136.40459    131.09245    132.04487    133.09813    135.32357    135.20972    134.63714    136.72016    136.55343    138.68672    143.13995    137.80558    138.98152    137.08506    137.27077    137.43684    146.45888    148.65704    149.00062    149.84801    146.98877    140.85835     148.7923     151.0203    146.71283    154.80686    153.74732    149.97895    151.57338    156.23711    152.67905     157.6996    160.18921     160.4688    157.64327    166.00305    160.55986    164.61521     162.6823     165.2949    164.04227     168.5718    168.41157     167.7313    168.81646    166.40085    164.43845    167.74416    169.37383    167.49525    171.27418    171.37102    166.92005    172.92455    173.14392    180.11097    176.28248    173.64849    178.08042    172.40886    177.36717    187.35499    184.73737    183.46623    180.02093    179.63941    185.06606    190.13565    184.42687    190.05101    187.48148    185.57312    188.53116    182.90814    195.79085    183.93312     189.3415    189.98225    198.75508    196.08935    201.54661    209.54212    202.90106    196.35462    198.30964    202.78161    207.17199    210.00761    203.12935    208.02348    214.15642    210.59709    209.37075    213.56038    230.29854    221.17498    220.11978    213.02759    210.91026    214.65939    225.48407    232.67025    225.83424    234.82936    228.73162    221.76318    236.77579    224.13181    223.57546    234.43694     229.5392    231.18377    244.77823    248.44332     251.4373    233.83347    254.01169    247.53707    241.26256    256.49162     262.4815    248.30948    257.22834    258.22894    253.42293    268.59348    258.31832    281.47676       271.07    267.36018    264.71924    297.19657    297.21064    269.32529    289.58641    289.86248    289.89672    282.31576    329.31166    317.86796    296.45219    282.21445    321.87552    300.81314    339.48676    340.42728    366.12837    363.49975    344.25069    361.51514    361.36904    427.35281    392.24678    376.35121    407.91229    423.39745    427.57744    442.35718    530.35366    494.79072    549.29024     514.6214    472.24018    590.35123    729.01705    763.53018    682.40797    797.94521    810.01336    771.46575          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -2.00    258.67122    249.47722    228.63843    224.33494    222.16289    220.02763    220.41106    204.17472    200.20374    198.62259     187.2479    187.96427    192.27525    179.13422    175.57447    178.71643    171.86387     175.0712    170.97314    171.16966    167.14597    165.49249    168.77163    163.22846     173.0446    193.24414    238.23366    251.10632    235.81019    200.49281    169.32879    157.51884    159.28354    150.99446     151.8043    147.99789    147.40643    148.55745    147.89311    144.03425    146.62648    144.43685      140.207    142.41489    141.80807    141.53355    138.71289    138.30731    139.82388    141.92328    141.39238     138.2129    139.77699    136.30963    134.01709    136.95979    136.46856    141.51334    138.78404    142.63877    138.18251    136.09135    131.80254    131.16267    134.31605    127.41694    129.23678    132.50041    128.41672     136.8826    150.93595    167.42415    174.98349    171.17771    159.11715    142.91474    143.48482    137.45529    132.81643    134.16186    133.88541     133.8227    137.48034    130.96127    125.34838    123.25152    120.47276    127.15172    128.89325    141.17263    160.76294    172.69573    189.30796    188.42881    180.86948    173.89586    168.66697    159.20554    161.33999    150.01726    143.54067    136.77959    134.52601    130.82939    124.00943    121.51734    120.68871    119.17903    119.06027    121.01248    117.73179    119.28697    118.43654    115.42968    115.34532    116.15356    116.39547    116.09133    119.64621    118.24137    114.44438    116.77781    115.39387    120.88314    130.21472    134.27467    139.09447    138.53666    130.77262    125.40533    121.42834    120.81909    121.17654    120.40271    122.24655    120.61757    126.44585    143.23075     156.3387    164.94955    166.01236    159.80406    150.01749    145.03371    138.28541    130.68949    132.40609    132.83697    128.04921    123.67991     122.2681     125.4328    115.01773    114.83057    113.39592    113.26283    114.18019    115.60997    116.82298    113.98123    115.24701    118.34806    115.66926    119.45387    123.64125     124.3561    125.62036    124.91962    123.76791    127.93324    125.74904    121.85532    125.97407    132.56194    135.35429    129.68665    131.37408    137.00179    137.39933    130.71205    131.06561    124.04907    129.73926     127.7011    120.34857    122.47894    119.82408    117.42058    118.87546    117.65945    115.85335    117.00902     114.8902     121.9208     118.6306    123.73621     124.2425    126.52033      128.648    128.78568    125.74275    124.79977    128.18009     122.5724    129.66877    125.76586    124.84541    122.33786    128.07886    126.89402    126.22612    127.09647    124.97673      122.154    123.11972    121.21631      121.565    118.52174    119.60927    124.25201    118.49413    122.47201    124.39482    128.47063    128.04625    130.64761     125.1123    130.32815    132.52018    128.90663    127.91669    129.96697    133.71338     130.5889    135.07628    133.84574    137.11945     139.3339    141.26312    140.42852    141.12509    138.58398    138.61032    136.61817    137.59134    136.14352    132.09124    135.49657    132.13806    138.21893    136.52373    141.52703    139.56393     142.1627    134.13487    132.78861    133.20999    134.78403    132.29564    134.02777    138.44294     137.9214    140.62229    141.25209    142.54161    142.82976    146.45671    146.92173    146.61767    139.25181     137.7597    137.42499    132.21117    132.51099    133.36353    127.94804    126.83048    130.67852    135.42875    133.88483    133.65395    135.39714    134.69549      133.959    131.86386    132.77895    132.84678     133.8431    133.79357    133.36893    131.46336    131.25566    127.29799    130.04441    126.18391    131.42443    126.30433    126.93609    128.31349     131.1487    128.62052    125.11501    129.80075    127.30967    129.52041    130.17762    130.25083    129.87289    130.55666    126.14131    131.93852    135.80118    133.94591     133.9402    133.65561    132.53938    133.89366    133.28577    131.89525    133.65946    134.12154    132.15134    135.42022    133.13259     133.3931    137.51927    135.05185     133.7659    139.15957    136.05569    133.89483    138.54066    141.59061    138.90071    137.68823    138.73442    143.36858    142.17035    147.22853    146.01503    147.72162    142.71784    146.15111    150.24306    147.78677    148.65082    152.77775    149.39683    153.13815    149.96593    154.60762    156.59916    153.77552    153.94265    156.72696    153.86412    164.21992    158.93257    163.49608    163.63418    164.29783    163.67467    166.22135    165.40884    166.57433      168.029    166.80725    164.82447    162.67559    174.52091    167.66612    165.87839    169.66843    168.33033    168.40143    174.68348    172.20899    175.01034    174.29952    176.52088    172.27372    174.88451      180.964    181.25531    178.04222    180.19085     185.2448    180.61675     180.0437    191.49599    183.00566    180.84792    186.94335    185.65653    184.44196    191.30035    191.91206    186.30456    189.06763    206.21173    198.40419    198.82759    194.52631    195.27969    197.60954    198.69282    199.75163    191.85791    201.10454    203.17936    203.60967    202.57075    210.04653    208.26234     225.6975    215.23023     224.6347    206.19228    210.76847    219.34026    213.56254    228.49507     224.8375    227.76134    222.31172    221.56395    231.49039    232.88778    233.80213    235.25879    235.05123    242.69262    234.22674    230.99043     231.7455    246.48661    221.06614    251.19936    234.81419    225.27842    246.56792    256.01295     247.5156    255.74077    238.38098    252.90199    259.50007    264.36046    270.24643    268.43228    265.57103    250.40144    286.49209    275.37486    296.95803    289.14619    262.27024    288.40964    290.78181    310.90478    321.15944    322.82465    325.42532    293.01987    331.71022    346.24137    367.15564    366.76699    366.22764    313.52121    367.16945    359.88793    380.39354    431.80262    406.08383    401.90292    405.40765    412.81991    425.02108     559.4297     533.1685    517.90596     485.5433    507.93429    563.12263    755.94884    712.03188    845.54909    716.91393    775.47782    676.27252          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -1.50    259.18677    249.99986    225.52368    222.56576    219.28692     219.3044     216.0423     201.8573    196.05056    198.86364    189.07557    188.48071    192.90094    177.84551    178.27004    178.35687    172.07638    173.79306    174.79693    172.67842    165.11344    168.40903    169.43426    171.55591    175.13923     199.1111    234.29025    249.78135     236.3391    203.38673    170.22385    158.30832    155.20526    151.04346    156.55016    145.28958    148.06572    149.92061    146.83039    145.04435    143.43437    144.88633    142.78759    143.00666    136.39206    142.62141    138.87967    137.02259    138.40047    137.59204    138.21337    139.27997    134.87189    136.49083     136.9502    133.69267    135.49834    145.38641    144.02736    141.82348    138.76624    133.80083    130.46971     133.8311    132.18494    127.86997    130.83688    128.35088    132.71966    134.49191    147.79686    167.62424    178.93101    176.82331     160.8814     143.5378    138.14834    137.22256    131.05881    131.08019    133.06863    136.95366     134.1292    130.14646    125.89817     124.2207    119.50494     123.3119     127.0926    141.14251    159.17108    176.79216    187.86595    187.03768    178.04008    167.99304    164.58387    161.09084    158.45287      154.723     147.9764    138.03336    130.99908    129.62191    122.29848    123.03977    121.34005    119.48788    120.16778    122.33023    118.91496    121.41272    120.13377    117.92785    117.30011    115.35121    116.76294    116.01738    117.32539    115.31715    115.71319    115.01485    115.64659    119.54576    122.67722    134.02651    139.74649    135.32447    131.98784    127.88219    121.00866    118.96197    116.74065    119.53188    118.69087    123.31133    126.58936    141.70817     158.7079    169.94008    169.49685    161.73944    155.94531    143.49759    138.42011    137.35839     133.4752    131.73807    128.15085    127.09856    121.88714    122.01987    115.98693    116.21626    114.13338    114.05496    114.44886    114.76655    115.95242     116.6893    114.78414    114.46588    115.32453    119.13987    117.93288    126.15174    121.82314    123.60722    125.13777    125.68968    124.35137    125.10177    129.54747    132.53212    132.74214    128.78528    136.39626     135.8097    134.96538    133.57523    133.39151    126.98661     129.7358    124.44853     124.0633    118.76857    118.12062    119.71742    118.14317    116.32461    118.10358    113.16625    119.63376    118.56387    117.86986    125.58181    125.04428    125.99649    130.45482    131.16303    126.02632    130.06971    129.15783    125.02463    124.36142    123.94503    122.28763    124.80854    125.65211    126.80726     129.2417    127.81413    127.95046    123.34391    125.98612    124.83349    121.42849    117.55721    119.68979    120.98204    120.27479    123.33119    123.75219    126.36911    123.30219    132.62585    125.12885    130.56725    129.80398    128.12251    129.14758    127.72988    131.72407    128.18159    132.43136    136.46847    137.65582     137.8722    138.86629     136.4153    137.50725    140.66465    140.48443    139.35907    132.77304    134.80376    129.95157    135.32434     134.0339      128.036    130.82813    133.82968    132.04826    134.49631    135.07682    134.27942    132.07219    134.99877    135.19078    136.83806    140.49094     140.1498    142.85642    141.95593    141.21235    141.63385    147.16482     146.2576    142.61973     140.8865    138.38379    133.39789    132.42963    132.46012    131.75606    135.34988    132.67007    133.43911    127.17172    128.93878    135.64014    134.70719    136.75893    133.67126    132.98637    137.16733    136.03461    133.72566    130.66485    133.32435    135.30513    130.49385    132.92228    129.44874     130.7059    130.51249    132.04009    130.96902    131.37441    133.25515    130.03845    128.73776    133.03407    128.16316    126.58743    129.46471    128.99555    131.81811    127.74848    132.03523    128.77339    132.56844    131.46422    133.87398    133.75511    133.93925    128.66764    137.20355    136.60914    132.43106    134.20362    137.28079    131.94447    132.58322    137.78912    134.94442    134.38073    141.64426    136.30072    134.00038    140.47388    137.55413    135.40907    142.27918    138.19952    141.77789    141.84207     148.0441    144.82948    145.02965    145.64634    142.84945    145.16379    150.25862    151.11347    148.32191    147.11624    143.66006    149.88124    148.20604    153.75927    153.55542    151.60286    153.90544     156.7444    157.69953    163.14638     165.8637    163.79993    167.28771    163.21831    164.45289    166.73969    166.41385    163.93938     162.2723    163.29779    170.77528    163.10753     165.2466    166.84688    172.10139    171.39652    170.03798    165.98654    174.64921    176.51404    174.35949    176.61286     175.3476     173.8858    169.38726    179.84623    180.28482     178.0356    179.90196    181.15164    171.45441    186.83629    183.72685    182.86545    191.16677    179.49204    178.56294    185.57593      190.943    189.71209     188.7018    197.11568    195.91577     193.2971    192.12284    195.17575    204.16416    198.18195    194.97731    190.87865    200.77414    198.36102    208.92935    199.31302    203.23086    207.65286    201.24652    222.33953    218.55087    216.29852    209.48841    224.41816    216.92921    213.42461    216.14652    225.27532    222.00533    211.84591    215.24033     220.2791    233.30108    229.22285    232.67379    246.48461    223.10168    231.13984    233.50938    232.90965    235.64785    232.38707    247.11457    242.22181    242.30538    241.63362    234.69821     272.5136    240.39462    238.63956    242.37142    265.14141    270.99082    260.19974    252.88065    258.83602    267.29699    266.87855    276.83676    266.89045    274.59635    288.56275    274.05264    298.38871    305.98988    295.48116    311.35165    316.70104    311.61925    321.13924    326.91538    360.02494    343.30647    333.94411    344.33241     338.4861    364.17769      388.929    370.68309     417.6681    450.13847     394.0922     395.6461    433.53349    531.49578    562.00315    448.39851     517.4407    542.52245    602.36357    732.64102     699.9805    750.64417    637.03675    710.06912    750.25632          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -1.00    260.22368    247.67646     229.4042    224.95599    220.37404    219.80001    213.37606     201.5426    192.80433    199.07538    190.95709    189.42876    192.82241    175.94599    185.59564    176.98903    178.70672    173.20241    174.98467    174.56342    168.84394    169.13295    169.81295    170.80978    174.95503    199.36675    233.87628    251.06278    235.68037    202.27582    170.16741    158.46262    156.12307    151.84444    149.66187    151.19806    152.28874    150.53549     146.5411    145.86255    146.97052    146.09711    138.27118    145.33035     138.9815    139.92623    141.07197    137.86557     139.6562     138.2281    138.07664    137.04997    132.26581    134.08106    134.67244    134.13826    137.74984    143.50719    141.18956    144.83288    137.12694    135.38595     132.1807    133.91738    130.30858    126.29492    131.92967    131.13863    128.20508    135.26011    152.28403    169.69763    177.68743    173.48346    160.00721     149.0236    142.09558    135.97704    137.52764    131.79278    135.07063    135.25477     134.7455     130.8159    126.58832    123.68605    124.57134    125.21236    130.63525    142.70534    161.32896    179.69045    190.24345    183.00501     175.7244    165.37889     166.0423    157.74216    166.44162    159.41705    153.14315    143.79666    135.35571    133.00155    124.52537    122.43164    121.58038    118.09415    121.04367    120.96884    118.84922    121.52687     118.6176    118.60431    115.20539    115.89468    115.95897    113.92449    117.92901    115.51236    117.35381    117.85764    118.47612    118.79503    127.53688    136.70207    137.23715    137.78995    133.76032    126.58394    123.95391    116.83186    117.26522    119.22636    117.29054    123.61805    126.98278    145.23379     160.3839    170.39106    173.07689    169.12243    155.65518    142.82026    133.16893    130.66491    129.79933    130.69711    129.18363    123.29198    123.42551    119.58257    120.69411    117.03633    114.82109    113.42291    115.40666    112.71106    116.09446    117.04584    116.93912    114.45375    119.70359    123.38734    117.42441    123.71567    124.47889    127.06821    125.07136     124.1233     128.9689    127.38517    131.61374    126.67231    130.53744    129.88922    135.08726    137.75743    133.99552    133.28033    130.47743     126.4854    127.87187    125.79935    122.47937    122.28343    122.75697    118.39914    117.90428    120.59157    118.21048     115.8986    119.91449    119.27958    118.85252    123.63547    125.94791    127.15868     129.5234    129.34744    129.17738    129.19387    130.37391    127.82086     125.6788    126.32538    122.51906    125.04177    123.63553     127.0431    130.13121    129.38104    126.36455    124.01173    122.58748    120.44431    120.71625    121.23683    121.06988     122.2573    119.22395    123.57099    124.08388    125.39752    123.59693    127.58312    126.36004    131.47387    130.54297    129.84175    126.69163    132.78895    130.30335    130.47238    134.29997     135.4677    137.71436    140.27713    138.30496    135.56849    134.57883    138.84763    134.30572    133.80301     134.3735    136.64201    134.10348    135.05838      137.107    131.01395    134.05189    132.33227    132.82752    135.25558    137.14527    137.70835    139.91318    135.94822    134.89204    137.91721    140.85034    138.01735    144.14424    142.70389    142.65325     146.3266    146.54556     144.9027    142.62184     141.1156    139.01288    135.74827    133.78102    135.66545    129.79452    133.03567    131.38953    130.45316    129.68532    136.96412    133.24915    132.41781    136.94842    137.18556    132.79038    133.87074    132.67378    132.52242    131.55724    132.41326    130.18451    128.48794    131.84097     133.6926    130.24678    135.21071    132.36843    124.91377    133.64963    133.20653    129.25121     126.3237    130.66367    127.99522    125.51383    129.85809     132.4827    131.60749    133.57325    134.24656    132.34086    135.86379    133.94119    134.27842    132.49396    131.84763    133.35131    134.30076    134.09881    133.07335    129.28615     131.4676     136.3284     133.4092    134.31503    135.17238    136.71315    134.60625    133.58952    132.51013    139.18602    135.99072    139.34271    140.55471    142.03157    141.77693    144.63166    143.13931    143.91628    145.15723    145.20107    144.53516    145.39036    148.07586    148.43066    144.79787    143.40841    144.40664    147.26919    149.47309    150.63632     153.0904    154.34358    153.07158     155.6869    161.78834    162.24889    163.82314    160.54211     161.8303    161.97406    161.47933    163.52254    167.41358    160.68837    168.18524    166.81362    165.13399    167.69326    170.65994    170.75788    172.02506    164.53314    170.57308    171.64547    174.45989    175.72469    173.59894    174.05788    175.82597    176.39141    172.79298    180.53765    180.82361    181.84791    172.74666    179.47763    176.79242    179.11926    178.10715    180.16684    184.88781    182.52189    183.06855    184.18887    185.90928    187.56069     188.0793    189.59487    185.63009    196.84324     195.8657    200.46368    193.19202    199.42838    196.73075    205.31951    199.93498    211.64785    207.20459    201.80797    202.25981    198.56306    204.22393    222.37685    224.92447    208.81703    214.88331    210.77947     218.7929    215.43458    223.86199    231.32623    224.84241    220.96189    216.80423    224.77472    231.81814    230.10346     236.5349    228.61363    227.25767    221.19198    238.50237    238.12215     232.7666    236.30445    241.62995    241.81766     251.8242    239.95296     256.6162    243.91999    258.96265    239.39057    245.91945    242.23883    256.11925    248.36648    258.86368    268.02523    263.56568    275.39245    295.54928    293.35989     284.4247    292.49626    276.69896    287.89143    310.55261    304.66608    300.61099    286.87319    322.51772    300.34133    334.95904    377.11819    348.01301    355.62097    349.49028    370.46473    360.17942      408.978    427.40004    443.83756     405.3045    386.47871    390.64802    430.55765    537.06753    479.47544     545.6405    501.68443    493.91524    527.21682    781.56884    703.75587    714.72131    609.89314    640.11558    609.00588          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -0.50    261.31901    245.34324    233.84538     228.3116    223.27229    221.25342    212.37999    204.43151    191.78917    199.27765     194.2262    190.83544    188.31658    176.82471    181.31307    176.34617    181.57952    175.34903     178.5568    174.92914    166.54806    169.27651    168.23348    168.93427    173.14527    198.16604     235.0951    251.73054    235.49117    198.45056    166.28906    157.43704    152.68008    151.85406     153.4578    151.47745    152.79114    148.71114    148.82867    144.91079    145.35829    142.58236     143.0255    140.01308    138.28606     140.9062    139.96854    139.41052    136.77286    137.79811    139.35256    135.67238    133.60383    136.41705    137.29189    134.69127    137.37169    143.50388    145.60978    145.40471    137.40335    139.39194    132.99542    136.07651    131.61863    129.45173    129.84647     131.2236    128.18509    138.72258     151.4363     170.1743    180.22107    175.33184    161.07542     146.4611    142.06157    144.74232    137.09984    136.84708      136.644     138.4827    135.83753    132.98637    127.70362    125.33183    123.91804    124.03096    128.13638    141.54286    157.35624    179.40405    192.60677     187.1531    175.53669    168.17389    162.87044    164.05509    168.03252    160.21254    150.35875    139.80492     136.0722    130.24153    125.42752    122.27175    123.18317      121.508    118.57591    120.79925    119.47875      120.184    121.25771    116.11651     115.5558    117.97345    116.36678    117.27533    117.95007    117.75813     117.4996    116.02245    116.51725    120.97801    129.38979    137.01874    137.62493     137.7619    135.45321    128.21156    123.48951    117.30671    116.47177     115.4868    119.65487    122.79675    128.93474    146.53036     160.5913    172.79197     172.6806     168.7969    156.70883    148.36485     141.9008    134.28032    131.69965    130.49921    128.10974    126.81786    123.82443     120.7285    119.47627    118.46967    115.98458    112.10034    114.82648    115.19893    115.40679    113.30838    116.54045    117.28634    117.52565    122.96231      119.104    123.46167    127.13767    129.01175     123.1966    126.12636     128.7448    130.19178    129.99491     125.9876    124.49176     129.8406    134.51021    133.64265    133.73365    132.28754    135.37485    128.22759    128.11851    125.37005    121.70578    119.35165    117.94089    118.46638    117.20546     116.0667     117.8672    118.01193    118.92555    120.31308    117.41683    123.04184    124.62459    125.76742    127.72103    128.48192    124.81685    126.95699    128.22281    124.13932    122.71141    123.77439    122.54998    124.55113    127.51313    127.38566    128.51622     127.7591    127.31414    125.73908    121.56498    122.97946    121.17706    119.86829    117.10344    120.64082    118.27064    124.66606    123.20671    123.05911    125.41854    127.14744    125.90155    131.72486    128.57393    128.72807    133.69134    131.40786    134.06334    134.70397    137.00492    136.89848    138.47818    137.48065    136.94288    139.77097    140.68841    138.15063    136.04751    132.95329    135.95998    130.59607    132.76168     133.1818    140.17732    138.51584    132.65046    131.71498    135.43361    137.56137    136.78663    135.22479    136.54328    140.64068    137.20436    137.82319    140.51044    140.96413    141.45207    143.15782    144.10245    147.83845     146.0472    146.14385    144.62213    140.01821    138.72907    135.30484    135.32552    133.05215    133.75032    133.61726    128.06409    131.13862    132.55732    136.17246    135.93978    134.45404    132.59101    132.85572    132.86962     134.6806    132.78694    135.85661    133.80473     134.1547    129.81609    129.62762     136.5806    133.51475    130.23414    128.64453    131.00948     130.0999    130.93415     133.7022    131.48873    128.96974    128.88385    130.60217    126.80426    131.75462    130.40212     132.0297    133.29963    131.88648    133.08232    132.02241    132.66388    134.86018    130.09486    134.94467    128.46222    130.42089    135.92165    133.34955    134.23679    133.29277     133.0956    129.72347    133.71172    134.99115    135.52411    135.76732    133.68731    132.71878    136.76526    140.06607    137.03491    139.04632     140.9371    140.89491    145.20284    147.87782    147.84197    146.96257    143.14737    143.04161    141.95152    151.91327    150.03581    148.95863    149.35715    151.21185    149.21014    150.36966    154.34942    152.71527    155.68198    153.89436    155.74164    158.67367    161.77838     163.9951    163.35608    158.40018    164.54284    159.07884    162.42476    167.03373    163.29685    161.56963    167.98362    168.41338    161.90785    169.82098    171.71412    167.46884    171.68728    171.49049    171.18942    170.16642    169.54286    167.02656    172.30259    169.20085    169.16524    168.77074    180.20016     178.1914    171.03475    180.33851    168.87807    177.06929    176.35016    183.91831    178.88215    182.23092    187.11024    180.64355    187.48753    190.06408    185.50376    186.78302     203.2643    190.85944    188.27742    197.78487    198.87111    192.48435    194.66886    202.25845    194.79408    202.44113    199.98276    205.92292    196.75786    215.04248    203.73096    209.84262    218.43162    212.52623    210.99542    213.94193    214.90282    212.28267    216.61217    210.41958    221.54403    220.77713    213.11694    212.78748    223.42318    233.05492    222.58656    221.21366     224.8455    220.14861     229.0984    224.23797    237.18805     229.1716    250.36079    229.28592    243.90701    235.10471    249.74481    244.50938    269.49179    248.40668    245.17934    250.79414    260.30403    253.23264    252.62606    251.85446    259.32987    253.63202    259.34485    282.51234    259.37159    275.39206    275.13917    270.55895    284.20567    279.87784    313.71948     300.8394     341.3163    304.40722    299.72922     289.7959    341.71851    365.68902    342.27498    334.45956    338.69438    317.90254     403.0243    379.36392    405.15167    426.64266    413.63818    370.34668     421.2351    517.54413    480.14349      510.605    500.95528     548.3246    560.80819    610.22587    700.56661    708.48924    755.57226     707.9315    628.51965          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        0.00    262.48857    243.75884    237.00687    230.28711    224.90648    221.60308    212.61642    206.07279    192.77775    198.36656    195.46769    191.54947    185.67929    177.90783    177.37943    177.14317    182.99739    176.92938    181.77056    173.53777    161.78411    168.91426    165.69191    168.49564     171.2486    197.73068    235.14451    250.31981    240.41301    196.03779    170.76259    154.85465    156.13902    148.53597    149.96083    150.14788     152.0712    150.35063     151.1048    145.88631    146.10828    144.44104     141.5269    142.77167     141.9412    147.79735    140.78524    139.16142    138.76821    140.33071    138.16998    137.23743    136.32101    137.55255    136.65982    135.05001    139.31873     142.3257    142.01241    138.02828    137.62023    130.83806    133.24436     134.7721    128.25281    126.47077    130.19403    133.11609    132.65977    133.13685    149.12234    165.42236    179.56658    173.52248    160.71386    143.64758    142.06685    139.82277     142.7985    144.86923    146.42104    146.96028    137.39582    136.18595     127.4779    124.89312    125.80554    122.85144    129.12517    141.37555    158.40673    178.59233    191.13351    188.62029    181.02862    170.68402      168.311    166.68515    170.31548    164.18029    153.63387    141.43944    137.07658     130.8532    125.63547    126.09335     123.5892    119.38645    117.17467     118.8716    117.36709    124.13362    119.45355    116.65824    116.33882    115.79756    117.42323    113.14444    117.20182    117.04783    116.58201    117.09726    120.20055    117.86418    129.11227    137.01579    138.83388    138.39206       134.95    128.14489     123.9453    114.66026     116.7925    119.19711    118.89502    120.96646    131.90246    146.81635    160.59944    169.93179    175.78388    170.50806    155.29795    147.37254    141.02626    133.69511    132.22368    131.32828    128.72035    125.41107    123.85635    118.54092    119.40171    114.41193    115.97518    112.31353    115.40029    116.29551     114.1991    112.12325    118.42641    117.46551    117.91109    120.62871    121.84626    124.36384    125.47663    121.23624     122.4871    121.75675    129.39938    128.99512    129.22898    130.89191    124.51724     128.4186    134.63819    136.62609    136.98006    133.00934     134.8515    129.46543    128.89844    130.01932    121.54269    127.34009    120.86989     119.5146    115.50047    119.70017    118.31051    118.47536       118.34    120.07453    120.26929    117.48392    124.79489    124.67366    130.01451    129.41287    127.81091    127.20052    127.49797    125.24958    123.17841    123.59168    122.00749     122.6649    125.59771    127.96658     125.1177    125.23796    126.20072    122.53667    126.38429    123.23177     119.5849    121.74479    120.76533    119.22549    118.47923    122.30008    122.87225    123.60198    124.53447    125.18359    123.92191    127.14855    130.91287    130.38117    132.51331    130.21724    132.62535    136.79384    135.79453    135.75241    142.38053    141.86046    142.05699    137.58815    140.16114    140.25933    135.38535    135.43758    138.33977    132.04138    132.43446     133.0995    133.01005    135.83946     133.2626    133.32734    132.18858    135.68452    136.56887    138.95816    138.96557    137.18507    137.74522    138.66772    140.98445     141.6666    139.84851    147.82955    144.23091    148.25432    146.66346    143.68296    147.69703    143.78955    138.77805    131.33597     134.9265    133.95185     131.9252     134.4903    134.77802    128.98677    132.98632    134.72141    134.92202    137.17409    132.69164    134.72756     130.2235    134.06085    137.81347    132.58274    131.86417    137.21246    132.86874    132.47404    132.75626    129.22087    133.13715    133.28686    130.87231    128.59857    131.44452    128.74126    132.21153    131.19671    126.38216    130.31546    129.59952    129.98469    130.27928    131.15506    133.14939    128.70005     132.4486    127.28024    133.01183    132.44168    128.14631    137.38953    130.80102    130.40889    136.15667    137.00832    130.76088    131.26647    134.44295    132.20999    135.40931    133.80238    132.21952    136.18588    131.62909    131.37637    136.32133    140.31007    141.14778    140.40159    140.17267    139.89271    143.32752     144.7055    145.18774    147.62665    148.23634    146.54023    144.84938    147.49406    148.80575    144.93139    143.65466    146.61158    150.40511    147.74302    157.14457    152.68434    149.52414    149.99999    153.59747    154.50658    159.62187    158.40751    165.59169    162.97047    160.21626    162.19472    158.70938    166.60824    156.87329     165.2318     163.0276    164.00136    166.17038    162.94757    166.18458    169.11916    169.44519    170.52707    163.21774    172.52117    178.78709    166.01816    169.95295    174.68684    168.32786    177.98118     177.0415    176.74983    174.33365    176.92682    173.93768    177.32687    182.72355    177.36992     181.1179    178.38619     181.0591    174.71054    179.72925    174.60628    187.66672    188.19475    190.01057    190.24515    186.87908    190.52415    191.52344    191.13189     200.0632    197.38241    192.80807    203.95849    204.75827    203.19631    207.67482    206.11424    195.20661    210.09058    206.37004     210.1817    208.27293    210.27584    217.74457     217.2304     212.8118    219.43828     220.7859    222.22625    217.03195    209.66203    224.72075    221.27579    220.63252    222.45643    220.31311    213.19893    229.15667    235.09541      242.309    231.62513    248.78647    240.85846    236.05461    241.67341    241.28897    244.82262    236.54896     241.2496    242.22367    231.64933     253.2012    260.54805    252.38432    251.91985    249.03277     255.2473     262.4931    289.75219    280.60123    269.01728    268.31695    277.17675    275.06999    293.95929    293.42351    305.10503    306.54351    295.88848    305.97314    327.62321    329.18648    351.27368    342.33744    327.18673    337.97363    361.19361    407.19653    398.58812    430.48546    422.32143    383.11042    396.16297    432.34138    496.62889     462.1319    522.04415    494.70421    527.18096     575.1508    598.79621    715.13002    671.16759    581.85943    609.79731    683.91852          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        0.50    263.74372    243.53239    237.68633    229.47109    223.74747    219.34495    215.94808    204.46325    199.14579    194.15504    190.54974    189.93059    188.88726    178.97447    179.20575    182.47108    182.75075    172.89098    177.43971    168.04124    161.83217    167.77103    167.20977    171.35928    174.30429    196.85018    236.15611    256.48641    235.00747    195.70956    171.22533    158.53765    149.52338    151.77343    153.26705    152.20068    152.64264    147.16011    149.23302    140.71591    148.05101    142.22485     142.8081    143.10249    141.26693    142.16289    137.39223    140.98601    137.86905    137.35022    136.34734    136.22709    134.82309     136.6441    134.88482    134.98076    137.55195     142.4971    141.08681    141.48662    139.17806    137.01258    133.69274    135.49414    129.57347    126.18432    129.73569    125.50742    131.32746    134.84882     150.7288    165.18137    177.55667    172.51076    158.01149    144.81401    141.91977     144.0528    146.93341    150.35079    155.18413    152.10966    141.70378    131.82603    124.67923    122.34487    125.39253    124.98882    133.04398    146.39691    164.52175    183.00879    194.07939    193.13838    188.33069    180.23228    174.74136    166.55567    165.12842    161.10393    153.64649     139.2985    137.73261    130.60835    124.07295    121.20732    121.69993    120.12996    122.91564    123.26195    119.30903    119.73274    121.71441    120.63431    119.73116    116.71736    116.30563    114.63611    120.75873    115.88219    117.56624    117.35029    117.43109    118.46286    129.89912     135.2219    138.37471    138.93191    133.86467     128.7163    121.56207     117.9147    118.66366    117.82534    120.88756    122.25112    129.36888    148.87143    161.44345    174.41047    176.53976    168.11494    160.50041    146.25519    142.04601    134.98132     137.3115    132.70362     128.3805    129.16167    122.71516    124.20065    119.17477    116.42494    119.08463    114.73821    112.28487    119.47372    118.40768    114.79172    117.46297    116.19414    115.73587    124.78374    122.82133    124.22882    125.89353    125.42471     120.5889    124.51608    125.12432    126.34239    126.51258    125.00699    128.76811     127.5695    132.66855    135.26136    135.75107    134.05177    134.58455    130.49539    130.65511    127.11608    127.08118    122.39184    118.25518    117.70547    119.12039    118.12901    117.64497    117.77199    117.37198    120.13933    118.18624    121.82294    122.46897    125.29907    126.19929    127.92577    127.22753    128.39526    129.57763    124.79085    122.53059    125.71974    124.40233    122.56157    127.41714    129.09805    130.48163    127.99778    127.96082    122.61983    122.26442     119.9996    123.28164    120.86088    122.29001    119.86948    120.11859     118.9215    120.12897    124.26405    123.52165    123.97207    129.89118    127.12671    129.20384    128.34094    132.40334    128.03086    129.44776    135.54027     139.6836    135.45566    138.30823    141.40156    137.85768    136.54431    141.14992    143.16089    140.18953    134.16196    133.62269    130.11928    131.25362    130.19674    131.63847    131.16496    131.03755      130.724    134.82333    138.47805    136.64182     137.7615    135.28834    138.19841    132.30522    135.40702    141.60722    140.48853    141.81556     145.7528    146.34042    146.22504    145.26492    143.60389     142.9576    142.35086    137.96781    135.14576    135.84001    134.82707    135.03063     131.2279    134.39427    133.30612    132.14237    134.98199    136.78773    133.03292    137.61322     138.5243    130.47178     135.1697    131.30255    135.42725     135.0032    133.93684    130.51805    130.48488    134.95751    132.02818    131.89415    132.83885    132.25008    128.79105    130.87837    130.81548    126.73068    133.25093    128.69722    130.74237    128.10902    130.00953    133.89519    133.04507    131.76662    129.65089    130.81159    132.77061    134.99412    134.05499    133.74017     132.9495    131.46827    130.91311    134.08384    132.87703     135.6891    137.64846     133.8315    134.86772    134.13391     132.6031    135.61106    131.79746     133.1942    139.16951    138.83326    141.16668     139.5273    141.85973    142.46101    142.99246    143.51541    145.35452    144.70094    148.07959     147.5456    147.65349    140.71103    144.77565      148.902    145.51525    143.44385    153.02044      151.453    151.09745    150.49553    148.04516    152.16001    156.15685    154.80314    158.56548     155.5875    162.12108    162.24872    163.46533    162.80264    161.60733    165.36609    162.35387    168.68734    166.73399    173.13565    164.06883    163.24749    171.59409    169.82769    166.40057    166.08619    168.02252    169.27906    170.13779    176.51211    167.07397    165.39715    170.77066    170.35345    177.03612    174.97755    174.09221    173.74156    175.26499    178.13504    174.63939    185.78085    182.48132    184.13966    183.12719    178.50576    176.62889    182.52144    192.54742    188.57371    184.59762    181.03994    191.79531     188.2047    193.45822    189.82001    193.64384    189.04515    197.55986    190.21875    196.84749     201.1905    201.51241    203.38102    210.96899    201.52371    197.03187    215.59085    209.47138    213.74545    215.60964      212.495    216.86633    220.09439    219.19562    223.02248    216.90673    220.78932    215.79092    214.19056    224.52833    226.66814    221.32188    222.40567     228.1115    221.57671    230.99653    229.02873      249.906    234.05001    230.87168    239.58013    233.80215    246.48161    245.80756    236.43806    228.17596    231.12762    243.55283    256.82038    266.76618    253.74911    263.66143    256.02143    248.06511    260.32178    281.05957    290.48611    287.07441    263.74858    280.34377    293.09965     309.8803    293.39182    308.73732    307.56282    311.81964    301.52196    325.24909    355.74436    342.28199    338.66046    363.35381    321.59132    335.79679    389.86071    404.56246     350.7828    422.57155    424.11606    369.93306    399.02748    458.50382    425.02936     522.1581     505.8558    482.07529    602.13399     650.0979    758.98101    596.36407    757.51017    597.77179    670.97977          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        1.00    265.05849     243.3453    238.40932    228.68617    222.56093    217.16531    218.83562    203.44549    203.93759    192.57563    188.84847     190.6823    190.77982    179.54451    181.18258    181.90077    177.82431    171.30698    172.04898    170.56515    164.93048    164.23546    172.55146    166.26656    176.42806    199.65994    237.78756    259.77719    237.04045    196.19478    176.90284    164.07738    154.20901    146.73636    148.57657    141.85566    150.49284    149.30363     150.2694    143.91324     145.4847    144.04516     143.7126     143.9793    144.87439    139.32511    138.52498    140.70508    139.77591    140.59209    135.95757    135.64383    133.54017    138.16089    134.41573    134.60011    137.80946    144.88636    137.60972    136.55331    140.24775    137.69647    134.68785    137.91654    131.37767     130.9615     129.2554    126.97528    128.14879    133.62047    151.91036    168.47457    176.49239    171.67891    161.19156    144.82467    143.11268     143.9551    145.67144    153.23588     159.2038      148.451    143.63808    132.52554    126.55977    124.47317    122.26551    125.15792    129.37657    141.29791    161.93928    182.80094    199.95413    202.95178     198.4352    185.34904    176.05391    167.93132    169.52947     159.9005    149.28046    142.73265    136.54522    131.07838    126.02602     123.2663    120.19983    122.36636    122.34811    124.47667    123.16407     122.9501    119.47152    119.20862    117.93632    115.15123     118.6102    116.57075    119.43279    114.75548    117.14719    115.08627    114.61192    120.45536    129.31388    137.32063    142.99623     136.8727    133.16534    130.55023    121.03072    120.85049    117.76904    120.49789    118.98904    122.76208    131.28044    149.10151    159.54351    170.94708    174.28764    171.16113    155.85571    145.28771    140.46623    133.07669    131.48429    134.08781    127.59371    124.44419    127.81792    123.18797    119.46539     116.0643    116.47004    115.86569    115.79319    114.07664     115.4964    118.00797    115.07338    113.73601    116.55477    122.49079    121.16763    126.38381     124.3084    122.47057    119.92825    121.27879    120.57933    121.51505     123.9311    123.75237    128.11978    126.81446    133.05869    132.90823    133.93414    133.42698    133.03085    124.81044    129.66739    125.87694    124.26067    118.97924    121.55048    119.08158    116.68646    117.50936    119.05081    117.75334    118.37054    119.85314    120.04162    120.41517    125.74378    125.27229    124.90131    129.17322    127.37876    128.73175    129.38938    125.38534    121.90223    124.54035    121.43364    122.03095     126.8848    129.19477    127.37126    126.22495    128.68152    124.33007    125.95369    122.59161      122.594    117.60675    123.76873     121.3104    120.33642    121.42785    122.47566    122.11233    123.92459    125.61522    125.62633    129.58488      127.763     131.2315    130.57879    129.78529    131.37368    135.12382    138.74409     137.2159    138.18905     141.2189    138.58732    138.13424    139.94415    139.85448    135.70564    135.00435    140.60967    135.29124    134.88886    134.10986    136.70371    134.17159    135.00151    132.64869     131.2053    134.12394    135.92216    138.00913    138.55515    140.01879    133.48998     134.3805    137.75803    140.30096    143.38092    145.70164    145.90491    147.05487    147.47932    145.89563    144.87903    144.33129    138.08532    137.09083    136.20738    135.18157    132.93362    133.53053    133.35549    130.51612    136.19953    134.08487    132.30882    134.86249    136.86707    132.98736    132.57771    134.24995    137.99497    138.76594    132.47127    136.76245    134.45067    129.83801    134.17818    133.16915    131.27228    131.10491    132.60115    128.71511    128.13023     129.1364    131.02735    130.76434    126.32826    133.21398    127.10378    132.10278    131.15796    130.48875    131.54165    128.41675      128.236    134.10117    131.19308    136.10698    135.70822    136.75385     129.8493    133.66765    130.80584    131.09686    134.55138     131.0459    134.32247    137.57026    134.52477    133.23561    132.28277     135.0621    134.56743    135.68918    139.12933    137.00031    138.11157    140.86986    138.16949    137.55513     141.0022    142.14606    143.47562    146.69604    144.62885    144.50711    141.63602    147.51674    148.80832    148.16113    149.40263    144.07202    151.18682    152.04955    152.27801    152.20365     155.6779    155.68354    152.70973    151.01245    157.39187    153.46386    160.23036    163.08574    162.41742    159.96551    163.71535    162.22266    160.99685    161.04048    160.53183    159.02605    164.27307    166.29797    159.55545    171.85548    168.20284    165.10629    163.61749    173.33806    173.52031    172.29212    170.14771    177.21781    168.17402     178.6086    176.02035    177.41981    174.81795    173.09653    172.54958    171.82941     183.3603    181.36224    180.03548     176.9477    181.61589    168.36113    184.74229    184.84645    180.91562     196.8611    187.28989    183.72726    193.60075    197.96812    195.02102    192.69056    202.39523    196.61758    198.85256    201.36574    203.95046    205.27896    200.24725    199.81496    201.32155    211.37513     218.8957    214.55014    211.55641    212.14904    209.04447    214.12633    215.45184    213.59494    221.84588    211.91872    222.47614    219.59514     216.0598    224.02911    217.15713    233.72611    222.74198    218.74607    215.89641    243.57813    230.90428    239.43557    243.46758    225.65018    224.19604    226.83431    234.03518    234.18643      241.715    248.16054    246.69887    246.86329    265.62124    239.56671    262.38654    252.61305    244.19653     259.7494    260.68524    266.84614    272.81617    281.91823    268.55998    276.69195    269.39823    304.24796    299.32303    292.14175    309.98841    297.66754    311.37395    305.96325    315.71803    325.38202    360.96729    298.31305    355.41497    327.67519    377.38798    373.89281    381.47287    421.06879    382.39895    393.81348    452.17209    446.82282    487.62006    451.47987    452.16658    438.16178    467.42106    675.52034    593.61954    553.73221     723.4078    790.63806    686.04453          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        1.50    265.62882    243.57291    236.83089     227.7713     222.3538    218.99248    217.18989    205.58223     203.2255    197.34993    193.16633    195.21586    191.07651    179.69431     183.3462    177.02694    170.28874    171.60024    167.93079    174.30519    167.12409    163.09349    174.31496    165.20349    174.68532    196.93572    237.46417    257.15732    239.76437    198.38959     175.6709    161.43432    148.27241    147.79759    151.61094    148.59368    153.69288    147.20704    149.40824    142.60288    147.23686    144.24133    144.12633    145.92569    143.55472    142.51595    137.62455    136.44416    139.91317    143.07537    139.74081    138.55363    136.74277    140.29961    136.42096    139.33826    140.23671    139.45505    140.99468    140.33951    138.33396    139.34217    134.93894    132.00085    131.92188     127.9717    127.94727    130.30691    130.49936      135.367    149.73837    169.01151    180.22453    174.28904    158.83459    144.26123    142.25104    140.04402    142.95169    148.20799    152.94631    147.56879    141.28227    135.34022    127.69542    124.32973     125.3032    124.51474    132.02425    143.21229    166.40395    189.75228    205.39329    210.50734    204.23494    190.34112    179.42178    164.60504    164.40857    154.73127    148.09374    139.69989    136.66885    131.12764    123.71789    121.47052    119.48653    120.93979    123.07034    121.80039    121.31711    119.82125    120.58516    121.55624    116.57474    117.33304    114.49496    114.24198    118.55309     117.6997    115.83864    116.78186    119.26528    119.23866    129.31736    138.39747    141.98893    139.65724    135.42571    128.55796    122.89434      119.334    121.30906    117.87889    117.59605    120.96666      130.056    145.60118    160.14502    172.01381    171.20162    170.87723    155.61743    144.50608    137.29284     133.5618    132.68099    128.67664    128.72291    126.43841    125.41416    119.40166    118.24627    116.89589    113.48062    113.74686    113.64884    114.28051    114.34121    118.76348    112.75449    118.37872    116.08592    120.26515    123.49215    124.22271    125.44978     126.0008     122.8131    122.58238    121.68574    122.32292    120.30785    125.66662    125.22588    130.39464    131.09112    131.09988    132.02116    132.76561    131.06908    126.90602    125.24139    125.35861    122.48477    117.30026    116.28762    117.09821    117.51744    117.31127     114.5717    117.22833    118.16192     117.1371    116.08657    122.25829    121.34727    124.19926    126.64441    129.70318    127.16984    129.90227    128.42933     125.6301     124.9541    122.64585    122.36116    121.45547    128.19693    125.89578    127.75727    130.02866    127.86145    123.48696    127.57458    124.39022    119.62583    121.40879    121.01744    120.68533    119.61432    124.02828    124.18785    124.01187    123.03583     123.4751    123.21665     127.0216    126.75668    127.94388     128.3565    133.09941     130.5454    129.71919    137.01408    134.17521    135.30997    141.68182     139.0759    137.37243    139.89628     138.4866    135.90851     136.5756     136.8605    136.81808    131.81666    135.52915      135.616    132.57672    130.73083    132.73964    135.12552    140.32773    132.07611    134.63813    137.12914    136.30226    136.77249    136.40614     143.3091    145.14768     148.0302    146.72811    145.79285    144.63265    143.43944    146.29328    142.23796    142.29523    138.64964    130.42768    134.44282    132.43469    129.33819    132.20514    131.96324    133.79924    131.63385     135.0494    134.85551     134.3634    134.75497    133.97613    134.45388    137.97222    134.64179    135.09476    134.80295    134.21042     133.2461    131.02928    131.21857    134.93542    133.24605    131.17731    127.99854    126.62541    130.74549    130.60716    131.01835    128.40149    131.25205    126.22736    126.89026    127.26374    129.21573    130.21965    134.87271    128.37608    133.17497    134.61666    132.45879    132.54998    131.73685    130.38639    130.47395    130.99878    134.33951    132.56395    133.36785    131.61146    133.95284    129.21417    139.11322     131.7342    131.96081    134.67214      133.784    129.69975    141.79326    140.04198    139.40919    140.85757     142.1243     140.6072    142.63207    145.12895    141.93683    144.69715    143.96708    143.90573    142.84264    148.03949    150.03038    144.76617     147.5558     152.2743    148.21406     147.5568    153.07315    154.82694     154.5602     149.3765    153.15501    153.00414    161.56863    156.77732    163.67516    159.78844    160.82798    160.60378    162.52344    168.36294    167.87682    163.51106    165.70862    163.99362     159.7975    168.82681    161.04974    170.44981    168.36836    166.52862    167.15773    168.20885     170.9419    171.73959    172.14436    167.47436    171.59457    172.33158    175.15305     175.6917    176.66898    179.46688    173.25562    183.98888    179.22964    175.34481    171.83862    173.95093    180.58964    176.64989    183.74901    184.44835    185.27007    188.84271    195.57999    181.78655     184.5616    191.17568    195.06614    202.86963    200.91935    197.50694     188.8984     197.6507    203.69464     204.4851    202.06266    202.96418    194.27628    195.78211    213.10431    210.08305     203.7033    209.53523    211.32494    204.14162    212.11842    214.78309    221.10796    221.00135      217.702    217.69864     222.1507    223.10465    222.15326    217.64681    225.11581    224.56349    224.93624    229.15776    224.51049    224.79678    226.45074    241.34186    232.60689    232.65557    239.12466    243.25661    229.52775    238.97239    225.70111    239.45414    255.59368    242.76521    253.06618    259.93804    261.13063    251.74174    251.56796    272.31639    262.72584    269.56007     271.1001     277.4964    251.81133    297.04532    291.86278    280.56068    304.32088    291.70652    310.56277    316.68944    337.02748    355.72687    351.13766    335.38514    331.30086    319.51313    356.42025    417.74802    431.51845    395.44209    411.21425    412.02591    412.47107    542.53048    474.54814    437.92413    434.08066     569.1977    502.84374    678.33899    644.45241    609.03839    625.70827    666.52864    688.34873          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        2.00    265.51369    243.98508    234.49985    226.69944    222.30384    220.78755    215.60944    207.57403    202.14443    200.80857    194.42619    195.87346    188.82879    182.70139    183.38219    179.49092    173.38362    175.09465    171.21071    173.12488     165.3957    166.39668    170.81257    167.85351    171.85337    192.99017    237.26628     251.9389    241.22567    201.14588    170.95886     160.6035    154.87188    149.24338    147.49446    146.69351    150.27081    155.43416    151.87861    143.52649    144.12732    144.68103    143.03286    144.04765    143.18617    138.90267    136.58847    137.55264    138.85524    137.54915    136.22172    140.32518     137.7422    133.91479    132.29895    133.41698    136.62141    143.35145    142.36329    140.45998    139.21451    135.29775    135.05333    125.83463    130.77915    124.35189    127.81943    130.85309    132.71775    134.28649    149.13697    169.70722    181.33707    173.16643    156.96738    139.29096    141.23039    140.02941    142.17199    143.55323    147.00718    140.53255    139.10271    133.70284    128.03019    125.91356    127.25432    126.34841    127.02776    143.78859    162.80964    190.76914    206.93934    208.34746    202.54553    180.18192    172.42397    164.65574    164.06231    155.90274    144.08503    140.68684      137.081    130.82588    121.17679    124.85527    121.94933    119.46488    120.40108    120.71361    117.35203    120.98562      120.289     119.9737    114.07036    118.00076    114.00493    117.86413    117.98965     117.8907    117.79343    118.02449    117.10852    119.33234    127.62118    134.51981    138.02237    138.41766    133.45939     126.2947    124.17796    121.69787    116.37796     120.1253    121.15312    123.68633    128.45058    147.88439    164.00972    172.75445    174.77032    168.03337    155.40616    146.08946    138.30752    131.73198    131.60783     129.2561    130.67458    124.55621    126.02331    124.32726    116.66313    116.48766    117.08416    110.90114    117.87564    115.07436    118.03189    115.86768     119.3934    111.39689    113.24563    120.62001    122.01425    121.73927    126.40293    125.29581    122.61224    124.14322    122.00996    122.10811    122.08724    122.51138    126.95251    129.73305    132.88893     132.5373    135.44428     132.7217    133.64391    126.61987    128.42835    125.56987    118.60503    118.59566    119.89244    117.96249    117.99948    114.77327    116.90236    116.76878    120.79769    116.68852    121.58767    122.72367    122.88323    129.87926    126.75119    127.27485    126.41289    125.99077    127.07396    123.80288    126.91232    122.81595    124.87289    123.94248    128.66863    128.72278    131.55639    128.30784    128.65739    126.38459    124.01827    122.22344    121.61395     121.0763    118.99052     119.6732    119.77549    121.40268    122.66842    123.83998    123.67825    124.15415    122.79947    126.02087    127.30865    126.58714    128.53092    128.97311    131.38153    129.95386    133.10145    137.00047    139.23177     137.3939    140.91409    143.14833    139.41346    143.32801    143.02854    139.89953    136.22826    137.92628    129.80242    135.52615     136.6249     135.5497    133.76064    131.09917    133.71394    135.86399    137.59477    139.97112    138.83655    135.39524    132.38171    136.53797    142.99276    145.21353    142.86499    145.73001    148.80262    147.29203    147.29503    146.35792    147.30624    140.98688    141.91182    136.16652    137.73463    134.64125    132.26785     133.1555    134.76658    134.43309    131.46704    136.45835    137.37857    132.78841     138.5703     137.6209    134.54238    135.44527    134.57742    136.25859    129.98674    130.21844    132.02922     130.7969    136.87088     127.7975    130.81486    131.61683    132.91144    127.86713    130.01963    130.49478    128.80324    132.31839    127.16852    127.56019    125.10705    130.93491    133.30732    134.38009    129.97024    132.47445    131.81081    132.54004     131.6634    134.69049    132.91455     133.6082    132.75222    135.31163    133.73123    136.06196     132.1355    135.95476    132.92397    133.56299    133.21077    131.07884    135.53973    134.35922     138.0155    140.11102    138.55533    139.32048    138.40503    140.09636    143.14876     139.5232    139.58337    139.79278    141.08948    142.60342    145.82048    144.71399    144.64472    144.18876     150.8473    147.76144    144.79456    146.93304    147.21401    147.08498    151.43242    149.08874    155.60887    154.18461    151.14103    155.32626    162.26177    155.44696    160.05478    160.30579    158.95571    158.31799    162.26934    164.83271    167.50259    164.40911    166.29699    161.62488    160.23071    166.52763     170.7293    163.62583    165.34425    160.86775    172.88765    168.33078     164.6902    170.54462    168.43112    169.99925    175.50444    169.87461    174.26708    169.05711    176.32821    169.64585    177.57429     174.7889    179.74516    171.93826    171.17679    176.36589    178.51883    175.55344    178.11663    183.20021    194.18061    182.16938    189.81778    187.48539    191.03394    196.36205    197.09738     192.6401    199.25994    192.67019    202.51141    198.87868    203.48848    198.06987     205.0914    200.11065    197.53386    198.76564    205.83187    210.48189    210.70813    215.21413    212.52865    208.12556    212.31272    209.98813    221.84511    214.19068     224.2016    215.07454    220.06197      226.769    223.65631    220.79404    219.75326      216.796    220.36838    217.89938    227.09101    228.79601    228.29442    236.95669    225.34755     236.4751    253.54526     237.3563    253.21513    234.57772    226.04853    233.64674    254.69455     255.2075    252.42446    251.80786     252.6387      239.169    258.71582    278.64338    272.03484    286.19554    277.76451    267.65749    255.85788    298.13677    313.36864     298.7736    301.93305    298.78384    316.33897    318.92699    334.33503     320.8256    325.99075    325.73757    318.59199    338.41922     427.4366    397.18736    366.96368     412.2301    404.15772    375.15356    396.08428    492.07976     451.2744    438.84726    482.79732    492.41526    534.46108    594.91332    648.13438    700.44707    616.54551    677.90336    685.16894          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        2.50    265.41548    244.34492    233.00467     225.7329    222.71155    220.31718    220.77345    203.64189    199.79839    201.56117     191.9194    193.23202    185.06036    186.50279    183.49052    180.11772    175.53796    177.61164    168.89618     171.3429    168.99859    171.76413    165.80636    162.87529    174.48336    194.11012    237.77287    248.54412    240.08327    201.74145    169.74736    161.85078    159.96459    150.58261    146.51466    144.00982    152.04772    151.21784    147.06007    141.19123    144.31192    144.77206    142.51359    143.23913     142.6429    139.30741    139.52879    140.97083    141.39643    138.28034    139.18551    137.80821    135.76333    135.41972    135.39166     136.4468    138.58988     140.6892    145.66384    139.54487    135.88979    133.50666    129.22467    133.40008    126.73026    128.23093    126.86506     130.5576    134.67227    138.01787    148.64178    170.72901    179.66431    174.13389    159.41659    145.93348    140.60465    135.59532    137.17124    140.08993    135.48498     140.2708    144.46962    131.43404    129.86921    127.12452    129.33031    126.28844    131.81692    140.52444    158.88242    184.33526    202.56274    203.42733    184.25048    172.17076    158.51716    152.11072     158.6564    153.51169    147.70448    142.44428    138.16416     132.4292    125.22453    123.87467    121.08438    123.29453    117.46406     125.1659     120.2023    121.11179    117.97642    116.52788    119.42984    118.50612    114.38502    112.65623    119.18714    117.72455    115.85418    121.33749    120.98234    121.36991    130.00049    134.68641    140.76412    139.25167    132.00713    127.21256    121.07779     116.1731    117.22442    116.42795    120.35655    119.43751    129.37791     149.4585    166.34896    172.23192    177.33131    168.34221    152.62023    144.43169    136.84425    131.80739    137.74562    133.33729    127.62799    127.71542    128.39428    121.35289     122.3034    119.02335    116.46207    115.40142    115.86259    117.22364    113.01737    119.68843    111.67051    110.98763    115.64289    122.92749    122.66913    126.09251    125.73275    128.35378    123.44819    122.09026    122.71554    124.28186    121.46795    123.00104    129.93323    130.57846    134.96196    135.41269     132.0227    134.50651    127.67339    127.04927      124.097    123.28894    121.35602    120.22018     116.8147    120.35174    118.57921    116.66669    121.62884    119.31501    118.84376    118.89101    119.64338    119.64526    125.50148    125.61839     128.9185    127.25859    125.67849    126.49493    124.68852    127.55254    125.01272    123.85283    123.82474    126.82226    130.20797    126.47939    127.40372    125.85544    126.68325    128.36423    126.23596     123.8873    125.25407     125.7307    119.82224    125.28635    117.87256     124.7949    123.39502    124.05415    122.91188    122.94124    128.63377    132.10812    128.02282    132.05843    128.60224    133.65431    132.12246    128.66583    132.85544    138.33711    139.12717    140.36083    138.59644    137.19509    139.23059    140.71718    135.87913     136.5662    136.46538    134.71308    134.76779     137.8764     136.0674     137.4406    136.45189    135.18908    129.56551    135.98682     136.2482    134.54467    137.41355    139.84674    133.23353    137.70558    143.41097    138.28052    142.81175    145.54147    144.64196    148.28007    149.52261    150.57538    143.23404    142.91564    137.30194    132.53149     137.2881    136.18595    133.86619    132.23475    130.54561    133.27845    131.06346    131.90467     135.8064    132.53056    134.68622    132.93062     134.0417    133.09212     135.1542    135.43799    131.49101    134.34245    130.01979    129.16912     132.7579    131.70906    130.76198    130.25684    127.17344    131.80595    128.66174    137.13268    129.68541     131.7287    130.57074    126.03584     126.6838    129.61837    132.24988    132.76059    130.40394    133.60967    131.16334    139.06125     135.0477    135.14882    133.92536    136.50682     130.3561    133.51758     135.8314    134.50269    137.22083    132.01688    132.02611    133.61069      133.417    135.39094    133.51528    130.07335     136.3824    134.63188    133.38209    134.65328    142.90149    143.03801    139.24365    137.26664    134.85266    143.26982    149.25593    144.27295     145.7698    145.38898    144.43808     151.9822     149.2572    142.17637    146.52792    147.00452    144.04535    146.07962    150.08168    151.80321    148.44042    153.19933    154.29002    154.53814    159.46477    158.96117     160.7053    153.47594    156.72891    163.40799     155.6283    166.93513    164.46391    167.76995    164.05675    158.75538    161.21309    164.33865    167.41778    162.36974    168.56978    169.27358    161.41575    165.03467    169.57318    170.41344    178.00599    174.09807    168.88464    169.08855    167.45741    167.17594    173.20838    180.38481    177.11087     171.0372    183.42662    181.81062    181.27429    177.16888    175.17608    184.93336     179.0991    182.75481    184.97337    189.28862     184.8988    188.91257    188.77042    188.67982    197.83409    188.34083    192.73471    188.08904    203.27916    186.79167    203.79948    192.20626    198.01268    208.49086    197.63946    204.35482    210.80669    207.40428    212.78125    210.41424    205.67643    210.73167     209.5969    224.06288    217.79024    216.02891    212.85982    211.40601    214.70554     223.5874    225.46098    212.09531    221.54353    214.39995    218.21665    230.21834    229.32181    226.26693    236.43668    222.41456     219.1964    229.70544    235.46089    262.83119    242.39379    239.35138    249.21363    226.47725    255.04597    257.48372    249.13637     257.0738    258.97566    264.00253    267.78513    255.17636    262.84616    277.78784    265.79526    267.06911    272.69595    293.09227    280.61161    281.96683    298.26825    316.89395    290.06327    333.00839    343.71974     345.3195    339.18571    317.95814    334.08598    346.21618    386.27083       412.44    404.63086    382.06736    359.23155    375.99337    408.23297    484.08394    442.13287     484.4628    504.12341    450.22033     456.8694    542.16664    652.42168    645.36018    631.11208    573.90439    641.61067          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        3.00    263.53412    247.07286    235.64582    225.29561    223.23889    219.10044    226.65476    199.41748    197.77913    202.45921    194.17132    189.45503    190.09642    184.45567    184.48216    175.57932    176.79389    179.19221    166.22302     171.3978    170.62632    171.66959    168.83823    165.79137    167.90933     197.9264    233.82886    252.00243    237.64352    198.82019     171.3076    163.56829    161.65077    153.11416     147.9994    145.74162    153.11898     149.8252    143.00788    140.21797    140.41791    141.76693    146.12552    143.21256    146.10151    145.31467    137.93137    141.55072     137.3292    138.55656    139.81249    136.06855    136.81158    135.10187    135.53606    134.37195    144.90869    142.00539    145.39102    139.47052    139.37291    139.26508    129.64874    135.97401     127.8539    133.18672    133.85836    134.55621    129.16074    135.59642    150.27444    169.56497    176.29052    174.52569    152.85306    147.76913    137.41379    136.98344    134.88287    134.94434    139.43358     141.5214     144.5084    137.64738    132.61274     124.9291    130.43129     124.9885    129.55954    141.10764     161.7264    183.23034    193.08431    187.80399    172.82701    157.56562    148.79288    148.05868    157.87073    154.73183    148.89734    142.16342    134.89292    129.22885    125.61301    121.35153    121.57635    120.11704    120.97061    122.46659    116.60456    117.07032    121.94112    116.84501    113.07791     118.2625    120.03721    115.15766     117.3418    114.83174    116.79077    114.63629    117.44531    122.79732    130.17709    133.26287     133.3339    141.06202    136.14251    132.23897    117.70255    119.49763    118.93449    118.79464    119.61669    124.79859    129.68551    146.99404    162.67459    176.21779    179.67498    169.75538    156.81726    145.04225     136.4215    137.34318    131.06148    131.62555    132.12748    126.01669    118.44525    121.52664    120.78738    118.73209    118.87886    114.91216    117.57768    116.43953    115.09045    115.38395     115.4725    116.95135    116.81515    120.38167    121.79506    127.17153    128.12894     126.9484    123.55862    122.12724     125.6391    125.50251    126.59249    125.58981    124.61383    129.36021    136.45921    134.04743    132.19313     130.9559    129.48427    121.68729    126.35819    122.39152    121.42573    122.71777    114.14284    119.30731    117.17898    117.94464    117.53119    118.11098    119.72529    120.59791    119.74453    126.15517    124.27255    126.59726    130.17681    130.00035    125.00251    126.86629    125.00785    127.35336    123.45387    122.83556      122.379    123.55945    128.00642     129.3551     132.1097    129.76156     128.1994    121.31787    127.80866    124.21987    121.51022    122.55743    122.19562    121.93212     119.4806    124.89425    124.62825    126.88188    128.73606    126.90618    124.93803    128.22514    128.55979    128.54536    133.14851    131.66985    128.45692    130.65155    134.23682    134.08442    139.86742    143.49807    140.58236    136.13455    138.56313    144.54838    138.74736    140.92266    135.78745    137.31242    131.96428    137.84401    134.49901    132.88403    135.71678    134.38357    133.14342    134.17175    133.52971    137.61534    136.48488    133.58588    131.50067    133.91585      141.496    140.30172    145.84843    147.72862    143.15436    141.79244    150.00423     152.3845    145.81254    141.14483    143.65295    139.36932      134.891    137.61515    138.94334    129.93699    132.66485    134.58515    133.92286    133.19093    134.07344    139.34622    138.69729    138.60348    135.19322    137.74988    135.66269    135.47286    130.90564    132.99116    130.71837     128.9179     133.1455     134.0701    131.52732    133.93207    128.29175      126.667    129.37643    130.53952    129.11518     136.2826    132.33452     128.4209    128.59046     128.2373    129.30409    129.93871    131.36671    128.74648    132.05013    128.76013    135.60186    134.43871    137.78041    133.81247    134.41795    134.72318    131.96677    132.11196    132.63496    131.48644    128.16214    135.08236     135.1446    132.77685    132.36238    130.85429    135.87766    131.20198     134.9574    137.11859    138.30696     139.7071    139.24949    138.64926     140.5567    144.87499     141.3583    148.57656    146.73936    144.95102     139.6876    146.54234    153.77223    143.43915    146.71667    142.12026    150.55681     149.5989    147.17863    152.49825     153.5035    152.82459    153.46268    151.91379     156.7256    157.27307    162.27976    160.67567    162.08842    158.71988    159.08038    162.11075       169.26    162.18738    157.94177    161.32121    159.66566    164.32381    167.43768    167.78023    163.32902    163.81989    169.05172    166.82176    171.42226    170.42314    168.45888    168.30794    165.39463     164.1079    177.40669    168.44505    174.32882    177.35782    172.62747    177.28873    182.57232    172.59062    174.23263    179.00698    178.01145    185.25762    176.93331    181.27686    182.71121    180.30448    189.89763     184.7462    178.54266      202.487    197.53292    188.12955    194.90528    190.93152    193.00869    192.83235    201.26923    202.75633    198.92579    202.69016    206.30091    203.10195    210.55098     207.2956    205.71061    211.81881    205.85843    210.98474     210.4315    209.63159    215.84734     220.9015    220.13434     207.5414    209.97558    227.47529    214.17312    221.28241    223.73143    215.55454    221.33095    232.38913    233.16792    220.27685    231.07209    230.47862    232.58276    225.15184    252.34883    241.11217    244.75631    239.15868    224.76287    232.52637    255.97493     251.1413    249.44542    260.57118    245.31698    233.88283    257.08633    245.57091    278.11829    272.42724    257.43212    257.13357    267.76761    278.95835    309.87101    279.52492    294.48987    286.38178    296.12424     296.8839    313.73526    358.53397    323.70062     351.5567    309.02416     350.1441    359.76947    382.11508    389.88994    406.73581    403.18459    376.43533    400.17207    466.42219    419.15505    498.57924    414.65492    406.99689    556.49605    673.43272     655.2718     653.0533     701.2862    610.89084    531.70773          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        3.50    260.84784    249.95365    238.19747    225.00245    223.44162    217.77756    221.30542    202.62864    197.31716    203.40901    200.36609    185.09688    194.90582    183.70948    186.93835    174.43526    173.87682    176.17443    174.08479    175.32872    169.48076    167.85213    172.22338    167.21817    168.87308    198.00046    236.27385    256.48712    239.92608    195.42616    168.10551    161.10009    162.33617    149.45745    152.42168    150.42115    149.73342    147.29411    149.27312    140.52775    140.12621    141.22179    144.62773    142.22356    142.06787    144.89455     137.0042    138.98829    135.65534    142.44541    137.55636    138.68531    136.42443    133.79102    137.45131    134.77495    139.32969    142.43226    145.20071    142.37573     143.5772    142.92904     134.4875    135.71856    133.52545    130.89102     127.5984    128.59899      127.791    136.06773    149.22248    168.31947    179.68997    171.24067    160.53136    144.97223    139.65017    142.02002    137.39399    135.90296    142.02739    143.69873    145.67608    138.85846    130.81249    131.03477    129.78979    126.14368    129.69711    140.99348    160.53993    180.79872    186.84454    185.17133    170.24451    153.63832    143.81646    147.32339     159.4797    148.28708    148.97831    138.33851    134.28394    131.04803    119.19175    124.22246    120.67931    121.17674    120.42226    123.94271    120.76501     118.2251    120.05939    118.97123    116.59223    116.57579    114.68503    113.96016    120.56421    117.55497    116.51041    115.95581    118.56165    118.44488    125.61759    137.18636    139.89697    137.37122    139.13962    130.18703    121.42962    121.62023    118.02108    116.64895    120.75327    122.43438    130.40631    144.35408    162.42088    170.37835    173.64378     169.3567    154.71714    142.94616    134.97461    131.98403    128.56389    130.22881    128.08029    124.11078    126.51525    118.54362    117.64953    117.41993    113.87536    113.92359    113.88976    118.86819    115.29226    113.84582    117.96731    115.81959    115.56848    124.14902    119.89877    122.30248    125.46244    128.36178     122.3578    124.48486    123.87953    128.77405    125.55728    126.78043    126.78607    128.01567    132.58358    134.78554    132.05025    130.42892    131.77441    124.71411    126.90001    126.61656    122.51372    123.08325    122.06064    119.06742    116.13838    120.88496    119.21112    118.87892    121.26962    118.76449    121.25443    121.96656    125.51184     127.3507      126.327    128.47849    130.32786    128.19307    130.85011    128.33371    123.40426    121.90069    124.25244    125.53713    128.80674     128.7021    127.55207    125.84878    128.65442    128.20173    123.64567    122.94537    121.50429     120.9453    121.14151    121.44172    119.95029    121.92326    122.52337    121.84933    122.31638    125.47483    124.68746     127.5015    127.37478    127.86966    129.98965    133.30422    131.42382    129.38032    135.92472    135.08219    139.51315     146.2477    145.69118    140.15399    145.48722    140.64028    139.47787     142.1146    138.21693    135.43717    131.43822    132.70287    132.25096    133.63495    133.86669    133.02004    129.34858    132.62709    137.07573    137.01209    133.31307    138.93545    135.45227    137.35038     141.3507    143.66706    146.37807    146.40415    147.93728     143.9672    145.81135    146.51558    146.28137    143.62793    138.80798    134.43244    136.07647    134.28554    131.26343    133.46084    133.76713    133.25018    138.40703    137.23039    138.51373    137.50285    140.67356    136.03988    134.67967    133.61952    135.52078    135.07677    131.02924    130.96777     132.2626    132.43596    131.99426    128.74469    131.26538    126.87894    128.97569    130.92738     129.2608    130.93612    129.72224    131.16013    132.66423    130.45674    126.71915    127.92452    127.63579    127.07598    130.01436    130.13439    129.03173     133.7296    136.59306    137.39723    136.66533    131.09643    133.51471    133.93782    131.82529    132.07787    136.83259    132.95606    132.92053     134.6324    129.80322    131.76272    135.02946    133.39018    132.47046    130.09028    139.46164    137.70034    138.31069     139.1221    139.99097    137.97337    140.49381    145.19503    142.66185    143.04965    144.90055    146.08505    143.13223    143.61092    151.69578    145.70944    145.03269    147.59816    140.98003    148.10404    144.59858    155.29349    154.39206    154.63193    153.89894    160.13307    162.32314    159.49436    160.11628    164.43872    157.16881     157.8941     162.2585    160.25677    162.26716    165.37337    165.31306    165.54344    161.15235    158.69542    159.75178    162.75792    165.56348     165.9017    160.58996    165.47938    170.07466    172.74123    169.58259    164.70845    170.28164    170.74114    180.39619    170.31679    171.46033    174.53122    170.43991    172.23131    172.55634    167.26185    182.54719    183.50342    175.88952     179.7325    177.16097    174.47518    187.18269    184.31352    182.51258    185.01238    180.64757    198.46662    188.53573    196.61257    190.12831    194.40435    188.74865     196.9192    197.63357     193.0997    203.68822     196.8523    205.35769    206.18448    207.36956    210.69792    209.10707     213.2567    213.52113    206.79159    208.91905    210.32444    216.83311    215.56385    206.41769    211.13716    218.96755    225.91808    227.57694    211.15077    214.86389    214.45301    214.05966    225.34632    222.16777    223.32946    222.87309    222.82606    230.29727    223.10874    229.15903    245.42875      242.822    236.74394    238.44833    225.48468    262.98307    244.78459    235.42076     245.3241    254.98863    254.49521    264.90817    261.42497    265.79977    257.13572    269.09589    274.50079    260.15373    297.93871    292.91942    293.11636    293.32689    285.03263    295.51445    291.16418    326.73486    311.00868    328.98258     321.0824    331.14204    338.88141    406.22261    387.59615    374.31838    394.66358    347.76058    355.92104    443.54129    437.26546    454.10997    423.53746    469.72767    479.58132    520.29047    588.25461    528.11099    617.68297    558.94731    594.34752    616.93161          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        4.00     258.2071    251.74311    238.81128    222.77168    223.32142    216.14456     211.0492    206.31202     197.2045    202.43378     200.0239    184.60584    186.99042    186.46792    190.57605    174.91985    173.59587    175.76434    177.23924    176.01638    169.60858     164.5871    170.57707     165.9013    172.34044    196.70251    235.01754    260.14082    237.20074     198.2513    160.89732    155.50403    159.67651     149.7716    156.92338    150.61926    149.06258    147.54244    153.02464    141.65636    142.22902    142.85278    142.41919    144.24411    143.85164    141.14011      139.298    138.39642    139.92621    138.11766    140.44084    139.48774    135.30049    135.31715    135.71455    136.47215    136.53157    145.58284    151.54792    141.82475    134.05426     132.8281    133.26925    132.16855    131.49054    126.12244    127.80705     126.8038    131.88223    133.06924    148.74801    170.84547    176.66198     174.8472    160.85469    146.34269    140.33561    139.15987    133.62106    138.29616    143.50664    147.65133    148.04447    143.30744    133.12657     127.0812    125.18685     128.0725    128.97354    141.98753    159.91526     178.7307    191.60596    184.22566    170.78433    156.00548     148.2664     143.2014    146.77757     149.8558    141.91128    140.05524    135.40239    133.43807    122.30261    120.87051    122.43387    118.31762     121.8973    122.75415    117.67519    121.76468    120.67192    119.10312    118.44263    114.99893    115.75181    113.39867    117.60209    115.77092    119.39052    115.49114    119.58506    120.52973    129.28363    133.48074    139.66738    137.64359    132.91686    127.31018    122.63073    120.98795    119.62298     116.4291    120.76009     125.7777    125.76939    147.95322    157.13108    171.43467    173.26687    169.56625    154.53863    143.12503    137.62414    131.46318    130.15453    132.55646    129.94744    125.96997    121.86624    120.86703    119.80934    118.03516    118.56952    118.79932     114.9031    115.78561    117.69189    114.80632    116.91383    118.69685    117.51253    121.43365    122.29022    123.72313    126.56239    123.26642    122.53919    124.10179    126.77522    126.61386     123.7674    127.49139    129.69538    128.29674    135.11334    133.88794    136.01566    133.17756    133.03272    128.59868    123.44475    125.18274      121.528    121.57555    118.83599    118.81799    116.05155    122.16242    117.83052    117.59509    118.18201    121.84466    118.13815    123.67001    126.54806    124.26904    128.84417    132.26293    130.71721    125.03126    128.75305    127.15779    124.39313    123.73484    123.51787    124.48929    127.74217    128.07927    129.83357    126.71638    127.22707    126.50834    124.37541    123.05463    123.04732    123.38188    118.51617    121.92257    121.89404     123.4532    122.57022    122.56034     124.6941    123.19319    124.57627    126.78526    129.99782    129.75613    131.43621    132.69459    131.25767    130.12657     135.9942    141.63722    140.38448    144.48223    145.52811    143.69294    144.07867    146.32568    141.38963    141.72924    139.53821    135.41456    130.99858    134.20039    132.30299     133.1842    132.61332    135.93987     128.3647    137.29167    133.96319    133.84063    134.02918    138.37445    134.69825    138.78655    139.45996    142.14572    142.97455    144.64248    146.89322    147.71763    151.98567    149.79997    147.39915    144.91116    144.04454    136.95481    136.59727    134.23835    134.47819    131.79835    131.93362    132.48342    131.14164    135.77728    138.96674    135.62337    136.62262    137.18578    135.45813    135.56889    136.11693    133.66909    132.27413     134.6342    133.71876    130.55224    132.11515    128.61654    134.30445    130.66213    129.04839    128.20987    129.76832    128.46775    130.64577    130.70754    129.19909    131.71919    125.54476    130.31685    128.06075    131.00115    131.67019    127.74173    129.92234    138.31781    136.16269    135.11604    132.35035    134.59014    132.77237    132.86765    137.01068    131.99479    132.76356    129.22437    131.57173    134.07141    132.58893    133.90713     134.4331    130.56897    131.02721    132.57171    134.34388    139.37822    138.85437    140.74036     140.0426     144.3681    142.34925     139.8933    144.45084     142.9002    142.51333    146.56973    144.88311    150.00442    144.85195    141.73807    149.45611    144.06513    151.13836    151.84569    150.59226      154.241    155.27699    154.73637    148.44569    152.29139    163.87319    160.56275    158.11433    163.06546    163.98656    155.56639    165.26929    162.23209    165.56187    159.84929    160.19914    158.69088    163.79632    169.84362    165.02405     170.4972    162.41904    164.14569    161.91642    170.87541    177.10499    168.19816    175.49049    165.59529    170.45759    169.02664    172.98662    174.81815    169.36016    177.78772    176.10831    168.88683    181.94578    176.44649    177.94175     180.9855     171.3805    179.56484     174.6535    174.23018    192.31891    181.70152    186.35262    193.46326    187.07215     193.3301    191.35731    192.22628    194.65411     193.9817    188.48271    190.35534    204.38636    195.69442    201.50758    195.92713    206.46269    196.77889    209.35874    203.92944    197.62228     207.0359    211.02728     202.9748     211.8201    208.39677    219.96878    212.20914    210.78002      215.138     212.7952    213.91669    216.64709    223.70513    222.44849    227.34347    219.63754    225.05753    227.34866      226.698    224.38871    224.26874    232.76806    229.38656    240.28014    238.27348    241.23699    238.22753    231.21749    235.93859    237.79368    252.80785    249.28453    243.21379    248.63351    251.89439     261.6393    262.45638    265.81663    253.00112    276.70574    257.07048    264.25196    297.31459    276.86885    294.94426    293.54215    302.12872    291.14662    320.40987    315.32465    366.26223    312.48613    334.10683     321.2855    335.40994    364.42839    382.30105    355.06012    367.51321    407.44157    382.08666    405.48048    465.97623    389.94584    467.92457    453.40606    419.87295    567.10936    623.80883    709.27645    616.59474    619.46379    599.74623    508.98584          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        4.50    259.85163    249.79312    238.19379     220.2271    223.38321    214.55427    208.52423     208.0335    199.77485    198.22811     196.0368    184.74869    181.16725     188.4858    185.38017    177.09689    183.01822    178.06104    178.92775    175.16683     168.3079    165.39606    167.45769    165.54803    173.19528    194.29054    232.67316    255.86904    240.17906     199.5609    170.35147    157.75369    153.67215    150.42617    153.99693    151.85708    153.56237    142.78195    149.97103    145.13193    141.55925    144.57325    144.06104     141.7561     146.9864    138.24646    135.33666    136.57936    136.60175    139.04923     142.6993    133.75787    136.07666    131.46031    130.45537    134.50471    138.87282    144.08115    146.31906    139.01563    137.53071    136.67236    133.36737    132.49771    129.43928    127.02193    129.87605    129.80461    130.77049    134.77994    150.78558    172.46504    180.08417    171.70049    157.15676    143.77424    141.23694    141.09975     136.4358    137.94562    142.07749    148.37142    149.63216    143.64927     133.2103    128.17549    126.71702    127.45044    126.77579    142.77393    163.71613    179.56361    188.76615    184.69595    173.57944    161.34772    151.87196    142.41962    146.45642    143.33866    142.12532    137.52135    138.00388    129.29522    125.07784    121.02659    116.96027    121.11981    118.38144    121.77454    119.83146    119.24284    118.69525    118.74549    117.78229    116.11682    116.78699    114.60794    117.56791    117.83935    118.67697    117.50081    117.06336    122.17617    128.12255    138.74245    138.44076    138.42551    134.91412    128.06271    122.23947    119.80109    120.43624    119.35112    119.11325    124.17892    129.00538     146.9839    161.60755    169.00661    173.96251    166.99584    153.61949    140.83329    137.19637    133.66094    131.13904    127.49292    130.37925      125.288    122.72902    121.16985     115.7399    117.61944    117.96394    114.25338    113.94869    115.74823    118.11345    117.86194     117.7339    118.83412    115.66491    122.25638    120.77472    125.03777    125.76265    126.07703     127.3254     126.3171    129.50407    133.89264    127.35205    130.53454    129.54932    130.09164    133.08436    135.73836    135.87223    135.04304    132.57333    128.68981     126.0704    123.34201    122.81707    119.44145    121.14215    120.29895    115.18864    118.19929    118.61817    119.71455    120.35079    119.00537    117.04842    121.19874     126.7372    127.54929    128.96679    129.36707    130.11687    128.82462    131.01283    126.44735    123.13706    125.60784     123.7479    123.44962    127.47461    127.62704    131.11985    127.71551    128.20784    126.19822    123.49673    124.33094    123.73343     122.9911    122.25741    120.71007     118.8144    123.51925    125.50881    123.98525    124.09903    125.66667    126.35978     124.8821    127.67992    132.60314    129.63545     133.6782    129.67288    130.38954    134.16709    137.96521    140.02537    141.38021    142.23942    140.72855    142.59438    141.79962    140.93193    139.55319    137.13479    134.51354    131.95443    132.84963    131.94047     133.8396    133.14173    131.86669    131.29854     134.1273    133.68384    136.02348    134.80235    136.35618    134.73288    137.80384    140.06707    141.86083    142.85522    148.65214    146.73306    147.44051    149.40295    148.37248     144.8337    141.68492    141.54757    136.12465    135.86885    134.34103    133.13143    130.49286    132.25277     133.0411    130.39347    137.79266    137.13025    135.49011     135.8588    136.96721    132.92907    137.12646    137.46185    135.65234    132.31317    133.30824    131.34114    131.17773    133.45541    131.14216    132.42519    129.08522    132.53064    130.23728    128.47115     130.3946    125.69744    128.99291    131.34176    131.75885    126.56286    133.27294    130.89739    131.93769    130.94685     132.0652    130.26028    133.47134    132.00136    134.95973    136.89914    135.59885    133.72712    134.52813    133.04933    130.96166    134.52048    127.76926    131.30635    131.11349    128.90052    136.04404     131.1476     132.4321    133.62026    133.60328    136.20105    137.34208     137.3719    139.84795    135.81159     139.7787    140.73903    148.24499    141.86269    142.81543    144.79585    144.65573    145.24254    152.79237    149.84262    147.29243    144.84645    149.05237    147.21812    148.35344    152.36076      158.952    155.50536    158.02974    159.66964    150.10803    151.14636    163.70254    163.57006    163.90407    157.27735    160.95159    161.14716    164.27592    162.80839    156.16604    160.98775    161.48822    157.61174    158.98659    162.78163    163.94566     166.9368    160.79976     161.5126    166.96112     168.5239    170.68535    168.40377    171.77232    169.99988    171.14387    175.49238    175.02723    176.83212    177.35503    180.28402    171.66334    184.47296    166.43694     175.1247    177.12576    178.32578    172.33651    173.92621    181.49587    177.96295    188.15441    182.30439    186.48345    185.07352    189.68602    186.63603    191.96008    191.66701    185.60142    196.44771    197.11582    199.58353    202.84193    197.82591    193.34718    200.75414    202.58997    211.18632     213.2715    210.52628    214.01124    205.87835    207.29022    215.99536    213.59623    213.69556    209.23776    214.98192    213.14861    210.31018    222.30347    226.39815    223.11828    222.13997    212.95746    218.52747    234.65848    216.77323    223.82784      238.296    221.32463    227.62476    235.77785    243.78015    240.30487    228.27421    234.16959    225.83534    230.26066    254.95937    245.21347    245.13781    237.82655    247.87657    241.66093    254.21304    258.61117    269.26027    277.35795    264.03359    274.22494    276.69655    277.65845    269.81842    296.61306    298.92556    295.48879    271.81844    307.68513    315.27737    331.25809    305.42513    306.22734    323.81345    302.58753    383.75416    367.96614     386.4425    387.42345    401.26043    353.54437    442.12863    471.45171     473.3481     417.8991    414.12076    437.72104    545.35355    528.19889    665.55094      631.906    539.32504    629.78101    660.45849          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        5.00    262.34757    248.05411    237.82331    219.79551    227.57671    214.26742    214.15297    209.72904     202.0786    196.06952    194.49514    190.71067    180.25066    189.83152    179.38804    177.02938    182.54594    178.53772    183.98777    171.76078     167.2215    164.79028    168.31467     167.6965    172.92403    192.99522    231.08075     251.1732    239.45435    197.31854    168.52373    153.23681    154.63723    149.20014    153.01119    151.42375    155.09891    151.94215    148.31201    145.60861    143.36575    144.00659    142.38721    140.54153    143.62586    139.48086    135.07888    133.66479    134.73017    137.49677     140.5783    133.45878    133.18823    134.36555    130.70187    138.80822    136.85561    143.96239     145.5908    138.02162    137.79085    137.67419    132.88078    131.60621    132.69183     131.3276    127.28023    125.25586    132.95927    137.26013    154.41819    172.59231    175.74451    176.19589    157.14725    146.43639    140.56434    140.30709    137.31431    137.69145    144.19823    147.55894    147.84937    143.55862     132.6278    129.04498    125.33809      123.849    131.09818    142.88956     163.5248    181.54966    190.03444    183.22392    176.06879    161.58179    150.19148    143.46257    144.59353    140.92423    139.69102    135.67515      133.079    129.84284    122.54931    121.47917    119.23241    121.23767    122.82192    120.04949    121.87507    118.81405    118.63133    118.31887    117.25811    116.11921    121.66741    116.17541    118.09851    114.82125    117.01481    118.60162     118.2631    124.42983     129.0908    135.69572    141.35949    138.66428    133.97465    128.74428    120.80191    120.68672    117.41862     118.8022    123.34333    122.19187    131.48113     145.4833    159.93305    172.71829    172.05805    165.36107    155.54413    141.80377     135.6927    129.69687    129.49077    130.33879    126.38662      122.259    125.44518    121.02411    118.99248     116.5055    115.13188     116.5175     114.8345    118.80117    117.36747     118.0633    116.62231     114.7063    117.43341    121.03586    124.60583    125.23335    123.13273    124.65584    123.36841    121.35686    127.03981    127.15108    128.74097    127.98666    129.08123    129.99193    135.57082    131.94683    137.13921    134.14322    128.90007    130.48299    124.45636    126.00372    120.37373    117.11032    121.63803    120.40555    117.12653    120.56979    119.28658    120.40003    118.12862    118.74381    118.55972    122.06185    125.67941    126.37634    132.01225    132.99589    130.52901    130.35473    123.64916    127.22541    123.99665    125.28936    125.49022    126.03917    128.46882    127.90694    130.79119     128.7431    126.48217    126.61799    119.69117    119.36328    118.80554    120.55415    122.30723    121.92356    119.37174    123.77301    125.62116    124.95007    126.95234    125.27264    127.15644    125.55953    129.13964    130.54085    131.39776    133.12587    128.74709    128.94114    132.58284    135.61186    137.60237    140.82153    139.85733    139.01678    139.97593    140.61421    139.78829    141.77172    134.86586    135.04644    129.62733    135.98106    133.68524    130.10667    135.38714    134.82743    129.31923    134.45833    133.31815    134.23715    135.65755    135.60946    133.45596     134.6142     139.4501    143.70923    144.38085    146.37111    147.53838    146.38875    150.74229    147.71117    145.64299    144.46786     143.2019    135.67109    135.70472    137.64792    135.64978    134.78669    133.66988    134.47404    135.34443    136.85226    138.32589    135.40148    138.25689    138.53448    136.96163    133.66728    137.27705    134.91634    135.21385    134.30771    135.78935     130.3759    134.38358    131.39861    134.68676    132.82765    132.69414    131.56193    124.76557    130.07736    131.36198    129.21346    130.75738    129.79445     129.2705    132.39448    130.24921     131.7842    133.18191    130.59836    131.16833    131.96926    132.39748    135.69494    134.00841    138.02758    131.57297    133.33581    133.20885    133.00882    130.42324    127.74869    128.03192    133.21061    132.67373    128.21738     135.5486    132.63518    133.41653    133.59569    135.17704    137.12597    139.89591    136.85178    141.29045    139.07422    138.45498    143.01995    143.19076    144.37302    142.87134    143.83164    142.75277    144.78298    147.18619    144.28645    144.48334    146.87225    149.02016    148.27409    152.41747    153.15263    155.28402    154.57596    158.20608    155.80555    156.37073    161.18435    162.25604    162.23748     160.6969    165.22519     158.7957    163.87091    163.24202    163.97983     158.0829     165.5034    160.93126    169.12555    162.23563    171.24759    168.93106     166.7916     162.7115    166.78076      168.494    167.72387    173.94618    172.43197    164.97589    161.45444    173.84096     177.4484    168.84808    167.00273    173.17001    170.40432     177.0285    174.76618    180.31409    186.87763    175.36809    180.35802    174.32071    191.40396    184.52886    179.31096    187.39305     189.3082    188.92434    194.27847    194.40447    197.88684    200.42256     186.4022     190.6251    199.46507    191.63538    191.25688    192.09311    201.63886    201.06264    195.03655     208.6272    209.25772    200.74185    205.64084    202.34821    213.72053    203.43886    216.67497    204.66764    210.13149    209.39271    217.26631    216.54974    221.48192    215.11185    214.40011    228.39279     218.9484    212.54916     229.4797     229.4037    232.93484    224.02743     230.8938    228.05196      229.209      237.146    237.81576    232.34967    230.89262    226.02274    228.25983     244.2782    250.80847    259.86443    246.63826    250.05787    236.21427    239.16213    268.16553    256.63122    257.65083    263.98327    260.73711    281.87768    287.01007    301.08337    291.58835    307.47607    301.66713      290.147    327.45485    324.38658     342.5608    324.80245    337.87932    321.68932    351.92667    379.90164    387.97503     347.4386    389.73223    369.00259    384.46768    433.79216    490.82834    433.21383    482.98185    493.63165    420.93828    589.22962    567.71925    686.90239    496.82432    616.18399    624.78479    646.00907          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        5.50    264.79138      246.509    233.87916    222.27111    232.94862    214.22442    217.52072    206.70537    199.68352     197.3569    194.04993    195.59082    182.75447    187.24854    179.97667     173.0282    177.84449    175.62189     182.0935    176.22808    168.33805    162.42235    170.50477    166.95414    173.08906    199.19715    232.07961    252.49614    238.59558    198.04457    166.37424    157.84235    155.76638    151.07107    153.11535    149.45513    149.24377    150.54613    146.75702     142.4459    144.51024    141.59262    146.82277    142.48779    140.49649    140.28396       136.75     135.4723    139.90341     138.9782    139.66706     138.8597    140.03533     134.0261    137.66977    133.55686    138.38545    141.58863    143.79809    142.70739    137.35551    136.10814    132.13534    133.54458    131.14888    130.47668     124.3065    129.76158    130.51579    135.31897    149.59553     168.8791    176.80182    172.54429    160.30989     145.7163    141.31197    142.29985    143.28334    141.79617    147.88038    145.78996    147.74825    141.94016     136.9011    131.20878    129.04767    127.46781    130.07525    143.00636    162.58348    178.91341    188.26992     184.3576    176.40739    163.90441    153.83754    142.35134    141.58334    139.68168    133.98343    133.42762    132.50934    132.61495    124.86419    118.66159    118.03935    120.68962    122.56087    121.11662    123.44986    115.61158    115.94417    120.36774    118.59035    114.75259    116.43904    117.96477    120.20548    115.92478    115.85907    116.81683    117.21043    123.33605    122.22039    136.10824    141.81973    141.73574    134.46725    126.58885    122.72482    118.97748    122.59986    115.58982    121.52379    123.71989    133.17633    143.25178    164.10008    172.59297    175.07533    165.60615    154.41503    143.40486    133.79087    135.66752    134.68586    133.30849    128.10104    126.54692    122.54553    124.61404    118.62583    116.31323    115.23054    118.94052    110.09685    115.55429    118.64594    116.76592    117.32787    119.59112    117.20864    118.11457    122.57367    124.91933    128.81046    122.22875    125.80724    126.16342    127.20205    123.59219    126.98992    131.32133     128.3565    133.24931    137.83324    135.85229    136.83376    132.37144    130.45551    129.71832    121.04296    125.66328    125.53869    119.88904    120.09116    115.73207    118.09934    116.77126    116.63779    122.23211    119.19388     122.6326    119.89278    125.33939    129.00574    128.33744    126.28549     132.6699    132.18562    129.17226     128.3852    124.93067    123.70795      123.651    124.55803    127.21815    127.11559    130.41014    128.12394    126.10238    126.28622    125.91451    124.13644     124.6887    120.49761     122.0401    117.98248    124.31104    118.85216    124.43879    125.67917    128.33755    126.68635     122.2096    132.41421    124.13263    128.97653     131.3405    132.89741    131.16587    133.64814    131.94294    129.39643    132.68317    134.50033    141.26778    140.00819    144.29556    136.54452    143.60344    137.99797    135.78959    133.48016    135.90726    130.27857    133.65963    132.83607    136.86722    135.23367    133.06044    129.95265    134.85918    134.14975    136.48532     137.6475     139.1931    136.43806    135.28042    138.57638    141.93629    146.09529    145.40996    145.26896    144.94704    147.81385    150.28668    144.68901    141.44849     143.4946     135.9734    135.70584    138.80014     136.9207    133.87248    134.82122     137.2047    130.56569    140.78839    135.53988    140.44685    130.69763    137.83376    138.10459    134.20456    137.59024    134.23785    133.60245     130.5995    136.65319    129.00043    130.02729    135.39446    135.37269    136.41219    132.73155    128.26247    126.16889    130.17988    130.54968    131.09308    127.17403    133.11226    127.86943    132.98776    127.44408    131.28616    131.53301    130.10185    128.34148    132.13729    131.38847    135.29242    133.89685    133.67483    133.20856    131.95606    130.69347    136.04392    132.05246    132.49732    128.55862    132.74215    133.56438    132.71458    131.19035    132.30537    134.27168    129.32599    133.10121    138.44733     138.8074    138.00374     138.0937    142.83135    138.13497    141.86446    145.31575    143.97875    142.91952    142.53493      143.481    149.56133    152.65708    151.10801    149.99409    144.23715    147.79394    152.96123    152.39526    151.39485    150.38588    157.45293    152.80022    151.24024    157.57188    154.80641    163.33998    158.15225    158.64744    165.90522    157.77211    158.77203    163.24666    164.06173    159.84333    163.27241    160.61563    168.66351    164.56846    166.38606    164.92322    164.45078    161.15125    168.73351    169.41113    170.77672      167.441    164.03644    169.25634    160.67404    174.32834    182.75233    169.85657    172.92128    173.91575    172.87885    181.75113    176.11909     184.5024    180.68784    177.36689     182.2302    168.40924    182.47918    186.04119    188.95867    187.13355    184.09482    185.60484    184.17201     200.1023    196.68269    193.59128    190.34915    193.45878    190.32191    196.30318    192.39021    190.35059    195.09955    200.75187    205.19142    207.40304     214.6113    200.79405    210.38001    208.08548    208.07416    208.28432    212.66257    207.16397    211.53498    211.96955    214.59169    204.59861    219.63982    218.84087    214.65321    213.06967    218.84868    220.96501    230.74342    227.76864    226.02292     223.3073     220.6418    219.43646    228.02038     244.2047    229.77164    231.02638    234.64403    235.70576    236.40292    248.67859    243.23642    254.08819    247.65451    249.54704    236.25692    247.97551    260.98818    262.96302    257.37125    257.13809    256.55614    250.79846    280.56386    300.41629    296.54015    275.86309    291.41074    277.48886    335.60623    337.04282    343.65957    302.89512    319.36247    322.12644    310.92313    361.75637    356.78478     340.7336    371.04256    391.13315     369.3506    477.03107    458.81264     519.1042    424.99102    429.63585    471.31696    543.95695    597.23942    579.34962    581.37921    598.38444    545.15444    541.83886          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        6.00    266.95905    244.76923    229.62409    224.38839    232.66994    216.50714    213.98779    201.42412    197.26842    199.48163    192.15684    193.48466    188.17469    184.35489    180.80565    173.42955    174.99606    168.41871    178.13172    178.51657    164.97778    163.92089    171.71407    165.41165    174.18257    196.35449     235.6432    249.94922    240.70837    199.64381    164.06796    158.60751     159.8827    148.32405    152.62802    149.71024    151.54585    143.39243    145.35393    142.68707    145.72441    140.29931    141.09385    142.54932    143.11821    143.84772    133.80858    145.43007    135.90209    138.25201      139.451    137.54426    135.13185    135.05413    135.48597    136.07428    138.11843    141.05076    145.25451    139.30892     139.0504    134.95961     133.7621     133.2618    131.98024    128.21848    128.93107    131.19488    132.69895    138.56429    148.10709    172.45417    177.06458    175.01783    153.81741    147.02448    143.97646    149.52082    145.56012    150.74338    149.90242    152.18712    148.19556     146.2475    139.47622    130.65291    128.66389    130.11093    129.79543    141.84124     162.5156    177.69753     186.2309    185.30591    175.09686    164.48386    153.63053    142.70336    135.87035    133.76682    132.27846    135.16079    131.67319    134.68825    126.46771    119.99538    119.77081    118.76566    122.24513    119.23982    120.26671    120.90728    123.34952    119.90697     114.5974    118.99157    117.00741    111.71363    119.99486    118.65621     115.4253    115.96972    118.05887     123.9042    127.08053    137.33939    142.49792    140.14898    137.22244    130.72161    122.84953    120.21987    116.76485    118.73149    123.89329    122.91651    130.35296    145.13123    163.76927    171.24233    170.23925    167.08855      156.387    143.53785    136.32776    130.16956     129.3803    128.39256    129.63102    127.15937    125.96553    126.35254    117.71326    118.48266    114.81926    116.16246    114.91587    114.17885    115.67863    117.06321    118.16528       118.03    118.89752     125.4426    124.69343    125.41732    123.95161    126.29427    128.29745    122.46111    125.48285    125.55966    127.08495     130.7715    133.12684    137.66079    138.48989    138.79342    138.18779    133.98177    128.12251    129.78119    124.49545    123.89285    122.63968    121.75923    120.55754    119.26101    120.44612    119.09227    118.89911    121.08353    121.25414    122.40114    123.11186    125.68594     123.7019    130.03469    128.95383     130.0129    128.34048    127.15022    127.43328    127.47185    125.98316    127.25241    128.28701    126.01098    130.97562    130.85047    129.40097    126.76931    125.46072    124.27202    118.75057    123.21998     122.2209    123.61284    122.37337    123.79495    128.00941    125.23287    125.13696    128.37234      127.131    128.17603    129.02442    130.51607    130.71104    134.02412    132.00904    132.02011    130.19058    133.15876    134.72749    133.96717    135.37506    137.40388    139.08116    141.54356    137.15204    139.03904    137.76965    138.03287    134.00766    131.98984     131.1539    130.96779     132.3877    131.63857    133.20934    134.27713    129.73362    134.21283    135.96057    136.57409    134.98902    135.02478    135.59097    132.87783    138.83984     142.7433    143.85305    147.72297    146.02419     143.3749    143.75928    150.82121    141.38342    144.88307    142.74233    141.52482    133.14607    138.04662    134.57152    130.31835    133.88324    134.66918    135.00043    133.85067    133.21756    137.49984    139.86481    134.98822    134.95139    135.35425    134.56406      138.107    137.87226    133.35838    131.86211    130.46938    130.88837    132.47882    135.47882    129.05882    134.34616    132.23389    126.39481    131.22235     128.3461    129.18928    132.08777    132.06036    126.00915    131.86584    131.60905    130.31569    129.64303    129.04588    130.49875    132.86724    131.04438    132.72923    132.14072    136.28927    131.25256    129.88318     131.4099    133.57925    131.96434    132.81398    130.70733    131.69207    132.06733    132.49599    134.33605    136.33035    133.18902    133.00802    137.95256     139.0421    135.61376    137.79446    142.05501     142.5753    139.14064    142.94226    143.90852    144.29817    145.26487    147.16326    144.28878    146.26732     150.2574    151.06543    152.11348    150.15901    149.80341     153.7685     154.8237    153.36899    155.94659    155.27781    156.45732    151.50096      159.651    157.72797    159.94071    159.85857    159.67001    155.59771    160.52856    158.63399    162.13681    158.79426     162.3295    160.81911    155.84627    164.68468       166.04    164.31939    163.11514    160.50539    165.27823    166.81412    170.91011    171.15707    167.73166    170.92773     170.4517    170.28094     171.5045    168.49672    170.21606    171.13506     174.0686    170.12399    177.98146    177.70958    186.13742    186.10224    183.19254      180.669    179.65719    184.69496    177.53516    182.17234    187.67222    183.14029    190.17887    189.68133    185.98008    199.18291    188.50817    185.65755    191.83918     190.5431    202.64393    201.95083    194.59441    203.41109    204.53202    197.19123    208.38851    205.95203    216.57407    202.66157    199.73004    212.18458    200.08712    214.70072    218.37568    204.01734    208.82041    206.58047    212.93155    221.91738    216.62554    212.07631    218.68412    210.32297    215.02143    211.20162    231.34341    231.11354    235.29282     226.5302    223.72284    220.03013    239.69059    237.08739      238.996    239.86527    233.77857    228.42865     250.4979     254.0647    244.22377    245.27889     238.1994    241.27203    252.75016    254.81566    254.53547    274.06838    245.11485    265.23131     259.4163    283.00485    280.20643      285.241    272.06629    307.22581    309.61969    334.94067      302.751    330.22159    322.90233    337.47452     349.8448    277.77209    379.46269    401.21504    402.47729    413.91929     352.6565    389.11299    460.20619    425.32276    532.03234    446.36595    504.37998    455.03935    575.97208    537.44237    600.99182    590.67356    626.29096    638.33098    608.16037          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        6.50    269.36925    242.71497    228.02195    228.11304     224.4603    219.54412    210.77366    202.04661    202.27277    201.65961    190.05733     192.5684    187.47173    185.82925    182.42302     175.1352    173.63655    172.33478     175.7807    177.39149    163.62128    165.28129    169.78381    170.62833    175.07857    195.16698    234.83587    252.33516    238.74353    199.28565    165.34914    158.24188      158.427    148.27621    150.67115    149.09983    150.80099    146.43552    143.12793    142.82058    146.82211    143.90733    140.01945    141.57218    145.14359    141.88801    139.56842    140.96014    138.31209    143.49749    141.49482    137.39783    134.30747    133.86824    132.49537    135.44047    138.83835    142.78343    146.23351    143.90516    138.65183    137.52342    134.50879    134.30516    131.31196    130.31767    127.59808     129.3816    134.26541    139.76515    147.45912    171.68852    177.72152    173.25055     158.3206    148.01793    145.57751    147.24431    146.88772     149.4109    145.78197    146.58569    148.04396    144.07903    139.92465    130.35545    126.68106    126.08677    133.31412    143.91939    162.53477    177.13188    185.86754    185.10861    177.10929    162.14372    156.35235     144.5975    139.22311    134.01356    137.15283    131.23143    138.75784    129.08288    124.83327    121.32072    119.83691    122.44863     122.1133    123.28728    120.66991    119.09158    119.29735    117.82341    116.29534    117.99798    115.78257    115.86463    118.35675    118.48448    116.60735     112.6775    116.74403    124.06356    128.47855    137.24511    141.34136    141.03144    135.44613    129.07741    119.46746    118.66176    122.26735    119.32043    121.60275    122.27358    131.64308    144.26332    161.02635    171.17137    175.31262     169.0831    159.74909     143.6346    137.37031    132.62636    130.08909    129.25976    129.88112    127.03576    124.55528    123.57883    120.05085     117.5499    117.37458    115.93279    117.91815    119.88802     117.1681    116.91918     118.4615    117.26078     119.7353    117.57508    124.72537    125.00952    125.75678    126.23337    125.13813    120.76171    128.60012    127.32134    130.19347    127.58597    135.02847    134.90425    136.39952    134.95751     136.0223    131.36127    129.22606    129.12149    124.58785    125.61384    124.32062    120.25872    121.70191     121.0791    119.06375    119.49813    121.39251    119.34157     119.9738    117.81176    122.79342    120.80794    125.43594    126.18205    128.95384    131.34655    130.03236    127.92076    128.28329    126.77078    123.07029    123.02347    127.53637    124.76145    126.73373    129.58812    129.34546    128.65644    127.95716    124.87136    122.79171    124.87373    123.99966    122.48554    121.84363    123.37475    123.65801    126.19901    127.56605    125.75077    125.26971    130.78096    130.14926    128.48896    132.47044    130.57513    130.28561    131.65843    132.92057    129.42971    133.26242    137.23903    135.77845    140.14595    140.48957    141.99882     137.6098    137.97606    138.95556     131.7003    134.95705    133.61843    131.47134    130.44858    132.54385    128.55158    128.52761    132.19204    131.71021    133.51198    133.30118    134.47127    134.22273    133.01795    136.12878    133.10519    138.85547     137.8038    142.77099     146.9379    149.97652    148.39434    145.23921    148.66463    147.54749    144.35817    139.72768    138.02452    136.66253    134.88077    133.02432    135.64746    134.90012    133.13748    132.97252    133.47897    136.04445    135.69312    134.85883    139.26783    137.07577    132.28437    137.50669    135.58479    133.76561    131.47302    133.11582    126.56692    131.77616     131.2747    130.14658    129.72996    130.38518    130.96979     128.8551    129.26144    129.97215    130.04632    130.15036    130.37769    127.72063    128.59408    131.45699    130.36094    133.04468    137.57482    127.55358     130.3205    129.66547    133.72443    130.48104    131.21259    136.07531    132.79259    134.45313    135.35899    137.09283    131.22962    130.76745     131.5065    131.30631    135.05801    131.28444    132.22218    132.44862    134.66494    132.40637    138.09169    135.16323    135.74272    137.60214    136.14429    141.25826    147.30063    142.92586    143.34633    146.73416    139.68084    145.24655    141.74743    148.34333    147.93855     144.0414    146.27181    151.39395    151.81387    162.12687    158.76349    156.87273    153.12195    150.33794      153.354    155.77186    156.64762    160.42643    156.95724    158.13657    158.98632    155.03188    159.69231    162.14577    162.18653    163.58474    166.67071    158.57206    159.75328    163.28764    164.40717     163.9722    161.01952    167.85878    166.94923    173.90393    174.00863    172.60838    168.91803    165.61448    173.90285    174.28683    177.15433    170.01527    171.94731    170.12979     174.0257    172.52372    176.14694    177.65138    182.12791    183.59013    178.07776     176.6956    177.72426     183.8496     177.9173    182.36073    178.42797    185.64427     186.7873    183.34669    189.87362    187.24884     193.9448    186.20694    195.18474    193.94388    198.32246    204.97433    197.26336    203.22659    193.50723    202.47843    199.03912    202.20621    211.91211     200.8245     205.8708     202.8096     215.2007    217.97533    211.19256    212.94014    215.51907    215.76112    218.04919    222.29576    217.65719    214.55024    216.47391    212.33337    230.34201    220.41536    222.49017    224.37903    231.90296    231.42184    223.91637    233.02132    238.51357    240.84347    230.78976    234.07415    242.74764    253.47005     245.9094    246.35971    236.79746     246.6905    241.29421    259.79316    265.10722    265.31294    255.18399    263.75347    266.98346    260.76832    274.42727    281.50016    287.19299    293.33832    308.52663    305.63531     318.0923    319.83084    305.36038     331.6946    330.99733    324.21316    318.94766     362.9389    362.11896    392.60566    361.17331    344.20655    386.73562    474.85933    486.67155    443.98117     450.8128    444.28376    472.90241    575.63154    692.02074    531.14063     469.6409     586.3883    273.40825          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        7.00    271.05103    243.39769    231.29284     232.4429    217.32838    223.27057    211.53535    206.10915    206.83498    202.44347    194.00667    192.00373    185.90328     185.4276    180.22491     175.7834    174.82384    176.62066    174.14583    171.94692     166.6175     165.1509    169.34862    168.06133    169.78238    197.44349    232.78668    250.28423    234.30662    193.82673    166.84577    160.37821    151.96789    152.81107    149.95451    148.44261    150.16222     152.0273    148.02289    143.87326    144.35241    140.82857     142.3935    141.74557    143.87978    144.15817     138.7423    139.47235    135.94654    139.66657    137.69901    140.97944    138.96569    136.14482    134.03711    137.87974     140.9912    145.43443    148.28324    142.25306    138.79753    135.66889    133.33778    132.61341    135.13602     133.8564    130.08781    132.41249    131.56034    136.98289    155.22714    175.07084    180.52441    178.08305    158.79235    144.20656    140.46808    143.27301    147.83916    145.17844    147.94398    148.08376    149.45951    149.55528    145.68808    131.69304    128.82512    128.23266    136.11168    145.06579     163.8508    179.50241    189.92564    181.16603    169.86787     157.9824    149.95851     140.7812    133.93045    132.05592    134.86291    128.53073    134.39188    128.92885    124.92858    122.34909    121.35738    120.33831     122.4938    122.03631    120.01046    117.23566    120.64096    118.11471    117.27625    119.02709    118.50519    115.23984     117.9078    120.21719    117.70079     119.0732    116.62443    123.48013    126.28178    138.42714    140.59694    141.03369    135.69192    128.62023    123.93529    120.36056    117.81697    120.01385    121.90724     122.7147     130.6889    143.47192    160.16109    171.36127    175.55785     167.4933     158.5097     144.4946    136.02795    132.07448    127.87615    133.15859      131.508    129.63504    123.83285     123.1371    118.51969    116.05278     116.6155    119.80175    113.04201    113.61344    118.26839    119.34648    117.41754    117.17715    118.47266    117.84332    122.31865     125.1322    126.71263    123.75073    126.72874    124.35148    124.37433    123.92298    123.85707    124.96738    132.66399    130.28471    130.95648     133.4119    131.71913    134.53937     131.5713    128.46328    125.20134    127.01338    122.32451     119.3007    123.42814    122.31396    122.13638    115.50864    121.84651    123.10499    120.14669    120.37756    119.90389    119.83571    123.83618    127.99282      128.941     128.7918    131.78962    126.12052    127.41721    126.69894    128.94355    123.88324    122.83551    126.04125    126.36124    128.34677     126.6259    128.68104    128.45328    124.06686    122.77926    126.57465    126.61895    125.36662    125.80901    119.38947    122.21933     121.0568    126.82797    126.16871    129.32665    130.06647    129.11674    128.88573    133.59077    129.14042     128.6005    134.61383     133.9231    133.28925    137.22971     135.8381    136.25042    138.79694     138.0321    139.48466    135.91949    141.88527    140.44603    138.21477     136.9134    139.40083    136.01531    134.14159    133.79815    134.23066    133.13027    132.22983    130.27611    132.33349    134.60031    135.71124    136.19355    132.95465     135.4726    135.26569     137.1455    146.26641      144.297    150.35341    149.12519    145.67144    149.69497    147.22252    144.94013    146.20092    144.56863    139.09221     134.7513    140.72377    137.57971     133.4295     130.8234    134.34152    134.13807    137.06224    134.82476    137.86749     135.1999      137.651    137.28552    132.58335    132.29086     133.3796    134.16933    133.44759    131.07755    130.03965    132.67999    133.35253    132.85545    135.11701    132.44176    129.93172     128.9082    130.51827     131.4774    129.91007    130.52867    132.75727    127.90743    128.74155    133.85176    131.40594     134.0617    131.25191    129.57012    131.19134    134.19121    134.99024    132.69673    136.72957    135.35503     127.9183    135.86563    131.44749    132.13777    132.23533    128.36416    131.66984    130.43354    135.42187     128.8175    133.63808    134.54334    129.16892    133.32212    140.24144    133.40552    139.88286      134.215    143.96945     136.4376    142.95815    149.24252    144.65992    144.84999    143.57409    142.58224    144.77002    143.75672      146.534    148.35714    149.07899    141.40719    148.74677    153.79818    151.81453    149.64838    156.03909    157.78632    151.98071    155.20928    160.54133    155.89829    160.70643    159.93534    160.29756    157.52988    163.36364    159.70357    159.55616    162.69668    163.60462     161.5489    159.98731    160.71198    165.94639    168.74651    168.72284    164.02306    167.97933    167.43643     169.2171    169.09431    173.54551    168.74535    170.02006    174.92335    175.85903     176.1846    171.76733    168.63569    169.82168    175.89752    177.41941    171.68979    176.62106    173.74431    177.53937    175.11179    185.80084    189.15926     182.2567    183.34533     182.9135    186.03865    195.40056    185.52488    193.13669    188.61264     194.7082    189.52736    193.34238    194.18097    203.04851    199.54226    196.32832    192.61243    194.29005    207.35393     209.0891    205.70826    208.50024    200.31964    208.57581    207.21415    211.75435    205.86366    212.56695    211.39879    209.55633    219.76835    218.38416    219.19895    216.43591    221.83667     222.4369    216.82039    220.60072    237.96684    226.01399    217.14353    217.41399    221.39013    237.15549    245.87125    240.35072    233.07442    237.34375    233.84977     228.5733    242.95809     248.4452    244.23724    235.74667    251.26011    256.58149    252.06166    263.20214    260.60977    270.19823    266.61252    271.51912    251.78428    267.61882    302.60623     276.0081    294.44545    280.96844    319.72232    319.48356    324.39782    317.36446    308.66975    340.52084    360.72027     315.1645    352.54193    344.48343    384.83757    377.27194    376.26782    384.34543    504.60494    537.55835    451.09425    435.19279    482.14181    418.97802    553.00803    570.90693    393.28404    552.73512    585.13726    534.93692          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        7.50     269.9347    244.68867    234.21271    234.65441    218.68717    227.15562    212.59496    207.86269    205.83538    202.22223    197.69992    193.36284    189.82987    182.65596     177.1641    177.82405    176.99039    176.86163    172.89548    167.93265    167.05936    162.37679    169.70745    165.99937    172.20539    192.95553     234.0071    250.57308    238.36553    194.57497     167.7362    159.85577    152.99664    153.08065    152.34199    147.11024    152.85135    152.81418    149.68763    146.58944     141.3273    144.21261    144.29327     141.5408    140.32454    144.74685    142.93902    138.98297    139.50229     143.3065    138.93914    141.92917    138.82442    131.27534    136.30486    132.86888    140.86566    143.59674    149.60792    143.51052    134.91867    133.47987    131.09696    133.44825    134.76938    134.00502    128.35322    131.35339    128.98276    137.34566    149.55631    171.96691    176.22199    175.08396    157.07345    146.91171     146.1825    142.45555    143.86391    142.86784     144.4078    149.60495    152.63836    153.74695    145.28274    133.53312    127.52249    129.06814    131.72656    144.70299    160.24809    181.20553    188.49151    183.00638    168.23441    152.61061    145.03519    142.37547    133.65002    135.79136    133.03394     134.3593    133.46774    129.67871    125.19871    119.46176    121.49878    120.49061    120.18036    121.45803    122.05669    118.39878    118.56496    118.49767    118.85911    118.66627    117.21954    120.27869    118.35481    116.65366    118.44184    116.90164    116.01771    123.49769     126.0108    137.43106    140.23242    140.88023    134.94411    128.79472    124.08501    121.18904    116.27352    121.32334    120.88693    123.04403    134.53438    143.86665    162.64985     173.8087    173.40185    171.37106    156.70903    146.33328    136.42316    132.77834    128.02421     130.3781    129.57034    129.32179    123.51996    121.53633    122.81994    114.27884    116.72232    120.56045    116.88584    122.37766    121.95961    115.52986    117.58782    115.73168    116.32877    116.14067     124.2055    124.26075    127.58755    123.85446    123.27649    123.45319    123.79379    122.42653    119.82598    125.69627    128.68685    129.22283    128.50501    130.64801    134.47209    133.71631    132.00618    131.47956    125.30895    126.11402    126.88981    125.98995    125.69661     122.0118    122.97783    121.60636    121.25693     119.4972     121.6682    123.47935     119.2138    120.00805    129.37128     130.0954    129.26121    129.90614    134.26249    131.10375    127.60175    127.96311    128.18059    127.46957     124.7133    127.02227    125.09989    128.77936    130.58912    128.49467    126.12993    130.88134    124.77983    125.48668    126.02322     123.1706     122.6663    123.29662    123.28496    121.37961    124.60159    124.20599    127.50217    126.86797     130.6727    129.24357    130.63967    132.20021    136.56238    131.22689    134.82432    130.34985    133.29727    132.28792    138.04059    136.32522    138.77727    139.63834    138.87173    139.22376    135.08794    140.05979    141.15893     139.3948    134.41009    133.63385    131.58417    134.13076    135.42163    133.76437    133.55219    133.40592    133.62642    136.83813    134.18264    135.04912    136.78243    136.57253    137.94795    144.59831    139.90792    142.79952    146.54844     149.0959    149.12931    147.89435    142.54481    146.38319    140.40966    139.25937    139.75527    139.84691    134.73027    131.92039    133.28837    136.63703    132.45327    131.80099    134.46289    136.10834    137.77398    136.94028    138.11415    132.46453    135.86463    137.78693    134.81389    133.87152    135.66803    130.12575    136.11162    131.65876    131.57772     132.5662    132.82551    132.31221    128.52247    132.92466    131.34846     130.3088    131.84013     129.7723     129.8955    130.51361    129.71167    129.96228    131.22428    133.83064    132.41532    129.83569    135.44275     132.9838    132.35477    134.66118     132.7232    131.96796    133.88529    132.94543    132.46856    130.53773    133.90802    132.98157    132.19857    131.53772    132.46673    133.91867     130.0831    133.48221     135.6392    137.57849    135.35411    139.68724     138.7767    139.83946    140.67537    146.53312    144.17725    143.31954    145.12473    147.89294    142.88623    142.77367    144.93625    145.43294    146.06144    146.71554    142.91341    148.62423    149.79809    157.86715    150.71064    159.81749    157.67163    153.85488     151.9064    159.58942    161.76783    162.04146    156.68659    158.88363     161.0166    161.90023    160.56509     165.4513    157.70287    159.80224    155.12217    165.55817    169.88912    164.63969    168.80339     163.3889    161.67839    160.82448    167.30243    167.38975    171.98346    168.45596    163.62701    163.32153    168.97945     172.2093    165.75247    175.56756    173.12957     170.6606    181.01879    180.59445    177.05773    172.10497    177.95473    181.97108    176.28515     180.9542     185.4314       181.21    188.03879     183.6396    186.95141    189.27469    189.33482    186.69385    197.56506    191.20562    196.83692    194.56224    197.64708    199.06314    196.19783    195.80947    196.28728     197.3802    200.79823    210.63788    207.26764    212.39853    214.39135    211.52497    212.22163    213.70041    205.54245    216.97741    215.73112    211.52449    211.07086    222.51577     226.0544     233.7836    215.52354    221.56261    222.44887    223.86404    224.71238    232.82757    229.26373    222.21465    229.34699    232.82521    241.02004    242.29735    239.93711    234.36907    231.87866    248.86624    253.70803    249.80674    249.41718    251.96311     245.4319    256.09721    262.33675    264.24669    273.21762    262.65015    253.41692    261.53173     257.5119     293.7405     285.5061    273.44095    313.81897    298.49718    297.64748    329.28562    327.50202    298.28679    328.55521    317.76513    293.14698    328.81054     346.0197    360.48931     397.8424    384.74658     337.4083    385.89156    422.21626     478.0119    450.63333    505.77365    473.58775    434.83642     634.3865     591.7105    530.53107    511.27931    636.00958    566.75508          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        8.00    269.06485    245.75085    234.74784    233.84553    219.19807    227.22729    214.03378    207.04761    204.59297    200.62155    192.90252    196.38493     193.4377    182.75971     182.5261    180.95399    178.43413    176.18338    171.06157    165.48111    166.18828    166.43973    167.58274    166.44306    173.10095    194.84726    241.77686    253.60361      238.077    196.29187    171.95229      156.244    154.83623    154.99333    151.40016    146.69386    153.20818    150.97199    148.65499    148.60444    141.29751    151.98477    144.14824    143.18226    138.92209    142.25071    138.35941    136.99432    141.95426    138.59946    141.31456    140.85463    137.82874    132.28007    137.40953    139.62836    138.57079    141.37099    143.15439    140.60147    138.98229    134.22223    134.42897    133.87129    132.28663    131.51435    129.96866    129.96448    131.81413    140.52398    153.30581    173.45825    178.53709    174.04294    159.51542    146.40204    144.11687    142.81695    145.98787    143.84963    145.58298    150.76184    157.31272    157.63493    147.24697     131.8367    126.20993     128.6244    132.82968    143.11752    161.20657      182.381    187.58328    175.29089    165.59065    146.83225    138.91708    133.72947    130.58202    135.79792    133.79599    132.09537    131.54164     129.4504    121.39324     122.1278    120.09514    117.55951    124.35428    119.00504    119.74144    116.74262    119.97541    117.86006    118.96598    120.97564    116.25212    122.96276    116.48099    117.86163    116.25149    120.22042    119.66888     124.5469    126.54587     138.7362    140.94723    136.78212    135.12886    129.85131    126.30039    123.35862    123.12208    120.74387    121.14891    124.13733    133.17916     145.2277    164.55107    170.56816    171.52973    170.83051    156.75769    144.12571    135.71086     133.7657    134.67313    134.15863    129.88856    129.82038    124.79264    123.92868     119.2357    118.81109    117.00433    116.44446    113.46519    118.12912    119.77403    116.08655    119.06045    119.52223     121.5621    115.33194    124.25133    125.36497    125.73926    126.16803    124.12341    122.56664    121.61626    124.76451    121.44501    124.98152    128.82912    128.75918    127.77187    133.51942    134.36787    133.01461    129.20913    128.10205      127.991    123.83691    124.20838    121.70368    121.93596    120.49272    121.31728    118.37843    120.81796    120.95582    118.79104    119.11194    122.17325    124.22716    125.66439    127.35412     130.2549    129.61285    133.64447     129.2636    128.80909    125.43428    127.66889    125.01571    124.28004    125.95927    125.97114    130.78756    130.29019    125.45364    127.33271    127.64929    124.15714    123.34913    124.19394    121.65154    126.29609    121.50774    125.28348    122.73961     125.8153     124.8914    126.58243    127.53721     126.3446    131.67647    129.77871    132.74794    131.70974    129.03821    134.64878    133.20322    131.16272    132.50437     138.1962    141.81828    141.95552    141.61769     138.2247    138.60023     135.5613    136.82958    138.26835    135.18639    135.98872    131.06878    132.79946    137.26299    131.73467    132.06168     132.8309    128.66218    132.17905    138.31009    135.78247    135.30891    137.59602    135.93127     137.7797    139.12673    145.44998    144.88034    145.95597    147.79124    144.10453    148.06069    149.41476    144.97448    143.31361    138.64206    136.26149    136.98688    136.55376    134.97535    135.00545    134.02689    134.78727    133.87133    136.35478    133.89082    137.59249    138.38915    136.50396    132.68048    136.06821    133.52517    135.30565    132.33204    134.43097    134.59788     132.3994    135.05027    131.65085    130.13371    136.14291    129.94777    131.94818    131.07844     129.9956    128.50199    129.71238     130.5947    125.81599    126.45868     129.7589    130.26887    133.12732     134.3132    131.19683    130.52968    132.87143    135.08193    133.16251    134.59684    132.70865    130.99593    131.17361    131.21651    134.34874    135.91375    131.04517     131.0107    132.31024    134.40343    134.27535    132.56443    131.60521    135.50322    136.61783    139.36379    136.42417    139.92166    141.77493    140.91259    140.18156    146.65103    143.15468     142.7933    142.12068    141.76377    144.03716    144.53476    145.45133    141.23063    141.87625    144.62002    143.08904    148.43877    149.74977    149.24533    147.70846    154.14291    153.03011    148.43589    152.16543    158.05576    168.20861    162.26605    164.43537    158.69768    159.40153    161.37832    161.91804    162.85217     161.6637     160.7539    159.14606    165.52063    159.80133    162.73355    160.32538    160.82243    156.73003    165.55126    167.42745    169.57752    169.56912    168.33889    168.26746    166.41579    166.65998    172.61494     169.6136    170.72619    171.28374    175.17255    174.35154    183.78984    175.68353    179.49153     182.6338    169.74387    177.14665      182.565    178.75794     184.0001    178.12245    183.52859     184.4856    192.15794    190.64368    186.13999    185.38941    195.54872    187.02805    193.81473    204.74797    204.07378    201.09486    196.17529    194.13469    196.57507    201.48019    210.89477    204.89662    212.11127     202.8639    210.33422     207.3036    212.10257    217.51149    213.15638    210.53335    213.93777    212.13212    219.29212    206.71049    222.14048    212.32981    215.49912     212.6917    223.28143    230.93648    227.97609    233.74917    216.88072    233.40835    226.07958    247.95785    241.48665    239.02697    236.79839    228.67288    237.15447    249.47797    236.94859    248.73472    236.38854     248.6189    233.88251    247.44894     245.1549    268.76009    263.36962    269.32424    265.56358    272.75003    278.39685    286.25428    289.04377    287.55063    288.77337    294.06062    322.83151    310.37255    301.26367    351.36028    288.17159    300.04095    338.96414    341.97834    386.96732    355.63114    353.50058    351.57555    376.90563    437.09115    446.23016     418.6238    427.01034    431.02268    469.40999    637.04914    541.49803    587.99295    595.23598    458.55137    569.23099          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        8.50    267.38675     247.0879    234.60482    233.00399    221.52804    220.20371    215.37981    206.09395    202.22286    197.41385       189.11    195.84324    195.56273    183.01693    184.78265    180.97223    176.22086    173.66973    169.71647    164.48053    165.73095     169.1127    167.46216    167.15511    172.39559    195.28277    237.88885    253.42509    236.20136    196.10065    169.69617    156.10378    156.00231    153.33268     148.4448    149.57517    150.77239    151.92941    144.49152    146.14707    145.17897    137.87044    142.61554    139.36458    142.22131    139.47653    138.24681    142.60282    134.43895    136.52378    138.63944    135.75141    139.68388    133.94063     133.9396    133.05863    142.54658    142.82801    145.37402    143.75445    140.76309    134.50315    132.81722    133.28612    132.02151    131.87405    128.93159    132.07491    133.06202    137.71982    152.97853    173.72116    181.87354    175.92631     158.7663    146.77339    145.30742    147.06542    145.95586    146.18936    147.39333    156.88303    156.61064    159.59957    149.04326    134.19353     128.3793    126.23246    134.00111    142.36914    164.16293    182.17154    188.14508    178.74044     160.1312    143.54022    132.62497     133.1761    130.13025    130.91589    133.51539    132.44612    132.83773    130.06567      124.994    117.58999    118.35818     123.4292    120.99926    119.97924    122.48138    116.75104    120.11596    118.58637    116.53493    116.18709     119.5558    117.97017    116.00751    118.32524    118.02517     118.2138    118.10773    122.28924    130.20692    137.01967    140.61677    141.67595    137.35099    130.48425    122.70481    118.14287    120.67421    121.65766    125.23523    127.83756    134.58809    144.39287    158.81693     170.7351    173.01103    165.93774    155.26262    143.98413    135.60071    134.41354     133.0438    131.82664    130.69085    127.40051    124.08397     123.5902    117.71435    117.51055    119.65375    117.11052    116.92597    116.59261    117.28065    116.66756    118.83771    118.53098    119.18953    117.92791    124.79121    125.04389    125.94102    123.74404    125.04226    121.45629    121.83528    122.74003    127.19524    127.25393    121.71287    130.10508    126.16976    131.23494    129.77895    135.41427    132.36293    128.94671    129.12508     126.5342    123.88372    122.05343    123.78494    122.03417    120.08181    121.10483    120.19963     122.7192    119.24998    124.92352    122.25738    122.72559    126.67487    129.05188    130.55245    130.58338    134.85237    129.96246    126.89428    126.72076     126.5042    125.94427    124.35837    128.33221    130.02785    128.62983       126.01    126.10034    127.23842    127.31584    125.80696    121.90547    124.40366    118.49726    119.46408    120.47571    123.04946    121.28472    122.20124    123.83783    126.22851     129.6184     128.6069    129.56709    127.84049    130.71568    133.86707     134.8257    136.63947     135.7676    132.50183    137.96872    139.74977    141.00846    142.97805    140.82229     139.7355    135.97972    137.36902     136.7008    136.69981    136.18557    132.87772    130.77023    132.98463    134.60767    133.08759    132.15081    136.18443    131.43325    133.62498    138.11139    137.68012    134.73066    138.94439    137.37371    141.53474    144.67599    148.13609    145.03913    150.42542    146.68551    144.60222    150.10948    148.05473     145.2486    141.94369    141.47459    136.72798    136.07286     134.9114      133.236    132.85186    133.76329    134.34046    134.89149    135.10108    137.45745    135.10074    137.71114    137.87214    132.21512     138.4255    134.43056    134.81989     132.8552    133.24962    130.41435    130.72383     131.2662    130.15192    130.48515    128.18909    132.39503    128.21378    127.66794    129.53034    126.54627     128.8262    131.26963    132.54699    127.28169    128.26461    126.56631    132.72927    133.45589    133.38373    128.13486    137.02649    136.73364    131.51605    137.50279     132.2249    130.58103    133.67209    131.63575    131.35629     129.7929     128.5372    126.30524    129.00844    130.18309    129.28023    132.70085    131.90137    134.78754    136.67969    137.40913    136.68607    140.85597    141.68493    138.78752    138.30626    139.53216    145.52274    143.57416    143.56547    147.81342    149.85572    149.77436    143.39567    146.46202    146.36205    146.32582     142.0129    147.74324    149.22599    151.83897    148.52777    151.23337    154.95968    152.63217    155.29967    157.97552     162.1205     164.0576    160.00648    160.42452    157.55602    161.18939    158.98294    160.39194    162.31471     162.4079    160.99721    161.14854    169.81287    167.14702    165.12563    160.40395    159.57735     151.7193    162.41938    167.39522    166.68244    167.37009    169.38821    166.25257     167.6417    170.78706    176.44729    175.77716    170.08679    172.32733    172.13465    180.19325    169.19083    177.04005    178.79899    178.95522    185.09647    188.11524    182.51334    181.38098    180.60417    183.53754    177.59575    190.19755    186.88616    186.57115    187.32913    191.16935    182.63276    192.85877    197.81221     197.5759    201.02013    192.60097     197.7233    194.32884    205.03608    197.80314    201.04587    200.16557    201.55384    205.48553    212.48934    214.94901    223.74705    221.43185    215.08494    212.40044    208.32062    213.05471    220.28018    210.89882    227.61228    210.16092    217.04269    221.81656    232.15173    218.11072    219.39022    227.77924    221.77901    231.25798     235.2747       235.12    234.34425    231.98154    232.07314    231.29757    244.45908    251.59132    245.26141    237.29123    247.66119    240.45841    269.61864    258.13895    262.51961     275.5505    264.62543    260.20945     262.4029    289.67778    283.04693    288.54172    314.51865    311.39443    284.48598    342.61246     346.1782    335.24838    333.98821     341.0843    341.78546    340.01092    387.76608    398.67915    391.44891    357.82586    377.40711    362.37447    480.30343    478.60908    407.96584    425.29961    455.97114    462.11195    553.58714    537.89925    506.94811    591.51381    626.47053    653.07164          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        9.00    262.76197    248.21339    234.32088    230.40318    225.00161    214.52576    213.61589    205.70874    200.53785    194.30603     191.7022    192.84163     194.8355    180.65068    183.22321    180.01636    175.48779    169.95362    169.27765    168.78974    164.51998    166.86605    166.64595    165.02757    167.73867    197.19915    238.02687    252.84894    236.20484    195.83683    164.58206     153.5075    155.04959    151.32293    152.23402    148.66718    149.48397    152.03601    147.53358     147.2229    145.77463    138.37015    142.52349    142.86106    140.96288    143.58513    142.19306    141.68964    136.80325    139.27628    137.33491    137.19414    137.46562    134.20115      140.266     137.7206    140.45561    143.82107     146.6235    144.73718    135.70206    136.85084    135.99802    133.72151    135.02804    133.02113    132.18235    130.78322      133.499    141.16057    152.14313     170.7814    180.26513    173.16887    162.07791    149.82183    144.75437    142.91888    151.48842    153.32802    152.11642     161.4267    159.93392    157.75641    146.46904    134.05045    125.16242    128.40795    134.47942    144.04875    163.01805    180.82393    184.91868    174.09228    153.74863    142.81559    134.06278    128.20029    129.06934    132.08455      128.992    131.91241    132.40627    128.28977    122.31898     122.8831    120.32963    119.30887    117.90857    118.69367    120.04167    116.21543    116.88487    118.64495     116.5582    117.42931    119.09243    119.41381    115.17939    116.92828    119.03034    117.95766    121.69984    122.49927    130.51804    133.11747    141.83188    140.23862    135.61899    131.52108    123.76409    120.63133    121.83195    119.66244    121.86805    124.65101     131.0901    143.11258    161.03326    170.76114    173.03053     168.5166    157.65845     146.9592    135.05097    130.18635    133.48099    134.36157     132.2474    126.73019    120.44754    126.30835    119.51494    120.68036    119.52604    119.95973     119.3774    118.76064    118.41661     123.1828     121.4468    117.57127    119.13785    117.26084    123.86069    125.40708    124.49001    131.39313    128.87203    125.71339    120.67415    123.96927    125.24063    127.42734    127.69614    131.85707    132.96137    132.17176    135.65031    127.94456    130.52066      130.296    128.70936    123.34384     120.4824    123.76236    124.98232    125.44105    120.48014      119.794    121.20975    121.15354    121.88289    120.02654    124.25867    125.68036    128.89837    127.01806    131.07729    130.18755    135.45426    131.33725     128.1377    126.58074    129.73559    124.27641    127.18265    127.95413    127.71687    128.46558    130.18683    130.41204    127.15221    129.66877    124.84431     121.5659    121.44766    124.58275    124.91412    123.69229    122.92759    121.03795    122.91835    126.61437    125.69834    127.80794    126.56512    131.00414     128.4939    134.40265    131.40411    133.00499     130.9891    132.07706    132.03923    136.40108    135.84113    136.48987    138.13147    138.59402    140.36745    139.99161    138.82641    135.19936    133.86498    133.73729    134.47239     131.3443    137.00239    130.83108    134.65062    136.59236    138.27351    132.42921    133.06223    135.81057    138.60843    137.85238    137.17194    137.96432    133.80837    144.27083    145.43173    146.86559    148.47669    147.67712    149.15232    145.79946    146.05859     143.9433    141.79009    139.29995    137.01478    135.32269    134.36718    132.36923     133.5062    133.55748    138.50744    134.79511    137.69973    136.93168    138.45987    132.79571    138.06652    134.84107    132.69943     132.4536    136.19553    131.61571    132.08897    133.67487    131.46372     131.5168    129.86736    132.13991     131.3657    131.03479    128.08086    127.87576    131.49025    131.46884    129.67074    127.32796    126.58793     130.3377    130.79819    128.64411    129.75469    131.56788    132.32601     128.9054    130.93575    133.20733      129.185    131.42265     132.5181    132.75194    130.61396    130.99755    133.43482    128.69515    130.51124    128.61408     130.9951    131.74444    130.39787    131.43417    133.68561    134.32411    130.68614    136.46674    140.08624    137.62951    141.66194    142.56336    138.71663    138.91325    143.41958    141.93491    145.22891    147.27233    145.03927    146.08293     152.5509    147.30189    154.26829    150.45863    149.47241    148.55796    151.95796    153.74548    151.57093    155.21402    147.84272    152.48402    153.03807    154.96003    157.33956    153.19792    162.67024     159.4732    158.46047    159.75514    162.02586    166.05211    159.22459    161.47167    160.91936    157.31863    158.58005    158.69427    161.78548    160.18804    161.06326    154.00057    168.34899    171.43676    167.40969    166.80353    167.89574    166.70395    173.44001    172.10853     170.5785    174.66486    176.15297    174.54461    178.96597    181.83235    175.63956    170.25905    176.78162    177.15197     174.3721    183.77772      178.636    186.32313    179.15905    183.22128     187.2245    190.89341    183.64545    183.31331    180.35831     190.2835    186.27843    184.79056     197.3177    194.88418    203.07923     198.0971    201.15011    190.92305    201.66372    204.19578    201.73207     210.8806    214.66256    201.88068    209.35961    213.13153     213.8905    217.45982    212.44383    206.30299    211.64742     215.5978    215.82945    204.55053    205.76064    219.62926    203.12964    224.10337    219.36522    224.42514    228.05317     228.1583    219.24704    222.31668    228.07919    239.16911    224.11322    236.22372    237.63182    237.04102    249.49582    243.50681     243.5019    248.15712    242.98977     247.6069    256.15133    259.85227    258.78959    254.25405    258.53058    270.54723     266.2116    274.05609    289.36209    277.82794    301.98936    295.73754    278.64555    286.63326    331.78664    338.32267    334.48268    319.33775    321.15931    323.26407    390.59251    370.23435    367.54141    358.83594    360.38907    362.27345    459.41112    424.21364    453.84578    431.53752    396.28681    443.43349    480.95797    510.44613    479.31165     598.6136    655.11202    618.48935          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        9.50    258.96068    248.32507    233.39922    227.54766    227.12821    214.67859    210.47432    206.19911    200.07385    191.94631    193.45441    192.42725    189.57725    178.57433    180.21108    178.29991    176.90556    170.42623    171.46857    176.20917    166.59304    166.18876    165.01281    163.82896    171.88458    202.60079     241.5796    253.04088    237.27186    196.55876    166.79329     157.1371    155.32257    147.07519    148.40743    148.23635    150.27569    148.02855    151.15305    150.47129     148.2955     145.6215    144.37631    139.87189    143.49957    139.64693    142.23869     140.2457     137.8873    138.84888    137.41014    139.62865    140.80135    134.82848    135.93771    135.85851    137.49469    142.71728    146.71778    143.26534    139.25719    137.85614    140.69787    134.23799    134.53657    136.25127    133.07386    134.05887    136.66758      137.771    152.13347    173.45445    178.57269    176.03838    162.24838    149.73597    145.97067    150.38396    152.55596    149.54589    153.32931    163.49133    166.36659    159.16733    148.60236    135.39165    128.70863    124.58375    130.06354    146.64279    164.59475    182.95478     182.3277    173.43732    153.51244    137.77649    129.40333     127.9837    128.31672    129.90078    130.19594    132.32594    133.21866    131.35474    127.00655     118.1253    118.91524    121.67141     120.3581    118.89683    124.94214    118.42044    120.30511    119.50072    117.93695    116.41027    119.48483    117.78857    117.96464    119.22512    120.67367    117.57457    118.79324    121.20344    133.78819    135.34548    141.16258    140.11958    135.19375    130.19469    126.74187    119.23981    122.05693    121.01585    123.67454    124.15373    133.33263     148.3071    160.17173     172.1106    171.20379    165.44286    151.88503    144.60455    132.30461    130.04142    130.18463    130.67697    128.22006    127.92169    122.66421    121.35576    119.74806    119.08996    119.20168    116.52828    115.42668    114.68818    116.26057    117.98882    116.97198    120.70407    120.31474    120.14781    122.82377    123.75944    127.16603    127.40313    128.54545    125.59219    119.67723    127.52658    128.00238    128.67265    129.66121    132.09065    133.35394    132.03294    133.60438    132.42056    130.97012    131.73016    128.07131    120.67843    126.12866     122.4157    121.23152    121.69518    121.99218    120.82726     120.6293    119.53918      121.964    121.96328    122.91652    124.57573    123.70498    128.61142    129.66511    134.90857    132.96913    131.56226    126.93735    126.60027    125.16604    126.52542     127.6166     127.2539    128.73096    128.57162    126.05482    127.44922     126.3167    124.37048    124.55587    121.00671    122.75397    123.78719    122.44591    121.55651    122.78987    122.03624    123.10642    123.11021    126.31973    127.23307    129.80002    130.18662    129.56225     133.1566    132.92251    133.13331    130.93063    133.82542    134.37918    135.34161    136.24459    141.56925    142.84025    140.88879    142.11594    138.70744    140.70863    137.59943    137.96808    133.63218    136.14597    132.31271    132.85974    137.07751    135.53949     137.2985    135.10813    134.60978     136.9133    137.34862    137.90865    134.51728    138.64348    139.97587    136.49702    143.34286    145.79372    146.36323    145.51591    147.35532    147.28851    145.49014    147.89547    143.82616    147.21768    139.11051    139.02498    136.39632    134.21038    137.55882    133.22194    131.03602    136.92444    132.45586    136.23201    133.73684    136.83787    137.34287      138.444    137.04872    131.23714     130.8973    132.66547    138.08095    135.25585    136.75966    129.72341    133.42427    131.66819    129.84603    131.76925    129.55783    129.75327     127.0174    128.32361    133.79117    132.24176    132.81103     131.5344    125.41306    130.30303     131.0341    130.17826     131.7576    129.91326    127.97271    130.85872    133.16979    134.15546    130.83901     133.2787    131.58237    129.51506    131.76341    128.78338    134.00327    128.61831    131.78419    126.83296    132.43945    131.47616    131.29531    132.86395    131.53969    132.22753    136.05746    138.05543    137.92692    138.41447    138.66351    139.85032    140.23295    139.45305    136.89736     142.3254    137.65171    144.08313    148.21072    148.74706    150.75256    145.71818    147.07502    151.02506    154.26961    146.83102    159.18667     155.5594    150.49603    155.61843    150.52451    149.73684    156.25154    162.44685    159.11358    161.78976    156.56436     160.6246    161.31464    162.39313    159.24402    159.21187    164.27742    159.30657     160.7371    163.22103    160.69271    160.81509    161.34815    156.66788        166.3    166.09345    167.42089    164.80506    164.06728    166.16312    161.77669    167.43412    175.47264    171.54458    169.42499    163.38628    169.71954    175.18905    181.14981     180.7683    173.74511    170.60471    176.15258    174.51401    182.25062    183.01485    187.66601    180.53884    186.90274    187.96635    186.98254    189.48159    192.39936    189.19695    191.88331    186.90892    191.39942    195.29682    204.86298    202.65034    197.09484     193.1352    195.60237    204.31389    215.89406    213.51607    210.43926    200.69871    201.45126    210.96221    209.29054    217.97079    222.39982     203.0016     209.1736    208.93318     219.7682    222.07167    210.33494     216.6494    206.21793    211.65565    219.42165    222.74965    222.57373    220.11067    218.60405    211.90711    226.63233    230.48754    237.63863    234.15938    235.05957    237.42825    229.08641    250.82138    249.43279    242.93989    259.06686    245.41068    236.62587    250.97239    256.61614    258.77506     262.6704     265.9286    273.59693     257.0705    279.71788    285.70101    276.87658    281.12648    277.42156    291.67668    336.78676    332.56995    324.47112    350.71124    335.77294    349.69474    337.22228     375.3669    370.53634     366.1423    384.10991     378.7418    349.57215    463.56694    425.54852     437.3472    447.15061    490.92934    461.91041    493.01119    508.32159    385.51483    566.52108    630.74805    604.57926          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       10.00     258.5587    246.36615    232.54236     226.3062    226.10662    213.69174    210.47188    208.06964    198.35675    193.83818    193.11794    193.69844    187.82431     183.3987    177.88037    177.17787    176.71137    174.03057    174.57565    174.12105    169.24952    164.28216    162.76733    163.21339    174.75587    197.60938    243.52768    250.14737    228.56626    196.57104    166.95091    161.26538    154.30576    150.72661    150.62448    145.01987    148.32985    150.82494    149.54963     148.1446    139.27092    144.17185      142.605    146.15593    143.93612     139.2511    144.03156    136.28663    140.34428    140.98346    137.22498    141.17539    137.53755    134.71359    134.61105    135.44713    141.48657     143.5926    151.78765    148.98161    138.57984    134.18849    133.89042    137.47323    129.89368     133.3423    130.89048    131.02857    133.64265    138.66932    153.61057    172.10473    182.02168    174.60221    159.13877    147.19461     143.6246    148.10277     155.7897    154.99004    156.92669    162.46065    167.09744    161.68612    146.58664    133.43829    131.00703    128.89569    133.31409    141.02138    166.02692    180.77639    183.51123    169.51834    153.77699    137.93485    131.76811    125.07263    127.59049    130.28907    133.56345    133.79428    134.79327    129.18472    125.16616    117.54729    116.59547    121.37088    120.46574    118.11765     120.2494    118.77615    117.31581    118.93838    119.31101     120.5102    122.59791    119.00341    115.79207    117.40876    117.88486     118.4398     118.9658    125.59119    132.77576     136.5199    142.77476    139.85786    131.73851    132.93477    124.19997     120.4687    119.02343    119.63353     123.1153    124.66832     135.7937    147.15588    158.97992    173.63258    174.00622    164.43442    155.05781    143.05774    132.52786    129.98392    127.95264    132.03427    130.17306    124.35619    125.53565    121.65103    124.12882    119.47974    117.47402    116.89967    116.42227    112.96118    118.94665    117.52092    116.44587    118.78922    120.52659    120.58736    122.14928    126.87356    126.83635    126.76662    128.05548    128.75841     120.9101    125.64935    130.70535    129.70874    128.79704    132.93384    132.15647    132.01028    131.69825    134.67476    126.28133    131.71687    127.73798    120.96727    124.26969    127.95686    124.93395    128.27064    129.04516    127.84618    124.79599    121.92261     120.0858    124.86032    125.88547    122.73933    128.32246    130.89849    131.34152    131.45543    134.85301    131.34273     128.4787    125.33648    127.73446    126.05062    126.19468    130.82384    129.93499    127.21174    132.89071    129.24631    130.93901    125.80207    121.33956    122.72437    122.35313    121.98714     125.3768    121.31365    123.25628    121.94292    118.37493    124.75704    125.80268    128.02639     126.2572    129.88413    127.53813    134.61703     133.5731    132.84926    131.58252    134.90691    136.30691    128.99003    139.06625    143.06949    140.29039    140.95538     140.0786    142.71917    139.69391    144.60196    135.67719    134.66548    134.52556    128.25427     130.5831    137.90521    136.79045    135.09346    133.15002    137.45222    132.08678    135.02798    137.38435    138.93979     133.8635    137.42208    138.06785     140.9958    142.31539    144.87142    146.89925    147.53716    149.78016    145.54691    146.19586    147.27557    144.69111    142.33571    139.86723    133.52735    135.24852    137.38319    131.97638    134.12416    134.18657    132.93293    135.06898    137.10977    134.95535    137.49227    139.22286    138.97185    132.72532     134.9309    136.21521    138.21958    138.86167     134.8619    129.33388    130.55144    134.06824    126.84996     131.0168     128.7604    132.57588    130.89238     132.6337    130.01608    129.55145    130.61307    131.61919    132.06866    130.92629    131.23749    130.10171    136.08657    131.24563    127.51262    131.65053    133.38667    134.76786    135.00999    130.60467    132.81564    133.50133    133.21339    129.54967    134.51068    128.53961    129.21755    130.43723     131.5044    131.15912    132.83177    132.22484    133.46115    132.15048    136.44321    141.42636    137.32256    138.59715    139.69224    140.11192     139.8518    143.38775    142.05286    141.32738    141.84999    148.59535    141.52864     144.8393    147.76612    149.48491     148.3814    143.06475    146.46414    150.34168    154.36461    151.95334    150.86281    153.38653    151.88716    154.35287    157.15666    158.02975     159.9139    156.20386    158.80092    159.38677    159.87973    162.10254    161.42249     158.1455    157.93571    154.89775     155.7078    163.54328    159.27695    161.98779    161.95444    160.41831       165.25    163.11119    165.27101    166.99598    167.45791    164.10306    166.31871    170.41586    167.23326    166.28756    168.15475    167.26062    172.56512    170.56869    178.52526    169.63585    177.68203    177.41218    171.63476    175.04872    181.53962    185.07009    185.03807    186.18442    188.38688     183.1045    181.66874    194.85921    186.66587    183.95425    188.88398    185.56554    188.57204    194.32959    194.44978    194.60657    190.79429    201.37318    195.68861    211.85228    207.70741    204.66127    206.38599    209.58706    207.72558    210.91633    215.35625    202.57886    221.94473    205.30532    208.42721    201.07271    217.98181    217.67188     215.0682    221.04383    205.80616    209.37332    220.98342    227.37523    214.03224    218.30828    226.51283    223.29242    233.81122    232.62263    226.91574    236.18772    236.50281    225.65451    226.06146    244.14787    247.51828    255.72984    248.54347    235.73061    246.22454    265.87648    244.17235    243.09634    254.48645    269.09812    270.94421    277.37587    284.88924    291.86576     291.3369    272.13659     283.5804    280.83195     286.2668    291.05218     295.1281    326.60538    309.44476    322.84325     336.4983     359.1672    376.13044    363.86662    373.11329    387.51504    352.95069     480.8555    433.36016    429.10065    453.35841     458.8158    463.93094    685.40856    650.54456    476.59411    495.65761    527.80248    546.29833          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       10.50    262.84351    244.24417    233.05272    227.43589    225.64213    215.61524    213.43024    210.13116    196.42769    199.65548    193.86866    193.86084    190.16501     188.1213    182.85721    175.78117     178.5891    168.49997    177.27835    171.90782    170.43314    162.27363    161.55495    161.06106    175.36409    194.01747    238.30947    248.79425    232.89203    197.48422    167.46541     160.0949    154.39063    156.27888    148.78935    146.08214    148.91184    146.42204    147.36396    145.98444    145.76416    146.35305    144.73125    146.37614    145.47508    142.17222    143.78081    137.11854    140.18951    136.94723    133.11806    139.18265    139.47796    134.55551    138.70768    135.69165    141.18784    144.23513    146.19759    148.10687    137.04775    137.23896    138.28633    139.71345     133.6657    133.26742     132.9859    133.29728    134.13683    143.37273    151.52766      169.258    181.77926    173.14496     157.2948    146.57673    143.72609    156.62386    162.80564     165.5056    165.19338    164.00306    159.05269    153.84666    147.05887    133.62341    127.91508    124.49761    133.54071    141.29293    164.33167    183.14128    182.26157    170.19446    150.49776    136.47335    129.77262    125.96163    127.24286    127.64614    130.11184    132.61342    132.74971    126.67078    123.91391    121.03476    118.26176    121.22655    121.74348    121.58846    120.55932    119.06554    118.05746    120.14557    119.11048     116.1723    120.20913    119.20388    118.97594    117.80731    117.83571    116.40386    120.36699     122.1912    132.46639    135.20984    141.51178    141.74182    136.52294    128.82969    124.05758    119.00691    119.70831    122.24858    121.19052     124.3101    134.68094    149.21547    157.57746    168.61207    172.18746    165.34233    155.66661    144.97842    133.98385     128.9738    127.42028    129.69542    131.35265    129.40791    122.26525    121.72338    118.62608    118.40765    119.03446    116.19827    114.87585     120.5847    117.85745    119.19673    118.43701    117.70981     118.3929    120.81624    119.14182    124.93797    126.19477    125.14723    124.90645     125.0457    122.45226    127.45857    129.78006    132.05858    131.04129    130.14151    132.93976    131.98545    137.27183    133.65151    133.60126    125.39837    126.12153    125.55763    120.63097    126.16884    127.61513    132.51418    132.65732    128.44894    123.74109    123.01438    121.67659    122.53117    123.30942    126.11357    124.36941    129.57115    129.70818    132.04818    131.35218    131.76899    125.69993    125.00268     123.7783    126.79777    123.24487    127.68858    127.12356    124.81818    130.80856    130.53728    126.68621    125.46749    126.17559      120.835    123.41917     120.3528      122.717    123.13754    121.59947    122.71495    120.48544    125.44805      125.105    123.50833    132.05829     127.9841    132.75338    128.06929    133.28351     133.5753    134.40253    130.42358    136.12415    134.12917    140.48923    143.55404    140.24399    139.88339    142.60011    142.94263    140.28976    135.24531    136.41393    135.62991    135.60604    133.90835    130.26345    133.94954    130.99077    134.38167    131.51155    134.49572    132.95858     135.5238    135.47057    131.65543    140.13576    139.01971    139.26606    141.87224    144.26566    147.69068     147.6324    149.86264    151.50123    144.34751    143.81091    146.66597    143.64712    138.26297    134.95159    132.59042    135.35375    133.23262    130.68119    135.48009    139.58388    136.06642    136.51256    137.34648    136.03593    133.24374    137.20405    136.84892    134.55166    133.05431      136.626    133.30538     134.9703    133.94061    131.77536    130.82919    136.34888    133.90394    131.89432    132.03377    132.70116    130.08864    134.54572    132.96177    130.57328    129.85359    131.85843    128.17038    130.66785    129.11039    129.00817    133.40335    132.30401    129.72143    134.93535     135.5768    132.03624    132.03479    132.72942    133.65555    129.97362    135.52987    135.37082    133.17878    130.06639    128.75873    130.14123    129.44774      133.891    132.23993    137.12462    135.43692    135.72704     134.8066    135.91149    139.04226    138.11892    138.25529    135.87108    144.10872      148.821    142.96461    145.01333    146.25159    143.65416    147.46527     144.3876    145.65994     143.8368    145.49023    146.44427    145.37821    146.77545      150.331    149.31686    154.58644    149.31446     156.1032     154.9262    154.36086    157.33296    156.78986    163.96827    160.64234    156.44643    164.53257    168.13762    160.30484    157.70558    158.70665    158.08172    159.48775     161.4305    159.03029    164.27472    161.93036    162.15163    160.25367    163.35498    164.87426    165.40099    166.02574    167.39923       168.53    165.60736    168.34709    172.59166    172.42925    172.77499    173.04818    170.49534    167.32697    160.73082    169.74791    185.92752    175.02399    177.52692    182.88899     185.4288    178.67041    178.28475    179.60991     181.9159    182.31711    184.86729    190.74321    191.55726    189.89878    185.59602    190.13754    199.77937    195.76961    199.59207    195.06745    192.55353    201.11004    203.25717     207.6588    205.78949    209.25823    209.62637    199.54502    207.07946    210.40482    218.09552    204.40663    207.22001    196.61946    213.62707    216.44924    204.60064     218.1096    210.65744     214.6686    209.57752    212.60665    218.96511    222.16374    223.53918    212.17469    217.55602     236.9251    243.01168    235.04847    232.74774    233.05324    236.00973    229.29269    238.98594    236.39501    239.48539    239.12773    250.36721    231.34694    251.30651    247.70107    245.37592    262.62381     252.2955    260.06094    262.80672    283.59325    290.95123        274.3    281.91838    271.83166    278.84672    313.86301     297.8359    324.17814    349.04409    316.03457     331.7205    367.12678    365.77909    357.32007    407.47595    357.43144    356.98526    387.46152    434.20643    426.12958    441.20337    468.77641     401.4024     502.0802    514.54356     532.4636    542.04152    536.63031    606.14523    577.08263          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       11.00    266.72468    243.53468    234.66199    228.58166    225.25174    219.82161    215.32722    213.52808    195.71347    198.04331    196.15458    193.82961    190.51579    188.75258    185.90568    179.05325    183.14892    168.69791    173.88934     171.4957    170.20737    166.56111    162.91513    169.75821    173.26121    193.16041    241.87065     248.7285    235.47217    191.46771    167.00812    161.37947    159.13277    157.46943    151.47635     149.9106    152.40444    151.53073    150.19243    144.71546    146.99398    146.05798    147.32973    145.52475    147.08714    139.05477    140.98129    137.16613     141.2949    138.09415    140.71726    137.19436    140.49554    133.83421    136.16752    139.16693    140.06999    144.79668    147.65657    147.60856    140.97879    137.98547    135.70128    136.36014    131.31475    136.17312    135.91516    132.95659    133.26985    136.46546    154.87017    176.64965    183.05808    175.66537    160.67071    141.11634    150.04913    163.01951    179.29352    179.34431    170.15621    160.15334    156.43355    156.42165     147.5611    137.43158    131.39952    129.41407    132.04283    144.81113    162.35253    180.48792    181.18942    168.95515    147.75833    132.39552    126.33494    128.10515    124.45721    127.52232     132.2775    131.09612    131.37859    127.10875    128.24243    121.68692     117.2023    120.09352    116.81325    123.44791    118.81166    122.28637    117.82489    115.35022    115.90684    116.92834    118.04356     116.9183    120.20345    115.23669    120.42055    117.64425    120.72797    122.61076    129.37251    135.73653    141.57422    142.96366    135.96643    128.99896     124.7783    121.75446    118.11711    122.08122    122.34264    126.59369    133.57863    149.23998    162.47011    168.45182    173.73057    167.80198    155.96355    147.95493    137.64974    131.16437     131.2998    130.18129    127.68816    125.16851    125.69967    121.27127    120.84926    118.66725    121.81997    118.01455    117.21237    115.85645    115.09641    119.27978    118.81879    117.08376    121.05658    122.25439    122.47473    124.86773    127.15704    129.94629    126.22893    123.14338    127.21758    123.83152    128.20632    124.96324    130.53888    131.16573    129.69607    130.89163    136.01904    134.59966    134.47454    130.18756    129.64554    126.93473    118.51895    124.88256    125.38022    133.09591    123.08186     128.7463     118.6887     121.5534    119.72409    120.88343    121.01258    124.24537    126.99786    130.10441    131.50051    134.03936    131.60925    129.46163    127.65601    124.78141    124.15536    125.33264    125.35273    125.20819    124.81259    129.76121     131.8024    127.29949    124.62249     124.0964    124.65331     119.3991    123.32645    124.57247    123.01069    120.19514    127.20697    120.95864    122.42667    125.79037    125.45831    129.08606    128.21716    130.37054    132.06614    130.14688    134.37182    132.30208    138.00754    134.53322    134.55186    133.13215    139.43014    142.39389    140.72637    146.65809    141.40271    142.31275    139.79834    136.69973    139.53408    138.76958     139.9593     132.5598    137.11015    134.36746    135.86296    131.60257    139.34618    137.86134    130.31673    134.01136    134.20971    137.47495    138.12316    137.76638    141.04494    143.54562    141.42276    147.49563     146.3373    146.33373    147.23147    144.83922    146.37629    144.93336    140.47618    139.58396    138.63227    134.38609    132.81964    136.36048    134.26386     131.7481    135.56929    135.63706    132.92209    138.77324    135.68651    137.83632    132.77035    136.99911    131.16976    134.91542    135.29045    133.02211    131.72187    131.83712    132.03151    125.05461    132.21295    131.81678    130.94863    130.71558    135.32492    128.22317    128.66543    136.72058    132.33353     133.1853    130.01189     130.4285    129.58256    130.20971    131.53004    127.74668    131.47944    130.70855    128.43386    135.69649    129.28654    133.17114    135.71792    134.05019    127.61582    135.23761     134.5444    128.17085    128.93218    128.53075    129.27388    130.44488    129.83425    130.84593    130.85337    129.10619    134.28746     133.0038     134.4639    137.61019    139.29648    141.10429     146.2613    143.92845    148.33468    147.22094    151.88417    144.23765    146.96075    144.67777      152.754    149.51699     151.4594    152.45249    144.72894    144.39238    146.03527     148.4862    150.09433    149.61718    150.24815    154.37041    154.77627    159.76015     159.7282     156.1361    155.96555    156.79845    159.83387    158.52747    165.77991    158.28942    157.52145    158.82615    158.95686    151.99999    158.26802    164.16051    163.42055    157.73754    162.63402    161.10759    161.37553    162.11535    165.93525    165.04002    169.99092     166.4313    167.12421    168.61686    171.62152    171.62175    171.82319     169.5856    174.65071    180.44121    173.95643    180.23247    180.11358    167.96221    171.65856    176.79565    188.14216    190.19636    181.28259    176.39786    182.11058    179.32713    192.16473    186.70944    179.25699     193.3475    190.59642    190.86212    194.01489     191.8668    191.40501    196.19645    197.62084    191.60727    205.76823    210.53561    204.61294     201.6874    200.85401    201.63176    205.48339    213.57824    215.16157    208.52148    210.81268    208.19495     215.3108    222.95504    218.81744     212.3627     219.2705     209.0749    215.62899    233.08259     229.8444    207.62617    222.98982    219.62825    215.00306    226.29211    237.81271    237.23419    246.93252     220.2788    225.29486    236.04871    250.50669    229.36217    247.83645    241.80154    240.37317    230.95528    273.80416    258.11882    242.11624    272.96334    260.73318    273.59249    256.30602    294.13797    295.07468    279.88263    289.95628    295.13877    290.82985    329.72565    322.72699    341.00677     319.0603    326.87847    328.42803    378.87514    350.70817     366.2742    381.79608    354.61713     382.9516    353.34545    473.20916    490.81343    397.66239    482.69021    467.12838    448.94482    571.15045    649.36209    612.49407    533.20866    504.33448    622.17289          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       11.50    268.13183    244.98681    236.13829    227.79696     224.4184    223.70622    216.72516    217.54213    195.69036    187.82702    199.25505    190.32941    189.41472     189.8962    183.02039    182.50179     178.9365    173.89932    173.29155    171.73855    169.76299    166.50092    166.55313    173.50416    174.49352    195.09802    239.95728    246.05213    234.87606    195.41898    162.10634    162.12445    155.30149    155.88721    152.92779    146.95631    153.38853    152.56389      149.421    149.48727    144.66034    145.34154    143.09036    143.10461    144.45371    147.17355    144.54092    138.89442    137.64867    139.89877    139.52728    139.62215    137.85879    132.24685    135.44098    134.85708    141.44904    144.32293    147.79875    147.67139    136.88307    142.16449    137.18806    135.55915     132.8377    134.10271    135.71698     133.1114    133.37786    138.27194    152.91939    178.66938    185.95546    175.39398    160.61495    150.99232     153.7769    169.78747    184.45692    184.39803    172.46517    162.67613    152.25818    148.67047    142.85026    134.54104    126.70661    128.77514    131.31392    143.20709    160.59296    177.95598    181.96423    167.65222    151.70285      136.456    127.24392    123.98333    126.41365    127.26851    131.96636    130.45636    128.20303    127.77105    125.99258    119.67654    117.32035    120.76583       118.81    120.93835    119.35387    120.75776     115.4569    117.78139     118.1641     119.2746    120.79369    121.13106     118.6433    116.25784    119.68744     115.8005    117.32351     127.5652    129.53903    135.63399    138.84886    143.25624    137.84216     129.0921    124.30682    120.58577    119.79815    121.74484    121.43488    127.56734    131.32239    150.72323    161.36258    173.98523    178.10621     167.7517    159.89476    144.87548    140.15383    132.20662     134.4708    132.29766    130.16127    129.41527    124.37375    116.38557    119.29502    117.94232    114.31853    117.28648    119.50253    114.88463     116.5672    119.18709    116.80283    120.36412    118.45866    121.01227     119.1539    124.90167    124.42298    127.52474    124.23769    123.82953    121.87417    118.97274    121.81885    124.65398    129.80361    132.75279     131.3492    134.26751    133.40735    131.01539    135.31842    132.91391    128.29421    127.29985    125.56465    125.25022    123.44052    122.04157    120.82753    122.28012    119.48616    119.36768     119.1915    120.82456     123.4226    124.01064    125.71335    126.18319    133.40894    128.79825    132.03877    127.89368    130.65847    125.52754     126.2916    127.78213    126.73012    124.38422    129.87669    128.15747     131.6452    128.30822    130.39269    125.64625    123.20652    123.18242    119.31199    123.50074    122.33649    121.79988    119.41115    121.01877    120.94266    124.17767     126.5546    124.09046     128.8008    129.53118    133.54391    129.61743    131.38845    132.76589    132.13986    131.70584    135.11333    134.71743    135.44136    141.34831    140.15035    140.29972    141.91185    141.60087    140.00447    143.16864    143.07121    142.20208    140.84764    137.14323    135.28162    134.51805    135.77577    131.71059    136.29436    134.62719    135.59547    132.27722    138.72242    139.46447    136.39005    140.02031    139.77625    138.81779    143.19872    144.10701    147.76169    145.34558    150.26821    146.95532    144.97781     143.4966    141.72074    136.98078    138.63833    134.06801    133.23774    133.13505    130.89491    137.03159     137.9029    134.78527    131.68176    137.56391    136.22862     137.5838    137.24587    136.75933    135.91455    132.57714    131.81598    134.09859    131.61986    133.40171    131.26473    131.07946    130.05239    133.50354    130.90118    132.22511    133.03154    130.33024     129.8586    129.68404    127.55748    127.74262    126.94629    131.54333    128.85869    129.02641    128.76907    131.01427    131.31074     127.6356    129.47448     132.4885    129.70402    133.72621    131.38816    134.40752    128.61377    130.83654    131.82046    131.23638    129.75332    133.75065    132.45932    130.66516    129.89792    133.28168    130.12638    132.00052    130.33783    130.43047    134.27708    136.00853    138.40896    139.49991    140.08628    140.68008    140.74743    142.45591    148.61465    147.50475    144.40424    147.90932    142.97616    148.51681    145.34764    144.67608    143.15526    146.84928    145.79534    150.11161    147.93636    145.05108    151.98669    155.68577    153.71833    150.31654    159.73848    157.02449      157.697    157.90391    159.17348     156.9363     162.5727    156.08829    157.87933    153.97692    159.65937    156.91442    160.59462     160.0862    163.00123    159.37593    158.00536    166.70009    154.02855    168.75428    170.52064    171.34897     169.3218    164.23893    163.71549    172.70828    170.66378    172.86091    174.90603    173.04229    168.73028    172.60617     176.2632    178.20061    166.20857    177.99776    171.85613     173.6854    184.85677     188.4989    182.25348    182.29547    173.61919    181.85758    184.66746    192.99624    187.74712    183.95457    189.67132    187.28758    195.24792    196.04329    199.10738    201.86404    194.23134    196.46745    200.98078    205.51697    200.58372    198.60556     203.6406    202.89077    202.26446    204.66803    208.77889    206.40711    203.06846    206.87136    219.51043    215.76161    212.52395    216.23335    201.93765    211.57331    207.41611    212.46525    207.07809    222.96622    208.85786     217.3183     215.9449    229.10239    241.85041    238.03812    235.78661    228.33334    236.30733    232.89798    242.57704     233.1188    249.78431     249.1052    228.33742    252.29658    259.45423    270.04942    267.96769    262.59805    251.01558    269.08542    276.24919    269.93826    316.08216    295.31321    293.32358    295.64298    288.89807    317.88125    303.36987    304.20579    315.97798    318.00714    324.02902    351.28773    422.87033    369.36663    371.04594    361.29408    359.89767    375.74781    481.75197    399.43204    461.47688    415.36639    450.42017    469.88232    434.10195       591.56    575.44002    563.69514    648.16336    520.50601          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       12.00    267.31766    247.60562    238.93964    226.35046    223.54224    226.28021    219.11342    217.71094    194.63359    183.45707    200.88488    188.05231    187.50189    190.87114    180.72391    184.65056    175.25511    170.97358    174.44337     174.6905    168.55219    165.89201    167.04886     167.4438    175.74427    196.63305    236.74632    249.99786    235.70905     193.1634     168.0959    160.94486     154.8158    156.49423    152.62051    147.87477    151.35761    149.65581     145.9243    149.26241    142.71594    145.88183    142.15373    146.28554    145.84457    141.02092     145.6355      137.729    138.75122      134.272    143.50383    137.97594    140.16232    139.41471    132.82142    136.31711    143.96319    147.39769    148.30579    151.01135    141.94171    137.15451    136.30361    135.63525    133.60968    134.36829    135.60987    131.81392    133.90326     140.3141    152.12049    171.29748    181.90446    175.48062    155.65708    152.28393    156.54706    171.12383    184.40739    182.22582    169.75387    157.42714    150.76443    146.04337    141.34049    135.35584    129.05214    129.89048    132.61714    144.34506     163.3869    180.72198    187.39756    171.01209    149.71495    137.38957    128.29857    125.13152    124.82391    123.92608     131.4719    129.22247    132.08829    131.31313    125.70463    123.59382    119.11759    120.25076    121.77768    122.05821    120.74539    118.49128    118.83429    112.93453    116.76266    120.76363       121.12    121.54485    115.93186    114.89501    116.99648    118.59476    117.65236    126.53193    132.46523    138.12324    140.35291    137.96338    134.92689    129.26397    125.40776    121.33001    117.30707    120.70884    121.72292    124.38504    134.61494    147.35236    161.73354    170.53094    172.45042    169.76741    157.56424    145.72649    138.49786    129.01512     131.7323    132.75616    130.57868    126.59563    121.78566    121.52389    118.35608    115.87317    115.84575    115.04921     116.0085    116.77093    116.82841    120.40014    118.52052    119.29828    118.89039    119.33337    124.86937    122.68161    127.81057    127.57433    122.98642    123.93105    119.90864    118.35009    122.10187    126.94583    128.54264    128.64523    133.02596    133.90143    133.73473    133.15167    131.19771    132.11456    135.11197    130.67187    125.43675    125.63991    123.07676    121.88042    120.17588    122.00406    119.83706    123.87657    120.82311    123.49919    121.16669    124.29543    127.91312    125.91351    129.38006    130.38482     131.2751      130.819    132.23199    124.69924    123.52378    126.77358    124.82291    127.72021    123.28157    128.98695    125.38517    130.57696    127.59791    122.85173    124.35085    124.01359    119.51409    124.04301    118.10728    121.07283    120.74696    118.83173    123.18575    119.07647    129.12329    129.09926    127.51586     129.1174    129.86664    132.02855    131.87335    130.87415    133.74227    131.55585    134.22007    139.65012    135.14042    138.42236    141.90696    145.59167    146.43034    145.41148    141.92675    147.75999    140.97026     143.3338    136.58704    137.19787    135.32941    129.28802    134.44628     132.4357    134.50732    138.36473    137.12891    135.20202    134.74418    135.52261    136.43586    136.93874    138.40673    141.21105    139.79242    149.36521    146.18043    147.41013    147.91626    144.85147    143.89838    140.97723    142.26653    137.75521    139.89157    135.50259    129.33484    132.43606    135.68938     129.9106    133.38805    134.51303    135.64385    138.06712    138.26108    136.20411    140.61691    135.58977    137.98871    131.39756    133.42921    135.40542    135.03165    132.15313    130.20109    130.05322    135.46705    133.90257    132.00478     134.5779    133.70116    132.42315    129.10959    129.80336    132.14738    134.45777    127.93459    128.37029    128.13922    131.79568    129.78716    128.74128    131.09296     130.6238    131.56235    134.24727    132.49043    133.96658    130.94901    133.11162    133.16222    130.82613    130.40367    129.78161    130.31322    128.67645    128.71524     131.0584      129.978    131.22068    130.97773    128.76469    133.80499    134.58266    136.79415    141.46571     140.4896    138.58359    139.41576    141.59662    141.00592    143.88521    140.50433    144.62896    145.76554    144.84842    144.08882    143.83382      143.095    144.05726    149.84956    142.89157    146.78741    150.09212    147.72901    151.69163    146.06171    156.54451    146.44804    153.98599    157.62074    157.34862    159.74873     157.8909    157.38377     160.7491    162.01884    161.34065    155.56461    154.81674    155.16095    154.11923    160.82633    161.41622    164.02855    162.85343    158.81956    158.21595    159.53449    164.39931    164.21008    175.84774    158.69497     169.8437    164.15542     171.6648     172.7018    169.10492    165.04142    172.18887    170.66558    174.22809    178.63217    175.65755    175.30109    182.34779    177.83557    181.20491    184.36426    180.11862    181.63806    174.53298    178.41418    187.01045    185.33699    191.63845    189.38926    182.24087    184.66635    192.94154    202.43272    201.15465     196.0086    200.92564    193.68235    198.01634    203.73699     201.5727    194.42733    202.34588    204.25459     203.4474    201.04151    215.30893    210.43999    204.21497    208.38367    206.37611    221.31962    227.08203    210.00414    214.37656    214.47641    203.26876    212.47364    224.38699    217.93689    219.95392    206.12851    220.51293    223.66745    234.39423     222.1333     234.7106    231.15618    235.90496    233.71388    231.22915    238.57718    224.88655    221.67743    251.20048    252.77624    250.33873    265.45637    272.18866     261.1744     252.2922    263.88351    275.65568    269.22524    270.22448    293.13982    302.53802    291.62269    269.63355    286.03636    307.15536    311.98172     328.4441    331.94415     341.3506    294.42025     379.9064     343.1204    373.21889    351.97185    392.04894    398.90434    386.80476    464.98712    421.18897    421.55573    461.82111    452.34168    526.05728    640.90036    634.81465    592.21234    556.86893     516.0515    459.30111          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       12.50    266.99507    247.19923    242.35579    225.69583    223.45945    229.15116    223.05046     217.3584    193.53607    188.81926    199.20773    192.90239    186.93236     186.5805    178.35398    184.00826    176.63487    171.69046    172.37378    179.34721     171.7043    164.71113    166.28052    166.52424    175.61559    197.99108    238.78873    250.81229    233.22484    194.68254    162.69341    161.84683    150.81623    156.34873    152.27489    149.34187    153.55438    149.61829    146.80494    149.91024    144.94399    144.09121    145.43484    143.95219    146.38629    142.99136    144.40533    138.30879    141.63372    134.00116    137.38313    139.18783    141.58016    138.87088    133.60938    138.57405    144.47468     142.6886    150.23472    147.94978    142.85153    135.80344    136.42969    135.32927    134.84105    135.68967    131.53275    134.70441    134.11886     139.2179    157.00714    175.46696    181.23013    180.06733    162.02152    153.80993    156.45364    168.37991    176.79722    176.61384    159.15151    153.50626    153.64469    145.19482    143.28428     134.6879    128.28766    129.46503    136.87463    148.31423    169.15993    182.14878    185.96434    169.43761    152.76775    133.51599    129.07986    129.11858    124.55183    126.36591    128.23269    129.91851    130.04362    132.28736    124.48636    123.11378    118.70347    122.09779    115.69447    120.96481    121.13615    122.46498    119.18378    115.50003    117.03594    120.48819    118.71728    118.26803    119.46973    117.51921    117.21582    120.08036    121.54183     124.2781    130.54983    137.30838    140.17524    138.68377    133.71956    129.91806    125.79633    119.71779    120.80969    121.40335    119.88031    126.48892    135.37033    146.73661    158.84366    173.45722    173.83067    166.34069     159.9544    145.43682    134.11714    130.72441    130.03522    131.39551    128.24431    127.32388    124.54024    122.23629    117.61903    117.44906    118.91719    117.79674    117.73517    117.29725    115.55305     118.1963    117.36022    118.34198    116.45674    122.29329    123.26708     121.6695    126.92931     125.0714    126.45953    122.18885    123.35294    120.58396    120.14448    126.63069    127.00323    127.79785    131.86828    136.14746    131.28094    138.61172     135.8417    136.02286    130.96819    128.53856    127.97143    123.44621    120.23032    122.18686     120.5732    118.81784    120.95793    119.90194    122.90962    120.99289    123.03286    124.71639    126.00295    121.59651    129.80055    130.94308    130.44485    128.96591    128.63209    127.80935    122.73657    126.01225    128.65138      127.875    127.69983    125.42732    126.97715    129.81409    125.27513    125.02625    124.28562    120.08093    121.49707    121.29557     122.7156    120.48809    121.35611    122.34179    123.05249    121.42085    124.65399    127.84366    128.96176    131.19429    129.00244     128.7044    129.23287    131.92834    131.75744    134.76402    136.47481    139.32861    137.57086    143.98944    144.88673    143.57041    142.33116    145.90933    140.77153    139.36886    141.49681    135.92226    137.43313    132.77272    133.43529    128.73401    132.94615    133.84173    135.29725    137.27101    138.13683    136.99402    134.41486    139.64675    136.20091    139.41282    140.51257    139.96052    141.50061     146.0424    147.16904    152.88832    144.05804    145.29033    141.62418    144.30756    143.05423    141.46337    135.88496    134.23578    129.44435      134.836     135.5995    134.87681    132.76429    133.61426    136.14108    136.18802    139.53775    136.46233    136.45697    134.57597    137.43799    132.92581    133.55623    130.10402    134.37352    131.89239    132.64247    133.93202    134.74454    137.37948    133.79761    138.65893    135.39531    134.67391    127.84992    131.33028    130.30412     136.4456    131.38272    131.47155    128.36405    129.71742     130.3029    129.69446       134.39    132.54631    129.33968    131.09759    134.05651    130.85553    134.35089    132.21375    130.84394    129.35808     131.7243     135.2647    130.55217    130.18794    126.48449     127.9468    129.83529    129.68745    133.69044    128.29757    130.22979    133.36653    132.56673    137.08981    137.38105    138.89471    137.84048    139.68985    134.58811    145.01033    139.75131    138.85754    143.46521     142.9524    142.22749      144.886     145.7014     149.5556    145.05127    144.56556    144.70804    152.17024     154.1821    152.34865    154.01926    148.28414    149.67671    153.76087    158.87733    152.23316    161.72951    157.76124     157.3909    155.95713     161.2623    159.95592    157.15567    162.64745    161.24321    159.87328    160.01751    162.84866    159.74589    158.96142    162.83969    160.65806    159.97611    164.37041      166.472    163.97198    168.05952    163.59911    166.40183    169.49032    170.80449    166.12637    168.95233    170.14036    168.29515    170.38686    172.44554    176.41483     177.4652    175.61986     170.5277    177.42716    183.08742    176.72513    179.74635    189.16758    189.82582    188.72408    187.15578     185.1304    178.10604    187.33056    188.61333    182.72785     191.8932    195.48982    194.53577    201.87173    202.51517    189.90584    204.65003     198.3582    204.87787    208.26434    195.94839    203.55708    201.77472    211.22267     210.2815    208.44184    215.66806    222.74636    215.20308    221.20768    210.76478    207.11383    214.64386    216.21314    212.28686    211.06019    217.83515    220.27399    221.96862    220.02343    215.74206    226.20631    232.34145    226.28161     246.3381    231.74928    220.94491    239.55191    245.08491    251.19428    246.46445    244.17704     251.4618    227.93508    260.54117    261.49508    259.69826    267.07452    265.45318    248.18658    271.91397    286.43277    300.55846    285.78958    294.36908    267.97898    286.24006    296.88733    276.53649    346.06369    332.27449    336.45315    325.55755    376.83836    364.66452    350.74927    371.36928    365.95347    354.08349    391.35091    432.87712    440.97726    436.43547     488.8003    455.77781    470.89935     599.5718    649.41418    612.12335    469.71558    548.07481          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       13.00    266.93051    244.79481     244.3442    227.61522    223.90007    228.92055    226.23592    218.85577    194.52256      193.193    194.78062    196.83999    191.66833    183.72261    180.42023    181.90661     177.9762       176.31    172.04697    178.15281    173.35514    165.80885    167.44145    173.22599    175.67977    197.97114    241.90937    250.36274    231.39322    199.01717     165.2225    159.45999    153.55067    161.30309    151.77098    148.68815    151.42389    148.77505    149.23653    152.08085    145.52664    145.40456    145.92374    141.40348    143.37548      143.547    148.96543    138.20495    141.71231    141.40792    137.54227    137.72434    138.56435     138.1401    135.77919    139.33026     140.5882    146.21315    147.04054    147.71432    147.08942    138.31843    136.43898    138.31515    133.48422    134.47028     134.9845    137.14508     135.1463    137.37443    159.37533    173.64068    177.40307    177.47743    158.92625    152.05539    162.19187    173.58688    172.18074    170.04945    159.34382    147.59894     146.3669    147.11061     137.9815    133.26655    130.22436    129.10687     136.1198    146.14891    165.60752    183.01928    186.29651    174.90253    153.80133    136.35078    127.29351    129.21584    124.20131    123.40344    124.96652    129.15727    131.85051    127.70549    125.66303    121.43588    119.31822    118.64581     118.8678    121.33386    120.14475    120.28147    118.26352    116.17103    117.51856    117.23945    119.06284     117.8264    118.45879     121.8032    118.39199    119.05828    120.77069    125.53197    127.85061    136.50212    135.86492    135.01893    133.88439    130.81613    124.22517    120.49827    120.74303    119.96466    125.94881     125.5602    133.41841    146.19389    159.97396    167.19804    173.33001    167.71013    156.69481    147.05576    135.69395    134.27761    131.44975    131.54587    133.67022    131.72877    128.16895    121.31584    119.09219    122.06319     119.5652    120.07068    117.69122    116.31563    114.48402    114.32202    118.29649    119.88945    118.01542    119.00366    124.42701    124.28957    124.27768    128.41335    127.19257     122.5494    125.55713    120.69945     119.3159    122.45074    130.29647    136.28585    134.38165    136.96684    136.54524    137.82656    135.48828    136.22684    131.14086    130.93015    126.63276    123.13157    122.11453    121.15945    122.36011    119.39525    119.61836     122.1148    122.27108    122.61395    123.80793    128.06998    125.84994     126.2105    127.25055     128.7907    130.78262    132.65881    129.84519    128.06627    121.09018    125.69903    129.41134    129.05822    128.27024    129.47851    127.21571    128.03953     129.9964     122.5186    124.85076     117.7049    119.17497    119.06781    120.35915    122.61641     124.4207    121.24278    122.80608    123.23171    125.07059    127.01612     126.2446    124.69859     128.2372    127.37119    130.83438    133.60124    134.15147    136.27163    132.39024     136.5991     139.0611    145.84884    141.79624    145.42486    141.87956    142.10271     140.4456    134.23971    136.94967    136.54696    135.82871    133.40588     132.2482    129.19502    132.12798    135.36068    132.44947    137.03472    133.93569    134.29977    135.25665    136.76046    134.18375    137.70545    140.06592    139.29587    139.08739    143.43343    144.31652      146.244    150.47906    147.96055    143.92173    142.26419    139.87569    139.80453    136.00233    140.05655    133.79983    134.59797    134.40366      136.221     135.6775    135.47154    134.94412    134.41732     140.3626    134.49217    137.40603    136.46926    137.68133    135.90485    133.33748    133.71523    134.80278     131.0824    133.67203     132.4869    134.18164    133.71609    136.50485    136.51278    134.52378    132.54948    130.62693    134.62074    130.83032    125.95234    131.60355    130.01393    129.15557    128.63752    129.03481    133.84473    130.42652    135.10832    128.99376    130.36388    132.22098    133.91574    130.99555    129.77415    129.31236    132.52552    129.70314     131.8499      130.122    130.38818     129.0093     126.4126    129.83733    129.80412    132.47905    134.26987    133.00232    133.50535    136.08182     138.0619    138.99406    141.08332    143.81259    140.57517    140.09328    141.05231    144.75788    141.35392    137.90483    141.46061    140.47839    146.86584    145.21993    141.94516    151.76288    148.77011    148.69574    150.92395     154.7191    155.84486     152.8861    153.70703    157.69781    151.56749    157.61247    160.85333    159.14923    151.84829    152.93154    157.51819    160.07547    158.88449    159.86188    153.92361    152.00159    156.11308    155.19461    160.10331    159.48364    163.05911    162.07369     158.6802    165.97535    169.64588     166.5036    168.99728      168.681    164.79876    165.01863    170.68706    169.85069     171.1754    170.11489    165.78569    166.40035    175.46347    172.46236    179.39181    174.08536    173.45737    172.51298    175.43785    178.36896    185.60276    184.95668     182.6846    181.37696    179.28421     194.2737     186.1976    193.02384    195.62978    192.25559    193.25556    198.64341    194.62241    195.48597    198.17107    197.33023     190.4409    201.22966    198.84991    198.07782    196.14815    204.58581    200.18628     195.5015    209.99027    206.69766    202.15832    210.28678     207.9904    209.62143    221.96385     215.3189    220.63128     210.9369    211.63313    215.91473    222.57173    222.52716    209.36152    223.56986    216.64418    220.54462    232.45499    239.21482    226.70905    235.37874    224.21668    224.81341    231.95262    252.97515    244.68795    223.38151    239.54141    242.10335    240.39947    260.38439    276.04669    252.39313     262.7082    267.83962    263.55334    277.50847    292.51845    293.98209     277.6669    282.64796     291.3482    279.99544    318.67264    313.51822    330.47955    329.13459    289.91049     299.6574    347.89052    378.80304    362.40847    371.60699    361.79743    384.35186    420.30162    469.99451    486.05406    405.76521    508.09287    428.37062     571.3815    513.95581    589.11532    515.83859    645.12241    580.88345          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       13.50     265.6498    241.32628    240.03693    230.41653    220.57042    225.68485    224.25865    220.70723    197.82968    197.03829    191.93266    196.89972    194.23265    185.36333     182.7672    180.62117      178.147    177.58089    174.29262    178.26827     173.9021    166.21741    163.00134    177.47059    179.82817    198.73639    241.98937    250.29925    234.53049    196.26543    167.14282    158.41086    158.23332    157.80875    152.06604    147.63484    152.19504    150.11321    144.75916    151.90634    149.70693    142.37957    144.43491    145.50752    146.28127    146.49149    148.83675    141.92702    141.57964    140.18866     139.3182    138.16536    135.53368    140.38257    135.71002    136.44647     144.2129    142.04626    151.39927    149.24591    144.06157    138.73614    136.59187    137.56162    132.81073    138.00488    131.65911    135.08103    137.01395    139.52768    156.12241     173.9017    183.43515    178.13829    160.04873    151.95384    161.91906    172.96309    179.23968    167.19213     155.7662     147.9741    142.89782    137.78117    132.98349    134.90556    130.73263    126.12477    136.07736     145.7776    164.79262    182.83606     184.4332    171.96939    153.15262    136.32786     128.5616    126.18537    125.00465    125.78141    127.26373    127.84093    128.24095    126.89567    123.41131    121.57446    121.56514    115.50255    122.41561    117.56589    120.22721    120.64452    120.84767    115.60941    119.05588    118.66891    122.09483     119.4067    118.26728    119.46384    116.25511    114.99285    121.00066    124.40334    133.73678    140.15038    141.89857    140.17835    134.77321    127.24347    126.06313    122.90485    117.18096    117.39355    122.00524     123.7563    133.50063    143.15038    161.78516    167.24361    173.91515    170.64838    156.58496    145.48871    138.51184    135.65717    126.48874     132.2748    136.07558     131.1908    125.67543    123.14527    118.12064    118.57564    117.39698    119.21305    119.23195    115.92647    116.10981    117.10695    116.86144    120.88523    119.24815    121.05668      122.534    125.47746    123.73162    127.40021    127.73905     124.6752    119.83528    125.42209    122.76137    127.18703    130.98322    132.18974    131.34542    135.77154    135.57119    134.70406    141.18967     134.2592    129.04006    128.25313    126.02541    119.25969    121.58437    122.16441    120.41948    120.22285    120.39532    116.71037    119.98402    122.96254    121.91214    124.65622    125.48018    130.38145    128.06073    128.81966    132.60992    129.42063    132.83832    128.24096    124.19055    125.37898    125.71721    128.14738    129.16278    127.89754    130.70362     126.5058    125.51762    125.04733    123.08912    122.98879    121.93174     120.3706     120.8142    120.02018     121.2738    121.28314    123.22622    122.77945    122.68017    125.86071    128.98779    127.50539    132.30372     128.6114    128.04126    130.50348    132.21563    131.77957     130.1929     134.9294    137.25173    136.55115    138.79942    139.41499    137.66337    139.89783    139.76828     136.6837    139.68523    136.48866    136.65147    134.38451    131.38697     133.1927    127.69304    127.79916    134.16503    135.41677    136.79811    135.22004    136.41121    139.86679    135.64586    139.87011    139.50806    138.60315    140.49325    142.84971     147.8825    147.57214    146.36348    149.10541    144.46555    145.78455    145.20018    139.01135    135.24787    137.02514     133.7385    131.01748    135.01864    137.39062    137.77044    139.06237    138.80541    131.58847    138.79957    136.80561    135.02301    138.64323     139.2159    133.94973    133.99059    134.94848    131.67034    131.98906    131.60524    131.83651    126.22712    130.60396    130.02041    129.37974    129.61006    132.00144    127.43164    129.94153    126.24886    132.10701     133.6685    129.37664    130.01046    126.86046    127.62556    130.06028    134.65556    128.65501    131.55214    129.41853    133.30787    131.74132     134.4723      131.318    129.77549    127.46695    129.32161     131.4632    126.55981    129.60346    130.24876    127.55191    131.16875    133.58378    129.57536    131.23874    130.33689    134.25427    136.11834     139.1098    140.15444    138.15112    141.51317    138.72369    138.58589    143.67871    141.10495    140.77144    142.13512    142.61426    137.07936    147.79992    141.78986    142.10645    144.34139    141.08687    145.71561    147.81428    145.55761    152.33242    152.94454    150.66458    154.28593    148.63622    153.22796    156.19918    157.00942    157.62022    157.98023    159.71264    161.36449    158.79923    158.50964    156.40555    159.12741    160.08046     163.0123    149.32556    159.39647     160.1586     158.9341      159.343     159.5596     164.8186     166.4391    165.61541    159.85038    166.74828    162.60686    167.17563    167.65759     168.9648    166.49006    169.60949    168.69137    165.65688    170.49646    183.35296    173.66992    176.83615    172.99931    178.02865    182.41989    186.01815    185.39248    180.56955    178.14981    180.97467    192.15482    185.07532     192.4633    178.54797    182.55513    185.36591    198.92216    193.78264    190.92367    195.70042    196.41589    198.87732    190.52922    202.32162    197.56625    199.57231    205.65184    197.28095    195.73309    206.59554    212.69142    208.50309    220.20038    206.81306    210.18623    223.67478    211.78848    211.54266    218.79286    207.28096    206.55321    218.05338     226.7372    223.38752    217.12144     225.1774    221.11719    230.69102    232.74029    227.21588    222.37676    239.10553     230.2037    230.56343    234.65691    237.52911    244.94832    238.83042    251.02338    249.41704    257.52455    256.71282    244.06365    261.34651     253.8328    244.27219    291.61129    287.26543     285.7587    277.10754    279.79282    313.97127     293.5974    313.74766     339.6648    314.59929    330.76292     325.2504    333.99249    359.05107    361.16093    388.73282    344.08447    353.48808    352.16559    400.13564    458.90682    433.28122    424.99621    437.60358      391.966    614.99828    636.08808    624.15998    627.22464    573.29113    603.33918          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       14.00    262.85243    241.76876    235.15231    232.74718    217.61132    222.56309    220.93855    217.74754    201.18928    199.68952     190.4903    194.75318    191.89389      185.671    177.78282    177.53891    183.59969    176.26271    172.53589    179.22337    172.06072    167.63713    163.18028    171.76419    175.40614    204.89534    242.39904    251.14704    235.76119    198.33634    167.03714    162.81853    158.71263    155.51011    154.52769    151.41736    149.98617    150.78854    147.53126    151.50587    151.08328    143.15872    140.66585    146.39263    146.67091    142.96678    145.80873    142.13549    132.19027    143.18747    133.46975    137.14647    139.57344    138.30614    135.97679    137.32829    143.38519     144.5848    146.98887    149.73742    145.34228    143.76315    136.38823    137.94372    134.20981     136.8389    133.92976    134.14547    132.21152    139.90631    157.86095    174.82943    183.66224    181.47041    162.69516    155.60997    169.28263    187.51056    192.69273    182.60813    163.86847    150.62581    144.72472     145.0481    140.00204    134.83781    132.39686    128.14605    134.78785    145.86793    165.15154    182.31702    185.12622    172.58739    152.84709     135.0223    128.05477    125.12602    124.51222    126.31692    123.26377    131.37227    130.82599    128.56764    125.26752    122.63202    119.82245    115.90671    119.41955    120.12895    117.99582    118.24475     117.0662    114.83967    117.28735    113.59658    119.95737    118.74281    115.75186    118.93982    116.13757     117.2968    120.18085    126.54705    131.13244    137.17964    138.46371    138.43767    134.10542    129.28137    126.18072    119.73006    122.88589    118.33455    122.46271    128.67558    133.91963    143.48436     159.2659    167.09815    170.91441     165.5509    155.58918    146.12383    136.49577    134.71176    130.11139    133.56728      134.095    129.51893    128.23404    128.09095    118.16013    116.98264    118.58818    118.23621    114.53601    118.41122    117.86451    114.44579    118.86217    119.83606    120.70893    119.63296    122.89147    125.51784    121.57017    131.51845    124.76226    126.73131    125.16377    122.81665    123.84119    125.73503    128.92651    131.82952    135.19998    136.97824    137.71603    134.22559    131.98065    130.62374     130.2105    125.80679    125.35808    123.03893    120.19401    122.81372    118.96317    119.28388      121.479    123.13963    119.46863    118.87716    121.06724    121.17722    126.95476    127.52619    126.86999    131.87136    129.93495    131.00962    130.38065    126.47543    120.76849    123.56606    124.57704    126.84526    126.81578    128.43457    125.61291    125.95275    127.62687    128.38457     123.0176    121.37268    120.60353    118.11797    121.94514    120.44458    122.55387    121.20676     124.2287    123.29829    121.95444    126.02961    130.32036    132.41105     127.2955    128.66809    126.94195    131.46322    132.09506     132.8162    133.97289    134.66321      137.284    136.78188    140.39936    140.41814    140.77534    138.22761    137.47192    136.21251    134.47228    136.45403    134.91172    134.18395    137.91379    134.99763    128.67646    129.78789    133.48783    134.74891    132.20105    133.50223     135.5499    136.39985    133.49829    137.60749    140.11168    141.49856    139.93949    141.05235    146.56464    146.43793    146.60756    146.89575    145.02888    143.17686    139.94624    139.03596     136.5904    135.60863    133.05789    133.00982    134.55127    133.57859    137.41182    138.50076    135.30781    136.27769    137.70729    134.89821    132.63843    140.28691    130.38448    131.08922    132.25756    129.25616    132.51091    131.64845      134.518    135.33577     128.6136    130.06805    134.16427    130.72766    133.61349    129.27057    131.86926    130.26502    133.32276    130.92712    127.36044    129.45136    130.11374    126.24382    129.92426    131.28299    127.80441    128.59314    130.51309    127.59388    128.02596    131.38499    132.68395     126.9928    129.99149    127.52622    128.27456     133.5244     127.4866    130.59225    129.75215    130.12006    128.79799    130.34177    130.65308    136.21539    136.46943    133.92147    135.99218    138.28445     138.4425    141.59379    136.43867    140.62331    142.43205    143.13373    144.21665    141.86018    140.64253    135.97271     141.7241    138.82385    143.97428    148.08319    139.47648    144.23866    143.96695    143.43502    152.16712    146.71075    149.98636    149.07716    149.01283    151.82427    153.25239    155.75834    156.70444    152.23022    159.77583    158.12061    161.09478    158.85023    158.96578    158.76941    161.49003    157.55277    161.69343    165.16829    162.25882    159.52257    159.98472    156.97144    161.83907    163.87835    162.99148    166.40673     167.9182    166.29394    160.88593    167.23805    170.71231    166.40563    166.16832    173.26393    164.18112    172.99044    168.23564    173.75016    178.68661    171.05323    174.47386    172.69157     175.8718    180.25709    183.56381     182.9997    179.01002    179.71431     189.0819    187.20625    189.50454    183.64017     187.8305    184.81761    196.40864    190.70983    196.00666    192.02709    195.21734    191.21089    196.20985    213.04432    206.82005    200.80314     197.3879    198.76365     204.1398    210.58231    223.45104    210.32599    213.72271    201.43983    206.19892    216.39828    206.42957    224.73214    206.89215    217.99575    220.11142    223.38694    216.53031    220.62619    218.49726    227.81181    216.80059     232.3435    214.24341    235.41925    248.89293    236.67792    230.93267    230.23325    242.15059    257.77129    231.44844    246.48469     245.8085    237.40884    261.78925    260.24756     259.7949    264.66739    265.62447    262.16092    292.30264    306.77634    264.43495    298.79523    273.67877    284.81016    291.47787    301.45134    318.79055    298.65395    295.17614      298.098    324.12847    362.71968    327.78891    350.69329    347.57254    379.83334    370.21776    449.81196    406.04187    387.97976    507.49948    408.10257    515.00357    558.09619    577.25008     610.0084    663.38404     489.9474    545.04792          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       14.50    263.65846     243.6321    234.48864    234.94214    219.35832    220.56082    217.96018    212.81914    203.64732    198.91768    191.55704    190.92561    189.78631    189.07201    176.46097    173.13504    184.00486    174.96851    172.27859     174.4029    173.46622    172.63578    165.07379    171.12359    174.79302    203.67362    243.64888    256.82025    231.38345    197.12373    166.77879     157.8287    153.83056    154.78158    150.38641    147.09706    148.04048    150.34177    149.74647     149.2066    150.67855    144.88632    145.93405     147.0799    145.49933     139.7132    143.71916    141.73963    139.02751     139.2751     139.2265    141.10234    140.29225     141.0899    135.23696    136.72228    140.93616    146.93774    148.74252    147.36734    145.28376    139.17814    138.99422    137.02511    135.48708     134.6613    130.26809    133.91549    139.29717    142.88017     160.1533    175.14593     184.7103    183.09576    162.59141    160.36336     173.2828    201.52015      215.545    204.01243    180.53889    161.31337     148.3692    142.60971    137.02483      135.949    133.12443    133.19468    136.64227    145.09996    163.02909     182.7766    186.08678    173.46697    153.05433    138.24615    130.15877    128.25751    127.61709    127.36963    126.99105    131.23228    129.80916    130.13263    123.41762    122.29916    118.26369    115.00933    119.56611    121.67595    121.05871    123.11068    118.55702    116.96134    115.21333    117.68504    119.01945    120.24105    119.98309    118.46684    115.77374    116.84915    121.14492    124.90161    128.20597     136.9947    141.21572    136.28043    134.24093    128.63447    123.27711     121.2078    119.11482    118.44317    118.87841    126.38716    136.98159    147.40177    158.82371    170.96577    169.78657    165.51754    159.56627    142.87637    135.47556    133.81198    127.17045     131.2235    132.13547    129.63482    125.99759    122.35939    124.48441    115.54279    116.57527    117.81821    116.66532    112.10917    116.50362     118.3907    117.54401    120.86769    116.02108    120.54622    119.17915     124.4997    125.29833    130.47532     130.7993    130.73519    131.41181    125.45676    125.80942    128.26514    128.47619     133.3336    135.02687    133.12001    133.95938    131.76733    133.16805    131.09972    126.99004    123.70892    125.03286    121.24898    120.89044    116.62398      121.326    120.83769    118.23892    122.77607    119.06964    122.04305    121.39181    124.08023    127.39362    126.25814    129.38387    132.94841    128.92812    128.29081    129.36612    128.29716    124.82458     122.2134    123.18253    125.41415    131.44583    125.74744    129.58385    127.72686     126.6836    126.42556    121.62649    121.15006    122.85622    118.60296    118.36052    118.06728     124.8139    120.75497    124.12064     123.0745    124.33625    125.85021    124.42441    127.50156    130.09776    129.34953    130.83699    132.52038    129.07932     133.6371    131.54524    133.50405    132.62499    137.54319    137.56258    140.79438    139.78033    138.54051    137.33098    135.82307    132.17931    137.31074    135.88276    134.37221    133.70685    136.98727    134.07703     138.0294     134.9471    135.90727    133.99592    136.42957    131.73668    136.68138    136.09493    137.82423    138.02808     136.6363    141.38136    143.35506    143.37296    144.62469    146.72621    145.88218    141.99264    137.40246    139.41057    136.28803    137.30344    135.30292    133.54634    130.95766    131.91513    132.58182    134.81817    135.12813    133.77205    136.36513    133.36228    136.38348    137.84892    138.98555    135.93306      133.046    132.59823    132.04821     131.4935    131.69599    131.36639    128.64419    130.41879    130.17912    129.23461    131.35338    130.72349     128.9374    132.95908    124.07841     128.1343    126.81566    127.20571    127.72677    128.04132    126.99189    133.37411    130.21852    130.22292    129.48966    128.49379    124.43167    130.29313    131.54981    131.57049     130.1076    130.70519    129.28162    130.34695    132.54062    130.26038    131.62573    129.56542    127.42499    130.93674    133.70511    133.17946    132.65212    135.80443    133.36127    132.03939    137.94228    136.58917    135.54137     138.9686    137.09071    135.87486    141.01251    138.66207    141.27353    140.19706    142.43007    143.43874    139.82926    144.63372    142.33515    146.19875    141.52542    141.85833    144.53908    148.73991    146.84631    150.16971    150.69824    147.43085    152.45334    155.91419     157.7829    157.33098    155.49247    154.57142     153.2311    158.65795    163.27767    159.91082    157.36678    158.87631    158.25891    164.63727    158.73906    161.04562    161.75374    160.02618    157.03255    163.13845    155.42892    164.02036     160.4116    167.02453     155.3904    166.73616    168.58531    169.35182    169.73644    165.54208    166.95103     169.3625    170.92994    179.98645    168.95267    168.67031    174.28714     173.1809    170.28667    174.64302    177.86032    175.46187    177.50878    178.88359    179.01875    188.93311    189.16911    187.81173    182.61236    189.83221    186.68891    189.12759    193.37268     193.4019    193.34333    196.12546    200.28216    193.12222    201.47151    192.76512    200.21092    207.61361    210.03704    193.46741    200.92313    205.89596    205.64776    204.97181     210.3781    204.19848    216.64114    217.08802    217.64234    215.64671    218.85662    220.72475    225.92534    222.43152     214.5907    219.49518    220.41973    225.26136    213.27753    243.97105     224.3254    230.39966    229.66051    228.26308    232.59754    238.79003    249.59091    251.24103    236.56471    242.53377    232.60039    260.03604    266.28177    276.35633    257.95216    264.43358    264.45116    275.37808    289.01918    283.35853    277.27824    295.88381    318.64536    305.08241    337.20243    353.62174    330.07753    334.85449    307.95516    309.80681    338.86813     396.9128    349.40823    374.51072     386.2233    343.50484    430.69989    426.74956    431.73909    469.68936    484.75794    452.14378    433.22838    530.25574     580.6127    700.43867    545.10734    662.27612          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       15.00    268.74005    245.69264    236.43511    236.16627     223.5009    220.27396    216.10646    211.20948    205.47565    197.75611    196.78152    190.70403    189.13695    189.76812     180.5935    174.29357    175.63794    177.14786    175.71199    173.89983    178.22112    172.33972    165.61361    170.47409    179.22714      202.703    238.16107     254.9147    232.19257    193.82382    169.04549    158.59175    158.82761    154.89203    152.23187    148.91657    151.43195    153.77874    147.06037    148.07338    149.89803    144.85195    146.59398    145.14089    138.83738    139.89959     144.4135    145.55874    141.11994    138.87045    138.05553    140.49178    140.60639     139.8576    137.83316    138.03013    144.23805    147.05535    150.04139    144.76826    145.05537     137.4517    140.49115    137.39492    133.95707    129.86997     135.6983     135.0449    133.20766    139.38564    157.27526     175.0797    185.65818    176.88642    166.08745    158.25284    181.62134    209.64445    229.51652    219.81834    198.55833    173.21096    155.65961    147.27037    141.91412     132.9411    130.18591    130.97692    134.70054    149.01244    165.88459    181.88559     184.4105    173.31453    152.85105    137.28532    133.13902    132.11305     135.6001    129.59403    126.47284    129.45846    132.85094    126.62639    124.76992    121.27055    122.97964    116.95361    119.71097    121.99855    121.72223    118.88124    118.22832     118.7614    113.95024    119.66715    119.79758    117.80991    120.59053    118.33905    119.21443    115.49607    121.21546    121.93666    129.46099    137.59924    138.17857    135.49042    133.46873     128.9443    124.71051    123.44002    119.55133    119.83467    118.82309    125.71786    132.99289    146.73566    161.49291     171.1597    169.91092    168.55992    155.88718    146.28544    136.13849    132.10249    133.32206    128.08393    126.64938    129.31297     127.8373    123.36031    123.08722    116.60047    116.50051    112.26993    116.22161    118.30145    119.21641    118.09313    115.66682    118.79645    118.47336    119.62192    122.31336    125.18253    127.70776    131.39639    138.35422    135.92404    134.20074    131.34144    128.36385    123.60675    130.31336    132.67666    134.39394    136.19706    136.78528    131.72162    132.43824    131.79684    127.87111    127.91632    123.01038    122.41593     117.9227    119.78036     118.7289    119.51439    117.95584    119.21635    120.59443    119.51544    124.07451    121.64922    128.58433    128.45818    129.35264    130.25059     131.0782    130.32148    128.82416    127.61032    123.84031    123.96068    122.32842    126.19602    127.23209    126.90042    129.42961    127.45967    126.93897    125.66719    121.69507    122.10917    120.60954    119.21343    120.63184    120.24387    119.79233     120.3365    119.89111    121.15124     123.1368    123.41632    127.37077    129.36311    127.81592    130.73795    129.45147    127.86201    129.37721    129.59145    134.40821    134.60931    134.88875    137.39726    135.82939    137.35751    139.69637    137.83269    137.30696     137.9944    133.36091    139.31502    135.91383    132.68835     136.2669     132.5822    134.50897    135.35122    135.42643    130.36156     135.0538    133.73291    129.96272    135.98773    134.99927    134.18633    140.54313    142.74405    137.75601    139.88186    146.04376    149.21714    148.51155    144.27744    140.07909    139.18795    141.83884    139.93849    132.77993    135.61777    135.82383    132.97082    131.66243    134.86269    132.30705    135.37155     133.8696    135.23562    133.05248    136.23781    134.22137    137.65981    134.30551    132.33114    132.26176    131.91458     133.9962    132.51225    130.52124    128.19677    132.70258    126.01866    128.78626    128.81033    128.96594    131.97808    130.71838    124.87652    128.12958    129.62311    130.77856    129.22934    127.71048     125.9488    126.28332    130.71526    127.45576    129.18171    128.05385    127.59314    131.98663    135.92905    131.20822    129.93304    131.86751    132.89777    133.61404    129.34447    130.85939    131.89332    130.80166    127.37053    131.88765    127.46012      130.684    129.68183    131.13924    134.79355    135.00403    136.42267    136.68928    138.85143    139.24123    139.17313     138.5787    139.16034    139.14492    142.63839    139.43261    139.89153    141.07364    137.91882    138.43422    143.79341    141.30925    139.22821    142.80402    145.42974    152.84619    149.13709    149.73758    146.07267    150.35606    149.41998    153.90358    153.46101    157.09553    154.70591    153.82725    156.45863    154.52157     153.8822     161.6557    161.59363    160.05073    154.06787    159.51853    160.41878    160.71752    160.13899    158.56407    161.69206    162.57581    162.40354    163.32843    162.60964    156.74994     161.4825    163.77834    168.25286    173.21075    171.45055    168.33002    169.28029    166.26375    169.17979    169.88253    170.75299    175.91755    174.57244    174.96955    165.86615    183.87358    173.36999    185.45842    184.92576    187.43404    182.26178    189.47388    187.94846    187.00576    181.47087     191.0694    185.84585    186.57263    192.54937    195.71796    197.69686    191.66053    202.29759    195.25871    206.91106    198.25449    208.00799    196.46317    198.75846     203.7636    216.06905    213.90603    219.02743    205.88326    213.57387    213.74698    211.76618    225.54045     223.1166    207.87117    217.88731    203.68045    217.41656    215.32137    224.60776    224.30454    224.00322    214.50175    226.20425    242.77331    239.15342    236.12027    234.68391     227.0286    248.16044    246.60817    243.69451    240.69502    252.46251    257.93244    248.11292    250.86843    242.77095    263.31005    274.40664    259.49098    254.92027    280.89719    275.24667    282.42497    292.53837    290.58356    293.76826    311.59389    353.19939    303.61872    341.59632    328.10385    310.94676    347.91169    374.37166    352.36121    376.60804    343.48769    382.49485    373.72969    474.92065    394.91625    450.81499     457.4799    459.84339    337.04669    623.92263    441.45944    600.83753    616.58244     536.2984    476.76434          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       15.50    272.99978    248.66075    238.10483    234.51811    225.99257    220.94204    215.92885    213.76746     204.9753    199.60324    197.62399    192.70517    189.87451    189.33607    180.14379    179.89553    175.16175    181.65713     176.5798    173.78654    176.49067    169.34642    165.20901    168.72225    179.34512     201.8436    241.62983     254.0751    230.47747    194.13863    169.04701    161.20041    158.59926    155.89178    153.32477    152.27307    148.90352    150.74363    148.37893    152.34598    147.08184    142.29249    148.51017     142.7843    142.90205    143.32643    144.95138    143.37056    139.64821    137.97839     137.8442    141.53796    142.65591    136.66601     136.7671    137.11345    137.04721     145.8188    152.01635    149.55304    146.46974    142.98634    139.74907    135.76618    135.93076    133.46019    131.67374    138.18568    135.08995    141.91746    157.86264    175.88171    182.92875    179.96437    164.58093    158.07283     182.1449    219.76081    237.03457     233.4392    207.14761    178.52796    159.62153     146.0873    140.59463    135.67781    129.10746    129.09016    133.32906     147.6887    170.15478    181.60697    186.31682    174.55542    149.72955    137.91555     136.2237    133.86398     135.0111    129.37985    128.79204    130.65267    133.97803    129.29394    125.48096    124.26524    119.73451    119.66278    118.86459    118.01527    122.63248    121.92579    117.13075      117.253    114.44836    116.16153    119.62947    119.25415    117.49737    120.76613     119.2446    114.78971    120.86664    124.27981     131.6289    137.96919    143.72469     137.6165    132.30946    130.61175    125.21214    121.08472    119.58506    122.69415    118.84809    124.74826    133.92169    146.03391    161.65068    174.71832    174.57859    170.63862    160.33542    146.20512    136.49147    132.33127     127.8317    125.98372    125.71779    127.84494    123.38141    124.56656    119.45133    115.05611    116.15865    118.61623    116.55842    116.20344    117.18464    117.70219    116.22805    119.75245    118.28137    121.89797    124.20374     124.0451    127.14701    127.77615    136.09226    133.31555    130.86773    129.36298    132.78374    129.35604    133.56376    133.26921    140.09752    139.61718    133.13682    134.24918    130.34689    130.90698    127.36818    126.66656    123.01059    121.37552    120.78673    122.16797    117.79644    120.70645    122.53103    117.41299    120.18354    119.00676    120.65863    123.90407    127.14046    126.19403    129.02676    129.96924    132.16368    131.22133    127.04098    128.34819    125.08761    123.59958    123.97468    126.53181     127.5181    128.67003    129.41434    124.39646    123.96496    126.35523    123.10334    121.85152    116.72435    119.24484    119.12659    119.38864    122.89258    118.12776    122.69519    120.43883    125.34038    124.07953    125.26108     127.8445    130.70014    127.27191    127.65933    128.01448    133.40202    131.49117    133.85648    129.85588    137.61188    137.50047    134.26763    138.80221     137.2648    137.45655    138.85648    136.50381    135.12343    136.56926    135.89574    135.11934    131.89623    131.78063    129.40296    130.87547    131.85851    133.35741    134.65713    135.27109    132.17931    134.27554    139.84117     136.7845    138.24467    137.17678    137.99236    138.68102    143.85033    144.27925    146.14688    146.02501    144.54966    139.81657    137.24967    139.37078    136.91943    136.66231    136.22248    134.38571    132.98905    133.44491    132.80464    133.91326    136.59015    134.21638    134.03063    134.01939    136.88293    135.58645    133.68034    129.28728    130.77897    132.98841    132.80719    130.31965    132.50887    130.08117    131.91493    125.73964     131.4307    130.59684    128.69865    127.62344    131.65898    130.51167    130.79438    126.43083    130.36998    131.26173     129.0639    129.62202      123.561    126.75976    129.88805    128.26176    127.23616    131.03948    128.67476    131.98324    133.25564    129.00826    129.18168    131.13895    125.70512    127.34468    130.54593    127.53576    130.17125    125.42427    129.17791     129.8385    131.46769     128.9633    129.76821    130.24942    132.50683    133.32641    136.27849    140.41576    137.51199    137.62517    142.53863    137.83315    137.98454     137.8454    139.35142    139.81935    138.18262    140.54841    143.93083    145.52752    144.21019     143.6354    144.14794    140.43333    149.94514    147.50139     149.5198    147.02278    150.26621    150.19566    149.51532    154.06197    155.61399    157.20559    156.88583    155.37089     158.0143    156.55774    155.78059    153.39562    157.77125    158.09524    154.54671    158.48472    148.52674    159.59185    156.11252    160.84478    159.12454     164.7767    163.74662    165.83669    158.81232    161.56534    167.25104    159.64013    173.95326      167.473    172.65708    167.61296    169.59402    170.32758    174.16705    173.09984    172.54086    172.90415    172.64228    170.14263    168.81665    176.28998    173.82564    188.34817    176.80212    180.21781    186.93009    196.83026    187.23535    183.00576    186.44263    191.17236    190.25334    193.70571    198.09072    196.00351    184.30521    199.59616    193.26911    213.06569    205.77094    196.95151    196.38718    197.03728     201.3337    215.54492    212.33216    213.81597    207.73846    200.09871    206.02945     220.6971    210.37053    215.94183    208.84431    201.81735    206.74068    226.52574    223.33728    221.57937    226.00567    217.11416    209.57994    221.74143    236.92078    241.97928    223.27611    239.13416    229.55019    228.49924    258.11372    243.89127    245.84956      258.716    260.24636    245.63362    266.60744    229.46242    269.17839    267.24653    268.72358     255.9327    294.54499    277.92629    287.65585    287.04755    292.51181    288.38889    328.28418    304.46124    299.54088    364.09624    335.70286    309.13731    328.03426    361.99376    411.42334    368.08144    374.83735    369.54755    369.11774    438.64354    383.79672    450.85909    478.23533    456.54842    438.97935    604.88962    580.80679    403.20377     596.8139    631.35242    711.97062          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       16.00    271.81485    251.29998    239.85126     232.3351    226.66801    221.11823    218.95306    215.96191     201.3432    199.64989    191.61042    194.76977    193.32037    190.27679    177.43303     182.2155    176.07752    179.52774    176.31148    174.40081    176.14653    168.48233    167.69079    171.21584    177.17966    205.40593    241.34188    257.91524    229.42625     198.1263    169.02902    162.28567    155.33371    155.71062    149.53405    153.73898     147.2812    144.11046    156.45522     151.3978    148.21034    147.61056    144.20134    142.15941    143.90767    143.88652    141.30123    146.60125    142.71576    135.16564    139.53451    138.69347    141.20022      134.634    138.57664    139.73226    141.99697    144.92453    145.65753    147.56841    143.75883    140.63987    132.18698    136.35704    136.95068    132.75007    133.87323    133.11738    135.75103    142.13328    156.76549    175.30779    182.98193    180.07369    165.45476    159.46143    187.27633    231.37209    248.91875    238.32159    212.27408    179.51083    161.65124    150.22517    142.45465    135.52444    132.29328    133.25285    135.74596    148.45526    165.12261    184.97281    187.74315    173.59044    156.27674    139.72931    135.30575    133.41682    133.43974    125.97077    130.42233    128.74841    132.06409    126.19918    126.11296    122.14585    117.43387    119.07105    119.92797    122.28721    119.04652     120.1266    117.78975    117.44304    114.93462    116.03898    119.94638     122.1094    121.65701    119.81791    119.77828     116.8657    116.08892    124.69115    133.32293    139.53027    138.73427    138.46446    135.98343    129.22283    124.52444      122.258    119.73824    120.69406    118.49077    126.01096    132.69245    145.31893    161.42321    171.79187    171.43035    166.73421    151.61327    143.27756     137.3662    132.30282     130.4531    127.38303    122.15063    125.96766    123.91086    120.86231    116.55893    117.01425    112.77296    118.50597    118.76915    118.67372    115.94857    117.56079    114.82194    117.83885     117.0094    122.59265    119.16896    123.33228    127.98319    125.28797    129.75238    132.46045    131.96572    133.90558    134.50015    131.76898    134.28592    136.62736    135.62452    138.64601    137.10218    136.49396    131.89043    133.99264    134.32177    128.90227    128.06624    124.29594    123.09169    120.72535    121.58307    119.52003    119.01842    118.39145     121.6589    116.81911    124.96018    125.50417     125.1241    125.61603    130.67256    130.16675    130.22094    132.50147    126.87638     126.2177    128.35208    123.18317    122.53215    126.70483    127.42658    129.30613    130.16732    127.60853    121.48338    124.35702    124.45586    125.25612    119.53438    115.45738    118.64744    116.44596    124.40745    118.78987    117.31957    122.81491    123.22045    123.67249    121.55052    130.37656    133.41843    127.79792    128.57198     127.7769    125.63657     132.9841    130.52368     133.3108    135.69324      136.704     134.0746    139.62427    138.73441    136.07958    138.36127    138.29927    135.30427    133.28036    134.45956    131.96893    131.20236    130.99588    132.05121    129.21229    130.96006    133.50875    133.70002    135.09997    133.60318    133.31253    135.51405    137.49493    136.80469    136.89654    139.39032    141.36749    137.45089    143.25869    147.79625    142.44352     141.2698    135.32672    137.21816    140.05946    135.99463    134.49604    133.32502    134.82132     133.5618    135.15846      133.709    132.63272    134.85022     135.1877    132.18813     135.6965    134.41769    131.95568    132.70863    134.87694     131.2039    126.45688    131.79476    130.48172    128.59227    127.06534    130.36946    125.04217    127.04974    126.90716    129.53832     129.6802    127.48702    129.59333    127.85535    129.09743    128.97367    129.61537    127.20425    127.05099    124.44054     134.2001    129.76548    130.40438    125.36522    128.98872    130.58344    129.58107    128.70886    129.67373    129.55859    127.54265     127.8311    128.26584    128.17219    126.70984    123.84446    124.78007    127.58545    129.87779    131.52297    128.96312     135.6846    133.88682    132.63761    131.99075    139.23368    137.12113    137.68083    137.68113    139.26756    137.62206    142.12001    143.73113    141.56455    139.94279    139.42804     132.7471    137.17779    139.21026    143.05027    142.92726    144.72621    144.86457    144.05412    149.81201    148.99829    149.21687    152.17293    147.18435    152.70111    156.07797     154.5852    153.46842    157.13753    152.54414    152.08771    158.38732    157.59784     158.1106     156.3176    154.48544    154.80414    157.19506    158.79001    155.11063    158.59486    159.94204     161.0597    163.45481    163.19382     161.9819    166.00093    165.62045    165.12163    169.95544    166.81047    168.04158    178.58649    173.52634    171.24151    160.21462    175.46401    178.53455    172.02713    171.97389    179.54226    175.68751    178.30342    181.42792    179.06428    183.68574    185.95001    174.62986    184.40436    190.61796     184.9948    182.77323    187.84227    189.06623    197.70116    192.00502    202.70872    194.26875    197.92876     201.6481    190.33823    202.39994    212.21561    202.82546    203.95602    200.17294     203.4468    213.12471    208.54282    207.95643    209.25488    202.01667    211.92027    220.12355     219.0248    215.70289    215.09783    209.32799    209.69563    229.20496    221.25912    223.17444    229.03689    225.13223    228.57877    224.23599     226.6845    244.21783    226.70674    238.63813     230.2381    240.20237    258.90025     256.6104    243.82102    239.87207    220.91696    245.93616    291.02322     263.3649    271.48379    270.22695    258.73676    261.89411    287.19872    296.14558    292.24101    278.31293    305.21296    292.04432    327.86804     294.1153    372.08356     330.7578    322.25967    324.72671    336.59254    380.98868    368.07698    393.45471    408.79853     419.6821    464.30283    367.29341    437.67836    474.00572    510.13868    474.65593    503.49729    656.92296    609.25849    595.13088    502.00634    600.39237    693.54274          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       16.50    268.63817    252.58157    241.67625    231.37392    227.02097    224.79713    222.89288    212.59488    197.86495    195.28617    192.40305    195.13175    193.19038    191.55843    182.68769    181.85526    173.59533    177.96035    170.85379    174.66763    179.77165    168.38747    170.66864    170.53984    177.01888    206.29859    236.63137    257.78636    230.86692    197.73041    167.32124    159.76355    156.17132    158.25827     153.0761    155.07134    153.13184    146.53523    148.26071    149.81004     148.9241     149.8641    142.77943    146.95602    146.29914    142.92014     139.4544    145.06986    140.71856     139.2364    138.44466    142.87286    139.58496    140.50317    138.38437    137.37924    143.58624    147.38533    150.01753    151.40311     140.6519    143.33922    137.26645    133.02667    136.02539    134.94546     133.5807    134.76589    138.67226    141.84908    157.30604    173.62393    183.46726    181.92504    165.18515    162.99488    194.78458    233.95734    256.14331    239.96623    212.57464    180.34975    162.51136    155.63015    147.75726    138.18631    133.05013    130.96272    137.58473    150.74883    164.62788    181.91191    186.57228    172.33546    154.20925    136.54195    133.67604    128.53701    127.37214    125.56021    130.16157    129.27061    130.05194    127.99496    128.84567    120.78453    121.50579    119.89262    112.50868    118.17495    121.28801     117.2785    118.65464    116.60236    119.05956    114.74913    117.25775     119.2654    118.89873    119.88435    117.31249    116.60914      119.832    124.95088    130.10603    137.69751    141.09728    134.41558    134.53318    130.32675    126.69936    123.44553    118.73482    119.69824     123.3816    122.75352    132.88593    145.33229    161.48951    171.41144    171.10687    165.12167     154.6945    143.81776    134.39227    127.93404    127.74191    127.64966    126.75515     122.7071    124.02656    121.63357    118.05278    117.16968    115.52101    113.23846    115.56944    115.93565    117.01109    116.04148    116.77301    114.43189    119.37643    121.59577    119.28209    123.21186    123.76531    126.05685    129.46278    133.35575    133.22248    134.45639    135.11989    137.66968    130.32061    136.95419    136.00598    141.31701    141.25469    139.08105    135.78463     134.1397    133.77898    130.91882     130.2678    129.27313    121.51835    122.95701    123.50253    122.70955    124.23976    121.94498    123.07686    119.86464    120.26242     123.3328    126.70135     128.3232    130.14255    129.74529    129.48718    130.07726    128.41518     124.5279    122.19591    126.21566    121.20085     121.4845    128.87147    127.25051    125.75849    127.45836    124.39311    120.70196    122.20153    121.75824    120.13383    116.46937    117.76512    118.25448    122.41681    122.06907    119.50783    122.24946    122.61635      121.256    123.87643    123.62117    124.79162    128.12022    128.87927    127.51335    125.85574    132.17259    130.81284    133.88463    139.34829    137.07101    137.32136    135.25681    137.51548    138.53017    137.49321    137.42735    134.05959    137.75938    136.75971    132.63835    130.89747    129.13719    132.73312     129.3079    133.97958    135.20395     134.1643    133.94302    129.18895    130.55856    132.19223    134.35445    133.62262    138.53736    136.89463    140.91506    140.29598    148.19482    145.68677    145.54128    142.06138    141.23915    135.83147    136.93349    134.83824     134.7785    135.30039    135.95451    135.38365    134.24524    132.21897    131.79552    130.79506    131.72157    133.70809    130.09642    134.07443    128.72779    132.57218    132.60725    133.15728    128.80623    130.65474    129.31685    129.77605    127.70469    130.03628    124.35904    124.68807    128.29966    129.39366    128.03007    129.23323    128.26387    123.11891    125.17783     123.6874     127.5375    125.45561      128.118    126.89489    126.13623    126.21264    125.89577    127.85714    130.96374    127.18615    129.16422    130.48502    130.15004    131.18978    130.18103    128.07493     125.5776    124.44508    126.88325      129.314    124.63214    126.83544    127.66306    129.36535    126.38487    131.20587    131.69978    133.03676    129.84833    139.59276    139.65545    138.64577    139.94954    140.45265    134.12778    138.76046    138.18726    140.92378    142.96411    138.71336    140.55317    134.23999    142.28011    146.90973     143.8999    145.10488    147.95672     151.8295    152.59845    152.37108    150.72131     151.2266    152.21966    149.22453    158.20348    156.24304     156.8649     158.5378    154.60764    154.00972    161.62201    160.80558    153.33336     158.5901    149.45666    156.44912    157.10764    161.95874    164.75378    165.43027    158.14179    160.16896    163.35004    164.27438    162.70843    167.30194    168.98174    164.59221     168.2765    167.49316    169.17709    171.57832    172.67094    175.96271    169.56105     170.7363    176.93022    165.51041    180.39681    175.51668    177.25877    182.65355    183.30734    185.00644    186.25573    183.44293    180.05093    193.87151    183.73862    188.53775    179.56178    190.11555    187.59444    189.77243    200.33297    198.82315    197.70378    194.44931     195.9222    197.41692    204.40154    201.21268    205.84462     199.7465    200.26826    200.96898    214.21728     213.6967    213.21671    215.96381    212.19284    208.42632    221.00994    217.30469    222.64816     221.3824    215.83102    211.77275    218.14745    225.89069    226.62079    219.72607    225.38367    219.66474    222.35173    227.48337    239.92449    224.99861    241.01851    247.80521     239.1551     253.8438    254.84872    249.64271    254.91368    242.32729     250.4653    267.11491    272.34167    270.26032    270.03436    270.81725     272.8679    294.03918    289.74783    300.94796    292.25519    287.49085     300.4794    354.94745    336.77153    334.44727    330.84997    329.78461    316.59666    366.78598     353.6803    349.83986    382.81025    399.60151    435.75369    390.66698    418.92783    513.96457     445.1544    510.49842    478.14204    582.04887    724.60814    750.30401    799.37958    563.10031      709.606          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       17.00    265.92351    252.89864    242.85005    231.79737    225.86337    230.03096     220.2029    209.04756    198.64929    195.16165    196.42984    190.48094    188.19211    190.61736    186.93688    177.45061    174.92744    179.88204    174.62395    174.15029    176.74129    168.13462    169.01071     171.6591    178.80828    207.21009     235.5429    255.80388    231.58845    196.46586    166.51482    155.42595    155.00313    156.54013    159.40249    153.25074    151.27646    151.77751    152.48882     150.9643    147.77994     153.8647    143.35474    139.34356    145.03724    146.30295    139.91202    143.66256    142.02671    138.54547     139.0989    140.96127    137.66715    138.19629    135.67286    140.26261    144.13772    145.21825    150.52415    150.18853    145.20142    142.46238    138.45599     138.6626    137.64863    135.85354     133.1677     140.6068    137.63961    144.82668    160.72653    175.05687    185.49061    174.04164    165.06166    164.88142    193.62422    231.97673    248.24518    237.22556    205.95535     177.4352    171.26586    162.19241    151.16611    139.29752    135.27019    134.54968    135.89993    145.77229    165.02601    183.04372    183.85637    172.07272     153.0871    134.68509    128.81125    131.38107    125.61042    130.00521    129.82935    128.94005    126.10967    128.05589     123.2847    121.62912    115.96503    120.33161    119.08878    116.87173    117.74505    118.26774    116.54835    117.95654    113.92343    116.18951    118.92647    120.58953    120.07637     120.7101    120.62019    119.16797     115.8437    122.75432     131.4805    137.14721    141.13694    138.57169    131.99424    128.12512    122.93262    121.55197    123.37884    121.24441    119.24078     123.4391    134.10885    148.83183     160.5197    172.68377    174.10586     161.6732    157.12486    144.66232    134.47059    130.28185    128.82734    127.53307     125.9521    124.50297     123.1176    119.33253    116.59876    115.26161    112.87905    114.35039    117.68324    118.35946    118.19015    117.00869    117.83434    116.40963    117.52022    119.41543    122.56072    123.11022    126.14927    127.20441     120.1951    131.17197    128.66638    133.01316    131.28925    131.34732    133.31766    135.88664    138.62546    137.94182    140.01903    139.05664    140.69444    134.45464    132.99568    131.97706    127.57826    125.62631    123.15163    121.27146    126.42315     123.2332    122.67432    121.21951    120.03453    118.58826    122.27302    125.09379    124.53077    125.82018    131.06546    129.12417    129.08462    131.29136    126.20961    124.12879    123.49429    121.34629    123.89947    121.43678    124.46308    128.12874    128.12793    127.52697    124.52421    121.85158    123.95854    123.30451    120.13221    116.92557    118.40307    118.10754    116.25872    120.00246    118.53435    119.39308     119.4774    121.60644     120.8318    128.51333    128.15249    130.56999    124.46356     127.2532    127.46071    129.11242    132.63216     132.5561    136.64391    139.40163    138.09108    141.62338    142.33169    139.55864    137.42475    142.28417    136.55563    134.44243    139.12682    133.40289    129.08349    128.62355     123.8049    133.10599    125.83966    133.05092    131.46268    131.23023    133.16055    128.70716    127.89082    134.08398    134.11086    138.81602    138.44586    141.92694     140.0157    140.88853    142.76078    143.61578    140.45058     139.9439    137.62411    135.45751    133.46454    138.32382    133.41679    135.45363    135.39137    133.42052     133.6688    130.75344    135.32033    130.51179     134.2134    132.45745    132.61934    131.80652    131.31929    131.52509    127.99878    126.00262    129.17021    128.00007    128.48981     127.6764    128.78697    127.10429    125.27251    129.46713     129.5411    126.22911    133.16603    129.57955    123.13243    126.80238    130.83963     126.8107    127.64756    123.24417    126.46135    129.96317    128.32435    129.01495    129.16953    130.61762      126.858     127.7575    129.82872    130.39253     128.3541     126.0683     127.9171    125.61524     125.6099    128.50238    126.23531    125.97915    127.37207    125.32883    132.48826    130.34346    128.76011    131.01507    130.06912    131.78402    131.62238    136.54081    139.41588    140.19327    136.33462    136.56502    139.31202    139.06368    141.30052    139.00603    138.67316    139.34893    141.05547     143.3455     143.0402     146.1913     142.0551    144.88387    145.66838    147.36344    149.63781    150.78721    154.86417    149.81627    151.39509    156.62821    151.44605     155.5808     153.7942    154.19097    155.89827    157.93031    159.51218    159.52121     157.6583    157.15581    158.63801    160.19737    166.40666    162.88888    159.81947      163.298    159.97625    163.76911    169.29285    168.30869    165.20372    162.69818    167.41647    161.52064    166.94575    168.25001     173.0462    171.79381    164.81335    168.33829    170.04457    175.49899     179.9032    171.38079    184.61258     182.1471    189.02272    189.44165    177.30131    177.14876    182.98005    177.95356    186.90581    190.23909    191.89088    186.61356    189.00282    187.39796    190.34978     199.8061    192.47045      200.386    193.14046    199.95045    195.22012    203.93252    210.49445     209.7126     205.2018    205.87974    205.67361    212.11436    217.25198    207.90999    209.10074    220.62413    215.97536    228.04148    214.87682     208.6876    228.98862    219.95499    216.87268    230.48624    224.18567    222.00976    232.40743      221.919    235.18073    220.54228    238.46635    235.03914     240.3782    232.62184     243.5432    247.19993    258.17273    267.76236    245.72061    232.13369    236.63145    254.25666    269.10178    259.97395    270.62491    266.16414    272.37805    273.84224    315.74878    306.60275    286.75118    289.28878    290.68115    310.90998    320.16229    320.43497    344.93237    328.32664    331.50913    333.17746    403.74493    377.68787    410.15436     417.2533    411.84028    422.54288    464.55143    470.82967    524.75736    435.98319    488.92503    499.69157    546.14947    662.14204    595.49053    576.36405    559.67413    560.58923          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       17.50    267.17323    254.38526    241.60416    232.81489     225.4882     229.4406    217.04714    212.68767    200.00938    199.45781    198.21784    185.86848    190.31479    187.02347    186.89168    177.41062    179.53987    178.11345    181.07799     176.1467    173.84172    164.68259    168.35077    173.99621    180.84513    206.80249    241.18719    251.05819    231.35895    196.25156    168.47512    157.28086    157.60896    155.14458    155.94688    151.11875     151.3546     152.9447    147.34811    148.31787    148.54279    152.22384    144.70925    144.23984    146.25381    143.51756      141.853    141.41023    143.76593    136.43818    136.38594    137.86983    139.42377    136.74787    140.33963    137.54558    139.15659    145.76871    148.36272    148.68633    150.26714    140.34641    140.25314     135.2674    134.74482    133.97115    135.01159    136.63173    135.61465    146.10105    159.37179    177.07863     183.8976    179.10718    166.86787    164.96611    183.54742    218.44416    230.94204    222.02004     189.4823    174.60636    173.89899    171.80169    159.26033    146.66836     137.4312    133.77203    137.67708    142.95709    165.43968    178.39819    184.21908     169.8626    153.34216    139.00435    129.64847    126.90625    127.86193    123.31999    126.58829    130.41002    126.48258    128.56136    124.97162    123.40684    120.78974    117.04528    119.28212    117.39196    119.66629    120.32501    119.50749    116.01058    116.86523    118.48045    118.42256    120.39691    119.47097    116.36336    117.57814    117.59695    117.06725    124.57658    130.07576    135.13386    143.39665    137.52897    135.87676    125.68174    124.20715    124.77828    119.94249     122.3957    122.92152    125.74077    134.49813    149.73547    162.10115    168.46754    171.87582    163.20656      151.213    140.15246    136.16357    130.96833    124.25846    124.75219     123.5211    119.25564     119.8675    120.71618    117.50083    114.66789    116.45453    115.11974    113.66403    116.46519    117.89991     115.8043    118.24612    119.21449     117.9861    124.25774     121.4893    124.79893    125.09315    125.43577    123.94399    129.29586    132.31693    129.81286    131.93783    130.60962    133.03364    135.35851    139.34201    144.43526    141.81318    145.22672     142.6707    135.51215    136.84478    136.23393      128.972    123.94407    124.34113    123.13021    116.55938    121.65965    123.35674    118.22247    119.31537    120.16214     121.2686    121.01024      121.619    128.17171    127.75916    127.84126    129.06386    126.23309    128.06645     124.9221     123.7443    122.98082    120.00441     125.4679    124.44077    127.81893    124.50723    129.98261    125.59034    121.84818    122.74406    120.97474    117.76458     122.2696    119.15606    117.33884    115.61744    121.22331    122.82001    119.32438    118.82849    120.64818    120.97257     124.1633    126.89439    128.48978    126.10886    125.18479    126.79384    126.09187    128.99479    131.28257    140.86441    142.91782    144.76187    139.75057    145.57194    144.88363    143.21904    143.25542    140.93021    138.21179    131.31768    132.74115    129.38618    131.03834     126.3789    127.27716    123.61656    128.22889    129.87159    130.92313    128.88673     128.9672    132.16178    130.70616    133.10575    132.89383    137.63505    139.42328    143.21769     141.5388    144.16552    142.45067    142.04197    138.11652    138.25967    136.57512    136.19103    137.12619    133.84023    134.21328    131.83705    130.30511    131.17152    132.75572    134.33317     134.2674    135.38883    136.02458    134.83153     135.6468     134.9645    132.53207    130.97362    125.69578    124.85982    129.03004    128.01876    121.80405    128.02613    131.28746    128.56656     130.5928    128.22554    126.79208    123.50435    127.40042    124.51264    126.45886    124.53055    124.88477    127.54689    126.86894    126.03712    124.07913    125.35417    128.17311     126.9632     130.1139    128.16195    127.53748    129.37511    129.67978    129.42841    127.39914    126.41577    122.51666    125.81614    129.16864    124.02638     121.3491    126.29867    124.16551    125.57875    130.49587    130.22606    127.66807    130.06961    133.13686    130.86851     136.5963    136.37316    138.81282    141.08818    137.08792    134.61229    142.70272    138.36578    142.31524    140.37743    140.52991    140.08756    144.10409    140.53907    145.16908    142.21827    141.63517    147.44646    146.86606     149.2958    148.34038    150.18968    152.92589    153.57479    152.05514    156.23496     156.8035    155.92321    160.19603    152.43327    156.25659    159.90778    156.37275    156.03634    160.34881    155.92537    157.78391    159.73956    161.56697    150.96901    158.71296    153.98928    159.50001    168.07873    167.95513    166.49818    163.90293    164.17418    166.71882    168.86878     161.0998    167.13968    171.20817    167.94963    169.09065    180.12685    174.16291    173.69988    172.25978    172.69844    180.03241     178.9845    181.50429     182.3346    183.66416    182.50684    183.78865    185.47238     193.5642    184.93519    198.31649    188.45536    195.29732    184.02768    196.68102    200.40998    200.99416    199.75714    196.84678     192.2427    203.58562    205.55171      206.832    195.09703    203.79662    210.96808    217.42776    213.81408    218.04754    219.78605    219.24538    214.21996    229.25979    225.77409    214.86637    218.23188    220.66801     217.6371    234.16904    234.77375     225.8568    224.81973    224.20199     229.5705    249.54461     242.2656    247.86793    232.23704    239.62557    239.67077    252.99161    257.98121    261.02867    243.98451    250.26841    243.38905    254.35513    265.80543    283.02547    289.18946    274.33286    277.73135    277.92304    309.00449    326.75602    306.86115    299.90227    331.10194    305.46903    371.96629    376.22044    380.12866    358.15711    350.44997    366.48821    352.72057     444.2954    419.15886    416.82905    397.65679    438.23651    464.43994    534.66619    544.20303     522.5356    476.22131    517.84591    778.99225    664.87771    686.73313    643.24761    699.43956    731.92067          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       18.00    270.82419    256.99173    240.88097    231.90864    226.41119    223.97034    216.99626    213.63032    201.53246     201.4639    200.34293    188.14247    193.99107    190.42309    188.15589    181.60766    180.99553    176.71416     176.9699    176.40293    170.45746    168.32761    167.19544    172.89731    178.85455    206.98321    239.51371    248.49674    230.07622     194.3413    164.61872    157.02159    160.22683    153.93722    155.85339    155.89814     150.6802    152.49624    146.72078    146.59796    148.89624    148.39964      144.519    142.72643    145.34081    144.67695    141.03176    142.75158    145.68421    141.48866     139.0292    139.32858    137.61424    141.58254     137.1646    137.83391    141.48078    147.76032    146.92173    153.97514    144.87934    138.44507    139.71397    136.64846    134.38356    137.58383    134.66481    134.54896    140.09285    146.31828    159.09891    180.26968    185.90385    178.37436    165.29491     163.1532    175.68997    194.84228    206.46149    193.10698    177.30294    169.29356    176.89996    176.73279    165.22311    149.55871    138.30879    134.74766    140.54213    145.58522    166.47077    180.10595    184.73792    174.24001    153.26267    137.67989    127.32959    123.09164    124.09055    124.00764    125.76133     129.2403    128.53401    127.75417    123.33496     121.0993    119.80294    117.07858    119.61356    114.45863    119.33834    120.50798    117.15872    117.92109    117.26451    113.78051    116.61196    120.36391    118.68036     118.9403    116.03601    119.70141    119.60627    122.22636    129.01207    136.42318    142.05934    141.63589    135.65344    129.75305    126.04177    119.35625    120.24815    120.10513    121.07489    125.58558    134.58257     148.3099    163.54255    170.20897    172.43903    165.17448    150.08894    141.17705    135.37457    132.67411    125.46171    125.37528    127.15664     123.3858    122.96498    119.90944    116.39884    121.18039    114.94127    110.30697    115.08512    117.14651     114.9991    117.83891    117.90667    118.28424    116.76643    120.54968    125.38821    122.65647    125.45215    126.16093    125.12434    129.38908    130.54821    130.88103    134.48633     137.3591    135.15668    130.13818    138.07816    139.91377    144.55306    143.55818    143.26983     135.9959    135.17377    134.50183    128.12244    128.30026    127.68329    124.29224    120.17289    120.27683    121.29935    117.61013    117.06061    117.95078    118.94458    121.91701    117.80485    124.12191    124.80239    128.53237    132.99521    125.97279    130.39122    124.35185    121.57039    125.33465    123.56344    124.24102    121.02154    125.43712    126.49567    128.04807    126.24677    122.02896    119.67438    119.23867     120.1449    118.19322    120.56452    118.74504    113.17721    114.09908    120.28066    120.33248    121.75637    122.50313     124.3654    123.36593    127.00748    129.22969    126.90467    126.72285    127.99927    125.93829    127.24356    130.39037     136.9555    137.07341    141.35147     143.2371    136.65166    144.25433    143.48422    139.35919    139.05704       137.71    130.41619    133.33387    129.08004    127.71689    128.10136     126.0306    124.52292    126.81789    128.37023    130.98819    128.36823    128.85358    127.24516     131.9334    133.12625    135.64138    137.82094    140.85988    144.84664     139.8183    145.26565    144.54276    136.75477     138.8451    139.96798    129.80484    134.46852    131.05528    130.65842    129.28917    129.51879    127.76759     129.5297    130.52677    131.53479    133.52862     135.2765    135.46779    131.27237    137.37997     132.5631    130.49588    131.27359    130.86098     128.5388    130.15143    128.60825    131.63511    129.15903    133.52135    130.88817    127.36586    127.94316    125.37753    125.52025    126.94039    121.92481    123.08004    124.76249    123.16365    124.67316    124.74082    124.76965    126.49652    130.14235    130.45089    129.12712     128.4815    127.65283     125.0896     127.2832    125.83049     127.7306    130.94828    126.31363    127.00667    127.79543    128.79987    128.06537    125.22708     126.5735    122.53815    129.27957    128.05166    127.66783    130.37768    130.68202    127.85052    130.47245    135.31379     134.0319     135.1507    138.61579    136.40178    133.32638    138.84632    138.65571    136.62797    143.79523    140.96908    140.69721    140.56469    143.85927    139.87991    146.35303    143.09617    142.63707    145.98528    143.12712    151.11494    154.66804    146.35516      151.265    150.52703    155.01363    153.18496    153.80111    154.43962    155.85539    157.51423     158.5208    159.87884    155.66242    152.41495    156.09451    155.84163    157.92428    161.66493    155.23849    160.19583    159.72903    161.88662    164.93495     163.8103    165.59769     165.3977    162.08258    162.03952    170.27913    175.41861    175.53275    175.40394    170.73129    172.19674    180.19863    181.78696    183.70379    180.16937    182.02077    174.69198     184.5474    187.74007    185.45443    184.73299    176.93984    186.37419    186.01067     191.5469    194.89493    195.85451    188.22369    195.12845    194.36319    205.12732    192.34573    194.91692    198.42526    201.64077    203.83456    206.39102    210.13361    210.71131    209.87974    202.13777    210.32741    220.86619    209.38235    214.61246    218.76684    219.14429    218.84543    222.87307    225.80447    226.73577    235.42741    224.26776    225.25256    234.02788    232.57672      248.131    234.80713    246.72107     237.8358    241.59779    246.72206     245.6679    249.06984    248.78029    259.72399    253.73774    268.75778    276.38466     268.7737    263.70431    262.04712    273.51048    283.47007     298.2609    290.60341    295.69471    284.23047    290.33422    318.98197     310.8666    328.21676    338.71619    326.00438     302.5043    350.78913    380.39371    334.04838     343.0798    365.00341    352.20421    398.96762    429.39444    432.77832    404.63688    436.97257    435.62014    539.38924    525.48416    566.29282    580.20496    530.08817    542.31719    698.46014     763.5995    750.56998    797.87176    671.42366    662.39424          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       18.50    275.52799    259.98504    241.16132    230.36854    226.43893    220.62044    216.95895    214.17434    200.84515    200.83952    199.44608    192.31275    190.70994    193.85044     188.5442    179.01739    179.94502    179.48856    172.19663    177.15485    173.67838    172.63689    163.93753    171.56349     175.1295    205.17271    237.39645    251.03774    235.50927    193.09036    165.50108    158.69157    156.96395    150.97713    157.41875    149.48789    150.44459    151.26863    147.74206    145.48367    148.31586    155.86099    145.92277    149.35395    142.83248    143.02058    137.78724    141.31444    148.93216    141.26087    141.36436     141.5357    140.10466    141.28839    139.79371    138.22914    139.04616    148.58422    148.84205    150.34983    143.84898    141.26604    141.39182     138.1576    136.20186    135.46518    135.79455    137.94925     137.2858      146.106    159.99239    176.62335    183.99881    178.30992    160.44333     157.3992    166.34671    182.36355    184.83066    177.29975    166.18202    167.95377    176.06299    178.78504    167.69509    151.70782     134.6496    134.55152    135.25031    149.26567    166.62304    183.62477    184.45114    170.38412    152.03193     135.7054    126.61511    124.42511    125.32178    123.99876    127.77343    127.23034    128.35379    126.32044    124.29182    123.12923    119.75489    117.07523    118.27029    118.52618    118.56713    116.58922    118.66564    117.20642    115.54595    116.37203    113.10792    116.40444     118.7702     119.3024    118.36954    119.05874    120.92478    120.69095    129.11054    140.05183    139.54141    138.04589    130.03589    126.09357    122.38449    120.09769    116.79313    117.11636    118.64747    124.70076     132.1883    144.77956     160.4402    167.05376    172.63585    163.48991     153.4903    139.56024    131.06009    130.56895    128.43144    127.71069     124.9483    121.18128    119.87421     118.1386     115.0745    116.77286    116.11792    116.29803    110.05358     114.7526    118.33963     115.2586    113.38736    114.95461    118.19951    120.12244    121.74656    126.27579    126.69034    126.50087    127.31897    123.99684    129.33919    125.22811    128.85172    132.16922    131.28036    132.00022    135.27975    139.05508    143.07704    140.41258    140.37072    137.86728    131.82306    130.23583    129.54067     124.1387    127.24168    122.66691    121.65694    116.48139    123.16606    117.60202    123.90809    123.80655    121.28653    120.51954    122.48531    130.95829    124.45675    129.65363    130.40033    125.43156    130.48118     126.5227    123.00122    122.81221    126.30151    123.50723    122.10697    122.25237    127.44461    125.39501    123.00471    123.91368    115.29591    122.91932    116.10643    117.87514    118.49095    117.23695    116.94697    116.62157    118.00959    120.80774    120.46906    120.90264    121.29802    121.23569    124.85255    125.99207     128.4781    126.77138    126.85458    123.30695    126.19556    131.35876     134.4211    135.76085     134.7649    139.16844     139.8055     139.8005    135.82832    136.62975      136.677    131.67134    131.30954    128.21996    129.27978    127.68994    125.72414    130.30699    123.76403    126.73608    128.63229    128.82344    125.83367    132.45182    127.58459    131.07403    132.87587    136.37574    137.68088    136.46432    140.32727    140.30161    141.52714    144.61122    142.52758    140.64902    135.90767    131.30976    128.56241     129.3593    130.14053    126.35211    127.65647    128.48674    130.63624    130.05085     130.7975    132.47174    127.63613    132.22326    131.08019    133.33716    126.59869    132.33898    130.72819    129.70353    129.09729    128.34707     127.4154    130.14657    129.59209    130.26929    129.59956     128.3101     130.0318    126.60524    125.93887    125.13058    126.95199    120.47559    126.57558    127.60322    125.32359    123.46747     126.9662    123.16551    126.78594     125.1954     128.2415    128.54472    129.00785     127.9219    129.60592    127.98058    129.36054    128.12304    127.90988    126.69934    125.14189     123.7631    126.44544    126.71019    129.34579    128.23809    127.91142    127.24915    132.40046     128.5254    129.84387    131.04606    132.22697    133.98637    135.57812    134.13557    130.71933     138.0567     134.3562    141.40037    138.32559    140.76667    139.28732    145.24366    139.86476     143.7409    143.58536    145.18661      141.023    139.02075     148.8219    144.98757    151.87397    153.14864    150.50657    149.96819    151.69364    150.92581    165.20038     149.3118    154.61184    154.07175    153.70287    156.87675    157.15485    159.88594    153.35112    162.63444    157.37373    155.86362      160.212    159.62755    163.39202    160.26041    153.98129    158.42937    165.58431    166.17004    167.68635    168.83361    171.27859    167.81672    159.12714    170.52197    169.81401    173.58248    169.33901    166.19829    178.69472    177.51197    174.43597    181.63978    185.53946     180.5267    185.78583    184.56618    183.47614    183.53636     181.4783     186.5741    192.71613    190.59366    194.54512    198.49151    194.58595    192.77673    192.63725    202.15452    201.55564    195.17326    199.07851     202.0598    207.79732    215.38068    205.86398    210.04305    208.78645    210.86386    203.85998    215.94125    215.34987    211.46862    216.72698    214.42939    221.28325    229.64606    223.83589    236.26082    228.23168    228.58387    222.51543    236.11969    242.04327     240.3301     231.8102    251.41999    239.55894     261.9886    265.45546    236.12248    254.05725    246.13103    259.60663    272.11419    285.97499    259.01075    265.60529    259.39373    274.10798    292.78169    304.45996    297.63045    272.89207    294.74848    298.07932    309.23451     343.0419     320.6156    326.74639    328.30753    321.05727    322.54983    357.29853    371.10212    340.68209    362.01486    356.95336    368.51776    426.14054    423.60238    438.57486     442.5372    484.39791    483.24579    568.82244    509.16702    587.62261    560.58899    544.79896    546.74015    898.09285    874.64669    808.87934     937.1736    816.30653    889.65339          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       19.00    275.91422    263.13652    240.95095    230.14833    227.20287    218.14434    215.68462    215.85485    205.55409    200.56604    196.71205    193.29454     187.5921    192.63707    186.23761    178.00886    176.08675    179.88037    167.10603    174.17495    173.87151    175.04697    164.28081    170.40645    178.92422    205.36048    238.48382    246.99576    237.08045    193.02463     163.3104    160.75341    155.59684    157.07266    152.21521    153.84544    149.26308    148.42122     153.3229    151.84385    146.50378    151.74004    143.82022    146.20507      145.111    146.18088    141.85116    143.03942    141.08039    133.75514     137.3134    138.36894    138.46817    140.18221    141.81243    138.52302    141.04701    145.56447    149.79462    149.81705    141.50309    140.86236    139.01613    134.80722    137.15511    135.61654    135.98164    135.97425    142.24341     148.5987    156.78958    175.85437    185.98022    176.95295    162.86032    157.33422    160.37968    172.36619     178.5341    168.58398    159.62721    163.64624    169.17602    175.01837    159.05265    148.33346    138.68045    132.26052    138.82234     151.6352    169.44657    185.24755    184.17947    171.71531    149.60591       134.01    125.93806    124.64106    121.94442    126.71941    125.89024     130.3666    128.40131    121.22721    123.56248    124.45853    119.74855    119.84571    120.06971     118.7426    119.99615    119.97419    118.56574    117.35676    117.83291     116.7683    115.99181    113.57763    118.57809    120.82919    119.07358    117.35524     119.5351    122.93621    127.10518    139.51398    141.02711    137.04509    135.11322     126.2412    121.91352    121.63411    117.91129    119.62882    119.60547    127.79014    136.59157    146.54157    158.86179    168.25382    167.77964    161.90528    154.85874      137.527    132.11551     128.1056    129.32241       126.19    125.08334    122.44439    120.59421    118.78556    119.27933    115.53971     111.6953    115.08499    111.55753    117.34386    117.51336    116.14503    115.40842    115.74062    117.86276     117.2718    121.62745    123.41432     125.6171    126.09055    124.81361    123.73347    123.96324    121.89838    121.73099    130.01038     129.8161    132.25968    132.91749    138.26748    139.73353    138.89988    138.62543    135.83597    132.30012    128.28785    124.48253    126.60345    124.45354    120.13187    122.62626    118.26678    120.23353    121.78191    120.00167    119.37921    118.76591    119.63261    119.93638    126.55899    128.23553    127.79812    129.15388    127.44702    126.08013    126.02113    120.78992    121.32366    124.01308    125.39201    121.85864     124.2003    128.77335    128.06836    122.43105    120.17714    119.56608    119.76935    117.80566    119.86733    118.63248    115.85759    114.66291    113.75827     118.6633     121.0506    124.09715    121.91363    125.23944    125.33836    124.30348    126.08332    127.62158     127.2088    126.90034    123.56571     127.3098    130.67164    130.88208    131.75546     133.1852    134.90468    135.21662    138.13991    135.47965    133.44276     132.5941    131.40325    128.85049    126.29133     125.6972    126.26482    125.20438    124.79871     124.2754    122.97072    128.92866    130.47179    131.20669    128.84742    129.17388    127.97685    132.29284    133.53089    135.82139    142.46382    142.94827    141.82523    144.50711    144.09597    140.14997    141.06136    135.76403    134.77417    132.12826    130.43678    127.26752     128.1349    125.22544    129.57635    130.40911    130.45654    129.74701    131.18173    132.33992    131.17609    131.77799     131.3185    132.95436    131.61641    129.74697    129.19428     133.9231    128.08559    125.81617    132.14008    131.69604     126.1236    128.55119    124.15511     132.3743    126.48199    123.30463    127.46785    124.27975    123.84121    124.49194    125.98323    123.86485    123.29744    120.50079    123.87004    124.42025    127.35074      125.827    128.79152     127.4695     126.6883    130.29449    130.09469    128.68186    128.22641    129.19675    125.81817    124.29999    127.71895    126.66604    129.26141    126.76896    124.07095    122.50267    127.75498    131.57886    132.51104    128.17367     128.8492    131.07725     136.1841    134.01692    137.48352    138.47349    135.02716    136.07339    139.68264    143.32252    140.04455    144.51977    140.66771    144.45985    143.38442      143.462    142.38837    143.13762    148.45753    148.67881    146.62077     147.4324    148.79047    148.61227    151.01834    150.22049    152.91209    153.64691    155.55677    156.59852    154.75891    156.58689    157.32367    157.05234    161.57808    158.87848    162.65063    157.97218    158.11213    157.34259    167.28575    162.12102    166.60803    160.80829     164.8789    161.64487    169.86068    168.50591    169.52074    166.78897    172.02544    168.56986    170.67166    178.71268    175.46594    171.08175    174.00793    179.07351    172.29439    175.69131    171.93934    184.38359    182.15543    185.05959    182.52459    180.99598    190.12348     184.2326    182.44209    190.22781    192.32748    193.52985    193.05144    198.97089    197.94286    193.75745     198.2742    190.71381    196.99789    198.94329    196.65139    206.23906    212.45038    218.52851    209.94554    214.84872    218.18615    212.06414    219.78155    222.31938    224.10526     220.3229    220.24881    220.22153    225.13248    235.99994    231.58184    228.69372    227.23538    228.53925    244.78406    246.52618    244.50706    241.72252    249.53328    236.82155    252.40388     248.1537    251.14262     262.5485    263.35938    256.37274    294.62157    283.23842    265.40358    266.22433    266.42872    276.84037    286.15012    298.53958    310.19594    297.87546    313.12954     305.7972    307.85258     361.4923    329.59293    319.58209     352.9481    320.76748     324.9088    379.60473    417.21802    410.73311    408.93264    410.07469    416.72623    470.99552    476.71408    498.02836    449.36049    498.28705     481.5229    627.90718    607.20073    570.71528    597.43389    566.86168    594.47534    829.76172    769.86587    849.20133    928.81285    839.67649    870.12108          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       19.50    272.24036    264.38476     241.4813    230.97689    227.64196    214.53686    216.24312    217.42832    210.26786    204.67037    196.33118    193.52894    189.38035    193.36528    181.27713    181.83708    176.65169    174.42711    177.04035    173.07791    166.69036    174.52926     165.0744    170.54634    178.76957    203.71822    237.65509    250.29978    234.98339    187.81219     168.8861    159.15825     163.4183    157.81102    149.62022    154.16125    153.26668    149.97156    148.55615    153.09657    145.15257    152.61186    148.27255    145.77081    141.94418    141.81345      144.465    146.09883    138.71347    138.23047    142.16343     141.0644    137.78186    138.58567    140.26326    136.21763    140.72545    143.45933    145.03988    147.83106    146.60882    144.47092    140.41704    139.19602    136.66027    134.57763    137.53152    135.79292    139.92355    145.52589    158.94688    177.10727    189.36078    176.39419    164.31683    152.94598    162.22648    169.09671    168.54922    169.66799    164.02018    159.08644    165.43872    164.19905    154.64974    141.01933    132.01409    134.73977    137.63416    152.63349     172.5244    183.64933    186.46534    169.46129    152.11033     134.0415    128.71377    124.46651    123.52794    125.29143    126.08269    127.83076    124.84077    127.49103     125.4753    119.03203     118.2824    120.51192    117.55812    119.79698    117.77217     120.6586    120.35813    114.71299    118.81528    120.32773    117.22955     115.9391    116.76803    118.79806    119.91872    118.47274    121.51689    125.30673    130.78606    139.54427     140.3459    138.27928    134.88129    127.90652    122.47855    118.20052    116.40956     118.3932    119.89881    124.67645    134.70116    142.60015    161.83454    169.17307    171.40383    164.08956    154.89479    137.04717    131.36738    125.83343    126.84768    123.71866     124.8942    123.25583    117.15143    119.12602    119.39422    114.47865    116.04709     112.9851    112.32002    113.59301    117.64347    117.13892    117.80091    117.58315    117.89904     116.9739    119.21619    121.88382    126.71114     123.8676    122.87032    126.06474    117.20457    122.67272    123.64218     126.2807    127.35984    131.73966    131.46184    133.76721    138.39668    139.83028    140.76285    133.79605    128.33915    126.11238     123.7914    124.93671    122.45903    118.84255    120.34218    117.50245    117.99608    117.70678     121.9614    119.62178    119.49788    121.99175     120.1628    124.88064    126.38466    127.62958    128.04873    130.48348    123.99793    123.73995    121.03927    121.23573    122.06691    119.42084    122.07178    123.28241    127.59289    125.35998    122.59134    119.80719    122.50378    120.46436    115.64567    114.85859    116.50998    115.97713    117.23705    115.94783    117.73986    120.14081    119.99927    123.90605     122.0163     121.6506    122.29677    127.27287    127.90393    124.81278    127.00409    128.08335    125.21419    126.81467    134.43884    134.16491    134.03363    132.61187    139.11174    134.41606    141.23708    136.54832     132.0222     127.8753    130.31507    128.39033    124.52715    125.06192    125.64087    125.23103     126.9348    125.53868    122.74257    125.94389    130.36348    129.97376    127.52138    128.66454    134.38821     133.8563    137.27649    137.62337    140.93962    141.96167    141.49448    142.80706    139.19463    138.46022    133.50769    137.75399    129.54053    133.42137    127.95462    130.55901    128.84576    127.70452    129.20777    130.27643    131.22508    132.86225    133.73846    129.68135    130.53143    128.62035    133.09265    131.09443    131.48197    128.41759    133.12534      127.586    131.40126    127.29617    126.08395    125.18999    128.23195     126.9637    125.23388    125.41286    123.55648    125.83063    125.83948    127.45665    120.95604    124.00194    124.92949    119.82944     124.3624    122.88813     122.0614    127.16185    127.07301    127.40986      127.974     129.6259    126.29366    126.60125    127.37584    127.32395    128.56175    123.56584    124.71796    124.95879    125.65969    133.36232    128.42747    128.34748    129.49226    130.39782    130.84608    133.85635    128.32974    129.86796    135.66019    134.34439    136.33669    138.05642    137.97242      144.161    137.95979    135.14668    143.50875    142.61644     141.2313    141.83987    140.08534    142.67198    141.33955    141.97148    146.15432    142.91796    146.19975    144.00007    149.20608    148.39803    152.23801    152.55542    154.16385    155.94312     153.3449    156.90648    153.18233    152.50718    154.39047    158.87785     156.8033    159.96349    159.26553    154.27792    159.68122    152.10741     160.5737     161.0225    163.58173    162.22935    160.20821    163.39127    160.46418    167.84047    172.05909    171.16919    174.04279    168.39457    170.36751    179.31066    173.39848    178.57057    176.12787     172.6038    176.25659    179.03902    177.55645    179.12295    178.25821    177.17888    180.64593     192.4285    179.35311    183.73473    187.69269    189.98593     182.3951    194.14459    200.68251    193.87708    193.62654     200.6603    191.27149    201.98881    187.61546    202.48867    202.06094    199.70839    199.69285    214.88437    212.77572    216.09196    206.37358    210.10707    214.92123    224.99405    227.64363    223.97286    219.55218    221.65387    217.96423    229.43595     231.4834    237.19835    228.37223     236.1284    230.61792    244.93384    248.18271    235.03033    246.76468    254.32689    254.40358    253.83991    265.01289    262.15308    261.34195    269.14451    257.58379    279.05581    276.83778    290.55678    291.81674    280.42146    294.73232    308.98769    317.84223    339.33091    299.08731    320.58156    301.08338    323.04913     360.0839    362.17236    328.30176    328.63857    355.99994    371.71323    361.07121    411.69499    411.62835     383.6968    401.69416    398.20689    493.39306    478.37223    495.68354    447.11361    488.74257      468.018    600.49556    596.25588    577.23088    582.78398    636.55427    737.04525    933.50894    907.60124    879.84169    948.88862   1070.31867          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       20.00    268.78539    265.23727    242.38366    231.66322    226.66927    214.96978    219.15274    219.77146     215.3033    205.75936    196.97863    193.01878    192.10615    191.51901    183.07012    184.64951    179.60773    172.79138      181.658    173.47034    168.53043    175.93358    170.35676    165.15357    180.07867    201.99432    233.73081    251.54816    229.90006     191.9446    169.81279     163.6842    155.99752     154.6078    148.83735     157.9793    150.90377    149.03549     148.8937    150.24415    150.76761     152.4148    150.53746    148.24179    143.56818    145.11919    138.29656    142.42369    143.25093    141.42413    139.01096    140.07429    138.48428    139.83103    140.47085    143.19787    147.72454    146.17735    149.17841    148.52556    138.98426    143.80703    140.04123    137.70436    135.41469    134.65322    136.23308    134.56284    140.69677    149.98996    157.90196    173.67219    183.10696     175.1364    161.12555    155.17023    157.55571    161.31371    171.39428    171.81188    159.59839    156.85279    158.14901    153.86283     146.4901     139.5053    135.00375    132.99543    136.35487    152.39176    171.91921    183.90816    183.04523    167.34804     149.7328    138.64401     129.8593    122.48522     123.6126    125.16021    126.75777    128.77537    128.90925    125.84788     122.6183    120.92324    118.32107    118.50024    118.86425    119.56807    117.00186    118.20951    118.45964     118.2245    118.62876    118.98385     116.7681    113.21728    121.80321     117.4498    118.39115    117.50382    118.84855    124.14371    126.51155    135.72032    137.83859     139.4976    136.68572    128.53895    122.24687     115.2464    116.67041     118.3074    120.59937    127.86945     131.2022    144.81466    162.78791    169.30305    173.69752    166.50229    151.90496     138.0102    131.99329     128.9234    128.30196    127.24344    124.76369    122.26832    119.51836    118.88789    120.73146     115.2592    115.03175    115.85596    116.90272    113.54119     117.5317    116.50732    116.36247    113.78864    117.04311    118.92061    117.99432    122.78723     121.5837    119.32676    123.63301    120.76707    121.02381    123.31605    121.54813    129.65269    133.22695    132.51984    138.54778    135.74477    138.17941    141.47366    136.33249    132.69127    124.74377    124.02495    124.07276    122.19147    122.05685    120.75305    121.19345    119.69246    118.92066    116.66966     119.0528     117.2275    119.39584    119.76599    122.39742    125.75931    123.30326    127.47573    129.26605    128.52933    125.10882    120.84659    123.07006    121.05477    123.78093    121.89165     121.7415    121.53362    122.28342    125.39287    124.67123    121.97151    120.53804    115.40852    116.16948    117.47193     117.9073    118.67887    118.41841    118.45641    116.61596    119.04523     120.7468     120.6917    123.55745    123.98534    121.05877    128.26164    122.57127     127.6269    129.78227    128.80248    128.54256    125.05436    129.91398    132.73871    133.46249     137.1338    134.59948    136.98063    137.56663    135.77642    134.41256    130.57356    128.52081    124.42957    123.29175    125.60019    123.29681    126.95126    126.13265    124.52793    120.96773    125.30438    125.66221    124.72059    127.08714    125.84914    126.37598    132.57511    132.82507    136.47374    141.37691    143.87592    140.18406    136.79964     140.3807    139.72528    133.94646    132.76753    128.02873    128.26335    128.57131    130.02875    129.01955    131.23352    123.50158    124.57358     129.4257    127.34725    129.55306    130.17173    131.11557    127.20774    132.30521     128.0696    128.69914     127.7717    128.20303    127.28361    125.07026    126.75778    124.25493    124.15666    125.41892    126.10763    124.72381    130.42681    122.72921    124.62888    125.06269    122.41401    119.87523    121.89152    123.02292    122.98058    124.07765     123.0476    124.26729    126.96779    127.05317    125.43174    127.05437    129.79961     120.7115    123.81906    126.49853    126.04495    127.15493    127.91961    125.07768    127.23662    127.57571    124.81923    122.12937    127.95559    125.28913    126.16069    127.42487    130.63968    128.09058    132.49235    130.66068    134.56071    134.74028    133.68987    137.30371    136.15982    134.70315    138.49247    138.95445    141.56185     143.5171    142.51761    140.91618    143.13946    142.46891    145.32546    143.31794    142.11764    140.69973    139.65509    154.76228    148.84501    149.23356    151.05677    151.35242    147.28431    156.01397    154.55368     153.5085    151.75534    157.41656    155.24822    157.47265    155.76119     153.3489    156.41636    156.94314    150.60558    157.46858     164.2321    161.90661    159.08662    158.89644    165.68029    168.68394    165.62986    168.88185    168.14637    168.94244     164.4005    166.78763    174.24772    168.53971     167.9934    167.81968    165.00418    169.33228    176.77021    173.02369    170.89662    173.90425     169.5271    175.38557    179.97733    175.53485    179.88968    180.97236    176.97769    179.64546     191.7448    187.53229    186.03279    174.81781    187.38938    187.31717    203.32218          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan     308.2087    309.22511     313.8018    318.75471    328.29771    310.72243    337.57269     374.3018    357.85431    348.94206    344.61724    353.05136    390.26136    417.99606    427.46186    425.79158    426.86231    392.78107    417.56969    503.95027    519.42475    511.77307     508.0124    491.57646    529.86487    683.52559    628.80667    702.50986    656.89911    657.60776    823.98193    986.90508    965.38147   1072.57187    977.39461   1009.51559          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       20.50    271.46081    266.14533     242.6501    231.18922    224.53484    216.34407    219.69948    219.60879    218.40731    200.79188    197.97446    193.74977    196.14724     189.1925    185.67677    181.32918    180.01702    175.86395    177.44871    169.92533    169.20578    181.13847     166.1517     162.8841    177.41384    199.03274    233.94067    246.75021    230.06352    192.03622     170.6271    161.98294    152.82715    153.12955    156.32539    156.65214     150.5715    154.26748    146.02951    147.07073    150.36993    148.82351     146.8605    144.16404    148.46443    144.79366    142.12761    144.80466    148.49935    139.62088    136.72993     139.4935    138.82612    140.79513    143.03178    143.41832    145.25638    147.20611    150.02789    148.03968    140.61491     140.8196    140.97955    140.14005    134.33114    133.58379    134.86185    132.66679    138.69397    145.80382     160.2112    178.75328    185.15541     175.9037    162.61188    151.76954    153.38435    159.06455    168.53632    170.75351    165.36036    158.67957    151.83478    150.10751    137.86742    135.59875    135.46479    132.96869    138.35796    151.18352    168.12969    182.02161    181.38972    170.54055    152.29196    136.80246    130.06184    121.12749    121.43314    122.33297    123.95514    123.96832    129.98145    123.70552    122.40201    121.28458    120.22604    118.55292    120.72518    118.18643    119.22915    114.11433    119.61262     118.7033    114.87947    117.65659     117.5937    116.88596    117.79406     122.3728      122.423    117.97092    118.22316    122.69205    129.65974    134.20105    138.23253    139.23313    135.35006    128.03894    125.72578    118.74532    120.32144    119.42992    118.67944    122.53868    136.81634    145.97403    159.10551    171.46817    171.46195    160.95895    152.87726    140.99438    129.83193    127.50293    128.01032     126.1934    124.82305    123.83584    125.21212    116.08337    120.47348    117.56095    113.69294    117.88363    111.40522    114.95998    114.60719    115.06678    115.74602    117.37444    119.17411    116.60289    119.00152    124.23744     123.2503    122.39071     121.8801    120.72929    123.76947    122.82621    126.86135    131.75956    137.21971    136.35317    134.48428    135.28402    134.87102    139.11685    138.08536     134.4418    131.41777    124.63889    125.12316    121.56356    120.51257    122.22805    118.11324    121.47089    118.08731    117.07205    118.69569    117.70137    118.43362    118.21794    122.02836    121.65342    126.31577    125.88115    128.95261    125.14676     126.2332    119.76046    120.82937    121.88758    120.14793    120.87813    123.90853    122.89229     123.2544    122.14496    122.04196    123.06581    122.23308    118.90207    115.25426    119.57456    118.66799    115.67939    119.91884    116.81835    117.88043    120.07658    116.39985    122.51697    119.92233    121.84138    122.57762    120.13429    128.53893    125.71297    127.63773     125.9403    126.70246    128.28314    129.69499    132.63855    131.71331    134.64996    133.89291    135.75178    131.81811    133.84552    135.60369    129.24885    131.03958    128.61236    124.38875    126.30312    125.10816    124.41006    122.20762    123.99132    125.26681    123.40754    125.73208     129.0209    128.99835    130.72068    129.14601    131.79014     130.9847    133.10639    136.18027    138.88303    138.49352    134.18934    137.45704    141.58952    131.36503    131.23398    128.91232    126.94776     124.3957    122.85384    124.50828    125.71944    124.66944    124.36697    126.64443    129.75643    126.52403    126.90175    124.74905    125.09555    124.49397    129.71734    125.09197    127.86163    126.78487    123.94303    123.76957    125.33596    123.56929    122.80879    126.48714    120.77534     122.0943    123.77927    123.71292     124.3406    123.44477    121.53891    122.21861    120.11991    125.33904    122.54901    121.18181     123.1466    125.18058    123.84335    124.46688    128.32965    125.25843    123.38783    126.22856    126.43377    123.24278    126.41418    129.16928    128.68862    125.65593    127.67107    129.17948    126.04315    126.65861    124.82562     126.6826    125.97055    127.44157    129.86046    126.41828    131.75876     131.1612    130.70714    135.81258    132.49962    136.04109    136.11201    138.72156    133.15084     135.8695    135.06204    137.13606    133.69236    132.87977    134.35399    139.08889    146.00467    144.84424    140.99773    138.63664    139.80999    143.41021    144.21316    145.84873    144.66721    141.00773     144.2458    143.35137    147.67997    144.20773    142.84025    144.50502          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       21.00    276.07925    265.09172    242.23524    230.87476    221.98333    218.35263    218.75296    216.98581    214.55104    198.03928    199.42668    194.41298    198.40804    189.88047    189.24625    181.38335    179.96666    177.60147    178.87906    169.31031    176.26186    173.80963    166.18965    167.10758    177.48678    202.56386    234.67309    243.83739    231.68781    192.65193     170.0776    158.54186    157.50154    152.01474    160.38068    156.11726    148.82014    149.00928    149.98374    143.00774    149.92072    143.64509    146.62657    148.81438    144.79983     148.1934    144.56246    139.10578    143.60965    142.28562    142.03851    141.33613    138.85585    139.58436    138.66658    140.38004     145.5602    146.38217    148.68089    149.48531    140.17045    136.71049    138.07497    141.62489     136.6035    133.82083    134.42336    133.24744     136.6731    146.08524    161.64437    179.93033    183.55275    175.28414    163.10545    155.60276    155.00093    158.23608    164.17536    172.37302    168.90535    154.81311    153.87725    149.06393    139.06242    135.03706    132.28905    136.15243    135.75561    144.91405    164.79674    179.41471    182.73073     168.2064    151.19981    140.33948    127.31507    122.16415    122.30913    121.34018    124.34104    130.54598    127.46468    123.78115    122.41546    119.03985      118.968    118.49538    118.43918    118.29003    122.75343    120.78122    120.73678    117.55733    114.51333    114.58708    116.65884      119.961    114.60163    113.06931    114.65061    118.00042    119.53611    124.26366    128.72508     134.2229    140.13791    139.47035    133.45922    127.65662    122.67267    119.93885    116.99137    120.56571    118.37948    121.80277    135.07733    144.28517    156.60364    167.58496    167.90591    165.59619    150.34835    144.19387    131.69365    130.28998    127.92355    127.73659    126.06751    121.82035     122.3311    116.96651    114.25895    117.54263    117.86089    112.01202    114.20696    115.18371    111.83171    112.80471    116.43773    119.35828    117.99362    117.12766    116.44575      122.317    123.75253    125.82276    122.94774    121.09654    124.18936    122.22959    127.36595    128.20949    133.95509     133.7038    135.86007    133.28974    134.11932    139.44261    135.05642    136.15963    129.54083    129.41034    121.34191    122.08319    120.32343    121.89885    118.22784     119.0381    116.25219    116.84848      119.081    116.19629    114.82582    120.05725    118.82341    121.30087     128.3013      124.539    126.57684    127.40493     127.4133    119.66521    118.88183    120.96881      121.414    119.31976    121.14925    119.81559    123.28237    124.25456    124.12816    118.31451    122.54308    117.43015    116.60857    116.53446    116.49296    115.75777    116.15868    116.62513    116.26338    115.53122    119.88712    120.11161    119.85217    120.73077    121.21008    120.83898    127.42259    128.01156    126.30882    123.62454     127.1765    128.56713    123.90482    127.45743    132.54098    132.71486    131.55002    136.64096    126.54684    132.15846    136.13971    130.47303    128.50432    128.45569    127.49943    121.15571    124.67828    124.84527    125.11128    125.27483    126.10168    122.67913    125.49344    129.79934    127.02556    128.22946    127.05397    129.10621    135.93821    137.99555    133.67947    136.58477    138.28868    137.05012    136.38112    136.32883    135.83924    132.40628    130.47434    125.16009    122.98215    124.81965    125.14799    123.67269    125.73402    123.22064    120.00234     128.6273    132.21256     128.6794    128.68878    131.05886    123.94128    128.74208    126.28886    124.89292    122.89979      120.677    121.82772    119.44142    124.74844    123.12344    118.64749    121.28602    121.43878    119.79968      121.689    122.60514    122.12446    121.56221    122.15221    117.82093    121.43708    120.24471    122.81426    122.45756    123.33308    120.31433    124.72775    123.79791    125.00558    125.08391    126.22723    122.92762    127.75424    128.94761    122.74654    122.52514    123.91272    123.87822    124.10289    128.34647    122.12175    123.64512    126.59171    123.51417    127.30857    127.60168     125.1156    123.96684    123.43927    122.61784    128.24275    125.48283    128.43122    129.47521    123.90599          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       21.50    280.09767    262.33028    242.71497    231.13093    220.63486    222.97813    218.17577    214.90129    210.05373    198.18159    199.56041    194.94865    196.35441    188.21849    190.53383    185.99116     179.8759    178.21937    182.47264    174.62504    173.96865    174.27704    169.18962    171.81067    179.49738    207.06094    229.54371    249.17747    230.46482    193.11816    167.69178     164.5233    156.70569    153.27457    154.74515    153.36714    149.74662    151.22095    153.10346    146.95318    146.10959    153.71313     145.5849     149.9278    143.70729    143.12998    143.75329    144.41915    145.00233     141.9004     144.7855    141.16536    140.28848    141.83274    141.28049    134.76667    145.12661     147.6379    152.06718    148.00391    145.73115    138.95379    138.47958     139.6887    137.38887    134.93856    134.82515    135.62034    139.51765    147.29591    162.02457    176.57716     188.1923    173.31381    164.18064    158.10539    159.17396    163.10161    163.22031      167.672    162.88949    155.67246    156.84333    147.53273    141.09069    131.92737    131.30104    133.38021     137.3298    152.83868    167.26105    180.51935    181.25693    169.45032    151.22327     137.4466    125.43799    124.89404    120.91407    123.43967    125.22323    128.15925    127.31029    126.88254    124.13483     119.3857    119.84056    118.35046    117.59697    120.57324    121.74399    116.53995    116.56806    118.95794    114.96177    115.80909    115.37764    120.11344    117.08408    115.26173    116.90664    117.41467    121.23511    127.37229    134.92736    138.74649    139.72197    140.07498    133.84104    127.19725    124.41204    119.19961    114.89006     118.9189    119.26852     126.2183    134.15394    145.41021    156.66287    162.93608    163.79351    163.12952    158.17008    142.34509    136.99992    127.05394     128.0503    124.56378    125.57479    120.57358    120.91704    117.48769    114.91398    114.34232    115.56315    116.91346    113.14802    112.54586    112.80448    115.16799    114.12414    113.75954    116.69671    121.65358    118.92089    119.08126    126.95469     125.0496    125.42797    122.81523    125.13045    126.44289    129.86913    133.65264    132.58346    134.32462    134.38546    132.34111    133.11244    135.53689    134.13283     134.5194    128.41201    122.89334    123.33173    121.16433    119.90188    116.99412     116.0567    115.70449     116.5241    113.79035    115.70148    119.96238    119.57682    121.22517    116.82524    123.44474    123.71454    126.49416    125.40767    125.00464    123.25653    122.16146    119.39677    120.35316    120.49515    120.63881    118.94945    121.68002    120.76005    120.97752    121.26035    121.59301    116.30012     115.7278    117.99717    114.99363    116.49184     116.8915    116.86598    117.40619     116.1313    117.93749    116.77959    116.76447    119.26217    122.18753    122.33284    123.81879    124.47113    121.73274    124.51829    125.60845    123.86002      125.357    126.95367    133.07103    135.40921    133.73059    134.71448    132.77108    129.59742     128.6922    136.24097    131.27098    128.13375    125.58846    126.56312    122.81798    121.80958    121.65704    126.59035    125.51442    121.78285    123.67815    124.90918    125.18328    126.20884    128.46491    127.72038     130.0337     131.6554     136.1743    136.87336    139.84219    137.98065    137.60649    132.41113    134.20325    133.25363    127.40399    129.93128    128.42467    122.14384     123.3179    126.50442    124.33624     124.7832    123.37326     121.5599    127.20172    126.93909      127.236    129.66002    127.94893    122.82631    125.93872    126.79802     118.5914    127.54511     121.2925    121.57464    115.09807    124.88306     115.6007    124.33734    119.97577    118.98573    118.53388    118.74089    120.80917    116.31313    118.49251    117.83667    115.88428    115.85471    122.54878    118.88474    117.82827    118.90206    121.24441    122.19186    120.85558    122.27882    120.17013    120.33547    119.41272    118.61704    120.54994    116.83807    118.37395    116.21901          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       22.00     280.2284    258.17025     244.7804     231.9393    222.96984    228.47552    218.25437    214.06139    208.67565    197.46028    198.92746    191.60615    192.97956    186.87459    189.81928    185.24098    180.74461    180.75472    178.86046    174.13732    173.77624    179.82624    168.35085    171.61755    176.88576    205.37893    231.44335    245.68547    230.23417    194.44149    169.34636    165.20189    160.55932    157.65141    156.50933    154.31205    151.62032    149.26755    153.33473    148.09111     151.0874    148.06776     147.7957    150.57439    144.49376    144.76835    144.15666    140.07626    145.25256    145.39322    143.59758    138.26163    141.00667    141.73122    142.69846    139.62949    148.15553    145.05385    149.44413    145.17939    145.20174     139.6386    137.97924    140.46712    136.54911     134.2047    136.50874    139.25371    140.00911    141.87811    171.09536    179.51639    184.12004    174.65965     162.1366    156.86146    162.92739    164.71883    164.26268    162.69979    159.30835     155.5944    156.96484    157.70541    148.71844    137.81093    130.42601    133.38603    140.49026    149.89809    168.43788     180.1792    179.50585    169.73805    152.33417    135.97202    128.98788    127.17924    120.58636    121.99861    127.92327    126.95448    129.91681    127.54054    122.97681    119.05007    121.75755    118.21058    117.02491    119.60955    119.93755    118.12226    115.73312    116.15499    118.65013     118.2769    116.94871    117.83727    118.57184    116.21356    118.42105    117.40325    120.68092    126.34955    133.73724    136.71538    138.97421    140.44361    133.41472     130.0042    123.65429    120.47076    115.78567    121.45827    122.66391    124.13988    129.73208    144.43343    157.92343      160.546    169.34225    162.95235    153.77227    147.60697    138.12389    129.49107    128.21293    126.63812     123.8297     119.6107    120.96166    117.29135    112.43433    115.50824     113.9023    115.79329    114.53949    114.45779    113.72704     111.4233    116.47078    113.94653    115.31793    113.59227    115.72472    116.19594    122.08561    120.99356    123.99088    129.87246    138.47678    144.14562    139.94639    139.24183    138.09211    137.39454      133.651    134.80616     137.1879    132.91373    130.49684    135.69237    128.13586    127.67469    126.32293    118.31707    119.71274    117.55378    118.16155    115.04448    115.26932    114.70541    115.11201    116.66807    118.78358    117.98322      121.045    121.11021     124.2385    127.93547    129.56344    124.92672    123.43779    118.85537     119.0721    116.07739    121.01245    122.52162    121.78451    121.00547    123.03797    120.75283    119.17137    120.43985    118.47212    115.54092    118.77918     111.7826    113.55522    114.48029    115.12324    111.65122    114.29831    115.27834    117.55513    117.95066    117.92949    120.48365    121.06796    124.22497    119.06946    124.92712    121.09782    122.22642    125.56516    125.20107    130.25611    129.24818     130.7361    132.76927    137.00652    134.53938    134.58605    132.22733    128.32801    130.54529    125.97031    126.09735    123.42855    121.97831    123.04576    121.67813      119.664    123.65122    123.85305    124.55928    120.31569    127.77798    123.78248    129.25638    129.63532    129.93176    128.54779    134.74517    136.05851    137.19297    135.23065    137.70603    133.88388    133.25721    132.42614    131.45031    126.84138    123.02165    122.38111    121.89435    120.00127    123.98414    121.95028    124.07663    126.43626    123.35521    127.00061    127.08162    125.28476    125.26884    126.12398    120.90128     127.0847    121.90509    120.19017    122.50926    123.62523    115.26664     119.6771     117.7119    115.38475    116.69554    116.48948    113.72215    115.95629    118.58235    111.93309    117.22727    114.56458    115.78329    113.13332    111.85508          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       22.50    278.94555    259.42677    247.32074    232.58998    223.69821    226.14432    219.10271    216.34429    208.64125     200.6799    199.05052    187.61099    188.56815    186.94931    190.47036    181.37278    184.16387    182.77586     178.5259    175.20731    182.29684    177.66566    173.03565    171.35822     180.2614    208.70787    233.53373    247.30957    233.61427    193.64143    167.91363    159.76157    159.33976    154.19219    155.11886    156.59377    155.90222    153.02674    151.68704    150.44921    151.85223    151.52748    149.06158    147.51074    140.64994     142.1212    142.91487    142.89379    147.01194    145.02915    142.59954    142.63501    135.62502    138.60435      142.589    141.45414    145.60485    150.02125     150.6291    148.59685    141.83677    139.96902    135.93444    138.82607    139.61379    136.09119    138.38976    136.60823    134.57564    149.50555    163.40121    178.56315    181.20574    179.56569    163.30056    164.19634    163.57663     168.3949    163.08348    158.35609    159.55971    155.99556    161.86589    162.99206    154.20265    138.01662    130.43655    131.51466    134.59627    146.21964     169.1771    182.60121    184.19231    167.37046    148.58728    137.39504    127.29582    123.65569    126.91384    124.00097    122.34367    127.54419    128.36936    126.08996    120.07497    119.53069    116.17789    116.86665    118.76262    119.49996    120.02918    118.57585    117.44081    115.77683     115.1186    116.41926    116.77953    117.53413    118.52674    116.97053    118.12277    118.91846    120.16319    124.83385    132.57535    138.89333    138.50174    139.21561     137.7282    126.74733    120.62656    118.74834    119.01244    117.30297    121.19034    123.50609    131.00502    144.27757    156.32705    164.01471    162.57044    161.49129    147.54919     139.5584    138.53567     130.9675    127.59453    129.46256    123.73953    119.21751     118.9064    115.24492    114.45722    114.10791    112.40654    111.04615      112.565    109.88275    112.87976    112.97184    112.66207    114.10348    115.51133    114.51126    118.09822    121.98068    122.40079    123.22615    128.90933    137.96972    151.11656    153.75845    145.15995    134.56203    133.58547    131.18239    131.47182    133.47371    133.30933    128.79652    132.60644    128.95267    127.66819    127.41345    120.20913    122.59342    117.27435    118.36621    117.39917    115.95471    117.39219    115.25899    117.67392    116.90364    116.02263    117.16729    119.12308    119.70784    123.01521    125.04782    125.55165     125.3275    124.98067    121.06846    121.16246     117.1415    120.50556    120.37186    121.61935    122.39819    118.81943    122.74142    118.41661    118.09962    117.93741     115.1887    114.41704    113.20884    113.68316    116.63864    113.14363    113.54793    115.16175    117.03832    118.23529    113.58159    121.71908    117.11792    122.00947    122.26594     123.7286    121.52382    120.89446    125.41643     126.5449    124.60794    121.68039      124.863    131.95156    131.62811    130.84712    131.42279    131.63373    126.98835    123.80095    130.08603     124.7063    121.82096    123.32224    120.95893    117.68341    121.89042    120.64548    122.34701    123.90082    124.10111    121.34311     122.9043    123.56317    126.04062    127.11557    129.87235    126.67009    130.71021    136.67401     138.3668    137.21369    135.38576    135.17173    130.52233    132.89704    129.63858    125.36189    121.47441    122.07323    120.77214    125.16582    120.72183    122.63306    118.88167    121.30947    120.40587    126.16307    123.61646    121.29281    119.08139    123.66137    119.97108    119.43186    118.30833    120.45887    112.73884    114.25932     109.9466    108.45202    111.26504          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       23.00    278.23636    261.93584    247.37796    232.94266    221.38196    223.63576    221.25554    216.56646    208.93346     203.9085    198.29405    191.09823    190.50461    183.71301    190.36462    185.19886    182.34585    183.71337    180.36421    176.70754    181.60956    175.24095     177.2413    170.00844    185.50988    208.96332    235.92477    247.76191    232.23675    195.48266    167.78072    162.35421    156.53961    156.70352    156.39592    156.01334    155.12976    147.15208    148.33359     153.6636    154.79689    147.20208    151.01842    145.11315     145.6028    147.37164    144.58047    143.38463    142.31924    144.45579    145.82817    143.54931    139.91761    139.07085    138.31246    139.62046    146.09476     147.7171    149.57827    147.03448    142.70404    146.73306    132.69821    137.43022     137.6115    134.92486     134.4962    136.91345    141.84999    149.73765    162.49649    180.42356    183.91237    179.60975    162.81949    164.51275    167.46688    166.37043    160.54848    158.59968    155.54163    157.23674    165.22735    164.01432    155.91758    141.77905    134.55368    133.30581     135.2827    146.34028    169.56554    182.69742    177.39539    172.47589    149.19435    136.16632    127.52751    120.87026      123.397    121.04246     125.4292    126.05659    129.48936    123.36101    122.01131    120.16292    116.24551    119.26702    119.47276    118.01277      120.259    117.47541    117.36402    113.03922    118.26434    117.19502    118.27031    117.05096    119.36894    117.95763    114.91252    115.36859    118.95928    123.23268    129.72157    139.38638    140.10481    136.98141    130.36798    123.37857    124.06087    118.37236    117.78025    116.18381    116.96244    122.94689    131.01981    143.89609    159.00408    164.80161    161.76134    160.27247    151.04826    138.69661    131.73966     131.0194    129.02989    126.23313    121.43839    122.62207     120.9172     121.4307    118.51537    114.25014    110.79407    115.44845    115.54681    114.28132     115.5454    115.08205    115.36525    115.61879    116.60759    114.58886    118.23913    123.55433    116.76717    121.35271     131.2037    139.03044    142.64051    141.93744    131.04614      124.561     127.2126    129.18911    131.51314    131.52531    132.50751    131.10529    130.44615    129.93399    129.17605    126.64758    125.90775    124.71311    118.90313    118.44268    119.04555    117.89148    113.85259    116.93892    115.95819    114.62501    118.88225    119.00101    117.36214    119.98538    125.42482    124.85988    125.44504    124.74311    123.59365    121.03711    121.56138    116.99021    118.17494    119.74879    123.45783    123.76057    121.90947    123.30127    117.48039    118.30768    117.42666    113.38209    113.72671    114.38448    112.37912    113.58522    112.73568    112.45022    116.24343    117.03436    116.23369    114.76291    114.58391    121.16871    121.51693    120.75344    123.95369    119.62033    125.01766    120.75477    119.74884    124.93344    124.78834    128.26264    124.53329    130.02785    130.61088    126.09492    128.34663    127.15707    125.32768    125.30151    121.65105    120.35358    118.74054    121.59671    118.37649    117.65581    118.74532    123.68431    122.04022    122.17561    119.32438    120.90711     126.2689    125.41421    123.16205    128.64651    125.30887    129.74085     133.6863     132.3504    133.96056    135.24335    133.01535    128.92689    127.19987     127.3067    124.49558    122.71841    120.68361     118.3496    118.57551    116.23499    118.83715    120.86849    115.67978    116.36161    115.63401    113.77289    116.98975          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       23.50    276.52315    263.91342    248.27926    234.97637     223.6198    222.89323     223.8407    213.75208    209.56824     206.2076    197.54263    197.16933    198.90327    191.21853    189.52965    186.01541    180.22572     177.3878    182.35468    173.65716    175.12326    174.35813    177.64495    169.59176    187.38209    207.34668    232.54358    249.55287    228.65425    197.90607    165.56942    159.43852    155.40653    159.09837    154.12683    154.76411    156.45985    151.67559    149.78184    153.70941    145.18875    148.29858    149.15353     147.6007    141.36581    142.98916    142.77921    138.97927    143.39806    143.47952    143.26803    144.45894    137.47399    141.95066     141.8775    144.63352    143.83077    150.21978    149.89989    142.56237    141.67179    138.76058    136.55125    137.92096    134.82815    135.22214     136.7832    137.97147    138.61545    147.38378    165.11245    178.67879    187.78976    173.86376    162.57091    160.50126    168.39573    169.13481    166.35908    162.97904    157.59022    157.14071    161.81421     164.3857    160.10904    147.43808    135.28662    134.43762     134.5089    148.52054    169.11028    182.52215    186.42689     166.0961     151.5309    139.10636    129.26185     123.7119    122.82865    123.39861    128.04467    126.07956    126.71511    124.01036    124.19692    116.82312    115.21048    117.50901    116.19935    116.60562    119.83721    120.10794    119.43143    116.75667    121.11313    120.03936    115.97613    114.86266     117.8374    114.16771    117.27359    115.90202    118.53229    123.19194    128.67846    135.18186    136.47817    133.74237    132.79607    127.50228    121.87831    119.00728    119.40947    120.42275    115.34029    123.24267    130.82252    143.02329    157.43071    165.41694    165.38375     157.6306    150.32228    139.17934    130.13653    128.85472    127.17803    121.90285    126.71197    124.93676    125.12315    124.97191    121.32153    113.58967    111.16233    113.00437    114.62157    114.56346    113.27678    111.67562    111.64732    112.59911      114.327    115.41904    115.49979    118.19554    123.58353    121.99706    123.33452    125.96449    129.20576    125.67521    125.74865    123.27025    124.06316    124.81674    128.78917    132.49993    133.88503    129.28072    127.40465    128.23905    125.12553    122.20478    123.51374    122.86998     121.2055    115.97665    117.53081    116.30408     115.1316    115.01894    115.95645    114.61061    114.31458    116.73565      116.979    119.85172    122.96933    123.58311    122.87087    123.90441    121.12341     120.2761    117.50559    115.57951    116.33151    121.60335    122.43543    122.79793    121.63799     122.3301    118.69521    117.59909    117.63271    119.33793    114.45983    112.39917    112.38849    113.33111    114.17928    111.49171    113.86382    113.48475    113.56207    114.86369      115.357    119.46372    121.30787    119.34061    122.61373    122.96902    120.24987    119.61116     121.0424    123.38727    123.92162    126.14638    124.15735    130.76744    129.74045    127.70819    126.25799    125.54057    125.00521    121.18925    123.20188    120.14627    118.19209    119.12828    118.81043    112.21634    120.28589    117.66961    118.03711    119.63613    119.14183    119.27034    121.51242    123.88537     120.2719    123.13709    127.97108    123.15509    128.58596    127.20035      129.314    129.35221    128.76581    127.90761    122.65874    118.97576    116.64539    114.02752    105.93695          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       24.00     273.1821    262.24975    251.44863     239.2724    226.77318    222.89255    221.69328    213.91282    210.34202    209.40217    200.80032    193.89898     196.7047    196.56234     188.5308      188.066    182.23943    174.15017    182.09577    171.01016    172.62397    176.27437     179.1694    172.34451    185.36831     205.2444     238.3632    246.95551    227.76369    199.82064    166.38586    159.01783    156.68702    152.86381     155.8467    158.55896    151.91075    153.22602    148.99157    147.43181    153.53403    149.73816    146.32944    150.85399    147.04651    142.42857      143.993    143.23556    147.07798    141.04822    143.87248    141.72622    136.45773    140.28554    137.48677    143.02224    149.26486    147.84628    151.91892    144.17283    136.14603    137.90669    136.85964    139.36459    140.02626    137.71988    134.32009    136.01754    138.16369    148.36075     162.7019    180.19903    188.59303    176.59247    164.81524    159.49203    158.21446    163.65898    167.60466    167.39169    163.72979    160.20139    161.02287    156.94124    159.19748    146.84058     136.2376    129.02943    138.40934     150.6071    167.97799    184.96246    182.69368    170.35771    154.20634    139.25646    129.67177    126.55597     126.7422     124.8699    125.62976    125.38319    128.29871    121.71436    120.29157    122.76217    119.89464    112.94097    115.29461    120.21861    117.77323     120.8172    116.53123    116.99843    114.26352    116.91388    121.07175    117.14116    117.84107    116.76379    115.68672    116.08989    117.21935    115.60524    131.14918    136.78752    140.21576    137.25458    133.56252    128.25574    122.67551    120.56782    119.27145    116.71542    118.64896    116.19364    131.70653    139.72795    156.49386    164.33877    165.96999    158.04256    148.55854    136.27223    130.16239    122.84902    123.84176    123.17919    124.40903     125.5232    123.76334    122.81217    119.48162    117.59043    112.66199    113.55657    113.76622    113.56289    111.97838    118.15858    112.29954    111.97129    112.39737    115.30342      116.383    121.62011    118.80903    120.21707    121.78274    121.47345    123.35316     122.6661    125.69273    125.43304    123.31449    130.41552    128.51883      128.462    130.40325    130.95163    124.62948    125.08964    123.97121    126.85279    122.51186    121.15804    117.34418    115.71913    113.43018    114.82825    117.19112    113.49085    116.23717    116.61159    113.05559    115.74754    118.39398    120.65126    124.09017    124.00395    123.03664    122.60915    121.43777    121.51183    118.41749    119.02103    119.10541    116.06972    119.24166    118.59817    121.00803    121.42451     120.0958    115.11824      114.566    116.46295    113.77396    112.51299    111.97664    111.97272    111.27029    110.35114    112.28591    114.69423     115.2746    114.66144    115.52246    117.95247    121.55538    119.41669    125.14828    119.84742    119.01182    124.23782    124.23631    121.32117    124.91601    127.05364    127.00821    123.25826    126.77821    127.01718    127.14026    121.57261    124.63582    113.38125    122.31859    119.56635    118.11198    118.55765    118.69972    115.38348    113.97576    115.94154    114.56031     115.8911    117.96727    114.25031    114.82993    116.12651    119.57063    124.09266    122.77565    117.42756    119.12698    114.69826          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       24.50     270.1368    260.86357    257.56564    239.61501    229.33863    220.01055    221.42551    217.95841    213.72907    209.25773    202.92423    193.12055    194.86361    190.13305    187.71896    189.58043    183.21954    174.05461    179.06698    171.60771     171.8094    175.94774    177.07729    173.68813    183.42644    205.28759    237.44471    245.10091    226.36035    199.77346    170.68264    156.18604    157.36463    146.59274    154.40628      154.186    157.66198    153.29032    150.82068    149.77245    151.50704    145.87685     150.7395    148.56722    147.91198    142.86018    144.27771     141.4488     144.6587    143.70713    143.53266    139.12127    137.30999     140.7579    141.28371    140.88351    144.51752    154.21759    149.17283    146.14889    144.23041    139.43101    140.59089    137.41955    139.46381    138.11094    134.42256    136.08667    136.19924    148.35461    160.98869    179.67665    187.45926    177.92718    161.41523    156.73746    159.60164    163.86717    170.92445    170.20721      166.024    157.09053    157.30013    154.94692    154.27246    143.04568    137.54224    135.69839    135.01231    151.22531    168.79171    184.39465    184.60364    173.97183    156.15018    142.18479    132.56436    127.42816    124.55163    129.07253    125.39024    125.89662    126.01441    126.87056    119.37265    118.73297    120.21376    114.30888    115.79492    118.73868    119.98294     118.5788    116.74179    117.60178    115.98634    115.22328    115.78096    119.74098    118.96884    117.21501    119.69876    116.89314    116.59202    122.59678    127.78926     133.5586    136.64809    133.87434    127.11421     127.2945     122.0162    115.66994    117.51795    116.06023    117.15125    121.05534    130.13483    140.71226    153.96224    162.90531    164.22087    158.21024    144.30418    133.94503    129.84868    124.75468    119.47854    119.76004     125.4627    124.49366      122.248    121.92346    117.99835    115.51277    115.40897    109.72423    111.65331    116.50844    111.28621    111.74342    114.70768    112.04355    114.07862    115.80612    122.17014    120.20615    118.83471    118.36759    117.56293    121.61681    119.08532    122.61825    122.49038    124.80495    126.16056     127.2999    129.07411    129.72069    131.31946    130.62109    129.01224    124.79312     122.5432    117.28717    119.84717    119.49334    115.18252    117.34228    114.44325    111.90705    112.33079    114.99393    111.70117    112.37296    116.52836    114.26361    117.59034    120.68855    124.19636    123.75657    123.69563    119.48436    119.12492    121.17486    119.67186    115.32657    115.62761    117.04141    120.28343    118.28199    121.77593     118.9505    114.85971    115.56874    113.40907    112.72262    114.01543    110.46289    110.68421    112.15455    109.30892    109.98799    116.26066    115.60695     113.4149    116.54568    114.26186    115.58909    118.90208    120.14171     120.9892    119.82763     118.9265     118.2636    118.40811    122.04528    124.41004    121.91835    127.61804    128.39159    129.06608    126.33047    127.70279    119.84162    122.45171    117.54089    113.64965    118.07086    113.57877     114.3691    115.52767    114.31059    112.11665    111.25072     114.0613     107.9973    108.94947          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       25.00    270.03424    259.92088    264.48637    237.06586    230.03607    219.11556    222.07131    222.23916    216.38588    207.71266    199.69615     195.7304    193.04967     185.9818    185.75942    190.16186    182.71962    176.67799    177.33565    169.30229    172.67235    168.94052    176.03449    172.07485    181.16063    205.80633    235.50614    249.52214    226.92122    192.38334    173.08111    156.79946    158.25821    155.38632    156.44113    154.11116    160.13863    155.49466    149.65287    144.50066    155.35274    146.65684    149.21955    148.71544    144.91112    142.80739     142.5088    140.32797    139.18365    143.22942    139.31009    143.00986    139.57403    141.79258    139.46533    141.37748    141.69497    149.41212    148.00906    146.48205    141.88826      138.587    139.65057    139.97906    138.06714    136.49714    137.78688    134.79018    145.60678    146.27701    162.70153    181.43178    185.95121    175.16656    162.58044    151.28446    162.54665    170.58398    177.58016    176.61765    165.55561    156.99559    157.33998    152.31652    149.54489    142.59621    137.03275    130.51355    141.72005    150.55537    171.64798    188.51057    190.24129    180.37603    155.96044    142.35151    134.49615    125.14529     123.5418    123.11365    125.54313    126.58052    128.14238    125.55491    120.18676    120.04429    117.92566     116.7288    113.23246    118.53773    118.76677    118.02264     114.5185    116.85152    116.77909    114.73907    116.04015    117.64703    118.98596    117.10916     115.7948    116.21008    116.95866    121.75747    130.18359    130.75872    133.40383    136.07758    129.17657    124.40241    119.65216    119.62215    118.32759    118.96054    120.73687    114.79245    129.31017    142.20248     155.1229    163.18375    162.32639    159.08665    146.74468    139.27817    128.18744     122.2598    123.61908    123.34028    121.94588    120.89062    118.89736    118.18746    114.78576    114.03112     111.9816    109.66224    109.45903    111.58314    113.29093    113.26781    112.54279       113.33    113.05127    115.25536    116.22025    119.74832    122.94277    118.60622    118.58458    118.43995    118.87925    118.65374    119.22862    121.73664    123.71135    120.12778    126.74425    128.35098    125.63365    127.66865    127.23355     124.5319    121.75576    120.76511    119.40631    116.21351    116.88764    113.23611    112.58023    111.39485      112.344    114.07021    113.27405    115.82347    114.53026    113.55015    118.45117    118.30386    120.16474    120.57475    121.89789    119.80551    117.42852    119.72661    115.30814    114.88864    112.99331    116.40831     113.7547    123.99229    118.72144    118.43363    119.23978    118.02282    116.34933    110.46562    110.48272    109.63104    111.78608     110.2229    107.30701    111.93878    110.90168    112.65057    113.78955    113.75632    118.47905    115.87346    118.23593    118.56032    118.65121    119.42783    117.46036    119.75486    114.85409    119.06212    122.28103    125.82617    123.02365    123.12515    121.49279    126.64608    123.08015    123.69278    118.73563    119.18451    112.62382    111.63131    109.09114    106.66087          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       25.50    272.79358    262.22307    267.33979     234.4321    230.54538    219.90309    223.57311    221.24641    211.80593    208.47991    198.15067    194.06273    183.82677    192.08574    190.47151    186.10076    183.65848     174.6801     179.4787    168.96294    171.59854    170.34201    176.18211    170.55924    178.47534    204.70748    229.92899     239.8901    221.34801    190.48348     173.9816    156.70033    159.73583    158.17182    150.66908    157.58085     155.7391    150.10583     154.5366     147.4974    149.86976    148.94059    148.31823    147.99278    146.94469    144.50065    145.80185    145.38454      140.791    141.35608    140.76256    142.35702    138.61745    137.56858    141.10423    141.09302    143.87843    151.10542    152.85923    147.81763    141.37517    142.42265     133.5333    137.66233    138.53578    138.50862    137.70568     135.6728    140.97622    150.71236    163.66894    181.89136    188.77686     175.9378    161.68593    155.84319     164.6885    176.05816    181.92356    178.24929    171.67837    158.45038    158.86564    158.38524    148.94432    141.05161    136.60946    131.12591    137.49535    151.81507      169.173    184.14169    186.84825    175.21911    154.14065    139.11441    131.14054    127.65264    124.87329    126.15209    127.03938    124.71447    122.46202    125.04051    118.42033     117.0941    116.92647    116.95752    114.05576    121.17395    116.77497    117.92052    114.51959    119.87394    116.26002    112.50216    116.08663    115.31389    114.87564    114.15442    117.87837    114.79017    114.57971    120.50793    130.72521    136.11207    135.07484    132.88288    131.69424    124.19668    118.82796     117.9234    121.18102    116.52012    115.49425    121.10382    129.21788    140.79129    152.40622    160.74131    163.93627    161.14264    147.73138    136.17556     130.4264    124.41006    123.71141    121.96909    119.80007    115.89441    115.60344    116.50704    115.36853    112.21104     113.1145    110.43117     108.5269    112.65991    113.14579    110.59079    111.77435    111.08948     113.2261     115.1189    119.87687    119.76611    121.60112    119.92976    116.79678    116.67863    121.71498    116.36036    117.33768    117.40755    119.82123    121.97053    120.48102    125.05288    128.85658    125.81334    127.46115    122.85417    122.83007    120.28147    117.11277    116.66613    117.54997    113.87302    114.30426    112.22322    112.28837    115.02071    111.83669    112.17104     115.6117    116.77963    114.62176    119.58279    119.32867    121.15763    123.78242    122.03935    118.22284    114.35766    118.91269    115.03382    115.43244    116.82692    115.14598    115.99764    115.95097    119.82267    114.33352     118.4561    117.52116    114.13843    112.98328    110.92308     110.5428    110.52254     106.4293    107.70574    110.98968    111.09749    113.67844    116.26359    113.01909    119.45119    114.61959    118.52607    116.35195    119.79001    118.94171    120.13469    114.20386     122.6925    119.60873    116.84656    121.07003    117.06768    115.42356    112.52339    110.53021          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       26.00    275.01368    263.45768      262.694    236.61271    231.58954    221.06605     226.3949    217.54377    211.25497    208.64697    200.76663    195.85051    185.28928    195.65373    192.39969    179.65261    181.01722    174.70223    177.47335    173.37097    170.21832    172.36346    176.74005    174.38851    181.84399    203.23381    233.27882    238.91533    219.69758    193.05973      175.237    158.59704    162.47578    156.55435    155.38944    156.43717    154.08802    154.00853    149.63459    149.58392    145.04889    149.08819    148.86365    144.76628    146.72395    145.36836    141.27657     141.7972    141.24158    145.89987    144.29012     136.9897    138.90655    140.81746    148.16468    146.22204     147.6951    144.94671    149.07246    151.75235    140.98313    141.29227    140.58352    138.47561    139.68044    137.91776    139.10956    141.55801    139.39402    148.85958    165.37565    180.15874    187.16727    180.52496    166.92585    154.48229    166.30014    178.63612     180.7405    178.21633    169.87662    162.25789    164.44709    157.32259    148.13318    140.60582    133.76507    135.81348    136.45932    151.41221    171.64843    184.03766    188.18052    174.20342    154.46027    137.55037    130.43003    126.85583    123.12699    125.23429    125.02173    127.98772    126.11338    125.02673    122.17532     118.3737      116.809    118.38364    118.49653      116.962    116.82896    118.15318    117.01888    115.50722    116.35142     115.7998    116.20317    116.56618    117.51748    119.84052    116.06093    113.45732    117.92649    120.88878    128.33136    133.98325    135.54546    137.05538    128.48515    124.43848    120.72156    118.57089    118.47594    119.56996    117.12031    122.65192    127.72766    142.23213    153.88573    162.36357    163.97221    158.18518    150.49958    134.87996    129.63531    125.84318    122.62066    124.19467    123.25078    121.91815    119.19842    117.86525    114.71578    112.30727    112.41971    111.39775    112.30282    110.41063     117.2104    114.27366     110.2706    113.26543    112.40157    112.92986    117.17548    118.45056    117.54873    119.11911    119.16641    118.91402    111.94614    119.51621    117.34814    119.44461    119.77646    122.39836    122.01045    128.19672    128.11387    130.75839    129.04942    124.02679    122.76143    117.22876    119.26986    115.77217    114.77193    113.77949    111.76257    112.04195    109.45332    112.47999    111.19674    112.87782    114.48193    114.21454    111.92538    117.19669       120.52    120.90799    123.16165    122.05135    118.91931    116.23833    117.29066    114.35422    113.64547    115.10773    118.81225    117.17684    119.34474    118.10294    117.45924    119.11263    114.51111    110.26797     108.5698    107.74232    109.67514     106.7551    107.26475    108.14596     110.3456    108.62185    113.13225    111.81723    112.64167    115.60623    110.62764    113.99857    113.87444    112.75274    112.31154    109.05606     109.4046    105.45197          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       26.50    275.36462    263.82609    257.89478    239.39222     232.0618    221.43067    224.63758    214.79006     213.5703    207.58209    204.03996     199.1908    191.82568    195.76024    193.57903    183.74787    184.16308    176.55603    181.83567     180.2884    173.54306    176.89359    171.75856    178.03423    180.06873    206.53886    235.60255    244.87821    223.66972    191.01701    172.28621    163.55066    159.09485    158.37974    157.07093    154.96087    156.62361    153.12446    152.42337     151.1212    154.03209    149.19229    148.72278    147.53902    144.83508    147.27266    146.06978     141.9121    140.93501    145.67608    142.77797    141.27839    146.88352    140.44306    143.34083    140.48508    146.31118    147.25445    151.07367    148.88484    144.13827    139.44221    139.80521    137.83171    136.31459    140.70335    136.62078    138.49195    139.83423     149.2957    163.36511     179.0161    185.20324    177.02915    163.79696    155.35041    163.43608    170.09134    179.47057    174.73511    169.90218    166.45563    165.21858    159.41602    150.06701    139.17631    134.33034    133.57995    138.02512     146.1592    169.36497    186.42252    185.15878    174.67704    159.29478    135.36396    129.07561    125.89367    122.81574    125.83653    125.26655    126.24576    124.40924    125.93828    127.66768    118.58569    119.27804    116.41609    116.64462    115.68618    118.58244    116.79089    116.52616    116.21763    116.61635    115.39228    113.18039    117.00101    117.10869    115.41068    114.60899    115.55665    118.98083    119.46748    125.77764    136.09978      134.873    134.37115    130.78022    122.07804    119.31755    117.11408    116.31124    116.90082    118.56468    120.26975    130.31151    137.60184     154.5131    162.70665    166.76998    158.47285    148.13874    136.41339     128.7203    126.10021     124.7019    122.99264    124.73245    120.29985    116.82861    117.18752    118.48776    117.16575    113.20492    112.34992    110.78727    108.00662     115.1423    116.10203    114.80735    111.23332    113.32839    113.80173    116.34592    116.73376    116.27073    121.31177    120.93499    119.47363    113.27459    119.57664    118.01661     118.5151    119.30671    121.56242    122.90783    123.85171    126.07092    127.32586    122.45809    122.16118    120.01638    120.60178    117.14789    116.95168    114.00658    116.54631    113.55578    116.53601    107.33376     110.4255    109.33304    109.88642    109.01493    109.30068    114.28955    115.50375    119.55794    119.19091    119.96778    119.78614    120.45559    114.93635    114.89502     112.2872    113.57707    116.16104    116.98804    115.60632    115.07229     117.7387    112.83131    111.74604    110.39309    111.65264    105.97463    107.34053    107.85723    104.71383     108.3636    104.81577    106.35865    104.99642    109.52053    110.18528    111.93446    108.43883    107.01116    103.53001          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       27.00    273.41577    262.94439     255.4053     242.0516    231.86879    223.03412    222.82266    213.86726    213.23351    205.60023     200.5474    199.87078    193.62591    194.12105     194.8509    188.64902     188.5173    179.27304    183.49338    175.92077     175.2225      176.822    173.73809    178.03936    180.69982    210.40741    233.09193    244.91297     223.8704    201.77549    172.67435    164.66912    155.63921    159.01986    154.45323    154.69626    156.35178      150.021    153.16073    153.04507    153.70801     149.3682    147.36396     147.8915    147.62675    146.45338    142.06872    148.75235    145.19002    144.41744    142.65309    145.43358    145.72285    137.91089    140.64333    139.75843    144.01213    144.93064    150.97127    151.53057    146.98616    138.43137    137.28533    139.03332    142.58279    136.92309    138.00873    137.66646    139.55996    154.03828    164.04888    178.25102    187.81732    181.76916    164.15287    155.11838    160.41513    166.57946    172.37735    163.28523    162.25869    166.18296    163.23159    156.75122    146.38685    138.72495    131.08729    132.95826    137.56793    150.90176     169.0052    188.25135    187.96246    182.52683    162.38706    146.18243    130.47361    127.51128    126.61577    124.82766     125.6699    125.42054    129.37812    126.29149    122.90864    118.34469     118.6808    114.74683    118.21688    115.33299    118.78471    117.11709    118.69311    114.58065    116.29425    117.81047    113.91172    113.28163    114.81818    116.99067    115.77173    115.24921    116.97939    119.51791    121.92652    133.17748    140.24669    131.50256    128.15375    123.98948    122.68794    116.67018    117.01657    116.04959    116.23527    117.14908    130.54035    139.54753    152.55511    163.94955    163.97104    159.54289    151.02128    134.29576     131.1647    127.47344    125.40257    121.73193    124.57202    121.23401    116.95626    116.43949    111.90929      111.221    115.52851    113.63109    110.94487    110.07806    112.08566    111.77046    114.80513    115.27647    110.82429    113.81026     112.8423     117.3229    118.36616    120.99171    120.50111    119.30396    121.55248    119.47877    120.26298    119.00006       121.03     122.4546    123.34263    124.75364    125.10606    128.00775    126.29786    125.85989    120.50509    121.69104    114.24613    116.56786    113.26611    109.79758    114.33654    111.58937    107.52792    108.35851    111.15437    111.11877    112.74322    113.80131    111.59641    112.06314     116.0391    119.81381    119.56932     121.9723    116.37809    117.51516    112.16088    112.77547    109.65358    110.95746    111.27963    117.41403    111.50625    118.45052    113.77814    109.87101    110.52641    107.19055    106.75162    104.15113    100.44467    101.50624    103.89371    102.51085    103.53569    101.01793          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       27.50    272.53768    262.15968    254.92915    242.98281    233.25101     226.1136    219.59267     216.0439    211.93552    204.65684    198.64398    199.08262    194.06701    193.06824    191.89366    186.96006    188.21517    180.05467    178.85132     177.5378    175.81565    175.51588    178.74276    174.55143    184.01921    204.03316    231.47324      242.306    221.62706    198.94413    171.54947    164.08079    156.23414    155.08438    155.15359    155.08629    155.96894    152.57092    153.30564    151.90897    148.95412    148.75144    150.30708    150.11636    146.80603      148.763    144.78876    142.76281     139.6541    139.45692    137.58523    141.68708    140.48674    142.85446    137.09764    145.98868    145.23639    146.96973    150.45367     150.6895    146.01078    140.00403    139.70035     135.2099    139.97199    135.61848    142.18718    140.02967    139.58211    151.05457    166.81572    182.99039    185.67668    177.59452    159.69023    154.57871    149.91765    156.57025    162.79117    160.02605    163.22882    161.13651    163.70615    153.58681    146.13632    134.17968    135.94839    132.81397    139.39355    150.52318    170.06487    183.13529     188.6405    184.81221    168.29888    151.71243    140.80941    134.21373    126.58871    127.76413     128.5307    129.10776    126.99898    127.61832    121.01351     119.4063    118.63214    117.46583    113.92182    115.25266    113.72112    115.96177    115.91901    116.04894    114.85427    112.15317    116.46809    110.86024    117.48971    118.12827    114.18824    112.74195    116.67414    118.48003    123.86985    134.81532    135.31151     135.2189    129.44391    122.64033    119.91235    116.98109     117.2627    114.03514    115.43785    118.62225    126.19366    136.76238     149.9238    163.82938    163.76935     160.2911    149.04305    136.01241    128.74971    122.17163    125.42156    124.22346    121.62046      123.149    118.83361    115.54968    115.89422    113.49741    111.22261    112.09282    111.30136    109.94938    108.36185     109.5955    112.07382    110.61332    111.81263    113.42594    114.25973    113.64916    117.39038    121.01325    118.68335     119.0387    120.83353    124.44861    123.87305    125.26953    123.16148    124.15701    121.43657    123.46555    125.83371    126.24252    129.39604    125.67348    124.76483     120.4032     116.3245    113.63348    114.82707    112.37059    111.77472    108.54234    109.11536    107.44771    107.82528    111.74048    110.62554    112.56148    111.55717    111.08925    112.76249    115.91595    118.32814    115.84472    111.31464    114.87075    110.72953    110.73589    111.18658     113.6625    111.73113    115.03703    115.22545    113.08258    111.73922    109.70075     109.8874    105.65353    103.37171    100.88377     96.80528          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       28.00    272.17483    261.48564    255.50934    243.32661     233.4635    227.01091    217.58953    219.49818    211.03752    205.72776    199.36488    198.99084    193.73211    193.96443    189.81117    184.37282    188.22146    179.58098    175.36937    177.80336    174.12846    176.54988    176.88421    174.85918    183.42369    201.74676    228.27773    239.93864    223.35279    194.71607    172.29314     165.9706    160.58105    154.07101    159.29376    157.70269    155.27749    151.18328    150.45796    151.33779    148.81153    150.21361    148.32143    152.09578    147.25403    146.44237    147.96642    140.33521    142.81957    143.07761    141.56854    146.32067    143.13275     146.5171    139.28237    142.15275    144.52853    149.80003    150.24883    152.74829     150.1603    144.01684    135.70521     135.2767    140.82982    139.14581    140.73515    140.74942    140.33147     150.4821    165.80869    174.77048    184.62911    176.52173    161.32229    154.79839    148.71269     154.6159    155.37182    154.50003    155.86009     162.6174    164.97376    152.47734    146.87341    136.39764    135.15582     132.9301    138.44045    148.59808    167.01495    180.84919    191.57824    182.07847     169.4227    156.19495    145.29793    138.49527    132.15009    128.94808    131.45794     126.8009    125.94559    125.41545    119.28907    116.40348    114.37291    113.97012    117.26156    117.53679    117.09576    112.74066    115.96265    117.99134    114.55451    118.60442    113.42963    115.83152    113.55268    115.62972    114.65625    112.89783    114.68298    118.67163    126.38853    130.28847    133.29897    132.98735    129.45576    125.79966    116.66941    115.20156    113.44187    116.91701    118.58066    121.23807    128.37591    138.27358    150.53863     157.8692     163.5883    156.06918    147.73074    139.25639    130.90617    125.24588    123.31365    120.68326    123.25704     120.9108    118.24998    117.93603    110.43127    111.55933    111.87009    110.68473    110.77669    111.94124    110.38679    110.11488     109.7329    111.14893    110.69377    112.23508    113.94334    116.42109    118.69342    123.05749    119.25252    123.78069    126.52617     130.4387    125.20476    128.07018    125.61755    126.30347    122.96272    128.37764    126.32679    121.48995     130.6686    123.38547    120.57225    118.12581    115.70419    112.88095    112.10066    109.88919    108.58912    108.64951    106.87691    108.03857    108.10809    110.61636    110.44874    109.09031    109.69191    114.12745    113.82538    115.04753    116.07407    118.45974    113.88505    113.47599     111.6333    107.93195     109.9908     110.2404    110.63322    110.42908    108.82334    108.38672    103.50849    102.68345          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       28.50    271.95637    260.77176    255.43945    242.25347    231.19354    226.26975    218.87625    219.36961    208.57084    203.83515    198.26204    199.62502    194.18354    194.62127    190.98902    182.41627    183.17085    181.32321    179.94524    179.58153    173.84126    175.02957    176.24629    177.06018     181.1636    199.93526    225.39632    239.59778    225.61379    199.89356    176.60616    166.80178    156.08691    158.83231    159.08421     153.2234     157.8643    149.04798    152.77824    149.93937    148.84262    149.89314    149.48063    145.78212    148.88796     147.2089    143.51065    139.28524    141.92651    144.19904    142.52149    144.49177     141.1631    144.51195    142.26952    142.30351    147.02003    150.49106    150.06424    151.27522    146.28929    139.86724    139.58793    140.00224    142.91376    139.84869    141.32357    141.92479    137.48255    150.80751     163.5269     179.8968    183.66495    173.93462    162.43076    156.52009    147.65948     153.0038    151.94836    157.85435    157.00274     159.8869    167.27426    161.59927    147.75874    140.26246    133.31183    133.78188    141.21284    153.02526    168.58677    178.82635    183.65571     176.1206    164.44535    153.46541    146.95989    137.65403    131.77143    126.62104    127.76605    128.46024    127.23119    125.35919    120.42587     118.2063    120.06246    116.68976    116.70439    120.28688    115.81725    114.38619    115.76989    114.26523    112.04587    116.73786    115.75728    118.72017    114.19037    115.70566    114.41308    113.41658    114.28565    117.74944    123.79965    133.20138    136.06531    135.12604     129.9609    123.81223        118.5    117.68728    116.46888      120.963    117.40385     120.1239    126.46125    137.59826     155.7422    157.04979    166.99925    159.62852    146.77137    135.73026    130.89194    126.85005    121.26089    128.73206    122.72671    123.33034    118.26061    116.56515    112.31671    112.67753    110.02715    112.65135    112.75958    113.90503    110.97217    108.00975    112.48103    108.79924    111.23823     113.5681    114.12646    115.18136    115.62679    121.03659    123.85295    128.15236    134.94222    138.09389    132.67957    128.89604    127.72264     124.2842      125.503    126.58278    126.55518    123.82707    124.55586    120.81695    121.93555    118.22932    111.88557    113.44559     108.1722     111.1722    111.31562      108.848    106.53982    108.34274    110.64603    105.12546    107.49819    110.66731    108.38727    113.10657    112.91317     113.0197    113.29374    116.05625    113.89708    112.49829    111.49452    109.18356    104.44625    101.08951    100.98298          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       29.00     270.8037    260.42738    254.72898    241.00024     229.9843     228.2728    221.70793    218.12802     211.7164    204.74509    201.82939    199.71366    194.01977    193.37369     190.5252    183.25941    182.43664    182.37633    182.67345    180.84039    175.49226    174.59453    175.72968    177.60274    180.55471    202.21849    227.03373     236.9779    221.98743    192.86955     167.9787    162.36569    159.82378    159.31856    155.05025    153.51692    155.03298    155.55459    155.27562    151.78211    148.56328    150.50067    147.97635    144.78337    148.08476    148.07214    145.05548    142.28717    144.30613    140.62865    144.08833    146.24401    140.33613    142.21849    138.80187    138.97608    144.93816    149.33172    147.03969    147.47691    140.15612    142.56574    139.09484    140.55683     139.4006    138.40829    138.33532    137.97473    138.93313    150.02007    167.17279    182.75871    184.17053     174.9292     162.6969    156.73354    151.18719    152.66978    154.79484     157.8672    161.00581    169.62943    171.15813    166.60139    153.76325    139.52296    130.77626    131.62447    138.30252    154.54729    165.07505    178.41927    179.07584    172.10219    154.47778    144.06014    139.59136    132.26925    132.69168    129.62552    128.64721    128.38174    125.88726    125.53002    122.24528    118.29353    115.62828    118.22759    115.49887    114.78248    117.41547    113.77615    111.21695    114.20533     112.3438    113.34063    113.61541    114.46845    114.26756    110.95358    115.91884    115.47259    115.35437    121.05883    126.04594    126.15914    131.10703    136.05851    129.80231    124.03013    117.86914     116.7065    115.61023    109.01184    119.07394    119.72288    129.71894    139.63302    157.56608    162.14593    163.59398    159.40677    151.33533    136.16796    130.05029    123.40154    124.71229    126.63407    124.48805    127.95204    125.57393    121.82103    117.23884    112.59896    110.26068    109.45749    110.33202    108.86909    109.71178    109.88143    108.27542    111.15092    112.18003    112.61696    111.94857    111.54872    116.05758    115.48861    122.57387    127.10307    127.52899    130.65468    125.94185    122.21286    122.92393    122.93587    124.68949    124.28105    126.88036    126.97401    121.10932    122.93229    115.95849    115.46638    116.04381    109.68282    110.65733    104.66825    104.93203    108.15492    107.68785    106.75802    106.05978    106.22322    106.16571    106.59257    107.97959    111.44928    112.90703    110.67302    110.17016    112.32802    109.84996    103.28688          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       29.50     268.7006    262.05012    254.44172    241.21554     231.7682    231.21003    224.30599    219.95343    216.51134    211.01508    204.88185    199.26262    194.54663    190.24516    190.71226    186.26874    190.12313    180.64227    175.25691    176.09173    176.00982    177.76854    171.86611    177.68248    186.27432    204.18052    227.06596    236.44422    226.90171    195.29377    171.78671    164.07707    159.04364    160.08993    156.46619    153.60908    152.90461    154.55606    154.54103    152.55033     150.9107    151.12747    149.82088    149.81111    149.66136    151.73535    146.29248     147.5654    148.59001    137.86374    146.75722    139.74277    141.93756    148.18134    142.05655    143.84707    148.17799    149.43306    146.95388    150.51074    145.77221    142.02409    139.53488    139.09543    137.55897    138.43456     138.6379    142.92366    144.10216    152.42832    167.46602    182.02512    184.65014    180.11383    163.64246    151.53552    156.53674    157.62954    158.43586    160.07553    156.24506    168.55829    173.33683    166.82157    150.62957    140.62438    132.38272    134.82669    141.90324    154.98249    168.06179    181.51392    177.50479     170.2227    150.96271    140.84207    137.48759    132.04639    132.21656    132.34156    128.62967    127.72024    127.61843    123.76858    121.58589     117.1433    116.62859    118.05451    115.79706     117.4424    117.24761    115.67758    111.86359    115.48936    112.75893    112.72268    114.87018    114.27756    112.04218    112.87106    110.71258    113.37659     114.6091     117.9148    122.67641    132.47905    132.82582    131.45333     127.5083    126.02706    119.02239    114.96002    116.62707    111.12046    114.77704    121.51297    127.58955     141.4487    149.23303     163.2173    162.02386    157.89244    146.69301    136.31828    131.92513      126.313    125.25422     125.4107    125.07976    129.68685     126.3061    124.15292    118.61518    116.46341     109.7415    110.98661    110.90829    111.36592     108.1653    109.70531     111.8167    107.55944    109.88618    110.22683    115.69971    115.83273    114.59016    113.28955    117.88769    120.20302    120.70594     118.0846    120.32599      117.062    116.66495    122.42073    124.83934    125.67223    125.74774    127.03994    122.68934    116.61985    119.70188    114.44557    112.47461    112.68263    106.59112    106.94013     101.5473    107.19765     103.0777    103.61594    104.61383    105.23531     99.29081    107.62304    106.95757    103.98961    107.39732    106.36488          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       30.00    271.14643    263.12346     255.6997    244.66043    231.60887    230.18566    223.01692    222.26974    214.45376    213.99744     205.0867    197.14127    193.02422    192.37229    192.41498    189.79699    189.02091    181.70414    172.19349    174.65298    172.34188    174.57701    173.01875    176.67985    191.63932    204.39442    227.46229    236.35259    223.74591    196.49306    176.95834    165.79598    158.09526     156.6809    159.05663    151.23148    150.34504     156.3704     157.9211    150.35667    151.45234    150.50527    146.59276      146.017    144.74509    150.57891    146.54342    142.58641    143.74844    146.38665    141.07539    143.73842    143.43605    142.73112    140.81962    138.78629    144.18875    144.50411    155.93248    147.14177    148.82176    144.33067    139.27753    138.82291    141.73596    139.47156    137.55503    139.37391    144.69727    156.31927    166.27803    181.15387    185.27745    181.48415    163.78053    154.93713    151.01269    157.43998    158.59438    161.88999    164.02164    161.92132    168.50743    164.23902    151.93748    139.70138    137.38904    132.89004    139.07799    153.09934    168.90924    177.48636    180.26958    170.39238    153.66178    140.56127    130.08507    134.87868     138.9701    139.16001    132.00924     127.8938    126.37826    126.10531    122.10037    118.68007    110.96885    114.53838    116.12208    116.73197     115.9047    115.35119    116.18859     112.4692    116.41321    114.22392    118.11781    114.63506    110.30695    113.66053    112.42172    113.46047    118.99327    116.90978    122.28748    129.70968    132.29161    130.95022    127.84301    123.13123    121.41176    118.44023    114.38061    114.77095    112.66383    121.03013    128.58109    137.13104    149.31981    159.21109    165.07475    155.38475    147.70121    135.45483    128.62757    120.65294    123.51565    122.28657    120.99216    121.95008    123.75988    122.27528    118.69587    115.33659    110.63146    109.82162    109.46408    110.99687    109.44319    113.01515    108.22896    105.08334    108.21353    109.82662    112.28037    114.49991     115.9557    115.45861    114.13895    119.01803    115.42042    115.40717    117.85125    114.42694    118.11241    117.67529    122.96082       127.44    124.98565    124.29242    118.29364    115.19146    115.36462    116.64728     114.3259    113.73935    110.88416    103.80129    103.52267     102.7971     104.0372     104.0544     100.5211    101.71363     98.47382     93.32372          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       30.50    275.42038    262.86991    258.53708    246.41559    229.79206    229.03414    221.93817    224.00176     210.9461    212.97301    201.12081    198.22887    191.30349    194.19799    190.57432    190.09553    185.59149     178.5452    176.93162    176.37006    170.42648    171.90323     176.4787    175.35804    191.45354    209.76314    227.93963    236.34841    222.75282    191.35645    172.86242    165.40982    160.92441    160.88749    152.28707    153.12026    155.18507    154.53204    152.88253    157.02457    152.29178    151.17616    151.13123    148.21146    145.14185    146.57756    146.46428    148.49362    140.39186     139.4163     145.6418    147.06144    141.92457    139.83342    141.18868    139.46249    147.74303     151.3876     151.4393    146.90975    145.54733    143.10231    141.18201    137.12616    137.89548    134.61502    139.70115    139.61339    142.29247    154.91221     168.9126    183.03594    188.11157      174.992    162.26332    156.78045    150.36381     151.5323    157.95972     159.0792    158.57016    161.16151     162.0695    158.37953    152.35142    138.87936    132.67358    136.33367    140.85782    155.22254     173.4346    183.73701    182.17475    171.98614    158.53827    145.21758    130.94079    137.88054    143.77993    140.20342    134.88236    128.29537    129.17532    123.25585    120.57765    118.93148    119.21389    117.96628    116.74156    115.20672    118.35208    117.78405    114.68821    113.86814    112.97574    115.97034    114.47553    117.12823    115.07593     113.0206    108.51058    111.79567    115.76669    119.44192    127.14152    130.63213    133.30546    133.05787    126.76222    122.75845    120.81494    116.48804    114.94907    114.17925    116.96256    115.83077    128.69597    137.27989    149.24911    159.85217     161.4873    158.23021    147.50913    133.91919    127.42277    123.68971      122.749    126.31645    123.91594    123.87245    119.42853     117.2709    113.13887    113.57128    110.68372    107.16429    108.50814    108.18834    110.96404    109.45542    109.41873    107.29964     108.7415    110.38245    113.95564    112.22573    117.18695    111.97844    116.54887     112.9227    115.09413     112.2442    118.37295    118.62467    120.32529    122.00271    124.93874    128.77206    123.20375    123.20193    117.11606    113.85539    112.20597    115.01514    107.15853    105.39238    105.24904    106.98158    104.27166     99.74147     98.21336     99.57019          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       31.00    279.44475    261.39536    262.80308    246.92132    227.87795    229.65513    222.43275    226.85464    209.94077     211.2244    201.37279    200.91375    190.08182     194.2099    188.24807    185.62412    182.03542    180.96486    181.57727    171.35229    169.76708    173.27316    175.29555    173.47961    183.30634    207.78148     222.3805     230.8689    216.83151    189.92855    172.97758    165.26837    163.01594    158.96825    162.02674    153.87381    149.55391    155.95391    154.86256     150.1501    152.92929    151.60071    147.84957    146.91286     147.3237    145.10262    148.68116    153.70642    149.84673    143.06682    145.35396    140.25138     139.3632    139.68621    141.22482    146.09355    143.10771     152.6888    152.77888    150.25765    145.93699    142.25806    146.14059    139.33505    137.71438    143.07352    138.30288    142.59947    144.16656    159.77608    174.52516     184.2492    188.29504     173.6487    165.26924    155.61476      152.268    156.74484    153.56768    157.56716    157.46223    160.83949    160.15898    158.25011    146.78694    141.19341     135.3175    136.57956    143.25317    156.05598    171.69746    182.19165    184.64454    169.89525    157.10109    148.08014    137.89407    141.64025    140.02005    140.94899    133.12439    131.89085    126.48855    126.80241    122.89925    117.86825    119.45681    115.06751    117.12628    115.75454    118.42197     120.3623    115.77669     115.0139    112.63592    114.88421     113.5598    117.43433    113.91743    115.39068    110.12292    113.19142    113.81017    116.69544     122.8542    128.75609    134.83592     134.2417    128.42901    124.31772    118.79395    115.93807    113.41862    113.80883    115.95054    118.57042    127.09397    136.82154    150.03806    157.02748    165.46984    157.64438    146.12112    136.28038    128.83308    122.48718    125.82026    123.49816    124.30492    120.55202    121.29352    118.13437    112.81968    113.53793    109.33402    109.86941    105.87836    110.82011      107.421    107.43236    109.00058    107.39317    107.44722    105.82976     108.1985    112.31873    112.25028    115.02645    115.14789    113.38943    115.08221    115.82035    118.53902    115.87124    119.19686    123.99007    123.00983     124.9807     123.2631    122.61281    117.77233    114.84342    113.33507    107.59654    107.99496    103.80405     99.46883     94.96743          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       31.50    281.86936    261.59941    261.22272    247.38322    229.88712     229.5057    224.07351    223.89616    209.26493    209.29383    203.20637    204.72095    188.23893    191.60218    189.26707    180.42552    179.76272    182.61148    186.39267    167.41607    171.34904    173.17677    176.05621    173.93195    185.43961    205.05616    221.86114    229.16673    215.81864    193.02984    173.03582    160.91506    162.50217    158.41269    160.92345    159.96361    154.23792    155.73351     152.0008    149.16168    148.13646    153.03251    148.72732    150.42102    139.07369    150.14714    147.54349    146.83669    148.54914    145.24752    144.22447    140.99684    142.47736    141.94989    143.79804    144.90163    145.62202    147.33185    152.17151    149.89362    145.97076    145.06518    143.94416    137.49431    137.52138    136.96458    137.17096    142.17141     146.3848    155.99814    174.41398    184.07161    187.37191    176.49481    166.56054    153.21939    151.21811    154.55877    159.79823    152.24068    156.84329    154.67129    158.14988    153.54781    142.37218    142.84892    134.69457     135.6493    139.13558     153.3365     168.7143    184.64902    184.23315    169.44983    156.68277    142.24908    135.35841    137.06824    135.90369    133.40422    133.21272    132.52715    128.02411    122.50267    118.89777    117.65288    116.37495    117.47407    116.42123    116.16256    112.75673    114.54658    114.27409    111.21242    111.76842    113.57665    114.97815    114.47528    113.06372    114.23223    110.42497    115.81519    109.84912    121.34413    125.07799    127.68949    134.24082    135.47747    128.45136    120.68288    120.85127    114.11497    116.66381    112.33266    116.16764    119.87955    124.59662    136.37489    151.11439    162.73173    162.63882    160.55026    147.27728    136.54868    126.88078    123.92707    124.00851    126.69362    127.91011    129.78167    122.92347    123.85455    116.35605    111.16762    107.95264    109.60645    108.01966    106.83483    108.45789    109.45554    106.38846    108.81761     109.6823    106.99529     109.3086    114.55592    114.47037    115.16569    112.47501    114.53943    113.18462    111.47013    116.45418    114.97006    115.32655    116.72783    121.25444    120.25896    124.58865    121.29028    116.22417     114.2771    103.98374    100.86319          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       32.00    282.50648    263.89005    260.44418    249.03166    232.09274    225.87274    224.46266    217.18031    208.39935    207.54868    205.40845    207.28056    192.46035    190.56544    192.93176    185.54748    183.20272    185.02828    186.86373    177.17577    174.66246    173.74285    175.38924    178.63688    185.35089    206.40898    232.76857    225.43094    216.49927    195.84087    175.22837    158.93661    159.70737    161.91626    161.28229    158.95342    152.50369    153.20827    155.54622    153.07985    149.23797    155.84131    146.41206    148.62191    153.66413    145.84269    145.57707    149.43509    143.93678      144.725    140.72848    139.83756    140.46291    144.73311    142.69068    143.72873    150.62457    145.47228    151.35558    149.83821    144.56724     139.2821     143.0398    142.33363    142.71904    136.09778    139.08062    145.53468    147.60943     156.4514    169.74856    186.09378    187.44826    178.94937    166.17272    155.28497    155.69002      158.026    160.08945    155.51532    153.10528    155.00602    156.64326      155.314    149.35366    144.24045    135.82502    136.27517    144.40859    150.61066    170.08157    187.09775    187.78188    175.23139     155.4287    143.70889    142.54018    136.77645    131.39577    126.73647    123.33042      127.288    126.80415    123.95853    119.92922    121.59691    116.01703     117.7316    115.28619     115.8558    115.29105    118.85887    113.95235    113.86636    115.36295    114.01576      116.112    115.68774    113.36752    114.25096    116.83449    112.23798    111.58467    118.04271    124.94219     129.4625    132.51169    133.05741    131.26592    123.60256     115.9144    115.03564    115.19858    113.43283    114.16977    119.96324    126.53706    131.13615     150.0046    157.73337     160.6112    158.10584    150.55004    140.55239     129.0354    124.95681    123.99994     125.0768    129.50403    125.44724    128.21438    122.34167     119.3579    113.73169    108.75572    109.81769    110.20733    104.97376    107.95681    107.25211    106.00348    107.18274    108.38229    106.34478    105.94164    108.14584    113.49692    112.51677    113.93452    112.40893    111.78684    110.26084      112.569     112.5978    112.54995    117.43804    113.56993    120.46945    115.91224    114.82392    105.78767          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       32.50    282.28013    264.93685    261.35417    251.95537    234.77758     222.0145    221.86403    213.07083    209.30118    204.18056    210.27062    207.93677    193.14573    191.02778    190.47963    186.39676    182.97938    186.40499     183.1529    179.83207    174.85971    174.61617    175.56102    179.34798    187.09116    209.40959    232.77359    227.67625    217.49552    190.84124    174.05155    159.23913    161.16959    161.81093    162.21673     157.6018    152.25982    152.06914    155.47781    153.95534    153.59691    153.44991    151.51852     140.6106    152.61256    148.35735    150.59742    148.16372     144.9261    145.15163    138.62157    144.86197    142.35995    143.32271    141.84784    145.49088    150.55465    147.07263    151.13753    151.58711    142.25261    145.31144     135.0317    138.52802    139.08977     136.6168    139.25171    141.39498    147.18579    156.49544    170.32194    186.56997    182.14038    177.23759    166.53132    154.43221    160.30073    163.76835    166.40042      164.835    159.77978    156.43577    156.87962    154.28477    148.96938    143.40369    143.41812    133.10808    137.34322    154.19252    169.52655    185.14087    190.63977    176.71177    155.18948    144.31482    142.81719    140.00194    133.59591    125.99831     130.7228    125.38865    124.26482    122.03313    121.09242    117.82384    119.50834    117.26301    117.03346    119.33608    116.48173    115.85035    112.13307    112.12101    114.37371    111.77769    111.30742    114.02755    113.40503    114.83786    112.14084    112.95239    112.90207    114.93655    123.45793    127.87836    134.84918    133.43023    128.26661    125.73861    116.83688    115.66211    112.96034    116.99277    114.71765     119.3882    128.65549    134.89106    146.43034    160.98647    164.78341    161.32959    151.59891    139.26423     131.3987    126.87361    123.94583    122.60742    124.04282    124.09914     123.9049    121.51099    117.87264    116.20594    111.29773    103.97279    110.94265    107.74426    104.06454    104.16493    108.55968    109.54659    109.27775    105.56586    111.65349    108.41098    111.72582    112.84996     112.1411    110.81743    110.04065    108.86725    104.95323     110.6561    107.68802    106.24654    105.11501    100.27976          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       33.00    281.82052    264.41711    261.97457    252.02912    237.16249      220.969    218.69501    212.00583    207.67192    201.52031    214.86234    205.21522    191.66373    194.77162    187.00585    183.54286    180.82479     180.7293    185.66351    179.01606    174.17254    175.83974    177.88445    178.44958    187.59581    214.62243    227.25255    228.32342    214.52568     189.3302    173.68551    167.50459    163.47847     159.2964    161.05507    157.62498    158.15438     150.6598    156.34194    151.28924    155.31425    159.43989    152.08336    154.59468     143.5229    145.31387    151.07358    143.32493    146.63991     148.3231    143.89755    147.29293    144.45469    143.39435    143.45646    139.04958    150.54234    153.00575     147.5231     145.4762    143.64931    141.59265     140.9253    142.13281    139.67537    138.66266    139.98105    140.39073    146.05665    155.06863    170.21446    187.67444     188.0816    178.40132    165.47876    159.07636    163.47896    174.60096    177.68892    173.38334    161.95939    157.62921    162.93055    156.72387    155.24718    145.40401    139.72538    135.95573    141.65362     151.3591    171.28675    186.02765    181.35339    172.07383    156.71615    144.08554    138.35426    134.92405    130.37716    124.56923    129.38347    126.65929     128.0417    120.10458    119.74211     116.1414    116.35683    116.16979    118.77011    114.48841    115.10476    115.51826    113.51844    114.21532    112.33393    114.17468    110.92619    113.95737    118.38913     114.0002    114.26798    109.63897    114.63155    120.56974    120.20156    128.06627    129.57864    130.58712    129.95254    123.68728    119.42457    115.46481    111.02278    112.90079    118.21294    118.17064    127.92118    136.31014    144.50864    159.97319    166.53142    160.49734    154.40946    142.24893    134.18318    120.60493    123.31084    121.34052    120.02293    120.62409    118.93748    115.55192    116.26592    114.65977    110.57936    108.47064     107.7702    103.88702    105.45384     104.3259     105.6316    104.62968    105.09669      104.457    106.50918    108.39698    107.07768    111.35923    108.85032    109.56477    109.51121    103.97119    104.35301     99.73062          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       33.50    281.20409    264.90451    261.60075     250.3386    240.47104    223.38855    220.20124    212.80867    205.65885    204.50478    211.34278     198.8075    199.64018    197.32928    180.94102    187.22893     179.4344    176.74228    186.05081    181.31199    174.87939    175.51506    174.74543    183.24894    193.19481    207.86751    222.78654    227.75279    211.51511    192.64602    178.07095    166.04082    160.94238    158.28848    156.10354    152.74246    154.95527    153.95813    154.38564    153.62979    154.46572    151.93519    152.59206     153.8565    144.04414    149.90175    149.02359     142.8399    149.03243    146.10781    143.91729    143.41268    144.03224    140.70887    140.71319    148.34921    151.97914    157.56702    151.37528     149.2426    145.40739     142.1458    146.66492    139.22773    140.60837    139.00805    139.55758    139.57787    148.61473    150.11664    172.64665    187.54358     187.2113    180.38086    165.15741    159.84582    162.62484    174.41991     179.4061    173.89382    168.45209    164.26495    160.60473    157.04565    153.95934    148.67159    140.46096    136.54513    144.87981    153.61651    172.84119    181.99544    181.09222    171.06344    155.85151    142.21751    135.39215    130.90648    125.15874    124.56486    128.48399    122.03695    127.29293    122.28287      122.251    116.75976    120.62997     117.4583    117.38642    120.75287    117.69152    116.41012    112.23108    115.13978    112.55268    114.82901    115.37048    112.33972    113.83655    112.46516    109.93104    110.86528    110.16789    116.52236    121.43043    126.00929    132.41432     132.4388    127.97811    121.88054    119.54887    114.27868    110.82194    114.00435    114.08108    118.39573    125.73878    135.39835    146.62127    153.39238    164.55549    158.88067     151.1479    142.81446    131.83371     124.4966     120.5383     120.5761    120.38917    117.25825    118.92414    110.24298    108.13452     110.1278    109.09121    108.71098    105.66855    108.38679    104.51049    105.14381    103.56082    105.71829    106.32922    106.16673    104.87232    107.52323    108.53939     107.9741    108.31317    109.47387    102.13406          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       34.00    280.12771    266.64004    261.89956    247.69289    244.03043    227.91783    221.57495    214.13971    209.51245     207.1984    206.50765     195.4179    202.83862    197.01529    184.25318    188.01579    180.16311    182.53329    184.44194    181.99647    174.44776    172.04111    174.22846    179.27646    186.57738    205.61642    229.83283    229.21648    215.07725    188.61248    173.45671    161.07726     160.2222    160.47618    160.74604    154.79966    153.04034    152.54577    154.27829    155.05251    157.39154    152.84221     152.7083     149.5241    146.72932    150.54154    148.00691    149.05229    146.62861    147.19772    145.30627    140.38642    139.00312    142.58571    144.71815    144.21333    150.87886    150.19024    154.21131    150.94442    148.92643    140.37818    139.22216    136.67435    137.73145    141.10308    139.44163    144.01765    143.82271    157.42457    171.69694    186.12075    191.33342    179.75031    164.22155    160.59113     157.3422    169.59308    174.46714     168.5604    167.14251    165.56167    163.44397    162.11336    156.38562    150.99174    143.32895    133.48355    139.96465    152.66625    171.52944    183.99088    184.50813    172.33214    156.00542    144.44855    131.83017    124.88633    125.99589     122.1321    126.43613    126.05768    125.43214    127.11395    117.17402    119.21713    115.45208    117.61661    116.57584    116.77486    118.40559    115.30391    119.90488    111.21627    113.61023    115.84134    112.85835     114.3888    110.76868    114.24709    114.16914    109.20187    113.16198     116.6869    121.33837    126.51036     133.5332    130.71764    129.13714     122.3482    118.84485    114.60416    113.58369    113.44824    112.83885    118.83108    121.89562    132.80695    145.57603    154.10593    161.41039    158.55273    146.14618    136.00992    131.23464    123.52563    120.11247    115.24116    119.35146     117.2354    114.78015    111.48353    110.55854    109.76616    104.64967    104.87389    105.18605    104.37129    103.55426    106.58246    102.32482    100.55523    105.01583    103.86955     104.5255    105.10434     99.56855    101.85199          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       34.50    278.58607    268.28348    260.13289    246.50752    244.33923    230.88662    218.02376    213.67556    213.66295    207.09349    203.11386     200.2495    204.54657    199.33384    189.59542    185.69108    181.86358    180.57104    181.82201    185.12433    175.31663    182.53055    171.59366    171.78374    184.84905    206.77002     225.9878    231.01129    211.97826    191.75147    168.57858    163.47945     162.0801    161.99326    159.22503    157.51078    154.11975    154.42392    152.82706    154.99803    158.39568    152.53027    150.83077    152.06597    147.50881    147.76874    147.20011    148.28136    141.79007    144.73291    145.69888    141.87197    141.88013    142.44072    142.49434      147.838    148.59952    156.24216    151.55309    151.96489    146.32411    139.40115    141.77258     141.9686    136.74966    143.19561    139.41626    137.97939    146.17556      157.317     175.3837    184.12423    191.79755    184.32419    165.43871    156.56167    160.55357    167.13086    169.45944    167.81368    163.32548    170.82432    169.24397    166.96486    161.05752    148.36385    142.95047    137.91855    142.74537     154.1504    167.29063     182.4638    182.57281    173.36432    157.73026    142.13929    131.63566    125.93511    122.83626    125.02789    126.57534    126.53868    126.25339      125.236    119.96669    117.64373    116.47366    115.88456    119.00774    120.53448    118.55435    116.52938     113.2434    110.35136    111.06291    115.21582    115.89148    113.54966    114.29853    113.00628    114.15973    112.69397    112.98251    116.11964    121.99942    127.85419    127.57688    130.57689    128.00221    124.75457    121.77093    120.07504     112.8423    107.57572    113.41489     118.9029     120.1401    133.94077    144.21333    154.79716    157.52932    155.21219    145.33389    138.56467    126.50629    120.50578    117.76406    114.97581    114.45056    113.00148    114.97866    110.46802    107.11361    105.89435    104.60105    101.58223     101.9348    102.79443    103.94244    102.89539     96.99418    105.67094    102.61096    101.12957    102.32587     93.42238          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       35.00    281.24476    266.67881    258.85469    246.35849    244.39066    232.38722     214.4392    214.48518    216.58712    205.25955    204.01669    200.76599    206.46853      199.624    190.40143    191.87682    183.64043     179.6366    180.33769    182.30527    176.79745    178.60887    174.04587    172.05388    183.73758    204.31058    227.56341     230.6785    207.61025    192.61256    171.87932    163.22626      161.242    161.98424    161.04175    156.12387    153.20339    151.28347     149.3792    155.67911    157.53479     155.6966    152.92241    155.11886    146.61927    147.91684    140.32337    149.46546    147.33498     144.5941    148.13273    145.51375    143.84608    144.88091    141.80038    145.98892     148.7682    152.74824    157.44786    152.25204    144.50837    141.69054    137.87847    136.78251     144.8742    142.99458    141.81927    139.85575    142.12225    158.23981     171.3401    184.99182    187.69289     185.9318    169.78744    154.49401    156.42693     161.4138    159.67788    162.55527    160.06289     164.2767     171.5392    172.61344    169.94447    152.17892    142.85623     140.4881    144.26286    156.84763    167.08358    182.02783    182.79934    172.78181    155.96236    140.55457    132.15229    128.81209    125.56844    125.84898    132.76797    128.14355    124.64965    126.92592    120.58581    117.34399    120.07438    120.23188     117.0118      120.493     120.3572    119.57959    116.34677    113.60278    112.17381    113.05789    112.87114    114.34313     115.1995    111.14772    109.66758    113.83327    115.28628    117.26383    121.58195    127.54777    129.95982    128.06602    130.45825     123.4761    118.25588    117.82599    118.86195    113.65757    110.45472     115.0666    118.11171    133.58553    141.93288     152.7577    159.23222    153.42594    149.63825    134.74726    127.09316     121.1674     116.2694    116.56576    111.38457    115.17385     111.8389    107.33675    103.57592    103.08309    100.30604     100.6288    103.32078     99.46631    101.75506    100.62554     98.91233     96.25531     93.52479          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       35.50    285.37383    264.50659    260.03492    246.81635    243.38485    232.65895    216.35808    214.67668    216.53167    206.57306     207.2768    204.11287    204.19056    197.45004    187.61667    196.53423    183.41265      184.224    183.43415    180.33906    179.36302    177.44587    177.54439    173.44622    184.64474    207.00836    230.76033    236.02841    210.42961    191.52448    173.02118    162.39313    160.95419    164.21511    159.95109    159.83827    158.78361    154.65686    153.50335    156.22592    151.00311    155.13849    151.10434    152.16748    148.83584    147.22531    148.13502    143.19081    146.81571    145.23562    142.99448    148.83288    141.35458    146.70197    144.36322    141.53382    147.11182    146.58754    155.37332    155.81939    146.97893    142.03469    145.59728    139.45227    142.01163     135.9515    142.37322    142.53481    140.46112    156.11221    171.53055     185.7506    189.56299      185.883    168.32336    159.48593    154.36998    165.47809    168.57993    164.59477    159.40144    167.16713    175.45874    176.50743    169.27932    158.44825    147.29849    140.59322    141.76491    153.12422    173.04605    180.29446    182.52027    172.51664    154.79387    142.02463    131.85961    127.34072    126.26434    127.34456    128.08441    128.97451    127.32353    127.47882    123.78173    118.63581    119.94866    116.05831    117.11212    121.11842    119.72222    120.09837    118.42893    116.48786    110.79045    114.13732    111.26133    116.62533    112.05589    111.19792    110.71532    111.89518      112.718    114.40891    120.68361    128.00908    133.37947    131.68809    126.89389    120.70206    117.89659    117.04252    113.34415    111.47121    114.46237    112.07124    121.81852    131.80576    142.14139    153.25031    156.76313    153.90875    146.66611    138.57588    135.57528    122.40079    117.69738    116.76118    113.70711    108.45246    108.55221    109.16851    107.50812    102.66666    102.22344    101.34142      97.7063    102.71242     99.66963      97.2656          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       36.00    287.21255     261.7615     261.0101    248.76711    241.30903    231.91979    216.16493    214.64963    213.42364    213.00281    207.44719    208.36395    199.05703    197.36993    189.43216    193.19832    185.76799    186.65583    178.44935    181.80059    184.17902    176.40545    178.56591    174.18868    188.47437    207.61384    228.32931    229.67751    214.84453     193.0931    170.69049    163.10805    160.84121    160.95509    156.32161    162.51669    157.70443    156.33347    152.59835    153.73689    150.34854    155.28842    153.93101    147.08427    153.42115     150.5052     147.0093    146.10948    147.75307    147.32516    145.59355    147.63304    141.92896    140.85442    144.88696    144.89745    146.33878    151.49303    153.43186    155.15464    146.76207    145.91786    141.36092    140.52333    143.51314    140.39988    140.76298    141.22054    140.86844    154.44978    170.34626    187.70626    190.10333    185.18115    171.66055    163.44945    159.68472    164.43792    167.95122    166.92691    163.75854    173.27035     177.0716    181.23497     176.9301    169.57487    152.29112    142.05496    145.49592    154.66325    170.50227    183.13829    182.75348    173.59032    157.45927    144.06195    136.20738    126.12911    128.84681    130.83941    131.58603     132.0715    127.59767     124.4393    120.61404    120.11641    122.02781    115.49291    119.07365    121.48277    120.39317    117.36651    117.06277    114.54217    112.48386    113.39609     113.7252     113.4255    113.63582    112.37664    110.72372      109.614    114.12863    117.87231    116.26794    127.74801    130.45769    132.50687     123.1311     123.4497    120.96837    115.44797    110.81174    110.32344    112.48301     115.4263    118.58015    130.30218    140.15307    151.74394    158.72585    153.21914    149.04794     135.7749    127.69388    122.06108    116.54933    115.73734    113.09959    113.93518    107.71086    107.04397    103.84328    101.17639     100.5042     98.33559     95.71036     92.14685          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       36.50    286.60681    262.40116    259.49258    251.28923    239.69965    232.85183    217.49376    217.57626    209.09122    216.51788    208.86378    208.12021    200.46539    202.48395     196.3256    189.45542    187.78819    186.34364      183.078    188.32375    185.26617    174.12389    172.96657    172.73413    187.80549    209.11187    220.77199    226.20786    215.76581    191.05302    169.30093    164.33004    161.83998    161.43732    160.36772    157.05932    157.22098    152.72151    153.56455    152.94697    153.01637     148.9372    149.73797    147.93917    153.02862    144.80669    146.34475    145.64613    145.15769    144.62559    149.00088     145.7692    144.76592    140.86384    144.60887    142.85544    149.10069    152.08743    152.19359     149.3127    148.03834    143.51801    140.03694    138.80338    139.68124    140.33573    143.71319    145.10807    146.59145    155.93653    168.33739    185.44597    189.00619    182.29494    171.78257    157.61951    159.51812    164.17852    170.22691    167.32792    174.57782    180.87132    186.34938    187.59588    180.51256    171.25329    156.80609     148.5433    141.86959    154.68288    171.52177    183.65006     184.6284    175.57623    159.33302    143.31887    136.12846    127.29376    129.76848    127.78244     131.2397    131.88151    132.23802    125.70803    123.64693     121.4068    115.92656    115.81972    118.62306    120.15396    119.49174    120.16423    118.21849    115.84565    113.06869     112.8047    114.47569    112.32095     112.7656    113.40162    111.62023    110.81432    111.23238    114.55544    119.15852    122.47185     127.7561    135.35342    127.08195    118.01322    116.09393    113.77773    111.17595    111.37693    112.74102    113.88001    121.07309    127.09137    138.60284    147.40148    156.34546    157.11915    149.27147    138.27273    126.73764     119.1773    118.20344    113.29987    115.11337    110.53517    110.13748    105.79228    103.74485     96.92175     96.66872          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       37.00    285.77839    266.32117    257.85651    254.38729    239.42079    235.18576    222.27284    219.31143    204.73257    213.52345    210.45387    207.11688    201.59511    209.34749    195.88196    187.89135    187.94912    188.69867    185.08241    185.21858     184.3444     170.6346    173.45618    171.81358    189.33509    205.75476    217.37281      228.308    213.90397    189.84043     168.1532    162.86947    155.62982    164.99554    163.71682    154.50717    161.59175    156.50279    156.17281    152.15659    152.33008    159.73384    150.51241    152.28323    151.42607    148.96711    148.81777    149.30175    144.64972    147.68804    146.95887    145.85791    147.15591    144.28112    144.40119    146.04329    148.30071    153.97127    147.77921    146.55613    148.87499    141.90308    139.04839    137.90823    141.79351    141.33745     136.9637    143.02592    148.08118    157.44109     171.4169    184.56336    184.71128      181.516    172.74033    162.18077    163.11364    167.64189     171.2154     176.6345    182.04525    185.21127    193.20835     188.1871    177.84718    161.61689    153.68247    141.70024    148.65862      155.541    171.85176    190.00985    185.90582    176.81711    160.77984    151.15863    137.83197    131.53502    131.89516    132.58469      126.484    127.02968      127.239     123.0706    118.89439    121.69042    118.20228     118.7118    119.16268    117.32758    120.38318    118.82641    119.93986    119.31283    117.34154    111.79848    109.52103    112.40117    113.85271    110.45685    112.38751    112.40605     109.0976    111.29529     119.1978    127.39042    125.67186    129.77536    125.19496    118.09736    117.33012    113.39497    110.70924    107.91659    109.56853    113.08704    115.22206     123.7937    136.39813    146.49262    155.09991    161.24032    152.62323    140.14483     131.6096    124.44251    120.27521    116.28946    116.15552    111.77693    106.49042    104.19596     93.58563          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       37.50    286.31658    269.86715    255.40764    252.63959    238.73009    237.60407    225.04125    216.91736    206.93422    212.85103    206.29866    205.47484    201.59369    204.46935    195.16343    188.94817    188.91833    188.20659    187.49816    180.26555    185.02212    175.32965    173.96923    174.12502    190.32297    204.25797    223.46839    231.87109     212.7488    188.60839    169.95697    163.52237    162.64276    160.45482    158.32004    149.13029    163.28765    159.48486     157.0854    156.42917    152.01392    154.60746    154.07611    151.78184    152.76517    149.91583    143.84376    148.96877    149.62412    147.54496    141.06544    145.64564    151.45695    143.89722    148.25009    147.09154    147.79782    146.47918    152.92497    154.07545    148.41815    152.55932    142.94693     139.7914    141.74595    139.42289     139.1048    142.30999    143.20662    158.20931     171.8131    185.85131    183.02386     182.7125    172.56448    169.89786    168.41669    173.18486    175.07672    185.75716    189.00899    187.25629     198.2171    193.55581    175.43725    160.15745    148.97002    144.51356    143.30362    153.52251    169.92232    187.71293     191.6166    184.18469    169.32132      154.738    151.28804     145.3221      136.905    131.61397    127.72807    125.39329     123.9989    126.47682    121.77819    120.09607    121.02978    118.39007    118.53829    118.13024    123.17251    118.30711    118.15622    116.91632    115.50362    114.19389    112.82426    114.83748    112.35872    110.14155    109.30073    112.09821    113.47586    109.82786    119.52655    121.43244    130.12254    129.91502    126.07621    121.46896    116.63617    113.69983    108.79586    110.12087    108.79861    117.00864    118.56875    126.10292    134.72908    150.17318    157.51871    159.67352    154.08096     145.9894    132.63083    123.70588     117.7155    116.28075    113.67529    109.37773     96.99625          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       38.00    290.06376    271.10699    251.47111    250.50739    236.10397    240.91424    226.37259    215.08224    209.62437     214.6329    206.57306    205.67026    203.62921    203.26418    194.95332    185.64305     188.8441     187.3277    188.51645    181.19339    183.82167    179.87094     174.0218    180.47247    186.07152     203.8202    223.11503     225.7577    215.88671    189.81239    177.08192    167.56362    162.37017    160.66472    160.74315    157.20647    157.90137    157.39547    157.33512    154.37029    154.09546    149.36061    152.69031    150.63225    155.14725    152.80282    141.68529    148.74335    144.43619    146.17447    145.40319     143.9445    147.64319     144.3869    143.86517    143.67915    146.49796    149.21293    150.35827      150.971    147.33078    145.31222    141.45974    139.09135    138.99506    139.08256    140.20276    145.39313     149.8639    161.76353    168.26411     182.4189    186.82608     181.5654    170.81905    168.24599    168.25326    181.53201    186.25428    193.74011    200.43393    195.46254    196.42042    195.57664    183.66606    162.06172    144.73141    138.02052    140.69218    154.12269    166.41514    185.84096    190.28346     186.5957    169.51547    163.47815    157.70702    159.81193    151.85801    139.60392    134.20204    126.57779    126.40477    127.17591    124.29881    120.69648    118.77003    116.99969    115.74349    119.17771    118.93563    117.39155    118.69843    118.02254    116.53561    113.40818    115.18343    111.06499    113.56653    107.78642    112.31208    108.45746    111.97042    111.52502    117.17154    125.61375    125.94484    128.41956    125.45151    120.55095    114.73808    111.39298    110.65876    108.57503    110.17752    111.03838    115.08328     126.8329    140.19031    145.41554    158.53644    162.43643    154.67634    143.89037    132.41079    122.05334    115.04026    107.13399    101.61094          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       38.50    290.13738    271.49117    251.62276     248.7493    237.25651    242.49444    228.13335    216.76286    212.24932    211.55767    205.64413    205.14423    203.53488    200.80807    195.23692    182.92988    185.25688    187.92178    186.10008    184.70913    183.76435    180.38604    173.86558    180.20924    184.10483    206.38976    218.84788     228.0226    220.37432    196.78164    175.42317    164.49301    159.47314    159.91657    160.94177    159.34456    155.34685    156.37643    159.06494    160.80969    153.85493    148.52157    150.81379    152.52894    149.61493    151.31899      152.055    151.23553      143.516    142.25514    146.67825    146.27207    145.25388    145.68127    145.14725    146.29071    148.72487    146.75113    151.70087     155.5762    149.39877    146.64839    145.24681     137.3276    140.13687    140.34489    143.24435    138.59562    147.59227    158.14475    172.19892    181.19665    187.21119     177.4105    171.19435    161.67989     169.1703    177.23039    194.60291    205.38706    212.72639     201.9068    199.81958    192.81898    179.83882    161.60587    150.32759    142.65245    145.39979    156.37623     166.4523    187.62965    190.35891    186.37949     172.4046    167.20734    168.24252    160.50714    155.91616    138.56784    131.84739    128.99962    127.59438    124.58142     121.1933    120.62427    122.12728    120.23778    119.79425    117.80576    117.77715    120.88697    119.99472    118.63599    115.61364     111.8976    114.02863    110.74644    112.67873    112.90885    108.07723    106.93757    108.51309    113.88538    114.62205    121.96387    125.39579    124.62999     127.0401    120.20474    115.82959    114.68649    108.84773    108.47196    109.93388    111.72034    115.08798    125.26003    134.14025     149.3126    153.71196    158.27408    148.42717    136.13575     126.0409    112.66724    102.39679          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       39.00    287.72543    270.95047    255.33749    250.17394     240.7415    241.50097    227.37987    217.59905    213.29503    211.06046    203.80263    206.35917    206.32709    199.47344    198.51646    187.55114     185.2742    184.04381    185.05129    185.38718    182.29994    182.00036    175.15334    177.69542    186.34531    204.21823    218.17738    224.58291    214.81874    194.84038    175.52598    166.84673     157.9487    159.69819     161.6627    160.85395    156.67514    160.48417    159.27423    157.19872    156.34579     150.4931    153.12761    153.14486    151.60453    149.59957     154.2561    142.33216    146.71701    145.89688    145.16761    147.14658    145.96702    144.95513    144.51951    145.18383    147.64275    147.97601    153.23437    148.80852    150.91463    144.24571    140.34762     137.8959    137.50965    136.06133     141.0243    138.60932    146.51583    157.83378    172.09726    187.85056    184.59406    182.46244    168.00052    161.94203    161.89674    177.30001    197.98542    214.09028     212.4496    204.06056     196.0397    184.28431    180.82185      165.693    146.91348    140.87506    138.06742    154.41953    170.38438    185.51927    192.24721    188.74331    187.35066    168.42553    161.69881    155.63756    148.57704    135.15963    129.57441    129.67686     124.9682    125.39041    120.79562     118.6642    116.90068     119.5388    115.96711     119.8472    120.78868    121.06709    115.48073    118.43443    111.68224    112.74882    115.58902    114.00024    112.71809    112.80031    108.63914    107.93016    110.18913     110.7773    119.21155    122.70104    122.92407    127.93327    124.63499    119.90964    114.13872    113.08643    106.71887    109.67386    107.10053    109.06326    111.06683    125.19961    133.32918    142.23835    148.17892    145.53228    140.36909    123.86418    109.32016          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       39.50    284.38031    267.11881    257.53623    253.91851    241.91897    239.15104    225.03359    213.65163    212.71254    213.00534    206.12903     208.5629    206.65761    200.83538    197.91691    188.98248    189.98886    181.91984    182.17122     182.3943    183.24984    182.71491    180.02293    175.90477    192.52108    202.63825    218.28035    224.70951    214.70928    196.56011    180.07964    165.93151    157.74851    159.28333    162.43085    160.23689     158.0798    158.36821    159.22196    154.51098     154.6489    151.23462    152.78199    149.11928    150.03922    150.83313    152.83821    144.44228     144.7705    147.65299    143.71887    149.95596    146.40273    147.64663    145.25134    151.76475    146.38723    151.57683    151.42885    150.05109    148.06955    143.82761    144.26456    140.22351    136.45274    145.81193    145.42176    141.70956     151.1315    154.16576    171.17162    184.53287    185.86298    181.40848    171.41653    161.60028    160.85869    164.60795    187.87335    207.86175    205.31705    199.51579     190.6128      179.628    175.57823    168.49805    146.63829    142.45795    141.53138    152.11387    162.23021    186.48109    204.06282    204.99228    200.56872    181.65792    162.31419    149.26051    138.42015    131.52534    126.13139    126.67043    127.13084    123.55243    122.44406    119.43511    119.59917    116.64687     119.7373    118.75767    120.91142    117.88597    119.28372    117.68803    112.62325    113.14708     111.2065    113.46455    113.47864    109.60095    109.12454    110.18982    107.41239    113.04837    116.95572    122.59968    124.43445    124.81362    124.34517    122.31675    116.89229    112.70724    109.35168    107.07181    103.72837    109.09216    110.26942    120.56035     129.0471    138.72916    139.83621    130.75903    110.75964          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       40.00    281.51531    266.13197    255.31373    254.97731    240.32396    239.76891    229.06296    213.81136     214.3973    211.65534    208.08859    206.98265    204.04807    198.92546    196.45207    187.85148    190.78208    180.07532    181.64096    180.71528    184.64734    184.47544    178.13508    173.63576    196.31628    206.32765    219.59285    223.16857    218.43194     194.7952    171.65792    161.18575    164.80411    162.40341    164.53761     154.5832    158.37374    160.97752    155.42415    160.85349    155.83006    150.21186    155.97674    151.34956    150.64212    157.58323    147.03025    150.79335     148.8123    148.57498     144.1685    147.23569    146.59113     148.8987    144.74777    148.90287    147.08156     153.0416    150.70388    149.09949    147.85828    148.30187    144.91375    141.90946    144.88156    144.28174    140.75717    144.55508    147.75348    156.26986    169.10801    182.85593    180.73706    179.61646    166.35693    156.75508    161.67194    164.90073    178.43372    193.63296    196.09584    189.12729    184.73597    182.23782    176.43373    161.57565    148.35612    142.52406    142.12887    151.36609     167.2838    186.49058     201.3207    216.28267     213.3947     195.9427    170.56169     146.6278    131.94867    130.85372    124.35234    130.48417    126.55152    125.28394    117.31864    121.38795    118.64441    118.51162    117.82509    120.26654     120.1058    118.50658    115.84912    115.02492    115.10384    114.57506    116.37849    111.68444     115.4158    112.00536    111.17261    109.71778    109.04405    108.43823    115.94727    120.85431    124.88452    125.73487    125.30858    121.23014    118.78286    109.97706    106.84839    106.14233    106.70968    103.74788    112.13369    120.04592    121.17401    123.77184    117.22622          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       40.50    286.75015     268.0617    251.91924    251.96678    239.96203    243.04056    232.70779    214.54309     216.5674    209.46769    211.71924    202.76835    207.37989    196.64717    201.69326    190.53783    190.80154    184.61106    180.41401    176.96464    182.38444    179.13363    177.79649     171.8849    189.01845    203.47797    218.03051    222.64326    215.32581    183.34136    171.73304    174.24515    160.77171    157.32025    162.92608    158.33546    156.74614    156.45248    160.33793    160.90633    156.19838    148.76474    147.75503    150.49763    150.31554    151.96429     155.5093     150.3175    151.86602    143.67367    150.48209    150.20881     147.2047    149.14586    145.99947     148.6261    148.85708    149.99949    151.97786    152.72429    148.72571    144.10841    143.19907    139.99808      140.102    136.90569    138.03982    137.81371    148.11879    159.64577    174.51676    185.02326    188.92784    178.22392    167.84466    155.56287    161.04097    169.27056    176.66605    188.15073     186.4728    184.38027     181.3186    183.39444     182.0673    169.95862    145.70913    141.68798    141.51392    149.59569    166.65646    183.01214    197.48889    214.19874    207.18733    194.79074    169.76022    149.10154    134.04773    131.96688    126.45702    126.39193    127.71716    124.87471    127.92134    118.62363    117.38162    120.02643    116.18756    119.44121    120.04069    121.25247    122.48274    114.62425    114.74019    113.49448    115.82951    115.76669    112.16187    115.30835    113.49656    109.66268    108.09425    110.47412    115.26361    122.88072    122.85466    124.96976    124.10452    120.02323    115.59971     109.9389    108.63287    103.78454    108.84882    105.78421    106.68795    111.10431    107.06517          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       41.00    289.22033    272.90133    254.54962    248.85894    239.50923    246.52073    232.14662    213.92493    218.44533    208.63963    211.33382    200.45015    206.06721     198.0486    204.66731    190.98614    186.86023    182.53509     182.2138    177.66149    179.35426    177.17251    179.89432    179.68161    185.47242    202.37715    211.00583    214.81504    216.31288    191.93826    171.90625    173.33358    160.12654    161.35543    161.03716    162.13242      156.934    155.61767    163.36773    159.14357    162.17592    151.90031    151.14125    149.16253    150.61262    149.80889    151.60905    150.01461    151.44838     147.4229    143.94334    148.10302    151.42494    144.60613    148.14564     148.8549    151.58237     148.2545    148.03931    155.86906    144.57728    150.38795    144.40313    144.07157    138.96074    144.76585    137.41005    142.13867    142.79524    157.76655    174.68817    183.34173    189.64249    187.95258    167.83961    162.90688    162.35976    168.41309    177.05616    183.01363    185.78192    185.58483    184.82723    186.71188    190.90131    170.89848    153.02934    145.03348     141.3821    155.33964    166.86998    177.49346    190.47939    191.71735    193.34036    180.55663    165.87269    154.10918    138.30887    128.39448    125.02066    125.44467    124.45342    125.98369    122.40611    123.76467    115.65623    116.64276     117.6707     118.4393    120.83917    119.34906    120.70683    118.98107    112.13059    113.52246    113.87787    111.64376    113.98909    111.03745    113.26096    109.69626    106.61753    109.57385    112.03516    117.11302    124.61624    125.96268    122.58432    117.62619    112.42524    108.56213    104.88732    101.80921    103.26088    100.71112     99.38291          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       41.50    288.91445    275.07965    259.00378    250.64897    238.98254    241.69173    235.85455    214.27483    216.44627    208.77396    209.30201    198.69054    208.99383    200.77042    202.79256    195.69755    186.79092    183.85721    192.07751    179.56124    180.28541    180.55722    179.47355    182.57008     184.5744    201.03536    216.30188    214.30548    207.54931    193.86299    174.84767    166.11162    163.31748    165.52775    162.28841    161.47369    156.96755    161.05646    162.67342    161.19395    153.17903     149.3874    156.32195    150.55012    153.06442    152.43438    149.19268    152.54461    149.75184    147.63827    151.20323     145.3221     143.7247    140.24924    147.01475    147.68061    147.60082    155.04114    152.25028    151.23691    147.95363    145.13427    146.35095    144.07388    142.52004    141.56133    143.35828    142.37027    145.08593    157.00702    173.34628    180.49671     188.3535    180.87104    169.20304    161.99078    163.84182    167.04812    178.45097    186.68778    186.81905    184.94639    183.88756    186.38502     182.1906    167.86998    155.80579    141.92575    144.00629    152.37118     166.7973    175.09882     181.0434     183.6895    178.09728    168.52367    163.04546    153.88193    141.77964    127.89709     126.8394    125.15666    129.55086    125.05137     123.6507    124.31804    120.93084    115.98321    119.45264     118.4603    121.48964    118.23659    118.25164    118.56422    116.97751     115.3917    111.96428    114.63012    112.87214    111.30312     112.1153    111.42511    110.78472    109.61575    110.52502    118.82621    125.24831    124.05359     122.2167    120.58161    106.95249    110.86184    104.74371     101.7327     97.32291          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       42.00    286.74839    275.16329     262.9588    253.33121    240.17542    237.84023    239.27336    216.98948    213.41437     209.3803    205.21559    197.73262    210.52494    200.48503    204.53984    198.39956    190.67616    188.76428    196.80056     181.5864    179.21342    182.27444     174.2748    180.24111    185.27018    205.51038    214.37177    217.09741    213.49234    190.02389    173.02878    165.93839    170.10356    161.60812    163.12035    162.40223    155.33217    161.80011    157.73064    159.81286    157.74875    155.07109    158.37527    152.34415    153.86541     150.4762    147.07606    153.32116    152.94548    150.24034    146.51494    143.55722    145.40149     148.7469    147.81538    145.42896    148.01356    150.31415    152.82572    150.65866    146.65919    139.69074    142.06981    142.09918    142.94259    143.62652    142.70127     143.4191    143.63385    157.90063     170.9345    181.96538     186.2661     181.5274    172.92498    164.66958    159.78312    172.11199    177.17824    181.50721      181.123    186.77825    182.39696    182.33908    170.75985    159.31631    151.29188    140.39249    144.95457    158.51349    167.10809    176.34916    181.13643    181.33903    171.16927    163.64795     149.9832     142.9456    132.04966    130.99177    125.00507    128.43049    125.64292    129.49012    123.40281    121.98818    123.97826    117.34999     117.9866    116.42847    119.91877    115.55629    118.40498    116.59836    112.49895    113.66013    109.77418    115.46564    111.82134    114.39482    110.41248    111.61394    111.76838    110.23741    111.38275    116.73615    123.66803    122.26785    122.95658    117.21278    111.31964    106.44182       98.435     94.20531          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       42.50    288.15688    275.14147    261.04966    255.86759    243.37597    234.85671    232.94109    217.86599    215.35757    207.58949    203.32125    199.68152    206.00857    198.40897    197.57309    195.83228    194.16913    188.63404    192.78775     184.0652    177.18231    181.41966    177.97888    181.02192    190.59797     202.3196    216.82857    222.11106    211.42972    193.89793    176.72898    164.40151    167.89842    167.86368    160.95383    157.92745    162.47547    161.93206    158.11002    161.98997    156.68488      154.137    153.85191     151.1114     152.5824    146.41276    147.73336    159.92075    149.64591    152.73697    147.02025     141.5313    145.61057    149.55057    147.19202     148.4674    152.58664    155.21385    155.87158    153.35245    145.52784    145.23392    142.55833    140.02848    144.76969    145.72841    139.35259     143.6605    147.21277    153.23203    170.60039    183.40254    188.87811    184.86609    172.58593    165.39903    157.56851    163.71425    171.32386    176.36899    177.60214    180.51574    183.15937    172.06767    163.28232    151.26202    143.74723    140.61847    143.30715     153.1804    168.03394    179.07647    186.47145    178.47239    175.14118     160.4065    148.67185    139.09517    136.07658    127.57571    128.13542    125.23345    123.67606    127.11355    120.08938    121.21702    116.96738    120.63457    120.07317    116.27026    120.82231    118.57803    118.02019    116.55193    114.43219     110.8941    114.52917    113.14122    106.92684    110.97078    110.61487    110.96892    109.64625    111.96706    113.86144    116.22691    117.99271    124.17571    118.75698    117.89751    108.09937     97.09608          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       43.00    290.25948    275.01389    257.74284    257.99119    247.59023    231.67274    227.23506    220.94896    216.86041    207.38423    205.26417     203.6604    205.20381    198.31773    194.62601    197.85164    193.70878    187.33761    192.91642    182.84269     172.9363    175.46635    183.61577    184.87029    192.39557    199.53726    217.04709    218.65727    208.30039    191.34785    178.15542    162.70626    168.61459    167.06177    156.33284    158.02276     161.5071    160.80377    160.16947    161.56777     157.0639     152.1661     153.6456    149.03548    153.42575    154.01062    149.22627    152.09033    150.59341     151.2566    148.04118    146.98729    144.88896    142.00829    148.73608    148.13161    148.70165      154.816     153.9589    151.77231     145.9603    145.74558      144.998     139.0129    139.81143    143.26782    142.90176    145.93174    141.99784    154.78112    169.94857    185.66172    186.76567     186.1201    173.08449    163.29523    159.70338    155.97254    163.64212    167.41368    169.15492     170.4411    168.97286    173.56786    157.84847    148.25413    142.73373    142.12272    143.16075    154.06858    169.74023    181.08172    190.63593    181.27435    170.22288    161.14992    146.85343    136.15696    130.94569    128.71221    126.95356    123.36638    126.93013    124.02513    124.42144    118.85602    116.91071    119.84261    118.80524    116.42223     115.9203    118.44347    117.49417    118.25362    115.69186    114.82254    115.40895    108.46823    111.39612      110.508    110.11998    108.28301    107.23045    111.48448    112.73086    113.85954    116.59336    120.61499     116.7434    105.23916    100.39165          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       43.50    292.87852    274.62262    257.64006    255.69721     247.5195    232.11653    225.52746    225.57655    217.49078     211.9329    207.28825      203.554    202.91667    201.04219    196.34234    198.92117     190.6084    189.95431    188.54271    183.10924    182.86766    176.63444    183.65311    185.80741    191.83696    202.55356    210.95802    217.58669    208.57031    189.92693    178.49881    167.14018    168.95305    168.51835    160.72442    160.26196    162.04958    158.71489    155.79393    158.28647    161.41652    157.74035    156.83722     155.9473    151.39798    147.87623    146.91251     149.4567    149.47899    150.15579    153.74855    148.43791    146.40492    144.51598    146.89203    148.99635    145.83155    153.75374    154.03094    151.51046    143.29337    146.75044     144.7728    141.97442    143.50287    145.27725    142.42377    147.26777    145.03612    153.90852     169.6464    176.06888    184.52413    181.26321    173.91433    162.33667    150.73018    155.38943    156.78143    158.45241    163.73909    167.43817    162.84985    161.67455    158.16827     143.8381    143.05494    138.29003    145.42385    154.92093     164.8079      182.469    187.25715    184.72956    167.49107    154.86985    142.03508    138.31522    132.37123    129.61249    125.51319    127.39111    125.66176    122.22705     121.3638    119.26669    118.96326    117.74133    118.37204    118.54981     116.5286    115.68889    115.49405    114.88856    114.17944     116.7609    112.77173    113.02158    114.08084    112.09445    112.46725    107.61835    108.67938     106.6349    113.81162    113.73171    114.28479    111.23374    106.73745          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       44.00    294.61892    273.31557    260.85314    251.44089    244.78184    234.45578    231.33494    228.50654    221.80526    211.53343    206.26954    204.42878    199.38998     201.0543    198.76775     195.3779    188.30158    192.31996    188.53339    179.92207    197.80698    180.42514    186.20883    181.69472    189.89566    208.26406    208.59031    220.24908    205.46695    189.32573    179.73325    169.38624    166.65305    160.64939     161.3941    161.43258    160.37939    162.18088    161.14204    157.22683    159.38002    157.37488    157.91336    158.28859    149.33223    151.35328    147.87329    150.84224    149.73528    149.83535    146.09298    147.39278     145.5395    145.82295    145.17385    144.21884    148.16341    152.65092    151.34071    151.39066    147.03541    144.30611    138.98956    143.92408    141.86981    142.04371    144.24703    146.08574    147.24484    154.91706    167.18167    182.17713    183.96075     184.0009    170.03879    165.19893    158.92701    156.20305    149.73871    154.18925    158.09311    163.75206    167.22789    161.10882    152.45563     145.2361    139.43127    138.78823    143.83317    151.39003     160.7007    179.17111    182.70649    184.45506    169.93497    150.84798    140.03574    134.48428    130.29958    125.99964    126.31563    126.08895    125.03832    128.39949    120.84648    120.21274    118.17383    114.78703    118.22344    116.98177    117.78342    117.98837    113.40124    115.39536    118.82579     113.6007    117.73841    112.94052    113.66001    108.18814    109.06642    108.30767    108.70771    109.06525    106.03272    111.21417    109.27306      98.4887          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       44.50    296.00066     271.0247    263.91401     248.2393    243.41438     235.7334     235.7994    227.25552    224.00482    211.54996    203.24832    208.05521    199.58423    198.14237    200.05136    193.24813    187.05437    190.23051    184.60316     176.7859    185.45223    177.66707    185.08471     179.2846    187.03354    203.48099    213.74059    219.03024    205.47939    187.82996    179.79218     176.8247    168.27546    166.10855    162.66629      160.864    153.92636    163.11359      159.573    162.83916     158.8544    157.35594    156.32627    154.59636    152.99621    152.66064    154.07648    146.95211    147.35094    153.10953     147.2402    151.53033     147.8655     144.9783    144.15052    146.82704    149.14746    149.18679    155.27736    153.77074    147.10321    145.20756    140.34746    141.61333    140.56307      143.286    143.73015    143.04541    147.36787    161.44376    169.69169    180.89174    185.00504     182.7537    176.10768    165.99893    155.53445    153.39828    154.07442    155.51414    162.96127    164.40197    168.22434    158.85716    151.77633    149.73715    142.00173    137.67592    138.34998    149.46229    162.89751    176.85851    181.32054    180.17997    173.34614    159.04549    143.79628     133.7642    129.18789    130.44114     127.8342    128.56194    130.37164    125.77275    122.76928    122.76597    118.85701    118.21332    116.80033    116.90749    120.41226    118.45759    114.93904    118.36639    116.22588    116.35325    113.27423    111.59338    110.26454    113.83196    109.54852    106.04588    105.95703    106.12071    105.94784    106.68655          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       45.00    297.59598    268.15989     265.0991    247.13109    244.20536    232.64926    236.17589     227.0047    223.62497    214.46915    202.33031    208.56935     203.6492    197.95726    196.77723    193.08311    191.99376    189.66891    186.56258    183.04166    177.75378     179.0207    180.28305    178.58533    191.61081    205.15243    216.63432    211.03473    205.98839    189.88075    174.15036    175.07109    174.60394    166.67195    165.01429    162.83151    154.27159    160.15907    160.20524    161.04131    155.07904    158.15816    157.26308    154.33553    151.24428    151.88953    150.57169    149.22852    150.67532    154.03146    148.03451      156.627    142.46114    148.19216    153.54031    146.81634    150.41789    153.80448    147.51161    150.65739    146.36225    145.29968    139.45265    141.48785    142.00406    144.39171    146.56013    145.75417    146.85252     160.2509    170.62632    174.77479    180.91514    181.18383    173.47382    164.52559    156.07911    157.00687    160.99879    158.74256    164.81103    166.72281    170.23426    163.90298    155.84679     149.6637    139.83535    142.09969    141.26676    151.60692    163.38182    178.19713    181.22831    176.93579     173.0526    161.14783    150.50321    141.42217    137.83926    128.26547    126.73037    130.29542    127.85016    125.38877    126.00818    121.49611    118.52917    114.38236    117.80218     120.5491    118.14907    119.56921    114.24995    119.27817    114.10569     112.4949    113.43407     113.5435    111.95078     110.0367    108.47253    108.68655    105.05396    101.06563     96.06534          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       45.50     298.7036    272.53843    265.23355    246.94276    243.91918    229.02566    233.38203    231.21736    225.31585    215.57301     205.8941    206.22063    207.23777    198.44059     194.1929    194.31484    198.93703    190.75785    187.18619    187.76188    180.29991    181.57326    175.14641    178.16037    194.78701    204.16766    221.27795    210.52377    201.31868    193.44404    179.65945    170.61615    168.93315    161.87012    168.22192    169.22079    158.73901    164.00987    156.30206    159.72654    164.08414    160.73246    154.26334    153.90178    154.01726     152.7973    148.14207    156.82847    154.04992    150.45684    151.19863    151.25616    150.47156     150.2991    150.27786    145.97139    149.86524    152.69952    148.68566    154.58989    147.39165    142.62084    142.69328    142.66743     141.2402     139.1791    140.22548    140.97637    147.71684    159.79461    169.73236    184.30131    183.64628    177.22352    168.69303    157.48349    154.76301    160.02391    158.34589    163.26971    164.81489    172.86983    178.15225    169.31559    153.24052     141.6293    143.71787    138.37886    140.10539    150.42642    161.91434    170.63542    182.33223    182.14733    175.61522    167.79027    158.55348    147.17875    136.88037    133.13474    127.97129    127.81059    131.48079    125.08435    126.02478    120.21828    119.69369    117.25865     116.0112    118.03719    113.03565    115.40508    116.92522    118.07958    114.81262    112.88801    115.61178    115.35683    111.91627    107.20865    108.33847    106.43988    101.91361          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       46.00    300.52495    274.62734    263.40785    246.52263    242.48304    228.59561    228.57911    230.74365    223.14962    217.78616    211.53917    205.44471    206.71951    199.10363    191.71435    192.45842    193.15682    190.00604    187.28786    184.36095    178.67287    181.10582     174.1935    177.05279    189.70307    200.93237    218.58635    212.56211    205.29522    191.37103    175.35092    170.77127    166.86078    166.39366     168.0897    165.76292     162.3746    158.62703     157.5476    156.57672    158.80176    161.39893     157.3758    158.31719    154.08139    150.28027    152.96527    143.53859    152.71514    150.77064    152.49461    148.32355    149.89795    145.90086    153.13521    146.23498    151.20581    149.12155    152.79393     152.2494    148.52049     147.5545    143.06053    142.86656    142.47356    140.03721    138.02057    139.80854    148.88271    161.04233    171.33498    182.66989    188.90996    185.30656    166.77896    159.88054    157.98947    158.79341    161.65445    162.92248    169.00575     174.1231     184.0957    180.99979    159.68611    149.23346    141.09015    139.93592    144.37167    151.74852    159.64323    168.31059    176.73231    181.48269    180.17974    168.18072    157.96471    148.64094     142.4506    134.42286    129.83425    129.51611    124.65793    125.41249    120.85568    124.05183    121.02212    117.69249    118.29642     117.8916    117.12341    117.43419    116.16967    119.59125    118.27764    113.93731    116.83029     111.2777    113.84224     108.3946      106.366     96.79719          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       46.50    303.98897    274.42782    262.60532     245.9174    241.25552    232.42628    223.96859    226.61417    219.74852    222.70078    213.33968    210.53428    204.67546    199.07424    193.92892    196.85811    190.24369    185.44351    189.53644    180.86436    175.57333    175.67988    182.52926    180.88146    189.47614    202.61968    214.40642    220.46732    209.45038    189.20204    174.74947    170.56555    168.16922    169.53068    170.98363    164.12764    163.92927    159.48173    155.54377    158.28675    157.06366    157.09837    158.05017    155.86209     159.4977    152.67387    153.59853    148.10369    151.66656    151.63483    150.74626    150.53115    149.92592    152.00359    149.79091    150.48987    149.89236    152.33773      159.181     151.5359    152.20181    149.71414    146.34676    146.50245    143.87714    143.25577     142.5173    144.84822    145.54865    155.43069    169.35808    177.73502    183.70781     178.5622    167.54961    161.49677    151.49358    157.67462    161.36786    167.20449    170.30285     176.7438    181.08338     185.3921    171.88003    152.21872    148.15436    143.95527    144.24026    147.35145    162.81035    170.68833    178.51575    181.57065    176.08164    175.21066    156.88806    148.00735     140.2431    132.35401     128.8871    127.65556    124.71067    126.50894    124.45635    122.39583     119.5054    119.73171    117.91149     115.7053    114.90093    117.86891    115.05385    115.66266    113.40417    114.36806    112.96807      110.493    109.23473    106.66934     98.69561          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       47.00    307.46613    273.59453    263.05707    248.69192    239.01064    238.14469    227.98665    224.52176    221.26843    227.43406    211.29015    208.91098    202.74866    200.71584    196.61119    197.59426    195.16167    183.53013    188.51069    186.25725    168.56849    179.39228    181.66735    179.76237    194.70926    197.63119    210.31601    220.04971    210.00907    188.15241    175.91338    165.86409    166.09594    170.14584    165.01509     160.0763      161.511    162.68973    153.03295     154.6673    154.41837    161.79988    153.26241    150.25819    153.30596    150.04252    157.94017    150.12268    151.62048    149.35616    148.87944    145.96468    150.75066    142.83725    151.61993    151.19263    147.06375    155.45845    156.06823    151.07814    148.55432    149.77756    141.60253    141.68837    144.99122    146.78906    144.84191    141.84751    147.79676    154.77279     170.3367    178.84063    183.81207    181.30305    168.77165    163.63156    161.56732    156.88341    167.06318    166.76243    168.35639    177.60122    181.89396    177.40415    167.90995    157.64413    147.50524    137.34931    140.00031    146.36016    162.45564    172.12193    177.44501    179.52117    179.87743    166.10031    154.61114    142.05319    139.33897    135.03036    129.57195    125.55877    127.17487    125.50292    123.99593    122.71009    117.11456    125.62855     121.5382    117.47455    117.71205    114.36895    116.47015    113.04602    114.95008    109.94268    115.55181    109.05838    104.54491     100.3185          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       47.50    310.93045    278.26036    264.70594    253.22024    240.17815     240.0143    234.24382    223.54801    218.79168    223.48451    212.25739    203.95247    202.03795    202.94122    196.66611    196.61521    198.50844    183.15338    188.33093    190.34431    177.65941    185.50406    177.73719    180.18053    190.33302    200.15863    210.55465    213.45481    208.15025    188.89675    178.93869    171.32058    163.70789    166.24652    166.24546    165.48425    164.65543    162.35783    157.57272    154.52105    155.76505    158.15587    150.00477    152.41225    151.13315    148.02467    158.86873     154.3952    149.71267    148.94743    146.88281    152.08277     148.9042    151.05072    153.11207    150.92442    154.88542    152.05368     157.8853    150.98884    149.91546     146.1803    144.20228    144.52902     145.9822    146.26235     140.8773    143.03206    146.05875    157.03114    166.55433     178.0017    180.68343    183.59555    173.79967    159.49003    161.83067    153.94045    166.23815    169.32557    170.81301    172.95416    172.33629     170.3765    163.93909    154.49505     148.7483     141.3261    144.59932    151.68616    162.59152    177.36536    181.33278    182.25441    176.98825    170.12493    157.40229    150.75251    134.04273     132.6302    133.24363    128.09977    125.07034     122.9436    124.72656    119.55366    119.83925    121.43001    115.67549     118.6245    118.05549    118.12793      114.875    115.17373    116.23393    113.70153    110.48831    105.85092          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       48.00    313.94768    283.02174     264.0202     256.1537    244.28802    238.41594    240.54137    224.35534    215.72835     220.3289    213.18493    200.23727    201.12833    197.88898    198.00311    199.48004    195.04361     183.8508    183.94513    188.78348    179.74062    179.95331    173.40254    182.23714    187.86735    200.00801    215.56094    213.71827    212.65278    192.20479    177.06081    172.33038    163.84645    165.15323    166.34356     163.0346    164.78621    167.47996     156.5666    156.49579    154.68626    153.96922    155.24901    164.62046    155.87617    156.25583    158.87772    154.53208     149.4802    148.45912    148.52752    150.73204    152.27855    149.78678    151.79802      151.089    155.66598    158.41126    157.28573    151.48513    150.92769    147.24215    143.28178    147.64682    143.67132    138.48678    148.51501    143.39473    149.20984    154.96367    166.85503    177.88768    182.49544    175.81821    176.14053    167.32909    161.47395    165.09708    168.33634     170.2469    169.05271    172.93595    166.72097    168.67994    160.66179    148.31597    143.59564    139.43522     139.0318    150.96918    162.56225    171.62506    185.01388    186.43964    177.49991    172.51861     161.1266    151.62467    141.40859    133.76553    132.95242     127.5324    125.49076    128.50564     121.7257    120.69208    124.32071    115.94784    121.76803    120.81722    117.57922    118.36557    116.74294     118.0528    112.33473    111.66608    108.76111          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       48.50    313.30152    287.87826    262.01309    255.81834    249.81938    236.74354    236.61075    227.64008    222.40644    219.68029    212.45553    198.23592    202.11432    196.53683    197.19895    197.52888    191.20808    187.84198    183.52565    187.76607    183.79262    178.93559    174.07378    180.32432    187.39312     194.4435    216.94976    214.83043     208.2338    196.71286    182.75035    170.56637    165.42745     165.3378    163.21824     162.3155    165.43956    160.10571    160.89677    159.88678    154.28349    154.52051    155.83606    153.28972    152.89301    155.72899    155.74874    154.37479    155.53059    152.34772    150.88127    149.12445    149.83066    148.12495    151.08918     154.3073    155.02971    154.51547     154.4938    153.36715    149.69272    144.84532    144.44371    144.88698    142.60209    146.24414    146.44919    145.14735    149.07884    153.83152    163.24812    181.04199    182.19796    185.45874    175.31626    167.26735    169.80161    174.82138    174.51502    178.78672    176.22648    176.77414     170.7017    170.10435    160.83032    150.01624    142.53397    138.85464    140.99364    148.25291    160.99803     172.0422    185.01251    184.70789    180.30531     170.9264    160.56589    147.70579    146.13006    138.40757    131.75801    129.50965    128.68318     126.8319    124.37727    118.91013    120.40398    122.85167    118.67565    117.41663    117.30349    117.76407    119.73721    116.69616    111.28908    105.46311          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       49.00    312.70634    290.85134     261.2677    254.48386    249.55178    237.27483    231.12563    229.10103     226.6723    219.33019    212.79319    195.36699    201.46319    198.16042    197.28119    190.00453    192.21422    189.79634    187.79946    188.89566    188.55469    176.47834    178.22112     179.2134    190.42148    197.23354    212.47518    213.92741    207.67241    197.47546    178.25754    170.01593    159.45653    167.54293    161.64119    162.42287    163.49166    158.25264    155.50614    164.21721    156.89257    160.88699    153.90354    156.20032    155.93829    157.41356    158.25046    153.60291    151.26365    150.70015     149.4884    150.35303    152.20614    147.54242    151.67842    153.81152    147.16512     148.9411    151.32214    151.73664    142.30901    140.64074    145.99341    143.86111    146.18144    142.98489    147.18171    148.15841     147.1407    152.11766       166.04    177.95922    182.62603    183.71924    177.19774     174.8514    172.67775    181.63273     184.2426    192.56224    187.75609    183.20244      179.426    173.13477    163.63385    151.97103    146.15608    133.80742    138.15082    145.60563    162.21383    172.29686    183.73489    188.99631    179.30067    171.22771    154.47819    147.90854    142.79737    129.83958    129.40917    129.28598    125.87919    126.61275     125.6047    121.56082      119.342    120.35393     115.5049    118.09524    115.99503    115.01791    113.39614    114.52633     101.9734          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       49.50    312.90716    291.73628    261.82656    254.30501     246.6399    239.11177     226.8792    228.88005    224.53184    224.00331    211.47493    199.82194    198.62544    201.76938    197.93581    186.50572    192.11984    189.73835    188.58022    184.55805    185.89445    179.96448    178.10354     179.2796    191.90613     204.1678    212.62237    217.34221    207.22665    196.25917     180.2612    163.25424    166.74524    167.74042    162.87799    165.03413    165.95623     157.9587    162.94105    163.84012    158.60354    160.05872    155.24025    154.87941    156.67295    156.58886    156.74107    151.41334    153.62632    155.96813    150.44675    151.87552    147.37563    148.16773     150.7045    148.70601    157.11159    155.60783    150.85148    154.37086    146.58887    147.42527    147.51668    143.89043    144.67194    144.47762    145.08303    147.09367    148.83558     152.0455    171.47521    175.58009    188.98262       183.73    178.81158    170.57291    179.65888     185.0007    195.38483    195.34543    198.91029    193.55769     192.0329    183.96591    173.66894    156.39969    145.65129    143.67322    140.41849    145.72854    159.29935    175.37559    184.79725    186.40541    183.25262    170.90707    161.85877    150.11333    143.21109    135.09381    133.87943    129.75558     130.6145    127.15763    127.93233    122.43471    122.28396    116.78561    120.58019    121.94336    116.99254    116.98106    113.53381    105.88441          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       50.00    313.90104    293.78773    261.29548    255.92991     242.8941    240.41243    228.55339    230.67555    219.49051    226.68087    212.72242    205.01528    198.55983    205.34256    200.68051    192.86232    191.18882    190.04822    186.74687    185.55436    187.30925    186.24589    175.62553    180.91372    189.69539    200.94924      208.871    215.42442    201.69905    197.35718    182.48123     164.6331    170.55979    166.80254     165.0189    167.48114     164.2707    167.37583    166.07937    158.12111    167.12022    157.83645    154.57477      161.222    153.92848    154.40812    159.46475    152.42809     156.6747    150.49704    154.91355    145.83013    148.64921    149.82743    149.07172    156.20668    159.45784    154.09255    162.45464    154.53575    145.10044    146.20391    144.51416    139.31581    144.55051    144.59082    143.15373    145.72129    148.22868    156.15394    170.37294    180.38827    182.75155    182.78078    176.17178     175.0481    180.14221    191.87791    198.84888    205.89717    210.03965    208.11315    207.14808     197.6043    178.44922    162.97002    147.84073      143.683    142.14974    145.58149    155.17225    166.70263    184.02633    186.86542    188.04266    173.91331    171.95318    155.71271    145.69042    137.05363    132.17166     129.1359    126.98553    128.66142     125.0708    125.00854    125.97122    117.25888    119.56699    119.55585     117.4756     112.1842    105.72105          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       50.50    312.78221    297.00078    260.99984    259.03778    242.61814    242.54531    233.16451    228.36163    218.98121    223.04916    210.70899    210.34443    194.35729    201.67314    197.88073    197.58286    194.96913     184.8421    186.44023    193.93212    183.65263     187.0472    175.61012    179.34744    188.17001     197.6678    213.34009     213.3249    200.89237    194.42568    181.58493    169.44404    164.70005    166.03601    160.97364    162.69948    162.92375    161.78054    166.66556     158.1592     158.7607    157.82369    153.45023    154.03821    158.89288    154.42971    155.81436    153.71011     158.1316    161.42229    155.72741     147.7188     150.2128    152.95147    148.45108    151.20432    151.38968    155.14205    156.76427    157.31029    154.17122    150.93536     142.0938    142.48693    145.50126    144.34759    144.38264    144.23768    145.04894    157.55171    171.35351     175.7371    180.56956    182.59287    177.96582    178.45176    179.69053    195.52562    211.62225    219.20952    214.42092    211.83341    217.28953    206.96737    186.53281    166.18838    153.42528    141.41523    140.87641    152.83953    158.31747    170.47616    179.33733    188.34893     187.6586    177.82607    164.53956    147.98031    146.68858    136.23416    131.71602    129.48772    128.56869    125.37154    126.20873    125.17684    123.19003    120.92755    120.11733    117.40039    114.15111    106.26938          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       51.00    313.27767      300.026    261.84117    261.36004    245.00323     239.9732    238.82547    227.15532    220.12945    216.70047    203.69469    211.65345      196.256    198.83276    192.18957    193.69804    193.48907    178.95744     187.2713    191.66273    183.62761    182.06907    183.64028    177.24142     189.9812    199.35719    214.12241    207.39952    200.37122     190.9429    181.06329    175.64811    162.01086    169.26052    163.01545    157.07032    165.72394    159.74689    162.57663    162.87892    163.46014    160.91047    159.88874    155.17425    151.89059    157.00334     162.4393    154.31342    151.86314    153.98581    154.44274    157.69644     148.9802    143.53929    147.67958    154.24632    153.68535    158.27137    151.74301    156.51459    152.19276    151.86974    142.33605    144.10612    139.34492    138.71234    145.47625    146.31475    146.70776    156.24633    169.10449    179.66524    181.61402    186.22766    176.36379    178.42099    184.47514    199.37156    219.66827    230.74573    220.64866    212.01595    210.09436     198.6053    182.92496    161.09307    147.92367    146.78294    142.04176    145.20999    154.59777    164.44938    179.78545    183.47617    184.93783    171.89247    159.56025    151.94539    145.56251    141.21053    132.16791     127.8467    129.21919    124.41233    126.85548    129.07248    122.13704     122.1183    124.74244     114.0246    106.14557          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       51.50    315.46065    301.30618    271.51101    260.06769    246.67914    239.66523    236.01843    227.20612    219.30463    215.82168    206.18684    211.34306    200.79455    200.71192    192.79556    193.25876    192.32645    185.59995    190.07262    185.77636    186.76258     183.4737    183.45729    180.68472    191.98513    202.17486    208.88968    208.77677    209.43073    194.74245    179.95891    176.22641    167.83517    167.61415    162.23291    159.39955    164.93979    163.32598    164.46987    163.45998    161.84643    160.15681    161.57518    156.56125    152.13324    162.80713    154.17231    155.93958    155.19335    155.44441    151.73351     149.1296    149.19299    151.58224    149.20062    149.26067     153.6706    155.89195    159.41402    155.17777    152.11553    151.23167    149.73137     145.6677    139.03606     145.9916     140.8906     146.4233    143.61892    153.20181    168.42604     177.1226    187.49988    183.46917    181.39681    183.43573    184.73651    205.86081     227.3151    243.70341    232.36021     215.6429    196.68238    182.61743    168.55676    157.99591    145.73594     142.7399    145.34442    149.21999    154.80564    166.38722    166.02105    177.02036    175.47415    168.48493    158.41402     154.4094    143.44942    137.43734        138.3    129.78856    127.40366    129.80321    126.56728     124.5118    126.23847    119.43445    115.03393     111.1109          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       52.00     319.0202    299.63858    280.71689    257.34956    246.28455    243.27954    232.63863    229.68373    218.77764    218.80838    204.96789    212.84005    202.97231    198.70054    192.29168    196.91007    194.36682    189.95932    188.91103    188.19085    188.94783    179.17769    184.77704    185.50234    190.75986    203.00742    209.12317    209.22247    208.70473    201.88125    183.54162    173.38509    168.81888    171.94829    164.64407     162.1074    167.32694    166.05488    158.77684    163.78784    167.62282    159.34677    160.40423    158.72277     155.4375    157.15322    150.22836    154.37138    151.26398    156.30087    159.01332    150.57522    153.15634    149.20931    149.44122     153.5828    152.88535    151.73658     158.7663    150.68149      152.028    148.32608    153.73583    149.53941    143.20304     143.7175    151.46526     147.3424    151.26193    161.21783    166.94689    179.44291    184.96349    186.62307     182.6575    179.29743    187.80212    202.53226    228.76908    243.56374    239.58061    213.01778    197.90493    172.78979    158.39594    151.40763    136.83432     138.1298    140.85802    144.82819    151.32442    169.37967     176.4728    173.23486    172.22114    161.85253    157.99141    149.69964    143.35249    142.07677    134.43872    131.75647    129.16783    126.14155     122.3749    123.62077    124.63521    116.83835     109.9601          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       52.50    319.28345    296.11649    289.28083    254.79557    244.13242    246.37952    233.11396    228.79837    218.14989    219.51949     201.8997    213.43238    205.57872    199.84779    194.39446    197.10973    192.38435    189.39341    188.08108    185.36549    188.52812    183.60949    187.35056    190.57584    191.27326     194.0336    210.06912     209.5681    202.44939    196.12747    180.42705    172.08782    171.06645    168.07017    167.04146    163.59684    165.76378    159.95995    162.48988    161.48139     159.2462    158.70298    158.22282    157.18117    160.01803    153.22884    154.13071    157.60027    156.39376    157.01988    153.30621    152.25353    150.90564    152.10271    148.02644     151.0021    158.13821    155.34961    156.63467    159.16455    155.62406    150.54666    148.02783    146.84702    142.10676    142.48881    142.09746    149.15647    151.77681     154.8666    165.40148     177.5921    191.91479    188.35672    182.23693    180.21355    186.33508     193.2261    218.51379    239.35617    237.17997    222.04922    198.55313    170.91692    155.73882    147.15522    142.74159    136.95395     142.1407    147.88102    155.51529    167.12123    176.82288    176.97979    169.05153    160.73856    156.14473    150.25994    144.75131     139.6295    134.57553    134.67473    130.18561    133.41895    124.69943    125.64961    121.61864    109.65734          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       53.00    318.12945    295.28669    291.35632    254.56376    242.51129    242.44837    233.95769    224.91854    217.93706    218.77242    212.53451    214.87013     209.3965    206.69103    196.58562    194.14686    189.04566     186.7119    181.46607    188.38348      190.113    183.29785    187.73322    188.83112    190.21472    197.81536    212.56973     211.0411    207.99053    195.54674    176.26683    169.78921     170.0498    168.45034    170.04883    163.45968    168.85796    164.42596    164.09336      156.241     163.0101     169.2337    161.11589    164.43005    161.99135    155.06382    159.79806    157.08634    156.94088     154.7301    152.06758    156.14607    153.70924    149.96281    153.42263    153.74385    151.04248    153.20014      158.442    156.91337    158.37593     151.2039    151.16206    145.53391    147.47571     146.7417    143.91798    145.99777    145.99114    153.40855    162.03763    181.37762    184.10948    184.15999    184.38172     177.2487    178.39786    193.90216    207.52969     226.9739    230.87074    219.63857    201.55872    174.86961     157.4494    147.14927    139.87693    135.23083    137.92339    146.15124    160.66136    164.49728    172.89902    177.90126     179.9667    166.95527    156.99273    146.43927    141.65406    138.87574    131.04042    131.91736    126.36808    132.57143    124.17674    116.15482     110.5619          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       53.50    318.90654    298.53878    290.75994    261.62843     242.7092    235.97523    233.12633    221.87864    217.18375    217.59231     220.0582     215.7594    208.73872    207.86839    196.07507     190.3829    192.11868    180.66187    182.90053    195.10676    187.90547    184.17913    186.93265    183.94937    193.44525    204.07441     205.1527    211.41368    206.85949    197.26362    178.55245    177.27747    168.92527    169.28494     166.2922    164.52714    167.30684    166.34199    165.66366    165.32556    165.14549    162.33126    162.37737    164.01599    158.55986    160.19517    156.67838    151.67852    150.99271    154.91013    151.71516    152.84084    155.19078    153.82329    155.24001    147.89945    160.14317    154.14481    156.18944    153.10557    151.59326    152.24675    146.07543    149.10868    144.77397    148.23592    146.77924     146.2812    149.89577    152.51164     167.1035    175.91498    185.27323    188.61787    177.05553    169.51841    177.07251    178.84004    196.53022    210.18698    221.54339    217.18756    206.17722    177.25985    163.66124    150.39093     139.6526    138.74923    138.90019    148.45255    158.62284    165.42853    177.03686    184.33345    190.33802    184.22075    166.68996    153.83773    141.78828    136.29548    130.11994    132.49869    128.17401    127.26081    125.00241    118.69544          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       54.00    319.50614    302.50343    290.05504    265.84382    247.83594    232.40161    233.42471    220.87963    216.41643    215.95988    214.91648    212.90723    204.86979    210.77389    193.62937    187.50578    192.19483    182.09784    192.41722     188.6591    187.01183    184.67424    184.29664    188.19554    197.96325     206.6213    205.74914     205.9827    207.41867    190.06096    182.72681    180.48859    164.24956    170.41017     165.0431    166.44071    165.04082    158.39875    164.07488    162.55804    164.23044    161.58891    163.62135    158.77275    159.80852    160.50781    159.12077    150.21285     154.4237    153.82343    153.18791    149.08136    144.66294    152.89769    151.09963    154.71431    155.72106    159.39329    156.82589    150.58733    150.96465    149.19987    146.29234     148.5255    147.76762    149.12755     149.2432    149.57766    152.37863     150.6284    162.69457    178.10531    188.77101    187.11002    177.10242    176.52649    168.08304    175.95763    184.90388    200.93189     210.3776    206.96986    195.44412    182.18621    163.97967    152.47547    145.98389    143.47009     141.2479    146.85518    153.43023    169.33117     177.0981    187.33271    197.86963    202.24261    195.37424    173.02503    152.23288    139.86297    134.79394    127.15137    128.25016     124.1785    115.41665          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       54.50    318.93531    303.43722    288.35234    267.23305    254.27591    234.29085    232.19462    222.07587    215.63712    212.83212    208.28839    207.94818    205.59084    207.05659    191.26407     184.9951    189.23241    186.28838    190.81565    186.01977    184.15995     186.0164    180.05668    185.86468    198.23083    203.57744    203.64669    203.49177    200.48107    191.34992    185.86259    171.96953    168.12812     176.2209    170.30587    170.39261     166.2107    162.93758    164.23344     161.8115    164.05305    162.17273    159.28211    164.75008    162.51713    157.57855    165.44055    154.38518     157.1813    152.05802    150.62455    152.60158    156.98141    149.48933    154.74295    151.27182    157.93617    163.82058    157.35113    156.35271    156.16993    147.66703    147.74076    149.42058    145.65903     145.1544     148.1464    148.52517    148.16663    153.05139    163.47757    176.96393    180.69471    186.69756    175.65556    170.04392    164.79726    173.00665    176.98382    187.76745    191.03863     195.8711     189.5587    176.98894      166.046    146.43666    145.23841    143.02749    140.69853     149.0654    160.24635    164.76547    175.83499    190.15252    198.04044    205.99215    201.07903    184.75984    168.28893    146.78179    134.88701    128.94491    129.18904    121.52871          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       55.00     316.5164    305.21032    287.06507    266.60731    260.23864    238.03135    229.21181    221.77204    214.53302    212.84066    209.67454    206.35599    204.42703    199.93901     197.2269    192.36448    190.27472    187.91949    181.88693     186.5854    189.18276    189.05321     179.9384    183.54618    195.56829    204.85651    211.21372    206.28311    199.63323    193.86347    178.57162    171.65971     172.1723    172.81002    163.56505    170.37134    175.48463    164.90239    162.36322    164.12146    165.85498    167.60884    160.69273     164.5752    165.90458    159.88965    162.75058    162.69681    154.19639    153.69027     157.2798     150.3696    153.27154    147.59667    155.58017    154.07701     154.5494     159.6773    160.36593    151.25884    154.55782    154.07629    152.88602    146.46971    148.08334    150.17386    151.36432    150.42553    153.63399    155.84152    169.58486    180.09027    179.66108    185.90334    178.88438    174.47531    168.51956    173.19803    179.84854    184.33497    182.53398    186.23142    181.58316    171.57076    155.71708    150.02964    147.32567    145.97037     145.1098    151.21686    153.97958    164.20481    177.22118    184.79083    195.67839    200.11022    192.00125    180.16704    168.43182    154.80405    135.84977    129.42693    122.05468    110.99775          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       55.50    313.88335    307.81494    288.48304    270.39905    262.73339    239.43598     229.9908    220.52633    216.29304      213.552    214.32521    205.12546    205.77103    198.04953     203.1126    200.04842    193.42836    189.58689    184.29181    187.76027    187.68684    194.52273    178.08815    186.64777    195.92571     201.3397    209.08156    204.21716    194.50439    187.39619    185.67061    175.27456    172.97909    172.55462    171.47243    170.40296    167.47629    165.39911    164.55576    169.56269    163.01143    156.75733    159.48946    163.97349    159.97474    159.87257    158.82498    158.58356    157.26258    155.93434    152.78827    153.59583    152.99374    153.01629    150.56698    155.74322    154.94018    157.11851    164.98956    155.87845    151.84944    152.54344    146.96476    143.11254    144.24907    143.60361    145.79456    144.73704    152.48305    154.67999    171.35979    176.12442    179.88543    182.96067    182.29137    169.98923    171.07125    170.44492    182.44435    189.52387    190.05385     181.5057    174.49205    168.95815    156.63491    152.06509    141.14507    140.61533    139.26713    147.54552    155.16823    163.17449    171.39484    182.37539    185.54643    186.83083    174.29734    171.46036    159.83945    147.89693     135.4472    124.97322    115.65012          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       56.00    315.10421      304.631    292.62289    275.87418    261.73155     240.2483    234.78454     225.1516    218.15828    213.17719    215.20671    201.56313    207.71463    199.69706    203.02881    196.79505    196.35431    189.45196    185.84928    185.52088    183.18766    185.74182    180.29821    187.18133     196.2491    195.82366    208.50372    212.55817    194.67846    190.43535    178.56029    178.00134    176.37681    171.66873    174.19517    169.43644    168.84076     165.5898    166.36665    158.73904    161.61985    156.42894     161.3781    165.39358    156.18193    156.63203    160.75377    161.65144     159.0291    152.82885    152.19271    151.93892    155.76778    156.76374    153.50932    149.69457    157.29056    159.56967    160.57045    159.08678    156.32094    151.36892     145.6841    152.35114    147.59938    143.95492    152.82304    142.99762    156.17732    159.86228    167.88358    178.52082    184.52127     183.7186    180.90465    171.84015    168.37656    177.65292    185.77826    196.87654    197.62062    191.74763    176.11608    163.75499    152.27774    146.61712    146.02007    143.20527    139.01406    150.70495    149.34469    164.93084    167.28115    174.85236    178.83392    171.75041    166.45916     151.6611    144.10929    137.74383     132.3988    115.32777          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       56.50    318.18413      299.272    294.36995    278.06747    261.43763    242.56889     240.2552    229.21281    215.99884    216.05013    212.88851    202.97736     204.2636    200.30571    200.29722    188.20632    198.06409     184.5422    184.07941    182.57741    183.38152    184.88427    182.81302    185.85577    193.55197    202.06316    211.78498    210.88962    199.77976    191.39858     178.2452    171.35138     171.5344    164.70407    174.22767    173.46661    170.83231    166.41539    169.92224    165.19388    164.15011    164.56221    160.49733    159.80929    160.82623    158.98019    159.68026    157.63299    154.91517     160.6576    157.28479    151.34401    154.35445    153.34143    161.84358     155.4173     156.9856    157.31174    161.32937    154.95707    154.98058    150.61579    144.63951     147.3119     143.8444    144.36326    147.91163    143.79049    150.77246    159.73845    175.03818    183.39945    185.27026    180.75735    177.25689     173.3676    172.77641    177.53349    194.12092    202.74091    203.92989    189.67876    179.31373    166.35729    151.89333    146.27304    138.05058    141.12389    141.60954    144.04817     155.0153    159.24331    166.01165    172.74488     168.6674    169.66823    154.67087    147.80952     136.0377    132.25971    120.93481          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       57.00    323.10346    293.35549    292.92677    277.04089    261.64222    249.39258    237.54798    226.67012    215.97828     219.8857    208.80744    205.44768     199.6225    201.87046    201.54953    194.07427    196.31652    189.06363    186.32793    180.25027    187.93172    186.50446    188.29833    184.14792    190.76476    201.86904    210.48832    207.79887    211.27178    190.21312    181.58017    172.10317    174.04612    171.29236    176.06468     170.7402     164.4773    164.39398    173.04417    162.31393    163.07706    162.47305    160.13679     158.2524    157.63571    151.01145    160.10579    159.23275    157.75615    159.61864    154.55888    145.17759    149.69274    152.83061    156.63267    155.44184     156.3121    153.37548    160.71823    164.33528    152.88637     154.2263    151.99814    148.72004    149.94532    140.72961    145.35938     149.9025    154.52394    159.00911     169.6509     179.1328     185.5738      183.685    179.37816    172.04465    176.01633    190.96656    193.26479     198.5377    195.29024    189.32127    178.89025    168.08387    154.45623    143.86959    137.40124    136.98074    141.72601    146.42413    151.77309    158.38292    170.51237    175.03036       169.01    163.98562    154.27075    141.99697     132.1011    121.31448    114.83289          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       57.50    326.94957    289.65142    292.91537    271.91156    262.29914    255.43849    233.27669    222.07776     221.1184    216.95559    207.97283    208.93054    202.44694    203.63707    205.63761    199.50776    196.94054    204.94048    190.99882    184.20777    189.42665    187.06426    188.28803    180.93202    190.62592    207.59483    203.60926    210.04668    202.93679    198.21437    178.80223     171.4153    171.18582     169.4342    174.21711    165.97251    165.12416    169.12375    169.95098    162.33536    164.70993    161.83568    164.52374    168.71653    158.17781    159.40885    158.61809    157.13728    156.52994    157.68533     158.4623    153.51007    154.03422    150.89087    152.86378     159.7978    158.27641     158.6147      161.752     157.2199    159.66477    154.79748    151.86191    153.01497    146.48306    142.38029    150.44079    147.55479    148.74702    158.20177    166.79975    179.71023    184.87599    183.78759    180.16081    170.45932    175.85593     181.6486     188.5181    193.01769     190.4004    185.88441    171.50144    161.01447     148.6778    146.63809    143.10101    139.80482    141.07499    137.57584     152.5869    157.53694    163.29163    172.01239    166.93344    163.31549    150.83484    134.54276     129.6207    117.14989          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       58.00    328.51899    289.71782    292.53119    272.72719    263.51786    257.57774     234.8606    220.20861    227.78894     213.0813    210.62776    212.04206    203.51551     205.2879    199.14881    201.50912    198.74736    203.25415    193.91074    184.48797     183.3087    186.43682     190.2179    180.27755    188.22715    195.58645    206.00144    209.34543     200.5697     197.8727    181.05638    171.31435    172.39562    168.47507    172.40361    168.02504    167.83268     165.6019      166.486    168.88219    167.64512    161.47522    159.70016    166.44843    160.28387    159.15129    162.84129    156.24816    156.09764    163.31246    158.17485    153.66495    155.51731    152.10264    151.47565    154.66575    157.61857    166.44054    158.74006    161.12209    158.54031     157.7413    156.10885    151.56269    152.80186    147.20881    146.37776    151.37465    152.14467     156.9386    168.15012    178.60701    185.69498    180.12729    181.08695    177.24257    172.86861    179.98082    184.83582    185.12469    179.26208    175.68866    174.45975    167.34804    157.93956    147.42702    144.33067    139.55353    140.17944    142.71411    153.15173    161.71698    167.07508    174.65128    165.31433    155.15104    146.99785    137.07423    122.47458          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       58.50     329.5428    293.63726    292.19071    275.45074    264.46841    256.02765    244.08855    224.92003    228.33875    215.92839    212.04475    208.45181    200.53262    204.61846    193.13568    203.09805    196.67979    199.65571    191.17041    187.67238    177.77404    183.08893    186.09151    181.92083    189.85332    201.10336    208.82477    203.47782    207.33625    197.47858    187.49524     174.1721    172.66702     169.2008    168.22417    170.97659    167.63928    168.24419     169.9943    165.77054    166.12661     162.5093    161.27167    161.21827    160.73711    166.28577    155.84963    160.01303    157.48851     161.7708    157.82351    162.46291    156.04968    156.98913    158.38548    155.75849    151.62173    163.40288    164.55754    162.99285    157.80681    155.80396    155.81079    142.59198    151.32071    148.33279     147.2664    149.72932    154.20339    157.62004    168.67574    178.51491     186.9277    190.28722    179.31018    175.01995    175.52361    175.93487    175.87018    178.48167    175.29219    174.11922    170.79614    164.75247    150.50087    140.21534    146.32138    138.89152    138.64323    140.84131    150.96929    157.85824    157.29666     164.8993    166.39421     152.0689    146.17093    130.85693          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       59.00    328.21411    296.70623    291.73178    279.61978     265.2412    254.19886    256.38166    225.93253     226.8111    221.53981    213.28331    204.62445    199.65621    197.32368    195.36592     195.9538    198.78731    200.40841    187.92358    191.93372    176.35098    180.89633    183.35495    182.58241    189.71258     203.5582    203.81345    206.84408    204.43562    197.32347    187.92265    183.34072     174.0734    170.49319    172.79942    171.86232    169.75087     166.8557    170.10022    159.16837    164.45444    162.78383    170.84023    158.79329    154.23945    162.79004    157.55913    157.84759    157.50885    155.41628    152.37385    159.23318    154.39797    154.83282    160.80512    154.22217    161.08786    160.17279    155.14772    160.18824    160.92838    158.15388    154.72906    154.76022    151.45873    145.93363    145.10602    148.61356    153.71431    155.82472    165.51969    175.77394    183.33994     186.5674    183.55094    170.71967    165.84292    170.14499    171.19985    168.00201    173.65587    171.59819    166.96204     155.6374    150.27491    143.80067    144.37118    134.24535    143.54268    143.61653    148.00706    155.95724    164.78031    167.12588    159.83568    152.36785    143.07695    124.92526          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       59.50    324.22821    299.65184    292.13585    277.66734    259.77679    255.01215    255.27898    228.49312    225.64254    220.62094    215.01717    205.04649    197.70766    195.04517    198.60646    193.70426    205.24442    200.58898    194.81301    191.93815    178.96221    184.45523    182.93387    184.88689    191.74086    205.31809    204.62086    206.17182    204.16804    201.76307    187.85124    181.58212    177.29407    173.70128    169.10015    169.24909    168.97097    162.07044    164.70003    164.58284    165.16507    169.36076    168.77719    169.55405    158.12942    159.85858    160.43657    157.92929    158.90435    156.29037    153.26901    156.93489    159.81624    156.04304    155.74485    160.95143     159.4622    159.97035    164.85452    164.02493    164.10173    161.54764    155.51799    152.19476    155.89848    146.65801    152.19068    156.97925    151.19086    155.50202    166.19205    172.67577    181.38184    186.74193    178.85687    174.87995    170.14219    174.05457    174.59235    163.31982     164.6074    168.27435    163.48093    152.15914    148.32593    143.05823    138.44583    139.91372    135.71123    141.44287    155.90375    157.83371     162.7086    163.35612     159.8792    144.12457    131.46844          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       60.00    321.89882      300.547    294.32659    275.10588     253.1585    255.81624    250.71667     235.8764    228.24616    218.19005    218.36213    208.75314    196.09955    199.62321    200.23052    195.34193    207.78687    194.22727    191.79635     189.0663    188.77856    183.36568    183.16279    189.42529    190.31778    204.85955    203.94888    206.29405    207.11975    200.26054    184.90843    181.89528    177.95121    171.85758    165.94973    168.63683    166.41857    167.61624    166.14235     162.7075    165.25453    166.74019    164.57359    168.23129    161.68477    157.58245     162.8647    159.13407    157.81255    155.37989    158.00086    155.70646    158.84484    154.56244    161.47499    165.29119    161.34628    161.96359    165.03313    160.76591    158.82134    148.77751    157.60826    150.24482    153.50771    151.13713    149.44814    153.77954    153.72964     157.0249    168.03136    179.03858    186.18073    183.26255    177.62668    172.27749    168.92331    171.77278    177.29771    168.79208    163.24569    160.45043    159.89125    152.19574    147.38468    142.16523    134.59663    136.53063    137.22826    143.91593    147.45577    159.79809    170.69013    162.22977    154.59122     144.7629          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       60.50    320.15583    299.51239    297.60722    275.33889    252.47521    255.41358    247.15223    242.04518    228.48432    218.25972    220.42054    210.36368    196.57108    199.90115    197.32799    195.82348    202.41785    192.04188    187.99377    187.13813    197.23569    181.60649    183.03525    190.46075    188.43557    206.12142    200.44184    209.50749    210.04437    202.64641    184.08341    175.22509    179.41053    170.97935    164.23608    164.91017    168.90014    166.25162     162.5992    164.00663    165.43702    163.68335    165.45783    158.66635    165.79946    162.63817    150.48814    163.81981    157.98021    157.73667    159.57965    157.30754    158.82643    160.31135    156.12244    156.97659    164.24411    166.94597    166.49192    160.24458    153.25694     154.3621    156.06678    154.59977    151.34214    152.65891     147.2494    155.13379    155.51901    163.01368    166.33921    175.55814     180.5438    186.03948    180.19393    174.02623    167.52967     168.9494    175.37537    176.83118    167.62627     164.7296    155.29871    153.45752    144.41502    134.32679    142.25119    138.74708    139.58295    145.37096    148.21139    154.27221    159.70019    166.73241    157.80735    133.57258          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       61.00    317.74425     300.4594    293.03063    278.02701    260.94913    252.28304    245.84387    243.16292    228.72489    217.60896    214.55309    207.06233    198.85682    198.21065    198.70019    191.68586     193.8889     191.5621    186.51202    185.79183    195.89698    181.76343    186.53394    188.68522    191.86607    200.78382    196.39062    207.85502    202.69046    192.86988    186.74904    183.00765    177.44738    172.38641    170.85896    167.11427    167.29642    171.36201    168.65686    175.79373    167.17754    166.01037    159.49773    160.77463    164.36315    163.36954    163.84418    158.98791    158.61255    160.47929    161.33194    156.29601    159.28183    158.04024    156.18947    158.47645    160.25373    160.42643    166.94861    159.13625    160.32975    154.83584    153.53604    153.45736     149.1747    153.13279    153.04864    153.59201    152.65302    162.43403    174.51256    178.10917    187.48501    178.49662    171.87166    166.58212    166.31692    162.80461    165.60694     172.2618    171.30802    167.65146    158.23859    156.26105    145.58302    140.83232    140.21452    142.81248    142.36708    144.20702    148.67408    155.67195    161.55301    163.07462    150.67979          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       61.50    316.10154    301.53094    286.79013    279.91422    266.12979    251.47584    246.23185    242.48797    232.67496    216.37618    210.02359    204.75546    201.67676    198.00171    198.89024     190.2582    192.58937    190.84805     190.4346    187.59829    197.26044     187.4094    190.80218    186.18259    193.18068    199.47875    197.75341    202.72036    202.59948    197.78595    183.58158    183.45168    183.88561    175.08686    174.93103    174.16747    165.36608    165.08134    169.66292    166.78224    163.56096    161.89332     161.7733    161.15739    160.41842    168.93543    159.90706    165.87106     160.5433     159.6289      158.804    158.36547    155.61954    157.47434    158.29029    159.87667     155.9128    161.74437    169.39747    167.29823    162.38859    155.33708    150.36523    151.45209    153.31265    150.41443    151.30018    154.33508    154.45695    159.94395    167.23145    179.96004    189.40483    187.64572    179.10128    172.51521    164.34114    164.21988    163.86515    163.13476    171.27686    170.31244    159.91143    151.53132    148.66096    149.53303    141.22178    139.61897    140.61326    142.50767    144.26768    154.10206    159.13206    158.49499     129.8033          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       62.00    315.21548    302.13994    283.40258    279.89607     266.3914    253.02216    249.16788    241.33572    235.80148    220.36078     210.0256    202.79776    199.21744    200.11361     192.3625    196.36779    197.26557    190.78937    195.79921     188.8003    192.93122    193.13343     200.2552    183.85974    194.01119    195.75366    201.65291    198.76512    202.00868    202.01929    181.96949    180.03951    182.97189    177.98103    179.53296    176.57549    172.24452    162.83795    170.23341     163.0824    169.12127    164.62183    163.33523    162.05472    161.45932    161.95185    157.67582    160.30206    161.88455    160.70582    158.41077    161.52457    153.01589     151.0794    154.80509    156.87125    158.27103    164.44813    167.12208    159.76952    166.99295    153.21897    149.18698    149.54663    148.95508    152.21441     152.6906    151.33063    149.58109    164.34564    168.31872    179.31925    186.43713     181.8342    186.25784    169.18139    165.71668    153.60571    154.72912    162.40558    167.57178    160.27157    159.44866    156.55487    154.49366    144.71739    139.34618    138.12299    135.26209    142.10545    141.80906    152.92369    156.91582    146.98697          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       62.50    315.71153    302.63227    287.88577    276.66833    264.18984    254.88174    251.38852    240.15399    236.37826    227.18858    204.63348    206.98198    198.37468    203.01153    191.29642    195.88373     196.4408    195.50355     195.9227    188.85916    189.08974    191.00571    189.23009    190.39321    189.16998    197.38399    202.93318    197.40418    195.96394    201.90793    185.70543    181.62822    179.81044    176.46859    174.24303     172.5398    168.70412      168.456    164.44117    163.16925    168.49119    164.02624     164.8823      165.941    164.99047    160.61624    165.16404    167.74731    166.50528    158.85859     161.5125    158.83723    159.24775     156.3524     155.9761    157.97943    161.38276    165.32127    160.40073    161.71256    164.54494    156.07268    156.68329    153.70995    150.41442    150.42142    154.09324    149.07078     154.2111    158.65039     170.0108    185.54872    187.70774    188.86072    185.07162    182.13862    168.82386    159.40366     157.1999    158.07307    155.90926    160.42046    151.72204    153.63823    145.02124    154.00999    145.38999    141.97575    141.54857    143.64978    143.70264    151.12036    148.39756    130.71117          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       63.00    316.22657    304.14728    291.98521    275.13866    262.96176    256.04306    251.45408    237.61201    233.68601     229.0299    214.05467    207.27801    204.65279    200.18825    197.42235    196.88922    193.77452    200.52917    193.52275    188.61468    192.72031    188.28531    188.83084     188.9605    196.15648    197.65694    201.58353    207.68878    196.31634    189.75594    190.38663    178.81383    183.80051     180.0583    171.43473    169.14647    168.81573    170.91562    170.10519    163.32133    164.24203     163.4934    168.39304    157.50245    158.43154    165.84294     163.0346    159.88716    161.63045    160.97903    164.60845    158.33351    158.62509    158.75174    158.24337    158.05344    157.27738    165.09051    159.97141    163.17881    164.43218    156.83396      158.067    155.93562    153.09128    151.55683    151.61643    152.78611     158.5755     165.1929     168.3778    177.74838    187.29933    190.76222    184.43243     180.6633    164.37364    165.45219    156.47522    158.70884    155.94177    150.95485    151.96053    153.95575    143.75508    148.22809    144.29141    138.99804     135.9966    144.73104    145.50929     149.5373    137.93857          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       63.50    318.08202    306.68927    295.42864    273.46335    261.71095    257.22294    249.10263    234.42591    228.39262    226.22552    228.40145    203.35017    210.90945    199.08947     198.2601    199.93076    190.40912    193.10081    195.44887    190.34812    190.63323    187.87301    185.67484    190.84977    197.37411    198.91685     204.3877    209.11844    196.63219    196.76325    179.44319    178.28005    168.62346    177.78912    176.11999    172.67797     173.2201    174.60057    174.24522    174.55027    166.33428    159.02247    170.47532    157.94212    165.43606    172.12463      167.871    172.20483    158.56476    162.83324    163.20473    161.29429    155.31376    158.11885    153.11157    155.50898    159.75393    161.58593    164.06776    159.22956    164.27238    155.96988    155.13015    156.52007    155.18324    160.90913    153.58598    151.71624    150.66319     158.0555    169.35163    179.99587    183.58428    190.36989    182.19758    178.17225    172.27764    165.70486    161.05949     156.2855    151.35987    152.69868    148.24417    149.73806    148.77332    146.46912    143.91016    144.24799    143.96744    140.76622    145.39156    142.31827    128.47473          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       64.00     319.1771    304.87617    297.39299     269.7599    260.76843    256.30897    246.08098    231.15904    224.98299    225.49534    228.73478    209.73039    207.60693    203.88416    197.50444    202.97882    186.70228    189.31601    193.93975    187.66201     189.8231    187.42011    181.30666    195.40234    199.12983    206.62756    206.06138    211.86314    195.10404     197.6327    183.68713    181.31325    178.73765    176.84133     175.2168    176.35333    167.92101    172.50461    171.65835    180.11259     170.8417    165.98207    168.16112    161.58536    160.40659    167.07781    168.35377    164.99852    165.86429    158.71205    162.19309    158.95107    162.13754    156.50021     152.0693    158.01688    159.13656    167.88411    161.60112    159.53702     158.8587    158.80675    155.78324    160.12905    162.06178    156.08576    152.65991    157.68222    154.44081    159.29765    169.09496    181.27357    188.11961    180.94571    184.96059     179.7656    170.28369    158.21541    156.59034    156.38089    155.43498    147.79438    153.58693    148.80109    145.98003    142.26336    138.04993    131.75763    137.66078    140.72529    140.04342    131.14377          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       64.50     317.9059      302.618    296.08453    265.56999    261.11666     254.7324    245.48741    239.46817    231.34873    228.10656    226.42909    218.08836    204.47567    205.91497    198.12324    201.41604    191.08091    193.91152      188.955    178.64078    186.16875    189.51546    184.74066    193.17067    198.21868    202.88981    202.80211    208.69958    196.16172    194.50879    188.43393    180.86719    174.73101    173.54279    171.15916    175.67769    174.83252    172.69566    175.27965    180.07287    165.91676     172.9009    166.89378    164.75149    161.72808    169.59474    160.60666    163.60272    166.06512    168.60274    158.41157    156.24863    159.80948    161.40894    167.39696    156.62879     158.4966    170.25129    162.25968    164.06649    160.51637    156.33793    157.69169    157.91685    155.80381    154.52785    153.19128    156.31428    160.03322    162.24613    168.64243     172.1898    186.68859      183.436     185.7662    175.28366    170.93767    163.01204    156.96367    155.88778    155.38404    163.16863    157.40444    145.63749    146.14146    145.91152    146.80758    137.12395    138.98851    137.76077    133.33767    123.46589          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       65.00    315.26218    300.63429    293.79359    268.97979    266.30191    254.47105    244.54062    244.44028    239.77566    230.56008    225.98537    222.04341    210.32646     203.0241    196.12706    201.43827     193.9817    196.03475    192.93413     183.6273    190.60077    190.24904    190.37008    191.81029    198.76342    197.52581    212.26675    202.96813    199.49584    192.39317    187.14016    178.81516    175.97605    173.82904    175.03755    173.35955    167.61283    167.11467    179.11925    172.22531    171.38964     171.6938    169.59707    163.11526    162.89132    168.89592    165.48722    166.88164    163.82045     160.6508    165.60052    164.35222    162.25745     161.9572    163.43271    163.31589    161.58656    165.05066    167.92306    164.25937    159.00177    160.90679    159.88428    155.72778    155.47008     158.5235     155.8426    159.83869    156.01149     166.6458    172.27829    178.30547    182.03027    186.07407    185.13551    172.64668    169.54833    166.67099    161.10922    152.98826    153.39861    162.23927    152.92747    152.10998    150.76559    140.04186    140.26127    138.37202    137.06684    141.52595    134.53853          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       65.50    313.51309    301.46446    292.38749    273.52565    269.93553    253.85843     242.5233    242.18088     242.3895    232.92723    225.20058    220.35376    213.89108    203.58659    198.65812    190.40649    201.03238    191.36123    194.29663    189.32038    191.03613    188.07544    194.65189    194.84725    199.96355    199.35493    211.35251    203.13296    206.96286    191.20172    186.63644    178.84751    178.35512    174.75068    179.58249    174.34093    171.25718    173.43007    178.28647    170.92432    174.51323    175.40802    174.00226    168.02191    158.98647    160.85172    164.68414    164.05235    163.92298    160.37875    166.16192    162.67019    163.80854    162.75562    166.95108     164.4671    163.31651    164.77012    166.91145    167.24883    164.89182    153.91589     158.7153    161.65837    155.76852     158.2429    158.10548    158.16874    156.54049     165.8604     167.9742    184.39444    187.15968    180.28796    183.40843    181.94327    167.66164    163.89299    155.86167    153.33591    155.74893    154.93899    151.66988    153.63439    146.17088    147.82044    141.79613    134.70747    139.15792    133.62627    122.23452          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       66.00     316.3754    302.29103    292.64456     274.9831    270.26229    255.89471     240.6254    239.02227    243.02566    233.79176    222.42044    214.20277    211.37227    206.15019    199.57771    188.49567    203.28038    194.32725    186.15771      188.937    191.48776     185.2794    192.14769    188.78772    199.47467    205.57917    206.80757    205.26314    209.23805    189.95883    186.80881    173.40434    172.80108    176.89949    181.03252     177.0562    171.43363    167.24862    173.35029    175.44991    172.03874    176.57361    174.77064    174.26596    172.19527    168.94498    165.99424    161.85545     156.3247    160.92775    162.93704    162.84214    158.65829    155.17532    164.11419    163.23361     164.9396     171.9437    173.88854    159.49108    160.86518     158.8854    158.76126    162.81294    159.91863    158.17022    154.73983    160.49665    161.53728    165.18404    174.20385    177.32128    187.87914    194.42785    179.76009    176.61229    164.33058    160.73691    158.60292    155.55431      155.988    157.75696    156.71559    151.24262    149.95618    151.01999    139.10718     145.6577     137.3475    131.25307          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       66.50    320.20867    302.62134    293.88936    276.22442    270.54791     259.1345    240.94574     238.8504    236.09202    233.30754    219.23166    211.33783    209.03569     210.2844    203.45775    196.23326    201.35449    196.81629    185.81401    189.59879    197.49662    182.68176    193.49078    188.43276    199.76682    198.41347    202.20556    213.99352    206.59471    194.17743    190.09228    177.89023    168.97425    173.20294    172.29137    173.46757    173.89284    168.94362    174.82498    173.54158    171.73856    169.79217    172.60569    170.61569     172.2488    171.07204     173.6129    162.30051    159.97743    164.21777    161.96471    160.17262    157.77476    162.87931    163.41771    168.48247    166.60678    169.47987    172.18423    176.28252    166.65981    160.98912    161.02702    159.75157     158.4091     156.8639    156.88594    155.24432    158.14117    169.27945     171.7963    184.17146    193.20049    191.85565    188.51929    182.50573    173.05477    161.52758    154.78885    154.45441    154.56253    156.72745      150.194    153.85542    150.52967    148.05959    144.74468    141.80819    132.86314    121.98449          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       67.00    321.38138    303.56116    295.79771    278.71884    274.20269    261.93116    243.81667    240.36958    226.41484    237.76042      215.241    215.11401    211.21007      212.175    207.97734     203.2475    196.59414     194.7677    185.20554    191.06932    194.14323    188.11491    191.82483     196.5049    193.99782    197.75888    208.01867     209.7646    211.02637     198.2699     189.9252    177.23242    174.51463    173.65015      177.905    171.51035     177.0837     174.6837     173.7282    174.87361    169.19216    167.12637    168.94284    171.71334    168.96428    175.27013    172.66645    168.07884    159.78745    167.49298    165.89854    157.40402    162.43984    157.98248    169.84766    163.39533    167.44345     169.0073    172.20083    174.73439    171.70123    170.78538    157.69437    157.32767    156.91983    153.32831    152.26801    158.59233    152.24209    164.82467    170.47716    182.60778    192.71738    186.57911    191.61739    189.50352     179.2671    169.24432    159.48154    157.62713    149.92217    157.34411    153.05373    151.02825    147.36393    143.61973    144.84913    134.91894    131.63031          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       67.50    317.75502    304.91033    297.58712    280.89213    274.31311    263.19034    246.40994    238.95493    229.13471    229.94819    212.29438    217.62972    214.01892    210.57679    205.71719     209.7216    192.60038    191.78211    187.24957    184.76872    190.12595    192.45862    187.71136    197.28174    200.43293    199.56225    212.50722    206.78231    207.27539    196.36355    191.03883    175.26848    180.84678    179.43663    177.93461     169.9378    173.07991     175.1365    172.80336    168.39743    168.32392    171.18854    169.73982    171.57775    171.60568     165.9406    167.36737    169.91664     162.1537    160.21025    160.01253     162.7562     163.3762     158.3425    160.85128    163.83248    165.85886    169.49522    172.79036    169.85828    175.25617     163.6439    169.75157     162.3937    144.87101      160.899    151.85745    158.55707    154.43064    160.76908    171.41348    181.20555    187.16119    188.66777     187.7484    183.48356     176.8708     167.4907    160.90134    154.22408    155.46434    151.53214    153.48169    157.12791     148.7962    151.51444    143.62854    140.20704    131.25887          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       68.00    314.31362    306.05502    293.87315     278.7439    270.46666    263.63737    247.11017    240.12926     233.2577    223.38629    213.24664    218.44766    214.25739    215.15648     204.2747    208.66862    198.94383    189.90211     190.1518    183.60188    189.36319    190.07593    186.41764    192.97154    196.26031    203.82015    205.12542    209.77371    202.39298    195.96401    198.31231    182.75038     182.0536    186.20598    175.02385    175.99159    172.09241    172.15722    176.77911    168.82636    169.88134     175.3371    174.08652    169.37576    169.86535     168.4114    171.39829    168.63954    178.34988    166.59349    169.54095     155.3381    157.89733    165.16172     162.9739    163.31605     166.2389    169.84666      171.147    170.83676    168.08335    164.27993    164.38798    171.01702    162.43966    156.16819    153.82042    156.24659    159.05792    156.56354    175.28895    184.54449    189.04262    189.90815    192.45674    185.84104    179.33566    167.75341    161.16256    152.94478    156.26854    152.48859    147.44791    152.22783    142.26474    144.65839    141.95995    139.73193          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       68.50    312.49346    304.32284    289.27529     277.2971    265.33644    262.80547    247.30552    241.52941    233.73991    226.86794     217.2761    216.67999    213.49129    213.36291    206.80402    205.59706    203.15162    190.88199    190.71143     197.9603    188.96423    185.02945    183.29619    189.23542       193.47    199.71148    202.16751    204.07223    203.06977    194.30967    190.08058    184.33339    183.18636    181.83837    186.81806    167.07385    167.84978    174.22623    175.44958    174.09044    172.08129    172.87463    171.88282    174.62781    173.20612    167.64716    165.73643    168.43376    165.18644    166.54147    171.90002    159.51196    160.54332    160.91961    162.61152    165.92892    170.36572    173.32397    172.91094    169.39484    165.87909    163.67236    163.93865    158.57402    156.56262    152.68042    163.15125    157.35701    160.10367     165.8187    170.84927     185.0074     189.2386    191.39777    190.28497    179.23822    170.76784    167.61934    164.21944    158.46172    157.73797    155.76513    157.44546    150.86857    143.48888    143.08938    141.39827    135.38236          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       69.00    320.54029    302.21978    286.23374    276.71576    263.52955    259.04085    248.84092    236.78589    237.68865     231.1657    220.59735    215.55957    212.83963    211.95344    205.42904    208.27352      200.052    192.01446    194.71194    188.27594    191.22565    183.95009    187.82212     199.9344    200.02156    197.39708     198.7974    202.06382    202.12097     196.4155    193.47105    182.63551    182.18733    179.44418    187.36035     178.8188    168.83407    171.49884    167.40051    174.68028    170.08643    173.20956    167.79578    167.49332    170.94361    172.04033    168.78772    170.29375    165.39485    166.53523    172.26659    167.11544    164.05675    165.90105    163.00417    160.46645    165.05617    172.44917    172.30414    175.83243    167.58825    168.28431    164.68497      158.484    160.91511    158.96225    163.12482     160.3484    165.67855    161.07992    169.70725    182.11865    188.24134    192.68478    185.76213    186.68341     174.7247    164.25733    156.30047     154.3906    152.88538    152.11334    156.67513    150.19051    156.03443    139.75558      138.454    125.07554          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       69.50    330.30534    300.02533    286.28488    275.38703    262.05713    256.11205    249.89823     229.8713    242.97468    233.14185     223.1749    215.17406    212.09494    215.92371    204.63687    208.56871    203.69965    196.04828    203.53167    185.00358    191.79964    190.51088    195.83946    196.34308    198.70027    201.19921    196.88397    200.48117    201.00324    197.55449    194.74637    183.22533    176.30036    177.62142    185.52613    182.72464    183.00186    173.62566    171.71478    176.82022    171.37467    180.44412    172.25321    176.25523    169.88347    168.53424    165.20079     171.3619    172.27953    171.21237    167.63832    164.85013    162.19937    169.62806    172.41664    167.96778    163.22965    170.51083    173.80042    174.83126    167.84343    163.32541    167.46944    162.04559     158.1508    162.42712    158.58636    166.12004    160.81052    161.70726    168.78586    178.71398    183.08877    196.89085     196.4857    189.14246    174.09528    163.93767    159.99414    158.06753    157.86384    154.41406    156.06412    152.35462    146.94494    148.55808    137.85063          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       70.00    340.58877    306.15701    287.21684    273.02406    261.10928    256.73177     249.9566    231.95641    237.38341    235.37725    225.26005     214.1729    210.99646    216.26624    209.37476    205.88991    212.09279    201.20007    199.93645    193.89508    192.49718    187.22868    191.32726    189.62026    198.38611    201.13868    200.08484    198.21611    201.03721    190.92121    189.59751    184.53891    184.07433    177.75969    182.46615    185.16286     182.9933      175.677    171.76432    176.90393    168.02694    176.19381    175.54629    168.98753    166.84513    166.51229     162.8263    168.47549    169.42277    169.73693    164.18991    173.88649    164.46201    169.55487     176.0426    170.84404    171.74617    166.43488    170.53384    170.66006    171.42297     168.5018    165.34173    166.81994    159.34223    160.87029    158.18968     162.5939     163.2611     166.2484    171.92066    187.03937    189.35043    194.80135    190.17954    186.87444    178.11969     166.5016    154.41192    156.19223    153.35777    156.80136    150.35576    151.37165    146.16743    137.01087    131.47905          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       70.50    336.12188    313.18278    288.67139     277.1398    258.49268    258.98087    246.21191    238.95161    234.12328    234.47923    226.45822    213.04876    210.04086    215.25498    216.74854    206.67612    208.13981     202.3489    198.46471    200.35872    195.12318    184.78353    191.39381    189.75552     196.6228    194.12033    203.67215    197.72349    202.18537     190.6403    189.37118    186.25325    186.19704     172.8566    181.36907    177.72361    177.35253    181.82137     170.8087     182.8478     179.2214    173.26335    170.56206    177.94402    167.30495    171.46172    177.31027    169.06907     169.8209    172.76632    171.70232     166.5555    164.67659    173.46132    168.68112    171.26548    169.81252    170.98151     174.9252    173.23999    171.99132    166.67218    164.75826     163.8328    161.40778    158.63954    159.62336    143.38357    157.67961    168.12983     174.7602    185.90881    188.58401    191.16814    190.04667    183.22252    172.32186    168.58469    161.70911     151.0341    158.99032    156.17068    149.25827    149.03414    146.02247    137.77334    129.30552          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       71.00    333.39103    319.47382    293.66455    282.62044    254.99352    260.56549    242.75821    242.10809    233.61991    233.21386      226.528    218.99793     207.4515    214.78505     218.2256    211.13001    202.82173    200.23311    203.79773    204.14052    196.36599    199.10327    192.90291    190.84125    191.96951     192.8558    202.08179    202.37806    201.86045    193.34297    186.84973    185.82264    186.20725    174.41043    185.28627    170.20412    171.37542    180.30206    176.57006    177.55273    182.00775     175.0753    168.36295     167.4003    168.35033    169.65923    169.81713    173.10861    173.70889    164.59212    171.38684    171.54988    177.77658     167.5581    167.26761    175.95919    169.81776    172.18802    184.58543    179.38887    171.11032    166.10976     166.5787    159.11316    158.27054     158.7131    159.70277    157.61349    163.27399    166.23998    174.67469    181.68228     192.8985    196.15751     194.8419     189.6836    182.10802    162.91259    160.91057    157.83568    163.21627    148.54671    157.44804    139.13371    142.83452    137.22553          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       71.50    332.96876    318.68759    299.01788    286.82537    256.59483    262.89888    244.61742    239.29958    228.45786    231.80783    224.67224    223.95574     213.2098    214.48368    218.54277    209.91789    194.48035     202.3393    203.40085    205.42876    200.20199    198.94148     200.1566    192.06149    190.60642    197.21173    197.53493    202.97526    201.72038    191.29673     191.0154    185.66376    182.30866    179.68265    176.96908    174.98747    176.27223    174.02373    180.23269     173.9749    185.36074     178.2582    176.15784     166.7399    168.51047    166.88912    165.67184    166.54901    169.01013    167.40487    167.20238    170.27578    171.08708    169.90813    167.93477    169.83991    170.24918    163.61898    177.59169    179.57161    183.95598     179.4661    169.04716    162.27104    170.98798    158.67243    159.20666     157.8078    160.77167    171.70662    176.24299    181.52855    186.90851    197.89726    195.09329    189.49156    179.42471    173.43078    167.53593    158.21727     152.9923    156.49316    153.27779    148.15443    141.50556    129.76613          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       72.00    336.22818    318.34886    302.18754    287.22152     260.1987    264.27622     251.6368    234.99586    228.06596    234.36351    223.32892    221.29379    220.53673    211.87141    210.67518    205.81501    197.23877    199.96072    203.92966    199.47427    199.63072    203.06606    197.32407    197.43976    190.97872    193.00271     194.5024    197.55959     199.9305    186.25255    194.24192    184.59925    181.94854    181.61147    181.23349     182.0114    184.33515    177.80156    179.49183    177.10431     178.8186    176.06684    173.82989    179.99009    172.54723    165.38505    165.26775    169.32681    166.55863      168.944     170.8171    175.17166    165.31465    166.43929    173.53151    169.53749    171.60173    176.07184    176.35833    181.60433    180.38517    179.21055     174.4777    172.07281    161.38661    157.56957    156.39193    160.19292    159.97582    163.81855    176.12929    182.34864    193.22115    195.52961    191.94444    191.27361    182.72042    164.52713     162.2784    158.62646    156.75501     158.8833    151.39377    148.64694    141.36558    128.37521          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       72.50     338.9217    318.50109     300.2263    288.79496    266.14139    264.23439    256.20514    236.21266    233.58427    238.01252     219.7404    218.42686    223.22675    208.61496    206.14071    205.90467    207.92451    200.48693    207.47416    196.08487    196.73344    204.76725    197.02404    198.68081    199.47041     199.4323    197.67632    192.36519    193.54178    186.56808     190.2392    187.43143    187.07195    183.99071     182.5942    175.85231    187.42222    177.44394    176.62456    176.83841    178.08474    174.14899    178.10387    174.49501    180.26171    176.49097    173.70522    161.80427    166.18123    165.27955    169.33783    173.52948    163.60131    159.18326    170.55216    169.04153    162.47945    168.23996    177.51136    177.62017    176.54891    174.18599    168.06318     165.2203     164.4242    167.17175    163.09355    150.79608     162.0936    156.73638    178.28651    179.36028    191.48785    197.54187    194.49148    195.12878    185.23843    174.01771    172.15337     159.4925     163.2929    155.05477    152.75054     147.4334    145.04602          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       73.00    340.82384    319.49555    298.45887    293.23927    273.55737    263.92655    254.58378    240.37787    238.65416    236.15743    217.36012    213.63797    222.22417    203.98022    200.82683    207.52135    202.49028     202.3161    205.40858    201.32229    193.00738    199.58809     198.4381    196.14958     205.6024    207.28161     203.7399    199.36283    194.33979    194.87195    188.56596    186.04166    190.05382    185.06067    181.87967    180.25871    182.91256    166.68915    179.46589    178.31169    172.74849    175.28973    179.69536    176.38186    166.78073    173.43856    178.69488    167.39154    165.01152    165.05731    170.53044    166.67861    163.52275    173.30118    173.97058    172.43393    174.14919    175.43493    174.62378     177.4949    179.07737    176.28509    172.60094    172.43399    165.95605    162.01893    169.15563    167.87322    164.67473    168.38315      175.387    179.77785    197.78284    190.81266    193.07195    187.85338    180.96234    173.09297    162.86616    164.57637    162.33839    151.96801    154.86631    147.35028    135.61046          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       73.50    336.01946    321.23635    298.65762    298.00297    279.98162    264.61379    253.10195    240.54856    240.26111    231.43408    220.27083    210.23367    213.11115    198.29044     200.6391    207.16018    197.05602    203.13721    200.80112    199.14692    196.17038    196.63242    189.39891    195.98541    200.32847    203.31525    206.49777     205.9677    196.95446    193.25355    192.44218    186.74318    188.65889    181.43715     180.9994    177.31424    178.71427     178.4172    179.40065    178.76769    179.52988    174.29101    181.66891    178.82477    176.59877    172.95227    180.17139     178.7989     170.0222    174.97069    166.73629    162.63607    168.46595    171.19086    168.49331    167.96896    172.53341    175.12714    174.95449     180.3423    176.14531    176.16023    175.60442    169.87293     160.7789    172.84214    169.38172    165.56607    168.01195    170.50884    183.80905    185.40378    186.88461    196.47866    197.09527    188.83511    190.08913    174.04881    166.03488    163.59982    160.59739     155.4943    149.20473    148.01068    135.26063          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       74.00     329.4882    321.66903    299.26586    301.52694    282.35621    266.70613    256.69321    239.15794    239.07957    226.70394    222.62536    218.88821    206.45765    207.63846    206.58601    205.57267    198.28615    205.59552    199.15876    197.29691    199.67531    197.58364    191.36548    200.60398    196.19389    202.25543     204.1521    202.00111    204.09245    199.82145    184.21535    183.42703    184.63405    177.51652    181.72038    182.15462    181.46853    172.19221    177.71379    183.81983    181.40299    174.72093    173.16483    173.11132    170.39434    178.75612    177.28103    175.75918    175.51182     164.7708    169.50682    170.35807     167.8219    165.92166    165.22975    164.27477    167.05407     181.7937    180.25252    169.94231    179.08884    171.34582    171.31552    168.99056     167.5518    170.64651    161.12889    167.30604     167.0341    167.28461    182.32027    192.11957    194.13011    199.16272    195.45509    189.01426    182.55012    176.12895    164.14938    166.66648    156.32653    155.75404    155.64737    144.73853          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       74.50    321.88209     317.9699    298.61844    299.25316    286.03763    271.48682    263.90573    241.02055    238.03885    224.42747    220.39844    223.42106    209.18706    218.34585    205.90515    207.25486    201.29945    206.73598    199.10652    201.70612    200.10098    200.91031    198.35191     196.7533    199.89971    201.49049    203.63347    204.15406    201.40255    204.67969    185.20809     185.9492    180.74667    178.32917      176.536    183.87792    181.46265    177.95692    173.99899    177.77269     179.0964    174.52922    172.83702     177.9319    169.69564    174.31382    174.04225    171.57014    177.83384     173.5749    172.48226    173.91812    168.41075    171.73625    163.92861    173.37702    165.46404    174.63188     176.3236    175.64495    177.20926    176.39825    172.23043    173.69817    165.87919    164.80556    163.95813    169.32381    167.28036    170.40244    178.78414    183.98588    193.87527    198.78796     205.9718    196.79807     185.1937    165.14927    167.21396    159.22569    154.28031    155.74215    154.00381    146.86274          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       75.00    323.23631    312.99414    293.97589    296.57009    287.55239    276.80294    267.87619    245.49284    239.85486    227.47905     219.0596    215.52679    211.76359    214.63744    205.18641    209.62441     205.8598    203.32687    199.64269     203.6711    203.18375    198.31208    202.05535    196.26061    191.93341    203.45308    202.76035    204.58141    199.03085    200.41069    191.25997    195.95801    186.87076     174.8013    174.51399    177.69939    178.19327    174.98033    176.69365    172.68743    178.42793     176.8816     176.9259     171.6671    174.30887    174.90642    171.03554    173.88778    171.34837    178.25442    176.80307    166.58266    168.93008    169.63707    172.00663    177.12961    172.51534    179.71759    176.75863    177.12237    176.67341     171.8038    171.43627    171.78882    163.80698    164.95087    163.61335    169.41008    176.62689    174.99826    183.23376    181.97399    194.12782    196.94074    209.19249    201.34137    188.94519    178.97448    172.55874    163.94583     165.0495    154.88034    154.80436    140.85328          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       75.50     324.9223    310.47867    288.78335    294.63717    286.19641    277.80019    265.51313    248.17466    244.29962    232.04458    218.90744    211.76348    209.65177    211.35809    210.30688    211.27174    205.32073     198.3465    202.62922    202.60786     199.8307    185.04622    199.27793    204.10637    184.51223    201.89784    202.88319    191.63139    198.88316    200.20208    196.70187    196.40659    189.38899    184.19238    181.15763    177.71361    179.15744    176.77604    173.03264     181.7433    177.81015    170.66152    170.84295    177.43086    177.92199    179.92928     182.3117    174.13635      168.952    170.94232    178.25492    174.43174    173.87045    173.76859    180.42823    169.45119    176.49612    182.37719    177.03742    173.68471    169.91004    171.42576    170.93307    172.61094    162.21491    166.85329    167.97956    166.84232    168.20844     168.0648    174.26992    185.12839    195.38911    203.24841    202.38103      198.705    195.63678    183.80482    171.02629     171.0808    159.35129    155.12548    151.46026    135.71621          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       76.00    325.54869    310.33117    287.22503    294.16873    282.77362    271.99991    262.34504    248.93764    248.38763    235.80358    220.35577    217.53864    207.51167     207.0451    212.10273    208.93624    201.91959     197.5218    200.48316    197.04608    196.15849    192.25241     196.8113    198.23186    183.68717    196.28745    200.21798    204.64077    201.18127    202.75212    202.41596    187.47693    184.99786    194.71904     186.7776    190.05928     185.6197    185.95033    178.50132    173.55156     179.5471    168.17365    170.31982     173.1807    170.25215    181.13667    172.41172    168.35202    174.02537    169.31987    180.66857    177.91335    174.22235    174.12623    167.70068    172.55289    172.83258    178.85868    188.28583    183.46121    181.70046    182.40755    170.00835    164.93117    165.13349    162.03518    171.41672      162.412    171.20851    171.77432     179.6451    187.32974    194.22797    196.70913    197.92483    198.02466    187.60976    185.28656    172.95024    163.09577    166.59658     155.7708     154.5036          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       76.50    324.66676    310.59974    293.48721     292.7635    278.44239     267.1387    262.77797    250.79686    250.21877    237.20831    232.81709    222.22762    214.23856    208.75867    208.49475    203.71298    200.49542    200.18625    202.85151    195.33577     196.0955    194.35657    200.40799    192.25908     194.3672    197.80698    199.48049    202.78982    202.67983    203.49844    198.37439     187.6193    181.66779    195.34209    189.75706    191.12913    185.04568    188.53385    181.01698    175.80148    178.15383    179.99163    174.27049    178.61319    172.59501    175.74888    177.28513    175.13702    170.38859    175.93573    177.59933    168.17325    174.98043    162.47424    174.64136    171.82128    179.21379    180.72445    183.11369    183.86698    182.05402     180.7826    179.79378    169.44028    168.12343    165.69001    166.26772    163.33376    167.00006    173.10888    180.06739     180.4239    191.66464    193.17651    210.14309    201.57494    191.48584     177.3474    167.55731    163.95743    159.61978    160.09518    146.49336          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       77.00    324.54733    310.62426    298.14797    288.26919    275.75128    268.92906     262.8817    252.60045    250.04817    237.12485    241.32419    223.19966    220.22771      214.321    208.36785    200.20898    201.89777    205.54321    207.21024    195.10066    193.16543    192.72477    200.47177    194.17051    199.82149    197.92088    194.15794    203.48259    201.79982    201.66191    199.28169    186.13549    194.24729    186.48245     186.3656    190.41284    182.17724    187.52795    184.02463    184.30681    182.56009     182.1212    174.27446    182.41582    173.94399    172.78202    173.76549    172.89342    173.12083    176.57674    172.46923    171.47501     168.0059    170.63173    181.15734    178.73553    173.42492     186.8465     185.2632    179.13555     169.3165    184.97773    175.50848    173.87854    171.08052    173.70933    171.66374    163.64163    169.56473    171.79791    172.63105    189.01779    191.45653    198.49293     199.6255    201.50241     200.8982    182.32164    178.05225    162.92234    164.98003    155.91891    128.42255          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       77.50    325.49507    310.02063     299.8838    284.47141    276.76668    269.21458    266.09549    253.42841    242.97942    237.60071    236.44256    225.14846    221.41094    219.05313    209.70913    201.76046     204.1739    203.95761    207.45079    195.57546    189.82833    194.28511    200.29438    204.51264    196.92572    197.22046    197.96606    210.71203    200.14074    200.63346    194.40191    187.30281    192.42267    181.85437    187.32645    195.68629    180.45415    182.00239    179.54667    179.02406    185.84513    186.11542    181.73005    183.05679    172.63805     174.4719    170.04058    174.46952    172.65213    169.84104    178.16931    174.88169    170.64683    174.71865    172.69205    174.61361    181.45216     182.7871     182.2547      175.556    181.32041    185.44929    170.60767    176.09931    174.14515    175.75144    172.31694    170.71385    174.82717     180.9707    173.05533    186.20691    200.79375    198.21317     201.9255    195.82416    196.27892    188.02482    176.70396    164.41124     158.4028    158.04041    142.85072          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       78.00    327.80045    308.92456     300.6537     283.2535    278.62757    269.76443    270.75866    256.12264    236.77557    239.30933    233.03859    228.14485    222.67993    222.94916    214.88758     204.5243    204.48926    201.81605    200.06206    198.18417    188.76977    206.16879    200.88323    196.37643    194.84245    201.47875    204.26485    203.99947    202.06392    197.23608    193.79237    185.19203    194.38361    185.00136    186.90367    190.90595    183.49239    176.98077    176.82561    181.67061    181.91221    186.00314    190.30129    192.29067    173.02394    176.63001     172.5858    172.19245    168.98406    173.50923    177.08384    166.43987    168.05543    168.50209    169.04443    181.05642     180.1634    177.99344    178.10943    182.26593    180.20896     183.0052    177.36337    175.06578    170.88198    177.69402    168.38305    170.98329    181.08943    172.04528    185.39362    188.93673    206.38667     203.5434     200.5649    196.62351     185.3414    176.73009    170.64033    165.61042    165.43772    162.11815    140.73207          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       78.50    328.94643    307.45796    302.51132    284.48898    279.88619    270.55101    268.15217    260.06582    240.04638    239.71169    231.80141    229.40179    223.95052    224.80019    218.90628    215.61169    206.96484    201.56178    202.06256    201.59071    188.30452    209.66834    189.34434    187.51357    194.91382    196.66143    209.56834    198.65794    201.73697    191.98085    195.17831    187.82559     194.1886    185.88778    191.00996    182.03703    179.37876    182.85693     180.4103    182.17812    190.50973    180.85621    185.63799    188.72142    175.33007    180.02627     181.6575    174.20417    178.35842    178.90447    175.36268    170.08321    171.01587    169.00813    166.73392    170.32955    174.33803    178.62664    182.54163    186.47565    189.67158    180.38443     173.7303       175.04    171.03829    176.59809    169.64953    167.57716     176.6733    174.66737    184.04489    192.75079    204.09018    205.08607    207.28385    200.27435    196.48155    190.82379      176.089     169.6393    164.03351    152.95199          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       79.00    328.00709    305.37103    300.71815    284.70937    279.49536    269.33653    262.51467     256.7768    245.32472    234.77345    230.22879    230.39505    227.42784    225.70802    215.64503    220.36937     215.4906    208.99358    205.32919    202.19315    192.72165    195.57146    194.16429    186.37531    200.29384    190.28476    201.18525      201.484    200.10668    195.46313    196.16511    192.01681    193.46077    190.71833    181.41205     180.9369    194.21335    186.66285    179.56231    178.52647    187.68884    187.15591    176.97005    182.59881    175.06959     181.2677    180.87903    186.18482    180.67252     181.8134    176.52999     168.9114    175.08439     168.1937    163.49017    171.99193    174.47647    181.22644    182.27911    185.51878    178.88856     187.5409    174.24773    174.74733    173.91988    165.75855    170.08974    167.75239    180.62906    169.07218    178.95479    190.13342    193.82478    199.49302    203.52251     204.1483    197.95777    192.42733     180.5974    171.39508    164.97074    154.96481          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       79.50    324.02072     303.7514    297.40083     284.2089    278.53887     264.6529    258.97638    252.12702    248.20222    231.47929    227.76662    226.68215     232.1082    225.69394    216.88034    219.05157    217.00171    214.04683    209.06644     207.4751    196.51641    198.31898    193.06969    190.45492    195.03433    190.90447    200.16683    205.54736    201.04756    192.22555    193.08104      198.701    191.45549    193.98547    187.93034    185.56949    187.55228    190.42045    182.87569    188.48237    184.19997    186.94769    180.86465    188.53044    183.55974    180.28692    174.73581    181.98671    181.57525    182.70245    180.72328    177.13226     176.4806    182.57499    176.31601    170.76765    174.55847    177.32058    183.08987    183.49071    183.35343    181.25244    171.83765    169.95069    172.63224    165.41126    164.40183    164.83112    164.88821    178.19105    187.45435    187.53742    199.97463    205.87026    200.33408    203.56194    198.16245    184.48429    177.35061    173.68219    171.65662    157.59416          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       80.00    320.82897    307.36308    294.18079    285.23967    275.55223    259.51582    258.47123    253.77523    248.70221    235.49712    225.46429    221.80347    227.44332    222.34689    219.60261    214.46789    213.52028    215.55155    210.36693    211.46002    210.13439    205.65336    189.31901    194.70674    191.81867    193.57565    206.71635     205.5564    198.82497    194.56316    192.78483    198.89919    187.48401    183.40232    185.73056    189.70626    178.60047    189.81153    180.17793    192.42205    191.84604    177.21811    180.88852    183.59154    188.47938    173.89282    177.83211    182.12382    181.60328    182.46737    172.65762    176.66197    180.15034    185.10972    179.82156    186.44001    175.14835    180.91404    183.12167    182.11804    186.97767    181.45644    177.70747    166.17739    169.71602    172.39217    174.25164    169.62832     172.3461    173.24698    176.14564    188.97675    198.82351    200.81586    210.69226    205.04416      199.035    189.07428    174.60527    176.20734    164.87934    155.67558          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
;

_pd2d_meas_2theta_phi_intensity_down
;
         241   4.00         4.20         4.40         4.60         4.80         5.00         5.20         5.40         5.60         5.80         6.00         6.20         6.40         6.60         6.80         7.00         7.20         7.40         7.60         7.80         8.00         8.20         8.40         8.60         8.80         9.00         9.20         9.40         9.60         9.80        10.00        10.20        10.40        10.60        10.80        11.00        11.20        11.40        11.60        11.80        12.00        12.20        12.40        12.60        12.80        13.00        13.20        13.40        13.60        13.80        14.00        14.20        14.40        14.60        14.80        15.00        15.20        15.40        15.60        15.80        16.00        16.20        16.40        16.60        16.80        17.00        17.20        17.40        17.60        17.80        18.00        18.20        18.40        18.60        18.80        19.00        19.20        19.40        19.60        19.80        20.00        20.20        20.40        20.60        20.80        21.00        21.20        21.40        21.60        21.80        22.00        22.20        22.40        22.60        22.80        23.00        23.20        23.40        23.60        23.80        24.00        24.20        24.40        24.60        24.80        25.00        25.20        25.40        25.60        25.80        26.00        26.20        26.40        26.60        26.80        27.00        27.20        27.40        27.60        27.80        28.00        28.20        28.40        28.60        28.80        29.00        29.20        29.40        29.60        29.80        30.00        30.20        30.40        30.60        30.80        31.00        31.20        31.40        31.60        31.80        32.00        32.20        32.40        32.60        32.80        33.00        33.20        33.40        33.60        33.80        34.00        34.20        34.40        34.60        34.80        35.00        35.20        35.40        35.60        35.80        36.00        36.20        36.40        36.60        36.80        37.00        37.20        37.40        37.60        37.80        38.00        38.20        38.40        38.60        38.80        39.00        39.20        39.40        39.60        39.80        40.00        40.20        40.40        40.60        40.80        41.00        41.20        41.40        41.60        41.80        42.00        42.20        42.40        42.60        42.80        43.00        43.20        43.40        43.60        43.80        44.00        44.20        44.40        44.60        44.80        45.00        45.20        45.40        45.60        45.80        46.00        46.20        46.40        46.60        46.80        47.00        47.20        47.40        47.60        47.80        48.00        48.20        48.40        48.60        48.80        49.00        49.20        49.40        49.60        49.80        50.00        50.20        50.40        50.60        50.80        51.00        51.20        51.40        51.60        51.80        52.00        52.20        52.40        52.60        52.80        53.00        53.20        53.40        53.60        53.80        54.00        54.20        54.40        54.60        54.80        55.00        55.20        55.40        55.60        55.80        56.00        56.20        56.40        56.60        56.80        57.00        57.20        57.40        57.60        57.80        58.00        58.20        58.40        58.60        58.80        59.00        59.20        59.40        59.60        59.80        60.00        60.20        60.40        60.60        60.80        61.00        61.20        61.40        61.60        61.80        62.00        62.20        62.40        62.60        62.80        63.00        63.20        63.40        63.60        63.80        64.00        64.20        64.40        64.60        64.80        65.00        65.20        65.40        65.60        65.80        66.00        66.20        66.40        66.60        66.80        67.00        67.20        67.40        67.60        67.80        68.00        68.20        68.40        68.60        68.80        69.00        69.20        69.40        69.60        69.80        70.00        70.20        70.40        70.60        70.80        71.00        71.20        71.40        71.60        71.80        72.00        72.20        72.40        72.60        72.80        73.00        73.20        73.40        73.60        73.80        74.00        74.20        74.40        74.60        74.80        75.00        75.20        75.40        75.60        75.80        76.00        76.20        76.40        76.60        76.80        77.00        77.20        77.40        77.60        77.80        78.00        78.20        78.40        78.60        78.80        79.00        79.20        79.40        79.60        79.80        80.00        80.20        80.40        80.60        80.80        81.00        81.20        81.40        81.60        81.80        82.00        82.20        82.40        82.60        82.80        83.00        83.20        83.40        83.60        83.80        84.00        84.20        84.40        84.60        84.80        85.00        85.20        85.40        85.60        85.80        86.00        86.20        86.40        86.60        86.80        87.00        87.20        87.40        87.60        87.80        88.00        88.20        88.40        88.60        88.80        89.00        89.20        89.40        89.60        89.80        90.00        90.20        90.40        90.60        90.80        91.00        91.20        91.40        91.60        91.80        92.00        92.20        92.40        92.60        92.80        93.00        93.20        93.40        93.60        93.80        94.00        94.20        94.40        94.60        94.80        95.00        95.20        95.40        95.60        95.80        96.00        96.20        96.40        96.60        96.80        97.00        97.20        97.40        97.60        97.80        98.00        98.20        98.40        98.60        98.80        99.00        99.20        99.40        99.60        99.80       100.00       100.20       100.40       100.60       100.80       101.00       101.20       101.40       101.60       101.80       102.00       102.20       102.40       102.60       102.80       103.00       103.20       103.40       103.60       103.80       104.00       104.20       104.40       104.60       104.80       105.00       105.20       105.40       105.60       105.80       106.00       106.20       106.40       106.60       106.80       107.00       107.20       107.40       107.60       107.80       108.00       108.20       108.40       108.60       108.80       109.00       109.20       109.40       109.60       109.80       110.00       110.20       110.40       110.60       110.80       111.00       111.20       111.40       111.60       111.80       112.00       112.20       112.40       112.60       112.80       113.00       113.20       113.40       113.60       113.80       114.00       114.20       114.40       114.60       114.80       115.00       115.20       115.40       115.60       115.80       116.00       116.20       116.40       116.60       116.80       117.00       117.20       117.40       117.60       117.80       118.00       118.20       118.40       118.60       118.80       119.00       119.20       119.40       119.60       119.80       120.00       120.20       120.40       120.60       120.80       121.00       121.20       121.40       121.60       121.80       122.00       122.20       122.40       122.60       122.80       123.00       123.20       123.40       123.60       123.80       124.00       124.20       124.40       124.60       124.80       125.00       125.20       125.40       125.60       125.80       126.00       126.20       126.40       126.60       126.80       127.00       127.20       127.40       127.60       127.80       128.00       128.20       128.40       128.60       128.80       129.00       129.20       129.40       129.60       129.80       130.00       130.20       130.40       130.60       130.80       131.00       131.20       131.40       131.60       131.80       132.00       132.20       132.40       132.60       132.80       133.00       133.20       133.40       133.60       133.80       134.00       134.20       134.40       134.60       134.80       135.00       135.20       135.40       135.60       135.80       136.00       136.20       136.40       136.60       136.80       137.00       137.20       137.40       137.60       137.80       138.00       138.20       138.40       138.60       138.80       139.00       139.20       139.40       139.60       139.80       140.00       140.20       140.40       140.60       140.80       141.00       141.20       141.40       141.60       141.80       142.00       142.20       142.40       142.60       142.80       143.00       143.20       143.40       143.60       143.80       144.00       144.20       144.40       144.60       144.80       145.00       145.20       145.40       145.60       145.80       146.00       146.20       146.40       146.60       146.80       147.00       147.20       147.40       147.60       147.80       148.00       148.20       148.40       148.60       148.80       149.00       149.20       149.40       149.60       149.80       150.00      
      -40.00   -163.92064   -171.98843   -391.59225    -602.0436   -140.53683   -518.17176   -494.75502   -216.80347    -400.3087   -398.34026   -463.45687    -447.0876   -673.06886    -384.5061   -491.00764   -421.39958   -556.08259          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -39.50   -316.22822   -202.02991   -366.83589    -413.4298   -346.72452   -587.49985   -530.86687    -91.11847   -409.66797   -368.99386   -351.34804   -430.61883   -732.73031   -615.48372    -527.2191   -585.03227   -462.47065          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -39.00    -461.0293   -263.23299   -370.99398   -277.48271   -580.49673   -549.84361   -534.41183   -140.03408      -369.74   -460.15298   -223.89481   -491.83418     -586.155   -595.05662    -655.8996   -587.36001   -576.38921   -480.03682          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -38.50   -531.28623   -344.09235   -430.64712   -432.55897   -786.96433   -513.23008   -555.44832   -366.45551   -286.63358   -545.73321   -344.76033   -492.92934   -461.80515   -587.73658   -690.49832   -542.20095   -573.75031   -417.69431          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -38.00   -537.84783   -358.53104   -537.56293   -497.86728   -624.70648   -529.85706   -477.69317   -633.38509   -315.01758   -560.45895   -427.24078   -432.34366   -526.15556     -631.759   -814.40081   -631.71591   -531.37105   -500.22165   -524.78651          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -37.50   -545.60798   -306.93369   -631.16482   -472.25341   -479.98787   -611.48426   -384.01766   -730.59108   -376.36884   -420.18043   -571.36258   -538.52193   -576.16709   -576.67755   -712.03547    -590.3581   -608.08279   -564.76186   -378.24569          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -37.00   -459.65618   -234.48696   -642.45136   -346.68287   -476.35513   -677.48981   -354.07032   -641.33749   -361.61061   -299.83529   -652.09204     -575.488   -399.91452   -513.80966   -585.45812   -393.28724   -712.15885   -507.14226   -406.35642          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -36.50   -389.63995   -175.28093   -590.25315   -277.63222    -519.6672   -683.22353   -434.39653   -728.26555   -325.69316   -327.57863   -613.78161   -502.23666   -481.23138   -484.75561   -557.24447   -512.04251   -759.89088   -619.21186   -517.38266   -700.36915          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -36.00   -402.99791    -217.9224   -450.57223   -193.52424   -474.61974   -730.31754    -436.4404   -760.90473   -414.98836   -436.40151   -572.46061   -627.44736   -668.80608   -331.28088   -496.05324   -658.43915    -818.5326   -493.63162   -461.96398   -489.09208          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -35.50   -490.35351   -198.93627   -291.82409   -179.97939   -409.18634   -859.47077   -270.13219   -487.85744   -558.12539   -510.47743   -580.50203   -757.06199   -683.86738   -387.40897   -435.98327   -546.36827   -934.99873   -379.46239   -522.05858   -344.72336   -722.56379          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -35.00    -583.8448   -116.63252    -246.0377   -156.17364   -362.13263   -568.39734      -58.378    -227.5778   -580.89802   -557.02075   -555.14958   -619.78109   -609.37176    -469.7732   -612.82093   -501.03845   -616.29506     -419.784     -486.725   -383.28481   -580.91389          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -34.50   -504.14012   -116.04107   -297.12623   -118.13585   -360.73515   -386.93454   -236.02325   -338.61424   -532.03778    -576.4838   -435.28248   -462.98379   -500.87965   -477.95878   -553.81355   -533.22788   -452.78824   -400.68598   -233.81413   -655.24244   -524.07324   -677.52859          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -34.00    -534.9402   -234.72866   -432.50782   -202.10217   -447.64644   -374.77842   -406.13721   -491.19078   -468.34775   -524.48643   -463.63793   -422.05586   -513.34406   -499.74735   -453.14109   -424.76139   -338.44212   -546.48407   -364.77989   -695.89646   -489.90367    -665.3629          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -33.50   -675.10547   -301.15591    -541.2475   -326.19593   -563.11324   -369.69621   -410.27992    -473.8014   -440.72085   -508.82448    -583.6511   -328.80013   -566.20124    -444.8857   -403.75277   -369.83514     -348.291   -789.71155   -409.40971   -637.96302   -510.33578   -638.83064   -358.41129          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -33.00   -712.18747   -270.26061   -565.49293   -448.13055   -575.40889   -389.71135   -317.55455   -356.95854   -471.27031   -552.13697   -581.48895   -258.07588   -632.20399   -413.22571   -439.90081   -357.49911   -359.57971   -645.98558   -427.05203   -683.67543   -611.48859   -693.46901   -346.01572   -658.73054          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -32.50   -610.53951   -188.02334   -573.37985   -299.80418   -412.37323   -415.91168   -226.27157    -356.8118   -461.91954   -492.80829   -427.36788   -243.17775   -597.44615   -429.28318   -481.87162   -403.75306   -332.21852    -512.6671   -422.73892   -525.08115   -645.68952   -626.29242   -528.20035   -616.72226          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -32.00   -450.09624   -124.38904    -591.0728   -192.33166   -220.36781   -379.02466   -347.21807    -528.2857   -374.50136   -403.39422   -356.36244   -196.38393   -523.66321   -577.21581   -300.13957   -333.56635   -423.39262   -460.84017   -621.47157   -613.51896   -629.49728   -427.25677   -494.10101    -554.9578   -381.57406          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -31.50   -361.39613   -139.23175   -618.45167   -262.24899   -196.40466   -321.13072    -451.9616   -584.18266   -276.73527   -368.67489   -453.64014   -260.57455   -450.92468   -638.68695    -267.3649   -401.60906    -501.1575   -491.95403   -687.15196   -722.28174   -495.87266   -367.64857   -557.83779   -598.73682   -573.83427          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -31.00    -311.5154   -193.55966   -502.00945   -508.75376   -295.36954   -354.15005   -514.21973   -531.60853   -286.98311   -318.71356   -517.60472   -181.71796   -429.93972   -668.53802   -356.14472   -615.32413   -560.67465   -552.47655   -636.03075   -713.56997   -495.21162   -417.81771   -640.16934   -553.71046   -424.72984   -334.81697          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -30.50   -276.14803   -209.76121   -404.53754    -687.8978    -347.4547   -395.15796   -546.12254    -474.8293   -419.27951   -306.94429   -337.38306   -365.02602   -408.89222   -688.41652   -490.87409     -585.469    -471.5887   -608.33276   -543.39966   -503.24108   -552.47214   -455.26261   -598.30391   -460.43321   -345.41699   -275.40534      28.8127          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -30.00   -256.22976   -188.36229   -350.97816   -731.61281   -307.70304   -365.05308   -537.30777   -477.72916   -542.38165   -364.32704   -322.84865   -696.36897   -389.61858   -643.40494   -390.42295   -460.15161   -277.71252   -633.36289   -593.07674   -459.14105   -496.40103   -574.22598   -582.07811   -190.33197   -345.58109   -262.97762     73.67832          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -29.50   -392.88167   -278.07261   -370.52415   -588.82695   -418.40048   -300.37827   -503.64271   -345.42723   -557.58355   -455.10085   -496.76729   -578.34264   -499.14691   -499.50215   -464.14103   -464.16819   -327.06723   -455.90887   -614.85695   -517.77604   -565.00582    -592.1393   -576.47422    -496.5248    -347.2094   -155.53968     28.91056    -16.97099          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -29.00   -526.45502   -340.60246   -357.54293   -461.91845    -526.5118   -254.79076   -386.05436   -308.16275   -490.12174   -476.08777   -410.29828   -494.29344   -577.72635   -409.95769   -678.65577    -545.1516    -566.3498   -296.94568   -555.82831   -516.15946   -510.97344   -524.45694   -613.68764   -577.98894    -399.0213   -159.63344    261.06611    -40.70137   -327.26722          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -28.50   -649.79361   -381.18759   -311.70545   -379.83542   -548.30995   -322.36719    -237.4125   -357.89736   -403.20729   -351.36711   -321.84465   -427.49181   -520.30048   -440.61151   -708.17379   -550.30479    -691.5117   -439.19951   -469.23231   -483.96648   -520.76288   -451.72939     -576.647   -318.70119   -365.87006   -107.17876    269.67667     81.63592   -229.45057    -484.2216          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -28.00   -577.07596   -388.91441   -325.32363   -476.67419   -487.00832   -414.88189    -312.5483   -366.92382   -420.89284   -266.30636   -471.72482   -396.80093    -486.3065   -391.22284   -640.38429   -467.82566   -839.74262   -393.36841   -366.07755    -512.8404   -658.19597   -526.34872   -516.49516   -453.85754   -430.22502   -177.76494    216.30121    175.08044   -142.93867   -469.13216   -424.09953          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -27.50    -421.1588   -401.76847   -313.86995   -563.77398   -419.45718   -466.72028   -408.80771   -254.55659    -485.0106   -324.85014   -507.87664   -443.95974   -345.34988    -414.4988   -566.34178   -354.52028   -603.96678   -428.41378    -497.1268   -494.34861   -738.21948   -666.46528   -516.92626   -574.96627   -433.46253   -126.63224    251.98957    174.09846    -99.84776   -396.07246   -611.30764          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -27.00    -243.4505    -422.7407   -271.84009   -531.37354   -442.60756   -348.16865   -488.00751   -330.94569   -283.31893   -498.23849   -338.40756   -485.42891    -257.5877   -478.01233   -573.21466   -385.26495   -542.05796   -509.89539   -386.94118   -457.52179   -700.60307   -624.01191   -461.51757   -746.95845   -504.80111     54.04659    338.36681     223.6526   -112.08664   -429.92678    -571.6885   -520.81826          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -26.50    -203.9618   -476.55162    -248.3453   -424.26114   -567.09436   -125.64445   -480.33374   -584.18964    -127.3315   -548.01042   -213.60468   -410.76041    -378.4648     -461.231   -465.83803   -505.13146   -559.16207   -676.48885   -286.77774   -456.41912   -682.76673   -603.93292   -468.43343    -473.1325   -425.22778     18.55006    461.92816    364.10311    -91.20223   -280.25133   -494.08332   -613.16348   -495.66883          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -26.00   -287.54708   -539.00871   -262.84776   -373.01657   -663.09819   -116.52095   -366.20537   -609.53245   -249.88514   -514.87785   -356.99681   -130.14304   -525.25317   -452.04291   -406.58804   -412.52807   -549.98941   -569.14749    -394.4258   -477.61535   -468.44351   -373.16827   -335.84368   -420.61491   -329.68457    121.22281    377.88862    288.17977    -65.44042   -272.98555   -480.72927   -610.09198     -298.304   -625.95831          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -25.50   -358.32351   -551.83689   -333.71298   -426.67352   -575.43194   -355.89968   -239.87188    -424.3375    -322.9265   -503.40693   -420.91914   -216.49875   -508.29761   -425.50644   -422.99384   -464.40475   -571.38124   -395.07751   -372.00512   -412.20914   -526.99665   -447.06778   -501.09441   -349.15741   -243.40652     97.08769    358.69643    343.05132     29.20025   -229.28319   -472.86563   -299.50796    -472.7451   -479.90866   -520.65701          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -25.00   -326.72329   -406.43261   -484.42444   -526.24656   -493.96847   -517.74923   -139.39166   -415.36435   -330.60219   -459.89957   -483.49855   -440.92317   -464.67772   -376.22338   -448.08317   -577.36851    -456.2911   -305.95617   -443.94413   -283.52721   -561.84794   -438.30883   -355.11419   -313.92212   -436.29902     -4.55528    362.59868    290.05962     -3.14861   -112.83776   -423.59248   -447.62589   -519.47928    -344.2052    -447.4537   -600.76094          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -24.50    -70.79742   -188.27793   -620.59669   -568.54662   -465.01099   -492.78843   -294.88309   -562.15794   -365.24137    -483.4836   -539.62345   -566.25534   -427.56689   -271.39678   -621.62609   -703.82224   -334.54304   -375.35725   -530.31554   -451.15499   -343.06199   -432.91297   -260.37681   -438.26057   -301.02986   -108.84902    508.11207    359.47896    -85.69207   -129.56333   -405.24836   -458.98468    -391.2453   -376.29813   -540.43638   -695.90901   -475.36231          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -24.00    284.16281     67.97253    -700.0187   -502.39639   -486.19429   -360.92194   -366.26213   -593.55261    -404.6493   -497.80396   -458.42344   -600.75758   -221.97617   -116.38779   -576.58394   -621.17621   -473.63547   -580.00515   -626.39406   -448.50746   -503.51442   -538.70184   -369.13533   -525.29524   -157.59255   -152.37486    519.09572     414.8274    314.11702   -264.78466    -402.7022   -488.60359   -396.01548   -746.59967   -777.83509   -541.68288   -384.88592   -597.15778          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -23.50    566.83345      22.5898   -696.29634   -396.14863   -523.60433   -243.23018   -275.47665   -413.07871   -482.47446   -518.72341   -417.85157   -603.75414   -290.84541   -239.35236   -359.56283   -611.92998   -520.34585   -492.02094   -541.00791   -268.47239   -620.73648   -572.91579   -526.08052   -477.65259   -355.14193     -2.77228    595.05542    467.23929    209.75008   -409.98087   -470.59696   -715.77723   -349.95244   -606.01189   -524.66499   -606.99223   -507.75165    -691.7138   -495.45811   -550.81932          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -23.00    182.15356   -150.15585   -693.45229   -285.34638   -478.14823   -241.00252   -244.61181   -217.31482   -575.25609    -545.8911    -442.7183   -602.79429   -521.57968   -365.88795   -344.06132    -514.6899   -339.09401   -317.45465   -469.02312   -549.78043    -455.8158   -624.72494   -708.09598   -500.94162   -311.19414    -37.51689    587.37573    697.29026    323.27518   -128.00182   -592.31832   -650.01351   -432.19119   -488.21804   -579.19715   -753.23625   -543.03831   -627.68381    -540.7852   -739.80443   -474.56996          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -22.50   -159.85374   -375.27238   -708.97706   -294.31837   -371.36715   -227.16326   -254.40649   -348.28272   -537.90308   -527.16342   -426.57004   -524.26075   -495.73541   -329.41537     -425.941   -446.76523   -278.40403   -470.81841   -496.07962   -636.36429   -419.72086   -355.74927   -726.23901   -321.86892   -213.51641      8.02722    721.14158    708.90565     451.6932   -248.13775   -424.80487   -444.67158   -441.66426   -484.46097   -493.85395   -409.71913   -531.29206   -602.26318   -578.79788   -535.95918   -610.84385   -596.45143          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -22.00   -354.22868   -278.69012   -727.96715   -338.01218   -258.86372   -213.16559   -399.23459   -509.01771   -444.18717   -435.76349   -438.83531   -464.46593   -535.59365   -335.12368   -503.43116   -328.72196   -291.95909   -516.87693   -529.43688   -517.10734    -511.4828   -360.89962   -645.13759   -354.68931    -229.4102    144.41681    700.79707    725.07424    365.50315   -284.67833   -305.69598   -446.73215   -509.89477   -462.26158   -471.03858   -516.10454   -487.53548   -653.95167   -509.55959   -698.79115   -715.14134   -509.92383   -727.44869          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -21.50   -263.08456   -218.49778   -690.80229   -362.77248   -222.90092   -207.29111   -546.36362   -513.14613   -433.97334   -322.07322   -466.54525   -334.12325   -442.86433   -376.10041   -597.14209   -301.74443   -387.18566   -449.74923   -451.18633   -415.45466   -439.76966   -411.41554    -496.7936   -515.56132   -215.60253     89.49238    673.71466    754.18406    365.42088   -165.86208   -328.34471   -537.47272   -448.08215    -605.5713   -650.94842   -707.75014   -619.47635   -414.83137   -581.42244   -702.90091   -788.67539   -588.59963   -771.06685   -697.75404    -575.2966          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -21.00   -131.21239   -237.73554    -633.7603   -351.99572    -248.1183   -230.01413   -466.71573   -392.18606   -398.51562   -322.33417   -458.99557   -163.59494   -242.41125   -427.86439   -677.98443   -390.07554   -552.99075   -453.52695   -465.58074   -327.36915   -423.99768   -400.52441   -392.82542   -508.97573   -319.21513    283.57446    569.04264    857.81355     546.0181    -88.33262   -318.56636   -566.96149   -527.34376   -512.26166   -599.99341   -686.85459   -327.21564   -357.82798   -697.84028    -540.9242   -563.10949   -693.61891   -697.24046   -605.15387   -421.90111   -529.09391          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -20.50     20.29539   -401.15992   -573.34764   -366.56889   -339.70776   -288.68388   -411.67853   -306.09676   -437.16849   -366.06466   -394.37541   -247.30152   -374.59607   -560.70961   -641.72705   -450.89385   -653.40612   -525.86601   -446.05602   -558.40965   -541.72006   -449.91404   -461.73384   -418.70668   -420.96329    292.45256     669.0865    925.65555    592.59898     54.73936   -351.25868   -608.75844   -569.37579   -454.94632   -369.92261   -521.84154   -502.53652   -412.80816   -690.10825   -405.65384   -448.05095   -612.13794    -581.3637    -702.3708   -339.88422   -541.08043   -473.26577   -627.33299          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -20.00     -16.1695   -565.00451   -510.36253   -410.66754   -308.17992   -363.93572   -332.37094   -378.89247   -507.14821   -291.82165   -432.84382   -391.48412   -420.69485   -540.78645   -557.33619   -447.00454   -596.67802   -466.74666   -460.33645   -565.51122    -367.2547   -527.58262   -490.55435   -464.57621   -173.02315    104.78736     803.4128    922.87698    459.42824    140.11027   -411.32153   -590.47184   -490.65229     -509.633   -557.81029   -503.09615   -648.97674   -421.55976   -449.72142   -523.62072   -506.06482   -551.36699   -470.54484   -519.04225   -477.57524   -586.23738   -825.14583   -487.69967   -610.33481   -612.18663          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -19.50   -112.71208   -717.92079   -475.07138   -414.46971   -164.51656   -396.76623   -312.11824    -479.6311    -657.6735   -155.14591   -589.53148   -405.97197   -396.22282   -514.74981   -471.86166   -478.06405   -395.61013   -412.52888   -657.20495   -541.84168   -301.60742   -624.89698    -424.8414   -404.05494   -235.21462   -108.10411    827.06693   1172.03538    483.79449      4.61358   -520.54484   -415.03956   -450.00682   -576.59531   -356.29483   -454.59152   -615.71417   -468.96017   -463.37009   -551.69216   -496.53413   -489.97956   -412.78856   -337.55509   -460.00122   -529.73398     -495.226   -453.05043    -485.4084   -616.21989   -635.40107          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -19.00   -241.40165   -771.50993   -510.65332   -365.41591    -307.4292   -397.26884   -338.24671   -389.48285   -773.99282    -185.8118   -326.75394   -446.60027   -386.88767   -570.41996   -445.20555   -507.08312   -403.64259   -374.87983   -643.76138   -548.87112   -350.98979   -712.62831   -559.98455   -250.18704    -319.2532      22.6921    682.65071   1096.38794     819.8016   -210.30501   -509.11114   -404.94497   -408.68652   -575.74557   -580.56452   -453.66034   -455.92448   -425.15257   -460.99862   -391.33495   -548.22906   -508.26645   -511.13927   -490.21884    -663.8638   -452.06007   -578.05162   -563.71684   -580.04774   -703.10563   -790.61681   -593.57463   -538.28892          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -18.50   -292.48452   -724.48781   -566.43408   -278.61795   -506.87893    -372.2158   -381.55333   -368.40557   -689.00115   -392.67324    -47.17105    -508.2448   -408.87508   -588.40782   -389.01573   -518.97719   -442.62936   -431.63307   -482.33247   -496.36812   -480.84514   -600.25373   -631.95674   -410.15359   -401.05603    193.28068    975.68172    877.27574    924.06332     85.61873   -302.42991    -347.2738   -255.96225   -461.20819    -652.0226   -539.91161   -476.80234   -568.04223   -508.94685   -525.86088   -351.64115   -547.04501   -348.76056   -615.00982   -652.11787   -599.80295   -532.48227   -656.08779   -579.61188   -609.23941   -686.80778   -720.29867   -785.76121   -635.47674   -749.44882          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -18.00   -353.78118   -667.03935   -458.21266   -222.77118   -606.72453   -352.53285   -395.64813   -537.62754   -498.37868   -551.91662   -196.45768   -489.33137   -453.19883   -531.72142   -271.63686   -523.69644   -621.17914   -492.21327   -531.16161   -390.99345   -467.69721   -650.24394   -537.01222    -457.3178   -350.88109    400.98007    978.23763   1054.13349    773.13429     58.96246   -278.57108   -427.72682   -446.20391   -436.96656   -449.04471   -437.03145   -448.27531    -559.6346   -467.76388   -558.52638   -311.48576    -583.0303   -462.12177   -560.88402   -606.87893   -513.70898   -600.81451    -752.0145   -474.76677    -672.7194   -527.96183   -654.34375   -494.59429   -591.41404   -673.77258   -273.66559    -232.8239          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -17.50   -422.59019   -557.77945   -359.19973   -311.06681   -518.58141   -358.35331   -358.43244   -690.83508   -397.39286   -593.50236    -346.4157   -408.76734   -460.79442    -373.4455   -293.78034   -461.95682   -557.85294   -367.29801   -596.65921   -428.87968   -303.29435   -546.44533     -436.591   -413.79482   -222.97078   -115.62571    848.08763   1209.94865    678.83537    144.76872   -272.97059   -533.14127    -569.7785   -545.71841   -339.59256   -521.61234   -439.94664   -474.63901   -538.51744   -588.54987   -523.27839   -494.08976   -435.77287   -573.75545   -544.83273   -539.21581   -469.19115   -566.37767   -561.52548   -483.75613    -676.4138   -523.91974   -745.32478   -534.03292   -442.07241   -546.58663   -348.29094    -45.03495    -47.92609          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -17.00    -458.0594   -425.07445   -317.02422   -427.25775   -393.15354   -522.05371   -332.38613   -662.51904   -474.44124   -563.72813   -462.88075   -376.08624   -478.15701   -180.95755   -342.48931   -486.69508   -286.23626   -426.90779   -523.36244   -504.61634   -342.30476   -338.75758   -417.41079    -405.0285   -280.16973    225.93851    760.56598   1238.27483    976.71926    247.68418   -289.71049   -403.00835   -346.97415   -542.31549   -382.80301   -265.97031   -452.84761   -599.69926   -533.28703   -579.16801   -417.89506   -466.08889   -480.39695   -280.02064   -523.79039   -478.54782   -516.36764    -682.3908   -496.55638   -453.73948   -592.02896   -572.96138   -580.56764   -544.90565   -564.25299   -587.14871    -301.4431    -20.34521      9.27532   -277.07697   -361.11062   -723.82531          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -16.50   -472.96989   -332.51344   -312.06124   -503.55689   -566.12546   -645.81696   -366.90016   -494.39753   -586.72795   -452.67574   -576.39628   -322.52132   -403.91989   -218.86378   -400.11349   -510.81375   -366.43859   -571.47846   -407.99456   -338.55478   -371.40987   -337.12554   -317.00999   -329.69839   -185.16171    207.29454     774.9926   1155.72809    842.48223    137.68051   -231.69323   -439.32582   -378.11765   -580.36878   -484.98246   -392.35648   -386.39909   -488.61166    -513.7191   -500.72588   -632.08239   -463.65516   -465.20734   -274.16164    -570.1162   -562.64017   -489.00296   -554.28314   -552.72379   -677.08805   -489.30192   -426.47959     -568.611   -550.29335   -410.86152   -452.93533    -54.86148     -1.02537    313.83709    -48.39313   -272.51424   -592.79752   -564.89372   -599.75752          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -16.00   -480.83259   -346.02957    -292.7242     -486.275   -729.36407    -524.7157    -313.6301   -403.44522   -508.49263    -366.7255   -412.61145   -240.22647   -409.73225   -413.57601   -298.09693   -441.58575   -415.46079    -498.6226   -396.26405   -462.34625   -384.93306   -486.45066   -478.15528   -262.83794   -389.03098    183.91422    722.64943    1091.4892    620.64499     86.46063   -288.80329    -300.7094   -490.44571   -473.60421   -602.95247    -546.8747   -286.72016   -521.71478   -411.71134   -477.05813   -382.66966   -417.90904   -563.17038    -411.7607   -687.95989    -525.3115   -396.27974   -607.00073   -610.72705   -504.66463   -586.16182   -438.54241   -537.00694   -407.39391   -601.82867   -359.91649   -221.21941     149.2582    158.32694    -20.71128   -367.62304    -521.2395   -420.17001   -564.04226   -518.52809   -585.84256   -749.00086          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -15.50    -603.9791   -364.35477   -321.11268   -466.50413   -678.28611   -290.36487   -278.55183   -307.26177   -435.68122   -437.45553    -251.0547   -264.44523   -542.06115   -344.14162   -262.68881   -485.20301   -374.51845    -534.8299    -274.3579   -511.77352   -389.09361   -547.42713   -451.06593   -339.89136    -315.7056    353.23694    917.95707   1053.77668     573.3832    139.49886   -353.59478   -276.77895   -516.28467   -392.59428   -498.57414   -435.91652   -410.44434   -486.12635   -402.42123   -457.46159   -518.19417   -538.60324   -490.59255   -431.82225   -483.44387   -583.50013   -499.68118   -500.91887    -532.8651   -456.60946   -692.73561     -573.826   -506.67029   -515.89308   -450.05419   -195.38356   -241.82943    207.23088     236.7703    141.63557   -358.90048   -731.65868   -497.33831   -544.72062   -650.03938   -508.32521    -583.4263   -656.73925   -599.85971          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -15.00   -760.11997   -391.67494   -342.17618   -439.37937   -474.28608   -161.90371   -445.47816   -213.74187   -433.06329   -541.95385   -348.50698    -287.3766   -448.54811   -148.68262   -242.33633   -636.11476   -371.95034   -502.60188   -277.05073   -319.98754   -365.77763   -378.92551   -426.81999   -324.61729   -240.08833    236.16098   1007.80213   1005.28953    691.56611     37.98546   -366.87574   -258.82996    -537.4418   -394.21533   -390.75664   -390.93134   -472.37585   -533.33928   -434.22488   -489.13416   -437.37907   -429.41163   -515.37185   -491.80314   -399.07447   -716.73781   -667.20018   -662.56654   -469.82083   -476.63313   -575.18973   -517.17851   -543.21281   -462.39733    -428.7658   -434.37898   -153.64038    300.74409    144.54273    300.75785   -417.00259   -496.52681   -488.76643   -513.36177   -563.29088   -432.38783   -536.16768   -519.02309   -672.43952   -347.76503   -411.02959    -86.62529          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -14.50   -882.00983   -468.59652   -335.00271   -404.34461   -301.85629   -306.28815   -616.71988   -214.91938    -403.4738   -435.64814   -500.14855   -351.99158   -343.51231   -461.98619   -204.54764   -498.08039   -386.55421   -383.26515   -218.40831   -307.80301   -270.37451   -473.45636   -290.66945   -365.68344    -469.0575    207.58798    672.10275   1099.20929    853.15797     89.25865      60.2108   -363.92128   -504.72921   -413.10185   -580.91699   -330.73629   -490.88112   -556.53879   -397.27757   -541.68387   -419.00339   -495.31215   -333.21979   -406.33271   -596.03564   -521.13958   -477.70141   -624.02364     -320.461   -267.16265   -493.28129   -381.19812   -596.97012   -358.20566    -511.1199   -283.97881     88.56768    313.29565    479.63409    222.57255    -97.63571   -591.37635   -439.22347   -623.31789    -383.5909    -340.4992   -638.97299   -593.08183   -643.88459   -527.32081   -223.95481    -30.65283    -27.63163   -167.12311     -284.944    -175.5423          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -14.00   -619.58858   -582.62998    -309.8246   -373.96516   -321.55346   -473.64339   -487.79738   -148.95243   -335.37233   -332.85373   -578.88394   -469.34783   -340.23745   -647.73472   -455.38066   -408.51093    -395.7221    -306.2676   -296.61921   -368.40121   -210.45959   -547.38942   -226.39084   -371.03828   -372.42267    255.90603    820.18865   1269.83781    987.15168     212.3355   -294.30667   -473.44352   -492.54319   -498.92806    -546.4955   -310.34397   -267.44102   -525.12679   -187.72766   -347.80324   -693.21312   -475.12656   -411.11594   -489.15838    -494.8571   -667.51841    -465.4801   -518.15984    -435.6212   -670.13181   -340.75039    -373.6792   -571.62703   -421.22331   -353.85592   -536.69732    112.28176    420.35103    575.58945    517.90798   -128.99057   -422.24333   -416.18737   -474.47727   -457.35967   -552.26679   -413.47207    -542.4398   -468.28739   -256.15645    -90.66169   -131.48127     225.1133   -117.09216   -234.46518   -165.94827   -364.53982   -529.51718   -565.85697          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -13.50    -390.6624   -528.08934   -346.22982   -286.45695   -370.99705   -431.06538   -326.62686    -84.26425   -314.28522   -333.46649   -562.06753   -599.88572    -336.1449   -511.42095    -537.4973   -544.24356   -384.20232   -396.24226   -393.87263   -496.04067   -217.66352   -343.41444   -145.98534     -352.531   -136.86357    404.60649    967.74887   1226.34084   1112.20953    333.24941   -410.65064   -397.18854   -385.25428   -576.72929   -498.24821   -393.98468   -313.91071   -356.17956   -463.66074    -355.4823   -356.67101   -551.40895   -318.93213   -541.79663   -443.82523   -516.08501   -308.24679   -302.81703   -417.06302   -498.34159   -362.97799    -576.5853   -580.93382   -608.04203   -494.16329   -413.90361     20.50623    428.89101    700.82503    352.68455   -171.04514   -294.64349   -393.15084   -359.14387     -473.583   -596.86016    -475.2759    -666.2514   -332.34537   -340.97742   -163.52043    -86.60196    135.16254     63.95005   -129.91402   -137.70836   -232.36501   -332.56916   -389.60731   -537.96312   -370.54508   -597.47198   -568.28563          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -13.00   -258.52405    -440.9847   -375.94881   -227.60933     -336.533   -361.68304   -338.02997    -60.03298   -413.94194   -315.75493   -394.62507   -670.07394   -249.03511   -453.10675   -540.70297    -588.2992    -286.2065   -391.14177   -460.39426   -404.67729   -193.97199   -202.70804   -199.68964   -505.45342      4.92974    184.44019    988.01423   1428.73311   1057.42255    294.63878   -418.79793    -78.34412   -341.67805   -528.56104   -425.19049   -337.77473   -389.72581   -252.16875   -381.42695   -330.39253   -629.67524   -579.32026   -321.00631   -639.03986   -555.03851   -353.48615   -386.08163   -529.62574   -517.97169    -442.5818   -471.19342   -378.22195   -481.06823   -528.87196   -402.85064   -376.87932    -51.85548    360.96814    569.91815     344.4791    -20.22477   -343.80239   -559.50636   -461.82665   -363.59269    -438.5766   -652.23432    -431.0006   -542.92194   -255.99998     15.28473    387.44993    333.80219    289.73711    -92.35318   -151.72269   -259.15336    -269.4827   -428.44814   -532.12799   -469.20583   -566.81266   -515.04213   -592.00739   -440.57351   -511.10052   -546.83605          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -12.50   -348.15755   -367.09912   -383.39398   -264.01314   -250.26229   -387.45465   -418.24165   -209.22006    -534.5193     -291.671   -284.17854   -529.75702   -217.44281   -468.21099   -502.77069    -562.9357   -370.19434   -361.98542   -287.66946   -248.37414   -297.64934   -299.00704   -483.41663   -400.62977    -136.6299    297.24165    953.77928   1480.30423    925.49444    462.18499   -306.00172   -349.98609    -508.5705    -335.6116   -487.18118    -297.8659   -259.84268   -309.27771   -287.04206   -414.69256   -568.05862   -537.48787   -327.64877   -671.21043   -405.15249   -518.00061   -518.33866   -232.03856   -483.93526   -579.83587   -492.03479   -494.39342    -476.7008   -433.63214   -327.87555   -270.18805    152.93962    673.25231    713.25591    599.71357     189.3875   -244.59142    -460.1543   -423.71628   -450.96697    -500.4255   -422.38754   -536.36222   -484.65919    -366.7976    -23.82689    209.26302     287.1412    272.14582     28.41482   -247.10948      1.86693    -266.5179   -348.53451   -551.64998   -560.48158   -684.41368   -734.93708   -493.84535   -464.98395   -524.85873   -500.82059   -560.31195    -561.6133   -587.85472   -471.44739          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -12.00   -415.89551   -381.18234   -387.18778   -429.05169   -203.20871   -479.67843   -490.24089   -361.32564    -458.2419   -275.28107   -353.92232    -430.2462   -289.50388   -428.84977   -462.35386   -502.15152   -509.68807   -478.09809   -313.90764     -188.551   -340.27529    -514.4025   -413.68744   -304.29377   -204.08123    274.01625   1129.85694     1724.404      940.537    208.32117   -184.98691   -406.23472   -431.52001   -478.05401   -564.91853   -392.11391   -497.08377   -382.91429   -327.40298   -576.27382   -595.24102   -431.70825   -412.53958   -319.21198   -359.48944   -394.08507   -571.08543   -542.23753   -420.42746   -409.84211   -386.95562   -514.12631   -390.97639     -693.045    -339.7739   -377.07027    208.40868    670.49369    933.81925      439.107    272.38785   -307.93113   -419.25659   -517.93353   -542.18089   -419.38791   -506.06593   -439.84394   -505.35102   -256.64473     13.57766    442.46294    513.41126    407.46547     112.0527     -9.80703   -121.06721    -27.23044   -450.37755   -396.62777   -477.99414   -564.96092   -560.39808   -579.69913   -488.00087   -731.94592   -517.58087   -608.80537   -427.44285   -460.87611   -267.81615   -403.12953   -413.31074   -463.47311   -499.07589   -553.38194          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -11.50   -448.35205   -414.37424   -445.33576   -565.35415   -276.06535   -529.95083   -561.12427   -506.55256   -375.78388   -407.33767   -359.90412   -455.59698   -325.29914   -306.25535   -281.49603   -428.05943   -343.86683   -519.73438   -493.71906    -305.3645   -356.99414   -478.52772   -328.50955   -106.57659    -264.7599    128.59389    988.33236   1732.58207   1206.35327     291.9814   -258.70427   -439.72254    -288.9041   -554.10338   -573.23562    -417.4335   -194.18907   -454.43018   -577.08685   -650.85485   -499.20469   -473.53497   -449.53275   -467.24616   -455.87627   -339.12269   -545.73867   -519.37426   -479.17465   -419.16341   -461.35572   -388.21979   -559.48351   -304.91149   -445.70927   -362.60191    121.46178    531.48743    806.74875    523.47018    301.72463   -162.92894   -537.92554   -451.12206   -217.70196   -447.45561   -359.15211   -440.47978   -381.17009   -267.82904    -16.95027    403.84715    471.30271    514.50071    207.01571     24.84393    -65.36797    151.17522    -333.7115   -456.52006   -554.50723   -485.48706   -420.19704   -525.65662   -504.32769   -544.95735   -454.16634   -516.18273   -529.51856    -511.9728   -355.55698   -395.20834   -324.10685   -522.13237   -369.90039   -547.57177    -509.5556   -569.25889   -551.02062     -585.627   -558.96123          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -11.00   -445.94588   -448.15877   -513.15286   -529.90661   -321.99667   -490.49213   -521.17158   -670.25927   -298.29211   -541.48281   -206.12092   -443.12404   -415.96246   -384.78827   -105.82487   -432.91706   -219.22141   -409.98646   -396.90858   -431.13624   -339.67736   -507.82853   -330.54898   -179.43989   -253.83487    417.79765    992.55377   1447.92859   1355.42934    380.40918   -239.68444   -323.96386   -318.79518   -439.82234   -389.26198   -410.60164   -255.32234    -613.0891   -277.17211   -570.71499   -315.26753   -315.50495    -306.7858   -354.57988   -482.11614   -417.60149     -337.815   -399.07597   -305.74837   -406.40481   -433.10129    -552.2823   -396.81734   -341.79278   -328.13992   -339.32567      8.28434    685.26224    953.94822    716.42181    333.08664   -255.10652   -384.79133   -236.83083    -522.7798   -586.96505   -412.71977   -483.31121    -401.3119   -314.35992    168.48204    569.66709    774.52583    398.02643     135.6123    124.78265    -63.98972    -11.42953   -317.82175   -486.48034   -455.89661   -510.38618   -550.20998   -397.30894    -483.5467   -542.06261   -559.53561   -456.66751   -346.21963   -457.46569   -300.75314   -449.25172   -368.82308   -360.82159    -477.5278   -582.81817   -597.96958   -399.95603   -520.65302   -379.68058   -371.75725   -453.92791   -474.81326   -334.23837   -374.79674   -512.62514   -631.23714          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -10.50   -450.20637   -434.87418   -559.78253    -443.1169   -281.75633   -487.84652   -321.00339   -624.40899   -207.81494   -386.09637    -138.7991   -435.10747   -368.15098   -487.33574   -324.95667   -403.02719   -119.84456   -110.99553   -361.12293    -614.7444   -373.16695    -552.1748   -283.10433   -216.34243   -228.50174    425.55889   1174.17576   1604.37921   1162.34332    343.65122    -54.93303     -301.117   -262.60742   -380.67484   -265.48607   -486.11572     -407.676   -426.39754   -220.24917   -447.84935   -341.82862   -484.23685   -321.13876   -320.17772   -421.25599   -369.29588   -373.09762   -437.43815   -482.55708   -428.73982   -466.34316   -445.58089   -605.02199   -291.92499   -528.09858    -353.2056    130.65862    606.54268    896.00345    690.16905    316.94205   -303.93092   -244.07123   -329.21575   -433.12017   -495.70158   -429.61875   -339.64505   -537.63588    -336.4865    300.48838    439.83458    676.70494    646.08439    404.61044     37.45925    110.12334   -136.47017   -450.70195   -405.42024   -504.98457    -732.0773   -634.40699   -498.97465   -483.22274   -384.08894   -412.10419   -340.40195   -504.66373   -566.67359   -291.75574   -264.11583   -401.42468   -292.91842   -496.86718   -401.47802   -553.88453   -547.95563   -488.26454   -441.18863   -457.44101   -432.52787   -436.41638   -404.86087   -512.93253   -496.89813   -587.30966   -605.57085   -575.25882   -543.53974   -621.06627   -614.46113   -571.26252          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -10.00   -468.63173   -436.43425   -529.21468   -361.26139   -235.23485   -505.90277   -179.97403    -393.8219   -285.37814   -230.94658   -124.79055   -479.51841   -245.81297    -303.3948   -441.80771   -292.65774   -289.23333    -78.05073   -437.06843   -517.91751   -374.97749   -614.95345   -152.69931   -212.00828   -283.59186    440.80971   1484.87395   1684.39779   1121.88134    339.75918   -125.59828   -298.75689   -184.99669   -419.67873   -330.00254   -470.26567   -292.37316   -379.51228   -550.02226   -530.33879    -418.6434   -485.68281   -402.73796   -490.75213   -421.47426   -373.91917   -470.95553   -552.22922   -442.03296   -390.41994   -533.43493    -331.5558   -379.40916    -406.5163   -499.63207    -65.23318    121.42883    754.64928   1123.77782    727.81953    185.46544    120.71047   -352.90079   -344.48708   -408.85159   -456.16049   -487.28658   -293.18831   -495.84451   -131.96156     172.1686    530.91556    826.99587    763.01238    354.70305     78.90359    240.47635    -25.25733   -263.78064   -481.24457   -501.01777   -408.85589   -425.55618   -438.39982   -511.57546   -410.93023   -444.13426   -405.05207   -419.80768   -493.22941    -371.9652   -138.30596    -268.5619   -224.67371   -606.86142   -610.91003   -564.85259   -476.64576   -397.17422   -553.85982   -321.69938   -384.06202   -180.34087   -464.64063   -561.80792   -557.30642    -468.7496   -626.50525   -639.11672     -451.119   -663.86693    -479.2447   -511.44069   -595.48248   -659.25222   -476.09046   -439.79573   -308.30143   -448.46457   -419.82321   -551.12319          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -9.50   -507.06283   -418.13828   -513.72231    -297.0723   -272.12319   -486.88481   -228.46315   -263.24907   -399.17026    -298.5566   -145.62294   -522.21117   -267.10137   -217.36249   -414.09656   -298.34707    -373.5284   -425.12703     -297.139   -388.81655   -326.77651   -475.66506   -138.00171    -71.22503   -222.23864    462.52912    1193.0321   1797.72973   1282.30331    631.36729   -243.87927   -207.67414   -435.21481   -298.86745   -387.40374   -232.58539   -324.94723   -311.39382   -333.21603   -389.12563   -350.94151   -607.53494   -300.29561   -444.92371   -299.28664   -412.66431   -355.55141   -365.25297   -429.02126    -382.6801   -355.91471   -231.56684   -384.10779   -431.25688   -348.72812   -139.64314     55.84689    549.72663    1142.6844    628.12192    227.73894    -10.89145   -517.54527   -335.11471   -443.94515   -277.83322   -281.53359   -394.05351   -407.74659   -205.55457     77.92897    554.79904    853.66577     702.6004     254.0094     70.31343    201.01723    -12.92265    -98.04996   -227.62971   -488.64636   -468.40184   -543.53094   -374.25728   -503.61411   -392.74037   -404.58136   -494.60218   -392.63374   -365.41872   -289.16067   -273.06392   -234.02232   -313.31249   -334.16001   -407.06149   -538.79051   -571.51316    -507.7187   -413.24939   -406.59335   -424.54054   -253.82032   -355.46543   -457.77479   -370.31268   -486.26854   -526.21731    -549.2102   -474.21386   -521.35977   -653.03417   -573.42326   -520.26961   -649.76451   -531.99671   -283.61036    -33.42215   -234.66297   -379.17156    -480.0169   -422.33184   -563.16776   -427.36257    -475.3406   -205.61472   -136.96433     -75.8895   -298.97251          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -9.00   -554.78605   -344.83375   -494.55469   -223.42168   -445.40909   -409.84471   -288.85192   -296.42673   -434.98203   -350.18287   -376.18323   -441.43268   -307.85725   -232.33662   -347.33701   -321.12597   -275.53934   -425.61677   -232.69951   -293.46312   -409.61764   -316.58525   -354.23044   -356.67647    -98.79984    402.80169   1221.30087   1774.02426   1680.37291    420.24074   -199.85178   -251.32572   -182.14184   -199.08843   -455.16899   -412.64226    -500.5919   -371.84302   -468.41857   -485.55372   -414.80782   -385.01387    -477.0674   -507.26917   -359.62634   -417.69198   -338.16881    -476.6546   -386.50816   -466.92111   -289.26323   -417.26705   -542.50954   -451.04154   -394.46619    -80.56396     86.72124    644.75884   1139.32981    892.79577    413.83694   -135.61316   -488.02384   -297.26038   -459.32928   -332.56193   -367.63856   -405.61042   -291.48968   -133.89642     24.44806    513.60275    903.95927    851.43957    474.09866    356.63927    307.17483     -4.18563   -170.90024   -169.91259   -274.49607   -517.24009   -553.89343   -427.38052   -510.90897   -567.12725   -481.88051   -551.32842    -736.6109   -442.85125   -143.74908   -247.79805   -114.27948   -353.43306   -464.45135   -395.87995   -453.45587    -506.1649   -414.90793   -503.18195   -253.19593   -277.93761   -279.81812   -317.62411   -307.53563   -297.38996   -412.28873    -533.5894   -589.78386   -524.52763   -478.03718   -595.11322   -447.35129   -540.51041   -500.26477    -498.9403   -323.97706   -294.57432   -233.85504   -232.88533   -386.98244   -452.24762   -412.47031   -379.61693    -419.1774    -81.03928    -23.83762   -148.62546   -230.50316   -444.30228   -462.79753   -522.40991   -438.18817   -553.83229   -576.95821   -421.79804   -290.11293   -231.97888          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -8.50   -535.94377   -252.73044   -437.49372   -253.79292      -647.13   -328.99189   -335.99716   -387.45656   -421.69458   -354.31014   -544.43776   -341.15296    -380.4386   -241.68722   -267.38262     -370.718   -302.37322   -344.42244   -360.47235   -277.12132   -426.79118   -164.53508   -330.54204   -443.33813     15.67032    157.81607    1233.8409    1660.2497   1514.94646    288.95099   -155.65217   -236.28794    -298.0102   -333.36504   -384.26018    -521.1506   -507.18447   -481.72638   -225.31362   -670.02725   -286.52719   -366.10347   -335.34201    -393.6556   -326.26826   -480.59453   -425.47407    -353.8672   -476.12007   -538.56617   -404.09089   -396.35466   -385.35688   -395.41988   -344.73553   -223.15571    132.43227    614.73368   1174.16305   1046.20829     361.9518   -223.99132   -237.42996   -290.74385   -408.76963   -415.00809   -241.67417   -249.85532    -446.1422   -169.27307     50.61508    623.03772   1046.31208    644.87481    523.63278    365.72026    361.12926    252.71276     10.12497   -413.25542    -358.5029   -609.64139    -300.7041   -589.74605   -403.17066   -482.41474    -443.9132   -369.96465   -397.13116   -287.60684     10.49912     -47.9068   -297.28189   -336.89065    -289.1327   -394.79559   -527.96552   -407.53348   -503.89152   -252.98846   -273.44091    -185.9365   -245.64209   -201.27461   -335.77974   -334.77209   -480.62008   -548.19225   -423.66207   -589.57882   -546.67492   -580.28351   -459.64065   -403.70848   -345.73487   -304.71965   -272.17619   -135.51358   -269.33037   -196.77951   -257.23853   -517.29511   -423.25903   -369.55419   -252.46662    -21.16185    -65.55917    -75.07255   -312.88301   -375.42627    -409.4825   -385.74813    -651.6021   -505.56078   -622.09249   -491.22806    -222.7128     20.40027    427.30119    718.92558    542.66663    265.41603    -37.29392   -276.30181   -446.79307   -409.65972   -552.75461   -490.21677          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -8.00   -450.23036   -215.60406   -348.95564   -325.88347   -567.95251   -254.55828   -384.22753   -584.24436   -291.09408    -370.1517    -504.7435   -320.04398   -443.50701   -197.62722   -422.46496   -372.33419   -342.41526   -316.23561   -417.64222   -282.66459   -385.64108   -280.78497   -354.91696   -408.81076    -72.92797    172.74877   1173.26623   1720.70201   1299.69923    270.50314   -215.61551   -361.74744   -280.71998   -395.96839   -232.95783   -228.84091    -408.5504   -385.04632   -330.36631   -413.37468    -584.1962   -403.21823   -457.33657    -392.8023   -326.43109   -426.41707    -536.4623   -364.69487   -467.78975   -215.18248   -358.77439    -177.3823   -462.87774   -360.35817   -319.08855    -205.6868     85.62029    595.99991   1085.73527    766.64128    619.19304    -95.71141   -293.11619   -291.26899   -317.47542   -367.07558   -459.44725     -423.548    -326.6198   -156.06528    121.90433    527.31841    924.07856    655.16663    696.02931    452.56423    254.78194    135.48137      3.69179   -392.03168   -368.63713   -328.70296   -441.47877   -435.40779   -482.32363   -457.94085   -444.58906   -427.98486   -313.22246   -334.84408   -169.22513    -88.84946    -30.75056   -145.89639   -334.69692    -340.6775   -379.94942   -478.27879   -362.49317   -448.42804   -222.94426   -294.83899   -129.90961   -308.77055   -535.25199   -411.00457   -519.31803   -455.57129   -454.97645     -604.573   -503.61075   -420.30948   -556.05262   -530.56584   -505.24169   -196.50594   -328.27528   -188.74823    -41.96149   -261.91103   -364.15075   -315.63213   -499.62669   -381.73602   -246.46927   -107.42484   -161.84107   -199.52164   -214.58242   -162.86974   -504.73179    -579.3691   -497.40232   -478.19335   -489.02247   -400.60596   -202.72095     55.35825    334.18954    685.06112    438.83699    238.59842     -0.57121   -225.29222   -272.67482   -530.09112   -650.79132   -415.16432   -489.23762   -497.06196   -621.51201    -506.8757   -530.59896   -673.80298   -474.35388   -610.77744   -563.03442   -612.96399   -498.78494    -442.9715          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -7.50   -363.62214   -222.37796   -309.99073   -366.20499   -470.88653   -148.40113   -427.60854   -740.15755   -113.48086    -427.6489   -480.69189   -305.73331   -462.23337   -279.58328   -497.42865   -263.71083   -194.55258   -268.81127   -413.18576   -150.61021   -484.65409   -419.64845   -373.13402   -345.60462   -210.00352    247.82474   1047.60752   1646.73202   1234.40091    436.22772    -55.16314    -153.7365   -304.66166    -366.4116   -311.48115   -348.97708   -344.81713    -319.3854   -497.37731   -342.91591   -429.69834   -393.31338   -474.84044   -479.12722   -293.90485    -264.3897   -391.55917   -414.07938   -437.83746   -470.78207   -439.71131   -426.38899   -298.65707   -434.61481    -292.0735   -198.26517    236.35532    700.63776    982.71267    865.87734    498.52835   -256.44989   -259.59367   -212.00348   -296.25355     -422.267   -240.52627   -427.65227   -412.36044   -202.26283     245.5884    781.33526    979.67123    990.33849    550.90851    379.85316    343.53714    189.92166    -25.83008   -159.89969   -358.33447   -405.97734   -514.75842   -357.85716   -431.74924   -380.76326   -284.52325   -438.95845   -394.72903   -328.03581   -284.03757   -246.49446    -56.33078   -194.08053   -271.31317   -454.22217   -331.31418    -332.6792   -429.96674   -276.40032   -384.43502   -205.15488   -405.90114   -182.60973   -276.53371   -490.86588   -464.37847   -419.77692   -473.43966   -408.82816   -643.41009    -534.3559   -488.79058   -492.97957    -373.0864   -427.85541    -274.3203    -54.36928   -117.22483     -250.651   -205.24447   -348.05686   -484.81603   -167.40942    -69.91155    -84.88152     58.82464      39.8195    -97.35062   -173.98688   -305.16394   -309.93366   -491.21298   -404.90891   -466.51823   -309.08477    -95.27001    130.97934    362.13878      701.162    866.23785     411.8295    182.01433   -110.14851   -334.85716   -597.20477   -380.34075   -622.93012   -414.47273   -248.97226   -421.31618   -585.10367   -572.12961   -663.12657   -528.97606   -555.99394   -544.56154   -534.40803   -413.20996    -514.8057   -464.19766   -492.51716   -476.01767   -368.14146   -473.80372   -393.51552    -345.5701   -331.51524    -379.6081   -629.31525   -503.33081   -676.35719   -657.66147   -694.62025          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -7.00   -362.74742   -243.73543   -285.46398   -286.44949   -440.91645    -32.01053   -475.75871   -323.57551   -117.88752   -469.25099   -510.02654   -326.44561   -414.10991    -424.8562    -399.9689   -232.98745   -186.43022   -222.14947   -329.44831   -222.23071   -461.44028   -500.04943   -271.75392   -336.79001   -169.89891    273.93486   1159.93952   1624.50335   1360.14613    710.86314    -89.74777    -81.62539   -374.83823   -372.88838   -305.98514   -451.58249   -350.83889   -324.29886    -360.6186   -397.13635   -308.63363   -488.04913   -499.92183   -391.04232   -344.52074   -307.50926   -382.85526   -547.71756   -484.97386   -389.00698   -356.86355   -321.11427   -359.67332   -349.05998   -249.18366   -306.62095    252.09276    838.43879   1263.54897    926.72346    568.48117   -202.75013   -374.92886   -471.04048   -325.37408   -527.50273   -300.01817   -483.12878   -297.27747   -186.00198    269.06417    835.40923   1437.80304   1052.02419    580.57963    314.08242    507.44925    410.33521     99.15389   -173.01039   -323.95978   -373.68414   -332.95823    -411.5855   -528.39893   -409.98488   -425.50565   -433.42999   -394.23988   -281.21632   -202.89502     11.68954     23.51374   -271.49391   -338.37187   -324.41113   -368.88487    -356.6483   -480.46267   -415.07829   -329.20216    -73.90534    -80.34858   -238.95447   -246.04545   -329.92336   -382.78954   -385.94134   -548.62486   -576.44822    -586.3066    -572.0742   -402.07028   -420.62108   -325.90832   -200.85014   -110.46262   -101.51287     -2.54774   -143.02845    -176.6049   -296.12686   -239.43967   -231.54423   -161.38372    124.82622    -14.66491     45.99821   -155.77381   -336.77266    -304.3129   -346.22039   -505.25251   -477.56948   -284.05295   -198.02849   -204.05649    207.76135    581.77975    879.55646    961.86771    621.70096    281.18681    -34.65583   -312.07059   -469.71004   -494.56441   -361.93713    -690.0568    -470.3448   -362.57794   -556.37814   -506.52083   -615.38706   -505.56301   -540.17266    -529.9992   -493.85498   -448.60295   -426.93852   -511.55088   -452.05757   -400.58587   -407.92441   -500.78918   -405.57177     -353.938   -417.40999   -480.58185   -547.14573   -599.38869   -563.54119   -513.22857   -491.95803   -504.77061    -606.0191   -431.19509   -600.82009   -453.67228   -536.71661   -545.39748   -652.83165     -605.048   -617.06542   -583.40225   -578.92283   -747.46852   -637.71501   -702.22936   -773.13071   -620.89367          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -6.50   -413.67988   -239.42407   -270.88737   -193.42956   -464.61537    -82.69487   -477.87915    152.75282   -275.49156   -312.11871   -540.08243   -391.63106   -258.09278   -356.87248   -462.59834   -239.22167   -306.72571   -294.64595   -265.66247   -339.83514    -308.5311   -519.06636   -244.73067   -298.91963    -68.44585    402.86579   1233.13334   1697.97462   1492.24363    511.05847    -48.67808    -94.43475   -316.18662   -378.79003   -322.98508   -383.98725   -379.74388   -399.22133   -691.29043   -335.94368   -337.80164   -423.84526   -265.58147   -443.97978   -433.02866   -404.39055   -345.81999   -394.75674   -417.46381   -359.22942    -457.7037   -322.90317   -341.95785   -336.88053   -316.42068     -14.9587    115.25623    923.86181   1290.86619   1178.21388    548.42588   -224.95617   -235.84428   -244.38077   -336.80609    -296.7986   -394.43192   -320.28556   -388.48766   -190.72549    377.79374    737.40701   1045.50877   1006.54786     493.3934    454.51871    497.83184    329.91459    125.26482   -227.28889   -127.24074   -354.91914   -311.68289   -382.45526   -533.42041   -486.18852   -343.83409   -486.71827   -261.90851   -251.46021    -219.2208    -85.84903     68.44188   -216.05816   -243.39564   -280.11957   -429.43628   -345.78044   -327.02218   -223.28989   -324.26914   -365.89554   -194.56587   -152.34827   -147.40254   -227.64988   -402.87907   -342.17359   -402.52868   -509.68278    -363.2756   -530.40057   -527.20693   -425.14678   -383.67027   -336.88995    -112.8675    -57.29787     -7.82482   -179.25143   -207.09606   -444.41465   -469.72451   -274.65438   -108.80279      48.4365    236.54383    111.12745     -82.5822   -235.93808   -325.97973   -451.09859   -377.61964   -402.16981   -469.55719    -370.2156    -63.40587     270.2085    645.34178    866.86738     791.1543    697.14229    369.46081     98.11051   -296.18791   -370.79652   -382.34507   -528.82491   -525.65135   -485.26126   -446.29334   -551.46866   -543.57514   -553.31343    -606.6483   -508.65796   -597.72838   -448.07525   -419.86026   -380.76265   -369.44718   -400.65743   -483.15338   -303.12597   -402.29509    -346.7109    -320.7572   -366.71675   -341.91758   -312.46558   -375.19929   -494.98952   -583.01613   -593.89159   -490.96754   -364.32881   -565.30832    -495.0778    -601.7662    -451.6813   -471.50586   -516.22078   -618.08549   -541.85229   -540.38203   -584.08036   -632.55386   -640.99608   -623.51985   -684.15543   -621.71242   -508.11812   -564.09372   -563.51101   -478.90372   -502.51536   -362.98457   -464.05619    -527.3858   -460.61577   -462.32866   -448.63442   -545.12345   -624.86311   -632.90687   -395.94551   -581.09994   -582.54048   -602.52625   -600.36691   -521.67357          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -6.00   -468.69084   -247.28749    -274.5684   -398.78042   -470.67071   -134.10583   -402.12769     36.50483   -375.96832   -167.10016   -580.77503    -568.0557   -126.82478   -379.51454   -639.49671   -282.98531   -429.84826   -334.18932   -251.51303   -492.13558   -344.85997   -436.57468   -255.57949   -265.13915   -174.24268    418.65268   1221.73976   1883.60693   1489.69506    644.85481   -180.07947    -105.1776   -311.87194   -390.50331   -434.59099   -351.47886    -327.7193   -269.69519   -370.17249   -473.22569   -504.48034   -391.69444    -315.9642   -577.57239   -394.63514   -544.91783   -443.38766   -461.97437    -554.2633   -415.92859   -403.71055   -298.21466   -460.78264   -367.95544   -389.37155   -117.48614    148.25882    809.96451   1425.41343    1193.2898    656.65469    -35.80864   -296.33273   -231.07204    -380.5269   -306.32786   -336.15266   -206.43504   -326.65554   -182.54886     73.41124    694.31307     1042.478   1062.35176    613.20663     540.1348    506.21059    342.78966    165.54423    -90.49303   -210.47506   -369.52565   -400.84857   -432.53556   -456.47821   -387.10241   -485.58101   -413.46513   -347.76288   -305.52017   -250.87247   -187.72078    -32.71384     -180.151   -306.84146   -263.24508   -263.44944   -299.07147   -400.43592   -324.73355   -207.50929   -155.87827   -215.65419    -42.26935   -448.63977   -310.41288   -453.72435   -457.37641   -307.51455   -323.49799   -538.71334   -470.63834   -389.08367   -509.31309   -584.10664   -275.58875     76.30176    -34.75076    -70.35649   -233.96798   -164.16006   -171.94094   -538.96301   -348.66263      0.16111     227.5689    227.47309    104.68942      22.5083   -127.45402   -291.45978   -448.91682   -348.86458    -391.5964   -352.25204   -290.87321    -68.45174     328.0044    644.38951   1077.86015    747.34829    706.47761    335.72068    123.93966   -244.46249   -303.34652    -320.3985    -381.0268   -406.11764   -357.82047   -446.67052   -505.00971   -489.72362   -545.56024   -346.35698   -409.47534   -482.60737    -521.4868   -467.16503    -355.2225   -369.46922     -466.552   -418.41114     -461.473    -317.6923   -317.37322   -156.31336   -345.00436   -404.10943   -336.93961   -516.18749   -338.76761   -491.28606   -430.61398   -439.33834    -488.7802    -455.7503    -485.9674   -513.87008   -552.07092   -476.94491   -475.52971     -442.001   -665.84768   -440.12505   -592.57043    -557.7643   -609.99644   -792.22587   -584.63929   -558.49449   -419.47424   -375.30213   -524.13395   -551.31546   -395.66097   -280.45412   -331.98391   -291.11718   -384.88295   -400.18604    -390.0522   -460.00482   -389.35754   -542.55238   -504.58137   -423.45488    -379.2815   -338.74571   -421.33238   -560.45152   -524.57992   -499.32099   -561.00178    -560.0069   -597.50122   -642.16878    -642.7976   -757.35708   -586.39203   -670.48874     -720.672   -635.66689   -575.20668   -577.63875   -525.04234   -435.83246   -437.25685   -289.25684   -409.19094   -561.11954   -450.99044    -463.8261   -193.61578    -547.6466   -473.62085   -478.31784          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -5.50   -479.16314   -230.62303    -278.5515   -703.08295   -409.41506   -265.41019   -309.63125   -117.19689   -399.68112   -358.86589    -507.5113   -661.31775     -9.01082   -583.03268    -513.9163   -319.58414   -534.71759   -329.33666   -182.52901   -419.81646   -421.17067   -321.32765   -309.30637   -479.55823   -228.80375    309.49542   1140.65298   1978.07885    1533.2394    755.20389   -147.10715   -185.79671   -494.34488   -218.16577   -611.22778   -398.09986   -415.13741   -401.85153   -614.84104   -428.77722   -476.42291   -445.85705   -493.45365   -420.00996   -319.66543   -448.88572   -405.98253   -393.77764   -318.80682   -379.89038   -304.26058   -382.68207   -347.37723   -299.43452   -328.39571   -304.07379    111.90661    848.78346   1429.90131   1433.82434    516.15083    -51.01596   -304.58618   -339.02827   -311.39912   -355.99428   -459.99604   -347.43243   -232.64257   -232.16297    209.51553    758.10231   1168.02481   1151.66459    830.43188    599.30626    555.95323    448.96798     269.9024   -158.60964   -212.51323   -215.82674   -392.58878   -375.64562   -445.08447    -429.0504   -424.16541   -338.59583   -447.53382   -208.39558   -149.28807    115.86169    -72.50207   -217.80197   -225.26748    -336.5989   -402.23747   -267.12985   -280.46182   -323.89608   -293.18988    -87.94081    -32.86062   -146.81208   -130.62969   -247.83073   -352.29384   -321.41776   -399.32656   -600.67038   -448.56147   -433.92188   -446.53874   -429.90833   -178.17162   -112.17523     -70.0873     -62.7217     22.42819     -3.15626   -127.36173   -313.70611   -253.53861    -260.7624   -172.53908     -4.04619    226.43758     85.81509     24.54689     -66.5021   -231.98602    -362.9623   -447.47542   -341.01907   -481.56675   -336.29714    -65.17619    403.47265    926.14815   1014.01452   1079.82289    667.82763    522.35339     89.11189   -410.80773   -109.71304   -427.90427   -234.53292   -382.02186   -451.11996   -380.43912   -411.20587   -734.12061   -633.31691   -523.22781    -621.9091    -453.8417   -305.76417    -388.4145   -532.13561    -396.5001   -481.54665   -418.13965   -365.65667   -426.14183   -164.51121   -393.26495   -325.34535   -255.80432   -438.38328   -580.77703    -499.3702   -462.49145   -519.07943    -561.2745   -501.59146   -402.34092     -353.517   -426.55263    -403.1792   -476.25617   -498.57483   -367.46496    -389.2061   -614.76842   -504.05479   -602.54865   -520.09572   -474.20762   -438.00942    -364.2945   -505.51209   -455.19674   -584.07196   -470.32124   -285.19164   -343.17817   -407.01123   -266.08909   -260.04008   -342.57947   -294.56584   -285.01806   -399.24813   -405.89132   -479.06386   -430.00179   -366.74639    -256.5353     -327.195   -254.98864   -231.66472   -223.59917   -548.13824   -563.56689   -558.24217   -592.31702   -570.56432   -554.82268    -716.9911   -500.49527    -532.9392   -575.31632   -468.75812   -355.06239   -376.88015   -516.64661   -351.76783   -166.73938   -407.07894   -383.10867   -151.28811   -406.18311   -389.55767   -487.50543   -227.35425   -253.95367   -420.16591   -379.08227   -473.03915    -356.1484   -401.01518    -420.2402   -562.41929   -362.60085   -620.62433   -528.16223   -608.27592   -672.44424   -747.21014   -654.97487   -750.50956    -556.8292   -373.23624   -434.27063   -546.61737   -633.25802   -451.29432   -464.42004   -539.59879   -515.91017   -351.27789   -279.51728    -73.40951   -339.75651    -205.1963   -301.68724   -324.70815   -526.37405   -426.50659   -437.93998    -481.4284          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -5.00   -469.47056    -97.49759   -398.18924   -856.57324   -349.37422   -400.22129   -190.37808   -216.48109   -414.26362   -516.52038   -354.73849   -470.02347    -263.5648   -521.09309   -375.00506   -348.14185   -544.79381    -345.1107   -131.17653   -288.47386   -489.11503   -223.70036   -229.76951   -525.82113   -278.47036    153.30628   1014.78605   2013.59375   1569.28968    715.01535   -135.08524   -264.88851   -332.83025   -100.30799   -316.77095   -362.99192   -544.17105   -422.02427   -252.13603   -379.58513   -372.65497   -259.77567   -202.15308   -358.63052   -307.33873   -467.42841   -476.14464   -293.17763   -468.37079   -373.51367    -194.5072    -350.6688   -283.02741   -340.08676   -274.40258    -76.07539    116.17469    802.93452   1479.12794   1311.48343    598.58263    -102.2309   -242.80567   -302.41029   -325.48055   -245.99516   -289.46335    -282.8031   -388.37935    -55.87879    138.24032    863.25501   1017.34045   1163.07482    745.70915     681.6974    598.47692    365.66809     72.52284   -279.15342   -365.05047   -259.29906   -322.78084   -387.89393   -471.22949   -465.04383   -273.78642   -354.96297   -259.31916   -305.89179    -126.5916      23.8326    106.55135     -5.95596   -258.34008   -343.63746   -482.58847   -455.52573   -396.26414   -238.29487   -192.58591   -174.14655    -99.28081    -285.1378   -338.60447   -384.33926   -447.61827   -527.79492   -415.84824   -475.79026   -579.46297   -380.99248   -440.76146   -477.75852   -299.43478   -289.52799    -86.42307    253.55714    -70.79455     18.29722   -164.81141   -210.26193   -401.10545   -310.76323   -158.31114    174.86919    254.26945    263.72622     13.19216     -20.2962   -227.62269   -381.70995   -425.74883   -449.84902   -467.50988   -409.53209    -85.19327    255.72817    885.99908   1066.34595   1316.04177    943.31118     482.6573     54.14663     46.99282   -376.79815   -431.93855   -343.07882   -313.40149   -422.35287   -357.86642   -409.03748    -493.6211   -469.36055   -552.24096   -457.40749   -173.15441   -406.80308   -313.02723   -315.53472   -273.70034   -412.47715   -324.53174    -371.2668   -345.45406    -343.3489   -219.16599   -260.98156   -245.25208   -401.65276   -454.29524   -410.71104   -499.27918   -564.58143   -472.51959   -436.41138    -462.9618   -369.01828   -488.07862   -511.04438   -363.94975   -526.93189   -409.54777   -498.50613   -455.68628   -560.95226   -354.79086   -389.69315   -582.30679    -483.9971   -455.37752   -464.86877   -351.09098   -388.48465   -575.68075    -372.2393   -195.25126   -284.45652   -254.66705   -232.33366   -174.86589   -187.18389   -381.47116    -357.9348   -425.15734   -253.12111   -468.86345   -357.71876   -110.32916   -334.79582   -262.78829   -327.87857   -248.73248   -379.05567    -338.7668     -457.109   -437.24554   -471.09429   -516.76564    -493.9985   -439.97456   -546.58158   -540.59649   -420.29123   -365.76596   -377.32017   -305.01208   -261.03397   -205.35094     -38.7382   -251.83655   -321.86749   -257.50785   -213.39703   -363.22894   -264.31703   -307.06611   -341.07397   -385.70125   -293.44012   -207.70294   -246.74517   -472.00809   -443.25255   -513.21139   -598.45779   -545.08861   -624.47863   -671.07651   -584.43817    -570.5562   -551.81752   -414.20681   -439.54184   -569.33386   -594.51459   -240.04686   -418.44398   -386.93071   -425.98085   -291.84443   -277.50613    -143.0883    -170.3614    -74.89818   -254.20876   -175.95674   -126.11785    -45.92092   -310.76892   -401.71219   -511.57837   -453.13207   -450.08302   -592.13652   -639.68831   -671.10885   -522.27733   -500.36157   -363.48557   -426.66239   -443.18454   -412.49454   -634.79528   -507.67142    -569.3871   -839.72251   -608.19742   -657.74734    -785.6246   -789.84191   -774.79179   -732.92895   -689.42391   -678.15242   -807.96101   -808.37753   -756.60607    -822.5744   -737.90622   -805.95763   -819.90572    -803.3836    -743.4969   -671.12847   -737.30704   -707.86699   -766.45656   -680.79808   -654.28426   -623.42523   -673.21358   -660.83879   -739.67992   -640.43188    -593.7056     -768.452   -711.27253   -684.22618   -666.27093   -778.09719    -740.9239   -842.67992   -764.95291   -634.67648   -801.40665   -843.89736   -773.53823          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -4.50   -453.42317     30.85109   -556.61158   -605.29241   -376.78223   -447.08727    -90.26081   -315.99037   -453.19801   -390.77649   -280.15927   -270.58774   -476.09001   -300.89294    -235.7212   -214.62701   -532.12659   -353.06449   -167.62293   -135.69767   -443.04236   -267.98947   -284.48809   -437.61705   -268.81619    132.38145    985.26165    1835.4743   1600.80951    551.93394   -259.66136   -327.17933   -253.45034   -408.83957   -291.97674   -291.22331   -256.32157   -286.02694   -422.31473   -267.20152   -103.04551   -365.70612   -262.47491   -320.28649   -124.87114   -471.49624   -387.91926    -460.2682   -459.21278    -462.4425   -269.53256   -467.06147   -428.33576   -319.80802   -354.00414    -290.6089    320.80932     1018.676   1392.93003   1135.49256     548.0698    -10.78897   -223.12655   -342.22345   -446.18172   -210.74884    -429.1794   -342.28546   -317.46874    -68.45686    189.59069    929.60306   1090.71823   1361.82904    755.89679    514.24762    346.35348    489.10424     86.11621   -133.39798   -241.76045    -341.2845   -492.92368   -438.66815   -372.99251   -388.86711   -432.49314   -315.43369   -288.52779   -232.71287   -321.44309     -9.38456      80.1313   -290.52672     -257.316    -359.3118   -314.94561   -445.01942   -571.92559   -223.49079   -267.95378    -114.6349    -40.68292   -171.34302   -170.85185   -330.36387   -335.44311   -348.56884   -405.78577   -389.01761   -368.56907   -457.14911   -601.47444   -482.80581   -341.00789   -175.20818    -50.04344     45.48089     85.11419     202.6986   -105.95183   -208.05071   -297.77584   -152.97712    -64.86063     152.6742    273.65334    289.82118     99.01389    -15.97988   -281.51427   -348.11121    -347.6221    -414.7754   -441.68615   -231.71849    108.68548    406.33907    819.30741   1331.64125   1266.18296    1107.2251     402.4545     61.22027    -316.5457   -403.93366   -406.60362   -419.62801   -485.54439   -380.44774   -314.49768   -443.91552   -422.49865   -444.12371   -398.26737   -523.39205   -395.22047   -288.28434   -307.22697    -272.7083   -362.18821   -240.79058   -331.20108   -414.27992    -195.4397   -146.73114   -197.37348   -146.47574   -240.88098   -327.46019   -378.80885   -303.20579   -381.61732   -363.14889   -346.73235   -379.83088   -385.96072   -394.48989   -202.18601   -459.40948   -334.62872   -507.18501   -423.44388   -532.51525    -473.2882   -621.34216   -470.95868   -498.22642   -437.59403   -445.49976   -377.09754   -302.07262   -285.90802   -327.36275   -203.80871   -160.85534   -232.18525    -52.78527      75.1033    -44.19733    -84.36662   -165.53782   -180.25453    -278.6362   -466.42265   -271.10354   -260.68637   -207.72841   -281.10752   -167.98548    -54.95072   -176.52797   -283.18327   -157.84854   -283.70737   -365.13689   -512.69965   -502.88331   -513.47094   -502.35775   -518.16849   -420.55917   -457.93083   -503.95119   -487.32472   -315.78072   -236.45703   -266.21898   -193.43105   -276.61971   -385.86547   -149.97146   -231.78087   -277.73552   -174.21588   -442.82527    -301.7729    -239.5692   -379.55446   -310.51622   -141.06395   -323.20591   -472.45473   -420.01806   -505.46807   -606.54542   -545.40171   -514.31871   -538.82706    -548.0145   -591.55602   -676.93873   -503.64584   -504.58542    -346.2055   -340.42961   -309.52441   -314.36712   -338.77775   -275.12452   -257.62834   -283.61621    -76.11483    -54.35245    -30.36773   -185.07813    -60.10906    -68.10109   -248.47377   -333.88384   -344.89933     -407.809   -400.79591   -436.41998   -464.97663   -577.42182   -638.84401   -580.30419   -485.59707   -452.39111   -347.84741    -320.0645   -440.70734   -497.98221   -474.76788   -614.69927   -612.05999   -512.81914   -647.03572     -672.373   -592.68971   -668.51533   -801.04716   -691.68514   -550.29574   -663.04633   -793.01221   -572.54873   -706.38149   -701.49612   -557.60466   -645.34235   -705.21393   -596.53675   -635.68625   -584.10497   -723.26535   -695.30468    -678.9608   -563.10442    -591.5334   -530.03375   -500.90805   -642.04257    -464.8019   -537.43391   -448.91489   -498.04873   -525.68355    -663.2519   -582.40885   -468.31375   -471.46067   -703.78144   -545.17789   -520.97244   -630.89831   -600.15316   -724.62886   -567.11943   -538.74848   -587.50579   -731.17076     -689.686   -503.06722   -681.61747   -438.58421   -448.79824   -647.20779   -598.97519   -493.51974   -533.09114   -595.18343   -507.00673   -669.42227   -830.29101    -737.3706   -756.64801   -641.62141   -584.24151   -620.36867   -614.90131   -385.06492   -475.80642    -509.2297   -355.22719   -478.76552   -672.66215   -569.66775   -307.60777   -420.29056   -338.92255   -411.39333   -403.43114   -339.50695   -538.74769   -465.01548   -457.47112   -347.07161   -771.02179   -584.58196   -314.04008   -735.36799   -549.87811   -728.41732    -605.8998   -662.69006    -657.3621    -750.0293   -645.92092   -778.32019   -910.50501   -633.33135   -415.02079   -936.48644   -897.50194   -759.85167   -551.96053   -875.63193   -770.06743   -821.55941  -1074.11084   -877.92484   -823.66017   -549.95979   -858.18412   -937.86118   -806.20161   -775.33054    -866.8689  -1216.48407  -1151.30663  -1048.01844   -920.02413   -821.92912   -813.41055   -839.46593   -764.83047   -979.45229    -849.2351   -894.04695   -997.23009   -729.40832   -768.83639   -820.58326   -523.48015   -847.21528   -876.07229  -1018.65689   -716.95237   -944.20507  -1036.45713   -936.67408   -722.85063  -1076.23015   -769.89379   -822.28721  -1056.00584   -859.56506   -857.70489  -1462.50443  -1163.63765  -1099.03442  -1137.20874  -1163.88848  -1479.54726  -1164.37349  -1091.90141  -1278.40038  -1158.64553   -1400.7592  -1114.89848  -1113.00271   -1452.4651  -1369.50447  -1366.29717  -1135.99116  -1093.46771   -1083.0822  -1410.86505  -1234.41007  -1532.71965  -1398.65084   -1127.6531  -1485.75221  -1477.20674  -1035.29995  -1512.41266   -1140.7597  -2080.84686  -1375.77038   -1472.4285  -1479.82917   -1445.2993  -1382.89164  -1226.37172  -1579.68551  -1228.11101  -1486.96356  -1446.79692  -1589.49009  -1720.00443  -1455.33174  -1468.53197  -1778.44467  -1431.68695  -1235.46956  -1811.10754  -2172.03607  -1584.30746  -2157.28143  -1850.17409   -1666.6394  -1446.07298  -1166.48368  -1286.07841  -1262.27461  -1423.91854  -1221.39911  -1549.01756  -1436.78553          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -4.00   -504.84036     30.72608   -675.92953   -369.41707   -422.53548   -393.35411    -183.7753    -247.4415    -425.6516   -309.25011   -222.19034    -84.44807   -407.26994   -246.10444    -158.2725   -212.29501   -538.00981   -366.59657   -357.68134   -313.67009   -219.74585   -312.30254   -326.31416   -270.22646   -177.45493    279.29074    1238.5888   1676.23759   1616.00511      395.562   -177.75522   -333.19407   -309.29631   -539.09534   -428.02468   -180.14235   -163.11917   -447.06305   -412.27708   -439.35366   -454.91565   -363.07362   -373.88738   -517.27295   -307.03213   -378.71499   -422.99317   -575.62403   -336.74811   -474.95331    -206.4462    -89.74236   -488.76225   -184.74294   -416.18451    -73.98197    221.68359   1142.64642   1713.59423   1317.67485    545.74387    113.26825   -272.82016   -284.94171   -513.91863    -451.9231    -282.1921    -48.39925   -288.44596   -115.68045    296.46527    925.38358    1399.7892   1130.50417    706.80727    438.60009    577.52226    302.14084    107.24267    -91.52032   -284.39387   -228.16619     -367.111   -259.44389   -392.00677   -390.18657   -371.38613    -317.0941   -379.64359   -338.00203   -151.51245   -218.87696    -12.86817   -240.96538   -282.40023   -331.27851   -399.96197   -185.13927    -213.8864   -488.90931   -372.25394     -59.9395   -147.22893   -148.34892   -144.12435   -266.70446   -376.20661   -377.62597   -435.94207   -359.24495     -488.158   -412.08831    -356.6617   -286.97311   -254.92854   -318.11731    -63.15209     74.58237      55.2568     29.24183   -165.01945   -253.22314   -198.93794    -30.64262     16.87986    300.30843    369.13862    192.65322    231.17965    135.54281   -280.16632   -181.90171   -257.82848   -317.30206   -302.19841   -316.15405    -70.53382    521.04232    922.84088   1030.71221   1253.31696   1081.98573    515.73918    176.19117   -161.34332   -363.06468   -478.00171   -341.20049   -306.93762   -369.65297   -417.00905      -513.62   -508.22982    -526.3207   -420.09232   -474.88163   -388.79088   -301.49992   -376.78748   -324.22725   -272.14036   -344.62691   -339.02509   -244.94895   -215.88874   -289.05245    -148.4843    -31.52176   -179.12359   -139.61489   -300.25915   -291.11592   -455.92084    -306.5203   -509.71692   -321.51692   -207.92285   -328.20783   -425.87409   -334.43534    -259.8873   -423.50553   -372.15724   -461.94052   -530.07093     -447.008   -431.15166   -425.10379   -395.25944   -425.87686    -359.7622   -319.80058   -334.79806   -277.01134   -254.58057    -20.67149    -123.1133    -47.94401     82.23399     -8.31148    143.87564   -126.01212   -162.30096   -125.69546   -170.16273   -193.07017   -296.27045   -217.85936   -112.88643     48.59955     -8.23724     25.07753    -94.77038   -412.09922   -373.37614   -459.22609   -549.57796   -422.71903   -429.71117   -488.54803   -449.51293   -362.62476   -345.77565   -333.82434   -395.61331   -403.83655   -220.27681   -195.54341   -182.03943   -191.34903   -182.27891   -169.31076   -248.49379   -189.39677   -280.84654   -398.54711    -124.3662   -156.93184   -212.47426   -192.48602   -258.81799   -265.10004   -263.69008    -94.05974   -362.24171   -612.97481    -530.7909   -593.26075   -517.80761   -582.05068   -417.23334   -537.89705   -401.85311   -482.40332   -342.66554   -373.86279   -307.18902   -297.79286   -275.00317    -100.8397   -108.14723    -87.24907    -22.60888     -7.61069     88.86966    127.93443     83.20351     38.80922    -39.17896    -83.99645   -431.29893   -514.20569    -456.2935    -414.0096   -461.03847   -525.95058   -504.61669   -460.22943   -482.95444   -564.45204   -470.66101   -433.23618   -380.99338   -294.60153   -422.03062   -483.92104     -456.592   -471.24618   -526.60514   -552.51718   -586.03928   -571.26604   -614.70957    -538.4244   -566.13261   -564.97532   -670.88084   -520.06808   -654.28225   -684.31708    -641.1168   -736.57492   -526.83443   -685.94431   -680.26453   -503.15688   -653.40714   -438.93167   -486.51137   -564.26109   -699.35516   -484.70938   -702.05539   -346.85231   -515.03182   -526.87132   -459.38528   -443.92359   -375.16332   -585.32249   -594.16654   -562.23184   -692.06741   -694.26497   -621.14514   -627.04118   -433.66815   -519.48786   -538.18512   -477.62745   -749.11187   -516.18268   -567.44594   -629.76568   -484.87236   -452.48821   -314.06538   -470.48554   -453.39275   -505.61287   -755.99519   -503.74488   -709.87888   -504.89076   -433.01594   -695.00413   -557.62594   -511.38606   -512.63632   -816.73342   -637.59932   -506.18556   -471.68085   -433.55258   -504.24198   -327.90449   -395.06153   -260.77507   -347.39981   -211.57988   -255.12186     42.55804   -208.95682   -563.16657   -117.99371   -135.82218   -146.90139   -377.00785   -418.02646   -590.17609   -365.09814    -526.0012   -740.74022   -446.52165     -504.929   -659.13637   -478.76322   -333.61285   -710.19529   -334.61946   -481.13903   -604.89239   -748.63103   -773.45449   -644.15718   -566.79322    -448.4499   -390.71602   -765.94649   -620.95642   -562.15302   -114.48201   -656.98065    -852.5002    -713.0353   -803.37615   -724.01349   -562.10119   -919.37667    -921.8574   -815.46706   -885.60574   -921.42247   -802.76091    -382.2132   -746.91875  -1116.35895    -852.0743   -629.27459   -814.52486   -613.65159   -682.18954   -750.18752   -944.62561   -600.26601   -781.60477   -645.71456   -663.76851   -738.82858  -1036.94542   -622.73326   -812.19633   -697.70525    -687.5698  -1021.19168  -1204.17742   -745.98205  -1010.79001   -879.89102    -846.1223   -778.22656   -1052.6596   -933.73247   -1034.8492  -1000.02095  -1160.02782   -770.16049  -1077.71102  -1140.31504   -913.64848  -1230.77697   -1144.0046  -1149.80962  -1175.04878  -1354.21994  -1158.63509  -1202.69067  -1240.97706  -1205.32847   -1318.6393   -1194.0833  -1429.28907   -1280.5178  -1047.50902  -1283.47484   -1374.8905  -1512.56471  -1203.00506  -1321.95891  -1041.11484  -1659.58536  -1526.89839   -1024.2462  -1443.87476  -1374.11996  -1497.74919  -1450.09896  -1613.79313  -1444.35248  -1425.54774  -2016.14472  -1636.07757  -1538.00348  -1587.81002   -1771.8734  -1227.40808  -1682.39014  -1599.16168  -1721.24296  -1374.26633  -1614.74575   -1796.1709   -1116.5725  -1465.89769  -1652.90624  -1568.84478   -1085.4151  -1070.33646  -2070.48705  -1261.69075   -1134.5549          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -3.50   -558.27829    -81.08185    -565.5065   -304.81377   -463.73709   -333.94633   -265.60494   -101.68413   -292.25269   -487.66596     -88.9744   -113.10097   -361.52027    -218.8043   -116.53148   -394.31771   -486.68331   -341.78088   -343.13769   -260.88125   -188.46241   -291.76448   -155.72166   -179.51411     80.29481    198.43163   1358.87409   1805.05752   1581.31863     425.2433      1.97972   -236.60611   -267.27889   -424.34017   -400.02202   -229.24577   -323.64212   -283.37365   -404.31205   -578.48213   -459.45692   -228.08487   -358.80238    -376.7749   -239.71291   -645.19707   -434.43364   -245.48779   -246.92624   -316.57985   -298.77328   -303.65945   -277.62471   -320.65097   -177.19321   -436.48508    137.53479    895.68357   1809.98108   1536.36626    643.23589     32.37181   -223.50494   -312.70894   -346.58284   -528.94496   -627.67146    -96.76282    -430.1182    -294.1064    332.48468    845.81673   1387.32271   1313.00232    807.19277    514.83372    639.29634    269.66301    126.98666   -231.07421   -403.23447   -524.04334   -295.15307   -375.62248   -440.55051   -349.84802   -315.45465    -256.7009   -317.12416   -301.96186    -28.23875      0.84057    -74.20125    -39.81086   -217.15797   -203.82264   -504.87945   -476.66322   -260.79455   -433.24815   -216.44588   -406.74925     15.48566    -49.74281   -261.00092   -166.02938   -441.08552   -558.11798   -367.59687   -341.79338   -440.23835   -342.92148   -319.14325   -218.82297   -272.91138   -347.23451      6.22549    238.57411     70.29123      24.4923   -138.47138   -191.94077   -357.91293    -91.48522    125.71532    132.56476    541.48237    467.09696      69.0198   -190.86706   -301.35279    -167.0341   -364.91982   -249.52759   -192.31308   -163.17884     55.58673    467.12606   1033.36145   1319.99763   1585.77566   1132.32822    774.13564    330.32056     31.61658   -165.64672   -232.08148   -453.73103   -329.29957   -412.91495   -380.53795   -413.82653   -516.56478     -399.546   -470.80297   -397.69685   -369.28887   -377.74289   -380.50026   -207.83579   -268.95563    -383.4781   -322.93904   -298.72533     -2.37082   -184.40747    -83.22858   -192.56469   -149.25093   -282.81115   -326.65136    -410.3206   -298.27219   -227.60314   -311.53926   -255.72078   -255.99353   -473.96457   -306.39491   -365.58748   -291.08153   -278.67934   -397.96871   -577.65004   -476.81251   -328.90895   -345.40996   -552.11542   -521.83086   -450.89922    -457.5131   -286.18641    -182.3261   -129.04607    -78.96497   -104.31457   -142.14932    -59.21055      4.27543     51.07025    214.29058    129.77328    -184.3121   -113.43833    -123.1259   -180.92592   -128.40162   -152.91089    -72.20435    152.84957    135.38406     41.11553   -159.99213   -303.26846   -417.89051   -233.41394   -451.22012   -423.91449   -455.71433   -422.22797   -371.17435   -473.58383     -470.634   -446.96476   -370.66221   -238.60627     -114.658   -146.16242   -128.46767   -171.58844   -150.55368    -85.87046   -194.38482   -180.36355   -187.29863   -189.93821   -166.15929   -106.46072   -150.16978   -156.34883    -146.3021   -288.12962   -179.53921   -196.41293   -412.91476   -440.38969   -538.92511   -626.41673    -681.0506   -429.27709   -529.69895   -591.80229   -313.66381   -353.58883     -384.559   -309.66228   -264.34967   -159.82749   -187.49018    -32.05666     -95.2073    -112.3373    107.06094     -29.4055    315.63764    120.10337     191.1144     115.9804     59.96734    -26.46612   -156.67679   -270.16122   -371.99982     22.65547   -371.24092   -460.35513   -395.36639   -455.86801   -269.04126   -311.89406   -296.08749   -355.25047   -430.79109   -487.84567   -386.31315   -235.78893   -380.01266   -589.07175   -380.85334   -490.48344   -468.49767   -450.54599   -467.44149   -572.18175   -714.91253    -607.9698   -501.76549   -614.09955   -623.01161   -523.36824   -557.13109   -470.78316   -557.44536   -633.70971   -656.72174   -526.18406   -598.02664   -411.21204   -407.48997   -487.52959   -377.29337   -342.10284   -410.18212    -500.5711   -374.05258   -416.58294   -322.42105   -243.03093   -452.52185   -466.52611   -393.15074   -431.42067   -517.24188   -496.99855   -551.22205   -421.90351   -706.50531   -452.20959   -428.57019   -506.38975   -240.00202   -506.31955   -385.22509   -494.35322   -331.87568   -337.88636   -499.58053   -205.55078   -327.56492   -432.76834   -415.26713   -477.09593   -485.44949   -584.88709   -477.73402    -570.3311   -657.66593   -416.42926   -586.40359   -504.24584   -352.22637   -318.13056   -423.83133   -363.76058   -306.90596   -380.49256   -165.24932    -99.13731   -138.57969     -84.7535   -235.97554     34.77652     51.31922   -315.56876   -303.24076   -440.73289    -310.8613   -442.41697   -341.48388   -418.36029   -525.82281   -226.26954   -168.61553   -381.14258   -296.00367    -356.9649   -357.03084   -425.50009   -326.60143   -245.44045   -477.43976   -344.81669   -378.22975   -173.19291   -636.99616   -348.22224   -447.75561   -450.57413   -392.56352   -713.47851    -454.4616   -628.14865   -602.79349   -742.37769   -505.45354   -399.89416   -453.73099   -758.30832   -375.21463   -318.75263    -529.6658   -729.80473   -536.50059   -649.98295   -716.45218   -624.34161   -911.11622   -470.66089   -799.97139   -725.29056   -953.66422   -806.78822   -898.75759   -468.75151   -648.08251   -803.55492    -649.1793   -454.08428   -631.11809   -515.30764   -604.68821    -848.7111   -612.22346   -766.69718   -982.20779   -741.30516   -671.75408   -983.96745  -1188.40187   -975.28278  -1185.67949   -993.09933  -1010.54068  -1059.48028  -1036.73203    -961.8629  -1346.99095  -1049.91625  -1090.32362  -1308.07876  -1260.27673  -1252.07062   -944.52134  -1325.26205  -1253.96046   -954.72679   -1157.2689  -1076.07773   -1129.1306  -1147.51118  -1393.69247  -1309.56155  -1373.20063  -1580.19082  -1220.72502  -1447.14641  -1499.37994  -1390.14842  -1524.13595  -1104.69147  -1396.91576  -1499.25867  -1526.64913  -1334.76208  -1603.58543  -1220.00999  -1338.60717  -1101.12544   -1316.1325  -1437.62134  -1408.39506  -1299.65308  -1669.30886  -1079.74307   -1286.4875  -1241.95193   -975.31143  -1750.62015  -1278.56892  -1305.12204  -1552.11424  -1308.12562  -1081.00816  -1347.18956  -1396.04128  -1446.93455  -1374.49636  -2176.96279  -1514.09468   -546.43312  -1743.66811          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -3.00   -599.67633   -176.86133   -465.76979   -296.10099   -506.87652    -233.5139   -284.99646   -131.60436    -246.3527   -607.46396     45.64676   -263.28841   -352.39243   -276.75451   -201.43634   -405.98711   -394.74521   -302.37215   -288.41268    -65.51851   -550.83211   -128.64999   -326.45132    -363.6921    -42.32477      249.673   1343.84894   2024.92512   1409.43898    617.51865     -111.804     -0.03697    -430.4281    -317.8317   -367.53178   -390.08812   -347.46466   -341.17129    -421.3909   -632.09034   -405.75131   -340.73332   -464.93526    -376.8877   -295.04155   -444.65702    -328.1979   -266.68535    -316.5769   -201.88565   -446.06994    -353.1914    -267.5695   -267.15002   -261.18788   -110.73688    341.38691   1113.74246   1779.77714   1377.03789    653.68527      16.0179   -298.53274    -276.1097   -277.01901   -267.05022   -363.59214   -450.55234   -191.45356    -36.01132    480.82272    687.12865   1486.62934   1260.46694    917.05193    770.19601    551.16572    210.41798    -53.13195   -304.11395   -292.98194   -404.12252   -296.48462   -357.48398   -368.87757      -72.803   -475.83076   -456.59167   -226.30166   -316.09662    -96.66317   -174.25919    -118.2774    -76.46708   -149.34526   -185.88928   -472.13894   -182.53627   -357.99342   -332.46666    -81.94232    -89.45918    -91.20666   -232.08518   -163.14461   -327.56736   -290.60535   -444.17995   -212.20128   -432.88315   -604.55958   -366.20158    -450.9724   -268.63513   -184.98913   -104.54861     72.64198    149.98924     73.55501     70.35462    -22.12134   -223.66123   -130.51103   -221.92094    106.24582    334.24267    385.93654    257.43929    340.21008    -14.23973    -110.5659    -296.7998   -316.48106   -338.99498   -362.13955   -346.55977    188.60712    524.21774   1222.38733   1562.15094   1689.35082   1144.91309    902.62325    484.47414    252.67182   -183.33591   -225.47515   -167.91495   -405.06414   -351.77913   -312.97371   -228.46172   -365.20143    -514.5088   -393.02603   -331.83626   -375.43837    -349.8862   -210.68598   -141.33548   -238.88089   -419.68328   -169.09369   -315.94781   -408.10187   -214.16971     70.43843     41.98339    122.16914   -159.47407   -347.84754   -432.78137   -327.36201   -381.52553     -373.808   -217.94493   -292.03349   -273.60879   -248.08461    -317.2621   -335.15256   -256.09932   -525.47417   -354.17266   -580.45512   -355.35478   -470.91171   -404.87969    -437.0994   -302.89825    -306.4526   -207.48471    -80.98328   -262.27475   -197.21501   -104.11766    -64.04522    -67.62086    172.92693     -4.56734    175.05696     38.07391     86.87954     68.63085    -50.27863    -28.73464     -67.4064    -35.48829     33.05913    158.76844    -89.53414   -221.48484   -263.81643   -240.62721   -277.88343   -288.61966   -568.48249   -322.06474   -466.60778   -487.82054   -289.56557   -271.42372   -473.37302   -293.64126   -367.01878   -257.30685    -85.01192    -92.12189     20.51278     -65.5328   -388.51747    104.05578   -201.50377   -182.55438    -160.6419   -204.73352   -135.33634    -86.22638     25.64866   -244.22218    -77.35887   -311.05517   -200.32617   -278.98059   -298.31798    -512.0364   -396.95967   -309.52792   -692.52512   -434.51656    -391.7904   -430.02157   -309.77666   -467.20213   -251.76929   -274.76736   -161.99315    -188.7704    -82.86695     66.02312   -135.39529     55.38618     76.63005    148.65382     -70.7025    341.74442    140.24038    212.94051     -33.9273    126.70609    -20.42078    -13.90931   -296.29758   -316.04905   -364.05809    -373.7969   -397.84584   -218.71575   -467.47192   -446.71449   -395.36039   -381.88676   -249.43499   -350.21304   -364.25696   -371.01162   -292.23141   -539.77417   -552.45034   -557.49772   -360.70275    -531.9573   -597.46108   -709.87773   -622.36545   -649.84101   -745.31842   -712.21474   -518.50228   -722.48929   -484.66155    -341.6519   -464.08601   -630.74291   -579.38724    -433.4082   -435.77183   -441.16181   -436.31263   -432.12026   -286.76414   -225.36679   -507.78414   -532.19189   -365.34706   -302.13484   -259.70382   -332.99211   -505.09646   -298.35485    -700.5334   -384.38509   -423.83598   -247.09381   -617.15918   -441.46574   -456.27399   -223.04789   -451.18093   -282.94582   -289.06823   -598.00514    -261.3078   -229.77904   -233.66371   -405.18239   -431.18266   -438.79497    -305.0306   -585.01169    -362.8788   -487.39182   -531.94383   -503.94616   -463.37172   -376.56318   -441.86581   -375.93145     -411.614   -605.24497   -200.35978   -207.95237   -377.75834   -286.59426   -265.38701   -387.93861   -296.55498   -300.98172   -151.47668   -333.84333   -136.15283   -313.35215    -84.93906    -35.19992    -63.05888   -333.22545   -130.22743   -345.98077   -311.50927   -172.89616      8.17845   -211.57718   -361.97963   -422.01753   -273.95306      -273.91   -249.08655   -484.40409   -316.23493   -371.58725   -390.91796   -272.25613   -453.82628    -294.1896   -349.66086   -415.49819   -483.31193   -364.13345   -524.55174   -275.16986   -646.59121   -506.54599   -606.30201   -746.30313   -473.86392   -467.23483    -509.6983   -528.92801    -610.9747   -475.53868   -563.56109   -230.66477   -399.06799   -567.88803   -305.45636   -444.19773   -669.63248   -378.68294    -517.3154   -684.58757   -709.94053   -421.90308    -277.2286   -654.22254   -334.87155   -611.77522   -486.86088   -609.03816   -485.27314   -647.91405   -636.24203   -600.05284   -581.30498   -622.05874   -839.84341   -672.12245   -738.97616   -477.25232   -847.00351    -590.9212   -856.84821   -956.30722  -1053.91952   -961.94694   -943.59472  -1023.62607   -956.07065  -1247.24862   -984.17523  -1392.90757  -1459.74858  -1040.17516  -1166.79808   -947.55078     -674.882  -1176.24624  -1210.24307  -1008.08988  -1798.31681  -1303.01524  -1225.93762  -1371.25734  -1523.68844  -1572.86856     -904.942  -1287.43513  -1089.89999   -1205.1941  -1204.07973   -967.13732  -1164.86597  -1115.88474   -939.18629  -1210.66788   -797.21547  -1066.57629   -1331.0194  -1285.49979  -1469.12857  -1622.34966  -1691.01836  -1059.47799  -1234.99825   -1358.6864  -1573.85516  -1294.36739  -1667.26522  -1327.51176  -1394.80138  -1299.53592  -1790.04636  -1472.49962  -1441.86958  -1153.98079  -1753.23997  -1783.23564   -1805.7201   -787.91845  -1483.97226  -1277.15696  -1125.71098          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -2.50   -522.28337   -249.38938   -406.34261   -307.89105   -546.74031   -132.61797   -299.34316   -276.34541   -333.54166   -454.17591   -162.38862   -234.99878   -356.25253   -277.57837   -282.86118   -356.43952   -279.90219   -206.31845    -380.4715    -306.2586   -375.30666   -225.78266    -310.2011   -392.85254   -192.53883    450.01639   1290.95438   1806.85312   1527.04657    717.50896    -81.77693   -193.29412   -452.36265   -249.19089   -373.20705    -346.2657   -346.41728   -275.33775   -310.28326   -519.98946   -442.57345   -191.47114   -397.69227   -307.85407   -365.60894   -393.07918   -397.46883   -220.40014   -107.53978   -409.82605   -495.56877   -287.87283   -365.21659   -281.10105   -388.54756    -98.76778    198.12354   1066.21209   1670.26384    1548.4754    776.87948     14.50433     97.89947    -403.3453    -322.4631   -356.94164   -338.79028   -269.34033   -392.01112    -84.74454    254.74697    959.57428   1475.20838   1372.47397    927.42747    512.45095    723.33608    484.70724     35.40872   -141.48918    -412.6683   -530.16418   -492.30301   -233.84695   -326.01565   -398.20223   -461.43501     -515.141   -182.06367   -147.44548   -117.90739    -44.82947     17.54119    -59.97213    -255.4393    -10.98037   -313.87443   -227.08584    -403.1945   -365.77804   -167.82212   -152.96746   -115.95918    105.88979    -192.8032   -270.73131   -315.82704   -277.95997   -425.51378   -320.63641   -402.35577    -510.8825   -535.23169   -232.29901   -145.82703    -265.5191     43.20886    483.51212    206.98445     137.8683     75.16146   -168.66887    -218.1995     58.86203    167.67989    151.00783    333.47632    420.01681    355.26434    -27.24924   -176.63225     -367.575   -314.90817   -258.93098   -284.83924   -135.40636    107.17695    709.74215   1360.26211   1696.69777   1661.72334   1288.27777    685.85225    352.87255   -132.10748   -189.26092   -271.64984    -37.31131   -282.95236   -241.03354   -221.54345   -647.12384   -450.30377   -497.52889   -307.64783   -337.75352   -269.80585    -279.3639   -274.18039   -227.94334   -119.58128    -132.4392   -291.32964   -172.05525   -291.78772   -161.72358    -82.65319   -196.41435   -121.32252   -217.08224   -322.12877   -175.91983   -335.28855   -237.47655   -372.14035    -244.9794   -378.44474   -167.38156    -314.5252   -164.84847   -157.11782   -293.75662   -273.65801   -359.27671   -288.82867   -240.78846   -215.66795   -420.96923   -445.26852   -362.70239   -313.26701   -173.13048    -164.5622   -181.80119   -151.58757    -95.67595    -149.4233     60.76322    253.73742    189.95477    163.55911     26.06453   -152.30825    -33.78934    -43.81498     37.30115   -136.36875   -117.66054     85.94227     41.87677   -149.77901       65.453   -330.45265   -287.75888   -278.05352   -342.12106   -447.92286   -300.40516   -339.23587     -501.005   -431.33701   -324.57721   -176.57065   -264.05502   -216.31785    -36.37137   -217.44894   -158.94293    -82.86721   -130.53929      4.17174   -112.63347    -78.74455     55.00281    -84.19232    -53.76101   -254.19325    -74.53917   -139.95303   -122.73232   -255.67516   -198.98032   -234.06664   -235.38141   -337.00039   -415.60631   -446.33145   -346.80577   -557.84601   -522.93711     -351.685   -469.51894   -374.16744   -419.94385   -171.78857   -223.78489   -300.06238   -360.05698    -33.15927    -23.01863     -75.1527    -48.57804    257.23373    251.06131    351.49681    304.31051    402.10846    445.71949    123.53248     38.83818   -109.53373   -151.11683   -253.16317    -47.84762   -439.85591   -415.25761   -324.31813   -290.62612   -327.32018   -315.63856   -182.46135   -259.98571   -215.80828   -232.97074   -269.36296   -212.88195   -234.85956   -320.06849   -429.61591   -260.86961   -528.17464   -583.15812   -605.79448   -354.70342   -523.52577   -566.39412   -499.70718   -593.59408   -605.26767   -407.26066   -334.88669   -509.67723    -620.8459   -407.63021   -484.57444   -664.20882   -740.84629   -402.65072   -316.31391   -319.67384   -572.69867    -258.5165   -435.53231   -244.72098   -311.47726   -336.63988   -253.23436   -273.71351   -380.90758   -393.01493   -359.80875   -439.86261   -289.95822    -354.2852   -484.96778   -651.26008   -650.08069   -482.69478   -589.09472    -213.1179   -429.56255   -185.77899   -381.08667   -399.14351   -204.84647   -322.35955   -443.94658   -371.46783    -466.5227   -344.25202   -279.17147    -439.7675   -173.96163    -97.78023   -365.43372   -400.07872   -509.60378   -181.87257   -485.69108    -579.4681   -292.47398   -368.56982   -270.13056   -430.84076   -176.94678   -103.27293     95.62462    -48.82132     23.02833     19.34632   -193.23022   -241.61018   -197.68625    -83.19505     83.83624   -188.89441   -131.11258   -118.54561     -194.774   -270.46226   -130.34135    -56.33747   -394.46386   -225.25076   -228.41355   -150.52578   -332.87045   -474.85742   -256.73638   -230.53438   -328.43842   -247.32643    -68.33395   -202.09033   -327.01747   -495.32871   -481.38016   -160.94572   -425.53661   -373.96768   -324.42893   -343.99272   -451.23496   -476.42854   -687.61293   -350.47947   -437.95868   -418.18792   -402.95533   -480.53405   -207.95982   -567.68746    -328.9461   -376.75854   -571.41782   -413.96539   -502.68328   -629.88576   -477.68977    -742.0861   -758.67854    -548.8146   -435.23108    -581.6512   -660.06372   -538.35489   -229.98969   -399.77453   -319.94574   -724.85219   -768.80767   -302.05143   -923.17676   -270.89584   -646.36789   -441.05447   -322.63227   -781.33292    -459.8553   -513.67139   -580.02635    -1058.246  -1182.20735  -1173.05449  -1414.17174  -1121.55429  -1011.97556    -953.0819  -1042.26706    -746.7904   -970.37231   -1240.6842    -944.8328   -800.50295  -1219.81932  -1086.50192  -1502.11497  -1036.87123   -947.22333  -1308.46193  -1143.42188  -1194.53969   -1321.1474  -1122.79555  -1256.29404  -1431.63852  -1094.68759   -1417.8801  -1376.83538  -1349.45219  -1100.68776  -1263.25487   -1643.0563   -1217.3918  -1209.15078  -1182.46832   -1608.3891  -1280.51112  -1279.90284  -1016.46844  -1796.74176  -1336.93607  -1344.10443  -1414.19185   -958.87744  -1146.79972   -932.55056  -1427.36752  -1525.41364  -1893.66397  -1306.30576  -1570.75136  -1406.42452  -1082.66027  -1600.60269  -1229.75016   -478.61625  -1390.10095  -1801.14245  -1310.84973   -912.89398          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -2.00   -444.78866   -317.27965   -372.69715   -357.93588   -418.82922    -206.4302   -318.90968   -312.08137   -386.11206   -355.46897   -351.66096   -163.31989   -370.82874   -197.57301    -367.9069   -357.94625    -509.9055   -241.03121   -421.27401   -283.91383    -317.4805   -363.55425   -232.41279   -252.72408    -93.62024    662.30194   1160.69938    1821.3938   1371.82233    528.68634   -100.09886   -361.44181   -401.71198   -305.55718   -433.02741   -327.81416   -371.82963   -360.28826   -514.47658    -420.0173   -354.22414   -215.14788   -428.22141   -273.02655   -641.97879   -379.70048   -509.37138   -456.45273    -287.2359    -348.8349   -380.37116   -453.88985   -357.28232   -410.64316   -405.96253   -421.21867    316.96084    954.27681    1413.2706   1321.06764    814.48637    120.68478   -178.41522   -234.40445   -292.68266   -242.54558   -360.40823   -138.53764   -250.56667   -182.07105    392.52991    941.98838      1404.61    1254.9769    736.80467    795.55967    581.59918    447.99609    123.20414   -223.48901   -144.63809   -138.29541   -173.91488   -570.74925   -253.14004   -274.97584   -328.31076   -388.98495   -357.53595   -367.70782   -224.12934     26.18126     -5.50407   -131.28707   -193.95793   -355.87651   -435.91745   -476.56399   -403.12506     -315.337   -302.93332   -103.81011    175.69412     99.94546   -234.06151   -297.81024   -409.32124    -372.4167   -296.18895   -377.49076   -607.11546    -432.1016   -229.60262   -347.48335   -118.04485    -187.6459    135.98726    217.96617     153.0507    234.78412     74.41128   -166.91522   -258.09116    -59.96457    134.68255    417.39833    449.38894    533.23116    281.54238     82.52653   -139.58623   -168.51041   -377.26124   -406.54517   -245.97727   -119.14882    131.98332    614.86932   1181.56483   1724.82851   1810.06477   1451.13914    923.04787    461.88762     58.66591     25.35059   -265.68842   -128.56517   -259.81332   -272.83595   -344.58214   -227.27545   -449.73546   -442.07181   -256.85315   -327.78036   -377.13517   -326.17748   -232.97731    -91.42632   -130.69507      -9.6596   -122.29532     -245.559   -396.62258     -75.7702    -45.02357     -27.9915    -95.36812   -172.68934   -336.99515   -243.49083   -381.99191   -222.59976   -379.74138   -270.26497   -164.33385   -144.40464   -212.23427   -242.51429   -266.58627   -272.55909   -295.12263   -425.94385   -382.45365   -354.58025   -203.65333   -483.07181   -442.25659   -468.13422   -195.15524    -248.6991    -39.84575   -147.51365    -52.43321    207.49422     47.54098    -61.40936    223.95847    218.54313    213.56929    -43.70851     98.08597   -174.09142   -208.73859    -37.77231   -176.08057     -59.8883   -238.47703    -78.63045   -129.54399    -70.60929    -72.43687     -158.598   -235.94919   -265.06708   -382.51998   -481.49962   -421.74763   -352.82687   -200.17189    -434.8983    -241.1004   -342.21875   -428.40156   -369.31699    -80.30578   -114.85333      91.1327   -228.68886     12.83711   -152.58759     -16.1067     23.16779     15.29464   -119.03199   -138.62943      82.5765       53.713    -38.27064    -31.17932    -68.72463   -252.61987   -150.73908   -104.89977   -574.14777   -335.14866   -483.66145   -428.73299   -364.01122   -399.00208   -480.26427   -384.18649   -414.05726   -252.37296   -215.61951   -110.16169    -69.93789   -114.71347    -12.04549      75.6782    360.51454    251.73915    286.44445    400.37568    518.55574    358.84175    278.69771    217.66588    214.42466     53.86032   -237.68354   -238.37816    -285.1788   -291.04648   -188.38936    -145.2645   -226.97491   -289.34896      -230.91    -91.43937   -299.00301    -84.91365   -158.57876    -112.1707   -336.89254   -190.46891   -275.08399   -243.08253   -519.19816   -417.30365   -451.51168     -355.357   -533.96172   -541.29321   -539.52212   -368.55969   -474.58501   -360.33137   -465.20819   -712.84751   -363.22614   -415.85023   -568.10967   -438.69147   -347.56915   -344.88628   -359.00629   -225.42065   -328.71685   -294.35726   -251.20872    -60.14746   -301.46896   -201.75927   -173.49243    -43.68394    -53.40005   -143.51709   -330.60593   -345.98932   -275.32496   -240.79218   -425.64846   -274.77122   -351.80605   -213.04602   -333.72287   -385.02997   -432.79711   -193.61958   -445.88594   -103.29871   -268.72388   -364.72782    -262.7559    -60.67112   -319.45011   -270.95471   -432.46043   -380.18275   -364.17936   -365.57279   -243.49088   -296.26806   -485.42971   -457.58845   -385.95591   -437.28652   -149.66929   -289.60985    -380.5703   -307.15679   -195.73661   -302.22005   -155.73214     51.69054   -137.23929     44.87245     61.03059    -95.22323    117.50241    -81.68732    254.07856    -65.30188    -79.81212   -122.54836    -65.11634    -74.32683   -191.76076   -298.80267   -250.19723   -334.69084    -370.8399   -239.30502   -342.64506   -280.24152   -250.02675   -271.81413    -393.5026   -260.17925   -213.46982   -134.89444   -291.50351   -230.29308   -316.92907   -227.16129   -332.62967   -403.60924   -239.64325    -417.7396   -572.32589   -407.56346   -263.51137   -269.50293   -317.94594   -631.41336   -285.44641   -423.02288   -186.46819   -537.81025   -334.70592   -309.29639   -341.60721   -517.99258    -247.6301   -625.46334   -352.85079   -384.29914   -272.02634   -434.84298   -431.55291   -509.74324   -315.08543   -527.58309   -453.88317   -341.08925    -578.1631   -669.14656   -745.95348   -504.54938   -597.58815   -411.15773   -504.29568   -666.70146    -572.8622   -708.95082   -593.92645   -783.17807   -579.24286   -821.66267   -899.27361   -791.26854   -672.57918   -981.75107  -1041.13544   -793.29486  -1095.38634   -706.62935  -1038.42239    -766.9092   -1160.7883  -1311.67499  -1023.52564  -1201.10985   -903.74546  -1055.04213   -962.40266  -1361.74142  -1415.33322  -1179.04241   -837.28372  -1369.84839  -1215.47335  -1229.01662   -811.27714   -1393.6616  -1382.76644  -1243.67126  -1184.21656  -1183.49842  -1270.40637   -962.26925  -1251.35989  -1306.03026  -1044.02383  -1199.63617  -1734.09114  -1374.43159  -1251.09672  -1177.79944  -1203.25292  -1372.44469  -1553.41449  -1433.81545   -884.18513  -1339.08272   -1690.8182  -1417.43819  -1605.52005  -1507.60667  -1184.43567   -1581.4686   -1421.9889   -972.72942  -1265.66327   -745.45435  -1025.43509    -261.0144  -1209.08237  -1525.72639          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -1.50   -391.80169   -380.30751   -333.67322   -402.14969   -292.57987   -306.68334   -344.63646   -242.69956   -386.68315     -446.128   -254.97928   -297.20624   -282.60185   -226.39313   -349.35687    -333.6828    -600.1357   -312.81442   -418.38733   -110.68052   -426.26901   -467.53737    -302.1198   -298.58458    -23.73733    150.20335   1491.94787   2046.18229   1390.89656    547.02281   -128.31609   -218.06733   -383.39674   -290.36377   -504.73512   -355.08233   -227.45822   -447.78969   -227.82623    -58.58099    -389.2966   -314.41499   -444.38954   -326.94705   -573.70547   -407.62324   -406.27031   -397.31524   -434.15376   -348.19517   -350.27828      -357.87    -347.9253   -376.50415   -196.62612   -129.87251    190.05342   1089.34065   1784.91308   1316.93496    774.20349     35.36802   -175.99198   -185.66276   -435.24769   -322.25753     -359.732   -307.86766   -193.12061   -107.81254    206.27996     945.3466   1508.94947   1233.87503    999.02033    617.25505    559.62131    669.76044    260.54793   -195.07063    -76.41342   -388.18638   -283.27636   -331.21268   -410.21626   -338.64474   -254.18386   -176.18497   -333.48184   -144.01344    -96.90999     -6.31989    118.12442    -32.92769   -217.69388   -346.32209   -498.20385    -309.1831   -308.43167   -496.11297   -230.91579    -34.18688   -181.81855    -95.74018    -46.09815   -270.15352   -312.96617   -510.21114   -473.75217   -375.66898   -396.28519   -299.43468   -421.85293   -361.45444   -232.83027   -181.63169    158.43893    421.84625    270.79709    119.05107      9.15413   -199.57838    -90.35669     -99.8994    150.44157    487.22336     426.0903    548.61963    296.45963    123.98842    -67.24939   -224.14611   -198.31918   -257.18251    -251.9057   -105.74581    160.00896    724.71124   1449.48333   1602.72296   1946.69658   1560.91519    904.73787    525.51603     65.61351   -259.42006   -204.64772   -348.73797   -228.08477   -322.41779   -359.71946   -357.47395   -337.81112   -382.53011   -413.88313   -384.98156   -306.84214   -239.23725   -194.50818   -146.75224   -143.47042   -211.39483   -186.79739    -89.76392   -112.46208      1.09309     50.13028    -10.83535     76.40304   -202.35043   -309.62386   -230.67253   -355.94632   -470.14908   -390.86338    -142.2115   -186.93024   -200.28016   -163.07104   -169.65827    -76.52936   -363.35076    -320.1956   -374.36622   -375.28706   -311.80128   -304.05535   -296.45041   -418.47867   -162.11004   -225.52692    -151.0316    -92.33374    -95.72303     -8.24312   -235.78564   -146.58977    107.46188    156.22237    223.33248     67.17054     42.87116     67.16447    -77.52716   -123.89105    -223.6171    -84.37188    -34.16975    -72.34056    170.72184     41.92923     11.18052   -209.97906   -175.09559   -249.85319   -310.27461   -358.67518      -149.12    -314.7037   -400.45601   -200.18418   -285.21802   -286.73326   -326.12101   -318.51319   -199.54641    -91.02298      88.6439     38.98531    -35.75282     18.23773   -163.10969    -55.01477    -39.35462    -47.53023   -178.06419   -109.36579   -153.89647     -3.94921   -202.78953    -60.09151    -68.64669   -186.35534   -257.12708   -289.52959   -331.24402     -479.191   -294.73611   -435.10218   -412.76112   -349.12828   -302.76545   -349.43727   -294.51567   -345.14999   -139.04356   -233.38673    -71.61721     -15.0551    124.86403      45.7319    195.50165    283.75625    294.38527    473.90771    495.04819    414.29383     441.2259    357.09449    266.22892    144.30596    -55.48431   -184.03313   -150.51075   -309.21739   -275.05165   -275.57844   -167.15611   -127.31083   -226.56112   -351.34501   -147.93113   -166.54745     59.12485    103.88402    -61.20722   -203.24902   -277.60805   -354.25067   -533.56476   -388.09976   -524.36761     -251.699   -371.72337   -532.73925   -484.94157   -356.10669   -465.39979   -413.62545   -466.14222   -518.94173   -396.18105   -307.70946   -506.51848   -336.16499   -286.13746   -411.78573   -428.01863   -265.33763   -293.19938   -199.98899   -237.23069   -116.09348   -312.69377   -303.51387    -16.27081    -66.04776    -340.6366   -181.78553   -129.63757   -252.31048   -226.94592   -346.24846   -219.23572   -321.85221   -305.31517   -267.73166   -430.22633   -295.22393   -216.45385   -394.67401   -124.60388   -367.23003     99.85707   -317.99011   -288.19816   -183.56181   -260.50509   -138.32306   -400.29789    -78.78577   -246.89562   -324.75575   -287.24882   -356.06032   -213.28197   -326.72155   -140.41666   -345.99939    -95.23033   -292.92708   -216.13923   -191.09182    -65.48345    -83.73451   -101.22848        41.44     72.40225     97.89502      67.2259    214.32801     33.70265    182.97527    -27.08171    -19.14435     80.28583    -19.43382    -87.59661   -121.00186   -123.46908   -101.06845   -418.98599   -224.16411   -143.37022   -193.50228   -184.58135   -181.78767    -140.5339   -109.13102   -357.31129   -233.36436   -202.24707    -293.1081   -198.35495   -411.04366   -368.04456   -152.66186   -450.40308   -437.33304   -364.59484   -478.02582   -379.97989   -433.33095   -255.75852   -371.98308   -384.20141   -396.74173   -462.54758   -475.93329   -159.30783   -502.89396   -190.77569   -268.58553    -462.0801   -737.87005   -253.83364   -168.76432   -219.46879   -492.37047   -203.95753   -242.12585   -215.83701   -266.23449   -443.32452   -424.45406     -502.089   -510.19264   -483.74679   -287.11631    -428.6581   -552.13213   -362.81375   -614.66312   -637.40072   -374.72985   -818.59048    -381.7929   -430.09804   -397.82079   -571.27309   -752.70364   -897.24947   -746.58357   -904.27528   -587.89443   -754.35996   -708.51874   -754.34706   -868.15346    -490.4979   -572.14609  -1011.09527  -1247.55679  -1083.74137   -907.32514  -1152.94664  -1041.08073   -952.48444  -1017.38702  -1216.25313  -1370.58094  -1520.16144  -1071.29811   -1056.9419   -919.88301  -1160.80421  -1197.27442  -1084.99558   -927.48921  -1043.87822  -1135.22599   -879.59588  -1019.37649   -875.86692  -1055.57295    -1701.076  -1072.86429    -890.1157  -1431.27025  -1239.25939  -1386.74675   -617.56695  -1242.50289   -948.33624   -833.57438  -1135.91832  -1231.79389  -1300.14679  -1529.49572  -1681.32729  -1228.17394  -1529.72644    -957.0332  -1653.01917  -1019.70684  -1284.53919   -283.92722   -2114.6877  -1698.57973  -1142.29013   -484.11492          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -1.00   -427.01356   -385.16092   -294.50626   -412.07509   -210.37673   -293.09968   -291.86265   -248.24531   -370.63737   -503.09516   -192.21711   -409.40875   -165.68375   -301.64971   -253.97201   -276.13428    -469.2938   -367.99963   -309.62576   -155.36192   -421.67968    -346.4513    -254.4748   -196.53492   -182.06978    137.94315   1410.74858   2042.34908   1492.03092     614.2787    -87.16051   -145.56784   -369.46554   -127.16599   -371.68282   -304.89359   -353.36006   -404.43327   -368.17239   -218.63386   -309.00873   -462.22867   -398.65156    -362.1008    -261.9957   -364.69438   -408.85062   -403.54336   -376.99092   -199.63017   -363.31656   -290.81456   -272.56376   -291.03335   -262.15892    -112.1173    400.77503    990.61884   1606.40709   1446.18365    566.10798    149.79635   -175.43405   -257.51188      -266.74   -446.59482   -280.67603     -217.904    -180.7942   -322.44486    139.18813    956.26171   1433.81318   1238.98612    616.51322    589.83638    642.10185    350.32678    201.02203   -186.43656    -83.51476   -454.85839   -243.67986   -298.74029   -379.10409   -219.12624   -266.82598   -341.12792   -282.77545    -209.0246    -51.76472    -12.27143    -55.72063      5.80521   -174.95009     44.93899   -496.32551   -428.44925   -361.24506   -536.64499   -188.70876    -57.55304     65.43812    115.51362   -295.64785   -259.35098   -170.60899   -402.27373   -478.04478   -439.03637   -394.89908   -412.40796   -495.69124   -409.88953    -135.3088   -135.34505     88.91287    243.16694    241.71353    222.71975    -12.36304   -127.88941   -147.53153     -22.9836    150.34925    426.40021    460.98503    551.11866    277.00605    284.16896      2.20281    -181.0514   -195.03618   -193.93051   -269.48643   -251.37982     75.09343    855.79079   1470.80263   1832.47418   1921.84204   1548.18887    965.22774    555.09429    124.95342   -177.80076   -166.60837   -313.52896   -273.36353    -342.5109   -321.94204   -385.66871   -396.61228   -251.04773   -282.65957   -323.62337   -187.70433   -239.10357   -232.52403    -85.70439   -237.33728    -43.33472    -93.29139   -114.76606    -88.39546     94.99126     64.17909     36.56944    -24.28877   -144.16321    -51.82177   -241.36889   -330.36206   -290.46127   -191.51595   -177.31426   -309.94826   -224.60069   -381.34369    -181.4224   -217.19156   -220.42731   -282.56629    -246.0646   -319.27247   -364.90251   -420.44973   -417.45444   -415.55991   -373.41418   -102.80547   -244.55349   -173.55714     81.78595     39.57655    -84.59576     27.54619    -11.84835      37.5154    250.18553    247.68431     80.74785      1.90035    -104.3706   -176.38977   -134.01097   -169.53341    -84.66423      35.7415      37.3936     151.6241    -32.25597   -192.16147    -68.81161    -11.00613   -275.80486   -358.69549   -370.86392   -364.14345   -503.89721   -319.87622   -337.30429   -313.18489   -211.75788   -228.75209   -101.65935   -151.22863    -57.73275     71.42533    -64.44382   -111.76787     -19.7716     10.75937    -96.02494    216.74756     118.8978      -2.7159     13.75638      28.9329     -8.94029   -172.38446   -144.41874   -137.53111    -142.7689   -308.77592   -256.14591   -398.53325   -401.92282   -461.77605     -410.981   -273.63546   -336.73326   -291.22907     -147.639   -165.75869    -82.94323   -221.30613    -89.40865     50.86765     85.30807    105.60428    395.04397    462.03299    255.18553    607.94306    648.86195    413.25344    392.43948    270.53862    111.47374    206.97658     61.09644      36.6771    -72.06986    -73.27198   -167.84268   -195.81214    -295.8215   -146.46221   -228.74377   -161.34986     82.36135   -145.95602   -139.06825   -178.09834     -8.84439   -267.55468    -47.73214   -245.20634   -427.44635   -371.20832    -431.0898   -434.77494   -533.85527   -406.24797   -322.65878   -453.27199   -353.08774   -328.75023   -505.63948    -547.7421    -472.0718   -377.22398   -473.15385   -339.07361   -278.24444    -246.0926   -210.02416   -115.59864   -310.51076   -329.09522   -326.97086    -349.4321    -62.15864    -69.86267   -227.93313   -156.92417   -179.05233   -262.24357    -351.1265   -331.42486   -219.16452   -396.24777   -229.20189   -234.83301   -382.15174   -391.76972   -198.41209   -361.46914   -268.11328   -184.65277   -156.43646   -245.22287   -278.40643    -280.5278   -251.37587   -285.57769   -199.38521   -107.43518   -235.42011   -117.48651    -220.6816   -278.27964     -352.634   -299.37035   -158.77616    -260.6996   -327.32009   -284.38593   -170.95189     41.65956   -161.59346   -119.89815     -80.7121   -129.61728     -60.5529     233.6669    101.13934    112.70716     75.13166    -44.87317    214.96262     78.26878       16.337     35.02843    -13.60578     -7.92043   -126.27864    -66.39919    -72.81267   -123.43056   -144.43295     80.15392    -282.4417   -196.86987   -106.25568   -255.04572    -94.47609    -138.3377   -109.26771    -102.4712     10.18416   -185.51492   -147.07963     -78.7432   -167.54036   -205.86461   -359.24545     -99.6917   -310.39565   -337.69484   -453.25662    -415.6475   -287.13167    -241.0419   -386.34994   -295.17444   -512.15197   -414.63075   -452.91578   -436.03361   -402.86483   -397.45718    -331.9227   -346.92474    -302.3846   -554.93093   -627.78768   -405.83419   -221.29145   -394.12136   -356.41924   -463.22496   -428.77314   -289.68203    -486.2177   -471.90825   -562.27048   -611.56732   -483.03947   -265.87515   -337.47542   -728.26015   -593.37136   -733.01464   -560.76176   -538.74989   -406.06803   -614.01303   -526.78508   -708.35293   -761.64644   -524.35328   -758.47772   -918.96446   -800.40641   -575.08081   -520.72348     -915.172  -1203.22152   -881.72701  -1067.68507  -1033.40019   -903.93984   -927.43829  -1059.30475  -1413.14529  -1038.96159  -1231.31061  -1388.79554   -909.30322  -1156.65992   -864.83078   -897.03781  -1250.96753  -1003.00894  -1147.99672  -1233.45555   -1269.6824  -1234.23734   -568.67591   -678.97098  -1089.74712  -1137.00435  -1229.03943  -1192.10554  -1177.33101  -1041.63514    -989.7216  -1267.60407  -1377.89035  -1174.41311    -709.1652   -779.13764  -1657.56643  -1085.35009  -1526.04397  -1221.16572   -915.45203  -1156.70537   -698.95001  -1273.95622  -1553.62387   -462.46428  -1331.65629   -963.53923   -1126.0519  -1114.73361  -1254.33187   -671.06196  -1414.75643          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -0.50   -466.83131   -390.53625   -258.13958   -413.50787     -146.046   -198.77016   -150.00298   -370.30392   -315.17458   -444.79839    -235.6484   -246.00464   -201.26903   -348.34384   -321.07019   -384.60392   -389.08577   -410.38464    -266.9537   -310.42204    -413.6836   -404.33803   -300.34162   -149.57288   -198.48236    290.75484    1283.9279   1911.69504    1466.5901    668.85969    -24.88416   -358.63656   -329.05045   -195.13283   -444.08613   -360.80307   -365.22568   -219.83634   -359.03237   -382.58909   -370.55774   -378.46629   -208.01329   -471.03201    -454.9773   -310.87929   -471.21184   -326.45751   -313.60525   -161.70715   -272.27959   -572.20055   -235.73402    -451.9251   -235.83382   -160.65352    107.65419    1043.7118   1655.56036   1542.98535    500.79592     63.33415   -131.53738   -381.95483   -261.94969   -448.73036   -168.93203   -207.00892   -270.11055     -37.8398    161.84975   1105.88276   1368.30035   1354.94693    966.71126    660.82217    738.08628    458.95052    323.09431     21.02151   -200.32143   -341.44196   -268.14944    -224.5401   -358.40799   -362.51239   -349.97307    -299.8322    -389.6512   -238.28294   -107.78173     88.05283       -0.366   -274.88936    -327.7457   -184.60781    -291.4135   -279.18762   -401.48212   -539.92468   -252.30388    -19.05798    144.99205     -7.20048   -154.58331    -197.7666   -382.49276   -294.06264   -416.28349   -355.32432   -419.68292   -388.10705   -379.82887   -230.95007   -241.78583    -44.93647     53.11749    265.85673    237.69118    237.71972    -30.02866    -65.60218   -160.10174    -47.36318    197.41227    407.90292     564.4914     580.7759     290.3364     78.79045    -98.14958   -146.31137   -328.90409   -231.20832   -146.31366     26.56446    191.33428    829.07007   1600.09268    1955.9685   1965.73404   1712.74671    925.70017    499.65813     29.29205    -88.14453    -262.3106     -79.1687     -204.983   -250.38918   -478.27084   -327.95575   -283.22853   -222.87597   -349.33028   -321.31676   -337.04206   -175.98025   -239.01399    -70.48471   -263.97974   -211.82802   -202.76185    -91.66976    -96.48398    -76.73516      54.3652     51.25567    -89.35723    -36.03923   -168.57169   -135.51803   -168.96156   -655.62898   -347.23778   -220.07095   -128.51443   -294.97801    -91.17267    -79.07526   -383.88232   -239.54785   -319.77824   -390.23671   -338.14969    -425.1686   -317.83543   -388.71763   -322.52791   -391.39975   -258.11615   -213.52479   -177.20403   -117.64118   -130.13498      31.3147    205.84999    219.31537     94.82739    242.96394    163.47744    156.66155    101.81003     72.02149   -222.04014   -101.74118     77.66463    -30.52062    -88.72488     22.54825    -23.41943     93.33336      3.49999   -126.10816    -188.1385   -331.55635   -287.04238   -231.24749     -203.195   -315.02126   -304.84272   -378.64252   -204.86138   -368.14031   -278.69611   -130.39766    -89.87398    -32.05606     40.66949    -59.67008     36.17608    117.93488    -25.95467    -53.76791    -59.99063   -212.37577    -52.27109      9.01515    -89.72896     44.67289      1.82533     -96.4098   -116.92106   -357.64013   -158.73809   -313.16397   -115.69016    -302.2644   -330.96366   -289.68698   -266.84387   -242.58122    -95.05391   -145.71504   -275.63133   -223.88083    -99.50868   -185.46898    -12.33744    194.13733     88.60031    250.53515    553.49232    261.30354    469.62439    419.72449    506.85508    701.06032     333.1287    246.78439     99.16797     78.93907    -20.57867    -70.31873   -206.46839     21.54005    -54.36882   -196.94509     19.15909   -140.96877   -128.09391    -60.25648     73.89042    -30.79791    -157.0843   -191.78683   -225.50455   -293.59489   -196.09172   -161.93355   -311.78899   -334.76737   -295.15005   -341.14248   -444.35349   -559.50035   -462.19097   -462.20335   -253.92336   -448.18655    -306.2033   -236.30908   -394.86534   -299.12733    -370.9979   -271.32883   -316.15341   -178.63779   -260.01343    -275.8032    -267.0697   -233.37656   -138.59596   -109.69055    -75.19796   -186.07153   -214.73281    -62.27401    -96.00015   -193.09469   -218.32333   -198.26405   -140.21688   -382.37778   -303.56601   -232.05462   -248.10108   -353.34633   -321.98737   -235.78558   -337.85722   -253.67389   -223.82073   -249.17879     -51.5252    -78.77241      4.04527   -207.85347   -119.87947    -85.83528   -204.48107    -58.06574   -109.18141   -128.53753   -340.77043   -194.91514    -266.8318   -152.14834   -260.50448   -219.97066   -183.71561   -131.54669     -96.1877   -144.53471    -82.41526   -102.82956     78.44899    -83.33687     79.94892    338.18226    236.04798     54.37868    248.76218      51.8105    -27.10527      6.66792    -12.76393   -167.82665     76.45198     43.35861   -135.42002     82.19746   -108.01154     51.84479   -160.37297    -207.5651    -71.95582     26.74321   -147.23386     37.66548    -67.99129    -50.03435   -167.33618   -274.93202   -114.52365   -290.83204   -243.93612   -173.41363   -282.97399   -176.74712   -435.53357   -244.42394   -221.98902   -285.93001   -372.70222   -395.85567    -232.2763    -300.3995   -229.38382   -541.57233   -534.06462   -386.99374   -302.32974   -456.76879   -318.51937   -228.35531   -309.66299   -255.52268   -169.60193   -583.00523   -381.81586   -398.01463   -397.76867   -527.38077   -570.95489   -432.66698    -27.86663   -402.95076   -244.60638      -301.44   -467.24646   -548.99285    -350.6626   -347.60322   -537.60244   -329.75853   -581.42959   -563.60594   -442.68597   -551.93421   -664.50597   -696.52765   -834.78561   -908.96689   -859.33038   -622.28965   -792.30249   -667.92734  -1094.88979   -893.37049   -994.29308   -678.99085   -990.88477  -1066.93111   -1371.2412   -797.68948   -827.39736  -1361.18251  -1362.13618   -983.33232  -1215.52857  -1016.94526  -1141.79269    -836.4314   -675.67588   -1040.3455  -1274.59214  -1254.38094  -1011.97573   -793.44988  -1160.72936  -1825.35152   -972.20138  -1057.74335   -896.35655  -1397.53422  -1247.45203   -799.20443   -1177.5333  -1104.53615   -985.72962  -1204.33452  -1117.36414  -1213.57465   -1497.4055   -992.24387  -1168.66482  -1115.85902  -1249.97237  -1027.22959  -1400.86774  -1692.58137   -798.25158   -989.57834   -612.25979    -499.4271  -1072.27242  -1387.11032   -748.18617   -980.63841  -1103.12352          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        0.00   -447.40195   -311.61957   -232.51177   -387.06526   -149.79598   -147.55959    -115.0136   -429.98048   -281.43593   -366.03273   -253.27722    -163.4135   -246.20456   -382.77134   -397.61504   -454.60815   -334.33282    -445.3264   -267.84338   -478.03616    -366.0693   -520.95438   -393.71064   -194.71504    -15.93854    375.90366   1426.32951   1860.37942   1259.52936     791.7941   -112.25903   -252.62098   -386.11912   -264.23523   -464.98021   -394.85501     -331.791   -434.05147   -424.49651   -314.92899   -473.79779   -304.68072   -301.21927   -259.20816   -457.36327   -358.95958   -356.82957   -295.75265   -408.34694   -226.12073   -375.29233   -284.06782    -286.7417   -236.18396    -522.4559   -114.44757    226.63779    800.53116   1791.36752    1219.2932    687.96623     80.35448   -371.07671   -302.16537   -264.45856   -339.50859    -340.0652   -262.70089   -272.44743    -68.76445    309.66337   1021.59537   1556.63922   1648.80077    878.19593    760.16007    673.11424    391.00697     253.1668    -61.62883   -412.62209   -432.34982   -197.50098   -331.10123   -347.83851   -285.08395   -306.53574   -287.57435   -274.75798   -398.29197    -83.55354      2.38137     90.73113    -17.89649   -174.87679   -239.30398   -257.80893   -362.34403   -203.07088   -399.60753    -96.37018     15.65878    189.63531     -21.1919     23.92282   -230.32468   -254.21202   -360.88975   -414.86259   -367.17873   -405.61795   -303.76118    -197.6119   -268.85216   -180.91281   -168.91531    157.72162    312.24259    327.36673    117.40484    -18.39462   -197.78261   -123.78845   -208.92153    286.71732    438.84975    667.06851    560.78934    558.94931    271.60827      20.0608   -115.58932    -223.0397   -252.48433   -284.70126   -153.98687    357.75634    749.52287   1586.74325   1988.81951   2081.40979   1697.00981    1123.3183     505.2431    144.84208   -205.50775   -251.23681    -19.25601    -122.9375    -298.7003   -222.09025   -463.35582   -305.48675   -283.27096   -465.23296   -318.68743    -197.2001    -97.92951   -243.51396    -131.6072    -49.01177   -106.17605    -77.72698   -349.93108   -155.08676    -39.02471    -19.05333     23.44542   -177.22493    -92.49781   -183.76064   -379.39298   -319.39049   -376.33216   -279.07164   -182.92394    -51.11724     -95.6429   -223.23616    -82.01338   -299.36134   -329.04501   -293.32531   -346.52039   -409.73419   -316.30273    -427.5102   -387.05672   -254.73755   -193.47059   -248.14128   -320.78101   -163.93443    -96.41856   -136.66159       3.9159    184.16104    106.91822    405.24403    197.24309    226.28548     -3.32287     -3.73361    -79.43895    -13.27465   -206.14009     61.53911    -23.95719    103.70443    -31.93091    117.67088    -48.01554    147.61855   -127.70789    -76.34619   -277.32595   -426.57719   -517.44482   -407.88531    -346.3463   -397.38049   -515.21372   -346.27171     -242.999   -272.50182    -79.79858      8.65417    -32.62559     46.36086    -47.56079    152.10792    136.62698     12.91024     38.46046    137.69301   -175.00232     71.20499    106.84851      146.323       4.5665    121.63458    -47.14927    -89.67045   -120.40729   -284.57606   -234.09525   -322.22086   -397.59795   -207.62972   -273.93912   -250.75538   -276.25266    -217.2096   -206.39736   -250.46011    -16.81734     43.28582      57.9495    170.51816    160.72765    118.30821    455.77931    497.86532    519.88468    456.27956    530.80361    724.03851    448.34966    296.05831    386.96976    123.75174    130.32905   -215.97191      0.03452   -324.74466   -272.47762    -72.18264   -216.65294    -99.51228   -177.05608   -161.66521     88.10863     43.92317    124.38359   -126.86674    -55.51509   -200.08461   -301.45589   -239.20462   -350.56456   -301.22368   -258.07186   -306.51277   -338.72328   -497.71794   -395.41434   -472.47089   -391.49698   -313.97699   -406.40766   -327.20533   -430.38555   -244.67149   -408.48894   -399.28466    -93.36263   -287.14799   -107.27072   -296.88056   -124.31311   -300.38892   -128.02221    -90.03623   -141.58584      80.3119     26.01565   -104.99658   -210.10649    -30.27823    -225.5569   -341.87422   -215.57863   -244.03378   -324.41983   -243.90662   -303.68168   -126.20613   -243.06868   -249.38273   -141.31215   -228.94328    -57.20513   -226.20262   -204.90859   -213.93988    -67.16661    -17.39501    -99.50261    -75.78303   -261.61271   -216.60542    -68.55085   -286.07538   -200.76959   -234.01506   -213.38963   -203.62937     -6.61223    -332.6287   -249.06688   -183.92303   -114.25714   -171.03098   -189.20941    104.16497    -15.84943    153.10646    185.01232    196.92026      133.764    167.87437     24.72541     72.97939    125.37401     77.53758     62.69598    -31.85927    157.17363    -73.35265    -87.51658     27.00922     36.60791    -74.38604   -110.23956      9.03185     74.82561     -93.4044    -58.76754   -293.50729   -130.00006   -282.79137     61.14706   -121.78703   -219.00616    -60.80735    -72.04203    -183.8997   -218.86005   -167.38339   -176.39975   -211.33819   -265.04853     -516.367   -128.98684   -239.54794   -464.38456   -612.51024   -361.88599   -283.02216    -449.1964   -100.54836   -386.11013    -99.93185   -202.06954   -425.49666   -128.43781   -485.85186   -521.72595   -241.79769   -372.94098     -59.8938   -487.96839   -270.23428   -291.48839   -280.09322    -33.18805   -298.43061   -394.60655   -284.77268    -91.99382   -185.10847   -442.20713    -483.6873   -598.08592   -814.68239   -275.29006   -428.38977   -823.75832   -770.28981   -514.63264   -712.49582   -516.15795   -593.07618   -569.09953   -645.41616   -732.00787   -536.20977   -727.04739   -771.37451   -702.89618   -757.09806   -880.65356   -947.72713   -964.61382   -827.52688  -1148.18466   -874.46397   -872.32323   -991.70518   -1158.5158  -1230.60926  -1306.07989   -965.50994  -1351.40729   -953.61548  -1052.50945   -939.30529  -1368.29369  -1021.92907  -1043.55739   -962.31175  -1152.14191   -971.17363  -1124.98534   -1322.4187  -1151.13016  -1113.12664  -1483.05452     -826.291  -1149.48051   -850.21209  -1375.75278  -1228.81744  -1228.43251   -1376.0129  -1186.76014  -1225.92848  -1305.42677  -1339.33654  -1512.73231  -1283.03995    -907.6853   -791.86572   -1383.1008  -1702.39478  -1246.91599   -856.61812   -297.67913   -863.29154    -888.1015   -1637.5175          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        0.50   -326.13648    -80.58932   -225.55258   -303.39982   -306.24162   -195.57466   -346.33747   -324.61794   -300.84961   -236.05841   -199.83612   -331.56031   -317.71208   -380.79346   -446.88088   -274.34311   -327.70902   -460.50164   -349.03791   -496.82227   -181.04019   -284.96119     -341.938   -280.09279    -27.07764    291.16855   1440.13877   1738.15783   1497.96798     672.0595   -125.95261   -426.23369   -467.20291   -300.10156   -333.46222   -367.59254   -425.61288   -395.52571    -332.2149   -508.05097   -443.04931   -403.53059   -349.55806   -171.25481   -350.95332   -406.74543   -359.37883   -374.97283   -315.80683   -375.28584   -284.74602   -352.87447   -412.99647   -259.24105   -239.96939   -145.82737    228.55839   1136.32881   1457.10816   1246.77265    736.62129    -57.72079   -188.04911   -391.97488   -251.67508   -338.86085   -384.65598    -260.4775   -373.48858   -286.71235     334.6925    989.77938   1432.22338   1504.58672    945.23599    720.70487    802.65692    555.49471    101.00293   -134.08115   -456.45373   -536.69504   -327.41301   -393.53869   -197.55667    -356.5445   -304.60134   -192.53034   -268.29949    -132.1331     64.71123     90.25304     -4.54058    -28.49204    -82.48989   -212.17721   -420.71781   -387.83509   -548.87937    -373.9227   -290.03112    -162.8911      2.50891     18.97348    -36.86101   -253.01439   -255.08988    -445.2807   -310.21956   -368.12887   -278.49483   -458.73207   -402.47876   -414.35272   -150.70485   -153.05158     89.82789    458.68056    375.55425    209.62439     17.48414    -29.43622   -159.82625     20.50646    191.91854    546.84512    683.58429    597.03416     368.5269    278.77237    -54.14672   -283.25973   -231.80253   -244.21551   -230.03936   -119.67668    237.58876    867.15627   1201.24267   1995.48129    2016.8025   1676.16026   1088.24475    506.01615    -61.13912    -85.97269   -168.16664   -266.07763   -279.45612   -317.53569   -343.02536   -244.01851   -265.45538   -364.22218   -260.64259   -439.06674    -61.03814   -395.51584   -202.54754     -180.831   -233.97993     -18.4971   -164.71066    -179.6696    -40.17755    103.27646      3.66857     -1.39187     50.04485   -168.38245   -262.02151   -466.55319    -269.7575    -162.3664   -232.04473   -227.64648     -0.62298   -164.22883    -89.01282   -267.60839   -228.73058   -331.36328   -257.46993   -265.26693   -307.15536   -351.10801   -412.94649   -339.22766   -319.01527   -282.45912   -134.11293   -228.49097    -76.47705    -84.78748    -50.84582    -83.31423    241.95524     39.15751    156.38497     130.1269    202.99507    170.84129    112.77673    -91.50037    -67.94689     65.98945   -115.58091    -11.94282     64.84668    233.00883      8.44212      4.74302    -57.33908   -188.60389    -35.38302   -308.54925   -346.73418    -391.9773   -412.38038   -502.87319   -314.68803    -417.0806   -396.43264   -251.17698   -211.38355   -104.98384    -85.69772      24.8927    -10.83498    -31.25986    153.51903     41.49845     55.86608    -70.14279   -315.29261     -8.99068     35.83993     64.01105    245.27831    171.14115     -96.5528   -107.24408   -122.16919   -168.75578    -216.6576   -258.25337   -470.73394   -200.37722   -410.28813   -402.07078   -378.81532   -238.69748   -195.90297   -243.39454    -28.70414   -106.22026   -162.18037    -16.15751    -15.70225    204.16083      200.037    295.26866     390.1564    587.63897    481.82565    523.27689    577.45804    622.86596    470.74798    321.95649    254.09452    166.20281    173.83294   -179.45791   -123.34324   -246.77772   -137.95671    -98.12512   -222.75221    -17.84667       0.2767     30.32334     22.98208     34.04915     17.42444   -170.47832   -168.44105     16.43979   -254.40278   -285.67624   -264.54907   -252.77255   -352.41949   -327.42305   -233.27358   -345.79149   -525.45664   -514.44395   -284.66188   -384.12856   -232.23232   -355.18596   -398.52158   -182.97053   -248.85529    -291.5755    -289.0645   -252.35861   -379.85069    -64.21474   -172.02393   -336.17021    -47.41826     31.74016    -29.73699      9.31295     57.04287     41.60483   -299.77916     -1.03722    -43.34035    -71.60808    -215.9478   -174.12212   -181.91317   -276.01362    -301.8754   -258.44829   -322.96722   -395.98707    -94.68422    -63.32412     31.97776    -77.80196    -110.1966     16.36389    -76.53945   -252.41464   -282.75081   -149.02275   -300.77008   -201.46497   -135.53223   -301.63408   -243.16378   -241.38513   -107.40503   -123.60442    -56.65543   -295.47228    -93.37714   -223.15306   -125.12698    -96.38991    -32.33043     34.51003    362.32924     13.92096      8.74113    180.75249     24.49565   -117.87024    258.62137    -98.95143    124.42744    -12.53151    152.83181    120.00578   -142.49699     73.36353      54.2289   -180.39234    -89.24835   -136.18551   -202.66827    -37.57978   -277.65528   -335.49497     -8.07477      7.45024   -202.46155   -242.77572   -356.93137   -293.44639   -104.77312   -157.67613   -153.82668    -95.43145    -96.16784   -457.64196   -152.91133   -313.45868   -140.16027    -171.9814   -411.25547   -328.40597   -377.96462   -113.91794   -530.16131   -436.43533   -230.28179   -276.49524   -183.87929   -375.76264   -320.34588   -208.95411   -188.46006    -402.0752   -201.31923    -349.7339   -189.94979   -272.96164    -43.21295   -435.49488   -352.80737    -30.17961    -154.8572    -464.3007   -201.47675   -324.86248   -377.34911   -220.31827   -512.32353   -402.74056   -361.78636   -590.86318   -509.92072   -357.95805   -729.76965   -621.96227   -452.68006   -655.85156   -703.92296   -996.22251   -629.92621   -796.38971   -461.45591    -727.3629   -622.01484   -820.34568   -922.29915   -546.47976   -873.09039   -936.88323   -763.86346   -908.77561   -662.69412    -697.5879  -1105.08832    -943.6582  -1249.80983  -1465.01033  -1015.90949   -963.05229  -1314.27142   -834.66339   -954.37677   -929.35997   -961.02535  -1147.92529   -743.93876   -918.63671   -895.48666  -1305.84355   -927.70891   -872.99845    -929.8533  -1337.81234    -987.5241  -1294.71811  -1419.41163  -1054.96192  -1079.65013  -1333.45527  -1392.63131  -1400.13063   -1246.4185  -1231.52514  -1154.46173  -1157.14383   -996.75459  -1099.15609   -455.91378  -1494.47512  -1543.99938    -214.2207   -627.08279   -118.46928  -1159.21595  -1163.60931   -974.46469          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        1.00   -205.40746    152.16924   -218.47652   -220.03114   -462.57446   -244.52851    -553.7376   -232.13544   -290.07787   -170.20621   -189.42334    -419.7909    -372.9987   -345.89062   -493.82319   -243.30251   -318.97504   -373.86592   -264.23968   -473.72943   -281.11641   -149.35699   -338.07547   -405.25794    -79.00089    173.67561   1247.64393   1802.68828   1653.26664    610.08628    -273.3988   -371.66497   -468.47403   -375.85695   -326.59375   -443.38853   -396.44283   -363.70547   -360.77503   -515.48931   -123.35108   -371.43667   -559.78628   -315.35537   -356.81412   -294.42058   -272.52633   -300.37583   -446.55681   -283.91079   -381.99743    -376.4836    -421.9319   -488.44024   -270.13146    -74.39553    193.55401    1130.7819   1456.77192   1208.25336    587.34386      48.6334     -77.7453   -389.88762   -285.94952   -250.24947   -365.83543   -334.31248   -259.02984     23.08067    546.30045   1206.71337    1520.0357    1482.8124    979.76877    773.07392    533.18278    337.68162     65.52652   -307.47696   -480.52841   -567.59281   -567.31212   -364.24694   -166.79432   -260.76976   -379.86996   -291.41565   -282.54632   -241.75966    -95.52758    -44.01168    176.84852    101.59757    -383.8294   -304.77887    -359.1719   -307.30372   -427.26093   -502.97483    -188.2147      -25.845    158.95246      -8.8442   -110.45199   -278.76392   -194.20566   -360.11999    -300.3519   -301.82978   -268.13521   -330.65019    -297.4163   -257.09405    -63.53904     30.54137    141.32702    175.10854    342.87973    282.34896     22.61494    -32.32851   -268.96835    -45.06549    210.13675    462.45576     479.3405    538.57304    435.28771    127.17597   -120.95531   -237.82379   -169.81812   -154.80916   -286.51339   -166.30068    214.63127    868.24435   1501.26688   1963.24252    2110.4758   1562.67442     965.4637    430.49546     -2.43725   -159.50993   -269.61843    -12.68375   -353.90708    -216.5034   -254.34831   -273.43506   -425.91802    -363.4577    -95.47884   -278.68408   -203.64683    -211.0451   -264.38178    -10.86242     -5.89429   -190.43014     -158.263    -65.54726   -126.24414     12.88456     35.33164     38.37267    -18.20434   -184.88953   -203.65388   -176.66524   -218.06533   -329.61523   -413.94314   -168.94981    -164.9544    -98.42518   -260.90737   -252.96514   -339.45551   -377.62989   -293.65426   -381.28418   -394.16904   -363.41741   -278.03155   -489.89629   -241.69772   -258.63603   -296.62251   -141.18765    -70.38159   -315.81264     58.83412    -47.11481      112.668    260.70234    288.63418    109.57202    223.13155     64.65176    120.56825     70.77797   -167.60192   -237.02439   -107.38856     -16.5841    111.50375     -99.5189     89.45022     95.08645    -69.52173    -59.52623   -157.37771   -208.65442   -279.98268   -331.79016   -401.10293   -307.48477   -219.45173   -314.50648   -275.54283   -133.72793   -124.80128    -21.57078    -69.22208    -63.86386    -48.67351      0.17177     14.88728     -1.71483    -67.59404   -143.34061      6.12002     99.34971    205.21616     11.95613     91.90655    -17.67558     61.67325   -116.18557     12.67797    -45.48971    -65.48983   -312.76272   -217.20002   -247.38617   -251.30973    -361.3382   -302.78872    -226.0408   -203.41677   -126.99438   -107.19191   -104.55125      18.3859     47.66418    -63.27441    117.30195    232.51826    438.96673    593.95983    409.40343    615.87156    642.11497    555.09939    676.12258    583.35754    470.93399    188.22049     36.28949     76.30188    108.10941   -230.87334    -65.78107   -160.03488    -197.6599   -204.60111      76.9031    -79.15571     31.62407    -75.42598     52.85613    -17.34872      22.2158     96.22722    -86.01079    -438.2117   -154.00616   -195.32471   -287.73862    -219.3743   -381.54726    -394.9421   -386.32794   -296.82623   -440.80394   -346.31678    -308.5616   -408.68769   -191.53377   -322.14825   -131.82815   -215.01074   -130.19286   -228.90115   -201.72007   -350.92454   -144.64168    -90.26623      38.6608   -238.23455    -62.21561    -47.63612    -79.72057    -51.55013   -200.15895   -101.80008     -86.8975    -91.63972   -206.60215   -222.11519   -183.46074   -146.12404   -248.55069   -157.31456   -117.64249   -119.13614    -183.6643   -134.59436   -218.18174    -168.0565    -36.98789   -352.07283   -166.94318    -85.57811   -145.81073   -141.81984      8.90045   -206.00777   -229.15869   -163.66606   -140.14446   -358.21237   -225.67785   -504.16057   -222.35301   -365.69433    -17.79627   -164.84729    -43.99902    -61.20153    -98.89092    -42.29513     78.01822    -93.94113    125.13374     11.19454    192.40972    121.35003     85.72004    -34.47428    116.60209    268.85021    379.68025    141.95046     41.96358    164.46821   -151.19363    -36.00373   -185.82949     34.80745    166.54905   -171.38167    294.91362   -312.47433   -209.14285     76.62789    -13.66449    -356.7854    102.73618   -224.97742   -280.74056   -156.48527     43.07537   -236.46368   -309.51573   -384.75977   -298.62972   -205.29453    -45.83979   -439.10726   -349.34984    -150.2408    -41.87047   -482.64249    -99.94254   -385.18097   -348.57908   -171.00595    -157.9971   -292.15791    -354.1589   -241.34534   -314.60485   -305.53688   -630.03647   -292.82432   -219.86856   -309.18041    -80.04269   -407.37282   -263.73501    -238.0829   -438.95452   -504.42071   -148.88792   -333.61656   -416.46849   -444.46868   -281.58128   -467.49561   -597.21313   -540.10219   -534.02465   -384.59576   -583.84822   -420.01849   -612.43782   -622.35502   -540.07513   -830.24402   -975.57519   -756.78404   -669.16569   -678.73333    -616.0816   -773.54685   -571.42685   -752.20402   -989.58031   -859.46207    -656.8685   -758.17342   -562.35507   -828.96881  -1353.73742   -946.00003  -1055.79098    -655.3091   -913.01202   -869.44823  -1220.07201  -1041.00034  -1230.05091   -986.73506   -920.14307  -1031.49444   -998.83377   -834.80764  -1167.89756  -1276.35335  -1092.01908  -1117.29547   -553.83232  -1034.30971  -1068.10051  -1012.23213   -975.42587   -818.23189  -1404.84472   -690.05111  -1021.48515   -624.42272   -839.55367   -652.75252  -1179.04869  -1222.38666   -872.54253  -1231.84851  -1401.53547  -1332.14617   -873.19698   -956.80895   -724.00211  -1184.34615  -1056.69114   -537.44638   -1117.8111   -586.16522          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        1.50   -113.99438     165.2168   -258.71845   -316.88427   -380.51916   -334.85083   -513.62525   -188.67716   -178.02918   -266.68031   -264.21242     -391.386   -405.21328   -277.18272   -538.97383   -322.10198   -304.58433   -248.20704   -182.86493   -448.68156   -373.82761   -159.04865   -344.38224   -380.57188    -30.98547    284.31465   1306.93976   1912.32161   1564.35879    450.82079   -255.86256   -331.19697   -391.50027   -323.40129   -417.49708   -158.14658   -176.74155   -203.02971   -154.97008   -396.11739   -274.46895   -319.70824   -358.67628   -333.79562   -416.15782   -384.98853   -346.54314   -248.43808   -515.31362   -269.09014   -411.80091   -424.80127   -363.33407   -451.86538   -211.36099   -211.84772    235.39452   1145.03145   1271.49628    1360.4295    560.26703     50.78624   -214.78057   -349.45382   -184.59016   -176.53963   -143.46276   -215.99356   -359.76082     -53.8156    304.56043    992.28944   1372.37988   1378.63418    910.62537    804.35446    544.71262     410.3534      6.12285   -275.59937   -420.83574   -488.75107    -346.5478   -624.94082   -267.19457   -201.29579   -329.58275   -323.31255   -351.90878   -137.72058   -112.58045     84.74622    -70.24014      0.05427    -464.7637   -346.55861   -368.66914   -412.35924   -465.06323   -313.32095     93.13362     98.91539    -43.21059      9.18272    -90.02762    -122.0327   -234.34608   -407.97313   -196.03507   -501.31428    -300.1362   -233.01754    -285.3896   -292.18426    -78.83289    -45.01336    -41.84389    393.72786     354.8123    208.69495    -23.88662    -61.62901   -176.89616    101.50616    150.46575    409.79894    695.82062    531.52114    429.64351     90.09785    -81.39874   -183.51951   -282.12009   -251.28917   -212.37641   -122.99123    187.95717    850.87087   1517.92075   1833.07035   1988.12145   1806.96747   1190.40809    493.36219    103.28161    -34.03227   -216.42347   -180.23977   -220.79486   -300.61669   -200.22118   -400.77944   -250.38385   -351.31435   -301.85524   -339.63146     -248.914   -174.23724   -111.03655    -93.43239     10.50265    -35.69638   -225.44453    -49.58455    -89.75519    -90.29706    -45.55484       1.3156    -31.33469   -124.47323   -219.61318   -313.44767   -236.24158   -297.75147   -294.28345   -177.11192   -259.04749   -173.31059   -119.48782    -122.4464   -185.00163   -168.06939   -258.83381   -284.23625   -438.92531   -604.02739   -256.07263   -377.23181   -429.98392   -327.97648    -174.7151   -277.31348   -125.52352    -27.03701   -256.00568    -83.45128     -11.5968    210.43927    190.71634      188.671    153.89274    104.39925    152.47234    -45.73454    -73.34234     21.20337     -6.26514     -33.5459    -50.48338    -99.59901     56.25863    261.32306    -32.92718    -74.07573    -99.08159    -348.3815   -244.17674    -272.0133   -335.93504   -478.59468   -425.54509   -239.63295   -315.14412   -289.20214    -287.7566    -26.10855    -94.16609     83.50121    -10.82591    120.37389    -32.22443     84.70626     -6.68482    -66.14463    166.00316     80.74473    149.02025    226.17679     84.13878    -19.87575     138.2591    -56.20856    -184.4391    -27.59458   -217.96386   -217.53461   -152.00574   -268.82886   -180.70503   -281.87297   -287.60555    -50.92329   -277.97166   -139.25881     59.36897   -110.19188    -49.34622    187.98878    109.35481    131.40246     240.8338    386.42853    401.45859    467.99927    442.66476    683.62887    783.61707    399.26105     455.3601    461.19116    360.07903    200.24592    -68.38853   -285.89256   -249.54146   -222.64508   -107.21214    -44.53823       9.5852    -18.98239     -6.55584     -7.44176    193.69913      25.1204   -108.09849     76.49141    -178.8045   -121.73913   -209.88182   -251.41001   -150.76999   -213.75627   -224.65486   -251.94883   -351.10953   -442.20592   -404.61936   -388.42964    -353.5906   -231.68852   -379.69116    -301.3073   -325.99319   -305.65249   -228.28775   -180.03316   -286.59716   -287.91866   -205.08259    -83.84152   -131.88784   -284.13124   -142.38137   -144.30409    -81.62356     -16.6399    101.98875    -19.14747   -127.39002   -327.66004   -170.56103    -26.48853   -289.90013   -200.00899   -148.56669    -98.96605   -188.94088    -47.57449    -313.1181   -288.15139    -162.2888    -62.51734   -268.53867   -137.26462   -173.83638   -150.59763    -68.46469   -156.46043   -111.99049    -86.98349   -243.87943    -56.97738   -197.05255   -109.44518   -215.46915   -153.24542   -212.87878    -84.24126   -150.55973   -139.34371     69.33481    -114.4612     56.83498   -127.86183    134.68308    160.06064    206.36149    146.49943    420.78511    111.24693    300.07322     157.7912     -6.77142    209.06497    201.98039     68.20508     95.09305    203.68384   -125.06159    125.79821   -226.33466   -189.12755    131.41773   -157.81919    -81.74553    -98.74621     -67.5086    -75.04129   -256.34329   -119.67372    105.47413   -135.79548   -104.20413    -58.57594    -136.7003   -146.73267   -119.30132   -235.21799    -34.66652    -110.4521   -424.07575      1.00152    -17.99412   -168.91891   -205.99869   -378.42921   -459.38006   -354.94511   -318.08331   -209.67899    -70.03773     -104.822   -458.50196   -101.08192   -204.73896   -265.09472    -161.9225    -93.33469   -228.50756   -324.88735    -242.0909   -297.49755   -357.51063    -81.20968     -237.508   -521.54955   -390.34498   -326.61549   -301.86428   -193.67507   -448.74143   -202.56264   -374.24114   -454.45728   -440.08141   -225.44515   -646.91863    -357.7098    -467.8925    -335.7294   -352.60646    -708.6776   -773.42654   -839.78662   -771.20366    -744.2291   -530.68931   -769.72667   -644.92824   -924.56681  -1062.63158   -683.87325  -1253.63322   -885.01336    -981.6477   -859.61759   -854.86706  -1301.58546  -1102.80764   -732.29054   -977.74572  -1209.84192   -948.81306   -824.39137  -1307.64727  -1127.50939   -960.56018   -884.50613  -1250.51985  -1267.54487   -970.34594  -1230.86527   -773.36212  -1020.53677   -827.77271   -888.51094  -1085.20639   -598.26889   -640.40849   -791.08503   -364.34022  -1220.76227   -985.72729  -1284.47985    -711.9889  -1106.32123  -1241.14652   -963.34109   -680.46533   -746.24607    -1335.786  -1585.52009   -918.13231   -1119.4956   -708.44298   -273.62458    -864.1511   -245.64294    -44.04196  -1291.16862   -731.38819          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        2.00    -46.90731     55.93358   -311.89085    -448.0342    -280.1975   -422.44389    -469.9188   -146.23284    -92.14323   -381.57571   -324.14862   -413.62855   -386.52832   -418.12882   -366.49767     -351.691   -240.35695   -280.76332   -389.00857    -470.7939   -282.79942     -356.738   -343.59518   -255.00059     52.12179    407.80537   1440.86655   1989.07728   1490.66731    464.68007     -7.76897   -230.14556   -443.92948   -345.83211   -455.99127   -305.62019   -553.15475   -242.73024    -501.4395   -299.63438   -357.06929   -273.50268   -383.00105   -148.05991   -353.60275   -435.62885   -374.48503   -315.69318    -446.9427    -215.4626    -280.8472   -529.16601   -320.50379   -371.03059   -341.84553    -145.8828    290.26264    868.06128   1499.83299   1566.19144    761.97184    -76.87061   -291.63672   -404.02156   -417.19628   -278.10632    -40.84677    -151.7007   -331.10688      178.138    389.62688    890.82938   1398.55693   1517.35232    790.55885    670.40705      661.481    526.96465    100.02182   -179.68751    -463.4751   -381.14984   -370.06366     -258.664   -221.56346   -291.59682   -225.16222   -280.24072   -237.11765   -217.87962    -77.71655    206.21951     -148.776    -93.40536   -126.97762    141.41145   -187.39075   -497.85052   -412.54786   -295.47828     58.70648    114.63086    160.42642     39.13929     -83.7233   -228.63875   -412.21368   -347.05846   -182.77791   -535.15769   -407.03097    -363.0728    -304.4146   -366.37096   -183.05886   -127.64541    218.14297    196.43034    276.10678    191.45929    -66.99429   -116.84005   -248.02146     43.98177    257.15009    510.48935    546.53661     530.7699    450.26322     -9.52822    -13.74827   -236.65794   -226.61037   -324.56552   -341.60264    -21.94768    356.35522    865.50788   1467.92805   1985.96567   1966.27261   1564.81829    950.10055    558.54255    119.95637   -165.68233   -133.00126   -175.52772   -258.16056   -213.56082   -230.02169   -298.76574   -421.68957    -400.8782    -207.4436   -232.11014   -256.49177   -162.34244    -98.01938   -232.66241   -163.09507    -17.23375    -99.84801   -195.08235   -171.90555    -58.94673    116.34448     -1.23879     79.45382    -31.58441    -113.3124    -259.3632   -224.74551   -307.29611   -302.28031   -252.85059   -273.91981   -214.03401   -177.71628     -21.2107    -339.9871   -252.39437    -228.4282    -259.1743   -349.65687    -157.3106   -483.17335   -384.80764   -231.88329   -274.28648   -246.83825    -84.33043    -78.13031    -66.97769     39.02706    -11.48666    181.77794     11.55607     168.6934    234.21983     57.45149    271.00001     37.35091   -150.01721     98.70749   -182.55726     -9.29644     12.43658     30.64434    -21.36385    -79.21997    132.49699     20.33753   -152.59728   -183.77864   -198.98111    -193.4562   -176.11034   -222.46905   -349.56669   -192.78257    -261.6977   -507.00637   -136.23274   -157.70221   -295.63996   -129.94574     31.27131     63.21082     -7.38151     77.10339     46.19465    105.48229     46.92967    -187.1933    -31.97841    147.41272    172.46489    342.43333     183.0902     49.87816    -12.16905     27.83704     -95.9674   -276.41236   -169.88265    -171.9334   -113.66122   -231.06132   -301.36111   -266.87305   -229.21402   -204.60293   -127.73266    -77.95706     58.28201    -39.49587     56.17255    209.71016     27.63031     91.45475    358.42316    544.85329    658.30872    766.27137    535.25802    842.99972    589.47069    697.04755    451.88229    149.53349    142.55945    -58.56819    233.77831   -220.57679   -170.45938   -219.37761   -152.37698    -82.74914    -91.41411    154.60814    222.17508     43.83011     -58.1618     86.86381     76.38081    -21.75332    -79.05605   -126.83464   -171.74124    -13.42409   -413.31339   -282.77447   -302.39712   -379.81214   -365.10533   -254.88999   -266.23104   -440.11851   -292.66085   -282.62536   -370.08955    -34.04567   -308.04013   -252.49533   -296.94455   -126.02627     27.69794   -134.56966   -101.17594    -91.31561    -36.78548   -279.11729    -98.23271     -78.3669   -149.76559     35.32979    -18.57309    -37.48182    -76.39996    -99.03794     23.08893   -161.49934   -441.06357   -172.18946   -333.13666   -150.75801    -248.6196    -56.84918    -95.43548    -65.92055   -198.29064      3.61687   -207.61499   -124.95917   -343.24299     71.48494   -225.62742   -135.90027   -103.10569   -191.46621    -64.95979    -69.57565    -86.39755    -17.35022   -190.72505   -152.58486   -186.01785   -172.32176    -82.78934     -36.8464    -51.72784     51.80231    -55.64798     25.78848    162.54101    -13.75281    205.29539    159.81105    330.65081    258.71145     320.7125     66.47371    288.29641     49.23279     59.43201     61.70201    102.67861   -147.86605    -43.93288   -117.02635   -108.32472    113.66183   -201.00339    -26.01841      30.8223    -41.66881   -213.69649     76.68726   -153.69065    -51.76485   -137.04124   -296.71391    -212.5562   -218.96638   -378.82907     48.34486    -44.08664   -301.72536   -178.83696   -269.67327   -301.42297    -617.6307    -412.0406   -183.88732   -152.06203   -232.27631   -362.76365   -161.45732   -477.76711   -357.17411   -177.94491   -425.27466   -188.99745   -256.96078     16.30029   -260.29345   -314.87699   -271.14495    -268.1882    -24.06878   -339.74686    -330.4702   -346.08772     74.52132   -318.39929   -547.87636   -156.05531   -148.08323   -151.43254     -383.702   -445.36194   -413.48579   -534.88476   -264.53056   -475.60393    -602.4274   -299.47919   -651.90436   -460.52547   -579.79522   -723.10938   -553.03886   -519.45477     -541.827    -566.4008    -683.9487   -560.96141   -856.15016   -724.84635   -839.24631   -969.64898   -770.88029   -648.19058  -1006.24532  -1044.52877   -876.43117  -1165.74229  -1292.68498  -1011.75177   -998.68952   -993.98898  -1102.47743   -960.44627   -973.23169   -998.97685   -869.24266  -1124.82781  -1125.75896   -692.24812  -1443.01049  -1175.65783  -1004.20804   -925.87612  -1216.80889      -756.43  -1095.40677  -1224.29919  -1263.65617  -1198.55682   -702.37503  -1071.94646  -1374.79723   -916.61618  -1539.48807   -420.94724   -785.10599   -650.60879  -1125.49482    -54.77518  -1356.16098    -941.0138  -1739.19356    -831.7863  -1616.16396  -1201.00961   -374.75747  -1183.92947  -1120.95194   -810.10059  -1523.96717          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        2.50     22.39098    -55.46605   -339.26654   -493.55007   -275.22395   -346.98492   -432.64119   -191.53453   -131.50542   -533.67003   -357.95746   -478.54867    -334.1141    -610.0821    -196.0843   -353.65415   -298.84284   -358.90265   -450.21873   -431.29892   -399.38004   -330.64312   -284.59885   -356.33552    -20.72631    304.79166   1448.24884   2008.45775   1492.68302    552.15298     78.11497   -197.69444   -323.64724   -329.75927   -144.52145   -431.77024    -249.1034   -501.60423     -410.368   -490.17204   -347.16133   -214.17285   -390.57177   -263.30624   -229.10549    -287.2925   -408.23059   -380.09436    -75.61996   -364.40966   -350.94148   -449.61243   -399.74673   -406.81363   -394.63342    -79.88464    234.60389    961.74595   1486.41299   1645.91276    829.85957    101.12261   -270.59087   -177.93247    -230.9592   -361.16862   -242.57421   -303.72888   -174.30499       31.085    330.22026   1146.58087   1393.20219   1516.43009    938.68401    489.59301    659.61901    489.49654     257.8882   -142.85842   -407.79268   -266.71927   -176.75119    -66.28482   -493.65314   -267.89073   -315.27085   -364.02855   -227.30648   -160.63152   -122.32566    -90.52788     97.00417    -24.93012   -306.68396    -324.4648   -292.99905   -346.58324   -481.91126   -389.54826   -214.77906   -232.94197     84.08348     -13.9137    -108.3686   -305.91625   -309.49001   -243.06351   -413.30087    -336.1389   -347.25344   -317.76542   -275.94587   -324.93783   -255.86327     -7.51141    355.90734    400.37983    186.89221    316.19085   -166.25017       -13.03   -316.48621     17.63315    254.63366    513.99374    503.25636    623.43862    415.67131    164.21829   -166.12506   -163.46166   -155.01983    -169.8035   -122.51336    -65.77949     97.50011    820.72779   1666.17531    1894.1991   2187.47203   1813.65409   1183.81647    627.47712     78.20125    -251.5145    -34.61149     -55.2592   -343.93414   -246.96615   -403.95782   -334.09992   -359.01516   -270.86041    -288.2778   -242.84933    -218.0632    -165.2954   -216.90461   -199.11633    -51.15191    -68.75478     -3.81467     -94.6126   -242.88775    -62.86725     10.38254    199.84504     -6.53576    -41.27522   -106.93677   -194.91673   -244.54505   -350.75921   -391.59702   -484.94225   -254.89818   -119.19073   -299.84572   -186.48658   -187.49718   -178.58884   -329.91211    -437.0509   -328.27332   -296.02346   -435.13148   -292.30799    -190.4483   -185.72804    -305.8073   -215.36146    -53.06198      2.15466     45.93054   -134.93086    125.43157     -8.87831     85.72227    255.44108     399.6195    318.48865      85.3424     -2.29103    120.35737    107.79391    -45.98385      -9.2142     90.58275     31.40736     34.22398     75.25181    -96.36315     23.83612   -110.46549    -222.3813   -214.62763   -216.95491   -228.86044   -363.17947   -332.80238   -420.39064   -254.06383    -97.88266   -160.39747    -77.18831     59.11657     29.08718     22.58161    167.28983     53.34868      3.71821     127.8924    -63.32698    -36.34366    -67.75383    112.32717     23.00144      206.687    228.41919    -51.92018    -50.58359   -222.56752     -42.4844   -238.66396    -79.57487   -295.68067   -413.63125   -134.55482    -114.5423   -375.46298   -304.61282   -234.29734   -204.99292   -111.03152    -58.84654     55.54669    281.28381    275.35862    286.19718    287.37435    390.30288    402.92327    473.70389    601.56209    615.55667    575.67913    651.48588    460.44052    244.32295    318.56197    313.18654    159.85916   -151.43658     41.45141   -121.54843     65.22252    -41.09883     94.65479    120.27483   -223.16186   -229.15216    102.39857     37.48518      8.34897     97.13466   -105.18893   -106.56133    -116.6247   -168.43772   -410.63405   -212.18848   -365.46476   -168.34655   -285.41142   -196.41582    -483.9322   -244.28694   -280.74767   -299.40393   -317.06449   -237.82175    -312.7963   -329.61461   -187.42887      -296.74   -144.15976    -59.21179   -212.85268   -305.52999   -145.22107    -65.60866   -126.47871   -174.99288     71.04724     79.58644    -88.26525    -93.20922   -191.55497     -26.8031   -121.72186    -36.55115   -250.71667   -224.59574   -115.69915   -147.01384   -282.57569   -251.68613   -199.01247    103.34349   -118.37672   -118.24845   -210.74963    -69.67366    164.58396    -51.17802   -107.92001   -251.48056     26.31379   -271.78862    -57.10281   -253.49435   -255.93385    -37.86539   -204.71224   -140.33191   -201.83635   -269.83895   -158.23071   -189.41162     35.39842   -183.01969    -85.93058     -69.7322    -54.40656    225.41102     98.85351    207.05863    188.77811    159.02144     26.45285    125.17827    118.79391    215.10254     129.6591    153.70828     218.7989      70.5849     52.74618     27.80546   -179.14574     70.63298     29.55762     59.12516   -220.09731      8.77384    -102.2103     67.59408     30.89646    -60.79076     51.27365   -309.57955    -30.35247   -326.90689    -230.6831   -134.93463    -92.97621   -219.88404     -62.7471   -198.42545   -291.76697   -264.09074     29.30209   -299.21545   -311.47917   -180.70584   -249.45383   -377.38894   -411.49707    -15.74759   -280.78422   -378.89042    -31.47649   -254.94433     -28.7521   -167.77508   -244.38719   -461.19908   -551.24921   -348.25381   -268.39669   -399.86229      -434.71   -165.92335   -209.70719   -495.82602   -289.32213    -74.09729   -439.39116   -345.77935   -337.42822   -485.43315   -354.10448   -341.96787   -469.30037   -227.10117   -523.08589   -413.55271   -283.52728   -419.62172   -623.88173   -842.55955   -516.55636   -612.39008   -572.45821   -774.99073   -783.27842    -662.4617   -784.46394   -921.80299   -688.23235    -958.6461   -840.86484   -1022.8945   -830.94056   -864.48621   -781.37141   -871.37936   -965.63246   -730.30373  -1122.89095   -956.57832  -1038.83559  -1152.23874  -1225.95385   -771.02499   -952.03599   -844.05243   -616.00298  -1073.59539  -1128.20941   -831.20285  -1100.99926    -786.4378   -981.05648  -1025.03682  -1033.13228   -675.06144   -539.02873   -972.94579  -1079.39998  -1071.47218  -1215.88151    -999.1846   -878.48268  -1343.11321   -610.90439  -1257.27586   -871.86192  -1440.23378  -1170.00319   -871.72054   -651.58852  -1529.55421  -1080.48431  -1235.87502    -99.60478    -863.3295   -763.47798  -1305.78486  -1219.83704          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        3.00     -10.1333   -162.96944   -249.63421   -388.24273   -343.45291   -209.79462   -401.32609   -238.32134   -189.16056   -586.00208   -284.89551    -414.0024   -193.89011   -421.73861   -231.44306   -308.44095   -544.18714    -458.3136   -461.13595    -351.2086   -486.08232   -318.23192   -141.59658   -415.62406     52.18898    402.53964   1339.99603   2018.09622   1447.63423    619.14811     33.89905   -355.69456   -553.17786   -395.66631   -435.72527   -333.86015   -194.74533   -274.81565   -150.99843   -568.98418   -549.18994   -392.37411   -366.56687   -374.25981   -302.24199   -483.98318   -308.07745   -352.80259   -311.38294   -375.69175   -392.15794   -291.13569   -493.34048   -501.87904   -133.81073     55.06527    302.75914   1033.26528   1585.37587   1530.53297    717.71837    207.85488   -243.31642     11.22164    -485.2851   -395.97003   -206.63047   -174.38948   -199.09651   -121.01015    449.53458   1031.49893   1653.37771   1384.45984    826.39141    697.80331    512.27016    484.48849    265.32394   -137.27368   -128.43797   -342.00446    -368.7276   -315.46074   -270.92523   -269.84232   -418.96924   -400.55093   -369.29271   -339.69973    -22.83436     -5.77594     25.20952    155.50552   -304.28512   -449.89399   -328.15802   -227.97683     -382.615    -256.0691   -192.02142     86.49498    171.70219     -36.5293   -260.71247   -128.50094    -16.02818   -293.59325   -240.28239   -316.58326   -394.74426   -332.86141   -254.75742   -396.07688   -343.65755   -114.36566     104.7192    296.72112     433.3531    108.45961      74.9243   -126.86085   -229.47306    -44.36476    -30.55722    537.57596     559.5045    500.86854    585.33191     12.57336   -131.12076   -172.15774   -231.60314   -151.46496    -86.09448   -105.24384     187.4264    833.63634   1319.69897   1855.23403   2130.28539   1676.88572   1184.65831    736.10382    170.27436    -98.04019   -165.59267    -42.10653   -397.72116    -38.80039   -448.57765   -265.21072   -328.49309   -215.98312   -285.95795   -255.57709   -167.51362   -326.03659    -40.84617     15.22169   -293.56446    -214.1457   -216.69708    -42.80918    -69.77398    -88.54804     56.05721    -40.55645   -121.33375      13.5325   -145.83917   -363.46548   -239.35644   -208.16379   -325.69679   -157.08046    -32.09245   -240.19667   -128.59451    -79.99641   -176.41713   -222.98056   -302.08865   -393.32043   -485.78322   -254.84818   -330.14335   -249.16622   -332.89851    -397.1907   -191.44188   -238.66883   -124.08919   -230.11771    -17.78597     22.22762    124.05684     298.9747    224.36115    204.52492     40.62111    192.92835    194.57047    -47.29791    -22.17055    -211.7885     96.51865    -51.07066    175.92668     31.84073    261.53469    138.04739   -111.24132   -135.26679    -50.34463   -285.17809    -275.7358   -350.13125   -224.30439   -248.90608    -147.1852   -304.36961   -166.90921    -287.7596   -195.16601   -294.01857    -25.91742       13.236     41.00673    181.78832      5.71178    105.90672    -87.30949     90.64372    -28.88436    214.86964     37.02267     83.51618    109.16511     68.65613     77.26629     -52.4968      4.71471   -134.34796   -279.72356   -177.45046   -310.36723   -242.22489    -312.9947   -193.01263   -289.70647   -281.09939   -302.14441   -200.29826     -12.9915    -90.99818    -95.18325    154.00536    -22.45615    385.02209    158.35105    565.15609    389.31612    553.64074    662.25591    628.80861    856.30731    504.59657    436.37864    352.25431    212.00907    126.12977     40.73701   -182.15346    -55.49389    -10.88818     -39.9515    -52.98637     84.20309     56.73266    -36.51726    125.65263     -6.37925     68.44891    -45.93906    -84.41271   -187.59764     20.54259   -228.33572   -214.11446   -294.89069   -163.70004   -239.10696   -425.30045   -524.67978   -306.13202   -358.75889   -486.41959   -489.71342   -284.76029   -252.55781   -213.85322   -142.10765   -178.05314   -104.91596    -252.5018   -269.23315    -196.0483   -213.86862   -204.37217   -181.91492    -24.90128    -66.68916    -25.48884   -101.49319    -39.96408   -164.17049   -113.28847    -225.2507    -51.11014   -121.13947   -150.53715   -218.56423     -92.7948   -401.69964   -317.92603   -241.32903   -251.25009   -239.34788   -115.91303    -201.7918    -13.16601     11.68374   -191.83039      -7.8996    -104.1179   -234.25013    -30.66515    -11.29359   -256.00154     54.34524    -16.14289   -181.51817    -299.2823     -6.88534   -143.21999   -271.24388   -139.94207     -84.5405    114.81864   -250.07959   -223.05474    -13.46928     62.49021     47.38225    187.81281    190.50275    127.04116    123.15776      94.1304    286.55796      7.47475    185.16473    265.03728     96.30594    127.88115    286.81425    142.00325     87.58743    178.25977   -174.39386     79.44841     23.10157    -37.45131     35.30139      59.2011   -185.12208   -420.25239   -213.23175    -82.32568     26.94325   -146.81602    -82.78101    -95.38472     21.36469   -193.42216   -300.01773   -406.59923      59.9623   -202.67496     -164.747    203.09926   -354.34236   -295.71072   -269.95573   -450.58508     51.26757     89.71498   -137.15657   -259.26097   -102.93063    -436.5512   -302.48176   -435.33221   -305.02185   -356.10118   -273.53915   -225.03467   -397.91867   -189.29772   -496.82417    -41.20698   -352.81809   -306.60407    -517.4369   -164.24561   -351.20975   -161.53801   -400.89083   -134.43098     -247.294   -139.23238   -701.82145   -405.30012   -584.36205   -545.09922   -414.09663   -485.82328   -479.43237    -358.2614   -641.49098   -686.08878   -722.40276   -521.82252   -748.57027   -574.09186   -635.45019   -685.07637   -928.20197   -695.73539    -845.4495   -777.66374   -828.08799   -881.43585   -846.84059  -1055.42041   -840.54371   -854.90424   -914.79945  -1059.07647  -1057.42997  -1063.45129  -1117.24879  -1053.78571  -1183.24536   -947.45879   -969.38746  -1023.73883   -946.50806   -833.08946   -570.95043   -912.70875   -666.91148  -1065.44623  -1033.54372  -1167.36764  -1014.17716  -1164.28977  -1103.48401   -971.79113  -1219.30975  -1187.09748   -772.50313  -1274.47856   -953.89109  -1134.76425   -1109.4517  -1036.90843  -1274.90493   -934.06055  -1079.05599  -1009.35003  -1122.11775   -950.98802  -1039.10671   -860.54249  -1351.72788  -1116.71533   -719.71385   -632.09869    136.45819          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        3.50    -84.39527   -270.50297   -155.83339   -279.01984   -391.53031    -95.84181   -389.69057   -359.49819   -344.14098   -453.35306   -126.50102   -294.48584    -81.87647   -306.16749   -274.32121   -302.18314   -423.62926   -374.08047    -479.8575   -238.54785   -493.86291   -343.55649    -50.98854   -323.00561    -42.89881    353.55354   1152.71821   2098.71166   1118.17411    709.86341     -2.32652   -447.54631   -554.86289   -261.31263   -419.61305   -223.49888   -305.37035   -141.88911    -289.8601   -391.81935   -325.88615   -400.66364   -588.83625   -419.26327     -92.9774   -448.95708   -187.88311   -347.52644   -502.92092    -257.1322   -423.67583   -594.49839   -209.07188   -487.52466   -113.45782   -142.96425    130.65386    854.36822   1541.15516   1487.39138    686.79545      46.6078   -251.84642   -250.09055   -281.28456   -347.84961    -67.94097   -115.30133   -221.96392    -95.21261    301.04111    906.12555   1607.68651   1212.71588    791.69347    575.32118    526.31965    869.34958     82.64453    -76.51493   -327.65799     -95.6403   -458.83094   -364.78058   -303.38983   -239.94763   -397.81478   -199.49856   -291.53386   -201.40648    -93.97745    118.64854    129.00074    112.95145    -55.50795   -390.65488   -160.53858   -320.71361   -410.40941   -338.07337    -35.44464    112.58776    140.04077    -51.22232     34.74601   -231.95599   -330.56947    -325.3651   -280.10415   -414.40992   -276.68479   -397.03546   -154.60339     -372.425   -336.86332     61.84917     -8.01049    515.07314    341.77942    317.63056     78.36232    -98.49164   -282.13779    -68.43563     304.7851    430.54436    725.36718    531.59084    268.98265     96.64998    -37.35138   -129.80677   -193.36389    -94.17309   -122.34098    -34.01922    225.71512    773.00776   1365.16288   1840.81016   1895.01917   1620.56713   1210.70237    447.44422    180.38876    -17.31912   -150.33241   -241.71671   -280.35826   -157.82904    -243.8349   -427.92485   -336.46968   -221.91222    -409.0484   -242.54304   -255.31841   -282.08332   -158.71754      1.17286     36.00319   -185.32657      0.45388   -213.34041     75.30061   -131.73159     48.56284    124.64563   -186.38873     -46.3395     -5.59003   -206.76616   -260.11625   -160.24592   -270.46898   -299.87977    -161.1196    -67.00988    -148.5181   -221.53609    -264.7366   -235.95676   -337.31029   -315.57748   -386.39738   -301.10419    -339.8426   -433.81571   -311.48418   -391.53642   -246.45834   -213.83965    -224.4352   -140.53687     22.73756     52.12243      6.91167     47.70663    272.95033    132.72385    145.96316    125.26085     29.58721     -4.56286     26.87373    -62.25834   -105.00852     68.13892    -12.78218    101.84931     118.8305    -28.86236    -74.61269    -12.16105   -216.27983   -195.32847   -226.83737   -148.53714   -283.24941   -397.17685   -244.36423   -307.54291   -178.48127    -71.83766    -89.76266   -126.45991   -125.40616    -60.42565     46.48425    153.58468    -10.91032     50.91384    -17.32209     76.36597      8.11655     49.49784     10.06123     23.80357     27.25243    109.08368     14.86445     74.04265    -31.15337   -209.19719   -165.78921   -179.33402   -223.29685   -340.34628   -480.24789   -405.41371    -251.1274   -266.30402    -94.59579    -78.22875   -253.85067      74.7211     48.89951    124.01121     92.44324    239.13326    311.06964    411.30692     393.8493    583.89154    620.29719    652.47827    605.84781    506.42408    354.44004    461.76197    369.81616     79.76609    -62.83041      6.54654   -126.43724   -144.35966     52.64074   -121.42781   -108.91511    128.77145     59.93612    103.16619     42.61415    111.18452    116.40246    -122.6463    174.51291     -92.9472   -154.61068     -76.0744   -271.66768   -272.58678   -249.15162   -318.94562   -314.61609   -403.65669    -349.9709    -288.2127   -378.99205   -317.22804   -376.47009      -258.11   -164.35239   -244.20694   -224.39684   -265.28512   -309.37357   -279.80256   -198.26804   -102.94131   -137.61688     27.41401     -94.3255     84.46089     70.51706    218.64732     -33.3736   -117.40945   -122.31334     63.96134     35.91949   -317.01105    -14.58586    -10.28555     45.58691     -69.3063    -98.00416     -191.441   -144.04084    -80.21639   -189.80145   -118.73226   -301.25619    -71.06374   -211.96523   -178.18131      48.3548     68.95382    -84.27512      -90.334   -105.49448    -179.6956     16.50821      66.3985    -46.95095     -88.4254   -246.15429   -119.33432     -3.06826    -84.38117    -53.60106   -243.92859    -19.69712     43.10416     206.5618    -13.06939   -169.18539    147.05855    297.42553     65.88154     125.9492     290.8688     54.13395    155.37747   -172.30889     -22.6874    153.74823    104.26555      8.63161   -195.69621    -71.80545   -148.11132   -135.29716   -128.69991    -64.43532     46.98119   -332.12004     69.34633    185.14956   -178.53437   -142.72646   -195.51596    -71.74065    269.60641     11.24196   -265.83724   -187.30695   -129.11069   -138.64573   -375.38662   -439.77266   -375.57343    -86.34572   -244.05361    -323.4289   -100.77091   -306.68186   -231.15278   -113.72992   -302.64263   -121.07221   -252.04053   -565.80368      5.03644   -106.13009   -240.59662   -189.80279   -433.69873    -17.08294    -279.5913   -128.93313   -160.70948   -105.89329   -150.93652   -469.02908   -359.48287   -345.56873   -130.36937   -305.92387   -288.49289   -142.24445   -438.24143   -250.63327   -308.25937   -206.17953   -281.40044   -328.32687   -416.66591   -618.57629   -637.56109   -476.11697   -307.12881   -614.56875    -668.8208   -598.21074   -727.36129   -498.47182   -812.24625   -635.61162   -900.75276   -645.99416   -811.97587   -885.32392   -768.60762   -540.82937   -732.21693   -756.76695  -1171.33124   -939.28649  -1251.77568   -942.61509    -961.6479   -829.96909   -942.73542   -937.85356  -1007.51035   -989.35936  -1118.54736   -891.05456   -973.95062   -904.13086  -1183.12748   -932.21102   -1065.7321   -1015.3595   -748.25078   -841.24275  -1473.34944   -896.96204  -1131.18455   -823.13171  -1201.08813   -709.84069  -1269.54648   -976.95077   -826.86394   -976.27512  -1135.53353   -533.03582   -767.05923  -1346.50384   -700.99062  -1064.94082   -877.33147   -605.05217  -1071.96197   -488.29504  -1353.61286   -536.65268  -1023.16698   -811.13621          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        4.00   -148.42225   -382.52533   -148.80798   -223.77611   -378.25641    -20.41665   -377.62435    -486.2098   -503.29033   -369.95156   -177.77456    -191.1626   -294.09727   -360.90202   -294.22933   -300.26436   -302.44627   -233.43328    -331.4415   -392.79411   -246.23507   -410.93951    -78.15562   -162.72047   -171.38547     372.5043   1207.98254   2130.81733   1416.92045    437.32126   -185.56433   -283.18499   -336.19503   -273.16573   -318.48987    -596.7611   -260.31246   -291.65593   -329.07429   -421.72274   -415.38299   -406.00556   -280.95554   -278.46741   -346.69893   -271.17586   -154.33671   -552.86066   -257.07954     -256.765   -369.16786   -354.88184   -252.58259   -334.13809   -355.27474   -300.49777    175.64616    967.36752    1772.7381   1088.83399    873.04328    167.35494   -250.65567   -229.50013   -373.92498   -248.78751   -353.56666   -217.93601    -135.1594    -96.47422    398.37626    937.51986   1392.10864   1555.04855    866.38304    761.09869    583.49358    589.28655     274.6238    143.43276    -131.7514   -280.70204   -510.35453   -408.73685    -374.5791   -302.35968   -372.23434   -278.88855   -142.87679    -41.38484     71.24867     58.32383     15.94342     98.63245   -207.87525   -272.99439   -437.62827   -371.16509   -400.84244    -324.1713    -40.92469     97.93166    197.77153    178.76399     71.02509   -170.33559   -326.03023   -254.00746   -185.60203   -302.13227   -397.86325   -206.67611   -520.03405   -295.13765   -267.58177   -155.34806    202.95714    300.60085    312.39835    197.59551    176.70763    -110.8023    -98.81866   -154.16888    290.18156    563.14063    656.66518    666.17643    333.20096    177.41731    -12.95984   -244.75229   -237.61558   -364.06782   -264.47406    -39.94589    201.29682    892.03836   1546.37401   1758.15563    1978.0759   1608.35444   1064.90201    641.30001    102.13102    -18.84682   -262.93856   -115.24753   -274.65013   -229.93231   -209.24563   -281.03584   -316.30321   -388.26743   -341.43302   -310.22541   -174.92819    -208.9533   -198.98086     20.40633      0.58406   -157.79503    -56.18879    -119.5967     -5.89542     -5.66105     55.54656     -1.39318     78.21838      54.3666      1.70657   -222.53843   -347.18226   -236.28331   -145.10439   -269.69888   -202.97326   -194.01153   -247.29985   -187.71803     33.46131   -197.56266   -327.57112   -358.91506    -300.7213   -299.89722   -347.87055   -210.27554   -296.11227   -180.09611   -245.22174   -222.87965    -28.32964     26.41558    -32.09417    -20.69938     39.90957    134.18749    139.14446    308.93159     207.7796    157.26215    -13.99309    -67.24229    -30.60243    -20.30707    -22.17858      5.15985    -17.95049     52.47457    -53.88215     -1.94058   -133.01018    -44.13373    -98.51495   -294.87732   -308.50352   -357.32326   -278.05386   -457.87789   -268.19496    -231.4981   -180.45237   -161.94583    -26.50762   -216.58917     26.73229    -57.39353    -20.14942    244.36915     60.38431     212.0439    131.93378    121.02007     98.58595     -8.44818      4.61385     19.86646    135.18469    -31.40941    -70.79769    -91.42783   -148.17132   -134.19566   -114.69986   -212.22943   -356.44135   -285.05586   -322.12374   -258.61416   -255.24508   -317.82092   -177.58541    -152.8484    -51.94266    -84.44866   -101.60257     14.38871      61.6917    131.54487    213.66176    234.50392    375.38391    659.85814    728.19686    743.41207    573.49763    631.93621    588.16472    433.21303    473.92746    175.40724    116.32055    -89.70354    -45.48293    -101.0764   -166.67286   -243.29503   -203.23046   -156.47827    133.51653     25.76769    161.54972     96.20843     57.41342     80.50378    -16.78915    -84.13798   -195.01066   -259.97938   -165.88408   -235.83196   -278.76831   -294.51294   -294.37684   -342.92571   -360.17425   -433.28158   -174.10449   -293.19099   -277.04288   -252.06429   -219.85301   -203.13179    -359.1247   -227.07911   -142.69507   -264.89622    -62.56672   -146.72581   -214.29316   -148.11504    -13.51643     59.60438    187.67504     13.74382     68.33709     60.29305     23.69609   -136.34076    -44.15246    -23.84277   -137.32843   -137.08574   -122.12811   -116.53402    -97.40361    -253.7769   -206.88563   -168.91646   -331.00673   -263.19208   -162.19046    -16.66859     82.89224    -99.99579     47.70224    149.75067    -81.38964    -27.80074   -110.30825   -101.56173   -169.17731   -140.12358   -202.81698     59.87895     -5.64685   -142.25232   -193.42263   -209.12675     12.57455   -340.34348   -211.00576    224.61109    105.77357     79.69581    147.51015    215.08922     214.5775    143.34488    368.41493    273.59453      93.1767     57.46135    239.05888     -30.1571     77.88659    -52.28786     -7.45144    -93.02204   -113.05163    124.01566    -38.31734     82.34345     26.71912    -68.25494    192.60049   -379.27795     77.28968    -64.71007    -90.37254    -44.75101   -107.59839     -361.693   -246.69864   -184.61493     98.46545   -290.82008   -365.91008   -317.27584     68.13325   -176.37827    -35.36589    -186.0737   -256.22693    -121.1162   -291.04518    -381.2114   -394.79793      6.08344   -244.37452      57.5242   -225.66835   -257.22832   -118.89435    -222.3381   -120.13907    -101.0578   -361.86801    124.77311    -77.96182     -8.90962   -394.40928   -334.86826   -329.51217   -381.15149     54.06811   -320.63689   -327.18254   -366.03397   -393.19667    -399.8507   -383.27537   -262.45535    -458.1573   -351.26148   -440.35326   -626.52026   -774.88939   -446.14577   -707.42675   -690.65503   -599.73586   -756.67936   -387.28171    -626.6104   -807.27084   -740.00048   -724.36047   -819.42885   -768.27772   -840.21743   -837.33324  -1139.30761  -1017.08582   -889.62367   -989.81815   -923.57636   -961.96078   -874.29705  -1002.90751  -1048.94095   -921.10304   -825.85394   -943.18272     -831.909   -881.14692  -1209.38185   -947.65839  -1267.44858   -924.51037   -759.48987  -1013.92862   -909.09981  -1153.67146  -1184.96973   -996.12476    -1223.293  -1007.79986   -704.09903   -888.10294   -685.17791   -1221.7442  -1057.64328   -821.17576  -1048.52889  -1096.26566    -885.6476  -1319.16358  -1341.75642   -894.08706   -800.42105   -862.71576  -1234.04014   -776.06202  -1142.88612   -683.23492   -791.24251   -735.65876   -697.25121   -797.69875          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        4.50   -142.09025   -513.77981   -220.03831   -157.52259   -389.86607      8.99908   -414.41069   -340.03324   -550.68601    -346.7841   -358.36303    -94.49017    -441.5073     -395.083   -327.21887   -349.82646   -320.68669    -42.12139   -147.07906   -467.27551   -274.86068    -515.8443   -198.49556   -261.13957    -218.5507    413.39282   1411.15006   1896.90571   1583.74274    484.13984    -92.61271   -103.05727   -426.79319   -275.49794    -378.8661   -512.75083   -169.75828   -333.54177   -376.05271    -361.1468   -541.49012   -251.75748   -216.84801   -328.35193   -352.68457   -414.10958   -117.05262   -398.63548    -258.3526   -356.77287   -211.83653    -486.1045   -322.40474   -336.01152   -252.88837   -154.79674    159.20941    909.21017   1606.90824   1469.27307    741.03086    105.52909   -170.04933   -145.47979   -262.08232   -354.60543   -235.50916   -250.97836   -226.74246   -282.40575    540.99463   1006.30734   1502.40097   1495.88492    980.60524    778.43565    503.05865    586.85271      274.484    -37.92098   -230.63897   -317.46796   -385.15478   -399.36018   -371.76387   -397.85364   -290.00466   -347.95089   -253.52776     -60.1027   -141.26602    233.00395    -11.41516    117.52303   -161.27847   -291.23395   -237.18793   -279.12903   -307.67914   -193.02338    -239.7164     38.98296     52.63008    -52.93509     51.85223    -49.74417   -251.67964   -303.74784   -223.93267   -339.49621   -380.83774    -372.0126   -341.15636   -334.24926   -234.43167     -76.5119    194.04534    233.79789    291.59003    290.36753      9.77685    -95.30623   -112.27176   -105.67271    196.14888    530.86575    656.85938     666.8428     442.1076    269.27426   -132.01535   -330.96645   -329.37388   -339.87711   -123.90387   -160.16572    285.04025    913.91916   1442.73658   1947.96501   1878.90269    1623.2326   1218.82534    586.48672      4.37573   -100.11825   -167.47157    -219.8732   -214.21658    -251.4324   -312.72217   -279.18927   -358.09188   -294.92925   -306.80988   -243.88415   -265.52124   -208.51401   -168.08241     55.98887   -104.92701   -119.42681    -40.64013    -86.89543    -34.97477    127.87872     40.50162    152.48516    141.78458     -53.7557    -18.64343   -133.94225    -320.6776   -185.98566   -290.71752   -253.35752   -238.67684     -162.012   -241.54408   -229.20745   -320.26594    -243.2481   -323.72515   -388.75297   -331.42674   -340.63475   -374.71397   -438.13027   -294.28695   -332.59491   -239.60091   -178.15659   -101.58439    -95.86683      94.7896     56.77575      63.9167     69.87338    113.77565    184.98212    178.59584    142.02906      -13.826    160.85768   -232.56555     35.31105    -68.87229      2.93302    148.89032     69.99269     71.38976     -0.36618   -223.45105   -108.64092   -374.49099   -128.46325     -202.183   -304.07494    -254.5921   -358.00328    -349.7322   -401.04345   -329.36073   -207.67651   -114.21689   -225.90656    -46.93001     20.34238     43.66388    130.60887     57.00962    186.05393    102.31476   -153.51032    -58.61105    -75.28433     -2.07766     24.81062     34.13518     39.00244    -36.87368    -42.93478      16.9846   -147.90658    -262.1042   -228.27943   -282.09206   -360.04164   -251.87035    -340.2266   -316.61314    -47.77473   -160.61065   -143.07809    -265.6015    -53.80707     12.13868     93.11231    126.44978    139.94487    193.33992     369.4693    462.42752    478.09627    627.18555    655.98644    763.37538    808.54629    514.14498    411.26007    180.82204     77.82764    107.00312    -17.30304    -124.8508    -41.74418   -143.67861    -44.85607      6.24081     -5.17478    133.23652      211.265    210.15562     56.07091    -24.06854    136.30938    -52.41345    -36.78057   -287.10694   -103.47999   -341.55121   -251.66066   -204.49764   -384.36111   -294.05007    -411.0432   -299.36106   -280.88393   -332.22759   -249.52055   -258.42822   -256.43873   -260.96894   -208.73905   -298.98392      -4.4261    -73.67814   -157.37051   -279.82497    -146.7063    -98.36006    -74.70932     96.10012    -48.85131    117.56433   -104.17404      89.1053    -56.61599   -212.21197      27.5509   -212.61231   -133.21698    -232.3454   -188.02484   -110.91092   -135.62893   -214.14314   -243.00866   -202.20211    -79.14073   -158.52768    -42.76894      8.01211   -181.15759    -51.83359     48.92339    -18.90716   -219.76274      8.99269   -125.55096     21.67872   -153.44391   -184.78917   -184.44336    -204.9358   -124.94781   -289.73459   -133.13231   -118.57198    -17.77832    -83.76403   -217.13522   -105.13948    -26.88902   -101.03146    102.08835     31.83273     343.2675     82.84284    259.88567     -33.9966    392.62224     40.81526     74.77324    352.44527    294.22342     97.77748    -39.35725    112.02713    124.80531    104.11363     83.77129    183.71117     86.91187    -56.64127   -173.03311    166.09078   -128.32156    -25.46439     26.04799    -45.70834   -165.15655    -178.5935    -37.95815    -169.7942     80.65556   -236.51654   -235.18837   -127.82686       0.2977   -303.60009   -101.74939   -133.38781    -31.35365   -335.69144    -30.06466   -212.57009   -394.47698   -312.92858     -102.204   -152.42424    -90.56233    -73.82642   -138.91837     39.77012   -264.68124   -120.24717   -208.81103   -241.52807   -459.57064   -147.59692   -221.53186   -225.80558   -443.49093   -331.93196   -192.02837    -87.18291   -127.76983   -294.73684   -301.04701   -186.80059   -264.96223   -276.04195   -386.13503   -297.99069   -506.31817   -675.94887   -588.95889   -658.57661   -357.65717   -570.59375   -407.59508   -571.21709   -583.17372   -656.19013   -736.01412   -762.49118   -806.91827   -422.00222   -556.04842   -914.24385   -683.95517  -1039.66836   -799.33108   -739.82301  -1082.71669   -866.06993   -769.12341   -913.26997  -1051.93075   -906.01648   -745.89009  -1020.93557    -903.6365   -885.43717  -1065.27642   -702.64415  -1038.47396  -1140.59464  -1297.34353    -799.0154   -892.86019   -917.66838   -913.10477   -863.19357    -883.2665   -979.10847   -805.24357  -1123.89136   -916.98242  -1151.96543  -1300.69233  -1040.22216  -1003.02181   -937.57106  -1117.82542  -1416.87372  -1183.98098   -780.36324   -833.91715  -1543.05024   -877.68783  -1179.98953   -828.72675   -820.61582   -715.26792   -706.88646  -1039.45883   -874.09582   -765.61963   -368.01173          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        5.00   -124.72187   -654.52578    -306.6978   -127.97158   -349.93956    -148.2411   -512.75375   -124.91324   -583.94425   -333.85182   -390.25722   -275.50713   -447.44252   -395.82697   -357.39181    -329.9054   -331.72925    -162.1365   -234.62214   -355.52199   -479.89699   -536.69077   -330.22956   -124.55108   -114.38908    220.87776   1502.79666   1717.94026   1421.14945    486.05731    184.27326   -332.10437   -514.03584   -317.23889   -409.22261   -460.19739   -424.72902   -151.31818   -331.25424   -266.72588   -276.28471   -228.21329   -350.34103   -309.49954       -329.0   -321.17312    -259.9491   -175.15648   -492.28211   -341.75864   -358.86255   -457.79013   -374.34306   -410.83002   -124.91341     33.17843    272.64167    906.84606   1602.08523   1448.25112    810.82353    100.08729   -250.26515   -114.30076   -251.67995    -376.3187   -402.93341   -203.38248   -162.31775    -37.41702    335.00428     968.9424   1499.11248   1263.05063    929.27461    601.53995    680.49282     529.2038    212.89773     19.73435   -232.10267   -399.51463   -307.50968    -357.5144   -252.01752   -307.63726   -323.56353   -286.47009   -167.04416   -126.18331      7.81305      9.48246     86.18314     76.80514   -173.50314   -302.48657   -113.69013   -369.74607   -406.75555   -232.53446   -178.32308    -86.46263    172.66203      4.68798   -120.70533   -214.79903   -241.03422   -369.49757   -375.91232   -440.01067   -315.07369   -323.73486   -460.11684   -260.00403   -243.03792      5.36784    102.41974    339.90559    310.60914    275.62007    169.84615     -31.7603   -117.30202      50.6622    278.33472    398.10168    592.13952     548.4147    472.07748    327.19929    -65.99101   -186.70459   -331.61027   -235.55378   -159.71041    -83.15751    287.87853    803.40623   1502.77933    1884.0766   1951.46885   1834.09815   1090.81757    446.28914    131.89841    -98.58621   -230.71977   -228.15535   -134.69279   -261.30983   -400.19024   -348.60247     -267.477   -218.04505   -332.06545    -182.1114   -341.06588   -290.07117   -296.52277     86.85737   -195.36296    -16.34418    -196.3386   -200.22335     -4.95548    -49.23618    -30.92216    174.76708      3.81975    -109.6627   -357.05761    -347.6514   -339.51783   -292.35113   -218.09767   -121.06388    -175.5026    -44.10355   -226.34381   -135.24502   -213.75553   -215.21207   -352.87037   -432.66003    -223.5222   -280.25265   -307.06413   -234.97858   -458.05852   -331.68048   -245.95877   -177.30388    -196.6969   -101.70487    -84.21927      -5.5823    -67.37726     65.85697    332.70479    193.14535    287.86237    114.12663     20.10263    -87.80938    145.06321   -107.80098   -124.35304    -42.47356    181.41052    192.09555     74.55838    -18.95648    -64.14074    -167.8745   -173.33838   -155.66746   -186.59153   -251.45361   -321.25441   -317.64114    -228.7259   -189.20298   -247.91366   -159.67352   -234.42183   -107.77876     24.84033   -141.95994    117.39222     92.59512     87.01813     93.86327    -23.67024   -160.76233    170.43066    189.94846     12.80033      5.36596     39.84975     26.36005    109.07845   -293.67282    -196.2219    -52.55527   -324.56481   -174.18479   -243.41079   -232.18711   -310.40835   -342.91033   -184.28405   -329.26673     40.33243   -163.65888   -107.78703    -57.80203     125.0438     14.91554    -33.87494    205.42659    285.10189    365.46967    457.92081     641.0475    700.99864     816.0014    574.64411    773.36588    537.61634    418.27077    363.73382    383.12747   -101.37109     75.61627   -152.79009   -110.07324    -77.21321    -39.96322    -16.02704   -110.32678     71.40821     31.60565    126.75916    121.05214   -285.91725     24.21615    -35.40988   -123.47584   -120.81585   -173.76654    -139.4876   -261.56445   -340.27533   -229.21169   -349.17637    -223.8931   -417.73195   -362.55058   -445.85852   -163.54632   -345.04174   -344.06421   -263.56849   -309.89396   -192.07916    -295.9912     -8.28474   -168.58484    -78.13662     63.23428      0.61971    -34.24335   -133.67369     16.26431    115.05824     16.37024      11.9148     30.29072     38.29579     -5.24114    -70.45661   -285.22488   -132.58433   -202.09322    -74.97619    -397.0875    -86.56222   -122.27024   -264.78065   -131.78204   -120.66789    -84.80539    -76.44308    -22.89199      19.8335    -36.51311   -107.31396     58.40757   -124.57196     -81.8178    -62.66434   -239.76024   -166.48382   -265.90618   -115.20507     15.42878   -171.73375    -179.3389    -68.18239   -169.61721    -91.26313   -155.68425     77.52417      4.22313     97.87676   -123.16932     -3.40683    -55.19758    141.13201    174.97597    374.93869     32.57448    234.96404     99.75632    127.59067     73.71202     87.44952    232.30937    -12.45188   -128.07177     31.43148     75.24648    -14.00217      -79.775    -17.04918     -11.1941   -303.83685    -13.60661    -55.56522   -148.48455     66.77316   -292.33084   -129.16476   -103.58335   -120.37283   -250.46493   -140.76427    -498.2626   -141.43675   -527.10969   -200.58593     66.87006   -161.40081    -77.01172   -484.14648   -454.19361   -312.79697    -34.41718    -172.0878     -225.397   -105.00792    197.18927   -291.58648   -321.77647   -103.64389     44.97541   -257.94612   -116.76538   -600.63352   -162.05663   -455.42297   -118.53019   -305.67608   -289.30385    -246.8529   -335.59557    -247.2384   -347.43855   -183.01494   -338.59938    -416.6069    -352.7187   -220.74708   -413.78626   -146.00952   -352.46181    -446.5626   -599.91575   -586.78783    -515.7772   -449.14494   -752.23659   -461.27246   -682.53408   -825.74473   -447.00006   -946.03106   -769.55659   -750.82498   -928.60621   -858.65431   -530.53839   -948.09409  -1010.79655   -932.54151   -893.77478   -787.20771   -872.72955  -1319.37305    -694.1438   -812.69829   -638.27348   -921.21606   -961.05205  -1111.85156   -823.98477  -1021.43496    -788.1702  -1129.33554   -794.50349  -1019.07814   -945.09568  -1222.26634   -724.86247   -613.00965  -1099.54728   -919.69805   -822.20948   -954.70546  -1134.03694  -1091.63372   -957.14602   -945.90137  -1010.90433   -990.86254    -995.4436    -915.0816   -604.77967  -1309.62966    -1191.353   -802.18719   -866.85985   -886.85247   -983.66918  -1264.35008   -929.50832   -908.76315    -775.6301  -1191.35154   -326.36078   -808.63283          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        5.50   -110.44992   -629.53738   -252.27844   -185.90803   -280.85329   -311.04716   -576.15694   -157.30047   -441.39078   -355.44457    -344.8805   -413.91516   -358.45039   -369.99655   -339.33337   -181.05213   -353.68409   -219.34882   -419.74527   -311.30681   -519.36486   -427.88654   -387.61036   -135.62971   -306.83857    386.41976   1244.82791   1863.48011   1324.38893    447.63294     20.10679    -270.4238   -330.76404   -476.95969    -374.3103   -543.60546   -377.81441    -98.57301    -417.8605     -350.272   -230.37998   -316.68888   -372.13033   -299.41288   -366.87426   -308.88482   -274.13625   -255.71344   -354.42351    -360.0352     -386.441   -412.66351   -361.78402   -321.55364   -312.75661    -188.7276     189.6698    962.38647   1458.50266   1493.34125    750.38248     62.42015   -143.90729   -236.41509   -331.89083   -277.32229   -316.47757   -202.86439    -36.94426    -13.21659     396.4082    967.38482   1513.24026   1314.19934    736.02076    785.53459    646.74427    528.93476    426.18684    -60.10163    -57.13541    -160.0895   -205.09089   -381.62717   -344.12247   -134.32797   -400.49584   -232.07634    -266.5293   -300.29529    -29.94765   -143.40342    265.17525     -56.8692    -74.99286   -167.99842   -328.53455   -152.44184   -206.73408   -260.16571    -92.14864    -207.1715    -75.03047     55.60217   -144.78188   -123.05999   -312.45328   -310.69606   -237.20556    -337.7461   -183.65724   -385.08464   -420.66559   -368.97876   -272.91983     81.91086    234.38608    295.75742    375.60034    176.05441     194.9535    -89.76342   -290.95178    -10.70297     97.57921    299.29001    580.42718    624.32317    412.84199    134.85021    -23.56155   -151.19514   -203.24368   -233.56229   -228.25431    109.27078    342.50465     640.9708    1407.4826     1826.038   1968.51385   1856.94658   1118.07877    786.11656    289.63851    174.99346   -124.75571   -275.99373   -211.74149   -292.12585   -251.43621   -206.32384   -323.30489   -388.66218   -402.79056   -266.54538   -264.89376   -280.55345      9.81178    -86.71284    -139.8088    -80.03167    -99.81088     -28.0884   -167.74181     52.76583    197.48692     24.01517    -10.88432     -74.9651   -186.15785   -341.08277    -462.8864   -294.95827    -179.2185   -131.62833   -156.56815   -231.71672   -175.74753    -43.47169     -122.775    -134.5939   -237.30351   -427.40417   -259.58168   -273.58227   -383.05316   -288.17544   -461.79735   -112.18364   -353.08435   -307.88409   -217.36928   -120.83615    113.42769   -123.45245   -140.92304    163.12804     190.4359    255.55316    158.38717     63.63927    176.04886    108.65236    -47.72973    -17.17797   -172.28853    -55.87647     79.05143    187.03362     91.85657    168.90093    -25.88416   -294.60362   -148.34446   -147.08384   -204.11608   -317.36123   -253.60741   -334.42806   -397.18319   -345.88046   -294.00664   -306.61295   -195.60894    -11.77239    -92.70855      9.82499    -91.12263     78.93952      56.9629    -19.78052     25.21418      0.50081     22.31666    222.64629       38.094    -56.71192     19.73842     80.05227     87.52762     13.00467   -226.62905   -203.10656    -96.03439    -81.49085   -333.18604   -163.49611   -417.56658   -267.25677   -280.50516   -137.54302   -273.93164   -223.95374     92.21135    -19.60411     15.72255    -43.50041    -52.09429    213.99152    147.41083    305.59274    456.98596    795.37377    887.79236    597.48631    735.78252    562.26299    484.78401    406.79606    210.96078    239.80556     232.4569    -48.65847    -83.28059   -150.15104   -114.94067   -356.01482      66.2158     51.67681    -51.36447    253.49794     -6.90991    -36.32876     20.92162    154.47051     -3.14502   -185.52168   -115.50922   -259.99792   -132.68051   -338.54184   -297.80293   -218.47594    -340.7819   -457.94131   -460.42294   -236.79288   -405.78215   -204.58174    -248.3199   -349.74437   -347.26188    -259.1186   -222.06042     -120.679   -279.49757    -72.93509   -151.67141     -33.7785    -70.73524     95.35815    -97.86588     64.09033     15.34718    103.26552     77.77756     -40.2724      9.85679   -220.10742    -92.57919     -59.1613   -123.55876    -99.21737   -124.15464   -113.97182   -161.41834   -128.45344   -173.62628   -184.60396    -39.43817     -7.28309     12.83334     -70.9471     -93.9439     26.78529    -53.11789    -53.66184      2.64725    -31.73976   -158.25909    -203.5594     51.98308   -186.16875   -187.61751    -93.23812   -167.27621     23.43915    -81.37183   -180.96376     -0.85801     12.02243    -44.97803    -16.67838    101.53775     99.80707     98.65497    239.09879    211.55052    190.88817     47.62268    211.56698    319.59573    176.08294    160.56721     17.36874    187.88228     -0.15407    -17.51784    -62.98335    211.05776   -196.32644     39.67826   -165.21178    -30.39331    -65.34191   -150.12638       1.4794      3.47229     79.64855     49.10891   -124.49423      -4.5377   -162.85768   -144.86004   -211.58273    -50.21273     88.44156    -258.5884   -221.93757   -229.00786   -163.34947    -20.01758   -345.25523    -54.78639    150.07818    -77.42875    -103.6371   -195.82256      12.4197    -50.28925   -153.37073   -205.27828     -5.10092   -155.04808   -175.95034   -169.17836     -14.5303    -81.47785   -110.28307   -123.08146   -231.94131   -368.73349   -475.31959    -93.43074   -179.69713   -304.04452    -393.0664   -230.55677   -271.01663   -269.75536   -127.88187   -271.87477   -459.18227    -65.90681   -394.88792   -338.34487   -354.61222   -401.51002   -460.43511   -590.50393   -557.08513   -755.81611   -748.39935   -581.97435   -848.64098    -854.3876   -937.45515   -631.90189   -716.59351   -613.93164    -699.4668   -817.55942   -778.35777   -719.21291   -860.67508   -877.91451  -1008.13359  -1046.75328   -962.18241  -1181.00417   -943.08801   -994.53609   -803.74529  -1029.78385   -903.18578   -864.18866   -921.77755   -795.01107      -512.61   -925.04432   -981.29645   -968.99554  -1116.66091  -1070.95734   -834.79652   -1193.0413   -1360.4239  -1177.81872  -1070.98393   -659.13724  -1270.76075   -805.46532  -1488.77262  -1242.86341  -1145.19426  -1148.79452   -846.54573  -1051.68012  -1117.79924  -1180.65506   -863.87765  -1136.05167   -647.00233    -910.3908  -1184.54586   -227.82484   -764.48806  -1094.15046   -924.65082   -385.95428          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        6.00   -285.50865   -420.29109   -186.85383   -260.06059   -260.73619   -161.84022   -522.12043    -318.4288   -284.72332   -352.39958   -389.32738   -346.46084   -189.13242    -324.8747   -245.81753   -373.05383   -440.41416   -169.17686   -542.19344    -331.6346   -341.33912   -297.44267   -310.79684   -210.51315   -335.75111    334.70539   1454.04386    1773.1651   1429.77126    409.93465    -69.23002   -139.03729   -185.61994   -349.71148   -369.27604   -379.49237   -626.84931   -275.90729   -306.41378   -346.92169   -370.53443    -480.1258   -420.60868   -324.45968   -262.42757   -372.06162   -305.95474    -360.6366   -263.87685   -340.20025   -332.41243   -132.82766    -308.7149   -295.44752   -277.35438   -173.54534    325.40988   1046.34751   1428.49025   1607.17535    759.12564     33.99988   -207.02688   -191.33015   -221.11698   -406.94578   -302.78823   -346.10764   -179.54625    -57.20543    257.57733     941.2789   1353.21926   1277.79584     822.2337    658.68025    502.10113    549.01065    483.53406    118.12666   -229.45013   -270.81882   -295.10247   -231.07452    -466.2773   -227.24113   -241.76915   -376.67136   -190.21165    -41.86058   -124.43904    340.99284      68.1759    -36.37925    -219.8327   -304.14701    -95.02768   -230.90526   -321.52659   -343.57695   -281.19752    -78.86605     69.19533    134.44937     65.82822   -409.13306   -275.52869   -144.52572   -118.92198   -322.18684   -316.82779   -387.89653   -273.12024   -213.92927   -191.95791    -34.87492    -28.71375     74.96661    223.75437    210.83349     36.98996     24.10088   -214.23362     11.31793    434.30509     205.6152     650.4419    467.57684    328.80083    131.23078    -76.35546   -161.04213   -201.43135    -93.39425   -216.86033   -230.12158    211.41709    767.85774   1348.08111   1845.08424   2066.93823   1730.55362    1210.9343    604.13837    220.09409    -130.6036   -114.84725   -219.90104   -159.76754   -162.66825   -200.64371   -249.49034   -416.00383   -214.00398   -290.81355   -255.42442   -172.88377   -198.18502     -7.71746   -173.70017   -102.84326    -29.87598   -288.43302   -156.95836   -163.84631   -216.59244    -29.80505     76.42085   -108.71824    -76.06092    -211.8793   -159.83446   -253.54914   -281.71639    -218.4852   -438.69018   -112.20963    -81.37002    -192.7298   -197.17345   -166.14541   -253.09913   -147.79807    -318.1278   -301.43199   -523.20372   -277.66346   -291.03622   -359.41234   -232.87146   -371.54142   -262.10957   -113.09535    -12.54225   -249.74221    -43.11765    141.24983    241.11962    191.16054    349.38662    274.50594    125.78554     59.97547     14.45785     48.24204      78.6815     -1.97588   -125.65998      5.30486     96.80647    -33.27194     34.07205     -57.8012     94.44091   -234.49887   -299.70147   -219.99598    -315.1914   -418.14972   -344.93923   -242.06577   -115.19092   -234.98952   -268.44669   -201.57034   -112.30295     45.73074   -243.61165    132.60175     36.87939    122.11032    146.67885    -52.71581     29.70185    155.18193     108.5081    -32.50256    -22.95004      1.32023    130.86914    -76.18666    -42.71348   -176.29521   -108.92921   -293.17073   -212.50913   -303.50259   -324.46854   -241.41618   -392.80927   -194.62932   -178.78501   -158.62868   -125.18565     10.86119   -120.63377    -66.75066     52.98602    103.09627    241.99325    225.83379    384.55454     431.8276    703.34287    658.45121     695.4562    688.99135    525.56176    521.86032    259.99636    393.57728     73.38146     20.93432    130.80367   -198.07779     -8.04929    -238.5175    -97.01028   -186.04705    -98.03034     -7.19483    119.68926    171.17121    -40.34125    114.61749    -18.32708   -162.05565    101.82245       6.1066   -231.88807   -240.54909   -282.92266   -188.79172   -263.25115   -282.51445   -220.00516   -318.51615   -373.71399   -513.07441   -531.27244   -206.48509   -524.27042   -337.05278    -59.52633   -250.78544   -207.62693    -36.60929   -197.91243    126.09034   -263.43014     32.06269     78.29383   -131.27777     -40.7748     57.41522    103.93231    -26.52174    123.92094    -69.47167   -101.46548    -58.01607   -186.83694   -162.87842   -220.84642    -117.3043   -238.15627   -106.54013   -128.86133      7.25099   -176.88132   -196.50909    -62.67608   -100.82208    -29.05265   -106.83788    -51.83624    -38.62772    -22.66414   -106.04221   -104.10714     -51.5962    -39.37593   -146.10163   -161.49365   -175.33626   -200.27084    -72.90433   -107.19763   -208.95788    -167.4327   -121.72078      -91.763    -31.73206      1.47941     80.40929    112.74154     186.7226     79.10503    237.98874     85.01256    253.82259    217.85992    260.43539    238.12233      51.9079    229.58795    -36.27937    300.45709    -88.54669    281.21678    -70.71992      52.7997     36.07405    -140.0821    -77.78722   -128.40607    -31.55395    -40.91474     -0.96235     -66.6015    -122.7924      1.64527   -103.23292     56.38325   -356.15051     -5.49072   -370.27037   -250.41729   -247.43029   -229.70317   -377.72867   -136.96379   -481.84012    205.29039    -98.70391   -203.26596   -399.30135   -128.01254   -431.58204   -462.26463   -286.61901   -239.42706   -318.68139   -170.55813   -232.22592   -238.13716    -34.65283   -142.15385   -321.02158   -341.62789   -237.86348   -244.58886   -201.71088   -218.72317   -293.59134   -334.08107   -188.45822   -498.48866   -236.63325   -375.69304   -164.49341   -172.34593   -259.00495   -621.90787   -369.91131   -335.98822   -486.34149   -568.22324     -596.186   -589.46623   -638.52976   -461.95396   -601.47521    -679.1675   -964.06968   -263.11899   -599.39738   -701.66426   -788.10984   -721.70567   -750.02451    -745.4564   -802.95859   -803.54126     -934.006   -804.41613   -862.74263   -671.11097   -976.19719   -846.11258  -1031.07814    -890.2701   -776.49563  -1047.96478   -989.59394   -792.72438   -956.03218   -1065.7717  -1063.26558   -927.04947   -1303.8853   -875.02115  -1019.48011   -1123.5284  -1004.28698  -1413.37073  -1381.32396   -933.48573   -995.20138  -1081.05022   -843.16346   -1031.0242   -759.59421   -706.06985  -1211.80622  -1028.93146   -782.43122   -985.93641  -1233.33999   -781.62772  -1248.12903   -792.75076   -965.05585   -953.86157  -1686.32901  -1503.37719  -1021.92356   -1225.0584   -801.91262   -289.06777  -1022.28917          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        6.50   -491.80573   -199.22576   -194.33935   -291.99786   -327.38706      3.43496   -458.62483   -344.62767   -242.29955   -346.61816    -469.3042   -328.63817   -189.46042   -193.48116    -76.84293   -548.24071   -435.22603   -181.14103   -334.98211   -434.75985   -200.31568   -302.62697   -279.04619    -88.96127   -182.75717    279.31151   1414.52789   1676.38616   1369.10347      489.485    -12.23163   -209.77232   -452.81543    -349.4767   -317.67463   -285.00371   -386.42305   -421.13232   -256.44627   -319.02742   -406.87538   -298.83208   -369.14721   -440.36789   -308.23342   -395.51502   -367.35557   -419.27239   -430.38882   -212.82181    -94.04426   -386.14035   -332.34105   -310.91095   -252.76084   -163.33265    311.89214    1036.1915   1768.59354   1643.53182    596.39177    115.84306   -218.61559    -130.6718   -143.92056   -132.03619   -298.57442   -361.78923   -114.98534    -79.74194    364.98299   1030.14592   1513.18452    1315.6953    818.24735    550.18407    537.02056    569.09653    161.57139    167.55235    -265.6639   -387.31445   -136.42881    -97.16271   -408.66368   -260.59614   -325.15216   -349.80831   -300.38482    -108.6371   -101.75668    111.39462    -66.91659    -78.36441     39.69889   -155.34921    -17.00616   -385.73295   -300.00888   -261.17932   -225.85959    102.31172    165.32984      41.1618    -18.99888   -230.57888   -251.08611    -385.8716   -322.51818   -317.86303   -387.61313   -250.14666   -276.96871   -398.40696   -325.63841    -45.29222    187.55681    349.89954    420.03189    217.35297     35.50622   -176.18469    -88.08101    -21.62163    233.97825    376.99987    757.43301    585.73545    361.09367    109.06917     59.84538   -193.01587    -259.8127    -301.3507   -225.25627    -47.99416    265.82849    882.70054   1473.21373    1965.7363   2055.81718   1857.04701   1077.78108     549.9799     38.32576    -32.71083   -139.61897   -196.73484   -218.96238   -120.88249   -304.48276   -258.24521   -332.05255   -234.85635   -217.34307   -221.70225   -337.24574   -168.49321     -58.7463   -171.68062    -82.73396    -23.49351    -88.86253   -131.02706   -151.12655     17.41742    109.73301     27.38738    -152.7319      5.86962   -119.39086   -105.90598    -85.54444   -208.56356   -305.15198     -344.993   -323.44586   -153.85719   -187.02128    -74.73273    -244.4378   -285.65836   -366.38404   -314.74055   -300.90275   -176.37052   -456.88935   -340.59959   -383.35707   -255.23598   -164.68182    -176.4892   -153.56111     24.50004    -35.23847     -42.8375    113.66384    266.51121    218.37226    111.06699    301.08082    185.25606    276.45221     47.59025    -99.13743    -137.0117    -96.69844   -113.52963    116.12981     29.62254    213.36146     43.17578    -70.35641    -87.66614   -160.67283   -110.84328   -298.34937   -260.85154   -414.66479   -267.55111   -267.40522   -197.03216   -372.13299   -149.76213   -161.54597   -180.18825     42.86473   -139.97304      37.6618    -26.30993    156.42938    111.87494     22.78579    -51.82816      8.25869     69.84765      16.8017     41.98309      76.0673     73.82742     81.86017    -75.56581   -103.19323   -203.60465   -268.94356   -254.66629   -272.77444   -174.39898   -204.19959   -372.77338   -399.79288   -112.65583   -167.97844   -182.98625     -14.4296    -56.42064    -18.62913     85.20509     154.9729     142.4714    232.26609    476.81537    486.77425    550.04015    619.14965    589.95148    558.68886    421.51877    520.03538    479.56616    327.87579    158.19751      77.1681    -43.93526     53.36253   -143.76424    -87.41836     42.02854    -61.93635    -18.10145     46.47695    100.78139     35.59491     96.31118     73.64852     18.69893     -31.4396    -10.31827    -63.22282   -308.14739   -216.39309     -239.439   -326.56843   -300.80675   -258.04368   -257.27868   -349.09573    -353.3561   -245.07058   -333.90401   -195.10473   -285.34111   -241.24768   -376.60865   -176.60256   -246.96308   -200.07116   -314.24038    -51.57591   -139.26817   -106.77898    -31.56732    -98.49747    -28.36881   -148.24001    -96.52338     96.29469     178.0558       0.0824    -81.00132   -126.27911   -288.20625     95.32753     53.41549    -16.75032   -262.35455   -315.99392   -254.19708   -153.21912   -195.14779   -142.86471   -176.51455    -61.74011   -121.27291     -2.83007   -149.74237     92.05934     32.16539    -30.40372    -43.43464   -159.56479    -19.94505    -73.68309     -7.73613    -75.51353    -63.85818   -137.70955   -193.71767   -264.86455   -274.62959   -377.58471     39.10655    -96.51287   -161.29665    249.27518    128.71799    304.67925    144.37268    129.40537    227.05234    131.77027    126.37142    207.66412    161.52072     76.87891     91.81833     193.6736     36.61728     18.90263     76.16287    157.57611    -92.74404     96.54524     13.47254       4.1909   -298.35222     -9.54376     74.92214    -54.61045    -31.23167   -174.86133      -1.5654    120.08181    -57.58131   -187.61859   -161.54538   -251.58404    -61.34187   -236.56417   -319.38259    -19.34241   -126.00962   -621.07494   -166.31524   -215.64083   -137.59266   -149.68022   -339.32404   -118.68029   -279.74136   -275.40911   -295.88391   -246.81307    -47.43486   -244.03526    -101.5332   -556.61544   -277.81604    -198.5031   -652.73584   -100.06016   -308.94558   -256.97688   -371.69752   -186.77059   -432.78221   -209.17388   -399.29832   -159.31169   -104.51697   -161.90292   -353.62454    -297.6341   -344.14822   -375.47671   -541.16362   -397.28289   -560.98084   -453.58365   -435.89438   -628.24179   -425.73984   -336.29057   -396.70978   -748.53893   -715.45021   -878.80443   -738.96667   -902.75836   -731.01319   -737.84291   -960.67267   -738.59592   -914.62447   -492.59868   -952.86627   -708.26761   -946.29824  -1030.54653   -838.21962   -898.83938  -1066.02625    -795.8238   -881.25879   -956.43786   -809.85617   -732.80168  -1027.85438  -1069.69975   -858.45594   -913.87111   -782.94048   -851.76653  -1181.68801  -1409.12582   -949.67668  -1157.96816   -921.29045   -616.86621   -1489.0353   -583.58897    -954.6808  -1158.33501   -999.61062      -961.23   -881.56856   -955.98945    -959.6408   -463.52138   -723.06836  -1093.50383   -737.36861   -666.03959  -1113.33896   -906.81839  -1105.20144  -1050.65437    -355.2656   -936.68549   -256.68247          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        7.00   -678.70607   -140.05958    -337.3042   -313.92651   -392.99585    -61.55531   -402.07681   -260.25503   -210.47479   -359.93372   -308.71887   -410.55787   -249.00796   -225.03717   -183.75222   -343.27601   -316.52931   -287.10238   -258.77704   -256.51918   -319.09219   -359.37723   -382.23337   -106.32062    -211.5128    320.42214   1421.50014   1839.77465   1587.13544     713.0132     10.65949   -338.11398   -420.37351    -146.5922   -304.62299   -306.38552   -392.62303   -417.79147   -281.52299   -213.37845   -224.41299   -236.71468   -213.41464   -451.48925   -321.41457   -300.03318   -334.27453   -339.20469   -309.10866   -422.70959   -441.68238    -281.3829   -393.60858    -441.8066   -318.88785   -290.66192     315.4154   1198.44512   1657.69512   1351.19779    709.51609     88.95134    -21.56206    -252.2529   -339.77275   -202.23418   -212.32675   -145.92838    -78.52908   -233.99633    332.95275   1267.44795   1526.68895   1467.80197    905.76061    621.95426    785.07171    609.34139    111.38168     91.46556   -385.25136   -474.28122   -297.36611   -275.86882   -304.73786   -453.75025   -432.87233   -271.82227   -348.16122   -210.41668     12.36882    156.64001    273.69408    128.67068   -102.56385   -147.47728   -376.25129   -282.01099   -258.37645   -253.18629    -102.2573     97.92989      2.06653     71.16447     23.02903   -156.95912   -336.25998   -329.52335   -232.72486   -354.05354   -178.13544    -396.8185   -305.75684   -234.74747   -311.49838   -103.51198    189.42865    271.71929    279.19456    199.30505     177.8431    -98.20837    -154.9248      3.21966    274.22555    507.28057    709.66766    552.14732    383.54045    139.34118   -141.55584   -231.01223   -210.18505   -254.86652   -196.76552   -222.26332    246.88825     894.4989   1386.56955   2000.24779   1852.82509   1748.08258   1150.74883    668.80686     306.1974    -55.99517    -68.65167   -152.34788   -131.74281   -252.95876   -175.63062   -365.77783   -313.08135   -440.16001   -229.52922   -293.59917    -116.1707   -159.52577    -82.28451   -155.63096     -8.21759   -118.83223   -135.47575     58.85109   -155.97344    -12.03793      6.73741     13.19488     -20.2104   -113.60796   -209.10061   -392.29903   -195.25599    -223.4399   -129.04838     -349.447   -140.62801   -181.76697   -226.79878   -178.82745   -177.73031   -157.84563   -431.25205    -220.9879   -366.44658    -311.9854   -420.58245   -371.64824   -365.55507   -325.03871   -269.99685   -156.96823    -67.92334   -168.83457   -195.37206   -126.01825     -15.4702     200.5477    190.24129      112.622    194.70182     211.6944     13.70236    -42.56743     84.59259   -111.90032     53.16672     198.2687     48.94016     54.67353     97.98423     72.12625    -22.58894    -68.89406   -131.46547   -311.66732   -389.10634   -289.06148   -257.94724   -314.89938   -426.87886   -163.61324   -258.57185   -261.74148   -178.81301   -112.16058   -192.07517   -119.60828    149.48102    -30.12411    151.99443     15.73901     60.14691    117.74406     58.78116     64.17998     247.7403    147.12062     73.28463     50.07754     35.59825     63.01669   -119.92408    -85.85799   -206.26429   -178.02128   -252.95885   -221.84581   -303.37866   -336.36965    -254.4941    -81.64631   -230.89671   -101.90974    -69.34642   -190.25034     97.53427    180.43821    170.92425    232.43283    304.13073    384.45874    220.31976    620.42875    741.10719    656.90261    542.22763    708.85788    487.33259    355.14441    283.06006    105.55815    -14.11484    -79.23068    -61.01116   -138.65453      -101.94   -202.49193     67.32984    -36.18288    -35.59237     86.81598     87.37053     28.58671      73.1296    168.22603    -85.29688   -137.37392    -61.32236   -215.14685   -310.48634   -265.59825   -203.31628   -118.02427   -333.44213    -492.4397   -283.96941    -322.4818   -328.97185   -402.29129    -269.3441   -285.54633    -188.7608   -122.01435    -65.57413   -260.84332   -271.64121   -150.25869   -144.23642    -71.93512    -11.29806    -10.37337    -48.55277    -13.74711   -179.29853    131.92872    -31.42145     26.56666     37.61719     59.15509    -92.14087    -161.8235    -52.25421   -164.62848     -98.8746   -204.66107   -250.54193   -114.72582    -44.10228   -214.26532    102.31695     -24.4697     48.36414   -285.12211      5.93625    111.89557   -123.31484      95.7569   -216.91421   -480.77046   -216.80477    -45.43375    -62.04776      4.11947   -273.88565   -191.73816   -138.45679   -135.02763    -278.1591    -93.41119     45.17422     68.53264   -137.01892     63.99986    -35.94777    147.41505    228.90624     329.4495    153.65167    370.28086    252.95301    255.09752     63.66866    209.81131      106.485      188.886    -46.21445     84.31984     97.98402     38.72657      -94.619    -34.01971    -90.36583     -9.71318     79.18769      57.0902     51.76523     51.01772   -124.13166    -36.51986     85.32832    -76.70794     32.98964   -139.04553   -155.28071    -249.9191   -405.66712   -234.50394   -167.00821   -113.92562   -206.24336   -218.34739   -280.09262     35.36131   -392.64027   -297.68482   -320.75794    -25.74615   -113.51719   -249.41866     -4.45548   -225.66592    -509.2205   -385.33281    -67.81386   -161.66069     76.58663   -407.30139   -183.76245   -386.33241    -12.96333     47.72526   -140.46254    -80.88655   -429.67266   -291.40553   -261.95987   -253.45949   -104.16744   -450.31287   -376.58858   -277.22731   -587.56551   -322.89215   -315.16273   -292.17658   -262.76115   -541.03362    -650.7587   -615.45184    -716.6432   -415.12106   -512.95954   -601.11918   -596.59526   -779.98508   -842.36059   -721.49752   -653.05551   -914.75355   -697.51277    -784.8041   -599.74666   -801.30425   -817.00543    -854.2178  -1148.86335   -820.28225   -885.02414   -743.29218   -850.27848   -977.95993   -933.81749   -954.28475  -1009.58807   -961.81517  -1021.42116   -863.46049    -861.1727   -905.69524     -925.332   -955.02789   -915.99906   -1086.3985   -951.25057    -938.9954   -787.82998   -638.19324   -602.96252   -804.86743    -901.9079   -716.90292   -860.35112  -1080.12971   -898.03244   -981.12731  -1015.91761  -1215.74631    -869.7716  -1298.78004   -649.89106   -571.77735   -536.37174  -1254.89773   -897.44236  -1005.31029  -1501.60522   -553.13378   -456.52909  -1441.22687          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        7.50    -661.8143   -130.48232   -475.73174   -317.21467   -368.11365   -236.01375   -335.82938   -249.18384   -188.13062   -367.39853   -131.25111    -419.8347   -303.56744    -483.8333   -325.87205   -256.76965   -230.87534   -533.39089   -250.17245   -278.49031   -441.97142   -179.63389   -430.39878    -84.51241    -28.96297    380.58443   1501.41138   1911.29364   1500.28651    530.09763   -131.49489   -297.10941   -363.80235   -285.40067    -403.2546   -352.16079   -409.11039   -392.49887   -424.40194   -273.82359   -351.72023   -263.26189   -315.09446   -534.86964   -453.04467   -151.02421     -523.691   -347.18848   -235.83033   -321.38221   -340.25427    -352.4947   -469.01062   -204.71428   -421.28891   -255.74746    216.61409     914.5962   1590.70104   1413.31933    775.46525    -27.66419   -138.51486   -325.66384   -164.16143   -220.47941   -191.39979   -141.19687   -242.92824    -26.33213    336.63862   1131.44904   1520.13138   1236.06335    883.44219    490.66172    603.73225    532.41302    291.01496    -69.85971   -224.55521   -115.46087   -210.62292   -370.54836   -401.94874   -296.42355   -215.90301   -255.95779   -241.95148   -222.20397    -28.50691    189.71739     219.3869    227.93104   -175.03475   -288.82506   -134.07695   -233.37449   -449.77231   -189.70543   -135.81923     56.48946     33.01393     25.50236    -55.70589   -327.23227   -238.50894   -332.13221    -446.4261   -360.82097   -552.65077   -349.36352   -491.41788   -346.08958   -258.09288   -141.59135     37.24074    315.97844    312.27889    210.07173     33.41881    -73.91097      5.90827    -90.64753     59.32437    546.15136    602.01478    517.31471    357.43308    109.29795     -5.50729   -250.41271   -243.80244   -304.43806   -208.74893     -38.2036    293.98791    792.07019   1332.84009   1807.62933   1911.43923   1736.48377   1268.67146     425.7466     194.7406     72.28406   -108.56483   -175.97119   -143.11118   -189.39324   -353.79981   -194.23726   -304.24656   -351.36704   -506.82244     -442.231   -229.48127    -243.1617   -263.05986   -283.94659   -219.27083   -180.81291   -101.28637     28.38827     -88.6613     91.39298    -65.32368     40.63676    158.21561    -40.92231   -202.47403   -180.45848   -317.72274   -266.64801   -335.59228   -192.21458   -153.80892    -27.97298    -54.52441   -202.62938   -204.45701   -430.72652   -367.93096   -346.20141   -357.03203   -365.22462   -324.31251   -332.75024   -385.20898   -295.99526   -371.06239   -138.96035   -161.28021    -63.51056   -105.85131   -139.66497    -26.40828     97.49952     101.7512    108.21793    253.62324    137.10795    180.14781       27.137   -152.73227   -193.36918    -83.90382    -80.26169    154.35946      98.4588    104.48913    -15.54208     105.9656    -88.53987   -118.15651   -264.85801   -273.87403    -386.3676    -350.0667   -203.71584   -315.33637   -219.86539   -144.06151   -247.52183   -310.11904     -8.63994   -238.20274    119.65737     76.81856     26.89696    115.49956    -103.1496    135.78447     -65.0252      29.3497     96.07981    -84.35926       121.92    -18.63363     70.65798     51.90606    118.26432     55.63318     -153.498   -137.28979   -460.63034    -89.62891   -346.50783    -195.4962   -261.04796   -220.45414   -204.19543   -225.34887   -114.59756     -3.00452    -46.71718      38.4844    127.38948     97.83604    152.08574    193.64527    495.00782    281.68046    668.30221    718.46854    587.33398    572.35313    582.08536    552.21123    598.94217    234.23265    197.15892   -136.83262    -29.64142   -145.19553    218.49853   -118.26573   -223.13159    109.78909     37.52324    183.23436     13.83519      7.10433     -0.40076    103.99867    -17.59456    -11.77927   -179.88582    -46.96267   -171.64895    -97.99697   -190.12168   -405.13863   -439.75051   -186.89448   -299.36615   -358.47997   -401.18323   -238.42488   -334.34544    -49.54129   -281.35874    -321.6322   -303.82393   -102.39417   -241.78156   -209.51536    -96.14257   -173.05958    -29.41696   -144.43966    -39.14587    -62.83595     33.08092    -16.44681    -29.20486    -92.10578      6.64688    -31.50077      -0.7369   -131.70678   -122.02914    -92.41837     -0.41727   -112.33511   -316.53062     -0.76892   -113.52423   -259.16176   -114.85455   -100.70845      42.7048   -152.35717     85.47121    -62.65522   -181.90838    -44.56639   -225.13927    -95.53645   -119.56225    -50.99981     42.61507   -174.01955   -153.75366    -99.83711   -231.88855   -363.14465    -29.44011   -191.46816   -259.18149    -43.14926      2.67665     87.55069     84.08479       9.6867     27.88277    193.14572    257.95423     66.05117    320.36196    279.47557    294.16443    177.67909    158.14045     113.9345    -70.52213    133.31868    108.37322    109.90501     29.33379    102.42845    -45.77984     49.13592    -72.82822   -185.97192   -156.67069    181.39329       4.2929     12.36519    -37.65432     58.28829    -36.34523     30.69269     67.58519    -105.5402   -129.40306   -209.12209   -106.96952   -256.73666   -156.01024   -181.03251   -146.03708   -338.50132    -12.57142   -206.17709   -141.19974   -147.26514   -268.09355    -281.2697   -209.37649    -208.8423   -151.39894   -336.30474   -178.06294   -145.55644   -127.05013    -45.99498   -212.95031   -154.27477   -317.91683   -293.53234     48.28952     -62.0968   -662.78089   -272.44826   -340.63324   -344.26625    -402.3364   -333.91173   -127.33772    -190.3993   -281.94818   -545.53143   -468.47936   -418.35559   -417.08871   -363.42715     -569.636   -458.72822   -436.89024   -501.45154   -442.72491   -519.43839   -665.36837   -760.07505    -837.5949   -852.53535   -747.87174   -614.25621    -822.1439   -652.45137   -920.01764   -755.62459   -647.26723   -567.60626   -971.85463  -1059.19164  -1024.24198   -806.85785   -1023.2801   -930.00284      -881.44   -949.61157  -1086.50417  -1007.44098    -846.6849  -1176.09377   -903.75575  -1080.30218  -1058.38812   -802.51737  -1196.70806   -862.83716  -1008.76687   -891.32491   -553.52668  -1105.73595  -1131.37037   -843.26646    -615.8642  -1556.43094   -708.28837  -1003.21836   -940.82841   -658.21029   -924.99508    -590.1906   -947.62987   -686.62274   -936.28475   -865.04794  -1036.26589  -1095.25694  -1061.91169   -481.33407   -188.50914   -682.94706   -180.80777   -818.70537   -338.61408          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        8.00   -657.58595   -126.68481   -427.57375   -328.88745   -326.11497   -347.67411   -321.28671   -308.49651    -159.9951   -379.37406   -134.22717   -390.83954   -345.86423   -484.58752   -358.28728   -273.38206   -195.61513   -380.50509   -315.55906   -406.92906   -422.38737   -115.00422   -257.17443    -89.19957    -47.62394    396.57721   1325.89629   1864.65053    1474.9573    508.04829    -81.45562   -198.77465   -302.25949    -404.2598   -436.30739   -472.25745   -175.12729    -89.31131   -499.24489   -387.57239   -135.30749   -352.77895   -402.71854   -276.60391   -559.14185   -452.08758   -373.67322   -563.96188   -184.54155   -117.68871   -361.13202   -380.87428   -554.11496    -376.1575   -485.95927   -333.05251    457.37506     720.4925   1539.56381    1323.8858    786.90711      3.35221    -75.24969   -122.37952   -334.83095    -313.2789   -210.48275   -261.03215   -246.82648   -106.99433    206.34915    1063.1814    1533.0179   1262.36711    850.04646    649.02053    672.56763    657.49639    164.41867     82.28521    -244.0514   -218.36088   -281.32145   -387.20469   -378.08385   -368.77967   -207.23701   -182.32194   -258.16403    -159.4312     75.15293    149.75582    196.92402     71.36941    -35.65572   -224.33774   -268.57727    -332.1476   -254.99083    -36.90978   -325.95083     56.44946    211.45803    100.63995     38.26081   -216.45011   -496.42114   -292.80654   -309.73614   -320.41701   -304.38443    -382.9777   -434.69439   -163.44462   -259.95209    -88.07241    176.98174    529.88601    261.25485    342.74544    -50.25494   -182.18788   -174.40625     31.43072    294.96217     402.6073    719.22451     553.1624    423.41318    167.61435    -50.25005    -47.08914   -171.37946   -458.40391   -243.17332    -38.60888    255.01322    742.09668   1571.72456   1933.85068   2084.69105   1776.35154   1151.19042    613.47875    231.64556    -63.32958     -84.5963   -195.09159   -286.65823   -172.60256   -509.01813   -418.49024   -335.75782   -326.77849   -134.17478   -320.98477   -273.55183   -166.68568    -286.6322   -219.07482   -110.43194   -246.95592   -189.02683    -61.81066   -154.44957     65.25383     90.20439    103.31737     26.21945   -103.47016   -133.94081   -380.62113   -148.37124   -253.89344   -240.72648   -257.15311   -328.68595   -201.61443     -7.02103   -159.15822   -185.66878   -280.75809   -420.90803   -282.38606   -200.46784    -341.6601   -317.69857   -365.28311   -249.63905   -348.36599   -228.56492   -111.45151   -251.12755     76.68696   -147.31945     55.24314    -34.61841    191.56112     158.9751    290.34715    134.91529    181.52436     38.23384      8.74184   -110.01483    -51.45755    -22.64522     91.55255    -22.18904     143.8718    -61.65172    -18.81898    -22.09167     12.48464   -164.89332   -284.94778   -289.87793   -379.33735   -316.27043   -345.56089   -332.78819   -362.17878   -321.40898   -145.79575   -240.03224    -26.27085   -146.46351    157.33277     52.11929    157.95581    105.20328    176.15522     13.25624    -30.91124    155.47844     -1.73425    219.86065     99.21461     -3.90101     158.0919     58.05328    -27.49532   -202.98475   -381.62318   -195.42525   -233.16183   -351.95369    -263.5211   -468.72116   -118.54746   -166.70437   -189.71986   -148.37007    -133.5487   -163.52143     -92.9137    -72.04085      -5.5374     58.16055    130.05217    249.77083    343.25741    346.55399    649.01365    805.85503    636.31312    596.18853    728.50565    525.93792    502.99787    198.99094    200.09106    152.48966    -20.55972     93.78883   -254.31677     44.38793     21.71405    -41.51849     78.90361     107.6597    111.31162      89.0247     70.46098     13.95655    -82.54642     -5.30245     -280.015     -4.87135     45.45964   -121.90147   -281.63203    -427.0977   -234.99412   -170.08047   -457.12927   -276.68706   -236.29208   -316.01167   -472.63375   -256.98369     -98.5161   -247.64237   -496.57796   -334.61201   -145.15584   -140.80596   -422.52937    -128.6562    -90.81329   -159.54967   -268.84049   -278.32931    -71.58237     47.70869     43.98116      -71.653     31.50287    -112.4739   -230.68072   -199.66637   -157.70528   -120.52815    -93.83065   -114.01155    -80.58302   -230.97279   -125.37615   -268.45568   -281.07335   -105.58782     -80.1348    -100.8461      9.29713   -129.33976      2.37127   -103.30495     -9.89222     -3.57589      87.7963    101.54433    -15.57491   -367.59528    -295.1624   -176.13321    -88.30406   -322.61566    -80.19817    -91.74996   -166.81274   -124.56758    125.25701    -55.80207   -181.32713     -63.4824    196.82505    180.31326    347.19071    117.40265     54.96434     77.99791    392.57284    335.99172    145.95721      95.6167    223.81521     -1.57889     56.05512     14.95119    106.01551      9.14506   -192.81674   -186.18299     12.52677   -290.89314     52.03963    -71.16813    114.80449    -55.08533     30.78854     69.40285    -77.36898    -96.71687   -136.08732   -146.68362     -34.6897    -83.45725    -63.40752      0.65726   -149.99677   -128.47874    -98.74393   -222.37378    -94.04532     47.08204   -174.83047   -273.66505   -351.72034   -200.67412   -285.86009   -243.55391   -304.19206   -262.54393   -224.04423    -17.16682   -189.50132   -540.88599   -227.76083   -286.19678   -202.65311   -201.63849   -215.16906    -370.9584   -216.90245     10.76902   -206.68853   -353.37517    -62.98638     11.07537   -247.20683   -241.87812   -473.90757    -92.78973   -361.73952   -460.82036   -383.44229   -431.08867   -284.73961   -581.48555   -305.55461   -740.68746   -757.67118   -362.78247   -486.80494   -403.90743   -694.85977   -896.18932  -1123.05725   -717.92746   -599.48837   -459.52648   -696.44328    -904.1111   -812.87107   -863.46121   -968.41582    -780.2866   -699.55504   -926.49153   -888.07845   -920.16268   -821.06972   -842.42666   -816.84033   -740.21662   -828.25192   -835.54738   -891.29202    -1007.696   -724.09243   -866.70647  -1258.39697   -816.89672   -899.19412  -1298.74691   -847.65391   -638.88455  -1034.77739   -978.67109  -1248.10956  -1073.86065  -1030.87349   -924.54562  -1427.53771  -1530.37073  -1194.84428   -1411.2442  -1040.39338   -514.08677   -753.06894  -1276.12187  -1446.25345   -1227.2235  -1352.65345   -107.65485   -352.88951  -1497.71807   -401.35445   -631.42437   -984.89903          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        8.50   -607.31772   -280.80785   -309.53316   -375.42239   -330.32513   -395.47021   -335.23948   -338.87286   -283.17287   -401.94612   -158.61739   -421.52433   -243.74589   -389.74332   -413.99791    -295.2632   -269.85406   -155.77969   -316.28109   -423.68512    -293.1332   -111.59364    -202.4185   -341.35233   -242.97142    458.58156   1290.13519   1916.04832   1512.63325    632.38555   -411.30941   -249.77242   -280.41023   -295.49504   -357.68744   -394.80423    -16.90002   -197.23481   -550.42215   -517.52244   -469.83804   -353.14085    -341.5948   -196.00223   -415.83691   -465.98386   -415.57864   -419.53867    -370.4928   -382.91106   -198.96743   -520.94693   -368.87764    -385.0038   -223.05349   -149.27303    328.71753   1069.88661   1466.67622   1371.31679    671.22911   -155.43184   -233.15131   -228.86109    -278.6791   -402.98478   -425.40721   -195.15039   -235.03019   -192.75611    392.17584   1012.23709   1491.09455   1388.61582    817.39731    564.37656    594.44012      593.878    265.21983     82.65348   -248.84426     -48.2174   -429.42059   -397.70589     -254.812   -277.28092   -314.15625   -389.91182   -204.76344     -170.089     -20.4234    134.77548    202.72222    150.60669     37.85315    -85.74324   -372.05994   -460.14028   -258.23181   -153.54636    -27.57867    -32.59088      37.8498      78.3502    -84.47584   -262.12158   -395.98682   -360.46314   -248.10165   -318.27508   -380.53552   -455.54663   -413.85599    -351.3542   -172.89351   -175.82211    139.91542    158.62866    293.06547    264.57687     29.23528   -146.86788   -167.17722    -30.94294    199.30374    408.79743    538.55345    600.39066    434.02882    155.67434      11.0136   -151.77754   -234.94752    -225.5604    -96.81242     65.56641    378.96232    842.38394    1402.4427   1909.37538   2016.10254   1761.18905   1185.76911    570.07174    155.18327   -163.88859   -184.01531   -267.30154   -150.84026   -193.69814   -269.48263   -274.76416   -295.13659   -222.06367   -306.54051   -176.65266   -248.69918   -319.18866   -321.75619   -113.55583    -75.85971   -142.38198   -216.84573     22.93371    -63.02064     13.26028   -125.56499     181.6514    118.44057     -93.7171   -207.79726   -340.46907   -297.46308   -367.60277   -270.20362    -118.9289   -308.46025     19.29705    -213.7779   -290.01041   -231.44166   -207.07727   -319.43383   -375.99285   -273.74375   -467.15247   -393.14614   -327.67378   -527.02728   -297.71671   -324.37938    -98.30928   -173.64671   -286.32734    -13.77866     23.11266    -32.89842    293.90106    211.69055    142.26711    331.92943    123.12439    115.68005    -95.49824   -100.23282    -65.55958   -127.17491    -81.61048     -0.03288    181.85534    -51.96138     58.31926   -148.93391    -31.84712   -238.03596   -217.24169   -235.72226   -294.90075   -238.68583   -181.52667   -184.23056   -217.01109   -448.56718   -328.30924   -114.68653   -185.55891   -270.57818    -74.41666      62.8674     23.08894     25.70509    135.01349     57.58439    -38.41859     18.96819     -3.88111     90.51112     29.58618    180.90235     14.09121     82.52938    -61.02339    112.37947   -271.31101   -158.71099   -173.97701   -322.23716   -253.45813   -122.52534   -266.75704   -198.16117   -124.23214   -214.36423    -171.1383     35.87714   -122.85058   -121.92958    110.51623     92.33341    305.35245    337.46149    510.75759    368.10847    587.62427    758.58155    660.18139    810.35938     598.4729    428.92193    405.94957    300.61232    219.65532      32.6886     -89.5285    -19.44055    -33.61626    -71.38748    -30.76951    -24.80738     22.35799   -122.04863    120.74907      4.04132    -12.37202     76.62053    -25.44019      2.01618      51.1619   -219.71629   -287.59404   -295.71496   -343.95076   -282.02928   -379.09857   -251.32857   -267.46772   -426.87443   -243.33462   -301.45574    -85.99529    -266.4182   -307.58028   -262.47305   -288.43014   -184.08831   -136.17983   -131.92028   -293.91395    -122.2214     16.67679     25.10755     40.53343    -66.95861    -23.46951     26.06044     56.68498    -14.60603   -121.82588      -102.28     38.74145   -112.77357   -126.39442   -230.88443   -137.12272   -217.85475   -174.17733   -147.68042   -211.35999     -146.133   -160.34874   -134.01517    -98.95898    -18.68001    -153.4405    -24.71065    129.37514    -23.83541     79.31545     -7.75814    -62.14285   -127.26235   -122.82828   -187.80973   -218.10131    -45.11251   -297.45727   -130.87335    -39.86033    -93.57943   -129.97199   -155.09965    -63.52682    107.85256    -55.32239     36.57272    142.61717    154.70649    168.34412     42.44597    348.64965    271.11396    368.31125    163.07133    339.48575     155.3053    159.96956    105.06381    136.67645    184.24586   -137.91351   -237.12756     95.20569   -307.66917    -16.35526    157.17061     -5.03859    -95.91466    319.82068   -139.99854   -154.47744    114.45836   -135.37837    -26.04994   -130.34804   -142.79901    -65.48259   -109.08889    -51.10392   -293.19292     -61.2408   -293.56007   -205.94994   -152.77071   -145.38911   -124.98892   -155.26458   -417.64348   -170.02466    -92.41295    -77.32636   -270.16618     -87.7805   -139.72237   -115.58175    -227.6955   -420.34187   -267.81187   -225.22453     -72.4211    -278.4099   -443.74417    -262.3543   -156.87725   -376.33666     29.36526   -343.53035   -446.82291   -260.42922   -230.91537   -220.91291   -167.46992   -302.94965   -341.68378    -304.8585   -265.81687    -450.0244   -561.52865   -591.05623   -788.10186   -492.41754   -302.04293   -646.67907   -611.98538   -681.49459   -709.35359    -712.9248   -676.05544   -503.56603   -700.76277   -558.08956   -786.17292   -889.54408   -737.23444    -712.1656   -824.92793   -798.05452  -1030.72075   -790.45704   -914.53299   -947.09157   -780.27641   -893.85472   -898.38996  -1088.62386   -942.52171   -981.74044   -871.23276   -987.63588   -872.43914   -955.28331   -799.85218   -964.60426  -1213.35994  -1009.94131  -1411.97178  -1028.43407   -464.34898   -927.89494  -1053.64325  -1023.90109   -646.81815  -1288.37904   -805.71141  -1036.77094   -756.03314  -1019.04973   -645.15942   -914.69438    -965.1692   -594.88744   -431.93442   -956.61627  -1190.31077   -920.38028   -632.75669   -730.19906   -913.13369   -669.39884    -524.8879  -1049.46439          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        9.00   -385.26966   -413.98196   -205.37017   -302.06691   -374.24084   -442.95604   -295.28436   -342.53124   -406.35282   -355.59717   -283.56302   -449.23653   -157.75573   -479.92667   -455.79335   -297.74801   -226.62961   -293.06404   -258.88751   -355.55614   -340.63876   -135.75251   -275.67098   -483.10953   -110.46584    431.88841   1417.14804   1835.35023   1511.52365    748.50733   -400.80712    -91.96025   -327.32816   -237.38699   -287.14435   -288.29934    -225.4336   -265.31603    -376.4001   -434.13097   -485.61213   -324.28018   -179.43301   -169.38191   -256.10451   -256.02898   -456.49372   -269.50432   -260.04851   -367.77486   -403.75831   -193.09005   -323.82339   -282.24811   -308.50673   -158.27659    325.45864    927.20935   1439.93047   1364.01927    694.32779    -32.46364    -69.44079   -309.97597   -338.52281   -346.52125   -331.88981   -271.09432    -242.4635    -38.39037    364.81914    885.99157   1516.62639   1258.75292     668.1157    734.82834    676.53133    531.44286    223.34559   -115.57136   -178.01304   -258.36243   -329.04659   -399.87192   -371.52477   -270.52292   -311.72547   -301.57294   -271.15176   -111.55186     166.5443     25.88699    148.85923    134.09459   -147.72221   -240.85109   -238.25475   -258.97655   -356.88591    -161.2262    -18.92129     25.34926    108.86051   -110.75402    -54.55836    -77.12538   -309.57878   -223.80241   -248.48667   -320.21058   -284.28916   -351.11033   -425.54573   -301.64045   -125.16778    -76.23637    173.80267    265.18646    291.61821    260.19403       0.8402   -148.95659   -136.31438     10.34778    174.48605    336.42603    685.58568    714.78578    468.34237    147.21836   -207.50352   -248.61876   -342.03297   -209.90465     -250.239   -251.83516    263.72942    912.44298   1549.54524     1926.534   1928.36985   1559.43435   1023.47471    671.26762    190.07521      9.94607     -3.56041   -297.84862   -220.59733   -314.47269   -227.17827   -244.52691   -286.86824    -337.4521   -245.07144   -143.42119   -152.84339    -89.82106   -275.60454    -33.35674    -81.27654   -175.71439    -193.9659   -140.03857   -195.49827    -25.36856     88.40005     94.95418    136.80129   -198.67851   -135.63754   -217.77643   -247.89893   -247.27201   -215.86833   -152.14238    -54.13738   -119.83382   -191.91528   -149.37841    -171.8926   -317.26121   -157.35283   -447.51923   -467.72667   -424.52234    -286.0587   -393.55675   -435.67649   -210.86845   -290.36548   -324.29185   -210.06899    -73.93681    -56.64486     -0.60816     93.44301    190.07486     71.84108      262.317    320.70833    141.15174     88.14323     15.65434    -62.00569    -57.53264   -122.38433     38.76529     90.90648    208.79038    147.97775      -42.112     84.56077    -109.7919   -153.45752    -148.6046   -318.06396   -265.15634   -436.65021    -338.6687    -290.5077   -334.81849   -265.24286   -288.96947   -318.88604    -109.1019      9.57017      2.19072    -99.57143     60.95177     93.73778    -23.23915    119.38862     71.81481    -68.15915      250.448      2.71134    168.20114     22.75073     40.70572     41.29868    147.57659    -23.47727   -186.22678   -273.68438   -185.14166   -139.05329   -408.06806   -144.20052   -334.53151   -258.57404   -158.58975   -196.85154    -210.4522   -352.63224     -4.26097     35.66917    192.00513     151.9006    354.01174    421.18237    502.49256     699.5126    576.45807     698.0364    655.95198    894.30087     625.9322    490.84838    369.24596    391.78285    181.60449     73.51695   -152.39932     -5.04463    -27.04695   -104.77945    -41.35603     10.73155    -48.92366    -52.37177    140.87171     62.89861      8.08711     27.89716    106.63721   -118.86606     18.47411      1.42462    -261.1078     -90.5652   -239.88051   -131.41898   -321.13218   -345.31415    -225.6933   -435.84906   -300.77407   -350.05931    -311.3913   -324.03378   -250.69212    -293.5371   -322.38838   -100.12328   -234.67464   -121.43178   -232.07566   -168.71738   -212.05754    -13.16875     -99.1325     129.4754    -20.76021    -83.30285     -8.19895     85.64749     10.85064    -27.78809    130.48973     14.16822   -188.74304   -188.69002    -90.97741   -153.99752   -200.47734   -243.58315   -227.27739     -335.576   -176.76465   -220.61187    -76.64476   -270.25735    -28.38211    -87.66933     99.96533    104.75615     21.80408   -152.38928   -190.27999    -17.64113     66.86335    -189.0903   -176.63688   -155.65691   -204.05654   -191.33005   -186.72571   -109.65838   -111.21502   -119.18918    -71.42727    -82.04923     56.92702     67.97035    102.10127    158.89383    277.21694    144.44232     62.65247    204.74243     13.68954     257.0939    168.44505    127.36845    243.69036      89.2546    308.50878     63.82339    128.84982    -104.9107     -10.5094   -209.61834     97.17457   -144.28032    -59.00225      6.33287    -14.01431    133.42279    -83.84128    -46.28624   -197.13604    -90.74998     62.22706    -27.52563   -206.14563   -131.38538   -240.37485    -87.93458     -191.563   -149.18203   -263.85158    -60.78292   -206.70701    -70.54385   -274.49225   -390.53271   -397.89231   -359.67067   -371.75644   -130.76657   -115.07402   -188.85605   -294.11124    -99.80946   -275.68138   -223.26824   -231.00802   -335.24406    -197.9388    -184.2814    -161.1999   -128.55519   -166.97827   -165.86194   -394.41048   -144.75283    -40.30575   -229.33588    -28.07238    -58.11271   -257.75264   -361.26186    -550.3598    -61.26744   -132.85285   -313.60741   -328.72162   -758.78671   -588.72787   -325.77306   -484.25794   -595.00403   -926.59297   -496.79198   -625.29262   -810.31633   -528.36207   -682.16959   -747.70991    -739.4468   -526.28849   -793.92178    -796.1448   -887.90654    -848.3342   -835.01134   -896.67002   -762.02181  -1136.08045   -813.62177  -1098.53996   -941.87504   -1033.5203   -801.40284   -829.41215   -903.84511    -775.9847   -919.03542   -1003.7324  -1030.50044  -1069.12012  -1279.46321     -634.481   -844.98002  -1094.27275  -1072.47833   -920.91362  -1126.93183  -1308.90084   -709.34559  -1335.26479   -509.31809    -997.8067  -1401.14472   -880.07625   -1253.2529  -1087.05834    -688.3397   -284.18025    -612.1364  -1031.14525    -770.3111   -621.87961   -885.43758   -747.34276   -562.04555   -771.94225  -1380.47683  -1010.35152          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        9.50   -195.72762   -460.34892   -283.06709   -207.05087   -408.32179    -391.8679   -208.60462    -332.9652   -385.80742   -222.20236   -359.01389   -385.78386   -229.92423   -561.30153   -390.62454   -302.38014   -109.74663   -405.92217   -184.30866   -215.04323   -301.97094   -296.53497   -425.05632   -399.98478   -258.56618    125.94098   1319.56542   1902.04934   1750.17997    684.90437   -359.21353    -344.0385   -312.15289   -372.82894   -373.64422    -212.7013   -427.74366   -184.87441   -402.77948   -422.25498   -460.30275   -355.99701   -221.70313   -310.42202   -357.50255   -242.82302   -389.26126   -129.08765   -217.79158   -355.62691   -226.86525   -236.44174   -307.70886   -324.30845   -190.30431    -15.49895    228.93139    946.91113   1482.15096   1425.21983     578.5538    -65.36208   -170.07846   -258.79422   -269.73383   -242.43397   -199.12806   -307.10643   -150.26994    -41.81721    485.05437   1243.45604   1502.68315   1328.39057    691.74752     451.8208    561.49164    469.18398    272.75694    -86.23209   -121.23337   -198.95251   -241.50957   -521.39373   -450.56015   -377.07572   -403.84328   -264.78546   -241.39641   -231.10201     -0.91028     303.7459    126.50517    105.99694   -247.72154   -213.16737    -274.9613   -285.96493   -332.86324   -320.69754   -149.65451      4.59042     -8.41651     -8.21593      93.7516   -200.57072   -387.57488    -345.4695   -358.45657   -358.16586   -407.68704   -439.80167    -186.0128   -283.33994   -201.99123    -50.46625    273.23517    329.73387    206.80517    169.55189    -57.38471   -129.99214   -226.55035    -25.73277    146.58448    512.73954     573.1002    478.26757    395.99829     35.42057    -75.65595   -246.50105   -224.08617    -134.4153   -227.76776   -150.19283    285.51432    983.23125   1364.09247   1983.91295   1959.00396   1814.63006   1167.17254    625.65218    295.72877   -167.97448   -197.49806   -173.43475   -117.17836   -107.92174   -333.73664   -278.15907   -354.08385   -400.72182    -172.3497   -101.93325   -189.88323   -290.54219    -104.6373    -96.46247     21.32829     -8.39236    -40.85655     75.56595    -56.59754    -38.30707       97.155     40.49991     -3.54411   -128.64082   -227.37705   -201.24615   -322.70151   -327.98363   -343.85103   -258.03669   -250.69127   -167.48161    -59.42069    -170.4685   -125.33563   -272.90832   -216.18042   -200.41966   -380.72973   -357.15093   -510.80869   -331.21548   -324.61721   -367.51803   -157.62808   -231.06502    -99.86994     -94.5393     19.41354    -64.21776     15.39049    123.19466     110.5803    224.23352    221.23472    174.32181       3.6357   -139.10835     29.94476     -80.9193    -69.92779     10.69921     45.08304    126.90317    141.57241    155.48261    -77.60192   -111.68856   -230.83344   -309.89365   -312.53772     -269.704   -344.15035   -238.96325   -327.31622    -95.46957   -250.99445   -270.48904   -126.39555     42.26951    -70.63378     -35.3486     18.31461    161.89106    146.82994     10.36185    220.49453     78.51202    209.78858     94.73102    -53.98175    150.63281    -39.59302    101.30061     71.79155     27.80254   -144.18262     79.89769   -140.37763   -183.98517   -256.76779   -182.80692   -353.51153   -235.85975    -279.5945    -50.16474    -222.7874     -25.9892     -81.8028    -46.56283     65.96594     87.65587    192.19847    234.88037     427.2844    389.09181    576.38795    626.75904    781.76874    781.78646    707.59897    530.71337    563.84661    409.12129    451.87564     68.41522    111.39153   -122.81225     -61.4107    -98.14984   -142.79775   -113.67642     52.16141    -54.50599     35.09812     21.76161     72.06375    285.30466     73.97887    126.97895      82.6305   -123.02553     -123.705   -130.54736   -105.11482   -393.51315    -46.55417    -300.8963   -484.10525   -201.00644   -209.66156   -132.87051   -305.72808   -414.81571   -302.33358   -345.68786   -130.77275   -255.04157   -216.01202   -174.93885   -127.10479   -104.76766   -116.03908   -131.56743   -109.75118    -74.01249    -24.45707    -82.48617      25.7399     98.07607    -54.88146    -29.90372    -15.26551    -31.94674   -132.36979   -128.21191   -142.73768    -55.83491   -217.34487   -147.56277   -105.46756    -102.6874    -93.81468   -158.90091   -200.39859    -40.08803   -157.24796    -98.37446     23.78134     55.40471   -132.58759     37.38291    158.56826      3.78035    -18.77318    -246.2305   -297.23363   -394.10938    -70.61983     26.31008    -31.37262     49.08691   -399.17487    -76.44964   -332.53112    -61.46173    -26.51025    -99.77998     74.68789     84.35817    103.39613    180.85394     293.6626     271.3734    178.80198    443.80215    186.47792    181.48641    149.88251     274.8883     73.06799    114.31538    108.57092      4.87013     23.44395     -102.293    159.19742   -135.20424    -14.34442    152.91641    -52.96485   -102.87228   -304.69449   -243.20265     245.1127   -271.99917   -128.73742   -238.18591     77.26114   -294.94748   -496.00511    -65.33366   -295.76649    -218.6038   -305.08356   -171.79046   -375.28511   -193.23294   -198.26796   -259.56742   -188.83498   -217.11175   -229.28324   -238.33537    -344.1222   -127.97188    -227.1095   -186.89577   -102.60891    -82.58813   -143.19517   -231.16782   -226.62907   -130.52183   -192.86895   -240.21302   -214.64003   -109.73498     76.23537   -278.29494   -117.37356   -141.82534   -160.28701   -355.98466   -159.52896   -160.15342   -580.67745    -152.0894   -426.34762   -460.49734   -462.22458   -508.37997   -384.47553   -448.64477   -331.40958   -634.15317     -425.313   -617.56583   -741.43045   -545.72254   -922.08321   -651.44136   -803.46273     -666.944   -827.17186   -774.66525   -968.93002   -958.36026  -1011.86654   -967.08675   -880.56873  -1062.39211   -573.83039   -902.59037  -1003.88394   -954.14223   -685.78884   -922.71181  -1215.68236   -755.66025   -824.48108   -879.66443  -1018.55887   -880.45901    -823.0669   -928.41882  -1049.85368  -1013.81012   -842.96611  -1017.18865   -997.98752  -1197.14156   -813.63522    -983.9802  -1002.71993  -1058.31933  -1049.87016  -1135.09387   -663.06741  -1088.85471  -1114.50414  -1027.39468   -975.73675  -1060.38561   -952.26015  -1432.14481   -652.48612   -964.48464   -1058.9616   -878.01041   -312.89267   -434.68248   -799.98696     57.95643          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       10.00   -109.28842   -353.90877   -390.00537   -192.99238   -387.42755   -368.33728   -223.94317   -357.30657   -380.84972   -187.50364   -435.38461   -273.03993   -263.86799   -339.25757   -304.88922   -389.27836   -293.14794   -513.43154   -353.83983   -425.75246   -160.24455    -212.9871   -271.79053    -181.4008   -205.27126     355.3702   1085.25095   1986.56888   1586.48198    482.57259   -302.28943   -179.85689   -328.22285   -450.29841    -353.7915   -121.44749   -299.80787   -349.23632   -361.30141    -369.9116   -344.95916    -349.1496    -298.6384   -362.55232   -355.47944   -306.95966   -314.81099   -316.89792    -243.0524   -407.67942   -328.11286   -409.75537   -439.00252   -415.17135   -296.61811   -137.64173    381.64481   1086.63534   2103.76547   1388.89527     486.4968     296.5307    -326.0705   -292.71538      -9.6386   -358.60567   -307.12606   -362.60871    -257.9602    -57.27072    324.70973   1058.95583   1394.39487   1196.06942    708.11733    647.41765    669.22578    416.86662    154.94645    -35.25762   -118.90398   -405.29798   -274.83878   -347.02166     -219.007   -412.64176   -299.96105   -311.86579   -188.19232   -243.33744     13.56636    140.82453    168.26621      120.858   -209.80568    -64.96523   -318.99534   -264.63871   -245.76179   -124.11682   -124.55906    -66.46026     24.30248      34.6745     16.20337   -230.76426   -333.83322   -240.85328   -393.03195    -321.8611   -446.47399   -462.38691   -415.46486   -330.88425   -249.06518    -91.39724    251.37671    311.16617    238.61545    232.10675      6.62605   -144.12906   -205.44766   -112.34004    195.01458    574.99321    612.35884    596.94935    356.81598    133.83801    -81.38199   -147.39218   -267.57737   -185.46008   -332.41003    -81.27045    299.75218     771.1289   1487.42258   2008.20266   2120.21285   1615.71024    1171.8608    530.96265     175.1275   -168.18342    -118.5017   -200.17977    -194.5387   -160.04744   -240.10508   -295.24253   -226.07088   -339.92829   -388.73098   -358.42206   -237.59781   -224.90415    -40.52427    -61.78883   -132.17015     46.10205    -24.49715      29.3916   -207.56806    -63.91274     21.78604     80.53758     32.95936   -154.42113    -27.80519   -337.84469   -320.63139   -316.23112     -52.8854   -342.94552   -365.52593    -276.8093   -132.36637   -300.40081    -177.6713   -173.64269   -327.72993   -279.85697   -449.18861   -526.12809    -413.1965   -231.67569   -187.80315   -164.94026   -330.80415    -288.2575     -7.76449    -64.31292     41.00899    -12.33043    -34.46042       94.017     256.0053    354.96191    252.37131    199.04469    145.66627    -72.33312    -72.75235     58.05319    -64.47764    -55.00906     -5.92031     70.65151    112.88062    -99.02134     51.07574   -133.39455   -188.62713   -336.25914   -200.51544   -316.15636   -222.54671   -163.00915   -339.40034   -230.36263   -328.19375   -322.78805   -142.82949     23.57119    -53.62124    111.25057     193.7342    -30.87598     44.87354    227.81001     93.80727     -46.8923    198.56614     72.64966    106.26802    134.34207   -195.71181     36.21762    109.02279     -3.44275   -105.14402    -23.95658   -107.90202   -352.16215   -149.52149   -207.98358   -265.39074    -280.6818   -386.61175   -169.08223     -3.62126   -132.36691   -142.81188    112.75787    -84.92601    176.69603    224.37537    210.91318    284.33493    409.19232    417.24652     686.3908    717.68494    644.64183    776.63594    724.29858    370.48297    489.89025    233.79703     200.7362    147.16439    115.57504    104.42651    -76.89456     -0.01975   -113.03138     51.69858     22.09124     96.64679     23.73183     19.89298    -26.57033    231.54833    180.15807      24.0929   -138.64438   -167.77818   -312.39795   -191.88673   -196.52736   -154.57757   -386.33767   -167.91491   -357.65477   -414.69093   -317.06254   -334.80568   -402.66734   -333.69069   -438.56568   -378.78595   -283.60643    -229.2571   -170.73936   -186.79883   -232.44716    -177.2195   -325.81993    -87.50983    -66.26923     62.97641    -68.13914     30.03338      95.4422     -1.70286     42.90296     37.29838     -23.3337   -104.45747   -100.62609    118.55123    -12.96469   -140.67032   -114.98844   -104.85707   -209.14872   -143.90711   -261.40187     25.80842    -76.08983   -111.60576    -59.08164      3.53314    -46.87803   -118.61974     14.26507   -122.84452      7.19207   -255.72882    -50.56675   -159.80888      5.11203   -138.80551   -185.81118   -127.51346   -263.70637   -151.47833   -165.89157    106.77252    -76.03655     42.65444     -70.2318    178.80942    -83.99979    105.12423    108.95911    290.61715    178.71445    -16.53475     74.74915    178.03562    166.01592     22.47022    191.15252     92.57613    209.44405    -67.00728    185.70908    -22.15834    -64.15217    -33.88986      21.4857     -54.0599     29.79409     93.23347    -30.60663   -160.27102    -20.37044    -22.47072    -96.79575      68.3732    -72.32441   -204.45711   -161.98796   -130.98334   -305.01955   -306.05754   -467.43245   -228.00636   -481.99418   -145.45818   -178.66466   -254.96853   -398.60403    -290.5408   -208.81441   -254.49351   -245.81242   -291.88731   -109.74482   -317.51255    -46.07345   -119.76685   -340.14842      2.60228     -415.852   -122.29656   -348.83996   -282.81632   -313.10448   -161.09743   -561.31618   -299.01387   -265.28791   -241.82642   -347.59609   -374.96515    -214.1169   -289.00484    -75.29328    -213.4737   -325.36187   -336.75157   -560.14711     -236.229    -121.6922   -554.67074   -409.61785   -518.24671   -325.96714   -736.64949   -551.86768   -703.50605     -716.372   -828.04417   -527.58957   -711.77362   -659.41797   -848.36247   -665.23992   -790.14004   -912.55908   -679.05984   -855.47227   -755.83929   -943.16594  -1137.88801   -780.84722  -1036.41352   -933.02656   -834.33031   -761.61994  -1141.32075   -835.04855   -778.07931   -642.98231   -781.82843   -569.60578   -999.60887  -1308.74371   -910.92076   -1070.5648   -904.74588  -1195.05738  -1088.67268  -1192.26215  -1233.15418   -893.37801   -843.00068  -1243.40387    -483.6067  -1453.68157   -1081.0819   -599.17136   -827.88695  -1120.90861   -953.43816   -452.32638  -1058.68749   -890.14618   -678.12322   -797.39421  -1060.66907   -333.92689  -1007.80469  -1269.94963  -1506.10901   -735.10791          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       10.50   -142.70509   -263.52051   -426.12232   -290.33789   -369.54997   -403.61785    -361.8301   -420.83917   -426.24504   -281.38204   -546.91708   -293.98119   -262.07895   -220.41607    -350.3719   -501.25686   -389.17362   -399.57552   -606.94489   -430.05398    -91.62145   -103.19576   -310.25527   -141.26867    -69.19081    483.94007   1315.72419   1888.29011   1517.31692    513.37891   -234.17528   -241.13805   -404.87512   -372.96571    -372.6342   -345.04565   -277.26203     -285.352   -308.29558   -341.76271   -446.23937   -472.68339   -399.90684   -245.29548   -351.80465   -341.15624   -485.84598   -281.88649   -398.75615   -693.84076   -198.65589   -239.21881   -276.21378    -334.9845   -231.44935   -193.45266     290.7888    785.06259    1509.3642   1409.50861    673.12586     57.03926   -201.47695   -328.01937   -277.48856   -196.77786   -352.01492   -184.22769   -381.38721    -60.19975    394.87643    880.64582   1398.84418   1329.28819    901.24528     622.6796     640.5088     433.6007    148.73821    -64.50946    -217.8358   -452.11874    -447.5109   -356.66705   -408.81533   -449.90269   -300.83774     -430.389    -96.92924    -294.0608    204.75629    132.98337    191.58366     29.68066   -151.77512   -228.90075   -266.19199   -380.01226   -276.55002   -257.35692   -142.39023    111.65659     46.77353     78.39388    -40.35863   -203.80019   -278.33114   -309.08055   -319.83335   -247.64599   -242.88076   -271.39775   -306.80967   -313.84792   -301.06921    -61.96634     81.63788      385.567    291.75522    306.64255      6.50982   -163.13317     -4.71843     91.08062    116.59407    433.32204     622.3019    460.76782    448.61923    166.03605   -204.21865   -215.52984   -176.88723   -276.75154   -187.03187   -110.01289    281.07382     956.2227   1388.59945   2077.76409   2017.78317   1581.71864   1070.59257     706.3579    304.79934    -47.26167   -143.02208   -250.63932    -80.40272    -69.54122   -154.44049   -381.47588    -283.9168   -328.76594   -477.02017   -300.64952   -251.63696   -220.26043   -157.79567   -177.75031    -76.47375   -119.11483   -162.23409    -122.0829    -36.45258    -67.98515    -64.06355     20.65093     97.84706    -72.18387    -32.79488   -202.59111    -346.5384   -275.64838   -273.96514   -112.75826   -117.68661   -188.21479   -277.19374   -205.66897   -169.02709   -153.45382   -260.71672    -350.6404   -445.34951   -363.95648   -396.48854   -273.18922   -373.80197   -466.64984   -211.01365   -225.83301    -97.99555    -61.52873    -15.47105     48.77447      41.7834     120.0611    275.82684    352.22526      90.0014     38.73999     -7.46609     93.82452      2.94009    -44.06329    -50.00351     26.73852      3.84952     44.05012     27.08941     30.86599      5.03728    -34.37787   -292.60413    -109.3495   -378.44858   -224.07263   -275.89199   -417.10724   -304.48627   -403.79468    -71.17908    -36.38164   -218.79896   -193.94518     57.93247     43.67674      9.79759    131.98568    187.55827    135.77445     34.22071     18.18577     29.55337    -81.44352     67.41615      94.3398     261.4258     46.97544     39.29749    -37.66446    -47.27505   -149.46258   -317.42825   -206.90257   -272.60323   -267.56574    -300.4493   -254.42078   -217.13356   -125.54377   -299.93482    -92.25432    -10.35581     33.70569     39.90359     68.25197    111.22051    121.36157    288.10125    321.68457    515.39486    811.89402    659.54061    764.32104     705.7134    759.11599    661.05495    517.23304     286.5568     61.97646    -78.14795    -26.61536   -222.15591       7.1091   -235.86102    -98.65485     28.58073     67.95233     92.20812    -19.28903     86.53933     19.20906     14.56003    105.26621     71.63928    -22.76379    -141.3175   -248.69641   -393.19847   -181.86541   -290.12839   -327.21714   -283.55108   -434.26915   -349.95821   -306.71981   -243.40508   -317.90766   -321.19801   -241.03843   -234.88171   -291.01562   -175.23484    -279.3927   -117.39687   -209.16831     -214.662   -224.84195   -107.36949     96.51534     -13.2615    -41.40093     -8.32881    187.82257    141.54463    -66.85699      -8.5428   -125.74054      95.8637   -213.85385    -30.46955   -106.88066   -194.56983      0.63922   -204.56088   -145.05119    -31.39042    -95.39875   -141.92964    -78.83814    -64.20477   -135.56405   -200.33039    -51.30984     -85.7688    -77.81879    -179.7228   -117.33912    -68.21683   -243.40396    -46.71639   -162.30389   -216.15056     13.78974    -90.45943   -187.86427   -167.74219   -103.62566   -166.28689   -124.65354    -57.38764    156.46239   -147.90137     99.71495    -64.97535    209.30467    165.93847    122.91518    107.17848    156.35477    -39.52284     411.2746    259.87824    156.67332    175.82774    188.00496    -23.85987     143.7591      5.89873     41.98272   -252.79214   -224.34578    -13.99046     33.21209     43.65929     -3.25892     -3.83787   -119.70801    -43.41933     18.29765   -233.97824    -55.40572    -63.12783   -258.58531    -64.18208   -152.95972    -24.84822   -344.91232   -239.82608    -36.24616    -23.61534   -490.30886     27.03985   -175.67789    -157.6744   -286.42269   -140.23817   -233.77153    -98.68625   -221.61147    -91.18733   -223.64482   -396.93989   -216.81676   -130.60309    -246.7803   -248.70279   -162.78615   -269.90633   -515.89743   -156.25409   -242.21952   -223.84833   -326.18475   -323.04196   -200.25984   -195.59796   -265.00463   -410.24625   -496.23506   -146.19202   -232.48084   -437.27537   -675.49202   -431.48261   -952.98635   -348.43357   -564.76188   -407.10338   -442.84754   -566.72601   -593.35219   -417.12026   -787.29487   -889.78411   -672.62296   -793.88552  -1082.40201   -698.38934   -777.86325   -842.31922    -781.4481   -870.15502   -823.75174   -601.09205   -723.95165  -1126.37183   -664.39458  -1145.08886   -918.56679  -1090.16187   -932.81594    -974.7616    -894.6217   -813.88375   -870.04698   -1131.8918   -854.26218   -954.26019   -776.44876    -778.6933    -837.5708  -1095.81101   -811.07145   -660.53455   -834.03155  -1057.61324   -985.30469  -1534.60079   -851.00661  -1019.50111   -588.40932   -629.23624   -763.18483   -849.28835  -1130.66661  -1161.91873   -763.70866  -1191.08617  -1231.16012  -1019.97245  -1567.55781   -901.57278   -412.13604    -313.5582     156.5863   -592.74291   -716.95547          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       11.00   -148.15374   -231.05878   -361.83049   -368.37124   -337.43217   -469.56626   -414.22431   -476.79047    -468.2908   -365.20694   -492.03658   -306.95544   -386.31535   -317.58031   -314.82808   -606.68272   -375.58126   -356.55994   -428.45354   -325.09297   -198.35141   -353.80528   -300.12986   -175.52222    -19.26219     462.8232   1350.61072   1674.88904   1555.10359    432.76688    -177.9613   -263.19705   -453.34171     -259.786   -413.86338   -307.36666    -338.7079   -300.02733   -238.75871   -413.04859   -447.53971   -328.81018    -491.7219   -379.47519   -430.00924   -359.30316   -525.60937   -270.73815   -387.18679   -421.37717    -423.8598   -419.18193   -276.37668   -267.75775    -194.9736   -183.23911    210.01541   1103.83742   1410.92733   1167.86092    618.13638    177.23856   -319.33854   -253.59956   -269.35467   -317.94561   -251.11889   -276.85919   -208.29287     31.37751    461.65165   1009.50599   1528.75223   1211.59929    858.78289    472.86818    462.70283    530.86253      59.2199    -106.2233   -352.74437   -321.45028   -414.59355   -380.19554   -386.84608   -276.22097   -357.55011   -199.14535    -172.6456   -191.47641   -120.35777     49.88587     98.39877    -46.48341    -245.5206   -282.65242   -327.24505   -276.44259   -403.57987   -296.55442   -119.23021    -18.48681     -74.6905   -106.90798    -14.61828   -168.30872   -355.31638   -268.65242   -391.96442   -348.84494   -229.62851   -205.38433   -223.53064   -295.42071   -259.79407    110.14889     10.62744    418.15663    242.95808    176.77074     50.71448      5.30138   -252.07629    -44.45512     247.6972    520.60086    645.82428    621.26395    372.55777    109.04683    -38.42674    -76.27157   -117.94258     -89.2028   -175.65402     25.96659    213.83566    799.44825   1543.82638    1895.0048   2002.96282    1509.0577   1118.89609    651.65766    122.50487     -0.45901    -55.02994   -122.32226   -221.12046   -211.15625   -294.17295   -252.24202   -205.37691   -322.63196    -182.5069   -236.92311   -403.49116   -391.81655   -207.11076   -145.61349    -154.4041   -177.80909    -211.4338   -144.87267   -163.92269    -13.23103   -258.84346    -107.3471     91.41738    -31.99318     16.81256   -146.68379   -220.11864   -422.28339   -207.93276   -135.51973   -219.11567   -227.03674    -34.86079   -159.66524   -385.33377   -277.67174   -229.57342    -521.6621   -353.04569    -385.9989   -379.31026   -466.47591   -374.69406   -373.42515   -426.83201   -143.81772   -265.20051   -127.78824   -118.59833    -73.74473     16.46557    284.72369      195.936    224.65704    177.92681    204.93603     29.24625     69.70659     37.59638   -198.45194      87.1171     -9.82999    173.75264      2.33955    224.93894    206.74229    -47.85923    -95.18717   -178.45019   -211.89832   -231.98665   -219.38509   -191.77047   -394.21389   -357.49891   -258.27888   -312.25379   -177.41392    -92.23891   -162.05382    -54.78237    -28.44918     -6.97072     43.92058     44.22518    318.46913    186.29835     46.23286    -111.3058    -405.3706      63.6411     -40.7756    -72.67568    151.58996     -75.4483   -163.98652    -42.52347    -58.61869   -274.85032   -134.25067   -429.33555   -234.63028   -217.13827   -291.30704    -216.2577   -243.91919   -312.92216   -115.01109    -69.23333    -60.17674     136.1354    118.29861    167.16824    154.05233    289.18808    139.24088    288.55485    569.41347    692.94526    742.96192    707.03328    596.64801    539.81961    407.25826    270.53837     83.22126     -4.64479    -22.82681     -77.9008     53.12867   -199.97114    -25.11116     44.50555    -32.72969    118.96414     -24.4793      60.7909    -43.00547     52.54069    -12.29386     37.91013   -183.56999    -107.6524   -155.46196   -225.39642    -168.4568   -256.96644    -178.6507   -262.17375   -369.67491   -328.33265   -284.78742   -443.93059   -204.54409   -453.67996   -404.42675   -237.24431   -149.38028   -274.81863   -254.20029    -53.93933   -166.91784   -141.67422    -36.24046     20.90721    -85.30301    -58.13785     71.15319   -101.05242     20.76338    102.22199      63.9484    -88.59162     -39.6061    -91.81077   -219.47703    -96.04466    -54.25204   -269.03495   -199.47279   -156.50512    -79.17886   -137.07107   -101.19693   -165.99195    -76.34136   -115.01648   -132.12957    -121.1307   -136.85389   -244.36954   -145.33759      3.70277    -80.40355    -66.52108   -193.92473   -127.46436   -239.11823    -11.17223       26.582   -123.60793   -105.97064    -31.23592    -53.77878    -18.65081    -74.79029    217.07039     45.13945     12.36052    137.69075     18.85877    130.65946    239.73843    303.82496    220.47216    172.77353    410.06894    172.59543     355.6901    145.59319     70.30412     63.17755     37.95601      77.9509     -24.2235     17.08801    -18.04975       -7.417    -62.07601   -164.79774    -63.06995    -79.38957     -2.40979     88.97571     47.10663     64.16265     50.35376   -109.44004   -103.84914     16.98043   -154.43975    -86.67342   -242.02643   -207.11344   -195.81602    -87.64956    -274.9655   -198.59996   -164.15934   -205.89864   -175.61823   -198.86679   -113.10959   -133.44685   -250.21183   -269.52445   -144.31665   -303.13076    -135.8929    -337.2551   -324.66658   -241.16319   -221.71292   -250.53973   -275.13722   -144.41168   -220.49248   -238.31106   -324.25367    -221.1614   -163.78026   -271.95673    -81.44455   -455.63954    -435.5722   -340.99912    -362.8075     -88.4945   -306.65907    -720.4883  -1073.27202   -590.31717   -438.20171   -407.41632   -527.96793   -604.14697   -545.31553   -884.26277   -638.75244   -654.23178   -732.38797   -683.08341   -367.64779   -496.37704    -945.9566   -970.59873  -1046.57359   -801.65617   -823.69225  -1028.40352   -706.34086   -800.23977   -761.15895   -898.09639   -464.79115   -732.24221    -912.8498   -988.76479  -1130.27162  -1152.18075   -900.27888   -810.95079   -762.56813   -817.35817   -936.04437  -1232.90879   -932.28294  -1037.52774   -748.55005   -924.65995  -1110.84192  -1006.16659   -714.87732   -882.17025   -771.84964   -1065.5842  -1015.21038   -771.97237  -1049.72392   -644.35624   -715.07331  -1284.83661  -1041.07402   -987.78724   -449.89423   -645.33458   -977.51146  -1290.27672   -828.34524   -803.81527   -873.74394   -731.48638   -788.55447   -941.16808          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       11.50    -185.4431    -246.8559   -276.15114    -481.3983    -317.6337   -481.49826   -262.23297   -540.53454   -333.27878   -426.80948   -433.50486    -85.58725   -629.30929   -327.63339   -215.84533   -577.09188   -372.76919   -424.72196   -327.39105    -298.7486   -310.08943    -429.0602    -45.73914    -172.9857   -141.53119    395.80554   1336.53624   1994.07085   1472.36991    179.29752       0.1294   -229.16693   -421.37244   -282.81933   -281.03151   -322.78306   -494.11309   -555.24838    -216.6153   -384.93136   -623.19765   -273.38952   -584.55965   -369.49915   -364.40131   -346.14734   -468.44328    -265.6328   -438.73753   -460.83926   -414.49866   -373.82618    -425.0335   -500.21774    -237.6347   -136.35241     200.2723   1124.37432   1456.90159   1276.10091    550.23518    -42.47274   -310.53412   -246.82259   -319.82902   -235.04346   -231.53675   -210.20462   -140.21625    -71.12641    344.18343    902.09799   1451.91088   1290.98031    869.22724    672.23892    679.50706    422.00954    110.29036   -418.06102   -391.10701   -467.05616   -423.76782   -286.22498   -362.06687     -374.603   -309.13105   -420.89235   -200.65138   -188.84639     75.20543    208.86886     -6.86844     53.06322    -10.62808   -343.98623   -261.63738   -134.53181   -266.20935   -283.95085    -37.62954     51.63798     57.44644   -148.85629   -181.78282   -107.14028   -332.69224   -407.08293   -374.21465    -423.0679   -325.42809   -371.83879   -254.82944   -327.92979   -285.68147    -136.1672    271.53668    180.56565    422.16158    135.61978      48.7153   -189.30299   -241.03501    -31.65169    123.74604    378.56335    634.68709    712.60845    318.53065     55.61299   -138.99751    -14.96249   -240.12361   -356.26958   -192.01352     -0.27741    106.42153    722.10439   1642.68033   1955.78911   2142.91599   1718.51449    800.85367    430.55978     50.36994   -278.36996   -201.62267    -288.9905   -255.30709    -97.94048    -214.8896   -266.38585   -188.90763    -389.7995   -205.36414   -341.58665   -217.78895   -250.55943   -130.50887    -89.71191    -47.22996   -175.79732   -188.11964    -97.29806    -126.0254    158.29873     16.10392     -3.84823    -55.12186   -134.91846   -154.15538   -171.17673   -203.30177   -206.72536   -345.99052   -119.54229   -307.74566     84.74557   -157.59376   -182.23908   -178.15176   -296.81156   -317.31183   -325.97982   -353.37836   -216.64607   -415.03056   -388.47731   -290.24783   -282.99686   -332.27921   -155.24221    -76.27501   -235.73445     43.90167      3.83427    262.27519    157.80581    156.86513    253.18405    269.97916      41.5599     56.56886    -22.71998    -16.91006    -49.46079    -77.15798    -69.04967     25.52391    182.88314    142.08757     -52.3214    -10.53343   -168.16466   -145.91363   -208.19872   -278.15612   -221.15558    -308.9253   -285.59463   -178.15375   -354.92218   -288.11819   -221.78007    -137.1822   -142.92945   -115.83458   -109.06806    158.18428     58.72069    129.21974     89.34942     -5.01581    -36.63126      8.68425    112.60161    211.99862     61.30451     37.04221    -60.03194     38.01261    -12.18296   -235.79787   -209.71711   -217.41899   -273.79789    -352.0429   -263.55022    -344.1811   -299.77373   -177.91677   -132.87837   -141.94321     -6.15583   -108.70157     47.28187    -39.79156    199.93045     56.45889    237.08826    263.21007    300.40867    494.61353    439.32376     684.5893    844.22191    624.64722    660.77331    737.35935    405.38738    301.24143    131.06931     26.50821    -67.67851    -46.97144   -159.25253     28.36984    -93.14344     84.08731     -9.46346     92.26939      45.9988    -30.14104     34.82051     295.1438     28.52358    131.49452    -88.64815    -99.43397   -129.26776   -105.48014   -272.02535   -314.00897   -328.10554   -103.59171   -396.25577   -339.64497   -313.33749   -346.04093   -311.81991   -274.71932   -140.36108   -332.04529   -276.95893    -275.8134    -252.9166   -172.86549   -393.94877   -119.19519   -127.38553    -63.73011   -179.62741    -55.23338   -124.67547    -12.51632    -31.97082    -59.09278     87.95044    -65.57325     -3.88433   -138.36012    -97.67493    -60.62111   -109.07894   -252.15353   -228.04076   -160.44964    -259.3801   -161.20637   -235.99496   -136.92639   -201.06254   -127.91238   -115.10375     -9.47977    -33.07154    -75.73466     82.56722    -68.78561    131.06496    -132.8336     -73.5702     96.11745     79.01328    -26.87998    -59.18434   -152.28055    -16.33254    -19.22362   -108.95672   -122.33173    -59.17055    130.84726     96.88956    -30.89261    -37.35506     44.62649     10.60615     95.26098    133.71256    147.06758    256.48549    231.87592    163.62575     295.5217        -10.7    310.41747     59.37662   -124.98234      95.3696     167.3928   -304.12644     22.42645   -189.54735    125.71928      156.217    -64.73712   -223.37847     47.10658    200.17167    -55.82231   -184.91846    -60.16693    -147.8086   -148.74897   -106.82043   -195.49561   -224.94942   -104.26231    -393.0257   -239.64723   -429.74551   -156.05499   -350.92879   -599.75871   -270.55112    -77.60196   -269.18343   -221.74589   -153.11257   -337.73729   -324.74913    -80.30003   -270.78714    -246.1748   -293.35583    -31.01131   -193.59147    -80.73032   -351.55435   -397.70204   -262.77355   -338.95728   -313.88433   -129.82257    -49.28909    -516.0318   -224.50128   -373.68631      0.07556   -214.86938   -497.69133   -254.34665   -422.11973   -739.83834   -279.72228   -516.10147   -600.96305   -474.07905   -585.60645   -333.09858   -603.58551   -389.94731   -665.19612   -540.04026   -669.70403   -805.92559   -727.34625   -622.53154    -687.0945   -750.41871   -887.23901   -709.12888   -829.38598    -860.2439   -742.48022   -677.38524   -849.79674   -478.42771   -814.59283  -1282.76476   -847.51291   -956.86227   -889.79715   -844.07599  -1087.66538   -949.63803  -1093.88689  -1023.54362   -871.62141  -1077.97043   -692.98011   -742.38163   -557.27868  -1024.45039   -695.96721   -767.21688   -602.68174   -811.62651   -657.64022  -1239.27756  -1093.71194  -1223.72246  -1197.45898   -929.75675  -1055.65656   -792.56444    -975.9309    -917.1724  -1352.04822   -902.09927  -1238.60575   -958.77881   -1381.6787   -417.63974   -659.57959   -465.30804    -840.9173  -1390.90726   -550.56983          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       12.00   -276.31719   -275.73117   -280.58912   -606.13815   -320.89976   -335.62247   -160.01238   -510.58944     -83.7217   -492.82869   -465.74204      3.52485   -383.94667   -318.01819   -278.16379   -352.37228   -335.73875   -324.67155    -206.8976    -275.7682   -352.47375   -421.97018   -129.24102    -28.46088   -190.66924    468.35262   1230.98166   1742.58838   1198.45418    166.92798   -209.40679   -143.06554   -502.52456   -312.61559   -211.97244   -298.66073   -403.62789   -490.03292   -280.00228   -328.36819   -323.16412    -249.8732   -264.35935   -488.01442   -319.53299   -345.02922   -453.79095   -354.72289   -269.04641   -360.72715   -341.06138   -341.30196   -268.78964   -299.06144   -325.85758   -271.41041    205.52674   1049.82537   1663.51765   1038.04633     549.5125   -218.21052   -103.68333   -251.45188   -222.23646   -235.79152   -316.97778   -276.31275   -188.43692   -103.34795    266.48806    942.87305   1412.56799   1302.03823    845.17024    653.65368    444.23625    411.36769   -221.92707   -619.10719   -394.19359   -373.81649   -244.02376    -330.6203   -177.20409   -373.47896   -362.48792   -253.29362   -197.03401   -251.38142    204.33862     36.26919    295.45733     34.46811   -142.15816   -273.51261   -224.64331   -299.21411     -145.681   -284.47187   -265.13331    -70.17895     73.75197    -61.83626   -124.79103   -238.16268   -356.86316    -410.2416   -390.68636   -270.73499   -332.61464   -455.20412   -175.58463   -282.26334   -183.18397    -48.26357    -16.76683     209.4064    310.79032     44.26881    141.88631   -113.18511    -90.46656     -2.10751    515.37004    453.32841    671.13192     591.0503    451.86582     59.73284     -17.7995   -120.02115   -247.14192   -349.58327   -275.78554   -113.35202    247.06369    894.33006   1491.79631   1980.62369   2269.14266   1692.17046   1128.78868    401.80559      0.03781   -132.42316   -127.86149   -175.19029   -241.78366   -274.32495   -186.81575   -227.72751   -455.15996   -283.28691   -389.78476   -335.13266    -314.0655   -143.52054    -68.30927   -168.05564     -76.7615   -152.53845   -203.62031    -53.79769    -39.68797    -51.59621    -61.09303    -49.94979     73.87816   -240.29672   -343.73174   -258.43675   -167.30051   -413.79736   -208.38651    -200.1778    -150.6842   -103.48776   -212.66041    122.41956   -338.24933   -408.53967   -113.44664    -251.8552   -298.75556   -157.70644   -373.56964   -350.97303   -324.59201   -282.88547   -260.85092   -131.61464    -96.45282   -238.44421   -141.73907    163.68005    -44.11416     43.98078     87.55223    341.99884    166.00282     87.18668     56.23464     -9.36434    142.45897    -35.30853    -69.52016    -21.51101    214.60721      54.1247    -34.51878    -34.58385    -52.37099   -158.51874   -238.76959   -236.18812    -181.3794   -270.57744   -404.24156   -310.94855   -347.78753   -240.45447    -99.76476   -280.01691    -197.7368   -282.32816     37.94783    -16.60263     45.18213    -53.15861    -10.55013    189.11218     154.7697     54.70683    -92.90229   -105.47548      7.37078      41.3275    111.14716    120.49658    101.50045   -159.93448   -164.89491     -2.51707   -153.90667   -337.56447   -344.65782   -140.67744   -298.42617   -271.39079   -282.68887   -116.80242     -80.3307   -128.86602    -54.09088   -161.89366     19.48873     -33.3989    156.38204     35.13394    228.85499    550.36132    507.94662    534.17801    575.39243    775.36345    745.73416     689.6974    596.01246    315.30355    390.03099    341.23888     76.47798     -117.801    -14.21603    -11.82671    -98.47602   -123.68724    -81.27725    -91.90606    -24.39479    158.56893    251.94907    127.53952      1.05697   -155.36471     35.46236   -134.02484   -287.52369   -440.37536   -304.18417   -388.35193   -518.54786   -207.66943    -288.1162    -360.3907   -548.79354   -376.44037   -374.68444   -376.15627   -341.69835   -422.30171   -257.90107   -267.54108   -210.93029     -137.869   -250.87112   -234.74922    -12.79551   -171.30763    -69.02959      6.18552   -121.80237    -26.93632     76.00647    -19.71393   -123.35467     40.74564     30.35495     -31.7914    -70.52272   -269.55728   -185.38231    -57.97886    -225.2542    -82.32628   -240.27231    -97.03644   -204.92194    -63.93932   -147.18223   -178.59408    102.43723    -33.09494      -45.214    -48.96778   -187.02377    139.09123     54.42807    -26.43351    -18.82009   -128.66978   -145.18906   -134.20598   -189.51096   -150.45051   -205.72911   -102.30386   -419.71679     -8.90771   -235.83953    -103.0452    -39.07788    -33.18247     186.8181    169.70753    -52.29762   -189.59653    221.69042    293.21215     67.84821     256.2185    209.89132     152.8817    152.80599     17.33253     39.11051      58.5353     94.40175    -127.2532     -63.3338   -108.67072   -130.40353    -133.0422     20.46321     71.01467    -32.50366    -47.71557      9.55754     109.9105      16.7896   -119.74414    -140.3482   -268.20488      9.01503   -112.27027     -75.6006     15.15636    -332.6792   -343.63748    -98.65111   -206.79481   -187.76596   -144.70305   -239.80976   -116.19833   -223.35273   -347.08981     53.62973   -140.18454   -123.20309   -328.37073   -233.46439   -144.60021    -297.0473     -340.903   -321.47801     -3.93006   -281.48996   -246.96896   -329.83906    -95.44262    -82.73453   -199.20335   -183.44197   -222.09131   -240.42486   -219.72731     -396.217   -340.90107   -583.36778     80.56542   -343.51177   -353.23168   -418.91373   -390.57797   -310.97326   -280.25359   -535.03195   -525.31328    -888.3019   -655.97585   -672.52537   -643.53687   -672.42827   -410.02412   -781.40017   -435.68646   -546.08132   -731.55683   -875.40112   -961.85742   -626.50211   -839.64727   -882.44327    -822.1118   -969.17198   -965.05048   -771.32999  -1229.99525   -902.13718   -878.51605  -1052.83762   -952.48636  -1036.93445   -916.98093   -940.89101  -1175.03892  -1051.17803  -1034.58768   -929.59928  -1058.32841   -987.48391  -1246.84392   -846.91601   -790.18235   -709.74729  -1100.14781   -822.22729   -976.52688   -828.03672  -1250.44048   -867.26509   -968.07155   -596.17385  -1275.25675   -997.92191  -1033.83921  -1406.36902   -632.76469   -783.62864   -964.22253   -979.55579   -944.15499    -37.41655   -709.45801   -251.69864  -1216.37254   -567.45868  -1119.64613          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       12.50    -349.9811   -245.40693   -322.86363   -622.08317   -306.25292   -137.83642   -138.49839   -445.82588   -127.77869   -496.69877   -499.91676   -262.83316   -215.14997   -264.30513   -441.99124   -393.50379   -269.41064   -276.22628   -359.45422   -341.34105   -394.01629   -261.86295   -278.03277   -177.47376   -208.58901    640.18836   1246.25451   1816.44686   1428.38338    230.74086   -286.80634   -151.18155   -367.14872   -256.74128   -400.45915   -340.98011   -327.42849   -329.27623   -289.56254   -494.74763   -417.08178   -280.65315   -392.30674   -393.01313    -429.2972   -243.61096   -437.07031   -591.95092   -285.61042   -305.56258   -292.39497   -274.81277   -447.21877   -210.91336   -376.13388   -205.60484    276.70772   1057.66481   1581.62542   1272.45367    654.84108   -196.00787    -287.0725   -163.13045   -408.24547    -311.0399   -290.36875   -339.59201   -228.41598   -168.82786    483.24563   1105.94222   1370.53429   1274.12829    682.87205     435.6139    630.33999     295.6414    -42.89648   -541.40969   -349.39233   -325.09448   -436.43969      0.09649   -278.52407   -424.65337   -292.67317   -375.59667   -291.34592    -97.80212    104.37022    280.59202    197.79103     28.80624      4.87858   -255.50384   -300.41193   -353.97333    -288.1253   -165.99976    -364.8018     51.13564     48.19866     99.43621   -159.90444   -183.30951   -256.94627   -430.48592     -380.003   -366.11454   -310.89818   -221.89843   -282.20544   -279.96044     -117.284   -195.76128    148.90919     363.0803    232.44877      186.797   -105.28181   -132.18379   -168.79798     67.27793    285.41461    418.12433    665.36012    553.06652    252.35536    205.85863   -114.08663   -123.07471   -320.20158   -134.76235   -234.59219   -141.11661    373.87145     850.8906   1578.90986   1961.25499   2057.56582   1576.67903   1103.47868    482.72177   -111.01109    -99.39806   -151.09806   -210.53701   -250.45961   -259.12858   -324.70804   -296.24532   -408.52378   -420.45361   -339.98796   -240.41695   -292.66743    -66.28265   -172.85734    -46.20773   -150.63293    -46.17928   -144.36945   -119.75405    -18.12958     27.92876   -124.11523     72.42984    124.25937   -100.84447   -113.72167    -75.60321   -267.72404   -149.26568   -260.29833   -225.80541   -212.92356   -302.73038   -246.15703   -128.66967   -119.86935   -249.20732   -276.25992   -203.15553   -370.01069     -367.776   -341.92292    -368.5116   -317.34067   -250.31969   -321.47911    -268.7161   -187.71219     24.40586    -48.12286    160.56099     41.84186     -1.84497    183.64771    176.17171    221.45502    104.49907    133.39601   -188.91146     40.22533     90.94927    -10.83197     60.91639      165.701    135.94294     128.4595     96.71987      52.7119   -180.49545    -95.93761   -150.33042   -133.10223   -280.41635   -375.37859    -397.1446   -319.26209    -246.9469    -79.53593   -242.38534   -134.96087    -80.69638     62.50961     -4.27604    -11.21981     57.95773    114.47817       86.011      6.53463     65.38635    -70.15643    -95.13756      5.11812     16.30241    -31.36722    109.28897     43.14468    -35.41602   -139.44899   -198.69771   -182.06285    -199.5796    -199.5956     -290.619   -295.07296   -316.46363   -202.03903   -284.49812   -302.34141   -201.84344    -82.19899   -238.70493     52.93847   -124.39677    186.39069    186.94441    169.12903    350.14668    714.16248     600.2113    606.00269     650.1193    645.90191    712.25479    477.06271    449.30304    287.97557    295.31006    135.72186      36.1264     20.90588     77.59695   -166.73317     -50.1201   -151.58664     23.83042    119.99308     91.35411    160.23119     54.99368    116.20976     -6.05879     37.37506      85.4989   -125.17545   -250.57595    -258.8018   -149.76398   -245.25656   -406.56834   -247.86562   -518.05811   -297.85521   -306.68227   -331.88735   -453.53267   -375.89417   -323.14201   -174.18348   -130.90619   -233.85578   -374.58892    -37.67965   -234.92536    -75.67083    -53.61753   -181.04766   -140.36285    -10.87979     -4.42631     38.89136    -27.38809     33.24416     -2.69299    -16.75862     57.51086   -103.92221   -122.04524     39.05811    -45.05882   -146.09342    -119.4508   -152.88295   -139.25675   -138.56235   -285.01597     57.36054     -91.7882    -73.58231   -168.14134    -64.34085   -271.14121    -27.99312     40.83866      0.77377    -27.64103     56.56662   -345.50079     25.22912   -227.08586   -292.35954   -248.44012    -86.66545   -159.17502   -142.15031   -180.97111   -259.28201   -100.64623    -66.67004    -53.96887     29.39777    185.71249      61.9509    348.19563    109.79743     415.3272    222.54048    173.17311     80.39515    209.33468      3.69235     43.36969     47.38306     37.35505    -89.15917    127.72561    209.40331     31.42771    -79.41404    201.72314    -43.04296     67.51052    -81.95025     -28.6797     -2.74514    -99.15679   -202.23761   -152.12216     15.55661    -54.20551   -231.41784   -119.56305   -168.33686    -36.33869   -120.15619    -51.25909   -214.51405   -382.01226     -326.549    -17.81969   -178.56037     -62.0446   -228.22921   -398.79598   -250.09244   -247.77562   -287.09435   -266.31195    -53.01787    -363.1149   -199.30445   -131.56897   -268.17221   -149.50255      7.76909      29.0947   -136.21788   -181.17727   -183.81771    -168.1549   -218.10344   -245.20352   -161.61022    -173.6715   -123.07112    -188.5796    167.48226   -436.61904   -331.96328   -368.96219   -667.02476   -451.21706   -446.44196   -186.81297    -74.34151   -225.43192   -501.90996   -403.81521   -488.07135   -571.04565   -469.92767   -880.34169   -707.18167   -516.72652   -696.99889    -643.2598   -515.10902   -827.83744   -839.99038  -1272.61867    -703.9118   -685.53616    -709.8092   -767.71388  -1123.15715  -1156.78923  -1033.84804  -1087.03741   -962.64399   -937.28688   -727.68535   -985.96155    -894.9275  -1074.69904  -1222.76777  -1121.44758   -811.86453   -843.49425  -1182.97953   -737.51938   -1070.5943   -903.49626   -574.36367  -1092.92145  -1330.44747  -1065.75497  -1148.07334   -859.06792  -1032.67258   -931.36412  -1110.63326   -902.35013  -1101.02213  -1379.94299   -1008.7062  -1150.62411   -919.57351   -241.27295  -1383.80564   -443.76186    -377.7796   -872.75133   -725.69242     174.2417     96.69993          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       13.00   -369.29778   -178.19519    -354.0726   -459.60263    -246.3716   -279.47003    -93.38927   -420.39882   -340.98212   -307.25691   -510.76784   -407.47161   -247.81054   -259.48751   -352.42786   -550.01069   -307.38386   -309.61579   -458.10133   -424.04328    -394.0347   -186.30249   -365.16268   -302.66954   -197.75476    564.40533   1337.06993    1883.6311    1442.5785    293.47951   -148.81724   -296.30368   -253.22653   -430.05896   -340.67084   -197.17517   -454.53056   -362.80352   -250.25008   -394.24328   -322.42324   -297.33964   -348.12486   -399.52045   -319.61092   -323.84661   -409.05712   -521.17936   -511.73322   -385.26224   -276.03486   -403.80397   -403.05519    -323.7902    -297.1066   -178.88916     230.3414   1042.07839   1484.22628   1373.91347     700.3247    -35.01853   -187.59769   -311.17599   -280.84398   -211.11738   -381.35147   -450.70996   -193.09003     71.06114    382.19231   1080.02497   1678.18443    1293.8027    952.71982    609.12183     397.8689    155.37569    -78.77633   -252.81111   -286.73734   -282.59418   -191.91533   -452.70215   -274.47279   -332.00262    -339.4874   -269.17163   -499.29125   -114.60339     29.26899     61.19188    150.95916     52.66626    -86.76527   -318.81273   -319.45994    -251.0552   -332.22674   -115.89482   -193.71845    107.36512    191.52168     80.83003     -66.7113   -305.13534   -237.80593   -353.69647   -342.29314   -226.56252   -390.08092   -474.91282   -257.94242   -418.91343   -188.94104     33.85964    178.02187    223.97677    335.26613    114.20612    109.41497    -25.49665    -204.2678   -132.45867     132.6419    350.35135    705.30206    411.63708    308.85083    -78.00617     97.03212   -165.45383   -179.37415   -273.09701   -219.90223    -23.86329    366.96331    707.40874   1453.40846   1803.33444   1956.33976   1620.48279     1025.044     351.1757    159.95917     15.75071   -120.69996   -245.83089   -212.28064   -280.46739    -215.1804   -278.56795    -207.6967   -332.06233    -232.9884   -403.17674   -166.20098   -176.94148   -159.64872   -145.13865   -215.93658     -99.6119   -189.61035    -89.69778    -74.31548    -48.66895    130.80353    -44.21202    -70.77051    -31.44167    -148.1184   -136.54018   -461.95801    -151.0024   -190.55292   -221.71354   -215.77449   -283.93651   -268.15585     -22.2053    -205.8377   -382.17132   -394.75236   -330.11458   -293.72888   -322.94078   -298.01204   -283.60083   -310.24102   -164.39293   -230.97165   -278.96379    -169.8871    -83.53443   -279.88057      23.6113     50.36923     82.92117     45.10809    268.31023    280.53848    135.36275     61.95883    100.05711     36.77409    -85.25977    -47.73998    114.44797    -23.07823      83.8402     36.32841     85.89932    -52.84886   -130.13234   -115.03263    -252.6687   -208.42414   -394.55455   -237.07143   -300.25172   -261.75736   -339.61314   -377.65116   -199.78601    -95.77972   -218.61619      55.0752   -142.77212     31.18202     136.5555    103.68173    -58.07734     -8.33408    -52.14432   -110.64789     15.45268    -78.13488     20.20633     38.30065     99.41542   -208.77205   -126.93843     -84.3966     25.00549   -192.44456   -181.47044   -258.14189   -304.38853   -347.89576   -343.10613   -301.68279   -214.40218     -63.6645   -101.92492     19.49829   -224.87932    -25.04326     37.22297     56.99683    276.09095    394.71168    250.65593    619.40371    440.10055    700.37164    593.78905    794.71736    556.58728    525.61313    393.95179    347.95569    229.04333    122.35556      6.77728     36.20391    -13.41902   -238.30142   -123.04167     97.75769    -17.48144     61.84123      4.05386     92.75191    115.38088     86.31604     54.61666     68.64101      -98.777    -94.29901   -153.91856   -163.43685   -321.84794    -180.5469   -298.95851    -289.4911   -234.53617   -435.38727   -373.39512   -380.67259   -384.73345   -322.26563   -249.31249   -255.09441   -273.38501   -133.69264   -284.30848   -238.65957    -96.90628   -100.10171   -264.77113   -109.72822    -55.08242    -26.92399   -205.18975    -90.62076     33.15068     88.25958     23.89807     -8.01012      8.98949   -166.67466   -108.63656    -70.50408   -198.16207   -135.91652    -92.20472   -107.29209   -180.42038   -241.36117    -95.25276   -186.34088   -141.69221     55.05598    -96.40694     -45.0467    -94.29693     -5.00541    -226.2301    -97.46451    -16.28315   -176.85597   -116.53223   -175.12117    -34.41119    -45.05221   -147.13896   -181.71291    -200.9133   -108.27516   -193.67457    -12.28071   -145.99303    -228.6619     16.18362    173.59182    103.42582     13.69594    132.59648    199.33899    276.50804    123.36818   -108.72769    236.11295    219.84845    116.78105      241.455    147.64052   -186.31096     72.96326    124.43247    -49.93756    140.32522    -71.93365    -97.49742     48.78199   -111.64485    -62.71258   -137.92701    -54.99553    -58.77963   -196.29278   -210.39891   -170.46232   -173.48108   -127.43527     -4.92079   -217.64892   -206.67715     16.75503   -440.32008    -216.5157    -76.70305   -311.92597   -113.80332    -171.3312   -245.46861   -329.58454   -181.18258    -97.68091    -139.5519   -195.87216   -260.94119   -258.99318   -362.55555   -122.93927   -268.19414   -244.99728   -162.17827   -326.82154   -572.01825    -176.1532   -192.98815   -190.98059   -419.30382   -357.83583   -210.98254     26.51388   -288.11961    -15.74404   -317.13922   -347.23358   -144.76495   -483.41807   -185.24702   -555.91256   -603.34618   -538.74074    -209.2829   -283.88789   -711.73438   -745.60798   -477.88762   -302.92912   -578.71297   -511.03951   -586.91371   -603.25942   -321.87578   -559.59085   -550.95179    -388.7751   -926.50154   -802.63885   -402.64717   -860.63017   -1134.2924   -886.80307  -1012.60474   -823.50413   -878.48447   -828.72397   -826.68614   -700.71516  -1027.60549   -760.95861   -973.33213   -928.93536   -830.28316  -1022.36341   -838.88789    -609.5684  -1053.02522   -819.96823   -695.47916   -782.17644  -1014.13967   -792.98529   -1182.7303  -1150.08285    -881.2863  -1120.52455   -978.50717    -867.3187   -966.89381   -733.65216   -883.80613  -1248.41167   -834.39922  -1209.90238  -1458.08264   -264.15205  -1222.50814  -1091.46958   -659.40568   -956.89583  -1402.77117  -1073.03536   -583.51573   -531.93331          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       13.50   -345.03667    -90.65678   -252.34388   -315.72363   -221.43758   -467.03327   -239.80777   -459.03214   -437.27719    -70.15364   -464.68839   -364.35015   -214.98041   -290.75125   -284.25434   -523.32114   -420.59758   -241.40318   -416.44406   -484.06178    -240.2992   -178.79834   -275.26538   -360.62795    -95.81826    385.66389   1442.83081   1819.31753    1393.0562    447.32143   -189.92175   -201.98943   -291.61935   -363.59905   -280.09019   -336.68036   -387.29864   -398.66382   -346.09176    -367.6651   -361.80252   -335.30697   -252.93399   -386.68556    -398.0873   -362.08257   -320.03005   -333.42846   -208.41254   -169.39376   -429.12257    -288.1835   -409.92603   -328.42908   -334.00055   -113.29606    277.80476   1194.82381   1346.27898   1320.13804    657.29937    172.00508   -307.50887   -414.80336   -213.66718   -330.63351   -217.48963   -373.00006    -163.2879   -205.89325    451.07175   1083.68681   1393.79868   1386.78071    724.06208    638.71946     385.4536    471.43466    -60.79437   -189.43154   -320.66998    -76.05774   -416.58069   -267.82295   -338.57665   -253.95372   -341.84871   -245.64805   -154.29052    -93.39792     -8.75369    229.04335    167.21104     45.82896    -60.61616   -355.67383   -223.71182   -275.63813   -157.28357   -188.04572   -155.37687    109.34661    122.34479    -40.80245   -128.14364   -149.19513   -340.24227   -264.38093   -275.48815   -265.12836   -289.67819   -346.13719   -244.01902   -299.37006     -97.8624   -156.70734    229.24863    314.51323    108.71235    235.26311     -18.4267    -34.56639   -125.29576     -5.94497    260.15218     570.2076    653.49036    591.16634    262.93165    239.91347     37.89361   -168.02931    -212.1793   -199.70777   -203.33496    -39.59099    482.69256    794.23689   1496.04333   1903.71051   1873.52837   1493.85956     946.2961    640.09336    154.51103   -198.86957   -179.65845   -125.94733   -226.52143    -72.10176   -398.68563   -274.89536   -130.30851   -385.46454   -250.99733   -293.86843   -363.36954   -114.31807    -61.13658    -68.49764   -205.05916   -175.13809   -224.61228    -78.61122    -34.10396     43.80887     96.47404    -11.74706   -133.51283      8.89208    -82.20699   -331.30573   -340.70602   -345.42208    -245.3582    -88.13504   -179.06479    -84.64847   -157.61464   -157.64394   -300.81623   -213.64867   -158.66298   -361.44856   -356.74101   -424.66437   -362.88663    -372.8462   -292.89408   -257.65865   -141.27513   -128.49023    -82.85483     -2.44111    -98.06503    -80.17803    251.23514    192.71759    342.55441    161.52898    127.47008    227.85456       4.5362    -99.92167    -102.9136   -146.41239    -87.29343      4.42709     -6.05129    -24.56772    238.72246       80.469     -96.7378    -55.32436   -162.70999   -125.16383    -295.9459   -279.02814   -542.31647   -208.22863   -348.15415     -312.154   -410.31675   -107.48015   -172.48972   -135.82906     65.89436    -70.00875      22.5665    234.46979     57.28571    126.16449       2.7891    -17.06099      29.1634     -53.9284     83.25216     -0.15694     55.81007    117.94437     124.6827    109.22961     -56.8693   -122.72509   -323.15414   -254.63323   -406.12916   -372.59336   -254.53575   -208.31505    -188.7209     -173.142   -263.26105    -99.19521   -154.26652    -102.1749     69.31226    -31.07747    274.41351     61.30778    240.00192    371.83935    168.18937    493.51997    567.42838    646.34945    827.45407    502.12224    355.86765    467.66217    196.19147    160.59354     85.27787     26.55976     -1.74839   -170.20487   -113.56832   -203.14119     -0.57358    -16.28883    147.55578    -52.13324   -106.70751     70.49169    121.61711     -4.95001    -32.13535    -16.55897    -135.9753   -132.03322   -179.51137   -255.75792   -174.11905   -168.26223   -328.19744   -200.04605   -277.02906   -438.03051   -354.71938   -393.92585   -490.87872     -1.38039   -322.78342    -24.06748   -208.95767   -311.78009   -242.91501   -114.45452   -138.55211    -75.05744   -136.69579     42.86785   -132.58077     -9.83878      7.15029    109.72544     16.24272    -11.35475    -43.48817    -38.14258      8.17329    -68.62073   -238.32918    -58.61545   -158.71337    -70.91941    -193.2687    -196.6896   -286.25748   -199.43667   -172.45678     56.57905   -165.81802      7.58505     54.67723     40.83373    -59.56559   -129.08305    -14.76989    -47.27183    -80.84289      3.98397    -11.43593    -67.56553    -166.8412   -305.47153   -265.01535   -179.02874   -148.73738    -60.24755   -160.60845    -21.80211    142.09963    -35.36124      5.07265    103.68023     37.61881    175.96897     182.9025      4.31234    118.61776      4.67534    164.18486     25.45104    123.57117     85.14175     13.67014    -22.64839     99.07465    163.20757     128.4014    -83.06652    -54.27747    -38.70019   -174.32657    -92.14635     44.18852    -75.13041     12.63474   -146.55454   -352.16695    -45.41839   -177.19608   -165.23914   -175.49501     -8.54848   -101.68138   -193.48537      9.83309   -144.61458   -307.56931   -363.94268   -253.54026    -70.72211   -211.64563   -258.49514   -200.98946   -388.09943   -406.93439   -237.35148   -300.56339   -287.91967   -321.32389    -359.7249    -375.8056   -229.21742    -87.47747     16.67726     63.26594   -518.45528    -49.82287   -395.48842   -298.93512   -338.50269   -292.00766   -318.72498   -204.79226   -245.76771   -211.94749   -336.25105     -355.049    -420.6218   -240.92758   -203.44254   -351.74137   -632.38658    -59.45618   -599.30841   -234.28377   -398.35087   -326.41377   -623.86749    -455.1422   -483.76506   -384.52492   -473.04845   -594.46091   -689.00883   -984.95163   -630.19505   -771.08085   -692.14085   -991.01434  -1018.67653    -856.1951   -985.54377      -753.15  -1161.04896   -900.25988   -915.40223  -1125.32163  -1034.92645   -886.67639  -1047.65048   -961.32106   -985.98838  -1176.24026  -1102.05033  -1070.78938  -1153.01206  -1343.50217   -813.81686   -960.74072  -1199.50058   -1023.3866   -839.01484   -813.25869  -1023.09114  -1087.78805  -1263.41985   -797.99905  -1204.55491   -797.22305  -1073.75299    -765.5635   -869.80242   -845.48678  -1008.90924   -472.27182  -1400.55934   -908.25681  -1079.50834    -650.0402   -895.16949   -925.04098   -559.02764  -1176.53238   -565.90878   -532.32905          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       14.00   -308.43151   -201.81753   -126.80367   -357.77543   -236.83845   -459.59987   -413.39599   -430.23976    -476.1872   -104.38248   -320.34104   -308.55666   -233.00871   -257.88465   -400.40755   -470.06432   -289.66366     -255.708   -307.08319   -542.75725   -460.09038   -197.07279   -266.14311   -253.51331    -104.3136    493.81443   1385.71452   1822.50392   1189.40089    407.50044   -206.64858   -417.58781   -390.62681   -329.10605   -451.51064   -414.04237    -336.2844   -342.27378   -573.25261   -398.66855   -242.77786   -411.62929   -413.60212   -326.08082   -390.46531   -312.97467   -489.48385   -468.05155   -391.91248   -232.21133   -397.46856   -277.46084    -357.1192    -380.4263   -274.14318   -110.64013     191.5743   1007.27007   1555.94501   1243.93871    402.25769    -62.28376   -272.44293   -442.09108   -289.98499   -559.32958    -339.4346   -240.45695   -346.10109     79.56194    577.32864    991.20137   1367.68055   1219.88424    630.91605    560.03616       3.4319    172.19595   -349.54707   -221.79513   -345.59921   -228.31451   -140.13512   -396.51943   -392.67044   -297.27063   -226.74688    -210.4267   -339.89274   -102.45435    -18.22204    262.49137    241.56985    127.52685   -192.07938   -128.62133   -324.00713   -332.47455   -663.25211   -241.53119      1.91716     42.78502      3.56511    -16.35327     46.86246   -260.87193   -355.32286   -275.69823   -231.44643   -381.27374    -297.7375   -265.14993   -374.48199    -191.8342    -182.8697    -37.33133    109.30709     316.3108    321.64732    222.41603    -57.37399    -76.52849   -398.95293    -21.84884    367.08576    494.39777    690.43086    729.82665    384.03794     78.89766   -161.33753   -101.19093   -365.09781   -345.09232   -198.64039      18.9184    217.92489    643.20037   1471.56882   1788.37291   1988.35947   1612.42134   1086.04911    477.21802    -142.5365    -43.23131   -159.32497   -199.33436   -200.95134   -319.87445   -383.42031   -406.82101   -370.29118    -396.4712   -253.48755   -333.59241   -202.92016   -210.71021   -117.99205     21.60658    -91.51838   -195.09556    -105.6443       33.612   -447.71892    177.31552    -93.03072    -16.08701    -21.92105   -203.12331    -31.14044   -277.63982   -314.14576   -424.21582   -478.99907   -207.26259   -186.83046    -83.26371     -134.018   -195.78564     -95.6388   -287.60675     -98.7918   -250.34219   -493.15816   -404.28666   -366.95999   -400.47044   -274.60748   -152.36518   -273.92474   -279.70282    -84.52279   -149.40515   -129.85993     25.10922    113.04377     48.13204    109.79814     84.82156    181.55238    146.61449     11.63538    -27.56838    122.39751     -7.74434   -179.24503    -20.19589    -18.97797    126.82303     88.71443    -27.22807    -11.16619    -27.13307   -282.89228   -242.79481   -151.14377   -236.21026   -352.93561   -114.79394   -320.85778   -251.56208   -349.32588    -53.86456    -96.00059   -231.56017    -38.81613     52.35058   -106.73415    -12.74926    111.46502    -84.75493     96.14142      73.8474      63.7501    -48.89118     62.39123     -115.477       49.194    -60.40224      2.51879    339.37234   -170.76484     -227.375   -174.03055   -283.20082   -240.78716   -443.66698   -348.30954   -440.06114   -285.23799    -37.34567   -331.62595   -122.13008   -180.32955   -110.99275    -93.06829    115.36008    -35.85226     28.99167    239.52208     306.7818    513.10681    573.45684    620.39564    456.32534     637.4501    540.33593    451.20344    304.07499    308.04609    209.68652    221.87359    -48.40709   -125.66584      6.18909    -99.00989     27.38266    -22.80106    -35.05821     97.14836     26.46578     65.32794    149.87398    -40.79106    -92.43531    -11.06871    -125.0582   -214.83142   -320.77895   -214.13456   -256.08735    -218.3252   -179.16377   -375.40409   -286.60548   -300.63419   -270.83213   -333.31321   -276.32941    -336.5729   -185.59686   -266.16582   -230.98317   -204.89841   -138.52995    -244.0448   -231.33173   -204.57363   -139.69761   -115.51655    -80.57543    130.52853    -142.2387    -11.85382     183.7414    -97.41911      -9.1059    -87.06871    -62.78355    -89.99693   -216.40143   -101.30408   -178.63744   -225.67405    -36.09828   -219.53026   -132.09711    -56.54844   -100.70354   -174.54027    -36.98294    -45.19721    -99.43015    -59.79915     24.77813     39.91143     33.84691     67.49492     -9.51876   -137.68658     -71.3094    103.72116    -61.43901   -239.56447   -177.68901   -174.13302   -167.68742   -172.80371    -36.07559    -74.32414     35.59407    -82.82337      1.80031     -16.4052   -162.17784     35.89773     61.07432    179.73334    175.35663    211.39557    116.60039    419.79331     94.44311     72.59117    197.62402    -66.73857      5.58244     14.54139     -36.4438    -47.78685    -26.04919   -114.77399   -183.13293     70.62524      86.4525     -53.6952    129.87795    -69.73207      8.69638     38.11139    -97.14219   -158.69467   -266.69477   -171.12698   -164.80026   -301.35685     -68.0599    -90.77735    -23.69182      40.5062   -162.16985   -133.41093   -168.05847   -174.88851    -286.9991   -367.14779    -184.3521   -324.65159   -160.25797    -229.4742   -301.95266    -44.66343   -134.40782   -268.19959   -310.26235   -259.88346   -103.40415   -273.38951      34.1879   -218.08456   -405.31026   -296.34301   -157.68763    -108.2249   -134.15627    -35.28693   -319.70975    -383.1808   -291.92816   -477.05732   -352.01296   -193.97795   -217.79053   -221.32997   -360.25465   -392.84011   -170.72569   -712.63379   -525.65114   -366.39512   -763.23921   -781.86328   -537.42602   -575.34439   -624.25477   -612.82759    -921.7081   -816.06429   -573.98553  -1011.05788   -846.07635   -845.84066  -1227.21565  -1113.01508   -859.43126   -831.07336   -904.24736   -870.79027  -1177.09259   -753.59933    -905.6387   -1029.1112   -996.61922   -769.13708  -1083.54553  -1034.50554   -995.76403   -949.28927  -1269.73461  -1219.91964  -1082.34644   -717.97508   -903.68548  -1275.65263  -1081.12452   -971.89736  -1290.44165  -1396.17912  -1114.56151   -929.12884     -872.574   -826.70881   -970.57875  -1019.79283   -596.56331  -1313.39433  -1560.93759   -974.73346   -829.19517   -924.25261   -617.88954   -844.70539   -540.20636   -261.80169   -609.09064   -488.96618   -236.22995   -312.62404          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       14.50   -197.45038   -346.68565   -192.09325   -434.94737   -286.59995   -343.14598   -482.47077   -336.39454   -466.73368   -295.99444   -226.07289   -287.32336    -237.7489   -274.27336   -451.37656   -469.07821   -242.26467   -563.11543    -203.6241   -134.92011   -520.51852   -180.91599   -231.67166   -222.85567   -114.43356    434.35817   1328.73221   1796.59089   1072.15087    434.02256   -216.44655   -283.20804   -523.12096   -423.09081   -255.73041   -466.77083   -126.92568   -256.72009   -340.48487   -368.56813   -204.74082   -356.43608   -327.08188   -316.36885    -260.8617   -227.77882   -414.41459   -543.34167   -260.26391   -288.91713     -374.416   -292.55379   -162.99854   -304.77179   -284.47673   -144.09605    368.36998    975.54631   1569.79368   1263.43073    419.33219   -137.90054   -263.65213   -177.95052    -469.3015   -333.54073   -449.89504   -344.35878   -557.18213   -113.22696    469.57074    976.23729   1592.07243   1204.24233    749.96729    553.78162    552.65849     52.15511     -3.82177   -358.76562   -423.25803   -449.27309   -287.43979   -395.91468   -542.51921   -290.54685    -245.5233   -322.60527   -364.15397   -590.66146    185.55699    133.75517     60.19553    119.47374   -136.45004   -204.90327   -336.86974   -294.25717    -332.6381   -230.66118     81.54683    -32.92362     42.82252    -59.57407   -274.23842   -117.85908   -249.44325   -348.76529   -453.35852   -541.40912   -290.88155   -104.78015   -407.40305    -255.6147   -232.33333     53.06925    127.03975    252.09015    371.23759    147.24209    -98.01454    -67.17772   -245.71421     -4.51447     137.2008    476.46934    449.54218      523.634    305.70385     98.77373    -72.50708    -79.38099   -233.80558   -227.37977   -177.75517    -66.74242    418.71844    971.05284   1423.79423   1846.71275    1444.0765   1544.71823    943.49401    509.60937    131.31924   -125.23177   -113.81254   -101.73879   -216.21752   -271.39751   -313.98949   -125.03675   -267.71213   -315.86882   -255.97059   -378.07929   -221.10968    -193.6274   -159.37273    -135.0151     50.23766    -91.08487    -59.93019   -120.56647    -54.12381     41.81322    121.48624     82.66924     94.30294     27.92738     28.32413   -120.69468   -354.05163   -267.12872   -310.26748   -148.70515   -220.05553   -161.87512   -257.28854   -279.63408   -322.26294   -215.78049   -248.81249   -226.16569   -267.44028   -329.27606   -150.03791   -166.66481   -349.32134   -355.64583   -253.04276    -201.2121   -348.16666    -68.19599   -192.18727     22.17277     116.4684    -47.77224    202.41799    176.36726    144.07916    -31.13167     -2.96635    176.86554     45.82637     90.58977    -58.66247     11.63326     20.55678   -151.24929     76.23609      73.5001     66.65287   -110.60064   -248.12976   -299.77509   -308.62601   -237.27378   -352.51312   -435.02291    -216.4703   -239.16309   -363.69073   -225.44302   -145.60453    -24.56006   -141.78155   -115.22737    -34.67441     156.7559    -25.63725      31.2805     21.04947    193.11693   -113.85743     62.34162     65.02718    107.39355     -2.82405     52.61948      7.36255    -49.44858   -122.69235   -109.89129     12.85651   -178.03436   -266.34815   -252.57755   -163.94395   -297.61969   -322.64206   -271.75778    -159.5264   -102.09934    -66.83738   -113.98166     21.98546    -79.47261    116.39616    198.28986    340.59964    106.60144     365.7433    597.22876     666.6782    727.65038    666.15386    438.52131      494.389    356.84095    320.02027      17.0116     131.4664       6.6523    -77.44886    122.89361      -5.3457   -127.77398    -21.45727      4.23598    -74.89316     61.78369     49.22063    168.59962    -49.29225    101.21235     68.61835    -59.19186     11.94809   -210.86324   -290.57182    -298.3368   -335.24685   -279.39431   -297.54408   -470.21592   -385.40529   -358.98782   -371.03302   -254.23296   -275.66699   -228.60369   -529.91016   -140.26778   -235.63723   -141.58473   -237.44718   -154.58687   -185.99063   -103.22678    -33.91177   -187.56062    -79.09469   -159.73749    159.14805     38.05796     13.71982     78.48353    -11.34947   -215.56017      1.08533    -74.11054   -116.74346     38.73436    -79.23663   -218.66259   -227.96408   -111.39456    -80.60762    -172.5115   -182.03877   -211.48524   -127.97063      -71.591    -87.05051   -157.91579    -49.51209    -88.69822   -204.11406     55.63262    -69.67043     95.94293    -89.50046   -183.97172   -180.77929   -242.91115   -310.06201   -150.86503    -22.34954    -96.39022   -152.53142     -8.65069    -143.3009    135.64585    -54.79662     76.66257    295.64038     76.41514    146.64211    146.37769    155.83485    209.55113    179.87916    271.59104    194.70987    -25.96865     30.36873      6.27594   -115.71471    -22.13031   -136.03985    -40.82838   -107.62129    -61.47629    -81.48911   -206.78951   -238.80483    118.55008   -152.76915   -296.56137     71.71307    145.89539    182.81461     -6.25834    -23.55873   -168.97902   -154.86717   -109.04639   -209.22274    -47.99581    -93.84169   -266.60067   -343.31965   -178.37243   -217.52778   -304.25558   -492.31281   -394.48759   -339.12331    -312.5811   -132.63906   -343.27622    -91.49668   -500.13261   -246.93247    -380.8672   -317.45875   -373.12097    -289.6354   -480.11723   -449.94415   -207.79024   -203.99514   -108.23612   -202.49654   -277.00873   -476.69158     -94.2222   -437.25951    -354.1693   -390.17768   -212.67478   -427.21132   -253.41688   -314.07552     41.97213    -14.55839   -272.32333   -627.06211    -657.8408   -530.61726   -672.18693   -625.54488   -887.98738   -843.31755   -729.22204   -792.67329   -573.38784   -509.46877  -1046.11205   -734.82367   -894.75272   -913.59768  -1036.08953   -757.25526   -890.45508  -1182.87852    -665.7622  -1125.00118   -749.93498   -978.85453  -1019.99746   -922.70543   -908.76617  -1096.09677  -1026.61016   -814.43262    -939.6955   -858.24387  -1340.78436  -1018.62238   -779.32485  -1345.18987  -1264.32778  -1073.00164   -875.25191  -1155.32779   -733.42586    -1064.984  -1126.97804   -802.96195   -1224.5119   -766.28416  -1000.38287   -841.84326   -891.06228   -603.49722   -722.20446   -897.99572   -483.10303   -997.23829    -734.7263   -497.90092   -551.96581   -377.10276   -542.63382   -591.64335   -315.72452    697.70826          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       15.00   -102.86722   -422.25781   -352.73865   -458.46336   -324.45996   -313.96914   -514.08551   -270.48324   -423.82921   -353.46338   -230.63587   -259.32475   -109.63172    -186.7521   -374.97016   -370.24096   -358.56398   -415.48677   -147.25681   -160.91627   -295.83049   -357.26979   -247.95096   -226.98507   -202.67005    398.90927   1402.19122   1774.76423   1348.87875    194.22287   -243.20939    -154.5325   -459.03441    -418.8331   -153.29721   -378.05862   -272.86853   -172.36447    -464.6239   -266.78823   -505.06255   -360.34482    -532.6022   -335.08067   -330.91753   -361.86179   -250.25106   -232.44719   -399.22091   -382.19729   -443.78189   -292.45288   -329.95088   -294.41095   -369.82983   -153.87173    305.07752    988.25605   1436.06216   1256.16936    492.53721     11.30761   -169.51014   -313.01331   -330.10015   -288.35944   -257.00022   -213.58443    -347.1092    -103.6013    445.17216    994.46948    1523.3866   1228.96421    720.01124    602.09423    356.53617    293.23899    -46.52048   -472.92431   -146.05089   -211.79971   -419.64851   -277.42868   -305.19475   -371.19434   -216.93732   -239.97228   -377.40107    -93.86134    -96.87662     58.66647     45.02968     67.59997   -142.46516   -198.57819   -315.37485   -277.31769   -451.52076   -139.82313   -116.51866     34.72019     59.40204    -26.42188   -171.64204   -209.31973   -314.65057   -323.87802   -302.52324   -345.00139   -247.81332   -216.22024   -177.48002   -426.67884   -153.32536   -197.23759     88.58984    352.87118    236.97386      65.6388      5.31341    -53.35959    -92.09507     96.75479    376.98994    516.58383    626.04079    375.99457    388.91726     130.0648    -49.61059   -282.39655   -130.50438   -222.06722    -66.88795    115.01623    210.56403    876.46004   1525.36733   1947.83546   2128.69565    1636.1651   1057.00501     528.5984      6.12447    -91.01547   -143.91795   -191.70618   -291.99965   -142.72816   -133.21468    -92.79448   -304.27489   -322.64603   -383.02469    -317.4648   -307.09886    -451.3193    -76.94736   -192.67669   -167.23775    -84.86214     54.87246    -298.8019    -24.79261     103.5289    117.60757     146.2808      4.11987     28.77751   -102.32512   -211.88177   -222.50845   -194.29142   -228.89365   -246.41543    -200.0852   -124.47339   -184.47373    -81.12511   -176.66929   -217.87587   -251.00483   -205.99868   -283.36777    -394.6383   -456.56747   -142.44176   -355.65986    -420.4686   -197.77258   -252.05025   -171.65909   -214.51926    -24.00227     -22.2032    -22.63705    214.15611    218.02581     160.8648    153.36574    268.39531     33.78533    -21.23076    -53.82829    -18.38184    -77.66893     35.96387    -67.46622    144.13046     157.9705    -39.12892    -42.79665   -146.80311   -258.42494   -231.42462    -187.6506   -410.43177   -516.44873   -287.96916   -407.08719   -300.24337   -244.56885   -243.50524      3.40691   -132.04664     45.79315    -34.36923    -10.55475     61.81268     89.66142    235.24504    119.75508      9.91081     30.91849    103.90319     14.92919    -50.91581    159.03259     96.45737     22.97577    -115.5891    -10.03301    -71.25191    -61.23884   -190.81675   -186.93655    -72.55578   -225.33003    -316.2192    -79.70876   -277.96652    -73.96974   -102.68339    -12.49158    -92.58189     98.88938    126.06565    -34.68122     22.49565    181.55648    484.81219    572.81997    608.50547    819.78239    630.90612    669.33909    592.96856    425.75585    315.64952     217.3258     71.81055     89.73225     94.59929   -118.62643    -44.53607     -58.3886   -140.16257   -154.19607     28.37047   -111.46477    141.38096     78.67892    127.06662    -61.21925     99.46972    138.94035    -17.05453    -21.94109    -82.58805   -103.33446   -191.27222   -191.50637   -379.50486    -263.0394   -278.46532   -118.89827   -147.42132   -236.53394   -327.21721    -257.1603   -377.09699   -172.92968   -250.02961   -215.90306    -58.22825   -204.71928   -197.61882   -111.38234    -93.30551   -143.19463     37.43359     14.97045    -69.55338    -87.78074     94.11615   -198.94715   -161.20883    -175.7067   -223.54526   -228.70855   -259.66493   -223.84339   -220.57709   -179.06698   -286.13296   -315.27404   -127.22445    -29.90723   -292.03759   -243.62353   -246.99326    -35.26015   -127.82416   -148.38167    -66.49855   -205.88273    -22.38215   -126.59256    -14.27147    -36.25088    -46.40775   -255.00328   -122.75326   -170.79441    -92.20296    -226.9689   -259.37541   -206.06429   -158.92798    -80.70518    -52.79767     28.07315   -108.97872    -193.0878     39.66604     36.59888    176.28756     67.55933    142.10873     38.58553     99.07698    214.73247    203.48052     39.50623     46.38736   -142.40946     62.06985     59.37697    -75.47214    -46.32911     59.04665    -73.77012     -8.50151   -144.14235    -76.81098   -104.00503    128.74629   -183.29948     45.38689      6.87791   -180.05409   -214.40386    141.87042     12.80014     -80.1668   -194.47065   -283.91316   -258.85532   -216.98053   -275.72557   -353.29873    -268.3962   -126.50466   -343.22296    -339.9239   -427.41653   -671.21268   -285.11654   -288.42437   -316.04086   -187.44167   -185.51873    -228.3108   -209.52174   -257.43651   -367.69124    -99.48798    -72.39655   -298.62353    -55.67947   -232.56885    -182.3713    -69.38705   -496.49031   -235.83611    -109.9097   -425.79679   -351.46159   -112.35267   -440.08646   -397.25052   -606.98883    -527.9274   -649.66297   -317.88861    -299.5946   -583.84791      -263.32   -748.73283   -553.36843   -616.00467   -780.83002   -492.95028   -770.17081   -783.89106   -770.32943   -851.00909   -639.74066    -801.6215   -925.42857   -654.75422   -841.80478   -710.02732   -705.06957   -882.99418   -892.71851  -1146.80905   -818.10421   -783.86915  -1104.55382  -1365.59276  -1021.14666   -989.37288   -620.99607   -918.82497   -960.78719   -1234.1536  -1046.47747  -1091.17088  -1072.98237   -869.46492   -923.45954  -1018.76765   -746.18356  -1078.79775   -935.93194   -958.76871   -715.95359  -1072.33734  -1224.61854   -885.34223  -1063.19983   -981.11938   -740.92458  -1094.37471   -777.39482   -1051.6655   -823.77955    -710.2658  -1212.99791   -560.77907  -1054.42722  -1407.76389  -1497.49975    -74.62185  -1772.26151   -357.99354   -773.26078          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       15.50    -97.59912   -290.83745   -476.61963   -367.94635   -342.14565   -561.41371   -557.20951   -215.89293   -419.46787   -312.11449   -198.63297    -156.2887    -52.16979    -92.69234   -430.83977   -188.13185   -304.58581   -122.27103   -160.68522   -329.49565   -267.28163   -596.06789   -359.46057   -172.85055   -313.05606    485.07363   1326.60322   1721.96731   1037.87201    212.52862      1.18618    -76.56608   -329.81763   -291.15679    -481.3718   -341.35555   -226.52121   -319.70244   -393.38896   -267.81146   -484.21081   -375.44407   -617.47788   -181.61011   -368.93186   -180.16329   -364.43112   -440.58819    -308.8743   -288.04508   -370.06639   -405.12849   -318.69449   -414.40631   -386.48693   -135.41862    152.40809   1154.67879   1467.90553   1315.05663    461.08188     48.47707   -290.63571   -211.33817   -213.90217   -203.53938   -222.20738   -205.09965    -389.6529    -75.02539    456.33032   1126.93554   1315.14144   1136.22288    650.89812    694.93539    313.95142    277.78408   -299.48986   -449.07382   -570.39094   -301.50362   -366.27576   -274.26533   -458.34647   -256.88328    -205.8121    -194.7693   -134.70525   -265.03778    -38.39755    236.42914    272.70316    143.48279   -123.60855   -268.45226   -336.03402   -409.03105   -285.75366    -207.7797   -180.14719    105.77963     23.04338    -96.21806     -56.0194   -213.38518    -94.09311   -334.02627   -163.41002   -346.68065   -348.38952   -355.17677   -251.50773   -313.22544   -251.73375    -92.45809     87.03026    172.40126    301.22648    181.33819     58.95786    -57.79657   -148.73262     36.75704    175.27135    321.06713    580.46433    554.56992    382.13971     87.58093    -35.38622   -321.61796    -341.4322   -142.70304   -338.42874     28.91061      81.1014    919.65843   1277.28643   1945.09538    1898.5834   1641.56576    836.56302    300.69777      1.74426     -3.54065   -206.51659   -235.14968   -190.70883   -214.94989    -234.5273   -314.03922   -271.90094   -228.33398   -353.75094   -285.72398   -241.56745   -363.35369    -34.23385    -131.2423   -114.87612    -126.0222   -109.17174   -161.05792   -173.10002       9.3752     68.78458    207.33325    -41.80994   -206.58559    -95.80286   -364.12211   -351.54677   -381.32547   -285.00057   -287.80706     -82.5745   -139.01263     -93.9682   -243.00038    -109.7996   -215.79931   -287.58026   -197.22548    -134.5685   -437.32765   -260.25688   -326.48444   -396.01484   -363.77257    -299.6133   -152.41679    -111.6465    -77.04477   -229.80499     91.58506     30.91837    152.97586     59.98618    260.26618    149.85884    219.20057    208.32847    -38.28528    -92.52785   -110.45102    -73.39575     85.36592     43.79846     24.89214     39.23572     82.58975    -86.18372     15.25422   -338.25484   -208.29712   -274.72721   -317.97945   -427.17804   -481.93358   -322.63541   -345.19536   -291.54788   -189.31196    -141.2612    -81.35965     34.74396    189.84047     96.79064    108.49379    153.56701     73.29964    149.30705    118.21589     67.49701   -173.73303    140.16889    166.30611    126.60109   -103.94881     54.64852    -97.21943     -156.283    -96.84556    -81.22308   -265.57907   -344.40012   -276.41545   -175.79748   -192.98553   -177.42747   -249.84386   -115.71395   -138.71216     81.06626    -27.07497     -34.0102     26.13514    118.42233    203.86052    193.35184    292.51014    578.00161    536.63539    641.65118    657.21574    534.36501    598.99619     479.4798    259.92849    259.63264     70.23612    106.40624    -51.72706    -49.07888   -170.36256    -75.11905     -13.6929     35.08137     23.32955     159.4374     -4.49696     53.76468       3.1136    -84.23678   -140.11773      9.65415    -78.95069    -21.95497   -161.64501    -288.0039   -210.48002   -160.04831   -272.89147   -419.26369   -278.01379   -317.06498   -280.85258   -369.48808   -198.67821   -285.94627   -354.57258   -200.27489   -334.90422   -331.65442   -178.45683     -92.3295   -287.47711   -159.85851   -106.53845   -184.09905    -82.82828    -21.41476     37.45499     19.27404      6.46672    -24.88651    -32.99483     -7.46281   -199.53259    -72.46015     -190.218   -251.65152     -0.97629    -58.32414   -227.37044   -328.09332   -270.78149   -344.08647   -237.84792   -239.21771    -24.63244   -134.20164    -117.8737   -193.81449     24.58053     29.47201    -46.07477    -17.53717     -26.1962   -117.07349      2.65942    -16.90794    -92.66064   -162.07963    -75.32949   -102.36167   -267.25793      3.47274    -135.6243   -133.54524     45.87001       2.9252   -195.21458    -49.72506    196.58838    138.88584    100.60672     82.86974    305.91225     84.34586    319.21501    -75.91233    146.11398    203.49156    109.28209   -142.37422    -54.24906    -52.44378     60.76377    -58.41847     106.6523    -67.64897   -244.25474     33.11503   -282.97285   -253.12455   -148.70242   -123.86459     20.82733     -92.2997     43.76513   -116.49923    -10.88571    -132.0913   -280.99903   -154.79747   -298.29406   -154.97667   -110.06474     -416.054   -303.76101   -381.73063   -223.35832   -204.54444    -138.4333   -176.34198   -192.28231   -243.91442    -89.35246   -370.31614   -189.79082   -153.64522   -329.85424   -303.20726   -126.75687   -493.67934   -456.24121   -337.98989   -175.81153   -622.73751   -380.89549   -163.90624   -526.93412   -333.46638   -469.70369   -149.69354   -321.13183   -370.44697   -361.18558   -310.29422   -511.01393   -441.44372   -211.48004   -397.46617    -213.7884   -650.14249   -283.80738     -556.679   -256.51085   -527.59413   -795.20608     -678.815   -690.58147   -593.52133   -599.41502   -863.38017   -940.78072   -911.20348   -426.00481   -728.14611   -824.65148  -1139.58294   -950.18045   -763.63625   -948.84459  -1113.21484   -691.91191   -918.40213  -1075.17614  -1297.09827   -945.00802   -784.85035  -1304.27211   -742.06892  -1110.91181  -1240.14084   -919.13521   -869.89275   -816.72244   -916.62931  -1130.46499   -966.77259   -999.22102  -1239.27251   -1323.5876  -1231.07064  -1042.14009  -1299.55283   -876.92504   -1273.4304  -1463.76563  -1210.91409   -969.80269  -1240.98842  -1358.66485   -749.61572   -999.76029  -1754.70415   -623.99747  -1752.58701   -1220.8617  -1004.53782   -809.61771   -1482.8063     34.30781   -544.88208   -572.02377  -1324.57951          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       16.00    -235.7063   -143.38228   -456.03214   -306.61989   -388.78561   -877.44904   -490.01231   -185.10367   -374.83311   -272.70735   -129.28163   -284.86106   -129.16306   -312.19795   -500.78775   -248.31317   -259.64322   -186.75503   -176.70746   -311.03163   -211.53451   -635.94067   -342.11328   -457.84238   -306.52437    629.19332   1278.28488   1760.24796   1138.96009    357.37276   -113.67328   -146.49302   -177.64113   -261.76726   -476.34086   -295.24965   -405.70772   -254.47448    -192.7288   -199.16255   -436.15245    -412.1431   -417.24043   -281.33454   -438.39113   -418.36062   -356.71159   -248.37251   -267.42257   -459.48082   -379.61076   -258.55028   -249.68621   -318.89382   -241.80962    -81.27969    251.09932     936.9862   1538.63725   1207.43881    596.91072   -114.97353   -343.32537   -273.82612   -311.30597   -134.22808   -281.13857   -378.06536   -279.22621   -150.95985    328.32753   1102.49373   1295.85508   1354.74558    717.00639    519.71009    412.87081   -199.64643    -708.6748   -878.34399   -618.27202    -431.1022    -82.67084   -137.61826   -193.02386   -325.64485   -295.69464   -295.94401   -225.58259       -18.56     76.23164    141.04253    169.35236     84.76018    -188.8134   -264.13269   -407.90655   -352.61585   -191.99974   -191.07952   -382.21446    -31.70179      32.0211     55.93232    -75.66703   -271.06599    -86.58099   -383.55637   -340.51126   -326.78889   -424.77773   -263.45082   -322.99763   -323.63012    -71.09307     -22.6974    143.13355    291.68782    492.43841     48.55941     43.83451   -137.01807   -212.67017   -118.72125     80.38236    444.76644    494.60315    497.12809    344.21937    146.30886    -164.9829   -197.78474   -177.62209   -275.47707    -38.46199    -57.25602    349.81058    887.05529   1362.34048   1985.31578     1926.629   1458.46254   1133.23281    202.49148     31.80924   -384.60835   -282.07031   -134.21862   -201.25501   -362.31041   -206.80101    -357.8599   -191.53138   -330.78738   -260.93259   -312.64407   -235.85125   -202.72372   -173.29811   -166.30923    -81.12786    -69.26315    -36.35791    -137.8549    -46.99957    -21.42069     52.98299    -35.78831    -21.95708   -116.46306   -172.57848   -281.46462    -309.3371    -425.6582   -364.04201   -207.62113   -191.76181    -93.67291   -190.88145    -199.7321   -126.30796   -259.74268   -174.80739   -364.31759   -345.27567   -382.88003   -235.98908   -382.53564   -446.87411   -189.14315   -291.44935   -127.59464   -133.36963   -108.40813     92.70865      6.96809    -17.06325    -12.80253     98.00559    111.50527    247.89909    181.03937      0.57717    -65.76175     -75.6077   -102.49044    -79.10408     35.19785    109.63447     53.70271       3.9802     24.05227      7.79483      36.7936    124.81681   -243.31385   -293.42497   -291.76757   -114.75924   -291.72421   -277.38726   -189.41484   -170.70398   -205.74732     -144.743   -275.97962     -2.14105    -61.95271   -248.08732    101.08221    -65.41289     87.94524     -3.00328    188.58501     58.38559     63.33595   -167.50456     27.70799    152.34118     11.59692     17.22315     40.59022    118.97584    -243.5568   -161.64459   -173.11225   -172.28452   -113.53416   -277.30878   -377.72422   -333.36786   -252.59985     -7.54426     63.19542   -163.47276     31.45858   -159.91901    193.04138    106.20308    289.69056    336.52586     222.9649    520.38247    383.71957    566.95532    589.43107    438.51949     470.7575    710.16137    451.19134    365.91438     95.47019    142.11978    -58.37834      2.26157      8.34529   -147.05538    -25.15543     20.76827    -40.40605     35.17618     59.25848    -96.67764    189.49184    162.86101     -8.36151     33.35334    -26.05422   -103.51939   -388.93692     18.39144   -272.87723   -277.04624    -147.4962    -53.02851   -241.80253   -322.76447   -276.50164   -186.40508   -401.08117   -260.51954   -276.75196    -88.72059   -333.90579   -256.86947   -403.40615   -292.34927   -135.00195   -177.72592    -51.71736    -38.00202    -90.89072    -78.03611     70.94706     38.14592    146.10803     146.9121     49.41296   -149.54448    -29.87664    -16.87827   -114.47785   -128.88421    -49.66533   -192.64927   -261.44078    -312.0097   -162.07739    -175.7786   -203.18859   -255.64936    -55.54079   -157.24638   -139.25697     46.59282   -182.12621     39.18789     -9.40491   -209.84576    -28.07763   -217.58941   -123.60449    -172.3382   -213.24028    -13.58648   -124.08725   -211.00023   -186.47644   -176.58491   -152.81474   -208.85196   -261.97473   -147.45374   -154.98754     82.50324     29.76414    287.34661    -59.94713     12.14687     54.50373    157.28744    205.91462    135.85757     74.53075   -111.13148    -98.31036     71.38617      94.4507    -45.23829      5.18964     -63.1576     -4.79135     32.66205   -143.71152    141.15798   -167.75369   -127.49402    -27.59187      17.7939    -81.50399   -105.15054   -117.53684   -439.97414    -56.33659   -339.30144   -177.00851     -87.3901   -330.27835    -183.1969   -296.00811    -418.3757   -365.53417   -278.34417   -381.57502    -433.9609   -174.62197   -607.39634   -333.70789   -303.51483   -342.41677   -353.11922   -324.09513   -292.02227    -16.09658   -303.51163   -328.86324   -289.74968   -274.45317   -270.72569   -375.24901   -369.53508   -386.91072    -355.3089    -52.47628   -239.21583   -703.06978   -597.26243   -305.04098   -537.02044   -426.33596   -276.91911   -410.72912   -384.46859   -406.86202   -297.69989   -213.80709   -526.58068   -276.89632   -576.36721   -744.61818   -452.70979   -736.79357   -555.34925   -552.37032   -894.56864   -836.24632   -651.88546   -896.41152   -642.57622  -1049.36545   -663.07651   -735.68828   -671.91606   -899.77353   -564.37931   -725.12852   -806.27822   -652.69989  -1135.04045  -1265.08398  -1013.27595   -912.55492   -895.24277  -1082.41863   -935.65339  -1156.83354  -1083.10846   -1113.7912  -1043.57029   -752.80876   -952.41483    -1491.425   -955.64891  -1010.21958   -540.77873   -366.11489  -1148.22809  -1266.43937  -1359.68519  -1006.86639  -1455.39211   -733.13747  -1422.95112  -1014.81166  -1127.68905   -651.72391   -1051.9157  -1040.82274    -587.4121   -915.53771  -1500.18168   -928.39604  -1383.03075  -1080.65561   -845.63605   -529.24685  -1182.33657  -1319.71435  -1005.43932          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       16.50   -371.37691    -92.67186   -418.99024   -348.59899   -421.64391   -688.53072   -397.56268   -321.80236   -299.41785   -232.50938   -198.41344   -537.98524   -222.62555   -551.72783    -379.8287   -286.52855   -347.17374   -225.50283     -273.003   -215.62054   -160.24803   -378.79132   -239.43292    -454.7155    -213.0904    670.30792   1361.85849   1740.20714   1308.30502    281.01563   -201.48139   -326.73477   -206.94933   -412.40509     -242.838   -236.02519   -381.16103   -199.92379   -236.79143   -437.49767   -376.54507    -469.0057    -471.9735     -293.665   -419.80262   -298.22536   -352.12272    -214.4934    -282.5492   -188.29775   -369.65471   -343.35111   -275.89694   -318.28277   -379.09313    -36.59387    284.55456   1160.38693   1220.04981    1074.8135    624.85358     68.19949   -219.77677   -322.99651   -346.44609   -326.74818   -398.92128   -407.81757    -244.6912     16.19323    438.21275   1034.09495   1435.33921   1339.11646    625.85622    377.26622   -160.97461   -998.13178  -2333.30851   -1696.3774   -1384.5436   -436.50128   -383.55684   -328.03158   -431.54021    -324.1478   -405.65307   -520.03428   -270.38734   -209.56731    173.78637     -16.7448    -51.16541     45.61216   -135.32094   -231.26325   -234.61253   -293.47661   -227.56064   -240.38989    -145.1396    102.18388    116.58344    163.32451    -88.31354   -238.62717   -415.96159   -389.54207   -364.24946   -330.38956   -357.77196   -349.32157   -329.74302   -372.69186    -207.2534   -142.28364     13.92394    257.26202     313.0283    122.32666    -51.24525   -121.08958    -73.51339   -172.32839    269.38464    420.48988     745.8265    633.76514    269.84684     68.39354   -147.67102   -172.06805   -170.06747   -301.40777    -269.8276    -71.50809    218.61593    951.98907   1475.62923   2033.75239   2003.09791   1656.72714   1119.18887    710.52963     67.02482     18.50866     -43.3414    -268.3544   -143.70019   -325.08456   -142.20006   -299.29911   -286.75997   -337.11748   -276.66504   -374.68117   -235.78335   -116.12051   -189.97947   -222.70963   -311.65984    -208.6032   -204.39705   -210.11357   -174.34043   -123.40218     19.49754     -1.03939    -33.36941   -247.31201   -422.58118   -466.42314   -413.27416   -356.80666   -180.66697   -264.21205   -198.77864    -53.05474   -121.00926   -197.57647   -443.76529   -290.97645   -457.48725   -344.62794    -193.1633   -320.38302   -331.36776   -255.63976    -379.8854    -78.88458   -260.48363   -113.86782   -250.49767   -105.47177   -115.40678    -40.73588     28.61976    150.92067    307.80464    163.17446      -1.1169      282.267     66.35291    -69.65734    -86.04287    114.86493     -1.51349     -1.85065    145.87001    -33.56932    103.18875     103.2436    -42.45856   -181.52503   -119.26379   -339.33687   -296.48558   -305.88718   -270.21524   -333.10988   -396.96323   -285.99214   -307.33318   -240.32988    -189.6104   -227.25136   -125.13956     86.42051      -24.582     47.61446     57.99142    -95.73639     64.04174      9.98409    -20.84691     37.47357    213.17682    147.11047    -15.13523   -114.51715      46.4761   -103.51358      7.13648     -155.097   -186.05184   -233.94297   -211.21893   -360.93266    -267.0768   -313.18847   -248.51029   -258.86176   -290.13848    -41.90117    -50.12522      7.33146     141.5596    -25.30996    112.62173     172.2823    222.43989    391.18825    460.72015    581.57582     627.2765    604.16516    664.59254    529.79933    422.88669     436.2762    243.57972    258.87682     26.65402    -68.42011    -88.56544   -176.74623   -191.62939   -114.92584   -114.84157      8.83677      82.9205    202.74889     64.64886     43.70046    -15.88903    -10.93743    203.15546   -129.89066     12.35519   -155.34001   -223.69495   -231.91674    -96.96295   -326.31346    -236.9181   -296.53788   -417.86167   -362.38764   -327.40003   -408.50407   -333.48161   -304.26111   -224.31007    -409.5165   -323.20077   -229.47592   -263.56962    -94.87666   -206.91215   -288.08736   -275.27177     -60.5708    -92.10855      1.93027     29.95675   -100.06181    -26.27763    -79.59508    -95.57526      0.04109    -72.18549   -202.33386   -331.32568    -85.08265   -211.68634   -122.72582   -247.07565   -246.20776   -240.72029   -216.79307   -195.38485    -21.87947   -134.36043    -27.80091    -60.95418    -347.6523    -26.98767   -232.02804   -160.47589   -126.39631   -119.89132     40.64296   -226.08803    -36.60969   -295.61173    -128.2896   -270.61011   -125.31476     -4.66242   -168.10548   -198.76785   -170.33099     34.66962    -26.46427    -72.10195    -64.87683    -80.91886     52.24664     64.73995    177.83467    196.02711     67.11566     83.85966    -30.31243    224.42219   -108.56943   -129.73053     -24.3225   -219.92808   -165.12378    -34.51473   -245.85313    -26.48347     54.90131   -127.91153   -173.08176   -277.35616    -98.26596   -206.94064    -85.95915    -79.11492    -39.04389    -66.34102   -229.88853    102.19536   -220.15044    -248.6065   -130.18535   -272.82992   -121.66581   -222.76523   -327.95347   -536.98687    -56.57344   -247.14109   -241.59401   -229.82659   -315.42979   -254.74525   -218.43247   -380.68583   -301.07985    -598.9713    -362.4378    -41.75059   -349.81084   -441.71542   -419.58966   -237.17001    -281.7591   -401.50444   -419.13689   -243.69339   -186.48875    -280.5598   -401.19946   -188.01969    -226.2356    -82.75214    -435.3345   -459.66099   -437.63352   -582.71942   -545.48349   -165.21992   -522.70365   -488.01109    -855.6769   -506.58881   -694.52624   -799.79251   -459.36806   -816.20193      -789.16   -761.12848   -906.10187   -908.07236   -927.50568   -628.85285  -1247.85845  -1079.17303   -767.71544   -830.96545   -861.87055   -1093.7218   -920.74435   -784.58617   -715.17561   -824.48851   -697.32669  -1194.16825  -1215.05006  -1018.51556  -1061.56228    -869.0296  -1385.41766   -1120.9876  -1021.04324   -886.19939  -1092.51557  -1350.06371  -1158.32908   -835.96715   -851.46383  -1819.83814  -1266.39115  -1267.44409    -718.0737  -1049.58622  -1205.36166  -1300.38354  -1332.39704   -1011.0419  -1277.88199  -1687.76076   -821.68829   -982.62738   -398.48151   -947.75253   -841.12615  -1016.04584  -1249.27124   -692.73269   -1208.4891   -979.25236  -1806.28629   -402.03879    -804.6308          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       17.00   -453.99611   -235.73317   -378.95747   -414.18501   -317.40232   -492.74815   -399.41043    -420.0889   -335.37603   -242.90759   -308.81451   -503.53831   -267.63178   -318.97448   -281.62884   -318.32611   -291.14649   -261.84812   -447.94544   -115.74655   -294.95217   -218.08552   -203.76894   -361.24734    -30.80531    480.02614   1537.76297   1780.81727   1362.42605    156.52324   -136.55384   -621.82656   -174.36997   -497.85053   -256.12609   -266.46785   -264.96536   -260.24363   -523.86553   -441.25343   -491.05607    -435.8385   -284.45603    -73.03742   -415.43493   -371.68397   -326.83575   -399.12616   -389.97956   -306.01609   -317.36807   -280.94691    -379.9558    -270.1802   -418.85979   -145.90543    263.79387    950.94401    1339.0184   1264.98354    507.90205    -54.14096    -365.5664   -305.08069   -261.14434   -363.73497    -98.88766   -207.56715   -103.60451    -44.09008    344.90728   1255.44136   1308.40636   1255.98701     720.1232    395.74732   -357.32849  -1739.99551  -2888.20943  -2553.00094  -1409.91601   -509.95904   -155.02792   -263.62477   -508.47698    -304.3766    -78.59166   -307.58417   -472.97134   -206.90521    107.15481     82.12271    105.12545    -19.09364   -203.51831   -249.93565    -292.6763   -289.49259    -270.4277   -240.85112    -51.49827     -6.79686     30.84756    -60.02844   -235.62073   -242.77179   -165.58649   -231.39043   -356.66081   -300.36701   -412.90327    -433.0457   -365.48221   -211.39772    -197.6025    -14.79533    167.58507     245.5837    231.40223   -112.80437     78.41883   -193.58693   -176.97193    -88.84533    243.71596    580.04148    647.66631     548.9885    255.12567    122.29738    -38.78718   -240.34431   -214.90098   -233.05161   -156.71041     91.71717    338.55877    968.39386   1407.76725   2094.05859   2213.88373   1641.53907     943.7456    493.93069    258.50116   -140.49296   -258.74185   -248.77127   -120.26777   -175.13496   -438.29563   -315.92569   -352.59556   -409.56793   -275.47416    -379.0264   -410.23555   -366.03818    -89.42357   -119.52585   -109.73545   -153.72941   -196.65616   -173.39018   -154.51529      -9.9018     -3.26161    -73.69002   -119.02337    -209.8775    -690.7343   -453.39694   -341.75975   -415.47899   -275.82267   -200.59698    -68.72785   -138.23679   -189.35411    -230.1398   -287.39352   -333.28302   -324.40303   -370.97684   -152.43494   -398.94877    -444.4595    -298.2553   -358.26924   -411.75664   -373.96711   -189.43448   -128.20441   -180.20409    -57.74593    -78.61929     91.04239        90.03    209.07513     254.2926    188.44531     69.70521     77.24795     73.04362   -167.85114    -60.92655   -173.92721     -7.54937     51.20394    120.64963      46.7286     88.58612     27.91145      -9.8807   -229.70465   -285.07669   -319.44759    -55.27475   -258.58156   -433.51221   -299.47765   -234.23484   -254.79331   -173.78716   -210.89919   -133.18996    -71.24569     96.99412      57.2307    -96.38393     60.73357     114.0024      43.3027      -53.718     84.46076    -37.78534     55.92558    -19.00084    121.80136    -28.01489     62.81814    111.18179   -311.76125     -3.34584   -124.77257   -336.46032   -206.04145   -169.24224   -411.85143   -301.35905   -239.52496   -167.01832    -81.22434   -227.56527    -56.91747    -154.0782     81.96887     48.39938     60.75108    186.98947    263.13685    474.53893    400.62496    703.14108    660.33465    602.56585     533.3424    710.38818    486.75828    391.32665     305.4517    282.96615     27.73349     20.89827    -61.81047    -13.38575    -83.19595   -211.59968   -236.57887   -106.40343     -0.84315    -24.89645     91.47895     34.80522     37.63184     16.98729   -155.92633   -252.27074    -157.9302    -316.2725   -174.28851   -310.97751   -205.07323    -305.5065   -188.23379   -307.56871   -288.33482   -316.87828   -328.41546   -270.82121   -353.56274   -308.08373   -355.40385   -226.42377     -297.509   -327.04498   -168.75952   -126.66782   -268.56242    -167.8417    -88.48304   -157.31589    -47.76493   -120.53556    -83.52103    179.08904    -70.65313      -96.208    -76.15611    -42.49411    -57.28306   -165.71118    -97.08477   -243.20709   -131.78911   -225.11743      -180.43    -238.1143   -151.29147    -229.3728   -160.95127   -148.35012   -290.82987   -104.21029   -116.94522     -82.4702   -202.52588   -265.42929   -162.78012   -200.15707   -111.36159   -175.12572   -239.51986   -261.33208   -236.27398   -117.12268   -426.86019   -410.54272   -187.02766   -377.87694   -225.34099   -192.74363   -254.03082   -515.95238     79.30096    -57.72115      9.09281    188.69625     19.06516   -243.19859   -134.83666    103.32977     62.94786     -81.5456     58.21358     23.64579    156.66177     92.02694   -137.63051   -270.08659     49.84902     53.33697   -206.98189   -324.23017   -268.19815    -90.49596    -65.15028   -250.21576   -210.03698   -278.68003    -88.06513   -194.25808   -196.68446   -406.05924   -130.35319   -315.33846    -78.52329   -433.01194   -274.85813   -225.33902   -165.43489   -321.96833   -303.17044   -319.66968   -236.14082   -677.80436   -506.63235   -529.71675   -507.49359    -315.0226   -242.32806    -314.4409   -338.03886    -522.6216   -295.85781   -275.04069   -495.73246     -2.80995   -345.83666   -500.02111   -537.90921   -373.36117   -224.29005    -249.4207   -342.76936   -626.12728   -608.90365   -492.41971   -424.65941   -431.75132   -670.80243   -467.18445   -558.49824    -554.1967   -507.36323   -509.89708   -348.87706   -573.37577   -652.43275  -1007.91821   -679.20033   -645.92246   -681.33681    -759.8525   -743.71006   -690.44979   -832.73029  -1029.45192   -964.90342   -782.46227   -805.45907   -956.01432  -1105.86215   -983.79077  -1089.60561  -1050.81344  -1151.20044   -896.37227  -1341.26277  -1370.54202  -1012.02067   -843.23395  -1269.88554  -1025.64856  -1441.02157  -1136.60053   -1281.2625  -1038.62313  -1510.35682  -1045.12642  -1176.30681  -1209.05703   -990.01971  -1216.54832    -987.4245  -1442.40465  -1538.15113  -1408.55955  -1227.38133  -1108.69125  -1374.25499  -1556.94342  -1031.18973  -1201.06457  -1364.51695   -1547.4286  -1643.95083   -875.81688  -1561.54058   -881.16458  -1261.36932   -959.67331   -451.29207  -1316.06353   -355.40676   -791.37584   -127.41727   -923.93826          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       17.50   -457.50178   -401.78144   -369.14658   -469.11384   -107.63203   -414.13158   -398.98423      -245.39   -310.76219   -302.94093   -321.17341   -394.80673   -266.56894   -118.70539   -140.96175    -362.4421    -94.12765   -358.79282   -587.56659    -131.0467   -294.47799   -275.96224   -258.64635   -366.21117    -55.60003    483.39519   1220.45018   1709.51347    938.91411    330.04389   -120.52213   -432.26802   -175.68388   -215.86607    -369.8474   -295.19795   -469.83415    -296.4462   -158.09787   -315.89836   -422.26198   -477.03384   -228.75605   -298.25414   -396.60155   -194.88836   -273.56407   -389.60868   -412.28809   -297.85378   -314.91056   -263.38324   -353.37781   -312.49977   -244.13212   -301.72949    256.21808   1131.06914   1197.01476   1127.61414    540.62467   -181.92321   -180.80092   -346.49807   -284.83905   -159.45984   -209.21086    -278.8255   -314.94445    -21.69339    379.65054   1291.24586   1455.91198   1297.22716     871.7887    275.87541   -296.38271  -1784.77802   -2508.0553  -2183.89568  -1297.39826   -603.29709   -329.30291   -208.13747    -256.0586    -285.2322   -204.90468    -223.5711   -299.34032   -227.62637    -91.40372    212.20915    110.00374    100.93337   -245.41891   -302.15356   -301.87253   -419.69653   -281.47556   -263.94347   -137.92726    104.79627    -77.42749    -56.25494   -173.89655   -147.14242    -235.0091   -312.17258   -208.00704   -488.87244   -367.59272   -378.47725   -318.40952   -430.61323   -143.17391   -140.90951    196.45787    219.52537    130.25944    107.53524    155.08565    -90.50581   -188.65098    -17.37499    132.14226    498.17385    897.58737    460.93931    247.43612     41.01092     26.13778     -214.017   -286.92398    -48.51356    -137.9352     18.45078    356.91313    830.93166   1509.38292   1921.61261   2076.34568   1653.83998   1196.35995    446.53272     21.08747   -326.80169   -144.05275   -273.48475   -246.15494   -227.14912   -303.70718   -361.66263   -300.53231    -301.7857    -289.1756   -348.68346   -301.68872   -233.46023   -154.32103    -64.35772     -167.476   -219.23204   -137.30426     22.27053    -21.88755     -82.7963    104.57277     227.6499    -78.21733   -166.04177   -157.92327   -225.02787   -187.05635   -175.49829     -259.838   -410.44623   -173.45909     -1.15869    -76.40245   -121.75845   -166.22172   -313.45526   -185.59089   -326.26581   -440.32881   -392.10361   -359.11669   -330.01862   -277.12072   -289.21341    -58.23107   -188.64947   -248.38647    -48.22226   -151.28344    -73.94984    -96.24356     94.53549    147.03475    140.15925    275.35006     82.65086     19.92399     81.39661    -44.63269     -4.20891    -30.41592    116.72568    -74.61637    119.30798     -46.7028     99.61923    -20.59151    -97.46301    -45.22217   -224.22111   -142.08483   -348.03689   -294.01466   -424.25004   -353.84107   -139.08782   -104.52903   -227.16556   -134.06332    -83.49551    -39.20509    -29.74568     51.93017      98.7738     31.49038    174.51522    -70.50814    140.62492    167.98512    -93.97935     55.16129    -93.72734    -68.21301     89.11537   -125.16193   -159.18659   -113.66956   -124.05102    -48.39488   -165.29797   -212.71327   -235.04328     -120.276   -351.17618   -524.15635   -261.62741    -77.34758   -117.54272    -61.85206     -6.10142    -68.49629     21.90212    -66.11021    374.46983    195.77435    259.98957    486.19457    412.82223    602.85998    809.32409    636.64517    709.74424    508.84314    567.33559    341.88852     48.58172     18.32011     92.41849   -195.47502    -81.31499   -298.14478   -208.84007     33.88766     60.69444      103.901     61.88048   -107.47776     78.48788    -24.80146    -30.79411    112.93574    -36.44168     37.45847   -137.32814   -106.18033    -251.4545   -204.36803    -88.68041   -314.31538   -460.06908   -400.12112   -150.20125   -424.07224   -266.38391   -406.71301   -320.29793   -420.21641   -212.68631   -178.03306   -177.70728   -241.91146   -119.21834   -205.68991    -111.4314    -22.68435   -120.20484   -126.03904     19.00889   -113.88077   -111.53183    -34.98791    -81.35592   -129.14529   -226.31474    -18.55263   -120.12085     -160.737   -373.49588    -58.32888   -190.09553   -205.99032   -239.55813   -231.17349   -291.83953   -209.27461   -248.90032    -166.9899   -107.48018    -37.62218   -101.27957   -192.21741   -161.09738    121.65506    -141.1051   -192.83222   -202.58116   -286.19574   -241.16944   -209.46077   -158.93338   -233.05989   -340.59908    -82.11339   -110.76883   -149.54984    -11.36759    -92.62735     12.99522    -24.61229   -148.05464    -177.3618     37.88168     34.47565     52.40649      5.39032     -66.9505    147.36532    133.85251    -86.30599    -24.79921    -52.22875   -118.01178   -291.28643     -3.22194   -251.92273   -268.77966   -303.90736   -216.26859    -71.15449   -218.30503   -399.36163   -135.33218    -45.55681   -104.83418   -105.69915   -120.21344   -294.45583   -202.67661    -100.3245   -227.61051   -299.89565   -346.79944   -268.31588   -220.98925   -349.96522   -597.23911   -554.38821   -378.98182   -427.52946    -719.4032   -244.97423   -365.27054   -511.21002   -300.96139   -345.03884   -389.14256   -309.41563   -409.79069   -414.89337   -247.34443   -503.43341   -522.78088   -420.08125   -428.07232   -714.35325   -325.66452   -677.55525    -238.6129   -481.88978   -568.59003    -574.0614   -530.83291   -456.42704   -230.70945   -386.15758   -472.02213   -603.15623   -599.79602   -451.60831   -776.21987   -531.81122   -716.94676   -759.27285   -978.27565   -682.86513    -513.3158   -995.64676   -843.44128   -942.27672   -751.37577   -951.96009  -1091.54794   -792.69645   -835.17622   -734.65666   -858.09618  -1483.91287   -848.37628   -770.88727  -1225.42574  -1191.34175  -1076.52824  -1240.30697  -1196.26161   -706.42965  -1397.53664   -696.71383  -1207.79637  -1145.35189  -1237.48705  -1404.80753   -985.59537  -1276.07531   -881.98288  -1402.78461  -1326.64708  -1228.77247  -1301.43873  -1087.80657  -1074.54127  -1538.47313  -1010.51244  -1341.92959  -1376.67686   -742.04406  -1645.14639  -1270.36318   -1167.7609  -1402.01595  -1844.34378  -1239.40398   -334.43786  -1313.40829  -1599.88765  -1623.17518  -1317.17745  -1572.23757   -968.52381   -932.21172  -1880.50411   -1319.5192   -869.20603          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       18.00   -435.64897   -485.53718   -413.05359   -536.72172     25.52504   -391.85282    -346.8012   -156.00478   -451.90535   -295.22269   -310.70074   -360.49432   -272.61379   -328.53177    -16.84459   -322.17623   -305.38552   -389.44511   -339.02281   -220.74637   -251.61511   -406.23747   -315.76822   -316.39485   -140.05083    671.51425    1108.2427   1537.60826   1017.22581    347.00061   -153.49886   -318.36505   -125.01986   -143.28077   -336.03731   -401.14496   -382.94509   -423.19265   -222.00026   -223.76158    -411.3585   -450.17034   -226.21375   -416.82072   -348.96176   -286.60558   -267.42009   -180.01949   -332.75337   -390.65165      -460.98   -433.80121   -390.03076   -382.14028   -295.14128   -193.59483     70.31816    996.72296   1657.82392    989.19715    277.84493    -99.69382   -180.73247   -230.33427   -273.83924   -409.16315   -203.27602    -289.9943   -300.51351     83.50368    341.32774    999.97871   1549.29172   1228.58501    724.89291    334.43158   -364.15002   -945.58967  -1416.65988  -1419.38405  -1088.41556   -516.34242   -231.10029   -203.91846   -381.51495   -447.97026   -387.97738   -333.26302   -329.37356   -144.06557     11.06484    172.36412    118.89685     88.12371    -37.98514     30.37956   -270.46616   -392.78615    -345.0876   -296.80143     51.97698    113.80637    -43.20926    -36.11276   -126.64772   -188.73412   -199.64953   -197.46612   -247.94569   -355.85767   -194.62461   -243.66145   -330.14122   -293.89026   -280.17278    -47.92952    168.89578    253.73491    218.48044    211.33999    -16.26312    -86.05949   -166.75132    129.97247    325.15772    582.44109    623.72588    472.45812    285.07316     93.83082     -37.1995   -268.45275   -213.89091   -252.17842   -241.82637     56.32197    330.73343    918.95099   1677.84287   1975.78999   1848.27749   1447.29051   1063.57037    491.61735    122.09102    -36.06541   -245.80106   -163.25164    -95.52204   -395.76534   -386.31074   -291.48479   -378.37253   -247.33729   -324.00185   -270.60116    -271.1171   -248.57061    -53.79565    -48.20681   -144.85783    -41.59668    -46.54751   -158.24662    -66.38219    -93.59156     18.16807    -58.31089    -22.29739    -270.0028   -320.94253   -264.94213   -334.40256   -354.19171   -355.26695   -105.52868   -319.21374   -276.01762    -29.73357    -173.9344   -385.50983    -326.8628    -244.6228   -595.07265   -334.85116   -447.94203   -380.60894    -281.7886   -425.99029   -369.13801   -399.36368     -193.216   -201.04119   -244.69167   -202.28455   -163.73404   -173.12722    232.50646    312.51021     92.96723    -14.31561    214.80943     67.33204    -34.49459       0.0104    -14.74584     41.14284     139.8379     39.08236     51.81402    119.95275    341.95879     153.7472     35.09471   -117.29897    -328.6578   -199.90899   -463.52646   -381.47003   -332.59303   -371.43969   -151.69139   -219.53321   -307.94506   -246.44717   -255.30821     59.34936    -92.13325     44.62908    120.86168     48.42322     29.68966      7.73567      -4.8027     40.09948    -13.32983     30.64638      0.73564    -25.28821    125.48112      0.46737    -15.24939   -107.89757   -182.73196   -117.56139   -384.16096   -205.91004   -314.48831   -239.15603   -236.78329   -280.31389   -291.18475   -195.19069    -48.94215     31.74633    -89.75955     15.02058     79.06554     63.33016     171.9034    227.24927    368.07088    202.07371    572.47191    404.23085    647.65185    684.05686    739.65718     544.3897    296.58681    269.57891    137.65581     46.67884    -92.69039      3.66668     35.23185   -141.74561    -42.78981    -97.05889    -93.97833   -107.51909    137.76693     33.26833    -39.80701   -127.57348     -0.38251    -71.49523   -179.94244    -117.1674   -154.73226   -229.75064   -464.44684   -431.29152   -376.30461    -328.4307     -170.246   -357.37239   -351.57595   -219.53578   -339.55315   -262.39405   -273.28649   -443.20276   -276.47574   -234.97138   -438.64548   -175.67344   -285.52723   -309.30717    -167.3105   -214.87978    -64.81037     35.72122    115.22223   -176.48633    -25.74258    -51.83998   -280.40228   -109.60684   -111.60623   -121.12439   -208.14521   -162.44314   -109.56458   -280.15809   -208.88709   -250.43182   -174.25355   -162.08328   -231.37647    -344.9878   -306.15145   -250.94927   -195.31153    -90.69787   -191.13292    -140.7723     -4.69445   -303.76023   -251.53988   -164.55345   -248.73312   -402.71852   -296.03042    -191.4951   -358.77298   -131.64064   -338.06656   -262.10905    -56.20037   -216.99807    -94.61353    -127.1242   -140.48753   -194.74524   -142.21894   -227.83295     87.52871    -69.14334     115.8878      1.18746    163.02212    104.08717      24.7879     -7.56012   -192.54309   -171.74277   -163.14342    -44.64621     76.38331    -65.75642   -243.90233   -124.07623   -141.34788    -92.05605   -245.56528   -130.02143   -352.22305   -104.33584   -197.63486   -112.47111   -472.32509   -128.81637   -327.53105    -153.8838   -296.52598   -368.97546   -272.39491   -306.90138   -286.98845   -403.97951    -527.3045   -370.91966   -563.06413   -461.26785   -317.23617   -552.59307   -388.04478   -479.73107   -366.86834   -439.58751   -309.38032   -391.02962   -436.42208   -554.70832   -359.38892   -367.34808   -495.54028   -367.77795   -353.27199   -468.21932   -493.66754   -576.79018   -469.23218   -522.46688   -347.74835   -411.31046   -521.67051   -420.14161   -367.82898   -641.84376   -632.77349   -488.89278   -829.45911    -534.3158   -615.53697   -579.97598   -590.23132   -604.11709   -709.09462   -726.71239   -884.95183    -708.0768  -1084.51115   -986.59586  -1138.18645   -848.45895   -627.41756   -958.99979   -920.36012  -1064.92223  -1144.48579  -1122.30253  -1063.99544   -969.36178  -1534.88179  -1292.58436  -1081.37044  -1254.33137  -1419.48755  -1400.48866  -1207.93269  -1303.21281  -1074.43146  -1461.86622  -1383.15712  -1211.91073  -1045.22798  -1228.58682   -982.73361  -1935.82575  -1308.22371  -1177.33156   -1315.1172  -1137.54131  -1559.26693  -1397.44049  -1351.42371  -1213.36325  -1517.97855  -1397.19126   -1479.6478  -1203.85428  -1276.76324  -1399.47924  -1781.45235  -1207.79737  -2055.78702  -1566.88564   -966.04243  -1335.01366  -1608.77031   -2428.3938  -2214.45017   -1688.5109   -847.44639   -2606.5713  -1337.99435          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       18.50   -405.23121   -465.81893   -498.97499    -553.4844   -171.47247   -344.82313   -295.65974   -168.82166   -626.63018   -302.30212   -328.67281   -352.09476   -411.16342   -457.69244   -149.74723   -294.08073   -406.32093   -221.83823   -300.71773   -327.71762    -322.2681   -470.49721   -386.46318   -161.59883   -164.34483    572.70868   1107.31932   1552.40037   1087.44027    268.81403    -97.07336   -316.59343   -257.13257   -346.40087   -269.40167   -298.48733   -438.01469   -430.49532   -355.71748   -354.18496   -256.84284   -419.09439   -366.30367   -501.29818   -360.40211   -402.75407   -362.05309   -248.92184   -229.18992   -273.46867   -505.87006   -245.00548   -398.38869   -346.96492   -532.95316    -173.3313    338.78392   1040.74445   1363.02494   1209.61899    351.48405    -44.25231   -339.44134   -300.67856   -301.85174   -289.68181   -349.10816   -236.22812    -294.7674    117.05838    443.01039   1149.41116   1365.59522   1194.75389    728.44642     552.4059    316.25463   -167.43435    -477.7572   -554.93882   -665.46141   -433.11947   -328.87025   -144.51421   -357.73041   -318.21295    -456.0085   -186.36999   -405.18777    -57.20179     208.5064    167.24497    231.70923     21.04441    -80.75062   -242.12472   -231.02773   -241.08162   -327.78468   -279.07253    -40.24549    -33.62376    -24.93672    -21.20864    -63.35173   -332.40083   -237.47763   -383.82018   -328.01107   -259.75486   -409.22446   -383.97858    -433.7093   -300.90778   -340.18257   -154.95746     78.98149    293.64507    303.19618    236.86259    -91.00054   -114.85735   -220.44096   -101.09326    306.82329    469.75894    635.70143    535.85782    168.71889     21.41092    -15.21757   -100.73287   -267.17615   -211.35777   -275.10624    175.42396    278.90666    935.75036   1471.60524   2077.35024   1926.91188   1480.11563    965.75337    526.83541     82.56055    -88.09768    -107.2388   -280.42116   -179.19156   -332.91943   -477.55035      -135.93   -305.09833   -259.76307   -261.18163   -263.16697    -98.47189   -152.02034    -103.2178    -47.09426    -97.86749    -59.98871      7.95048   -243.40654    -14.25177    -99.87333    -22.64088      2.15473     12.67841   -145.59657   -178.18254   -124.97152   -300.22252   -312.54909   -261.53209    -239.6693    -66.85677    -24.29413   -220.99182    -87.89232   -343.24839   -245.08013   -358.32838   -311.41666    -260.8368   -284.15555   -278.90335   -438.53458   -369.39803   -254.84815   -206.20703   -150.96777   -152.84052    -48.57477      29.1968   -140.03616    -67.95401     109.7457    182.85974    266.87402     78.09997    -19.48158    130.90958    -89.28616    -27.67056     61.30162     80.78499    -10.08225    139.42268    227.56513    210.48539    115.39872    -239.7544    -19.17972   -158.92009    -96.76505   -194.09596   -284.40987   -313.32217   -290.59198   -411.12366   -268.06407   -448.29936   -208.56696   -211.03457    -57.15898   -189.38239    -11.64601     77.16031    -30.25103     28.66492     98.14214     25.13697     99.79421    -71.12786     96.01516     56.58043     -73.8108     33.08591      65.7809      8.12043    -23.12292      1.70975    -65.89165   -211.65138   -355.75238   -274.76414   -304.80262   -252.28272   -266.64305    -362.0107   -157.22432   -151.57799   -112.27097    -22.21781    -92.46848    -99.80704     34.02663    147.67095     35.94172    349.11663    301.66613    323.31639    655.17274     516.4917     650.8761    646.00979    708.42224    555.57333    292.62289     380.7574     73.38601    162.27962      0.39073    -128.0958   -144.49185   -154.48435    -80.42936      61.0705       1.9003   -154.93629     -1.47851    -30.77753    -91.86351    111.26874     51.00641     69.98801     -1.50059   -219.27599   -210.89754   -353.28792   -133.49935   -261.91977   -250.85507   -505.59627   -440.36248   -239.56022   -300.56072   -533.03493   -257.64654   -347.07888   -331.92827   -375.80522   -217.25681   -169.40728   -308.38824   -113.00441   -300.22628    -148.0923   -193.38413   -108.80223   -284.35739    -42.44258      5.88962    177.28107    -78.01202      -1.6012      2.81946    -56.52931   -169.25413   -158.09978    -74.98561   -250.91401   -170.51939   -292.47917    -198.9454   -184.49209   -293.78635    -172.8864   -200.57894   -344.05529   -215.53471   -281.11714   -199.49689    -184.0386   -128.72329   -288.55767   -317.24331    -97.29644     -15.8893    -75.92915   -121.47037   -140.88978   -201.93996   -204.36558   -138.08355   -290.10873   -184.58757   -348.75092   -368.68275     12.27688    -249.3612    -95.61669   -323.98475   -146.21488    -41.53627   -142.25589   -132.30099     23.76462     -4.91944      46.6729    131.72433      5.83613    -45.83883     -6.56898   -125.08459   -233.13635   -210.56286      -206.73   -256.16193   -208.07265   -451.00204   -349.82352   -269.27353    -268.6092   -453.14204   -336.20589    -213.8392   -380.99421     -196.562     73.55305   -211.71506    -211.4436   -341.12132    -96.18917   -302.14163   -232.52895   -474.38566   -206.79332   -397.07356   -604.89481   -434.81123   -493.36882    -371.6473   -555.69337   -458.22606   -282.62916   -525.37026   -382.29689   -621.98349   -529.21433   -267.77183   -444.84269   -488.54272   -449.97319    -558.4081   -413.50332   -542.55526    -433.3447   -743.67655   -686.63298   -629.91993   -578.50854    -639.1576   -567.39114   -599.30298   -633.93463   -788.36918   -362.41938   -748.39094   -712.98834   -555.89994   -697.25687   -643.61008   -714.21955   -616.06109   -723.55214    -761.5055    -760.4623   -712.63008   -964.82402    -784.1637   -797.38106   -921.66014  -1266.31777  -1035.29357  -1194.10983  -1204.91394   -812.25214  -1087.89017   -1044.7033  -1036.37876  -1156.97716  -1315.96113  -1550.95269  -1196.46476  -1994.15719   -889.41057    -1426.323   -1438.9838  -1128.78729  -1063.16164  -1523.52931  -1314.53538  -1376.09899  -1000.32194  -1738.45785  -1424.11835  -1382.40209  -1549.60989  -1369.50884  -1452.50488  -1939.07577  -1806.91285  -1520.48551  -2072.35247  -1279.87851  -1320.25158  -1504.97843  -1524.41812  -1734.69553  -2190.80889  -1249.94538  -1812.16184  -1878.48284   -1910.0905  -2104.64988  -2266.38919   -2213.0463  -1839.29017  -1792.90255  -1746.78028   -701.71854  -3022.20279  -2917.23571  -2443.11755  -2396.98732  -2404.92699          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       19.00   -324.55064   -417.17139   -544.63837   -564.00693   -321.13948   -277.51649   -294.48981   -150.13454   -490.00568   -358.01715   -318.87405   -425.33418    -532.4289    -457.6317   -261.31268   -370.71614   -319.56463    -237.0793   -429.33649   -373.63714   -350.42793   -314.43981   -427.39201   -186.55414   -205.00318    546.14673   1232.61423   1564.82858    1209.6002    524.70673   -100.82591   -289.39626   -192.40343   -398.52577   -375.13235     -310.845   -519.33902   -254.84966   -302.24174   -387.55992   -679.21108   -429.64952   -510.70015   -349.46461   -443.41083   -368.54245   -359.29029   -324.19441   -332.18622   -478.88299   -522.17007   -296.36649   -361.72519   -341.02501   -216.43326   -101.09969    303.05565     822.6515   1303.10657    947.83834    693.16592      143.534    -356.1701   -237.33836   -295.17917   -369.67534   -395.16276   -156.25267   -339.85864    -72.62917    448.73752   1258.80665   1298.19552   1181.76289    756.05963    495.97289    526.54758    223.49606   -269.71014   -355.59794   -277.29271   -484.22593   -100.51125   -318.69347   -544.43287    -490.1905    -388.6921   -199.57012   -214.02756   -224.11011    -18.18522    175.08148    -31.00522    103.87349     56.77532     -196.108   -265.87855   -361.36435   -319.30164   -327.90814   -330.95621     19.81443     95.62279    -33.11652    -123.3137   -258.68141   -231.73267   -259.32733   -219.44123   -432.49402   -385.23894   -346.89446   -349.87178   -283.35066   -214.16033    -29.20482     28.41664    267.07292    333.07096    172.16782      6.13607    -60.18076   -115.99894    -60.39987    162.53003    439.29666    548.85047    620.26457    292.89758     70.58897   -164.91474    -138.4601   -113.00309   -169.38225   -139.47087     -9.70724    210.11604    945.21644   1395.00248   1798.87953    1960.1414   1713.20145    976.20814    409.38538    168.06324    -58.06135   -153.47224     -149.908   -294.90698   -176.94129   -185.81806   -321.47777   -163.95703   -307.70884   -351.20749   -333.29659   -392.40347   -239.27119    -76.31894    -71.44573     29.09894   -214.27007     -46.7922    -64.44892    -67.91538      14.0022    107.17934   -216.75132     33.52721   -141.57765   -108.34316   -168.81625   -201.61762    -272.2088   -257.93748   -154.25045   -174.07922    -121.2351   -206.70854   -266.91967   -201.27293   -248.68545   -390.55547   -174.16142   -436.25269   -403.44683   -273.42307   -376.63891   -314.54747   -280.22175   -229.43766   -149.40255    -71.26446   -120.10073    -38.76632    -10.59575    101.37543     34.37536    181.25237    174.62923    281.76032    113.54722     27.99929     -22.4013      0.18024    -56.04162   -185.11601     82.24188     70.46146    109.51768     90.93006     62.54011      5.95371   -115.35601   -163.55954   -204.71799   -219.53252   -335.30099   -313.24498    -290.2899   -352.80266   -297.56709   -232.82376    -85.60122   -102.25393   -163.61889    -261.0915    -53.16901     -0.33549     92.93826    157.70304      24.3609     -6.19094     81.64277    -51.37176     29.83165     56.88684     70.27169    -28.47576    -21.48322   -182.04182     56.71416   -184.60494     37.87846   -233.00001    -11.77246   -161.05881   -440.75352   -287.68673   -235.62658   -161.28088    -48.80375    -205.0043   -373.64057     40.40896   -237.65056    -69.76151    -94.95922    -73.93026    269.38129      64.5105    132.38218    367.03927    444.64752    548.74682    704.73259    762.78455    584.68236    655.94203    601.23623    381.56854    170.97602    -65.32512      7.55152     18.87127   -147.26779      5.94067   -225.67595    -99.22649     92.59995     87.24733   -103.63801    164.06427     30.76584     49.60875     11.99595     65.94042    -84.62176    -94.23022   -204.20683   -229.91676   -300.23902   -301.45946   -393.84099   -401.51162   -331.90179   -296.09095    -303.4509   -279.37655   -394.56868   -328.23619   -334.71311   -136.03837   -331.93858   -153.62531   -161.29161   -204.27274    -307.1331   -358.98103    -98.97933   -199.59723   -145.36678   -118.33905    -81.42227   -114.97389     -4.26838    -99.97473   -207.86275    -95.21057   -201.42297   -128.21346   -276.72238   -249.90214   -318.76958   -171.09897     -429.135   -231.48487   -225.61323   -173.96736   -314.44015   -280.70754   -231.06472   -307.65094   -167.00351   -192.90167     -94.3526   -300.86516   -256.13822   -178.52476   -198.38085   -221.25897   -213.23117   -415.47388   -197.12143   -396.69376   -190.57856    -119.0718   -264.73056   -310.61326   -348.96648   -284.48553   -252.75052   -122.91849   -120.68864    -52.27985   -125.31491     20.11747    -67.09859    -21.17985   -178.85424    -56.60911    -69.11226      4.29433     -15.0794   -220.48423     25.72303   -112.90977   -186.96661   -335.03181   -447.01123   -186.93597   -299.16641   -360.88322   -307.77216   -267.81213   -594.98415    -196.4535   -215.19073   -348.22192   -483.16812   -386.39704   -256.87805   -400.79791   -162.57618   -211.73914   -345.83517   -183.53233    -413.0935   -481.09896   -566.38077    -501.4274   -468.50886   -496.63778   -550.99422   -350.37813   -699.64889   -490.25887   -693.28733   -487.99679   -442.39686   -458.68658   -553.69609   -408.66993   -530.49954   -393.08671   -733.43548   -562.68779   -630.27399   -608.06951   -727.13626    -846.1488     -521.357   -548.03925   -550.54263   -662.21769   -630.18806   -811.30126   -391.16097    -734.9719   -585.64274   -695.83413   -712.26866   -480.39089   -724.02802   -736.29373   -546.04254    -790.4198   -703.33681   -1003.9715   -811.13802   -783.28653   -1052.3533   -982.43154   -839.22196   -1100.3001  -1268.50379  -1194.86998  -1015.61281  -1075.88609  -1181.25028  -1312.58148  -1334.62001  -1117.65879  -1382.30236  -1439.35781  -1626.38445  -1363.47367  -1461.84515  -1158.88797  -1720.22525  -1083.66919  -1069.75069  -1352.98126  -1412.89953  -1308.94914   -1593.9786  -1588.52837  -1441.34646  -1711.53896  -1463.38017  -1569.08183  -1540.96424  -1510.55041  -1814.96837  -1448.14053  -1958.76458  -1963.54244  -1309.96727   -1524.4926  -1769.39351   -1817.0071  -2099.16691   -2162.9684  -1769.07252  -1736.13748  -1972.66831  -2668.33179  -2150.43999  -2018.47684  -1861.87026  -2079.18923  -2332.54587  -2232.40299  -2427.18547  -2142.79072   -2131.9795  -2206.59017  -1992.45271          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       19.50   -233.31481   -357.77987   -563.15245    -565.5985   -342.92013   -261.75059   -279.99456   -147.95931   -289.00533    -424.4618   -229.94402   -477.40882   -478.67201   -426.00998   -325.18137   -236.45443   -366.43003   -458.23646   -487.31514   -378.98977   -330.12993   -346.93892   -364.34593    -95.54659    -310.1601    478.15998   1333.95257   1592.40806   1307.78667    358.06196   -278.63859   -276.46403   -344.42955    -305.3298    -415.2153   -316.69876   -567.36223    -149.8778   -430.58595   -419.55233   -250.65346   -409.71641   -367.87671   -288.09396   -355.22927   -389.04276   -424.24581   -362.34188   -348.76267   -352.80795   -317.08216   -438.19383   -210.48795   -513.89816    -85.70822   -258.66156    397.12913    940.98777   1264.44352    969.83399      342.002    210.94136   -387.00889   -428.37151   -344.87716   -263.45686   -228.39147   -399.41121   -361.06792   -237.45263    623.27909   1088.83028    1357.2991   1303.65731    789.69716    429.84123    257.83764     84.95025    101.04302   -324.59867   -297.77111    -205.8364   -427.99023   -389.55911   -382.39586   -321.40164   -287.97106   -532.89379   -133.09886    -33.90198    -54.25791     49.77404     39.38476    -52.79691    -51.69575   -306.42256   -345.64661   -449.51504   -257.53353   -127.95934   -166.73239       5.7175     58.48217   -135.49766    -56.81047   -356.61262   -357.87199   -306.23465    -382.1537   -260.88333   -326.60437    -337.4928   -356.88182   -467.33735    -311.4261     58.51382    117.77421    335.75716    225.27976      71.5961     26.06787    -79.95954   -152.67868     22.16626    204.21304    337.83403    650.83262     412.9155    377.04114    105.59464     -1.89621   -250.09024   -106.98441   -269.43017   -106.48948    -84.80132    230.55834    851.42309    1683.2764   1999.79051   2081.43557   1597.89831   1117.77519     532.3824     67.79043    -70.05548      1.17911   -228.52661   -374.60874   -300.83213   -342.15705   -292.37946   -288.05904   -412.51433   -401.34051   -141.32152   -145.89919    -32.61882    -92.94055   -189.61858   -162.45954   -169.67373   -127.60986   -250.19595     -78.1094    -60.03203    -56.64903    -189.3466     34.22149    175.14677    -27.91144   -106.26903   -207.95943   -332.04447    -188.8422     34.47397   -292.40201   -167.01006   -179.17883   -146.88075   -339.89302    -329.7965   -284.42256   -249.99914    -384.2936   -410.87897   -215.71982   -330.04498   -328.50137   -307.73944   -361.83154    -259.2236   -159.80377   -191.51335     11.47111    -26.21548    -46.73678     67.16165     158.6234    205.65747    242.95866    279.76421     -58.5283    -26.74628    -11.59237    -32.70234    -43.15943    -78.72461     29.96839     28.85349     49.88693      16.1682   -103.19196    -58.74648    -91.49683   -268.62726   -343.59673   -339.33763   -329.56801   -389.35623   -485.68969   -256.18561    -258.3699   -115.57041   -196.46081    -10.87984   -141.91323     57.45074    -12.49342     61.34002     23.90727     71.01661    -23.42994    -45.35407     40.01434      3.33636      9.92855    -65.69532      0.26362     18.28819    -63.44273    -20.87688    -23.68974     -88.9452   -273.34639   -176.91964   -215.53159   -417.48191    -77.61892   -159.48718   -412.41259   -268.80682   -158.39321    -32.69115   -103.72994    -75.54102     78.36861     67.19392     71.53402    138.23141    129.73152    320.65803    422.48987      456.449    595.16865    629.19991     695.4821    610.47632    352.44319    559.01495    200.55365    128.70562    -22.15596    -76.55633    -53.68316   -174.57751    -57.27746    -77.68341   -104.28229    -96.86948      6.85835   -104.97634    -45.44873    -35.11487    -86.46963     76.58896    -75.77596   -234.93372    -79.45838   -225.57198   -238.00996   -399.54832   -289.48984   -242.94826   -304.24637   -310.11315   -229.79589   -449.93205    -355.8443   -260.73885   -327.96588   -162.68942   -441.92266   -295.62984   -494.42374    -202.3179   -302.69015   -318.10666   -198.96916   -169.56171   -174.97396   -225.30244   -172.57285   -160.02192     -5.62455    -58.17377    -14.54112   -313.22519   -140.93813   -188.59473   -144.00033   -290.17508   -372.67953   -305.70804   -206.08606   -309.02718   -212.27116    -281.7951   -234.33415    -343.5804    -354.8804   -275.38748   -158.38275   -218.53601   -309.72174   -184.61557   -150.54775    -54.75417   -380.22749   -332.19488   -326.03092     -386.241   -290.26442   -393.03983   -250.97719   -399.02376   -396.26291   -504.40399   -470.61546   -437.79966   -363.76412   -219.40005   -399.19832   -237.94448    -81.09887   -150.33408   -157.66027     73.49773    -94.42835    -62.60519   -150.33525   -202.01967     47.36714   -123.31571    -259.1554     -37.7458   -295.98661   -307.61188    -313.1398   -373.79971   -299.44571   -220.90979   -353.90683   -301.95646   -284.66034   -167.76786    -42.35746   -304.94421   -478.93721   -525.33039   -509.80424   -356.56273   -435.12602   -492.53228   -306.86556   -631.31285   -320.37896   -611.67292   -542.38897   -517.16355   -510.10205   -508.41782   -377.40499   -632.41578   -709.45807   -565.06493   -467.06198   -632.01238   -566.14561   -560.44682   -547.90214   -629.57415   -684.67073   -781.99168   -970.36392   -671.54797   -652.10219   -629.63465   -451.28467   -999.85782    -605.8788    -535.5588   -670.55572   -887.58035   -834.41238   -879.93248   -664.21081   -847.75602   -783.86243   -719.86558  -1022.44476   -638.02294  -1015.25359   -888.69084   -592.28826   -850.96338     -683.261  -1038.26267  -1040.89758   -973.84506   -768.89305  -1197.40072  -1238.05166  -1332.77522  -1056.80967  -1006.49033  -1228.52794  -1541.85326  -1252.23424  -1326.38668  -1410.84594  -1406.95864   -1362.8305  -1281.45409   -989.15125  -1235.38214  -1352.37187  -1032.60228  -1675.32344  -1103.80618  -1640.37971  -1349.28324  -1780.37301  -1259.28715  -1800.48389  -1458.09964   -1538.4535   -1545.2636  -1464.76675  -1899.38066  -2016.61165  -1946.68743  -2024.83052  -1867.06348  -1739.72541  -1475.70227  -2188.01362  -1911.95503  -2031.88453  -1508.90558  -2168.33355   -1991.2079  -1705.61226  -2388.23396  -2170.88789  -1852.89158  -2328.21793  -2198.07485  -2549.68061  -1610.23181  -2474.94686  -2652.92625  -3021.87172  -2517.27855  -1729.00958  -2764.81333  -1733.39192          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       20.00   -139.52484   -320.97632   -485.08183   -539.59005   -288.39084   -261.66177   -195.94446   -198.28154   -241.53667   -461.77318     -95.7862   -480.03565   -393.95416   -335.71718    -319.1626    -21.26205    -423.3614   -568.47819   -522.38708   -354.12671   -376.86422   -369.98123   -533.33574   -151.71697   -350.41147    460.71367   1349.24868   1647.36347   1303.45687    217.49262   -318.04894   -197.09985   -392.64463   -211.96014   -197.43065    -361.0187   -364.18252   -235.82606   -281.90755   -294.35601   -387.98783   -427.48013   -352.14563   -254.81261   -370.19499   -475.24985   -245.98957   -158.61462   -308.28568   -374.23477   -486.46412   -362.66148   -265.93155   -377.29236   -219.12018   -189.54086    234.89116    862.82827   1165.66589   1203.49062    352.31297     81.92759   -248.80893   -233.22389   -331.79321   -220.17024   -229.61452   -297.24462    -279.2871    -21.96853    482.53837   1125.98264   1480.67625   1180.77975    624.88301    739.43439    527.30149    297.57865     -3.07548    -39.31692   -351.93661    -347.6956   -203.26126   -313.36561   -437.52899   -263.02146    -323.5564   -283.34896   -159.31775   -162.20353     40.14341    288.74076     66.16896     -57.8099   -174.83607   -208.61376   -239.07014   -389.88962   -245.06995   -237.38501   -186.67445    -19.26862     48.61357    -16.12748    -31.36621   -111.42834   -256.85864   -343.27002   -225.45664   -262.75994   -301.20036   -319.34392   -436.20664   -385.89839   -188.54081    -28.79626      111.145    303.54771    195.96881    143.86871      -84.717   -180.04607    -65.58625     29.61868    316.72822    558.66984    762.26525    548.29465    281.01472    110.57898    -50.21042   -226.56959   -319.52857   -111.85428   -184.76163    -35.98965     133.7365    735.13503   1673.98364   1931.32033   1808.83205   1650.23524   1032.00113    557.26487    257.39802     21.91596    -39.52779   -231.97048   -208.17987   -168.55531   -306.52234   -329.36281   -323.16572   -408.14499    -175.7357   -226.52584   -186.36482   -237.51346   -142.61105   -181.68287   -176.85421   -127.20331   -137.16135   -153.25934    -136.3612     47.32956    121.19451   -119.86077      6.16006    -47.29746   -161.31734   -336.75337   -213.07502    -318.1522   -229.29889   -169.08973   -157.80332   -112.22289   -124.92622   -260.24978    -163.4645   -193.27036   -389.91417   -407.26364   -271.39261   -273.50357   -402.50482   -373.65419   -285.11977   -167.66742    -284.2157   -211.73953   -137.84235     -64.9832    121.99285     24.61194      9.84771     96.43816    157.28454     230.4576    183.38774    202.49506    178.07449     113.6197     25.36905   -133.00626     -11.3935    -52.56677     37.08915     -3.79793    101.79952    116.00842     17.27049     11.98029   -181.58255   -138.70356   -360.87305    -190.0134   -335.63115     -297.942   -370.20062   -225.72272   -258.94869   -209.75552    -82.28826    -75.82878    -13.63761    -13.67723   -155.76217     38.75119    114.10703     68.21075     29.23966     98.25525     15.01116     34.70649    141.59541      -3.7196      5.62233     36.84053     23.51613    -18.41822   -213.21692   -158.63581   -317.08777   -244.65433   -262.30065   -315.76724    -98.11117   -326.07848   -489.20033   -273.16737   -229.84592   -272.09419    -92.76057   -172.27939    -46.69974     43.86211     98.02566    113.25654    188.57024    225.57633     402.2864      473.801    613.72625    568.90355    723.44633    468.13601    451.12328    268.90133    144.80319    115.38225    118.44558     83.69574    -71.13615   -271.82957   -303.59595    -187.4327    -22.51674   -191.17234    -34.61958    -44.35771     44.55993    154.63025    110.08508    -96.79455   -112.23087     -66.5494   -153.05081   -254.03212   -141.91483   -264.98546   -404.72618   -279.76223   -419.12636   -241.33151   -211.57589   -348.12468   -344.93657   -468.33106   -381.75518   -325.09181   -473.88251   -275.84813   -380.52866   -277.51483   -201.37852     -204.268   -330.74691   -276.97624   -127.32153    -85.36875     93.72073     -63.2935    -101.1046    -200.0278   -134.47112   -108.27372   -201.04728   -228.06604   -108.95436   -270.18049   -249.22229   -206.09983   -405.16141    -248.4053    -348.8108    -239.1039   -273.09866    -185.8391   -329.06929   -277.61677   -261.34249   -167.27336   -157.42464   -381.71193   -263.81692   -308.48389   -267.28822   -314.22333   -333.35865   -244.02689   -287.35034   -295.01245   -312.84339   -340.41603   -292.93393   -541.33851   -393.15555   -320.89394   -373.58439   -398.37177   -321.83587   -272.70489   -193.18218   -192.85719   -416.89157   -245.62971   -261.22861   -188.97397   -283.30394   -163.35095   -198.91621   -152.34308   -350.35092     27.51743   -359.12094   -261.09917   -241.83661   -294.50382   -467.27963   -448.27994   -588.16883   -287.35497   -429.42568   -394.98832    -514.3968   -448.37698   -480.95492   -274.05295   -415.89898   -629.20117   -587.44614   -535.79968    -623.6378   -739.59126     -611.566   -528.92225   -672.63992   -716.53395   -722.96143    -594.7346   -771.02413   -661.10946   -856.74831   -893.57464   -608.87789   -991.85436   -873.63326   -917.24424   -904.37534    -917.6518   -843.02996   -902.37204   -798.23795   -750.16518  -1105.62685   -699.42825  -1027.68957  -1062.11378          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan  -1161.45218  -1723.87002  -1911.92323  -1681.39304   -1538.2081  -1669.79182  -1671.34373  -1856.15198  -1885.97981   -1888.9598  -1739.32562  -1511.37608  -1981.97059   -2066.8543  -2139.33311  -1775.17988  -1977.05455  -1833.95177   -1761.9569  -1924.39334   -2614.3609  -2540.59136  -1965.10879  -2295.57409  -1967.68196     -2146.26  -2238.08383  -2988.25057  -2469.45497  -2538.24554  -2887.54901  -1014.62609  -4911.40308  -3218.55076  -3463.71949  -2920.33121          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       20.50   -204.83781   -298.55893   -433.58279   -449.46269   -212.68286   -202.83895   -198.22713   -214.01896   -267.85931   -402.59872   -169.49116   -403.31106   -457.54699    -310.8253   -302.81464   -112.08456   -297.40338   -510.23639   -525.23276   -434.53266   -330.37105   -485.33018   -367.95182   -197.32985   -322.38957    405.88443   1314.82991   1591.69216   1058.24485    239.63043   -217.20031   -332.73746   -419.86035   -346.85801     -9.24819   -408.75694   -315.78308   -386.59915   -430.57105   -212.14944   -311.73966   -411.07175   -403.89665   -447.56088   -395.05914   -434.71563   -341.39329   -727.65459   -333.20485   -453.56267   -351.66035   -330.03684    -298.5442    -423.0018   -463.70616   -109.74434    265.05374    740.78005   1455.38185   1015.03705     553.5431     -0.42894   -160.94325   -411.65763   -315.48591   -291.28322   -246.35636    -234.4272    -158.8793     -9.02626    490.91271   1155.74158   1355.60098   1160.01659    669.55747     750.3868    117.03604     86.41732      66.9175   -212.64455   -455.93619    -321.4195   -357.96642   -226.15372   -169.29559   -214.81583   -272.52556   -349.07032   -406.98913    -14.37828       -5.435    199.77032    145.17798   -100.57632   -147.25143   -312.10099   -207.75373   -282.16213    -295.5166   -349.98289   -173.65645     -9.21292    -54.85777    -71.09849   -150.79919    -291.5516    -305.0707   -200.99164   -425.77873   -316.80788   -355.66564   -328.40772   -332.06646   -181.74119   -266.20209    -18.06616    349.28171    146.61266    333.19963      96.5982    -93.21942    -56.53094   -206.33472    -43.14157    250.88557    456.55964    544.17063    434.60063    478.55177     37.12597   -177.64101   -169.07596   -212.55303   -268.35916   -384.88705   -107.96393    392.09766     958.1116   1525.87884   1957.12966    2067.6783   1568.90181   1114.60172    633.03658    158.90471    -88.09413   -167.95591    -202.2511   -109.26446   -292.96826   -351.04653   -285.70624   -196.33298   -331.98136   -152.16151   -151.24397   -261.87735   -206.32555   -244.49649    -84.42734   -173.55688    -273.7541    -51.42164   -160.36694    -95.09854     46.01081     49.98925   -133.68515    -40.88672   -197.98026   -179.39793     -138.228   -146.66586   -222.15051   -306.79081   -411.62969   -221.89564   -127.65739   -305.50448   -261.99339   -542.27375    -241.4176   -267.62383   -338.81774   -236.05919   -271.66786   -263.42022   -286.35751   -323.14019    -334.8753    -213.4002   -260.99465   -194.17194     -8.89767   -161.20867    -64.97902      16.0503      41.5125    205.16407     137.9264    139.80307    140.02011     54.08303     39.47097   -148.69632    -68.26076     30.19371     57.17864     50.89946    -18.91958     55.04618     -88.1916   -111.10258    -150.9479   -127.44425   -283.41947   -220.94674   -354.51076   -319.86982   -270.54624   -407.88232   -253.33039   -221.95352     -274.922   -265.65291   -141.34808    -32.83326      22.4571    102.12574    290.40149     70.44317     -72.1451     22.28874     78.19888     60.97735    131.92651     40.39889    -68.44257     -14.8821      45.8044     82.86723   -148.30828      59.1062   -400.50169   -146.37031   -278.71368    -400.7957   -446.06534   -338.27035   -299.03327   -181.18759   -272.09899    -198.6461    -27.61571   -131.41583   -104.23883   -104.61027    -38.12118     17.64396    157.05607    123.32684    346.87966    468.85382    562.13529    438.39242    396.91636    414.25166    472.93613    315.67402    373.12921    244.36567    145.62533     -95.8158    -12.50437    -44.59141   -213.01829   -162.53265     -8.44942    -73.03314     -88.8971     93.53198     58.82158    -285.6841     -1.51055    -22.15797     52.12312    -10.38552   -152.65142   -176.21453   -158.90373   -239.77026   -264.66378   -195.88842   -371.60884   -476.01175   -327.59276   -416.40311    -372.4834   -392.60909   -278.87903   -316.30462   -313.39304    -363.8768   -352.84899   -240.21946   -287.00809   -242.16344   -213.25415   -135.54994   -144.71351   -289.67203    -95.79632   -114.88546   -214.09116   -111.53836   -153.94676   -119.50446   -233.15016   -237.43996    -222.7179   -210.27953    -336.3498   -404.95255   -262.52099   -256.98808   -376.29318   -339.42297   -362.55359   -395.47436   -258.10956   -279.22978   -532.75245   -276.21968   -306.78343   -391.10031    -276.7048   -306.08935    -301.4868   -303.21897   -417.17369   -442.70444   -357.63508   -360.55053   -366.69773   -439.72064    -338.3466   -439.75443   -576.79337   -408.49473   -553.56547   -550.46235   -480.00085   -282.77238   -405.98204   -409.24665    -438.6146   -388.19921   -540.85997   -387.11007   -658.29837   -555.88615   -379.81425   -464.70693   -372.15009   -625.04087          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       21.00   -336.71864   -309.29902   -443.99262   -444.86954   -130.45387   -165.97041   -258.99773   -185.60558   -276.42675   -383.26396    -302.5021   -363.31251   -514.35391   -326.47387   -438.16326   -292.19037   -215.40829   -434.79795   -449.74062   -465.92825   -390.18652   -492.65069   -329.03018    -176.9054     -17.1587    336.72631   1261.37687   1493.20684   1052.53952     228.1213    -49.22977    -478.6062   -419.12483   -413.41283   -283.90123   -443.53027   -408.62167    -322.3325   -351.90145   -348.50487   -323.20355   -291.89943    -431.4141   -467.07375   -309.88314   -469.11162    -403.6166   -499.74896    -387.2288   -419.99912   -306.75773   -333.83277   -129.29693    -302.8985    -212.8689    -71.11758    242.50193     1049.944   1215.93997   1154.21185    268.86508   -192.32623   -241.81462   -315.05986    -341.3365   -324.51542   -418.13371   -237.00134   -212.44207    -30.67934    458.40371    937.34539   1554.82628   1233.44475    867.06865    493.70298     483.5638    201.25423    -46.63639   -211.54871   -334.72814   -180.47981   -219.80114   -211.08395    -369.9854   -371.35291    -337.8687   -316.39137   -235.91503     -9.58685    121.77959    214.69368    128.38668    230.90337   -216.90435   -242.20004   -273.43325   -347.76367   -310.10592   -321.69463   -209.90761    -14.38359    124.78762    -45.31204   -201.56934   -314.02124   -281.69838   -413.67977   -420.95981   -410.37978   -266.81126   -337.89804   -422.05744   -302.88684   -210.32661    -95.28727    146.98458    332.04573    179.55241    119.47757    -73.78782   -215.14438   -150.90187    179.85662    233.13658    613.47513    674.44269    461.58078     395.1383     81.29352   -195.93039   -143.18699   -225.73486   -315.47011   -162.63399     42.30696    287.07689    874.45283   1575.44592   2012.20313   1842.41911   1740.86603   1193.90207    458.10009    125.30547    -159.0686    -230.0625   -187.11586   -173.58266   -280.51796   -307.80234    -240.4019   -194.87421   -309.40779   -332.64137   -123.49697    -78.61349   -155.44505    -205.7792    112.61474   -170.35389    -60.32847   -127.41974    -74.38512   -211.24228    -48.65328    -42.29456    -57.40897   -115.15041    -57.19535   -221.16019   -404.84503    -342.6567   -233.44173   -260.27346   -164.06943   -116.31186   -214.97492    184.60368    -244.2707   -244.93013   -154.58919   -253.29554   -396.53571   -281.99128   -411.40463   -307.44099   -226.38041   -341.96647   -372.95092   -292.34558   -245.20388   -183.88231   -169.89444    -56.57831   -113.23941     87.57139    147.84537    120.96546    361.65524    199.72718     75.39278    118.05192     60.12304      3.82769    -62.49539    -82.11977     31.41192     74.36107    119.74113    170.76764    162.29555    -30.85521    -70.75779   -269.88222   -111.08882   -273.05825   -252.46153   -385.55766   -263.25206   -261.87568   -171.29521   -313.53057   -190.25057     -96.1742   -104.26852    -75.93703    -107.4897    -10.97128     67.34801    -14.38446     13.76274     53.64007     95.50054      2.72687      9.53319    187.41439      1.06743    141.36617   -145.30579    -65.06902    -24.74982      4.70637    -74.34925   -192.50061   -181.64919   -205.68209   -374.41575     -283.846   -282.01066   -261.83355   -322.57458   -305.18914   -147.64666   -173.70703   -114.39549   -123.01678     -61.2413    114.19418    143.23925     238.5375    198.22387    412.29553    421.18347    309.46828     644.7307    590.41736    508.10228     450.6572    419.02669    150.80937    202.36291    138.70869     -43.0669      0.37129   -101.21802   -231.02204   -382.32659    -161.6326    -73.58037    -43.45447     34.77342     52.80289   -165.47042     90.12939   -100.29513    -98.13498   -173.39065   -211.29222   -143.56605   -261.98373     -307.248   -228.29925   -445.52298   -351.47666   -347.23841   -455.32047   -320.41635   -333.33326   -397.91904   -430.46538   -287.22328   -179.99714   -306.79664   -298.61753   -363.02767   -291.40988   -284.66461   -357.97719   -191.17687   -136.27936   -193.41727   -371.64714   -194.42769   -203.03124   -218.15265   -153.68582    -339.8169    -257.7333    -268.8954   -283.68663    -316.8722   -347.34053   -396.31586   -226.42888   -353.65469   -458.47952   -469.29259   -501.60367   -441.52674   -383.67805   -406.84105   -414.03178     -652.461   -398.22592   -500.30291   -491.63251   -489.80996   -561.86693   -492.33605   -585.11206          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       21.50    -470.9339   -354.08983   -465.37028   -523.16672    -31.53556   -239.50973   -244.51764    -192.3311   -258.98751   -247.84108   -314.14001   -364.23953   -254.71987   -316.66606   -420.09423   -486.99413   -290.15209   -335.26161   -343.76598    -212.8404   -291.69265   -360.08084   -265.54352   -257.66191     24.44715    253.53574   1259.77166   1492.84552    1201.7375    283.24451    -25.91082   -268.67485   -431.32366   -441.58445   -435.59411    -381.7351   -360.11968   -377.25778   -308.71501   -281.40051   -401.48033   -199.85732   -407.42621   -407.56397   -256.11883   -455.90812   -381.39282   -328.84014   -324.30694   -264.61849   -349.44377   -365.59427   -272.63927    -385.3276   -350.21769   -117.21526    377.96179    899.43526   1254.15693   1027.18819    554.91124    -78.42514   -210.63296   -330.14364   -299.15211   -219.01586   -389.33644   -186.19674   -389.17236    -14.86485    523.78557    837.60692   1244.05379   1235.12176    682.97865    551.26983    618.10503    224.47254     19.78376   -299.79965   -263.87741   -294.77226   -410.66847   -336.02364   -277.63235   -351.48507   -276.41238   -341.27218   -232.01369   -119.52333     64.55207    173.51501    257.40412      93.2658   -217.98625   -444.76295    -377.0684   -319.82204   -221.03175   -221.98498   -227.63423     137.1361     39.72984    -23.66472      66.6636   -225.30667   -326.13976    -431.5355   -406.90922   -264.62126   -286.42865   -320.55235   -213.69266   -320.96214   -138.46742   -133.36704     32.11284    369.13254    243.02541    109.48262    -127.7692   -135.75718   -110.55921    134.39179    354.72971    581.06804    804.59608     618.8228    455.80266     206.8251      8.21164   -185.88392   -131.42156   -249.21773    -266.3857   -139.90829    401.97676    671.88066   1388.14538   1994.22595   1889.09731    1676.0098    943.82703    548.82592     94.77278   -170.15191    -82.51654    -172.4913   -306.61296   -273.53044   -292.14861   -322.39442   -225.44844   -387.30174   -246.98865   -497.58362   -340.38386    -91.79522   -150.76681   -210.92883   -232.97945   -108.18488   -192.94808   -144.47907    -88.30251    -74.91705     40.99956     73.36828   -153.88156   -181.71844   -142.25792   -123.92835   -262.51933   -216.47685   -117.14571    -274.9608    -236.2342   -140.10803   -280.03315   -134.99388   -300.53491   -291.09738   -329.89879   -447.70613   -345.78165   -264.30171   -299.14905   -254.34481   -371.17876   -404.43463   -280.63412   -342.03336   -217.12102    -80.01244      9.90417    -84.90599    -16.06206     97.43874    176.74446     288.7855      87.1848    126.67572     167.3788     44.19535    -41.11897   -109.24584     48.02671     -0.52385     60.39832     16.91826    141.23018    -67.86308   -181.59643     44.48731    -80.69854   -131.17669    -268.6401   -366.26464   -371.97601   -267.59079    -329.3643   -298.90864   -279.29773   -239.06395   -199.60352   -104.17691   -128.79296    -41.95038    -26.40606    -44.76209      99.8386     16.36549     28.85993     32.96469   -108.42712    -12.19662     73.06322     35.31145    125.83322    -92.36329      9.80662      -0.1891   -192.59833   -190.86842   -127.77211   -328.33828   -169.80067   -204.53687   -382.77708   -465.56092   -227.73793   -271.25074   -260.70119    -40.79092   -122.70437     -34.9925      48.0441    -50.87778   -129.84631     79.61201    116.99803    331.77441    502.45363     449.7371     486.9049    619.75974    514.11958    552.56022    371.60478     352.6633    162.05811      98.6617     13.78467    -75.85751   -189.31647   -227.42568    -118.6306   -241.91364   -228.49269    -117.7645    -13.58258   -122.23861   -104.42368    -95.56201    -78.67323    -22.60802    -52.86345   -197.83143   -234.05641   -251.82213   -159.27307   -368.64328   -359.30991   -356.99431   -388.43577   -262.59758   -433.25891   -355.62182   -327.26542   -318.15147   -396.90303   -525.15281   -555.96314   -401.31562   -312.21994    -320.6649   -214.17461   -400.96681     -341.992   -404.58666   -294.72888   -228.80409   -342.09794   -182.49979   -292.59259   -249.25307    -99.12964   -399.36364   -384.37504   -389.63186   -395.32304   -448.96809   -500.47233          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       22.00    -503.2854   -430.96022   -444.49335   -577.05178   -121.30512   -215.27815    -210.8969   -313.48695   -234.41423    -38.19573    -318.8925   -399.00838   -241.87661   -419.43085   -422.69916   -358.08262   -299.96901   -191.68652   -362.50378   -106.21849   -324.17617   -200.34189   -279.59604   -179.22841    -25.80903    462.54671   1304.70293   1533.90799     1243.432    225.02309   -129.24086   -180.16193    -230.4972   -327.67425   -376.57186    -424.4532   -407.77079   -353.89178   -307.52702   -249.35729   -393.47375   -260.63217    -376.9994   -374.50109    -441.2452   -420.67483   -423.10532   -220.79476   -339.84242   -338.10783   -476.67395   -316.05292   -554.78558    -310.2875   -539.09207   -199.91259    374.12402     972.9081   1274.12799    1207.6657    408.48813   -340.88307   -287.62464   -330.96416   -330.93667   -148.57118   -257.92199   -250.05986   -347.14693     77.88049    454.05684    986.24712   1324.55497   1208.40842    909.24888    735.10251    397.08444    128.77734    -74.86263   -205.12458   -385.69523   -202.10118     -16.9407   -271.24602   -211.66287   -213.04495   -460.92239   -260.35443   -359.88273     -25.7855    211.15588    196.91029    128.97704      2.09272     16.66942   -257.33795   -344.53526   -345.73672   -548.33424   -302.93731   -134.11283    -11.96751     91.21505      9.67732    -77.92084   -321.23803   -339.60056   -417.39521   -224.55532   -125.44355   -405.00094   -339.61158    -331.2789    -175.9249   -252.11691   -122.91096    177.09217    305.23389    174.52665    140.48099     15.36229   -154.92291    -68.64126    145.50708    361.48164    608.81573    712.02983    469.95284    337.34093     -0.75454     -48.8394   -205.60638   -204.31729   -221.20771   -158.99342    -58.06533    392.10549    811.56173   1286.01414   1748.97249   1851.57009   1373.93913    887.22452    437.86092    155.73853   -109.82651   -156.62176   -341.99926   -204.86914    -216.4602    -357.9271   -151.18289   -238.46512   -330.30166   -327.74361   -200.12449   -195.82851   -177.57373     55.91821    -87.32113      1.58924    -79.90289   -301.25139     63.30048   -157.34994    -15.77182     19.25936      7.02885    -26.14375     35.07808   -104.75916   -146.42003   -293.60841    -61.61469     -9.37415   -228.32937   -252.12329   -135.24957    -30.79499   -104.03897   -162.56481    -229.1272   -166.71188   -286.45097   -113.67583   -329.21158   -217.60354   -295.93453   -270.84163   -393.16474   -362.61192    -280.9035    -219.1555    -17.33312     -79.7605   -126.51935    -45.63941     65.50207    144.28199    253.01511    251.65503    245.13681    -25.13415    -37.20428   -112.30239    -96.90483   -267.51825    -28.56109     90.41503    147.12315     74.82667    -17.92959     -7.08288   -127.32815   -165.27289   -326.31199   -144.81832   -227.38416   -331.24489   -315.88638     -328.448   -252.37918   -265.18097   -354.87845    -32.09154    -56.83889    -117.6467    -58.19003     74.76203    -45.17638     95.97388    -30.04771     -10.6209    -19.88824     34.87282     10.03713    -19.21631    -37.12693    -56.05697    -35.61301     46.26191   -162.57809   -236.88151    -202.7858   -106.67195   -254.32365   -280.44833   -343.03835   -342.96011   -236.78175    -289.2209   -189.02499   -189.13536   -245.14985   -241.49234   -122.48144    -49.02391    126.51303    -58.63886      0.12265    187.30865    101.77206    256.08423     337.4628    250.41138    509.83911    452.29283     538.5864    401.23488    400.98199    150.74071     -8.61968     73.84001    -77.97738    -64.85405   -177.51142   -269.17993    -82.85763    -97.78967   -218.75625   -113.24053     -1.46799    -76.97707   -207.99608   -179.72419    -72.70985    -53.90094   -172.65656   -240.15963   -477.05112   -453.57015   -275.59766   -264.49438   -323.04822   -517.69079   -421.90845   -394.37928   -314.31703   -437.06117   -510.38891   -337.20079   -322.28196   -471.09165   -335.00369    -365.1008   -502.71633   -434.97628   -446.35829          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       22.50   -490.05054   -414.96879   -388.89986   -653.70045   -234.42348   -306.46737   -253.35831   -418.63751   -244.91416   -200.67978    -383.6369   -407.29782   -418.61541    -461.9998   -388.05701   -109.07615   -306.49769   -169.78573   -245.27184   -125.00486   -333.86335    -57.58954   -342.22842   -208.27872     -7.61277    630.93182   1215.95759   1468.88371   1232.00378    238.03946   -266.14004   -227.85381    -230.3773   -401.08976   -408.37432   -552.31182   -415.62189    -295.2816   -373.72119   -369.24363   -483.49797   -251.26933   -154.07596   -455.32754   -395.77891   -275.65647   -266.41921   -326.23759   -376.39934    -548.8615   -450.73273    -492.0239   -518.69915   -353.57523   -456.19699    -28.43707    109.09842   1129.62812   1133.85811   1062.00419    298.55884    -85.80212   -210.27998   -314.97803   -198.63061   -236.66824   -350.50672   -218.02638     -243.202    109.37289     420.4326   1011.33759   1722.83898   1384.92655     846.0561    591.05063    487.39145    178.46383    -37.87226   -151.51105   -390.33645   -210.45809   -163.70911   -331.03532   -218.80128   -407.09302   -177.79536   -300.17699   -171.63079    -53.87249    100.32877    272.93702     90.97102    -11.31897   -157.14566   -364.06814   -358.85161   -420.42583   -192.81607    -16.74442   -133.19407     116.9532     82.95786     39.28156    -36.90094   -265.21223   -466.48492   -419.36595   -602.02848   -270.51499   -400.54515   -374.43269   -303.22574   -240.69751    -255.9637    -55.20331     56.03135    322.33226    381.45647     28.75681    239.72287   -128.74031   -115.50635     97.30836    302.99854    659.39456    801.32529    775.80589    474.69908    134.98568    -81.74515   -154.32918   -359.26729   -231.15652   -144.04354     -26.4707     434.3609    850.10536   1460.95475   1812.72312   1698.97144   1378.44641    931.39086    217.14369     69.22058     -93.4701     11.08509   -136.41123    -97.01245   -217.62475   -330.31548   -277.01741   -203.83652   -401.97173   -316.82182   -425.15821     -195.386    -175.3405   -132.78979   -143.22321     49.61287   -127.30124   -187.97644    -45.15458    -44.67965    -12.27579    125.54933     86.63601     -94.4056    -51.50897   -184.46754   -384.10235   -488.69673   -337.26423   -142.62662   -224.30263    -93.66099    108.53519   -105.76992   -187.25461   -115.52474   -177.65251   -281.10396   -385.44042   -230.82927   -339.76192   -433.58688   -430.93986   -330.37105    -388.2617    -288.1749   -139.88862    -87.66191    -128.6188    -31.24705    -66.20277     38.77452    125.15434     80.85156     193.2633    199.06933    213.50967    143.71487    -10.71361   -212.77562    -15.40822    -87.83687      15.6374    -59.43912     108.6695     15.11664    -29.34561    -40.07261    -92.67302   -127.99991   -188.50526   -429.90609   -322.81796   -265.01232   -224.81587   -263.70789   -195.58237   -278.32056   -172.90272   -355.74258   -143.76278   -137.76479     -45.4232   -121.06814    -64.94473     31.17317      0.69429     23.59452     65.18211    -84.14777      7.01994     81.68867    -42.18208    -62.19307    100.58331   -185.27809    -59.25168    -47.13714   -211.15961    -320.6478   -231.82314   -403.11881   -301.57314   -347.44862   -396.93293   -330.93601   -260.33565   -267.59139   -168.08233   -133.16634    -37.82376   -164.71184     -23.4579    -71.51546     -8.99109    161.89513     89.07739    137.49445    292.00085    413.42313    361.40278    392.73427    442.41616    566.22344    284.51364    307.12475     82.66778      20.7407     -6.07311   -118.20973    -56.03633   -242.23955   -194.33544    -76.81469   -108.00218   -105.23644   -106.94833   -173.42313    -38.36832   -121.55196    -11.05695    -91.63684   -386.03626   -268.72736   -376.93349   -526.65637   -427.27583   -376.33752   -283.74887   -419.57305   -413.08815          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       23.00   -489.62176   -357.05669    -363.2809   -597.89214   -280.91942   -478.73232    -422.4749   -344.48695   -237.22914   -369.74945   -471.42762    -335.7558   -378.40319   -404.02138   -284.66305   -107.17285   -403.82459   -146.85604     -70.9683   -317.08025   -403.64132    -57.71589   -342.79033   -309.19602    -16.58867    580.18477   1144.01359   1333.02538   1423.78676    258.59878   -157.55948   -252.62258   -354.05299   -534.05735    -329.9587   -539.22515   -291.06652    -474.1317   -326.33291   -443.91196   -484.74727   -294.73468   -260.60813   -285.79951    -453.6102   -484.41343   -225.57864   -184.98661   -265.77293   -293.50196   -299.33965   -261.97663   -302.52142    -193.5051   -395.78886   -207.90949    156.74852    955.89049   1193.69214    950.98863    530.49537     17.19005   -246.87767   -181.72034   -405.77142   -347.10726   -339.36261   -337.21165   -179.50001    -72.39511    556.75477   1319.12711   1479.56557   1124.36235   1019.01086    800.89805    648.94488    166.89899    116.74774   -236.58869   -297.85074   -377.15498   -294.18671   -235.55715   -292.08807   -475.94396   -549.96861   -349.21925   -291.64602    -94.89324    111.13323    274.49657     275.1889    -55.83088    -281.4794   -191.36111   -356.09824   -110.32006   -176.50409   -301.18183   -179.29239   -157.12177    -17.44341   -195.45409   -117.39587   -209.44531   -219.39186   -259.64313   -493.00144    -374.5033   -306.07317   -261.56601   -310.74505   -167.83274     -236.015   -108.71191     206.9082    227.54844    220.48165      75.1277   -122.07813    -237.0213   -260.45712    -15.49431    320.46703    393.48165    614.76273    587.77158    363.96593     -9.48405   -139.95348   -234.25701   -405.20181   -271.70034    -160.5433     40.32743    300.90286    778.56765   1424.84203   1759.64419    1878.9617   1410.59917    974.39448    437.37607    184.24775    -52.49439   -130.01095   -236.19309   -230.41796   -108.00592   -342.59266   -285.36055   -286.53461   -382.99368   -359.14216   -164.03103   -226.64064   -224.87711     10.33755     -25.0548    -317.5748   -197.01209   -133.92296   -240.18863     12.39998    -152.8084    117.19112     27.19071     50.32116   -128.06755   -227.54837   -457.43162   -286.98346   -233.22135   -133.51118   -205.77192   -263.20369   -160.49677   -202.42798     -5.03168   -198.43896   -351.48587   -304.10932    -267.3886   -368.58773   -337.06112   -314.99308   -496.30648   -328.22159   -154.08134   -221.63978   -197.87372   -145.41589      8.75397     -4.45084     -1.03636      0.81858    -23.18903    221.94134     72.74224    138.25613    150.63206    126.80755      6.76825    -10.56444    -86.02733   -223.41957    -36.71921    -84.84699     92.14673    -16.40273     99.83426     56.71851    -51.56473    -31.10903   -211.75847   -255.53719    -282.1556    -193.0049   -366.88712   -384.00418   -306.80904   -181.37928   -134.71019    -214.6042   -113.19122   -191.96641   -156.35951      -1.0368     12.11621     28.59556      7.93689    -17.85337    -27.27513    -97.27455   -186.25304     22.88053     71.01243    -14.20303     84.15803     -56.4177   -122.02522   -115.74108   -226.39063   -283.83263   -218.68516   -177.69672   -388.09319   -312.89684   -393.76313   -323.42623   -313.14725   -256.91172   -207.32146   -213.73746   -146.94424   -253.25863   -351.18895     -0.65232     34.79571    -71.56403     415.6149    292.32952    452.64138    416.42898    226.55976    428.83626    401.57548    427.52155    284.02009    102.46723     10.73068    -85.79933   -137.99686   -179.69355   -337.17403   -361.76795    -258.1524   -229.83057   -215.00733   -204.75065   -168.25194   -248.05473   -325.78987   -295.13359          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       23.50   -522.96843    -254.0591    -321.1383   -406.92187   -331.72697   -601.11764   -585.65463   -312.02499   -204.29739    -451.2246   -479.93909    -262.4365   -205.83959   -145.12455   -277.14244   -225.44177   -436.44266     11.11572   -136.02548   -314.64763   -357.46021   -158.22121   -297.43281   -304.55225     106.5334     630.8835   1060.80641   1401.59795   1201.91624    452.97502    -14.44035    -81.19019   -412.20303   -412.62189    -296.5324   -504.82225   -384.05456   -388.98813   -476.79019    -534.5222   -279.65851    -378.2616   -326.65067   -303.63277   -454.76515   -323.36555   -285.02103   -256.27155   -377.29264   -312.24302   -313.06574   -318.68355   -358.75637   -401.98023   -249.01091   -165.76895    245.81669    844.55248   1261.27626    772.14255     378.2607    -12.71913   -369.02236   -478.80173   -384.12396   -183.78421   -406.24447   -275.43354   -224.94249    -95.91571    557.99226   1124.95711   1455.34561   1093.79055    748.75842    528.58124    240.25992    398.32932     82.85945   -224.92722    -207.4541   -470.60485   -286.17721   -168.94734   -175.57787   -261.87152   -174.76005   -286.11415   -256.17912   -154.82897     34.48848     377.1852    153.07947   -190.02709   -305.50819   -286.73805   -351.66718   -321.77743   -389.64206   -146.53018    -242.1252    -43.27638     -82.1379    -202.0119    -282.9076   -250.63959   -238.39073   -269.45383   -290.24512   -346.91253   -427.63829    -417.6331   -408.43099   -330.00011   -250.86336     36.18873    134.28388    241.85284    177.11654     68.74376    -29.72714     16.58794    -12.47434     -2.56756    260.11965    519.13348    719.99038     501.3011    349.91345    100.85462    -81.99646   -217.75398   -153.91361   -206.79779   -130.00798   -111.59028    346.34533    778.57651    1566.2549   1885.63857   2010.95081   1455.12526    994.45926    439.00813     96.39141   -134.54675   -154.56781   -149.27983   -223.58159   -147.75187   -240.63982   -286.84075   -400.93184   -392.31751   -339.55805   -343.65879   -226.38586   -146.59192   -170.13411    -55.92502   -108.99558   -183.12469    -258.4187    -22.57336   -161.33947     14.83073    -53.81812    -70.44196    -67.95595   -170.19541   -228.62609   -161.71629   -225.31911   -277.74966   -270.35487   -178.50326     27.80176     37.09115   -159.07333    109.82115   -321.26819   -322.72174   -286.06147    -316.7267   -291.24201   -210.16604   -421.66215   -287.52222   -282.43064   -266.95086   -392.21087   -222.59773    -135.4761    -325.2786     -93.1364   -114.39207    -28.72987    -37.33279    135.94865    236.40515     30.27119      51.7974     64.56298     25.41851    -50.56099    -97.90447    -54.91551    -15.22124     94.23621     47.06387    -34.89228    155.88983     12.42725    -78.87702   -234.51735   -239.49843   -155.88277   -355.77239   -294.60098   -255.66717   -269.82356   -240.36571   -255.36563   -252.91905    -328.1807   -184.73238    -46.53711   -111.80702    -19.82359     -35.1108      1.84567    -12.32794    -61.81824    -76.05312     97.63959   -136.86094    -70.83635     49.28806      89.4864     21.47424   -166.75613    -66.96629   -115.64574   -137.74532   -185.35504   -281.14343   -189.43677   -395.61491   -397.89279   -274.99922   -237.49307   -231.93344   -255.99369   -321.02581    -201.7319   -157.49475   -170.87989    -99.81875      2.78928     53.97707     29.07916    111.45473    112.11619    226.53229    215.69134    305.58942    288.09559    263.39156    210.85717    116.18839     44.34145    -95.46393   -139.84014   -249.39442   -269.32023          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       24.00   -516.20076   -208.10773   -298.28617   -207.50141   -418.73315   -576.38785   -410.73768   -465.50363   -232.08763   -531.93776   -434.10667   -564.42732   -168.98998    -49.57623   -327.42615   -395.15775   -346.71402    -90.45013   -202.26666   -220.62782    -422.8225   -207.73562   -412.67789   -380.49396     59.89112     603.1394   1173.97852   1453.97273   1097.51974    411.09345    -364.3872   -186.81516   -476.74036   -218.71306   -196.47202   -377.56745   -267.44034   -485.93331   -444.49861   -368.58023     -506.458   -269.34219   -259.53825   -373.03766   -195.07906    -541.4485   -409.70625   -173.23235   -346.43225   -227.73895   -307.58349   -263.62831   -343.54282   -396.34892   -350.32141     87.16651     272.6394    950.22434   1347.20901    925.91433    465.85919    -220.6545   -361.76828   -235.39517   -361.22757    -459.1656   -274.64021   -430.34689   -158.98417    148.56139    486.57987   1309.04878   1500.21562   1012.45008    732.56278     467.7924    486.00628     96.93421    -71.53615   -108.85026   -132.14073   -336.09393   -248.82868    -98.35098   -389.46545     -310.468   -284.45868   -261.51403   -204.82115   -381.00566    172.52178    171.21119    145.96753     43.18864   -136.09582   -188.12414   -174.00276   -367.12608   -328.65681   -102.59164    -12.38891    -11.51016     -36.8658   -134.30624    -70.75025   -315.80282   -328.49115   -279.03863   -317.28794   -327.14782   -332.55379   -433.82058   -447.85219   -446.75016   -222.35892     -24.3104    114.80088    259.40327    250.04157    144.53273    -77.41601    -14.31919   -129.56337     113.9589    195.38402    467.46299    643.57271    576.90924    316.86063    191.46101   -219.20487   -130.25201   -258.66868    -216.2439   -328.38838     34.39201    141.60153    775.43945   1525.36297   1911.81864   1927.70092   1515.30281   1018.13765    508.20251     97.70226   -138.47111    -97.76833     -177.913   -104.46165   -110.24576   -303.64797   -354.94556   -391.10567   -329.45227   -265.93445   -282.78008   -305.16219   -265.94017   -172.82146    -155.4095   -176.25806     18.37593   -161.60046   -173.26774   -114.02441   -136.24449    130.90696    106.98252    127.06924    -75.58639    -93.62678    -59.86165   -201.91548   -186.11554   -136.86381   -128.43269   -120.53155      7.60802     -126.471      13.0151   -388.02973    -240.0334    -43.22477    -357.3699   -238.54845   -405.03727   -317.37453   -346.42885   -369.63986   -256.32547   -296.31422   -240.00092   -187.11237   -226.18697    -22.18325    -14.92483     66.65109     85.51859    195.06194     83.02243    108.03769    240.72897     17.43276    -65.38756   -139.38736    -126.1493   -189.33545     -1.64369    -84.65418    -53.53013    -34.89889    -10.23134     -2.05466    -62.84462    -84.70452   -239.14259   -165.07404   -362.55966   -313.14449   -243.07458   -223.03318     -344.314   -285.88909   -204.39327   -178.66528   -266.82759    -68.93154   -105.34952   -120.41792    -24.11499    -45.82896     -0.39325    -54.78582     -16.3787    -46.54333      5.80179   -159.69625     98.50005     17.05475     95.29178    -14.26118    135.90478   -126.92819   -263.53644   -163.76171   -120.56241   -287.45186   -584.10603   -400.55134   -343.16553    -364.1412   -267.18424   -251.25498   -240.93596   -152.77549   -155.47959   -219.28269    -69.29953   -126.64068   -148.58514     18.01953   -113.50235    177.68726     30.19723    -24.86578   -180.49379          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       24.50   -541.32071   -229.48567   -366.87575   -247.95835   -443.09113   -437.74713   -317.38269   -560.12185    -347.1641   -472.61972   -394.00178   -767.54158   -247.07666   -138.57795   -366.46113   -425.33434   -291.11258   -331.06634   -328.16036   -257.43984   -319.68156     -233.884   -565.79133   -403.78969   -149.44511     626.5437   1212.24241   1479.12743   1077.62007    369.58731   -195.47338   -235.77526   -497.47287   -372.10771   -487.44697   -346.19124   -366.00608   -380.40158   -616.76161   -361.10271   -521.11308   -337.87172   -317.02067   -348.69912   -302.61111   -311.23965   -600.19478   -280.84782   -305.28966   -206.97914   -297.59895   -214.43244    -241.3327   -375.36824   -353.66999   -179.11177     91.07901   1019.70402   1380.03715     985.3951    476.59935   -137.52588   -387.83312   -157.65609   -309.92965   -338.83581   -347.33265   -208.56993    -248.7726   -105.28329    722.12094   1057.33467   1443.93508   1196.13452    734.16502    447.31388    112.83847    497.63731    -86.19987    -12.86631   -213.34574   -306.49588   -329.58815   -163.81428   -324.43892   -351.57982   -391.84761   -371.65379   -211.10107    -73.03856    290.52534    112.26918    233.85363     53.03752   -177.58522    -299.0723   -390.12202   -180.06401   -160.19226   -261.29728    -26.24201    -92.86804       77.794    -55.99704    -22.52779   -217.63914   -194.51565   -122.57645   -228.50388   -325.10983   -355.84226   -297.51062    -413.3532   -298.18048   -232.59859   -222.46283    231.16841    355.58398     204.3633     10.16643      70.6993   -135.94751   -269.32282     -72.0028    182.69169     633.4966    579.05126    399.46587    337.01123    142.72777     19.05634   -177.12251   -218.97247   -272.92286   -150.74631    -40.18402    149.77375    747.32439   1427.27499   1826.50947   2183.31933   1559.36523   1058.53928    478.61048    143.74164    -74.75088     -5.38086    -83.16301   -164.98828   -240.34991   -252.57267   -299.36963   -337.79241   -322.39292   -264.70754   -258.07516   -296.58343   -272.68232   -230.58875   -117.32085   -152.44123   -156.60511   -128.57998   -123.47246    -92.51598     -73.1294     20.55472     22.89547    -14.46721   -155.86905   -252.74119     -9.67935   -145.00201   -356.24656   -172.69103   -198.76476    -201.4633   -268.67807    -76.75965   -283.72159   -174.19167    -274.5295    -211.7076   -451.89032   -366.30395    -350.3615   -336.67453   -288.75142    -291.4059   -284.28236   -336.11331   -221.86892   -109.89152    -165.1283     -5.19213    -86.57791    -19.49136    153.01629    -47.89209    145.76637    157.29305     89.93667    139.14517    -47.69711     91.67134    -42.13698     69.70466     20.73392     -33.5384    157.99316     74.25541     17.34484      60.9517    -57.74723    -98.84676    -64.02445   -257.95049   -465.21815   -390.66232   -357.14835   -340.73975   -266.70673   -223.32971   -228.58576   -238.58893   -144.06569   -154.45305    -92.34314     -4.28679    -57.94216     35.17701   -125.28733   -148.44886    -75.55066    -15.25279     -73.5153    -11.19702    -34.10334      72.2421    -37.66957    -64.86414     21.75222   -177.38382   -245.49978   -175.03456   -481.35285   -265.01636     -292.015   -336.49721   -205.77798   -370.12999   -189.00063   -284.79136   -330.07374   -372.54679   -368.81851   -259.10759          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       25.00     -451.832   -318.01998   -496.14478    -270.2146   -326.85503   -243.43838   -313.20395    -432.6053   -490.60661   -346.90443   -370.79295   -452.52242   -325.41498   -172.69453   -386.76881   -382.30091   -385.16772   -341.06528   -325.58344   -327.96418   -228.17042   -202.48292      -305.29   -241.84645    -93.73671     606.8091   1042.84612   1302.51732   1133.21042    409.77122   -236.62994   -266.35168   -351.06206   -489.34846   -351.22556   -183.40232   -253.28305   -285.98183   -348.68115   -269.78966   -485.54042   -324.02952   -447.10906   -379.77962   -342.29253   -480.46535   -353.33214     -285.021   -513.60139   -198.19688   -379.72509   -240.94554   -165.40522   -373.54011   -180.70002   -132.90038    308.29719    839.67983   1137.10529    794.83883    612.71696    -71.33348   -394.18256   -214.36098   -380.96756   -350.16651   -218.39245   -182.95051   -123.91381   -129.96721    457.71579   1216.38758   1560.74482   1088.46347    798.28407    479.02436     451.4792     83.85155     37.72925   -255.64626   -331.34461   -265.33697   -288.74113   -177.05845   -386.73527   -299.73392   -201.67554   -256.71456   -219.53813    -56.90822     182.9147    327.89051    157.09829   -224.05969   -132.64717     -193.937   -347.38896   -299.56323   -210.46822   -192.41785     21.38005     -7.24641    -12.71844   -178.94605   -124.35653     -316.983    -235.6399   -401.58708   -421.11199    -382.2269   -323.87221   -245.72584   -331.05292   -481.85594   -301.11075     75.95961    210.62406    193.89329    318.63445     66.90362    -21.70604    -102.0056     -14.0645     27.61536    236.00132    683.35932    342.87435    521.66081    429.21906    187.85587    -84.13296   -249.62999    -97.62307     48.71761   -295.10548    -58.61228    393.14769    691.01259   1445.25805   1820.15748   1858.53157   1522.32953   1031.96075    486.57493    149.75995    -42.74865     10.48474   -303.61552   -177.03092   -254.31074   -234.73389   -290.27616   -296.70728   -390.70808   -454.56877   -289.32289    -219.6239   -230.33842   -129.11957   -214.69919    -70.12255    -167.0233   -171.66965   -109.50234     11.55617    -35.74224     28.64592     66.39038      6.68666    -53.52766    -57.78868   -179.00477   -310.15814    -180.9354   -197.22895   -271.33753   -247.19061   -120.86173   -210.49753   -274.01777   -126.65534   -234.28153   -248.07619   -376.67462   -282.05721   -347.42129   -296.05805   -269.99027   -283.75703   -282.09898   -269.91291   -236.00429    -32.91345   -103.85316     -5.42933    -70.33764    157.69189     -50.5369     20.57035    274.20207    229.89484    176.21121     -8.31011   -197.90388    -74.73033    -22.50315     63.73144     59.94423     81.35608    162.37024     11.99118     65.52967    -50.35241    -36.03124   -442.40449   -229.97373    -31.54118   -291.88369   -425.78079   -520.27748   -343.46452   -320.26438   -240.16222    -202.6888   -290.07149    -290.5429   -246.35191   -136.02986    -25.31857    -92.66908    -80.27954    108.04558    -37.86394   -180.25292    -83.08074    -31.51091   -105.71078   -128.38429      35.2644   -149.98619   -170.43345   -232.36897    -26.58803   -287.18506   -202.75038   -355.66552   -229.96847   -399.34554   -533.63341   -456.53653          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       25.50   -208.48811    -366.7266   -496.06621   -301.96733   -201.65261   -133.37656   -296.30278   -253.79308   -306.41733   -326.74773    -409.8205   -220.28853   -299.67945   -468.86961   -509.02919   -413.29357   -395.15158   -287.43196    -80.78718   -289.72021   -388.92968   -315.95176   -237.02917   -250.23358     -70.4961    504.89044   1238.99219    1305.7939   1124.76868    391.84088   -264.88243   -344.79004   -180.82378   -366.04624   -321.97646   -265.68189   -264.39161   -248.59267   -286.65139   -513.42884   -162.27415   -124.50347   -352.32066   -307.40804   -442.52953    -606.4572   -368.42441   -272.35998   -297.59089   -339.37024   -489.67062   -287.88669   -638.10721    -205.2713   -287.35536   -152.22382    349.53219   1043.16351   1041.43704    878.15289    263.58912    -50.36331   -185.34241   -324.32717   -278.13147   -319.52236    -334.9279   -302.19227    -374.4693     -64.7377    514.26512   1045.69758   1421.25547   1073.38517    760.28023    738.07043    251.67533   -110.17713   -242.57647   -137.04717   -162.55409   -192.79834    -291.0343     -186.089   -264.54471   -269.72796   -351.28225   -439.10741    -44.35769   -299.87284    152.12329    328.16108     180.4843   -112.80035   -302.04665    -94.98582    -321.4493   -349.32974   -254.60622   -257.89334   -160.07194    -28.44366   -152.47834    -15.38113    -99.33705   -373.96901   -278.96629   -344.59281   -441.74645   -351.57199   -283.66079   -385.68833   -250.93348   -215.06438   -179.08268    -33.13285     70.91653    114.38272    195.10605     79.71619    -47.38237   -166.79646     -114.955     25.36579    295.31909       474.14    533.19977    427.66139    466.00015     73.63898     12.58662   -157.93253   -217.74283   -214.51349    -62.24219   -108.34538     298.7736    893.20012   1251.19931    1825.4756   1720.13712   1630.35732    961.17337    484.98415    153.86146   -179.77618   -210.09819   -123.43549   -157.10222   -263.05583   -276.70998   -367.86444   -422.04869   -304.98746   -251.88291   -247.78115   -265.58752   -129.06796   -120.40104   -181.22021   -141.72641   -219.80623   -123.37846    -85.78527      0.12378     -18.3836     10.27613    -103.3074     27.87651    -59.69467   -208.16885   -237.71461   -229.64555   -415.34856    -95.29849     -126.244   -180.06872    -57.69286   -130.29027   -154.94853    -70.45458   -141.01134   -307.64606   -403.92907   -315.31497    -227.7409   -275.52416   -309.92786    -300.0027   -277.03382   -261.69661   -273.78157   -268.41993    -69.08483     24.81471   -128.68981     15.55178     32.24625    118.00351    228.64439     91.57899    167.19833     -7.26384      -3.1977    -24.51784    -79.64651     49.21255     17.00482     191.6238    -77.08784     38.09572    -72.10974   -183.69191    106.35314   -158.57821   -302.08722   -437.87088   -439.60665   -158.87544   -512.72123   -334.65477   -284.36247   -227.54034   -396.64803   -168.40959   -271.39931   -117.65713   -131.20351    -73.18214    -60.20356    -38.66283     97.18219     -62.2627    -46.14519   -161.26163    -70.88761   -117.85083    -99.94461   -264.64944   -202.66226     -88.7072   -321.01927   -511.14769          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       26.00     15.77075   -394.33756   -284.75289   -331.53943   -200.24993   -195.71383   -244.49057    -229.9747     -62.4232   -340.75343    -373.8484   -229.86121   -316.66658    -523.2747   -485.72464   -512.38138   -321.41079   -280.98614   -126.13688   -193.87721   -460.21702   -305.85941   -213.78509    -354.7024     51.85018    602.42763   1244.12638     1295.725   1027.65941    335.92357   -188.12302   -233.02913   -294.53818   -270.87149   -349.46227   -347.61614   -266.86528   -325.34259   -254.07947   -401.94641   -331.09766   -241.05864   -385.21431   -354.43003   -360.80534   -367.51747   -351.84962    -494.6803   -312.74352    -302.5242   -229.85727   -309.92309   -506.41713   -175.19542   -157.79896    -58.92459     308.9558    977.60235   1331.95677    675.53408    315.78601    -63.48305   -310.05172   -396.71219   -336.11546   -326.24143   -228.99871   -206.47142   -269.89482    -50.30403    519.49524   1225.06672   1392.47934   1232.19965    845.01309    520.44529    431.04672    157.27697    -144.0896   -315.58397   -236.89712   -218.49149   -331.40671   -417.62209   -288.27675   -459.35396   -272.93284   -220.73895    -79.74236    139.82564    221.50793    295.70871      2.12694    -16.87417   -121.57544   -327.66768   -475.91588   -252.78571   -205.96028    -285.3048    -95.57882    -19.78172    -49.12913     29.24089   -293.16844   -158.07575   -110.44769   -356.59666   -338.71608   -437.08756   -357.80382   -258.17959   -383.59312   -277.14712   -215.07942   -107.71357    185.00763    119.41366    174.30673      40.0778     29.28766   -142.35135   -132.65658     25.14086    378.80009    397.91785    415.12605    555.45262     235.7335      86.8304    -57.77207   -119.31732   -229.39594   -136.07754   -449.66241     15.19674    319.96584    751.62198   1301.98788   1735.95696   1914.92195   1634.22114   1177.14648    606.94706      93.5509     52.74617   -275.13692    -97.55607   -198.40117   -223.50884   -246.25027   -301.23978   -207.40721   -162.71336   -241.54241   -177.51397   -293.14263    -98.52876   -137.47211    -94.60201     -24.0658   -156.79538   -259.27603   -158.23228   -165.67725     81.58301   -108.26595     -36.0883     13.17494    -44.47411     25.26733   -249.61101   -222.84772   -444.18814   -287.39365   -251.50982    -79.06201   -150.56597   -112.05704     -24.3203    -58.50878   -223.38928   -353.15909    -104.7226   -333.45958   -358.84743   -264.35723   -262.72806   -385.04443   -252.81015   -414.37061   -175.27711   -181.46423    -19.40205   -266.66832     -173.314     59.84799    119.13649     17.20778     50.41708     245.1061     73.66084      5.63797    -28.79728    -90.57306   -108.96922   -112.10647    -30.73752    -43.60619     68.18322     70.82109    -75.69272    -67.87125   -197.73304    -28.72507   -269.44738   -368.17364   -282.20917   -287.33885   -162.94388   -343.28194   -352.22968   -266.01158   -382.47258   -421.65337    -161.1728   -224.99459    -260.4968   -125.65183   -157.71219   -105.48884    -75.03853    -15.63263   -263.03809   -193.00754   -314.04893          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       26.50     126.2402   -359.62829   -126.40085   -371.81211   -235.49413   -338.49079    -289.8439   -319.64055     -4.95507   -344.44613   -364.06953   -251.66673   -364.24652   -403.99874   -260.46863   -500.81182   -287.63736   -215.47176   -280.71997    -68.55996   -363.86561     -76.8981   -155.29847   -360.27583    -13.17164    588.27214   1236.18906   1330.96572    966.81666    150.16829    -232.6183    -90.66663    -298.8157   -233.06252   -452.28041   -372.14599   -296.29826   -535.73166    -375.7003   -370.29754   -524.73828   -358.60659   -340.11733   -391.29997   -433.82007   -335.40999   -299.38941   -282.56041   -366.61954   -255.27483   -411.55848   -333.11892   -277.70082   -415.01056   -195.07858   -173.30058    248.23608    738.05074   1196.43315    835.36603    417.27422    -52.50184   -437.77465   -245.92429   -316.29437   -123.35598   -322.61532   -271.42141   -172.99905     49.96134    721.71317   1094.40623   1249.51449   1042.05859     662.1262    477.66995    452.90981    214.24155     68.84362   -119.29254   -374.99205   -359.32179   -304.83545   -484.58485    -383.5043    -366.6495   -309.49852   -376.38082   -224.56389    -61.09953    138.49305    380.60101    106.44629     -4.54876   -181.06702   -248.15938   -299.40291   -348.00209   -307.90641   -153.16537   -231.54138    -54.44371     40.15408    -10.94162    -136.7836   -214.01252    -342.3336   -250.34418   -236.51379   -346.32696   -347.71073   -349.09318   -443.35242   -242.75687   -261.95846   -125.39516     -0.41154    214.92333      241.306    100.44108   -132.32483   -104.00244   -183.71516     48.67954     96.96434    441.43915    626.88101    540.54604    353.31941    -33.56566   -108.17354   -239.99046   -326.33972   -234.80184   -233.34559     36.39717    243.21348    778.80106   1209.76155   1607.57854   1814.54637   1458.20046   1174.78131    491.20877    145.98744      1.76551   -151.69555   -192.53266     -61.8182    -65.37194   -216.75898   -206.98295   -454.48711   -337.00831   -258.27387   -314.36626   -194.82142   -157.95891   -204.05849   -110.28339   -316.24249    -49.40939    -93.43875   -171.93818   -210.04797     108.6009    -90.11376    155.86729     66.80882   -121.95545   -225.09805    -286.7984   -287.51136   -228.89548   -222.59092   -318.75037   -263.62567   -237.25825   -182.11341    -171.4812   -254.75313   -165.12164     -208.636   -182.09349   -363.74767   -254.51473   -365.90722   -517.46094    -291.6828   -219.37089   -377.39617   -205.61369   -317.75817    -134.8364    -49.05442   -152.42727     -52.8878     12.13482    102.09649    125.06892    169.80201    103.72267    -20.95071     92.96732    -78.43649     -6.92528    -82.62321   -118.81291    -91.03535    -81.46949     27.05544    -93.90762    -98.11474   -180.80065   -278.39057   -283.57038   -188.43648   -293.58265    -367.8711   -316.65397     -497.579   -396.03834   -329.96172   -372.43497   -411.87348   -271.36662    -166.4207    -246.4804   -421.98332   -239.72879          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       27.00    -28.65061   -316.53076   -190.19515   -383.72302   -251.83452   -418.89288   -357.70792   -383.23128     68.35076   -229.60021   -447.65291   -115.60833   -325.63617    -446.6034   -109.31411   -430.69614   -155.67195   -137.06262   -297.73652   -411.26811   -320.68049   -175.88983   -194.90607   -216.46749    -72.90842    524.42906   1093.94247     1355.385   1079.53698    215.34837   -171.56878    -199.2487   -222.75037   -196.03218   -417.79132   -310.76608   -414.37331    -272.0522   -164.85234   -386.01427   -470.97752   -367.64545   -333.07924   -375.10306   -351.21317   -470.79038   -365.49973   -445.51381   -412.30254   -229.56024   -255.41529   -343.32626    -343.3478   -320.11176   -476.81767   -151.44243    230.97958    858.81855   1014.64459    918.94681    300.99101      -58.834   -342.32709   -493.93514   -365.12089   -258.25263   -394.80905   -310.30999    -57.75794    111.68224     672.4786    1125.2883   1416.78523   1028.84801    372.16668    537.71228    498.65698     342.1131     38.07137     35.52983   -139.56084   -494.03324   -368.83554   -511.76494   -265.50784   -296.01514   -134.70248   -296.33795   -193.06728     -176.362    -18.70489    216.85289    207.70969    -59.03427   -286.75955   -269.12243   -546.68313   -377.51041   -186.92506     -193.371     10.75286    -62.65732     14.24761   -262.74008    -69.62219   -223.94692   -309.94023   -242.91044   -233.74697   -309.07597     -369.797   -277.52321   -371.58967   -322.74982   -168.73179     -41.3708    193.75739    195.78211    247.43076    210.92671   -198.00536    -81.84419   -137.92018     64.97641     240.9971     399.3278    446.14929    637.13937    405.96069     55.26044    -95.76117      -96.335   -104.01472   -156.20626    -332.6892    -99.50247    129.81098    757.93085    1296.0567   1596.80855   1862.47408   1842.60676    904.98203    541.78292    297.37934   -125.84144    -31.45782   -200.24914   -260.11448   -253.69945   -331.03843   -282.37194   -290.22547   -232.75842   -285.41337   -391.85862    -323.4144    -238.6036    -76.10611    -187.3622   -103.79156   -156.21786   -164.12087   -249.44363   -173.48718    -57.39852      0.92028     51.01871     -28.5015    -21.00017     -229.016   -181.89443   -382.23381   -202.22933   -286.16596   -270.87772   -271.16112    -78.62779   -316.67948    -117.3121   -100.49562    -237.0273   -272.24861   -261.01421   -264.78382   -270.54143   -310.49213   -236.12723   -343.51147   -337.80442   -460.23045   -336.80658   -187.42135   -129.62706   -105.94739    -113.1017   -137.02226    123.65721     10.15771    176.99366    140.21608      202.451    111.21364   -142.32181    -98.51841   -257.90957   -104.21197   -179.12463     26.95617    -62.51335    -27.14554    -78.76742   -142.47357    -98.67997   -295.39687    -240.6331   -369.24828   -442.95683   -364.78599   -409.65366   -445.78203   -301.88144   -464.13677   -321.12505          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       27.50   -138.85407   -308.40116   -350.65418   -344.41536   -142.23772   -302.31265   -438.10493   -350.96599    114.97962   -192.40792   -410.62265     47.71662   -353.72397   -385.13389   -135.61575   -434.74584    -98.02337   -244.34173    -378.0846   -485.38884   -240.79997   -344.63456   -249.77633    -46.10832    -17.04814    534.92506    894.56352   1440.26413   1046.52235    282.52646    -79.79901   -200.51579   -195.03845   -434.58749   -285.70341   -256.55681   -409.39894   -303.29966   -293.23999    -440.5323   -371.27704   -364.28542   -374.27537    -430.9869   -366.86475   -545.94982   -427.51676   -253.35833   -389.28357   -383.37155    -358.4123   -517.35805   -327.22755   -420.02607   -439.46855   -114.67926     209.9335    990.43796   1076.39168    847.16576    454.24554      35.2197   -354.92985   -376.74533   -369.27144   -445.08394   -204.02489   -307.17771   -294.04082      61.6006    479.58973   1080.08568   1411.76081   1108.28465    645.57811    548.96068    415.54502    173.64052     54.22851     -0.73336    -353.9382    -497.3179    -328.3928   -346.14086   -289.92653   -338.44869   -434.49045   -239.35042   -354.69601    -62.60457    202.40656    222.27743     -30.8974    -62.17644   -228.18313   -371.93337   -236.03612   -360.78942    -308.8426   -134.35588   -302.06422     89.22921      2.75371     50.27089   -191.47919   -306.85627   -395.65786   -271.85615   -420.46676   -301.30043   -317.68251   -338.94388   -312.07171   -429.35913   -225.08929   -164.49807    241.15753     179.0334    135.42302      68.3707       22.744   -109.70565   -100.99838     37.19901    247.50908    379.12628    516.81155    595.34536    388.37882      98.6799   -121.79988   -196.43049   -204.31456   -151.18947   -203.56271     -9.80619    234.65823     573.4459   1406.66739   1715.25312   1812.10829   1504.71586     960.7443    593.06741     97.92171    -56.09704   -112.34347   -140.64325   -138.39352   -121.52231   -377.79799    -280.4256   -407.89003   -265.72379   -273.19636   -283.81176   -247.50498   -258.80311   -218.26692    -82.06362    -25.92384   -206.89318   -120.69423    -73.95703    -84.18949    -26.67276     55.50068     -3.07766   -102.87834    -34.02717   -163.05934   -139.70322   -126.64892   -306.09521   -178.38575   -185.43225   -181.30751   -104.69754   -199.55378    -122.0786   -170.12763   -283.42422   -330.16937   -260.59174   -461.39134   -303.03959   -366.36855   -393.48596   -357.32376   -402.27211   -264.78576   -201.54291   -285.46484   -209.79646     -86.6137    -14.95446     11.90109     58.60583      6.85245     15.75906    -28.79531     113.9049    -88.26335     74.48472   -106.67702   -237.58467   -230.61526    -187.1404     40.19692    147.90087    -35.09707     44.97724   -152.66724   -217.70967   -322.29696   -381.67463   -380.99081   -388.34454    -512.1082          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       28.00    -163.3232   -336.30291   -520.39798    -295.8734   -132.62089    -217.2459   -515.10378    -230.7458   -341.47523   -280.04139   -393.47207    -96.77213   -397.10202   -319.10997   -260.25681   -411.51104    -93.56909   -405.58661   -479.41343   -485.50636   -222.39449   -414.55249   -317.39836   -233.54304    -38.47365    376.24313    940.61006   1384.48581    879.93341      327.113    -55.83383   -281.58581   -414.16132   -519.84399    -246.4496    -373.0188   -511.27856   -341.78495   -439.32568   -486.42077   -482.18953   -320.87602   -252.92731   -339.14124   -299.86528   -332.97224   -421.17193   -320.20364    -127.4587   -466.11928    -563.3175   -531.87043   -507.31033   -386.63641   -262.67629     12.58272    259.56624   1160.47435    1252.8726   1013.17661    399.92414   -239.88985   -315.08044   -355.96006   -370.82957   -363.39497   -310.27877   -283.72334   -174.55994    109.10636    382.16959    1308.0286   1393.12269   1184.03577     456.9017    576.29097    350.97894    335.93728     -5.75619    -36.03626   -259.43545   -394.72009    -58.29337   -386.45787    -245.6336   -452.91958   -230.03211   -296.96238     -54.8423   -109.56935     147.2708    242.40446     91.01164     37.99119    -225.5837   -468.07908   -434.62905   -371.73838   -363.58794   -266.19634    -337.5465    -100.1082    -30.83405    -36.78469   -190.00667    -429.1559   -311.03127   -221.51731   -260.87025   -365.81789    -289.0941   -280.32063   -329.55426   -302.32553    -274.4688   -181.41345     93.88206    312.30471    190.12617    111.47065    -20.28892   -138.77442    -278.1066   -110.13357    240.48495    431.52599    574.23475    503.27656     263.0633    144.62067   -106.87731   -246.43746    -95.22854    -83.35392   -125.49111      2.05873      246.486    731.25197   1310.58129   1726.64003   1940.22635   1693.44029    884.25092     485.8544    232.48826     70.91541     -26.0437   -136.10473   -292.73527   -293.52681   -214.36756   -378.51949   -346.08067   -292.24037   -281.87065   -250.10435   -117.49938   -201.49578   -120.70123   -223.36354    -89.87932   -231.44416   -179.79668   -114.19855   -145.54083    -61.97631     48.22401    158.70871     10.76093    -15.30156    -113.5531     -3.72161   -213.60003   -217.26948   -287.91553   -519.17117    -295.6915   -164.40704    -72.85449    -16.50776   -143.83634   -112.02083   -251.02263   -441.38145   -387.31778   -297.43316   -315.12608    -279.3055    -207.8363   -354.85585   -257.29477   -311.43989   -214.35543   -126.66333   -168.60252    -22.30759    -12.71817    -64.91185     -92.4973    220.53852    115.94958    165.30861     14.87683    -52.17905   -189.32365   -143.98589   -117.77611   -168.13831    -14.40575   -168.20579    -16.04008   -114.40979   -184.72591   -277.69849          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       28.50   -233.82218   -455.89159   -630.77868   -299.38542   -305.03435    -188.0306   -534.26963   -192.59755   -619.03118   -361.99477   -431.94778   -323.95102   -257.78669   -415.43726   -448.73628   -371.96244   -185.54957   -496.07472   -466.21155   -429.12429    -258.1933   -307.15339   -319.31887   -183.84893      8.20716    370.10863    1098.9542   1517.08259     862.7107    228.39844   -148.57876   -294.18058   -479.73724   -232.44976   -377.99325   -138.99999   -578.23319   -359.75927   -408.84924   -542.37378   -414.74125   -466.38728   -422.32774   -367.21422   -298.72972   -400.01026   -346.89664   -374.76576   -431.66017   -319.21118   -365.56089   -413.47078   -361.57096    -388.4584   -236.94857   -152.63264    292.94721    873.07901   1304.77724   1019.78464    398.51608      4.49439   -357.94237   -118.24922   -309.46165   -298.72189   -387.45098   -343.71887   -369.52346     21.37254    570.73508   1048.86901   1430.44491   1067.75714    754.31032    430.69396    320.81833    329.96456    -14.87561   -267.92445   -240.17734   -401.19934   -372.76724   -112.71935   -398.50486   -525.77816   -420.03891   -569.41595    -125.0251   -155.81035     76.78487    300.65597    258.69509     34.08719    -25.57551   -297.93182   -401.44645   -396.73719   -448.99547   -498.28096   -295.75672     38.07634    -68.21748    -37.54465   -155.46367   -253.05733    -273.9353    -47.21168   -270.42155   -293.36588   -362.31303      -195.71   -322.78244   -321.98242   -257.86867    -45.94456     23.66021    212.96314    436.06906     91.06638     89.71022     -189.356    -49.85526     13.33826    373.22307     358.9621    730.02132     550.0061    407.29237    113.11614     65.41199   -178.48921   -162.10716   -292.53342   -237.14371     45.34058    223.84387    808.45993   1264.86888   1794.19041   2219.64098   1464.61186   1105.15227    526.38062     283.6826    -35.90253    -44.25576    -52.64273   -167.51247    -90.86524   -474.99847   -266.59679   -312.02069    -173.1133   -360.35418   -374.46677   -385.90086   -273.69678   -249.46275   -130.07731   -170.96105    -83.48316    -54.41579     -71.1227   -200.44684   -103.48868     13.67157     20.00477     14.72902     -1.73732   -110.19349   -232.44752   -159.50196   -275.55134   -234.52444   -131.68034    -353.7502   -173.56289   -155.56014    -152.8467   -217.50491    -350.2937    -240.4598   -281.66132   -357.93998   -338.50813   -485.74843   -397.31054   -342.09173   -354.02756   -327.62637   -292.00471   -205.96362   -185.80923   -161.97816    -80.22057    -43.03328     98.28244       1.0982     17.85801    106.27014     82.36071    -15.37132   -159.11408     -47.1802    -76.80647   -215.74073   -246.79649   -219.57927          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       29.00   -402.96833   -569.09362   -662.78306   -309.22726   -489.14642     -194.707   -307.22784   -218.55024   -616.29601    -218.8711   -376.80019   -355.81911   -158.70821   -525.73439   -428.92556    -328.1354   -398.25438   -516.59349    -412.5218   -289.35109   -337.69939   -487.73545   -286.53066   -217.20321    -57.45386     620.0777    955.75972   1372.15766    944.85502    215.44811    -67.96171   -251.74179   -533.89317   -402.66376   -396.84372   -205.91065   -486.54946   -285.12212   -267.35995    -612.0888   -348.66207    -442.7844   -369.18551   -339.87068   -469.80602    -321.4412   -409.05935    -300.8753   -420.87255   -235.66097   -197.46546    -381.4134   -284.72669   -395.73935   -278.72903    -92.82319    323.13543    808.92899   1263.41818    858.60074    386.16469   -110.03449   -336.26753   -357.46615   -197.70989   -418.55614   -270.59229   -281.65297   -185.92543    -28.44875    378.61461   1110.42999   1500.40406   1260.99729     772.2328    476.53411    351.01763    322.47469     -57.7996   -471.68351   -430.77507   -428.46303   -204.87343   -108.80258   -185.27203   -352.60432   -353.48766   -273.33268   -197.30811     38.69491    126.54143    100.99347    235.40721    219.78862    -62.78733   -325.76454   -564.40926   -506.92567   -505.76478   -397.69156   -163.87402   -202.35432    -72.23556   -132.81481   -240.61815   -381.98834   -262.16642   -372.85631   -484.06946   -422.92478     -256.006   -374.97585   -270.12946   -151.78522   -132.96446      6.55642    225.49963    199.65217     338.7555    102.87534    131.20583   -173.72733    -91.67064       65.866    313.60212    559.18914    536.58715    506.97515     361.2008     103.6112   -165.77755   -124.76901   -265.93906   -216.70937   -171.22337    -41.36019    270.67805     755.1668   1450.34249   1728.15969   1717.93222   1686.26266   1097.42342     638.4135    350.64406    -59.80517   -377.79567   -230.69173   -371.42529   -324.69635   -333.82463   -384.90587    -191.4279    -317.4829   -278.68689    -280.1314   -205.87537     -179.894   -161.59456    -107.9417   -219.85112     -2.50962    -149.6164    -48.01695     -151.741     21.82448    -43.04571    150.83017   -131.79401   -101.24142    -90.58096   -245.13355   -307.02735   -404.67268   -162.64374   -234.28873    -22.10909     89.65843   -112.40377   -172.72454   -208.55032   -297.07211   -320.87874   -289.43292    -300.5071    -382.1303   -340.22934   -336.56063    -419.0068   -309.38763   -345.93757   -242.90496   -172.95283   -125.43602   -156.74483    -34.39128   -126.77653   -156.19288     64.73403     -0.66675     17.35475    -65.75748   -101.79571   -333.14935          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       29.50   -672.43387   -605.53833    -642.2988   -284.97566   -544.84458   -218.23044   -280.29856   -187.02065   -684.23025    -24.11669   -435.22422   -362.82461   -225.06601   -494.67843   -389.30808   -188.59826   -510.43309   -334.50761   -300.79499   -158.77385   -188.11646    -408.5507   -294.77876   -255.02228     77.55323    539.30171    1030.4621   1282.28467   1024.45064    297.97284     -121.893   -360.68175   -575.02923   -535.97528   -362.39482   -307.98413   -423.45109   -365.66202   -313.46545   -506.62839   -353.47444   -326.89137   -464.76641   -391.76465    -451.0541   -557.77491    -484.8494   -289.19881   -419.13072   -213.90847   -239.42864   -220.99558   -334.49679   -183.17178   -417.37339    -94.78602    329.58202    583.12056   1108.94599    860.45481    364.81584    -95.91287   -113.14062   -392.80349    -259.2584   -309.71187   -197.17877   -376.79985   -218.61753     32.73025    559.83801    999.22795   1370.88606    954.34035    709.61966    327.12937     598.8807    245.34358     71.62292    -63.01502   -577.38964   -432.53171   -185.60702   -244.27679   -452.01371   -308.10832   -323.75281    -389.2105   -204.45802    -53.13815    145.64151    313.20513    238.20257      29.3561   -102.72701    -248.6841   -319.76237   -416.05817   -376.24563   -266.33991   -192.74388    -84.94556    -59.40154    -15.91922   -110.28947   -214.08355   -416.14111   -299.17612   -320.03272   -330.70105   -376.09322   -327.77518   -332.41367   -204.17579   -345.80575    -89.95547     51.16299    201.90319    160.25838    129.28952     34.77373   -168.00757   -126.94148      6.50227     233.3045    616.64908    412.52115    600.86989    325.91442    -26.35521   -134.35694   -150.43486   -183.32066   -167.98259   -185.02189    -83.21296    108.46236    625.34884   1272.75876   1766.23162   1883.51921   1521.53419    947.13106    752.42452     80.57885   -221.35391    -92.93328    -86.67372   -152.00792   -164.79379   -409.96172   -237.47262   -261.48497   -263.75507   -292.34015   -328.75974    -249.1747   -384.86627   -115.59192   -180.48184   -142.77911    -82.02934   -181.68181   -239.70335   -138.48991     96.33034       4.9494     55.12043    -46.77887   -144.37077   -207.63572   -211.15463   -247.62409   -362.68326   -296.30362   -388.48817   -115.93725   -237.53997   -210.96115   -188.36751   -151.20226    -297.0778   -171.67357   -313.22259   -355.79896   -479.71252      -282.34   -452.69182   -346.47881   -382.74241   -186.55771     -325.713   -325.22142   -247.02238   -100.87215   -104.92477    -48.02248     -48.8606   -252.69199   -298.37634          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       30.00   -661.09357   -512.01153   -564.60955   -294.60616   -365.74777   -278.55052    -416.6913   -200.08278   -481.54783   -186.96766   -468.59056   -345.50455   -254.43568   -439.43142   -345.85588    -193.9536   -424.81823   -249.10901   -342.19043   -178.99091   -232.83895   -345.40134   -236.61621   -280.50459     15.01957    410.12035   1076.99416   1313.69916    810.81193    469.24368   -282.81473    -360.8356    -389.9998   -479.06245   -411.58567   -250.36969   -385.34214   -274.44317   -528.07494   -336.09298   -466.23065   -471.93373   -335.41825   -370.21344    -459.1178   -520.62363   -411.22896   -443.69624   -440.37006   -287.63252   -432.55553   -363.06662   -417.07228   -394.96883   -451.94257    -13.09142    326.45572    916.98481   1002.35735    850.43534    262.37882    137.03961   -283.04447   -418.58864    -245.8541   -422.65139   -151.97874   -303.63557   -174.01307    112.48357    526.91022   1281.71845   1417.11282   1151.66633     691.0836    551.62197     354.8924     95.41302   -270.72638     -93.2382   -499.24761   -461.39425   -300.69048   -310.14013    -560.0803   -379.55468   -461.81008   -507.51269   -251.95036    -18.97336     96.29069    357.90792    342.34649    207.26839      1.79879   -187.77757   -208.85217   -322.96443   -353.16559   -351.11396   -240.48226    -45.15268   -254.40299    -42.08824   -134.19899   -116.54043   -318.57806   -358.01382   -242.73118   -411.80287   -475.83588   -329.83762   -392.82673   -383.36555    -143.4885     24.71857    209.20266    139.70756       -3.445    166.85493     -30.4571    -18.50923    -91.11234    -28.04011    263.97256    388.68426    532.27868    614.37646    387.80703    267.92969    -34.64725   -280.72806   -213.94497   -130.60084   -124.06555    -76.13616    150.96652    707.29079   1199.97077    1585.9141   1879.01221   1791.26732   1174.98336    640.75001    260.12271     16.03978    -59.41499   -188.10694   -298.35265   -204.83414   -229.75184   -219.40384   -356.03397   -251.18875   -363.85571   -347.75345   -318.85791   -139.48697    -141.3978   -170.25819   -139.28881   -357.69617   -149.02731    -38.60899   -152.86084    -21.95449     19.72802   -120.69568    -52.66565    -90.01844     29.15833   -177.80957    -275.1947   -182.33766   -232.64912   -412.54247   -230.83828    -118.6838   -180.61567   -227.85016   -205.38558   -229.27336   -249.00961   -270.30106   -241.67753   -221.86001   -424.68216   -204.15892   -372.29684   -409.00365   -456.42175   -224.71512   -222.94146    -246.3598   -203.16272   -435.65847          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       30.50   -541.92237   -424.57759   -503.47685   -330.14735   -160.45294   -378.98006   -495.69531    -263.1852   -204.40082    -453.6525   -374.11128   -337.55445   -272.49863   -327.35611   -284.85522   -284.84143    -366.6545   -258.90324   -331.84273   -355.78734   -398.75224   -259.07828   -288.72897   -104.30954     40.75557    606.94496   1191.74472   1444.40599    601.48354     365.2085   -283.29142   -302.01736   -239.33056   -438.92866   -293.92828    -380.5029   -571.66921   -269.18204   -410.83655   -174.02993   -565.05198    -563.5107   -403.31386   -340.33408   -473.51743   -468.69898   -522.02445   -284.65652   -393.79164   -282.30093   -406.74105   -322.11984   -353.54234   -347.07843   -360.91556    -85.29822    453.12916    866.22716   1215.24899    833.08752    428.11439     -68.6591   -321.66685    -320.3129   -565.44395   -336.16484   -490.80307   -107.21283   -110.82482     59.90381    417.61142   1142.94564   1212.67592    1297.1974     789.7276    456.41173    326.00349     81.52389    -62.91459   -216.27053   -328.37364   -278.01329   -268.68063   -325.96098    -441.3963   -274.88772   -131.48051   -370.14826   -293.94052     20.71712    262.84428     511.8372    388.52753    174.93094     82.28252   -251.95444   -199.85428   -175.86281   -454.07998   -335.11114    -95.61645   -140.28695    -24.07647   -104.91238   -207.48995   -175.33172    -378.0471   -323.34647   -445.20045   -364.80118   -315.78608   -244.57049   -290.62617   -395.38798   -105.68072    -84.12337    223.37968    221.25139     95.37078    301.87368    -34.25576    -44.77462   -185.43899   -112.75247    123.52635    470.60603    587.59552    519.84988    348.42941    -81.35868   -152.45913    -45.65193   -192.87496    -160.0915   -110.96867    161.17524    295.42991    738.71435   1306.02935   1613.90903   1792.41476   1541.74088   1280.24294     665.7292     107.2382    -88.13585   -155.13127    -132.8715   -326.20282   -242.87532   -162.65642   -297.51008   -244.29042   -359.80229   -327.45294   -239.58849   -250.72317   -200.85867    -80.95702   -236.39481   -226.34412   -196.59533    -20.78536   -157.23591   -201.80118   -139.94346    158.54261    -69.91021     55.70574   -142.65056   -244.54705   -266.82679   -306.14451   -312.85458   -259.21367   -263.99188   -239.94649   -231.62661   -232.75732    -80.51283    -271.3645    -229.8669   -256.59582   -299.57729   -191.96354   -462.30941   -316.01003    -381.7646   -316.03561   -442.81704   -274.10951   -403.38116          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       31.00   -399.80287   -365.48779   -467.68474   -367.73399     29.93275   -496.18942   -472.80056   -275.04733    -103.3287   -537.63402   -400.73286   -374.32247   -356.96599   -293.99458   -305.06253   -334.08996   -444.48467   -278.62782   -287.72804   -374.97673   -348.93144    -459.8482   -369.97531   -174.92319   -160.44506    250.66736   1199.38601   1347.03902    483.87546     60.45724   -237.70119   -350.50261    -246.5502   -304.93591   -331.77906   -476.82692   -443.41662   -372.40831   -333.33726   -415.06231   -528.86006   -366.97394   -753.93353   -234.84247   -297.08199   -595.32753   -436.37194   -206.77965   -267.43958   -443.39116   -491.20937   -362.91332   -342.88411   -282.94677    -329.8268    -82.96489    401.83147    891.95765   1346.02089    920.59718    256.17412   -242.86286   -219.35863   -241.28164   -309.71394    -317.4123   -490.97382    -296.6961   -235.26721    126.70597    590.46701   1228.85422   1354.27096   1203.81049    554.95363    429.86846    366.62923    213.00831   -187.71532   -103.02631   -326.84668   -381.80975   -357.35325   -303.63274   -537.16967    -247.3642   -456.25109   -366.59826   -213.03357   -147.13863     148.9294    258.11519    335.25872    362.37546       25.119   -168.65794    -80.93029   -304.36904   -298.55233   -121.82607   -199.61582   -230.31307    -70.20189    -61.71321   -186.07229   -179.78944   -324.10976   -287.17594   -392.20256   -297.06707   -315.28075   -358.83083   -297.19568   -326.31621   -240.62908    -28.47205      0.89874    282.97287    270.27545      264.957    -14.71149   -140.76194   -256.23757       2.6116    162.61835     343.8115    482.81034    404.06869    469.70175    138.64824    -20.52011   -177.15213   -259.81852   -204.76431   -271.57266    -61.59514    326.52546    562.86964   1216.66424   1723.62793   1893.45386   1720.12656   1015.39007    613.70381     211.8647    -69.23032   -111.89798   -127.63629   -140.60705   -168.44643   -370.69146   -279.91181   -338.87106   -229.54164    -314.0817   -333.36875   -410.94829    -176.0667   -147.42404   -199.50393   -218.29858   -114.52884   -240.78049   -199.95358   -136.01056   -191.80124     -16.7783   -144.74225      3.32723   -121.46705   -147.39302    -141.1734   -328.48171   -139.92154   -238.10125   -157.31137   -231.69367   -243.58062   -169.59847    -79.15962   -240.02842   -262.19461   -356.53292   -232.81749   -320.58615   -490.67804   -463.82967   -566.54174          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       31.50   -359.36349   -425.92012   -454.87239    -396.9022   -111.50372   -550.81039   -435.89377    -299.6844   -284.10981   -386.87084   -427.43232   -318.96312   -459.05468    -357.7181   -373.38356   -477.87837   -406.55025   -434.65679   -422.89401   -370.84613   -270.70633   -404.01195   -365.57871   -324.47607     -54.2494     300.0443   1182.86829   1196.22495    503.50831    106.04344   -324.34187   -262.75966   -120.65025   -309.34197   -313.59877   -356.37611   -466.44882   -422.44663   -369.29067    -352.5082   -250.74068   -440.61132   -474.96536    -43.17159   -397.07351   -620.13167   -517.16296   -461.15991    -430.7423   -201.96119   -512.18757   -385.41313   -413.64056   -218.00084   -307.89382    -87.09687    328.53377    902.47561    978.82164    719.91539    329.83126   -141.47352   -147.41656   -312.04023   -282.88389   -293.33428   -327.71319   -279.79325   -315.52361     87.06724    525.57574   1339.21649   1519.61565   1121.37488    817.84483    525.38859    160.50788     303.2707     -0.48385   -371.37503   -242.26856   -339.25254   -194.02303   -315.15547   -422.75512    -237.8922   -327.56327   -367.77776   -321.42934    -15.56075     97.02852    362.32031    109.68618    -33.99567    -70.89896   -122.17845   -267.56578   -128.61574   -127.71238   -328.43537   -233.57402    155.87285    -41.22557    -23.33934   -103.01355    -136.9364   -356.04215   -297.46978   -376.08533    -290.9875   -168.32704   -305.69401   -373.12506   -257.84813   -241.74045    -96.32196    143.48935    176.47646     97.11627    200.92453    138.01054     68.15374   -212.43506    154.32709     201.1162    413.42914    578.00614    507.96386    542.63163    145.18698     55.14096     -77.2095   -152.33421   -312.52201    -68.97939     -2.58943    227.96185    709.18422   1082.00935    1646.6976   1838.87202   1777.75507   1081.63426    639.15496    305.66334     -5.97728   -201.07624   -126.71365   -140.12584    -224.9198   -377.09976   -255.84291   -323.77103     -214.314   -393.85228   -341.97424    -240.1942    -203.9386   -137.95683    -68.04638   -122.71386   -134.70868    -73.76136   -258.75386    -73.71774   -167.14361   -156.63573     48.60659   -255.44835    -60.84342    142.71522   -174.01349   -202.60208   -233.42982   -228.05541   -371.18992   -312.86078   -279.12846   -264.29596   -239.60892   -107.38171   -246.81375   -331.20902   -483.58764          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       32.00    -411.5928   -553.94172   -457.94344   -472.75149   -282.92527   -496.32304   -368.62808   -311.15131   -426.46536   -300.26439   -281.48801   -269.13358   -457.80966   -375.89062   -447.56317    -391.7362   -164.79907    -524.1728   -473.35464   -470.45861   -260.03105   -206.40086   -412.39068   -286.99957     91.97021    634.15025     920.1538   1083.72105    687.53611    281.39587    -241.1833   -114.59385   -245.23563    -330.9647   -387.17733   -245.23496   -361.76926   -350.79718   -297.02081   -402.56881   -463.16912   -358.42363   -280.09149   -332.58206     -385.526   -497.04697   -458.87619   -369.88326   -200.00111    -281.4246   -384.76889   -360.61664   -453.62674   -353.05509   -244.04175      3.73876    224.35912    857.07646    929.27448    751.03202    273.88329   -159.21847   -289.10768   -262.94412   -457.60362   -391.59056    -385.2217   -309.04578   -327.16409    167.63186     822.9262   1035.56314   1241.63097   1040.44006     712.1741    142.18947    190.00813    355.41171    -57.18365    -49.45752   -208.24646   -232.98534   -397.11559   -239.09707   -369.68955   -292.97862   -442.05188   -266.96612   -191.18421    -23.66577    164.15074    341.39464    276.92386     21.66943   -118.39433   -159.49637   -257.20381   -133.23934   -157.02801   -291.51276   -198.24336   -132.44641    -28.42191    -79.55667   -148.42439   -208.17663   -282.55339   -328.72357   -286.33639   -316.36892   -390.69775   -297.24172   -278.20932   -270.84299   -273.27252   -106.46847      97.1606    306.67527    136.47014    165.08383     31.92022     45.46026   -175.65953    -19.04287    177.47612    483.93648    532.49047    512.09317    352.79249    100.21205     76.47846   -203.05256   -245.05741   -260.64398   -170.07782      7.20407    207.71468     609.7879   1065.24968   1864.82131   1779.28411   1578.04326   1312.20513    699.84335    476.51202     67.39876   -211.44735   -355.98783   -256.27329    -164.0251   -210.09973   -138.27583   -390.31363    -287.3852   -272.40418   -334.62753   -184.84626   -167.86708   -225.69288   -326.57232   -271.54954   -149.61934   -154.80904   -153.33902   -162.96759   -136.49192    -88.68995   -217.17426    -14.45498    -122.7055     -9.56952   -203.73498   -150.70616   -289.90022   -296.72296   -186.74227   -166.18948   -221.74252   -199.14671   -390.30324   -369.00044          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       32.50   -462.35913   -596.45513   -486.51912    -592.8734   -452.87852   -451.62947   -279.56684   -327.71961   -498.67838   -247.80127   -401.79964   -374.05479   -445.30313   -414.67927   -325.70164   -343.99972    -96.30515   -431.83267   -455.92278   -502.80222   -279.60351    -344.7625   -337.17556   -196.10308    -15.63963      534.335   1026.13768    967.50587    721.57343    -56.70682     22.36812    -42.26983   -375.06764   -317.22765   -352.32795   -412.00816   -346.68771   -461.65901   -304.92432   -259.34956   -354.58723   -492.44995   -476.30494   -164.38682   -335.58773   -474.47178     -619.279   -368.11225    -280.5255   -288.52099   -305.74892   -477.50611   -324.70865   -300.14639   -278.12717     -20.6988    248.63154    949.99341    969.83065   1028.26803    359.74264     40.27301   -244.08262   -239.29752   -327.55287   -338.36489   -501.46574   -377.56846   -116.88194      66.9749    558.00367   1082.19555   1403.72535    977.23249     819.4655    406.28905    437.18075    250.97012    -70.13058   -152.40504   -498.24632    -250.3548   -178.55664     -404.256   -425.70047   -455.78612   -372.49441   -439.54172    -60.42945   -103.44736    207.07178    261.37582   -107.28949      8.49171   -147.03624   -310.06447    -134.4277   -188.80932    -249.7385   -108.73994   -173.46429     32.43391     -2.76023   -291.10919   -189.01985   -157.73843   -313.10542   -333.01608   -268.16255   -355.73882   -302.99578   -375.94139   -305.65728   -273.39179   -219.39255      1.40664      86.7961    117.06264    254.88942     94.37544    -88.99241     -70.9839   -118.64366    -57.16079    234.90181    340.02737    591.85916    608.84118    342.13753    223.55086    -23.22284    -77.51782   -206.99964   -207.27287   -244.96414     40.49974    249.53449    735.70555   1178.06194   1689.51814   1937.14393   1790.21488   1131.27049    684.19314    307.20887   -102.55953   -156.11981   -149.39624   -253.39333   -257.97084   -210.72694   -401.93478   -218.87858    -462.1592   -207.42053   -233.17323   -360.18613   -251.46517    -268.6548   -224.75449   -106.21873   -237.27053    -46.85725   -247.80366   -298.70194   -150.57991      -51.412    -62.69609    -57.53607     77.39827   -199.28138   -193.65805   -183.05829   -409.39762   -355.68769   -249.76499   -448.19178   -347.18695          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       33.00   -488.11463   -549.86307   -512.15985   -341.80063   -429.73799   -416.76428    -272.3169   -307.55982    -535.4168    -89.68043   -462.34252   -481.03576    -493.9413   -435.69567   -166.56563   -255.00238   -112.69705    -410.0885   -374.82302   -618.31822     -294.736   -687.63874   -228.61908   -219.86043    -49.78446    649.29863   1000.96108    957.02647    744.09842    140.95834   -259.67724    -213.2403   -350.91913    -350.6545   -405.51847   -341.73588   -399.12411   -353.56073   -468.29918   -343.47677   -311.33828   -364.54404   -210.86346   -374.24571    -387.0382   -341.53789   -298.07029   -435.92301   -322.90971   -458.82492   -394.14502   -487.14834   -391.96347   -315.85647   -177.83719    118.62495    280.07698    702.25777    779.14035    670.57354    253.90799    -13.55987   -210.84242   -288.30136   -332.81782   -298.94846   -321.65033   -277.28531   -105.27528     -4.41178    456.72336    1052.4162   1413.64455     919.0227    715.45855    693.15826    140.02935    195.90244   -136.30547   -185.67305     -72.9619   -271.22536   -208.55766   -391.33497   -395.09361   -368.32324   -427.06189    -285.3893    -75.87465     -56.0015    133.52184    235.62313    197.46744   -108.11986   -173.07175   -108.57571   -192.40826   -227.27493    -53.31508   -149.60055    -23.58483   -161.52588   -106.58091   -249.25739   -118.25691   -201.84993   -196.95896   -216.01418    -333.1882    -263.7924    -299.2721   -269.40923   -470.63093   -338.54615   -118.90807     55.29992    -27.89354    235.94169    134.09178    212.64442    -34.61031     -94.4902   -139.49369    -73.73963    301.28488    500.29576    591.78809    447.53229    434.18383     42.41368     84.78505    -95.45859   -296.09462   -304.18534    -62.32742    -77.34049     124.8383    600.56379   1365.47482   1747.83807   1713.91632   1565.36981   1214.50304    803.37745    171.53571     60.20281   -167.20163   -168.74198   -262.49366   -239.96013   -229.91931   -331.78307   -265.73998   -369.96067   -309.12988   -370.11013   -408.87366   -296.83127   -241.88056    -260.2822   -126.67703   -259.15707   -134.40131   -202.57441   -212.89658   -235.63293    -138.6289    -39.82176    -53.51589    -71.08615   -193.54952   -253.57697   -260.61638   -388.02148          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       33.50    -531.5387   -612.96433   -549.82079   -161.84918   -299.52678   -409.44249   -346.44324   -241.60969   -499.79517   -179.06017   -239.64337    -435.9163   -497.29889   -440.89131    -36.38902   -249.30407   -252.41719   -349.50047    -432.9613   -573.10434    -198.7193   -438.59604   -207.69744   -164.81243   -142.80153    480.29823    807.31953     897.5157    686.88234     179.3496   -331.60532   -310.63632   -179.24229   -507.56715    -396.8631   -265.66903   -286.98302   -442.44696     -452.215   -571.12891   -424.88087   -386.67672   -465.21154   -357.99643    -317.2544   -284.29313    -28.95382   -340.26736   -336.70932   -419.37262   -347.33189   -396.65246   -299.45514   -369.84167   -280.08725    227.89225    238.17882    713.63296   1013.21892    522.23457    386.42392    -75.26226   -413.66266   -465.07539   -292.87462   -353.22652   -166.00117   -334.01967   -242.98289    196.95179    631.56504   1141.01704   1138.44312   1128.95692    613.46428    560.92514    354.87681    159.95731   -287.47367   -263.87908   -400.28951   -106.57965   -164.75302   -318.20321   -398.83846   -405.32452   -375.88406   -188.74719   -184.78738   -139.77972    211.07017    468.30326    212.48718     126.5832   -105.26456   -317.31054   -480.20935   -315.28294   -289.90296   -201.66879    -54.35512    -26.83278    -36.16937   -227.88483   -215.01836    -321.4904   -227.88498   -279.06513   -286.20593   -349.09275   -215.68184   -357.73766   -261.26959   -326.20169   -181.80947   -196.53361     75.10862     126.2406    391.81872    151.93994    156.81211   -107.54217   -107.16412       3.2578    109.28855     426.0687    389.34648    470.23396    368.10251    227.39645      0.96937   -230.79213   -178.51483   -191.66046   -307.97069   -169.01167    166.37633    508.33693    1029.1374   1645.50667   1788.51979   1551.70086   1123.85124    681.95097    235.56688    -45.69747   -131.04821   -146.25668    -172.3285   -361.04671   -211.46537   -388.21198   -314.06956   -294.52237   -262.67038    -344.7429   -329.75181   -282.89532   -223.62398   -135.94101    -207.9393   -163.17536   -286.88595   -304.12436   -230.41293   -134.75443    -74.96264    -43.23777    -80.66203    -69.01813   -314.57344          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       34.00   -552.69339   -830.74957   -454.91985   -150.64601   -203.46195   -382.20812    -422.5468   -207.40523   -524.88921   -210.31524    -89.67725   -370.91911   -380.59611   -347.97041   -125.78766    -400.4497   -362.17702   -224.95805    -554.6241    -343.4659   -155.62232   -402.50516    -86.67216   -209.77275    -88.62818    486.87694    774.89139    1143.2804    669.58646     95.43474     -92.9676   -389.78842    -87.84187   -178.74131   -428.60225   -284.13775   -390.18852   -346.23671   -285.26014   -457.99826   -268.90088   -348.19079   -398.13315   -519.45959   -369.20676    -492.6918   -442.60009   -420.67483   -350.42279   -347.03494   -336.62887   -276.70119   -290.71108   -378.25197    -90.23525    -65.11473    209.95502    806.12583     944.7943    713.00563    333.87151    -11.45419   -383.74568    -342.1665   -327.18345   -280.51457   -392.34577   -263.64027    -43.13776     63.44674    593.67591   1058.60343   1469.30136   1014.55128    831.25397    566.42658    352.56513     14.08432   -267.31875    -264.9114   -418.78689   -294.84066   -240.04592    -199.0946   -259.74514    -253.9225   -196.95444   -357.17463      -3.1542     51.01409    293.21262    134.75976    131.16607     86.61211   -195.40167   -313.83126   -420.14322   -269.52848   -343.46596   -237.00362   -262.10314   -188.81614    -59.89351   -183.45602    -222.2771   -272.02972   -115.78631   -265.36825   -310.05531   -376.29987    -470.5788   -387.46996   -261.64498   -263.08427   -246.74073   -165.66363   -188.45266    142.05539     78.42593     105.7279     67.38704   -230.42739   -200.53309     51.79288    148.02117    269.26233    498.23218    596.99798    420.16541    193.81028     -9.16354   -169.74708   -261.99407   -243.53844   -361.36884     -0.82004     93.04206    644.32587   1136.76713   1458.99679   1954.01728   1618.46366   1193.86571    746.74055    283.79371     56.73713   -309.56604    -51.93477   -140.14893   -335.69758   -210.54124   -230.60205   -377.21842   -366.67345   -339.27222   -311.44785   -351.07705   -310.42308   -213.67984   -194.35367   -176.20586   -209.67938   -334.60345   -330.21603    -202.5315   -194.31319   -123.77256   -180.32251          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       34.50   -551.35098   -986.99533   -347.80189   -308.06023   -361.37129   -349.29565   -559.37886   -265.82663   -511.48582   -183.49113   -183.32686   -378.16147    -360.1714   -203.28209   -152.74596   -409.73655    -475.9043    -94.53149   -545.47649   -278.79634    -116.2156   -515.69431   -239.15846   -343.91433   -216.95907    483.72677    872.11701   1014.56611    740.89796    121.98499   -109.13576   -332.59689   -348.08908   -179.92582   -489.92216   -339.90936   -516.56166   -316.00572    -306.2858   -520.66146   -256.64882   -311.57304   -242.20264   -425.67105   -474.40729   -374.78028   -409.39955   -575.10856   -261.46815   -216.68618   -441.90369   -236.70062   -368.61033   -289.93742   -366.03092     64.01188     147.1011     816.8596     784.1815    816.21096    389.74081      9.24386   -267.90255   -391.78502   -265.94956   -406.22764   -354.39515    -316.7611   -201.67038     -32.1253    501.99253    988.66274   1428.77768   1237.89792    819.38968    451.18186    463.45237    -75.97042    -58.81225    -199.2396   -280.40127   -409.83904   -337.50521   -180.18243   -279.35746   -197.09212   -362.66305   -342.88941     -32.1794      5.20708    131.03834    316.63577    160.35608     66.51451     -1.10549   -470.44453   -282.98962   -288.31176   -246.23544   -191.02073   -200.70652    -73.10682    -31.76959    -79.51188   -197.45163   -183.53828   -255.00726   -428.65923   -377.47766   -364.78607   -263.70398   -249.41455   -500.48358   -207.73378   -303.33127     20.51278      8.47265    104.22581    292.71042     67.67012     33.99865   -108.69054     -98.8943    -36.22405    133.84262    409.88524     485.4285    493.41716    394.49384    196.51814    -90.22723    -78.37832    -84.44164   -335.09218   -225.56237    -32.11178     98.94411    643.61804   1162.71485   1786.14212   1796.48063    1515.4447   1121.02104    665.70756    135.09541    152.03868   -141.17461   -208.96118   -192.49719   -313.05686    -490.3324   -363.83906   -242.98937   -289.25367   -372.15115    -337.0291   -489.59597   -353.58736    -266.4329   -208.96206   -178.86042    -90.34945   -105.06179   -250.19162   -256.79995   -513.72273          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       35.00   -504.10388   -717.87037   -273.70373   -411.22058   -469.06033   -320.85387   -620.79707   -379.34642   -478.68563    -288.7915   -371.74173    -407.0304   -377.79086   -194.39519   -146.90676   -455.99665   -589.78455    -45.25118   -444.88776   -306.71416    -90.87827   -327.05209   -181.88139   -303.29888     -3.68136    577.06965    848.43052    881.57814    759.66097    143.42472   -247.68633   -114.75455   -521.66216   -409.43573   -358.31644   -328.52652   -420.01914   -205.99384   -360.43402   -416.70053    -192.2403   -274.22127   -373.87863   -352.80901   -365.95673    -265.8279   -256.45913   -477.65832   -289.59193   -370.69432   -479.78578   -451.46966   -182.18817   -272.33163   -440.45385   -273.84664    422.31736    581.06899   1017.11929    674.33835    299.24419     89.47044   -371.18575     -354.075   -394.15837   -227.49544   -411.26517   -335.44083    -59.45166       19.141    543.79298   1189.43076   1159.25843   1246.90903     795.0226    551.92181    239.32057     66.84155   -307.85746    -79.54563   -326.60582     48.25548   -309.23313   -236.71032   -335.03538    -133.4547   -337.49258   -366.98244   -407.38293    -70.81795    259.24138    311.31016    379.24445    130.30096   -221.04022   -175.33854   -374.55361   -484.76731   -290.58357   -118.80762   -180.71776    -54.62551    -66.15648   -186.44077   -186.54256   -141.66782   -287.77069   -253.04144   -296.44661   -334.56327   -386.83787   -295.34529   -291.92843   -197.89774   -207.83732   -131.55205     65.63553    243.33286     214.2326    109.44808       9.9916   -157.91843   -167.58303     -5.51784     53.22473    393.62674     526.4161    559.37441    522.78252    197.71229    -39.57637   -130.91072    -53.52422   -370.75113   -192.28149    -20.13151     35.72562    572.45326    907.12455   1463.90587   1796.82869   1761.72022   1136.46948     815.6284    347.40832     38.64797    -42.49431    -74.03403    -382.6462   -221.51661   -392.90948   -259.85918   -320.54475   -305.52148   -452.27738   -305.70526   -301.27927   -147.66996    -237.3309   -277.13366   -202.84504   -281.86183   -309.94317          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       35.50   -470.85107   -484.25873   -243.02757   -413.61094   -411.51523   -398.75829   -472.55056   -414.61206   -468.32837   -339.98812    -479.9077   -302.83467    -384.0091   -244.94483   -240.22935   -555.86775    -494.3224   -270.84201   -381.03788   -125.09558   -129.58957   -117.76765    -94.72526   -176.16001        63.06    470.20063    745.22248    903.61539     574.3385    175.12296   -167.68977   -302.62127   -328.73281   -355.82869   -404.36075   -452.74546   -146.13572   -340.76033   -312.77997   -331.03982   -264.60248    -530.4202   -446.38111   -423.14574   -411.42886   -426.10339   -477.10921   -438.19936   -369.71634     -370.176   -445.12513   -385.75967   -379.40261   -150.14604   -140.68163   -116.07523    389.63028    620.01773    959.07584    616.35924    402.51529    -23.26537      71.8848   -311.54867   -275.85274   -560.35037   -266.17899    -355.2178   -289.46343      76.7942    595.45092   1152.88165   1270.48777    997.72323    824.15119    195.28128     -71.6909    109.59205   -142.85774    -170.6181   -175.53432   -354.22816   -269.81379   -192.04487   -299.47028   -150.93389   -424.44327   -134.18425   -205.92597    -71.92392    185.90348     216.3311    322.61261       102.04   -179.70894   -234.24013   -216.60943   -238.96483   -247.54764   -152.95469    -44.44016   -215.58103   -245.72847    -61.53489    -184.4598   -332.56233   -233.54925    -415.1283   -322.62559   -395.01929    -452.4178   -377.37449   -320.78522    -390.0242   -160.53401   -194.98751     69.98499     64.60757    221.64938     -7.14795    -10.87307   -110.95533    -80.87423   -135.19498     51.43591    340.85037    483.31198    570.27506    514.29197    197.63852     36.59566   -240.92414   -311.63369   -178.30794   -171.35985   -105.96215    186.48063    536.78348    940.52913   1440.59053   1807.78895   1761.41249   1185.71364    705.23504    351.17698    140.21063     -1.67542   -170.15831   -226.60188   -162.55332    -385.3493   -274.74714   -411.08224   -290.54592   -408.31453   -302.06674   -216.00962   -294.06499   -325.13104   -223.08637          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       36.00   -453.62456   -286.50262    -205.2554   -421.83725   -286.67709   -524.19386   -339.28048   -370.05976   -341.55457   -405.17634   -440.52198   -157.02423   -330.06422   -289.51366   -435.20549   -422.54604   -216.53105   -198.16497   -431.35331   -256.81567   -311.16245   -202.65437   -209.71266   -280.10369     87.63231    491.42838    943.34877    945.33267     585.5148    109.98915     65.83065   -334.54632   -278.84444   -360.75664   -376.58143   -495.91736   -276.53211   -488.60738    -402.1043   -243.30127   -401.86362   -477.88083   -478.57378   -413.00416   -402.08548   -306.95174   -443.08004   -443.58875   -494.81957   -299.81553   -510.65629   -283.60502    -477.0564   -182.88638   -239.28293     49.59479    175.54325     712.3899    857.32315    768.29517     289.1984    -73.97761   -378.23108   -484.41327   -356.76616   -402.10156    -246.8384   -257.12117   -151.60856    264.03217    503.42357    1122.4946   1207.76992    1153.1927    740.28955     402.0713    215.99033    128.14533    -32.69432   -192.93233   -400.50556   -236.36984   -158.43026   -116.89608   -400.58407    -48.23936   -339.47305   -238.20168   -109.24255     56.66574     166.5334    415.06874     330.0591    237.67738   -151.16602   -269.11944   -256.78474   -129.57363   -153.42043   -246.71031   -228.67347   -110.70136     -37.9197   -103.55337   -245.35012   -175.87045    -288.6435   -327.37887   -403.84115   -332.20501   -393.95476   -319.81507   -278.87549    -279.9532   -250.19567    -12.57462     60.96002    110.97232     80.13753    183.03025     50.45205    -65.05001   -120.82059     18.07374     78.50088    335.12924    481.97788    449.56452    260.80489    237.66757      23.5176   -171.94075   -162.12904   -276.97985   -190.61228   -107.34188    141.50057    388.84427    945.78353   1486.67821      1660.49   1717.20868   1488.79992    853.08588    196.22875     -8.81957   -183.57127   -108.65928   -238.50554   -200.39003   -223.38047   -280.30173   -322.12473   -405.29855   -370.57896    -327.4633   -378.71574   -514.25755          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       36.50   -455.51661   -240.10635   -120.19258   -392.50535   -217.80301   -601.95609   -247.73995   -237.87146     -330.482   -528.86406   -386.34982   -116.12208   -269.82238   -309.33966   -464.03593   -266.42482   -198.51855   -273.57616   -375.74402   -181.72113   -203.23178   -396.20151   -232.77599   -311.03803    -42.90136    255.57988    894.41001     961.8075    590.66053    239.18554   -207.73891   -202.99469   -372.04626   -401.82727   -308.72372   -549.98496   -503.49135   -384.50814   -442.73819   -305.67315    -343.9231   -380.84776   -439.06329   -449.63158   -436.15761   -316.73968   -512.92098    -213.2442   -460.75987   -370.76423   -403.66593   -397.94321   -319.89372   -309.50682   -281.26414     45.75713    320.32267    593.77784      918.645    602.40882    207.29969   -129.86768   -390.19435   -377.50583   -293.29749     55.33014    -84.65065   -196.21193   -333.72926    238.36517    448.07919     933.6494   1576.77352    1151.2993    654.08775    307.91432    320.48651    359.59901   -467.50866   -316.33349   -267.61259   -119.65153   -181.38091   -294.51193   -194.72671   -525.93103   -264.24056    -205.6174   -370.65421    -31.87999    196.04822    265.39987    382.25519    -33.47062   -306.26971    -81.33544   -151.60987    -175.7714    -30.65888   -340.58966   -159.43175     -4.84806   -105.50752   -122.92077   -221.45339   -165.62567   -355.54936   -443.90209   -337.67467   -140.53428    -314.1765   -327.29915   -305.14318   -260.56714   -299.09667   -106.10517    -90.33962    181.75898      75.3994    -20.49935     -10.8299   -195.90135    -24.46417     22.18022    225.27801    325.76299    551.86549     468.2749    346.16452    279.89654   -189.59925    -29.52179   -147.20874   -192.67807   -226.73772     27.79003    101.42284    335.20056    965.32875   1369.32875    1710.3714   1741.70569   1356.58158     802.8481    335.47114      2.90697   -102.09868   -227.94152   -228.53088   -297.65711   -299.95504   -304.90832   -341.51042   -405.07215   -376.13953          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       37.00   -529.01634   -190.83881    -18.82639   -325.31015   -178.58723   -478.16476   -272.77007   -165.76921   -492.25075   -551.29178   -341.06746   -233.77618   -312.82807   -321.40428   -438.53702   -453.87617   -328.18286   -528.32474   -261.05733   -233.38656   -327.97966   -456.10131   -376.57134    -172.6754   -106.47836    265.72141    878.50429    869.72928     549.4237    240.33121   -235.55778   -313.58243   -233.70078   -443.95871   -381.89691   -385.65827   -726.52345   -490.07133   -431.29769   -446.92368   -488.87065   -514.93792   -320.73196   -278.29128   -386.40075   -418.54846   -377.24743   -393.54159   -409.26294   -365.61264   -442.73302   -281.94788   -298.95101   -259.39119     -2.56187   -141.98997    342.29714    662.16614   1028.19595    830.86833    254.47736    -27.39455   -347.36622   -317.86414   -209.99976   -193.12293   -290.46586   -281.68869   -204.98575     84.13547    466.60593   1128.74613   1224.68676   1014.97469    590.56631    640.95875    -56.72674     77.97807    -72.02891   -129.97772   -186.88522    -406.1634   -218.60287   -335.94584   -218.18518   -377.30815   -332.74213   -358.13247   -367.08778   -226.27459    149.06505    414.59039    239.55416     20.72206   -198.87913   -327.54987    -189.9749   -346.51699   -216.93818   -271.88947   -162.41367    -93.07995   -133.00151   -176.40007   -248.19416   -251.02533   -320.50155   -418.45973   -279.23502   -496.74798   -320.27484   -357.56373    -295.0309   -310.58167   -303.61749   -196.25021     10.92204    231.84052     51.05025    217.30447    -48.98155    -25.93876    -102.0532     -0.01457    106.71152    391.96318    323.69747    665.76743    456.85315    326.78395     91.98938    -91.34354   -155.93464   -247.87687    -89.50661   -188.83893      59.7145    384.95291    835.63525   1381.49406   1595.81856   1774.69138   1261.04714    703.92958    371.61232      47.5702    -47.42011    -35.51223   -336.77855   -401.25293   -311.18226   -252.14849   -444.80569          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       37.50   -695.99145   -109.58261     31.56076   -274.17847   -142.47074   -355.70774   -319.84372   -134.94063   -436.99553   -557.69376   -333.83396   -357.60878   -244.53613   -494.71268   -507.68044    -528.8562    -411.7733   -336.22969   -294.79285   -289.38734   -450.72886   -387.94921   -387.35316   -206.14423    -91.84155    277.00424    824.84821    994.86169     675.5219    293.68753   -181.00187   -341.56474   -367.98616   -465.00858   -516.73446   -270.01397   -480.94121   -406.28419   -551.27999   -431.69478   -404.99919   -339.48662   -389.84685    -358.2134   -443.28787   -369.91602    -481.0467   -296.54481    -251.2846   -537.81552   -318.60189   -328.88828   -295.53468   -525.37735   -271.36881   -137.05511    -33.28527    776.30179   1107.11634    889.69197    234.27068    -73.45377   -408.13254   -323.72186   -309.31015   -268.87599   -278.62551   -232.99282      6.58675    150.44209    471.21029   1148.21817   1137.66734    990.99955    668.30511    270.19785    179.42261    155.27436   -117.04101   -102.86615   -194.15868   -218.21343   -318.99597    -265.9374    -319.2674   -385.41003   -304.93292   -268.68134     -343.355    -51.99362    321.64725    464.89489     339.7984      86.0842    -46.30355   -107.82236     -1.60502    -41.62372   -111.75507   -219.88265   -178.10915   -159.26338    -87.09662    -91.12968    -64.83122   -280.98468   -293.40007   -392.75853   -357.12101   -237.85866   -286.99174   -244.24668   -282.15835   -178.02337   -212.67512   -187.02507    -74.64648     49.32865      61.2681    118.31248    -57.25163     -69.7842   -176.60951     43.85263    177.13594    274.00305    399.90667    692.65151    437.56357    161.52727    178.10373   -198.41004   -278.12014   -193.38942   -160.94333   -217.30233     57.00784    361.94952    788.71016   1316.15195   1837.39277   1564.62437   1439.41261    910.97605     278.8726     65.32949   -239.74743    -275.3283   -302.54892   -436.58634   -414.43067          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       38.00   -765.86976   -136.07137    -93.13014   -184.53182   -101.71289   -445.67384   -370.94784   -115.87502   -429.94767   -528.74503   -265.62055   -381.85772    -105.7769   -423.98802   -628.06819   -387.99496   -424.72155   -111.86364   -252.95607    -190.5599   -396.49504   -346.04068   -366.84267    -224.8713   -171.48249    328.07563    649.71081    885.18437    602.81152    248.43721   -210.16574   -496.40174   -446.61022   -324.65523   -380.86907   -270.85552   -182.54258   -324.78082   -354.52297   -440.45883   -466.40235   -383.16306   -466.64661   -194.00019    -124.8277    -364.1326   -593.95965   -268.95979    -411.0807   -487.84865   -328.43069   -383.49438    -474.8313   -402.34142   -448.65026    -77.65337    412.59831    821.18146    860.62724    546.07268    345.88067    -139.4383   -364.65369   -378.99612   -330.48514   -304.16407   -220.44343   -253.74208   -201.03654    270.20365     541.2574   1118.13118   1197.63283   1156.94874    772.05118    381.05751    110.57427      27.7319     549.3781    107.15014   -236.01204   -109.18561   -357.90134   -318.42856   -482.97687   -350.51562   -242.98231   -352.02961    -32.08235      2.05178     40.58415    240.19386    345.30751    277.64807     34.66786   -246.14592    -34.08779     -5.79912   -139.13908   -539.69729   -293.03272   -102.11895    -82.90392    -91.57264   -134.33833   -206.15468   -319.10478   -190.40764   -359.80891   -362.60108   -289.32967   -333.80555   -474.88683   -417.48548   -152.06562   -133.11355    -95.66214     36.87092     38.57058    -63.85241    -10.91161    -75.52362   -269.29726   -228.32203     54.90372    398.41846    469.09463     478.0992    439.90285    185.24103    -31.01851    -88.90844   -279.61034   -250.21505   -339.72452   -141.21696      55.0937    462.72935    713.93006   1370.84624   1492.70186   1436.52527   1279.67073    675.52468    301.54906   -116.78185    -95.94537   -378.65066   -399.99979          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       38.50    -526.4864    -306.6817    -316.9975    -44.67389   -207.24711    -545.8548   -443.86201   -221.44947    -400.8236   -329.67741    -255.8443   -337.32561   -209.78519   -233.03248   -458.04087   -312.54217   -389.13283   -308.12944   -459.84201   -175.44113   -393.06215   -353.85825   -463.09871   -276.69944   -234.70292    334.77739      774.997    742.45161     494.6014    164.82706   -132.31358   -433.45794   -450.44209   -382.77445   -377.31134   -209.86679    -620.2254   -382.70644   -422.99537   -349.20825    -260.4484    -193.4311   -428.27671   -339.62299   -392.71041    -346.6391   -448.69792   -371.45764   -390.51475    -294.6561   -413.85396   -456.20447   -365.22699   -115.09719   -435.27944    -77.20228    125.06664    603.46949    806.29529    446.31214    287.00769   -145.67226   -357.11245   -290.02708   -306.42957   -325.30299   -286.20789    -283.5496   -129.96227    284.56309    212.72781   1091.49138   1334.19202   1018.98139    862.17934    543.09512    425.69772    170.26089     236.2878    320.55374     50.08115   -333.60031   -367.32513   -345.00705   -240.91928   -360.50169   -358.30698   -317.52829   -189.80963    -24.15016    142.21734    210.75256    260.05374        406.3    -15.50989    192.78024    -76.99189   -157.44599   -513.40633    -357.1517   -250.07435    -116.3353    -37.95137    -87.18063   -319.74735   -319.17938    -196.9892   -384.28046   -415.02829   -326.66109   -466.20452   -355.90866   -356.27655   -327.29267    -408.9771   -141.37093   -105.72955    -66.87231    216.37546     19.74366     20.90265    -92.09864    -18.64552    -67.69102    -31.25459     30.57095    378.64154    515.77746    627.46287    161.93073    -22.14806   -257.81354   -157.89814   -116.32176   -104.94879   -126.02023    140.62191    170.96826    874.89757     1188.817   1415.65227   1563.61217   1097.36667    759.48103    216.13965    -68.68943   -380.51044          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       39.00   -302.13181   -489.90012   -526.18284    -68.96725   -405.97624    -447.5403   -503.65269   -344.83451   -280.44506   -418.02299   -288.72001   -288.37475    -131.7868   -231.94432   -156.18386   -398.25917   -413.28675   -367.72935    -496.9455    -321.0981   -471.79228   -304.24234   -502.24577   -350.00463   -242.49947    342.33346    812.06896    734.21457    592.80004    132.71266   -223.25829   -446.79088    -319.9286   -520.85775   -385.70838   -488.73272   -356.23637   -489.36101    -403.2282   -317.44281   -398.65948   -356.52756   -257.15495   -357.32992   -325.09848   -505.97581   -365.73441   -225.20797   -438.02801   -465.89727   -409.31911   -436.04817   -306.88545   -366.56149   -137.22364    -74.35982    120.45898    785.95538     722.7134     781.8353    220.58719     -2.22861   -285.45463    -287.4267    -187.2597   -193.88838   -304.42418   -155.02018   -132.67833    252.98419     429.4822   1086.26815      942.361   1042.21997    779.63748    408.45585    360.31447    134.51847    226.78542    -41.11159   -124.61763   -224.34527     -394.022   -129.14827    -346.4146   -305.82265   -425.17997   -392.15091   -221.70251     11.02067    163.00475    384.06985    423.43057    479.38264    170.52404     12.05834   -167.34929   -418.46678    -422.0637   -307.21813   -218.92071   -109.25042   -109.10019   -216.97462   -193.69886   -200.46027   -298.86951   -262.47179   -216.10647   -342.33457   -348.47602   -420.14376   -340.80037   -242.01414   -244.65092    -76.67757   -118.07252    124.27762     36.61277      62.0778    -53.05026    -91.13902   -165.02868   -118.54446       51.581    280.09554    545.48163    464.38542    348.10238    174.16808    -73.67065   -196.60357   -300.27598   -221.99704   -242.47535   -274.93329    -60.72897    306.95902    502.85371   1164.85081   1408.97412   1205.83572    973.53417    149.33041   -274.16742          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       39.50   -206.18962   -580.91835   -542.06648   -248.70931   -570.15747   -338.47726   -463.17847   -327.83625   -178.86992   -479.69402   -317.70926   -305.66268    -50.28396   -218.02842    -13.20835   -456.14586   -381.41976   -231.33977   -510.05735   -313.88615   -419.45623   -303.76896   -454.28424   -389.91068    -255.2228    268.37591     745.5982    744.27901    446.63482    155.34387   -139.05712   -350.86978    -350.4607   -275.12926   -399.37075   -526.82404   -486.40245   -174.56407   -446.84117   -432.86502    -369.9746   -380.73419   -431.51717   -300.04217   -379.50186   -344.71371   -363.87503    -448.3615   -256.18932   -257.13253   -413.68131   -222.17129   -373.21569    -342.8569   -295.22123    -24.24102    219.42412    610.77107    939.40158    680.36982    481.30663     36.58635   -419.64191   -429.88956   -334.82504   -586.37075   -311.06454   -203.52415    -108.2515    284.61814    523.17692   1029.55916   1183.54217    892.68846    705.87874    416.39363    344.39662    169.09035    239.70074      89.3733   -394.21219   -465.25539   -571.27338   -434.25485   -322.02789   -171.25483   -314.34142   -184.77595    -94.78929    -24.32578    258.67095    467.82305    636.55079    307.46506    287.67923    -131.6729   -173.84145    -19.24956   -393.26909   -386.14683   -211.95756   -167.75994   -184.80984   -257.56427   -173.45756   -288.76921   -339.97893   -286.97937   -306.19442   -462.19284   -277.39966   -351.89317   -394.92466   -449.82276   -195.17929   -229.22223    -29.67335     98.19989    181.49176     22.20408    -24.81706   -188.25067      5.39423    -45.49521    132.37798    138.18353    519.04604    623.41458    539.85241     98.79201    108.22823   -178.75939   -304.45564   -200.27001   -256.97495   -208.64127   -105.87329    319.65029    544.67998   1125.93244   1054.20138    838.52257     133.0149          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       40.00    -232.0231   -378.09893   -496.77458   -400.79357   -661.57938   -363.99411   -402.60319   -329.40068     -287.841    -252.4174   -422.15159   -228.61717   -293.61588   -160.61034   -175.73088   -401.42384    -211.4241   -438.89082   -415.04625   -317.07839   -391.28408      6.98687   -360.30879   -429.62541   -119.12243    331.14967    804.08919    713.85189    480.29491    168.62224     60.39686   -523.54056   -394.45047   -415.34164   -453.43622   -430.74753   -392.34481   -303.30809   -465.19922    -322.8825   -359.73312    -199.7105   -418.64876   -355.58424   -403.70448   -268.54553   -440.09199    -461.5202   -434.00077   -487.28572   -394.55765   -175.58568   -360.46279   -213.94717    -183.7544   -210.56855    434.74163    441.58055    883.78864    564.59619    298.57559    -86.89647   -243.81716   -444.10381   -290.93672     -53.4532   -279.07753   -392.24165   -310.72663    248.80187    564.91245   1236.67465   1281.38686   1117.52982    629.77407    438.77836    297.26858    139.50203     85.13451   -166.14246   -138.03677   -249.16623   -497.57882   -235.57749   -372.26374   -206.53795   -240.12688   -207.96242   -170.18816     11.84487    181.16779    411.74406    637.87523    737.97935    223.21301    -50.75321   -335.74165   -274.59619   -347.32606   -289.97983   -124.49964   -256.43224    -44.69505    -60.87698    -23.37836   -351.16723   -336.64558    -362.6036    -385.1026   -406.51749   -298.14548   -299.53174   -352.53585   -277.01962   -138.19823   -200.97857   -109.68814    -14.17762    257.05431    151.45416     28.66364    -23.11989   -129.20144    -94.70793      64.5809    301.15431     379.2095    523.66678    430.02806    242.29774     17.79265   -203.48422   -109.95732   -186.27924   -299.37832   -252.81864   -170.06772    151.05339     516.9934    506.87547    350.79648          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       40.50   -210.59868   -177.89271   -355.38475   -465.41878   -610.19051   -252.63671   -359.01132   -365.63036   -418.09143   -278.80198   -453.03387   -215.32779   -378.64198     -226.652   -342.27464    -340.4558   -279.57558   -458.09078   -293.83377   -355.86643   -435.66029   -264.57011   -294.23941   -444.96062    -40.95057    328.09248    552.17583    834.56411    545.15206    262.51796    -13.41262   -380.92325   -348.80794   -277.55862   -311.96771   -470.01605   -621.53864   -408.58506   -213.90452   -218.48794   -422.49214   -260.47545    -354.8352   -335.04424   -427.85966   -488.17134    -149.7936   -300.64953   -362.60459   -272.80246    -354.9043   -443.79047   -395.08209   -363.47117   -242.15575   -179.63312    207.80154    478.76236    752.80056    292.09774    175.17888    -102.9368   -275.67874   -401.20007   -266.05225   -185.79858   -264.70666   -310.25979   -128.33241    208.89903    335.15418     1392.441   1212.39311   1013.64493    862.75532    445.27816    150.31319    100.99127    -65.78349    -142.3354    -258.2695   -600.20655    -431.0037   -334.97652   -274.61961   -277.31408   -407.17314   -281.42605   -170.95903    -34.53324    209.52974    355.77521    546.70735    189.74031    182.68677   -205.08604   -408.91411   -403.78788   -324.47131   -333.57725   -269.80785   -121.35804   -116.42371   -184.80762   -212.89544    -368.8605   -375.48764   -177.13777   -303.78073   -534.43528   -299.66815   -440.95708   -332.57505   -365.63756   -238.12422   -218.50321   -110.72434     50.54369     22.23036    179.31751    -66.32869    -36.31529   -131.53673    -72.40869    141.22882      7.74955    479.46314    515.81029    343.00066    335.64572    179.10364    -58.69651   -327.69611   -304.69627   -252.14153   -304.58222    -94.99033    -36.46738     74.48007          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       41.00   -179.06578     17.09834   -362.69266   -500.20268   -496.23493   -121.94926   -414.34443   -259.70402   -399.12987   -612.30517    -337.8354    -456.2102   -409.88741   -230.64642   -268.82063   -391.85859   -274.47985   -620.11124   -305.32523   -296.86322   -414.61163   -271.55232   -390.52775   -269.57807   -133.47062    311.64523      376.808    707.77324    514.17057    141.00611   -379.16544   -372.30887   -348.06816   -182.86578   -343.17386   -451.29934   -334.80015    -470.6322   -369.17438   -436.41851   -232.59931   -348.58209   -401.96092   -294.51931    -379.4105   -400.61369   -291.61883   -402.31086   -301.58038   -496.94589    -515.1996   -328.01941   -379.79944   -177.30136   -161.17563    -33.28139     244.1104    448.72141    457.23167    685.88271    524.48824     11.29884   -233.89605   -208.77904   -254.38097   -163.23871   -408.37507    -345.1829   -215.60434    157.28846    542.60616   1182.70073   1240.72244    847.23528    685.68922    401.63367    262.60921    246.16877    -32.94662   -232.42383    -38.46025   -228.70797   -409.84554   -221.15563   -152.39756   -485.75895   -324.52127    -253.0357   -207.06596   -187.99993     24.35028    222.82175    481.87057    170.95042     10.81977   -295.30592   -415.46997   -448.45682   -397.90211   -346.22585   -171.24739   -168.73682   -112.12803   -205.39148    -65.42935   -182.05573   -255.40655   -401.33005   -221.22218   -340.35443   -316.18039    -466.5153   -131.28992   -399.52713   -299.50821   -176.25359     11.54906    149.62338      36.7082     68.33955    -48.95847   -134.23649   -301.19197    -44.32762     41.21568    278.22884    441.57087    455.29155     381.6369    141.91313     76.94026       -169.1   -298.13212    -240.0788   -277.33474   -361.22825    -330.6258          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       41.50   -137.38049    -32.79509   -358.55174   -536.83912   -379.02071   -251.20315   -507.53303   -276.00375   -261.51251   -702.80401   -311.88996   -446.61201   -310.40355     -239.554   -321.31773   -414.30283   -219.60756   -429.04188   -281.49659   -349.00785   -378.65125   -272.18182   -409.25456   -114.65555    -52.27561    420.05121    412.04426    618.98817    428.36307    144.86799   -115.39726   -573.02719   -327.35733   -308.79519    -192.4934   -561.74095    -326.3166   -304.25942   -432.14755    -448.2646   -248.38754    -297.9172   -376.07285   -615.20881   -316.11172   -331.93603   -465.97029   -394.79266   -307.64393   -325.35034   -517.24356   -357.49546    -501.0673   -321.86631   -238.16055     -27.5061    331.48171    672.02197    763.05066    696.90754     305.0513    -92.47584   -205.20217   -186.01489   -333.30813   -390.97984   -345.55184   -211.42809    -79.63419     22.77036    567.11612   1015.87794   1110.98907   1083.57145    745.43232    335.94575    239.12887    -39.74551     41.27354   -271.27656   -128.32401   -433.74954   -300.94132   -328.62751   -427.96555   -251.11106   -458.52718   -146.49597   -277.39288    -24.67939    136.76629    337.50832    345.81209       278.18   -176.35966    -215.5667   -356.85013   -294.50179   -546.64947   -398.51625    -253.5849   -183.65028    -80.93909   -126.60418   -174.82237   -257.49614   -393.45408   -314.88917   -378.21832   -389.30819   -438.67805   -332.08433    -338.6786   -297.46121   -295.37699   -226.19687    -98.44217    190.02262     75.69024     21.48962     45.04492    -12.89731   -155.07088   -180.71367    -24.84066    186.97839    367.23458    418.19286    504.16362    327.87661    -50.39192   -138.22859   -245.43451   -232.31362   -433.40435          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       42.00   -155.25211   -102.26247   -271.28276    -476.3255   -359.00713   -372.82772   -553.79558   -413.59734    -155.7987   -613.82439   -373.66617   -310.98994   -160.21936   -348.13517   -503.06608   -290.98702   -342.80273   -436.63502   -202.01342   -462.23598   -473.90501   -356.47097   -452.04867     45.32072     89.70598    294.21678    482.45865    657.66919    541.50212      14.4432    -42.28213   -402.03418   -550.40968   -373.67247   -216.21513    -347.1125    -308.1053   -259.12114   -231.74358   -378.41264   -225.46917   -365.67868   -411.48879   -567.58223   -214.83539   -255.63004   -325.49163   -356.87745   -309.37615   -296.22074   -375.20901   -377.52797   -297.14868   -239.62542   -229.72624     -4.29079    235.94009    529.62885    934.74284    782.20711    120.88113   -147.61072   -211.83494   -458.54787   -299.53567   -445.87907   -357.87153   -310.28429   -312.55379    -15.08514    509.67041   1024.16776   1276.62156    920.05069    697.22261    437.67808    366.98072     394.2561     -32.6566   -135.83079   -357.20272   -381.97485   -186.56642   -351.50324   -307.51401   -430.09987   -298.55353   -162.83706   -338.64756     55.22263    202.97621    221.59263    287.80926    454.75265    -91.32313   -394.84078   -444.56964   -460.92623   -451.28189   -120.85079   -441.03958    -206.9737   -239.90772   -341.45352   -290.16035   -235.65336   -203.77663   -368.80685    -447.8555   -273.27415   -504.89683   -324.83065   -203.56561   -294.09172   -287.26973   -145.65466     -3.53916     97.56258    147.20972     41.29154     63.59754     -2.56517   -170.54571   -167.61481    -67.09031    203.88446    395.81282    465.85537    323.08974    224.72389    -43.67578    -90.38257   -330.50485   -402.07651          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       42.50   -257.27299   -113.20548   -296.90975   -309.71365   -380.39711   -371.61526   -422.38651   -515.93355   -300.94848    -461.6755   -438.75974   -273.84033   -216.54807   -368.63134    -357.3183   -342.36666   -391.90183    -469.1446   -305.98371   -529.04368   -254.40153   -312.07228   -452.18294    -93.81107    195.38715    353.26445    372.14339    714.33366    446.76276    -19.11994   -188.05689   -167.79123   -426.00493   -432.11705   -331.23509   -385.61296   -321.08428   -280.24559   -432.82033   -318.25284   -325.00609   -334.90197   -590.82163   -473.05641   -503.94685   -493.92973   -303.88565   -492.24664   -300.14452   -415.26917   -297.24027   -301.64923   -309.40146   -259.54538   -407.53522     14.37276    243.02892    642.95781    568.10475    778.84245    273.72327   -128.62771   -126.64336    -314.3654   -255.37151   -538.18156   -317.21027   -147.72257   -100.17758      1.22448    645.13074     919.7963    1081.6777    926.64272     831.0779    403.22871    186.30411    126.84328   -335.19352   -178.98062   -540.72144   -603.61057   -328.85441   -230.40795   -412.22298   -330.50158   -308.92411   -335.91296   -239.08823    -78.27094     23.56193     378.1551    366.64017    207.14956     72.08305   -339.16907   -395.37339   -458.23953   -449.24756   -398.98066   -283.12736   -133.74618   -121.90751   -141.77738   -254.82131   -162.41616   -261.81997   -523.75046    -125.9902   -342.02225   -357.37113   -314.88796   -271.91177   -355.44598   -249.53202   -276.91813    -34.89827      21.1504    177.55488      2.15372    -81.10184    -67.04893    -73.11567    -68.05639   -138.93018    222.52892    225.19049    426.34265    346.91651    151.09659   -151.16157   -380.21835          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       43.00   -293.47321    -65.46794    -236.6037   -112.31051   -347.34295   -234.05584   -338.35804     -387.958   -616.72261   -366.97878   -466.76584    -350.3144   -408.42882    -292.1569   -342.40573   -367.69173   -374.04067   -111.62954   -400.81266   -394.43629   -281.03807   -219.91208   -493.36672   -227.83626     72.66726     478.6838    569.74166    554.23858    353.16531     40.98299    -84.25414    -19.60346   -442.07001   -415.88626   -280.68397    -329.0861   -325.35784   -538.17548   -329.23666   -530.97466   -456.91119   -399.74469   -351.48919   -544.97582   -435.01901   -462.17323   -377.35295   -330.03842    -530.5149   -239.86308   -346.52347   -255.65768   -358.87402   -339.03439   -137.52145   -156.11314      261.163    678.03317    658.30618    486.45599    335.71629    -21.42941   -287.85283   -385.93007   -396.16358   -468.31937    -83.53803   -125.32922   -102.97396     96.09951    569.52753    1077.1442    1311.6151   1152.18744     764.6333    334.52118     52.61863    -63.48339   -283.46518   -308.40931   -248.77055   -605.62983   -321.22257   -484.06592    -297.9554   -153.75504   -330.83516   -352.26677   -288.67675   -129.59973    441.77053    301.20031     52.86836    193.01814     145.2662   -364.04876   -393.34553   -396.65149   -322.22214    -402.9898   -356.70593   -345.58672      2.41187   -105.71829   -142.09109    -424.4972   -331.93583   -373.89218   -345.41458   -180.68061   -392.28784   -336.15359   -447.64734    -414.9427   -261.45284   -202.40845    -32.25384    -49.13888     51.19758      5.98318     18.14276   -221.37077   -213.87217   -168.45749    -98.13794     78.23391     341.3433    281.46254    195.19645    -54.22869   -308.18923          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       43.50   -269.73358   -134.35824   -184.16651   -125.94843   -194.74209   -243.15897   -257.35283   -187.94305   -932.12502   -256.95936   -396.70783    -170.2547   -319.62139   -338.40459   -469.80715   -210.14796   -450.36273    -63.47573   -352.46805   -325.64319   -303.20136   -450.45759   -279.65585   -169.48822     -2.37288    373.36357    701.84029    603.10533    392.86265     25.03237    -94.87791   -301.29411   -253.23642   -408.67376   -219.43493   -329.32808   -538.66926   -354.18305   -520.79441   -459.51248   -369.27484   -502.62411   -373.25633   -392.86827    -571.2198   -333.38095   -419.35839   -284.49798   -494.72429   -143.93935    -279.0907    -372.4365   -249.37914   -255.71154   -422.99379    -18.83649    341.96105    479.93397    544.77445    633.91102    236.53888    146.03124    -183.9739    -337.0025   -388.77736   -424.92125   -430.86347   -243.46581   -468.72162      7.84736    675.76978    987.91106   1196.45446     878.6928    641.98563    379.15639    234.56981     66.26618     27.43225   -532.25581   -275.87552    -252.0978   -395.15079   -374.36729   -332.48634    -335.0663   -388.09887   -423.06334   -150.66636   -179.29994    275.00387    171.88882    345.95593      4.48178     -92.0885   -298.37783     -79.7681     -262.177   -448.09038   -348.15334   -240.10519   -301.38244   -161.60391    -35.12714   -234.06549   -234.27884    -302.6493   -236.53525   -221.14368   -250.24938   -298.32085   -396.07796   -376.83032   -146.78208   -263.40673   -211.69246   -146.23033     55.39265     21.71662    125.14244   -200.83318   -148.29834    -81.28132   -168.98999    -86.33642     87.33484    195.61426     267.3332     16.68908          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       44.00   -173.79752   -191.07047   -120.20708   -150.99157    -30.38679   -380.03306   -206.84106    -106.8135   -533.01774   -177.78705   -311.11783   -160.59818   -305.04705   -443.47334   -507.82126    -103.1915   -371.58106   -258.43169   -394.03879   -388.22135   -447.87031   -417.71945   -134.95098     -73.7007   -110.81953     91.76503    753.60083    633.10141    441.50085    -19.86623   -277.96805   -354.58324   -293.07299   -325.00889    -369.4895   -326.39587   -402.98417   -513.92183   -556.39166   -363.38809   -404.25673   -563.85678   -451.80663   -430.50401   -509.20099   -321.50495   -333.27806   -216.16788   -384.43732    -287.8003    -359.3655   -444.09435   -424.62299   -312.35992   -255.68827      4.86984    435.47507    373.98752    683.02073    579.93304    237.86656      -55.711   -146.57365   -335.18918   -254.36299   -271.93802    -323.6804   -159.82219   -199.04675     90.95221    676.64824   1050.38018   1272.62275   1183.28543    809.49601    391.33646    360.67299     11.00417   -318.96668   -209.90591   -446.44072   -333.11992    -393.0802   -283.22463   -373.56048   -320.23677   -182.12388   -251.98638   -273.44483    -40.61934    297.29858     81.77852    285.53438    197.34388     84.35906   -161.75472   -105.91279   -481.44789   -486.62712   -346.47198   -178.27066   -304.78548   -359.17722   -155.20813     -167.181   -253.68591   -425.88214   -205.58564   -290.75496   -227.25896   -397.21657    -235.9898    -411.3705   -380.25018    -57.40988   -195.75665   -209.56405    -31.54283    107.08461    -63.32294    -86.21225     45.34838   -197.93954    -236.4679   -223.37954     73.98606     -60.2258   -249.58151          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       44.50   -243.63091   -214.95068   -123.20228   -111.02816     15.82521   -406.45379   -134.60679   -201.04671   -305.94533    -246.9571   -290.49628   -499.64494   -344.05602   -536.14774   -435.75733     -178.082   -225.86888   -278.11021    -478.1371    -208.3963   -235.65906    -675.3925   -264.75506    -37.21404    -96.08907    189.91039    610.55002    717.35072    621.98767    135.41488   -317.33945   -345.95333   -310.03492   -349.49156   -386.02366    -425.3568   -286.21739   -645.68044   -437.84234   -513.43724   -400.50746   -423.30836   -331.19264   -518.27402   -460.36094   -371.04941   -273.03874   -445.67454   -362.90154   -471.61724    -383.7375   -323.02448   -494.10532   -267.48903    -362.0407    -49.08614    -19.07175    320.27688    458.65429    761.57828    220.83965   -176.61372   -306.96633   -392.56997   -119.76351   -220.73531   -198.25309   -315.50655   -117.79319    162.87579    324.08544   1011.31882   1062.21178   1073.88672    672.63225    353.68941     82.03586     86.93753   -178.63543   -363.65548     -170.713   -404.75571   -383.98342   -390.31155   -332.81997   -394.51451     -529.836   -360.89936     -90.5317    -42.17048    -54.74759    239.65102    262.30581     55.21925     62.39136   -102.96247   -283.16155    -440.8861   -188.45593   -329.38833   -391.75265   -149.26105    -115.3629   -280.93635    -127.6539    -186.5723   -382.30436   -343.62597   -352.05039   -259.64444     -375.036   -410.32585   -481.81926   -351.60105    -323.4568   -225.14864   -105.83425     67.66207     51.85194    119.94695   -119.43218   -123.01373   -236.45098   -391.11864   -124.12603   -176.19876          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       45.00   -340.01294   -212.19033   -240.27688    -36.30822   -233.70894    -385.4739   -133.51851    -280.8291   -357.52041   -384.10135    -369.3599   -657.42187    -348.6895   -626.79799   -288.06945   -246.36155   -283.84601   -303.31588   -546.53888   -211.12983   -131.18113   -315.14551   -356.82157   -120.59528     17.22782    406.53747    573.90324    636.36315    413.06895    198.50946   -182.32138   -312.33923   -158.51537   -378.63444   -133.81129   -552.88646   -273.80973   -364.01973   -377.29087   -317.49046   -500.26145   -382.21056   -235.45783   -448.93289   -271.49917   -332.06331   -440.91156   -315.32074   -417.63684   -375.81488   -338.99207   -316.20926   -260.47947    -340.0163   -445.47178   -149.45518    131.73954    422.52774    530.44683    587.65288    269.27511    -22.23572   -104.74285   -369.67684   -303.71597   -348.68733    -238.1607   -296.71694    -51.87857     89.13724    419.75132     1042.538   1209.04541   1311.76964   1049.02771    209.70209    199.98594    123.08857   -103.43575   -295.12849   -336.77016   -395.63298   -230.08194   -364.40174   -332.03585   -163.47427   -250.82922   -329.48156   -336.36192    -46.83935    150.11613      178.756    318.06464    389.93097    223.15981   -142.12487    -67.02881   -364.38905    -258.6751   -277.47831    -274.4711   -332.87661   -140.66722    -232.2843   -153.70326   -347.74316   -324.64739   -240.76317   -332.47401   -216.70796   -356.22995    -390.1947   -392.29304   -375.37643   -420.79375   -347.07949   -270.43649   -117.13577    -53.25783   -106.39753    -90.75236    -182.0311   -138.90676   -291.29539   -218.77606          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       45.50   -380.08289    -302.2626   -387.88615   -196.39784   -442.17245   -388.18211   -249.86023   -274.01759    -386.9383   -505.91396   -462.48804    -490.8849   -500.37437   -468.59498   -123.48561   -257.17949   -329.64144   -471.82185   -406.89849   -292.89228   -262.72876   -116.63692   -467.37324    -84.30689      -5.2884     536.4725    773.04035    640.32914    240.94367    180.46266    -341.9047   -345.71798   -332.30733   -529.46193    -342.3456   -594.91611   -224.21434   -421.44035   -412.69262   -398.67239    -440.8209   -263.99409   -390.41727    -344.1366   -417.60795   -366.65162   -364.81271   -302.38464   -472.85814   -235.28477   -306.85704   -498.52911    -339.6941    -167.7244   -417.08063   -166.36761    132.10679    454.39239    677.05475    399.14646    225.14244    -93.07039    -234.9803   -277.23366   -254.19907   -337.77339   -369.69874   -320.31664   -156.36354    196.53728    566.63117    850.46184   1171.59395     904.6284    835.25115    372.89523    287.79169     -1.47622    -19.90466   -186.13173   -320.74865   -345.75342   -456.13086   -321.90302   -261.49681   -409.52883   -293.17606   -322.83667   -199.29352    -75.25387     87.63844     318.3807    341.23024    243.44492      90.5682    -65.66896   -156.35763   -315.16733   -368.59935   -284.69383   -384.04169   -157.26754    -130.3192    -16.10744   -214.70986   -336.84535   -467.27403   -198.76326   -426.96395   -304.49415   -369.62297    -420.8508   -273.32471   -310.36357   -530.93777   -140.03754   -149.05615   -228.46436     16.31631   -144.28314      6.95885   -336.80715   -264.62568          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       46.00   -404.52694   -383.02373   -566.46281   -381.95188   -547.00896   -440.71535   -287.24502   -229.45432   -560.90495   -490.63845   -388.06463   -320.03521   -540.29503   -327.47344   -163.15401   -315.89963   -391.69677   -508.25344   -306.38598   -335.02439    -367.2704    -389.3979   -511.83427    -17.58702    145.00271    371.32304    570.21454     537.1738    318.62033    106.78373   -324.46304   -448.99645   -414.86678   -370.51704   -359.84234   -405.17889   -424.55231   -326.36785   -419.24118   -553.84997   -218.22034   -353.87819    -134.7139   -153.24166   -336.44446   -255.88017   -297.48362   -410.70334   -312.23731   -322.46415   -286.78953   -507.77987   -361.04452   -348.17974   -299.54879      49.2807     51.75685    368.12609    707.43915    216.42441     94.00178    116.15258   -251.17365   -242.18949   -328.46386   -317.71475   -391.62076   -319.52596    -37.69349    359.23646    507.21428    785.55447   1287.30652    942.13065    775.20397    405.58614    394.93238     79.23509     -9.55953   -308.50916   -126.80684   -228.74637   -244.71177    -281.3455   -262.47661   -290.27211   -280.46359   -255.44548   -225.71698      1.42423    114.17533       9.9338     327.0481    387.14466     126.8895    -15.06073    -45.55744   -210.35798   -285.78761   -310.37557   -337.25701   -205.91065   -116.16666    -98.58739   -142.82204   -181.29256   -306.32323   -377.85592   -352.05518   -462.64313   -439.33218   -244.14604   -452.90874   -354.07394   -367.98948   -170.17789   -217.82252   -119.30043   -123.14282    -48.19969    -93.88172   -225.47496          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       46.50   -378.78045   -454.47292   -493.55856   -408.53804   -416.01423   -418.79846   -246.63508   -244.81152   -745.91018   -527.78147   -254.52096   -387.46796   -354.21437   -385.43374    -128.1546    -357.5032    -426.1992   -455.68786    -272.7318   -393.79975   -392.46029   -659.89707   -438.45001    -271.9258     65.78844    216.94643    575.41828    514.19796    384.84949     70.06221   -153.17837   -331.45708   -406.84386   -467.91358   -409.31589   -282.55591   -258.00369   -446.39962   -452.63596   -662.58285   -293.77471    -152.1001   -283.52865   -448.74087   -393.33925   -334.67408   -167.78326   -346.77308   -404.78259   -273.81178   -277.52143   -316.95366   -355.62784   -397.97379   -216.50383   -133.49547    233.04876    306.66815    638.93893    562.48665     93.25621   -138.37121   -400.63202   -443.83094   -359.47159   -366.72649   -274.88409   -437.68468   -102.50679    255.65074    503.17538    846.83509   1152.16342   1222.72118    973.79117    433.73722    184.36985    141.09058     11.50609   -220.49019   -159.64565   -424.25938   -257.66823    -593.8791   -151.50241   -272.73239   -300.16534   -345.23719    -18.88421     75.87258    -17.16134    395.43473    454.59407    155.72174    107.77883     78.24226    -291.3285   -512.71153   -408.82295   -428.81212   -338.40773   -309.86344   -264.77337   -230.00812   -261.93516   -186.59588   -238.14242   -180.76899   -383.44045   -405.43292   -566.23195   -317.36031    -330.9236   -316.66107    -214.6755    -258.2503   -186.62694   -133.25121   -133.37849    -50.98121   -284.15674          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       47.00   -389.43593   -490.89776   -399.67352   -438.88053   -326.51654   -552.29157   -275.04163   -298.33363   -496.40375   -513.98133   -201.41032   -338.21359    -88.21662   -334.00121   -123.11839   -433.75354   -195.67502   -397.37203   -214.42303   -453.65192   -513.65998   -418.62633   -546.27889   -191.95931   -238.02405    172.00953    671.69488    544.34495    447.70149    109.17558    -82.26544   -199.33022   -471.52344   -237.76626   -278.87749    -489.7006   -225.96774   -277.63608   -395.15412   -262.83408   -412.77142   -277.91681   -217.76463   -322.93675   -282.13327   -436.32831   -214.36218   -287.26154   -241.55485   -406.27603   -306.80309   -421.01243   -385.70174   -489.28292   -356.68075   -105.18982    154.38581    352.10087    625.48795    466.36853    103.16835   -147.73249   -190.58529   -274.70976   -348.56848   -357.33452   -244.56177   -217.44884   -158.75803    -24.60073    346.87867   1067.37549   1121.21972    985.65517    829.29389    585.97964    -25.82771    -24.29667     24.48722    -180.0279   -130.41542   -340.69618   -385.54898   -457.49738   -339.73954   -486.82321   -272.22458   -385.82278    -98.14599    -80.43825    238.58603     94.46418    389.84893    460.22925    -92.62559   -159.80408   -129.76032   -381.45295   -323.81555   -454.89265   -228.67434   -254.30454   -188.10956   -122.41115   -160.07015   -219.37154   -464.62809   -274.05081   -410.63012   -378.95067   -407.25298   -357.39202   -423.52386   -329.00731   -214.91974   -251.14813   -206.35142   -122.96278   -185.93925   -330.23017          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       47.50   -490.81496   -468.64315    -292.1993   -402.58728   -322.67741   -623.10886   -344.03637   -300.16375   -287.70271   -294.95319   -194.19853   -344.80679   -223.75608     -93.7156   -200.05979    -523.4399    -99.86695   -259.63536   -407.98388   -345.35874   -439.95845   -206.43537   -367.34971   -148.07663   -134.22167    435.14078    721.43536    677.29066    347.08815    183.12863    -98.94334   -225.43917   -363.19242   -342.97539   -152.20184   -519.96332   -245.15391   -491.87691   -348.92576   -303.68074   -465.72105   -352.47592   -480.07803   -319.84563   -377.65917   -613.71111   -224.54794   -263.29842   -418.82223   -306.21527   -322.35704   -555.07307   -454.10864   -270.40435   -326.97087   -124.36317    294.47543    314.13846    536.54051    437.57591    187.82352   -118.63645   -352.88907   -208.85367   -203.58698   -272.82314   -436.26368   -382.84026   -259.87514     13.70412    770.08819   1041.93178   1254.36369   1016.61412    677.97773    202.38953    164.04077    185.50107    -59.28707    -230.5463    -310.9534   -416.82242   -487.10016   -415.54711   -535.79476   -373.16505   -464.99007   -288.23133   -375.64982     -68.9816    106.73928    216.78541    418.55067     217.7643    154.26495     86.71896   -321.47217    -277.0684   -384.22174   -392.53305   -313.05784   -230.26501    -98.05688   -238.71139   -185.35607   -154.18938   -346.35429   -259.42921    -383.3549   -362.05611   -437.81864   -398.73298   -341.71231   -514.74328   -327.98698    -301.7716   -205.05271   -367.88583          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       48.00   -642.34731   -522.41533   -248.53064   -344.97888   -326.45207   -522.31946   -402.08229   -314.16239   -189.23995   -254.46329    -97.51017   -466.51039   -242.73728   -213.27005   -201.34587   -456.08269   -241.31339   -121.50436     -460.113   -212.59901   -546.29332   -160.03465   -314.59804   -184.91649    -16.22857    441.98061    468.67122    527.26261    241.56604    -12.81459    -85.03878    -104.6953   -138.05089    -359.5023   -375.08336    -212.2352    -377.1452   -606.54761   -319.55205   -440.30706    -397.1183   -494.75742   -540.21989   -276.65257   -414.50153   -394.99155   -295.28647   -315.84515   -511.22506    -311.0296   -111.09786   -527.02211   -310.06716   -394.14626   -226.62516    -60.27345    202.86584    421.31378    453.45637    316.61279    161.03998   -174.87196    -449.4946   -164.31232   -370.58495   -347.32002   -333.46624   -239.93795    -251.6988     74.59984    491.43609    884.76329   1042.38882   1243.05369    651.71383    311.68262    133.45674     33.43092   -317.67426   -214.40911   -234.59177   -536.90064   -323.10494   -486.61579   -213.49911   -347.08029   -251.14318   -348.88092   -279.51152     97.01958    174.72361    365.87213    538.94711    365.17012     49.99916      -5.5667   -233.35952    -83.68246   -267.29582   -310.10009   -188.03177   -223.37216   -215.66033   -276.79827   -392.83892   -209.41855   -145.62219   -364.71764   -313.53049   -116.26065   -364.09362   -331.74391   -217.22801   -434.29398   -493.20326   -482.04064    -381.5738          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       48.50   -570.91517   -652.02128   -285.03161   -384.18113   -312.11456   -455.18303   -315.65329   -415.35049   -304.50181   -381.08521    -103.9985   -636.50076   -261.95719    -91.21668   -250.36809   -317.45776   -179.04035   -226.45548   -377.06917    -52.34663   -461.31071   -108.77533   -230.12543     -299.192   -164.13498     66.80097    417.63747    363.92771    147.96963   -206.15381   -137.63159     -170.777     -98.5487   -275.26175   -274.47028   -227.87382   -488.43153   -351.57973   -389.02982   -404.65261    -417.9655   -592.08066   -357.33372   -408.52075   -265.64011   -275.29215   -407.60903   -459.65611   -576.36926   -504.38027   -369.05511   -404.22851   -454.21615   -194.81172   -279.83899   -154.54925    127.30875    567.04425     607.7722    430.27807     -1.52617   -113.04985   -337.66118   -133.20649   -389.38579   -323.49793   -389.83222   -382.81766      3.25237      60.7299    371.87305   1045.99429   1077.79575    972.99685     879.2088    447.32078     90.64348   -120.81506     91.78127   -265.48145   -268.97037   -257.77907   -493.23688   -440.89875   -361.74306     -9.65605    -410.2593   -365.38557    -229.0575     47.12641    -48.32425    207.16231    429.54089    365.59446    237.76302   -164.82283    -364.9073   -293.28114   -309.54001   -337.38538   -306.19107   -369.96827   -180.43916   -427.06197   -297.71968    -72.96294    -292.6537   -368.96237   -362.75846   -211.24433    -308.1138   -532.21494   -387.65589   -410.37713   -532.68819   -415.99871          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       49.00    -487.6759   -680.54839   -315.32613   -556.04863   -284.21596   -450.77043   -347.05384   -490.47223   -460.81981   -295.89394   -202.98941    -638.4897   -278.83675    -201.0812   -203.40294   -350.83616   -116.06191   -280.92841   -300.31709   -282.14166   -191.21777     -90.4555    -47.36876   -302.43672   -122.03331     85.68984    369.53791    445.57393    130.74953   -192.27952   -258.92603   -259.16918   -116.54737   -368.98504   -316.48476   -258.08441   -290.80036   -374.98569   -445.29465   -723.91303   -488.33707   -298.69694   -427.96139   -376.71943   -302.69724   -464.36141   -297.65094    -431.4998   -435.87516   -420.57873   -289.21658   -316.56689   -141.41717   -357.07924   -135.03881   -176.02783     175.2151    311.14561    679.68688    449.58484     416.3203   -117.39876   -257.63064   -208.85169   -156.30503   -267.76196   -319.95847   -138.79678    -68.25169    320.41089    570.15247   1147.67219   1154.25661   1127.64854    967.30388    333.96502    176.97405    -84.40979    -66.82421   -233.59969   -220.62874   -238.20442    -447.2092   -349.72521    -482.1222   -275.41133   -467.65097   -314.03327    -100.7693    -25.66377    165.18048     40.25818    403.81067    228.32436     30.25433   -199.76112   -318.54137   -397.68106   -568.34746   -263.98136   -363.44145   -340.39737   -351.90289   -263.20872   -212.68353   -262.27813    -250.3256   -314.82222   -328.98345   -306.59435   -405.87522   -431.76183   -353.01068   -330.60909   -321.80486          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       49.50    -443.1569   -402.84293   -334.79863   -562.76333   -222.49495   -348.69319   -515.93882   -514.47427   -581.81988    -216.8022     -224.431   -443.68726     -336.236   -418.84746   -486.06861   -411.86858   -233.12786   -324.04727   -274.50612   -528.16251   -322.54312   -235.66402     -32.9343   -243.01055     24.27157    147.27925    402.49917     715.0317    110.06303    -44.67463   -208.50052   -391.89729   -357.16821   -394.71964   -235.80037    -348.5984   -371.08343   -426.79316   -373.24446   -596.18688   -262.19097   -307.61157   -399.33696   -568.06954   -568.55778   -413.82359   -309.90285   -376.91953   -252.21946   -310.80785   -516.25583   -395.78438   -275.80878   -242.39354   -499.22729   -151.32448    240.93145    354.46763    367.82428    526.60388    132.28294   -229.44398   -253.37375   -272.64594    -161.0919   -242.00151   -408.07382   -231.54227   -303.07742    277.71457    286.28811   1018.23637   1032.44109   1050.33908    339.82801    351.64536    115.03933     -97.3077   -218.58462   -218.16551    -424.6509   -426.95121   -295.76312   -354.04498   -351.50159   -250.18357   -391.85969   -284.63052   -216.64093   -108.82334    197.74854    270.81143    420.32332    298.03267    169.38064    104.08229   -165.27929   -248.90289   -292.48576   -366.83217    -320.4239   -451.70753   -139.42544    -218.5388   -207.64808   -251.55353   -434.70131    -344.5416   -377.23454   -349.04516   -529.53999   -486.79576   -405.64009   -413.72985          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       50.00   -437.21057   -214.00269   -371.10561   -396.39691   -205.80053   -201.11504   -586.73507   -515.59958   -468.99229   -282.02956   -124.42635    -267.2782   -423.48449   -474.26319   -694.74563   -376.41047    -292.0836   -594.55663   -235.77026   -408.26671   -457.59053   -285.18067   -237.68496   -152.91458     95.58575     41.69066    546.96583     539.5324    204.75168      19.8957     31.81203   -351.77956   -178.95536   -495.43454   -109.92406   -242.20984   -404.58444    -245.1685   -344.42022    -318.7281   -519.32641   -261.76715   -427.38767   -341.73064   -485.91014   -341.95307    -387.3796   -353.77124   -218.66583   -396.08462   -364.48756   -494.23756   -333.52648   -445.72449   -335.80043    153.26025      98.9986    368.60769    652.77553    460.53214    114.89608    109.87144   -262.88629   -319.08812     -203.969   -408.89661   -150.02967   -257.12169    -77.27637    140.88727    455.36556     1171.585    833.71319   1013.86684    878.74098    170.61431     98.32871    249.09651   -230.03626   -337.71499   -460.66822   -412.71928   -171.35529   -383.81557   -525.29884   -315.53804   -417.51092   -330.58072   -365.42884   -124.04826    240.72738    414.39031    371.85648    561.60595    527.40523    325.66024    -25.69239   -335.33423   -422.50628    -406.6458   -177.94935   -294.94713   -220.22224   -277.89155    -212.9466   -200.36805   -234.29059   -390.75198   -408.17117   -352.64225   -465.12486   -502.69253   -393.93495          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       50.50   -273.62225   -113.65884   -427.49146   -336.25669   -299.78047    -50.94239   -622.46903   -460.74526   -318.98669   -236.18999   -179.42916   -245.17634    -508.8404   -308.48052   -480.26817    -230.2401   -167.20585   -456.86976    -254.2923     -285.179   -190.17007   -430.63265   -434.01551      88.4491    -28.34501    160.05938     389.6716    416.00214    259.48907    145.00727    -51.21621   -310.40824   -392.48226    -334.7224   -284.34296    -287.4076   -374.99039   -345.44335   -270.88983   -387.70944   -465.57933    -387.6813   -120.17007   -472.47108   -457.71786   -407.28711   -355.84876   -542.77063   -233.08393   -429.36748   -403.46065   -306.06392   -356.11201   -286.61394   -107.97186   -109.04442     97.14955     199.3552    411.17605    530.83274     94.93568    -28.75568   -320.53023   -247.44343   -170.07531   -332.49104   -330.94646   -121.15516   -220.70177     44.48806    223.29461    972.95205   1265.02168     916.6427    524.05211    396.74068   -164.86377    -14.67139   -247.46452   -384.84831   -377.84886   -418.22403   -355.62575   -192.35688   -397.80011   -167.62699   -351.37364   -199.50284   -262.84664   -196.07763    143.72174    347.40584    445.71433    592.35142    522.72984    288.39329    -41.11941   -439.82861   -331.65212   -248.71407   -401.77138   -326.64373    -292.2696   -256.73931   -180.78984   -244.69246   -194.33376   -379.52167    -482.6909   -344.00378   -297.13042   -416.31325          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       51.00   -103.81849    -74.80952   -521.13629   -440.73485   -378.00196    -16.16113   -695.05487   -327.80837   -229.40677    -96.29022   -379.39662   -347.85335   -433.89031   -178.22329   -281.20463   -338.58602   -254.30788    -349.4834   -398.61961   -336.91645   -245.84396   -573.74851    -317.8558   -126.39847   -131.40048    254.67398    218.06704    460.06741     196.4012     82.26901     -63.8438   -132.05245   -360.31564   -270.35312   -366.44683   -476.13477   -502.49576   -507.43502   -463.82977   -563.37018   -352.51801   -553.51633   -156.73438   -411.95025   -218.93289    -327.0181    -334.0471   -406.30196   -256.39119   -486.57465   -448.77651   -249.26928   -223.74893   -367.45451   -226.16366    -177.1928     75.68443    282.09828    423.17459    461.53846     90.27099   -167.18183   -192.69967   -210.99822   -488.01392   -460.44447   -213.69234    -70.11665     50.17337    361.56279    839.08712    853.81886   1112.88689   1058.22454    738.08996    341.48954     66.41462      20.4463    -329.6628   -233.95574   -743.07018   -518.44157   -712.54266    -582.4073    -319.7381   -649.14803   -257.13887   -342.89446   -231.50281    -294.6407    -67.02352    294.88706    323.74467    644.85346    296.15295      82.8418     -9.81699   -172.41004   -235.91304   -329.59208   -394.80129   -214.39466    -15.08705   -244.30129   -146.29979   -206.01725   -272.94962   -335.60235   -361.05446   -345.76346   -629.50285          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       51.50     57.75871    -91.87378   -414.47474   -467.11922   -422.16069   -130.28061   -634.03214    -198.8471    -235.7978     -10.8759   -391.85358   -400.80468   -341.48357   -131.02447   -323.69504   -532.70683   -437.28297   -482.18545    -367.4511   -317.53419   -425.68365   -412.75607   -232.70672   -330.54807   -107.90515    258.31139     235.2293    434.32628    311.06411    100.11588     54.02349   -241.14024   -401.37842   -471.17581   -412.93704   -483.56529   -538.16733   -350.49937   -355.16122    -381.3852   -301.51387   -458.58994   -373.51213   -423.52523    -231.6181   -192.43661   -291.02672   -419.47537   -241.04712   -335.36132   -469.57318   -379.37624   -144.55076   -407.75059   -384.26106    -26.79871    -64.65545     143.7195    539.38157    581.58887    309.66496     23.43067     -144.205   -335.35515   -135.14522   -235.95137   -275.97445   -232.55072    -76.66403     -9.56488    551.07638    992.70181   1079.99468   1008.92271    667.21379    183.61214     22.70738   -125.94818     59.04658    -263.8242   -506.17529   -265.49282   -584.62337   -525.57519   -393.51171   -375.73363   -193.47342   -369.36076   -226.27868   -261.99719     30.87688    221.14432    524.73454    452.22578    265.46591     117.4644   -160.37971   -168.73914   -288.84972    -264.7681   -529.22323   -286.33145   -138.52149   -200.30764   -305.25564   -253.36797   -190.96859   -408.31521   -397.59555   -432.72269          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       52.00    136.57365   -174.28837   -272.05025   -353.05857   -431.27326   -361.02995   -530.26486   -286.74054   -376.86187   -102.10208   -380.91903   -140.42668    -311.4291   -281.18221   -273.51618   -441.93978   -458.12672   -431.47326   -174.89395   -228.45744   -410.41055   -222.81329   -211.17141   -182.18549   -198.58223     84.29229    541.34779     402.6603    230.58851    231.43511   -102.60922   -207.32493    -317.2221    -542.9356   -417.80453   -504.54647    -371.7906   -353.86274   -356.51178   -380.39066   -221.40264   -240.43859   -334.89576   -570.82498    -386.7406   -503.53631    -381.1662   -291.71217   -428.65697   -318.90797   -333.79514    -360.0443    -206.8636   -327.06654   -401.60169     55.37846     -22.1918    527.28628     609.6433    188.21926    331.47134   -115.38214   -277.42312   -129.21429   -165.67923   -329.76444   -551.32829   -276.05939   -236.95041    -75.09093    338.56304    816.40141   1169.20864   1063.09288    858.00278    437.57823      89.9241    -95.74027   -433.18596   -195.30743   -695.29506   -416.28699   -224.40538    -388.7074   -437.99722    -463.9328   -342.61009   -421.66315     52.19637   -125.64048    363.94154    473.95996    529.51076    337.41647    276.34671     98.41319   -110.05981   -210.38187   -441.00655    -333.3023   -298.57075   -358.36911   -394.67263   -199.62169    -209.8202   -301.22588   -322.62547   -239.45675    -473.6734          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       52.50    -65.49053   -354.68226    -82.03901   -215.42972    -446.6744   -454.60346    -554.9645   -322.57362   -469.32955   -287.28677   -440.05829    -46.70612   -317.75677   -455.25457   -227.66661    -395.6054   -413.99909   -364.00244    -57.20453   -206.43972    -332.1907   -327.99404   -194.33996   -344.61399   -120.50721     139.4641    347.52842    259.83477    452.94199     94.78551   -111.46165   -148.91673   -517.53034   -546.64691   -393.78198   -174.78351   -354.76983   -253.25834    -332.7043   -364.12517   -256.11345   -335.55777   -228.26543   -375.32025   -445.48804   -427.30482   -493.20268   -213.96347   -308.94645   -301.54523   -312.66706   -208.38654   -313.05207    -318.2364   -309.66991   -298.18121      9.50019    342.21818    440.79626    273.72602     63.60562    -77.40714   -110.60163   -218.07213   -351.37878   -283.20913   -217.14521   -298.83118    -182.6549     -28.8163    602.55399    952.33164    961.70908   1103.01064    887.87006    525.53495    215.65542    177.82157    -47.89703   -400.06124   -407.37359   -331.80339   -498.01654   -467.84977   -276.49701   -266.59139   -306.51342   -412.35747   -226.22956   -116.91114    163.77023     424.2397    323.41212     412.3246    398.94605     23.43035   -280.50276   -303.85657   -347.90745    -327.1387   -364.03365   -360.85696   -286.43076   -314.47692   -269.09875   -319.76638    -492.7317   -406.71515          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       53.00   -314.42922   -564.02246    -92.06491   -155.73351   -468.74242   -406.42915   -611.11021   -144.74065    -517.0569   -342.25346   -511.86958   -297.96373   -313.97648   -424.37148       1.4601   -455.08877   -393.65573    -318.6301   -395.41559   -201.70806   -327.42023   -143.40541   -256.18799   -262.67163    -55.86475     119.2934    589.63817    467.89395    334.74356      7.17534   -175.35951   -297.50288    -331.4257    -457.9354   -300.09101    -196.3479   -156.83225   -297.84695   -322.97384   -259.62532   -160.28834   -355.82177   -374.07682   -101.82659   -312.14949   -464.97539   -491.33142    -416.6839   -249.65593    -427.3683   -331.14657   -473.27251   -481.22154   -480.80424   -267.17812   -214.59806     34.33061     393.9088    524.63641    228.45569     187.6734   -176.29985   -164.53353   -393.66838   -413.21645   -241.16154   -264.41939   -295.26726   -202.92852     18.19569    462.07613     827.1414   1270.88751   1397.14344    788.09649    596.72522    175.63298    -35.98661   -137.90597    -73.14716   -202.67294   -495.12359   -393.76838   -390.47362    -220.0102   -373.53053   -317.15967    -550.7626   -105.57651   -360.01304     -50.4407    460.38547    497.94999     415.2893    159.65778    -57.65639   -119.37178   -373.95603   -394.70446   -438.52543   -366.30034   -259.03864   -281.58983   -297.40435   -161.53666   -388.40352   -368.25165          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       53.50   -491.76979   -658.26918   -179.62508   -257.23653   -480.09614   -414.30978   -569.21953   -123.36346   -395.92698   -300.62043   -435.15922   -450.54575   -177.67959   -415.80589     132.9279   -378.47691    -354.1519   -505.66858   -411.04774   -206.35128   -408.43746   -223.48228    -312.0898   -251.33523   -125.49772     72.75841    239.38732    280.95141    265.03457    -28.62421   -228.29792   -251.91521   -313.07494   -251.93562   -220.32762   -265.18562   -368.23181   -195.32802   -257.20359   -636.01418   -498.98189   -368.72075   -433.99998   -365.20109   -418.56295   -574.12452   -413.53138   -280.96603   -499.16726   -397.90777   -332.30508   -458.22379   -401.20564   -309.52507   -191.65506   -322.91082     52.21442    224.80397    459.03716    347.73334    159.39346    -55.21393   -236.02797   -323.09849   -360.79516   -356.33657   -300.66875    -70.51606   -218.80905     26.82118    454.89198    630.01374     926.0712    993.61682    782.54136    185.61061    130.59252     37.24118    -60.30885   -274.90243   -537.11314   -406.53871    -80.38277    -335.2186   -389.88626   -587.28288   -379.44694   -245.81478   -238.63433   -115.58392     59.34684    394.16329    282.89145    371.42986     281.7368    -47.40795   -211.35082   -379.63219   -244.28125   -364.55487   -376.78399   -408.96233   -316.36356   -291.60058   -166.78379   -415.82234          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       54.00   -505.13276   -623.23358    -240.1465    -256.9291   -403.24087   -472.25328   -386.04801    -261.0645   -319.13586   -361.57808    -330.9021   -406.28591    -37.64105   -585.31231   -121.85052   -235.58074   -400.47899   -539.24723   -238.13532   -196.99056   -408.12853   -198.40093   -218.83438   -422.10235    -35.89894    148.49666    409.95614    224.23291    245.04113   -125.02338   -171.57672    -97.07728   -396.40364    -177.8429   -490.15811   -380.07358   -363.04214   -362.90162   -421.50236   -502.27755   -476.38124   -546.65732   -293.43459   -404.99216   -542.60401   -534.97912   -383.34911    -73.65928   -360.11874   -429.23462   -334.93101   -338.77895   -260.86979   -244.84286   -226.64742   -126.46969    238.95063    265.19713     406.8719    181.80309    231.97179     -10.9898   -257.19685   -259.44053   -254.05414   -222.53497    -242.9858   -277.98123   -139.83327      7.09303    432.83518    822.17927   1053.97035   1158.07176    705.33584    275.45375    407.60057   -131.72884   -140.39599   -160.31414     66.46877   -674.71283   -569.49358   -365.40354   -347.45439   -194.94494   -275.95517   -190.87833   -209.86108   -142.51207     75.83066    352.47457    483.02077    370.02481    519.84147    231.76825   -251.86505   -499.54969   -472.06101   -362.12214    -325.2214   -326.51414   -195.41538   -200.31754   -468.47469          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       54.50   -414.31824   -503.89606   -334.10224   -155.63429   -375.51314   -500.77423   -189.12834   -386.91979   -357.53524   -489.65316   -345.92145   -366.55402   -228.44374   -588.88654    -62.30084    -79.03502   -524.35395   -411.58385    -257.9058   -164.33809   -364.01015   -270.12599   -222.18296   -473.53575    -60.79965    303.78672    308.04508    391.07153      30.4378    -54.69324    111.77512   -512.11987   -293.51085   -160.62396   -403.76194   -361.57147    -257.2205   -353.18532   -420.54346   -529.13888     -457.588   -529.90782   -379.30874    -535.1997   -416.41281   -506.69103   -385.34871   -272.27134   -339.59123   -459.38487   -278.01326   -425.66463   -353.32287   -283.74777   -191.93708    -52.24124     73.55128    382.20911    222.11186    266.87768    111.07691     -66.8413   -328.22481   -291.18282   -245.37259   -325.00746   -305.23699   -307.46513   -134.40938      3.92024     303.9215    763.23315    922.45024    875.58483    817.71807    519.89993    244.52223    -26.32978    -50.94558   -117.65733   -369.76245   -495.93304    -819.4912    -445.2789   -234.10801     53.58154   -232.49149   -268.19122   -365.90234     -9.38845    -80.63683    227.95708    503.64265    680.95832    258.33417     63.46189   -274.24857   -325.70347    -515.2992   -611.76762   -390.92816   -383.91866   -341.72168   -371.15365          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       55.00   -322.95316   -493.88457   -402.75721   -258.06236   -350.09978   -579.55525    -134.3674   -588.64137   -407.67502   -598.40646   -363.13619   -414.25788   -381.09653   -410.46118   -255.55933   -151.27281   -524.65997   -402.81937   -465.76985   -431.92052   -425.85554   -423.81231   -271.00164    -435.7967   -273.87692    166.98671    189.61786     281.5147    -16.44385       -8.367     34.33617   -427.90333   -335.27339   -286.00875   -395.73755   -343.41939   -431.61192   -355.25635   -523.05085   -524.64534   -409.16154     -87.7869   -520.79516   -510.48753   -534.71872   -399.28887   -267.86668   -240.57211   -448.69999   -402.56067    -256.5974   -394.75072   -326.31416   -380.66405     -243.005     41.79771    112.59537    350.18748    337.15038    432.99815     275.0002    -67.56677    -10.26662   -304.87023   -154.63107   -290.28109   -199.18862   -287.66443    -232.4965     -9.86596    460.44605    738.27444    821.00064    769.92728    724.94411    219.34933    172.60972    -71.49496   -111.75263   -247.28936   -305.81825   -420.33431   -518.55297   -294.75038   -268.60806   -351.76692    -340.0719   -296.52834   -227.49418   -168.91576    -59.55894    516.19352     581.4394    374.50341    535.78047    107.04945    -316.6212   -503.62674    -446.9287   -442.45407    -385.1911   -368.19287   -365.41698   -442.78176          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       55.50   -338.94085   -592.19909   -468.86485   -346.32959   -245.83672   -555.00094   -199.48284   -793.61602   -527.11645   -501.51155   -424.75062   -405.10017   -427.43503   -224.01487   -406.06388   -227.10813   -334.53282   -399.05681   -483.07461   -584.54287   -304.69854   -525.37077    -166.1841   -329.01282   -182.03458     81.11101    190.28302    428.24204     79.74044    -154.3618   -239.78336   -353.77463   -378.06596   -123.08536   -448.90771   -468.62603   -476.40835   -497.67239   -497.23651   -289.20199   -335.64533   -215.47108   -557.65509   -370.38539    -539.0118    -268.9643    -394.3002   -555.79056   -407.01838   -339.36074    -465.7225    -298.0266   -454.56357   -318.00895   -216.34838   -155.10636    157.00867    237.78165    569.90401    198.42279    184.85553   -132.81255   -273.36414   -490.00107   -214.35168   -196.01599   -328.44498   -248.93237   -127.67035     88.46111    506.25402    734.69353    965.56627    875.14165    717.93238    702.42771     139.3318      1.89014   -184.28512   -166.65934   -350.02227    -355.1879   -440.52735   -311.92293   -307.02587    -694.8275   -258.78822   -366.91784   -312.50392   -154.51539     -2.54488    168.97594    370.54369    277.38126    570.71425    130.62268   -181.08724   -484.43576   -394.87495    -396.6386   -539.54522    -437.3574    -362.5041          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       56.00   -379.76876   -497.27698   -468.50274   -326.24519   -192.91156   -576.44854   -249.75704   -502.39093   -499.07246   -402.31849   -391.95927   -399.92221   -472.06237   -164.97501    -506.5113   -403.36127   -202.06901   -459.64103   -308.74833   -493.61371   -166.15581   -366.69781   -281.62825   -240.56021    -58.30855    -15.02152     516.4238     75.79333     59.31751    -78.83507    -307.5854   -403.50815   -584.95792   -253.32724   -249.90939   -420.21643   -530.62299   -462.12377   -519.76869   -438.97224   -202.78441   -346.77268   -399.82823   -302.59394   -514.78421   -209.98628   -225.47306   -270.65231   -495.92194   -254.68981   -505.77448   -472.67059    -270.0827   -334.41714   -258.14366    -56.41656    -79.39664    438.73187    303.44449    339.82489    183.97275   -215.71556   -197.51567   -240.59093   -100.49704    -432.0882   -226.05728   -298.75271   -229.45043     123.3376    347.78814    719.92987   1161.21984    869.71684    667.09035    625.96812     80.54564   -269.21066     10.98708   -379.90154   -410.34468   -216.57989   -487.41784   -657.92907   -320.87449   -265.70241   -198.47703   -377.70957   -185.73556   -161.74676   -158.85571     79.00124    409.03742    337.96627    420.48238     85.25132   -223.72501    -257.9686    -269.5373   -604.45456   -389.06282   -537.91564          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       56.50   -296.28587   -357.49469    -290.2111   -196.54346   -254.75575    -590.3363   -271.21785   -295.16275   -388.40992   -177.46248   -357.24699   -453.23726   -459.39542   -118.88578    -459.8993   -500.66917    -93.95001   -425.39536   -187.90235   -557.43088   -346.77834   -398.55561   -281.80042   -315.80405    -149.5573    272.77544     242.4261    159.31803    122.71903    -74.85314   -293.23512   -231.39689   -456.59571   -261.12752   -549.58662   -431.85188   -556.16935   -367.95474   -587.76479   -394.23503   -411.23234   -441.81488   -535.59904   -294.08022   -541.94572   -367.34209   -376.06053   -318.54297   -342.70675   -422.00099   -323.98322   -457.26522   -157.96163   -478.63441   -398.84306   -269.80312    -76.02257    408.18745     167.9063    326.64915    173.99196    -181.3812   -195.17316   -302.86675   -312.14123   -189.99152   -429.14153   -250.55616    -194.1048      5.85425    467.38752    745.53649    836.99373    942.59323     854.9745    656.25666    270.56968   -228.20805    -34.39929   -302.30401    -125.0269   -386.56412   -505.34114   -402.77322   -292.31892   -364.92477   -310.32539   -161.85089   -260.93671   -185.15444    -85.87456    -53.70521    243.05873    355.06482    270.92593   -168.98257   -196.66087   -332.16769   -431.69006   -333.21843   -494.10022          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       57.00    -89.83153   -232.66527   -174.71136   -145.53276   -264.65972   -477.04246   -197.50043   -294.77196   -338.24885    -10.53062    -413.4479   -437.52227   -456.13828   -228.25969   -355.53138   -417.06209    -31.32548   -367.39261   -273.72567   -553.81921   -441.51281    -400.4537   -311.12397   -416.64222    -62.46134    324.83213    241.28624    398.22008    232.79049     44.13087   -294.82714   -545.75239   -253.46079   -277.47683   -569.77277   -510.62318   -175.60421   -360.75418   -228.46975   -427.86259   -439.32569   -469.47971     -515.402   -371.06321    -438.4926   -307.23319   -474.09229   -474.51126   -301.70934    -388.5207   -482.27073    -528.2931   -297.07331   -178.34379   -313.40861   -168.17938    -16.94769    192.19662     276.6343     612.7512     68.79104   -159.02669   -181.44328   -373.54405   -388.81006    -85.38705   -453.00726    -93.56331    -89.78997    -27.97582    488.68224    949.93927    635.75935   1059.26939    692.06408    609.23797     321.8299    -256.9356   -205.32149   -265.94151   -248.36808   -354.43676   -471.84489   -535.73295    -421.5429   -418.81813   -330.42698   -221.72655   -457.70268    -96.03473    -46.94885     86.96868    210.71798    632.01126    442.31703    103.83116   -156.98909   -231.93515   -370.58015   -315.38613   -421.29211          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       57.50    -93.23373   -179.50911   -173.45887   -198.36607     -192.315   -260.09157   -137.14401   -319.58376    -307.9888   -290.36907   -416.29659   -494.19193    -471.8951   -279.23631   -390.99776   -343.24048   -135.73044   -127.36869   -274.85853   -202.09561   -442.88064   -382.66589   -311.62192   -356.32703    -80.13308    189.29813    227.69235    258.80647    124.87287     56.88073    -65.44464   -408.92657   -438.57928   -303.49256   -341.93008   -486.87698   -288.84849   -400.97054   -257.86791   -507.31465   -275.55476   -428.42424   -446.69792   -390.69068   -224.83352   -341.17421   -410.44955   -332.41624   -318.72964   -359.30796   -431.93895   -297.13262   -529.95533   -447.29206   -300.25259   -335.15597    102.73856    290.77038    409.32756    268.49154    100.39397   -225.87618    -48.93071   -305.59222    -376.3779   -372.54139   -272.96452   -398.74821     -6.92903    119.58505    316.56972    857.83921   1098.01515   1014.17791    937.86931    465.75141    422.27395   -105.57191    -97.99804   -150.33573   -452.51766   -403.87735   -402.44318   -439.25562   -338.88479   -412.51832   -352.66455    -393.1786   -214.22568   -335.76231    -92.30071    169.38733    377.32564    396.65827    272.37557    214.17037     13.17313   -286.11748   -326.23452   -567.05808          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       58.00   -234.54161   -262.07098   -265.48844   -300.93507   -277.84422   -106.02853    -89.30981   -291.33269   -247.85928   -381.45802   -371.94474   -771.38821   -415.65551   -175.95768   -365.68265   -310.20536    -272.9482   -103.14058   -212.97568    -99.38202   -420.78419   -497.10853    -96.58172   -258.01472   -109.23988    166.82526    332.62893    265.26289    327.15587     -2.10364   -257.00197   -299.70899   -554.76135   -416.43191   -473.48701   -264.85163   -221.65946   -341.27315   -313.47389   -421.67085   -489.63538   -160.19498    -368.2749   -151.55191   -305.23449   -312.44401   -402.69491   -584.13522   -583.19566   -414.54897    -330.9653   -257.73943   -456.44414   -589.31667   -267.88899   -147.99765     81.80833    344.92148    446.33841    173.25925    106.89181    -67.18517   -192.08476   -421.00218   -317.32936   -329.45554   -202.21789   -196.62666   -197.87894    -25.25137    247.80775    688.61671   1105.17371    879.52428    894.53967    586.91157    208.35819    -61.98396     71.21265    -200.1515   -480.89759   -312.03788   -430.22809   -527.65426   -424.74297   -402.12912   -303.72851   -393.96722   -232.38542   -259.25529    -45.58581    227.45237     224.0078    353.66228     510.9933    140.78148   -181.99941    -198.8322   -452.47987          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       58.50   -345.75896   -347.48771   -239.88108   -335.71386   -365.73709   -102.87263   -187.44239   -286.61647    -193.9643   -334.40432   -341.91305   -659.55581   -298.72919   -173.61343   -323.49005   -210.48666   -362.93209   -187.05792   -376.72931   -256.34426   -228.04962   -248.86062   -342.01026   -314.57963   -119.89238    103.59531    407.35975    352.64249    423.38961    -78.58729   -157.07623   -335.48053   -460.52416   -437.81019   -302.21187   -216.91567   -550.26576   -278.75817   -252.34314   -410.42249   -480.78335   -298.40444   -325.48668   -578.96329   -537.55599    -294.2781   -507.67582   -288.76417   -467.91056   -578.95919   -341.66049   -143.07338   -339.89155   -145.65059   -497.71092    -94.91161    213.99415    303.73685    515.76928    190.30636    182.14382    226.20649    -239.2409    -91.96259   -209.15634    -408.5739    -94.45966   -397.85854   -217.69518     75.69555    593.78741    568.99184   1035.09102    971.63715    903.11798    599.81574    122.37101    -92.12441    -17.07846   -185.89805   -382.66915   -164.46809   -473.64635    -325.8432   -562.32451    -355.5482   -159.47628    -343.3632   -274.38223    -223.6751    -57.75543    150.83093    291.57733    546.03717    188.66317    -54.06218   -118.63138   -129.72471          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       59.00   -387.74379   -333.14436   -164.73836   -296.51385   -284.33077   -217.89153   -304.73662    -205.8822   -232.73601   -300.12906   -420.21422   -451.71757   -263.26762   -174.95368   -446.43186   -253.74174   -510.61821   -194.81302   -379.86547   -308.27174   -140.38764   -320.62555   -325.86738   -276.80499   -120.22098    147.66186    297.84178    132.95103    354.20197   -101.24084    -267.8135   -125.35922   -194.62419   -301.54395   -496.18213   -396.40283   -423.14713   -280.11336     29.56341    -433.1222   -369.35065   -580.21394    -64.48633    -399.4383   -390.39339     -367.918   -324.06412   -276.38756   -537.70309    -44.12768   -270.90237   -275.82587   -425.18882   -259.98307   -119.79767   -305.71489    118.53257     85.81739    434.82393    280.98013    110.83702     45.38797   -318.66131   -286.26895   -375.24675   -274.78975   -412.56508    -342.8239   -369.60726   -107.70532    281.33027    667.13505    724.31137    1182.1806     662.7574    614.74034    102.56741    243.41749   -153.12532   -359.34974    -190.3134   -116.78255   -357.15767   -235.66602   -385.10721   -426.36092    -285.3268   -201.05877   -137.32716   -207.22185   -181.60442    296.51642    144.62332    364.55071    207.42044    -69.11533    -66.84115   -345.74764          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       59.50   -323.11875   -383.65533   -198.52917   -172.22833   -210.32999   -269.06262   -343.20736   -213.17522   -396.30932   -273.95884   -512.59191   -377.24234   -469.19046   -117.04267   -511.04729   -318.79281   -378.95309    -99.21606   -455.77895   -358.32099   -260.63976   -468.59936   -454.22427    -342.7003   -254.96785    305.32513    101.14689    179.34363    -18.30055   -178.44565   -203.73633   -159.63621   -251.02791   -434.40226   -489.45366   -349.19763   -336.49719   -273.28832   -146.86225   -346.39803   -404.15205   -288.53716    -435.8971    -377.4293   -439.72226   -380.38883   -288.95495   -268.44063   -451.49849   -341.41494   -273.08651   -256.42557   -337.36221   -371.76136   -141.44912   -149.50395    244.42414    205.44724     524.3853    343.41608     65.49314   -130.68412   -216.57809   -268.70706    -308.8045   -331.99988   -255.72751   -236.27128   -220.17232    -51.76548    410.21189   1009.67325   1033.41577   1254.13386    717.45324    509.16596    408.46165    -87.42425   -158.18435   -269.63712   -262.06883   -150.67606   -230.85669   -378.74712   -310.78946   -314.86547    -284.8694   -517.89309   -350.49787   -190.60614    -11.60061     293.5136    415.58868    328.25544    299.76198     47.10963    -26.84772          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       60.00   -236.97793   -453.86748   -203.94897   -121.28393   -200.79466   -300.79559   -369.07032    -448.4472   -468.60184   -236.63493   -601.27084   -289.63376   -436.21522    -70.35592    -392.2317    -260.8948   -125.14225   -292.23434   -307.60103    -378.2841   -289.22926   -476.31398   -381.91624   -304.86228    -79.51585    -25.25908      24.3348    102.15643     49.62654   -148.53181   -210.52646   -202.67023   -357.21184   -434.71221   -601.67474   -252.30671   -352.27268   -513.36181   -550.08393   -377.78561   -677.39534   -316.70463   -552.02423    -245.7213    -456.3313   -270.12558   -275.09537    -89.05376   -406.02666   -391.31407   -338.21429   -303.75686   -353.80885   -279.62313   -293.23771      6.03741      7.96749    113.03841    313.31527     363.8195    134.94848    -163.6166     -99.6763   -323.02953   -164.31099    -90.80887   -246.27244   -320.68546   -276.06405    -53.77952    272.59906    663.27595     890.6416   1043.11599    859.81258    535.84573     339.4313    -10.34717    147.72216    182.19109   -216.08733   -298.67926   -482.84797   -623.21393   -328.86922   -262.07875   -214.86805   -425.62819   -346.86828   -126.71918     -6.57351    107.17398    254.05192    181.46785    370.09689    109.41466          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       60.50   -203.14324   -537.01859   -178.83618   -113.77103   -226.19814   -324.34678    -441.2191   -482.81255   -389.98371   -237.25282   -676.09552   -292.14282   -259.90828      9.09361   -340.12415   -257.15111   -112.00825   -445.85302   -297.98168   -289.17925   -250.25529   -414.41559   -325.46878   -176.91378     60.20983   -218.28159     15.00705    132.24766    186.01558   -135.15691    -231.4803   -264.99996   -374.80431   -401.61471    -354.9894   -306.77892   -523.51912   -529.67808   -406.71001   -257.30403   -449.48766   -242.54797   -428.38901   -336.76657   -604.02836   -457.32684   -549.51282   -398.78827   -319.06106   -405.26385     -463.834   -264.70649   -205.29743   -366.41353   -471.42984     103.5275    167.97019       99.924    320.14287    307.41096    163.25809    -63.53756   -214.40435   -372.92726     -1.23389    -511.0913   -248.97671      16.1631    -84.60691   -114.22251    348.94612    697.69447   1037.55981    915.73359      959.543    608.30231    311.88272    -23.01528    -82.40946    -36.67998   -114.70672   -388.74594   -416.18878   -314.12033   -430.60668   -394.20772   -362.61807   -540.07862   -364.28785     -5.85625    -92.16668    187.52885    329.59104    448.52904    211.70231      6.05801          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       61.00   -193.06007   -478.42803    -170.6408   -289.04103   -250.60056    -325.7278   -508.60657   -267.30768   -368.07407   -468.74579   -540.54309   -357.84781   -463.88725   -116.02659   -312.14506   -319.52778   -407.13567   -218.90261   -396.55783   -386.75034   -404.86739   -322.99776   -383.08656   -196.34634   -153.57227   -281.63111    332.40005     236.8687     47.76634    -151.3742   -102.63946   -274.48415   -320.07848   -232.81115   -345.80003   -577.98233    -295.5939   -492.87738   -444.68594   -299.97367    -84.03288   -615.84803   -293.09132   -500.45316   -519.84494   -332.62627   -407.51656   -477.41979   -479.00161   -410.16618   -536.17924   -536.54978   -403.56285    -332.1179   -316.77357    -91.78069     108.8516     32.20445    209.07573     130.9447    172.23804    -93.17237    -177.8839   -204.27053   -420.50743   -290.86465   -323.62809   -293.91469    -142.5076     19.38627    533.13493    553.26902   1105.68736    1065.6657   1057.84825    562.21777     459.7604   -106.64151   -215.92342   -251.82838   -304.33171   -484.69846   -403.55969   -414.83725   -452.59138    -354.2859   -445.95452   -465.58553   -212.14209   -196.30053    -39.92351    136.75394    351.05096    257.22724     58.05139          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       61.50    -224.9249   -422.99242   -270.83358   -435.68381   -303.73226   -299.36056   -446.26698   -198.38351   -300.54709    -473.2848   -396.73866   -295.18824   -647.08947   -281.32178   -223.26036   -352.98155   -512.41131   -189.59305   -375.32634   -462.04391   -401.09571   -393.73438   -325.16413   -211.35731   -115.13824   -191.68188    103.99938    201.92817    121.25592   -109.93614   -129.14272   -292.76451   -453.46147   -296.60884   -237.20013   -539.13129   -480.93334   -494.85783   -277.08281    -333.5532   -287.60829   -520.67639   -330.80166   -342.00405   -324.18841   -304.38602    -338.2343   -497.77483   -470.97691   -406.41002   -437.01607   -389.58127   -574.22936   -455.28423    -365.2257    -31.06886     128.1729    226.40569    125.52714    423.03478    207.04679   -175.65236   -258.88478   -295.01913   -456.56595   -361.40384   -389.88386   -306.35229    -97.51994      28.3365    240.13448    618.96041    885.05522   1058.07708    905.19393    559.85723     255.1742     -7.41803   -216.53928   -219.70965   -349.61322   -292.91772   -594.97105   -357.73906   -168.29364   -392.90427   -241.09739   -364.91927   -283.70134   -351.07906   -182.21734    154.59811    283.26143    405.16675   -104.81124          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       62.00    -298.0304    -400.8436   -366.97805   -497.55127   -388.57637   -250.84151   -339.11197   -225.57157   -277.75556   -371.64091   -261.57346   -395.09406   -488.20665   -387.41447   -219.53612   -354.67743   -457.60176   -499.06402    -418.7543   -473.15355   -304.81605   -783.44925   -312.59433   -292.06851    -66.39501   -104.42244   -201.45145     75.51363     25.41048    -77.53284   -172.14011   -288.93013   -431.84197   -328.99262   -109.78295   -521.80568   -561.53832   -380.82502   -528.10342   -497.03101   -246.65243   -481.96718   -351.60532   -314.27884   -371.34269   -432.46838   -299.01116   -290.22461   -593.21927   -452.93152   -619.81852   -271.96259   -309.21435   -249.53077   -284.28512    -50.76659     97.57002     59.66336    188.70437    254.36865     -12.8857    304.29675   -246.89014   -247.04612   -248.77241   -429.18598   -332.82585   -241.42708    273.37844   -108.85877    166.07097    595.08031    784.62781    802.61335    790.13417    570.67058    272.02781    -26.66487   -254.19808    -66.26504   -191.63213   -227.03177   -585.28918   -341.09678   -334.51742   -301.49939   -376.27417   -351.45802   -197.18985   -129.47934    -60.10032   -120.59149     226.6301    203.37505          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       62.50   -381.15063   -399.88377   -353.18044   -450.72061   -495.68507    -177.6664   -244.30332   -343.92927   -406.53071   -406.79973    -67.95993   -439.53449   -430.16364   -329.26511   -200.74335   -310.94796   -494.98515   -578.47627   -551.55348   -355.25892   -439.32655   -431.14532   -361.18229   -546.74945     -27.2828   -163.22091    -40.78653     56.30218     63.34329   -176.66751   -166.29432   -230.21374   -441.90661   -356.47686   -291.94956   -452.85989   -598.51892   -323.05664   -541.69632   -336.81207   -254.10668   -507.99467    -335.8272   -593.34443   -494.12617   -440.33443   -319.34166   -426.19104   -472.99408   -373.95716   -334.89862   -514.38801   -582.72186   -261.30711    -49.21606   -248.02655    -33.45676     60.63293    284.32679    111.64261    136.84044     -4.42478   -143.44545     -68.1146   -306.36018    -255.6427   -300.18874   -128.90454   -251.74079     51.41262    338.68242    673.16952   1042.66822   1243.40457     729.1477    486.29337     41.64162     16.72312    -56.35677    -92.67095   -270.87922    -425.9752    -328.6702   -339.65367   -200.84103   -333.86587   -396.69755   -404.59584    -283.4506   -204.01258     19.52364    -32.37037    -23.89477   -311.08458          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       63.00   -403.51228   -430.91657   -314.79391   -506.07621   -465.65023    -78.86731    -212.7455   -527.43906   -404.23455   -392.78218   -100.59354   -376.06686   -470.63849   -204.40593   -176.17026   -183.98987   -833.22004    -545.7372   -546.08948   -303.19779   -577.94753   -264.45113   -316.35989   -413.90709   -233.96904   -172.38846    194.30763     -0.58653    -98.63413    -20.44675   -370.09458   -311.89288   -436.03141    -165.8418   -335.21725   -287.32812   -519.54003   -475.93899   -451.28607   -320.41941   -479.38196   -403.81715   -252.78544   -480.62321   -386.43534   -506.25135   -481.41255   -418.90549   -456.26693      -617.01   -436.83136   -451.36046   -380.39617   -329.28753   -285.17875    -68.94246   -102.90594     52.71973    220.96662    245.93945    213.61244    -48.09903    -84.48618   -418.52384   -372.15276   -364.38489   -323.59116   -440.67355   -210.83211    -77.12418    313.73322    608.45465    885.37679   1084.14189    968.82486     612.1969    398.08262     84.12051   -278.19712    -308.5717   -310.62909   -442.75714   -441.19659   -196.31694   -276.99013   -305.32744   -267.98529   -152.60537    -243.5142   -408.09079   -142.25548     25.17961   -147.31129          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       63.50   -332.37155   -494.09825   -260.90747   -592.75723   -455.75231   -306.24923   -324.45502   -564.04476   -223.84926   -362.79123   -226.79993   -363.13463   -415.94083   -166.58088   -286.14798   -241.27912   -685.61575   -291.29576   -432.98468   -539.64175   -479.28699   -445.69179   -420.06987   -348.80813   -315.51101   -118.41854    255.88893    -57.14217     41.93266     -15.0163   -234.76488   -206.39954   -555.97219   -291.70914   -226.00569   -362.15846   -311.10222   -333.67452   -382.53311   -440.07213   -393.74739   -329.62261   -253.03493   -247.43118   -239.72692   -555.12566   -211.80237   -423.19155   -507.75899   -180.84046   -354.67323   -433.90883   -283.04905   -323.25022   -250.19717     47.14775    -26.21296    103.10832    178.49836    256.69049    120.67776      6.98506     -132.623   -219.94608   -281.36266   -577.75262   -415.21633   -171.90405   -217.78125     94.05413    272.23518    490.15441    895.36458   1010.75843    856.10302    724.67479    205.68687    256.62659   -157.18844    -515.1838   -357.58767    -236.5684   -363.20825   -326.93298   -269.35949   -360.29354   -359.35912   -348.34399    -332.6144   -180.13115    -261.4528   -187.58518   -178.50774          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       64.00   -237.50644   -404.84868   -228.84495   -595.85116   -492.09616   -367.22605   -476.08257   -484.82845    -57.30646   -321.92513   -263.32209   -214.12875   -401.51016   -257.26818   -503.97636   -369.24889   -257.08398    -173.9806   -385.40438   -467.23566   -448.63524   -453.34393   -487.13511   -179.32594   -406.45506   -135.42471     76.22859     38.00333       37.617   -114.77781   -226.70958   -375.62431   -271.72747   -517.83868   -380.33539   -388.92687   -490.61037   -279.69524   -301.56691   -503.09246   -315.52983   -230.16359   -294.33161   -322.10553   -629.16426   -380.87934   -460.38087   -238.25144   -282.37298   -420.55496   -194.17285     -251.944   -246.52266   -381.06177     -401.733   -348.63908   -200.81214    149.16143    -87.64056    182.01423     184.0025     -51.5554   -247.73667   -341.81468   -271.39621    -309.2247    -300.5318   -353.02759   -250.69499    155.39981     242.8145    487.11943    926.15299   1024.31072    831.64816    586.10861    359.37925    490.21987     69.39693   -255.39937   -262.30733   -412.15771    -534.1172   -262.98961   -292.76879   -443.10245   -545.39949   -341.28844     -356.926   -230.96805    -83.05757   -402.82355          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       64.50   -178.69027   -293.45553   -204.83756   -558.54058   -541.41899   -231.19117   -494.70113   -480.58429   -167.40153   -291.30787   -277.19282    -44.53131   -362.23824   -293.67958    -526.7941   -419.45823    -52.18864   -189.72578   -352.51286   -306.56238   -503.97305    -402.0821   -503.09726   -156.64229   -228.63002    158.13904    -51.44659    216.14663     67.64368   -102.17541   -159.45742   -350.85233   -279.54676   -521.97094   -369.29053   -370.00748   -499.14544   -198.08254   -354.94398    -452.2278   -245.53522   -280.08526   -383.19021   -374.15993   -399.20948    -484.8045   -234.48947   -344.65488   -347.06448   -454.68352      -89.737   -327.75137   -333.67905   -317.70106   -273.64236    -204.6457    -313.3322     86.39993     73.17516     340.3931    108.17021    312.64853   -300.60211   -291.04946   -223.39532   -225.42256   -257.09961   -152.22089   -110.04624   -101.83261    120.58721    491.83064   1095.20753   1034.94633     804.8806    710.52135    117.28063     92.24111    -46.66516   -104.26792   -162.23638   -498.65641   -285.98356   -501.45504   -396.71698    -469.6304   -662.62787   -351.75081   -473.22857   -377.52363   -199.17292   -436.41345          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       65.00   -144.81971   -184.22442   -181.30546   -458.32688   -393.50726   -161.69667   -368.47111   -511.83668   -350.80374   -319.56422   -258.12841   -179.27977    -330.0976   -259.05354   -328.27612   -586.45173    -33.61409   -261.20521   -356.32206   -352.27352   -389.96391      2.28496   -403.05904   -370.32099    -117.6562    118.79066     27.00535    331.65078      14.9528    -90.61668   -256.38792   -157.61363   -381.56749   -391.47861   -319.65557    -547.1294   -496.13344   -544.17696   -440.59247   -249.88033   -353.11362    -411.4701   -397.80716   -367.52042   -196.60606   -323.33573   -343.92793   -210.39794   -394.56178   -280.04756   -299.46258   -393.71365   -337.92892   -265.38709   -345.97124   -300.36576     -5.92922    138.25413    208.89663     21.40865     35.10002     -45.0624   -315.19836   -335.60214   -249.85958   -159.02597   -301.72321   -333.76496   -226.52727   -226.15606    289.16825    555.92772      713.112    867.03958    860.79472    385.63267    116.72931    252.37742   -127.73505   -181.41234    -235.3215   -362.15826   -382.80859   -404.63757   -306.62822   -359.76328    -418.9851   -387.65655   -326.60776   -376.46844   -416.56558          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       65.50   -111.97924   -159.85233   -126.34722   -329.65442    -298.0116   -287.44095   -221.01794    -486.3122   -296.91261   -384.69874   -231.59158   -491.60584   -394.31637   -276.91978   -430.91741   -457.48559   -217.37462   -444.22347   -237.73743   -441.45655   -433.67789    -84.84616   -281.98763   -181.63391    -24.46315     30.91983    162.44977    129.28092     -6.80344   -307.42632   -149.77709   -415.54867   -472.60185   -379.39669   -270.63469   -529.14426   -481.23292   -243.19194    -301.5356   -251.19784    -346.3696    -397.3808   -390.96983   -406.28179   -331.20564   -397.42617   -559.43429    -284.5628   -347.22415   -302.78545   -570.39707   -448.14866   -172.29975   -201.04368   -256.34718   -130.85792     12.11958    227.11626      3.03619    213.56921     77.71196    -37.10136   -138.72385   -329.50399   -301.54386   -359.83296   -381.47354    -36.63976   -381.41306     11.95133    355.00966    621.03861   1084.17907    990.43036   1018.18515    642.24997    303.33206    175.18869   -219.21328   -134.17722   -311.00524   -314.74284   -214.97749   -286.07946   -353.08056    -458.5588   -295.53752   -410.56405   -475.67987   -359.65605   -344.07816          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       66.00    -32.67418    -174.4052    -66.61216     -187.035   -298.20344   -316.23897   -258.07822   -432.29873   -154.83882   -482.32245   -258.94092   -551.77847   -321.37073   -216.89429   -600.67549   -374.95006   -272.52866   -442.41299   -108.80189   -731.86561   -414.43395   -347.46565   -271.79134   -320.87216    -49.98538     28.51209     -4.82298   -149.19555     -9.34738   -243.58848   -252.82339   -135.88053   -523.75082   -380.62097   -301.22645   -406.03738   -267.56091   -376.82378   -458.83315   -463.99388   -354.74708   -425.72358   -545.77522   -491.88677   -342.23688   -530.94117    -539.4057   -368.14595   -278.59211   -298.89548   -303.69942   -364.33976   -374.39154   -251.73902   -130.19512   -307.44559   -172.79874    133.56696    237.37394    100.00048   -177.59687     -1.80899   -387.03427   -279.12754   -502.63284   -395.52178   -333.71092   -108.11122   -285.69965    -16.17168    339.94727    268.33134    955.66511    790.84806   1018.25523    557.94754    444.35593    -15.38241    -191.1707   -171.48574   -395.88537   -364.22384   -440.90449    -302.1561   -499.88706   -447.67583   -369.33174   -344.02489   -338.02545   -457.61202          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       66.50     13.18638   -214.60229   -112.55409   -348.74613    -238.5851   -267.92807   -422.27518   -374.66313   -137.94827   -336.19019   -369.27473   -513.61233   -241.12506   -221.38599   -395.82305   -415.96072   -265.50166   -353.50316   -149.16005   -587.44415   -345.12087   -434.32366   -276.64938   -413.22873   -242.18441   -148.43854    -84.46163   -312.08276    -76.03625     -93.4323   -305.72956   -379.36057   -331.13825    -307.0917   -240.31669   -327.27475   -277.31579   -303.51055    -425.6029   -307.97416   -428.91607   -332.83015   -486.57071   -503.28995    -329.2243   -562.14342   -475.23014   -424.30229   -528.04192   -300.02525    -169.8979   -470.94176   -120.87454   -281.01172   -304.98875   -201.85678   -114.15635    120.85089     61.81712    135.77379     70.32924     -13.9032   -319.32452   -431.16886   -266.30054   -220.34051   -244.69957   -241.59013   -148.44061   -171.99428    162.22062    786.82224    858.44152    957.99306     884.3375    644.57064      450.067    288.37187    -42.06664   -117.33484    -253.9843   -397.81236   -286.26164   -556.79747   -539.70617   -296.60522   -400.06542   -312.44988   -277.50162   -383.78718          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       67.00    -82.46122    -362.8047   -250.34058   -512.34429    -79.23817   -292.77937   -598.25672   -249.64176   -223.29154    -78.01838    -420.4075   -411.20028   -357.08076    -404.2757   -224.54244   -462.43985   -393.25302   -403.23389   -115.23443   -398.47288   -249.96005   -269.36479   -224.58536    -208.8953   -315.24714   -178.84418   -112.31885    -51.17638    138.30282    -97.57574    -82.06887   -498.21451   -448.73522    -352.7319   -316.40599   -385.84086   -275.99447   -425.14637   -250.38944   -333.43536   -409.77211   -290.98972   -323.70836   -393.19672    -352.2096    -390.5771   -530.27093   -257.59226   -374.45322   -450.83433   -319.39753   -283.08975   -277.83545   -398.21228   -342.72873   -185.95222     141.0249      5.66755    228.05481    114.21461   -211.88824   -220.03314   -101.56985    -256.1258   -214.50714   -298.81279   -463.73596   -493.94254   -184.35564   -142.38766    230.00104    532.12128     529.8716    1090.4607    917.15785    669.67776    412.95958     61.36571   -280.71766   -237.73503   -162.71848   -159.86814    -314.8432   -244.82564   -340.56804   -453.51177   -398.17621   -380.40086   -476.96901          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       67.50   -341.04544    -450.2477   -325.98146   -582.86526     -86.0028   -540.95526   -607.43746   -126.48145   -289.32743   -197.24602   -310.41136   -332.82821   -380.22893   -415.22991   -419.19068   -444.94701   -504.49879   -505.63916   -232.32604   -353.29427   -210.69812   -243.69093     -286.778   -246.03855   -150.51797     16.61259    -13.17139    -42.50282    179.90803      6.34032    129.68444   -346.32089   -510.58773   -408.52022   -458.08451   -418.01871   -270.69279   -297.29129   -367.66581   -372.49212   -503.45265     -326.985   -244.08562    -415.0345   -252.45514   -426.00785   -339.56882   -375.60042   -205.97496   -517.85056   -332.77184   -234.55329   -328.84705   -413.38114   -211.29096   -161.61441       4.3752       7.3469    320.29265     81.27088     22.64109   -257.86794   -153.96024   -250.26634   -366.22776   -209.82348   -281.28175   -351.37327   -233.87842   -110.46926    176.26152    724.13059    851.98869   1073.20863    970.66916    813.43666    369.11952    273.37696     49.71038    -66.67966   -175.28844   -331.68882   -374.86571    -361.5649   -572.14234   -303.36101   -494.96067   -330.14895   -373.57327          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       68.00   -536.32861   -400.30382   -293.35603    -418.0715   -187.05266   -806.54423   -529.11803   -216.70796   -211.56141   -298.98896   -246.37443   -459.90769   -298.00273   -323.50467   -516.19095   -396.29807   -400.06418   -318.25751   -342.27757   -315.79024   -160.81849   -204.31416   -414.51643    -285.3277     -94.4631   -221.65001     50.21289   -150.22543    216.41761    -90.44499   -129.56036   -211.28836   -500.52475   -402.66467   -385.86043   -588.62003   -231.27255   -249.45705   -413.54796    -355.5781   -315.91045    -320.0554   -341.33525   -445.12436   -314.14558   -362.51499   -247.80541    -287.0968      4.44572   -341.18427   -246.69254   -316.11483   -456.81754   -299.99818   -223.15163   -186.79583   -112.46878     -38.4666    -11.74368    136.00425       18.398    -99.73291   -269.33711   -384.36056   -417.10522   -351.32236   -328.92558   -212.29446   -118.65681     12.01856     149.0433    395.14313    937.78152   1385.08419    845.71624    900.01724    279.98793    209.67491   -300.39118    -136.3391   -290.39511   -348.88864    -478.0071   -246.41996    -83.15761   -174.51446    -93.28506   -292.12546          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       68.50   -650.54553   -381.59155   -265.13497   -318.06539   -289.52415   -656.15827   -603.70622   -252.02701   -119.85159   -349.45183   -182.66224   -474.47868   -265.52857   -350.70788   -440.45001   -392.14715   -393.98759    -365.5385   -377.56034   -395.19425   -227.21767     -128.579   -455.97554   -101.46339   -128.62374   -131.07795    -93.88496   -182.48071      0.54751   -114.26658   -144.01951    -197.6163   -485.58107   -414.40447   -448.35023   -693.94196    -237.1839   -264.28854    -270.4245   -519.66636    -459.1737   -414.41906   -331.93119   -274.04746   -415.12599    -373.2195   -286.91068   -196.91639   -418.48531   -281.28554   -142.83617   -409.87319   -289.43077   -417.13258   -410.80071   -129.81896    -12.89829    184.99122    427.73118   -139.38752    249.23224     59.54265    -85.47462   -408.62577   -379.50018   -316.48158   -337.84792   -330.99789   -271.44777    -98.23537    123.33732    587.35392     607.1596    941.41553   1071.83915    751.54451    445.23582    186.35163     -0.76389    -71.96123    -84.43826   -420.14467   -486.45779   -200.91983   -312.91203   -371.85085   -306.70468   -357.17518          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       69.00   -625.39509   -502.10077   -284.11786   -340.65627      -233.54   -415.85131   -752.43173   -282.98983   -187.29613   -382.28892    -98.92165   -331.09836   -329.12465   -272.79383   -458.33029   -295.76059   -399.72056    -538.3659   -342.98888   -443.10344    -570.9876   -286.47611   -412.10378   -378.84518    -99.77048    -66.06832    -116.5046   -113.18028   -142.91877   -217.76997   -320.76779   -338.22586   -348.80367   -283.59396   -280.80364   -461.82927   -522.18483   -349.72249   -347.60084   -171.66543   -484.25179   -383.84468   -326.56663   -362.43912   -520.20894   -257.08575   -221.52192   -612.12455   -379.93884   -450.72903   -195.24676   -387.12101   -414.64928   -387.48637     -318.954   -329.10516   -163.01477     77.97194     25.18959     52.93888     58.37403     32.71966   -122.16097   -176.53275   -270.22314   -453.85369   -370.33341   -305.51662     17.20379    -71.82622       4.5253    584.36975     926.8047    891.54363    996.28308    834.13688    470.71953     188.8913    -168.3672     38.85863   -255.94145   -327.39446   -233.37268   -331.37402   -309.80628   -453.38644   -216.53602    -368.1557          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       69.50   -609.00835   -753.73919    -304.3705   -355.84839   -173.23677   -258.70029    -418.8203   -360.05212   -294.73359    -393.7278   -220.18179   -290.15896   -283.42365    -204.4417   -553.34339   -258.40904   -512.42806   -533.61206   -352.13869   -577.69405   -439.46209   -400.67948    -379.5202   -353.48154   -113.68893    300.49446   -305.50344   -130.47202   -152.81024    -15.01589   -386.45695   -195.55608   -226.64679   -229.26452   -230.21873   -374.68877   -372.96632   -486.43249   -380.30957   -175.87799    -89.52503   -272.43155   -169.90162   -387.33187   -282.07976   -276.50309   -218.56308    -477.9277   -426.62361    -428.6539   -330.58016   -375.46019   -453.15126   -325.16528   -250.46973   -253.74128   -151.65876    114.88957    212.46264     22.37269    188.87401     -6.36097   -425.33083   -259.89768   -148.65042   -319.86261   -248.29703    -88.40549   -389.35631   -106.34673     87.19691    613.74752    807.92758    981.80911    836.82883    992.60851    497.01653     93.52463    132.45842    -63.14547     90.76266   -208.38603   -332.34961   -184.94332   -382.53585   -219.28664   -510.51533          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       70.00   -577.09226   -666.27572   -328.40201   -297.89105   -169.86028    -218.1005   -131.58657   -415.86379   -389.49754   -431.94624   -327.84397   -204.34063    -92.21407   -259.33215   -435.42166   -466.18688   -572.00871   -370.40588    -401.4174   -661.62258   -282.31285   -371.12075   -525.34795   -124.27592   -177.15632   -147.94884    -87.95942   -134.99827   -181.40624    -69.60096   -343.75308   -266.76727   -476.31195   -280.65137   -460.59931   -236.91522   -527.19632   -208.98575   -544.97188   -223.86684   -428.93488   -382.60054    -362.6986   -276.13907   -417.62734   -355.24239   -219.94506   -297.51285   -337.20867    -348.7234   -268.96448   -320.08265   -224.83365   -425.19247   -166.28401   -198.09119    -52.59382    224.23021    -46.72143    207.21009    -21.37636   -125.46721    -66.56226   -232.57471   -276.57303   -265.05189   -299.82591   -349.30518   -245.79996    -43.68471    222.52096    527.52986     997.9939    789.14032   1008.77657    860.19059    655.02977     44.27416   -196.41091     39.78803   -239.60509   -174.44911   -272.67038   -337.33946    -425.0569   -320.42706   -327.18735          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       70.50   -282.04003   -488.78173   -364.49938    -288.6929   -135.53849   -228.81693   -189.76662   -385.00303   -423.61452   -424.77691   -308.10484   -136.06833    -33.56121    -345.4012   -321.38588   -522.87269   -220.28865   -175.41271   -330.53383   -445.18856    149.90554   -393.82008   -454.06114   -146.61067   -230.72894   -168.99594   -248.53286    -95.83634   -144.03689    104.04508   -142.35845   -265.04249   -209.76243   -414.05409   -491.50062   -300.71105   -312.67633   -215.44592   -409.05114   -563.63201   -290.57979   -241.48964   -362.58792   -506.17832   -321.83339   -345.53514   -323.56054    -338.5511   -247.91729   -449.87269   -344.08508    -349.6984   -501.25068   -474.58183   -265.77517   -116.45764   -126.60128   -103.25587     80.77871    343.51143      4.02353    -191.5072   -234.02847   -346.56679   -210.44126   -379.41157   -275.28532   -152.19219   -207.72772    -67.11567     360.9633    534.39578    730.16754    773.28609    808.63395    541.49282    358.21133    166.14125    -65.13645   -266.58113   -298.13538   -316.49275   -225.56785   -476.04565   -396.04034   -506.20943   -478.66714          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       71.00     -50.6558   -334.45904   -430.77113   -262.27594    -79.61574   -244.52293   -260.32992   -326.12921   -314.65147   -378.70001   -342.39231   -259.13326    -82.65495    -277.4383   -274.43999    -382.9538    -24.60152   -333.33492    -285.5905   -282.84567   -152.86769   -227.69041   -319.50883   -207.70822   -438.70126   -163.92048   -232.39505   -195.26203   -196.09885    105.01183    -214.1217   -491.04069   -290.03685   -416.92024   -299.45025    -314.5144   -446.95485    -279.0857   -323.16848   -483.12094   -377.59764    -260.3448    -522.5956   -411.32122   -285.35184   -152.07549   -292.05988   -334.03546   -235.68028    -456.4366   -416.07055   -332.39688    -464.3064   -212.34013   -442.81654     -27.6755      -43.552    185.64084    135.77611    214.46109    127.65829   -156.73071     169.4035   -412.22278   -456.83259   -353.38376   -377.91876   -393.89147      -90.064   -183.19007    188.39586     510.2851    814.35763    876.43189   1059.23411    716.30145    378.24183      50.5432    -29.49167   -297.36571   -429.09864   -213.88933   -333.79463   -172.28653   -347.66454   -430.18665          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       71.50    101.25548   -449.65682   -448.10064   -281.67861   -252.60336   -149.06568   -248.32675   -398.67354   -258.33169   -360.89638   -384.97369   -271.28232   -211.88601    -95.55972   -165.35137   -305.00093   -154.36053   -392.49961   -275.49117   -509.02053   -372.64663   -187.29414   -345.75416   -344.07328   -246.56024   -226.42126     21.28741    -109.0713    -114.0297     76.37113   -298.09552    -379.3118   -347.88312   -404.51862   -317.92939    -328.0931   -448.62129   -388.16054   -214.26484   -308.66065    -279.7529   -396.15605   -486.16308    -538.4146   -332.54493    -260.5081   -259.81947   -243.27206   -420.22989   -474.43532   -312.18075   -454.08625   -348.99993   -244.08434   -319.35689    -98.02488   -277.71799    -41.14333     41.22411      52.6306     28.79913     43.38523   -170.88935   -444.73319   -403.03946   -446.45259   -463.11238   -499.21908   -146.96023    -25.02375     32.33593    569.88704    760.22044   1146.56626    979.95619    685.67651     661.3168    248.44576     89.36862   -150.79288   -411.26456   -517.33205   -336.90361   -301.33021   -313.61694   -637.37987          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       72.00    -64.79826   -530.63647   -392.97504    -425.5786   -495.43147   -155.73721   -244.92348   -561.65667   -262.02181   -226.42324   -416.59389   -204.16148   -268.17468   -113.65531   -410.82822   -263.51085   -314.20781   -231.60311   -281.21415   -416.59006   -316.08868    -209.4509   -363.21746   -431.15979   -151.63406   -168.36827      62.0984     63.66048     76.48471   -158.93718   -356.04506   -198.15313   -369.79648   -399.07166    -390.8796   -363.30161   -284.52399   -535.33315   -287.48934    -351.6095   -238.68307   -484.85743   -403.50774   -414.01014   -446.21526   -231.05959   -442.28761   -319.35289     -498.708    -316.3776   -369.57719   -495.19576   -318.86573   -349.01707   -281.91627   -331.84637    -37.06043    -60.92572    222.07498    -64.18524   -125.08459     57.32041   -184.46634   -468.37406   -459.51675   -627.27596   -190.86304   -294.75576   -268.28673    -60.25179    268.06283    452.53049    960.75558    987.58288    982.54153    728.30072    444.67146    232.75604    183.68809   -199.01833   -312.91615   -587.67246   -274.68831    -377.0771   -475.20359   -693.24813          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       72.50   -176.12358   -550.56933   -378.88658   -582.00571   -497.07333   -358.49186   -298.41704    -353.2178   -253.28034   -160.21251   -443.67397   -153.74629   -314.38996   -291.30983    -492.7811   -272.08427   -499.43623   -354.67989   -242.33779   -344.57101   -206.00292   -494.70723   -319.18094   -336.57957   -149.92702   -242.86249      59.0966      6.58706   -133.22451   -227.12803   -282.70873   -428.86507   -320.34535   -312.33166     -228.495    -398.0661   -215.28668   -429.16039   -418.47544   -327.68751   -490.49299   -289.19089   -244.92765   -355.65699   -207.40692    -591.0951   -305.73518    -392.1356   -274.53314   -510.56121   -421.22449   -352.28617   -444.64474   -321.55833   -257.00139   -270.24586    162.29548    101.09477    139.64063    -72.84577     38.12716   -114.68275      26.5997   -371.47111   -276.37967   -427.30176   -356.33076   -366.04207   -280.14183     -60.7877    372.21309    737.94987    745.56064    793.44065    965.39694    617.24697    524.63384    295.80323    -90.99698    -99.13112   -210.64455   -305.92243   -468.58075   -400.22994   -268.66129          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       73.00    -231.8682   -463.87297   -447.20347   -634.07715   -449.71129   -453.85797   -398.88141   -150.84249   -479.91386   -403.10942   -561.78916   -401.02029   -418.96234   -281.99227   -467.54612   -366.92703   -505.52872    -479.2963   -304.96775   -598.44326    -168.8035   -303.59701   -167.63406   -240.83083     37.33932   -309.70178      4.47785   -201.85933    -84.13968   -101.47044   -293.62142   -206.75224   -276.76932   -306.20637    -546.5462   -508.19509   -538.96029   -266.57193   -399.58902   -241.20983   -369.58857   -312.26711   -195.70657   -317.49098   -207.14025   -343.24182   -417.04034   -407.50091   -285.13896   -407.31732   -148.91227   -385.77064   -342.76002   -437.79485   -328.13847   -201.05334   -113.06574    205.78182    138.97078    -79.35218    154.90903   -346.22214    -76.88534   -394.74043   -193.27212   -245.55125   -373.89566   -231.56723   -242.36953   -249.08772      -2.1556    667.41085    979.76215   1007.95322       799.09    782.28388    371.25443     10.06753   -136.09249   -193.30753   -383.02947   -120.24436   -340.35159   -319.36687   -404.37883          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       73.50   -175.97313   -387.69853    -471.0307    -607.4069   -477.63674   -393.19865   -495.23499   -159.73743   -728.99403   -557.65563    -722.2339   -682.74024   -384.26166   -251.46962    -429.0061   -359.70442   -470.33939    -474.5304   -420.80326   -443.60011   -312.38369   -231.77852   -342.22555   -255.82821   -171.09939   -364.46042    -57.32279   -200.51456     84.15861   -232.60767   -555.39088   -127.84256   -482.13045    -295.9344   -521.72776   -427.82989    -623.8689   -363.00839    -186.6106    -281.3252   -105.29162    -36.04921   -396.05897   -429.16771   -321.18417   -450.81566   -359.48073   -182.76544   -257.94746   -443.51392   -490.18294    -419.6868    -58.83552   -382.97978    -72.09585   -189.84985      91.6793   -129.50632   -141.18163    142.38706     94.39513    -10.19631   -172.36541   -215.61297   -436.50172    -266.7354   -332.99052   -600.35127   -235.84828   -116.06867    266.97959     428.0228    889.95836    783.26336    686.20453    742.79105    359.97983     85.83205     -8.14655     34.29232   -252.99766   -465.99728   -399.19514   -126.88234   -430.99766          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       74.00    -74.77436   -331.71299   -299.55637   -559.74049   -511.99877   -378.71216   -372.52125   -177.97183   -678.52463   -453.44273   -756.74951   -476.34077   -355.02861   -379.83038   -256.84129   -401.31502    -304.1179   -466.97513   -224.52018   -226.42365   -276.86304   -313.69583   -356.50938     -372.055   -279.10448   -126.82776   -175.74624   -277.68054   -222.73502    -68.06243   -451.87302   -356.50181   -580.87406   -215.90547    -476.9011    -225.5176   -451.50356   -477.06037   -234.08804   -389.70532   -507.23764   -355.00694   -350.59311    -573.4559   -426.61372   -558.93497   -401.23813   -236.99102   -224.94956   -382.32627   -424.70914   -255.79931   -216.59063   -421.63555   -281.52915    -123.3051    -49.98564    138.80405    149.41236    320.44683    -10.05049   -156.25841     -245.443   -293.53869   -297.51376   -272.74162   -112.94515   -209.55638   -342.81253   -107.33492    130.79299    278.93686     798.2766   1050.04196   1015.99759    601.31747    711.32181    158.35137   -193.12876   -128.17333   -339.59893   -251.37574   -357.04632   -464.89854          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       74.50      41.2433    -309.6117     15.19568   -482.30488   -456.81364   -497.89117   -164.47701   -276.39778   -466.17426   -307.75077   -510.22553    -342.1997   -470.69157   -580.54621   -337.32203   -580.15364   -205.14982   -403.11227   -197.15547   -235.44657   -315.03287    -326.2528   -547.27829   -256.51056     71.94064   -136.70018   -291.30582   -246.97654   -132.32825    -29.15097    -12.54662   -472.48971   -346.69441    -482.7168   -368.81674   -673.91136   -397.39957   -257.01683   -404.55467   -316.34315   -365.78665   -559.41371   -562.81256   -566.34775   -507.03193    -431.6537   -297.97131   -408.43548    -394.6913   -358.52735    -192.7283   -460.84995   -489.17295   -439.28252   -304.27536   -195.86583    -75.23185    -64.98925    146.16688      1.42974    -54.67283   -129.38933   -188.53522   -182.46113   -402.66274    -78.60808   -300.27155   -398.27644   -129.27379     16.72142     68.12738    469.46638    749.02723    938.02888   1113.38252     769.5446    537.39297     169.6132     49.27633   -146.03247   -154.27238   -356.71414   -459.81631   -308.89794          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       75.00   -285.50089   -306.68855   -146.84042   -332.68713   -392.03698   -616.08787    -71.79328   -501.98493   -387.47286   -190.80388   -328.52532   -328.44658   -469.48999   -540.60007   -489.98926   -256.07076   -226.42808   -319.22976   -476.26948   -326.95776   -471.05228   -315.31054   -557.18847   -278.52211   -276.13044   -248.30079   -217.63288   -143.23801    -295.6394   -221.39895    -61.86687   -424.69477    -376.4569   -295.28079   -217.32727   -380.59649   -396.14952   -415.68639   -424.71364   -413.37247   -399.96246   -503.37273    -75.02178   -501.98703   -395.53863   -481.79319    -356.8466   -178.16551   -256.87053   -306.02865   -158.80571   -512.49803   -593.08318   -197.05518   -374.03239    -86.58904    -13.91211     61.30068   -114.86167    -67.70702     50.20435    -70.60379    -88.19443    -189.4233   -541.99121   -160.89929   -668.27477   -288.84369   -279.58257      59.5088    115.19555    420.11956    583.05237   1107.72429   1248.57699    956.88244    571.98642     172.8537    266.55349   -226.93345   -260.31543   -378.39524   -284.38872   -561.16859          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       75.50   -573.78921   -386.81704   -491.56854   -191.99054    -372.0906   -627.42892   -131.78552     -708.217   -583.44531   -184.59304   -360.09422    -314.7292   -321.97421   -434.09141   -561.72414   -164.29456   -388.16692   -256.36801   -436.91286   -381.20088   -319.01859   -295.69261   -464.60967   -437.68602   -392.79962   -307.17881   -158.43143   -249.52899   -311.44278    -201.7933   -208.80352   -125.09794   -308.46514    -259.1322   -363.05393   -553.56869   -232.47812   -402.46554   -337.47626   -479.80525   -216.63185   -519.28213   -220.90422   -333.97501   -340.87547   -214.84445   -326.92772   -399.14291   -304.33945   -373.01849   -362.23243   -320.54194   -452.41848   -281.61611   -260.24516   -306.91502    -96.29131     74.15353     55.19038    153.43329    348.67741   -441.42329   -171.55941    -101.9195    -13.12358   -410.78177   -349.74742   -104.88336   -277.90336    -161.3834     76.40838    443.23816    697.02235    842.47478    1109.9253    807.64791    631.36914    209.69739   -169.97622   -170.15442   -273.88572   -143.34914   -314.76005   -337.85612          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       76.00   -753.84116   -410.02929   -758.31151   -180.77699   -415.33862   -530.27378    -212.4172   -770.23504   -770.81702   -285.98437   -395.20976   -401.00959   -251.18669   -349.54433   -559.28775   -159.64155   -346.12184   -330.82399   -387.80458   -369.18604   -150.56782   -300.57362   -415.57982   -329.37903   -410.62104   -172.22652   -312.59811     67.79681   -195.94778   -312.10173   -421.79749   -348.96725   -263.78388   -263.50394   -405.59401   -214.30323   -422.79484   -439.28662   -375.54121   -462.02627   -297.30938   -419.94265   -294.98648   -309.18861   -375.33979   -621.18755   -571.36276   -405.30749   -221.73661   -477.32389   -398.28207    -298.4916   -342.16524   -265.67632   -282.26824     -276.573     -207.469     74.46643    -68.11298     25.16579   -288.67876   -102.01883     -38.7655   -328.96162   -267.19189   -405.66136   -441.69446   -280.76029   -249.22355     -88.1822      258.096    618.10963     573.2816    685.48761   1016.10165    562.41006    467.93088    423.93572    107.50588    -80.15776    -82.45881   -290.04379   -395.37915          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       76.50   -592.95537    -334.0248   -506.19522   -155.80894   -590.73681   -425.52758   -275.07147   -712.36879   -619.20569   -381.33863   -479.19359    -439.7278   -343.04709   -347.59794   -332.55527   -221.61751   -228.11296   -432.99792   -271.07934   -424.90324   -204.07824   -162.20415   -466.37812   -196.46764   -325.31774   -214.80355    -483.5382    -29.54544     70.85456   -190.47789   -404.61634   -451.58203   -317.12739   -337.53897   -407.06773   -176.85312   -524.41856   -414.51802   -295.75577   -350.21216   -352.17508   -403.42382   -475.87833   -360.79584   -252.96859   -526.55076   -494.18124   -522.88445   -387.94592   -517.39218   -451.75505    -73.13789   -328.32885   -172.19024   -366.49585   -244.42261     -30.5027   -232.03634    -13.46549   -149.85171    -58.85826    162.08237   -204.95917     23.92905   -265.69561   -322.24471   -206.21691   -121.57691    -91.02796   -153.42707     -7.31677      691.914    659.58161    825.74181   1129.94687    896.58557    458.47334    263.69903    224.89789   -118.82859   -230.37192   -308.35865   -456.34082          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       77.00   -520.39163   -345.29163   -306.77993    -266.6209    -747.0445   -434.13809   -288.20983   -374.29434   -485.52503   -439.67903    -592.9038   -283.25935   -380.04171   -391.68112   -304.37354   -374.53948   -201.85909   -611.97914   -261.58281   -450.48555   -357.09169   -162.56361   -249.56305   -282.06569   -207.52963    -111.8796     59.31094   -187.59408   -187.72744    -81.59288    -490.4423    -385.2334   -324.49213   -393.15082   -396.87666   -267.02106   -360.66168   -289.51631    -406.1463   -255.97845   -200.47827   -324.54465   -516.07482   -276.64134   -297.18185   -442.89552   -314.31665   -193.97565   -355.64668   -312.16567    -87.88633    -266.9333   -295.05347   -186.59024   -501.66442   -324.35073    -85.09943    -125.6929   -246.35247     12.13434    179.74866    -49.87002    -68.54839   -382.96914    -462.0597   -249.81525   -201.71893     -23.7876   -173.29989   -172.62441    101.23124    449.34223    646.13507    933.37316    756.05743    769.95643    682.64167    388.63814    132.70271   -196.05903   -559.36179   -255.82696   -367.77688          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       77.50   -507.57969   -408.84398   -300.85899   -355.85111   -589.25525   -450.36505   -431.57486     -98.7073   -471.68427   -399.09773   -591.74133   -209.19334   -250.69111   -478.80177   -380.10366    -414.7731   -177.72906   -162.54847   -322.66142   -207.52052   -501.38192   -490.48162   -183.12511   -563.49031   -367.07886     44.02749      77.0018   -295.64581   -229.07094    -96.76668   -569.22241   -337.55935   -452.49037   -290.34228   -319.78232   -190.26364   -229.98844   -239.35139   -468.45935   -578.80812   -400.08778   -606.49559   -539.05971   -392.24124   -423.42871   -491.26374   -330.67568   -477.43066   -321.80301   -369.43786   -263.37911   -265.09897    -352.2268   -362.29196   -160.15009   -246.34582   -102.84225     28.19431    -87.03605    179.67938    189.80219    -10.09596    -120.8623   -363.54954   -135.47919   -138.92565    -260.0999   -282.73948    -392.6335   -252.56254     15.32842    474.34704    651.82384   1065.18603    773.16846    795.06308    555.34001      88.2469    115.15473    -187.3733   -133.80707   -520.37949   -362.17842          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       78.00   -393.90653    -471.1761   -434.48506   -377.71447   -322.48112   -435.39955   -562.11176   -287.82468   -444.85585   -152.28034   -557.85496   -233.69739    -179.4975   -549.57127   -335.87615   -357.33723   -418.51012     13.56793   -364.21317    -238.3268   -523.17409   -467.75922   -279.63061   -382.77619   -369.69921     -51.2524    115.72383     40.27157   -261.90151   -137.52855   -268.04298   -298.52565    -308.4803   -291.09216    -419.5409    -74.31105   -407.06959    -365.8019   -449.52849   -437.02155   -327.55466   -446.30026   -494.06221   -274.90251   -159.61435   -465.85813   -437.19521   -405.51274   -235.70971   -252.87669   -296.46542   -309.13502   -368.19737   -326.55606   -440.74931   -262.07699    -18.10161      33.2659    -68.17101    124.23218     -60.3785     52.81706   -256.62952   -127.95457   -214.41962   -324.97289   -216.73691   -363.12502    -508.8128   -138.27526    220.26881    539.48809    739.70397    890.83691   1137.12535    900.57843       531.62     39.66111    102.67586    -192.7496    -32.35845   -219.71107   -336.57697          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       78.50   -262.73528   -471.31591   -542.45048     -383.857   -155.10342   -407.25917   -433.09187   -400.18332   -414.51497    -60.01097   -476.62939   -217.14062    -44.00178   -423.19976   -295.35907   -237.41669   -405.71935   -424.93699   -481.96219   -431.71057    -227.8966   -244.07832   -677.63334   -400.39005     27.39385   -129.94314    -34.22185     61.51995   -341.16574   -348.25606    -87.33811   -453.36731   -119.80329   -482.05104   -458.72426   -175.37917    -270.9746   -364.12821   -425.08245   -170.92847    -261.5603   -332.54184   -262.16831   -315.54456   -329.18813   -226.49133   -582.45439   -285.69292   -374.96929   -337.84132   -273.91744   -216.42751    -292.2727   -389.89326   -474.59536     -59.9783   -156.97284    173.71585    -49.55259     67.94137    -95.48435    -83.34892   -126.87984    -138.6753   -188.02119    -165.7463   -152.79247   -162.48385    -188.1274    -19.20448     216.2545    417.04323    618.21507    966.22901    974.08551    677.65881    536.20764    159.94755     87.59795    -55.00652   -460.52048   -363.29334          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       79.00    -178.1685   -498.69316   -472.17243   -461.38113   -243.72117   -373.78585   -289.15227   -387.31971   -351.34077   -312.20865   -446.41496   -255.90909   -201.27775    -354.4176   -420.73863   -301.28618    -351.3076   -589.23354   -560.08998   -427.84608   -118.95553    -23.58755   -395.82683   -448.84036   -210.00894   -146.24634   -559.39202   -278.08308   -254.93087   -133.57867    -65.03348    -474.3535   -407.61922   -641.18182   -604.54779    -295.8767   -264.33333   -367.64147   -227.94414   -334.62999   -247.33147   -317.13993   -282.69104   -454.05421   -349.50431   -347.38328   -374.19983   -354.73517   -314.16159   -355.72978   -234.85158   -418.40872   -421.62729   -271.87164    -225.7674   -246.68776   -251.29033    139.17163   -234.07025   -132.90725   -146.07551    -72.27607   -231.45613    -57.63403   -396.58407    -354.6819   -345.05307   -119.80762   -155.50142   -214.69377    230.56688    295.50254    751.26965    925.40583    965.79702    889.53477    854.02271    257.89305     43.37685   -164.49279   -463.54778   -387.70498          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       79.50   -287.45559   -546.22093   -383.06944   -587.54894   -363.66208   -322.13032   -343.21509   -415.16967   -409.28651   -488.88273   -458.96937   -336.49369   -421.09735   -385.44096   -390.09492   -466.94895   -522.60765   -613.88951   -497.05947   -323.33903   -201.48963   -108.26404   -252.51096   -504.82992   -297.61016   -182.84377   -213.95282   -108.52139     -6.66797   -117.55321   -170.33301   -216.28552   -388.10576   -230.05874   -353.12714   -189.45105    -196.5413   -320.66811   -337.43744   -309.54301   -250.47897   -283.23852    -514.4869   -421.21706    -76.59216   -402.95028   -344.10448   -568.54598   -372.48154   -607.67075   -441.73791   -283.39358   -158.30192   -246.87306   -407.26165   -313.18924    -40.10245   -257.80238     27.09079    122.10527    -71.35651   -156.56859   -243.14867   -437.70722   -253.24515    -367.7095   -373.05421   -298.91964    -169.9551    -59.86276     63.56775     244.3065    746.23103    683.60916    989.37809    905.73545    551.73895    517.14106    304.25223    -49.42446   -235.37966   -442.89694          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       80.00   -395.76694   -561.04948   -338.79818   -706.67349   -389.11088   -261.42105   -443.32649   -502.40633   -475.61183   -418.41888   -523.51828   -346.06912   -269.28778   -402.03748   -199.59784   -337.79341   -562.49033   -440.59309   -481.44118   -213.27917   -124.49945   -210.58767   -202.32467    -417.5261   -333.18395   -252.78981    -108.7103    122.90113    -49.75427   -178.47136   -136.01417   -479.72397   -293.97578   -262.17154   -311.56584   -289.24958   -197.98236   -325.64691   -208.20984   -661.78623   -300.17047   -325.55231    -605.3566   -314.92448   -338.32772   -315.67558   -508.12164   -386.11135   -327.18671   -673.66207    -309.0745   -403.42719   -282.64654   -381.64163    -201.8155   -288.87429   -116.76377    -66.27384    100.03828     54.47494    -90.46562   -157.22759    -331.1781   -218.72018   -411.05132   -265.25456    -525.0909   -319.23204   -332.30377   -110.94878    -13.87346    270.46613    650.60389    772.84669    916.66506    783.16923    551.84522    220.96889      27.4361     76.35868   -252.47037   -374.27081          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
;

_pd2d_meas_2theta_phi_intensity_down_sigma
;
         241   4.00         4.20         4.40         4.60         4.80         5.00         5.20         5.40         5.60         5.80         6.00         6.20         6.40         6.60         6.80         7.00         7.20         7.40         7.60         7.80         8.00         8.20         8.40         8.60         8.80         9.00         9.20         9.40         9.60         9.80        10.00        10.20        10.40        10.60        10.80        11.00        11.20        11.40        11.60        11.80        12.00        12.20        12.40        12.60        12.80        13.00        13.20        13.40        13.60        13.80        14.00        14.20        14.40        14.60        14.80        15.00        15.20        15.40        15.60        15.80        16.00        16.20        16.40        16.60        16.80        17.00        17.20        17.40        17.60        17.80        18.00        18.20        18.40        18.60        18.80        19.00        19.20        19.40        19.60        19.80        20.00        20.20        20.40        20.60        20.80        21.00        21.20        21.40        21.60        21.80        22.00        22.20        22.40        22.60        22.80        23.00        23.20        23.40        23.60        23.80        24.00        24.20        24.40        24.60        24.80        25.00        25.20        25.40        25.60        25.80        26.00        26.20        26.40        26.60        26.80        27.00        27.20        27.40        27.60        27.80        28.00        28.20        28.40        28.60        28.80        29.00        29.20        29.40        29.60        29.80        30.00        30.20        30.40        30.60        30.80        31.00        31.20        31.40        31.60        31.80        32.00        32.20        32.40        32.60        32.80        33.00        33.20        33.40        33.60        33.80        34.00        34.20        34.40        34.60        34.80        35.00        35.20        35.40        35.60        35.80        36.00        36.20        36.40        36.60        36.80        37.00        37.20        37.40        37.60        37.80        38.00        38.20        38.40        38.60        38.80        39.00        39.20        39.40        39.60        39.80        40.00        40.20        40.40        40.60        40.80        41.00        41.20        41.40        41.60        41.80        42.00        42.20        42.40        42.60        42.80        43.00        43.20        43.40        43.60        43.80        44.00        44.20        44.40        44.60        44.80        45.00        45.20        45.40        45.60        45.80        46.00        46.20        46.40        46.60        46.80        47.00        47.20        47.40        47.60        47.80        48.00        48.20        48.40        48.60        48.80        49.00        49.20        49.40        49.60        49.80        50.00        50.20        50.40        50.60        50.80        51.00        51.20        51.40        51.60        51.80        52.00        52.20        52.40        52.60        52.80        53.00        53.20        53.40        53.60        53.80        54.00        54.20        54.40        54.60        54.80        55.00        55.20        55.40        55.60        55.80        56.00        56.20        56.40        56.60        56.80        57.00        57.20        57.40        57.60        57.80        58.00        58.20        58.40        58.60        58.80        59.00        59.20        59.40        59.60        59.80        60.00        60.20        60.40        60.60        60.80        61.00        61.20        61.40        61.60        61.80        62.00        62.20        62.40        62.60        62.80        63.00        63.20        63.40        63.60        63.80        64.00        64.20        64.40        64.60        64.80        65.00        65.20        65.40        65.60        65.80        66.00        66.20        66.40        66.60        66.80        67.00        67.20        67.40        67.60        67.80        68.00        68.20        68.40        68.60        68.80        69.00        69.20        69.40        69.60        69.80        70.00        70.20        70.40        70.60        70.80        71.00        71.20        71.40        71.60        71.80        72.00        72.20        72.40        72.60        72.80        73.00        73.20        73.40        73.60        73.80        74.00        74.20        74.40        74.60        74.80        75.00        75.20        75.40        75.60        75.80        76.00        76.20        76.40        76.60        76.80        77.00        77.20        77.40        77.60        77.80        78.00        78.20        78.40        78.60        78.80        79.00        79.20        79.40        79.60        79.80        80.00        80.20        80.40        80.60        80.80        81.00        81.20        81.40        81.60        81.80        82.00        82.20        82.40        82.60        82.80        83.00        83.20        83.40        83.60        83.80        84.00        84.20        84.40        84.60        84.80        85.00        85.20        85.40        85.60        85.80        86.00        86.20        86.40        86.60        86.80        87.00        87.20        87.40        87.60        87.80        88.00        88.20        88.40        88.60        88.80        89.00        89.20        89.40        89.60        89.80        90.00        90.20        90.40        90.60        90.80        91.00        91.20        91.40        91.60        91.80        92.00        92.20        92.40        92.60        92.80        93.00        93.20        93.40        93.60        93.80        94.00        94.20        94.40        94.60        94.80        95.00        95.20        95.40        95.60        95.80        96.00        96.20        96.40        96.60        96.80        97.00        97.20        97.40        97.60        97.80        98.00        98.20        98.40        98.60        98.80        99.00        99.20        99.40        99.60        99.80       100.00       100.20       100.40       100.60       100.80       101.00       101.20       101.40       101.60       101.80       102.00       102.20       102.40       102.60       102.80       103.00       103.20       103.40       103.60       103.80       104.00       104.20       104.40       104.60       104.80       105.00       105.20       105.40       105.60       105.80       106.00       106.20       106.40       106.60       106.80       107.00       107.20       107.40       107.60       107.80       108.00       108.20       108.40       108.60       108.80       109.00       109.20       109.40       109.60       109.80       110.00       110.20       110.40       110.60       110.80       111.00       111.20       111.40       111.60       111.80       112.00       112.20       112.40       112.60       112.80       113.00       113.20       113.40       113.60       113.80       114.00       114.20       114.40       114.60       114.80       115.00       115.20       115.40       115.60       115.80       116.00       116.20       116.40       116.60       116.80       117.00       117.20       117.40       117.60       117.80       118.00       118.20       118.40       118.60       118.80       119.00       119.20       119.40       119.60       119.80       120.00       120.20       120.40       120.60       120.80       121.00       121.20       121.40       121.60       121.80       122.00       122.20       122.40       122.60       122.80       123.00       123.20       123.40       123.60       123.80       124.00       124.20       124.40       124.60       124.80       125.00       125.20       125.40       125.60       125.80       126.00       126.20       126.40       126.60       126.80       127.00       127.20       127.40       127.60       127.80       128.00       128.20       128.40       128.60       128.80       129.00       129.20       129.40       129.60       129.80       130.00       130.20       130.40       130.60       130.80       131.00       131.20       131.40       131.60       131.80       132.00       132.20       132.40       132.60       132.80       133.00       133.20       133.40       133.60       133.80       134.00       134.20       134.40       134.60       134.80       135.00       135.20       135.40       135.60       135.80       136.00       136.20       136.40       136.60       136.80       137.00       137.20       137.40       137.60       137.80       138.00       138.20       138.40       138.60       138.80       139.00       139.20       139.40       139.60       139.80       140.00       140.20       140.40       140.60       140.80       141.00       141.20       141.40       141.60       141.80       142.00       142.20       142.40       142.60       142.80       143.00       143.20       143.40       143.60       143.80       144.00       144.20       144.40       144.60       144.80       145.00       145.20       145.40       145.60       145.80       146.00       146.20       146.40       146.60       146.80       147.00       147.20       147.40       147.60       147.80       148.00       148.20       148.40       148.60       148.80       149.00       149.20       149.40       149.60       149.80       150.00      
      -40.00     237.2916    222.90559    212.65555    217.79453    208.25199    207.71034    204.62591    189.20726     196.2075      191.181    182.15662    186.91469    183.46798    180.76988    182.02314    178.09141    172.34672          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -39.50    234.99514    225.90897    211.71155     212.5608    210.38648    208.84035    207.17419    191.82586    195.99659    188.13091    183.56609    187.40039    183.67067    181.74713    182.95553     173.7564    170.34776          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -39.00    233.80398     228.7762    214.72315    209.14905    213.64083    208.22969    204.20155    192.73882    196.39422    186.34727    181.52013    190.95958    182.29433    181.51769    180.58583    171.19381    166.34947    167.30016          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -38.50    233.87984    229.14869    217.53735    213.92003    218.01226    207.97251    199.22019    194.66694     196.0396    190.23092    181.15152    193.80269    183.56313     187.3827    176.23925    173.16359    165.47053    164.61066          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -38.00    232.78165    227.74871    219.64884    217.10616    220.32917    209.84862    195.34622    198.26788    192.02462    195.26062    182.89396    189.07132    182.77663    188.40795    175.35322    177.53719    166.12657    165.46672    167.83536          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -37.50    231.37783    226.02675    221.29551    218.67954    220.09428     212.2001    192.06582    201.08275    187.55124    191.19248    188.96499     187.7272    183.72916    187.99732    176.40542    179.22786    169.13076    176.81586    168.24998          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -37.00    230.89036    226.22907    222.16731    220.06643    218.34057    211.98969    188.17822     196.6518    185.24499    187.22681    190.99887    190.74869    184.03809    188.35879    173.18766    174.12741    170.05354    172.41278    170.00426          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -36.50    233.52875    227.71865    221.50448    220.26853     216.6537    208.80179    187.46522    191.62271     186.1263    187.96268    190.20752    190.31612    181.98656    185.72863    174.13481    169.79411    169.46182    162.98703    172.89022    170.41137          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -36.00    235.89537    225.32377    220.15954    222.14403    215.10063    204.31237     194.1774     188.4924    186.15928     193.0972    195.90557    185.82065    181.96205    181.47718     175.4813    175.50347     170.0877    165.89659    172.65638    169.89297          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -35.50    237.01398    224.31862    221.23556    220.79853    213.14964    200.55709    196.15169    191.47671    189.26609    192.86082    199.79279    189.72325    183.59915    177.36539    172.92101    173.17034    168.83813    170.28551    171.08734    166.51467    164.42876          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -35.00     236.5119    224.84589    223.44345    219.23856    210.10553    199.99493    194.87908    195.48041    192.47665    189.00124    192.70536    191.79266    183.72692    175.60504    173.63485     169.1063    170.80336    169.80496    170.34643    170.88474    157.53481          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -34.50    233.51418    225.29107    226.02907    219.18806     205.1501    198.41172    194.58751    198.99438    194.47995    187.70264    190.61231    187.75276     182.2544    177.77588    171.36311    170.37271    170.74863     172.8301    171.75186    172.01362    161.06423    159.55936          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -34.00    232.43214    223.76321    225.07836    221.70949    203.34345    195.82886    193.52885    201.11657    195.72559    189.69435     187.4024    183.46631    182.11354    179.73067    172.33671    172.51531    169.78213    174.35088    172.41893    167.97267    164.26238    161.74186          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -33.50    233.24563    223.70695    224.12486     223.5919    206.67983    197.25464    193.45635    199.58702    196.63486    186.95875    183.78415    182.96518    183.32048    174.52919    175.19113    175.33468    170.88029    173.33957     174.1222    167.21617    164.34526    163.24932    162.68211          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -33.00    237.44468    226.50006    222.46811    224.28832    205.95462     199.7462    193.77056    198.59893    197.69262    187.73092    184.45076    185.66012    182.41865    170.65889    175.58672    177.80934    166.69666    173.74562    170.99044    166.84039    166.18769    166.09721    163.01438    161.05324          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -32.50    240.65345    231.98456    219.85633    222.53505    202.53875    201.56795    194.49208    199.28696    197.15145    191.26659    187.30192    187.00891    183.57613    173.66599    173.33206    176.90621    163.89452    171.89893    169.94974    162.85996    168.71432    163.43729    161.40434     154.5324          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -32.00     241.2572    237.05953    218.99322    221.11626    200.86823    204.77274    195.79684    199.34658    191.35966    193.14514    190.86577    188.12901    183.17067    178.70808    172.39773    177.45791    169.88658    170.36428    171.04316    163.25004    170.80189    159.53418    158.73206    159.26876    162.08386          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -31.50    242.07136    239.59399    219.85686    222.00876    204.15899    210.33621    195.88506     200.2801    182.52183    196.86296    193.67918    186.21121    181.85054    181.56604    175.87212    178.99785    173.63496    169.34216    169.11247    163.06116    167.92927    157.65172    153.05568    165.15158    165.10808          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -31.00    243.26474    236.35834    220.83552    224.10195    205.67184     208.1585    195.46767    202.01755    182.54721     200.7851    192.22076    182.92121    183.45159    183.12399    177.13898     177.7862    173.83632    170.38484    167.85272    163.34328    165.93359    154.01757    163.55133    161.03187    170.12731     166.7499          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -30.50    245.89755    233.95119    221.35802    224.71728    205.35856    203.53088    197.42826     201.0329    191.61982    192.05595    185.31147    184.21653    180.06723    182.66756    173.48681     177.6126    174.15345    175.64124    167.31474    165.62942     166.8178      152.951     164.0694    159.06388    168.24415    168.93921    166.42658          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -30.00    249.91393    232.36081    221.25737    223.07499    203.93018    202.51366    200.28253    198.29686    198.01671    185.45571    183.84132      187.708    175.57267    182.72454    169.57364    180.87347    177.96048    180.46739    169.43342    169.85101    164.78608     160.4081    165.04154    157.09244    158.57317    166.69776    167.95857          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -29.50    247.32367    236.18817    224.68754    221.59323    204.25268    202.60551    202.93807    199.09248    198.08573    185.81456    189.78087    188.54759    175.20824    180.66523     171.0253    177.97324    178.19448    175.37367     171.2801    167.15514    160.24583     161.6739    168.12305    158.00874     157.9937    164.30463    170.51779     163.9061          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -29.00     243.9993    240.41602    226.21205    219.90804    205.33581    201.24502    200.62133    201.20384    195.78993    187.25406    191.37288    188.08615     177.0151    179.09007    177.38317    172.03799    174.51353    172.59514    169.43339    160.12253    159.24742    162.37226    168.22451    161.80388    163.36371    165.59796    170.85739     166.4654    165.80186          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -28.50    241.79344    241.31533    225.75327    217.91097    207.60786    198.59977    194.53811    202.22205    192.57614    188.37948    190.85537     182.7306    177.12743    177.97626    176.67467    174.63334     176.8079    172.45092    166.07724    163.33842    163.93953    163.09702    166.06337    166.52609    163.50707    168.96143    167.94946    169.32062    167.34319    156.47032          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -28.00    240.55876     238.2766    227.70015    213.91064    210.13285    201.00446    193.89907     203.0447    189.50515    189.65569    191.63545    178.56718    175.81022    174.95605    178.26305    180.95668    180.31611    168.94366    165.62491    169.19402    165.77308    166.47301    166.32912    168.56851    163.91663    167.76868    172.28801    172.43511    164.34707    160.83741    146.51571          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -27.50    239.14542    236.14966    228.64686    210.31745    213.19175    205.85643    194.98889    202.56488    187.93805    190.26616    189.70948    178.89528     178.2746    180.04565    179.69905    179.98276    180.84849     169.7647    166.36734    169.44164    169.15559    173.13758    163.29696    168.85748    167.71681    166.24062    179.88455    178.80007    163.79937    155.97536    150.44037          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -27.00    237.42545    235.33141    227.83366    210.94617    215.60227    206.23211    198.81398    200.23389    189.58593    190.19968    188.47391     181.3218    178.20597     186.5583     183.7623    177.73435    178.90508    171.17061    168.35334    169.35411    171.05191    169.69387    168.60411    169.97995    169.71268    170.33469    181.75937     179.8469    167.18982    154.98708    150.67356    145.28398          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -26.50    238.29063    236.53555    229.17287    214.24866    216.38038    202.81414    202.06307    200.08106    191.10626    192.11105    187.13958    183.96533    176.29785    178.49697    181.60186    174.06483    176.00979    173.80256    170.06554     170.8109    171.40764    166.86514    166.08902    165.25987    157.87111    171.04337    175.96146       173.58    166.78605     153.4961     146.2172    147.14621     143.9877          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -26.00    241.09994    238.52645    231.87796    216.67063    218.00073    198.82221    202.73525    200.56494    191.74237    193.96293    181.71209    184.27625    178.11992    176.42163    178.50138    176.68039    174.19962    172.21736     166.4239    170.78001    165.65661    162.02371     163.2763    164.34284    160.23959    171.09112    179.33474    178.20579    169.74315     158.8456    147.72089    148.99023    141.28904    144.39207          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -25.50    243.28537    239.54543    235.04611    215.15649    215.36022     199.8636    200.20066    196.66382    190.94621    191.14508    181.26731    183.74825    177.96643     179.1806    178.94443    182.39308    171.91825    171.73229    162.37285     168.4101    167.04856    163.70482    166.82388    164.28123    167.24951    170.74873    183.39447    179.87288    174.92971    161.64644    153.98109    150.48206    146.06195    146.91645     146.8049          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -25.00    244.45648     240.2089     231.6779    210.28076    213.02204    199.39722     195.4597    194.68805    194.17889    194.14437    184.66255    186.16326    177.46785    178.09855    179.42675    183.06559    172.98846     170.7834    165.19813    167.34855    164.36833    164.00462    166.96498    164.02608    168.77721    175.42438    187.60076    179.78627    176.28111    165.72005    156.24261    148.53649    145.96237    152.02563    143.02974    149.44679          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -24.50    245.09856    241.82592    229.47349    207.68444    210.79949    199.51575    198.52627    195.21863      198.465    200.23188    184.94139    184.50631    177.85879    174.85356    178.63605    179.99182    173.69377     171.2818    167.89978    164.56737    164.55985    164.68169    166.92463    162.01438    167.11419    174.94799    180.41724       174.21    179.32624    164.02836    152.37305    147.34518    146.56635    145.32084    146.24897     145.2324    142.47521          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -24.00    246.22719    244.52158    227.92065    212.89914    207.34315    202.44006    202.59098    195.23221    198.34121    197.25796    183.39654    174.59192    177.78908    171.44606    175.67475    180.19216     174.4314    175.97027    169.65191    165.43358    167.71823    165.76139    163.61964    156.64489    164.69799    170.17638    181.52506     181.8215    173.56466    159.62641    152.84079    150.82577    146.44857    149.86003    149.83545    144.40515    144.32176    141.20183          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -23.50    247.21572    245.45224    224.90352    218.40935    205.62628    206.98436    204.95617    196.99791    197.61105      192.993    184.06955    176.95748    181.01461    172.07015    173.70697    178.54648    173.47003    172.17368    168.53342      169.981    169.18379    168.36145    168.19305    159.39582    158.68357    171.19554    184.24962    186.02616    177.79695    165.46997    154.36605    150.62849    149.10139    148.99238    147.38643    146.12833    143.57532    140.49285    140.51188    140.84337          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -23.00     246.9553    245.16644    220.54544    221.89713    208.32164    206.32408    206.97174    199.04826    198.59764    189.00374    182.58307    182.58219    184.84019    177.18356    179.18094    176.82747    174.14081    165.41634    168.31186    168.91168    164.93739    167.60727    171.36813      159.692    162.50622    177.40571     185.9236    189.50653     177.2671    168.87516     153.7187    147.82051    149.74186    153.43254    146.51492    144.34145    144.27508    143.43036    140.11002     141.0127    139.97829          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -22.50    246.98158     244.6419    217.64779    217.66902    212.05827    204.64761    207.94102    201.69111    200.98989    186.90472     180.1237    181.08212    185.94833    182.85437    183.40828    176.97587     174.0611    167.07101    168.61835     169.5466    167.61796    166.12703    168.73835    162.33068    163.30098    176.73311    179.42906    188.59736    175.59917      171.998    155.58338    154.13275    151.87961    152.46525    145.64014    144.91892    143.15361     143.6245    141.11237    134.83114    136.45387    140.03631          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -22.00     247.6358    244.65102    218.68601    212.05316    215.08851    203.65808    205.03684    206.33317    201.27733     188.1274    186.63394    180.05736    188.54708     182.2061    176.52151     176.7246    173.86894    168.94291      166.227    171.38948    170.42286    167.07145     162.6415     160.4058     159.8983    176.00677    182.31952     190.2715     180.4047     167.7014    153.34692    151.50118    151.71238    152.68071    145.84488    146.46288    143.95581    141.19591    137.77812    139.34491    139.96301    139.82565    138.01098          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -21.50    250.67366    245.69983    220.54825    211.46406    217.16135    202.09265    204.13005    208.43678    200.40153    191.54118    191.59399    180.79252     188.0756    178.98329    172.95408    175.24958    173.95151    171.24056    163.91689    169.79744    168.81822    164.99209    161.98091    161.61346    163.38014     173.7086    184.33774    189.56683    183.34835    164.04646    153.44325    153.28023    147.87113    150.34031    149.67054    147.24402    144.65722    141.58024    143.22331    141.93089    141.20093    141.52318    143.58875    132.85003    133.99081          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -21.00    252.93444    247.81167    222.22697    217.12087    217.99692    200.57664     208.1701    204.21776    201.83448    192.74507    187.92832     182.3754     184.9497    178.26421    179.70719    175.03491    172.86382    170.52601    167.36097    166.20155    167.19203    164.77116    163.88977    164.50264    164.51649    173.29968    182.99441    190.97611    183.72941       163.42    154.54857    153.66936    149.36653    148.02774     149.8063    147.53291    142.15439    139.02095    145.75921    142.20642    140.96071    136.71977    138.52322    132.77027    135.63662    132.46807          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -20.50    254.37733    249.41148    222.96521    221.99981    216.80816    205.24874    210.04445    198.90344    202.22245    189.30299    186.90146    183.63686    181.22974     182.0816    184.53037    169.66225    172.98644    168.58817    169.64291    164.59806    163.83852    164.00267    164.33562    160.23892    161.71949    171.54485    184.00855    189.63744    181.10079    159.75468    156.20746    150.97906    149.04535    149.61208    144.90471    151.21755    143.65594     146.0335    144.43103    139.12111    141.38524    140.06075    139.02441    137.89195    135.79187    132.16351    132.62115    130.93661          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -20.00    255.13407    249.70911    222.56708    221.30542    215.03499    209.44109    210.59596    199.86238     201.0675    185.90512    186.72777    185.52297    181.27156    181.22195    180.28284    164.16211     171.8938    172.20837    168.84236    169.64624    165.75563    163.92367    165.67242    162.52853    162.67346    168.14064    183.60652    189.68666    180.31941    159.42336    151.95412    150.51742    149.13472    152.06316    152.69455    150.17419    144.19449    147.14198    143.51912    139.32319     142.6632    144.32249    141.93956    130.11191    136.62227     134.7141     136.4726    131.29419    134.93649    128.90006          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -19.50    256.22005    248.78507    219.88409     219.4063    213.52725    209.76298    211.70885    202.60786    202.07829    185.51186    186.80763    184.68741     185.5822    178.32089    177.19774    170.72226    168.08216    174.77654    170.30494    174.02148    167.72168    167.43153    158.09674    163.00281    170.67182    173.02203     185.4992    195.31433    178.63278    159.79266    156.11265     148.4587    149.78412    148.26713    151.01315    147.85499    146.93222    145.05991    142.87093    141.86271    145.74048    143.74586    140.64877    137.84799    137.32268    133.52216    133.94269     132.6978    132.93128     134.6761    131.74743          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -19.00    256.18668    247.85171    217.60265    217.78776     217.1701    207.52735    211.49701    201.71776    201.79318    188.26448    180.65303    179.36025    188.86134    174.89756    175.91606    175.64956    169.01606    173.77669    170.50764    170.05903    169.60432    169.04254    158.18767    162.37744    168.53492    173.56605    186.13688    193.96013    187.78033    163.43523     156.8582    154.80256    148.44942    150.21895    149.27641    146.70226    145.42522    145.02979    140.56706    143.00698    145.66337    142.37306    140.15588    141.67602    139.55334    133.27262    141.08174    133.07614    133.87046    137.52643     128.9787    125.14918    125.95431          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -18.50    255.21469    246.26375    217.45083    216.48003    221.87793    205.06792    209.33927    200.52324     197.9569    191.80188    177.74612    182.41884      188.272    172.17809     179.4538    174.05841    167.84674    168.57746    172.06685    169.64965    170.02267    168.95442    160.89385    166.54801    163.76621    175.46815    191.17502    189.99717    186.10976    161.47086    154.32337     152.3037    145.18143    152.30274    151.50399    145.56247    144.79109      143.517    145.43353    145.89774    142.93204    144.98177    138.88014    136.59475    140.74833    140.36646    135.62362    136.38778     139.7884    133.71881    134.67845    133.58696    134.96848    126.35525    131.55962          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -18.00    254.61358    245.89361    223.99321    214.53782    222.35945    205.65961    208.99786    202.82712    194.47807    194.26421    185.11086    188.70508     184.6712    177.25547    185.72734    174.24655     168.0371    164.65787    170.06978    169.09675    168.62498    166.93756    164.38774     164.6959    171.88815     177.1906    192.56037    195.05063    183.94478    163.39782    157.00492    152.98354    151.24767    154.91174    149.36454    144.96313    142.54128    146.60324    146.77736    140.80706    140.00829    144.00779    134.99045    142.28837    138.14908    139.48781    134.57333    138.05033    134.22345    133.13488    128.39726    131.35749    131.40463    134.63433    131.01329    129.32715    134.09011          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -17.50    254.35552     246.0896    229.08046    214.60154    217.86078      207.749    209.46255    206.16617    193.03619    193.21978    189.36043    189.06166    180.92117     180.0531    184.91615    175.07913    168.05231    170.70554    169.52646    168.88584    167.03698    162.13095      160.258    158.60658    168.40433    177.97228    188.45271    195.55035    182.11994    163.09789    156.51308    154.43823    150.93834    150.90541    149.43692    151.99813    144.45821    146.96285    146.73238    139.53837    145.04103    142.80416    139.92541    139.46408    138.52546    138.52565    136.58152    134.39198    135.17661    132.70438    131.82137    130.14908    135.36388    125.51329    131.35186    134.76321    135.87837    141.37661    141.08988          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -17.00    254.70439    245.67637    230.81146    215.50778     214.5434    210.33793    208.55817      207.577    192.93998    191.96403    185.94293    188.29801    180.88273    180.08296    181.61764    177.84135    169.55775    175.82772    172.71331    169.43707    165.42457    158.03123    161.90141    158.34613    166.46596    173.78143    184.43777    194.33523    180.70355    165.25991    151.37653     150.6962    155.56689    150.83184    150.21744     150.0615    149.65598    147.28047    145.31521    142.00649    146.24698    145.00138    138.46045     136.3334    133.27359     136.3634    139.24933    139.79321    134.74531    134.73293    132.77641    132.34073    133.66772    133.80127    131.09434    132.27532    142.89658    145.64542    149.42562    146.91403    138.95396    128.87436          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -16.50    256.18214    244.29113    231.40787    216.21948    216.03198    212.11739    208.40725    207.13881    192.30773    194.32578    182.98251    183.41802    181.84482    181.05689     178.5811    178.99596    171.42974    178.25165    170.33825    170.31951     163.7016    163.20069    164.45819     159.9867    167.82844    169.41425    186.95451    196.44544    182.34843    168.21565    152.20402    150.54595    155.46552    155.77015    150.87255    148.18822    147.39848    143.59063    142.66038    143.38212    145.28773    142.54337    142.48054    135.55371    136.02477    137.76869    139.22229    140.40798    135.73611    139.27487    130.46167    132.66861    132.66117    132.84264    135.29826    133.20419    139.13825    146.84559    153.38326    146.51666    135.46496    132.68197    130.20172    126.21957          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -16.00    258.73191    242.12193    233.40198    217.79659    218.17617    210.68392    209.67264    208.13934    190.98526    195.81904     182.9246    177.57036    187.01948     180.7647    177.21664    177.23022    172.15552    177.30625    166.82884    168.12544    168.24372    161.87034    165.25025    162.58975    165.57464    168.06651    191.90517    197.49656    181.67522    169.11265    157.20076    149.79749    151.62031    151.71095     147.7651    146.14707    148.64478    150.95227    141.99984    143.40517     142.4557    138.56797    141.95034     138.3109    140.52177    136.78436    140.64726    135.92756     132.8832    136.15226    133.17513    131.95297    133.01842    135.60556    135.94733    131.18282    142.15993    151.88002    153.42218    143.35582    145.13965    133.28326    127.71352    127.18033    123.70346    121.70656    125.59393          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -15.50    261.35592    240.06167    234.43086    218.00384    219.71747    207.93266    211.33288     211.0989    189.51775    192.59036    183.07333    184.81683    197.00342    183.07474    174.07273    176.26749    172.77799    175.95156    164.42304    168.07682    167.97548    159.11891    162.04489    161.85981    165.59936     171.5238    193.23496     199.5861     185.1081    166.66651    158.65625    155.42406    149.35876    151.16206    147.89529    140.94944    148.21115    149.97998    146.25825     142.8854    149.97533    139.44786    140.36084    139.46727    136.84624    136.71476    137.59208     139.0693    136.66581    134.17673    137.17955    133.89415    134.35211    126.92905    133.74797    134.20512    145.38325    151.60749    155.78138    154.18035    142.08772     132.8748    127.61317    126.11001    126.25704    125.10359    122.94964    124.43678    121.95657          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -15.00     263.4714    236.98499    234.21878     217.6313    219.96468    206.83866    213.21422    211.88449    191.56833    189.72425    184.55042    192.07543     193.6469    186.26383    178.70747    173.05205    173.84696    173.00586    168.55541    169.30852    168.37003    168.60438    160.82137     157.9033    165.32563    171.24029    192.63937    196.37886    186.17577    166.98995    160.59077    152.58798    152.75276    147.83366    147.87315    142.18484    146.13452    145.65612    145.16879    144.94801    145.18449    143.69564     134.9614    144.07698    142.39053    137.92604    139.51262    137.75976    137.04975    137.44615    136.24559    134.91976    136.77701    132.21672    129.91779    135.96054      147.734    153.97109    159.20441    153.48449    145.08486    133.70943     128.8237    129.38968    128.83336    125.52823    120.55162    125.31263    123.74162    124.01605    128.78947    131.96384          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -14.50    265.06106    232.30938    233.23389    218.79806    219.54558    210.19709    215.32926    206.88798    193.67547    186.33561    186.56653    190.69788    187.56155    186.49914    185.92287    174.08344    174.55707    173.04052    174.87204    169.15987    170.66832    164.70633    161.36842    160.24791     167.3509    169.71804    190.19035    196.89574    182.97794    171.07506     157.7184    153.85782    152.46391    150.35057    151.20507    145.25817    146.77179     145.7396    143.67802    144.11552     148.2726    140.69625    141.28985    143.42256     144.9411    139.11468    133.69908    135.09922     138.5499    135.47753    133.54377     140.3708    132.66797    133.46888     140.0157    139.85502    147.91408    157.30308    159.29641    157.23221    145.31238    133.39899    129.07245    128.11896    127.74097     126.0133    124.74422    125.50268    125.01558    131.88155    131.69224    127.40642    133.17776    133.07226    125.90792    126.47554          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -14.00    265.47044    227.62349    232.85535    220.98377    217.53004    214.02914    213.50059    203.36234    194.18999    183.22442    187.85716    190.60487    187.88585    186.92183    176.94647     175.1932     174.8449    172.99793    168.42184    170.12866    168.88226    158.46098     165.2613    165.47083    165.81328    170.54085     189.2654     198.5093    178.66422    173.08364    159.00595    158.41511     158.9442    154.75724    147.96454    144.71101    146.49451    146.69838    141.00774    145.09669    141.58168    138.59002      140.054    142.99071    139.23379    139.05505     142.1652    136.05769    137.99583     136.6586    133.71596    137.36993    138.19846    136.40343    137.06373    137.39026    146.70286     155.6832    159.44182    154.73446    147.30463     136.8753    130.32952     129.5625    124.96721    126.70462    125.68427    125.64437    123.32344    124.85724     135.2769    139.38837    140.17248    134.38112    125.20035     123.3854     128.2708    124.20695    123.63047          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -13.50    265.94105     228.9349    231.96856    220.01063    214.59517     217.3572    211.67686    205.31547    193.61081    186.96929     188.0084    189.80611    188.45368    186.06472    173.40439    174.33846    174.76075    176.60412    168.02421    170.44634    167.93984    161.43514    164.46501    165.12755    164.55876    178.30845    192.57302    197.73679    186.56442    168.95375    153.65277    156.85773    155.95103    155.60448    148.48098     143.1479    147.37473    147.30028    140.86424    143.40842    144.78157    140.57694    139.84341     145.7829    138.91952    141.09126    139.23454    137.74264    135.24751    133.66052    135.56824    137.58392    135.99541    137.55712    135.26731    139.30246    148.35753    156.93367    163.36858    156.39577    148.22372    138.65324    135.38577     128.9145    129.49375    130.25759    123.19069    125.97338    120.75581    126.84452    133.03638    141.58081    141.39015    137.88505    129.23261    129.82944    128.09252    127.93117     128.8617    126.28062    123.27828    118.32373    115.25928          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -13.00    265.58563    231.72097     230.6614    218.45853    214.10742    220.71203    213.48555    206.48993     194.2497    190.49308    185.74831    189.08638    189.06862    185.08982    178.17467    173.90189    177.25457    173.74085    170.47451    170.00277    168.58073    160.25266    158.66968    168.25816    163.16771    176.64166     194.6808    195.99169    187.87765    165.15085    153.23387    153.23016     155.6402    152.11466     148.4191    145.25266    146.90661    147.93503    147.01159    143.59339    144.26028    146.16586    135.08604    140.84376    139.31839    138.27637    137.41176    137.17066    134.68566    135.35932    136.32801    134.05617    135.99349    137.56029    134.26792    141.61262    147.43829    162.14704    166.27471    161.58994    147.53219    133.32921    130.35595    127.78576    125.32027    128.33223    129.35072    126.79222    133.48792    129.27207    134.29189    142.59818    141.59998    140.27464    131.65055    130.87965    130.85209    134.38184     131.8496    124.04494    123.40895     120.4996     120.3142    114.81404     116.4111     114.7018    111.72633          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -12.50    262.01085    234.36975    229.74567    218.15514    215.61204    220.78748    215.51054    207.49612    197.22532    189.03535      183.024    188.75231    190.34402    185.66794    181.05129    174.00641    177.93167    169.94672    170.93301    170.12396    163.96517    160.36951    164.68764    167.18664    167.24187    173.59755    193.09857    195.41426     183.3116    166.07041    158.53779    156.56252    153.35834     147.1997    151.51313    146.88291    150.38698    147.06083    145.63294    145.94709    142.51861    141.56858    140.21206    140.25604    142.24062    142.96811    136.07217    132.41287     137.4323    139.10655    138.77976    134.72601    136.99359    138.78647    132.73084    138.51159    144.77124    158.25348     163.4186    158.72058    149.86326    137.61025    131.50168    128.93159    129.10574    127.69063    129.93483    125.92517    124.54772    131.81101    138.55672    142.30118    141.77967    139.72747    134.24104    133.10836    135.35446    135.74019    130.79712    130.16346    126.67669    124.71229    119.96302    120.53458    117.83974    114.77554    113.27624     110.7697    112.47226    114.15614    116.33441          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -12.00    258.14292    234.23957    229.09402    220.85184    217.03197    220.12634    215.89228    207.44129    192.91825    188.24061    181.28686    188.77639    189.44557    185.57274    182.88305    175.37639    178.16947    173.05052    172.88151    172.02412    163.77611    161.09641    164.06163    163.32069    163.34485    173.73149    192.06353    199.72458    183.38046    171.18791     162.5679    155.84486    149.85128    150.01996    150.82543    146.99711    149.91656    147.67956    144.31213    146.49713     141.0125    142.02041    140.52836    139.04204    142.83683    139.10758    140.97457    137.97131    136.73512    137.32194    137.46255    133.89332    137.10851    138.23763     129.7025     134.0843    147.06685    162.53431    162.49764    162.38988    150.13468     136.7886    137.10734    133.68981    130.13496    133.38396    128.33551     127.0519    126.31291    129.98442    137.05728    143.51501    146.30614    143.49395    135.98777    135.43856    135.13068    135.40544     131.2381    130.82495    124.37232    123.56047     121.1424    119.77818    117.54761    117.35142    112.42341    114.69567    113.05188    113.37619    116.01702    117.78403     115.7832    113.32313    111.36715    109.66285          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -11.50    255.87167     233.8663    230.33901    223.51751    217.25663    220.37744    215.05625    207.84121     188.7736    192.15447    181.18214    193.02602     188.3243    184.71787    178.58422    177.76363    176.62588     175.1369    174.12724    171.50037    165.04628    167.62546    163.35141    160.88036    167.12737    175.36338    193.87519    201.01352    189.53958    174.00176     159.0799    155.60319    150.10123    151.64248    149.57873     146.4939    146.11456    144.90436    145.10648    144.82637    139.98705    140.96004     138.4773    133.62064    140.65892    138.37121    139.54118    135.54822    135.97812    138.17543    134.80225    134.86237    135.10503    139.13213    129.93592     140.7867    148.79136    162.92404    161.98087    163.94448    150.88915    135.31353      137.487    131.35649    124.83791    130.67774    128.70398    125.16218    125.82544     134.6209    139.32057    148.21513    149.83828    144.21924    141.03844    132.50184    136.42112    136.89357    134.81927    129.65639    127.13118    126.36913    122.09477    120.59571    115.68757    113.82725    114.59222    113.88871    115.73099    116.77175    118.55656    119.68494     119.2916    119.06217     114.2481    110.94098    112.23282    111.83665    107.83908    109.53604    113.91796          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -11.00    258.31939     234.0241    231.95415    225.10978    216.80247     220.1149    213.85227    208.67132    191.68042    195.68159    186.56748    193.15297    187.96764    179.34843    172.87775    179.00088    173.76778    176.65905    175.81542    170.82267    160.54558    166.60444    166.86182    160.34433    171.64409    178.52804     191.6512    197.52641    192.78409    175.00331    161.12851    154.88375    150.92575    154.29763    149.41735    147.17455    141.30154    144.57843    145.69697    143.75571    142.67729    140.92091    137.07816     137.7532    144.26571     143.7903    140.24801    136.39302    136.85308    133.34358    136.60261    137.15793    136.70158    134.56086    134.42679    137.22605    148.74767    160.60016    169.19113    165.72595    148.35377    141.82005    135.08695     131.3522    133.45233    128.99773    129.58088     127.6264    129.46594    129.95059    141.54336    149.00006     151.2033    141.06655    139.81347    134.60849    145.57012    137.85859    141.20288    132.15024    131.08096    128.51784    125.36311    123.38063    116.81322    119.54964    117.29001    113.99973    116.49139    117.03437     117.4475    120.14889    120.93014    118.59488    118.90111     114.6546    112.66973    112.00076    113.43718    112.82244     116.2289    115.29853    113.76427    112.41296    111.29106     109.8225    108.72256          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -10.50    261.71704    237.02858     232.8555    225.66601    217.51105    219.14178     212.2376    206.74311    195.00778    197.50105    190.75449    189.90901    186.58066     174.7609    173.26223    179.13832    170.81088    174.45134      176.226    172.00302    164.28815    162.35265    165.22684    161.07449    171.83944    177.34597    194.02372    196.98275    192.79401    176.64725    154.95742    153.02182    150.75566    150.02961    148.98716    150.30517     147.6367     148.3782    143.69738    142.21785    138.32895    144.05477    137.10107    138.40192    142.73004    142.48375    138.25492    136.54749    137.33197     135.3775    133.63328    139.52104    140.55718    133.42436    136.50179    136.63127    148.05072     161.4663    165.41917    168.06774    149.25802    143.45882    132.41101    129.02001    131.48292    126.97314     126.8252    130.98089    129.45757    131.69816     140.8325    145.06606    156.34679    145.46718    141.30301    137.63417    138.68158    138.94251    140.68277    136.02254    131.15254     131.6683     127.0024    125.04941    122.00901    118.86283    118.79676    117.29973    119.49327    115.74944    118.93647    124.26366    123.19316    117.19229    116.70128    116.29028    113.07719    113.95336    112.58638      112.157    112.40308    115.77832    116.56649    111.62347     111.4998    110.13742    111.39048    113.53453    111.76312    112.60959     114.3083    110.25147    107.03232          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -10.00    264.53754    240.16885    231.05399    225.38347    218.26171    218.79508    210.89281     201.8358    194.73757    198.86571     190.5094    188.71859    185.81119    178.66498    176.35107    178.22539    173.47115    172.28865    175.36835    172.05603    170.07579    163.32383    161.05988    161.63454    166.87015    179.44842    200.21164    198.15105    195.25295    172.83406    159.72817    154.09006    148.64983    149.88463    148.66316     149.7231    149.80484    151.22898    149.68575    142.86128    141.48711     145.4651     142.1884    141.77897     141.8023    142.10728    138.76153    138.72265    136.34527    139.08875    133.30747    134.24951    137.52076    135.92276    135.68018    137.73911    149.00023    166.37643    172.13236    168.73347    146.29159    138.66106    132.74179    131.11855    130.09917    134.27247     125.6828    126.43702    132.23039    128.91583    137.77505     149.5546    151.57858    147.57938    141.30084    138.67153    141.73734     143.6398    139.09664    135.45909    133.34754    131.08133    127.56827    126.08102    123.88364    117.25899    117.21731    117.50748    117.91667    120.10664    123.80725    123.50015    124.73449    119.85098    119.99996    117.90147    119.28428    116.02427    115.40391    112.51248    112.32821    116.25135    113.61774    116.24813    116.08662    112.06894    114.49082    115.19781    113.76528    113.13491    115.27143    111.48089    109.77378    114.06665    111.34178    115.14713     115.2296    113.02612    113.22445    115.07964    115.12728          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -9.50    265.59821    241.78324    230.23895    224.75152     218.3864     218.7409    213.25722    198.96898    193.13234    194.80144    190.07809    188.19162    186.94615     181.9848    181.16214    173.62592    175.68459    169.78318    173.31284    170.24486    172.30921    163.58063    161.43579    159.22074     166.6015    183.28759     194.2142    200.37936    201.01785     178.8492    167.24245    156.01998     153.0623    149.91431       149.48    151.21219    150.52568    149.30232    145.17575    142.81001    142.88924    136.95016    142.53865    141.57921    142.48043    136.91998     137.9332    136.94422    139.11034    137.89159    134.51403    135.83911    133.92421    133.13904    135.51362    139.40164    147.96115    163.45481    172.04972     165.6965    152.89382    140.16614    136.58752    129.86818     129.0326    129.24915    126.95976    125.18732    125.84795    129.30949    142.64881    150.20485    150.95826    150.70049    140.38492    140.71064    141.97255    143.13657    139.56956    131.44781    131.04687    131.10857      127.263    126.59576    123.59691    118.73478     116.4939    119.62293    116.01625     118.0924    123.89891    127.16968    124.49413    124.06983    120.53349    119.79329      117.609    120.42409    115.28559    114.31089      118.131    117.68277    119.20386    116.82635    118.00413    112.09254    109.95031    114.07073    118.12312    113.67438    114.08002     111.6639    113.11323    113.12248    114.38819    115.97347     114.7551    118.46241    110.87246    115.28726    111.94425    112.90295    111.19073    112.29413     113.9425      113.245    119.00083    118.35571    117.08713          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -9.00    266.77831    241.07356    230.30635    224.42786    217.67214    218.82474    216.03042    200.52182    194.22524    190.74148    189.15246    187.59485    189.42372    184.50578    179.80532     169.8935     175.8733     172.6346    172.80397    168.21379    168.65496    164.02667    167.57346    161.87176     163.5387    182.39136    192.29654    201.93136    200.95991    175.39665    165.28275    152.40857    151.98073    150.63026    146.64386    149.26165    149.05868    148.59368    151.87253    145.08847    141.92968    144.08404    142.36566    140.34211    142.59616    143.78475    135.01447    134.83277    137.82697    140.38968    138.60357    137.87896    136.90041    133.22895     130.1625     139.9003    148.04806    160.05007    172.56308    166.22916    151.74309    139.83872    137.46224    129.50101    136.12338    128.48465    125.93025    127.88243    125.24297    133.41885    141.52309    157.50538    153.11391    149.22096    142.41386    140.62633    142.18044    138.82112     135.2111     136.5231    128.86075    129.59417    128.34743    125.08247    121.17344    122.12382    120.37328    116.28869    121.23831    121.73348     125.6345     123.4573    124.86771    124.65465    123.00887     123.4085    117.16984    118.87146     116.8335    118.38501     118.5494    117.62525    118.72741    117.01113    115.77651    111.85641    114.98221     112.5995    114.82291    114.19104    116.09323    113.02439    109.26985     113.1057    114.44616    114.15635    117.10157    120.20999     116.1116    115.28387    112.60321    114.84957    112.90619    117.44906    113.69079    119.53715    117.86303    118.26913    118.93564    117.30191    114.14797    108.37581    112.31681    110.54889    112.78443    113.85582    116.46225    118.19598          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -8.50    265.73824    240.13208    229.93861    226.09997    216.54509     219.1724       216.09    203.88539    196.61003    190.37699    188.55626    188.33654    191.39028    185.10169    177.38714    171.42065    175.31488    174.85736    173.19236    168.53445    167.09103    167.73236    169.06101    162.17116    161.18906    181.33202    195.51466     203.3378    193.56392    178.92309    161.19038    153.93126    151.09328    150.14408    148.99795    148.45583    149.75441    149.41731    140.04762    145.20719    141.75623    140.58652    143.98845    144.00656    141.83593    138.62381    137.58978    137.38346    134.64333      138.675    138.19907    138.63966     137.1856    130.80522    135.39764    134.92639    149.64608    160.44486     172.7937    165.55812    151.38135    140.89223    132.71594     133.0784    129.98899    125.42957    123.96373    126.47666    131.72536    131.64959    142.99593    147.81512    157.87845     150.0564    142.81651    142.07804    143.80164    145.52945    137.61394    129.97401    134.24063    132.38737    130.38402    127.78793    126.04112    118.46017    119.74101    118.45208    120.45415    118.67639    121.48066    127.63459    125.56481    130.28189    121.59949    121.17477    122.48319    118.04592    121.46195    118.90434    116.75037    118.65196    122.09121    119.91252    114.88833    110.12379    115.07677    115.61158    116.00112    112.24377    114.12197    112.50734    113.56309    115.54703    114.86263    116.65832    117.95217    117.78088    116.11299    116.94976    115.18592    111.62655    115.32808    113.92244    117.44172    121.92287    122.97633    124.91428    120.59078    115.89225    114.05878    110.95086     116.4664    115.39057    112.89702    118.44142     118.6338    122.84409    127.61503    135.31883     133.0984     130.7999    125.62553    119.83718    112.62381    115.83486    115.98384    113.49183          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -8.00    262.17411    240.40204    228.83536    229.24807    218.11095     220.5412    215.95123    210.48224    197.67468    190.64942    190.95811    186.59749    191.50249    185.20395    178.59166    173.26564    174.78268     172.0215    168.98232     170.5544    171.64652     164.7754     169.6561     162.7912    163.45053    177.71513    197.11299    205.31886    193.56784    180.12204    162.06188    153.59561    150.49127    146.63939    148.12262    150.41782     150.4224    147.05185     150.1419    149.84942    143.11152    141.26535    141.89273    139.36168    142.35925    144.20733    138.83753    138.54088    142.92206    134.94075    140.11022    139.28477    135.00976    137.58247    135.41135    141.96087    146.10201    161.17042    172.31699    168.39826    153.97908    140.15134    135.68989    131.72493    133.45235    130.49437    128.73098    129.96534    129.63498    133.38243    142.33256     148.8618    154.16657    150.64562    144.21304    143.35548    142.93174    145.49776    142.24818    135.90969    134.02559    132.02955    131.87636    128.88783    126.91094    122.25582     117.7464    118.40157     117.7958    122.11407    124.48195    125.99764    128.26888    126.44041    121.40208    121.01568    118.78938    118.99866    118.07142    117.71833    120.28146    121.85927     121.9822    120.38265    116.93782    115.03723    109.59553    116.42499    116.91767    111.51175    116.68931     111.8005    112.05898    116.10011    115.46573    113.95401    114.29847    120.38707    120.09417    117.27243    117.36256    114.99208    115.38242    115.48356    117.32922    121.29019    124.29537    122.58771    119.75001    117.18233    114.96733    112.54576    110.25987    113.45614    116.93512    118.55684    118.28437    123.54242    130.46909     137.0314    135.77798    133.69311    126.71417    122.66628    113.90134     114.2443    116.24922    116.70763    114.52655    113.23355    111.55614    112.80718    110.59082    110.54621    107.96947    107.96154    109.71516    110.88109    112.39547    113.31517          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -7.50    258.07578    241.07852    227.84533    231.95166    220.72057    223.90158    216.45603    216.02008    198.36786    192.77381    193.79324    183.42301    188.50129    180.26686     180.5334    173.93133    171.86383    173.98437    166.10216    172.86121     169.9315    165.04879    167.28286    166.83736    164.15423    177.13562     194.5573    206.11179    196.13447    176.20433    160.75757    155.44399     151.5803    147.13666    150.07683    143.34778    148.69001    149.02457    150.85628    148.04331    144.08568    144.22095    143.14511     141.3113    139.87194    143.37685    139.07138    136.24233    139.83089    139.31486    136.33452    139.31203    139.40479    132.64142    134.10265    140.38015    148.65216    161.75787    170.79371    170.21104     153.3604    138.83207    132.20453    133.39321    132.83235    129.57647    127.16198    129.80495    131.45189    129.53075    138.41233    151.94569    156.19836    156.81802    145.72622    142.36607    144.56321    146.42653    140.48437    138.83708    136.12463    136.77852    131.08044    129.63562    127.45266    122.01496     117.3616    119.67448    120.34347    120.79284    126.37781    127.97486     127.8112    126.10408     125.7017     122.0754    120.61233    117.56629    118.20814    120.23821    120.98022    123.30366    118.47192    119.22726     120.3831    116.33542    114.82263    112.95092    115.53087    115.35755    119.38505    111.33233    117.71691    113.40904    115.66581    119.19406    120.80838    120.45224    120.14166    119.73891    119.54978    113.76888    116.43525    113.99566    116.89681    121.31271    124.45472    122.12131    123.11021    121.27581    118.18464    111.10476    114.88381    113.88253     116.2728    116.66292    121.03742    124.81559    134.11153    138.22943    139.30642     135.4935    128.40439      124.173    118.45732    115.79294    117.18362    119.52463    116.14817    116.23043    110.61333    114.28093    113.37227    112.41803    111.27652    111.53141    112.68184    111.13878     110.4228    116.05866    113.45364    115.03082    111.46389    112.44103    111.76304    115.62466    118.06547    117.31524    112.16373    113.46141    116.14883    117.59896    117.57313    115.22597          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -7.00    258.59669    241.52336    227.08461    230.30393    221.46932    227.60497    217.31763    209.98919    202.20702    194.84384    196.30179    185.37683    186.71856     175.1019    183.62886    169.82965    170.90461    174.95253    169.97312    171.99243    167.80972    167.80199     162.3611    165.04881    165.67838    180.76764    196.73399    203.43177    197.85005    175.73464    163.52681    160.26891    153.80765    148.80574    150.58055    147.63022    148.12438    149.93815    152.35955    145.37541    147.66505    142.66075    142.59572    141.28638     138.5714    142.45414    136.18912    136.24746    139.31476     140.5632    137.96692    135.56762    136.91508    132.93634    130.28634    137.19164    150.64061    162.83594    174.75799    166.14525     156.3019     139.8072    139.69996    132.78457    131.30099    129.56379     125.0471    134.71905    129.24739     135.7003    139.73691    155.57555    155.63284    152.97593    147.54256    145.21783    146.40835    146.46886    141.36539    141.98681    138.57172    138.12876     133.7578    130.85256    128.46525     123.0975    119.82221    117.96398    120.47667    120.99391    124.71588     127.2122    128.97914    129.77027    126.25388    123.82765    122.06899    122.63219    121.56567    123.92503    121.35706    120.99197    122.54939    121.50543    120.03783    116.68442    113.54194    114.52415    118.25674    116.14133    116.31781    114.74534    114.25453    115.01943    115.13516    116.84487    116.98946    122.03764    119.53594     117.6122    116.56378    118.22212    114.60894    118.80889    119.75267    126.31338    123.66869    125.41455    119.11438    119.36806    113.91733    114.86693    114.67605    111.70315    114.39085    118.49035    120.19273    126.56669    138.99308     144.4328    140.45669    137.08381    128.63175    125.24209    121.75181    120.15564     117.5148    115.70666    118.94361    113.90687     112.7359    114.18993    112.51405    112.42053    111.09998    111.67712    112.80626     114.8565     116.6459    116.44743    113.89382    115.71351    113.42788    112.06746    116.57004    116.95401    117.94527    119.97496    116.06139    114.51514    116.35894    119.33678    116.58913    119.24541    119.69432    117.80252    117.76949    120.13379    120.92026    118.05384    117.12699    116.14484    113.35946    113.85391     114.3581     113.1729    116.03577    113.15084    112.15329    111.18129    112.89197          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -6.50     261.9764    242.11431    227.09066     228.2722    220.53353    226.05546     215.7866    203.29806     208.6172    195.08445    198.32803    186.90293    187.61035    183.22494    182.61234     167.7842    172.52533    173.78344    170.14943    171.35715    165.55924    166.83695    163.81495     163.5137    167.00688    181.28149    196.07843    205.81739    196.88243     178.7702    160.86205    157.77477    155.44397    147.42866     147.4841    144.58145    148.83209    150.59294    152.69912    143.06072    146.00051    143.21516    142.07613    141.84751    143.75472    138.66459     136.2835    141.54303    137.33806    138.81106    141.26474    136.90401    132.65026    134.94386    136.14196    140.47058    150.45573     165.3154    173.03688    171.09259    153.12699    142.35948    135.12738    131.07019    131.67801    129.08143    127.50347    129.71076    131.60685    132.30133    144.71616    154.18766    160.30571     150.0675    147.59214    146.74297    147.21798    146.66286    140.60633    143.15745    139.49751    139.77557    136.50159    129.90861    120.09223    120.33452    117.23104    118.54922    118.48802    120.96267    125.96376     129.1599    130.49935    130.97711    129.05386    125.28456    125.25656    123.56162     121.6909    122.58577    125.24327      124.461    120.77965    119.96858    117.87045    116.90045    115.74158    117.10694    118.05497    114.99219    118.13097    115.60202    116.00378    112.85271    118.40936     115.1952    119.43674    118.47884    119.78033    120.31566    115.51483    119.03754    115.56374    116.75293    118.39481      126.371    126.19113    127.97169    121.35135    119.50984    113.40831    113.80331    113.71064    112.04917    112.09718    116.84708    120.93378    129.73703    135.70525    140.25988    144.84479    137.77052    130.68702    125.46722    120.43267     120.5745    115.38861    116.05642    116.66284    115.34244     112.9816    113.88842    115.76786    112.35843    112.92492    114.13416    112.70717    113.26091    115.68013    116.33394    115.33324    115.17228    114.79313    114.31843    116.87169    116.92219    118.25948    118.70563    116.60729    115.53925    116.61404    119.95607    117.73966    118.00294    116.83528    114.93273    121.55652    122.61211    123.97481     122.9249    117.27267     115.6615    112.90643    117.65371    114.30797     112.2012    113.21113    114.33858    110.36019    114.72407     118.5112    117.75819    119.18981    117.73746     117.3349    117.02012    115.57685    119.07152    119.35944    119.07876    116.63978    116.31529    115.96507    116.28748     113.4846    112.49407    118.94437    117.10052    115.23815    114.08077    114.27352          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -6.00    265.16566    242.91389    228.77594    225.21216    220.31998    224.37645    210.74399     202.7545    207.54928    194.78683    199.92288    185.12786    182.23226    188.00719    180.46375    172.62299    176.44651    175.80939    170.66661     172.0135    162.35639    162.97163    167.08395    163.37326    163.95715    175.66376    195.53769    209.49663     194.5389    181.80755     159.3548    153.57587    156.71369    152.39935    148.21608    146.49236     146.7306    148.12522    149.85375    145.49322    145.28852    142.47515    142.04769    144.76574    146.26381    136.41001    135.16727    140.95054     142.3458    140.29072    139.76174    135.53747    135.32658    135.68177    136.06513    139.78088    149.19414    165.66218    173.54548    172.47743    156.68467    144.93819    132.46528    132.31624    130.43793    129.67686    129.08447    129.22981    130.05284      132.896    141.02119    155.05576    154.58871    152.68823    147.67302    148.06196    147.16718    149.47155    141.30833    139.49802    139.95405    136.86328    134.42261    132.46778    124.98012    124.91332    118.36875    119.07134    121.92805    120.61762    126.99579    129.82581    129.93712    131.71735    133.14548    128.77805    127.11746    128.48565    127.33622     125.1261    125.99912    123.27767    125.98545    121.95181    117.77445    114.30157    115.19011    117.67606    117.28972    121.53678    117.26439    115.27721    115.58685    114.36546    118.25477    118.46269    119.24309    117.71706    124.79346    118.88762    114.77597     117.3449    114.94417    116.98169    117.78235    122.47929    125.91918      127.226     126.9381    118.39945    116.24214    116.20502    115.13034    115.67492    115.33782    118.47876    122.83998    127.37969    136.56784    143.88564    142.77878    137.06016    132.48789    122.46006    120.10421     124.7765    118.94899    116.66661     117.8562    114.82694     114.0488    113.65142    111.94703    113.04493    111.56276    112.16678    112.12726    115.95367    116.08327    118.18707    116.37527    115.22866    113.43327    113.54356    116.42482    118.23012    120.05636    116.10667    121.68929    120.96832    120.30461    117.10529    117.28064    117.25308    115.52226    117.68031    118.85782    121.29682    122.73934    121.62341     120.0616    115.37894    110.62316    117.74009    114.78188    117.03618     115.0052    118.17452    114.71514    115.55424    115.32334    115.23091    115.50993    117.67508     118.3947    115.30467    119.09839    117.67339      121.187    122.84701    120.31328    119.58584    117.74839    119.64075     118.2185    119.00304    117.95082    117.72124    117.52237    121.84604    118.87209     120.9164    116.53594    115.96683    114.29269    120.42455    116.79613    118.02736    116.23386    116.96271    118.47142    117.58452    119.83007    115.87962    117.44343    120.67187    122.70278    117.44571    121.12401    128.50287    125.42713    122.27569    121.77308    124.33216     124.4084    123.05737    122.11936          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -5.50    265.44236     243.3203    230.64924    222.08688    223.01821    221.80945    208.18309    202.66292    198.31622    190.46444    199.07999    184.85583    173.11858    184.53896    176.66876    176.80745    178.95985    179.03156    177.23915    168.88255    163.50749    160.67476    169.77656    170.20685    165.03567    175.40163    194.93059     206.3722    195.29463    178.16363    159.92644    152.73576    158.81733    148.95049     150.9179    148.44524    147.98131    151.26526     152.6152    146.53984    144.15047     139.7362     143.0812    140.57589    145.62523    139.01692    139.79384    138.18987    133.85571    140.63473    134.76256    138.87201    133.45563    136.11034    134.35947    138.86752    152.84617    165.08142     177.1351    173.15241    156.74415    143.53317    137.35991    133.88984     131.7367    129.26685    129.60974    131.55628     130.8605    132.36782    137.93982    152.16864    158.56847    152.93305    147.02642    144.30434    146.87129    145.93627    139.51552    137.09214    137.30419    136.65094    136.84862    131.37203    128.08216    122.19528     118.5514    116.53001    120.91721    122.80654    127.03112    131.29683     138.3163    143.06344    140.77032    137.24834    131.05485    133.42763    130.71967    131.56058    125.60562    124.84908    123.96246    122.63599    113.76608    115.69955    117.43698    113.93721    115.39602    118.21251    117.78166    115.38158    115.95083    113.57502    118.43775    118.67684    125.47444    121.67416    122.43225    122.60891    120.77673    116.36393    116.19183    117.87238    119.71506    125.48228    126.98912    123.74442    126.11743    121.12276    117.22656    114.71156    113.82365    113.58904    116.60785    118.83886    123.16589    130.34847    137.07881    143.23377    146.30497    140.97577    137.90044    126.32907    123.97832    124.73613    121.10567    119.04977    118.13661    115.35863    118.75183    114.88397    115.60482    113.99135    114.01428    114.15982    112.75722    113.13292    117.14668    116.09565    118.23325    115.95868    116.22398    120.64463    123.94328    121.45756    119.80574    121.04183    119.22539     119.8408    118.19413     116.6357    119.57113    118.91632    118.93588    120.88415     120.4303    121.62114    123.97349     119.3305    118.76252     118.4883    107.29505    115.89998    116.27959    114.49224    113.51901    116.18168    113.54105    112.17433    115.52215    117.21386    117.77887     118.9962    118.02792    116.90975    118.46702    126.28009    123.52376    120.72725     125.1092    119.26226    120.23086    123.15415    119.27783    121.82249    120.29878    117.80538    119.97804    121.98545    120.43252     121.7908    114.56068    122.73846    117.96617    118.89804     121.3006    116.72063    116.82511    118.84438    119.17404     122.6254    121.31464     118.4844    121.86687     123.2437    123.18717    124.05002    123.24971    127.57268    125.60501    126.12625    127.87566    124.22319    126.52283    127.35003    128.72596    128.74803    127.43605    130.00091    125.48749     128.3739    129.57461      125.066    123.58131     123.9773    122.68431    125.59009    126.93374    124.96459    124.00499    125.48916    123.51259    129.69923    128.44738     131.7176    129.37112    129.02238    128.86448    128.94413    135.14573    133.05028    135.30845    136.13854    128.86308    131.77587     134.5279    133.18182    134.66787    130.63993    133.15464    124.84002          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -5.00    264.92136    242.43435    230.64235    220.95927    225.91285    219.15406    212.39554    203.99483    194.90225    187.30218    196.72777    188.62068    182.36701     183.0127     174.7044    180.25412    178.75895    175.48639    177.03223    166.08549    167.29294    160.58207    166.31534    172.08527    171.16879    177.60961    194.37604    208.14788    194.10294    174.92277    161.18614    151.38304    150.54896    151.15737    151.26192    151.47926    149.28927    147.86944    147.33617    140.87778    142.04155    139.32205     141.7624    141.18319     147.7485    142.58708    140.79691    136.03624    138.85021    140.75247    143.44422    140.24974    134.05227    137.89037    136.86848    140.02403    151.66029    163.95805    178.38911    168.60399     156.8984    144.70601    138.48701    134.90351    132.90683    129.22012    129.71026    131.23873    131.18198    131.77516    142.95938    151.10706    157.08152    155.13628    149.59284    144.27165    147.51922    147.57447    138.10396    136.68238    134.92309     139.6307    136.13256    133.11509    125.48507    120.32928    122.09867    118.19115    119.58114    125.42026    127.31914    135.10834    147.54834    153.60744    154.66345    150.65228    143.81662    139.21761    136.05031    133.58824    129.23095    129.60821    123.37726    124.38533    118.68004    120.97759    119.06998     114.9787    115.12293    118.49092    119.41834    114.91422    116.46636    114.67252    115.40316    117.63496    118.92549    122.17354    123.88717    120.98502     117.2515    115.48474    116.22936    116.70893    120.17467    125.85172    128.75496    126.94829    125.56613    121.44693    116.64684    113.48495    114.44993    115.38508    114.62875    119.75129      121.609    130.86989    140.84851    144.99868    149.53471     143.6997    136.93317    128.08947     128.8326    128.56886    125.83216     122.8424    119.05668      114.246    115.38928    115.12426    117.56084    113.53867    114.54129    116.42918    113.80122    118.13521    115.52477    116.76098    115.43167    116.61067    112.99441    118.28251    117.00411    122.43635     123.0605    123.03099    119.03299    118.00077    119.76366    120.59968    122.00869    121.30676    121.67828    118.68488     121.8677    121.17997    124.94901    120.01812    119.12623    115.52824    115.99009    117.51942    116.62249     116.1571     113.3817     116.0815    115.85324    115.40451    116.02231    118.70835    119.08607    122.90165    120.12583    120.47035    126.26381    123.03478    123.81088    124.87789    125.31334     121.6231    123.01306    122.86471    120.81574    118.38552    122.65868    120.42223    123.69489    122.68806    121.85774     121.5317    122.14413    119.10106    117.18426    119.81204    117.44959    119.81975    122.10297    120.26115    117.51762    120.26462     124.5869    124.07608    125.13884    122.78428    126.76022    125.04214    128.29695    128.16365    124.73536    126.79575    126.33988    124.74416    131.01518    127.40552    132.92008     129.8613    128.77932    129.84741     125.1367    127.06922    130.15542    128.52075    128.92931    125.66912    121.86946    126.09575    124.16786    126.78249    124.29269    126.50807    124.46432    128.24387    127.15033    129.25094    129.73903    129.92804    130.42511    128.53489    130.17074    136.08672     132.8981    134.51705     135.3479    134.36026    138.10932     135.9074    133.39599    134.60965    132.40671    132.35519    131.31941    129.06845    133.82575    126.31983    127.96267    126.19009    126.50338    131.13379    131.83672    126.27666    130.54701    130.60681    129.65353    128.92191    132.57972    129.97209    126.84379    128.73481     130.5794    126.66097    125.58388    128.47501     125.0902    128.11449    125.42633    125.22973    125.10593     122.0995    128.65901    129.54272    125.10537     126.8603    124.95494    129.32086    130.16468    127.05318    125.41805    129.26595    125.39931    130.09134    129.20104    133.92221    128.09689     130.6409    128.90796    129.85816    130.48361    129.25757    131.54267    132.19387    130.87036    131.36486    132.79727    131.66203    138.68316    130.39302          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -4.50    264.37124    241.65931    229.70542    224.73132     223.5909     219.9044    216.00642    205.32589    198.00223    192.02787    194.47471    189.50573    190.92171    182.98787    177.70864    178.61354    175.44272    174.19649    174.07748    164.91566    167.89352    161.32449    169.30852    168.23156    177.14839    176.15796     198.0108    204.68087    202.37469    175.38328    162.38197    150.91044    157.40617    151.44408    145.20924    150.85643    144.98623      142.082    153.21919    145.90504    138.35356    143.72702    145.89833    145.16281    147.44609    139.02421     140.1334    138.92682    139.52377    139.80797    139.81075     139.1302    134.27287    137.55061    137.87285    139.60802    152.18236    169.00583    179.58299    172.83936    155.29268    144.55172    134.86079    135.86539    129.52822    127.76969    127.91039    126.78885    128.93766    134.54196    137.59391    153.17618    156.93259    157.14161    149.74033    144.38516    143.83116    143.34582    142.96442    134.62435    135.62383    139.00395    137.98224    134.12976    126.54661    122.66006    120.06472    119.59336    122.59607    121.89435    129.24453     137.7175    154.71586    162.37322    162.90884    156.33219    149.72948    141.38351    144.78784    142.47211    136.99283    131.00875    128.61575    122.21934    120.20413    118.63341    116.18831    116.71966    115.87612    115.59355    119.06084    116.19624    120.85842    116.02895    114.21792    117.30835    120.21744    120.76985    125.68129    121.96394    117.99105    116.43694    115.26916    117.93595    122.89819    127.10771     131.1735    128.02951    127.06689    121.84624    115.76541    115.43702    117.72711    112.30226    117.85892    119.87737    122.09186    130.79593    142.94218    148.17316    148.52196    145.82305    138.16006    131.31007    129.44902    126.98741    123.58433    121.85772    121.38506    113.57629    113.98158    115.46646    114.38795    112.56195    111.82538    112.17309     116.5974    115.52684    115.70745    117.05937    118.94761    116.97032    116.73742    117.19219     119.7463    120.17452    122.25898    121.32245    120.70311    119.70353    120.50475    117.48211    117.21628    121.98627    120.27432    119.27269    125.73427    122.72145    123.54071    126.94229     119.2447     120.1961    117.41179    116.86597    117.59395    118.63986    117.35606    116.08059    114.41763    118.80607    116.91935    117.43797    119.90614    118.62119    117.63314    123.29318    123.18746    125.48429    126.11803     128.2455    127.76667    121.05708    125.10894    122.98431    120.22472    124.26623    122.14291    118.26238     122.6246    123.99723    124.09115    125.36679    124.27695    121.85267    120.25473     122.5512     119.5851    119.63848    118.64072    120.15004    117.72641    122.33202    122.59749    125.19103    122.36646    127.83546    121.96879    125.88506    126.11058    126.74612     131.1499    122.91579    128.00933    121.69075    128.49755    130.53514    127.64751    129.47614    130.40765    132.12205    131.52142    128.66131    127.47646    130.09706    132.03173    123.72406    124.24877    125.99942    126.18331    127.86117    127.25491    129.06813    123.25709    130.64543    130.72393     131.2354    131.68025    131.79115    131.05836    127.33109    133.52202    135.20216    137.12111    136.32647    137.02459    134.18634    142.39661    137.46819    136.96474    139.14702    141.96511    145.66689    144.71019    138.50253    131.36977    129.22792    129.52999    127.67664    125.81355    131.37559    131.26208    129.65265    130.46463    131.59159    129.19922    128.44467     131.6044    132.73729    128.54957    129.67239    123.70744    123.42241    129.77869    126.21979    127.39452    126.02173    127.08822     126.2463    126.46047    125.97233    126.04945    128.20861    128.09712    126.37512    125.93987    128.70519    125.34849    129.07118    127.45041    125.83113     127.6991    127.65552    128.29985    132.75979    133.57649     130.3589    128.46854    129.90206    136.91117    132.81693    135.44716    135.34772    131.90888    134.92947    136.74654    131.42985    134.26354    134.53256    134.66809    135.58124    137.88375    137.38178    139.59828    141.68425    138.75039    138.97145    137.17867    144.89748    139.85819    146.09875      144.153    141.66591    141.78087    150.98486    146.82258    152.45066    153.47617    146.81136    146.01128    149.60887    155.87512     152.8019    154.07279    157.14374    156.72591    159.90227    158.14984    159.41172     161.0527    163.93137    161.62036    163.81138    163.48092    167.38809    169.16225    161.93105    164.06476      159.156     169.8754    170.00373    162.26926    170.41628    166.28845    173.03497    172.20816    165.68525     174.1615    172.90023    176.16214    169.42907    170.64406    172.77123    173.57512     170.0245    178.04172    177.31761    174.58437    178.38374    188.21957     187.0842    172.74998     182.3298    175.29737    174.99459    190.50979    193.85614    195.85252    183.14389    187.70105    190.92504     195.5611    207.60611    202.33588    200.48201    202.76068    202.57138    198.50978    192.30868    206.26083    204.38774    203.66189    209.92976    216.63166    201.83178     226.4957      214.948    225.50665    220.42805    222.93767    219.00471    213.90761    239.75286    221.11228    230.19568    225.29546    227.53145    219.61073    242.15454    231.34172    239.13043     247.5285    224.96144    239.90144     249.7061    244.00924     248.9176    243.17563    252.83689    237.01359    247.65926    258.83532    260.86468    259.81237    280.41322    258.18765    260.99719    271.15179    269.26812    275.25271    276.40512    280.36513    280.42411    276.25971    297.38707    286.68855    298.78977    297.23205    297.82832    292.34657    349.85036    338.35448    328.34866    319.43313    325.93801    312.89165    347.38129    375.13863    380.55124    385.31738    356.05624    386.31641    386.98395    447.14241     417.7648    459.03497    449.23292    422.69379    458.04749    508.21582    576.02088      564.479    554.71664    568.10817    551.32357    602.70091    827.52948    760.10338    806.66922    785.84866    854.52907     838.9709          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -4.00    262.29605    240.54358    228.77435    228.33523    220.04956     224.5156    214.00113    204.66759    199.76607    196.14246    191.97416    187.30944    188.69914       182.72    180.40697    176.97035    172.93021    176.59897    172.81012    161.23081    171.51678    163.15921    171.37695    161.46699    172.04424    181.39349    200.78544    204.19009    207.82977    176.71537    161.51853    148.43486    155.88564    150.24248     150.1394    147.14075    150.23731    149.89875    144.79234    140.91557    144.35909      142.195    145.80724    145.04021    141.84782    140.74644    140.73907    136.01532     140.1217    138.54089    139.00581    135.57476    135.26589     135.9803    137.57991    136.91701    151.08711    164.81966    181.70474      170.135    164.12405     142.2531    136.02057    130.33035    132.20669     124.9954    129.22105    128.06438    129.66513    135.88435    143.71999    153.60523    160.17519    157.71857    152.44358    144.07983    146.26882    147.78484    139.68239    136.41014    136.01303    135.16237     141.5207    129.89571    126.62023    123.44348    121.60677    119.01519    123.59973    123.64753    131.38925    141.45915     153.5917    161.91519    162.39766    155.76846    145.79525    142.30959    146.10279     146.7314    142.38292     136.7698    128.87195    123.67415    118.65211    119.83504    120.22926    116.89236    116.84764     117.0023    116.09637    118.59096    117.37507    115.34788    116.63824     118.6027    121.51577    121.83471    126.54604    124.01273     117.1626    118.67906    116.91798    116.82205    120.47826    125.56991    129.21474    128.25754    125.87757     124.6977    116.75158    116.99937     115.1225    117.43858    115.69648    120.07651    122.02067    135.49198    141.30091    148.51137    147.89183    144.88411    136.31631    127.10188     126.9146    125.59445    121.58178    119.95476      123.151    116.60619    116.79916    117.15122     116.0087    114.08324    113.63741    116.14832    115.85934    113.21731    116.55278    117.94386    116.32389    116.04652    118.03032    117.12371    122.90915    120.17735     120.7079    120.40022    121.60735    118.08904    118.65927    120.81298    117.02354    118.97492    122.07581    118.95895    123.88081    126.10232    124.25001    123.94487    122.18719    121.00559    119.71852    122.19218    121.16698     117.3929    115.74515    117.36886    112.87566    114.93349    115.63484    118.57049     120.6436    119.34458    117.79602    121.23829    124.03459    124.68499    127.73132    127.05331    123.93175    126.12978    127.92669    123.25417    123.71214    121.99047    124.01153    123.39009    126.11353    126.05216    130.00984    124.51429    123.58147    123.90598    120.58801    121.63738     122.0694    124.63488    120.69896    120.02827    118.95065    126.14865     122.4952    120.18785    125.93236      127.768    125.23391    126.49103    128.59138    127.88588    129.77484    129.58434    125.43549    125.60977    128.78169    129.48742    131.57924    129.43569    134.48779    129.96237    131.82553    130.35481    131.39158    132.32008    127.96043    130.26034     125.6901    127.60615    128.74229    126.97856    130.48238    126.87983    129.54794    133.58359    129.67076    130.23057    130.01602    132.58509      129.297    131.39871     132.5474    136.37427    136.54622    134.22403    138.79138    138.03883    142.58994    140.64682    138.23437    135.17516    143.91581    141.70691    138.65484    134.29288    133.31171    129.75094    129.57009    130.82507     128.9057    130.95946    133.13711    132.77263    130.39356    132.24325     128.4431    133.52518    132.30991    132.45779     130.0033    130.02758    129.24935    124.47589    131.39905    126.29162    127.47997    128.33758    130.33501    130.05256    129.88801    131.23624    126.65464    129.86435    126.26325    128.85353    125.23251    130.28036    131.53659    130.20665    129.55503    130.70371    129.14114    133.75663    131.07965    130.06327    132.70962    131.67061    131.44262     126.8917    132.62121     133.4506    136.66535    136.50099    135.98773    133.40862     135.8052    136.15329     138.2213    135.89292    135.56418    136.32238    137.60284    135.65105    143.09778    141.16426    146.58129    137.64858    136.77356    146.31022    148.03098    143.50341    146.80878    146.11245    144.43619    150.27173    149.78698    146.74809    148.96955    146.88917    147.86973    150.12302    148.42358    157.60624    153.86181    157.61836    158.28916    154.90624     163.4605     163.7547    160.24271     158.3381    161.21536    162.48491    169.72605    168.41909    165.57256    170.99081    160.32222    165.98145    170.38218    175.24315    168.98267    166.95992    165.97421    164.04451    169.14569    167.81698    172.44985    173.64958    176.04443    175.92611    175.77289    176.24873    181.75553    177.70595    176.67625    182.51462    184.69791    175.13608    186.23472    180.87752    190.41455    182.20266     188.1724     189.8595    185.09095     193.1007    194.20783    191.57914    186.37391    197.79139    197.50324    201.22425    200.95862    203.18693    202.44275    204.28923    198.13698    207.44868    200.95744     208.8506    212.57784    201.88632     210.8026    212.99914    213.42499    219.87961    230.33329     217.2319    220.45146    219.34399     223.9861    223.31365    233.20246    225.51066    237.79592     233.2621    239.18059    242.54677    230.25956    233.90235    235.78319    229.88986     236.9549    249.25292    244.71092    237.16409    247.52025     245.7196    243.80129    243.17191    249.94928    263.60541    269.34272    261.49012    269.54419    257.40916    265.64826    281.69142    262.27428    270.66304    279.11697    288.00807    285.82295    290.42372    302.37597    296.33471    313.08912    298.35775    296.38158    300.35984    324.90284    332.51064    315.17325     320.5788    331.80298    340.21924    358.47661    366.95071    367.61561    370.18237    408.23556    374.40111    447.15814    426.10595    464.43625     435.8738    419.06714    445.51841    490.62461    546.77908    550.62223    537.12584    557.97625    534.65754    687.94218    840.32247    847.82837    817.86691    870.89523      797.013    843.76749          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -3.50    260.03082    239.25164    229.00334    227.21435    219.03219    227.00982    211.86337    203.37845    199.66526    197.52936     193.0081    186.15765    186.49557     182.5794    183.37922    175.45971    178.22194    174.10246    169.26915    163.81575    172.25131    165.06802    167.20902    163.81638    167.96842    181.01389    197.90693     207.3857    204.64837     176.6779    165.43263    155.05573    155.47585    146.80086    152.24027    149.31141     151.6121    150.10911    145.48622    144.76805    143.72324    140.13935    140.95341    144.28248    140.99753    141.72849    137.14689    139.07061    139.93184    138.17967     139.4742    136.70498    136.80547    136.81311    138.53966    138.40667    148.42014    171.18439    180.84347    173.38633    160.44608    145.91811    136.86696    137.05325    137.09584    128.80366    130.11688    129.82179    130.30007    138.53443    143.00155    157.99928    158.29908    155.69352    149.60028    145.86105    145.07482    146.89471    139.23114    137.74464     139.8859    133.76468    133.90199    135.38915    123.76998    122.52571     123.0994    117.08991    125.26514    125.32446    131.67544     140.8278    149.29946    157.33935    157.38651    156.14539    150.63619    143.13455    151.65533    150.30519    141.75743    134.82681    126.54428    125.01314    117.06556    117.71793     116.7008    117.53465    117.86495    118.27018    117.33068    119.64294    116.46632     117.9111    117.42079    117.29075    124.23231    122.02483    122.07372    124.72664    117.15436    117.23805    114.56153     114.9527    122.47952    124.74488    129.10555    133.01412    124.85114    123.09889    117.85588    116.62227    115.67826    112.81648    116.47219    115.56476    119.60382    134.70547     144.8826    150.05075    152.08911    147.00839    137.21239    125.98878    123.03934    123.73231    122.54486    119.46107    121.78011     117.8883    120.49196    117.26624     116.5902     115.5243    111.89925    110.46827     112.9209    119.39536    117.49586    117.11447    118.04891    117.82632    115.65344    123.29014    119.27143    118.93093    121.63065    122.46464    118.75417    123.54661    123.13304    120.45769    120.76821    123.85946    122.93272    121.42324    124.29322    125.12408    123.93725    124.01081    122.85762    118.32855    123.06056    118.52591    119.77788    119.61805    118.32547    116.43621    117.90827    118.30262    117.90838    121.94517    119.55745    122.64823    118.50704    124.70911    128.56165    125.17304    129.12293    127.83771    125.82049    125.38511    125.78981    125.83923    123.13558    125.11448    120.62556    126.41727    124.59512     126.8098    129.23443    126.34601    128.15764    123.13647    124.07656    124.35294    123.67821    121.38906     118.9068    122.05455    119.00418    122.92409    127.08921    123.45494     125.7268    125.65264    125.53721    126.60993    126.66814    130.45308    127.54045    126.91171       129.55    129.53723    134.76852    136.35053      135.119     133.0466    135.50121    133.26783    138.12154    139.56411    136.26114    134.37758    132.65587    131.33353    128.99354    130.71268    131.85495    130.33195    130.57973    129.82058    127.64662    133.09632    133.11104    131.62404    129.82307    130.87105    130.63328     132.3937    134.69798    135.44826    139.52849     137.8312    140.12875    134.90203    139.88598    138.87916    141.36929    136.22374    138.39241    131.32888    135.26964     131.8172     131.4431    132.49538    128.94588    129.46552    129.38494    133.32848    135.10911    133.22829    130.78923    133.02073    130.26915    134.33832     131.1096    131.63422    130.78455    125.38474    128.19524     129.8733    128.62537    128.18527    131.07961    124.52785    123.38329    127.54171    125.70819    126.97221    127.98415    128.45246    131.24814    127.57488    126.21484    130.41488     130.2047    130.44809    130.24479    130.99375    128.04491    129.66794    131.20001    131.21449    131.47657    134.60351    131.84319    134.62017    134.44784    134.76663    133.03808    128.40299    131.59104    132.49143     135.8202    135.59509     134.0862    134.76051    131.47602    134.43768    138.25069    142.50691     138.7646    144.33292     139.8607    142.78097    143.95453    145.71293    148.34528    145.30059    146.49942    146.90838    149.47413    158.03592    154.69175    149.97528    148.55432    148.71087    145.62733    146.40652    153.97474    153.57278    154.46265    149.00857    158.02986    161.09174    160.10872     159.8315      167.708    161.40119    163.35576    164.02503    160.96711    172.50309    167.01065    170.12173    170.05562    168.38762    164.44069    176.71199    169.69015    170.44949    167.72002    168.62525    174.06266    169.74281    175.86086    177.35368    167.25954    178.36117    179.88043    176.74803    181.17917    177.40349    177.19469     176.8754    182.13539    178.32904    183.18788    184.53502     185.0527    181.77335    183.52056    183.76783    187.88287    186.78317    194.52544    193.48126    192.42943    194.06306    189.11779    201.73767    203.19111    195.46008    202.19357    208.68038    191.34527    197.02723    212.86522    209.76234    209.08272     207.8488    200.71713    211.66174    216.97512    214.96499    213.38061    218.05307    213.66619    213.49133    220.66697     234.6381    237.39615     224.2607    230.97196    236.94364    234.83001    235.58367     236.8809    226.34406    238.29157    231.90357    228.91945    250.18065    240.91644    243.12117    257.05609    246.23098     250.7842    248.15948    262.75797    258.43422    259.95789    266.83487    249.12542    240.12371    277.28875    260.66765    284.83965    264.31202    276.41245    278.75838    277.91477     305.3998    291.03282    282.87176     299.3361    290.33392    312.55759     319.1579    311.40552    331.31321    322.62366    315.93435    335.99277    309.25356     353.8572     368.0533    352.94289    375.16823    363.51462    358.71694    411.74752    404.04093    404.91996    444.67646    454.36738    447.15002    493.95572    554.08907    569.47471    531.03956    547.70792    553.89134    627.01337    858.82041     743.3329    826.93989    857.47131    766.46569    728.15856          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -3.00    257.94224     238.2287    229.37491     224.7222    221.16732     221.8773    211.48906     202.6255    198.62656    197.84768    194.48622    185.87533     184.3611    182.88237    181.92805    175.75924    178.91411    172.51947    167.21974    170.76413    165.43676     164.1194    169.71045    166.48847    168.82745    178.60747     195.0165    210.40397    198.75587    170.95144    157.47453    157.05079    146.53854    149.38419    149.37287    148.98371     145.9056    149.30737    147.28683    146.75421    143.05883    137.46628    137.93379    144.08951    140.62552    141.66786    138.37283     136.7186    143.45139    130.83498    138.80176    138.93838    136.16512    133.74813    134.60573    141.51496    153.13709    170.58527    186.35529    175.48752    157.71475     144.3104    137.15294    133.52305    130.11449    129.83373      125.686    130.41628     129.1815    135.88309     143.1136    151.61996     161.0361    157.89294    150.77688    145.67238    145.99998    144.78568    141.29362    136.06912    134.35671    136.16322     136.6077    132.41658    125.46852    119.09515    120.92828    122.00677    120.79958    122.15184    132.01801     136.5484    145.52944     156.8968    160.93139    160.62229    160.17819    152.71874    155.63746    145.53315     138.5078    131.75745    130.75695    124.81029    119.43533    120.31273    119.30191     116.6796    116.67144    123.50436    121.89281     117.9566    119.19495    118.18783    114.48363    117.63608    121.21735    121.97039    127.79648    125.26772    121.47616    117.04213    115.70036    117.09038    119.98386    125.58737    131.90438    128.42106    124.84974    119.39571    116.16382    117.42152    114.43263    118.79981    116.28177    118.13511    122.38547    133.40186    148.30899    151.09207    153.97359    147.87687    136.79714    131.56127    127.42467    121.72316     121.8748    122.77592    122.48542     116.6086     117.4173    114.49868    115.89167     115.0542    115.43338    117.14759    117.36944    117.98784    117.01352     119.1368     117.9542    119.57898    119.12704    116.87336     121.9462    118.86121    122.92374    123.31886    120.74181     125.3913    122.45829    122.42199    124.08096    122.79332    122.46888     123.9688    125.61134    123.70021    127.03473    122.14367    123.69983    118.49898    121.16212    119.63328    117.20303    116.32088    114.83525    119.20333    117.64558    115.93733    119.54041    118.42782    118.28776    124.71377    117.90169    127.81966    122.16553    125.52916    124.89528    130.49862    127.68755    126.55424    126.89712    124.04665    124.48579    123.58236    121.14992    121.11275    128.18527    125.67584    130.57641    125.44115    125.47357    118.93111    121.21455    122.72271    121.36167    124.04301    120.89632    121.83306     117.4204    120.92961     122.3821    123.20423    124.95715    125.90789    124.04671    127.42895    129.88739    128.48997    128.93434    126.06521    129.16493     132.5765    135.36978    136.83204    138.22329    142.08329    137.59806    142.05475    139.16125    140.24271    138.88095    134.30532     133.0119    132.69547    130.24947    131.51933    134.50152    129.81903    134.03616    132.75804    133.04114    135.16906    137.42669    138.14627    135.04332    135.94731    131.40289    131.69213    134.75814    140.95955     138.9053    142.45428    142.89222     144.8855    144.23791    142.80539    136.27109    138.06763    136.44806    131.44527    129.60641    130.58779    130.47618    128.25756    131.81617      127.409    131.06458    136.18452    134.18365    132.35844    133.71155    136.28057    130.34129    135.68773    133.86259    133.92542    131.67166     130.4991    132.13822    126.89671    131.91669    130.12499     134.6994    130.71646    131.48003    127.27657    128.24684    133.81134      131.171    130.39738     129.1955    127.79553    128.80828    132.10055     126.8982    130.24537    128.24279    128.44336    127.73856    130.73386     130.1498     130.1408    131.30227    134.31301    133.99506    133.00902     135.9098    132.26602    135.94988    134.12475    127.28772    126.67825    129.59602    135.86443    131.97467    138.12258    133.50938    129.85068    135.60279    141.46014    139.34747    142.24707    140.93654    139.79446    142.21829    145.22497    142.57449    147.26684    147.03555    144.49874    146.10859    149.05568    153.29707    150.94535    153.56243    147.77142    145.42596    155.13635    149.77027    153.63588    157.44068    156.86518    154.25254    156.68574    160.72425    160.68586    164.94551    164.13703    160.52336    163.38514    160.43571    170.22222    165.81544    162.93931    161.62752    169.52694    168.19392    169.62327    167.64471     170.5893    171.06748    172.76076    168.23149    169.36296    174.40747    181.11575    180.06201    171.26056      175.851    176.20206    182.81883    183.27215    180.47895    182.62354    177.29634    177.46212     182.3446    182.64984    188.30278     183.8579    184.64815    189.28988    186.18745     188.4686     192.4484    189.32563    191.19378    190.92378    194.51492     198.8501      198.583    193.38865     199.6297    200.05792    204.13231    196.20964    216.91148    205.79608     210.5909    212.89949    214.19549    210.05233    213.93778    221.29478    224.76496    220.55075    215.50352    224.24892    211.36248    216.39462     219.9719    215.23692    220.22349    215.54824     223.6481    237.11103    237.40071    231.16018    238.78749    232.88443    221.04746    239.82533    244.20908    251.42891    236.00776    251.29899    234.95521    251.19926    267.38563    259.27984    261.03537    262.81187    251.44894    252.68058    250.97991    281.40636    286.85456    283.48519    273.85147    248.84665    275.37119    289.17096    288.11676    270.56879    297.51159    283.47924    293.76492    303.96248     323.7081    310.41289     326.3807      315.628    301.20955    332.23443    383.28488    342.38631    385.83433    373.58699    365.83435    333.16242     417.6294    416.96925    413.80599    409.88068    429.29489    410.35956    481.62621    501.44067    570.17546    539.04307    518.14911    483.10408    638.23429    848.07472    842.89403    850.31315    809.56354    803.77066    759.06189          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -2.50    258.82673    239.82132    229.48546    222.98501    223.06877    217.56588    211.39257    202.27533    196.62696    194.19035    194.87269    186.91939    184.65378    184.34142    180.32449    176.38417    176.07666    171.91297    171.55024    169.22725    163.87683    164.32203    167.12544    167.98879    169.96676    175.79953    195.57685    208.29479    199.35399    174.78296    161.42594     158.5611    148.05333    150.97138    147.27211    146.96478     147.0942    149.08167    145.46346     142.0566    143.72742     139.5764    140.63409    141.66855    139.90215    138.64689    139.28712    138.98851    143.42456    141.07485    146.33977     136.6024    139.84809    135.95603    135.90545    139.25782    151.44435    168.27821    181.13388    171.35823    160.68357    145.49452    139.86829    136.56403    130.40359    128.87861     129.8609    127.95895    130.97616    130.94268     141.9614    155.14558    161.91492    160.77049    152.51729     145.4041    148.42835    146.12747    134.08954    136.05028    137.51889    134.93994    135.18022    131.08493    124.07189    121.99656    120.93945     121.9206     118.8595    123.15628    125.78859    137.56046    144.01764    155.94976    162.19724    168.08304    162.37315    154.86102    154.55333    146.73983    138.00346      129.524    128.42004     125.4222    120.26252    119.01787    119.82623    117.34093    119.66221    123.00831    117.05322    125.00064    115.53008    120.17967    115.97114    122.25848    123.44175    124.68081    123.78929    125.21397    122.25923    117.63065    119.88384    114.34704    120.87111    129.96257    132.58935    127.13689    126.06531    122.44124    117.19297    119.90798     113.8414    116.20767    119.90388    118.33255    125.23317    133.63512    143.31962    152.68797    153.95263    149.33911    139.16771    135.63497    129.01376    126.17517    123.27469    123.01459     123.0743    116.10899    119.79454    115.41914     116.7341    111.59475    113.14088    113.34033    112.61698    117.00893    116.33527    115.50562    116.40852    120.83383     116.7378    116.77142    120.52066    119.46761    125.91763    123.68881    125.89105    122.69379     126.5195    126.99214    123.35637    123.82914    123.44894    119.51908    125.91105    126.17636    124.05612     121.6808     117.7843    122.09941    120.49838    121.35053     115.8958    116.38419    118.10885    117.21874    118.03858    117.08326    118.83292    116.22985    120.55905     120.4183    121.88908    124.55891    126.51402    126.17679    127.74404    124.90064    126.76251    129.13684      123.754    121.22561    123.71971    121.05237    124.08768    120.80196    125.12397    124.18351     124.1333     120.4481    124.76455    119.19642    125.36989     124.4912    120.21656    120.33484    118.62338    120.44077    118.79948     122.6646    123.74346    122.97262    126.19651    129.76304    127.18346    127.69635    132.40918    129.50619    127.04468    130.15994    132.40487    129.30242    131.67744    131.80639    138.82839    140.72247    138.81045    140.69627     142.1361    136.13099    134.70129    136.71576    134.40837    135.14015    130.54466    131.53718     136.4688    132.41182    136.69241    144.89907    143.98704    143.30618    142.44132    139.38103    135.24648    132.30531    134.07716    131.48689    133.77945    138.71687     142.8477    137.74019     137.8574    139.18329    144.29919    143.96644    141.62679    140.38059    136.69094    133.28646    131.28636    134.19899    129.21465    130.13148    129.04229    127.16881    130.11859    132.45673     133.6251    129.80045    132.27335    132.37392    133.95566    133.74056    134.31406    133.84591    130.80975    129.54276    133.63775     130.6135    131.61871    129.02158    126.68415    126.68121    129.90044    130.43031    130.85933    128.30229    130.67993    128.13951    127.39269    128.92377    126.45693    131.32729    135.85165    129.86374    129.36705     129.0366    130.33404    126.92311    135.78859    133.40324    130.84371    129.58943    133.80695    129.67671    132.38185    133.61546    132.30286    136.50791    133.76809    131.92186    136.50277    137.62318    137.02425    131.29299     132.6801    134.14393    140.90958    141.68727    137.67199    141.52375    139.53929    143.35754    140.69146    146.68254    146.82337    147.86177    146.26379    143.09199    141.04263    152.21996     146.0258    147.18363    149.18594    148.93419    152.34546    145.90603    155.42108    157.17037    153.06515     160.0299    160.05805    159.84848    159.75287    160.64476    163.19748     163.5723    156.81875     165.6807    163.24199    166.80145      167.925    171.76818    169.07274    166.56189    164.52031    170.70168    168.26538    169.95265    170.50526    167.63647     172.1505    171.87147    176.35599    176.20086    172.86812    173.33083    181.04614    178.80488    179.56825    180.78856    183.44543    179.23411    182.01135    175.32864    190.69978    180.56927    177.66604    184.09112    183.78655    185.86866    188.54198    190.62157    199.75225    192.31671    181.00755    197.69033    194.35122    204.01686    198.27209    204.49098     196.2915     201.9809    194.68098    209.63527    211.71537    214.52887    210.43051    209.94853    200.39439    205.14657    216.34287    222.06396    217.14962    206.66219    222.31614    214.80547    219.03297    230.14896    239.90466    231.89535    225.67575    225.80657     222.2164    231.69935    230.10271    229.95479    225.43521    229.26646    234.48517    248.93369    250.95744    247.00435     239.5769    255.71754    243.60742    239.64936    253.80735    258.53726    251.03767    250.55403    249.55017    259.69641    275.63039    264.21861    268.95661    269.55359    275.00727    244.37601     277.4513    293.10332     281.7768    306.24207    271.98238    291.01391    289.77428    309.05874    322.47849    318.85085    342.82305    320.03029    317.50977    336.97422     360.5563    361.50721    342.35942    349.65528    388.56387    356.54614    427.18276    411.89855    414.66364    387.04772      423.283     440.6054    473.40966    522.87821    549.65169    545.77686     490.0519    531.57933     649.5738    795.87557    686.26248    762.67162    813.19652    672.25653    694.69373          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -2.00    259.88563    241.94612    229.11468    223.00349    223.49637     219.8023    211.19469    202.92994    195.10333    191.92638    195.53187    188.23946    187.08231    187.47743    180.16595    177.27538    177.61733    175.98953    171.23116    170.68951    164.47502    164.46979    164.28676    168.60854    167.50999    174.64859    194.12481    207.68133    195.78495    175.31968    162.62636    156.62477    151.20738    149.43068    146.46598    150.65148    152.93378    150.00368    150.18223    144.92321    144.32783    144.54666    143.84942     141.5995     145.8303    138.51291    140.75985    141.28063     143.3534    139.09054    139.30785    139.88697    135.03904    135.77734    139.28133    138.60937    152.84125    167.90546    176.37816    172.11667    156.09378    144.70991    138.84337    132.95012    133.51003    132.25605    131.60038    125.77528    137.35727    137.63492    142.23457    155.38505    162.25267    155.37561    150.54969    146.11468    144.16439     146.4307    136.48985    133.19703    134.44565    129.90453    132.62903    131.28168    124.47073    122.49044    119.96733    125.25497    123.51624    122.71718    128.93507    134.10644    145.13393    153.87743    159.61006    163.20455    164.24772    158.74073    158.23385    147.31883    142.44135    132.07005    132.31801    126.64341    124.56396    123.52293    117.97146    120.45535    118.74266    123.72901    118.60295    120.15279     118.6292    113.28131    114.69335    119.19974    124.04128    126.65554    126.71176    125.61806    120.24514    114.97473    117.62207    118.70669    121.49679    128.16172    133.59084    132.72294    127.49789    122.98942    116.13558    119.20164     116.3774      114.884    116.32035    119.80752    123.60961     135.2009    148.41441    153.62991    156.67673    151.76646    141.51019    135.24549    132.78193     127.5991    127.90609    126.26377    120.50256    118.23157     119.2728    118.58924    114.07278    111.88861    117.83402    109.56328    117.88957    118.86421    116.80737    120.61729    120.28382    116.51245    116.27903    120.80199     121.9812    120.32152    125.02075    125.91403    121.81069    123.48743    126.21483    125.12319     127.2349    124.71621     127.0057    126.07794    128.18111    127.08435    127.96618    126.71417    126.48023    120.80719    118.98974    113.90141    115.66616    111.30577    114.60948    121.14674    116.41103     120.1576    117.59206    122.20144    120.55623    120.86292    120.22164    126.12823    122.12993    124.65783    126.60829    128.68025    127.91313    126.30941    125.89031    122.44603    122.67252    124.46787    120.94481    123.62902    127.03626    127.17058    126.19573    124.61395    121.34167    123.11303    123.77652    121.40179    121.76468    120.91606    117.61008    120.24005    121.09802    122.10311    121.17559    129.57786    125.59821    128.77521    128.51398    130.43032    129.62545    130.41821    132.83803    126.62469    126.78682    125.66124    130.82749    135.75974    135.97147    136.92964    139.21889     131.8092     134.8072    137.03054    137.21288    137.47336    132.32262    133.86919    131.16446    129.12008    133.48344    134.46954    136.57469    139.61712    135.98641    135.92745    138.22513    139.03083    134.73657     133.0272    134.00263    133.85524    136.35998    139.97774    139.48347    141.38857     140.2043    139.89633    144.90996    143.23386    141.46966     138.1659    135.30036    131.20991    130.27019    133.82569     134.3657    130.29785    128.48258    132.27181    130.08961    130.09107    133.92252    134.70346    132.85299    136.08702    127.77296    133.52299    131.62907    133.64917    134.50665    134.73263    131.17389    128.35112    130.90718    131.56548    125.09691    129.09224    129.06346    124.40861    125.42118    128.75412    128.49115    124.89442    130.02535    129.35778    130.07158    130.97391    128.21766    128.22822    131.76727    131.44402    131.10775    133.67182    136.25421    132.79697    133.04105    128.80895    131.46145    132.75905    132.72583    135.67461    131.20562    134.45022    133.91802    131.44486    136.58068    128.49703    134.88895    131.19266    134.65052    134.22022     136.4201    143.60092    135.79952    137.17615    138.05458    141.61287    138.36082    147.53212     146.8598     147.1655    147.26936    141.48476    145.86511    149.07808     148.7507    145.53538    149.07169    150.54285    148.88677    149.94993    155.23371    153.27526    156.56836    153.35875    152.92142    155.73151    161.53618    162.27755    162.95962    165.47552    160.06814     164.6989    162.43132    165.41941    166.92292    168.18312    166.01092    166.92579    168.62144    166.58171    168.52772    168.62324    170.32261    166.96122    164.42699     175.4176    170.71647    177.45767    179.10183    179.38832    173.83312    171.41222    179.26427    177.04982    177.97101     179.3436    180.69236    178.42883    190.51695    188.60514    186.28102     183.0308    189.42199    181.89708    184.32178    192.68743    193.75614    194.03578     189.6872    189.93425     196.8102    196.84063    204.61461    190.47793    199.56433    198.36344     200.0461    199.86789    218.79762    209.28811    207.55734    206.52316    212.68689    199.78829    209.83781     222.3231    229.78299    218.81626    211.35636    212.97261    219.54874    231.38265    228.15913    222.86025    220.11624     226.1353     232.6757    234.53376    232.61554    232.74257    230.14893    237.01468    222.30316    234.61366    244.76083    249.75886    243.09504    233.75357    231.30863    245.95688    239.37197    256.27752    267.18362    249.07046    251.11405    243.28867    268.78241    255.24843    258.57871    277.72758    253.36856    253.21996    279.22567      290.604    292.06062     282.9495    288.84359    286.00674    293.61277    306.79707    317.84659    319.54606     320.2141    297.67396    312.16779    335.02155    362.04112    384.16501    364.46021    366.84802    349.73922    355.39545    383.20762    433.96337    427.38192    371.97945    427.53857    451.27135    419.08135     545.5905    513.21302    523.91838     512.7833    522.31739     563.5866    652.83993    773.17729    660.15477    677.53527    703.92254    766.04848          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -1.50    260.66986    243.95936    228.58428    223.02899    224.06191     222.9278    211.13003    204.60715    194.09198    194.42534    195.92612    188.30109     187.8833     186.8394    182.73947    177.05301     178.0238    178.52589      170.127    173.41858    166.34426     163.5675    168.82254    168.49946     166.9034    178.96966      196.664    205.14215    198.25875    177.84526    161.07919    154.87921    151.20598    149.60061     151.8638    149.50695    150.33641    150.17519    151.13542    143.64951    146.91901    139.27721     142.8991    138.54628    144.17457    137.93774    135.85709    140.08234    142.23448    134.38147    136.18727    139.03683    137.51256    136.93361    138.61382    139.69084    154.31514    168.67318    180.03428    170.23188    158.08591    144.96659    137.48727    133.11222    132.97965    126.02904    126.46885    127.67747    133.52057    134.57182    145.47059    155.78083    162.23807    157.19789    151.08085    147.50377    144.45503     147.5535    140.20886    137.58965    131.37766    129.31135    130.65398    129.93022     124.6342     123.4503    122.66852    121.55477    120.58296    123.46939    127.83807    136.99454    144.11757    154.24288    159.05907    157.54365    158.44709     158.3508    162.50332    154.48246    143.74935     138.9028    127.55675    125.12924     120.9337    121.03405     119.6661    119.08444    117.79854    119.95192    120.00751    119.87783    117.60497    120.34278     117.6225    122.91886    122.21511    125.97513    127.13414    124.21607    121.74708    119.57941    115.59951    118.39208    126.07471    127.22346    132.30297    133.98409    127.49424    123.54694    118.14447    114.31266    117.58044    116.15775    118.73123    120.44544     124.9482    134.97457    147.55006    156.06172    159.39052    151.37761    144.02749    138.46951    130.68537    126.40367    125.83025    125.62792      121.172     121.2607    118.29585    116.98425     113.7908    114.59071    115.68117    115.01787    114.67716    119.46268    117.75314    120.94742    118.45584    121.39865    119.81033    120.24591    122.92513     124.6303    124.52547    123.06456    126.02952    126.93385    128.03729    124.58023    128.32366    130.07529    128.86052    126.65919    127.56596    131.43888    125.34143     125.8446    123.13124    124.13504    123.41864    122.41505    119.33874    120.27526    114.78683    116.73179    118.98819     115.0391    120.10104    120.32089    123.66737    118.51685    121.23565    127.57791    125.62827    127.50395    127.33906    128.60777    126.33958    126.81633    124.50703    119.91916    123.70486    121.25454    123.48692    125.68905    124.92441    128.79382    127.76973    127.30174    125.45654    121.90378    122.19977    118.24245    122.90073    118.14958    122.58737    121.89197    120.00718      121.254    128.57633    126.35177    125.94945    123.17032    126.89666    130.70336    131.93336    128.17336     130.4252    129.51037    128.85279    127.30975    130.17357    131.37744    134.85454    133.70495      135.244    130.26932    135.75991    135.29977    133.17024    132.16402    132.87231    134.37153    129.72071     131.0752    130.13531    128.89216    132.08549    134.28695    131.61922    135.27838    135.71128    136.71204    136.08573    133.48559    135.29537    135.71675     135.6167    136.81436     143.4441     139.1242    142.00464    143.72918    146.03877    144.38686    141.51477    138.22663      135.324    134.56633       133.85    134.12311    130.51257    131.29804    134.24628    133.65312    131.81818     136.3657    137.28277    132.85561    132.12854    135.69876    129.07261    133.09474    132.90249    134.81574    135.60419    132.80523    132.44715    130.39125    131.91266    132.60237    131.43898    132.64603    131.77159    127.39836    132.06634    128.91023    129.76175    131.93414    128.78571    128.71096    126.06987    130.54166      130.841    131.00346    129.93124    129.28283    130.55724    134.12491    133.90224    134.13484    132.61151    131.41739     130.9943    131.03433    132.42538    132.89615    136.78954    133.84907    133.49511     135.9404    133.49085    134.71809    134.98073    130.19124    131.22189     135.0184    133.44712     137.6842    139.48085    141.30758    140.45626    140.71806    140.57346    141.86061     145.0245    147.24651    138.95431    141.98135    143.39346    145.75416    148.22651    148.35138    145.31945     147.0984    148.78816    152.43148    152.33093    157.76794    151.84401    159.67243    153.71754    158.05231    165.90297    158.88612    163.43293     160.3121    162.07528    162.70337    164.76356    171.26379    168.21958    165.74005    166.12557    164.07741    168.12954    171.40286    163.65074    171.75524    169.78887    165.16822    168.65108    174.05689    171.41916    175.89034    174.15114    173.50955    175.71363    173.74653    177.98058    179.99632    177.87137    180.04196    173.52566    178.81172    189.16702    181.17516     189.4278     182.7491     176.1475    186.56025    182.38151    183.72932    187.34826    194.54533    196.67409    191.19558    194.37559    189.63479    197.79161    200.91497    197.39835     201.7308       193.52    199.89436    202.76838    200.76817    212.36494    191.31609    220.58784    202.16278    222.07436    217.53762    211.92552    214.17923    209.61572    214.31477    219.28239    227.69705    224.67362    226.08039    226.42142    222.50817    220.88221    243.44349    233.11775     211.1538    228.37723       227.95     225.2177    246.48386    241.33741    240.08098    232.67381    232.35745    232.16318    232.94897    246.28251    245.24778    253.45909    251.25945    241.36121    244.16039    267.62062    253.53814    248.41567     271.0568    279.33677    267.94006    260.46659    267.41608    283.19652    283.02678    295.06263    259.09626    287.76947    308.45954     295.2082    301.07591    299.99009    319.68184    290.97938    344.23067    354.65852    334.58582     354.0844    354.18163    344.17753    335.41965    381.96455    408.33146    414.45998    424.36118    427.28493    445.77935    428.01103    492.17919    498.03838    448.97414    526.09323    503.32339    545.50452    738.55597    698.36891     824.3253    764.26952    756.98888     685.7669          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -1.00    260.08202    246.19117    230.06593    224.23447    223.43653    220.95281    211.61451    206.50207    193.63579    196.36229    195.45621     188.3117     188.2498    184.41865     187.1742    175.63722    176.79058    174.81184    172.19276    171.32708    164.52555    162.86665    167.90996    164.09507    167.57039    181.87178    195.76573    205.20789    201.54681    177.45416    158.49525    155.41259     153.2607    149.46439    154.42536    150.17638    152.45168    151.72517    150.09082    141.96208    143.70769    143.65477    143.58095    135.85121    144.46006    143.10802    137.17271    135.70288    138.26696     139.1077    138.45183    136.86598    133.10608    136.58402    134.65325    139.50243    153.25687    165.44718    178.87225    168.90124    157.21545    144.00157    137.64884    130.56507    128.62844    130.27064    128.39477    128.50981    130.21255    132.49408    142.82401    159.69821    162.87511    156.55869    151.71829    147.50351    145.72164      146.866    141.00327    135.63162    135.27225    132.87285     129.4406    129.17917    123.88023    121.77558    123.01742    118.87342    119.84297    125.48373    132.96632     138.9636    144.13714    149.25278    152.22799    152.43321     160.9725    159.70208    162.16578    158.24171    148.07321    140.30095    130.80873    126.45137     119.3344    119.24647    120.19907     119.9545    118.62864    121.93362    119.43295    122.54734    120.36499    117.37945     119.0777    119.00764    123.65803    124.37419    127.10352    123.92441    121.99366    119.76023    116.09839    116.31637    129.08352     131.8078    135.66413    132.07675     129.8998    122.06373    121.09246    117.62499    115.81907    117.41707    115.59163    121.28558    127.66134    140.41971    149.42137    158.16893    162.87071    156.67208    143.93185    136.84646    130.59914    125.08913    124.15073    125.59442    122.23858    119.27789    118.72875    118.29331    115.48719    113.65104    114.81957    113.30466    113.15657    116.96177    119.61944    119.62004    119.90113    117.54071    120.26276    125.03434    120.03192    122.21895    126.28415    129.91136    126.11327    129.08588    125.67668    127.25195    128.04594     125.5776    125.40677     125.4006    126.49671    129.62879    127.44683    127.04626    124.56823    122.01528    122.98834     121.8537    119.79497    117.25903    117.50135    118.19796     114.7585    118.17741    117.74288    119.86635    121.44837    122.66511    122.52268    125.50617    128.50306    127.55435    129.24017    128.58862    125.40224    124.78083    124.86703    126.42283    126.03616     123.5881    122.83895    117.61188    120.78349    128.59489    127.17343    126.75169    127.10914    121.25691     121.1292    122.81631    120.14631    122.75874    121.65854     120.5642    121.74508    125.43504    124.88323    124.50097    127.05361    129.36236     126.6751    128.45551    131.24396    131.49936    128.14179    128.11115    127.88788    130.93437    132.76813    131.53747    134.94952    132.33434    133.97686    134.21037    135.62792    132.62283    133.58723    130.22107    131.94324    129.66126     129.6236    129.76389    129.09973    131.90352    137.12585     134.5631    132.44214    133.71093    138.77029    138.03458    138.29287    134.95703    134.79509    140.62899    139.56718    138.38104     142.9936    141.08968    142.64215    144.39507     142.4488    143.31305     141.3289    139.06172    137.38702    133.57622    132.26243    133.51854    135.22743    135.42881    130.69362    133.91783    130.88389    134.53832    136.56193    137.39056    133.89906    135.09992    134.99808    133.04004    132.22337    129.52894     132.7216    133.33387    131.99655    129.81247      133.379    131.63555    131.57701    129.38763    129.67318    128.82017    132.57499    132.92991     130.0505    127.46293    127.46851    131.26737    129.62034    128.53899    126.53704    132.60722    130.19716    131.67239    130.31207    129.12151    134.06263    132.74523    131.62594    135.15414    133.48621    130.77734    136.27887    135.86634    129.98382    132.00567    129.17731    131.60953    133.73173    134.69274    132.57566    132.65237    132.62434    132.85858    134.15087    135.02148    137.04675    137.86234    141.63769    139.36622     139.9282    145.44366     147.4169    147.75836     141.9049    142.18437     141.4491    147.54463    149.19992    144.39628    149.85512    147.08505    146.30699    151.76511    152.63923    154.27031     153.2923    152.59634    155.01939    156.84158    162.25237     161.6074    162.99511    163.24727    159.93098    161.87832    164.69436    166.74143    166.79681    167.70392    170.02257    163.22226     168.3298    167.81419    173.36203    170.16837     168.0353    163.36422    171.96099    174.25724    178.31416     174.7703    170.77598    173.09781    176.98529    174.28253    173.26408    181.13686    172.15454    176.43165    178.95148    179.16784    184.28967    181.48382    187.58926    180.82061    177.54681    182.58142    181.96015    192.35997     185.4407    189.16855    193.24246    200.08233    197.99045    200.98598    194.43468    203.23522    192.06502    197.12271    201.77253    201.15146    205.50512    201.64652    214.07316    208.10964    207.99253    214.75868    221.35664    216.05752    219.05059    219.65434    220.19598      215.576    206.98477     230.0616    229.87015    224.02866       222.54    219.46804    217.94911      236.018    225.75508    215.15417    235.36905    226.59475    215.54854    228.87803    240.79684    250.51003     239.1192    234.37685    245.82925    242.01446    235.10221    248.68687    255.88539    242.76388     247.8268    242.29921    265.27348    247.71574    269.06466    259.54679    253.96072    256.63476    261.70503    282.31483    279.21685     294.9805    272.97557    290.47959    268.52448    318.15738    288.79107    306.79381    315.76069    307.26212    294.87784    317.35074    353.02832    331.14064    351.96465    347.93667     366.6439    351.46168    366.71101    408.23338    409.76779    420.27989    405.24753    386.84402    434.82201     480.1662    491.11111    520.84245    515.45705    486.68711     582.8849    678.74931    778.24827    703.69031    656.22126    552.20358    665.01182          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -0.50    259.54101    248.41108    231.73276    225.74018    222.26292    215.34466     212.4708    208.57456    194.31764    196.47205    190.50452    189.75834    191.70324    180.57414    187.45669    174.77272    175.19003     174.0312     172.9117    171.45515    164.82389    165.42545    167.01961    164.75228     168.2933    183.45519    191.62762    205.87232    200.85294     174.9112    156.07775    154.85935    155.25706    147.01535    153.94852     149.7325    155.32327    152.17855    150.40074    144.89127     144.6537    142.54437    143.43991    142.72397    143.45557    139.69096    136.70698    140.10071    140.04213    135.33831     143.0249    141.93363    134.13449    136.12648    135.83913    141.93797    151.39322    165.91009    176.76897    173.97607    157.53063    142.70327    139.19419      134.634    132.79423    130.16049    128.37747    131.68269    132.30071    132.35869    140.42196    158.68891     158.9346    159.49535    152.91245    148.31508     148.3891    151.58735    144.00872    143.14799    138.69822    138.09454    134.23879    131.62834    124.35577    121.94529    122.37765    118.46952    123.60158    125.86891    130.80684     138.9902    144.14191    148.56325    153.60342    152.56119    158.46732      162.389    167.31061     162.1783    148.03099    138.22988    130.20715      123.536    122.13844     123.2552    123.02778    118.74911    120.83425    121.33004    122.48401    121.28917    120.00957    116.85894    117.42397    122.63943    126.37835    125.88532     125.8019    126.41768    121.49457    119.02118    117.55994    119.33697    126.53716    132.12149    130.54445    134.99868    127.19019    122.03183    119.89242    118.20024    115.42152    117.10821    116.13584    120.24712    123.70357    141.06764    152.41153    161.17116    158.73715    157.85704    145.77968    141.79462    132.82715    126.16836    122.86096    122.98573    125.32148    120.40205    117.23825    118.33242    116.13088    114.65383    112.87559    115.46311    116.00246    119.39675    118.79257    119.37278    116.08213    114.16819    118.37839    118.17222     120.6674     124.9581    128.03941    124.44968    122.68837    130.16451    128.60346    129.84426    125.90506     125.6485    126.49656    121.88197    127.55481    125.45341    126.31011     127.7962    129.67409    121.93903    127.94229    123.22278    121.18428    116.79675    117.11566    118.05556     116.8571     116.2253    119.12479    119.88787      122.241    120.53764    122.29449    123.75075    127.98106    125.42239    126.98707    129.44094    124.92988    125.14521     127.1444    122.95143    123.13117    120.27554     120.9751     121.2503    126.06747    128.58735    125.44732    125.12475    121.58062    122.32425    122.64225    122.71893    122.99477    122.66937    118.84532    121.77489    119.73098    126.88749    126.52439    123.83397    130.08648    124.52117    125.40245    127.84107    129.65364    127.40533    128.70273    130.52168    132.90003    132.36822    133.02113     133.5044    135.97789    132.29707     131.8806    132.60752    130.33294    134.30817    129.01767    128.05198    134.64277    133.94286    130.06443    135.34976    132.54998    130.44897    129.34712    131.17147    133.50948    135.51517    139.41285    138.81028    135.71707    140.81148    136.45795    136.36058    139.56115    141.30272    144.75499    142.10663    145.07398    141.32811    143.76869    144.96045     142.2378    136.21126    137.33498    134.77814    134.08232    132.79838    132.90933    133.28833    132.43338    131.83855    133.21846    137.19674    134.96782    135.37393    135.33921    137.83419    133.02284    131.84717    136.32927    135.49639     132.2443    132.05325    131.27376    126.80914    126.74601    132.72555    130.77898    133.02372    130.92822      127.901    132.43523    130.75131    128.95337    127.15837    127.98782     129.9202    129.74152    131.68542    134.24439    129.79236    130.61859    132.47293    130.16124    130.33239    131.45283    133.83927    135.62119    136.79903    134.46468    132.16322    135.26292    133.37389     131.5293     134.7538    130.77896    133.56111     132.4934    136.26421    135.94129    133.70792    131.27332    131.72351    138.12676    138.90521    141.63851    137.12583     141.1935     140.7723    143.72198    144.19373    142.93095    140.87946    142.45844    143.58359     144.0353    147.54808    148.51847    149.78363     149.3567    146.64423    146.89299    150.56774    151.39866    150.51143    154.31272    157.44501    150.43077    154.44785    162.04307    160.07871      163.194    160.34141    161.47176    161.66889    164.89933    158.41857    165.60723    164.11935    163.24116    163.99777     161.1114    173.45708    172.66738    170.11697    168.49444    168.54111    167.86688    169.17508     177.4937    174.22976    168.28219    169.85978    168.83606    174.90515    178.99652    178.92472    175.19118    178.21621    178.21223     172.1955    184.52904    181.42803    183.09245    178.84289    184.04634     181.1856    188.01705    192.81179    182.50084     192.0829    190.71865    198.98919     199.5927    195.94374    190.15557    197.75561    197.13055    199.54755    194.30678    195.69415    207.27069    202.56031    197.27682    202.09935    207.64018    207.15338    211.59417    216.20462    210.22726    213.11209    224.71818    216.07365    210.46224    226.30132     224.5671    217.66329    223.74701    206.77529    218.32656    230.88108    228.00785    227.16778    228.20331    212.51239    233.75508    228.02838    237.98353    253.68515    241.53094    231.17311    229.89822    241.33724     242.6796    245.45216    250.78568    247.29218     266.0575    242.07546    265.14918    262.28785    265.78034    246.63609    253.87999    249.56396    274.38847    261.00358    265.90786    272.60005    269.01741    287.74827    265.90903    262.26446    301.19404    311.36995     299.6813    298.57616     315.5089    297.35497    348.54811    344.40107    328.23422    364.48267    321.45001    359.57877    405.83557    378.30212    413.85549       412.32    405.43888    392.01516     443.5809    480.87275    506.97746    511.31805    452.82379    541.33014    536.86556     674.8886     680.4434    698.22992    744.42671    697.51736    679.72964          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        0.00    258.74869    248.65342    233.87741    227.69238    222.33828    212.84516    214.28158    209.65509    195.61721    196.43667     187.0828    191.06457    193.54669    178.12334    186.40658    174.90214    174.50384    174.84696    173.66389    171.73089    166.59723    168.65243    166.48047    169.77085    168.90743    183.75415    187.92521    204.98581    196.84593    176.50257    156.49215    150.85423    152.68507     149.0604    155.36589    146.44557    152.30068    149.10353    147.50495     144.4707    149.08448     144.3366    146.35754    142.21781    146.71344    143.72197    138.94816    132.67847    139.76472    139.01952    138.71517    139.49024    136.27517    132.81961    131.20912    139.65753    151.32675    164.83388     174.8961    170.75933    157.14933    139.35557     134.5719     132.0915    135.91337    127.99263    127.82004    130.60355    130.93031    134.09007    144.70135    157.00161    163.17439    158.54608    150.51748    148.83716    151.08269    149.00528    149.07445    148.45705    148.79057    141.89612    140.02791    129.84893    123.05937     121.2109    120.90199     120.8394    124.54567    125.19826    131.31222    138.40482    142.29891    152.40331    157.13788    160.06605    160.78558    164.67185    167.27623    161.60213    148.39131    136.25936    135.94316    126.29168    121.21634    120.85878    117.25134    119.67572    120.60596    122.43546     120.7667    121.12807    118.78629    117.43925    119.75054    120.67835    121.84562    124.90846     129.2286    124.46324    124.35664    119.92881    118.39315    117.69802    127.21758       128.22    135.27775    134.85719    129.59658    126.37506    118.41229    118.56293    116.75736    116.16856    117.92373    123.38436    124.88068    137.95068    151.07863    159.56004    161.34725    158.61301    145.38717     143.1762    134.91168    124.94491    123.79642    121.65469    119.36655    119.65853    119.69021    121.54576    116.49914    114.02063    116.09115     110.3263    117.15915    120.19725    119.18698    120.25294    118.28497    119.31632     117.6993     121.2409    125.20275    125.19348    125.99488    127.54564    125.57627    124.68305    129.42869    129.85953    127.50269    128.26628    124.88131    120.68727    123.56378     128.8755     125.5026    129.29289    128.60672    127.06429    125.04701    124.68741    125.64858    120.47143    118.86365    119.24758    114.33254     118.9074    116.33461    118.99004    122.93876    121.92643     117.9149    121.54188    128.57925    128.91554    128.43424    128.63712    128.78702    127.09872    123.46426     121.4282    124.48831     121.3301    121.67809    122.79295     123.7664    126.89735    130.25876    124.39652    126.11334    120.84416    119.27215    119.08715    118.22308     119.0793    121.92151    120.27659    119.78693    122.52478     122.8813    120.28988    125.56319    125.87187    126.86648    128.83729    126.57444    129.83201    130.64555    133.15982    131.53969    133.04652    134.97967    134.06965    133.91614    137.34971    134.61832    130.57483    135.45124     135.8141    134.35998    132.56454    134.92076    133.36078    130.37998     133.6111    132.97615    132.58791    134.55887    132.70867    132.44188    137.38475    138.80177    137.13973    137.57502    136.18943    132.69551     135.7363    134.95205     141.9919    142.95106    145.26841    143.93287    143.01546    144.88056    141.49346    141.53316    141.73892     139.1511    135.35582    135.62944    136.55663    137.77534    131.46702    130.68889    132.68576    133.24074    137.77869    135.17266     133.3679    134.49528    135.77652    131.15491    136.37963    134.96424    132.89578    132.84881    130.89568    130.86728    129.34146    132.72669    129.06202    130.22474     129.4289    130.24535     129.0737    130.44233    131.36102    129.72273    125.86279    127.49781    129.23634     128.3955    129.03246    136.69543    128.54126    130.77705    128.53349    131.41753    134.72449    131.18639    134.43092    131.95902     135.1764    134.14551    134.33404     135.5902    131.85832    136.51154    132.28955    134.66987    131.75699    131.14846    134.74838     134.8181    134.77806     134.6543     134.1122     140.2657    145.03068    139.82832    134.99189     139.0632    137.96822    136.68932    146.13952    143.30371    145.62667    140.76095    146.97506    143.51298    145.81483    150.00656    149.11041    147.62302    152.95683    148.49066    148.74846    150.44217    150.42415    152.38873    156.73017    155.68242    159.03399    158.27249    158.93525    162.74176    161.90479     164.0562    159.89751    161.18116    170.58419    161.47442    166.34592    163.21399    163.86267    160.79418    162.15251    165.83702    170.31694     168.6504    166.48924    167.63189    165.43019    173.60433    174.15031    172.93846    172.21755    166.51554     170.4323    175.21378    176.89144     177.0308    168.34048    177.75066     177.2027    181.62719    178.27165     182.4956    179.06313    185.03365     182.8324    185.19435    190.41457    189.32617    189.06201    184.56466    185.09238    191.61647    188.30993    183.87904    193.92913    198.36424    194.80405    186.00648     201.4913    195.68372    201.91361    206.40464    197.25134    206.47955    205.15204    212.80757    213.22829    208.99166    212.35123    213.27779    207.40276    212.15789     212.7835    226.12663    223.82638    217.16931    219.57895    205.00982    227.71152    219.77034    226.56246    233.60828    230.72461    225.77473    223.79255    231.23734    234.29579    243.46605    237.72581    234.48328    240.09951    232.11965    251.82828    245.91075     245.6412    227.35455    232.46903    262.67309    257.49944    269.48824    269.43559    256.20898    271.61667    260.33623    281.15017    274.59866     278.6496    278.97804    280.51813    292.78313    317.84282     297.1439    301.19335    295.97821    304.09993      313.419    313.91626    351.90032    375.52423    350.86567    376.72428    340.71642    344.17592    400.17603    382.43906    407.47392    384.54757    361.52905    404.00405    460.51223    492.99549     537.3238    523.60942    489.32002    512.58388    589.61771    618.72631    870.53604    623.91733    616.99429     614.6424    712.08198          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        0.50    257.50244    245.34822    236.90999    230.55581    225.14787    217.54616     218.5965    208.19919    198.66903     196.0035    188.30061    192.02534    190.37145    180.49809    182.19828    178.29613    177.69124    177.81836    176.71196    167.80483    170.11936    163.59599    165.49838     168.0206    169.55858    180.86228    194.98819    205.42777    202.60671    174.43175    161.02424    154.16553    154.73342    152.37636    153.42942    150.11942     151.2432    152.85835    146.97086    147.10206    151.40468     145.6496    142.54115    145.47742     140.4588    145.52857     139.0196    140.00833    137.90051    140.12511    136.66601    137.21089    134.11241    137.80281    136.89201    142.50035    151.72988    169.64378    177.24559    173.55766    157.52775    143.94324    135.55051    133.62074     131.9917    125.08804    130.80808    129.26984    125.93409    134.68505    142.17443    155.08792    159.11371    161.05037    148.49158    150.45799    152.10478    156.34139    154.67365    152.66313    158.16798    148.24735    140.81519    130.02278    125.54255    122.79971    122.20746    124.59979    120.91411    129.37963    134.50739    140.79923    149.06587    157.26702    165.03196    168.06298    165.36408    165.44052    165.98152    159.82616    148.23097    141.25028    131.26853    126.44004    123.48703    121.73393    117.90251    120.59095    119.93938    119.89626    121.15638    122.88597    117.04422    119.45132    118.44289    120.48197    122.51077    125.09669    128.15524    126.16295    122.54281    119.89607    117.07884    116.61894    125.58552    128.15178    133.60152    135.77293    129.37222    124.80046    119.99475    116.73847    117.51386    115.34991     117.4585    117.45313     125.9045    137.97765      147.845    159.88132    164.08235    158.63563     148.6145    142.82106    134.29601    129.68055    126.67049    126.19055    122.83289    119.48239     123.5153    116.37635     118.5592    115.00168    112.29835    115.66048     115.0576    117.40185    118.54503    119.11787    117.40115    116.29086    116.84525    119.88317    123.52917    124.74493     124.6101    126.22127    120.01549    125.91116    121.48433     122.7786    126.74049    126.80941    125.19547    122.50244    124.78185    126.59933    128.54322    128.85898    131.04108    127.07758    125.62081    121.96303     121.8399    119.23042    117.59124     116.7989    116.52183    115.35679    116.45285    120.12684    119.64004    120.81374    119.12806    123.99449    127.48656    127.07613    125.35098    129.35978    126.63058    127.24661     127.8059    126.06346     123.0971    122.40273    124.06131    122.93027    125.81546    125.92272    127.27688    128.03794    124.12893    122.08591    121.74831    121.53613    125.74389    120.86872    121.56606    124.02394     119.2743    123.56327    122.57165    127.64488    124.59568    126.82464    124.48305    131.39765    132.22124     128.3597    129.63573    131.29793    131.01708     129.9234    134.72735    133.71701    135.93411    134.65239    136.70597    131.57505    137.03301    139.22018    134.04056    131.46703     128.2456    131.94782    128.62114    130.87552     134.9792    135.58021    132.15323    132.09601    130.79857    135.39389     138.3129    134.85844    136.69238    137.47033    133.37215    135.12694    139.08862    138.13669    141.98272    147.50845    142.67824    143.47747    145.95201    144.83108    143.95066    139.62227    138.04365    135.62753    134.54101    134.14195    131.59885    134.13704    132.05895    133.01948    129.28543    137.52862    136.68146    134.89185    135.39311    136.70114    133.84608     134.4792    133.48859    133.01941    135.68106    132.61462    131.44123    130.56217    133.31814    130.54623    132.90971    128.51108    129.93171    127.56646    128.53193     130.6962    130.29608    127.59392    129.07937    128.27651    126.50642    130.18559    129.14657    131.91488    130.58099    132.12878    130.59428    135.20022    132.02033    131.77823     133.2754    134.53898    131.29005    132.68835    134.51194    130.47684    133.61352       131.83    134.27915    131.52414    134.80732    134.43869    132.94876    136.46238    132.88341    135.61274    137.60421     137.3128    140.57569    137.13491    141.60351      138.819    135.67438    148.77923    146.20553    144.94724    149.03398    143.96679    140.52359    141.92737    154.66009    141.61814    146.54124    147.72739    146.44987    150.64954    148.58142    154.88943    150.86115    152.86851    149.33843    154.80497    158.74777    161.66561    159.32224    161.51765    162.20429    167.70278    160.03444    162.72583    170.99869    163.72953     165.2588    165.63392    166.67126    159.06655    163.73186    160.72641    165.86529    166.64329    170.27363    170.57648    170.29422    174.39394    166.99035    168.02639    170.00877    172.79941    170.65194    174.37044    176.67205    175.65223    175.22931    173.38393     179.4349    181.36477     181.9338    180.03843    179.36768    180.92761    181.10656    180.07723    189.11892     187.8312    187.26347    185.57123    189.24815    190.85021    194.76842    200.65814    191.38384    189.01411    192.73496    192.24679    197.37564    200.28252     207.5109    202.41198    207.73878    209.55493    209.30176    215.03519    206.86989     211.2105     214.7505    216.88865    207.82589    209.14794    220.58215    217.56445    227.70314     218.1085     219.9535    224.26991    219.47882    227.84976    223.75606     218.3323     218.8829    245.15143    224.02008    229.16023    225.46559    230.90329    232.64571    230.57444    237.42745    233.57509    250.72216    240.61517    234.32034    236.53852    245.25719    239.14698      247.364    262.48323    259.17043    260.78535    266.73435    275.94258    286.05643    268.05089    256.64419    279.49543    270.62036    303.98516    265.11413    306.50057    307.49051    319.77755     269.3955    297.77074    334.25936    337.53492    330.21404    374.15239    340.65789    338.76092    380.95439    426.95056    432.93764    415.58053    388.25376    404.32554    397.45805    499.78241    475.03066    496.49591     445.5526    477.83748    574.92391    352.46089    640.22478    684.74638    619.11485    664.44465     663.0968          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        1.00    256.27323    242.05183    239.94294    233.45873    228.03855    222.14549     222.4183    207.03017    201.34816    196.61085    189.66834    191.19137    189.02477    181.92025     180.9579    178.45265    179.88237    177.17012    173.79797    167.93271    168.21252     161.9593    164.94591    168.00486    168.23735    175.68363    196.77782    209.10509    199.46627    175.96435    157.22361    154.31871     156.0032    152.25287    153.92682    150.04291     145.7485    148.71635    147.00167    145.99179    142.71911    144.10181    143.77203    144.17683    140.74558    143.05044    136.72145    138.50936    139.02678     139.2311    137.50098    139.44447    135.22985     135.0738    141.29499    139.82357    147.95019    170.34475    175.77684    175.25842    154.73904    142.90853    135.01661    131.62505    134.24098    126.66652    129.50365    131.00179     130.9385    136.57871    142.35655    157.12591    160.31097    160.69169    149.16616     147.2219    147.33295    150.68042    152.56953    153.44162    156.71907    150.60274    139.94716    132.51842    125.47847    120.43516    120.34466    121.13973    122.85652    125.37882    133.59072     144.9569    159.12909    171.83692    180.33547    173.85385    175.32756    163.74963    164.59747    154.67517    148.03893    137.77734    132.30141    127.06714    121.85276    121.70404    117.97739    118.16577    120.82333    122.08567    119.50024    119.41967    120.64612    115.95162    113.58632    120.32956     120.9945    125.51631    126.05809    122.82461    123.48197     122.5995     116.7945    115.97949    127.26649    129.96116    131.70855    135.04755    129.30527    125.69137    119.35236    118.41108    115.39893     116.9833    119.98408    119.69531     126.1905    139.20731     150.5746    160.78672    161.87972    155.74912    146.14264    140.67241    135.63523     129.5244    128.13621    126.01768     123.3812    123.69852     120.2258     117.0552    117.50003    116.64484    111.15385    115.44575    116.88519    115.11045    118.63868    120.42783    119.17065    118.38349    120.62248    118.11926    124.65733    124.15631    125.27546    126.68367    120.14397    124.39784    123.29328    123.21472    123.18432    120.20353    121.26847    122.20837    126.63633    125.23064    125.70912    126.51602    127.98688    123.13595    123.77748    120.31935     121.0256    117.45596    114.99682    117.08993    116.33454    118.69512    118.27097    118.51222    121.94669    124.80177    121.44611    128.47111    126.07802    127.76594    128.41633    126.54255    125.91754    128.43943    127.09809    123.60705    119.65667    122.77075    125.33673    121.46812    127.26946    123.83996    126.60475    126.99424    125.37073    121.64886    124.62493    123.40012    122.95557     123.1866     123.2053    118.78337    117.88835    120.58673    122.16062    122.56971    125.53944    127.02887    123.86542    128.87481    129.70936     129.1795    126.32629    128.63169    129.44852    127.42986    134.74108    133.34565    133.34802     133.3951    132.40238    131.51698    137.31564    132.53376    134.04303    134.87727    132.32265    130.50581    133.16176    132.89661    135.28234    135.28381    132.69101    131.18438    133.28909    135.39239    135.82673    134.20444    136.73521    133.42649    133.96351     137.6972    138.24983    137.38416    142.54044    143.47006    142.92331    144.10295    145.89664    146.64376    143.41297    143.22616    139.26594    132.64266    135.54998    135.19027    131.91272    130.91911    131.76313      133.552     131.9264    134.99855    135.10325    133.75158    133.09357    139.27547    138.42604    134.39989    134.70268    135.96775    139.58872    129.89192    130.67622    132.30057    130.35303    130.19521      131.438     130.5697    131.86745    125.91143    126.30949    130.24294    131.70275    130.84181    128.60835    124.91282    127.77868    129.56014    130.05776    132.10379    127.74743    132.51083    131.93694    130.26862    134.13655    131.86391    135.19593    132.83742    130.23229    132.62939    135.00894    134.26193    132.42681    131.87442      133.058    133.79906    136.52021    136.17737    136.26417    133.21444    129.59348    133.53803    137.27682    139.70242    136.11368    142.34745    141.19347    136.90133    142.29589    142.29505    144.29588    140.99947    143.54036    143.30873    143.00156    147.99268    147.82402     147.0346    148.80344    144.95712    152.42121    149.36997     150.9133     154.0591    154.36625    153.69065    156.72674    155.55427    153.38659     158.7541     160.8006    160.29283    161.13138    158.67761     165.0029    167.72775    161.14247    168.06314    162.33742    164.78991    160.49498    160.11182    161.01981    160.12301      164.484    169.48629    166.71921    166.77207    174.98803    175.50147    170.74269    172.86675    175.59988    171.51858    170.96887    177.52585    174.07856    175.77006    177.72705    180.10238    181.50443    180.89123    176.09711    181.47084    173.18833    180.98296    174.26704    180.71169    181.64692    188.56903    185.23398    186.56674    183.30647     189.3986     198.9244    196.06659    192.23858    185.15558    200.82347    202.10093    193.52049    213.73426    202.25421    202.87675    202.75793    203.90606    205.98838    205.41509    212.10676    212.26409    214.38625    208.32295     203.0797    225.49938    216.82422    219.89633    223.87381    209.73006    211.64676    226.80892    221.76539    226.87126    216.72221    220.79335    222.57703    240.90896     234.4187    237.96809     223.5048    225.87936    237.92386    226.01751    239.73872    239.42674    242.90917    241.45816    231.79987    233.95096    236.20397    268.08698      240.437    255.76515    230.02377    251.58656    248.12134    283.87148    271.28103    269.71719    280.99933    268.10952    281.46128     301.4749    297.50305    289.15749     321.7651    300.62767    294.56447    308.43149    339.49962     352.0633    313.23794    348.76777    317.43204    369.61665     369.0026    407.26529    420.67659    382.18983    359.23532    398.83216    417.31882    468.32031    503.06571    496.26924    493.13111    512.58964    553.91249    612.27625    623.30162       716.82     633.7144    563.35757    601.32018          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        1.50    256.14431    241.05208    238.30965    232.38126    227.50833     221.7763    221.62913    207.57018     202.5092     199.7028    191.32286    187.57632    190.01369    182.50364    182.18917    176.45119    181.24399     174.1995    169.94205    169.88243    165.35659    161.73095    166.38134    168.98455     165.0699    178.40531    196.86567    209.29975     196.9052    179.55922    156.03046    156.48203    158.09506     151.9719    153.25808    153.85587    148.90417    147.94676    147.87806    141.19445    145.93191    143.02148    141.69134    143.19321    139.30131    143.28628    139.14139    136.62087     135.8626    141.03879    140.73644    137.76563    137.11678    133.43957    134.58742    133.82267    150.40499    165.77692    178.19619    174.92451    154.70535    141.56765     135.9852    132.24078    133.56998    129.25869    126.69284    126.51773    131.45693    131.44115    142.66421    151.56854    160.92311    157.77088    147.98743    146.87957     148.5796    152.36616    149.75768    151.88661    152.50712    149.16833     138.3464    133.78822    126.61942     125.4689    123.38896    122.63229    119.12407    126.54303    133.31706    146.88984    166.54111    186.22491    184.67888     179.5356    170.85116    161.24035    162.81405    155.46085    145.62666    135.55818    131.30445    127.73851    121.82781    121.73451    121.98599    122.37007     120.2784    120.68245    119.32353    122.82385    123.51981    119.90937    118.16941    121.05888    124.46536    126.38239    128.17819    123.06362    122.95037    118.07385     119.7999    117.91431    121.96084    129.26356    134.66685    134.03348    129.96517    124.40481    119.28613    114.39255    120.01528    117.35689    119.95886    122.14507     126.9584    142.27414    150.48366    158.79047    160.30055    157.55555    144.48393    137.55955    132.16785    128.43327    127.68108    123.18295    124.87253    119.67531    122.32124    117.13766     115.9766    116.14999    114.26135    112.60327    118.13773    118.29382    118.92326    117.09583    118.71303    121.26701     115.3132    119.32656    122.72597    120.32913    128.19257    124.46523    120.25158    123.49343    122.69233    119.54161    120.65003    122.21175     122.6119    122.01886    125.89064     127.5467    124.85809    124.50475     123.7871    118.88025    122.66553    120.12765    112.80748     122.3892    119.45605    117.08494    116.14277    117.20782    120.16452    118.88872    120.37047    120.38447     122.3337    122.69932    125.93652     125.9819    127.63149    129.81158    125.56341    127.61903    129.37993    123.28434    121.84405    117.48801    121.57685      123.524    128.33859    126.35517    126.40438    130.10882    125.03157    124.28951     122.3132    120.65462    122.22991    122.02927    118.47206     124.2811    118.10927    118.80781    123.65255    123.82552     123.8696    122.20611    124.87694    127.05375     129.2978    128.67181    129.67753     130.8296    130.33205    129.61084    133.11775    131.52799    134.44343    133.38651    132.77516    129.11533    135.06166     136.9113      136.782    135.79694    137.71486    132.64959    128.57305     130.5565     132.6284    135.35598    131.91749    129.43046    131.33248    135.97083    135.70267    136.28419    135.89623    136.51213    133.56396    135.05952    138.16133    140.75562      145.713    147.66591    144.00785    145.69397    145.96138    144.56746     142.8296    143.00624    137.87306    134.47549    133.85671    131.18811    134.94005    133.88577    134.49858    135.47541    130.32694    135.91692    133.97352    134.35033    135.11512       133.95    134.78071      134.159    134.13984    135.02227    132.11085     135.1266    130.66769    128.11966    133.22937    131.53755    130.49498    128.16101    131.19986    126.11661    131.41552    129.65299     130.9516    129.39823    129.22002    127.74334    127.23131     125.5136    130.55419    131.38113    128.03845    126.32197    130.74195    130.43988    132.19565    129.93012     129.0185    135.97197    132.12591    133.13123    133.54059    131.22491    133.73459    134.91013    133.26453     134.6278    133.65776    135.16541    132.42281    133.19887    135.23359    131.79425    135.68407    136.57882    137.84561     139.3658    141.10762    141.78844    139.09625    143.96646    142.52673    145.49467    141.20493     140.9922    141.84742    145.97436    146.32531    150.53006      143.695    146.65883    147.67432    148.87544    151.31165    151.98307    154.40339    154.64491    153.05408    156.74293     158.2919    160.73574    160.26383    162.86043    159.53201    158.96843    165.51148     167.6688    160.57828     159.8957     163.4743    166.19317    158.04757    167.63294     167.8155    159.50993    161.83909    158.79977    163.76072     175.1306    167.09174    171.31844    170.48244    163.90333    170.57029    165.69973    173.27636     174.6604    178.23303     171.0009     176.4848    168.26678    171.25873     178.0004    179.49375    173.04782    183.51686    178.06144    182.06138      188.363    189.44289    183.78128    183.56587    190.63838    187.46704    198.03805    192.12098    207.07589    192.82028     195.7601    196.07682    195.98076    199.95582    207.30089    198.20744    201.39304    202.76196    205.97285    204.84692    220.98745    211.58638    212.82903    211.49056     199.8958    220.05155    218.99833    222.85883    218.24077    212.03881    218.11936    220.75975    217.23015    224.69478    221.79065    217.71621    218.54376    222.01788    234.67643    233.65375    230.07497    227.93852    243.79059    223.53252     236.1364    249.25967    239.54812    244.28238    225.83089    237.64657    225.53941    251.76002     267.3207    249.76139    254.84945    253.04059    258.10483    267.48892    275.22492    285.97494     259.3621    272.30423    266.77042    281.59743    316.27129    300.97777    300.62274    282.22762    299.52326    289.56958    296.84899     338.1921    325.52383    328.07828    353.32796    318.10014     345.1912    383.11909    390.30646    380.97036     385.1305    405.57529    401.60501    382.14762    509.32493    466.92572    494.82512    518.99288    511.29518    502.10153    601.28132    746.51669    546.20856    625.98205    760.02978    700.49849          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        2.00    257.11209    241.28959    235.16426    230.47619    226.69798    221.46309    220.88453    208.33078     203.0073    201.00992    193.19371    187.88864    190.97206    180.54206    181.13335    176.20336    176.74748     168.3674    172.97689    172.33095    164.08936    162.25598    169.19778    169.40929    164.06821    182.90456     196.8425    210.63323    196.19706    176.71915    158.21166     157.7694    157.96482    149.97635    148.81423    148.87834    149.70636    155.33142    146.46788    142.77284    145.22146    143.21821    141.41836    142.89423    142.24928    141.49241    139.54893    140.87601    139.22288    139.00047    142.36826    132.73764      135.471    136.19896    135.92781    141.61441    150.46851    164.08503    180.19368    177.34655     154.0302    142.12339    141.48634    135.26966     131.2536    125.01785    125.78435    132.16262    132.56582    134.12364    142.69599    154.75619    163.71355    161.97022     152.4854    146.68992    146.29526     153.3558    144.45066    143.19429    138.97886    143.18135    136.65192     131.6688     124.5778    123.55611    122.29698    125.43813    125.82463    128.71528    131.39153    146.39113    166.04281    181.94305    178.96646    171.01598    164.43302    157.85553    157.25333    152.08709    145.46625    137.59977    130.64074    128.24134     123.0549    122.67902    119.77259     121.2547     123.8656    124.31335      118.889    121.37969    119.96025    119.77877     117.4848    121.58121    123.44473     124.7794    130.41077    126.04076    121.35905    119.64076    118.62234    116.00379    125.62889    131.00577    131.37388    134.72979    131.83731    123.39857     121.3536    118.55429    119.08835    117.94728    117.64952    117.42749    126.66242    140.70998    151.56792    158.80005     161.4264    156.22797    144.30693    138.21644    130.18756    126.94306     125.4087    122.88477    122.13542    122.08043    120.38511    120.00896    117.44205     115.2989    115.50583    115.02621    119.29622    117.94911    119.18477    118.87391    118.31777     117.3886    119.83153    122.95834    126.09033    128.90733    125.12762     125.2937    124.07425    125.97885    123.49782    121.05879    122.43762    123.12379    119.15764     120.8515    125.88354    126.63737    128.54483    126.64379    123.89676    122.41807    120.01663    122.45948    119.06292    115.26342    114.85476    115.15753    116.22556    117.67719    119.86272    119.35134    119.57808    119.68282    121.72842    122.98505    123.37999    124.86827    125.92072    128.69807    127.16965    128.48355    124.42126    125.69168    124.18388    124.59614    124.31385    121.85328    123.49031    127.73102    123.60485    126.37974     128.5594    126.48148    126.27839    125.68935    119.65789    123.23201    125.79539     123.2856    116.54022    122.15396    123.05859    122.84879    123.41218    126.73328    124.77951     129.9983    129.43819    127.40525    130.28503     127.9371    129.11857     124.9268    128.10062    134.26227    133.43926     135.8262    136.61121     135.0394    139.32293    134.65635    134.89451    131.73285    133.20441    134.94011    131.64699    136.42007    131.69219    131.77399    135.70613    131.43266    129.12695    136.17055    139.15778    138.02861    136.87684    139.11875    134.91528    135.32346    137.97982    139.87009    140.96608    144.02231    146.73215    145.48233    152.20435    146.60977    143.16919     140.0203     136.4102    136.45193    135.47004    135.84601    134.21478    131.29536    134.00943    134.41686    130.37909    134.66339    137.02152     135.4489     135.2921    136.69771    130.90958    133.41671    133.46001    133.64035    131.30967    133.49037     130.4516    131.61321    131.87547    131.69632    132.16101    132.23603    127.58414    128.45248    132.58192    130.90174    128.54403    130.66754    129.45617    129.74097    124.83691    131.38471    135.24175    129.23413    130.88329    129.30205     129.0684    133.13902    132.28596    133.25046    132.22115    131.84202    133.38263    133.68366    134.88476     134.6741    131.95672    128.73585    131.35425    131.77485    136.23586    131.78482    135.36628    132.34163     131.4884    137.86875    135.15393    137.92447    137.80154    138.28411    139.26424    142.40016     138.2699    139.02224    143.68457    140.45271    138.72443    141.18807    144.90831     145.6865    147.89972    146.70194    148.83647    145.61143    145.31208    149.66299    149.07186    150.65979    151.10106    153.29911    150.01189    154.48502    159.74523    157.82944    160.40238    160.39825    161.67437     164.0743    162.64183    169.22485     161.1112    161.44879    158.05388    161.37182    163.25864    167.66395    166.81592    158.33377    165.27377    163.71449    166.05006    164.04201    170.14188     168.9951    169.11351    172.63895    174.71627    163.55708     174.4688    175.33965    176.09601    176.50883    172.84215    175.65233    178.02454    172.14766    171.47163     182.0182    174.65733    181.84161    182.96794    180.50425    193.44743    195.54776    186.67558    185.69239    192.69199    194.78198    193.47238    192.18562    198.84809    197.03692    191.82741    191.54689    195.02212    197.62186    207.10923    200.62005     204.5937    203.72849     213.5104    217.84997    215.81928    210.12756    207.68885    212.63938     209.9358    215.18708    214.30886    221.87692    214.78386    219.74932    215.32739    217.47734    222.63785    226.36679     223.4507    216.65566    216.58291     225.9118    238.76225    235.22724    223.77745    222.52706    230.93235    234.50893    249.40831    236.15786    236.60687    253.20762    244.89415    235.00766    233.75427     256.2932     262.1198     244.7761    245.74922    261.54075    264.02599    271.41368      262.667    256.75028     267.4411    272.59176    266.58508    299.93528    305.69754    297.12383    288.12865    286.79416    311.98536    308.96397    318.57442    361.65041    361.89932    340.42686    322.06755    343.81447    399.65064    378.25487    373.76965    363.71801    348.71922    379.89439    430.31601    437.08187    532.52023    432.23442    483.83949    468.05876    572.64725    618.81462    648.70903    692.96387    598.10297    588.72413    691.37143          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        2.50    258.27932    241.48824    233.01473    228.75815     226.7952    221.54934    221.10579    209.65396    201.74506    198.43639    195.46801    191.27723    192.32174    177.62212    179.13649    175.00878    174.28691    167.61411    173.11273    174.32628    171.03715     165.8185    164.27601    166.42615    172.80142    181.25614     196.6546    211.81236     199.2065    176.44248    159.31244    158.08139    155.62971    148.87651    146.17293    145.35002    144.33294    146.83203    146.97886    145.38783    142.51531    145.56354    141.44854    142.49744    146.11226    136.89894    135.29985    140.75383    136.67399    141.47656    137.29078    139.95891    135.17378    133.38807    133.97567    135.86485    147.57248    166.51891    177.36825    169.31828    159.74116    144.93675    138.15735    131.58949    131.66994    129.10019    129.63827    128.20257    127.89232    132.20795    146.31424    160.96137    161.17106    161.62233    152.34003    146.70845    148.37882    151.22113    145.33632    138.44966    139.10945    143.18502    135.11611    132.49096    127.22369     129.2901    125.57805    126.29959    124.17595    125.36296    133.37419    145.04273    160.24676    168.73835    168.01361    155.89171    152.99857    152.81712    159.25106    152.01285    143.40046    137.71147    133.43066    124.93196    122.38577    119.51754    120.38011    118.87852     121.7213     122.0438    119.75784    122.78637    120.43186    121.69213    119.35059    121.08281      121.308    127.90714     127.1827    127.78208    118.98791    123.53057    119.91638    120.34166    122.14751    130.17801    131.60575    135.18474    128.71838    120.31536    119.57474    117.56473    117.26853    115.31262    118.63119    122.99056    125.21883    140.39412    154.76691    162.46688    166.46122    158.80623    146.42851    139.67717    132.81424    128.88398    125.94438    127.69839    126.73415    118.94243    121.41045    121.08511    118.31922    118.78673    116.17792    116.63219    116.17464    114.65809    121.30839    119.81161    116.39918    121.29288    120.42922    119.91398    123.50855    126.29708    125.21704     125.8516    124.75357    127.68205    124.63989    121.88317    123.79964    125.01277    122.95279    125.81522     126.1867    127.37752    129.93779    126.21837    127.00038    120.02712    122.38171    118.05849    118.13616    117.00526    115.36249     113.9399    116.74056    121.05602    118.03837    119.20033     118.8043    123.31334    121.43649    124.41096    123.38914    127.57366    129.92659    128.58537    129.99538    128.90616    125.32165    123.69441    123.17332    126.40032    127.66978    122.38915    124.85141    126.60594     128.2557    127.57967    128.08362    124.65914    126.01668    125.97886    121.41645    119.30665    119.44291    121.47184    119.73856    125.11215    126.51482    120.27205    123.70981    124.75423    124.85648    126.19696    126.75975    130.26188    130.73699    128.44752    130.09127    127.91947    132.22843     132.8991    131.84388    129.82558    132.88114    131.96371    135.34073    131.17303    130.51931    132.66705    135.74169    131.74181    131.31603    138.55758    136.51357    133.22941    133.87689    136.03677     131.1245    134.02803    135.42531      135.712    134.17034    136.11671    136.06927    137.00851     139.4552    140.23523    140.54933    143.50222    147.21587    142.06001    145.54895    146.27563    144.71206    143.65118    138.85006    136.54011    134.14585    132.40293    133.02589    131.62011    135.29152    136.31065    130.14333    136.17158    136.34706    137.56403    140.35128    136.01631    137.45742    132.59422    134.50954    135.03618    131.76343    130.38113    130.52861    129.29642    131.79308    129.92639     127.8075    126.61435    134.97647    128.91376    131.62868    128.09491    130.38756    130.43792    132.59745    130.87178    126.88247    130.43692    134.43947    134.72913    127.52488    130.40693     131.4265    138.65589    126.86405    133.31135    133.74473    136.55937    135.30508    129.49388    136.26751    129.79114    129.10685    126.54655    131.08862    130.39086    135.80702    134.49145    134.24433    132.22039    133.50596    132.58426    139.70922    142.02623     137.5175     140.7155     137.8508    139.32514     135.0006    138.10886    145.67936    146.56141     141.0618    145.72679    144.44927    148.42859    151.12131     146.9095    142.25206    147.05686     142.7639    150.65143    146.80266    148.86112     148.5297    153.77239    154.55771    155.14888    159.40688    158.43498    160.19944    157.44306    162.37619    163.01006    156.84783    161.61194    166.67304    167.97386    164.31384    159.60316    163.47682    168.23964    164.81253    161.94278    163.78893    167.21496    162.27033    170.22982    172.18178    169.79635    165.27928    167.58664    170.76373    165.51413    181.00547    173.88878    172.94431    175.10709     174.7908    175.71399    175.66712    176.46901    185.95922    179.90963    176.25085    177.19327    181.54593     176.9362    186.87804    181.66401    185.68468    185.42655    187.38911    192.70899    189.63316    192.99074    193.53555    190.65085    195.01639    202.63694    203.96164    199.08476    194.43425    206.55354    206.85053     203.7576    208.36185    212.38762    209.59342    208.46035    214.26447    210.15481    211.91098    214.27129    215.66736    216.51319    216.62513    207.61263    215.27724     211.9587    232.68986    216.02283    216.18212    223.44335    216.90633    219.74917    231.96685    231.35566     218.7034    216.58546    222.82377    242.76893    237.86341    234.68878    244.25692    242.48804    231.27325    228.22014    246.56265    252.44523    257.31314    238.49597    239.98852    247.77341    266.62614      254.183    269.90869    263.23889     275.3912    272.53558    275.71567    313.60315    307.63635    291.85463     285.3092     294.2548    300.20403    298.45568    332.32161    331.92312    329.78429    351.00313     341.9321    318.42408    391.19666    394.40212    366.73307     406.2967    359.29517    384.94586     383.2003    459.97987    480.93186    411.07456    479.32042    482.24494    525.06586    691.59885    557.63097    487.24187    559.25844    743.51391    749.57786          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        3.00    258.87646    241.22966    235.90927    227.61941    227.73814    221.88421    221.30893    210.88389    200.92044    196.89624    195.13301    192.69948    192.53467    176.80666    175.13614    171.87146    174.14319     170.8073    173.65207    173.84679    172.12469    167.16228    164.48132    164.72996    164.38951    178.40925     194.9541    206.48693    204.14043     181.1477     162.7742     155.1916    154.64317    149.07082    152.53306    149.22092    148.65833    146.00121    146.86702     147.0358    146.19216    145.03028    139.20376    141.28505    138.82975    142.43369    142.72043    135.81103    136.22847    136.28113    135.49607    136.99243    132.83312    137.43078    135.38952    140.47663    150.86299    169.80421    177.92609    174.50742    158.08928    145.02792    136.95714    133.54125    133.87927    132.50649    130.74655    130.57558     131.4033    130.11728    143.14226    158.98194     163.9158    162.85998    155.54098     150.4086    144.19203    147.48816    140.16092    138.26133    137.11754    142.86942     140.4894    140.08871     128.2118    126.82783     123.0334    121.91166    124.22005    125.42164    134.57897    140.86167    147.72162    155.29636    148.96947    148.15178    146.44663    145.00988    153.31672    148.84901    148.58032    138.13603    132.53511    125.83481    122.64866    120.83622    121.90429    119.00407    116.65902    120.71445    119.49309    121.19237    121.74589    121.14237     116.2047    119.59132     124.2088    126.74993    124.89169     122.5536    118.19153    119.17582    117.61557    117.05468    128.60666    133.54411    129.75049    129.76401    129.57427    128.32221    118.64615    117.55292    117.36973    116.95952    118.22304    121.44261    128.57031    138.68387    147.70303    160.77057    162.82895     158.9991    146.11338    138.83639    132.28068    126.64109    124.84389    124.46946    124.15532    124.29594    123.56221    120.41571    117.50162     115.6183    115.53277    113.45479     116.5256    116.10719    119.25495    121.67324    118.41585    117.48307    118.85552    123.19996    119.54816    124.66073    123.42963    127.38856    125.31109    124.25332    124.23236    125.01633    128.84343    126.35546    123.86297    122.22617    129.01375    126.57991    123.02626    125.84617    122.66869    121.09641    118.89609    118.34482    116.72114    117.08296    117.81779    115.30289    118.00652    120.55447    119.97271     119.3642    121.88364    122.79046    121.43701    122.21855    125.01028    127.95301    126.40482    132.73888    124.99875     126.8215    125.37041    123.94094     124.5051    120.92156    122.93851    121.83576    122.95385    122.62349    129.23519    128.63052    124.89302    124.99734    125.20087    122.99311    123.35687    123.88834    120.07956    119.60237    121.50123    123.70339    124.12019    124.93056    124.46779    124.66592    121.28309    128.52303    129.43511    130.79563    128.14303    131.77249    130.22016    127.95447    129.70021    135.52776    134.42906    135.89542    136.58292     135.3996    136.21049    138.62003    136.49635    138.18312    134.17512    138.81876    134.84685    132.83006     133.3501    131.31892    133.55983    134.56459    130.99542    133.68203    136.20162    138.06115     132.5815    135.81175    132.79665    132.81199    138.43442    139.40761    142.43117    143.38325    147.50243    140.09542    148.60959    146.17235    145.00927    138.12528    137.70592     132.3091    134.03162    136.37975    133.74858     131.0658    132.35741    138.37226     135.0105    135.70603    138.27627    133.76617    134.87372    133.89298    135.74512    137.84966    134.96615    135.90116    137.78706     129.3738    132.58959    127.83683    128.65075    131.25732    134.50123    129.55782    131.29133    128.67609    127.46996    132.12399     129.8762    132.41863    129.01897    129.10974    123.02405    129.45229    126.57662    126.02883     129.6855    129.83416    130.15781    129.19288    130.01453    134.18717    131.89563    137.82109     135.7873    130.43521    133.96843    133.61676    133.46106    133.07549    132.32235    130.25051    133.75248    134.11024    127.80364    130.39135    134.35651    130.97158    131.93024    133.11976    134.95548    138.56056    135.54545    138.59896     140.1834    145.28158    140.20008    141.04708    141.03822    142.72977    145.56717    148.53109    149.72813    147.35037    142.03646    139.71398    146.76282     145.1746    153.09182    148.63259    149.27597    150.61585    153.14166    155.67042    155.82903    159.48692    160.15727    160.12691    157.87942     162.8382    157.61579    159.56223    161.09955    165.09447    159.53643    165.57565    161.15225    165.65152     162.2848    170.06601    170.07839     168.7367    169.24803    168.65852    164.78306    171.67012    175.88048    173.62527    166.29531    172.19726    175.94772    171.50154    168.80902    173.75764    175.82666    171.04785    174.87086    177.63516    174.96382    175.95584    177.64831    176.06939    182.36551    187.17811    188.18999    182.89206    190.27981    188.75761    184.11626    194.02942     187.8954    198.59805    198.53582    200.49792    190.31305    192.01534    201.18771    205.31601    204.65859    197.04306     198.9688    199.21593    209.41584    212.59168    206.32944    210.41395    206.75424    207.73871    212.67276    219.39532    219.58578    208.27101    208.92659    214.62158    209.26751    219.45776    226.48182      218.547      214.459    228.12655     223.2113    214.01246    226.99814    224.06605    221.04377     232.4298    226.03554    228.45344    240.22518    238.55166    235.59347    247.66273    235.71405     237.5537    251.55662    246.54661    260.00467    239.23508    256.89107    253.72147    241.18129    267.15785    270.48311    268.00633    275.88616    255.43369    264.55506    306.49175    277.33239    289.63027    266.76059    302.94731    308.88374    308.34607    330.29781    310.65081    360.87897    322.75251    336.78286    336.30996    367.61974    403.00903    392.24859     382.8351    362.40461    375.19499     426.6726    464.81765    500.90866    436.37092    492.92871    493.38403    576.51281    675.27783    678.28769    643.16538     592.1906    580.42662    645.69704          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        3.50    258.99786    241.10011    238.88206    226.49952    228.48188    223.51629    219.05895    209.23435       201.97    197.51872    192.37609    193.60648    191.95639    178.18749    176.55655    171.19151    176.50476     173.2199    179.32833    168.98939    166.03235    166.22866    166.18868    164.31244    164.90621    178.25336    196.64167    204.57532    194.97587     180.4377      166.847    153.20684    154.36728    150.63819    147.55432    148.87684    148.65092    142.76337    148.64798    145.63238    146.89922    139.88089    141.38896    146.61848    135.19062     143.2821    142.72518    137.31985    139.65007    139.62192     134.7781    133.74483    136.20426    137.93193    135.45845    141.12361    146.06229    168.36816    179.27065    173.18802    159.50588    139.52159    136.86466    136.01497    129.88726    130.90909     126.4374    131.73032    130.13669    132.95578    142.59146    158.68562    165.50678     158.7218    150.93885    147.80817    148.74236    149.34408     138.6154    136.23985    139.52839     142.8292    143.70647    139.02833    131.19809    129.92729    124.97434    126.01849    123.99898    126.06388    129.57368    136.75805    145.91523      148.747    149.16726    142.34872     140.2716    142.54465    149.28545    147.66472    142.16438    137.26964    130.04376    129.03234    118.15087    121.91964    121.02457     118.6523    118.19679    124.33737    119.87149    121.12867    122.40449    119.44297    119.82231    119.23219    124.31735     125.7746    127.25583    125.70816    120.69399    117.57149    117.61835    121.97933    124.83874    131.21278    134.10209    135.89693    128.97589    124.82614    121.88167    118.84971    118.25527    119.58746    115.42849    120.59303    125.80336    139.74723    149.42878    158.57064    165.54973    157.84948    149.50658    132.31027     133.3914    128.92977    124.50156    120.88249    123.67281    119.93433    122.02178    118.33306    116.23991    117.49481     115.3259    113.17828    117.52594    118.34862    118.71014    119.27087    119.16855    120.55429    120.39545    120.35339    121.97052    124.00473    125.26127    125.98568     125.8668    125.70244    124.29887    123.31488    127.27178    124.38259    123.19674    121.60147    126.40662    123.40317    126.66507    124.37978    124.09886    118.85497    118.67369    118.90355    118.97597    118.26362    115.00273    116.75451     115.8171    120.91989    120.65778    120.19465    120.44404    124.60494    122.52039    127.07907    126.20648    127.79882     128.2494    129.29757    129.76306    125.34431    125.01065    126.65546    126.02219    121.22279    122.71087     121.0739      127.168    127.03213    128.56047    122.74153    122.90124    126.69998    123.45663    123.57844    124.27366     121.9183    122.09769    119.50754    120.38727     123.5305    126.19659    125.70043    122.47495    126.28842    125.90143    127.31571    129.40337    130.43673     127.8922    129.06319    129.33338    127.08227    133.51341    134.69476    135.60297    141.38122    141.16087    138.72337    138.35766     139.8786    135.91151    137.63954    132.58381     132.5961    130.00761    131.62075    132.47424    133.86323    135.88952    133.76108     129.7604    135.38169    132.85696    136.32282    132.81707    136.09313    135.86167    137.42798     138.7338     142.8207    143.73158    143.60698    146.94694    145.23229    146.49163    144.89513    141.86363    143.84473    141.74053     138.3366    133.86775    135.07187     133.4656    133.13411     133.6216    136.25115    134.88343    135.34454    137.55481    133.09344    134.97768    139.82487     135.6099    134.17192    134.54854    131.78766    136.61941    131.10392    131.28712    127.80312      134.682    130.14879    131.00882    130.61019    132.96115    130.61563    129.38149    126.32984    131.08801    128.72057    131.59564    129.01537    129.84195    128.76071    130.04969    135.06814    131.79536    133.43261    129.11502    134.52685    132.54518    128.78831    131.21527    136.50734    132.57325    127.22981    133.08343    133.44614    133.95772    131.34554    134.88506    133.81907    133.62507    132.66001    140.29849     138.5276    131.46418     133.8507    135.59725    134.10654    137.20889    138.81243     138.0082    140.47519    139.40173    140.79499    141.81646     145.3147    143.40804    144.40031    144.06668    143.84302     148.2559    144.21371    146.21293    142.27233    141.55052    146.19514    160.34475    149.51635    151.16042    152.30444    154.56589    158.91344    159.37718    156.73392    158.22583    153.84804    155.41757    156.38825    155.76192    167.14457    161.39489     167.1998    163.52666    163.38009    161.76911    166.26704    163.66863    165.91952    160.07752    166.14103     163.2867    165.10421    166.83373    174.62505    174.44442    170.86127    172.79021    166.75856    175.01918    168.18525    164.15436    177.29346    172.50169    177.55948    183.56284    176.84939     184.1693    175.76147    184.82223    174.93107    183.14028    186.08204    185.98012    181.10263    188.78892    188.96909    182.10725    194.65879    190.80152    190.53278     194.7295    196.75813    195.07651     191.9948    193.25246    189.19887    198.98871    205.94216    194.98648    205.15797    209.19646    210.87593    212.42249    208.07739    211.89599    205.11299    215.85586    210.61566    213.10264    216.78621    217.87732    207.82987     207.4814    226.57453    228.44319     217.3058    215.28222      216.613    213.42559    226.76982    218.29421    233.35584     218.7709    228.58571    224.09107    238.91368    234.81458    237.92446    247.78714    229.26652    220.07666       235.83     252.5776    264.30152    242.18839    265.39665    255.09169    247.04354    253.47914    264.59729    259.82061    265.03283     257.2678    266.70289    263.39988    285.38708    289.97567    302.99197    297.20547    290.98684    304.65922    302.18094    321.22977    310.46413    326.18927    330.04147    324.06434    338.46298    374.52016    382.59081    405.70801    367.36436    411.48611    396.01864    415.41622     453.6534    499.10366    497.78771    471.69486     445.0496    542.15232    659.19287    619.86117    650.54771    674.54883    686.49201    560.68274          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        4.00    258.91607    240.70108    239.45863    228.02394    227.82482     226.2187    215.80687     207.5857    202.95325    196.49395    191.80984    188.46671    190.08265    183.66662    182.92616    171.39002    178.16855    176.34253    175.36018    166.50219    164.10442    162.52109    167.97358    165.01183    168.10336     178.8198    195.88564    208.73876    194.48598    172.52864    159.70985    153.72355    154.27857    153.16996    143.82887    153.77646    151.41701    149.86767    146.32016    143.59896    148.78839    143.20623    144.13015    146.82682    138.80018    137.48458    138.50494    139.71181     138.6129    139.66941    138.84295    135.12745    135.37412    135.44276    140.71227    141.66884    145.37411    169.69997    178.88271    167.00092     161.0714     142.9504    134.59317    132.35992    128.87062    127.90037    125.79923     128.7554    130.70399    130.02991    143.12479    155.58592    162.43026     161.9482     151.3679    148.51537     148.1981    147.51015    139.28788    139.93815    148.14028    147.21391    146.65891    139.98535    130.69286    127.56392    124.58595    119.69655    123.56608    126.58392    130.09149    136.79398    140.58312    144.52593    145.07312    147.24751    144.79447    141.50837    146.94472    144.29225    139.37807     136.2628     127.5269    127.35414     121.6541    122.74655    120.01847    120.72109    122.14994    120.58344    124.67928    119.02968    118.98986     119.2986    121.55126    120.41215    126.26702    124.25667    127.35274    122.37899    122.99594    119.54369    114.22653    119.07526    123.56401    131.32661    134.99469    136.21203    128.38308    124.07827    118.74252    120.66702    118.14071    117.96932    118.13841    119.39421    126.71738    140.86549    150.03926    157.41079    159.57524    155.39281    147.62239    132.29705    126.93136    124.67989     121.9872    121.48122    121.63952    120.21216    119.33851    117.98172     115.9268    117.20055     113.3505    116.42461    115.26566    116.86732    119.51581    119.36919    120.81426    119.38487    117.79515    122.09438    123.07931    123.55347    124.72459     126.0232     125.9509    124.22801     124.0906    127.98661    124.59783    123.53137    123.08877    121.11856    125.52229    124.90744    125.09332    127.55443    128.70283    122.29085    121.38185    121.08623     120.3829    119.46048    118.17861    117.10754    112.77535    119.39532    118.11789    119.86316    123.77203    123.59278    121.66224    124.12961    125.39203    126.23264    128.56169    129.53024    126.48571    125.50638    127.27508    121.19655    124.21724    123.00744    122.79272    125.10544    125.46684    125.72837    130.54106    124.00945    126.13523    124.50144    124.68012    123.39454     123.2826    124.73848    118.72594    123.13144    122.37675     125.2366    123.52428    122.41304    122.88639    127.80601    127.42965    126.68798    128.11685    131.60377    130.53708    134.06424    132.42306     129.9052    130.89324    135.45987    136.83101    138.86115    138.79736    140.37922    139.70526    136.99522    137.61994    135.80132    135.46421    133.48821    129.22805    131.50799    132.07067    131.18556    131.72854    132.39819    130.09079    133.72583    139.12775    132.77556    133.58346    134.67448    135.67619    132.67185    140.60502    141.37569    141.81848    143.45124    147.12946    142.14267    149.12511    148.22583    147.43953    141.10521    139.32081    138.91036    135.20454    132.51336    137.19874    133.15997    133.04409    131.72192    133.30919    135.51239    136.54576     134.9165    135.40873    135.31244    133.63691     134.2474    134.28838    131.47156    133.75953    132.97737    131.25036      129.159    133.63258    127.59723     129.3761    127.78301    131.06931    130.98655    133.18267    130.85853    132.17014    131.59232    127.36949    131.24399    128.12928    131.39219    129.11372    130.75112    124.76709     131.9437    128.77704    129.47328    134.13767    137.19346    132.22385    133.37506    130.40364    132.98854    132.22072    133.43696    130.01313    128.34121    133.65729     130.2725    134.97201    131.87291    128.33613    132.48139    136.35799    132.41875    135.82717    134.54684    135.54813    135.40955    141.96983    139.17215    145.73248    142.10166    145.78857    145.90182    143.71101    144.71224    140.23047    144.05507    145.95927    146.76882     146.8087    147.27771    143.14391    147.54388    153.65063    155.97587    153.80372    151.65031     153.5227    153.92204    159.55439    159.33613      156.718    159.92734    160.12847    163.22182    162.31345    161.83965    164.80739    165.13909    162.68529    158.89944    157.46606    170.90954    166.44739    161.47241    163.17948    157.58643    168.85722    162.79937    170.71207    172.14496     174.3337    170.61987    176.84281    168.98965    181.94378    172.34305    174.26303    171.47998    175.74985    165.32356    179.20362    173.85548    182.83954    180.31397    179.20285    185.51198    180.71199    184.18112    177.67306    182.45282    183.45592     186.0208    183.63908     192.4198    185.15525    192.54475    192.95332    195.07701    189.52092    194.13869    199.42571    197.92852    194.51451    192.70073    195.78648    195.61187    209.91469    208.40609    209.57142     213.4746    214.76562     207.3209    208.35895    225.68132    216.25287    212.23718    218.62957    211.83052    215.69985    217.44072     228.6408    226.07464    215.66533    220.06656    212.93052    231.51646    225.88105    224.54008    225.10427    223.66819    224.33978    229.00588    241.89158    244.52542    236.66781    239.89876    248.35674    245.10618    246.30707    252.39961     236.7813    253.13663    251.99746    244.21024    263.37965    260.65088    267.57103    254.92655    258.95334    267.48997    269.20575    286.37254    309.89537    280.32193    275.46194     286.4829    304.18461    316.75883    362.54512    333.84893    334.39746    336.84474    309.05404    342.32779    383.07139    387.75259    361.96599     407.5364    375.24167    377.00167    469.53678    494.83695    447.54793    444.98354    476.37374    448.26301     535.6185    712.36082    672.82631     677.4265    624.33271    645.29702     706.6508          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        4.50    260.38791    238.50116    238.04514    229.86156      227.203    226.41613    213.26213    206.26603    202.35515    192.26816    192.60939    182.25644    186.97138    185.24364    181.63281    175.17958    176.15952    180.25977    168.83685     165.8639     165.4013     162.8425    165.95244    167.31561    167.76271    179.27808    199.93102    205.78864    194.20954    171.86623    161.23292    151.24706     153.6797    152.46729    145.91632    152.27362    147.91457    149.87469    150.56012    146.64537    146.48338    142.76481    142.29406    143.13151    136.01398    142.14956    140.41022    140.37155    138.12038    142.12632    132.63348     137.0317    136.78286    134.54455    134.79322    144.58355    148.26296    171.85826    180.82308    171.15338    155.89598    143.00265    137.17372    133.85109    133.49673    130.40281    128.08579    128.61348    130.80139    136.38312    144.18587    156.84738    160.11228    160.52995    153.68761    146.62065     152.0071    147.02287    143.01042    137.63052    142.48624    146.24366    143.38638    139.08047    135.82703    125.17972    123.22094    125.42852     121.5539    125.25711    132.69418    133.87192    142.05939    145.84985    146.74112    148.18642     145.9207    142.28748    143.69005    142.63886     138.9741    132.89134    129.39007    128.13259    121.14743    119.94564    121.42293     121.7542      122.093    123.16496    122.54704    121.27898    116.97475    117.61698    119.62393    122.47166    126.72566    127.63893    128.69912    126.34005    121.27431    121.12977    119.72668     118.8003    122.41995    130.95678    133.15781    133.39445    132.04605     125.1239    119.93384    117.52501    120.07835    118.66073    117.85952    121.20706     126.0906    139.57113    147.62141    159.26673    159.26293    154.23646    147.37299    134.57531    131.46221    126.83441    123.30755    119.44621    122.97304    117.99094     120.1121    119.14985     116.1288    114.74063    117.62512    114.36097    115.96936    119.84886    120.11601    120.52308    118.83658    119.19477    121.73835    123.62818    120.89682    123.35727     126.0355    124.42419     128.0712    127.26176    129.25749    132.54914    130.63827    124.77031    126.40035    122.84074    125.40417    126.33995    126.75605    128.05859    127.43833    125.23397    122.37237    120.85598    117.90892    119.73518    119.05002      117.992    116.83963    118.73402    121.46794    118.69443    120.94907     120.8216    121.17054    123.87179    126.65579     127.9119    131.66323    130.60695     129.4828    127.06874    124.84405    125.26905     120.3576    124.01227    124.09259    123.88158    127.66021    128.38185     126.0494    124.58233    122.51052    126.43714    122.95233    121.10642    122.10511    120.66276    120.79228    123.57784    118.35771    123.65844     127.9471    122.70639    128.64205    124.95931    126.74741    127.58053    128.06896    129.53192    130.84417    132.11506    129.68683    129.63575    133.83194    132.85326    135.15292    138.56463     140.4402    136.88019    133.75617    135.09477    135.09061    134.26801     134.6713    133.56795    129.25755    129.72781    130.22277    131.66013    133.30495    133.90116    132.95951     134.6668    132.93284    130.78907    134.54298    137.57401    131.93255    135.47564    136.86603    141.95999    143.46537    144.48569    146.60129    146.04322    148.56951     146.7498     146.1749    141.38656    140.06377    135.27638    134.00115    134.14073    135.05952    132.13035    134.17002    134.99289    134.73079    136.12438    136.71522    137.66857    138.35024    137.97536     136.5379     136.1837    137.32023    133.66477     131.3636    133.64794    134.16854    132.28155    129.97349    133.11388    130.09787    128.37036    128.82339    127.61001    126.86243    128.01308    131.21203    132.13017    128.04418    126.85519    128.38117    130.45873    126.57723    133.45268    130.58426    130.23919    128.75298    132.38934    132.52357    131.34064    134.61965    137.02268    131.55836    131.11514     135.4133    133.04042    134.07689    133.43675    132.65682    131.73692    133.94957    131.56773    135.12131    130.02457    132.23487    135.77036    136.52007    136.53068    139.67999    137.03332    138.95789    142.10584    139.18518    142.20059     143.3486    145.53816    145.67358    145.63461     145.8811    146.54119    148.53258    146.92794    142.42032    146.69561    143.84792    151.65744    146.51348    155.44733    159.46784    157.68272    153.32426     157.9706    161.69808    158.29729    154.76589    158.22273    157.66015     157.1077    157.29261    161.25409    158.79145    162.79198    162.20261     162.1339    160.97759    170.48362    170.43217    165.61401    165.67186    165.02005    164.37299    167.73724    168.54322    176.83022    171.68193    169.21552     167.4943     170.6344    179.61264    182.44461     179.9331    173.15603    179.44245    175.54043    177.36465    172.64647    185.78596    179.22235    168.06496    175.88251    180.70978    176.25627    191.74377    186.46176    176.33943     186.8832    184.85882    196.64696     191.4472    196.84704    186.88857    189.16026    191.71513    196.76113     194.1348     198.3102    195.41887    204.76191    196.11398    194.15198    209.96806    205.24197    208.91952    215.70121    216.23365    216.76617    212.94693    213.87458    216.46745    223.72566    215.52943    215.11027    218.55705    220.19375    220.35653    223.44433     217.6332    211.51712    223.06406    223.47626    235.48144    226.03551     231.9842    229.27402    228.63166    229.15713    241.44112    234.83631    241.98786    235.82924    226.28192    234.57286    249.54436    247.00163    246.53062    252.38811    242.87317    248.47737    262.64808    260.92292    259.95577     265.0623    266.81343    263.40672    272.10003    302.65883    285.28587    282.31086    288.26143    293.44733    292.12234    288.68406    346.59186    326.11677    328.34831    308.55869    339.41991    351.97293    376.00997    371.30074    384.74959    393.13089    392.91031    383.90362    427.01235    469.24565    488.01386    487.02973    457.52605    437.28521     542.1273    714.31243    588.48319    587.29562    588.91193    647.54173    437.58465          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        5.00    262.20639    235.96652    236.42865    230.96786    226.58053    216.95301    211.14992     204.9125    201.86725    192.01528    193.83002    187.20304    182.01804    183.80446    179.44754    176.87802    177.64187    172.27923      177.172    168.00252    169.24328    166.05432    169.03911    166.71782    169.29253    180.32416    205.63781    203.77823    195.48352    173.18937    160.46286    150.24003    151.74657     152.3261    147.33981    151.14261    154.61049    148.84636    147.42488    145.73883    146.11448    142.19069    139.90687    140.64502    137.68737    141.83149    136.76236    136.47844    139.78079    140.68518    136.91995    138.12879    137.28189    133.44991    135.04774    137.82375    148.77829    167.77152    180.65577    172.93779    158.27661    146.45466    138.54355    133.29708     136.4762    127.34452    129.90902    126.18115      130.374    133.31101    144.21931    159.81063    163.51314    158.27317    150.78385    147.04892    147.93016    146.24658    137.74177    139.37761    143.53776    146.50326    145.49341    140.82485    131.17958    124.05711    124.52254    122.85592    124.83569    125.46907    133.74834    135.47884    140.67575    147.18883    151.03587    149.24798    147.46981    143.18145    142.29433    139.03723    134.54628    131.46722    126.67263    125.61607    123.45724    119.32398    116.76936    117.65627    120.71005    120.36395    120.20852    119.65286    119.64585    118.73023    116.98916     117.0975    123.78464    126.66858    126.58839    126.34976    124.14667    116.04652     119.2923    122.16505    121.41659    131.46575     133.2144    132.16748     130.5934     125.9768    116.40263    118.79804     116.1762    116.67074    117.12576    120.78996    127.83066     137.9539    149.84557    156.88369    160.85533    157.75222    146.36656    132.60541    131.68466    127.19093    123.36129     122.7732    122.35567    118.08759     122.1338    119.64456    117.25796    117.78415    115.27399    117.59629    119.15571    119.86305     119.0658    120.89583    123.20288    121.24901    115.90704    121.59907    122.76389    126.46747    124.72122    125.43735    127.16768    124.76528    128.12476    128.98088     129.8737    126.00025    124.41669    122.96453    125.19866     127.8857    128.55041    128.69981    123.47959    123.85451    119.26095    119.73881    122.87217    118.14859    115.71432    117.90531    118.92701    115.22549    120.41189    122.91859    120.73168    120.71904    122.99135    127.72479    127.03007    126.75913    130.44374    132.52523    126.72487    126.25126    126.69997    124.44699    124.18509    123.51904    122.01426    122.78292    125.21234    127.62103    129.23465    124.92065    125.37215    127.84874    118.85481    125.44811    121.89171    120.41126    124.64548    120.99908    118.29459    126.00608    126.68747    127.25229    126.44404    127.99408    128.99958    129.04746    131.42186    129.90467    130.64478    130.02406    129.73554    127.98574    129.28178    133.55364    131.74316    136.68719    132.16524    133.84581    131.94033    134.32106    137.51977    132.44148    133.79185    130.87339    128.13972    131.93405    133.91323    129.07672     133.4131    134.25316    131.84029    131.09305    134.46813    132.26177    134.95866    135.77223     137.7643    132.49612    141.87959    140.56993    141.78167    144.65313    147.41039    144.48439    146.28549     147.9281    145.63687     143.8474    136.10658    139.03362    137.50582    131.95613     139.2797    133.94662    134.60505    135.75095    132.43047    140.88888    139.79957    135.28994    137.89726     137.0669    138.22294    130.42777    137.05179     136.5821    135.40773    133.34015    133.06986    132.47428     131.9179    132.80187    130.47465    131.80519    133.19689    130.50578    129.38813    128.27801    132.29762    130.42859    131.16843     131.4764    128.38899    129.25197    131.01432    132.16172    129.11643    133.26438    130.46516    134.18888    133.25706    134.04398    135.22218    135.89468    132.54675    132.20725    132.27241    134.68483    133.18375    125.09408    130.98512    128.32852    128.78814    132.94854    130.29062    129.54677    132.88892    133.64223    132.91674    136.97829      137.516    136.89472    143.18068    141.03523    142.27026    147.90243     143.6621      144.509    142.44622    143.57976    144.48203    146.25653    139.78108    146.67286    149.35333     144.9032    150.29434    145.34435    150.68439    157.16166    154.75926    151.04995    156.59575    152.73206     156.3834    157.85951    160.29279    162.01205    159.86928    162.45831    159.64424    164.15721    154.67274    164.11095    165.58169    161.48137     162.4628     164.3005    163.20238    169.44533    160.84519     164.8223    161.59152    166.34853    169.37781    168.46653    168.39618    163.81841    167.30803    168.12834        182.3    166.72263    168.20493    168.68492    174.86849    170.80246    169.77726    169.36473    174.31099    175.10285    184.89172    177.31918    172.59158    189.28378    181.69241    179.05727     187.1411    179.20555    178.93781    184.09837    189.26844    200.55986    189.40981    193.96141    189.51803    192.17194    200.26744    201.60971    203.40293    196.83974    203.35186    198.87765    213.96837    212.09572    206.63579    212.91033    212.57281    213.89404    206.89058    220.03816     209.7785    219.08185    210.29616    217.40066    207.26765    217.91945    223.69991    212.42034    210.64153    214.29138     208.1929    232.47346    223.22382    224.38102    222.40697    229.37127    222.93383    236.04024    234.78355    227.47209    234.74282      253.214    238.96678    229.55638     249.6121    243.10288    261.97483     244.8881    243.10266    241.23932    245.03405    272.00053    266.39395     253.7374    273.49206    258.18083    275.92611     280.3505    303.44766    290.01073    309.72871    282.13955    269.93858    294.95192     342.8681    326.42703    321.10646    303.83184    327.16829    333.90258    416.26679    348.03198    330.00808    381.83443    355.48592    369.06968    424.32963    489.86958    427.98345    462.83313    475.67015    430.01039    588.49335    673.75925    689.30183    608.64482    724.87843    553.97773    564.88659          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        5.50    264.01315    236.17333     236.1679    231.13033    225.75633    207.42237     210.2627    206.10923    201.69243    198.06382    195.00116    191.74526    183.20755    185.41242    181.45551    175.32884    180.22211    169.38434    175.53979    169.10017    172.31931    171.40914    172.35899    165.20457    168.95803    184.36818    195.08723    207.38914    195.78075    174.30006    160.71833    159.74467    154.48826    151.45666    147.01689    146.38234    148.53393    148.87314    147.06682    142.01302    142.66159    141.03644    137.58344    140.81426    140.17582    142.40106    133.88131    137.72238    140.83117    138.61119    138.01522    136.61053    141.36988    135.98003    135.27143    136.96685     149.4029    166.53281    181.93567    174.74776    160.01286    143.45661    135.56031    134.18312    132.82482    131.21824    128.06306    132.04156    132.15093    135.66397    143.53915    158.66577    162.46564    157.74046    148.65242    146.10486    149.22381     152.2925    147.16472    146.99314    149.76206    142.78415    147.47738    141.68642    133.97629    126.16954    125.97855    125.92066      122.278    126.69247    132.14603     131.6299    143.16506    146.66254    151.81138    156.07337    150.41098    140.54259    140.27921    133.59461     134.2435    133.77861    127.02644    128.01051    121.67673    121.12073    118.94171    119.85783      119.588    122.22505    122.74708      118.724    119.06882    118.39008    114.70776    122.96378    124.03471    125.28862    127.45196    125.49999    121.23115    120.02411    117.67319    120.13776    124.45435    130.74092    138.36436    135.72005    132.11868    128.39527    119.23794    114.77326    114.21211    117.44509    119.36917    120.10936    124.97618    136.26727    150.45247     157.4534    160.00343    156.44551    146.12343    137.42107    130.10178    126.90719     123.0518    123.05184    123.08301    117.64854    121.38104    120.38663    117.02618    117.86528    117.65422    115.09862    116.88836    119.84639     119.9504    119.73893    121.44271    122.76859    118.79162    119.84444     121.2922     125.9191    127.83258    130.64447    128.08037     121.6852    124.72688    127.68034    126.20302    126.27423    126.88565    126.60818    126.64505    129.04375    130.10075    127.51753    126.25058    119.60976    121.48052    119.54682    119.18223    117.71012    115.35546    118.59488    113.80859    124.70479    121.96435    120.16282    124.28133    124.71709    119.42357    123.41262    123.93576    128.21576    129.22072    130.03695    133.26937    128.36924    128.35154    130.55236    123.24302     121.0817    126.72673    123.41661    128.53759    126.39905    127.56768    127.78608    125.89276    121.04286     122.7316    121.99097    124.40155    123.81276    122.77162     121.7785    118.93917    127.79275    127.15833    124.96991    129.09386    126.16186     127.1062    122.36295    132.42413    129.72492    129.30946    129.47045    130.14168    133.33969    130.34429    132.21931    132.09766    134.53227    134.32202    136.67151    133.02253    130.80061    134.81244    135.68209    126.24183     131.8214    130.18086    131.17348    130.74354     133.7895    134.64624    132.62727    131.38461    133.72325    133.59007    134.36267    137.73585    136.60904    138.86111    132.54336    142.12734    141.66684    141.07535    145.47749    145.71237     144.5091    146.30063    143.17099    141.51252    141.88988    138.72026    140.71779    134.61454    136.56354    136.37412     134.6947    130.07059    134.03514    136.36537    139.49072    135.09575    138.85271    136.24816    138.65873    136.46607    131.23073    134.99271    133.23042    128.96742    130.51685    133.35416    131.63275    134.47537    132.92438     132.5288    138.83166    130.19312    129.25666    128.69249    128.93795    126.98414    129.12858    126.34783     130.7562    127.86391    130.74997     130.0468    129.92757    131.63905    132.34113    127.74237    132.51962    133.78592    130.32345    131.04548    129.46895    134.41488    132.66823    131.96571    132.79671    130.86182    130.69118    132.09682    129.01958    130.51879    132.58928    126.47442    133.84458    138.69301    132.27031    132.48577    139.26149    137.63838    136.02325    138.72403    140.71076    141.69602    141.58912    138.57572    144.28441    144.90604    149.15328    144.14566    142.66259    147.99302    147.84519    149.45266    148.76311    149.73263    150.08985    153.19775    152.28028    152.37813    151.25759    151.93666    152.33194    155.88648    157.65429    159.74376    163.33446    156.96834    156.61547    161.16716    158.70291    160.31664    159.41041    165.12037    164.86757     161.4331    166.92164    162.40885    165.61586     161.5842    162.28498    161.30681    163.25104    168.18737    163.34084    165.45882    168.28552    169.01286    172.80807    169.08127    170.91866    174.31513    173.04823    176.17337    170.59979    171.40714    176.27102     174.3007    175.71323    181.87764    171.90629    182.77446    176.91033    184.84204    178.24396    185.36985    186.41993    188.51767    191.56656    189.25155    179.09894      196.672    196.26298       193.77    193.16517     196.0741    197.11367    196.96599    197.49235    200.63772    204.01705    209.47815    216.26864    207.89725    206.03759    212.14852    205.96227     204.2132      219.977    211.68018    214.50585    214.93536    212.81964    208.92151    212.46053    221.17461    215.30882    221.53528    208.45168    212.96537    230.12862    222.41556    231.50906    229.18411    232.23882    223.61298    231.46629    236.77391    233.43183    240.28605    232.92119     237.0324    229.60152    249.52593    240.53388    247.11518    248.94324    249.30469    243.87345    246.56268    255.88012     271.2566    254.00424    262.53121    251.25105    264.50173    284.06633    293.23094     272.9288     292.9671    298.32045    280.90049    275.72081    356.88674    339.61981    302.70795    330.11049    327.78624    348.68486      351.951    417.19969    378.14631    346.73303    406.63324    358.41279    423.98601     443.1286    465.58414    427.01664     465.8113    418.68543    536.76096    625.53965    525.22173    589.65397    578.58503    637.75462    591.40775          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        6.00    266.54212     239.5773    236.02854     231.4647    226.28199    208.30387    213.23122    208.50377    201.05931    201.98353    194.09639    193.38985    189.36346    187.52844    183.70825    177.03484    179.86656    174.82925    170.74605    172.09104    170.03201    162.69951    168.91415    162.67135    166.74776    182.60154    196.78214     207.4383    195.76038    175.42574    160.16922    158.47856     149.8796    148.70686     148.0497     144.7858    148.01219    149.76146    145.20382    144.56814    141.49115    140.35672    144.57062    141.87312     140.5658    139.17172    135.99014    141.72682    140.45518    137.92514    138.08431    139.67263    136.15656    138.07426    135.10846    141.12859    153.50394    170.60373    179.11664    174.87105    159.69411    142.84346    135.35607    134.54952    130.93758    130.19601    130.72574    131.22322    132.30089    135.34921    144.55055    157.84977    161.36012    160.03119    150.08074    147.30206    152.11617    155.44337    152.80221    152.20269    147.78022    149.86706    147.42619    142.87932    138.38801    127.11444    124.16936    125.84159    123.66016    128.25835    128.37365    132.36807    140.41182    145.97136    152.73082    153.83199    148.00441    141.01823     136.9177    135.31377    134.80508    131.68219    123.07281    124.83272    120.63463    123.84163    115.67011    121.78969    116.15327    121.68342    121.73973    120.50402    122.11423    115.80979    116.98991    120.23916    123.58329    124.92221    126.13381    123.30715    122.46553    118.80118     117.0359    124.36729    122.50515    132.58304    134.80007    134.32432    130.81908    126.61718    120.69295      117.162    116.03296    120.95795    121.54629    125.28332    125.57337    136.85957    150.11385    156.62604    163.21212    154.68921    148.98672    136.76054    127.57978    123.99775    121.45008     124.4297    120.64925    120.19699    119.78184    118.84414    119.91887    117.15994    117.47853    117.17854    116.21788    120.24745    118.05197     121.8375    120.57597    121.50721    118.70367    122.20451    123.23662    125.20115    124.05711    124.11264    128.00349    122.75866     125.6211    125.76695     127.4611    130.87052    129.44779    130.28426    129.77696    130.77993    129.42739    127.75282     122.1599    119.54088    118.46855    118.28127    118.00119    118.90543    120.32537    117.84719    115.55983     120.0577      121.428    121.96379    122.69855    124.06415    123.74011    123.61711     129.0847    128.66491    128.71655    129.51487    130.33652    125.40205    125.79244     128.8388    122.07227    122.68858    123.04362    124.83387    122.06402    127.25127    126.69916    126.30975    125.18726    122.33098    119.30822    124.04676     123.2081    121.41968    120.44055    122.16803    123.95091    130.22293    124.29366    124.97665    128.26977     128.2102    128.54632    130.03991    129.90869    131.68132    131.85166    130.12318    129.01547    132.35842    130.37538    132.26051    131.11501    136.42429     133.2245    132.90544    134.68733    133.09752    134.19142    134.44817    132.88689    133.46819    129.59968    130.52492    130.71899    132.13027    131.56098    129.09905     128.7048    132.22103    132.60051    136.54291    134.14532    135.82045    136.59211    135.29382    137.12137    145.09669    143.47399    146.85908    144.47487     147.5498    144.74468    151.18159    146.02126    143.13588    142.86184    138.38896      136.308    134.06511    133.37367    137.10389    140.24448    135.93672    134.61476    136.88561    136.87061     139.4048    138.06538    135.34163    136.01596     134.1463    136.99226    139.61174     130.9462     131.9745    133.09271    128.62475     131.2151    133.09787    132.64267    130.76298      131.632    130.70274    124.60149    128.94102    131.32453    133.15434    128.65206    131.05989    127.35862    138.61189    130.02134    129.51314    133.65733    126.91962    128.83138    130.39129    129.23923    131.80765    131.72597    135.91883    133.03903    127.16767    129.60162    131.92834    131.98505    132.17732    130.30606    130.65207    130.46067    132.72315    132.74561    130.38264    132.52165    131.82928    135.73462    141.10598    143.41952    133.01833    138.15729     137.7854     135.9991    142.57689    145.66878    141.83315    142.44098      143.625    143.97104    147.21136    148.93189    151.30284    151.57493    147.73788    147.58404    155.73525    156.80181    158.44661    153.24882    153.89936    156.33527    152.74268    157.62142    158.64246    159.28997    159.32817    159.75264    155.84677    158.75043    165.56003    163.49998    164.81733    158.02488    160.58111    159.88491    167.98991    166.28469    163.70962     161.5268    166.30812    161.33736    166.82062     172.7708    167.28856    171.26979    167.74918    164.72034      170.112    169.43551    171.51648    170.94348    178.31404     170.5664    169.10213    182.62542    180.77337    180.77315    183.95784     180.1879    174.65481    172.09702    183.62472    184.34226    192.00329    181.04378    187.79938    190.04224    192.55307    194.34761    190.05812    192.47833    186.87227    187.56406    204.05927    196.78199    200.30306    192.97219    199.39183    195.84758    194.86871    205.74856    207.05527    203.78474    207.31347    208.20805    198.43007    206.41376    211.71611    206.83606    223.91401    213.58512     213.2844    209.68548     216.0626    227.46626    215.20118    213.73932     217.6648    216.32645    232.80724    222.49535    220.23177    235.97212    216.89777     225.2435    217.86429    231.44357    235.92449    235.89466    238.51037    240.23626    225.28512    237.04212    242.41328    246.98658     246.7703    243.60701    239.17813    245.96969    266.62457      270.356    260.88988    264.79243     262.7506    274.15865    298.15657    277.63203    267.04899    297.78581    279.14037    306.61515     316.2755    354.46421    300.33487    358.20414    320.96051    338.51915    308.26892    376.30931    341.28694    365.21447    349.74931    354.75919    337.59757    447.59696    440.91569    475.11589    423.54376    461.90799    433.08358    660.02202    699.13094    531.76231    589.22137    572.73659    538.89066     641.3789          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        6.50    269.30927     243.1678    237.36277    230.43133    228.72634    211.33077    216.50224    210.62941    198.87782    202.17699    192.64131    194.29929    193.28437    186.68557    185.85206     178.5995    178.79922    178.23809    167.77046    176.47394    167.65294     162.0639    165.81619    158.08328    165.14494    180.50305     199.6415    205.27188    196.48227    175.16846    157.98591    151.73013    152.66713    153.98538    148.46342    147.17917    146.91576    146.63744    145.16848    144.03316    143.19718     140.4259    138.76217    141.97344    138.28594    138.50265    138.52047    137.01667    140.46627    140.87521    139.49571    139.23544    135.66876    137.51556    138.93109     140.1692    148.88635    168.57149    182.07128    176.25625    158.55042    146.19167    134.34849    132.81359    131.94994    131.46373    132.61388    131.76732    133.33924    136.12275    144.87903    154.47415    158.39622    160.58434    149.76963     150.6709    153.43941    153.99391    152.32202    152.74808    145.94693    147.41579    144.67351     142.6583    135.28946    128.61963    128.65296    125.45648    122.44239    126.93211    126.23498     133.9677    141.57846    146.03916    150.39889    149.49416    146.28287    141.65085    133.94579    131.77746    129.74717    131.25936    130.95744     127.3046    123.47992    121.56933    121.12522    117.95379    120.36381    120.66382    122.24783    118.38727    118.54069    118.90537    121.27718    119.49421    125.24482    126.32571    129.94002    126.51818     122.5803    119.32363    118.25173    122.16142     123.4364    131.87603    136.12513    135.51055    127.80383    125.27105    118.46401    115.12285    118.70518    119.98647     118.6309    124.09966    125.31722    135.58955    149.71816    158.54821    162.53981    159.64295    147.67879    137.21486    131.93641    127.24059    123.89025    122.94423     123.1381    121.22184    119.95651     118.3673    121.39999    116.90837    114.90133    118.71945    114.06466    118.47799    115.68001    120.78376    120.47762    119.41335     121.5763    125.07325    124.10733    122.91033    125.97528    127.42705    124.29846    124.36272    127.01994    128.63787    129.04055    128.14311    133.49161    130.17573    129.64167    128.62551    128.66148    125.12074    124.51855    123.55451    118.22764    122.09999    117.72053    116.15511     120.3779     118.7542    116.99243     119.1695    120.73592    120.64737    122.18646    124.85864    125.46271    122.75197    127.04787    127.13028    130.03074    130.42259    131.99864    128.84169    127.07645    124.30057    125.93071    124.46737    126.17604    126.51538    126.81863    127.21925    125.32199      125.051    125.65914    124.84506    124.30618     126.4273    120.63383    122.32053    122.16773    123.41455    126.09243    123.09158    126.96745    126.63108    128.34574    131.27344    130.38081    129.40801    133.05911    132.03252    131.75803    132.72349    131.82055    134.26128    133.05288    131.76729    133.21573    135.04741    132.02008    135.11409    132.41767     135.1519     132.9781    133.05423    133.02712     134.1606    129.20851    130.93457    128.61687    135.19551    129.38486    129.26525    131.08314    129.63546    135.44477    135.38639    135.61573    137.92474    139.31698    133.13309    137.08976    144.12588    144.25148    146.78428    146.94138     141.6208    144.96455    147.76561     147.3636    146.27234      142.856    138.37672    133.45842    135.16047    137.69885    133.71128    133.83419    134.40287    135.13319    134.93837    135.09126    134.57546    138.05132    136.74228    137.10736    133.59114    137.35484    135.51168    132.08154     135.1331    136.58598    130.42952    131.62087    133.59518    130.63758    132.10972    129.96968     130.4495    130.86286    128.06692     132.4306     132.4482     130.9807    134.75089    130.91547    130.32713    132.98504    130.61341     127.4557     129.2225    126.88973     130.1908    135.53584    134.42948    135.65746    132.60033    134.14316    128.21526    131.86061     135.3596    132.36776     128.2551    132.70239    126.48901    134.77825    132.94202    134.56836    132.45592    131.14268     134.6272    136.91041    137.09487    135.97091     134.4998    139.65449    135.64377    141.43963    140.15266     144.8372    144.23196    138.94843    141.22837    144.81981    147.22689    145.34254    148.78319    144.42339    145.91572    148.71293    151.44624    154.67941    156.00466    155.47463    151.25658    152.15454    155.17787    155.93172    155.88952    160.13391    155.62476    161.65425    156.36399    158.68697    164.41546    157.69849    165.83361    159.63464    160.29781    155.03411    163.57835    162.22101    168.94079    160.78181    167.07999    160.65356    167.41177    170.37906     168.7632    172.43391    168.70411    166.60254    169.67878     168.6492    168.45759    178.84386    174.05872    173.83683    173.05925    174.00853    181.37882    175.59525    175.02829    177.26783    173.73193    175.88922     175.5371    185.81741    189.29956    188.00196    182.96253    180.76065    184.30035    197.51309    190.48863    189.50768    184.80895    194.61121    190.16416    197.83581    205.91231    194.30172    195.81812    195.53843    198.14686    210.48194    203.57994    206.94852    208.62563    211.35111    203.30672    201.21079    219.94021    210.12199     206.9289    210.50364    212.06556    212.68408    226.63109    215.38218    221.22022    212.94079    216.80169    214.51899    225.64124     224.0173    221.02393    228.58943    223.25511    226.31938    229.97436    245.97168    225.96818    240.46913    232.36075    236.44293    228.37828    255.16886    249.00435    245.78435    240.83504    253.13002    246.26646    251.13886    258.89212    265.83938    258.76829     267.9994    268.69787     271.6752    289.99248    292.96509     282.3067    299.63209    291.04832    295.18199    301.53872    320.72965     326.3188    335.68101    341.07989     275.4945    344.24945    390.90393    378.46644    411.31562     353.7422    373.21466    374.87303    422.92189    502.58307    427.25176    416.09064    419.96978    486.14069     606.6912    531.47073    600.50191    636.58712     545.4889    580.67107          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        7.00    271.41564    242.89334    240.88767    229.28101    230.52024    213.84465    218.84226     211.8639    197.16316    199.60287    193.02099    196.29669    196.08668    184.66725    182.26009    177.94869    176.50177    179.98951    169.83483    174.67769    169.10541    164.77353    165.25769    161.58581    168.79734    179.11746    199.69107    205.84384    190.87466    174.70484    155.93012    150.78515    157.32701    149.48532     147.6202    147.57842    148.95704    145.43183      145.635    142.82805    147.76297    138.62585    144.79027     141.0204    140.26698    139.02528    139.36808    138.18069    136.88514    140.62314    142.18577    135.84256    136.91576    135.00814    136.78594    140.80198    149.04701    169.35491    181.55011    177.89657    161.15416    144.97704    137.72252    134.16419    133.38095    132.51953    131.05378    132.81673    133.80935    134.06069    147.90573    160.61285    164.86447    161.78483    157.70416    152.58106    154.67731    156.50502    146.29083    144.98413     149.4974    147.03543    147.09478    153.98872    142.68576    128.91093    126.77432    127.10063    123.72813     128.0655    130.52544    137.51632    139.44053     143.1197    145.88782    143.99656    143.26416    139.81032    134.61431    133.88364    130.11226     130.5007    127.90804    126.53122    120.26725     117.9793    117.82734    118.03654    122.26187     119.3342    124.22491    118.29972    120.84242    120.43913    117.84507    121.39988      125.857    126.58622     130.6318    125.44593    126.04814    119.11063     120.1905    122.22747    125.48915    134.20673    133.23371     132.8691    128.98216    128.90376    121.21224    117.91907    117.71889    120.88014     119.6067    122.63201    128.21556     137.7942    149.23431    158.67314      162.934    157.32341    147.06522    137.23373    130.50596    127.29632    123.02705    122.06277    125.32356    119.70075    118.74844    121.28515    113.03202    119.01038    117.01673    118.41145    116.48687    120.69611     118.3122    120.41587    120.46437    123.01914     121.6514    120.87141    125.94883    123.13245    127.24304    126.87301    127.18539    121.36126    125.23574     125.5091    125.72603    124.14715    124.02255    127.37634    123.72889    126.75582    127.89467     126.7084    124.94701    123.84065    125.16107    121.65762    118.74183    119.96139    121.47212    121.62461    117.39013    118.20339    116.47807    123.55602    123.27135    125.80023    121.77596    126.48391    126.40384    130.69251    129.75856    128.42896    129.98386    127.73258    127.10867     124.9485    124.75505    124.40131    126.20682    123.72107    121.38883    125.40107    129.19042    129.10542    127.75258     128.4313    124.68472    126.17221    122.94859    117.93711    123.48926    120.68837    126.01494    123.82289    123.97825    123.99725    124.49284    129.62041    132.12826    127.68991    131.98617    130.13611    131.71675    131.79319    129.64835    130.79615    129.83078    131.75394    132.26114    132.99235    134.61565    134.01579    130.22859    135.49652    135.35779    135.99622    135.49764    136.09978    135.76162     128.7472     132.7143    132.07707     130.9804     131.7104    129.44872    132.97787    135.84457    135.69221    138.68203    136.25806    136.19575    134.94884     138.3388    138.56561    143.39481    146.01109    143.30476    146.24263    143.94517    149.90842    146.98462    144.28035     141.5324    141.59167    136.49224    139.40582    137.31733    133.52409    136.91106    137.41347    135.22542    134.71947    137.50201    136.20442    134.91195    136.39064    137.43052     133.2022    136.91169    135.05965    134.11291    131.32151    135.62624    130.80283    129.35792    132.57003    131.84388    133.00312    130.81937    130.85334    125.55521    130.79319    131.71032    128.13942    131.49761    130.69082    129.19819    129.66446     130.7145    129.20058      128.362    134.02253    129.40257    132.80704    132.28683    132.61408    133.57638    136.51593    135.61324    134.57057    129.97686     134.2355    133.98032    128.46561    128.91518    132.89696    132.20809    131.03163    131.89958     130.3865    132.46846    132.87556     134.9343    136.28892    133.93772    139.55779    133.87584    139.24568     144.7777    144.03886    146.86529    144.81796    149.65249    144.30737    141.21397    140.91731    149.03208     146.8964    144.18927    144.46907    145.55134    147.46068    152.20819    154.37749    154.77757     154.8005    151.94701    156.64704    151.33331    161.84269    159.55105    159.18313    157.43066    156.92168    156.93723    166.23718    160.07802    157.99806    161.04519    156.48148    154.81402    167.03591    164.08678    166.29994    171.37995    170.11602    163.72518    162.94033    167.55467    167.84553    166.90323    168.64027    165.97969    168.50266     172.5258    179.24642    170.24465    168.94783    173.08847    174.81757    178.63406    175.26506    175.17277    175.31393    175.62084     175.0432    172.25619     175.3265    178.14588    173.83931    186.13403    180.44491    185.52989    190.83486    189.70168    188.83277    192.50816    186.42869    187.91646    192.37283    194.70696    211.32622    195.44252    197.81315    197.07782    200.58543    203.89542    204.06962     200.1092    196.34371    207.99698    206.51239    214.06877    218.37707    217.61741     216.6457    215.09497    216.19934    210.48637    217.89432    223.91572    222.86073    212.53284    206.51928    220.06816    216.93328       225.64    222.36594    219.78091    234.71764    222.87615    231.82695    238.58556     229.5137    234.27646    224.38591    232.70009    230.43151    256.04222    244.68321    240.24635    240.63753    244.66659    241.34372    258.53446    260.74769    261.51524    273.70161    257.71519    271.15613    264.59124    302.84318    311.09843    290.53258    286.41068    293.03262    294.83088    347.97801     321.5127    338.60578    336.67095    316.44511    295.02225    296.61107    380.94683    351.27912    389.17424     365.6949    373.39998    402.16798    429.09992    518.40988    451.69044    442.37597    472.69408    470.64543    557.69151    644.93944    649.24494    517.21149    677.56998    685.99422          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        7.50    271.47858    241.54634    243.96531    228.60591    229.94218     215.7079    220.54354    210.47406     199.6994    195.46257    193.21312    196.27087    189.93829    181.51386    177.92457    177.93797    174.86552    180.90567    174.29616    173.34731     168.5639    164.65985    165.08295    162.85421    168.30514    176.43496    197.32655    207.90877    194.67018    177.04258    158.32504    154.23398    153.82242    146.98964    148.79323    148.40276    146.67716    146.43314    148.17964    145.38322    147.07764    145.00254    144.47989     143.4227    139.06508    141.18515    141.64129    137.42008    138.34664    144.23246    140.55436    134.48526    138.50553     132.4077    136.05784    142.76923    151.28383    174.06997    180.86401    177.29652    160.22916    143.93572    141.66866    133.36225    130.95432    131.50411    130.17002     132.2661    131.56294    133.76653    146.05917    156.30495    162.17701    155.06362    150.48284     146.6434     151.4755    154.38033     148.8652    145.23632    142.95509    149.26739    152.80163    151.68096    146.42861    131.38916    126.60725    126.45747    123.16135    123.77995    128.22846    136.41778    139.67071    143.02942    142.83512    138.15566    140.66874    136.09819    129.93733    131.77845    131.11678    128.41149    128.18045    126.53859    121.83221    120.72593    121.07983    117.34435    118.41089    122.35731    123.00673    115.68329    117.12977    119.03532    120.46059    124.26563    125.80141    126.06937    125.21251    127.66142    123.78604    120.10064     120.0027    123.88055    122.73799     134.4358    130.26792    136.02713    135.79815    129.10215    121.91615    117.95188    118.34037    119.20924    118.77432    125.27323    126.94488    139.51414     152.4293    158.09759    161.39756     157.1656     144.4463    135.53158    126.31288    124.40126    123.12875    125.32149    123.60365    125.61896    119.16364    117.63957    120.08024    119.20235    117.03106    119.70094    112.32955    120.88207    116.81125    121.57658    121.56269    122.23669     121.3963    117.77569    122.67041    126.61253    129.31181    125.84512    126.01359    124.69352    120.39552    125.21387    120.78672    121.02226    126.27313    125.10026    123.88007    127.48368    125.19053    126.83814    124.58466    124.02223    125.03691    121.87117    122.77434    121.87945     117.1485     120.3876    118.02661    120.24955    120.50873     122.3669    119.09742    126.34511    126.59663    121.41972     124.5237    131.01552    131.62773    131.75841    128.83053    125.41409    123.87108    124.53918    127.20188    120.93608    123.50214    127.04802    128.64036    124.92178    127.51501    125.72634    126.52803    125.05917    128.73477    127.52794    122.52257    123.55717    124.95348    126.75626    120.15933     120.4109    122.80807    127.40093    127.45972    129.57152    128.49729    130.57097    130.14394    132.71589    129.25382    131.67945    129.82772    129.10683    127.52327     134.4294     131.8897    133.46484    134.67902    137.75125    133.18717    137.86263    134.61246    134.22476    135.44457    137.91638     135.3523    132.23518    133.62164    130.31889    133.03857    135.06852     129.9602    128.86041    131.35834    133.76096    135.14559    134.99922    135.90943    135.93074    135.44478    138.77191    142.35371    144.60984    143.32934    144.89262    145.28055    147.15541    149.27772    145.30523    140.24836    139.76458    136.01905    137.58487    135.75418    134.09319    137.63782    131.73815    137.50092    135.35374    136.14665    138.44244    135.42021    137.57266    141.19674    133.90281    132.43012    136.70421    134.41664    132.12224    132.19231    134.57151    132.48199    131.71808    134.52044    130.06255    132.34586    128.64958    130.72843    133.12441    130.59057    135.33117    132.67729    129.19283    128.42124    126.87634    130.06364    132.65416    130.95436    130.08907    131.12642    130.19777    134.29712    134.36896    132.75001    133.51447    131.55143    130.51158    129.74692    131.90776    130.39135    130.85108    132.02039    132.23082    131.98675    134.02821    132.53271    129.39068    135.12618    130.31865    135.54704      137.169    135.41896     138.0047    139.69337    136.72469    140.03508    144.16364    141.76508    140.27768    142.95056    149.04237     145.9041    143.24103    146.63992    145.57853    145.25352    139.13683    145.10719    144.70048    150.69461    153.95869    154.89567    156.85498    153.94879    153.18837    156.52786    160.57563    160.90052    158.94216     157.2918    158.70387    159.95382    163.76986    157.04287    159.97859    156.29709    162.34647    158.53819    154.10567     164.6587    167.13335    158.59124    160.63114    166.24832    162.37785     167.5906    165.85254    169.77449    167.19574    169.05154    166.57748    166.50464      173.718    170.30821    176.45029    171.35213    176.33592    174.43621    176.88869    175.73599    180.68721    176.17513    177.00015    177.01896    184.81811    177.94335    178.82746    177.79845    183.90095    181.72614    192.96147    190.82969    194.43348     189.6829    198.77409    183.82588    195.35375    193.93462    202.87127    197.21456    204.56703    199.15013    201.98012    207.98459    205.53789    204.66218    210.72231    210.16123    203.39302    202.79145    214.28213    205.19009    214.96942    205.61554    214.88949    208.86417    221.62208    214.12529    227.17818    218.83902    215.76753     217.8058    229.92218    225.88855    224.91796    230.34556    235.53786    225.72185     224.4875    241.67196    237.53755    235.30356    241.04639    233.91708    233.34905    256.84855    257.74105    255.70784    241.78212    233.89184    238.91755     253.3961    269.94331     275.1519    264.72076    277.63937    263.82871     265.4796    290.25566    277.40177    294.85328    290.61132    272.94532    309.77211    320.39431     320.4242    327.61839     298.5416    320.89905    332.61172    318.96451    394.00797    397.27523    366.01198    355.97069    377.16859     372.8232    424.73037     452.5956     407.9597    466.28464    445.18346    408.05967     541.8375    553.32868    523.49425    552.80369     665.8138    435.65119          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        8.00     271.8166    240.55058    243.18555    228.39066    229.86933    218.45141    217.48016    208.16073    202.20358    195.27381    191.37468    193.88454     185.3494    182.71757    181.96229    177.82062    174.42058    175.66338    173.56451    171.48712    167.66457    165.26857    163.28958     162.3254    168.20714    181.63646    198.26503    208.02087    195.71378    175.45625    161.26017    155.69102     152.3007    151.45948    148.72951    150.67779    150.37113    151.08838    145.65334    146.88433    146.23841    143.70595    147.96841    144.20362    139.21703    141.82578    139.13985    141.50335     140.7677    144.12451    139.67162    140.07346    141.99348    137.06716    131.44238    141.32245    152.27754    168.74525    181.00395    175.11979    160.19289    144.13923    142.75756    131.03741    132.89383    129.82139       129.81    130.88197    133.18054    134.62531    145.35679     156.3978    164.68896    158.31269    156.69936     148.9899    149.19912    154.02444    149.82257    145.46106    145.26905    146.41333    155.68159    154.13276    145.41449    134.99055    124.58081    127.50948    122.99723    123.64538    129.60367    135.50098    136.89832    139.46472    138.94643    136.70208    132.31668    135.11147    129.81413    131.44353    130.57818    125.73118    129.78621    129.39615    119.52355    118.52661    118.57819    118.94011    118.19567    120.17357    121.58329    118.12894    117.20829    118.89142    119.12735    121.80685    128.65785    130.29884     125.2944    125.35633    121.22679    124.60251    125.76891    121.86005    124.41731     133.4018    138.56882    135.38341    127.13976    124.93869    119.71979    119.98016     120.1905    116.07817    120.13239    123.34387    128.78081    139.90511    154.37108    159.54855    158.97949    154.96963     151.2589    137.20057    128.32591    125.93324    127.35959    124.74995    123.18814    122.78644    121.28714    119.35164    119.69099    117.70185    119.00007    120.30567    116.16616    118.29393    121.03151    121.40721      121.359    122.10772    121.90262    119.72063    123.47016     125.0652    124.44713    128.48964    126.41963     124.5763    122.09001     121.9933    122.89998    120.50392    123.60041    127.09139    120.59752    126.09605    123.52322    124.14316    124.11033    122.31155    123.32152    119.13228    119.83887    119.99929    117.47273    118.46537    119.25789    117.79331    121.54003    119.95562    125.31946    122.88899    125.44448    125.02566    124.73764    126.70275    127.81412    129.88428    130.03128    128.58451    123.58723    126.23396    125.24353    123.40091    126.97846    125.80687    124.64838    125.65211    129.21334     127.9011    128.66321    126.27061    124.80776    123.52109    124.26538    124.15573    123.29718    122.41768    125.17147     123.8328    125.08867    124.88176      127.563    127.91867     127.5914    129.83606    130.57764    132.36319    130.94994    132.22352    130.09606    131.29071    127.95547    131.08336    132.49693    137.87099    137.01937    135.79278     135.3343    135.68551    134.90042    135.74301    136.32385     134.7297    135.30339    128.69926    130.99212    129.56644    133.74913    134.66583    135.25244    130.41685    136.91245    136.53143    134.09547    137.02887    135.92401    135.39234    140.70538    137.55142    144.36856    146.79098    148.24647    147.19414     144.3252    149.94208    145.03894     146.4852    140.81569    139.26928     135.9559    134.14303    130.28649    133.97605     134.5516    131.32514    131.58235    131.81989     133.8781    136.59985    139.02776    134.83959    138.09228    137.14745    136.60007    132.39556    135.64894    130.68655    134.13041    134.20007    128.47892    131.14691    133.60558    128.65542    129.94899    133.87902    127.29546    133.44589      133.094    126.97325    130.39784    135.29621    127.98415    127.61737    125.80183    132.22589    128.84926    130.88412    129.77051    132.20647    133.45541    127.78051    130.37828    133.89146    132.82349    134.22578    132.47945    135.44083    134.05641      130.146    128.03329    127.93112    130.29019    131.13397     131.3347    131.75927    131.26081    133.09906    136.17347    132.84443    141.68812    138.32285    142.14761    139.36891    143.20856    141.27835    143.00117    143.89865    143.00086     143.9826    142.83899     140.0602    145.73602    142.34456    140.90199    146.82451    140.50268    144.91358    151.82572    150.60939    153.61283    148.70531    154.88476    155.69364    152.05786    163.04165    158.26429    161.17181    160.14493    163.84732    157.63356    164.11458    161.05673    157.78449    167.24941    161.84109    162.72279    155.14871    166.28937      157.936    166.02148    161.30568    160.79643    159.86095    164.45016    168.12861    169.14617    168.31102    167.27695    169.83868    167.63963    173.09379    171.40238    172.23835    166.52144    170.12971    180.78405     173.1295     177.5521    179.31947    184.30416    177.09712    172.80667    184.53628    180.99581    184.83191    179.12867      181.687    186.00523    189.23493    187.70522    187.40087    187.48776    186.09619    191.75568    192.74341    199.89652    199.55314    199.19098    192.23998    198.25073    204.74336    205.60092    202.95066    202.93656    200.26142    201.71922    207.94448    216.52657    213.99459    208.44104    209.26243    212.26929     215.0043    205.34297    224.06207     223.1103    218.90603    216.75189    214.00942    210.38014    210.96896    221.84169    214.88316     215.9543    228.65465    236.80691    227.85304    228.22758    243.81652    241.97022    232.54177       240.53    239.59214     255.1621    232.86016    243.64206    243.66518    242.74263     241.1083    257.13029    257.79539     259.9593    259.04114    258.96743    247.27091    274.24483    306.94114    311.23481    287.47616    297.79271    318.38952     334.3763    315.97217    330.22254    306.35222       311.47    345.03447     331.1873    325.78905    380.72492    359.87252    395.78063     392.5269    391.56997     405.1545    472.27698    475.93138    416.80893    503.98499     474.1973    486.31525    500.19973    521.39763    646.86684    583.79484    549.87213    598.01241          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        8.50    271.37992    241.90039     241.8504    227.88256    228.88606    224.00294    215.17467    206.32718    203.57458    198.20166     189.6783     192.3926      184.981     185.1821    183.56478    177.32303    174.73409    169.65338    171.96596    170.23901    168.12791    166.80346    167.88895    164.39222    169.50858    183.33604    200.19311    204.89686    194.00807    174.69789    156.99882    157.24559    152.81647    149.45922     149.3068    148.69829    152.65985    149.13589    147.72692     144.7232    148.84539     141.9078    147.43889    143.67485    140.16538    143.39368    141.31432    138.46611    138.20778    140.58548    137.25121     137.6353    139.87486    135.68031    133.96678    143.32465    152.79133    169.19225    180.91639    173.42313    157.21286    143.08752    137.11799    134.47857    136.43664    133.63189    128.14963    131.24061    133.68901    137.49251    143.92205     159.1273    166.86538    162.43233    154.58273    147.82784    151.87222    151.33927    154.49626     150.2498     152.4359    156.06385    156.68052    156.94651    145.87029    132.73715    125.41609    124.96848    121.89758    125.62741     129.2376    135.66099    138.05487    135.79423    130.54747    135.18201    128.20031    126.95621     129.1624    132.33139    128.17715     132.5302    129.90729    126.38793    124.66134    120.64654    118.19553    121.53093    119.23561    120.38667    119.37612    119.04584    121.19824    119.74641    118.61435    118.93216    125.32395    131.84297    127.94299    129.24793    124.43909    119.80392    120.34882     121.7539    124.47889     130.1934     136.1884    135.32855    132.79283    125.49096    124.48875    117.80114    117.79168    120.96298    119.71631    122.14773     131.7215    138.46882    149.09365    162.95821    159.52021    156.33927    146.98066    137.41263    131.45218    125.66127     127.0074    124.56217    121.32887    122.42518    121.01153    119.15872    121.16952    116.39426    116.41656    115.97964    117.76693     117.6933    119.81338    123.32831    121.89862    122.09118    118.78177    119.31048    121.97815    124.56095    125.19607    129.23873    127.52009    121.13857    119.79466    125.83466    126.16516    123.08878    125.56377    120.90935    120.06607    126.39157    124.92894    126.88577    125.41938    125.39402    119.95447    117.06366    118.99159    121.30149    118.04254    120.03657    123.18131    121.96367    121.13704    121.73852    123.09779    122.81029    124.05435    125.54376    126.20735    128.99961    130.29785    129.55359    133.18161    128.49036    125.98484    128.06382    125.36686    123.79394    124.96344    126.08976     125.6797    128.70385    124.99755    130.50079    122.19628    125.80575    124.92673    122.13466    120.84941    126.72874    122.66033    122.81697    121.68365    120.20309    128.44961     127.8206    127.42485    126.98522    128.70338    127.90562    131.54326    132.75802    133.27505    131.96877    129.38521    129.86686     130.0755    132.96477    132.96409    134.64503    133.55373    139.42924    136.60519    134.61463    133.25092    132.87379    133.09125    134.91397    132.46258    128.92736    132.50042    130.50167    130.72659    132.68237    133.20446    133.44798    133.97857    135.11927    135.57051    138.72977     137.1968    138.53187    139.27679    144.08009    143.24849    148.18516    146.08597      145.842    144.61974    146.77866    143.94295    141.93184    140.64666    139.51948    135.27138    133.01257    130.75074    133.06534    134.50865    134.95634    135.90616    135.77673    135.44757    136.33753     135.4062    136.66475    137.81856     135.6414    136.12372    133.44164    136.80324    132.65623    134.96963    133.51538    128.26236    130.59884    129.83761    133.82432    132.69747    131.46597    131.87933    132.86063    128.06273    130.49478     131.0626    132.15409    129.04028    127.95879    127.92832    130.78171    127.29904    129.40447    125.25389    131.85447    131.19667    132.58951    129.38543    135.32108    129.63812    131.32099    133.18597     137.4791    129.20188     126.8856    128.31102     128.2848    132.86757    131.68807    136.63717    134.29562    132.66123    132.61565    132.78835    136.30938    138.67273    140.35838    135.07191    139.08778    138.94067    141.07898    142.14201    142.84972    143.23707     143.9149    146.94273    147.52064    145.23539    146.46977    145.11257    146.51608    144.09593     148.9869    147.22332    150.51827    153.76058    149.98179    154.54559    151.16433    155.21578    155.01983    156.36135    161.28881    158.64802    155.51801    160.01263    164.49385    162.08573    159.59166    157.80681    161.23391    163.77171    156.32982    163.45377    167.42087    161.97944    162.21081    167.20453    159.73007     165.8807    166.35436    170.96665    166.67037    163.57459    172.04993    169.39677    175.38242     171.9623    172.24875    169.29741    169.99085    167.63018     176.8422    175.99082    175.57749    174.64401    174.83588    176.99096     180.7664    179.54452    184.28189    180.00637    183.27848    183.38422    188.79353    188.91995    190.72215    189.61538     195.7631    191.22795    188.65483    191.31131    194.10676    197.00309    198.92258    198.57225    199.40301    205.05866    214.33015    198.32891    207.29492    198.72346    196.44474     205.9664    214.12057    213.84765    216.66214    215.33588      215.079     211.5378    213.46633    220.27499    215.22686    219.99847    210.07209    213.78745    222.44577    233.97968    228.26237    225.35401    221.06274     220.9506    227.36233    238.38213     237.4303     235.7285    222.83833    229.95056    227.82826    262.73546    249.73676     243.6917    232.41812    237.26374    243.66977    258.59578    265.18725    261.46475    273.78371    273.64107    261.85447    267.25405    296.37197     294.2297    276.40273    316.31399    308.61565    300.92895    352.93629    310.35368    316.52096    283.85257    299.86275    322.93767    341.24789    390.43749    405.33213    364.36684    391.81906    343.45497    358.29284    461.46797    429.29449    427.60235    440.28533     461.7445    420.17031    562.13333    683.38756    664.22091    541.50157    660.92321    599.67463          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        9.00    267.87114    242.30628    240.78038      228.424     227.2563    229.19225    214.89987    203.45516    204.17387    198.12163    194.99441    191.49734    183.65549    186.28759     181.0238    177.54144    172.19547    173.42218    170.42108    170.11211     168.4005    169.83457    169.80829    167.34703    169.23724    181.71933    201.54702    199.56327    192.22595    175.05238    155.92796    156.43706    154.58808    144.18674    154.80083    145.66882    150.39527    149.36319    145.88373    147.33501    141.38744    145.69942    144.06344    145.68946    138.84996    142.07766    139.76661    139.65263    139.57694    140.47102    141.71734    136.81096    140.11491    134.05846    136.80697    142.92342    153.04604    169.97162    176.99193    176.74553    159.77544    138.46046    134.28988    135.43144    133.04747    134.90046    129.52971     130.0364    133.44322    136.50118    148.19084    162.50062    167.31506    159.74207    152.15612    150.98625      150.925    154.18239    150.30482    144.01349    150.91653    158.93324    161.13036    157.50658    148.26143    134.96634    125.33129    127.57114    127.33046    127.91049     133.7294    135.09999    136.04405    137.30449    130.21136    126.05705    127.17453    126.56471    127.64057    130.08968     126.4015    125.53916    129.18052    126.76184     121.7827    118.00374    119.82589    118.71572    118.06602    119.74098    120.15847    117.92786    115.52906    119.27758    121.75745    121.71039    124.98235    127.24432    124.78852    127.09248    122.74929    116.52455    122.90792    121.85447    128.42615    133.89097     133.3451    136.30848    134.30453    123.86105    122.61751    118.12506    117.88813    121.85939    116.98702     123.9353    130.46048     136.8223    155.70953    158.31119     162.2205    154.58738       146.62    137.98605    128.96196    131.04489    127.12115    126.55692    119.24206    122.27869    123.46978    120.53664    118.15634    119.48558    117.01554    115.41987    117.83028     117.8386    122.62835    118.94417    123.34724     119.4063    118.08368    119.59485    123.56859    125.84142    127.14439     125.4068    127.78327     126.4512    120.26753    123.55696    128.84814    124.55119    125.55649    128.23444    125.14906    126.24434    127.73651    122.27444    125.57902    124.79733    121.65704    124.18796    123.20423    120.09932    120.02355    118.64883    117.54044    120.02866    119.44104    120.14965    123.49367    123.92954    125.11645     124.8236    127.63737    130.15558    129.21674    132.17964    130.66745    131.23229    128.58013    127.08955     126.5612    124.64845    124.34028    126.18514    127.09195    123.59832    127.02757    128.20066    125.17607    125.05225    125.06461    118.90552    122.60543    121.21222    124.29513    120.82082    124.25344     120.0916    124.72056     124.4581    126.77184    127.53288    129.50212    130.62921     126.6862     129.1816    133.17348    131.88927    132.22467    133.80077    128.40209    131.28673    131.90093      136.316    135.25195    130.58432    134.92451     133.0373    138.40032    134.17562    135.66228     135.6353    130.99124    131.00004    131.49317    133.69552    133.66707    135.16281    137.00825    132.13357    134.83636    138.31092    139.17374     133.8759    133.51181    135.88117    140.24385     140.2083    148.96665    147.08414    144.52785    147.91897     144.5756    145.71909      146.425    140.95175    141.58574    136.76603    136.41358    132.68202    134.99473    128.75218    131.64022    130.94431    133.24983    132.70467     139.0252    139.44268    136.82772     136.9839    139.62987    134.53221    133.22526     135.2742    133.15968    133.94412    131.24519    130.90066    131.32307    131.59854    127.24724    131.02166    127.94227    128.80868    128.09759    128.01922    131.90283     131.9111    130.24253    126.63345    131.17227    132.33478    130.38768    130.47703    128.53842    131.33639    131.29026    129.41652    133.77374    133.86971    132.89367    134.61396    135.41612    132.76844    128.11076    130.47946    132.13069    130.46766    130.57544     128.6367    129.09813    129.06682    132.87451    130.57231    133.91415    134.82118    133.04994      135.342    138.39492    139.14954      142.247    138.85003    141.70199    137.20886    141.87413    139.85721    141.97369    142.87692    145.25935    145.46342    150.84553    150.37002    150.74803    153.40297    150.40821    151.60968    149.39582    155.82078    153.02057    153.12426    148.77326    150.51079     150.8931    159.54959     158.7785    157.83692    158.91934    157.42974    158.39535    156.76855    163.24522    160.62524    161.61304    156.26469    160.99082    159.87822    160.15294    167.45941    155.74441    165.57599    154.80698    161.57319    166.21328    171.31126    168.16885    165.46941    167.75571    168.95758    175.13236    171.69515    173.39387    170.52901    172.62234    168.21008    167.45196    173.92931    172.09261    179.01874    167.31276    179.80972    173.25174    181.46554    179.50257    180.41566    184.31143    183.82007    181.59382    192.08041    190.31646    191.36204    186.70775     188.6554     188.2237    196.04737    194.63801    199.80806    197.40675    201.38668    200.01343    195.12146    204.07338    205.15629     204.5896    204.81382    204.81065    208.60698    202.66197    212.80516    209.07573    215.79094    218.14073    205.00578    202.00855    229.94713    213.32003    212.28151    210.12764    213.77134    211.60924    232.87944    228.01396    214.89028    224.16797    222.58213    223.71705    226.60326    234.38204    237.38521    223.48947    222.00047    231.64361    227.96048    240.68173    246.95106    233.23617    254.96513    231.86165    240.50212    259.39235    262.63055    270.43428    264.07721    259.37263     271.4984    251.92491    282.16094     342.3704    318.67847    304.67496    268.68299    274.23384    315.45358    310.83045    312.80651    342.28192    335.70498    329.29029      339.823    324.27009    362.48042    383.18256    393.10386    387.01826    361.96727    469.67425    465.88796    467.34457    433.99643    430.39313    418.44561     546.3649    539.55353    566.22458    501.54363    547.61564    554.13437          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        9.50    264.94289    242.53178    237.63332    229.14711    225.40926    224.74688    214.45013    201.92527      200.383    195.49645    200.06235    193.41766    182.54306    187.75527     178.7263    178.36588    169.71564    175.37574    170.93266    172.40945    167.16371    165.63075    170.02487    167.54944    168.48391    176.27425    201.15092    203.50047    195.70896    180.15418    162.31281    155.65443    155.80727    154.16569    149.89706    148.43689    148.85019    146.09255    148.66459    150.00842    146.90089    146.58792    145.77053    143.53052    141.18532    140.67115    140.07049    139.45688    140.83983     137.3995    138.90108    137.46094    134.20615    134.51682    134.45888    146.20229    150.17633    172.29823    179.58049    180.51371    155.53203    142.88096    141.32707    135.97501    135.59636    136.07119    128.26356    133.00086    135.44508    134.18948    144.01038    166.01134    167.24057    159.30083     148.6028    150.69037    152.83212    155.14246    155.63994    153.22962    153.05087    161.92045    164.84978    158.22117    145.40303    133.85978    128.62484    126.14318     125.2404    125.96928    132.21632     136.1453    134.07096    132.44045    130.78641    126.83276    125.76569    125.28281    126.77024    125.05958    129.59711    128.84874    128.60025    125.32121     121.0473    116.64845    119.82284     119.0371    119.72692    118.97699    119.86758    119.46284     119.2485    118.05368    118.94766    121.14032    127.73273    129.79849    128.68286    127.09087    124.14778    119.85412     116.3433    120.79156    127.78024    126.02859     136.0115     133.5361      131.854    125.29995     123.2909    119.01133    115.83416    119.73787    117.65184    122.31991    130.50869    142.44704     151.3394    159.72181    161.31572    157.23165    148.09792    139.83719    127.02462    127.56746    127.02405    124.78863    123.82375    124.33003    118.63298    120.41074    116.90861    118.49298    118.38039    116.85239    119.78395    117.77024    121.48027    122.62221    122.94361     121.2752    119.64156    119.23172     124.4031    126.34899    125.94314    130.90372    125.94603     125.7936    122.25287    125.64375    127.58437    128.28387    128.44589    126.95302    123.76569    123.80797    124.52201    123.66038    124.51661    120.48413    124.40562    119.53926    120.00747    121.77026    123.23007    120.41615    121.31532    122.24322    122.15573    125.72126     123.0241    125.03792    122.46438     126.9896    126.98477    129.04663    133.92901    133.97371    132.08043    130.58121     126.5702    125.51934    126.02696    125.07836    128.96969    126.27706    126.27017    127.72804    125.61534    126.90551    123.90673    125.57915    124.79642    119.42881    121.42035     122.8954    121.62359    119.72149    123.89277    122.51434    123.56925    122.93131     126.8541    127.51314    129.09436    127.97465    128.87088    134.07753    131.10848    134.15864    132.04871    133.60534    129.47318    129.06816    134.91954    135.46974    136.02151    134.42627    135.09553    130.68111    135.77731    133.41047     134.0971     133.9673    134.28071    130.69434    132.93958    132.85058    138.21383    137.55955    135.08463    136.29764    134.76608     136.2032    136.98048    135.47866    134.87107     134.2531    139.02785    142.20475    144.84427    143.70398    144.50461    145.81355    147.31481    144.54021    145.82267    143.89124      143.184    138.19856    137.53962    135.12997    133.47446     133.0801    134.27866    133.52813    134.14272    135.80046    136.56363    135.43867     135.3923    136.21529    134.19789    137.85052    134.72317    135.89694    136.39285    134.61077    136.32756    134.10128    130.02686    130.15035    129.54817    128.24341    129.08145    131.58792    130.67269     128.1119    133.73426    130.85823    129.79342    131.53023     131.6991    129.30096    132.00127    127.05593    131.39177    128.86327    131.15588    129.93078    132.37959    130.91966    133.54547    131.73659    131.79034    131.58633    129.41689    132.87813     134.1724    130.91638    129.64095    129.97407    128.02962    130.84039    129.06104    130.12535    131.16886    128.77167    136.63772    133.32016    137.88235    140.42308    139.17492     141.8363    140.43108    138.81801    143.42458    144.90009    143.20265     147.4176    148.32963    143.26519    151.78083    146.46911    146.82075    144.76414    144.13594      145.348    152.61226    152.67981    151.56137    156.94401    151.97242    152.35414    148.74864    158.50606    160.86779     156.4289    157.16986    159.32964    159.70448     158.3065    161.06118    162.76385     159.3078    160.74243    156.36149    155.80509    162.40754    160.31828    164.50161    160.45841    162.51254    164.40305    170.06404    166.32613    165.80667     162.7788    170.03974    164.20527    166.81945     169.2362    169.06531    169.73024    169.34541     169.7287    172.63312    179.45779    180.38275    181.52747     176.2003    175.44417    187.23404     184.3842    185.78468    183.50788    182.36933    183.20997    185.89348    187.49225    191.63717    188.36013    183.31992    188.64192    194.46364    192.12595    194.84863    198.66307    193.46865    193.82478    195.83623    188.49685     202.8648    208.87625    209.57041    202.22783     212.0422    205.44004    201.39738    208.20764    214.07954    210.86443    211.55363    219.75238    209.42964    213.67876    210.58699    216.54973    214.87567    216.77777    221.12775     211.6762    226.80536    220.58603    224.24581    226.11057    217.36144    220.80545    235.71843    236.47634    227.23725    229.67302     237.4473     233.7344    248.74434    255.69848    244.22743    238.31006    245.82077    245.36994    248.53409    266.42946    273.87235     246.4837    235.66135    265.62247    279.67651    284.12334    274.10128    299.21069    301.96451    294.42321    277.24083     349.6557    325.03524    312.71641    294.11839    330.70817    328.85372    352.18513    386.72566    374.28281    321.51421    404.50055    381.28402    374.41212    497.82216    478.17261    475.31346     459.8925    401.60859    481.91264    575.95969    634.07508    478.93783    361.03043    570.22609    588.86072          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       10.00    264.06672     244.6536    234.24282    229.61362     222.5053     220.9031    215.64822    204.05968    196.51449    196.15058    198.42481    195.64863    182.36769    185.42607    176.76966    176.42318    169.94448    177.03522    175.91376    174.26821    164.44851    163.35092    169.01366    166.13689    166.80015    185.17139    199.33147    205.01111    202.67895    183.62698    164.47947    157.69812    160.46143    154.53935    152.23365    150.24598    148.31847    147.97286    153.19291    148.52642    142.62407    144.16113    144.18909    142.88163    139.89436    142.43737    143.34212    138.45492    140.69441    140.28152    138.55033    137.06088    137.60616    135.20278    137.44492      138.311    153.63655    169.71752    182.80969    176.78599    158.35588    147.32063    137.22596    133.92696    137.99434    133.44848    134.41651    132.31353    132.27235    138.84636    146.36972    160.74433    167.48773    159.78554    152.52076    149.42805     151.5767    156.00202    159.93254     157.4064    158.00918    159.84444    165.94279    159.32501    145.25287    136.16844    128.51963    126.14954    125.21684     125.0279    130.64023    136.44061    133.93876     130.8607    127.92206    121.81636    125.83798    127.47089    126.96531     122.8195    127.49023    130.68605     126.0067    126.99195    126.57878    120.34199    117.64674    120.12867    118.16566    119.94643    122.82417    121.69693    118.79069    119.39126     118.1402    121.88602    125.15118    127.89442     129.2364    126.61582    125.40098    119.35781    118.81117    121.82504    126.45274    132.84188     136.7928    135.42406    131.72612    124.54918    123.39676    120.44198    121.57226    119.09371    119.87428    122.97345    128.94437    140.11861    148.31036    159.23487     159.5429    154.74839    143.07106    138.66404    126.24943     122.5473    124.13107    123.96818    120.87573      122.078    119.30478    117.86892    118.71309    116.38292    116.84508    119.63934    117.43327    117.06316    120.29899    121.44022    125.43601    122.22804     121.4718    123.44317    121.78627    123.67404    123.42338    127.09221    125.23466    126.45849    118.59604    130.45181    129.40394     128.5419    130.39719    126.99716    125.01748    125.25134    127.09066    126.65571    123.48315    123.96919    119.60719    115.68808    120.78233    127.04299    123.64965    124.84682       126.34    125.21937    123.78265    123.80746    125.22859    124.26879    128.23701    131.68531    121.91524    129.02966    131.49871    134.91697    132.56995    131.70454    126.06083    124.83773    127.40015    122.65575    124.37843    125.56007     129.1785    126.02215    129.12343    129.76198    125.66715     121.8415    122.42267    123.21017    125.18073    122.10335    121.44415    123.34238    120.66352    121.70264    121.90427    126.80175    125.94059     129.1572    129.31267    129.84625    130.29568    131.17367    133.04896     127.7634    133.06049    133.02847    136.18506    131.90106    131.16488    136.95932    135.93739    137.29447    139.89269     135.5757    135.68112    136.97099    134.69866    135.35942    134.80239    134.69182    130.22938    131.68511    134.52287    133.18116    135.82261    129.97523    135.42507    135.62116    134.89413    134.31081     135.9281    138.69014     136.1047    139.74219    141.93308    145.88884    146.01431    147.61151    147.55001    149.52915    150.47429    146.28129     143.6741    139.21574    138.06532    132.27457    133.41439    133.20177    133.75583    131.52836    136.32968     134.3231    136.18655    136.72593    135.56621    138.76333    138.48509    133.20609    134.01607    135.99878    136.03124    137.10127    134.58852     135.3538    131.57486    130.91284    131.22419    130.67196    133.34706    131.96707    129.35463    124.84863      132.174      134.213    130.12481     132.6306    130.84632    129.46251    131.51181    128.64292      133.273    130.23358    131.95175    131.99559    132.92377    132.34396    131.74941    133.13646    133.31527    135.77952    132.92098    132.54572     131.2163    132.37469    129.38626    130.25068    130.28498    127.03408    133.28238    133.39475    132.98111     134.3503    131.22953    134.10406    138.97064    138.17783    140.97657    140.67226    140.36336    138.71984    139.78786    144.40434    144.45314    145.75794    145.50251    141.26309    144.77635    140.96499    147.66031    147.01023    147.90842    144.82179    142.10164    152.10166    152.10184    154.54073    148.78759    152.15651    154.90942     160.2887      158.178    154.45034    156.41077    162.87466    165.69379     159.2138    160.02966    165.17334    162.05432    157.07999    158.86031    157.31711    163.71812      162.229    162.38027    157.67609    158.20709     164.4839    168.43912    165.00347    171.86628    164.29994    162.35283    161.46134     171.0666    173.23019    169.00587    160.26529    175.58784    164.82227    170.90057    168.58185    181.22396    182.30661     168.0668    176.90444    176.62525    177.26023    188.06841    184.29813    180.90637    180.72494    185.26996    187.13774    196.88262     188.0016    189.37553    187.45351    191.22929    184.68169    197.73191    196.94496    190.73342    201.36438     194.9316    197.98039    207.72586    208.78447    213.78964    206.78164    206.26011    198.94001    199.82002    214.19768    205.19566    199.62024    208.51919    223.36536    206.10826    212.70735    223.89589    227.48386    208.98343    202.82893    214.13047    211.30272    228.94558    229.95145    218.27411    224.55995    216.63576    232.26588    240.70453    234.47662    235.29525    236.55903    230.95875    234.13526    229.07858    248.15978    254.11527    248.99436    248.52973    240.68914    264.52614    247.34129    278.07133     251.7182     259.3895    265.06403    262.24196    275.88994    281.77108    296.80483    282.14262    306.12976    271.37799    345.25334    326.87326    322.44001    320.82935    335.46858    314.64952    351.70073    354.12811    392.68789     358.8339    374.18828    356.36649    353.92098    454.41488    451.23116    487.75976    475.17521    395.05764    422.85322    470.05319    410.82393    680.79215    594.31833     718.3382    624.17872          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       10.50     265.6138    247.16728    232.11883    229.97335    219.30392    222.21867    219.90976    206.13096    196.55349    199.10525    196.50568    193.18862    182.25938    186.07245    180.98416    173.59831    170.49045    176.99565    183.33163    177.22743    165.04171    162.41311     168.5859    164.62477    166.71862    184.14568     202.3987    205.12399    194.56259    173.63982    163.43819    156.05849    162.84544    152.30742    152.64505    146.46299    148.29392    148.84669    148.58142    148.76266    141.34923    143.09167     142.1484    145.37552    140.19407    142.18498    142.57118    137.90173    144.33004    143.15093    141.23639      137.264    135.11458    137.27414    141.12709     142.0614    153.26023    173.31127    179.55463    177.33269    153.04857    148.16325    139.01866    138.08054    135.02515    136.43269    133.52926    131.48361    133.98837    137.96789    144.47678     159.8522     165.0702    164.36469    153.22138    152.49656    153.54097      164.839    170.11375    166.58206     165.4389    162.63992    161.88283    154.33363    145.42697    136.83704     128.1916    124.62314      126.705    129.34231    132.85861    137.02579    133.12067    128.88186    126.42131    124.24376     124.1133    126.26112    127.99152    123.48545    129.01046    129.57671    128.37329    126.61883    124.22001    118.82488    120.31381    118.39659     119.6183      119.865     118.5295    119.42326    116.85598    120.25261    119.63373    120.72254    125.80255    130.54542    131.00446    128.49784    125.42461    120.57957     120.2984    121.45153    128.04709    129.06649    135.10485    133.76884    132.79256    127.44859    125.04743    124.84612    120.50776    119.00333    121.84009     124.3672    130.22015    139.56278    148.21266     159.7468    162.82102    155.78263    145.39854    135.41281    129.66614    123.62323    126.57697    123.03817    121.24102    121.42552    119.87029    117.93709     119.8957    118.80159    115.44786    116.64905    116.99417    117.07822    123.50768    119.22167    119.72379    120.16103    121.80739    122.98968    119.22739    126.85889    124.55415    127.13164    126.94149    126.44791    124.86501    127.17532    130.17282    127.96174    128.39168    127.54706    119.61382    124.24868    126.59934    126.12068    125.41399    123.65594    123.47562    118.75488    120.66483    122.69563    125.43503     133.3041    131.44983    130.19695    124.39776    123.40324    123.60723    124.35022     125.1368    127.83324    123.93598    129.01513    133.43136    132.05078    131.41712    128.87476     127.5323    123.75658    127.06972    125.65852     124.5687     127.0213    125.06627    126.01886    127.61471    126.45431    127.38243    123.39526    123.14846    121.52566     120.9017    124.65717    120.14768     123.1917    124.35993    126.62838    123.27842    125.53814    124.99933    128.09474    126.98648    127.77471    128.61488    128.18186      134.106    132.36526    134.41307    130.23044    134.02969    133.99604    138.60554    136.05683    137.90791    134.87473    136.65015    138.25561     138.1421    138.09153    134.54832    136.66985    133.73069    131.82179    126.17846    131.35528    134.30862    134.89019    137.68674     132.4758    132.83672    136.05023    133.35323    133.81749    135.50526     139.5231    137.48927    136.62349     145.4627    149.46196    147.19045    146.45994    147.18301    143.58745    147.52833    144.86683    141.69288    138.73057    135.22608    132.20563    132.53311    132.87652    135.85781    134.68364     138.9586    135.67152    137.23581    138.10824    137.49221    137.51318    139.23925     135.8938    133.85859     138.8827     134.0484    137.33959     132.2897    134.24745    133.37848     133.9353    131.58684    128.94668    128.88483    130.65204    129.52497    128.31253    134.04603    130.29285    128.62408    130.10686    130.07289    130.12933    130.02003    129.93594    130.82496    132.57851    133.61096    131.31688    128.08192    132.28545    132.45228    136.31085    131.75413    132.56291    131.34984    136.80949    130.33888     131.7011    133.05073    126.00303    132.52878    133.28372    133.35213    132.58337    130.40638    134.87414    132.88909    138.38664    135.06037     138.9813    138.28126    137.81637    135.72124    143.54256    144.35865    144.98717    147.53456    144.99993    143.14928    143.35968    145.68733    147.64833    145.05478    147.57922    143.37515    143.13401    142.25452    148.92175    145.17525    150.99691    152.60312    152.09382    155.25561    161.11657     155.4742    155.25091    159.89866    154.39096    156.22182    158.66643    158.67126    157.04589    161.02472    156.69378    158.65842    157.55258      165.045    162.65847    157.42718    160.83988    160.67841    155.33056    162.73471    166.96922    164.14133     168.2244    169.03975    165.46186    167.77647    173.85687    169.78803    169.12064    165.43225    168.02348    169.80816     179.8043    175.89658    171.37195    170.55594    171.08495     170.1893    175.09658    170.41063    175.77693    183.33414    184.18159    182.67877    184.16587    185.80775    193.71029    187.66968    184.23735    184.00122    189.54139    193.47212    199.04348    196.49315    193.77524    196.29586    195.82746    207.25851    207.96457    216.70559    197.97726    209.72997    208.68501    202.75765    200.41106    218.59555    199.94413    215.60389    206.27086    208.33671    226.62619    222.12783     211.7153    207.98994    217.52644    213.92094    218.17504     221.8037    232.08081    218.47044    224.13438    225.43962    220.41971    231.84724     224.5306     239.3968     215.4288    227.96843    227.98491    249.43619    238.79355    248.10875    253.78186    256.82006    230.76126    270.22161    248.33946    263.59211    249.89839    268.42674    246.04893    256.16071    281.25843     285.0324    288.45675    282.01923    269.79937    285.80549    322.33259    331.65463    302.48686    307.16467     308.2459    332.74785      354.052    387.62387    370.01606    375.53402    374.21754    365.36251    375.37019     460.3455    474.51686    473.65811    466.12915    445.69989    487.32716    575.10505    672.93012    729.86325    586.32665    586.89694    619.02031          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       11.00     266.4978    248.00939    231.51387    230.85882    220.92116    225.55856    222.45592    208.47802    197.50502    197.95293    195.16709    190.33432    182.94958    192.88601    184.60591    175.78026    174.66659    177.07024    175.94641    178.71521     165.7734    161.77888    167.06634    166.08933    168.58318    181.98808    203.82843    208.52376     193.3383    171.44519    161.22621     154.5005    154.46228    152.39901    148.37001    146.98293    148.78064    147.87451    150.61132    149.30632    146.59675    143.57088    146.49203    139.98298    145.14373    143.20263    142.25446     138.7869    138.84922    140.38524     143.1006    137.56438    139.84213    135.67921    139.18613    141.39492    148.57288     168.7756    181.11931    176.11837    160.80382    141.47376    138.52988    137.95613    135.18925    131.55485    131.96306    134.87547    137.13868    138.02327    147.07693    159.59889    167.41499    161.77706    151.57318    145.55512    156.16637     166.9342    180.43395      181.808    173.96269    162.46896    155.06587    147.41744    145.96617    132.03502    131.52436    122.60993    122.94173    126.29448    131.00738    136.70524    134.22874    131.19587    128.50949    122.84875    123.02127    125.57381    125.24025    123.63357     129.4049    127.85658    128.57469    129.05252    124.43841     118.7177    117.84528    120.84035    120.76222    118.67172    118.54258     120.6259    117.32416    120.90828    122.60174    126.70283    124.39479    128.85709    125.55735    123.74502    123.96175    121.31751    120.36641    122.89962     126.3128    131.66626    139.34757    133.88083    132.62307    126.90763    121.25518    120.81358    118.35024    122.91325    122.04431    124.55796    130.27012    138.73351    151.08146    159.78413    160.30194    151.79741    148.96232    136.89952    129.63283    124.61737    125.05151    125.85178     124.1744    122.67003    121.57564    117.75881    120.54995    117.92338    115.83903    117.78392     119.3827    118.51993    120.02764    122.04321    122.42452     122.0834    121.84009    118.94931    116.88529    126.09441    128.22259    127.08157     124.2813    123.68107    126.25791    123.83528     125.3491     123.5298    124.04442    128.57761    123.90979    125.62525    130.73877    129.84853    124.15921    125.98988    120.78108    124.06017    120.00854    124.33488    125.98507    127.72142    126.42443    124.76579    117.14487    120.26919    121.07047    123.94344    120.26203    126.82971    122.79177    130.20322    127.71147    130.74142    131.51545    129.84677     126.0605    122.01481    126.87344    124.88941    126.60971    128.65548    130.35714    127.72309    130.29214    127.26907    126.67111    126.99046    127.84997    125.13563     124.3912    120.11835    120.76496    122.57154    121.52161     125.0357    119.53084    128.59329    127.80119    128.42423    126.97508    130.12389    131.87877    129.35599    129.27053    135.93695    133.08495    132.35417    134.85909    135.02031    135.24357    139.23666    138.87153    135.93201    138.76432    137.16812    132.86857    136.13868    135.67793    135.46823    135.41036    133.55127    131.49243     136.0879    135.80664    133.83861    134.54371    132.65753    135.79696    136.64564    137.81545     136.4207    136.05026    138.89217    134.09098    138.90106    142.70756    147.39055    146.90482    146.20553    148.36615    142.06282    147.15184    145.58902     141.6114    137.05479    136.85946    134.37021     132.5386    134.25974     131.6103    133.67566    135.72457    135.44909    132.71071    141.18216    137.76559    134.36276    135.01146    135.75712    134.42638    135.22719    129.49251    131.28354    132.96636    131.17053    130.51031    127.17507    130.15985    130.70713     132.9511    133.49215    134.97296    130.67735    129.91561    127.50255    127.87335    130.61862    134.04917    129.23264    127.72841    131.62394    128.45718    132.99183    129.74325    132.08387    131.83221    131.64234    131.60564    130.47694    133.78925    133.99139    125.73481     132.6587    131.01402    132.40046    129.03588    128.60086    127.91188    128.77592    133.76813     131.5188    128.75352    135.08446    133.67553    132.47164    137.14532    141.87383    139.17702    140.32907    139.52664    136.75859    148.24773    145.54634    147.40915    145.37643    146.79195     147.0315    147.94229     153.5663    145.10512    149.32419    144.43981    144.76243    145.02179    148.43094    147.52191    146.04586    152.69722     152.4037    151.80507     159.1965     158.1233     159.5985    157.96775    159.91937    159.62382    161.32823     158.5126    154.79355    158.97528    157.02861    158.30547    151.36436    163.75849    162.52917    164.24337    164.12665    164.10066    165.72851    167.20321    164.27531    167.16265    167.27262    168.04193    162.62024      168.705    174.83989    172.16046    172.18259    176.25358    166.51837    177.17348    174.44018    172.68573    173.47449    176.22965    173.55011    177.90039    181.87878    183.32973    181.66437    185.32651     180.4375    183.19981    182.69943    187.95711    194.83844    182.30555    187.37826    183.44584    190.80401     199.6445    199.59375    196.70501    196.93045     201.6862    204.22973    200.92705    213.20859    208.34799    202.96797     199.4247    197.53247    201.11442    209.23867    199.79541    204.41598    203.12517     215.9659    215.74194    224.10235    210.43784    221.04655    214.87194    222.90044    208.34176    233.42813    213.74719    215.01895    229.38652    218.47393    216.27098    209.63465       243.43    233.58716    236.44083    230.06934    221.35196    233.17478     240.8546    244.21766    244.63717    250.94483    246.94558    252.47372    249.86893    254.27142    264.70897    268.56163     254.2899    274.64584    262.41572    277.46735    284.82347    297.14435    282.71955    289.24244    288.84301    294.54792    313.96516    330.28726    307.53312    344.09576    320.93819    363.93422    380.09542    378.26488    362.76744    351.85201    360.96473    382.24997    472.34031    439.36319    384.49458    441.86028    434.73913    481.70004    632.04789    635.97175     664.1307    521.02671    587.88713    607.56778          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       11.50    265.41072     247.4601    231.18067    230.02402    222.93488     225.9405    217.08297    210.93008    197.68056    193.51712    193.59997    187.69762    185.11786    189.29485    184.69588    178.15508    176.18973    177.90521     173.3745    171.70597    165.17915     166.9468    165.95248     165.6804    169.23499    181.88906    203.22385    205.25834    195.87021    173.96276    156.19127     152.3356    154.08203    150.53058    145.06376    150.07091    153.77186    146.93567    148.11561    149.29253    146.97092    143.02877     144.1586    143.02318    147.12266    143.04834    142.34013     142.1632    138.86891     140.2665    138.53014    139.50069    140.67167    141.02665    136.17125    143.17145    150.88274    171.28229    182.91176    178.13369     156.9304    144.77145     140.0026    134.49864    135.30547    130.38197    132.68673    131.54949    134.39613    136.73717    144.02241    162.03222    167.27377    162.75012    154.12333    152.74177    165.68144    176.50506    188.16074    186.38255    175.29106    160.37764    152.97639    148.49318    139.33051    133.98692    128.44161    125.82663    125.90943    126.44792    132.94937     134.1495    133.18267    130.35918    124.99944    125.90472    121.18792     123.8869    125.50968    124.18452    123.73272    127.20787    127.45302    122.86637    119.92745    117.29827    118.19114    119.69619    117.51874    120.54345    119.86682    119.59287    116.51397    118.70961    123.68282    122.78257    127.59617    127.54168    130.57865    126.50786    125.07225    123.43569    120.87411    123.45227    127.02673    133.27608    134.18403    134.58747    129.40276    124.18442    120.97623    118.88032    117.71121     120.2534    119.49848     123.7942    129.33426    138.38265    151.36335    159.15794    162.46383    156.95687    155.21491    141.86459    135.36207    128.36784    123.23212     125.9195    124.07617    120.29252    120.91463    117.20143    116.34488    118.65427    119.51911    116.99239    119.17749    119.47909    119.42896    120.19385    123.91586    121.23408    121.93525    119.02666    122.40811    125.73712    126.35689    126.81121    129.19547    124.23053    125.39069    120.91169    123.58233    123.84421     126.0208    126.22216    127.44562    126.35736    127.50401    129.63773    126.85245    124.90825    125.60399    122.17487     116.8581    121.80734    119.63567    121.25948    117.49681    120.90522    118.74175    120.81061    123.48163    125.69379     127.5515    124.88224    127.15965    129.33597    131.03326    131.02069    133.24844    128.79472    127.48499    123.10253      124.413     126.8455    126.24905    123.95445    129.15332    125.68504    127.65607    127.08562    124.67819    123.02347    121.60752    121.59167    119.49981    123.60741     121.2608    123.30703    122.60624    123.50524    125.10528    123.25873    125.20314    126.46552    127.60949    129.56785    130.03772    129.81515    131.32081    131.12983    128.24466    131.32749     134.0592    134.38188    131.29718    136.46397    133.28212    139.80997    138.96743    141.20453    138.52208    140.58811    139.85437    139.06892    138.64677    139.30232    132.17582    137.10685    133.32698     136.9451    133.69837     133.3944    136.25602    135.10139    135.94023    136.09611    135.89343    141.12832     136.5122    135.83404    143.05794    148.59997    149.51247    147.85945    146.52496    145.67675    138.07761    142.23538    142.74342    142.69827    137.52757    131.27838    133.24219    135.82862    132.69291    134.87288    134.62207    137.28922    130.28945    136.55651    137.00506    136.53084    136.15271    134.90377    135.41286    132.33963    131.77302    134.94044    132.84127    132.72719    133.79121    130.99489    131.46856    129.81265    128.34052    129.98406    132.98241    126.83281      126.563    129.99638    130.32071    129.85504    127.27544     129.4282    127.21007    130.95756    130.22098    131.65119    132.53645    133.96279    131.40279    127.99533    132.08425    129.46329    130.71644    136.19202    130.67744    129.05732    129.65711    126.59288    132.32419    128.92935    128.91878    125.99078    131.67813    130.48165    129.87542    133.40331    131.83458    131.77693    133.19974    137.31441    136.76524    139.09177    138.02236     139.9149    141.08246    140.45511    139.69334    144.46213    146.84337    143.44474    140.02358    148.53363    146.90569    142.56684    144.07034    147.83805    148.08195    148.39474    150.84038    149.76997    149.27098    150.15967    149.21287    154.25244     159.1919    158.18186    158.00664    154.04037    158.81304    155.43424     161.3327    159.76468    166.30981    159.66176    155.60158    156.10304    159.49937    160.55239    158.36217      156.032    165.93929    161.08913    158.21689    157.62033    166.00475    167.87896    168.04248    165.72389    167.70457    172.30032    169.96837    165.74674    165.06248    165.81654     165.1918    175.72253    172.72222    175.15408     176.9838    177.42109    179.97086    179.27038    176.82002    176.59636    182.81561    180.56684    187.34197     177.7587     185.7023    190.32482    190.53902    190.11463    187.82703    186.53818    193.83656    198.35008    197.60467    199.70609    200.51026    199.10399    196.45335    201.92777    204.68258     206.4978    197.99209    206.62504    199.38338    221.15912    221.37973    209.03351    213.71837    222.81027    212.79495    222.67654    224.49702    220.85377    219.16014      211.763    207.96566     220.1561    225.67362    222.25722    213.19779    222.18043    219.45084    226.35679    247.16343     228.1966    221.46017    227.27493    229.14211    232.53381    236.95774    256.85127    237.21098    239.49529    239.00179    247.11814     269.1527    257.46426    263.14408    268.10796    248.30265    267.70202    273.42041    288.29204     297.7905     294.4955    275.82143    276.90736     287.7751    306.69013     302.1232    322.86518    313.72679    331.03404    321.06392    367.50143    386.59372    393.64699    355.66906    354.03761    366.53808     383.2104    496.54308    479.09307    402.10263    481.90775    496.77885     503.0014     563.9135     541.4927    564.28651    561.86237    670.25821    512.84503          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       12.00    263.20428    247.30704    232.27601    229.72552    222.85514    221.52915    210.60852    212.09608    198.12873    193.42213      191.749    186.21021    186.06151    179.02745    183.42402    178.85031    176.82845    174.89226    172.80811     174.2184    164.19788    168.61288     164.8891    165.15217    168.87026    183.23686    202.58644     206.7521    198.29732    173.01891    162.35996    154.03861     154.6823      148.656    148.46578    141.77481    155.41831    144.41729    145.69256    152.40974    144.39297    142.24527    140.10914     144.4155    145.01169    143.15278    142.34421     138.0449    138.44532    139.82544    141.92123    139.84694    136.83777    136.77467    138.43824    141.74627    155.55916    174.19643    183.57493    176.83139    160.14466    141.60047    133.30111    136.21364    134.05559    130.39405    134.62386    130.74759    132.73296    140.10048    146.36944    159.34342    168.10933    164.03055    153.15774    156.13605    164.80934    181.58992    186.46956    184.18929    175.06427    155.95291    152.24369    145.79103    140.35051    136.06149    128.20425    125.30264    126.48351    129.38925     133.2565    136.41609    139.42638    132.17611    128.58051    129.09286    124.84979    123.09149    123.74953    121.25451    127.63878    125.59758    127.64477    127.30872    122.09439    121.61937     117.7998    123.76717    120.40963    119.79359    119.35947    120.37618    118.83679    114.40312    116.84757    122.33931    129.18168    129.01743    127.62626    125.35075     123.5229     120.4503    120.24076     120.1324    126.03957    136.06403     130.7055    135.68209    132.07998    124.87512     121.0801    122.46214    115.57192    118.71256    121.38911    126.72184    131.35512    141.77622    151.64149    160.02982    163.55979    159.16317    151.44483    140.60403    135.29782    126.32334    124.11451    123.84538    122.08133     120.1019    121.03045    119.98908    116.82135    115.70122    117.29538    115.65288    118.00296    118.48554    115.51329    119.15164    121.71437    121.69034    120.03007    120.73621    122.61997    122.55359     127.0778    123.81607    122.72663    127.18883    122.00549    122.47712    124.87186    125.92005    124.07177    125.89696    128.66983    129.96679    129.75782    130.54749    131.16823    126.81588    126.43483    122.74208    121.62019    123.39925    118.98486    117.68524    120.91477     120.3547    120.25774    120.14068    123.38368    122.78223     127.2166    128.26184    129.72532    124.82985    131.54538    133.15458    129.68722    127.93842     127.1406    124.27178    122.97848    127.76858    121.60263    125.09103    126.28087    124.64045    123.83388    127.06582     126.1946    128.21401    123.16501    122.27355    120.07143     121.8726    126.35145    124.82422    120.49619    119.79408    124.57861    123.43839    127.20898    124.83025    127.36953    127.26636    130.73626    124.71816    131.10624    131.71195    129.57607    128.08351    130.11651    133.07168    134.21493    136.26921      135.326    136.34409    138.00365    139.98833    138.06556    136.79717    137.37713     136.5693    138.13134    132.14606    133.10794    129.07145     134.5072    131.55166    133.61584    137.73529    136.81137    136.19204    137.95944    133.78128    137.52456    136.82772    137.07508    138.34321    139.93044    144.92475    144.52775    149.51614    147.64576    146.04286    146.24252    142.64335    140.92992    137.82342    137.31134    133.65163    132.23912    135.03468    133.65165    136.74139    134.81278    136.44772    134.04978    136.43981     135.8378    133.90412    132.91043    131.48278    138.68619    135.21676    135.13574    136.73239    135.38959    138.86721    130.62234    127.24119    132.72417    135.06223    134.72878    133.88929    132.67925    130.26616    128.39202    132.74305    135.86328    131.34111    128.41571     129.9324    127.44676     129.8852     128.8189    128.13647    128.11803    131.16608    129.17616    132.95931    131.57354    132.35491    130.06218    131.33531    131.65367    130.93574    130.59258    130.48619    128.04964     126.2969    126.16548    124.02692     132.8303    127.08636    131.36022    131.99668    133.13095    133.39416    137.20303    137.24668    138.73202    138.15137    137.98627    137.48748    142.08932    142.41302    142.22071    142.40538    144.99456    145.98414    144.78716    143.49151    148.97887    144.75233    147.26646    144.86557    142.90315    147.08358    148.65756     153.8345    150.18619    150.01327    152.80582    150.31222      159.904    160.88558    157.52503    158.84194    156.98331    155.75563    160.40959    157.28504    161.12387    153.78151    155.84379    154.46003    162.24599    162.78628    168.26847    162.14318    159.76657    154.47752    159.46422    166.90957    169.02272     162.5014    174.77432    160.81542    168.69639    174.86627    167.62549    166.99447     166.8248    169.44746    168.35984    167.80391    177.19616    179.92838    174.89739    182.62687    171.75077    180.78093    185.51532    180.93499    177.71647    178.11888    184.37703    183.77429    182.32554    189.92624    187.84774    193.79146    188.73817    191.61272    191.90499    192.62475    191.21127     197.4717    196.70749    189.90743    200.68408    208.37343    205.63929    210.21457    203.55432    203.70435    209.68251    196.28686    214.00964    215.43436    196.06818    206.40497    213.80801    206.54392    215.48049    223.52835    214.83656    222.15653    214.25672    211.71999    230.00134     220.8568     234.3482    214.59841    218.19758    226.14453    230.49726    233.68925    241.22093    239.34962    231.54849    229.78441    256.16773    236.07203    239.99742    238.58631    248.03496     231.9058    274.18427    262.25326    262.83828    267.54144    249.13119     265.9279    272.38108    308.04265     288.7714    300.59192    279.90169    301.63157    288.09007    286.77842    296.26157    306.87862    308.20364    291.05935    338.41439    388.53227    386.05447    400.22159    384.57549    401.53612    367.81037    408.86834     475.0869    425.45251    429.22874    484.52206    436.78694    594.15428     536.2992    517.56626    563.29366    636.85968    533.42537    568.58492          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       12.50    261.85002    247.14432    232.72276    229.29569    221.60487    217.44492    214.81659    214.06854    197.81629    198.30712    192.60368    191.25107    187.31995    182.51281    181.18899    176.32287    173.57445    173.49995    171.48536     179.6304    168.01744    165.14063    162.99181    166.46613      171.471     183.5966    202.70559     208.2883    198.29858     171.0491    161.60734    156.17983     154.9386    156.38568     149.8131    147.00203    151.54111    145.63163    146.08663    150.58757     148.1132    139.78217    144.84295     141.2968    141.83638      142.525    140.12107    144.15823    140.58988    142.24017    138.43377    140.52875    141.38635    141.42572    135.87965    140.64592     154.3482    170.94438    177.85099    176.53962    158.51762    144.67165    139.76839    135.22996    136.49937    129.99214    137.69755    131.38633    136.47315    139.17582     148.5117    161.76704    165.24572    166.83207    153.88634     158.0628    165.38989    177.66124    184.74591    176.65968    161.73635    151.50969    148.48068    144.35317    138.37279    136.53883     127.0687    127.69034    126.89108    126.80552    132.78734    137.24078    138.89726    134.42547    127.14419    123.67722    123.69101    121.27958    123.08222    125.36143    122.34601    126.61271    129.95794    125.39835    121.30171    118.81229    114.51855     114.8464    119.07683    119.14241    121.02585    122.16548    118.28119    117.29548    119.59195    124.35083    127.58177    129.38339    131.72454    127.23346    121.65278    120.11543    117.68654    122.30632    127.87215      133.793    132.68368    136.34592    129.69781    126.65313    123.59729    122.77644    118.61163    117.37067    120.52523    125.24822     128.5888     142.1617    148.81168    156.67594    160.94835     154.8059    149.62315    137.85042     129.7222    126.19738     124.3893    123.45874    119.15413    118.66474    121.39402    119.10213    117.63509    114.58998    119.27575    114.46583    116.40106    120.13489    117.97284    119.81467    120.46397    121.78018    122.70638    122.13129    121.00426    123.48142     129.5981    124.87746    124.80211    123.73567    125.49279    120.14852    126.27644    122.68131    124.63041    123.44462    126.18243    130.57058    128.26038    129.60343    130.21306    129.41219    127.78615    126.25725     120.8148    121.18599    119.90979     122.7539    119.83318    119.82653    123.20875    120.24402    124.18317    123.93527    127.66246    128.03668    123.62583    125.41715    129.31035    131.90799    131.27515    132.13814    127.61794    125.50361    123.94363     122.7115    127.38621    124.97278    125.48386    130.72433    123.09055    127.19016    126.90152    123.81416     122.7868     120.5998    118.46228    118.87657    120.65265     120.5856    121.51241    121.93911    126.35401    122.06199    124.71905     126.7008    128.22061    129.13725     131.8736    131.68819    130.87644    131.34063    132.87318      136.365    134.43953    135.77898    134.06035    137.35936    137.00155    142.94207    139.23561    139.35396    136.44461    134.90251    139.26617    134.46945     131.5187    133.15311    134.55722    129.92478    132.73911    134.40671     133.0111    137.15327    135.33846     133.5031    134.30049    137.21467    136.06477    137.54702    136.13149    138.51373     143.7176    149.22154    146.44482    146.34822    145.55065    146.62548    139.66555     144.1728    142.20154    139.90918    138.62483     135.2794     132.3722    137.31933     130.9103    136.09202    139.60386    138.53849    136.47601    134.69234    136.94339    136.43555     138.2796    136.07551    139.43514    134.27868     134.1801    133.06511    135.88016    134.52063    133.03115    130.91742    136.68773    137.18883    135.06115    133.37558    136.99916    131.43214    132.03913    131.92544    134.18138     128.0084    129.95708    133.95635    127.39573     130.1253    128.50698     128.6019    129.68722    129.78551    131.38945    132.78062    132.50298    130.39629    129.97371    131.80838    131.93136    127.23944    129.71021    132.79351    128.91835    129.14577    129.37064    130.23569    129.59965    130.16341    134.29615    133.04947    133.33684    131.17898    135.78012    135.75894    134.26758     132.5809    139.83726    136.76714    136.89775    143.78431    142.45769    142.38748    147.15313    144.37791    142.43319    144.20205    144.64598    145.08495    146.71597    146.80897    148.34678    154.44033    147.82374    150.75175    154.42532    150.41309    152.77565    155.66033    153.58589    158.32604    157.91443    158.96257     156.6368     157.2563    158.65757    163.77221     156.9047    158.49933    155.69674    158.22357    160.77256    158.74695    158.79596    165.92096    161.82845    158.39456    155.55069    163.80308    172.82942    165.27676     162.7365    162.82595    158.76265    168.36193    175.84229    163.23978    171.57512    167.83786    169.84977    177.38437    176.04331     175.0595    174.00172    173.48402    170.93003    177.30825    182.48981    180.67211     179.5044    184.37442    184.79207    179.89534    186.29524    184.83919    187.61814    188.54973    190.67029    190.16799    199.32574    195.77073    193.62461    200.87855    196.55195    200.35247    199.38751    195.54809    208.04786    200.63492    203.34255    206.03262    197.57178    213.17414    213.95927    216.10038    199.22554    215.55512    211.38897    212.32566    210.49995    217.82808    210.66489    207.39464    204.92963    216.53905    234.62161    219.75441    222.25193    223.41185    220.51689    224.87977    245.03907    241.39778    244.15107    239.04028    226.16406    228.26481    240.59201    235.80679    246.27745    250.79558    250.94171    252.52651    263.81119    262.73446    260.85118    260.80599    267.18749    263.51611    288.27508    294.97513    294.08954    295.34177    276.89004    284.60525    264.82069    314.80049    334.83962    347.75978    308.55709    312.84144    314.89276    369.15669    372.44202    351.25408    353.14706    376.02492    358.91843    403.29678    428.11857    468.27361    405.00724     488.1298    492.24478    500.28406    711.80431    653.14409    519.50116    553.94992    544.28221          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       13.00    263.98777    247.01288    232.23638    226.82767    221.85562    219.40656      219.828    214.97363    197.09014    200.14251    196.09466    193.99576      186.573    186.53691    179.40219    172.54606       174.15    175.55177    172.92951    173.97327    172.91517    165.54267    160.81362    167.52755    170.14336    182.04038    197.01537     208.6968    195.22288    173.02139    160.85482    161.84913    154.65908    158.59909    146.91111    148.34551    148.82351    151.56361    144.70979    147.18909    142.66879    140.20719    143.37619    143.81788      147.542    145.50818    142.94932    137.79178    138.01892    137.86068    141.32199    137.89281    141.01598    137.83897    137.42009    143.15695    153.75536    172.78812    180.81394    177.55507    161.73217    143.68371    136.87412    134.82268    135.10846    129.56522     132.1435    132.34683    133.39728    138.50297    146.65941    159.29583     170.9197    168.07068    159.25473    155.72965    163.69205      174.486    177.29728    170.36289    155.25368    144.53252    145.43741    142.10746     138.7896    132.56927    129.27705    125.04358    128.89681    129.43938    132.47356    135.85586    138.43717    134.09312    126.74175    121.17362    121.75318    124.35341    123.03984    122.47239    122.56994    127.44811    126.51484    124.93987    122.73654    119.67154    119.83478    118.81447    117.68974    117.97303    118.84174    119.99344     115.1347    115.84751    121.71815     125.4182    124.48554    130.74704    126.77467    126.13504    120.19808    120.04134    121.85214    125.31334     124.8414    134.64326    132.62335    136.14523    132.76194    125.42338    122.80689     119.0499    118.01178    119.69326    118.96884    123.70194    129.88305    141.40215    151.15369    154.10699    161.81764    157.67342    151.91872     140.2204    131.14381    127.95326    124.10447    125.15586    123.74071    125.36676    118.32742    120.48699    114.66322    117.91017    116.72059    118.35278    116.64377    117.13912    119.43749    118.95459    121.60083    120.10536    121.83147    120.46236    123.08246    123.08499    128.68523    125.98357     126.9631    126.73628    123.75641    121.96067    119.33006    122.57144     123.8487    123.37317    128.85882    129.74331    128.87866    129.96607    128.22898    128.71379    125.90189    124.48814    124.13644    117.70141    115.61213    116.86483    121.32718    117.50611    121.32748    121.10935    123.40047     122.6091    125.05634    126.13074    127.06396    125.52673    124.15759    130.10915     129.1855    130.30804    126.42431    127.25103    122.45401    123.83761    125.94336    123.52753    127.27586    125.71148    124.01816     129.1952     124.7782    126.76039    122.91953    122.14456    120.21027    118.86367    118.53269    123.38233    120.32386    122.45096    123.05002    122.41221    126.83907    126.47156    131.02923    125.37734    125.84697    130.26136    130.38001    134.43884    131.53034    132.11864    135.01109    135.40096    136.25174    134.50671    136.33575    137.48622    137.61051    137.38288     133.7544    131.62622    133.43839    133.31189    134.56935    135.91191    130.35197    131.46587    132.99705    131.84407    132.64212    132.53354    139.00723    133.58691    135.94642    137.07505    142.51381    135.00608    136.14078    136.91395    141.06269    146.71857    145.11978    147.30667    147.37334    145.47808    145.94284    144.20485    141.94381    138.28352    136.41892    137.46274    134.98991    134.46769    132.57216    134.17131    132.97687    133.75196    134.86847    135.52383    138.18384    136.87835    139.10663    133.37691    135.42784    131.51857     133.4955    131.72498    133.12521    130.46749    131.25765    130.84016    132.08182    134.42254    131.64878      130.941    131.68703    135.77946    125.91038    131.92282    131.97755    132.34531    130.49704    132.79835    127.89711    128.80702     131.1264    130.94174    127.95088    129.89742    128.60084     133.5562    132.45873    128.91211    132.11246     133.0744    131.03559    134.39268    131.62124    133.77964    130.08556    130.40234    122.88634    130.20235    128.73813    132.04946    128.29225    136.03672    133.82146    132.07043    136.52423    136.89833    138.02131    135.53584    134.62046    136.62378    137.66286    139.80602     140.8746    137.95683    138.60975    143.71899    137.15753    146.66816    138.87617    150.09201    147.86524    142.00339    147.23601     147.9213    148.23991    149.27655    155.44236    152.61601    152.55279    153.87524    156.62762    159.89484     161.7741    158.58643     156.4395    156.50774    154.84567     160.3581    162.52476    160.93242    160.20988     154.9094     155.0831    156.35559    158.55901    159.81704    162.97214    159.50841    154.94959    167.94629    165.69734    164.45885    163.02387    170.11114    156.97296    167.95198    168.14366    172.74333    171.51062    177.34309    172.41668    173.79294    177.12733    171.61474    175.91737    176.99961    170.42955    176.41926    182.43601    183.44946     181.4815    181.19675    178.77926    180.80264    192.04191     189.0898    187.59901    186.86318    185.97141    186.32384    198.97547    203.12217    196.01654        195.6    202.83352    195.03022    193.29193    211.82505    199.13657     200.4379    196.91892    206.08647    197.26323    214.46359    216.81066    213.09226     211.9599    213.80322     208.7153    212.80456    226.00649    218.07076    205.43511     205.9316    204.78659    214.31707    211.44815    219.43025    215.71509    212.76762    218.13848    222.46749    225.31888    229.45619    242.04619    220.74348    240.18178    246.77488    243.89954    242.16225    239.88379    237.32563    249.64458    247.08265     265.4102    258.84677    248.45571    259.91045    269.44145    268.30938    267.12966    284.75222    273.40342    291.83003    273.76675     312.3091    292.97525     301.5516    313.85978    330.95523    317.23469    313.69684    328.15297    355.66223    386.32252    399.10402    385.72787    379.81029    390.12647    413.22983    473.47382    482.00448    440.65553     547.5334    455.69473    647.93258    562.11809    653.38143    541.80156    461.24435    726.75148          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       13.50    268.42906    247.00777    231.41602     223.2848    218.81768    221.13833    221.34124    215.63099     198.3975    197.12513    195.65133    192.33045    185.61752    189.35991    178.36105    173.06832    178.00982    171.98663    176.78701    171.85639    171.48705    165.45437    169.04332    168.77602    172.41705    180.68046    203.66859    208.90432    194.92523    173.96397    159.58717    161.21958     155.1525    155.04093    147.85971    148.43966    149.35685    154.86443    147.43361    150.49054    147.88346    143.84943    140.48545    139.67031    143.90418    140.68203    142.91831    140.27634    139.02781    137.21653    143.74906    144.90732    139.96619    138.94224    137.16239    142.14332    153.66409    172.76857    179.68781    179.16799    159.70034    144.42614    137.40138    137.46341     133.0237    130.43704    136.04795    134.98726    129.55895     140.5242    147.28542    161.86127    168.06948    166.74874    155.22743    161.07684    168.53117    181.22997    178.17034    171.07354    155.23604    147.66023    142.94595    142.96651    138.86707    135.80555    125.44024    127.41252    126.40601    130.02643    133.65688    138.13059    138.70122    131.96981    127.91977    127.66691    121.53007    121.89856      124.354    123.44313    121.82237    127.52026    126.07653    125.39606    119.70426    120.41685    118.58484    114.50879      120.259    119.30392    119.49891    118.86656    118.07846    117.02558    120.78738    122.08349     128.8383    132.65769    128.65051    129.08021      121.975    116.66743     119.5354    122.89223    127.73249    135.45072    137.70098    135.69451    132.38912    128.15416    121.22835    120.62131    118.52176    115.46936    118.46413    123.00685     131.1552    138.51295    149.35477    157.12183    160.27394     154.3183    144.87691    142.39178    136.31528    129.66529    126.12051    127.38647    124.87788     128.2809     123.9238     120.1324    116.10207    119.78014    119.32592    119.68106    121.63016    118.39509    120.29309    120.22841    120.59573    118.08951    120.63525    121.49138    124.95505    121.55245     124.9292    128.43925    127.63055     123.5452    121.48866    124.46509    120.20757    125.01639    123.09378    124.66915    126.91505     126.3288    128.72269      127.688    130.48098    127.32917    125.25969    124.57238    122.66308    116.16991    121.06804    119.31555    117.84928    120.17024    119.42558    119.53176    119.90127    125.48584    122.17855     125.3071    124.39673    128.19033    127.49116    128.42681    130.02821    129.29082    127.57228    126.60208    122.74043    125.10139    122.97319    126.62012    126.38137    128.83915    126.72984    125.64478    129.50744     122.7825    123.65922    123.82091    118.96583    118.43115    125.51773    119.43873    120.74655    118.55034    122.11812    123.82436    124.38157    127.73608    128.83004    130.00429    130.03566     128.8287    126.16238    131.43116    130.68022    130.82696    131.49234    132.47372    130.95843    129.33616    135.82793    136.70525    138.34367    133.96127    133.17255    134.85287    134.44572    135.14722    135.07405    133.37809    133.28118      132.417    130.18208    130.16829     134.1977    135.31314    133.91418    133.80942    134.71863    137.31866    134.90845     136.3702    138.64926    142.71734    133.92637    145.76164    146.35809    145.65996    147.07742    147.93707    143.80098    145.26118      142.861    138.73655    137.86416    137.42047    130.33077    134.27374     134.3072    138.15171    137.95627     138.2764    138.32733    137.14302    140.54785    137.59396    136.42343    137.69757    137.87071    132.68227    134.06466    136.08195    131.64805     130.5646    127.77835    131.73466    129.10386    130.18228    129.06705    134.56235    130.84095    129.57703    130.48328    128.08943     128.5544    130.69001    125.94024    130.51938    132.62464    125.91505    131.37203    127.00331    132.41416    132.10063    128.61121    131.42706    131.36616     133.1189    131.31415    127.89201    130.34877    129.37886    131.34373     130.0468    128.20222    129.33521    128.86883     124.1781    128.25825    132.25687    133.58602    132.85482    133.15261    137.31016    134.71319    141.98842    136.45232    139.75585    140.25419    139.23774    139.82651    141.66129    141.36602    139.49621    139.78939    138.46011    141.58277    141.39223    143.58638    145.06813    143.63177    142.93951    146.60758    144.25595    152.55462    150.60463    148.80828    153.39055    150.87936    154.73198    158.16754    158.31939    158.90806    157.24139    158.56231    153.36267    162.20951    158.78773    157.66582    158.79453    159.52508    159.42623     152.2465    162.97405    160.53769    161.87817     156.2176    159.32525     155.6371    164.02219    163.01365    160.56326    167.97146    162.69504    159.05942    168.50873    170.37953    172.50074    163.74035    167.31774    163.77038    174.18146    179.89011    181.29132    177.01288    180.62541    168.02231    172.97308    186.25316     191.7247    185.33134    181.45253    180.18494    180.11848     182.0892    191.55513    188.71229    187.64272    186.08556    180.06972    195.10637    205.86497    194.76341    198.19927    197.98701    195.61322    193.30205    205.10854    198.85067     209.0607    203.50709    202.40889    205.43293    211.54556    213.11014    208.58213    197.19172    204.86371    207.89071    218.96993    211.61744    211.96771    203.06051    207.51824    213.84238    216.47599    230.01426    210.45564     222.3285     226.3133    222.82459    225.62858     227.4957    222.53094    228.95511    243.14355      234.959     228.8823    245.04909    230.60625    254.87738    245.18547    246.13932    246.85308    258.93882    269.29014    257.73591    263.10177    257.38431    264.79794     291.5903    287.99304     273.9086    282.14023    275.68133    269.50475    296.25731    308.88746    338.71539    312.34754    331.42419    341.49357    296.62808    358.45393    369.73338    380.38839    388.47114    382.68999    343.27114    474.95048    395.74186    418.91054    492.63944    488.00368    400.74179    606.07147    614.24352    549.62206    582.56725    541.04567    586.45432          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       14.00    272.77445    247.38864    231.13169    223.94556    216.95885    222.06017    223.20543     212.2711    200.83101    194.64627    191.35848    191.23564    187.01947    190.25674    179.60809    174.07084    176.68096    166.76543    173.17883    173.87198    172.51141    165.37715    167.06531    171.66985    172.09685    183.99994      204.155     208.0112    196.41412     173.8167    159.04948    154.59894    158.24542    152.61011    151.70708    148.97422    149.98592    148.62602    147.69867    150.65931    146.88831     145.3902     145.4741     142.1821    144.57235    140.95704    135.13559    141.08769    137.96954    139.78111    140.46042    140.99485    143.16527    143.64282    136.83688    135.72034    150.31467    171.15525    180.96076    179.22608    157.37053    141.64694    137.71942    139.66322    133.76363    134.43091      132.842    132.80077    135.59699    138.05163    148.97123    160.68276    166.80998    166.74704    158.30333    158.57566    174.16432    194.16345      195.226    184.23673    166.63306    150.86281    146.17551    145.34404    138.10905    132.45207    130.37439    125.73496    129.86702     127.3384    133.84054    131.96251    141.31621    134.20768    127.72804    121.98506    125.33974    122.85206    124.63629    124.85254     119.8483    127.48324    124.22337    125.52626    122.48194    123.76184    120.71736    116.25571    119.78095    120.74682    118.50968    118.31446    120.12764    119.36598     114.0223     122.8093    124.94166    129.85911    130.12183       127.33    119.93809    120.83176    119.71147    125.49867    125.61814    132.92614    136.48992     131.8113    130.47726    125.99574    122.23446    118.27822    120.35153    117.60886    119.34678    120.84995    128.79126    139.44116    152.65068    156.91474    161.06436    155.75976    147.45275    139.15046     129.6549    128.00419    126.05599    128.53544    126.84228    125.68948     126.2794    121.39514    121.03797    118.35842    118.79882    120.13781    119.39637    118.25163     117.4123    120.75277    118.02782    120.17238    120.11155    123.07768    122.86969    122.97817    124.70529    128.92283    128.73947    127.63815    123.34191    124.80158     121.5059    123.57073    125.50522    128.58198    127.34767    131.57514    127.95179    123.55302    125.85288    128.12659    122.47194    120.50646    121.49212    117.08041    118.78282    118.70238    121.24679    121.73691    118.89348    119.16726    122.86577    122.01688    123.32566    126.56613    126.15674    125.74146    126.70123    127.57055    132.42206       128.43    128.00669    123.87257    125.10311    122.31544    125.79239    126.10205    128.87677    127.71396    126.20034    123.01073    125.80166    123.09994    121.91902    119.74102    120.16777     119.5662    121.66309    120.97821    122.61317    122.26887    119.94449    123.82182     119.2251    126.22594    126.83211    126.13732      133.387    133.31283    127.40048    131.42394    128.64142    129.38071    129.25881    131.96334    134.90346    131.22222    135.09309     134.8216    133.22784     136.6734    131.06087    130.44892    134.70504    134.74828    133.42862    135.57089    133.10712     135.0445    128.77329    132.45747    132.90697    137.07136    136.94287    136.70077    133.79612    134.08234    138.86315    139.70107    138.20154    139.99468    139.15821    140.35234    144.06969    147.23516    149.07228    144.92718    142.40451    142.20474     141.2276    139.69648    136.92444    136.14735     135.2458    129.22483    132.97131    135.07389    136.95088    137.90604    136.47865    137.51134    138.48323    137.31336    136.71334    134.98521    136.14158    128.81891    127.58616    132.76819    133.89272    129.17504    130.64912     129.0872    126.75195     132.7183    131.63452    127.95975    131.12457    131.74582    130.49842    131.60145    129.45354    130.61404    130.01959    130.13932    129.45513     127.5655    132.32114    130.75364    129.61925    132.34039    130.69954    126.95902    133.89486    131.35697    128.01717    130.63088    130.82591    128.16376    128.89567    132.42395    124.82802    130.97906    128.26145    129.86119     125.6496    129.34588    129.86525     130.0084    132.33057    133.85802    134.33641    131.42723    138.34031    138.40939    140.28813    141.58183    139.01449    139.09759    136.91796    138.70437     141.8761    141.18829     139.8364    139.55096    144.98959    142.84203    145.07043    143.72142    146.31996    146.83271    147.08795    152.03879    146.43621    147.77645    150.38977    148.45857    155.64823    159.56662    165.54289    153.63883    158.84302    159.55968    164.62639    162.31074    163.76603    163.08633    157.43743    153.51784    160.63616    157.64052     162.7667    156.25215    162.54148    154.61401    162.31147     163.1359    164.71645    159.79963     167.5008    159.78976    165.23557    164.35857    166.50134    168.09738    167.09256    167.35796    165.71628    171.04319    167.05396    172.00011    170.97211    175.37114    174.54984    176.14253     175.1409    172.66389    185.06538    169.37047    178.35142    181.56996    183.70733    185.12373    183.04733    184.85575    191.63296    186.37597    191.13764    190.81395    185.46226    203.40164    197.10713    187.75365    197.02288    207.04047    194.34669    191.88776    203.57113    199.70036    202.76051    209.65004    209.10445    211.49961    212.95673    209.31348     203.1206     208.1089    224.63304    216.83224    217.65795    214.77175    210.77696    216.65907    219.33365    218.73715    232.14659    223.16166    218.58835    226.51813    230.97859    222.69145    236.29843    243.50601    240.35046    233.70332    244.91124    244.63198    248.69112    244.51781    244.03842    243.44436    253.07506    260.82053    256.66457     259.6757    263.02004    275.02165    278.80692    278.31591    282.79066    279.73461    296.74311    298.14017    312.50569    308.97016    329.71718    310.24659    326.43466    309.38716    318.48648    374.75122    355.01572     367.5159    354.58585    332.09721    404.78298    418.68419    436.57952    450.46221    433.27348    398.77531    462.31162    538.65594    585.30334     559.4009    509.50854    598.27962    502.82507          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       14.50    273.76018    247.86849     235.3379    224.89035    220.44358    222.07074    222.15666    207.29849     202.1581    195.42469    190.35894    189.07329    189.48586    187.23475    181.28069     176.3343    175.10805    175.93684    172.77138    173.27747    174.03255     166.1457    164.56199     167.5557    171.51198    179.67417    202.31744    207.87428    197.71261    175.74024    160.55488    153.90315     155.3278    149.31038    151.99306    145.74112    149.56665    149.45394    148.68872    147.73669    151.79228    146.27162    147.54301    142.12398    144.62929    143.27227    146.29656    144.42247    142.33857    138.60212    139.10839    142.15847    140.30326    141.81639    136.47746    141.12266    150.21503    168.84372    180.22436    175.28031    160.79451    143.47285    138.10168    137.33032    134.86119    130.33818    135.10018    133.19167    133.18564    140.16578     148.8079     165.7329    170.08325    163.20428    157.21316    160.97167    181.19316    208.63632    215.67342    204.76327    181.90112    158.57891    149.05207    141.19662    137.48961    132.58093    128.23302    122.19094    125.08036    131.69681     135.5397     140.4913    139.33562    134.26581    129.32318    126.02434    127.31261    123.66149    128.54805    125.64655    124.54046    125.27137    128.55737    125.22711      120.955    122.02539    119.15312    116.11422    119.38665    120.78919    121.77085    124.25161    118.68278    118.00367    118.50047    124.14092    127.04113    130.69898    126.60508    125.95377    123.32768    119.98882    123.56733      122.519    128.11683    131.70934    134.50005    130.56288    133.58441    126.39026    123.09709    118.50326    119.62109      117.114    117.56683    121.80255    129.46022    140.77556    148.68617    162.17802    154.84655    156.73958    147.65192    137.36608    131.01118    129.39418    123.47425    121.76856    124.34896    125.42633    124.03188    118.47982    119.09117    115.80474    116.64933    115.77173    116.14228    117.77178    117.52114    117.64019    118.94747    121.08176    120.77731    120.71303    124.50599    124.33829    124.75507    132.52902    132.92877    131.06679    129.93789    126.71702    124.67685    126.00536    122.10255    125.82358    129.39861    130.12508    127.67647    126.43775    126.43108    124.68412    123.10893    120.45272    124.47453    119.54465    120.90491     117.7143    119.41253    118.55532    120.06813    121.02548    117.38789    124.45119    127.61161    127.98891    126.53306    126.92482     126.5842    131.32289    129.69039     129.3759    129.36804    126.02294    123.82241     116.5355    124.32852    124.05399    129.94712    128.27154     125.5197    123.31314    125.35913    119.19545    126.49106    120.07349    121.85398    122.23433    121.96756    121.83687    119.67151    119.64856    123.41716    123.21826    125.51839    126.89301    125.13654    127.80274    124.75033    129.01413    129.77205    128.84001    131.26631    128.09512    129.86749    131.54474    129.51066     129.9387    135.35614    133.54549    135.48604    131.21415    133.70884     132.0098     131.3381    131.58557    133.40147     133.0262     134.5602    137.60862    135.63453    137.07211    134.68439    133.27849    137.22524    134.50385     132.9184    135.79885    135.49222    137.62104    135.01535    135.79909    136.80021    142.28398    144.60521    144.62731    145.03324    143.06509    139.29164     140.0587    139.04675     134.6422    134.28985    136.55009    135.30899    133.29526    134.03913    134.14613    138.76724     134.0392    135.41596    135.41074    132.63664    138.81288    137.37358    136.62663     135.7712    130.14837    130.00421    132.12285    129.95344    131.28485    134.21724    131.57299    128.68271    128.36959    130.41906    125.57605    129.46721    126.29548    129.38376    129.42364    129.14249    129.02413    126.34266    129.81537    125.94043    127.53824    129.94937    132.88782     129.2721    130.18765    131.19152    129.05583    131.34403    130.98185    132.07244    129.97515    131.10719     130.2974    129.60826    129.99302    133.60937    128.98016    129.94784    127.96562    124.93246    134.54305    128.78944    130.13026    130.50869    136.56725    131.29145     136.8341    135.66226    138.02001    136.84327    137.14598    140.25483    142.54758    136.77041    138.91316    139.56272    144.37114     137.6767    144.85228    145.12044    142.51941    144.20508     144.5817    140.03236    142.60334    149.57364    150.22204    149.35395     145.4998    152.16731    150.14761    158.48113    157.68685    154.27973    153.91927    151.63876    156.67759    156.42653    160.75136    160.59439    158.56182    155.77461    154.34804    155.94494    160.96361    161.90585    159.98293    162.80673     153.8106    158.17047    162.99362    153.44182    164.01542    172.69055    156.34854    161.36941    161.49961    167.09916    167.14178    168.74304    172.20522    168.03036    169.98424    172.86756    170.04022    178.37059    169.23412    164.03932    172.31279    183.53874    183.64083    180.14648    185.38662    178.51072    181.26586    184.48226    188.02161    183.33875    184.01377    182.55183    187.58016     198.8181    193.31964    197.13653    195.33475    197.05338     196.9237     201.8393    199.93867    203.76744    188.79958    201.78263     210.8034    203.90802    211.69042    216.73741    202.44565    210.54381    213.20442    193.33713    228.03007    228.79609    218.63658    221.43278    211.43039    214.69612    222.14109    219.13221    217.51386    219.59647     219.1475    211.80859     229.0757    225.70091     227.5674    246.67688     240.7885    220.53318    227.77687    255.75832     247.9435    246.42335    247.50213    241.69392    244.28493    260.20992    251.07046    247.46021    272.51803    257.79925    261.53068    282.07442    286.01916    283.46963    283.52992    302.37165    311.86085    321.34941    321.14246     326.2926    331.29205    296.17928    335.55306    343.51013    382.11917    378.49727    364.11189    377.38264    366.89336    331.47534    488.80897    446.82709    431.45193    446.69215    471.73808     427.8876    575.33822    576.76881    573.13786    581.06824    500.26476    535.24888          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       15.00    272.13257     248.2349    241.08052    225.24899    225.66851    221.00571    216.54283    205.49624    200.98975    195.72459    192.16662    187.08791    193.24971    184.70113    182.04185    177.96278    174.27059    178.88772    176.62861    172.32953    174.89609    166.43716    165.62203     163.4245    172.76371    179.12741    200.13105    211.81983    198.35307    175.49822    161.46072    154.85551    155.53165    154.62578    150.66056    149.18544    152.62232      146.611    147.67433      144.425    153.15522    145.54603    145.24596    141.76889    148.08885    144.57516    141.13634    143.55173    138.29767    141.98094    140.64369    138.08289    140.85032    141.77243    142.28946    140.97619    153.62627    167.33241    178.10807    173.41962    161.86613    140.99512    139.95613    139.72441    133.75832    128.05182    134.51504    133.64174    134.93936    141.46231    151.03673    161.28566    168.90929    165.69426    160.10099    163.26474    186.53978    221.53961    229.81681    222.56379    199.19062    170.42832    152.97896    148.15853    139.86532    136.29436    129.05849    128.74684    124.71662    130.79916    135.84072    137.34157    138.65008    137.46752    130.47253    126.07189    133.05105    129.59499    128.64896    125.51681    127.30488    125.08069    127.06322    125.27204    124.50948    118.96099    117.96452    117.51669    115.22137    117.21097     118.7243    119.47392    119.21886    118.36676     117.0169    121.27766    126.35663    127.20436    125.57894    124.95376    122.25383    117.07648    118.29023    119.85607    127.93257    132.50331    134.33534    131.20746    130.47612    126.74975    121.37779    123.18514    117.65937     118.0712    119.85489    121.71925    130.44907    140.21198     154.9839    160.14062    160.53017    160.98872    149.16599     139.1463    130.73467    128.48184    123.18914     121.3479    126.69644    121.66169    121.27572    117.82878    118.63193    113.23585    115.52861    111.48324    115.46458    122.99786    115.89005    117.71078      119.545    120.12557     117.5114    122.82147    120.95645    124.22876    127.93588    134.67013    138.60288    139.24532    134.59487    126.78377    124.95363    123.94935    126.12102    128.92043    129.58517    127.76709    128.14364    124.76124    127.92645     122.5554     123.5671    122.14743    120.13977    118.77428    120.64651    119.64551    118.48556    118.35207    121.85527     121.2952    123.29085    123.92042    122.86666    125.40334    125.10117    129.09997    126.07885    129.04162    130.71803    130.04059    126.74151    125.18652    123.17785    118.52322    123.12127    122.07254    126.62489    125.76745    128.88116    124.59959    123.94261    125.93434    123.65967     122.4998    122.63479    118.84245    117.58292    124.15379    120.95974    120.64847     122.6445    120.91201    121.97281    123.71789    126.50017    126.03637    128.23961    127.87766    129.46517    126.77305    131.36854    129.96959    130.01548    134.09551    131.84587    132.04991    134.40303    137.45375    132.70327    135.95377    132.64265     133.9068    129.04611    134.71617     132.7835    133.87328    133.02309    137.52837    135.75664    135.08901     135.9285    136.66273    133.40395    134.67813    132.76161    132.93855    135.86584    132.64284    136.18118    135.63903    140.46898    140.74063    145.38827    147.20793    143.59869    147.31682    144.17839    136.57293    138.00106     135.7419    135.72427    133.89646     135.3262    135.05328    133.14763    134.86889     130.8861    133.53648    134.60316    134.02327    130.31563    135.42085    134.61012    136.91682    131.67754    130.67306    132.32355    132.12423    132.28278    131.70008    132.72198    131.68113    128.40996    123.93078    128.52397    126.17205    128.90991    129.95536    126.11878    127.43702    128.66255    130.63965     128.2435    125.99602    129.24991    126.32486    128.57367    128.43704     127.6372    130.43219    130.48475    129.75455    129.14291     125.7013    133.28876    129.13324    129.45662    127.29053    131.11361    131.54042    135.24802    133.99497    130.11729     129.8659    131.30096    130.89371    129.88262    127.05196    129.53298    132.92121    135.85374    135.27466    137.73087    138.24961    137.37938    137.64797    135.50433    137.71826    143.93928    140.94625    141.18652    138.53561    142.10892    139.14394    145.38342    145.71587     139.0043    148.53669    146.56555    142.85683    149.73157    147.47714    146.55737    155.45568    148.70089    150.46819    150.17421     151.6767    158.65156    155.45838    154.17103    155.78325    154.72554    160.08183    161.39636    153.23634    157.30491     158.0093    161.49734    157.51415    160.79899    157.83609    160.95798    159.60475    163.63362    164.93981    165.48937    161.25207    165.19259    162.06501    161.18312    161.45869    167.02012    175.34904    171.97732    170.67371     167.6632    166.04898    173.76453    175.14652    172.01941    166.97479    175.96522     172.4902    183.12218    181.58887    179.99332    182.63891    177.55365     174.3127    182.82228    186.78604    186.33657    190.98914    186.07224    188.53852      193.752     197.4756    186.66028    187.00195    192.37884    192.20643    191.83352    196.68201     205.9878    201.61544    205.31665    205.84239    203.65944    204.25441    215.80971    213.74485    204.01735    200.46146    199.46252    212.47032    218.19596    200.87517    213.57134    215.63142    217.24306    215.30341     211.9276     219.7379    216.19904    225.61155    210.61231    228.46825     254.1739    233.13086    234.49231    234.21403    236.60853    243.56949    260.02795    272.23249    241.82837    239.86612    259.22501     257.4032    268.60055    267.19184    257.07977    259.81878    257.84414    277.17257    278.41769    290.30222    277.34445    271.10924    297.72144    299.03512    299.86335    320.79893    312.14346    331.50584    293.37654    331.02247    340.32825    379.39157    387.17005    364.21708    357.49826    371.67834    336.79042    502.97395    422.46261    385.15288    416.67557    419.28737    435.05837    619.00551    638.74964    610.99213    689.95156    458.37662    671.70916          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       15.50    271.24817    247.75137    245.46149    225.93344    228.24697    219.59872    211.84003    205.72281    199.15464    194.40725    192.83147    187.25769    193.42147    182.73802    185.47086    179.26306     175.3747    178.69738    176.15847    168.94277    170.25225    166.01359    169.61163    161.80215    169.56057     181.8378    201.00974    213.20207    193.98047    174.91722    163.56314    156.58993    155.06167    155.79166     152.1832    147.10348    147.92228    148.11351    148.76965    146.28947    149.46332    143.28013    146.81227     141.1171    143.70991     141.3194    143.69687    142.10257    140.19464    139.32583    135.66292    140.23779    139.72071    141.94605    138.12723      145.008    154.50675    172.25661    182.82409    173.95507    162.03153    146.83825    140.78802    135.76278    137.05464    134.01883    131.80583    137.60429    132.87441    139.89739    150.50235    160.22408    167.31043     168.6431     159.4764    160.51395    191.08553    221.64039    242.30548    234.66057    209.71355    178.42743    161.12824      146.576    142.42095    133.09839    129.96275    130.27714    126.16954    127.19602    131.50695    139.06886    143.89909    133.84187    131.61334    129.03311    127.42907    128.94485    131.53754    125.94872    127.50952    124.54876    125.14551    125.68018    122.76847    119.87142    124.31678    121.42543    117.84548    120.85049    116.82759    119.45376    117.37106    116.12914    113.77554    122.83323     126.0134    130.49258    129.05138    126.00561    120.85343    119.57676    121.83361    122.30117    125.44892    130.34401    136.38755    131.06306    127.77189    127.63936    122.83775    116.29786    121.77118    114.79072    120.13716    123.15264    131.92488    140.71338    153.55868    160.08783    163.46045    162.09867    149.93057    138.66438    129.15708    125.54388    124.50231     121.0739    120.92695     116.9001    117.95159    119.03946    116.86499    114.40551    114.52207    114.55993    119.09162    119.30271    119.58892    119.89699    117.82902    118.45166    118.58313    120.69339    124.07091     124.9521    123.81908    130.89601    135.22446    137.70499     134.8312    130.52596    129.12653    122.07141    124.99253    128.54433     129.5337     132.0566    128.66587    128.02379    123.05689    126.21878    129.06886    124.49589    122.12964    120.90421    119.50256    120.31528    119.15797    118.90712    121.90747    118.66565    124.33047    121.82624    125.36042    125.05474    125.71117    127.64387    129.92155    127.55198    130.53398    130.17155    127.27534     123.6583    126.56294    120.11385    120.84465     124.1445    127.54993    125.30824    128.37594    126.92544    120.43199     125.7276    122.76132    119.91542     119.5193    118.85716    119.22423    120.81659    121.33717    120.65647    119.92314    120.53792    121.79182     121.9693    128.92941     129.7692    131.32071    129.44374    129.53638    124.54557    127.20464    128.42292    132.22749    130.69798    137.11693     131.8421    134.77327    134.05364    135.45387    137.13583    134.19665    136.12436    133.64859    133.16055    133.22711    130.44943    131.52386    132.28595     135.3314    126.84234    133.98114    131.53986    134.68815     133.3555    135.35537    133.44684    135.70123    134.99821    137.29456    136.13986    142.33718    140.08558    139.97356    145.17717    145.30964    142.49022    141.97604    138.50053    136.52783    138.03553    133.52987    135.32749    133.11343    136.19207    127.87117    135.79854    133.91658    134.96463    139.35148    137.23061    131.87823    133.41628    133.40327    136.04004    135.07944    130.37112    133.52315    130.78158    133.58611    132.16804    126.39008    128.79901    129.57836    126.34027    129.92342    129.47687    128.10697    126.19628    123.20739    125.38115    126.65154    130.56739    129.81827    130.80475     128.9201    128.55593    126.11594    129.57736    124.72829    130.23266    129.90363    129.34042    127.17611    129.88408    128.32099    133.96239    129.79752    128.97441     126.7754    130.42425    128.95157    129.51374    127.38314    125.62551    131.55415    129.51584    126.49105    129.04353      130.466    129.97685     133.1736    137.26416     135.1954    138.77123    135.86297    136.04929    139.94929    134.52186     142.3052    141.84618    135.24375    134.41564    139.17022    140.70194    139.17481    143.55966    139.93948    144.49727    146.30646    146.77329     143.1613    148.16726     145.8121    147.78243    150.65113    158.74145    151.96993    152.70421    152.35022     156.3866    158.08195    154.95666    155.75368    151.04545    154.47536    158.25073    150.50512    160.00349    153.83821    156.41287    151.56624    158.86683    162.19069    161.10083     161.5461    160.84136      166.017    164.05482    165.88813     168.1661    163.17106    161.29919    171.54523    166.77957    173.08057     169.6816    167.63369    171.31579    174.56866    171.22249    174.30599    175.65868    173.49593    171.79243    183.69759    183.34637    181.48945     176.9523    183.61357    177.65312    188.39299    185.42976    182.68371    193.44228    183.33585    191.18586    187.87344    204.22148     194.5071    190.95652    199.25296    191.04762    204.94644    208.80233     207.8633    201.01662    199.99673    203.29387    197.50934    210.56034     213.4071    195.84815    203.10363    195.34068    217.86748     219.5173    213.85822    219.33144    215.44119    212.03451    205.21697    214.78264     212.3571    215.57025    214.13434    226.89352    212.98732    237.11432    231.05702    230.42402    239.29633    236.86451    228.18863    263.77697    253.68279    233.56599    241.77913     246.3845    253.56131    243.76522    261.20544    266.21228    238.00123    273.84815    265.81429    263.83151    298.31372     292.5566    280.13413    304.11272    296.82464    288.59507    331.37947    361.49582    302.28846    300.24372    319.56716    319.35458    327.41491    390.60834    375.79844    378.42528    391.02412    369.91977    358.10453    445.26019    502.63391     425.6073    479.68758     464.0705    510.88452    544.05726    645.82782    528.67649    640.61347     586.8574    578.88377          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       16.00    268.66222    246.82623    242.98328    227.15799    225.17421    219.51641    213.08769    206.39707    199.84586    194.76769     194.9354    189.88762    190.38391    184.58437    190.31351    180.83357    174.96237    174.00685    175.25118    169.47849    167.97553    169.68945    170.66115    158.05064    167.66596    186.01926    198.07891    209.90801    196.99798    174.89648    162.38268    159.09225    153.56057    157.97924    146.09881    147.81607    150.45835    149.33368     145.9176    146.91739    150.73394    147.88623    143.08719    140.92133    144.72735    140.82295    141.41734    146.19291     144.0617    139.44359    140.87577    145.09983    137.48931     142.3178     136.9693    140.60044    155.74606    168.89853    180.87417    174.79621    164.50724    147.77687    138.18915    135.87373    132.84852    131.16148    132.74812    135.63249    137.11592    138.78117    148.28899    159.76068    165.98294    168.31925    157.26136    165.41849    193.77987    230.10295    252.06564    244.53308    211.01609    179.05954    162.07433    149.76756    143.00026    132.09074    130.32028    127.36295    125.67321    131.57468    135.48299    140.54707    142.78225    135.98268    131.80041    129.92046    130.28707    130.78858    124.58026    128.18975    126.23472    125.46034    129.28209    125.53973    123.56662    120.50587    116.04664    119.48051    115.43401    119.83094     122.8789     117.2103    114.96281    117.57889    118.77719    120.62167    124.37703    128.91316    126.65905    126.86285    120.92949    118.73797    117.37474    122.36254    128.75678    134.42944      136.823    134.43664    129.38507    124.79009    121.65923    119.30642    117.14295    118.01135    119.56112    123.63485    129.38527    140.90072    150.25072    161.56914    161.87563    150.67091    149.16821    140.23905    131.49342    121.55839    121.79658    120.17455    117.35356    119.64287    115.85011    113.58163     119.1559    113.88666    114.07915    117.97636    116.42983    117.27671    118.87844     120.4326    120.30683    117.25093     117.9207    124.06895    123.00583     123.5223    128.06076    125.28249    129.30284    132.89273     132.6202    132.84452     133.6842    131.16028    129.00727    132.21685    134.14189    131.81047    131.04791    129.58938    127.91551    128.48296    129.68091    126.10744    122.94417    120.19099    120.87464    120.85647    120.66239    122.89112    120.93552    121.58811    127.98788    119.29039    125.77708    123.62122    125.61265    130.85204    130.25601    123.38113    130.59433    125.67914    126.77994    126.66813    123.74718    121.79669    119.79272    123.95968    127.11779    123.87176    126.74316    124.74982     121.8107    121.30156    125.76224    120.83812    123.00023    112.74833    122.51537    122.07788    118.49695    122.06564    118.34974    121.31325    126.75974    124.26436    128.07855    127.85753    126.06778    129.31055    127.95191     127.1092     124.8292    129.58727     128.8412    131.68641    130.38007    132.95522    128.52989    132.36843    133.43044    134.39246      134.029    136.59993    133.77844    132.25229    132.76698    131.52687    131.88991    131.73912    129.55917    130.43166    134.36799    135.32441    137.50444    135.74874    133.44817    133.61693    134.55568    132.69163    135.27139    135.72589    139.92221    137.77594    137.93111    139.51756    142.09645    141.96123    139.42987    138.49723    136.77783    137.14317    135.09644    135.66803    136.64026    134.64815    133.25001    136.41749     133.3601    135.38159    134.10638    133.34236    133.83987    134.38687    133.32981    133.05937    134.33076    134.46104    133.89396    128.71505    131.19691    129.82185    128.82552    126.94402    129.78526    124.37795    126.77727    127.13476    129.02857    129.07344     128.8933    127.40584    127.25025     128.3252    127.64066    128.05678    127.62622     126.3841    125.14462    124.95601    131.64559    127.52494    128.11484    126.62186     125.3245    130.69973    129.96112    128.55487    131.57316    129.31944    125.68836     126.3361    127.75993     128.5969    130.37712    126.69421    126.17267    129.23486    126.30521    131.87443    129.39801    133.44435    129.23242    130.22919    135.82775    136.40333    139.34808    138.04949    141.05901    137.48555    141.65465    139.62855    137.72975    143.15224    138.88692    138.28926    142.87795    145.71883    145.74626    146.49158    145.04525     141.6576    150.41543    150.64161    148.76418    148.82486     151.1027    153.46899    149.68989    160.86299    150.60069     158.3287    154.64342    156.50127    157.56548    154.83675    155.94949     159.5502    155.11871    156.93673    157.86981    158.48092    160.96016    162.89792    164.75607    157.91457    163.66726    162.35879    159.36379    162.05639    161.98078    159.54632    166.32467    161.39829    168.16252    169.67421    178.35436    163.15057    163.75757    171.33766    178.77575    166.61974    173.14235    168.40933    170.81441    177.91525    179.07433    180.00457    182.98535    185.47396    175.02064    177.85207    191.99933    190.55697    182.84556    189.57483    189.97113    185.53495    191.01422    187.64725    194.90506    194.69335    198.28553    192.55797    192.89857    211.55522    211.71495    207.43416    199.24817    198.16632    202.47328    209.32203    212.54676      196.895    206.16468    199.75125    200.96119    212.82599    209.77971    216.08906     211.2561    215.64562    212.67792    224.34835    235.21799    227.38134    224.85496     219.4945    224.98715    225.10054    240.06099    237.93717    221.08258    248.61786    227.28327    226.95743    259.58726    234.24493    247.26302    256.25029    239.87574    244.87594    268.03528    269.00527    257.79752    268.59277    274.06596    281.17149    295.36564    288.60314    291.34267    296.27491    295.20594    297.26785    309.36768    261.13371    347.27222    346.97198     360.0127    336.20897      370.741    388.13775    416.32554    374.03075    404.43626    350.47915    367.08807     416.2355    376.27104    460.96324    520.39625    481.90664    541.53693    607.10691    564.50168    534.17914    563.07824     638.2915    585.48129          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       16.50    264.95964    246.98716    240.03301    228.02162    223.11022      218.512    214.59117    215.13045    202.37154    197.62174    197.05219    193.42779    189.13704    186.41909    184.60097    182.37167    176.60362    174.17599    173.44609    173.18335    169.97281    169.11569    167.55098    160.76384      164.839    185.15536    201.33877    211.00918    197.52349    175.72382     166.2036    158.81941    153.77208    157.21482    149.03853    147.38389    150.69163     146.9096    150.30688    151.91448    145.05484    147.92774    141.85862    140.48584    143.79144    142.38486    142.26175    145.63762    137.94987    143.13842     140.8719    141.02398    137.46116    141.07848    138.08286    143.57441    155.01186    172.80369    180.20502    177.87201    160.43499    144.34562    138.50246    138.74609    137.05826    135.47684    134.08516    135.36982    138.12435    138.03419    151.79083    163.62442    168.33009     168.6008    159.26404    164.39566    198.55205    240.20511    258.43594    243.10205    212.79997    180.64953    166.53943     150.9636    142.92414    136.87664    130.48361    130.09723    129.38183     127.0562    136.47317    141.25307    144.08582    134.21689    131.12255     130.3013    130.43452    126.94953    127.16269    121.72752    126.03217    124.10609    124.88111    125.88355     119.7128    118.97645    114.78501     118.9993    117.18128    119.48207    116.74561    118.32278    117.87111    118.53317    117.81173     117.8649    122.70307    128.75827    125.66051    125.93874    124.58742     119.8859    120.01874    123.79744    127.98198    133.65244    133.37072    138.75751    129.95031    128.07583    118.82727    121.64541    119.37356    117.65825    120.39611    122.79912    130.64264    137.58807    149.82961    161.06087     163.1568    154.55688     148.7734    138.75658    130.70391     124.4549    120.73726    119.51256     116.7839    121.79588    120.57537    120.56808    114.10388    117.98236    118.16168    115.44377    114.74177    117.92219    119.30649    118.38125    120.95757     117.3526    121.21615    119.93987    122.04897    122.64368    123.16565    122.55618    124.59371    130.54648    136.29976    134.10205    133.22966    133.35596    128.27631    135.43457    130.54733    132.69261    131.12836    133.31002    132.65997    130.46392    128.37166     126.2751    123.86416    122.00834    121.57987    120.09226    122.62069    120.86445    122.02929    125.02178    124.60017    120.75069    125.23845    126.14163    126.05612    128.07657    129.56409    133.26577    129.68947    130.60037    126.35052    125.81986    123.41469    124.79777    121.30866    124.76634    128.31579    126.80613    124.04985    127.94208    123.60159    123.17951    121.10657    122.15307    120.96082    122.53221    119.25743    117.38873    115.77317    118.99768    122.59206    122.56297    122.50147    123.01809    121.99333    127.81431    128.25578    127.11013    128.47249    127.39001    126.83994    131.53693    129.09772    128.48189    129.37944    133.18024    129.77918     131.0534    132.39404    133.62678     132.3174    135.86231    136.39933    132.23378    131.00224    131.33337     128.8625    128.73592    127.64844     129.6011    131.50649    133.00669    132.91679    135.92749    130.01049    131.40544    131.61855    132.71096    135.39932    140.73185    134.60724    140.68044     137.7459    141.87323    144.63256    142.59192    143.96461    140.95909    133.12726     137.8349    135.75832    135.69076    139.04866    139.85259    133.85574      133.163    137.69391    134.62791    131.67269    131.17533     136.4877    130.48815    132.46153    134.71034    136.17862    130.26695    128.24088     127.1475    129.12821    127.41147    126.19371    128.21151    125.52286    126.28045    126.12129    127.03031    129.32954    128.78182    124.31172    128.53546    125.29215    124.69962    122.49402    126.16152     127.5666    125.33282    124.63559    127.13793    127.80501      126.584     128.6887    127.88755    130.54684    133.80025    132.88257    128.80505    128.86635     129.9305    125.92036    125.64712    127.00511    126.27442    127.61634    130.19044    130.56271    126.36085    129.91604    128.97856    135.22644    128.98066    133.60241    133.57044    133.59671     137.4976    135.62563    135.59968    137.36902    136.26158    144.44928    139.84034    136.36686    137.56948    138.93075    141.33798    135.88634    140.53583    143.45723     142.5929    145.07532    145.61472    148.64743    153.14422    150.98381    151.43455    147.25877    148.61779    150.96375    157.48287    156.10072    158.22629    155.19053    155.42143    157.46674    156.33045    157.30166    155.64046    159.18255    155.30324    155.55324     158.8953    155.01872    160.48157    162.37292    159.15318    160.00355    155.62389    160.56197    165.78275    168.35377    168.26955    162.51372    160.45551    176.30656    169.50268    171.13575    171.05365    163.55829    168.00375    176.54563    167.24008    172.16072    171.78261    178.83997    177.93331    181.25142    182.58433    182.17576    179.88053    177.85085    178.71969    190.25499    192.27375    187.36785    189.12791    188.15965    187.06828    194.72787    192.30074    194.60147    197.34137    188.05769    186.59211    200.30268    200.56105    201.52447    210.60812    202.30222    193.32945    216.92737    213.69375    208.32138    213.49941    202.71683    205.48946     214.9206    223.54986    218.73929    221.70827    210.90499    216.52352    215.41611     232.5199    226.00359    228.03541    230.06629    242.35205     222.1594    227.69918    238.75517    231.22639    233.11657    231.85091    237.95394    239.72506    247.48008    249.67378    251.93582    242.08021    247.54971    258.36007    259.60005     273.4789     274.8866    275.69867    279.57814    272.78335    300.35266    296.12487    309.64557    298.96668    302.12936     273.6804    370.88768    337.19444    338.59789    329.05387    304.16491    329.81714    371.25888    391.75878    371.14924    377.54199    405.27271    374.72742    387.24648    441.36601     479.1793    406.33099    442.55039    481.50041    518.06188    648.79623    685.96241    726.11118    490.60299    499.66601          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       17.00    261.22659    249.58046    237.68116    228.71652    224.44259     216.6304    212.96694    224.44649    206.16615    199.86842    198.06312    193.14888    190.43058    184.08172    180.01073    181.73398    180.01077    178.50288    171.21104    178.57763    175.45372    169.08424    166.30055    162.51161    168.93437    184.68588    203.57904    208.88944    193.32539     172.9786    168.75049    164.05968    151.99324    153.67187    155.26152    150.85155     153.1673     148.7982    149.96381    149.59537     147.2162    154.44728    142.83757    142.93913    147.31838    145.45512    142.55374    141.74907    137.32764    139.34502    140.67814    142.03061    141.52291    140.51587    141.74755    143.27651    152.30701    172.44874    180.98129    178.33733    156.91526    149.04233    137.97966    137.21872    135.86716    133.37392    135.82633    133.59385    140.88973    137.18592    151.47108    165.72333    170.47432    166.70751    161.71273    169.46658    200.97844    238.67222    251.64916    235.92242    202.93981    176.13774    168.72321    163.36519      152.039    139.71705    133.73359    128.29462    129.82225    130.76358    136.18267    141.43966    137.58739    136.59687    130.31389    127.88838      126.254    120.87202    124.49302    123.99133     122.0205    123.88208    123.72828    124.08772    123.58252    121.53639    118.07238    119.93233    119.57489    115.58436    119.93338    119.34921    118.04011    117.02612    119.26885    123.83978    124.07161    127.07697    130.76855    125.10519    124.74406    120.63827    118.14742    121.87903    128.06032    133.16121    133.41402    133.99835    129.49529    122.38546     120.6249    119.22764    118.79635    120.33826    120.20561    125.90611    130.34341    143.61256    152.80648    161.01379    161.44814    153.98315    147.38389    136.97897    129.92825    124.31109    123.05065    118.18072    117.70231    116.99968    120.99413    118.42257    118.07065    117.16604    116.15228    114.28477    118.23555    121.71171    120.14441    120.97238    118.78026    116.14005    119.50476    124.43954    121.73882    122.83925    126.49578    125.95146    122.59135    127.66057    130.80319    134.01435    131.72551    132.01555    127.20157    131.80655    133.74355    133.13377    136.16281    135.46848    135.11934    131.02939    127.55622    129.06888    122.77231    122.09589    123.78852    120.03109     121.2685    123.19411    121.98965    123.34811    121.03779    121.53425    123.49318    126.60118    124.91507    129.34182    130.31075    128.58888    127.57571    127.34279    125.56128    123.19399    123.87947    120.81037    122.81798    120.69558    124.43981    126.00774    126.03454    122.93988     125.1493    122.10931    123.83066    122.33192    120.32652     118.2257    116.99718    118.58592    118.58366    119.16799    120.93465    122.17506    123.47677    123.29456    121.79872    127.58502    129.91165    125.95829    128.41183    129.13864    126.30535    125.90421    127.65913    132.44032    133.14383    135.60098    137.84256    135.35192    132.41876    135.98033     137.0069    133.75452    136.88847    130.52151    133.17378    127.14093    130.68267    130.40388    124.87348    126.15225     125.8471    127.96033    130.38882    129.81041    129.24584    131.34713    128.42143    133.14154    133.18228    135.77555    134.24687    142.11628    142.75143    141.25674    142.85125    143.07852    138.77571    137.56545    135.01697    137.08843    131.79466    134.16411    137.16848    134.80435    133.63888    131.04061    135.52826    133.64233    130.26371    132.91811    131.26695    133.21844    134.02296    134.88222    129.49256    134.35221    129.62704    131.96644    126.99957    129.74759    127.12176    129.30672    130.06936    125.88775    125.59119    127.51643    121.58337    127.79629    125.90982     127.4434     126.9153    124.87619    125.87425    126.98089    127.63252    125.36323    125.46325    126.22761    129.83443    130.61891    128.87776    126.77864    125.85619    129.47817    127.66605    127.48458     124.7679    127.58857    128.33627    124.09901    126.24146    126.05472    126.24251    127.71488    123.53738    125.05865    128.19267    129.98637    129.27077    128.75984    133.31613    131.06725    134.36626    139.60167    138.54774    136.48238    135.56353    136.92964    138.39575    139.77818     143.3589     139.8425    141.21181     142.7449    139.82668     147.2521    139.74152    139.63136    146.34065    144.38309    147.79393    151.08923    151.96031    148.03797    149.66169    150.59752    152.33397    153.98679    154.65365    154.77112     154.2962    157.38764    154.87653     154.7539    160.93102    155.14682    156.88471    156.39222    156.10366    158.87789    157.16589    155.84913    163.47458    159.35455    157.78555    159.87839    165.40135      164.446    162.93104     168.0642    165.33564    175.23716    171.56903    172.77819    168.34079    168.14977    169.39307    169.17463    173.34711    168.21827    172.41726     171.0849    166.77008    177.96309    192.74887    186.13935    181.97926    180.70608     181.7538    184.29943    189.29432    186.15836    191.35365    191.23688    186.54136    187.41687    199.03151    194.68029    202.59858    197.49025    196.70338    197.55954    208.27149    205.37198    219.10744    202.26996     202.8649    205.78301    207.15001    218.84602     208.2833    213.13201     211.8804    209.03931    210.41978    225.07819    210.05575    226.91446    211.16218    226.97612    210.50449    224.57434    222.83851    230.63555    228.85799    223.98607    219.20495    231.14569    239.51458    248.76527    248.49555    239.88513    231.48165     252.9723    261.26364     262.6559    254.37854    258.69337    240.00894    229.80439    278.01615    275.73589    277.17591    275.37764    278.88001    264.42725    318.31531    297.80076    285.80574    304.69844    301.65742    308.39337    325.25235    349.70513    337.57584    364.68945    344.51746    343.20257     412.8743    431.16806    391.40252    402.02282    379.92463    365.75983    477.57768     437.1439    516.15724      433.806    471.29592    483.98298    537.22364    634.90064    713.36215    545.43765    520.31898    602.25672          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       17.50    262.92953    252.28078    237.17966    229.81612    227.63723    218.19248    212.15907    221.44642    209.32691    200.21526    196.55034    189.19839    191.34232    183.04869    187.04308    179.15642    183.69368    177.44876    173.06133    172.81235    175.37702    167.48723    166.46842    163.33687    178.12706    184.47395    200.35841    206.56181    184.02526    175.33179    165.11204    159.87573    155.09158    156.96324    152.66144    153.61338    153.09705    150.35151     148.9426    147.52341     149.5199    146.42355    141.23158    145.97876     146.5557    140.29657    139.96874    144.51418    143.52885     142.6106    141.16634     143.1252    142.98908    144.19767    139.72145     144.5399    154.44271     173.2168    178.49851    176.74603    156.86941     151.2403    141.55727    136.63611    138.22411    136.04864    135.35152    141.87943    135.58898    138.67557    151.16833     166.0914     168.5479    163.80326    161.14417    167.90626    193.41044    227.51725    231.39661       220.57    190.96437    174.83613    172.02737    169.59832     159.7572    143.62472    135.57795    129.54204    129.26164    130.28527    136.30177    141.46419    141.11407    139.13711    129.38323    125.81596    121.24089    121.43278    125.00177    120.36768     124.6822    122.86333    123.47937    124.02181    120.79777    115.70166    117.56865    116.98534    116.37025    118.29754     116.7479    116.85279    119.58931    113.83623    118.34793     120.8672     123.1139    128.97159    124.30393    126.24143    121.74947    121.25102    116.93057    122.47886    126.78604    135.19884    136.49515    134.54691    127.09758    123.82902    120.92698    117.66249    117.64537    118.37818    117.06862    122.39231    128.70516    141.61371    151.93554    159.19797    159.80306    158.61188     144.5692    138.48751    128.26497    124.69639    121.17217    121.84353     118.1592    119.45496    118.50888    119.04031    115.54401    117.34149    116.06837    114.54462     115.5705    117.80012    119.58709    120.07885     118.5663    119.26124    119.05728    121.24427    122.58056    126.26851    126.22212    127.17565    123.91042    127.99148     129.7017    128.54792    130.71389    129.61795    130.88392    130.79792    132.44948    131.59656    133.85975    139.47248    138.03358    134.53606    133.30658    131.11007    127.49948    121.75846    118.92541    120.93101    117.52661     119.8366    123.08543    117.32751    120.85093    122.58956      123.155    124.71296    126.27431    125.80835    127.83467    128.08916    128.48267    125.82628    123.97294    122.23732    122.19222    120.91433    122.87041    122.54474    125.98538    129.48723    125.64733    124.74663    124.41176     124.2677    119.87466    122.59804    118.25345     117.8691    116.65831    115.24434    113.80936    120.94149    118.29796    122.65491    119.71057    122.68719    119.63358    124.71785    126.12924     127.0217    125.86349    125.99833    125.47614    125.73819    125.99886    131.62954    136.56562    142.35961    141.66894     141.0051    140.68792    138.01045    134.72661    138.41091    133.21757    134.36384    129.33488    130.25877    128.71614    129.93499    127.63466    130.86017    124.38162    127.71524    131.50174    128.22872    131.82571     129.7421    129.30787    134.19577     130.8394    136.78242    135.33194    137.40722    141.53595    140.37899    143.23709    142.12565    137.50489    136.78767    140.81688    132.39202    134.23972    129.33028    130.95949    133.94711    125.78809    130.76569    131.89849     132.4817    134.96346    134.29159    137.77088    134.69356    134.25912    133.05401    130.78145    133.99282    132.08195    130.91961    128.24186     126.9862    122.73247    123.96947    131.15157    126.39537    122.96076    127.73157    126.33722    124.59478    127.21272    128.56555    125.33869    123.15152    125.82485    127.13162    128.20995    125.17143    124.56137    126.89208    130.40654    130.15048    127.37466    128.73582    128.31197    128.71143     128.0197    130.31541    128.69687    129.56386    128.87959     125.4292    128.16607     130.0742    120.22097    126.72706    126.40636    129.20016    126.97796    130.03158    127.24386    130.30756    133.58105    134.93036    130.94864    136.16591    134.36893      133.413    132.34581    136.33067    135.84996    137.97467    143.24641    143.18451    136.57676     139.6402     140.4935    139.39568    142.32157    142.33336     143.2652    144.11431    143.00908    143.47493    143.53282    147.16792    153.21221    149.43668    151.38126    152.44695    160.66443    155.09193    156.33161    150.73605    155.29182    151.62126    156.73154    158.53138    159.73676     157.7361    158.11105    161.59773    160.79853    160.05484    153.99575    157.24644    159.87142    159.89599    163.58013    166.26756     167.1668    160.84581    163.91465    166.92115    162.03103    177.72658    166.39472    173.93568    163.78586    169.19701      170.644    175.31057    175.76904     178.3613     181.0136    174.15391    178.43273    181.98538    182.71359    186.43223    184.83469    184.49817    185.80632    186.95409    182.96616     188.9643    191.91352     189.2202    184.50295    200.43938    196.88277    198.16955    201.17742    199.14686    199.35961    204.15714    202.64233    216.82517    206.13046    202.52181    202.81119    218.72449    215.89103    218.07135    207.00159    218.27945    218.46019    232.84511    218.71608    231.17102    221.43676    218.66493    230.28993    233.35843    219.88023    228.54083    242.71531    221.32946     228.0334    243.96513    235.63938    243.37577    251.93919    233.83275    239.47216    274.53078    260.93194    252.64691    258.98696    256.82705    262.24486    257.76593    275.78669    280.47659    276.75765    287.53839    282.13525    295.00936    307.63736     320.1423     310.7583    311.86947     322.7869    311.15709    365.14982    357.20543    331.62667    322.17814    345.75693    368.86546     373.9629    441.93999    397.54195    374.98327    422.38341    415.64963    453.78541    486.85498    506.60567    498.49556    516.18317    501.23047    690.55231    633.01682    708.62657    740.81855    763.08132    696.26619          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       18.00    266.59157    254.88818     237.1939    232.60325     229.8512      221.827    214.29603     218.9979     208.5385    200.93036    196.31331    191.05974    192.66659     183.6878    189.77047    172.96022    181.09567    175.63811    174.06956    173.08666    173.60185    171.11596    167.18778    164.58575    180.29047    178.42971    205.99285    208.05483    190.90294    175.66067    164.96983    154.33376    155.29909     157.0415    154.39151    155.43356    148.24502    151.05366     152.4408    148.11699    149.75176    148.50794    143.16169    147.56751    143.90884    147.55191    142.70532    143.77486    144.04975    142.06737    141.63596     138.7168    142.72324    142.61724     141.7057    143.57919    151.44327    171.28075    176.91852    174.64119    161.96823    151.97865    141.38473    137.50092    136.13433    134.79655    133.76492    136.01994     139.2311    138.72511    150.55969     164.3773    168.89553    165.70499    160.47455     163.6654    179.81166    202.19899     209.6973    196.01223    175.70204    170.54943     173.0294    172.20585    165.19506     146.0519    137.12045    126.80294     132.7241    130.11708    132.35356     139.0707    140.95089    138.45493    127.89246    127.31333    126.06309    124.71179    122.28237    122.37643    123.07028    125.12309     124.2522    123.89073    123.77771    118.25363    117.97978    119.84416    117.12394     116.3382    117.44955    117.55021     118.2162    119.85142    119.48748    122.11976    120.05521    127.50134    127.85913    126.51592    121.74411    120.84242    119.92675    119.57077    126.32501    128.96788     136.4375    137.12924    131.88447    121.39425    121.65042    119.11617     116.8916    120.74637     118.8158    124.41929    130.28685    141.94127    151.71303    158.68226    158.82391    152.35879    143.61601    135.84238    129.87506    124.15147    123.14686    117.45937    120.56743    116.98019    116.40465    119.42009    117.63923    114.09809    116.26504    113.71268    116.30202    117.96134    118.67036    119.45993    120.39056    116.88599    117.13491    122.52449    122.45035    126.30149    125.06413    127.39915    128.32806    126.26469    133.29518    133.21104    130.31004    132.01018    130.62022    128.00023    133.23428    134.32396    135.98579    137.62415    139.41605    135.84015    129.20597    127.13684    127.88572    126.02713    125.40956    122.19854    122.31631    120.14412    122.80916    122.53549     124.7219     123.1509    119.21832    121.93569    125.83083    125.40189    128.82604    126.40644    129.12865     124.6392    126.45269    124.63037    119.79878    121.00987    124.84374    119.57037    124.41716    123.51702    126.65798    122.71662     122.6588    123.27212    115.24158    120.56108    121.07146    121.55925    115.05921    115.54164    112.14667    119.57875    117.18357    115.57166    121.02083    122.97855    120.73479    119.15893    128.50037    129.73698    126.96473    124.78876    127.41124    128.42565    129.60833    129.16049    131.24832    134.90723    136.27134    138.04345    136.89459    140.67622    138.27425    137.07293    137.07426      136.244    129.62487    128.49109    125.15815    124.45005     125.7867     125.8432    124.95621    124.61004    126.67689    129.00791    128.21083    127.54101    127.90927    126.67749    133.42292    137.05178    139.16041    136.67966    138.83936    139.90006    139.17303    145.47216     139.6023    135.99449     137.1978    131.31081    137.29649    130.33679    130.06569    128.16344    130.07823    128.27932    129.94696    132.99804    135.23931    133.27707    136.93658    137.44043    130.28446    131.89953     129.3485    131.30868    130.95998    130.81747    125.59394    130.15104    127.77219    128.98093    127.42858    130.54099    131.21163    127.16454    126.10626    126.33012    129.69853    128.72072    129.29031    120.88304    122.54231    124.28711     121.7678    121.25452    123.30717    125.72358    127.12494    127.61622    128.04071    128.66104    127.86258    125.17153    128.39131    131.63487    127.39604    127.22792    129.59127    127.46768    127.31643    126.56328    127.99714    123.62509    124.54985    127.41293    126.43806    134.05363    127.57775    132.29448    130.26148    132.99489    130.77043    138.35246    133.63066    135.51666    136.12777    132.36408    136.78826    141.67351    137.65559    144.64853    136.97343    139.15588    139.76663    141.62723    138.12474    141.95609    143.36658    143.55705    144.16998    145.82179    149.04558    155.00943    147.64883    154.11998    155.02132    153.75071    154.60967    154.88726     156.9709    154.16633    156.87702    154.48108    156.15408    151.65662    160.06072    149.57711    157.10124    157.38017    158.81955     164.9141    166.82412    159.04634     158.5155     154.8996    166.12695     166.2256    163.21277     168.8866    169.77094     166.1373    167.72439    173.99916     173.4266    169.21165    167.44821    170.88774    170.11123    177.93986    179.83895    178.70248    185.00785     172.7519    182.47597    185.51068    181.22696    182.48264     189.2106    186.10478    185.74062    194.30679     195.0748    190.76224    188.66154    196.94272    193.73979    203.54611    204.61137    197.64642     199.9771    199.19049     201.8456    204.96714    211.42701    209.21574    208.04185    210.49921    206.31621    219.72498    215.62567    223.13113    212.70752    214.78986    210.22616    231.92073    227.73758    227.59486     228.0635     228.9437    221.63152    237.36264    228.10401    235.49113     230.6843     231.0634    227.76375    251.01408    262.06794    254.10917    250.82422    245.40043    234.61545    271.67026    281.50327    270.28606    261.74353    279.42506    267.49015    271.72336    285.13736    281.41584    292.62838    290.62387    283.18938    288.90928    330.15999    305.43311    338.48567    306.65993    306.96787    309.01929    356.18482    391.24789    364.66634    372.49128    350.42714    357.15797    430.56012      442.553     422.2741    425.02475    455.97927    443.77478    550.16661     586.2787    523.95484    531.91568    543.67709    561.16421    813.71451    912.88435    828.20053    785.31291     873.5397    765.65713          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       18.50    269.85615    257.90023    237.90703    234.40735    228.37224    223.49189    216.50466    214.81363      207.838    202.14825    199.20865    196.70817    194.41409    182.42309    187.42874    175.16453    179.90223    176.58744    175.95279    177.42224    171.83202    177.03499    167.68741    165.94727     176.3778    185.01322    203.80212    209.91685    194.50613    174.13158    166.98072    157.95862    154.11631    159.37199    156.98971    150.41947    148.31105    148.40218    156.87901    146.76406    148.04917    144.42166    143.95513    143.96582    143.27249    143.14221    141.28442    142.70775    141.69998    138.87825    139.99087    141.41993    142.15397    142.25157    144.06178    143.06644    155.45544    169.64578    182.09934    172.92069    158.31008    149.21862    140.09902    138.42227    128.25433    134.79975      136.896    138.75483    138.11077    145.01457    150.15311     166.6667    170.93123    164.80745    157.02799    159.46843    175.34044    187.93102    186.37831    176.84222    168.46051    165.82148    175.48506    175.18846    166.96632    151.18973    133.61356    128.77944    133.26382    132.83822    135.26354     140.4062    139.91792    137.73972    129.63314    124.87843    120.56703    122.03989    123.79888    119.82629    124.18334    126.22847    124.34341    123.87839    118.31127    120.77298    116.50322    120.86385    118.58484    113.50436    120.38939    119.88805    116.63311    121.42147    120.20392     121.1655    123.29603    128.50908    128.56709    125.66398    121.74792    119.67734    122.38383    121.54868     127.6072    133.16025    135.54672    136.80292    129.75749    125.00164    121.75386    119.44224      119.221    118.72638    120.74227     125.6043    123.46854    145.08889    150.82304    159.32598     160.6934    154.81283    144.53241    135.44005    132.44313    122.76307    119.40795    120.68113    118.10334    114.94216    120.07709     117.2602    115.84349    113.48507    117.38072    115.58602    114.52068    117.73057     120.4155    119.10549    118.69029    122.90263    116.76161    122.25312    122.95341    122.78713    124.71998    125.01611    130.51074    124.48757    130.39081    124.38768    127.78418    128.58927    129.69339    126.07631    131.06695    134.64611    135.30384    134.14211    133.85895    134.51668     125.9806    130.01893    126.71925    125.48074    121.54965    121.80435    123.13384    116.00635    120.50542    119.90979    122.27708    120.29171    126.40062    122.98364     125.4815    125.44273    128.71847    132.90681    127.20484    127.21603    127.20899    125.10054    120.35076    121.80756    119.05972    119.97189    119.77868    123.12883    125.71847    122.11714    121.97318     120.9822    117.73419    118.87041    117.45754    114.86214    118.34747    119.68084    117.26477     114.5965    122.12803    121.97379    120.05503    122.88351    120.55749      120.608    124.61715     126.7948    126.80928    127.86735    125.66024    127.09787    122.60628    127.02235    127.08366    130.29913    130.14406    133.08723    134.98012    136.01175    131.12624     131.4649    132.72168    131.43874    127.84217    128.81879     128.3513    126.36386    127.28653    125.66119    125.73968    128.64649    129.28809    128.62479    126.81023    123.86892    127.01404    125.38001    134.03727    134.79922     133.6344    141.57437    140.16176    140.54148    140.93176    142.57524    139.04517     138.5675    135.20994    131.31454    128.78822    130.43794    129.33511    127.22709    128.95335    125.90392    125.04208    129.43915    135.01687     134.6939    134.08236    130.95228    132.97696    133.64275    134.35211    135.10064    129.53003    130.30532    126.31373    128.17539    128.09927    127.00768    128.38709    130.49871    132.37434    120.20686     132.9734    128.53376     126.5285    125.10808    127.34188    119.23755    125.46537    124.92204    128.42671    124.13789    125.24966    125.01972    126.32852    125.74651     124.2807    129.42316    129.07697    127.31401    128.13043    130.07678    130.99242    128.56417    126.39805     123.4504    126.98732    127.42367    125.08302    126.76033    125.57341    121.06673     126.8756    128.30989    130.25593    130.66664    129.61184    130.88726    135.13857    134.73044    137.32963    135.46339    130.74837    135.91313    135.20416    138.47437    137.99309    143.26599    142.58366    143.03996    139.29685    141.87289     143.2811    140.83849    139.53719    145.81926    145.96971    147.98217    148.11898    146.80628    150.19411    152.48688    151.88799    147.56601    157.67252     155.4929    150.31589    153.53202    156.83164    157.63432      157.141     157.1804    158.59581    159.15188    156.04321     160.0632    160.99368    157.63974    159.18037    154.28199    157.54367    163.25177    165.83668    165.97503    167.11856    164.14587     168.8938    160.92106    160.93819    171.24126    173.10355    173.29014     174.1631    167.13947    176.12593    179.65273    175.48011     176.8303      178.576    173.88091    183.24011    179.61835    185.47943    193.75989    195.65908    181.62163    189.87554    197.66943     193.4438    195.68599     198.4808    190.40673      182.667    197.37918    200.16261    199.36098    202.52361    200.66713    194.90286    213.17665    212.54632    208.68019    198.20981    208.05828    215.57845    229.07873    224.46741    215.79504    217.99055    221.79192    223.65903    228.40684    223.73497    223.96605    222.17525    223.93572    223.54699    238.56139    258.57765    241.02266    244.39997    237.69099     247.8569    244.87782    255.10021    253.46737    247.23052    258.49742    264.56471    255.01256    294.44869    264.77202     281.0308    265.80948    253.25891    274.24794    290.68772    302.29508      313.733    290.56965    300.69002    302.03584    316.59198    333.70792    324.84065    333.24449    356.86862    356.18178    414.37867    399.83877    360.68086    368.89822    373.85353    387.31641    459.17532    454.16828    442.29201    500.49794    465.91546    437.55355    582.77519    589.40584     578.4867    559.47704    576.36672    549.58232    741.41012    897.68889    938.62469    897.27078    939.44683    862.97031          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       19.00    267.87867    261.56199    239.40665    234.70898    228.10174    221.81453    217.24592    211.20582    208.70516    202.32161     199.8689    198.27481    194.41304    182.13361    188.29318    176.94279    175.55672    175.72073    177.95805    180.28972    170.46575    175.49765     166.4218    166.59516    174.78267    186.67282    198.78107    207.93597    196.50129    176.83772    165.83201    159.48759    155.99299    156.45706    155.53253    154.60654    152.99649    150.95209    153.72345    144.93756    154.66534    146.86082     146.5016     144.0042    142.71571    143.88387    144.41305    141.72014     143.9219    141.47817    140.01584     141.2518    139.99318    142.81238    139.40502    145.45952    155.47696    166.29132    180.33875    175.01581    162.63872    150.81891     142.5418    132.57615    138.33846    134.58372     133.1349    134.94564    139.63659    148.12138    155.94245    163.96468    169.36313    167.45385    157.95358    162.06302    170.60364    176.25452    183.51288    172.86481     161.2507    166.15033     166.2978    170.69767    163.12173    148.33336    134.72542    132.13628    128.69026    132.26152    139.57427    139.22667     144.6727     138.1127    128.14288    128.36615    120.44227    119.68923    123.05538    119.28245     120.9801    126.70958     124.0315    122.36516    120.80332    121.18048    120.20395    116.93334    118.04755    120.06937    116.94862    119.12403    117.93299    119.07754    118.11853    122.26793     122.1437    125.05693    128.47122     125.6019    123.70235    121.25636    121.99008    117.41686    130.90431    129.38072    132.64635    133.63649    129.68886    124.79649    118.81957    119.82406    117.29568    117.94526    118.33084    122.08294    129.49469    138.28508    147.44996    161.00891    158.94753    155.58934    146.90749    131.67236    128.34625    121.08037    121.34173    120.32488    117.29304     116.7468    115.39372    114.96173    115.39024    115.26561    116.07471    116.10851    113.85183    118.84357    119.64483    119.91905     120.3464    120.00424    119.38417    118.34609    121.48595    124.97671    124.94696    129.96221     123.2235    118.91316    123.26091    122.28096    121.43586    128.66331    128.62168    129.39365    126.64699    131.73544    133.30989    134.12197     130.8207     132.0834    127.50481    122.24134    120.23046    121.55408    120.47568    119.27803    122.60274    118.69303    120.79092    121.25947    121.13537    122.14607     125.1961    122.16158    124.60624    127.63543    124.98382    129.16238    128.56539    126.09526      125.566    125.09273    123.85941    120.45915    119.97696      120.122    121.75775     125.0863     124.7513    120.90394    121.69583    121.43123     119.4924    119.63724     115.3034    118.26206    117.26906    116.19676    116.74655    115.62975    118.89396    120.60828     124.3477    121.88983    123.30219     123.8106    125.36999    125.57335     126.3097    124.29554    126.98346    127.55046    125.15723    127.96598    129.72651    128.52366    125.97539    131.73649    130.00106    130.23641    133.41734    134.52672    126.40242     127.0512     127.0953    124.81027    125.75789    125.23467    123.36308    128.06037    125.77384    124.25073    128.34065    126.46588    130.79903    130.11664    131.15995    128.71057    133.11811    132.53637    136.74777    135.72705    137.99692    143.18285    140.31659    143.29343    143.08578    139.14838     136.1104    132.52752     130.8422    131.28963    126.86766    128.11811    128.97737    130.47598    127.56377    128.61815    132.31563    129.52157    129.20046    131.42071     133.1007    133.28179    131.74048    130.53335     130.9909    132.16544    129.07146    123.29882    129.08021    131.57002    130.10193    127.44996     129.0655    124.73264    129.54074    126.33943    125.96496    127.15974    125.93172    124.62517    123.89044    126.30607    126.20709    122.60548    120.57427    123.94272    121.73668    124.50056    126.23692    126.72772    127.43474    122.71781    128.05219    129.36609     127.7287    130.85476    125.81116    130.95269    124.94948    129.51116    124.77019    126.63506    125.94952    122.44664    128.75899     127.8215    130.57603     130.6107    132.88466     129.9887    131.09774    133.91755    135.33531    133.69168    134.13626    133.56362    138.17151    142.78643    142.79956    141.91979    141.89143    139.66062    134.89385    141.80873    142.07295    142.91505    141.50973    140.38888    144.83551    145.56746    148.86379    151.12719      149.007    154.70453    153.65824    152.82931    156.86884     150.7497     156.0085    153.02303    157.14905    153.39885    157.69338    158.41747    156.60789    160.25452    154.86981    162.19958    163.86003    159.97763    163.98447    164.03257    158.01475    165.52315     166.0893    167.60355    171.37057    167.54899    167.33607    163.09531    166.67693    176.39722    176.82733    172.38868    176.23344     169.1019    175.23645    177.80299    176.00337    181.35485    173.45707    177.15267    186.39003    188.43699    186.37809    183.83902    188.55937     185.1634    185.21071    196.67076    197.92583    196.77638     198.8087    198.99912    198.87657    202.78878    204.12404     200.4624    205.38088    201.56769    206.11819    209.34336    215.72335     212.8973     215.2022    216.84169    214.67378    221.40027    218.97212    223.30596    223.14421    217.77938    212.13323    232.10207    228.83013    226.52733    217.22512    230.73002    229.79302    239.89522    248.90561    235.51831    249.73968    231.84882    253.73586    248.29639    259.44758    263.36808    249.88044    257.40923    252.82281    295.87602      278.728    274.88832    277.09461    268.98183    267.36133    296.58673    294.57605    304.71348    292.74369    316.29286    297.84313    319.81356    344.92345    349.43389     326.7118    335.78293    348.42129    344.73629    408.80816    395.06825    401.58451      394.098     385.3556    400.27378    449.65461    477.89888    478.91984    466.44175    448.00536     475.3232    611.20865    593.39666    600.51132    580.89159    605.94901    625.08819    863.65225     881.1275    853.95793    857.63066    964.64257    940.76209          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       19.50    263.47206    262.03884    240.38847    234.50991    228.55129    220.19735    218.44449    210.37912    209.53446    200.36494     202.8226    198.13192    192.77148    184.92609    189.06251    177.46433    177.19661    172.24319    174.77937    178.59754    170.34166    172.77809    166.32118    165.11744    174.54396    185.69223    199.43897     208.5716    195.80828    170.16756    160.70463    158.73479    152.18257    152.57862    154.00324    155.98198    156.50318    152.60226    146.86796    147.76407      150.964    150.67545    148.80757    148.73069     140.3626    140.43426     144.1524    144.58225    144.55471    139.78567    138.78092    136.96896    143.44514    138.48796    140.86259    150.23672    160.33281    168.59568    178.00649    172.17119    159.22613    148.59592    142.71806    138.95063    140.46758    136.28404     136.0585    141.51496    136.88508    147.30376    154.93872     163.1818    168.26195    166.63565    153.63209     158.7606    165.60366    174.08359    174.94936    173.64664    160.19881    154.53654    162.66587    164.45826    153.37431    141.06055    134.41696    130.58539    129.95415      132.488     137.5318    143.34112    142.67303    136.60329    132.25675    126.94607    122.87805    118.47783    119.42117    117.34585    123.70212    125.29242    124.87911    121.50384    121.15973    121.98026     116.1251    119.00175    119.55534    117.24096    114.37533    119.85517    116.96797    119.54436    118.76199    121.81435    125.65889    124.84153      128.564    125.66369    124.77189    119.92598    119.87521    122.83774    127.31822    133.19412    137.11806    135.69483    133.07491    128.18042    120.85185    119.69629    122.08888     117.9483    119.61061    123.38543    130.82464    138.27876    152.57748    159.57483    160.58953    154.62277    146.52447    132.20241    130.19701    123.41021    119.15884    118.61463    119.60602    118.51927    117.20353    117.40772    114.94782    115.82452    116.72727    113.04876    117.58594    117.83338    119.28987    118.17917    119.90685    119.36127     118.4927    117.58811    122.70811    122.91603    123.56111    126.01453    124.27394    122.85648    119.20791    124.78375    119.84498    121.45587    124.69201    128.83321    128.40738    130.72665    132.73217    133.75988    134.19117    129.64156    123.89893    120.08029     124.5563    119.87915    117.35826    119.36519    119.19049    117.68026    120.59865    120.13837    121.58112    122.97139    123.51028    121.30249    122.80339    125.18129     127.2561    127.57732    129.21313    127.76921    123.52821    123.50879    121.39477    120.89187    120.81697    122.68476    121.42998    122.49063    125.23281    124.33075    123.14976    121.39297    118.14202    117.94139    118.29756    116.86109    116.77284    116.34325    115.79621    117.54105    114.94465    120.17777    121.78877    116.78829    122.18254    125.21597    121.09572    124.80454    125.59066    122.93267    125.39415     127.3462    124.40925    126.29257     128.1627    129.18149    130.59996    129.66403    132.10462    133.39443    137.98438     133.5072    128.23397    132.27979    127.06706    127.48552    127.57609    125.36417    130.51664    122.99819    124.38321    127.96747      123.674    127.20806    126.91953    130.72094    127.91477    128.89288    130.58958     135.4363     136.1366    139.05678     140.8289    139.81873    140.17435    139.66783    143.54617    137.59214      131.749    132.76853    129.72497    133.98689    133.63283    129.74738    129.78436    128.91165    126.00069    127.35916    130.67928    131.17495    132.39846     129.5393    131.44055    127.90067    131.56618    129.57386    127.09591    128.05234    131.47302    125.73484    129.25418    128.28434    127.41116     124.8036    127.21669    122.47576    127.26033    124.03181    125.47927    123.51828    122.19818    122.01144    122.16196    124.31527    123.76868    120.14675    124.18759    124.15883    119.70484     126.7872    124.28844    128.31496    123.41862    127.38815    125.75637    132.72903    125.86143    128.50758    130.64078    128.24883    123.62953     126.2554     124.4324    124.57834    124.80029    127.49292    130.73349     127.1819     129.3669    132.42275    128.90827    130.41338    130.03859    132.82599    137.91637    135.50543    134.24772     134.5364    138.78488    138.00318     144.6712    138.65305    142.42984    143.49657    139.11422    137.20357    140.09339    140.66128    142.76952    142.05395    144.02272    142.28585    148.19322    150.33637    151.42188    151.78619    151.59544    149.56018    155.71164    157.31916    156.18318    158.13072    153.38302    152.36981    155.16643    156.70596     159.3599    157.48168    159.51955    157.09774    160.37485    162.92242    161.90242    159.04831    167.12769    159.59484    162.42721    174.50987     169.9193    168.82888    168.06975    164.17891    175.47259    170.25802    179.02101    182.03778    178.33159    176.06735    178.33216    177.15064    179.82952    173.81411    175.60476    178.87238    173.94294    182.50711     185.6986     184.4394    183.61009    182.58604    187.29054    195.90029    193.84811    191.98966    195.17642    192.43732    192.23993    204.07029    212.20111    201.84627    204.94744    208.90529    200.57055    213.54287    210.05042    216.50579    211.32015    210.26439    220.26148    220.44634    221.76516    229.21294    214.68779    229.61214     226.8623    225.54457     234.3135    236.28869    222.21218    237.75734    232.81058    244.50954    239.11442    241.25163    243.68117    248.07044    248.57891    253.62082    269.91726    263.02921    271.46661    259.91794    264.66785    289.35764    271.83501    270.55328    294.21597    278.44615    278.45384    284.20866    336.49214    306.39605    306.44455    310.77591    304.33606    332.93988    339.62313     358.9878    344.74431    350.30735    372.42835    383.57143    387.55071    394.26457    420.69622    404.53062    406.23902    401.86358    484.94644    473.75213    475.52011    502.87918    489.90395    453.79852    617.68082    588.50915    653.36026    593.97251     641.3929    698.16015    859.01104    943.63624    948.19083    1036.6624   1004.94983          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       20.00    260.81272     261.7016    241.09019    234.60192     227.7162    223.41435    221.13643    209.86948    213.24754    199.84253    206.72728    195.96725    192.12481     185.2591    191.70072    179.57194     183.2444    174.32055    172.22692    179.07691    169.25182    171.00293    168.74183    166.11848     173.5653    184.44807    204.74327    210.31345    195.18082    176.67841    157.49108    157.55077    155.48538    148.93455     153.0703    152.27922    150.67726    146.47067     151.5153    147.64156     148.6904    148.97819    149.82518    143.10457    143.88338    144.38014    143.65862    139.60932    139.86743    140.75086    142.06323    135.83406    138.96339    139.23113    141.03716    141.76383    154.65487    166.99108    177.89368    169.89689    158.76889    142.30355    146.27566    138.20871     137.1847    135.85468    137.16218    133.64631    142.42097    145.49042    154.92943    168.53976    168.96923     163.2321    154.23704    158.78067    165.87522    167.72308    175.65375    171.24334    163.34422    157.50846    154.48213    153.86541     145.4109    137.22213    131.56416    128.37942    126.78326    134.40547    139.84831    137.63013    140.33442    138.11216    132.97844     126.9178    124.14747    119.80234    123.34426    119.93743    124.16229    125.85993    127.25657    121.51631    118.40916    117.36238    116.82877    117.41471    117.54352    118.54143    116.91934    116.42053     120.0488     119.7058    120.16716    117.91241    124.22229    129.02349    128.93261    124.54985    122.34731    122.12611    119.02858     121.3032    124.36853    132.75396    136.38971    133.28247    128.53125     127.1329    120.61442       119.13    120.10072     121.8036     122.6809    122.61757     129.8106    138.29135    153.10134     159.4427    158.99982    155.44924    144.84815    135.77966    127.06866     124.4354    122.43869    121.40013    119.45796    116.79742    113.37147    116.82893     117.1107    115.80291    112.69262    114.37421    114.82977    114.10011    119.37376    120.01119    120.05917    119.29796    119.28003    121.10475    118.36133    123.29045    126.48567    126.26697    122.93437    123.14612    120.36567    123.62808    126.07937    127.03569    125.52452    127.37876    130.66936    131.68939    133.48775    133.51588    131.59876    129.46586    122.94012    121.34729    121.26708    118.19235    115.60764    120.69937    117.10757    118.00564    115.05159    118.02014    120.80773    121.38327    121.05677     122.5622    118.35086     125.6559    125.01044    128.11026    127.67698    125.15422    124.97399     120.8648    120.96783    119.89311    120.58119    120.05552     122.2195     118.6907    122.73828    122.40974     119.3136    119.75358    119.55591     117.6299    116.82241    116.33184    116.55413    117.21544    117.33893    117.55315    116.60696    121.58288    117.47505    119.15643    122.25391    121.59503    124.67251    125.37019    125.40498    124.67068    125.49767    124.99774    126.65956    123.57955    127.49564    127.84692    126.71867    131.12311    130.17356    128.07675    133.50001     133.1133    135.11931    129.69105    128.18114    125.27074    120.17134    124.74787    124.53464    125.04013     125.6077    125.61236    122.77168    124.81448    126.90323    125.34243    125.65181     129.0765    128.18496    132.15791    133.04385    137.59774    138.04327    137.53429     139.2708    135.83567    136.01135    136.14845    134.99098    129.26654    128.51465    127.85817    127.67926    130.05894    124.80199    128.09723    127.66698    125.56909    127.39557    127.78945    133.85301    131.78138    129.18486    128.66476    131.37856    127.11901    130.38256    125.83385    128.80077    125.54733    125.63203    126.14288    126.27361    124.36931    127.23952    127.27026    121.31752    125.93421    124.37548    118.97492     125.6985    123.04492    121.88714    122.38669    123.08383    124.53745    118.89612    123.97093    123.64618    125.74605    124.59432     125.9427      125.284     129.4163     124.2406    126.23186    125.46956     124.4138     125.0242    126.62967    120.90284     124.2792     123.2777    125.93806    121.79168    125.37136    125.23139    125.68215    128.39815    133.23219    128.69814    129.51788    131.86988       131.09       141.36    136.64112    138.97988    138.66159    136.02154    132.23639    138.33458    138.14829    145.12331     138.5572    137.43954    138.24455    146.55309     141.4544    146.71399    146.26932    140.33838    146.97899    142.05422    150.35854    152.91122     147.7493     145.8043     150.8533    158.22002    152.61664    156.76672    150.17097    153.26247    153.58566    152.71299    156.78222    151.56977    149.98998    155.55698    146.03442    155.75646    154.23124    156.58693    158.99256     158.0982    156.66904    167.09026    162.71133    163.29371    162.33568     166.9246    170.00825    171.31056    170.54818    176.19369    166.78647    172.98834    165.89451    168.09362     173.2261    173.81573    178.91624    174.52257    179.29294    174.14567    188.37001    176.88857     181.0572    180.44177    178.53977    171.54245    196.05313      188.694    187.23658    192.19419    190.60948    188.02055    198.46097          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan    289.06149    295.71328    317.31163    305.88555    315.60469    308.03043    344.10356    345.99484    344.27425    358.30816    348.31846    339.27569    362.03774    418.43706    418.52997    428.59186    402.96234    385.10892    419.19854    487.73622    530.79535    500.05277    481.48034    498.47294    509.66877    633.88316    646.36473     664.6346    640.93305    610.73933    863.39963    756.11172   1188.73698     1128.829   1096.53242    983.78273          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       20.50     264.3071    261.55019     241.5431    232.60235    225.58916    229.73547    220.78614    211.15829    214.39378    202.11466    202.24794    193.59333    189.83972    188.30456    194.49323    181.19206    181.24248    176.18988    173.74453    175.49583    170.57774    171.09636     170.4963    166.16915    173.67181    177.89892    202.30956    209.84768     192.2502    177.45208    157.50885    156.15346    154.88848    155.64529    151.74423    154.56148    147.43166    144.09487    148.59596    148.68972    146.01803    144.87962    144.94557    145.30694    144.98498    146.85893    146.74497    142.78057    138.10838    138.06592    136.98903    139.54324    140.85722    137.61226    144.96933    144.01759    155.47251    171.27025    177.28019    170.65396    162.13462    145.45867    142.95152    142.44104    135.19696    135.09043    136.27012    136.65059    136.59015    140.64169    156.13107    163.24545    172.37359    164.21703    156.79369    160.24646    165.03432    169.30065    173.94019    168.58332    163.33778    156.45617    151.12899    146.10349    141.51615    133.49652    128.85834    127.54151    130.36172    134.73702    133.75492    140.77935    140.80148     133.1831    130.09016    125.97367    127.17472    123.79223    117.31999    121.14064    123.11131    124.18198     126.5186    123.43477    118.34108    119.05201    119.67073    119.47339    120.72834    119.05972    119.66596    114.26994    116.07225    117.09162    117.38985    121.57658    123.04066    126.46574    124.86823    125.81142    118.85055    117.69312    116.11286    117.26258      124.205    128.97495    134.85558    134.04583    129.00431    123.47595    124.82504    116.62122    118.62816    119.33979    118.29521    121.13184    132.10134    140.83533      152.762    161.32416     162.3145    154.70182    145.05219    135.17417    125.39169     124.4662     120.4199    119.96126    118.30915    119.15978    113.14202    115.42396    115.89116    117.54194    118.75324    115.73324    116.88785    117.53438    118.80895    117.28903     121.0008    121.83233    117.95603    116.45163    117.51799    121.90511    123.67872    125.42891     125.7159    126.19733    119.92025     126.1484    128.67155    129.31646    129.47802    132.93336    130.48994    129.25909    130.99347    137.29219    131.50266    129.26633     127.0986    123.81999     117.9443    119.85312    117.60071    116.94465    118.15427    115.22992    119.57929    120.71427    118.03232    119.68679    122.36815    121.54277    123.53717    123.58141    126.77115    128.22123    124.52142    126.77148    125.47249    119.95791    120.53312    119.41212    124.33723    121.22748    120.64128    123.29071    122.29799    124.16054    121.97533    120.19297    119.13082    118.24882    116.08928    117.96194    116.73363    116.85088    119.66007    117.24534    116.55438    118.53133    117.72231    120.54424    122.65245    123.69248     124.0928    124.18261    128.55662     127.4317    123.83625     124.2943    124.17282    125.02809    127.52119     124.7759    127.34866    124.39783    131.42995    128.13127     128.3574    128.65693    128.28947    128.92848    128.80869    127.11122    123.79996    125.19257    122.08578    123.60722    123.24213    122.89937    122.16645    125.94831    125.41543    127.42338    128.05864    130.76342    131.25411     129.5987     132.2171    135.29528    137.45462    134.43578    139.20851     134.1175    137.44464    135.53254    133.54263     131.1571    129.84449    124.53719    125.93792     128.1098    127.52371    124.19701    126.65709     128.5535    125.98612     129.6977    129.73972    125.20649    125.82651    128.23431    127.14923     129.2252    128.84641    123.64101    127.70599    122.43467    121.61507    127.10216    125.60991    126.38461    121.74709    122.00495    121.68326    125.39526    123.77518    124.65808    120.83414    122.24289    120.59395    118.61784    117.86243    120.58644    122.41294    123.42354    124.99941    120.32067    126.10177    123.25011    127.38847    128.48312    126.25422    123.95797    126.36444    126.44579    127.43005    124.08761    125.82799    122.35087    127.02335    126.41876    123.96998    127.93508    128.07382    127.76562     131.0839    128.44626    127.54835    130.03199    130.58438    127.86931    136.99256    134.52031    134.75372    135.77928     137.4751    133.25828    137.35832     135.5656    134.06006    136.17603     134.4999    137.10383    137.01519    144.18353    142.43352    140.35056    140.74493    138.56542     145.8375    144.39775    143.23315    141.10847    143.32486     143.7876    150.13313    145.89654    144.01984    141.71865    144.89432          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       21.00    269.20618     262.6885    241.67477    229.59785    224.66195    234.04721    219.97752     215.2702    210.09873    202.15103    196.98608    191.25752    188.20465    196.27962    191.59847     183.4837    177.75953    176.26352    176.44473    173.61475    175.09721     172.4769    170.83032    165.36521    172.03433    180.00055    195.41795    204.83347    196.60197    174.60968    159.71191     158.3575    155.04353    156.06241    153.08792    156.32051    145.94922    147.63333    151.14332     152.2878    148.18445    142.03438    147.39343    142.23101    141.33827    145.84746    145.84352    145.50118    144.48677    142.91658    136.08455    136.57462    143.75735    138.84513    143.57634    143.51433    154.61325    166.60478    176.73054    172.08686    159.18344    145.14263    141.10907    140.92704    135.27246    135.99673    134.31783    141.07098    140.07592    144.01372    159.58954    161.68278    171.64041    163.45418    161.20248    154.88866    164.51963    165.61564    168.24479    171.59535    165.67703    155.10553    149.84461     145.3678    137.91045    133.22175    129.00159    129.75416    129.22951    133.16131    138.75287    140.79174    137.89735    138.93382    133.86245    126.28076    122.68168    124.14775    122.38207    120.40687    123.53775    125.74966    125.85855    124.53484    120.17443    119.63943    118.68239    119.34158    117.84767    122.01319    118.76421    113.51368    119.81619    117.68326    119.21047    120.45666    121.60385    128.85855    125.29462    125.78772    123.16478    120.17141     119.6011    121.60244    125.93155    129.21336    139.03303    131.56609    129.13491    124.90132    121.61903    119.70889     115.8889    121.21041    117.65707    121.22323    127.88009    136.10165    149.35079    158.49237    158.16949    156.88343    142.09727    136.32051      125.785     122.8323    122.69437    121.21911    121.17999    117.86046    118.53791    117.10471    115.25877     114.3335    117.27886    113.37891    117.59843    114.30436     119.2752    116.51405    119.04999     116.5761    117.93389    118.48008    117.42053    120.60235    122.10116     123.9739    123.78252    123.55714    123.55161    121.22704    125.49421    125.67701    130.11445    128.09916    133.39682    129.34518    128.48216    134.92355    128.50729    127.16142     123.9403    123.73468    114.87323     115.2371    115.74338    117.01607    116.77367    120.26455    117.00411    113.17587    117.12457    121.01099    123.39176    119.69345    123.57919    121.67953    126.34734    126.15322     126.0238    123.61804    122.58409    120.03998    118.62904    123.04336     119.4467    117.95321    122.82281    123.02296    121.18202    121.39978    122.79288    119.35193    116.44617    119.63545    113.05257    117.35421     116.7092    117.71038    117.44643    119.77936    117.03346    117.48421    119.13609     118.9038    120.00322    124.99617    123.35756    122.47476    124.77202     126.8824     124.6276    125.92469    122.86383    123.66132    122.66108      123.695     127.8046    124.72711    127.83835    128.80752    128.46724    130.18388    128.76912    126.81549    128.43161    125.60437     125.3109    121.64198    123.94968    123.17611    123.33217     123.7327    125.60889    121.31612    123.85507    128.61014    124.82402     127.3042    131.22948    127.32159    135.34705    131.98666    134.34724    138.09155    137.21222    133.74631    134.20688    133.09138    132.50724    128.68921    126.93897    122.78715    124.37253    124.40148    124.16005    124.66812     125.0762    122.54597    126.51505    129.72054    128.84353    127.49313     127.4713    129.90536     126.3718    125.38473    124.56456    123.91629    126.19994    123.19719    122.26591    119.84618     122.4665    120.14334    125.05791    118.88223    120.89383    117.99922    121.58086    123.69417    118.84776    123.14802    123.01831    118.66283    123.17565    121.72154    123.17528    120.92118    123.70548    121.31438    125.68865    123.43638    126.31087     121.5991    125.22193    121.08211    119.86124    129.19462    122.74652    124.10465    120.34841     119.5645    122.28934    126.48422    128.10093     121.0557    118.81705    121.41966    123.27036    126.94606    126.39707    128.71943    126.43515    125.34027    124.89706    129.97676    125.44252     125.1359    126.17432          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       21.50    273.05824    263.75965    241.29179    227.22553    224.36798    230.69291    219.88097    218.05316    206.12235    198.18689    199.22851    188.29322    190.93109    195.50597    190.64383    187.83314    177.49136    176.21912    177.12717    174.39848     174.5113    172.66605    168.72816     168.7887    171.14346    182.53995    195.99053    205.81822    197.90403    173.98522    159.23439    160.16078    158.19193    157.58206    153.43514    155.92097     148.9481    148.63125     149.1767    150.71418    150.77831    154.25926    142.90864    147.06361    145.06472    144.80199    144.30508    141.58481    146.16888    142.06863    136.71218    140.07317     139.1297    137.37853    143.30188    152.09924    156.30505     173.4486    179.49291     173.3305    158.07668     147.1392    137.85099    138.22867    138.24108    137.01023    134.18174    133.28403     140.9939    146.07093    157.69792    162.58487    171.19603    164.58028    155.58549    155.90242    164.66658    167.79829    163.89648     168.7892     162.5555    156.72859     154.2214     150.6472    139.17334    130.28216    128.94695     129.7346    130.84117    131.70209    137.74136    138.50829    139.43297    134.74438    130.19764    124.57455    124.21855    123.41115    120.52858    121.44139    121.20282    126.69517     122.7561    124.17059    118.22527    114.10696    119.17769    119.49775    119.65834    117.32882    117.03306    118.47342     114.3199    115.25911    117.37314    121.67551    122.35053    129.34179    126.01349    122.26389    120.58211    119.97556    121.96888    121.19332    130.17199    132.99267    139.89507     131.9704    131.23853    125.57669    119.86204    118.40194    115.89458    118.74507    118.36998    120.91946    128.68121    136.10718    149.36441    154.37512    158.97154    157.76878    145.51999    141.80918    131.03141    123.22289    122.31518    120.79974    118.92058    115.35752    118.63883    113.00547    114.10863    113.72102    113.41444    117.34587    115.64258     116.8429    115.66632    114.41507     115.6356    117.18636    117.53703    117.98449    120.09902    119.31917    124.33746    123.76932    124.13924    123.49155    127.20172    128.34712    127.83352     132.6803      126.872    130.37099    129.36717    128.94936    126.49361    128.99738    130.39853    125.48085    126.56661    124.82632    121.91508    117.17351     118.1791    116.83624    117.65315    119.13414    118.04725    114.61177    117.49368    120.03177    118.90238    120.36961    120.46743    124.57129     123.7396     126.7931     124.4587    125.34433    122.84464    120.33379    118.04819    118.39733    119.51703    119.75847    119.85022    122.38312    121.99689    122.75467    122.85804    120.97496      120.967    119.10223    115.01256    115.25025     113.3819    113.66546    115.57622    114.44305     114.0285    116.28284    116.45143    122.01405    118.19344    118.33072    124.59325    123.88196    122.33587    125.30977    125.23788    122.46588    124.02846    123.01886     124.9812    129.06354    125.14285    130.87399    126.80047    126.42098    126.28624    126.31141    128.91301    129.95234    129.94665    127.18402    124.48753    122.79243    122.76744    122.93036     125.5421     125.4678    121.05678    121.19054    123.30442    123.10933    124.97412    124.93783     128.5237    129.63148    132.96027     135.0806     136.2368    138.71202    138.24189    136.21312    134.52787    133.16338    132.11014    128.24934    127.66291    126.96659      121.236    126.06155     126.0138    123.30695    125.82523    123.04411    123.21628    127.96471    126.50239    129.11479    127.74938    125.98809    127.27346    123.86007    124.49163    125.87173    120.38417     123.2712    121.46349    117.61061    122.13572    120.37068    116.28949    122.26191    122.25982    114.81566    119.83338    120.85861    120.71192    122.31357     119.1775    116.44867    118.81398    120.96002    118.50295    120.56011    118.96489    116.96371    120.78562    120.50764    120.51627    122.24139    120.95205    116.49502    119.84182    118.95692    122.31103    117.42446    114.81676          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       22.00    274.30192    264.43307    241.96366    229.83831    223.54105    227.41338    218.74331    215.78624    207.19708    194.53953    201.34062    193.45982    191.55979    194.22494    187.24194    189.30397    177.01511    179.24139    174.28595    173.71838    172.77619    167.96418     173.7461    168.38007    171.32382    185.41292    197.89461      205.568     195.2404    177.17445    159.92279    158.36537    160.22038    153.13239    150.35718    157.99375    150.71371    150.52266    143.32812    147.33018    150.50097    152.87085     144.0034     145.8423    146.76216     143.0148    143.84195    140.47749    139.50243    141.52145    142.78575    139.90214    142.25096    139.99153    145.20702    145.60549    160.09809    172.34698     176.9506    170.75224    159.94624    148.28202    142.02139    137.92319    137.79058    135.81556    136.75547    136.35627    140.58489    141.56495    151.57585    163.72146    172.62732    165.25348    157.67626    159.52103    165.67667     172.5064    165.65916    162.34088    161.29706    157.84851    157.33684     156.0363    145.12462    141.13588    127.80929    125.00589    124.92936    132.59187    137.80914    139.51122    141.15032    136.68151     131.4708    127.35525    126.53403    122.48535    120.56849    121.96327    122.74616    121.11737     122.9812    120.66372    121.84006    116.26963     115.9329    116.52505    118.68528     117.5091     117.1882    119.17378    113.87996    118.63949    118.53958    122.20876    122.52042    128.01727    129.70462    122.49911    119.94162    119.69996     121.3747    121.53258    127.65802    132.30234     132.0955    133.41075    130.82387    127.25512    121.41608    117.12384    114.52303    116.39892    121.65792    121.82177     127.6574    137.93485    147.82429    155.56521    158.30346    152.56737    148.77762    139.26793    133.90896    127.99393    122.75419    119.59947    117.00097    116.63767    120.18412    111.87955    110.90463    111.72741    113.58504    115.01484    115.04701     114.1234    115.34988    116.88278    119.99608    119.32903    114.32256    120.77806    115.58617    120.51072    122.07768    120.37518     125.9433     131.9814    137.58785    143.51539    137.92895    139.81662     134.7773    130.83365    128.98673    128.01343    129.43156    123.04461    127.86551    127.15164    125.87751    124.03581    120.63012    115.14473    115.23182    114.87764    115.23195    118.63806    114.43935    115.12037    116.51015    115.65598    119.17273    119.17588    122.27341    122.11077    122.95349    124.72504    125.20109    122.38163    124.06156    120.14992    118.43416    118.49599    118.96765    120.67329    118.18616     122.4943    122.69828    119.37103    119.04707    120.01535    118.72699    112.09369    117.12633    115.93763    116.04394    116.14093    112.94775    115.95001     117.7643    113.07973    115.21179    120.24869    118.40888    119.34696    121.05754    121.91593    124.72261    126.66666    124.94057    120.34393    120.69329     120.1863    124.36757    127.59056    126.88385    131.11296    129.50811    128.74137     134.5215     127.8112    128.71438    127.71712    124.86595    124.58623    121.01948    121.45866    118.59949    119.77396    123.30176    121.04735    120.58998    123.98287    120.51942    124.82422     126.5185     128.4131     127.2347     125.5425    129.40684    131.65048    134.06148    137.23105    137.10744    133.87682    130.75512    133.39517    132.38853    125.56885    125.34485    126.15884     122.9352    124.02684    121.06677    123.00399    126.99049    127.55164    126.00178    125.74165    126.68346    128.26657    124.64246    125.27958    123.28425    119.30899     122.6175    121.53551    119.22717    129.48307    116.49847    116.82434    119.99472    114.67943    118.96029    117.45963    117.77117    115.92095    115.00692    115.05762    115.12901    112.10704    116.22769    112.65566    110.64263    111.26646          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       22.50    274.56167    265.29433    244.64953    232.79415    221.48539    226.18552    217.76851    216.15964    213.24239    199.65194    200.51695    195.13802    191.08202     189.7893    182.27672    188.67722    176.82305    180.27478    178.16665    173.90877    170.53423    175.18992    175.26985      166.409     173.8542     189.0544    199.08353      204.242    187.18561    172.39769    163.31099    157.16047    161.07899    157.35433    152.63709    159.74572    150.63374    153.60918    144.96427    151.45211    148.88982    151.72648    146.40846    146.67423    146.46241    142.37529    142.66747    145.79169    145.48269    144.23718    143.14727    137.85744    140.28787    141.20698    142.24904    147.35546    158.72927    175.18121    177.42947    171.71357    157.35241    146.64782     140.3305     137.0939    140.04233    140.55462    135.29749    138.75794    138.77067    142.56474       156.11     168.8457     169.8723    163.64331    160.57731    163.47899    171.70379    174.35244    167.94208    160.21247    160.51943    153.68584    161.57804    161.03725    153.49363    138.19906     132.2398    129.07827    131.08834    130.97626    133.96467    139.27251    137.27302    136.74178    125.30013    125.45005    124.81303    120.56956     118.4057    119.10614    122.28657    122.43418    123.86904    122.58687    118.70326    118.96809    116.61678      114.441    119.79162    116.46465    121.09956    119.70388     118.3111    113.99505    119.40397    121.07103     125.8364    127.97827    128.06727    118.15124    121.19436    119.00548    120.23482    121.55878    127.97181    135.44981    132.85512    137.94093    127.40752    123.53626    122.33387    118.12448    118.64037    113.55758    120.47725    122.04076    129.10509    139.55323    147.74016    154.34051    151.97173    148.19979    142.35671    135.03841    133.57596    129.96891    122.19282    121.77882    118.62152    115.26637    118.27622    116.76703    112.26628    110.21412    113.64717    115.61633    114.31372      115.729     117.7884    115.50448    120.60415    115.90006    117.45605    119.51845    117.39009    120.49228    118.43231    123.74399    130.14705    136.43486    150.68999    157.49139    144.21208     134.7946     128.1232    127.98899    126.48034    126.13798    128.68356    120.61767    127.05035    127.86864    123.42352    122.12829    117.46579    118.22908    114.36036    117.83538     114.3672    117.82384    117.07744    115.15536     115.5373    121.00298     119.6436    121.66925    122.05771     120.7471    121.66515    124.02823    126.34627    128.85108    123.63933    121.73997    122.04729    114.56682    121.62113    120.42044    123.01198    121.86754    121.77866    120.28012    118.07488    120.44623    116.85924    115.68881    115.19161    113.73947    112.33475    114.89136    114.12843    112.84875    115.11571    116.16664    114.11646    115.25226    117.48031    119.66688    122.43495    123.37591    123.76576    119.60182    122.03367    120.91009    122.97135    123.42569     122.8221    121.84482    126.54168    123.68367    129.19538    124.47571    127.05301    128.07274    124.01284    125.79423    123.36338    121.86518    120.84583    120.86842    117.34586    119.27325     122.2988     122.5109    120.74445    121.84335    120.03484    125.46122    124.33558    126.72306    126.12972    128.64832    128.39807    131.11497    130.97982    133.71617    136.11177    133.16031    131.58086    130.37093    129.24897    127.15164    127.88639     121.6857    124.04676    121.09486     121.4481    122.93513    117.88913    120.22171    123.29504    120.39597    124.84617    122.80227    122.67033    120.34676     119.7965    120.13955    123.00984    116.91843    113.69953    112.03198    111.47395    108.54679    110.75408    113.76299          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       23.00    274.86384    266.36029    249.30988    233.87361    219.57157    228.17042    220.71008    217.86888     216.3352    204.52237    200.18863    191.68921    190.86442    187.24883    180.75797    187.60362    177.23916    181.20276    184.34086    171.69198    174.08681    177.26724    168.76438    164.25455    172.99047    183.22053    200.48193    205.56919    193.94619    174.63712    163.54792    154.48668     158.4602    158.70248    155.67922    157.34263    157.57848    147.98773    147.76044    147.98495    153.30114    151.64983    150.28025    145.13394    139.65673    146.80103    142.64604    145.10842    141.63705     140.0381    143.83252    139.26715    136.90427    138.04292    142.82807    145.30139    156.88159    175.05899    175.76869    169.08932    155.99862     146.1495    144.58621    139.14896    138.84123    134.82286     132.3652    139.31453    140.81159    141.10362    160.04632    172.45259    171.87539    165.37449    165.28729    164.89893     174.0658    171.61461     168.2722    160.35452    154.96094    157.03172    164.27025    162.95143    155.44542    142.41345    134.26353    128.44022    129.71763    129.71229    138.97327    140.99568    141.33237    138.08782    132.83041     127.0377    122.82967    121.75872    120.76975    120.89323    122.77331    127.34269    124.65935    121.31422    116.81257    116.35203    114.47754    117.60954    114.06363    116.38492    118.77626    118.12928     119.9222    114.81372    119.97907    124.32964    125.54867    125.09559    124.23333    122.71568     119.6614    121.03131    119.92285    122.73487    126.59139    128.00325    133.05627    129.72726    128.18386    123.82453     121.3624    118.11893    120.62817    112.67108    116.80882    123.62962    130.69855    134.67712    145.99322    154.13427    153.59877    151.04481    141.93009    133.20033     131.5175    125.01748    122.75489    119.33807    119.35595    118.64873    116.95463    119.18577    113.98339    112.54315    113.91417      114.286    113.59952    114.43115    116.75298    120.04309     115.2069    118.98113       117.64     114.6308    116.17602    120.07101    120.16764    123.87865    127.22618    135.61974    143.39728    139.96796    133.30684    122.41216    126.36957    122.75308    125.17826    126.20026    125.71041    126.65733    122.79084    127.07604    122.81435    123.88998    122.41772    123.96062    118.86099    115.99019    115.94896    120.06199    117.89832     113.3204    123.02015    117.02339    115.93185    118.93685     121.9803    122.54081    125.08244    123.94789    125.47254    122.96127    123.52563    119.83464    119.49747    115.74018    118.44316    117.93013    119.09643    123.59176    121.71727    119.29331    115.87715    118.17801    118.44824    115.97393    116.89966      114.179    113.48022     112.9133    113.63413    114.83612    117.85532     112.5577    116.00627    115.80506    117.84854    119.68378    121.29462    121.65245    122.78332    121.69141    121.25932    118.51713    121.75445    122.49716    122.92172    124.07029    120.81562     125.9865    121.69367    124.38546    125.16881    119.24152    123.67996    124.58294    123.25267    120.66248    118.92795    120.17578     117.9215    119.99275    116.84108     120.1878    120.49074    124.10812    123.42492    120.82239    122.68162    124.33051    122.60688    124.51677    125.54591     127.3115    132.77861    135.12477    133.02002    132.61563    131.54578    128.06082    128.83196    127.21487    119.89745    121.41357      122.133    119.76087    118.86103    118.50179    118.06025    117.18488     116.1279     118.0666     113.0712    114.85468    112.03871          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       23.50    274.63967    266.62788    253.59972    234.87314    221.41063     229.4611    226.06381    216.59512     213.7663    204.20507    199.66242    189.90003    189.09126    188.91319    183.23292    187.38683    180.27987    181.10387    179.08383    170.42298    175.69928    173.39404    168.90655    164.95129    172.53128    174.69393    197.30374    204.17945       196.41    180.13649    163.84473    163.42236    155.89066    153.84607    151.91874    152.01233    147.78127     149.2732    152.16657     150.3786    141.74423     148.9379    151.65116    145.01019    140.69832    141.70696    141.25944    142.60694     148.0972    142.20918    144.58646    140.56793    141.91762    143.83035    143.34978    150.09961    159.80292    172.14443    176.49701    172.47459    156.11581    146.06762    143.89225    139.15669    141.47238    132.25336    138.59007    140.20273    138.75424    146.45269      158.418     169.2672    165.04321    170.75934    159.23855    162.00517    176.63131    170.46968    167.14284     160.0023    158.04629    160.51684    164.21209    164.45634    158.74267     146.6814    131.04251    128.01914    130.80655     138.5247    136.13355    142.40055    142.33495    132.34007    135.81865    128.18062    124.10548    123.92611    120.67485     121.5451    121.40229    126.05214    124.27488    124.00132    119.34443    118.02191    117.52989    115.84895     120.8573    118.68783    119.03993    116.23361    116.13844    114.02154    118.37739    121.17283    124.47259     126.5214    125.44011    123.46685    119.59894    119.48123    120.40746    119.42866    125.18629    132.98789    135.24437     129.2502    129.11454    123.64857    118.83004    121.32158    115.81755    116.47364    115.41968    119.69307     130.2248    138.09873    148.72847    156.19124    155.87596    149.27499    142.75304    132.69714    127.35088    123.01891    122.61653    118.84008    119.75301    120.06281    120.31412    119.96665    120.33076    114.83269    112.83001    114.42468     118.4354    113.34456    118.04886    120.32675     116.3151    115.69819    117.80466    117.41464    118.64726    117.45143    118.47059    120.38798    122.09704    127.52039    126.72395    126.50492    126.67949    125.52138     118.9294    124.72382    123.92297    124.40892    126.61173    126.23663     121.7371     122.1628    122.62907    117.61083    119.11792    117.65141    117.53253     113.7588     112.9873    113.08253    116.71444    116.47754    117.16252    115.36559    117.63768    117.44116    120.56738    121.93576     122.1484    123.46458    122.94815    125.83199    121.35779    120.97578    119.06319    118.62647    115.36261    119.23604     120.9083    120.47249    121.81924    119.59145    119.25892    118.17566    115.98415    112.79254    113.60741    112.95798    115.30208    109.20667    110.86792    112.08406    113.99233    115.32356    114.41474    112.69795    117.79951    120.65078    120.17091    123.40576    119.66958    121.01751    120.27461     120.4479    123.09251    120.66573     120.7389    124.67251     120.9871    122.69099    124.19652    122.16646     122.6188    121.28643    123.45002    118.56241    117.60484    120.31433    118.37083    113.91519    119.02615    117.49377    117.81654    120.45755    119.98267    117.42879    118.99355    120.41622    119.86647    118.90737    123.09063    121.50313    126.38509    124.91654    127.80413    129.23432    129.32235    129.98967    126.67306    127.60683     119.6287     121.3881     118.6169    115.69903    109.40122          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       24.00    273.74762    263.19725    255.93685    236.99643    220.78541    224.19793    222.34542    216.00207    211.70735    203.55583     193.8458    194.52277    190.18475    190.00666    187.67456    186.90811     178.1051    179.90005    175.35344    174.14098    170.40526    176.25742    170.55734    164.49845    177.08133    176.92364    199.91308    202.94995    197.15248     182.6091    163.35167    158.18505    155.74567    149.11692    154.96154    152.07001    148.74235    151.30462    151.76797    147.73655    156.62849    145.45981    152.69335    147.72271    143.55065     146.0255    141.83173    143.45676    145.23629    143.15226    141.33604    140.92802    144.38889    144.30041    140.43043    146.86626    153.62692    171.75821    182.29001     169.1606       154.26    147.69448    142.49246     134.4561    137.15821    135.60647    136.69802    137.63952    142.39923    145.42608    156.56135    169.58308    174.11901    169.98158    158.17778    160.91456    168.16989    171.34297    172.06175    169.01347    158.51316    159.90722    157.86625    155.83181    154.71841    143.57112    134.39757    127.97725    126.89872    135.00073    139.31462    143.91978    144.99409      137.647     132.0396     127.8082    124.96872    121.17638    121.08223    121.93759    122.81926    122.96559    125.67232    122.76183    119.31421    117.80408    118.15532    116.06393    116.76574    119.55093    119.10503    118.32994    114.32328     119.3421    116.25131    117.82272    123.28897    126.26088    128.20251    122.04648    116.45312    116.39119     120.8778     120.4746    125.71852    129.84749    134.89328    129.88681    126.34105    124.08759    120.38401    115.58462    117.16944    116.56375    117.82462    122.66522    129.18276     135.5983    148.60069    151.89057    156.75315    149.36769    142.70101    128.21547     123.2805    119.75796     119.6588    114.81733    120.15365    121.70115    121.50942    118.27927    116.48738    117.21618    112.30206     112.1707    115.93733    115.09016     118.8657    118.46716    115.42163    114.33402    118.35506    116.02293    119.92466    119.27929    120.72779    118.18254    122.59373    120.48864    122.86032     124.4943     124.2655    122.34749    120.89367    125.45987    123.82878    121.42354     125.4835    122.49969     124.6778    119.51174    119.51119    116.69526    117.02314    115.18061    116.45918    113.33856    111.01742    114.76657     114.4149    115.34555    115.61724    117.03262    115.25433    116.78368     120.3815    119.19948    119.83271    121.99763    121.73354    124.12721    121.42959    120.94112    118.18005    117.22093    113.96169    114.42784    117.66759    120.35589    121.82119    120.66607    117.90673    116.01887    115.61241     115.6959    112.57272    111.69573    109.52849    113.56228    110.20488    112.26985    112.86766    113.21219    115.89404    114.34336    114.90434    122.03149    119.88141    121.16945    124.36049    120.82089      119.273    120.21327    119.95434    117.78766    121.56681     119.6103    122.57692    121.25371    123.34621    127.18298    124.80861    122.13074    120.74942    116.25148    117.61383    118.60353    118.57855    114.98335    114.30832    114.75186    111.75485    116.36568    117.72129    114.81988    115.56067    117.39235    114.12916    119.55844    117.27288    120.32859    117.96485    123.84241    116.14601    117.78804          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       24.50     271.3246    259.73457     254.7333    236.74323    218.94076     218.0959    219.61852    215.83519    211.55249    208.31385    189.83844    197.95576    194.81662     188.2147    192.80923    187.39285    176.58255    180.15488    172.44906    178.44901    170.39004    174.90646    176.37154    170.66504    173.46546    182.86101    205.88734    205.85722    196.25456    178.98999    163.42351    158.98311    157.54626    152.41197    151.53029    153.81611    152.19304    154.66566    152.32714     149.1148    153.13243    147.44204    153.36261    148.84914    144.46107    146.86569    145.02518    142.30229    145.65837    145.35936    143.73977    140.15839    142.53487    142.73201    141.83085    144.99426    157.21803    171.37489    179.02148    171.97315     157.3265    146.29368    139.77645    139.46187    137.00167    139.00712    134.70618    135.36321     138.5363    145.41527    155.05533     168.5942    177.15756    164.17759    156.49307    161.48506     164.1278    173.61283    175.51537    174.04495    163.84024    159.39666    153.98325    152.78522    150.42023    148.52188    132.69947    129.50255    131.66094    136.58036     135.9379     147.8281     144.2556     141.4921    133.45961    133.17852    127.25189    121.12462    123.63284    120.71457     124.1705    121.08246    122.76788     121.4704    119.16113    120.28002    116.19594    110.25189    120.01423    118.17959    117.50296    116.89639    118.39266    120.21905      115.791    120.90429    121.42502    126.45157    127.53929     121.8983    119.66068    116.35863    117.96174    121.98757      124.836    133.47157    130.23982    132.63075    127.02314    120.43852    119.67443    117.76206     114.6072    114.62817    116.48556    118.99842    125.90042    137.18582    149.61545     154.6917     157.9036    149.77829    141.27353    134.70572     127.0682     119.6103    122.41638     116.2676     118.6658    119.53181    119.41878    116.48729    115.76398    111.95245    108.44137    111.83686    112.57546    113.48705    116.96154    117.10855    113.94696    114.68836    116.01049    117.25893    121.02584    118.42925    121.32017    120.44004    118.63125    122.44332    120.04782    120.63641    119.25991     120.1171     119.7289    120.29031    126.02138    122.67517    122.90398    121.55942    123.84567    118.27642    119.51767    116.34673    116.56942    119.66061    113.56926    113.27331    111.21862    114.36476    113.03485    113.43443    115.74747    114.81639     116.3468    115.75377    118.49665    124.49494    122.18218    121.81885    123.04026     119.0692    121.10941    119.42296    116.55568    116.21565    114.15547    114.02354    117.62458    119.74537    120.48688    118.97993    116.40631    116.80595    112.56339    116.57875    113.99739    109.86901    111.52042    111.96713     107.1103    108.25384    113.33233    112.50151    112.29801     114.9882    115.67327    120.76625    120.43641    117.85092    119.41404    120.12432    117.46853    116.61094    119.38827    119.62686     118.6364    121.09398    124.12464    122.96325    123.95821    124.74214    125.99635    121.32804    120.95999    117.94225    115.12593    115.50923     112.0553    112.49159    113.69329    110.74223    109.18918    109.72212    112.92783    110.50988    110.12904          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       25.00    269.63586    256.39017    252.01553    234.81045    220.39231    221.00245    219.97812    216.39578    212.58981    214.81702    193.35881     195.9923    197.61858    184.75702    195.06399    185.59292    182.48479    180.57093    170.76582     181.4594    175.08796    172.00864    178.54699    168.35385     173.1985    186.68298    207.82095    206.20262    196.00357    172.02405     167.1014    159.01242    153.84878    155.58198    153.38348    155.58003    152.11809    157.23655    153.11834    146.98194    150.99844    151.62104    151.29258     152.6424    143.37274    146.59038    143.68897    145.46256    143.55566    143.82855    140.88978      142.791    142.20249    140.78541    137.91363    143.58292    154.94746    173.45963    179.55822    170.24218    158.06826    148.18421    141.84984    139.44212    135.46094    134.66195    133.42221    137.22189    139.75745    144.61392    156.06086    173.81797    173.56095    162.80048    155.26569     155.6721    166.02395    178.72319    178.77654    172.86705     168.8165    161.09572    153.70129    151.55641    146.23981    137.91733    136.31361    128.50958    130.05852    132.45568    142.68413    148.20787    149.64321    141.35782    137.57855    133.93715    129.56406    123.52012    118.82726    119.00484     121.3727    123.52515    125.51848    123.44781    119.26777    117.47576    115.49787     116.6247    115.50116    115.84268    117.24122    116.21078    115.78947    117.65034    118.72924    119.65708     124.0327     125.8071    124.62895    122.64535    119.13793     115.3616    114.58651    120.78067    124.11127    126.30271    133.13657    132.98592     126.9881    124.40521    120.30249    116.84023    114.41761    115.42572    119.14206    115.55171    126.68514    135.40775    146.96856    153.01499     153.5569    150.89396    137.28824    131.49829    124.65195    116.63532     117.4894    118.76667    115.84605     114.7456     115.1827    114.33199    115.00682    111.66992    112.61584    111.44461    116.73408    113.96154    118.05971    116.34305    119.11018    115.04469    112.74721    115.58428    116.79098    119.46299    121.58285     120.0178     121.2484    116.15186    119.83792    120.91633    118.93226    120.18209    120.62992     121.5489    117.12014    124.19169    125.31363    122.73169    123.01422    121.66512    115.22344    120.18725    116.54858    115.63977    111.89331    110.76884    110.73843    113.73783    112.76331     110.9806    109.92406     115.6845    115.80786    115.86354    118.18386    119.21111    121.29406    121.26571    122.99206    123.12046    117.46884    118.72296    113.67928    114.54003    116.37281    114.92402    115.41782    117.93227    120.43959    119.23591    118.56152    115.75401    116.05716    113.66623     113.4914    111.20335    114.13869    114.64456    110.62228    104.96564    110.73884    115.75053    114.08951     113.3786    115.28061    115.99318    117.57839    117.41365    118.93832    116.97455    118.23978    119.02437    117.68226    119.23261    118.12072    118.82496    120.87818    122.70127    122.68847    121.24178    122.56631    117.72664    117.80942    116.87394    111.60408    109.17883    108.98958     104.5387          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       25.50    270.49175    258.47355    250.57478    235.00783    221.33515     223.8698    218.95587    216.75686    209.83302    213.12258    195.56334    194.49665    198.93854    188.55674    191.38381    184.84125    183.16611    177.03296     171.9723    181.23293    174.00829    171.02586    175.08606    164.70423    175.54529    181.53889    202.73963    205.46639    194.37615    171.84341    168.25394    159.91055    156.77938    148.30556    152.94039    154.13308    153.82597    154.65282    152.51562    150.14134    146.14622    148.35931    146.14006    150.08146    145.86387    144.47994    137.62432    142.45925    141.08077    143.62349     146.5832    144.26008    141.68558    137.34086    140.54166    143.19822    160.47304    171.49075    178.07205    174.42486    155.01725    142.36834    140.16245    137.45601    133.27633    138.84945    136.39836    140.60005    138.32202    143.80123    157.69072    166.51323     172.5455     166.4178    158.59491    161.76123    168.31458    181.33964    186.22429    177.07969    173.05847    162.74394    155.35519    152.54276    145.43117    138.39067    131.95906    124.62959    130.78104    133.52087    138.96429    147.89954    151.73097    145.61814    135.98232    134.50829     127.1357     124.6234    122.55841    123.26037    121.02783    125.09005      122.335    125.63328    120.83917    117.72778    115.71312    117.16668    114.76401    122.31679    118.39244    118.75247    115.34245    114.88733    113.45405    118.32984    122.51137    122.36712    128.00667    123.02632    118.98751    116.86347    114.23098    125.35533    122.24048    127.83908    134.21242    131.88822    127.62564    118.84561    117.15299    116.23444     116.8557    113.69524    115.75692    121.89541    123.81374    133.57904    148.32895    154.42255    153.01304    147.26621    139.55956    130.82946     123.6832    118.69769    117.40725    116.85385      116.899    113.18245    116.28675    113.15531     113.6523     112.7865     111.0873    111.78199    108.76783    113.48773    113.18966    115.22468    113.02157    116.26287    114.41143    113.53999    119.26417    119.64683    116.90277    120.53577    118.25635     118.1659    115.79653    117.46223    119.16558    121.80829    116.88021    119.40683    118.01216     123.1573    122.43448    121.97778     121.5167    120.06498    116.88344    117.73237     115.1616    113.08838    113.84011    112.61496    111.25049    112.67089     111.6157    114.74197    113.59836    113.67799    113.30187     114.7658    113.23321    118.01355    119.05767    122.26901    117.91696    120.75007    113.78936     117.6605    117.57691    110.76562    115.40839    114.66106    113.35015    116.35654    118.28001    117.45771    114.25236    116.59536     114.7988    114.08757    110.37536     113.8492    111.82971    108.41633    107.40606    112.77813    110.38647    115.56106     111.9785    112.21313    114.22281    113.04948    116.75697    113.35433    118.01891    115.26895    115.64488    114.70535    114.59841      117.242    117.80403    117.13109    116.38646    117.44497    112.26638    111.59125    109.80861          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       26.00    271.65407    260.29598    253.04287    239.22624    224.00068    225.48163    214.88918    216.15021    208.80533    210.18323    199.27563     189.5088    193.17285     191.4496    188.40155    182.36623    182.73877    174.43282    177.46479    176.68619    172.61039    171.99611    168.08714    170.91415    176.49315    181.43108    197.29768    203.44774     192.2162     173.9023    168.11857    160.75642    161.96064    155.04556    155.51918    151.86855     155.9131    152.79537    153.64928    155.72493    149.78697     149.8699    148.20881    147.45622     149.6976     145.9153    145.93418    143.50675    144.31991    143.51364    145.49958    143.28914    144.13422    143.40634    138.91305     144.6851    159.74856     171.7177    178.95667    174.59874      157.362      145.646    140.86242    137.34059     136.9237    142.46504      141.059    137.09641    141.64022    147.02366     156.3357     168.7446    173.64454    165.56955    154.47505    157.74532    173.32117    182.63061    186.55863    177.67658    171.76659    162.72843    162.74085    154.16159    146.58042    138.48449    130.48088    130.26344     132.3977    130.92839    138.24771    148.17294    149.34878    141.14733    137.71542    130.23834      127.942    119.39397    123.27304    122.64796    122.92384     122.9396    121.38728    121.04904    117.72715    116.63515     117.3367    117.75322    116.73335    115.69398    117.88522    117.14504     118.7286    116.36665    118.85821    117.79109    120.20467    126.46508    123.53836    122.02747    122.46207    117.59379    117.36462    117.09509    126.17226    126.09474    133.53897    131.52906    128.73885    121.33183    116.37312    116.12871    114.68603    117.25603     118.5587     118.0948    124.34493    134.70311    146.73335     152.7517    154.49599    150.70592    143.76714    129.60751    125.56517    120.99003    121.04482    118.32102    117.43942    112.77957    112.84254    114.25581    113.78937    111.37755    113.16653    112.35494    113.64272    113.72768    115.41081    117.67151    115.85178    114.64212     113.3885    116.02449    118.67191    119.50512    118.55933    123.68512    121.31438    119.42454    114.39498    117.70389     115.6127    117.45668    118.37412    119.94584    119.03728    116.71637    121.54705    120.05429    124.55538    120.29708    120.30457     115.5163    116.74299    116.18606    114.77486    116.14065    113.57367    113.11536     112.1377    111.57377    115.43936    119.00801    112.74158    114.40109    114.78108    115.15064    119.24842    120.38984    119.50395    119.78112    119.28094    115.03407    113.61065    115.59079    112.84746    114.23855    115.88104    113.02575     117.0226    117.66391    115.58073    119.69201    112.42311    110.62284    107.87593    106.98647    110.04625    109.85419     110.0364    106.35193     107.9471    111.47578    111.10067    113.58961    114.34904    111.18639    111.26657    112.76547    114.80006    113.77215    113.97149    108.38105    111.76484    101.29332          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       26.50     272.3419    262.92522    254.92398    240.99148    227.08266    225.93607     217.0177    214.92803    211.82411     210.1069    204.33001    186.75442    188.23804    188.38309    186.60424    186.57262    182.38103    174.29363     179.9674    176.23721    174.94728    175.20339    168.13033    173.29748    172.54846     179.5335    195.56413    200.21078    193.25922    174.31365    169.84036    158.59162    157.42568     156.0973    159.14144     154.1056    158.84282    159.66884    153.63622    149.64345    153.46097    148.70502    149.15678    146.50924    145.61909    143.93026    139.50515    143.44626    148.39509    143.27504    147.55807    144.09418    143.69463    139.59583    141.97213    144.97807    157.33762    172.89694    177.45481    173.30774    158.35547    145.47265      141.961    140.55763    139.19775    139.59668    138.92101    135.80034    141.62867    146.00702     159.2898    172.85609    171.28188    167.05002    159.05424    157.21842    167.70396    181.59077     182.1297     178.4641    168.32242    166.35584    163.27962     156.0736    149.32387    136.48449    133.48547    129.77864    130.85902    134.53129    147.01007    146.33112    149.45052    145.47388    140.32974    127.94036    126.63428    123.69048    120.21055    120.32355     123.0361    125.67677    125.38376    122.51071    119.61422    118.47861    119.10135     115.0067    115.67474    115.91449    119.63656    116.12127    118.34165    116.29017    114.48775    118.78746    119.01084     123.9161     123.2912     119.8796     118.8781    115.69433    119.72271    114.30738    125.11301    129.65822    132.04924    131.57294    127.49615    122.42552    117.11293    114.62988     116.6944    114.56229    119.66606    121.46303     124.1291    133.16078    144.93699    152.56922    155.11657    150.00698    140.73184    132.83651    126.33293    121.03578    123.85832     119.6015    118.00647    114.52297    113.31497    113.84808    114.57073      112.611    113.54516    111.93711    115.77717    109.81468    114.42205    115.58717    112.65074    115.97028     114.3137    115.89434    114.45067    115.37263    117.00124    121.70183    119.91287    120.38667    117.64231    117.15498    118.15214    116.57495    114.61295    118.32058    120.51557    114.93027     122.1032     120.3639    121.36775    118.86312    118.31005    114.36993    111.32601    110.34118    115.74329    112.94343    112.91805    110.86434    111.46661     108.8596    110.19846    117.08666    113.84581    114.43556    115.82396    113.00885    118.32789    120.13926     116.8827    118.23318    116.30283    117.52884    113.34394    110.26287    111.72979    113.10378    113.67248    112.54238    114.81937    113.89779    113.54891    110.76703     109.3768    113.02937    107.35198    107.37718     106.0596    108.74283    106.93966    105.91558    105.92656    108.77684    107.61965    106.07761    105.97199    106.92492    106.45723    104.46108          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       27.00    271.87215    261.21984    253.65256     239.6144    230.05672    226.79535    218.94849    214.29015    214.72515    207.87415    199.63914    190.41633    189.15456    186.32712    187.03941    189.44204    178.27374    175.01214    184.05367    171.43014    174.70754    170.98714    170.58996    171.25616    171.35308    178.10834    195.65588    205.22193    199.10719    174.96751    166.89023    160.86979    155.16146    155.94539    160.00451    156.19839    154.58855    149.13507    158.07456    149.11734    151.32751     148.6467    153.48836    146.97356    147.60364    145.21822    145.72356    148.41127    142.00318    137.55187    143.46278    144.78059    145.77459    143.89183    136.96901     144.8783    153.58452    170.33538     178.6791    172.75158    160.58331    141.44624    137.98401    135.04378    136.74992    136.47488    138.62966    137.41277    141.00605    146.70471    161.29054    170.26088    175.03232    167.73218    162.45134    157.82313    163.10853    174.25486    175.39591    167.53844    162.03652    166.49443     161.9023    156.71837    146.74829    137.19697    131.97787    132.74185    129.61878    132.77955    137.19308    149.31653    153.39268    151.88945    144.24395    132.83991    127.69764    126.96412    125.37578    123.07287    123.56179    122.52043    124.51937    122.40924    117.68851    119.87795    117.56949    116.62253    113.86568    115.62307    115.58226    118.42292    115.62698    117.24749     117.3609    116.88503    123.57541    122.40311    124.09619    122.39869     123.2754    118.64168    117.94451    118.50179    123.35593    127.92714    131.36581    127.71606     127.9259    121.94372    119.17853    115.68305    115.15924    111.50716    119.39877    117.92034    122.83564    130.58966    145.66064    152.22707    155.86067    154.92004    143.22537    134.53685     123.8517    121.05149    120.79204    121.38909    116.34468    118.76724    111.83163    113.52603     109.9971    110.29616    110.84272    111.06118    110.02935     111.3397    113.28005      117.248    119.94555    113.99003    116.09382    114.44993     113.9003    115.97881    116.54667    120.63348    122.23792    120.65762    118.83962    117.81223    117.69782    117.44925    118.23571    116.66916    116.97772    117.07181    117.94658    120.53758    121.33807    119.22079    118.43494    118.08822    113.81543    112.40047    113.22554    113.14366     109.0808    107.32534    112.06287     108.9021     113.0408    115.00816    118.26758    113.78533    113.85268    116.98642    115.88273    118.85604    118.98668    120.64371     117.7976    111.53859    107.73488     111.1325    112.22296    109.85067    112.08989    114.50729     112.4786    114.98949    111.00978    108.15959    107.93317    106.69795    106.31264    104.00123    107.73304    104.99142    104.12578    102.79292    106.35744     97.45821          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       27.50    273.66599    262.28649    252.71761     236.9362    231.83097    225.45259    217.76941    215.31877    216.82392    204.14663    200.51672    193.07346    184.54766    190.81186    190.80323    186.91155    180.32119    177.71532    179.97328    172.30351    174.44738     167.3352    173.59453    170.64352      177.259    180.21787    196.26265    206.23539    196.84595    177.19297     166.5692    165.86795    155.52599    156.38387    157.81896    151.26963    154.19479    151.26193    154.25938    149.84319    153.16681    147.33559     148.4028    149.28938    145.07502     146.8507    145.34439    143.14588    143.58693    143.08118    147.00986    145.15629    145.44973    145.31057    139.67931    146.13906    153.05065    169.09312    174.40879    173.36822    160.79115    142.42456    142.86062    139.99715    139.59605    140.72062    140.23267    139.38273    141.28495    144.39124    162.05757    169.61084    173.24733    168.19038    161.42901    156.32071    158.94902    165.08548    168.03142    161.78179    158.60642    161.77039    162.13286    154.87549    144.20609    137.52786    129.05207     129.7136    132.79191    133.91807    142.38857    145.95876    157.64099    158.08334    150.20325    142.60294    135.62042    129.96747    127.48513    123.16699    125.26288    124.91888    125.21216    119.47737    118.48634    120.53083    117.91366     112.7786    114.96279      116.843    112.67774    120.14372    115.06533     116.4307    119.96362    118.79935    123.68543    123.39107    124.68247    121.87933    123.31499    114.34455    114.80374    121.32129    122.35194     128.2855    129.87068     132.3357    127.85583    120.40656    120.65308    111.59541    114.42546      112.517    115.39568     118.6074    122.72481    132.70693    140.40124    153.66044    152.32554    148.30432    143.03106    128.14265    128.10696    120.81737    120.20128    118.67259     117.3536    116.72609    117.70043    114.91374    116.23806     111.1296    109.00526    110.10617    112.56672    110.47319    110.89539    114.72281    117.29191    116.29152    113.96497    115.93094    116.26854     114.8935    116.03391    117.76322    120.85043    120.93748    121.71756    125.72252     126.2276    120.61895    119.13864    120.86419      121.751    118.33439    119.76731    118.63367     125.0504    121.14758    117.81458    117.69662    118.21816    110.96062    111.01591    110.88535    109.70216    107.59469    108.55393    106.77181    109.72955    110.00528    111.47869     116.4369    113.96742    115.50561    115.13373    119.15106     118.8413    118.20674    116.15765    112.63093    111.24914    112.60637    111.03882    107.59311    110.87803    110.23909    118.18331    110.46528    110.46907    109.70179    109.26099    103.58363    103.57871     97.71715     93.62767          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       28.00    277.74952    266.32727    253.78148    236.02596    232.41871    224.89972    215.56163      218.337    219.24728    206.58111    203.18499    194.35144    186.79708    189.38061    189.76765    189.28903    185.37878    181.88367    174.84611    172.93774    174.03806    171.72991    174.50406    168.62983    180.25873    185.03629    196.65415    204.34135    192.67438    175.48238    165.91317    162.67738    153.29755    154.50384    157.17832    154.73472    152.28117    155.23912    152.78509    151.93228    149.43886    146.64261    146.48689    148.64504     149.5289    150.87074    144.50953    137.21486    142.24284    146.50764    143.36503    138.50286    144.82086     141.3038    140.26067    149.48044    160.29753    173.68343    178.02879    172.61267    156.59125    150.23859    139.43969    137.73937     138.1803     139.9416    140.67215    142.72015    143.93651     144.7755    162.47216    176.80788    172.19233    168.17563    159.21443    157.62908    154.10232    155.57849    158.61311    155.10036    154.94669    158.46173    165.06263    150.65486    145.91349    140.55336    131.66899    129.67796    127.53777    133.53662    137.52879    147.04471    153.01631    152.53504    152.87438    148.23838    143.66999    132.61053    131.59195    121.11204     122.9916     126.6162    125.54269    123.01334    118.01393    118.16758    116.01915     115.9655    115.17336    112.97744    116.14478    113.42657    115.24027    115.94967    114.94794    117.88059    123.49228    122.55597    123.29004    123.84461    119.11584    114.66634    118.53273    118.61687    124.08876    126.00285    131.80636    129.16485    128.78531    121.03384    121.61523    111.49116    112.16702    117.94989    115.59349    119.51103    123.68163    136.87977    143.46892    150.49453    157.31132     150.5459    142.95077    130.73265    125.40175    119.47371    121.50685    118.75401    115.50051    116.52312    115.43886    111.74309    110.88126    110.12608    112.13424    112.37281    112.54422    113.50444    111.76628    112.64709    113.61366    115.17658    112.63137    116.58558    115.75737    115.40972    119.34827    119.07353    124.58836    124.78104     128.1867    126.88402     126.7441     125.3634    122.21327    122.56644    122.19727    120.32039    124.29592     119.7291    124.26071    122.54501    119.40387    117.60892    114.84088    112.60815    107.02701    109.72304    109.28682    107.27101    107.71955    109.30152    108.56842    108.49333    111.15659     112.3002     111.5431     112.5702     116.1667    113.92734    115.20627    115.35527    113.41863      111.337    111.39115    109.17043    108.09406    111.46659    106.99123    109.51965    108.33202    105.49668    103.21268     97.94307          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       28.50    281.04652    268.16327    257.16316    238.86018    233.49602    226.19356    214.60008    219.25116    221.86298    213.01444    202.63503    193.90009    192.34913    188.80076     189.6615    187.36032    186.53754    184.11023     175.5248    174.09629    174.85865    174.66349     171.7805    168.90602    177.99004    185.74223    198.58789      207.773     189.0847    172.31972    161.95521    164.33591    153.84799    154.67334    153.36224    154.63226    154.22219    154.53522    153.35475     156.1778    149.65797    148.20561    143.74094    150.63593    148.13826    145.71933    144.81937    146.05284    150.51073    140.11428    142.10634    143.88402    145.18096    140.41149    140.43877    147.74186     158.4901    171.71787    178.41395    176.45946    159.12444     151.8775    148.01226     140.9135    136.28276    136.17913    139.64537    139.65082    143.52633    148.72656    159.16222    169.22977    173.10441    169.92827    160.46841    155.31776    157.02063     159.6104    158.44115    156.24305    157.02495    164.09796    168.14978    158.65364    149.49594    141.31407    131.74596    130.84559    128.99102    131.86765    138.31617    142.79867    147.67242     149.4615    145.30655    144.89625    146.06022    135.20104    130.48693    126.93366    122.48157    124.63678     121.9772    123.07202    117.34135    118.37222    116.92679    112.62497    117.80986    116.13273    116.69864    112.82471    114.55308    113.85655    116.84034    118.56032    122.11492    124.97639     123.2138    122.82321    117.89568    117.28225     116.2604    121.06276    120.41173    130.79438    131.98443    128.33968    126.49076    121.36263    118.91659    114.73469    115.75093    113.94668     114.2355    119.59056    125.00592    133.50332    146.55272     150.4541    158.42385    150.72656    144.01595    134.75023    124.08162    122.35208    117.70108    119.59321    119.18831    119.16574    119.97665    112.83572    114.17268    111.15833    110.31375    111.63086     112.3694    111.60977     112.7425    112.38764    109.09885    112.12422     114.7143    113.03861    115.20148    117.34874    116.38246    121.68862    126.16315    128.07912      136.286    137.85086    132.19299    128.59697    125.57124    120.90061    119.16905    121.15954     119.6165    123.39462     117.3918    119.79639    116.78139    113.51696    109.93453    106.78336     107.6736    109.75819     106.2179    103.05447    108.77272    106.08616    109.99921    106.84892    110.53979    110.70637     115.3735    115.74739    113.47445    112.92542    110.34202     114.4538    111.84399    108.27877    109.19423    104.89509    103.39637    103.00961    100.83388          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       29.00    281.13498    266.78875    257.51626    243.11972    237.45673    227.49668    218.91382    214.30038    218.66932    215.01237    205.04084    195.58814    191.63125    189.02478    187.73352    187.10953     188.0837    187.29486    176.56601    175.50821    177.79413    169.02797    172.54508     172.9389    180.00206    180.90688    200.28846    209.02584    194.63723    181.08118    165.09473    163.83158    161.27657    152.82946     152.1139    156.66389    154.72976    151.99403    152.99553    154.91277    148.66506    150.42781    145.13645     145.2369    146.72371    148.92668    145.82964    146.03174    148.64299    140.74621    139.43525    140.76586    141.42644    143.55167    144.51242    144.86907    159.82319    168.57174    176.95618    171.35882    159.84603    149.13676    140.01917    138.46773    140.91185    135.62102    140.02697    141.65209    144.95938    148.49789    156.24593    174.11126     175.4292     169.7757    160.01695    156.36121    153.62325    158.89033    156.11753     159.2605     159.6505    165.18672    171.34549    166.07085    145.92559    133.83888    130.04851    132.34495     131.8235    135.87752    141.26422    141.56231    143.80184    144.60454    141.40517    137.00445    136.60045     133.2277    129.39117    127.32623    127.17485    124.08412    124.96131    121.77488    118.78503    117.00232    115.41297    113.71063    114.06482    115.70206    116.86588    114.44099    113.81632     115.5225    116.16689    118.27515    122.78503    120.26127    127.04156    118.10007    118.62767    115.44939    116.45756    118.58807    120.77732    131.05506    127.35807    132.30949    126.91605    120.73983    117.88557    114.72905    115.12156    108.95459    117.35827    119.17496    125.78556    137.09541    146.71907    150.92896    151.06788    154.41581    142.37976    131.10441    125.85093    120.34205     120.1352    116.45511    121.85039    125.61855    122.29702    119.28992    116.32772    112.41517    108.75383    108.52796     111.2441    112.74162    109.24012    112.65924    111.32246    110.98311    113.33197    113.32918     115.5358    117.58568    115.41088    117.21566    122.03221    123.28281    126.67223    128.82675    126.70089    123.84607      118.149    117.12942    119.52403    121.93143    122.00454     121.2602     118.3899     118.2459    117.80061    112.57226    108.42999    105.37046    107.71735    105.27208    105.53816    107.98187    105.98363    106.99202    107.11635    107.87899    107.58068    109.75245    111.18519    110.54671    109.81386    113.67142    110.38714    110.56854    102.41044    103.25804          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       29.50    277.96031    261.45046    257.92372    248.35302    243.59137    228.06861     220.4307    209.69911    216.40098    212.72633     209.0005    198.22992    187.00558    185.51593    184.30905     187.4542    184.60512    184.47957    177.40432    173.69267    175.96256    170.69217    173.34658    174.41043    176.92626    183.53283    195.97366    200.81367    196.93605    180.29814     160.1861    162.59733    165.98202     150.6725    153.03422    154.20525    154.08744    151.10192    155.09987    153.47071    147.66098    145.11764     147.8762    144.92461    145.72346    148.80162    144.77132    145.81282    141.72811    142.55223     144.5173      147.476    142.84018    142.92106     139.1628    145.83136    160.85167    169.10225    176.30379    171.44329    156.96599    149.05901    143.35163     141.8204     140.7047    140.30703    139.44844    141.84026    141.51858    149.16624     161.3986    171.17325    175.52632    165.89375    156.06968    150.92482    156.51823    163.04337    161.93997    160.10914    161.82251     168.6864    173.99193    167.50656    148.82846    139.93296    130.38565    131.80488    134.06681    135.21282    141.17936     144.8564     144.1069    141.95228    135.33357    134.54418    129.70209    131.61532    130.57557    129.29889    127.23923     125.8205    123.61422    122.45181    118.89725    115.94079    110.49465    113.25353    118.18912    116.48873    115.89175    117.61092     113.8185    115.64594    116.06871     119.0113    121.61046    120.62628    121.28638    116.26974      119.251    115.26119    114.07693    118.81636    121.00164    128.38485    126.98709    126.76116    128.02809    122.65611    116.10407     116.2462    114.61206    112.51295    115.78727    120.38447    123.53734    134.45682    146.29291     155.5617     153.1982    147.82088    140.48023    130.29037    123.46138    116.37715    117.87507    118.42048    121.94208    123.64417    122.24614    120.98173    117.55295    116.22943      109.208    110.17382    112.58035    110.56185    113.09083    112.72381    114.42464    110.24875    112.52998    115.33509     112.0669    111.93989     112.8685    114.40737    117.95672    119.67097    121.66482    117.76429    116.56451    118.68723    115.04272    119.11808     119.8891    121.66646    119.48073    120.06457    117.81135    114.72904    115.91365    113.19448    111.37522    109.89224    106.07945    104.87366    103.25386    106.88859    106.45097    107.31769    106.17592    107.35042    105.49918    107.11048    108.42246    108.48366      103.839    102.96249          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       30.00    274.96387    258.79336    256.91257    252.08193    237.35489    228.54368    217.96946    211.04974    214.99217    211.51234    210.24359    197.08647    190.74101    187.49208    186.04767    189.04549    184.20463    184.03024    178.28704    172.35303    175.39279    172.16552     173.7329    173.90344    175.01524    188.98823    193.69757    199.17395     191.9136    178.93807    160.56144    160.01195    165.11049    150.84366    151.00727    153.89878    149.16064    152.61023    153.53388    153.80695    154.51389    147.41655    146.14753    143.98221      149.893    150.02456    148.05123    152.90979    141.43601    146.05318    140.43405    142.41177    144.98087    142.77631    147.52927    146.08833    154.09112    169.12577    175.35607    169.18177     159.3424    145.70121    145.35768    139.49428    139.58364    136.19866    139.69987    142.03897    145.06045    150.12976    159.02769    175.81472    174.27041    168.99094     160.5329    156.63852    159.89663    158.71245    160.06478     161.8401    164.10179     164.4797    168.84784    161.84429    153.04029    142.37155    137.52562    130.43958    130.94306    135.47662    147.29879    146.27914    146.98422    141.56452    139.92694    133.07615    129.62956    131.56888    137.78366     136.2034    134.16158    125.53647    121.27962    123.91319    121.15546    117.94289    116.88088    112.97478    118.51696    115.50729    117.07982    113.17932    114.14104     112.4018    117.14086    116.19894    123.18474    121.88316    119.87977    118.83558    115.65622    117.09618    115.63494    116.30917    123.69548    127.54499    129.76968    127.95212    126.72637    120.82853     121.5618    118.68451    112.32728    117.30488    112.15948    115.82828    123.53842    132.89975     142.6348     149.9275    154.35143    148.39083    139.38256     133.9794    123.15671    119.81334    116.66143    120.33551    118.21187    120.02396     119.1345    115.70435    118.45931    112.92671    113.14721    108.50589    110.33119    109.88385    111.94137    109.49224    114.79711    105.82794    114.25487    109.08678    111.97061     112.4508    113.71129    117.28554    114.73859    114.98492    118.82717    119.23488    115.63059     116.5018    114.76965     115.3995    119.60689    122.51023    121.29011    120.21187    118.36338    110.79809    111.69027    112.83618    113.78634    106.36235    107.04109    105.21687     101.3774    102.16497    100.88776    103.29426    102.28321     100.8087    101.84453     96.56393          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       30.50    274.30559    257.25626    255.38147    252.65045    229.21184    228.02892    215.99633    214.79962    212.73814    212.07761     209.4853     193.5926    195.28853    188.55826    188.95103    191.24759     186.2752    186.40651    178.93784    170.93048    175.49439    168.59522    170.55394    176.12793    182.48707    197.13015    198.81797    199.77083    189.63486    177.10885    168.11761    158.37739    160.19613    154.53085     151.6868    156.32832    151.09311     156.6333    152.12805    151.93837     152.1434    150.94813    147.11184    142.80298     149.1407    147.37503    146.32226    148.31731    142.00962     147.2233    144.48636    143.49617    144.70245    143.67721    149.89192    150.09184    157.09381    167.64058    178.77805    175.21538    160.51198    146.35748     141.4971    138.73332    136.81857    135.29714    143.25924     143.0577    145.54185    152.52327    160.43965    173.06756     176.7351    172.16736    160.49767    156.74682     157.8887    160.00096    157.13157    159.37055    158.04063    160.79648    165.72711    160.89109     150.9717    138.24902     132.8693    130.67473    137.32898    138.33037    146.22673    147.41843     148.3639    145.43106    140.12119    135.51522    127.94947    136.29603    140.42514      139.469    133.75016    128.02661    125.30004    124.79271    120.18579    115.84496    113.24959    116.63182    117.67524    115.22133     114.8018    115.45969    113.79445    112.34483    113.07793    117.03613    120.00967    123.70372    119.12769    120.33837    115.95524    115.88556    114.32827    114.87749    122.75855    128.55892    130.52734    130.45742    120.93288     121.1397    118.96558    117.92591      113.069    112.05392     114.4699    117.04372    123.68727    132.01268    143.55649    149.52757    151.88015    146.97418     140.5546    131.26193    123.45123    118.54044    117.63884    118.40131     118.1251    117.61552    118.05981      116.229    111.73102    109.91794    110.91662    106.22777    107.59812     111.9529     110.1374    110.61949     112.5921    109.19654    109.56754    110.87177    112.01727    115.14846    113.66311    115.35121    113.93357    115.33241    118.33475     114.3357    116.63469    116.58604    118.25823    119.59008    119.22023    124.35398    121.45902    115.64703    114.43885    110.16298    106.72203    109.53686     106.7002    107.99782    103.54917    106.14007    101.02042     98.76003     98.99554     97.52151          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       31.00    274.61002    256.15714    253.65144    248.61209    224.67342    226.76147    217.82752    217.39959    210.02047    211.82201    207.78326    195.77675    194.66099    189.36772    189.06459     189.5745    185.55034    185.15697    180.27492    175.23693    175.91438    171.04541    174.31932    170.97931    181.83145    191.99543    201.35417    200.74577    190.67876    176.68934    167.48576    158.89395    161.89701    154.76849    155.18669     161.8305    152.31563    156.21154    155.00492    153.64531    154.80493    147.07841    150.91695     144.2406    148.67394    145.87494    148.11398    146.64658    143.60418    142.82236    144.82029    140.59346    146.36593    147.00598    146.68861    152.20561    160.62016    171.41898    179.67133    169.62518    160.43191    148.46376    143.64061     139.8783    137.35991    139.22559    142.68373    139.11121     144.8608     149.9209    166.79613    173.24888     174.0604    168.82841      158.249    155.91037    159.62755    161.98303    160.78133      158.036    156.90891    160.11584    159.30149    153.75333    150.44541    139.67914     134.7312    136.06328    133.30783    138.06178     141.2152    148.62619    148.04363    143.04325    137.23287    137.62245     129.7543     135.7812    138.91067    138.92975    128.93959     131.9686     123.9914    124.66505    119.77455    117.59142    117.66182    118.05918    114.84249    115.75547    116.36411    117.79333    114.84477    114.29211    113.51207     120.4554    121.75957    119.61577    121.05756    120.75665     119.1298    112.43764    117.15527    119.83838    119.92684     125.1812    128.00882    127.19137    124.15656     122.1137    117.07846    113.41959    116.79316     111.9381    111.90681    114.50751    126.27486    131.73881    141.48362    152.07158    154.18053     150.6221    137.45915    129.79433    126.41709    118.14713    114.35609    121.44974    124.50046    117.33285    120.79545    117.25925    113.05992    110.93905    106.68557    108.66542    108.01844    110.64351    109.74639     109.9638    111.05584    110.87324     108.6531     110.0379    112.98979    113.30766    114.78699    115.64423    115.11709    113.01271    115.72706    116.54141    117.77958    114.03581     117.1576    119.65781    119.56047    119.63915    120.68001    116.01212    111.96922     112.6305    108.43531    103.91515    105.51216    102.24161    100.68026     97.94545          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       31.50     275.4362    255.43522    255.33051     246.0059    230.68712    225.85525    222.57168    216.36206    209.88989    206.41649    207.44462     198.6546    192.20926    192.44754    187.38647    188.37614    181.98616    181.44919    184.05765    181.02583    178.98638     169.2549    172.68266    170.10237    179.67936    195.03728    200.58585    197.17083    190.21754    179.72812    165.46353    165.55705    157.82873    163.07982    152.74156    152.06988    156.52722    156.29499    151.99699    149.28935     152.4521    148.72631    146.10875    147.18902    149.37528    145.84363    143.50533    147.23019    143.88971     146.7308    143.59186     142.7188    141.28627     143.3184    146.78284    151.91075    159.70909    169.96943    180.07541    169.01039    158.65872    148.97175    142.49909    140.30876    136.09399    139.40092    139.45466     140.5003    147.32732    150.90488     167.8705    177.35853    178.03208    170.52188    159.49744    156.03429    160.10986    157.38367    157.69314    155.08225    157.80007    159.01467     156.4797    155.64007    148.48885      138.656    133.95998    130.53233     134.0291     141.2785    143.72912    147.60446    151.70995    148.11368    141.17854    137.39358    135.53588     134.3495    134.14661    130.11831    128.21076    123.96766    122.33833    122.52449    120.97816    117.24302    118.73497    116.16341     115.2633    115.84939    113.21786    114.33175    114.07863    116.71297    113.90748    112.87425    120.11924    123.55028    122.82463    121.32926     116.5367    113.47401    113.99636    117.54112     119.2638    125.84628    129.90642    131.28873    127.35074     120.8605    119.11582    113.21012     111.5536    114.86246    113.56683    116.42178    121.71375    130.76862    140.34486    152.75334    152.58509     150.4573    138.48897    128.87159    124.29642    115.93606    118.76608    120.43095    121.14546    122.33778    121.38508    117.42401    116.68846    113.51962    110.25999    106.72127     106.0876    107.79482    107.38536    108.64344    109.89524    110.01284    111.54652     106.2518    106.90947    112.18616    113.55105    114.54127    114.64008    116.19308    111.25569    113.90472    111.47709    113.43225    113.37127    113.62983    115.84208     117.1672    117.57476    115.75762     112.8878    107.26265    102.06105    100.58529          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       32.00    275.66008    254.94185     256.5951     246.7731    234.01058    226.83272    221.92937     215.5893    211.18855    202.85479      207.804    199.27457    192.82186    192.94218    183.05246    187.47825    184.40452     181.2586    187.47589    176.41624    173.83136    169.88315    174.36557    175.30613    181.90093    190.10199    200.37722    192.42022    187.37651    176.79793    167.70338    167.02542     160.0144    164.74339    152.95418    150.54304    155.19471    152.02749    154.89108    150.77088    153.59989    148.23634    148.13949    144.38832    147.12508    150.85456    145.77651    143.70396    143.99428    138.85512    144.14726    141.79414    143.42434    143.47434    144.02489    152.09756    157.76195    171.28487    174.72822    170.43244    160.86743    148.16698    141.88539    140.96247      141.797    138.19467     136.0856    140.80798    144.40411    152.47921    166.30946    172.37277     171.9869    168.21868    160.66669    152.83745    161.05968    165.38294    161.65106    158.10115    155.47782    154.79741    154.84365    148.88562    148.03855    142.17388    134.39061    129.43589    136.06614    138.82125    144.15657    153.71488     156.2249    145.68581    138.57386    137.47379    137.87776    131.81806    130.50462    126.72066    126.29376     121.8409    124.17318    124.09845    116.47284    117.87129    116.03381     116.9124    117.53832    113.20421    113.71091    114.82491    114.57407    113.65443    116.83164    113.00929    122.26768    120.78669    121.51484    117.08178    115.90761     115.0534    112.83391    116.19157    121.84437    126.51559    129.59188    126.84034    126.21193    120.49625    118.72146    115.40104    113.07787    115.25082    112.72505    117.13856    122.63652    128.27972    141.65553    150.83341    153.97549    149.83903    143.32373    132.29552    122.08502    121.47983    119.32619    120.73801    120.25412    123.99261    124.78842    121.41309    115.56378     109.5406    108.91743    108.13258    106.85776    104.80467    108.82889    108.60872    109.04675    110.52317    111.61038    109.65768    108.13057    109.79677    114.39586    113.59869    115.53161    112.04685     116.6738    110.75025     114.2401    111.37855    114.18027    113.69937    113.68268    109.48908    110.98969    109.68979    100.84739          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       32.50    275.28712    255.29928    256.77432    250.80102    234.99036    228.06667     219.3577    214.81854     213.0626    207.15493    206.45637    202.93095    193.69391    192.57257    185.10228    190.43088    188.75176    183.27856    184.46258     178.1525    173.20294    172.70579    172.24459    177.42249    175.76969    189.44173    203.32592    192.96278    183.52192    176.94984     166.1209    159.15306    161.29521    158.78236     156.5796     151.2536    149.91672    150.26417    153.37049    153.78865    149.05921    150.89004    149.18616    150.66385    149.27396    148.31055    146.17866    146.54461    148.09918     144.7449    146.49238    141.80803    142.52078     143.8009    141.32423    149.86249    158.09265    172.07544    172.32363     170.0356    161.41499     144.5427    138.20875    138.10931    134.04087    139.79983    136.57388    140.03828    143.52085    153.95353    165.50424    174.32566    179.24271    171.20286    163.69531    158.13243    165.24098    168.54255    173.72923     166.9319    158.94329    152.67783    160.04842    154.22864    150.10037    142.95417    138.20689    132.00637    132.30339    135.43635    146.32555    151.52457    151.24418    144.08961    139.01632    135.75813    136.48647    136.00089    129.41396    126.73724    123.22167    122.21713    121.10283    121.60856    115.85542    117.78101    113.12779    117.11444    115.39315    118.50517    116.32198    116.83174    113.84667    113.65637    115.49628    115.18065    118.79766    120.82466    121.93803    120.12452    119.27271    115.17658    114.79905    114.36761    120.49604    125.21858    127.13663    129.71172    123.60992    121.34628    114.07384    114.17435     111.0453      116.351    115.77121    116.27111    121.94206    132.51659    141.98696    153.28371    156.96254    152.44324    141.87439     135.9099    124.49509    120.54834    118.93357    117.97238    119.41993    120.11357    119.08335     116.9403    115.36703    114.35863    110.44914    107.24112     108.6531    107.22402    107.40597    107.27304    108.67935    108.61723    109.31885    106.17546    108.35332    109.28674    110.10833    110.89117    112.58631    110.72163    111.35101    116.72663    106.27731    109.87827    106.87459    106.51581    105.88159       98.249          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       33.00    275.10807    257.52975    255.82006    248.08796     234.9403    227.48629    219.86798    210.93108    213.35391    214.09682    204.92462    207.78483    193.67038     193.5558    189.51305    189.63228    182.81308    187.07347    178.28407     181.6675    173.60285     171.3989    171.70547    176.11059     175.4968     189.6145    201.07611    192.50783    185.31508     174.2495    169.57346    163.36349    156.56354    160.25329    155.42961    154.45181    149.38521    153.35959    154.21123    157.59463    149.36043    150.93965    145.92682    151.12054    148.20088    147.13348    144.44092     144.0938     152.0834    150.37572    142.09188    144.88419    146.89466    143.93493    142.23667    147.75446    162.27177    170.32153    172.00732    170.32082    160.71481    148.98838    144.97696    138.12066    137.64586      136.591    141.18421    138.50571    142.95417    151.17834    165.46133    180.18002    180.72032    167.83012    157.83938    158.96071    164.53892    178.85542    181.73233     171.2623    160.69029     155.9018    159.80686    156.39026    151.11693    143.50404    135.99328    132.27586    134.35547    140.13076    147.36183    152.12274    151.27637    143.57053    138.16705    133.91127    130.03645    133.95045    124.51489    126.70131    119.19329    126.09194    121.35744    121.95769    119.02655    116.47719    114.50541    115.15482    117.99577    117.75534    116.86091    114.10679    115.84482    112.62008    112.32126    118.61804    117.90654    120.30038    124.80228    120.91998    117.01461    113.96689    116.72241     115.4368    117.30095    127.75502    129.18826    126.92812    123.93944     119.4645    115.38032    109.68937    111.65773    112.04615    112.75848    116.35936    121.23583    131.58507    139.74628    151.42791    156.04815    157.16221    147.64782    133.11226    127.17147    119.91163    120.56884    114.17349    114.82537    115.57093    114.90558    112.83972    107.23914    109.28584    109.44335    109.59447    106.97163    105.82273    107.19072    104.95357    106.78754    107.92137    106.20883    106.44532    107.57244     108.7708    108.05475    110.22037    109.41754    110.95106    108.64177    108.50074    102.90315     99.48608          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       33.50    277.39125    261.36523    255.40378     247.6528    237.04824    224.68631     221.0327    205.69322    212.80832    213.42785    204.36654    211.64005    198.02357    193.08777    190.51916    188.10705    180.89773    187.49824    178.18804    181.13782     171.4394    173.78158    170.29163    172.80766    179.51474     186.9427     199.5415    192.96005    187.50271    172.89397    174.13012    161.93424    155.60675    163.12071    156.05977     153.8103    149.59817    154.35283    154.03083    157.81411    152.11779    154.88224    155.03783    144.85419    145.43316    147.96345    143.77822    150.20662    144.49475    148.23584    145.35574     146.3307    144.45857    143.50137    146.58779    152.98894    164.41884    166.22071    171.87048    163.93937    158.30069    146.47046    141.46461    139.84989    139.81203    142.22523    135.88992    140.77276      145.577    149.95548    167.86092    178.76331     177.2784    170.30793    161.76365     158.9092    168.41121    177.76586    183.38264    176.88375    168.93117    163.38588    163.51925    159.27996    149.71692    146.12343    136.03749    136.19517     135.2356    140.25996    146.39957    148.55527    146.60186    145.56618      140.045    131.97281    128.36465    126.96581    123.54161    119.25695    123.55859    125.74291    122.02717    121.19197    114.07361    118.09816     116.9734    116.95395    115.17019    119.80412    114.17041    116.92227    112.85132    114.57142    113.57746    115.17665    119.21469     120.2751     121.9547    117.38971    117.50124     115.3213    110.52204      116.892    118.95853    123.30331    129.10493    125.46246     125.2927    121.72513    117.83306    112.54564    109.38213    112.41623    112.47512    115.27459    121.84896    130.66697    140.34666    146.58345    156.76775     149.6625     143.5637    137.67768    127.45541    119.92366      113.822    113.67104    115.43549    109.17604    113.20737    112.93679     109.6175    106.31129    108.00116    106.14149    104.62107    108.76674    105.86818    105.20223    107.13103    107.52601    108.55395    104.59239    108.01291    106.96262    107.20931    109.88151    110.00359    106.59016    101.99246          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       34.00    280.15385    264.52434    256.95983     250.4592    240.42773    225.68676    222.23139    208.59177    213.28585    212.61461     201.8063    210.92298    200.08544    191.12285    189.92782    187.24951    184.35836    183.09607    179.93801    178.20092     170.1252    178.50192    165.83291    173.63973    178.16555    194.16172    195.70282     201.8252    190.92119    171.36953    167.08366    165.31514    156.01121    160.19443    157.55102    156.53165    158.77717     157.2242    149.64456    155.51112    151.45053    154.85563    151.28606    147.10801    150.47279    150.94328    149.82173    150.16423    146.10045    147.15776    148.19651    141.24597     146.5038    148.52951    147.54811    148.84231    160.10782     176.6379    176.25175    168.98244    155.54475     146.1756    143.66166    142.53801    139.13871    140.97853    138.89265    136.64258    143.83161    153.35668     164.8169    180.51369    179.89407    169.03639    161.09865     159.3543    166.84177     172.6419    178.01646    171.53412     171.2525    165.71868    164.93411    161.17552    153.52427    147.44837     135.0375    138.41452    135.91072     137.2296    143.78812    149.85178    147.84369    148.43911    138.19402     137.1928    131.66414    122.35043     122.6226    121.39529    120.67446    121.96592    124.40458    122.71101    119.25735    114.71917    115.10692    118.56876    118.58264    119.20709    117.93537    116.81856    114.95997    113.46318    115.33643    118.60345    118.36889    121.10508    120.92166    121.69835    116.68714    114.81475    116.62916    116.14475    118.60169    120.91859    130.66414    128.84515    122.46871    119.05904    117.50489     113.2596    113.53475    110.91709    115.00417    114.87565    121.12302    128.65766    140.62455    145.09276    151.06664    149.35082    141.61401    132.02949    126.39671    116.41566    116.62614    112.07503    110.99543    110.00847    112.63989    109.89565    108.42067    106.35786     103.2134    103.07933    105.43987    105.55423    103.07255    103.83232    103.97891    102.75236    106.69552    107.63956    104.09831     102.8733    103.32566     97.95513          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       34.50    283.40073    265.70534    258.89341     252.1555    241.10278    228.97249    222.08394    215.15296    215.20878     212.0783    203.70026    206.47135    199.86649    189.68101    188.35144    182.36682    187.86574    184.67069     181.9655    181.17619    170.94185    177.85349    171.04363    175.68145    178.52774    193.11083    196.96472    204.52694    191.17408    173.76244    162.23878    163.49836    155.76658    157.31538    157.51138    153.37144    154.98605    155.32195    154.34622    152.94898    150.94467    151.65699    149.20248    148.85551    149.00658     145.6251    145.67653    145.07255    147.42206    145.25001    145.92611      138.079      143.754    144.56846    142.65407    151.66464    159.46314    169.06024    177.06477    169.04859    157.86678    146.98271     144.7948     141.5523    138.25745    142.20467    141.27509    137.12425    142.14788    150.99304    167.49054    173.11752    182.94468    179.78603    164.08646    158.24234    164.14515     171.4746    167.71834    168.27893    165.06633    172.05172    167.63069    165.36468    160.23716    148.58309     139.8882    134.14623    137.44941    138.73997    141.99638    151.87173    147.30956    149.03702    139.49087    138.87418    126.11885    125.23161    124.98857    126.21402    122.46975    125.19853    124.86673    125.93992    120.58524    117.81799    114.10852    115.88257    115.27255    115.85865    118.71999    119.62722    113.68649    114.42981    114.81196    117.20339    119.16367       121.95     120.1773    118.59542    114.78029    115.47939    112.15809    112.56599    118.79249    125.99529    126.84833    126.68838    123.78003    117.28316    116.50156     115.5366    114.27391    111.28276    117.10421    112.80645    120.35274    126.83271    139.72788    147.38741    149.88385    148.14306    141.72082    131.50222     124.1465    114.35414    115.50433    109.57208    108.57474    108.94951    109.70728    108.52807    106.88828    104.58956     103.7218    103.51527    105.83251    104.86467    104.89991    104.07017    104.95028    102.89294    102.98744     99.68555     93.75734     97.22937          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       35.00    284.08402     264.8673    260.05848    250.75731    239.33852    232.49893    221.67815    216.68278    213.22124    210.63485    207.54727    200.72687    199.36888    191.12285    186.88008    186.42991    190.04688    186.70109    181.09327    182.58576    169.77793    173.88226    175.71235    177.68313    175.95535    190.90225    195.43586    200.12721    190.33032    175.48932    161.69159    165.83662    158.16672     154.9373    159.52874    153.36445    157.04309    154.12528    147.67745    152.22796    146.30956    151.96976    154.46579    146.12614    150.26066    150.02448    149.70421    154.39929    145.38985    150.49608    146.26875    144.04305    142.56787    147.04279    146.07095    150.81304    160.56684    168.59243    178.38375    165.34421    157.14411     147.2801    140.49983    141.59093    140.25499    139.20887    141.01227    139.11858    143.87938     151.2537    168.26643    175.90778    175.25543    178.61198    167.24939    157.16924    164.24775      169.782    167.31234    162.35195    161.63267    166.63461    171.43761     172.9623    165.95499    151.62413    143.92871    138.53846    137.37751    140.59586    143.54975    152.44404    153.16516    145.82062    142.15102    137.41005    128.32378    123.83035    123.86289    123.20148    124.97563    124.96285     121.9377    126.46175    120.82892    114.19704    117.79783     117.3856    116.13975    119.91151    123.59235    116.37921    113.92814    116.83156    115.39198    116.22962    118.44709    121.72013    120.82163    118.27618    116.63364    116.46159    110.16063    112.23984    117.35767    123.61848    127.14677    125.52456    124.79524    119.06718    116.16855    116.01952    110.50475    114.09579     112.1058    115.03676    118.63894    130.09933     134.8992    144.88702     151.4641    144.97414    140.38223    131.46653    123.46263     116.3177    112.31845    111.94184    108.47981    108.10291    108.26748    106.88914    105.96155    106.56111    103.75861    103.06071    101.03878    101.84577    102.41995    103.00181    103.09472     99.95651     92.59681          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       35.50    283.50338    265.36932     260.2451     246.1896    238.49286    228.74315    221.71729    219.04662     211.9879    212.69029    208.95898    201.59687    197.97762    190.17665     186.6077    191.06289    188.29427    185.10829    180.36927    181.36319    172.96956    169.49913    174.98367    175.24401    174.02388    191.85788    196.50598    206.85001    182.40934    175.99466    162.07137    161.72292    162.42429     162.7288    159.29992    155.55373    154.72485    152.87224     151.7452    152.81702    150.03942     153.0674    154.48649    150.65584    141.63653    149.36197    147.31896    152.86433    148.29093    148.90347    146.16028    143.76899    144.82893    149.91045    141.21639     150.7309    157.32099    169.50562     178.8853    174.92382    158.06743     146.9272    141.14532    139.14089    140.53857    140.82356    137.15425    140.53568    145.12043    150.54277    166.94315    180.46063    179.02486    173.01081    165.54689    163.00867    162.72735    170.58269    170.44976    162.62006     161.4406    167.71195    173.17707    173.09062    176.50559    159.96819    146.35062    135.32033    136.84612    142.86048    150.48074    146.75974     149.6503    148.28289    139.78278    132.05569    126.48943    127.00809    126.56539    124.62525    128.20656    125.64595    123.78248    126.43429    118.17089    114.34534    114.67855    118.49037     118.9204    119.43308    120.23058    121.26499    118.26035    115.86281    113.97394    113.64352    114.18408    116.22015    119.42801    119.27693    113.69685    116.04914    111.46254     116.0044    118.13049    122.17695    129.21585    125.78362    120.86019    118.92738    115.40994    113.76947    111.57759    111.93961    111.42813    112.14274    120.11568    127.10986    135.29026    143.74018     147.8953    149.24061    136.10573     134.3853    121.74846    118.53348     117.2257    108.74067    109.33532    104.03158    106.57666    105.89776    102.34611    101.85701    103.37547    100.31122     95.53257      99.2683    100.03487     96.86566          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       36.00    281.59493    267.22398    257.75714    247.16911    240.07194    228.39623    220.16761    223.11171    213.43971    213.44887    205.99048    205.01111    198.60116     190.5553    189.10956    190.85823    184.02785    183.40206    185.20461    181.67417    179.64193    170.81424    178.97969    170.31503     179.9959    192.56102    194.43705    202.44686    194.85339    171.11437    163.87292    157.91368       160.43    162.83419    160.68269    154.88477     156.7899    153.71555    152.85157    153.04158    152.63928    153.22426    151.60807    151.91386    152.76342    148.73668    148.00227    150.80233    142.67062    149.00636    149.45861    145.67948    149.41092    137.99864    150.71823    151.95194    160.02323    170.43691    174.14481    166.72351    156.22203    149.15293    142.98981    141.64048    140.21593    144.46752    137.13393    139.88788     144.9817    150.61369    166.62175    176.53841    179.30633    177.36717    171.24884    163.63036     165.3336    170.10285    171.32488    166.25325    174.31222    172.69645    173.40119    184.20172    177.27118    165.55504    152.88442     139.4119    134.89314    140.83615    149.91835    152.47164     154.9924    147.06336    139.94614    136.66126    125.69605    125.02955    124.34505    127.47718    129.56832    130.70871    126.02981     123.8892    121.26447    116.34636    118.25093    113.47137    120.94455    118.02291     120.0654    117.24692    114.73612    112.70124    114.81366    116.98608    119.67441    119.54066    120.95611    119.99822    120.54316    111.88303    116.32544     114.8476    120.15382    123.55792    125.87174    124.60914      124.205    122.69938    119.03702    112.96802     112.3521    110.95527    110.97804    110.63413    117.32779    126.04783    134.24153    144.16939    147.73056     146.3597    142.38019    132.23902    125.47802    116.75153    115.48106    112.51264    111.40828    106.47385    102.93892    102.45941    106.57535    105.07562    100.27783     99.86582     95.88608     92.01344          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       36.50    279.18293    269.40748    254.90967    249.97532    242.86901    230.47734    219.01564    223.61038    211.89287    212.50658    201.65917    208.18814    202.19008    195.35137    191.36296    191.30555    180.93422    180.24375    180.34252      182.018    184.18333    170.14607    181.84271    175.52387    179.36838    185.68527    197.50473    198.61459    192.77435    174.52332    160.48144    162.70385    158.52537    162.73467     159.9772    159.16834    162.67668    153.34058    151.80191    152.56772     153.6405    153.08749    156.85534    153.76275      152.915    146.55723    147.34741    149.27421    144.75039    148.31172      146.543    147.23873    146.51551    147.68559    143.33884    147.42268    156.97638    167.57482    177.10342    173.89705    159.51097    151.67185    145.16956    143.52717    136.98237    143.60561    136.06505    139.58573    144.38727     149.0479    167.61357    175.65649    183.27889    176.72222    167.27635    165.15811    165.99417    170.60187    174.57144    170.09466    173.70635    184.98466    183.18513    187.22159    177.41987    171.42767    155.36675    144.97363    140.09191    142.66794    151.06255    158.41694    155.79326    153.93222    143.10081    134.04055    132.31623    126.99397    123.51988    129.01588    130.25488    126.18834    128.56482    122.21858     121.8277    117.66721    116.51898    116.42576    119.51941    119.48741    118.69008    123.09122    113.31479     113.6079    114.33713    110.87415    118.83419    119.46883    120.27356    117.04843    115.63619    114.00758    112.62231    115.48962    112.16292    125.71251    127.47272    126.45433    122.40303     116.8602    113.39068      111.901    111.99086    111.25073    110.29331    114.53499    117.74741    122.21422    136.93187    142.34271    146.58326    148.89943    143.46878    135.85816    122.44842    119.02616    114.37973    107.91126    108.68254    105.48108    106.29582    102.72167     98.01146      97.6816     92.32073          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       37.00     277.7947    270.02927    254.17621    254.62339    244.44155    234.30386    221.11743    223.27438    207.55545    210.96391    204.52966    210.81029    203.14773    200.80871     190.4644    191.45868     182.5897    175.25431    177.30197    182.68754      183.444    172.26606    177.76173    176.80145    180.18938    190.88436    193.45301     199.9561    187.39399    174.41114    165.24096    159.34175    158.18956    160.13795    161.34109    161.13377    163.85003    160.91957    152.11181    153.03315    150.81462    154.44393    148.71119     152.1456    151.85829    148.54778    147.41139    145.52367    146.64458    147.76032    145.83516    146.97308    147.40674    144.92872    144.97827    149.84193    160.22877    172.14768    174.19977    167.88862     162.5885    147.71796       139.91    141.51198    141.46975    138.99451    142.85265    141.04041    144.06459    152.10212    165.52188    173.59817    180.31453    178.40731    168.36869    165.73235    168.88507    175.62102    179.49481    176.22879     178.9084    189.42511    193.26726     189.1923    178.27957    162.79245    153.54721    141.50888    137.92458    141.23936    147.35594    156.59708     162.6212    156.54542     145.9073     139.7104    133.62881    136.44808    129.61753    129.01856    125.66866    127.12364    123.99173    122.56515    120.47926    120.29163    117.21758     113.8052    116.64468    119.48066    120.71457    118.04656    119.86843    118.06223    120.84824     110.1629    118.44007    119.97902    123.08844    116.68186    116.21484    113.78156    110.90943    113.48249    118.78148    122.96135    124.55598      126.939    121.51249    122.11141    113.92487    112.29335    108.68704    114.23469      108.184    111.29389    116.05238     123.5856    134.09084    142.60923    147.54469    149.58604    144.97872    136.79644    127.92829    118.97068    117.23884    109.57115     110.3571    107.96312    103.74271     99.12035     92.99782          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       37.50    276.05651    267.55239    255.03205    250.08838    244.83894    236.15228    222.20486    221.21749    212.16414    211.81264    210.01694    207.92843    202.43363    199.75702    192.68915     189.3206    188.95685    178.83875    180.33005    184.63706    182.88657    178.52539    176.68991    174.04276    178.89645    192.65404    192.66006    199.93865    189.89901    178.26619    165.32777    161.78797    164.17627    158.86981    161.98119    158.63619    159.65261    157.53021    147.62309    147.57711    153.94684    153.86697    153.05796    150.41452    154.63866    150.10218    148.02246    151.52869    147.61038    147.34724     145.1149    144.09658    149.95248    149.08195    145.84229    149.51342    156.06576    173.12404    175.23694    174.00184     160.3527    148.84913    142.11657     142.9073    134.77769      140.882    143.57874    142.60058    147.67477    155.01803     161.9648    176.13088    179.84687    175.33075    166.63873    164.00322    167.52836    175.67596    178.38975    183.87377    187.57958     191.1935    197.88466    193.62052    175.57075    160.62614    150.46966    142.76891    140.56403    143.43033    150.73249    157.97933    164.36131    158.10283    156.64583    147.07342    148.10301    149.26919    137.89341    130.22664    128.53452    123.69453    123.57581    124.27813     118.8873    119.22273    119.36944    119.30957    116.80077    119.60256    122.60406     120.7937    117.39747    116.74846    117.23411    117.56522    119.71569    122.55397    121.88129    119.45309    114.75628    111.23463    112.87426    111.87884     119.2163    119.02554     125.1628     125.0955    119.86908    115.96545    115.03111    110.21604    112.16251    109.87422    108.97954    108.95863     115.1008    123.62608    130.96654    142.95076    151.80997    152.54959    146.82813     141.8602    129.94846    120.58643    113.19676    110.94792    106.93585    102.19068     93.40005          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       38.00     278.5605    263.91573    253.80215    246.86597     244.6897    238.97723    223.23621    221.95898    216.17915    213.74375    209.08333    205.63572    203.36103    195.97375    196.38622    190.23983    189.29029     185.0154    180.62405    180.65167    178.27502    186.03694    175.08797    177.40152     177.9186    190.51466    192.63044    199.32672     186.6222    183.45318    167.06448    165.62649    164.02743     159.5664    154.55103    158.24943    153.85376    157.25877    153.02876     154.2255    156.02533    146.72139    149.41003    152.03868    153.79025    151.07817    144.12558    147.53069    146.29756    145.28324    144.35142    144.83455    146.41541    141.41481    145.00541    151.21265    156.38671    168.16412    171.58616    170.89347    162.46722    148.41935    144.46997    143.01446    135.63469    141.28987    141.58753    143.56215    151.19286    154.02065    161.87425    176.78911    181.23332    176.03045    170.03638    164.08287    172.53331    182.24879    187.32398    194.42746    195.39942    196.64214    197.02827    197.88008    182.52907    161.03678    141.18571    141.89016    140.74363    140.37633    147.81215    157.33817    163.35177    167.06112    157.25483    158.18869    161.79027    158.54011    150.24809    129.10161    127.03631    125.48253    123.96656    120.93353    120.00193    119.39381    119.28813    118.38583    116.95583     118.0339    119.02833    120.01014    119.74563    117.34401    118.22572    116.72145    118.95468     116.7673    116.19382    117.45928    113.86221    111.28773    112.93912    111.00626    111.26924    123.22558    126.68661    127.01208    122.43369    118.18783    116.18946      112.218    109.82184    108.01699    110.75965    108.54141    114.76082    123.44851    133.56182    139.28668    149.16949    153.94978    146.20343    138.93983    128.76344    122.11637    111.76656     104.2224     97.35154          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       38.50    283.77104    262.08709    252.78285     245.6417    243.34592    242.84879    223.48784     222.2378    217.99995     212.2946    209.40249    207.43039    203.66973    195.45914    196.04971     198.0753    185.57825    182.66431    179.72172    175.63241    174.73614      182.495    170.23793    179.50337    178.73492    188.09787    198.83092    204.91612    186.47821    179.88975    170.19283     160.5334    161.14793    160.44717    156.85803    159.68461    156.75259    159.88178    153.95984    155.12063    153.47278    153.34687    149.19358    150.04797    153.80628    150.11606    150.11932    145.39173    148.69008    151.30752    148.12413    148.45734    148.81879    149.47513    150.50708    148.88233    159.95899     162.9335    172.44368    164.96153    158.87796    152.24436    144.88065    136.92485    139.72506    143.12211    138.12682    140.35226     148.2097    152.81998    162.75718    171.34746    179.80464    176.88979    167.37346    165.46609    169.93529    181.99003    201.07693    205.56313    209.45276    202.52007    193.93821    192.06198    177.35519    160.27266    149.56439    134.89969    135.93588    138.00871    154.31459    154.69175     165.6948    164.12948    160.65977    154.07656     159.5651    161.52976    148.76499    137.49296    128.42812     122.9401     120.9689    122.03011    120.31718    120.50542    117.51328    120.03823    119.54437    119.64216    119.56758    116.88194    117.71134    116.06126    115.14044    115.26975    116.88285    117.51535    118.42448    117.71538     116.1275    115.65448    112.27739    113.68305     114.2526    116.93482    122.57363    126.35651    123.95429    117.25388    112.71463      111.792    107.58836    105.03327    107.75647    113.57094    111.87483    120.78299    134.42627    142.31744    144.37435     148.6672    143.13109    133.16649    124.80469    111.22626     98.95941          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       39.00    287.45725    263.48662    252.38179    248.44535    241.50975    242.95444    227.50256    216.35031    217.89765    210.78058    211.93058    209.01985    202.47734    197.83298    194.70059    193.56869    186.46111     183.0137    180.96845    179.92789     183.2922     174.3817     169.0173    179.10202    174.51578    187.73592    195.33611    198.56444    194.26841     179.8346    172.30312    161.63658    157.39486     165.6016     160.8141    162.73085    159.38848    160.46601    153.80356    154.01421    154.71382    155.94647    153.41472     150.8111     150.6754    156.05503    150.28073    147.15018    151.24328    148.03166    145.30511    148.11756    145.25238    149.11832    149.88434    152.43298    160.12745    170.35265    169.44158    165.67519    159.06671    150.52308    141.49505    143.07588    137.48241    140.20017    140.71277    141.54855    143.87089    154.13785    165.22511    175.71493    176.75788    170.71879    169.62413    165.49915    166.15434    177.47004    198.08736    215.56913    212.43538    201.29247    191.98802    189.04405    177.93754    160.60381    145.37216     136.4912     132.8613    140.87588    150.03794     161.4427     167.2541    171.27967    170.45881    159.46392     158.3537     154.0817    145.51407     135.1088    130.80525    126.97431    122.06558    124.95704    121.58088    119.69688    117.78683    119.15629    117.36859    120.94843    121.10491    119.56844    116.41733    116.80924    115.70085    118.04718    116.15595    119.97451    116.51146    115.10429     115.7087    112.24407    112.50432    111.45559    113.07394    117.13096    121.68715    123.54195    122.20275    118.03989    114.69928     114.3017     110.1128    105.60814    106.07824    112.27595    114.12456    117.89189    129.73348    137.37002    141.95425    139.69034    134.17139    119.59584    102.23902          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       39.50    288.53954    269.24187    253.54692    250.67254    240.05087    241.12814    229.18299    208.72186    216.78368    211.97784    206.78881     206.5205    201.66208    200.73245    196.23306    187.91086    189.16624    184.28572     183.5231    181.61916    178.51936    175.74516    174.68035    177.99889    182.07481    192.04881    195.66686    192.39927    191.38957    181.92117    172.09286    162.66933    158.24695    162.04682    164.11559    162.89045    161.93939    158.02821    155.51132    156.03556     150.2357    149.33192    153.36817    153.54407    152.24185     153.7663    149.22109    147.73092    149.27097    147.39312    144.42832    149.42603    146.82826    146.49722    151.58787    151.01197    155.54553    168.69525    172.49069    165.71034    156.79577    148.28568    147.18774    141.55219    135.69231    140.12439     137.4998    138.50176    146.33146    152.59686    169.39701    177.84249    179.01438    170.68224    168.76412    162.75599     164.5866    175.20371    189.01127    208.48451    209.74413    204.17749    191.00135    186.74359    176.07856    163.20209    148.75627    142.34012     138.4692    138.58184    149.48625    163.20333    175.94818    184.08991    181.51682    177.07451     156.9963     145.5663     136.3297    130.64481    124.41899    125.24142    129.84478    122.91663    121.68486    119.51411    117.90701      118.853    117.09868    118.23122    122.24173    117.78215    114.44985    117.01361    116.74198    118.01911     118.9957    122.43648    118.21616     116.7397    114.41688    112.11486    111.53505    112.26081    110.54276    117.54156    122.05459    122.09821     120.7929    116.19816    113.71752    110.91352    110.39857     106.4949    103.39351    105.14832    110.40639    113.74067    124.41379    132.47199    134.00736    127.47683    107.46259          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       40.00    287.88119     270.5593    251.66171    251.66693    238.90704    240.70814    229.83812    209.58743    213.76775    214.58788    206.93853     203.0258    201.76843    204.99628    195.76452    186.58584    190.64268    180.54672    181.69949    181.98552    172.87095    177.64783    179.13324    173.45123     180.5006    189.17369    198.39808    192.62752    188.39803    181.04723    170.33563    163.14561    164.14369    159.28669    162.46353    162.52879    161.47172      155.626    158.25746    158.70953    150.38174    152.06816    152.89341     151.9229    150.07347    150.38493    146.00159    150.57495    147.00082    149.98823    146.81261    146.96376    145.92019    148.48688    151.75813    152.44508    158.65669    164.38184    168.43663    165.41089    155.51371    151.06358    146.37953    143.74825    140.91173    140.93353    139.09404    142.46224    143.14882     153.1703    165.48205    175.88831    176.29255    172.91657    164.91826    161.79321    162.60827    171.47594    183.31339    196.90506    196.19175    189.59053    186.52726    183.80914    173.54485    158.14505    145.39646    140.21188    133.02534    140.94178    148.70091    165.98109    181.86524    197.89561     198.8083    188.48451    165.60097    142.11481    134.49547    129.10538    126.51792    129.46207    126.79528     124.1478    121.65455    120.42397    115.17928    120.08057    119.15134    120.83489    115.46408    119.83592    116.43425    115.89047    113.70635    116.98909    116.96816    118.73779     119.3307       116.99    117.45778    108.70246    112.96642    111.61354    115.69223    118.53367    119.60704    123.20377    122.11958    116.46525    117.46852    109.62352    105.92721    105.14122    105.26208    104.71812    107.18699    118.05516    121.40068     121.6863     113.9388          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       40.50    290.01898     268.1453    251.45451    249.95086    239.83849    239.60578    229.39858    216.62439    212.53062    215.51738    209.83365    201.04071    200.51109    203.67223    195.39012     191.5533    189.70107    181.65456     181.4189    181.53272    177.27165    179.89882      180.564    172.88856    181.47585    183.40954    190.90249    193.44087    187.40051    179.10007    170.93608    171.70479    166.29445    158.40426    163.11662    160.58713    159.70671    156.00154    155.27384    154.47242    150.84562    149.54781    147.91198    149.15969    149.97839    148.31091    149.73028    146.19288     150.8278     147.9528    148.03513    146.98096    143.98985    139.96032     150.6024    149.63942    161.94269    165.71382    176.02975    164.25385    155.20366    156.35517    145.83741    145.66278    137.58098    135.74259    137.03096    141.08684    144.07625    155.57866    163.26215    180.33933    179.30543    172.78591    167.93162    160.13148    163.92642    171.33408     181.2478     182.5633    186.66086    182.69254     178.7209    182.68582    180.47123    163.37624    150.70513    143.43723    139.78123    144.06559    145.17845    159.98843    174.72666    190.32928    196.51246    184.83439    168.23879     148.6006     132.3416    127.21683    128.09292    125.98947    126.56416    124.63313    123.17659    117.60132    112.61624    118.84198    118.03722    120.87852    120.67807    118.67352     119.5919    118.06489    115.14283    114.85733    116.29461    118.64436    120.54246    118.32051    115.93505    110.39851     111.7904    108.63769    114.85395    115.42403    120.14727    123.40319    120.08866    118.44331    110.30397     107.6629    106.61513     104.7777    102.57602    104.60267    104.06906    108.98754    107.21741          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       41.00    292.43512    262.05322     252.0885    248.30338     245.5137    239.16792    232.74328    225.00169    213.61008    210.51189     208.5867    198.93859    201.44729    203.30242    197.24558    193.23939    187.94569    179.75087    182.09208    177.25947    182.42023    183.82761    178.98126    177.48379    178.16445    186.36168    189.04079    195.40079    185.04435    184.31638    172.66907    168.82173    162.17866    154.70217    159.25925    162.40821    161.10751    156.11059    155.49829    156.66474     161.7977    150.55672    151.33074    149.72402    150.84561    147.76866    150.85329    152.41126    152.67366    144.00624    146.86779    143.61009    142.58489    146.86937    147.74232    154.31163    162.93365    162.89547    172.44116    165.17997    156.80716    154.04877    142.72766    138.69697    144.49971    140.32426    139.00074    144.95596    144.57709    152.76628    168.27145    174.55112    183.93264    175.50309    162.58383    160.02082    166.56189    174.43329    181.68864    183.33295    188.36903     184.1559    181.34496    184.33653    184.24535    169.53435    150.94762    138.74127    136.68922    141.64279    148.48582    154.25305    161.06935      172.296    178.38776    170.53568    159.47108     146.9863    132.84643    126.18878    123.98785    124.18984    124.90105    119.64146    122.73447    120.72221    115.79203    116.07277    117.68094    118.44472    116.85029    120.25242     120.8976    116.59575    115.78682    115.69447    114.37989    115.72686    118.83226    119.59579    115.57744    111.98447    108.70509    111.13503      112.854    118.09747    123.02075    119.94049    120.85683    116.00981    109.04896    108.57942    106.40682    103.92559    101.55156    101.24748     99.11601          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       41.50    295.13038    262.42475    252.63378    244.78226    250.62806    237.79166    240.20168    227.68711    212.06502     208.9861    207.48961    202.60859    203.78763    206.07798    197.56952    190.35832    189.47256    179.53824    184.74498    178.93997    182.64371    184.44427    177.18644    180.86524    178.82668    186.67734     189.5031    190.79107    187.44323    181.26951    174.20292    161.23851     160.2096    159.82332    162.16504    161.12449    158.70428    154.89344    157.58743     156.5043    159.32999    151.27834    151.51188    153.42019    154.53585    149.37182    148.56973     149.5544    151.17553    147.17731    140.93122    142.22132    145.19301    148.94501    147.77165    155.21307    161.50805    169.66391    169.01296    168.25816    159.04998    149.62622    145.70528    143.65311     143.7096    142.83172    140.07522    139.07124    145.07624    153.16238    164.93364    174.91682    182.23936    175.28977     167.3239     164.5826    167.58027    173.09066    183.53092    185.62328    183.39307    187.19381    181.96837    182.29449    179.04617    165.87548    154.99606    142.27611    139.17322    140.27263    144.94101    151.87731    155.92025    161.92715    161.70776    158.71129    155.48795    147.68116    140.29956    128.32597    122.85687    126.26108    121.82523    120.39954     122.1647    117.48173    118.20668    116.35042     116.5713    119.31252    113.72577    119.50654    119.70531    117.24312    114.37557    115.20293    113.06364    115.96804    121.06064    115.97861    115.08107    114.89253    109.92751    110.24473    111.58326    115.12306    115.34363    123.53168    120.37215    117.02862    108.17748    108.13791    102.55107    101.52107    100.35837          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       42.00    297.60721    267.14506    252.81784     243.7878    248.50625    234.92211    242.63669    226.55804    209.67745      211.686    205.23059    202.35227    205.64663    203.96964    197.72512    188.16872     186.6273    186.66698    184.96684    178.39732    175.38829    180.82032    179.42856    178.01552    181.68997    185.89734    195.44279    193.96782    188.07723    171.29961    173.44218    164.79344    162.15746    160.18275    159.85714    157.09725    157.06622    155.98598    154.76892    156.07507    161.12152    151.42224    147.44991    149.82007      148.961     156.7296    154.48498    153.44513    151.18897     151.3244    147.13208      145.131    145.74662    144.85055    148.25616    153.23264    162.55571     165.7091    172.62407    165.01436    158.02721    150.58708    146.67619    141.05519      143.914    143.03534    143.68545    140.15651    146.98189    156.22372    163.85069     176.4574    179.81081    173.05546    164.99699    160.68655    166.17806    172.39687    177.62326    182.74969    181.11541    185.21807    184.22917     178.2882    172.73895    158.67393    147.87982    141.74468    135.66093    139.72758    148.14261    155.61826    153.21832    157.03477    156.26657    154.82095    147.15789     140.7636    135.83577    133.62521    120.94198    123.71331    124.42475    122.14242    124.90373    120.37117    115.43338    118.25174    116.52234    114.70454    116.88442      120.123    118.06093    116.24911    115.41065    112.47019      114.297    116.78743    117.90587    117.78461    113.22993    111.61157     112.8022    108.06116    111.90836    114.72426    121.80601    118.59885    119.14796    112.56461    108.07041    101.94093    100.09102     92.64331          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       42.50    295.15245    272.93912    254.51964    245.24611     241.7337    232.35145    230.97275    226.27874    214.00084    211.88955    204.65426    199.93781    204.49019    194.64865    197.05537    187.84441    185.55097    187.19152    180.25059    172.90274    178.44224    181.84136    183.42573    177.17008     189.0979    188.56819    193.84519     191.3857    185.80867    174.60472    172.52828    166.36724    165.51656    161.77037    159.50093     156.9288    156.63987    158.53461    162.85135    157.11989    157.23411    153.13453    153.23076    149.84992    150.77422    156.43424    153.37881    145.12428    149.05951    147.18853    150.42542    141.87185    145.85654    147.67943    150.57872    156.66367    160.68852    168.54718    172.28072    163.92014    159.11816     144.8214    146.18832     138.5886    143.18125    143.55232    140.59302    143.67391    144.47045    152.92374    159.37477    172.75231     178.2984    177.23943    166.10936    163.12129    163.05372    170.18117    168.87625    173.06572    175.85396    177.77612    177.44737    172.85387    159.45377    149.18021    144.04549    136.21548     133.4458    144.49532    149.59117    157.97309    159.46692    156.37309    157.58739    157.15724    146.00406    136.01927    125.01991    129.37466    128.52242     122.4793    122.29925    120.78154    124.35778    119.92999     117.0038    117.25985    116.15974     115.9485    116.10959    117.39471    115.57276    114.88679     115.4639    118.62298    114.44585    114.08176    118.68931    118.55454    117.38955    113.37306    109.18715    108.58435    111.08054     115.1529    116.87975     119.0197    115.69429    112.15059    106.22393     95.38908          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       43.00    292.38365    279.80924    258.77043    246.91149    237.33347    233.99811    225.45617    226.79912    218.83747    209.70644    205.87684    201.15336    203.01414     199.0371    202.37567    194.45905    191.82507      183.496    181.38435    178.76126    184.07262    177.11447    182.91814    175.24858    188.63499    186.55102    194.73983    196.83647     186.0282    173.52711     171.9824    169.76347    167.63625     164.2142    158.96862    156.32904    158.62926    157.40552    156.58341     158.7575    152.71318    158.37153    155.29731    154.80973    152.96313    152.06939    153.16426    151.07848    155.10478    152.40512    151.63926    148.18739    147.63021    145.25935     151.6085    153.15164    157.75353    170.76835    171.74979    169.05108    158.94281    149.22897    140.25746     142.0101    142.28805    142.87316    145.57779    137.66253     141.2162    150.21183    163.76966    175.18983    182.03046    178.30652    168.25717    156.82764     158.0227    156.52068     167.8122    169.03545    165.41158    168.10468    174.17324    168.27841    159.59189    140.86942    143.97297    138.15701    136.16881    142.22185    154.10818    158.55267    163.51732    160.87844    156.51141    152.44546    141.33615    137.54653    129.71052     123.2617    126.13441    126.72442    122.33879    120.58922    119.28993     117.8123    120.48977    120.42046    117.48528    117.88007    116.22215    118.49664    115.83881    117.33125    116.11453    118.49888    117.35066    117.91056    117.19298    117.60553    115.00336    113.06929    111.66615    108.86038    111.72315    111.16977    116.06206    116.20899    114.67927    106.86576     93.04865          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       43.50    289.68152     276.9046    262.81338    250.42746    238.47395    236.06748    228.07137    227.83601    219.94308    209.99312     207.1362    201.42753    198.82753    198.49131    199.93556    197.79773    193.16513    186.39021     181.1684    184.36205    184.89465    172.87708    179.56092    178.56581    183.97978    188.62869    195.57356     196.8069    186.72971    176.35384    174.26758    166.46647    160.84561    165.93835    164.98569    158.51348    164.05497    159.77865    159.55681    157.32503    158.07325    156.22332    153.91686    147.74409    150.48484    154.73755    151.39426    148.27662       153.56     149.7337     148.0202    146.83636    147.66982    149.15496    149.93214    153.67023    160.57297    167.11209    170.27022    166.98309       157.76    150.77234    141.63146    139.28365    143.08037    145.82839    141.76981    141.12925    147.54002    149.69591    162.44522    174.13171    179.20173    174.46626    163.16417    163.62111    158.19236    159.50255    159.37089    160.70802    162.11626    162.68271    167.72786    159.23309    152.77982     144.7029    137.32775    137.10187    135.38612     142.1184    148.25759    153.99211     163.3407    159.02898    152.90025    145.19644    138.33345    133.02394    129.73703    125.38852     124.0322    126.85118    125.07163    124.20098    119.79402    115.92798    118.34052    119.00492    117.97065    120.86158    114.24006    114.51561    116.38531    119.12303    114.89806    115.02044    117.65204    118.03593    118.75041    116.75258    116.64727    113.28425    110.70754    110.11144    107.86439    111.10104    112.94002    113.72259    102.31017          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       44.00    288.18025    274.43744    266.30733     249.6011    240.18664    237.08898    232.09597    229.29552    220.69042    209.38909    205.96626     201.2025    198.62202    196.82948    194.84463    193.86674    188.64406    185.88046    180.00829     183.7146     185.4717    177.52052    175.39996    179.34064    181.96306     188.0491    191.71207    184.29716    185.66138    177.98999    172.62036    162.96424    161.27079    169.11614    163.64394     162.1134    161.10069     162.1205    160.68418    156.18877    159.95289    156.60099      159.076     152.0499    149.24475    148.49078    156.03053    148.14244    152.03334    147.07664    152.60334    152.77245    150.35773    149.41085     150.7388     152.1053    160.62765    166.05534    167.98642    171.72532    158.03542    149.33251    150.65872    141.69298    142.04003    138.69639    142.74921    146.87244    149.58174    153.54235    165.04068    173.73039    176.04375    174.73847    167.67823    164.38971    157.73644    158.58805    155.10494    157.94075     160.5275    163.58624    164.83422    161.87971    155.53119     146.3249    138.72572     134.1442    136.94082    138.79784    151.75359    157.41142    161.11454    157.51331     154.1945    143.46354    136.95651    132.49193    130.48352    127.74813    127.06332    127.50422    125.27469    122.60133    119.84685    119.71964    117.30386    120.99275    121.00539    117.97421    117.99939    114.75245    115.62588    115.46056    113.21693    113.21989    117.46098    118.23041    117.15395    117.53906    111.22529    109.39162    110.52421    108.28994    110.45631    108.57541    108.19212     99.66893          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       44.50    292.10473      273.573    266.45205    247.71238    240.30298    239.79055    232.27599    230.69996    220.91495    208.07138     206.5369    201.39324    203.00737    195.18641    195.08553    191.38016    187.13383    185.18529     180.5743    184.34053    183.08343    174.02735    178.93159    176.62324    187.88391    185.61978    192.33992    189.88556    189.98938    179.30122    169.14471    164.85314    159.38015    163.35062    161.43512     162.5269    157.46239    158.84813    161.55724    158.46587    157.27286    157.45192    156.24144    152.39019    150.54851    150.62965    153.41281    148.87882    149.05244    145.51173    153.35331    150.70638    146.24549    143.08958    146.92319    149.43189    161.80459    168.94866    174.70989    167.29749    159.01437    153.65208    138.43775    144.25372    140.05124    144.85715    142.68131    145.68591    145.43729    159.91791    162.70602    171.67276    176.02928    175.11197    168.61992    160.76428    158.60975    159.43689    157.64389    158.09207    162.12158    162.74502    164.65108     163.8369    151.70798    148.01249    144.27021    139.94682    136.43528    144.52258    143.34069    154.29421    153.76924     153.5304    152.67902    148.25265    137.13117    129.32481    130.71975    130.05809    131.64066    125.82157    123.74129    123.93881    119.93205    120.35059    117.30489      115.313    118.25427    116.19952      113.887    121.49736    117.30092    117.42995    115.83125    116.99221    116.39985    117.85993    116.74304     115.4198    110.24227    110.40504    107.40673    106.89403    105.15509     100.8749          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       45.00    295.66124    273.84241    265.44574    247.68588    238.22541    240.26226    227.42919    229.81505    221.67077    207.94024    209.72519    200.34215    204.35911    191.24787    195.59408    193.96449     188.4435    187.72602    180.56017    180.67561    185.98211    175.18292    183.56477    181.33138    192.72141    188.43327    196.00616    199.57194    193.98701    173.88616    169.24418    169.49006    163.59575    168.56179    164.82207    163.78134     158.5792    159.38956    158.97991    162.04412    157.82956    159.30942    156.89408    154.76873    149.50531    153.64842    147.85468    154.84068    152.78709    152.69988    150.43267    154.35535    148.70617    146.78741    149.62864    150.62855    158.76862     168.4883    168.86252     168.0852    159.77242    150.98284    146.38164     146.5613    142.21986    137.67609    142.51207    142.49717    147.19438    152.58748    165.17207    177.03475    179.80793     178.3061    170.30208    160.43904    162.19356    157.88306    162.48666    159.16908    163.81834    169.55234    169.99636    163.09875    150.15304    142.90249    146.04331    141.41286    138.56894    135.05467    145.79471    154.26111     152.9622    159.81252    155.39698    149.60337    140.91077      134.631    130.76951    129.90034    129.54274    126.10489    123.87949    124.33919    123.33329    114.69281    116.42991    117.74719    116.40232    114.81911    116.87144    114.52535    117.97384    109.34844    115.73419    114.58466    118.37246    114.59332    118.96942    112.22124    115.78625    108.22504    105.56743    102.33826      94.2901          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       45.50    297.36143    271.79823    264.31455    252.20572    237.21873    238.39156     223.4046    226.81054    225.24135    209.28642    212.51028    199.81736    203.90613    193.32002    198.58823    194.03491    186.47713    190.36447    186.49994    178.12623    180.74848    174.03532    180.29961    180.46601    191.75603    191.42329    202.37897    194.71231    193.08577    179.43893    172.02625    170.07713    164.49824    162.68331    164.36737    166.96746    152.46611    157.36032    154.34813    158.85624    158.57156    159.00695    155.92585    154.48239    150.93473    146.29407    148.15086    146.09849    149.94525    149.79378    150.85646    152.46214    154.86031    148.82912    147.63365    154.69589    163.14554    159.87763    169.62777     166.4556    161.13757    150.78035    147.47783     142.3857    138.70967    145.98886    142.76941    143.90633     147.9051    152.36876    165.74801    172.00544    178.99307    171.94966    165.73271    158.38839    157.58248    164.74053    158.77199    160.84364    168.14815      170.353    175.03862    168.79234    152.84887    145.33591    141.70474    141.11564    140.21262    141.99626    142.40991    153.36386     158.2998    157.98306    161.72574    157.96229    153.26297    145.71682    133.36621    132.09455    129.83101    125.16237    124.60259     125.7881    120.81755    117.88835    120.66427    116.36324    115.58186    115.23718    115.52863    121.28188    118.70568    115.83381    117.36703    115.05479    117.93139     118.0209    114.54277    119.88543     111.0692    109.69653     98.63834          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       46.00    297.51652    271.12798    263.48354    255.24427    238.97199    233.74367    222.18327    225.32963    229.53606    213.13312    212.32995    205.25002    200.04508    197.62612    198.42702    192.11902    184.90145    189.86242     191.0435    179.45732    181.21604    176.81749    175.51984    176.03006    185.16689    190.26114    198.25917    188.92109    186.75355    178.96559    173.41391    174.00464    165.96833    163.00976    166.79415    169.82062    161.94156     150.6423    157.27166    156.13637    158.99727    158.59468    153.88196    156.59053    157.55964    151.19691    149.55115    150.90634    153.11022    147.65893    152.06515    150.44271    150.88578    148.63758    153.07839    152.53818    160.59096    169.77961    168.96537    169.54385    162.73683    150.15221    146.10848    143.85519    143.45449    142.30286    144.62279    144.18769    145.02494    153.26705    164.19607    173.16249    180.90514    172.65711    161.34521    159.58411    156.18928    161.37435     162.6966    163.93666     167.1316    174.85397    183.57826    179.20856    161.30697    148.75609     141.9346    138.93166    134.60808    141.46469    145.22484    151.07105    154.71627    156.93287     155.8149    159.23826    153.61989    147.77513     137.2915    131.86259    128.93387    123.26782    122.94581    125.84973    121.67697    119.09759    121.46656    117.68334    116.45776    122.14849    115.01866    115.31965    116.49231    118.66156    116.57081    111.61142    114.54229    112.06161    118.40814    116.37442    114.70964    100.73391          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       46.50    299.26604    271.83097    264.72597    255.03464    240.42409    232.29906    223.56167     222.8657    231.62816    214.69046    208.56239    211.47794    200.29025    200.33249    201.21969    192.84557    182.73143    186.99667     186.5631    182.60416    176.14605    182.85449    177.17253    176.85116    185.46175    186.79861    199.36449    197.85302    184.74965    175.25634    171.69328    170.61501     168.3275    166.51942    168.11134    165.68925    164.32075    157.60147    157.46674     157.5182    164.08163    153.54658    155.34239    152.92203    158.55574    158.59474    151.12794    151.45899    151.60484     148.2684    149.52672    155.38135    148.73212    148.16174    151.48092    152.58698    156.62456    164.44192     164.3275    163.14137    158.65593    147.52305      144.444    145.08158    144.61343    136.56458    142.92771    140.42848    147.23653    155.73472    163.14359    176.98118    176.39062    172.28982    169.04551    161.17155    160.22002    163.51244    159.80251    165.68889    173.79156    178.41893    185.06161    182.71115    169.15263    151.98188    140.77704    139.71225    136.04574    140.16099    150.34838    151.75387    156.16246    156.45225    159.78874       154.73    152.01157    142.47455     139.0872     133.7028    127.32978    127.27978    124.21188    120.03761    121.39541    124.18328    118.09372    119.78548    116.77778    118.47688    122.82479    117.86159    116.16775    116.47556    117.32275     116.7776    111.00107    114.83412    115.54846    108.84469    101.43196          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       47.00    301.63446    274.08233    265.94938    253.66473    242.34767    232.52944    227.57835    215.99453    226.40003    214.90156    205.40155    207.25099    204.73305    201.93381    199.82344    194.75129    187.86548    187.75283    188.97988    187.36496    171.72125    178.58761    178.77978     176.4538    178.93829    184.42062    199.98177    199.24929    185.63181    168.80751    169.51787     164.2402    168.71343    166.87056    162.92166    162.74585    158.16359    153.49268    158.04602    157.21178    161.72057    160.35044    155.95335    159.33281    157.02516    160.81577    152.60686    148.25793    151.58249    146.23927    153.70405    147.39975    146.02167    151.20129    153.91978    157.35673    159.28948    164.61318    168.15904    164.26973    161.10247    150.29811    151.16514    142.78792    142.07628    139.71965    139.07669    145.03393    148.93963    154.71156    161.14897    177.22056    177.90214    177.77944    170.07002    165.50865    160.51956    160.85588    165.38177     167.0488    171.54539    173.33729     179.0208    176.90276    166.20815    156.72421    140.20133    143.58958    135.23296    142.35197    147.36143     154.2118    157.03294    161.68778    162.16872    156.97631    153.35365    143.14415    136.63397    131.64536     127.6946    128.13444    121.84888    121.61979    123.44311    120.54918    121.09136    119.33144    117.65642    115.10736     114.6868    119.11132    118.56605    115.37251    111.92028    117.11938    115.77044    111.52832    107.66793     94.98681          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       47.50    303.79445     277.4441    267.13284    254.87996     243.6148    231.30524    230.47805    216.71524    223.74753    220.06507    205.38538    203.39259    205.59213    202.20573    193.93046    198.30524    194.66695    190.58873    187.85083    189.58292    175.61279    175.06392    176.83067    182.36819    177.61973    190.18725    197.09218    201.21639    191.47877    177.61506     172.7234    169.37602    168.85398    165.26175    156.47704    161.41287    163.00719     165.3344    154.14113    155.16842    159.67638    160.46823    158.13725    154.83619    152.06266    157.10002    150.58917    150.93231    148.66906    147.52643    150.97047    145.21139    146.75567    147.09143    155.55564    161.25089    163.49135    164.22331    165.75071    164.38448    155.95933     152.6439    150.49165    147.22555    144.96301    147.63773     141.7791    146.17279    141.55468    153.12546    168.89485    172.44693    175.77379    174.37648    169.94853    164.28179    161.83687    164.13071    168.49164    169.37213    172.71318    172.09117    167.67066    168.71826    158.52049    150.82403    147.16141    137.67693    137.89684    142.91354    146.23824     150.9455    161.76247    157.55879    165.34395    155.85586    153.76002    147.08086    137.75014    128.86305    129.15278    124.38665    125.40967    122.09213    122.13198    119.23882    120.55155    116.22068    117.85304    119.09634    118.00222    114.87246    112.52653    115.05842    114.09606    115.93262    111.59041    105.74372          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       48.00    306.07467    280.68356    267.02596    256.08435     245.0101    231.20444    232.00676    221.82499    223.45619    220.55738    209.22858    203.77564    202.06446    195.53477    192.18218    197.93286    194.83645    192.52439     187.5149    192.08644    180.99245    174.71822    175.42285    181.42538    180.50228    189.56602    192.46755    195.88372    194.51307    178.70771    174.00367    165.43871    166.88391    168.89493    163.16603    162.23602    166.58766    163.33945    157.06464    160.36071    158.94988     155.9125    159.45253     161.3145     150.9457    156.79214    156.98224    155.13906    154.29372    153.34673    146.62603    151.78717    147.54096    153.01015    152.46586    151.71609    165.02867    164.55355    164.89942    163.52711    161.43772    150.80372    146.13973    143.50019    141.01254    145.42734    140.36097    139.95923     144.2884    148.97468    163.20074    173.56237    177.07524     174.0307    173.66856    165.98589    164.94024    169.63337    172.51779    167.34326    170.06594    169.37745    172.80022    165.40263    155.99972    148.46732    137.80859    138.27404      138.259    140.55247    145.59304    156.40066    161.19987    167.19611    166.61778     158.9247    153.83486    146.00307    142.00122    130.84484    128.15078    128.27053    124.42448    123.82514    121.85872    119.84633     121.8254     118.0969    116.54746    116.52829    115.78618    113.04056    113.21495    115.90675    109.84675    114.07448    104.73615          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       48.50    308.65112    283.80087    265.81188    255.45957     245.5645    232.83257     229.1615    222.09654    223.81745    215.81385     214.0911    208.45817    202.32396    193.71681    192.76167    194.02719    191.77318    189.28804     188.7071    190.62009    182.38215    176.60187    172.38614    185.61587    180.72559    194.87652    194.08883    193.39434     188.4861     173.3704    169.70899    165.12602    166.64999    166.68666    166.33689    161.33361    163.33655    160.11423    158.19952    156.62633    157.53309    156.75906      154.959    156.89829    152.50901    149.21328    159.22781    153.26702    152.94043    153.10308    147.20288    149.01111    154.58401     152.5358    147.41728    155.77946    158.73609    167.09926    163.89698    165.17356    156.74121    151.45031    146.58525    142.97325    146.17749    142.68295    147.86529    145.95566     149.3064    152.36779    163.94023    172.73816    180.26785    173.60843    175.77595    168.52299    168.09118     174.0845      178.496    176.90017      175.854    174.00318    170.82059    166.41511    159.96105    150.65977    140.01003    135.62011    135.99611     140.6832    145.63708    158.10129     155.4856    167.00968    168.05523    159.64563    153.67256    150.62525    140.57555    135.01012    131.69723    128.65886    127.53219    127.96436    123.78544       122.75    121.42225    119.90943    117.65441    117.55987    119.18384    116.59212     117.3334     112.9263    110.96669     104.7438          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       49.00    311.33001    286.15399    263.20606      253.188    245.44157    235.54712    227.06179     223.4966    225.28181     215.6196    215.46148    212.15876    199.14347    196.79721    192.39809    195.04608    191.54166    188.05419    188.33482    187.93586    184.24476    180.91095    168.92169    182.63605    181.45156    192.10171    187.47361    197.31422    186.37645     177.8858    169.75618      168.947    167.84289    168.38318    162.84503    159.84419    162.45142    160.61116    158.00182    166.19717     156.8423    155.53239    151.90309    154.15052    153.08194    152.67277     152.4505    153.63726    150.70245    146.50685     144.5209    147.62478    148.35079    148.40067    148.76376     156.4836    167.50444    171.50784    166.61014    166.14181    157.23011    152.46233    143.52915    145.67481    139.03007    141.89483    142.55556    146.97188    143.04924    152.74228    161.18992    173.98667    177.04547     177.3097    176.89601    169.11903    175.60207    183.51013    187.31318    187.16769    184.62397    186.64478    182.72668     167.2723    162.12182    149.71798    144.07729    135.62608    136.75692    141.25815    150.73894    158.60797    162.32174    166.85146    166.33683    160.99346     153.4031     144.7947    141.16008    134.60366    129.58875    128.68019    127.56085    126.45086    123.29627    120.20885    119.79196     116.5685    121.98964    115.25212    118.04828    115.36802    113.53537    114.02148    102.87291          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       49.50    314.33971    288.67962    261.30664    252.74012    246.91134    235.49296    226.64819    225.14361    226.16203    219.37632    215.54029    211.15375    192.54638    196.90442    193.81208    195.58152    192.24116    193.61665    188.02706      188.243    187.71604    182.10269    169.54744    175.83952    181.94556    186.13223    192.20522    195.99043    182.54913    183.00513    174.57703    171.49574    165.61825    167.15551    165.86027    161.49647    163.66481    162.61273     162.6427    167.92082    153.38157    159.57782    149.41823    152.63528    153.51952    156.76207    157.24297    152.12217    154.64721    154.71794    153.74244    148.85981    151.26433    144.31802    148.15456    157.10114    161.94078    169.06449    174.22198    165.44522    162.15408    150.28393    145.94327      144.946     145.1487    143.20202    142.67809    144.30638    144.52811    151.89352    165.52709    173.44029    179.50188     173.8276    169.49547    170.50143    180.30403    186.42546    195.54959    196.09185    203.58379    197.02985    188.76971    179.62762    166.36926    156.14494    144.47077    143.96271    139.32491    141.50429    146.22742    157.29462    159.31667     166.2937    166.76369    158.96714    150.25288    146.57471    141.07072    134.29586    128.94337    131.71505    125.34576    126.46593    125.47493     123.5002    121.62266    119.57102    119.17793    114.89045    118.08929    116.92327    114.56492    104.47271          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       50.00    317.67884    292.03722    263.21968    254.21428     247.8138    234.54204    229.41507    222.31243    226.43435    216.58057    214.50117    207.50348    197.08085     195.5534     195.2194    193.33507    190.56318    196.08208    191.56762    186.12054    187.85489    184.21433    177.15205    177.72727    183.37468    186.76202    193.24857    193.88962     189.1254    186.84659    181.08501     168.2011     167.6645    167.80678    172.22491    157.27268    166.41106    168.28035    161.09999    164.30615    160.49882    157.47885    156.42447    167.42515    154.47592    156.18105    158.72261    152.08161    150.27077    157.98673    151.06882    149.09962    146.59314    150.94034    151.34801    154.79049    164.38991    169.71896    173.59753     168.1365    156.78849    147.69353     148.6717    148.17262    139.57625    147.64721    140.87591    149.88696     145.8517    158.48272     158.7116    179.93683    178.19387    178.73887    172.18518    172.30247    187.56088    195.45626    200.36351    205.09154    205.22868    208.54128    206.02914    201.98551    179.48882    162.26733    143.29802    142.17998    138.41559     140.3245    148.58032    157.27954    164.98014      174.497    171.50387    169.52522    160.46946    148.25086    142.23851     135.1558    131.02524    126.25563    129.89204    128.04705    123.11663    124.08307    123.91899    121.65933    121.12758     116.7125    115.96754     115.1662    104.90746          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       50.50    319.38317    296.22309    266.63367    255.23606    245.22995    234.98365    233.72367    221.06086    226.72113    210.94077    214.54585    211.46441    204.04267    197.82942    195.03996    190.31275    187.57543     194.1942    195.66525    185.86241    187.68863    184.89477    182.88633    176.11069    183.22848    188.08622    190.44115    200.10946    193.36508    187.42848    176.25814    165.77198    166.19492    169.02017    168.11501    161.86545    164.61039    162.46863    163.83815    156.57212    164.28256    159.18109    158.96154    156.37929    152.53709     155.1726    154.38311    155.13771    152.97071    157.14647    154.83551    155.19983    152.83617    154.10343    153.04629    152.30072    166.06243    167.99492    164.71823    172.03161    163.05506     153.9282    147.91283    146.37927    150.00304    143.88532    142.42317    145.28072    150.90919     157.0205    165.28833    173.80024    179.80117    175.20984    176.05397    178.70717    183.93545    198.42996     213.3173    217.06975    211.96349     212.0021    214.31096    208.41822    188.22575    165.42572    149.21873    138.11676     142.2116    148.28286    147.08809    155.54855    160.69222    170.82508    169.45848    169.22508    155.55312    149.24459    143.29271    135.69806    133.41046    130.02678    127.77286    128.06442    128.09491    124.84067    120.19513    118.74868    119.93361    117.98462    114.33814    104.34927          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       51.00    320.27199    298.12187    268.46847    254.05398     241.3956    236.65076    238.99011    222.16374    226.67333    211.74484    213.83214    214.78991    207.36239    197.57074     195.2877     189.1224    187.49558    191.04126    191.55452    189.83212    190.00273    190.36046    183.14595    175.58902    183.68744    189.14163    191.57785    190.85571    187.86859    180.19685    172.41895    171.58119    170.39928    168.71811    163.13087    170.59638    163.16151    163.18413    164.36629    161.42841    164.37368    162.78465    153.08931    157.59309    150.32162    159.70651    159.44045    152.91399     153.0016    154.70124    154.92107    156.34442    145.79874    146.58841    154.68954    160.17828    161.43357    164.69035    163.91829    165.73878    162.66539    150.17599    148.07727    144.18712    147.85154      142.615    141.41485    145.00722    149.45502    157.40843    169.54463    175.06184    173.66031    185.22707    177.50228    177.54848    186.68144    201.49586    221.86744    233.37278    221.07138    210.58742    209.74828    199.47903    182.02751    166.43682    148.94506    144.06537    142.57751    147.31681    147.60773    152.57582    159.30685    163.80893    166.73156    163.18678    154.51369    148.49351    145.68451    139.07902    128.58744    127.27011    126.66345    131.38461    124.65865    123.44902    120.63912    118.75748    118.19931    114.73438    107.41613          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       51.50    320.33937    298.33504    271.87371    253.22327    238.60903    239.35779    236.62862    221.52425    224.56384    216.55726    213.72649    210.97014    208.71285    198.73288      199.028    190.65435    191.59473    190.75027      188.436    188.64798    180.61886    186.90446    184.36014     181.0156     183.0921    189.50256    192.49226    193.73513    187.01536    175.23579    170.86988    170.37458    173.67758    164.12647    160.36921    166.96355    162.57189    162.76268    164.58753    158.44295    164.27854    162.23052    154.78066    160.70229    153.35883    156.90871    158.56232    152.87341    148.78871    149.55302     153.0692    158.44589    159.09455    149.44688     154.1648    155.50741    159.83855    163.38114    168.27195    166.24819    162.64862     159.0492    143.98724    142.88827    146.73078    141.59521    141.54895    145.46121    144.72391    154.75007    167.73989     179.0029    177.61788    175.78847    179.96694    178.45476    187.83591    209.17446     229.2673    237.80554    231.57605    209.74651    195.66803    184.26148    173.49428    160.32663    150.38472    142.52497    139.05884    141.35524    147.24678     152.5523    156.31239    159.65861    159.77457    157.30101    154.75854    146.42318    137.88374    138.27554    138.77668    129.44314    126.50406    129.37143    127.91009    126.43342    120.85615    124.17291    115.79785    108.77885          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       52.00    319.79088    298.58327     274.7407    254.63303    239.82099      243.123    233.17904    226.10739      221.024    217.41395    215.32784    210.23866    210.55085     199.8574     198.2856    191.64272    194.55815    185.64614    183.93967    178.57508     179.9519    186.57119      187.135     181.8566    180.17801    193.15652    193.27376    194.06637    187.14195    178.41404    180.64936    173.33467    171.59121    158.70844    163.29893    161.95026    165.86712     161.2811    159.12787    160.52874    161.94964    159.52103    159.06075    160.63315    155.42675    154.55767    156.04181     154.3144     153.0332     155.8349    155.15377    151.65733    155.70004     153.8206    149.75617    156.79491    155.89045    170.03414    169.46296    172.68483    163.63875    154.29723    146.64179    146.40952    144.88744    142.87361    148.32664     146.9197    140.66083    160.11702    163.37901    175.88792    182.97815    182.58073    176.76459    187.09891    191.16028    206.37966    227.03812    249.76041    240.64047    217.72701    191.96072    174.85122    160.50028    149.62428    142.83934    137.03374    140.44245    142.38712    145.10051    155.63632    158.93082    158.54744    155.14599    153.67862    148.79319    146.00936    144.34664    138.77173    136.35794    127.96538    127.57553    125.82174    127.02842    124.48087    122.59073    115.09951    107.14021          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       52.50    317.93446    298.60346    279.62873    255.92276    243.18746    244.60572    233.59622    230.05507    217.28256    216.45038    217.17015    206.40421    209.88894    203.58981    193.95244    191.30539     190.1496    187.32672    181.66462    175.20264    182.70469    182.99598    181.38323    183.60621    183.41985    186.21904    192.93234    191.26583    187.90654    180.33424    178.14068    171.54007    169.72922    169.24613    163.23463    160.90852    163.43632    162.71862     157.2188    162.78704    155.61641    159.95297    161.86287    162.77394    159.99935    154.03393    152.91152    154.72807    151.95366    159.91154    154.22724    154.10946    154.33316    155.55614    154.90716    152.82574    162.66714    162.85275    167.46603    159.82469    161.49948    156.08561     145.4459    146.99146    146.16394    145.37061    149.98396    143.59154    143.77787     156.6288    162.05444    174.33372    177.03971     184.6561    179.80667     175.4227     186.8666    201.18281    220.72737    241.33559    238.29058    223.15991    197.63637    173.33187    155.77406    143.80979    141.23631    139.74832    137.19774    134.98557    145.35792    156.16885    157.47461    157.02109    157.39123    151.51871    146.42455    145.71146    144.62717     137.7174    133.45814    132.31571    130.18909    127.20126    122.53517    122.71475     113.6788    109.45601          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       53.00    315.79402    298.41872    281.71832    257.47262    246.29722    244.24627    235.41079    229.57351    215.66196    220.44744    214.54978    206.47058    205.41729    209.35734    190.05501     193.3467    180.80591    191.24472    181.27022    181.81621      181.378    184.36637    178.51657    182.13997    188.15617    186.50677    191.34029    197.98596    185.32665    185.48071    171.07729     168.5939    170.53195    167.75206    167.72672    158.65137    171.56011    161.01685    159.91681    162.50819      157.534    158.55407    163.07842    158.62287     158.0649    157.99403    151.49124    158.06918    152.85145    155.88843    157.29328    151.77748    154.01038     153.8904    154.31927     158.7537    164.12737    163.33731    164.10189    171.75771    160.82395    156.89732    149.13922    142.43962    149.77907    145.80507    145.69602    142.23061    147.74929    154.63178    166.04078     174.1459    176.02991    179.16929    176.23554    174.87073    178.25952    194.85344      210.652    227.94364     230.2243    223.50559    199.64088    176.87497    155.74824    146.49721    135.34346    140.09112      136.738    143.04277     146.6771    146.39019    154.78806    156.58611    162.55236    155.24816    150.46115    146.56136    137.83007    137.90386    132.25674    129.88144    131.04546     128.6728     124.9519     117.0453    114.60084          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       53.50    314.88797    297.57742    281.80316     262.9186    248.64266    246.83526    233.51749     227.4793    219.57618    222.71241    212.16424    210.31211    202.87527      210.508    191.00328    196.44543    181.72596    190.61674    180.83132    184.55353    181.90789      184.634    181.70585    182.10823    190.63344    193.75029    194.46735    194.76842    185.45386    185.48065    174.82328    174.69844     170.6905    168.61957    167.42725    154.27946     167.6329     157.9268     158.5313    160.00975    163.28186    165.63118    159.53458    162.49717    162.30475    160.62969    158.06835    154.33062     153.3368    157.52911     158.6023    149.39794    156.95345    154.66533    152.91433    162.81823    161.02345    164.05046    165.72208    167.96933    160.60357    156.25394    154.34634    147.95703    146.50155    146.00461    142.37793    146.40466    142.97688    154.75985    158.77396    167.50859     177.5801    179.25436    173.89812    176.87799    176.61118     182.0248    200.00425      206.921    219.00922    219.49349      200.205    179.56444     159.3625    148.39788     141.1035    139.46015    139.13577    141.79792    143.38335    149.22245    154.79586    164.54948    171.92078    174.37975    164.83979    152.62861    138.51289    135.78948    130.53851    132.28543    126.72187    126.25452    125.14674    119.63031          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       54.00    314.53947    296.63276    281.24789    268.86706    248.46746    246.36565    230.38146    227.21187    221.53807    216.05288    206.90921    208.25663    205.13787     209.0282    196.99622    194.40766    186.73636    187.60498    179.32936    183.44931    185.51328    184.08734    179.75617    180.36403    188.91241    190.49208    193.67915    189.17014    184.34357    182.63232    173.57827    179.28385      174.008     166.8452    170.18826    165.78615    164.47275    160.49653    161.03208     156.5985    162.36809    161.32891    156.48573    160.14661    156.16996    161.25005    157.74074     154.2451    151.17905    159.52771    156.63744    155.10581    149.54835    149.55683    158.08773    158.53217    162.07832    166.50706    165.67723    167.72297    160.64307    155.37683    148.70814    147.79551    153.52635    144.13687    143.62888    142.22488     148.3815    152.26592    164.02715    170.32837    178.82024    178.28223    174.78626    173.68666    168.11204    178.51905    185.63195    201.71135    203.97163    206.99031    197.11521    176.64494    165.08932    147.33219    143.76714    140.65507    136.83354    140.46815    149.20212    149.11666    155.78059    168.32118    182.99536       190.32    187.66321    174.01147     148.3692    141.97772    129.78988    125.00023    129.13888    126.87866    120.73291          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       54.50    312.97526    296.94294    282.21125     275.3138    247.38983    241.37451    228.06344    227.42954    222.77243    216.03033    205.46175    207.26635    206.69635    203.93453    197.62623     194.6504    191.16251    189.27572    181.50099    184.12736    187.03837    182.69895    184.23135    181.15582    186.84086     191.9664    189.52488    186.42679    191.15985     181.7479    172.20668    174.40789    171.68279    171.43628    172.09231    167.07553     162.3926    162.85458    155.42638    158.94862    160.17307    162.93314    162.34819    160.17057    158.46532    160.75061    162.66277    151.52472    147.95319    156.70396    155.66907    152.54701    144.91128    154.64063    152.81096    155.36582    163.49488    173.84741    167.80336    166.52177    160.05693    155.14175    146.55983    146.73442    144.23478    147.28074    145.72293    150.25701    150.65093    156.48785    162.72181    173.13046    177.40329     172.5261    172.87204    172.36502     164.0343    170.84269    185.22994    187.69352    195.29958    197.81564    191.03901    178.68783    161.36374     153.4005    144.80975    139.50241    139.89493    141.33866    143.82005    150.96086     167.3763     171.0708    186.99881    196.21473    192.50853    179.78242    163.67599    143.54255     133.9601    129.05715    123.98758    120.08013          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       55.00    311.80968    298.96856    283.88726    278.23401    247.19312    234.30664    225.68297       227.09    225.40363    217.39934    208.49046    209.71733    207.14779    197.16214     198.9089     198.6234    188.61022    191.13939    179.92048    184.90443    182.89354    181.75092    184.13208    181.35855    185.33987    192.79736    194.47381    184.64119    189.22749    182.69573    175.19412    178.54703    168.39605    168.83377    171.20889    167.63168    166.58358    160.82331    160.83577    158.51738    161.22917    162.54625    157.25016      159.262    159.33725    159.61669    163.12495    153.97125    151.77118    157.99122    150.87536    152.94187    154.39569    151.24051    155.27132    154.36911    162.63203    164.34584    172.10903    165.78224    164.37917    154.63932    144.31583    146.55426    145.40957    143.49105     147.9555     145.9743    151.60053    154.96815    164.08399    173.98972    175.04171    177.45573    178.96034    174.31463     168.4004    169.96434    185.13409    185.19416    184.18448    186.17336    184.00229    172.89907    163.60721    149.36129    143.96129    141.58186    140.12342    146.49098    143.77949     152.8473    159.44575    165.42816    181.65546    192.35571    190.00635    181.80827    157.37751    151.53729    135.63425    128.96362    121.70373    112.41791          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       55.50    312.96361    302.69405    283.23006    278.67248    248.60469    238.94122    230.53869    226.44808    223.34842    215.25165    215.38426     210.3266    203.78864    193.89138    199.77902    200.51371    189.40239    188.50307    185.87509    186.06611    182.93145    179.44773    178.38476    182.17768    191.48355    192.53588    195.51787    187.22191    184.50684    181.04495    177.40315    177.18325    170.36315     160.1363    170.53085    168.46064    169.40617     164.1681    163.73779     165.6572    162.83311    162.62728    163.26254    161.16515    159.38369    167.48084    164.60992    167.32029    159.87286    154.75638    159.97191    150.27533    154.38103    151.70763    155.13767    161.31982    160.92234    169.87298    168.33667    170.19767    161.99137    152.21531    143.24105    146.65464    142.10618    146.12879    144.27551    143.65122     149.0486    152.72894    164.99509    171.84855    174.36039    178.50092     174.0618    173.48736     168.9972     181.8785    183.86546    192.78322    191.65817    184.86424    174.03838     168.4112    154.67791    151.12098    141.64544    142.49756    137.31025     144.6431    146.04429    155.81295    156.70506    157.79913     171.4352    173.99517    176.54972     167.5334    156.33582    146.17815    133.89483    125.52424    113.37218          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       56.00    314.44989    304.64847    280.96531    278.41472     254.0203    244.94773     234.4632     226.2829    221.43057    214.83442    215.90973    209.25201    200.94296    199.26558     201.9738     198.9495    191.33168    184.28375    187.82462    181.90179    183.07245    182.38204    179.85149    180.73042    191.37277    191.45771    197.38504    194.89939    178.45808    182.08693    176.06725    172.46824     172.7659    170.75534    172.48266    166.10197    170.39094      170.546    167.44841    164.24623    166.09732    158.61733    160.66265    157.21129    156.35169    162.37184    161.38581    152.61936    163.48104    155.89247    154.80707    154.32121     152.0418    154.87289    159.29988    158.05262      163.412    171.66367    169.76789     173.0502    162.88328    158.96068    153.96132    146.88268    152.17524    145.08866    152.04522    145.40867    152.20874    153.16959    166.78413    172.26615    180.11917    178.97759    169.47709    173.44109    171.92155    180.63298    191.86857    199.98898    199.95104    188.17623     174.0013     164.3998    153.81485     147.6011    141.36018    137.29958    133.42359    138.10802    148.11421    151.60066    160.41141    160.38117    165.33011    165.60092    157.71474    148.33587    143.41498    135.06154    127.29211    124.92999          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       56.50    315.20599     304.6923    281.21747    276.52595    258.79201     251.9312    236.65968    225.03971    222.61334      216.516    210.46789    207.24154    203.07858    204.97837    202.70229    198.70746    190.22538    186.80286    186.35965    183.21987    182.13432    185.64127    178.69485    183.91012    183.56414    188.14977    196.15328    197.58969    185.20194    184.83055    176.84533    173.02133    174.01073     174.6712    168.15768    171.62505    172.75461    172.86448    173.20233    164.46479    166.45651    159.82773    155.24236    159.41261    155.72955    162.21965    156.41864    157.49403    161.82378    158.81242    152.76013    153.79685    152.18057    153.48058     154.7199    159.16823    163.79674    166.75365    170.87744    170.59156    163.42937    155.74731    152.12715     149.1358    148.87881    151.90529    148.06108    147.78333    153.37518    156.18281    167.19724    169.25644    179.32154    180.17099    170.92836    174.41817    175.47772    181.78938    193.79056    202.75128    197.71671    184.77389    179.78398    164.05313    149.09138    144.03201     141.3589    138.58536    135.99344    143.34371    143.52427    148.99155    156.21384    156.28134    153.17735    155.11096    151.76333    140.93398    138.20651    126.47642    120.99158          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       57.00    315.23971    303.58557    283.04085    275.99583    260.67371    257.55633    235.91689    223.24698    221.16722    217.33015    209.05739    205.89016    207.64168    201.43549    197.56569      201.495    188.81775    185.79622    185.17793    183.37309    178.35662    185.60684    182.03325    183.93873    183.02604    186.89468    193.77679    193.78773    189.38761    180.92708    174.05472    174.45982    173.37524    170.65502    168.50834    175.75878    172.43443    170.55381    170.94561    164.90373    161.78288    154.30559    161.65092    157.31908    159.95397    159.33318     153.2569    157.36455    158.90854    156.95232    159.19731    156.40448    154.82398    151.94112    154.03019    159.53552    162.44905    164.29403    171.78992    163.75263    167.39472    157.62882    155.97885    147.58402    144.80437    142.38202    142.73165    147.94285    143.59783    150.49456    163.80662    178.86572    176.02897    184.59181    173.39695    173.39666      176.142    188.99847    195.86232    201.45815    195.07013     187.4458    177.39704    163.19319    151.14341    143.54216    136.59223    137.60793    137.71587    140.96872    142.97186    149.46177    152.21349    153.95741    153.38972    149.80174    139.59642    133.85975    134.18273     123.5264     112.7342          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       57.50    317.54973    301.58607    283.94204    277.56267    260.41338    260.87449    234.36786    221.83758    220.26785    211.58103     209.0161    204.91606     207.3524    197.31844    199.21681    202.53499    189.17716    182.60044    183.80567    187.44819    179.54088    182.81268      188.205    182.82336    183.99645    186.23607    196.55453    185.83829    185.28627     188.1303     176.8998    171.21388     173.1705    171.25769    166.22081      174.002    164.36399    164.14913    166.83767    166.68294    164.80925     156.1075    165.64794    160.50712    156.36702    165.89105    165.84611    164.47729    153.01789    151.15762    156.32001     157.8575    152.98032    153.38165    150.20088    164.59102      165.206    168.44334    170.24387    169.80889    167.00819    160.30309    154.12305    149.37198    148.34391    152.26691    145.75456    140.02578    155.12755    155.66293    167.47381    176.24573    179.76248     183.4103    179.65972    172.50928     173.3345    183.21968     187.0084    196.45103    186.91927    177.12792    172.46004    163.27172    152.75218      146.859    132.08183    137.53119    139.56129    133.25068    145.32453    152.01522    150.17565    154.78942    153.91279    149.76688    139.34248    132.30491    124.57968    114.11433          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       58.00    321.26052    299.60682    288.07587    276.26999     261.7506    258.46848    236.90511    223.65096    223.03247    206.52174      207.629     202.8042    208.25485    204.15602    199.41394     199.9972    193.32053    185.79318    189.11536    186.55361    183.11933    186.40194    191.18505    182.94454    189.28426    192.55531    201.31612     192.2655    186.17625    190.72577    179.73347    168.14012    170.36644    171.78021    167.86867    168.46889    168.62419    169.44445    173.80815    164.88021    170.57292    158.39277      161.654    160.98826    158.96561    158.36577    160.46918    162.20579    159.69425    151.65208    154.15725    156.59962    153.85741    153.37629    157.96215    155.26831    164.42791    173.17175     172.8292     170.6956    164.19425    156.33781    151.17677    150.43178    153.04456    145.95205    146.55985    149.18848    148.18661    155.97944    169.37617    172.78559    177.48746    179.68104    179.20579    176.65423    174.36838    188.78217    180.98302    186.36004    183.06714    178.60047    167.94279    163.00751    151.46795    144.28207    144.55678     141.4718     137.1693    139.57657    146.36798    151.45119    150.85911    154.10313     153.6807    146.42537    140.18474     131.7084    119.37541          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       58.50     325.4089    299.83169    293.74926    272.39118    263.93784    253.97412    242.92188    228.24678     220.8599    205.58526    205.60561    204.94928    210.10194    210.06668    199.39891    197.33391    193.43991    190.42411    195.67931    175.93798    183.18606    180.13163    184.74784    185.81624    188.55687    191.70423    200.89233    192.98257       186.57    185.13502     178.5383    170.37431    166.79263    171.24729    168.47493    165.97335    173.41492    171.29868    171.06744    165.66727    168.81026    166.68915    160.77809    159.43359    158.14884    158.62385    159.22226    156.61584      159.671    161.94525    165.58136    157.37396    157.72237    156.21139    157.81495    158.26255    158.39615    166.10878    166.89051    171.08461    159.05293    155.52269    153.29563    148.80758    147.44188     145.5345    146.31004    148.69405    149.97555    157.35003    161.54266      169.552    181.55708    181.28855    176.93637    176.28367     181.8048    180.06733    178.37933    179.64123    179.60188    176.34042    167.53351    159.22273    154.36702    145.19365    141.47302    142.15113     141.0461    138.61681    145.62156    146.91118    153.04307    155.57911     148.8819    147.09061    135.50289    126.11581          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       59.00    327.54432    300.97481    297.46148    266.17661    265.27134    252.96906    248.18698    228.91283    218.88796    205.32341    206.37922    208.69734    203.77675    208.29209    198.77638    193.31197    192.18248    190.40521    193.57988    184.76459    181.39377    182.71884    181.97441    187.03401     187.2345    183.98324    194.34155     197.9059    186.24816    185.75233    174.71875    183.99809    171.60312    170.50173    169.57332    165.09861    169.78714    167.70588    167.80388    168.91389    164.97126    162.26587    160.78315    166.17693     156.2062    152.18182    159.17294    157.89587    165.65824    162.17301    157.25147    158.01745    161.67633    158.61566    161.76157    158.53151    161.32695    165.12855    170.43853    170.27624    164.65987    160.94128    156.06448    154.99396    152.57634    149.96928    152.10702    151.85587    155.72071    155.87069     169.3134     177.0111    177.14606    184.38735    181.66489    170.67331    177.71844    170.15045     174.1244    174.10757    175.50953     170.1796    162.64284    164.07015    151.98843    145.36743    142.57796    140.52145    141.15499    140.72392    142.84309    147.40664    157.00658    152.46304    148.41303    142.09824    131.47625    123.25892          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       59.50    324.44802    301.88257    293.26486    272.76456    264.36242    254.89248    247.15269    230.29029    219.76219    207.72469    210.63882    209.27384    200.79566    203.95469    197.02081    193.64428    191.95478    190.63315    195.82188    187.92937     186.6351    184.50552    183.47981    189.33473    186.24148    182.36791    194.86385    194.46804    191.74493     182.9556     177.7766    181.48167    173.95605    171.50114     171.4308    162.50967    163.22607    162.54646    165.81478    166.93915    162.72076    163.43609    160.62561    168.08546    166.67338    159.27842    158.35819     160.2825    159.55847    159.89369    155.39756    156.14341    159.38506    158.89584    158.61021    155.21373    166.07345    162.55391    170.34285    163.97244    165.92246     161.6068    157.27963     151.3218    153.18013     156.6999    153.47455     148.3063    147.27949    150.45378    164.56318    176.79103    176.84596     181.0511    177.03737    170.11514    170.48895    172.02932    174.81447    170.51225    166.16773    165.21988    160.59285    155.25344    145.56275    143.13812    139.70528    135.28105    141.36001    145.66443    144.26968     149.3241    149.37316     147.9063     149.8195    140.44824    127.36135          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       60.00    320.47572    301.05578    292.64716     280.3592    263.39537     256.6976    244.33025     236.6293    220.48212    210.72348    216.28082    204.21281    200.71556    198.69941    196.18361    196.81164    190.20454     196.0959    189.05157    189.29987    189.54823    179.71477    179.53714    186.36255     182.5847    186.62319     192.9655    189.92823      191.209    186.48213    182.28928    176.87474    173.56031    172.00568    169.53438    164.32459    171.44774    163.65118    164.54737    163.47723    166.64052    163.31843    158.27186    166.24592    163.08224    156.33348     154.5985    162.12793     164.0622    153.27142    154.38649     153.8233    159.99329    161.20018    160.65103    159.98884    171.85886    175.92084    165.50456    163.68311    159.02969     159.3839    153.06787    150.63762    150.40818    150.09143     157.5341    149.87365    155.69753    158.37278    164.01824    170.52687    178.34733    186.53345    183.56308    178.74771    168.96483    169.03623    178.99381    173.53598    168.50418     163.0835    154.67429    153.58669    146.65981    143.00035    143.42801    139.11093    138.15714    142.78446    145.04375    149.23627    153.67655     152.4783    140.35939    134.55359          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       60.50    315.23758    298.58923     295.0465     284.7151    261.18793     256.9884    241.72801    240.92289    221.25589    211.50603    220.26952    203.08601    201.16038    193.45879    195.36123    199.43018    190.15732    201.04954    187.11898    190.66659    189.05285    178.10819    180.14089    180.34269     179.5223    194.59204    190.44893    190.23175    192.30281    186.08259    185.02572    181.98967    175.12407    173.09433    164.22353    168.11553    170.00921     163.9874    172.01099    170.15145    162.54642    167.93352    169.40561    166.83258    162.47959    164.44174    159.59033    157.03527    164.91444    158.48869    154.08267    152.59101    153.88094    157.33704    163.02704     162.1621     166.2517    169.09029    170.27209    170.07768    164.30988     158.1122     155.1986    153.55619    153.97321    154.96331    154.72974    150.97197    151.15815    162.92449    169.13394    170.93732    186.61803    182.14498    181.53074    168.42731     167.1855    167.51313    171.34447    174.90059    166.10281     164.4074    154.99974    149.14573    146.26826    141.87618     139.9703    138.52801    139.52809    139.53224    143.35177    151.05479     154.8603    152.49076    146.33661    126.29949          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       61.00    314.70407    297.31073    292.88701    282.85317    261.32208    252.44563    238.92399    238.59074    225.83678    215.20587    215.80182    205.43672    201.52961    194.51175    193.61474    203.92178     198.3583    196.23943    193.22508    194.58206    191.15886    187.89412    185.07199     188.4393    185.22219    187.36045     184.4213    191.84065    190.24679    181.72954    175.79763    179.58805    176.82298    173.42207    170.45002    164.64774    163.51827    169.06332    168.11161    167.56456    163.59466    168.92178    166.81784    163.73304    159.79466    166.45022    160.44205    158.98315    159.54132    155.41101    157.00456    160.63929    156.05366    158.93184    159.36171    163.95866    164.91692    169.43901    171.50184    168.11043    165.26246    160.62581    154.22013    150.37375    151.58382    153.51842     155.0327     150.8773     153.1347    156.42411    165.06143    180.23573    182.44023    179.27237    178.94954    171.77099    162.26884    163.72299     162.6211    173.89241    172.46006    168.50993    158.80108    154.95043    150.42358    139.80726    136.23896    134.34904    140.47779    133.86227    142.21101    151.43387    150.52437     148.1325    137.70466          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       61.50    312.75369    297.36029    288.67224    281.73464    262.85829    250.72254    237.48575    237.00703    231.19697    222.32463    209.71451    205.98196    201.70216    198.61394    196.52174    205.69196    205.50275    195.23464    194.76682    196.94323     188.2357     190.4502    187.68335    186.57266    185.60192    191.37171    189.37037    192.73881    200.14235    185.86237    179.06021    176.10018    173.49567    179.52034     173.2892    173.44072    163.93168    168.82997    163.56639    170.74128    170.02967    163.75462    162.40723    159.70785    165.37416    164.36788    164.84485    165.69174    157.05001    152.62044    157.06317      158.334    157.23298    154.70174    156.12905    162.18028    161.46283    169.20507    178.01723    173.86547    166.79175    158.90626     157.8339    153.62025    146.33758    150.94004    145.43259    149.15557    154.20602    159.00587     163.6161    174.63296    183.23046    184.37747    179.72004    169.64603    163.51208    161.17397    165.34871    165.93367    174.28299     169.5198     156.7055    154.59737     155.8723    142.96226    136.24165    141.15025    134.11766    141.35009    143.07641    146.59392    151.84217    142.81271     127.3405          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       62.00    309.41088    298.62865    281.22675    279.77738    264.66561    254.25133    243.42369    237.37978    231.03248    224.89937     209.0291    205.16858    203.57593    199.09035    200.11899    197.48492    203.23827    196.32247    194.48519    192.73959    183.13704    193.47731    186.62154    184.36334    184.91127    196.41895    196.26545    191.14808    194.20881    183.68747    179.99938    176.19495     174.5396     176.5292    176.27864     176.4318    172.19653    166.38271    169.88136    167.65518    165.44529    164.46923    165.64908    161.17683    164.42032    166.83015    170.68585    160.49059    159.75717    152.82614    156.25949    161.24274    152.48196    161.27349    159.32159    161.32739    169.25625     169.3562    171.47992    167.25866    172.77176    164.18153    159.18285    152.82693    143.00471     157.8246    152.36724    151.53571    155.84432    161.37054    167.92592    176.68069    181.25755    183.47659    180.19349    169.03686    168.94467    159.81812    160.72029    162.72961    165.15004    162.54761    156.62863    155.99001     152.9351     146.8928    138.63789    138.65497    137.63526     136.8711    140.40962    145.64161    145.79467     137.2335          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       62.50    307.35544     302.0953    277.84353    274.02069    264.98204    262.07023    249.40502    239.97229    228.47288     225.0973    206.89762    202.26863    205.75967    197.87248    199.32684    194.91665      199.894    191.55595    195.77593    188.32113    190.63034    188.86312    184.43419    195.81472    183.15084    186.90597    195.82763    188.65872    185.97155    181.59955    179.82208    181.16297    174.53214    173.26792    173.76874    172.53158    178.12914    172.43122    167.65548    164.34768     166.2296    162.09559    165.89771    163.57278    166.30581    168.10763    165.50166    158.52476    160.99252    164.58044    150.88618    156.12965    155.90707    147.81326    158.33823    156.39766    165.56769    169.14839    169.06205    173.08655    169.56766    160.67539    155.18918    154.78665    152.22498    148.32185    144.54659    148.76584    156.44139     159.2408    168.04182    175.34373    182.52373     185.4794    180.20704      179.742      166.685    166.29978    157.55808    158.85145    157.02794    157.10592    156.32114    150.07647    149.40602    143.81681    144.33477     139.8696      142.229    135.88618    143.22624    145.08957    142.09654    124.12859          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       63.00    309.16832    305.48392    276.34748    268.49679    264.77129    272.19378    251.68418    244.13249    231.00529    227.69124    211.96806    206.30842    203.91603    197.33796    188.99413    196.84617    198.96359    188.41954    190.87687    185.81448    193.75839    189.38337     185.9727    195.19946    185.85301    185.52211       193.47    189.59433    189.94793    189.61812     184.9539    183.47359    171.48877    164.77543    172.72026    174.24667    175.48689    174.72925    167.31542    163.33786    165.77818    165.09594    160.71628    164.17554    156.28027     169.0752    162.00827    164.49853    160.37462    159.60261    161.95563    160.08109    156.55651    153.15758    161.04531    160.18308    168.76758    166.41809    172.08068    175.37316    168.69986    163.63421    156.04769     156.3841    156.16173    157.88093    149.23533    152.61014    152.05396    160.71729    168.37411    176.55348    189.91383    180.59283    186.36804    182.61079    175.81062    166.13923    164.50721    153.29454    161.00826    156.14921    156.13366     148.1545    144.15191    143.69712    145.61288    143.14506    143.42615     138.3416    143.11259    141.67326    130.29864          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       63.50    313.63376    308.69577    277.28841    267.60339    263.81298    268.03126    249.66031    245.33471    235.20961    229.97022    219.61985     211.3991     202.7026    194.99777    188.85798    200.10052    196.07006     193.3039    184.91269    192.53718    191.10012    196.87652    182.73119    192.68846    193.26897    192.27515    191.41615    190.14053    197.33249    182.23149    180.72025    178.85856    181.93047    175.65073    176.86001    177.87778    168.77511    172.20465    176.10576    167.45522    166.96113    163.42397    165.30374    163.43495    166.18182    160.46762    162.96744    163.91747    161.90879    164.99052    164.01292    161.93436    153.15949    153.44281     154.7972    164.10638      166.828     169.3535     172.0282    173.07546    167.92821    158.90952    160.84826    161.80132     157.4876    159.36819    148.96808    152.71563    151.25816    152.56849     166.9873    175.08208    185.22006    186.40548    180.85225    181.39277    168.38684     165.2002    155.33677    152.29331    155.38466    156.69318    151.53127    150.36179    144.38074    144.18449    141.38799    138.11365    139.74211     138.7318    138.69919    140.52183    130.32193          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       64.00    316.45182    307.07607    281.02312    271.38424    264.81328    264.53324     250.0767    242.63687    236.72144    227.33766    221.07532    209.68769      201.436      194.471    196.84145    200.31025    192.07736    194.65464    183.77006    188.19566    190.30852    193.25545    187.25207    188.04304    196.09414    199.55629    192.14533    190.93777    199.15188    183.77202    184.06144     182.0497    179.12713    174.01506    174.20531    176.92107    168.92818    170.49075    175.52688    171.47351    170.84577    162.07641    169.47785    164.23848    172.21638    159.50832    164.99042    161.82646    161.12306    158.29794    161.04226    163.44302     157.8834    161.37031    156.89061    161.99743    173.13462    163.79674    173.24945    170.46084    166.12762    164.35486    161.94719     158.3398    154.54516     157.0513    154.70865    156.18773    151.85327    158.29816    166.16375    174.09357    185.60881    182.39076    179.40152    179.67303    175.14567     163.9293    167.47597    159.74864    160.12697    155.94042    146.02161    151.37641    141.57745    144.13449    146.10667    135.54983    137.86488    137.09115    141.13345    134.06048          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       64.50    317.32734    304.64645     284.4065    274.47859     268.4465     261.4664    252.24366    245.65228    233.88368    220.00343    219.98596    209.28659    202.18002    195.29947    197.23452     194.6306    196.65374    196.02452    186.81841    185.62263    194.65284    191.95109    187.25747     191.7609    193.85722    194.48869    196.34283    188.97074    185.19694    181.33488    185.21887    173.83781    172.03059    174.72588    174.61019    174.67302    181.18386    173.25158    176.38471    171.83769    172.78935    169.70665    172.67277    167.79098    170.43467    169.38549    162.81641    166.20334     160.0566    161.56181    155.72681    162.39654    160.37109    168.86536    163.54469    156.26054    167.02198    173.52088    168.95125    168.55485    165.59738     166.3879    160.50313    157.44776    151.71067    154.49137     151.0212    160.95585    152.76614    157.93518    165.16606    171.15087    184.96395    186.57155    182.31116    178.95625    168.99254     162.0379    159.18577    158.75142    149.84587    155.63366    158.12981    150.63136    144.33831    143.85696    146.38347    138.23806    143.48787    136.80857       135.71    127.78551          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       65.00    316.66955    302.07578    284.39699    277.49052    267.30074    257.43333    255.16994    249.58543    231.75874    216.06455    221.15711    214.12376    205.81765    195.72441    195.64837    192.98474    199.98334    197.07074    192.04264     192.6494     193.2987    187.30713    190.05133     194.0182    193.62726    192.08299    197.63129    194.88772    186.88344     180.9473    187.35959    165.16962     168.8517    173.59541    177.81423    168.06692    171.31246    171.86588    172.90725    171.08136    172.08377    176.39512    168.83482    164.29163    163.49737    165.08306    163.09261    160.87876    165.59573    161.22036    159.79581    167.23396    156.84078    160.39809    165.40845    168.25193    170.81212    169.64887    169.35809      172.973    165.45772    166.46248    167.00992    153.04533    158.26813    155.27203    155.01292     157.6403     154.0659    164.21099     169.7756    176.71027    176.70721    184.94484    177.04401    176.96231    169.15828    165.89257    161.07618    157.31185    160.56126     156.8605    152.41148    151.96813    148.02644    144.53198    138.41963    135.58201    136.74523    137.53653    134.27631          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       65.50    315.10919    303.32728    283.09668    280.17408    262.86401    253.93027    256.68007    249.86638    235.87419    223.06475    223.19856    217.53899    205.44881    201.32727    198.53069    195.77227    199.84033    198.04668    193.00786     194.6838    190.11941    192.36151       183.71    184.54111    187.86136    187.60344    196.46293    198.35956    187.04354    179.19835    173.73765    173.02318    175.73547    173.94305    181.66538    175.99305    172.01237    174.01839    175.79447    170.27052     175.8782    172.64089    170.34431    170.75618    163.92281    167.13142     171.1266    164.84571    162.55491    162.28368    159.29486    157.54624    156.09104    161.39497    166.28236     166.8149    171.38487    170.31068    168.52395     171.8434    170.14685    160.35575    163.24342    157.67789    154.58533     158.5505    152.35196    158.29327    161.86775    156.78124    174.67097    181.38993    181.42525     182.8371    179.01679    177.06101    168.71257    167.59857    162.18665    153.85048    154.43266    154.28738    156.43825     154.1856    148.78162    145.88662    140.50082    136.63268    136.85968    132.95105    120.07499          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       66.00      314.712    304.43193    284.54646    282.09464    262.21053    257.56101    252.62782    247.13709    242.61632    228.03501    221.51846    215.06059    205.09417    214.97854    199.61036    198.01121    196.09364    198.22606    192.53505    197.95313    187.77315    195.69264    182.73121    186.30213    189.66888    201.36963    191.69788    197.66033    193.68757     186.8699    170.77667    176.15833    175.51564    176.58098    172.73174    178.80257    175.34637    173.90709    174.81285    174.87178    173.47097    177.79586    175.80094    173.10245    163.83059    164.59937     167.3219    164.03268     163.9426    164.23083    156.66382    161.91331    163.22695    161.89127    166.35319    161.95695    170.03468    173.92209    175.73056    159.86861    172.16323    166.04799    158.14397    159.33659    159.84589     151.8692    153.17554     155.2643    162.69446    165.26085    165.96967    174.41875    185.00668    183.48678    184.44345    177.47206    169.03147    160.97205    160.30842    157.78888    156.50116    154.92466    157.57335    153.11755     144.3008    147.84955    140.25083    146.94356    136.45272    128.94482          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       66.50    315.80086    304.65808    288.16841    280.68955    264.84559     261.1515    247.19625    243.50573    242.52946     230.4694    219.37101    213.71271    208.18714    217.07988    197.65611    198.59341    196.34909    199.96423    191.06877    196.45442    192.08092    197.56039    185.48949    190.06974     190.7402    197.86246    195.08524    205.31991    198.37988    191.20417    181.43144     181.2911     180.4198     176.3069    176.45374     178.4146    173.61674    168.54647    175.40867    178.17391    175.54224    174.64948    176.63385    169.71244    167.51998    166.34136    166.03399    165.00622    168.15971    161.75336    153.95685    167.10418    161.00195    161.73772    160.98334    166.11629    161.49489     178.3958    180.73048    176.69855    167.92344    159.13947    159.91571    159.38929    154.02689    157.38203    154.10938    155.19205    160.84746    162.16644    170.78458    183.83554    183.71452    187.56782    187.13842    179.56924    172.85821    165.03295       156.55    154.01532    152.68729    152.44891    150.71631    151.31744     144.4882    146.78784    139.56359    137.09299    131.53698    119.84925          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       67.00    317.21083    302.56369    290.14746    279.87486    267.14274    261.46505    248.22914    240.26863    236.20169     230.0177    221.44274    216.39112    209.77843     209.3899      197.203    204.06814    199.01138    199.79203    189.18033    193.52069    193.87568     195.5781    180.62875    179.62484    190.27862    194.31394    191.16817    199.58092    190.65595    189.95687    188.82358    182.17402    178.55617     178.4577    170.10143    174.36336    171.76357    170.71695     171.5026    174.81538    172.67725    168.62679    174.25197    169.64252    170.55763    173.96251    168.26218    161.54862    164.27543    164.21177    164.98423    160.86294     160.7039    164.85812    161.17072    159.92169    179.91382    169.51367    176.90206    173.77425    169.25622    165.51169    158.56291     160.4543    157.56621     153.2885    152.76883    153.31232    155.42322    162.02615    172.85253     180.0741     184.1292    190.88384    195.94288      181.665    174.47961    167.37572    164.85946    154.28892    152.22338    151.01006    151.77566    148.65178    147.02644    148.86429    139.97195    137.06745    138.76386          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       67.50    319.59981    301.25035    289.73456    279.57633       269.11    259.25078    250.08023    237.85949    232.73385    229.12883     224.8178    216.91011    210.87497    209.57389    203.18882    213.99143    199.05449    197.70386    189.07676    195.00835    192.82492     192.6545     184.8297    186.47955    191.71821    184.33574    187.16142    195.66641    191.39429    189.88994    185.24815    183.68277    177.74485    178.24877    166.60317    170.28677    172.12244    172.91691    174.15988    167.11005    176.15569    171.30788    174.98321    161.22693    172.49699     166.4931     166.9445    163.09231    166.66055    159.60703    160.97861    160.29746    160.19732    161.73178    160.07759     167.4208    167.74767    173.00788    178.96654    174.31876    176.69702     173.0613    160.56503    161.12929    158.64687    145.81991    155.22987    156.80622    156.56452    159.66213     169.3415    176.34852    192.18067    190.53052    188.43188    183.57354       175.33    164.12035    165.91721    160.81474    153.82153    155.51842     149.9817    148.94205    146.40052    144.72667    141.88079    135.20482    124.63597          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       68.00    322.02594    299.86915    288.73196    278.91797    271.67762    258.84741    249.88605    240.60059    233.57374    225.59341    223.51392     217.9763     211.7085    208.63817    206.17588    212.85576    202.29659    189.21573    192.45809    193.12044    194.42907     191.8213    190.63892    188.12958    191.89498    191.74458    195.70946    192.08835    193.76122    192.93715    186.90384    183.42995    180.37422    175.90965    171.63906    169.50516     177.1975    171.85877    172.66284    168.86143    180.27524    175.95293     167.4679    164.66729    168.31919    167.59333    168.42695    169.76449    171.98976    167.90245    160.66186    163.54154    156.57823    165.00848    162.97611    167.99136    170.70392    174.65368     174.6415    175.50059    175.52236    172.99744    159.48716    156.90879    159.63238    154.65014    157.41063    152.17724    151.27766    164.32205    165.28388    179.21076    186.77491    194.83868    186.71296    185.76163    173.25443    174.98308    162.64749     156.6892    161.33138    149.29874    146.88825    151.42632    146.07512    143.55619     138.2337    133.67559          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       68.50    324.72979    303.10263    287.93281    280.48845    272.91655    258.96853    247.65597    241.37157    233.07048    225.60621    220.63377    216.71836    212.19849    210.71385    213.42874    211.77029    205.58982    196.16863    190.50796    186.54358    193.83509    194.63963    198.71664    180.51268     187.9631     195.1482    195.31217    188.30559    193.70534     185.9026    188.08209    175.56557     181.1747    175.73305    179.36988    182.98606    181.55846    171.99635    171.75075    174.92669    181.13876    167.95522    171.61908    173.65198    167.77263     169.7698    167.46315    171.15619    171.10218    164.35419    164.31626    161.32608    163.11612    163.87626    160.27937    167.12695    163.24554    174.80499    175.83735    169.79556    181.98997    173.52096    163.02411      158.333    158.73583    151.59429    159.51717    156.26782    157.92871    163.43955    167.02457    178.77717    185.73935    186.52132    187.67941    183.02515    176.10741    165.06249    160.08455    153.86852    154.57433    162.94268    154.38805    155.33955    144.31036    140.19713    137.03083    122.95588          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       69.00    329.66753    306.49986    287.28099    283.03435    271.33006    256.11363    245.71061    243.88073    232.06296    231.74085    222.52391    211.21716    212.50735    217.49212    213.17475    205.72902    206.83059      206.433    194.85735     186.8844    187.74709    194.31147    196.65389    184.52815    191.36271    191.81109    197.63381    184.65261    192.16016    181.36784    186.64291    193.74185    181.80806     177.7492    179.02318    177.88796    178.49349    172.43118    169.16985    172.47399    174.71853     168.1073    169.23249     169.1733    168.64889    169.95034    170.56959    164.48137    166.33356    168.20649    169.56467    170.36211    165.39849    164.45457    161.48752    162.46508    170.37232    167.37616    173.40336    174.02225    169.20985    170.47764    163.71373    158.74455    165.24015    161.05608    158.09769    158.93183    162.43299    162.95595    172.88378    183.30582    190.23989    186.13334    193.21208    188.38377    176.24064    170.23178    161.11764    160.17559    158.00575    154.16994    154.66313    156.57145    146.69179    139.94517    137.22852    123.49974          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       69.50    335.88931    309.58427    288.12205    282.63743    269.64002    254.33672     245.6475    249.31474    233.19561    232.71773    226.60032    206.91512    211.86362    213.68715    200.98333    200.86507    206.64825    206.93431    204.86425    188.72631    183.51067     189.3939    190.88167    187.39961    192.11574    195.29049    199.09783    191.17094    192.85199    184.06119    187.45516    182.54472    176.33676    181.52048    181.61114    175.46683    179.43538    177.02878     172.8875    173.43324    177.65591    167.28602    161.88313    161.06671    168.21267    169.01749    167.26832    168.62798    168.45239    165.95416    171.42065    162.29746    170.21388    166.44129     165.8295    162.23126    169.68523    174.27743    176.10163     174.8605    175.30074    171.26495    169.80204    163.13085     163.1996    157.68405    158.93068    159.30387     161.3277    168.93167    172.30749    180.01253    182.68134    191.39574    189.39416    180.28977    179.37006    168.04788    163.83955    161.46562    150.70508    153.91378     154.9807    152.96039     145.3411    141.43195     138.8065          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       70.00    342.46649    311.54906    290.29532    279.54596    268.39907     255.0761     246.9687    246.12498    233.84161    230.09687    226.36149    210.41089    207.84955    210.05705    200.30269    206.75948     207.4055    206.91044    202.71188      201.316    183.51166    187.16079    190.10962    186.45761    193.44465    186.90119    189.25299    198.54298    194.00984    191.08765    192.33703    180.06863    182.90762    183.16816    177.93204    179.74193    176.12718    170.37474    176.86076    172.40905    164.40541     173.3742    170.22413    169.75782    169.67747    170.78959    174.11132    170.64539     167.0814    164.99262    168.30735    170.75307    167.61238    171.46398    165.27656     165.9168    173.51509    169.34733    178.27484    175.60807    180.98437    168.99163    169.53543    159.86973    156.60787    157.45819    156.82549    157.38871    166.46058    168.26639    174.28111    177.11868    192.16351    192.67913    187.76718    183.95661     176.8689    174.71287     162.9784    152.43937    155.98696    160.35695    151.27932    146.34858    147.70984    147.90921    131.83419          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       70.50    336.25498    313.62914     293.7052    277.35917    267.67702    256.40905    249.59927    240.29582     235.7125    225.43198    220.10561    215.11246    206.10594    209.19649      202.986    209.80492    205.71258    208.92313     202.6355    200.13287    196.51932    185.53079     188.7474    184.34822    189.70257    185.40047    190.40972     198.9043    192.33421    186.53101    191.95703    184.50824    182.77985    173.50329    172.35536    179.14616    180.62565    181.43037    172.71943    174.84837    168.27879    170.51252    172.11385    162.56534    175.53116    167.53709    169.92096    166.69913    158.48987    173.70624    172.45316    166.06254    166.98441    169.10857    170.39577    166.26325    176.76169    181.44603    178.85059    175.50746    168.34175    173.23792    169.07042    158.37655    159.96063    162.08316    162.72706    166.03749    166.67589    161.37669    179.95526     179.6992    186.33525    192.08528    200.55886    183.59001     181.2958    163.96567    163.85715    150.18804    155.36094    155.56234    148.08237    148.92462    144.96133    142.21356     122.7819          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       71.00    330.57108    315.49383    301.54195    276.05367    267.63875    256.01547     252.4674    239.50687    233.52576    224.01835    217.87248    215.85435    206.31142    209.08073    206.99252    206.73511    203.66606    206.78748    203.09526    200.40521    200.14414    202.86901    186.93876    184.74276     188.8237    186.80395    189.35025    197.05451    190.83948    185.49793     187.1014    179.94443    177.23749    187.78838    170.96176    178.02692    180.82444    183.83318    175.39151    171.30096    177.82405    169.75074    174.91442    173.56433    175.40523    168.50512    168.69725    168.73993    167.15343    164.65493    166.42245    165.94143    169.87042    163.01027    171.06893    165.63683    177.46053    174.97841    185.75955    184.66323     176.2152    172.70412    164.41768    160.18832     162.6887    157.40351     164.4725    159.16961    159.52742    157.00552    169.38124    181.56401    193.22765    190.54465     191.5128    188.49286    181.36872    164.47673    159.00833    158.61706    152.38807    153.47797    145.82634    144.21743    147.45801    135.79307          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       71.50    326.10151    312.82337    307.99011    277.13223    269.53052     257.4881     255.9267    240.89554    234.45511    225.06179    220.81039    215.35155    212.26236    209.36802    210.48695     204.8689     210.9218     200.4891    204.10678    197.58191    199.04188    194.86834    188.94731    190.39365    189.57891    184.32726    187.09963    195.91933    185.64274    185.06028    189.63746    177.47523    175.17654    187.78403    177.05538    180.65882    172.81901    184.82286    181.57931    175.74049     179.8192    172.70916    175.60687    171.57331     173.3258    166.49017    165.54142    166.26561    171.60176    167.99453    165.73717    164.38319    169.25384    169.29406    169.21649    171.25069    166.59056    179.89676    178.89553    178.10526    174.08389    175.50334    170.59271    160.51647    159.95439     161.5899    161.65153    164.16341    161.49659    165.89786    175.99907    176.98043    189.05913    195.79966    193.25463    186.78058    179.15567    168.81377    169.84471    159.84109    158.71751    157.85838    148.52447    144.44091    138.61234    134.04218          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       72.00    325.57505    310.55683     311.0904     281.9215    271.66909    257.11825    260.01321    240.49479    234.85168    226.23534    221.55282    216.06473    217.42328    208.99102    205.71998    204.89488    213.96853    193.16978    207.98234    198.48863    193.90929    191.19117     191.7465     196.1616     192.8477    197.97536    191.57853     189.5748    181.92968    179.98429    189.20103    176.36335    180.85094    181.97213    188.21243    180.90168    175.44782    178.66962    182.24952    181.54529    183.95432     171.0082    177.70305    179.19386    172.27374    170.15337    166.90868    170.28951    168.73134    167.11111    173.17534     160.3147    173.62886    170.56057    162.21347    171.05567    167.87031    176.08085    181.83253     184.8586    179.45609    179.14142    173.81038    168.02319    167.15172    162.07155    154.58539    153.93248    162.58482    172.19824    176.90841    184.00188    192.54517    191.34163    192.36768    183.56812    182.43111    167.53295    165.09805    160.37967    156.80332    156.73482    150.95829    148.88566    143.46466    132.71779          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       72.50    325.26956    309.98275    305.55085    285.60027     274.8828    254.09745    261.92293     240.3086    230.74734    225.88098    217.52172    215.78138    213.50873    207.91206    204.45266    206.46813    215.41058    196.46728    206.58557    201.81177    192.86772    201.60482    195.26604    199.23024    202.37403    201.87693    191.42527    188.49236    184.25534    191.27094    184.56622    188.89326    175.81604    181.60126    180.96785    174.70362    179.54422    185.98904    181.43222     176.7359    179.57672    170.51127    174.43014    178.15408    174.67056    178.61314    173.20592    166.61817    160.66868    168.74363     171.0685    170.43716     170.6709    168.11206    172.35268    178.59034    174.17034    184.27764    187.84025    181.45964    181.31984    180.10378    174.03056    170.25399    167.04643    164.43128    166.31113    172.39858    158.67121    162.75453    165.94749    185.90237    189.46054    193.95775    193.75712    186.67751    176.26807    173.79631    165.08176    161.07483    156.14952    158.25131    156.30698    148.98275     138.7012          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       73.00    325.28511    313.37468    299.99554    286.78612    278.13257    253.49885    256.62572    240.38822      231.925    223.55542     214.4069    214.98113    211.34345    209.12027    207.84208    204.01123    206.66819    200.91739    199.89789    198.64495    193.85365    202.89773    196.52348    195.96909     193.4723    203.49229    193.98268     191.6687    189.08355    186.21422    186.60129    187.10714    178.99086    178.81312    180.67466    179.30114    182.93063    170.64946    180.25924    177.67482     177.4612    175.88816    176.11155    170.20195    177.35191    178.56991    175.43143    173.43275     168.5372    164.65859    161.40681    172.65755    171.64944    165.34376     170.7267    168.56999    172.02162    178.21708    186.67824    183.85726     183.2137     179.1262    167.32813    171.03761    169.81292    163.60054    166.51347    164.51508    167.94608    165.08678    170.11292    179.68288    192.95247     191.8768    196.03782      190.603    178.01084    166.10924    163.17829     158.4503    156.49025    152.03472     154.2943    149.39566    138.84862          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       73.50    328.57268    315.39118    298.04806    288.09233     280.8258    263.16805     251.7704    237.36164    236.61161    224.90528    222.72173     215.2023    214.36067    207.87555    209.24074     204.1057    199.81213    204.22142    190.10522    201.32474    194.65316    199.58765    194.70361    189.29743    189.44998    197.04618    198.29479     203.5372    191.94275    183.27187    188.68759    179.36512    173.08599    178.48146    184.43953     179.8779    177.28527    166.64473    181.03297    173.75048    174.62867    176.08493     178.6761    171.96856    172.80825    174.39208    171.96317    170.77028    173.07058     170.2792    167.61991    169.43792    150.41941    170.09209    169.78586    160.34868    169.88168    175.65466    179.15254    180.58267    180.66946    178.14848     172.9352    169.99828    164.23173    163.02997    159.37224    165.89396    165.89391    172.99054    179.13986    184.16625    189.83395    193.33832    187.47738    182.87105    183.17969    174.00927    167.73691    157.38425    164.60407    160.33756    154.83327    147.90342    134.85799          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       74.00    333.26914    315.97559      299.587    290.74226    281.38214    273.07944    254.23633    234.80003    237.46935    227.02651     229.5985    218.60551    215.85215    206.95942    210.55909    206.74806    199.13955    204.65769    194.42181    195.92213    194.68393    199.70632    197.23984    199.79716    194.83101    200.32865    199.26413    204.96299    198.09746    185.41511     184.0049    179.63306     188.7977    178.67722     181.7969    176.15889     176.5805    182.27388     176.2612    178.78494    177.72333    176.95151     182.6842    173.36352     179.5546    175.13441    176.61602    173.54082    169.70892    164.63476    173.07724     168.9251    171.44963    165.98953    163.46474    167.50073    181.87217    171.54898    184.44926    181.06437     177.8439    175.39385     176.9895    173.07214    161.69991    165.78617    172.21089    163.92166    169.04156    166.38219    178.69968    183.35023    200.07786     201.2807     198.1432    184.50912    182.95903    169.22065    162.06823    159.10404    158.05138    154.60656    156.29208    148.26193          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       74.50    338.70246    316.38108     301.9544    295.17335    281.89866    275.88084    259.64638    240.22603     235.9133    226.01416    228.32213    220.10135    217.41302    208.68921    205.57933    208.90142    199.78853    204.67817    194.08616    194.08849    196.12636    198.56899    204.04576     202.8681    201.30132    197.95814    196.66281    201.88136    199.48531     190.9257     187.4896    190.80315    178.55657    182.19197    181.43966    178.37919    182.76292    173.84635    172.36923    177.39342    175.75263    175.11969    174.14886    175.40758    176.58894    173.89542    168.08594    174.45185    167.41794    163.85395    171.07831    175.56849    174.91461     171.0172    166.18026    173.43303    176.58154    177.36014    177.79732       178.25    177.24677    179.05441    169.02236    168.43629    172.69887    164.98218    158.64762    168.17065    167.61079    166.60389     174.7811    186.50758    191.43772    204.32374    200.46832    198.50338    186.36694    177.34738    164.34274     161.3007    159.05162    153.93197    161.22313    141.41204          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       75.00    333.78545     317.3767    301.89615    298.59246    283.83693    274.66946    261.49663    250.96076    236.28721    224.61476     227.2543    218.15595    217.74747    209.23648    203.85994    200.50573    201.42914    209.10828    194.15594    197.90865    196.54974    196.59097    195.52087    203.65122    200.22913    196.83935    198.43019    204.03324     198.3315    192.91009    193.66391    194.73898    178.65616    179.14996    181.83948    192.49449     178.6206    186.48778    175.67335    175.64484    175.52238    176.83643    175.46375    170.61317    175.19122    177.03869    174.58888    176.17746    177.47917    172.75375    168.97001    166.34645    172.02904     171.6561    175.07216    171.62172    170.31196     174.7871    183.42601    176.25049    178.06443    180.33462    173.05149    169.60654    171.96045    168.13926    167.56828    164.40271    158.60167    171.61131    171.55305    190.41411    190.96625    202.15075    206.36759    194.37581    189.66069      180.684    169.86892    164.25696    161.62863    157.35729    152.44086    143.46587          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       75.50    328.11906    313.65111    301.82422    297.87875     286.8551    270.71882    257.27674    258.98419    239.01182    226.49185    225.15301    215.65318    210.69202    208.19025    203.27943    196.72548    201.69975    211.84065    199.55904    200.77722    193.85607     194.3265    190.29106    206.56322    193.18676     203.8202    200.51279    207.35339    193.34834    197.50092    196.00496    189.85798    189.25727    191.25649    183.35106    182.30529    176.64894     180.7536    172.79219    175.53002    177.20598    181.00745    175.18349    178.23891    177.95307    178.69343    169.55556    172.52715    175.76305    172.19649    171.09787    176.57162    166.90918    170.50775    175.99427    175.95311    183.45365    180.57411    179.67251    179.76191    180.55277    172.95094     165.6119    172.30873    164.49971     165.6466    165.53215     167.0259    168.05673    166.69863    170.17847    186.20037    197.05293    200.05487     199.9118    201.17638    190.23308    182.47925    170.09087    164.95285     164.8606    161.18375    149.54412     133.2902          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       76.00    323.77861    306.67711    301.61279    293.58531    288.54629    265.84026    253.07787    262.67774    241.04411    230.82958    224.55003    214.19104    207.33612    208.74829    202.52562    202.73331    201.44893    208.42023    200.12675    193.50969    194.36476    203.04996    195.39666    193.84797    194.49833    207.37543    206.45033    197.95039    193.22965    192.55832     191.8283     198.2491    193.34457    184.51565    187.60699    185.36258    185.16899    176.85083      176.583    173.03039    180.00967    172.59166    172.31325     174.5377    174.13591    181.31474    178.11056    170.17695    168.37281    175.96904     176.5929    175.11566    168.19435    172.74459    169.45378    168.94776    182.19902    182.81726    181.26034    186.18015    181.63563    183.70858    175.54672    165.85829    167.99977    169.02106    168.89437     165.7013     163.7729    169.71056    176.77304    189.85935    194.96806    200.88042    201.28873    206.72112    194.22669    179.55789    169.74947    160.61815    165.65655    165.77868    153.68714          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       76.50    326.16089    300.19385    301.00073    289.79928    284.63442     262.5525    257.24762    265.22253    246.16053    232.16866    232.56293     215.8216    212.32249    206.30193    201.96681     207.2531    200.82637    205.34611    196.37268    196.81376    203.50994    201.86063    200.65909     194.7865    205.75573    201.25141    206.82076    193.68115    198.65137    197.81005    189.01254    197.08523    193.92799    176.37078    186.28375    178.24091     181.8551    185.18017    182.79228    180.97646    176.67998    174.90284    177.76143    174.53209     177.3929    168.14194    174.21954    176.32363    169.67481    176.99322    169.46723    178.04157    167.08721    172.60485    171.86207    173.99328    174.63799    176.06162    174.45115    183.56647    182.56278    171.62524    174.95039    169.13411    167.68371    162.62763    167.30684    162.18397    162.19077    173.46295    172.10712    187.64224    195.23711    196.74518     200.0055    190.10236    197.49625    179.77751    178.79161    167.88609      162.128     165.9172    150.02298          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       77.00    328.43004    300.15265    300.72602    293.61649      280.626    265.50122    260.97327    260.98913    250.61654    229.43602    239.95617    218.25438    219.49931    200.14092    202.39353    207.15453     201.2996      201.725    194.55666    201.97947    197.10821    196.40459    192.55775    197.82811    205.54689    195.81464    198.11351    196.18727    198.37549    198.58327     190.2682    194.59667    185.93672    182.40361     189.5669    191.94446    175.09063    185.38027    177.69495    183.94421    171.77547    177.30703    176.02925    173.01072    174.71382    177.81698    170.94166    176.39707    174.14426     172.8607    168.43122     172.6174    174.40115    174.24064    174.54025    172.69718    176.28643    185.36719    179.85012    179.32406    187.78655    180.58906    180.57037    172.89866    169.60694    162.49171    170.83739    164.68554    159.67694    170.31329    167.00243    182.71979    188.95628     194.5813    193.63764    194.26284    198.60032    180.78738     169.5553    167.45218    172.37364    161.46251    138.27083          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       77.50    329.89905    301.76582    300.11916     298.3189    278.66692    269.18746    263.68315    255.50241    243.45256    230.06159    244.43346    224.44347    227.45937    207.73916    205.20428    210.36529    203.10549    196.72062    194.77289    194.45536    192.15591    199.53921    195.39404    194.74668    199.15199    192.76899    199.13106    199.61734    196.60308    197.97557    190.63194    189.90374    188.83531    186.67504    180.96751    189.82586    185.36485    187.15151    183.78393    189.41988     179.4292    175.96345    184.12843       171.03    171.58805    176.69352    177.85864    173.54541    168.81926    172.13146    177.92929    171.21687    176.04315    168.89268    178.14528    174.47498    187.90585    186.34145    181.79126    184.53434    188.17192    181.67266    178.88904    182.11768    167.97172    163.49041    167.96211    173.94529    171.89914    171.40489    179.95838    178.58258    200.55002    197.42758     194.7326    195.56624    192.40831    182.09005    168.42812    173.42353    165.05621    158.13497    141.53445          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       78.00    327.77384    302.75861    298.64001    298.95262    278.30909    271.19903    266.37253    252.61222    238.10903     233.9243    243.62814    234.78331    226.73626    224.50456    218.90047    216.19555    202.49681    195.62442    203.32789    189.98806    198.00096    196.86491    199.04381    187.59998    192.56803    191.63962      205.624    198.83681     204.9318    193.60948    193.51743    191.80003    183.90922    185.99871    189.01855    185.12244    188.29014    184.61091    181.66023    182.02479    181.57117    189.12952    184.89248    189.76183     183.1826    175.69824    176.13006    181.17874    174.82995    170.67874    168.01898    171.52669    173.94407    171.90944      173.358    172.90322     170.9177    172.44372     186.6348     176.1531    180.96207    179.76162    177.68553    175.88156    171.01515    170.71142     170.3901    167.84634    170.44192    177.73024    176.66551    187.05153    202.10645    199.05697    194.59588    193.52413    187.32873    188.23967    173.02107    166.83298    165.63805    152.61568    138.50925          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       78.50    325.68766    305.57293     296.7996    291.69738    278.73739    271.87199    264.13823    252.18682    243.59944    238.50725    238.00757    234.57606    223.42111    218.34851    227.42735    219.44744     206.8563    205.84033    205.06703    191.71119    193.48006    192.73245    187.87642    188.61478    193.17748     196.2836     210.4643    194.42482    204.65261    181.47271    185.93276    195.54188    182.71816    183.79549    185.08865    186.45971    192.76136    179.87162    179.33925    185.25565    177.90164    177.31509    182.76487    181.16271    191.21841    181.68058    179.54119    179.79565    173.55719    167.93588     170.8511    170.03427    176.01439    171.45801    180.19784    172.11134    174.36652    181.88928    183.03019    185.18631    183.09103     177.9672    173.44593    176.24937    173.70336    174.74524    173.98814    166.29396    171.96641    177.17629    181.05997    190.67122    195.89686    206.27286    204.19561    204.51612    196.67475     190.3921    172.15192    165.05511    166.28717    156.25845          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       79.00    323.28084    309.71782    298.45795     283.7686     279.1864    271.06153    259.39519    250.47419    246.39479    247.69577    236.90731    228.64436    226.83259    210.87964    220.59092     221.1441    214.79167    211.04192    204.02496    200.40516    197.80693    194.04798    191.11861    194.02037    192.93583    201.45217    202.07228    193.17347    193.71436    199.23427    182.38986    191.06713    182.04056    189.94442    190.86168    184.53499    180.66761    188.77043    185.67426    181.71099    178.30412    183.73938    184.73415    179.53619    181.09677    180.61793    177.48508     180.3677      181.429    173.92565    177.68362    175.81218    171.06376    173.26394    167.92678    175.31265    183.05941    174.41947     178.4851    187.53202    176.05575    180.48597    178.14619    175.03782    173.99945    171.38797    171.73038    177.27858    175.76584    169.75444    180.69121    186.01385    193.41703    203.17117    202.77978    202.29362    194.47905    184.37455    176.66559    170.84205    169.77881    155.70757          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       79.50    319.37548    313.80743    300.77647    284.82934    279.90879     268.6283    255.91752     246.6061    242.86224     251.3163    239.36596    229.44858    231.01638    218.98385     219.1597     223.9462     215.7954    213.81729    205.04399    202.81683    205.49716    203.33624    194.37607    191.33617    189.57188    201.20265    192.47985    194.89947     195.7964    195.09822    187.85151     182.3021    189.27576    189.47903    191.18782    181.53422    172.18591    205.78854    187.17018    188.64182    183.33067    181.61219    191.49827    183.45024    172.58369     180.6002    178.96375    173.48583     181.8529    188.23274    175.62936    178.68158    180.18875      174.257    170.52761    176.46881    173.06991    177.40237    177.59364    189.03177    179.17539    180.74777    180.09924     176.6545    170.79689    167.37752    172.38537    168.67482    167.79192     167.5967    186.11422    187.37193    195.60734    196.38441    197.27556    206.43464    197.43434    189.34743    177.05677    173.87478    171.17595    152.54078          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       80.00    314.89655    311.93284    302.23633    288.96778    280.63817      266.383    256.06991    247.37919    241.99261    242.81396    234.71539    233.14403    223.94949    221.61584    222.29351    219.09454    218.90914    216.95356     206.6919      203.339    204.49564     204.2015    188.68998    193.82666    191.33261    191.85556    191.22701    194.88224    195.69085    192.21096    192.82049    188.66766     191.6543    189.69636    183.16058    188.26607    182.91685    190.79336    188.52857    181.67729    187.36786    183.28625    185.36939    182.66168    181.35918    177.79748     180.8068    175.74051    178.19937    176.75559    183.89576    181.11285    172.31092     177.5136    168.42765    178.26887    183.30864    185.88452    189.94301     190.9711     183.4484    182.33483    180.17363    179.39993     176.2402    177.15156    171.94513    170.21035    175.51771    167.71268    172.67298    182.76808     199.9072    208.69916    205.10645    198.59024    194.57679    183.17698    176.32628    168.93547    162.25132    150.94693          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
;
    """
    return s_out

def template_for_pd2dt():
    s_out = """

data_powder2d
_setup_wavelength     0.840
_setup_field          1.000
_setup_offset_2theta -0.385404
_setup_offset_phi     0.000

_chi2_sum  True
_chi2_diff False
_chi2_up   False
_chi2_down False

_texture_g_1 0.1239
_texture_g_2 0.94211
_texture_h_ax -0.66119
_texture_k_ax -0.0541
_texture_l_ax 3.0613

_range_2theta_min     4.000
_range_2theta_max    80.000
_range_phi_min       -2.000
_range_phi_max       40.000

_pd2d_background_2theta_phi_intensity
;
     2   4.50        40.00        80.00      
 -3.00 -350.0        -369(1)     -400.0      
 41.00 -350.0        -404(2)     -400.0  
;

loop_
_exclude_2theta_min
_exclude_2theta_max
_exclude_phi_min
_exclude_phi_max
0.0 1.0 -5.0 -4.0

_pd2d_instr_reflex_asymmetry_p1 0.0
_pd2d_instr_reflex_asymmetry_p2 0.0
_pd2d_instr_reflex_asymmetry_p3 0.0
_pd2d_instr_reflex_asymmetry_p4 0.0

_diffrn_radiation_polarization 1.0
_diffrn_radiation_efficiency   1.0

_pd2d_instr_resolution_u 16.9776
_pd2d_instr_resolution_v -2.8357
_pd2d_instr_resolution_w  0.5763
_pd2d_instr_resolution_x  0.0
_pd2d_instr_resolution_y  0.0

loop_
_phase_label
_phase_scale
_phase_igsize
Fe3O4 0.02381 0.0

_pd2d_meas_2theta_phi_intensity_up
;
         241   4.00         4.20         4.40         4.60         4.80         5.00         5.20         5.40         5.60         5.80         6.00         6.20         6.40         6.60         6.80         7.00         7.20         7.40         7.60         7.80         8.00         8.20         8.40         8.60         8.80         9.00         9.20         9.40         9.60         9.80        10.00        10.20        10.40        10.60        10.80        11.00        11.20        11.40        11.60        11.80        12.00        12.20        12.40        12.60        12.80        13.00        13.20        13.40        13.60        13.80        14.00        14.20        14.40        14.60        14.80        15.00        15.20        15.40        15.60        15.80        16.00        16.20        16.40        16.60        16.80        17.00        17.20        17.40        17.60        17.80        18.00        18.20        18.40        18.60        18.80        19.00        19.20        19.40        19.60        19.80        20.00        20.20        20.40        20.60        20.80        21.00        21.20        21.40        21.60        21.80        22.00        22.20        22.40        22.60        22.80        23.00        23.20        23.40        23.60        23.80        24.00        24.20        24.40        24.60        24.80        25.00        25.20        25.40        25.60        25.80        26.00        26.20        26.40        26.60        26.80        27.00        27.20        27.40        27.60        27.80        28.00        28.20        28.40        28.60        28.80        29.00        29.20        29.40        29.60        29.80        30.00        30.20        30.40        30.60        30.80        31.00        31.20        31.40        31.60        31.80        32.00        32.20        32.40        32.60        32.80        33.00        33.20        33.40        33.60        33.80        34.00        34.20        34.40        34.60        34.80        35.00        35.20        35.40        35.60        35.80        36.00        36.20        36.40        36.60        36.80        37.00        37.20        37.40        37.60        37.80        38.00        38.20        38.40        38.60        38.80        39.00        39.20        39.40        39.60        39.80        40.00        40.20        40.40        40.60        40.80        41.00        41.20        41.40        41.60        41.80        42.00        42.20        42.40        42.60        42.80        43.00        43.20        43.40        43.60        43.80        44.00        44.20        44.40        44.60        44.80        45.00        45.20        45.40        45.60        45.80        46.00        46.20        46.40        46.60        46.80        47.00        47.20        47.40        47.60        47.80        48.00        48.20        48.40        48.60        48.80        49.00        49.20        49.40        49.60        49.80        50.00        50.20        50.40        50.60        50.80        51.00        51.20        51.40        51.60        51.80        52.00        52.20        52.40        52.60        52.80        53.00        53.20        53.40        53.60        53.80        54.00        54.20        54.40        54.60        54.80        55.00        55.20        55.40        55.60        55.80        56.00        56.20        56.40        56.60        56.80        57.00        57.20        57.40        57.60        57.80        58.00        58.20        58.40        58.60        58.80        59.00        59.20        59.40        59.60        59.80        60.00        60.20        60.40        60.60        60.80        61.00        61.20        61.40        61.60        61.80        62.00        62.20        62.40        62.60        62.80        63.00        63.20        63.40        63.60        63.80        64.00        64.20        64.40        64.60        64.80        65.00        65.20        65.40        65.60        65.80        66.00        66.20        66.40        66.60        66.80        67.00        67.20        67.40        67.60        67.80        68.00        68.20        68.40        68.60        68.80        69.00        69.20        69.40        69.60        69.80        70.00        70.20        70.40        70.60        70.80        71.00        71.20        71.40        71.60        71.80        72.00        72.20        72.40        72.60        72.80        73.00        73.20        73.40        73.60        73.80        74.00        74.20        74.40        74.60        74.80        75.00        75.20        75.40        75.60        75.80        76.00        76.20        76.40        76.60        76.80        77.00        77.20        77.40        77.60        77.80        78.00        78.20        78.40        78.60        78.80        79.00        79.20        79.40        79.60        79.80        80.00        80.20        80.40        80.60        80.80        81.00        81.20        81.40        81.60        81.80        82.00        82.20        82.40        82.60        82.80        83.00        83.20        83.40        83.60        83.80        84.00        84.20        84.40        84.60        84.80        85.00        85.20        85.40        85.60        85.80        86.00        86.20        86.40        86.60        86.80        87.00        87.20        87.40        87.60        87.80        88.00        88.20        88.40        88.60        88.80        89.00        89.20        89.40        89.60        89.80        90.00        90.20        90.40        90.60        90.80        91.00        91.20        91.40        91.60        91.80        92.00        92.20        92.40        92.60        92.80        93.00        93.20        93.40        93.60        93.80        94.00        94.20        94.40        94.60        94.80        95.00        95.20        95.40        95.60        95.80        96.00        96.20        96.40        96.60        96.80        97.00        97.20        97.40        97.60        97.80        98.00        98.20        98.40        98.60        98.80        99.00        99.20        99.40        99.60        99.80       100.00       100.20       100.40       100.60       100.80       101.00       101.20       101.40       101.60       101.80       102.00       102.20       102.40       102.60       102.80       103.00       103.20       103.40       103.60       103.80       104.00       104.20       104.40       104.60       104.80       105.00       105.20       105.40       105.60       105.80       106.00       106.20       106.40       106.60       106.80       107.00       107.20       107.40       107.60       107.80       108.00       108.20       108.40       108.60       108.80       109.00       109.20       109.40       109.60       109.80       110.00       110.20       110.40       110.60       110.80       111.00       111.20       111.40       111.60       111.80       112.00       112.20       112.40       112.60       112.80       113.00       113.20       113.40       113.60       113.80       114.00       114.20       114.40       114.60       114.80       115.00       115.20       115.40       115.60       115.80       116.00       116.20       116.40       116.60       116.80       117.00       117.20       117.40       117.60       117.80       118.00       118.20       118.40       118.60       118.80       119.00       119.20       119.40       119.60       119.80       120.00       120.20       120.40       120.60       120.80       121.00       121.20       121.40       121.60       121.80       122.00       122.20       122.40       122.60       122.80       123.00       123.20       123.40       123.60       123.80       124.00       124.20       124.40       124.60       124.80       125.00       125.20       125.40       125.60       125.80       126.00       126.20       126.40       126.60       126.80       127.00       127.20       127.40       127.60       127.80       128.00       128.20       128.40       128.60       128.80       129.00       129.20       129.40       129.60       129.80       130.00       130.20       130.40       130.60       130.80       131.00       131.20       131.40       131.60       131.80       132.00       132.20       132.40       132.60       132.80       133.00       133.20       133.40       133.60       133.80       134.00       134.20       134.40       134.60       134.80       135.00       135.20       135.40       135.60       135.80       136.00       136.20       136.40       136.60       136.80       137.00       137.20       137.40       137.60       137.80       138.00       138.20       138.40       138.60       138.80       139.00       139.20       139.40       139.60       139.80       140.00       140.20       140.40       140.60       140.80       141.00       141.20       141.40       141.60       141.80       142.00       142.20       142.40       142.60       142.80       143.00       143.20       143.40       143.60       143.80       144.00       144.20       144.40       144.60       144.80       145.00       145.20       145.40       145.60       145.80       146.00       146.20       146.40       146.60       146.80       147.00       147.20       147.40       147.60       147.80       148.00       148.20       148.40       148.60       148.80       149.00       149.20       149.40       149.60       149.80       150.00      
      -40.00    -331.2243    -441.7184    -165.4574   -255.04425   -266.21728   -345.16936   -495.28545   -445.75888   -551.80308   -360.60401   -499.51811   -371.54139   -543.69325   -260.78826   -559.88329   -376.08355    -586.5902          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -39.50   -367.06001   -334.89957    -19.03981   -368.40767   -388.36626   -387.11145   -460.28206   -257.13416   -504.58813   -293.77682   -538.43688   -351.89403   -553.04223    -348.1433   -620.61851   -525.51391   -465.59408          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -39.00   -468.35805   -361.54946   -227.15401   -493.43758    -402.6348   -366.44569   -407.45424     -222.144    -347.5662   -408.85764   -551.88214   -282.41867   -610.12649   -427.75663   -568.01653   -710.07063   -528.54547   -663.15661          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -38.50   -606.87762   -452.83686   -464.09166   -396.85362   -332.25873   -373.29971   -287.82815   -416.85128   -235.26827   -526.47234   -567.25814   -261.42718    -596.6799   -309.53023   -507.12002   -586.46382    -613.4283   -397.89457          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -38.00     -694.919   -444.04782   -690.42553    -399.0268   -351.31291   -411.50137   -321.51628   -591.22956   -469.20376   -552.73062   -384.39595   -318.94882   -485.05304   -370.98635   -586.14907    -313.8733   -579.58312   -405.05731   -361.05863          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -37.50   -776.64279    -389.6432   -592.42334   -500.83156   -443.00434   -445.60401    -433.7251   -625.85655   -650.49811   -533.33019   -383.43967   -313.73056   -427.70581   -405.10723    -673.1846   -251.04336   -568.76724   -350.39637   -437.89349          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -37.00    -759.9152   -403.08861   -406.31955   -595.27785   -580.63246   -426.02261    -461.6027   -503.81581   -677.59363   -574.19638    -581.5985   -212.35447   -560.65991    -313.5657   -718.18155    -276.7793    -586.0278   -397.06087   -597.00431          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -36.50   -622.32881   -468.05825    -294.5157   -571.29413   -648.17915   -446.46198   -408.01001    -393.2001   -638.52494   -640.76154   -627.90651   -153.34645    -752.2095   -432.56585   -532.28515   -268.03385   -536.83857   -454.56064   -563.62727   -611.19243          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -36.00   -408.22453   -433.81354   -287.31635   -566.13651   -611.57294   -403.37151   -385.10632    -338.8391   -637.83769   -488.58669   -442.06497    -387.6944   -747.82186    -573.8694   -371.46455   -401.37807   -498.82525   -527.58375   -478.61673    -564.5507          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -35.50   -195.69738   -458.10344   -343.98894   -508.44441   -464.27254   -259.97047   -315.34288   -406.54311   -522.92514   -432.05187   -327.82603   -538.89756   -623.01659    -613.1353   -424.10743    -487.4463   -605.57891   -500.91466   -560.62472   -649.01162   -552.29683          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -35.00    -51.61986    -545.7686   -356.82895   -445.88495   -477.19686   -301.56418    -239.9977   -453.38839   -404.87399   -473.11957   -560.21834   -513.21903   -596.29412   -633.56666   -436.05303   -458.23075   -536.97248   -561.02371   -609.25462   -712.55607   -342.57897          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -34.50   -141.07709   -652.32446   -323.16753     -339.882   -648.70799   -441.07129   -318.93268    -422.2607   -386.15297   -523.84226   -842.91082   -434.17602   -444.81724   -610.84596   -602.57222   -422.62862   -489.37714   -666.73081   -530.29763   -719.86539   -457.87854   -571.90117          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -34.00   -220.03948   -602.56132   -312.47363   -321.67966   -773.98057   -629.28022   -384.45695   -353.41355   -417.79793   -488.56292   -705.97128    -399.2673   -416.61658   -572.56335   -664.72808   -417.44663   -643.53203    -554.5594   -530.28122   -704.24267   -552.33726   -506.20309          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -33.50   -288.59472   -558.98594   -417.74055   -321.58814   -651.64213   -627.14768   -410.52984   -462.57485    -447.3001   -480.63344   -491.92209   -443.78678   -594.81614   -461.56083   -550.68127   -472.47918   -626.95133    -439.9089   -355.50723   -615.55997   -393.78178   -638.19608   -443.00848          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -33.00   -336.06024   -572.50214   -544.39902   -298.79376   -584.42852   -521.08092   -451.36269   -397.50536   -530.45097   -507.24479    -507.4618   -472.55293   -741.07073   -298.42998   -570.68737   -549.43261   -444.61918   -419.80141    -554.1978    -645.9341   -447.49782   -613.55254   -552.64233   -518.12843          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -32.50   -331.86914   -497.88154   -528.03735   -349.04939   -540.72379   -407.28995   -427.26277    -93.83066   -589.25829   -438.41163   -486.32543    -535.8371   -460.59374   -492.17678   -611.88387   -469.85735   -472.74598   -452.14986   -591.14929   -816.25275   -622.17885   -443.27293   -692.63625   -320.31898          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -32.00   -270.72193   -389.31544   -462.84953   -395.17792   -506.33182   -283.37453   -414.74294   -184.80903   -642.02256   -349.61173   -421.29349   -630.42046   -353.00543   -659.69204   -492.93884    -423.9467   -605.00916   -388.49291   -431.57343   -672.98854   -365.63105   -446.04473   -553.87593   -387.53381     -3.96384          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -31.50   -223.16184   -272.48133   -349.44672   -329.66483    -478.5868   -238.99251   -390.40347   -368.54314   -676.06483   -351.24934   -419.18783   -547.83406   -691.47091   -492.00603   -490.69783   -488.40325   -508.18989   -435.75403   -443.29438   -542.16394   -370.35205   -395.76928    -524.0925   -557.33743   -146.47015          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -31.00    -205.5983   -185.11152   -285.08914   -164.21877   -412.88074   -290.62387   -385.73567   -405.81849   -600.64954   -433.14762   -417.98895   -411.38821   -886.81628   -546.76445   -434.05341   -613.15566   -444.09018    -484.8501   -494.99523   -562.73795   -439.36367   -275.75726    -622.2061   -484.85684   -336.71388    307.80236          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -30.50   -188.40366    -153.4209   -223.79855     25.05848   -316.80252   -339.39147   -493.03484   -451.01281   -408.61022   -473.97068    -390.6445   -369.45991   -576.58755   -493.26698   -635.21881   -544.76023   -425.43296   -558.35591   -441.17032   -473.29966   -376.91358   -466.16387    -601.9044    -452.8139   -328.86306    271.92318    650.09466          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -30.00   -172.31842   -176.60273   -102.03358     69.61031   -215.39666   -338.60142   -562.14617   -534.76451   -197.55878   -511.36645   -362.26233   -431.43702   -496.88139   -520.69946   -792.45839   -498.38863   -399.01671   -593.43736   -411.07168   -498.45045   -187.13588    -612.5478    -494.2902   -469.97985   -158.13735    429.80731    836.76616          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -29.50   -233.23526   -267.61512    -46.57541    -93.31525   -316.13216   -271.06385   -515.17719   -442.74206   -265.36314   -356.61416   -406.65881   -579.87562   -530.25425   -494.21783   -508.85248      -455.37   -422.73093   -595.81462   -447.94958   -344.54629   -232.86313   -510.41903   -412.34874   -302.05751      12.2394    277.33651    689.99443     738.9267          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -29.00   -300.58291   -322.67892    -60.04166   -244.02947   -425.08041   -244.25011   -556.66338    -309.2195   -461.74312    -47.94922   -422.31716   -654.77153   -571.28936   -365.60515   -302.40438   -464.19834    -395.2134   -670.38188   -459.14392   -415.62785   -398.56996   -484.12775    -412.8805   -333.45816    -56.69861    311.24953    763.19721    845.60518     68.66308          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -28.50   -392.50715   -309.37017   -139.88248   -386.57886   -465.42732   -301.23442   -588.29631    -231.5878   -536.48998   -232.08162   -403.80988   -487.85322   -531.22732    -382.7279    -247.0571   -555.30849     -373.122   -383.11052   -494.28892   -548.70794   -501.99908   -511.37095   -495.86247   -583.82477   -102.51601    565.87924   1011.31753    845.25418    136.41228   -230.96445          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -28.00   -481.03301   -256.68149   -242.19558   -566.85254   -447.92252   -378.43821   -470.84401   -274.71514   -575.10674   -517.20175   -404.98917   -335.93778   -497.62829   -385.43935    -272.5263   -551.86565   -356.72928   -122.51841   -461.64744   -572.63679   -500.40013   -551.73643   -561.26322   -481.10714   -213.40482    317.87994   1086.13502   1236.26115    369.24809   -137.90658   -554.39932          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -27.50    -539.2661   -199.80133   -308.76114   -675.71086    -452.1373   -419.72488   -451.60622    -381.9162   -640.30276   -615.32105   -365.18411   -349.00219   -406.27529   -347.77394    -323.8505     -438.439   -487.46448    -216.3031   -551.79068    -628.7448   -389.93588    -524.7648   -469.60438   -490.19501    -215.6707    429.08306   1242.37708   1350.57577     325.0766   -256.33944   -506.98616          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -27.00   -570.45735   -147.47918   -336.13876   -640.00318   -501.15556    -458.6669    -507.4006   -457.92189   -579.48865   -503.28948   -271.83559   -393.47215   -377.93062   -261.26031   -269.11304   -491.09559   -581.69652    -449.5632   -669.07264   -470.12111   -510.61155   -418.74734   -325.75296   -308.78052   -230.16213    606.76961   1171.57101   1344.94729    365.23042   -188.54079    -564.3297   -514.04892          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -26.50   -457.21714   -169.14712   -301.76638     -459.476   -517.25996   -518.89258   -523.69824   -415.59614   -541.61359   -394.72301   -246.68958   -404.73742   -496.42293   -344.17138   -477.72003   -277.66956   -468.73696   -510.82461   -686.33304   -410.98215   -569.77094   -440.40881   -487.71767   -419.54657   -218.72788    298.11232   1161.44916   1492.03168    687.51374    -36.02519   -522.11864   -494.05792   -558.22482          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -26.00     -288.442   -237.00917   -248.28262   -309.59265    -445.8851    -515.8947   -481.68359    -336.5975   -459.97202    -488.9034   -407.45568   -446.22153   -625.58943   -383.77743   -453.98168   -287.37043   -445.91151    -540.6656   -538.49156   -539.08749   -494.41053   -529.19069   -575.24633   -596.79772    -99.85317    524.78461   1288.11576   1552.32418    792.30936     48.74909    -447.4424   -554.78279   -575.63018   -507.75648          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -25.50   -186.96627   -372.66701   -175.83428   -276.14775   -302.04117   -376.06108   -389.00321   -409.11446   -472.95195   -608.97781   -462.01755   -422.22953   -473.01798    -493.5833   -270.61913   -396.19649   -379.02224   -529.06775   -436.85632   -549.30543   -403.90206   -486.42989   -394.51002   -447.99212    -86.98998    787.48848   1402.61974   1507.16231    746.91627    -54.34565   -613.27709   -391.94952   -574.74673   -639.59741   -621.25733          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -25.00   -175.74198   -421.76979   -195.31584   -284.72185   -226.83398   -261.23424   -329.56312   -483.76355   -418.04186    -467.8067   -280.02224   -444.52593   -332.38941   -561.93739   -237.92484   -418.06963   -340.70157   -285.08475   -396.42622   -568.37168   -425.91301   -589.01445   -362.92377   -387.72491    -33.27346    861.90515   1356.33189   1359.19083    789.51717    -24.69598   -439.72445   -799.54465   -655.01098   -763.05254   -462.90125    -465.6649          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -24.50    -200.9337   -401.53785   -229.16139   -266.82776   -224.01494    -225.5067   -326.16491   -497.31296   -281.82176   -265.73159    -98.85155   -504.50887    -331.9552   -360.17886   -421.80279   -465.23813     -466.932   -240.10275    -459.6983   -548.63923   -595.43167   -662.27076    -373.9681   -277.67562   -114.76749    659.95401   1596.12972   1370.96482     871.7427    117.97017   -385.13543   -511.46633   -773.83797   -736.71442   -536.41256   -471.55833   -589.74948          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -24.00   -190.34467   -334.24455   -256.16447   -290.39668   -223.67861   -255.22946   -371.90943   -508.62961   -217.99218   -371.76789   -200.95111   -485.57954   -169.68764   -270.04439   -436.34686   -531.03651   -571.85854   -399.05572   -543.16342   -436.17984    -519.0454   -433.12325   -268.14248   -378.42724   -108.28579    677.73215   1759.82359   1745.60201    968.87832    100.53082   -272.42048    -432.3906   -528.55675   -489.35545   -691.69319   -632.58673   -607.13396   -608.95604          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -23.50   -175.81527   -357.41343   -215.21876   -328.40331   -245.45954   -316.66473   -462.34119   -447.91033   -240.45526   -509.43625    -297.6193   -401.66817   -181.93735   -470.67871   -408.41368   -456.19306   -571.81424    -536.3981   -359.70114   -322.31158   -538.43065   -363.93537   -311.06854   -418.67003      4.85538    682.91645   1824.06833   1788.62468   1277.04134    233.17623   -178.28546   -539.50015   -442.23576   -460.48499   -506.38873   -486.77463   -539.21959   -689.36002   -466.72615   -616.75097          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -23.00   -259.55455   -434.96223   -125.35733   -368.40736   -273.04909   -414.01436   -470.34942   -357.47267   -370.33984   -476.27406   -336.87682   -412.89039   -299.40723   -578.08023    -480.0293   -226.10696   -487.39175    -544.5686   -204.91323   -302.12731   -465.45047   -503.76247   -429.08615   -290.34916     83.58453   1000.85907   1599.36705   1832.92097   1306.92783    195.48641   -453.54763   -641.75788   -453.86456   -418.08198   -284.42126   -467.79044   -607.50297   -690.62441    -416.0345   -712.38749   -547.02699          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -22.50   -346.84322     -466.137   -100.48248   -399.09159   -322.25478   -461.27882   -379.69999   -510.65724   -611.10581   -404.89819   -358.89349   -453.91717     -318.285    -384.3474   -545.60681   -122.43557   -476.12008   -533.98625   -170.85152   -318.12125   -362.81627   -504.83893   -372.64521   -261.22398    -40.16326    649.20285   1754.85001   2113.67365   1253.68916    181.47305   -403.43226   -456.50973   -469.65844   -521.95558   -368.78734   -495.46736   -626.40785   -651.25883   -455.10761   -427.32499   -629.80658   -430.40916          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -22.00   -413.06127   -503.95266   -193.90906   -454.98544   -392.92598   -452.23975   -331.38825   -608.61683   -669.92698   -479.40077   -358.39777   -379.84521   -313.50731   -357.21592   -647.64524   -330.40767   -566.85421   -588.84922    -327.4453   -450.90041   -292.98445   -421.97976   -513.30663   -210.72087   -157.02546    543.20814   1861.40761   2020.23523    1379.2288    155.04957    -329.8937   -608.47981   -255.67662   -478.89678   -382.27699   -440.28727   -601.50347    -536.0187   -553.20677   -646.81036   -494.99086   -456.55646   -462.68018          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -21.50   -404.34894   -553.41708   -316.42263   -482.26019   -319.14311    -438.2397   -357.51417   -539.05101   -446.48443    -639.9822   -387.62201   -318.80535   -285.76055   -443.84743   -679.11424    -534.5184   -570.58091   -434.81436   -429.54108   -453.66677   -338.67733   -350.14957   -479.87063   -301.56481     20.91238    713.09076   1994.47139   2051.67694   1540.81279    242.46206   -218.72212   -459.84605   -534.57448   -372.72457   -280.56205   -499.51436   -410.96821   -482.34096   -521.49816   -652.13009    -622.7677   -512.77694   -679.60547    -604.3037   -648.32974          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -21.00    -453.7302   -607.49127   -321.95929   -459.48349   -206.60313   -388.98206   -360.07321   -431.28525   -333.07713   -674.74127   -440.90016   -336.41948   -312.87517   -393.61631   -621.80586   -674.93756   -529.04106   -263.27978   -379.93836    -288.9744   -314.56607   -280.61333    -194.0241   -321.03524    -17.63987    627.34263   2031.98585   2335.63548   1518.01929    132.28718   -172.55163    -396.1334   -468.75417   -522.24367   -275.76625   -373.63397   -552.96149    -465.7595   -529.51111   -475.96793    -465.6473   -576.95096    -596.3065   -527.53801   -761.40344   -561.98354          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -20.50   -553.21942   -495.21845   -255.12402   -457.96448   -179.66189   -250.80841   -393.08168   -385.97112    -316.7391   -580.12173   -468.97905   -345.47585   -388.11258   -254.23278   -578.62797   -525.07456   -490.87092    -268.9452   -392.53379   -349.43488   -304.16918   -331.85035   -371.02059     -35.1782   -133.86877    502.41622   1941.29684   2558.68592   1266.94554    298.50582    -43.74669   -318.82502   -402.67195   -328.63083   -262.78336   -538.05292   -503.40832   -561.65637   -537.69355   -581.37581   -542.74819   -565.64012   -619.24982   -662.19076   -495.83439   -635.50675   -540.18317   -776.79732          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -20.00   -606.79421   -438.72108   -157.16875   -434.53714    -226.6931   -172.97728   -384.80177   -373.96956   -269.33908   -490.03615   -436.22778   -316.92729   -361.18153   -303.55977   -447.12106   -453.92164   -443.99342   -247.53571   -462.69582   -402.00064   -275.53877   -483.03754   -571.24412   -236.74744    -10.97578    557.02392    2002.7707   2467.89234   1178.40407    443.34834   -158.47648    -300.6645   -527.03618    -327.6897   -411.04948   -595.89165   -590.77682   -488.89375   -490.68093   -603.51512   -616.81918   -647.57329   -784.89674   -555.73327   -354.14997   -448.38799   -669.84628   -609.04252    -604.2092   -515.60185          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -19.50   -631.49847   -445.44888   -304.91009   -385.79853   -308.19238   -264.97152   -327.53682   -368.67216   -463.88129   -295.36973   -404.58978   -273.63699   -202.78679   -411.18313   -404.44784   -381.84595   -311.04687   -261.02966   -358.65176   -461.67227   -355.86783   -439.38585   -521.08555   -337.30847     -36.2368   1069.21651   2149.62273   2798.28849   1568.72538    375.33954   -229.78422   -283.04486   -463.26346   -442.31273   -585.08936   -544.36635   -482.67895   -571.81505   -436.63397   -578.61462    -549.4122   -483.04283   -678.88997   -431.57137   -564.86616   -469.21482   -730.61569   -671.48063   -694.19688    -593.3181   -669.78705          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -19.00   -578.32609    -474.8397    -484.7779    -344.9233   -385.27866   -420.95309   -277.00639    -436.2346   -691.17358   -241.90853   -285.44412    -140.1459   -177.40798   -351.80646   -369.04489   -357.43426   -185.68529   -449.12671   -278.82508   -517.66999   -709.20928   -441.54199   -520.40398   -210.35837    -54.88782    847.27556   2004.19748   2696.46814   1622.76837    317.80971   -224.58099   -493.73641   -254.93522   -473.28348   -423.48404   -467.02686   -464.80924   -350.75373   -412.26235   -552.03335   -658.74428   -489.28649   -558.45036   -659.43409   -593.18697   -670.08869   -586.43546   -681.20913    -596.7934   -682.45305   -770.83896   -435.44266   -622.01713          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -18.50   -432.51851   -486.20597   -583.23069   -405.53508   -402.80431   -542.35238   -300.29136   -401.72937   -615.95032   -369.65502   -167.89986   -108.11762   -588.44025   -301.27971   -452.91694   -432.88453   -196.47196   -431.09736   -233.29082   -459.20477     -526.074   -526.28468   -525.94767    -209.6761   -146.40976    709.46112   2300.80071   2704.24462   1828.95896    411.24806   -197.18082   -464.90652   -229.90793   -386.14666    -413.6424   -400.37207    -487.4396   -589.26029   -450.81429   -396.55362   -328.06205   -600.67109   -552.02019   -781.04819    -522.5856    -520.5826   -496.19344   -588.26939   -574.77095   -656.23634   -752.74232   -604.89561   -416.31278   -418.69106   -420.29739          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -18.00   -332.86202   -533.65358   -309.55158   -419.05725   -322.29201     -510.974   -312.96313   -292.42019   -436.78269   -406.76861   -282.41373   -158.83606    -674.8684   -427.04704   -625.34623   -449.39994   -500.05576   -367.82449   -355.53035   -374.01409   -404.49704   -503.07361   -402.00075   -283.49993   -216.65054    668.03204   2173.92683   3079.65504   1984.20775    409.66757     -96.2877   -384.99095   -309.62455   -410.48017   -421.85059   -429.65695   -424.81998   -459.23604   -392.30007    -479.4045   -493.31267   -594.10727   -610.71949   -471.56036   -506.43559   -616.25152   -471.01188   -790.06921   -583.64538   -660.01623   -462.78113   -484.53146   -552.11925   -646.30681    -557.2254   -753.99983   -384.31349          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -17.50   -290.36587   -536.42345    -79.68767   -380.14235   -205.88939   -396.15386   -309.27973     -201.229   -355.72711   -333.49536   -421.06313   -264.09238   -387.41181   -496.28377   -494.73323   -372.55456   -616.77883   -373.33541   -493.54812    -331.6429   -441.62122   -421.64958   -301.92812   -275.04414   -164.15944    908.39029   2222.69971   3202.76807   2045.54924    607.09756   -205.55561   -229.61036   -324.90902   -197.90688   -468.64865    -451.1599   -330.32431    -293.3827   -376.84239   -463.30543   -378.87886   -410.33955   -541.31334   -536.25346   -411.95866    -539.2182   -504.56395   -590.36193   -603.62945   -546.11522   -673.35682   -507.68546   -569.56135   -603.86347   -518.07806    -566.5664   -473.11166   -673.69077    -528.5936          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -17.00   -359.82652    -503.7039    -48.79794   -366.61365   -129.50547   -288.80768    -367.4624    -56.97612   -264.46477   -268.36546   -482.31643   -341.75912   -275.19979   -444.88832   -381.06445   -402.49438   -539.91808   -357.76077   -424.85638   -245.15137   -187.46616   -369.24525   -358.69321   -279.15168    -67.37581    933.86348   2477.52871   3157.75158   1714.40147    682.02458   -147.64086   -312.12561   -317.50388    -308.7061   -471.45014   -521.59861   -290.77207   -461.35879   -413.48838   -481.68377    -367.8263   -321.44286   -549.24456   -346.29982   -488.45709   -689.61131    -459.2413    -539.6499   -543.83537   -311.94094   -537.83337   -506.56981   -542.20025   -582.93972   -550.19619   -687.71778   -455.81168   -546.46462   -567.70941   -657.26478   -574.83304   -596.92529          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -16.50   -337.86934   -450.63983   -148.04354   -370.27593   -307.45004   -244.04189   -368.16572     89.16002    -199.3542   -259.20908    -522.9325   -450.34478   -132.20728   -301.03861   -501.95784   -357.43954   -219.56922   -337.50352   -310.52809   -263.29764   -144.89873   -237.41557   -314.55654   -289.53502     76.63247    910.99344   2358.76659   3057.00956   2134.63114    721.37631   -174.47567   -352.69878   -288.36762   -368.50331    -447.5537   -470.80382     -444.204   -366.69785   -418.27273   -467.61008   -348.10066   -484.19938   -414.72684   -467.59951    -494.8764   -533.68745   -457.35388   -653.04993   -603.02488   -558.06666   -611.79851   -583.46981   -476.94104   -607.54007   -596.27015   -552.95235   -437.40421   -483.34856   -438.09497   -576.83007   -689.61565   -529.40954   -444.93529   -559.71497          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -16.00   -233.71267   -430.04205   -207.05003    -282.1046   -441.90308   -331.97029   -333.90759     24.57569   -381.80778   -216.12986   -498.12452   -607.86116    -66.25713    -42.19653   -450.72746   -437.02777   -135.11912   -385.10642   -301.37786   -257.52429   -478.44425   -259.92187   -305.91908   -373.94402     104.7832    898.76087   2633.06103   3123.74023   2228.56043    948.13385   -140.12397   -430.85671   -317.54334   -190.84113   -494.62556   -420.54377     -490.176   -239.82219   -519.12653   -332.03227   -404.29989   -510.56043   -291.28519   -500.11054   -504.80145   -457.97686   -536.75654   -554.83227   -634.80928   -545.16813   -596.42589   -583.40872   -476.08494   -609.04363   -494.85942   -514.92431   -557.23465   -525.94608    -483.1442   -458.82912   -648.57901   -555.37727   -636.64168    -638.3438   -597.15592   -571.85381   -545.79891          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -15.50   -171.32104   -436.26043   -223.73203   -246.88328   -489.66577   -445.42093   -335.79988   -310.35079     -514.855   -221.21668   -446.93713    -492.8456   -113.01503   -150.20473    -99.64436   -533.40565   -322.65944   -361.55272   -361.88171   -359.83274   -273.19282   -354.55276   -475.97715   -369.67604       4.7018    982.47027   2678.37777   3266.65718    2291.6341    566.37109    -33.31655   -306.39845   -310.33022   -413.11994   -422.91657   -448.99664   -459.06435   -270.12316   -343.42019   -209.74463   -708.39005   -548.11877   -626.53815   -603.31996   -377.36307   -539.73569   -600.94667   -597.79803   -370.07306   -587.51268   -616.79193   -592.64854   -317.61832   -558.42966   -499.73019   -650.07743   -465.14713   -476.70517   -515.30822   -549.60812   -503.67528    -580.8382   -519.91026   -612.67942   -574.78188   -552.92123   -532.01759   -646.01084   -539.26865          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -15.00   -167.47619   -479.67958   -260.64123   -309.13451   -532.79931   -462.72838   -426.98727   -532.05635   -485.83145   -249.27365   -245.76512   -351.36499   -214.74394   -497.71618   -143.51769   -537.58294   -216.76291   -227.98188   -333.78889   -448.79897   -280.31436   -300.96928   -458.09705   -170.90991   -109.40009    977.04213   2451.86462   3413.42734   2291.82443    710.73075    -52.46202   -305.20901    -341.2244   -402.17814   -427.09547   -392.14517   -457.49551   -364.95763   -420.49827   -329.61381     -405.754   -342.95779   -435.27818   -568.86643    -451.4652    -574.0811   -578.34642   -578.71659   -393.08868    -486.4415    -639.3593   -595.09478    -446.9715   -506.87148   -655.58498   -589.97712   -568.75558   -599.57107   -548.11428   -487.99608   -334.74145   -732.59468    -247.6186   -582.01423   -464.24977   -641.55332   -562.80451   -481.75872   -528.33301   -303.17199   -281.63725    146.63378          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -14.50   -238.10164   -515.79521   -328.15266   -463.04229   -594.90294   -283.87655    -501.6268   -448.46157   -383.59519   -331.03933   -123.29186   -405.80546   -322.00975   -399.86546   -360.14898   -520.86153     68.20692   -372.64801   -362.72873   -402.53892   -449.05882   -411.44599   -188.43722    -244.5341    -39.57791    912.85009   2466.50312   3398.66018   2443.65288   1011.72891   -123.00878   -337.82402   -172.89826   -333.78517   -517.78918   -382.16698   -429.84351   -421.21564   -322.58519   -406.04976    -265.7888   -411.66864   -425.69128   -385.19699   -392.21196   -451.17321   -374.77144   -487.94222    -453.6501   -564.54834   -490.32535   -485.56783    -738.9919   -446.27962   -442.89492   -618.42295   -502.21853   -478.67949    -534.8055   -376.51486   -523.64489   -510.58231   -525.23985   -591.81168   -531.63307   -425.55027   -468.44865   -629.95461   -592.02366   -523.29054   -208.06101    412.04896    328.22991     32.30775   -269.18657   -632.36718          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -14.00   -437.36299   -517.95195   -446.02187   -630.62602   -383.16011    -99.64912   -264.58969   -412.76449   -256.98307   -366.80586   -275.28838   -477.36689   -447.26763   -360.17474   -414.16256    -457.5366   -200.31788   -475.22584   -276.59553   -356.22913   -420.29494   -461.99348   -314.60256   -325.72239     68.25439    971.02042   2471.48349   3456.04518    2607.3286   1069.77936    162.28084   -406.42524   -291.90852   -271.07344   -601.66115   -458.91414   -469.34279   -324.64711   -467.75807   -363.15178   -258.28499   -577.16551   -481.18187   -480.74186   -360.76244   -482.97306   -388.01516   -599.00785   -579.22202   -578.11516    -693.3071   -505.02563   -564.68349   -549.77622    -429.4909   -695.37661   -435.96879   -541.92199   -490.01385   -568.64394   -632.42687   -559.48828   -391.35684   -487.26688   -428.43017   -460.24688   -437.89166   -574.33426   -425.99542   -251.66739    142.82656     389.5409    435.27255    184.34523    -86.82842   -268.31996   -539.87096   -535.93936   -506.11438          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -13.50   -596.43674   -453.53953   -545.17944   -512.86892   -114.92379   -251.03458      -4.6103   -456.85959   -213.65185   -257.30074   -450.00022   -450.58899   -472.24855   -306.97139   -376.61123   -450.53788   -353.87696   -452.84141   -262.85785   -401.90376   -478.23406   -229.20174   -463.32344   -299.45602     12.03956   1146.14844   2528.40705   3539.21215   2550.76333   1022.17851      5.43162   -286.20854   -464.45822   -282.38344   -211.94208   -426.81341    -256.7966   -353.19394   -512.31438   -284.33611   -350.33651   -252.94297   -383.51386   -415.11858   -366.85019   -298.30269    -478.7926   -540.35685   -354.03383     -586.078   -416.39306    -436.2969   -490.70235   -497.56061   -367.58329   -461.23004   -438.97014   -419.57704   -259.57529   -501.84364   -547.82091   -561.49658   -614.42225   -487.73842   -651.38678    -494.4505   -629.93019      -460.03   -375.04437   -201.91573    148.74229    303.00359    643.81543    333.46978     -74.1432    -300.2303   -442.10476   -385.14937    -564.0679   -465.77821   -507.26639   -536.35205   -557.08704          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -13.00   -662.50193   -386.59749   -543.80671   -286.29934    -88.86235   -464.97127      5.27337   -488.58583   -372.45623   -176.39393   -472.92755   -353.22719   -354.20989   -297.70353   -293.02842   -366.76654   -389.10447   -448.15526   -350.61199   -407.79644   -527.27066     -244.106   -486.41751   -369.97243     11.57213   1070.69011   2672.70242   3557.84951   2744.27467   1008.32375     36.71359   -190.13019     -293.467   -367.25915    -229.9391   -367.82175   -477.33284   -398.71128   -346.90602   -415.48782   -391.56293   -298.49673   -563.11977   -399.53711   -405.14932   -354.29921   -396.00998   -478.40945   -307.48761   -298.90686   -427.31622   -355.51954     -686.633   -452.27223   -478.02811   -452.54477    -462.9564   -312.38565   -366.56663   -595.31662   -358.39281   -403.71785   -559.56322   -519.16378   -478.99002   -640.04016   -482.84765   -519.68945   -247.60649   -376.38978    352.45894    696.19405    691.06841    492.87578     58.49953    -192.7039   -389.15706    -343.6286   -466.83107   -732.23528   -550.51738     -409.376   -520.22852   -548.27994   -522.95282   -621.24169   -596.72286          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -12.50   -539.64641   -334.93681   -447.11596   -197.24121   -187.50821   -487.61357    -29.68517   -425.44907   -551.09749   -236.86076   -408.40611   -107.80357   -240.16683   -374.83886    -304.3637    -151.3321   -244.47574   -491.55747   -430.68745   -405.94426   -417.45437    -326.1382   -292.19215   -349.29226    157.65922   1017.46518    2801.1657   3697.91768    2936.0999   1089.68736    -13.98981   -129.47775   -267.99609   -414.29462   -370.49168   -354.31697   -434.57069   -265.27266   -409.47259   -412.46809   -624.63746   -433.71963   -591.05331   -369.05481    -418.0611   -589.03176   -455.81941   -414.80248   -368.86731   -294.14157   -465.83002   -332.82739   -637.22358   -520.36586   -523.83859   -560.82231   -456.72406   -429.59265   -379.49971   -193.45922   -379.43063   -461.28844   -447.97093   -366.59278   -366.48042   -383.06674   -492.72093   -420.35065   -464.48184   -157.02573    137.17638     617.0299    851.78784    553.67068    152.48224      0.95608   -294.56885   -376.26679   -452.24164   -502.67779   -578.56132   -435.53169   -545.18463    -441.4674   -547.96857   -554.98619   -640.58342   -471.03331   -410.06519   -202.52825    239.54014          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -12.00   -439.22327   -308.77364   -359.70109   -338.42674   -242.26283    -357.2031   -145.17081   -365.41669   -397.60269   -272.79324   -257.76272    -14.17524    -65.84964   -347.74563   -288.90531   -329.84708    -84.09927   -337.21403   -405.25171    -406.8227   -438.76929    -392.3246   -470.90008   -280.94304     105.4198    879.78477   2930.99347   3735.01159   2861.86831   1266.74911   -130.58409    -89.65722   -303.17623   -413.32416   -279.64649   -454.80274   -270.16933   -181.90817   -531.17824   -449.43783   -346.25517   -431.01157   -460.62074   -334.35062   -344.88639   -442.22528   -499.55072    -345.1012   -400.53746   -422.17195   -407.36632   -487.45104   -448.57976   -397.79545    -374.2395   -442.36475   -500.23276   -657.62799   -271.86589   -195.14333   -352.85855   -324.35559   -533.05845   -348.74619   -541.83334   -288.53551   -504.71993   -497.12232    -404.7127   -165.98299    130.78695    865.94101   1011.15766      787.964    263.38025     67.60801   -219.09963   -272.29603     -416.183   -516.93877   -431.50221   -524.42474   -463.68752   -405.09283   -490.40729   -408.60438   -587.30696   -399.42427   -306.87916    -56.12847    329.58158    618.14582    509.96618      3.07877   -208.28753   -373.05218          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -11.50   -398.76696   -299.58898    -363.6089   -470.10054   -256.45623   -260.49724   -355.48957    -335.1682   -252.44361   -423.07466   -182.31668   -262.86551    -36.34253   -244.04842   -432.83592   -527.98454   -257.31886   -218.46792   -140.94826    -386.4223   -525.46591   -403.12597   -502.69631   -172.57412    135.02958    732.52766   2688.45672   3609.00873   2958.93962   1407.17143     73.71632   -102.86782   -295.52485   -596.48704   -398.61402   -560.03726   -196.30798   -149.38925   -571.63795   -560.64695   -456.76116   -507.53588   -488.05383    -368.3284   -379.20335   -437.02624     -411.824   -310.76248   -257.85993   -427.56496   -423.27028   -340.87292   -477.33112   -414.99641   -496.08298   -568.08476   -170.87099   -528.24779   -386.08678   -304.77316   -423.30813   -386.53694   -403.33722   -563.77815   -318.27713   -580.33689    -332.3915   -523.11454   -406.00716   -222.60384    262.03106   1033.42111   1285.69381   1003.21813    288.68069     34.40845   -210.53632   -396.99034   -493.20124   -375.18008    -584.6177   -532.00287   -496.83243   -529.62739   -524.50623   -449.78968   -460.40935   -423.96347   -322.91488    -57.63618    594.32068    882.89419    762.27864    285.00464    -13.75007   -344.10105   -623.27954   -570.68863   -515.93863   -510.09458   -551.89684          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -11.00   -462.37265   -308.62189   -418.94806   -441.00608   -280.88942   -316.38209    -448.0153   -297.25747   -310.72808   -571.51033   -316.53172   -401.16693   -175.56855   -310.60135   -587.33078   -272.38924   -318.03274   -248.68797   -144.30533   -330.03232   -569.21067   -383.56814   -188.96478   -365.09898    194.85148    988.23079   2864.81136   3540.48135   2973.36162   1395.80874    234.99801     -166.158   -326.31061   -294.54984   -423.40953   -433.60676   -274.75619   -223.44417   -353.56978    -367.6942   -216.12518   -339.12582   -596.51419   -585.74715   -451.21776   -382.64692   -568.76703   -419.40187   -432.01203   -476.74743    -602.9242     -798.577   -451.99157   -479.06387    -396.2399   -483.21107   -343.44359   -413.01429   -387.80993   -340.58285   -384.46741   -350.65264   -315.48358    -363.9829   -451.56021   -562.15299   -528.42179   -434.94558   -422.55531    -233.3422    465.95496   1170.79631   1170.56276    999.72213     427.6404    205.63316    -20.80913   -142.49715   -459.37668   -560.14741   -564.50167   -634.28505   -387.41427   -406.18323   -478.50157   -414.53053   -451.98231   -403.52387   -285.65101    173.95017    583.86548    985.55193    735.43066    584.11923     66.91394   -213.81998   -483.02509   -419.74917   -385.80905   -394.04995   -304.01216   -325.26563    -390.3391   -396.93582   -488.76923    -530.9485   -595.83239          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -10.50   -541.63432   -337.72594   -490.66413   -355.59599   -336.65758   -392.79606   -332.63531   -261.60285    -389.3182     -450.283   -375.30397    -397.2283   -307.55362   -364.41052   -430.48335   -336.67082   -165.42702   -276.61665   -140.72457   -250.55206   -522.95784   -351.67349   -229.38071   -391.12608    239.75103    810.85899   3135.50582   3801.33619   3052.87668   1310.53093     75.10188   -207.86697   -284.30076   -396.23531   -314.64331   -526.23277   -330.27946   -256.54404   -428.29205   -407.73255   -322.91728   -581.35566   -527.73098   -575.00112   -426.18523   -421.26065   -640.83489    -287.3244   -377.29585   -424.34171   -553.78856   -510.58827   -488.76502   -353.52403   -299.69916   -354.65018   -455.47912   -325.38108   -217.73685   -378.65844     -344.778   -381.27506   -290.41206   -366.47943   -391.01734   -445.69906   -473.92104   -495.08711      -369.19    -62.22253    390.24963   1179.61594   1335.33727   1065.16663    569.67914    170.36553    -67.02344    -279.9228   -472.49663   -609.46636   -622.31303   -612.55156   -456.31892   -442.78245   -486.00086   -364.13622   -564.80435   -413.72506   -281.78088    121.60011    882.75586   1245.96314   1189.84072    610.07903    146.16584   -276.16612   -386.27177   -478.70176   -441.82384   -518.57199   -298.43316   -232.94172   -247.26981   -288.48734   -363.30384   -425.02474   -451.53397   -509.18171   -598.10657   -616.05994   -570.39926   -676.53154    -585.8089          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -10.00   -637.66038   -370.78613   -477.15502   -275.95943   -387.92935   -348.07827   -279.45811   -242.92229   -325.20496   -293.31874   -217.70263   -432.19986   -435.86209   -473.98885   -368.31183   -584.50662   -506.82396   -270.16934   -152.20903   -288.72612   -441.74044   -463.20146   -342.15009   -156.49202    277.12597     843.3279    2989.5484   4147.23531   3151.39701   1227.33807     160.9589   -178.26835   -163.05487   -426.20029   -207.33987   -282.33875   -405.20949   -386.29892   -220.23662   -405.42897   -392.53286   -495.78292   -411.52037   -457.96269   -223.54878   -406.53762    -478.4923   -270.13598   -423.05937   -392.69331   -327.13544   -318.62912   -507.20666   -433.14745   -406.50838   -497.28685   -533.34001   -167.08046   -311.10923   -403.66001   -456.59547   -422.25535   -290.81968   -407.89275   -541.70666   -530.55609   -468.26979   -313.66089   -249.95871    -60.49743    352.00559    957.70256   1694.38619   1268.11363    707.77105    218.06381   -105.06466   -233.82031   -456.39959   -469.60052   -545.66548   -319.49922   -448.92457   -407.78724   -380.43041   -369.56443     -474.254   -332.30443   -136.37969    288.89801    945.48409   1400.04614   1326.35691    913.90159    100.43401   -273.97884   -356.13419   -404.12509   -528.42327   -432.74589   -344.84952   -336.47992    -98.49879   -152.46604    -306.4479   -348.20291   -561.20261   -542.81455   -512.73562   -516.54831   -582.89399   -475.66001   -452.69381    -382.8401   -533.61748   -559.29617   -541.76275   -572.61012   -461.70105   -478.02462   -484.07772          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -9.50   -724.22385   -406.19165   -431.16843   -244.20856   -383.75221   -271.43685   -231.00503   -273.55148   -204.50808    -272.9182   -146.54319   -506.40319   -465.71113   -506.61051   -401.12947   -555.44011   -623.52146   -211.58086   -108.86104   -338.49928    -313.5772   -377.64217   -322.09596    -52.19344    339.06784   1088.12623   2584.80983    4102.1832    3320.6924   1125.67772    182.48785    -99.64406   -274.70195   -337.95587   -304.37527   -225.84767   -421.99733   -353.37993   -459.84124   -313.37732   -434.00296   -628.89607   -425.93787   -454.60877   -467.74904   -457.41711   -371.76783   -472.08434   -417.30389   -316.87895   -385.94525   -227.00339   -487.69165    -355.9449   -585.20577   -409.54383    -291.1575   -269.41128    -270.7004   -251.23238   -383.91934   -381.89801   -307.18066   -300.34179   -353.43589    -443.6969   -331.57833   -392.07914   -190.48539   -113.33318    454.26232   1212.21051   1615.58739   1338.86416    727.91218     60.62674    -71.77775   -284.07356   -414.38354   -531.40148   -441.96833   -542.76764   -428.12997   -307.75846   -265.71085   -292.27827   -479.98904    -353.2771   -212.12039    215.13528    966.45503   1642.03347   1463.86126   1036.99139    438.92071    -26.13971   -278.11459   -419.55217   -396.65672    -458.5914    -405.6838   -244.29823    -73.92649    -177.2597   -433.24885   -245.32282   -523.58923   -498.93611   -289.03714   -519.41088   -632.21711   -476.12122   -471.90442   -632.51218    -615.9222   -564.05519   -355.52177   -485.67906   -400.36291   -492.21908   -451.52042    -501.7416   -620.06338   -415.01601   -473.69568   -123.05216      8.47486   -236.44078   -445.69018          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -9.00   -788.38547   -424.40378   -404.69359   -223.32704   -302.44936   -276.04554   -112.80563   -367.50251   -161.79656   -262.11213   -227.04044   -353.40019    -447.7693   -370.51923   -357.38559   -507.50874   -361.78815   -289.71998    -93.68197   -325.12845   -397.46304   -350.83388   -166.19645   -185.53481    256.57182   1087.32654   2952.15103   4056.46883   3313.10252   1508.53182     54.15918   -356.78884   -243.00071   -242.58872   -294.35241    -92.82553   -257.74672   -499.05544    -308.1674   -436.42812   -357.03803   -195.96081   -431.63914   -323.15523   -548.88853   -491.81675   -441.06986    -524.3356   -422.80946   -380.56727   -500.90233   -274.65035   -363.35676   -298.42696   -486.67011   -355.07378   -393.43622   -349.53356    -372.8348    -244.1862    -345.7099   -441.04982   -274.28017   -314.57075   -354.70905   -466.64349   -423.32486   -439.66503   -490.31988   -148.04559    444.66941   1080.44071   1563.10553   1264.32934    746.33339    132.97131      2.89301    -54.42772    -85.85059   -430.35003   -474.85736   -483.37722    -183.3647    -314.3902   -563.58031   -422.56616   -516.84959   -495.54472    -92.98081    389.07358   1050.18067   1513.49052   1882.85712    1234.5303       447.13     39.86232   -301.28291   -434.95531   -360.93423   -359.25266    -171.0187   -138.52612   -221.42931   -195.36746   -303.79664   -386.93482     -480.011   -509.74216   -456.72443   -448.90262   -553.28479   -544.64574   -607.03466   -495.32023   -500.76715   -414.05082   -264.17645   -494.37822   -414.37702   -593.58418   -453.61674   -566.85553   -288.20841   -311.15634    -215.6862    -57.37254     49.24781    -38.27203   -164.52636   -271.74223   -305.34954   -611.91504   -542.04889    -464.5921   -517.15587   -311.47218   -334.66353    272.23775          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -8.50   -664.72189   -420.94875    -422.1859     -89.8534   -214.69064   -346.08966   -159.45178   -410.36121   -159.91363   -252.10636   -271.53825   -113.96943    -434.3519   -317.30154   -312.53693   -353.98443   -332.36437   -332.39066    -62.70839   -277.34539   -442.18883   -471.81606      -138.98   -224.93942     19.33526   1092.15899   2996.04948   4218.36855   3179.77168   1405.33605    -42.74927    -291.8933   -384.83857   -308.21389    -12.73515   -350.27912   -306.39346    -395.0095   -365.49983   -472.29174   -319.57907   -220.21891    -660.3791   -285.57181   -609.01384   -545.28465   -444.60931   -473.98889   -416.75878   -278.03846   -486.63066   -560.84685   -504.15906   -402.16708   -360.32336   -355.41977   -309.37377   -165.91152   -316.85108   -349.32985   -250.22381   -307.60441   -419.63615   -280.44406   -467.47176   -282.26372   -281.49703   -218.89509   -331.30078   -154.52927    332.74219   1193.35745   1728.15226   1276.33806    745.37171    236.92673     -3.44484   -128.62201   -212.61215   -474.81964   -377.23829   -377.17669   -248.34111   -293.24725    -420.5336   -349.02191   -452.26685   -117.91414   -180.09393    419.92527   1319.10678   1836.79613   2076.56183   1352.59025    604.51999     152.0467   -282.50844   -251.86409    -262.6361    -330.2043   -226.37109    -27.07259   -236.92791    -33.62316   -401.27621    -318.9621   -367.67762   -294.81074   -494.25983   -600.34216   -564.14321   -480.26161   -428.68653   -528.73121   -425.76355   -508.89419   -410.58437   -481.75722   -465.88432   -377.67946   -461.01505   -534.32185   -420.95898   -405.83395      -3.7218     14.43596     84.52004    -23.75832    -26.73785   -223.77783   -378.25129   -481.82365   -470.40289   -408.49624   -486.17355   -315.14139    -54.45409    190.79174    789.49686    738.78611    774.92299    507.78008    162.31171   -269.84435    -195.7805   -301.18978   -454.61743   -463.25684          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -8.00   -383.35512   -365.51641   -450.00434    123.86341   -286.75136   -402.31696   -258.44724   -310.27726   -176.86667   -262.36442   -317.60873   -314.52799    -444.2116   -347.64011   -341.05656   -401.86612   -320.18285   -168.13568   -139.46613   -297.11856   -393.53782   -346.97927    -147.0587   -205.12939    116.34503    986.53802   3030.17478   4285.34145   3476.59342   1395.91875      -18.722   -103.53266   -258.03322   -479.94334   -110.28573   -116.18398   -441.44327   -138.00977   -523.42215   -373.72832   -538.49727   -274.93104    -459.4781   -275.18616   -195.61281   -361.36175   -432.24348   -389.72506   -520.05453   -240.81305   -498.67051   -501.28342   -327.69579   -327.26893   -480.65069   -551.99736   -387.39042   -258.65474   -329.39865   -198.55718   -442.68221   -279.06916   -497.78731   -354.73649   -161.93928   -524.90682   -488.93209   -305.08955   -261.99045   -136.82965    626.89724   1363.22528   1661.30366   1429.93352    914.53197    268.74581      -54.413   -104.78057   -190.41482   -403.79362   -291.86755   -382.84198   -363.39233   -160.43617   -305.36066   -320.38232   -336.99164   -415.23203    -63.12649    428.94979   1321.05403   2007.59987   2079.86846   1456.89114    631.93331    -53.60518   -360.20121   -291.67645   -353.92494   -401.33984   -247.81723    -45.97147   -262.97495   -138.83083   -277.33406   -356.52431   -383.26365   -496.37649   -506.80022    -634.4546   -461.69389   -422.18935   -383.14725   -563.29876     -412.724   -386.42946   -448.61618     -524.759   -491.22969   -482.41249   -537.51483   -521.53276   -246.50771    -249.0557    -16.21068     94.38845     66.90528     37.54835   -180.97278   -310.05879   -331.50322   -408.25684   -602.14857   -500.54163   -380.35575   -369.75909     -1.47019    318.16966    859.12751   1248.40937   1009.66239     829.4369    342.33076    118.56919    -184.9825   -298.81479   -302.41655   -293.24299   -370.98414   -399.29304   -496.52553   -498.61333   -572.55645   -458.22123   -507.71173   -640.56262   -563.73675   -532.83713   -580.01428   -521.43772          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -7.50    -99.98236   -307.66524   -371.22912    297.77142   -404.39309   -411.42404   -301.08144   -196.52247   -203.43397   -331.83079   -403.16385   -624.97467   -437.21014   -383.09539   -362.49105   -533.83129   -179.09797   -129.60489   -196.14157    -432.8911   -257.37009   -356.96919   -205.80165   -198.24009    -39.75391    944.12314   3207.99368   4334.71951    3521.4352   1537.49315    314.99059   -315.30826   -252.40149   -501.28287    -238.7404   -238.11259   -358.30019   -311.66525   -418.12298   -443.16636   -484.60654   -286.67049   -352.96807   -344.68171    -283.9049   -369.58264   -450.13603     -450.169   -419.31151   -372.26437   -273.57736   -501.44076   -371.71969   -347.46519   -454.73595   -476.72801   -286.14111   -347.23476   -527.51111   -113.69226   -385.56162   -357.18876   -403.16107   -479.15412   -398.90209   -380.73535   -232.87464   -525.69992   -295.02716    -54.93213    456.48876    1369.6552   1711.46256   1591.68787    921.71927    246.11828     -38.6554    -68.14739   -164.86234    -214.7112   -322.00593   -417.65786   -294.20363   -365.94664   -320.79238   -388.01595   -245.74426   -299.13849   -168.97941    356.72614   1182.43764   1895.70903   2054.61429   1348.30821    869.45473     85.29073   -239.15391   -321.27817   -271.03316   -428.51282     -215.071    -55.21834    -78.98101   -113.94685    -234.7267   -284.87409   -313.53941   -510.49784   -489.58613   -542.17555   -549.05618    -471.4043   -456.43651   -377.98057   -526.79231   -509.52988   -329.56819   -406.08744   -367.85945   -350.72816   -399.50418   -477.71319   -319.40519   -481.01902     77.44582     96.57639    129.10805    176.73347     67.55609   -233.90571   -320.32896   -319.16641   -473.20225   -342.48941   -488.72379   -307.82236     42.46883    460.14796    864.00043   1289.46655   1121.13959   1129.48028    392.19841    203.59352    -80.88281   -119.05365   -300.59833   -370.51579   -382.13878   -488.58996   -455.38468   -427.85331   -666.32549   -555.92253   -757.59017   -489.48097   -471.18553   -592.19872   -418.51471   -645.98856   -484.82707   -528.49568   -533.67849   -517.67925   -303.68255   -542.59308   -437.57966   -349.29711   -405.16787    -537.2779   -649.71195   -619.22065   -711.59899   -527.04638          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -7.00   -173.52587   -266.45646   -277.15682    207.86617   -412.23387   -412.31958   -307.39869   -158.34428   -278.88292   -417.99121   -531.72101   -303.44379   -431.05751   -449.98359   -396.13707   -426.81953   -115.28836   -219.46576   -246.49513   -238.14252   -166.38329   -428.01579   -329.32473   -189.90637   -108.30635    986.67385    3302.6944   4258.44642   3594.66021   1643.42212    219.92107   -283.91866   -331.57195   -374.02979    -271.8581   -416.49235   -216.81278   -254.09503   -388.86046   -353.96065   -423.83427   -348.38715   -134.30752   -373.98681   -365.57726   -260.09017   -468.57391   -344.34712    -316.7928   -479.62911   -392.31926   -392.43116   -361.98991   -468.31039   -424.25802   -491.43563   -252.89846   -356.98034   -306.30183   -404.45411   -291.66093   -378.44798   -285.01164   -468.33102   -454.56299   -404.65012   -360.89613   -426.38282    -455.0396    166.07589    606.84317   1390.83447   1717.89191   1817.32086    998.86791    404.24149     70.58971    -45.75089      20.0149    -85.85726   -426.23775   -263.39943   -437.19989   -307.26294   -403.32501   -664.13903   -296.34746   -128.13062     81.71712    444.25074    1589.7925    2134.4295    2344.4452   1681.60945    776.88988      96.5539   -300.77373   -329.97558   -347.69773   -285.49165   -106.85538    -197.8469     71.45255   -179.49717   -261.25802   -383.39015    -407.1198   -452.54228   -392.50429    -509.6053   -352.72956   -355.78515   -535.80932   -474.38169   -460.60708   -432.38538   -445.03124   -353.82593   -354.38701   -415.77695    -460.5871   -362.15134   -413.91223   -232.23955     89.65409     197.1335    263.58967    228.48958    122.79289   -187.63394   -375.92056   -332.97099   -505.32363   -455.71667   -495.89647   -277.52812     21.56164    433.39118   1048.35726   1423.86373   1456.99051    1068.1494     425.4528     133.4382   -252.65791   -316.53514   -350.95673   -237.64955   -353.72868   -173.18017   -519.77112   -591.34822   -415.23841   -399.22398   -407.07625   -609.70782   -581.68679   -485.66591   -601.20651   -493.17281   -553.49329   -507.18477   -505.84873   -401.17289   -488.51427   -404.72886   -369.09941   -300.57089   -547.52241   -365.26979   -398.86147   -474.57717   -629.22722   -557.59124   -309.97943   -346.67516   -235.99697    -311.8246   -473.57004   -300.91344   -475.62161   -530.77229   -518.08341   -595.33845     -501.029   -447.72946   -505.70217   -648.87648   -660.06098   -637.40379   -601.80479          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -6.50   -444.64928   -290.38244   -233.45391     98.08723   -297.62068   -396.34988   -266.14775   -131.25122   -397.70801   -391.21623   -506.97839     -1.20085   -459.35009    -361.7116   -447.88582    -297.9878    -28.01511   -337.26006   -285.79014    -81.37505    -90.38714   -272.59581   -414.56628   -174.89849     69.66816   1347.26354   3225.83348   4055.92319   3741.94779    1493.8164    198.21538   -211.11798   -353.80709   -317.48761   -318.56892   -323.85158    -293.5385    -357.9456    -155.2939   -397.59389   -391.86369   -349.08606   -343.16204   -319.09009   -318.89948   -370.42122   -333.62996   -475.31355   -394.55079   -382.94615   -363.85367   -274.40976   -342.06282   -452.98856   -365.38902   -409.26098   -303.94368   -345.39625   -260.02181   -276.29057   -257.97649   -634.47576   -425.63662   -492.29939   -329.70706   -519.76111   -335.12294   -396.74802   -250.01876   -112.47343    539.28651   1332.73371   1833.48594   1590.57012   1017.35384    492.50715    128.73861    -15.74629   -142.10234    -49.64739   -306.83689   -426.18926   -176.29993   -500.55248   -287.11382   -455.34799   -305.68388   -126.96122    167.29887    566.77932   1561.68399   2428.76053    2551.5355   1642.41199    889.69292    271.00075    -177.7313   -415.75147   -441.06025   -305.17392   -293.74655     70.86961    -28.82058    -33.86609   -238.56869   -272.74643   -333.75854   -475.17137   -369.30451   -465.60337   -389.00649   -525.71231   -591.11881   -529.34632   -450.77634    -359.0127   -341.92964   -509.81475   -336.24244   -456.90273   -398.48498   -443.41903   -392.47908   -316.30995      4.87224     209.1711     261.8997    396.63093     70.79923    -49.55472   -224.37368   -256.19857   -418.42181    -511.3099    -301.5525   -205.81439      2.01304     510.2234    887.09887   1298.29196   1361.41238   1201.69404    504.05644    203.18171     36.35426   -153.33625   -358.04487   -402.56623    -211.3478   -268.76471   -340.83914   -475.68429   -554.14637   -431.33688   -546.64752   -580.70046   -477.33795   -538.88679    -594.9162   -617.16844   -477.15256   -492.97842   -455.96682   -353.56908    -320.9057   -347.23656   -347.89731   -330.84685   -295.42346   -392.54254   -553.85781   -475.80914   -515.19826   -394.49295   -422.58824   -143.33526   -222.42958   -204.99187   -390.48179   -405.10743   -328.12075     -363.682   -519.01061   -442.30749   -465.01312   -534.71362   -612.46672   -480.57736   -389.91586   -351.68838     -618.901   -563.21394   -572.72674   -467.32473   -608.54186   -576.76888   -567.98458   -558.78407   -470.28308   -284.64572    -457.7702    -468.5868   -426.95867   -465.29347   -497.75312   -654.94834    -631.2763   -614.09574   -584.40648   -444.06179   -457.35625          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -6.00   -690.80901   -306.57217   -317.16018      85.3239   -213.64982   -398.30271   -172.99896   -300.58002   -354.75745   -334.34475   -370.80676   -106.90562   -482.69388   -330.99286    -471.4313   -177.48104   -240.01806   -377.72437   -302.74151   -126.37407   -331.73344      -257.36   -502.68531   -112.83968     18.01099   1198.82764   3264.08201   4147.96742   3858.59314   1521.18667    381.17698   -180.31878   -246.96207   -157.38229    -297.3421    -199.9148   -395.77778   -200.56677   -567.13845   -466.47523   -349.20406   -397.68725   -451.47521    -200.2418   -339.08095   -333.28335   -403.18365   -407.41942    -283.4303   -363.16466   -346.51511   -374.23288   -501.42584   -395.41587   -361.38435   -402.21265   -265.88401   -156.27174   -288.17306   -252.74149   -126.76859   -317.12931   -294.10063   -337.32681   -260.52366    -401.1203   -229.56717   -324.79741   -353.34773    -19.54646    475.73531   1398.10973   2000.96871   1900.12999    948.93976    394.02183    352.49465    179.67833   -294.03993   -365.86327    -371.1031   -313.68734   -317.91605   -390.98131   -233.59019   -349.28842   -220.76057   -229.66611     80.61388    609.09383   1465.31734   2492.54091   2414.13316   1626.24696    666.99168    154.48673    -234.9686   -440.67187   -168.79488   -215.81514   -131.12378   -117.24863    111.26232   -112.83873   -336.26458    -136.0444    -392.1322   -481.73654   -406.81993   -508.58888   -440.06482    -451.5551   -446.39078   -392.09536   -588.55052   -441.30416     -352.469   -342.26635   -435.94432   -397.12757   -435.33361   -426.43487   -255.19448   -152.37653    -37.61491     54.32275    489.27611    331.18155    156.86106   -142.66939   -262.79573   -471.48166   -316.79982   -227.03569   -377.01454   -229.84377    179.64449    763.77043    1063.0224   1433.24866   1453.43454   1119.97311    565.56221     91.09337   -179.84245   -308.34676    -70.67112    -70.20987   -288.44047   -241.38172   -453.48747   -262.10695   -532.02217   -413.28984   -463.10171   -544.25877   -497.18168   -602.93275   -409.14444   -527.36064    -418.1799   -415.03979     -446.688   -527.20173   -340.17359   -338.93326   -226.18493   -297.02867   -337.92725   -445.37183    -523.1664   -383.40872   -572.43463   -427.94829    -337.8322   -276.50484   -261.74501   -129.00488   -207.07836   -265.78201   -291.01595   -440.40052   -169.20024   -449.27077   -388.86085   -456.94746   -451.84729   -570.69811   -490.45523    -566.6862   -510.00845   -612.29715   -580.89872   -619.87069   -505.35135   -364.09586   -447.06711   -330.08233   -240.79821   -241.84341   -258.17065   -498.14462   -313.81943   -522.97624    -356.8337   -527.63767   -395.12266   -460.00287   -461.70971   -428.79594   -361.81387   -427.98596   -587.47735   -493.10844   -595.02335    -604.4385   -473.85841   -618.94628   -708.81842   -516.97858   -638.88163   -676.62022   -477.00093   -725.58052   -514.15365    -529.4784   -516.95767   -455.40827   -472.25708   -197.27508   -324.97298   -335.29878   -336.67063   -541.20605   -485.86734   -277.17172   -341.37757          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -5.50   -693.02989   -325.89354   -402.00735      97.6481   -353.51217    -372.9289   -146.26569   -484.09415   -143.70315   -298.78362   -357.91232   -193.02002   -511.45562   -519.64705   -402.84667   -146.38708   -480.69102   -335.67734   -312.68179   -283.69337   -431.55621   -252.09357   -597.12846   -280.64096    -71.68373   1008.09794   3282.23519   4723.97822   3910.60137    1665.2509    325.03904    -207.3788   -299.42239   -453.79264    -331.1273   -237.57779   -347.44774   -311.86083   -418.73311    -449.9655   -379.28521   -353.13217   -336.49722   -442.18735    -278.7925   -350.90991   -324.51999   -263.36406   -221.61682   -408.41255   -413.50707   -542.94826   -410.04581   -356.85246   -442.05516   -452.06721   -262.20649   -299.70635   -183.08512   -232.39617   -325.38549     -428.302   -232.33071   -309.41621   -372.96024   -303.74663   -261.09829   -278.32506   -278.46973    114.93448    783.87241   1212.18659    1934.1591   2029.79288   1049.78781    341.59829      96.3508    -165.4236   -184.36817    -209.7711   -246.38988   -471.58004   -372.28751   -369.00066   -332.88664   -359.88358   -241.55123   -404.88957    -65.73382    490.71928    1683.1587   2680.32704   2946.69506   1936.81207     644.9237    297.97511   -204.58592   -316.19173   -379.25296   -408.64943   -107.98309   -111.12272    -65.64046    -37.94058   -188.43674   -177.25691   -458.82752   -418.94688   -349.25022   -434.29432   -394.52276   -475.15841   -376.66855   -391.06411   -364.09263    -400.8773   -388.01416   -302.84171   -284.29925   -360.97878   -473.94969   -252.07328   -358.25383    -53.62302    193.98424    361.56162    409.89798    191.10147    279.81163     93.40165   -168.28564   -331.98871   -352.08052   -274.33463   -353.98607    -81.71361    206.02544    827.53453   1151.08835   1448.77525   1760.01673   1259.52927    767.18072    308.87989   -162.14681    -35.55583   -185.83113   -203.93735   -193.16232   -233.13525   -364.09993   -400.40223   -317.83871   -396.65336   -493.37206   -423.28798   -361.84123   -422.85275   -509.22914   -414.30642   -485.19081   -424.35152   -463.85556   -532.63839   -420.51932   -398.64663   -303.10488   -373.89001   -348.32217   -459.90484   -701.76245   -491.88014    -522.7157    -405.2599   -335.27834   -326.84967   -201.34348     31.88653    -90.32705     -39.1968   -234.30818   -326.45681   -341.57569   -329.64636   -371.18157    -502.7968   -363.64551   -391.50019   -518.93451   -404.65727   -577.50933    -526.0167    -619.9802   -457.68107   -401.82291   -530.49415   -352.47119   -313.62236   -450.85997   -318.27072   -166.58084   -334.87098   -301.74206   -390.76352   -242.43404    -313.0378   -244.99842   -450.57553   -140.60349    -317.6902   -368.67941   -140.62311   -101.01319   -508.59462   -348.45911   -672.41425   -407.47312   -497.73004     -554.679   -409.25716    -481.6245   -548.46204   -580.04391   -658.81567   -563.17983   -406.56817   -460.17714   -464.61211   -414.95392   -413.42554   -384.53689   -137.35813   -148.48242   -311.58442   -267.41604   -312.74905   -332.19569   -151.20313    -319.3937     -3.58674   -174.81728   -176.87636   -387.04194   -538.61529   -546.96523   -577.48668    -758.8419   -744.49438   -644.87094   -568.45335   -506.55082   -466.28787   -581.51209   -439.54222   -410.30557    -403.4499   -536.91737   -461.38876    -421.3569   -338.57048   -537.63878   -411.69826   -439.45853   -108.19175    -349.4975   -260.78947   -320.84589   -427.00346   -404.97387   -435.10978   -384.16587   -594.38942          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -5.00   -646.42732   -341.24329   -393.50366     65.75465   -488.93456    -359.9344   -279.47134   -522.13745   -103.62681   -278.45087   -378.33813   -240.82162   -488.90169   -489.67841   -376.22352   -174.57547   -657.19569   -408.52733   -173.85423    -317.3681   -504.73693   -107.58912   -486.61609   -575.55424    -73.13803    813.21755   3138.09936   4304.21726   3184.13779    1638.1983    220.55833   -293.98124   -186.08904   -282.27639    -337.8892   -208.03885    -217.9411   -357.96056   -329.18427   -484.34036   -311.08823   -319.15791   -437.93622   -350.44664   -199.36924   -389.63707   -330.99744    -508.0869   -349.89858   -450.30347   -371.48947   -481.37087   -409.18181   -320.52352    -449.8424    -386.0871   -296.14965   -370.98259   -369.86985    -68.80335    -237.3715   -390.26408   -435.48115   -344.79725   -381.44972   -593.87492   -481.20422   -291.16635   -312.01907    -94.07345    687.20558   1439.26355   2092.25816   1851.91786   1202.12953    442.99773     103.9272     38.45882    -35.41219   -156.23315   -327.31486   -483.96696   -413.07318   -484.43568   -248.85985   -365.18622   -371.61084    -350.7603    -56.27274    556.77617   1563.29812   2406.72395   2798.40646    1761.6526    929.20528    233.61038   -162.67963   -177.51055   -424.77016   -272.05433     13.33062    111.47049    163.67406     41.60578   -256.30895   -328.85922   -352.82327   -552.68049   -249.90059   -232.62965   -499.14644   -388.59516   -285.89057   -333.67748   -595.94547   -366.47331   -220.34338   -373.64214   -281.82882   -288.08642   -419.96828   -345.65298   -166.59157   -235.98224    228.73314    281.72241    442.80005    327.42347    147.79172    -48.55077      -3.3748   -327.04739   -296.90418   -302.34719   -171.93601    -76.35535    -20.21751    682.67078   1576.68873   1791.10322   1796.94336   1503.87065    830.67822    306.53262    -59.97897   -141.19115    -106.8708    -73.61305    -86.55543   -118.07527   -283.46962   -469.17636    -373.7325   -409.32874   -521.72611   -490.03844   -557.43846   -374.52736    -403.9028   -387.86686   -438.26783   -464.74407   -348.28921   -354.25002    -75.08929   -229.27325    -60.39475   -195.45272   -330.07957   -367.62727   -380.45842   -562.26877   -484.33462   -258.64362   -336.69495    -67.93272     33.66916   -118.21738    -10.48512   -151.24169   -201.00702   -145.50889   -310.44834   -287.15551   -332.58005   -471.18876   -573.39851   -519.83141   -418.75404   -523.30743   -443.22199   -425.94654   -418.47505   -343.56598    -630.6216   -368.78622   -193.52486   -108.89421   -123.28863    -65.51706    -72.70084     37.18261   -263.84231   -392.17388   -444.71095   -253.06002   -330.29115   -361.53147    -55.57782   -287.02044   -140.34387   -205.41634   -324.77265   -400.48675   -366.96778   -458.33247   -591.62957   -358.64392   -499.42098   -497.70181   -374.73519   -596.93767   -458.27585   -561.26186   -503.98353    -400.0452   -209.94228    -33.23996   -223.90763   -187.94694   -267.32644   -314.01115   -226.69819   -172.15619   -205.79782    -57.24781    -22.35769   -184.33276    -79.96111    -16.47274   -262.11018    -212.2311   -175.75937   -358.63426   -482.27807   -562.37483   -348.21478   -571.87653   -556.88013   -668.29157   -609.61666   -615.97245   -478.95883   -637.18415   -629.60212   -391.75702   -422.27319   -414.09518   -429.06302   -364.32618   -275.84454   -347.87451   -130.69897   -106.94727    -44.15551   -129.71208    -34.33457     -1.31608    -101.7761   -360.33266   -242.46876   -348.01475   -451.04757   -440.28404   -629.07299      -611.69   -462.76953   -593.52304    -762.6836   -542.51919   -478.11806   -585.56376   -548.92924   -547.09503   -551.49689   -559.23653   -556.94602   -651.47636   -682.93438   -701.23946   -754.04448   -698.44984   -680.48438   -765.41234   -971.80429   -656.51386   -704.08365   -940.11847   -765.72266   -851.44249   -838.93125   -933.97189   -625.06392   -747.02452   -873.99636   -674.32835    -651.2257   -696.44441   -645.66973   -823.44144   -766.34547   -713.88717   -675.46923    -716.1789   -785.54294    -660.2767   -626.87404   -641.73399   -717.92456   -727.86837   -806.54369   -727.65123   -717.25662   -762.41263   -739.96941   -811.52147   -590.09764   -613.46985          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -4.50   -615.81542   -354.84687   -339.35665    -85.02776   -360.26085   -404.78689   -393.48666   -532.83417    -238.3169   -251.92891   -324.29387   -318.62112   -458.10599   -294.75638   -488.19047   -305.55904    -389.8627   -459.59165    -14.94812   -250.42359   -593.64598   -161.26758   -389.10279   -533.24822     -4.69755    790.22228   3072.28545   4500.95634   3580.65523   1636.28361    179.03206   -248.85708   -248.82526   -475.29219   -289.56988   -196.00715    -39.16719   -251.62692   -339.67747   -436.33996   -245.14209   -342.50789   -357.56102     -504.089   -444.05636   -415.66787   -298.58911   -372.25679   -457.05521   -294.79349   -261.10473   -578.99973   -369.07752   -167.38009   -156.91254   -325.36299   -354.61908   -304.16849     -81.1012   -388.35552   -186.16388   -394.23898   -186.67454   -319.36462   -387.17134   -258.57187   -355.81582   -374.23683    -260.2539     26.67385    513.23809   1486.20792   2268.43872   1880.28804   1096.19802    410.31861    186.97155    -53.48332   -151.57235   -326.82733   -369.24121   -376.13185   -411.13437   -413.59124   -359.64357   -287.69189   -231.95495   -211.85253    -32.49903    587.93277   1396.76024   2492.65761    2577.0624   1747.15139    970.18047    350.33398   -222.94414   -402.09656   -325.82288   -314.19907   -219.68826    -55.85285      82.3338     -14.7732    -205.4925    -85.97598   -253.77313   -355.12994   -482.30987   -422.04884   -439.21257   -360.82458   -418.12197   -356.46644   -423.78594   -329.00404   -385.38342   -366.52531   -267.20699   -439.44421   -246.44769   -371.52525   -289.03899    -68.40705     76.30357    680.83509    592.87057    617.01316    219.23958    100.88765   -155.75322   -145.05878   -315.40514   -426.81916   -254.50466     -4.77607    273.41075    998.30682   1334.41179   1846.93644   1987.36023   1616.83408    996.01628    272.69408     27.98075   -192.21865   -229.68878       -69.57   -144.46196   -248.38446   -226.18558   -395.12773   -341.02322   -407.81329    -466.8064   -408.13452    -331.0147   -361.45422   -415.25438   -397.32428   -467.71121   -406.14722   -351.04894   -446.01469   -301.35939    -93.93327   -274.66964   -181.35801   -226.83137   -352.76846   -384.09273   -431.52285   -398.22801   -388.47077   -145.00344     -6.07373     -2.87487     -6.16084    -95.73903     46.59926     -7.25564   -268.52948   -238.21856   -280.99756   -397.95679   -279.15254   -434.66388    -372.1111   -511.08279   -430.10651   -396.69927   -466.55244   -498.17201   -341.87252   -406.29335   -257.42507   -238.07078    -50.85167   -208.70687      28.1973    -57.50162    -32.47723   -139.28855   -226.85789   -226.00572   -178.50294   -272.99273    -88.10595    -68.60038    -97.02635    -58.03235   -165.66045   -173.17524    -439.5921   -272.50477   -357.23175   -506.35625   -589.25453   -615.91326   -452.70673   -525.81266    -544.9358   -444.58393   -486.24769   -507.98503   -416.14116   -281.69787   -205.15207   -171.71637   -230.83178   -117.62884   -216.83886   -326.58735   -176.53535    -190.4828   -127.86836   -257.35793     80.97939     83.88854     58.35704      9.25719   -143.26162   -231.60008   -325.97973   -343.35953     -324.854   -529.00699     -541.122   -462.88448   -456.75824   -382.62045   -410.27778   -617.66276   -449.06893   -453.06504   -509.62602   -440.97447   -245.33663   -400.32412   -325.31751    -140.6203   -176.98663     47.69827      2.23832    -73.67334    -51.81453       53.334   -174.92325   -191.50167   -225.49633   -210.05791    -292.6613   -388.08329   -389.65917   -437.44836   -548.76158   -576.97369   -538.95576   -585.51695   -403.54241   -444.21989   -394.73969   -576.68625   -471.23037   -502.14213   -548.86697   -489.77956   -620.05708   -617.06137   -549.09049    -657.2876   -687.18808   -876.60303   -666.34484   -649.31844   -585.95579   -595.72666   -709.70102   -677.88523   -692.79259   -737.68833   -550.98749   -723.31101   -633.83985   -713.58402   -582.26758   -696.38304   -630.36235   -690.81636    -699.5209   -627.16078   -646.17262   -518.34476   -660.42307    -601.8315   -579.60502   -549.18634    -524.5329   -645.15703   -716.66606   -704.07502   -376.30557   -580.52696   -502.15534   -506.79686    -674.4895   -736.69807   -750.52178   -559.93179   -614.31943   -627.18329   -446.43583   -660.14302   -628.21078   -563.92975   -617.56444   -397.52101   -535.29731   -574.33404   -415.54608   -772.70961    -637.1318   -559.03426   -647.94246   -622.60042   -591.59393   -574.45905   -712.70501   -728.38439   -564.77364   -498.14036   -473.33321   -414.52073   -387.66316   -511.94609   -522.84493   -456.77181   -567.35643   -372.95423   -220.28455   -480.97295   -575.71457   -488.43595   -215.64348   -292.80543   -384.14713   -383.68359   -698.08057   -705.71604   -392.33154   -479.88547   -673.77436   -564.80378   -643.98694   -595.85609   -681.21457   -909.68955   -643.71324   -506.70868   -694.50579   -651.31292    -264.4382   -975.22709   -823.92283   -753.10848   -921.02206   -716.23655   -713.11688   -776.19729   -759.12602   -721.65204   -769.78064   -837.12042   -892.85768   -832.33126   -874.39505   -891.04046    -484.9546   -799.89793   -807.74858   -936.88162   -653.51692   -809.02991   -923.86351   -908.11624   -861.58121  -1030.64325   -770.96168   -743.26025   -783.63716  -1049.19956   -649.14377   -817.62155   -966.39252  -1084.92945   -815.15343  -1002.11205  -1174.51411   -971.92816    -797.3829   -1105.9816  -1138.75831  -1041.66081  -1223.38618  -1035.35848  -1170.40858   -957.95838  -1041.21317  -1121.12859  -1306.72161   -996.30283  -1218.64689  -1158.90095  -1038.23174  -1242.73796  -1062.20447  -1410.47048  -1062.47618  -1205.35752    -1232.964  -1014.59642  -1504.83835  -1286.69022  -1290.20451  -1259.27744  -1393.26273  -1102.07346   -984.21422  -1355.55314  -1254.85539  -1403.32974  -1554.06026  -1581.02309  -1064.43894  -1193.95444  -1633.63822   -1433.0084  -1961.79169  -1549.89967  -1290.31054  -1609.89534  -1315.77968  -1524.33631  -1732.04665  -1324.02189  -1581.91894  -1818.18005  -1421.34307  -1124.30414  -1444.99812   -1653.6598   -1529.8281  -1543.55248  -1666.42103  -1840.31439  -1494.67141  -1253.84009  -1479.70529  -1223.51092  -1528.43394  -1440.16053   -925.40929  -1751.35604  -1655.00796  -1374.86439   -596.39664  -1567.38263  -1724.94212  -1219.43618  -1886.82106  -1176.31887          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -4.00   -397.82097    -348.1117   -297.07028   -221.44477    -173.7025   -499.19348   -384.07064   -539.56729   -341.82115   -233.50933   -253.91012   -437.51304    -419.3442   -320.51111   -483.44138   -355.21874     -92.0644   -509.82846      7.44784   -195.14847   -392.51696   -271.78577   -319.27474   -158.71872    223.59727    906.93677   3251.70652   4396.60902   3941.15094   1652.54895    -72.83012   -230.60741   -213.14746   -482.14843   -215.98253   -281.94641   -214.58774     29.64403   -231.07066    -439.1662   -332.46853   -389.93764   -268.35173   -327.60319   -432.52124    -398.1022   -495.25336   -490.33473   -355.85023   -356.37333    -338.5175   -416.66445   -471.89451   -423.02206   -130.62495   -300.93811   -207.88048   -262.73366   -320.25571   -304.15895   -377.74887    -33.74576   -285.48982   -488.74275     -310.609   -373.03894   -384.01311   -392.73569   -130.50732     -47.7527    659.26424   1712.51473   2117.48908   1979.53519   1100.28504     397.9552     245.7665    -25.29764   -213.42323   -218.64542    -312.9377   -406.61709   -354.72818   -349.37075   -246.53136   -215.37499   -271.87501    -82.57427     -8.40027    375.73351   1213.38296   2190.87974   2342.50726   1930.27381   1091.27664    313.77048    -91.72214    -332.4113   -105.58463   -445.12076    -220.1409     131.4715    140.09282     67.96899    -31.87317   -271.56414   -321.47709   -344.99202    -543.9777   -317.20043    -368.8544   -379.91826   -412.87133   -385.63409   -389.79809   -391.82272   -424.42026   -331.85223   -359.39357   -267.14597   -284.94508   -314.32572   -188.19769   -200.52749    263.11818    401.61796    629.79646    481.38941    347.12029      -10.099    -84.01726   -117.24832    -436.6763   -332.12247   -284.75705   -118.38289     276.9886    812.94178   1460.40067   1914.79602   1911.20062   1562.71169    889.61952    408.75098     76.75093   -120.16416   -100.65549    -23.92589      6.80433   -112.17429   -285.14643    -300.9949   -389.33371   -437.48688   -417.27188   -355.20215   -211.66228   -341.18295   -340.39864   -374.16558   -418.93722    -295.1926   -388.67798   -318.27211   -214.16138   -307.12147   -162.20868   -256.88722   -220.33909   -383.92456   -231.67544    -213.3264   -475.61357   -246.04478   -115.05798   -191.43517     28.53675      117.636    -74.36708    -48.51035    -23.22724    -86.98014    -319.5847   -445.05392   -319.63951   -294.49292   -267.27404   -468.35368   -404.51525   -447.85073    -478.2609   -375.39085   -205.36761   -424.05822   -243.35692   -223.48192   -305.16671     29.90242     -23.3139    160.06632    -61.74253    -16.25425     -137.967   -160.78205   -270.81381    -176.8308    -72.49066   -137.48773     39.18449    142.49184    110.36367    179.77991    -77.90731   -201.80352    -307.7168   -501.20607   -579.64958   -450.06601    -446.5204   -551.65963   -457.32987   -621.17166   -352.37607   -524.77004   -282.62377   -402.51831   -307.94344   -282.73507    -24.03811   -156.06646   -104.61606   -153.58824   -232.59125   -292.03743    -41.94973     -82.2644   -120.35556    135.05475     66.34347     -8.30838    -28.23608   -170.99962   -181.92529   -251.64098   -395.87314   -276.92526   -569.88712   -347.86834   -447.47911   -332.67866   -592.70203    -389.3992   -451.46316   -360.73874   -271.38131   -258.38639   -337.87776   -355.62952   -309.69534    -65.00176   -116.27383    -61.72221    -32.28145     10.94749    -61.78541    105.04528    165.84137    -101.6501   -113.90762    -253.7717   -378.06239   -261.94053   -483.25292   -365.94046     -573.878   -562.91582   -450.42855   -477.03599   -425.28445   -428.96444   -382.67271   -480.90138   -207.30485   -316.90598   -453.85248   -420.66905    -472.5552   -421.70536   -548.02023   -579.20284   -569.31531   -466.91389   -589.03828   -583.41833   -630.58059   -754.91033    -676.5267   -459.69267   -577.12668   -663.58345   -583.72437   -651.36985   -600.34696   -766.10673   -735.07661   -465.61134   -588.79255   -632.16464   -456.57365   -465.95232   -574.17094   -565.18374   -485.87354   -651.72523   -587.25537   -457.65205   -483.18156   -569.40801   -504.15563   -610.96519   -573.32322    -542.3965   -600.33887   -606.38227   -470.67016   -552.20178   -603.79145   -542.82497   -374.64773   -488.20413   -478.98057   -581.24729   -730.21602   -449.98321   -404.07125    -397.3083   -511.96188    -393.0354   -579.01842   -474.67414   -346.28591   -477.64407   -473.07454   -593.38726   -505.57255   -671.91071   -527.04689   -515.12361   -448.21039    -671.7173   -364.25761   -380.01013   -420.47148   -503.26483   -298.15173   -502.15865    -113.0846   -463.56799   -195.16858   -157.65007   -316.65637    -93.67722   -273.12841   -354.86283   -255.20476   -195.82467   -214.67917   -377.55837     -540.222   -394.52814   -281.54371   -473.12039   -309.38365   -531.45563   -366.08531   -428.84059   -325.78667   -306.24568    -290.3876   -464.35215   -732.61015   -528.50281   -357.90281   -688.81837    -643.0174    -692.0711   -633.93277   -576.84537   -568.34204   -906.80506   -709.12487   -641.93306   -748.53727   -719.30483   -659.74486   -806.17869   -810.23425   -251.16116    -504.7047   -778.13675   -618.38807   -866.33558   -787.38555   -522.54777   -673.57343   -861.38689   -672.49488   -644.77044   -192.89741   -522.82705   -759.99812   -551.67971   -982.00313   -780.41892   -530.15174   -604.91693   -559.93397   -830.20465   -745.62565   -750.78771   -942.78883   -844.50883   -709.98134   -829.60025    -875.7632   -817.92439   -800.20398   -687.43972  -1052.33311  -1049.24106   -868.99545  -1053.17361  -1054.33978   -831.51092  -1045.78452  -1125.38171  -1055.88153  -1328.16834  -1159.27249  -1150.14898  -1357.95248  -1216.23304  -1504.75313   -1229.9496  -1048.30062  -1294.15664  -1721.36585  -1591.02061  -1290.30231  -1351.57426  -1179.41205  -1354.97568  -1374.43914  -1477.19553   -1304.5938  -1193.77739  -1238.83243  -1383.90392  -1281.40788  -1719.69314  -1403.33386  -1678.39471  -1409.25474   -1313.7943  -1510.76165  -1529.18002  -1382.92046  -1623.05171  -1410.17815  -1343.87615  -1277.71693  -1702.43152  -1690.17746   -995.54075  -1965.77371  -1385.85093  -1600.46228  -1647.96094  -1202.33228  -2135.96438  -1795.05324  -1315.34337  -1382.42228  -1417.30577  -1638.68414  -1639.59268  -1643.22569   -422.79037  -1106.57832  -1021.18201  -1325.40099          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -3.50   -169.45775    -321.5222   -412.55023   -386.78812    -32.34399    -577.4976   -379.91296   -542.29283   -423.32569   -309.45244   -264.98382   -501.22108   -413.34097   -360.16084   -396.08416     -300.715   -307.44349   -262.27021   -236.38921   -230.71074   -277.35744   -374.76163   -286.36714   -263.74536    166.37517    938.03063    3195.2693   4293.47621   3781.91849    1528.2786    198.88861    207.46533   -259.27384   -440.31023   -284.29358   -332.59092   -219.13731    -69.26328   -257.52275   -470.38606   -275.40812   -303.14588    -345.4215   -241.96022   -385.04716   -403.63526   -269.96643   -542.80664   -438.77313   -441.39324   -291.98769    -401.0542   -332.96888    -357.2602    -51.75829   -171.89788   -462.99233   -203.94665   -101.92111   -187.22624    -84.76747   -277.32262   -411.78904   -351.98934   -364.99345   -383.04042    -184.1458   -520.66419   -251.67786   -157.35627    687.40195   1550.00823   2396.37348   2000.16462   1117.97242    783.31895    223.21743     41.08143   -156.86477   -137.53208   -270.48575   -416.28269   -327.76183   -293.19455   -235.51648   -396.14729   -315.02069    -95.08841    -13.55583    300.49972   1472.22084    2273.4009     2426.152   1862.80006    874.77234     95.90607   -128.47038   -217.09979   -272.35086   -147.77734   -195.07264     52.89989    272.41895     62.69482    -20.85257    -60.98598   -244.45028   -630.07123   -313.60505   -292.76652   -245.90246   -472.57909   -267.28827   -415.00427   -418.98378   -283.18781   -303.76049    -337.2663   -325.22912   -241.12344   -263.34021   -319.44378   -255.05948     43.84043    171.21159    576.80955     663.0993    460.17077    319.62092    237.22895   -137.05161   -282.46086   -174.90717   -316.12437    -88.00005     44.19462    200.57732    904.33269   1571.85526   2060.56718   2024.41299   1787.35298   1107.37266      646.919    143.50445     60.59116    -47.87927    -50.73889     -118.679   -245.37775   -244.19243   -412.51083   -240.02343   -381.72384    -328.2832   -362.95128   -421.67438   -427.81658   -363.38614   -320.34601   -320.15209   -397.33086    -246.8058   -281.47306   -112.13311    -15.03464    -52.47892   -208.76074   -256.11079   -218.13662   -394.24525   -381.02735   -135.41863   -170.25674   -240.75252    -72.17735     64.74589     11.85696    109.47865    148.68939    -57.95532      -188.64   -169.77567   -308.25978   -286.15548   -277.72247   -295.04153   -380.74244    -463.1622    -480.6862   -392.47922   -480.53468   -293.78598   -237.86724   -279.94283   -337.75463   -287.30289    -25.88365     56.61408    156.21673     96.84741     62.99225     25.52323    -24.07313   -161.97383   -240.74038    -15.33111     81.56139     -51.1779    338.06925     60.15491    -77.89925   -113.32735   -128.62603   -316.90001   -355.42165   -495.01599   -551.33233   -328.97386   -506.53464   -398.49233   -327.62377   -416.52431   -307.55797   -343.68617   -258.57864   -329.48181   -131.02437   -148.11476   -284.98656   -213.50267   -201.27834     65.60043   -138.45826    -87.05621    126.11172     59.50813    137.60309    -79.31083    -65.21577      -2.9889   -108.51676   -131.38676   -151.98455   -525.95041    -61.57549   -239.49197   -499.79479   -360.22801   -479.81744   -435.32023   -465.79543   -444.64873   -371.09244   -419.15172   -250.18568   -281.87125     -93.8544   -193.74481     55.31288      3.92822     63.62433    179.73169     -6.32695    436.90115    123.57962    290.79542    171.30435     64.20017     -6.12552    -88.46247    -67.40532     -330.946    -405.2625    -374.8411   -322.41866   -532.30741   -459.33359   -251.88219   -361.37743   -274.74653   -310.23383   -263.49241    -316.1791    -350.3101   -444.54193   -692.31695    -543.6122   -581.94012   -595.94575    -826.8963   -472.46316   -490.86645    -585.1457   -710.76345   -451.30944   -726.09089   -594.85282   -732.92506   -612.88429   -593.62169   -596.34802   -668.91563   -447.34457   -717.92432   -624.04824   -436.98137    -633.6284   -316.30437   -363.54632   -404.83657   -534.91661    -501.9593    -403.7506   -290.34196   -491.56048    -476.4686   -495.80915   -371.54438   -368.94443    -379.9497   -484.90529   -551.07146   -450.25639   -661.62182    -527.9661   -635.54796   -557.47741   -707.59358   -455.38467   -137.22934   -636.01787   -357.39102   -622.12973    -293.5734   -443.76358   -352.91077   -422.02643   -256.80742   -466.92296    -563.8595   -316.19019   -407.66108   -539.22086   -429.88606   -357.37537   -408.13033   -289.32142   -338.66135   -194.51013    -540.4001   -525.88763   -246.04936   -516.41957   -407.12031   -366.10503    -50.71148   -174.39467     -31.4176   -175.57748   -279.32384   -339.44695   -173.93863    -115.3974     -1.30071    -84.89066   -260.50869   -338.66921   -361.56184    -292.4099   -197.99642   -372.43681   -469.44097   -483.83638   -334.15884   -519.86088   -305.28889   -338.49944    -411.6063   -380.06385    -402.2972   -586.30262    -442.8394   -383.47833   -706.38121   -291.83144   -744.51621   -309.07836   -318.41085   -674.48551   -482.63128   -461.10755   -375.91928   -550.55753   -335.63349   -703.52263   -729.98786   -794.02667   -453.40782   -484.51426   -450.02273   -361.42629   -699.20985   -739.23786   -770.08102   -362.21559    -474.4792   -511.92507   -678.48668    -637.1924   -495.59281   -582.83927    -734.0834   -448.47102    -454.2021   -405.99933   -554.73422   -811.42902   -675.52909    -541.0072   -794.58606  -1037.76065   -702.96448   -407.40892   -637.64204   -663.45315    -817.1585   -714.85922    -785.0431   -862.07934    -990.2017   -821.08713   -879.06912  -1095.74465  -1032.42426   -893.93791   -975.13043  -1028.45877  -1061.68316    -1145.009  -1369.40139  -1192.50073  -1200.68816   -1119.3678  -1500.02866  -1356.73074  -1339.32278   -1202.6552   -987.34984  -1310.05242  -1517.54592  -1277.06888  -1286.24334   -1261.9906  -1397.05229  -1070.29856  -1001.99531  -1475.17959  -1193.72707  -1333.49465  -1047.66743  -1318.25435   -956.39182  -1392.52826  -1582.03208    -1164.924  -1066.54748  -1364.39429  -1268.93326   -521.45999  -1528.38434  -1175.10793  -1404.63962  -1424.85394  -1330.40376  -1499.39353  -1890.88553  -1723.67733  -1450.44308  -1661.55038  -1385.91751  -1480.74847   -983.14259  -1536.21065  -1274.66708  -1428.30953  -1456.80215  -2026.81878   -930.99477  -1750.72634   -970.04541          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -3.00     27.31091   -294.13809   -538.34659   -552.51542     48.81398    -596.6634   -480.14688   -469.41116   -453.35515   -366.05923   -293.04226   -532.86943   -422.25628   -313.41645   -289.44088   -291.81053   -474.54653   -157.71844   -471.10945   -362.70299   -454.27916   -276.89484   -153.27605   -176.79145     67.67814    805.35171   3095.15628   4404.11162   3855.67233   1651.41453    199.29006   -251.83955   -191.96207   -335.07858   -356.99798   -313.44561   -131.32751   -438.67781   -280.18766   -240.47518   -518.76572   -317.05882   -296.85781   -336.99114   -342.23443   -530.19558   -341.65625   -510.97249    -528.5246   -273.65773   -499.41472     -473.145   -398.73702   -389.68542   -385.15245   -332.19498    -63.08429   -240.09146     -8.09986     -2.78078   -261.15332   -298.35529   -317.28737   -253.51688   -411.45521   -499.52735    -259.9305   -374.21796   -213.78795    -54.94209    507.97879   1402.11721     2326.953   2091.05649   1337.53939    447.43825    227.42454    -73.86674   -140.78929   -341.28993   -401.39205    -230.5646    -337.1475   -180.71412   -390.85568   -410.05781   -282.82223   -137.21792   -134.93442    493.22095    1429.6818   2373.67544   2825.13867   2016.25703    939.93407     213.7967    -45.44376   -179.38409   -214.01066    159.97843    -87.33539    244.58955    233.05429     61.64924     83.05392   -246.39827   -154.88927   -261.97346   -383.48677   -292.07846   -333.01415   -464.18678   -347.31998   -343.73033   -314.74284   -416.39447   -395.93996   -252.97998    -259.7075    -409.9074   -313.82587   -173.62723   -261.77851     -2.49742    370.15638    722.97216     689.7593    438.85279    352.35415     17.64984     60.30738   -274.88713   -201.07657   -301.83568    -311.2598   -185.00496    175.29938    953.11977   1527.85298   2081.38659   2282.85847   1966.52082   1190.32948    745.20907    278.60804      9.76079    -81.84498     47.08148    -92.58394   -193.04326   -150.99899    -224.4338   -377.77923    -405.0595   -396.09891   -396.25399   -461.09426   -165.71993   -247.24925   -333.76246   -256.92531    -388.6403   -328.70786   -216.01209   -195.96503    -91.98758    153.37328   -148.02097   -122.64951    -19.96706   -323.01252    -383.9914   -174.37478   -355.04733   -123.02718    133.79384      45.4678    -48.99263     44.00933    -42.71187     85.02225   -229.54028   -160.49817   -153.09032   -313.83799    -213.6994   -325.19871   -278.10522   -254.34776   -283.07228   -531.62407   -403.29669   -406.08314   -213.50968   -275.64494   -154.15161    -97.22151     -40.0992     78.04303    -10.68397    -13.46579     73.72393    -39.92039   -182.28439    -77.22057     71.54475    -47.24656     43.07856    -47.76358     77.46848    119.75869    -137.8894    -34.95257   -155.17021   -278.08583   -352.60264   -369.94687    -260.3606   -374.65523   -332.88621   -442.05586   -240.25986   -495.71383    -324.9465   -334.98829   -465.72341   -229.14949   -200.51093    -61.34874   -303.40554   -101.57415     15.56508   -227.48582    -52.12636    -67.15165     62.38973     11.74666     29.03881    -32.35985     45.24126   -124.00475   -161.34576   -224.19856   -186.05958   -249.79423   -348.47542   -594.55398   -301.27221   -517.03824   -342.17057   -372.00601   -394.24914   -354.98544   -325.24717   -220.94848   -227.32828    -237.3784   -106.07868     20.62048   -122.39584    244.54814     22.33358    395.34427    495.94938    283.93799    193.94583    305.62996    143.05419    165.80426    203.04384   -246.49453   -104.14604   -217.36332     -269.324   -298.48591   -519.02226   -481.15701   -227.14604   -240.37097   -486.29961   -293.92911    -454.2138   -245.23575     -195.411   -530.27846   -299.87935   -402.38317   -379.81797    -565.0178   -423.84101   -346.13113   -570.20205   -551.75678   -414.85862   -479.55199   -465.63808   -580.19578   -588.50086   -605.20161   -517.34605   -555.12199    -394.1107   -647.67424   -654.95709   -326.56197   -536.58004   -469.99134   -643.32274   -483.09245   -525.88192    -234.9611   -383.29554    -418.5827   -357.25206   -320.30103   -262.05793   -370.42579   -274.52884   -407.60423   -452.40928   -324.50466   -414.04615   -686.11706   -635.25068   -688.21496   -454.37498   -378.37035   -595.78822   -348.09384   -360.88054   -504.93009   -312.06137   -422.54316     -384.123   -432.82496   -399.61426   -317.56398   -425.77596   -392.32101   -421.94375    -385.5778   -427.91704    -401.3817   -501.27975    -566.6629   -442.13438   -392.06182   -574.49471   -277.48292   -276.06928    -335.9901   -182.46177   -238.29404    -396.0872   -497.54072     26.64036   -169.72353    -396.3363   -288.52192    -27.05054   -111.21626   -188.64694     -2.33175   -224.52264   -137.03452   -268.67203    -52.35573   -381.98702   -249.04592   -208.88287   -249.02376    -412.8105   -315.01904   -359.44479   -358.42745   -365.70193   -298.10252   -349.81416   -368.81298   -397.18115   -127.19273   -501.72804   -296.88887    -183.6802   -439.34903   -478.82357    -416.1187   -443.66422    -424.6248   -517.54183   -604.20538   -468.49245   -609.27661   -517.64032    -493.9391   -581.51417   -399.85847   -582.82558   -680.75359   -434.12059   -653.89484   -556.98776   -849.07616   -611.40181   -582.84177   -526.28114   -485.51599    -675.8333   -201.34009    -379.0391   -738.12182   -285.57718   -719.89059    -691.1199   -527.36449   -517.28707   -700.66809    -84.58439   -489.56421   -418.53173   -787.55882    -536.9712   -594.07215   -472.23662   -847.59355   -731.75479   -529.88318   -635.02813    -993.7823  -1013.67568   -828.67567  -1026.96446  -1025.19033   -992.69409    -943.7685  -1048.68041   -931.21556  -1018.69831   -1129.3923    -853.2988  -1239.54195  -1117.46424  -1106.69018    -982.2456  -1059.99792  -1138.86848   -1198.1555  -1293.54743  -1333.84561  -1084.59751  -1068.52376  -1295.46417  -1357.88135  -1112.79145  -1202.49465  -1022.25793  -1396.45177  -1415.92015  -1319.20674  -1324.43564  -1541.46426  -1175.72395  -1387.87513   -1188.7434  -1322.01664  -1421.33977  -1319.64983    -922.1077  -1512.88301  -1451.38271   -1335.0997  -1213.99431  -1787.07163  -1107.16307  -1394.45032  -1665.60004  -1419.62005  -1214.47553  -1377.47404  -1595.15492    -727.6599   -540.70324  -1095.63746  -1153.85003  -1519.66402  -1480.25482   -505.47681  -1311.46073   -828.21457  -1116.60901  -1513.62003          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -2.50   -124.26159    -378.0484   -540.68711    -629.6652    108.55281   -603.34714   -600.84038   -358.61232   -383.30551   -354.63053   -240.15016   -409.88324   -382.75153   -308.82741   -220.64902   -297.71721    -597.5064    -296.2907   -394.73666   -439.82442   -291.48307   -179.17399   -108.22181    -84.55118     35.44998     724.8443   3176.93109   4580.90235    4076.7805   1512.17692    330.45426   -313.62641   -274.26824   -350.65377   -358.51246   -180.37815   -225.66111   -215.80759   -417.16258   -372.98491   -201.92715   -228.52594   -334.03378   -357.77051   -293.50972   -397.02033   -294.98076   -314.24324   -237.96803   -435.65692   -368.21341   -360.50922   -556.15312   -421.25101   -472.76766    -230.2185   -269.73203   -343.74857   -154.01713   -169.90993   -183.08132    -72.71543   -207.62992   -219.33099   -282.59717   -278.02408   -222.30978   -251.04412   -201.50163   -125.62522    793.85168   1614.84912    2204.0108   2152.86352   1173.50251    553.79987    258.95556        -3.66   -328.86863     -2.38192    -234.8838   -137.77237   -334.09504   -184.66715   -307.70291    -314.6134   -180.96734    119.95419     20.23711    341.86205   1429.32783   2597.35896   2321.81427   2216.87073   1063.39745    323.64265    -89.03617   -107.22774   -358.04313   -220.20721    -81.74987    -36.76298    115.24114      -28.622     76.92957   -180.64242   -360.57419    -337.6913   -326.48164    -502.7057   -413.74474   -424.72696   -362.55526   -371.30775   -278.37083   -339.55523   -353.53899   -365.91679   -304.59242   -250.47088   -236.84227   -391.43927   -133.27327     18.79964    190.44112    527.59294    466.56276    609.64935    441.28781    102.22376    -62.90664   -208.00593   -344.29264   -334.21761   -326.61487   -113.01709    272.63401    942.44049   1572.99273   2169.99264   2388.94571   2146.77516    1453.1156    986.61921    306.93939    244.43777     37.13964     52.00066    178.85016    -92.68041   -178.59415   -201.06637   -353.09716   -389.48759   -364.21814   -393.48296    -314.4181   -368.52345   -340.50837    -188.8178   -231.94018   -345.56562   -231.41317   -397.67005    159.92823    -26.64539    -97.77143    -37.87832    -52.32639   -323.86284   -247.08751   -182.20708    -418.6248   -292.43441    -43.83064     49.77687    115.09007    301.47144    253.70644     77.34168     66.40896    -76.96098   -164.24251   -158.52208   -314.81535   -338.47734    -77.62759   -228.11115    -355.3272   -357.98312    -413.7825   -323.34127   -322.16272   -150.16673     -76.9345   -192.22999   -113.35729    -39.12074    105.03735     167.1855    144.88894    199.81547     26.03356    -11.02423    -90.42926   -211.54264    -73.95911    -55.30279    -92.41523     10.48307   -282.24671   -203.65178   -235.89995    -137.1946   -107.28521   -251.10326    -339.5073    -289.8636   -344.29037   -368.40055    -343.3262   -323.75247   -553.72116   -319.40359   -185.44133   -265.43826    -389.9957   -410.05212   -102.54458   -143.41076    -39.93699     14.26998    -28.24729   -183.06853    -33.82975   -111.35794    162.78781     15.57152     16.13816    337.31013    199.75584     -4.32434    -49.85178   -202.98068   -188.84669   -196.13884    -64.95472   -181.49801   -297.28716   -409.63962   -386.03249   -479.91195    -319.6726   -189.24682   -206.56455     -245.983   -160.87434    -32.39832    -30.98132    -84.00496     98.62062    135.51156    443.39745    443.02587     456.7251    346.50805    500.37729    372.26267     47.10414    174.31737     56.07238   -242.19153   -129.99878    -332.7712   -255.15032   -285.33665   -409.38332   -287.25998   -295.73138   -345.21484   -340.06108   -267.69078   -107.99461   -480.19773   -189.03415   -156.03578   -349.68291   -125.47203   -342.81186    -574.6878   -349.29913   -470.28846   -336.88301    -530.0039   -557.44116   -639.73423   -433.75205   -684.25968   -518.67038   -620.03311   -435.93091   -460.76483   -474.03511   -370.59719   -411.97653   -376.52398   -361.69578   -465.70593   -446.45994   -506.72044   -497.19962   -383.72037   -208.27345   -180.12963   -296.23328    -370.3413   -159.56489    -182.5371    -431.2189    -69.76756   -261.67376   -322.51256   -258.28104   -233.13917   -502.78435   -392.24693   -367.58171   -363.87777   -440.83203   -541.06865   -383.12691   -348.75233   -300.77144   -385.02842   -375.24213   -391.55046   -333.94682   -393.96793   -311.49708   -340.39277   -311.76616   -237.23791   -309.32136   -386.34559   -282.85139   -382.19964   -490.05632   -499.29876   -231.52535   -213.07438   -441.30318   -414.51658   -290.08156   -286.71756      -34.569    -180.7378   -113.92955    -45.31742     151.6432    -89.49764      19.5422     59.95682     42.64523    -96.23055    -225.8116   -191.79668     -27.5752   -150.59926   -134.74071   -137.03345   -115.11381   -164.53985   -144.81915   -419.57795   -392.45247   -268.41522   -188.19508   -391.71582   -224.42954   -112.22678   -180.38284   -297.43542   -245.24328    -169.4427   -392.82832   -298.56624    -229.5916   -267.67919   -110.18999   -582.17432   -557.16783    -418.4106   -384.80495   -508.16704   -410.31961   -366.21291   -626.50345   -399.58855   -337.94559   -377.89197   -426.92901   -390.68693   -529.93951   -310.90867   -313.08854   -566.47032   -589.09843   -612.67562   -555.81733    -309.2085    -385.7071   -509.68624   -212.73593   -449.90427   -476.20987   -494.92691   -220.82012   -649.82248   -428.87174   -205.27006   -625.21087   -491.87234   -859.30983   -793.87991   -641.84257   -826.50465    -625.1482   -644.31015   -457.40559    -708.4456   -892.15819   -849.06018   -954.56098   -871.05798   -852.10078  -1123.22769  -1051.38593  -1106.66159   -807.02054   -706.61001  -1001.04427  -1295.47618  -1139.91282  -1151.22372   -852.29599  -1325.11735  -1255.25881  -1141.64576  -1115.73502  -1317.04246  -1230.93587   -831.55142  -1231.52336  -1279.72719   -884.37105  -1391.91254  -1355.08465  -1296.33858  -1316.67497  -1194.49459  -1385.42907   -960.41117  -1064.41661  -1531.54462  -1318.11803  -1079.16788  -1270.01847   -957.63808   -831.27447  -1179.20864  -1570.45159  -1419.94312  -1461.34884   -843.81804   -917.97017   -1011.0548  -1261.97749  -1558.37788   -997.18507  -1533.70202  -1052.29718  -1195.40952  -1025.37822  -1203.27723  -1258.74449   -948.18804  -1389.31718  -1362.12306  -2085.81525  -1876.67477  -1138.32338          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -2.00   -272.16486   -493.35681   -420.94706   -508.88955   -115.39689   -547.41333   -543.32768   -294.15707   -303.83189   -353.26661   -187.25442   -246.99443   -313.83963   -358.15839   -249.39964    -379.2866    -535.8358   -374.17833   -359.35933   -413.70721   -195.18715   -210.62547    -123.2168    -46.39396      94.2178   1112.19316    3553.3468   4615.67319   3634.16394   1538.00521    237.53429   -265.62365    -201.4635   -346.92971   -363.62338   -108.83398   -430.57342   -289.44539   -101.95138   -136.91669   -617.97986   -338.21809   -462.96326   -356.22252   -344.02155   -346.95878   -189.00496   -347.71376   -300.65632   -341.17093   -205.69449   -476.35719   -380.89188   -262.89608   -277.44694   -218.39168   -183.13103   -253.05011   -215.87777   -173.56542    -297.6937   -165.31118   -278.78518   -352.35653   -323.28026   -277.88518   -268.23347    -299.6159   -118.43221     35.03139    720.30217   1438.10895   2410.89127   2095.48739   1207.12902    495.46023    134.63771    -95.05859    -112.0435   -292.95367   -171.15189   -169.20455   -314.48305   -193.61421   -131.05267     -98.6054   -281.95996   -230.92041   -116.64843     663.6182   1532.92043   2472.64203   2626.05728   2167.60844   1509.04211    321.86047   -159.28665   -125.53415    -457.7554    -50.78325   -202.01135    106.33581     248.5697     55.40424    -88.95449    -68.66921   -188.41809   -255.84937   -410.37859   -370.89288   -382.62248   -383.94148   -404.87868   -238.93864   -326.44342   -164.41166    -218.9248   -255.25401   -219.29549   -228.78755   -305.02021   -256.26598   -194.59949   -184.46608    167.91951     703.5396    846.00818    632.70671    410.50698    -65.97574     53.27538   -367.91443   -267.08751   -223.71647    -374.3459   -112.48102    337.25938   1046.74616   1758.76435      2195.37   2408.96612   1817.44994    1328.6708    840.60104    191.98789     50.25805     70.20536    -34.65063    117.14101     94.73597    -198.6971   -139.56892   -312.23351   -205.32533   -299.98401   -268.89638   -370.43438    -373.5083   -397.57738   -303.13807   -322.25445   -470.67367   -227.97827    -62.37637   -119.75694   -250.32267    113.52226   -180.40878   -114.58439   -131.74102   -275.53599   -382.08631   -358.46232   -125.98401      12.6252     80.59163     95.20879    172.09845    239.46293     15.01303     125.6076    -153.8583   -353.67526   -266.51137   -139.39575   -393.89711   -248.65436   -449.38857   -368.62495   -476.68694   -272.16291   -364.83119   -293.03686   -258.86435   -300.36698   -176.35444    -13.66037    -83.04228    214.17105     14.05178    166.86827    240.92721    341.73975     35.50426    -72.99448    -36.32557    -20.91178   -110.43415     37.86418    119.22411     44.20315     62.71463    -140.1135      3.41837   -276.37001   -327.45646   -263.11644   -295.89237   -517.86492   -526.85684   -272.16513   -396.11698   -133.02539   -363.52182   -321.07608   -164.26028   -123.70209    -55.25596    -40.26877    -12.79259    -11.83158   -150.98866     21.31111    -22.36791     68.12698     83.16247    -11.77029    133.96576     292.2111    120.23498     54.79577    -22.59654    105.88008    180.73593   -118.62764   -147.95029   -360.66003   -134.21887   -442.47979   -388.38888   -220.82357   -129.25696   -350.52451   -313.32914    -93.36223   -236.93404   -154.16947    -46.12021   -110.67359     -21.9409    184.10547    307.19256    281.35112    536.34373    449.19249    667.82607    537.21686    497.03241    478.09323    111.41306     23.46469    101.46645    -196.2517    -95.53499   -294.85748   -274.05055   -243.60843   -429.51504   -415.63011    -212.7038   -282.43578   -140.20373    -45.80205   -192.88802   -109.37714   -209.58217   -251.54856   -280.20627   -279.47322   -206.60466   -340.86694   -382.95064   -448.68432   -541.84281   -457.45977   -421.09538   -542.42043   -447.72642   -326.23392   -513.56604   -273.96925   -475.29297    -446.9816   -362.39567   -452.01069   -532.56116   -429.00991   -528.49382   -324.12754   -447.47696   -186.35823   -267.80481   -267.80287   -274.22227   -172.37446   -271.22691   -243.16403   -189.44263   -264.67476   -265.88132   -179.04311   -269.38407   -457.18265   -373.20281   -384.45506   -284.72409   -421.07087   -380.78704    -441.2126   -252.41365   -288.27488   -488.60679   -334.46764   -234.81816    -15.24866   -357.43985   -134.40506   -322.58829   -356.98128   -170.29978   -464.85736    -84.36673   -331.98178   -366.33289   -455.94849   -334.08946   -499.86866   -287.51158    -350.5826   -404.04931   -269.55768   -394.85259   -222.85047   -205.14173   -282.17072     -25.8767    -74.57781    -96.75438    -21.38269     41.27203   -191.08452    -52.70855     62.08955     69.50585    -73.74848   -187.35209     57.27548   -115.94565     -49.7424   -219.83459   -382.80281   -195.07811   -346.08123     -53.5594    -152.7383    -299.7981    -42.66288     -57.4735   -291.29747    -218.5622    -87.74719   -397.69368    -402.4456   -332.23013   -281.93523   -251.62671   -352.04038   -273.11229   -537.05815   -238.91877   -260.61369    -70.80553   -444.08361   -357.02953    -123.2507   -423.12198   -523.56449   -544.22506   -487.62655   -566.45429    -319.8353   -181.33181   -543.92618    -67.83962   -197.93556   -497.13975   -241.56177   -435.61831   -468.68189   -518.88523   -280.20139   -394.10001    -431.3471   -520.21859   -287.35719   -559.90251   -657.01626   -366.95738   -409.54459   -733.25044   -480.00393   -714.03961   -525.20348   -388.84348    -617.6525   -602.27341   -373.02172   -775.64274   -326.53051   -633.21972   -741.79134   -699.46449  -1023.83241   -397.69085   -841.37898  -1062.12936   -833.91743   -798.60866   -579.04263   -666.25948  -1207.54375  -1344.87306   -842.89708  -1151.96274   -975.28033  -1112.92092  -1218.15605  -1189.87738  -1494.26699  -1008.52244  -1460.31277   -833.31178  -1346.55382  -1113.92899  -1449.69654   -1258.2361  -1152.55347   -827.36235  -1246.77962  -1101.69622  -1478.82032   -1373.0453  -1220.58007  -1136.60643  -1300.31327  -1286.87319  -1556.98734   -889.98932  -1523.38143  -1466.73114  -1333.07811  -1176.34151  -1505.24767  -1206.38567  -1432.36156  -1244.21087    -761.7343  -1554.71858  -1346.58935  -1640.80537  -1357.74396  -1505.09363   -846.10159  -1176.72081  -1206.92054   -627.75034  -1100.01678  -1795.30809   -1061.8495  -1370.87044   -850.40467          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -1.50   -365.50074   -589.79663   -298.42114   -395.80958   -336.77826   -489.61178   -394.05707   -274.03742   -217.94847    -361.1698   -159.12329   -194.83807   -336.96917   -271.82677   -247.27831   -392.40088   -371.14985   -385.49116   -356.46599   -344.92494   -219.65523   -422.74758   -218.46015   -195.87757    -62.59538   1161.00164   3419.51188   4429.93526   3665.86136   1616.43584    209.24425   -221.00292   -101.00707   -285.34267   -399.18853   -108.26233   -408.04762   -393.16871   -338.37624   -457.48969    -282.6061   -336.55591   -304.85335   -486.59083   -473.15257   -247.05028   -342.27817   -450.73971    -354.0662   -354.07391   -374.57447   -453.80688    -421.0606   -377.75104   -318.90389   -348.78384   -409.95543   -313.78634   -391.12493   -261.33002   -277.35169   -165.22824   -345.92574   -206.69158   -378.02988   -307.76421   -307.10407   -301.17804   -295.98599     19.21913    565.03464   1769.92024   2507.86073   2051.12186   1370.51375    514.78972    142.61704     157.8414    -65.20154   -173.28576   -212.75991   -115.50401   -236.88342   -209.94692   -303.82966   -282.38918   -180.08776    -116.6351    -93.28256    630.84392   1461.21181    2561.4596   2771.97428   2230.90374   1173.51669    439.89774     72.31787   -112.37884   -378.10817   -213.89738   -171.66712    427.28227    267.26278    257.39307    -79.63592   -144.16346   -271.44736   -372.54444   -185.68739    -374.8233   -468.82162   -416.55554   -272.52091   -404.80245   -355.29682   -271.04795   -242.84171   -263.22201   -369.27563    -131.1197   -295.47036   -160.75606   -235.29463     -3.05314    279.39539    584.02027    820.82208    668.35328    325.74862    263.33408      2.01971   -195.20443   -167.85167   -305.67361    -11.25317    -21.49978    423.71392    1049.8861   1851.75517   2725.92619   2646.20017   2232.10486   1479.71571    741.01153    394.27099    157.30009      4.80626    109.62367   -111.82421    -57.01851     -106.574   -256.69007   -198.03322   -331.48668   -304.22805   -412.15241   -387.05088   -348.82189    -362.2531   -386.23407   -235.57269   -233.57482   -229.94299    -276.7538     50.25157    117.08919    147.68452      2.61209    -92.86146    -319.8066   -315.18948   -335.44536   -109.18678   -333.36743    -49.13123     12.65488     55.01904    203.84075      238.294    170.37215    -37.99797    -13.05754    -57.84247   -166.11557    -94.42012   -174.74817    -59.52905    -284.6354   -386.45179   -338.90387    -426.6806   -198.78685   -359.14313   -302.77177   -291.16264   -112.49405    -92.05375      2.26396    160.74682    103.03556    179.09412     208.7551     66.21945      6.16284     26.78669     50.86396    -66.33593     -9.22163    -44.95909     43.20236    179.29675     11.66016      8.44796    -70.03055    -49.73799   -367.43079    -368.4226   -476.67084   -291.63661   -405.46956    -341.3273   -301.71871   -169.00982   -406.25144   -255.46095   -222.34318   -129.19996   -126.29104     31.61292     77.58363     -18.8402      33.6001     76.20403    -90.69407    -52.88644    225.48486     216.9212     77.51292    368.17042    280.26557    176.48051     58.35978    181.08155   -123.31845   -144.88086    -231.3087   -396.15526   -262.15217   -235.00763   -397.52475   -202.86566   -343.34014   -376.38816   -182.91576    -242.9528   -315.28108   -144.96056   -267.75058    -112.1124     -8.56658    162.35976    210.73964    211.17225    494.44967    521.58294    497.77244    480.14656    507.88878    486.99856    247.38634       9.0789     60.83324     -3.49581   -192.72764   -229.35354   -261.93925   -300.75161   -399.90387   -186.93367    -226.9459   -109.90305    -290.4734   -225.36691    -72.46861   -124.94905   -362.54858   -260.96713   -112.10952   -232.35085   -203.69291   -348.37504   -431.33771   -232.02387   -355.97879   -357.66194    -450.5347   -412.77107   -400.77336   -608.38408   -432.82839   -377.39262   -341.91723   -420.16131   -468.68392   -310.31863    -396.9846   -328.38026   -221.58242   -322.10363   -263.42696   -329.23423   -184.79837   -191.87113    -70.75435    -29.18755   -337.17511   -123.88767   -178.06028   -173.19987   -227.76899   -110.14154    -34.21296   -388.55519   -257.83236   -351.25198   -254.65619   -487.56498   -378.03453   -429.12856   -354.03562   -318.83959   -292.15695   -210.00088   -133.97423   -118.31005   -226.79431    -11.49879   -132.95164   -308.30191   -168.58433   -157.41536   -291.66141    -264.4004    -293.9042   -428.64375   -278.50935   -173.44744   -411.28822   -477.12158   -183.99496   -245.60841    -23.95947    -55.42201    -91.86441   -118.49977   -194.24384    -32.63412     64.87737     218.7698     54.70257       5.2484     -5.03683     -31.6408    -80.69353     164.6106     91.77647    -57.03145   -128.21653    -65.97534    -45.72746    -27.86276   -206.58855    -47.10824    -23.48694    -134.6604   -291.80548   -238.62498    -353.9629   -185.01221   -223.30107    -194.8648   -246.28359   -263.12283     -314.303   -297.87772   -495.14111    -10.89382   -121.32437    -243.0058   -162.29269   -461.95566   -258.71184   -443.96494   -499.95819   -462.94646   -322.94855   -250.26528   -476.51093   -387.34579   -342.41377    -370.3381   -231.87366   -340.94041   -550.07818   -401.62237   -326.65207   -344.00419   -318.37407   -407.76018   -414.08617    -346.6291   -268.51563   -301.95841   -411.84316   -602.58095   -244.11829   -612.20973   -501.93608   -502.23116   -432.14536   -443.83033   -590.91671    -392.5919   -810.85723   -323.97441   -527.89085   -575.37451   -891.99137   -328.22943   -507.85659   -640.69683   -761.50931   -773.56168   -617.50937   -324.39712  -1056.31235   -757.23261   -662.18842   -999.32495  -1003.25799   -554.35866   -920.18154  -1093.02622   -1387.1896  -1116.00895  -1050.96671    -769.2854   -1226.8233  -1142.96502  -1180.84274  -1141.10457  -1299.60178   -892.51902  -1128.34082  -1163.06453  -1216.56554   -1104.1635  -1170.76844  -1338.29805  -1437.69907   -755.09721   -989.71602  -1234.73431  -1223.47492  -1206.44248   -905.35267  -1108.87732  -1059.18956  -1167.05212  -1186.87377  -1072.21722  -1548.04065  -1157.86514  -1041.76677   -1392.3177  -1533.88141   -755.25638  -1450.62079   -918.28125  -1508.30054  -1299.05233   -529.24974   -659.89337    -1415.104  -1460.30589  -1720.89876  -1349.73262   -1098.7281   -831.46517  -1467.56792   -1418.3975          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -1.00   -275.88118   -427.91241   -211.05558   -349.57116   -325.35499   -358.32529    -312.9737   -282.23702   -188.43223   -365.26191   -140.39857   -168.00678     -380.839   -130.58997    -227.9045    -333.5464    -45.19707   -228.43048   -417.60469   -382.58274   -268.70418   -290.40054   -294.07743   -225.09364      3.28538   1307.34918   3569.37304   4389.09881   3573.18458   1551.46232    218.58226   -190.16573   -158.68257   -393.89428   -378.23144   -156.47108   -451.89527   -409.83111   -252.05857     -304.327   -452.00603   -550.76221    -327.7924   -365.94889   -359.05278   -296.17791   -308.68146   -369.77258   -286.45196    -149.0679   -421.95769    -381.5794    -298.5119   -110.59963   -316.91498   -349.93885   -327.93527   -310.43275   -128.58959   -222.38661   -336.99904   -199.32403   -410.48353   -382.06722   -352.44266   -397.56237   -225.17164   -334.85079   -187.67541     26.10805    796.73465   1735.99366   2450.31108   2120.33407   1405.58054    633.36668    363.36841     89.29045   -195.56506    -97.61091   -203.65356   -162.96112     15.39368   -187.28436   -243.12647   -301.09328   -205.66123   -199.50044    -54.48295    493.71221   1448.69888   2647.51677   3029.73319   2311.17244   1294.68125    416.35927   -129.77471   -491.67542   -475.85639   -322.52966     37.87818    393.82323    292.40272    158.93237     54.17525      40.7602   -296.18191   -283.65137   -373.91502   -133.93872   -251.97967   -445.28607   -385.67775   -381.18146   -364.68108   -271.50009    -246.7074    -58.44699   -241.67445   -304.11293   -187.66527   -302.30164   -143.92281     82.01899    306.07409    679.62297    854.99998    826.93601    595.80317    267.08003      200.779   -204.53479    -37.26692    -199.6173   -127.52105    -55.58596    531.58065   1383.38175   1870.36331   2642.75016   2775.82001   2313.14614   1662.97507    855.87122    385.75818    266.70762     38.88366     26.30547      42.6424     21.57953     -6.05553   -299.60327   -289.32993   -362.49447   -412.68268   -224.13598    -324.8298   -326.46496   -160.66258   -234.08534   -238.86292   -232.53085   -202.05344   -224.18913   -129.07854     -4.49272    217.48046     31.56401   -162.91246    -66.21122   -154.85882   -152.07818   -323.60416   -152.01875      3.96981    -25.81982    221.03224    293.58566     71.10058    122.01806     18.18989    -82.03223   -154.60194    -60.60936   -145.83407   -192.26506    -60.61531   -295.10686   -418.73497   -401.76307   -253.89928   -397.02296   -197.58651   -181.87509   -210.80987    -96.87483    124.85048    274.17097    184.14026    179.95254    229.63887    229.64857     359.0514   -115.77689     -0.93733     39.52164    151.82454     66.79106    -38.02708     36.31823     148.3885    152.83749    130.27937    -90.80573     -36.5953   -313.34398   -312.87063   -208.46345    -230.3341   -401.33393   -262.96445   -487.82247   -359.83553   -255.10866    -93.87575   -114.90883   -219.38485    -70.07272    -18.46087    101.45236     67.12025    118.65199     76.71949     68.98295    -15.23284     65.24746    198.73851    271.54276    211.78888    102.36377    112.16131    130.42901     228.6458    263.31734   -206.03359   -179.50545   -348.52589   -221.04632   -297.97627   -242.63709   -509.97339    -296.5014   -314.44389   -256.07526   -156.64741    -52.70902     -30.8461    -63.94475     22.62988     65.08599    220.81327    288.70168    277.32333    562.58875     575.8098    487.55717    547.81028    611.52479    248.33994    338.22752    152.13412      20.9431    -34.94921   -137.87725   -122.22584   -169.58268   -338.46669   -127.83811    -68.45383   -329.69936    -72.30088     33.38468   -158.77316   -209.98983      3.88738   -133.54633   -159.59047   -241.62714   -252.63593   -412.28825   -127.31236   -289.55609   -328.53884   -384.07422   -360.14196   -588.24773   -377.58034   -345.98309   -406.06631   -365.84643   -413.35772   -366.87128   -345.54052   -376.04547   -412.44115   -159.81637    -312.5923   -236.05476   -350.29245   -366.90645    -258.4656   -109.06468    -39.12235   -111.21528   -228.31635    -94.66492   -101.07741   -155.77523    -25.18223   -214.97001   -198.76695   -242.44246   -316.59544   -502.20143   -354.51589   -223.35397   -314.87725   -348.81057   -287.93579    -174.0812   -111.46138    -16.59472    -60.29278   -168.55243   -226.12319      1.25776   -183.47436   -157.48335   -190.16095   -188.10667   -161.89772   -294.42943   -206.89091    -194.6196     19.24377   -215.95504    -98.84239    -63.73304   -151.47041   -167.98521   -126.18531     29.91427   -140.18384   -233.99249     -3.97056     98.88053    -96.22725    138.05623      57.5786     33.26005    373.14848    -34.53737     75.01967     118.2092    -39.20963     -8.73857    -80.66258    -18.41878      4.59933   -211.03273     88.49201   -148.83469     13.03355    -415.5051    -26.95011   -151.72516     48.11921   -133.36532   -271.93225   -284.78718    -33.93891   -202.01846   -204.88219     -38.8935   -269.75377   -183.49168   -511.49722   -325.52955   -136.64398   -210.95773   -335.67017   -452.94454   -242.10063   -367.10614   -200.10412   -476.86968    -15.82242   -181.29912   -415.83204   -406.29447    -450.4825   -207.72937   -189.16937   -621.36692   -517.57286   -229.01469   -528.96058    -154.4713   -132.59903     34.68446   -238.70081   -428.30874   -335.62879   -348.60333   -267.59726   -725.48923   -399.25247   -364.66542   -442.73096   -553.28165   -314.91417   -333.18568   -329.90178   -515.10511    -492.0484   -490.67186   -407.73463   -260.91837   -320.42749    -583.0267   -744.97455   -725.34432   -795.68053   -801.58127   -659.72229   -934.73905   -966.76675  -1132.17187   -857.32175  -1204.47004   -696.49638  -1046.05976   -706.38786  -1126.04266  -1074.89757   -770.58626   -771.59927  -1375.69906   -907.57634  -1073.92712  -1259.09669  -1415.27612  -1526.01993  -1625.68202  -1067.21968   -1299.2859   -826.95298   -923.81748  -1502.41181  -1259.57162  -1278.28677   -984.21174  -1172.99972  -1098.94953   -1025.4389   -910.72917  -1497.39576  -1054.58396  -1234.86319  -1024.83219  -1384.85507  -1549.22237  -1105.44219   -948.15904  -1632.54758   -732.09382  -1327.25512  -1316.42423  -1294.02299  -1136.51264   -935.91342  -1430.39147  -1040.82807   -951.84054  -1424.96535   -958.98853   -998.51695   -754.70535   -362.83413  -1137.31469   -392.52038          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -0.50   -185.04851    -265.0281   -127.92038   -319.29108   -208.92209   -173.13495    -308.5591   -326.09367   -279.70412   -365.68653   -193.57811   -277.14752   -245.50738   -318.54857   -362.94451   -394.33306   -169.06453   -383.27847   -352.43698   -323.68184   -350.28079   -278.41179   -261.72574   -185.59266    -43.02225   1328.68406   3659.05278   4444.43398    3497.0109   1334.42046    212.41862   -171.42435   -152.25041   -466.17783   -290.92927    -87.47273   -309.46224    -219.9801   -281.35598   -220.49705   -282.57868   -236.67261   -306.57372   -265.50859   -333.77058   -473.44416    -599.7105   -136.32561   -319.54852   -422.19617   -254.27577   -401.70343   -346.02617   -315.89995   -309.44821   -367.16104    -240.4972   -304.39054   -382.39634   -251.87925   -238.55425   -302.16252   -281.38135   -440.97519   -452.05859   -245.02913    -324.1976   -420.55304   -303.43954     36.71878    726.32311   1761.12229   2534.56616   2143.44902   1350.52021    597.71673       71.845     51.17399     24.50251   -131.76836   -323.22096   -254.87969     -243.005   -224.10032   -228.96206   -216.44708   -306.23109    -97.56589     51.03753    604.49024   1454.86705   2650.96876   2946.53875   2344.81526   1279.17587    265.17052   -152.24658   -387.32247   -321.79537   -150.36811    -40.75447    141.03479    354.32208    157.02873    204.59271    -90.97652   -324.03783   -229.71923    -184.4659   -294.73104   -319.95215   -183.50621   -403.96807   -339.83162   -120.39594   -405.82525   -214.93723   -176.79682   -217.33429   -232.63031   -170.13597   -276.44045   -136.93171      35.5975      390.069     667.4127    938.65351    729.16852    609.88928    190.80141    -53.81666   -121.55269    -333.9885   -112.41327    -31.95955     35.28527    538.25943   1332.07768   2096.04314   2838.44533   2759.34104   2470.14195   1605.74279    963.41831    507.13183    354.97571    114.41549    144.68374     -0.72243    -64.73252   -126.93914   -333.86369   -282.88901   -334.87015   -309.09017   -275.35596   -300.47263   -375.43723   -106.36589   -285.68802   -397.04138   -147.97928   -133.33744   -303.58501    -48.13049     -10.8491    132.08187     55.70945     89.16463   -219.34595    -235.1162   -218.71097   -277.25956   -258.96021    -42.30497    121.37915    260.21522    382.36029    262.31405    302.01893    137.08033      41.1675   -125.57936    -73.36622   -131.31538   -130.75869   -245.82685   -218.86401   -263.04262   -431.67085   -300.14898   -308.19364    -387.3269   -204.91256   -166.79618   -236.87121    187.28051    118.54199    200.94935    244.59629    185.54669    215.18815     138.0707     24.91681     22.00689    -79.42626    -21.62091    108.90499    -26.25658    125.56343    213.82179    144.66113      30.4983     36.98498    -32.97543   -162.78979   -345.17841   -270.64321   -316.37743   -328.09817   -318.59167   -309.92595    -440.4831   -256.36739   -127.65897    -169.9589    -74.97173    -70.44557     55.75682     -73.3983     58.73953    180.22697     72.55519    127.97729    179.86203     76.95032    111.50995    302.86344    221.97159    425.93711    230.90477    303.06715     56.52591    -53.00133    -23.17363   -242.23962   -139.21471   -208.25273   -180.47915   -291.82921     25.59596   -231.86459   -222.82493   -300.98024     20.25456     94.92927     -4.27425    -23.30492     292.1488    144.50301     199.1884    548.37646    326.02467    665.36317     378.2761    758.92278    502.61655    524.34798    280.85402    236.95487    113.82934    102.54489    -67.70574    -78.15037   -221.92444   -136.16139   -223.26621    -93.10652   -178.86311    -50.05952    -161.1324      -87.462     -0.90873      -63.158   -101.43775   -115.82592    -204.5906   -198.85214   -187.75946   -277.17003   -377.17001   -342.14723   -535.03821   -246.11405   -422.32544   -335.97539   -414.51869    -371.5156   -360.24257   -357.66367   -442.57077   -359.13764   -388.61955   -530.17601    -379.2479   -424.60657   -315.83451   -398.38877   -229.90043   -209.77371   -178.46802   -203.46466   -143.33367    -87.10185   -151.75782    -179.8757    -59.42266    -48.98288   -135.38875   -152.64882   -148.76111   -207.71278   -120.52628   -361.34703   -272.81628   -269.15847   -168.53927   -187.81503   -217.99114   -185.94331   -165.39583   -231.91403   -191.07586   -111.15916    -81.42683    -91.20108   -186.56442    -72.11534   -286.14559    -95.42026    -65.96795    -36.41666   -164.90608   -109.38439   -154.91237    -307.3118   -176.13196   -274.07824    -73.13506   -360.40241   -114.73019   -234.72835   -115.53063     15.26787    -53.16587      7.17885    -41.36927      3.19476      2.78412     87.43766    121.74214     28.37224     23.90629    143.46607     90.87747     59.93759    -36.69913     -4.33982    -38.01596    -46.02503   -143.47807   -263.95882     -26.3233    -46.20069   -211.61669    -64.50096   -132.91417   -114.62729   -184.60397   -136.70566   -146.30935    -403.3447   -150.88446   -167.47024   -132.17072       7.2711    -78.11661   -242.12983   -282.51273   -488.67022   -387.96614   -265.51152     -61.3871   -178.44073    -156.0162   -363.71926   -225.08164   -448.87007   -562.15771   -428.57958   -341.99588   -266.00631   -196.82388     28.48356   -502.95093   -272.48685    -15.17729   -107.48817   -476.99965    -205.0812   -450.34398   -421.89958   -361.20315   -377.83993   -409.54263   -304.86716   -488.43881   -220.96424   -625.85777   -431.67041   -417.31917   -572.54141   -308.34922   -449.36306   -487.13728   -520.70005   -696.26088   -398.67324   -467.45104   -651.01702   -739.88116   -787.79881   -827.30671   -655.26226   -591.97758   -492.33868   -676.22731   -869.67479  -1066.24946   -874.61975   -838.01959   -932.51515  -1318.01321   -752.08607  -1207.66959   -901.36434   -992.36919   -927.45446  -1109.27879  -1244.73047   -876.31001   -915.99041  -1181.21246  -1004.27708  -1186.33607  -1233.13283  -1143.37672   -978.16946   -945.80984   -823.47389   -1510.6512  -1088.21506  -1088.12077  -1448.16134  -1114.74215   -814.41957  -1239.75127  -1231.30908  -1515.42081   -627.28991  -1152.34302  -1035.19643   -1205.9403   -905.01609  -1167.62275  -1283.06322  -1197.90717  -1147.09436   -737.06299   -1505.8402  -1179.71395  -1644.27692  -1127.02592  -1188.25474   -854.01656   -547.32073   -700.51398   -963.77147   -872.18354   -210.69891     -588.816          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        0.00   -181.81366   -186.03683   -111.89463   -323.41287   -168.36268   -101.33639   -310.66994   -357.57937    -353.2843   -342.46357   -234.01086   -347.56363   -214.06063   -455.28626   -435.02022   -435.92646   -358.37536   -586.62894   -239.69887   -245.41324   -428.19062   -323.32801   -162.18518    -89.57789   -161.95834   1148.26874   3442.17064   4476.76731   3853.07549   1235.47597    332.51538   -108.14073   -227.10929   -281.85776   -351.38684   -400.48486   -361.95304   -403.49045   -493.17021   -422.01689   -232.83857    -479.9557   -286.82534   -224.00706   -355.43573   -344.93285   -303.35797   -282.40994    -473.7121   -243.83432     -326.312   -271.25571   -462.89042   -457.71068   -513.10737    -201.2595   -385.03908   -269.41488   -239.30423   -151.16682   -246.84734   -240.87973    -293.6492   -200.83028   -266.53598   -242.96942   -271.72804   -307.95688   -101.98423    -53.09309    613.77114   1622.90301   2384.39955   2214.30618   1371.87053     510.8246    483.26855     20.60033     -153.546   -335.37851   -324.48404   -374.30622   -134.77598   -456.48733   -267.05218    -131.9575   -140.16455   -158.80439      82.3483    586.45079   1608.57685   2707.87219   3183.60589   2383.22612   1502.31391    418.20954   -120.77738   -308.07036   -148.98971   -417.07076   -210.44399    165.64498    334.08877    195.93613     60.92905   -137.52814   -253.82424   -338.43939   -284.67904   -248.21072   -181.60645   -167.89859   -177.69859   -327.89173   -409.44385   -229.45446   -286.93869   -222.11357   -182.54811   -250.24409   -311.09907   -454.22776      -125.97     58.89515    496.64571    831.32351    959.63958    779.50777    590.58127    355.62284    -18.65418     -60.7671   -131.28084   -200.49189    -73.37587     -0.78904    487.01215   1199.44938   2202.43617   2754.03738   2850.18282   2389.02736   1649.09742    919.13535    497.22468    230.64575     28.43224     47.48823    169.98552    170.82118      7.64978     -6.43374   -169.32518   -262.10618   -316.14519    -276.3891   -283.80053   -204.81378   -191.20614   -247.84069   -166.97732   -220.85981   -185.30924   -213.09367   -152.60273    162.25911    -30.96358     19.31719    -66.20969   -182.47299   -216.51063   -247.33611   -422.85788   -241.79104   -204.48918    103.82803    221.56913     331.4732     276.4028     58.67237    105.33027    -150.6673   -196.52619   -180.68011   -228.01343   -333.87563   -201.41397    -184.4474   -360.85073    -367.7039   -266.66227    -331.0691   -334.96744   -257.95103   -280.18034    -82.05698     92.37378    -69.61794    202.91394    228.19627    205.67648    258.84669    271.89053     12.16278     -78.5393     82.51963     55.89458    -57.35043     41.85867     213.5864    -79.19119    150.96249      -54.931    -12.81392   -197.85639   -238.68398   -236.40126   -356.78708   -277.45274   -245.20273   -420.21284   -379.56231   -410.56919   -119.71614    -58.89682    -82.23761    -19.18989    -88.88153    134.52579    -75.79538     18.55547    -68.25975    114.56108   -146.57813     36.17231    202.39987    308.78548    355.82378    428.89707    275.70761    214.07199    146.56564    108.86753   -110.48536   -171.12976     -51.2298   -189.22902   -234.73221   -153.75472   -255.49946   -208.25174   -408.84389   -190.25494   -184.70642   -132.67325    -37.15522    -69.47447    -34.86007     39.05546     63.75384     244.7792    654.70203    515.11353    510.31211    424.69152    584.71093    786.75946       640.39    629.50429     433.3996    196.26005    162.39232      98.3238     -5.51026    -84.84684     50.62692   -144.61619   -141.59046    -28.84302    -65.64652     11.48181    -37.75803   -180.88946     78.35299     52.90865    -66.67362   -120.70418   -181.01952   -120.34201   -218.19779   -409.51201   -250.78344   -230.88556   -385.40957   -451.78994   -331.28999    -483.7353   -428.63118   -513.41934   -574.37983   -293.49398   -434.54353   -450.89232   -197.29214   -277.69591   -157.83739   -216.75883    -273.4341    -209.5109    -67.38457   -226.92449    -79.77582    -52.54872    -91.81763   -103.86401   -250.87749    -85.33355   -129.27617     -1.74738    -41.35941   -164.11564   -113.25517   -200.63062    125.24665   -159.80688   -205.85461   -284.42724   -190.51024   -126.65516    -164.4606   -219.30537   -345.61216    -10.26071   -128.35971   -222.16176   -157.46334     -37.5561   -410.31157      2.39151    -67.70306     -4.68643   -199.78804   -124.91372     -4.52411   -162.31304   -192.71708    -194.6141    -243.3963   -118.29806   -160.18392    -53.26642    105.12134   -105.23164     -9.61797   -311.81127     74.94587     -9.96631     26.12606     73.05362    224.70249    254.09935     80.05281    205.18228    182.83819       -9.818    -38.44585    -30.78184    -30.19513     28.68884     70.72149   -183.19309    -26.47958   -161.19947   -301.62527   -169.75603    -73.16756     49.40132   -141.32741   -242.51497   -250.94662     50.22254   -207.15878   -209.77771   -151.82689   -123.64556   -208.16218   -233.09231    -115.4272   -244.80424   -271.08439   -286.43176   -200.61069   -420.24598   -260.56216   -265.01759   -402.71111   -289.32962   -434.35864   -435.67173   -139.13405   -198.21113   -348.24787   -229.88434   -360.64041   -424.54175   -105.78054   -602.51251   -415.12761   -366.52071   -297.51391   -236.99588   -204.66741   -273.98077   -224.28042   -197.02988   -156.28137    -469.6874   -265.73923    -25.08245   -358.97711   -371.76716   -138.38424   -626.77991   -335.29992   -657.20391    -252.9697    -705.6214   -930.17151   -692.39396   -404.40738   -979.44916   -494.16861   -731.98733   -602.77639   -856.28988   -723.70448   -835.96076  -1048.25338   -734.10903   -794.04525   -849.36082  -1100.78166   -942.31192   -796.60492  -1131.87724    -736.2588  -1165.06913    -748.0727   -936.04523   -993.34711   -752.37447  -1091.00719   -1090.1662  -1352.44976  -1189.26923  -1020.64934    -1283.284   -968.95305   -902.57765  -1038.79749   -930.14486   -816.92813   -1297.0029   -1094.2767  -1107.40094   -1078.1568   -807.18708  -1021.60455   -1213.8578  -1283.73279   -764.94825   -1320.5639  -1744.06937  -1159.62479  -1294.86926  -1166.01557   -932.43533  -1311.73498  -1095.54453  -1276.22638  -1028.25051  -1688.29712   -615.73359   -1462.5673  -1529.55214  -1329.21872  -1037.80766  -1448.81332   -1479.9963   -1288.8543  -1026.80976          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        0.50   -325.12151   -258.55938   -225.18556   -399.11666   -293.64479   -290.42273   -329.11688   -363.70398   -383.59031   -246.81534   -235.06511   -305.41227    -516.2964   -383.44952   -234.12864   -336.02801   -361.00107   -561.23509   -155.61895   -283.91793   -388.05574    -207.6387    -171.8573     82.51731     342.0198   1107.94097   3396.73776   4761.05094   3658.09997   1453.60798    260.97914   -281.45232   -113.81721   -135.82648    -330.5935   -244.84264    -53.71775   -313.08443   -434.12338   -373.82999   -157.79927   -305.60716   -311.26549   -384.89243     -383.191   -533.45827   -259.84749   -230.65309   -381.58279   -303.84782   -404.81901   -289.25492   -385.28303   -402.83734   -460.87077   -383.84437   -386.97132   -222.37638   -155.87548   -225.25207   -261.69113   -221.82826   -351.31608   -360.33512   -312.07579   -302.30631   -363.79832    -208.4811   -186.08754      67.2184    735.84324   1603.07863   2560.00717   2043.79733   1478.01192    750.98378    333.68663    188.80075    -96.37916   -224.48811   -308.77188   -289.54428    -59.45945    -391.5976   -388.39576   -135.95602   -303.66076   -347.09183    129.06536    479.02805   1768.21107   2765.44407   3154.68466   2294.22999   1272.09204    551.92018     25.26827   -331.24414   -382.55192   -212.30087     98.31673     44.65866    293.05487    206.96442     80.24152   -193.52919   -224.68333   -247.48704   -390.38415   -416.76587   -438.19891   -290.76497   -450.23244   -467.37927   -252.66197   -223.74377   -258.57656   -292.90191   -160.31238    -253.4177   -195.73468   -185.53923    -73.36673    131.65768    457.16423    730.24165    718.88724    755.57707    520.74615    429.50984      7.29257   -116.87805   -220.48026   -274.84678     -90.1625    -55.07065     467.4153   1236.05877   2195.82446   2918.88078   3030.08606   2401.01132   1722.71279    810.88354    271.40827    123.94042    128.41412    448.67922     130.3853    159.35057    -159.3689   -168.18849   -142.13349   -445.36536   -229.23281   -356.34994   -428.86595   -344.65685   -378.07025   -119.97741    -60.86082   -301.41416    -54.96219   -200.05318   -184.06624    -87.63658    113.05486    -26.69577    -213.7294     -109.467   -202.68508   -266.18507   -211.14982    -239.8126    -92.54443    243.02381     87.21322    250.52269     119.5874    237.60755    138.97646     45.13097   -158.45132    -76.40316   -129.21312   -223.28811   -172.15726    -254.5506    -296.9005   -267.45636   -236.51251    -244.5125   -309.48065   -228.65037   -214.70735   -196.24473     -8.18159    119.55922     116.7813    275.47717    148.42095     242.9238     61.77127     72.75421    -77.57212    -58.03853   -118.58743    -42.55357     90.13752    191.59432    234.30497    -43.48298     -5.77866     173.4103   -151.47552   -294.88698   -230.07119   -298.98647   -154.04883   -464.06997     -366.259   -360.41629   -350.41574   -344.18781   -118.72973   -103.44875    -71.79538   -101.88759   -103.10133    220.80933    149.23656    125.67625    -10.64354    127.84226      52.1755     131.4076    299.27283    257.01182    312.77117    390.56115    259.58044    154.33033     94.85729    145.40683    -37.18537   -254.90927    -190.8982   -329.78048    -321.3766   -337.61028   -244.61027   -256.84128   -102.95104    -123.4469   -281.73051      44.2655     68.77093    143.67395    184.50819    170.53551    318.24514    469.10905    519.47611    677.38778    646.06212    672.39566    615.23698    579.29057    477.56799    414.48514    238.79686     277.4738    -106.0561    -24.74544   -212.86926   -188.09172    -16.04721   -141.97711    -12.62046   -122.29405     76.72388     20.83876    -58.69837    -26.74764    -55.24091   -104.34031   -107.54576    -68.20622   -235.51614   -227.54977   -194.11905   -235.12257   -248.96869   -331.64651   -305.14652   -443.17214   -517.39779   -419.40333   -468.46058    -326.8523   -411.40292   -284.22156   -285.73592   -456.96018   -215.71928   -178.12297   -322.78564   -185.94659    -97.79375   -220.92915   -204.22636     -28.3857   -334.74117    -81.32157   -156.71239      -99.871   -129.82044    -41.62771    -54.04211    -56.77464   -131.77574    -80.79032    -36.57791   -107.99993   -285.68925   -135.86326    -50.22108   -149.72761   -211.63612   -217.76821   -218.03438    -88.91149     41.12365    -180.5806      3.86715    -97.20403   -155.49201    -60.09574   -260.59615    -41.95778   -269.39509   -230.00867    -49.02457    -69.15429     19.22205   -308.16964   -289.64796   -167.97794   -256.19626    -88.17998    -98.94959    115.39481    -70.65036     54.65181    -63.64937      9.33474    209.12522     39.19317     448.1265     59.12761    441.14655     233.5769    105.44158      9.53685    127.34474     66.76152   -151.13414     53.43447    -97.37112    238.29509    -77.15932    -56.10506   -341.25971    -23.59959   -227.15861    -42.50887    -75.42123    -24.32086   -154.27928   -121.11273   -198.62284    -119.4129    -17.12506    -79.45771   -125.73591   -442.63391   -627.60584     -1.65097    -170.1804   -184.26062   -213.67044   -340.46821   -305.55053   -403.48074   -178.57778   -442.26253   -452.88098   -125.11008   -380.10421    -58.88783   -162.62337   -157.89017   -137.30161   -309.44545   -338.87265   -141.88739   -315.24784   -360.55048   -220.07907   -349.93806   -204.95439   -405.10366    -361.4504     68.99553   -419.83638   -391.94561   -250.64186   -232.19344   -449.60269   -120.73152   -444.39323   -259.32303   -380.85549   -770.49155   -729.87866   -501.78881   -472.27999   -618.46072    -339.8666   -567.44866   -615.96952   -566.66964   -711.00882   -776.81784   -859.46767   -931.65544  -1038.12954   -777.50578   -864.62901    -579.6147   -896.24585   -806.03126   -739.99585   -746.96224   -695.33651  -1059.20514  -1032.93536   -538.35618   -952.28325  -1126.16221  -1262.41329  -1087.23347  -1144.68572   -973.32785  -1207.93516  -1162.85802     -894.586   -950.49994  -1211.76926  -1079.90431    -892.7802  -1122.48615  -1314.44325   -1039.9176   -891.18198  -1433.80476   -622.27045  -1115.53439   -883.49656  -1169.39182   -778.32391   -981.22165  -1196.32315  -1180.42744  -1207.84024  -1557.50506   -994.89819  -1072.99213  -1208.61903   -902.97682  -1167.22626  -1271.50551  -1015.68057   -922.77163  -1037.05934   -608.99806   -963.94137   -648.52975    -342.0032  -1116.64619   -293.85918          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        1.00   -468.04991   -330.97256   -339.96856   -478.33738   -419.52344   -474.62113   -338.91999   -365.62828   -392.77007    -251.3234    -271.8932   -296.71147   -582.39288   -333.00622   -175.75043   -270.73953   -346.31644    -338.8432   -142.69848   -265.33189   -209.02058   -134.46482   -231.26153    107.22271    149.33245   1309.42874   3242.79482   4935.25058   3785.46184   1631.04421    369.34838   -245.53392   -155.85516    -67.19484   -193.99428   -134.75141    -221.5859   -203.16027   -279.15411   -226.35767   -283.81643   -513.92096   -349.90643   -245.37021    -509.0392   -369.93008   -331.05117   -372.99098   -352.60341   -228.08431   -326.98954   -391.80057   -368.08535   -227.34563   -391.86859   -165.19597   -394.41935   -433.28559    -43.68951    -143.9968   -212.57766   -365.13421   -276.69182   -299.15024   -313.97445    -341.5279   -365.79001    -158.0064   -243.26179    -32.27366    784.03831    1619.2712   2411.48468   2186.63764   1384.52319    530.77836    246.38955   -144.75797   -349.66618   -383.82011   -428.39625   -519.32154   -356.45363   -302.20719   -296.60926   -292.95334   -191.51314   -116.57823      96.9129    515.86333   1704.52526   2767.47598    2860.8087   2614.51074    1420.6385    705.98738   -287.94556   -197.17936   -454.67331   -208.50787   -144.05217    134.50049    379.15815    120.23826       49.765   -234.22646   -256.21402   -196.38001   -281.67236    -302.3118   -247.02433   -286.87373   -335.59065   -347.34662   -358.73473   -306.57917   -192.22432    -113.1147   -228.45028    -129.8746   -272.94889   -226.97781   -218.18828     79.28835    433.65645    594.18424    842.43665    944.32708    664.90892    517.34562    122.74136     35.68458   -317.92154    -56.01387     -107.093    111.13018    391.17366    1354.3962   1969.87013   2673.61723   2972.86005   2535.28527   1519.10799    866.29329    285.67802    191.46089    116.28177    101.15442      82.9733    174.41557   -238.18316   -169.13656   -253.69962   -432.21024   -287.66848   -207.74232   -147.66433   -248.06396   -272.12244   -212.57101   -300.91135   -199.97467    -78.99349   -170.73907   -154.62876     77.84604    -33.40021      36.5381      5.99171   -146.89977   -256.99057   -304.75436   -283.20732   -172.43942   -104.38136    183.97643    271.57452    259.57063     243.0067    218.50857    164.75869     25.00177    -58.54098   -188.08574   -208.02082   -150.26706   -245.91405   -245.50098     -104.498   -280.62501   -339.52667    -256.7623   -322.03426   -293.02169   -297.55005    -157.7277     97.71947     -10.6471    234.47887    342.15788    304.50888    121.80226    213.38567    187.90708     35.21239     95.85424       1.9194     86.65121     87.15164    212.30851    -61.98746    185.01148    110.68363     54.28267    -96.73262   -194.57755   -439.94214   -141.13342   -333.34675   -397.58511   -277.70297   -341.64182   -317.31693   -239.13376   -232.61148   -239.29283   -100.09887     19.43443    -66.83828     30.24265    102.39432     32.39613    191.70012     55.42401      5.23604    -32.61625    248.71627    316.53399     299.8954    420.58497    251.75281    141.60335    -17.39327     10.33809   -144.49127   -370.92514   -215.27568   -298.78369   -331.30623   -214.86051   -135.37011   -274.15928   -174.03418   -162.56455    -80.27716   -184.58873     28.10264    164.45759     34.13581    164.22492    251.88013    330.85038    554.54838    588.02567    595.72491    718.99665    675.12376    680.57238    541.08762    426.93133    332.43317    182.19081     70.45761     -73.3918   -244.19779   -206.02897    -49.65439   -200.96184      22.4011   -127.32261     -8.36606     41.72648   -118.63024    -50.79832     -32.9846     39.51914      3.13299    -40.91784     -8.16499    -117.3585   -364.11458   -360.03811   -146.53237    -283.8064    -344.5297    -343.4503   -357.62123   -450.49896    -478.3436   -222.83402   -224.45435   -265.38265   -339.65747   -215.45223    -183.6381   -275.28239   -217.50704   -220.25064   -277.22973   -168.66158    -81.96659     27.02364     -73.4954    -18.65631    -85.93061     57.03139    -12.50786     28.11317    -88.97852     -42.3294   -221.02151   -166.05082   -241.46923   -230.93209   -154.01291   -193.67971    -222.9143   -158.45187   -206.72305   -136.59433   -119.40097   -131.75135   -129.09252    -96.10688     88.54639   -115.61858   -138.43331   -239.40739    -25.00492    -28.57926    -84.80817      9.06164   -249.06154    -160.8392    -204.6972   -245.35401   -143.49491    -95.09258   -362.42399   -197.12241    -84.12583    108.93553   -141.41611   -207.79844    100.25526    -44.92573      91.8854      196.042     13.92054    241.27294     114.8074     64.39264    168.04754     308.4121    -86.20579    -50.90083   -142.07078     89.78511    -18.65161   -179.46589    -318.0858     44.01614    175.22915   -113.19022    137.86233   -119.74256    100.31791   -168.22042    -73.05274   -102.46402    -75.75387    -40.68133      50.5265   -236.43602   -162.89392   -120.77309   -233.38565    -29.80515   -181.97356   -241.38847   -331.47484    -99.45781   -393.21259   -173.86889   -166.68603   -251.27495   -150.48283   -354.48025   -173.86647   -318.37145   -303.79238   -156.70723   -403.47382    -140.5192   -332.17385   -163.03573   -245.07793   -377.72288   -288.02805   -271.97274   -264.52763   -300.28328    -180.8877   -528.22939   -174.32767   -229.92872   -322.96872   -252.54703   -182.27664   -226.53709   -353.96131     -320.468   -362.04315   -469.26768   -664.45783   -339.29783   -583.08984   -661.50093   -596.81072    -529.2589    -625.7682   -805.54122   -789.45907   -646.97156   -777.93975   -824.11951   -852.52721   -804.95828   -498.34773   -816.79366   -728.38122  -1096.27172   -837.89325   -914.94843   -880.99631  -1194.16218   -995.98985   -861.98623   -979.78328  -1241.91533     -976.098  -1080.43748   -897.89843   -750.27634  -1031.14854  -1084.72224   -937.65977  -1308.71331   -744.02158   -794.51124  -1045.74466  -1048.56399  -1231.17208   -1036.2377  -1521.61826  -1141.82117    -489.5142   -821.48482   -992.42147   -865.56484  -1423.07894   -941.25184   -820.93631  -1100.28306   -810.41104  -1169.98252   -830.27091   -883.56274    -1299.215  -1058.17032  -1712.32739   -994.81526  -1033.01653   -809.41544   -634.23927   -807.31941  -1288.50996   -171.51976  -1261.20192  -1405.21052   -1448.8061          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        1.50   -406.21967   -334.87912   -394.97295   -379.33985   -306.37986   -446.42861    -245.4133   -347.70642   -333.75593   -496.48367   -378.58542   -341.46629   -362.72052   -307.41613   -242.48514   -225.16959   -329.63968    -70.24805   -147.59475   -232.44639   -142.55184   -176.50628   -285.54607      1.43019     83.59396   1354.99447   3355.64997   4720.49666   3876.60311   1573.54287    232.07554   -106.40031   -245.67068   -173.71991   -406.83051   -346.42365    -166.2698   -142.96816   -495.15235   -447.41075   -304.38251   -395.16799   -429.27994   -315.89358   -276.60823   -502.96309   -376.80953   -192.19835   -122.51397   -503.05504   -390.25637   -433.06713   -281.09565   -357.65737    -350.4987   -437.21155     -265.842   -211.90879   -211.01927    -99.96587   -169.41794   -263.55307   -272.10746   -333.58596   -288.06852   -357.34176   -194.51272   -233.16447   -115.65161    -56.42821    557.23852   1771.78262   2565.11605   1994.32547   1191.82707    445.34918    206.74948    230.89363   -202.02051   -494.90557   -489.49433   -649.36206    -256.9483    -205.8725   -320.10522   -325.19745   -318.51299     -84.3437    101.15256    738.68853   1697.12433   2975.00105   2958.96599    2389.0912   1041.20605    489.29718   -131.69836   -115.12601   -266.85009   -165.85932   -142.20331    235.35625    418.03157    222.17494    193.69864    -89.61871   -422.00487   -288.13226   -319.11067   -347.22631   -303.99213   -532.43282   -512.81669   -299.17768   -360.61406   -259.46711    -186.9418    -94.80833   -133.87278   -332.13544     -51.7391   -128.57487   -135.10613    112.79903     309.7182    599.46126    1028.9204   1017.12882    604.55003    365.84944    219.21703   -221.79291   -151.54873   -221.51272   -219.11214     41.40108    497.62592   1176.52838   2236.74158   2468.87614   2788.93389    2538.4625   1909.06735    917.61002    484.41934    298.88903    215.44531    180.88362     63.03497    -133.4256    -40.27661   -158.91707   -202.56877   -191.54354   -334.15386   -319.98944   -109.34461   -432.15306   -205.06309   -378.43373   -142.75476   -419.68143   -229.77016   -151.83664   -154.96638    -84.14463     17.09495    119.54886     62.23291   -176.17174   -180.26813   -101.70912   -129.33079   -131.70447    -18.26504     15.57336    185.99397    419.55113     144.0255    226.21904    -98.39816     -3.47522     -51.2098      8.32328    -110.9169   -154.25695   -191.51182   -188.59984   -456.48277   -337.55426   -350.25671   -346.44635   -247.72347   -200.16723   -186.41856    -61.80234    130.11162     14.90751       76.573    231.83676    228.42543    339.40251    177.47885      83.6458     -42.5874     72.16533    -20.45939     98.81553    154.31241     219.0849    258.46208      34.3771     44.02374   -123.93504   -286.79734   -223.24218   -296.72992   -311.78784   -252.71907   -394.93768    -403.4944   -302.71083   -227.80445   -193.88589   -234.76056   -159.51892    -84.97634    -22.13519    104.70437     72.08402    160.40207     140.7809    167.11367     43.37178    117.78765    114.43026    142.49694    188.95455    290.89917      249.634    333.28726    236.26398    151.95814    -17.88699    127.04555   -199.73055    -193.2826   -209.72701     15.85754   -223.99805   -215.46832   -294.76382   -221.64589   -146.71007     -18.4097    -67.95907    -11.43971     14.05977      71.4858     31.85474    192.38172    484.16575    423.02723    698.55443    558.91084     684.4103    639.41057    634.82672    598.27453    454.09584     343.4532     35.54101    287.36038     12.89618    127.80597   -136.78296     43.83939     26.94324    -69.74841    -35.46497     130.8537     74.56112    -86.00842    -17.21056    -57.55894   -173.71229     74.19593   -179.30037    -79.20496    -96.55334   -333.81065    -240.3807   -421.60456    -179.4838   -343.48701   -421.47645   -391.25862   -251.66281   -410.82031   -393.09999   -262.67552    -316.0288   -327.10609   -149.21161   -334.00096   -315.14165   -189.40355   -143.92829   -284.86643   -259.84324   -184.08907    -76.61984     61.59418    -22.40231    151.21662   -161.90782     68.87441    -10.33555   -144.85363   -257.60037    110.82599   -148.00266   -241.07218   -139.35702   -177.24414    -321.8018    -93.18284   -209.53826   -200.85242   -204.11459   -221.09326    -153.9391   -117.76932   -164.86654    -50.65599   -166.92082   -205.65646    -29.32645     44.02028   -103.11077     -7.15946   -247.04216   -108.77474   -216.47183    -60.05127   -177.67998   -124.93413   -188.87284     34.26075   -101.34469     -82.2045    -190.9475     -14.2529    109.48235    -67.28765    104.41952    282.86366    232.72505    206.21261    250.69697    219.75645    226.25464    218.15695    172.75108    223.60003     166.0553    -20.36686   -214.26855     19.11196     -2.06278   -127.77488    -122.2423    188.66189    -13.16874    -83.11859     45.51118   -261.00322   -157.60326   -241.68765   -210.62355     -8.15502   -102.20824    -206.6048      -19.821   -144.00085   -155.54845     78.20483   -226.43662    -85.43289   -207.01404   -513.69229   -215.13824   -254.94816   -353.37693   -344.06015   -484.76499   -311.62666   -297.51917   -469.95213   -489.40483   -500.12328   -389.83962   -466.65193    -244.2622   -287.25946   -181.57836   -262.82444    -398.6915   -252.99852   -292.00464    -97.35726   -282.94643   -340.20406     42.76324   -183.51643   -431.15838   -200.34574   -191.96798   -472.03605   -186.35738   -347.62694   -407.46052   -434.81417   -317.09406    -247.9477   -538.04833   -518.91405   -454.22462   -445.18446   -326.00262   -567.84824    -647.8474   -713.91414   -651.81709   -715.41294   -774.20342   -535.67957  -1016.72468   -734.79067   -868.17625   -683.12639   -798.20512   -846.43519   -765.30579   -725.97523  -1138.46371  -1045.68392  -1021.78012  -1153.10447  -1008.14966   -721.79436   -1112.2859   -832.59102  -1043.15042   -742.61104  -1110.04221   -1028.1273  -1055.20403   -916.06104  -1155.63207   -836.45132  -1071.12238  -1108.47539   -942.83708   -939.98145    -1286.134  -1167.57671  -1153.54966  -1049.24363   -944.61422   -947.23877  -1128.07976   -668.67533   -1253.5822    -844.2663   -993.32845  -1435.67589  -1189.75617  -1434.16963  -1870.79002  -1092.43565   -998.50487     -818.236  -1438.89918   -813.29737    -455.5648   -1037.1879  -1186.10212  -1302.93096   -852.75098   -877.77102          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        2.00   -158.19075   -299.75646   -426.27261   -241.97828    -179.3015   -425.47347    -140.5034   -328.47413   -292.71961   -630.63766   -364.70868   -389.19556   -254.20998   -291.20197   -248.76368   -129.27712   -387.69143   -233.99054    -90.35465   -218.01869   -395.95115   -383.34911    -312.5056   -162.63518     127.7333   1253.41182    3526.7923   4349.73332   3779.97873   1617.82562    213.23946   -108.89836   -274.21932   -217.74259    -292.7108   -182.53233   -237.48173    -52.31106   -490.36804   -450.21234   -417.61378   -282.89187   -302.17878   -341.00999   -271.64267   -475.08709   -486.62619   -342.45589   -410.83959   -405.21484   -304.89777   -382.81038   -327.39199   -303.29603   -258.39534   -177.15535   -199.77588    -301.8527   -180.58799    -74.61521   -188.12968   -271.15981   -444.77807   -264.91688   -223.38132    -88.93626   -354.94068   -228.18612   -300.86799    -61.61211    682.20214   1754.45152   2381.50872   2095.43028   1412.35716    619.17644    -29.96125    102.51193    -80.21851   -358.90429   -362.36923   -275.22197   -492.69123   -209.77122   -313.82951    -86.10202   -188.30577     53.58367     28.95749    535.50006   1685.08679   2791.85451   2760.53172   2147.00561    825.70602      140.105     61.23036    -176.4562   -311.29977   -265.92709   -173.79742    202.08651    402.56729     104.8656    -33.54666   -202.91286   -198.86801   -369.97982   -357.99247   -359.54265   -257.65968   -469.29757   -320.18599   -325.37958   -289.10112   -303.09525   -251.38383   -153.26697   -170.15848   -208.46844   -234.62002   -304.25532     -96.9419     55.55093    351.42635    782.66223    793.84116    835.74122    604.49098    360.12155     74.39721     -8.28732   -170.89413   -157.44211    -98.14726     99.77855    428.68267   1245.89722   2135.32417   2774.03363   2868.90309   2504.15868   1551.64649   1137.10075    578.95482     331.5358    144.60765    130.06943      90.8761     73.29218     -9.13162   -241.64239   -317.26034   -283.02714   -286.71644    -228.2053   -359.30754    -343.0704   -395.25885   -177.88897   -384.98903   -246.35212    -51.27235      -190.99    -17.61964     65.75553    116.62073    145.64201   -130.52911    -110.8845   -170.01481   -287.74411   -232.86443   -198.49077   -116.80498     49.17895    199.67741     98.87303    323.52411    165.93282    268.80382     81.39218   -129.08418   -187.59239   -151.77558   -156.42528   -210.12805   -261.54042   -266.58332    -231.9076   -215.65105   -191.48147   -167.93874   -211.62195    -231.1515    -34.99584    -36.46506     95.18562      47.7224    220.10262    166.55561    217.93789     -31.8498    122.99803    -43.38477    -200.1576    -46.44776      7.27841    145.56476    221.00113    152.84799     78.55136    -15.59262     -90.7993    -32.75739    -200.6735   -388.28513   -480.43235     -6.99322   -314.09801    -189.2851   -239.23261   -311.94797   -302.10495   -307.40513   -290.86893    -11.61567     29.40685    -43.73544     -38.2551     67.32985    133.23482     141.5469     83.60914    145.88076    217.89168    226.39184    452.67924    355.13533     367.7157    238.57532    209.20831    243.32169    -40.30534     -44.9977    -62.65376    -256.3645   -150.27392   -141.09684   -184.75243   -174.24074     -187.258    -77.14361   -159.32697     -9.44283    -60.90069     38.33241     19.91101    381.68506    100.11977     34.70697    489.12315    478.15001     744.7094    730.71873    852.27137   1003.83057    901.89897    562.62763    340.18275     420.6006     36.44077     86.39297     -4.75611    -94.37335   -123.60816      4.85176    -33.16258   -114.19173     76.19388    107.15505    142.26553     72.68512     39.68005     35.96332   -136.93121     82.65284   -123.84635    -131.5805    -155.2029   -339.00882   -278.88845   -384.39134    -81.90325   -250.41203   -248.06456   -335.55137   -444.81396   -334.45628    -400.9394    -352.2958     -184.291   -146.83979   -407.90416   -329.84292   -197.11011   -125.51836   -275.97605   -236.75278    -71.48271     -69.5782   -108.15934    -59.48629    -68.66108     92.22803     -48.2693     16.63946     83.82134     31.75784   -118.71221     23.32455   -224.94964   -311.74944   -304.52381   -208.80519   -253.12393   -142.53905   -359.62701   -387.04472   -369.04252   -107.49958   -168.69968     -61.6819    102.74288    -37.20475    192.07039     23.01304   -129.65189   -152.69589    -92.22855    -173.7902     64.31623   -106.43386    -29.52484    -86.77162   -218.83198    -85.83488   -248.28986     -98.4873   -295.09645    -77.97483     88.18643    -69.77509     73.54251    -61.35452    -12.49439     -5.51414      -4.0297    266.32025    184.14218    308.41062    202.56041    156.74712    155.89803      58.3736     61.68368     14.98164     39.07389     96.47793     43.94106    -97.76851     137.1711     26.52878     68.44558     50.86187    101.10033   -325.37378      83.9929     -63.6181    216.50705   -221.19973     98.73997   -162.06343   -213.96775     -9.69375     17.73252     114.3167      53.2597   -355.30967   -354.68951   -172.54509   -235.12928   -156.84315   -103.44117   -108.40655   -272.76058   -389.71948   -292.50096   -186.62311   -188.19752   -170.63709      -321.77   -290.67075   -261.38505   -212.48538   -343.01664   -339.38089   -241.89181     -76.3359   -229.74666    -92.95273    -87.25185   -258.70069   -130.44545   -212.76615    -269.1956   -112.30612   -380.80602     -90.2575    -278.3486   -290.06952   -331.49799   -233.69795   -361.31485   -209.20758   -596.16676   -616.21612   -506.77527   -427.91945   -708.44265   -462.60382   -635.14219   -753.90606   -631.93283   -982.97498   -820.91224   -744.54371   -815.49279   -472.89436   -869.98702   -737.52434   -891.23785  -1079.59731   -944.66225   -815.72636  -1099.51104  -1100.06731  -1173.75716   -990.73489   -939.66932   -913.34136    -783.2398  -1012.81115  -1018.81759   -944.88445  -1122.72623   -754.89686   -825.37717   -882.29395  -1131.05146  -1193.85806  -1183.71195  -1177.83525   -785.61246   -928.44317   -724.92257   -794.08583   -1074.1995    -914.7615   -771.75072  -1060.45547  -1149.60549  -1102.28278   -839.78963  -1035.56988  -1304.20283  -1029.36082   -643.47709   -653.89971  -1142.72618   -179.77176  -1055.50445   -978.94497   -421.76439   -983.06969   -994.95485  -1048.15405  -1624.75653   -368.30038  -1052.31745    -617.2519          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        2.50     96.96423   -261.92267   -443.50997   -166.59813   -206.26045   -443.00874    -365.1154   -260.96217   -324.87889   -531.96584   -193.78057   -432.60782    -192.0143   -277.77764    -277.2263    -54.39788    -396.7762   -348.00769   -118.50805   -312.49755   -347.48792   -347.40793   -316.77174    -128.1382    -26.43776   1090.02825   3589.03703   4235.79719    3664.7291   1549.52977    199.12477    -85.73128   -219.73331    -91.44447   -557.24546   -203.30283   -411.11829   -372.96094    -298.8241   -330.12957   -307.13335   -250.28434   -247.97261   -345.11504   -341.76523   -358.54875   -461.43522   -304.76096   -217.02087    -432.0459   -332.45247   -356.28597   -469.15745   -381.77745    -315.6373   -263.17153   -248.68903   -212.37807   -292.19163   -262.65179   -417.54413   -179.64332   -197.08369   -301.22736   -285.40133   -307.99307   -331.04782   -320.58147   -168.72271   -108.67309    876.02167   1808.81535   2310.97397   2123.20968   1463.91603    666.32842    204.30496     10.23553   -127.47214   -379.17494   -370.39292   -434.51876    -151.8625   -177.87731   -315.95735   -261.54795     -97.9642   -162.91614    152.13348    645.83828   1499.53984   2594.96171   3012.37813   2194.78468   1084.02807    323.86054    105.06972   -261.85805     31.31428   -368.93953     -43.1467    276.80386    220.60356    312.13703     22.61014   -115.87104   -200.47003   -312.91523   -164.85527   -486.97484   -300.17036   -439.00432   -327.46923   -239.65764   -288.46332   -255.63166   -166.51348   -256.85328    -74.00068    -191.3624   -331.03765   -573.90472    -81.01456     58.55414    446.26205    786.54117    864.82774    737.66046    589.13822    321.88875     55.92302    -50.00802   -226.82867   -289.28503     -25.3637    -84.17986     564.2806   1241.42834   2166.21433   2627.28092   2918.35254   2365.85436   1602.38237   1112.27956    407.35237    226.95833    107.75444    198.12403    284.65988    140.06882    -162.4388   -103.41723   -332.86014   -248.90056   -254.94893   -256.63398   -397.80392   -303.64089   -183.13255   -299.86055    -31.23471   -256.32959     -79.0616     32.77095   -262.67604    -22.20317     150.9404    -45.59306     63.12297     26.12417   -309.57243    -304.4126   -223.44984    -38.86282   -199.70015     90.34611    214.82339    294.35826    116.72505    353.78733     70.89743      2.33532   -138.60407    -236.7396   -191.92238   -116.47322   -256.61094   -123.33671    -290.2175   -574.67202   -314.20631   -376.96825   -263.53346   -278.63094   -308.73777   -149.32856    -99.15198    -26.56792    111.98931    302.34387    138.02174    361.50754     10.41676     36.87092     -7.91583    -23.55789   -125.75166    102.43038    161.01597    239.18263    102.53643    115.43112     18.26868     12.84085      0.71575    -30.94604   -324.71895   -261.72012   -341.75935    -373.8419   -380.07087   -373.39282   -491.18074   -378.12367   -235.77753   -192.60035      0.69147     12.56873      8.66677    289.08427     31.58802     26.31095    250.44445    115.71698    308.85491    176.61115    333.50658    282.43541    292.44876    282.61991    167.87958    198.54591     98.71924     72.46494     -91.0192   -128.93206   -336.41011   -179.86221   -397.36789   -290.40397   -336.33137   -263.44531     -85.9582     23.37648      0.65217      9.86055    168.25278    121.34952     206.3034    288.94323    220.48921    244.80057      550.216    634.06726    766.88204    737.10481    635.58369    862.32559    445.69521    475.81139    393.79383    203.35233    123.44771     -9.94636    -42.54853   -110.93126    113.26794     40.10221    -49.06489     66.74024    -83.48149    327.05177     80.73526    147.83268    -41.06154    125.77524    201.32253    -19.95508   -128.70949    -190.7484   -210.77273   -116.26667   -190.03797   -397.17418   -427.89082   -337.33058   -171.05515   -213.67922   -434.30291   -331.24806   -424.46359   -405.79966   -375.09554   -237.32142   -338.83715   -375.41662    -269.9364   -184.04317   -208.93522   -163.16574      82.1931   -243.65997    -74.79076     48.88039    -47.17674    -35.16792    223.11481      4.04735    -36.59648    134.74297    100.56612   -211.47741    -246.0954   -136.52673    -90.21181   -123.00127   -123.38056    -28.58661   -265.36213     66.03817    -31.03365   -201.24454     -0.06118    -46.85776    -200.2701    -78.26757   -148.55026     33.75804    -27.24792    -73.48609    -133.5182    -64.21137   -339.37927    -61.26706   -288.52835   -361.10295    -124.1248   -201.96652     -65.5506   -115.27388    -87.16821    170.48998      64.1425     19.61696     13.76951     34.93111     58.45503     16.06135    150.91997    267.38299    209.94357      95.8921    225.44814    176.47788     -11.0899     41.16308     98.25488     33.94617     -111.905   -227.47702   -128.77915    -14.45599      7.47527     17.79717      6.71626    -17.07856   -141.33479   -288.01637    -81.16593   -161.73889    -41.24654    -41.68963    104.37594    184.09745   -174.67916   -241.85192    -115.5631    -47.21641   -121.54427   -295.00292   -309.61687   -119.63856   -433.60436    -49.62828   -294.27783   -159.49106     -0.33741    -56.80269   -508.99395   -227.06834   -189.13505   -238.87887   -215.54327    -101.0958   -247.35917   -351.04235   -331.09836    -85.29238   -348.63112    -48.99757    -302.6689   -452.97737   -215.90792   -287.41876   -223.34471    -316.1938   -395.34562   -391.38528   -147.29808   -186.98129   -411.29537     -298.333   -455.89154   -489.74421    -276.9565   -378.79372   -604.21787   -469.25731   -386.68663   -661.46447   -426.24773   -383.79802   -709.16219   -712.76995    -810.1415   -612.73644   -802.62797   -785.39849   -581.30049   -808.34197   -620.70584  -1151.05739   -626.85912  -1070.51321   -970.16825   -867.82949   -847.89539   -893.11228   -789.35358  -1229.51475   -889.92694  -1034.20562  -1161.85128  -1049.23289   -1062.4468   -997.33401   -711.92221   -755.79961    -767.2078   -990.57593   -807.72338   -721.65285  -1184.09411  -1169.42105   -891.98195  -1469.02345   -865.22872   -853.73522  -1163.99079  -1018.19235  -1074.62615  -1387.14141   -847.53962  -1609.95347  -1056.85741  -1395.36186   -668.96744   -659.20966  -1145.83506   -597.02606   -727.49411  -1117.15674  -1285.65402  -1460.91112   -780.05381   -1309.1499   -174.23984  -1130.52751     -35.7439   -400.52416   -909.05011          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        3.00    146.28065   -214.98188   -413.50725   -211.42506    -352.9257   -464.73545   -640.90555    -196.4997   -346.47352   -450.45779   -198.05491    -220.0988   -121.84855   -392.75903   -431.34802     -3.58072   -328.94508   -421.47504   -160.17251   -408.66238   -307.10999   -285.29831   -237.21533     -16.2153    203.53824    947.59423   3606.57604   4764.00575   3676.00384   1412.75351    245.99993   -229.17915   -248.02365     -293.464   -487.17866    -163.6488   -269.35073   -595.79584   -191.56521   -215.85632   -291.89735   -257.51457   -466.78983   -312.14209   -281.31547   -366.55218   -311.36053   -616.23471   -340.09233   -476.33783   -407.21939   -252.95824   -251.11439   -317.69069   -231.28475   -205.25256   -310.34534    -140.4891   -216.69096    -71.58915      -360.06   -258.57095   -168.29846   -245.32185   -365.48411   -259.26603   -351.30388   -101.45287    -150.8092     28.27831    685.38791   1794.46985   2541.51542   2238.20143    992.81871    762.21341    222.30763    -12.68262   -339.49516   -225.20039    -292.3383   -323.15399   -275.88516   -249.96408   -338.79822   -120.05559   -146.53578    -167.6552    135.40472    558.02209   1832.59221   2844.27269   3193.14713   2407.48139   1389.38487    350.36093    -12.48923    -129.9323    -91.35893   -235.78015    -16.93093     280.3289    416.44344     79.02029    177.34329    -77.22356   -254.46469   -370.15732   -416.34954   -270.48383   -265.67827   -288.38106   -361.63585   -315.52603   -190.65776   -358.78485   -186.80209   -140.11489    -79.99916   -301.43432    -302.3734   -226.69126    -78.57481     97.62921    437.97838    672.61607    579.34761    809.39432    420.73539     355.3309    149.71553   -215.26778   -239.36556   -104.91462   -157.04233     25.83368      617.652   1108.03496   1901.06508   2752.90962   2674.90305   2385.20578   1738.28191   1218.88845    740.35124    172.22166    209.71242       8.2292    217.53284     107.5583    -44.02836   -176.60523   -398.69008    -236.8717   -320.64966   -330.00861   -223.26826   -274.31396   -333.12441   -232.87898   -323.36007    -293.5117   -206.88448    -13.66821     -6.30133     33.38557     73.82452     -19.6478      -16.998   -168.99395   -156.10974   -268.71222   -373.35694   -106.50114   -113.88431    262.03681    347.86516    261.90093     38.56929    139.86536    -57.84372      9.54687   -100.63064   -145.38678   -207.23943   -238.55146   -229.31812   -175.87067   -304.81211   -274.81327   -251.04666   -306.76988   -230.86934    -84.54558   -162.89988   -258.38252    155.07611     51.07579    364.89285    459.98743    424.42233    190.49149     57.97184      5.81329    -52.97654     60.76772    -54.72228    197.21102    297.14263    207.55161    -31.70685     286.1681      1.08676    -235.2911   -211.66222   -395.16246   -216.08353   -299.79426   -173.21357   -354.25086   -308.19557   -270.47346   -376.44306   -211.31883   -271.64591    -99.15406   -178.00092   -105.96246     139.9207     74.05562     65.11225     91.10547     97.40588     72.10967     71.07635    366.12507    268.94192    430.60435    323.24037    478.33475    317.41519     323.6328    119.84016     54.91496    -42.52664    -147.5385   -114.19387   -303.00803    -143.5692   -312.83428   -232.16755   -209.43926   -237.49539    -77.69224    -35.58589    -90.58718   -315.50693    -10.86153      89.4341    160.76453     372.6057    437.16676    535.29572    819.32602    644.83593    659.53341    706.76679    708.06379    682.46686    616.00369    366.06016    217.25275    106.99348   -227.58546   -194.65718     60.38519    -61.75974    -79.95116     99.13076     -2.90808    -29.41333     40.39113    156.52719    102.90226    -66.50581    -43.24622    -44.19689   -228.56435    -45.90588   -136.88974   -129.07724    -77.18378   -385.20891   -496.75759   -283.32724   -323.58993   -392.20738   -192.03819   -304.00212   -406.90688   -289.06873   -314.99184   -323.88445   -270.55346   -294.65768    -91.84838   -326.07827   -166.62503    -34.20181   -169.40478   -106.55049   -122.70634    -80.25913     67.43407     136.5483    334.00264    176.13953    -60.37192     -45.8923   -217.83835   -190.34669   -109.80337   -126.65189   -142.06188   -394.86423   -129.32001   -123.56057   -444.88397   -355.60566    -94.43113    -95.53633    -56.33373    -13.06348    -51.92113    -79.35425   -130.45142    -119.1939     105.6573    -86.40231      7.07751    -287.9946     60.21595     -3.29936   -152.76015   -201.27287      -17.584   -189.88266     161.4023     16.36994   -183.25917    -36.35145   -124.67667   -170.17336    -81.90127     188.1069     96.89217    150.75124    120.41477    267.10741    151.75202    122.40709    132.41527      295.432    237.85862      3.05681      17.8415    103.37866    106.59089     26.18779    -12.46763    -50.51994    207.21069   -224.11534    182.01001   -234.90823   -107.91785    -34.81354   -168.41998     58.70248    -15.05209    138.93732   -235.56203     47.69749    -31.00451    -114.0956   -148.06477      1.80621   -164.96931   -220.10736   -421.41536   -338.97245   -483.22275    -486.4007   -456.73138   -184.48158    -107.8758   -194.89121   -102.58352   -509.14789   -156.94607   -187.80922   -102.28246     36.62553   -324.97977    -185.5032   -139.59894    -236.1955     -206.623   -263.33289   -209.77219   -259.05591   -341.41923    -41.07763   -324.38787   -400.48567   -350.38972   -340.15041   -342.48933   -401.90382   -183.31635   -154.65841   -368.18921   -123.56631   -484.03004    -452.5636   -426.15298   -575.30003   -531.74951   -298.90815   -461.41771   -754.37054   -624.75569   -617.71082   -704.95602   -565.29297   -715.50638   -675.60372   -879.93547   -454.34275   -864.43649   -738.62034   -898.15906  -1092.08497   -861.99974   -821.12048  -1023.40633   -804.66722   -1109.7488   -842.54699   -968.16145  -1081.61526  -1067.45279   -854.80409  -1082.37423   -1100.4902  -1133.63296   -719.11377    -1108.012   -879.00173   -824.62628  -1223.38849  -1113.48871  -1129.22637   -897.89484    -772.5596   -734.08198   -738.59659  -1004.55025  -1057.34824    -943.2613   -822.07152    -860.6087   -729.11409  -1185.30727  -1208.36352  -1017.70029  -1540.62959   -1259.9268   -486.52163   -840.65079   -730.71352   -768.41107  -1137.61111   -996.97655   -736.38179   -891.68574   -131.84921  -1064.62427   -821.77972  -1004.82082   -742.96815          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        3.50     96.82127   -169.47014   -385.68956   -260.21815   -446.60862   -410.07554   -532.21851    -349.8257   -291.47598   -404.82943   -353.39375    116.13477    -65.23372   -500.78849   -501.23012    -58.77362   -355.06513   -351.01106    -79.14501   -437.58789   -337.49169    -254.6599   -244.65333    -37.02943    190.39711   1159.35316    3690.5602    4786.0659   3394.90167   1593.73996    404.62425   -314.68382    -155.5841   -385.57106   -315.21794   -395.78431   -271.13034   -347.35208   -390.33288   -457.01622   -303.01624   -342.00158   -540.63808   -197.77628   -431.09135   -452.91531   -432.21941   -167.69901   -341.53577   -400.53989   -447.51941   -494.81812   -208.79661   -453.87501   -258.97313   -377.24816   -393.57666   -238.52973    -88.52575    -33.05702   -533.20159   -455.34951   -353.19852    -431.0128   -375.37474   -429.94366   -262.14636   -400.10317    -269.9782     67.82043    558.97762    1812.4899    2503.7042   2076.57468   1288.81212    689.01936    371.60136    208.02576   -174.72119    -128.8373   -248.56475   -215.91522   -347.88442   -190.50705   -327.08716    -45.88042   -479.22052   -199.72903    193.95207     731.6569   1605.01288   2664.39679   2865.92148   2414.57313   1324.13858    182.63624    -59.92966   -180.80467   -418.74669   -196.85074    -20.36696    245.96511    391.42167    146.65221    170.62488    -33.26488   -181.64952   -197.81887   -161.36109    -452.5616   -319.72733   -339.20909   -339.72797     -351.595    -29.74755   -274.52317    -97.09481   -199.97682   -315.65214   -159.95712   -236.34375   -135.75882   -237.03518   -166.69143    405.70186    804.08272    994.85642    776.11367    683.67173    332.82572    151.40186   -140.05904    -81.86744   -190.55483   -105.29636    -18.76156    481.16665   1260.73634   2136.61628   2526.94889   2746.77968     2343.127   1597.27326    986.64654    294.58552    171.31241      99.7264    144.78709     90.95878    149.38511     33.65784    -97.91179    -173.6097   -309.25583   -352.89218   -300.09076   -251.74804   -460.32552   -205.56423   -303.61146   -259.02703    -213.2582   -153.42293   -179.46579   -100.75116     81.37204     33.26022    -26.84629    -22.43071   -132.75525   -181.00118    -80.15181   -221.80577   -172.72015   -129.21982    127.71877    245.07795    283.30517    304.49544    220.61803    174.98342   -144.10208    -17.95001    -91.60247    -72.59469   -171.42752   -188.53831   -282.42857   -257.12002   -320.82076   -184.61422   -198.81356    -233.9538   -179.11196   -150.91306   -113.47219     -56.8702    185.08103     323.7205    264.69517    335.74398    158.92743    175.44591     15.90841    -98.42175     79.14982    -58.05243     -12.8954    102.41175    205.36608    165.64276     230.8064    -48.22735       9.9927   -147.71523   -203.97354   -151.17292   -270.01586   -317.55963   -377.23975   -323.22437    -201.0894   -270.56012   -240.21836   -110.91484   -143.80804   -160.77156    -88.58529     74.28389    -54.29969     82.45753     45.50619     52.20372    176.39682    155.18309    257.83421    396.57184    530.82455    350.50303    267.77668    185.69682    171.15062    147.31168    115.10252     27.47954   -134.67342   -211.61952    -196.6238    -92.41333   -206.02984   -254.29888   -150.11359   -237.74964   -145.36293   -127.69464     79.83198   -147.77759    -11.07085     166.1471    299.52691    225.76844    411.09094    481.11192    674.36707     830.8913    796.63104    779.11858    682.78239    679.29686    581.44971    102.92327    248.55376    206.86061     96.64935     18.79747   -123.52806   -149.84238   -213.78675    -97.94728    -53.79062   -140.79206    129.17703     48.66965    -67.74307    167.06964    102.67277    112.63003   -248.46215    -127.5663   -233.59355     36.08701   -254.46017   -295.27132   -186.61511   -387.26175   -240.81293   -311.34076   -277.79925    -229.2458   -181.17172   -259.91679    -84.95272   -109.85558   -356.38563   -225.69624   -221.35232   -292.87667   -158.02842    -107.6852   -151.52478   -206.30792   -156.93486    -51.57856     73.07699     30.43833    128.23548   -112.32863   -101.68715     84.88335    -82.56262   -148.31112   -242.70245   -156.52036   -282.17777   -258.24366   -169.52902     79.79738   -145.03777   -210.36674   -135.79001   -352.29399   -136.49077    -24.80257   -172.23995     22.39324    -80.51817    121.18707     -23.7777     -47.1882       8.4703    -11.60003     -17.3482   -100.24919     92.22202    -97.67928     -118.013   -266.44904     -92.4616   -195.98644    -20.45699   -137.59868    -69.81876   -174.05147   -193.47362    207.16044    185.14594    -86.55966     -2.30065     92.87012    280.36374    171.10575    288.27992    162.11147     81.60983     95.07202    -29.14048     -5.93367     -36.8122     67.11645    112.56181    -85.18036   -109.35808    114.00076    224.04994   -185.27005     -244.222    212.14056     -8.44709   -252.85456     76.22153     21.92919   -155.37893   -260.21786     66.66992    -82.28442   -227.15935   -104.53105   -301.48104   -177.51601   -311.04316      8.96076   -131.83776   -235.59606   -236.89664   -160.61159   -211.82921   -295.26117    -277.6086   -156.71296   -154.53739     -60.0753   -166.25016    -51.62285   -344.34254   -340.04674     32.32729    -11.60761    -76.11439    -241.5899   -259.60208   -238.32464   -273.08875   -377.97305   -300.50025    -97.25254   -252.12027   -215.94223   -201.16873   -285.37256   -384.82484   -297.75578   -548.28665   -121.55376   -314.86662   -446.49473   -362.81024   -549.97057   -521.52586   -676.09009   -602.77219   -491.31954    -598.6421   -552.27159   -701.07192   -604.54535    -608.5405   -694.13776   -844.16605     -750.659   -554.36373   -729.39568    -664.3215   -828.31769    -710.1279   -975.70406   -731.11473   -911.63094   -664.06134   -759.07189  -1086.07003  -1020.67075  -1196.67354  -1152.87227  -1119.99675   -900.71352   -847.00298  -1009.53145   -794.42374    -813.5904   -705.76976   -1166.0696    -749.1053  -1023.33156   -895.53097   -963.52851  -1147.05571   -1047.0542   -873.95259  -1060.16265   -672.21209  -1096.89668  -1312.38406  -1071.47751  -1014.50488  -1396.23887  -1165.68367   -512.76439  -1095.85458  -1582.98349   -945.09548   -764.32681   -896.97447   -813.98797    -803.1055   -880.73936  -1138.66359    -349.0634   -225.37066   -598.96022   -907.28261   -856.32378          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        4.00     48.44132   -163.45622   -291.57821   -249.98544   -296.96316   -271.77964   -260.68822   -517.10447   -228.82737   -344.92446   -273.75903   -147.35015   -165.04586   -619.66201   -494.84944   -169.00858   -415.57291   -287.59855    -16.08924   -424.37803   -578.29659    -467.1549   -471.12164    -153.9207    122.17537   1215.63855   3466.03953   4848.01904   3548.08557   1489.00808    404.29212   -237.04833   -133.69414   -305.91665    -10.71711   -212.69998   -174.57611   -267.18539   -262.29716   -269.59366   -283.57725    -158.6659   -206.99715   -268.58825   -227.46293     -308.918   -502.61488   -230.39787   -482.01648   -472.69726   -259.31467   -270.43683   -379.84411   -363.03759   -399.22197   -341.49435   -258.33894     -16.3248   -144.14234      6.34293   -271.78016   -259.31496   -257.46778   -392.44414   -373.62688   -234.66747     -308.218   -206.64797   -227.76366    240.68636    745.58999   1683.78971   2332.83565   2217.30877   1358.55425    645.50662    533.31401     33.19253     26.18175   -253.86448    -145.9288   -284.92511   -178.71069   -335.71331   -276.82628   -290.88334   -334.58321   -273.30158     28.96451     631.5946   1650.96075   2749.33783   3273.82466   2457.83483   1320.96145    518.75319     82.31821   -218.23773   -131.74063   -234.38114   -110.38648    251.08994    388.13659    207.72513    133.18285    -141.3713   -243.76065    -212.3271    -244.9447   -424.11797   -315.33262   -237.52207    -344.9672   -399.18436   -334.01755   -249.73414   -215.32137   -103.91232   -277.33054   -173.20872   -309.75785    -185.6345   -101.13759     38.62228    466.06572    836.73353    922.24334    690.15296    432.26305    391.54905     49.99898    -42.68454   -143.08576   -176.60264   -117.09851    110.36253    492.68558   1264.50467   1977.82065   2518.18698   2911.03084   2429.66331   1616.25041    998.28429    508.18366    296.54912    137.08133    259.69727     93.32021     40.09169   -162.38435   -114.54805   -317.63623   -277.81642   -416.56377   -208.76477   -224.39197   -277.89542   -260.20442   -204.12188   -253.40198    -235.8472   -210.04709   -161.31681     59.63662     -38.9922    159.00392     40.45441   -120.71411     -113.185   -295.40829   -123.52127   -104.67526   -225.48421    -46.14701    180.59267    329.46664    126.83375    275.30124    230.66925    320.22298   -139.46361     35.58953     19.50002   -239.48249    -73.32996   -222.34195   -308.58523   -323.65467   -302.08271   -281.06496   -321.07815   -299.95067   -215.75324    -94.30394    -92.92876    -32.95697    129.28191    234.74394    332.08262    253.26158    274.83558    112.13686     74.05683    -57.65768     -97.6281    -80.35717    105.21966    211.63009    178.28524    212.17171     65.92597     88.73406   -205.74318   -186.71888   -169.17423   -250.16355   -350.06013   -265.26402   -329.12292   -292.01026   -246.61947   -334.44846   -202.20673   -185.60735   -106.71312   -113.94123    -33.44403    100.54999     29.84029    175.17384     68.79116    174.14174     41.67428    200.53577    215.57597    325.34217    325.26532    479.76276    426.99663     278.8275    388.79785    132.47955     -5.44931     88.13615    -123.3661   -222.26508   -184.28995   -289.56939   -298.52569   -271.41364    -33.55211   -144.39689    -73.72354      -80.198     -73.8737    182.65186    104.71919    171.25181    194.44739     109.3968    398.43652    523.08625     518.3783    889.28918    830.34443     912.6278    594.05221    415.46551    498.98358    351.22049    200.67227     61.83803    -50.27868   -100.54152   -251.37839   -202.65653    -93.55666   -101.44998   -104.47921     60.42353     61.93531    108.20835     87.50578     54.05994     88.73258      -0.7993    -53.76641   -223.37116   -157.99115   -198.70391   -208.75237   -348.65594   -127.42101   -263.11299   -197.68092   -338.10474   -273.93358   -303.58097   -492.28686    -372.4584   -326.84679   -190.19221   -171.30924    -92.90047   -311.17937    -49.18094   -218.89688    -36.55185   -196.52962      7.00214     47.83024     29.90492    -79.91131    -16.46032    -26.52246    -30.77973    -32.33146    -59.70585     72.58253     113.8675     -57.9197   -176.11132     24.31383   -121.03764    -88.05008   -154.02587   -308.50195     -257.252   -257.57272   -104.67006   -199.94601   -163.93412    119.18009    -17.35696   -126.41638    108.93686   -161.92608   -201.80989    -17.95689    157.65024   -218.87528   -262.97816    -93.96225    -81.85808    -87.22753   -134.60204    -81.54419   -203.08351   -187.83836     95.69922      29.6089    -35.48915    141.35907      6.94219     67.53887    125.52779      373.681    150.86992   -186.52095    128.95111     45.03637    504.52914     64.31175    269.01219    151.68658   -141.78892    -33.21618    -37.78319    -33.83821      0.99737   -259.84432    -33.97278    -24.83566    -40.75417    114.42051   -204.42791   -216.66632   -131.13172     36.21298    -21.69097   -258.19589    132.34581   -134.29709      135.117     86.19063    -111.7473     13.40954   -158.13155    -389.9255   -341.40198    -99.12303   -233.95404   -290.65543   -144.37815   -198.00224   -586.19421   -148.87585   -315.44448   -498.79118   -244.47689   -353.78821   -193.75083    -36.33624    -225.3993   -434.96527    -39.63511   -122.70384   -500.59598   -242.60566   -367.98501    -481.5321   -142.61657    -90.29428    -92.63133   -156.31009     -71.7503   -202.47873   -316.44007    -77.83541    -411.9689   -181.85839   -358.44542   -400.15639   -329.49851   -227.91363   -256.87467   -345.66372   -450.78571   -522.47377   -645.02515   -667.98728   -647.95084   -642.88096   -747.60742   -563.94702   -723.26317   -764.02774   -612.73038   -587.00224   -957.86115   -613.39411   -690.97068    -765.2952   -864.96042  -1008.10787  -1044.69474   -849.96441   -941.13039   -895.18389   -972.88638  -1012.24203  -1165.01883   -801.82039  -1048.12361  -1041.75852   -873.10992   -925.77504   -866.52863   -951.29803   -802.70232   -1021.1865  -1040.58895  -1099.81271   -601.89855  -1102.03872   -908.52732  -1410.96027   -871.09925  -1109.93413   -761.08238   -846.36027  -1069.12954  -1052.21618   -812.88193    -811.5263    -986.4543  -1234.34337   -760.05067   -736.51975   -581.27367  -1184.61785   -955.34342   -570.34264  -1274.21185   -660.31642   -991.34144   -793.95923   -635.83078   -280.08978   -876.29473          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        4.50   -110.35964   -291.47432   -142.00513   -226.14715   -141.47291   -155.88746   -191.31435   -536.46316   -202.38835   -241.14548    -66.07708   -509.85573   -244.70294   -514.09633   -425.94287   -201.96661   -545.18036    -253.8155    -10.60368   -339.70476   -527.40337   -293.20355   -166.90397    -63.10402    177.30722   1118.70278   3317.17417   4534.00605   3758.73859   1457.76985    159.04889   -435.24468   -291.11267   -254.14082   -122.01954   -160.40704   -280.67378    -48.28626   -276.18563   -233.52808   -413.69785   -377.38656   -274.96394   -331.59928   -492.21913   -303.05311   -451.37238   -249.18845   -471.89348   -153.07835   -420.57123   -355.69688   -470.29507   -307.26447   -294.96494   -301.26692   -278.99007   -406.44082   -176.87628    -187.8383   -208.82664   -269.12998    -340.0689   -279.29629   -254.21396   -324.49284     -318.245     -234.967    -51.57884     151.1469    611.51318   1712.88736   2570.41233   2053.19136   1323.68688    293.80305    284.36058     24.48718   -178.63029    -193.8088   -166.80019   -178.47058   -110.28033   -263.76422   -396.82014   -297.85532   -181.44082   -293.58877    -95.57192    512.60982     1704.245   2781.71988    3287.9229   2589.45879   1262.72128    541.18703    -47.78305   -256.60558   -311.04438   -198.50037     37.04584    285.04925    341.26268     49.54498     74.68632    -58.45908   -262.33454   -286.61528   -270.95066   -344.26454   -429.13924   -320.85203   -287.82199   -263.62656   -298.34221   -307.89464   -252.13659   -281.67184   -328.96732    -304.3536   -355.88473   -194.99092   -214.41641     85.94305    453.70106     594.6568    954.20846    905.83579    630.11374    248.15579     54.23837    -48.75187   -271.12786   -233.11647    -83.71284     94.55831    466.07809   1277.47922   2069.95704   2545.95404   2924.32644   2444.67873   1544.49971    911.43443     457.0805     230.0317    141.06573    299.35855    257.75169      20.0521   -131.93082    -97.18578   -155.33943   -324.48653   -323.41376   -170.46677   -170.26835   -244.59216   -292.02892   -311.14322   -288.59518   -170.56301    -54.72397   -189.70281   -148.35284     84.22385     -16.0214     41.93398    116.19444   -109.75955   -256.43887   -139.87485   -144.93323   -103.62032    -91.27176    114.53907    284.45438    432.83636    190.22479    399.27461     82.03657    -59.36235    -53.28308   -189.59357   -124.95287   -292.47554   -122.37739   -306.46797     -392.061   -317.54152   -224.51193   -184.87888   -381.10588   -283.16939    -80.43639    100.84591     18.35624    -109.9782    247.80185    182.19063    281.02153    137.40833    133.61878    133.13302     34.07902    111.81262     -9.18633    123.85683     94.40769    152.27963    235.66888     69.03432     15.04658   -108.77061   -124.36871   -216.42814   -183.62531   -305.27739   -217.69128   -321.02692   -264.17417    -302.5832   -344.18682   -139.59532    -59.98886    -68.57845    -81.84155     88.80743     -39.7065     158.0178     52.17674      41.3449     37.86992     74.20284    217.95117    198.11134    228.59533    365.81805    295.11713    391.30813    332.57139    255.03984   -128.93875    316.18818    -22.25663   -161.59361   -234.64259   -310.97054   -215.11589   -281.66056   -306.83191   -310.50089   -192.24785   -139.30833     -10.6433    -85.29408     72.96677     68.26264    140.30196    202.79696    409.22559    400.26131    399.23216    599.17728    772.14753    908.05816    883.96182    648.81264    533.49028    471.77097    332.24765    294.07762     32.82449   -122.49597    -88.83295   -176.54968     -35.9731    -54.46893   -133.23988     33.39189      39.6584     62.42061    105.63263    128.56999     99.75664     -1.60205    -91.07262     -83.7912   -162.12056    -28.99628   -176.83214    -28.40434   -341.99328   -233.35316   -344.44384   -365.89489   -336.99316   -362.09392   -275.48918   -238.99077   -281.46514   -324.02733   -137.92158   -105.20311    -65.83674   -249.60855   -232.25364   -175.03123    -70.58658    -80.38004   -106.39624     16.75811    -29.92076     98.11137      34.9986     69.31139    202.52751      75.1299      7.70709     25.34697    -82.00443     21.31819   -167.44075   -132.73437   -119.80411   -273.52244    -50.41763   -116.88629   -136.15863   -191.73326    -82.23126   -195.74997   -261.47291      1.60257   -137.50906    -10.56276    -30.68024   -125.86814    -79.94876      53.8513    -18.32541      15.4642   -147.44212    -66.33613    -98.08608   -258.19026    -99.63096   -125.14867    -77.20359   -123.47226    -53.17878     57.48566    -91.50068      -7.4767    229.59758     86.60517    -17.02761    339.13345    215.13208    369.31703    151.86128    133.74943    441.01244    288.52251    226.31668     56.46447      255.753    -36.93016     139.0203    -63.38388     39.56481     44.61651   -250.19474    -29.87192      8.29228     88.74133   -173.40792     30.24121    -86.07023     40.72032    -69.01919    200.49586     60.60586   -197.97305    -253.5174   -144.29926   -206.77479    -80.12902   -438.09991   -243.36542   -227.97034    -348.2735   -223.29743   -213.35945    -78.35874   -243.32286   -252.51028   -332.43083   -132.67766   -496.94224    -21.92534   -375.17802   -349.38443   -268.22311   -122.70725   -193.70841   -375.15547   -103.29705   -377.86337   -428.96392    -72.88425   -139.35343   -131.10444   -448.33142   -306.48869    -55.03452   -378.44577   -110.75589   -195.92879   -205.12694   -318.16244   -297.40464   -365.94239   -359.44649   -635.28149   -322.85759   -568.63675   -461.68519   -458.55226    -348.3679     -473.946   -544.94932   -770.74186   -550.90635   -651.85678   -551.92034   -801.90642   -615.43942    -678.9123   -894.12028   -717.62861   -633.17502   -848.26973  -1028.33826   -961.90951   -802.44087   -892.93558   -692.40828   -833.66203   -928.89502   -961.41096  -1122.09754   -890.59326   -895.31649   -896.47788   -879.44415   -1062.7413  -1175.25559   -1015.8912   -780.60838  -1143.80463   -938.69854   -956.15727   -983.99028   -590.70663   -966.82472   -755.07905   -577.27967    -792.7409  -1061.77462   -938.88867   -698.32536  -1038.07962  -1316.79542  -1315.76426   -1112.5321  -1101.91007   -996.84355  -1036.54372  -1173.14559    -519.0958   -402.43577   -764.93689   -959.32673   -1324.2412    -71.77629    -673.9487  -1455.43198   -745.95586   -839.19471   -954.94578          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        5.00   -298.19594   -418.83773     13.40871   -241.24341   -120.57349   -174.51784   -314.14669   -523.13499   -197.14092   -230.80433     -97.0439   -291.04845   -220.27037   -203.06923   -332.21811   -214.51567   -370.51408   -231.57554   -171.54883    -143.2373   -353.51924   -201.79503    -90.75596   -257.93223     205.1896   1378.59185   3322.59747   4324.85341   3624.72167   1454.27926    107.93253    -78.96112   -335.82354   -181.02003   -292.89191   -246.84975   -187.75894   -515.79996   -278.52917   -458.94501   -289.68054   -355.92339   -382.01566   -237.26357   -237.04875   -268.45728   -382.92414   -326.44755   -106.88673   -301.38465   -484.71013   -211.17004   -331.90282   -310.99595   -397.36663   -321.37187    -246.0442   -200.16421   -197.89118   -296.48173   -217.60614   -174.58246   -368.39321   -257.96241   -197.24083   -370.33856   -380.96588   -106.36343   -183.69963     98.99069    782.38735   1782.90118   2336.69017   2292.77745   1355.22649    492.87096    175.91776      70.8751    -85.05004   -234.70269   -292.65157   -211.60369   -231.55818   -338.02243   -250.58028   -311.69462   -241.57696   -118.72286     61.04742    675.49099   1886.99691   3025.72935   3473.25604   2544.68945   1575.05682    413.06122     24.52711   -134.08961      -88.436   -173.20055    -16.05339     71.37349    432.31228     267.9088    167.76139   -152.83815   -324.93615   -196.63458   -292.64398   -377.75653   -343.14275   -178.97151   -384.52788   -326.50194   -318.43962   -234.46569   -108.81977    -95.51408   -162.40708   -370.68249   -117.70942   -158.62854    -50.56263     31.89039     306.4009    671.28284    886.64749     899.7178     662.3829    365.86526     -28.3497    -79.69851   -153.96081   -218.91337    -62.69595     97.62106    561.36938   1176.66134   2014.83653   2722.59593   3020.35034   2447.46225   1831.10424    927.98259    454.49054    251.20741     93.94421     93.54365    247.13889     14.89677     41.15795   -142.88527   -431.50162   -209.02778    -198.6584   -244.67128   -189.15654   -250.71607   -177.68587    -243.6235   -151.67163   -218.38299   -257.41488   -147.68699      2.56567    -71.95673    -42.77278    110.97583   -172.63559   -102.08173    -332.6463    -181.5822   -248.11108    -56.74958     -6.25296     95.12231    294.04687    175.45865    242.16407    247.21592    -39.74123      5.51651    145.31022     33.61113   -281.23365   -134.51297   -188.20285   -211.20085   -248.52444   -282.95332   -324.59188   -314.20074   -106.75042   -302.66164   -166.00818    -64.30941     90.00863     43.88843    161.64379    113.14502    500.68787    130.64151    189.10024     107.1807    104.91746    -75.05913     28.85322    221.45591    176.39513    307.96106    144.82886    169.08942     -0.22016    -34.06252    -90.36608   -222.98921   -279.21508   -355.71466   -157.82684   -247.08734   -254.17423    -341.8283   -327.69863   -221.85089    -158.6899    -97.81642   -209.41969     -7.83157    182.97851    156.17501     47.87016    -46.50498     78.52223     47.51939    124.16612    300.87231    212.31876    360.14261    380.92978     198.4305     387.3724    182.74802    175.91238     27.69045     -7.23306   -172.72606    -66.93871   -201.31844   -361.53653   -313.20613   -257.83144   -118.52844   -170.14869     16.64383     97.97947   -172.99654    -114.8386      37.8098     53.80847     34.81264    505.88158    504.18791     695.9244    644.11463     846.8165    849.05929    813.49357    710.50179    608.64502     555.2717    295.10017     90.32939     62.67573     23.39368   -130.96283    -30.37518   -101.26957     -86.5711    -87.49065     -12.3922     12.03419      14.2691    172.23677    156.13918   -102.75253     234.1112     23.58427    -51.06483      -63.034   -214.81265   -299.78065   -192.73383   -330.18555   -296.28566   -203.78261     -269.877   -454.73064   -350.39723   -197.87684   -302.38117   -209.80429   -138.85516   -304.48681   -210.92489   -278.83364    -297.5334   -153.27716   -149.58941   -277.62993    -99.60615     -249.474    -61.78494     -9.95632       3.5189    133.35293     78.21819     87.49056     98.15335    -11.33801   -133.51819    -34.21142   -199.91262   -118.54827     -68.2691   -123.79041    -62.06062    -23.81582    -175.4057    -30.06975   -258.02085     60.99227    -19.12657   -130.98931   -122.20649    -58.39825      0.73165    -32.22927     55.15743    -99.29713     -90.6083    -99.67603     73.31386   -221.36945   -155.79837   -109.26758    -118.6877   -199.24463    -76.23146    -35.27854   -206.45749   -130.57599   -142.72094   -214.25784   -170.89768    129.71475     -78.9668    109.35665    224.74509      35.2743    240.78516     80.82195    354.14157    167.22177    167.80832    195.76017    172.17802     54.28522     -2.10419     103.0778     36.70073   -127.64625    -99.67574     37.21428    -64.09788    -18.89646     32.21555      8.28106     63.50558   -397.90572    -91.98914    100.12686    -42.01428    -89.43767    -29.71611     29.95813     15.19866       18.835       6.4558   -119.76865   -189.76566   -316.83914   -404.43457    -43.89569   -179.17334   -324.67114   -378.02182     14.75603   -321.90713   -365.68672   -244.98005   -218.79094   -298.53143   -199.50902   -134.72893   -243.57303       0.7656   -387.68061    -340.4257   -355.65672   -159.02664   -135.63926   -427.15421   -207.28678     -82.4926   -115.04237   -439.80118   -334.99195    -202.7007   -369.49721   -309.61675    -472.2885   -446.11491   -623.94752   -416.53781   -523.78583   -279.08095   -323.20989   -431.67028   -379.03761    -528.3504   -533.67598   -541.15836   -507.64005   -772.58119     -742.001   -417.34267   -498.92316  -1084.80016   -726.22841   -758.09102   -566.42953   -809.07292   -732.99882   -841.47715   -765.43063    -796.4136   -952.70631  -1169.77258  -1139.79637   -990.90677  -1135.45175   -946.55414   -938.98635  -1085.99265  -1041.86279   -785.24151   -789.85125   -765.51819   -1185.6879   -857.21853   -920.55038  -1073.70142  -1247.60718   -931.87062   -968.97251  -1254.54834   -1344.1512   -824.25276    -960.8387   -873.42173  -1182.78624  -1376.41633  -1242.73763   -668.53846   -637.76275   -857.94891  -1079.98017    -537.5393   -680.38156   -902.95529   -1152.3281   -1117.5357   -707.23666  -1053.86326   -1176.8976   -418.78058  -1087.34458   -876.02773   -802.00161  -1094.35924   -411.49892          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        5.50   -493.76279   -421.54067   -175.28937   -322.63211   -120.05204   -207.14206    -391.4627   -483.77648   -318.14995    -389.8355    -187.5503   -149.14357   -245.26966     -264.361   -263.89304    -229.6239    -95.52909   -264.90804   -227.15371    -281.8091   -363.59402   -349.10104   -153.55076   -392.07895     29.11335   1010.21738   3208.31428   4561.12468    3477.9798   1644.14797     274.3587   -105.29841   -482.93048    -181.7414   -268.36022   -260.15105   -330.05966   -195.45344   -408.55113   -341.00132   -303.66062   -140.05758    -429.8377   -327.11305   -175.26927   -247.31367   -503.59349   -291.18626   -209.66431   -366.28166   -387.17207   -169.38097   -344.95364   -395.66557   -196.75582   -185.37645   -336.78512   -326.15839    -66.74287   -230.43436   -159.74406    -319.0566   -361.33424    -162.2388   -140.11585   -428.05696   -281.30956    -258.9425   -228.82565     68.45934    625.13357    1783.6724   2390.92923   2250.99798   1169.86617    505.08393    194.92005    143.07476     21.73144   -149.05594    -85.77339   -117.41345   -341.30188   -354.22204   -206.26159   -255.54825   -307.98081    -120.9141     32.62075    659.95126   1576.63218   2772.98503    3304.4426    2662.1335   1489.85085    679.01497      83.4878   -264.13377   -292.50243    -36.14397     -32.2722    260.75144    404.19486    450.15281     -0.54272   -128.13609   -142.13444   -229.50846   -172.34516   -183.15562   -427.70202   -308.74531   -287.02036   -505.31374   -220.46265    -77.90551   -328.29858   -155.64447   -186.34485   -131.68962   -160.12779   -369.01701   -148.72734     -6.30437    239.11975    706.75472    778.58023    980.85003     500.5212    267.30045     68.67737   -130.35075   -105.61832    -205.6589    -78.72165    100.50943     385.1529   1224.62667    2413.1482   2774.86153   2880.70044   2392.48043   1601.92737     755.0248     302.4934    226.41907     135.5932    184.85462    210.90378    261.92465     37.51217   -115.08997   -106.96281   -233.34763   -277.94904    -349.0035   -241.06784   -219.57497   -243.13749    -215.8532   -310.20982   -189.62794   -164.86193   -214.41017   -112.74992     -5.46352    103.46194    -51.20102    -49.57286   -142.77794   -140.08873   -283.37081    -333.1614   -132.41811   -209.27869     95.42746    231.71907    206.28908    -22.68179     65.77432     59.54113    -94.57025      5.19852   -139.51996   -273.33837   -342.53429   -226.80516   -173.89471   -329.04656   -266.28732   -357.42331   -280.07592   -240.75533   -173.31482    -62.39972     92.98014     79.35671    163.48992     28.42657    231.85039    322.94253    218.50101    201.86662    164.86458     74.90634    109.62374     93.68908    -33.10442     252.8566     142.9224    282.44767     129.6276     83.80214     95.17334   -156.71761   -260.65559    -93.32041   -366.63058   -120.81074   -221.51502   -345.91244   -234.94492   -273.95529    -96.53744   -145.81107    -115.7644    -11.29364     -2.45595     -4.10254     60.19009      214.829    111.44562     33.98523    126.73709    231.04559    155.69002    250.79739    249.47898    271.06479    207.01257    376.50145     28.46098    472.99857    -17.58392    -120.6916    -46.61577   -209.05007   -405.14647   -402.36921   -356.89094   -264.66958   -258.94253     -192.824   -156.17406      33.1303     68.64669     49.29946    162.93524      81.1933    171.37101    362.86584    330.79436    506.66109    881.64936    631.25196    635.06446    526.60872    838.42384    531.00909    444.19541    404.75973     32.76346     229.4689   -110.02753      -99.947    -162.5317     43.35336    -47.19068     -24.5546   -138.32974    138.09782    179.10825     57.02851     129.2866   -170.34052     39.65307     -17.1329    -57.50056   -275.98351    104.85123   -225.74966    -343.5061   -130.08617    -416.6207   -377.14417   -297.48964   -335.03029   -240.37777   -282.85151    -248.9051   -310.17844   -215.48112   -261.27002   -108.97583   -279.93063   -172.41304    -268.1778    -74.90509   -217.60453   -391.05854   -177.98335   -110.91375    -85.56311     126.8483     61.93064    -83.27353     23.60732      48.2492    -85.12809    -34.74085    -74.96887   -121.17425     61.27402   -197.58818   -134.01576   -101.62166   -127.31339    -94.57387   -159.96481    -78.14877    -94.70956   -141.10088    -83.96394    -60.50147     27.61798     10.29561     -6.01215    -96.43493    126.51446    -29.10423    -58.54449    -91.54668    -95.80693     24.01718   -140.61868   -227.18501   -114.24691   -130.37448     -8.80701   -174.39844      -67.845    -21.23766     46.86708     25.52538     99.75437     79.85197    199.52722    117.84337    377.39711    317.45008    241.14518    104.79615    187.54139    156.92299    126.87555     73.18826     27.92288   -127.89762    128.08131     66.82143    -73.21395    121.60601    -87.28336   -156.69082   -117.23467     -49.0443     35.37122    125.91234     45.48592     -0.04394     -62.0243   -161.18927    -45.17419   -211.59565    227.49221   -441.96347    -37.39049   -133.50809   -189.91716    -97.01625    -72.62166    -334.1841     67.17683   -175.12577   -297.66462    -87.99472   -203.63202    -33.30537    -79.11245   -312.15561   -428.73151   -194.66816   -316.54532   -205.22763   -152.81687    -70.63963   -240.96107   -238.01087   -150.25443   -393.67729   -278.08348   -270.37349   -425.48225    -97.59122   -236.31157   -449.93809   -301.28701   -373.51736     16.84986   -344.15539   -221.92397     -7.69835   -425.02397   -228.10623   -363.81308   -209.41527   -335.63759   -396.12576   -604.97618   -675.69163   -457.04938   -677.63156   -614.13503   -574.69673   -645.58494   -764.83248   -788.80737   -631.57328   -755.23321   -713.25312   -722.36183   -722.52955   -907.06155  -1084.03557   -827.38884   -863.21958   -813.24571   -944.53797  -1103.49588    -835.3887   -775.34412   -919.58084   -730.51655   -878.45575  -1134.46904   -766.18758    -935.0351    -713.6444   -883.64269   -957.53872  -1145.92538  -1108.57569   -988.36087    -921.8912  -1009.57403   -997.86262  -1318.07178  -1179.80559  -1118.38664  -1160.95009   -974.48844   -913.39283   -707.25969   -861.91578  -1172.46092  -1098.13305   -928.54268   -924.04584   -959.37933  -1195.46984  -1510.22199   -767.50322  -1322.45699   -895.69139  -1056.05759    -910.2401   -796.41419   -380.75104  -1077.22739   -323.02344  -1014.98176          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        6.00   -458.95534   -299.85417   -389.99925   -406.83595   -106.45377   -225.05082   -364.68034   -421.17676   -436.33422   -423.96735    -236.2346   -341.60027   -328.34113   -396.47945   -291.63159   -268.80177   -171.92195   -351.80401   -230.14515   -357.69578   -372.72075   -184.14311   -240.67181   -414.80173    -32.17959    991.62559   3327.37532   4444.06678   3516.43827   1709.14713    257.82789   -121.95179    -312.3474    -368.0777   -278.40805   -364.69628   -321.94312   -184.89143   -352.28747   -378.40038   -251.21893    -299.2457    -89.31709   -295.77095   -343.49269   -444.95069   -266.72492   -466.39341   -269.71579   -338.18853   -308.80746   -363.93426   -412.95409   -224.75625   -317.03104   -345.45359    -241.6031   -405.11493    -103.4949    -77.66987   -144.09843   -327.95106   -206.54387    -42.05213    -278.2338   -308.82575   -178.27393    -268.6866   -414.39208     66.31534    639.11125   1807.89004   2424.56183   2252.39752   1137.36309    583.87386    118.77142    188.95345    100.24253    -67.88208    -68.20942     38.18663   -202.52452   -164.21659   -305.32933   -140.58978   -125.08167    -61.62842     45.41104    652.79364   1792.90648   2774.84261   3296.85988   2589.72255   1384.38416    416.35846   -157.36429   -310.57446   -136.38033   -310.21934    -15.64573    174.30784    341.72907    334.53575    201.98447   -135.86628    -43.54948   -126.65186   -296.29187   -264.31864   -286.38985   -217.51987   -487.51273   -239.91434   -130.05424   -142.87585   -255.52064   -176.94116   -322.58462   -157.78384   -224.20803   -236.41347   -102.75619    124.53112    391.70742    659.55647    991.69282    911.38701    585.97622    409.76637    156.13531    -47.80116    -93.27229   -196.08045   -154.95596     55.64711     535.6427   1131.75524   2266.63201   2664.57601   2514.97807   2272.78114   1686.37706    1019.0099    405.14276     181.2411    144.85491    173.31175    224.22886     32.29933    -18.67059   -146.22877   -246.46591   -246.15914   -234.77492   -245.92682   -270.83256   -269.98099   -328.93709    -362.4826   -243.31119   -279.14191   -281.67694      -1.2295    -49.37773    -87.33229   -132.45265    174.01927    -77.01162   -165.76633    -199.4465   -351.23615   -307.33092   -196.17474   -189.10045     11.20511     94.57813    223.32544    205.04396      268.115    159.79255    -89.05327    -62.57046   -250.61537    -81.88432   -156.53577   -175.61915   -365.68061   -337.04754   -161.02084   -339.26412   -176.46262   -274.68874   -209.64708   -184.25905    -68.15651     80.20349     57.94383     90.24238    241.70557    391.21079    224.78614    139.80769     73.31209     50.72667    -120.4638    247.08405       18.939    224.83573    168.74463     83.93266    201.68949      71.1248    -76.12628     -204.002   -222.52475   -308.60347   -462.50611    -250.4942   -463.69203   -425.28144   -220.84329   -162.53437   -285.63104   -218.52272   -183.17572    119.02263    -156.9656     43.93835     89.98247     85.00645     84.92299      0.92618    -95.28815    -67.98225     38.07176     284.7149    423.85553     341.7178    405.16478    207.47517      200.098     48.32232    107.71375     46.55256   -124.66731   -231.32159   -253.89524   -412.01072   -278.10054   -139.71288    -221.1814   -194.88268    -87.01451    -43.35186     21.89852    -140.8024   -105.85075     45.60543    373.07168    114.89054    634.69926     692.1168    623.85447    726.08715    836.55783    830.96906    691.17005    230.55102    658.96671    557.63586    185.30397    232.23226     62.22537    -10.13441   -136.21329   -175.57025    -81.19683      4.12332   -121.74307     49.23344    -42.83028     33.69105     21.34214     30.92622      50.9693     72.48285   -175.22043      -189.67   -152.01337   -171.13053   -144.57077   -204.39104   -183.49035   -743.98004    -240.3131   -440.50895   -453.09228   -239.62214   -133.95613    -427.5507   -246.81278   -262.80029   -359.14765   -152.89177   -217.16048   -238.24707   -159.56032    -72.79609    -76.86935   -113.79438    -53.63365   -112.19683   -143.26688    -17.91865     54.50712    135.18319    -174.6005     73.13342     81.89419     83.69107    -18.78603   -182.31741   -174.05848    -66.71128   -101.92718   -125.39208    -78.28273    -47.06972     -8.66382    -12.01595     46.62581    -32.02523     97.14764     35.40152     -7.61658   -121.14546       2.5009    -14.77028     27.97582    -16.93884   -281.44533   -211.02164    -42.56801      -3.5335   -153.06983   -294.45662    -31.30049   -139.49954   -177.07067    -21.78455     35.20744    -120.6355    -16.58168    126.51298     63.72411    287.33585     98.12963    298.08882    326.61229    304.91581    117.51619    261.44128     53.69076     63.82436     42.58646     166.5005     92.47305    124.70757     16.90293   -106.28357     -1.96503    116.71872       15.356    -179.0635    107.42374     86.84038    -13.91155    -55.71189    -42.58324     -5.48279     -21.8042    -86.58349     41.69236     64.86172   -172.49822    112.74692    -79.65087    -152.7906     13.33722   -378.01098   -394.74312   -259.72454    -185.3199      -263.46   -220.83528   -494.79918      77.1375   -160.58522   -240.59684   -471.55923   -373.04259   -240.83213   -375.37241   -163.40576   -266.63504   -403.18678    -77.33031   -335.98635   -368.01853    -415.1825   -233.25109   -220.39545   -120.49076   -481.06003   -173.31952    -15.70534   -267.28833   -322.19858   -383.30682   -317.39073   -393.08144   -304.80583   -463.84906   -454.43795   -493.24291   -318.34018   -384.51002   -426.70269   -495.18312    -642.5371    -632.4194   -551.16476   -518.21271   -693.86677   -654.33175    -544.1368   -659.06695   -785.18942   -443.25937   -684.28814   -899.12565   -861.75391   -717.30367    -683.6122   -569.80288  -1041.02808  -1116.77145  -1150.82047  -1131.11778   -835.76716   -927.74165     -948.924  -1046.22773  -1124.33773  -1124.32308    -956.4517   -1033.2266   -923.02561  -1127.99778   -793.02336     -672.013   -843.76802  -1322.73239  -1190.89908  -1365.59127   -836.55498   -975.98118  -1070.06818  -1086.58803   -756.82881   -771.62194  -1075.43075    -754.5185  -1005.45342  -1332.57071   -872.96296   -978.08882  -1170.33482   -568.42858  -1149.08552   -811.91907  -1234.78525   -718.73719  -1153.44374   -120.21896   -903.55337   -821.09031   -754.14482   -583.06902   -1120.2142          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        6.50   -424.04167   -180.46693   -441.16379   -340.64511   -118.83865   -228.19171   -319.19217   -361.15056   -450.98121   -271.69026   -282.21991   -367.55202   -258.20747   -413.88296   -438.47031    -329.9103   -235.89351   -303.19842   -247.76839   -372.55525   -349.54447   -135.30595   -104.44491   -308.29516     59.45953   1028.88472   3368.86902   4593.61596   3564.53926   1527.24748    197.96424   -136.65907   -484.84784   -204.75295   -365.59461   -306.32159    -265.7994   -212.71728   -268.19802   -423.44802   -353.47654    -537.3523   -351.28879   -291.99412   -395.51504   -182.14526   -422.65145   -336.18284   -247.53503   -385.33028   -234.60686   -295.15392   -367.44659   -425.29295   -355.98063   -275.43537   -176.70157   -271.58976   -135.88238   -201.40125   -244.29579   -218.42897   -290.16261     -2.04352    -340.9683   -342.48938   -413.93447   -238.95097   -419.12798     244.6573    760.84018   1736.90884   2452.21245   1974.04747   1177.53473    525.79919    312.23553     52.15192     80.73064   -169.35321   -200.27617   -234.53363   -174.16043    -294.5851   -166.40056   -219.96861   -378.08133    -50.36295    175.47118    801.06458   1748.00703    2833.0993   2726.73822   2404.86705   1352.76714    496.68297    -19.39011   -344.06469   -157.53773   -114.72411    -23.03271    581.32734    569.56273    129.76878    101.75291   -137.90668   -216.65291   -350.27247   -261.16662   -332.99909     -240.973   -287.85078   -372.39625   -341.35386   -362.36063   -291.25481    -240.2644   -103.10438   -134.04402   -151.32024   -249.07946   -254.96195      -72.889     38.42787    398.88567      584.719   1225.39616    946.39763    516.59751    340.77818     95.40004     22.06385   -131.84547   -203.84479    -79.29187    -13.27476    419.96908   1193.21726   2124.79058   2636.69016   3097.04043   2369.35848   1748.79501    836.16879    548.26077    449.54718    146.04266    211.02783     73.04389      67.5293    -54.40436    -66.11224   -331.77244   -341.98753   -208.10741   -271.10084   -243.21222    -230.9161   -219.64942   -325.29535   -142.99051   -210.56744   -154.35724    -112.4503    -93.24482     61.47036     75.74683     69.25625    100.20097    -91.21926   -251.76017   -312.79341   -241.64458   -258.38622     75.52961    236.97382    222.49803    274.43622    118.83202    263.19645      6.43608    -54.43999    -52.18865   -206.17172   -168.00628   -228.49656   -215.18321   -227.20259   -246.00828   -208.87868    -227.9886   -107.22571   -216.88335   -170.64624    -42.73677   -111.70224      6.08579     60.36134    206.05799    247.59613      263.218    152.81082    400.92835    147.49898    -56.45402   -100.31792     30.10363     33.67953    131.46884    233.50379    191.04277     94.91385      3.92108   -155.60022   -182.84851    -93.70284      -305.25   -237.78753   -277.74092   -354.00774   -264.39842   -130.29577   -398.62689   -154.79261   -262.20883   -131.87135   -130.24862     45.65562     12.91778      96.8543    212.03846     92.50527     70.05346     97.12666    267.17985    232.93639    390.46516    402.21113    180.90516    454.39522    308.66493    237.82919     82.09154    193.14399    -61.86609   -144.25603   -186.07519   -117.15898   -213.20962   -243.56271   -281.47379   -329.64775   -124.39216   -117.79821    -87.46929    -13.82208    134.72953    132.21913    -32.35883     169.2778    288.23562    345.70923    519.37204     586.2068    805.07991    749.18021    787.68788    678.76414    519.21707    475.77089    267.53121    313.47859    237.36856    -42.44477    -91.23586     -19.9376   -213.33638    -35.35354    -47.28983     39.26483     40.31001     -4.60967    152.50975     46.37783    186.27153     84.21543    -54.91482    -16.22501    -56.03221   -283.19653   -200.71614   -160.07126    -185.4607   -225.04694   -180.86094   -298.38579   -332.48105   -299.37049   -236.44437   -287.11776   -311.75771   -161.54701   -339.29951   -483.35418   -230.81832    -83.49276   -239.62448   -171.88722   -169.83187   -190.92759    -10.43501   -177.43587       85.607     80.32517      4.88592     19.91139     27.86726    -35.09972     31.53134    175.09721    -57.88606   -107.68858   -155.80473    -61.08542   -201.08214    -91.90203    -64.24166    -31.23219    -83.96593   -223.34224    -32.06438   -201.26795     -5.51427    -18.27822     -179.564     64.17417   -245.32659   -108.69666    -65.43258     52.64841    -45.89777     -49.9194    -32.15682    -74.87375   -181.05882   -158.81712   -200.54466     37.85407   -140.74342   -192.06199   -148.99881     17.59959    -59.52594    117.49502    -14.11571    151.51919    161.26487     149.2456    265.67702    114.72201    368.28487    183.64967    247.13109    209.14013     78.22065    161.04556     75.99273      0.15714     264.3897    -51.42797    -82.66238     93.59158     90.65436     27.18712     22.03726    -52.26856    -49.93918    -11.95022     -12.5117   -222.50549    -49.71537     -12.8266    -90.78134     62.75854   -128.75178   -266.17809   -163.73319   -102.78507   -315.14632   -214.88929   -113.38274   -325.51736   -422.31389   -290.80509   -682.73068    -46.45736    -17.35987   -162.66168   -243.04601    -23.17759     64.83731    -67.35973   -162.22823   -357.63502   -184.13774   -503.95341     23.59669   -326.92026   -184.62632   -144.39044   -320.53181   -155.50758    -392.9852    -32.51212   -299.62992   -397.14544    -36.64016   -206.43137   -526.32377   -466.24001    -83.16058   -489.31563   -252.00936   -352.03489   -250.04071   -576.80226   -349.69122   -548.31797   -584.15719   -516.08453   -395.13107   -531.08427   -604.91818   -752.30938   -595.86938   -718.14611   -635.63983   -696.48987   -596.53809   -965.24502   -836.23455  -1071.63542    -900.4279   -617.46622   -897.38879   -938.30469  -1029.27006   -803.18495   -980.96734   -771.99038  -1131.67814   -954.06138   -961.12042   -967.32451  -1065.11232   -913.74569   -934.22694   -1021.5381    -770.3313   -702.36041  -1166.19485  -1043.28696   -759.38584   -997.06757  -1481.17158   -808.21759   -555.77132   -885.52903   -906.62398   -944.14631   -1313.6833   -740.78759   -696.45699    -915.0623   -824.10457   -443.74502   -724.81171   -775.90297  -1368.44628   -962.72879   -847.93425  -1446.21742  -1226.38735   -808.99225   -761.19335  -1241.51495   -401.65551   -620.48988   -694.20261   -252.26707          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        7.00    -402.7282   -143.93025   -194.67306   -266.80024   -148.31299   -207.98436   -195.39964   -320.37917   -442.73628    -209.0288   -266.75851   -185.01357   -139.03608   -383.09622   -390.21948   -431.13331   -304.51075   -249.65127   -313.77219   -358.24063   -198.49903   -118.88431   -217.32253   -243.54176    103.52156    989.85833   3277.15345    4468.8008    3331.5473   1241.52908     145.1761   -136.36418   -135.62256     -111.688   -331.41144   -301.45447    -63.25167   -236.61903    -373.6211   -352.03122   -242.71799   -119.77495   -340.90168    -284.2886   -316.66103    -348.5677   -472.70324   -257.68593   -362.88464   -186.52837   -206.23771   -415.62864   -351.87424   -556.32504   -224.83743    -390.6889   -149.47869   -296.36495   -307.71271   -206.14503    -36.97376   -286.37877   -523.22452    -63.98244   -120.94039   -334.79624   -484.30871   -214.43137     -95.9729     84.50898    651.44543   1985.31759   2504.01174    2329.3318   1118.41844    537.92775    295.34723    205.97696   -119.50384   -246.55765    -73.78877   -311.51875    -64.88534    -251.5465   -432.33205   -180.02531   -274.20543   -200.10296    -26.47958    733.51844   1897.70241    2829.3156    3203.0629   2714.98543   1270.31075    465.70909     124.8649    -170.0547   -181.04319    -89.96992     -57.7951    280.72162    260.58833    214.10636     11.61307     -48.7909    -53.50885   -291.85531   -327.39692   -277.08525   -335.61732   -253.93389   -229.20059   -273.17848   -329.09807   -352.21055   -407.26939   -166.20845   -175.10341   -243.38851   -268.82537   -168.16862    -69.45552    -31.26577    437.43146    818.24954    945.93654    719.74604     675.9242    163.52868     -0.50805    -39.25402     -90.6351   -234.40707    -31.19377     86.50439    481.82302   1137.48034   2221.01132    2789.3311   2793.04877   2192.10417   1533.32874    984.74333    494.38731    223.28881    155.32299     42.94481    276.58054      5.10796   -117.86908   -206.98695    -271.6624   -353.93011   -230.93273   -307.90359   -391.17888   -294.95121   -330.19695   -275.90927   -109.45925   -137.18392   -297.78092    -56.09554    -52.15862     -27.1047    -20.06069      22.9493    121.62489     -234.996    -189.8835   -157.69793   -245.60229   -165.72393   -178.86132     54.10281    282.24825    286.27852    204.60355     67.92163      1.58886    -72.22768    -23.16874     67.89914    -21.01847   -250.24927   -159.18412   -373.95129   -227.54313   -251.79232   -231.17646   -284.01322   -351.30313    -60.51814   -126.89208     20.40141     96.39228     86.83138    142.57084    322.80832    334.09241    255.92798      56.9373      -3.9914     96.85099     10.69472    -21.26985     24.21306    140.79002    221.18596    320.92471    259.98663     65.71811    -70.15022    -51.28325   -218.76467   -447.92101   -231.38742   -365.80116   -411.59625   -229.31986   -350.78082   -278.08292   -176.13819   -236.80634   -128.41806    -68.18094    -81.39131    136.46291    227.93984    270.38321    113.09573     74.94974      14.7986     129.0258    263.52152     239.1271    278.88542    271.39996     75.52091     220.1174    200.38373    196.94627     41.12536   -211.23682   -121.36051   -238.67074   -290.99316   -349.07215   -256.40315   -243.67059    -85.69111    -137.9977   -187.53917     92.37228   -118.57663    188.61132    222.23736     52.69146    214.62153    292.20229    555.50347    473.58226    733.97966    795.49449    662.87603    677.15511    695.04987    660.10204    412.41155    490.58136    244.09823      6.46206   -165.01525   -104.13721     -42.6073    -109.2203     -3.68704    116.05736      5.67403     13.43437    200.68905     82.03129     20.93114     25.69291    -36.47321    103.10806   -175.90688   -234.77899    -25.32146   -183.53148   -209.96568   -322.75701   -239.19262   -162.26874   -294.66152   -243.45992   -346.77684   -247.47589    -209.6383   -305.78046   -305.32761   -295.95383   -311.32043   -401.05034   -125.23678   -224.64157    -38.46779   -114.93657   -137.45383     -11.2968     -2.43494    -37.35908    -55.17238    -39.76408    139.41494    116.05592     -9.69109   -112.81866   -159.20167    -81.20484    -23.93451    -15.43201    -69.50763   -130.89065   -173.83974   -157.72267     41.67544   -211.18884    -82.26085   -174.39291    -66.42669   -277.73114   -173.81854   -175.39614    109.43764    -72.34991    174.72177      43.2355      34.7259   -222.55066    -71.99802    -35.65241     -180.379   -119.87545    -78.21689    -21.03301    -15.40556   -189.78623   -273.65921   -464.84533    -17.76813   -119.73824   -281.11885    169.89219    -80.12953     78.67721    283.68073    120.60576    199.33367    267.87257    312.87806    263.79467    321.35087      2.80152     70.97035    -23.43576     -0.99495     17.83362    233.74123    221.77807      12.7786     17.98463     27.30425     -6.41802   -104.65949     71.29827     86.46203     -69.6607    117.45749     -55.7815     -9.25315    -21.45142      16.5722     -8.72566   -227.62736   -110.55044   -236.87616   -477.29352     62.47621   -174.54768   -282.36875   -376.97115   -259.08959   -122.86662     -8.50714   -298.16056    -73.34057   -259.64308   -284.76076   -306.04975    -297.0032   -383.45356   -358.91169   -240.61393   -244.86461    -488.7611   -124.03183   -326.41035   -253.55478   -226.21676    -44.95402    105.53548   -251.70624   -119.38041    -215.6936   -262.55584   -215.43872    -67.34214   -334.59716   -255.85573   -296.95272     -38.4196   -328.12518   -268.11577   -521.76386   -593.11552   -389.39004   -498.83253     -497.865   -767.55582   -692.86937   -656.43158   -623.60244   -697.77257   -575.39362   -762.73522   -633.68616   -645.03577   -681.27632  -1145.92892  -1009.39234    -950.5289   -942.88233   -711.83284   -967.06179   -854.87794   -798.79396   -657.34348   -845.94686   -768.24674   -876.56563   -837.98808  -1066.85767  -1084.81608   -928.22649  -1134.73303  -1129.61753   -747.19211   -719.26929  -1121.66708   -852.54563  -1026.62888   -608.51872   -1068.3423   -943.86937  -1458.15648   -713.10638   -961.67981  -1004.64143  -1332.81814  -1055.31337   -288.18363    -830.1539  -1023.09474  -1148.59615   -799.44519  -1138.36661  -1210.57649  -1705.30803  -1418.31447   -839.00296  -1169.68455  -1100.57408  -1018.74503   -743.41685   -524.91044   -248.22407   -400.35391   -986.64657          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        7.50   -500.25141   -119.36656     24.07533   -210.32386   -390.47298   -179.37069   -100.62502   -303.19043    -471.6029   -249.06122   -265.80716    -92.44788   -181.77387   -355.21662   -305.93079   -330.22101   -365.23583   -165.64674   -373.49994   -291.66081   -290.47465   -156.79942   -376.83538   -156.38518     -34.3832      1076.79    3172.5913   4482.46326   3547.52263   1500.90318    150.63952    -57.02934   -225.02412   -384.23649   -207.79712   -369.64941   -223.21777   -171.18728   -368.94725   -403.06701   -320.42859   -354.72243   -347.08408   -359.01395   -209.04955   -391.25128   -413.82579    -313.5022   -350.80318   -373.83254   -402.34235   -396.92319    -452.0262   -308.55336   -354.48461   -388.94091    -255.0536   -470.23427    -79.76468   -267.82423   -284.86554   -285.69365   -190.48385   -169.06713   -115.47834   -311.07729   -347.58987   -283.46558    -174.2556    139.64515    879.54705   1725.50665   2363.18011   2313.27465   1312.99507    650.57584    146.87666    148.85385   -146.29405    -66.96463    -468.5245   -111.08276   -276.61444   -219.57982   -220.20652   -343.83383    -241.7328   -217.65834    249.25365    854.11051   1709.30617   2906.08937   3214.16352   2537.79889   1481.44434     424.1133    -60.21126   -183.44816   -275.71129   -183.93842    202.93996    209.77703    269.75803    170.68236     98.16809    -90.66355   -402.48677   -251.19351   -273.11799   -263.03323   -254.70101   -333.15557   -397.39222   -323.37724   -133.11695   -148.09306   -196.60871   -125.89511   -167.69102   -181.13891   -335.72581   -201.80575     80.08389     26.75856     398.8479    845.87564    921.94086    903.81611     686.6844    182.08826     59.80068     41.04675   -248.86225   -170.50404   -206.26248     21.92186    467.51161   1202.81408   2299.63291   2808.88465   2603.93551   2487.37906   1578.47223   1049.68055    483.23764    362.60806     48.61936     59.70721    128.42333    -68.66903     77.46892   -201.70086    -256.9704   -186.27717    -263.8165   -267.10274   -281.86577   -226.90737   -241.35674   -334.81084    -172.3469   -321.06217   -313.73864   -267.83416    -59.46446     24.91829     14.56276   -197.90515      -69.739   -125.16166     -50.4506   -312.99876    -242.9006   -312.13194    -59.13011    -53.46727    279.48301    155.44107    139.72573     115.7205    125.64785    260.54973    -67.62227      3.13789   -205.76305   -175.95055   -333.48998   -299.07385   -366.83456   -263.57516   -393.64887   -331.43387   -198.67438   -345.05078    -97.62924    -93.47569     55.69214     35.77912    120.73478    177.92529    430.62791    288.53473    295.57485     100.0601     -7.10591     62.65785    -32.40995    121.48422    271.04338    128.75159    194.51603    112.80522     31.06433    -94.25136   -232.20641   -259.51556   -286.00243   -307.04362   -271.12242   -295.62117   -251.92389   -249.06808   -267.71523   -324.66207    -48.96658     -1.63886    -56.65709     -1.15895     90.30193    -31.25572    153.51268      6.23396    173.33424      1.97772    135.73359     -18.5505    345.53113    213.52043    433.16405     234.8408    150.64827    154.52048    160.58855     76.54713   -104.38665   -217.98579   -191.95365   -102.66688   -160.49629   -110.54358   -350.92233   -265.85269   -152.90767     22.75424    206.71043    161.40166    124.31814    224.33423    -61.51934    320.34018    285.95889    187.84576    660.91333    644.64629    653.60136    819.27803    733.16676     632.5879    538.36828    559.15447    243.35752    268.32675     18.61516     42.77213     10.68871    -94.68601   -144.85736   -193.98385   -162.91084    -47.01638   -137.01534    -80.72534      53.1979    224.63732    -29.53939     66.32443    -93.35492     -44.2134   -159.65684      0.85059    -86.53328   -280.54034   -484.63008   -126.20989   -291.83348    -346.1234   -384.35115   -448.61886   -196.91276   -309.53199   -215.12689   -391.64872   -409.58571   -212.47071   -413.28799   -124.75515   -165.17422   -119.43744   -193.17797   -278.37768    -30.26486   -115.37934     -2.94523    -78.45667    -17.22299    -71.46592    -133.6955   -120.49223    -77.85186    109.72709      -2.9576   -129.90414   -221.63759   -229.78226   -157.79504   -119.96184    -64.17171   -294.63293    -52.95135   -159.50255    -47.23714    -13.24362    -138.7256   -101.50524    -48.79838     -71.5103     -0.36617   -103.13686    -14.81195     68.56816     82.10195    -53.46473   -199.71165   -229.62568   -156.50217   -200.52707   -267.37285   -146.87788   -214.43289    -85.87834    -62.96191   -203.15667    -123.0196     59.68784    -40.81306    199.65335    237.13474      155.721    235.88105    248.62563    279.78429    275.42893     96.17095     81.90065      2.62637     91.28571    179.76796     92.98526    -23.55805    -35.09029    -84.89546    -54.23345   -248.88905      1.56801    -93.36981     37.90826   -108.69897   -168.04665        42.84   -141.97423    -109.5418    -83.64588     13.28505    -66.79526    -82.93435   -195.85441     36.13471   -182.91135   -314.54961   -210.18226   -223.62574   -115.00569   -377.56987   -354.58203   -204.16135   -163.49725   -226.34931   -378.63784   -206.08166   -264.49709   -196.62278   -227.92922   -266.06817   -144.59472     -65.4875    -32.46508   -319.57075   -331.89455   -333.46284   -187.23883   -399.47136   -353.05175     -267.776    -87.37479   -129.02652   -387.69941    -348.8085   -334.21266   -575.50393   -118.91226    -348.7362   -254.88102    -365.7289   -353.86229   -674.70953   -546.78204   -435.92765   -673.27802   -598.60873   -515.17794   -626.35156   -641.87529   -752.21815   -431.91374   -951.37505   -781.95998   -603.49757   -671.31177   -805.01945   -833.79956   -863.26538   -754.79132   -788.96513   -938.82692   -590.74191  -1339.91069   -944.37722   -980.12466   -955.57595   -982.61326   -801.90103  -1033.65582   -955.17689  -1063.41323  -1012.70598   -907.31023   -609.67794   -981.52559   -806.28615   -590.05815    -807.7559   -468.46556  -1351.93732   -917.05942   -936.14269  -1358.99165   -898.79412   -764.88632   -754.37403   -851.99818   -703.85151  -1217.46868   -780.00812   -834.44387   -763.68174  -1315.91393   -809.03074   -1170.5556   -941.44002   -742.88142  -1037.73768  -1130.09635    -912.4337   -998.82543  -1059.14731   -674.57201   -526.95261   -773.36491   -922.35421    -795.3163          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        8.00   -587.93382    -90.77862   -146.68641   -187.52501   -586.82829   -183.02223   -327.13853   -273.07475   -514.25078    -275.6044   -307.53015    -77.11564   -242.02881   -227.21362   -561.45911    -53.69854   -357.80081   -205.78495    -299.7628   -195.43747   -379.23592   -165.83498   -101.70284   -171.50583    -13.87445   1235.25077   3188.47686   4641.53173   3450.60638   1242.90632    380.23157      3.17009   -458.36167   -227.63667   -260.72406   -419.48811    -435.2235   -367.12932   -342.88273   -371.42621   -263.86977   -398.25493   -297.70381   -540.18135   -359.02775    -316.1626    -473.5026   -289.61796   -380.89394   -472.00348   -375.87715   -453.30212   -474.15115   -381.18301   -326.18505   -384.29133   -139.27987   -145.57485   -181.11097   -293.96614   -284.67136   -405.25168   -345.87992   -249.23227   -366.87566   -235.40835   -422.00916   -173.21025   -183.81692    240.83143    843.45274   1869.75523   2248.51834   2288.01345   1264.99821    440.29432    368.98099     78.83371    162.94217    -19.97277   -279.36675   -146.10219    -281.3015    -92.10755   -244.27202   -385.09273   -298.10959   -159.60359     142.8619    730.33519   1760.22795   2803.00776   3243.19869   2505.45698   1746.80772    421.64211      2.02121    -95.39734   -278.86631   -284.64101    -22.57079     273.7938     304.4225     186.8871    178.20033    -98.57739   -221.42651   -243.17238   -394.52409   -251.39076   -339.76126   -450.50957   -374.73373   -326.12013   -454.62249    -279.8647   -272.07187   -276.51325   -124.47009   -140.03473    -260.4204   -342.73634   -140.18585    125.59436    409.71501    851.58188    850.96548    753.08463    728.07146    226.69692    264.98197   -110.27011   -234.00816   -101.89161   -123.72885    -62.29588    490.48778   1088.68065    2187.8212   2747.83186   2701.37484   2615.52724    1891.7977    938.88637    474.88185    408.44102    212.22737    255.12754     84.61614     22.42669   -158.44234   -287.17544   -146.80064   -250.70348   -219.25507   -357.17355   -149.30339    -229.9465   -248.95371   -179.29235    -258.6309   -298.94003   -307.33808   -126.68045   -121.05196    -46.29581     24.89074     50.03212     35.40729   -133.97613   -231.48881   -244.60581   -315.44547   -105.25792       61.638     -74.4544     111.7156     201.3848    194.19543     77.89137    207.40657    101.77648    -93.18195   -107.57245   -229.24361    -187.5123   -123.76151   -294.33709   -235.02563   -211.46931   -360.71312   -249.91257   -327.14627   -300.62663   -160.70568    -42.06572    -38.94131      -93.696    110.26025    161.79501       7.1408    321.34362    222.07105     70.42046     26.35282    -17.08092    -30.86739     -2.49144     30.67564    170.64568    149.75928    133.56052    -14.12545    -69.59298   -109.22489    -242.4574   -340.54113   -333.99283   -294.23352   -243.67219   -350.94564   -172.83196   -217.75146   -226.78276   -178.28446    -65.88522     13.17093     31.78899    -34.55786     74.92268     61.64956     55.48059     34.81572    103.29223    218.56967    129.38221    106.62942    311.67533    382.62153    276.80159    295.03625     256.1645    150.44239    -18.37145   -104.12753    -12.01711   -139.21898   -144.44661   -175.76844   -139.93392   -214.45655    -126.7045   -189.17004    -96.98208    -60.26397   -154.24801     22.55674      76.7612     27.29799     64.88654    364.71879    467.26044    550.57626    659.49117     855.4938    850.42668    709.79052     701.3748    754.38396    486.79268     514.4824    177.22155    176.29038     99.40821    -87.76986   -217.24971    -41.73993   -164.67465   -104.47349     79.36515      4.63005     56.11043    -21.56688     236.1892     47.30162    114.73566    -28.52614      7.29805    -66.07429   -345.43209   -259.88336   -265.75381   -164.87673   -438.75592    -75.71039    -406.0112   -380.70754   -138.26524   -472.68439   -331.37734   -164.79791   -197.99561   -223.31767   -132.76543    -79.61678    -209.5233   -231.91805   -208.97538    -96.33518   -148.08416    -26.30069    -42.27723     67.10028     31.20905     -30.1679    -25.87311      3.14663    -16.90959     54.57132     -69.3207    -73.40548   -243.66058    -84.88978    -71.22166   -161.84813   -169.41187   -202.27899   -229.19064   -115.91042   -123.58666   -206.79777   -168.33728    -76.42224     34.71561    -61.15061    -79.80691     46.42271   -112.92772       4.0278    -58.79466     31.14458     -12.3845   -252.18664    211.46362   -170.07532   -148.63264    -96.04579    -134.0337   -161.45561    -37.57268    -14.22067    -75.97157     -18.6954    -86.60865     57.14216    281.83339    233.01464     97.17987    263.55895    250.94172    626.36646     81.97621    231.68666    172.78782    187.55797     53.38133    167.48504    132.75521     10.52207   -197.29075     -88.5018   -108.74206    213.45033    218.56345    -61.27764     56.69402    -51.92371     -81.2449   -165.34054   -144.88149    119.06915    -71.79607    -124.0028    -90.25601   -111.51828   -101.33088   -228.65865   -231.29294    -74.93107    -86.14293   -178.13274    -17.76277    -157.1999   -294.12983   -233.23634    -30.35074   -398.00104   -167.39766   -259.26735   -154.74689   -309.90928   -480.00924   -103.22984    -124.5691   -199.93735   -320.55953   -252.79494   -161.25717   -185.35274   -256.46338   -277.87571   -230.59979   -368.98362   -500.21229   -304.50701   -397.24964   -345.33552   -264.13014   -393.36509   -447.31329   -103.49743   -337.74849   -316.30055   -209.39651   -713.73646   -318.39227   -403.37751   -353.20782   -429.73104   -375.24558   -294.18139   -525.18483   -566.49745   -594.05281   -768.33121    -465.5807   -779.17378   -674.49415   -812.90635   -889.54824   -942.07388   -784.61468   -850.78537   -848.59398   -887.20747  -1076.28552   -837.41741   -683.11684  -1112.69138   -813.45703   -791.19274   -772.30798  -1012.70925    -792.4472   -830.89445  -1082.74098  -1078.63792   -936.86061   -853.65777   -991.87795   -934.66942   -941.50753   -1078.3569   -974.50893  -1068.32061   -891.47191   -843.14338   -787.38032  -1120.67411   -502.65596   -987.63616  -1218.12697   -854.21212  -1320.10684   -770.61832   -793.59552    -938.5761  -1035.92202  -1347.20995   -960.71978   -963.85239   -985.46135   -614.22162  -1233.67923  -1205.40466   -398.50472    -791.3183   -787.36054   -449.51977   -945.61778          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        8.50       -597.1   -295.87163   -413.53669   -190.30031   -538.81569   -286.05207   -562.90885   -225.47984   -410.00697   -335.63393   -303.73155    -56.18739   -189.98199    -40.55108   -609.19592    -37.31691   -225.53223   -269.00853   -276.66448   -242.74635   -379.37026   -187.33772   -162.08955   -324.73568    168.60604   1259.31999   3177.89963   4532.48446   3276.50355   1315.99719    152.52537   -218.94264   -441.07924   -333.72129   -503.09634   -431.33948   -416.04107   -347.81232   -314.89747   -339.78983   -591.53809    -424.1639   -291.52766   -427.68459   -504.62619   -394.40899   -438.09652   -342.29591   -336.09799   -471.67653   -503.20552   -595.58995   -337.12129   -506.46561   -335.38309   -274.55122   -334.94569   -288.30414   -143.68095   -106.83365   -262.27446   -450.49364   -377.65335   -341.67723    -367.5336   -314.78767    -337.3202    -236.1397   -245.68627      68.0621     696.4566   1831.86687   2540.08322   2128.15445   1161.60488    489.23889    268.64591    -32.74781     18.32908    -336.2383   -228.17147   -131.55512    -168.9661   -353.39458   -296.57273   -379.22416   -126.96807     50.01931     23.77704    897.63643   2003.59499   2922.37409   3523.96737   2699.19057   1492.03019    427.58055    -88.53215    -74.27712   -160.01765   -240.60964    -12.63701    349.34048    295.89647    333.15593    -54.64115   -164.16977    -304.5569    -337.6084   -310.91124    -201.4944   -302.15788   -372.58904   -199.43079   -372.77879   -226.40036   -268.07549   -193.74791   -106.36033    -94.44136   -282.93702    -224.4509   -315.26638    -98.84536     15.72728    426.61565    752.91366    981.37978    897.24152    548.90947    338.51537       9.4959     31.12054   -166.27465     -192.619   -105.70699   -193.24808    466.13945   1193.38949   1974.78335   2784.20725   2878.72009   2256.93139   1723.03175    859.84907    556.75121    344.65383    173.36769    139.59554    228.55742     -5.97604    112.40064   -240.51031   -215.58688   -259.32527   -308.97121   -212.29998   -147.71788   -218.90528   -301.40342   -306.43481    -264.8628   -114.29027   -196.60007    -85.67766    -65.35897    -98.40538     18.00563      1.65506   -109.00933   -200.32211    -91.45113   -163.24013   -138.49632   -163.95989   -113.22635    203.28571     187.3849    106.92671    211.03956    199.31051    -68.32718     68.42056     20.00632    -67.00757    -43.91988   -215.13146   -215.28431   -247.74442   -471.07808   -253.45034   -194.85372   -394.60789   -155.69815   -191.22748   -132.99524      -81.899     74.14507     234.6769     -30.4503    293.41893    235.38231     98.51817    143.37569    184.97244     66.89585    -60.81842     -8.07412     -0.17461    129.19662    213.01145    102.72605     81.80427    -36.44424   -296.54901    -47.04146   -236.45789   -274.04612   -281.06374   -339.71686   -383.51428   -351.68168     -340.011   -146.74535   -406.26596   -196.87491   -195.02078   -209.19596    147.54764     55.11633   -153.81332    136.95426    142.14128    173.92985    125.33054    201.71718    152.07741    220.75285    428.46147    421.06519    444.81219    222.77313    283.24175    -38.72715     52.39625   -149.08692    -31.08351    -212.5887   -209.72211   -232.91805   -360.46987   -237.90054   -160.27638   -123.75793    -89.29272      32.8475    105.20408    125.49412    182.01987    111.66847    306.75634    453.84205    298.44174    668.24352    630.78978    757.12634    603.95479    551.43396    660.73952    544.13144    449.55144    246.74847    152.72763     145.3553    -33.91883    -47.63251   -104.61845    -45.89074     -0.59565    -70.91279    -75.61191    -74.11309     45.29528    118.55986     62.22631     29.07478     11.93788    -71.35235     11.74164     17.91007   -129.56822   -129.16285    -71.31036   -199.27764   -294.96429   -237.06187   -364.24723    -326.0742   -335.90016   -199.61315   -241.91017   -260.08694    -307.0955   -170.51666   -219.30136   -288.18012    -64.30394   -158.22716   -388.57304   -260.90009   -178.83543   -215.80196    -46.71845    -85.46475     90.06219     64.06387     74.53161     89.65271     181.2336    -99.09025    -86.79616   -124.11187     -4.84405   -260.90089    -187.8664   -165.72698   -427.74031   -260.25424     30.94432    -15.57455   -164.35394   -107.68047      5.86457    -47.62616   -139.62142    -43.70611    121.42217   -167.44011     94.41591    -121.8003     -46.7093     13.87751    -53.13635    -28.87723   -163.14099    -58.82553    -97.85249    -78.87927    -69.92742   -224.14352   -134.80354    -34.66896     12.01943     125.9174    138.66501     70.70158     34.85805    180.84364    189.01123    171.17055     97.67106     63.14171    216.76218    235.90362    257.35604     67.98754     90.58287     138.3426    341.88528    275.94539    -29.24492    -20.61139     -0.97878    -17.13309    140.32588    135.55449   -199.27815    -60.87979    -12.40319   -177.69855     52.03713   -172.11996     -81.7259     98.03717    135.98675   -168.04135   -296.34194    -72.76126    -21.78051    -62.70181   -178.15324     30.55028   -121.66544    -198.6208   -427.01497    -214.3963   -170.26872   -251.80052   -387.92387   -253.30732   -203.74589    -13.34853   -461.68138   -103.22602    -165.1897   -323.62423   -278.50137    -84.60241   -148.57276   -332.62445     -265.432    -197.2118   -259.04662   -409.59912   -252.82698    -190.5699   -188.65373    -66.00835     -444.476   -114.25335   -169.55616   -620.59832   -246.51117   -238.68623   -418.32789   -443.58277   -483.84571   -398.89021   -304.96286   -541.79692   -447.40636   -738.33299   -587.10182   -431.75546    -705.5383   -729.88998   -520.18709   -535.87736   -766.98503   -585.58797   -660.76484   -765.82643   -739.44864   -733.16033   -754.99553   -980.11089   -724.13926   -802.13136   -994.17253   -971.02484   -630.92732   -962.60375   -690.37148  -1136.96272   -957.66208   -922.61955  -1118.14591  -1027.47329   -699.81633  -1017.41239   -854.60393  -1021.19516   -738.70364   -671.10585  -1104.35926  -1473.78458  -1168.64133  -1001.29987   -751.84666   -257.86656  -1019.04367  -1103.85605   -1227.6159  -1042.48496  -1106.31773   -664.73838   -868.49188  -1036.92026   -750.73026  -1164.56058   -687.42972   -897.42224   -872.64445   -1246.8326  -1091.85167   -696.75299  -1112.26201   -572.10541   -697.22954  -1111.82358   -924.56041          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        9.00    -401.4474   -472.38057   -600.62235   -241.18606   -398.60923   -429.09755   -555.35529   -231.93852   -318.94381   -351.93288   -242.69558     -49.8945   -171.78944   -179.64807   -358.82043   -108.57254   -152.23378   -180.96794   -269.24183   -381.83663   -341.14463   -295.78977   -311.90341   -194.80706    310.36413   1241.49164   3015.52172    4529.3105   3310.47226   1444.38838    320.26922    -231.8257   -267.02778   -194.35731   -430.80757    -375.5817   -337.85771   -243.09843   -271.12794   -302.08512   -359.27561   -297.98851   -238.98056   -340.56701   -489.51989   -291.89764   -486.85616   -507.10395   -216.91342     -449.806   -268.54223   -289.58677   -359.40601   -184.69539   -417.31056   -234.38059   -251.81483   -265.08246   -154.60003    -170.2278   -186.43641   -366.02059   -400.89699   -269.72338   -318.86253   -290.63197    -430.2675   -297.70653   -359.15813    -33.87531    579.30218   1742.89021    2352.7397   1866.42526   1144.33342    590.96114    173.80892     19.54121   -138.33146   -330.37966   -168.90789   -243.00248    -183.5738      1.94761   -524.75057   -328.11426   -228.88032   -113.16611     86.87788    643.35403   1621.25059   3018.15371   3290.67783   2371.53281   1202.44929    352.65007    113.76785   -161.49539   -275.45428    -83.97518     87.09216    117.45167    250.51919    262.17952     12.64607   -236.66983   -203.12037   -283.32851    -459.4134   -340.38878   -339.35687    -302.3042   -328.68687   -326.32003   -328.83471   -231.35369   -311.35072    -214.7905   -199.95729   -178.12542   -218.04359   -241.74484   -186.37401     127.2649    425.99486    891.35496    926.49596    836.83676    665.93992    287.60499    -30.17665     -4.34493   -243.02173    -74.57303   -230.69321     23.51631    417.33201   1280.66076   2001.30582   2743.41657   2757.40767   2426.32974    1875.9177    856.37794    376.83353    136.46228    139.37787    113.21436     62.87328    -75.66991   -107.32432   -185.56673    -73.49639   -301.68188   -264.08617   -202.68975   -336.47443   -385.74009   -330.31927   -204.77009   -366.89479   -245.08417   -275.85433    -43.98742    -45.08458   -172.57401     66.85276      1.64292      -87.688   -100.67412    -96.91696   -327.51342   -327.77822    -55.19712    -29.78803     31.20842    246.47114    223.10562    224.89748     28.53267    -16.90745    -49.10083   -148.67928   -125.73478   -207.26716    -78.16722   -281.91102   -244.15504   -246.51407   -207.35179   -218.27897   -330.38697   -298.04169   -170.39583   -186.66078     13.68095     89.77177     24.84972    238.46662     235.9316    387.44311    361.61497    149.95014     17.51945    140.47162   -102.94041    121.79768     39.09716    148.39371      5.30906     52.70475     131.3557    113.23547     42.74888   -147.79568   -341.49386   -200.38136   -213.66834   -377.21605   -382.36406   -276.22266   -343.17041   -319.21492    -327.1727   -253.66742    -53.89804    -88.71781     69.03641    145.06231     -0.21133    109.23973     50.64232   -101.75904     152.5254    184.03037     91.12991    258.95034    304.32101    540.49183    322.79219    382.49785     262.8773    137.42547     29.39622    131.61428    -288.6805   -125.32004   -313.26452    -37.65995    -283.6717   -204.91332   -158.04992    -255.4457   -194.55252    164.61777    -23.63276    116.44559    264.19759    -19.00678     23.33415    427.09111    354.69295    655.26589    673.09342    622.99639    742.06589    772.28816    717.21793    645.50527    394.92326    138.76651    155.44151    123.40048     78.69993    -77.60131   -175.22645   -177.56157    -10.59921    241.25948    -42.42575     -0.28516    -17.47527     50.73857    160.96989     47.32296    -50.70692     -9.24027    -67.27091   -102.97435   -206.08627    -91.29433   -114.19223   -182.70709   -331.92532   -362.26725   -348.03399   -398.72086   -425.00601    -318.2241   -210.69271    -219.7998   -183.20858   -189.51885   -194.56597   -204.28868    -91.89562   -194.06791   -152.86163    -82.46428      5.92441    -211.2607    102.00782      65.3882   -101.99878     58.38401      1.09035      6.43143     24.17233     64.47427     41.83796     -81.0126    -64.72025   -121.53746   -233.14062   -257.99263    -148.9395   -159.16596    -96.49871     57.72604   -174.67048    -78.09992   -161.74033      44.2164    145.41226   -165.77783     -37.3212     22.29168    -50.55462   -142.17006    -74.36618    -13.95364    -35.30177   -111.05906     20.53147   -137.74027   -107.51217   -213.83447   -187.54384     -6.75656     11.91225    -73.89728    -246.5911    -21.31441    111.31265    114.05662     77.29415    115.56113    229.65239    290.05546     51.98245    379.34674    283.91398    261.79977     72.66954    140.02163    133.52604    105.57697     55.64816     70.45386     53.81031    174.71012      74.6859     35.49009     14.46166    -54.30056     10.99662     -6.97811     43.47647   -146.79319      7.45212   -185.15152   -176.94861   -141.55065    -37.08834    -94.10285   -198.91483    -324.3132   -151.18594   -189.19687   -181.07293   -158.05393      6.69033   -238.58954   -215.27104     58.09198   -315.15763    -64.95877   -277.80057   -195.35957   -185.59474   -135.14341   -443.01675     -77.3525   -263.42444   -276.07888   -164.06711   -278.06046    -200.8057   -288.09431   -306.97433    -55.47883    -254.8122   -370.39609   -212.39281   -309.35089   -210.53023   -370.60636   -222.53973     78.19348   -139.20812    -163.2123     -1.15939   -343.99675   -381.54023   -187.31682   -402.37809   -160.99989   -159.92223   -468.27124   -423.13417   -475.68767   -662.21374   -556.53077   -841.70957   -639.35778   -809.93493   -796.97276   -751.38259   -620.44424   -695.15252   -630.10617   -714.66204   -763.29118  -1023.35833   -885.35016   -968.29775  -1001.11182   -580.43078  -1102.71077   -769.76048   -918.35609  -1025.53732  -1060.45393   -1237.9229   -876.52228    -734.9818  -1131.51743  -1134.32953   -648.43242   -826.25406  -1243.79657   -811.20811   -908.47549  -1141.47933   -951.94305   -559.08021   -678.64488  -1451.24386  -1098.90561   -830.88837   -817.62626   -813.95824   -970.41622   -1072.5898   -886.90583   -783.85049  -1049.08385   -604.23398   -833.30035   -776.09691   -319.43322   -914.28268   -484.00443   -686.70696   -679.39644   -829.48505   -497.72332   -781.95352    -837.5836   -900.14956          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        9.50   -242.71322   -532.79096   -453.71921   -277.32867   -298.47993    -423.9231   -460.08784   -300.87322    -291.0424   -279.34718     -246.701   -217.00433   -182.98472   -350.27499   -402.46226    -409.0935   -128.76576   -207.91398   -331.24999   -493.44962    -325.1963   -190.26234   -475.96031   -104.30166    323.03888   1244.37474   3038.78835   4482.38476    3488.3668   1487.91459    321.13965   -209.64478   -189.80922   -295.85082   -250.97682   -251.53609   -304.99301   -450.37093    -189.9767   -309.34374   -425.30432   -406.15832   -284.17876   -358.04456   -487.92128    -349.1688   -344.23897   -280.68373   -223.55375   -230.23839   -257.81757   -349.51854   -387.05848   -254.38262   -415.95229     -94.6962   -307.24799    -90.31026   -258.38825   -221.00821   -427.02146   -392.23724   -137.80084   -240.04004   -324.94163   -233.10109   -338.47092   -121.10034   -233.37442     29.00353    668.92786   1685.59055   2162.69857   2167.69543   1326.94842    624.61545    268.25174    111.44041      41.2536    -215.8832   -314.45487   -174.07988   -147.52488   -335.05044   -224.61977   -234.15386   -219.90319    -58.96085     85.29841     668.1717   1936.62162   2857.13767   3175.97391   2583.55652   1205.45835    360.17724     39.79868    -162.4721   -122.32975    -120.3961    136.53151    231.23607    255.55803    193.30777    110.34299   -207.20505    -213.0809   -310.98364   -254.89046   -464.85863   -448.33985   -366.88797    -401.1303   -284.74228    -329.1718   -190.36306    -245.0835   -123.72547     39.10081   -230.76898    -232.7048   -295.22094    -58.47014     70.90791    365.89668    634.94316    882.38029    640.74873    562.35178    230.33132      23.7057     23.56409   -164.65955    -66.36813    -60.78816     46.67156    499.84113   1428.18289   1996.30904    2908.5491   2872.54481   2394.77778    1589.4709    971.99948     472.5712    292.26303    179.45796    178.27911    153.60421    -89.73875    -72.89945    -64.87669    -268.0577   -311.84062   -209.08742   -227.96622   -224.42153   -221.46129   -277.61301   -157.70034   -197.55364   -173.24382   -172.17946   -191.17245   -109.26061     50.42474     51.24669    117.06998    -62.39192   -198.15749   -135.93018   -207.12324   -295.89521   -254.04163     -6.12057    131.93608    196.76344    281.64497     172.5394    269.99605    182.00089    -42.05753     -0.66439   -120.70271    -97.34239   -233.72618    -194.5268   -317.82466   -361.38167   -281.59784   -218.07076   -237.48033   -263.99563   -238.36851     40.65666    -84.47265    -72.08894     44.32922    172.00488     222.3199    181.97056    245.88951    188.10585     17.46203    -83.43339    -47.43708    -12.58546    110.83081     91.07448    282.66034    208.64563    249.35604     99.65292     36.97313   -208.38394   -261.85273    -290.2661   -345.10795    -361.9028   -373.72548   -271.23114   -382.85962   -282.64276   -235.18318   -150.81594   -169.25591    -11.72585    -43.68897    111.73093     65.41826    205.89659    213.67874     60.15784     96.58004    131.04495    270.83079    158.45877    203.69868    358.62045    245.10191    226.86639     222.2994    214.87294    -27.86869    -13.39262    -73.83734   -144.20847   -133.37966   -154.56357   -224.00425   -218.40768   -282.12918   -158.61673    -236.9934     50.50243     32.48019     15.59373    153.05164     46.64858    157.47424    353.21495    491.24644    636.02438    569.29441    595.59226    740.56418    690.41814     787.2442    554.93289     466.4576    243.45435    290.36481     92.07743    -53.55017    -54.59147   -184.92391    -16.01318    -25.42386   -165.96586    -58.81669     95.51787     16.56205    -32.57953    186.50526     67.33709    181.56377     13.54391    -65.14807   -275.30754    -208.0116   -134.17506   -148.80603    -74.64179   -393.95826   -463.11856   -307.55077   -247.81292   -238.45739    -294.6636   -133.72442   -476.50035   -249.57069   -347.71821    -90.35455    -323.0274   -206.09485   -135.08708   -119.87008   -237.27311   -184.26024    -23.48408     18.58838       60.102     150.9699     48.60283      53.1574     -7.71416     52.52933     83.18528    -60.76013   -145.91496   -181.94017     34.71404   -137.79948   -187.93219    -97.87502   -107.93176   -260.24886    -62.29726     72.01099   -244.83583    -64.55016    -75.97405    107.52473     -1.28912     -6.97618    -52.76443    -56.67347     84.77049    185.01645   -150.78731   -181.86456   -137.45694    -43.40862   -119.35412   -125.91652   -216.40949    -15.46935    -65.23051     28.12414     -65.0938    -75.79069    -32.40304    -75.11171     64.41983     76.63167    175.32242    261.56437     111.2294    226.50348    319.93363    209.72525     151.2828      319.115     52.03024    232.55574    169.14019    157.18118    -15.14049    -40.17032     80.83369     71.67988    -39.73678   -227.07941    162.05996     -2.49505      5.95102    -32.76646     24.02071     -0.05974    -237.6283    -92.08644      79.1618   -170.13818   -282.65836   -300.63756    -98.06349    -92.54027    -20.66248     -219.116   -124.13557   -185.59288   -276.12594   -233.96733   -245.65838     -304.074    -232.8967   -230.91715    -34.35319   -304.68041   -380.13313    -32.78028    -81.74675   -226.75116    -13.77814    -60.59502   -175.68415    -225.8895    -343.2856   -132.99117   -158.38577     -184.585   -131.49894   -275.09745    -96.33708   -248.39952   -193.21931   -178.64257    -33.49683    -76.18627    -289.0105   -401.59597   -109.48658   -346.15766   -440.93146   -675.76169   -504.97529    -417.6229   -343.80689   -669.71601   -581.54037   -469.10567   -765.36379   -531.34208   -546.16454   -606.92988   -724.25917   -866.92424   -708.68315   -765.92507   -764.83117   -858.24432   -811.43039   -881.92946    -616.7728   -736.04735    -846.3265   -835.77156   -826.57878   -1326.3177   -985.97356   -649.02415  -1031.54061   -549.44243   -959.74368  -1058.14775   -980.37551  -1045.08894    -819.8359   -976.06765   -771.18265    -907.3494   -800.01415  -1027.03866  -1212.39939   -870.78423   -816.84978  -1102.31934   -820.02214   -973.04438    -1361.748  -1036.09383  -1044.44455  -1360.09179   -887.04083   -834.95843  -1160.54811   -843.80125  -1096.73912   -828.44646   -695.51223   -798.17438  -1199.74921    -777.3263   -730.24565   -345.81041   -614.86759   -373.31971   -823.96946   -486.44227          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       10.00   -173.87323   -395.07866   -316.26338   -290.29988   -250.39292   -388.88719    -437.0974    -456.5442   -272.94137   -257.80138   -385.24766   -412.05909   -161.65596   -268.49359   -557.57216   -398.58274    -39.35268    -325.0771   -287.40293   -272.42737   -240.76221   -149.62208   -435.77485   -136.15584    187.10376   1145.52805   3313.98884   4411.79218   3406.29263   1470.50881    160.86497   -340.01277   -170.34005   -385.13487   -401.12825   -118.44941   -225.72676   -178.74957   -255.13453   -346.05589   -178.63347   -261.59051   -241.84868   -350.14657   -328.40216   -292.57339   -511.25161   -280.58156   -264.57193   -303.16983   -310.54507   -231.99885   -208.73792   -402.72349   -467.67028    -550.4247   -336.67146    -66.94635    -51.84683   -174.39738   -357.63591   -301.52274   -277.04515   -316.32087   -350.60421   -327.65023   -369.70388   -356.88489    -95.05303    -99.02556    817.86237    1908.2347   2525.15765   2089.85204   1208.45939    484.16024    146.27847    114.51614   -208.82154    -176.7548    -280.1946   -299.14034   -183.03803   -230.38479    -357.6193    -66.10295   -264.88529   -123.47197    -68.40549    654.30924    1794.4761     2878.776   3237.43895   2340.20494   1356.54679    282.72893    -65.11672     14.60494    -45.39154   -251.34913    137.07138     148.9293    367.74223    163.31881   -114.81679   -180.06466    -143.7375   -386.65025   -330.52631   -160.31657   -225.07734   -278.29924   -170.26709   -469.14611   -243.71152   -284.52691    -334.9908   -159.83484   -224.81168   -344.95991   -330.56666   -330.40615   -102.99542    121.85211    420.04889    616.51962     977.5892     960.2708    595.65189    282.38816    -35.85491    -32.96835    -212.2379   -192.59434    -79.96889       26.401    560.75533   1299.20057   2045.30088   2887.51005   2838.30939   2213.76172   1719.01462     792.2315    451.11434    263.85914    192.69794    121.92409     161.1024     52.35614    -25.33414   -120.93621   -336.13109   -282.28263   -385.89435   -190.91132   -391.18027   -286.90522    -379.2978   -162.78655   -301.49039   -260.19044   -163.81007    -97.77537     23.50866     13.82143   -201.60771   -193.53737   -138.12431   -315.25054     -94.5677   -298.61761    -203.5695   -229.56713   -151.48125     97.68346    228.22262    104.84877     270.1212     91.56401     13.48851    -39.32378   -180.08268   -147.19066    -68.22965   -151.34384   -240.11215   -227.12435   -419.31552   -151.49935   -175.93801   -251.58118   -348.02342   -183.39745    -31.16214    -64.70514    105.60466      6.54514    172.34505    230.73344    250.92586    218.95285    177.54869    193.59002    144.35852     -49.2652     53.52583    107.74164    203.29382    219.27498     44.46217    -30.85009    -37.41348    -93.66192   -122.39277   -181.88974   -243.24737   -201.31575    -305.8527   -336.49162   -406.93333   -410.10559   -228.85759     -250.485   -155.82919   -275.56647    -48.05601     96.92703     -42.5739    201.60346    205.86932    113.58533     66.66542     -7.04781    239.07527    346.30649     336.3587    416.91305    306.99954    378.98356    257.81365    517.96057      3.44888     -83.3974   -127.22301    -63.19892   -233.26185   -442.89791   -215.15525   -353.70655   -154.73954       6.6251    -82.65486    170.74216     27.14411    130.21359     207.6157    -39.63915    259.10539    220.97695     288.9371    525.21075    513.90103    537.32208    709.56391    771.26619    766.57175    696.71032    527.11359    411.20354    398.66466    285.66212     46.36453    187.03936      3.57099     29.03439    -35.99853   -187.54978    -60.02919     32.32887      0.34839     53.63828    147.56198     92.54104     79.94974     43.42329    -58.59207     -42.4233   -146.52898    -39.29758    -173.4484   -340.21564   -352.56393   -174.76296   -358.30603   -158.05541   -370.34937   -279.70739   -315.99671   -234.86828   -492.94498   -294.50155   -210.11693   -202.19151   -141.41004   -109.58183    -96.34256   -200.40112    -57.69411     42.76399   -132.83576   -130.32482     92.58331     75.96942      77.2126    126.25856    -80.95773   -104.16005    -18.76952    196.01208    -30.25561    -45.31039   -151.39146     26.83527   -248.21206   -252.15387   -164.24959    -54.45806    -34.96819     -5.81704    -51.59168       8.1338    -82.29863     -19.1989     -3.81059     84.22003   -118.85708     75.76702    -39.73954     24.51426    -34.85295    -171.8506   -133.40552    -72.41804     -5.75795    -16.32529     -9.94188     -8.70529   -146.44215   -240.07091    167.15796   -243.33736   -266.99252    -88.37334     15.63655    163.75573     40.81874    203.16827     32.62291    132.86013    314.65224    202.84674    103.21591    292.43262    118.22523     89.77428     82.90191     44.19791       66.945     159.2328    -52.90566     44.92733    175.79904     31.33095    -55.64592   -160.44183     66.28228     40.29476   -210.26623    -69.12036    -19.84271   -291.45888     -224.403     56.53529    -16.35234    -194.4334    -99.57429    -55.20761   -125.32651   -508.35717    -61.81012   -373.97178   -482.56649   -217.47985     21.93328   -226.48721    -181.0971   -344.39741   -241.13544   -428.92418   -322.63588   -171.79557   -194.08563    -73.09593   -255.34389   -232.92911    -61.43719    -89.68433   -106.59104   -249.54162   -332.26039   -271.13399   -232.66915    -42.30504    -18.37188    -90.67815   -300.71072   -541.29678   -324.79825    -596.3658   -151.50768   -414.37923   -484.37923   -217.50853   -379.06972   -419.79641   -417.19941   -204.80738   -330.05416   -559.99002   -550.28668   -471.89616    -634.2922   -540.32136   -515.47552   -456.47629   -666.74869   -671.37108   -647.05209   -624.06276   -801.40663   -593.01409   -768.78505   -898.69959   -980.12786    -784.2179   -879.50901   -970.90274   -820.92839    -931.0315  -1006.42081   -977.16064    -757.7304  -1042.14057   -896.34239   -761.01064  -1199.78445   -839.05876   -1117.1294   -783.53068   -945.41731  -1257.40889   -757.47581     -522.527    -940.1548   -844.52061   -753.43609    -780.3527   -924.55044   -618.06098  -1249.59984   -837.13012   -916.77456  -1094.77327  -1022.99279   -680.38088  -1102.02409    -830.4523  -1021.95479  -1149.26922  -1009.83519   -783.94191  -1288.99271  -1265.11012   -876.06415  -1047.97434   -642.46853   -648.82508   -489.24762   -311.56577          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       10.50   -221.75523   -287.14725   -325.81437   -291.48832   -224.63331   -335.64564   -472.22187   -611.57289   -205.54203   -316.20969   -459.26841   -440.78743   -119.27053   -247.80686   -330.73538     -87.0865    -20.42611   -176.54577    -88.97677   -260.93713   -394.54373   -233.24938   -294.92685   -260.67135     92.68538   1185.05073   3189.08005   4457.54408   3415.73499   1351.08069     73.61506   -353.84004   -297.54016   -302.32992   -291.62747   -409.17399    -247.1541   -287.32916   -166.22533   -418.35694   -434.59317   -260.94146   -188.73232   -213.07418    -282.0501   -420.57228   -384.64216   -368.55454   -417.49846   -405.16534   -133.59899   -390.40114    -255.5857   -475.40133   -252.25755   -378.64943   -296.54291   -165.46297   -154.12181   -294.84824   -329.78432   -222.02791   -355.67976   -169.44651   -398.76071   -368.31589   -445.95649   -317.82864   -196.35836     68.86302    732.33957   1626.00693   2298.69471   1945.60408   1035.49084    387.18826    248.67098     21.98189    -93.78355   -329.67521   -450.54122   -375.97729   -395.87761   -215.97564    -114.9026   -358.38561    -50.06906   -152.89914     280.8562    803.69739   2029.12755   2865.36793   3081.13304   2373.25998   1182.94472    555.17871     17.85059   -271.78637   -242.38913   -146.40803    -134.0201    364.60391    216.21165    262.72367     50.95085    -91.21583   -149.77391   -327.59886   -288.97556   -164.73643   -270.65941   -336.98287   -438.71132   -227.47805   -252.68884   -159.01624    -224.2294   -140.81179   -184.34403   -162.52149    -208.9046   -144.04692    -61.68321     66.17528    444.47639    729.73456     949.9631    783.17201    660.16061    274.67549    -66.98028   -147.79345   -215.52328   -139.55256    -41.08044     15.70486    510.60984   1465.26974   1915.30704   2691.63026   2751.99092   2199.69932   1624.37396    921.50265    355.97221    118.91946    247.93142    169.76587    250.22718    234.27059    -42.13316   -119.86975   -134.64166   -216.55862   -453.79404   -165.89949   -387.56685    -162.9705   -215.10617   -169.06921   -340.01051    -141.2437   -261.74472    -25.14989   -140.48076    -244.3115    -55.79601     -6.46329     21.86136   -109.56891   -279.34671   -295.89734   -369.07852    -310.2467   -244.04436     80.98845    314.57262    136.61286    195.70852    221.63131     20.23577    -70.30206    -89.69197   -192.93491   -121.23068   -283.89361   -328.49797     -334.966   -315.35428   -252.21894   -301.84175    -345.1203   -283.97994   -226.74497   -148.18725   -152.57226    -74.83587     64.18128    115.22901    350.65361      333.723    348.64561    121.85256    165.29401    -18.68765      -2.2856      0.59777     44.15261    117.80437    181.75955     96.46821    125.77458    -64.55915     -1.05442   -138.27325   -233.15542   -302.51011   -337.02556   -180.89353   -298.71258   -182.70136   -337.68432   -400.91391   -324.36991   -131.87428    -98.38132   -185.54089    -37.49105     60.57919    -15.43199    -74.89802    187.21186     30.72998     87.15297    256.78973    230.97003    229.89097     302.0692    298.67575    390.51919    237.03892    191.93848    171.64227    116.76322     71.83303   -122.79213   -209.83722     43.17561   -279.93823   -285.18217   -218.86159    -168.2843   -194.32014    -66.25425      6.51093     34.16247    -32.59813     79.05061    138.89603    201.15689    386.10285    554.64309    609.31735    630.44589    855.60761    861.40785    767.06498    625.37254    446.32909    484.85393    353.54128    420.84518    212.08344    -66.63383     18.33766   -199.96109   -184.87412   -122.63166   -131.90706     38.27675    169.39684     73.22527     -7.34887    160.93368     80.02946     14.21854       -84.24   -142.36999   -144.03718   -197.04101   -225.33683   -282.14659   -192.29042   -357.15821   -458.22628   -429.49025   -145.81126   -329.84332   -194.79006   -331.40286   -394.28568   -215.62371   -179.16064   -268.85394   -280.90066    -44.18932    -54.38585   -107.45976   -255.11099   -158.70393    -24.43464    -54.87972    -283.7909     27.24193      39.9232    -33.43423    -42.43849    -35.48065      7.49225     39.66023   -127.65938    -41.11791   -265.34104    -52.27065   -133.81759    -19.85854   -249.52068    -168.7326    -85.49432     63.85855    -32.38846    -76.10337   -116.56343     42.16776   -181.06768    131.81317    -65.11031     91.93745   -138.71541    -77.41953   -193.95888     31.86887   -285.30669    -78.01035    -53.78863   -221.51765   -139.89444    -120.7473     58.45177   -197.31886   -101.00997     35.85781     12.46143     -6.14627     -7.77393     95.47709    -43.54947    311.63033     67.99502    195.46736    112.82606    232.43426    205.88113    283.39315    117.68662     42.24806     32.27831     18.29798     32.77594      6.63092     23.89813    -73.57866    -111.4071    -83.14216   -164.29064      5.84367     -7.81734     95.81654   -191.27781      0.25166     37.30077   -329.58879       4.6284     29.47711      4.60534   -181.33867     10.12714   -161.27441   -247.29779   -164.37624   -129.33001   -112.56853   -465.26884   -192.91193      5.73242   -315.59048   -123.82258    -91.99038    -61.74285   -154.29906      48.6249    -54.81269   -238.58115   -212.97902   -129.29645   -183.85394   -188.06425   -193.32919   -264.11719   -297.62662   -128.01904    -92.98889   -174.42217   -310.72273   -336.07688   -265.89964   -250.08713   -223.40126   -284.68809   -278.57317   -375.91028   -194.11025    -408.9336   -367.01736   -520.08567   -563.53083   -333.76546   -485.49578    -473.3186   -764.42582   -543.05061   -621.76001   -643.54027   -435.68909   -841.41172   -814.09712   -801.96881   -691.77998   -791.17021   -553.00339   -749.44343    -841.4246   -624.36162   -684.32994   -872.16604   -828.30488   -870.43701   -996.41547   -923.30594   -915.12084  -1179.44661   -709.38584   -758.20897   -817.09934   -726.87264   -989.86572   -720.73134   -695.04288   -855.79273   -927.96021  -1038.71656   -482.61434   -918.35456  -1207.26442    -632.7516  -1015.66581  -1095.17992   -826.12056  -1284.11435   -1157.2529  -1169.01161  -1025.57854   -617.40474   -734.95134  -1153.00178  -1112.00261  -1078.40329  -1306.24031    -998.3754   -712.40196   -856.53565  -1197.78619  -1169.69618  -1523.41372   -568.70049   -562.01212  -1034.34418   -546.87343   -185.40589   -577.43768          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       11.00   -286.50909   -235.13555   -443.35499   -291.47846    -277.1096   -311.33067    -480.1424   -498.15066   -102.55561   -260.88123   -351.73615    -469.2172   -177.44814   -324.89075    -95.59989   -242.12047   -257.50177    -44.33669   -141.44155   -361.07804   -430.35607   -223.81734   -217.46703   -277.15743    200.65304   1171.61864    3352.9553   4644.32524   3456.52648   1348.88664     44.16161   -308.97697   -301.59935   -199.85188    -316.5503   -395.43753   -316.80618   -292.52566   -363.71081   -291.41081   -407.14274   -462.91873   -476.06303   -340.17012   -496.29862    -248.5565   -581.83192   -349.74331    -385.4566   -369.85472   -346.11011   -384.04627   -291.27663   -415.74092   -245.34998   -364.85053   -221.13859   -198.01038    -93.25333    -42.51167      -255.32   -340.11406   -124.89182   -367.06587   -260.03528   -374.36578   -292.95871   -187.05667   -160.66841      6.29092     734.4008   1741.64009   2401.53953    2150.4676   1124.86278    419.74945    309.64583    325.64281      0.12667   -503.27163     -435.873   -367.27869   -425.97453   -302.58845   -258.89588   -320.76638   -501.20694   -260.50578    200.42998    623.79572   1885.84311   2783.33097    2942.1498   2203.13677   1167.13825     397.8862     19.14913   -239.84526   -380.62635   -125.81917    153.94747    203.41222    279.56449     196.1232     11.24341   -108.94512   -210.14917    -183.8643   -281.84625   -362.18487   -259.11269    -166.1907   -207.18592   -221.37871   -322.19091   -194.55836   -200.03391   -108.99597   -348.11808   -242.37995   -379.87816   -112.26978    -194.3033    128.51927    358.49474    772.30639    826.17008    830.60696    700.09876    196.66678    -14.21455    -132.6801   -234.17376   -223.83174   -139.53149    107.41379    677.54026   1288.32832   2271.61223   2633.06068    2758.3464   2216.07394   1653.76201    887.18715    497.67731    346.25737    223.98882    146.72577     27.47635    -61.46062    -57.63176   -135.90193   -212.88788   -290.46859   -365.76984   -154.63272    -289.9181    -243.1404   -286.45751   -357.88672   -201.00894   -212.50249   -160.58537   -222.54043     21.83623    -13.22226    129.86199    109.45081     -52.9944    -44.95546   -103.21975    -340.2368   -244.49254    -82.81018    -85.43943     48.46824    226.63992    285.41274    309.40694    221.84945     161.7151   -102.88311     76.92239    155.95746   -286.86777   -208.68896   -435.35624    -249.9333    -360.0341     -318.393   -289.46766   -267.24811   -105.92315   -284.65387    -15.62548     -121.307   -168.59685    230.07418     124.7798    223.94443     14.61768    393.95921    193.95461     96.79181    -60.11108   -136.09965    131.19864   -238.97122    146.84261    131.74071     82.03377     90.93574    121.25954     -3.98906   -124.23471   -204.69553   -400.46243   -335.49047   -449.68637   -224.12024   -237.74619   -281.30842   -212.79919     -98.0499   -347.36057   -223.25256    -70.35458     41.65391     39.63506     11.00531    142.12465     56.83999    125.67042     -9.57142    -50.56807      311.066    264.15819    250.23199    248.77684    243.96581    292.73343     249.9003     148.5425     97.34612     96.07263    -16.41318   -200.19871   -189.91644   -240.92156    -259.4097   -197.69033   -349.15979     -31.3972    -107.4426    -56.60639    -74.99307    197.38262    -82.25034    188.60741    244.37591    186.58639    398.84967    658.96924    643.77717     550.0535    720.83758    773.13887    707.12887    672.48497    429.83148    306.77225    309.91455    141.88216    -46.07869   -126.02832     -22.6459   -137.69846    -44.64522     73.94404     69.19614     34.26821    151.79458    112.38423     98.22047    112.72866   -113.90152    -92.13714     59.82371   -218.78314   -166.06263    -99.54299   -312.15379   -284.39458   -218.08506   -179.80637   -185.51421   -204.23491   -285.10631    -289.7944   -257.30568   -151.19433    -231.4187    -245.5981   -319.45284   -203.81058   -146.27706   -202.30212   -212.90548    -25.18893   -236.67846     14.34254    -18.18252     -3.77733    -61.43498    -70.44258    120.47512     28.74613     22.34056      4.20353   -164.51873     -76.3358    -33.46341     48.49472   -216.93488    -74.30597    -92.99542    -39.14226   -132.52848    -69.17937   -304.09639     61.19365    -53.75745    -35.69224   -112.46763   -101.64258   -136.71751   -100.20666     96.71042    172.98278    -138.2736     73.65117    -62.50528    -77.30164   -229.67356   -232.96174    -69.90345    -82.45252     -8.64813    -156.0742     -6.67572    -153.3247   -164.81381   -111.19768     90.21301     64.80287     23.61923     123.1224     191.7847    127.27518    365.31559     100.8716     13.49091     100.0048    206.00742    158.25121    209.00305      9.71942    -23.04355     129.5719     21.90372     76.12081     42.49797     76.52522   -141.50427    138.02195   -245.80477   -256.64584    -37.27607     18.33101      8.77333    -57.84792   -130.80555   -183.60685    -78.35153     -1.84386   -117.61984     40.72519   -256.22109     -83.5885    -89.48846   -244.46521   -248.49837    -14.72856   -239.51339   -620.30272   -253.00785   -302.11184   -502.28525   -245.38582    -146.6683   -240.95557   -282.51411   -251.87414    -19.52886   -247.35022   -505.64343   -369.14586   -562.58916   -221.98029   -336.29788   -284.11209   -137.76036   -116.37375   -145.52696   -133.80363    -93.18009   -389.70779   -321.40735   -200.07114   -492.80208   -437.76089    -90.62929   -321.28242   -496.61816   -409.33267   -289.04656    -576.2171   -474.24702   -834.71069   -462.18846   -542.77128    -452.9733   -700.46181   -871.71519   -971.20975   -560.73411    -566.9918   -512.89376   -819.07065   -592.11483   -634.15671   -915.79995   -828.81093   -720.40589   -953.25893   -838.98122   -965.99661   -701.99757   -970.15353   -910.81783  -1007.03327   -712.73843   -950.99657   -804.04227   -805.02575  -1061.02508   -994.44288   -779.18875   -831.10387   -907.88295  -1049.44775   -693.44712  -1128.93257  -1081.53042   -632.64273  -1236.20162   -695.19156  -1259.86008   -988.83833   -921.98449   -915.84291  -1106.83494  -1134.71073  -1200.07333   -913.71153   -912.93609   -1186.7301   -893.60832   -864.58161  -1353.33898  -1029.87843   -526.51582   -988.55277   -610.52072   -627.00463  -1057.69404  -1053.27129  -1177.89794   -652.53712   -890.62075          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       11.50   -389.77225     -218.358   -535.17584   -333.53465   -338.38457   -300.96008    -433.1486   -377.77749   -169.31549    -29.88632   -296.94817   -429.91586   -363.19564   -377.77702   -215.47495   -333.78047   -312.55416     13.80413   -279.25061   -425.69342   -403.63841   -324.06135   -264.00175   -310.73554    171.46227   1020.05442    3299.5504   4207.50174    3420.6709   1513.27394     119.2358   -202.36587   -102.30409   -219.75624   -248.89964   -125.19267   -307.34379    -27.26797   -194.70997    -207.1701   -359.69097   -337.84994   -264.35993   -382.64881    -478.6734   -253.18156   -408.64856   -268.47567   -268.92265   -405.99543   -494.64255   -459.19404   -371.55108   -425.36249   -341.68407   -342.15434   -481.23291    -165.6373    -87.50164   -139.33705   -255.89279   -292.63036   -360.84993   -241.53045   -195.68652    -119.0599   -377.86629   -388.32058   -298.01953     72.36299    597.90274   1984.89824   2535.41095    1875.8356   1282.01239    474.75079    281.99968    -20.51682   -343.06795   -587.90053   -421.36056   -305.45407   -205.55687   -408.66176   -412.34148   -148.60017   -166.33793    -84.84358     46.25275    705.80125   1761.85619   2772.03961   2979.32168    2164.1103    980.19069    472.57608    -60.01247   -158.05333   -296.17866   -128.54237     60.26363    167.34545    207.20676    118.36363    -11.53746   -161.20112   -267.11169   -313.79275   -195.54471   -358.77424   -277.75243   -229.04898   -363.73727   -252.91397   -300.83536   -285.41924   -214.09984    -253.1936   -190.56581   -121.67352   -201.12542   -294.65969    -58.64777      3.71051    451.48684    808.64836    931.93253    852.34845    630.57044    245.50564    -47.13155   -193.28947   -132.23629   -106.44812    -74.85909     52.30625    449.23995   1391.02968   2149.69295   2907.68519   2912.79994   2334.16805   1682.66137    906.37339    475.30266     179.7508    174.93476    201.25854     33.38375   -125.94558    -25.52169    -85.28783   -205.15898   -266.25255   -374.02043   -233.96948   -341.54562   -233.14525   -352.48819   -250.42672   -178.35262   -360.29865   -294.60811   -158.02713    -79.89815      -5.8001    127.01473     13.67651    -27.15639   -192.15494   -280.62089   -145.72186   -159.76819   -194.61461     -69.5388    -57.70195    356.67662    240.88104     139.6458    180.75734      35.1487    -85.93276    -25.35761   -128.48606   -245.98665   -242.65531   -251.82302    -376.5718   -277.53884   -345.91039   -266.10773   -239.12116   -237.95041   -193.91417   -197.34387      3.68781   -142.79706     39.82553    315.21804    407.09393     82.32103    250.02602    351.57701     31.12582    -51.30988     79.04674    -17.38608    100.11712    157.01894    187.82812     225.6166    251.95729     -9.46851   -100.51042   -118.49789    -211.8871   -319.39385   -290.77029   -378.19419   -266.43282   -149.06078   -344.48551   -235.37353    -99.18294   -117.43779   -148.35352    -35.80079     61.16136    143.47871     130.5943     57.27415     29.46499    151.65783     77.95938     96.92765    316.16458    192.04143    316.02045     410.1983     97.65141    272.42437     63.13307    254.43677    138.39398    -11.17754   -223.98244     17.98155    -97.22352   -327.01911   -338.82453   -336.73148   -316.55665   -233.93862    -47.34465     -1.12877    131.90215     -6.04904    123.05722    132.66423    166.35857    380.75703    349.69865    476.73913    488.75836    655.35576    822.94933    833.77478    726.20516    491.38428    196.98225    210.98472     179.4083    176.85877    144.85321     35.79061   -183.24413   -203.85456    -66.35923     23.91766    111.66566     55.29964   -136.16424    225.32383    129.84253     -32.3075     50.33979    -15.11681     56.83539    -30.59391   -152.01779    -189.1247   -218.59521   -134.27794    -311.1417   -327.29758   -361.35072   -225.72606   -273.77156   -143.44933   -274.54396   -345.61789   -223.11424   -300.69807   -297.93956   -274.78272   -210.75342    -89.43694     -80.6583   -116.65413   -200.26989    -12.85825   -185.29393     10.72555    -13.21105    210.58596    220.16386   -143.36607   -154.78101    -87.63215    -16.95647   -102.75401    -125.2248   -156.74603    -45.50586   -162.57539   -268.53437   -109.61722    -79.77123   -192.05948    117.72917   -265.24503   -173.19917   -135.42266    -88.26484   -119.28029     -0.50927   -126.53991    -86.99275    -78.68335     106.6922   -150.54335     -9.59332    -59.82287   -113.56715   -301.46927      -32.163     57.20197     -10.7974    -35.91721   -111.45299   -164.28465     74.49741     21.36368     13.59273    -89.87952     34.95574    172.01544    133.55929     59.72712     275.5457    249.90176    304.91986     224.8467    314.02233    195.33732     -0.25138    -49.05691    -37.79309     12.85519    -20.52996   -198.59434    -95.69901    -38.71216     -17.5848   -182.43918    -122.5559     12.80578    -16.55984    -62.98531   -305.86616     32.41528    -271.5528     -89.9078    -129.2151    -58.54027   -106.51609   -158.87215   -188.77573    -87.83215   -337.07804   -174.36167   -174.44771    -17.51642   -152.84537     -77.6077   -235.07193   -472.42576    -176.2564     24.28039    -42.86558   -202.49528   -187.74633   -295.10461    -74.94942   -229.37062   -194.41523   -318.34531    -86.83077    -86.65058    -140.2343   -163.06281   -244.30345   -218.63754   -280.27876   -244.86435   -162.46828    -457.2776   -171.61457      0.03314   -293.18743   -524.73525    -29.40577   -230.78849   -155.98083   -453.87879   -279.47151   -481.84575   -142.47008   -510.97007   -369.27911   -544.00807   -374.76445   -499.43213   -532.97729   -525.89025   -580.70954   -721.75091   -601.58416   -533.55013   -809.45354   -655.11129   -839.71631   -831.90353   -401.47586   -832.95041    -839.8408   -723.16142  -1011.15955  -1077.98467   -821.10618   -888.31018   -866.45231   -664.18483   -979.90223   -967.29622  -1047.70461   -753.18683   -800.39787   -752.19671   -906.01916   -837.63713  -1092.12784  -1134.55018  -1057.85462  -1073.01128   -921.97108  -1060.15313  -1139.50263   -636.95748   -849.66658   -760.78743  -1127.50388  -1502.04678  -1309.89732  -1147.18842   -918.89048   -951.76373    -778.5591  -1073.21145    -502.0334    -913.5374   -630.28998  -1640.82616   -648.17252   -334.98793   -677.51984   -1033.3279   -948.99896  -1520.93264   -678.82764          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       12.00   -495.34743   -211.57305   -455.86781   -392.26312   -361.39719   -219.50607   -377.86308   -387.93312   -297.20991     77.39386   -385.55914    -366.7775   -360.01438   -404.48669   -161.75745   -153.67383   -268.29346   -228.23005   -378.78138   -307.22851   -347.44389   -320.24446   -185.05375   -250.91539     92.55776   1265.45923   3185.58924   4158.92733   3383.10376   1488.79772     66.61592    -84.25077   -244.72982   -335.07293   -215.59489   -471.19314   -249.50106   -400.10491   -181.99895   -387.70651   -191.39741   -488.28585   -271.67806   -352.04767   -252.55843   -195.41226   -432.47924   -462.59217   -401.58188   -494.41925   -373.40306   -427.84904   -227.80245   -384.82903   -259.98369   -401.86732   -319.77017   -202.86073    -60.44134   -245.45055   -378.95498   -244.57355   -398.99584   -244.91551    -258.1531   -419.64221   -367.50415   -135.85027   -242.62666     65.03999    798.68822   1699.41881   2255.54836   1894.11322   1092.26103     406.4952    125.93604    -81.07051   -274.84829   -519.51173   -570.86714   -521.76662   -248.40333   -254.18366    -265.4117   -289.83354   -296.60924   -117.03407    174.49488    717.44421   1710.59771   2815.94185   3069.39994   2319.00138    1210.4662    277.24864     56.81087   -182.25035   -231.67064    -22.57096     173.2314      71.3371    224.22559     252.8272     -32.6893     -209.901   -244.50425   -280.44214   -263.19442   -327.19636   -275.33526   -353.49073   -239.61974   -207.81081    -271.6495   -202.34282     -89.4581   -196.29001   -207.79341   -315.05402   -243.38886   -372.12868    -76.08763    146.16714    363.37214    792.35813    766.42273    789.40742    641.11482    327.38229    118.78547   -108.92644   -307.13851   -275.32196    -62.99342    -56.09182     361.5019   1195.42821   2033.67758   2729.72898   2647.87047   2310.23882   1586.96105    730.81644    514.94434      314.132     84.62452      37.0556    105.45416   -111.31441   -136.75899   -199.04843    -240.5326    -288.7649   -267.47832   -342.99148   -217.29377   -281.32912   -197.46328   -219.76353   -223.31821   -101.69544   -124.51478   -147.99598    -13.03135    -27.48687    -93.93206    -96.52717   -132.34496   -121.60658   -112.31062   -325.15138   -297.80521    -98.78898    -21.28705    129.87614    128.32881    304.70614    296.25072    112.23993     98.92372    241.06859     -43.7337    -108.5835   -410.53155      9.62021   -171.56714   -116.33462   -238.60088   -180.27835   -174.93907   -376.84166   -295.75306   -223.36689   -248.62748   -119.48463    -84.08901    171.11187    277.00702    202.02096    202.89089    199.90846    123.81328    -15.55946    -44.69117    -68.86378     99.39017     71.73134     83.32276    144.52775     91.15295   -107.20144     69.10729     89.41672    -67.99101   -209.13555   -225.98592   -250.40602   -417.65092   -414.58126   -104.11991   -172.17767   -431.28501   -105.24293   -187.79791   -214.59565   -174.77913     62.82776     32.22033    134.37759    177.15516    162.92022    168.78441     13.48179    153.67801    181.32929    354.39053     335.1499    206.01174     90.03443    308.12355    308.98965    105.01504      6.55718   -143.81944    -36.88091   -145.55093   -302.71108   -186.92153   -184.40843   -230.96294   -184.22967   -261.54141   -128.32159   -130.04057     24.85815    -31.72888    108.43184    197.65696    212.11589      272.438    470.92692    499.31935    561.74936    551.90331    849.41053    705.29028    760.84991    535.26901    669.02441    349.11354     65.54292    208.71079      -71.995     92.57219    -76.28727     77.77813     -60.5637    -63.57097      5.83156    209.14578    -43.60332     97.23966      3.88473      93.2884    166.75806    -15.98505    276.88846    -60.70888   -221.05893   -242.46842   -204.75302   -361.86013   -382.95828   -263.81705   -300.93264   -292.31454   -284.74535   -299.06418   -317.81997    -158.5258   -218.06782   -215.72075   -188.98806   -237.28941   -246.74062     -5.11549   -221.27745   -114.14372   -220.47354    -46.80237   -114.42954    -90.69151    -59.39474     -95.3077    -88.65466     -8.54695    -73.07283     -73.3057    -59.19377   -223.11472   -170.45967   -232.19081   -126.81064     -32.3158    -106.7806   -119.29233   -278.07645   -132.13731    -74.39132    -46.47634    -67.37425    120.83318     -29.7512   -168.92857    -51.76422   -175.59082    -94.87668    -35.29118      56.3535    -15.24046    -116.6954    -19.19897   -121.88126   -170.08637   -152.17688    -46.45795     29.75695   -444.78291    -18.33372   -106.01551    -54.98181    -48.32058     -10.8043   -124.39088    301.16475    236.10605    261.01366    -41.75628     70.07755    277.63214     220.2077    179.35439    355.60878     -4.52113     93.56901    135.69998    255.56241      8.10308   -126.32642     -63.5079     46.80778   -181.48935    -58.18414   -119.12029     56.66844   -187.92026     -9.00637      82.0674     100.0679     -9.16501   -223.84237   -146.53315   -124.46836    -40.44204    104.02313     -68.8677     -59.3377    -228.3535   -192.93474   -252.74976     69.27178    -62.63379   -189.29851   -431.78281   -527.02269   -274.47811    126.21891    -123.4038      71.2126   -355.88412   -202.81215    -93.63408   -104.80087   -231.40528    -88.39812   -177.89731   -305.77145    -364.0574   -244.20093   -303.16718   -409.01929   -303.13958   -434.62935   -407.49281   -253.79732    -341.9193   -519.33757   -291.14386   -570.99007   -781.70225   -165.53058   -106.02432    -616.3811   -286.86282   -648.67119   -637.72309   -639.81485   -505.47029   -352.54149   -397.50959   -500.84468   -633.02865   -241.34701   -647.56312   -541.38239   -723.78058   -896.94768   -561.82459   -726.01968   -812.38447   -969.95876   -904.98772   -934.06814   -750.56689   -913.95915   -964.32301   -778.58602   -707.10739  -1034.48518    -889.6129  -1118.71654   -820.59873  -1059.68067  -1054.92486   -752.96101  -1018.82629  -1212.84713  -1020.52355   -873.65856   -760.07719   -842.30745   -873.04172   -583.18872   -849.71069   -931.53422  -1095.41362  -1119.13202   -984.33389   -438.88966   -489.92562   -965.69924    -325.7923    -815.6153   -761.59857  -1072.40061   -1266.6458   -758.12868  -1050.94968   -962.09275    -838.4366  -1611.91037   -873.07646    -893.2011   -475.38088   -756.99124   -485.28719   -948.44462    -626.0433     25.06557          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       12.50   -561.37323   -276.93366   -373.63316   -420.47399   -329.63258    -78.25845   -460.39028   -395.03869   -411.48692     -27.6686   -376.61869   -236.31989   -359.84808   -250.53154    -47.86129   -232.31384   -197.40924   -362.84193   -193.04357   -198.44086   -430.95216   -244.22148   -248.86838   -388.56424      21.4016   1194.89609   3189.24815   4075.51716   3068.44003   1466.54279    215.73419     27.00882   -193.99751   -230.88278   -201.27293   -318.13252    -268.9138   -237.22646   -273.61736   -404.71847   -308.67766   -527.42194   -293.91593   -271.25742   -297.41142   -471.58159   -410.78782   -308.36581    -437.3914   -383.54507   -273.34461   -349.01011    -84.55134   -211.79562   -315.81475     -418.096   -394.75757     -106.021   -248.58524    -195.9404   -377.23634   -244.55041   -197.07896   -248.51295   -250.65338   -367.14097    -325.1278   -217.29462    -62.11228     76.46444     783.9403   1939.83045   2380.03535   1993.55388   1292.75508     393.7627    450.21135   -311.03472   -207.41814   -464.77137   -497.46107   -356.31801   -325.72799   -541.18454   -260.71338   -187.39422   -103.10009   -219.07671     -2.61607     839.1555   1914.95874   2949.27612    3231.5838   2172.78406   1252.49789    380.31662     -3.40945   -215.35842   -217.84976   -190.13508     -2.98021    159.48085    320.74806     113.5334    -51.67758    -33.06472   -280.42515   -251.02466   -253.40923   -408.11228   -403.85001   -235.92068    -292.9556   -282.50483   -262.63555   -347.79859   -161.74845    -84.10353   -241.26825   -148.98179    -91.85923   -190.89192   -180.57188     48.65274    269.88751    806.66829    971.17257    835.46805    472.62677    297.50443     33.52153   -161.59234   -180.44669   -230.00735   -230.68302    117.12189    518.72807   1285.53515   1960.39548   2819.16666   2817.67235   2412.85939   1881.03276    731.92573    209.73926    309.39994     217.1068     99.75473    105.82765    115.87024   -140.30078   -151.93472   -190.85441   -141.97805   -394.92905   -343.59955   -328.69255   -242.93301    -269.9666   -220.61821   -232.32732   -247.33774   -173.57786    -45.86225   -117.46518     46.06365     52.07641    -24.67281    -63.16258    -150.2265    -89.23786   -171.21708   -142.72515   -208.16547   -107.85501    -94.88383    159.65869    192.93393    237.77559     49.69671     81.79927    -90.59386   -111.32799   -245.58469   -176.11992   -225.96518   -142.55239   -323.65094   -195.40142   -251.52327   -256.75827   -348.29975   -305.68268   -226.50078   -222.34654     -3.42817     91.99563      29.7517    198.26271    151.97239     71.19329    290.55504    161.56646     46.50127    -54.19052   -105.13175     79.65555    156.29575    211.50398    274.01215    142.46914    121.90271    146.34929    -97.37806    -88.93021   -151.85605   -159.72387   -267.57745   -235.33542   -281.64331   -348.60264   -198.81007   -412.28622   -371.04784   -211.92593   -124.92866     -11.6407   -143.02911    226.47082     16.78464    145.68159    159.83212     181.5293    267.65062    178.88742     139.2378    278.82064    172.95547    174.68136    200.39181    240.39966    223.81476     254.5099    -81.14841     74.02504    -86.22393   -227.97746   -165.45115   -339.32365   -278.17917   -157.44273   -302.51551     21.42084   -196.18121   -163.53409     79.10026     77.80531     29.81442     49.38589   -105.47174     202.4152    358.89675    676.73976     570.3874    798.97263    842.13582    890.45312    610.44446    648.94323    587.65745    215.00218     104.3678    144.96864   -143.02722   -108.96787    -67.57706   -147.21529   -191.93539     34.87704     83.68063    -50.50371     34.68513    105.95465     29.93542     57.40914    117.90762    -29.12158    120.15585   -117.93765   -205.35621    -20.10445   -214.96655   -229.69934   -299.64614   -301.03067   -333.38146   -443.09836   -217.10977   -357.54424   -289.62169    -260.5494   -361.03123    -401.9038   -269.20742     92.11203   -171.11862    -98.78317    -66.22799   -280.62234   -107.39345   -152.78767    -42.25195     15.62548     42.76387    137.49143      39.2193     71.35334    106.96812    -33.67881     78.58607    -94.25886     -93.4858   -114.91058    -99.17714   -104.43421    -60.40893   -163.95202    -170.6657    -78.09332    -85.29078   -235.09112   -137.60733     73.13768      4.81199    215.02475   -111.74389    -76.97434     -8.45082    -64.01506   -260.57185   -208.82444   -200.26481    100.83937    -93.97817   -179.96289    -23.18106    -61.16837   -241.65809   -108.22026   -114.13843   -217.57006    -55.79145    235.12415    212.63097     81.52061     29.85459    354.85313    222.58262      72.5162    194.63799    222.03046    276.06529     91.53371     25.69486     68.63128     72.96061    107.56918     86.34984     226.2449    -97.56003      -22.117     29.81635    -92.51768     23.06962     54.87709    -46.05274   -132.18675     54.81364   -233.39957    -44.45837     30.70848     71.36339     -8.19362    -67.93388   -157.25402    -246.9978   -113.18852    -68.86826   -163.52347   -209.13536   -183.43031   -161.48761   -157.11845    -56.76186   -242.69092    -69.58437    -29.61151   -116.29631    -234.9768    -145.8529   -296.37883   -144.07349   -160.51075   -363.04386    -22.21779   -391.24303   -196.01962   -147.36724   -228.14486    -47.53168   -181.33693   -159.11819   -316.18225   -209.82689   -316.17054   -179.54745   -287.52862   -194.37537   -425.14426   -258.48273   -384.01709    -89.89764   -246.31996   -374.59933     -722.198   -239.11495   -190.05964   -497.41126   -480.88071   -521.13455   -440.95196   -700.14132    -656.0857   -294.73293   -465.30117   -356.03279    -875.2955   -682.44278   -345.44506   -807.23559   -753.30818     -613.317  -1128.86041   -884.77882   -642.06558  -1104.00883   -868.65276   -986.37959  -1015.77581    -932.4954   -950.69897   -682.07886   -764.91899   -917.29428   -791.67277  -1194.91092   -1021.4238   -940.11433  -1057.65477  -1047.48775   -720.14197   -754.02216   -956.55767  -1159.95311  -1103.05068   -818.82159  -1108.13699  -1079.03751   -915.47056  -1073.85113   -670.53656    -937.5237    -961.4202   -781.00805   -432.80422    -491.2514   -572.86781  -1032.97349   -581.18677    -959.4279   -880.42747  -1234.86596  -1139.36862   -199.86239  -1046.81393   -998.73103   -530.37141   -310.81035   -713.50005          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       13.00   -592.37277   -368.84435   -322.53392   -355.08895   -288.81873    -179.3763   -558.00522   -450.15885   -475.63531   -133.26378   -234.51989   -171.95418   -407.25765   -204.72454   -161.27414   -367.84366   -149.47821   -389.83556   -201.01533    -241.8971   -429.03994   -122.24657   -457.34278    -397.6847    170.67936   1168.67525   3419.42099   4092.28957   3266.87002   1467.70024    235.84535   -133.67639   -263.96954   -382.26316      -203.71   -261.53779   -313.08672   -216.53343   -316.27951   -325.11675   -272.72569   -418.34195   -277.96034   -123.38249   -403.36945   -415.48971   -394.14593   -405.97277   -420.68759   -168.69839   -368.98662   -184.99551    -246.8965   -275.14827   -428.51349   -314.06795   -299.83411   -156.36943   -231.54823   -155.68488   -145.58985   -372.60782   -157.67624   -311.28859   -342.90877   -323.32023   -285.49072   -252.49193   -142.15131   -166.74621    957.57785    1886.1229   2114.43233   2062.57908    975.51016    405.73204     97.56072   -156.05013   -322.79819    -583.2118   -391.75984   -306.21523   -324.48283   -347.86912   -189.47051   -163.50635   -215.77026    -259.7574     56.01233    720.91811   1998.09997   3107.50064   3227.21589   2165.38134   1393.15527    385.65695   -146.04331   -309.30636   -284.08706   -257.14602    -64.39793    289.94678    272.54123    176.52148     45.62586      0.94862   -195.68258   -200.56835   -255.22375   -312.02477   -421.96069   -416.79031    -265.1264   -324.63303    -340.3191   -296.49413   -261.70415   -240.18707   -208.09992   -248.87997      -270.78   -331.84879   -226.44537     62.56297    453.64171    688.87453    951.39583    917.30511    403.87648    465.12907     15.98802    -36.45729   -106.60293   -113.60704   -205.52375    -32.77533    448.98567   1127.13562   2093.42707   2682.27125   2707.85716   2379.28111   1634.31857     969.6377    376.42661    241.16979    233.06368    170.88303    112.25453     87.49152     -5.13304   -128.86171   -335.30799     -352.501   -368.70867   -320.50982   -294.25487   -295.64212   -260.31475   -168.54474   -225.46162   -241.80752   -230.23708   -132.82162    -257.5894     17.99867    -10.21411     87.81143      66.4272   -129.85806   -108.83103    -176.8759   -198.15035   -157.70798      2.25973    -89.97955    146.33248    239.46097    186.35602      8.31923     42.83035   -194.26159     19.31674    -155.4213    -51.02239   -218.37201   -178.33572   -315.05984   -430.16122     24.79138   -267.48809   -249.67857   -324.24474   -219.99211   -148.62852     21.60305    114.03163    -66.93723     213.0164    109.13412    240.43207     30.59376     223.1515     -4.23044    -172.4799     94.53402     88.49397      94.9969    199.17989    134.74755    -31.91687    203.70024     21.01402     -9.90698    -48.56573   -206.40666   -352.86996   -223.68937   -334.75769   -432.20433   -554.69831   -234.99877   -216.50184   -264.68578   -242.51495   -142.74297   -144.38064     12.04013    140.35292     85.40366     19.54039    103.56268     44.75519    136.87189    208.37456    -95.59239     66.96649    299.51405    352.77145     460.8471    212.56423    251.46319    152.70599    153.11712    -35.37674    -52.75522    -71.98854      36.5258     46.74863    -302.3612    -46.48224   -366.26098    -44.92423    -253.8503   -135.28793    -48.71267    -14.81042     75.07352    208.39604      39.6581    405.19316    397.48535    440.99038    655.66187     627.7206    795.45636    806.24592    610.80614    748.27681    555.99417    409.06181    238.50444    140.88586    -15.53776   -183.71098       2.4802     13.26522   -247.28419    172.48881    -41.46584     46.22773     23.03205     160.4314     98.09258    196.12887     28.81175     43.48913    -166.1399    -36.31122     -96.6209   -107.24671   -254.19876    -228.2234   -263.09047   -419.59594   -367.12261   -437.82703   -286.59487   -222.45811   -326.58434   -189.00288   -335.40436   -390.92146   -236.63859   -149.43053   -232.82017   -115.76871   -230.99096    -82.59291    -77.77246    -23.86652    -26.14806     46.58101    141.40378     -8.58689     37.46538      -68.077    120.59474     63.34943   -129.13132      18.7347    -43.03346    -94.90121    -312.5817   -117.72122    -75.52248   -185.80305    -86.79248   -264.99043   -152.95721    -144.0219    -40.57364   -114.54241    -176.9443    -94.49929     19.73139     67.76944     93.25926    136.88087     81.50071    -18.74633      1.31396   -196.04273    -77.20983   -201.42499   -176.90617   -277.28588   -118.73844   -344.57402    -35.83478    -86.10503    -40.64225    -96.68097    -69.93607     57.00154    143.84267      126.325    192.35882    206.80032    186.73209    427.17065     21.76727    213.60904    127.81665    297.46659    161.37873    -20.05112    157.99278     105.7156    -19.98132    108.18846      2.29965    103.86985   -110.07736       1.9992    -92.04818     20.23851    149.31972    -93.91172   -163.64273      -36.844   -203.04879   -230.32977    -30.71637   -177.22886   -261.11386    -73.46134    -104.0268   -446.91444   -192.30208   -102.56759   -291.19147   -410.32462   -266.97628   -354.32125     23.78105     43.38748   -239.21038    -274.3725   -134.14717   -281.94789    -271.5023   -263.87597   -267.34145   -193.99906   -298.42478   -288.57342   -268.53908    -283.1945    -371.6818   -395.89345   -442.23869   -340.72368   -289.73131   -317.82727     15.11675   -250.84889   -174.29794   -331.21162   -129.99647   -135.10772   -369.62361      0.27326   -112.85095   -379.83418   -103.80916   -442.60454   -494.55462   -249.23308   -448.87171   -444.95169   -648.90876   -481.72503   -738.43256   -629.83327   -676.90844   -757.76178   -702.23134  -1042.21851   -569.24909   -845.92598   -631.79338   -744.96366   -799.72284   -463.34189   -961.68936    -782.0979   -873.94276   -930.09571  -1009.39108  -1004.67431   -840.03087   -903.55444  -1031.35886   -936.47384   -902.68591   -882.36246   -916.83318   -778.07824  -1142.86444   -804.94344  -1061.33612  -1079.15408  -1171.87829   -850.58162   -650.94547  -1098.05213   -503.54482    -661.7204   -827.04327   -830.95338   -820.90912  -1158.10475  -1127.89652   -956.64563  -1157.56605    -894.9583   -900.32879  -1195.97212   -960.03163   -122.98708   -883.33316   -884.70054   -585.82104   -653.15304   -813.39815    -10.10221   -257.22553   -614.99733          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       13.50   -631.09211    -469.4643   -310.46944   -251.84843   -273.85369   -302.72211   -467.69667    -527.2713   -414.17091   -293.34472   -137.90262   -198.18947   -444.96222   -291.19652   -256.66639    -331.8684    -77.94501    -259.3153   -480.97557   -252.20947   -324.34309   -174.08169   -237.31162   -281.44627    236.33944   1224.16525   3438.53596   4197.44242   3524.77267   1199.21935    149.95074   -148.63579   -308.79823    -337.3192   -297.55676   -274.33984     -316.339   -205.86966    -378.2816   -379.29929   -360.06573   -443.01847   -237.16425   -264.85925   -499.76945   -449.87728   -378.81301   -307.55168   -228.57662   -480.79167   -414.00682    -65.42017   -269.35437   -353.39073   -397.99862   -302.62081   -190.14042   -106.92403   -112.41867   -396.64512   -198.96985   -347.59922   -377.41745   -447.07139   -184.02181   -171.43929   -366.49141    -301.5857    -71.07061    -44.12014    666.14269   1795.59535   2356.57864   1964.79983   1052.90024    454.15315     29.01722   -115.10388   -308.31408   -375.35665   -146.62181   -156.81051   -144.58037   -175.89994   -146.54483   -221.84179   -206.50594   -173.06513    -52.05361    846.86876   1813.55405   3097.54818   3117.61505   2168.47846   1148.69501    191.15687    -77.31645   -156.25993   -148.17756    -75.86593      77.2774    296.75859    420.89263    227.77349     39.65336    -48.11229   -128.02483   -286.11899   -300.88789   -315.60571   -333.23799   -389.84153   -343.12528   -358.56446   -129.20678   -268.34388   -313.09725   -267.12804   -122.56013   -426.15859   -210.06379   -403.31515     10.00952     81.22909    399.55751    888.51967   1015.33992    743.30637     565.7672    138.51029     39.85049    -56.21744   -187.30898   -202.20422    -53.19996    -70.17457     603.6434   1256.34878   2149.61734   2709.96862   3009.50801   2469.94817   1617.99936    947.20101    452.78759    246.98892    152.64885    158.42532    173.10769     48.39672     25.52062   -138.58973   -326.66156   -341.50852   -274.45396   -298.87706   -353.73302   -306.13997   -261.98402   -419.52849   -194.17472   -182.22181   -249.42586   -108.30139     -50.0884    -56.98963    175.49712    -46.43303    -84.76526   -290.37333   -173.14693   -282.11907   -254.65712   -169.70789    -15.26987    -87.77295     96.13358    237.45488    245.22036      175.381    -47.56048      53.3367    -94.42221     39.10889   -223.70917   -135.32083   -229.22198   -222.43296   -370.71032   -284.54961   -223.60272   -341.05329   -398.48551    -87.29379   -350.26563   -127.25028   -265.55154      15.8555    174.09846      76.6796    436.91035    120.23886     45.43899   -107.27093     56.85491     40.32897       6.8394    -19.89518     93.53649    280.46154    223.21639    154.58562   -119.57532    -24.95826   -161.65469    -220.4884    -257.1106   -255.93063    -280.5527   -144.86066   -255.85671   -436.33003    -292.6927   -147.01814   -188.69869   -180.74463    -19.32329     114.8281    -70.48708     45.36447    -57.58771    108.53657    220.52704    185.50104    241.55067    167.84559    286.41793    390.70897    335.88383    303.92719    300.59233    261.14255    -105.2398     11.12076    118.76946   -135.92173    -227.6679    -94.00001   -174.89167   -118.25423   -328.83696   -259.42329    -177.8411     -35.8972   -121.74687      5.88091     80.33453    131.04769    119.11848     54.18328    164.69221    265.83348    411.59522    633.98705    638.18457    703.67826    644.93676    619.31695    827.36964    653.22463    402.52446    356.44978     -49.1774     81.38598    -65.08385   -126.59428    -149.4733   -161.22471   -195.44882    -43.87517     -53.4779     32.64934    290.65434    200.86036     51.68196    146.43234      7.92086    -99.24098   -146.86869   -220.64357   -198.83674    -179.5278   -155.48808   -416.98961   -296.73212   -235.72094   -311.44444   -196.86772   -268.56854      -429.47   -335.83592    -303.7844   -401.94775   -115.20756   -101.26115      10.3745   -283.91714   -320.34916   -275.56645   -149.03225   -179.99794     13.07445     -1.61046     95.43294    156.65103     92.54608    -12.10655     -0.05669     41.15848     67.81348      1.03903     -58.3537    -65.96192   -119.47341   -414.96395   -132.55345   -257.13536    -71.84016   -111.01466   -280.83556     66.17936   -155.97486   -139.70015      5.86887    -37.06636    -22.35301     37.57651      -150.12    -27.17436   -218.74192     12.00133   -177.53537   -183.52672      2.15531     54.43712      3.36915   -223.24749   -138.38117   -115.92309   -168.37299   -220.65401   -209.67557   -113.55461    130.97478    -12.49819    226.61673     13.17482    123.40961    156.11821    108.73909     51.88614     318.9582    131.00195    169.07439     43.08964    157.46392    135.49414    -73.80792     96.02105     89.80586    127.62058    101.91489    -55.83579     45.29768    -95.05579     10.42768    -45.86246    109.31854    -80.26613     30.05566    156.70633    186.68198   -156.26905   -251.00964    -78.26807   -117.60829     -93.3323   -290.29465   -189.44382   -412.78554   -121.23819   -106.86256   -396.05636   -330.39636   -262.25221   -111.11928   -484.02352   -195.48804   -265.80604   -160.58022   -184.16343   -186.59293   -265.00334   -183.25171   -301.77577   -216.82002   -159.49883   -233.62573   -117.36495   -230.40013   -227.34199   -197.24122   -242.63627   -488.91527   -227.68867   -419.19256    -89.03746   -205.60774   -426.18596   -272.58849     55.27627   -372.31528    -178.8683   -465.54584   -673.78106   -520.81563    -301.9875   -212.88957   -248.91145   -338.38955   -620.91731   -559.36575   -402.32386   -708.37901   -727.05485   -682.82293   -615.87297   -726.86447    -816.7886   -601.94015   -651.03966   -800.98013   -836.00894  -1162.60992   -476.99802   -931.73468   -928.84445   -747.81826   -863.53252   -966.57403   -849.81004  -1010.62702   -903.43867   -955.52338    -841.5761   -982.44193   -869.81619   -765.32511   -982.53325   -805.38045   -908.86054   -994.09902   -903.49106   -994.27801   -888.47014   -443.47009   -1083.5101   -698.12328  -1180.71807  -1315.59218  -1024.83824   -902.76772  -1034.32588  -1436.94335   -600.31689   -613.34993   -719.11917   -762.56314  -1349.51512  -1427.05794   -749.78675  -1537.30445   -817.41232  -1060.97742    -885.2155  -1512.77321   -1059.5281  -1300.58454   -606.35499          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       14.00   -685.72887   -475.86577   -290.11475   -314.04642   -245.53953   -421.64155   -322.42474   -506.71941   -351.54922   -393.58397    -95.53387   -193.58301   -377.51752   -331.62154   -176.29877   -333.22242   -304.98357    -54.73219   -550.27441    -307.4644    -296.2224   -469.25704   -166.08341    -83.82674    163.60539   1237.58741   3465.84499   4214.41833   3505.87498   1419.17916     99.90518   -153.70949   -204.35939   -238.67832   -277.63855   -203.49392   -256.24305   -347.90081   -287.51526   -353.14815   -261.60884   -463.06477   -406.55694   -274.55682     -412.938   -376.82082   -214.58286   -347.28751   -330.61494   -376.03825   -496.56274   -224.50739   -224.45995   -311.64586   -383.29174   -142.90129   -138.46096   -231.34231    -27.87846   -216.58881   -107.37873   -394.43172   -378.08418   -241.85254   -269.60675   -259.48124   -324.36272   -161.07862   -149.43947     -6.16941    856.78466   1712.21214   2346.55368   1877.54237   1053.23516    349.38201    126.78332   -162.88534    -347.2982   -457.15763   -232.13403   -399.58652   -192.97954   -225.51316   -203.19972   -151.74091   -183.20018   -119.50652     22.48201    567.96953   1891.10767   3065.13848   3177.28102   2191.15159   1236.07788    358.49305     13.19724   -159.57051    -348.2476    -51.71283      9.71963    241.04926    238.22668    190.28342    -68.20512   -119.00698   -299.07166   -338.25523    -303.3217   -191.54614   -381.67048   -290.38395   -288.28768   -306.69815   -301.31842   -133.96607   -240.92215   -194.44453    -110.8798   -127.15434   -232.05601   -200.92149   -150.72374       115.32    465.10508    659.57738     778.4407    761.81441    448.96827    289.59745    104.18833    -11.84841    -72.96711    -82.04063   -186.10441     -9.78155    515.34097   1126.56764   1873.31105   2554.53325   2701.38267   2275.52632     1564.217    978.21727    412.20636    333.02238     96.17288     291.6788    222.82198     101.7515    -27.74054   -320.22406   -122.35916   -287.38503   -370.48452   -227.14182   -346.53835   -288.18054   -396.15943   -223.68611   -261.22702   -435.97427    -210.1742   -210.42033    -71.69247   -104.24243     96.18125    117.19912    -30.84676     74.61734   -101.66498   -178.77278   -319.67819    -86.03658    -169.8212     80.87514     56.82813     99.96711    132.49529    317.11526    139.90215    -28.81997    181.13734    -81.07836   -195.08589   -132.08006    -70.14993   -164.43739   -200.16868   -167.03435   -156.60632   -265.06738   -152.80005   -181.83064   -127.53435     -3.33656     -9.59541     121.6974    137.30727    403.91581    270.07338    209.26498      -0.9812     52.73043   -141.26301      6.85927    -80.28398     51.48868     -2.49628    149.69394     215.6372     122.5157      3.78101    206.84618   -139.09753   -145.69377    -158.5045   -269.30145   -194.83914   -245.20039   -354.42169   -217.66772   -140.62997   -332.51696   -122.95453    -40.03135    -34.71536      16.7267     -5.97635    125.08234    198.96526     68.55937     138.6948    189.14715    105.11169    118.79896     73.32627    437.52356    276.47263    230.15486    151.91783    250.51958    158.10542    -56.49098   -190.75559    -19.69613   -301.04476   -143.37896   -251.77038    -264.8167   -180.94978   -172.04077   -330.79687    -25.60283    -84.45132   -195.85175     17.95117    212.47621    130.17698    197.61452     255.4169    410.41444    485.92639    500.65933    742.75858    687.36841    797.37036    751.76853    660.94349    449.53076    388.19131    222.50143    134.29309     -7.88465     11.70985     -8.28258   -162.32055   -213.70748     -5.47719    -60.56028      0.52351    -27.50657     42.12529     81.29146   -112.90758    164.37053    -19.40321    -10.19684   -221.27939   -110.02847    -23.12355   -231.81321   -278.19249   -147.81483    -134.0264   -255.44686   -373.08397   -287.75356   -165.82114   -237.42287   -117.31678   -400.00676   -277.87825   -254.95638   -198.16145   -197.59596   -257.00397    -86.40735    -186.2163    -16.38421    -65.29987   -205.52352    -28.89071    -50.32222     84.05198    258.68402     75.95163     57.69414    -26.37083    -70.05806   -128.86141   -230.22633    141.10177     -75.5614    -76.88983   -194.80871    -72.47006   -115.84416   -316.52725   -134.65406    -180.0431    -78.35909   -101.57833   -107.24606    -24.57988    -22.86956    124.39438    -74.43464   -172.72972    -94.58029     42.17863      18.0678    -237.4166      -67.624   -111.22331   -194.62212   -157.42616     69.88079   -119.12217    -27.15035    -132.1455    -158.3706   -213.65727    102.42479    178.61401    113.70683    131.17802     57.72251     71.57668    269.98388    -98.58968     510.7812    -75.00774    110.20277    178.76941    156.10132     25.67789    109.24157      71.5835    131.58626    -88.67753   -267.18628    -48.52348      7.80749     57.85286     70.76493    -59.81738   -131.79974    -16.29402   -221.58722    -79.65462   -206.71704   -158.38907   -177.84809     -71.9588   -174.97933    -49.80274   -319.39287      38.7295    119.29392   -131.88606   -163.66591   -228.11704   -451.70429    -95.32534   -295.48164    -215.4152    -272.0074   -409.17897   -333.30429   -223.78942   -226.70773   -149.73089   -157.80258   -246.91539   -213.29582   -241.41236    -87.60677   -198.72155   -119.03925   -407.65637   -367.90996   -153.54722   -167.11633   -126.55733   -303.96324   -241.13169      9.36564   -379.86779   -195.88231   -321.73605   -443.24948   -277.96403   -372.63823   -221.97901   -497.84008   -346.04956      56.6329   -218.06568   -439.45385   -473.97352    -372.7971   -697.54747   -640.79441   -621.07197   -578.26985   -529.96758   -506.61492    -614.1518   -819.14265   -591.04596   -534.56904   -947.02317   -774.74523   -792.32312   -560.90253   -917.27251    -853.0206   -674.27084   -907.94558   -962.34118    -793.7607   -960.46062   -912.92453  -1057.58024  -1006.18086  -1072.55496  -1200.16281  -1179.26893  -1054.97874   -951.32348  -1153.55039   -907.27671   -985.15555   -690.05868   -720.42584   -954.83348  -1195.47647   -673.71189  -1123.00809  -1004.27793   -749.94003   -499.24709  -1046.08086   -571.11151  -1208.73335  -1157.62813   -600.13856   -951.69406   -672.97258   -867.37058  -1222.66427  -1441.75617   -680.32742   -576.08439   -876.99254  -1080.41102     43.85673   -784.50428          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       14.50   -656.43218   -452.05344   -279.36036   -386.64209   -285.90297   -530.08164   -270.47806   -420.83424   -352.22059   -394.29164   -167.72323   -147.22652   -232.56552   -433.97921   -105.45853   -400.15053   -411.90555   -318.24573   -456.24034   -309.63842   -273.87495   -324.66778    -178.9266    -49.66682     70.26093   1364.31197    3538.2952   4449.27729   3175.90194   1391.94268    143.73726   -188.28047   -144.73932   -374.65371   -223.03953   -300.11783   -429.42047   -234.67493   -491.24742   -307.44808   -507.69386   -357.19584   -392.91023   -411.56405   -398.12916   -266.43649   -441.28602   -529.07029   -259.24621   -239.25623   -170.56574   -353.90807   -351.51861   -297.76939   -348.28241   -345.56704   -246.96227    -90.25441    -237.1578   -245.57119   -227.56759   -202.09484   -467.98914   -370.39181   -490.24512   -225.88321   -208.95576   -437.58554   -220.48143    104.77119    848.15793   1832.57294   2332.95897   2101.37979   1046.64521    549.78593     49.10076       4.8151   -241.59562   -178.46268   -149.83752   -340.13153   -263.42448    -197.1821   -314.74628   -314.87164   -343.04533    -80.53431      3.58903     889.8742   1840.82212   2852.43205   3073.56339   2285.22474   1132.15478    321.33188     79.43405   -171.11572   -275.49774   -126.60329    -68.27266    195.84902    307.94757    161.04541    -90.12677    -93.89915   -214.62939    -73.19363    -193.3061   -397.06736   -532.62212   -449.03944    -67.38915   -492.28095   -263.48827    -303.0297   -263.70034   -111.82934   -150.61638   -235.21995   -366.23625   -225.05276    -178.6656    -40.91468    428.66131    747.58034    761.05875    853.85046    552.66996     48.56584    -46.47213    -22.56045   -182.68935   -104.40028   -130.46909    183.59723    432.94306   1187.10902   1811.75667   2780.89941   2697.45041   2138.38531   1665.73234      819.862    460.59896    259.61387     52.87092    203.42368    -57.36874    105.46611      6.70484   -162.75416   -243.50088   -199.32596   -347.69021   -297.21238   -246.96594    -249.3487   -171.99356   -330.63201   -214.62764   -104.82942    -132.2861   -215.19404     -87.6368     12.69962    -26.89886    -42.89114     -46.9868    -40.55317   -419.99966   -190.54273   -197.87869   -165.12679    -94.16754      6.57194    116.22593    227.10918    264.96207     71.76999     52.35725     38.26934   -103.53481   -159.67069     -1.54898   -274.89234    -94.25858   -416.88756   -329.00551   -391.53686   -113.20442   -362.88018   -202.51651   -279.15603   -300.47833   -172.80097      -52.069    232.90835     64.52643     258.1737    113.45466    220.51759    126.52203      5.92369     -5.77816    -77.03402    151.79887      6.72187      9.88206     97.90492     92.63908    207.23943    -220.7603    -23.72881   -183.57035   -142.28307   -343.67393   -489.83697    -322.2633   -396.26473   -314.20273   -387.03453   -167.32013   -319.19274   -213.98563   -162.34224   -167.41964    -36.36616     57.89197    146.14039    213.47441     57.15217   -110.40046    178.95079     34.48588     146.8882    254.82014    279.60773    460.25411    293.79449    251.05187    143.36481     93.57427      54.7397   -183.56836    -50.78547   -118.98791   -291.69722   -254.46683   -283.85771    -90.54442   -195.55597    -139.1825    -60.46845    -27.23265    -71.03944    -37.23207      75.9203    120.94467    155.67631    204.84711    237.33965    437.59768    649.53443    564.03497    563.69637    687.17603    809.30103    445.18057    306.77375    340.20932     29.90846    134.22071    -21.29636     34.75675    -13.09504     25.11427   -162.05227   -144.98961     33.29183      36.6239    100.53916    111.62795    281.43251    -45.55656    200.25455     42.21334   -217.91667   -107.02972   -179.35471   -211.80876   -177.76645   -212.61762   -222.43221   -406.53119   -453.37601   -297.62107   -112.34549   -286.13718    -243.5615   -281.95817   -130.34173   -268.12007    -142.6518   -394.25289   -172.41731   -228.59897   -251.08034   -230.51303     53.09041     -60.1758   -157.31255    -64.78213      14.5056     34.79425     -82.6735     -44.6455    -32.54917    -92.81035    -34.79838   -156.54027   -197.74641   -153.76814    -55.53843   -209.00358   -148.58422   -179.32484     -74.2702   -176.28248    -61.39243    -42.07409    -74.34343    -80.97201   -172.08291   -123.98329    -49.80844    -67.18707     75.21612    -72.72309     85.23065   -123.20519   -107.77301    -75.00539    -18.62252   -137.74111   -116.45286    -71.82786   -405.66727   -152.61882      -195.78   -106.75172      6.10673     -2.68623    -48.01943      232.799      0.88684     67.29548    171.72844     32.60862    280.36462     46.20391    167.87825    227.58797     84.76426      231.506     15.38624    155.09469    119.11655    180.13041   -129.34351    -20.81558   -153.49575     85.01314     53.62461   -210.74938   -193.15689   -109.03691     62.06713   -164.51575    -20.40552    -62.49156      2.40986    -83.92371    -41.92755   -167.63623    -31.69285   -122.90954   -218.37325   -180.43864   -365.32686   -323.68244   -282.08339   -297.03201   -485.98708   -291.14583   -115.51449   -120.66482   -329.23317   -603.80659   -140.97402   -248.76521   -359.62914   -174.93434   -208.83282   -200.11367   -326.83951   -266.42314   -266.13631   -158.77025   -263.83217   -461.02167    -66.52901   -417.43765   -302.79757   -518.71367    -310.2745   -116.42775   -327.22309    -212.3677   -326.83129   -249.63625   -246.31995   -444.21962    -11.15686   -320.50765   -538.34155    -540.6333   -520.63129   -648.01336   -558.21274   -546.37677     -824.465   -574.49873   -414.98563   -781.49455     -679.889   -604.89997   -641.63211   -328.80884    -889.9926   -687.47951   -419.59189   -810.27495   -664.43783   -894.65391   -1100.7532   -709.29206   -947.79988   -855.78903   -665.10859   -986.14334   -713.18715  -1319.45002   -999.44871  -1137.75695   -988.09776    -719.2501  -1101.27645  -1270.28976   -864.29804   -810.25951   -721.44768  -1207.27078  -1226.97519  -1111.64683  -1023.39019    -911.9872   -1323.0422  -1213.09534  -1027.83334  -1084.71113   -853.03389   -1009.7762  -1016.80143  -1255.44015  -1353.00677   -804.49638   -951.33037    -954.8703   -925.55888  -1155.75343  -1002.53026  -1016.43486     25.69465   -878.84709   -682.13246   -931.30988    314.82118    -458.1108          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       15.00   -524.15408   -383.29575   -268.11541   -429.65959   -408.36674   -572.07665   -338.24023   -377.02281   -280.74311   -312.14383   -253.25299   -161.56531    -95.04256   -524.79751    -261.5735   -441.02474   -210.58711   -355.93161   -271.36806   -321.61777   -314.52118   -235.75636    -55.31168    -92.40958     39.10662   1325.04349   3311.13915   4157.23275   2911.71168   1480.07387    247.18756   -149.26244   -324.50486   -202.62332   -436.19609   -339.34449    -184.0178   -261.68635   -329.48716   -252.16417   -384.49657   -267.97331   -404.82613    -429.3083    -95.51595    -224.3852    -516.5481   -343.19917   -200.52477    -341.1218   -312.11826   -386.47889   -280.61227   -366.27915    -330.5935   -354.63897   -242.28289   -185.50083      7.09011    -84.19032   -387.46117   -403.72829   -329.87626   -428.48593   -235.62896   -185.97067   -363.77392   -341.91494   -253.71832      -2.8612    897.01933   1868.67232   2416.53705    1926.9176   1081.06318    447.76667    172.39629     73.76828   -621.33119   -186.30457   -619.20583   -520.27477   -289.15098   -223.14362   -159.15324    -89.29589   -258.07516    -252.2468    264.33743    752.19494   1882.67499   2838.81913   3050.81647   2193.47288   1165.78776      371.083   -110.15949     -0.37179    -166.9918   -211.19038   -158.31463    217.65722    468.82384     87.96219     -52.2272   -239.92829   -185.81814   -255.43951   -334.23163   -382.75562   -483.88645   -440.40663   -295.42914   -320.28654    -398.1374    -161.6824     -249.444   -277.71383   -173.94538   -111.24371   -227.04081    -82.98393   -188.33048    360.13204    407.50179    743.41467    778.76485    944.15867    447.83002    271.39854   -211.69367   -225.51186    -69.12976    -82.02683   -138.26394     98.62214    508.04341   1227.47488   2007.15919   2760.91692   2665.17303   2202.31851   1569.82772    808.87368    425.61329     139.9273     49.99805    230.45278    261.17797     22.01061     97.63991   -238.88546   -356.72835   -440.83455   -265.62052   -305.25133   -270.45324   -357.81234   -226.62942   -124.22232    -77.29925   -215.30913   -292.65623    -93.47058   -140.09142    116.86785     81.80073     39.04281    -15.31699   -148.28416   -175.18743   -237.17094   -227.65627   -272.35946    -71.87343   -134.05419    346.72876    127.79885    305.09817    166.62148     97.73386     -79.7363   -160.15937    -39.25222   -195.08792    -141.0279   -261.04543   -237.40464   -293.08337    -338.1565   -328.22039   -216.24956   -260.69275   -224.79009     -45.0575    -70.04147    -55.69089       74.125    165.55494    146.72521    312.41531    256.31793    171.31651     10.26027    -15.82404      26.8766    -16.63666     93.56516     36.12508     42.84686      82.8435     -56.5541   -119.19767    -38.77773   -212.59921    -444.0348   -252.41257   -285.59468   -542.69707   -338.64847   -378.70872   -301.13874   -280.33771   -310.82653   -219.40224   -121.30173     16.45992    133.93147    111.80006     91.27656     23.51207    -44.91067    319.80595     40.88825    141.33687    340.69881    361.88438    218.99977    367.81498    262.45233    272.10591     366.8902     277.6705      7.62748    -72.21925   -294.27942   -122.46602   -229.22789    -19.80636   -278.72963   -178.16869      3.95639    -76.82551   -149.06902   -218.83478    -21.35696     10.75702    175.47448    119.06807    196.57859     38.73329    501.99538     580.0585    523.39651    822.27843    683.69722    575.49799    820.37181    558.70617     342.6859    335.87932     316.6885    291.07905    -38.25119   -116.93272     -5.40713    -17.37626    -54.68387   -177.61555    -12.57669    -17.07548     51.44823    137.73712    164.84456    109.04051    137.44729    114.81524    -22.39813   -167.42635   -159.72086   -226.45397     45.96466   -225.11287   -305.39298   -224.36183   -229.92997   -264.32124   -357.09749   -311.60016   -277.53943   -217.80641   -158.40683   -176.26548   -168.74943   -238.15631    -44.26401   -162.89225     -94.6293   -134.16163   -219.10146    -24.56801   -205.76095   -101.12131   -106.83541     16.53759      3.01455     -201.294    -39.01194    -59.26912    -25.88333    -18.48509    -99.27179   -134.38105   -185.44018   -152.18673    -209.8261     60.34149   -237.57827   -182.36068   -180.87749   -119.85773     -0.78081    -22.77302     -50.7234   -171.47329      5.16733   -188.51054    -59.12539     -27.8759    -69.51181     12.39597   -285.87984   -254.04261   -155.83664   -118.63317   -188.91861   -177.65196   -239.86253    -130.7452     32.36456     55.27551    -29.20675   -148.61822    -66.97457     94.02389     53.42932    148.87071    157.11605    144.87074     97.05323     133.7547    198.29346    154.07961    236.75367    153.60377     13.05459     109.0941      1.68583     17.94396     48.46534     56.45317    -70.61481    152.27641    184.45895   -203.26978     -6.45466    -35.66034   -130.12686     22.23007   -161.95277    -45.11039   -334.83938    -12.97763   -304.76267   -134.15487   -304.66425     -77.8496   -138.40094   -233.70052   -232.65301   -175.82976   -228.98428   -455.13517   -254.28905   -446.09364   -360.78746   -467.65826   -431.22355   -326.93586   -202.93958    -54.61779   -155.35854   -197.19477   -398.12196   -166.65131   -253.11772   -125.51457   -126.89578    -31.30637   -271.43939   -396.91438   -609.99622   -474.16374   -467.00432    -42.65292   -342.13713    -181.6439   -306.10654    -192.5015    -208.8681   -178.92918   -418.70769   -308.41563   -620.53292   -631.42992   -575.68449   -566.69387    -65.66261    -621.4586   -565.76362   -540.78257   -768.39792   -430.68728   -453.99888   -571.75729   -643.17437   -792.57084   -543.00145   -961.78158   -544.53305  -1019.54989   -642.06368  -1176.02181   -725.98464   -706.73341   -973.06062   -999.05877  -1185.08358  -1000.62355   -957.96407  -1354.83733   -918.22692   -825.95769   -571.73943   -953.52953  -1121.24425   -901.17382   -998.95659   -872.69401   -873.38572  -1132.48937  -1056.23068  -1072.45228   -826.17643  -1131.64051   -963.39868  -1042.10698  -1717.92398   -824.31709   -690.08062   -829.64071   -567.70298  -1245.73935   -843.82639   -916.58987   -985.22207  -1057.69554  -1078.76484   -891.04802  -1070.05156   -743.85797   -776.81791   -864.56245   -1578.9725    -348.5211   -205.21705   -588.66978   -365.87049   -680.31335          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       15.50   -379.78204   -363.04977   -249.53056    -455.1023   -537.21515   -505.29294    -391.5731   -447.51255   -184.04545   -132.15406   -293.36463    -296.4266   -144.71708   -606.26464   -299.71996   -416.97297   -291.03684   -349.24822   -186.06393   -294.04998   -250.28765   -134.71088   -284.76765   -205.78598    144.57922   1236.92119   3327.98115   4065.35538   3016.62595   1436.55344    237.15473   -229.09952    -61.70944   -274.26596   -174.96315   -323.93693   -393.35851   -230.73592   -192.37924   -184.95172   -218.77955    -439.6357    -404.0436   -352.90258   -339.06217   -260.17112   -421.67521   -318.22091   -371.65615   -376.39072    -384.0806   -438.28482   -425.77904   -308.32001   -320.27064    -216.1499    -268.8184   -109.16806   -142.71469   -227.52275   -263.14657   -336.01169    -406.4903   -227.76309   -326.03108   -339.36099   -233.22132   -418.10214   -141.79445     45.70565    662.06287   1702.21122   2317.02357   1991.81362    1146.1396     469.3839    285.29596    -80.51859   -273.74609   -475.67908   -395.97985   -364.64032   -231.87806   -203.70191   -372.84196   -271.17803   -291.17021     -86.5569    196.84956    685.27165   1971.49838   2886.94151   3031.52988    2289.6207   1105.89851    280.69442     70.02324    -99.25533    -13.31079   -126.58313   -103.75598    189.36957    251.72946    190.87168    161.76674   -120.50285   -321.28224   -290.20522   -340.87153   -224.11969   -277.47142   -406.00105   -337.51459   -388.38952   -376.77021    -326.0507     -92.5885   -169.25604   -230.15843   -176.58514   -281.92847   -226.19432   -224.59838     38.84761    455.43105    848.08598    927.97802    701.45995    531.69305    252.58669     19.69884    -18.29263   -143.73889   -126.84011   -127.29026     96.63691    493.27984   1184.63341   2024.86011   2916.07296   2835.34468   2455.68342   1577.79194    738.29457     371.9194    111.14557     130.6971    123.61612     78.11277   -112.77354     10.15101   -134.95857   -165.36161   -196.35014   -385.87702   -196.73752   -279.39973    -209.2482   -271.82171   -285.03639   -134.42301    -224.5305   -267.24524   -177.19242   -127.68729     -74.8408     80.74336    -130.1409    -79.95124   -190.58229   -115.46938   -243.32374   -447.63798   -314.66859     -2.98138     -53.3645    140.00186    214.05454     -31.4569    181.59162     22.49031    -31.46132    -86.66645   -197.18145   -167.48772   -241.81997   -251.11141   -275.00691   -213.69454   -323.48746   -331.88908   -231.85422   -373.03337   -204.11292   -210.09233   -148.50123      5.91353     32.17896    104.63382    330.15785    120.28842    224.80084     63.62759    102.68171     -2.16724     70.74497    131.84426    -51.43682      158.751    294.90777      37.9285    -12.99157     -3.37227    -233.0935    -225.6985   -198.18012   -265.37122   -256.47163   -340.83885   -357.18471    -315.9468   -264.97733   -321.95448   -265.25431   -184.49719     -41.3383    -85.06657     -1.59303   -110.20673    142.32525    162.97315     90.07274     73.87933     78.23879     52.74241    252.13802    179.32699     71.36781    252.27762    190.30395    241.62234    210.45626     70.57635    -12.85485    -83.46502   -147.84163   -179.23452   -363.13688    -238.0252   -306.03182   -135.17126   -195.93503    -167.7403   -156.38637   -140.07376    209.64756     64.32369     57.10445      94.3337    305.55519    315.13106    368.42029    379.04627    504.78942    635.56295    628.99075    823.06244    618.53092    536.15085    491.19591    287.33754    271.94717    145.13933     51.92669     21.59763   -246.58592      14.7204   -171.50064     146.3554     86.51558    -43.71105    256.15822     24.58791     90.80663   -116.17478    116.25483     65.29673    -34.40336   -290.95356   -196.91449   -156.40137   -225.10804   -305.73671   -313.60211   -363.70787    -244.5675   -256.49395   -374.43907   -298.78004   -148.31254   -160.64136   -360.59181   -112.94362   -214.22641   -340.87059   -263.25418   -213.08667   -226.39716   -119.49231   -166.99509    -20.27337    -26.95294     -4.91157    -57.93603      3.40966      9.83444     98.72214    -34.18576   -153.14935     23.14051    -83.97173    -38.61982    -82.20064     -64.8427   -220.89776    -84.26252   -229.93604   -131.27391    -83.63523   -189.32204   -209.82236    -125.0909    -67.62114   -162.05973     -81.0477    -151.9856    -92.77626     -3.15639   -100.59772      2.89659     -111.474    -81.22996   -180.14167     14.58015   -183.50051    -188.3167    -272.1287    -217.1412    -85.15584     49.55259      2.20952      -8.2327   -113.69884     67.13456    -91.28952    -33.39852     -6.92167    114.99992     68.38471     75.78385     31.49876     157.1575     46.70398    290.42679    183.21117      8.77217    -40.05046     95.83284    -56.09314    -97.97274   -120.15537    108.06755   -156.54107     62.57573   -104.00602    -44.34582     -9.37486     15.64571    -99.14406    -12.34513   -175.23592   -123.83828   -249.68241   -141.59399   -331.41843    128.91927   -260.89442     -116.373    -252.9072   -105.36756   -231.63872   -219.79728   -326.96041   -313.25301   -255.02343   -226.13002    -96.47538   -288.43052   -358.68477   -394.31122   -384.84592   -410.61152   -254.27931   -180.64069   -333.89685   -431.58394   -229.91337   -440.32568   -273.41992   -242.27501   -422.88352   -307.00075   -145.44108   -297.48556   -354.63526   -581.88809    -97.37838   -276.88884   -142.69693   -191.85284   -389.25959   -632.48339    -268.7943   -670.26132   -609.99919   -293.55424   -458.48004   -920.09359   -523.02408   -493.87878   -664.65374   -650.99952   -523.00581   -520.67486    -670.5356   -788.50653   -556.02609   -676.83996   -643.16762   -544.83435   -631.66946   -622.71205   -570.50132   -1043.5191   -870.10315   -677.51767  -1033.01123  -1035.12655   -736.83495   -843.21889  -1274.16757   -766.41927  -1088.89091   -531.31077   -980.30873    -938.0344  -1050.20161   -851.25779  -1182.99559    -879.1522  -1208.75588  -1118.11462  -1147.90413   -744.26079   -1032.8207  -1062.09676  -1116.12659   -719.77226    -828.6497   -862.73307   -961.28278  -1011.86139     -1308.43  -1057.31996  -1174.13152  -1344.55615   -607.70589   -1120.3016   -599.06023   -921.04595  -1490.96085   -896.34352   -789.99736   -446.53848   -785.12678   -203.94492    -867.8074   -813.50457   -723.12202          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       16.00   -377.35964   -398.78181   -199.05001   -486.15155   -540.28664    -415.4335   -250.04826   -566.04666   -262.44695       7.5075   -440.50015   -306.47323   -298.48632   -565.50281    -141.2671   -512.63872   -484.76588   -276.85413   -120.73997   -283.63815   -201.09269   -193.60473   -287.50106   -358.08923    105.47358   1427.55688   3257.77547   4485.19316    3124.4752   1443.75451    258.29093   -223.61491   -258.29312    -272.9239   -390.16797    -422.1829   -435.61376   -350.84462   -297.36745   -224.64696   -377.49525   -509.96122   -313.88674   -439.81172   -421.25169   -432.90876   -200.27087   -404.26949    -398.3848   -419.01949   -340.77054   -366.49783   -314.91891   -256.42658   -309.18127   -266.35747   -272.20016   -169.54329   -130.86586   -266.08122   -345.44903   -307.14264   -172.21566   -254.96891    -431.9887   -257.18861   -139.02507   -463.75074   -138.14844    -86.16662    775.58492   1813.66865   2334.13028   2125.10182   1048.23045    324.86367     35.32762    -322.6483  -1032.59736  -1165.51092   -831.98258   -434.15677   -236.64555    -349.7517   -158.93355   -100.18448     -313.241   -197.36139    120.16419    818.15839   1824.01549   2863.34608    3094.6602     2094.867   1016.22793    288.08512    -45.26132   -116.39623    200.94217    -20.11435    -86.30804     96.16827    231.15116    140.73537     29.07339   -176.89524   -145.61208   -209.78363   -238.72831   -378.53701   -334.01243   -287.81045   -256.54037   -273.71409    -354.8237   -210.64517   -191.18319   -268.16833    -98.43693    -46.02408    -10.27561   -146.10918   -115.06735    -86.73365    431.72666    862.37273     802.1447     857.8639    562.77682    327.01056     -3.34671    -76.69745    -235.9828   -211.71534   -259.93261     81.68599    294.31314   1259.37811   2033.07696   2567.66061    2640.3296   2142.29912   1323.76338     795.3779     644.0877    243.28809     30.51303     83.94552     33.70712     17.97966    -83.35012   -228.30823   -164.84007   -282.65987    -319.6612   -161.09341   -280.64329   -235.78436   -355.81434   -286.27387   -170.56245   -163.55407   -189.24492   -118.63275     16.31397     32.75409    -99.37237     30.68183    -70.82608   -263.01085   -331.56863   -277.08596   -277.42497   -230.78953    -46.56635      75.8856    163.58097    282.27495    137.39161     39.40318    167.45008   -192.83203   -189.30229   -194.88435   -319.92601    -286.6801   -268.54445   -286.93976   -311.18897   -368.82574   -275.68442   -274.53091     34.96651   -280.56705   -282.45057    -75.98164     -35.8711     14.77595     96.80818    321.12831    207.25258    153.11561    -31.76394     95.89707   -119.27574    140.46157    112.00263    112.07889     63.16194     -56.6505     66.52014    166.40497     83.42005    -88.29729   -252.98636   -264.02995   -211.33062   -295.98843   -272.36144   -318.00212   -411.58114   -226.37348    -143.8247    -381.6858    -98.41674      70.9351   -155.32143   -169.22007      64.6783    175.66151     86.91888     31.90801     76.49683    219.89292     252.6794    195.41829    343.47458    268.27126    241.98653    218.09511    400.22848    279.64437    325.50745    -16.75034    -202.7261   -107.32562   -216.13241   -248.98709   -157.71926   -261.56112   -158.71656   -109.74287   -230.13651   -129.42754    -87.00154      48.4864    135.06659     -1.91539    101.39624    124.53029    289.27985    279.63971    610.77082    503.66002    726.28619    670.43717    714.85429    654.78237    476.63255    441.09321    311.23127     118.5469     23.66185     22.53703    -10.37769   -164.80652    126.16332    -27.16093     90.80238    177.93337     39.91874    131.56188     52.36462     29.90662     53.07295     57.45493     32.27038    106.87108    -82.89313   -222.20142     -242.751   -213.45952   -164.98761   -139.19597   -205.89389   -265.81021   -212.37617   -287.67437    -347.5237   -354.36366   -362.59253   -247.91422   -331.31513    -201.7119   -205.66455   -183.88849   -274.86179   -228.03281   -129.29846   -165.47773      -44.681    -80.20492    -68.34947    -96.79159    -19.40933    -44.50831    -38.14442     78.42958    -64.67927    -29.56118   -237.40862    -14.36256    -88.81651   -156.54761   -225.09426   -184.93363   -114.10797    -275.2387   -175.88086   -249.75051   -129.05891   -122.36796   -262.87098   -132.82435     -9.33558   -160.71859   -139.01649      -1.5037     15.35905    -54.77146   -121.64873   -167.05091    -178.0194   -107.06062   -227.90115    -54.26865     93.55311   -151.65623   -185.13469    -108.5194   -288.07285   -169.97342   -113.95123   -110.42703     70.83592     49.82865     63.03182    162.89858    271.05627    234.69995    149.28058     65.06443    147.30138     69.37983    153.12466     88.80422    107.23029    101.14474     51.36157   -149.38774    -89.53884   -141.65687    -44.14993   -313.70049    162.03813   -181.61255     68.66923    -97.94035    -74.24739    -39.54011    -45.68769   -170.81268   -169.45418   -114.44568    -23.82197    -89.21921   -227.01897   -302.30464   -134.43337   -263.07831   -161.83025   -264.13525   -415.66931   -362.43488   -161.57867   -378.17943   -304.05946   -300.67538   -430.72376   -320.22299   -335.69509   -365.59423   -154.99613   -300.65488    -148.6179   -341.70004   -306.11856   -497.98555   -381.43523    -75.07359   -186.54364    -214.8288   -356.10965   -362.42407     -62.2908   -710.94958   -363.45815    -580.8667   -263.23006    -270.3764   -354.20516   -311.29509   -195.82214      -360.33   -609.60227   -372.52863   -189.49395   -408.68642   -407.78116   -571.45771   -716.66673   -549.73699   -707.50183   -740.82539   -742.83996   -801.67139   -767.84441   -822.56657    -978.8852   -655.39517   -887.53566  -1146.14037     -606.093  -1357.67285   -866.95255  -1002.84305  -1031.30852  -1251.90288     -959.958   -927.38507   -538.83559  -1115.04363   -813.23109  -1036.24528   -782.70662    -944.4136  -1040.61452  -1016.54589  -1086.71508   -893.02419   -970.71011   -845.20069   -923.31639   -996.64527  -1415.49628  -1145.78555  -1478.27803   -769.61566  -1041.58635   -1219.1016  -1048.33892   -985.31659  -1107.87615   -849.58759  -1089.66203  -1759.84916  -1365.29757   -551.52756   -680.09215  -1318.25044  -1502.91969  -1049.80254   -911.91944   -1424.6514   -725.43848   -884.01416    -205.1413    -26.36646   -1134.3425          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       16.50   -470.74107   -438.14693    -185.2314   -450.01487   -551.45212   -428.26619    -52.89097   -569.74427   -386.25308    -36.12306   -444.04041   -225.94827   -289.56967   -491.27152    -243.3553   -594.42968   -303.20261   -201.37191   -203.52564   -149.24007   -421.47567   -339.98158    -38.23221   -103.53205    126.51733   1535.47569   3149.07118   4050.55432   3159.30441   1393.07496    146.38323   -141.44274   -164.28161   -281.76115    -285.6331   -404.72712   -532.55893   -308.33225   -284.57235    -297.6123   -329.69338   -403.72378   -434.48943    -412.3458   -490.13627   -240.63127   -356.09051   -535.27577   -347.46877   -265.80943   -231.30376   -394.33961   -360.83478   -367.66555   -327.99856   -258.72169   -244.46437   -313.45487   -182.14558   -258.05126   -375.46059   -364.79713   -273.39056   -320.36169    -298.6874   -278.73509   -206.97035   -270.12002    -49.20519     95.94707    782.99558   1771.19795    2206.1513   2035.88713   1109.89973     177.5809   -514.69626  -1396.73364  -2564.75821  -1978.52935  -1248.27503   -561.52709   -332.80247   -245.29268   -291.75556   -302.48192   -283.24381    -12.29418    121.01978    740.61339   1850.11734   2816.15221   3068.79186    2020.7486    935.11949    231.55492       5.6352   -226.70248   -137.94374     -4.54899     69.35983    165.95747    385.27905    197.32202     85.52822   -299.54334   -312.31191   -302.34592   -352.98604   -295.34934   -333.55139   -353.06771    -318.1524   -282.09462   -426.29175    -186.2003   -155.43034   -260.00617   -146.58434   -144.60428   -239.31854   -295.60121   -174.27938    217.00497    538.76383    782.21633    808.72148    853.26724     542.9523    265.26137     18.39501   -136.51441   -304.59149    -48.09127    -92.94554     56.72543    590.24708   1206.03254   2210.38846   2723.80638   2812.48868   2367.55271   1571.94331     961.5503    342.70606    284.56669     -5.68603    203.00384      2.36927    -80.00366   -240.66734    -227.2856   -291.52966   -204.98222   -146.33339   -367.87783   -460.00672   -252.53437    -218.5503   -116.91154   -346.38604   -286.65598    -52.92418    -86.81339     -21.6438      97.4596     15.79681    -142.7441   -132.46596   -270.04877   -402.77296   -548.48584    -458.6292   -305.02203   -236.03752     49.95149     36.56372    273.99726    243.30587    268.18923    -17.05226     54.74391    -19.86178   -117.45662   -168.44056   -382.05423   -184.44845   -393.76097   -314.22368   -369.85029   -328.52157   -311.34174   -227.94055   -162.95054   -326.10203   -101.99241    -47.38403     142.4213    182.95472    285.17249    168.59011    182.55997    167.75882     20.14744     30.09485    -27.18588     49.51503    143.50948    145.65747    298.02295    117.02173    180.51722     15.71957    -111.9598   -168.59302   -252.41464    -357.4167   -334.77442   -248.21907    -444.9298   -311.01625   -276.28316   -309.27481   -120.28021    -90.91533   -187.35654   -139.10471     79.09058     13.97472     41.83034      92.5648    202.44055    165.60601    110.90226     94.60507    138.02126    195.85423    321.74176     287.3788    468.83367    277.90144    110.92977    265.39881       4.3263   -234.56478     -8.47617   -384.28106    -121.2208   -268.35082   -205.74645   -249.37835   -228.72628    -83.43331     -47.8944   -139.40349      5.33421     96.46966     -48.8042    112.92235    167.41367    178.11831    413.84018    449.07808    600.96653    501.74855    669.73379    616.59289     705.9086    506.51319    533.15359     381.5275    176.20139    -154.1405   -110.55546    -30.72544    -236.6084    -216.1701   -118.28976   -269.99104    122.21654    -94.88602     56.68798   -175.81648     87.60853    281.03698     47.49246   -270.42862     65.25503    -53.68741   -237.20256    -95.11979    -149.0296   -245.54767   -240.73167   -294.94201   -324.68703   -204.40554   -311.90754   -367.74171   -151.86799   -153.36992    -372.2782   -159.33022   -183.82397   -213.55283   -181.81654    -42.31996    -401.8352   -190.29524    -39.23306   -142.04982    -49.29034     -9.25977    -50.65021   -119.76473   -111.29629   -136.52301    -13.97767     -36.8458     15.85766   -210.43007    -46.44849   -374.96702   -339.25445   -134.12319   -186.94023   -173.30702   -194.23355    -35.91544   -142.36723      0.89288   -123.40386    -78.99044    -44.80104    -40.79674    -21.57454    -68.46644    -11.02582     21.89313    -68.51679   -110.62518     55.35327   -100.43025    -87.71693   -167.26143   -336.78799   -188.60123   -267.98166   -155.85949    -157.4967   -198.46503     12.56682   -162.11075      2.23679     64.98846      30.0389    127.92327    178.52988     62.24687    148.37352    -45.47499     63.93576    195.42641    148.29454    159.33364   -137.21503     86.40702   -142.75917    -188.1701   -128.31962     78.46813   -115.54497   -179.98716    -94.47537   -156.40589   -132.57626   -129.61149   -158.24848   -104.95408   -103.48494    -53.51071    -67.65349   -138.37408   -215.75731   -284.51505   -159.77948   -103.93432    -341.7128   -300.09889    -168.1134   -362.64082   -195.23738   -262.34022   -374.89092   -345.31502   -264.56729   -145.31179   -270.98993   -331.68078   -346.97907   -192.19409   -341.95097   -293.37632   -273.82503   -421.30919   -300.07069   -104.49841   -201.80473    -278.6447   -397.69328    -169.2668   -153.52492   -192.69998   -460.78542   -351.15633   -611.07775   -367.76294   -203.39805   -289.27297    -450.6865   -326.03775   -397.51918    -368.1789   -401.85438   -776.34733   -340.06809   -438.34626   -591.71237   -438.84686   -674.72895    -639.6974   -558.63262    -741.8802   -437.62641    -694.1011   -727.94447   -730.62725  -1096.34731   -865.09461   -810.35742   -742.08913   -982.35253   -624.54291   -882.43677  -1059.97115   -909.31907  -1068.23189  -1197.19535   -927.59483  -1077.23528   -892.19783  -1111.72157  -1006.49973  -1190.07813   -956.43288   -998.98203  -1338.54488  -1172.28713   -958.67088  -1220.71304  -1173.93378  -1060.18381   -979.91029   -748.60738  -1534.80752   -807.52241   -962.37488    -1435.898   -997.45152  -1196.12598    -789.0319   -756.72997   -976.86367  -1240.13336  -2122.00101  -1381.46567   -940.26589   -729.71036  -1368.02067  -1008.63599  -1658.25089  -1117.37366  -1200.44428  -1072.02824   -846.35278  -1755.37727  -1118.60673  -1152.46988          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       17.00   -592.80293   -386.67753   -216.64665   -390.24568   -523.40858     -530.618   -138.90235   -523.72926   -345.56876    -114.4823    -381.1659   -272.68888   -119.07424    -237.5035   -406.53152   -607.91742   -144.35915    -34.80057   -391.14672    -80.92631   -188.20794   -395.35791   -173.12607      -18.956    178.78669   1544.99017   3224.82095   3992.55942   2959.49423   1334.87759     14.53382   -272.65242   -167.73473   -160.68612   -341.10893   -144.95239   -288.31087   -221.27001   -343.63269   -282.12441   -275.36122   -300.86277   -396.32147   -308.27603   -422.43565   -514.21069   -333.35655   -303.77886   -478.38495   -317.74017     -442.232   -330.47353   -420.75747   -409.39958   -383.95912   -515.00962    -364.7464   -400.15042   -145.87728    -239.7318   -163.56195   -403.98657   -348.08158   -366.19401   -289.75551   -406.89257   -142.65835   -102.27439   -423.61251    213.47378     869.8936   1740.65053   2230.36884   1837.72769    960.28104     14.25587    -818.4403  -1849.18392   -2946.1359  -2698.55308  -1474.59574   -550.94765   -536.43495   -330.61362   -100.42693   -249.05523    -45.94482   -249.76705     60.60026     638.8144   1809.22361   2821.54025   2829.57541   2118.73155    974.10003    165.95492    -13.50918   -346.83767   -116.14563      2.44548     51.91685    160.51555    228.35047    139.34141   -144.87502   -177.68533   -247.14815   -208.55488   -247.25141   -337.43905    -322.4525   -308.68535   -255.29391   -264.52838   -225.41435   -178.41996   -121.84452     -437.289   -379.62934   -189.29916   -223.33196    -49.74486   -103.83914     78.80408    331.01233    740.38694    952.79519    831.94926      515.199    241.84502    -86.65206    -76.65385   -280.89411   -206.55345    -75.26674     17.13546    464.92623   1225.23706   2201.03934    2819.5696   2863.12315   2320.37506   1637.47059    947.15242    440.46045    166.76827     49.46566    116.00544     37.35038     46.07888    -98.06535   -264.90206    -224.9017   -346.16534   -184.54461    -343.0896    -288.5302    -322.5642   -145.69411   -236.77411   -355.26436   -356.07153   -244.12806   -195.57958    -91.26158     -9.77443     97.26624     80.51619    -24.56336   -286.04584   -321.67705   -415.86199   -401.23762   -255.49411    -73.19468    -57.41097     31.59979     243.6487    259.99942    118.96455     -1.43099    -84.63577   -153.43586   -168.25232    -296.9369   -266.04206   -260.67513   -106.09866    -275.1144   -272.37062   -217.43911   -185.61485   -272.69428   -193.98643   -216.25608   -206.72528     54.28504     99.83056    175.94087      267.634    236.52254    207.91728     29.30806     16.53564    -117.4233     -6.20734    -43.39124     15.83006     102.2969    192.82811    127.69274    110.63265     33.14733    -75.87114   -141.85349   -302.44929   -269.76195   -264.32122   -320.36588   -370.77617   -372.80093   -426.40551   -198.22053   -110.38439   -286.40314   -254.18944   -216.93898     -85.8348   -115.90934     60.64603     29.22984    -64.27895     19.46739     91.23743    105.16628    193.40599    129.84467    294.60871    410.60732    495.46717    229.05925    375.45517    221.45812     -6.08437   -115.13996   -153.73937   -215.16996    -146.2881   -146.66356   -398.53343    -54.49577    -207.6266   -462.72626   -109.57235    -39.59665     -1.43588    -19.36621     38.22692     31.98704    250.39829    262.67795     611.4248    385.11288     690.1921    630.39504    653.55731    664.97855    549.81464    460.08129    493.66266     327.1004     48.54444    -48.94536     97.93274   -191.75795   -198.59144   -219.39354   -257.71572    -47.98292   -137.24113     51.54172     11.24381     81.73124   -123.49675    162.88117    -27.99277     76.08617   -102.62047      19.5768   -119.16589   -160.29462   -150.05979   -412.86778    -260.3943    -177.6394   -268.90186    -197.3126   -378.26196   -204.30205    -279.2812   -417.91494   -408.44159   -309.43628   -200.06433     -258.146   -322.41202   -140.54679   -196.33287   -142.14287   -283.61533     10.08084     13.02238    -93.29493    -83.93511     11.42358   -140.97795    -83.79261      41.4181    -98.90847    -69.63603    -73.14763    -52.52242   -127.56556   -121.90801    -222.2712    -135.6802   -255.20078   -185.28077    -73.02986   -293.63466    -84.86538    -86.35441   -179.09298   -298.08767     28.74732   -274.57274    -55.78354   -197.96759      9.97615    -57.60884    -62.52179   -170.03085   -192.76141    -83.52778   -114.08676   -218.17932   -228.78512    -39.03591   -322.94133    -128.3213   -162.84531   -204.99563   -113.39326   -160.14781     60.33266    -30.57016     21.35935     108.2506     56.18268    123.68583     76.37967    152.40271     47.42677     92.93398    139.99038    -13.83125   -208.10345      0.88434   -213.61357    -48.69248    -77.83429   -240.86012   -185.98458   -186.16641    -179.2692   -126.57431    -327.0512   -127.14772   -136.70082   -258.61344   -197.19128     35.21247   -182.94052   -714.76608    -65.36077   -262.19026   -379.69628    -78.69256   -359.53085   -353.07833   -311.82656    -19.28261    -497.1664    -217.7913   -143.06011   -416.52411   -541.15933   -369.83634   -389.53245   -323.83657   -352.07931   -306.97935   -439.58063   -360.57172   -264.18743   -266.05748    -193.8992   -344.73456   -448.06028   -432.40937   -399.86919    -300.2502   -470.45212   -788.81223   -359.28499   -411.63328   -608.93688   -563.18754   -386.84552   -626.02167   -508.29632   -353.42677   -693.80377   -543.82323   -509.46972   -453.28168   -675.17839   -519.94731   -437.44209   -344.27572   -599.84858   -425.17249   -767.25459   -738.52094   -604.63894   -815.89991   -910.36922   -603.94719   -759.70718   -602.36055  -1041.92048   -885.98975  -1059.69857   -853.97373   -959.30646   -969.56835  -1049.81386   -925.51838  -1145.35458   -913.18341   -876.33452  -1157.08035  -1020.06257  -1237.69339  -1154.44441  -1179.90479  -1248.86476  -1091.77301  -1636.80333  -1279.05188   -825.81929   -882.12225   -1000.1984  -1270.24769  -1098.94071  -1111.63923   -1176.7621  -1089.45418   -1005.1294  -1432.25707  -1345.10329   -1471.4579  -1219.42021  -1177.27203  -1605.19681  -1684.80269  -1107.99021  -1493.81151  -1721.55544   -617.81855  -1373.93012  -1226.57033   -463.95519   -1047.8503   -632.15055  -1087.20364   -263.67538  -1009.22091          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       17.50   -484.57887   -304.45389   -242.35842   -330.38365   -448.37884   -450.76072   -255.27784      -462.15    -314.9259   -125.47543   -347.42506   -323.83143   -125.95967    -20.75337   -400.75523   -543.61506   -152.23838   -113.50539   -510.03656   -363.23627   -226.16286    -287.6796   -128.79442   -171.76457    229.53553   1430.61436   3248.27923   4001.98808   3041.92586    1097.5149     135.0214   -210.49805   -159.52668    -79.80716   -386.96601    -56.65353   -290.29512    -286.0371   -207.35807   -298.49699   -289.87073   -389.77725   -213.56105    -255.7918   -457.16757   -437.66238   -309.95772    -321.4592   -472.34945   -367.45453   -174.22412   -429.32228   -363.94137   -370.11205   -289.60038   -288.96373   -304.72947   -283.96197   -111.17096   -181.42841   -298.29866    -417.2941   -545.53593    -429.3609   -382.03966    -328.7559   -236.85567   -268.13276   -104.11506    210.92911    823.07532   1763.18347   2177.53432   1921.38072    1075.5983    248.38148   -548.36014  -2406.02818  -2839.85676  -2623.20771   -1460.6078   -483.08198   -221.36058   -279.04741   -320.15273    -76.24325   -162.23836   -120.97568    -81.11017    700.33956   1616.56636    2551.3817   2601.34693   1890.47501   1056.80334     90.60423    -74.80337   -235.39836   -180.73549    -60.04354     40.52167    288.77882    131.18853      89.1941    -89.60368    -70.09947   -259.18231     -269.223   -262.95535   -307.80726   -244.97133   -336.77815    -250.7107   -243.17156   -420.54208   -256.87893    -191.4604   -129.40295   -281.55512    -88.59985   -192.69145   -240.50303    -58.79811       20.563    517.14058    798.55685    1021.8323    635.05288      620.439    254.56567     64.38051     -231.065   -115.33248   -224.46348   -104.54751     164.2419    498.12417   1424.62435    2001.5899   2635.45905   2563.66009   2086.18742   1496.05267    875.54634    306.74589    323.85659    209.62469     59.19291     52.60863     22.85931   -100.41764   -108.95087   -127.84955    -84.26765   -354.52444   -399.04655   -239.21876   -275.17469   -203.42086   -207.89648   -194.69531   -331.63744   -129.68772   -234.61405     -6.55872    -55.94102      8.87989      16.1615     -94.4296   -167.13321   -278.19582   -351.50668   -302.55622   -142.36191    -71.88356     21.88378     16.06104    140.04603    197.78431      1.58381    -55.41271    159.53161   -104.85459    -46.67606   -214.64209    -258.3017   -232.38719   -318.54196   -172.85803   -351.01373    -12.26432   -250.98374   -217.59757   -238.43127   -265.37947   -190.88754   -154.95251     16.68021     78.64397    256.78446    144.02493     138.0711    179.75708    120.14043      6.56417    -22.00706    -71.99216      4.44484     93.51162    174.61233    227.86606    247.39291     60.24364     40.18439   -131.69382    -24.12458   -121.97116   -292.22384   -231.69467   -269.23419   -201.90272   -287.43594   -473.97913     -330.226   -110.81699     -53.1842      0.59352    -80.65399   -138.22633    217.95488      6.22801     12.39786    -42.48531      4.34625     409.4089       37.203    133.21536    118.87324     83.38361    168.78053    168.38661    248.76422    -10.14989     74.65462     -49.0173    -14.95557    -87.97711   -314.25983   -233.86294   -170.13511   -262.64771   -217.00247   -145.28119      56.5282     -63.0383    -87.63794     -16.9069     66.19203     84.41392    167.20142     323.9035    372.69896    534.95751    594.66677    670.13136     669.9444    678.03091    345.37741    648.29549    429.78682    230.31603      98.3593     173.0762    209.06413   -116.00961    -196.8213    -65.50479   -139.29937   -174.23261    -40.05066    -28.85224     77.98287    -48.46579     62.06732      52.3505   -118.16618    -70.23678   -161.65021     -78.1356   -178.55399   -216.32736   -257.31676   -301.81048   -343.84344   -440.77807   -323.44652   -351.64028   -447.91293   -246.62353   -243.99697   -294.05407   -220.14196   -236.44843   -189.98158   -270.78625   -171.53158   -317.27524   -151.17981   -357.52091    -235.8595   -144.48453    -238.5406     11.68475    117.27334    -87.70547    -50.84199     -31.0182    -67.09458   -119.39645   -182.12398   -129.69069   -207.52506   -126.43298   -127.14081   -228.28154   -211.64925   -210.89546   -314.66381   -192.87345   -129.98919   -125.15306    -91.12167   -160.99942    -36.27038   -169.41583   -168.36533   -210.41412     -8.98722     69.73328     42.38533    -80.73237   -189.48768   -314.02791      10.9889    -201.9986   -135.39001    -120.9353   -301.91671   -137.61929   -283.35542    -88.02249    -49.36646     62.62348   -219.30857     15.55406     -2.61675     52.43104      1.28256    -38.98268    250.62087     84.48836    442.62151    163.12434     -50.1996    111.58686    -24.07207     29.10935   -154.32895    -59.89085   -190.38103   -278.49083    -24.60999    -104.1659   -375.72703   -112.71417   -152.77217    -87.57875   -175.87288   -356.05001    -248.2201   -294.70861   -139.40373   -268.42955   -424.99276    126.59932   -179.31317   -243.73343   -312.61085   -315.47315    -167.4395    -405.7511   -273.07993   -114.56043   -413.37176   -558.83814   -474.61179   -817.05252   -344.89285   -459.57005   -432.97797   -160.09036   -327.20351   -572.04998   -405.51235   -229.32948   -331.31451    -515.7617   -279.55828   -232.38014   -294.82785   -309.12165   -479.70343   -350.18733   -221.23318    -443.6858    -231.7822   -669.41135   -212.44433   -261.75204   -426.75667   -308.96103   -663.23322   -496.80831   -571.06768   -604.63559   -619.19974   -658.88478   -963.91164   -475.12327   -550.30095   -506.00363   -787.58547   -700.40653    -808.1677   -951.48541   -908.18424   -723.07681  -1125.89697  -1025.46015  -1278.19378   -934.83005  -1021.32667  -1141.68395   -946.46859  -1165.17829  -1090.35696   -1309.3366  -1256.35106   -980.71717  -1141.40189   -973.85533  -1070.82006   -987.60121  -1053.73367  -1293.94122   -1101.7559  -1218.80639  -1175.49206  -1231.63991  -1551.10445   -945.75094  -1153.88603  -1592.84699  -1288.19482  -1583.44844   -878.05392  -1514.09649  -1610.71705  -1522.32757  -1763.45274   -695.16702  -1371.91132  -1145.90522  -1014.47079  -1112.50213  -1556.86035   -1338.2858  -1753.92592    -964.5706  -1242.31247  -1020.70603  -1294.15433  -1863.51159  -1313.02521   -898.51997  -1530.49206  -1264.54368   -934.28384          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       18.00   -241.29844   -353.91965   -216.64894   -214.69462   -358.55992   -208.19766   -299.04912   -361.64592    -285.4468   -204.94376   -319.16694   -349.40666   -265.00677   -268.64082   -413.87518   -297.83529   -258.26983   -243.22198   -360.33619   -332.22722   -374.15237   -217.10432    -184.3583   -139.78556    289.58927   1256.66219   3112.84317   3976.70745   2905.27871   1096.17366    158.58015   -303.74867   -210.34826   -149.21152   -285.07176    -256.3121   -298.62514   -362.71557   -345.22493   -317.34569   -214.74722    -362.7914   -447.55007   -293.83401    -433.1374   -237.16334   -455.29552   -315.80457   -434.81131   -501.55329   -242.71176   -532.79351   -364.40028   -228.02284   -290.42174   -346.34396    -238.5487   -290.40259     84.14459   -168.62696   -206.45296   -457.94217   -322.08067   -275.06313   -328.33973   -264.64068   -306.41733    -353.2532   -266.02144    132.40191    690.01574   1640.52172   2336.49553   1999.87901   1058.99469    274.09961   -467.16185  -1266.11928  -1659.39572  -1510.53392  -1011.42507    -534.9198   -339.95143   -392.13326   -256.06383   -203.91446   -238.32091   -179.15479     28.34721    692.30692   1548.25073   2704.19289   2845.16695   1955.03296   1091.42977    306.94952   -107.60052   -204.52427   -219.25175    -21.32128    -48.87223    291.10107    241.13011     62.20873     -8.32162    -76.89475   -207.28026   -298.14944   -301.67825   -273.19991   -325.22331     -251.492   -330.14868   -162.06251   -188.01559   -284.65537   -216.17174   -216.75431   -273.09265   -303.03653   -158.90636   -279.95413   -153.90885     117.9646    390.65702    776.65077    985.84418    740.20634     729.9564    330.92211    -27.33187   -180.22824     -163.382    -21.82896      55.4482     77.66269    637.02366   1349.61848   1886.70852   2646.54521   2739.87024   2246.78436   1432.94404    807.62819    486.01091    132.33442      9.97654     102.0078    238.60619     -2.55699     -49.5153   -319.69882   -284.03888   -149.72195    -99.52334   -319.14634   -263.40696    -93.65421   -277.34858   -161.00042   -216.28525   -331.61594   -192.40988   -235.62418    -51.03888    165.98426    -38.37773   -145.93042    -173.8265   -205.07667   -265.53984   -165.55252    -275.6759   -176.93604   -301.54873    -35.21676    100.61269     179.4327    356.74372     92.93097   -101.92211     26.29332   -371.67084   -117.74805    -142.7182   -221.15164   -197.77719    -45.69251   -347.00925     -209.387   -263.41134   -198.67949   -147.72609    -87.96759   -136.76196   -203.83416    -19.90981     97.66816     31.14482    341.00924    248.23122    -56.06905     67.24035    -43.07613     275.8077     96.18374     24.05902    186.69153    219.19387     79.76595    -16.99477    134.91191    271.19746     15.77556   -163.55628   -267.24932   -342.72963   -266.28088   -194.07846   -361.80061   -109.31697   -295.31496   -214.00275    -316.3389    -47.30292   -116.49306   -118.64186     50.87422     49.05224     74.49772     88.81946     86.69677       82.056    240.19524     90.93683     331.1363    150.33709    228.91292    215.62058    230.97311    184.39675     69.18283    270.43314      90.7319     71.55947   -211.26624   -120.22515   -309.05861   -378.92545   -248.76099   -251.99457   -133.96552   -172.54428   -178.65949    -59.42993     50.69378     46.79315     14.57893     -5.32262     264.5871    157.19983    354.68878    510.69022    269.29704    639.45014    728.77474    817.44217    726.81839    422.34144    390.19831    179.37681    114.37421    122.23336    101.53604    -100.4317    -39.30218    -41.62214    -87.70788    -124.4764    -62.24523    -40.01479     19.50492    101.79194     -4.09306    122.79731    -26.92556   -162.93698    -88.62349    -149.5323   -247.67697   -109.68349   -206.59593   -281.82777    -431.1113    -364.1129   -367.47103     -253.131   -329.53892   -313.56382   -321.21622   -309.71526    -33.83262   -271.27772   -305.87548   -210.02522   -240.22382   -303.03184    -88.68632   -230.17152    -45.70556   -186.13768   -140.50886    -44.35734   -212.01045     -70.7155    -73.93212    -46.35904    -54.58213      5.28057    -145.4892     -96.5024   -113.73538    -86.37127    -229.2598   -227.58207   -310.03362   -243.90802   -160.19118   -214.30341   -166.44232   -199.95684   -257.14934   -375.20729     -7.76094   -120.81641    -80.01205    -54.15336   -168.04262   -236.45003       4.2034    -70.18304   -134.96806   -267.68977   -163.10134   -301.31287   -329.39315   -277.37554   -345.84371   -352.33487   -187.86767    -69.86239   -247.45871   -199.62899   -216.50143    -94.33245    208.10926    -43.74012     51.87088     -1.99716    -12.92274    -33.85888     63.43407     63.49822    -16.73162    200.60187    -85.41552     70.56119     44.00249   -116.65904   -179.93656    -11.34212   -281.70894   -115.23858    -93.62333   -306.04108   -204.95931    -81.94785   -277.00208   -293.01619   -280.59875   -135.24589   -145.70135   -252.29831   -351.74618   -110.48511    -391.2589   -424.61428   -584.24698   -279.22973   -323.85504    -261.0205   -485.49736   -303.83724    -665.0061   -544.27714   -529.90807   -599.56658   -301.37427   -386.36131   -458.39499   -540.70664   -365.98107   -408.90396   -432.95549   -236.36356   -308.31112   -269.11545   -717.43624   -597.05927   -192.97191   -460.72242   -580.80191   -458.28106   -554.38551   -424.44141   -363.23337  -1000.87994   -550.49945   -445.07847   -739.37541   -498.60627   -509.29028   -829.69801   -551.91722   -655.25418   -671.45002   -807.63459   -654.53441   -850.60296   -777.35449   -933.58297  -1020.81322   -950.02546   -890.47608     -838.792  -1052.22441   -734.95577  -1286.15387   -975.37674   -1004.0849  -1063.60325  -1016.00632  -1146.50556  -1335.83601  -1561.14638  -1409.06668   -906.35559  -1365.67244  -1208.16265  -1495.52733  -1170.88252  -1297.05365  -1471.79064  -1596.51442  -1264.89001  -1602.55299  -1150.07085  -1146.81578   -1204.0401  -1193.78157  -1381.11559  -1529.74515  -1362.01328  -1366.76741  -1205.11638  -1029.59199  -1194.61908  -1391.26264  -1167.69759  -1365.27955  -1219.42777  -1324.84557  -1771.07898  -1563.34891  -1497.92097  -1587.73837   -2148.0578  -1769.49034  -2459.91028  -1394.80347  -1592.43414  -1925.49147  -2089.46791  -1170.64673  -1204.69577  -1660.50325   -184.69664  -1582.22666          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       18.50     38.99571   -563.21782   -199.28916   -112.36219   -370.11258    -88.77214   -301.86668   -324.22769   -321.90603   -359.69071   -281.33358   -450.42297    -84.53641   -425.64202   -351.02438   -346.85122   -310.96744    -260.8111    -354.5682    -43.19972   -114.88016    -216.9504   -278.55458   -155.31554    232.01079   1377.99768   2915.83843   3917.79484   3260.95315   1194.82237    -51.00987   -297.85579   -199.34978    -45.40567   -187.57346   -251.36922   -185.66835   -292.61318   -221.22044   -272.84975   -238.61695    -353.1862   -495.28769   -347.16932   -341.90955    -259.4214   -318.32649   -215.42578   -395.11528   -499.46094   -409.50998   -439.88846   -347.29656     -397.545   -314.29209   -467.86083   -106.45383   -156.69714    -93.30146   -467.55706     11.17288   -308.34464   -338.16387   -351.33828   -379.87871   -294.44103   -409.43565   -423.84835   -344.07881     16.69584    701.46299   1784.57399   2361.80386   2002.78084    950.75259    453.34158    -43.18098   -546.59561    -854.4589   -907.03691   -583.42762   -355.43889    -345.1406   -148.13947   -187.58131   -281.08658   -267.91344     32.62393     -9.34124    562.26784   1813.25119   2702.40416   2960.47166   1906.06961    938.79968    177.17229     17.77177   -125.09245   -118.18848    -23.60607   -160.42954    -26.25197    301.45623    231.19794    -21.74625   -152.88407   -314.86896   -409.48976   -300.58359   -321.73454   -272.06908   -460.01113   -384.30163   -329.53365    -295.6003   -418.53469   -216.92862   -164.46611   -214.39662   -193.38186    -133.0964    -366.4607    -47.97801     45.56318    303.85786    726.83562    857.08242    759.62888    439.94361    173.53233     89.70364   -161.74876   -228.57157   -225.39306    -115.3732    -29.63887    469.53971   1346.48254   1907.84557   2375.60046   2728.93124   2036.38514   1399.41185    820.71971    445.75471    215.40449     224.6998    135.23514     -8.76639     11.17245   -165.14695   -178.57595   -269.66246    -130.8692   -256.27491   -421.46618   -202.37822   -158.68799   -203.68876   -166.84969   -232.36862   -257.44303   -192.03416   -201.12066   -175.32388   -145.18994      1.43545   -144.83181   -279.62049   -339.61252    -290.3362   -230.25414   -298.58549   -213.70153     12.68227     52.86224    129.49166    176.39283    223.82543    108.30248      58.2744    -57.00396   -135.75174   -196.80005   -176.53026     -114.268   -217.83851    -359.0367   -310.95005   -361.44772   -352.58719   -318.51562   -316.17433   -257.27088   -106.54437    -97.09794    -10.08791      22.4259    157.77746    234.94812    317.58579     164.5035    211.12032     69.57993    -36.79395    -23.75905      5.89862    133.66221     82.73058    268.77454    215.79032      -1.8784      5.14513    -20.77912   -149.39657   -224.56769   -232.97024   -194.99282    -352.6602   -178.84932   -267.43383   -280.89678     -211.034   -241.55421   -168.12871   -137.88096    -45.78627    146.38026    116.91833    170.96086    107.85866      64.2018     105.1487     33.49664       -6.842    301.28618    153.24155    177.99807    291.94478    318.88172    423.07552    129.72727    121.08553     49.43027    -23.84502   -247.23756   -183.51908    -77.23994   -290.89907   -208.10676   -149.92034   -143.71837    -68.32621    -99.31762   -143.40279      2.34386     75.36084     21.45257     30.09349     97.61994    260.37572    406.46244    399.60996    494.27898    641.91514    817.36282    597.35049    637.55494    751.80174    455.70551    273.78589     209.8146    131.40161    -42.71133     -9.00141   -326.77749    -55.37228     31.59296    -27.00961     -65.8333      53.5956     63.55089    -48.03098     56.90731     79.60185    -61.83736   -102.80161   -122.06039     -9.51308   -192.71834    -79.82161   -256.65362   -277.40639   -382.16923   -387.52053   -243.97653   -452.04263   -483.25728    -450.0126   -307.06194   -408.78308   -200.10599   -256.45707   -167.72024   -238.90337   -285.43813   -245.36105   -195.53349   -337.75503   -166.75297   -137.63383     30.17589   -109.15044    -99.54106    -42.19504    -125.1406   -149.70759   -131.47116    -54.90874   -162.31087   -190.29813   -156.77233   -188.83013   -205.63822   -203.21567   -202.76596   -239.96234    -220.3246   -191.08917   -340.16167   -266.56086   -152.60881   -201.13034   -165.53676   -131.61586   -209.67527   -120.62071     45.08646     -214.123   -202.45896    -83.63462   -153.75101   -223.85057   -267.22974   -261.20894   -195.29591   -411.87076   -327.92117   -350.46502   -567.97268   -281.89591   -197.86628    -100.5584   -215.65943    -56.74924    -58.92287   -224.74174     34.84512    139.55676     57.45551    -39.37257     -50.0059     -6.33367    107.41455     13.56736     14.14827     64.61105   -257.09348   -157.01483   -183.79989      1.28063   -341.24391   -147.93148   -216.41052   -135.17975   -287.80628   -301.17515   -112.10714   -339.06608   -279.16348   -271.95711   -387.50594   -357.27927   -595.55572    -296.1759   -547.94254   -516.33812   -206.89735   -320.24695   -209.59255   -495.43109   -538.42223    -440.8851   -473.53345   -548.95224    -481.7887   -589.63856   -531.95418   -711.85505   -577.57633   -398.05643   -311.21845   -346.80383   -498.31456   -476.29486   -618.97603   -288.75545   -806.98743   -401.82819   -291.78682    -480.1612   -646.01586   -603.75651   -590.44394   -606.82346    -690.9852   -556.86408   -541.38889   -520.77004   -400.72632   -701.45598   -506.72387   -663.95077   -482.45115    -716.3267    -606.4966  -1020.19351    -919.1101   -633.18199   -933.88753   -778.97409  -1104.41158   -926.94589  -1161.50122   -880.15423  -1127.98751   -965.66706  -1462.20387  -1205.38403  -1384.95434  -1284.99926  -1008.14912  -1125.75769  -1220.65946  -1097.19701  -1367.24983  -1515.17108  -1146.70481  -1074.51352  -1208.48309   -946.74258  -1156.61362   -1476.9372  -1358.66049   -1242.8251  -1123.84038  -1620.99795   -1717.4296  -1567.70496   -927.09946  -1577.27168   -1429.9568  -1458.01788    -1302.356  -1734.57782  -1453.28077  -1459.79446   -1752.8773  -1322.53163  -1735.33481  -1090.03983  -1474.53676  -1841.47016  -1640.65319  -1820.84466  -1997.51778  -1653.87774  -1701.73758  -2027.27528  -1512.72685  -1532.69418  -2030.17365  -1751.14854   -2439.8088  -2889.36833  -2830.35567  -1528.87921  -2019.14861          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       19.00      4.49215   -809.52318   -300.93997   -124.10689   -360.96285   -231.64387   -300.45087   -308.36772   -324.96696   -404.39788   -100.08144   -367.08662     -1.28288   -336.48377   -331.34845   -413.69993   -312.25606   -236.24633   -464.08964   -121.24749      5.32722   -326.93182   -239.70051   -161.24936    -33.24362   1433.79695    2748.3556   3770.58306   3046.67653   1265.74668   -110.99008   -323.74269   -160.18774   -331.46896   -197.59511   -509.25097   -168.67205   -268.05976   -350.00151   -330.31312   -371.18054   -302.71096   -422.85615   -335.08646   -380.34128   -328.70562    -365.9803   -372.24748   -301.15324   -196.15342   -390.74349    -282.9676   -322.64922   -406.71334   -380.69017   -305.67101   -234.54102    -87.01029   -337.74646   -220.28543   -214.53665   -163.03976   -224.93789   -331.26139   -446.47818   -364.01319   -330.26662   -237.47378   -196.86962     81.01152     638.5685   1762.60423    2416.6974   1818.97307   1046.52385    402.68735    105.93523    -23.93244   -232.53905   -413.98649   -302.61246   -423.35609   -164.64335     -198.459   -470.23862   -224.23131   -190.56888      -38.107     43.83893    959.49095   1840.25866   2779.44297   2946.75502   2028.31083    625.16266    347.19658   -153.38727   -206.38903    -34.52466   -165.59718   -165.42373    186.48461    140.90199    127.03308   -115.82479    -61.35822   -214.26376   -559.09458   -447.62239   -408.19908   -376.99041   -285.07902   -371.94437   -531.22532   -193.83172   -258.88024   -154.21433   -236.88641   -229.79061   -123.82539   -234.66384   -141.66285   -187.83952      62.5732    253.07944    733.23693    929.39781    908.66422    512.97268    238.08025    -44.00003    -93.28075   -108.67786   -262.64478    -49.13479     16.34263    515.47336    1364.5867   2023.10587   2511.10193    2695.9087   2020.46212   1453.86059    733.40174    338.65552    150.65129    328.45275    160.62332     51.19249    110.98882   -118.09694   -295.44677   -205.73932   -224.69878   -218.53485   -386.60212   -282.21208   -290.56009   -233.91769   -275.47434   -206.42958   -328.80442   -208.21899   -259.87414   -261.79676   -105.04282     82.93385    -74.65445   -138.67459   -172.36027   -199.93648   -112.43088   -241.33586   -282.48227   -221.76571    159.31853     78.47982    328.71846    337.05101     92.95299    137.70772   -124.99215   -141.88452   -186.19097   -241.88094    -195.6083   -239.58937   -276.16533   -114.54626   -383.28944   -226.56233   -344.77891   -176.39161   -233.64363   -195.61851      0.18277     30.13439    213.02633    175.55587    187.88716     37.82146     96.74051     84.71866   -122.75073    -64.43677     24.46432      29.8846      0.26934    167.15909     81.04025    227.11907     16.34619     72.90428    -55.06554   -142.78467    -267.1676   -340.27287   -302.96479   -222.37347   -401.87098   -253.91301    -367.8419    -336.1879   -106.84909   -264.59063    -73.41235    -35.72804    -18.01233     90.63731    -55.05121     50.98163    239.54984     -13.8036     28.69454     38.10783     167.1811    178.33758    280.85174    236.02204    284.50227    224.55253       67.508    147.19311    -27.26009    -87.18563    -46.88046   -207.20788   -209.53568    -225.5606   -190.61984   -226.15653   -192.58156    -29.08513   -106.82218   -275.47049   -106.67989   -268.57346    101.12498     63.45722    232.35067    147.22857    501.68273    426.34981    656.36989    831.36404    760.32209    906.38402    731.96987    628.03374    423.64764    412.08072    126.23146     276.7269     10.73022   -142.61572   -141.07944   -105.10592   -188.83369     -7.04953   -124.85174      32.1693    -22.95886     49.38942   -151.42369    -91.87649      6.07572     23.79792   -173.67224   -271.26346      6.82819   -210.92136   -383.82415   -201.96224   -139.38083   -320.24734   -383.96106   -363.15468   -214.84993   -551.05468   -302.93922   -279.65309   -285.47869   -363.09862    -286.4842   -258.60762   -172.46069   -270.25975   -164.36302    -108.3831     -310.416   -157.21206     -8.48749    -301.7782   -277.09098    -60.99698    -95.59245   -140.16968   -112.89147   -145.46844   -109.64546    -142.4718   -211.80256   -280.87064   -489.53335   -244.68837    -204.5787    -227.4197   -311.15727    -81.40045   -184.86481   -128.65146   -348.25531   -157.21389     -38.9258    -309.9697    -95.78327   -124.91199   -124.64137   -288.26516    -23.87384   -302.33364   -315.33116   -244.17123    -237.5112   -407.96052    -53.75242   -256.53785   -296.34254   -210.09353   -331.82298    -86.94883     65.31599   -151.08316   -147.91012   -224.38393    -211.9269    -87.66877    -19.04707    -89.98294    -25.05222   -133.07212    211.70588   -112.56766     38.89269    -78.45705   -157.16198   -181.36829   -197.06254   -370.58592   -295.38689   -364.50928   -378.28093   -301.51887   -363.58953   -144.33829   -295.32348   -301.35681   -478.51585   -256.57401   -159.14943   -278.95634   -282.63793   -341.45149   -380.29658   -438.27053   -547.58738   -307.51416   -472.62375   -508.76199   -570.36098   -594.18777    -454.3076   -492.60327   -529.72673   -528.39996   -521.44844    -654.6903    -581.7383   -457.72731   -510.26364   -516.57132   -510.74096   -706.02485   -589.22321   -458.01448   -536.11225   -591.48562   -451.30304   -618.52254   -560.82968   -461.94124   -554.74834    -584.7051   -384.84167   -547.79957   -725.36863   -631.67995   -703.34058   -712.79492   -804.89031   -838.26262   -769.40748   -658.23527   -863.81918   -635.33309   -667.65049    -639.7379   -739.84505   -956.07125   -728.45909   -940.56827   -804.94667    -791.2809   -841.42478  -1298.73207  -1129.41041    -980.9456    -976.9374  -1097.35286  -1076.62421  -1029.97487  -1389.29208  -1415.81804  -1230.46244  -1190.39466  -1695.86769  -1363.07177    -1262.512  -1067.39801  -1324.03648  -1136.21198  -1443.14783  -1704.33493  -1693.22371  -1500.91383  -1678.46527  -1547.45694  -1448.74035  -1690.39616  -1460.99755  -1376.96684  -1945.97631  -1507.67813  -1105.42945  -1814.98299  -2352.50402  -2110.76073  -1558.02061  -1910.83566  -1756.88799  -2234.82941  -2061.20666  -1728.28398  -1992.24784   -1300.1831  -2321.73397  -2766.52943  -1315.08408  -1607.24583  -2420.26196  -1640.31306  -1816.45549  -2426.70176  -1436.39157  -2713.84913  -1694.61056  -2033.21383  -2476.93625          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       19.50   -208.52004   -792.77477   -402.79882   -182.03432   -343.36824   -338.05961   -267.09173   -262.87453   -328.71773   -280.08499   -235.73382   -203.93833   -284.88553   -309.06834   -393.57379   -348.24856   -324.35998   -357.43051   -310.21736   -147.00252   -114.05033    -285.2817   -218.35767   -148.25737    166.12069   1289.32177   2891.52357   3804.60934   2681.92592    1194.8103    -14.37061   -350.22533   -388.52497   -357.25786   -276.13981   -401.01946     -8.85764   -414.38649   -179.99228    -384.7011   -465.00038   -325.18713   -410.31796   -301.59743   -312.54931   -388.02616   -490.07976   -463.09732   -403.51478   -404.74462   -347.38077   -251.46325   -338.87063   -305.06726   -317.07876   -187.67657   -257.83231   -238.84931   -188.07684    -256.5859   -326.30256   -270.34237   -230.37613   -333.88948   -231.12572   -326.36881   -219.54083   -298.41752    -219.6998    237.92559    919.38093   1779.08203   2512.62052   2079.87194   1136.69447    418.22468     25.56796    -68.45756    -70.73308   -168.65126   -326.99705   -318.14345   -214.18994   -286.93303   -168.83532   -141.98033   -140.85266   -132.57328     61.16877     937.0241   1884.20432   2753.02993    2733.1025   1888.64711   1111.39833     241.6085    -52.50972   -111.41968   -310.99705     23.17872     42.36057    178.62669    289.66224    123.82539   -182.06953   -126.90943   -115.99551    -198.9161   -387.90947   -347.60555   -342.89664   -388.35195   -282.31911   -369.25151   -376.05511   -329.31905    -262.5013   -122.73161   -202.19438    -171.5305   -213.84423    -78.46616    -90.26062    139.00548    474.68608     497.3676    795.13291    907.77385    690.26253    273.39428      1.69137   -158.09118   -263.77134   -118.43886    -21.20424    -39.94854    669.53773    1274.9194   2085.16965   2489.27935   2674.54389    2263.2982   1537.46878    658.11225    392.91274     289.0642    259.88681    161.90079     41.50694     -33.1972    -29.90345   -177.43983   -293.85726   -305.31734   -323.38688   -189.92606   -341.71243   -170.69957   -418.00657   -347.37028   -321.03007   -324.56731    -47.09566   -145.98016   -139.45349   -102.56614      22.3936    104.55827     58.70352    -34.67018   -264.00105   -227.18494   -140.90607   -226.28515    -62.20765     69.40645     92.10893    127.80742    183.07905    148.35275     84.69623   -122.46703    -23.02219     -134.021   -235.05823   -258.59905   -173.99094   -232.60009   -440.26069   -368.51293   -231.16152   -228.11546   -220.29856    -160.3805   -102.07238   -174.09566   -114.28314     63.23586    101.02404    220.59853    242.77187    175.73104    143.35404     55.67249    -26.84475     15.50617     -7.09442    136.63786    136.06593    191.35902     71.88588    261.68896    -25.53117   -148.01518    -46.88769   -366.84009    -235.8851   -222.56484   -232.38524   -314.76615    -329.3668   -272.04185   -250.36212   -222.34779   -250.47837   -131.97387   -101.35122   -110.92025     65.13756      0.42881    117.77413    105.38653     23.98117    119.63006    125.17661     32.93122    338.56629    293.22185    323.84987     248.4883    245.84639    302.65861     31.48552    281.47014   -191.81451    -88.91616   -146.62224   -218.75085   -123.47082   -236.78784   -285.62467   -178.58265   -288.24012    -16.45541   -163.38769    -16.07235      7.33994     90.98769      67.2777    141.88517    216.62806    310.86725    482.85092    515.84737    657.18147    691.76841    659.87409    667.94235    378.93606    468.90747    521.84067    273.58919    133.44234   -101.87566    -28.28103   -149.96358   -158.92133   -210.43226     15.30359    -59.27708     20.38924    -36.27772     54.76233      42.7443     78.39184   -143.87702    -11.68384     -62.6871    -197.1347    -24.31622   -413.26999   -185.38691   -104.78381     -332.959   -468.76993   -409.07156   -363.39644   -338.60484   -286.82003   -259.93355   -317.17039   -398.65146   -237.29699   -246.25547   -228.99767    -287.6955   -226.46708   -324.00363   -243.25142   -289.87658    -43.46575   -200.95183    -85.94001    -75.78475    171.85481    -83.77818   -160.63054   -129.70967     -76.4974   -122.33065   -175.84015   -174.55382   -326.25528   -302.96434   -388.21584   -221.57961   -251.58599   -199.06852   -352.98082   -141.84618   -404.77513   -217.30445   -178.45841    -196.7518   -373.51111   -229.60483   -315.52063    -13.39903    -355.2245   -226.93116   -416.58516    -213.7019   -361.24131   -257.03477   -267.76479   -196.50212   -315.07792   -292.33407   -270.32069   -290.16358   -200.59274   -307.41974   -370.88912   -164.18375   -258.02719   -104.60452    -114.1805   -109.16549   -136.07136   -135.37215     18.75872   -160.63897   -157.81209   -108.25142   -201.65209   -257.30535   -157.52494   -271.56387   -274.47562   -609.14074   -165.60398   -292.69539   -339.82609      -399.14   -417.27398   -333.93384   -351.89496   -465.18262   -384.89063   -358.04778    -441.0153   -327.18335   -340.26539   -341.80793   -548.14311   -400.30885   -464.01419   -402.82488   -483.99829   -369.80818   -705.86755   -429.76733   -471.76403    -806.7593   -626.75223   -560.98898   -547.28025   -465.89988   -455.17381   -575.15706   -609.69088   -608.29207   -647.45637   -539.06228   -734.01201   -503.73742   -810.12707   -547.91284   -631.45331   -596.14961   -591.12387   -535.23851   -530.71776   -624.79292   -770.75075   -842.91102    -767.6839   -867.01815   -712.83042   -911.89225   -899.93884   -560.32808  -1013.75775   -927.98379   -736.69681   -681.77014   -757.21282   -1031.1103  -1193.49882   -919.98601   -924.90863   -946.41926   -888.81188  -1413.02866  -1182.57675  -1173.71912  -1332.70667  -1413.11653  -1165.30783  -1001.96782  -1426.13831  -1269.47563  -1277.96217  -1387.31633  -1295.01897  -1559.14794  -1440.36451     -1657.63  -1759.77773  -1734.06442  -1503.51144  -1603.17925   -1243.6264  -1370.63896  -1372.42598  -1819.82389  -1359.65891   -1554.3424  -1511.47387  -1995.14071  -1460.30049  -1420.15329   -1983.7185  -1987.42915  -1681.21867  -1960.11644  -2077.66179  -1392.96725  -1755.29304  -2015.94389  -1672.12082  -1726.90354  -2042.93721  -1506.21151  -1691.59501  -1620.81224  -2331.01163  -2227.82535  -1978.77972  -1800.30267  -2270.03616  -2367.13554   -2286.5591  -2452.80613  -2169.70699  -2321.98304  -3132.46901          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       20.00   -495.82978   -708.58845   -521.23647   -198.81026   -372.27838   -365.85116   -181.91977   -188.73579   -346.25585    -138.8294   -496.87515    -172.2023   -457.79831   -211.73805    -383.6422   -275.55925    -355.9531   -132.05189   -237.65588    -60.12725   -344.81665   -308.97216   -232.07112   -248.19429     355.3637   1157.05825   3042.36928     3844.426   2791.08618   1182.30768     73.55694   -338.69405   -180.76195    -225.0767   -349.50583   -246.95722   -127.66412   -257.65283   -302.03314   -306.57304   -333.41566   -261.75306   -319.27486   -380.80465   -309.48635   -527.85845   -353.59306    -351.7583   -515.64928   -483.81823   -305.32929     -413.207   -218.45522   -347.03998   -302.67087    -309.6188   -265.09477   -177.42362   -182.71865      89.6588   -234.10811   -315.93504   -244.63852   -327.44434    -75.27247    -367.3251   -354.80892   -226.17875    -85.40784    471.93131    763.42896   1806.76172   2269.49111   1786.96512    959.23235    408.89271     51.63837   -175.43938    -232.8807   -344.73327   -113.81837   -109.75923   -407.61367   -210.82234   -223.55565   -380.97399   -335.23561   -275.65719     10.17852    818.29993   1914.50166   2890.56924   2801.51644   1819.98662    766.88806    142.72938     32.72136   -244.81939   -171.60792   -190.08667    107.05426    123.16015    136.59205       8.6414    150.77352   -338.11856   -341.22718   -335.46573   -349.43243    -310.5326   -300.90127   -244.01804   -308.51374   -440.70733   -315.53163   -363.77363   -340.64237   -199.54298    -53.33993   -155.19772   -208.45597   -229.12705   -205.58832      0.11921    297.54922     730.8146   1005.56178    794.95753    660.36993    241.46783     72.13614   -104.14558   -195.35801   -198.90711      6.59163     -12.2629    365.23824   1290.76494   2179.08985   2714.06239   2800.69908   2449.51623   1478.74543    871.01885     476.4904    166.59187    161.05749    168.91711     45.67677    -14.61193   -181.89964   -218.92232   -263.92951   -276.66384   -330.89006   -197.99504   -331.25802   -307.59871   -349.06962   -254.34953   -347.35938   -270.42653    -67.27903    -99.02699   -111.07201     -66.3403   -124.26582      49.5725     35.65299    -80.97106   -358.38922   -361.25811   -257.86993   -146.60402     44.95139   -121.25154    -54.02507    330.11412     65.58065     268.8904    264.37098    -25.91324    -48.01411   -128.63612    -54.46601   -376.91426   -220.46884    -290.2175   -250.63301   -206.81027   -248.21226    -278.3503   -253.62839   -192.82028   -187.15647     -143.151    -55.60596     -73.3411    235.79198     58.06395     286.5788    233.37952    -53.77823     25.26387    -34.99268   -111.68076     -2.09329    -86.42034     -9.32969    -24.61899     48.38895     25.56915    -60.38093   -110.19797    -71.25132   -254.74106   -191.83486    -62.37526    -201.7252   -337.34725   -372.28204   -346.77744   -231.84857   -239.28484   -196.32057   -145.91416   -244.42692     16.04272     73.70931     27.43769    -28.88295    114.66658    102.02382     78.08087    130.83473     145.2726    181.06181    143.96305    313.99555    464.96586    285.71455    306.89514     37.42416   -122.38826    -57.92772    -95.31502    -231.9183   -277.16418   -132.65486   -291.31608   -228.92409    -134.8148    -76.13648   -108.76984   -171.08541     -63.6301     32.16544    -66.53926    112.11283    223.19422    249.48115    130.66053    277.82822    429.41951    633.93461    775.63151    814.16065    503.36402    501.61632    371.63481    588.84673    313.77489     -2.02562   -101.49994    -79.41233   -324.44366   -311.41148    -186.4628   -322.62265     -9.60307     51.90028    217.20062    -49.68237    -17.09204     31.62492       9.9999    -85.28894    -55.68104   -223.59496    -137.2549    -97.99646   -171.26079   -298.58651    -382.0738    -392.0248   -312.37755   -351.05595   -445.53228    -431.7828   -459.69954    -274.8734   -335.01424   -128.78936   -313.57926    -410.9013   -218.71258   -169.99418   -136.39982   -365.53533   -276.43051   -172.19055    -54.47671   -189.49391     83.17985    -67.17035    -98.94841    -213.3873    -52.16001   -270.82143   -223.16814    -185.7528   -198.63761   -245.65124     -296.878   -231.07025   -256.90382   -326.66936   -297.31817   -376.80309   -268.72789   -252.93012   -172.49173   -436.59401   -311.38397   -134.80927   -251.77925   -140.02721   -338.26526   -333.74133   -153.68854   -253.45913   -190.80084   -237.29799   -406.89643   -509.45315   -355.65727   -328.51174   -445.53793   -486.58446   -404.29627   -386.82617   -373.91927   -168.30886   -275.77239   -380.92407   -125.68764    -400.4879   -208.03154   -117.84202   -168.97985    -68.50099   -206.06863   -296.80309    -268.3943   -110.21198   -282.99867   -363.01729    -368.5301   -244.64194   -452.49776   -258.31865   -434.48739   -432.47994   -451.04955   -726.82059   -401.01969   -578.53755   -475.82458   -408.78881   -380.75753   -406.00144   -466.98986   -606.72257    -552.3387   -513.58631   -668.04649    -698.8199   -785.35257   -690.08134   -704.28708   -575.27995   -886.30943   -848.51156   -713.57373   -572.77291   -865.53403   -708.77692   -838.21459   -893.28725    -913.6882   -786.15769   -862.73932   -777.73089   -742.91281    -855.0102  -1099.12807   -922.10075  -1030.93063   -731.88754   -992.86026          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan  -1801.73109  -1650.46389  -1709.98777  -1666.33724  -2017.98055  -1799.84766  -1942.57894  -2257.40987  -2139.38867  -1771.49874  -1614.77768  -1938.48923  -2096.37927  -2273.32961   -2319.4738  -1799.08368  -1957.38504  -1519.30898  -2162.36569  -2310.57265  -2441.33877  -2619.70733   -2332.0078  -2762.30346  -2949.10869  -2472.09412  -1665.79914  -2931.56647   -1790.7944  -2293.29215  -2384.88732  -2923.84884  -2514.15688  -3355.04524   -1974.2483    -2823.397          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       20.50   -486.51302   -650.51951   -576.03661   -214.93172    -435.7111   -370.42632   -221.32457   -178.89665   -343.12228   -221.17276   -488.31025   -122.27491   -542.05913   -273.24279   -400.46967   -338.01858   -325.37399      5.79637   -173.24994    -90.54588   -408.37545   -567.71895   -449.75127   -170.92531    340.90848   1088.00071   3027.63853   3753.48967   2807.12081   1231.95383    152.80896   -446.03663   -245.70584   -138.69581   -302.87703   -280.10186   -266.37782     -44.7496   -326.52858   -173.97476   -247.26919   -347.21733   -265.36986   -271.87717   -382.28546   -378.18166   -465.98934   -421.23512   -387.41993   -404.81236   -255.19669   -402.70542   -327.76895   -286.75161   -359.76053   -299.07971   -269.49556    -53.99322   -174.62604   -148.51045   -387.57908   -279.48723   -265.95727   -262.46694   -234.22993   -376.03502     -70.3527   -270.79502   -132.10894    178.09759    865.30281   1757.22271    2124.9883   1968.30826   1024.96582    352.80103    141.43399   -103.14609   -367.95827   -260.07659    -125.6451   -245.67039   -324.06055   -345.24794    -328.3999   -229.85513   -229.29901    -12.89823    245.23131    835.64963   1807.47907   2637.60353   2645.90843   2071.05318    876.52805     55.99812     -12.0613   -255.01583   -107.70039   -304.13675    -40.20764    164.47542    274.65207     165.2535    -98.40995   -384.40099   -256.81231   -194.10996   -343.62804   -215.50328   -179.80453   -410.42104   -271.88294   -262.42444   -256.78732   -181.66353   -209.94103   -160.36457   -204.33287   -205.96832   -376.80002   -225.06992    -10.30257    129.67545    560.64594     693.5867    944.69107    893.44482    592.92361    349.73574     75.95173   -156.80692   -177.66427   -196.09996    -90.75911    -99.18312    447.88801   1209.52057   2018.95267   2705.38511   2709.21139    2124.3411   1326.42925    835.75397    380.00746    247.89382    343.07658    108.03258    -55.78439     42.30933    -91.32244   -253.68085   -180.91679     -326.551   -245.54979    -70.23468   -274.36144   -344.03154   -225.18663   -283.04724   -277.85867   -218.99829    -256.2258   -135.49977    -62.75943    -23.26855     84.84513     26.46337   -126.16671    -23.70486   -236.49815    -59.35493   -125.56218   -193.16063   -274.33855    -81.31614    275.68067     288.9807     71.51365    198.99092      19.5198   -319.30003    -122.5204    -95.64945    -78.51272   -179.60013    -129.1241   -283.00633   -316.86548   -219.37832   -269.60981   -303.86928   -193.42255   -152.55602    -87.08068    -91.66723     -34.6317     81.24922    257.98137    192.91511    237.48185     255.7446     67.64844     38.90755    -94.60179    -40.86549    -145.0131    -33.19518     95.15657     32.67433    -21.35988     71.06001    116.62647    -47.06967   -133.20321   -342.70531   -261.44689   -270.19712   -324.64865   -257.69695   -273.09955   -245.34188   -357.45659   -346.10981   -214.22704   -177.14309   -118.66072   -103.59107    -129.3087     79.41976     55.32748     78.18125      36.2753    121.24098     78.16867     10.72919     -2.27932    186.00155    327.02835     32.88757    335.00831    -17.36536     61.96051     24.97715    -55.49735   -155.75875     73.92066     13.88046   -257.11325   -274.70994   -192.91104   -272.81216    -251.6616    -33.57887    -66.09076       5.4178     -5.72318      11.0384     49.30769    -37.63098    225.06361    159.52489    217.12891    559.91384     564.9093    707.15989    604.48333    612.01951    454.57221    355.86789    339.86386    169.79941    177.28867   -176.60394    -26.45965   -273.96463   -126.32547   -143.07141   -232.87883    -90.24997     31.09369    -72.63102    -41.69628   -343.22581     74.65469    -11.61868     -98.4632   -170.87226   -137.64319   -164.92759   -214.62207   -201.06472   -186.09291   -416.36033   -413.64269   -299.53363   -393.13326    -82.52588   -420.71739   -415.74155   -456.27602   -280.72118   -317.89932   -463.42073   -293.23289    -295.4478   -296.58089   -405.16263   -168.92594   -277.85144   -260.55523   -248.06957   -219.55806   -284.50062   -187.36826   -120.44806   -116.62187   -171.79238   -287.56554   -315.12344   -310.48495    -97.53715   -244.75996   -391.80675   -471.74668   -258.33718    -231.4562   -382.84433     -360.889   -561.37997   -345.45567   -511.27705   -426.99322   -343.88807   -316.99007   -412.29831   -272.85145   -270.05909   -386.38507   -538.65459    -189.8768   -359.76472   -389.98831   -317.01691   -296.58816   -313.38739   -378.23174   -469.96434   -671.71381   -492.73909   -618.34374   -500.05947   -420.41716   -370.54106   -420.07984   -349.64036   -522.66422   -289.89861   -688.34273   -326.39292   -496.05339   -558.84248   -453.24894   -456.87741   -726.66693          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       21.00   -321.90699   -520.97063   -548.79113   -292.62975    -369.6547   -326.65811   -337.11864   -293.68402   -271.68336   -311.81305   -519.51902   -219.34835   -630.46721    -525.3451   -195.27225   -349.93959   -286.27771   -116.51264    -129.7856   -155.15657   -351.09215   -459.36087   -431.53323    -22.36046    146.32167   1277.74744   2945.33475   3720.53305   2735.29515   1259.57337    359.24512   -478.35345   -224.29244   -148.31535   -301.80912   -163.91152   -459.59707   -246.64176   -456.26058    -205.0661   -229.62945   -378.24126   -242.43687   -529.13405   -360.65441    -367.5072   -527.29175   -357.68754   -460.74424   -325.24798   -487.46297   -386.68495   -299.33899    -245.6894   -379.83812   -231.22825   -142.30506    -26.30305   -359.32372   -158.48346   -295.00597   -298.17961    -250.3062   -342.39054   -412.76467   -270.52028   -189.26243   -209.29554    -83.65481    132.58608    961.81944   1834.08037   2103.11701   1678.14784    824.97236    285.70991     -7.40748     -4.44676   -387.21883    -238.4224   -395.00458   -265.40656   -303.83546   -196.33795   -285.47107   -192.28815   -172.45367   -114.68576    218.85199    740.26033   1679.63045   2663.74438   2679.40033   1918.20846    946.61024    118.04341   -169.47873   -232.28092   -285.18533   -154.91624     20.19048    186.73305    215.72874    158.62217     -8.58774    -69.84264   -266.88586   -266.41406   -371.66864   -239.45353   -288.42191   -495.07965   -391.45837   -345.29192   -268.78948   -221.06442   -327.00967    -69.66576   -166.70558   -176.19561   -239.95272   -346.33578     -5.57108    112.36279    405.03788    939.09332   1009.04542    772.48908    716.19053    286.82875     52.95902   -101.79472   -147.82709    -108.8065    -55.78193     30.70412    517.91995   1126.43235   2180.90214   2445.43847   2757.27104   2096.75535   1390.67769    878.92156    442.54188    240.93125    190.51532    295.28291    140.44083    246.77454    -21.71095   -260.90819   -173.59012   -316.33526    -440.0329   -111.55835    -75.44782   -200.01089   -190.19114   -323.00212   -193.78418   -452.03066   -364.45493    -199.5071   -105.35428    -29.24698     -65.2414      12.2748   -149.16687   -161.83383   -106.68977   -188.37235   -201.74148   -229.30276   -113.59341   -139.36832    228.04686     34.41599    313.70656    247.40382     72.39458   -123.56242     43.05931    -74.25682   -296.95841     -8.78991    -371.4711   -171.88719   -340.87197   -304.42936   -262.62408   -171.74843   -236.23892   -145.82645   -130.05878   -140.67325    151.40309     -27.1993    170.89978    238.51379     94.45289    184.14708    176.71292     33.06733       0.7365     58.02678     52.57731     73.92086    121.12074    101.59687    144.41721    216.38432     160.7582   -113.17572   -105.59519   -154.44499   -245.34841   -276.73451   -321.85395   -271.60105   -369.29052   -146.16668   -265.52036   -257.63588   -128.44671   -105.75299    -30.46457    -70.05206     47.60705     14.04925      3.71544    174.99235     98.69846     20.48465    135.67541    133.15202     96.13098    213.57012     197.7777    215.48919    158.81781     64.59201     23.85188    -17.15048   -110.48275   -386.50659    -368.5081    -242.4708   -352.47563    -207.0718   -301.33677   -236.97912   -130.99363   -121.55129    -97.21405   -127.94792    -44.68378    236.94172    166.12344    187.92593     238.6375     250.0586    257.77954    390.85301    530.89138    530.40616    496.96468    547.18856    548.28886      527.169    252.43154    144.92471    174.59595     27.96946      -61.854   -186.55291   -136.23185      7.16402   -185.65335      71.7787   -127.14936     51.80974     92.07303    -99.30357     112.6546    123.06281    -99.65908     11.10238     -172.054   -253.75484   -166.17106   -276.79941    -282.1923   -362.40728   -388.61496   -282.82728   -268.22838   -269.97038   -380.09078   -499.60733   -413.42544   -330.69793    -305.7707   -392.52145   -262.29941   -286.18682   -321.41458   -347.85302   -149.51357   -278.97772   -284.56013   -109.52592    -207.4724   -134.24375    -76.51906   -216.17812   -192.48294   -395.64877   -180.49231    -232.1162   -190.17503   -228.36227   -285.63459   -442.45833   -396.75791   -583.14924   -510.86179    -484.5306   -474.57485   -364.36231   -477.59432   -469.12762   -342.68526   -457.69608   -385.42924   -578.77403   -491.15014   -413.96805   -550.94247   -610.68856   -589.83602          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       21.50   -132.91596   -413.37211    -490.9023   -425.56894     -313.356   -201.78987   -351.97145   -426.32921     -57.6056   -258.75763     -552.043   -379.53113    -454.0665   -424.50053   -122.74039   -364.04078   -304.70885   -265.83329    -38.87487   -468.76395   -362.11379   -338.47427    -294.2061    -68.98605    325.28017   1317.26884   2600.46229   4008.93072    2630.2376    1081.4542     36.05139   -281.04741   -222.46797    -276.8983   -264.46307   -225.47305   -347.28816   -437.87963   -293.42874   -402.79114   -369.26017   -404.92186   -315.58279   -476.27104   -144.64842   -437.67694   -472.75249   -430.96689   -559.93494   -404.65922   -434.45305   -483.27346   -285.17171   -285.37692   -379.84857    -98.70386   -349.14054    -76.22042     10.50586   -158.75921   -164.94886    -375.6989   -396.55538   -209.22077   -328.22022   -309.52256    -132.6537   -246.53534    -203.5636    105.34586    807.35026   1914.76865   2198.67677   1813.87776    970.11372    385.97686    274.62147    -53.47052   -192.30362   -148.61634   -283.63566    -62.58997   -107.20437   -105.07282    -324.6904   -142.01256    -342.7752   -129.66165    102.82298    538.18805   1723.08568   2771.79404   2600.14435   1845.14459    865.82705    271.27327    -167.0188   -140.39887   -263.38424   -136.94934     -3.79535      97.4645    163.22833    251.22261   -142.81172   -119.88843    -312.8844   -360.27805   -372.60684   -341.88932   -272.17029   -284.56803   -309.94322     -455.424   -223.70045   -236.18488    -96.59262   -169.88798   -231.36872   -181.36169   -132.94507   -265.64366    -77.04647    236.96912      642.146    916.29028   1027.65278    955.94871    639.50467    112.62795    -88.55087    -65.82645   -268.35758    -137.8381    -95.92878    118.52481    543.79365   1128.49319   1808.98193   2382.98473   2268.11651   2321.17648   1764.14251    691.80532    478.16165    200.71317    199.33383     56.47483    143.06749    -28.52714    -63.09329   -189.25364   -341.53719    -192.6908   -285.15863   -410.11016   -343.54443   -219.76717   -225.65394    -93.11107   -213.26798   -166.13466   -181.46552     23.12015     26.15623    -23.82744     40.70737     19.48906     28.46307     -141.424   -237.74434   -373.62731   -192.97939   -263.59225   -184.78801     16.11472    316.50622    185.70784     257.2585    219.99182    151.47606    -22.58751    -55.57327     -79.9605   -169.12624   -204.99915   -431.66874   -307.61879    -131.5361   -318.64894   -262.65515   -227.09678   -205.06168    -259.2432   -136.82082   -124.17198    -53.09955    359.39108     19.74145    269.59365    198.23566    188.68866     51.45389      9.15076     78.15497    -23.77198    111.61842     21.92989     11.65318        3.877     98.07232    155.02191    -27.63822     -5.88911   -237.24797   -270.01211   -254.11921   -377.69913   -381.32584   -339.00223   -243.31852   -308.84758   -315.49661   -295.33129   -152.99115    -74.85003    -92.80667   -165.20626    117.22193    112.97421       7.2432     -2.61223     20.93555     78.74197     64.33563     90.05785    153.75056     91.44481    119.77937    319.57705    186.48322     268.9054     111.8068    127.11755    -32.97056   -136.40523   -290.63328   -231.33423      9.39834   -358.14889   -226.46212   -342.68368   -242.53736   -169.29446    -113.4601     18.46117   -122.66368     -4.61155     46.12707    122.44766     156.0761    284.21261    561.23276    402.61243    421.14076    711.66714    600.01934    512.49022    578.86418    337.96353    363.66468    185.27546     85.90741   -153.35785   -176.25813   -214.78164   -219.73824   -242.58884    -83.83572   -324.69068    -97.23863    -49.75901     19.78693    -72.89301    -95.24074    -90.75232    -65.23641    -97.46372   -213.96143   -277.98438    -400.9365   -313.32143   -421.47629   -342.65532   -472.32774   -381.45707   -407.58323   -360.63286   -259.48137   -408.70337   -479.53352    -434.7372   -306.39355   -486.42414    -281.5316   -239.35574   -368.44602    -229.1774   -296.81474   -242.69849    -326.5669   -376.58389     -40.5926     -248.979   -254.55481   -105.65021    -198.0122   -353.39868   -451.83356   -280.38918      -492.08    -473.8731   -396.75282          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       22.00   -150.67136   -354.58366   -517.22213   -478.09021   -428.34735    -58.17715    -263.5646   -411.05442     -3.62854   -197.01057   -371.97334    -401.1158   -331.32986   -250.06365   -189.80096   -402.01706   -385.68756    -515.4685    -38.89366   -292.14486   -322.45529   -145.42541   -145.27742   -121.41789    505.45592   1243.92627   2896.55249   3821.86049   2701.20382   1067.33383     95.73008   -170.37231    -301.1022   -305.64025   -257.69823   -140.45211   -275.90107   -401.10084   -390.52501   -190.37545   -196.45306   -271.33524   -253.18728   -423.83924   -474.57017   -388.73604   -591.83531    -349.0025   -425.27829   -422.30384   -354.88989   -362.62508    -348.0167   -485.01002   -460.23341   -296.19343    -350.0277   -147.90116   -109.95924   -152.73104   -135.23552   -189.28492   -221.96912   -382.81622   -409.59046   -310.09401   -198.93179   -174.95212   -299.87853     90.87438    886.76967   2012.27297    2198.2005   1781.76456   1033.28058    470.94602     51.43909   -188.39085    -41.71518    -62.90099   -152.79499    -85.61423     50.28034    -88.85205   -290.38338   -212.05307    -62.97125   -209.30827    231.36872    797.76485   1708.37994   2501.42041   2527.28619    1943.1302    989.12574     325.3743    -173.7937   -320.17289   -305.24454   -198.98334     -67.0718    141.80021    145.05765     94.91942   -162.75365   -260.25878   -198.40486   -228.72326   -297.29868   -361.71767   -317.74877   -483.34571   -324.00828   -286.23413   -346.89424   -199.26812   -131.84152   -380.71317   -162.99128   -177.06205   -244.82547   -228.48838     13.49385     228.2695    515.83586    977.07614    923.95405    785.72226    554.95505    233.47464    196.80062    -13.46293    -97.15586    -111.5543   -122.29262      1.05334    588.91761   1038.52665   1777.76113   2318.40786   2577.67941   2092.23125   1383.99125    791.19872    377.82706    166.92082     75.73251     49.91567     24.80741    -81.68742    -48.88969    -312.7062   -244.56165    -292.1811   -309.51753    -216.9385   -216.13505   -278.54535   -165.07826   -227.12361   -428.36338   -158.98417   -257.17052   -227.21819    -85.48893     13.45004     16.53789     78.44444     51.56846   -249.06594   -112.27692   -262.64546   -319.81195   -241.98253   -208.19975     58.55266    268.84856    312.15601     334.5489    185.71492     157.4609   -232.22668    -56.11854    -31.56606     81.90818   -301.07585   -350.11095   -262.33367   -317.16177   -303.53422    -71.09474   -225.73263   -104.31128   -121.07522   -197.51971    -121.0291     61.30477    177.69512    208.81132    247.01895     135.7799    235.38653    153.66422     55.61955      5.69932      8.28089    -16.26485    201.24567     82.65649    -39.58043     10.01644     83.89656    -75.44747   -178.50877   -125.29672    -34.70291   -291.05933   -180.70701   -222.38327   -231.73562   -271.19013   -338.30632   -260.91999   -212.88651    -99.26482   -152.11095   -145.34381    -93.00193     18.93211     131.6746     30.67472    -65.03469     51.63572      54.8402    122.79789     169.1305    185.30642    140.92201    109.89127    310.99086    418.84285    159.34446    -13.62826    163.81491    -77.95287   -127.97272    -90.47415    -228.1997   -223.59077   -345.39006   -257.50768   -199.22378   -229.71537    -62.35092     -26.0389    -99.20622    -41.72652    -37.72615     76.96531    138.57771    128.56382    151.49057     236.2029    316.58714    343.31962     659.8367    448.03708    708.31537     391.1004    326.80189    435.02183    267.91885     40.88107      7.83963   -197.11071   -202.34048    -214.3311    -78.84714    -64.89797     -41.1231    -307.6018    -83.87952     -77.7262     20.56358     -27.6839   -249.07709   -152.71874    -181.2759   -170.86181   -330.83722   -326.36161   -412.87848   -326.20753    -218.7997   -397.17995   -391.05537   -347.74381   -427.00135   -454.78984   -275.32932   -616.86945     -415.875    -266.1983   -501.53693   -461.18074   -495.00992   -338.48128   -493.58805          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       22.50   -258.54129   -369.38371    -579.3408   -446.24672   -570.40831     23.49998   -146.21102   -373.15707   -343.98324   -246.58427   -192.75477   -377.67276   -334.80459   -315.06617   -252.50976    -225.8522   -465.31027   -463.20485   -205.21076   -145.02656   -216.30822   -259.65854     -205.257   -220.28207    280.17596   1415.58339   2972.61622   3697.25237   3208.71387   1121.64589    182.43032   -149.69421   -207.22167   -187.23931   -345.88566   -144.84489   -299.07077   -102.67475   -361.19058   -241.72116   -378.05867   -231.03471    -208.1771   -355.59338   -446.10954   -406.19817   -427.62837   -440.43527   -334.70594   -506.96473   -234.51046   -411.98423   -287.46107   -269.77033   -303.19681   -381.69723   -273.46722    122.20512    -85.40592    -70.05718   -343.95851   -377.09009   -394.81805   -377.04768   -409.23155   -271.62308    -226.3464   -245.76108   -267.98473    136.25964    718.57625   1878.07332   2256.54954   1764.86121   1086.32867    503.87584     13.19878   -162.49388   -317.35467   -178.63367    -415.5661    -293.5839      12.1265     -19.9211    -103.5209   -191.74121   -328.69257   -105.73003    -26.87161    710.94798   1744.14925   2507.84852   2842.57071     1623.363     778.3808    301.45314      -35.607   -182.74253   -161.99497    -277.6726     64.15962      151.387     223.4647     165.2551    -32.47812   -199.08891   -226.42708   -218.95838   -351.87146   -607.96685    -328.7401   -190.71579   -393.45532   -323.11961   -354.53204     -190.417   -179.82098   -137.17831    -61.26314    -97.13123   -211.24119   -270.52237    -126.8917    134.74118    645.82595    930.99856    1132.9105    745.31233    557.16504    267.43461     53.41276     208.7399      -189.85   -193.23185   -183.84816     51.35494    369.68364   1150.12679   1864.43607   2415.62319   2437.68015   2095.37489   1214.22874    812.84825    207.61732      62.4099    286.51804    134.56919     31.04248     85.97076    -96.86694   -156.33472   -229.79508   -201.09598   -347.51207   -291.05611    -323.7407   -317.55522   -244.29939    -131.6157   -165.83471   -165.05934   -283.35636    -48.28055    -64.10694     34.17019     214.7388     122.9067    -29.86232     47.81038   -271.52386   -351.51736   -385.30399   -284.69815    -49.07071     76.62728     -39.3976    211.98843    268.43266    170.74108     -35.2661   -218.04186   -126.18953    -66.16391   -299.83646   -191.61628    -51.22664   -180.82392    -283.4705   -208.26319   -357.63819   -242.82359   -263.83467   -161.14894    -319.1688       9.1213    137.99914    -49.21507     61.65956    114.23504     312.3226    246.09861    158.53632    148.05508    -34.15866    -98.24144       14.458     15.48736    132.16987    171.56353    122.46963     63.40968    -25.56431   -207.23307   -220.06764   -215.44783   -297.67331   -357.54719   -376.80046   -413.85645   -282.36923    -203.6544    -329.4548   -194.69563   -263.64199     52.38879   -164.04426   -146.60789      6.46489    -84.75906     21.41854      50.1881    104.11063    103.10646    107.24591     86.87247    101.19866    205.78435    187.01269    254.19808     72.39931     82.99611    292.08258    -16.70959   -100.55505   -290.95877   -248.03023   -347.73834   -269.83427   -266.13764   -381.75174   -198.24371   -254.61063   -260.65962   -149.67889   -170.95734   -107.37571    -44.60509     38.93505    -76.14915    171.10375    209.00378    346.62781    152.18298    393.53062    619.29989    498.45214    605.28374    499.82673    448.10301    318.02772    222.33576    -17.93301     73.65152   -234.60435    -144.4938   -219.41186    -218.9888   -146.16848    -66.37571    -67.13208     -1.63045   -100.76586   -207.16612      3.32441     -82.3862    -79.08571   -309.86654   -251.37291    -294.2196   -320.74264   -289.44935   -343.52847   -404.91963   -575.84003   -472.26291          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       23.00   -328.31746   -338.42774    -500.0522   -354.48917     -628.622     22.60237    -200.4604   -327.23126   -543.99367   -284.49033   -208.09638   -215.17057   -428.68866   -426.66564   -294.44396    -192.0364     -528.152   -417.36458   -371.60763    -65.01894   -305.00032   -287.43217   -298.82343   -250.95629     60.41002   1536.35904   3032.28158   3800.31982   2879.56096   1009.58844    211.83872   -120.07989   -434.42896   -174.97894   -289.14618   -315.10449   -272.52741   -288.68793   -335.77301   -270.31659    -43.76736   -428.13375   -335.34122   -294.65804   -355.97589   -340.29389   -280.35879   -450.30069   -301.29848    -212.0774   -335.67949   -403.54717   -377.08086   -429.05833   -348.85526   -412.31697   -311.23997   -113.10581    -47.33791   -224.33034   -338.90298   -225.76177   -411.47293   -389.98794    -280.2752   -294.54807   -324.80905   -262.37173   -130.04114    -78.92847    847.36042   1822.25462   2396.36888   1773.74803   1035.39099    268.11602   -126.28682   -275.67688   -375.99612   -239.24618   -250.53679   -261.51329   -261.60994   -372.78368   -129.72017   -185.36382   -165.84645   -282.78718     78.63386    637.53828   1793.15548   2646.76687   2552.77153   2192.83928    886.79199     213.4719   -153.66197    -73.26802   -218.72459   -150.67205    110.59963     62.39442    272.77928    113.46827     77.99183    -97.57836   -211.94294   -309.01833   -195.63378   -588.29959   -406.96391   -268.10638   -239.51849   -318.91727   -349.95253   -276.56668   -212.87563   -183.70519   -169.69431   -146.27576   -227.83054   -169.91454       2.1122     72.03228    362.29266    856.75956   1080.88444    876.90964    420.11723    193.36116     75.06125     12.53303   -139.62454   -161.03333   -210.85684      54.4469    526.54745   1021.36948   1925.18164    2439.5605   2449.18165   2207.94674    1411.2106     771.8157    246.01081    170.23811     41.27412     80.87659     40.55563     39.21324     -98.2396    -154.7337   -176.43468   -294.23379   -189.33471   -336.40402   -430.14043   -340.05874   -161.85148    -88.97043   -263.29657   -324.59064    -212.1473   -144.89708     76.28681     25.26704     43.14122     -1.10814     43.37832   -157.78725   -318.18512   -393.28617   -249.64505   -196.11049    -198.2371    -95.05052     140.2236    189.39782    219.81069     -3.84282     73.79377     95.35056   -276.38479    -51.07518   -112.56487   -170.84624   -251.26608   -410.95746   -268.87578   -229.92261    -141.1989   -356.99355   -216.95641   -248.21401   -173.04564    -26.19287   -128.64258    -99.23832    107.08843     81.27829    255.98275    205.42687    135.16718    -14.61298     36.18296    109.82793    -63.03433    -12.83977     38.23497    220.87466      74.6335     62.80899      0.61382      35.8579   -286.38294   -110.30431   -126.41961   -196.97248   -341.74452   -359.07106   -342.09547   -281.67837   -315.82692   -106.91234   -231.83432   -256.16303    -10.17467    -24.22203     70.47081     82.31554     60.49999    -51.29899     58.15918     -3.29054      45.2113     68.93326    110.52833    158.75627    250.06041     214.7031    220.63508    261.32002    112.31053     13.77585   -157.71326   -208.16867   -182.72285   -226.74353    -242.0546   -246.63456   -163.32068   -256.77276   -239.42251   -270.21899   -154.13796   -174.77805    -23.93864    -90.32198    103.26772     95.59455     88.54934    270.46479    372.52841    234.00155    553.72199    615.04175     388.3487    429.95668    430.51645    281.40095    285.21692    110.51932     11.39498    -24.28252   -108.23153   -147.58753   -249.25323   -369.82901   -323.19198   -119.98907   -189.18555   -247.36864   -391.24676   -177.03645   -372.68099          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       23.50   -286.00562   -309.73115   -384.84396   -301.01403   -593.96511    -59.71864   -381.61931   -285.27876   -396.65875   -227.31738   -274.78785   -141.74273   -581.52517   -327.02398   -351.82305   -117.16078   -512.14175   -462.66145    -183.2493    -84.68471   -366.83741    -72.82731    -429.8735   -142.58775     86.24867   1419.22772   2779.62593   3519.10234   2867.49631   1173.25013     20.84693   -156.81146   -400.02359   -342.06767   -190.47101    -54.56942   -227.53204   -457.61113   -272.84112   -296.06353   -392.05251   -249.80704   -419.09696   -434.47424   -321.75358   -368.14334   -189.59236   -426.27238    -386.1941   -316.04367   -376.90266   -349.80763   -369.16089   -397.50869   -271.48506    -282.6047   -181.28678   -264.92645   -209.96011    -89.54857   -243.81338   -281.61268    -386.2397   -243.29358   -504.97712   -415.36759   -193.77926    -223.8451   -108.18806     36.08075    948.38936   1575.18718   2445.91461   1814.06066    1041.0402     287.8083     207.1953   -257.26694   -314.19148   -392.32395    -69.79285   -173.57276   -278.19371   -317.48671   -124.15214   -126.39299   -120.52449    -58.50988    101.93469     719.4362   1937.99734   2517.86724   2624.40198    1853.0879    820.41284     71.45792   -149.74701    -59.22953   -191.64812   -137.60563    -15.80446    195.80948    232.54759    146.58073     10.89476   -375.25011   -245.73775   -290.85332   -330.80714   -328.53885   -334.30241    -228.6886   -514.20829   -264.60551   -402.72479   -285.96151   -187.99993   -102.18327   -173.91054   -432.75757   -357.60829   -281.99346   -122.23268     71.95121    504.69211     736.8071    870.11155    530.80298    409.62733     222.7073     32.56955    -89.02608    -43.52356   -165.31486   -145.89916    112.51767    371.94952    887.72119   2010.88347   2463.67821   2566.59713   2262.73672   1437.53477    687.86285    417.33576    208.65647    239.37992    265.51729     45.77692    124.90352    -84.38277   -147.82155   -174.91366   -268.49042   -230.82409   -160.92686   -237.42752   -355.39092   -325.66154    -99.15617   -228.35575   -255.41386     10.82729   -131.53605    -50.86419   -125.84622    -51.09887     66.05147   -297.18026   -118.96105   -155.26773   -233.10838   -125.39254   -154.39276    -14.28704     79.67754    162.31565    120.02731    291.24481     50.71622     98.06509    -57.48843    -55.46387   -194.95132    -29.61787   -203.49002   -308.44136    -305.2323   -328.97478   -273.63298   -266.01919   -318.34053   -147.09511   -117.94187   -159.15715   -264.27921      -6.4326      60.9764    -42.34691    144.94793    281.90351    266.63029     55.15957     22.18196     28.25021      12.4391    -47.68477     77.22593     48.35873    206.74634     102.3805    198.39962    -33.38072    -41.62317    -195.8223   -232.09821   -231.65955   -306.25556   -279.83481   -287.23579   -405.13707   -244.29155   -230.76306     -328.847   -223.09621   -181.50088    -19.11668      3.26699     31.24755    -37.68264    149.11509      3.44055     30.68749     35.24726     40.05721     37.60601    109.33076     166.0768    157.65999    266.78162    299.07822    253.61674     92.77435   -105.59448   -120.40252    -213.8255   -261.69784   -365.95333     -324.829   -211.47534   -315.84056   -321.19611   -212.73783   -212.26698   -306.01517   -151.98023    -38.94909     -7.42827     12.44632    -53.21567     80.52596    -51.65696    450.75095    282.86411    338.49413    519.25228    430.98384    313.84032    262.03034     161.5234    131.96824    -78.67176   -102.12603   -295.69999     -408.225          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       24.00   -171.68916   -322.80464   -296.93128   -335.93433   -566.85155   -139.17935   -402.19227   -348.69144   -431.85069   -265.08755   -266.25414   -213.95286   -510.75363   -262.46895    -417.4978   -167.94762   -279.28401    -367.0901   -103.95366   -256.32887   -219.73164   -175.58231   -503.84094     -26.0971    365.12618   1252.66486   3051.25503   3495.95548   2877.74274   1330.53181    -41.98958    -302.8148    -283.9493   -226.57621   -514.10069   -384.53673   -204.40893   -330.50966   -293.62473   -345.39882   -218.11588   -398.13004   -291.55188   -368.25939   -362.39043   -377.55883   -343.03298   -469.49935   -404.69885   -195.28612    -153.8261   -265.96935   -338.03766   -382.25811   -229.26396   -417.12585   -304.38251    -135.0159   -100.22885   -122.05878   -343.57154    -37.40415   -389.05987   -295.22283   -271.25075   -232.12451   -386.97794    -71.98152   -242.37135    120.28439    939.42282   1918.70388   2307.25455   1694.38601    962.32592    276.61454     68.29615    -62.70752   -129.80813     24.24034   -266.17293   -112.49061   -280.24633   -180.16208     -19.3013   -258.79109   -342.95868     19.83773    205.80693    868.23863   1837.43046     2593.673    2566.4894   1786.10363    891.55131    229.29311   -162.96664   -221.43157   -223.61741   -131.26617     71.53694     66.10841     14.88178    149.49383   -167.00528   -187.36899   -274.27468   -282.36198   -366.31224   -337.76399   -258.83919   -267.93892   -250.74215    -429.1182   -291.18641   -269.98757    -321.8584    -108.2202   -171.61223   -203.89705   -152.15205   -145.62436   -243.40367    100.53865     467.3548    687.23501    842.09947    605.51184    561.38771    227.17057      7.94056   -170.94569   -281.57381   -197.61202    -83.07901      16.4929    499.07901    1052.5332   1734.91213   2515.45207    2558.2904    2132.9449   1628.52626    751.90421    385.63464    306.49034    -13.72472    199.94247      57.2691   -136.34474   -121.62664   -197.45717     -324.428   -300.24305   -309.33478    -84.07126   -443.85806   -279.86769   -358.05006   -292.23601   -158.18542   -208.54736   -229.04695    -53.82835    -80.34335    -51.33626     81.01797    -53.80163     72.73975   -169.41913   -179.61652   -240.89147   -235.75486   -120.39517      88.5552     19.72096    243.14668    307.01542    205.57552     54.28483    -10.23926    153.60783     -71.0595   -166.07216    -37.58209     42.93534   -167.72423   -107.16151   -170.08052   -242.30985   -382.16843   -305.27741   -252.67408   -250.98613   -182.89644   -124.27172    -91.31835     86.80865    252.17168    180.01281    236.12936    307.46215    201.05399     48.76048     59.26637     -9.76164    -40.82039       0.9013     69.33562     178.1739     64.90451     57.21154     12.37464     -59.7067    -75.04385   -164.01491   -257.07155   -207.45076   -337.76336   -230.36252   -212.31205    -293.0493   -232.36208   -363.19851   -341.45016   -199.43354   -174.12328    -55.84247    -91.64131     80.94344    228.14603      2.99053    135.83809    119.54703     -72.6007    169.16679    231.46307    256.44109    223.64751    244.94539    149.36381    157.42648    101.06516    -33.12937    -88.24652     -44.2958   -160.88527   -299.96238    -271.4875   -233.08559   -350.73789   -282.53972   -265.47191   -256.51021   -317.92893    -32.90856   -135.26379   -209.19268   -125.39074     80.71256      8.96391     51.78602    149.09031    -15.53448     63.59705     75.75726          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       24.50    -66.72831   -310.09481    -324.9887   -502.78147   -472.98964   -172.35521   -382.26684   -371.99528   -566.03881   -346.61221   -300.06522   -282.58337   -522.62898   -283.17324   -252.77296   -244.23544   -114.87856    -247.2955    -248.1558   -350.45412   -151.24557   -237.86447   -381.22853   -200.88458    297.01503   1297.27287   2975.32104   3608.91308   2669.88157   1129.44799    122.51259   -365.42197   -238.28421   -110.99838   -241.16757   -389.26367   -151.19174   -223.27875   -335.55005   -591.43279    -284.3784   -352.89051   -247.22917   -342.15406   -355.42818   -387.87254   -505.88358    -447.2575   -427.82139   -383.25245   -485.03765   -217.09354   -337.42013   -442.84362   -245.73172   -381.09392   -351.87089   -160.03383   -222.43051   -168.52489   -350.10052   -198.35001   -337.30992   -369.24979   -271.01146   -430.64277   -210.37496   -315.09675   -272.46749      3.81757    858.82603   1930.87567   2054.30332   1898.09814    953.14802    332.34815    219.81098    -30.87177   -194.83675   -139.80928   -310.65207   -355.35712   -106.03726   -287.56355   -157.99201   -244.37766   -214.18301   -170.20497    175.48514    889.97723   1913.72592    2827.4463   2520.10703   1743.99193     903.8105     170.0793    -31.33973    -90.10612   -239.49894   -168.62135    -74.14133    101.74901    331.45998    160.81272    -12.41942    -10.30719    -235.3679   -221.90808   -249.92595   -342.90507   -198.80067   -270.51582   -394.53647   -267.83355   -408.52406   -436.62342   -227.33025   -173.12797    -251.8369   -213.84918   -232.03882   -121.38045   -179.45263     92.85366    369.81455    529.36447    969.65537    651.95706    514.48441      102.875   -118.01485   -190.43204   -128.83277   -213.29384    -65.76407    167.54491    516.25938   1168.89113   1945.89218   2383.55216   2558.38653   2121.87875   1396.37596    868.00646    536.65871     90.53406    247.90223    -66.76766     88.35225    -55.72495   -108.96314   -225.16851   -329.98619   -267.19992   -293.87267   -296.98566   -322.43698   -298.10209   -138.90051    -138.9573   -306.52754   -230.94222    -19.35158   -149.63283   -167.01178     -5.61033     46.00064    -99.33682      6.23768   -127.16082    -200.2288    -84.05016   -207.09743   -231.92448       9.2753     56.00675     70.17142    170.87536    370.72319      4.49032    -36.45941     -6.06006   -108.47464     -55.8329   -186.95663       -193.8   -193.26631   -191.96733   -241.53912   -362.20479   -254.66044   -358.19262   -187.21174   -320.25759   -261.98842    -89.18384     19.08189    196.88796     79.98177    349.92105     20.98412    245.95139    147.38122     35.19768      31.3089      4.00634    -15.22164    -50.15196      43.4548     87.99916     151.7332    -56.79834     93.99969    -55.04842   -193.83631   -135.61242   -322.15584    -238.0233   -264.56228   -390.31517   -433.11464   -277.22666   -355.59663   -338.29789   -326.39093   -298.43079   -263.19334   -148.31887   -143.11899     64.20114    156.09357     22.72888     65.02419    -64.04042     76.05439    172.38901    112.22823    132.30587     19.99507    217.39268    131.38482    -70.69849    -33.36163     -127.076     -9.52743   -167.60349   -264.45856   -304.10409   -214.49211   -251.58108    -377.0642   -354.90839   -280.39858   -260.85138   -262.16521   -381.06449    -388.0663          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       25.00    -24.69421   -268.26327   -396.58818   -653.39143   -283.46106   -278.24913   -313.33548   -302.73493   -506.08053   -323.53494   -393.07324   -283.98568   -523.90566   -236.86737    -35.50806   -240.01775   -132.16824   -254.90251   -371.71852   -322.01285    -68.87086   -158.01354    -205.8479   -283.24376    200.02675   1202.41905   2753.11677   3903.20175   2439.91471   1057.03545    -19.58063   -283.13018   -152.83992   -391.75244   -414.62126   -330.25149   -215.80508   -182.78285   -371.04117   -412.97046   -553.06199   -421.53782   -324.95718   -510.16736   -330.92273   -483.80947   -382.11152    -377.3425   -354.80552   -354.92588   -281.81154    -344.8394    -407.5988   -459.39498   -328.31346   -305.54989    -423.3352     53.98703     23.46258   -252.03123   -359.62161    -409.7859   -512.73224   -415.75446   -325.17023   -429.70733   -295.62029   -190.71896    -59.69969    113.14038    986.27878     1804.337   1965.56269   1626.79292    915.91545    319.18229     35.68455   -176.22858   -227.72881    -72.26155   -269.24795   -203.52212   -151.57827   -187.14241   -341.77881   -134.14183   -269.05052   -148.51341    -35.10715    894.89929   1904.62679   2394.85041   2521.17549    2001.7146    734.34341    144.01456   -112.28578   -279.79073   -221.66253    -23.67258   -152.02277    142.51349    110.69815     51.24975    -38.97515   -270.42415   -305.65331   -145.08028   -293.47314   -316.47755   -477.66308   -352.16731   -326.56698   -261.45264   -329.56134   -219.30425   -266.10877   -184.33627   -191.62571   -261.84474   -197.13583   -248.79805   -152.86592    245.43444     456.7814    768.61354    682.85597    745.22692    575.56446     50.20945     34.67398    -13.99208   -151.84099    -64.50877    -18.18106     22.89952    444.62272   1137.24849   1833.63507    2401.9619    2402.5994    2009.2419   1379.17587    884.19861    345.05596     30.67485     312.5519    121.06566     65.58459   -110.09881   -180.84613   -146.42678   -287.75949   -363.51162    -409.4629   -190.04667    -301.1325   -179.39201   -287.34901   -358.17926   -351.62513   -183.78365   -324.54995   -184.38699   -134.32507    -34.60031    -29.43246     95.36477   -118.75382     27.28044   -247.54665   -175.28482   -182.13312   -217.24498   -222.90776      32.4366      1.81349    105.91564    104.44305     72.48271     51.77944     -58.1416   -121.85985    -90.26725   -170.21755   -210.90277   -300.90863   -203.70635   -270.40274   -199.92343   -184.06031    -290.9929   -228.54793   -196.98265   -140.44943   -156.20308     -90.3895    126.40037    139.21158     221.8424    177.98625    398.46011    163.51647     42.05208    -63.25535    -25.94748   -185.21233     28.22409    105.47894     76.34052     134.2061    -14.33189     32.37277    -27.97479   -287.18582   -212.87883   -347.27966   -274.07946   -237.71201   -521.57132    -309.7686   -255.49769   -370.56449   -258.08713   -350.74731   -111.73032   -219.65794   -100.91024     42.26959   -158.68605    107.34318     -3.67221    -44.89129    -43.13655     81.70052      -4.9242     31.59313    125.17214    105.92026    234.89768     67.57372     81.82249     32.13752   -199.71989   -203.19704   -223.26367   -298.10919   -352.76065   -489.22132   -491.31313          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       25.50    -67.50737   -224.61266   -416.52889   -607.72833    -79.03636   -388.35315   -263.12853   -283.38062   -385.00046   -341.16641   -387.29632   -234.85799   -307.62296   -261.23929   -229.10599    -237.4036    -64.03859   -165.43574   -369.72656     -2.81126   -242.37547   -104.64506   -241.17148   -256.28964    268.80886   1099.83884   2831.74411   3211.51869   2389.38892    907.38519    -71.92194   -362.20467    -194.0272   -254.61593   -346.62795   -289.28281   -343.87379   -190.00302   -381.09535   -445.24771   -623.29933   -452.03969   -479.10342   -584.17071   -229.93328   -437.75456   -350.34369   -517.90998   -294.06554   -270.81014   -309.32272   -308.60265   -346.06115   -453.06149   -339.49084   -504.25272   -284.08065   -224.38476    -351.2308   -253.77429   -178.98142   -280.73254   -406.19532   -272.31775   -145.14581   -221.26438   -363.00077   -219.27501   -255.79028    147.23053    913.71517   1616.85208   2010.77066   1618.27245    954.79434    235.56736     99.22862   -335.82023   -236.34651   -239.58411   -279.80195    -87.03788    -68.37276   -166.13214   -186.35111   -252.85339    -345.5998   -204.79657    161.07355    887.73624    1593.9237   2507.25167   2379.41063   1861.34869    716.85213    345.32257   -150.60721   -218.58698   -218.76565   -136.67038     10.11891    177.64536     153.7047    270.37009   -152.33967    -98.08383   -258.15979   -230.86231    -361.7855   -258.51733   -291.81714   -313.99404    -380.3847   -161.99468   -300.22624   -145.72965   -286.68021    -94.54784   -291.55526    -163.0759   -275.98171   -193.81616   -311.08461     36.17213    415.76576    827.15106    918.05157    719.49282    477.09853    159.77005     -54.1466   -173.98735   -167.10441   -107.29089     42.21928     62.07182    355.82106    857.73196   1749.90008    2309.7395   2562.53657   2127.65535   1439.30504    950.71859    295.96651    505.43934    157.77081      101.133     46.53579     53.52552   -141.36498    -67.49368   -228.99687   -287.11568   -233.45564   -349.35367   -229.24746    -258.7419   -196.81132   -175.89974   -103.63552   -327.22641   -233.65711   -233.21046    -27.31629      16.3654     97.46579    133.31527     13.16721   -149.58509    -226.9691   -175.11509   -281.16408   -303.96467   -192.13227    -30.28077    147.05476    163.70217     140.6793    108.09717    114.90373   -120.77545   -102.08717   -135.64971    -67.34927   -206.57893   -274.75981    -254.8226   -365.85571   -342.24741   -368.97064   -262.99672   -296.32191   -214.39026   -127.93869   -217.58153       8.4143     79.23848    113.68865    229.25754    274.15435    155.30349     97.37959    -53.69473    103.04185   -120.45406     59.46211    -92.91141   -115.50896    -51.93406    -24.01851     75.82079    -63.51509    -39.94781    -31.72351   -225.11994   -263.92958   -346.46141   -313.15501   -341.71515    -425.4062   -267.95961   -468.38758     -184.839   -281.57368    -305.8287   -117.83047    -77.62141   -123.69081    -49.13016     34.31951     47.91119     31.11687   -143.25462      0.45632    126.23811     26.22919     59.34691    -55.63324     -43.2554    -50.36868   -173.21334   -334.31464          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       26.00   -135.95685   -170.22088   -341.49759   -403.57655   -112.12821   -388.06423   -273.99807   -327.82302   -335.74437   -376.30057   -434.46168   -110.75734   -279.31914   -286.14484   -314.77737   -312.19603   -135.38535   -196.79145   -319.98762   -183.39869   -171.42785    -60.93331   -331.42782    -86.01125    198.20179   1258.62144   2891.19349   3234.80054   2308.98894    852.49674   -192.14313   -525.61015   -355.33572   -302.76225   -431.62644   -288.67681   -343.85015   -431.41045   -522.21148   -393.81779   -203.55128   -377.84678   -420.10082    -468.5537   -442.86512   -352.35924   -174.78487   -356.02799   -322.46103   -274.80735   -320.94446   -467.31094   -521.74622   -222.36085   -539.60148   -395.13753   -348.45054   -226.14032   -184.81264   -254.62508    -93.61357   -246.24697   -309.87716   -384.80099   -408.99404   -351.63127   -341.01998   -347.09438    -22.42154     14.35319    796.44148    1815.8209   2305.93451   1748.45707    890.88274    320.10915    -89.88189    -104.8307   -169.25371   -330.66303   -319.60008   -248.57566    -24.37771    -380.3158   -294.47391   -180.29617     -167.776   -199.65756     -2.42901    623.72823   1551.08696   2675.94247   2542.01479   1662.02633    835.22951    250.53485     -60.2731   -324.25347    -219.8347   -146.05508    -51.61107    104.09919    180.79382   -131.16091   -104.95156   -266.74346   -258.61123   -368.71978   -387.75126    -431.2098   -383.44599   -367.13293   -442.10717   -267.78291   -163.72407   -297.15602   -143.98484   -143.31369    -170.7589   -189.57769   -277.92679    -48.78123   -178.24998     98.45384    548.48565    765.40109    732.79392    800.85565    544.96733    258.14736     18.71477   -144.34991   -146.32502   -299.10845     57.66844     31.33599    237.36689   1117.86167   1841.57661   2452.43839   2527.19293   2101.32857   1528.73779    791.23108    506.54733    171.27444     70.82196    239.33266     24.31944     22.44902   -185.16157   -250.81065   -158.64583   -255.36602   -256.80008   -196.37173   -363.69498   -291.20415   -144.66286   -280.17786    -174.5021   -151.87052   -259.59104   -149.01638    -81.27576   -104.61633   -114.82689     27.53413     -10.5934   -168.67246     11.54739    -248.0279   -164.13791   -351.31839    181.81652     74.08576     76.42757    152.70335     47.66688    125.34117    -59.42853    -54.51045   -134.96585   -173.93535    -148.0752   -191.30549   -150.56736    -54.10659   -252.45226   -282.70301    -170.2974   -361.62253   -154.47163   -135.66921   -294.15746   -218.44708    -100.3777    178.21695    163.32409     141.1901    169.71537      137.618    107.62561     98.20666   -124.65377    -53.12765    -79.42433     43.12926     -1.92835      75.5513    128.13842     45.52795     20.82208     37.40209   -232.70494   -177.59145   -423.27581   -405.65641   -508.09071   -342.28333   -416.55613   -351.95073   -185.86201   -276.91642   -239.82084   -272.07403   -119.51859    -75.98724   -158.62601    -51.03991    -95.31908    -84.89477    -49.92483    -33.83914   -269.44867   -364.37392          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       26.50   -211.27376   -145.80087   -285.67585   -293.45173   -269.70414   -275.76088   -387.33939   -350.82478   -305.70106    -365.1294   -480.90005     -126.176   -293.12898   -205.35178   -409.42292   -194.61076   -191.54052    -298.2857    -233.9995    -423.4733   -191.10568   -200.87945   -188.66829    -29.35431    193.69211   1356.59025   2780.99341   3475.68122   2234.94608    556.70795     63.76391   -225.44419   -247.01975   -280.05514   -421.82776   -253.53806   -296.95269   -300.97507   -451.29689   -541.92797   -383.80077   -331.47453   -191.76644   -287.19656   -395.23631    -459.7545   -323.00927   -314.78607   -367.40027   -428.50029   -318.03871   -321.64004   -361.22641   -331.79492   -313.11692   -494.07602   -421.17664    -184.4647   -226.56758   -293.65563   -446.66246   -215.00192    -66.66304   -323.00848   -250.60718   -262.79184   -157.00585   -320.63295   -276.84011    268.95805    830.99804   1768.06871   2112.86904   1808.96807   1015.37187    285.38251     92.96674     -27.7843    -32.59897   -197.51932   -303.08485   -198.67728   -399.81868   -404.06061   -386.32175    -232.3996   -168.87146   -207.27066    242.64353    793.78514   1731.23652   2651.06307   2403.65508   1747.77384    755.85046    153.88075    -18.20596    -24.92639   -225.19584    -58.00113   -110.96839    -72.93076     197.3442     78.24204     14.90422   -199.15882   -174.69205   -250.10958   -268.34208   -214.78076   -369.25045   -386.42363   -280.27155   -260.59326    -275.9947   -269.98131   -200.55547   -225.33191   -208.04448   -147.31957   -279.57264   -105.80323   -235.03222     53.90968    421.02007    504.15357    889.15885     898.8981    500.66908    135.68084      6.44088   -175.10287   -156.41542    -76.99786   -184.74602    149.57605    273.02421    995.52547   1797.77982   2413.08784   2496.27288   2051.07323   1302.28384    892.59305    440.87915    207.15651    187.51333     154.3735    -85.31235   -142.50239   -138.27961    -81.93583   -177.69402     -190.733   -119.20218   -218.42722    -222.1182   -267.39331   -174.70447   -174.40082   -130.47222   -154.84934   -209.93896    -96.94524   -148.55032    -45.92947    -69.24651     31.51644     22.64017   -276.02368    -272.0797   -168.53993   -195.60011   -155.75534    -50.23594     -2.27804    -56.64442    248.66637     81.16625     33.23258    -10.44198     92.63237   -175.70612   -241.16564   -135.45388   -241.11988   -261.87927   -154.56172   -210.35378   -268.04764   -228.12985   -271.01555   -275.48716   -151.32926   -218.21384   -186.69341    -77.75193    -36.65079    -76.83828    150.23006    119.62142    136.16356    147.65963     -48.4956      34.1327   -116.12751    -74.89062     -0.29353     79.20586      1.92277    -30.68532     47.21881    -67.45736     -91.3813   -126.07499   -149.87615    -372.8659   -347.45891   -445.09163   -284.78167   -293.91522   -369.89077   -403.71934   -352.49914   -331.85156   -325.85263   -240.57299   -217.28705   -171.91621   -319.90198          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       27.00   -269.53412   -174.04692    -338.9219   -301.56868   -447.31309   -187.91086   -531.31602   -372.35803   -195.73469   -477.82501   -228.99627   -255.21266   -291.61537   -169.86446   -474.70242    -84.15933   -297.42341   -277.62836   -245.24392    -472.9673   -326.03442   -224.53264    -79.01807    -29.12471    397.18034   1421.75566   2783.21377   3154.25688   2036.31331   1248.22667    104.68194   -194.68156   -251.84573   -174.71212   -276.87763   -182.06148   -311.27204   -191.34535   -510.61353   -285.60192   -333.08884   -351.23827   -339.12308   -207.81693   -299.08681   -190.09407   -427.32294   -373.36551   -354.10342   -473.68774   -263.64802   -262.55623   -248.78615   -204.94302   -326.85501   -377.79047     -52.5332    -80.46061    -180.9532   -184.85045   -381.22557   -221.26076   -377.60475   -232.27614   -485.70894   -348.95331   -361.35823   -354.46139   -261.17191    215.24441    903.36868   1494.26761   2038.20789   1775.71306    885.48102     359.4188    154.92337      -100.97    -88.61553   -163.36048   -122.58834   -204.61845   -310.23397   -469.24884   -251.85014   -222.61055   -312.16942     -0.13515     42.13042    886.14315   1877.58594   2649.17896   2449.85309   1887.93731     800.1331    207.30955   -100.64714   -247.24969   -319.54845     -94.1432    -96.91177    186.40518    253.24156   -110.76099    -33.64783   -231.39148   -202.76793   -353.76692    -335.1113   -321.72142   -354.89337   -187.87387   -441.58042    -303.2129   -333.21128   -281.79327    -159.0769   -156.05953   -236.44372    -96.89321   -182.83302   -189.29732   -177.68507    135.18997    344.05257    769.23923    976.23938    645.89084    575.80612    231.17357     53.83893    123.97307   -211.02389   -153.16531   -177.87848     96.22754     385.3881    1003.2153   1769.48463   2393.95781   2421.81867   2166.42433   1610.30145     909.7836    447.82133    238.77047    153.11189     46.13637    204.18581       4.6261    -86.34544   -315.29458   -303.17718   -276.01138   -435.92634   -402.27119   -125.53513   -307.55663   -232.87723   -205.19585   -292.44133   -238.91915   -151.15255   -132.30617   -136.54484   -141.91685     -0.92015      18.9217      0.09394    -27.45778   -237.58577   -246.24275   -338.06616   -124.69822   -109.29093    -76.35321     13.09884    121.62037    107.64006    185.15975    118.46903    -72.85354   -133.11827   -179.26849   -194.49777   -311.63568   -238.69133   -205.64766   -343.70611   -376.96821   -237.86808   -311.37799   -272.95868   -226.65387   -186.46127   -214.14505   -109.73947     30.45522    253.32656     91.64642     143.8572    122.61826      77.1546     45.82238    -15.91215    -52.77403    -96.22743     51.70893     -32.3774     99.48782      5.06354    -45.67327    -94.03415   -302.58319   -196.64479   -234.09136   -390.40358   -383.44565   -328.67258   -421.17098   -380.97859   -354.81497   -372.26599   -490.98789          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       27.50   -320.38472   -202.81442   -505.90895   -398.53527   -286.71029   -193.79328   -524.61611   -348.65055   -122.73139    -478.7179   -197.22799    -143.8113    -402.2773   -172.92222   -382.32163    -94.93879   -321.15501   -400.63984   -283.11877   -345.46626   -263.72937   -155.69925   -158.21991   -191.54645    278.82952   1302.49905   2773.11624   3239.56219   2273.45369   1201.16023      69.8678   -343.35861   -213.61763   -249.98359   -153.51065   -416.26103   -237.34661   -124.38149   -264.76964   -220.34134   -217.42183   -500.87561   -427.15105   -299.16594   -186.71557   -246.60606     -410.921    -493.4011    -343.9424   -370.04539   -296.57033   -320.00448   -302.25465   -188.57184   -253.07632   -369.38462   -340.93403   -149.70257    -33.98662   -136.77788   -253.13827   -299.39643   -386.57861   -459.17063   -452.66659   -198.07532   -362.34269    -73.25719   -154.67742    308.00603    903.10807   1854.93194   2144.03559   1732.89325    743.22573    583.02941    143.89149      -29.983    -71.08875   -133.09722   -354.24104   -245.72483   -200.68439   -214.39654   -158.20514   -160.63196    -77.31903   -255.87014     46.57095    819.04106   1798.26227   2394.07584   2387.23667   1673.17157    825.94922    -41.82687    -30.77468   -114.95826   -136.60275   -209.53567    114.96507    125.87289      70.2631    120.59009    -96.88506   -302.80064   -349.43585   -254.86895   -330.19693   -299.34932   -311.75366   -275.29458   -389.55905   -275.22182   -321.12628   -293.49366   -282.53257   -375.38717    -242.4632   -225.61798    -94.39005    -11.62432   -119.57057      -4.3866    305.57694    725.37252    868.78109    667.56839    556.96889    183.78984    -12.71531    -77.15185   -135.10216   -282.76287    -34.49178    -51.39693    391.74419    939.86424   1779.80787   2577.03176   2375.28395   2148.39945   1521.67872    892.07392    458.49008    299.88937    243.32246     86.85823     65.93777     72.43977    -112.8466   -302.25343   -198.82141    -145.5571   -433.50435   -330.91384   -234.52245   -307.21938   -253.82278   -166.12967   -208.41185   -269.43205   -305.79419   -168.63634   -209.85466    -92.67676     -20.4483    -12.83309     -7.79118   -148.54603   -187.44146   -242.60293   -248.12808   -162.03639     -55.6726    -58.99608    309.72112    297.87975      0.86922     95.40719    241.14255     -33.0082    -72.16152   -213.17834   -138.06491   -204.80126   -303.80606   -248.60208    -199.8053    -260.1308   -363.95324    -294.4315   -249.87218   -247.60137   -172.61432    -250.2043   -117.59624     31.51658    138.10705    104.70299     224.1162    148.14507    133.30148     42.59045   -142.31614    -250.6866   -347.39562   -173.49172     25.53461     77.91195     30.44598    -52.90881    196.04174   -231.69653   -146.01921   -334.78209   -428.05213   -495.51572   -392.19605          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       28.00   -351.30605   -230.97757   -755.20422   -494.67694   -107.82533   -183.41795   -315.25429   -303.40644   -267.75277   -433.67835   -326.26961   -264.51086   -395.72024   -214.95723   -233.69689     -9.94002   -215.14587   -465.67957   -293.42466   -184.58131    -20.11493   -354.96638   -296.76412   -162.90987    211.59329   1473.53725   2672.62362   3204.83668   2421.74517    1077.8729    -87.73597   -310.03708   -397.53554   -376.20932   -105.50366   -433.76284   -242.78052   -210.36555   -123.45585   -308.63244    -389.4014   -369.35851    -259.7534   -236.08093    -531.7721   -375.38559   -519.12707   -505.43464   -454.59031   -381.31873   -248.83291   -465.30578   -196.80491   -330.37526   -186.79009   -225.18346   -305.08013    -148.8216   -190.32712   -104.91626   -438.54786   -512.75606   -332.55747     -252.527   -528.13147   -328.40666   -460.96745   -336.21137   -238.96244    113.23008      951.378   1583.43035   2068.63926   1589.06055    852.29207     478.3986    173.55725     24.29235       6.9638   -267.38669   -209.97985   -201.48397   -361.52155    -220.8379   -285.60546   -267.00185   -261.57636     -95.7147    161.64022    642.05917   1695.80721   2377.72085   2486.98969   1675.03574    603.02549     196.6007   -107.07448   -336.49364   -301.55388   -347.94197    -20.58072     99.58542    165.91608    -11.82137   -146.94788   -133.48007   -162.81932   -282.07938   -211.56722    -393.4138   -445.19429   -315.11629   -234.16913     -91.3452   -342.89912   -130.52088   -201.62235    -208.3446      1.28768   -246.63309   -226.98903   -202.08784    -54.93093     49.87411    290.60455    662.27399    864.31895    834.28681    464.30996    201.20148     -6.14477   -130.89583   -302.00043   -231.32953    -96.18028       0.8955    368.48334    938.19119   1826.88553   2276.79311      2499.33   1859.52162   1316.55425    973.99891    424.69011    202.59088     13.83685    124.06397   -112.48589     44.51069   -229.93607   -183.77332   -283.16856   -400.12548   -165.99564   -279.19029    -194.5097    -57.13927   -361.23819   -195.88614   -231.73794    -289.9596   -165.91535   -173.32616    -70.12944     22.26021    -18.17205     34.60096   -163.90626    -126.1782   -166.71821   -141.70474   -355.49053   -269.68829   -155.99627   -126.34356    145.52384    145.96972    185.08577    118.02567    101.91037    -29.85857   -144.05921   -184.97942   -127.92952    -244.8773   -269.78294   -250.37926    -282.4003   -345.33608   -351.83371   -138.20136   -202.06401   -275.54029   -336.54409   -130.46676   -113.58817   -111.21754    -19.82496    147.01417    116.28826     89.85922    111.95522    -35.19772    -24.60922   -179.17928   -111.29762   -102.40351    -96.47402    -138.9823   -167.06883    -64.56564   -145.74183   -421.48482          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       28.50   -353.74683   -246.72438   -673.28524   -470.78121    -30.77008    -170.7837    -23.83721   -319.47186   -291.55805   -504.82604   -355.45547   -530.49173   -337.66752   -336.26927   -377.41392    -41.02474   -285.29302   -345.89221   -348.57702   -209.93222   -122.90703   -347.12497   -380.72798    -178.7556    293.04803    1492.4986   2664.48739   3273.98016    2202.7657   1084.13159   -268.02879   -398.91655   -270.48734   -157.44617   -143.85536   -330.96419   -298.56641   -278.91294   -200.94183   -215.39653   -317.99023   -570.58887   -373.95782   -178.95994   -406.78219   -345.25675   -396.10557   -360.07546   -566.88855   -435.87146   -403.38621   -377.04772   -323.52338   -290.37069   -348.97992   -288.07951   -271.86695    -91.07703   -310.04416   -274.97455   -152.21641   -365.64075   -408.79768   -479.62738   -377.41924   -324.56665   -291.08083   -305.51702   -300.60413    204.95382    776.20686   1781.02367   2116.48683   1596.57684    922.01171    346.99112     91.11597     26.23747   -137.72558    -56.92222   -318.04904   -147.63696   -121.86965   -462.97712   -403.46526   -317.25923   -196.77958   -145.58082    -58.01361    729.32451   1737.31677   2398.53459   2191.86933   1596.62134    771.79029     147.0673   -247.50664   -370.77687   -216.37625       102.01    105.49153    151.63935    109.78808    -66.93492   -147.46603     -92.2212   -297.53853   -236.25459    -342.2797   -253.30419   -300.75323   -340.60779   -218.34241   -281.65118   -411.36444   -320.85466   -153.64425    -18.46113   -259.87227   -198.97007   -182.85831   -214.63463   -107.65911     32.80282    134.52894    769.93125    722.44228    803.57073    520.17041     305.0317      -7.6895     21.09242   -196.14128    -77.03736   -109.75132      -1.3037    451.11491   1028.30222   1852.60586   2228.20899   2589.14779   2281.71548   1580.72839    836.98274    586.25944    163.54128     97.01919    103.40014    109.86816    -67.85971   -222.66962   -147.53071   -291.02403   -343.51489   -435.64488   -356.30955   -374.36584   -215.83002   -154.82603   -265.32738   -140.76749   -319.80926   -136.32334   -230.88302      3.71024   -112.86377   -118.33041    -16.05076   -139.73418    -29.39265    -137.0994    -92.13873   -252.33673   -293.05305   -205.28916    -51.31631     31.73311     46.99652    128.13957    137.40805     33.41265    164.15842    -28.80096    -94.10252    -37.01585    -64.88202    -386.2862    -212.5263   -302.69775   -279.70023   -224.99078   -294.24584   -376.50234   -168.14855   -238.44505     -97.8533    -65.65836   -117.67535       50.258      84.7212     93.23053     74.76984     74.22326     10.14365   -135.02305   -243.24624     -202.894   -249.23637   -305.19916          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       29.00   -323.59651   -217.13759   -458.01308   -397.10165   -128.15828   -189.86463   -137.87757   -330.54327   -165.41027   -360.59422   -362.23938   -379.02256   -388.99849   -269.78847   -396.07624   -228.04692   -346.29106   -391.62089   -348.47209   -237.81349   -314.17407   -407.45423   -241.98095    -24.38512     71.26385   1298.99438   2436.95997   3012.35128   2163.03906   1012.66207    124.52226   -247.46849   -270.25025    -264.3005   -369.97896   -353.84487   -343.35046   -378.07359   -392.50852   -253.06377   -342.35296   -214.67755   -498.26104   -214.87486   -378.36824   -258.97652    -339.7763   -289.20532   -402.35818   -251.53784   -432.17872   -215.91299   -314.74756   -542.06461   -219.77965   -270.54787   -148.56166   -124.70801    -73.34857   -245.87655    -275.2545   -208.28602   -413.93348   -389.29836   -384.46716   -245.23784   -272.17848   -176.19021   -245.22809      347.847    894.99726   1677.61919    2070.3506   1584.91463   1034.01024    442.85344     30.67177    -61.91446   -253.08027   -120.13494   -362.29885     46.59484   -304.54265   -179.85908   -507.74437   -115.17995   -180.27999    -53.94299    220.60112    850.99757   1459.63068   2233.14852   2320.12145   1840.67507    797.56385    107.06636   -146.75317   -317.09036   -387.51332   -381.53572    -31.43244     -0.59332     27.80898     86.32111   -155.09206   -283.09277   -380.02652   -362.98687   -308.72471   -396.56682   -278.72159   -420.13824   -353.37217   -138.53072   -311.58816   -258.12897   -262.81795   -224.21745   -184.49174   -113.41287   -296.43763   -160.07082    -74.71309     25.48163     212.0396    736.32938    776.59766    767.17343    641.72324    247.30094     71.88504    -56.72773    -131.5473   -185.27693   -120.00285     99.08846    424.71026   1100.17867    1961.8277   2666.79768   2935.89618   2236.33554   1593.18346    841.14146    380.35806    -51.38686     33.65172   -162.28582     67.86381   -169.93559   -152.37538   -106.62577   -198.33199   -354.49584   -410.75143   -177.65416   -101.46645   -230.87967   -390.02266    -52.63112   -293.39672   -269.49422   -252.79017    -157.8092    -69.62693     -8.20102    -25.72089    -82.93726   -138.03145   -328.37649   -156.42782   -323.55732   -293.77135   -322.98552   -258.58573    -75.32428    151.32862     142.7541    158.02631    189.20026    -10.02781    -33.68876    -81.48136    -33.76694   -215.32408   -301.67207   -224.04027   -301.27275   -374.11469   -238.29961   -325.99098   -368.61792   -231.10835    -274.6296   -239.55649   -263.05049    -77.14205    -141.7903     18.99629    -66.92189     60.05763     23.42556     -3.51631   -326.98716          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       29.50   -260.32217   -225.53142   -394.80293   -330.50752   -335.83104   -237.97996   -231.94938   -326.45463     -5.80985   -228.31539   -450.50965   -260.89115   -346.55482   -165.11168   -304.52756   -341.09071   -545.19755    -188.0456   -393.59938   -263.01486   -494.94706   -301.88973   -188.40509   -136.48928    343.64902   1420.06454   2260.28417   3148.03524   2219.54351   1145.23434    233.93122   -274.65862   -279.06067   -311.39558   -283.54349   -244.14709   -300.61837   -313.98976   -431.79768   -316.37679   -324.50091   -247.94304   -439.55145   -254.17616    -552.0199   -508.62976   -428.30144   -442.00766   -419.67488   -210.25253   -214.45423   -328.89361   -337.33295   -259.17314   -358.20279   -310.92207   -152.59767    -92.50486      2.82136   -171.85607   -210.23216   -219.42433   -469.53042   -411.18543   -345.13652   -479.56261   -359.66145   -302.60941    -67.74122    451.86243    799.14828   1821.76256   2163.06766    1762.1683    870.58607     84.63781   -120.50882   -349.80217   -133.42749   -161.80639    -85.43028   -224.87174   -447.07018   -101.15473   -312.05419   -470.74397   -117.82304    -41.75583    223.97515     873.0116   1480.14868    2482.4348    2437.4909   1756.40584    672.13093    199.87063   -133.11937   -268.51639   -192.48114   -299.64213   -163.18173    282.83302     24.23174     52.45354     -66.2292    -98.51391   -216.67068   -436.84575   -277.19335   -318.12071   -314.27273   -285.81774   -261.11754   -322.07854   -298.40804   -235.22055    -69.95513   -132.58632   -141.28892   -312.49037   -231.42665   -141.95469    -99.77412   -156.98751    320.94621    720.15641    923.03273    804.54867     482.5579    402.89245    -58.22836     -23.9462   -211.88121   -180.87513      7.04883    156.94374    526.52298   1135.88589   1741.15422   2445.98244   2542.23046   2280.48943   1553.68816    790.32366    487.49072    183.86654    -32.41639     32.91396     27.85639     59.55022    -111.4521   -148.29055   -227.32471   -167.80585   -278.34505   -349.87795   -221.36609   -341.14899   -317.05887   -243.71122   -329.95124   -129.26533   -255.65903   -214.24364     -58.8215     -67.7001   -103.42331   -101.69079    -11.91651   -104.20252   -221.22984   -230.73625   -261.95221   -130.20723   -299.78453     66.11876    174.22215    196.52453     125.1387    104.13103   -147.66823     -44.4253   -142.72913   -234.80548   -280.83313   -295.08588   -288.76709   -228.34203   -256.91681   -432.75882   -303.05275   -282.42823   -263.66679   -299.54098   -316.33577   -309.56312   -284.70842   -152.55219    -97.23803   -187.35011          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       30.00   -312.51325   -219.62627   -541.49509   -289.69805   -329.25169    -241.5374   -211.12628   -333.99418   -133.68562   -290.48061   -527.89663   -320.42184   -310.40799   -254.18366   -326.13888   -456.10997   -496.16375   -247.87242   -441.10418   -396.84225    -434.0849   -256.12392   -321.69002   -118.22323    456.21881   1332.87543   2269.37955    3044.3877   2233.51424   1064.66575    256.51674    -265.6904    -253.8273    -376.3249   -333.80789   -168.24295   -258.00205   -270.31756   -526.95362   -290.84065   -243.91406   -428.06818   -353.99924   -384.94575   -432.89659   -332.72429   -152.69071   -285.61413    -188.9319   -174.11564   -288.67007   -290.07869   -305.88973   -256.94393   -326.82631    -223.3452   -213.45061   -338.87311   -118.27713   -149.93009   -269.03399   -210.50117   -281.81174   -285.93217   -422.39817   -343.06418   -358.37147   -234.15885   -115.02223    302.63777     904.9677   1915.08962   2085.86438   1749.76199    856.01656    289.41644     51.50416     -80.4842    -163.7437   -253.83081   -550.56159   -118.62043   -224.18297   -333.81474   -221.53902    -86.86601   -227.48108   -205.45513    356.70833    899.00125   1813.81059   2507.52203   2368.92614   1771.07827    938.27174    405.91498     -50.2159   -106.45769   -390.67431    -98.39097   -107.55712    229.20774      73.6663     98.51846    -39.07252   -299.36591   -225.57565   -329.57576   -404.46681   -203.62873   -308.29054    -351.8184   -299.40005   -320.75659   -293.59188   -108.65937    -87.16966    -59.42598    -31.74888    -77.07502   -202.65557   -138.13907   -159.94057    -24.24621    363.36697     618.6506    866.24782    673.48761    520.66825    101.69633   -116.17995    -49.03324   -119.84133    -204.2794   -164.82209    105.21194     298.9595    770.47103   1450.57094   2494.50627   2481.24166   2232.07925   1548.56011     959.6123    398.08168    269.06712    147.46726      81.7779     40.10043    122.26574   -154.24177    -241.8913   -324.03028   -300.72653   -336.78354   -218.12118   -265.18817   -268.47936   -298.78865   -322.63501   -186.69112    -63.85453    -264.1706   -300.05552   -106.39485   -132.42793     42.19637     23.78259   -152.98527     -9.51482    -100.4247   -287.13656   -327.99688   -179.66252    -99.94331     19.81142      71.5181    154.83679    218.32254    -87.16906     -6.56343    -59.88448   -178.12675   -156.71164   -253.60838   -445.45456   -166.78306   -306.88326   -216.31425   -287.95574   -326.55697   -377.63767   -301.95326   -272.85453   -313.78997   -284.87849          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       30.50   -455.59412   -219.91768   -656.63019   -271.57067   -261.91761   -179.14699   -122.34602   -303.30754   -201.08295    -359.2788   -550.79723   -421.66235   -299.57955   -325.05109   -321.69312   -444.51512   -265.12034   -379.95816   -271.75446   -428.98966    -199.1724   -283.62102   -521.76146   -173.63707    217.07574   1154.15156   2479.95051   2874.52433   2385.73444   1193.03524    -11.42424    -80.93391   -267.18864   -433.03309   -425.81096   -261.48976     -320.701   -345.94125   -511.17845   -403.90825   -112.71266   -291.69569   -367.97114   -334.57491   -267.33081   -331.79651   -370.02265   -410.01299   -517.45362   -518.15593   -307.62527   -408.02845    -361.1566   -312.92795   -482.33122   -329.51193    -75.59915    -174.1787    -37.90785   -286.32002   -234.19364   -273.42817    -447.2754   -326.29021   -252.69536   -297.29927   -414.52866   -259.96803   -147.71565    416.81814    972.67911   1769.40138   2096.19505   1535.53519    962.65804    456.60847    -64.45461    -81.02989   -223.52147   -345.01529   -396.72758    -430.7758   -352.47219   -266.26586   -283.29782   -216.51803   -267.61176   -228.43726    146.07859    927.16679   1793.44454   2486.66857   2422.81243   1858.52694   1073.30192    351.99296     13.51805   -229.10516   -210.97813   -272.52179   -180.18434     79.20967     43.91088     54.17787   -181.16866   -150.78823   -258.43465   -283.47147   -235.81862   -386.23187   -245.29636   -210.19331   -332.90816   -301.81969   -144.60263   -263.01998   -126.22786   -201.78835   -281.11176   -141.83983   -123.45342   -162.78665   -122.62788    -52.59212     97.75761    543.96967    831.32579      744.134     609.7966    360.52069    161.27317    -17.70403   -260.57417   -199.04841   -211.33948    -55.40857    436.87666    919.62944    1609.6524   2173.74869   2529.49503   2298.39315    1772.2534    885.40091    688.83197    369.18707    148.81351    -11.22829     45.91537   -111.18367   -214.67438   -143.57935   -151.22184   -237.21266    -346.3878   -302.91451   -290.04806   -287.96233    -61.97998   -277.64522   -261.35487   -221.47061   -233.26517   -145.47678    -57.69201    -53.91947     43.17799     -5.86413     41.07672    -66.79024    -69.98188   -285.00738   -110.25692   -190.55766   -162.88362   -207.98876    -36.50954    158.76755    217.00883    242.02657     67.83452    -46.74049   -233.64406   -271.47607    -73.70366   -237.47096   -206.93677   -318.98247   -424.17888   -290.08063   -466.89187   -447.54133          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       31.00    -633.9937   -230.63234   -705.06717   -193.19734   -259.50152   -101.48153   -112.19891   -330.90542   -123.37752    -284.9118   -486.71265   -431.38483    -318.8527   -353.24008   -273.27173   -245.20418    -196.5859   -288.15478   -155.63467   -253.56049   -133.92083   -295.69891   -285.67501   -152.13835    263.61958    945.87974   2432.64664   2715.66156   2322.54071    936.76393     22.60448    -21.26105   -402.68633   -422.92383   -415.16376   -524.61303   -237.38226   -450.19895   -286.02726   -367.98356   -294.87604   -511.89596    -419.6216   -334.06279    -345.6255   -449.55643   -383.47935   -295.13245   -392.72692   -439.09973   -369.90485   -403.37685   -260.63487   -348.21066   -318.47073   -277.90672    -38.05317   -236.69761     -14.6743   -116.56967   -399.69727   -215.94066   -405.57399   -337.70058   -322.61082   -397.24215   -267.47671   -208.76896   -107.17281    176.35999    931.64423   1765.49994   1896.10194   1579.43331     873.9592    271.14175    284.24575   -161.02454   -275.37482   -367.67455   -358.00528   -463.05788   -276.72363   -254.85672   -435.25464   -233.82118   -302.91767   -262.04903    163.71028    808.61313   1751.95517   2422.10377   2373.22169    1556.8034   1015.01459     -29.1711    -65.15379   -225.05273   -241.85171   -218.60756   -265.57948    161.64732    144.17349    -61.77241    101.83727    -94.52231   -302.53665   -313.22332   -242.30263   -328.37739   -352.84166   -180.79147   -170.75216   -249.41576   -354.96872   -306.53706   -194.47369   -295.16697    -58.51077   -116.80169   -260.42602   -244.13273   -170.50238    -59.57205    259.11378     495.6626    790.81153    731.93373    532.22658    340.21938     75.78197    -125.6584    -72.56852    -223.5723    -75.84117   -100.39209    448.54727    852.55492   1668.76887   2266.40561    2616.4282   2129.18327   1440.04559    825.36875    358.83617    147.23309    166.13743     87.56559     -7.31082    -79.93458   -165.38413   -243.79704   -171.00425   -269.50098   -311.54696    -241.0374   -279.20098   -399.84449   -332.32013   -227.51159   -320.03244   -285.22549   -201.50693   -245.59033   -150.91346    -67.34202     57.67806    -62.68091    -31.88355   -136.31539   -106.94677   -310.25673   -134.88957   -308.86199     66.14928   -189.46698   -192.40852     69.71872     11.11106     59.82166     98.14917      -8.7694   -121.94203   -163.82565   -245.23748   -281.87563   -359.31789   -404.59361          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       31.50   -685.94305   -272.48953   -523.78596   -225.22811    -300.6513   -208.17255    -226.3866   -399.44477   -101.96879   -113.28795   -534.85539   -501.81606   -449.24115   -355.77263   -344.53742    -42.84159    -257.6311   -159.22156    -74.79304   -162.26175   -234.95869   -232.73501    -69.12395    -68.61902    400.93979   1036.67677   2338.90941   2711.14259   2051.58944    695.97901     12.25965    -62.49848   -193.92889   -310.13536   -396.25486   -212.36517   -191.50064   -413.26726   -214.52941   -344.48194   -226.49955   -295.87769   -275.54179    -471.4223   -352.91986   -232.99614   -565.24804    -87.17919   -345.56482   -453.54166   -365.64131   -471.26879   -438.35334   -436.15718   -371.54483   -356.42816   -294.14875    -35.88515   -213.85094   -230.78385   -242.32744   -322.34468   -428.06625   -327.29626   -277.99198   -304.43441   -214.33481   -268.40969   -182.66239    150.15562    781.51732    1460.3827   2033.17028    1572.3287    786.24861     94.02226    145.73975    -19.20737    -94.08076   -207.50703   -325.48852   -169.07451   -452.88105   -237.73121     71.38966   -316.38297   -258.17512   -164.98131    249.65656    707.34442   1411.90768   2288.56387   2268.94377   1639.82848    758.63364    207.68784     -8.36644   -176.09508   -125.85056   -109.87098    -14.85062     39.40068     95.60971     92.79834    -148.2141   -121.59315   -324.04649   -327.41192   -279.19101   -336.42746   -294.66283   -325.66925   -271.30584   -226.64634   -262.47228    -79.59066   -131.18824   -231.47763   -145.05525   -294.32894   -339.19257    -59.11208   -104.85778   -144.58097    237.20822     785.6485    810.79742    703.55609    407.81629    203.03002     -28.3106   -173.55644   -254.57097   -128.58772   -106.32577     -62.1445    361.98281    917.46519   1798.65543   2388.93219   2484.79484   2267.99821   1629.92426    954.68908     471.4048     320.6012     -4.07864    -42.26203    145.64057    -65.02002   -213.29469   -277.36887   -270.88971    -354.1939   -192.19366   -157.74739    -355.7964   -295.64946   -164.59665    -99.05862   -267.96396   -308.83144    -191.7508   -133.10196   -158.44852     24.49906   -107.33113     21.58729   -112.04007   -104.33642   -179.98933    -178.9724   -247.54843   -309.16948   -250.10609    -86.26572     -4.77302    -21.58775     60.86114    264.77874         9.18   -123.76632   -223.89593   -269.59131          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       32.00   -596.83016   -262.74594   -341.96768   -378.24314   -328.07536   -285.93026   -209.49156   -472.29296   -181.72211     13.79858   -659.55563   -454.48964   -406.88343   -292.27649   -380.56588   -289.70707   -231.61441   -138.13052   -303.00107    -356.7172   -384.98871    -34.74384   -275.50368    147.90915    589.99049   1325.94296   2369.18205   2737.01601   1685.79015    877.76289     112.6865   -109.25506   -205.43382   -289.55976   -321.00539   -141.48555   -315.95373   -331.28966   -398.97493    -357.9164   -534.31524   -305.86552    -225.8809   -281.51958   -498.63126   -465.16142   -489.32397   -294.35902   -217.15638   -583.20295   -289.31291   -358.04036   -421.80761   -290.50506   -425.43112    -339.6914   -399.63959   -180.70561   -194.91854    -24.64781   -203.25877   -241.70604   -341.84339   -151.09271   -271.89883   -374.88076   -232.89495   -271.22006   -158.18319    180.60252   1033.24334   1853.67063   2235.44976    1710.0133    980.63882    310.40261    125.24162     42.67019   -109.38045    -362.6633   -233.77942   -204.69195   -282.24972    -109.7342   -268.14676   -180.21734   -286.81208    -46.02805     74.07632    607.09837   1572.60383   2123.30618   2403.83004   1420.30349      849.554    269.73759     16.47968     70.16981     -4.59488   -313.60469    -56.76346    221.98319      6.39806    107.51845    -89.15812     -224.607    -371.5317   -276.29362   -342.52757    -263.6254   -383.79836   -293.81005   -315.26343   -144.41616   -169.70394   -252.32094   -315.05841   -113.69158   -194.75385   -203.62372   -186.69339    -89.54137    -134.0028    -76.40863    315.28169    678.44961    917.25066    874.61215    639.69767    344.44963     97.74075   -150.83479    -64.40782   -250.77068    -61.68412    -62.19005    389.20925    795.76196    1614.1205   2212.78252   2401.23073   2048.86716   1618.61369    970.92649    621.84517    277.77155    268.59738     50.56323     71.31934    109.58451   -109.10843   -226.96569   -312.59127   -216.74075   -269.54247   -141.61328   -372.02552   -471.27204   -206.09321   -179.51809   -302.79455   -223.09831   -212.87635   -197.67472    -93.44686     17.77837   -125.78175     12.80929      2.52598     71.47589   -112.18796    -209.4606   -278.91064   -313.50499   -215.38752   -230.24052    122.52816    -104.9325     -6.81336      3.11389   -294.80277          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       32.50   -481.10432   -199.32892    -184.7046   -642.26719   -276.44484   -287.02283   -168.26895    -333.5444   -209.73694    -125.5287   -475.45919   -219.20926   -196.95752     -275.469   -226.71639   -436.06644     18.09862   -239.19661   -463.47827   -347.62712   -379.19394    -67.07211   -307.70625    -30.90287    363.93805   1246.06256   2195.82832   2653.54014   1760.98754    933.71181    105.81514   -216.59389   -301.15443   -312.43196   -228.78984   -290.71388   -234.56359   -341.76523   -367.59612   -420.99071   -277.84535   -333.69435   -431.31341   -356.44189   -431.03744   -252.14344    -258.6228     -166.435   -385.87578   -363.49867   -312.96933   -329.24474   -395.83337   -252.43564   -487.03752   -231.66223   -325.70383     -244.223    -67.66632    -81.73603   -188.14446    -216.0873   -336.18431    -323.2264   -335.95403   -246.03133   -207.34266   -252.17806   -205.28396    267.56213     826.0819   1547.19584   1854.33277   1451.37396    861.26447    354.68577   -166.00881    -36.22985   -127.26505   -100.16878    -161.1838   -250.10561   -273.24027   -107.30856   -461.78936   -224.06649   -261.74342    -83.71222     14.31382    666.29262   1476.91588   2229.24577   2078.61317    1575.7521    782.73923    339.75163    -18.99607    -38.42406    -72.77156    -57.53962    -94.18505     12.99794    161.49303      4.31265    -96.01834   -169.37108   -284.69202    -304.6748   -264.43911   -217.16272   -255.96748   -296.04135   -292.43296   -359.75946   -325.88755   -295.50091    -201.9339   -112.84422    -84.96447   -164.68087   -130.47614   -255.56612   -208.27558    -49.50175    121.55723     594.4311    959.29068    744.73169     555.3897    328.57479     48.07483   -131.98009    -180.2645   -211.22417    -88.67089     63.42692    276.10328    695.67891   1638.83833   2165.86454   2422.13762    2046.4634   1431.12568    957.04134    506.01471    292.35778     -3.77308      4.00756     69.77945   -108.12966    -45.72118   -133.83047   -265.88764   -239.85745   -284.49885   -343.57134   -236.40444    -285.1323   -359.19037   -289.02265   -373.42183   -279.67153   -250.40644   -325.29303    -85.11812    -32.28957    -43.65634      -3.3003   -125.02823     -67.1024   -242.19393   -195.79514   -360.51032   -269.29371   -215.56898   -213.33308   -328.44118   -307.22478          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       33.00   -368.19711   -149.46176   -142.86677    -561.4129   -102.11991   -320.77543   -186.09847   -191.20577   -217.14914   -218.56604   -361.88089   -324.73854    -77.74021   -377.63981   -181.10539   -389.59324   -129.82083   -204.59862    -281.2456   -194.22353   -185.38903   -233.61656   -190.05138   -238.24249    375.58171   1137.93404   2231.46571   2521.02352    1969.1684    936.26773    150.20778   -119.40657    -310.9186   -155.67325   -179.88427   -483.34722   -177.41619   -360.25796   -525.34212   -317.83438   -404.32822   -416.50109   -280.45523   -529.91455   -550.99259   -450.20384   -265.58815   -352.96888   -233.77473   -383.38349   -388.30494   -508.88005   -454.93863   -329.78895   -429.55558      7.52791   -416.92909   -364.48646   -205.87598    -15.96438   -244.47769     26.46413   -297.22356   -198.17853   -301.23018   -359.29861   -352.56824   -206.49946   -271.52948    112.95228    867.16924   1671.33403   2251.52128   1374.92532    975.82896    364.37379     -3.23335    -12.14299     -12.1066    -153.8802   -151.47018   -248.81823    -269.5016   -339.89013   -107.16833   -200.83983   -332.18921   -196.26903    264.82726    628.06139   1343.41596   2082.23002   2196.38628   1479.43271    773.79612     39.05892     75.87715    -33.38673   -185.29162   -123.71963   -290.02216     57.08025    -20.88438    -94.64023    -52.44646   -182.09455   -257.74183   -238.37576   -320.98829   -322.69349   -259.61075   -322.14686   -385.34676   -325.06828   -416.80422   -290.98699   -234.94464   -209.60462   -141.73062   -191.45946   -219.77448   -100.97085    -86.70779    -65.39821    253.46428    446.72517    690.03173    817.09582    696.69236    236.68701    -14.77641   -126.26189   -321.06466   -275.82493   -249.35918     25.14272    305.73201    942.28622   1423.90669   2009.91334   2280.14292   2099.34785   1544.10437   1004.12264    426.41825     82.28324     92.54421    106.25039    112.60345    -23.66285     60.07116   -122.24926    -266.4271   -171.56465   -252.29378   -253.27159   -401.28107   -315.92647   -216.15142   -181.59914   -284.55147   -247.46378    -324.6616   -206.39026    -300.2534   -116.40505    -38.53643    -40.35516    -83.64074   -199.14096   -398.56134   -199.89709     -235.067   -333.10054          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       33.50   -396.71753   -210.97604    -120.4811   -420.30104     154.7266   -418.80735   -252.44495   -192.15971   -279.98119   -317.75004   -445.30306   -405.97028   -337.32223   -391.80791    -276.8869   -405.40469   -194.74008     39.96294   -224.96946   -255.05547   -236.36189   -229.43861   -284.65127    -39.18516    486.11095   1023.72975   2232.46136   2487.28639   1862.95078    914.67942    184.27411   -274.52972   -317.26185   -423.81973   -303.98524   -441.08295   -291.13661   -429.92893    -389.8956   -406.38733   -502.65304   -241.11882    -247.4731   -586.12665   -553.53028   -396.66471   -335.01166   -363.63164   -353.19181   -469.52659   -417.98166   -423.53919    -237.2577   -173.30428   -328.70006   -132.63348   -283.96594   -273.61769    -211.3291    -53.39684   -343.23332   -367.43898   -346.60603    -269.2487   -297.71852   -244.03367   -192.03485   -194.75243    -50.31544    250.79157    855.42733   1569.59458   1969.28686   1379.77778    839.99925    389.80475      2.43519   -290.69878   -265.17378   -400.83462   -378.26832    -54.49274   -281.34456    -86.47214   -261.01175   -288.64362   -463.84084   -176.18386      187.761     698.2298   1713.16035   2174.62887   1949.42905   1499.55316     631.2822     86.06999   -208.74916    -22.97998   -180.43635   -135.02039   -116.18486    -107.1237    105.13182     31.71093     30.16622    -278.0891   -184.03694   -357.23437   -478.82073   -351.54007   -334.38817   -340.41089   -239.21462   -166.34771     -222.161   -250.42418   -296.33636   -194.09488    -55.31276    -17.27756   -165.33982   -158.23958    -60.10595     64.78556    303.13054    493.72969    859.89072    805.64987    582.06571    309.69954    154.86697   -145.31052    -288.8095   -179.01373   -159.85196     13.03845    306.64395     737.5222   1429.47104   2061.90631   2514.01654   1977.40042   1647.60181   1098.75208    498.95241    241.66524    119.27402     87.97166    126.55106     20.40741   -127.16482    -80.63713   -335.96019   -244.34091   -340.66709   -244.62714   -189.49971   -251.98491    -219.4858   -215.77748   -151.62772    -269.8861   -191.70397   -273.87926   -109.73077    -42.96827   -103.89707     -142.855   -134.29769   -238.09271    -212.2983          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       34.00   -424.86447   -376.26715   -172.10475   -292.92587    296.47393   -428.37261   -385.43683   -289.86262   -374.27903   -355.54594   -378.02862   -370.55501   -466.69348   -380.13044   -288.90886   -339.96431   -258.57048   -266.23778   -222.53133   -357.53053   -244.06971   -276.87154   -318.84875    -31.93375    299.66622   1214.66971   2251.30322    2622.9408   1942.11295    717.34893    108.12799   -364.70314   -430.76332   -377.97447   -372.76042   -422.58734   -256.89567   -251.54665   -410.19063   -242.64131     -398.727   -202.01783   -435.99123    -492.9723   -375.85039   -129.12838   -465.14277   -305.65054   -451.50108   -209.04586   -277.58344   -379.17506   -280.64597   -357.27262   -341.50972   -362.02609   -315.53115   -137.05961    -31.46626   -300.78469   -300.28888   -425.85716   -286.82251   -189.57909   -137.94268   -358.26306    -91.63691   -201.65029    -58.77042    226.73631    793.09532   1593.64404    1998.6403   1517.44543    976.47525    202.35711    213.07962   -105.52195   -360.90908   -471.96949   -136.50937   -318.83995   -250.53402   -249.11964   -181.23053   -351.07034   -197.45686   -160.91638      122.333     693.6568   1334.90454   1887.24356   2143.39241   1434.95834    471.81948      15.3298      -147.58   -174.94053   -164.98873   -132.60136     -4.82208     55.02624     -5.70601    128.37235     -60.1913   -160.76438   -314.26643    -390.9852   -228.82061   -271.09673   -444.62018   -292.21357   -367.30041    -300.7082    -253.8287   -247.87853    -96.31217    -62.47619   -136.17191   -102.75212   -259.21774    -118.9173   -143.24678      2.23305    132.04175    531.03864    780.41539    715.07053    473.38209    410.68809    131.10551    -50.65738    -92.43083   -231.95017   -245.25332   -100.34696    110.32397    868.61974   1579.35344   2038.42312   2452.19439   2195.95256   1468.01825   1114.96606    566.66788     113.5499    114.06861    126.31947    -43.23764      2.97342   -262.17619   -168.93969   -154.76668   -374.92478   -312.15962   -281.83316   -293.84884   -321.38166    -220.7765    -156.6719   -271.83303   -244.31835   -365.23502   -135.96767   -197.77667   -126.28494    -95.03444   -291.92362          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       34.50   -452.63494    -465.5871   -269.72482   -286.07889     10.92244   -441.60036   -474.22363   -320.01213   -264.07466   -341.29831   -293.93414   -456.19407   -473.97485   -407.85677   -287.20519   -199.43325   -301.81077   -340.77467   -240.64076   -453.81652   -152.42513   -390.92415    -347.2827   -109.34886    306.07033   1120.39506   2237.26952   2559.73029   1825.43313    801.73499   -161.01733   -224.51046    -365.4395   -389.75428   -445.90452   -473.23586   -434.39794   -173.25591   -286.38881   -500.89274   -278.63361   -398.05367   -302.14684    -402.0298   -400.96432   -448.83876   -525.62208   -550.89462   -467.03058   -315.06753   -453.37754   -492.62074   -223.01756   -437.12743   -357.52861    151.32789   -257.35912   -103.40142    -51.98538   -188.57314    -336.3368   -291.24609   -245.70873   -450.63974   -277.65009   -181.86122   -427.49676      -210.66    -86.47834    302.53123    864.66828   1525.50375   1870.08722   1213.89065    769.88438    356.13797      2.46574   -239.36118   -282.42844   -300.51771   -207.45809   -414.24346   -334.53026   -168.47345   -404.35196   -294.91047   -108.59356   -177.08826     57.26913    633.29363   1407.64402   2134.01168   2015.92715   1528.13559    527.45677   -154.52056   -238.69466   -337.83133   -256.59885    -54.79913    -39.12905     14.56949     63.34255     96.12965     27.36668   -267.44673   -237.11424   -252.38756   -361.07044   -295.84776   -305.46592    -358.2075   -235.82141    -375.7063   -425.26398   -402.83486   -185.14181   -248.38668   -195.89177   -175.31499   -163.20282   -106.23317   -159.77284     30.82622    260.25448    570.17345    668.85017    769.12936    528.04603    235.54007    148.95564    -48.10124    -81.11849   -247.82968    -37.22589     -7.77091    228.96537    802.43867   1402.62475   2193.35034   2189.11591   2285.99243     1585.605   1191.87624    641.91339    112.59083    116.10623    -12.75054    107.68049   -164.35516   -119.72206    -85.24431   -137.61596   -281.53654   -449.64502   -288.17483   -347.01707   -291.79023   -342.81893   -245.07309   -200.41381   -262.26843   -468.01956   -319.62033   -198.12723   -315.82741          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       35.00   -406.32595   -285.05384   -356.01061   -348.40516   -235.12069   -488.61908   -430.63483   -301.03365   -134.30585   -353.95495   -302.58123   -597.74917   -406.49111   -344.85881   -322.20334    -101.7118   -282.05173   -348.52164   -296.66108   -452.64857   -120.34542    -291.4093   -532.97143   -295.08795    519.61061    1211.0862   2211.46543   2492.41474   1743.38968    954.85606     20.93651   -340.77948   -128.42592   -430.18902   -356.66324   -454.82916   -389.80876    -342.9005   -272.61611   -289.87928   -356.48571   -444.17885   -331.33274   -415.04052   -329.62988   -372.88771   -531.28626    -273.3714   -417.81501   -211.35772   -470.55014     -340.436    -167.2271   -374.37087   -329.37336    -84.37777    -81.30543    -31.12976   -225.49584    -192.2908   -189.17026    -262.7789   -403.70595   -321.25326   -447.39546   -296.86694   -250.95707    -324.9937       11.709    147.19326   1041.11433   1703.40704   2025.76625   1658.38269    775.18314     87.87357    -27.07398      -165.79   -200.53874   -124.60473   -380.43983   -206.09555   -225.27208    -146.6123   -145.10372     -221.049   -140.03139     -85.1978    247.17896    789.56286   1242.79736   1991.46571   2074.05116   1268.96935    617.93774    191.68998   -185.56435   -271.56162    -120.9171   -128.68647   -253.22113      6.20336    192.79066    -45.31796    -93.61353    -205.3226   -146.92717   -426.09228   -314.28222   -432.78028   -369.94432   -382.32288   -266.46844    -299.7571   -250.01649    -220.6839   -275.00375    -117.4118    -93.44753   -184.15456   -184.78783   -303.74391   -176.08026    136.20757    332.00445    479.87252    636.28726    935.62812    572.26368    328.62858     73.54684      2.06979   -159.27491   -126.20485     -65.5121     88.42159    211.47533    689.68432   1241.98666   1883.04959   2397.43094   2196.68537    1688.5816    950.03961    657.18772     318.1843      8.91231    150.68552    -30.83675    -59.97544    -38.30969   -104.39527   -235.39281   -223.97268   -355.58718   -316.02853   -362.10004   -310.59822   -322.88094   -236.21041   -342.02146   -329.10196   -360.67611          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       35.50   -327.74103   -160.27891   -421.99052   -381.17847   -210.40445   -417.86495   -302.59185   -256.15613    -49.59141   -395.37375   -328.33353   -591.00263   -293.16902   -243.87976   -356.56527      8.39798    -321.8444   -400.43922   -265.02791   -416.33575   -253.94611   -106.37317   -561.21746   -220.72836    355.73001   1141.22811   2057.58086   2474.82226   1848.64545    967.59302   -120.83143   -247.25725   -382.87368   -538.55977   -341.75376   -334.07172   -301.40139   -258.27011    -256.4003   -269.71084   -455.95846   -249.31739   -288.74867   -490.68526    -268.2989   -387.11465   -432.43161   -449.18966   -351.22745   -378.87255   -195.92199   -383.13922   -309.58345   -686.02102   -427.96131   -352.47962   -271.79745    -82.17275   -163.64122   -170.93072   -166.06582   -273.14431   -168.81623   -279.05466   -325.81897   -315.60967   -428.44513   -194.91673   -123.13413     278.4535    887.74153   1416.70458   1774.67364   1587.23388    735.99009    303.95297     -5.85349     90.26357   -419.18606   -234.34078   -176.39729    -245.7914   -280.53821   -174.02674   -349.67602   -360.75141   -189.02893    -26.24672    266.47533    649.96463   1413.49884   1687.86914   2045.54083   1632.58425     920.3079    258.04503    -84.10349   -200.25909     -61.1057   -135.00365    -42.96945     54.45566   -216.89182    217.51039    -119.2028   -160.58437   -278.83079   -409.48498   -217.62798   -348.67056    -375.7715   -298.49452   -265.23593   -266.95869   -257.23916   -305.44078   -160.52648    -218.5908    -48.94971    -67.84248    -198.5628   -238.02754    -45.95663    167.88194    194.79902    424.60891     743.0513    716.07977    715.43647    274.74303    -99.92298   -175.83106   -325.74734    -79.05372   -246.97048     -8.93513    284.71005     601.8208   1178.21054   1762.98932   2254.45538   2003.59766   1572.68401    973.73058    721.31182    337.61751    183.42449      40.2927    -65.95471    -65.72449    -19.09456   -123.31372   -127.68034   -313.09892   -402.39622    -338.0384   -276.67076   -403.10697   -383.88782    -446.9556          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       36.00   -302.36976    -91.80205   -373.07436   -406.22569   -107.52445   -258.46012   -286.16238   -195.07448   -224.20178   -469.69439   -308.15147   -504.12363   -187.57159   -249.99738   -406.77435     -82.4204   -408.79906    -441.1305   -189.28328   -275.98684   -322.05061   -208.37631   -320.45011   -106.06982    335.10115   1291.79401   1870.67599   2268.45783   1606.52264    812.76871      0.03403   -222.91477   -350.83635   -345.06862   -366.77192   -303.59327   -458.71981   -240.35606   -344.49512   -298.17129   -122.47065   -350.59447    -471.0358   -513.22205   -376.42629   -362.42208   -187.12719   -324.12565   -309.62809   -395.45874   -306.88736   -245.93461   -435.84842   -361.48539   -433.13347   -194.24022   -272.83948     -73.1918   -147.80518   -186.54451    -234.7306   -365.21112   -315.71375   -456.79241   -347.55011   -362.97227   -442.62168   -292.90099   -103.08416    229.05176    784.17279   1646.95452   1726.65481   1526.61752    880.45878    350.64844    142.35885   -236.04556   -344.55905   -335.35778     -1.78795    -54.35647    -205.7405   -372.42585    -317.5046   -345.52616   -361.70239    152.30557    203.75388    769.83932   1239.94926    1992.9451   2075.62515    1604.9274    735.75097    269.59922   -106.46391     -219.015   -450.28859   -124.47163    -12.46281    -68.89351   -152.09281    -60.38743    -98.39066    -98.11922   -181.45733   -217.92903   -313.72461   -214.52554   -348.68757   -322.67916   -238.39218   -256.12823   -275.66312   -256.96366   -294.59853     -96.9261     -4.02786   -149.81275   -214.87363   -240.87345   -163.17582      36.5288     98.96489    474.84003    752.55277    678.50497     342.3024     427.7582    120.06928   -137.69671   -150.80522   -214.56455    -61.60506   -153.22184    261.19705    661.52256   1417.91004   1808.25125   2385.34476   2156.84528   1368.80786   1153.80705    553.20749    162.11625     -3.26254    -40.33349    -133.3559      -4.3269   -168.88776   -137.76759   -356.55911   -274.17831   -385.76909   -277.72475   -371.71732   -407.57621          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       36.50   -311.02967   -207.57382   -221.26172   -436.63389    -83.19703   -213.04047   -360.63286   -224.32044   -318.22363    -581.2637   -477.71577    -381.6043   -260.93761   -386.24073   -390.21867   -305.82248   -566.42221   -331.96243   -288.11432   -296.58205   -367.50327   -276.29214   -283.93216   -214.66293    567.00062   1104.91871   1884.66824   2334.75437   1728.71834    711.22071    140.89965   -385.61168   -257.88356   -445.38602   -234.13717   -157.56236    -251.3936   -385.20355   -407.23165   -354.46733   -264.52843   -318.52621   -413.45909   -410.68257    -561.9553   -378.86394   -369.06841   -260.11256   -523.85328   -440.33846   -319.58198   -313.66236   -521.70189   -242.41145   -393.06465    -246.3624   -341.78492   -164.90645     48.56802     40.47844   -187.28728   -423.55205   -327.21188   -188.68876      -474.42   -318.17021   -218.85797    -297.8021    -18.12424     344.5845     932.4431   1562.29493    1743.7001   1412.83899     777.6854    315.26596    101.34154   -312.31427   -237.49275   -465.29257    -323.1238   -170.11823   -250.53932   -318.87542    -14.97037   -407.43682   -163.54578    -88.23184      59.0043    666.34194   1318.17799    1973.0654    2226.5853   1527.96868    723.33482    205.87063     -60.4885    -267.1848    -13.22193    -69.28595    -83.23121    -69.61228     64.48605     64.91857    -29.19991   -308.34186    -290.8864   -271.60578   -351.02577   -391.48271   -376.18875   -349.90475   -414.59646   -278.99662   -430.55439   -195.49727   -284.36145    -42.00989   -213.26444   -188.22608   -148.17926   -269.28206   -183.69453    -89.75205    214.33321    455.01446    635.10545    823.28566    513.17551    284.80195    177.89305    -48.52535   -237.20019   -147.50482   -142.14548     146.5018    193.78624    643.90893   1071.65531   1616.74608   2201.58758   2006.48572   1625.24013   1095.52177    533.54738    206.42253    196.38565     63.60704      3.33788   -209.90276    -85.58268   -265.20087   -300.37401   -397.04842   -308.51482          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       37.00   -195.81857   -389.88016    -54.01905   -472.45162   -104.24527   -245.72769   -367.63596   -297.96785   -247.35416   -446.27562   -468.17618   -250.83247   -175.05965   -505.43117   -230.44575   -352.83591   -692.65571   -273.38738   -337.37719   -303.99199   -396.32464   -297.44596   -487.96064   -338.08044    527.89859   1213.05594   2007.32654   2256.43073   1843.38355    653.10176    121.21321   -268.89916   -173.77993   -361.21229   -188.52879   -199.42877   -412.40992   -463.60056   -307.86257   -274.84632   -455.03669   -602.71349   -485.03038   -418.42521   -388.14302   -403.32038   -295.29793   -436.32792    -367.9717   -378.71844    -312.8071   -310.61474   -448.55424   -549.40683   -319.35009   -414.84649   -301.72487    -55.63283   -240.91692    -92.99667    -227.6709   -398.57578   -381.04308   -265.22463   -131.91909   -380.19759   -183.10308   -212.82708   -276.10066    319.13824    791.18142   1507.45948   1672.74067   1415.72476    722.52844    544.44353     11.33424    -71.93687   -199.25181   -230.38187   -292.95987   -301.72678    -80.41612   -239.24197   -207.21702   -485.24495   -209.34471    -56.85258     38.31293    509.32118   1382.74917   2199.30244   1912.25328     1345.423    859.16155    158.07649    -47.46925    -18.55762   -154.78599   -162.03243   -130.10733     19.53392     59.91664   -191.10938    -78.12714   -117.92653   -280.86211   -396.85817   -261.87508    -279.7827   -231.41834   -317.26877   -286.72639   -398.11856   -270.02133   -153.97021   -172.51231    -66.82552   -234.75932    -207.5514   -189.26083   -274.68621   -195.17645     -48.5921    156.64135     390.2634    559.69984    696.64528    450.98434    290.63503     129.8214    -122.2052   -136.70382   -250.69718   -109.87353    -32.68448    165.52641    523.93288    981.82713   1677.29852   2079.42686   2185.70358   1720.99474   1106.00131    655.32904    206.23328    242.80657     82.51786   -109.87124   -151.13001   -152.79306   -121.93841   -302.36213          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       37.50    -13.49567   -597.37111     13.70355   -422.53261   -146.41145    -284.8834   -336.70061   -361.43399   -246.31475   -423.63167   -278.50048     -200.919    -35.19378   -352.99471   -330.35484   -220.00074   -492.21687   -473.28729   -280.19114   -346.88116   -386.29469   -332.94111   -417.48728   -265.41709    490.17083   1220.29198   2130.85612   2403.90035   1809.34171    712.47928    112.73286     11.90549   -172.76481   -248.22469   -444.43329   -172.02883   -376.37312   -141.79384   -429.39174   -527.04845   -322.10689   -362.51152   -228.66304   -315.86538   -316.05979    -421.6047   -339.60283   -259.30784   -461.17344   -445.20577   -322.29727   -195.71792   -448.54759   -444.26337   -256.56359   -278.83247   -279.58716    -65.09859   -279.07047     56.85485   -123.81901   -494.46756     -393.183     53.99736   -413.05088   -379.31002   -256.44416   -275.14375   -257.47309    137.11366    867.06659   1626.16851   1507.03459    1477.0951    981.66817     308.9207     79.88407    -74.58081    -84.08527      2.13851   -130.80939   -297.06582   -290.32214   -399.77438      119.586   -318.14187   -319.35397   -187.06856     62.02467    570.17462    1054.7234   2012.14352   2091.40287   1796.76066     670.5373    283.65415    -93.07232   -133.89748    -51.74814   -232.69915    -31.38255     12.98599     -6.49374   -157.45442    -79.30764   -358.48354   -345.64056    -248.7428    -420.3946   -355.95544   -292.09041   -276.96228   -421.80621   -312.03822   -259.55056   -178.11066     -225.323   -162.62445    -91.68758   -250.11078     -13.3588   -143.75564   -313.78188    -59.90341    197.28593    305.68372    555.93986    724.27388    644.94373    345.86822     77.22722   -106.92811   -321.07667   -121.91377   -139.83468   -157.39714     67.75298     455.9536   1013.18608   1527.05222   2186.36669     2102.647   1643.26305   1180.91401    567.82458    227.51104    -77.06759    -37.71414     -9.38289   -328.97236   -305.81984          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       38.00      63.8911   -623.55748   -181.85091   -350.52717   -163.88228   -435.37183   -277.50818   -382.81848   -509.63253   -604.04568   -230.85178   -208.29661   -136.21597   -332.68416   -479.00449   -360.43281   -329.95311   -601.73205   -274.93352   -161.81307   -410.85205   -150.53754   -343.40284   -147.84795     201.0603   1328.38326   2201.21474   2151.08238   1789.55279    662.47196   -110.01934    205.39074   -300.41358   -475.26642   -507.01021   -273.78391   -473.95586   -291.81776   -269.14275   -463.85284   -364.03505   -363.38374   -315.91291   -275.77021   -344.83761   -267.80081   -452.84811   -295.32824   -348.56853   -355.61964   -339.05795   -491.71081   -370.56265   -348.16927     -397.047   -263.52259   -276.13659   -376.65801     48.28991   -133.54978   -294.32046    -264.3338   -378.96099   -414.90456   -377.73113   -301.98657   -228.51894   -202.85304   -243.16686    224.99798    879.19186   1473.44076   1664.63424   1244.57883    772.96975    272.01535     26.14797    -47.43195     41.25583    147.43526   -136.20372     -5.44865   -184.18446   -264.97298   -161.04368   -183.16026    -181.7317   -297.00178     77.69276    666.83754    1339.2628   2002.78244   1993.78432   1592.14043    938.20191    418.07554    128.93551     37.67974   -128.70194   -253.83227   -145.03085     52.62395     -8.48788     46.45266   -100.19464    -96.84874   -105.07718   -277.64684   -344.51554   -285.13315   -377.69658   -313.71387    -346.9681   -290.55761   -376.04442   -229.48731   -119.01825   -271.50465   -261.33186   -166.79276   -335.62334   -227.37343    -163.5976     18.25301    236.28308    503.70251    473.80171    840.92496    590.20924    530.11045     207.8889     -45.5147    -134.3087   -243.32856   -184.23923   -158.24117     59.47039    586.20243   1060.09034   1743.07864   2131.99291   2233.36441   1692.34014    991.45873    648.59364     93.71393    -74.29358     -47.7349   -367.45237          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       38.50    -78.06576    -481.6831   -282.87018   -255.59996   -150.79775   -540.20951   -169.12549   -400.31565   -694.69335   -455.21375   -373.09599   -234.14633   -240.91167   -452.09188   -379.56839   -609.16701   -396.64405   -516.27322   -416.55284    -18.85869   -469.13311   -239.17038   -360.91916     11.19482     74.04634    1178.4512   1750.45676   2304.56441    1822.3712    952.29504   -129.09606   -171.24041    -405.0018   -352.34474   -327.28305   -330.96834   -488.97209   -407.26544   -309.62046   -319.47139   -320.38098   -328.04515   -407.12628   -275.97047   -178.88535   -303.73043   -315.20898   -193.41563   -477.17507   -229.81656   -354.37779   -349.23161   -404.31105   -255.97943   -427.02154   -376.37107   -309.62805   -191.05444   -221.73529    -31.05246   -173.87095   -250.24769   -444.30308   -257.52489   -326.15534   -317.20482   -449.14151   -338.11113   -138.21175    329.39223    784.62993   1484.91069   1688.94541   1227.01694    649.25927     273.5294     25.53409     97.06422    108.99851    226.23306    -27.92699   -128.96717   -364.93103   -432.01946   -444.64312    -185.8407   -243.61843   -131.24448     78.96255    433.46592   1125.06603   1850.73698   1911.35729   1750.72506     909.7818    476.12321    205.73911   -201.94812   -132.20845   -197.87742    -89.93034    -29.30839    -45.55817    -73.75193   -142.07148   -354.87041   -265.05246   -267.09193   -299.78623   -387.39978   -370.36771   -169.92845   -394.95293   -398.90772   -449.03924   -176.09079   -235.71167   -126.66108   -207.30896    -97.14253   -255.37027    -97.10181   -201.42671   -193.91555     54.26835    339.92474    572.81109    667.38527    374.42231    206.40184    205.70958     -1.32997   -114.69746   -252.27484    -243.6247   -144.03573      2.34339    386.37147   1071.90667   1744.85183   2019.82971   1845.53555   1431.03779    732.03133    414.57624     57.77481   -237.52669          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       39.00   -175.14019   -327.63851   -301.88279   -264.67553   -193.11879   -586.19149     -23.1181   -479.77192   -388.74332   -142.55155   -487.85491   -361.90232   -208.31642   -439.87238   -403.36576   -450.15548   -463.31259   -711.33556    -392.4472   -274.73529    -108.5218   -348.66534   -265.97805     23.39442     57.84759   1034.57645   1595.66433   2235.95984    1863.0727    1050.4153    -76.76495   -313.76065   -291.60913   -288.95174   -377.12615   -329.91225   -348.12016   -131.50913    -293.7828   -316.21669   -397.95068   -293.73993   -407.12142   -407.99379   -253.66963   -395.66459   -425.56879   -370.31333   -297.96597   -284.75394   -432.20445   -375.99008   -304.15946   -242.98737   -395.28585   -294.65093   -231.49402      -7.7283     92.97391   -120.99173    -79.94146   -256.32424   -311.09334   -270.41927   -281.72922   -418.10778   -275.19833    -278.8859   -120.38205    281.52009    637.41885   1393.53346   1810.09458   1479.18574    725.22067     463.6212      75.1035    -95.37918    182.32421      0.49293    -19.18726     -90.0155   -170.87359   -159.72287   -126.35387   -205.62364   -310.10126    -122.0408    104.03617    511.09241   1287.72666   1911.12566    2120.7671   1505.24816   1052.65315    311.99223     99.82972    -46.04144   -339.88914   -234.68021    -97.22576    -61.19207   -144.91275    -81.44811    -64.93809   -207.86336   -333.30438   -309.52495   -189.08651   -253.11747   -400.06674   -371.38685   -332.03982   -244.26721   -341.74454   -236.90454   -141.85054    -82.68821      0.86275   -185.61252   -232.22604   -201.81316   -272.46218    -82.84921    288.80034    347.76756    601.80461    692.55908    622.59874    284.75096    -34.30789     -39.8286   -273.52539   -312.23764   -237.51588   -141.76871    -74.66479    380.19117    945.44962   1294.30314   1641.46248   1671.23694   1229.32953    533.49621    -31.26972          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       39.50   -154.22092   -247.47495   -273.98342    -317.9549   -322.41768   -506.26487   -114.18033   -615.24372    -106.7956     31.73972   -387.72741   -426.93921     -85.7803   -408.40259   -347.23927   -498.01765   -519.31132   -542.45169   -171.11397   -415.29541     -34.4873   -205.16717   -218.91611   -224.79695    401.22111    988.02913    1875.7586   2253.75641   1620.58785   1019.46439    174.54103   -392.21677   -352.70143   -277.17788   -293.69515   -380.41596   -220.48344   -166.09437   -384.73237   -356.19647   -355.18348    -83.92951   -390.07466   -387.67652   -340.65669   -357.17042   -465.63763   -324.07387   -436.25671   -512.27109   -484.69948   -392.47305   -312.48602    -319.5087   -411.36107   -358.86094   -321.75107    -80.33053      16.1194   -255.74171   -198.08833   -212.17411   -313.24615   -403.44884   -393.05161   -380.23167   -299.52435   -352.53297    113.80957    319.03864    863.70315    1332.2078   1670.76559   1118.99914    996.46197    233.13616    135.53703    -66.28168   -131.58368   -246.11832   -255.16573   -478.15807   -157.52311     -5.91468   -163.26224    -193.6886   -223.32332    -197.0462     58.41161    487.16859   1154.30976      1950.48   2109.10143   1811.21867    944.78858     442.7007     16.43174   -100.46615   -322.24205   -242.10715    -95.22267     -4.13084    -64.17325   -101.10915   -160.09715   -147.58128   -257.45986   -206.59088   -485.82705   -297.00075   -285.21018   -454.53625   -361.60339   -454.00461   -301.12681   -145.52182   -145.04471   -179.06674   -189.23208   -204.52683    -79.02161   -116.14714   -121.70887    -72.59672     59.90294    305.58209    569.36864    574.08975    421.91212      393.846    148.80528    128.21006   -191.19654   -211.97219   -249.74265   -112.65856   -122.18573     317.8175     843.8683   1271.06757    1489.3313   1032.51869    298.14343          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       40.00     -62.6512   -289.81099   -382.38977   -339.39722   -403.87104   -384.83728    -286.7513   -632.56935   -227.50791   -199.87582   -387.07743   -453.30025   -156.50633   -494.48715   -461.47395    -562.9571   -467.28604   -444.57027   -131.91117   -302.97726   -203.32533   -108.35387   -232.15786   -122.14007    628.70441   1026.74458   1965.65382   2009.60996   1502.85091     894.6183    127.79784   -262.12796   -328.45052   -368.05706   -459.96419   -451.42785   -241.58043   -156.71976   -170.94034   -484.72701   -396.23879    -103.1572   -389.86061   -449.39877   -301.08682   -339.16388   -320.90451   -410.33741    -398.4542   -410.51222   -372.52715   -410.70891   -264.05953   -380.29082    -415.4353   -329.64882    -12.90823    -210.4315    -158.3769   -250.76773   -268.78967   -206.85864   -199.16296   -376.12954   -388.95238   -330.93326   -359.36281   -178.05125   -138.35634    344.10522    751.08935   1403.50299   1580.50573    1520.4249     886.2366    330.03844    210.15838    -23.12267      47.1967   -167.04246    -56.14719   -161.66334   -305.91888   -249.76757   -167.21296   -208.95253   -364.83733   -202.59764    225.42227     441.8195    1087.0704   1996.03363   1877.70872   2017.92708   1181.03582    494.90337   -152.12248   -248.54957   -161.26068   -255.62934    -17.74234     -9.34577      65.2756    -68.51726    -162.6361   -234.51841   -182.90298   -233.24061   -239.87184   -283.96179   -311.52598   -210.57793   -333.25799   -360.89755   -174.24845   -355.43527   -165.61497   -173.76806   -149.68062   -110.32393   -245.91148   -198.73455   -154.52933     -25.4625    236.95072    434.08573    460.60709    682.43822    606.55722     436.1921      70.0024      4.86408   -246.58361   -219.46542    -234.6575   -156.27497   -216.86422    183.81221    714.71896    759.57021    388.61947          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       40.50   -267.66341   -292.05717   -570.98497   -283.18878   -325.38581   -381.94144   -451.14769   -577.74444   -353.49182   -348.11796   -336.15382   -447.71926   -411.16595   -448.22536   -590.79923   -239.68796   -374.93223   -389.34643   -207.87555    -163.3714   -201.67345   -239.62969   -322.91359   -148.42986    370.56358   1140.02001   2062.13173   1964.69071   1552.58687    776.25148     98.48664   -229.09211   -186.76577   -402.64891   -439.05697   -314.66704   -485.82636   -345.74755     -5.91432    -359.9604   -477.43993   -220.02685   -296.61352   -352.96593   -324.53861   -349.68711   -378.33819   -416.94846   -320.80741   -510.27886   -367.91746   -289.85121   -445.37024    -424.4463   -384.48298   -306.32001   -263.08381    -339.1494   -191.80379   -282.27335   -417.69486   -330.14398   -257.31756   -214.43164   -447.04533    -339.0881   -265.55924   -391.52034    -24.30422    312.42316    871.19406   1580.59002   1731.76923   1199.90301    724.64801    161.02375    -91.71442   -147.35624   -141.27303   -219.89073   -260.00034   -380.60217   -176.67885   -113.10714   -294.51954   -573.84169   -164.48597    -186.5681     77.93376    539.16971    964.20027   1711.45774   1878.35421   1771.78182    912.82273    328.59266   -223.87751   -215.95232   -437.54608   -370.11562   -156.04499    -91.22168     17.86402      -5.8377   -177.79671    -293.7141   -253.72145   -356.36603   -279.69683   -475.13621    -167.1216    -207.1674   -501.90741   -201.26786   -316.87062   -290.62457   -223.36937   -292.49341    -28.83711   -109.14586   -260.31496   -323.89743   -147.96909    -76.05716    126.59693    427.29577    724.75293    656.32151    547.14872    411.45156    -12.42452    -48.55298   -180.36524   -189.65146    -289.1207   -292.23046   -153.29409    -44.72655     81.95991          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       41.00   -452.38251   -253.90294   -601.98359   -160.30011    -180.8408   -452.84878   -495.97039   -609.78083   -389.20033   -433.45635   -227.31091    -415.5716   -335.28724   -549.85163   -509.27422    -203.4386   -465.64492   -276.84954   -282.09016   -230.33036    -55.52618   -424.67335   -229.07317     80.47751    372.37771   1064.01233   2031.64489   1833.16201   1796.17269     745.8035    341.30049   -212.88102   -225.21478   -584.17465   -293.62124   -418.80032   -357.12327   -247.26737   -294.41208   -462.71946   -477.34988   -349.32652   -457.87756   -399.53875    -311.8087   -280.86929   -339.42649   -304.19858   -403.70938   -542.90225   -553.81358   -432.96821   -332.86014   -163.76688    -404.3535   -197.56562    -201.8183   -257.91491      -91.169     38.25296   -231.32901   -147.71701   -395.01858   -336.08874   -323.08512   -219.68748    -357.0044   -502.58712     -73.3194    182.10286   1000.04455   1563.62383   1735.27889   1172.10685    664.86076    384.76818   -106.97519     -25.3461   -242.74347    -214.1198    -155.9991    -267.8116    -522.5118   -378.92782   -452.78895   -316.33313   -218.88644   -130.10569     53.53498    556.26333   1350.63219   1750.48862   1885.59873   1442.88134    880.46122    271.13734   -227.12452   -120.85536   -418.63471   -367.10928    -78.47116   -125.32293    -19.57138   -271.60758    137.83411   -311.85601   -173.10092   -322.22934   -290.45866   -368.32972   -365.57587   -309.93018   -378.52593   -328.43193   -268.79078   -233.48772   -267.69417   -138.80184   -161.20605   -138.67687   -270.85662   -214.87715    -218.9281   -239.46505    166.43906    310.19508    458.37521    625.04207    570.32321    312.28367    183.12683    -40.48806    -14.10414   -225.36917   -249.04617   -206.15706   -305.16942          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       41.50   -616.71724   -254.71631   -540.73036   -146.37772    -71.68249   -431.01229   -597.67548   -609.31155   -303.62701   -514.18484   -256.07099   -318.89522   -392.68415   -578.57119   -378.90871   -387.16706   -419.06642   -251.23832   -349.30447   -329.64072   -191.32351   -428.01813   -251.93433     21.03789     354.9938    833.13238    1716.7538   1670.56995   1631.71933    837.00835    156.22955   -388.33999   -404.39889   -566.69168   -189.90159   -260.84491   -227.17911   -116.38013   -358.42361   -445.36576   -336.07748    -212.8007   -536.86589   -359.76005   -546.62859     -272.287   -453.86166   -374.04442   -288.92931   -451.89839   -308.12473   -387.72703   -224.12773   -343.60703   -368.73731   -305.51215   -288.21496    -56.65657    -20.53611   -154.30211   -198.35903   -114.50957   -240.18299   -289.37132   -331.25717   -229.39635   -348.93384   -279.72063    -72.79657    114.99173    911.86387   1406.74475   1702.79424    1316.1599    963.65884    336.88533     -0.52767   -192.68786     -8.03973   -431.02125   -237.85105   -188.84753   -190.41328   -264.63018   -380.70429   -204.08956   -388.80038   -268.73904     97.22831    581.86851    939.54393   1562.52573   1877.75882   1380.29197     884.4265    369.46725   -193.08232   -369.87212    -443.2807   -154.52322   -228.83058    -33.74922   -185.26703    -43.95598   -201.95336   -269.40758   -153.85293   -322.84166   -276.35855   -258.61342   -407.34372   -388.77887   -323.85349   -298.86484   -322.91037   -358.37248    -126.4719   -256.55199    -261.7423   -163.44123   -132.21086   -192.93142   -297.07464   -225.34413     14.23538    249.08089    437.70368    405.19269    560.36448    379.68811      87.2615   -129.48414   -242.28311   -359.83627    -276.3022          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       42.00   -693.31338   -337.12448   -479.52258   -188.00936    -36.89527   -349.20972   -618.67683   -589.23884   -271.77416   -423.60038   -275.79679   -425.45886   -453.66715   -294.77589   -193.76934   -462.05281   -230.45715    -329.4148   -331.87324   -358.29393   -257.12962   -275.82423   -225.53532    -33.49839    350.35084    887.06073   1655.56289   1560.46046   1389.94303    602.80289    -34.67009   -291.15392   -439.99379   -379.42514   -353.11026    -68.49977   -389.03708   -352.32522   -154.90973   -457.26642   -243.27042   -347.89395   -391.64277   -403.12067   -571.81038   -383.99602   -160.50323   -450.52854   -593.18657   -555.01062   -383.41567   -248.59073     -317.044   -139.27709   -334.68652   -176.60541   -222.06675     -87.6544     -93.3242     -6.75601   -244.92775   -283.54807   -310.39511      1.63241   -289.93515   -363.73823   -495.88466   -240.93367   -167.19286    248.40774    760.37531    1507.8215   1289.85699   1419.26872    884.07399     380.4351    -66.30282   -160.30606   -228.65584      -437.21   -223.40503   -219.49371   -310.19317   -293.38622   -213.08779   -210.45362   -364.14347    -55.07638     83.43273    383.54599   1011.78989   1631.30103   1865.71584   1593.49702    723.54633    294.80347   -239.56097   -315.74626   -385.01695   -315.13514   -184.08566   -132.05434     67.57214     74.87079     14.26468   -172.57589   -374.30223   -265.51781   -345.68659   -413.51873   -342.32803   -241.60902   -271.41529   -446.11364   -510.35499   -192.00755   -319.43916   -253.90746   -170.34686   -236.04271    -91.57782   -205.88869   -116.62951     -8.71383     40.98325    213.64098    423.81357    521.97315    644.32709    318.21036    105.20285   -243.41028   -429.07229   -451.78663          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       42.50   -510.10477   -441.75004   -331.92755   -316.96048   -195.65722   -233.26703   -444.40392    -605.7371   -282.21812   -294.48046   -384.04907   -590.30854   -427.45505   -174.10231   -323.64777   -273.10993   -217.72671   -286.03857   -103.68204   -468.38201   -205.09984   -186.00093   -295.51106    134.20143     108.4394    789.15488   1628.85502   1666.24373   1243.00711    798.70447   -104.83187   -285.37672   -332.78478    -467.1216   -323.82326   -457.00142   -501.75381   -205.97889   -305.05307   -316.68153   -337.83738   -530.85019   -354.00819   -406.95116   -489.16668   -317.27565   -370.10029    -600.9282   -288.84008   -430.37152   -272.22493   -408.88025   -237.85253   -445.67778   -425.08887   -251.93638   -219.31176   -102.88451   -104.39551   -170.60781    -259.7906    -132.4518   -139.11642   -266.76579   -331.12896   -255.57261   -318.61163   -290.11455     21.13457    182.59105    762.02237   1320.56909   1562.94257   1481.25603    849.12273    273.14174      20.7003    -28.92585   -453.92075   -178.75531   -322.83213   -208.81865   -316.42245   -356.99981   -322.84084    -91.89619   -240.47864   -144.50734     -98.6983    457.60617    949.51308   1486.42688   1693.08442   1430.85733    681.15306     53.53661     75.44087   -257.81496   -545.36244    -67.68684   -232.29378   -132.55923     30.18504   -149.15499   -164.29288    -210.2538   -365.75284   -267.35136   -238.96519    -283.7131   -573.25475   -282.28935   -307.22618   -365.87121   -278.01817   -412.41125   -279.21176   -277.48723   -185.44171   -284.94141   -178.23482    -86.20001   -219.92745    -90.56041    -34.75964      288.229     286.6703    422.58983    501.84537    281.93466   -192.71075   -257.75652          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       43.00   -362.57869   -568.64391    -350.9529   -503.90072   -464.33084   -184.82618   -468.32535   -572.83894   -317.21441   -251.94301   -496.48813   -496.17585   -520.46695   -233.56314   -403.83552   -149.57883    -298.0302   -189.15131    -64.08001   -357.43459   -226.04865   -348.72297    -495.9519     47.95676    119.80105    797.13834   1630.37456    1570.8263    1026.7613    587.64862     50.88144   -188.82578   -210.69601   -633.62601   -271.07474   -364.94813   -130.09931   -403.87962   -352.79647   -542.37169    -198.1035   -291.73715   -395.23228   -574.22127   -346.09625   -441.29875   -425.68733   -220.38687   -232.21085   -327.37581   -452.46449   -536.84176   -222.13831   -308.27298   -355.66918   -269.83474   -221.56995    -46.06318     -75.0541     27.25627   -318.49129   -334.72778   -358.11568   -433.82825   -402.23041   -441.87604   -313.35055   -265.58634    127.23267     50.99504    559.80462   1531.03285   1685.44234   1369.82865   1023.44777     338.9853     45.92537   -117.75687   -166.30207   -318.48208   -295.61531   -240.23469   -313.05623   -333.62628   -302.80727      -212.91   -254.21457    -47.23032     94.51324    261.21232   1069.57073   1401.45287   1897.60316   1527.30463    695.08008    128.93041   -161.44043   -275.85488   -353.51202   -276.88407   -306.22131   -262.53114     -3.05569   -188.87493   -313.85244   -311.30272    -99.24205   -167.51355   -232.77367   -479.88678   -313.42254   -310.45066   -332.12679   -295.33069   -381.93964   -380.11411    -250.1854   -132.86046   -186.91031   -187.69604   -253.18609   -226.88846   -168.66518   -238.17524    -63.07558    309.90102    410.57048    348.96013     354.5847     43.36385   -200.20428          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       43.50    -261.4729   -482.38193    -426.7339   -585.43606   -642.62275   -175.91741   -577.81003   -492.17602   -355.85642   -327.68591   -443.23529   -337.66326   -439.55823   -201.86589   -376.42003   -140.35387   -204.24289   -233.02969   -485.01663   -443.86858   -404.41722   -340.64779   -375.03472      5.71257    281.18816    912.50625   1368.84354   1759.07751   1283.31308    801.88235    148.14534    -82.72763   -119.10435   -545.49317   -439.09972   -335.55789   -111.78163   -607.73667   -413.75153   -381.00365   -328.70514   -472.34165   -594.60811    -462.4435   -397.97829   -400.72866   -314.79331   -260.06421   -287.85598    -267.2552   -351.35415   -346.93737   -350.59139   -245.73412   -179.59683   -414.72336   -274.92542    -238.8249     23.19546   -116.64736     -77.1571    -323.7594   -312.53369    -406.0763   -285.89197   -256.01169   -237.10327   -331.35283   -205.78105    123.24678   1011.27136   1309.37608   1467.65911   1570.16383    980.55422    452.67288   -103.25477   -151.14138   -295.00831   -459.18944   -344.24555   -364.19369   -307.08351   -265.63914   -401.06966   -157.45867    -208.1604   -236.17115     161.9585    635.97381   1011.76022   1582.82021   1754.52649   1379.40171    689.86921     390.4692   -217.13678   -327.47867   -242.45279   -309.00232   -335.62706   -142.49561      -87.874     98.50552   -168.99864   -227.15502   -201.36542   -268.28269   -422.72579   -390.22191   -391.70095   -270.69077   -328.43189   -301.94793   -228.66162    -432.2968   -212.61371   -305.55222   -299.60943   -203.20454   -213.51667   -201.53353   -250.72025   -109.01365     27.20895    181.00345    275.17668    399.70199     61.33224          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       44.00   -215.19134   -393.91883   -535.56538    -476.0131   -612.01223   -169.69948   -423.96563   -427.76601    -287.4734   -525.38313   -428.09961    -491.0174   -406.17367   -175.46255   -360.50355   -185.21344   -295.31656   -249.33454   -429.44783   -402.92725   -573.70068   -375.40277   -347.92952      21.0385     244.1809    839.38931   1310.65822   1690.29329   1380.63666    686.42653     38.46415    -86.46835   -158.33316   -418.64135   -355.26494   -515.87653   -270.25389   -497.96944   -777.17254    -354.2658    -426.3126   -339.54147   -397.11718   -458.28882   -372.88972   -328.24634   -464.11277   -483.81276   -356.91805    -363.6964   -373.83952   -444.36793   -335.57482   -330.31628   -560.74745   -390.65719   -142.86121    -12.59157    -40.31334   -319.06936   -194.76647   -184.43477    -200.7221   -364.85184   -231.45459    -322.6032     -170.266   -329.12268   -263.63852    303.27037     912.1496    1266.2675   1664.65088    1549.7451    977.22187    271.97665    -90.23317   -106.49654   -203.77506   -281.05412   -433.42391   -191.70311   -600.16421   -277.71051   -171.91372   -231.04149   -177.82198   -409.07767     79.69496    338.46128     826.7059   1332.21742   1589.90207   1459.26757    868.06657    389.66932     10.43425    -92.70794   -303.68735   -197.66052    -300.3471   -149.70757    -60.67467    -32.49438    -28.86216    -22.26593   -238.28297   -315.72008   -199.23209   -447.89982   -330.17366   -351.23012   -367.55167   -345.65356   -444.90431   -340.99468   -282.54085    -73.20084   -204.61027    -64.66929   -208.85748   -357.09561   -218.80144   -156.56114   -159.18015    145.51279    -81.01648     -5.57852          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       44.50   -251.72515   -317.49712   -335.35444   -338.48666   -426.09308   -225.48288    -253.8673   -322.36374   -249.91144   -585.75993    -284.4608   -624.82151   -434.36861   -335.38735   -321.57623   -199.03215   -444.54336   -214.28477   -276.47623   -301.63742   -469.74224   -432.67895   -173.92713   -140.37756    228.26963    819.38162   1439.77144   1640.86274   1127.65121    521.86247   -124.08054   -370.89076   -253.80208   -349.14758   -495.98911   -400.25196   -407.87084   -495.55402   -333.45601   -443.44469   -471.01054     -587.063   -512.47669   -370.37132   -540.46314   -270.64778   -422.62834   -349.51185   -308.52622   -309.04197   -336.94392   -504.09668   -410.86062   -419.60389   -236.51678   -356.86764    232.13207    -45.45283    -85.51096    -50.35474   -181.06726   -144.96615   -245.75959   -325.71947    -297.1572   -422.06922   -230.15275   -156.25058     36.69371    147.85814    559.39977   1402.37288   1536.13873   1432.89396   1017.24754    518.22368     88.64912    -68.33161   -142.01853   -247.41902   -155.39683   -337.60775    -408.4727   -358.83144   -149.47853   -219.23495   -286.73731   -306.03032     60.64595    450.76798    785.86822   1372.71854   1508.40261   1347.83917    922.67368    270.38101     38.14679   -140.54859   -167.96469   -313.66653   -187.70716   -125.80507     15.45709   -172.45235    -53.77127     -38.2158   -126.38803   -313.24012   -223.63735    -300.6008   -199.24287   -343.71304   -162.13176   -402.00428   -444.62153   -260.59152   -222.30529   -165.82054   -214.06919   -162.49417   -141.78275   -333.83071   -418.79745    -47.10165   -176.99948   -229.27643          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       45.00   -258.36682    -245.8317   -113.71709   -178.78344   -264.58203   -309.64298   -118.37043   -259.02892   -279.79634   -488.42387    -39.29128   -447.24502   -242.08757   -568.75114   -131.99578   -197.68336   -443.49783    -251.9471   -304.32162    -411.9851   -293.54602   -306.26506    -201.0304   -181.67162    141.55529     977.5484    1389.2879   1511.13536   1173.11381    595.53978    -56.68402     -386.363   -315.68149   -256.16704   -423.57477   -499.28811   -179.38122   -535.49744   -559.53325   -491.40574   -397.76044    -398.8827   -406.75159   -284.25405   -462.05236   -293.81242   -462.78014   -500.21099    -499.1618   -413.44408   -385.98932   -472.04539   -376.34149   -414.27684   -479.12161   -230.98165   -258.20422    -54.79167    -34.53473     -4.09763    -66.25398   -265.71135   -322.75619   -233.20264   -230.57656   -338.61201   -328.88907   -291.09163   -266.61182    418.79413    684.61595   1218.28718   1488.42934   1544.74155    918.88299    449.33581    184.43531    -32.46602   -380.04375   -205.13307   -100.04436    -49.89144   -153.84111   -316.91195   -354.38798   -284.74125   -273.22461   -242.06086    126.28803    362.16665   1058.98889     1636.006    1568.9449   1561.28449    881.24794    343.94507     49.18032   -238.55426   -213.77708   -300.56334    -286.7313   -146.42797     26.12298     -78.9225   -233.54417   -179.07308   -316.44342   -262.45299   -295.77276   -363.53025   -340.32776   -300.54911   -392.16742   -430.13423   -358.21148   -337.69561   -316.36917   -185.77227   -253.07027   -223.79552   -170.18265    -400.4025   -238.85953   -393.55699   -303.98265          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       45.50   -218.29923   -285.65591     24.04393   -167.52815   -155.37495   -301.68129    -71.39864   -269.39595   -378.04549   -608.56772    -14.63814    -325.1651   -235.24812   -444.53255      3.25525   -395.58244   -448.28094   -342.95416   -447.82032   -554.99468   -275.31504   -205.41271   -204.45396   -210.42664    208.23427    969.95525    1490.5496   1501.42645    1191.5677    496.35788     -34.0453   -356.70642   -266.82707   -144.43154   -462.82295   -421.59988    -80.47804   -404.94781   -485.62534   -483.86308   -507.13882   -548.50195   -482.87048   -310.15632   -324.98431   -417.09788   -406.56635    -572.7527   -429.29325   -304.41062    -382.8694   -408.80126   -368.71975   -368.13968   -454.59341   -257.32044   -171.82418   -181.06228    -47.23127   -155.34436    -36.39974   -281.98106   -368.16726   -362.63789    -167.8553   -272.86488   -305.09793   -139.64566    -125.1284    409.19452     752.5035   1465.46695   1521.78538   1473.15194    836.12876    520.11727    -49.11343   -137.24076   -146.46483   -247.75707   -393.62872     -163.783   -390.60743   -201.17089   -344.96606   -391.30183   -240.12124   -120.30898     25.04285    361.46264    799.07309   1549.72614   1495.75103   1349.35505     923.5699      664.438     46.69725    140.83637   -256.84553   -262.03812   -206.48668   -154.56307   -111.85661   -174.89224   -247.71768   -246.33716   -314.67534   -275.55481   -284.30562   -291.61347    -457.7453   -345.51513   -322.00977    -293.1292     -353.537   -285.61648   -311.98535   -187.01818   -235.48422   -187.18373   -322.23842   -341.01086   -451.06937          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       46.00   -161.82277   -334.18739     45.95922   -246.76716   -208.62323   -261.48822   -179.88743   -272.60776   -373.00971   -499.62525      3.91782   -312.14807   -337.72579   -574.41887   -219.30945   -536.45894   -353.58511   -468.04242   -262.75681   -379.67285   -102.71297   -344.83974   -209.43578    -154.4451    261.08967    699.86972    1360.2866   1557.38789   1270.81783    416.72244     69.81265   -298.05672   -143.39168   -339.02517   -401.06417   -314.81828    -450.0724   -268.90074   -479.36368   -429.57891   -482.69656   -598.63536   -391.16996    -408.7909   -258.92098   -225.01849   -422.08741   -249.88237   -442.06099   -465.27162   -388.06198   -296.51068   -385.42416    -465.2432   -318.77689   -397.17627    -307.4626    -82.79334     59.66541   -286.42974   -326.66513    -189.1126   -343.67899   -347.47016   -327.15287   -187.56109   -238.93368   -127.11953      -8.3521    152.02952     712.7882     961.4327   1596.45025   1602.66481    747.23264    215.94819   -169.02427    -76.87631   -293.11701   -244.95525   -448.34517   -509.71934   -136.05889   -222.26477    -27.55757   -349.48177   -326.42068   -182.77218     97.48197    307.39795    925.20063   1362.75901   1616.66507   1487.12834   1014.15563    357.48288    258.49104    -55.34183   -258.23932   -175.81466   -243.47874   -186.99381    -28.08829   -124.23058     29.55679   -124.96839   -346.65502   -286.39252   -352.39112   -471.46662   -273.45086   -304.28335   -330.01118   -301.35985   -404.04253   -475.19204   -323.49012    -294.0896   -179.82178   -304.00564   -109.35055   -339.84962          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       46.50    -193.4129   -391.41699   -123.27387   -334.27528    -276.4245   -247.21335   -410.87007   -266.40876   -278.53487   -341.10327    -19.31031   -361.85012   -354.45461   -572.75589   -242.65966    -430.7398   -288.14941   -479.64577   -221.89476   -339.87156   -289.03916   -425.33481    -231.4093    -73.55399    325.94567    665.42281   1169.71328   1273.87998   1150.93931     720.0184    154.53852    -362.1431   -297.66109    -317.7367   -261.78219   -215.83039   -336.26562     -1.91598   -365.99267    -341.8485   -523.20342   -408.90124   -535.45619   -566.25972   -515.62398   -426.18435   -333.73733   -216.71904   -302.50834   -399.47191   -264.00549   -573.19246    -424.0645   -376.58638   -162.94148    -251.8403   -216.03082   -236.19861    -50.32712   -132.38746    -45.23765   -341.09992   -125.71504   -318.09573   -374.22254   -261.06901   -338.58738   -196.73673     -93.2013     90.42604    802.88172    1237.3442   1367.09098   1481.80469    922.88058    469.04108     82.67489   -147.46591   -209.15726   -364.74482   -176.43672   -125.88735    -27.46628   -136.34202   -211.39815   -468.13274   -221.37993   -179.42657    104.49926    270.90527    848.66612   1468.06647   1513.06168   1446.11822   1132.51613      364.646    -12.92902   -328.68195   -252.44136   -268.74135   -346.21205    -52.48914    -146.4049   -318.77161   -263.83422   -178.51849   -252.07636   -307.06074   -368.49427   -356.31712   -212.14739   -339.03593   -344.98711   -360.39879   -346.77003   -342.61329   -446.26493   -292.11235   -231.69823   -331.53241   -442.82996          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       47.00   -273.11067   -439.43384   -257.10135   -399.15156   -421.10123   -290.52928   -455.87724   -328.79001   -323.43895   -319.98186   -147.96537   -411.76758   -334.38554   -255.08534   -194.92906   -449.32739   -332.82139   -370.32933   -219.49334   -367.53021   -243.55785   -376.19035   -286.78495   -289.36583    261.57961    722.23318   1172.04329   1256.66952    1394.5122    570.19984    147.03084   -168.98224   -262.26868   -244.22603   -304.84863   -273.39631   -453.79196   -331.73215   -167.22195   -397.61394   -475.93208   -238.50908   -462.65263   -351.52344   -483.88004   -368.18169   -315.10831   -326.27514    -95.75964   -356.88696   -544.15305   -241.71108    -330.7193   -566.34417   -324.49457   -462.18057   -226.14605   -254.02189   -225.20391   -116.50327    -79.47947   -232.88563   -121.53429   -143.60219   -312.74612    -35.19357   -321.49325   -365.57805    -83.40665    136.69109    796.98693   1173.29509   1619.54617   1209.42721    879.44056    396.59789     78.14891    -80.93137   -242.56343   -119.43012    -282.0775   -141.16119   -210.22536   -223.87586   -284.94695    -223.3308    -300.1322   -267.12367    135.11239    506.65289     854.3391    1244.8777   1556.06237   1357.03346    962.98779    603.70848    -213.2238   -183.05814   -300.54201   -317.98243   -151.79419   -177.19243    -52.33417    -90.23668   -119.42505    -96.97128   -224.57205   -337.41637   -346.22761   -311.98167   -369.83665   -239.83071   -400.67191   -363.34844    -285.3599   -233.18914   -279.60515    -133.3274   -303.39719   -484.51921          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       47.50    -383.7116   -407.54547   -350.49442   -433.83593   -588.13324   -375.14707   -390.09356   -273.06342   -534.23501   -309.49805   -296.76754   -507.21679   -338.47677     -7.53978   -319.87789   -474.06046   -287.43187   -377.23045    -207.4853   -511.32349   -273.64106   -349.67207   -270.95033   -240.47382    148.56142    893.93036   1225.36978   1177.17449   1121.09238    427.15119     -38.8973   -138.76078   -341.92344   -387.81193   -240.69871   -281.77411   -321.04879   -333.01781   -502.13225    -439.0984   -428.62534   -227.72712   -323.04546   -368.72387   -360.96461   -509.76929   -489.29091   -383.24402   -488.20539   -283.11718   -571.20637   -374.94927   -336.41614   -547.42823   -347.55171   -444.18625   -248.33877   -156.04954      66.6702   -146.09168   -355.51044   -291.39167   -257.85239   -241.69404   -406.83926   -154.39006   -291.77485     -229.361    -26.41572     85.64417    870.24278   1290.94084    1544.8653   1299.39136    899.36056     427.1283    -41.76281    -96.01674   -425.49635    -80.57433   -266.56054   -255.08759   -487.76573   -275.51559   -465.62998   -212.72071   -428.19376     -174.735     45.40942    399.08361    872.39179    1386.8026   1403.00227   1421.39723    830.32223     498.5929    -97.41724   -125.92193   -276.27448    -103.7093   -149.29774   -214.61051   -218.06147   -141.19541    -85.55906    -40.46148   -215.93568   -208.30873   -324.97797   -302.54131   -306.95597    -375.5869   -364.03586   -423.47475   -364.25889   -274.20357   -250.77859   -358.11772          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       48.00    -486.4594   -347.26535   -397.22042   -510.59819   -734.28797   -443.11586   -304.37124   -151.15879   -658.50696   -150.96696   -426.30356   -456.25842    -345.0068   -299.33809   -456.76966   -362.61955   -201.91638   -363.59153   -203.43457   -458.40372   -367.80399    -291.6647   -166.51885    -41.67251    169.93122    784.57452    1342.9363   1298.55618   1037.71176    480.82738    -22.84813   -162.06546   -314.33643   -279.45012   -392.13692   -410.37857   -253.30732    -46.60029   -444.45489   -424.27011   -205.71725   -439.51021   -446.78689   -451.55396   -394.08071   -601.21373   -441.10287   -331.21454   -457.14002   -431.62107   -462.17623   -285.24755   -366.38705   -544.08544   -273.14805   -362.05303   -247.73382   -146.45905    -96.65036   -171.44049   -142.58972   -224.71502   -417.57969   -135.89986   -357.71322    -246.0449   -357.37197   -259.52478    -19.10892    159.12831    477.54538   1209.58615   1471.12641   1198.83146    907.29127    514.84009     43.12479   -308.11304   -219.36447   -193.17481   -316.54788   -131.66395   -157.51499   -523.51113   -220.69719   -340.68784   -286.52457   -165.40759    -64.67746    321.25085    565.36123   1337.96909   1534.87204   1226.70026    955.57932    379.22735    -50.82458   -338.31942   -346.04912    -66.65198   -421.51305   -286.78852   -106.97985    -197.0204    144.77552   -255.66828   -421.96225   -297.84635   -167.82516   -326.85974   -397.27739   -310.70634   -424.60815   -264.53928   -311.65034   -320.60512   -398.92257          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       48.50   -457.33036   -258.66733   -385.31543   -482.94461   -687.30758   -530.13762   -231.95553   -102.42206   -395.53772    -56.04053     -366.091   -374.71352   -275.36427   -411.56214   -522.61169   -283.51931    -316.3644   -306.06844   -238.06896   -469.11577   -405.79306   -214.68697   -306.76983   -252.23181     126.4084     580.3207   1430.11433   1215.32551   1022.55678    510.51618      1.59592   -184.25379   -311.28337   -360.55566   -344.01976   -383.55573   -271.80238   -381.19022   -460.53677   -349.83318   -334.72476   -286.01239    -613.0987   -419.13506   -387.08426   -412.56048   -448.20978   -625.50036   -328.74103   -405.89315   -453.25983   -514.81684   -273.83334   -479.54791   -230.60854   -346.64341    -219.4032    -275.2232     40.62228    -81.54969   -127.79035   -246.64866   -408.70722   -308.74021   -279.87252   -285.13065   -296.17022   -304.24921   -205.59778     49.53081    675.67457   1080.08378   1378.12211   1472.96707    995.59297     307.6495    120.13872   -284.91693   -120.26029   -283.40834   -325.06312   -624.90298   -430.81213   -216.70239   -172.25446   -303.81841   -321.58443   -379.63304    -95.24201    545.88955    700.77539   1358.97709   1503.66269   1264.28261    851.75243    264.86206      38.8416   -242.42803   -356.76773   -229.24681   -307.27784   -127.35993   -180.41743    -74.76346   -169.58017     32.46581   -220.99429   -222.00083   -382.11359   -244.71416   -371.52138   -353.41876   -365.32586   -383.79291    -348.5363   -350.86632          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       49.00   -425.88971   -142.20925   -389.19489   -298.71523   -530.08919   -571.88853   -237.97749   -174.31243    -204.2174   -217.30228   -327.54429   -371.60083   -371.00614   -398.76336   -677.74631   -332.13559   -340.49796   -304.54373   -513.30938    -493.4654   -253.88383   -204.11118   -394.36106   -202.27013    152.71668    620.86491   1480.53152   1138.91524    989.09383    581.75404    -157.6633    -155.3978   -119.65853   -381.49836   -386.25806   -495.56317   -409.93508   -404.25464    -408.3772   -373.59898    -474.6896   -366.60459   -550.92181   -407.16105   -255.67288   -347.23825   -496.71193   -506.52927   -503.36416   -352.66893   -412.29573   -351.44678   -460.63941   -345.10641   -315.85383   -198.12452   -128.91705     -1.73589    131.69271   -257.39144   -168.78182   -190.30188   -407.11386     -128.976   -254.54726    -244.6642   -310.99101   -216.02067   -333.20628    -22.31311    765.68212    880.89354   1398.87284   1226.72228    969.14879    495.12829   -183.16842    -53.41632   -142.42507   -498.05252   -452.97024   -229.64194   -370.60551   -490.74883    -95.44459   -383.58968    -460.7504   -150.14447     68.04623     68.90149    812.89558    853.30273   1503.04278   1546.81556    993.01725    447.93611     31.32131   -320.88836   -192.27237   -284.19656   -276.28222   -256.99609   -211.34189   -218.98122     52.56099   -183.67735   -390.69677   -265.71374   -393.08246   -340.50264   -356.68126   -249.88164     -353.469   -444.40542   -382.00843          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       49.50   -394.07591   -123.52522   -352.76612   -173.68092   -392.42756   -464.13402   -321.26061   -177.29717       -124.0   -256.33279   -410.38306   -298.09923   -533.50753   -326.60686   -540.17612   -463.72034   -307.32417   -317.16932   -556.68153   -720.64131   -205.88468   -114.14791    -306.5607    -154.3053    273.57204    526.62684   1339.78098   1242.01773    990.34075    264.59641   -223.48208   -155.08945   -315.00038   -446.07715   -469.81425   -409.10214   -404.89055   -134.74392   -324.29339     -377.011   -379.43467   -378.51598   -443.59515   -420.35876    -348.2198   -461.52831   -235.85163    -384.6794   -473.02325   -224.18191   -409.98794    -345.4654   -150.90142   -349.05926   -532.83975   -113.15872   -162.49131   -210.55601    -49.53744   -173.70047   -194.46659    -189.0947   -279.06238   -284.44058   -186.71624   -360.41457   -388.67575   -117.78887   -107.32549    331.34932    836.44584   1033.05415   1675.52459   1309.40905    793.96583    436.29356    -34.48249   -251.89305   -118.60206   -267.25089   -374.07515   -178.89124   -457.46608   -377.83502   -420.47078   -256.80977    -265.0831   -250.50217   -181.44175    459.32258    716.39613    1381.4238   1586.51233   1468.50583    1077.2373    546.18941     74.71029   -430.45228   -482.87489   -334.59365   -397.89106     58.53061   -139.89191    -78.93317   -146.72087   -124.72357   -224.00745   -360.49573   -347.47429   -321.59786     -364.362   -438.23639   -257.70107   -435.31395          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       50.00   -361.89768   -129.60721   -187.14195   -229.46058   -326.18084    -443.1088   -223.37894    -39.49411   -247.04361   -225.34366   -494.41188   -362.79821   -543.76106   -262.47843   -279.78353   -361.58239    -409.3572   -339.47442   -338.60517   -535.52838   -248.92738   -362.19968   -188.09705   -342.89995    288.42412    584.97543   1230.41795    1246.6317    700.25618    505.64749    -55.38188   -331.74235   -300.04774   -379.46655   -427.71284   -427.77018   -292.11145   -544.03966   -344.85365   -227.12643    -458.1793   -283.22938      -392.65    -186.5526   -421.66017   -525.35555   -430.51757    -298.6119   -332.08382   -512.02626   -447.58569   -339.78207   -485.43649   -363.39483   -484.33658   -247.47804   -112.25682    -48.38529   -373.07401     20.99552   -373.46305   -102.02082   -330.15018   -349.02189   -230.77037   -300.01288   -317.52287   -123.77503     64.17444     154.8598    650.18832   1263.34989   1175.60609    1346.7654    737.37836    413.69202    198.70444    -71.37525   -275.48635   -211.89634   -207.14797    -411.1136   -364.16239   -596.18586   -353.97066   -457.89451   -215.52845    -221.8071      7.85911    286.29428    708.45662    1070.7309    1579.9154   1592.76773   1076.33208    649.41049    -41.92031   -208.00456   -127.90705   -275.25552   -173.03258   -153.98334     -110.856   -152.30893   -131.43947   -286.37717   -296.79617    -243.4331   -348.74265   -230.36595   -475.38498   -501.33955   -381.41995          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       50.50   -313.12684   -160.34308   -109.44315   -377.67924   -286.39652   -559.93476   -160.11576    -81.87028   -394.52259   -294.28006   -442.68012   -358.78037   -599.19091   -373.56939   -306.55461   -295.01362   -296.74986   -396.87057   -359.14247    -240.1284   -318.19112   -328.26707   -107.79384   -284.48471    106.16783    738.31461   1231.03818    962.84174    699.11132    529.64467     65.37425    -156.4678   -262.92124   -249.86776   -361.10966   -437.26643    -338.9347    -545.3121   -396.93442   -385.53209   -294.72697   -349.91468   -433.42049   -474.52133   -413.82758   -430.40796    -325.8368   -215.90106   -226.27469   -478.07387   -435.82451   -179.11364    -518.2153   -350.61603   -222.01408   -477.42829    -59.74845    -45.22578    149.81546    -59.66883   -153.55655   -294.39462   -272.12145   -344.51793    -233.4373   -325.93284   -454.41771    -92.10657   -137.80197    152.34488    696.28878    950.98846   1322.72732   1288.28462    615.63974    451.16615     -78.1461     -8.83465   -397.16317   -369.25296    -536.7118    -69.69019   -620.71253   -164.03319   -491.17094    -230.3776   -170.99506   -193.80393   -186.17909    459.86991    683.70335   1166.13145   1691.27309   1516.03994   1260.90159    853.20426    106.40331     73.86891   -121.18409   -217.55795   -184.49379   -223.37068    -155.8041    -63.15243   -351.07163    -127.7848    -300.2397   -305.52244   -373.98304   -480.06025   -401.57578   -416.04962          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       51.00   -340.25678   -379.84552    -139.4288   -443.65462   -268.72901    -610.6205   -225.87719   -286.59975   -397.78089   -314.08004   -431.40538   -348.37545   -533.18072   -491.05664   -562.00954   -309.15018   -302.77839   -340.33213   -361.18435    -348.2135   -350.78325   -253.80201   -258.27089   -152.55004    -203.3429    778.14911   1028.39088    983.72016    813.54466     408.8108    -81.43092   -182.70758   -260.29305   -477.40501   -279.82627   -461.28932   -509.86804   -549.82006   -383.35835   -489.03861   -326.51965   -391.90132   -406.64709   -347.84383   -348.57312   -389.76344   -523.26707   -219.33553   -146.35878   -435.75021   -417.04575   -337.57478   -430.74159   -448.63218   -213.74841    -36.30741   -215.56069    -97.57058    -43.12701    -88.06527   -117.50794   -320.22956   -245.82969   -342.86366   -163.95437   -270.32894   -270.89538   -110.99829   -167.43141    240.01585    700.06482   1058.62409   1332.04314   1407.64652    789.41363    246.75932     28.74747   -270.25123   -372.87595   -797.26188   -688.17875   -584.48482   -547.64666   -204.25829    -334.8995   -253.73182   -318.01431   -193.40824   -197.67705    166.65754    430.93013    916.74498   1485.83995    1568.7903   1256.29922    672.94503    194.52383      65.3456   -403.02148    -269.3527   -215.84614   -326.39825   -157.95279    -75.94007    -15.18673   -285.19446   -310.21974   -333.35793   -248.66449   -366.22495   -358.25708          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       51.50   -442.75901   -536.23003   -341.25867    -406.0381      -230.36   -587.96439   -246.73546   -395.54453   -346.85393   -261.99099   -365.28302   -465.22588   -389.46415   -481.23777   -574.04218   -241.16895   -230.12207    -359.6351   -406.60731   -322.67987   -354.09255   -165.76001   -242.42398   -318.08512    270.27015    668.80384   1095.84061    904.83956   1004.25807    304.78775   -181.71303   -322.49538   -310.84928   -411.42232   -334.68951   -503.77384   -453.93953   -482.85613   -252.29674   -420.37737   -369.31983   -314.35782   -505.42496   -448.12761   -425.64477    -57.23497   -256.93029   -293.93917    -445.2252   -421.60973   -651.68374   -350.24512   -354.11146   -325.32535   -311.94694   -389.83041    -85.60628   -242.96929    -122.8613   -205.49241   -158.54244   -185.96469   -167.75113   -420.14717    -317.5194   -327.29707   -234.14793   -170.08177   -228.95293    163.95196    367.96773   1079.29613   1327.29307   1443.19124    910.19014    273.79961     11.37935    -38.32109   -369.77988   -535.47492   -808.05317   -456.61828   -417.42822   -507.08107   -248.40234   -482.83877   -117.70252   -150.35599    -40.86231    189.76355    513.11802   1086.72829   1387.21399   1269.65748   1070.44354    692.19614     71.52754    -73.39859   -305.28511   -275.99617    -323.1619   -208.61113   -325.93735   -177.59391   -271.97503   -251.73638    -335.8115   -371.83467   -494.79788   -443.51464          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       52.00   -541.72149   -610.04025   -407.48641   -469.58825   -214.50143   -541.19165   -142.13748   -234.22032   -446.37205   -211.79122   -393.27593   -420.89892   -382.26898   -261.74916   -416.96422   -273.04178   -266.59015   -454.27231   -500.78191   -282.33647   -322.77501   -134.43836   -205.92595   -302.87979    288.71224    773.80988   1143.55716   1142.99296    972.27288    246.32135    -15.55934    -383.8328   -325.30032   -305.40585   -492.21889   -569.54845   -423.21606   -495.81197   -333.20953     -304.258   -195.90237   -380.12907    -235.7911   -503.89872    -263.4498   -332.53844   -260.65432   -312.71963   -538.27594   -403.96968   -305.87176   -460.13713   -201.45846   -274.08102   -373.53737    -93.35869    -239.0891    -35.97421    -31.58445   -261.14743   -143.50653   -323.97217   -364.44514   -444.99751   -261.02554   -349.50792   -164.49767   -276.44779    -64.84104    192.25581    606.09112    1132.1597   1329.63082    1446.3938   1125.91212    334.97515    -155.8105   -367.30983   -345.12975   -655.14115   -679.55329   -625.70341   -349.92747   -395.76431   -278.17911   -318.94073   -154.63358   -172.55821    123.96979     56.43386    432.78356   1001.66533   1428.75646   1409.01832     930.5105    686.07243    156.20746    -97.77398   -238.33037   -367.36757   -124.79576   -142.71565   -163.57029   -136.30404   -123.68674   -228.30374   -366.30906    -299.1882   -385.31237          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       52.50   -373.42114   -497.63141   -270.88583   -636.96397   -240.12586   -507.54525      3.93988   -134.46591    -513.2695   -199.11901   -511.47781   -354.39874    -542.8242     -53.1974   -370.11343    -51.78714    -375.1661   -295.71505   -407.02771   -429.67617   -296.63441   -270.63114   -169.28016   -319.24661    348.98395    491.09367    907.09497   1255.66081   1024.58727    382.46896    199.06825    -76.92752   -526.00379   -403.83747   -557.19675   -314.84695   -390.01372   -364.48842   -164.35603   -522.36281    -48.18593   -279.19101    -302.4629    -445.2952   -267.99591   -366.51933   -369.64029   -325.94876    -402.5868   -370.03174   -288.31427   -408.87553   -267.30666   -377.07468   -104.17075   -382.81343   -150.09795    -57.79514    -98.80891    -68.65855   -235.11907    -170.8093   -257.70892   -341.74334   -466.56746   -450.08752   -216.38256   -319.68994   -155.64692    225.45688    588.32979   1126.45091   1569.87953    1221.5278    957.50162    649.55343    -23.41494   -129.46216   -368.82105   -277.26361   -809.10017   -286.50056    -149.0686   -147.12051   -361.76456   -411.87803   -148.99099   -115.87043   -210.25916    206.94402    490.48738    941.77622   1326.37989   1362.53315    957.86475    494.07877    152.84238     -9.52245    -253.6198   -259.11784   -339.85723   -122.42824   -235.99697   -106.01894   -285.14696   -337.36246   -354.56616   -493.21183          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       53.00   -192.89681   -181.46072   -141.48519   -815.71912   -281.84959   -475.20508   -198.05178   -237.66203   -406.06803   -122.11842   -545.34307   -328.33749   -582.15837    -33.30232   -349.15339   -349.63116   -276.56895   -174.69116   -269.06016   -217.82078   -346.86659    -27.13121   -144.51371     20.29805    276.37954    449.42124    872.89254   1112.47027    845.04619    453.92931    173.91037   -179.74291   -264.82884   -371.91598   -534.48952   -399.12609   -219.26047   -333.67439     -255.068   -355.07153     -248.107   -416.69856   -369.11829    -516.2061   -537.61231   -504.14909   -505.87676   -394.85694   -414.02011   -529.52948   -390.55244   -487.21403   -293.08103   -347.96567   -545.01787   -434.91728   -456.32748     36.59349    -74.16391   -135.48506   -232.94113   -273.78533   -223.45318   -295.41175   -320.90027    -240.4052   -386.44048   -195.70565    -47.92105    280.70967    563.88887   1028.06647   1427.94273   1408.48158    892.73197    365.44239    100.56756   -261.04555    -403.3951   -187.99374   -591.27786   -528.89719   -341.98297   -454.11603   -388.93964    -323.4534   -232.29787   -213.46049   -208.60494    261.23156    633.91085    850.68083   1341.90086   1435.20437   1238.19527    545.01685       9.0771    -113.2665   -303.97899   -302.77389   -433.85541   -274.49877   -314.55369   -266.86078   -161.56085   -216.63081   -400.45586          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       53.50    -86.20924     89.35705   -220.25569    -746.7281   -304.24864   -354.63798    -454.2167   -281.31502   -240.81357    -42.34049   -665.50022   -327.00927   -669.88781   -245.48567   -301.51474   -567.10657   -226.02294   -163.71374   -274.21178   -115.49192     -322.102   -221.34077   -111.43047    -65.35941    213.28253     512.2511    989.37164   1100.07904    588.63311    664.49862    154.12271   -243.03145   -252.37363   -364.57071   -276.30385   -424.73883   -399.40083   -210.98694   -444.56355   -344.76574   -310.06775   -610.68902   -430.98272   -508.32571   -402.63781   -513.60624   -284.90423    -486.7456   -653.88554   -413.19134   -476.91618   -352.77926   -390.11097   -439.98344   -305.26758   -189.45518   -262.67428    -84.60226    -47.07405     -3.66223     57.26554   -269.58397   -126.91994   -264.04572   -311.37587   -261.01709   -306.25873   -240.87996    -233.9446    302.30627    440.49085   1004.68148   1295.70498   1364.39007    769.54387    410.52178    146.39369   -236.29047   -339.17214   -625.61443   -263.92286     -370.712    -780.8402   -450.70901   -164.56724   -325.50745   -264.79287   -207.97835   -230.27578    225.88987    520.98098    1028.3743   1375.04781   1477.08364   1258.31706    559.49271     399.2702   -192.52532   -521.31654   -220.65359   -300.68506   -177.84018   -226.53441    -41.84986   -278.47261   -390.39577          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       54.00   -119.18571    135.78964   -503.63715   -606.60384   -325.07266   -171.86875   -462.37079   -129.94227    -203.2637   -152.17176   -605.86757   -284.42209   -626.11369   -305.43854   -308.90129    -426.6304   -274.69987   -170.74964   -364.18643   -179.80883    -62.59154   -266.65657   -182.58397    -297.7455      88.1393    393.39775   1105.28655   1063.90795    602.29318    470.61303      55.3596    -64.04035   -203.35292   -351.20759   -599.61884   -274.02118   -389.62729   -608.79435   -350.57936   -436.49597   -405.54698   -455.98596   -317.61669   -453.73608   -294.36794   -271.64071   -271.96356   -144.99763   -455.42658   -353.99376    -486.9332   -457.66519   -313.32226   -376.49146   -216.16926   -174.99383   -132.00081   -148.79903     37.52891   -191.75283   -115.51493    -62.66318   -241.30991   -313.74414   -336.91424   -237.35073   -424.72779   -167.04482    -197.5776    336.86428    447.62851    1009.6378   1684.28382   1425.30824     827.0421    606.15387      68.9516   -135.42804   -107.91646   -319.29603    -561.5219   -482.96463   -523.23475   -651.06073   -401.04889   -510.55782    -114.0615   -189.24944   -198.25755      82.5929    389.40058    896.07338   1389.41248   1372.64447   1155.43454    679.37307    294.48027   -119.88095   -426.29686   -456.98809   -406.37569   -221.77621   -385.20531   -351.13742   -367.87747          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       54.50   -148.49289   -119.13105   -567.60206   -395.94804   -368.74686    -60.18337   -298.40014   -193.35705    -308.5027    -267.4799   -409.55772   -183.14092   -249.67553   -345.12204   -318.90477   -220.66896   -425.23867    -281.8314    -438.7746     -97.8459     -7.53951   -255.35836   -250.66774    -59.19432     39.39136    306.53853    830.45277     938.5721    783.94293    421.21317    -55.03146   -138.33935   -194.69683     -185.664   -345.13405   -373.65545   -409.95658   -332.41202   -354.84461   -312.43854   -442.18172   -316.38358   -302.59574   -377.64854   -365.23411   -315.68542   -302.51579   -556.48587   -458.98971    -338.8636   -339.42179   -364.75824   -175.99035   -270.91251    -195.5806    -213.8197   -262.12598     -5.84757    -99.92379   -170.82585   -194.10013   -200.83084   -284.03337    -361.4083   -193.34303   -295.80437   -291.08114   -298.68511   -174.73576    116.86551     727.1731   1006.54425   1245.04369   1335.19034    813.80492    521.98287    198.40014    -95.38465    -49.40956   -377.57844   -259.66009    -694.2443   -235.19591   -567.07806    -312.5115   -377.18212   -204.47865    -20.41097   -187.41935    290.85982    527.38455    832.65418   1129.57488   1396.83861   1208.46151    483.12528     52.94828   -397.04063   -382.16587    -332.5537   -407.57546   -352.23172   -318.12934   -405.72161          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       55.00   -170.84868   -279.25691   -405.34731   -330.84652   -391.08976   -137.66256   -170.94476   -179.31012   -416.91777   -384.46283   -303.73595   -271.39486   -134.21319   -439.52645   -342.38605   -377.21035   -319.47917   -399.02309   -223.72714   -295.74749   -212.44555   -285.55415   -171.80138   -105.56594     36.42821    456.24805    624.73005    795.84743    757.89314      259.727     44.77157   -350.10083   -387.85896   -380.96315   -326.35522   -612.89563   -509.24491   -505.17603    -409.0136   -401.93354   -332.11568   -361.73407   -343.87354   -565.88397   -368.25267   -428.82306   -415.83087   -362.93059   -283.88523    -512.2459   -431.49566   -305.37546   -346.75042   -327.84525   -255.89865   -174.88766     25.89704     46.26012    -67.97281     92.85365   -170.69613   -376.20127   -342.64715   -465.10914    -97.31762   -208.89608   -178.72681   -284.05313   -141.27786     58.23647    371.96197   1046.79044     1146.887   1267.60331    804.05393    794.15984     153.2294    -87.30795   -190.36647   -251.04111   -434.29793   -408.34765   -507.52258   -287.83516   -454.21343   -432.28215   -467.80719   -244.52259   -160.06891    164.84981    443.60918    991.80178   1251.40636   1322.31029   1110.02789    888.80391    132.59177    -363.6157   -829.15815   -526.30183   -232.29009   -376.99731   -363.80093   -367.37984          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       55.50   -243.28499    -345.4348   -270.79689   -422.20802   -337.65162   -297.62241    -219.5391    -62.27845   -511.33239   -407.81214    -220.6545   -330.49443   -218.16077   -418.12213   -284.45382   -565.82263   -289.42596   -536.57243    -193.2925   -414.01192   -376.90265   -367.30257    -20.32756   -169.68566     281.8467    552.12953    565.62149    590.68387    766.15612    244.29318    189.09365   -291.57792   -377.38543   -282.77028   -494.33438   -340.88089   -265.65446   -490.74141   -396.35231   -412.16505   -364.61687   -330.71619   -325.77535   -538.95923   -501.07904   -410.47258   -405.83371   -193.17099    -266.3798   -398.41654   -356.03967   -404.40631   -316.66492   -475.59138   -347.49247   -337.80032   -218.52228   -142.79259     68.78788      3.38309   -104.51671   -278.28222    -189.7081   -374.58821    -185.4213   -201.45019    -142.8495   -254.86651   -156.04313    208.15232    538.35546    811.11592   1121.49005   1211.61463    1131.3248    425.15013     19.77822    -44.67904   -437.35754   -173.30821   -359.92468   -692.28491   -137.36981   -243.62939   -508.55846     20.20734   -333.37609   -208.93661   -304.69167    -72.78296    273.42508    876.40917    1130.1832   1315.39854   1150.28227    702.84435    131.56661   -240.42216   -361.21929   -297.73552   -315.31431   -128.03295   -370.95334          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       56.00   -334.38566   -357.96741   -177.61557   -507.92048   -188.27872   -335.94394   -282.20371   -199.12572   -442.52325   -367.38057   -185.81933   -281.05484    -207.4812   -299.81163   -219.17396   -352.36056     -324.295   -741.84689   -363.04254   -315.76267   -343.44987   -338.71486    -15.76913    -56.71964    291.76101    433.72074     778.6512    818.78271    669.33345    402.65604    -67.19461    -179.7794   -312.07652    -234.7928   -329.86859   -259.32499   -498.99344   -659.64428   -220.49808   -374.52919   -511.23722   -247.66065   -415.39658   -664.85606   -302.49452   -336.88664   -423.47155   -340.69775   -447.34613   -402.20193   -364.30649   -214.17566   -466.59328   -387.21859   -360.97013    -272.8855   -130.22217     63.29025    -21.29517     -49.6578   -232.26271   -261.11466   -389.58989   -250.79019    -88.01108   -222.26384    -238.3979   -218.27124    -84.30888     -34.9803    554.27426     877.8468   1525.99925   1442.68923   1163.49336    316.09598    375.65866   -162.36297   -389.62253   -486.90469   -431.65497   -286.88326   -432.67471   -554.03383   -326.54129   -315.09091   -406.26841   -150.46804    -398.1501      7.68046    305.07992    880.98764    866.01395   1098.51582    949.06952    531.26984    234.86192   -128.58021   -304.91223   -458.78663   -262.92304   -424.81859          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       56.50   -424.52455   -371.05396   -176.49843   -494.42968   -112.13257   -298.51851   -303.33462   -296.82646   -440.26781   -246.50883   -278.84779   -209.44281   -176.17384   -255.34881   -206.62548   -105.08431   -374.27811   -613.99009   -347.95552   -309.19968   -245.61211   -349.72798   -124.47518    -122.3125     274.0618    505.71206    747.06167    859.67823     629.7079     362.8557   -136.09851   -143.02714   -326.54441    -284.6902   -265.49455   -209.44932   -648.08028   -546.41562    -56.46019   -436.78321   -192.52188   -228.91243   -398.98655   -399.40635   -443.69848   -549.48296   -366.68185   -421.50841   -446.53575   -397.28217   -486.65386   -445.70037   -373.95372   -437.38108   -442.09177   -364.42985    -35.10603    -37.33465    -51.74371   -201.05456    -37.29885   -168.28673   -301.22628   -292.72862   -346.84051   -256.58957   -190.29563   -199.09186   -214.40106     14.04891    613.06278   1147.69265    1200.0245   1237.20996   1141.03031     504.3703    263.35672   -137.94197   -411.21096    -418.8795   -393.50285   -455.63258   -526.92129   -400.23113   -452.85357   -284.97776   -257.98943    -189.3411   -281.88446    -85.34269     391.5893    667.43983   1105.15186    980.05424   1020.83652    590.25683     181.8418    -55.61365   -316.34135   -407.18692   -390.98842          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       57.00   -513.69405   -358.14264   -202.49511    -360.7072    -203.3145   -361.99823   -348.07492   -249.27433   -507.40588   -153.21676   -267.81214   -226.46839   -336.08694    -73.42289   -181.01456   -255.44436   -506.46126   -414.60422   -383.65863   -243.69335   -284.72799    -56.03777   -318.49706    -93.10318    181.63265    545.40022    753.37994    746.42648    830.31458    244.06194      45.6727   -133.45091   -493.81571   -334.88825   -323.43743   -157.90301   -403.73511   -388.40855   -323.05372    -289.4724   -630.27914   -363.34383   -347.62635   -525.23361   -111.77288   -466.59381   -558.12345   -342.20006    -412.0634    -280.2669   -402.44788   -403.43628   -383.35102   -365.11566   -239.47096   -352.97368   -192.40136    -96.83675   -206.03685   -108.38572     -155.873     -65.3835   -198.66063   -411.13079   -435.51809   -183.78033   -238.25312   -107.76449   -174.31569    132.76283    714.88116    811.76578    906.45546   1331.27395   1100.32509    536.14815    190.69255     135.8401   -348.83292   -393.68902   -268.61095     -331.607    -592.8349   -391.50167   -423.92966   -369.02557   -339.52451   -210.76744   -217.39139   -224.96399    294.96307    540.39396    979.96975   1069.30391   1094.15999    644.81471    166.34356    -96.49495   -226.45695   -428.05802   -395.99614          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       57.50   -587.31378   -233.49432   -191.14996   -246.52456   -397.25366     -420.367   -388.01618     -155.767      -608.22   -287.11031   -175.28724   -422.32589   -313.89406      8.52569    -52.94745   -289.20119   -490.75788   -376.47221   -397.71222     -404.868   -283.58537   -195.47567   -405.43604     59.41101     39.41014     619.8581    685.14182    787.95123    662.49805    299.05231    117.35189   -162.92333   -351.24833   -390.27553   -194.99906   -315.72808   -730.15318   -286.83561   -346.33719    -80.02807   -482.96917   -423.50553   -241.03199   -159.58934   -262.58318   -426.29086   -243.95204   -482.53631   -223.08553    -152.0485   -334.51109   -338.09166    -594.6278   -243.72255   -368.14789   -247.22491   -100.99779     -98.6058    -42.47762    -20.75101    -76.10836   -131.22669   -229.31535   -312.14212   -387.90477   -252.37256   -354.64308   -239.26668     65.60564     71.68339    489.42327    1117.0501   1221.86441   1219.58084    815.54414    358.99569     264.7878   -127.73596   -251.35462   -377.67734   -296.14418   -289.95888   -482.51062   -478.55152   -379.90696   -379.72428    -364.2422   -439.78181   -285.17504     43.19215    297.30612     666.2972   1069.51323   1056.82552   1085.63949     952.6282     64.84567   -140.67169   -145.96351   -444.94505          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       58.00   -547.19427     63.24677   -405.34972   -221.13586   -444.03677   -479.76176   -408.87839   -132.91828   -760.30328   -394.16953   -292.96315   -601.52808   -224.47321   -199.38361   -132.74703   -381.97025   -547.32185   -339.29241   -314.33299   -441.38936   -309.75033   -388.69497   -442.96002     41.13923    -57.76091    480.20547    627.30707    766.40481    387.72469    199.14171      80.4344    -256.5308   -423.91519   -302.37018   -620.03915   -485.39704   -653.24333   -391.66079   -273.24709   -281.64672   -384.55372   -526.72216   -313.92964   -476.29712   -379.52428   -309.24104   -459.21535    -300.3578    -485.2418   -335.18659   -264.52766   -447.33193   -257.02911   -337.67866   -279.34874     -23.4935   -369.65823     27.79771    106.40424    -57.86675   -222.45435   -217.06698   -224.49932   -273.04827   -348.51592   -199.99438   -271.71765   -352.19457   -257.04921     54.23665    462.34645    898.51607   1222.90873   1290.76391    856.91985    732.88762    151.38833   -102.08059    -66.13792   -333.19585   -362.47713   -556.69041   -400.10601    -289.4747   -505.15938    -409.7038   -474.25399   -141.15435   -181.27419     -295.061     216.1316    518.31632    896.47613   1212.05524    951.60521    591.18798    249.07703   -177.51546   -372.22963          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       58.50   -401.47903    146.91223    -623.2406   -117.66951   -335.95736    -566.1222   -498.33917   -304.69307   -746.76025   -479.87395   -409.38964   -491.29433   -266.75164   -201.08304   -169.67528   -536.05927   -562.76872   -349.88004   -430.95757   -156.76276   -414.96307   -336.74167   -290.65888    -73.04347    115.68474    308.40669    541.66441    664.51371    526.98434    274.53129     41.21752   -364.75515   -317.23217   -507.69912   -308.40923   -480.35168   -629.42581   -339.17028   -308.47201   -324.60415   -564.52106   -400.54582   -399.81767    -388.6955   -368.43329   -453.85367   -285.44245   -539.63933   -367.34692   -450.01428   -281.74683   -470.01103   -335.86981   -409.88015   -328.97794   -172.81127    -98.72674     13.72059   -130.87648    -93.87273   -250.65054   -268.50259   -281.24797   -266.05871   -273.28641   -274.75942   -340.04495    -85.90738   -207.48003      66.2302    467.17855    957.90413   1431.59916    925.95731    967.82452     749.4644     28.42765     39.82744   -493.78282   -306.54989   -609.41561    -188.0409   -487.45678   -441.04069   -438.13495   -289.62248   -310.47199   -137.65115   -195.51038   -148.30753    298.73821     517.0035    702.80553   1043.32796    833.51966    487.59496      3.04386   -275.15861          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       59.00   -220.00551     117.9731   -713.72059     71.17421   -181.55029   -586.42085   -615.30135   -407.62109   -603.70597   -483.55927   -339.02492   -381.92558   -436.74895   -115.02259   -121.63303   -459.45334   -443.45434   -247.77843   -353.14231   -246.53618   -479.64205    -381.6547   -441.81185   -178.99937      59.6073    412.09457    451.58015     465.4221    179.37943     420.5326     -8.98878   -265.44762    -478.8052   -317.11034   -290.90667   -639.23801   -491.29266   -362.59415   -296.82291   -145.69228   -501.61179   -385.30219   -268.25965   -372.95389   -227.65334   -456.36099   -411.67883   -260.98505   -333.58131   -436.45511   -388.88128   -499.10882   -306.74296   -279.62402   -413.18416   -518.67221   -119.86938   -182.37612    -41.73383    -97.26423   -109.01438   -309.93676   -208.15483    -239.7476    -44.64576   -191.93578   -339.54555   -135.89717    -47.57595    289.74281    435.45135    815.75981   1197.27955   1438.83703    988.29872    470.92123     19.31582   -310.84718   -391.60874   -220.23064   -327.79279   -398.70207   -225.04638    -386.1618   -403.72942   -216.37536   -221.68987   -227.61373   -155.09935   -112.46157     89.33401    546.06584    769.21428    925.30462    800.95798    468.81609     77.69709   -326.72057          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       59.50   -195.47218   -197.44273   -438.06933    -79.96601   -214.86705   -525.26248   -461.76166   -222.69186   -418.14248   -451.55286   -294.89578   -361.20357   -438.48473   -175.92686   -129.28234   -452.92357   -328.56615   -199.07866     38.29041     -29.1673   -454.21445   -676.95981   -606.39754   -152.76294     230.0409    500.88144    678.54762    645.07896    461.63848    314.85607    -81.17914   -339.34232   -459.55432   -262.08778   -320.98525   -599.84984   -492.86757   -360.49952   -241.20958   -182.18614   -147.81949   -441.91567   -546.40353   -461.36055   -369.99492   -395.13642   -458.47813   -157.87276   -385.40393   -278.64512   -253.54506   -491.99716   -610.03197   -357.29132   -365.13132     -81.2136    -26.12242    -88.33106       66.081    132.60958   -277.55269   -305.24698   -373.21342   -204.60827    -192.8706   -256.32185   -435.31805   -287.07055   -124.91813    -60.90234    298.34021    883.16041    1023.3013    1265.3731    972.63826     629.0918    217.04265    -62.79465   -270.14927   -335.83445    -86.21638    -62.97602   -360.22819   -633.15949   -419.01212   -388.63232   -321.28991   -273.67894   -149.52681   -140.05133       27.189    417.44546    731.67542     796.0868    785.94618     439.6593   -106.61934          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       60.00   -259.71675    -484.7465    -301.2367   -409.56706   -283.40548   -452.56569   -398.19965    -86.08976   -359.27633   -456.52803    -370.4229   -373.47236   -388.15601   -328.67653   -174.19824   -400.69291     -343.122   -238.10747   -233.43762     35.94123   -424.19012    -357.6909   -363.79266   -403.87826    325.45006    340.17978    568.17855    641.23095     501.5768    269.46957   -260.30197   -275.03822   -299.19666   -308.91351   -307.00552   -715.24324   -316.46493   -436.69801   -224.55651   -545.67903   -342.86709   -381.76193   -558.75221     -385.512   -471.17941   -337.16928    -405.9387   -412.41504    -371.4095   -322.26571   -556.15918   -335.38373   -361.04109   -284.85153   -355.31095   -311.50116   -152.85364    -91.53576   -225.90656   -203.19881     -2.14159   -127.73594   -321.50531   -425.21551   -291.17174    -261.2144    -247.0753   -391.10736   -103.27813     -1.50732    487.50074    878.75064   1221.10266   1309.66049      833.496    635.68292    239.78517     -6.30556   -159.60883   -413.22154   -193.24623   -242.02167   -295.49248    -412.1244   -430.28408   -252.30057   -422.31453   -235.61939   -266.06988   -209.90317    190.53756    484.33473    697.10831   1010.69154    539.26536    348.32902          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       60.50   -429.91238   -737.49839   -304.98851   -636.78292     -390.877    -363.8043   -448.50515   -156.29466   -483.75315   -495.41986   -350.80897   -415.26489   -326.17775   -342.72084   -344.57508   -396.98006   -389.59797   -149.49556   -284.56318   -116.46047   -430.72601   -230.91909   -360.37647   -379.83521     32.00856      33.9038    506.93773    485.62834     345.6218    437.85911    -120.7529   -251.85162    -397.3591    -24.04628   -273.21593   -472.58569   -402.26418   -181.02762    -383.4193   -195.68725    -504.9266   -543.25358   -688.26331   -341.51085   -577.38501   -553.23758   -226.67591   -510.13264   -441.69336   -417.18465   -350.10444   -368.97293   -543.13896   -349.52793    -351.9129   -372.47715    -90.93167    -100.6929    -90.94694    -74.21297   -131.29833   -375.80796   -279.05846   -543.63252   -362.37883   -487.18129    -344.2637     43.85191   -266.14345     -3.74924    398.40283    833.80575   1092.80685   1152.06367    762.98225    654.17693    341.99136     38.77663    -78.23189   -247.03152   -188.97528   -528.95241   -497.18539   -484.72163   -332.94415   -244.14788   -485.97474   -352.41703   -328.50601   -135.70271      10.9436    395.25852    623.68586    808.51125    608.26819      23.1796          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       61.00    -374.7142   -727.80592   -395.20164   -622.19902   -306.52912   -244.36032   -404.97215   -357.17535   -547.88076   -443.87804   -150.39751   -383.98691   -226.47617   -278.36164   -410.52331   -528.98993   -236.64887   -223.24223   -263.49266   -358.57201   -110.06238   -267.30234    -397.0779   -351.23447     -3.09732     86.26555    977.67711    498.04326    295.96981    107.70943       32.061   -324.78957   -563.37422   -451.71597   -235.99358   -529.20202   -356.48042   -232.71279   -374.37631   -314.74068   -292.88999   -402.82266   -489.67311   -664.71127   -444.10361   -397.96298   -298.39248    -351.3302   -341.89723   -414.08287   -180.60053   -379.77959   -542.57896    -341.8345   -209.90597   -375.52187     -224.658   -135.10757    -44.06538    -73.15409   -102.64321   -184.76344   -514.86081   -298.49534   -286.15179   -377.63883   -487.75145     -249.325    -273.1319     -6.21796    297.74295    927.52158   1125.97375    978.56886    1206.5777    511.72206     327.4724      7.52705   -257.28553   -360.92683   -178.94299   -370.22122   -307.05525   -484.33328   -404.30762   -492.62001   -466.58129   -372.40102   -298.50312    -97.41408    190.90996    334.65283    566.10046    756.59717     496.7573          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       61.50   -297.24654   -602.67266   -452.59661   -515.36258   -242.70664   -206.21435   -295.13526   -507.72585   -477.23382   -361.63654    -26.58933   -393.61457   -266.10744   -361.14543   -368.56584   -584.24591   -123.01453   -233.90582   -242.40685   -345.75336   -107.28165   -187.86879    -432.0884    -288.1081    229.90876     166.8724    688.45571    414.02936    500.30676     200.7688    -29.78021   -170.87163   -307.39786   -415.45078    -473.7042   -339.69377    -469.6068   -248.72086   -396.87228   -502.44464   -478.37239   -327.19472   -397.71241   -218.31735    -397.0578   -284.31735   -321.68192   -380.55669    -558.2449   -534.54115   -384.65338   -355.90523   -432.57388   -337.53686   -104.04684   -340.10677   -282.54189      6.38995     19.34485    -94.98382    -46.69393     -3.23045   -289.72771   -265.28052   -385.26425   -228.21256   -237.18966   -400.83994   -223.03019      1.85256    460.37715    892.43882   1361.04659   1248.63887   1064.42506    704.14238    349.49445    -32.28678   -245.66631   -146.80403   -488.23168   -549.44144   -289.69019   -385.15513   -139.75003   -417.53406   -345.35436   -418.41868    -42.65079    -69.26436    294.09789    462.85705    442.60922    399.73257    235.81479          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       62.00   -197.87462   -417.17749   -484.70752   -351.14047    -186.3686   -236.69108   -262.34989   -506.74478   -373.35738   -276.94585   -224.79034   -376.15893   -187.39922   -451.17499   -430.17735   -216.61272    -288.1879   -437.92754   -141.25285   -340.52059   -525.67405     84.13074   -541.99199   -125.65773    166.53209    396.42502    426.07887    423.97327    398.82259    222.31023   -156.35776   -130.77918   -174.12142   -236.17585   -458.10494   -304.45715   -428.67652   -193.79304   -454.55361   -493.66971   -373.04601   -316.12308   -416.75663   -421.66535   -306.67031   -300.84572   -286.11506   -409.39964   -464.97541   -420.74667   -542.15542   -191.96421   -281.29645   -379.49042   -298.85175    -348.4734      50.3542   -117.21539     118.5441     68.63326   -301.07397   -157.29442   -190.70247    -439.3048   -214.36948   -301.21647   -196.99544   -398.65661   -185.14584   -139.00552    261.39359    837.57561    1128.5271    906.35722    1087.5495    494.91741     267.7911   -186.76399      6.02052   -178.34256   -163.60816   -387.30756   -362.45896   -368.59684    -409.1186   -356.89575   -335.47686   -403.93861   -228.68025   -108.86479    -82.12137    375.42941    377.19455    304.46205          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       62.50   -197.69573   -343.45471   -431.95517    -324.0813    -42.43567   -190.30321   -396.96182   -546.18956   -332.71156   -383.03599   -292.92049   -390.59977   -104.65318   -377.84259    -427.4226   -223.79332   -195.68044   -530.76315   -300.82893   -484.47633   -260.54931    -24.95435   -389.13349   -211.78934    155.47688    495.72932    403.65188     509.4863    171.62448    129.98917    -36.67648   -177.44112   -204.40668   -232.14151   -431.65344   -427.06032   -588.86936   -196.61221   -344.60946   -444.32047   -390.64853    -375.4483    -582.2724   -450.73294   -233.24642   -157.31039    -259.8876   -209.83278   -338.05719   -497.92688   -353.14461   -473.10371   -335.67285   -333.73393   -316.98181      30.8629   -291.60881    -98.45301   -130.04939   -152.06839    -63.01915   -145.46469   -184.72657   -150.37754   -322.09037     -382.288   -307.76846   -464.17601   -137.13498    204.15366     307.7918    711.46308   1054.84812   1097.37642   1031.13315     745.9294    347.60456     98.19913   -164.70821   -277.38317   -176.10868   -480.54899   -355.60001    -313.4189   -158.91802   -184.83022   -300.63174   -310.55163   -227.93517    -105.0871    124.98084    310.93943    419.52098    235.46256          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       63.00   -261.12465   -376.18851   -321.21398   -357.26698    108.16142    -32.30534   -581.59902   -590.79123   -405.48753   -459.01598   -218.27158   -517.86644   -175.61711   -342.82874   -387.53668   -580.55064    -12.42604   -384.09679   -565.46872   -546.12563   -211.38791   -212.32594   -358.91066   -242.16726    235.74334    421.66229    509.45983    598.59688    303.39309    321.23695   -218.80368     92.74191   -302.95196   -310.36514   -485.98673    -363.9724   -405.41828   -325.17422   -147.80385   -515.48894   -352.06947   -377.07911   -376.87661   -310.10403   -265.63453   -367.01548    -568.4528    -399.8481    -26.61017   -373.32815   -432.46618   -365.42869   -254.72966    -342.0532   -220.16434   -220.57151    -349.7006   -105.38383   -101.38948   -136.13351    -64.44959   -270.61237   -406.27007   -539.00926    -352.3862   -533.22405   -120.61241   -269.29615   -149.62603    -78.10127    332.22806    718.12624   1046.14758   1194.79772    790.91447     937.2369    206.56188     85.25813    -226.2607     -99.2256   -349.65622   -503.35287   -524.93324   -569.47479    -354.7509   -149.11994   -315.71029   -316.89253   -107.29512   -117.82228     76.32245     165.5026     50.32839          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       63.50   -353.57462   -507.64626    -189.2838   -417.59157    -23.14952   -244.87123   -553.25926     -379.823   -352.38415   -442.70012   -208.86674   -600.86236   -214.43133   -390.58043   -369.87029   -498.36996   -164.31902   -420.42153   -478.39487   -150.60695   -419.19189   -144.17237   -295.86901   -289.41698    180.13013    347.15894    515.11463    506.86998    191.95698     316.8275   -145.49722   -170.21786    -77.54594   -429.96392   -297.80814   -176.11043   -484.79339   -307.96532   -446.53765   -531.92611   -317.51118   -287.23406   -205.76221   -320.31276   -338.39535   -303.21431   -368.62832     -573.717   -383.96498   -579.15499    -75.83365   -241.63179   -374.29917   -228.05075   -262.35318   -234.96592   -208.55134   -151.65205   -163.17627    -19.12767    -62.33365   -165.09267    -79.68609   -378.08382   -514.29801   -425.18499   -354.78013   -269.59316   -145.01817   -138.22624    341.15605    588.87531    961.38798   1158.93604    750.40627    645.98926    379.50345     26.59011   -183.20773    -258.9401    -359.9161   -431.23984   -245.21636   -267.42985   -155.65674   -283.81018   -555.40358   -340.70751     -380.173    -99.98764    -79.80644     36.55594   -222.27735          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       64.00   -432.08121   -347.35679   -221.66168   -369.48616   -303.15337   -442.88568   -492.26808    -38.39537   -237.32094   -344.26221   -342.31132   -606.25237   -307.10576   -481.95366    -73.22433   -216.82264   -317.54104   -522.89329   -486.19205   -231.72248   -510.89477    -245.0636    -280.4542   -379.32777    219.09947    388.65976     323.2176    617.94758    313.71029    110.64547   -122.76781   -428.51078     -93.5986   -229.08764   -414.42633    -20.55955    -290.9899   -198.89638   -463.49365    -395.8375   -283.67144    -483.0264    -190.1672   -526.22455   -180.55378   -229.45093   -530.04677   -268.99544   -228.76277   -336.39774   -358.77466    -278.5881    -330.2647    -34.38605   -135.29752   -203.16083   -186.78662    -217.9921     30.71815    148.40693    -28.94437    -97.48039   -461.21686   -112.35149   -475.32506   -412.18505   -410.61563   -218.62259    -44.01714    -24.00472    310.70424    669.97924   1022.34579    959.25941   1335.72787    646.65999     295.0803   -176.19814     51.50814   -143.74333   -467.78342   -253.33004    -299.9891   -140.86066   -325.96946   -415.97776   -241.11224   -380.52791    -182.0535    -256.0565   -126.13004    -41.66686          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       64.50   -443.80065   -195.92888   -284.19897   -204.37206   -591.38711   -557.14578   -395.12394   -175.01755   -348.94728   -209.23759   -542.68188   -610.27742   -517.07741   -403.86222     47.73039   -394.28865   -289.56101   -255.01135    -445.9824   -311.01755   -493.33884   -243.52099   -290.83958   -181.46796     -1.33638    244.58361    395.18783    576.30726     595.8187     77.36506   -296.17374   -351.97767   -166.31293   -233.50769   -306.61684    -163.0802   -296.36773   -163.44489   -265.84018   -331.92241   -294.44773   -239.85092   -439.14115   -320.14526   -261.49791     -2.45832    -191.7019   -320.93999   -426.31247   -319.65569   -461.92985   -207.96132   -369.28541   -310.06211   -150.13084     14.16174    -38.10373    -70.03449    -18.58725   -179.23862     32.57529   -210.60771   -194.59052   -321.65678    -399.0137   -470.51096    -235.2423    -279.0018   -353.28459   -140.77012    269.06522    498.03534   1040.35491   1146.19097     982.0519    653.22654    243.69177     24.39295   -204.74846   -290.76711   -477.20221   -395.06386   -419.90859   -252.46673   -245.61599   -313.75016   -412.83832   -414.39489   -322.07825   -352.38944    -24.83629   -315.60466          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       65.00   -354.98725   -105.24728   -280.65343   -204.35147   -520.56167   -536.12929   -223.04573   -201.43709   -440.76684   -150.48634   -541.27883   -490.73706    -531.2325   -233.52238   -239.80475   -713.03274   -306.07476   -267.06853   -478.29235   -439.11684   -235.88767   -163.15331   -194.22223   -159.62518   -143.88571     95.12816     93.47899    454.50316    509.31323     106.8337    201.06444   -152.42689   -184.11088   -255.46388   -398.29093   -365.16889   -419.02938   -275.81769   -241.44458   -323.97974   -502.74009   -283.54647   -303.90093   -307.84306   -445.92773   -339.64273   -438.07912   -466.83876   -629.31569   -155.47061    -430.9505   -346.62207   -348.07876   -293.46199   -212.16434    -81.94096   -205.65092   -232.51886    -44.80605   -279.31418   -203.27454   -234.93725    -253.2643   -372.38356   -217.03288   -411.44961   -271.78294   -281.63357   -167.96197    -17.48679    253.06079    630.70401    781.32807    984.60559    919.07364    600.43162    265.08825     93.03309   -104.50762    -64.07649   -343.51393   -404.74587   -407.16313   -606.00097   -519.02761   -402.23062   -234.60651   -256.36696   -233.71798   -185.06646     -173.407          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       65.50    -208.0355    -194.7684   -275.87022   -289.05878   -393.07194   -435.96285      -69.219   -252.56637   -364.22808    -231.4527   -325.79958    -487.2932   -398.56149   -304.76914   -325.03936   -383.85494   -370.25265   -282.86229   -493.75056   -357.65023   -273.74793   -304.63417    -35.95344   -242.78299   -240.85011    112.68284     37.81697    393.07029    310.44632    247.61091    -31.10769     -97.0074   -255.22319   -431.14998   -474.19099   -442.64618   -277.41353   -328.10737   -489.67358   -358.57601   -372.02733   -574.23975   -353.56568   -412.05688   -324.42228   -421.88439    -567.8177   -255.11228    -336.8353   -183.87405   -323.71836   -200.64198   -268.65123   -122.11861   -329.63614   -346.45404   -121.31646   -156.68816   -210.86058   -213.06355    -41.44254    -122.6847    -31.75696   -256.77961   -443.76322   -256.11878    -65.72308   -250.61662     67.24978     -9.89881    456.98941    769.46215   1001.47559    894.99257    843.18159    479.88027    398.68469    122.89487    -47.81339   -222.92475   -424.75587   -270.47937   -280.92535   -458.22561   -379.10284    -368.7802   -348.03953   -492.24729   -380.94117   -277.86637   -304.87203          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       66.00   -182.89845   -293.51475   -312.63353   -338.51862   -282.40206   -526.58121   -125.78712   -444.37206    -275.9534   -340.72254   -248.89046   -498.52434   -455.88951   -314.24841   -296.48787   -322.03029   -357.32109   -267.17103   -464.33218   -225.66395   -232.07667   -364.15631   -225.73507   -237.56792    -80.18626     23.08402    145.08816     351.3311    223.40066    222.51221     15.94595    -21.71116    -199.2501   -377.36929   -436.62893   -381.01513   -303.16528   -340.19163     -198.928   -396.56887   -177.20593   -549.86816   -162.00127    -430.2466   -147.77864   -499.34064   -379.54957   -455.88927   -235.23039   -541.93261    -352.2198   -584.14806   -400.92453   -219.32702   -284.90994   -274.61954   -123.00356    -74.21899    -99.20113    -55.80864     68.87997   -124.75332   -159.88747   -272.34236   -402.13095   -528.64862    -296.2143   -198.61143     -83.7201    -16.95882    439.84445    671.41439    923.50539   1159.93616    957.06163    563.33413    448.96761    152.41402   -115.93939   -178.07826    -321.1936   -311.05131   -440.03546   -450.58742   -238.90486   -229.23514   -216.67506   -225.75835   -295.25846    -460.2942          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       66.50   -266.49973   -374.93317   -396.11438   -375.22407   -368.09489   -606.74393   -301.70747   -596.48242   -292.25022   -428.93174   -241.89659   -409.86604    -489.3057   -392.07857    -283.3164   -441.75487   -549.25681   -332.68837   -344.20347   -319.87579   -455.26428   -551.25247   -244.89515    -134.1967    -33.98236    -50.79014     24.42564    408.53761    411.88675     179.2342   -244.03861   -243.58249   -213.64223   -328.19477   -373.16013   -194.10176    -407.1674   -205.72683   -330.21534      8.90508   -192.55886   -309.52515   -236.44554   -386.86693   -639.17934   -463.09168   -243.75085   -363.87211   -261.29178   -346.58305    -360.8775   -264.51814   -232.63326   -370.87043   -258.44298   -462.53223    -53.74064    -262.2817    271.44762   -173.04938    -98.48135   -166.58647   -253.66681   -222.92226   -301.23452   -193.94521   -263.87373   -238.90002   -176.46143      6.21801    552.05354    684.21484    954.81216    893.78292   1057.43251    717.17755    381.78434    201.66209   -197.63388   -246.58727   -197.45174   -535.29826   -309.72171   -486.57287   -428.94895   -385.19057   -325.40124   -377.65209   -123.85813   -362.89555          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       67.00   -335.11055   -430.68375   -501.78034   -379.42395   -531.96951   -563.36266   -265.68321    -350.9631   -462.95379   -457.42607   -251.74126   -325.37024   -406.09855   -529.48372    -416.1493   -347.40993   -496.17157   -200.57911   -351.48689   -449.67189   -549.87178   -448.30947   -210.39947   -215.28062    -60.47514     -2.45971    213.14452    373.15797    301.31696     313.4464   -227.39403   -285.38531   -485.55448   -413.50415   -248.41314    -142.9829   -475.83039   -203.82777     -401.425   -268.91688    -16.45183    -87.38709   -396.20933   -225.93362    -239.0206   -518.64453   -402.09556   -320.82854   -388.19216   -173.86818   -538.09068   -305.48748   -386.43294   -348.12663   -391.16663    -38.07552   -439.35641     89.20924    161.71672    -40.74752   -228.06313   -278.88573   -311.89882   -123.04201   -373.05721   -205.78395   -382.28861   -254.39096   -251.55655    107.76767    182.11003    756.59457    806.94996   1139.64451    1329.5417    875.97026    450.82447    133.74064   -238.30173   -221.49747   -311.19509    -277.2747   -312.17231   -499.66032   -400.42729   -424.78342   -410.91275   -311.66373   -330.34656          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       67.50   -368.02378   -450.79696   -579.10867   -336.91812   -455.73669   -458.84191   -207.44559   -130.92332   -470.55524   -365.64795   -295.40518   -249.47361   -353.27525   -557.39015   -463.97412   -102.08442   -230.51294   -318.13715   -397.33504    -357.8305   -455.18272   -175.84078     -128.281   -438.34007      5.59236    -48.36547    486.61765     166.2524    289.67537    368.10839     28.61095   -168.13644   -186.59179   -282.60482   -259.87863   -385.49389    -419.8571    -383.9975   -470.14959   -359.59347   -141.50869   -161.55989   -419.62991   -381.80957   -177.73105   -401.49382   -296.71262   -328.17035   -316.60279   -222.41116   -300.95941   -341.33324   -274.34823   -591.13119   -317.65235   -373.13595   -238.48263      0.84869   -317.43776    -67.95784    -71.33966      3.58238    -76.33529   -239.14833   -166.09793   -209.23571   -295.91084   -219.38266   -234.58523     -37.0768    165.44552    480.89706     903.1386    893.92827   1026.94718    690.99743    556.20532     182.3647     -49.3063    -172.0533   -294.74741   -172.59302    -360.2712   -334.44936   -411.17393    -329.4489   -196.39458   -253.17075   -406.91461          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       68.00   -382.25564    -436.6514   -458.43342   -285.25862   -296.52203   -418.49497   -272.14712    -62.00673   -282.76306   -222.06484   -273.34932   -257.11545   -357.82873   -523.68979    -483.8364     -228.724   -287.59959   -752.50568   -352.23504   -430.75037   -361.10441   -162.71383   -127.47286   -466.35012    -87.22558    124.84146    263.54962    -76.94905    190.32128     26.57637   -152.38587   -152.21475    -43.36087   -361.03092   -240.37524   -427.35475   -519.86768   -319.99462   -488.94502   -414.63902   -276.26707   -474.15102    -485.2757   -403.18076    -260.5937   -310.12047   -284.40984   -317.46757   -397.20706   -364.27775   -534.20354   -304.60278   -452.97215   -369.99891   -133.00003    -198.7352   -120.62669    -73.64243   -162.86741      1.81216    -42.65195     20.13656     -187.243   -397.09446   -325.33205   -279.52958   -413.14757   -254.97353   -232.93157    -432.4079    295.84777    712.41101    949.12417    979.80382   1022.81933   1053.51669    475.71825     64.21434   -165.35235   -166.99728   -223.40578   -325.88985   -426.13497   -265.97205   -228.21323    -262.5031   -281.24186   -285.06725          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       68.50   -391.53835   -283.11197   -296.99363   -278.07512   -112.49573   -369.22111   -428.58273   -129.45023   -180.39307   -251.65443    -199.0707   -274.32125   -402.28094    -360.9913   -349.89117   -331.94214    -345.8504   -546.20342   -487.70141   -570.05945   -354.66962   -224.81746   -183.48408   -315.10028   -203.95714    227.01011     225.2061    -82.42667    325.83153    -69.67658   -120.81835   -311.11158   -286.07917   -399.96341    -384.7732    -164.8184    -468.8581    -403.5733   -572.25916   -441.75524   -436.52592   -470.30667    -558.2688   -550.72731   -199.46269   -330.51404   -375.72723    -380.7724   -222.03297   -275.70639   -305.42294   -443.79201   -464.71578   -312.05954   -272.78349    -96.69734   -119.70003   -131.28102     65.45307      61.1391    -36.68088    -35.99579   -201.26716   -268.08727   -231.51704   -345.18224    -172.3374   -391.29958   -250.59198   -156.47446    322.39127    700.17594    952.50667     1078.079   1085.09992    764.80606    537.87335    270.93281   -210.87551   -125.74761   -397.22627   -261.42451    -284.2957   -171.75552   -373.11678   -310.56647   -591.27305   -488.74474          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       69.00   -464.86739   -112.77962    -182.6004    -287.0234    -67.51536   -338.31796   -534.71642   -254.06506   -253.76635   -398.65457   -238.05625   -229.90787   -439.44033   -240.78029   -250.54964   -344.15644   -384.76117   -347.00336   -480.30357    -298.7255    -343.3197   -314.71141    -242.0138   -335.31085   -125.36096       90.961    202.46327     73.29108    255.21808    110.85116   -129.64785   -295.63558   -242.24412    -387.0867   -170.42521   -324.13193   -447.00919   -380.00501    -442.4934   -610.90392   -582.75505   -435.50761   -400.68467   -430.85668   -450.94709   -292.37447   -460.17472   -634.73061   -351.87659   -362.00336   -405.95215   -327.53714   -514.23691   -489.11589   -296.40967   -234.26481   -166.98178     90.75244   -103.87015   -175.61216   -155.09626   -264.14707    -91.27148   -547.61181    -46.92113   -314.25166   -389.63686   -300.51223   -268.23952    -25.48673    336.47372    589.35646    956.27859   1018.30961   1293.57463    852.96469    400.72249    191.99245    -187.4857   -266.91347   -361.49367   -327.81784   -222.60634   -172.05574   -324.55941    -553.7117   -380.15176   -489.46181          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       69.50   -522.91322     62.15732   -237.48177   -225.03843   -139.11759   -330.20291   -532.15959   -257.02552   -316.40093   -385.79608   -362.88093   -330.78678   -427.93973    -269.7245   -298.84158    -408.0057   -479.85019   -582.13755   -502.28779    -217.7612    -268.3039   -364.63568   -233.70535   -314.30706   -247.10586     51.30827    190.24675    176.46617    171.15462    244.56132   -198.42726   -403.20265    -91.95283     -452.764   -267.41717   -532.37047     -589.902   -313.03375   -415.31416   -106.31908    -311.3327   -398.71795   -488.10594   -217.47084   -351.20033    -286.3634   -490.22371   -191.83079   -388.13152   -479.74926   -344.34375   -324.21636   -348.98581   -551.44156    -392.6078   -129.31417    -24.19741      3.11364    124.81949    -84.02369   -111.13608   -194.77541   -368.37583   -182.75876    -227.6431   -315.38975   -286.90251   -297.64357    -43.34155   -145.62494     171.7738    475.84482     919.6164   1206.63241   1086.69363   1327.49472    648.39521    210.67498     -139.671   -265.18074   -399.57464   -306.74194   -300.72946   -432.66558   -321.10486   -316.37037   -498.88526          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       70.00    -553.7621   -164.59999    -362.3136    -77.11279   -263.05464    -343.4301   -531.69994   -302.58476   -428.75332   -209.69922   -432.47643   -338.48973   -365.21472   -320.36438   -269.33752   -394.48867   -536.49606   -495.98716   -657.08221   -359.95433    -239.3211   -271.35307   -209.90051   -295.91143   -210.38159   -158.87414    180.77698    289.83511    293.75764     47.12042    -72.43493     -203.385   -185.31587   -260.25024   -433.30872   -373.01323   -161.79092   -288.41522   -356.06391   -423.79098   -244.76616   -395.25185   -270.44032   -449.85064   -308.24888   -275.47604   -512.63441   -416.96126   -269.36091   -352.29368    -326.0984   -350.47919   -238.35486   -135.10396   -487.14933   -280.12689    -72.16693    -13.87955   -115.67137    -98.06113     65.58838   -274.19917   -264.92689   -187.97273    -384.0749   -596.38283   -199.06287   -284.25818   -315.57753    -71.14499     63.46667    463.46509    663.27049   1008.71274     939.4261    747.51061    502.74374     64.93271   -154.20728   -162.27985   -337.09161   -258.07347   -288.23468   -419.81837   -379.44336   -315.30678   -385.45657          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       70.50   -547.90082   -396.00107   -519.53878   -153.76383   -368.18489   -362.86983    -458.6619   -430.69938   -528.30238    -222.4863   -519.97799   -178.69943   -234.93645   -336.90376    -83.98006   -439.64157    -376.5248   -269.21778    -556.7583   -382.46857   -198.95596   -198.30197   -310.73705   -201.04258   -226.70001   -124.02137    121.09076    290.94536    254.95434    103.44314   -188.47286   -252.42112   -208.84958   -204.40279   -440.08387    -331.6167   -491.25906   -277.28964    -298.8405   -390.86614   -142.74268   -410.32755   -355.65981   -431.74623   -305.15279   -477.79695   -511.64006   -563.45879   -407.27519    -441.5662   -305.13213   -383.53822   -276.01451   -368.14808   -555.11992   -184.06621     -29.5962    -280.4986     10.40528   -177.65935    -16.73692   -176.36543     -303.166   -180.99085   -206.58719   -252.57628   -168.62871   -149.58697   -311.89565   -150.50412      63.1023    345.17308    921.97709    996.14323    931.69414    736.18767    345.79662     140.7053   -281.34676   -343.14054   -337.38088    -302.0899   -412.31026   -281.46519   -266.81246    -389.0006   -303.07043          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       71.00    -557.5207   -552.29149    -510.3926   -336.00268   -424.69858   -613.00647   -381.52505   -528.77217    -549.8548   -222.61442   -494.80434   -217.02081    -57.66734   -378.00224    -92.25239   -439.46147   -350.90348   -416.53891   -212.61937   -328.15769   -241.36348    -88.61933   -423.82444   -107.46788   -305.06547    -37.69163    247.24179    432.11321     201.5276    124.15917    -18.53431   -416.48228   -299.81338   -379.79526   -323.67017   -399.91591   -350.55078   -267.83287   -248.81511   -442.91409   -256.90491   -473.33712   -322.47962   -211.93157   -314.85803   -625.21291   -683.85338   -270.44619   -356.88074   -473.35178   -196.14161   -375.14812    -378.6271   -278.17883   -444.43973   -111.19352   -315.55652     104.3271     23.13001     62.02264    -36.54644     24.24833   -309.28949   -262.99983   -329.95684   -191.75745   -305.98287   -191.85751   -396.88843     -121.186    303.08249    667.36783    987.55362   1220.66195   1150.61237     797.8584    442.59357      2.22165   -191.55171   -356.45025   -349.90631   -348.62368   -237.67686   -251.64771   -326.68513    -454.6665          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       71.50   -567.43305   -400.51505   -463.00723   -452.25381   -242.40176   -910.42436   -360.90565   -598.19586   -488.96243   -197.30362   -373.70073   -295.24037    -203.1469   -607.83747   -181.82587   -432.77258   -314.57675    -599.9633   -330.32895   -525.26241   -333.30446   -128.39433   -429.01983   -185.27099   -182.26018    -26.41289    344.71461    283.27081     59.82219       3.0424   -192.26509   -354.16429   -298.77545   -319.10939   -258.63317   -423.69196   -118.65789   -197.25961   -315.81164   -258.72623    -615.6801   -217.68162   -231.92678   -141.13609    -341.9189   -433.53681   -616.84073   -181.37976   -371.58638   -303.71268   -259.05704   -470.65886   -406.70086   -308.33608   -145.40417   -278.61053   -196.28509    -33.69736   -123.41353    -83.38495     -74.2463   -129.14213   -111.01773    -308.3886   -182.92741   -351.49526   -392.46295   -185.27159   -114.85256     16.84182     205.2222    501.71118    731.43793   1064.51613   1197.23825    549.41772    581.95525     99.88206   -124.35464   -107.70425   -212.64817   -493.71099   -467.12871    -254.2635   -383.77372   -400.88769          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       72.00   -632.14194   -365.92666   -461.05199   -332.88056    -17.62577   -975.96106   -413.19807   -686.04284   -295.76171   -235.41634   -343.21758   -189.86103   -311.49904   -538.08643   -322.76679   -406.02423   -448.29505   -721.52198   -578.55248   -484.38235   -409.84523    -125.7212   -435.05276   -257.91765   -104.02123    -34.45694   -187.19349    156.17305   -158.15039      12.8885    -32.18945   -209.61735   -434.62683   -399.07486   -441.20021   -333.16153    -65.71631   -335.95118   -414.56534   -175.15019    -203.9509   -305.51929   -217.98273    -518.1313   -397.02624   -337.35317   -407.07297   -343.87893   -249.46234   -513.07957    -52.98946   -764.74457   -275.25695   -233.55286   -417.05085   -358.15001   -232.26728    -95.27003    -146.6854    -130.1108     57.69081   -303.33428   -384.29483   -563.54994   -427.30488   -341.88545   -270.71344   -365.49064     28.53026   -190.13688    118.25507    304.69911    478.90355   1119.00819    989.45787    784.17521    699.35684    112.27131    -87.72604   -336.55631   -352.27925    -456.2536   -357.79412   -460.26153   -411.10028   -557.72614          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       72.50   -672.96949   -463.18688   -512.46343   -333.48293    -64.45975   -764.14823   -464.08562   -578.64466   -178.13395    -239.2836   -387.32522    -23.20847   -328.08046   -451.19089   -235.34753   -384.34537   -581.26939   -479.01619   -201.09287   -343.76581   -534.09913   -366.79549   -410.22046   -386.86436    -48.77272     22.14112   -158.50057    -67.59353    235.76395     103.3168     35.83434   -393.89048   -389.89725    -448.9112   -501.32804   -391.37936    -348.0736   -442.08263   -377.12552   -251.55595   -314.71233   -368.80528   -679.12048   -566.53353   -584.90796   -474.42052   -361.48854    -321.0128   -298.81649   -344.77896    -263.0268   -352.21913   -241.71752   -345.89036   -307.29456    -466.6441    168.38461    -58.19188    -83.75754     23.80345      6.71677    -40.82583   -397.12716   -395.16461   -325.49315   -250.71083   -292.50903   -371.62229   -310.01831     59.75338    302.57688    264.49994    943.68766     937.7052    763.32328    820.85119    516.92506    -11.77728    150.49939   -276.50889   -408.04065   -310.04074   -226.80832   -468.14197   -370.85455          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       73.00   -692.24755    -618.0288    -511.1739    -335.7118   -234.39816   -635.38859   -448.46758   -466.51722   -243.60559   -122.77027   -408.24978     84.52597   -348.78169   -427.01242    -62.18492   -434.36962   -406.83705     -143.458   -164.33539   -391.78163   -489.85805   -286.02833   -430.59443   -380.21463   -419.31882     58.60361     59.64738     -80.2956    372.59566     46.12731   -209.55459   -238.62503   -480.96411   -522.96053   -446.76269    -671.7983   -280.97357   -440.56076   -304.74098    -378.7529   -411.07714   -380.10295    -571.1036    -295.0589   -428.02882   -505.84231     -369.736   -334.36809   -412.05917   -437.42363    -405.3006   -423.19966   -176.67704   -362.40717   -488.22684   -456.96056   -185.63689      4.26227    -97.64605    145.70251    159.94725     23.16553   -350.46384   -237.99833   -404.83546   -293.87786   -462.76883   -210.05408    -232.4879   -168.69116    132.00216    458.57781    844.17757    909.55289    751.89075    573.26866    437.76761    314.31525     -61.3141   -159.92678   -249.89745   -339.63381   -366.15507   -139.51591   -359.62906          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       73.50   -741.63943   -688.16627   -484.28487   -253.88236   -368.04105   -491.01165    -419.8027   -389.02917    -320.9965   -159.04158   -323.89777    195.15245   -296.51175   -379.43508    -43.11045   -218.21319   -272.93441   -159.89856   -389.65031   -322.12804    -258.5428   -194.60315   -180.15798   -318.35788    -362.6554   -224.78695      79.9722    176.46108    220.26423    -73.91071    -293.4723    -160.6021   -322.95809   -464.45194   -435.38713   -400.36881   -358.65019   -325.93254   -402.75534   -358.52532   -359.65499   -311.58986   -327.30595   -266.94722   -455.45492   -469.68404   -398.61523   -456.98583   -420.39508   -522.61091   -353.55126   -397.65056   -332.87425   -482.98711   -210.70992   -426.75948    133.74275   -296.72598   -173.54563      70.4768    -97.71762    -142.8404   -318.51215   -101.28408   -253.10041    -273.0605   -412.19615   -390.72252   -204.56749   -286.07856    372.46581     449.8257    856.61517    868.04266    989.64755    752.00257    644.60757     73.39757    -90.44759    -96.33723     -300.095   -516.29289   -313.87798   -358.97094   -378.97295          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       74.00   -804.21893   -664.88777   -520.82027    -137.1432   -281.51609   -340.34327   -294.04174   -357.29638   -374.49874    -385.8816   -315.84394   -130.76402   -237.89174   -269.88853    -99.13963   -146.50832   -178.48926   -192.61033   -344.43733   -212.79034   -117.79791   -367.04968   -249.59535   -175.73239   -196.41511       14.261    -115.9644    177.37911    304.69616    -75.32653    -322.8933   -357.32703   -249.86404   -250.52386    -500.9204   -292.48703   -279.86331   -312.00969   -372.51229    -394.8761   -489.85398   -372.00796   -128.02105   -303.92033   -537.15057   -310.73551   -138.77019   -544.36515   -424.93076   -332.93833   -320.65868   -367.81767   -325.24778   -376.65385    -223.4616   -280.62099    -36.85248   -337.04252    316.57292     15.83996   -119.24628   -358.11737   -130.33642   -268.27257   -395.22888   -496.60035    -91.04628    -398.2031    -286.3819    120.19414    257.04618    820.82328    689.68127    987.16598   1243.25479    852.47613    671.44146    341.55763     19.19418   -215.82023   -230.68989   -323.07041   -353.34133   -425.52071          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       74.50   -877.44967   -595.08882    -625.0886    -268.0984    -85.03784   -307.57328   -127.11738    -339.0239   -430.64322   -449.27931   -365.89888   -301.97264   -218.40741   -222.25288   -112.54959   -355.31968   -142.11722    -217.2568   -372.92405    -426.3543   -227.19231   -375.51252   -387.38407   -238.28975   -196.58739     11.05166   -100.98848      42.6766    107.87855     73.74582   -165.90607   -582.88975   -244.80065   -126.27365    -439.5619    -25.92549   -333.16292   -443.73365   -374.53322   -351.55045   -401.15454   -231.07786   -223.23813   -368.21919   -350.85704   -434.96706   -353.19793   -449.53577   -673.68751    -364.3294   -470.27149   -198.04229   -503.70864   -351.43042    -156.5942   -365.98753   -119.97945     44.67164    -85.73206   -179.45074     22.97503   -235.80775     83.08754   -386.08377   -152.81856   -241.27528    -389.6798   -213.76226   -239.72272    -47.37739    389.87556    528.66486    686.56399    743.56225    961.73427    934.75928    707.11576    343.33318   -185.82679    -16.24356   -315.68351   -316.35209   -292.93118   -450.99933          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       75.00   -917.65163   -523.85186   -591.68153   -418.64026    -42.95261   -334.85201    -49.98264    -296.5917   -453.50845   -373.57375   -301.59353   -161.62363   -211.93347    -364.3885    -153.7091   -304.63137    -189.6216   -331.64909   -578.12331   -654.71619   -636.32777   -475.65696   -424.71605   -282.07076    -20.74464   -188.38938     67.95931     22.81113     46.31469     71.93338    -84.70441   -364.19355    -198.4068   -447.38038    -369.7238   -283.03673   -392.04184   -351.36342   -424.05043   -447.04242   -331.71551   -304.11224   -388.70186   -262.24975   -241.89098   -538.79411   -205.05513   -409.57076   -326.88779   -358.18735   -321.98842   -179.01492   -333.47044    -507.1653    -393.3135     -87.3846   -105.45035    -53.09535    -70.40454     -2.67591   -243.29252   -180.24839   -279.03753   -328.92693   -436.08797   -400.31238   -179.04161    -400.3466   -260.82038   -267.71551    125.90128    919.95843    745.28356    815.93611   1080.23421    928.26849    515.90409    199.38567     -53.0865    -151.1977    -309.5115   -360.64039   -312.31966   -694.84333          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       75.50   -936.89579   -483.84246   -482.08446   -474.45335   -184.84314   -408.40306    -82.46921   -331.76626   -479.06449   -376.77455   -194.60054   -149.66189   -242.89272   -357.13027   -389.73254   -185.40948     -318.292   -333.65253   -522.78039   -721.48199   -453.05237    -655.8753   -227.28019     47.07587    -47.06382    -80.41969     20.92538    193.53172     15.56593   -168.84796    -85.18743   -494.84207    -266.7682   -432.40008   -370.13706   -404.57825   -240.90768   -514.21907   -472.96668   -526.17453   -208.60639   -236.15504   -503.59428   -334.77797   -332.01599   -546.44067   -492.18204   -391.80587   -176.43968    -439.5361   -552.71286   -296.63714    -354.3576   -402.36565   -419.30544   -157.50349   -179.77886    -22.61259    -83.97835   -150.22241     -1.50235   -168.45692   -188.73929   -475.42759    -239.7976   -342.23717   -285.00447   -395.17782   -254.42349   -157.79195     -52.6234    377.47592    557.27832    886.84844    843.08054    812.21629    628.22927    221.93454     65.32688   -363.33084   -341.39391   -303.51543   -427.35686   -288.61917          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       76.00   -930.87832   -439.48826   -412.30553   -360.13694   -360.67162    -519.7157   -160.11893   -509.20806   -504.64517    -422.6685   -242.91508   -397.40253   -277.31939   -230.90518   -411.65847    -440.9808   -299.62552    -78.46324   -372.68799   -545.16508   -315.05418   -478.35144   -162.07587   -238.78094   -225.17005    -89.65817    -60.44071    133.87727     20.26155    -61.37029    -81.70897    -305.7886   -229.52631   -360.01802   -347.01148   -527.83324   -523.66392   -352.72553   -379.60786   -381.77775   -399.46844   -395.17668   -325.58216   -320.47656   -435.97549   -427.55683   -259.90228   -315.18531   -201.17133   -166.40904    -304.5031   -210.97803   -433.97597   -213.22897     14.21792    -98.32471   -157.40174   -246.15567     17.65049   -134.79008   -285.88707   -160.70356   -295.72983   -142.86979   -187.10615   -269.43315   -526.69241    -314.9787   -324.87595     -92.0174    255.15867    597.64725    660.69208    915.18871    969.34682   1009.97909    390.58639    327.90148    103.52319   -210.54281   -275.73045   -371.99513   -253.50037          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       76.50   -725.82528    -399.1408   -526.71221   -267.66509     -396.038   -602.45354   -322.61854   -646.82274   -414.29768   -448.81744   -443.26607   -544.17159   -442.74821   -206.03931   -219.25988   -544.12886   -378.21322   -169.50619   -266.89663   -376.29792   -309.37692   -389.75158   -223.84771   -220.24032   -489.46404   -246.93076    -120.4756     12.90617     96.33813     -74.5958     67.34661   -378.95669   -123.01435   -207.80783    -427.6557   -388.75339   -431.23405   -374.48807   -424.17803   -368.81734   -371.65456   -330.21487    -338.2834    -372.6279   -356.06074   -365.81655   -321.30092   -422.18342    -314.5627   -379.70769   -362.82236   -230.78511   -461.79665      78.7701   -137.38765   -222.83847    -82.26832   -157.84324     74.93015    145.03043   -134.72807     -20.0976   -236.16893   -265.70815   -245.05677   -313.16449   -494.09617   -444.87239   -158.07398   -139.29981    267.17397    466.23259    858.23104    837.39277    1251.8966    766.88071    553.89975    249.86579   -154.72633   -113.77141   -246.65996   -329.31339   -290.97087          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       77.00   -515.12676   -397.98605   -594.81059   -217.76716   -456.25993   -481.89652   -475.16412   -542.80103   -304.25352    -413.2262   -458.35672   -496.01388    -564.6346   -212.53688   -136.96301   -283.85703   -380.92905   -556.39418   -273.22148   -271.35761    -65.77051   -471.50095   -193.47268   -311.04472   -332.61923   -295.21432     88.76331   -101.95538   -207.78033   -106.01989   -248.40756   -136.20229    -86.32198   -302.62061   -245.49897   -367.20915   -362.26069   -267.43022   -391.80726   -203.42875   -308.26191   -490.33592   -523.24318   -291.18104   -316.21694   -328.42159   -533.09739   -159.34062   -345.99676    -381.5499   -421.42794    -449.6464   -214.91154   -324.34979   -376.15582   -378.58869    -58.27677     -3.92974    -147.0096   -216.56182    -58.67255    -43.06076    -79.11368    -375.2018   -281.23031   -176.96127   -488.79189   -375.04431   -194.13734     37.90267    -37.27658    351.45449     541.4543   1000.99273   1009.46588    850.17823    917.32372    364.98406    129.55828   -187.55308   -408.86474   -351.61667   -340.47213          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       77.50    -334.2441   -409.75561    -551.1645    -168.2224   -570.22873   -364.59122   -587.37147   -426.17715   -209.54526   -358.11559   -176.77866   -397.61555   -619.74704   -291.15453       5.4694   -385.08821   -358.09848   -428.25081   -247.72964   -165.46114    -24.06894   -487.77734   -298.93579     -543.788   -363.16775    -42.09772     -3.04054   -142.17319     12.16941    -43.81747   -213.53599   -141.59757   -277.65215   -283.11238   -432.21898   -450.88361   -288.72655   -165.44433   -280.13974    -479.6948    -383.4967     -620.942   -315.95484   -374.50144   -361.29093    -380.8727   -291.43876   -284.01973   -276.48535   -260.02926    -478.7531   -530.44327   -295.52096   -327.34428   -318.31993   -256.20291    -28.21979     35.47875   -108.45627   -241.31466   -118.35644     30.10514     31.09574    -215.8805   -250.02749   -375.66382   -459.04511   -435.15004   -119.89656   -222.62833    -41.91697    322.29027    838.53897   1247.69442    952.33795    878.86382    711.18014    308.57442   -117.56271     -77.5296   -291.88122   -276.61167   -351.56685          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       78.00   -309.56391   -421.03707   -457.37246   -211.66927   -670.12773   -378.26133   -687.56331   -307.66444   -164.45204   -347.60281    -86.63967   -324.97189   -503.51897   -423.98437   -138.27229   -546.58934   -348.08236   -281.61815    -17.55803   -131.02073   -345.06192   -384.34368   -461.57709   -302.90259    -343.6483   -212.42039     15.00274    216.77035    175.03058     66.12538   -276.22865   -273.19075    -333.8964   -247.85276    -431.2905   -381.58298   -372.31539   -285.47879   -292.91558   -437.10734   -355.92592   -259.31169   -275.05483   -299.30463   -391.24978   -397.48665   -376.20059   -327.19406   -230.09996   -275.74753   -154.38824   -257.98817   -328.12916    -226.5322   -228.55856   -158.01755   -296.16912    -57.05818     11.27189    -19.35779    -79.15302    -57.70963   -265.41366    -323.3719   -308.60668   -346.42888   -435.13479    -399.3036    -407.4026   -218.25978     -148.131    221.88183    768.21446    878.95012    908.51527     950.0809     740.1029    220.86771    195.64623    -81.74679   -468.51124   -526.53101   -255.33061          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       78.50   -247.26455   -471.46889   -402.38674   -409.24662   -684.05764   -446.50385   -606.40109   -190.75305   -274.00825   -350.81567   -128.86375   -357.61736   -436.38921   -342.22675   -274.73705    -396.9667   -435.01837   -216.13161   -257.24802   -238.30376   -294.33669     -377.336   -318.43067   -163.68855   -295.33421   -153.19276    178.24017     49.92766      66.6684    -74.75978   -136.16926   -267.30389   -276.18503   -159.25347   -234.41026   -247.94501   -198.72732   -313.76561   -302.38435   -547.36995   -202.69388   -309.67881   -335.25969   -410.04144   -295.77927   -407.89518   -448.88303   -468.29849   -236.53711   -445.49677    -251.9726   -296.38658   -255.96938   -297.71716    -355.9598     -214.784   -110.81053    -169.3383    168.18248    180.11888    -64.75621    -72.30836   -137.11418    -209.9982   -389.60454   -222.93886   -223.35969   -175.02868   -254.95579   -424.93135     32.04447    179.52504    830.13161    907.20746    912.76698   1022.86153    852.28908    427.56427    -29.02477    -128.7628   -150.63338   -398.56963          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       79.00     -191.357   -524.51637   -572.39208   -590.41662   -546.22614   -449.75845   -500.01361   -152.41273   -373.08394   -255.64513   -165.86675   -456.90786   -407.85958   -215.30791   -325.55623   -235.35478   -584.20133   -269.32694   -514.19817   -472.67253   -246.21548   -611.63829   -489.25736   -205.91261    -69.90812   -210.38449   -109.02782    -31.36231   -205.00369   -143.05408   -232.27544    -318.8844   -295.25442    -20.57332    -361.8228   -341.48489    -88.50543   -265.48104   -418.97712   -424.22767   -252.07574   -345.10148   -108.94924   -379.37943   -153.46535   -395.67865   -385.87816   -341.12435   -155.62421   -335.85201   -413.59229   -187.36234   -408.37919   -292.54087   -268.63938   -270.39465   -513.24897   -119.31189   -159.51316    -174.6724    -44.60244   -323.71799   -220.28369    -228.4638    -234.9735   -166.14119   -165.21003   -196.25417    -36.79237   -186.48601    199.93134     281.1105    713.13698    891.19289    897.35726    846.12577    631.74321    310.95221     27.32342   -143.20084   -205.25042   -285.76042          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       79.50   -282.40753   -595.91552   -805.25262   -546.25535   -373.95952   -335.11133   -492.34976   -126.32746   -533.59705   -141.58377   -224.11725   -419.56019   -314.22936   -294.02978   -364.37225   -134.61536   -451.10216   -307.40595   -425.25108   -520.77895   -185.46495   -277.74698   -501.93026   -265.87469    -62.23796   -303.80804     67.61872   -195.11059    -52.90405   -161.34423   -368.43778    -99.94786   -296.07065   -173.92183   -248.38403   -273.95974   -332.79156   -291.08784   -401.55294   -223.00023    -239.1733   -405.97148   -363.77542   -107.53591   -320.76637   -153.40802   -412.35691   -428.80017   -107.37033   -172.21611   -245.46953   -345.16505   -219.07865   -163.08848   -345.61928   -113.63792    -162.7367    -53.38856   -151.83578      5.74595      7.42165   -106.04105   -284.42647    -335.7154   -210.15982   -329.31149   -283.80665   -411.63274   -168.73653   -280.50969    149.09487    507.77267    806.65422     760.1184    752.43362    699.29001    660.40729    263.00841     63.94198   -278.09674   -147.99749   -368.48408          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       80.00   -415.60603   -756.04936   -974.24548    -486.2735   -373.27947     -222.746   -524.12208   -161.53505   -703.41996    -43.28048   -189.46099   -339.55578   -345.71543   -399.58403   -283.56471   -219.31739   -375.05698   -271.09664   -373.65473   -502.68124   -412.51639    -142.7559   -286.60872   -402.03877   -112.15137   -314.86726    -59.38157   -290.17011     95.80588   -176.99834   -234.27267    -367.0441   -317.67251    -36.25947   -333.71054   -168.50386   -272.97435   -334.08341   -307.62095   -420.44603   -395.34939   -551.74624   -183.72217   -263.31379   -533.58368   -316.30312   -565.74334   -482.30078     -402.616   -487.56466    -327.5556   -247.92572    -201.1052   -122.09788   -340.92013   -334.55363    -95.02816   -132.81941   -106.84172   -240.57189    -43.99911     -5.89024   -144.95535   -358.75122   -391.41211   -215.43739   -558.37677   -310.35342   -275.60068   -127.04337    -64.77177    387.60237    505.14945    854.01978   1034.36811    904.25589    728.81356    337.63569    124.66343    145.20652   -231.26874   -181.88248          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
;


_pd2d_meas_2theta_phi_intensity_up_sigma
;
         241   4.00         4.20         4.40         4.60         4.80         5.00         5.20         5.40         5.60         5.80         6.00         6.20         6.40         6.60         6.80         7.00         7.20         7.40         7.60         7.80         8.00         8.20         8.40         8.60         8.80         9.00         9.20         9.40         9.60         9.80        10.00        10.20        10.40        10.60        10.80        11.00        11.20        11.40        11.60        11.80        12.00        12.20        12.40        12.60        12.80        13.00        13.20        13.40        13.60        13.80        14.00        14.20        14.40        14.60        14.80        15.00        15.20        15.40        15.60        15.80        16.00        16.20        16.40        16.60        16.80        17.00        17.20        17.40        17.60        17.80        18.00        18.20        18.40        18.60        18.80        19.00        19.20        19.40        19.60        19.80        20.00        20.20        20.40        20.60        20.80        21.00        21.20        21.40        21.60        21.80        22.00        22.20        22.40        22.60        22.80        23.00        23.20        23.40        23.60        23.80        24.00        24.20        24.40        24.60        24.80        25.00        25.20        25.40        25.60        25.80        26.00        26.20        26.40        26.60        26.80        27.00        27.20        27.40        27.60        27.80        28.00        28.20        28.40        28.60        28.80        29.00        29.20        29.40        29.60        29.80        30.00        30.20        30.40        30.60        30.80        31.00        31.20        31.40        31.60        31.80        32.00        32.20        32.40        32.60        32.80        33.00        33.20        33.40        33.60        33.80        34.00        34.20        34.40        34.60        34.80        35.00        35.20        35.40        35.60        35.80        36.00        36.20        36.40        36.60        36.80        37.00        37.20        37.40        37.60        37.80        38.00        38.20        38.40        38.60        38.80        39.00        39.20        39.40        39.60        39.80        40.00        40.20        40.40        40.60        40.80        41.00        41.20        41.40        41.60        41.80        42.00        42.20        42.40        42.60        42.80        43.00        43.20        43.40        43.60        43.80        44.00        44.20        44.40        44.60        44.80        45.00        45.20        45.40        45.60        45.80        46.00        46.20        46.40        46.60        46.80        47.00        47.20        47.40        47.60        47.80        48.00        48.20        48.40        48.60        48.80        49.00        49.20        49.40        49.60        49.80        50.00        50.20        50.40        50.60        50.80        51.00        51.20        51.40        51.60        51.80        52.00        52.20        52.40        52.60        52.80        53.00        53.20        53.40        53.60        53.80        54.00        54.20        54.40        54.60        54.80        55.00        55.20        55.40        55.60        55.80        56.00        56.20        56.40        56.60        56.80        57.00        57.20        57.40        57.60        57.80        58.00        58.20        58.40        58.60        58.80        59.00        59.20        59.40        59.60        59.80        60.00        60.20        60.40        60.60        60.80        61.00        61.20        61.40        61.60        61.80        62.00        62.20        62.40        62.60        62.80        63.00        63.20        63.40        63.60        63.80        64.00        64.20        64.40        64.60        64.80        65.00        65.20        65.40        65.60        65.80        66.00        66.20        66.40        66.60        66.80        67.00        67.20        67.40        67.60        67.80        68.00        68.20        68.40        68.60        68.80        69.00        69.20        69.40        69.60        69.80        70.00        70.20        70.40        70.60        70.80        71.00        71.20        71.40        71.60        71.80        72.00        72.20        72.40        72.60        72.80        73.00        73.20        73.40        73.60        73.80        74.00        74.20        74.40        74.60        74.80        75.00        75.20        75.40        75.60        75.80        76.00        76.20        76.40        76.60        76.80        77.00        77.20        77.40        77.60        77.80        78.00        78.20        78.40        78.60        78.80        79.00        79.20        79.40        79.60        79.80        80.00        80.20        80.40        80.60        80.80        81.00        81.20        81.40        81.60        81.80        82.00        82.20        82.40        82.60        82.80        83.00        83.20        83.40        83.60        83.80        84.00        84.20        84.40        84.60        84.80        85.00        85.20        85.40        85.60        85.80        86.00        86.20        86.40        86.60        86.80        87.00        87.20        87.40        87.60        87.80        88.00        88.20        88.40        88.60        88.80        89.00        89.20        89.40        89.60        89.80        90.00        90.20        90.40        90.60        90.80        91.00        91.20        91.40        91.60        91.80        92.00        92.20        92.40        92.60        92.80        93.00        93.20        93.40        93.60        93.80        94.00        94.20        94.40        94.60        94.80        95.00        95.20        95.40        95.60        95.80        96.00        96.20        96.40        96.60        96.80        97.00        97.20        97.40        97.60        97.80        98.00        98.20        98.40        98.60        98.80        99.00        99.20        99.40        99.60        99.80       100.00       100.20       100.40       100.60       100.80       101.00       101.20       101.40       101.60       101.80       102.00       102.20       102.40       102.60       102.80       103.00       103.20       103.40       103.60       103.80       104.00       104.20       104.40       104.60       104.80       105.00       105.20       105.40       105.60       105.80       106.00       106.20       106.40       106.60       106.80       107.00       107.20       107.40       107.60       107.80       108.00       108.20       108.40       108.60       108.80       109.00       109.20       109.40       109.60       109.80       110.00       110.20       110.40       110.60       110.80       111.00       111.20       111.40       111.60       111.80       112.00       112.20       112.40       112.60       112.80       113.00       113.20       113.40       113.60       113.80       114.00       114.20       114.40       114.60       114.80       115.00       115.20       115.40       115.60       115.80       116.00       116.20       116.40       116.60       116.80       117.00       117.20       117.40       117.60       117.80       118.00       118.20       118.40       118.60       118.80       119.00       119.20       119.40       119.60       119.80       120.00       120.20       120.40       120.60       120.80       121.00       121.20       121.40       121.60       121.80       122.00       122.20       122.40       122.60       122.80       123.00       123.20       123.40       123.60       123.80       124.00       124.20       124.40       124.60       124.80       125.00       125.20       125.40       125.60       125.80       126.00       126.20       126.40       126.60       126.80       127.00       127.20       127.40       127.60       127.80       128.00       128.20       128.40       128.60       128.80       129.00       129.20       129.40       129.60       129.80       130.00       130.20       130.40       130.60       130.80       131.00       131.20       131.40       131.60       131.80       132.00       132.20       132.40       132.60       132.80       133.00       133.20       133.40       133.60       133.80       134.00       134.20       134.40       134.60       134.80       135.00       135.20       135.40       135.60       135.80       136.00       136.20       136.40       136.60       136.80       137.00       137.20       137.40       137.60       137.80       138.00       138.20       138.40       138.60       138.80       139.00       139.20       139.40       139.60       139.80       140.00       140.20       140.40       140.60       140.80       141.00       141.20       141.40       141.60       141.80       142.00       142.20       142.40       142.60       142.80       143.00       143.20       143.40       143.60       143.80       144.00       144.20       144.40       144.60       144.80       145.00       145.20       145.40       145.60       145.80       146.00       146.20       146.40       146.60       146.80       147.00       147.20       147.40       147.60       147.80       148.00       148.20       148.40       148.60       148.80       149.00       149.20       149.40       149.60       149.80       150.00      
      -40.00    233.54349    224.10645    208.27566    216.41348    205.63116    216.02289    204.60888    197.05559    186.23034    183.14186    184.47784    179.19186    186.02513    181.23491    182.75614    176.69782     167.5986          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -39.50    232.66974    223.49483    209.67221    217.60858      209.059     220.4587    201.25281    198.89958    188.39998       177.37    185.73035    182.52026    184.40737    179.30679    180.05557    173.97157    169.35174          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -39.00    232.51012    223.97423    215.90039    218.09426    211.72563    216.21584    199.31511    196.81975    188.71256    184.65146    186.43862    183.56351    181.07582    181.70731    175.49167    170.83482    169.69325    167.02211          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -38.50    232.61476      224.171    221.85087     217.1526    213.16458    211.96458    196.77904    193.95383    189.21666    186.24361    186.57205    183.98342    182.44787    183.83478    174.68287    169.67856     170.3779    162.26836          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -38.00     232.5503      222.426    226.86797    215.82756    212.43038    211.02957    195.80521    192.92192    192.75947     186.2519     184.0049    186.04187    185.59195    183.75307    172.77978    168.97817    172.75163     166.7277    158.95614          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -37.50    231.53965    219.79472    224.41141    214.12238    212.57219    210.51038    198.28605    192.71909    197.71459    189.18966    186.22118    187.75275    185.64275     182.6954    176.61038    167.88444    169.78491    166.51928    164.54764          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -37.00      229.571    216.42607    221.95407     215.9899    213.95626     209.4996    199.56638    192.70378    198.87677    190.69984    190.58618    185.44325    185.54426    181.28515     175.2209    167.26843    169.81113    169.83847     171.9999          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -36.50     228.9676    213.77403    223.14889    220.70069    218.80729    207.53389    198.57968    193.94826    193.17537    191.35845    189.09932    181.56635    191.54335    180.06618    171.49689     167.0191    173.53623    168.27729    172.63965    166.19627          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -36.00    229.55962    221.57707    226.93724    224.78615    220.78543    205.26777     196.1312    195.32487    191.23812    190.67976    187.77314    182.69407    190.49643    178.88234    170.51658    166.65738    175.54164    168.48932    169.04799    162.96702          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -35.50    230.14804    227.74761    227.54743    223.19286    219.69472    203.09549    196.79081    194.88663    191.48823    188.74749    188.57533    186.73938    185.22285    178.49354    174.94622    168.69647     184.4311    170.18429     169.7373    166.87068    159.44535          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -35.00    231.61355    231.95199    227.46889    217.83406    217.54551    202.18693    199.37194    194.94057    192.73565    183.60377    188.09477    188.45244    188.54325    175.69517    181.23807     173.8582    178.27102    173.71431    171.24715     169.4364     149.7518          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -34.50    233.54579    231.51579    228.42689    213.80791    216.75867    202.53424     199.2307    197.71553    194.73277    179.89582    185.90347    191.76891     194.7271    175.63607    180.40006     177.4566    175.12981    176.41758    171.07216     172.8724    159.17606      163.769          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -34.00    234.15662    230.23873    227.96721    213.94891     216.9032    203.32533    196.42059    201.25748    194.96653    185.97888    185.96423    196.00146    193.38419    180.12136    177.61477    174.53958    174.14777    177.09429    168.46979    174.32987    161.97393    159.06323          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -33.50    233.45952    229.90289    225.94966    213.85822    215.39985    199.50408    194.56079    200.09114    192.32872    193.46326    187.27721    196.37323    189.48046    179.87566    173.75785    173.19434    173.78964    179.79806    167.53465    165.93554    160.18497     156.0124    158.34144          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -33.00     232.5815    230.38959     224.0867     212.2275    212.01099    198.06087    196.93047    197.10465    193.11121    194.03963    188.95663    192.35958    186.12041    180.03179    172.64717    173.00156    173.46054    174.02931    170.75086    165.90399    162.28039    153.99236     157.2169    161.34156          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -32.50    232.08944    232.00153    224.04091    214.14215     207.8074    200.17296    199.66181    196.18518    192.65742    190.50944    191.90336    190.50384    181.01795    183.74525    174.03373      171.442    174.22647    168.24722    172.00799    169.76026    162.33039    162.53122    160.50452    164.50957          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -32.00    232.43748    232.84804    222.68387     215.7015    204.82007     204.4343    200.57901    193.24762    193.71793    189.36333    190.99623    190.39985    179.50834    183.49193    173.07973    174.77602    173.05347    169.58739    168.09207    168.96356    166.49064    164.21576    161.16237    162.24206      164.209          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -31.50    233.20043    233.41327    220.03223    214.03079    206.23132    208.26652    203.76374    189.02624    195.62395    189.49545    187.98087    186.34037    183.15687    178.93398    174.47648    176.91042     174.2266    173.05467    168.71601    164.13334    165.04325    158.58495     165.5952    164.77718    163.75794          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -31.00    235.98503    235.85247     220.5463    214.27747    208.80569    206.65895    204.07904    186.77924    195.74107    189.09082    187.93687    183.84539    184.38295     176.2044    169.87662    177.31281    176.81652    177.39906     174.1401    161.95498    164.12537    162.71519    166.43415    168.96919    163.17753    178.98754          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -30.50    239.33531     237.0788    223.83775    217.24501    209.22601    204.82666    198.47462    191.49853    193.26759    190.78892    187.32854    184.51547    187.31941    181.20782    167.44095    176.10253     175.8525    174.73224    170.62832     166.0402    165.46979    162.46885    165.09006    166.44147     166.4721    181.63966    186.84213          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -30.00    243.21946    237.11014    227.60561    221.34298    206.83248     205.8839    194.24344    197.11067    190.41321    191.70217    188.88163    187.32349    194.20871    182.45652    170.50843    171.56062    174.40144    168.74951    168.05467     167.1883    165.63956    161.42911    160.96785     164.8078    174.23471    178.42379    188.30295          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -29.50    243.40133    235.44319    227.56324    220.61423    208.46059    201.59715    197.21746     199.6107    189.40839    191.36483    192.04378    185.54255    191.83766    176.11275    172.80577    178.13214     175.7923    170.24678    166.90306    169.25854    165.69296    164.43194    163.09852    163.01879    179.17993     181.8132    186.33295    183.16734          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -29.00    243.82729    235.21875    227.22972    219.27445    210.42889    195.60814    199.77843    202.57469    191.84808    189.98513    190.72391    186.00202    185.37077    170.48855    172.15467    179.62911    178.25192    170.83367    167.22958    171.77163    165.11354    159.83783     161.5944    164.61477    174.46448    181.35728    192.73142    187.65507    170.84962          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -28.50    245.08425    237.46535    226.72542    217.49595    210.03597    196.08517    199.87594    202.72367    195.60369     192.7612    189.33213    185.33361    182.00845    175.48739    176.47177     178.8169    175.86684    169.69032    169.49252    169.28579     164.3143    157.03288    162.79376    169.12456    173.75232    180.35669    192.39583    191.33381    168.04477    158.02699          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -28.00    246.14331     239.4078     227.4318    215.73704    206.88314    201.77573    198.82566    200.02246    200.08569    193.19164    188.60345       183.08    181.48071    175.97408     179.2278    180.97122    176.29043    169.42885    172.36052    168.30154    163.93211    160.63995    165.80467    167.26754    171.08894    181.03138    191.86986    194.35653    173.13075    158.87674    154.00351          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -27.50    245.84467    240.29214     229.1474    214.75778    206.29931    204.79152    198.70605    197.19665    202.70704    190.35737    186.42607    183.88906    180.10882    177.29362    177.62828    178.08066     180.1197    175.51583    173.44275    165.61323    163.90761    167.80445    167.54861     166.3628    169.65351    182.25799    192.71894    198.94616    178.20244    162.95326    152.08699          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -27.00    244.28748    240.20459    231.89528    215.33918    207.15103    204.01488     196.8715    195.66528    197.90935    191.37353     185.3371    183.05151    178.20739    181.39671    177.42884    175.85637    177.86191    175.97448    174.32367    167.56149    166.15683    167.01698    168.80464    165.29289    167.42812     183.8482    195.84256    204.21073    179.39893    161.64134    154.46508    148.92121          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -26.50    242.77155    239.31112    232.26334    215.95505    207.34053    202.17981    193.53553    195.31286    194.03469    194.01398    183.25411    181.21011    177.69431    180.88123    181.17803    174.56199    174.56224    172.54515    172.96222    170.01099    169.00832    168.91698    167.50018    166.81124    168.22599    179.91731    201.50996    207.13862    178.35834     162.1297    157.40266    147.62346    146.60191          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -26.00    242.82875    239.20604    231.12436    216.93657    207.38293     201.6649    194.10118    199.70828    193.93669    193.64864     178.6771    180.01377    174.21383    177.38346     181.7425    174.33065    178.38681    169.13423    168.67988    167.38856    170.41952      166.224    168.89045    165.12941    168.37987    181.93009    203.04402    202.37436     189.4892    159.97957     156.9357    151.18417    145.77304    141.72179          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -25.50    243.85024    240.85807    229.20191    215.22165    208.19874    202.54507     199.5958    204.65618    194.12843    193.51469    177.29895    179.37834    173.11082    184.17255    174.22307    177.44291     179.7482    167.65854     165.6906    170.46845    166.64155     164.5241    165.63476    162.34483    167.33045    186.19795    205.66403    204.78893    189.57938    164.95141    157.71886    153.52712    147.91234    145.01272    144.00611          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -25.00    246.21702    241.90342    227.56438    209.65992    210.66341    203.24767    201.45312    204.80182     195.1887     192.3679    179.51051    179.47295    175.96234    189.86046    170.95357    182.79567    173.00189    171.20112     163.0409    171.85975    167.15036    163.72281    164.28804    161.92667    175.72115    176.40122    206.31309    204.85825    188.04631    166.54934    157.88557    152.08779    147.73714    149.13253    143.97865     144.1241          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -24.50    248.11533    241.50812    225.50123    204.77119      214.844    203.70678    204.32205    204.22258    195.46776    191.72182     182.5658    181.20161    178.36502    185.64218    177.22429     181.4936    168.99241    173.66028    164.95193    169.07764    166.88383    167.26005    163.10943    161.81016    172.88911    182.77815    206.91868    204.65472    186.58564     169.3127    157.91394    151.54992    149.42652    148.02697    144.48511    142.98306    144.54101          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -24.00    247.95725    239.90413    224.45557    209.12315    216.15906    204.34281      209.154    206.33752    196.05643    192.69479    188.72891    185.28727    178.21524    181.28116    179.06712    179.95378    170.45974    173.27692    169.56219    166.80235    166.27278    170.88392     174.3184    161.89866    168.93284    187.96252    209.90674    215.14153    194.80099    173.44581    159.29053    150.05472    149.13426     144.7015    148.77357    141.64847     144.5635    141.76487          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -23.50    246.41304    239.77021    227.31079    214.23495    215.67951    205.82773    207.62047     205.9087    199.12772    191.99115    191.16445    186.73598    180.74502    179.21299    177.79945    178.17901    170.96697    170.85707    171.22562    167.29319    167.46067    172.47821    172.02497    162.79912    165.40826    188.67693    209.42984    211.76627    197.25652     176.8594    165.15244      150.411    150.49608    145.09619    144.91263    142.35009    146.43038    140.90244    139.81504    143.16569          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -23.00    248.14208    239.86302    232.15509    217.68836    212.03076    207.78897    201.39412     204.8044    201.50863    189.29982    188.97262    188.26432    181.76048    181.24266    176.95813    175.84779    171.82991    171.19991    168.93499    170.33234    168.36599    166.34582    169.90223    164.17388    171.18034    186.55149    207.76238    212.15891    195.86466    171.16322    158.05886     152.5738    152.02776    144.13144    144.50838     145.2578    142.34172    138.88438    139.27277     137.5551    143.23554          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -22.50     250.3811    240.34041    232.29355    214.75603    208.39835    209.35616    199.52335     208.1258    197.90177    187.61152     190.0838    187.36184    178.60891    183.57569    180.49187     177.7957     172.4952    171.81398    167.41329    171.83298    167.98798    167.07452    168.60827     162.5218     172.7921    187.51565    203.92205    221.28732    194.67042    172.90998    155.97752    153.49475    150.14833    148.55324    145.43956    148.11059    139.15162    140.15837    140.33997    139.95549    140.25093    133.33294          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -22.00    252.88123    240.48508    225.00485      211.359    206.36269    211.00017    202.24857     211.2369    195.97151    189.86341    187.46665    182.90035    174.33819    184.25647    181.13516     176.8606    174.01292    175.26295    170.67357    168.48174    167.90351    167.86932    165.44105    161.37599    165.78768    189.07387    207.00477    218.71181    200.57331    176.07545    156.15768    154.26482    149.60387    150.22102    149.36946    150.01006     143.1258    140.96028     143.0086    136.90173    141.74611     136.3726    140.25134          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -21.50    256.40264    239.63712    218.13872    210.02492    206.69297    211.85572    202.43351    208.62198    198.46581    191.97239    184.80335    182.46787    178.02353    187.65708    178.55496    176.91793    172.07074    177.54339    172.47544    166.86162     167.3583    166.63621    165.73957    161.14737    168.37295    192.83558     212.0475    217.06838    203.88627    174.31456    159.49624    151.09349    152.12375     152.1392    149.53891    151.35537    140.71952    143.06209    145.73536    141.97285    138.97087    140.02925    139.66354    130.78569    137.32584          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -21.00    259.13324    237.87023    218.72318    210.55855    208.36745    211.14921    204.35677    202.87878    200.11586    190.23515     187.6298    186.39019    184.35733    187.60514    178.94943    181.03869    171.16083    172.08655    170.72514     164.9959    164.77554    169.34698    165.29973    162.09953    168.43481    192.83985    213.26815    218.31031    208.20707    173.12761    162.91478    150.71105    151.54578    149.58044    146.93239    145.68864    145.04784    143.02976    148.12802    141.26385    143.11599    141.05553    138.55345     132.7028     133.1454    130.90836          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -20.50    260.63629    241.20435    221.85267    210.24693    210.77492    208.90967    209.76877    200.38697     200.0156    188.73716    187.94044    188.02354    184.08158    183.29198    179.63515    181.70376    175.40957    171.72171    170.70283     169.9776    164.58298      173.389    166.05379     165.3515    171.30702    189.12486    209.79911    221.04282    206.62245    171.29182    156.97345    149.73844    149.39215    153.06797    145.23768    145.36399     141.3105    145.67877    144.06197    141.20841    143.52139    141.20485    139.55667    139.56792     132.4428    133.58657    127.59268    135.83639          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -20.00    258.27857    246.00262    225.20907    213.11397    212.64973      208.555    211.34576    203.98255    199.91496    188.04906    184.40258    189.12189    184.20018    180.46861    179.78925     181.4778    176.54495    171.37453    167.86498    172.92482    160.33258    171.92146    165.98303    163.77853    168.06681    188.59323    218.36618    226.73889    201.88906    172.47893    152.39562    153.01606    155.23958    150.57399    146.81593    145.76401    148.76864    145.77766    141.64712    142.37106    145.38248    143.18774    141.34898    143.09421    137.20363    136.17567    137.18362    131.84898    131.99931    131.07508          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -19.50    256.68564    251.45864    226.64458      216.211    213.12479    208.23483    210.25889    206.34168    201.16344    185.82391    179.71309    189.86896    183.74907    177.89756    181.23205    173.21506    173.37267    170.57729    172.15784     169.0875    162.52131    166.52732    162.19314     163.3294     165.1065    190.61642    223.80381    232.22756    203.64691    174.14597    154.64983    150.65205    155.76382    154.31727    145.97123    147.29653    149.04973    144.27998    142.29533    142.77339    147.45782    143.61265    144.87939    137.84358    138.71363    134.11198    135.55486    134.88134     137.0291    131.20497    129.70039          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -19.00    255.01276     249.7635    230.43644    218.37468     213.6439    206.36005    208.40806    206.26663    203.41576    186.60956    180.47673    189.44173    182.26624    178.48772    182.40823    171.85567     170.7184    173.67788    175.20796    169.07155    173.42738    165.03076    163.17872    164.25671    167.20426    191.95076     219.4629    231.13896    207.10648    174.19102    160.09678    156.23531    152.37931    154.52302     150.6784    146.59849    148.60724    148.93945    145.40511    139.91247    142.60338     139.4379    137.86364    140.18568    137.94522    135.88539    140.86933    136.28287    137.26343    130.73766     131.6058    131.79947    127.64132          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -18.50    257.30884    246.02068    233.95358    219.85218    214.97816    205.65347    205.31304    205.91307    203.09041    189.78478    184.02763    189.57569    181.09412    180.16501    180.92074    176.65731    169.45867    174.57862    172.66347    172.75108    170.88736     170.8519    164.60084    165.12353    169.92714    192.24455    221.13012    230.35103    209.37893    175.22609    162.53991    156.35751    153.68987    151.43037    153.46181    148.16479    146.23267    140.25512     141.8014    143.37776    144.29441    144.60103    136.20339    139.85854    136.65235      137.143    136.26686    138.89286    136.18249    131.64017    129.88987    129.41538    129.22598    121.30417    127.99557          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -18.00    259.84686    244.39202    227.39395    222.11078    215.91816    209.71374    202.63001    207.69842    200.22072    190.60135    186.59741    188.77082    181.69376    185.23331    177.19192    178.91613    168.89021    175.90285    168.59459    177.73058    170.51639    171.13086    161.42722    169.76067    168.18368    192.05567    217.27477    232.28069    210.11686    177.58767    159.11138    154.26749    148.94931    148.66955    146.99108    148.77641    144.64579    146.62174    146.38026      143.315    139.43407    149.19819    138.37735    137.14153    137.26919    138.40822    134.31272    138.70411    135.40214    133.24236    130.57788     133.3436    129.99275    125.81633    130.93111    135.43889    131.61971          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -17.50    262.09543    243.59481    222.01458    223.48312    215.65048    215.82832    202.42146    212.10837    198.37395    188.09908    186.30813    188.61537    184.33769    187.13589     178.7374    178.00806    170.64069    176.96861    167.26523    175.73719    167.99394    167.04067    162.36223    170.29586    169.81603    196.39017    215.53362     232.7021    211.24222    181.32204    153.69149    151.95609    152.90855    150.64427     150.2613    148.92706     144.7288    144.24378    144.32842    147.63423    148.09366    144.96009    139.07068    140.77111    139.03872    136.08913    137.44719    140.56026    135.31435    129.89803    129.66243    134.43745    132.53738    128.49218    132.39491    132.21442    128.75376    135.46819    133.78183          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -17.00    258.98516    242.93454    221.90764    223.11788    215.53953    217.16794    203.14399    217.06891    197.45428    185.35525    187.80862    189.09631    184.05933    186.14464     180.2757    180.58749    174.64811    175.95665    174.43029    172.98953    169.50134     164.0308    162.34559    159.89957    172.01725    195.68863    219.07606    234.04288    204.06927    181.47513    157.82443     151.7972    151.07352    149.77279    152.01081    151.09956    144.87025    147.23379    142.37326    147.01465     143.0227    139.81808    136.12558    134.45648    140.97269    139.54106     135.3095    136.74199    136.90196    130.96989    133.39963    131.79546    134.64095    130.96511    130.99928    128.62012    134.23116    132.43696    136.59431    137.38539    133.48417    132.81511          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -16.50     256.6933    242.05041    225.48823    221.52798     216.6941    217.43989    206.98261    220.03735    197.39545    187.33308    190.59223    187.75606    181.61596    185.75954    179.38037    179.45763     171.0889    173.37115    175.36395    170.44385    169.96387    167.57971    159.83177    164.70026    173.29533    192.92029     222.5976    235.74883     210.5637    183.54214    158.12889    150.27462    146.12062     147.8632    150.15646    151.96665    146.20848    145.95811    147.34001    144.96913    141.49414    145.57387    136.89185    138.45091    140.17179    140.17562    131.44815    142.21229    139.34312    136.27366    134.47022     134.4154    133.71344    132.17242    132.66487    130.01228    130.51797    135.94241    134.17727    135.87514    133.64157     127.3378    125.88803    122.77985          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -16.00    255.73236    240.21249    228.32784    221.51203    217.24787    217.16911    211.21608    217.60964    199.93751    189.16956     192.7259    185.87364    180.31997    183.95782    176.72357    176.18247    168.79998    172.44034    174.57498    168.32011    169.38107    166.15654    157.85708    165.79106    166.14261    190.80411    225.22481    237.05817    209.55971    185.88262    160.17337    157.64837    149.19466    150.01277    147.24115    148.90038    148.60487    143.08126    146.35505    147.88871    145.47047    145.21489    141.67427    140.59139    137.73723    140.43036    137.84612    134.79626    133.82649    132.24496    134.07827    134.07219    133.07882    132.77409    134.63736    132.70467    134.90428    134.07376    138.97982    135.33008     132.7712     129.2023    123.90955    127.14676    121.87484    124.45584    119.01244          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -15.50    259.65791    237.84884    228.31223    221.82349    215.43209    217.58402    213.11754    211.90027    200.59755    189.44214    195.19783    184.50276    178.61882    183.39902    175.98235    175.89919    173.24622    170.85548    173.98086     168.4751    169.14042    163.05537    160.15703    161.78641       164.41    193.83313    224.52028    232.67105    210.58997    182.36025     159.4667    153.36775     153.2324    151.87192    144.51468    147.43036    150.99469    146.18595    144.35069    143.12711    140.72522    141.33423    140.74912    139.65638    137.03115    143.45757    137.96596    141.26168    136.28234    136.67138     136.0382    136.10696    133.36387     135.2228    130.50837    133.33727    135.01048    139.95781    142.09054    139.65586    139.04414    130.90255    126.84564    124.75249    127.21392    126.44674    122.39169    122.43161    122.54871          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -15.00    263.66818    236.15982    228.97136    221.63941     212.8998    217.83439    212.22578    208.35813    199.26104     188.7717    192.09917    182.70541     179.7687    185.47275    176.70299     174.4958    174.34547    172.12438    174.80285    170.34794    170.98312    167.32138     161.3263    161.65132    166.92538    197.29579     221.2646     238.4372     214.0593    179.90433    162.00487    154.01628    155.45197     156.3637    147.36236    148.80968    151.08746    150.14522    140.01303    141.73813    139.78584     139.3786    140.44246    140.26666    141.52474    140.89451    132.13491    143.81947     130.2266    137.62485    134.96935    135.23414    132.41572    131.35665    135.59187    135.58267    136.04387    141.39191      143.475    140.30211    136.64699    134.83788    130.98692    128.21671     123.5571    123.20271    126.79858    125.26691     120.8009     127.9338    130.24118      137.761          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -14.50    267.17615    234.95216    230.98716     220.2853    211.74759    215.19448     211.6415    205.05953    197.26758    189.86494    189.26335    183.52898    182.19795    185.80322    178.30291    174.48429    173.39155    176.84514    177.90437    171.69206    174.60925    170.18977    161.03753    163.71455    172.49809    194.09084    224.82939    238.03621    218.12181    183.23583    164.15478    156.05143    156.33205     155.4431    152.32804    145.28128    145.24284     150.1271    143.77116    142.73558     146.5678    141.97902    140.85214    142.40946    133.37335    140.30081    139.39603    140.90903    138.56878    133.72831    136.44431     138.2987    131.97822    133.17023    131.67726    134.36619    135.83357    139.98228    138.12892    139.54881    138.88789     131.0785    131.17361    130.64294    127.72167    125.33831     128.1477    126.91518    125.29974    125.13582    134.12071    141.24349    139.12555    138.86123    127.10558    122.51604          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -14.00    266.51569    233.86874    233.49572    218.79657    213.61778     212.3764    213.11412    202.13981    194.19987    191.89602    186.49323    185.70876    185.97534    185.60251    177.35553    176.28833    174.00068    179.91681    177.16144     172.9985    172.84573    170.62845     169.9024    166.17697    174.68778     191.3106    224.47291    236.36507    223.57186    184.16088     163.3359    154.71741    152.24458    149.33911    146.79676    144.40873    144.91866    144.23147    156.78104    144.72065    144.46082    143.62164    143.53835      142.568    138.24035    140.20938    137.68059     137.8611    140.12566    134.97813    138.11258    135.08092    134.68539    136.28092    134.37043    134.23506    141.40913    141.17309    140.44622    136.04862    141.52289    132.60967    131.05097    129.06832    127.84328    124.55712    126.70489    124.93419    125.85157    127.22628     135.6357    144.98267    143.46245    135.98676    132.16861    124.49411    126.47464     126.5358    122.08744          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -13.50    266.19041    237.98705    234.81993    219.01513    214.24642    214.12272    214.58168    205.19743    192.21714    190.41054    184.18612    183.65826    186.68823    185.09799     176.8019    178.10239    173.49606    179.19586    174.67529    171.39893    169.44139    164.38624    171.23776    168.87753    170.57637    193.48729    224.23187    239.38017     222.1914    183.23441    163.16772    153.61939    154.63535    148.96652      146.914    146.97468    145.93914    143.33562    148.15876    146.89214    143.20429    139.16764    143.55981    138.79968    140.91164    138.08031    134.66059    135.84197    136.99206     131.2025    137.05922     129.7958    136.31797    133.60894    133.69727    136.17024    135.07476    139.63807      145.117    141.28731    136.17918    132.27197    129.81952    130.46673    129.65398    127.07765    126.49701    126.70036    127.71196    128.77881    139.91112    146.16745    148.14854     143.2415    130.63638    127.54277    125.81205    129.22771    127.54697    126.66041    123.10806    116.72992    115.94644          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -13.00    265.30553    242.71666    234.05984    220.69668    216.17161    216.86276    211.15582    207.26466    196.76847    189.11362    187.00596    182.12801    188.94754     184.3087    176.09896    177.97824    171.41172    176.45223    172.83726    172.11826    166.93251    163.53482    167.07088    167.79534     166.7496    190.51558    224.89817     236.0279    225.24109    186.28743     163.0686    154.01725    154.44164    147.64041    147.76965     146.7504    150.28989    146.73914    145.78806    146.79256    141.29431    140.31389    141.54917    134.73664    138.69779    146.50818    133.13295    139.22156    135.41962    134.94561     138.2157    131.53721    138.00436    136.45366    133.90187    134.24854    136.13548    140.61366     142.6534    141.16613    137.23135    132.07349    132.51724    130.28476    129.34525    130.94865     125.4211     126.0098    127.01733    129.47632    144.05531    146.60815    151.36183    146.42518    133.40881    128.96767    129.00303    128.60461    129.53354    128.31355    123.16228    124.49837    120.78677    119.15538    118.24705    113.68156    116.85549          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -12.50    261.29689    246.61267    232.12867    222.39023    218.50665    217.75886    206.74488    208.93002    203.47794    191.24244      188.352    183.51443    190.85554    178.86218     175.6854    174.25234    170.46034    171.86486    171.47475    174.43984    170.55158    162.77639    161.54567    165.13219    172.42894    191.75778    227.42855    234.63396     227.4626     189.2688    161.12398    151.75316    152.25923    152.34153    149.18349     149.8339     153.7363    146.41211    144.83616    147.99681    145.84138    148.32971     144.1626    141.01134    137.18564    134.86202    139.07072    140.71092    132.15859    136.90368    133.08487    136.81141    138.09356    134.66012    130.99867    134.25802    133.82182    140.58223    144.91782    139.35379    138.62388    132.00119    134.30471    128.16726    129.66764    123.24018    127.70969    122.84876    125.29215    127.86007    142.07153    150.80831    155.64384    151.92669    136.37109    132.11773    129.56107    130.45495    130.42803    128.39059    124.81351    118.01576    122.81907    117.97354    117.79078     115.4757    114.91846    114.95538    118.85527    122.36911    129.10897          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -12.00    256.95664    244.54673    230.55938    226.35517    219.34386    216.60054    207.40048    210.59989    203.38922    193.51746    188.76691    185.00183    186.96779    177.61591    176.44337    173.20065     170.2942    170.79715    171.06883    176.01899    169.09604    163.01844    161.21853    164.44982    171.06575    193.29663    230.87983     238.1479    225.34926    196.51572    161.92501    154.27289    152.05195     152.5467    150.07397    154.01449    149.74434    146.03073    145.32128    148.95026    141.64748    143.55098     142.4413    138.09828    140.79429    138.96055    139.25021    136.37904    138.61501     137.5712    138.01621    138.72275    134.64537     137.4595    132.37732    135.73593     137.2199    143.70544    139.97232    141.59421    133.64508    134.58695    127.01277    130.73916    129.92915    127.30589    127.51364    123.78236    129.46396    135.26196    144.67064    155.32869    160.59748    152.27669    136.25145    131.37435    127.51304    129.72362    128.81831     127.3539    125.78066    124.72129    122.84836    123.29019    124.20643    120.08492    117.29912    118.26724    117.98859    125.88427    134.75882    138.70569    137.94783    129.12307     118.5167    116.00083          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -11.50    254.18907    243.06525    228.97744    229.32279    219.37257    215.68338    210.20603    209.48499    201.72162    192.92233    189.62295    187.57553    185.27974    179.76384     179.7272    173.35933    171.79988    171.25696    174.52616    174.03644    165.68952    163.39365    162.08964    167.93525     166.9799    193.63288    229.43254    239.31121    226.71084    197.17023    164.62526    154.62621     152.9347    150.97983    146.75486    146.32618    149.34628    151.46879    150.56178    150.37948    141.68517    140.11836    139.01841    140.72402    142.01787    140.62136    139.95555    133.68693    134.68136     135.2089    139.27057    134.75928    131.22909    131.40224    134.83176    137.47676    136.90428     143.6313    142.60149    145.15881    137.33177    133.14679    131.99491    132.45505    130.81103    129.39221    128.42922    121.64529    125.59498    132.69628     145.4069    154.34095    163.42102    153.67522    136.59196    134.32997    130.94059    133.73276    132.17933    129.81582    127.42682    126.72305      124.421    120.33853    119.34647    117.25835    115.35951    115.80115    120.67141    126.10253    137.41118    143.90357    142.38692    130.46513    123.20034    119.47158    111.95015    115.23349    115.16251    110.10639    114.91118          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -11.00    255.70304    242.44943     227.0888    225.83228    219.87213    215.37193    211.35974    207.35203    199.28983     192.4999    188.57311    189.43671    187.36643    181.45369     182.5556    172.72399    172.64506    172.90219    168.94554    173.80028    167.01453    163.16623    163.88274      164.534    166.80863    192.71765    232.04482    240.43601    225.03921    194.07868    164.05073    158.00702    154.58032    150.72932    146.00349    149.65521    150.38002    149.79284    142.28077    146.59723    140.13622    142.25148    144.96428    145.45185    137.60754    144.28397    135.05485    139.60352    133.35686    134.04868    139.61855    136.29627    134.69331    132.57472    129.84411    136.67852    137.85257    138.17877    142.21243    142.88912    134.62448    135.48375    131.61872    130.31984    129.05542     129.1554    124.41332    126.57697    132.29123    130.89026    142.99797    158.39776    164.63165    157.33766    141.21832    134.55007    135.04585    131.58797    129.87154    133.98509    129.80031     130.7605    125.82721    122.78274    117.97486    117.72267    115.07062    117.63506    122.72032    131.20152    139.12945    148.39947    144.11457    137.31986    125.33749    119.04975    115.79172    112.77654    114.37871    113.58537    116.27162    114.32502    115.23457    113.57924    113.66521    110.87688    105.47562          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -10.50    257.50682    242.36815    225.58307    222.36879    219.16476    214.63673    209.84818    205.80852    196.88726    194.61875    189.74166    189.34128    186.98941    183.21462     179.8737    175.00165    172.41848    172.63388    163.37226    174.32717    169.75492    161.88751    163.03722    161.80655    174.19438    192.19534      235.852    243.13747    224.62552    191.35482    165.21909    157.43687    157.17887    151.07614    149.84998    147.39696    148.62392    147.83126    148.56996    144.09455    140.26384    142.77292    144.96825    144.40073    141.35416    142.49681    142.59712    138.13817     138.1351    137.94643    139.28975    133.29189    133.65689    133.07722    133.62522    133.44408    138.31642    141.41442    143.39409    140.64202    136.30349    134.69243    132.88884    132.79762    128.86714    132.12454    128.19516    127.35154    128.94721    133.36384    143.89373    160.27834    163.05448    151.00477    144.77017    135.98827    130.54932    133.52653    136.41274    133.40403    130.18416    131.93167    125.66424    123.55375    124.07381    119.35233    117.81237    115.32777    121.14335    127.46602    140.48481    151.04837    151.51074    138.98445    131.10339     120.7077    117.74807    115.90163    115.06965    112.88601    117.12937    118.35989     118.0369      114.561    116.57602    111.44617    108.04973    111.90727    112.04235    114.82549    111.58772     110.4086    109.91735          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -10.00    258.91526     242.1914    227.44786    221.70193    218.19731    217.31128    208.24239    203.61247    195.63346    196.89288    192.61485     190.3907    187.05028    187.42011     178.3567    178.14667    173.45909    172.40109    168.13698    171.20505     170.6848    156.91649    160.47567    162.47835    173.34293    193.94583    237.14329    248.94123    224.41055    192.35161    164.98961    157.06568    151.12142    153.04173    150.20953    149.43755    144.82144    148.85813    150.11225    145.33041    140.24349    144.30259    144.07485    140.25509    136.62227    142.01594    136.07394    139.53751    136.87712    136.96233     140.6955    134.14922     134.5051    133.03325     131.6259    135.74154    139.74071    145.67325    144.80289     138.6431    133.58161    134.26584     135.8717    129.14818    133.26597    133.69703    128.63742    128.00035    130.35931    133.31336    143.44865    158.14167     165.9497    160.87599    145.94425    135.51274    133.79156    136.07715    137.48439    133.67832    134.16935    131.03058    126.31351    126.15466     121.3813    119.14277     121.7556    119.07425    120.51713    134.34231    144.21345    156.62152    153.39542    144.96574     130.3729    125.45268    118.33455    117.29167    113.37657    114.12721    117.26555    117.33526    117.84091    117.43984    115.96152     113.4382    113.27769    112.18405    114.11856    113.38477    112.51268    112.37566    111.13095    112.15993    109.87416    112.94944    107.65607     108.3818    109.86694    107.91125    109.83026          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -9.50    259.68107    242.03433    229.79974    224.30198    218.05198    220.14965    211.74285    202.62613    194.83931     194.2766      192.591    192.80897    186.34677    187.57542    179.24234    177.09399    174.31164    175.63617    173.72817    170.39597    166.43175    159.85064    159.60962    166.38091    173.96448     195.5453    233.25503    248.80761    227.78265     193.5096    167.09307    156.32498     156.8026    152.33314    148.55576    146.77923    148.32301    147.70804    144.93143    147.25476    142.54089    146.68722    146.09768    142.90678    144.46131    142.81538    138.22331    136.53138    136.00298    136.35128    136.44107    132.72605    138.86778    134.35511    133.39578    135.14741    135.30021    139.23389    142.52247    141.45794    136.39141    132.72206    136.56972    131.03052    126.41359    134.50549    120.96185    127.15862    127.32808    135.74082    145.40992    160.63818     166.4142    157.94552    145.92369    135.32862    133.35105    133.86331     135.2108    131.83304    130.91855    133.12138    132.08952     127.1524    125.70028    118.34412    119.40581    118.62969    123.45406    131.66762    146.20863    160.89773    157.12148    148.41162    136.59426    126.97204    121.29059    119.76616    115.56938    119.05107    120.47221    119.69849    119.94852    119.36593    114.50425    111.34302    113.43176    114.42423    113.63482    114.21071    112.65698    112.22973    109.95795    113.76186    113.54501    111.57036    109.22482    112.30352    112.27657    107.97395    109.42368    111.75955    111.14652    117.66686    118.83137    118.24758    120.81414     120.1551    115.96656          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -9.00    260.76468    242.12653    231.06677    226.93433    218.68775    221.08724    215.38242    205.16002    196.03946    191.07719    190.78613    192.58959    184.69785    180.72581    177.30798     175.2094    172.12216    174.58489    176.26085    173.56792    167.65845    164.48551    160.98823    168.50783     174.8829    195.30048    228.88332    246.96335    235.04386    195.96378    170.14005    153.61615    155.85945     150.2091    148.65467    142.81607    148.48181    148.40817    141.36178    144.15536    147.38584    140.74396    148.40192    141.13566    145.40617    143.83805    140.44475    142.68043    132.77883    137.06399    133.78484    133.08432    134.08955    137.17479     133.1045    137.44528    138.96192    142.80453    145.48023    144.95458     136.4394    135.80264    133.42938    129.72034    130.46495    127.48822    125.91972    126.10231    130.20151    132.70591    147.33073    158.65346    165.89388    157.56537    148.57898    140.45747    136.41608    134.51753    132.51698    127.74875    129.74256    131.66279    127.00991    127.32674    123.47979     118.3451    120.63933    124.26887    125.25427     136.2465    151.18197    164.06558    161.89568    153.30258    139.97664    128.11304    123.59259    118.52628    118.50576    118.59538    120.99323    123.55593    120.77487    122.01773    119.45043    114.26683    115.25285    115.46339     114.6716    113.35276    117.00727    114.89913     112.0653     113.4688    111.87875    111.57383    111.97977    112.51908    109.35192    116.16959    113.46473    111.36412     112.0959    115.77205    119.24415    118.17061    125.83012    122.91659    119.16077    118.23188     113.4064    111.87848      113.894     115.8078    111.44863    115.35068    119.85499    126.10835          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -8.50    261.10065     242.3896    230.80177    228.24223    218.83399    220.89205    216.02495    206.41102    197.90513    193.83852    188.10418    192.51474    186.30366    180.39734    174.30098    175.86322    171.08093    173.69691    172.34079    172.85418    169.23199    170.77594     163.7324    166.81623    178.33758    196.44756    230.38016    248.70564    228.35059    195.02232    170.41246    163.96585    151.58658    149.31373    150.05371    150.62728    152.09382    151.16771    143.04832    152.92498    141.95572    145.43994    150.83956    141.93998     142.7159    139.26394    140.73753    141.85678    140.65358    134.65212    136.21032    136.69794    137.94162    133.32779    134.77396     135.7439    139.72824    140.33122    142.57078    137.36085    134.19555     135.1193    131.90605    130.68189    130.67196    131.38891    127.88546    127.61384    130.75116    133.79784     147.5686    160.70775    167.09752    160.96797    151.40664    141.63246    136.13259     137.4538    136.38376    130.66373    132.44156    133.45182    129.10807    130.75873    123.45372    122.21861    120.03316    120.61445    124.65602    134.45932    150.56161    163.54175    163.29576     155.6538    137.68139    127.59887    122.60262    119.52615    112.21376    116.51021    118.20607    123.14133    125.02166     122.1277    120.27491    111.14576     118.6031     116.0038    115.30146    116.32661    113.83714    111.87787    112.30565    115.88258    111.29626    112.09057     115.0096     113.3634    109.84468    111.41306    108.75042    109.47499    115.24451    119.19766    120.36711    123.67585    126.29327    123.58265    120.68117    119.53413    113.69977    110.32386    112.84698    114.05888    116.09966    116.69493    118.98842    126.81058    134.83718    141.24166      140.403    136.84901    130.30466    121.75693    118.37603    115.39544    116.57969    114.95035          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -8.00    260.12611    240.32644    230.13955    228.75961    222.45529    220.07058    216.41524    202.29458    198.72295    196.04371     184.4988    192.05679    187.85798    185.53228    173.32072    176.08554    171.21674     176.8237    170.53451    171.81721    172.06944    167.80982    165.32246    163.75595    171.82721    198.37771    233.93076    252.04607    234.40791    196.85493    168.64064    163.54566     155.8553    151.68361    152.93501    151.56544     150.5627    154.76052    147.03354    146.65428    141.20375    142.20082    144.55764    141.09353    139.18358    142.01564    137.01709    138.63529    138.95138    138.38317    136.37173    140.68634    131.47811    129.97473    133.94417    133.51083    136.78629    141.46983    142.51856    141.87269    136.96525    132.53109    134.77913    130.39704    130.47077    131.65862    129.39654    129.85222    131.99683    134.97767     146.4543    163.45087    168.02185    164.91005     148.7675    141.94161    136.12892    139.34718    136.83735    133.98893    133.37562      136.305    133.65362    129.09742     125.4047    123.85374    119.62332    122.10043    126.30559    134.23625    151.70704    164.15017    165.59256    155.09273    140.89657    130.18025    124.97165    122.98601    117.42071    122.52823    123.59467    126.24232    122.18953    122.64475    117.66259    113.80635    114.75196    117.70835    114.23661    118.91017    117.57297    113.13711    115.95735    115.99514    113.70042    111.37387    111.60262    111.70407     112.6589    115.04711    110.53073    114.74674    113.20196    114.88239    118.02104    124.68821    123.24944    123.98449    125.69048    117.44481    115.60761    110.61582    112.87726    118.17332    115.33159    116.48329    121.11614    126.82951    138.65735    144.10684    143.84801    139.14844    133.85054    122.31794    122.63209    119.29113    119.54189    116.81847    116.78304    116.86268    115.14446    118.11502    110.59426    109.88908    110.94035    112.38194    111.43328    109.43294     111.1067    108.12989          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -7.50    258.62471    236.52705    228.53717    228.81945    226.54272    217.83943     215.3177    198.09758     198.4239    195.46439    184.11724    191.28677    189.28771    184.05305    174.17006    176.07572    176.87526    174.62035    170.61159    169.18391    165.12831    166.66272     168.2779    165.45291     170.5762    199.76719    233.07998    250.26846    234.93721     197.2294    168.10279    158.75471    157.41594    155.36961    147.87372    150.97932    148.82292    150.84386    149.33979    147.86517    145.14203    146.27267    139.71866    141.65127    143.42866    139.84223    138.75025    143.02747    135.81768    137.53893    135.31211    138.23723    136.98051    133.32822    130.06695    133.15646    137.84373     140.1929    144.16365     142.4593    137.64308    133.86353    130.83522    130.10036    130.66703    127.81434    121.58083    131.14066    130.51726     134.6383    147.96813    164.69996    171.14008    162.33411    148.64833     143.3715    135.68572    135.03684    137.54533    135.40735    137.80565    134.39816    133.65149    130.48989    126.25196    123.99096    118.43447    121.38244    126.78523    136.73989    149.22078     163.8656    164.50959    156.74534    142.62132    131.74117    127.74983    119.99181    119.41499    124.97798     120.8875    124.85374    122.11944    122.59854    118.13428    115.39457    115.73805    116.81816    118.66204    119.47644    117.84593    115.07795    116.58762    113.62036    114.58397    113.30806    113.03196     112.5076    113.99791    115.90876    115.42828    112.47826    113.87545    117.96742    118.28237    124.46552    125.69453    123.60576     126.7129    119.08931    115.89746    110.95728    113.52285    112.29655      115.494     119.3769    124.12234    129.41339     140.8025    145.90905     146.9124    140.80896    138.56398    124.11411    123.39777    127.08633    120.04495    118.36863    119.99598     118.0201    114.56385    117.38489    114.11823    116.48044     111.7154    108.47509    109.28445    112.03612    111.23419    115.31254    114.06723    114.81703    110.64018    111.73583    113.28403    119.89931    115.13378    119.00307    115.08824    111.72265    115.00714     114.2305    117.10388    115.49002          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -7.00     257.5902    232.05261     226.7733    228.05696     227.6555    215.72282    213.63141    201.69264    197.50118     196.3468    186.09565    194.22451    188.33802    181.61698    179.80104    176.82392    178.43499    169.53483    172.98279    172.19051    161.73723     166.7473    171.39615     165.2578     172.5349    197.77858    235.68707    246.90021    235.19527    196.49654    168.07828      161.881    159.28539    148.63262    149.50275    152.61849    150.12294    153.94706    151.98098    148.97437    143.56991    143.34774    139.10853    143.78261    141.49242    142.14104    137.37916     140.1398    139.00696    139.00593    135.09074    139.95599    137.84083    131.17078    134.92267    135.63722    137.56481    141.13415      144.181    138.08517    137.59739    137.10491    137.56487    133.28963    132.55578    128.42161    130.19495    131.49924    128.77073      135.072    148.91249    163.46395    166.58341    163.06347    153.26596    140.42731    135.32093    137.93206    134.68977    136.24194    140.05373    135.47096    134.80477    129.21451    126.32542     126.2938    119.46906    122.57381    124.09088    132.53413    154.53095    165.90558    167.91179    164.18103    147.48328    131.35696    127.84101    126.75879    120.33297    123.17783    123.40374    123.43245    122.70553    123.10633    120.92851     119.1163    117.16101    116.99262    117.56486    117.01529    114.89467    113.91581    114.05677    114.87084    115.11665    113.09462    113.68072    112.07713    112.19627    113.08474    113.36036    114.72141    117.70795     118.6445    120.12067    126.02824    130.11589    126.27397    122.30076    122.23311    116.77517    116.72066    115.35971    114.46127    115.89092    117.63275    124.52855    130.58394    141.59868    145.37141    153.70404    143.73978    136.19809    127.57658     127.4202    121.20373    122.42211     121.7252    121.16349    116.80547    117.31619    116.27787    117.14939    115.35695    109.36615    113.02403    109.22703    112.46216    114.10749    112.92985    108.93413    114.32977    111.75767    112.25642     116.4907    116.60094    115.34019     116.8688    118.86606    117.94231    115.70513    118.39537    116.01498    118.49048     118.0271    120.88659    119.21332    123.37183    125.86012    123.70424     121.9464    118.34752    118.45194    117.02403    114.56341    115.38402    111.36561     113.3878    113.77329    111.41569    112.98616          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -6.50    257.25607    233.94242    226.12229    226.94584     226.9124     216.0856     213.6257    206.40898    197.60056    198.01053    188.32571     196.5997    185.21345    178.87241     178.3678    176.91433     176.7062    168.94965    173.76769    172.78552    165.33211    164.80633    170.85158    165.06083    174.81115     196.7261    233.01866    244.60705    238.33951    200.19031    164.94254    163.23058    161.78238    152.39355    146.71194    153.43844    149.23875    154.11723    149.11408    148.61709    143.18698    141.43785    143.83941    141.36802    141.65872    140.72565    137.97824    137.09884    137.19687    139.93284    136.15046    140.84935     133.6418    132.61619    134.57498    137.78929    138.50617    138.95226    146.37709     137.1429    138.96433     138.2927    137.03914     134.6285    129.15229    132.28058     126.4516    126.73838    132.17526    137.05958    147.77637    162.39935    169.14766    166.46498    152.04566    142.60549    138.42501     139.3232    136.08507    133.90845     141.0699    140.37599    136.10862    130.10996    126.90821     122.6489    122.61019    117.25279    126.79521    135.41599    161.27106    172.54712    173.75731    160.49266    147.48427    134.59402    131.23593    124.85898    120.74488    124.73984    124.26158    128.15829    126.15499    122.91472    118.95297    118.91665    113.24484    118.43181    117.55787    121.08754    115.78109    115.97484    116.41133     115.3631    117.15884    112.79876     116.2313    114.57199     116.1608    112.60464    115.87463    112.53165    116.89779    114.60136    120.59464    127.13292    128.58455    127.99983    123.95597    122.26501    116.85703    115.10047    114.65455    114.68931     117.5294    120.84591    125.41298    134.55677    144.18864    150.70863    149.17654    144.69172    139.11768    129.42004    126.01336    125.30309    124.40738     119.6898    119.08238     116.4941    114.18034    115.93309    111.26841    112.69924    111.40817    114.52323    111.46895    114.13864    113.80525    112.49647    114.30626    116.47521    113.15599    115.00564    119.06104    119.85452    118.69433    117.69912    115.98921    117.28087    117.29378     118.8025    116.72344    118.41494    118.26001    117.45487    121.71509    127.55278    127.86979    122.66676    122.99668    119.91138     118.1598    117.64415    116.01744    117.09115    114.87077    113.71464     112.2809    113.01986     116.4945    115.62517    112.75167    114.20097    115.09902    116.19547    119.67286    120.55162    122.03013    117.80873    118.13371    114.48256    117.20654    115.47692    116.51693    116.47852    114.71797    116.05527    117.68712    118.50805    118.91084          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -6.00    257.05253     236.7499    228.61294     226.5072    226.09584    216.49868    215.47805    205.48722    198.43815    199.19769    190.81616    192.38663    185.42768    179.58621    172.48893    175.67206    177.81889    168.31542    175.41545    166.81091    169.69024     164.8594    172.83248    164.91829    172.63744    194.48739    231.95457    246.05493    240.43083    203.60222     165.1566    162.29639    155.11529    152.29624    151.94571     152.7107    150.68196    149.21362    146.79413    144.21563    144.65954     144.2605    147.37154    138.51941    145.21723    142.17153    136.16158    140.22878    132.51309    137.72869    135.93484    138.31866    136.18727    134.71675    132.63112    135.48221    136.61392    143.51166    143.75718    139.33632    139.20266    136.67765    133.82864    132.31163    129.76711    127.13153      126.245     127.3179    130.26653    135.51702    147.63438    164.63955      171.736    169.69767    152.75283     143.0114    140.93892    140.96974    137.12481    136.07294    137.19174    138.61821    137.29169    133.03463    125.87154    122.48152    120.60004     120.2037    125.78305    137.47055    158.87897     168.9106    173.24884    164.14057    151.50857    140.25822    131.00603    129.69799    127.92948    125.25048    125.27221    126.67742    127.47182    122.68881    121.76316    115.86576    114.63417     115.5175    113.86098    117.94737    114.51946    116.38979    118.14662     112.6562    117.91188    112.75335    119.25163    113.69311    116.26545    117.73202     114.2719     115.5755    115.46059    119.46407    121.82963    128.56331    130.28144    128.63334     125.3622    123.22088    116.83136    115.64644    115.03751    116.73745    118.39406     118.4537    123.02854    136.24104    145.75971    152.14123    153.57534    147.77697    137.43557    129.73334    123.32423     123.6918    124.46715    121.25339    121.54114    120.46145    116.92822    116.20257     117.1471    114.50336    111.12661    113.48391    109.37897    112.81726    115.29202    116.09302    113.05485     113.0762    113.74222    119.99106    118.24247    117.37526    118.30856    118.55118    119.71816     114.6689    115.54767    118.69156    120.26376    119.17157    118.51603    123.39028     123.4231    127.77263    127.86897    128.89484    123.75727    122.12537    115.84101    116.70354    118.11014    114.13364    117.76301    115.89739    113.29797    116.25952    116.44409    114.64061    116.57796    119.43158    114.25174     116.2026    117.53487     119.9158    120.68198    124.68858    120.59649    117.55239    120.33184    118.13372    120.50826    118.82933    118.29188    117.23528    121.30791    118.03668    119.55184    121.00897    119.55607    116.16356    116.88016    117.04579    114.83967    117.33262    118.31817     117.0009    116.66288    118.30844    117.75747    119.91763    118.61981    121.78419    120.04996    118.66609    121.68317    122.45453    122.56752    125.69661    124.37428     124.9186    127.18191    126.22831    128.81271          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -5.50    258.25715    238.31219    230.97697    226.42598    223.68631    218.16604    216.74076    204.48717    198.64068    195.78582    193.22682    188.14443    186.92088    186.95091    177.85878    175.94002    177.45128    168.46699    179.16165    172.31742    169.59666    166.10209    177.06379    170.91195    167.77228     193.0019    231.04666    252.10603    241.80012     199.8879    171.04026    162.64682    155.15721    151.50209    151.95662    150.26697     148.6045    147.69471    144.14031    148.02287    145.67876    144.44712    139.49533    146.53611    141.22837    137.88573    134.36612    137.68742    139.22544    142.27615    137.83686    138.99593    135.87516     136.0184     133.8571    139.60957    137.98163    143.96391    145.70491    141.95791    135.27715    133.44858    137.06757    133.64412    127.73183    128.63647    129.88387    127.61245    130.15495    137.79089    149.70819    164.53126    169.88492    167.35652    153.01274    141.77717    139.54002    138.03413      136.906     135.5361    135.41778    140.94898    137.53892     135.8699    126.45213    118.22142    122.33298    124.80572    126.76833    141.36457    160.97484    176.11832    182.85765    172.52501    160.69827    145.07687     137.8861    131.50577     128.0866    128.76962    128.53227     127.3443    124.51971    124.54128    121.39395    117.63921    116.13446    115.93474    115.43924    120.09523    119.65551    115.15709    115.49039    115.56977    115.56886    114.96701    111.76825    113.32832    114.48657     112.2385    112.86845    114.80427    118.61116    118.67524    121.10233    126.93656    132.98001    131.36691    125.26432    123.65268    115.57828    116.19352    115.92598    109.74654    117.25238    120.85246    123.35285    136.08348    149.93746    154.33297    156.98638      148.422    143.02209    128.28525    129.44051    128.04409    129.96614    124.95666    123.17634     120.4454    116.67897    114.64333    114.17712    111.80629    111.61181    112.67521    111.41764    114.24022    112.59474    113.27884     118.7401    115.40469    115.14646    118.20589    114.24371    121.11849    120.84683    121.28071    120.60649    121.49235    120.64388    118.79737    118.81507    118.99788    122.71841    122.44349    125.39537    134.66923    130.58229    130.45541    123.96195     123.7502    118.31446    118.90243    117.60509     115.6858    117.11943    116.07601    113.02136     113.6032    116.18954    117.49803    114.91521    116.15695    116.36025    120.97627    122.91938    118.76272    120.53507    122.33123    120.96781    120.17517    125.46336    119.79165    123.23717     118.1957    115.70215    118.94209    119.99534    125.15667    121.13583       122.98    119.02714    121.40238    115.86449    120.31829    120.75388    116.37838    120.70918    118.22405    118.95018    118.79214    121.12184    122.66105    120.77304    121.04743    124.39603    127.56262    129.12413    125.66927    127.72364     127.5199    124.68757    124.04902    129.45305    132.21401    128.67846    138.14307     130.8127    133.43816    131.95528    131.35476    129.05334    129.22332    125.56802    127.83359    127.73053    126.02458    123.24879    127.91644    126.35731    122.20524    127.60898    123.52982    125.88154    125.36593    128.40065    127.30527    128.59936    127.16121    133.72298    131.21335    132.76243    133.25731    138.22899    132.30015    134.64379    133.86502    133.66167     130.1072    126.18587    129.54118          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -5.00    259.81096    236.79416    232.83343    226.21454    221.43637    220.36334    216.50339    203.69904    201.32405    192.59617     196.0575    186.89867    193.34805    187.83625    181.84621     177.2733    174.46155    172.94223     181.8516     173.6283    168.49434    168.20755    168.28811    166.59487    169.44299    193.41873    230.26098    251.63999    238.10867    197.66503     172.3223     163.5218    155.12531    152.68834    152.53066    148.38225      149.245    150.67665    145.11287    143.97333    142.55295    146.71528    140.03388    142.16495    142.01545    143.67758    138.14385    138.41974    137.16217    138.44354    136.82484    141.84985     130.5694    136.10875    134.96966    136.78131    137.93045    141.40717    147.65323    140.89738    140.08739    136.22753    127.62535    134.31744     131.5614    130.32889    126.76709    128.23779    134.84612    135.17701    149.17857     166.7164    174.33017    163.94674    152.55249    145.45162    138.46039    138.82684     133.5271      134.527    133.34075    142.41077    135.48082    132.72376    126.53938    127.11723    118.29889    123.69764    128.70455    137.16592      158.964    176.80399    188.10477    184.16367    173.51914    158.55769    146.67858    140.55348    137.32098    135.68282    131.93178    129.82312    129.11077     124.0653    119.80599    121.29663    117.24044    118.22007    115.52882     117.1656    118.95706    116.27548    119.37924    115.88327    114.64212    113.13483    115.17315    113.61169    116.23307    115.25529    114.45351    114.56785    113.37923    116.33147    120.63146      128.648     129.7231    129.92147    127.75817     122.8994     115.9949    115.79134    116.85493    114.06772     118.4845    119.27066    123.44928    135.20071    152.60109    159.14618    159.61279    156.01253     142.1705    138.36301    131.59336    133.75116    133.56319    126.40451    121.90472    120.95188    120.61346    116.90172    113.96209    113.66181     113.4276     113.9838    113.23448    115.72763    114.33533    116.21698    115.77748    114.66021    114.56059    118.36711    118.90569    121.13163    121.03402    123.15589    119.10579    118.99602    118.73945    117.71544    121.59271    121.42921    123.17817    122.76311    126.29926    127.70361    127.27906    127.73436      125.496    120.91039    121.04538    123.61038    117.26099    121.65484     116.6566    116.09594    115.02208    117.36016    116.13838     115.0544    117.04633    118.46279    116.08047    119.18108    122.71575    125.00859    125.07404    124.42185    123.86199    122.47734    122.54593    121.56208    120.42582    120.45414    120.96935    119.63354    123.02694    122.09649    123.86004    124.49683    124.56372    122.09343    117.02614    119.56935    118.29984    115.56816    116.66899    117.16087    118.64862    118.06319    121.03864    119.98877    121.02786     122.6481    125.05464    122.43707    128.70023    126.75613    122.86449    128.37718    128.77004    126.56002    127.47892    132.81526    130.07529    134.92408    131.18952    132.23886    130.45904    131.75518    130.64654    130.79673    127.83539    130.40025    123.16154    127.70402    125.05563    126.20927    127.49681    128.14683    125.82641    129.23572     124.8031    126.64075    131.47878    126.02037    129.95891    129.79223    131.08223    136.18217    136.13829    135.95231     136.3843     137.5037    135.88424    136.93311    133.75533    135.85039    133.48654    130.91362    129.93149    131.90309     130.7888    124.60772    132.05096     132.3056    126.24206    130.98256     131.2779    131.69368    130.19542      129.788    127.44966     127.9125     130.1788    128.67377    129.22469    127.42699    130.65156    126.82286    127.64164    128.11514    129.64609    126.92982    129.63384    129.25863    127.87818    126.93284    127.94594    130.73456    126.19766    125.22705    127.08665    126.70368    127.47777    127.79794    128.13859     128.2842    128.39926    128.93469    128.05282    132.91816    129.88379    132.37987    129.50103    127.92306    130.28619    132.63773    132.84112    132.70696    130.28901    128.75409    132.18076     133.3804    131.96677    130.39628          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -4.50    261.33076    235.44893    234.64281    225.51322     221.0393    221.98697     216.1693    203.15016    207.05345    193.87624    196.84337      188.087    197.90949    184.49113    182.64699    177.85052    176.47743     174.7884    182.09569    168.63077    171.62044    169.25555    163.05465    163.68926    175.02457    195.86912    236.80833    246.69759    241.50302    200.38183    166.24227    163.33317    153.97717    150.09656    152.47631     149.0967    153.43104     151.3913    149.32894    141.62705    142.88501    142.47995    140.73163    141.70784    142.12339    145.04027    139.09358    141.48394    137.39092    140.33595    138.06944    138.76279    138.69021    136.51029    134.82083    137.25482     132.3573    142.13931    144.02132    139.90602    139.05082    136.07963    131.89971     132.9008    136.89281    127.20605    126.31251    128.73342    131.58135    135.51702    149.21868    167.33676     171.8855    171.28882    156.25473     141.9249    140.03715    137.59189     134.1735    131.35351    137.41767    139.75105    140.47706    134.76927    127.12157    123.61177    123.80622    122.21318    127.98115    139.07706    158.09307    179.15943    188.84485    191.50731    179.77508    165.99962    152.65863    142.56779    145.30536    143.31771    139.92573     135.1769    130.23006    126.68465    116.25758    118.41883    114.44551    118.22422    119.27304    119.95239    118.19964    117.50234    112.46145    115.37074    115.42668    113.71251    116.40588    114.18878    115.38846    112.87813    113.54073    115.20559    114.52192    116.44966    124.53535    128.95651    132.82561     132.0169    126.98189    122.32103    118.20411    117.06973    115.55387     116.0008    120.05811    116.58667    126.02115    139.97204    151.89283    158.87866    160.28155     153.4484    143.12122    134.83455    133.28713    131.28616    129.32156    126.24682    123.48828    118.33103    120.92904    117.94925    118.27922    115.48282    116.35137    114.83872    113.22181    113.47634    115.73597    116.50565    114.17126      114.166    112.82666     119.1507    120.41495    118.62277    121.43123    121.02897     119.0913    115.98446    120.32948    118.54612     122.3702    123.06992    125.48237    126.74495    127.00723    130.61675    128.70741    128.27074    126.45181    121.65289    119.97782    120.45143    119.02986    118.59643    117.82573    120.60886    115.95572    120.25395     117.4312    119.58677    117.81488    114.61728    118.31396    122.36768    121.24169    124.76341    120.67534    127.38672    126.02856    122.52321    124.37766     123.1567    120.73694    123.11177    121.55045     123.4962    126.68759    126.98654    124.57297    122.51357    126.26036    122.37268    117.85503    119.28828    121.21834     120.9855    121.34866    118.69173    118.11289    121.24932    121.47388    121.68373    121.54957    122.39941    123.19651     122.5505    126.77224    129.90916    128.05443    129.27265    129.95254    127.29428    130.17147    133.39611    136.92049    134.07508    134.84864    133.36915    135.20229    134.61793    132.84535    131.53007    130.81881    129.25959    123.72921    125.75954    127.06643    126.31402     126.7979    130.20033    128.00705    127.25426    131.66998    130.86106    132.13287    130.88164    130.60136    129.43364    132.70787    136.34744     136.3075    139.87914    138.27597    138.57403    138.49376    140.25852    138.08542    141.30683    141.11431     146.6978    141.95506    137.36633    133.74024    131.21567    129.50731      130.629     127.4673    132.74681    125.72447    130.02068    130.30103    131.58716    130.07313    131.73646    132.94219      130.682    127.96168    126.07262    130.32136    127.10333    128.73037    127.04999    129.18695    129.14783    127.63973    128.15369    126.70689    128.51855    126.63476    127.48835    129.95831     130.1089    125.13473     132.7554    129.19122    133.22108    130.10116    130.22008    127.70723     128.3733    129.71514    130.78935    132.27175    131.17157    131.21019    130.94905     133.8106    131.01446    134.57895    128.58827    135.48619    131.58005    135.68664    135.00049    133.43771    138.35735     133.4487    134.02075    138.40707    138.05559    141.60301    140.91539    140.30426    142.89311    140.31716    146.72973    143.79704    148.76682    144.71501    144.33787    143.24171    147.75716    148.91581    157.73756     150.0387     147.1774    149.59684    151.15708    151.55849    156.54009    154.18724    156.79122    152.83888    157.89888    159.47955    162.50187    163.04392    165.48018    168.23861    158.56574    159.73167    166.09648    172.36395    162.93092    167.99746    160.67904    160.89536    166.50542    164.05481     166.2434    165.79416    162.81717    168.21741    175.16042    173.15434    168.78938     173.4634     176.0589     172.8406    177.82414    174.82014    185.41018    186.38826    178.51791    174.87635    169.03912    182.17481    189.38773    182.58891    178.01176    177.19886    185.75438    182.20047    187.98698    195.75361    183.92435    185.94605    188.66529    189.11203    192.85568     200.6322    194.26955    207.03012    191.35029    206.28936    195.52997     203.5723    204.76391    208.08609    211.86945    207.75829    203.88306      210.979    219.68483    213.34541    225.30836    207.95635    214.46132    223.58481    232.77447    227.36509    221.24169    231.03306    231.94813    230.32847     239.5916    234.93959    243.78894     245.2955    240.24285    226.75596    246.70857    250.57986    247.70256    246.68831    244.88026    251.91889    249.49728    269.13027    268.83497    264.79094     248.7314    260.03124    268.84036    277.80122    286.13845    266.37739     282.0277    292.72456    283.20978    292.62328     280.7707    296.52874     305.2915    297.21943    331.57588    311.18893    338.66144    336.97193    328.28703    337.56056    334.59876    331.89606    355.37069    379.29578    370.78136    381.98076    384.09947    394.28717    383.23196     430.4004     426.1049    480.09058    445.18948    426.13911    442.57805     492.8044    583.08991     556.1249    537.18426    548.55861    576.84167    673.23177    770.23122    896.93648    826.26717    782.42367    777.52232    841.11197          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -4.00    260.19052    237.41516    236.16865    225.00433    220.98665    222.85602    214.96717    200.40788    208.97549    195.33769    196.96122    191.59987    192.40838    180.98416     183.0071    179.19661    179.09656    173.94577     179.2491    171.00936    167.00421    172.15112    163.44714    165.92766    174.25057    195.11608    231.96158    245.38078     244.5003    201.77183    168.51556    156.05002    157.67648    150.18866     151.8303    147.96494    149.47026    147.16504    147.14771    143.84433     145.6354     141.5014    143.41848    145.21061    139.99675    145.09129     138.4579    142.06704    135.66207    137.52487    138.86814    135.65382    131.99372    137.43839    139.25711    141.31546    140.74927    142.07705    144.07529     138.4181    139.96687    134.19964     135.0406    133.41059     130.9176    128.78554    131.40729    125.40149    131.76742    139.16052    148.12426    166.72301    173.54205    169.37461    154.96143    139.30877    139.47584     137.4725    133.97395    131.21339    137.76531    144.24747    137.18641    133.77909    125.44293    123.41378    122.75633     123.8654    128.40438    137.91777    159.23156    180.57304    191.62218    192.17225    179.58698    163.08623    152.86155    145.81574    149.02338    151.13761    145.58367    138.35036    131.10811    127.25671    120.94712    121.24322    117.35356    114.13243     117.2093    119.36168    116.14973    116.50145    115.62959    112.88455     114.8549    116.24709     113.1659    110.75213    118.62552    116.39637     114.6534    114.76783    115.87055    118.39183     121.9033    130.48735    132.64931    133.81144    130.19592    123.51088    121.36747     116.4878    114.38468    115.83248    116.02288     122.3553    129.06515    139.51289    152.34356    161.24329    161.42155    154.84606    141.69021    133.69303    129.28613    129.67716    127.14725    127.38361    124.91487     122.0454    120.30101    119.78562    115.73992    117.03705    114.66485    110.97286      112.797     115.0849    114.19569    116.04093    117.57989    116.05595    112.63446    118.28398     120.1893    122.14237    120.57764    122.69031    121.18832    120.74618    117.61136    119.17335    122.90431    123.42668    125.36218    125.53121    128.61203    129.07202    133.30558    130.44333    127.60667    122.44947    121.58924    125.51727    121.93911    117.05945    118.64495    118.75699    115.76307    117.64346    117.84807    116.55421    118.77469    120.65644     117.8961    121.37169    122.90254      125.912    125.71729    128.38863    127.35667    126.50082    123.35548    123.36711    125.39718    121.35914    123.40788    122.81796    127.85968    128.78899    129.11021    127.96677    127.06685    124.73398    125.15306    122.75546    122.72833    119.68288    118.97178    120.64156    116.35738    124.02759    121.36249    122.52364    123.67743    121.99212    125.64593    126.49279     127.5674    130.27434    129.32063    129.78948    129.05266    125.09403    132.38987    133.75467    138.58461    136.52382    136.52022    135.05959    134.26244    136.41549    134.29036    134.01572    132.98345    128.10378    127.08199    128.27493    131.64899    129.57424     129.8872    129.15722     127.9714    131.01699    134.34461    128.45827    132.92935     135.8021    133.32048    134.19907    132.93509    136.49029    136.07564    136.07448     139.9782    138.03685    141.68032    139.72999    135.78885    135.69893    140.32387     139.9996    139.70551     137.4133    131.21962    132.33856    128.39566    131.25339    126.68804    133.94795      133.234    129.88927    134.29856    129.75164    130.98363    133.41053    131.39356    131.92261    127.33493    130.88296    131.06817    127.23497    129.37884    127.63417    127.89731    128.17272    127.57058    125.22358    127.87473     127.3753    126.02534    129.56635    129.18114    130.98211    129.09008    131.49759     126.6616    130.40899    132.60963    129.83793    129.95941    132.87879    130.84476    131.36737    134.95982    134.12494    129.20646    131.46937    137.86055    136.39467    135.35026    134.10655    136.90708    133.19707    131.22808    135.33317    135.31292    135.90039    135.02252    133.87345    134.04184    140.38326    140.47074    141.10134    142.38518    140.29808    135.99775    139.62892    144.24092    146.87602    149.03069    146.18911    146.63923    144.57245    151.28115    154.10331    154.27003    144.76513    147.50653      154.984    157.52063    156.09395    154.70303    157.18871    156.43757    158.62276    162.28617     163.0065    156.91715    164.27593    166.26526      161.811    164.10548    169.39041    162.91693    170.07892    163.19262     170.4347     163.6546     171.6663    172.50367    164.91659    173.31774    173.50168     166.3679    174.95261     169.3059    175.28113    174.63577    176.99089    181.94021    177.26352    178.32948    179.98548    183.93545    184.23536    174.46422    182.07939    188.91224    187.97492    181.24103     186.8724    182.26263    183.24292     189.1336     187.9701    192.45487    198.13239    192.64522    192.68957    188.51425    198.33233    207.56054     203.7754    205.07791    202.06007     204.1156    198.01443    201.96532    199.93391    215.55366    208.29718    211.79832    204.58198    212.07289    215.00674    217.20025    208.52182    219.97667    226.04583     220.2411    230.02095    233.90185    226.97992     229.9316    230.97827    226.62541    236.85721    247.73055    234.49288    240.39835    240.58635    236.87871    247.77766    249.17571    249.44497    241.13643    249.08177    247.01335    247.38838    265.73482    256.06557    272.90416    258.40006    253.00357    265.10456    281.58256    267.29963    264.15709     271.7086    272.66051    281.08601    281.76896    296.14311    299.74803    282.52938    294.97515    305.55843    287.56661    339.11983    329.86599    346.78508    323.00729    319.69424    333.80281    360.31689    374.24911     376.8229    357.33006    367.51512    342.30813    383.69297    427.36409    408.67357    458.00912    436.53437    430.77407    436.90457    499.90497     556.6507    555.04355     514.9277    501.74875    561.96988    627.61425    764.02268     779.4736    661.74288    775.22912    796.65462    793.07861          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -3.50    258.91051    242.44721    234.83349    225.46916    221.71164    223.40183    213.65583    196.72195     206.1394    198.05525    194.39764    193.72028    188.77964    178.53277    182.84192    181.86024    176.94566    176.49248    176.79301    171.23054    164.28784     174.9125     169.6732    170.14738     173.5232    194.86912      230.622    249.18562    242.95732    196.17297    167.96982    156.11538    161.51933    149.24123    151.37986    148.07145    149.20058    148.84747    152.64342    141.91303    140.51576     144.2177      143.082     143.9752    141.23672    141.56758    136.42416    138.26515    140.75663    138.31066    140.47344    139.52481    132.90229    129.88093    133.57352    134.40904    136.42862    140.75431    140.11155    143.46016     136.5092    136.87328    136.21802    132.49821     130.0872    128.49227    130.78675     131.7274    132.82878    136.20131    150.47079    169.38212    176.82791    168.26833    154.28307     144.6585    137.86904    136.02868    131.73215    132.72349    135.40117    141.84058    137.64509    134.53694    128.20971    123.87424    124.76695    121.30867     130.1073    141.97161     160.8706     178.8881     189.6406    187.35871    176.56175    165.31146    158.79235    150.52521    150.96873    150.79663    143.11166    137.39535    132.61135    124.47912    119.96324    118.53538    116.72197    117.93047    120.25842    121.56975    118.32324    121.42489    117.58722    115.57554    116.32405    115.93265    113.26191    113.62795     117.2547    115.73323    113.45228    118.68601    117.69432    119.05727     124.7963    134.57267     134.1006    132.69382    130.77035    126.71571    118.92364    118.41891    117.85734    114.87545    118.94429    123.33935    124.42247    140.82476    150.82005    162.08895    161.87571    156.98944    144.34421    135.03108    129.70176    130.15183     124.6239    126.86296    125.74749    124.87915    122.01786    119.41153    116.38822    115.08236    116.35565    114.15524    115.25718    117.29498    113.07606    113.60252    112.26117     117.1513    117.66081    117.67253    120.45644    123.00071    124.43954    121.67277    119.98597    119.62221    121.45393    122.38389    120.12623    125.46358    126.50219    122.65419    133.38098    128.98349    131.38885     130.8027    128.04871    125.99586    124.59363    122.80838    119.21423     119.6352    118.75985    115.41746    117.70147    119.44773    116.21715    121.73344    117.75818    118.00482    120.04041    122.34119    125.28463    124.38096     125.4467    128.36135    128.60273    127.56138    126.63946    122.51517     124.9897    121.65973    121.46431    123.30952    127.51167    132.58718    128.91857    128.24467    126.72212    123.82422    125.57304    124.69958    123.26508    121.84627    118.51679    119.51434     118.4751    120.48521    123.87888    124.03766    123.25841    124.95377    123.81873    127.94754    129.52309     129.3827    127.77191    130.34185    129.19988    128.57223    131.12439    137.06692    138.14557    141.60605    139.10295    139.12692    141.87729    142.04325    140.78402    138.58014    134.62918    130.35648    126.13414    129.16266    129.50257    130.76392    128.54581    130.78175    130.63669    135.33483    132.76873    133.89579    135.57281    134.98703    132.07166     132.4028    136.27128    137.46089    139.92326    140.64028    144.75332    140.57206    141.20813    143.13608    141.22513    140.27257    137.21887    133.75068    131.23628    129.23801    130.60448     128.2151    130.87612     128.5825    129.48396    129.45075    131.34204    136.03786    130.36762    132.46747    129.11274    132.59024    135.05817    130.84381    132.59038    130.16352    129.10772    123.92569    131.50931     128.8087    127.12367    127.72728    130.40058     127.0881    128.32023    128.49329    131.80464    128.22693    128.00705    130.92475    127.00588    131.03791    128.52871    131.35344    132.03877    130.08443    126.20886    127.83502    131.39396    133.69997    134.56058    135.90593     134.1069    128.84444    134.16351    133.10659    133.05114    133.46823    135.95597    133.75114    136.13353     138.1337    138.77522    135.81539    134.35658    135.46386    135.39524    140.59508    139.49244    143.61514     146.2559    142.50395    142.15892    147.24028    147.40551    146.35918    144.99254    143.28366     146.6027    154.21661    151.31721    153.18425    149.74922    147.87617    150.55255    150.40489    151.00301    155.38885    160.26386    159.63953     162.9891    156.54311    159.71163    161.42924     159.9364    164.56777    161.26384    160.44045    164.40963    164.31532    166.56621    164.58668    167.95245    169.05415    167.39684    169.42394    166.83068    170.40808    172.41295    173.73514    170.33857    173.56017    172.70003    177.04244    182.10405    174.70529    179.57465    179.07175    182.07708    183.84291    183.69132     182.5362     184.3929     180.9821    186.62306    182.40956    190.82595    184.73943    182.89752    181.95521    186.38516    187.30554    192.96168    197.57249     195.2988    196.60765    194.00239    201.53307    196.35977    202.70819    201.71195    204.03958    196.79376    205.19289    205.07066    215.55947    202.96756    196.09577    216.97297    212.32104    211.10971    206.51432    208.31488     223.4344    221.92514     220.1356    208.76638    232.83998    226.06776    223.85027    237.31823    246.43487     240.2533    229.82823    232.78747     244.1544    233.30293    228.37749    220.83511    250.41695    248.08751    243.87182    240.01644    238.99759     245.7533    243.21344    254.08587    254.38406    262.95047    257.36842    255.06646    257.54894    281.39594     273.5375    277.09644    260.61144    275.25523    267.27746    293.05279    298.81497    283.18796    280.45568    286.20455    289.82001    301.91064    334.00959     313.9878    343.59093    332.37676    327.90336    330.83614    329.77948     355.6788    381.39705    380.17492    372.15074     382.4607    375.46784    443.20301    443.68566    438.00678    453.46056    419.58472     426.1156    490.04555    527.22213    599.79538    547.68639    532.96339    515.55322    584.57862     792.2995    839.08418    741.45522    802.69835    870.59546    761.70753          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -3.00     257.8204    247.33133    233.66384    226.16079    223.51242    221.93634    217.47449     198.1891    204.03142    200.16783    191.43034    194.97428    188.45296    179.28081    181.45987    180.47887    175.95006    177.56999    173.54018     170.6892    168.52193    167.66629    170.14046    171.10613    171.66584    194.80192    230.94821    254.53763    241.00081    200.35936    170.74203    154.01334    158.93351    150.33781    152.90365    149.39499    149.68387    157.32782    146.55839    150.10766    141.14686     143.8289    145.43747    143.58507    145.52628    144.27316    134.69352    142.05817     139.9324    138.44994    138.57716    137.73231    133.99499    133.63891    131.07911    137.67412    134.94408    138.79755    145.36831    142.93542    137.66838    135.30452    134.73221    134.93957    131.22011    127.04205    126.47079    130.07414    135.52993    137.66358    154.75121    164.26146    175.13443    174.71036    157.23644    141.32543    138.14156    140.58024    133.29658    133.12102    135.13003    138.51159    136.82802    130.24822    127.97948    125.24894     120.2573    126.40894    126.80688    138.78906    157.97697    175.21052      189.798    188.87631    178.72919    173.03104    165.08428    154.10067    151.72392    146.27417    141.79413    138.04614    134.32653    127.16079    123.52452     117.4016    120.42533    117.92001    121.30893    122.24732    119.73918    119.55638    118.40694    116.38476    116.77681     119.6134    117.33799    111.77594    116.13988    114.78434    115.06436     116.8659    116.23205    120.34055    129.62761    136.30669    136.12148    130.45017    130.70159    125.35544    117.95886    116.86583    115.24769    115.58505    120.72796    121.63947     126.8401    139.64912    153.59537    162.02197    162.82998    159.50875    147.41735    135.45866    130.82272    129.37497    128.38114    127.58144    128.86701    123.65321     120.7247    120.61216    118.67275     115.8184    111.39676    112.83147    116.10948    113.56535    112.20419    114.41793    116.59678    113.58916    113.34029    118.49355    117.67529     123.5862    120.98049    124.37879    120.27171    120.91446    122.46201    121.45362    122.72912    126.99706    126.35421    126.30792    128.39162    132.68726    129.80842     129.3351    131.18121    125.70631    125.00783    124.97808    123.49142    121.83346     116.4733    115.13972    115.10643    118.81768     117.9723    119.63347    117.15093    121.38782    120.61007    122.60181    124.37113    126.99881    127.42829     130.1023    127.09719    125.13641    130.37316     126.9365    124.80617     119.3708    126.10755    123.84782     124.4157    126.43486    127.10696     129.3665     127.3561    122.17242    121.66158    119.38645    121.41674     124.1179    119.66193    120.74005    119.81034    121.04223    122.10061    122.37657    126.09226    123.34267    126.66078    128.72124    130.21855     129.6701    128.57946    127.35247    127.14136    132.59685    136.26295    138.26565    141.32118    138.72498    143.89363    143.86643    147.06855    147.61423    142.42087    142.64739    132.51371    135.82026    128.48463    133.68104    131.12157    135.22055    134.66137    136.70612    136.42855    138.65418    136.06978    137.55443    140.62406    136.54469    135.07787     135.8949    135.21611    133.68284     139.4312    140.51933    139.43036    136.89721    140.66593    145.35112    139.90757    137.93994    137.61007    133.96285    132.54384     132.3582    128.82858    130.01007     128.6668    127.91713    129.41523    134.48886    128.86515    137.19516      132.737    129.22871    134.34558    133.22432    134.54479    132.71051    134.67737    127.42411    131.55077    131.16059    132.27626    130.92714    128.26009    130.65157    126.75685    128.03701    127.37494    132.17644    130.75744    125.72054    133.44098    130.31846    124.29418     127.7394    127.51747    128.94865    128.35299    129.27415    131.59279    128.15799    133.97468    130.75671    131.95451    133.46267    132.61476    132.23521    136.22112    133.57203    134.45483    130.79881     135.1551    135.70893    135.87127    137.39327    129.53203    133.78266    137.92323    137.50207    137.18248    139.06329    140.80926    141.84295    141.79618    146.44095    144.06875    149.31925    146.32967    145.45036    146.49488    145.09594    147.47029    150.48886    154.93707    155.95127    150.38012    146.43846    144.47786    151.71001    154.53282     153.7917    149.76386    154.94737    157.90243    160.68208    161.63107    166.27607    164.29507    162.12376    162.45809    157.79405    165.93929    166.53787    170.40817     167.7285    166.00839    165.53854    166.59896    173.93175    167.92656    169.80229     171.6271    169.31874    174.10304    175.05659    179.08583    177.99177    177.72243    179.26841    177.71571    175.86525    185.04539     178.4978    178.36349    186.03589     184.0705    179.11571    194.57583    188.53832    181.70876    190.22626    183.23033    186.68121    185.62881    188.15652    197.37537    201.12977     190.3662    194.49518    195.08974     201.4634    208.62362    199.98086    192.39885    194.97265    199.91687    202.76423    206.77994    202.51584    207.46341      198.824    208.93262    210.47244    213.11943    214.60053     213.9577    219.34141    213.76427    220.93086    223.44858    233.88025    229.77361    233.31595    230.90451    225.81279     229.6528    242.49922    236.97165    230.60879    240.99724    237.73511    234.18744    249.70411    238.16097    252.01862    245.06148     248.8723     235.8119    234.09787    258.17019    267.98822    265.99869    242.70768    258.97472    254.95844    268.39114    275.52153    244.85289    260.55726    264.02302    264.59465     273.9784    294.18143    289.80483    279.55599     302.1219    299.12249     281.8126    302.78193     322.3859    324.81954     328.5243    312.79934    340.66014    335.70302    366.04643    348.43674    368.38182    376.51137    372.44554      357.108    458.10909    478.29147    394.58648    447.39396     432.8937    434.56071    472.63717    551.72899     484.9583    471.81085    521.43556    500.09549    608.98899    798.64434    800.40405    750.74037    714.00776    775.19507    744.05061          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -2.50    258.27143    248.81951    231.64264    226.12325    225.06174    220.67872    222.01009     202.9355    202.57231    199.29605    189.07063    192.11864    189.84163    179.48992     179.2271    177.95449    175.90393    176.47007    169.01961    170.51521    168.45833    165.23021    169.51629    167.39221    171.11165     194.5827    230.98042    254.65506    241.68991    196.54536    173.57732    155.89658    162.30456    151.47014    151.40825    147.56261    152.41822    148.46414    149.21756    140.58839    145.46648    143.02851    141.11042    140.89253    143.14562    142.88836    138.75181    137.16297    140.00042    141.00957    141.74988     136.2639    135.62546    131.46664    132.90639    135.83893    137.79608    141.96423    144.78837    138.07622    140.36332    136.67806    129.03429    131.36169    131.53664    127.06353     131.7973    130.04257    134.00126    134.85242    152.83681    164.89972    175.21748    174.12065    155.73226    146.70574    139.53541    137.95034    132.75473    130.43353    132.39326    133.88862      135.329    135.36237    126.96932    122.21711    123.25017    123.29385    130.42836    142.44549    157.83432    176.73319    186.22693    188.74908    180.86999    174.49843    168.66723     156.5468     160.0154    148.09268    141.92196    136.33795    133.26904    128.16319    120.50636    122.73223    120.85724    122.10354    121.16348     123.4066    119.74702    121.64838    118.14755     117.8981    117.00176    115.76375    111.95814    114.02407    118.21872    113.18629    114.06684    113.40743    116.32651    115.78654    123.17623    128.62524     133.4207      132.987    128.65972    123.73431    118.37381      119.539    113.56362    117.30463    120.16385     118.0612     126.1884    142.52337    154.00683    161.95837    167.51898    164.72511     149.2615     141.3882    131.20808    129.01544     129.7498    126.97847    127.35149    122.83911    122.68446    120.48803    115.86499    115.79742    113.61124    113.91953    113.56069    117.18588    116.43087    117.62022    115.15414    116.95129    113.95455    119.56588    119.00884    119.89178    121.44323    121.98305    124.71387    123.91909    123.48259     127.6426    121.91025    126.32637    128.60064    128.42021    129.85768    131.01789     134.5497    130.66547    131.07735    125.61725    122.07826    123.29719    121.70553    120.97244    118.56368    118.47738    118.06827    119.19836    118.04212    118.40181     115.7028    118.16325    120.81394    122.18183    126.86537    127.89554    127.14846    126.83528    127.28471    127.09851    127.46265    127.37605     123.6377    121.20943    124.46239    121.99391    130.15528    125.42425    128.95286    127.94589    125.50962     124.6941    122.53193    122.00371    116.60941    121.23557    120.23066    123.51621    117.53996    123.24391    121.82524     125.2564    124.52457    125.75724    127.10418    131.17463    128.32886    131.90674     130.3585    125.23511    128.33018    132.17495     138.2142    137.92572    140.70803    144.53691    144.37185    144.19893    143.43805    141.75963    136.61582    136.34059    140.38157    134.23985     128.8213     133.2837    131.77155    136.30956    141.22943    144.36216    140.59924    141.17131    144.69617    139.43455    137.59992    133.08773    133.42686    133.97011    138.48332    140.90352    139.43616    139.84095    143.41625    141.41803    141.36849    139.91835    140.48434    140.40375    136.75921    135.01946    134.60831    135.25431    129.63537    133.17905    129.96113    129.96758    127.74493     133.6743    132.41403    133.19731    131.82662    133.16679    130.68239    131.67462    131.08745    130.96091     130.6272    130.03137    129.19192    129.75248    134.08442    131.97825     129.2699     134.0447    132.24769    126.18387    128.95325    129.13557    129.68322    132.94501     130.4513    128.38229    126.08789    129.92038     129.8434    129.66374     125.6786    131.89932    131.29426    130.85562    137.59601    126.97567     136.2002    136.58354    133.87463    132.69394    134.56476     130.7607    132.00264    136.40459    131.09245    132.04487    133.09813    135.32357    135.20972    134.63714    136.72016    136.55343    138.68672    143.13995    137.80558    138.98152    137.08506    137.27077    137.43684    146.45888    148.65704    149.00062    149.84801    146.98877    140.85835     148.7923     151.0203    146.71283    154.80686    153.74732    149.97895    151.57338    156.23711    152.67905     157.6996    160.18921     160.4688    157.64327    166.00305    160.55986    164.61521     162.6823     165.2949    164.04227     168.5718    168.41157     167.7313    168.81646    166.40085    164.43845    167.74416    169.37383    167.49525    171.27418    171.37102    166.92005    172.92455    173.14392    180.11097    176.28248    173.64849    178.08042    172.40886    177.36717    187.35499    184.73737    183.46623    180.02093    179.63941    185.06606    190.13565    184.42687    190.05101    187.48148    185.57312    188.53116    182.90814    195.79085    183.93312     189.3415    189.98225    198.75508    196.08935    201.54661    209.54212    202.90106    196.35462    198.30964    202.78161    207.17199    210.00761    203.12935    208.02348    214.15642    210.59709    209.37075    213.56038    230.29854    221.17498    220.11978    213.02759    210.91026    214.65939    225.48407    232.67025    225.83424    234.82936    228.73162    221.76318    236.77579    224.13181    223.57546    234.43694     229.5392    231.18377    244.77823    248.44332     251.4373    233.83347    254.01169    247.53707    241.26256    256.49162     262.4815    248.30948    257.22834    258.22894    253.42293    268.59348    258.31832    281.47676       271.07    267.36018    264.71924    297.19657    297.21064    269.32529    289.58641    289.86248    289.89672    282.31576    329.31166    317.86796    296.45219    282.21445    321.87552    300.81314    339.48676    340.42728    366.12837    363.49975    344.25069    361.51514    361.36904    427.35281    392.24678    376.35121    407.91229    423.39745    427.57744    442.35718    530.35366    494.79072    549.29024     514.6214    472.24018    590.35123    729.01705    763.53018    682.40797    797.94521    810.01336    771.46575          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -2.00    258.67122    249.47722    228.63843    224.33494    222.16289    220.02763    220.41106    204.17472    200.20374    198.62259     187.2479    187.96427    192.27525    179.13422    175.57447    178.71643    171.86387     175.0712    170.97314    171.16966    167.14597    165.49249    168.77163    163.22846     173.0446    193.24414    238.23366    251.10632    235.81019    200.49281    169.32879    157.51884    159.28354    150.99446     151.8043    147.99789    147.40643    148.55745    147.89311    144.03425    146.62648    144.43685      140.207    142.41489    141.80807    141.53355    138.71289    138.30731    139.82388    141.92328    141.39238     138.2129    139.77699    136.30963    134.01709    136.95979    136.46856    141.51334    138.78404    142.63877    138.18251    136.09135    131.80254    131.16267    134.31605    127.41694    129.23678    132.50041    128.41672     136.8826    150.93595    167.42415    174.98349    171.17771    159.11715    142.91474    143.48482    137.45529    132.81643    134.16186    133.88541     133.8227    137.48034    130.96127    125.34838    123.25152    120.47276    127.15172    128.89325    141.17263    160.76294    172.69573    189.30796    188.42881    180.86948    173.89586    168.66697    159.20554    161.33999    150.01726    143.54067    136.77959    134.52601    130.82939    124.00943    121.51734    120.68871    119.17903    119.06027    121.01248    117.73179    119.28697    118.43654    115.42968    115.34532    116.15356    116.39547    116.09133    119.64621    118.24137    114.44438    116.77781    115.39387    120.88314    130.21472    134.27467    139.09447    138.53666    130.77262    125.40533    121.42834    120.81909    121.17654    120.40271    122.24655    120.61757    126.44585    143.23075     156.3387    164.94955    166.01236    159.80406    150.01749    145.03371    138.28541    130.68949    132.40609    132.83697    128.04921    123.67991     122.2681     125.4328    115.01773    114.83057    113.39592    113.26283    114.18019    115.60997    116.82298    113.98123    115.24701    118.34806    115.66926    119.45387    123.64125     124.3561    125.62036    124.91962    123.76791    127.93324    125.74904    121.85532    125.97407    132.56194    135.35429    129.68665    131.37408    137.00179    137.39933    130.71205    131.06561    124.04907    129.73926     127.7011    120.34857    122.47894    119.82408    117.42058    118.87546    117.65945    115.85335    117.00902     114.8902     121.9208     118.6306    123.73621     124.2425    126.52033      128.648    128.78568    125.74275    124.79977    128.18009     122.5724    129.66877    125.76586    124.84541    122.33786    128.07886    126.89402    126.22612    127.09647    124.97673      122.154    123.11972    121.21631      121.565    118.52174    119.60927    124.25201    118.49413    122.47201    124.39482    128.47063    128.04625    130.64761     125.1123    130.32815    132.52018    128.90663    127.91669    129.96697    133.71338     130.5889    135.07628    133.84574    137.11945     139.3339    141.26312    140.42852    141.12509    138.58398    138.61032    136.61817    137.59134    136.14352    132.09124    135.49657    132.13806    138.21893    136.52373    141.52703    139.56393     142.1627    134.13487    132.78861    133.20999    134.78403    132.29564    134.02777    138.44294     137.9214    140.62229    141.25209    142.54161    142.82976    146.45671    146.92173    146.61767    139.25181     137.7597    137.42499    132.21117    132.51099    133.36353    127.94804    126.83048    130.67852    135.42875    133.88483    133.65395    135.39714    134.69549      133.959    131.86386    132.77895    132.84678     133.8431    133.79357    133.36893    131.46336    131.25566    127.29799    130.04441    126.18391    131.42443    126.30433    126.93609    128.31349     131.1487    128.62052    125.11501    129.80075    127.30967    129.52041    130.17762    130.25083    129.87289    130.55666    126.14131    131.93852    135.80118    133.94591     133.9402    133.65561    132.53938    133.89366    133.28577    131.89525    133.65946    134.12154    132.15134    135.42022    133.13259     133.3931    137.51927    135.05185     133.7659    139.15957    136.05569    133.89483    138.54066    141.59061    138.90071    137.68823    138.73442    143.36858    142.17035    147.22853    146.01503    147.72162    142.71784    146.15111    150.24306    147.78677    148.65082    152.77775    149.39683    153.13815    149.96593    154.60762    156.59916    153.77552    153.94265    156.72696    153.86412    164.21992    158.93257    163.49608    163.63418    164.29783    163.67467    166.22135    165.40884    166.57433      168.029    166.80725    164.82447    162.67559    174.52091    167.66612    165.87839    169.66843    168.33033    168.40143    174.68348    172.20899    175.01034    174.29952    176.52088    172.27372    174.88451      180.964    181.25531    178.04222    180.19085     185.2448    180.61675     180.0437    191.49599    183.00566    180.84792    186.94335    185.65653    184.44196    191.30035    191.91206    186.30456    189.06763    206.21173    198.40419    198.82759    194.52631    195.27969    197.60954    198.69282    199.75163    191.85791    201.10454    203.17936    203.60967    202.57075    210.04653    208.26234     225.6975    215.23023     224.6347    206.19228    210.76847    219.34026    213.56254    228.49507     224.8375    227.76134    222.31172    221.56395    231.49039    232.88778    233.80213    235.25879    235.05123    242.69262    234.22674    230.99043     231.7455    246.48661    221.06614    251.19936    234.81419    225.27842    246.56792    256.01295     247.5156    255.74077    238.38098    252.90199    259.50007    264.36046    270.24643    268.43228    265.57103    250.40144    286.49209    275.37486    296.95803    289.14619    262.27024    288.40964    290.78181    310.90478    321.15944    322.82465    325.42532    293.01987    331.71022    346.24137    367.15564    366.76699    366.22764    313.52121    367.16945    359.88793    380.39354    431.80262    406.08383    401.90292    405.40765    412.81991    425.02108     559.4297     533.1685    517.90596     485.5433    507.93429    563.12263    755.94884    712.03188    845.54909    716.91393    775.47782    676.27252          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -1.50    259.18677    249.99986    225.52368    222.56576    219.28692     219.3044     216.0423     201.8573    196.05056    198.86364    189.07557    188.48071    192.90094    177.84551    178.27004    178.35687    172.07638    173.79306    174.79693    172.67842    165.11344    168.40903    169.43426    171.55591    175.13923     199.1111    234.29025    249.78135     236.3391    203.38673    170.22385    158.30832    155.20526    151.04346    156.55016    145.28958    148.06572    149.92061    146.83039    145.04435    143.43437    144.88633    142.78759    143.00666    136.39206    142.62141    138.87967    137.02259    138.40047    137.59204    138.21337    139.27997    134.87189    136.49083     136.9502    133.69267    135.49834    145.38641    144.02736    141.82348    138.76624    133.80083    130.46971     133.8311    132.18494    127.86997    130.83688    128.35088    132.71966    134.49191    147.79686    167.62424    178.93101    176.82331     160.8814     143.5378    138.14834    137.22256    131.05881    131.08019    133.06863    136.95366     134.1292    130.14646    125.89817     124.2207    119.50494     123.3119     127.0926    141.14251    159.17108    176.79216    187.86595    187.03768    178.04008    167.99304    164.58387    161.09084    158.45287      154.723     147.9764    138.03336    130.99908    129.62191    122.29848    123.03977    121.34005    119.48788    120.16778    122.33023    118.91496    121.41272    120.13377    117.92785    117.30011    115.35121    116.76294    116.01738    117.32539    115.31715    115.71319    115.01485    115.64659    119.54576    122.67722    134.02651    139.74649    135.32447    131.98784    127.88219    121.00866    118.96197    116.74065    119.53188    118.69087    123.31133    126.58936    141.70817     158.7079    169.94008    169.49685    161.73944    155.94531    143.49759    138.42011    137.35839     133.4752    131.73807    128.15085    127.09856    121.88714    122.01987    115.98693    116.21626    114.13338    114.05496    114.44886    114.76655    115.95242     116.6893    114.78414    114.46588    115.32453    119.13987    117.93288    126.15174    121.82314    123.60722    125.13777    125.68968    124.35137    125.10177    129.54747    132.53212    132.74214    128.78528    136.39626     135.8097    134.96538    133.57523    133.39151    126.98661     129.7358    124.44853     124.0633    118.76857    118.12062    119.71742    118.14317    116.32461    118.10358    113.16625    119.63376    118.56387    117.86986    125.58181    125.04428    125.99649    130.45482    131.16303    126.02632    130.06971    129.15783    125.02463    124.36142    123.94503    122.28763    124.80854    125.65211    126.80726     129.2417    127.81413    127.95046    123.34391    125.98612    124.83349    121.42849    117.55721    119.68979    120.98204    120.27479    123.33119    123.75219    126.36911    123.30219    132.62585    125.12885    130.56725    129.80398    128.12251    129.14758    127.72988    131.72407    128.18159    132.43136    136.46847    137.65582     137.8722    138.86629     136.4153    137.50725    140.66465    140.48443    139.35907    132.77304    134.80376    129.95157    135.32434     134.0339      128.036    130.82813    133.82968    132.04826    134.49631    135.07682    134.27942    132.07219    134.99877    135.19078    136.83806    140.49094     140.1498    142.85642    141.95593    141.21235    141.63385    147.16482     146.2576    142.61973     140.8865    138.38379    133.39789    132.42963    132.46012    131.75606    135.34988    132.67007    133.43911    127.17172    128.93878    135.64014    134.70719    136.75893    133.67126    132.98637    137.16733    136.03461    133.72566    130.66485    133.32435    135.30513    130.49385    132.92228    129.44874     130.7059    130.51249    132.04009    130.96902    131.37441    133.25515    130.03845    128.73776    133.03407    128.16316    126.58743    129.46471    128.99555    131.81811    127.74848    132.03523    128.77339    132.56844    131.46422    133.87398    133.75511    133.93925    128.66764    137.20355    136.60914    132.43106    134.20362    137.28079    131.94447    132.58322    137.78912    134.94442    134.38073    141.64426    136.30072    134.00038    140.47388    137.55413    135.40907    142.27918    138.19952    141.77789    141.84207     148.0441    144.82948    145.02965    145.64634    142.84945    145.16379    150.25862    151.11347    148.32191    147.11624    143.66006    149.88124    148.20604    153.75927    153.55542    151.60286    153.90544     156.7444    157.69953    163.14638     165.8637    163.79993    167.28771    163.21831    164.45289    166.73969    166.41385    163.93938     162.2723    163.29779    170.77528    163.10753     165.2466    166.84688    172.10139    171.39652    170.03798    165.98654    174.64921    176.51404    174.35949    176.61286     175.3476     173.8858    169.38726    179.84623    180.28482     178.0356    179.90196    181.15164    171.45441    186.83629    183.72685    182.86545    191.16677    179.49204    178.56294    185.57593      190.943    189.71209     188.7018    197.11568    195.91577     193.2971    192.12284    195.17575    204.16416    198.18195    194.97731    190.87865    200.77414    198.36102    208.92935    199.31302    203.23086    207.65286    201.24652    222.33953    218.55087    216.29852    209.48841    224.41816    216.92921    213.42461    216.14652    225.27532    222.00533    211.84591    215.24033     220.2791    233.30108    229.22285    232.67379    246.48461    223.10168    231.13984    233.50938    232.90965    235.64785    232.38707    247.11457    242.22181    242.30538    241.63362    234.69821     272.5136    240.39462    238.63956    242.37142    265.14141    270.99082    260.19974    252.88065    258.83602    267.29699    266.87855    276.83676    266.89045    274.59635    288.56275    274.05264    298.38871    305.98988    295.48116    311.35165    316.70104    311.61925    321.13924    326.91538    360.02494    343.30647    333.94411    344.33241     338.4861    364.17769      388.929    370.68309     417.6681    450.13847     394.0922     395.6461    433.53349    531.49578    562.00315    448.39851     517.4407    542.52245    602.36357    732.64102     699.9805    750.64417    637.03675    710.06912    750.25632          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -1.00    260.22368    247.67646     229.4042    224.95599    220.37404    219.80001    213.37606     201.5426    192.80433    199.07538    190.95709    189.42876    192.82241    175.94599    185.59564    176.98903    178.70672    173.20241    174.98467    174.56342    168.84394    169.13295    169.81295    170.80978    174.95503    199.36675    233.87628    251.06278    235.68037    202.27582    170.16741    158.46262    156.12307    151.84444    149.66187    151.19806    152.28874    150.53549     146.5411    145.86255    146.97052    146.09711    138.27118    145.33035     138.9815    139.92623    141.07197    137.86557     139.6562     138.2281    138.07664    137.04997    132.26581    134.08106    134.67244    134.13826    137.74984    143.50719    141.18956    144.83288    137.12694    135.38595     132.1807    133.91738    130.30858    126.29492    131.92967    131.13863    128.20508    135.26011    152.28403    169.69763    177.68743    173.48346    160.00721     149.0236    142.09558    135.97704    137.52764    131.79278    135.07063    135.25477     134.7455     130.8159    126.58832    123.68605    124.57134    125.21236    130.63525    142.70534    161.32896    179.69045    190.24345    183.00501     175.7244    165.37889     166.0423    157.74216    166.44162    159.41705    153.14315    143.79666    135.35571    133.00155    124.52537    122.43164    121.58038    118.09415    121.04367    120.96884    118.84922    121.52687     118.6176    118.60431    115.20539    115.89468    115.95897    113.92449    117.92901    115.51236    117.35381    117.85764    118.47612    118.79503    127.53688    136.70207    137.23715    137.78995    133.76032    126.58394    123.95391    116.83186    117.26522    119.22636    117.29054    123.61805    126.98278    145.23379     160.3839    170.39106    173.07689    169.12243    155.65518    142.82026    133.16893    130.66491    129.79933    130.69711    129.18363    123.29198    123.42551    119.58257    120.69411    117.03633    114.82109    113.42291    115.40666    112.71106    116.09446    117.04584    116.93912    114.45375    119.70359    123.38734    117.42441    123.71567    124.47889    127.06821    125.07136     124.1233     128.9689    127.38517    131.61374    126.67231    130.53744    129.88922    135.08726    137.75743    133.99552    133.28033    130.47743     126.4854    127.87187    125.79935    122.47937    122.28343    122.75697    118.39914    117.90428    120.59157    118.21048     115.8986    119.91449    119.27958    118.85252    123.63547    125.94791    127.15868     129.5234    129.34744    129.17738    129.19387    130.37391    127.82086     125.6788    126.32538    122.51906    125.04177    123.63553     127.0431    130.13121    129.38104    126.36455    124.01173    122.58748    120.44431    120.71625    121.23683    121.06988     122.2573    119.22395    123.57099    124.08388    125.39752    123.59693    127.58312    126.36004    131.47387    130.54297    129.84175    126.69163    132.78895    130.30335    130.47238    134.29997     135.4677    137.71436    140.27713    138.30496    135.56849    134.57883    138.84763    134.30572    133.80301     134.3735    136.64201    134.10348    135.05838      137.107    131.01395    134.05189    132.33227    132.82752    135.25558    137.14527    137.70835    139.91318    135.94822    134.89204    137.91721    140.85034    138.01735    144.14424    142.70389    142.65325     146.3266    146.54556     144.9027    142.62184     141.1156    139.01288    135.74827    133.78102    135.66545    129.79452    133.03567    131.38953    130.45316    129.68532    136.96412    133.24915    132.41781    136.94842    137.18556    132.79038    133.87074    132.67378    132.52242    131.55724    132.41326    130.18451    128.48794    131.84097     133.6926    130.24678    135.21071    132.36843    124.91377    133.64963    133.20653    129.25121     126.3237    130.66367    127.99522    125.51383    129.85809     132.4827    131.60749    133.57325    134.24656    132.34086    135.86379    133.94119    134.27842    132.49396    131.84763    133.35131    134.30076    134.09881    133.07335    129.28615     131.4676     136.3284     133.4092    134.31503    135.17238    136.71315    134.60625    133.58952    132.51013    139.18602    135.99072    139.34271    140.55471    142.03157    141.77693    144.63166    143.13931    143.91628    145.15723    145.20107    144.53516    145.39036    148.07586    148.43066    144.79787    143.40841    144.40664    147.26919    149.47309    150.63632     153.0904    154.34358    153.07158     155.6869    161.78834    162.24889    163.82314    160.54211     161.8303    161.97406    161.47933    163.52254    167.41358    160.68837    168.18524    166.81362    165.13399    167.69326    170.65994    170.75788    172.02506    164.53314    170.57308    171.64547    174.45989    175.72469    173.59894    174.05788    175.82597    176.39141    172.79298    180.53765    180.82361    181.84791    172.74666    179.47763    176.79242    179.11926    178.10715    180.16684    184.88781    182.52189    183.06855    184.18887    185.90928    187.56069     188.0793    189.59487    185.63009    196.84324     195.8657    200.46368    193.19202    199.42838    196.73075    205.31951    199.93498    211.64785    207.20459    201.80797    202.25981    198.56306    204.22393    222.37685    224.92447    208.81703    214.88331    210.77947     218.7929    215.43458    223.86199    231.32623    224.84241    220.96189    216.80423    224.77472    231.81814    230.10346     236.5349    228.61363    227.25767    221.19198    238.50237    238.12215     232.7666    236.30445    241.62995    241.81766     251.8242    239.95296     256.6162    243.91999    258.96265    239.39057    245.91945    242.23883    256.11925    248.36648    258.86368    268.02523    263.56568    275.39245    295.54928    293.35989     284.4247    292.49626    276.69896    287.89143    310.55261    304.66608    300.61099    286.87319    322.51772    300.34133    334.95904    377.11819    348.01301    355.62097    349.49028    370.46473    360.17942      408.978    427.40004    443.83756     405.3045    386.47871    390.64802    430.55765    537.06753    479.47544     545.6405    501.68443    493.91524    527.21682    781.56884    703.75587    714.72131    609.89314    640.11558    609.00588          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -0.50    261.31901    245.34324    233.84538     228.3116    223.27229    221.25342    212.37999    204.43151    191.78917    199.27765     194.2262    190.83544    188.31658    176.82471    181.31307    176.34617    181.57952    175.34903     178.5568    174.92914    166.54806    169.27651    168.23348    168.93427    173.14527    198.16604     235.0951    251.73054    235.49117    198.45056    166.28906    157.43704    152.68008    151.85406     153.4578    151.47745    152.79114    148.71114    148.82867    144.91079    145.35829    142.58236     143.0255    140.01308    138.28606     140.9062    139.96854    139.41052    136.77286    137.79811    139.35256    135.67238    133.60383    136.41705    137.29189    134.69127    137.37169    143.50388    145.60978    145.40471    137.40335    139.39194    132.99542    136.07651    131.61863    129.45173    129.84647     131.2236    128.18509    138.72258     151.4363     170.1743    180.22107    175.33184    161.07542     146.4611    142.06157    144.74232    137.09984    136.84708      136.644     138.4827    135.83753    132.98637    127.70362    125.33183    123.91804    124.03096    128.13638    141.54286    157.35624    179.40405    192.60677     187.1531    175.53669    168.17389    162.87044    164.05509    168.03252    160.21254    150.35875    139.80492     136.0722    130.24153    125.42752    122.27175    123.18317      121.508    118.57591    120.79925    119.47875      120.184    121.25771    116.11651     115.5558    117.97345    116.36678    117.27533    117.95007    117.75813     117.4996    116.02245    116.51725    120.97801    129.38979    137.01874    137.62493     137.7619    135.45321    128.21156    123.48951    117.30671    116.47177     115.4868    119.65487    122.79675    128.93474    146.53036     160.5913    172.79197     172.6806     168.7969    156.70883    148.36485     141.9008    134.28032    131.69965    130.49921    128.10974    126.81786    123.82443     120.7285    119.47627    118.46967    115.98458    112.10034    114.82648    115.19893    115.40679    113.30838    116.54045    117.28634    117.52565    122.96231      119.104    123.46167    127.13767    129.01175     123.1966    126.12636     128.7448    130.19178    129.99491     125.9876    124.49176     129.8406    134.51021    133.64265    133.73365    132.28754    135.37485    128.22759    128.11851    125.37005    121.70578    119.35165    117.94089    118.46638    117.20546     116.0667     117.8672    118.01193    118.92555    120.31308    117.41683    123.04184    124.62459    125.76742    127.72103    128.48192    124.81685    126.95699    128.22281    124.13932    122.71141    123.77439    122.54998    124.55113    127.51313    127.38566    128.51622     127.7591    127.31414    125.73908    121.56498    122.97946    121.17706    119.86829    117.10344    120.64082    118.27064    124.66606    123.20671    123.05911    125.41854    127.14744    125.90155    131.72486    128.57393    128.72807    133.69134    131.40786    134.06334    134.70397    137.00492    136.89848    138.47818    137.48065    136.94288    139.77097    140.68841    138.15063    136.04751    132.95329    135.95998    130.59607    132.76168     133.1818    140.17732    138.51584    132.65046    131.71498    135.43361    137.56137    136.78663    135.22479    136.54328    140.64068    137.20436    137.82319    140.51044    140.96413    141.45207    143.15782    144.10245    147.83845     146.0472    146.14385    144.62213    140.01821    138.72907    135.30484    135.32552    133.05215    133.75032    133.61726    128.06409    131.13862    132.55732    136.17246    135.93978    134.45404    132.59101    132.85572    132.86962     134.6806    132.78694    135.85661    133.80473     134.1547    129.81609    129.62762     136.5806    133.51475    130.23414    128.64453    131.00948     130.0999    130.93415     133.7022    131.48873    128.96974    128.88385    130.60217    126.80426    131.75462    130.40212     132.0297    133.29963    131.88648    133.08232    132.02241    132.66388    134.86018    130.09486    134.94467    128.46222    130.42089    135.92165    133.34955    134.23679    133.29277     133.0956    129.72347    133.71172    134.99115    135.52411    135.76732    133.68731    132.71878    136.76526    140.06607    137.03491    139.04632     140.9371    140.89491    145.20284    147.87782    147.84197    146.96257    143.14737    143.04161    141.95152    151.91327    150.03581    148.95863    149.35715    151.21185    149.21014    150.36966    154.34942    152.71527    155.68198    153.89436    155.74164    158.67367    161.77838     163.9951    163.35608    158.40018    164.54284    159.07884    162.42476    167.03373    163.29685    161.56963    167.98362    168.41338    161.90785    169.82098    171.71412    167.46884    171.68728    171.49049    171.18942    170.16642    169.54286    167.02656    172.30259    169.20085    169.16524    168.77074    180.20016     178.1914    171.03475    180.33851    168.87807    177.06929    176.35016    183.91831    178.88215    182.23092    187.11024    180.64355    187.48753    190.06408    185.50376    186.78302     203.2643    190.85944    188.27742    197.78487    198.87111    192.48435    194.66886    202.25845    194.79408    202.44113    199.98276    205.92292    196.75786    215.04248    203.73096    209.84262    218.43162    212.52623    210.99542    213.94193    214.90282    212.28267    216.61217    210.41958    221.54403    220.77713    213.11694    212.78748    223.42318    233.05492    222.58656    221.21366     224.8455    220.14861     229.0984    224.23797    237.18805     229.1716    250.36079    229.28592    243.90701    235.10471    249.74481    244.50938    269.49179    248.40668    245.17934    250.79414    260.30403    253.23264    252.62606    251.85446    259.32987    253.63202    259.34485    282.51234    259.37159    275.39206    275.13917    270.55895    284.20567    279.87784    313.71948     300.8394     341.3163    304.40722    299.72922     289.7959    341.71851    365.68902    342.27498    334.45956    338.69438    317.90254     403.0243    379.36392    405.15167    426.64266    413.63818    370.34668     421.2351    517.54413    480.14349      510.605    500.95528     548.3246    560.80819    610.22587    700.56661    708.48924    755.57226     707.9315    628.51965          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        0.00    262.48857    243.75884    237.00687    230.28711    224.90648    221.60308    212.61642    206.07279    192.77775    198.36656    195.46769    191.54947    185.67929    177.90783    177.37943    177.14317    182.99739    176.92938    181.77056    173.53777    161.78411    168.91426    165.69191    168.49564     171.2486    197.73068    235.14451    250.31981    240.41301    196.03779    170.76259    154.85465    156.13902    148.53597    149.96083    150.14788     152.0712    150.35063     151.1048    145.88631    146.10828    144.44104     141.5269    142.77167     141.9412    147.79735    140.78524    139.16142    138.76821    140.33071    138.16998    137.23743    136.32101    137.55255    136.65982    135.05001    139.31873     142.3257    142.01241    138.02828    137.62023    130.83806    133.24436     134.7721    128.25281    126.47077    130.19403    133.11609    132.65977    133.13685    149.12234    165.42236    179.56658    173.52248    160.71386    143.64758    142.06685    139.82277     142.7985    144.86923    146.42104    146.96028    137.39582    136.18595     127.4779    124.89312    125.80554    122.85144    129.12517    141.37555    158.40673    178.59233    191.13351    188.62029    181.02862    170.68402      168.311    166.68515    170.31548    164.18029    153.63387    141.43944    137.07658     130.8532    125.63547    126.09335     123.5892    119.38645    117.17467     118.8716    117.36709    124.13362    119.45355    116.65824    116.33882    115.79756    117.42323    113.14444    117.20182    117.04783    116.58201    117.09726    120.20055    117.86418    129.11227    137.01579    138.83388    138.39206       134.95    128.14489     123.9453    114.66026     116.7925    119.19711    118.89502    120.96646    131.90246    146.81635    160.59944    169.93179    175.78388    170.50806    155.29795    147.37254    141.02626    133.69511    132.22368    131.32828    128.72035    125.41107    123.85635    118.54092    119.40171    114.41193    115.97518    112.31353    115.40029    116.29551     114.1991    112.12325    118.42641    117.46551    117.91109    120.62871    121.84626    124.36384    125.47663    121.23624     122.4871    121.75675    129.39938    128.99512    129.22898    130.89191    124.51724     128.4186    134.63819    136.62609    136.98006    133.00934     134.8515    129.46543    128.89844    130.01932    121.54269    127.34009    120.86989     119.5146    115.50047    119.70017    118.31051    118.47536       118.34    120.07453    120.26929    117.48392    124.79489    124.67366    130.01451    129.41287    127.81091    127.20052    127.49797    125.24958    123.17841    123.59168    122.00749     122.6649    125.59771    127.96658     125.1177    125.23796    126.20072    122.53667    126.38429    123.23177     119.5849    121.74479    120.76533    119.22549    118.47923    122.30008    122.87225    123.60198    124.53447    125.18359    123.92191    127.14855    130.91287    130.38117    132.51331    130.21724    132.62535    136.79384    135.79453    135.75241    142.38053    141.86046    142.05699    137.58815    140.16114    140.25933    135.38535    135.43758    138.33977    132.04138    132.43446     133.0995    133.01005    135.83946     133.2626    133.32734    132.18858    135.68452    136.56887    138.95816    138.96557    137.18507    137.74522    138.66772    140.98445     141.6666    139.84851    147.82955    144.23091    148.25432    146.66346    143.68296    147.69703    143.78955    138.77805    131.33597     134.9265    133.95185     131.9252     134.4903    134.77802    128.98677    132.98632    134.72141    134.92202    137.17409    132.69164    134.72756     130.2235    134.06085    137.81347    132.58274    131.86417    137.21246    132.86874    132.47404    132.75626    129.22087    133.13715    133.28686    130.87231    128.59857    131.44452    128.74126    132.21153    131.19671    126.38216    130.31546    129.59952    129.98469    130.27928    131.15506    133.14939    128.70005     132.4486    127.28024    133.01183    132.44168    128.14631    137.38953    130.80102    130.40889    136.15667    137.00832    130.76088    131.26647    134.44295    132.20999    135.40931    133.80238    132.21952    136.18588    131.62909    131.37637    136.32133    140.31007    141.14778    140.40159    140.17267    139.89271    143.32752     144.7055    145.18774    147.62665    148.23634    146.54023    144.84938    147.49406    148.80575    144.93139    143.65466    146.61158    150.40511    147.74302    157.14457    152.68434    149.52414    149.99999    153.59747    154.50658    159.62187    158.40751    165.59169    162.97047    160.21626    162.19472    158.70938    166.60824    156.87329     165.2318     163.0276    164.00136    166.17038    162.94757    166.18458    169.11916    169.44519    170.52707    163.21774    172.52117    178.78709    166.01816    169.95295    174.68684    168.32786    177.98118     177.0415    176.74983    174.33365    176.92682    173.93768    177.32687    182.72355    177.36992     181.1179    178.38619     181.0591    174.71054    179.72925    174.60628    187.66672    188.19475    190.01057    190.24515    186.87908    190.52415    191.52344    191.13189     200.0632    197.38241    192.80807    203.95849    204.75827    203.19631    207.67482    206.11424    195.20661    210.09058    206.37004     210.1817    208.27293    210.27584    217.74457     217.2304     212.8118    219.43828     220.7859    222.22625    217.03195    209.66203    224.72075    221.27579    220.63252    222.45643    220.31311    213.19893    229.15667    235.09541      242.309    231.62513    248.78647    240.85846    236.05461    241.67341    241.28897    244.82262    236.54896     241.2496    242.22367    231.64933     253.2012    260.54805    252.38432    251.91985    249.03277     255.2473     262.4931    289.75219    280.60123    269.01728    268.31695    277.17675    275.06999    293.95929    293.42351    305.10503    306.54351    295.88848    305.97314    327.62321    329.18648    351.27368    342.33744    327.18673    337.97363    361.19361    407.19653    398.58812    430.48546    422.32143    383.11042    396.16297    432.34138    496.62889     462.1319    522.04415    494.70421    527.18096     575.1508    598.79621    715.13002    671.16759    581.85943    609.79731    683.91852          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        0.50    263.74372    243.53239    237.68633    229.47109    223.74747    219.34495    215.94808    204.46325    199.14579    194.15504    190.54974    189.93059    188.88726    178.97447    179.20575    182.47108    182.75075    172.89098    177.43971    168.04124    161.83217    167.77103    167.20977    171.35928    174.30429    196.85018    236.15611    256.48641    235.00747    195.70956    171.22533    158.53765    149.52338    151.77343    153.26705    152.20068    152.64264    147.16011    149.23302    140.71591    148.05101    142.22485     142.8081    143.10249    141.26693    142.16289    137.39223    140.98601    137.86905    137.35022    136.34734    136.22709    134.82309     136.6441    134.88482    134.98076    137.55195     142.4971    141.08681    141.48662    139.17806    137.01258    133.69274    135.49414    129.57347    126.18432    129.73569    125.50742    131.32746    134.84882     150.7288    165.18137    177.55667    172.51076    158.01149    144.81401    141.91977     144.0528    146.93341    150.35079    155.18413    152.10966    141.70378    131.82603    124.67923    122.34487    125.39253    124.98882    133.04398    146.39691    164.52175    183.00879    194.07939    193.13838    188.33069    180.23228    174.74136    166.55567    165.12842    161.10393    153.64649     139.2985    137.73261    130.60835    124.07295    121.20732    121.69993    120.12996    122.91564    123.26195    119.30903    119.73274    121.71441    120.63431    119.73116    116.71736    116.30563    114.63611    120.75873    115.88219    117.56624    117.35029    117.43109    118.46286    129.89912     135.2219    138.37471    138.93191    133.86467     128.7163    121.56207     117.9147    118.66366    117.82534    120.88756    122.25112    129.36888    148.87143    161.44345    174.41047    176.53976    168.11494    160.50041    146.25519    142.04601    134.98132     137.3115    132.70362     128.3805    129.16167    122.71516    124.20065    119.17477    116.42494    119.08463    114.73821    112.28487    119.47372    118.40768    114.79172    117.46297    116.19414    115.73587    124.78374    122.82133    124.22882    125.89353    125.42471     120.5889    124.51608    125.12432    126.34239    126.51258    125.00699    128.76811     127.5695    132.66855    135.26136    135.75107    134.05177    134.58455    130.49539    130.65511    127.11608    127.08118    122.39184    118.25518    117.70547    119.12039    118.12901    117.64497    117.77199    117.37198    120.13933    118.18624    121.82294    122.46897    125.29907    126.19929    127.92577    127.22753    128.39526    129.57763    124.79085    122.53059    125.71974    124.40233    122.56157    127.41714    129.09805    130.48163    127.99778    127.96082    122.61983    122.26442     119.9996    123.28164    120.86088    122.29001    119.86948    120.11859     118.9215    120.12897    124.26405    123.52165    123.97207    129.89118    127.12671    129.20384    128.34094    132.40334    128.03086    129.44776    135.54027     139.6836    135.45566    138.30823    141.40156    137.85768    136.54431    141.14992    143.16089    140.18953    134.16196    133.62269    130.11928    131.25362    130.19674    131.63847    131.16496    131.03755      130.724    134.82333    138.47805    136.64182     137.7615    135.28834    138.19841    132.30522    135.40702    141.60722    140.48853    141.81556     145.7528    146.34042    146.22504    145.26492    143.60389     142.9576    142.35086    137.96781    135.14576    135.84001    134.82707    135.03063     131.2279    134.39427    133.30612    132.14237    134.98199    136.78773    133.03292    137.61322     138.5243    130.47178     135.1697    131.30255    135.42725     135.0032    133.93684    130.51805    130.48488    134.95751    132.02818    131.89415    132.83885    132.25008    128.79105    130.87837    130.81548    126.73068    133.25093    128.69722    130.74237    128.10902    130.00953    133.89519    133.04507    131.76662    129.65089    130.81159    132.77061    134.99412    134.05499    133.74017     132.9495    131.46827    130.91311    134.08384    132.87703     135.6891    137.64846     133.8315    134.86772    134.13391     132.6031    135.61106    131.79746     133.1942    139.16951    138.83326    141.16668     139.5273    141.85973    142.46101    142.99246    143.51541    145.35452    144.70094    148.07959     147.5456    147.65349    140.71103    144.77565      148.902    145.51525    143.44385    153.02044      151.453    151.09745    150.49553    148.04516    152.16001    156.15685    154.80314    158.56548     155.5875    162.12108    162.24872    163.46533    162.80264    161.60733    165.36609    162.35387    168.68734    166.73399    173.13565    164.06883    163.24749    171.59409    169.82769    166.40057    166.08619    168.02252    169.27906    170.13779    176.51211    167.07397    165.39715    170.77066    170.35345    177.03612    174.97755    174.09221    173.74156    175.26499    178.13504    174.63939    185.78085    182.48132    184.13966    183.12719    178.50576    176.62889    182.52144    192.54742    188.57371    184.59762    181.03994    191.79531     188.2047    193.45822    189.82001    193.64384    189.04515    197.55986    190.21875    196.84749     201.1905    201.51241    203.38102    210.96899    201.52371    197.03187    215.59085    209.47138    213.74545    215.60964      212.495    216.86633    220.09439    219.19562    223.02248    216.90673    220.78932    215.79092    214.19056    224.52833    226.66814    221.32188    222.40567     228.1115    221.57671    230.99653    229.02873      249.906    234.05001    230.87168    239.58013    233.80215    246.48161    245.80756    236.43806    228.17596    231.12762    243.55283    256.82038    266.76618    253.74911    263.66143    256.02143    248.06511    260.32178    281.05957    290.48611    287.07441    263.74858    280.34377    293.09965     309.8803    293.39182    308.73732    307.56282    311.81964    301.52196    325.24909    355.74436    342.28199    338.66046    363.35381    321.59132    335.79679    389.86071    404.56246     350.7828    422.57155    424.11606    369.93306    399.02748    458.50382    425.02936     522.1581     505.8558    482.07529    602.13399     650.0979    758.98101    596.36407    757.51017    597.77179    670.97977          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        1.00    265.05849     243.3453    238.40932    228.68617    222.56093    217.16531    218.83562    203.44549    203.93759    192.57563    188.84847     190.6823    190.77982    179.54451    181.18258    181.90077    177.82431    171.30698    172.04898    170.56515    164.93048    164.23546    172.55146    166.26656    176.42806    199.65994    237.78756    259.77719    237.04045    196.19478    176.90284    164.07738    154.20901    146.73636    148.57657    141.85566    150.49284    149.30363     150.2694    143.91324     145.4847    144.04516     143.7126     143.9793    144.87439    139.32511    138.52498    140.70508    139.77591    140.59209    135.95757    135.64383    133.54017    138.16089    134.41573    134.60011    137.80946    144.88636    137.60972    136.55331    140.24775    137.69647    134.68785    137.91654    131.37767     130.9615     129.2554    126.97528    128.14879    133.62047    151.91036    168.47457    176.49239    171.67891    161.19156    144.82467    143.11268     143.9551    145.67144    153.23588     159.2038      148.451    143.63808    132.52554    126.55977    124.47317    122.26551    125.15792    129.37657    141.29791    161.93928    182.80094    199.95413    202.95178     198.4352    185.34904    176.05391    167.93132    169.52947     159.9005    149.28046    142.73265    136.54522    131.07838    126.02602     123.2663    120.19983    122.36636    122.34811    124.47667    123.16407     122.9501    119.47152    119.20862    117.93632    115.15123     118.6102    116.57075    119.43279    114.75548    117.14719    115.08627    114.61192    120.45536    129.31388    137.32063    142.99623     136.8727    133.16534    130.55023    121.03072    120.85049    117.76904    120.49789    118.98904    122.76208    131.28044    149.10151    159.54351    170.94708    174.28764    171.16113    155.85571    145.28771    140.46623    133.07669    131.48429    134.08781    127.59371    124.44419    127.81792    123.18797    119.46539     116.0643    116.47004    115.86569    115.79319    114.07664     115.4964    118.00797    115.07338    113.73601    116.55477    122.49079    121.16763    126.38381     124.3084    122.47057    119.92825    121.27879    120.57933    121.51505     123.9311    123.75237    128.11978    126.81446    133.05869    132.90823    133.93414    133.42698    133.03085    124.81044    129.66739    125.87694    124.26067    118.97924    121.55048    119.08158    116.68646    117.50936    119.05081    117.75334    118.37054    119.85314    120.04162    120.41517    125.74378    125.27229    124.90131    129.17322    127.37876    128.73175    129.38938    125.38534    121.90223    124.54035    121.43364    122.03095     126.8848    129.19477    127.37126    126.22495    128.68152    124.33007    125.95369    122.59161      122.594    117.60675    123.76873     121.3104    120.33642    121.42785    122.47566    122.11233    123.92459    125.61522    125.62633    129.58488      127.763     131.2315    130.57879    129.78529    131.37368    135.12382    138.74409     137.2159    138.18905     141.2189    138.58732    138.13424    139.94415    139.85448    135.70564    135.00435    140.60967    135.29124    134.88886    134.10986    136.70371    134.17159    135.00151    132.64869     131.2053    134.12394    135.92216    138.00913    138.55515    140.01879    133.48998     134.3805    137.75803    140.30096    143.38092    145.70164    145.90491    147.05487    147.47932    145.89563    144.87903    144.33129    138.08532    137.09083    136.20738    135.18157    132.93362    133.53053    133.35549    130.51612    136.19953    134.08487    132.30882    134.86249    136.86707    132.98736    132.57771    134.24995    137.99497    138.76594    132.47127    136.76245    134.45067    129.83801    134.17818    133.16915    131.27228    131.10491    132.60115    128.71511    128.13023     129.1364    131.02735    130.76434    126.32826    133.21398    127.10378    132.10278    131.15796    130.48875    131.54165    128.41675      128.236    134.10117    131.19308    136.10698    135.70822    136.75385     129.8493    133.66765    130.80584    131.09686    134.55138     131.0459    134.32247    137.57026    134.52477    133.23561    132.28277     135.0621    134.56743    135.68918    139.12933    137.00031    138.11157    140.86986    138.16949    137.55513     141.0022    142.14606    143.47562    146.69604    144.62885    144.50711    141.63602    147.51674    148.80832    148.16113    149.40263    144.07202    151.18682    152.04955    152.27801    152.20365     155.6779    155.68354    152.70973    151.01245    157.39187    153.46386    160.23036    163.08574    162.41742    159.96551    163.71535    162.22266    160.99685    161.04048    160.53183    159.02605    164.27307    166.29797    159.55545    171.85548    168.20284    165.10629    163.61749    173.33806    173.52031    172.29212    170.14771    177.21781    168.17402     178.6086    176.02035    177.41981    174.81795    173.09653    172.54958    171.82941     183.3603    181.36224    180.03548     176.9477    181.61589    168.36113    184.74229    184.84645    180.91562     196.8611    187.28989    183.72726    193.60075    197.96812    195.02102    192.69056    202.39523    196.61758    198.85256    201.36574    203.95046    205.27896    200.24725    199.81496    201.32155    211.37513     218.8957    214.55014    211.55641    212.14904    209.04447    214.12633    215.45184    213.59494    221.84588    211.91872    222.47614    219.59514     216.0598    224.02911    217.15713    233.72611    222.74198    218.74607    215.89641    243.57813    230.90428    239.43557    243.46758    225.65018    224.19604    226.83431    234.03518    234.18643      241.715    248.16054    246.69887    246.86329    265.62124    239.56671    262.38654    252.61305    244.19653     259.7494    260.68524    266.84614    272.81617    281.91823    268.55998    276.69195    269.39823    304.24796    299.32303    292.14175    309.98841    297.66754    311.37395    305.96325    315.71803    325.38202    360.96729    298.31305    355.41497    327.67519    377.38798    373.89281    381.47287    421.06879    382.39895    393.81348    452.17209    446.82282    487.62006    451.47987    452.16658    438.16178    467.42106    675.52034    593.61954    553.73221     723.4078    790.63806    686.04453          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        1.50    265.62882    243.57291    236.83089     227.7713     222.3538    218.99248    217.18989    205.58223     203.2255    197.34993    193.16633    195.21586    191.07651    179.69431     183.3462    177.02694    170.28874    171.60024    167.93079    174.30519    167.12409    163.09349    174.31496    165.20349    174.68532    196.93572    237.46417    257.15732    239.76437    198.38959     175.6709    161.43432    148.27241    147.79759    151.61094    148.59368    153.69288    147.20704    149.40824    142.60288    147.23686    144.24133    144.12633    145.92569    143.55472    142.51595    137.62455    136.44416    139.91317    143.07537    139.74081    138.55363    136.74277    140.29961    136.42096    139.33826    140.23671    139.45505    140.99468    140.33951    138.33396    139.34217    134.93894    132.00085    131.92188     127.9717    127.94727    130.30691    130.49936      135.367    149.73837    169.01151    180.22453    174.28904    158.83459    144.26123    142.25104    140.04402    142.95169    148.20799    152.94631    147.56879    141.28227    135.34022    127.69542    124.32973     125.3032    124.51474    132.02425    143.21229    166.40395    189.75228    205.39329    210.50734    204.23494    190.34112    179.42178    164.60504    164.40857    154.73127    148.09374    139.69989    136.66885    131.12764    123.71789    121.47052    119.48653    120.93979    123.07034    121.80039    121.31711    119.82125    120.58516    121.55624    116.57474    117.33304    114.49496    114.24198    118.55309     117.6997    115.83864    116.78186    119.26528    119.23866    129.31736    138.39747    141.98893    139.65724    135.42571    128.55796    122.89434      119.334    121.30906    117.87889    117.59605    120.96666      130.056    145.60118    160.14502    172.01381    171.20162    170.87723    155.61743    144.50608    137.29284     133.5618    132.68099    128.67664    128.72291    126.43841    125.41416    119.40166    118.24627    116.89589    113.48062    113.74686    113.64884    114.28051    114.34121    118.76348    112.75449    118.37872    116.08592    120.26515    123.49215    124.22271    125.44978     126.0008     122.8131    122.58238    121.68574    122.32292    120.30785    125.66662    125.22588    130.39464    131.09112    131.09988    132.02116    132.76561    131.06908    126.90602    125.24139    125.35861    122.48477    117.30026    116.28762    117.09821    117.51744    117.31127     114.5717    117.22833    118.16192     117.1371    116.08657    122.25829    121.34727    124.19926    126.64441    129.70318    127.16984    129.90227    128.42933     125.6301     124.9541    122.64585    122.36116    121.45547    128.19693    125.89578    127.75727    130.02866    127.86145    123.48696    127.57458    124.39022    119.62583    121.40879    121.01744    120.68533    119.61432    124.02828    124.18785    124.01187    123.03583     123.4751    123.21665     127.0216    126.75668    127.94388     128.3565    133.09941     130.5454    129.71919    137.01408    134.17521    135.30997    141.68182     139.0759    137.37243    139.89628     138.4866    135.90851     136.5756     136.8605    136.81808    131.81666    135.52915      135.616    132.57672    130.73083    132.73964    135.12552    140.32773    132.07611    134.63813    137.12914    136.30226    136.77249    136.40614     143.3091    145.14768     148.0302    146.72811    145.79285    144.63265    143.43944    146.29328    142.23796    142.29523    138.64964    130.42768    134.44282    132.43469    129.33819    132.20514    131.96324    133.79924    131.63385     135.0494    134.85551     134.3634    134.75497    133.97613    134.45388    137.97222    134.64179    135.09476    134.80295    134.21042     133.2461    131.02928    131.21857    134.93542    133.24605    131.17731    127.99854    126.62541    130.74549    130.60716    131.01835    128.40149    131.25205    126.22736    126.89026    127.26374    129.21573    130.21965    134.87271    128.37608    133.17497    134.61666    132.45879    132.54998    131.73685    130.38639    130.47395    130.99878    134.33951    132.56395    133.36785    131.61146    133.95284    129.21417    139.11322     131.7342    131.96081    134.67214      133.784    129.69975    141.79326    140.04198    139.40919    140.85757     142.1243     140.6072    142.63207    145.12895    141.93683    144.69715    143.96708    143.90573    142.84264    148.03949    150.03038    144.76617     147.5558     152.2743    148.21406     147.5568    153.07315    154.82694     154.5602     149.3765    153.15501    153.00414    161.56863    156.77732    163.67516    159.78844    160.82798    160.60378    162.52344    168.36294    167.87682    163.51106    165.70862    163.99362     159.7975    168.82681    161.04974    170.44981    168.36836    166.52862    167.15773    168.20885     170.9419    171.73959    172.14436    167.47436    171.59457    172.33158    175.15305     175.6917    176.66898    179.46688    173.25562    183.98888    179.22964    175.34481    171.83862    173.95093    180.58964    176.64989    183.74901    184.44835    185.27007    188.84271    195.57999    181.78655     184.5616    191.17568    195.06614    202.86963    200.91935    197.50694     188.8984     197.6507    203.69464     204.4851    202.06266    202.96418    194.27628    195.78211    213.10431    210.08305     203.7033    209.53523    211.32494    204.14162    212.11842    214.78309    221.10796    221.00135      217.702    217.69864     222.1507    223.10465    222.15326    217.64681    225.11581    224.56349    224.93624    229.15776    224.51049    224.79678    226.45074    241.34186    232.60689    232.65557    239.12466    243.25661    229.52775    238.97239    225.70111    239.45414    255.59368    242.76521    253.06618    259.93804    261.13063    251.74174    251.56796    272.31639    262.72584    269.56007     271.1001     277.4964    251.81133    297.04532    291.86278    280.56068    304.32088    291.70652    310.56277    316.68944    337.02748    355.72687    351.13766    335.38514    331.30086    319.51313    356.42025    417.74802    431.51845    395.44209    411.21425    412.02591    412.47107    542.53048    474.54814    437.92413    434.08066     569.1977    502.84374    678.33899    644.45241    609.03839    625.70827    666.52864    688.34873          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        2.00    265.51369    243.98508    234.49985    226.69944    222.30384    220.78755    215.60944    207.57403    202.14443    200.80857    194.42619    195.87346    188.82879    182.70139    183.38219    179.49092    173.38362    175.09465    171.21071    173.12488     165.3957    166.39668    170.81257    167.85351    171.85337    192.99017    237.26628     251.9389    241.22567    201.14588    170.95886     160.6035    154.87188    149.24338    147.49446    146.69351    150.27081    155.43416    151.87861    143.52649    144.12732    144.68103    143.03286    144.04765    143.18617    138.90267    136.58847    137.55264    138.85524    137.54915    136.22172    140.32518     137.7422    133.91479    132.29895    133.41698    136.62141    143.35145    142.36329    140.45998    139.21451    135.29775    135.05333    125.83463    130.77915    124.35189    127.81943    130.85309    132.71775    134.28649    149.13697    169.70722    181.33707    173.16643    156.96738    139.29096    141.23039    140.02941    142.17199    143.55323    147.00718    140.53255    139.10271    133.70284    128.03019    125.91356    127.25432    126.34841    127.02776    143.78859    162.80964    190.76914    206.93934    208.34746    202.54553    180.18192    172.42397    164.65574    164.06231    155.90274    144.08503    140.68684      137.081    130.82588    121.17679    124.85527    121.94933    119.46488    120.40108    120.71361    117.35203    120.98562      120.289     119.9737    114.07036    118.00076    114.00493    117.86413    117.98965     117.8907    117.79343    118.02449    117.10852    119.33234    127.62118    134.51981    138.02237    138.41766    133.45939     126.2947    124.17796    121.69787    116.37796     120.1253    121.15312    123.68633    128.45058    147.88439    164.00972    172.75445    174.77032    168.03337    155.40616    146.08946    138.30752    131.73198    131.60783     129.2561    130.67458    124.55621    126.02331    124.32726    116.66313    116.48766    117.08416    110.90114    117.87564    115.07436    118.03189    115.86768     119.3934    111.39689    113.24563    120.62001    122.01425    121.73927    126.40293    125.29581    122.61224    124.14322    122.00996    122.10811    122.08724    122.51138    126.95251    129.73305    132.88893     132.5373    135.44428     132.7217    133.64391    126.61987    128.42835    125.56987    118.60503    118.59566    119.89244    117.96249    117.99948    114.77327    116.90236    116.76878    120.79769    116.68852    121.58767    122.72367    122.88323    129.87926    126.75119    127.27485    126.41289    125.99077    127.07396    123.80288    126.91232    122.81595    124.87289    123.94248    128.66863    128.72278    131.55639    128.30784    128.65739    126.38459    124.01827    122.22344    121.61395     121.0763    118.99052     119.6732    119.77549    121.40268    122.66842    123.83998    123.67825    124.15415    122.79947    126.02087    127.30865    126.58714    128.53092    128.97311    131.38153    129.95386    133.10145    137.00047    139.23177     137.3939    140.91409    143.14833    139.41346    143.32801    143.02854    139.89953    136.22826    137.92628    129.80242    135.52615     136.6249     135.5497    133.76064    131.09917    133.71394    135.86399    137.59477    139.97112    138.83655    135.39524    132.38171    136.53797    142.99276    145.21353    142.86499    145.73001    148.80262    147.29203    147.29503    146.35792    147.30624    140.98688    141.91182    136.16652    137.73463    134.64125    132.26785     133.1555    134.76658    134.43309    131.46704    136.45835    137.37857    132.78841     138.5703     137.6209    134.54238    135.44527    134.57742    136.25859    129.98674    130.21844    132.02922     130.7969    136.87088     127.7975    130.81486    131.61683    132.91144    127.86713    130.01963    130.49478    128.80324    132.31839    127.16852    127.56019    125.10705    130.93491    133.30732    134.38009    129.97024    132.47445    131.81081    132.54004     131.6634    134.69049    132.91455     133.6082    132.75222    135.31163    133.73123    136.06196     132.1355    135.95476    132.92397    133.56299    133.21077    131.07884    135.53973    134.35922     138.0155    140.11102    138.55533    139.32048    138.40503    140.09636    143.14876     139.5232    139.58337    139.79278    141.08948    142.60342    145.82048    144.71399    144.64472    144.18876     150.8473    147.76144    144.79456    146.93304    147.21401    147.08498    151.43242    149.08874    155.60887    154.18461    151.14103    155.32626    162.26177    155.44696    160.05478    160.30579    158.95571    158.31799    162.26934    164.83271    167.50259    164.40911    166.29699    161.62488    160.23071    166.52763     170.7293    163.62583    165.34425    160.86775    172.88765    168.33078     164.6902    170.54462    168.43112    169.99925    175.50444    169.87461    174.26708    169.05711    176.32821    169.64585    177.57429     174.7889    179.74516    171.93826    171.17679    176.36589    178.51883    175.55344    178.11663    183.20021    194.18061    182.16938    189.81778    187.48539    191.03394    196.36205    197.09738     192.6401    199.25994    192.67019    202.51141    198.87868    203.48848    198.06987     205.0914    200.11065    197.53386    198.76564    205.83187    210.48189    210.70813    215.21413    212.52865    208.12556    212.31272    209.98813    221.84511    214.19068     224.2016    215.07454    220.06197      226.769    223.65631    220.79404    219.75326      216.796    220.36838    217.89938    227.09101    228.79601    228.29442    236.95669    225.34755     236.4751    253.54526     237.3563    253.21513    234.57772    226.04853    233.64674    254.69455     255.2075    252.42446    251.80786     252.6387      239.169    258.71582    278.64338    272.03484    286.19554    277.76451    267.65749    255.85788    298.13677    313.36864     298.7736    301.93305    298.78384    316.33897    318.92699    334.33503     320.8256    325.99075    325.73757    318.59199    338.41922     427.4366    397.18736    366.96368     412.2301    404.15772    375.15356    396.08428    492.07976     451.2744    438.84726    482.79732    492.41526    534.46108    594.91332    648.13438    700.44707    616.54551    677.90336    685.16894          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        2.50    265.41548    244.34492    233.00467     225.7329    222.71155    220.31718    220.77345    203.64189    199.79839    201.56117     191.9194    193.23202    185.06036    186.50279    183.49052    180.11772    175.53796    177.61164    168.89618     171.3429    168.99859    171.76413    165.80636    162.87529    174.48336    194.11012    237.77287    248.54412    240.08327    201.74145    169.74736    161.85078    159.96459    150.58261    146.51466    144.00982    152.04772    151.21784    147.06007    141.19123    144.31192    144.77206    142.51359    143.23913     142.6429    139.30741    139.52879    140.97083    141.39643    138.28034    139.18551    137.80821    135.76333    135.41972    135.39166     136.4468    138.58988     140.6892    145.66384    139.54487    135.88979    133.50666    129.22467    133.40008    126.73026    128.23093    126.86506     130.5576    134.67227    138.01787    148.64178    170.72901    179.66431    174.13389    159.41659    145.93348    140.60465    135.59532    137.17124    140.08993    135.48498     140.2708    144.46962    131.43404    129.86921    127.12452    129.33031    126.28844    131.81692    140.52444    158.88242    184.33526    202.56274    203.42733    184.25048    172.17076    158.51716    152.11072     158.6564    153.51169    147.70448    142.44428    138.16416     132.4292    125.22453    123.87467    121.08438    123.29453    117.46406     125.1659     120.2023    121.11179    117.97642    116.52788    119.42984    118.50612    114.38502    112.65623    119.18714    117.72455    115.85418    121.33749    120.98234    121.36991    130.00049    134.68641    140.76412    139.25167    132.00713    127.21256    121.07779     116.1731    117.22442    116.42795    120.35655    119.43751    129.37791     149.4585    166.34896    172.23192    177.33131    168.34221    152.62023    144.43169    136.84425    131.80739    137.74562    133.33729    127.62799    127.71542    128.39428    121.35289     122.3034    119.02335    116.46207    115.40142    115.86259    117.22364    113.01737    119.68843    111.67051    110.98763    115.64289    122.92749    122.66913    126.09251    125.73275    128.35378    123.44819    122.09026    122.71554    124.28186    121.46795    123.00104    129.93323    130.57846    134.96196    135.41269     132.0227    134.50651    127.67339    127.04927      124.097    123.28894    121.35602    120.22018     116.8147    120.35174    118.57921    116.66669    121.62884    119.31501    118.84376    118.89101    119.64338    119.64526    125.50148    125.61839     128.9185    127.25859    125.67849    126.49493    124.68852    127.55254    125.01272    123.85283    123.82474    126.82226    130.20797    126.47939    127.40372    125.85544    126.68325    128.36423    126.23596     123.8873    125.25407     125.7307    119.82224    125.28635    117.87256     124.7949    123.39502    124.05415    122.91188    122.94124    128.63377    132.10812    128.02282    132.05843    128.60224    133.65431    132.12246    128.66583    132.85544    138.33711    139.12717    140.36083    138.59644    137.19509    139.23059    140.71718    135.87913     136.5662    136.46538    134.71308    134.76779     137.8764     136.0674     137.4406    136.45189    135.18908    129.56551    135.98682     136.2482    134.54467    137.41355    139.84674    133.23353    137.70558    143.41097    138.28052    142.81175    145.54147    144.64196    148.28007    149.52261    150.57538    143.23404    142.91564    137.30194    132.53149     137.2881    136.18595    133.86619    132.23475    130.54561    133.27845    131.06346    131.90467     135.8064    132.53056    134.68622    132.93062     134.0417    133.09212     135.1542    135.43799    131.49101    134.34245    130.01979    129.16912     132.7579    131.70906    130.76198    130.25684    127.17344    131.80595    128.66174    137.13268    129.68541     131.7287    130.57074    126.03584     126.6838    129.61837    132.24988    132.76059    130.40394    133.60967    131.16334    139.06125     135.0477    135.14882    133.92536    136.50682     130.3561    133.51758     135.8314    134.50269    137.22083    132.01688    132.02611    133.61069      133.417    135.39094    133.51528    130.07335     136.3824    134.63188    133.38209    134.65328    142.90149    143.03801    139.24365    137.26664    134.85266    143.26982    149.25593    144.27295     145.7698    145.38898    144.43808     151.9822     149.2572    142.17637    146.52792    147.00452    144.04535    146.07962    150.08168    151.80321    148.44042    153.19933    154.29002    154.53814    159.46477    158.96117     160.7053    153.47594    156.72891    163.40799     155.6283    166.93513    164.46391    167.76995    164.05675    158.75538    161.21309    164.33865    167.41778    162.36974    168.56978    169.27358    161.41575    165.03467    169.57318    170.41344    178.00599    174.09807    168.88464    169.08855    167.45741    167.17594    173.20838    180.38481    177.11087     171.0372    183.42662    181.81062    181.27429    177.16888    175.17608    184.93336     179.0991    182.75481    184.97337    189.28862     184.8988    188.91257    188.77042    188.67982    197.83409    188.34083    192.73471    188.08904    203.27916    186.79167    203.79948    192.20626    198.01268    208.49086    197.63946    204.35482    210.80669    207.40428    212.78125    210.41424    205.67643    210.73167     209.5969    224.06288    217.79024    216.02891    212.85982    211.40601    214.70554     223.5874    225.46098    212.09531    221.54353    214.39995    218.21665    230.21834    229.32181    226.26693    236.43668    222.41456     219.1964    229.70544    235.46089    262.83119    242.39379    239.35138    249.21363    226.47725    255.04597    257.48372    249.13637     257.0738    258.97566    264.00253    267.78513    255.17636    262.84616    277.78784    265.79526    267.06911    272.69595    293.09227    280.61161    281.96683    298.26825    316.89395    290.06327    333.00839    343.71974     345.3195    339.18571    317.95814    334.08598    346.21618    386.27083       412.44    404.63086    382.06736    359.23155    375.99337    408.23297    484.08394    442.13287     484.4628    504.12341    450.22033     456.8694    542.16664    652.42168    645.36018    631.11208    573.90439    641.61067          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        3.00    263.53412    247.07286    235.64582    225.29561    223.23889    219.10044    226.65476    199.41748    197.77913    202.45921    194.17132    189.45503    190.09642    184.45567    184.48216    175.57932    176.79389    179.19221    166.22302     171.3978    170.62632    171.66959    168.83823    165.79137    167.90933     197.9264    233.82886    252.00243    237.64352    198.82019     171.3076    163.56829    161.65077    153.11416     147.9994    145.74162    153.11898     149.8252    143.00788    140.21797    140.41791    141.76693    146.12552    143.21256    146.10151    145.31467    137.93137    141.55072     137.3292    138.55656    139.81249    136.06855    136.81158    135.10187    135.53606    134.37195    144.90869    142.00539    145.39102    139.47052    139.37291    139.26508    129.64874    135.97401     127.8539    133.18672    133.85836    134.55621    129.16074    135.59642    150.27444    169.56497    176.29052    174.52569    152.85306    147.76913    137.41379    136.98344    134.88287    134.94434    139.43358     141.5214     144.5084    137.64738    132.61274     124.9291    130.43129     124.9885    129.55954    141.10764     161.7264    183.23034    193.08431    187.80399    172.82701    157.56562    148.79288    148.05868    157.87073    154.73183    148.89734    142.16342    134.89292    129.22885    125.61301    121.35153    121.57635    120.11704    120.97061    122.46659    116.60456    117.07032    121.94112    116.84501    113.07791     118.2625    120.03721    115.15766     117.3418    114.83174    116.79077    114.63629    117.44531    122.79732    130.17709    133.26287     133.3339    141.06202    136.14251    132.23897    117.70255    119.49763    118.93449    118.79464    119.61669    124.79859    129.68551    146.99404    162.67459    176.21779    179.67498    169.75538    156.81726    145.04225     136.4215    137.34318    131.06148    131.62555    132.12748    126.01669    118.44525    121.52664    120.78738    118.73209    118.87886    114.91216    117.57768    116.43953    115.09045    115.38395     115.4725    116.95135    116.81515    120.38167    121.79506    127.17153    128.12894     126.9484    123.55862    122.12724     125.6391    125.50251    126.59249    125.58981    124.61383    129.36021    136.45921    134.04743    132.19313     130.9559    129.48427    121.68729    126.35819    122.39152    121.42573    122.71777    114.14284    119.30731    117.17898    117.94464    117.53119    118.11098    119.72529    120.59791    119.74453    126.15517    124.27255    126.59726    130.17681    130.00035    125.00251    126.86629    125.00785    127.35336    123.45387    122.83556      122.379    123.55945    128.00642     129.3551     132.1097    129.76156     128.1994    121.31787    127.80866    124.21987    121.51022    122.55743    122.19562    121.93212     119.4806    124.89425    124.62825    126.88188    128.73606    126.90618    124.93803    128.22514    128.55979    128.54536    133.14851    131.66985    128.45692    130.65155    134.23682    134.08442    139.86742    143.49807    140.58236    136.13455    138.56313    144.54838    138.74736    140.92266    135.78745    137.31242    131.96428    137.84401    134.49901    132.88403    135.71678    134.38357    133.14342    134.17175    133.52971    137.61534    136.48488    133.58588    131.50067    133.91585      141.496    140.30172    145.84843    147.72862    143.15436    141.79244    150.00423     152.3845    145.81254    141.14483    143.65295    139.36932      134.891    137.61515    138.94334    129.93699    132.66485    134.58515    133.92286    133.19093    134.07344    139.34622    138.69729    138.60348    135.19322    137.74988    135.66269    135.47286    130.90564    132.99116    130.71837     128.9179     133.1455     134.0701    131.52732    133.93207    128.29175      126.667    129.37643    130.53952    129.11518     136.2826    132.33452     128.4209    128.59046     128.2373    129.30409    129.93871    131.36671    128.74648    132.05013    128.76013    135.60186    134.43871    137.78041    133.81247    134.41795    134.72318    131.96677    132.11196    132.63496    131.48644    128.16214    135.08236     135.1446    132.77685    132.36238    130.85429    135.87766    131.20198     134.9574    137.11859    138.30696     139.7071    139.24949    138.64926     140.5567    144.87499     141.3583    148.57656    146.73936    144.95102     139.6876    146.54234    153.77223    143.43915    146.71667    142.12026    150.55681     149.5989    147.17863    152.49825     153.5035    152.82459    153.46268    151.91379     156.7256    157.27307    162.27976    160.67567    162.08842    158.71988    159.08038    162.11075       169.26    162.18738    157.94177    161.32121    159.66566    164.32381    167.43768    167.78023    163.32902    163.81989    169.05172    166.82176    171.42226    170.42314    168.45888    168.30794    165.39463     164.1079    177.40669    168.44505    174.32882    177.35782    172.62747    177.28873    182.57232    172.59062    174.23263    179.00698    178.01145    185.25762    176.93331    181.27686    182.71121    180.30448    189.89763     184.7462    178.54266      202.487    197.53292    188.12955    194.90528    190.93152    193.00869    192.83235    201.26923    202.75633    198.92579    202.69016    206.30091    203.10195    210.55098     207.2956    205.71061    211.81881    205.85843    210.98474     210.4315    209.63159    215.84734     220.9015    220.13434     207.5414    209.97558    227.47529    214.17312    221.28241    223.73143    215.55454    221.33095    232.38913    233.16792    220.27685    231.07209    230.47862    232.58276    225.15184    252.34883    241.11217    244.75631    239.15868    224.76287    232.52637    255.97493     251.1413    249.44542    260.57118    245.31698    233.88283    257.08633    245.57091    278.11829    272.42724    257.43212    257.13357    267.76761    278.95835    309.87101    279.52492    294.48987    286.38178    296.12424     296.8839    313.73526    358.53397    323.70062     351.5567    309.02416     350.1441    359.76947    382.11508    389.88994    406.73581    403.18459    376.43533    400.17207    466.42219    419.15505    498.57924    414.65492    406.99689    556.49605    673.43272     655.2718     653.0533     701.2862    610.89084    531.70773          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        3.50    260.84784    249.95365    238.19747    225.00245    223.44162    217.77756    221.30542    202.62864    197.31716    203.40901    200.36609    185.09688    194.90582    183.70948    186.93835    174.43526    173.87682    176.17443    174.08479    175.32872    169.48076    167.85213    172.22338    167.21817    168.87308    198.00046    236.27385    256.48712    239.92608    195.42616    168.10551    161.10009    162.33617    149.45745    152.42168    150.42115    149.73342    147.29411    149.27312    140.52775    140.12621    141.22179    144.62773    142.22356    142.06787    144.89455     137.0042    138.98829    135.65534    142.44541    137.55636    138.68531    136.42443    133.79102    137.45131    134.77495    139.32969    142.43226    145.20071    142.37573     143.5772    142.92904     134.4875    135.71856    133.52545    130.89102     127.5984    128.59899      127.791    136.06773    149.22248    168.31947    179.68997    171.24067    160.53136    144.97223    139.65017    142.02002    137.39399    135.90296    142.02739    143.69873    145.67608    138.85846    130.81249    131.03477    129.78979    126.14368    129.69711    140.99348    160.53993    180.79872    186.84454    185.17133    170.24451    153.63832    143.81646    147.32339     159.4797    148.28708    148.97831    138.33851    134.28394    131.04803    119.19175    124.22246    120.67931    121.17674    120.42226    123.94271    120.76501     118.2251    120.05939    118.97123    116.59223    116.57579    114.68503    113.96016    120.56421    117.55497    116.51041    115.95581    118.56165    118.44488    125.61759    137.18636    139.89697    137.37122    139.13962    130.18703    121.42962    121.62023    118.02108    116.64895    120.75327    122.43438    130.40631    144.35408    162.42088    170.37835    173.64378     169.3567    154.71714    142.94616    134.97461    131.98403    128.56389    130.22881    128.08029    124.11078    126.51525    118.54362    117.64953    117.41993    113.87536    113.92359    113.88976    118.86819    115.29226    113.84582    117.96731    115.81959    115.56848    124.14902    119.89877    122.30248    125.46244    128.36178     122.3578    124.48486    123.87953    128.77405    125.55728    126.78043    126.78607    128.01567    132.58358    134.78554    132.05025    130.42892    131.77441    124.71411    126.90001    126.61656    122.51372    123.08325    122.06064    119.06742    116.13838    120.88496    119.21112    118.87892    121.26962    118.76449    121.25443    121.96656    125.51184     127.3507      126.327    128.47849    130.32786    128.19307    130.85011    128.33371    123.40426    121.90069    124.25244    125.53713    128.80674     128.7021    127.55207    125.84878    128.65442    128.20173    123.64567    122.94537    121.50429     120.9453    121.14151    121.44172    119.95029    121.92326    122.52337    121.84933    122.31638    125.47483    124.68746     127.5015    127.37478    127.86966    129.98965    133.30422    131.42382    129.38032    135.92472    135.08219    139.51315     146.2477    145.69118    140.15399    145.48722    140.64028    139.47787     142.1146    138.21693    135.43717    131.43822    132.70287    132.25096    133.63495    133.86669    133.02004    129.34858    132.62709    137.07573    137.01209    133.31307    138.93545    135.45227    137.35038     141.3507    143.66706    146.37807    146.40415    147.93728     143.9672    145.81135    146.51558    146.28137    143.62793    138.80798    134.43244    136.07647    134.28554    131.26343    133.46084    133.76713    133.25018    138.40703    137.23039    138.51373    137.50285    140.67356    136.03988    134.67967    133.61952    135.52078    135.07677    131.02924    130.96777     132.2626    132.43596    131.99426    128.74469    131.26538    126.87894    128.97569    130.92738     129.2608    130.93612    129.72224    131.16013    132.66423    130.45674    126.71915    127.92452    127.63579    127.07598    130.01436    130.13439    129.03173     133.7296    136.59306    137.39723    136.66533    131.09643    133.51471    133.93782    131.82529    132.07787    136.83259    132.95606    132.92053     134.6324    129.80322    131.76272    135.02946    133.39018    132.47046    130.09028    139.46164    137.70034    138.31069     139.1221    139.99097    137.97337    140.49381    145.19503    142.66185    143.04965    144.90055    146.08505    143.13223    143.61092    151.69578    145.70944    145.03269    147.59816    140.98003    148.10404    144.59858    155.29349    154.39206    154.63193    153.89894    160.13307    162.32314    159.49436    160.11628    164.43872    157.16881     157.8941     162.2585    160.25677    162.26716    165.37337    165.31306    165.54344    161.15235    158.69542    159.75178    162.75792    165.56348     165.9017    160.58996    165.47938    170.07466    172.74123    169.58259    164.70845    170.28164    170.74114    180.39619    170.31679    171.46033    174.53122    170.43991    172.23131    172.55634    167.26185    182.54719    183.50342    175.88952     179.7325    177.16097    174.47518    187.18269    184.31352    182.51258    185.01238    180.64757    198.46662    188.53573    196.61257    190.12831    194.40435    188.74865     196.9192    197.63357     193.0997    203.68822     196.8523    205.35769    206.18448    207.36956    210.69792    209.10707     213.2567    213.52113    206.79159    208.91905    210.32444    216.83311    215.56385    206.41769    211.13716    218.96755    225.91808    227.57694    211.15077    214.86389    214.45301    214.05966    225.34632    222.16777    223.32946    222.87309    222.82606    230.29727    223.10874    229.15903    245.42875      242.822    236.74394    238.44833    225.48468    262.98307    244.78459    235.42076     245.3241    254.98863    254.49521    264.90817    261.42497    265.79977    257.13572    269.09589    274.50079    260.15373    297.93871    292.91942    293.11636    293.32689    285.03263    295.51445    291.16418    326.73486    311.00868    328.98258     321.0824    331.14204    338.88141    406.22261    387.59615    374.31838    394.66358    347.76058    355.92104    443.54129    437.26546    454.10997    423.53746    469.72767    479.58132    520.29047    588.25461    528.11099    617.68297    558.94731    594.34752    616.93161          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        4.00     258.2071    251.74311    238.81128    222.77168    223.32142    216.14456     211.0492    206.31202     197.2045    202.43378     200.0239    184.60584    186.99042    186.46792    190.57605    174.91985    173.59587    175.76434    177.23924    176.01638    169.60858     164.5871    170.57707     165.9013    172.34044    196.70251    235.01754    260.14082    237.20074     198.2513    160.89732    155.50403    159.67651     149.7716    156.92338    150.61926    149.06258    147.54244    153.02464    141.65636    142.22902    142.85278    142.41919    144.24411    143.85164    141.14011      139.298    138.39642    139.92621    138.11766    140.44084    139.48774    135.30049    135.31715    135.71455    136.47215    136.53157    145.58284    151.54792    141.82475    134.05426     132.8281    133.26925    132.16855    131.49054    126.12244    127.80705     126.8038    131.88223    133.06924    148.74801    170.84547    176.66198     174.8472    160.85469    146.34269    140.33561    139.15987    133.62106    138.29616    143.50664    147.65133    148.04447    143.30744    133.12657     127.0812    125.18685     128.0725    128.97354    141.98753    159.91526     178.7307    191.60596    184.22566    170.78433    156.00548     148.2664     143.2014    146.77757     149.8558    141.91128    140.05524    135.40239    133.43807    122.30261    120.87051    122.43387    118.31762     121.8973    122.75415    117.67519    121.76468    120.67192    119.10312    118.44263    114.99893    115.75181    113.39867    117.60209    115.77092    119.39052    115.49114    119.58506    120.52973    129.28363    133.48074    139.66738    137.64359    132.91686    127.31018    122.63073    120.98795    119.62298     116.4291    120.76009     125.7777    125.76939    147.95322    157.13108    171.43467    173.26687    169.56625    154.53863    143.12503    137.62414    131.46318    130.15453    132.55646    129.94744    125.96997    121.86624    120.86703    119.80934    118.03516    118.56952    118.79932     114.9031    115.78561    117.69189    114.80632    116.91383    118.69685    117.51253    121.43365    122.29022    123.72313    126.56239    123.26642    122.53919    124.10179    126.77522    126.61386     123.7674    127.49139    129.69538    128.29674    135.11334    133.88794    136.01566    133.17756    133.03272    128.59868    123.44475    125.18274      121.528    121.57555    118.83599    118.81799    116.05155    122.16242    117.83052    117.59509    118.18201    121.84466    118.13815    123.67001    126.54806    124.26904    128.84417    132.26293    130.71721    125.03126    128.75305    127.15779    124.39313    123.73484    123.51787    124.48929    127.74217    128.07927    129.83357    126.71638    127.22707    126.50834    124.37541    123.05463    123.04732    123.38188    118.51617    121.92257    121.89404     123.4532    122.57022    122.56034     124.6941    123.19319    124.57627    126.78526    129.99782    129.75613    131.43621    132.69459    131.25767    130.12657     135.9942    141.63722    140.38448    144.48223    145.52811    143.69294    144.07867    146.32568    141.38963    141.72924    139.53821    135.41456    130.99858    134.20039    132.30299     133.1842    132.61332    135.93987     128.3647    137.29167    133.96319    133.84063    134.02918    138.37445    134.69825    138.78655    139.45996    142.14572    142.97455    144.64248    146.89322    147.71763    151.98567    149.79997    147.39915    144.91116    144.04454    136.95481    136.59727    134.23835    134.47819    131.79835    131.93362    132.48342    131.14164    135.77728    138.96674    135.62337    136.62262    137.18578    135.45813    135.56889    136.11693    133.66909    132.27413     134.6342    133.71876    130.55224    132.11515    128.61654    134.30445    130.66213    129.04839    128.20987    129.76832    128.46775    130.64577    130.70754    129.19909    131.71919    125.54476    130.31685    128.06075    131.00115    131.67019    127.74173    129.92234    138.31781    136.16269    135.11604    132.35035    134.59014    132.77237    132.86765    137.01068    131.99479    132.76356    129.22437    131.57173    134.07141    132.58893    133.90713     134.4331    130.56897    131.02721    132.57171    134.34388    139.37822    138.85437    140.74036     140.0426     144.3681    142.34925     139.8933    144.45084     142.9002    142.51333    146.56973    144.88311    150.00442    144.85195    141.73807    149.45611    144.06513    151.13836    151.84569    150.59226      154.241    155.27699    154.73637    148.44569    152.29139    163.87319    160.56275    158.11433    163.06546    163.98656    155.56639    165.26929    162.23209    165.56187    159.84929    160.19914    158.69088    163.79632    169.84362    165.02405     170.4972    162.41904    164.14569    161.91642    170.87541    177.10499    168.19816    175.49049    165.59529    170.45759    169.02664    172.98662    174.81815    169.36016    177.78772    176.10831    168.88683    181.94578    176.44649    177.94175     180.9855     171.3805    179.56484     174.6535    174.23018    192.31891    181.70152    186.35262    193.46326    187.07215     193.3301    191.35731    192.22628    194.65411     193.9817    188.48271    190.35534    204.38636    195.69442    201.50758    195.92713    206.46269    196.77889    209.35874    203.92944    197.62228     207.0359    211.02728     202.9748     211.8201    208.39677    219.96878    212.20914    210.78002      215.138     212.7952    213.91669    216.64709    223.70513    222.44849    227.34347    219.63754    225.05753    227.34866      226.698    224.38871    224.26874    232.76806    229.38656    240.28014    238.27348    241.23699    238.22753    231.21749    235.93859    237.79368    252.80785    249.28453    243.21379    248.63351    251.89439     261.6393    262.45638    265.81663    253.00112    276.70574    257.07048    264.25196    297.31459    276.86885    294.94426    293.54215    302.12872    291.14662    320.40987    315.32465    366.26223    312.48613    334.10683     321.2855    335.40994    364.42839    382.30105    355.06012    367.51321    407.44157    382.08666    405.48048    465.97623    389.94584    467.92457    453.40606    419.87295    567.10936    623.80883    709.27645    616.59474    619.46379    599.74623    508.98584          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        4.50    259.85163    249.79312    238.19379     220.2271    223.38321    214.55427    208.52423     208.0335    199.77485    198.22811     196.0368    184.74869    181.16725     188.4858    185.38017    177.09689    183.01822    178.06104    178.92775    175.16683     168.3079    165.39606    167.45769    165.54803    173.19528    194.29054    232.67316    255.86904    240.17906     199.5609    170.35147    157.75369    153.67215    150.42617    153.99693    151.85708    153.56237    142.78195    149.97103    145.13193    141.55925    144.57325    144.06104     141.7561     146.9864    138.24646    135.33666    136.57936    136.60175    139.04923     142.6993    133.75787    136.07666    131.46031    130.45537    134.50471    138.87282    144.08115    146.31906    139.01563    137.53071    136.67236    133.36737    132.49771    129.43928    127.02193    129.87605    129.80461    130.77049    134.77994    150.78558    172.46504    180.08417    171.70049    157.15676    143.77424    141.23694    141.09975     136.4358    137.94562    142.07749    148.37142    149.63216    143.64927     133.2103    128.17549    126.71702    127.45044    126.77579    142.77393    163.71613    179.56361    188.76615    184.69595    173.57944    161.34772    151.87196    142.41962    146.45642    143.33866    142.12532    137.52135    138.00388    129.29522    125.07784    121.02659    116.96027    121.11981    118.38144    121.77454    119.83146    119.24284    118.69525    118.74549    117.78229    116.11682    116.78699    114.60794    117.56791    117.83935    118.67697    117.50081    117.06336    122.17617    128.12255    138.74245    138.44076    138.42551    134.91412    128.06271    122.23947    119.80109    120.43624    119.35112    119.11325    124.17892    129.00538     146.9839    161.60755    169.00661    173.96251    166.99584    153.61949    140.83329    137.19637    133.66094    131.13904    127.49292    130.37925      125.288    122.72902    121.16985     115.7399    117.61944    117.96394    114.25338    113.94869    115.74823    118.11345    117.86194     117.7339    118.83412    115.66491    122.25638    120.77472    125.03777    125.76265    126.07703     127.3254     126.3171    129.50407    133.89264    127.35205    130.53454    129.54932    130.09164    133.08436    135.73836    135.87223    135.04304    132.57333    128.68981     126.0704    123.34201    122.81707    119.44145    121.14215    120.29895    115.18864    118.19929    118.61817    119.71455    120.35079    119.00537    117.04842    121.19874     126.7372    127.54929    128.96679    129.36707    130.11687    128.82462    131.01283    126.44735    123.13706    125.60784     123.7479    123.44962    127.47461    127.62704    131.11985    127.71551    128.20784    126.19822    123.49673    124.33094    123.73343     122.9911    122.25741    120.71007     118.8144    123.51925    125.50881    123.98525    124.09903    125.66667    126.35978     124.8821    127.67992    132.60314    129.63545     133.6782    129.67288    130.38954    134.16709    137.96521    140.02537    141.38021    142.23942    140.72855    142.59438    141.79962    140.93193    139.55319    137.13479    134.51354    131.95443    132.84963    131.94047     133.8396    133.14173    131.86669    131.29854     134.1273    133.68384    136.02348    134.80235    136.35618    134.73288    137.80384    140.06707    141.86083    142.85522    148.65214    146.73306    147.44051    149.40295    148.37248     144.8337    141.68492    141.54757    136.12465    135.86885    134.34103    133.13143    130.49286    132.25277     133.0411    130.39347    137.79266    137.13025    135.49011     135.8588    136.96721    132.92907    137.12646    137.46185    135.65234    132.31317    133.30824    131.34114    131.17773    133.45541    131.14216    132.42519    129.08522    132.53064    130.23728    128.47115     130.3946    125.69744    128.99291    131.34176    131.75885    126.56286    133.27294    130.89739    131.93769    130.94685     132.0652    130.26028    133.47134    132.00136    134.95973    136.89914    135.59885    133.72712    134.52813    133.04933    130.96166    134.52048    127.76926    131.30635    131.11349    128.90052    136.04404     131.1476     132.4321    133.62026    133.60328    136.20105    137.34208     137.3719    139.84795    135.81159     139.7787    140.73903    148.24499    141.86269    142.81543    144.79585    144.65573    145.24254    152.79237    149.84262    147.29243    144.84645    149.05237    147.21812    148.35344    152.36076      158.952    155.50536    158.02974    159.66964    150.10803    151.14636    163.70254    163.57006    163.90407    157.27735    160.95159    161.14716    164.27592    162.80839    156.16604    160.98775    161.48822    157.61174    158.98659    162.78163    163.94566     166.9368    160.79976     161.5126    166.96112     168.5239    170.68535    168.40377    171.77232    169.99988    171.14387    175.49238    175.02723    176.83212    177.35503    180.28402    171.66334    184.47296    166.43694     175.1247    177.12576    178.32578    172.33651    173.92621    181.49587    177.96295    188.15441    182.30439    186.48345    185.07352    189.68602    186.63603    191.96008    191.66701    185.60142    196.44771    197.11582    199.58353    202.84193    197.82591    193.34718    200.75414    202.58997    211.18632     213.2715    210.52628    214.01124    205.87835    207.29022    215.99536    213.59623    213.69556    209.23776    214.98192    213.14861    210.31018    222.30347    226.39815    223.11828    222.13997    212.95746    218.52747    234.65848    216.77323    223.82784      238.296    221.32463    227.62476    235.77785    243.78015    240.30487    228.27421    234.16959    225.83534    230.26066    254.95937    245.21347    245.13781    237.82655    247.87657    241.66093    254.21304    258.61117    269.26027    277.35795    264.03359    274.22494    276.69655    277.65845    269.81842    296.61306    298.92556    295.48879    271.81844    307.68513    315.27737    331.25809    305.42513    306.22734    323.81345    302.58753    383.75416    367.96614     386.4425    387.42345    401.26043    353.54437    442.12863    471.45171     473.3481     417.8991    414.12076    437.72104    545.35355    528.19889    665.55094      631.906    539.32504    629.78101    660.45849          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        5.00    262.34757    248.05411    237.82331    219.79551    227.57671    214.26742    214.15297    209.72904     202.0786    196.06952    194.49514    190.71067    180.25066    189.83152    179.38804    177.02938    182.54594    178.53772    183.98777    171.76078     167.2215    164.79028    168.31467     167.6965    172.92403    192.99522    231.08075     251.1732    239.45435    197.31854    168.52373    153.23681    154.63723    149.20014    153.01119    151.42375    155.09891    151.94215    148.31201    145.60861    143.36575    144.00659    142.38721    140.54153    143.62586    139.48086    135.07888    133.66479    134.73017    137.49677     140.5783    133.45878    133.18823    134.36555    130.70187    138.80822    136.85561    143.96239     145.5908    138.02162    137.79085    137.67419    132.88078    131.60621    132.69183     131.3276    127.28023    125.25586    132.95927    137.26013    154.41819    172.59231    175.74451    176.19589    157.14725    146.43639    140.56434    140.30709    137.31431    137.69145    144.19823    147.55894    147.84937    143.55862     132.6278    129.04498    125.33809      123.849    131.09818    142.88956     163.5248    181.54966    190.03444    183.22392    176.06879    161.58179    150.19148    143.46257    144.59353    140.92423    139.69102    135.67515      133.079    129.84284    122.54931    121.47917    119.23241    121.23767    122.82192    120.04949    121.87507    118.81405    118.63133    118.31887    117.25811    116.11921    121.66741    116.17541    118.09851    114.82125    117.01481    118.60162     118.2631    124.42983     129.0908    135.69572    141.35949    138.66428    133.97465    128.74428    120.80191    120.68672    117.41862     118.8022    123.34333    122.19187    131.48113     145.4833    159.93305    172.71829    172.05805    165.36107    155.54413    141.80377     135.6927    129.69687    129.49077    130.33879    126.38662      122.259    125.44518    121.02411    118.99248     116.5055    115.13188     116.5175     114.8345    118.80117    117.36747     118.0633    116.62231     114.7063    117.43341    121.03586    124.60583    125.23335    123.13273    124.65584    123.36841    121.35686    127.03981    127.15108    128.74097    127.98666    129.08123    129.99193    135.57082    131.94683    137.13921    134.14322    128.90007    130.48299    124.45636    126.00372    120.37373    117.11032    121.63803    120.40555    117.12653    120.56979    119.28658    120.40003    118.12862    118.74381    118.55972    122.06185    125.67941    126.37634    132.01225    132.99589    130.52901    130.35473    123.64916    127.22541    123.99665    125.28936    125.49022    126.03917    128.46882    127.90694    130.79119     128.7431    126.48217    126.61799    119.69117    119.36328    118.80554    120.55415    122.30723    121.92356    119.37174    123.77301    125.62116    124.95007    126.95234    125.27264    127.15644    125.55953    129.13964    130.54085    131.39776    133.12587    128.74709    128.94114    132.58284    135.61186    137.60237    140.82153    139.85733    139.01678    139.97593    140.61421    139.78829    141.77172    134.86586    135.04644    129.62733    135.98106    133.68524    130.10667    135.38714    134.82743    129.31923    134.45833    133.31815    134.23715    135.65755    135.60946    133.45596     134.6142     139.4501    143.70923    144.38085    146.37111    147.53838    146.38875    150.74229    147.71117    145.64299    144.46786     143.2019    135.67109    135.70472    137.64792    135.64978    134.78669    133.66988    134.47404    135.34443    136.85226    138.32589    135.40148    138.25689    138.53448    136.96163    133.66728    137.27705    134.91634    135.21385    134.30771    135.78935     130.3759    134.38358    131.39861    134.68676    132.82765    132.69414    131.56193    124.76557    130.07736    131.36198    129.21346    130.75738    129.79445     129.2705    132.39448    130.24921     131.7842    133.18191    130.59836    131.16833    131.96926    132.39748    135.69494    134.00841    138.02758    131.57297    133.33581    133.20885    133.00882    130.42324    127.74869    128.03192    133.21061    132.67373    128.21738     135.5486    132.63518    133.41653    133.59569    135.17704    137.12597    139.89591    136.85178    141.29045    139.07422    138.45498    143.01995    143.19076    144.37302    142.87134    143.83164    142.75277    144.78298    147.18619    144.28645    144.48334    146.87225    149.02016    148.27409    152.41747    153.15263    155.28402    154.57596    158.20608    155.80555    156.37073    161.18435    162.25604    162.23748     160.6969    165.22519     158.7957    163.87091    163.24202    163.97983     158.0829     165.5034    160.93126    169.12555    162.23563    171.24759    168.93106     166.7916     162.7115    166.78076      168.494    167.72387    173.94618    172.43197    164.97589    161.45444    173.84096     177.4484    168.84808    167.00273    173.17001    170.40432     177.0285    174.76618    180.31409    186.87763    175.36809    180.35802    174.32071    191.40396    184.52886    179.31096    187.39305     189.3082    188.92434    194.27847    194.40447    197.88684    200.42256     186.4022     190.6251    199.46507    191.63538    191.25688    192.09311    201.63886    201.06264    195.03655     208.6272    209.25772    200.74185    205.64084    202.34821    213.72053    203.43886    216.67497    204.66764    210.13149    209.39271    217.26631    216.54974    221.48192    215.11185    214.40011    228.39279     218.9484    212.54916     229.4797     229.4037    232.93484    224.02743     230.8938    228.05196      229.209      237.146    237.81576    232.34967    230.89262    226.02274    228.25983     244.2782    250.80847    259.86443    246.63826    250.05787    236.21427    239.16213    268.16553    256.63122    257.65083    263.98327    260.73711    281.87768    287.01007    301.08337    291.58835    307.47607    301.66713      290.147    327.45485    324.38658     342.5608    324.80245    337.87932    321.68932    351.92667    379.90164    387.97503     347.4386    389.73223    369.00259    384.46768    433.79216    490.82834    433.21383    482.98185    493.63165    420.93828    589.22962    567.71925    686.90239    496.82432    616.18399    624.78479    646.00907          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        5.50    264.79138      246.509    233.87916    222.27111    232.94862    214.22442    217.52072    206.70537    199.68352     197.3569    194.04993    195.59082    182.75447    187.24854    179.97667     173.0282    177.84449    175.62189     182.0935    176.22808    168.33805    162.42235    170.50477    166.95414    173.08906    199.19715    232.07961    252.49614    238.59558    198.04457    166.37424    157.84235    155.76638    151.07107    153.11535    149.45513    149.24377    150.54613    146.75702     142.4459    144.51024    141.59262    146.82277    142.48779    140.49649    140.28396       136.75     135.4723    139.90341     138.9782    139.66706     138.8597    140.03533     134.0261    137.66977    133.55686    138.38545    141.58863    143.79809    142.70739    137.35551    136.10814    132.13534    133.54458    131.14888    130.47668     124.3065    129.76158    130.51579    135.31897    149.59553     168.8791    176.80182    172.54429    160.30989     145.7163    141.31197    142.29985    143.28334    141.79617    147.88038    145.78996    147.74825    141.94016     136.9011    131.20878    129.04767    127.46781    130.07525    143.00636    162.58348    178.91341    188.26992     184.3576    176.40739    163.90441    153.83754    142.35134    141.58334    139.68168    133.98343    133.42762    132.50934    132.61495    124.86419    118.66159    118.03935    120.68962    122.56087    121.11662    123.44986    115.61158    115.94417    120.36774    118.59035    114.75259    116.43904    117.96477    120.20548    115.92478    115.85907    116.81683    117.21043    123.33605    122.22039    136.10824    141.81973    141.73574    134.46725    126.58885    122.72482    118.97748    122.59986    115.58982    121.52379    123.71989    133.17633    143.25178    164.10008    172.59297    175.07533    165.60615    154.41503    143.40486    133.79087    135.66752    134.68586    133.30849    128.10104    126.54692    122.54553    124.61404    118.62583    116.31323    115.23054    118.94052    110.09685    115.55429    118.64594    116.76592    117.32787    119.59112    117.20864    118.11457    122.57367    124.91933    128.81046    122.22875    125.80724    126.16342    127.20205    123.59219    126.98992    131.32133     128.3565    133.24931    137.83324    135.85229    136.83376    132.37144    130.45551    129.71832    121.04296    125.66328    125.53869    119.88904    120.09116    115.73207    118.09934    116.77126    116.63779    122.23211    119.19388     122.6326    119.89278    125.33939    129.00574    128.33744    126.28549     132.6699    132.18562    129.17226     128.3852    124.93067    123.70795      123.651    124.55803    127.21815    127.11559    130.41014    128.12394    126.10238    126.28622    125.91451    124.13644     124.6887    120.49761     122.0401    117.98248    124.31104    118.85216    124.43879    125.67917    128.33755    126.68635     122.2096    132.41421    124.13263    128.97653     131.3405    132.89741    131.16587    133.64814    131.94294    129.39643    132.68317    134.50033    141.26778    140.00819    144.29556    136.54452    143.60344    137.99797    135.78959    133.48016    135.90726    130.27857    133.65963    132.83607    136.86722    135.23367    133.06044    129.95265    134.85918    134.14975    136.48532     137.6475     139.1931    136.43806    135.28042    138.57638    141.93629    146.09529    145.40996    145.26896    144.94704    147.81385    150.28668    144.68901    141.44849     143.4946     135.9734    135.70584    138.80014     136.9207    133.87248    134.82122     137.2047    130.56569    140.78839    135.53988    140.44685    130.69763    137.83376    138.10459    134.20456    137.59024    134.23785    133.60245     130.5995    136.65319    129.00043    130.02729    135.39446    135.37269    136.41219    132.73155    128.26247    126.16889    130.17988    130.54968    131.09308    127.17403    133.11226    127.86943    132.98776    127.44408    131.28616    131.53301    130.10185    128.34148    132.13729    131.38847    135.29242    133.89685    133.67483    133.20856    131.95606    130.69347    136.04392    132.05246    132.49732    128.55862    132.74215    133.56438    132.71458    131.19035    132.30537    134.27168    129.32599    133.10121    138.44733     138.8074    138.00374     138.0937    142.83135    138.13497    141.86446    145.31575    143.97875    142.91952    142.53493      143.481    149.56133    152.65708    151.10801    149.99409    144.23715    147.79394    152.96123    152.39526    151.39485    150.38588    157.45293    152.80022    151.24024    157.57188    154.80641    163.33998    158.15225    158.64744    165.90522    157.77211    158.77203    163.24666    164.06173    159.84333    163.27241    160.61563    168.66351    164.56846    166.38606    164.92322    164.45078    161.15125    168.73351    169.41113    170.77672      167.441    164.03644    169.25634    160.67404    174.32834    182.75233    169.85657    172.92128    173.91575    172.87885    181.75113    176.11909     184.5024    180.68784    177.36689     182.2302    168.40924    182.47918    186.04119    188.95867    187.13355    184.09482    185.60484    184.17201     200.1023    196.68269    193.59128    190.34915    193.45878    190.32191    196.30318    192.39021    190.35059    195.09955    200.75187    205.19142    207.40304     214.6113    200.79405    210.38001    208.08548    208.07416    208.28432    212.66257    207.16397    211.53498    211.96955    214.59169    204.59861    219.63982    218.84087    214.65321    213.06967    218.84868    220.96501    230.74342    227.76864    226.02292     223.3073     220.6418    219.43646    228.02038     244.2047    229.77164    231.02638    234.64403    235.70576    236.40292    248.67859    243.23642    254.08819    247.65451    249.54704    236.25692    247.97551    260.98818    262.96302    257.37125    257.13809    256.55614    250.79846    280.56386    300.41629    296.54015    275.86309    291.41074    277.48886    335.60623    337.04282    343.65957    302.89512    319.36247    322.12644    310.92313    361.75637    356.78478     340.7336    371.04256    391.13315     369.3506    477.03107    458.81264     519.1042    424.99102    429.63585    471.31696    543.95695    597.23942    579.34962    581.37921    598.38444    545.15444    541.83886          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        6.00    266.95905    244.76923    229.62409    224.38839    232.66994    216.50714    213.98779    201.42412    197.26842    199.48163    192.15684    193.48466    188.17469    184.35489    180.80565    173.42955    174.99606    168.41871    178.13172    178.51657    164.97778    163.92089    171.71407    165.41165    174.18257    196.35449     235.6432    249.94922    240.70837    199.64381    164.06796    158.60751     159.8827    148.32405    152.62802    149.71024    151.54585    143.39243    145.35393    142.68707    145.72441    140.29931    141.09385    142.54932    143.11821    143.84772    133.80858    145.43007    135.90209    138.25201      139.451    137.54426    135.13185    135.05413    135.48597    136.07428    138.11843    141.05076    145.25451    139.30892     139.0504    134.95961     133.7621     133.2618    131.98024    128.21848    128.93107    131.19488    132.69895    138.56429    148.10709    172.45417    177.06458    175.01783    153.81741    147.02448    143.97646    149.52082    145.56012    150.74338    149.90242    152.18712    148.19556     146.2475    139.47622    130.65291    128.66389    130.11093    129.79543    141.84124     162.5156    177.69753     186.2309    185.30591    175.09686    164.48386    153.63053    142.70336    135.87035    133.76682    132.27846    135.16079    131.67319    134.68825    126.46771    119.99538    119.77081    118.76566    122.24513    119.23982    120.26671    120.90728    123.34952    119.90697     114.5974    118.99157    117.00741    111.71363    119.99486    118.65621     115.4253    115.96972    118.05887     123.9042    127.08053    137.33939    142.49792    140.14898    137.22244    130.72161    122.84953    120.21987    116.76485    118.73149    123.89329    122.91651    130.35296    145.13123    163.76927    171.24233    170.23925    167.08855      156.387    143.53785    136.32776    130.16956     129.3803    128.39256    129.63102    127.15937    125.96553    126.35254    117.71326    118.48266    114.81926    116.16246    114.91587    114.17885    115.67863    117.06321    118.16528       118.03    118.89752     125.4426    124.69343    125.41732    123.95161    126.29427    128.29745    122.46111    125.48285    125.55966    127.08495     130.7715    133.12684    137.66079    138.48989    138.79342    138.18779    133.98177    128.12251    129.78119    124.49545    123.89285    122.63968    121.75923    120.55754    119.26101    120.44612    119.09227    118.89911    121.08353    121.25414    122.40114    123.11186    125.68594     123.7019    130.03469    128.95383     130.0129    128.34048    127.15022    127.43328    127.47185    125.98316    127.25241    128.28701    126.01098    130.97562    130.85047    129.40097    126.76931    125.46072    124.27202    118.75057    123.21998     122.2209    123.61284    122.37337    123.79495    128.00941    125.23287    125.13696    128.37234      127.131    128.17603    129.02442    130.51607    130.71104    134.02412    132.00904    132.02011    130.19058    133.15876    134.72749    133.96717    135.37506    137.40388    139.08116    141.54356    137.15204    139.03904    137.76965    138.03287    134.00766    131.98984     131.1539    130.96779     132.3877    131.63857    133.20934    134.27713    129.73362    134.21283    135.96057    136.57409    134.98902    135.02478    135.59097    132.87783    138.83984     142.7433    143.85305    147.72297    146.02419     143.3749    143.75928    150.82121    141.38342    144.88307    142.74233    141.52482    133.14607    138.04662    134.57152    130.31835    133.88324    134.66918    135.00043    133.85067    133.21756    137.49984    139.86481    134.98822    134.95139    135.35425    134.56406      138.107    137.87226    133.35838    131.86211    130.46938    130.88837    132.47882    135.47882    129.05882    134.34616    132.23389    126.39481    131.22235     128.3461    129.18928    132.08777    132.06036    126.00915    131.86584    131.60905    130.31569    129.64303    129.04588    130.49875    132.86724    131.04438    132.72923    132.14072    136.28927    131.25256    129.88318     131.4099    133.57925    131.96434    132.81398    130.70733    131.69207    132.06733    132.49599    134.33605    136.33035    133.18902    133.00802    137.95256     139.0421    135.61376    137.79446    142.05501     142.5753    139.14064    142.94226    143.90852    144.29817    145.26487    147.16326    144.28878    146.26732     150.2574    151.06543    152.11348    150.15901    149.80341     153.7685     154.8237    153.36899    155.94659    155.27781    156.45732    151.50096      159.651    157.72797    159.94071    159.85857    159.67001    155.59771    160.52856    158.63399    162.13681    158.79426     162.3295    160.81911    155.84627    164.68468       166.04    164.31939    163.11514    160.50539    165.27823    166.81412    170.91011    171.15707    167.73166    170.92773     170.4517    170.28094     171.5045    168.49672    170.21606    171.13506     174.0686    170.12399    177.98146    177.70958    186.13742    186.10224    183.19254      180.669    179.65719    184.69496    177.53516    182.17234    187.67222    183.14029    190.17887    189.68133    185.98008    199.18291    188.50817    185.65755    191.83918     190.5431    202.64393    201.95083    194.59441    203.41109    204.53202    197.19123    208.38851    205.95203    216.57407    202.66157    199.73004    212.18458    200.08712    214.70072    218.37568    204.01734    208.82041    206.58047    212.93155    221.91738    216.62554    212.07631    218.68412    210.32297    215.02143    211.20162    231.34341    231.11354    235.29282     226.5302    223.72284    220.03013    239.69059    237.08739      238.996    239.86527    233.77857    228.42865     250.4979     254.0647    244.22377    245.27889     238.1994    241.27203    252.75016    254.81566    254.53547    274.06838    245.11485    265.23131     259.4163    283.00485    280.20643      285.241    272.06629    307.22581    309.61969    334.94067      302.751    330.22159    322.90233    337.47452     349.8448    277.77209    379.46269    401.21504    402.47729    413.91929     352.6565    389.11299    460.20619    425.32276    532.03234    446.36595    504.37998    455.03935    575.97208    537.44237    600.99182    590.67356    626.29096    638.33098    608.16037          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        6.50    269.36925    242.71497    228.02195    228.11304     224.4603    219.54412    210.77366    202.04661    202.27277    201.65961    190.05733     192.5684    187.47173    185.82925    182.42302     175.1352    173.63655    172.33478     175.7807    177.39149    163.62128    165.28129    169.78381    170.62833    175.07857    195.16698    234.83587    252.33516    238.74353    199.28565    165.34914    158.24188      158.427    148.27621    150.67115    149.09983    150.80099    146.43552    143.12793    142.82058    146.82211    143.90733    140.01945    141.57218    145.14359    141.88801    139.56842    140.96014    138.31209    143.49749    141.49482    137.39783    134.30747    133.86824    132.49537    135.44047    138.83835    142.78343    146.23351    143.90516    138.65183    137.52342    134.50879    134.30516    131.31196    130.31767    127.59808     129.3816    134.26541    139.76515    147.45912    171.68852    177.72152    173.25055     158.3206    148.01793    145.57751    147.24431    146.88772     149.4109    145.78197    146.58569    148.04396    144.07903    139.92465    130.35545    126.68106    126.08677    133.31412    143.91939    162.53477    177.13188    185.86754    185.10861    177.10929    162.14372    156.35235     144.5975    139.22311    134.01356    137.15283    131.23143    138.75784    129.08288    124.83327    121.32072    119.83691    122.44863     122.1133    123.28728    120.66991    119.09158    119.29735    117.82341    116.29534    117.99798    115.78257    115.86463    118.35675    118.48448    116.60735     112.6775    116.74403    124.06356    128.47855    137.24511    141.34136    141.03144    135.44613    129.07741    119.46746    118.66176    122.26735    119.32043    121.60275    122.27358    131.64308    144.26332    161.02635    171.17137    175.31262     169.0831    159.74909     143.6346    137.37031    132.62636    130.08909    129.25976    129.88112    127.03576    124.55528    123.57883    120.05085     117.5499    117.37458    115.93279    117.91815    119.88802     117.1681    116.91918     118.4615    117.26078     119.7353    117.57508    124.72537    125.00952    125.75678    126.23337    125.13813    120.76171    128.60012    127.32134    130.19347    127.58597    135.02847    134.90425    136.39952    134.95751     136.0223    131.36127    129.22606    129.12149    124.58785    125.61384    124.32062    120.25872    121.70191     121.0791    119.06375    119.49813    121.39251    119.34157     119.9738    117.81176    122.79342    120.80794    125.43594    126.18205    128.95384    131.34655    130.03236    127.92076    128.28329    126.77078    123.07029    123.02347    127.53637    124.76145    126.73373    129.58812    129.34546    128.65644    127.95716    124.87136    122.79171    124.87373    123.99966    122.48554    121.84363    123.37475    123.65801    126.19901    127.56605    125.75077    125.26971    130.78096    130.14926    128.48896    132.47044    130.57513    130.28561    131.65843    132.92057    129.42971    133.26242    137.23903    135.77845    140.14595    140.48957    141.99882     137.6098    137.97606    138.95556     131.7003    134.95705    133.61843    131.47134    130.44858    132.54385    128.55158    128.52761    132.19204    131.71021    133.51198    133.30118    134.47127    134.22273    133.01795    136.12878    133.10519    138.85547     137.8038    142.77099     146.9379    149.97652    148.39434    145.23921    148.66463    147.54749    144.35817    139.72768    138.02452    136.66253    134.88077    133.02432    135.64746    134.90012    133.13748    132.97252    133.47897    136.04445    135.69312    134.85883    139.26783    137.07577    132.28437    137.50669    135.58479    133.76561    131.47302    133.11582    126.56692    131.77616     131.2747    130.14658    129.72996    130.38518    130.96979     128.8551    129.26144    129.97215    130.04632    130.15036    130.37769    127.72063    128.59408    131.45699    130.36094    133.04468    137.57482    127.55358     130.3205    129.66547    133.72443    130.48104    131.21259    136.07531    132.79259    134.45313    135.35899    137.09283    131.22962    130.76745     131.5065    131.30631    135.05801    131.28444    132.22218    132.44862    134.66494    132.40637    138.09169    135.16323    135.74272    137.60214    136.14429    141.25826    147.30063    142.92586    143.34633    146.73416    139.68084    145.24655    141.74743    148.34333    147.93855     144.0414    146.27181    151.39395    151.81387    162.12687    158.76349    156.87273    153.12195    150.33794      153.354    155.77186    156.64762    160.42643    156.95724    158.13657    158.98632    155.03188    159.69231    162.14577    162.18653    163.58474    166.67071    158.57206    159.75328    163.28764    164.40717     163.9722    161.01952    167.85878    166.94923    173.90393    174.00863    172.60838    168.91803    165.61448    173.90285    174.28683    177.15433    170.01527    171.94731    170.12979     174.0257    172.52372    176.14694    177.65138    182.12791    183.59013    178.07776     176.6956    177.72426     183.8496     177.9173    182.36073    178.42797    185.64427     186.7873    183.34669    189.87362    187.24884     193.9448    186.20694    195.18474    193.94388    198.32246    204.97433    197.26336    203.22659    193.50723    202.47843    199.03912    202.20621    211.91211     200.8245     205.8708     202.8096     215.2007    217.97533    211.19256    212.94014    215.51907    215.76112    218.04919    222.29576    217.65719    214.55024    216.47391    212.33337    230.34201    220.41536    222.49017    224.37903    231.90296    231.42184    223.91637    233.02132    238.51357    240.84347    230.78976    234.07415    242.74764    253.47005     245.9094    246.35971    236.79746     246.6905    241.29421    259.79316    265.10722    265.31294    255.18399    263.75347    266.98346    260.76832    274.42727    281.50016    287.19299    293.33832    308.52663    305.63531     318.0923    319.83084    305.36038     331.6946    330.99733    324.21316    318.94766     362.9389    362.11896    392.60566    361.17331    344.20655    386.73562    474.85933    486.67155    443.98117     450.8128    444.28376    472.90241    575.63154    692.02074    531.14063     469.6409     586.3883    273.40825          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        7.00    271.05103    243.39769    231.29284     232.4429    217.32838    223.27057    211.53535    206.10915    206.83498    202.44347    194.00667    192.00373    185.90328     185.4276    180.22491     175.7834    174.82384    176.62066    174.14583    171.94692     166.6175     165.1509    169.34862    168.06133    169.78238    197.44349    232.78668    250.28423    234.30662    193.82673    166.84577    160.37821    151.96789    152.81107    149.95451    148.44261    150.16222     152.0273    148.02289    143.87326    144.35241    140.82857     142.3935    141.74557    143.87978    144.15817     138.7423    139.47235    135.94654    139.66657    137.69901    140.97944    138.96569    136.14482    134.03711    137.87974     140.9912    145.43443    148.28324    142.25306    138.79753    135.66889    133.33778    132.61341    135.13602     133.8564    130.08781    132.41249    131.56034    136.98289    155.22714    175.07084    180.52441    178.08305    158.79235    144.20656    140.46808    143.27301    147.83916    145.17844    147.94398    148.08376    149.45951    149.55528    145.68808    131.69304    128.82512    128.23266    136.11168    145.06579     163.8508    179.50241    189.92564    181.16603    169.86787     157.9824    149.95851     140.7812    133.93045    132.05592    134.86291    128.53073    134.39188    128.92885    124.92858    122.34909    121.35738    120.33831     122.4938    122.03631    120.01046    117.23566    120.64096    118.11471    117.27625    119.02709    118.50519    115.23984     117.9078    120.21719    117.70079     119.0732    116.62443    123.48013    126.28178    138.42714    140.59694    141.03369    135.69192    128.62023    123.93529    120.36056    117.81697    120.01385    121.90724     122.7147     130.6889    143.47192    160.16109    171.36127    175.55785     167.4933     158.5097     144.4946    136.02795    132.07448    127.87615    133.15859      131.508    129.63504    123.83285     123.1371    118.51969    116.05278     116.6155    119.80175    113.04201    113.61344    118.26839    119.34648    117.41754    117.17715    118.47266    117.84332    122.31865     125.1322    126.71263    123.75073    126.72874    124.35148    124.37433    123.92298    123.85707    124.96738    132.66399    130.28471    130.95648     133.4119    131.71913    134.53937     131.5713    128.46328    125.20134    127.01338    122.32451     119.3007    123.42814    122.31396    122.13638    115.50864    121.84651    123.10499    120.14669    120.37756    119.90389    119.83571    123.83618    127.99282      128.941     128.7918    131.78962    126.12052    127.41721    126.69894    128.94355    123.88324    122.83551    126.04125    126.36124    128.34677     126.6259    128.68104    128.45328    124.06686    122.77926    126.57465    126.61895    125.36662    125.80901    119.38947    122.21933     121.0568    126.82797    126.16871    129.32665    130.06647    129.11674    128.88573    133.59077    129.14042     128.6005    134.61383     133.9231    133.28925    137.22971     135.8381    136.25042    138.79694     138.0321    139.48466    135.91949    141.88527    140.44603    138.21477     136.9134    139.40083    136.01531    134.14159    133.79815    134.23066    133.13027    132.22983    130.27611    132.33349    134.60031    135.71124    136.19355    132.95465     135.4726    135.26569     137.1455    146.26641      144.297    150.35341    149.12519    145.67144    149.69497    147.22252    144.94013    146.20092    144.56863    139.09221     134.7513    140.72377    137.57971     133.4295     130.8234    134.34152    134.13807    137.06224    134.82476    137.86749     135.1999      137.651    137.28552    132.58335    132.29086     133.3796    134.16933    133.44759    131.07755    130.03965    132.67999    133.35253    132.85545    135.11701    132.44176    129.93172     128.9082    130.51827     131.4774    129.91007    130.52867    132.75727    127.90743    128.74155    133.85176    131.40594     134.0617    131.25191    129.57012    131.19134    134.19121    134.99024    132.69673    136.72957    135.35503     127.9183    135.86563    131.44749    132.13777    132.23533    128.36416    131.66984    130.43354    135.42187     128.8175    133.63808    134.54334    129.16892    133.32212    140.24144    133.40552    139.88286      134.215    143.96945     136.4376    142.95815    149.24252    144.65992    144.84999    143.57409    142.58224    144.77002    143.75672      146.534    148.35714    149.07899    141.40719    148.74677    153.79818    151.81453    149.64838    156.03909    157.78632    151.98071    155.20928    160.54133    155.89829    160.70643    159.93534    160.29756    157.52988    163.36364    159.70357    159.55616    162.69668    163.60462     161.5489    159.98731    160.71198    165.94639    168.74651    168.72284    164.02306    167.97933    167.43643     169.2171    169.09431    173.54551    168.74535    170.02006    174.92335    175.85903     176.1846    171.76733    168.63569    169.82168    175.89752    177.41941    171.68979    176.62106    173.74431    177.53937    175.11179    185.80084    189.15926     182.2567    183.34533     182.9135    186.03865    195.40056    185.52488    193.13669    188.61264     194.7082    189.52736    193.34238    194.18097    203.04851    199.54226    196.32832    192.61243    194.29005    207.35393     209.0891    205.70826    208.50024    200.31964    208.57581    207.21415    211.75435    205.86366    212.56695    211.39879    209.55633    219.76835    218.38416    219.19895    216.43591    221.83667     222.4369    216.82039    220.60072    237.96684    226.01399    217.14353    217.41399    221.39013    237.15549    245.87125    240.35072    233.07442    237.34375    233.84977     228.5733    242.95809     248.4452    244.23724    235.74667    251.26011    256.58149    252.06166    263.20214    260.60977    270.19823    266.61252    271.51912    251.78428    267.61882    302.60623     276.0081    294.44545    280.96844    319.72232    319.48356    324.39782    317.36446    308.66975    340.52084    360.72027     315.1645    352.54193    344.48343    384.83757    377.27194    376.26782    384.34543    504.60494    537.55835    451.09425    435.19279    482.14181    418.97802    553.00803    570.90693    393.28404    552.73512    585.13726    534.93692          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        7.50     269.9347    244.68867    234.21271    234.65441    218.68717    227.15562    212.59496    207.86269    205.83538    202.22223    197.69992    193.36284    189.82987    182.65596     177.1641    177.82405    176.99039    176.86163    172.89548    167.93265    167.05936    162.37679    169.70745    165.99937    172.20539    192.95553     234.0071    250.57308    238.36553    194.57497     167.7362    159.85577    152.99664    153.08065    152.34199    147.11024    152.85135    152.81418    149.68763    146.58944     141.3273    144.21261    144.29327     141.5408    140.32454    144.74685    142.93902    138.98297    139.50229     143.3065    138.93914    141.92917    138.82442    131.27534    136.30486    132.86888    140.86566    143.59674    149.60792    143.51052    134.91867    133.47987    131.09696    133.44825    134.76938    134.00502    128.35322    131.35339    128.98276    137.34566    149.55631    171.96691    176.22199    175.08396    157.07345    146.91171     146.1825    142.45555    143.86391    142.86784     144.4078    149.60495    152.63836    153.74695    145.28274    133.53312    127.52249    129.06814    131.72656    144.70299    160.24809    181.20553    188.49151    183.00638    168.23441    152.61061    145.03519    142.37547    133.65002    135.79136    133.03394     134.3593    133.46774    129.67871    125.19871    119.46176    121.49878    120.49061    120.18036    121.45803    122.05669    118.39878    118.56496    118.49767    118.85911    118.66627    117.21954    120.27869    118.35481    116.65366    118.44184    116.90164    116.01771    123.49769     126.0108    137.43106    140.23242    140.88023    134.94411    128.79472    124.08501    121.18904    116.27352    121.32334    120.88693    123.04403    134.53438    143.86665    162.64985     173.8087    173.40185    171.37106    156.70903    146.33328    136.42316    132.77834    128.02421     130.3781    129.57034    129.32179    123.51996    121.53633    122.81994    114.27884    116.72232    120.56045    116.88584    122.37766    121.95961    115.52986    117.58782    115.73168    116.32877    116.14067     124.2055    124.26075    127.58755    123.85446    123.27649    123.45319    123.79379    122.42653    119.82598    125.69627    128.68685    129.22283    128.50501    130.64801    134.47209    133.71631    132.00618    131.47956    125.30895    126.11402    126.88981    125.98995    125.69661     122.0118    122.97783    121.60636    121.25693     119.4972     121.6682    123.47935     119.2138    120.00805    129.37128     130.0954    129.26121    129.90614    134.26249    131.10375    127.60175    127.96311    128.18059    127.46957     124.7133    127.02227    125.09989    128.77936    130.58912    128.49467    126.12993    130.88134    124.77983    125.48668    126.02322     123.1706     122.6663    123.29662    123.28496    121.37961    124.60159    124.20599    127.50217    126.86797     130.6727    129.24357    130.63967    132.20021    136.56238    131.22689    134.82432    130.34985    133.29727    132.28792    138.04059    136.32522    138.77727    139.63834    138.87173    139.22376    135.08794    140.05979    141.15893     139.3948    134.41009    133.63385    131.58417    134.13076    135.42163    133.76437    133.55219    133.40592    133.62642    136.83813    134.18264    135.04912    136.78243    136.57253    137.94795    144.59831    139.90792    142.79952    146.54844     149.0959    149.12931    147.89435    142.54481    146.38319    140.40966    139.25937    139.75527    139.84691    134.73027    131.92039    133.28837    136.63703    132.45327    131.80099    134.46289    136.10834    137.77398    136.94028    138.11415    132.46453    135.86463    137.78693    134.81389    133.87152    135.66803    130.12575    136.11162    131.65876    131.57772     132.5662    132.82551    132.31221    128.52247    132.92466    131.34846     130.3088    131.84013     129.7723     129.8955    130.51361    129.71167    129.96228    131.22428    133.83064    132.41532    129.83569    135.44275     132.9838    132.35477    134.66118     132.7232    131.96796    133.88529    132.94543    132.46856    130.53773    133.90802    132.98157    132.19857    131.53772    132.46673    133.91867     130.0831    133.48221     135.6392    137.57849    135.35411    139.68724     138.7767    139.83946    140.67537    146.53312    144.17725    143.31954    145.12473    147.89294    142.88623    142.77367    144.93625    145.43294    146.06144    146.71554    142.91341    148.62423    149.79809    157.86715    150.71064    159.81749    157.67163    153.85488     151.9064    159.58942    161.76783    162.04146    156.68659    158.88363     161.0166    161.90023    160.56509     165.4513    157.70287    159.80224    155.12217    165.55817    169.88912    164.63969    168.80339     163.3889    161.67839    160.82448    167.30243    167.38975    171.98346    168.45596    163.62701    163.32153    168.97945     172.2093    165.75247    175.56756    173.12957     170.6606    181.01879    180.59445    177.05773    172.10497    177.95473    181.97108    176.28515     180.9542     185.4314       181.21    188.03879     183.6396    186.95141    189.27469    189.33482    186.69385    197.56506    191.20562    196.83692    194.56224    197.64708    199.06314    196.19783    195.80947    196.28728     197.3802    200.79823    210.63788    207.26764    212.39853    214.39135    211.52497    212.22163    213.70041    205.54245    216.97741    215.73112    211.52449    211.07086    222.51577     226.0544     233.7836    215.52354    221.56261    222.44887    223.86404    224.71238    232.82757    229.26373    222.21465    229.34699    232.82521    241.02004    242.29735    239.93711    234.36907    231.87866    248.86624    253.70803    249.80674    249.41718    251.96311     245.4319    256.09721    262.33675    264.24669    273.21762    262.65015    253.41692    261.53173     257.5119     293.7405     285.5061    273.44095    313.81897    298.49718    297.64748    329.28562    327.50202    298.28679    328.55521    317.76513    293.14698    328.81054     346.0197    360.48931     397.8424    384.74658     337.4083    385.89156    422.21626     478.0119    450.63333    505.77365    473.58775    434.83642     634.3865     591.7105    530.53107    511.27931    636.00958    566.75508          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        8.00    269.06485    245.75085    234.74784    233.84553    219.19807    227.22729    214.03378    207.04761    204.59297    200.62155    192.90252    196.38493     193.4377    182.75971     182.5261    180.95399    178.43413    176.18338    171.06157    165.48111    166.18828    166.43973    167.58274    166.44306    173.10095    194.84726    241.77686    253.60361      238.077    196.29187    171.95229      156.244    154.83623    154.99333    151.40016    146.69386    153.20818    150.97199    148.65499    148.60444    141.29751    151.98477    144.14824    143.18226    138.92209    142.25071    138.35941    136.99432    141.95426    138.59946    141.31456    140.85463    137.82874    132.28007    137.40953    139.62836    138.57079    141.37099    143.15439    140.60147    138.98229    134.22223    134.42897    133.87129    132.28663    131.51435    129.96866    129.96448    131.81413    140.52398    153.30581    173.45825    178.53709    174.04294    159.51542    146.40204    144.11687    142.81695    145.98787    143.84963    145.58298    150.76184    157.31272    157.63493    147.24697     131.8367    126.20993     128.6244    132.82968    143.11752    161.20657      182.381    187.58328    175.29089    165.59065    146.83225    138.91708    133.72947    130.58202    135.79792    133.79599    132.09537    131.54164     129.4504    121.39324     122.1278    120.09514    117.55951    124.35428    119.00504    119.74144    116.74262    119.97541    117.86006    118.96598    120.97564    116.25212    122.96276    116.48099    117.86163    116.25149    120.22042    119.66888     124.5469    126.54587     138.7362    140.94723    136.78212    135.12886    129.85131    126.30039    123.35862    123.12208    120.74387    121.14891    124.13733    133.17916     145.2277    164.55107    170.56816    171.52973    170.83051    156.75769    144.12571    135.71086     133.7657    134.67313    134.15863    129.88856    129.82038    124.79264    123.92868     119.2357    118.81109    117.00433    116.44446    113.46519    118.12912    119.77403    116.08655    119.06045    119.52223     121.5621    115.33194    124.25133    125.36497    125.73926    126.16803    124.12341    122.56664    121.61626    124.76451    121.44501    124.98152    128.82912    128.75918    127.77187    133.51942    134.36787    133.01461    129.20913    128.10205      127.991    123.83691    124.20838    121.70368    121.93596    120.49272    121.31728    118.37843    120.81796    120.95582    118.79104    119.11194    122.17325    124.22716    125.66439    127.35412     130.2549    129.61285    133.64447     129.2636    128.80909    125.43428    127.66889    125.01571    124.28004    125.95927    125.97114    130.78756    130.29019    125.45364    127.33271    127.64929    124.15714    123.34913    124.19394    121.65154    126.29609    121.50774    125.28348    122.73961     125.8153     124.8914    126.58243    127.53721     126.3446    131.67647    129.77871    132.74794    131.70974    129.03821    134.64878    133.20322    131.16272    132.50437     138.1962    141.81828    141.95552    141.61769     138.2247    138.60023     135.5613    136.82958    138.26835    135.18639    135.98872    131.06878    132.79946    137.26299    131.73467    132.06168     132.8309    128.66218    132.17905    138.31009    135.78247    135.30891    137.59602    135.93127     137.7797    139.12673    145.44998    144.88034    145.95597    147.79124    144.10453    148.06069    149.41476    144.97448    143.31361    138.64206    136.26149    136.98688    136.55376    134.97535    135.00545    134.02689    134.78727    133.87133    136.35478    133.89082    137.59249    138.38915    136.50396    132.68048    136.06821    133.52517    135.30565    132.33204    134.43097    134.59788     132.3994    135.05027    131.65085    130.13371    136.14291    129.94777    131.94818    131.07844     129.9956    128.50199    129.71238     130.5947    125.81599    126.45868     129.7589    130.26887    133.12732     134.3132    131.19683    130.52968    132.87143    135.08193    133.16251    134.59684    132.70865    130.99593    131.17361    131.21651    134.34874    135.91375    131.04517     131.0107    132.31024    134.40343    134.27535    132.56443    131.60521    135.50322    136.61783    139.36379    136.42417    139.92166    141.77493    140.91259    140.18156    146.65103    143.15468     142.7933    142.12068    141.76377    144.03716    144.53476    145.45133    141.23063    141.87625    144.62002    143.08904    148.43877    149.74977    149.24533    147.70846    154.14291    153.03011    148.43589    152.16543    158.05576    168.20861    162.26605    164.43537    158.69768    159.40153    161.37832    161.91804    162.85217     161.6637     160.7539    159.14606    165.52063    159.80133    162.73355    160.32538    160.82243    156.73003    165.55126    167.42745    169.57752    169.56912    168.33889    168.26746    166.41579    166.65998    172.61494     169.6136    170.72619    171.28374    175.17255    174.35154    183.78984    175.68353    179.49153     182.6338    169.74387    177.14665      182.565    178.75794     184.0001    178.12245    183.52859     184.4856    192.15794    190.64368    186.13999    185.38941    195.54872    187.02805    193.81473    204.74797    204.07378    201.09486    196.17529    194.13469    196.57507    201.48019    210.89477    204.89662    212.11127     202.8639    210.33422     207.3036    212.10257    217.51149    213.15638    210.53335    213.93777    212.13212    219.29212    206.71049    222.14048    212.32981    215.49912     212.6917    223.28143    230.93648    227.97609    233.74917    216.88072    233.40835    226.07958    247.95785    241.48665    239.02697    236.79839    228.67288    237.15447    249.47797    236.94859    248.73472    236.38854     248.6189    233.88251    247.44894     245.1549    268.76009    263.36962    269.32424    265.56358    272.75003    278.39685    286.25428    289.04377    287.55063    288.77337    294.06062    322.83151    310.37255    301.26367    351.36028    288.17159    300.04095    338.96414    341.97834    386.96732    355.63114    353.50058    351.57555    376.90563    437.09115    446.23016     418.6238    427.01034    431.02268    469.40999    637.04914    541.49803    587.99295    595.23598    458.55137    569.23099          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        8.50    267.38675     247.0879    234.60482    233.00399    221.52804    220.20371    215.37981    206.09395    202.22286    197.41385       189.11    195.84324    195.56273    183.01693    184.78265    180.97223    176.22086    173.66973    169.71647    164.48053    165.73095     169.1127    167.46216    167.15511    172.39559    195.28277    237.88885    253.42509    236.20136    196.10065    169.69617    156.10378    156.00231    153.33268     148.4448    149.57517    150.77239    151.92941    144.49152    146.14707    145.17897    137.87044    142.61554    139.36458    142.22131    139.47653    138.24681    142.60282    134.43895    136.52378    138.63944    135.75141    139.68388    133.94063     133.9396    133.05863    142.54658    142.82801    145.37402    143.75445    140.76309    134.50315    132.81722    133.28612    132.02151    131.87405    128.93159    132.07491    133.06202    137.71982    152.97853    173.72116    181.87354    175.92631     158.7663    146.77339    145.30742    147.06542    145.95586    146.18936    147.39333    156.88303    156.61064    159.59957    149.04326    134.19353     128.3793    126.23246    134.00111    142.36914    164.16293    182.17154    188.14508    178.74044     160.1312    143.54022    132.62497     133.1761    130.13025    130.91589    133.51539    132.44612    132.83773    130.06567      124.994    117.58999    118.35818     123.4292    120.99926    119.97924    122.48138    116.75104    120.11596    118.58637    116.53493    116.18709     119.5558    117.97017    116.00751    118.32524    118.02517     118.2138    118.10773    122.28924    130.20692    137.01967    140.61677    141.67595    137.35099    130.48425    122.70481    118.14287    120.67421    121.65766    125.23523    127.83756    134.58809    144.39287    158.81693     170.7351    173.01103    165.93774    155.26262    143.98413    135.60071    134.41354     133.0438    131.82664    130.69085    127.40051    124.08397     123.5902    117.71435    117.51055    119.65375    117.11052    116.92597    116.59261    117.28065    116.66756    118.83771    118.53098    119.18953    117.92791    124.79121    125.04389    125.94102    123.74404    125.04226    121.45629    121.83528    122.74003    127.19524    127.25393    121.71287    130.10508    126.16976    131.23494    129.77895    135.41427    132.36293    128.94671    129.12508     126.5342    123.88372    122.05343    123.78494    122.03417    120.08181    121.10483    120.19963     122.7192    119.24998    124.92352    122.25738    122.72559    126.67487    129.05188    130.55245    130.58338    134.85237    129.96246    126.89428    126.72076     126.5042    125.94427    124.35837    128.33221    130.02785    128.62983       126.01    126.10034    127.23842    127.31584    125.80696    121.90547    124.40366    118.49726    119.46408    120.47571    123.04946    121.28472    122.20124    123.83783    126.22851     129.6184     128.6069    129.56709    127.84049    130.71568    133.86707     134.8257    136.63947     135.7676    132.50183    137.96872    139.74977    141.00846    142.97805    140.82229     139.7355    135.97972    137.36902     136.7008    136.69981    136.18557    132.87772    130.77023    132.98463    134.60767    133.08759    132.15081    136.18443    131.43325    133.62498    138.11139    137.68012    134.73066    138.94439    137.37371    141.53474    144.67599    148.13609    145.03913    150.42542    146.68551    144.60222    150.10948    148.05473     145.2486    141.94369    141.47459    136.72798    136.07286     134.9114      133.236    132.85186    133.76329    134.34046    134.89149    135.10108    137.45745    135.10074    137.71114    137.87214    132.21512     138.4255    134.43056    134.81989     132.8552    133.24962    130.41435    130.72383     131.2662    130.15192    130.48515    128.18909    132.39503    128.21378    127.66794    129.53034    126.54627     128.8262    131.26963    132.54699    127.28169    128.26461    126.56631    132.72927    133.45589    133.38373    128.13486    137.02649    136.73364    131.51605    137.50279     132.2249    130.58103    133.67209    131.63575    131.35629     129.7929     128.5372    126.30524    129.00844    130.18309    129.28023    132.70085    131.90137    134.78754    136.67969    137.40913    136.68607    140.85597    141.68493    138.78752    138.30626    139.53216    145.52274    143.57416    143.56547    147.81342    149.85572    149.77436    143.39567    146.46202    146.36205    146.32582     142.0129    147.74324    149.22599    151.83897    148.52777    151.23337    154.95968    152.63217    155.29967    157.97552     162.1205     164.0576    160.00648    160.42452    157.55602    161.18939    158.98294    160.39194    162.31471     162.4079    160.99721    161.14854    169.81287    167.14702    165.12563    160.40395    159.57735     151.7193    162.41938    167.39522    166.68244    167.37009    169.38821    166.25257     167.6417    170.78706    176.44729    175.77716    170.08679    172.32733    172.13465    180.19325    169.19083    177.04005    178.79899    178.95522    185.09647    188.11524    182.51334    181.38098    180.60417    183.53754    177.59575    190.19755    186.88616    186.57115    187.32913    191.16935    182.63276    192.85877    197.81221     197.5759    201.02013    192.60097     197.7233    194.32884    205.03608    197.80314    201.04587    200.16557    201.55384    205.48553    212.48934    214.94901    223.74705    221.43185    215.08494    212.40044    208.32062    213.05471    220.28018    210.89882    227.61228    210.16092    217.04269    221.81656    232.15173    218.11072    219.39022    227.77924    221.77901    231.25798     235.2747       235.12    234.34425    231.98154    232.07314    231.29757    244.45908    251.59132    245.26141    237.29123    247.66119    240.45841    269.61864    258.13895    262.51961     275.5505    264.62543    260.20945     262.4029    289.67778    283.04693    288.54172    314.51865    311.39443    284.48598    342.61246     346.1782    335.24838    333.98821     341.0843    341.78546    340.01092    387.76608    398.67915    391.44891    357.82586    377.40711    362.37447    480.30343    478.60908    407.96584    425.29961    455.97114    462.11195    553.58714    537.89925    506.94811    591.51381    626.47053    653.07164          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        9.00    262.76197    248.21339    234.32088    230.40318    225.00161    214.52576    213.61589    205.70874    200.53785    194.30603     191.7022    192.84163     194.8355    180.65068    183.22321    180.01636    175.48779    169.95362    169.27765    168.78974    164.51998    166.86605    166.64595    165.02757    167.73867    197.19915    238.02687    252.84894    236.20484    195.83683    164.58206     153.5075    155.04959    151.32293    152.23402    148.66718    149.48397    152.03601    147.53358     147.2229    145.77463    138.37015    142.52349    142.86106    140.96288    143.58513    142.19306    141.68964    136.80325    139.27628    137.33491    137.19414    137.46562    134.20115      140.266     137.7206    140.45561    143.82107     146.6235    144.73718    135.70206    136.85084    135.99802    133.72151    135.02804    133.02113    132.18235    130.78322      133.499    141.16057    152.14313     170.7814    180.26513    173.16887    162.07791    149.82183    144.75437    142.91888    151.48842    153.32802    152.11642     161.4267    159.93392    157.75641    146.46904    134.05045    125.16242    128.40795    134.47942    144.04875    163.01805    180.82393    184.91868    174.09228    153.74863    142.81559    134.06278    128.20029    129.06934    132.08455      128.992    131.91241    132.40627    128.28977    122.31898     122.8831    120.32963    119.30887    117.90857    118.69367    120.04167    116.21543    116.88487    118.64495     116.5582    117.42931    119.09243    119.41381    115.17939    116.92828    119.03034    117.95766    121.69984    122.49927    130.51804    133.11747    141.83188    140.23862    135.61899    131.52108    123.76409    120.63133    121.83195    119.66244    121.86805    124.65101     131.0901    143.11258    161.03326    170.76114    173.03053     168.5166    157.65845     146.9592    135.05097    130.18635    133.48099    134.36157     132.2474    126.73019    120.44754    126.30835    119.51494    120.68036    119.52604    119.95973     119.3774    118.76064    118.41661     123.1828     121.4468    117.57127    119.13785    117.26084    123.86069    125.40708    124.49001    131.39313    128.87203    125.71339    120.67415    123.96927    125.24063    127.42734    127.69614    131.85707    132.96137    132.17176    135.65031    127.94456    130.52066      130.296    128.70936    123.34384     120.4824    123.76236    124.98232    125.44105    120.48014      119.794    121.20975    121.15354    121.88289    120.02654    124.25867    125.68036    128.89837    127.01806    131.07729    130.18755    135.45426    131.33725     128.1377    126.58074    129.73559    124.27641    127.18265    127.95413    127.71687    128.46558    130.18683    130.41204    127.15221    129.66877    124.84431     121.5659    121.44766    124.58275    124.91412    123.69229    122.92759    121.03795    122.91835    126.61437    125.69834    127.80794    126.56512    131.00414     128.4939    134.40265    131.40411    133.00499     130.9891    132.07706    132.03923    136.40108    135.84113    136.48987    138.13147    138.59402    140.36745    139.99161    138.82641    135.19936    133.86498    133.73729    134.47239     131.3443    137.00239    130.83108    134.65062    136.59236    138.27351    132.42921    133.06223    135.81057    138.60843    137.85238    137.17194    137.96432    133.80837    144.27083    145.43173    146.86559    148.47669    147.67712    149.15232    145.79946    146.05859     143.9433    141.79009    139.29995    137.01478    135.32269    134.36718    132.36923     133.5062    133.55748    138.50744    134.79511    137.69973    136.93168    138.45987    132.79571    138.06652    134.84107    132.69943     132.4536    136.19553    131.61571    132.08897    133.67487    131.46372     131.5168    129.86736    132.13991     131.3657    131.03479    128.08086    127.87576    131.49025    131.46884    129.67074    127.32796    126.58793     130.3377    130.79819    128.64411    129.75469    131.56788    132.32601     128.9054    130.93575    133.20733      129.185    131.42265     132.5181    132.75194    130.61396    130.99755    133.43482    128.69515    130.51124    128.61408     130.9951    131.74444    130.39787    131.43417    133.68561    134.32411    130.68614    136.46674    140.08624    137.62951    141.66194    142.56336    138.71663    138.91325    143.41958    141.93491    145.22891    147.27233    145.03927    146.08293     152.5509    147.30189    154.26829    150.45863    149.47241    148.55796    151.95796    153.74548    151.57093    155.21402    147.84272    152.48402    153.03807    154.96003    157.33956    153.19792    162.67024     159.4732    158.46047    159.75514    162.02586    166.05211    159.22459    161.47167    160.91936    157.31863    158.58005    158.69427    161.78548    160.18804    161.06326    154.00057    168.34899    171.43676    167.40969    166.80353    167.89574    166.70395    173.44001    172.10853     170.5785    174.66486    176.15297    174.54461    178.96597    181.83235    175.63956    170.25905    176.78162    177.15197     174.3721    183.77772      178.636    186.32313    179.15905    183.22128     187.2245    190.89341    183.64545    183.31331    180.35831     190.2835    186.27843    184.79056     197.3177    194.88418    203.07923     198.0971    201.15011    190.92305    201.66372    204.19578    201.73207     210.8806    214.66256    201.88068    209.35961    213.13153     213.8905    217.45982    212.44383    206.30299    211.64742     215.5978    215.82945    204.55053    205.76064    219.62926    203.12964    224.10337    219.36522    224.42514    228.05317     228.1583    219.24704    222.31668    228.07919    239.16911    224.11322    236.22372    237.63182    237.04102    249.49582    243.50681     243.5019    248.15712    242.98977     247.6069    256.15133    259.85227    258.78959    254.25405    258.53058    270.54723     266.2116    274.05609    289.36209    277.82794    301.98936    295.73754    278.64555    286.63326    331.78664    338.32267    334.48268    319.33775    321.15931    323.26407    390.59251    370.23435    367.54141    358.83594    360.38907    362.27345    459.41112    424.21364    453.84578    431.53752    396.28681    443.43349    480.95797    510.44613    479.31165     598.6136    655.11202    618.48935          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        9.50    258.96068    248.32507    233.39922    227.54766    227.12821    214.67859    210.47432    206.19911    200.07385    191.94631    193.45441    192.42725    189.57725    178.57433    180.21108    178.29991    176.90556    170.42623    171.46857    176.20917    166.59304    166.18876    165.01281    163.82896    171.88458    202.60079     241.5796    253.04088    237.27186    196.55876    166.79329     157.1371    155.32257    147.07519    148.40743    148.23635    150.27569    148.02855    151.15305    150.47129     148.2955     145.6215    144.37631    139.87189    143.49957    139.64693    142.23869     140.2457     137.8873    138.84888    137.41014    139.62865    140.80135    134.82848    135.93771    135.85851    137.49469    142.71728    146.71778    143.26534    139.25719    137.85614    140.69787    134.23799    134.53657    136.25127    133.07386    134.05887    136.66758      137.771    152.13347    173.45445    178.57269    176.03838    162.24838    149.73597    145.97067    150.38396    152.55596    149.54589    153.32931    163.49133    166.36659    159.16733    148.60236    135.39165    128.70863    124.58375    130.06354    146.64279    164.59475    182.95478     182.3277    173.43732    153.51244    137.77649    129.40333     127.9837    128.31672    129.90078    130.19594    132.32594    133.21866    131.35474    127.00655     118.1253    118.91524    121.67141     120.3581    118.89683    124.94214    118.42044    120.30511    119.50072    117.93695    116.41027    119.48483    117.78857    117.96464    119.22512    120.67367    117.57457    118.79324    121.20344    133.78819    135.34548    141.16258    140.11958    135.19375    130.19469    126.74187    119.23981    122.05693    121.01585    123.67454    124.15373    133.33263     148.3071    160.17173     172.1106    171.20379    165.44286    151.88503    144.60455    132.30461    130.04142    130.18463    130.67697    128.22006    127.92169    122.66421    121.35576    119.74806    119.08996    119.20168    116.52828    115.42668    114.68818    116.26057    117.98882    116.97198    120.70407    120.31474    120.14781    122.82377    123.75944    127.16603    127.40313    128.54545    125.59219    119.67723    127.52658    128.00238    128.67265    129.66121    132.09065    133.35394    132.03294    133.60438    132.42056    130.97012    131.73016    128.07131    120.67843    126.12866     122.4157    121.23152    121.69518    121.99218    120.82726     120.6293    119.53918      121.964    121.96328    122.91652    124.57573    123.70498    128.61142    129.66511    134.90857    132.96913    131.56226    126.93735    126.60027    125.16604    126.52542     127.6166     127.2539    128.73096    128.57162    126.05482    127.44922     126.3167    124.37048    124.55587    121.00671    122.75397    123.78719    122.44591    121.55651    122.78987    122.03624    123.10642    123.11021    126.31973    127.23307    129.80002    130.18662    129.56225     133.1566    132.92251    133.13331    130.93063    133.82542    134.37918    135.34161    136.24459    141.56925    142.84025    140.88879    142.11594    138.70744    140.70863    137.59943    137.96808    133.63218    136.14597    132.31271    132.85974    137.07751    135.53949     137.2985    135.10813    134.60978     136.9133    137.34862    137.90865    134.51728    138.64348    139.97587    136.49702    143.34286    145.79372    146.36323    145.51591    147.35532    147.28851    145.49014    147.89547    143.82616    147.21768    139.11051    139.02498    136.39632    134.21038    137.55882    133.22194    131.03602    136.92444    132.45586    136.23201    133.73684    136.83787    137.34287      138.444    137.04872    131.23714     130.8973    132.66547    138.08095    135.25585    136.75966    129.72341    133.42427    131.66819    129.84603    131.76925    129.55783    129.75327     127.0174    128.32361    133.79117    132.24176    132.81103     131.5344    125.41306    130.30303     131.0341    130.17826     131.7576    129.91326    127.97271    130.85872    133.16979    134.15546    130.83901     133.2787    131.58237    129.51506    131.76341    128.78338    134.00327    128.61831    131.78419    126.83296    132.43945    131.47616    131.29531    132.86395    131.53969    132.22753    136.05746    138.05543    137.92692    138.41447    138.66351    139.85032    140.23295    139.45305    136.89736     142.3254    137.65171    144.08313    148.21072    148.74706    150.75256    145.71818    147.07502    151.02506    154.26961    146.83102    159.18667     155.5594    150.49603    155.61843    150.52451    149.73684    156.25154    162.44685    159.11358    161.78976    156.56436     160.6246    161.31464    162.39313    159.24402    159.21187    164.27742    159.30657     160.7371    163.22103    160.69271    160.81509    161.34815    156.66788        166.3    166.09345    167.42089    164.80506    164.06728    166.16312    161.77669    167.43412    175.47264    171.54458    169.42499    163.38628    169.71954    175.18905    181.14981     180.7683    173.74511    170.60471    176.15258    174.51401    182.25062    183.01485    187.66601    180.53884    186.90274    187.96635    186.98254    189.48159    192.39936    189.19695    191.88331    186.90892    191.39942    195.29682    204.86298    202.65034    197.09484     193.1352    195.60237    204.31389    215.89406    213.51607    210.43926    200.69871    201.45126    210.96221    209.29054    217.97079    222.39982     203.0016     209.1736    208.93318     219.7682    222.07167    210.33494     216.6494    206.21793    211.65565    219.42165    222.74965    222.57373    220.11067    218.60405    211.90711    226.63233    230.48754    237.63863    234.15938    235.05957    237.42825    229.08641    250.82138    249.43279    242.93989    259.06686    245.41068    236.62587    250.97239    256.61614    258.77506     262.6704     265.9286    273.59693     257.0705    279.71788    285.70101    276.87658    281.12648    277.42156    291.67668    336.78676    332.56995    324.47112    350.71124    335.77294    349.69474    337.22228     375.3669    370.53634     366.1423    384.10991     378.7418    349.57215    463.56694    425.54852     437.3472    447.15061    490.92934    461.91041    493.01119    508.32159    385.51483    566.52108    630.74805    604.57926          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       10.00     258.5587    246.36615    232.54236     226.3062    226.10662    213.69174    210.47188    208.06964    198.35675    193.83818    193.11794    193.69844    187.82431     183.3987    177.88037    177.17787    176.71137    174.03057    174.57565    174.12105    169.24952    164.28216    162.76733    163.21339    174.75587    197.60938    243.52768    250.14737    228.56626    196.57104    166.95091    161.26538    154.30576    150.72661    150.62448    145.01987    148.32985    150.82494    149.54963     148.1446    139.27092    144.17185      142.605    146.15593    143.93612     139.2511    144.03156    136.28663    140.34428    140.98346    137.22498    141.17539    137.53755    134.71359    134.61105    135.44713    141.48657     143.5926    151.78765    148.98161    138.57984    134.18849    133.89042    137.47323    129.89368     133.3423    130.89048    131.02857    133.64265    138.66932    153.61057    172.10473    182.02168    174.60221    159.13877    147.19461     143.6246    148.10277     155.7897    154.99004    156.92669    162.46065    167.09744    161.68612    146.58664    133.43829    131.00703    128.89569    133.31409    141.02138    166.02692    180.77639    183.51123    169.51834    153.77699    137.93485    131.76811    125.07263    127.59049    130.28907    133.56345    133.79428    134.79327    129.18472    125.16616    117.54729    116.59547    121.37088    120.46574    118.11765     120.2494    118.77615    117.31581    118.93838    119.31101     120.5102    122.59791    119.00341    115.79207    117.40876    117.88486     118.4398     118.9658    125.59119    132.77576     136.5199    142.77476    139.85786    131.73851    132.93477    124.19997     120.4687    119.02343    119.63353     123.1153    124.66832     135.7937    147.15588    158.97992    173.63258    174.00622    164.43442    155.05781    143.05774    132.52786    129.98392    127.95264    132.03427    130.17306    124.35619    125.53565    121.65103    124.12882    119.47974    117.47402    116.89967    116.42227    112.96118    118.94665    117.52092    116.44587    118.78922    120.52659    120.58736    122.14928    126.87356    126.83635    126.76662    128.05548    128.75841     120.9101    125.64935    130.70535    129.70874    128.79704    132.93384    132.15647    132.01028    131.69825    134.67476    126.28133    131.71687    127.73798    120.96727    124.26969    127.95686    124.93395    128.27064    129.04516    127.84618    124.79599    121.92261     120.0858    124.86032    125.88547    122.73933    128.32246    130.89849    131.34152    131.45543    134.85301    131.34273     128.4787    125.33648    127.73446    126.05062    126.19468    130.82384    129.93499    127.21174    132.89071    129.24631    130.93901    125.80207    121.33956    122.72437    122.35313    121.98714     125.3768    121.31365    123.25628    121.94292    118.37493    124.75704    125.80268    128.02639     126.2572    129.88413    127.53813    134.61703     133.5731    132.84926    131.58252    134.90691    136.30691    128.99003    139.06625    143.06949    140.29039    140.95538     140.0786    142.71917    139.69391    144.60196    135.67719    134.66548    134.52556    128.25427     130.5831    137.90521    136.79045    135.09346    133.15002    137.45222    132.08678    135.02798    137.38435    138.93979     133.8635    137.42208    138.06785     140.9958    142.31539    144.87142    146.89925    147.53716    149.78016    145.54691    146.19586    147.27557    144.69111    142.33571    139.86723    133.52735    135.24852    137.38319    131.97638    134.12416    134.18657    132.93293    135.06898    137.10977    134.95535    137.49227    139.22286    138.97185    132.72532     134.9309    136.21521    138.21958    138.86167     134.8619    129.33388    130.55144    134.06824    126.84996     131.0168     128.7604    132.57588    130.89238     132.6337    130.01608    129.55145    130.61307    131.61919    132.06866    130.92629    131.23749    130.10171    136.08657    131.24563    127.51262    131.65053    133.38667    134.76786    135.00999    130.60467    132.81564    133.50133    133.21339    129.54967    134.51068    128.53961    129.21755    130.43723     131.5044    131.15912    132.83177    132.22484    133.46115    132.15048    136.44321    141.42636    137.32256    138.59715    139.69224    140.11192     139.8518    143.38775    142.05286    141.32738    141.84999    148.59535    141.52864     144.8393    147.76612    149.48491     148.3814    143.06475    146.46414    150.34168    154.36461    151.95334    150.86281    153.38653    151.88716    154.35287    157.15666    158.02975     159.9139    156.20386    158.80092    159.38677    159.87973    162.10254    161.42249     158.1455    157.93571    154.89775     155.7078    163.54328    159.27695    161.98779    161.95444    160.41831       165.25    163.11119    165.27101    166.99598    167.45791    164.10306    166.31871    170.41586    167.23326    166.28756    168.15475    167.26062    172.56512    170.56869    178.52526    169.63585    177.68203    177.41218    171.63476    175.04872    181.53962    185.07009    185.03807    186.18442    188.38688     183.1045    181.66874    194.85921    186.66587    183.95425    188.88398    185.56554    188.57204    194.32959    194.44978    194.60657    190.79429    201.37318    195.68861    211.85228    207.70741    204.66127    206.38599    209.58706    207.72558    210.91633    215.35625    202.57886    221.94473    205.30532    208.42721    201.07271    217.98181    217.67188     215.0682    221.04383    205.80616    209.37332    220.98342    227.37523    214.03224    218.30828    226.51283    223.29242    233.81122    232.62263    226.91574    236.18772    236.50281    225.65451    226.06146    244.14787    247.51828    255.72984    248.54347    235.73061    246.22454    265.87648    244.17235    243.09634    254.48645    269.09812    270.94421    277.37587    284.88924    291.86576     291.3369    272.13659     283.5804    280.83195     286.2668    291.05218     295.1281    326.60538    309.44476    322.84325     336.4983     359.1672    376.13044    363.86662    373.11329    387.51504    352.95069     480.8555    433.36016    429.10065    453.35841     458.8158    463.93094    685.40856    650.54456    476.59411    495.65761    527.80248    546.29833          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       10.50    262.84351    244.24417    233.05272    227.43589    225.64213    215.61524    213.43024    210.13116    196.42769    199.65548    193.86866    193.86084    190.16501     188.1213    182.85721    175.78117     178.5891    168.49997    177.27835    171.90782    170.43314    162.27363    161.55495    161.06106    175.36409    194.01747    238.30947    248.79425    232.89203    197.48422    167.46541     160.0949    154.39063    156.27888    148.78935    146.08214    148.91184    146.42204    147.36396    145.98444    145.76416    146.35305    144.73125    146.37614    145.47508    142.17222    143.78081    137.11854    140.18951    136.94723    133.11806    139.18265    139.47796    134.55551    138.70768    135.69165    141.18784    144.23513    146.19759    148.10687    137.04775    137.23896    138.28633    139.71345     133.6657    133.26742     132.9859    133.29728    134.13683    143.37273    151.52766      169.258    181.77926    173.14496     157.2948    146.57673    143.72609    156.62386    162.80564     165.5056    165.19338    164.00306    159.05269    153.84666    147.05887    133.62341    127.91508    124.49761    133.54071    141.29293    164.33167    183.14128    182.26157    170.19446    150.49776    136.47335    129.77262    125.96163    127.24286    127.64614    130.11184    132.61342    132.74971    126.67078    123.91391    121.03476    118.26176    121.22655    121.74348    121.58846    120.55932    119.06554    118.05746    120.14557    119.11048     116.1723    120.20913    119.20388    118.97594    117.80731    117.83571    116.40386    120.36699     122.1912    132.46639    135.20984    141.51178    141.74182    136.52294    128.82969    124.05758    119.00691    119.70831    122.24858    121.19052     124.3101    134.68094    149.21547    157.57746    168.61207    172.18746    165.34233    155.66661    144.97842    133.98385     128.9738    127.42028    129.69542    131.35265    129.40791    122.26525    121.72338    118.62608    118.40765    119.03446    116.19827    114.87585     120.5847    117.85745    119.19673    118.43701    117.70981     118.3929    120.81624    119.14182    124.93797    126.19477    125.14723    124.90645     125.0457    122.45226    127.45857    129.78006    132.05858    131.04129    130.14151    132.93976    131.98545    137.27183    133.65151    133.60126    125.39837    126.12153    125.55763    120.63097    126.16884    127.61513    132.51418    132.65732    128.44894    123.74109    123.01438    121.67659    122.53117    123.30942    126.11357    124.36941    129.57115    129.70818    132.04818    131.35218    131.76899    125.69993    125.00268     123.7783    126.79777    123.24487    127.68858    127.12356    124.81818    130.80856    130.53728    126.68621    125.46749    126.17559      120.835    123.41917     120.3528      122.717    123.13754    121.59947    122.71495    120.48544    125.44805      125.105    123.50833    132.05829     127.9841    132.75338    128.06929    133.28351     133.5753    134.40253    130.42358    136.12415    134.12917    140.48923    143.55404    140.24399    139.88339    142.60011    142.94263    140.28976    135.24531    136.41393    135.62991    135.60604    133.90835    130.26345    133.94954    130.99077    134.38167    131.51155    134.49572    132.95858     135.5238    135.47057    131.65543    140.13576    139.01971    139.26606    141.87224    144.26566    147.69068     147.6324    149.86264    151.50123    144.34751    143.81091    146.66597    143.64712    138.26297    134.95159    132.59042    135.35375    133.23262    130.68119    135.48009    139.58388    136.06642    136.51256    137.34648    136.03593    133.24374    137.20405    136.84892    134.55166    133.05431      136.626    133.30538     134.9703    133.94061    131.77536    130.82919    136.34888    133.90394    131.89432    132.03377    132.70116    130.08864    134.54572    132.96177    130.57328    129.85359    131.85843    128.17038    130.66785    129.11039    129.00817    133.40335    132.30401    129.72143    134.93535     135.5768    132.03624    132.03479    132.72942    133.65555    129.97362    135.52987    135.37082    133.17878    130.06639    128.75873    130.14123    129.44774      133.891    132.23993    137.12462    135.43692    135.72704     134.8066    135.91149    139.04226    138.11892    138.25529    135.87108    144.10872      148.821    142.96461    145.01333    146.25159    143.65416    147.46527     144.3876    145.65994     143.8368    145.49023    146.44427    145.37821    146.77545      150.331    149.31686    154.58644    149.31446     156.1032     154.9262    154.36086    157.33296    156.78986    163.96827    160.64234    156.44643    164.53257    168.13762    160.30484    157.70558    158.70665    158.08172    159.48775     161.4305    159.03029    164.27472    161.93036    162.15163    160.25367    163.35498    164.87426    165.40099    166.02574    167.39923       168.53    165.60736    168.34709    172.59166    172.42925    172.77499    173.04818    170.49534    167.32697    160.73082    169.74791    185.92752    175.02399    177.52692    182.88899     185.4288    178.67041    178.28475    179.60991     181.9159    182.31711    184.86729    190.74321    191.55726    189.89878    185.59602    190.13754    199.77937    195.76961    199.59207    195.06745    192.55353    201.11004    203.25717     207.6588    205.78949    209.25823    209.62637    199.54502    207.07946    210.40482    218.09552    204.40663    207.22001    196.61946    213.62707    216.44924    204.60064     218.1096    210.65744     214.6686    209.57752    212.60665    218.96511    222.16374    223.53918    212.17469    217.55602     236.9251    243.01168    235.04847    232.74774    233.05324    236.00973    229.29269    238.98594    236.39501    239.48539    239.12773    250.36721    231.34694    251.30651    247.70107    245.37592    262.62381     252.2955    260.06094    262.80672    283.59325    290.95123        274.3    281.91838    271.83166    278.84672    313.86301     297.8359    324.17814    349.04409    316.03457     331.7205    367.12678    365.77909    357.32007    407.47595    357.43144    356.98526    387.46152    434.20643    426.12958    441.20337    468.77641     401.4024     502.0802    514.54356     532.4636    542.04152    536.63031    606.14523    577.08263          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       11.00    266.72468    243.53468    234.66199    228.58166    225.25174    219.82161    215.32722    213.52808    195.71347    198.04331    196.15458    193.82961    190.51579    188.75258    185.90568    179.05325    183.14892    168.69791    173.88934     171.4957    170.20737    166.56111    162.91513    169.75821    173.26121    193.16041    241.87065     248.7285    235.47217    191.46771    167.00812    161.37947    159.13277    157.46943    151.47635     149.9106    152.40444    151.53073    150.19243    144.71546    146.99398    146.05798    147.32973    145.52475    147.08714    139.05477    140.98129    137.16613     141.2949    138.09415    140.71726    137.19436    140.49554    133.83421    136.16752    139.16693    140.06999    144.79668    147.65657    147.60856    140.97879    137.98547    135.70128    136.36014    131.31475    136.17312    135.91516    132.95659    133.26985    136.46546    154.87017    176.64965    183.05808    175.66537    160.67071    141.11634    150.04913    163.01951    179.29352    179.34431    170.15621    160.15334    156.43355    156.42165     147.5611    137.43158    131.39952    129.41407    132.04283    144.81113    162.35253    180.48792    181.18942    168.95515    147.75833    132.39552    126.33494    128.10515    124.45721    127.52232     132.2775    131.09612    131.37859    127.10875    128.24243    121.68692     117.2023    120.09352    116.81325    123.44791    118.81166    122.28637    117.82489    115.35022    115.90684    116.92834    118.04356     116.9183    120.20345    115.23669    120.42055    117.64425    120.72797    122.61076    129.37251    135.73653    141.57422    142.96366    135.96643    128.99896     124.7783    121.75446    118.11711    122.08122    122.34264    126.59369    133.57863    149.23998    162.47011    168.45182    173.73057    167.80198    155.96355    147.95493    137.64974    131.16437     131.2998    130.18129    127.68816    125.16851    125.69967    121.27127    120.84926    118.66725    121.81997    118.01455    117.21237    115.85645    115.09641    119.27978    118.81879    117.08376    121.05658    122.25439    122.47473    124.86773    127.15704    129.94629    126.22893    123.14338    127.21758    123.83152    128.20632    124.96324    130.53888    131.16573    129.69607    130.89163    136.01904    134.59966    134.47454    130.18756    129.64554    126.93473    118.51895    124.88256    125.38022    133.09591    123.08186     128.7463     118.6887     121.5534    119.72409    120.88343    121.01258    124.24537    126.99786    130.10441    131.50051    134.03936    131.60925    129.46163    127.65601    124.78141    124.15536    125.33264    125.35273    125.20819    124.81259    129.76121     131.8024    127.29949    124.62249     124.0964    124.65331     119.3991    123.32645    124.57247    123.01069    120.19514    127.20697    120.95864    122.42667    125.79037    125.45831    129.08606    128.21716    130.37054    132.06614    130.14688    134.37182    132.30208    138.00754    134.53322    134.55186    133.13215    139.43014    142.39389    140.72637    146.65809    141.40271    142.31275    139.79834    136.69973    139.53408    138.76958     139.9593     132.5598    137.11015    134.36746    135.86296    131.60257    139.34618    137.86134    130.31673    134.01136    134.20971    137.47495    138.12316    137.76638    141.04494    143.54562    141.42276    147.49563     146.3373    146.33373    147.23147    144.83922    146.37629    144.93336    140.47618    139.58396    138.63227    134.38609    132.81964    136.36048    134.26386     131.7481    135.56929    135.63706    132.92209    138.77324    135.68651    137.83632    132.77035    136.99911    131.16976    134.91542    135.29045    133.02211    131.72187    131.83712    132.03151    125.05461    132.21295    131.81678    130.94863    130.71558    135.32492    128.22317    128.66543    136.72058    132.33353     133.1853    130.01189     130.4285    129.58256    130.20971    131.53004    127.74668    131.47944    130.70855    128.43386    135.69649    129.28654    133.17114    135.71792    134.05019    127.61582    135.23761     134.5444    128.17085    128.93218    128.53075    129.27388    130.44488    129.83425    130.84593    130.85337    129.10619    134.28746     133.0038     134.4639    137.61019    139.29648    141.10429     146.2613    143.92845    148.33468    147.22094    151.88417    144.23765    146.96075    144.67777      152.754    149.51699     151.4594    152.45249    144.72894    144.39238    146.03527     148.4862    150.09433    149.61718    150.24815    154.37041    154.77627    159.76015     159.7282     156.1361    155.96555    156.79845    159.83387    158.52747    165.77991    158.28942    157.52145    158.82615    158.95686    151.99999    158.26802    164.16051    163.42055    157.73754    162.63402    161.10759    161.37553    162.11535    165.93525    165.04002    169.99092     166.4313    167.12421    168.61686    171.62152    171.62175    171.82319     169.5856    174.65071    180.44121    173.95643    180.23247    180.11358    167.96221    171.65856    176.79565    188.14216    190.19636    181.28259    176.39786    182.11058    179.32713    192.16473    186.70944    179.25699     193.3475    190.59642    190.86212    194.01489     191.8668    191.40501    196.19645    197.62084    191.60727    205.76823    210.53561    204.61294     201.6874    200.85401    201.63176    205.48339    213.57824    215.16157    208.52148    210.81268    208.19495     215.3108    222.95504    218.81744     212.3627     219.2705     209.0749    215.62899    233.08259     229.8444    207.62617    222.98982    219.62825    215.00306    226.29211    237.81271    237.23419    246.93252     220.2788    225.29486    236.04871    250.50669    229.36217    247.83645    241.80154    240.37317    230.95528    273.80416    258.11882    242.11624    272.96334    260.73318    273.59249    256.30602    294.13797    295.07468    279.88263    289.95628    295.13877    290.82985    329.72565    322.72699    341.00677     319.0603    326.87847    328.42803    378.87514    350.70817     366.2742    381.79608    354.61713     382.9516    353.34545    473.20916    490.81343    397.66239    482.69021    467.12838    448.94482    571.15045    649.36209    612.49407    533.20866    504.33448    622.17289          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       11.50    268.13183    244.98681    236.13829    227.79696     224.4184    223.70622    216.72516    217.54213    195.69036    187.82702    199.25505    190.32941    189.41472     189.8962    183.02039    182.50179     178.9365    173.89932    173.29155    171.73855    169.76299    166.50092    166.55313    173.50416    174.49352    195.09802    239.95728    246.05213    234.87606    195.41898    162.10634    162.12445    155.30149    155.88721    152.92779    146.95631    153.38853    152.56389      149.421    149.48727    144.66034    145.34154    143.09036    143.10461    144.45371    147.17355    144.54092    138.89442    137.64867    139.89877    139.52728    139.62215    137.85879    132.24685    135.44098    134.85708    141.44904    144.32293    147.79875    147.67139    136.88307    142.16449    137.18806    135.55915     132.8377    134.10271    135.71698     133.1114    133.37786    138.27194    152.91939    178.66938    185.95546    175.39398    160.61495    150.99232     153.7769    169.78747    184.45692    184.39803    172.46517    162.67613    152.25818    148.67047    142.85026    134.54104    126.70661    128.77514    131.31392    143.20709    160.59296    177.95598    181.96423    167.65222    151.70285      136.456    127.24392    123.98333    126.41365    127.26851    131.96636    130.45636    128.20303    127.77105    125.99258    119.67654    117.32035    120.76583       118.81    120.93835    119.35387    120.75776     115.4569    117.78139     118.1641     119.2746    120.79369    121.13106     118.6433    116.25784    119.68744     115.8005    117.32351     127.5652    129.53903    135.63399    138.84886    143.25624    137.84216     129.0921    124.30682    120.58577    119.79815    121.74484    121.43488    127.56734    131.32239    150.72323    161.36258    173.98523    178.10621     167.7517    159.89476    144.87548    140.15383    132.20662     134.4708    132.29766    130.16127    129.41527    124.37375    116.38557    119.29502    117.94232    114.31853    117.28648    119.50253    114.88463     116.5672    119.18709    116.80283    120.36412    118.45866    121.01227     119.1539    124.90167    124.42298    127.52474    124.23769    123.82953    121.87417    118.97274    121.81885    124.65398    129.80361    132.75279     131.3492    134.26751    133.40735    131.01539    135.31842    132.91391    128.29421    127.29985    125.56465    125.25022    123.44052    122.04157    120.82753    122.28012    119.48616    119.36768     119.1915    120.82456     123.4226    124.01064    125.71335    126.18319    133.40894    128.79825    132.03877    127.89368    130.65847    125.52754     126.2916    127.78213    126.73012    124.38422    129.87669    128.15747     131.6452    128.30822    130.39269    125.64625    123.20652    123.18242    119.31199    123.50074    122.33649    121.79988    119.41115    121.01877    120.94266    124.17767     126.5546    124.09046     128.8008    129.53118    133.54391    129.61743    131.38845    132.76589    132.13986    131.70584    135.11333    134.71743    135.44136    141.34831    140.15035    140.29972    141.91185    141.60087    140.00447    143.16864    143.07121    142.20208    140.84764    137.14323    135.28162    134.51805    135.77577    131.71059    136.29436    134.62719    135.59547    132.27722    138.72242    139.46447    136.39005    140.02031    139.77625    138.81779    143.19872    144.10701    147.76169    145.34558    150.26821    146.95532    144.97781     143.4966    141.72074    136.98078    138.63833    134.06801    133.23774    133.13505    130.89491    137.03159     137.9029    134.78527    131.68176    137.56391    136.22862     137.5838    137.24587    136.75933    135.91455    132.57714    131.81598    134.09859    131.61986    133.40171    131.26473    131.07946    130.05239    133.50354    130.90118    132.22511    133.03154    130.33024     129.8586    129.68404    127.55748    127.74262    126.94629    131.54333    128.85869    129.02641    128.76907    131.01427    131.31074     127.6356    129.47448     132.4885    129.70402    133.72621    131.38816    134.40752    128.61377    130.83654    131.82046    131.23638    129.75332    133.75065    132.45932    130.66516    129.89792    133.28168    130.12638    132.00052    130.33783    130.43047    134.27708    136.00853    138.40896    139.49991    140.08628    140.68008    140.74743    142.45591    148.61465    147.50475    144.40424    147.90932    142.97616    148.51681    145.34764    144.67608    143.15526    146.84928    145.79534    150.11161    147.93636    145.05108    151.98669    155.68577    153.71833    150.31654    159.73848    157.02449      157.697    157.90391    159.17348     156.9363     162.5727    156.08829    157.87933    153.97692    159.65937    156.91442    160.59462     160.0862    163.00123    159.37593    158.00536    166.70009    154.02855    168.75428    170.52064    171.34897     169.3218    164.23893    163.71549    172.70828    170.66378    172.86091    174.90603    173.04229    168.73028    172.60617     176.2632    178.20061    166.20857    177.99776    171.85613     173.6854    184.85677     188.4989    182.25348    182.29547    173.61919    181.85758    184.66746    192.99624    187.74712    183.95457    189.67132    187.28758    195.24792    196.04329    199.10738    201.86404    194.23134    196.46745    200.98078    205.51697    200.58372    198.60556     203.6406    202.89077    202.26446    204.66803    208.77889    206.40711    203.06846    206.87136    219.51043    215.76161    212.52395    216.23335    201.93765    211.57331    207.41611    212.46525    207.07809    222.96622    208.85786     217.3183     215.9449    229.10239    241.85041    238.03812    235.78661    228.33334    236.30733    232.89798    242.57704     233.1188    249.78431     249.1052    228.33742    252.29658    259.45423    270.04942    267.96769    262.59805    251.01558    269.08542    276.24919    269.93826    316.08216    295.31321    293.32358    295.64298    288.89807    317.88125    303.36987    304.20579    315.97798    318.00714    324.02902    351.28773    422.87033    369.36663    371.04594    361.29408    359.89767    375.74781    481.75197    399.43204    461.47688    415.36639    450.42017    469.88232    434.10195       591.56    575.44002    563.69514    648.16336    520.50601          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       12.00    267.31766    247.60562    238.93964    226.35046    223.54224    226.28021    219.11342    217.71094    194.63359    183.45707    200.88488    188.05231    187.50189    190.87114    180.72391    184.65056    175.25511    170.97358    174.44337     174.6905    168.55219    165.89201    167.04886     167.4438    175.74427    196.63305    236.74632    249.99786    235.70905     193.1634     168.0959    160.94486     154.8158    156.49423    152.62051    147.87477    151.35761    149.65581     145.9243    149.26241    142.71594    145.88183    142.15373    146.28554    145.84457    141.02092     145.6355      137.729    138.75122      134.272    143.50383    137.97594    140.16232    139.41471    132.82142    136.31711    143.96319    147.39769    148.30579    151.01135    141.94171    137.15451    136.30361    135.63525    133.60968    134.36829    135.60987    131.81392    133.90326     140.3141    152.12049    171.29748    181.90446    175.48062    155.65708    152.28393    156.54706    171.12383    184.40739    182.22582    169.75387    157.42714    150.76443    146.04337    141.34049    135.35584    129.05214    129.89048    132.61714    144.34506     163.3869    180.72198    187.39756    171.01209    149.71495    137.38957    128.29857    125.13152    124.82391    123.92608     131.4719    129.22247    132.08829    131.31313    125.70463    123.59382    119.11759    120.25076    121.77768    122.05821    120.74539    118.49128    118.83429    112.93453    116.76266    120.76363       121.12    121.54485    115.93186    114.89501    116.99648    118.59476    117.65236    126.53193    132.46523    138.12324    140.35291    137.96338    134.92689    129.26397    125.40776    121.33001    117.30707    120.70884    121.72292    124.38504    134.61494    147.35236    161.73354    170.53094    172.45042    169.76741    157.56424    145.72649    138.49786    129.01512     131.7323    132.75616    130.57868    126.59563    121.78566    121.52389    118.35608    115.87317    115.84575    115.04921     116.0085    116.77093    116.82841    120.40014    118.52052    119.29828    118.89039    119.33337    124.86937    122.68161    127.81057    127.57433    122.98642    123.93105    119.90864    118.35009    122.10187    126.94583    128.54264    128.64523    133.02596    133.90143    133.73473    133.15167    131.19771    132.11456    135.11197    130.67187    125.43675    125.63991    123.07676    121.88042    120.17588    122.00406    119.83706    123.87657    120.82311    123.49919    121.16669    124.29543    127.91312    125.91351    129.38006    130.38482     131.2751      130.819    132.23199    124.69924    123.52378    126.77358    124.82291    127.72021    123.28157    128.98695    125.38517    130.57696    127.59791    122.85173    124.35085    124.01359    119.51409    124.04301    118.10728    121.07283    120.74696    118.83173    123.18575    119.07647    129.12329    129.09926    127.51586     129.1174    129.86664    132.02855    131.87335    130.87415    133.74227    131.55585    134.22007    139.65012    135.14042    138.42236    141.90696    145.59167    146.43034    145.41148    141.92675    147.75999    140.97026     143.3338    136.58704    137.19787    135.32941    129.28802    134.44628     132.4357    134.50732    138.36473    137.12891    135.20202    134.74418    135.52261    136.43586    136.93874    138.40673    141.21105    139.79242    149.36521    146.18043    147.41013    147.91626    144.85147    143.89838    140.97723    142.26653    137.75521    139.89157    135.50259    129.33484    132.43606    135.68938     129.9106    133.38805    134.51303    135.64385    138.06712    138.26108    136.20411    140.61691    135.58977    137.98871    131.39756    133.42921    135.40542    135.03165    132.15313    130.20109    130.05322    135.46705    133.90257    132.00478     134.5779    133.70116    132.42315    129.10959    129.80336    132.14738    134.45777    127.93459    128.37029    128.13922    131.79568    129.78716    128.74128    131.09296     130.6238    131.56235    134.24727    132.49043    133.96658    130.94901    133.11162    133.16222    130.82613    130.40367    129.78161    130.31322    128.67645    128.71524     131.0584      129.978    131.22068    130.97773    128.76469    133.80499    134.58266    136.79415    141.46571     140.4896    138.58359    139.41576    141.59662    141.00592    143.88521    140.50433    144.62896    145.76554    144.84842    144.08882    143.83382      143.095    144.05726    149.84956    142.89157    146.78741    150.09212    147.72901    151.69163    146.06171    156.54451    146.44804    153.98599    157.62074    157.34862    159.74873     157.8909    157.38377     160.7491    162.01884    161.34065    155.56461    154.81674    155.16095    154.11923    160.82633    161.41622    164.02855    162.85343    158.81956    158.21595    159.53449    164.39931    164.21008    175.84774    158.69497     169.8437    164.15542     171.6648     172.7018    169.10492    165.04142    172.18887    170.66558    174.22809    178.63217    175.65755    175.30109    182.34779    177.83557    181.20491    184.36426    180.11862    181.63806    174.53298    178.41418    187.01045    185.33699    191.63845    189.38926    182.24087    184.66635    192.94154    202.43272    201.15465     196.0086    200.92564    193.68235    198.01634    203.73699     201.5727    194.42733    202.34588    204.25459     203.4474    201.04151    215.30893    210.43999    204.21497    208.38367    206.37611    221.31962    227.08203    210.00414    214.37656    214.47641    203.26876    212.47364    224.38699    217.93689    219.95392    206.12851    220.51293    223.66745    234.39423     222.1333     234.7106    231.15618    235.90496    233.71388    231.22915    238.57718    224.88655    221.67743    251.20048    252.77624    250.33873    265.45637    272.18866     261.1744     252.2922    263.88351    275.65568    269.22524    270.22448    293.13982    302.53802    291.62269    269.63355    286.03636    307.15536    311.98172     328.4441    331.94415     341.3506    294.42025     379.9064     343.1204    373.21889    351.97185    392.04894    398.90434    386.80476    464.98712    421.18897    421.55573    461.82111    452.34168    526.05728    640.90036    634.81465    592.21234    556.86893     516.0515    459.30111          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       12.50    266.99507    247.19923    242.35579    225.69583    223.45945    229.15116    223.05046     217.3584    193.53607    188.81926    199.20773    192.90239    186.93236     186.5805    178.35398    184.00826    176.63487    171.69046    172.37378    179.34721     171.7043    164.71113    166.28052    166.52424    175.61559    197.99108    238.78873    250.81229    233.22484    194.68254    162.69341    161.84683    150.81623    156.34873    152.27489    149.34187    153.55438    149.61829    146.80494    149.91024    144.94399    144.09121    145.43484    143.95219    146.38629    142.99136    144.40533    138.30879    141.63372    134.00116    137.38313    139.18783    141.58016    138.87088    133.60938    138.57405    144.47468     142.6886    150.23472    147.94978    142.85153    135.80344    136.42969    135.32927    134.84105    135.68967    131.53275    134.70441    134.11886     139.2179    157.00714    175.46696    181.23013    180.06733    162.02152    153.80993    156.45364    168.37991    176.79722    176.61384    159.15151    153.50626    153.64469    145.19482    143.28428     134.6879    128.28766    129.46503    136.87463    148.31423    169.15993    182.14878    185.96434    169.43761    152.76775    133.51599    129.07986    129.11858    124.55183    126.36591    128.23269    129.91851    130.04362    132.28736    124.48636    123.11378    118.70347    122.09779    115.69447    120.96481    121.13615    122.46498    119.18378    115.50003    117.03594    120.48819    118.71728    118.26803    119.46973    117.51921    117.21582    120.08036    121.54183     124.2781    130.54983    137.30838    140.17524    138.68377    133.71956    129.91806    125.79633    119.71779    120.80969    121.40335    119.88031    126.48892    135.37033    146.73661    158.84366    173.45722    173.83067    166.34069     159.9544    145.43682    134.11714    130.72441    130.03522    131.39551    128.24431    127.32388    124.54024    122.23629    117.61903    117.44906    118.91719    117.79674    117.73517    117.29725    115.55305     118.1963    117.36022    118.34198    116.45674    122.29329    123.26708     121.6695    126.92931     125.0714    126.45953    122.18885    123.35294    120.58396    120.14448    126.63069    127.00323    127.79785    131.86828    136.14746    131.28094    138.61172     135.8417    136.02286    130.96819    128.53856    127.97143    123.44621    120.23032    122.18686     120.5732    118.81784    120.95793    119.90194    122.90962    120.99289    123.03286    124.71639    126.00295    121.59651    129.80055    130.94308    130.44485    128.96591    128.63209    127.80935    122.73657    126.01225    128.65138      127.875    127.69983    125.42732    126.97715    129.81409    125.27513    125.02625    124.28562    120.08093    121.49707    121.29557     122.7156    120.48809    121.35611    122.34179    123.05249    121.42085    124.65399    127.84366    128.96176    131.19429    129.00244     128.7044    129.23287    131.92834    131.75744    134.76402    136.47481    139.32861    137.57086    143.98944    144.88673    143.57041    142.33116    145.90933    140.77153    139.36886    141.49681    135.92226    137.43313    132.77272    133.43529    128.73401    132.94615    133.84173    135.29725    137.27101    138.13683    136.99402    134.41486    139.64675    136.20091    139.41282    140.51257    139.96052    141.50061     146.0424    147.16904    152.88832    144.05804    145.29033    141.62418    144.30756    143.05423    141.46337    135.88496    134.23578    129.44435      134.836     135.5995    134.87681    132.76429    133.61426    136.14108    136.18802    139.53775    136.46233    136.45697    134.57597    137.43799    132.92581    133.55623    130.10402    134.37352    131.89239    132.64247    133.93202    134.74454    137.37948    133.79761    138.65893    135.39531    134.67391    127.84992    131.33028    130.30412     136.4456    131.38272    131.47155    128.36405    129.71742     130.3029    129.69446       134.39    132.54631    129.33968    131.09759    134.05651    130.85553    134.35089    132.21375    130.84394    129.35808     131.7243     135.2647    130.55217    130.18794    126.48449     127.9468    129.83529    129.68745    133.69044    128.29757    130.22979    133.36653    132.56673    137.08981    137.38105    138.89471    137.84048    139.68985    134.58811    145.01033    139.75131    138.85754    143.46521     142.9524    142.22749      144.886     145.7014     149.5556    145.05127    144.56556    144.70804    152.17024     154.1821    152.34865    154.01926    148.28414    149.67671    153.76087    158.87733    152.23316    161.72951    157.76124     157.3909    155.95713     161.2623    159.95592    157.15567    162.64745    161.24321    159.87328    160.01751    162.84866    159.74589    158.96142    162.83969    160.65806    159.97611    164.37041      166.472    163.97198    168.05952    163.59911    166.40183    169.49032    170.80449    166.12637    168.95233    170.14036    168.29515    170.38686    172.44554    176.41483     177.4652    175.61986     170.5277    177.42716    183.08742    176.72513    179.74635    189.16758    189.82582    188.72408    187.15578     185.1304    178.10604    187.33056    188.61333    182.72785     191.8932    195.48982    194.53577    201.87173    202.51517    189.90584    204.65003     198.3582    204.87787    208.26434    195.94839    203.55708    201.77472    211.22267     210.2815    208.44184    215.66806    222.74636    215.20308    221.20768    210.76478    207.11383    214.64386    216.21314    212.28686    211.06019    217.83515    220.27399    221.96862    220.02343    215.74206    226.20631    232.34145    226.28161     246.3381    231.74928    220.94491    239.55191    245.08491    251.19428    246.46445    244.17704     251.4618    227.93508    260.54117    261.49508    259.69826    267.07452    265.45318    248.18658    271.91397    286.43277    300.55846    285.78958    294.36908    267.97898    286.24006    296.88733    276.53649    346.06369    332.27449    336.45315    325.55755    376.83836    364.66452    350.74927    371.36928    365.95347    354.08349    391.35091    432.87712    440.97726    436.43547     488.8003    455.77781    470.89935     599.5718    649.41418    612.12335    469.71558    548.07481          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       13.00    266.93051    244.79481     244.3442    227.61522    223.90007    228.92055    226.23592    218.85577    194.52256      193.193    194.78062    196.83999    191.66833    183.72261    180.42023    181.90661     177.9762       176.31    172.04697    178.15281    173.35514    165.80885    167.44145    173.22599    175.67977    197.97114    241.90937    250.36274    231.39322    199.01717     165.2225    159.45999    153.55067    161.30309    151.77098    148.68815    151.42389    148.77505    149.23653    152.08085    145.52664    145.40456    145.92374    141.40348    143.37548      143.547    148.96543    138.20495    141.71231    141.40792    137.54227    137.72434    138.56435     138.1401    135.77919    139.33026     140.5882    146.21315    147.04054    147.71432    147.08942    138.31843    136.43898    138.31515    133.48422    134.47028     134.9845    137.14508     135.1463    137.37443    159.37533    173.64068    177.40307    177.47743    158.92625    152.05539    162.19187    173.58688    172.18074    170.04945    159.34382    147.59894     146.3669    147.11061     137.9815    133.26655    130.22436    129.10687     136.1198    146.14891    165.60752    183.01928    186.29651    174.90253    153.80133    136.35078    127.29351    129.21584    124.20131    123.40344    124.96652    129.15727    131.85051    127.70549    125.66303    121.43588    119.31822    118.64581     118.8678    121.33386    120.14475    120.28147    118.26352    116.17103    117.51856    117.23945    119.06284     117.8264    118.45879     121.8032    118.39199    119.05828    120.77069    125.53197    127.85061    136.50212    135.86492    135.01893    133.88439    130.81613    124.22517    120.49827    120.74303    119.96466    125.94881     125.5602    133.41841    146.19389    159.97396    167.19804    173.33001    167.71013    156.69481    147.05576    135.69395    134.27761    131.44975    131.54587    133.67022    131.72877    128.16895    121.31584    119.09219    122.06319     119.5652    120.07068    117.69122    116.31563    114.48402    114.32202    118.29649    119.88945    118.01542    119.00366    124.42701    124.28957    124.27768    128.41335    127.19257     122.5494    125.55713    120.69945     119.3159    122.45074    130.29647    136.28585    134.38165    136.96684    136.54524    137.82656    135.48828    136.22684    131.14086    130.93015    126.63276    123.13157    122.11453    121.15945    122.36011    119.39525    119.61836     122.1148    122.27108    122.61395    123.80793    128.06998    125.84994     126.2105    127.25055     128.7907    130.78262    132.65881    129.84519    128.06627    121.09018    125.69903    129.41134    129.05822    128.27024    129.47851    127.21571    128.03953     129.9964     122.5186    124.85076     117.7049    119.17497    119.06781    120.35915    122.61641     124.4207    121.24278    122.80608    123.23171    125.07059    127.01612     126.2446    124.69859     128.2372    127.37119    130.83438    133.60124    134.15147    136.27163    132.39024     136.5991     139.0611    145.84884    141.79624    145.42486    141.87956    142.10271     140.4456    134.23971    136.94967    136.54696    135.82871    133.40588     132.2482    129.19502    132.12798    135.36068    132.44947    137.03472    133.93569    134.29977    135.25665    136.76046    134.18375    137.70545    140.06592    139.29587    139.08739    143.43343    144.31652      146.244    150.47906    147.96055    143.92173    142.26419    139.87569    139.80453    136.00233    140.05655    133.79983    134.59797    134.40366      136.221     135.6775    135.47154    134.94412    134.41732     140.3626    134.49217    137.40603    136.46926    137.68133    135.90485    133.33748    133.71523    134.80278     131.0824    133.67203     132.4869    134.18164    133.71609    136.50485    136.51278    134.52378    132.54948    130.62693    134.62074    130.83032    125.95234    131.60355    130.01393    129.15557    128.63752    129.03481    133.84473    130.42652    135.10832    128.99376    130.36388    132.22098    133.91574    130.99555    129.77415    129.31236    132.52552    129.70314     131.8499      130.122    130.38818     129.0093     126.4126    129.83733    129.80412    132.47905    134.26987    133.00232    133.50535    136.08182     138.0619    138.99406    141.08332    143.81259    140.57517    140.09328    141.05231    144.75788    141.35392    137.90483    141.46061    140.47839    146.86584    145.21993    141.94516    151.76288    148.77011    148.69574    150.92395     154.7191    155.84486     152.8861    153.70703    157.69781    151.56749    157.61247    160.85333    159.14923    151.84829    152.93154    157.51819    160.07547    158.88449    159.86188    153.92361    152.00159    156.11308    155.19461    160.10331    159.48364    163.05911    162.07369     158.6802    165.97535    169.64588     166.5036    168.99728      168.681    164.79876    165.01863    170.68706    169.85069     171.1754    170.11489    165.78569    166.40035    175.46347    172.46236    179.39181    174.08536    173.45737    172.51298    175.43785    178.36896    185.60276    184.95668     182.6846    181.37696    179.28421     194.2737     186.1976    193.02384    195.62978    192.25559    193.25556    198.64341    194.62241    195.48597    198.17107    197.33023     190.4409    201.22966    198.84991    198.07782    196.14815    204.58581    200.18628     195.5015    209.99027    206.69766    202.15832    210.28678     207.9904    209.62143    221.96385     215.3189    220.63128     210.9369    211.63313    215.91473    222.57173    222.52716    209.36152    223.56986    216.64418    220.54462    232.45499    239.21482    226.70905    235.37874    224.21668    224.81341    231.95262    252.97515    244.68795    223.38151    239.54141    242.10335    240.39947    260.38439    276.04669    252.39313     262.7082    267.83962    263.55334    277.50847    292.51845    293.98209     277.6669    282.64796     291.3482    279.99544    318.67264    313.51822    330.47955    329.13459    289.91049     299.6574    347.89052    378.80304    362.40847    371.60699    361.79743    384.35186    420.30162    469.99451    486.05406    405.76521    508.09287    428.37062     571.3815    513.95581    589.11532    515.83859    645.12241    580.88345          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       13.50     265.6498    241.32628    240.03693    230.41653    220.57042    225.68485    224.25865    220.70723    197.82968    197.03829    191.93266    196.89972    194.23265    185.36333     182.7672    180.62117      178.147    177.58089    174.29262    178.26827     173.9021    166.21741    163.00134    177.47059    179.82817    198.73639    241.98937    250.29925    234.53049    196.26543    167.14282    158.41086    158.23332    157.80875    152.06604    147.63484    152.19504    150.11321    144.75916    151.90634    149.70693    142.37957    144.43491    145.50752    146.28127    146.49149    148.83675    141.92702    141.57964    140.18866     139.3182    138.16536    135.53368    140.38257    135.71002    136.44647     144.2129    142.04626    151.39927    149.24591    144.06157    138.73614    136.59187    137.56162    132.81073    138.00488    131.65911    135.08103    137.01395    139.52768    156.12241     173.9017    183.43515    178.13829    160.04873    151.95384    161.91906    172.96309    179.23968    167.19213     155.7662     147.9741    142.89782    137.78117    132.98349    134.90556    130.73263    126.12477    136.07736     145.7776    164.79262    182.83606     184.4332    171.96939    153.15262    136.32786     128.5616    126.18537    125.00465    125.78141    127.26373    127.84093    128.24095    126.89567    123.41131    121.57446    121.56514    115.50255    122.41561    117.56589    120.22721    120.64452    120.84767    115.60941    119.05588    118.66891    122.09483     119.4067    118.26728    119.46384    116.25511    114.99285    121.00066    124.40334    133.73678    140.15038    141.89857    140.17835    134.77321    127.24347    126.06313    122.90485    117.18096    117.39355    122.00524     123.7563    133.50063    143.15038    161.78516    167.24361    173.91515    170.64838    156.58496    145.48871    138.51184    135.65717    126.48874     132.2748    136.07558     131.1908    125.67543    123.14527    118.12064    118.57564    117.39698    119.21305    119.23195    115.92647    116.10981    117.10695    116.86144    120.88523    119.24815    121.05668      122.534    125.47746    123.73162    127.40021    127.73905     124.6752    119.83528    125.42209    122.76137    127.18703    130.98322    132.18974    131.34542    135.77154    135.57119    134.70406    141.18967     134.2592    129.04006    128.25313    126.02541    119.25969    121.58437    122.16441    120.41948    120.22285    120.39532    116.71037    119.98402    122.96254    121.91214    124.65622    125.48018    130.38145    128.06073    128.81966    132.60992    129.42063    132.83832    128.24096    124.19055    125.37898    125.71721    128.14738    129.16278    127.89754    130.70362     126.5058    125.51762    125.04733    123.08912    122.98879    121.93174     120.3706     120.8142    120.02018     121.2738    121.28314    123.22622    122.77945    122.68017    125.86071    128.98779    127.50539    132.30372     128.6114    128.04126    130.50348    132.21563    131.77957     130.1929     134.9294    137.25173    136.55115    138.79942    139.41499    137.66337    139.89783    139.76828     136.6837    139.68523    136.48866    136.65147    134.38451    131.38697     133.1927    127.69304    127.79916    134.16503    135.41677    136.79811    135.22004    136.41121    139.86679    135.64586    139.87011    139.50806    138.60315    140.49325    142.84971     147.8825    147.57214    146.36348    149.10541    144.46555    145.78455    145.20018    139.01135    135.24787    137.02514     133.7385    131.01748    135.01864    137.39062    137.77044    139.06237    138.80541    131.58847    138.79957    136.80561    135.02301    138.64323     139.2159    133.94973    133.99059    134.94848    131.67034    131.98906    131.60524    131.83651    126.22712    130.60396    130.02041    129.37974    129.61006    132.00144    127.43164    129.94153    126.24886    132.10701     133.6685    129.37664    130.01046    126.86046    127.62556    130.06028    134.65556    128.65501    131.55214    129.41853    133.30787    131.74132     134.4723      131.318    129.77549    127.46695    129.32161     131.4632    126.55981    129.60346    130.24876    127.55191    131.16875    133.58378    129.57536    131.23874    130.33689    134.25427    136.11834     139.1098    140.15444    138.15112    141.51317    138.72369    138.58589    143.67871    141.10495    140.77144    142.13512    142.61426    137.07936    147.79992    141.78986    142.10645    144.34139    141.08687    145.71561    147.81428    145.55761    152.33242    152.94454    150.66458    154.28593    148.63622    153.22796    156.19918    157.00942    157.62022    157.98023    159.71264    161.36449    158.79923    158.50964    156.40555    159.12741    160.08046     163.0123    149.32556    159.39647     160.1586     158.9341      159.343     159.5596     164.8186     166.4391    165.61541    159.85038    166.74828    162.60686    167.17563    167.65759     168.9648    166.49006    169.60949    168.69137    165.65688    170.49646    183.35296    173.66992    176.83615    172.99931    178.02865    182.41989    186.01815    185.39248    180.56955    178.14981    180.97467    192.15482    185.07532     192.4633    178.54797    182.55513    185.36591    198.92216    193.78264    190.92367    195.70042    196.41589    198.87732    190.52922    202.32162    197.56625    199.57231    205.65184    197.28095    195.73309    206.59554    212.69142    208.50309    220.20038    206.81306    210.18623    223.67478    211.78848    211.54266    218.79286    207.28096    206.55321    218.05338     226.7372    223.38752    217.12144     225.1774    221.11719    230.69102    232.74029    227.21588    222.37676    239.10553     230.2037    230.56343    234.65691    237.52911    244.94832    238.83042    251.02338    249.41704    257.52455    256.71282    244.06365    261.34651     253.8328    244.27219    291.61129    287.26543     285.7587    277.10754    279.79282    313.97127     293.5974    313.74766     339.6648    314.59929    330.76292     325.2504    333.99249    359.05107    361.16093    388.73282    344.08447    353.48808    352.16559    400.13564    458.90682    433.28122    424.99621    437.60358      391.966    614.99828    636.08808    624.15998    627.22464    573.29113    603.33918          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       14.00    262.85243    241.76876    235.15231    232.74718    217.61132    222.56309    220.93855    217.74754    201.18928    199.68952     190.4903    194.75318    191.89389      185.671    177.78282    177.53891    183.59969    176.26271    172.53589    179.22337    172.06072    167.63713    163.18028    171.76419    175.40614    204.89534    242.39904    251.14704    235.76119    198.33634    167.03714    162.81853    158.71263    155.51011    154.52769    151.41736    149.98617    150.78854    147.53126    151.50587    151.08328    143.15872    140.66585    146.39263    146.67091    142.96678    145.80873    142.13549    132.19027    143.18747    133.46975    137.14647    139.57344    138.30614    135.97679    137.32829    143.38519     144.5848    146.98887    149.73742    145.34228    143.76315    136.38823    137.94372    134.20981     136.8389    133.92976    134.14547    132.21152    139.90631    157.86095    174.82943    183.66224    181.47041    162.69516    155.60997    169.28263    187.51056    192.69273    182.60813    163.86847    150.62581    144.72472     145.0481    140.00204    134.83781    132.39686    128.14605    134.78785    145.86793    165.15154    182.31702    185.12622    172.58739    152.84709     135.0223    128.05477    125.12602    124.51222    126.31692    123.26377    131.37227    130.82599    128.56764    125.26752    122.63202    119.82245    115.90671    119.41955    120.12895    117.99582    118.24475     117.0662    114.83967    117.28735    113.59658    119.95737    118.74281    115.75186    118.93982    116.13757     117.2968    120.18085    126.54705    131.13244    137.17964    138.46371    138.43767    134.10542    129.28137    126.18072    119.73006    122.88589    118.33455    122.46271    128.67558    133.91963    143.48436     159.2659    167.09815    170.91441     165.5509    155.58918    146.12383    136.49577    134.71176    130.11139    133.56728      134.095    129.51893    128.23404    128.09095    118.16013    116.98264    118.58818    118.23621    114.53601    118.41122    117.86451    114.44579    118.86217    119.83606    120.70893    119.63296    122.89147    125.51784    121.57017    131.51845    124.76226    126.73131    125.16377    122.81665    123.84119    125.73503    128.92651    131.82952    135.19998    136.97824    137.71603    134.22559    131.98065    130.62374     130.2105    125.80679    125.35808    123.03893    120.19401    122.81372    118.96317    119.28388      121.479    123.13963    119.46863    118.87716    121.06724    121.17722    126.95476    127.52619    126.86999    131.87136    129.93495    131.00962    130.38065    126.47543    120.76849    123.56606    124.57704    126.84526    126.81578    128.43457    125.61291    125.95275    127.62687    128.38457     123.0176    121.37268    120.60353    118.11797    121.94514    120.44458    122.55387    121.20676     124.2287    123.29829    121.95444    126.02961    130.32036    132.41105     127.2955    128.66809    126.94195    131.46322    132.09506     132.8162    133.97289    134.66321      137.284    136.78188    140.39936    140.41814    140.77534    138.22761    137.47192    136.21251    134.47228    136.45403    134.91172    134.18395    137.91379    134.99763    128.67646    129.78789    133.48783    134.74891    132.20105    133.50223     135.5499    136.39985    133.49829    137.60749    140.11168    141.49856    139.93949    141.05235    146.56464    146.43793    146.60756    146.89575    145.02888    143.17686    139.94624    139.03596     136.5904    135.60863    133.05789    133.00982    134.55127    133.57859    137.41182    138.50076    135.30781    136.27769    137.70729    134.89821    132.63843    140.28691    130.38448    131.08922    132.25756    129.25616    132.51091    131.64845      134.518    135.33577     128.6136    130.06805    134.16427    130.72766    133.61349    129.27057    131.86926    130.26502    133.32276    130.92712    127.36044    129.45136    130.11374    126.24382    129.92426    131.28299    127.80441    128.59314    130.51309    127.59388    128.02596    131.38499    132.68395     126.9928    129.99149    127.52622    128.27456     133.5244     127.4866    130.59225    129.75215    130.12006    128.79799    130.34177    130.65308    136.21539    136.46943    133.92147    135.99218    138.28445     138.4425    141.59379    136.43867    140.62331    142.43205    143.13373    144.21665    141.86018    140.64253    135.97271     141.7241    138.82385    143.97428    148.08319    139.47648    144.23866    143.96695    143.43502    152.16712    146.71075    149.98636    149.07716    149.01283    151.82427    153.25239    155.75834    156.70444    152.23022    159.77583    158.12061    161.09478    158.85023    158.96578    158.76941    161.49003    157.55277    161.69343    165.16829    162.25882    159.52257    159.98472    156.97144    161.83907    163.87835    162.99148    166.40673     167.9182    166.29394    160.88593    167.23805    170.71231    166.40563    166.16832    173.26393    164.18112    172.99044    168.23564    173.75016    178.68661    171.05323    174.47386    172.69157     175.8718    180.25709    183.56381     182.9997    179.01002    179.71431     189.0819    187.20625    189.50454    183.64017     187.8305    184.81761    196.40864    190.70983    196.00666    192.02709    195.21734    191.21089    196.20985    213.04432    206.82005    200.80314     197.3879    198.76365     204.1398    210.58231    223.45104    210.32599    213.72271    201.43983    206.19892    216.39828    206.42957    224.73214    206.89215    217.99575    220.11142    223.38694    216.53031    220.62619    218.49726    227.81181    216.80059     232.3435    214.24341    235.41925    248.89293    236.67792    230.93267    230.23325    242.15059    257.77129    231.44844    246.48469     245.8085    237.40884    261.78925    260.24756     259.7949    264.66739    265.62447    262.16092    292.30264    306.77634    264.43495    298.79523    273.67877    284.81016    291.47787    301.45134    318.79055    298.65395    295.17614      298.098    324.12847    362.71968    327.78891    350.69329    347.57254    379.83334    370.21776    449.81196    406.04187    387.97976    507.49948    408.10257    515.00357    558.09619    577.25008     610.0084    663.38404     489.9474    545.04792          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       14.50    263.65846     243.6321    234.48864    234.94214    219.35832    220.56082    217.96018    212.81914    203.64732    198.91768    191.55704    190.92561    189.78631    189.07201    176.46097    173.13504    184.00486    174.96851    172.27859     174.4029    173.46622    172.63578    165.07379    171.12359    174.79302    203.67362    243.64888    256.82025    231.38345    197.12373    166.77879     157.8287    153.83056    154.78158    150.38641    147.09706    148.04048    150.34177    149.74647     149.2066    150.67855    144.88632    145.93405     147.0799    145.49933     139.7132    143.71916    141.73963    139.02751     139.2751     139.2265    141.10234    140.29225     141.0899    135.23696    136.72228    140.93616    146.93774    148.74252    147.36734    145.28376    139.17814    138.99422    137.02511    135.48708     134.6613    130.26809    133.91549    139.29717    142.88017     160.1533    175.14593     184.7103    183.09576    162.59141    160.36336     173.2828    201.52015      215.545    204.01243    180.53889    161.31337     148.3692    142.60971    137.02483      135.949    133.12443    133.19468    136.64227    145.09996    163.02909     182.7766    186.08678    173.46697    153.05433    138.24615    130.15877    128.25751    127.61709    127.36963    126.99105    131.23228    129.80916    130.13263    123.41762    122.29916    118.26369    115.00933    119.56611    121.67595    121.05871    123.11068    118.55702    116.96134    115.21333    117.68504    119.01945    120.24105    119.98309    118.46684    115.77374    116.84915    121.14492    124.90161    128.20597     136.9947    141.21572    136.28043    134.24093    128.63447    123.27711     121.2078    119.11482    118.44317    118.87841    126.38716    136.98159    147.40177    158.82371    170.96577    169.78657    165.51754    159.56627    142.87637    135.47556    133.81198    127.17045     131.2235    132.13547    129.63482    125.99759    122.35939    124.48441    115.54279    116.57527    117.81821    116.66532    112.10917    116.50362     118.3907    117.54401    120.86769    116.02108    120.54622    119.17915     124.4997    125.29833    130.47532     130.7993    130.73519    131.41181    125.45676    125.80942    128.26514    128.47619     133.3336    135.02687    133.12001    133.95938    131.76733    133.16805    131.09972    126.99004    123.70892    125.03286    121.24898    120.89044    116.62398      121.326    120.83769    118.23892    122.77607    119.06964    122.04305    121.39181    124.08023    127.39362    126.25814    129.38387    132.94841    128.92812    128.29081    129.36612    128.29716    124.82458     122.2134    123.18253    125.41415    131.44583    125.74744    129.58385    127.72686     126.6836    126.42556    121.62649    121.15006    122.85622    118.60296    118.36052    118.06728     124.8139    120.75497    124.12064     123.0745    124.33625    125.85021    124.42441    127.50156    130.09776    129.34953    130.83699    132.52038    129.07932     133.6371    131.54524    133.50405    132.62499    137.54319    137.56258    140.79438    139.78033    138.54051    137.33098    135.82307    132.17931    137.31074    135.88276    134.37221    133.70685    136.98727    134.07703     138.0294     134.9471    135.90727    133.99592    136.42957    131.73668    136.68138    136.09493    137.82423    138.02808     136.6363    141.38136    143.35506    143.37296    144.62469    146.72621    145.88218    141.99264    137.40246    139.41057    136.28803    137.30344    135.30292    133.54634    130.95766    131.91513    132.58182    134.81817    135.12813    133.77205    136.36513    133.36228    136.38348    137.84892    138.98555    135.93306      133.046    132.59823    132.04821     131.4935    131.69599    131.36639    128.64419    130.41879    130.17912    129.23461    131.35338    130.72349     128.9374    132.95908    124.07841     128.1343    126.81566    127.20571    127.72677    128.04132    126.99189    133.37411    130.21852    130.22292    129.48966    128.49379    124.43167    130.29313    131.54981    131.57049     130.1076    130.70519    129.28162    130.34695    132.54062    130.26038    131.62573    129.56542    127.42499    130.93674    133.70511    133.17946    132.65212    135.80443    133.36127    132.03939    137.94228    136.58917    135.54137     138.9686    137.09071    135.87486    141.01251    138.66207    141.27353    140.19706    142.43007    143.43874    139.82926    144.63372    142.33515    146.19875    141.52542    141.85833    144.53908    148.73991    146.84631    150.16971    150.69824    147.43085    152.45334    155.91419     157.7829    157.33098    155.49247    154.57142     153.2311    158.65795    163.27767    159.91082    157.36678    158.87631    158.25891    164.63727    158.73906    161.04562    161.75374    160.02618    157.03255    163.13845    155.42892    164.02036     160.4116    167.02453     155.3904    166.73616    168.58531    169.35182    169.73644    165.54208    166.95103     169.3625    170.92994    179.98645    168.95267    168.67031    174.28714     173.1809    170.28667    174.64302    177.86032    175.46187    177.50878    178.88359    179.01875    188.93311    189.16911    187.81173    182.61236    189.83221    186.68891    189.12759    193.37268     193.4019    193.34333    196.12546    200.28216    193.12222    201.47151    192.76512    200.21092    207.61361    210.03704    193.46741    200.92313    205.89596    205.64776    204.97181     210.3781    204.19848    216.64114    217.08802    217.64234    215.64671    218.85662    220.72475    225.92534    222.43152     214.5907    219.49518    220.41973    225.26136    213.27753    243.97105     224.3254    230.39966    229.66051    228.26308    232.59754    238.79003    249.59091    251.24103    236.56471    242.53377    232.60039    260.03604    266.28177    276.35633    257.95216    264.43358    264.45116    275.37808    289.01918    283.35853    277.27824    295.88381    318.64536    305.08241    337.20243    353.62174    330.07753    334.85449    307.95516    309.80681    338.86813     396.9128    349.40823    374.51072     386.2233    343.50484    430.69989    426.74956    431.73909    469.68936    484.75794    452.14378    433.22838    530.25574     580.6127    700.43867    545.10734    662.27612          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       15.00    268.74005    245.69264    236.43511    236.16627     223.5009    220.27396    216.10646    211.20948    205.47565    197.75611    196.78152    190.70403    189.13695    189.76812     180.5935    174.29357    175.63794    177.14786    175.71199    173.89983    178.22112    172.33972    165.61361    170.47409    179.22714      202.703    238.16107     254.9147    232.19257    193.82382    169.04549    158.59175    158.82761    154.89203    152.23187    148.91657    151.43195    153.77874    147.06037    148.07338    149.89803    144.85195    146.59398    145.14089    138.83738    139.89959     144.4135    145.55874    141.11994    138.87045    138.05553    140.49178    140.60639     139.8576    137.83316    138.03013    144.23805    147.05535    150.04139    144.76826    145.05537     137.4517    140.49115    137.39492    133.95707    129.86997     135.6983     135.0449    133.20766    139.38564    157.27526     175.0797    185.65818    176.88642    166.08745    158.25284    181.62134    209.64445    229.51652    219.81834    198.55833    173.21096    155.65961    147.27037    141.91412     132.9411    130.18591    130.97692    134.70054    149.01244    165.88459    181.88559     184.4105    173.31453    152.85105    137.28532    133.13902    132.11305     135.6001    129.59403    126.47284    129.45846    132.85094    126.62639    124.76992    121.27055    122.97964    116.95361    119.71097    121.99855    121.72223    118.88124    118.22832     118.7614    113.95024    119.66715    119.79758    117.80991    120.59053    118.33905    119.21443    115.49607    121.21546    121.93666    129.46099    137.59924    138.17857    135.49042    133.46873     128.9443    124.71051    123.44002    119.55133    119.83467    118.82309    125.71786    132.99289    146.73566    161.49291     171.1597    169.91092    168.55992    155.88718    146.28544    136.13849    132.10249    133.32206    128.08393    126.64938    129.31297     127.8373    123.36031    123.08722    116.60047    116.50051    112.26993    116.22161    118.30145    119.21641    118.09313    115.66682    118.79645    118.47336    119.62192    122.31336    125.18253    127.70776    131.39639    138.35422    135.92404    134.20074    131.34144    128.36385    123.60675    130.31336    132.67666    134.39394    136.19706    136.78528    131.72162    132.43824    131.79684    127.87111    127.91632    123.01038    122.41593     117.9227    119.78036     118.7289    119.51439    117.95584    119.21635    120.59443    119.51544    124.07451    121.64922    128.58433    128.45818    129.35264    130.25059     131.0782    130.32148    128.82416    127.61032    123.84031    123.96068    122.32842    126.19602    127.23209    126.90042    129.42961    127.45967    126.93897    125.66719    121.69507    122.10917    120.60954    119.21343    120.63184    120.24387    119.79233     120.3365    119.89111    121.15124     123.1368    123.41632    127.37077    129.36311    127.81592    130.73795    129.45147    127.86201    129.37721    129.59145    134.40821    134.60931    134.88875    137.39726    135.82939    137.35751    139.69637    137.83269    137.30696     137.9944    133.36091    139.31502    135.91383    132.68835     136.2669     132.5822    134.50897    135.35122    135.42643    130.36156     135.0538    133.73291    129.96272    135.98773    134.99927    134.18633    140.54313    142.74405    137.75601    139.88186    146.04376    149.21714    148.51155    144.27744    140.07909    139.18795    141.83884    139.93849    132.77993    135.61777    135.82383    132.97082    131.66243    134.86269    132.30705    135.37155     133.8696    135.23562    133.05248    136.23781    134.22137    137.65981    134.30551    132.33114    132.26176    131.91458     133.9962    132.51225    130.52124    128.19677    132.70258    126.01866    128.78626    128.81033    128.96594    131.97808    130.71838    124.87652    128.12958    129.62311    130.77856    129.22934    127.71048     125.9488    126.28332    130.71526    127.45576    129.18171    128.05385    127.59314    131.98663    135.92905    131.20822    129.93304    131.86751    132.89777    133.61404    129.34447    130.85939    131.89332    130.80166    127.37053    131.88765    127.46012      130.684    129.68183    131.13924    134.79355    135.00403    136.42267    136.68928    138.85143    139.24123    139.17313     138.5787    139.16034    139.14492    142.63839    139.43261    139.89153    141.07364    137.91882    138.43422    143.79341    141.30925    139.22821    142.80402    145.42974    152.84619    149.13709    149.73758    146.07267    150.35606    149.41998    153.90358    153.46101    157.09553    154.70591    153.82725    156.45863    154.52157     153.8822     161.6557    161.59363    160.05073    154.06787    159.51853    160.41878    160.71752    160.13899    158.56407    161.69206    162.57581    162.40354    163.32843    162.60964    156.74994     161.4825    163.77834    168.25286    173.21075    171.45055    168.33002    169.28029    166.26375    169.17979    169.88253    170.75299    175.91755    174.57244    174.96955    165.86615    183.87358    173.36999    185.45842    184.92576    187.43404    182.26178    189.47388    187.94846    187.00576    181.47087     191.0694    185.84585    186.57263    192.54937    195.71796    197.69686    191.66053    202.29759    195.25871    206.91106    198.25449    208.00799    196.46317    198.75846     203.7636    216.06905    213.90603    219.02743    205.88326    213.57387    213.74698    211.76618    225.54045     223.1166    207.87117    217.88731    203.68045    217.41656    215.32137    224.60776    224.30454    224.00322    214.50175    226.20425    242.77331    239.15342    236.12027    234.68391     227.0286    248.16044    246.60817    243.69451    240.69502    252.46251    257.93244    248.11292    250.86843    242.77095    263.31005    274.40664    259.49098    254.92027    280.89719    275.24667    282.42497    292.53837    290.58356    293.76826    311.59389    353.19939    303.61872    341.59632    328.10385    310.94676    347.91169    374.37166    352.36121    376.60804    343.48769    382.49485    373.72969    474.92065    394.91625    450.81499     457.4799    459.84339    337.04669    623.92263    441.45944    600.83753    616.58244     536.2984    476.76434          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       15.50    272.99978    248.66075    238.10483    234.51811    225.99257    220.94204    215.92885    213.76746     204.9753    199.60324    197.62399    192.70517    189.87451    189.33607    180.14379    179.89553    175.16175    181.65713     176.5798    173.78654    176.49067    169.34642    165.20901    168.72225    179.34512     201.8436    241.62983     254.0751    230.47747    194.13863    169.04701    161.20041    158.59926    155.89178    153.32477    152.27307    148.90352    150.74363    148.37893    152.34598    147.08184    142.29249    148.51017     142.7843    142.90205    143.32643    144.95138    143.37056    139.64821    137.97839     137.8442    141.53796    142.65591    136.66601     136.7671    137.11345    137.04721     145.8188    152.01635    149.55304    146.46974    142.98634    139.74907    135.76618    135.93076    133.46019    131.67374    138.18568    135.08995    141.91746    157.86264    175.88171    182.92875    179.96437    164.58093    158.07283     182.1449    219.76081    237.03457     233.4392    207.14761    178.52796    159.62153     146.0873    140.59463    135.67781    129.10746    129.09016    133.32906     147.6887    170.15478    181.60697    186.31682    174.55542    149.72955    137.91555     136.2237    133.86398     135.0111    129.37985    128.79204    130.65267    133.97803    129.29394    125.48096    124.26524    119.73451    119.66278    118.86459    118.01527    122.63248    121.92579    117.13075      117.253    114.44836    116.16153    119.62947    119.25415    117.49737    120.76613     119.2446    114.78971    120.86664    124.27981     131.6289    137.96919    143.72469     137.6165    132.30946    130.61175    125.21214    121.08472    119.58506    122.69415    118.84809    124.74826    133.92169    146.03391    161.65068    174.71832    174.57859    170.63862    160.33542    146.20512    136.49147    132.33127     127.8317    125.98372    125.71779    127.84494    123.38141    124.56656    119.45133    115.05611    116.15865    118.61623    116.55842    116.20344    117.18464    117.70219    116.22805    119.75245    118.28137    121.89797    124.20374     124.0451    127.14701    127.77615    136.09226    133.31555    130.86773    129.36298    132.78374    129.35604    133.56376    133.26921    140.09752    139.61718    133.13682    134.24918    130.34689    130.90698    127.36818    126.66656    123.01059    121.37552    120.78673    122.16797    117.79644    120.70645    122.53103    117.41299    120.18354    119.00676    120.65863    123.90407    127.14046    126.19403    129.02676    129.96924    132.16368    131.22133    127.04098    128.34819    125.08761    123.59958    123.97468    126.53181     127.5181    128.67003    129.41434    124.39646    123.96496    126.35523    123.10334    121.85152    116.72435    119.24484    119.12659    119.38864    122.89258    118.12776    122.69519    120.43883    125.34038    124.07953    125.26108     127.8445    130.70014    127.27191    127.65933    128.01448    133.40202    131.49117    133.85648    129.85588    137.61188    137.50047    134.26763    138.80221     137.2648    137.45655    138.85648    136.50381    135.12343    136.56926    135.89574    135.11934    131.89623    131.78063    129.40296    130.87547    131.85851    133.35741    134.65713    135.27109    132.17931    134.27554    139.84117     136.7845    138.24467    137.17678    137.99236    138.68102    143.85033    144.27925    146.14688    146.02501    144.54966    139.81657    137.24967    139.37078    136.91943    136.66231    136.22248    134.38571    132.98905    133.44491    132.80464    133.91326    136.59015    134.21638    134.03063    134.01939    136.88293    135.58645    133.68034    129.28728    130.77897    132.98841    132.80719    130.31965    132.50887    130.08117    131.91493    125.73964     131.4307    130.59684    128.69865    127.62344    131.65898    130.51167    130.79438    126.43083    130.36998    131.26173     129.0639    129.62202      123.561    126.75976    129.88805    128.26176    127.23616    131.03948    128.67476    131.98324    133.25564    129.00826    129.18168    131.13895    125.70512    127.34468    130.54593    127.53576    130.17125    125.42427    129.17791     129.8385    131.46769     128.9633    129.76821    130.24942    132.50683    133.32641    136.27849    140.41576    137.51199    137.62517    142.53863    137.83315    137.98454     137.8454    139.35142    139.81935    138.18262    140.54841    143.93083    145.52752    144.21019     143.6354    144.14794    140.43333    149.94514    147.50139     149.5198    147.02278    150.26621    150.19566    149.51532    154.06197    155.61399    157.20559    156.88583    155.37089     158.0143    156.55774    155.78059    153.39562    157.77125    158.09524    154.54671    158.48472    148.52674    159.59185    156.11252    160.84478    159.12454     164.7767    163.74662    165.83669    158.81232    161.56534    167.25104    159.64013    173.95326      167.473    172.65708    167.61296    169.59402    170.32758    174.16705    173.09984    172.54086    172.90415    172.64228    170.14263    168.81665    176.28998    173.82564    188.34817    176.80212    180.21781    186.93009    196.83026    187.23535    183.00576    186.44263    191.17236    190.25334    193.70571    198.09072    196.00351    184.30521    199.59616    193.26911    213.06569    205.77094    196.95151    196.38718    197.03728     201.3337    215.54492    212.33216    213.81597    207.73846    200.09871    206.02945     220.6971    210.37053    215.94183    208.84431    201.81735    206.74068    226.52574    223.33728    221.57937    226.00567    217.11416    209.57994    221.74143    236.92078    241.97928    223.27611    239.13416    229.55019    228.49924    258.11372    243.89127    245.84956      258.716    260.24636    245.63362    266.60744    229.46242    269.17839    267.24653    268.72358     255.9327    294.54499    277.92629    287.65585    287.04755    292.51181    288.38889    328.28418    304.46124    299.54088    364.09624    335.70286    309.13731    328.03426    361.99376    411.42334    368.08144    374.83735    369.54755    369.11774    438.64354    383.79672    450.85909    478.23533    456.54842    438.97935    604.88962    580.80679    403.20377     596.8139    631.35242    711.97062          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       16.00    271.81485    251.29998    239.85126     232.3351    226.66801    221.11823    218.95306    215.96191     201.3432    199.64989    191.61042    194.76977    193.32037    190.27679    177.43303     182.2155    176.07752    179.52774    176.31148    174.40081    176.14653    168.48233    167.69079    171.21584    177.17966    205.40593    241.34188    257.91524    229.42625     198.1263    169.02902    162.28567    155.33371    155.71062    149.53405    153.73898     147.2812    144.11046    156.45522     151.3978    148.21034    147.61056    144.20134    142.15941    143.90767    143.88652    141.30123    146.60125    142.71576    135.16564    139.53451    138.69347    141.20022      134.634    138.57664    139.73226    141.99697    144.92453    145.65753    147.56841    143.75883    140.63987    132.18698    136.35704    136.95068    132.75007    133.87323    133.11738    135.75103    142.13328    156.76549    175.30779    182.98193    180.07369    165.45476    159.46143    187.27633    231.37209    248.91875    238.32159    212.27408    179.51083    161.65124    150.22517    142.45465    135.52444    132.29328    133.25285    135.74596    148.45526    165.12261    184.97281    187.74315    173.59044    156.27674    139.72931    135.30575    133.41682    133.43974    125.97077    130.42233    128.74841    132.06409    126.19918    126.11296    122.14585    117.43387    119.07105    119.92797    122.28721    119.04652     120.1266    117.78975    117.44304    114.93462    116.03898    119.94638     122.1094    121.65701    119.81791    119.77828     116.8657    116.08892    124.69115    133.32293    139.53027    138.73427    138.46446    135.98343    129.22283    124.52444      122.258    119.73824    120.69406    118.49077    126.01096    132.69245    145.31893    161.42321    171.79187    171.43035    166.73421    151.61327    143.27756     137.3662    132.30282     130.4531    127.38303    122.15063    125.96766    123.91086    120.86231    116.55893    117.01425    112.77296    118.50597    118.76915    118.67372    115.94857    117.56079    114.82194    117.83885     117.0094    122.59265    119.16896    123.33228    127.98319    125.28797    129.75238    132.46045    131.96572    133.90558    134.50015    131.76898    134.28592    136.62736    135.62452    138.64601    137.10218    136.49396    131.89043    133.99264    134.32177    128.90227    128.06624    124.29594    123.09169    120.72535    121.58307    119.52003    119.01842    118.39145     121.6589    116.81911    124.96018    125.50417     125.1241    125.61603    130.67256    130.16675    130.22094    132.50147    126.87638     126.2177    128.35208    123.18317    122.53215    126.70483    127.42658    129.30613    130.16732    127.60853    121.48338    124.35702    124.45586    125.25612    119.53438    115.45738    118.64744    116.44596    124.40745    118.78987    117.31957    122.81491    123.22045    123.67249    121.55052    130.37656    133.41843    127.79792    128.57198     127.7769    125.63657     132.9841    130.52368     133.3108    135.69324      136.704     134.0746    139.62427    138.73441    136.07958    138.36127    138.29927    135.30427    133.28036    134.45956    131.96893    131.20236    130.99588    132.05121    129.21229    130.96006    133.50875    133.70002    135.09997    133.60318    133.31253    135.51405    137.49493    136.80469    136.89654    139.39032    141.36749    137.45089    143.25869    147.79625    142.44352     141.2698    135.32672    137.21816    140.05946    135.99463    134.49604    133.32502    134.82132     133.5618    135.15846      133.709    132.63272    134.85022     135.1877    132.18813     135.6965    134.41769    131.95568    132.70863    134.87694     131.2039    126.45688    131.79476    130.48172    128.59227    127.06534    130.36946    125.04217    127.04974    126.90716    129.53832     129.6802    127.48702    129.59333    127.85535    129.09743    128.97367    129.61537    127.20425    127.05099    124.44054     134.2001    129.76548    130.40438    125.36522    128.98872    130.58344    129.58107    128.70886    129.67373    129.55859    127.54265     127.8311    128.26584    128.17219    126.70984    123.84446    124.78007    127.58545    129.87779    131.52297    128.96312     135.6846    133.88682    132.63761    131.99075    139.23368    137.12113    137.68083    137.68113    139.26756    137.62206    142.12001    143.73113    141.56455    139.94279    139.42804     132.7471    137.17779    139.21026    143.05027    142.92726    144.72621    144.86457    144.05412    149.81201    148.99829    149.21687    152.17293    147.18435    152.70111    156.07797     154.5852    153.46842    157.13753    152.54414    152.08771    158.38732    157.59784     158.1106     156.3176    154.48544    154.80414    157.19506    158.79001    155.11063    158.59486    159.94204     161.0597    163.45481    163.19382     161.9819    166.00093    165.62045    165.12163    169.95544    166.81047    168.04158    178.58649    173.52634    171.24151    160.21462    175.46401    178.53455    172.02713    171.97389    179.54226    175.68751    178.30342    181.42792    179.06428    183.68574    185.95001    174.62986    184.40436    190.61796     184.9948    182.77323    187.84227    189.06623    197.70116    192.00502    202.70872    194.26875    197.92876     201.6481    190.33823    202.39994    212.21561    202.82546    203.95602    200.17294     203.4468    213.12471    208.54282    207.95643    209.25488    202.01667    211.92027    220.12355     219.0248    215.70289    215.09783    209.32799    209.69563    229.20496    221.25912    223.17444    229.03689    225.13223    228.57877    224.23599     226.6845    244.21783    226.70674    238.63813     230.2381    240.20237    258.90025     256.6104    243.82102    239.87207    220.91696    245.93616    291.02322     263.3649    271.48379    270.22695    258.73676    261.89411    287.19872    296.14558    292.24101    278.31293    305.21296    292.04432    327.86804     294.1153    372.08356     330.7578    322.25967    324.72671    336.59254    380.98868    368.07698    393.45471    408.79853     419.6821    464.30283    367.29341    437.67836    474.00572    510.13868    474.65593    503.49729    656.92296    609.25849    595.13088    502.00634    600.39237    693.54274          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       16.50    268.63817    252.58157    241.67625    231.37392    227.02097    224.79713    222.89288    212.59488    197.86495    195.28617    192.40305    195.13175    193.19038    191.55843    182.68769    181.85526    173.59533    177.96035    170.85379    174.66763    179.77165    168.38747    170.66864    170.53984    177.01888    206.29859    236.63137    257.78636    230.86692    197.73041    167.32124    159.76355    156.17132    158.25827     153.0761    155.07134    153.13184    146.53523    148.26071    149.81004     148.9241     149.8641    142.77943    146.95602    146.29914    142.92014     139.4544    145.06986    140.71856     139.2364    138.44466    142.87286    139.58496    140.50317    138.38437    137.37924    143.58624    147.38533    150.01753    151.40311     140.6519    143.33922    137.26645    133.02667    136.02539    134.94546     133.5807    134.76589    138.67226    141.84908    157.30604    173.62393    183.46726    181.92504    165.18515    162.99488    194.78458    233.95734    256.14331    239.96623    212.57464    180.34975    162.51136    155.63015    147.75726    138.18631    133.05013    130.96272    137.58473    150.74883    164.62788    181.91191    186.57228    172.33546    154.20925    136.54195    133.67604    128.53701    127.37214    125.56021    130.16157    129.27061    130.05194    127.99496    128.84567    120.78453    121.50579    119.89262    112.50868    118.17495    121.28801     117.2785    118.65464    116.60236    119.05956    114.74913    117.25775     119.2654    118.89873    119.88435    117.31249    116.60914      119.832    124.95088    130.10603    137.69751    141.09728    134.41558    134.53318    130.32675    126.69936    123.44553    118.73482    119.69824     123.3816    122.75352    132.88593    145.33229    161.48951    171.41144    171.10687    165.12167     154.6945    143.81776    134.39227    127.93404    127.74191    127.64966    126.75515     122.7071    124.02656    121.63357    118.05278    117.16968    115.52101    113.23846    115.56944    115.93565    117.01109    116.04148    116.77301    114.43189    119.37643    121.59577    119.28209    123.21186    123.76531    126.05685    129.46278    133.35575    133.22248    134.45639    135.11989    137.66968    130.32061    136.95419    136.00598    141.31701    141.25469    139.08105    135.78463     134.1397    133.77898    130.91882     130.2678    129.27313    121.51835    122.95701    123.50253    122.70955    124.23976    121.94498    123.07686    119.86464    120.26242     123.3328    126.70135     128.3232    130.14255    129.74529    129.48718    130.07726    128.41518     124.5279    122.19591    126.21566    121.20085     121.4845    128.87147    127.25051    125.75849    127.45836    124.39311    120.70196    122.20153    121.75824    120.13383    116.46937    117.76512    118.25448    122.41681    122.06907    119.50783    122.24946    122.61635      121.256    123.87643    123.62117    124.79162    128.12022    128.87927    127.51335    125.85574    132.17259    130.81284    133.88463    139.34829    137.07101    137.32136    135.25681    137.51548    138.53017    137.49321    137.42735    134.05959    137.75938    136.75971    132.63835    130.89747    129.13719    132.73312     129.3079    133.97958    135.20395     134.1643    133.94302    129.18895    130.55856    132.19223    134.35445    133.62262    138.53736    136.89463    140.91506    140.29598    148.19482    145.68677    145.54128    142.06138    141.23915    135.83147    136.93349    134.83824     134.7785    135.30039    135.95451    135.38365    134.24524    132.21897    131.79552    130.79506    131.72157    133.70809    130.09642    134.07443    128.72779    132.57218    132.60725    133.15728    128.80623    130.65474    129.31685    129.77605    127.70469    130.03628    124.35904    124.68807    128.29966    129.39366    128.03007    129.23323    128.26387    123.11891    125.17783     123.6874     127.5375    125.45561      128.118    126.89489    126.13623    126.21264    125.89577    127.85714    130.96374    127.18615    129.16422    130.48502    130.15004    131.18978    130.18103    128.07493     125.5776    124.44508    126.88325      129.314    124.63214    126.83544    127.66306    129.36535    126.38487    131.20587    131.69978    133.03676    129.84833    139.59276    139.65545    138.64577    139.94954    140.45265    134.12778    138.76046    138.18726    140.92378    142.96411    138.71336    140.55317    134.23999    142.28011    146.90973     143.8999    145.10488    147.95672     151.8295    152.59845    152.37108    150.72131     151.2266    152.21966    149.22453    158.20348    156.24304     156.8649     158.5378    154.60764    154.00972    161.62201    160.80558    153.33336     158.5901    149.45666    156.44912    157.10764    161.95874    164.75378    165.43027    158.14179    160.16896    163.35004    164.27438    162.70843    167.30194    168.98174    164.59221     168.2765    167.49316    169.17709    171.57832    172.67094    175.96271    169.56105     170.7363    176.93022    165.51041    180.39681    175.51668    177.25877    182.65355    183.30734    185.00644    186.25573    183.44293    180.05093    193.87151    183.73862    188.53775    179.56178    190.11555    187.59444    189.77243    200.33297    198.82315    197.70378    194.44931     195.9222    197.41692    204.40154    201.21268    205.84462     199.7465    200.26826    200.96898    214.21728     213.6967    213.21671    215.96381    212.19284    208.42632    221.00994    217.30469    222.64816     221.3824    215.83102    211.77275    218.14745    225.89069    226.62079    219.72607    225.38367    219.66474    222.35173    227.48337    239.92449    224.99861    241.01851    247.80521     239.1551     253.8438    254.84872    249.64271    254.91368    242.32729     250.4653    267.11491    272.34167    270.26032    270.03436    270.81725     272.8679    294.03918    289.74783    300.94796    292.25519    287.49085     300.4794    354.94745    336.77153    334.44727    330.84997    329.78461    316.59666    366.78598     353.6803    349.83986    382.81025    399.60151    435.75369    390.66698    418.92783    513.96457     445.1544    510.49842    478.14204    582.04887    724.60814    750.30401    799.37958    563.10031      709.606          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       17.00    265.92351    252.89864    242.85005    231.79737    225.86337    230.03096     220.2029    209.04756    198.64929    195.16165    196.42984    190.48094    188.19211    190.61736    186.93688    177.45061    174.92744    179.88204    174.62395    174.15029    176.74129    168.13462    169.01071     171.6591    178.80828    207.21009     235.5429    255.80388    231.58845    196.46586    166.51482    155.42595    155.00313    156.54013    159.40249    153.25074    151.27646    151.77751    152.48882     150.9643    147.77994     153.8647    143.35474    139.34356    145.03724    146.30295    139.91202    143.66256    142.02671    138.54547     139.0989    140.96127    137.66715    138.19629    135.67286    140.26261    144.13772    145.21825    150.52415    150.18853    145.20142    142.46238    138.45599     138.6626    137.64863    135.85354     133.1677     140.6068    137.63961    144.82668    160.72653    175.05687    185.49061    174.04164    165.06166    164.88142    193.62422    231.97673    248.24518    237.22556    205.95535     177.4352    171.26586    162.19241    151.16611    139.29752    135.27019    134.54968    135.89993    145.77229    165.02601    183.04372    183.85637    172.07272     153.0871    134.68509    128.81125    131.38107    125.61042    130.00521    129.82935    128.94005    126.10967    128.05589     123.2847    121.62912    115.96503    120.33161    119.08878    116.87173    117.74505    118.26774    116.54835    117.95654    113.92343    116.18951    118.92647    120.58953    120.07637     120.7101    120.62019    119.16797     115.8437    122.75432     131.4805    137.14721    141.13694    138.57169    131.99424    128.12512    122.93262    121.55197    123.37884    121.24441    119.24078     123.4391    134.10885    148.83183     160.5197    172.68377    174.10586     161.6732    157.12486    144.66232    134.47059    130.28185    128.82734    127.53307     125.9521    124.50297     123.1176    119.33253    116.59876    115.26161    112.87905    114.35039    117.68324    118.35946    118.19015    117.00869    117.83434    116.40963    117.52022    119.41543    122.56072    123.11022    126.14927    127.20441     120.1951    131.17197    128.66638    133.01316    131.28925    131.34732    133.31766    135.88664    138.62546    137.94182    140.01903    139.05664    140.69444    134.45464    132.99568    131.97706    127.57826    125.62631    123.15163    121.27146    126.42315     123.2332    122.67432    121.21951    120.03453    118.58826    122.27302    125.09379    124.53077    125.82018    131.06546    129.12417    129.08462    131.29136    126.20961    124.12879    123.49429    121.34629    123.89947    121.43678    124.46308    128.12874    128.12793    127.52697    124.52421    121.85158    123.95854    123.30451    120.13221    116.92557    118.40307    118.10754    116.25872    120.00246    118.53435    119.39308     119.4774    121.60644     120.8318    128.51333    128.15249    130.56999    124.46356     127.2532    127.46071    129.11242    132.63216     132.5561    136.64391    139.40163    138.09108    141.62338    142.33169    139.55864    137.42475    142.28417    136.55563    134.44243    139.12682    133.40289    129.08349    128.62355     123.8049    133.10599    125.83966    133.05092    131.46268    131.23023    133.16055    128.70716    127.89082    134.08398    134.11086    138.81602    138.44586    141.92694     140.0157    140.88853    142.76078    143.61578    140.45058     139.9439    137.62411    135.45751    133.46454    138.32382    133.41679    135.45363    135.39137    133.42052     133.6688    130.75344    135.32033    130.51179     134.2134    132.45745    132.61934    131.80652    131.31929    131.52509    127.99878    126.00262    129.17021    128.00007    128.48981     127.6764    128.78697    127.10429    125.27251    129.46713     129.5411    126.22911    133.16603    129.57955    123.13243    126.80238    130.83963     126.8107    127.64756    123.24417    126.46135    129.96317    128.32435    129.01495    129.16953    130.61762      126.858     127.7575    129.82872    130.39253     128.3541     126.0683     127.9171    125.61524     125.6099    128.50238    126.23531    125.97915    127.37207    125.32883    132.48826    130.34346    128.76011    131.01507    130.06912    131.78402    131.62238    136.54081    139.41588    140.19327    136.33462    136.56502    139.31202    139.06368    141.30052    139.00603    138.67316    139.34893    141.05547     143.3455     143.0402     146.1913     142.0551    144.88387    145.66838    147.36344    149.63781    150.78721    154.86417    149.81627    151.39509    156.62821    151.44605     155.5808     153.7942    154.19097    155.89827    157.93031    159.51218    159.52121     157.6583    157.15581    158.63801    160.19737    166.40666    162.88888    159.81947      163.298    159.97625    163.76911    169.29285    168.30869    165.20372    162.69818    167.41647    161.52064    166.94575    168.25001     173.0462    171.79381    164.81335    168.33829    170.04457    175.49899     179.9032    171.38079    184.61258     182.1471    189.02272    189.44165    177.30131    177.14876    182.98005    177.95356    186.90581    190.23909    191.89088    186.61356    189.00282    187.39796    190.34978     199.8061    192.47045      200.386    193.14046    199.95045    195.22012    203.93252    210.49445     209.7126     205.2018    205.87974    205.67361    212.11436    217.25198    207.90999    209.10074    220.62413    215.97536    228.04148    214.87682     208.6876    228.98862    219.95499    216.87268    230.48624    224.18567    222.00976    232.40743      221.919    235.18073    220.54228    238.46635    235.03914     240.3782    232.62184     243.5432    247.19993    258.17273    267.76236    245.72061    232.13369    236.63145    254.25666    269.10178    259.97395    270.62491    266.16414    272.37805    273.84224    315.74878    306.60275    286.75118    289.28878    290.68115    310.90998    320.16229    320.43497    344.93237    328.32664    331.50913    333.17746    403.74493    377.68787    410.15436     417.2533    411.84028    422.54288    464.55143    470.82967    524.75736    435.98319    488.92503    499.69157    546.14947    662.14204    595.49053    576.36405    559.67413    560.58923          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       17.50    267.17323    254.38526    241.60416    232.81489     225.4882     229.4406    217.04714    212.68767    200.00938    199.45781    198.21784    185.86848    190.31479    187.02347    186.89168    177.41062    179.53987    178.11345    181.07799     176.1467    173.84172    164.68259    168.35077    173.99621    180.84513    206.80249    241.18719    251.05819    231.35895    196.25156    168.47512    157.28086    157.60896    155.14458    155.94688    151.11875     151.3546     152.9447    147.34811    148.31787    148.54279    152.22384    144.70925    144.23984    146.25381    143.51756      141.853    141.41023    143.76593    136.43818    136.38594    137.86983    139.42377    136.74787    140.33963    137.54558    139.15659    145.76871    148.36272    148.68633    150.26714    140.34641    140.25314     135.2674    134.74482    133.97115    135.01159    136.63173    135.61465    146.10105    159.37179    177.07863     183.8976    179.10718    166.86787    164.96611    183.54742    218.44416    230.94204    222.02004     189.4823    174.60636    173.89899    171.80169    159.26033    146.66836     137.4312    133.77203    137.67708    142.95709    165.43968    178.39819    184.21908     169.8626    153.34216    139.00435    129.64847    126.90625    127.86193    123.31999    126.58829    130.41002    126.48258    128.56136    124.97162    123.40684    120.78974    117.04528    119.28212    117.39196    119.66629    120.32501    119.50749    116.01058    116.86523    118.48045    118.42256    120.39691    119.47097    116.36336    117.57814    117.59695    117.06725    124.57658    130.07576    135.13386    143.39665    137.52897    135.87676    125.68174    124.20715    124.77828    119.94249     122.3957    122.92152    125.74077    134.49813    149.73547    162.10115    168.46754    171.87582    163.20656      151.213    140.15246    136.16357    130.96833    124.25846    124.75219     123.5211    119.25564     119.8675    120.71618    117.50083    114.66789    116.45453    115.11974    113.66403    116.46519    117.89991     115.8043    118.24612    119.21449     117.9861    124.25774     121.4893    124.79893    125.09315    125.43577    123.94399    129.29586    132.31693    129.81286    131.93783    130.60962    133.03364    135.35851    139.34201    144.43526    141.81318    145.22672     142.6707    135.51215    136.84478    136.23393      128.972    123.94407    124.34113    123.13021    116.55938    121.65965    123.35674    118.22247    119.31537    120.16214     121.2686    121.01024      121.619    128.17171    127.75916    127.84126    129.06386    126.23309    128.06645     124.9221     123.7443    122.98082    120.00441     125.4679    124.44077    127.81893    124.50723    129.98261    125.59034    121.84818    122.74406    120.97474    117.76458     122.2696    119.15606    117.33884    115.61744    121.22331    122.82001    119.32438    118.82849    120.64818    120.97257     124.1633    126.89439    128.48978    126.10886    125.18479    126.79384    126.09187    128.99479    131.28257    140.86441    142.91782    144.76187    139.75057    145.57194    144.88363    143.21904    143.25542    140.93021    138.21179    131.31768    132.74115    129.38618    131.03834     126.3789    127.27716    123.61656    128.22889    129.87159    130.92313    128.88673     128.9672    132.16178    130.70616    133.10575    132.89383    137.63505    139.42328    143.21769     141.5388    144.16552    142.45067    142.04197    138.11652    138.25967    136.57512    136.19103    137.12619    133.84023    134.21328    131.83705    130.30511    131.17152    132.75572    134.33317     134.2674    135.38883    136.02458    134.83153     135.6468     134.9645    132.53207    130.97362    125.69578    124.85982    129.03004    128.01876    121.80405    128.02613    131.28746    128.56656     130.5928    128.22554    126.79208    123.50435    127.40042    124.51264    126.45886    124.53055    124.88477    127.54689    126.86894    126.03712    124.07913    125.35417    128.17311     126.9632     130.1139    128.16195    127.53748    129.37511    129.67978    129.42841    127.39914    126.41577    122.51666    125.81614    129.16864    124.02638     121.3491    126.29867    124.16551    125.57875    130.49587    130.22606    127.66807    130.06961    133.13686    130.86851     136.5963    136.37316    138.81282    141.08818    137.08792    134.61229    142.70272    138.36578    142.31524    140.37743    140.52991    140.08756    144.10409    140.53907    145.16908    142.21827    141.63517    147.44646    146.86606     149.2958    148.34038    150.18968    152.92589    153.57479    152.05514    156.23496     156.8035    155.92321    160.19603    152.43327    156.25659    159.90778    156.37275    156.03634    160.34881    155.92537    157.78391    159.73956    161.56697    150.96901    158.71296    153.98928    159.50001    168.07873    167.95513    166.49818    163.90293    164.17418    166.71882    168.86878     161.0998    167.13968    171.20817    167.94963    169.09065    180.12685    174.16291    173.69988    172.25978    172.69844    180.03241     178.9845    181.50429     182.3346    183.66416    182.50684    183.78865    185.47238     193.5642    184.93519    198.31649    188.45536    195.29732    184.02768    196.68102    200.40998    200.99416    199.75714    196.84678     192.2427    203.58562    205.55171      206.832    195.09703    203.79662    210.96808    217.42776    213.81408    218.04754    219.78605    219.24538    214.21996    229.25979    225.77409    214.86637    218.23188    220.66801     217.6371    234.16904    234.77375     225.8568    224.81973    224.20199     229.5705    249.54461     242.2656    247.86793    232.23704    239.62557    239.67077    252.99161    257.98121    261.02867    243.98451    250.26841    243.38905    254.35513    265.80543    283.02547    289.18946    274.33286    277.73135    277.92304    309.00449    326.75602    306.86115    299.90227    331.10194    305.46903    371.96629    376.22044    380.12866    358.15711    350.44997    366.48821    352.72057     444.2954    419.15886    416.82905    397.65679    438.23651    464.43994    534.66619    544.20303     522.5356    476.22131    517.84591    778.99225    664.87771    686.73313    643.24761    699.43956    731.92067          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       18.00    270.82419    256.99173    240.88097    231.90864    226.41119    223.97034    216.99626    213.63032    201.53246     201.4639    200.34293    188.14247    193.99107    190.42309    188.15589    181.60766    180.99553    176.71416     176.9699    176.40293    170.45746    168.32761    167.19544    172.89731    178.85455    206.98321    239.51371    248.49674    230.07622     194.3413    164.61872    157.02159    160.22683    153.93722    155.85339    155.89814     150.6802    152.49624    146.72078    146.59796    148.89624    148.39964      144.519    142.72643    145.34081    144.67695    141.03176    142.75158    145.68421    141.48866     139.0292    139.32858    137.61424    141.58254     137.1646    137.83391    141.48078    147.76032    146.92173    153.97514    144.87934    138.44507    139.71397    136.64846    134.38356    137.58383    134.66481    134.54896    140.09285    146.31828    159.09891    180.26968    185.90385    178.37436    165.29491     163.1532    175.68997    194.84228    206.46149    193.10698    177.30294    169.29356    176.89996    176.73279    165.22311    149.55871    138.30879    134.74766    140.54213    145.58522    166.47077    180.10595    184.73792    174.24001    153.26267    137.67989    127.32959    123.09164    124.09055    124.00764    125.76133     129.2403    128.53401    127.75417    123.33496     121.0993    119.80294    117.07858    119.61356    114.45863    119.33834    120.50798    117.15872    117.92109    117.26451    113.78051    116.61196    120.36391    118.68036     118.9403    116.03601    119.70141    119.60627    122.22636    129.01207    136.42318    142.05934    141.63589    135.65344    129.75305    126.04177    119.35625    120.24815    120.10513    121.07489    125.58558    134.58257     148.3099    163.54255    170.20897    172.43903    165.17448    150.08894    141.17705    135.37457    132.67411    125.46171    125.37528    127.15664     123.3858    122.96498    119.90944    116.39884    121.18039    114.94127    110.30697    115.08512    117.14651     114.9991    117.83891    117.90667    118.28424    116.76643    120.54968    125.38821    122.65647    125.45215    126.16093    125.12434    129.38908    130.54821    130.88103    134.48633     137.3591    135.15668    130.13818    138.07816    139.91377    144.55306    143.55818    143.26983     135.9959    135.17377    134.50183    128.12244    128.30026    127.68329    124.29224    120.17289    120.27683    121.29935    117.61013    117.06061    117.95078    118.94458    121.91701    117.80485    124.12191    124.80239    128.53237    132.99521    125.97279    130.39122    124.35185    121.57039    125.33465    123.56344    124.24102    121.02154    125.43712    126.49567    128.04807    126.24677    122.02896    119.67438    119.23867     120.1449    118.19322    120.56452    118.74504    113.17721    114.09908    120.28066    120.33248    121.75637    122.50313     124.3654    123.36593    127.00748    129.22969    126.90467    126.72285    127.99927    125.93829    127.24356    130.39037     136.9555    137.07341    141.35147     143.2371    136.65166    144.25433    143.48422    139.35919    139.05704       137.71    130.41619    133.33387    129.08004    127.71689    128.10136     126.0306    124.52292    126.81789    128.37023    130.98819    128.36823    128.85358    127.24516     131.9334    133.12625    135.64138    137.82094    140.85988    144.84664     139.8183    145.26565    144.54276    136.75477     138.8451    139.96798    129.80484    134.46852    131.05528    130.65842    129.28917    129.51879    127.76759     129.5297    130.52677    131.53479    133.52862     135.2765    135.46779    131.27237    137.37997     132.5631    130.49588    131.27359    130.86098     128.5388    130.15143    128.60825    131.63511    129.15903    133.52135    130.88817    127.36586    127.94316    125.37753    125.52025    126.94039    121.92481    123.08004    124.76249    123.16365    124.67316    124.74082    124.76965    126.49652    130.14235    130.45089    129.12712     128.4815    127.65283     125.0896     127.2832    125.83049     127.7306    130.94828    126.31363    127.00667    127.79543    128.79987    128.06537    125.22708     126.5735    122.53815    129.27957    128.05166    127.66783    130.37768    130.68202    127.85052    130.47245    135.31379     134.0319     135.1507    138.61579    136.40178    133.32638    138.84632    138.65571    136.62797    143.79523    140.96908    140.69721    140.56469    143.85927    139.87991    146.35303    143.09617    142.63707    145.98528    143.12712    151.11494    154.66804    146.35516      151.265    150.52703    155.01363    153.18496    153.80111    154.43962    155.85539    157.51423     158.5208    159.87884    155.66242    152.41495    156.09451    155.84163    157.92428    161.66493    155.23849    160.19583    159.72903    161.88662    164.93495     163.8103    165.59769     165.3977    162.08258    162.03952    170.27913    175.41861    175.53275    175.40394    170.73129    172.19674    180.19863    181.78696    183.70379    180.16937    182.02077    174.69198     184.5474    187.74007    185.45443    184.73299    176.93984    186.37419    186.01067     191.5469    194.89493    195.85451    188.22369    195.12845    194.36319    205.12732    192.34573    194.91692    198.42526    201.64077    203.83456    206.39102    210.13361    210.71131    209.87974    202.13777    210.32741    220.86619    209.38235    214.61246    218.76684    219.14429    218.84543    222.87307    225.80447    226.73577    235.42741    224.26776    225.25256    234.02788    232.57672      248.131    234.80713    246.72107     237.8358    241.59779    246.72206     245.6679    249.06984    248.78029    259.72399    253.73774    268.75778    276.38466     268.7737    263.70431    262.04712    273.51048    283.47007     298.2609    290.60341    295.69471    284.23047    290.33422    318.98197     310.8666    328.21676    338.71619    326.00438     302.5043    350.78913    380.39371    334.04838     343.0798    365.00341    352.20421    398.96762    429.39444    432.77832    404.63688    436.97257    435.62014    539.38924    525.48416    566.29282    580.20496    530.08817    542.31719    698.46014     763.5995    750.56998    797.87176    671.42366    662.39424          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       18.50    275.52799    259.98504    241.16132    230.36854    226.43893    220.62044    216.95895    214.17434    200.84515    200.83952    199.44608    192.31275    190.70994    193.85044     188.5442    179.01739    179.94502    179.48856    172.19663    177.15485    173.67838    172.63689    163.93753    171.56349     175.1295    205.17271    237.39645    251.03774    235.50927    193.09036    165.50108    158.69157    156.96395    150.97713    157.41875    149.48789    150.44459    151.26863    147.74206    145.48367    148.31586    155.86099    145.92277    149.35395    142.83248    143.02058    137.78724    141.31444    148.93216    141.26087    141.36436     141.5357    140.10466    141.28839    139.79371    138.22914    139.04616    148.58422    148.84205    150.34983    143.84898    141.26604    141.39182     138.1576    136.20186    135.46518    135.79455    137.94925     137.2858      146.106    159.99239    176.62335    183.99881    178.30992    160.44333     157.3992    166.34671    182.36355    184.83066    177.29975    166.18202    167.95377    176.06299    178.78504    167.69509    151.70782     134.6496    134.55152    135.25031    149.26567    166.62304    183.62477    184.45114    170.38412    152.03193     135.7054    126.61511    124.42511    125.32178    123.99876    127.77343    127.23034    128.35379    126.32044    124.29182    123.12923    119.75489    117.07523    118.27029    118.52618    118.56713    116.58922    118.66564    117.20642    115.54595    116.37203    113.10792    116.40444     118.7702     119.3024    118.36954    119.05874    120.92478    120.69095    129.11054    140.05183    139.54141    138.04589    130.03589    126.09357    122.38449    120.09769    116.79313    117.11636    118.64747    124.70076     132.1883    144.77956     160.4402    167.05376    172.63585    163.48991     153.4903    139.56024    131.06009    130.56895    128.43144    127.71069     124.9483    121.18128    119.87421     118.1386     115.0745    116.77286    116.11792    116.29803    110.05358     114.7526    118.33963     115.2586    113.38736    114.95461    118.19951    120.12244    121.74656    126.27579    126.69034    126.50087    127.31897    123.99684    129.33919    125.22811    128.85172    132.16922    131.28036    132.00022    135.27975    139.05508    143.07704    140.41258    140.37072    137.86728    131.82306    130.23583    129.54067     124.1387    127.24168    122.66691    121.65694    116.48139    123.16606    117.60202    123.90809    123.80655    121.28653    120.51954    122.48531    130.95829    124.45675    129.65363    130.40033    125.43156    130.48118     126.5227    123.00122    122.81221    126.30151    123.50723    122.10697    122.25237    127.44461    125.39501    123.00471    123.91368    115.29591    122.91932    116.10643    117.87514    118.49095    117.23695    116.94697    116.62157    118.00959    120.80774    120.46906    120.90264    121.29802    121.23569    124.85255    125.99207     128.4781    126.77138    126.85458    123.30695    126.19556    131.35876     134.4211    135.76085     134.7649    139.16844     139.8055     139.8005    135.82832    136.62975      136.677    131.67134    131.30954    128.21996    129.27978    127.68994    125.72414    130.30699    123.76403    126.73608    128.63229    128.82344    125.83367    132.45182    127.58459    131.07403    132.87587    136.37574    137.68088    136.46432    140.32727    140.30161    141.52714    144.61122    142.52758    140.64902    135.90767    131.30976    128.56241     129.3593    130.14053    126.35211    127.65647    128.48674    130.63624    130.05085     130.7975    132.47174    127.63613    132.22326    131.08019    133.33716    126.59869    132.33898    130.72819    129.70353    129.09729    128.34707     127.4154    130.14657    129.59209    130.26929    129.59956     128.3101     130.0318    126.60524    125.93887    125.13058    126.95199    120.47559    126.57558    127.60322    125.32359    123.46747     126.9662    123.16551    126.78594     125.1954     128.2415    128.54472    129.00785     127.9219    129.60592    127.98058    129.36054    128.12304    127.90988    126.69934    125.14189     123.7631    126.44544    126.71019    129.34579    128.23809    127.91142    127.24915    132.40046     128.5254    129.84387    131.04606    132.22697    133.98637    135.57812    134.13557    130.71933     138.0567     134.3562    141.40037    138.32559    140.76667    139.28732    145.24366    139.86476     143.7409    143.58536    145.18661      141.023    139.02075     148.8219    144.98757    151.87397    153.14864    150.50657    149.96819    151.69364    150.92581    165.20038     149.3118    154.61184    154.07175    153.70287    156.87675    157.15485    159.88594    153.35112    162.63444    157.37373    155.86362      160.212    159.62755    163.39202    160.26041    153.98129    158.42937    165.58431    166.17004    167.68635    168.83361    171.27859    167.81672    159.12714    170.52197    169.81401    173.58248    169.33901    166.19829    178.69472    177.51197    174.43597    181.63978    185.53946     180.5267    185.78583    184.56618    183.47614    183.53636     181.4783     186.5741    192.71613    190.59366    194.54512    198.49151    194.58595    192.77673    192.63725    202.15452    201.55564    195.17326    199.07851     202.0598    207.79732    215.38068    205.86398    210.04305    208.78645    210.86386    203.85998    215.94125    215.34987    211.46862    216.72698    214.42939    221.28325    229.64606    223.83589    236.26082    228.23168    228.58387    222.51543    236.11969    242.04327     240.3301     231.8102    251.41999    239.55894     261.9886    265.45546    236.12248    254.05725    246.13103    259.60663    272.11419    285.97499    259.01075    265.60529    259.39373    274.10798    292.78169    304.45996    297.63045    272.89207    294.74848    298.07932    309.23451     343.0419     320.6156    326.74639    328.30753    321.05727    322.54983    357.29853    371.10212    340.68209    362.01486    356.95336    368.51776    426.14054    423.60238    438.57486     442.5372    484.39791    483.24579    568.82244    509.16702    587.62261    560.58899    544.79896    546.74015    898.09285    874.64669    808.87934     937.1736    816.30653    889.65339          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       19.00    275.91422    263.13652    240.95095    230.14833    227.20287    218.14434    215.68462    215.85485    205.55409    200.56604    196.71205    193.29454     187.5921    192.63707    186.23761    178.00886    176.08675    179.88037    167.10603    174.17495    173.87151    175.04697    164.28081    170.40645    178.92422    205.36048    238.48382    246.99576    237.08045    193.02463     163.3104    160.75341    155.59684    157.07266    152.21521    153.84544    149.26308    148.42122     153.3229    151.84385    146.50378    151.74004    143.82022    146.20507      145.111    146.18088    141.85116    143.03942    141.08039    133.75514     137.3134    138.36894    138.46817    140.18221    141.81243    138.52302    141.04701    145.56447    149.79462    149.81705    141.50309    140.86236    139.01613    134.80722    137.15511    135.61654    135.98164    135.97425    142.24341     148.5987    156.78958    175.85437    185.98022    176.95295    162.86032    157.33422    160.37968    172.36619     178.5341    168.58398    159.62721    163.64624    169.17602    175.01837    159.05265    148.33346    138.68045    132.26052    138.82234     151.6352    169.44657    185.24755    184.17947    171.71531    149.60591       134.01    125.93806    124.64106    121.94442    126.71941    125.89024     130.3666    128.40131    121.22721    123.56248    124.45853    119.74855    119.84571    120.06971     118.7426    119.99615    119.97419    118.56574    117.35676    117.83291     116.7683    115.99181    113.57763    118.57809    120.82919    119.07358    117.35524     119.5351    122.93621    127.10518    139.51398    141.02711    137.04509    135.11322     126.2412    121.91352    121.63411    117.91129    119.62882    119.60547    127.79014    136.59157    146.54157    158.86179    168.25382    167.77964    161.90528    154.85874      137.527    132.11551     128.1056    129.32241       126.19    125.08334    122.44439    120.59421    118.78556    119.27933    115.53971     111.6953    115.08499    111.55753    117.34386    117.51336    116.14503    115.40842    115.74062    117.86276     117.2718    121.62745    123.41432     125.6171    126.09055    124.81361    123.73347    123.96324    121.89838    121.73099    130.01038     129.8161    132.25968    132.91749    138.26748    139.73353    138.89988    138.62543    135.83597    132.30012    128.28785    124.48253    126.60345    124.45354    120.13187    122.62626    118.26678    120.23353    121.78191    120.00167    119.37921    118.76591    119.63261    119.93638    126.55899    128.23553    127.79812    129.15388    127.44702    126.08013    126.02113    120.78992    121.32366    124.01308    125.39201    121.85864     124.2003    128.77335    128.06836    122.43105    120.17714    119.56608    119.76935    117.80566    119.86733    118.63248    115.85759    114.66291    113.75827     118.6633     121.0506    124.09715    121.91363    125.23944    125.33836    124.30348    126.08332    127.62158     127.2088    126.90034    123.56571     127.3098    130.67164    130.88208    131.75546     133.1852    134.90468    135.21662    138.13991    135.47965    133.44276     132.5941    131.40325    128.85049    126.29133     125.6972    126.26482    125.20438    124.79871     124.2754    122.97072    128.92866    130.47179    131.20669    128.84742    129.17388    127.97685    132.29284    133.53089    135.82139    142.46382    142.94827    141.82523    144.50711    144.09597    140.14997    141.06136    135.76403    134.77417    132.12826    130.43678    127.26752     128.1349    125.22544    129.57635    130.40911    130.45654    129.74701    131.18173    132.33992    131.17609    131.77799     131.3185    132.95436    131.61641    129.74697    129.19428     133.9231    128.08559    125.81617    132.14008    131.69604     126.1236    128.55119    124.15511     132.3743    126.48199    123.30463    127.46785    124.27975    123.84121    124.49194    125.98323    123.86485    123.29744    120.50079    123.87004    124.42025    127.35074      125.827    128.79152     127.4695     126.6883    130.29449    130.09469    128.68186    128.22641    129.19675    125.81817    124.29999    127.71895    126.66604    129.26141    126.76896    124.07095    122.50267    127.75498    131.57886    132.51104    128.17367     128.8492    131.07725     136.1841    134.01692    137.48352    138.47349    135.02716    136.07339    139.68264    143.32252    140.04455    144.51977    140.66771    144.45985    143.38442      143.462    142.38837    143.13762    148.45753    148.67881    146.62077     147.4324    148.79047    148.61227    151.01834    150.22049    152.91209    153.64691    155.55677    156.59852    154.75891    156.58689    157.32367    157.05234    161.57808    158.87848    162.65063    157.97218    158.11213    157.34259    167.28575    162.12102    166.60803    160.80829     164.8789    161.64487    169.86068    168.50591    169.52074    166.78897    172.02544    168.56986    170.67166    178.71268    175.46594    171.08175    174.00793    179.07351    172.29439    175.69131    171.93934    184.38359    182.15543    185.05959    182.52459    180.99598    190.12348     184.2326    182.44209    190.22781    192.32748    193.52985    193.05144    198.97089    197.94286    193.75745     198.2742    190.71381    196.99789    198.94329    196.65139    206.23906    212.45038    218.52851    209.94554    214.84872    218.18615    212.06414    219.78155    222.31938    224.10526     220.3229    220.24881    220.22153    225.13248    235.99994    231.58184    228.69372    227.23538    228.53925    244.78406    246.52618    244.50706    241.72252    249.53328    236.82155    252.40388     248.1537    251.14262     262.5485    263.35938    256.37274    294.62157    283.23842    265.40358    266.22433    266.42872    276.84037    286.15012    298.53958    310.19594    297.87546    313.12954     305.7972    307.85258     361.4923    329.59293    319.58209     352.9481    320.76748     324.9088    379.60473    417.21802    410.73311    408.93264    410.07469    416.72623    470.99552    476.71408    498.02836    449.36049    498.28705     481.5229    627.90718    607.20073    570.71528    597.43389    566.86168    594.47534    829.76172    769.86587    849.20133    928.81285    839.67649    870.12108          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       19.50    272.24036    264.38476     241.4813    230.97689    227.64196    214.53686    216.24312    217.42832    210.26786    204.67037    196.33118    193.52894    189.38035    193.36528    181.27713    181.83708    176.65169    174.42711    177.04035    173.07791    166.69036    174.52926     165.0744    170.54634    178.76957    203.71822    237.65509    250.29978    234.98339    187.81219     168.8861    159.15825     163.4183    157.81102    149.62022    154.16125    153.26668    149.97156    148.55615    153.09657    145.15257    152.61186    148.27255    145.77081    141.94418    141.81345      144.465    146.09883    138.71347    138.23047    142.16343     141.0644    137.78186    138.58567    140.26326    136.21763    140.72545    143.45933    145.03988    147.83106    146.60882    144.47092    140.41704    139.19602    136.66027    134.57763    137.53152    135.79292    139.92355    145.52589    158.94688    177.10727    189.36078    176.39419    164.31683    152.94598    162.22648    169.09671    168.54922    169.66799    164.02018    159.08644    165.43872    164.19905    154.64974    141.01933    132.01409    134.73977    137.63416    152.63349     172.5244    183.64933    186.46534    169.46129    152.11033     134.0415    128.71377    124.46651    123.52794    125.29143    126.08269    127.83076    124.84077    127.49103     125.4753    119.03203     118.2824    120.51192    117.55812    119.79698    117.77217     120.6586    120.35813    114.71299    118.81528    120.32773    117.22955     115.9391    116.76803    118.79806    119.91872    118.47274    121.51689    125.30673    130.78606    139.54427     140.3459    138.27928    134.88129    127.90652    122.47855    118.20052    116.40956     118.3932    119.89881    124.67645    134.70116    142.60015    161.83454    169.17307    171.40383    164.08956    154.89479    137.04717    131.36738    125.83343    126.84768    123.71866     124.8942    123.25583    117.15143    119.12602    119.39422    114.47865    116.04709     112.9851    112.32002    113.59301    117.64347    117.13892    117.80091    117.58315    117.89904     116.9739    119.21619    121.88382    126.71114     123.8676    122.87032    126.06474    117.20457    122.67272    123.64218     126.2807    127.35984    131.73966    131.46184    133.76721    138.39668    139.83028    140.76285    133.79605    128.33915    126.11238     123.7914    124.93671    122.45903    118.84255    120.34218    117.50245    117.99608    117.70678     121.9614    119.62178    119.49788    121.99175     120.1628    124.88064    126.38466    127.62958    128.04873    130.48348    123.99793    123.73995    121.03927    121.23573    122.06691    119.42084    122.07178    123.28241    127.59289    125.35998    122.59134    119.80719    122.50378    120.46436    115.64567    114.85859    116.50998    115.97713    117.23705    115.94783    117.73986    120.14081    119.99927    123.90605     122.0163     121.6506    122.29677    127.27287    127.90393    124.81278    127.00409    128.08335    125.21419    126.81467    134.43884    134.16491    134.03363    132.61187    139.11174    134.41606    141.23708    136.54832     132.0222     127.8753    130.31507    128.39033    124.52715    125.06192    125.64087    125.23103     126.9348    125.53868    122.74257    125.94389    130.36348    129.97376    127.52138    128.66454    134.38821     133.8563    137.27649    137.62337    140.93962    141.96167    141.49448    142.80706    139.19463    138.46022    133.50769    137.75399    129.54053    133.42137    127.95462    130.55901    128.84576    127.70452    129.20777    130.27643    131.22508    132.86225    133.73846    129.68135    130.53143    128.62035    133.09265    131.09443    131.48197    128.41759    133.12534      127.586    131.40126    127.29617    126.08395    125.18999    128.23195     126.9637    125.23388    125.41286    123.55648    125.83063    125.83948    127.45665    120.95604    124.00194    124.92949    119.82944     124.3624    122.88813     122.0614    127.16185    127.07301    127.40986      127.974     129.6259    126.29366    126.60125    127.37584    127.32395    128.56175    123.56584    124.71796    124.95879    125.65969    133.36232    128.42747    128.34748    129.49226    130.39782    130.84608    133.85635    128.32974    129.86796    135.66019    134.34439    136.33669    138.05642    137.97242      144.161    137.95979    135.14668    143.50875    142.61644     141.2313    141.83987    140.08534    142.67198    141.33955    141.97148    146.15432    142.91796    146.19975    144.00007    149.20608    148.39803    152.23801    152.55542    154.16385    155.94312     153.3449    156.90648    153.18233    152.50718    154.39047    158.87785     156.8033    159.96349    159.26553    154.27792    159.68122    152.10741     160.5737     161.0225    163.58173    162.22935    160.20821    163.39127    160.46418    167.84047    172.05909    171.16919    174.04279    168.39457    170.36751    179.31066    173.39848    178.57057    176.12787     172.6038    176.25659    179.03902    177.55645    179.12295    178.25821    177.17888    180.64593     192.4285    179.35311    183.73473    187.69269    189.98593     182.3951    194.14459    200.68251    193.87708    193.62654     200.6603    191.27149    201.98881    187.61546    202.48867    202.06094    199.70839    199.69285    214.88437    212.77572    216.09196    206.37358    210.10707    214.92123    224.99405    227.64363    223.97286    219.55218    221.65387    217.96423    229.43595     231.4834    237.19835    228.37223     236.1284    230.61792    244.93384    248.18271    235.03033    246.76468    254.32689    254.40358    253.83991    265.01289    262.15308    261.34195    269.14451    257.58379    279.05581    276.83778    290.55678    291.81674    280.42146    294.73232    308.98769    317.84223    339.33091    299.08731    320.58156    301.08338    323.04913     360.0839    362.17236    328.30176    328.63857    355.99994    371.71323    361.07121    411.69499    411.62835     383.6968    401.69416    398.20689    493.39306    478.37223    495.68354    447.11361    488.74257      468.018    600.49556    596.25588    577.23088    582.78398    636.55427    737.04525    933.50894    907.60124    879.84169    948.88862   1070.31867          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       20.00    268.78539    265.23727    242.38366    231.66322    226.66927    214.96978    219.15274    219.77146     215.3033    205.75936    196.97863    193.01878    192.10615    191.51901    183.07012    184.64951    179.60773    172.79138      181.658    173.47034    168.53043    175.93358    170.35676    165.15357    180.07867    201.99432    233.73081    251.54816    229.90006     191.9446    169.81279     163.6842    155.99752     154.6078    148.83735     157.9793    150.90377    149.03549     148.8937    150.24415    150.76761     152.4148    150.53746    148.24179    143.56818    145.11919    138.29656    142.42369    143.25093    141.42413    139.01096    140.07429    138.48428    139.83103    140.47085    143.19787    147.72454    146.17735    149.17841    148.52556    138.98426    143.80703    140.04123    137.70436    135.41469    134.65322    136.23308    134.56284    140.69677    149.98996    157.90196    173.67219    183.10696     175.1364    161.12555    155.17023    157.55571    161.31371    171.39428    171.81188    159.59839    156.85279    158.14901    153.86283     146.4901     139.5053    135.00375    132.99543    136.35487    152.39176    171.91921    183.90816    183.04523    167.34804     149.7328    138.64401     129.8593    122.48522     123.6126    125.16021    126.75777    128.77537    128.90925    125.84788     122.6183    120.92324    118.32107    118.50024    118.86425    119.56807    117.00186    118.20951    118.45964     118.2245    118.62876    118.98385     116.7681    113.21728    121.80321     117.4498    118.39115    117.50382    118.84855    124.14371    126.51155    135.72032    137.83859     139.4976    136.68572    128.53895    122.24687     115.2464    116.67041     118.3074    120.59937    127.86945     131.2022    144.81466    162.78791    169.30305    173.69752    166.50229    151.90496     138.0102    131.99329     128.9234    128.30196    127.24344    124.76369    122.26832    119.51836    118.88789    120.73146     115.2592    115.03175    115.85596    116.90272    113.54119     117.5317    116.50732    116.36247    113.78864    117.04311    118.92061    117.99432    122.78723     121.5837    119.32676    123.63301    120.76707    121.02381    123.31605    121.54813    129.65269    133.22695    132.51984    138.54778    135.74477    138.17941    141.47366    136.33249    132.69127    124.74377    124.02495    124.07276    122.19147    122.05685    120.75305    121.19345    119.69246    118.92066    116.66966     119.0528     117.2275    119.39584    119.76599    122.39742    125.75931    123.30326    127.47573    129.26605    128.52933    125.10882    120.84659    123.07006    121.05477    123.78093    121.89165     121.7415    121.53362    122.28342    125.39287    124.67123    121.97151    120.53804    115.40852    116.16948    117.47193     117.9073    118.67887    118.41841    118.45641    116.61596    119.04523     120.7468     120.6917    123.55745    123.98534    121.05877    128.26164    122.57127     127.6269    129.78227    128.80248    128.54256    125.05436    129.91398    132.73871    133.46249     137.1338    134.59948    136.98063    137.56663    135.77642    134.41256    130.57356    128.52081    124.42957    123.29175    125.60019    123.29681    126.95126    126.13265    124.52793    120.96773    125.30438    125.66221    124.72059    127.08714    125.84914    126.37598    132.57511    132.82507    136.47374    141.37691    143.87592    140.18406    136.79964     140.3807    139.72528    133.94646    132.76753    128.02873    128.26335    128.57131    130.02875    129.01955    131.23352    123.50158    124.57358     129.4257    127.34725    129.55306    130.17173    131.11557    127.20774    132.30521     128.0696    128.69914     127.7717    128.20303    127.28361    125.07026    126.75778    124.25493    124.15666    125.41892    126.10763    124.72381    130.42681    122.72921    124.62888    125.06269    122.41401    119.87523    121.89152    123.02292    122.98058    124.07765     123.0476    124.26729    126.96779    127.05317    125.43174    127.05437    129.79961     120.7115    123.81906    126.49853    126.04495    127.15493    127.91961    125.07768    127.23662    127.57571    124.81923    122.12937    127.95559    125.28913    126.16069    127.42487    130.63968    128.09058    132.49235    130.66068    134.56071    134.74028    133.68987    137.30371    136.15982    134.70315    138.49247    138.95445    141.56185     143.5171    142.51761    140.91618    143.13946    142.46891    145.32546    143.31794    142.11764    140.69973    139.65509    154.76228    148.84501    149.23356    151.05677    151.35242    147.28431    156.01397    154.55368     153.5085    151.75534    157.41656    155.24822    157.47265    155.76119     153.3489    156.41636    156.94314    150.60558    157.46858     164.2321    161.90661    159.08662    158.89644    165.68029    168.68394    165.62986    168.88185    168.14637    168.94244     164.4005    166.78763    174.24772    168.53971     167.9934    167.81968    165.00418    169.33228    176.77021    173.02369    170.89662    173.90425     169.5271    175.38557    179.97733    175.53485    179.88968    180.97236    176.97769    179.64546     191.7448    187.53229    186.03279    174.81781    187.38938    187.31717    203.32218          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan     308.2087    309.22511     313.8018    318.75471    328.29771    310.72243    337.57269     374.3018    357.85431    348.94206    344.61724    353.05136    390.26136    417.99606    427.46186    425.79158    426.86231    392.78107    417.56969    503.95027    519.42475    511.77307     508.0124    491.57646    529.86487    683.52559    628.80667    702.50986    656.89911    657.60776    823.98193    986.90508    965.38147   1072.57187    977.39461   1009.51559          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       20.50    271.46081    266.14533     242.6501    231.18922    224.53484    216.34407    219.69948    219.60879    218.40731    200.79188    197.97446    193.74977    196.14724     189.1925    185.67677    181.32918    180.01702    175.86395    177.44871    169.92533    169.20578    181.13847     166.1517     162.8841    177.41384    199.03274    233.94067    246.75021    230.06352    192.03622     170.6271    161.98294    152.82715    153.12955    156.32539    156.65214     150.5715    154.26748    146.02951    147.07073    150.36993    148.82351     146.8605    144.16404    148.46443    144.79366    142.12761    144.80466    148.49935    139.62088    136.72993     139.4935    138.82612    140.79513    143.03178    143.41832    145.25638    147.20611    150.02789    148.03968    140.61491     140.8196    140.97955    140.14005    134.33114    133.58379    134.86185    132.66679    138.69397    145.80382     160.2112    178.75328    185.15541     175.9037    162.61188    151.76954    153.38435    159.06455    168.53632    170.75351    165.36036    158.67957    151.83478    150.10751    137.86742    135.59875    135.46479    132.96869    138.35796    151.18352    168.12969    182.02161    181.38972    170.54055    152.29196    136.80246    130.06184    121.12749    121.43314    122.33297    123.95514    123.96832    129.98145    123.70552    122.40201    121.28458    120.22604    118.55292    120.72518    118.18643    119.22915    114.11433    119.61262     118.7033    114.87947    117.65659     117.5937    116.88596    117.79406     122.3728      122.423    117.97092    118.22316    122.69205    129.65974    134.20105    138.23253    139.23313    135.35006    128.03894    125.72578    118.74532    120.32144    119.42992    118.67944    122.53868    136.81634    145.97403    159.10551    171.46817    171.46195    160.95895    152.87726    140.99438    129.83193    127.50293    128.01032     126.1934    124.82305    123.83584    125.21212    116.08337    120.47348    117.56095    113.69294    117.88363    111.40522    114.95998    114.60719    115.06678    115.74602    117.37444    119.17411    116.60289    119.00152    124.23744     123.2503    122.39071     121.8801    120.72929    123.76947    122.82621    126.86135    131.75956    137.21971    136.35317    134.48428    135.28402    134.87102    139.11685    138.08536     134.4418    131.41777    124.63889    125.12316    121.56356    120.51257    122.22805    118.11324    121.47089    118.08731    117.07205    118.69569    117.70137    118.43362    118.21794    122.02836    121.65342    126.31577    125.88115    128.95261    125.14676     126.2332    119.76046    120.82937    121.88758    120.14793    120.87813    123.90853    122.89229     123.2544    122.14496    122.04196    123.06581    122.23308    118.90207    115.25426    119.57456    118.66799    115.67939    119.91884    116.81835    117.88043    120.07658    116.39985    122.51697    119.92233    121.84138    122.57762    120.13429    128.53893    125.71297    127.63773     125.9403    126.70246    128.28314    129.69499    132.63855    131.71331    134.64996    133.89291    135.75178    131.81811    133.84552    135.60369    129.24885    131.03958    128.61236    124.38875    126.30312    125.10816    124.41006    122.20762    123.99132    125.26681    123.40754    125.73208     129.0209    128.99835    130.72068    129.14601    131.79014     130.9847    133.10639    136.18027    138.88303    138.49352    134.18934    137.45704    141.58952    131.36503    131.23398    128.91232    126.94776     124.3957    122.85384    124.50828    125.71944    124.66944    124.36697    126.64443    129.75643    126.52403    126.90175    124.74905    125.09555    124.49397    129.71734    125.09197    127.86163    126.78487    123.94303    123.76957    125.33596    123.56929    122.80879    126.48714    120.77534     122.0943    123.77927    123.71292     124.3406    123.44477    121.53891    122.21861    120.11991    125.33904    122.54901    121.18181     123.1466    125.18058    123.84335    124.46688    128.32965    125.25843    123.38783    126.22856    126.43377    123.24278    126.41418    129.16928    128.68862    125.65593    127.67107    129.17948    126.04315    126.65861    124.82562     126.6826    125.97055    127.44157    129.86046    126.41828    131.75876     131.1612    130.70714    135.81258    132.49962    136.04109    136.11201    138.72156    133.15084     135.8695    135.06204    137.13606    133.69236    132.87977    134.35399    139.08889    146.00467    144.84424    140.99773    138.63664    139.80999    143.41021    144.21316    145.84873    144.66721    141.00773     144.2458    143.35137    147.67997    144.20773    142.84025    144.50502          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       21.00    276.07925    265.09172    242.23524    230.87476    221.98333    218.35263    218.75296    216.98581    214.55104    198.03928    199.42668    194.41298    198.40804    189.88047    189.24625    181.38335    179.96666    177.60147    178.87906    169.31031    176.26186    173.80963    166.18965    167.10758    177.48678    202.56386    234.67309    243.83739    231.68781    192.65193     170.0776    158.54186    157.50154    152.01474    160.38068    156.11726    148.82014    149.00928    149.98374    143.00774    149.92072    143.64509    146.62657    148.81438    144.79983     148.1934    144.56246    139.10578    143.60965    142.28562    142.03851    141.33613    138.85585    139.58436    138.66658    140.38004     145.5602    146.38217    148.68089    149.48531    140.17045    136.71049    138.07497    141.62489     136.6035    133.82083    134.42336    133.24744     136.6731    146.08524    161.64437    179.93033    183.55275    175.28414    163.10545    155.60276    155.00093    158.23608    164.17536    172.37302    168.90535    154.81311    153.87725    149.06393    139.06242    135.03706    132.28905    136.15243    135.75561    144.91405    164.79674    179.41471    182.73073     168.2064    151.19981    140.33948    127.31507    122.16415    122.30913    121.34018    124.34104    130.54598    127.46468    123.78115    122.41546    119.03985      118.968    118.49538    118.43918    118.29003    122.75343    120.78122    120.73678    117.55733    114.51333    114.58708    116.65884      119.961    114.60163    113.06931    114.65061    118.00042    119.53611    124.26366    128.72508     134.2229    140.13791    139.47035    133.45922    127.65662    122.67267    119.93885    116.99137    120.56571    118.37948    121.80277    135.07733    144.28517    156.60364    167.58496    167.90591    165.59619    150.34835    144.19387    131.69365    130.28998    127.92355    127.73659    126.06751    121.82035     122.3311    116.96651    114.25895    117.54263    117.86089    112.01202    114.20696    115.18371    111.83171    112.80471    116.43773    119.35828    117.99362    117.12766    116.44575      122.317    123.75253    125.82276    122.94774    121.09654    124.18936    122.22959    127.36595    128.20949    133.95509     133.7038    135.86007    133.28974    134.11932    139.44261    135.05642    136.15963    129.54083    129.41034    121.34191    122.08319    120.32343    121.89885    118.22784     119.0381    116.25219    116.84848      119.081    116.19629    114.82582    120.05725    118.82341    121.30087     128.3013      124.539    126.57684    127.40493     127.4133    119.66521    118.88183    120.96881      121.414    119.31976    121.14925    119.81559    123.28237    124.25456    124.12816    118.31451    122.54308    117.43015    116.60857    116.53446    116.49296    115.75777    116.15868    116.62513    116.26338    115.53122    119.88712    120.11161    119.85217    120.73077    121.21008    120.83898    127.42259    128.01156    126.30882    123.62454     127.1765    128.56713    123.90482    127.45743    132.54098    132.71486    131.55002    136.64096    126.54684    132.15846    136.13971    130.47303    128.50432    128.45569    127.49943    121.15571    124.67828    124.84527    125.11128    125.27483    126.10168    122.67913    125.49344    129.79934    127.02556    128.22946    127.05397    129.10621    135.93821    137.99555    133.67947    136.58477    138.28868    137.05012    136.38112    136.32883    135.83924    132.40628    130.47434    125.16009    122.98215    124.81965    125.14799    123.67269    125.73402    123.22064    120.00234     128.6273    132.21256     128.6794    128.68878    131.05886    123.94128    128.74208    126.28886    124.89292    122.89979      120.677    121.82772    119.44142    124.74844    123.12344    118.64749    121.28602    121.43878    119.79968      121.689    122.60514    122.12446    121.56221    122.15221    117.82093    121.43708    120.24471    122.81426    122.45756    123.33308    120.31433    124.72775    123.79791    125.00558    125.08391    126.22723    122.92762    127.75424    128.94761    122.74654    122.52514    123.91272    123.87822    124.10289    128.34647    122.12175    123.64512    126.59171    123.51417    127.30857    127.60168     125.1156    123.96684    123.43927    122.61784    128.24275    125.48283    128.43122    129.47521    123.90599          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       21.50    280.09767    262.33028    242.71497    231.13093    220.63486    222.97813    218.17577    214.90129    210.05373    198.18159    199.56041    194.94865    196.35441    188.21849    190.53383    185.99116     179.8759    178.21937    182.47264    174.62504    173.96865    174.27704    169.18962    171.81067    179.49738    207.06094    229.54371    249.17747    230.46482    193.11816    167.69178     164.5233    156.70569    153.27457    154.74515    153.36714    149.74662    151.22095    153.10346    146.95318    146.10959    153.71313     145.5849     149.9278    143.70729    143.12998    143.75329    144.41915    145.00233     141.9004     144.7855    141.16536    140.28848    141.83274    141.28049    134.76667    145.12661     147.6379    152.06718    148.00391    145.73115    138.95379    138.47958     139.6887    137.38887    134.93856    134.82515    135.62034    139.51765    147.29591    162.02457    176.57716     188.1923    173.31381    164.18064    158.10539    159.17396    163.10161    163.22031      167.672    162.88949    155.67246    156.84333    147.53273    141.09069    131.92737    131.30104    133.38021     137.3298    152.83868    167.26105    180.51935    181.25693    169.45032    151.22327     137.4466    125.43799    124.89404    120.91407    123.43967    125.22323    128.15925    127.31029    126.88254    124.13483     119.3857    119.84056    118.35046    117.59697    120.57324    121.74399    116.53995    116.56806    118.95794    114.96177    115.80909    115.37764    120.11344    117.08408    115.26173    116.90664    117.41467    121.23511    127.37229    134.92736    138.74649    139.72197    140.07498    133.84104    127.19725    124.41204    119.19961    114.89006     118.9189    119.26852     126.2183    134.15394    145.41021    156.66287    162.93608    163.79351    163.12952    158.17008    142.34509    136.99992    127.05394     128.0503    124.56378    125.57479    120.57358    120.91704    117.48769    114.91398    114.34232    115.56315    116.91346    113.14802    112.54586    112.80448    115.16799    114.12414    113.75954    116.69671    121.65358    118.92089    119.08126    126.95469     125.0496    125.42797    122.81523    125.13045    126.44289    129.86913    133.65264    132.58346    134.32462    134.38546    132.34111    133.11244    135.53689    134.13283     134.5194    128.41201    122.89334    123.33173    121.16433    119.90188    116.99412     116.0567    115.70449     116.5241    113.79035    115.70148    119.96238    119.57682    121.22517    116.82524    123.44474    123.71454    126.49416    125.40767    125.00464    123.25653    122.16146    119.39677    120.35316    120.49515    120.63881    118.94945    121.68002    120.76005    120.97752    121.26035    121.59301    116.30012     115.7278    117.99717    114.99363    116.49184     116.8915    116.86598    117.40619     116.1313    117.93749    116.77959    116.76447    119.26217    122.18753    122.33284    123.81879    124.47113    121.73274    124.51829    125.60845    123.86002      125.357    126.95367    133.07103    135.40921    133.73059    134.71448    132.77108    129.59742     128.6922    136.24097    131.27098    128.13375    125.58846    126.56312    122.81798    121.80958    121.65704    126.59035    125.51442    121.78285    123.67815    124.90918    125.18328    126.20884    128.46491    127.72038     130.0337     131.6554     136.1743    136.87336    139.84219    137.98065    137.60649    132.41113    134.20325    133.25363    127.40399    129.93128    128.42467    122.14384     123.3179    126.50442    124.33624     124.7832    123.37326     121.5599    127.20172    126.93909      127.236    129.66002    127.94893    122.82631    125.93872    126.79802     118.5914    127.54511     121.2925    121.57464    115.09807    124.88306     115.6007    124.33734    119.97577    118.98573    118.53388    118.74089    120.80917    116.31313    118.49251    117.83667    115.88428    115.85471    122.54878    118.88474    117.82827    118.90206    121.24441    122.19186    120.85558    122.27882    120.17013    120.33547    119.41272    118.61704    120.54994    116.83807    118.37395    116.21901          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       22.00     280.2284    258.17025     244.7804     231.9393    222.96984    228.47552    218.25437    214.06139    208.67565    197.46028    198.92746    191.60615    192.97956    186.87459    189.81928    185.24098    180.74461    180.75472    178.86046    174.13732    173.77624    179.82624    168.35085    171.61755    176.88576    205.37893    231.44335    245.68547    230.23417    194.44149    169.34636    165.20189    160.55932    157.65141    156.50933    154.31205    151.62032    149.26755    153.33473    148.09111     151.0874    148.06776     147.7957    150.57439    144.49376    144.76835    144.15666    140.07626    145.25256    145.39322    143.59758    138.26163    141.00667    141.73122    142.69846    139.62949    148.15553    145.05385    149.44413    145.17939    145.20174     139.6386    137.97924    140.46712    136.54911     134.2047    136.50874    139.25371    140.00911    141.87811    171.09536    179.51639    184.12004    174.65965     162.1366    156.86146    162.92739    164.71883    164.26268    162.69979    159.30835     155.5944    156.96484    157.70541    148.71844    137.81093    130.42601    133.38603    140.49026    149.89809    168.43788     180.1792    179.50585    169.73805    152.33417    135.97202    128.98788    127.17924    120.58636    121.99861    127.92327    126.95448    129.91681    127.54054    122.97681    119.05007    121.75755    118.21058    117.02491    119.60955    119.93755    118.12226    115.73312    116.15499    118.65013     118.2769    116.94871    117.83727    118.57184    116.21356    118.42105    117.40325    120.68092    126.34955    133.73724    136.71538    138.97421    140.44361    133.41472     130.0042    123.65429    120.47076    115.78567    121.45827    122.66391    124.13988    129.73208    144.43343    157.92343      160.546    169.34225    162.95235    153.77227    147.60697    138.12389    129.49107    128.21293    126.63812     123.8297     119.6107    120.96166    117.29135    112.43433    115.50824     113.9023    115.79329    114.53949    114.45779    113.72704     111.4233    116.47078    113.94653    115.31793    113.59227    115.72472    116.19594    122.08561    120.99356    123.99088    129.87246    138.47678    144.14562    139.94639    139.24183    138.09211    137.39454      133.651    134.80616     137.1879    132.91373    130.49684    135.69237    128.13586    127.67469    126.32293    118.31707    119.71274    117.55378    118.16155    115.04448    115.26932    114.70541    115.11201    116.66807    118.78358    117.98322      121.045    121.11021     124.2385    127.93547    129.56344    124.92672    123.43779    118.85537     119.0721    116.07739    121.01245    122.52162    121.78451    121.00547    123.03797    120.75283    119.17137    120.43985    118.47212    115.54092    118.77918     111.7826    113.55522    114.48029    115.12324    111.65122    114.29831    115.27834    117.55513    117.95066    117.92949    120.48365    121.06796    124.22497    119.06946    124.92712    121.09782    122.22642    125.56516    125.20107    130.25611    129.24818     130.7361    132.76927    137.00652    134.53938    134.58605    132.22733    128.32801    130.54529    125.97031    126.09735    123.42855    121.97831    123.04576    121.67813      119.664    123.65122    123.85305    124.55928    120.31569    127.77798    123.78248    129.25638    129.63532    129.93176    128.54779    134.74517    136.05851    137.19297    135.23065    137.70603    133.88388    133.25721    132.42614    131.45031    126.84138    123.02165    122.38111    121.89435    120.00127    123.98414    121.95028    124.07663    126.43626    123.35521    127.00061    127.08162    125.28476    125.26884    126.12398    120.90128     127.0847    121.90509    120.19017    122.50926    123.62523    115.26664     119.6771     117.7119    115.38475    116.69554    116.48948    113.72215    115.95629    118.58235    111.93309    117.22727    114.56458    115.78329    113.13332    111.85508          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       22.50    278.94555    259.42677    247.32074    232.58998    223.69821    226.14432    219.10271    216.34429    208.64125     200.6799    199.05052    187.61099    188.56815    186.94931    190.47036    181.37278    184.16387    182.77586     178.5259    175.20731    182.29684    177.66566    173.03565    171.35822     180.2614    208.70787    233.53373    247.30957    233.61427    193.64143    167.91363    159.76157    159.33976    154.19219    155.11886    156.59377    155.90222    153.02674    151.68704    150.44921    151.85223    151.52748    149.06158    147.51074    140.64994     142.1212    142.91487    142.89379    147.01194    145.02915    142.59954    142.63501    135.62502    138.60435      142.589    141.45414    145.60485    150.02125     150.6291    148.59685    141.83677    139.96902    135.93444    138.82607    139.61379    136.09119    138.38976    136.60823    134.57564    149.50555    163.40121    178.56315    181.20574    179.56569    163.30056    164.19634    163.57663     168.3949    163.08348    158.35609    159.55971    155.99556    161.86589    162.99206    154.20265    138.01662    130.43655    131.51466    134.59627    146.21964     169.1771    182.60121    184.19231    167.37046    148.58728    137.39504    127.29582    123.65569    126.91384    124.00097    122.34367    127.54419    128.36936    126.08996    120.07497    119.53069    116.17789    116.86665    118.76262    119.49996    120.02918    118.57585    117.44081    115.77683     115.1186    116.41926    116.77953    117.53413    118.52674    116.97053    118.12277    118.91846    120.16319    124.83385    132.57535    138.89333    138.50174    139.21561     137.7282    126.74733    120.62656    118.74834    119.01244    117.30297    121.19034    123.50609    131.00502    144.27757    156.32705    164.01471    162.57044    161.49129    147.54919     139.5584    138.53567     130.9675    127.59453    129.46256    123.73953    119.21751     118.9064    115.24492    114.45722    114.10791    112.40654    111.04615      112.565    109.88275    112.87976    112.97184    112.66207    114.10348    115.51133    114.51126    118.09822    121.98068    122.40079    123.22615    128.90933    137.96972    151.11656    153.75845    145.15995    134.56203    133.58547    131.18239    131.47182    133.47371    133.30933    128.79652    132.60644    128.95267    127.66819    127.41345    120.20913    122.59342    117.27435    118.36621    117.39917    115.95471    117.39219    115.25899    117.67392    116.90364    116.02263    117.16729    119.12308    119.70784    123.01521    125.04782    125.55165     125.3275    124.98067    121.06846    121.16246     117.1415    120.50556    120.37186    121.61935    122.39819    118.81943    122.74142    118.41661    118.09962    117.93741     115.1887    114.41704    113.20884    113.68316    116.63864    113.14363    113.54793    115.16175    117.03832    118.23529    113.58159    121.71908    117.11792    122.00947    122.26594     123.7286    121.52382    120.89446    125.41643     126.5449    124.60794    121.68039      124.863    131.95156    131.62811    130.84712    131.42279    131.63373    126.98835    123.80095    130.08603     124.7063    121.82096    123.32224    120.95893    117.68341    121.89042    120.64548    122.34701    123.90082    124.10111    121.34311     122.9043    123.56317    126.04062    127.11557    129.87235    126.67009    130.71021    136.67401     138.3668    137.21369    135.38576    135.17173    130.52233    132.89704    129.63858    125.36189    121.47441    122.07323    120.77214    125.16582    120.72183    122.63306    118.88167    121.30947    120.40587    126.16307    123.61646    121.29281    119.08139    123.66137    119.97108    119.43186    118.30833    120.45887    112.73884    114.25932     109.9466    108.45202    111.26504          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       23.00    278.23636    261.93584    247.37796    232.94266    221.38196    223.63576    221.25554    216.56646    208.93346     203.9085    198.29405    191.09823    190.50461    183.71301    190.36462    185.19886    182.34585    183.71337    180.36421    176.70754    181.60956    175.24095     177.2413    170.00844    185.50988    208.96332    235.92477    247.76191    232.23675    195.48266    167.78072    162.35421    156.53961    156.70352    156.39592    156.01334    155.12976    147.15208    148.33359     153.6636    154.79689    147.20208    151.01842    145.11315     145.6028    147.37164    144.58047    143.38463    142.31924    144.45579    145.82817    143.54931    139.91761    139.07085    138.31246    139.62046    146.09476     147.7171    149.57827    147.03448    142.70404    146.73306    132.69821    137.43022     137.6115    134.92486     134.4962    136.91345    141.84999    149.73765    162.49649    180.42356    183.91237    179.60975    162.81949    164.51275    167.46688    166.37043    160.54848    158.59968    155.54163    157.23674    165.22735    164.01432    155.91758    141.77905    134.55368    133.30581     135.2827    146.34028    169.56554    182.69742    177.39539    172.47589    149.19435    136.16632    127.52751    120.87026      123.397    121.04246     125.4292    126.05659    129.48936    123.36101    122.01131    120.16292    116.24551    119.26702    119.47276    118.01277      120.259    117.47541    117.36402    113.03922    118.26434    117.19502    118.27031    117.05096    119.36894    117.95763    114.91252    115.36859    118.95928    123.23268    129.72157    139.38638    140.10481    136.98141    130.36798    123.37857    124.06087    118.37236    117.78025    116.18381    116.96244    122.94689    131.01981    143.89609    159.00408    164.80161    161.76134    160.27247    151.04826    138.69661    131.73966     131.0194    129.02989    126.23313    121.43839    122.62207     120.9172     121.4307    118.51537    114.25014    110.79407    115.44845    115.54681    114.28132     115.5454    115.08205    115.36525    115.61879    116.60759    114.58886    118.23913    123.55433    116.76717    121.35271     131.2037    139.03044    142.64051    141.93744    131.04614      124.561     127.2126    129.18911    131.51314    131.52531    132.50751    131.10529    130.44615    129.93399    129.17605    126.64758    125.90775    124.71311    118.90313    118.44268    119.04555    117.89148    113.85259    116.93892    115.95819    114.62501    118.88225    119.00101    117.36214    119.98538    125.42482    124.85988    125.44504    124.74311    123.59365    121.03711    121.56138    116.99021    118.17494    119.74879    123.45783    123.76057    121.90947    123.30127    117.48039    118.30768    117.42666    113.38209    113.72671    114.38448    112.37912    113.58522    112.73568    112.45022    116.24343    117.03436    116.23369    114.76291    114.58391    121.16871    121.51693    120.75344    123.95369    119.62033    125.01766    120.75477    119.74884    124.93344    124.78834    128.26264    124.53329    130.02785    130.61088    126.09492    128.34663    127.15707    125.32768    125.30151    121.65105    120.35358    118.74054    121.59671    118.37649    117.65581    118.74532    123.68431    122.04022    122.17561    119.32438    120.90711     126.2689    125.41421    123.16205    128.64651    125.30887    129.74085     133.6863     132.3504    133.96056    135.24335    133.01535    128.92689    127.19987     127.3067    124.49558    122.71841    120.68361     118.3496    118.57551    116.23499    118.83715    120.86849    115.67978    116.36161    115.63401    113.77289    116.98975          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       23.50    276.52315    263.91342    248.27926    234.97637     223.6198    222.89323     223.8407    213.75208    209.56824     206.2076    197.54263    197.16933    198.90327    191.21853    189.52965    186.01541    180.22572     177.3878    182.35468    173.65716    175.12326    174.35813    177.64495    169.59176    187.38209    207.34668    232.54358    249.55287    228.65425    197.90607    165.56942    159.43852    155.40653    159.09837    154.12683    154.76411    156.45985    151.67559    149.78184    153.70941    145.18875    148.29858    149.15353     147.6007    141.36581    142.98916    142.77921    138.97927    143.39806    143.47952    143.26803    144.45894    137.47399    141.95066     141.8775    144.63352    143.83077    150.21978    149.89989    142.56237    141.67179    138.76058    136.55125    137.92096    134.82815    135.22214     136.7832    137.97147    138.61545    147.38378    165.11245    178.67879    187.78976    173.86376    162.57091    160.50126    168.39573    169.13481    166.35908    162.97904    157.59022    157.14071    161.81421     164.3857    160.10904    147.43808    135.28662    134.43762     134.5089    148.52054    169.11028    182.52215    186.42689     166.0961     151.5309    139.10636    129.26185     123.7119    122.82865    123.39861    128.04467    126.07956    126.71511    124.01036    124.19692    116.82312    115.21048    117.50901    116.19935    116.60562    119.83721    120.10794    119.43143    116.75667    121.11313    120.03936    115.97613    114.86266     117.8374    114.16771    117.27359    115.90202    118.53229    123.19194    128.67846    135.18186    136.47817    133.74237    132.79607    127.50228    121.87831    119.00728    119.40947    120.42275    115.34029    123.24267    130.82252    143.02329    157.43071    165.41694    165.38375     157.6306    150.32228    139.17934    130.13653    128.85472    127.17803    121.90285    126.71197    124.93676    125.12315    124.97191    121.32153    113.58967    111.16233    113.00437    114.62157    114.56346    113.27678    111.67562    111.64732    112.59911      114.327    115.41904    115.49979    118.19554    123.58353    121.99706    123.33452    125.96449    129.20576    125.67521    125.74865    123.27025    124.06316    124.81674    128.78917    132.49993    133.88503    129.28072    127.40465    128.23905    125.12553    122.20478    123.51374    122.86998     121.2055    115.97665    117.53081    116.30408     115.1316    115.01894    115.95645    114.61061    114.31458    116.73565      116.979    119.85172    122.96933    123.58311    122.87087    123.90441    121.12341     120.2761    117.50559    115.57951    116.33151    121.60335    122.43543    122.79793    121.63799     122.3301    118.69521    117.59909    117.63271    119.33793    114.45983    112.39917    112.38849    113.33111    114.17928    111.49171    113.86382    113.48475    113.56207    114.86369      115.357    119.46372    121.30787    119.34061    122.61373    122.96902    120.24987    119.61116     121.0424    123.38727    123.92162    126.14638    124.15735    130.76744    129.74045    127.70819    126.25799    125.54057    125.00521    121.18925    123.20188    120.14627    118.19209    119.12828    118.81043    112.21634    120.28589    117.66961    118.03711    119.63613    119.14183    119.27034    121.51242    123.88537     120.2719    123.13709    127.97108    123.15509    128.58596    127.20035      129.314    129.35221    128.76581    127.90761    122.65874    118.97576    116.64539    114.02752    105.93695          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       24.00     273.1821    262.24975    251.44863     239.2724    226.77318    222.89255    221.69328    213.91282    210.34202    209.40217    200.80032    193.89898     196.7047    196.56234     188.5308      188.066    182.23943    174.15017    182.09577    171.01016    172.62397    176.27437     179.1694    172.34451    185.36831     205.2444     238.3632    246.95551    227.76369    199.82064    166.38586    159.01783    156.68702    152.86381     155.8467    158.55896    151.91075    153.22602    148.99157    147.43181    153.53403    149.73816    146.32944    150.85399    147.04651    142.42857      143.993    143.23556    147.07798    141.04822    143.87248    141.72622    136.45773    140.28554    137.48677    143.02224    149.26486    147.84628    151.91892    144.17283    136.14603    137.90669    136.85964    139.36459    140.02626    137.71988    134.32009    136.01754    138.16369    148.36075     162.7019    180.19903    188.59303    176.59247    164.81524    159.49203    158.21446    163.65898    167.60466    167.39169    163.72979    160.20139    161.02287    156.94124    159.19748    146.84058     136.2376    129.02943    138.40934     150.6071    167.97799    184.96246    182.69368    170.35771    154.20634    139.25646    129.67177    126.55597     126.7422     124.8699    125.62976    125.38319    128.29871    121.71436    120.29157    122.76217    119.89464    112.94097    115.29461    120.21861    117.77323     120.8172    116.53123    116.99843    114.26352    116.91388    121.07175    117.14116    117.84107    116.76379    115.68672    116.08989    117.21935    115.60524    131.14918    136.78752    140.21576    137.25458    133.56252    128.25574    122.67551    120.56782    119.27145    116.71542    118.64896    116.19364    131.70653    139.72795    156.49386    164.33877    165.96999    158.04256    148.55854    136.27223    130.16239    122.84902    123.84176    123.17919    124.40903     125.5232    123.76334    122.81217    119.48162    117.59043    112.66199    113.55657    113.76622    113.56289    111.97838    118.15858    112.29954    111.97129    112.39737    115.30342      116.383    121.62011    118.80903    120.21707    121.78274    121.47345    123.35316     122.6661    125.69273    125.43304    123.31449    130.41552    128.51883      128.462    130.40325    130.95163    124.62948    125.08964    123.97121    126.85279    122.51186    121.15804    117.34418    115.71913    113.43018    114.82825    117.19112    113.49085    116.23717    116.61159    113.05559    115.74754    118.39398    120.65126    124.09017    124.00395    123.03664    122.60915    121.43777    121.51183    118.41749    119.02103    119.10541    116.06972    119.24166    118.59817    121.00803    121.42451     120.0958    115.11824      114.566    116.46295    113.77396    112.51299    111.97664    111.97272    111.27029    110.35114    112.28591    114.69423     115.2746    114.66144    115.52246    117.95247    121.55538    119.41669    125.14828    119.84742    119.01182    124.23782    124.23631    121.32117    124.91601    127.05364    127.00821    123.25826    126.77821    127.01718    127.14026    121.57261    124.63582    113.38125    122.31859    119.56635    118.11198    118.55765    118.69972    115.38348    113.97576    115.94154    114.56031     115.8911    117.96727    114.25031    114.82993    116.12651    119.57063    124.09266    122.77565    117.42756    119.12698    114.69826          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       24.50     270.1368    260.86357    257.56564    239.61501    229.33863    220.01055    221.42551    217.95841    213.72907    209.25773    202.92423    193.12055    194.86361    190.13305    187.71896    189.58043    183.21954    174.05461    179.06698    171.60771     171.8094    175.94774    177.07729    173.68813    183.42644    205.28759    237.44471    245.10091    226.36035    199.77346    170.68264    156.18604    157.36463    146.59274    154.40628      154.186    157.66198    153.29032    150.82068    149.77245    151.50704    145.87685     150.7395    148.56722    147.91198    142.86018    144.27771     141.4488     144.6587    143.70713    143.53266    139.12127    137.30999     140.7579    141.28371    140.88351    144.51752    154.21759    149.17283    146.14889    144.23041    139.43101    140.59089    137.41955    139.46381    138.11094    134.42256    136.08667    136.19924    148.35461    160.98869    179.67665    187.45926    177.92718    161.41523    156.73746    159.60164    163.86717    170.92445    170.20721      166.024    157.09053    157.30013    154.94692    154.27246    143.04568    137.54224    135.69839    135.01231    151.22531    168.79171    184.39465    184.60364    173.97183    156.15018    142.18479    132.56436    127.42816    124.55163    129.07253    125.39024    125.89662    126.01441    126.87056    119.37265    118.73297    120.21376    114.30888    115.79492    118.73868    119.98294     118.5788    116.74179    117.60178    115.98634    115.22328    115.78096    119.74098    118.96884    117.21501    119.69876    116.89314    116.59202    122.59678    127.78926     133.5586    136.64809    133.87434    127.11421     127.2945     122.0162    115.66994    117.51795    116.06023    117.15125    121.05534    130.13483    140.71226    153.96224    162.90531    164.22087    158.21024    144.30418    133.94503    129.84868    124.75468    119.47854    119.76004     125.4627    124.49366      122.248    121.92346    117.99835    115.51277    115.40897    109.72423    111.65331    116.50844    111.28621    111.74342    114.70768    112.04355    114.07862    115.80612    122.17014    120.20615    118.83471    118.36759    117.56293    121.61681    119.08532    122.61825    122.49038    124.80495    126.16056     127.2999    129.07411    129.72069    131.31946    130.62109    129.01224    124.79312     122.5432    117.28717    119.84717    119.49334    115.18252    117.34228    114.44325    111.90705    112.33079    114.99393    111.70117    112.37296    116.52836    114.26361    117.59034    120.68855    124.19636    123.75657    123.69563    119.48436    119.12492    121.17486    119.67186    115.32657    115.62761    117.04141    120.28343    118.28199    121.77593     118.9505    114.85971    115.56874    113.40907    112.72262    114.01543    110.46289    110.68421    112.15455    109.30892    109.98799    116.26066    115.60695     113.4149    116.54568    114.26186    115.58909    118.90208    120.14171     120.9892    119.82763     118.9265     118.2636    118.40811    122.04528    124.41004    121.91835    127.61804    128.39159    129.06608    126.33047    127.70279    119.84162    122.45171    117.54089    113.64965    118.07086    113.57877     114.3691    115.52767    114.31059    112.11665    111.25072     114.0613     107.9973    108.94947          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       25.00    270.03424    259.92088    264.48637    237.06586    230.03607    219.11556    222.07131    222.23916    216.38588    207.71266    199.69615     195.7304    193.04967     185.9818    185.75942    190.16186    182.71962    176.67799    177.33565    169.30229    172.67235    168.94052    176.03449    172.07485    181.16063    205.80633    235.50614    249.52214    226.92122    192.38334    173.08111    156.79946    158.25821    155.38632    156.44113    154.11116    160.13863    155.49466    149.65287    144.50066    155.35274    146.65684    149.21955    148.71544    144.91112    142.80739     142.5088    140.32797    139.18365    143.22942    139.31009    143.00986    139.57403    141.79258    139.46533    141.37748    141.69497    149.41212    148.00906    146.48205    141.88826      138.587    139.65057    139.97906    138.06714    136.49714    137.78688    134.79018    145.60678    146.27701    162.70153    181.43178    185.95121    175.16656    162.58044    151.28446    162.54665    170.58398    177.58016    176.61765    165.55561    156.99559    157.33998    152.31652    149.54489    142.59621    137.03275    130.51355    141.72005    150.55537    171.64798    188.51057    190.24129    180.37603    155.96044    142.35151    134.49615    125.14529     123.5418    123.11365    125.54313    126.58052    128.14238    125.55491    120.18676    120.04429    117.92566     116.7288    113.23246    118.53773    118.76677    118.02264     114.5185    116.85152    116.77909    114.73907    116.04015    117.64703    118.98596    117.10916     115.7948    116.21008    116.95866    121.75747    130.18359    130.75872    133.40383    136.07758    129.17657    124.40241    119.65216    119.62215    118.32759    118.96054    120.73687    114.79245    129.31017    142.20248     155.1229    163.18375    162.32639    159.08665    146.74468    139.27817    128.18744     122.2598    123.61908    123.34028    121.94588    120.89062    118.89736    118.18746    114.78576    114.03112     111.9816    109.66224    109.45903    111.58314    113.29093    113.26781    112.54279       113.33    113.05127    115.25536    116.22025    119.74832    122.94277    118.60622    118.58458    118.43995    118.87925    118.65374    119.22862    121.73664    123.71135    120.12778    126.74425    128.35098    125.63365    127.66865    127.23355     124.5319    121.75576    120.76511    119.40631    116.21351    116.88764    113.23611    112.58023    111.39485      112.344    114.07021    113.27405    115.82347    114.53026    113.55015    118.45117    118.30386    120.16474    120.57475    121.89789    119.80551    117.42852    119.72661    115.30814    114.88864    112.99331    116.40831     113.7547    123.99229    118.72144    118.43363    119.23978    118.02282    116.34933    110.46562    110.48272    109.63104    111.78608     110.2229    107.30701    111.93878    110.90168    112.65057    113.78955    113.75632    118.47905    115.87346    118.23593    118.56032    118.65121    119.42783    117.46036    119.75486    114.85409    119.06212    122.28103    125.82617    123.02365    123.12515    121.49279    126.64608    123.08015    123.69278    118.73563    119.18451    112.62382    111.63131    109.09114    106.66087          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       25.50    272.79358    262.22307    267.33979     234.4321    230.54538    219.90309    223.57311    221.24641    211.80593    208.47991    198.15067    194.06273    183.82677    192.08574    190.47151    186.10076    183.65848     174.6801     179.4787    168.96294    171.59854    170.34201    176.18211    170.55924    178.47534    204.70748    229.92899     239.8901    221.34801    190.48348     173.9816    156.70033    159.73583    158.17182    150.66908    157.58085     155.7391    150.10583     154.5366     147.4974    149.86976    148.94059    148.31823    147.99278    146.94469    144.50065    145.80185    145.38454      140.791    141.35608    140.76256    142.35702    138.61745    137.56858    141.10423    141.09302    143.87843    151.10542    152.85923    147.81763    141.37517    142.42265     133.5333    137.66233    138.53578    138.50862    137.70568     135.6728    140.97622    150.71236    163.66894    181.89136    188.77686     175.9378    161.68593    155.84319     164.6885    176.05816    181.92356    178.24929    171.67837    158.45038    158.86564    158.38524    148.94432    141.05161    136.60946    131.12591    137.49535    151.81507      169.173    184.14169    186.84825    175.21911    154.14065    139.11441    131.14054    127.65264    124.87329    126.15209    127.03938    124.71447    122.46202    125.04051    118.42033     117.0941    116.92647    116.95752    114.05576    121.17395    116.77497    117.92052    114.51959    119.87394    116.26002    112.50216    116.08663    115.31389    114.87564    114.15442    117.87837    114.79017    114.57971    120.50793    130.72521    136.11207    135.07484    132.88288    131.69424    124.19668    118.82796     117.9234    121.18102    116.52012    115.49425    121.10382    129.21788    140.79129    152.40622    160.74131    163.93627    161.14264    147.73138    136.17556     130.4264    124.41006    123.71141    121.96909    119.80007    115.89441    115.60344    116.50704    115.36853    112.21104     113.1145    110.43117     108.5269    112.65991    113.14579    110.59079    111.77435    111.08948     113.2261     115.1189    119.87687    119.76611    121.60112    119.92976    116.79678    116.67863    121.71498    116.36036    117.33768    117.40755    119.82123    121.97053    120.48102    125.05288    128.85658    125.81334    127.46115    122.85417    122.83007    120.28147    117.11277    116.66613    117.54997    113.87302    114.30426    112.22322    112.28837    115.02071    111.83669    112.17104     115.6117    116.77963    114.62176    119.58279    119.32867    121.15763    123.78242    122.03935    118.22284    114.35766    118.91269    115.03382    115.43244    116.82692    115.14598    115.99764    115.95097    119.82267    114.33352     118.4561    117.52116    114.13843    112.98328    110.92308     110.5428    110.52254     106.4293    107.70574    110.98968    111.09749    113.67844    116.26359    113.01909    119.45119    114.61959    118.52607    116.35195    119.79001    118.94171    120.13469    114.20386     122.6925    119.60873    116.84656    121.07003    117.06768    115.42356    112.52339    110.53021          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       26.00    275.01368    263.45768      262.694    236.61271    231.58954    221.06605     226.3949    217.54377    211.25497    208.64697    200.76663    195.85051    185.28928    195.65373    192.39969    179.65261    181.01722    174.70223    177.47335    173.37097    170.21832    172.36346    176.74005    174.38851    181.84399    203.23381    233.27882    238.91533    219.69758    193.05973      175.237    158.59704    162.47578    156.55435    155.38944    156.43717    154.08802    154.00853    149.63459    149.58392    145.04889    149.08819    148.86365    144.76628    146.72395    145.36836    141.27657     141.7972    141.24158    145.89987    144.29012     136.9897    138.90655    140.81746    148.16468    146.22204     147.6951    144.94671    149.07246    151.75235    140.98313    141.29227    140.58352    138.47561    139.68044    137.91776    139.10956    141.55801    139.39402    148.85958    165.37565    180.15874    187.16727    180.52496    166.92585    154.48229    166.30014    178.63612     180.7405    178.21633    169.87662    162.25789    164.44709    157.32259    148.13318    140.60582    133.76507    135.81348    136.45932    151.41221    171.64843    184.03766    188.18052    174.20342    154.46027    137.55037    130.43003    126.85583    123.12699    125.23429    125.02173    127.98772    126.11338    125.02673    122.17532     118.3737      116.809    118.38364    118.49653      116.962    116.82896    118.15318    117.01888    115.50722    116.35142     115.7998    116.20317    116.56618    117.51748    119.84052    116.06093    113.45732    117.92649    120.88878    128.33136    133.98325    135.54546    137.05538    128.48515    124.43848    120.72156    118.57089    118.47594    119.56996    117.12031    122.65192    127.72766    142.23213    153.88573    162.36357    163.97221    158.18518    150.49958    134.87996    129.63531    125.84318    122.62066    124.19467    123.25078    121.91815    119.19842    117.86525    114.71578    112.30727    112.41971    111.39775    112.30282    110.41063     117.2104    114.27366     110.2706    113.26543    112.40157    112.92986    117.17548    118.45056    117.54873    119.11911    119.16641    118.91402    111.94614    119.51621    117.34814    119.44461    119.77646    122.39836    122.01045    128.19672    128.11387    130.75839    129.04942    124.02679    122.76143    117.22876    119.26986    115.77217    114.77193    113.77949    111.76257    112.04195    109.45332    112.47999    111.19674    112.87782    114.48193    114.21454    111.92538    117.19669       120.52    120.90799    123.16165    122.05135    118.91931    116.23833    117.29066    114.35422    113.64547    115.10773    118.81225    117.17684    119.34474    118.10294    117.45924    119.11263    114.51111    110.26797     108.5698    107.74232    109.67514     106.7551    107.26475    108.14596     110.3456    108.62185    113.13225    111.81723    112.64167    115.60623    110.62764    113.99857    113.87444    112.75274    112.31154    109.05606     109.4046    105.45197          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       26.50    275.36462    263.82609    257.89478    239.39222     232.0618    221.43067    224.63758    214.79006     213.5703    207.58209    204.03996     199.1908    191.82568    195.76024    193.57903    183.74787    184.16308    176.55603    181.83567     180.2884    173.54306    176.89359    171.75856    178.03423    180.06873    206.53886    235.60255    244.87821    223.66972    191.01701    172.28621    163.55066    159.09485    158.37974    157.07093    154.96087    156.62361    153.12446    152.42337     151.1212    154.03209    149.19229    148.72278    147.53902    144.83508    147.27266    146.06978     141.9121    140.93501    145.67608    142.77797    141.27839    146.88352    140.44306    143.34083    140.48508    146.31118    147.25445    151.07367    148.88484    144.13827    139.44221    139.80521    137.83171    136.31459    140.70335    136.62078    138.49195    139.83423     149.2957    163.36511     179.0161    185.20324    177.02915    163.79696    155.35041    163.43608    170.09134    179.47057    174.73511    169.90218    166.45563    165.21858    159.41602    150.06701    139.17631    134.33034    133.57995    138.02512     146.1592    169.36497    186.42252    185.15878    174.67704    159.29478    135.36396    129.07561    125.89367    122.81574    125.83653    125.26655    126.24576    124.40924    125.93828    127.66768    118.58569    119.27804    116.41609    116.64462    115.68618    118.58244    116.79089    116.52616    116.21763    116.61635    115.39228    113.18039    117.00101    117.10869    115.41068    114.60899    115.55665    118.98083    119.46748    125.77764    136.09978      134.873    134.37115    130.78022    122.07804    119.31755    117.11408    116.31124    116.90082    118.56468    120.26975    130.31151    137.60184     154.5131    162.70665    166.76998    158.47285    148.13874    136.41339     128.7203    126.10021     124.7019    122.99264    124.73245    120.29985    116.82861    117.18752    118.48776    117.16575    113.20492    112.34992    110.78727    108.00662     115.1423    116.10203    114.80735    111.23332    113.32839    113.80173    116.34592    116.73376    116.27073    121.31177    120.93499    119.47363    113.27459    119.57664    118.01661     118.5151    119.30671    121.56242    122.90783    123.85171    126.07092    127.32586    122.45809    122.16118    120.01638    120.60178    117.14789    116.95168    114.00658    116.54631    113.55578    116.53601    107.33376     110.4255    109.33304    109.88642    109.01493    109.30068    114.28955    115.50375    119.55794    119.19091    119.96778    119.78614    120.45559    114.93635    114.89502     112.2872    113.57707    116.16104    116.98804    115.60632    115.07229     117.7387    112.83131    111.74604    110.39309    111.65264    105.97463    107.34053    107.85723    104.71383     108.3636    104.81577    106.35865    104.99642    109.52053    110.18528    111.93446    108.43883    107.01116    103.53001          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       27.00    273.41577    262.94439     255.4053     242.0516    231.86879    223.03412    222.82266    213.86726    213.23351    205.60023     200.5474    199.87078    193.62591    194.12105     194.8509    188.64902     188.5173    179.27304    183.49338    175.92077     175.2225      176.822    173.73809    178.03936    180.69982    210.40741    233.09193    244.91297     223.8704    201.77549    172.67435    164.66912    155.63921    159.01986    154.45323    154.69626    156.35178      150.021    153.16073    153.04507    153.70801     149.3682    147.36396     147.8915    147.62675    146.45338    142.06872    148.75235    145.19002    144.41744    142.65309    145.43358    145.72285    137.91089    140.64333    139.75843    144.01213    144.93064    150.97127    151.53057    146.98616    138.43137    137.28533    139.03332    142.58279    136.92309    138.00873    137.66646    139.55996    154.03828    164.04888    178.25102    187.81732    181.76916    164.15287    155.11838    160.41513    166.57946    172.37735    163.28523    162.25869    166.18296    163.23159    156.75122    146.38685    138.72495    131.08729    132.95826    137.56793    150.90176     169.0052    188.25135    187.96246    182.52683    162.38706    146.18243    130.47361    127.51128    126.61577    124.82766     125.6699    125.42054    129.37812    126.29149    122.90864    118.34469     118.6808    114.74683    118.21688    115.33299    118.78471    117.11709    118.69311    114.58065    116.29425    117.81047    113.91172    113.28163    114.81818    116.99067    115.77173    115.24921    116.97939    119.51791    121.92652    133.17748    140.24669    131.50256    128.15375    123.98948    122.68794    116.67018    117.01657    116.04959    116.23527    117.14908    130.54035    139.54753    152.55511    163.94955    163.97104    159.54289    151.02128    134.29576     131.1647    127.47344    125.40257    121.73193    124.57202    121.23401    116.95626    116.43949    111.90929      111.221    115.52851    113.63109    110.94487    110.07806    112.08566    111.77046    114.80513    115.27647    110.82429    113.81026     112.8423     117.3229    118.36616    120.99171    120.50111    119.30396    121.55248    119.47877    120.26298    119.00006       121.03     122.4546    123.34263    124.75364    125.10606    128.00775    126.29786    125.85989    120.50509    121.69104    114.24613    116.56786    113.26611    109.79758    114.33654    111.58937    107.52792    108.35851    111.15437    111.11877    112.74322    113.80131    111.59641    112.06314     116.0391    119.81381    119.56932     121.9723    116.37809    117.51516    112.16088    112.77547    109.65358    110.95746    111.27963    117.41403    111.50625    118.45052    113.77814    109.87101    110.52641    107.19055    106.75162    104.15113    100.44467    101.50624    103.89371    102.51085    103.53569    101.01793          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       27.50    272.53768    262.15968    254.92915    242.98281    233.25101     226.1136    219.59267     216.0439    211.93552    204.65684    198.64398    199.08262    194.06701    193.06824    191.89366    186.96006    188.21517    180.05467    178.85132     177.5378    175.81565    175.51588    178.74276    174.55143    184.01921    204.03316    231.47324      242.306    221.62706    198.94413    171.54947    164.08079    156.23414    155.08438    155.15359    155.08629    155.96894    152.57092    153.30564    151.90897    148.95412    148.75144    150.30708    150.11636    146.80603      148.763    144.78876    142.76281     139.6541    139.45692    137.58523    141.68708    140.48674    142.85446    137.09764    145.98868    145.23639    146.96973    150.45367     150.6895    146.01078    140.00403    139.70035     135.2099    139.97199    135.61848    142.18718    140.02967    139.58211    151.05457    166.81572    182.99039    185.67668    177.59452    159.69023    154.57871    149.91765    156.57025    162.79117    160.02605    163.22882    161.13651    163.70615    153.58681    146.13632    134.17968    135.94839    132.81397    139.39355    150.52318    170.06487    183.13529     188.6405    184.81221    168.29888    151.71243    140.80941    134.21373    126.58871    127.76413     128.5307    129.10776    126.99898    127.61832    121.01351     119.4063    118.63214    117.46583    113.92182    115.25266    113.72112    115.96177    115.91901    116.04894    114.85427    112.15317    116.46809    110.86024    117.48971    118.12827    114.18824    112.74195    116.67414    118.48003    123.86985    134.81532    135.31151     135.2189    129.44391    122.64033    119.91235    116.98109     117.2627    114.03514    115.43785    118.62225    126.19366    136.76238     149.9238    163.82938    163.76935     160.2911    149.04305    136.01241    128.74971    122.17163    125.42156    124.22346    121.62046      123.149    118.83361    115.54968    115.89422    113.49741    111.22261    112.09282    111.30136    109.94938    108.36185     109.5955    112.07382    110.61332    111.81263    113.42594    114.25973    113.64916    117.39038    121.01325    118.68335     119.0387    120.83353    124.44861    123.87305    125.26953    123.16148    124.15701    121.43657    123.46555    125.83371    126.24252    129.39604    125.67348    124.76483     120.4032     116.3245    113.63348    114.82707    112.37059    111.77472    108.54234    109.11536    107.44771    107.82528    111.74048    110.62554    112.56148    111.55717    111.08925    112.76249    115.91595    118.32814    115.84472    111.31464    114.87075    110.72953    110.73589    111.18658     113.6625    111.73113    115.03703    115.22545    113.08258    111.73922    109.70075     109.8874    105.65353    103.37171    100.88377     96.80528          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       28.00    272.17483    261.48564    255.50934    243.32661     233.4635    227.01091    217.58953    219.49818    211.03752    205.72776    199.36488    198.99084    193.73211    193.96443    189.81117    184.37282    188.22146    179.58098    175.36937    177.80336    174.12846    176.54988    176.88421    174.85918    183.42369    201.74676    228.27773    239.93864    223.35279    194.71607    172.29314     165.9706    160.58105    154.07101    159.29376    157.70269    155.27749    151.18328    150.45796    151.33779    148.81153    150.21361    148.32143    152.09578    147.25403    146.44237    147.96642    140.33521    142.81957    143.07761    141.56854    146.32067    143.13275     146.5171    139.28237    142.15275    144.52853    149.80003    150.24883    152.74829     150.1603    144.01684    135.70521     135.2767    140.82982    139.14581    140.73515    140.74942    140.33147     150.4821    165.80869    174.77048    184.62911    176.52173    161.32229    154.79839    148.71269     154.6159    155.37182    154.50003    155.86009     162.6174    164.97376    152.47734    146.87341    136.39764    135.15582     132.9301    138.44045    148.59808    167.01495    180.84919    191.57824    182.07847     169.4227    156.19495    145.29793    138.49527    132.15009    128.94808    131.45794     126.8009    125.94559    125.41545    119.28907    116.40348    114.37291    113.97012    117.26156    117.53679    117.09576    112.74066    115.96265    117.99134    114.55451    118.60442    113.42963    115.83152    113.55268    115.62972    114.65625    112.89783    114.68298    118.67163    126.38853    130.28847    133.29897    132.98735    129.45576    125.79966    116.66941    115.20156    113.44187    116.91701    118.58066    121.23807    128.37591    138.27358    150.53863     157.8692     163.5883    156.06918    147.73074    139.25639    130.90617    125.24588    123.31365    120.68326    123.25704     120.9108    118.24998    117.93603    110.43127    111.55933    111.87009    110.68473    110.77669    111.94124    110.38679    110.11488     109.7329    111.14893    110.69377    112.23508    113.94334    116.42109    118.69342    123.05749    119.25252    123.78069    126.52617     130.4387    125.20476    128.07018    125.61755    126.30347    122.96272    128.37764    126.32679    121.48995     130.6686    123.38547    120.57225    118.12581    115.70419    112.88095    112.10066    109.88919    108.58912    108.64951    106.87691    108.03857    108.10809    110.61636    110.44874    109.09031    109.69191    114.12745    113.82538    115.04753    116.07407    118.45974    113.88505    113.47599     111.6333    107.93195     109.9908     110.2404    110.63322    110.42908    108.82334    108.38672    103.50849    102.68345          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       28.50    271.95637    260.77176    255.43945    242.25347    231.19354    226.26975    218.87625    219.36961    208.57084    203.83515    198.26204    199.62502    194.18354    194.62127    190.98902    182.41627    183.17085    181.32321    179.94524    179.58153    173.84126    175.02957    176.24629    177.06018     181.1636    199.93526    225.39632    239.59778    225.61379    199.89356    176.60616    166.80178    156.08691    158.83231    159.08421     153.2234     157.8643    149.04798    152.77824    149.93937    148.84262    149.89314    149.48063    145.78212    148.88796     147.2089    143.51065    139.28524    141.92651    144.19904    142.52149    144.49177     141.1631    144.51195    142.26952    142.30351    147.02003    150.49106    150.06424    151.27522    146.28929    139.86724    139.58793    140.00224    142.91376    139.84869    141.32357    141.92479    137.48255    150.80751     163.5269     179.8968    183.66495    173.93462    162.43076    156.52009    147.65948     153.0038    151.94836    157.85435    157.00274     159.8869    167.27426    161.59927    147.75874    140.26246    133.31183    133.78188    141.21284    153.02526    168.58677    178.82635    183.65571     176.1206    164.44535    153.46541    146.95989    137.65403    131.77143    126.62104    127.76605    128.46024    127.23119    125.35919    120.42587     118.2063    120.06246    116.68976    116.70439    120.28688    115.81725    114.38619    115.76989    114.26523    112.04587    116.73786    115.75728    118.72017    114.19037    115.70566    114.41308    113.41658    114.28565    117.74944    123.79965    133.20138    136.06531    135.12604     129.9609    123.81223        118.5    117.68728    116.46888      120.963    117.40385     120.1239    126.46125    137.59826     155.7422    157.04979    166.99925    159.62852    146.77137    135.73026    130.89194    126.85005    121.26089    128.73206    122.72671    123.33034    118.26061    116.56515    112.31671    112.67753    110.02715    112.65135    112.75958    113.90503    110.97217    108.00975    112.48103    108.79924    111.23823     113.5681    114.12646    115.18136    115.62679    121.03659    123.85295    128.15236    134.94222    138.09389    132.67957    128.89604    127.72264     124.2842      125.503    126.58278    126.55518    123.82707    124.55586    120.81695    121.93555    118.22932    111.88557    113.44559     108.1722     111.1722    111.31562      108.848    106.53982    108.34274    110.64603    105.12546    107.49819    110.66731    108.38727    113.10657    112.91317     113.0197    113.29374    116.05625    113.89708    112.49829    111.49452    109.18356    104.44625    101.08951    100.98298          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       29.00     270.8037    260.42738    254.72898    241.00024     229.9843     228.2728    221.70793    218.12802     211.7164    204.74509    201.82939    199.71366    194.01977    193.37369     190.5252    183.25941    182.43664    182.37633    182.67345    180.84039    175.49226    174.59453    175.72968    177.60274    180.55471    202.21849    227.03373     236.9779    221.98743    192.86955     167.9787    162.36569    159.82378    159.31856    155.05025    153.51692    155.03298    155.55459    155.27562    151.78211    148.56328    150.50067    147.97635    144.78337    148.08476    148.07214    145.05548    142.28717    144.30613    140.62865    144.08833    146.24401    140.33613    142.21849    138.80187    138.97608    144.93816    149.33172    147.03969    147.47691    140.15612    142.56574    139.09484    140.55683     139.4006    138.40829    138.33532    137.97473    138.93313    150.02007    167.17279    182.75871    184.17053     174.9292     162.6969    156.73354    151.18719    152.66978    154.79484     157.8672    161.00581    169.62943    171.15813    166.60139    153.76325    139.52296    130.77626    131.62447    138.30252    154.54729    165.07505    178.41927    179.07584    172.10219    154.47778    144.06014    139.59136    132.26925    132.69168    129.62552    128.64721    128.38174    125.88726    125.53002    122.24528    118.29353    115.62828    118.22759    115.49887    114.78248    117.41547    113.77615    111.21695    114.20533     112.3438    113.34063    113.61541    114.46845    114.26756    110.95358    115.91884    115.47259    115.35437    121.05883    126.04594    126.15914    131.10703    136.05851    129.80231    124.03013    117.86914     116.7065    115.61023    109.01184    119.07394    119.72288    129.71894    139.63302    157.56608    162.14593    163.59398    159.40677    151.33533    136.16796    130.05029    123.40154    124.71229    126.63407    124.48805    127.95204    125.57393    121.82103    117.23884    112.59896    110.26068    109.45749    110.33202    108.86909    109.71178    109.88143    108.27542    111.15092    112.18003    112.61696    111.94857    111.54872    116.05758    115.48861    122.57387    127.10307    127.52899    130.65468    125.94185    122.21286    122.92393    122.93587    124.68949    124.28105    126.88036    126.97401    121.10932    122.93229    115.95849    115.46638    116.04381    109.68282    110.65733    104.66825    104.93203    108.15492    107.68785    106.75802    106.05978    106.22322    106.16571    106.59257    107.97959    111.44928    112.90703    110.67302    110.17016    112.32802    109.84996    103.28688          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       29.50     268.7006    262.05012    254.44172    241.21554     231.7682    231.21003    224.30599    219.95343    216.51134    211.01508    204.88185    199.26262    194.54663    190.24516    190.71226    186.26874    190.12313    180.64227    175.25691    176.09173    176.00982    177.76854    171.86611    177.68248    186.27432    204.18052    227.06596    236.44422    226.90171    195.29377    171.78671    164.07707    159.04364    160.08993    156.46619    153.60908    152.90461    154.55606    154.54103    152.55033     150.9107    151.12747    149.82088    149.81111    149.66136    151.73535    146.29248     147.5654    148.59001    137.86374    146.75722    139.74277    141.93756    148.18134    142.05655    143.84707    148.17799    149.43306    146.95388    150.51074    145.77221    142.02409    139.53488    139.09543    137.55897    138.43456     138.6379    142.92366    144.10216    152.42832    167.46602    182.02512    184.65014    180.11383    163.64246    151.53552    156.53674    157.62954    158.43586    160.07553    156.24506    168.55829    173.33683    166.82157    150.62957    140.62438    132.38272    134.82669    141.90324    154.98249    168.06179    181.51392    177.50479     170.2227    150.96271    140.84207    137.48759    132.04639    132.21656    132.34156    128.62967    127.72024    127.61843    123.76858    121.58589     117.1433    116.62859    118.05451    115.79706     117.4424    117.24761    115.67758    111.86359    115.48936    112.75893    112.72268    114.87018    114.27756    112.04218    112.87106    110.71258    113.37659     114.6091     117.9148    122.67641    132.47905    132.82582    131.45333     127.5083    126.02706    119.02239    114.96002    116.62707    111.12046    114.77704    121.51297    127.58955     141.4487    149.23303     163.2173    162.02386    157.89244    146.69301    136.31828    131.92513      126.313    125.25422     125.4107    125.07976    129.68685     126.3061    124.15292    118.61518    116.46341     109.7415    110.98661    110.90829    111.36592     108.1653    109.70531     111.8167    107.55944    109.88618    110.22683    115.69971    115.83273    114.59016    113.28955    117.88769    120.20302    120.70594     118.0846    120.32599      117.062    116.66495    122.42073    124.83934    125.67223    125.74774    127.03994    122.68934    116.61985    119.70188    114.44557    112.47461    112.68263    106.59112    106.94013     101.5473    107.19765     103.0777    103.61594    104.61383    105.23531     99.29081    107.62304    106.95757    103.98961    107.39732    106.36488          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       30.00    271.14643    263.12346     255.6997    244.66043    231.60887    230.18566    223.01692    222.26974    214.45376    213.99744     205.0867    197.14127    193.02422    192.37229    192.41498    189.79699    189.02091    181.70414    172.19349    174.65298    172.34188    174.57701    173.01875    176.67985    191.63932    204.39442    227.46229    236.35259    223.74591    196.49306    176.95834    165.79598    158.09526     156.6809    159.05663    151.23148    150.34504     156.3704     157.9211    150.35667    151.45234    150.50527    146.59276      146.017    144.74509    150.57891    146.54342    142.58641    143.74844    146.38665    141.07539    143.73842    143.43605    142.73112    140.81962    138.78629    144.18875    144.50411    155.93248    147.14177    148.82176    144.33067    139.27753    138.82291    141.73596    139.47156    137.55503    139.37391    144.69727    156.31927    166.27803    181.15387    185.27745    181.48415    163.78053    154.93713    151.01269    157.43998    158.59438    161.88999    164.02164    161.92132    168.50743    164.23902    151.93748    139.70138    137.38904    132.89004    139.07799    153.09934    168.90924    177.48636    180.26958    170.39238    153.66178    140.56127    130.08507    134.87868     138.9701    139.16001    132.00924     127.8938    126.37826    126.10531    122.10037    118.68007    110.96885    114.53838    116.12208    116.73197     115.9047    115.35119    116.18859     112.4692    116.41321    114.22392    118.11781    114.63506    110.30695    113.66053    112.42172    113.46047    118.99327    116.90978    122.28748    129.70968    132.29161    130.95022    127.84301    123.13123    121.41176    118.44023    114.38061    114.77095    112.66383    121.03013    128.58109    137.13104    149.31981    159.21109    165.07475    155.38475    147.70121    135.45483    128.62757    120.65294    123.51565    122.28657    120.99216    121.95008    123.75988    122.27528    118.69587    115.33659    110.63146    109.82162    109.46408    110.99687    109.44319    113.01515    108.22896    105.08334    108.21353    109.82662    112.28037    114.49991     115.9557    115.45861    114.13895    119.01803    115.42042    115.40717    117.85125    114.42694    118.11241    117.67529    122.96082       127.44    124.98565    124.29242    118.29364    115.19146    115.36462    116.64728     114.3259    113.73935    110.88416    103.80129    103.52267     102.7971     104.0372     104.0544     100.5211    101.71363     98.47382     93.32372          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       30.50    275.42038    262.86991    258.53708    246.41559    229.79206    229.03414    221.93817    224.00176     210.9461    212.97301    201.12081    198.22887    191.30349    194.19799    190.57432    190.09553    185.59149     178.5452    176.93162    176.37006    170.42648    171.90323     176.4787    175.35804    191.45354    209.76314    227.93963    236.34841    222.75282    191.35645    172.86242    165.40982    160.92441    160.88749    152.28707    153.12026    155.18507    154.53204    152.88253    157.02457    152.29178    151.17616    151.13123    148.21146    145.14185    146.57756    146.46428    148.49362    140.39186     139.4163     145.6418    147.06144    141.92457    139.83342    141.18868    139.46249    147.74303     151.3876     151.4393    146.90975    145.54733    143.10231    141.18201    137.12616    137.89548    134.61502    139.70115    139.61339    142.29247    154.91221     168.9126    183.03594    188.11157      174.992    162.26332    156.78045    150.36381     151.5323    157.95972     159.0792    158.57016    161.16151     162.0695    158.37953    152.35142    138.87936    132.67358    136.33367    140.85782    155.22254     173.4346    183.73701    182.17475    171.98614    158.53827    145.21758    130.94079    137.88054    143.77993    140.20342    134.88236    128.29537    129.17532    123.25585    120.57765    118.93148    119.21389    117.96628    116.74156    115.20672    118.35208    117.78405    114.68821    113.86814    112.97574    115.97034    114.47553    117.12823    115.07593     113.0206    108.51058    111.79567    115.76669    119.44192    127.14152    130.63213    133.30546    133.05787    126.76222    122.75845    120.81494    116.48804    114.94907    114.17925    116.96256    115.83077    128.69597    137.27989    149.24911    159.85217     161.4873    158.23021    147.50913    133.91919    127.42277    123.68971      122.749    126.31645    123.91594    123.87245    119.42853     117.2709    113.13887    113.57128    110.68372    107.16429    108.50814    108.18834    110.96404    109.45542    109.41873    107.29964     108.7415    110.38245    113.95564    112.22573    117.18695    111.97844    116.54887     112.9227    115.09413     112.2442    118.37295    118.62467    120.32529    122.00271    124.93874    128.77206    123.20375    123.20193    117.11606    113.85539    112.20597    115.01514    107.15853    105.39238    105.24904    106.98158    104.27166     99.74147     98.21336     99.57019          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       31.00    279.44475    261.39536    262.80308    246.92132    227.87795    229.65513    222.43275    226.85464    209.94077     211.2244    201.37279    200.91375    190.08182     194.2099    188.24807    185.62412    182.03542    180.96486    181.57727    171.35229    169.76708    173.27316    175.29555    173.47961    183.30634    207.78148     222.3805     230.8689    216.83151    189.92855    172.97758    165.26837    163.01594    158.96825    162.02674    153.87381    149.55391    155.95391    154.86256     150.1501    152.92929    151.60071    147.84957    146.91286     147.3237    145.10262    148.68116    153.70642    149.84673    143.06682    145.35396    140.25138     139.3632    139.68621    141.22482    146.09355    143.10771     152.6888    152.77888    150.25765    145.93699    142.25806    146.14059    139.33505    137.71438    143.07352    138.30288    142.59947    144.16656    159.77608    174.52516     184.2492    188.29504     173.6487    165.26924    155.61476      152.268    156.74484    153.56768    157.56716    157.46223    160.83949    160.15898    158.25011    146.78694    141.19341     135.3175    136.57956    143.25317    156.05598    171.69746    182.19165    184.64454    169.89525    157.10109    148.08014    137.89407    141.64025    140.02005    140.94899    133.12439    131.89085    126.48855    126.80241    122.89925    117.86825    119.45681    115.06751    117.12628    115.75454    118.42197     120.3623    115.77669     115.0139    112.63592    114.88421     113.5598    117.43433    113.91743    115.39068    110.12292    113.19142    113.81017    116.69544     122.8542    128.75609    134.83592     134.2417    128.42901    124.31772    118.79395    115.93807    113.41862    113.80883    115.95054    118.57042    127.09397    136.82154    150.03806    157.02748    165.46984    157.64438    146.12112    136.28038    128.83308    122.48718    125.82026    123.49816    124.30492    120.55202    121.29352    118.13437    112.81968    113.53793    109.33402    109.86941    105.87836    110.82011      107.421    107.43236    109.00058    107.39317    107.44722    105.82976     108.1985    112.31873    112.25028    115.02645    115.14789    113.38943    115.08221    115.82035    118.53902    115.87124    119.19686    123.99007    123.00983     124.9807     123.2631    122.61281    117.77233    114.84342    113.33507    107.59654    107.99496    103.80405     99.46883     94.96743          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       31.50    281.86936    261.59941    261.22272    247.38322    229.88712     229.5057    224.07351    223.89616    209.26493    209.29383    203.20637    204.72095    188.23893    191.60218    189.26707    180.42552    179.76272    182.61148    186.39267    167.41607    171.34904    173.17677    176.05621    173.93195    185.43961    205.05616    221.86114    229.16673    215.81864    193.02984    173.03582    160.91506    162.50217    158.41269    160.92345    159.96361    154.23792    155.73351     152.0008    149.16168    148.13646    153.03251    148.72732    150.42102    139.07369    150.14714    147.54349    146.83669    148.54914    145.24752    144.22447    140.99684    142.47736    141.94989    143.79804    144.90163    145.62202    147.33185    152.17151    149.89362    145.97076    145.06518    143.94416    137.49431    137.52138    136.96458    137.17096    142.17141     146.3848    155.99814    174.41398    184.07161    187.37191    176.49481    166.56054    153.21939    151.21811    154.55877    159.79823    152.24068    156.84329    154.67129    158.14988    153.54781    142.37218    142.84892    134.69457     135.6493    139.13558     153.3365     168.7143    184.64902    184.23315    169.44983    156.68277    142.24908    135.35841    137.06824    135.90369    133.40422    133.21272    132.52715    128.02411    122.50267    118.89777    117.65288    116.37495    117.47407    116.42123    116.16256    112.75673    114.54658    114.27409    111.21242    111.76842    113.57665    114.97815    114.47528    113.06372    114.23223    110.42497    115.81519    109.84912    121.34413    125.07799    127.68949    134.24082    135.47747    128.45136    120.68288    120.85127    114.11497    116.66381    112.33266    116.16764    119.87955    124.59662    136.37489    151.11439    162.73173    162.63882    160.55026    147.27728    136.54868    126.88078    123.92707    124.00851    126.69362    127.91011    129.78167    122.92347    123.85455    116.35605    111.16762    107.95264    109.60645    108.01966    106.83483    108.45789    109.45554    106.38846    108.81761     109.6823    106.99529     109.3086    114.55592    114.47037    115.16569    112.47501    114.53943    113.18462    111.47013    116.45418    114.97006    115.32655    116.72783    121.25444    120.25896    124.58865    121.29028    116.22417     114.2771    103.98374    100.86319          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       32.00    282.50648    263.89005    260.44418    249.03166    232.09274    225.87274    224.46266    217.18031    208.39935    207.54868    205.40845    207.28056    192.46035    190.56544    192.93176    185.54748    183.20272    185.02828    186.86373    177.17577    174.66246    173.74285    175.38924    178.63688    185.35089    206.40898    232.76857    225.43094    216.49927    195.84087    175.22837    158.93661    159.70737    161.91626    161.28229    158.95342    152.50369    153.20827    155.54622    153.07985    149.23797    155.84131    146.41206    148.62191    153.66413    145.84269    145.57707    149.43509    143.93678      144.725    140.72848    139.83756    140.46291    144.73311    142.69068    143.72873    150.62457    145.47228    151.35558    149.83821    144.56724     139.2821     143.0398    142.33363    142.71904    136.09778    139.08062    145.53468    147.60943     156.4514    169.74856    186.09378    187.44826    178.94937    166.17272    155.28497    155.69002      158.026    160.08945    155.51532    153.10528    155.00602    156.64326      155.314    149.35366    144.24045    135.82502    136.27517    144.40859    150.61066    170.08157    187.09775    187.78188    175.23139     155.4287    143.70889    142.54018    136.77645    131.39577    126.73647    123.33042      127.288    126.80415    123.95853    119.92922    121.59691    116.01703     117.7316    115.28619     115.8558    115.29105    118.85887    113.95235    113.86636    115.36295    114.01576      116.112    115.68774    113.36752    114.25096    116.83449    112.23798    111.58467    118.04271    124.94219     129.4625    132.51169    133.05741    131.26592    123.60256     115.9144    115.03564    115.19858    113.43283    114.16977    119.96324    126.53706    131.13615     150.0046    157.73337     160.6112    158.10584    150.55004    140.55239     129.0354    124.95681    123.99994     125.0768    129.50403    125.44724    128.21438    122.34167     119.3579    113.73169    108.75572    109.81769    110.20733    104.97376    107.95681    107.25211    106.00348    107.18274    108.38229    106.34478    105.94164    108.14584    113.49692    112.51677    113.93452    112.40893    111.78684    110.26084      112.569     112.5978    112.54995    117.43804    113.56993    120.46945    115.91224    114.82392    105.78767          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       32.50    282.28013    264.93685    261.35417    251.95537    234.77758     222.0145    221.86403    213.07083    209.30118    204.18056    210.27062    207.93677    193.14573    191.02778    190.47963    186.39676    182.97938    186.40499     183.1529    179.83207    174.85971    174.61617    175.56102    179.34798    187.09116    209.40959    232.77359    227.67625    217.49552    190.84124    174.05155    159.23913    161.16959    161.81093    162.21673     157.6018    152.25982    152.06914    155.47781    153.95534    153.59691    153.44991    151.51852     140.6106    152.61256    148.35735    150.59742    148.16372     144.9261    145.15163    138.62157    144.86197    142.35995    143.32271    141.84784    145.49088    150.55465    147.07263    151.13753    151.58711    142.25261    145.31144     135.0317    138.52802    139.08977     136.6168    139.25171    141.39498    147.18579    156.49544    170.32194    186.56997    182.14038    177.23759    166.53132    154.43221    160.30073    163.76835    166.40042      164.835    159.77978    156.43577    156.87962    154.28477    148.96938    143.40369    143.41812    133.10808    137.34322    154.19252    169.52655    185.14087    190.63977    176.71177    155.18948    144.31482    142.81719    140.00194    133.59591    125.99831     130.7228    125.38865    124.26482    122.03313    121.09242    117.82384    119.50834    117.26301    117.03346    119.33608    116.48173    115.85035    112.13307    112.12101    114.37371    111.77769    111.30742    114.02755    113.40503    114.83786    112.14084    112.95239    112.90207    114.93655    123.45793    127.87836    134.84918    133.43023    128.26661    125.73861    116.83688    115.66211    112.96034    116.99277    114.71765     119.3882    128.65549    134.89106    146.43034    160.98647    164.78341    161.32959    151.59891    139.26423     131.3987    126.87361    123.94583    122.60742    124.04282    124.09914     123.9049    121.51099    117.87264    116.20594    111.29773    103.97279    110.94265    107.74426    104.06454    104.16493    108.55968    109.54659    109.27775    105.56586    111.65349    108.41098    111.72582    112.84996     112.1411    110.81743    110.04065    108.86725    104.95323     110.6561    107.68802    106.24654    105.11501    100.27976          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       33.00    281.82052    264.41711    261.97457    252.02912    237.16249      220.969    218.69501    212.00583    207.67192    201.52031    214.86234    205.21522    191.66373    194.77162    187.00585    183.54286    180.82479     180.7293    185.66351    179.01606    174.17254    175.83974    177.88445    178.44958    187.59581    214.62243    227.25255    228.32342    214.52568     189.3302    173.68551    167.50459    163.47847     159.2964    161.05507    157.62498    158.15438     150.6598    156.34194    151.28924    155.31425    159.43989    152.08336    154.59468     143.5229    145.31387    151.07358    143.32493    146.63991     148.3231    143.89755    147.29293    144.45469    143.39435    143.45646    139.04958    150.54234    153.00575     147.5231     145.4762    143.64931    141.59265     140.9253    142.13281    139.67537    138.66266    139.98105    140.39073    146.05665    155.06863    170.21446    187.67444     188.0816    178.40132    165.47876    159.07636    163.47896    174.60096    177.68892    173.38334    161.95939    157.62921    162.93055    156.72387    155.24718    145.40401    139.72538    135.95573    141.65362     151.3591    171.28675    186.02765    181.35339    172.07383    156.71615    144.08554    138.35426    134.92405    130.37716    124.56923    129.38347    126.65929     128.0417    120.10458    119.74211     116.1414    116.35683    116.16979    118.77011    114.48841    115.10476    115.51826    113.51844    114.21532    112.33393    114.17468    110.92619    113.95737    118.38913     114.0002    114.26798    109.63897    114.63155    120.56974    120.20156    128.06627    129.57864    130.58712    129.95254    123.68728    119.42457    115.46481    111.02278    112.90079    118.21294    118.17064    127.92118    136.31014    144.50864    159.97319    166.53142    160.49734    154.40946    142.24893    134.18318    120.60493    123.31084    121.34052    120.02293    120.62409    118.93748    115.55192    116.26592    114.65977    110.57936    108.47064     107.7702    103.88702    105.45384     104.3259     105.6316    104.62968    105.09669      104.457    106.50918    108.39698    107.07768    111.35923    108.85032    109.56477    109.51121    103.97119    104.35301     99.73062          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       33.50    281.20409    264.90451    261.60075     250.3386    240.47104    223.38855    220.20124    212.80867    205.65885    204.50478    211.34278     198.8075    199.64018    197.32928    180.94102    187.22893     179.4344    176.74228    186.05081    181.31199    174.87939    175.51506    174.74543    183.24894    193.19481    207.86751    222.78654    227.75279    211.51511    192.64602    178.07095    166.04082    160.94238    158.28848    156.10354    152.74246    154.95527    153.95813    154.38564    153.62979    154.46572    151.93519    152.59206     153.8565    144.04414    149.90175    149.02359     142.8399    149.03243    146.10781    143.91729    143.41268    144.03224    140.70887    140.71319    148.34921    151.97914    157.56702    151.37528     149.2426    145.40739     142.1458    146.66492    139.22773    140.60837    139.00805    139.55758    139.57787    148.61473    150.11664    172.64665    187.54358     187.2113    180.38086    165.15741    159.84582    162.62484    174.41991     179.4061    173.89382    168.45209    164.26495    160.60473    157.04565    153.95934    148.67159    140.46096    136.54513    144.87981    153.61651    172.84119    181.99544    181.09222    171.06344    155.85151    142.21751    135.39215    130.90648    125.15874    124.56486    128.48399    122.03695    127.29293    122.28287      122.251    116.75976    120.62997     117.4583    117.38642    120.75287    117.69152    116.41012    112.23108    115.13978    112.55268    114.82901    115.37048    112.33972    113.83655    112.46516    109.93104    110.86528    110.16789    116.52236    121.43043    126.00929    132.41432     132.4388    127.97811    121.88054    119.54887    114.27868    110.82194    114.00435    114.08108    118.39573    125.73878    135.39835    146.62127    153.39238    164.55549    158.88067     151.1479    142.81446    131.83371     124.4966     120.5383     120.5761    120.38917    117.25825    118.92414    110.24298    108.13452     110.1278    109.09121    108.71098    105.66855    108.38679    104.51049    105.14381    103.56082    105.71829    106.32922    106.16673    104.87232    107.52323    108.53939     107.9741    108.31317    109.47387    102.13406          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       34.00    280.12771    266.64004    261.89956    247.69289    244.03043    227.91783    221.57495    214.13971    209.51245     207.1984    206.50765     195.4179    202.83862    197.01529    184.25318    188.01579    180.16311    182.53329    184.44194    181.99647    174.44776    172.04111    174.22846    179.27646    186.57738    205.61642    229.83283    229.21648    215.07725    188.61248    173.45671    161.07726     160.2222    160.47618    160.74604    154.79966    153.04034    152.54577    154.27829    155.05251    157.39154    152.84221     152.7083     149.5241    146.72932    150.54154    148.00691    149.05229    146.62861    147.19772    145.30627    140.38642    139.00312    142.58571    144.71815    144.21333    150.87886    150.19024    154.21131    150.94442    148.92643    140.37818    139.22216    136.67435    137.73145    141.10308    139.44163    144.01765    143.82271    157.42457    171.69694    186.12075    191.33342    179.75031    164.22155    160.59113     157.3422    169.59308    174.46714     168.5604    167.14251    165.56167    163.44397    162.11336    156.38562    150.99174    143.32895    133.48355    139.96465    152.66625    171.52944    183.99088    184.50813    172.33214    156.00542    144.44855    131.83017    124.88633    125.99589     122.1321    126.43613    126.05768    125.43214    127.11395    117.17402    119.21713    115.45208    117.61661    116.57584    116.77486    118.40559    115.30391    119.90488    111.21627    113.61023    115.84134    112.85835     114.3888    110.76868    114.24709    114.16914    109.20187    113.16198     116.6869    121.33837    126.51036     133.5332    130.71764    129.13714     122.3482    118.84485    114.60416    113.58369    113.44824    112.83885    118.83108    121.89562    132.80695    145.57603    154.10593    161.41039    158.55273    146.14618    136.00992    131.23464    123.52563    120.11247    115.24116    119.35146     117.2354    114.78015    111.48353    110.55854    109.76616    104.64967    104.87389    105.18605    104.37129    103.55426    106.58246    102.32482    100.55523    105.01583    103.86955     104.5255    105.10434     99.56855    101.85199          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       34.50    278.58607    268.28348    260.13289    246.50752    244.33923    230.88662    218.02376    213.67556    213.66295    207.09349    203.11386     200.2495    204.54657    199.33384    189.59542    185.69108    181.86358    180.57104    181.82201    185.12433    175.31663    182.53055    171.59366    171.78374    184.84905    206.77002     225.9878    231.01129    211.97826    191.75147    168.57858    163.47945     162.0801    161.99326    159.22503    157.51078    154.11975    154.42392    152.82706    154.99803    158.39568    152.53027    150.83077    152.06597    147.50881    147.76874    147.20011    148.28136    141.79007    144.73291    145.69888    141.87197    141.88013    142.44072    142.49434      147.838    148.59952    156.24216    151.55309    151.96489    146.32411    139.40115    141.77258     141.9686    136.74966    143.19561    139.41626    137.97939    146.17556      157.317     175.3837    184.12423    191.79755    184.32419    165.43871    156.56167    160.55357    167.13086    169.45944    167.81368    163.32548    170.82432    169.24397    166.96486    161.05752    148.36385    142.95047    137.91855    142.74537     154.1504    167.29063     182.4638    182.57281    173.36432    157.73026    142.13929    131.63566    125.93511    122.83626    125.02789    126.57534    126.53868    126.25339      125.236    119.96669    117.64373    116.47366    115.88456    119.00774    120.53448    118.55435    116.52938     113.2434    110.35136    111.06291    115.21582    115.89148    113.54966    114.29853    113.00628    114.15973    112.69397    112.98251    116.11964    121.99942    127.85419    127.57688    130.57689    128.00221    124.75457    121.77093    120.07504     112.8423    107.57572    113.41489     118.9029     120.1401    133.94077    144.21333    154.79716    157.52932    155.21219    145.33389    138.56467    126.50629    120.50578    117.76406    114.97581    114.45056    113.00148    114.97866    110.46802    107.11361    105.89435    104.60105    101.58223     101.9348    102.79443    103.94244    102.89539     96.99418    105.67094    102.61096    101.12957    102.32587     93.42238          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       35.00    281.24476    266.67881    258.85469    246.35849    244.39066    232.38722     214.4392    214.48518    216.58712    205.25955    204.01669    200.76599    206.46853      199.624    190.40143    191.87682    183.64043     179.6366    180.33769    182.30527    176.79745    178.60887    174.04587    172.05388    183.73758    204.31058    227.56341     230.6785    207.61025    192.61256    171.87932    163.22626      161.242    161.98424    161.04175    156.12387    153.20339    151.28347     149.3792    155.67911    157.53479     155.6966    152.92241    155.11886    146.61927    147.91684    140.32337    149.46546    147.33498     144.5941    148.13273    145.51375    143.84608    144.88091    141.80038    145.98892     148.7682    152.74824    157.44786    152.25204    144.50837    141.69054    137.87847    136.78251     144.8742    142.99458    141.81927    139.85575    142.12225    158.23981     171.3401    184.99182    187.69289     185.9318    169.78744    154.49401    156.42693     161.4138    159.67788    162.55527    160.06289     164.2767     171.5392    172.61344    169.94447    152.17892    142.85623     140.4881    144.26286    156.84763    167.08358    182.02783    182.79934    172.78181    155.96236    140.55457    132.15229    128.81209    125.56844    125.84898    132.76797    128.14355    124.64965    126.92592    120.58581    117.34399    120.07438    120.23188     117.0118      120.493     120.3572    119.57959    116.34677    113.60278    112.17381    113.05789    112.87114    114.34313     115.1995    111.14772    109.66758    113.83327    115.28628    117.26383    121.58195    127.54777    129.95982    128.06602    130.45825     123.4761    118.25588    117.82599    118.86195    113.65757    110.45472     115.0666    118.11171    133.58553    141.93288     152.7577    159.23222    153.42594    149.63825    134.74726    127.09316     121.1674     116.2694    116.56576    111.38457    115.17385     111.8389    107.33675    103.57592    103.08309    100.30604     100.6288    103.32078     99.46631    101.75506    100.62554     98.91233     96.25531     93.52479          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       35.50    285.37383    264.50659    260.03492    246.81635    243.38485    232.65895    216.35808    214.67668    216.53167    206.57306     207.2768    204.11287    204.19056    197.45004    187.61667    196.53423    183.41265      184.224    183.43415    180.33906    179.36302    177.44587    177.54439    173.44622    184.64474    207.00836    230.76033    236.02841    210.42961    191.52448    173.02118    162.39313    160.95419    164.21511    159.95109    159.83827    158.78361    154.65686    153.50335    156.22592    151.00311    155.13849    151.10434    152.16748    148.83584    147.22531    148.13502    143.19081    146.81571    145.23562    142.99448    148.83288    141.35458    146.70197    144.36322    141.53382    147.11182    146.58754    155.37332    155.81939    146.97893    142.03469    145.59728    139.45227    142.01163     135.9515    142.37322    142.53481    140.46112    156.11221    171.53055     185.7506    189.56299      185.883    168.32336    159.48593    154.36998    165.47809    168.57993    164.59477    159.40144    167.16713    175.45874    176.50743    169.27932    158.44825    147.29849    140.59322    141.76491    153.12422    173.04605    180.29446    182.52027    172.51664    154.79387    142.02463    131.85961    127.34072    126.26434    127.34456    128.08441    128.97451    127.32353    127.47882    123.78173    118.63581    119.94866    116.05831    117.11212    121.11842    119.72222    120.09837    118.42893    116.48786    110.79045    114.13732    111.26133    116.62533    112.05589    111.19792    110.71532    111.89518      112.718    114.40891    120.68361    128.00908    133.37947    131.68809    126.89389    120.70206    117.89659    117.04252    113.34415    111.47121    114.46237    112.07124    121.81852    131.80576    142.14139    153.25031    156.76313    153.90875    146.66611    138.57588    135.57528    122.40079    117.69738    116.76118    113.70711    108.45246    108.55221    109.16851    107.50812    102.66666    102.22344    101.34142      97.7063    102.71242     99.66963      97.2656          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       36.00    287.21255     261.7615     261.0101    248.76711    241.30903    231.91979    216.16493    214.64963    213.42364    213.00281    207.44719    208.36395    199.05703    197.36993    189.43216    193.19832    185.76799    186.65583    178.44935    181.80059    184.17902    176.40545    178.56591    174.18868    188.47437    207.61384    228.32931    229.67751    214.84453     193.0931    170.69049    163.10805    160.84121    160.95509    156.32161    162.51669    157.70443    156.33347    152.59835    153.73689    150.34854    155.28842    153.93101    147.08427    153.42115     150.5052     147.0093    146.10948    147.75307    147.32516    145.59355    147.63304    141.92896    140.85442    144.88696    144.89745    146.33878    151.49303    153.43186    155.15464    146.76207    145.91786    141.36092    140.52333    143.51314    140.39988    140.76298    141.22054    140.86844    154.44978    170.34626    187.70626    190.10333    185.18115    171.66055    163.44945    159.68472    164.43792    167.95122    166.92691    163.75854    173.27035     177.0716    181.23497     176.9301    169.57487    152.29112    142.05496    145.49592    154.66325    170.50227    183.13829    182.75348    173.59032    157.45927    144.06195    136.20738    126.12911    128.84681    130.83941    131.58603     132.0715    127.59767     124.4393    120.61404    120.11641    122.02781    115.49291    119.07365    121.48277    120.39317    117.36651    117.06277    114.54217    112.48386    113.39609     113.7252     113.4255    113.63582    112.37664    110.72372      109.614    114.12863    117.87231    116.26794    127.74801    130.45769    132.50687     123.1311     123.4497    120.96837    115.44797    110.81174    110.32344    112.48301     115.4263    118.58015    130.30218    140.15307    151.74394    158.72585    153.21914    149.04794     135.7749    127.69388    122.06108    116.54933    115.73734    113.09959    113.93518    107.71086    107.04397    103.84328    101.17639     100.5042     98.33559     95.71036     92.14685          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       36.50    286.60681    262.40116    259.49258    251.28923    239.69965    232.85183    217.49376    217.57626    209.09122    216.51788    208.86378    208.12021    200.46539    202.48395     196.3256    189.45542    187.78819    186.34364      183.078    188.32375    185.26617    174.12389    172.96657    172.73413    187.80549    209.11187    220.77199    226.20786    215.76581    191.05302    169.30093    164.33004    161.83998    161.43732    160.36772    157.05932    157.22098    152.72151    153.56455    152.94697    153.01637     148.9372    149.73797    147.93917    153.02862    144.80669    146.34475    145.64613    145.15769    144.62559    149.00088     145.7692    144.76592    140.86384    144.60887    142.85544    149.10069    152.08743    152.19359     149.3127    148.03834    143.51801    140.03694    138.80338    139.68124    140.33573    143.71319    145.10807    146.59145    155.93653    168.33739    185.44597    189.00619    182.29494    171.78257    157.61951    159.51812    164.17852    170.22691    167.32792    174.57782    180.87132    186.34938    187.59588    180.51256    171.25329    156.80609     148.5433    141.86959    154.68288    171.52177    183.65006     184.6284    175.57623    159.33302    143.31887    136.12846    127.29376    129.76848    127.78244     131.2397    131.88151    132.23802    125.70803    123.64693     121.4068    115.92656    115.81972    118.62306    120.15396    119.49174    120.16423    118.21849    115.84565    113.06869     112.8047    114.47569    112.32095     112.7656    113.40162    111.62023    110.81432    111.23238    114.55544    119.15852    122.47185     127.7561    135.35342    127.08195    118.01322    116.09393    113.77773    111.17595    111.37693    112.74102    113.88001    121.07309    127.09137    138.60284    147.40148    156.34546    157.11915    149.27147    138.27273    126.73764     119.1773    118.20344    113.29987    115.11337    110.53517    110.13748    105.79228    103.74485     96.92175     96.66872          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       37.00    285.77839    266.32117    257.85651    254.38729    239.42079    235.18576    222.27284    219.31143    204.73257    213.52345    210.45387    207.11688    201.59511    209.34749    195.88196    187.89135    187.94912    188.69867    185.08241    185.21858     184.3444     170.6346    173.45618    171.81358    189.33509    205.75476    217.37281      228.308    213.90397    189.84043     168.1532    162.86947    155.62982    164.99554    163.71682    154.50717    161.59175    156.50279    156.17281    152.15659    152.33008    159.73384    150.51241    152.28323    151.42607    148.96711    148.81777    149.30175    144.64972    147.68804    146.95887    145.85791    147.15591    144.28112    144.40119    146.04329    148.30071    153.97127    147.77921    146.55613    148.87499    141.90308    139.04839    137.90823    141.79351    141.33745     136.9637    143.02592    148.08118    157.44109     171.4169    184.56336    184.71128      181.516    172.74033    162.18077    163.11364    167.64189     171.2154     176.6345    182.04525    185.21127    193.20835     188.1871    177.84718    161.61689    153.68247    141.70024    148.65862      155.541    171.85176    190.00985    185.90582    176.81711    160.77984    151.15863    137.83197    131.53502    131.89516    132.58469      126.484    127.02968      127.239     123.0706    118.89439    121.69042    118.20228     118.7118    119.16268    117.32758    120.38318    118.82641    119.93986    119.31283    117.34154    111.79848    109.52103    112.40117    113.85271    110.45685    112.38751    112.40605     109.0976    111.29529     119.1978    127.39042    125.67186    129.77536    125.19496    118.09736    117.33012    113.39497    110.70924    107.91659    109.56853    113.08704    115.22206     123.7937    136.39813    146.49262    155.09991    161.24032    152.62323    140.14483     131.6096    124.44251    120.27521    116.28946    116.15552    111.77693    106.49042    104.19596     93.58563          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       37.50    286.31658    269.86715    255.40764    252.63959    238.73009    237.60407    225.04125    216.91736    206.93422    212.85103    206.29866    205.47484    201.59369    204.46935    195.16343    188.94817    188.91833    188.20659    187.49816    180.26555    185.02212    175.32965    173.96923    174.12502    190.32297    204.25797    223.46839    231.87109     212.7488    188.60839    169.95697    163.52237    162.64276    160.45482    158.32004    149.13029    163.28765    159.48486     157.0854    156.42917    152.01392    154.60746    154.07611    151.78184    152.76517    149.91583    143.84376    148.96877    149.62412    147.54496    141.06544    145.64564    151.45695    143.89722    148.25009    147.09154    147.79782    146.47918    152.92497    154.07545    148.41815    152.55932    142.94693     139.7914    141.74595    139.42289     139.1048    142.30999    143.20662    158.20931     171.8131    185.85131    183.02386     182.7125    172.56448    169.89786    168.41669    173.18486    175.07672    185.75716    189.00899    187.25629     198.2171    193.55581    175.43725    160.15745    148.97002    144.51356    143.30362    153.52251    169.92232    187.71293     191.6166    184.18469    169.32132      154.738    151.28804     145.3221      136.905    131.61397    127.72807    125.39329     123.9989    126.47682    121.77819    120.09607    121.02978    118.39007    118.53829    118.13024    123.17251    118.30711    118.15622    116.91632    115.50362    114.19389    112.82426    114.83748    112.35872    110.14155    109.30073    112.09821    113.47586    109.82786    119.52655    121.43244    130.12254    129.91502    126.07621    121.46896    116.63617    113.69983    108.79586    110.12087    108.79861    117.00864    118.56875    126.10292    134.72908    150.17318    157.51871    159.67352    154.08096     145.9894    132.63083    123.70588     117.7155    116.28075    113.67529    109.37773     96.99625          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       38.00    290.06376    271.10699    251.47111    250.50739    236.10397    240.91424    226.37259    215.08224    209.62437     214.6329    206.57306    205.67026    203.62921    203.26418    194.95332    185.64305     188.8441     187.3277    188.51645    181.19339    183.82167    179.87094     174.0218    180.47247    186.07152     203.8202    223.11503     225.7577    215.88671    189.81239    177.08192    167.56362    162.37017    160.66472    160.74315    157.20647    157.90137    157.39547    157.33512    154.37029    154.09546    149.36061    152.69031    150.63225    155.14725    152.80282    141.68529    148.74335    144.43619    146.17447    145.40319     143.9445    147.64319     144.3869    143.86517    143.67915    146.49796    149.21293    150.35827      150.971    147.33078    145.31222    141.45974    139.09135    138.99506    139.08256    140.20276    145.39313     149.8639    161.76353    168.26411     182.4189    186.82608     181.5654    170.81905    168.24599    168.25326    181.53201    186.25428    193.74011    200.43393    195.46254    196.42042    195.57664    183.66606    162.06172    144.73141    138.02052    140.69218    154.12269    166.41514    185.84096    190.28346     186.5957    169.51547    163.47815    157.70702    159.81193    151.85801    139.60392    134.20204    126.57779    126.40477    127.17591    124.29881    120.69648    118.77003    116.99969    115.74349    119.17771    118.93563    117.39155    118.69843    118.02254    116.53561    113.40818    115.18343    111.06499    113.56653    107.78642    112.31208    108.45746    111.97042    111.52502    117.17154    125.61375    125.94484    128.41956    125.45151    120.55095    114.73808    111.39298    110.65876    108.57503    110.17752    111.03838    115.08328     126.8329    140.19031    145.41554    158.53644    162.43643    154.67634    143.89037    132.41079    122.05334    115.04026    107.13399    101.61094          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       38.50    290.13738    271.49117    251.62276     248.7493    237.25651    242.49444    228.13335    216.76286    212.24932    211.55767    205.64413    205.14423    203.53488    200.80807    195.23692    182.92988    185.25688    187.92178    186.10008    184.70913    183.76435    180.38604    173.86558    180.20924    184.10483    206.38976    218.84788     228.0226    220.37432    196.78164    175.42317    164.49301    159.47314    159.91657    160.94177    159.34456    155.34685    156.37643    159.06494    160.80969    153.85493    148.52157    150.81379    152.52894    149.61493    151.31899      152.055    151.23553      143.516    142.25514    146.67825    146.27207    145.25388    145.68127    145.14725    146.29071    148.72487    146.75113    151.70087     155.5762    149.39877    146.64839    145.24681     137.3276    140.13687    140.34489    143.24435    138.59562    147.59227    158.14475    172.19892    181.19665    187.21119     177.4105    171.19435    161.67989     169.1703    177.23039    194.60291    205.38706    212.72639     201.9068    199.81958    192.81898    179.83882    161.60587    150.32759    142.65245    145.39979    156.37623     166.4523    187.62965    190.35891    186.37949     172.4046    167.20734    168.24252    160.50714    155.91616    138.56784    131.84739    128.99962    127.59438    124.58142     121.1933    120.62427    122.12728    120.23778    119.79425    117.80576    117.77715    120.88697    119.99472    118.63599    115.61364     111.8976    114.02863    110.74644    112.67873    112.90885    108.07723    106.93757    108.51309    113.88538    114.62205    121.96387    125.39579    124.62999     127.0401    120.20474    115.82959    114.68649    108.84773    108.47196    109.93388    111.72034    115.08798    125.26003    134.14025     149.3126    153.71196    158.27408    148.42717    136.13575     126.0409    112.66724    102.39679          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       39.00    287.72543    270.95047    255.33749    250.17394     240.7415    241.50097    227.37987    217.59905    213.29503    211.06046    203.80263    206.35917    206.32709    199.47344    198.51646    187.55114     185.2742    184.04381    185.05129    185.38718    182.29994    182.00036    175.15334    177.69542    186.34531    204.21823    218.17738    224.58291    214.81874    194.84038    175.52598    166.84673     157.9487    159.69819     161.6627    160.85395    156.67514    160.48417    159.27423    157.19872    156.34579     150.4931    153.12761    153.14486    151.60453    149.59957     154.2561    142.33216    146.71701    145.89688    145.16761    147.14658    145.96702    144.95513    144.51951    145.18383    147.64275    147.97601    153.23437    148.80852    150.91463    144.24571    140.34762     137.8959    137.50965    136.06133     141.0243    138.60932    146.51583    157.83378    172.09726    187.85056    184.59406    182.46244    168.00052    161.94203    161.89674    177.30001    197.98542    214.09028     212.4496    204.06056     196.0397    184.28431    180.82185      165.693    146.91348    140.87506    138.06742    154.41953    170.38438    185.51927    192.24721    188.74331    187.35066    168.42553    161.69881    155.63756    148.57704    135.15963    129.57441    129.67686     124.9682    125.39041    120.79562     118.6642    116.90068     119.5388    115.96711     119.8472    120.78868    121.06709    115.48073    118.43443    111.68224    112.74882    115.58902    114.00024    112.71809    112.80031    108.63914    107.93016    110.18913     110.7773    119.21155    122.70104    122.92407    127.93327    124.63499    119.90964    114.13872    113.08643    106.71887    109.67386    107.10053    109.06326    111.06683    125.19961    133.32918    142.23835    148.17892    145.53228    140.36909    123.86418    109.32016          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       39.50    284.38031    267.11881    257.53623    253.91851    241.91897    239.15104    225.03359    213.65163    212.71254    213.00534    206.12903     208.5629    206.65761    200.83538    197.91691    188.98248    189.98886    181.91984    182.17122     182.3943    183.24984    182.71491    180.02293    175.90477    192.52108    202.63825    218.28035    224.70951    214.70928    196.56011    180.07964    165.93151    157.74851    159.28333    162.43085    160.23689     158.0798    158.36821    159.22196    154.51098     154.6489    151.23462    152.78199    149.11928    150.03922    150.83313    152.83821    144.44228     144.7705    147.65299    143.71887    149.95596    146.40273    147.64663    145.25134    151.76475    146.38723    151.57683    151.42885    150.05109    148.06955    143.82761    144.26456    140.22351    136.45274    145.81193    145.42176    141.70956     151.1315    154.16576    171.17162    184.53287    185.86298    181.40848    171.41653    161.60028    160.85869    164.60795    187.87335    207.86175    205.31705    199.51579     190.6128      179.628    175.57823    168.49805    146.63829    142.45795    141.53138    152.11387    162.23021    186.48109    204.06282    204.99228    200.56872    181.65792    162.31419    149.26051    138.42015    131.52534    126.13139    126.67043    127.13084    123.55243    122.44406    119.43511    119.59917    116.64687     119.7373    118.75767    120.91142    117.88597    119.28372    117.68803    112.62325    113.14708     111.2065    113.46455    113.47864    109.60095    109.12454    110.18982    107.41239    113.04837    116.95572    122.59968    124.43445    124.81362    124.34517    122.31675    116.89229    112.70724    109.35168    107.07181    103.72837    109.09216    110.26942    120.56035     129.0471    138.72916    139.83621    130.75903    110.75964          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       40.00    281.51531    266.13197    255.31373    254.97731    240.32396    239.76891    229.06296    213.81136     214.3973    211.65534    208.08859    206.98265    204.04807    198.92546    196.45207    187.85148    190.78208    180.07532    181.64096    180.71528    184.64734    184.47544    178.13508    173.63576    196.31628    206.32765    219.59285    223.16857    218.43194     194.7952    171.65792    161.18575    164.80411    162.40341    164.53761     154.5832    158.37374    160.97752    155.42415    160.85349    155.83006    150.21186    155.97674    151.34956    150.64212    157.58323    147.03025    150.79335     148.8123    148.57498     144.1685    147.23569    146.59113     148.8987    144.74777    148.90287    147.08156     153.0416    150.70388    149.09949    147.85828    148.30187    144.91375    141.90946    144.88156    144.28174    140.75717    144.55508    147.75348    156.26986    169.10801    182.85593    180.73706    179.61646    166.35693    156.75508    161.67194    164.90073    178.43372    193.63296    196.09584    189.12729    184.73597    182.23782    176.43373    161.57565    148.35612    142.52406    142.12887    151.36609     167.2838    186.49058     201.3207    216.28267     213.3947     195.9427    170.56169     146.6278    131.94867    130.85372    124.35234    130.48417    126.55152    125.28394    117.31864    121.38795    118.64441    118.51162    117.82509    120.26654     120.1058    118.50658    115.84912    115.02492    115.10384    114.57506    116.37849    111.68444     115.4158    112.00536    111.17261    109.71778    109.04405    108.43823    115.94727    120.85431    124.88452    125.73487    125.30858    121.23014    118.78286    109.97706    106.84839    106.14233    106.70968    103.74788    112.13369    120.04592    121.17401    123.77184    117.22622          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       40.50    286.75015     268.0617    251.91924    251.96678    239.96203    243.04056    232.70779    214.54309     216.5674    209.46769    211.71924    202.76835    207.37989    196.64717    201.69326    190.53783    190.80154    184.61106    180.41401    176.96464    182.38444    179.13363    177.79649     171.8849    189.01845    203.47797    218.03051    222.64326    215.32581    183.34136    171.73304    174.24515    160.77171    157.32025    162.92608    158.33546    156.74614    156.45248    160.33793    160.90633    156.19838    148.76474    147.75503    150.49763    150.31554    151.96429     155.5093     150.3175    151.86602    143.67367    150.48209    150.20881     147.2047    149.14586    145.99947     148.6261    148.85708    149.99949    151.97786    152.72429    148.72571    144.10841    143.19907    139.99808      140.102    136.90569    138.03982    137.81371    148.11879    159.64577    174.51676    185.02326    188.92784    178.22392    167.84466    155.56287    161.04097    169.27056    176.66605    188.15073     186.4728    184.38027     181.3186    183.39444     182.0673    169.95862    145.70913    141.68798    141.51392    149.59569    166.65646    183.01214    197.48889    214.19874    207.18733    194.79074    169.76022    149.10154    134.04773    131.96688    126.45702    126.39193    127.71716    124.87471    127.92134    118.62363    117.38162    120.02643    116.18756    119.44121    120.04069    121.25247    122.48274    114.62425    114.74019    113.49448    115.82951    115.76669    112.16187    115.30835    113.49656    109.66268    108.09425    110.47412    115.26361    122.88072    122.85466    124.96976    124.10452    120.02323    115.59971     109.9389    108.63287    103.78454    108.84882    105.78421    106.68795    111.10431    107.06517          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       41.00    289.22033    272.90133    254.54962    248.85894    239.50923    246.52073    232.14662    213.92493    218.44533    208.63963    211.33382    200.45015    206.06721     198.0486    204.66731    190.98614    186.86023    182.53509     182.2138    177.66149    179.35426    177.17251    179.89432    179.68161    185.47242    202.37715    211.00583    214.81504    216.31288    191.93826    171.90625    173.33358    160.12654    161.35543    161.03716    162.13242      156.934    155.61767    163.36773    159.14357    162.17592    151.90031    151.14125    149.16253    150.61262    149.80889    151.60905    150.01461    151.44838     147.4229    143.94334    148.10302    151.42494    144.60613    148.14564     148.8549    151.58237     148.2545    148.03931    155.86906    144.57728    150.38795    144.40313    144.07157    138.96074    144.76585    137.41005    142.13867    142.79524    157.76655    174.68817    183.34173    189.64249    187.95258    167.83961    162.90688    162.35976    168.41309    177.05616    183.01363    185.78192    185.58483    184.82723    186.71188    190.90131    170.89848    153.02934    145.03348     141.3821    155.33964    166.86998    177.49346    190.47939    191.71735    193.34036    180.55663    165.87269    154.10918    138.30887    128.39448    125.02066    125.44467    124.45342    125.98369    122.40611    123.76467    115.65623    116.64276     117.6707     118.4393    120.83917    119.34906    120.70683    118.98107    112.13059    113.52246    113.87787    111.64376    113.98909    111.03745    113.26096    109.69626    106.61753    109.57385    112.03516    117.11302    124.61624    125.96268    122.58432    117.62619    112.42524    108.56213    104.88732    101.80921    103.26088    100.71112     99.38291          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       41.50    288.91445    275.07965    259.00378    250.64897    238.98254    241.69173    235.85455    214.27483    216.44627    208.77396    209.30201    198.69054    208.99383    200.77042    202.79256    195.69755    186.79092    183.85721    192.07751    179.56124    180.28541    180.55722    179.47355    182.57008     184.5744    201.03536    216.30188    214.30548    207.54931    193.86299    174.84767    166.11162    163.31748    165.52775    162.28841    161.47369    156.96755    161.05646    162.67342    161.19395    153.17903     149.3874    156.32195    150.55012    153.06442    152.43438    149.19268    152.54461    149.75184    147.63827    151.20323     145.3221     143.7247    140.24924    147.01475    147.68061    147.60082    155.04114    152.25028    151.23691    147.95363    145.13427    146.35095    144.07388    142.52004    141.56133    143.35828    142.37027    145.08593    157.00702    173.34628    180.49671     188.3535    180.87104    169.20304    161.99078    163.84182    167.04812    178.45097    186.68778    186.81905    184.94639    183.88756    186.38502     182.1906    167.86998    155.80579    141.92575    144.00629    152.37118     166.7973    175.09882     181.0434     183.6895    178.09728    168.52367    163.04546    153.88193    141.77964    127.89709     126.8394    125.15666    129.55086    125.05137     123.6507    124.31804    120.93084    115.98321    119.45264     118.4603    121.48964    118.23659    118.25164    118.56422    116.97751     115.3917    111.96428    114.63012    112.87214    111.30312     112.1153    111.42511    110.78472    109.61575    110.52502    118.82621    125.24831    124.05359     122.2167    120.58161    106.95249    110.86184    104.74371     101.7327     97.32291          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       42.00    286.74839    275.16329     262.9588    253.33121    240.17542    237.84023    239.27336    216.98948    213.41437     209.3803    205.21559    197.73262    210.52494    200.48503    204.53984    198.39956    190.67616    188.76428    196.80056     181.5864    179.21342    182.27444     174.2748    180.24111    185.27018    205.51038    214.37177    217.09741    213.49234    190.02389    173.02878    165.93839    170.10356    161.60812    163.12035    162.40223    155.33217    161.80011    157.73064    159.81286    157.74875    155.07109    158.37527    152.34415    153.86541     150.4762    147.07606    153.32116    152.94548    150.24034    146.51494    143.55722    145.40149     148.7469    147.81538    145.42896    148.01356    150.31415    152.82572    150.65866    146.65919    139.69074    142.06981    142.09918    142.94259    143.62652    142.70127     143.4191    143.63385    157.90063     170.9345    181.96538     186.2661     181.5274    172.92498    164.66958    159.78312    172.11199    177.17824    181.50721      181.123    186.77825    182.39696    182.33908    170.75985    159.31631    151.29188    140.39249    144.95457    158.51349    167.10809    176.34916    181.13643    181.33903    171.16927    163.64795     149.9832     142.9456    132.04966    130.99177    125.00507    128.43049    125.64292    129.49012    123.40281    121.98818    123.97826    117.34999     117.9866    116.42847    119.91877    115.55629    118.40498    116.59836    112.49895    113.66013    109.77418    115.46564    111.82134    114.39482    110.41248    111.61394    111.76838    110.23741    111.38275    116.73615    123.66803    122.26785    122.95658    117.21278    111.31964    106.44182       98.435     94.20531          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       42.50    288.15688    275.14147    261.04966    255.86759    243.37597    234.85671    232.94109    217.86599    215.35757    207.58949    203.32125    199.68152    206.00857    198.40897    197.57309    195.83228    194.16913    188.63404    192.78775     184.0652    177.18231    181.41966    177.97888    181.02192    190.59797     202.3196    216.82857    222.11106    211.42972    193.89793    176.72898    164.40151    167.89842    167.86368    160.95383    157.92745    162.47547    161.93206    158.11002    161.98997    156.68488      154.137    153.85191     151.1114     152.5824    146.41276    147.73336    159.92075    149.64591    152.73697    147.02025     141.5313    145.61057    149.55057    147.19202     148.4674    152.58664    155.21385    155.87158    153.35245    145.52784    145.23392    142.55833    140.02848    144.76969    145.72841    139.35259     143.6605    147.21277    153.23203    170.60039    183.40254    188.87811    184.86609    172.58593    165.39903    157.56851    163.71425    171.32386    176.36899    177.60214    180.51574    183.15937    172.06767    163.28232    151.26202    143.74723    140.61847    143.30715     153.1804    168.03394    179.07647    186.47145    178.47239    175.14118     160.4065    148.67185    139.09517    136.07658    127.57571    128.13542    125.23345    123.67606    127.11355    120.08938    121.21702    116.96738    120.63457    120.07317    116.27026    120.82231    118.57803    118.02019    116.55193    114.43219     110.8941    114.52917    113.14122    106.92684    110.97078    110.61487    110.96892    109.64625    111.96706    113.86144    116.22691    117.99271    124.17571    118.75698    117.89751    108.09937     97.09608          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       43.00    290.25948    275.01389    257.74284    257.99119    247.59023    231.67274    227.23506    220.94896    216.86041    207.38423    205.26417     203.6604    205.20381    198.31773    194.62601    197.85164    193.70878    187.33761    192.91642    182.84269     172.9363    175.46635    183.61577    184.87029    192.39557    199.53726    217.04709    218.65727    208.30039    191.34785    178.15542    162.70626    168.61459    167.06177    156.33284    158.02276     161.5071    160.80377    160.16947    161.56777     157.0639     152.1661     153.6456    149.03548    153.42575    154.01062    149.22627    152.09033    150.59341     151.2566    148.04118    146.98729    144.88896    142.00829    148.73608    148.13161    148.70165      154.816     153.9589    151.77231     145.9603    145.74558      144.998     139.0129    139.81143    143.26782    142.90176    145.93174    141.99784    154.78112    169.94857    185.66172    186.76567     186.1201    173.08449    163.29523    159.70338    155.97254    163.64212    167.41368    169.15492     170.4411    168.97286    173.56786    157.84847    148.25413    142.73373    142.12272    143.16075    154.06858    169.74023    181.08172    190.63593    181.27435    170.22288    161.14992    146.85343    136.15696    130.94569    128.71221    126.95356    123.36638    126.93013    124.02513    124.42144    118.85602    116.91071    119.84261    118.80524    116.42223     115.9203    118.44347    117.49417    118.25362    115.69186    114.82254    115.40895    108.46823    111.39612      110.508    110.11998    108.28301    107.23045    111.48448    112.73086    113.85954    116.59336    120.61499     116.7434    105.23916    100.39165          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       43.50    292.87852    274.62262    257.64006    255.69721     247.5195    232.11653    225.52746    225.57655    217.49078     211.9329    207.28825      203.554    202.91667    201.04219    196.34234    198.92117     190.6084    189.95431    188.54271    183.10924    182.86766    176.63444    183.65311    185.80741    191.83696    202.55356    210.95802    217.58669    208.57031    189.92693    178.49881    167.14018    168.95305    168.51835    160.72442    160.26196    162.04958    158.71489    155.79393    158.28647    161.41652    157.74035    156.83722     155.9473    151.39798    147.87623    146.91251     149.4567    149.47899    150.15579    153.74855    148.43791    146.40492    144.51598    146.89203    148.99635    145.83155    153.75374    154.03094    151.51046    143.29337    146.75044     144.7728    141.97442    143.50287    145.27725    142.42377    147.26777    145.03612    153.90852     169.6464    176.06888    184.52413    181.26321    173.91433    162.33667    150.73018    155.38943    156.78143    158.45241    163.73909    167.43817    162.84985    161.67455    158.16827     143.8381    143.05494    138.29003    145.42385    154.92093     164.8079      182.469    187.25715    184.72956    167.49107    154.86985    142.03508    138.31522    132.37123    129.61249    125.51319    127.39111    125.66176    122.22705     121.3638    119.26669    118.96326    117.74133    118.37204    118.54981     116.5286    115.68889    115.49405    114.88856    114.17944     116.7609    112.77173    113.02158    114.08084    112.09445    112.46725    107.61835    108.67938     106.6349    113.81162    113.73171    114.28479    111.23374    106.73745          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       44.00    294.61892    273.31557    260.85314    251.44089    244.78184    234.45578    231.33494    228.50654    221.80526    211.53343    206.26954    204.42878    199.38998     201.0543    198.76775     195.3779    188.30158    192.31996    188.53339    179.92207    197.80698    180.42514    186.20883    181.69472    189.89566    208.26406    208.59031    220.24908    205.46695    189.32573    179.73325    169.38624    166.65305    160.64939     161.3941    161.43258    160.37939    162.18088    161.14204    157.22683    159.38002    157.37488    157.91336    158.28859    149.33223    151.35328    147.87329    150.84224    149.73528    149.83535    146.09298    147.39278     145.5395    145.82295    145.17385    144.21884    148.16341    152.65092    151.34071    151.39066    147.03541    144.30611    138.98956    143.92408    141.86981    142.04371    144.24703    146.08574    147.24484    154.91706    167.18167    182.17713    183.96075     184.0009    170.03879    165.19893    158.92701    156.20305    149.73871    154.18925    158.09311    163.75206    167.22789    161.10882    152.45563     145.2361    139.43127    138.78823    143.83317    151.39003     160.7007    179.17111    182.70649    184.45506    169.93497    150.84798    140.03574    134.48428    130.29958    125.99964    126.31563    126.08895    125.03832    128.39949    120.84648    120.21274    118.17383    114.78703    118.22344    116.98177    117.78342    117.98837    113.40124    115.39536    118.82579     113.6007    117.73841    112.94052    113.66001    108.18814    109.06642    108.30767    108.70771    109.06525    106.03272    111.21417    109.27306      98.4887          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       44.50    296.00066     271.0247    263.91401     248.2393    243.41438     235.7334     235.7994    227.25552    224.00482    211.54996    203.24832    208.05521    199.58423    198.14237    200.05136    193.24813    187.05437    190.23051    184.60316     176.7859    185.45223    177.66707    185.08471     179.2846    187.03354    203.48099    213.74059    219.03024    205.47939    187.82996    179.79218     176.8247    168.27546    166.10855    162.66629      160.864    153.92636    163.11359      159.573    162.83916     158.8544    157.35594    156.32627    154.59636    152.99621    152.66064    154.07648    146.95211    147.35094    153.10953     147.2402    151.53033     147.8655     144.9783    144.15052    146.82704    149.14746    149.18679    155.27736    153.77074    147.10321    145.20756    140.34746    141.61333    140.56307      143.286    143.73015    143.04541    147.36787    161.44376    169.69169    180.89174    185.00504     182.7537    176.10768    165.99893    155.53445    153.39828    154.07442    155.51414    162.96127    164.40197    168.22434    158.85716    151.77633    149.73715    142.00173    137.67592    138.34998    149.46229    162.89751    176.85851    181.32054    180.17997    173.34614    159.04549    143.79628     133.7642    129.18789    130.44114     127.8342    128.56194    130.37164    125.77275    122.76928    122.76597    118.85701    118.21332    116.80033    116.90749    120.41226    118.45759    114.93904    118.36639    116.22588    116.35325    113.27423    111.59338    110.26454    113.83196    109.54852    106.04588    105.95703    106.12071    105.94784    106.68655          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       45.00    297.59598    268.15989     265.0991    247.13109    244.20536    232.64926    236.17589     227.0047    223.62497    214.46915    202.33031    208.56935     203.6492    197.95726    196.77723    193.08311    191.99376    189.66891    186.56258    183.04166    177.75378     179.0207    180.28305    178.58533    191.61081    205.15243    216.63432    211.03473    205.98839    189.88075    174.15036    175.07109    174.60394    166.67195    165.01429    162.83151    154.27159    160.15907    160.20524    161.04131    155.07904    158.15816    157.26308    154.33553    151.24428    151.88953    150.57169    149.22852    150.67532    154.03146    148.03451      156.627    142.46114    148.19216    153.54031    146.81634    150.41789    153.80448    147.51161    150.65739    146.36225    145.29968    139.45265    141.48785    142.00406    144.39171    146.56013    145.75417    146.85252     160.2509    170.62632    174.77479    180.91514    181.18383    173.47382    164.52559    156.07911    157.00687    160.99879    158.74256    164.81103    166.72281    170.23426    163.90298    155.84679     149.6637    139.83535    142.09969    141.26676    151.60692    163.38182    178.19713    181.22831    176.93579     173.0526    161.14783    150.50321    141.42217    137.83926    128.26547    126.73037    130.29542    127.85016    125.38877    126.00818    121.49611    118.52917    114.38236    117.80218     120.5491    118.14907    119.56921    114.24995    119.27817    114.10569     112.4949    113.43407     113.5435    111.95078     110.0367    108.47253    108.68655    105.05396    101.06563     96.06534          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       45.50     298.7036    272.53843    265.23355    246.94276    243.91918    229.02566    233.38203    231.21736    225.31585    215.57301     205.8941    206.22063    207.23777    198.44059     194.1929    194.31484    198.93703    190.75785    187.18619    187.76188    180.29991    181.57326    175.14641    178.16037    194.78701    204.16766    221.27795    210.52377    201.31868    193.44404    179.65945    170.61615    168.93315    161.87012    168.22192    169.22079    158.73901    164.00987    156.30206    159.72654    164.08414    160.73246    154.26334    153.90178    154.01726     152.7973    148.14207    156.82847    154.04992    150.45684    151.19863    151.25616    150.47156     150.2991    150.27786    145.97139    149.86524    152.69952    148.68566    154.58989    147.39165    142.62084    142.69328    142.66743     141.2402     139.1791    140.22548    140.97637    147.71684    159.79461    169.73236    184.30131    183.64628    177.22352    168.69303    157.48349    154.76301    160.02391    158.34589    163.26971    164.81489    172.86983    178.15225    169.31559    153.24052     141.6293    143.71787    138.37886    140.10539    150.42642    161.91434    170.63542    182.33223    182.14733    175.61522    167.79027    158.55348    147.17875    136.88037    133.13474    127.97129    127.81059    131.48079    125.08435    126.02478    120.21828    119.69369    117.25865     116.0112    118.03719    113.03565    115.40508    116.92522    118.07958    114.81262    112.88801    115.61178    115.35683    111.91627    107.20865    108.33847    106.43988    101.91361          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       46.00    300.52495    274.62734    263.40785    246.52263    242.48304    228.59561    228.57911    230.74365    223.14962    217.78616    211.53917    205.44471    206.71951    199.10363    191.71435    192.45842    193.15682    190.00604    187.28786    184.36095    178.67287    181.10582     174.1935    177.05279    189.70307    200.93237    218.58635    212.56211    205.29522    191.37103    175.35092    170.77127    166.86078    166.39366     168.0897    165.76292     162.3746    158.62703     157.5476    156.57672    158.80176    161.39893     157.3758    158.31719    154.08139    150.28027    152.96527    143.53859    152.71514    150.77064    152.49461    148.32355    149.89795    145.90086    153.13521    146.23498    151.20581    149.12155    152.79393     152.2494    148.52049     147.5545    143.06053    142.86656    142.47356    140.03721    138.02057    139.80854    148.88271    161.04233    171.33498    182.66989    188.90996    185.30656    166.77896    159.88054    157.98947    158.79341    161.65445    162.92248    169.00575     174.1231     184.0957    180.99979    159.68611    149.23346    141.09015    139.93592    144.37167    151.74852    159.64323    168.31059    176.73231    181.48269    180.17974    168.18072    157.96471    148.64094     142.4506    134.42286    129.83425    129.51611    124.65793    125.41249    120.85568    124.05183    121.02212    117.69249    118.29642     117.8916    117.12341    117.43419    116.16967    119.59125    118.27764    113.93731    116.83029     111.2777    113.84224     108.3946      106.366     96.79719          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       46.50    303.98897    274.42782    262.60532     245.9174    241.25552    232.42628    223.96859    226.61417    219.74852    222.70078    213.33968    210.53428    204.67546    199.07424    193.92892    196.85811    190.24369    185.44351    189.53644    180.86436    175.57333    175.67988    182.52926    180.88146    189.47614    202.61968    214.40642    220.46732    209.45038    189.20204    174.74947    170.56555    168.16922    169.53068    170.98363    164.12764    163.92927    159.48173    155.54377    158.28675    157.06366    157.09837    158.05017    155.86209     159.4977    152.67387    153.59853    148.10369    151.66656    151.63483    150.74626    150.53115    149.92592    152.00359    149.79091    150.48987    149.89236    152.33773      159.181     151.5359    152.20181    149.71414    146.34676    146.50245    143.87714    143.25577     142.5173    144.84822    145.54865    155.43069    169.35808    177.73502    183.70781     178.5622    167.54961    161.49677    151.49358    157.67462    161.36786    167.20449    170.30285     176.7438    181.08338     185.3921    171.88003    152.21872    148.15436    143.95527    144.24026    147.35145    162.81035    170.68833    178.51575    181.57065    176.08164    175.21066    156.88806    148.00735     140.2431    132.35401     128.8871    127.65556    124.71067    126.50894    124.45635    122.39583     119.5054    119.73171    117.91149     115.7053    114.90093    117.86891    115.05385    115.66266    113.40417    114.36806    112.96807      110.493    109.23473    106.66934     98.69561          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       47.00    307.46613    273.59453    263.05707    248.69192    239.01064    238.14469    227.98665    224.52176    221.26843    227.43406    211.29015    208.91098    202.74866    200.71584    196.61119    197.59426    195.16167    183.53013    188.51069    186.25725    168.56849    179.39228    181.66735    179.76237    194.70926    197.63119    210.31601    220.04971    210.00907    188.15241    175.91338    165.86409    166.09594    170.14584    165.01509     160.0763      161.511    162.68973    153.03295     154.6673    154.41837    161.79988    153.26241    150.25819    153.30596    150.04252    157.94017    150.12268    151.62048    149.35616    148.87944    145.96468    150.75066    142.83725    151.61993    151.19263    147.06375    155.45845    156.06823    151.07814    148.55432    149.77756    141.60253    141.68837    144.99122    146.78906    144.84191    141.84751    147.79676    154.77279     170.3367    178.84063    183.81207    181.30305    168.77165    163.63156    161.56732    156.88341    167.06318    166.76243    168.35639    177.60122    181.89396    177.40415    167.90995    157.64413    147.50524    137.34931    140.00031    146.36016    162.45564    172.12193    177.44501    179.52117    179.87743    166.10031    154.61114    142.05319    139.33897    135.03036    129.57195    125.55877    127.17487    125.50292    123.99593    122.71009    117.11456    125.62855     121.5382    117.47455    117.71205    114.36895    116.47015    113.04602    114.95008    109.94268    115.55181    109.05838    104.54491     100.3185          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       47.50    310.93045    278.26036    264.70594    253.22024    240.17815     240.0143    234.24382    223.54801    218.79168    223.48451    212.25739    203.95247    202.03795    202.94122    196.66611    196.61521    198.50844    183.15338    188.33093    190.34431    177.65941    185.50406    177.73719    180.18053    190.33302    200.15863    210.55465    213.45481    208.15025    188.89675    178.93869    171.32058    163.70789    166.24652    166.24546    165.48425    164.65543    162.35783    157.57272    154.52105    155.76505    158.15587    150.00477    152.41225    151.13315    148.02467    158.86873     154.3952    149.71267    148.94743    146.88281    152.08277     148.9042    151.05072    153.11207    150.92442    154.88542    152.05368     157.8853    150.98884    149.91546     146.1803    144.20228    144.52902     145.9822    146.26235     140.8773    143.03206    146.05875    157.03114    166.55433     178.0017    180.68343    183.59555    173.79967    159.49003    161.83067    153.94045    166.23815    169.32557    170.81301    172.95416    172.33629     170.3765    163.93909    154.49505     148.7483     141.3261    144.59932    151.68616    162.59152    177.36536    181.33278    182.25441    176.98825    170.12493    157.40229    150.75251    134.04273     132.6302    133.24363    128.09977    125.07034     122.9436    124.72656    119.55366    119.83925    121.43001    115.67549     118.6245    118.05549    118.12793      114.875    115.17373    116.23393    113.70153    110.48831    105.85092          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       48.00    313.94768    283.02174     264.0202     256.1537    244.28802    238.41594    240.54137    224.35534    215.72835     220.3289    213.18493    200.23727    201.12833    197.88898    198.00311    199.48004    195.04361     183.8508    183.94513    188.78348    179.74062    179.95331    173.40254    182.23714    187.86735    200.00801    215.56094    213.71827    212.65278    192.20479    177.06081    172.33038    163.84645    165.15323    166.34356     163.0346    164.78621    167.47996     156.5666    156.49579    154.68626    153.96922    155.24901    164.62046    155.87617    156.25583    158.87772    154.53208     149.4802    148.45912    148.52752    150.73204    152.27855    149.78678    151.79802      151.089    155.66598    158.41126    157.28573    151.48513    150.92769    147.24215    143.28178    147.64682    143.67132    138.48678    148.51501    143.39473    149.20984    154.96367    166.85503    177.88768    182.49544    175.81821    176.14053    167.32909    161.47395    165.09708    168.33634     170.2469    169.05271    172.93595    166.72097    168.67994    160.66179    148.31597    143.59564    139.43522     139.0318    150.96918    162.56225    171.62506    185.01388    186.43964    177.49991    172.51861     161.1266    151.62467    141.40859    133.76553    132.95242     127.5324    125.49076    128.50564     121.7257    120.69208    124.32071    115.94784    121.76803    120.81722    117.57922    118.36557    116.74294     118.0528    112.33473    111.66608    108.76111          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       48.50    313.30152    287.87826    262.01309    255.81834    249.81938    236.74354    236.61075    227.64008    222.40644    219.68029    212.45553    198.23592    202.11432    196.53683    197.19895    197.52888    191.20808    187.84198    183.52565    187.76607    183.79262    178.93559    174.07378    180.32432    187.39312     194.4435    216.94976    214.83043     208.2338    196.71286    182.75035    170.56637    165.42745     165.3378    163.21824     162.3155    165.43956    160.10571    160.89677    159.88678    154.28349    154.52051    155.83606    153.28972    152.89301    155.72899    155.74874    154.37479    155.53059    152.34772    150.88127    149.12445    149.83066    148.12495    151.08918     154.3073    155.02971    154.51547     154.4938    153.36715    149.69272    144.84532    144.44371    144.88698    142.60209    146.24414    146.44919    145.14735    149.07884    153.83152    163.24812    181.04199    182.19796    185.45874    175.31626    167.26735    169.80161    174.82138    174.51502    178.78672    176.22648    176.77414     170.7017    170.10435    160.83032    150.01624    142.53397    138.85464    140.99364    148.25291    160.99803     172.0422    185.01251    184.70789    180.30531     170.9264    160.56589    147.70579    146.13006    138.40757    131.75801    129.50965    128.68318     126.8319    124.37727    118.91013    120.40398    122.85167    118.67565    117.41663    117.30349    117.76407    119.73721    116.69616    111.28908    105.46311          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       49.00    312.70634    290.85134     261.2677    254.48386    249.55178    237.27483    231.12563    229.10103     226.6723    219.33019    212.79319    195.36699    201.46319    198.16042    197.28119    190.00453    192.21422    189.79634    187.79946    188.89566    188.55469    176.47834    178.22112     179.2134    190.42148    197.23354    212.47518    213.92741    207.67241    197.47546    178.25754    170.01593    159.45653    167.54293    161.64119    162.42287    163.49166    158.25264    155.50614    164.21721    156.89257    160.88699    153.90354    156.20032    155.93829    157.41356    158.25046    153.60291    151.26365    150.70015     149.4884    150.35303    152.20614    147.54242    151.67842    153.81152    147.16512     148.9411    151.32214    151.73664    142.30901    140.64074    145.99341    143.86111    146.18144    142.98489    147.18171    148.15841     147.1407    152.11766       166.04    177.95922    182.62603    183.71924    177.19774     174.8514    172.67775    181.63273     184.2426    192.56224    187.75609    183.20244      179.426    173.13477    163.63385    151.97103    146.15608    133.80742    138.15082    145.60563    162.21383    172.29686    183.73489    188.99631    179.30067    171.22771    154.47819    147.90854    142.79737    129.83958    129.40917    129.28598    125.87919    126.61275     125.6047    121.56082      119.342    120.35393     115.5049    118.09524    115.99503    115.01791    113.39614    114.52633     101.9734          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       49.50    312.90716    291.73628    261.82656    254.30501     246.6399    239.11177     226.8792    228.88005    224.53184    224.00331    211.47493    199.82194    198.62544    201.76938    197.93581    186.50572    192.11984    189.73835    188.58022    184.55805    185.89445    179.96448    178.10354     179.2796    191.90613     204.1678    212.62237    217.34221    207.22665    196.25917     180.2612    163.25424    166.74524    167.74042    162.87799    165.03413    165.95623     157.9587    162.94105    163.84012    158.60354    160.05872    155.24025    154.87941    156.67295    156.58886    156.74107    151.41334    153.62632    155.96813    150.44675    151.87552    147.37563    148.16773     150.7045    148.70601    157.11159    155.60783    150.85148    154.37086    146.58887    147.42527    147.51668    143.89043    144.67194    144.47762    145.08303    147.09367    148.83558     152.0455    171.47521    175.58009    188.98262       183.73    178.81158    170.57291    179.65888     185.0007    195.38483    195.34543    198.91029    193.55769     192.0329    183.96591    173.66894    156.39969    145.65129    143.67322    140.41849    145.72854    159.29935    175.37559    184.79725    186.40541    183.25262    170.90707    161.85877    150.11333    143.21109    135.09381    133.87943    129.75558     130.6145    127.15763    127.93233    122.43471    122.28396    116.78561    120.58019    121.94336    116.99254    116.98106    113.53381    105.88441          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       50.00    313.90104    293.78773    261.29548    255.92991     242.8941    240.41243    228.55339    230.67555    219.49051    226.68087    212.72242    205.01528    198.55983    205.34256    200.68051    192.86232    191.18882    190.04822    186.74687    185.55436    187.30925    186.24589    175.62553    180.91372    189.69539    200.94924      208.871    215.42442    201.69905    197.35718    182.48123     164.6331    170.55979    166.80254     165.0189    167.48114     164.2707    167.37583    166.07937    158.12111    167.12022    157.83645    154.57477      161.222    153.92848    154.40812    159.46475    152.42809     156.6747    150.49704    154.91355    145.83013    148.64921    149.82743    149.07172    156.20668    159.45784    154.09255    162.45464    154.53575    145.10044    146.20391    144.51416    139.31581    144.55051    144.59082    143.15373    145.72129    148.22868    156.15394    170.37294    180.38827    182.75155    182.78078    176.17178     175.0481    180.14221    191.87791    198.84888    205.89717    210.03965    208.11315    207.14808     197.6043    178.44922    162.97002    147.84073      143.683    142.14974    145.58149    155.17225    166.70263    184.02633    186.86542    188.04266    173.91331    171.95318    155.71271    145.69042    137.05363    132.17166     129.1359    126.98553    128.66142     125.0708    125.00854    125.97122    117.25888    119.56699    119.55585     117.4756     112.1842    105.72105          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       50.50    312.78221    297.00078    260.99984    259.03778    242.61814    242.54531    233.16451    228.36163    218.98121    223.04916    210.70899    210.34443    194.35729    201.67314    197.88073    197.58286    194.96913     184.8421    186.44023    193.93212    183.65263     187.0472    175.61012    179.34744    188.17001     197.6678    213.34009     213.3249    200.89237    194.42568    181.58493    169.44404    164.70005    166.03601    160.97364    162.69948    162.92375    161.78054    166.66556     158.1592     158.7607    157.82369    153.45023    154.03821    158.89288    154.42971    155.81436    153.71011     158.1316    161.42229    155.72741     147.7188     150.2128    152.95147    148.45108    151.20432    151.38968    155.14205    156.76427    157.31029    154.17122    150.93536     142.0938    142.48693    145.50126    144.34759    144.38264    144.23768    145.04894    157.55171    171.35351     175.7371    180.56956    182.59287    177.96582    178.45176    179.69053    195.52562    211.62225    219.20952    214.42092    211.83341    217.28953    206.96737    186.53281    166.18838    153.42528    141.41523    140.87641    152.83953    158.31747    170.47616    179.33733    188.34893     187.6586    177.82607    164.53956    147.98031    146.68858    136.23416    131.71602    129.48772    128.56869    125.37154    126.20873    125.17684    123.19003    120.92755    120.11733    117.40039    114.15111    106.26938          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       51.00    313.27767      300.026    261.84117    261.36004    245.00323     239.9732    238.82547    227.15532    220.12945    216.70047    203.69469    211.65345      196.256    198.83276    192.18957    193.69804    193.48907    178.95744     187.2713    191.66273    183.62761    182.06907    183.64028    177.24142     189.9812    199.35719    214.12241    207.39952    200.37122     190.9429    181.06329    175.64811    162.01086    169.26052    163.01545    157.07032    165.72394    159.74689    162.57663    162.87892    163.46014    160.91047    159.88874    155.17425    151.89059    157.00334     162.4393    154.31342    151.86314    153.98581    154.44274    157.69644     148.9802    143.53929    147.67958    154.24632    153.68535    158.27137    151.74301    156.51459    152.19276    151.86974    142.33605    144.10612    139.34492    138.71234    145.47625    146.31475    146.70776    156.24633    169.10449    179.66524    181.61402    186.22766    176.36379    178.42099    184.47514    199.37156    219.66827    230.74573    220.64866    212.01595    210.09436     198.6053    182.92496    161.09307    147.92367    146.78294    142.04176    145.20999    154.59777    164.44938    179.78545    183.47617    184.93783    171.89247    159.56025    151.94539    145.56251    141.21053    132.16791     127.8467    129.21919    124.41233    126.85548    129.07248    122.13704     122.1183    124.74244     114.0246    106.14557          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       51.50    315.46065    301.30618    271.51101    260.06769    246.67914    239.66523    236.01843    227.20612    219.30463    215.82168    206.18684    211.34306    200.79455    200.71192    192.79556    193.25876    192.32645    185.59995    190.07262    185.77636    186.76258     183.4737    183.45729    180.68472    191.98513    202.17486    208.88968    208.77677    209.43073    194.74245    179.95891    176.22641    167.83517    167.61415    162.23291    159.39955    164.93979    163.32598    164.46987    163.45998    161.84643    160.15681    161.57518    156.56125    152.13324    162.80713    154.17231    155.93958    155.19335    155.44441    151.73351     149.1296    149.19299    151.58224    149.20062    149.26067     153.6706    155.89195    159.41402    155.17777    152.11553    151.23167    149.73137     145.6677    139.03606     145.9916     140.8906     146.4233    143.61892    153.20181    168.42604     177.1226    187.49988    183.46917    181.39681    183.43573    184.73651    205.86081     227.3151    243.70341    232.36021     215.6429    196.68238    182.61743    168.55676    157.99591    145.73594     142.7399    145.34442    149.21999    154.80564    166.38722    166.02105    177.02036    175.47415    168.48493    158.41402     154.4094    143.44942    137.43734        138.3    129.78856    127.40366    129.80321    126.56728     124.5118    126.23847    119.43445    115.03393     111.1109          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       52.00     319.0202    299.63858    280.71689    257.34956    246.28455    243.27954    232.63863    229.68373    218.77764    218.80838    204.96789    212.84005    202.97231    198.70054    192.29168    196.91007    194.36682    189.95932    188.91103    188.19085    188.94783    179.17769    184.77704    185.50234    190.75986    203.00742    209.12317    209.22247    208.70473    201.88125    183.54162    173.38509    168.81888    171.94829    164.64407     162.1074    167.32694    166.05488    158.77684    163.78784    167.62282    159.34677    160.40423    158.72277     155.4375    157.15322    150.22836    154.37138    151.26398    156.30087    159.01332    150.57522    153.15634    149.20931    149.44122     153.5828    152.88535    151.73658     158.7663    150.68149      152.028    148.32608    153.73583    149.53941    143.20304     143.7175    151.46526     147.3424    151.26193    161.21783    166.94689    179.44291    184.96349    186.62307     182.6575    179.29743    187.80212    202.53226    228.76908    243.56374    239.58061    213.01778    197.90493    172.78979    158.39594    151.40763    136.83432     138.1298    140.85802    144.82819    151.32442    169.37967     176.4728    173.23486    172.22114    161.85253    157.99141    149.69964    143.35249    142.07677    134.43872    131.75647    129.16783    126.14155     122.3749    123.62077    124.63521    116.83835     109.9601          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       52.50    319.28345    296.11649    289.28083    254.79557    244.13242    246.37952    233.11396    228.79837    218.14989    219.51949     201.8997    213.43238    205.57872    199.84779    194.39446    197.10973    192.38435    189.39341    188.08108    185.36549    188.52812    183.60949    187.35056    190.57584    191.27326     194.0336    210.06912     209.5681    202.44939    196.12747    180.42705    172.08782    171.06645    168.07017    167.04146    163.59684    165.76378    159.95995    162.48988    161.48139     159.2462    158.70298    158.22282    157.18117    160.01803    153.22884    154.13071    157.60027    156.39376    157.01988    153.30621    152.25353    150.90564    152.10271    148.02644     151.0021    158.13821    155.34961    156.63467    159.16455    155.62406    150.54666    148.02783    146.84702    142.10676    142.48881    142.09746    149.15647    151.77681     154.8666    165.40148     177.5921    191.91479    188.35672    182.23693    180.21355    186.33508     193.2261    218.51379    239.35617    237.17997    222.04922    198.55313    170.91692    155.73882    147.15522    142.74159    136.95395     142.1407    147.88102    155.51529    167.12123    176.82288    176.97979    169.05153    160.73856    156.14473    150.25994    144.75131     139.6295    134.57553    134.67473    130.18561    133.41895    124.69943    125.64961    121.61864    109.65734          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       53.00    318.12945    295.28669    291.35632    254.56376    242.51129    242.44837    233.95769    224.91854    217.93706    218.77242    212.53451    214.87013     209.3965    206.69103    196.58562    194.14686    189.04566     186.7119    181.46607    188.38348      190.113    183.29785    187.73322    188.83112    190.21472    197.81536    212.56973     211.0411    207.99053    195.54674    176.26683    169.78921     170.0498    168.45034    170.04883    163.45968    168.85796    164.42596    164.09336      156.241     163.0101     169.2337    161.11589    164.43005    161.99135    155.06382    159.79806    157.08634    156.94088     154.7301    152.06758    156.14607    153.70924    149.96281    153.42263    153.74385    151.04248    153.20014      158.442    156.91337    158.37593     151.2039    151.16206    145.53391    147.47571     146.7417    143.91798    145.99777    145.99114    153.40855    162.03763    181.37762    184.10948    184.15999    184.38172     177.2487    178.39786    193.90216    207.52969     226.9739    230.87074    219.63857    201.55872    174.86961     157.4494    147.14927    139.87693    135.23083    137.92339    146.15124    160.66136    164.49728    172.89902    177.90126     179.9667    166.95527    156.99273    146.43927    141.65406    138.87574    131.04042    131.91736    126.36808    132.57143    124.17674    116.15482     110.5619          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       53.50    318.90654    298.53878    290.75994    261.62843     242.7092    235.97523    233.12633    221.87864    217.18375    217.59231     220.0582     215.7594    208.73872    207.86839    196.07507     190.3829    192.11868    180.66187    182.90053    195.10676    187.90547    184.17913    186.93265    183.94937    193.44525    204.07441     205.1527    211.41368    206.85949    197.26362    178.55245    177.27747    168.92527    169.28494     166.2922    164.52714    167.30684    166.34199    165.66366    165.32556    165.14549    162.33126    162.37737    164.01599    158.55986    160.19517    156.67838    151.67852    150.99271    154.91013    151.71516    152.84084    155.19078    153.82329    155.24001    147.89945    160.14317    154.14481    156.18944    153.10557    151.59326    152.24675    146.07543    149.10868    144.77397    148.23592    146.77924     146.2812    149.89577    152.51164     167.1035    175.91498    185.27323    188.61787    177.05553    169.51841    177.07251    178.84004    196.53022    210.18698    221.54339    217.18756    206.17722    177.25985    163.66124    150.39093     139.6526    138.74923    138.90019    148.45255    158.62284    165.42853    177.03686    184.33345    190.33802    184.22075    166.68996    153.83773    141.78828    136.29548    130.11994    132.49869    128.17401    127.26081    125.00241    118.69544          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       54.00    319.50614    302.50343    290.05504    265.84382    247.83594    232.40161    233.42471    220.87963    216.41643    215.95988    214.91648    212.90723    204.86979    210.77389    193.62937    187.50578    192.19483    182.09784    192.41722     188.6591    187.01183    184.67424    184.29664    188.19554    197.96325     206.6213    205.74914     205.9827    207.41867    190.06096    182.72681    180.48859    164.24956    170.41017     165.0431    166.44071    165.04082    158.39875    164.07488    162.55804    164.23044    161.58891    163.62135    158.77275    159.80852    160.50781    159.12077    150.21285     154.4237    153.82343    153.18791    149.08136    144.66294    152.89769    151.09963    154.71431    155.72106    159.39329    156.82589    150.58733    150.96465    149.19987    146.29234     148.5255    147.76762    149.12755     149.2432    149.57766    152.37863     150.6284    162.69457    178.10531    188.77101    187.11002    177.10242    176.52649    168.08304    175.95763    184.90388    200.93189     210.3776    206.96986    195.44412    182.18621    163.97967    152.47547    145.98389    143.47009     141.2479    146.85518    153.43023    169.33117     177.0981    187.33271    197.86963    202.24261    195.37424    173.02503    152.23288    139.86297    134.79394    127.15137    128.25016     124.1785    115.41665          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       54.50    318.93531    303.43722    288.35234    267.23305    254.27591    234.29085    232.19462    222.07587    215.63712    212.83212    208.28839    207.94818    205.59084    207.05659    191.26407     184.9951    189.23241    186.28838    190.81565    186.01977    184.15995     186.0164    180.05668    185.86468    198.23083    203.57744    203.64669    203.49177    200.48107    191.34992    185.86259    171.96953    168.12812     176.2209    170.30587    170.39261     166.2107    162.93758    164.23344     161.8115    164.05305    162.17273    159.28211    164.75008    162.51713    157.57855    165.44055    154.38518     157.1813    152.05802    150.62455    152.60158    156.98141    149.48933    154.74295    151.27182    157.93617    163.82058    157.35113    156.35271    156.16993    147.66703    147.74076    149.42058    145.65903     145.1544     148.1464    148.52517    148.16663    153.05139    163.47757    176.96393    180.69471    186.69756    175.65556    170.04392    164.79726    173.00665    176.98382    187.76745    191.03863     195.8711     189.5587    176.98894      166.046    146.43666    145.23841    143.02749    140.69853     149.0654    160.24635    164.76547    175.83499    190.15252    198.04044    205.99215    201.07903    184.75984    168.28893    146.78179    134.88701    128.94491    129.18904    121.52871          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       55.00     316.5164    305.21032    287.06507    266.60731    260.23864    238.03135    229.21181    221.77204    214.53302    212.84066    209.67454    206.35599    204.42703    199.93901     197.2269    192.36448    190.27472    187.91949    181.88693     186.5854    189.18276    189.05321     179.9384    183.54618    195.56829    204.85651    211.21372    206.28311    199.63323    193.86347    178.57162    171.65971     172.1723    172.81002    163.56505    170.37134    175.48463    164.90239    162.36322    164.12146    165.85498    167.60884    160.69273     164.5752    165.90458    159.88965    162.75058    162.69681    154.19639    153.69027     157.2798     150.3696    153.27154    147.59667    155.58017    154.07701     154.5494     159.6773    160.36593    151.25884    154.55782    154.07629    152.88602    146.46971    148.08334    150.17386    151.36432    150.42553    153.63399    155.84152    169.58486    180.09027    179.66108    185.90334    178.88438    174.47531    168.51956    173.19803    179.84854    184.33497    182.53398    186.23142    181.58316    171.57076    155.71708    150.02964    147.32567    145.97037     145.1098    151.21686    153.97958    164.20481    177.22118    184.79083    195.67839    200.11022    192.00125    180.16704    168.43182    154.80405    135.84977    129.42693    122.05468    110.99775          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       55.50    313.88335    307.81494    288.48304    270.39905    262.73339    239.43598     229.9908    220.52633    216.29304      213.552    214.32521    205.12546    205.77103    198.04953     203.1126    200.04842    193.42836    189.58689    184.29181    187.76027    187.68684    194.52273    178.08815    186.64777    195.92571     201.3397    209.08156    204.21716    194.50439    187.39619    185.67061    175.27456    172.97909    172.55462    171.47243    170.40296    167.47629    165.39911    164.55576    169.56269    163.01143    156.75733    159.48946    163.97349    159.97474    159.87257    158.82498    158.58356    157.26258    155.93434    152.78827    153.59583    152.99374    153.01629    150.56698    155.74322    154.94018    157.11851    164.98956    155.87845    151.84944    152.54344    146.96476    143.11254    144.24907    143.60361    145.79456    144.73704    152.48305    154.67999    171.35979    176.12442    179.88543    182.96067    182.29137    169.98923    171.07125    170.44492    182.44435    189.52387    190.05385     181.5057    174.49205    168.95815    156.63491    152.06509    141.14507    140.61533    139.26713    147.54552    155.16823    163.17449    171.39484    182.37539    185.54643    186.83083    174.29734    171.46036    159.83945    147.89693     135.4472    124.97322    115.65012          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       56.00    315.10421      304.631    292.62289    275.87418    261.73155     240.2483    234.78454     225.1516    218.15828    213.17719    215.20671    201.56313    207.71463    199.69706    203.02881    196.79505    196.35431    189.45196    185.84928    185.52088    183.18766    185.74182    180.29821    187.18133     196.2491    195.82366    208.50372    212.55817    194.67846    190.43535    178.56029    178.00134    176.37681    171.66873    174.19517    169.43644    168.84076     165.5898    166.36665    158.73904    161.61985    156.42894     161.3781    165.39358    156.18193    156.63203    160.75377    161.65144     159.0291    152.82885    152.19271    151.93892    155.76778    156.76374    153.50932    149.69457    157.29056    159.56967    160.57045    159.08678    156.32094    151.36892     145.6841    152.35114    147.59938    143.95492    152.82304    142.99762    156.17732    159.86228    167.88358    178.52082    184.52127     183.7186    180.90465    171.84015    168.37656    177.65292    185.77826    196.87654    197.62062    191.74763    176.11608    163.75499    152.27774    146.61712    146.02007    143.20527    139.01406    150.70495    149.34469    164.93084    167.28115    174.85236    178.83392    171.75041    166.45916     151.6611    144.10929    137.74383     132.3988    115.32777          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       56.50    318.18413      299.272    294.36995    278.06747    261.43763    242.56889     240.2552    229.21281    215.99884    216.05013    212.88851    202.97736     204.2636    200.30571    200.29722    188.20632    198.06409     184.5422    184.07941    182.57741    183.38152    184.88427    182.81302    185.85577    193.55197    202.06316    211.78498    210.88962    199.77976    191.39858     178.2452    171.35138     171.5344    164.70407    174.22767    173.46661    170.83231    166.41539    169.92224    165.19388    164.15011    164.56221    160.49733    159.80929    160.82623    158.98019    159.68026    157.63299    154.91517     160.6576    157.28479    151.34401    154.35445    153.34143    161.84358     155.4173     156.9856    157.31174    161.32937    154.95707    154.98058    150.61579    144.63951     147.3119     143.8444    144.36326    147.91163    143.79049    150.77246    159.73845    175.03818    183.39945    185.27026    180.75735    177.25689     173.3676    172.77641    177.53349    194.12092    202.74091    203.92989    189.67876    179.31373    166.35729    151.89333    146.27304    138.05058    141.12389    141.60954    144.04817     155.0153    159.24331    166.01165    172.74488     168.6674    169.66823    154.67087    147.80952     136.0377    132.25971    120.93481          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       57.00    323.10346    293.35549    292.92677    277.04089    261.64222    249.39258    237.54798    226.67012    215.97828     219.8857    208.80744    205.44768     199.6225    201.87046    201.54953    194.07427    196.31652    189.06363    186.32793    180.25027    187.93172    186.50446    188.29833    184.14792    190.76476    201.86904    210.48832    207.79887    211.27178    190.21312    181.58017    172.10317    174.04612    171.29236    176.06468     170.7402     164.4773    164.39398    173.04417    162.31393    163.07706    162.47305    160.13679     158.2524    157.63571    151.01145    160.10579    159.23275    157.75615    159.61864    154.55888    145.17759    149.69274    152.83061    156.63267    155.44184     156.3121    153.37548    160.71823    164.33528    152.88637     154.2263    151.99814    148.72004    149.94532    140.72961    145.35938     149.9025    154.52394    159.00911     169.6509     179.1328     185.5738      183.685    179.37816    172.04465    176.01633    190.96656    193.26479     198.5377    195.29024    189.32127    178.89025    168.08387    154.45623    143.86959    137.40124    136.98074    141.72601    146.42413    151.77309    158.38292    170.51237    175.03036       169.01    163.98562    154.27075    141.99697     132.1011    121.31448    114.83289          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       57.50    326.94957    289.65142    292.91537    271.91156    262.29914    255.43849    233.27669    222.07776     221.1184    216.95559    207.97283    208.93054    202.44694    203.63707    205.63761    199.50776    196.94054    204.94048    190.99882    184.20777    189.42665    187.06426    188.28803    180.93202    190.62592    207.59483    203.60926    210.04668    202.93679    198.21437    178.80223     171.4153    171.18582     169.4342    174.21711    165.97251    165.12416    169.12375    169.95098    162.33536    164.70993    161.83568    164.52374    168.71653    158.17781    159.40885    158.61809    157.13728    156.52994    157.68533     158.4623    153.51007    154.03422    150.89087    152.86378     159.7978    158.27641     158.6147      161.752     157.2199    159.66477    154.79748    151.86191    153.01497    146.48306    142.38029    150.44079    147.55479    148.74702    158.20177    166.79975    179.71023    184.87599    183.78759    180.16081    170.45932    175.85593     181.6486     188.5181    193.01769     190.4004    185.88441    171.50144    161.01447     148.6778    146.63809    143.10101    139.80482    141.07499    137.57584     152.5869    157.53694    163.29163    172.01239    166.93344    163.31549    150.83484    134.54276     129.6207    117.14989          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       58.00    328.51899    289.71782    292.53119    272.72719    263.51786    257.57774     234.8606    220.20861    227.78894     213.0813    210.62776    212.04206    203.51551     205.2879    199.14881    201.50912    198.74736    203.25415    193.91074    184.48797     183.3087    186.43682     190.2179    180.27755    188.22715    195.58645    206.00144    209.34543     200.5697     197.8727    181.05638    171.31435    172.39562    168.47507    172.40361    168.02504    167.83268     165.6019      166.486    168.88219    167.64512    161.47522    159.70016    166.44843    160.28387    159.15129    162.84129    156.24816    156.09764    163.31246    158.17485    153.66495    155.51731    152.10264    151.47565    154.66575    157.61857    166.44054    158.74006    161.12209    158.54031     157.7413    156.10885    151.56269    152.80186    147.20881    146.37776    151.37465    152.14467     156.9386    168.15012    178.60701    185.69498    180.12729    181.08695    177.24257    172.86861    179.98082    184.83582    185.12469    179.26208    175.68866    174.45975    167.34804    157.93956    147.42702    144.33067    139.55353    140.17944    142.71411    153.15173    161.71698    167.07508    174.65128    165.31433    155.15104    146.99785    137.07423    122.47458          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       58.50     329.5428    293.63726    292.19071    275.45074    264.46841    256.02765    244.08855    224.92003    228.33875    215.92839    212.04475    208.45181    200.53262    204.61846    193.13568    203.09805    196.67979    199.65571    191.17041    187.67238    177.77404    183.08893    186.09151    181.92083    189.85332    201.10336    208.82477    203.47782    207.33625    197.47858    187.49524     174.1721    172.66702     169.2008    168.22417    170.97659    167.63928    168.24419     169.9943    165.77054    166.12661     162.5093    161.27167    161.21827    160.73711    166.28577    155.84963    160.01303    157.48851     161.7708    157.82351    162.46291    156.04968    156.98913    158.38548    155.75849    151.62173    163.40288    164.55754    162.99285    157.80681    155.80396    155.81079    142.59198    151.32071    148.33279     147.2664    149.72932    154.20339    157.62004    168.67574    178.51491     186.9277    190.28722    179.31018    175.01995    175.52361    175.93487    175.87018    178.48167    175.29219    174.11922    170.79614    164.75247    150.50087    140.21534    146.32138    138.89152    138.64323    140.84131    150.96929    157.85824    157.29666     164.8993    166.39421     152.0689    146.17093    130.85693          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       59.00    328.21411    296.70623    291.73178    279.61978     265.2412    254.19886    256.38166    225.93253     226.8111    221.53981    213.28331    204.62445    199.65621    197.32368    195.36592     195.9538    198.78731    200.40841    187.92358    191.93372    176.35098    180.89633    183.35495    182.58241    189.71258     203.5582    203.81345    206.84408    204.43562    197.32347    187.92265    183.34072     174.0734    170.49319    172.79942    171.86232    169.75087     166.8557    170.10022    159.16837    164.45444    162.78383    170.84023    158.79329    154.23945    162.79004    157.55913    157.84759    157.50885    155.41628    152.37385    159.23318    154.39797    154.83282    160.80512    154.22217    161.08786    160.17279    155.14772    160.18824    160.92838    158.15388    154.72906    154.76022    151.45873    145.93363    145.10602    148.61356    153.71431    155.82472    165.51969    175.77394    183.33994     186.5674    183.55094    170.71967    165.84292    170.14499    171.19985    168.00201    173.65587    171.59819    166.96204     155.6374    150.27491    143.80067    144.37118    134.24535    143.54268    143.61653    148.00706    155.95724    164.78031    167.12588    159.83568    152.36785    143.07695    124.92526          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       59.50    324.22821    299.65184    292.13585    277.66734    259.77679    255.01215    255.27898    228.49312    225.64254    220.62094    215.01717    205.04649    197.70766    195.04517    198.60646    193.70426    205.24442    200.58898    194.81301    191.93815    178.96221    184.45523    182.93387    184.88689    191.74086    205.31809    204.62086    206.17182    204.16804    201.76307    187.85124    181.58212    177.29407    173.70128    169.10015    169.24909    168.97097    162.07044    164.70003    164.58284    165.16507    169.36076    168.77719    169.55405    158.12942    159.85858    160.43657    157.92929    158.90435    156.29037    153.26901    156.93489    159.81624    156.04304    155.74485    160.95143     159.4622    159.97035    164.85452    164.02493    164.10173    161.54764    155.51799    152.19476    155.89848    146.65801    152.19068    156.97925    151.19086    155.50202    166.19205    172.67577    181.38184    186.74193    178.85687    174.87995    170.14219    174.05457    174.59235    163.31982     164.6074    168.27435    163.48093    152.15914    148.32593    143.05823    138.44583    139.91372    135.71123    141.44287    155.90375    157.83371     162.7086    163.35612     159.8792    144.12457    131.46844          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       60.00    321.89882      300.547    294.32659    275.10588     253.1585    255.81624    250.71667     235.8764    228.24616    218.19005    218.36213    208.75314    196.09955    199.62321    200.23052    195.34193    207.78687    194.22727    191.79635     189.0663    188.77856    183.36568    183.16279    189.42529    190.31778    204.85955    203.94888    206.29405    207.11975    200.26054    184.90843    181.89528    177.95121    171.85758    165.94973    168.63683    166.41857    167.61624    166.14235     162.7075    165.25453    166.74019    164.57359    168.23129    161.68477    157.58245     162.8647    159.13407    157.81255    155.37989    158.00086    155.70646    158.84484    154.56244    161.47499    165.29119    161.34628    161.96359    165.03313    160.76591    158.82134    148.77751    157.60826    150.24482    153.50771    151.13713    149.44814    153.77954    153.72964     157.0249    168.03136    179.03858    186.18073    183.26255    177.62668    172.27749    168.92331    171.77278    177.29771    168.79208    163.24569    160.45043    159.89125    152.19574    147.38468    142.16523    134.59663    136.53063    137.22826    143.91593    147.45577    159.79809    170.69013    162.22977    154.59122     144.7629          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       60.50    320.15583    299.51239    297.60722    275.33889    252.47521    255.41358    247.15223    242.04518    228.48432    218.25972    220.42054    210.36368    196.57108    199.90115    197.32799    195.82348    202.41785    192.04188    187.99377    187.13813    197.23569    181.60649    183.03525    190.46075    188.43557    206.12142    200.44184    209.50749    210.04437    202.64641    184.08341    175.22509    179.41053    170.97935    164.23608    164.91017    168.90014    166.25162     162.5992    164.00663    165.43702    163.68335    165.45783    158.66635    165.79946    162.63817    150.48814    163.81981    157.98021    157.73667    159.57965    157.30754    158.82643    160.31135    156.12244    156.97659    164.24411    166.94597    166.49192    160.24458    153.25694     154.3621    156.06678    154.59977    151.34214    152.65891     147.2494    155.13379    155.51901    163.01368    166.33921    175.55814     180.5438    186.03948    180.19393    174.02623    167.52967     168.9494    175.37537    176.83118    167.62627     164.7296    155.29871    153.45752    144.41502    134.32679    142.25119    138.74708    139.58295    145.37096    148.21139    154.27221    159.70019    166.73241    157.80735    133.57258          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       61.00    317.74425     300.4594    293.03063    278.02701    260.94913    252.28304    245.84387    243.16292    228.72489    217.60896    214.55309    207.06233    198.85682    198.21065    198.70019    191.68586     193.8889     191.5621    186.51202    185.79183    195.89698    181.76343    186.53394    188.68522    191.86607    200.78382    196.39062    207.85502    202.69046    192.86988    186.74904    183.00765    177.44738    172.38641    170.85896    167.11427    167.29642    171.36201    168.65686    175.79373    167.17754    166.01037    159.49773    160.77463    164.36315    163.36954    163.84418    158.98791    158.61255    160.47929    161.33194    156.29601    159.28183    158.04024    156.18947    158.47645    160.25373    160.42643    166.94861    159.13625    160.32975    154.83584    153.53604    153.45736     149.1747    153.13279    153.04864    153.59201    152.65302    162.43403    174.51256    178.10917    187.48501    178.49662    171.87166    166.58212    166.31692    162.80461    165.60694     172.2618    171.30802    167.65146    158.23859    156.26105    145.58302    140.83232    140.21452    142.81248    142.36708    144.20702    148.67408    155.67195    161.55301    163.07462    150.67979          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       61.50    316.10154    301.53094    286.79013    279.91422    266.12979    251.47584    246.23185    242.48797    232.67496    216.37618    210.02359    204.75546    201.67676    198.00171    198.89024     190.2582    192.58937    190.84805     190.4346    187.59829    197.26044     187.4094    190.80218    186.18259    193.18068    199.47875    197.75341    202.72036    202.59948    197.78595    183.58158    183.45168    183.88561    175.08686    174.93103    174.16747    165.36608    165.08134    169.66292    166.78224    163.56096    161.89332     161.7733    161.15739    160.41842    168.93543    159.90706    165.87106     160.5433     159.6289      158.804    158.36547    155.61954    157.47434    158.29029    159.87667     155.9128    161.74437    169.39747    167.29823    162.38859    155.33708    150.36523    151.45209    153.31265    150.41443    151.30018    154.33508    154.45695    159.94395    167.23145    179.96004    189.40483    187.64572    179.10128    172.51521    164.34114    164.21988    163.86515    163.13476    171.27686    170.31244    159.91143    151.53132    148.66096    149.53303    141.22178    139.61897    140.61326    142.50767    144.26768    154.10206    159.13206    158.49499     129.8033          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       62.00    315.21548    302.13994    283.40258    279.89607     266.3914    253.02216    249.16788    241.33572    235.80148    220.36078     210.0256    202.79776    199.21744    200.11361     192.3625    196.36779    197.26557    190.78937    195.79921     188.8003    192.93122    193.13343     200.2552    183.85974    194.01119    195.75366    201.65291    198.76512    202.00868    202.01929    181.96949    180.03951    182.97189    177.98103    179.53296    176.57549    172.24452    162.83795    170.23341     163.0824    169.12127    164.62183    163.33523    162.05472    161.45932    161.95185    157.67582    160.30206    161.88455    160.70582    158.41077    161.52457    153.01589     151.0794    154.80509    156.87125    158.27103    164.44813    167.12208    159.76952    166.99295    153.21897    149.18698    149.54663    148.95508    152.21441     152.6906    151.33063    149.58109    164.34564    168.31872    179.31925    186.43713     181.8342    186.25784    169.18139    165.71668    153.60571    154.72912    162.40558    167.57178    160.27157    159.44866    156.55487    154.49366    144.71739    139.34618    138.12299    135.26209    142.10545    141.80906    152.92369    156.91582    146.98697          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       62.50    315.71153    302.63227    287.88577    276.66833    264.18984    254.88174    251.38852    240.15399    236.37826    227.18858    204.63348    206.98198    198.37468    203.01153    191.29642    195.88373     196.4408    195.50355     195.9227    188.85916    189.08974    191.00571    189.23009    190.39321    189.16998    197.38399    202.93318    197.40418    195.96394    201.90793    185.70543    181.62822    179.81044    176.46859    174.24303     172.5398    168.70412      168.456    164.44117    163.16925    168.49119    164.02624     164.8823      165.941    164.99047    160.61624    165.16404    167.74731    166.50528    158.85859     161.5125    158.83723    159.24775     156.3524     155.9761    157.97943    161.38276    165.32127    160.40073    161.71256    164.54494    156.07268    156.68329    153.70995    150.41442    150.42142    154.09324    149.07078     154.2111    158.65039     170.0108    185.54872    187.70774    188.86072    185.07162    182.13862    168.82386    159.40366     157.1999    158.07307    155.90926    160.42046    151.72204    153.63823    145.02124    154.00999    145.38999    141.97575    141.54857    143.64978    143.70264    151.12036    148.39756    130.71117          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       63.00    316.22657    304.14728    291.98521    275.13866    262.96176    256.04306    251.45408    237.61201    233.68601     229.0299    214.05467    207.27801    204.65279    200.18825    197.42235    196.88922    193.77452    200.52917    193.52275    188.61468    192.72031    188.28531    188.83084     188.9605    196.15648    197.65694    201.58353    207.68878    196.31634    189.75594    190.38663    178.81383    183.80051     180.0583    171.43473    169.14647    168.81573    170.91562    170.10519    163.32133    164.24203     163.4934    168.39304    157.50245    158.43154    165.84294     163.0346    159.88716    161.63045    160.97903    164.60845    158.33351    158.62509    158.75174    158.24337    158.05344    157.27738    165.09051    159.97141    163.17881    164.43218    156.83396      158.067    155.93562    153.09128    151.55683    151.61643    152.78611     158.5755     165.1929     168.3778    177.74838    187.29933    190.76222    184.43243     180.6633    164.37364    165.45219    156.47522    158.70884    155.94177    150.95485    151.96053    153.95575    143.75508    148.22809    144.29141    138.99804     135.9966    144.73104    145.50929     149.5373    137.93857          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       63.50    318.08202    306.68927    295.42864    273.46335    261.71095    257.22294    249.10263    234.42591    228.39262    226.22552    228.40145    203.35017    210.90945    199.08947     198.2601    199.93076    190.40912    193.10081    195.44887    190.34812    190.63323    187.87301    185.67484    190.84977    197.37411    198.91685     204.3877    209.11844    196.63219    196.76325    179.44319    178.28005    168.62346    177.78912    176.11999    172.67797     173.2201    174.60057    174.24522    174.55027    166.33428    159.02247    170.47532    157.94212    165.43606    172.12463      167.871    172.20483    158.56476    162.83324    163.20473    161.29429    155.31376    158.11885    153.11157    155.50898    159.75393    161.58593    164.06776    159.22956    164.27238    155.96988    155.13015    156.52007    155.18324    160.90913    153.58598    151.71624    150.66319     158.0555    169.35163    179.99587    183.58428    190.36989    182.19758    178.17225    172.27764    165.70486    161.05949     156.2855    151.35987    152.69868    148.24417    149.73806    148.77332    146.46912    143.91016    144.24799    143.96744    140.76622    145.39156    142.31827    128.47473          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       64.00     319.1771    304.87617    297.39299     269.7599    260.76843    256.30897    246.08098    231.15904    224.98299    225.49534    228.73478    209.73039    207.60693    203.88416    197.50444    202.97882    186.70228    189.31601    193.93975    187.66201     189.8231    187.42011    181.30666    195.40234    199.12983    206.62756    206.06138    211.86314    195.10404     197.6327    183.68713    181.31325    178.73765    176.84133     175.2168    176.35333    167.92101    172.50461    171.65835    180.11259     170.8417    165.98207    168.16112    161.58536    160.40659    167.07781    168.35377    164.99852    165.86429    158.71205    162.19309    158.95107    162.13754    156.50021     152.0693    158.01688    159.13656    167.88411    161.60112    159.53702     158.8587    158.80675    155.78324    160.12905    162.06178    156.08576    152.65991    157.68222    154.44081    159.29765    169.09496    181.27357    188.11961    180.94571    184.96059     179.7656    170.28369    158.21541    156.59034    156.38089    155.43498    147.79438    153.58693    148.80109    145.98003    142.26336    138.04993    131.75763    137.66078    140.72529    140.04342    131.14377          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       64.50     317.9059      302.618    296.08453    265.56999    261.11666     254.7324    245.48741    239.46817    231.34873    228.10656    226.42909    218.08836    204.47567    205.91497    198.12324    201.41604    191.08091    193.91152      188.955    178.64078    186.16875    189.51546    184.74066    193.17067    198.21868    202.88981    202.80211    208.69958    196.16172    194.50879    188.43393    180.86719    174.73101    173.54279    171.15916    175.67769    174.83252    172.69566    175.27965    180.07287    165.91676     172.9009    166.89378    164.75149    161.72808    169.59474    160.60666    163.60272    166.06512    168.60274    158.41157    156.24863    159.80948    161.40894    167.39696    156.62879     158.4966    170.25129    162.25968    164.06649    160.51637    156.33793    157.69169    157.91685    155.80381    154.52785    153.19128    156.31428    160.03322    162.24613    168.64243     172.1898    186.68859      183.436     185.7662    175.28366    170.93767    163.01204    156.96367    155.88778    155.38404    163.16863    157.40444    145.63749    146.14146    145.91152    146.80758    137.12395    138.98851    137.76077    133.33767    123.46589          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       65.00    315.26218    300.63429    293.79359    268.97979    266.30191    254.47105    244.54062    244.44028    239.77566    230.56008    225.98537    222.04341    210.32646     203.0241    196.12706    201.43827     193.9817    196.03475    192.93413     183.6273    190.60077    190.24904    190.37008    191.81029    198.76342    197.52581    212.26675    202.96813    199.49584    192.39317    187.14016    178.81516    175.97605    173.82904    175.03755    173.35955    167.61283    167.11467    179.11925    172.22531    171.38964     171.6938    169.59707    163.11526    162.89132    168.89592    165.48722    166.88164    163.82045     160.6508    165.60052    164.35222    162.25745     161.9572    163.43271    163.31589    161.58656    165.05066    167.92306    164.25937    159.00177    160.90679    159.88428    155.72778    155.47008     158.5235     155.8426    159.83869    156.01149     166.6458    172.27829    178.30547    182.03027    186.07407    185.13551    172.64668    169.54833    166.67099    161.10922    152.98826    153.39861    162.23927    152.92747    152.10998    150.76559    140.04186    140.26127    138.37202    137.06684    141.52595    134.53853          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       65.50    313.51309    301.46446    292.38749    273.52565    269.93553    253.85843     242.5233    242.18088     242.3895    232.92723    225.20058    220.35376    213.89108    203.58659    198.65812    190.40649    201.03238    191.36123    194.29663    189.32038    191.03613    188.07544    194.65189    194.84725    199.96355    199.35493    211.35251    203.13296    206.96286    191.20172    186.63644    178.84751    178.35512    174.75068    179.58249    174.34093    171.25718    173.43007    178.28647    170.92432    174.51323    175.40802    174.00226    168.02191    158.98647    160.85172    164.68414    164.05235    163.92298    160.37875    166.16192    162.67019    163.80854    162.75562    166.95108     164.4671    163.31651    164.77012    166.91145    167.24883    164.89182    153.91589     158.7153    161.65837    155.76852     158.2429    158.10548    158.16874    156.54049     165.8604     167.9742    184.39444    187.15968    180.28796    183.40843    181.94327    167.66164    163.89299    155.86167    153.33591    155.74893    154.93899    151.66988    153.63439    146.17088    147.82044    141.79613    134.70747    139.15792    133.62627    122.23452          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       66.00     316.3754    302.29103    292.64456     274.9831    270.26229    255.89471     240.6254    239.02227    243.02566    233.79176    222.42044    214.20277    211.37227    206.15019    199.57771    188.49567    203.28038    194.32725    186.15771      188.937    191.48776     185.2794    192.14769    188.78772    199.47467    205.57917    206.80757    205.26314    209.23805    189.95883    186.80881    173.40434    172.80108    176.89949    181.03252     177.0562    171.43363    167.24862    173.35029    175.44991    172.03874    176.57361    174.77064    174.26596    172.19527    168.94498    165.99424    161.85545     156.3247    160.92775    162.93704    162.84214    158.65829    155.17532    164.11419    163.23361     164.9396     171.9437    173.88854    159.49108    160.86518     158.8854    158.76126    162.81294    159.91863    158.17022    154.73983    160.49665    161.53728    165.18404    174.20385    177.32128    187.87914    194.42785    179.76009    176.61229    164.33058    160.73691    158.60292    155.55431      155.988    157.75696    156.71559    151.24262    149.95618    151.01999    139.10718     145.6577     137.3475    131.25307          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       66.50    320.20867    302.62134    293.88936    276.22442    270.54791     259.1345    240.94574     238.8504    236.09202    233.30754    219.23166    211.33783    209.03569     210.2844    203.45775    196.23326    201.35449    196.81629    185.81401    189.59879    197.49662    182.68176    193.49078    188.43276    199.76682    198.41347    202.20556    213.99352    206.59471    194.17743    190.09228    177.89023    168.97425    173.20294    172.29137    173.46757    173.89284    168.94362    174.82498    173.54158    171.73856    169.79217    172.60569    170.61569     172.2488    171.07204     173.6129    162.30051    159.97743    164.21777    161.96471    160.17262    157.77476    162.87931    163.41771    168.48247    166.60678    169.47987    172.18423    176.28252    166.65981    160.98912    161.02702    159.75157     158.4091     156.8639    156.88594    155.24432    158.14117    169.27945     171.7963    184.17146    193.20049    191.85565    188.51929    182.50573    173.05477    161.52758    154.78885    154.45441    154.56253    156.72745      150.194    153.85542    150.52967    148.05959    144.74468    141.80819    132.86314    121.98449          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       67.00    321.38138    303.56116    295.79771    278.71884    274.20269    261.93116    243.81667    240.36958    226.41484    237.76042      215.241    215.11401    211.21007      212.175    207.97734     203.2475    196.59414     194.7677    185.20554    191.06932    194.14323    188.11491    191.82483     196.5049    193.99782    197.75888    208.01867     209.7646    211.02637     198.2699     189.9252    177.23242    174.51463    173.65015      177.905    171.51035     177.0837     174.6837     173.7282    174.87361    169.19216    167.12637    168.94284    171.71334    168.96428    175.27013    172.66645    168.07884    159.78745    167.49298    165.89854    157.40402    162.43984    157.98248    169.84766    163.39533    167.44345     169.0073    172.20083    174.73439    171.70123    170.78538    157.69437    157.32767    156.91983    153.32831    152.26801    158.59233    152.24209    164.82467    170.47716    182.60778    192.71738    186.57911    191.61739    189.50352     179.2671    169.24432    159.48154    157.62713    149.92217    157.34411    153.05373    151.02825    147.36393    143.61973    144.84913    134.91894    131.63031          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       67.50    317.75502    304.91033    297.58712    280.89213    274.31311    263.19034    246.40994    238.95493    229.13471    229.94819    212.29438    217.62972    214.01892    210.57679    205.71719     209.7216    192.60038    191.78211    187.24957    184.76872    190.12595    192.45862    187.71136    197.28174    200.43293    199.56225    212.50722    206.78231    207.27539    196.36355    191.03883    175.26848    180.84678    179.43663    177.93461     169.9378    173.07991     175.1365    172.80336    168.39743    168.32392    171.18854    169.73982    171.57775    171.60568     165.9406    167.36737    169.91664     162.1537    160.21025    160.01253     162.7562     163.3762     158.3425    160.85128    163.83248    165.85886    169.49522    172.79036    169.85828    175.25617     163.6439    169.75157     162.3937    144.87101      160.899    151.85745    158.55707    154.43064    160.76908    171.41348    181.20555    187.16119    188.66777     187.7484    183.48356     176.8708     167.4907    160.90134    154.22408    155.46434    151.53214    153.48169    157.12791     148.7962    151.51444    143.62854    140.20704    131.25887          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       68.00    314.31362    306.05502    293.87315     278.7439    270.46666    263.63737    247.11017    240.12926     233.2577    223.38629    213.24664    218.44766    214.25739    215.15648     204.2747    208.66862    198.94383    189.90211     190.1518    183.60188    189.36319    190.07593    186.41764    192.97154    196.26031    203.82015    205.12542    209.77371    202.39298    195.96401    198.31231    182.75038     182.0536    186.20598    175.02385    175.99159    172.09241    172.15722    176.77911    168.82636    169.88134     175.3371    174.08652    169.37576    169.86535     168.4114    171.39829    168.63954    178.34988    166.59349    169.54095     155.3381    157.89733    165.16172     162.9739    163.31605     166.2389    169.84666      171.147    170.83676    168.08335    164.27993    164.38798    171.01702    162.43966    156.16819    153.82042    156.24659    159.05792    156.56354    175.28895    184.54449    189.04262    189.90815    192.45674    185.84104    179.33566    167.75341    161.16256    152.94478    156.26854    152.48859    147.44791    152.22783    142.26474    144.65839    141.95995    139.73193          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       68.50    312.49346    304.32284    289.27529     277.2971    265.33644    262.80547    247.30552    241.52941    233.73991    226.86794     217.2761    216.67999    213.49129    213.36291    206.80402    205.59706    203.15162    190.88199    190.71143     197.9603    188.96423    185.02945    183.29619    189.23542       193.47    199.71148    202.16751    204.07223    203.06977    194.30967    190.08058    184.33339    183.18636    181.83837    186.81806    167.07385    167.84978    174.22623    175.44958    174.09044    172.08129    172.87463    171.88282    174.62781    173.20612    167.64716    165.73643    168.43376    165.18644    166.54147    171.90002    159.51196    160.54332    160.91961    162.61152    165.92892    170.36572    173.32397    172.91094    169.39484    165.87909    163.67236    163.93865    158.57402    156.56262    152.68042    163.15125    157.35701    160.10367     165.8187    170.84927     185.0074     189.2386    191.39777    190.28497    179.23822    170.76784    167.61934    164.21944    158.46172    157.73797    155.76513    157.44546    150.86857    143.48888    143.08938    141.39827    135.38236          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       69.00    320.54029    302.21978    286.23374    276.71576    263.52955    259.04085    248.84092    236.78589    237.68865     231.1657    220.59735    215.55957    212.83963    211.95344    205.42904    208.27352      200.052    192.01446    194.71194    188.27594    191.22565    183.95009    187.82212     199.9344    200.02156    197.39708     198.7974    202.06382    202.12097     196.4155    193.47105    182.63551    182.18733    179.44418    187.36035     178.8188    168.83407    171.49884    167.40051    174.68028    170.08643    173.20956    167.79578    167.49332    170.94361    172.04033    168.78772    170.29375    165.39485    166.53523    172.26659    167.11544    164.05675    165.90105    163.00417    160.46645    165.05617    172.44917    172.30414    175.83243    167.58825    168.28431    164.68497      158.484    160.91511    158.96225    163.12482     160.3484    165.67855    161.07992    169.70725    182.11865    188.24134    192.68478    185.76213    186.68341     174.7247    164.25733    156.30047     154.3906    152.88538    152.11334    156.67513    150.19051    156.03443    139.75558      138.454    125.07554          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       69.50    330.30534    300.02533    286.28488    275.38703    262.05713    256.11205    249.89823     229.8713    242.97468    233.14185     223.1749    215.17406    212.09494    215.92371    204.63687    208.56871    203.69965    196.04828    203.53167    185.00358    191.79964    190.51088    195.83946    196.34308    198.70027    201.19921    196.88397    200.48117    201.00324    197.55449    194.74637    183.22533    176.30036    177.62142    185.52613    182.72464    183.00186    173.62566    171.71478    176.82022    171.37467    180.44412    172.25321    176.25523    169.88347    168.53424    165.20079     171.3619    172.27953    171.21237    167.63832    164.85013    162.19937    169.62806    172.41664    167.96778    163.22965    170.51083    173.80042    174.83126    167.84343    163.32541    167.46944    162.04559     158.1508    162.42712    158.58636    166.12004    160.81052    161.70726    168.78586    178.71398    183.08877    196.89085     196.4857    189.14246    174.09528    163.93767    159.99414    158.06753    157.86384    154.41406    156.06412    152.35462    146.94494    148.55808    137.85063          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       70.00    340.58877    306.15701    287.21684    273.02406    261.10928    256.73177     249.9566    231.95641    237.38341    235.37725    225.26005     214.1729    210.99646    216.26624    209.37476    205.88991    212.09279    201.20007    199.93645    193.89508    192.49718    187.22868    191.32726    189.62026    198.38611    201.13868    200.08484    198.21611    201.03721    190.92121    189.59751    184.53891    184.07433    177.75969    182.46615    185.16286     182.9933      175.677    171.76432    176.90393    168.02694    176.19381    175.54629    168.98753    166.84513    166.51229     162.8263    168.47549    169.42277    169.73693    164.18991    173.88649    164.46201    169.55487     176.0426    170.84404    171.74617    166.43488    170.53384    170.66006    171.42297     168.5018    165.34173    166.81994    159.34223    160.87029    158.18968     162.5939     163.2611     166.2484    171.92066    187.03937    189.35043    194.80135    190.17954    186.87444    178.11969     166.5016    154.41192    156.19223    153.35777    156.80136    150.35576    151.37165    146.16743    137.01087    131.47905          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       70.50    336.12188    313.18278    288.67139     277.1398    258.49268    258.98087    246.21191    238.95161    234.12328    234.47923    226.45822    213.04876    210.04086    215.25498    216.74854    206.67612    208.13981     202.3489    198.46471    200.35872    195.12318    184.78353    191.39381    189.75552     196.6228    194.12033    203.67215    197.72349    202.18537     190.6403    189.37118    186.25325    186.19704     172.8566    181.36907    177.72361    177.35253    181.82137     170.8087     182.8478     179.2214    173.26335    170.56206    177.94402    167.30495    171.46172    177.31027    169.06907     169.8209    172.76632    171.70232     166.5555    164.67659    173.46132    168.68112    171.26548    169.81252    170.98151     174.9252    173.23999    171.99132    166.67218    164.75826     163.8328    161.40778    158.63954    159.62336    143.38357    157.67961    168.12983     174.7602    185.90881    188.58401    191.16814    190.04667    183.22252    172.32186    168.58469    161.70911     151.0341    158.99032    156.17068    149.25827    149.03414    146.02247    137.77334    129.30552          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       71.00    333.39103    319.47382    293.66455    282.62044    254.99352    260.56549    242.75821    242.10809    233.61991    233.21386      226.528    218.99793     207.4515    214.78505     218.2256    211.13001    202.82173    200.23311    203.79773    204.14052    196.36599    199.10327    192.90291    190.84125    191.96951     192.8558    202.08179    202.37806    201.86045    193.34297    186.84973    185.82264    186.20725    174.41043    185.28627    170.20412    171.37542    180.30206    176.57006    177.55273    182.00775     175.0753    168.36295     167.4003    168.35033    169.65923    169.81713    173.10861    173.70889    164.59212    171.38684    171.54988    177.77658     167.5581    167.26761    175.95919    169.81776    172.18802    184.58543    179.38887    171.11032    166.10976     166.5787    159.11316    158.27054     158.7131    159.70277    157.61349    163.27399    166.23998    174.67469    181.68228     192.8985    196.15751     194.8419     189.6836    182.10802    162.91259    160.91057    157.83568    163.21627    148.54671    157.44804    139.13371    142.83452    137.22553          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       71.50    332.96876    318.68759    299.01788    286.82537    256.59483    262.89888    244.61742    239.29958    228.45786    231.80783    224.67224    223.95574     213.2098    214.48368    218.54277    209.91789    194.48035     202.3393    203.40085    205.42876    200.20199    198.94148     200.1566    192.06149    190.60642    197.21173    197.53493    202.97526    201.72038    191.29673     191.0154    185.66376    182.30866    179.68265    176.96908    174.98747    176.27223    174.02373    180.23269     173.9749    185.36074     178.2582    176.15784     166.7399    168.51047    166.88912    165.67184    166.54901    169.01013    167.40487    167.20238    170.27578    171.08708    169.90813    167.93477    169.83991    170.24918    163.61898    177.59169    179.57161    183.95598     179.4661    169.04716    162.27104    170.98798    158.67243    159.20666     157.8078    160.77167    171.70662    176.24299    181.52855    186.90851    197.89726    195.09329    189.49156    179.42471    173.43078    167.53593    158.21727     152.9923    156.49316    153.27779    148.15443    141.50556    129.76613          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       72.00    336.22818    318.34886    302.18754    287.22152     260.1987    264.27622     251.6368    234.99586    228.06596    234.36351    223.32892    221.29379    220.53673    211.87141    210.67518    205.81501    197.23877    199.96072    203.92966    199.47427    199.63072    203.06606    197.32407    197.43976    190.97872    193.00271     194.5024    197.55959     199.9305    186.25255    194.24192    184.59925    181.94854    181.61147    181.23349     182.0114    184.33515    177.80156    179.49183    177.10431     178.8186    176.06684    173.82989    179.99009    172.54723    165.38505    165.26775    169.32681    166.55863      168.944     170.8171    175.17166    165.31465    166.43929    173.53151    169.53749    171.60173    176.07184    176.35833    181.60433    180.38517    179.21055     174.4777    172.07281    161.38661    157.56957    156.39193    160.19292    159.97582    163.81855    176.12929    182.34864    193.22115    195.52961    191.94444    191.27361    182.72042    164.52713     162.2784    158.62646    156.75501     158.8833    151.39377    148.64694    141.36558    128.37521          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       72.50     338.9217    318.50109     300.2263    288.79496    266.14139    264.23439    256.20514    236.21266    233.58427    238.01252     219.7404    218.42686    223.22675    208.61496    206.14071    205.90467    207.92451    200.48693    207.47416    196.08487    196.73344    204.76725    197.02404    198.68081    199.47041     199.4323    197.67632    192.36519    193.54178    186.56808     190.2392    187.43143    187.07195    183.99071     182.5942    175.85231    187.42222    177.44394    176.62456    176.83841    178.08474    174.14899    178.10387    174.49501    180.26171    176.49097    173.70522    161.80427    166.18123    165.27955    169.33783    173.52948    163.60131    159.18326    170.55216    169.04153    162.47945    168.23996    177.51136    177.62017    176.54891    174.18599    168.06318     165.2203     164.4242    167.17175    163.09355    150.79608     162.0936    156.73638    178.28651    179.36028    191.48785    197.54187    194.49148    195.12878    185.23843    174.01771    172.15337     159.4925     163.2929    155.05477    152.75054     147.4334    145.04602          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       73.00    340.82384    319.49555    298.45887    293.23927    273.55737    263.92655    254.58378    240.37787    238.65416    236.15743    217.36012    213.63797    222.22417    203.98022    200.82683    207.52135    202.49028     202.3161    205.40858    201.32229    193.00738    199.58809     198.4381    196.14958     205.6024    207.28161     203.7399    199.36283    194.33979    194.87195    188.56596    186.04166    190.05382    185.06067    181.87967    180.25871    182.91256    166.68915    179.46589    178.31169    172.74849    175.28973    179.69536    176.38186    166.78073    173.43856    178.69488    167.39154    165.01152    165.05731    170.53044    166.67861    163.52275    173.30118    173.97058    172.43393    174.14919    175.43493    174.62378     177.4949    179.07737    176.28509    172.60094    172.43399    165.95605    162.01893    169.15563    167.87322    164.67473    168.38315      175.387    179.77785    197.78284    190.81266    193.07195    187.85338    180.96234    173.09297    162.86616    164.57637    162.33839    151.96801    154.86631    147.35028    135.61046          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       73.50    336.01946    321.23635    298.65762    298.00297    279.98162    264.61379    253.10195    240.54856    240.26111    231.43408    220.27083    210.23367    213.11115    198.29044     200.6391    207.16018    197.05602    203.13721    200.80112    199.14692    196.17038    196.63242    189.39891    195.98541    200.32847    203.31525    206.49777     205.9677    196.95446    193.25355    192.44218    186.74318    188.65889    181.43715     180.9994    177.31424    178.71427     178.4172    179.40065    178.76769    179.52988    174.29101    181.66891    178.82477    176.59877    172.95227    180.17139     178.7989     170.0222    174.97069    166.73629    162.63607    168.46595    171.19086    168.49331    167.96896    172.53341    175.12714    174.95449     180.3423    176.14531    176.16023    175.60442    169.87293     160.7789    172.84214    169.38172    165.56607    168.01195    170.50884    183.80905    185.40378    186.88461    196.47866    197.09527    188.83511    190.08913    174.04881    166.03488    163.59982    160.59739     155.4943    149.20473    148.01068    135.26063          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       74.00     329.4882    321.66903    299.26586    301.52694    282.35621    266.70613    256.69321    239.15794    239.07957    226.70394    222.62536    218.88821    206.45765    207.63846    206.58601    205.57267    198.28615    205.59552    199.15876    197.29691    199.67531    197.58364    191.36548    200.60398    196.19389    202.25543     204.1521    202.00111    204.09245    199.82145    184.21535    183.42703    184.63405    177.51652    181.72038    182.15462    181.46853    172.19221    177.71379    183.81983    181.40299    174.72093    173.16483    173.11132    170.39434    178.75612    177.28103    175.75918    175.51182     164.7708    169.50682    170.35807     167.8219    165.92166    165.22975    164.27477    167.05407     181.7937    180.25252    169.94231    179.08884    171.34582    171.31552    168.99056     167.5518    170.64651    161.12889    167.30604     167.0341    167.28461    182.32027    192.11957    194.13011    199.16272    195.45509    189.01426    182.55012    176.12895    164.14938    166.66648    156.32653    155.75404    155.64737    144.73853          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       74.50    321.88209     317.9699    298.61844    299.25316    286.03763    271.48682    263.90573    241.02055    238.03885    224.42747    220.39844    223.42106    209.18706    218.34585    205.90515    207.25486    201.29945    206.73598    199.10652    201.70612    200.10098    200.91031    198.35191     196.7533    199.89971    201.49049    203.63347    204.15406    201.40255    204.67969    185.20809     185.9492    180.74667    178.32917      176.536    183.87792    181.46265    177.95692    173.99899    177.77269     179.0964    174.52922    172.83702     177.9319    169.69564    174.31382    174.04225    171.57014    177.83384     173.5749    172.48226    173.91812    168.41075    171.73625    163.92861    173.37702    165.46404    174.63188     176.3236    175.64495    177.20926    176.39825    172.23043    173.69817    165.87919    164.80556    163.95813    169.32381    167.28036    170.40244    178.78414    183.98588    193.87527    198.78796     205.9718    196.79807     185.1937    165.14927    167.21396    159.22569    154.28031    155.74215    154.00381    146.86274          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       75.00    323.23631    312.99414    293.97589    296.57009    287.55239    276.80294    267.87619    245.49284    239.85486    227.47905     219.0596    215.52679    211.76359    214.63744    205.18641    209.62441     205.8598    203.32687    199.64269     203.6711    203.18375    198.31208    202.05535    196.26061    191.93341    203.45308    202.76035    204.58141    199.03085    200.41069    191.25997    195.95801    186.87076     174.8013    174.51399    177.69939    178.19327    174.98033    176.69365    172.68743    178.42793     176.8816     176.9259     171.6671    174.30887    174.90642    171.03554    173.88778    171.34837    178.25442    176.80307    166.58266    168.93008    169.63707    172.00663    177.12961    172.51534    179.71759    176.75863    177.12237    176.67341     171.8038    171.43627    171.78882    163.80698    164.95087    163.61335    169.41008    176.62689    174.99826    183.23376    181.97399    194.12782    196.94074    209.19249    201.34137    188.94519    178.97448    172.55874    163.94583     165.0495    154.88034    154.80436    140.85328          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       75.50     324.9223    310.47867    288.78335    294.63717    286.19641    277.80019    265.51313    248.17466    244.29962    232.04458    218.90744    211.76348    209.65177    211.35809    210.30688    211.27174    205.32073     198.3465    202.62922    202.60786     199.8307    185.04622    199.27793    204.10637    184.51223    201.89784    202.88319    191.63139    198.88316    200.20208    196.70187    196.40659    189.38899    184.19238    181.15763    177.71361    179.15744    176.77604    173.03264     181.7433    177.81015    170.66152    170.84295    177.43086    177.92199    179.92928     182.3117    174.13635      168.952    170.94232    178.25492    174.43174    173.87045    173.76859    180.42823    169.45119    176.49612    182.37719    177.03742    173.68471    169.91004    171.42576    170.93307    172.61094    162.21491    166.85329    167.97956    166.84232    168.20844     168.0648    174.26992    185.12839    195.38911    203.24841    202.38103      198.705    195.63678    183.80482    171.02629     171.0808    159.35129    155.12548    151.46026    135.71621          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       76.00    325.54869    310.33117    287.22503    294.16873    282.77362    271.99991    262.34504    248.93764    248.38763    235.80358    220.35577    217.53864    207.51167     207.0451    212.10273    208.93624    201.91959     197.5218    200.48316    197.04608    196.15849    192.25241     196.8113    198.23186    183.68717    196.28745    200.21798    204.64077    201.18127    202.75212    202.41596    187.47693    184.99786    194.71904     186.7776    190.05928     185.6197    185.95033    178.50132    173.55156     179.5471    168.17365    170.31982     173.1807    170.25215    181.13667    172.41172    168.35202    174.02537    169.31987    180.66857    177.91335    174.22235    174.12623    167.70068    172.55289    172.83258    178.85868    188.28583    183.46121    181.70046    182.40755    170.00835    164.93117    165.13349    162.03518    171.41672      162.412    171.20851    171.77432     179.6451    187.32974    194.22797    196.70913    197.92483    198.02466    187.60976    185.28656    172.95024    163.09577    166.59658     155.7708     154.5036          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       76.50    324.66676    310.59974    293.48721     292.7635    278.44239     267.1387    262.77797    250.79686    250.21877    237.20831    232.81709    222.22762    214.23856    208.75867    208.49475    203.71298    200.49542    200.18625    202.85151    195.33577     196.0955    194.35657    200.40799    192.25908     194.3672    197.80698    199.48049    202.78982    202.67983    203.49844    198.37439     187.6193    181.66779    195.34209    189.75706    191.12913    185.04568    188.53385    181.01698    175.80148    178.15383    179.99163    174.27049    178.61319    172.59501    175.74888    177.28513    175.13702    170.38859    175.93573    177.59933    168.17325    174.98043    162.47424    174.64136    171.82128    179.21379    180.72445    183.11369    183.86698    182.05402     180.7826    179.79378    169.44028    168.12343    165.69001    166.26772    163.33376    167.00006    173.10888    180.06739     180.4239    191.66464    193.17651    210.14309    201.57494    191.48584     177.3474    167.55731    163.95743    159.61978    160.09518    146.49336          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       77.00    324.54733    310.62426    298.14797    288.26919    275.75128    268.92906     262.8817    252.60045    250.04817    237.12485    241.32419    223.19966    220.22771      214.321    208.36785    200.20898    201.89777    205.54321    207.21024    195.10066    193.16543    192.72477    200.47177    194.17051    199.82149    197.92088    194.15794    203.48259    201.79982    201.66191    199.28169    186.13549    194.24729    186.48245     186.3656    190.41284    182.17724    187.52795    184.02463    184.30681    182.56009     182.1212    174.27446    182.41582    173.94399    172.78202    173.76549    172.89342    173.12083    176.57674    172.46923    171.47501     168.0059    170.63173    181.15734    178.73553    173.42492     186.8465     185.2632    179.13555     169.3165    184.97773    175.50848    173.87854    171.08052    173.70933    171.66374    163.64163    169.56473    171.79791    172.63105    189.01779    191.45653    198.49293     199.6255    201.50241     200.8982    182.32164    178.05225    162.92234    164.98003    155.91891    128.42255          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       77.50    325.49507    310.02063     299.8838    284.47141    276.76668    269.21458    266.09549    253.42841    242.97942    237.60071    236.44256    225.14846    221.41094    219.05313    209.70913    201.76046     204.1739    203.95761    207.45079    195.57546    189.82833    194.28511    200.29438    204.51264    196.92572    197.22046    197.96606    210.71203    200.14074    200.63346    194.40191    187.30281    192.42267    181.85437    187.32645    195.68629    180.45415    182.00239    179.54667    179.02406    185.84513    186.11542    181.73005    183.05679    172.63805     174.4719    170.04058    174.46952    172.65213    169.84104    178.16931    174.88169    170.64683    174.71865    172.69205    174.61361    181.45216     182.7871     182.2547      175.556    181.32041    185.44929    170.60767    176.09931    174.14515    175.75144    172.31694    170.71385    174.82717     180.9707    173.05533    186.20691    200.79375    198.21317     201.9255    195.82416    196.27892    188.02482    176.70396    164.41124     158.4028    158.04041    142.85072          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       78.00    327.80045    308.92456     300.6537     283.2535    278.62757    269.76443    270.75866    256.12264    236.77557    239.30933    233.03859    228.14485    222.67993    222.94916    214.88758     204.5243    204.48926    201.81605    200.06206    198.18417    188.76977    206.16879    200.88323    196.37643    194.84245    201.47875    204.26485    203.99947    202.06392    197.23608    193.79237    185.19203    194.38361    185.00136    186.90367    190.90595    183.49239    176.98077    176.82561    181.67061    181.91221    186.00314    190.30129    192.29067    173.02394    176.63001     172.5858    172.19245    168.98406    173.50923    177.08384    166.43987    168.05543    168.50209    169.04443    181.05642     180.1634    177.99344    178.10943    182.26593    180.20896     183.0052    177.36337    175.06578    170.88198    177.69402    168.38305    170.98329    181.08943    172.04528    185.39362    188.93673    206.38667     203.5434     200.5649    196.62351     185.3414    176.73009    170.64033    165.61042    165.43772    162.11815    140.73207          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       78.50    328.94643    307.45796    302.51132    284.48898    279.88619    270.55101    268.15217    260.06582    240.04638    239.71169    231.80141    229.40179    223.95052    224.80019    218.90628    215.61169    206.96484    201.56178    202.06256    201.59071    188.30452    209.66834    189.34434    187.51357    194.91382    196.66143    209.56834    198.65794    201.73697    191.98085    195.17831    187.82559     194.1886    185.88778    191.00996    182.03703    179.37876    182.85693     180.4103    182.17812    190.50973    180.85621    185.63799    188.72142    175.33007    180.02627     181.6575    174.20417    178.35842    178.90447    175.36268    170.08321    171.01587    169.00813    166.73392    170.32955    174.33803    178.62664    182.54163    186.47565    189.67158    180.38443     173.7303       175.04    171.03829    176.59809    169.64953    167.57716     176.6733    174.66737    184.04489    192.75079    204.09018    205.08607    207.28385    200.27435    196.48155    190.82379      176.089     169.6393    164.03351    152.95199          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       79.00    328.00709    305.37103    300.71815    284.70937    279.49536    269.33653    262.51467     256.7768    245.32472    234.77345    230.22879    230.39505    227.42784    225.70802    215.64503    220.36937     215.4906    208.99358    205.32919    202.19315    192.72165    195.57146    194.16429    186.37531    200.29384    190.28476    201.18525      201.484    200.10668    195.46313    196.16511    192.01681    193.46077    190.71833    181.41205     180.9369    194.21335    186.66285    179.56231    178.52647    187.68884    187.15591    176.97005    182.59881    175.06959     181.2677    180.87903    186.18482    180.67252     181.8134    176.52999     168.9114    175.08439     168.1937    163.49017    171.99193    174.47647    181.22644    182.27911    185.51878    178.88856     187.5409    174.24773    174.74733    173.91988    165.75855    170.08974    167.75239    180.62906    169.07218    178.95479    190.13342    193.82478    199.49302    203.52251     204.1483    197.95777    192.42733     180.5974    171.39508    164.97074    154.96481          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       79.50    324.02072     303.7514    297.40083     284.2089    278.53887     264.6529    258.97638    252.12702    248.20222    231.47929    227.76662    226.68215     232.1082    225.69394    216.88034    219.05157    217.00171    214.04683    209.06644     207.4751    196.51641    198.31898    193.06969    190.45492    195.03433    190.90447    200.16683    205.54736    201.04756    192.22555    193.08104      198.701    191.45549    193.98547    187.93034    185.56949    187.55228    190.42045    182.87569    188.48237    184.19997    186.94769    180.86465    188.53044    183.55974    180.28692    174.73581    181.98671    181.57525    182.70245    180.72328    177.13226     176.4806    182.57499    176.31601    170.76765    174.55847    177.32058    183.08987    183.49071    183.35343    181.25244    171.83765    169.95069    172.63224    165.41126    164.40183    164.83112    164.88821    178.19105    187.45435    187.53742    199.97463    205.87026    200.33408    203.56194    198.16245    184.48429    177.35061    173.68219    171.65662    157.59416          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       80.00    320.82897    307.36308    294.18079    285.23967    275.55223    259.51582    258.47123    253.77523    248.70221    235.49712    225.46429    221.80347    227.44332    222.34689    219.60261    214.46789    213.52028    215.55155    210.36693    211.46002    210.13439    205.65336    189.31901    194.70674    191.81867    193.57565    206.71635     205.5564    198.82497    194.56316    192.78483    198.89919    187.48401    183.40232    185.73056    189.70626    178.60047    189.81153    180.17793    192.42205    191.84604    177.21811    180.88852    183.59154    188.47938    173.89282    177.83211    182.12382    181.60328    182.46737    172.65762    176.66197    180.15034    185.10972    179.82156    186.44001    175.14835    180.91404    183.12167    182.11804    186.97767    181.45644    177.70747    166.17739    169.71602    172.39217    174.25164    169.62832     172.3461    173.24698    176.14564    188.97675    198.82351    200.81586    210.69226    205.04416      199.035    189.07428    174.60527    176.20734    164.87934    155.67558          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
;

_pd2d_meas_2theta_phi_intensity_down
;
         241   4.00         4.20         4.40         4.60         4.80         5.00         5.20         5.40         5.60         5.80         6.00         6.20         6.40         6.60         6.80         7.00         7.20         7.40         7.60         7.80         8.00         8.20         8.40         8.60         8.80         9.00         9.20         9.40         9.60         9.80        10.00        10.20        10.40        10.60        10.80        11.00        11.20        11.40        11.60        11.80        12.00        12.20        12.40        12.60        12.80        13.00        13.20        13.40        13.60        13.80        14.00        14.20        14.40        14.60        14.80        15.00        15.20        15.40        15.60        15.80        16.00        16.20        16.40        16.60        16.80        17.00        17.20        17.40        17.60        17.80        18.00        18.20        18.40        18.60        18.80        19.00        19.20        19.40        19.60        19.80        20.00        20.20        20.40        20.60        20.80        21.00        21.20        21.40        21.60        21.80        22.00        22.20        22.40        22.60        22.80        23.00        23.20        23.40        23.60        23.80        24.00        24.20        24.40        24.60        24.80        25.00        25.20        25.40        25.60        25.80        26.00        26.20        26.40        26.60        26.80        27.00        27.20        27.40        27.60        27.80        28.00        28.20        28.40        28.60        28.80        29.00        29.20        29.40        29.60        29.80        30.00        30.20        30.40        30.60        30.80        31.00        31.20        31.40        31.60        31.80        32.00        32.20        32.40        32.60        32.80        33.00        33.20        33.40        33.60        33.80        34.00        34.20        34.40        34.60        34.80        35.00        35.20        35.40        35.60        35.80        36.00        36.20        36.40        36.60        36.80        37.00        37.20        37.40        37.60        37.80        38.00        38.20        38.40        38.60        38.80        39.00        39.20        39.40        39.60        39.80        40.00        40.20        40.40        40.60        40.80        41.00        41.20        41.40        41.60        41.80        42.00        42.20        42.40        42.60        42.80        43.00        43.20        43.40        43.60        43.80        44.00        44.20        44.40        44.60        44.80        45.00        45.20        45.40        45.60        45.80        46.00        46.20        46.40        46.60        46.80        47.00        47.20        47.40        47.60        47.80        48.00        48.20        48.40        48.60        48.80        49.00        49.20        49.40        49.60        49.80        50.00        50.20        50.40        50.60        50.80        51.00        51.20        51.40        51.60        51.80        52.00        52.20        52.40        52.60        52.80        53.00        53.20        53.40        53.60        53.80        54.00        54.20        54.40        54.60        54.80        55.00        55.20        55.40        55.60        55.80        56.00        56.20        56.40        56.60        56.80        57.00        57.20        57.40        57.60        57.80        58.00        58.20        58.40        58.60        58.80        59.00        59.20        59.40        59.60        59.80        60.00        60.20        60.40        60.60        60.80        61.00        61.20        61.40        61.60        61.80        62.00        62.20        62.40        62.60        62.80        63.00        63.20        63.40        63.60        63.80        64.00        64.20        64.40        64.60        64.80        65.00        65.20        65.40        65.60        65.80        66.00        66.20        66.40        66.60        66.80        67.00        67.20        67.40        67.60        67.80        68.00        68.20        68.40        68.60        68.80        69.00        69.20        69.40        69.60        69.80        70.00        70.20        70.40        70.60        70.80        71.00        71.20        71.40        71.60        71.80        72.00        72.20        72.40        72.60        72.80        73.00        73.20        73.40        73.60        73.80        74.00        74.20        74.40        74.60        74.80        75.00        75.20        75.40        75.60        75.80        76.00        76.20        76.40        76.60        76.80        77.00        77.20        77.40        77.60        77.80        78.00        78.20        78.40        78.60        78.80        79.00        79.20        79.40        79.60        79.80        80.00        80.20        80.40        80.60        80.80        81.00        81.20        81.40        81.60        81.80        82.00        82.20        82.40        82.60        82.80        83.00        83.20        83.40        83.60        83.80        84.00        84.20        84.40        84.60        84.80        85.00        85.20        85.40        85.60        85.80        86.00        86.20        86.40        86.60        86.80        87.00        87.20        87.40        87.60        87.80        88.00        88.20        88.40        88.60        88.80        89.00        89.20        89.40        89.60        89.80        90.00        90.20        90.40        90.60        90.80        91.00        91.20        91.40        91.60        91.80        92.00        92.20        92.40        92.60        92.80        93.00        93.20        93.40        93.60        93.80        94.00        94.20        94.40        94.60        94.80        95.00        95.20        95.40        95.60        95.80        96.00        96.20        96.40        96.60        96.80        97.00        97.20        97.40        97.60        97.80        98.00        98.20        98.40        98.60        98.80        99.00        99.20        99.40        99.60        99.80       100.00       100.20       100.40       100.60       100.80       101.00       101.20       101.40       101.60       101.80       102.00       102.20       102.40       102.60       102.80       103.00       103.20       103.40       103.60       103.80       104.00       104.20       104.40       104.60       104.80       105.00       105.20       105.40       105.60       105.80       106.00       106.20       106.40       106.60       106.80       107.00       107.20       107.40       107.60       107.80       108.00       108.20       108.40       108.60       108.80       109.00       109.20       109.40       109.60       109.80       110.00       110.20       110.40       110.60       110.80       111.00       111.20       111.40       111.60       111.80       112.00       112.20       112.40       112.60       112.80       113.00       113.20       113.40       113.60       113.80       114.00       114.20       114.40       114.60       114.80       115.00       115.20       115.40       115.60       115.80       116.00       116.20       116.40       116.60       116.80       117.00       117.20       117.40       117.60       117.80       118.00       118.20       118.40       118.60       118.80       119.00       119.20       119.40       119.60       119.80       120.00       120.20       120.40       120.60       120.80       121.00       121.20       121.40       121.60       121.80       122.00       122.20       122.40       122.60       122.80       123.00       123.20       123.40       123.60       123.80       124.00       124.20       124.40       124.60       124.80       125.00       125.20       125.40       125.60       125.80       126.00       126.20       126.40       126.60       126.80       127.00       127.20       127.40       127.60       127.80       128.00       128.20       128.40       128.60       128.80       129.00       129.20       129.40       129.60       129.80       130.00       130.20       130.40       130.60       130.80       131.00       131.20       131.40       131.60       131.80       132.00       132.20       132.40       132.60       132.80       133.00       133.20       133.40       133.60       133.80       134.00       134.20       134.40       134.60       134.80       135.00       135.20       135.40       135.60       135.80       136.00       136.20       136.40       136.60       136.80       137.00       137.20       137.40       137.60       137.80       138.00       138.20       138.40       138.60       138.80       139.00       139.20       139.40       139.60       139.80       140.00       140.20       140.40       140.60       140.80       141.00       141.20       141.40       141.60       141.80       142.00       142.20       142.40       142.60       142.80       143.00       143.20       143.40       143.60       143.80       144.00       144.20       144.40       144.60       144.80       145.00       145.20       145.40       145.60       145.80       146.00       146.20       146.40       146.60       146.80       147.00       147.20       147.40       147.60       147.80       148.00       148.20       148.40       148.60       148.80       149.00       149.20       149.40       149.60       149.80       150.00      
      -40.00   -163.92064   -171.98843   -391.59225    -602.0436   -140.53683   -518.17176   -494.75502   -216.80347    -400.3087   -398.34026   -463.45687    -447.0876   -673.06886    -384.5061   -491.00764   -421.39958   -556.08259          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -39.50   -316.22822   -202.02991   -366.83589    -413.4298   -346.72452   -587.49985   -530.86687    -91.11847   -409.66797   -368.99386   -351.34804   -430.61883   -732.73031   -615.48372    -527.2191   -585.03227   -462.47065          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -39.00    -461.0293   -263.23299   -370.99398   -277.48271   -580.49673   -549.84361   -534.41183   -140.03408      -369.74   -460.15298   -223.89481   -491.83418     -586.155   -595.05662    -655.8996   -587.36001   -576.38921   -480.03682          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -38.50   -531.28623   -344.09235   -430.64712   -432.55897   -786.96433   -513.23008   -555.44832   -366.45551   -286.63358   -545.73321   -344.76033   -492.92934   -461.80515   -587.73658   -690.49832   -542.20095   -573.75031   -417.69431          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -38.00   -537.84783   -358.53104   -537.56293   -497.86728   -624.70648   -529.85706   -477.69317   -633.38509   -315.01758   -560.45895   -427.24078   -432.34366   -526.15556     -631.759   -814.40081   -631.71591   -531.37105   -500.22165   -524.78651          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -37.50   -545.60798   -306.93369   -631.16482   -472.25341   -479.98787   -611.48426   -384.01766   -730.59108   -376.36884   -420.18043   -571.36258   -538.52193   -576.16709   -576.67755   -712.03547    -590.3581   -608.08279   -564.76186   -378.24569          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -37.00   -459.65618   -234.48696   -642.45136   -346.68287   -476.35513   -677.48981   -354.07032   -641.33749   -361.61061   -299.83529   -652.09204     -575.488   -399.91452   -513.80966   -585.45812   -393.28724   -712.15885   -507.14226   -406.35642          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -36.50   -389.63995   -175.28093   -590.25315   -277.63222    -519.6672   -683.22353   -434.39653   -728.26555   -325.69316   -327.57863   -613.78161   -502.23666   -481.23138   -484.75561   -557.24447   -512.04251   -759.89088   -619.21186   -517.38266   -700.36915          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -36.00   -402.99791    -217.9224   -450.57223   -193.52424   -474.61974   -730.31754    -436.4404   -760.90473   -414.98836   -436.40151   -572.46061   -627.44736   -668.80608   -331.28088   -496.05324   -658.43915    -818.5326   -493.63162   -461.96398   -489.09208          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -35.50   -490.35351   -198.93627   -291.82409   -179.97939   -409.18634   -859.47077   -270.13219   -487.85744   -558.12539   -510.47743   -580.50203   -757.06199   -683.86738   -387.40897   -435.98327   -546.36827   -934.99873   -379.46239   -522.05858   -344.72336   -722.56379          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -35.00    -583.8448   -116.63252    -246.0377   -156.17364   -362.13263   -568.39734      -58.378    -227.5778   -580.89802   -557.02075   -555.14958   -619.78109   -609.37176    -469.7732   -612.82093   -501.03845   -616.29506     -419.784     -486.725   -383.28481   -580.91389          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -34.50   -504.14012   -116.04107   -297.12623   -118.13585   -360.73515   -386.93454   -236.02325   -338.61424   -532.03778    -576.4838   -435.28248   -462.98379   -500.87965   -477.95878   -553.81355   -533.22788   -452.78824   -400.68598   -233.81413   -655.24244   -524.07324   -677.52859          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -34.00    -534.9402   -234.72866   -432.50782   -202.10217   -447.64644   -374.77842   -406.13721   -491.19078   -468.34775   -524.48643   -463.63793   -422.05586   -513.34406   -499.74735   -453.14109   -424.76139   -338.44212   -546.48407   -364.77989   -695.89646   -489.90367    -665.3629          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -33.50   -675.10547   -301.15591    -541.2475   -326.19593   -563.11324   -369.69621   -410.27992    -473.8014   -440.72085   -508.82448    -583.6511   -328.80013   -566.20124    -444.8857   -403.75277   -369.83514     -348.291   -789.71155   -409.40971   -637.96302   -510.33578   -638.83064   -358.41129          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -33.00   -712.18747   -270.26061   -565.49293   -448.13055   -575.40889   -389.71135   -317.55455   -356.95854   -471.27031   -552.13697   -581.48895   -258.07588   -632.20399   -413.22571   -439.90081   -357.49911   -359.57971   -645.98558   -427.05203   -683.67543   -611.48859   -693.46901   -346.01572   -658.73054          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -32.50   -610.53951   -188.02334   -573.37985   -299.80418   -412.37323   -415.91168   -226.27157    -356.8118   -461.91954   -492.80829   -427.36788   -243.17775   -597.44615   -429.28318   -481.87162   -403.75306   -332.21852    -512.6671   -422.73892   -525.08115   -645.68952   -626.29242   -528.20035   -616.72226          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -32.00   -450.09624   -124.38904    -591.0728   -192.33166   -220.36781   -379.02466   -347.21807    -528.2857   -374.50136   -403.39422   -356.36244   -196.38393   -523.66321   -577.21581   -300.13957   -333.56635   -423.39262   -460.84017   -621.47157   -613.51896   -629.49728   -427.25677   -494.10101    -554.9578   -381.57406          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -31.50   -361.39613   -139.23175   -618.45167   -262.24899   -196.40466   -321.13072    -451.9616   -584.18266   -276.73527   -368.67489   -453.64014   -260.57455   -450.92468   -638.68695    -267.3649   -401.60906    -501.1575   -491.95403   -687.15196   -722.28174   -495.87266   -367.64857   -557.83779   -598.73682   -573.83427          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -31.00    -311.5154   -193.55966   -502.00945   -508.75376   -295.36954   -354.15005   -514.21973   -531.60853   -286.98311   -318.71356   -517.60472   -181.71796   -429.93972   -668.53802   -356.14472   -615.32413   -560.67465   -552.47655   -636.03075   -713.56997   -495.21162   -417.81771   -640.16934   -553.71046   -424.72984   -334.81697          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -30.50   -276.14803   -209.76121   -404.53754    -687.8978    -347.4547   -395.15796   -546.12254    -474.8293   -419.27951   -306.94429   -337.38306   -365.02602   -408.89222   -688.41652   -490.87409     -585.469    -471.5887   -608.33276   -543.39966   -503.24108   -552.47214   -455.26261   -598.30391   -460.43321   -345.41699   -275.40534      28.8127          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -30.00   -256.22976   -188.36229   -350.97816   -731.61281   -307.70304   -365.05308   -537.30777   -477.72916   -542.38165   -364.32704   -322.84865   -696.36897   -389.61858   -643.40494   -390.42295   -460.15161   -277.71252   -633.36289   -593.07674   -459.14105   -496.40103   -574.22598   -582.07811   -190.33197   -345.58109   -262.97762     73.67832          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -29.50   -392.88167   -278.07261   -370.52415   -588.82695   -418.40048   -300.37827   -503.64271   -345.42723   -557.58355   -455.10085   -496.76729   -578.34264   -499.14691   -499.50215   -464.14103   -464.16819   -327.06723   -455.90887   -614.85695   -517.77604   -565.00582    -592.1393   -576.47422    -496.5248    -347.2094   -155.53968     28.91056    -16.97099          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -29.00   -526.45502   -340.60246   -357.54293   -461.91845    -526.5118   -254.79076   -386.05436   -308.16275   -490.12174   -476.08777   -410.29828   -494.29344   -577.72635   -409.95769   -678.65577    -545.1516    -566.3498   -296.94568   -555.82831   -516.15946   -510.97344   -524.45694   -613.68764   -577.98894    -399.0213   -159.63344    261.06611    -40.70137   -327.26722          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -28.50   -649.79361   -381.18759   -311.70545   -379.83542   -548.30995   -322.36719    -237.4125   -357.89736   -403.20729   -351.36711   -321.84465   -427.49181   -520.30048   -440.61151   -708.17379   -550.30479    -691.5117   -439.19951   -469.23231   -483.96648   -520.76288   -451.72939     -576.647   -318.70119   -365.87006   -107.17876    269.67667     81.63592   -229.45057    -484.2216          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -28.00   -577.07596   -388.91441   -325.32363   -476.67419   -487.00832   -414.88189    -312.5483   -366.92382   -420.89284   -266.30636   -471.72482   -396.80093    -486.3065   -391.22284   -640.38429   -467.82566   -839.74262   -393.36841   -366.07755    -512.8404   -658.19597   -526.34872   -516.49516   -453.85754   -430.22502   -177.76494    216.30121    175.08044   -142.93867   -469.13216   -424.09953          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -27.50    -421.1588   -401.76847   -313.86995   -563.77398   -419.45718   -466.72028   -408.80771   -254.55659    -485.0106   -324.85014   -507.87664   -443.95974   -345.34988    -414.4988   -566.34178   -354.52028   -603.96678   -428.41378    -497.1268   -494.34861   -738.21948   -666.46528   -516.92626   -574.96627   -433.46253   -126.63224    251.98957    174.09846    -99.84776   -396.07246   -611.30764          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -27.00    -243.4505    -422.7407   -271.84009   -531.37354   -442.60756   -348.16865   -488.00751   -330.94569   -283.31893   -498.23849   -338.40756   -485.42891    -257.5877   -478.01233   -573.21466   -385.26495   -542.05796   -509.89539   -386.94118   -457.52179   -700.60307   -624.01191   -461.51757   -746.95845   -504.80111     54.04659    338.36681     223.6526   -112.08664   -429.92678    -571.6885   -520.81826          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -26.50    -203.9618   -476.55162    -248.3453   -424.26114   -567.09436   -125.64445   -480.33374   -584.18964    -127.3315   -548.01042   -213.60468   -410.76041    -378.4648     -461.231   -465.83803   -505.13146   -559.16207   -676.48885   -286.77774   -456.41912   -682.76673   -603.93292   -468.43343    -473.1325   -425.22778     18.55006    461.92816    364.10311    -91.20223   -280.25133   -494.08332   -613.16348   -495.66883          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -26.00   -287.54708   -539.00871   -262.84776   -373.01657   -663.09819   -116.52095   -366.20537   -609.53245   -249.88514   -514.87785   -356.99681   -130.14304   -525.25317   -452.04291   -406.58804   -412.52807   -549.98941   -569.14749    -394.4258   -477.61535   -468.44351   -373.16827   -335.84368   -420.61491   -329.68457    121.22281    377.88862    288.17977    -65.44042   -272.98555   -480.72927   -610.09198     -298.304   -625.95831          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -25.50   -358.32351   -551.83689   -333.71298   -426.67352   -575.43194   -355.89968   -239.87188    -424.3375    -322.9265   -503.40693   -420.91914   -216.49875   -508.29761   -425.50644   -422.99384   -464.40475   -571.38124   -395.07751   -372.00512   -412.20914   -526.99665   -447.06778   -501.09441   -349.15741   -243.40652     97.08769    358.69643    343.05132     29.20025   -229.28319   -472.86563   -299.50796    -472.7451   -479.90866   -520.65701          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -25.00   -326.72329   -406.43261   -484.42444   -526.24656   -493.96847   -517.74923   -139.39166   -415.36435   -330.60219   -459.89957   -483.49855   -440.92317   -464.67772   -376.22338   -448.08317   -577.36851    -456.2911   -305.95617   -443.94413   -283.52721   -561.84794   -438.30883   -355.11419   -313.92212   -436.29902     -4.55528    362.59868    290.05962     -3.14861   -112.83776   -423.59248   -447.62589   -519.47928    -344.2052    -447.4537   -600.76094          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -24.50    -70.79742   -188.27793   -620.59669   -568.54662   -465.01099   -492.78843   -294.88309   -562.15794   -365.24137    -483.4836   -539.62345   -566.25534   -427.56689   -271.39678   -621.62609   -703.82224   -334.54304   -375.35725   -530.31554   -451.15499   -343.06199   -432.91297   -260.37681   -438.26057   -301.02986   -108.84902    508.11207    359.47896    -85.69207   -129.56333   -405.24836   -458.98468    -391.2453   -376.29813   -540.43638   -695.90901   -475.36231          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -24.00    284.16281     67.97253    -700.0187   -502.39639   -486.19429   -360.92194   -366.26213   -593.55261    -404.6493   -497.80396   -458.42344   -600.75758   -221.97617   -116.38779   -576.58394   -621.17621   -473.63547   -580.00515   -626.39406   -448.50746   -503.51442   -538.70184   -369.13533   -525.29524   -157.59255   -152.37486    519.09572     414.8274    314.11702   -264.78466    -402.7022   -488.60359   -396.01548   -746.59967   -777.83509   -541.68288   -384.88592   -597.15778          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -23.50    566.83345      22.5898   -696.29634   -396.14863   -523.60433   -243.23018   -275.47665   -413.07871   -482.47446   -518.72341   -417.85157   -603.75414   -290.84541   -239.35236   -359.56283   -611.92998   -520.34585   -492.02094   -541.00791   -268.47239   -620.73648   -572.91579   -526.08052   -477.65259   -355.14193     -2.77228    595.05542    467.23929    209.75008   -409.98087   -470.59696   -715.77723   -349.95244   -606.01189   -524.66499   -606.99223   -507.75165    -691.7138   -495.45811   -550.81932          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -23.00    182.15356   -150.15585   -693.45229   -285.34638   -478.14823   -241.00252   -244.61181   -217.31482   -575.25609    -545.8911    -442.7183   -602.79429   -521.57968   -365.88795   -344.06132    -514.6899   -339.09401   -317.45465   -469.02312   -549.78043    -455.8158   -624.72494   -708.09598   -500.94162   -311.19414    -37.51689    587.37573    697.29026    323.27518   -128.00182   -592.31832   -650.01351   -432.19119   -488.21804   -579.19715   -753.23625   -543.03831   -627.68381    -540.7852   -739.80443   -474.56996          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -22.50   -159.85374   -375.27238   -708.97706   -294.31837   -371.36715   -227.16326   -254.40649   -348.28272   -537.90308   -527.16342   -426.57004   -524.26075   -495.73541   -329.41537     -425.941   -446.76523   -278.40403   -470.81841   -496.07962   -636.36429   -419.72086   -355.74927   -726.23901   -321.86892   -213.51641      8.02722    721.14158    708.90565     451.6932   -248.13775   -424.80487   -444.67158   -441.66426   -484.46097   -493.85395   -409.71913   -531.29206   -602.26318   -578.79788   -535.95918   -610.84385   -596.45143          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -22.00   -354.22868   -278.69012   -727.96715   -338.01218   -258.86372   -213.16559   -399.23459   -509.01771   -444.18717   -435.76349   -438.83531   -464.46593   -535.59365   -335.12368   -503.43116   -328.72196   -291.95909   -516.87693   -529.43688   -517.10734    -511.4828   -360.89962   -645.13759   -354.68931    -229.4102    144.41681    700.79707    725.07424    365.50315   -284.67833   -305.69598   -446.73215   -509.89477   -462.26158   -471.03858   -516.10454   -487.53548   -653.95167   -509.55959   -698.79115   -715.14134   -509.92383   -727.44869          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -21.50   -263.08456   -218.49778   -690.80229   -362.77248   -222.90092   -207.29111   -546.36362   -513.14613   -433.97334   -322.07322   -466.54525   -334.12325   -442.86433   -376.10041   -597.14209   -301.74443   -387.18566   -449.74923   -451.18633   -415.45466   -439.76966   -411.41554    -496.7936   -515.56132   -215.60253     89.49238    673.71466    754.18406    365.42088   -165.86208   -328.34471   -537.47272   -448.08215    -605.5713   -650.94842   -707.75014   -619.47635   -414.83137   -581.42244   -702.90091   -788.67539   -588.59963   -771.06685   -697.75404    -575.2966          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -21.00   -131.21239   -237.73554    -633.7603   -351.99572    -248.1183   -230.01413   -466.71573   -392.18606   -398.51562   -322.33417   -458.99557   -163.59494   -242.41125   -427.86439   -677.98443   -390.07554   -552.99075   -453.52695   -465.58074   -327.36915   -423.99768   -400.52441   -392.82542   -508.97573   -319.21513    283.57446    569.04264    857.81355     546.0181    -88.33262   -318.56636   -566.96149   -527.34376   -512.26166   -599.99341   -686.85459   -327.21564   -357.82798   -697.84028    -540.9242   -563.10949   -693.61891   -697.24046   -605.15387   -421.90111   -529.09391          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -20.50     20.29539   -401.15992   -573.34764   -366.56889   -339.70776   -288.68388   -411.67853   -306.09676   -437.16849   -366.06466   -394.37541   -247.30152   -374.59607   -560.70961   -641.72705   -450.89385   -653.40612   -525.86601   -446.05602   -558.40965   -541.72006   -449.91404   -461.73384   -418.70668   -420.96329    292.45256     669.0865    925.65555    592.59898     54.73936   -351.25868   -608.75844   -569.37579   -454.94632   -369.92261   -521.84154   -502.53652   -412.80816   -690.10825   -405.65384   -448.05095   -612.13794    -581.3637    -702.3708   -339.88422   -541.08043   -473.26577   -627.33299          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -20.00     -16.1695   -565.00451   -510.36253   -410.66754   -308.17992   -363.93572   -332.37094   -378.89247   -507.14821   -291.82165   -432.84382   -391.48412   -420.69485   -540.78645   -557.33619   -447.00454   -596.67802   -466.74666   -460.33645   -565.51122    -367.2547   -527.58262   -490.55435   -464.57621   -173.02315    104.78736     803.4128    922.87698    459.42824    140.11027   -411.32153   -590.47184   -490.65229     -509.633   -557.81029   -503.09615   -648.97674   -421.55976   -449.72142   -523.62072   -506.06482   -551.36699   -470.54484   -519.04225   -477.57524   -586.23738   -825.14583   -487.69967   -610.33481   -612.18663          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -19.50   -112.71208   -717.92079   -475.07138   -414.46971   -164.51656   -396.76623   -312.11824    -479.6311    -657.6735   -155.14591   -589.53148   -405.97197   -396.22282   -514.74981   -471.86166   -478.06405   -395.61013   -412.52888   -657.20495   -541.84168   -301.60742   -624.89698    -424.8414   -404.05494   -235.21462   -108.10411    827.06693   1172.03538    483.79449      4.61358   -520.54484   -415.03956   -450.00682   -576.59531   -356.29483   -454.59152   -615.71417   -468.96017   -463.37009   -551.69216   -496.53413   -489.97956   -412.78856   -337.55509   -460.00122   -529.73398     -495.226   -453.05043    -485.4084   -616.21989   -635.40107          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -19.00   -241.40165   -771.50993   -510.65332   -365.41591    -307.4292   -397.26884   -338.24671   -389.48285   -773.99282    -185.8118   -326.75394   -446.60027   -386.88767   -570.41996   -445.20555   -507.08312   -403.64259   -374.87983   -643.76138   -548.87112   -350.98979   -712.62831   -559.98455   -250.18704    -319.2532      22.6921    682.65071   1096.38794     819.8016   -210.30501   -509.11114   -404.94497   -408.68652   -575.74557   -580.56452   -453.66034   -455.92448   -425.15257   -460.99862   -391.33495   -548.22906   -508.26645   -511.13927   -490.21884    -663.8638   -452.06007   -578.05162   -563.71684   -580.04774   -703.10563   -790.61681   -593.57463   -538.28892          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -18.50   -292.48452   -724.48781   -566.43408   -278.61795   -506.87893    -372.2158   -381.55333   -368.40557   -689.00115   -392.67324    -47.17105    -508.2448   -408.87508   -588.40782   -389.01573   -518.97719   -442.62936   -431.63307   -482.33247   -496.36812   -480.84514   -600.25373   -631.95674   -410.15359   -401.05603    193.28068    975.68172    877.27574    924.06332     85.61873   -302.42991    -347.2738   -255.96225   -461.20819    -652.0226   -539.91161   -476.80234   -568.04223   -508.94685   -525.86088   -351.64115   -547.04501   -348.76056   -615.00982   -652.11787   -599.80295   -532.48227   -656.08779   -579.61188   -609.23941   -686.80778   -720.29867   -785.76121   -635.47674   -749.44882          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -18.00   -353.78118   -667.03935   -458.21266   -222.77118   -606.72453   -352.53285   -395.64813   -537.62754   -498.37868   -551.91662   -196.45768   -489.33137   -453.19883   -531.72142   -271.63686   -523.69644   -621.17914   -492.21327   -531.16161   -390.99345   -467.69721   -650.24394   -537.01222    -457.3178   -350.88109    400.98007    978.23763   1054.13349    773.13429     58.96246   -278.57108   -427.72682   -446.20391   -436.96656   -449.04471   -437.03145   -448.27531    -559.6346   -467.76388   -558.52638   -311.48576    -583.0303   -462.12177   -560.88402   -606.87893   -513.70898   -600.81451    -752.0145   -474.76677    -672.7194   -527.96183   -654.34375   -494.59429   -591.41404   -673.77258   -273.66559    -232.8239          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -17.50   -422.59019   -557.77945   -359.19973   -311.06681   -518.58141   -358.35331   -358.43244   -690.83508   -397.39286   -593.50236    -346.4157   -408.76734   -460.79442    -373.4455   -293.78034   -461.95682   -557.85294   -367.29801   -596.65921   -428.87968   -303.29435   -546.44533     -436.591   -413.79482   -222.97078   -115.62571    848.08763   1209.94865    678.83537    144.76872   -272.97059   -533.14127    -569.7785   -545.71841   -339.59256   -521.61234   -439.94664   -474.63901   -538.51744   -588.54987   -523.27839   -494.08976   -435.77287   -573.75545   -544.83273   -539.21581   -469.19115   -566.37767   -561.52548   -483.75613    -676.4138   -523.91974   -745.32478   -534.03292   -442.07241   -546.58663   -348.29094    -45.03495    -47.92609          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -17.00    -458.0594   -425.07445   -317.02422   -427.25775   -393.15354   -522.05371   -332.38613   -662.51904   -474.44124   -563.72813   -462.88075   -376.08624   -478.15701   -180.95755   -342.48931   -486.69508   -286.23626   -426.90779   -523.36244   -504.61634   -342.30476   -338.75758   -417.41079    -405.0285   -280.16973    225.93851    760.56598   1238.27483    976.71926    247.68418   -289.71049   -403.00835   -346.97415   -542.31549   -382.80301   -265.97031   -452.84761   -599.69926   -533.28703   -579.16801   -417.89506   -466.08889   -480.39695   -280.02064   -523.79039   -478.54782   -516.36764    -682.3908   -496.55638   -453.73948   -592.02896   -572.96138   -580.56764   -544.90565   -564.25299   -587.14871    -301.4431    -20.34521      9.27532   -277.07697   -361.11062   -723.82531          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -16.50   -472.96989   -332.51344   -312.06124   -503.55689   -566.12546   -645.81696   -366.90016   -494.39753   -586.72795   -452.67574   -576.39628   -322.52132   -403.91989   -218.86378   -400.11349   -510.81375   -366.43859   -571.47846   -407.99456   -338.55478   -371.40987   -337.12554   -317.00999   -329.69839   -185.16171    207.29454     774.9926   1155.72809    842.48223    137.68051   -231.69323   -439.32582   -378.11765   -580.36878   -484.98246   -392.35648   -386.39909   -488.61166    -513.7191   -500.72588   -632.08239   -463.65516   -465.20734   -274.16164    -570.1162   -562.64017   -489.00296   -554.28314   -552.72379   -677.08805   -489.30192   -426.47959     -568.611   -550.29335   -410.86152   -452.93533    -54.86148     -1.02537    313.83709    -48.39313   -272.51424   -592.79752   -564.89372   -599.75752          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -16.00   -480.83259   -346.02957    -292.7242     -486.275   -729.36407    -524.7157    -313.6301   -403.44522   -508.49263    -366.7255   -412.61145   -240.22647   -409.73225   -413.57601   -298.09693   -441.58575   -415.46079    -498.6226   -396.26405   -462.34625   -384.93306   -486.45066   -478.15528   -262.83794   -389.03098    183.91422    722.64943    1091.4892    620.64499     86.46063   -288.80329    -300.7094   -490.44571   -473.60421   -602.95247    -546.8747   -286.72016   -521.71478   -411.71134   -477.05813   -382.66966   -417.90904   -563.17038    -411.7607   -687.95989    -525.3115   -396.27974   -607.00073   -610.72705   -504.66463   -586.16182   -438.54241   -537.00694   -407.39391   -601.82867   -359.91649   -221.21941     149.2582    158.32694    -20.71128   -367.62304    -521.2395   -420.17001   -564.04226   -518.52809   -585.84256   -749.00086          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -15.50    -603.9791   -364.35477   -321.11268   -466.50413   -678.28611   -290.36487   -278.55183   -307.26177   -435.68122   -437.45553    -251.0547   -264.44523   -542.06115   -344.14162   -262.68881   -485.20301   -374.51845    -534.8299    -274.3579   -511.77352   -389.09361   -547.42713   -451.06593   -339.89136    -315.7056    353.23694    917.95707   1053.77668     573.3832    139.49886   -353.59478   -276.77895   -516.28467   -392.59428   -498.57414   -435.91652   -410.44434   -486.12635   -402.42123   -457.46159   -518.19417   -538.60324   -490.59255   -431.82225   -483.44387   -583.50013   -499.68118   -500.91887    -532.8651   -456.60946   -692.73561     -573.826   -506.67029   -515.89308   -450.05419   -195.38356   -241.82943    207.23088     236.7703    141.63557   -358.90048   -731.65868   -497.33831   -544.72062   -650.03938   -508.32521    -583.4263   -656.73925   -599.85971          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -15.00   -760.11997   -391.67494   -342.17618   -439.37937   -474.28608   -161.90371   -445.47816   -213.74187   -433.06329   -541.95385   -348.50698    -287.3766   -448.54811   -148.68262   -242.33633   -636.11476   -371.95034   -502.60188   -277.05073   -319.98754   -365.77763   -378.92551   -426.81999   -324.61729   -240.08833    236.16098   1007.80213   1005.28953    691.56611     37.98546   -366.87574   -258.82996    -537.4418   -394.21533   -390.75664   -390.93134   -472.37585   -533.33928   -434.22488   -489.13416   -437.37907   -429.41163   -515.37185   -491.80314   -399.07447   -716.73781   -667.20018   -662.56654   -469.82083   -476.63313   -575.18973   -517.17851   -543.21281   -462.39733    -428.7658   -434.37898   -153.64038    300.74409    144.54273    300.75785   -417.00259   -496.52681   -488.76643   -513.36177   -563.29088   -432.38783   -536.16768   -519.02309   -672.43952   -347.76503   -411.02959    -86.62529          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -14.50   -882.00983   -468.59652   -335.00271   -404.34461   -301.85629   -306.28815   -616.71988   -214.91938    -403.4738   -435.64814   -500.14855   -351.99158   -343.51231   -461.98619   -204.54764   -498.08039   -386.55421   -383.26515   -218.40831   -307.80301   -270.37451   -473.45636   -290.66945   -365.68344    -469.0575    207.58798    672.10275   1099.20929    853.15797     89.25865      60.2108   -363.92128   -504.72921   -413.10185   -580.91699   -330.73629   -490.88112   -556.53879   -397.27757   -541.68387   -419.00339   -495.31215   -333.21979   -406.33271   -596.03564   -521.13958   -477.70141   -624.02364     -320.461   -267.16265   -493.28129   -381.19812   -596.97012   -358.20566    -511.1199   -283.97881     88.56768    313.29565    479.63409    222.57255    -97.63571   -591.37635   -439.22347   -623.31789    -383.5909    -340.4992   -638.97299   -593.08183   -643.88459   -527.32081   -223.95481    -30.65283    -27.63163   -167.12311     -284.944    -175.5423          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -14.00   -619.58858   -582.62998    -309.8246   -373.96516   -321.55346   -473.64339   -487.79738   -148.95243   -335.37233   -332.85373   -578.88394   -469.34783   -340.23745   -647.73472   -455.38066   -408.51093    -395.7221    -306.2676   -296.61921   -368.40121   -210.45959   -547.38942   -226.39084   -371.03828   -372.42267    255.90603    820.18865   1269.83781    987.15168     212.3355   -294.30667   -473.44352   -492.54319   -498.92806    -546.4955   -310.34397   -267.44102   -525.12679   -187.72766   -347.80324   -693.21312   -475.12656   -411.11594   -489.15838    -494.8571   -667.51841    -465.4801   -518.15984    -435.6212   -670.13181   -340.75039    -373.6792   -571.62703   -421.22331   -353.85592   -536.69732    112.28176    420.35103    575.58945    517.90798   -128.99057   -422.24333   -416.18737   -474.47727   -457.35967   -552.26679   -413.47207    -542.4398   -468.28739   -256.15645    -90.66169   -131.48127     225.1133   -117.09216   -234.46518   -165.94827   -364.53982   -529.51718   -565.85697          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -13.50    -390.6624   -528.08934   -346.22982   -286.45695   -370.99705   -431.06538   -326.62686    -84.26425   -314.28522   -333.46649   -562.06753   -599.88572    -336.1449   -511.42095    -537.4973   -544.24356   -384.20232   -396.24226   -393.87263   -496.04067   -217.66352   -343.41444   -145.98534     -352.531   -136.86357    404.60649    967.74887   1226.34084   1112.20953    333.24941   -410.65064   -397.18854   -385.25428   -576.72929   -498.24821   -393.98468   -313.91071   -356.17956   -463.66074    -355.4823   -356.67101   -551.40895   -318.93213   -541.79663   -443.82523   -516.08501   -308.24679   -302.81703   -417.06302   -498.34159   -362.97799    -576.5853   -580.93382   -608.04203   -494.16329   -413.90361     20.50623    428.89101    700.82503    352.68455   -171.04514   -294.64349   -393.15084   -359.14387     -473.583   -596.86016    -475.2759    -666.2514   -332.34537   -340.97742   -163.52043    -86.60196    135.16254     63.95005   -129.91402   -137.70836   -232.36501   -332.56916   -389.60731   -537.96312   -370.54508   -597.47198   -568.28563          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -13.00   -258.52405    -440.9847   -375.94881   -227.60933     -336.533   -361.68304   -338.02997    -60.03298   -413.94194   -315.75493   -394.62507   -670.07394   -249.03511   -453.10675   -540.70297    -588.2992    -286.2065   -391.14177   -460.39426   -404.67729   -193.97199   -202.70804   -199.68964   -505.45342      4.92974    184.44019    988.01423   1428.73311   1057.42255    294.63878   -418.79793    -78.34412   -341.67805   -528.56104   -425.19049   -337.77473   -389.72581   -252.16875   -381.42695   -330.39253   -629.67524   -579.32026   -321.00631   -639.03986   -555.03851   -353.48615   -386.08163   -529.62574   -517.97169    -442.5818   -471.19342   -378.22195   -481.06823   -528.87196   -402.85064   -376.87932    -51.85548    360.96814    569.91815     344.4791    -20.22477   -343.80239   -559.50636   -461.82665   -363.59269    -438.5766   -652.23432    -431.0006   -542.92194   -255.99998     15.28473    387.44993    333.80219    289.73711    -92.35318   -151.72269   -259.15336    -269.4827   -428.44814   -532.12799   -469.20583   -566.81266   -515.04213   -592.00739   -440.57351   -511.10052   -546.83605          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -12.50   -348.15755   -367.09912   -383.39398   -264.01314   -250.26229   -387.45465   -418.24165   -209.22006    -534.5193     -291.671   -284.17854   -529.75702   -217.44281   -468.21099   -502.77069    -562.9357   -370.19434   -361.98542   -287.66946   -248.37414   -297.64934   -299.00704   -483.41663   -400.62977    -136.6299    297.24165    953.77928   1480.30423    925.49444    462.18499   -306.00172   -349.98609    -508.5705    -335.6116   -487.18118    -297.8659   -259.84268   -309.27771   -287.04206   -414.69256   -568.05862   -537.48787   -327.64877   -671.21043   -405.15249   -518.00061   -518.33866   -232.03856   -483.93526   -579.83587   -492.03479   -494.39342    -476.7008   -433.63214   -327.87555   -270.18805    152.93962    673.25231    713.25591    599.71357     189.3875   -244.59142    -460.1543   -423.71628   -450.96697    -500.4255   -422.38754   -536.36222   -484.65919    -366.7976    -23.82689    209.26302     287.1412    272.14582     28.41482   -247.10948      1.86693    -266.5179   -348.53451   -551.64998   -560.48158   -684.41368   -734.93708   -493.84535   -464.98395   -524.85873   -500.82059   -560.31195    -561.6133   -587.85472   -471.44739          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -12.00   -415.89551   -381.18234   -387.18778   -429.05169   -203.20871   -479.67843   -490.24089   -361.32564    -458.2419   -275.28107   -353.92232    -430.2462   -289.50388   -428.84977   -462.35386   -502.15152   -509.68807   -478.09809   -313.90764     -188.551   -340.27529    -514.4025   -413.68744   -304.29377   -204.08123    274.01625   1129.85694     1724.404      940.537    208.32117   -184.98691   -406.23472   -431.52001   -478.05401   -564.91853   -392.11391   -497.08377   -382.91429   -327.40298   -576.27382   -595.24102   -431.70825   -412.53958   -319.21198   -359.48944   -394.08507   -571.08543   -542.23753   -420.42746   -409.84211   -386.95562   -514.12631   -390.97639     -693.045    -339.7739   -377.07027    208.40868    670.49369    933.81925      439.107    272.38785   -307.93113   -419.25659   -517.93353   -542.18089   -419.38791   -506.06593   -439.84394   -505.35102   -256.64473     13.57766    442.46294    513.41126    407.46547     112.0527     -9.80703   -121.06721    -27.23044   -450.37755   -396.62777   -477.99414   -564.96092   -560.39808   -579.69913   -488.00087   -731.94592   -517.58087   -608.80537   -427.44285   -460.87611   -267.81615   -403.12953   -413.31074   -463.47311   -499.07589   -553.38194          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -11.50   -448.35205   -414.37424   -445.33576   -565.35415   -276.06535   -529.95083   -561.12427   -506.55256   -375.78388   -407.33767   -359.90412   -455.59698   -325.29914   -306.25535   -281.49603   -428.05943   -343.86683   -519.73438   -493.71906    -305.3645   -356.99414   -478.52772   -328.50955   -106.57659    -264.7599    128.59389    988.33236   1732.58207   1206.35327     291.9814   -258.70427   -439.72254    -288.9041   -554.10338   -573.23562    -417.4335   -194.18907   -454.43018   -577.08685   -650.85485   -499.20469   -473.53497   -449.53275   -467.24616   -455.87627   -339.12269   -545.73867   -519.37426   -479.17465   -419.16341   -461.35572   -388.21979   -559.48351   -304.91149   -445.70927   -362.60191    121.46178    531.48743    806.74875    523.47018    301.72463   -162.92894   -537.92554   -451.12206   -217.70196   -447.45561   -359.15211   -440.47978   -381.17009   -267.82904    -16.95027    403.84715    471.30271    514.50071    207.01571     24.84393    -65.36797    151.17522    -333.7115   -456.52006   -554.50723   -485.48706   -420.19704   -525.65662   -504.32769   -544.95735   -454.16634   -516.18273   -529.51856    -511.9728   -355.55698   -395.20834   -324.10685   -522.13237   -369.90039   -547.57177    -509.5556   -569.25889   -551.02062     -585.627   -558.96123          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -11.00   -445.94588   -448.15877   -513.15286   -529.90661   -321.99667   -490.49213   -521.17158   -670.25927   -298.29211   -541.48281   -206.12092   -443.12404   -415.96246   -384.78827   -105.82487   -432.91706   -219.22141   -409.98646   -396.90858   -431.13624   -339.67736   -507.82853   -330.54898   -179.43989   -253.83487    417.79765    992.55377   1447.92859   1355.42934    380.40918   -239.68444   -323.96386   -318.79518   -439.82234   -389.26198   -410.60164   -255.32234    -613.0891   -277.17211   -570.71499   -315.26753   -315.50495    -306.7858   -354.57988   -482.11614   -417.60149     -337.815   -399.07597   -305.74837   -406.40481   -433.10129    -552.2823   -396.81734   -341.79278   -328.13992   -339.32567      8.28434    685.26224    953.94822    716.42181    333.08664   -255.10652   -384.79133   -236.83083    -522.7798   -586.96505   -412.71977   -483.31121    -401.3119   -314.35992    168.48204    569.66709    774.52583    398.02643     135.6123    124.78265    -63.98972    -11.42953   -317.82175   -486.48034   -455.89661   -510.38618   -550.20998   -397.30894    -483.5467   -542.06261   -559.53561   -456.66751   -346.21963   -457.46569   -300.75314   -449.25172   -368.82308   -360.82159    -477.5278   -582.81817   -597.96958   -399.95603   -520.65302   -379.68058   -371.75725   -453.92791   -474.81326   -334.23837   -374.79674   -512.62514   -631.23714          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -10.50   -450.20637   -434.87418   -559.78253    -443.1169   -281.75633   -487.84652   -321.00339   -624.40899   -207.81494   -386.09637    -138.7991   -435.10747   -368.15098   -487.33574   -324.95667   -403.02719   -119.84456   -110.99553   -361.12293    -614.7444   -373.16695    -552.1748   -283.10433   -216.34243   -228.50174    425.55889   1174.17576   1604.37921   1162.34332    343.65122    -54.93303     -301.117   -262.60742   -380.67484   -265.48607   -486.11572     -407.676   -426.39754   -220.24917   -447.84935   -341.82862   -484.23685   -321.13876   -320.17772   -421.25599   -369.29588   -373.09762   -437.43815   -482.55708   -428.73982   -466.34316   -445.58089   -605.02199   -291.92499   -528.09858    -353.2056    130.65862    606.54268    896.00345    690.16905    316.94205   -303.93092   -244.07123   -329.21575   -433.12017   -495.70158   -429.61875   -339.64505   -537.63588    -336.4865    300.48838    439.83458    676.70494    646.08439    404.61044     37.45925    110.12334   -136.47017   -450.70195   -405.42024   -504.98457    -732.0773   -634.40699   -498.97465   -483.22274   -384.08894   -412.10419   -340.40195   -504.66373   -566.67359   -291.75574   -264.11583   -401.42468   -292.91842   -496.86718   -401.47802   -553.88453   -547.95563   -488.26454   -441.18863   -457.44101   -432.52787   -436.41638   -404.86087   -512.93253   -496.89813   -587.30966   -605.57085   -575.25882   -543.53974   -621.06627   -614.46113   -571.26252          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -10.00   -468.63173   -436.43425   -529.21468   -361.26139   -235.23485   -505.90277   -179.97403    -393.8219   -285.37814   -230.94658   -124.79055   -479.51841   -245.81297    -303.3948   -441.80771   -292.65774   -289.23333    -78.05073   -437.06843   -517.91751   -374.97749   -614.95345   -152.69931   -212.00828   -283.59186    440.80971   1484.87395   1684.39779   1121.88134    339.75918   -125.59828   -298.75689   -184.99669   -419.67873   -330.00254   -470.26567   -292.37316   -379.51228   -550.02226   -530.33879    -418.6434   -485.68281   -402.73796   -490.75213   -421.47426   -373.91917   -470.95553   -552.22922   -442.03296   -390.41994   -533.43493    -331.5558   -379.40916    -406.5163   -499.63207    -65.23318    121.42883    754.64928   1123.77782    727.81953    185.46544    120.71047   -352.90079   -344.48708   -408.85159   -456.16049   -487.28658   -293.18831   -495.84451   -131.96156     172.1686    530.91556    826.99587    763.01238    354.70305     78.90359    240.47635    -25.25733   -263.78064   -481.24457   -501.01777   -408.85589   -425.55618   -438.39982   -511.57546   -410.93023   -444.13426   -405.05207   -419.80768   -493.22941    -371.9652   -138.30596    -268.5619   -224.67371   -606.86142   -610.91003   -564.85259   -476.64576   -397.17422   -553.85982   -321.69938   -384.06202   -180.34087   -464.64063   -561.80792   -557.30642    -468.7496   -626.50525   -639.11672     -451.119   -663.86693    -479.2447   -511.44069   -595.48248   -659.25222   -476.09046   -439.79573   -308.30143   -448.46457   -419.82321   -551.12319          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -9.50   -507.06283   -418.13828   -513.72231    -297.0723   -272.12319   -486.88481   -228.46315   -263.24907   -399.17026    -298.5566   -145.62294   -522.21117   -267.10137   -217.36249   -414.09656   -298.34707    -373.5284   -425.12703     -297.139   -388.81655   -326.77651   -475.66506   -138.00171    -71.22503   -222.23864    462.52912    1193.0321   1797.72973   1282.30331    631.36729   -243.87927   -207.67414   -435.21481   -298.86745   -387.40374   -232.58539   -324.94723   -311.39382   -333.21603   -389.12563   -350.94151   -607.53494   -300.29561   -444.92371   -299.28664   -412.66431   -355.55141   -365.25297   -429.02126    -382.6801   -355.91471   -231.56684   -384.10779   -431.25688   -348.72812   -139.64314     55.84689    549.72663    1142.6844    628.12192    227.73894    -10.89145   -517.54527   -335.11471   -443.94515   -277.83322   -281.53359   -394.05351   -407.74659   -205.55457     77.92897    554.79904    853.66577     702.6004     254.0094     70.31343    201.01723    -12.92265    -98.04996   -227.62971   -488.64636   -468.40184   -543.53094   -374.25728   -503.61411   -392.74037   -404.58136   -494.60218   -392.63374   -365.41872   -289.16067   -273.06392   -234.02232   -313.31249   -334.16001   -407.06149   -538.79051   -571.51316    -507.7187   -413.24939   -406.59335   -424.54054   -253.82032   -355.46543   -457.77479   -370.31268   -486.26854   -526.21731    -549.2102   -474.21386   -521.35977   -653.03417   -573.42326   -520.26961   -649.76451   -531.99671   -283.61036    -33.42215   -234.66297   -379.17156    -480.0169   -422.33184   -563.16776   -427.36257    -475.3406   -205.61472   -136.96433     -75.8895   -298.97251          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -9.00   -554.78605   -344.83375   -494.55469   -223.42168   -445.40909   -409.84471   -288.85192   -296.42673   -434.98203   -350.18287   -376.18323   -441.43268   -307.85725   -232.33662   -347.33701   -321.12597   -275.53934   -425.61677   -232.69951   -293.46312   -409.61764   -316.58525   -354.23044   -356.67647    -98.79984    402.80169   1221.30087   1774.02426   1680.37291    420.24074   -199.85178   -251.32572   -182.14184   -199.08843   -455.16899   -412.64226    -500.5919   -371.84302   -468.41857   -485.55372   -414.80782   -385.01387    -477.0674   -507.26917   -359.62634   -417.69198   -338.16881    -476.6546   -386.50816   -466.92111   -289.26323   -417.26705   -542.50954   -451.04154   -394.46619    -80.56396     86.72124    644.75884   1139.32981    892.79577    413.83694   -135.61316   -488.02384   -297.26038   -459.32928   -332.56193   -367.63856   -405.61042   -291.48968   -133.89642     24.44806    513.60275    903.95927    851.43957    474.09866    356.63927    307.17483     -4.18563   -170.90024   -169.91259   -274.49607   -517.24009   -553.89343   -427.38052   -510.90897   -567.12725   -481.88051   -551.32842    -736.6109   -442.85125   -143.74908   -247.79805   -114.27948   -353.43306   -464.45135   -395.87995   -453.45587    -506.1649   -414.90793   -503.18195   -253.19593   -277.93761   -279.81812   -317.62411   -307.53563   -297.38996   -412.28873    -533.5894   -589.78386   -524.52763   -478.03718   -595.11322   -447.35129   -540.51041   -500.26477    -498.9403   -323.97706   -294.57432   -233.85504   -232.88533   -386.98244   -452.24762   -412.47031   -379.61693    -419.1774    -81.03928    -23.83762   -148.62546   -230.50316   -444.30228   -462.79753   -522.40991   -438.18817   -553.83229   -576.95821   -421.79804   -290.11293   -231.97888          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -8.50   -535.94377   -252.73044   -437.49372   -253.79292      -647.13   -328.99189   -335.99716   -387.45656   -421.69458   -354.31014   -544.43776   -341.15296    -380.4386   -241.68722   -267.38262     -370.718   -302.37322   -344.42244   -360.47235   -277.12132   -426.79118   -164.53508   -330.54204   -443.33813     15.67032    157.81607    1233.8409    1660.2497   1514.94646    288.95099   -155.65217   -236.28794    -298.0102   -333.36504   -384.26018    -521.1506   -507.18447   -481.72638   -225.31362   -670.02725   -286.52719   -366.10347   -335.34201    -393.6556   -326.26826   -480.59453   -425.47407    -353.8672   -476.12007   -538.56617   -404.09089   -396.35466   -385.35688   -395.41988   -344.73553   -223.15571    132.43227    614.73368   1174.16305   1046.20829     361.9518   -223.99132   -237.42996   -290.74385   -408.76963   -415.00809   -241.67417   -249.85532    -446.1422   -169.27307     50.61508    623.03772   1046.31208    644.87481    523.63278    365.72026    361.12926    252.71276     10.12497   -413.25542    -358.5029   -609.64139    -300.7041   -589.74605   -403.17066   -482.41474    -443.9132   -369.96465   -397.13116   -287.60684     10.49912     -47.9068   -297.28189   -336.89065    -289.1327   -394.79559   -527.96552   -407.53348   -503.89152   -252.98846   -273.44091    -185.9365   -245.64209   -201.27461   -335.77974   -334.77209   -480.62008   -548.19225   -423.66207   -589.57882   -546.67492   -580.28351   -459.64065   -403.70848   -345.73487   -304.71965   -272.17619   -135.51358   -269.33037   -196.77951   -257.23853   -517.29511   -423.25903   -369.55419   -252.46662    -21.16185    -65.55917    -75.07255   -312.88301   -375.42627    -409.4825   -385.74813    -651.6021   -505.56078   -622.09249   -491.22806    -222.7128     20.40027    427.30119    718.92558    542.66663    265.41603    -37.29392   -276.30181   -446.79307   -409.65972   -552.75461   -490.21677          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -8.00   -450.23036   -215.60406   -348.95564   -325.88347   -567.95251   -254.55828   -384.22753   -584.24436   -291.09408    -370.1517    -504.7435   -320.04398   -443.50701   -197.62722   -422.46496   -372.33419   -342.41526   -316.23561   -417.64222   -282.66459   -385.64108   -280.78497   -354.91696   -408.81076    -72.92797    172.74877   1173.26623   1720.70201   1299.69923    270.50314   -215.61551   -361.74744   -280.71998   -395.96839   -232.95783   -228.84091    -408.5504   -385.04632   -330.36631   -413.37468    -584.1962   -403.21823   -457.33657    -392.8023   -326.43109   -426.41707    -536.4623   -364.69487   -467.78975   -215.18248   -358.77439    -177.3823   -462.87774   -360.35817   -319.08855    -205.6868     85.62029    595.99991   1085.73527    766.64128    619.19304    -95.71141   -293.11619   -291.26899   -317.47542   -367.07558   -459.44725     -423.548    -326.6198   -156.06528    121.90433    527.31841    924.07856    655.16663    696.02931    452.56423    254.78194    135.48137      3.69179   -392.03168   -368.63713   -328.70296   -441.47877   -435.40779   -482.32363   -457.94085   -444.58906   -427.98486   -313.22246   -334.84408   -169.22513    -88.84946    -30.75056   -145.89639   -334.69692    -340.6775   -379.94942   -478.27879   -362.49317   -448.42804   -222.94426   -294.83899   -129.90961   -308.77055   -535.25199   -411.00457   -519.31803   -455.57129   -454.97645     -604.573   -503.61075   -420.30948   -556.05262   -530.56584   -505.24169   -196.50594   -328.27528   -188.74823    -41.96149   -261.91103   -364.15075   -315.63213   -499.62669   -381.73602   -246.46927   -107.42484   -161.84107   -199.52164   -214.58242   -162.86974   -504.73179    -579.3691   -497.40232   -478.19335   -489.02247   -400.60596   -202.72095     55.35825    334.18954    685.06112    438.83699    238.59842     -0.57121   -225.29222   -272.67482   -530.09112   -650.79132   -415.16432   -489.23762   -497.06196   -621.51201    -506.8757   -530.59896   -673.80298   -474.35388   -610.77744   -563.03442   -612.96399   -498.78494    -442.9715          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -7.50   -363.62214   -222.37796   -309.99073   -366.20499   -470.88653   -148.40113   -427.60854   -740.15755   -113.48086    -427.6489   -480.69189   -305.73331   -462.23337   -279.58328   -497.42865   -263.71083   -194.55258   -268.81127   -413.18576   -150.61021   -484.65409   -419.64845   -373.13402   -345.60462   -210.00352    247.82474   1047.60752   1646.73202   1234.40091    436.22772    -55.16314    -153.7365   -304.66166    -366.4116   -311.48115   -348.97708   -344.81713    -319.3854   -497.37731   -342.91591   -429.69834   -393.31338   -474.84044   -479.12722   -293.90485    -264.3897   -391.55917   -414.07938   -437.83746   -470.78207   -439.71131   -426.38899   -298.65707   -434.61481    -292.0735   -198.26517    236.35532    700.63776    982.71267    865.87734    498.52835   -256.44989   -259.59367   -212.00348   -296.25355     -422.267   -240.52627   -427.65227   -412.36044   -202.26283     245.5884    781.33526    979.67123    990.33849    550.90851    379.85316    343.53714    189.92166    -25.83008   -159.89969   -358.33447   -405.97734   -514.75842   -357.85716   -431.74924   -380.76326   -284.52325   -438.95845   -394.72903   -328.03581   -284.03757   -246.49446    -56.33078   -194.08053   -271.31317   -454.22217   -331.31418    -332.6792   -429.96674   -276.40032   -384.43502   -205.15488   -405.90114   -182.60973   -276.53371   -490.86588   -464.37847   -419.77692   -473.43966   -408.82816   -643.41009    -534.3559   -488.79058   -492.97957    -373.0864   -427.85541    -274.3203    -54.36928   -117.22483     -250.651   -205.24447   -348.05686   -484.81603   -167.40942    -69.91155    -84.88152     58.82464      39.8195    -97.35062   -173.98688   -305.16394   -309.93366   -491.21298   -404.90891   -466.51823   -309.08477    -95.27001    130.97934    362.13878      701.162    866.23785     411.8295    182.01433   -110.14851   -334.85716   -597.20477   -380.34075   -622.93012   -414.47273   -248.97226   -421.31618   -585.10367   -572.12961   -663.12657   -528.97606   -555.99394   -544.56154   -534.40803   -413.20996    -514.8057   -464.19766   -492.51716   -476.01767   -368.14146   -473.80372   -393.51552    -345.5701   -331.51524    -379.6081   -629.31525   -503.33081   -676.35719   -657.66147   -694.62025          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -7.00   -362.74742   -243.73543   -285.46398   -286.44949   -440.91645    -32.01053   -475.75871   -323.57551   -117.88752   -469.25099   -510.02654   -326.44561   -414.10991    -424.8562    -399.9689   -232.98745   -186.43022   -222.14947   -329.44831   -222.23071   -461.44028   -500.04943   -271.75392   -336.79001   -169.89891    273.93486   1159.93952   1624.50335   1360.14613    710.86314    -89.74777    -81.62539   -374.83823   -372.88838   -305.98514   -451.58249   -350.83889   -324.29886    -360.6186   -397.13635   -308.63363   -488.04913   -499.92183   -391.04232   -344.52074   -307.50926   -382.85526   -547.71756   -484.97386   -389.00698   -356.86355   -321.11427   -359.67332   -349.05998   -249.18366   -306.62095    252.09276    838.43879   1263.54897    926.72346    568.48117   -202.75013   -374.92886   -471.04048   -325.37408   -527.50273   -300.01817   -483.12878   -297.27747   -186.00198    269.06417    835.40923   1437.80304   1052.02419    580.57963    314.08242    507.44925    410.33521     99.15389   -173.01039   -323.95978   -373.68414   -332.95823    -411.5855   -528.39893   -409.98488   -425.50565   -433.42999   -394.23988   -281.21632   -202.89502     11.68954     23.51374   -271.49391   -338.37187   -324.41113   -368.88487    -356.6483   -480.46267   -415.07829   -329.20216    -73.90534    -80.34858   -238.95447   -246.04545   -329.92336   -382.78954   -385.94134   -548.62486   -576.44822    -586.3066    -572.0742   -402.07028   -420.62108   -325.90832   -200.85014   -110.46262   -101.51287     -2.54774   -143.02845    -176.6049   -296.12686   -239.43967   -231.54423   -161.38372    124.82622    -14.66491     45.99821   -155.77381   -336.77266    -304.3129   -346.22039   -505.25251   -477.56948   -284.05295   -198.02849   -204.05649    207.76135    581.77975    879.55646    961.86771    621.70096    281.18681    -34.65583   -312.07059   -469.71004   -494.56441   -361.93713    -690.0568    -470.3448   -362.57794   -556.37814   -506.52083   -615.38706   -505.56301   -540.17266    -529.9992   -493.85498   -448.60295   -426.93852   -511.55088   -452.05757   -400.58587   -407.92441   -500.78918   -405.57177     -353.938   -417.40999   -480.58185   -547.14573   -599.38869   -563.54119   -513.22857   -491.95803   -504.77061    -606.0191   -431.19509   -600.82009   -453.67228   -536.71661   -545.39748   -652.83165     -605.048   -617.06542   -583.40225   -578.92283   -747.46852   -637.71501   -702.22936   -773.13071   -620.89367          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -6.50   -413.67988   -239.42407   -270.88737   -193.42956   -464.61537    -82.69487   -477.87915    152.75282   -275.49156   -312.11871   -540.08243   -391.63106   -258.09278   -356.87248   -462.59834   -239.22167   -306.72571   -294.64595   -265.66247   -339.83514    -308.5311   -519.06636   -244.73067   -298.91963    -68.44585    402.86579   1233.13334   1697.97462   1492.24363    511.05847    -48.67808    -94.43475   -316.18662   -378.79003   -322.98508   -383.98725   -379.74388   -399.22133   -691.29043   -335.94368   -337.80164   -423.84526   -265.58147   -443.97978   -433.02866   -404.39055   -345.81999   -394.75674   -417.46381   -359.22942    -457.7037   -322.90317   -341.95785   -336.88053   -316.42068     -14.9587    115.25623    923.86181   1290.86619   1178.21388    548.42588   -224.95617   -235.84428   -244.38077   -336.80609    -296.7986   -394.43192   -320.28556   -388.48766   -190.72549    377.79374    737.40701   1045.50877   1006.54786     493.3934    454.51871    497.83184    329.91459    125.26482   -227.28889   -127.24074   -354.91914   -311.68289   -382.45526   -533.42041   -486.18852   -343.83409   -486.71827   -261.90851   -251.46021    -219.2208    -85.84903     68.44188   -216.05816   -243.39564   -280.11957   -429.43628   -345.78044   -327.02218   -223.28989   -324.26914   -365.89554   -194.56587   -152.34827   -147.40254   -227.64988   -402.87907   -342.17359   -402.52868   -509.68278    -363.2756   -530.40057   -527.20693   -425.14678   -383.67027   -336.88995    -112.8675    -57.29787     -7.82482   -179.25143   -207.09606   -444.41465   -469.72451   -274.65438   -108.80279      48.4365    236.54383    111.12745     -82.5822   -235.93808   -325.97973   -451.09859   -377.61964   -402.16981   -469.55719    -370.2156    -63.40587     270.2085    645.34178    866.86738     791.1543    697.14229    369.46081     98.11051   -296.18791   -370.79652   -382.34507   -528.82491   -525.65135   -485.26126   -446.29334   -551.46866   -543.57514   -553.31343    -606.6483   -508.65796   -597.72838   -448.07525   -419.86026   -380.76265   -369.44718   -400.65743   -483.15338   -303.12597   -402.29509    -346.7109    -320.7572   -366.71675   -341.91758   -312.46558   -375.19929   -494.98952   -583.01613   -593.89159   -490.96754   -364.32881   -565.30832    -495.0778    -601.7662    -451.6813   -471.50586   -516.22078   -618.08549   -541.85229   -540.38203   -584.08036   -632.55386   -640.99608   -623.51985   -684.15543   -621.71242   -508.11812   -564.09372   -563.51101   -478.90372   -502.51536   -362.98457   -464.05619    -527.3858   -460.61577   -462.32866   -448.63442   -545.12345   -624.86311   -632.90687   -395.94551   -581.09994   -582.54048   -602.52625   -600.36691   -521.67357          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -6.00   -468.69084   -247.28749    -274.5684   -398.78042   -470.67071   -134.10583   -402.12769     36.50483   -375.96832   -167.10016   -580.77503    -568.0557   -126.82478   -379.51454   -639.49671   -282.98531   -429.84826   -334.18932   -251.51303   -492.13558   -344.85997   -436.57468   -255.57949   -265.13915   -174.24268    418.65268   1221.73976   1883.60693   1489.69506    644.85481   -180.07947    -105.1776   -311.87194   -390.50331   -434.59099   -351.47886    -327.7193   -269.69519   -370.17249   -473.22569   -504.48034   -391.69444    -315.9642   -577.57239   -394.63514   -544.91783   -443.38766   -461.97437    -554.2633   -415.92859   -403.71055   -298.21466   -460.78264   -367.95544   -389.37155   -117.48614    148.25882    809.96451   1425.41343    1193.2898    656.65469    -35.80864   -296.33273   -231.07204    -380.5269   -306.32786   -336.15266   -206.43504   -326.65554   -182.54886     73.41124    694.31307     1042.478   1062.35176    613.20663     540.1348    506.21059    342.78966    165.54423    -90.49303   -210.47506   -369.52565   -400.84857   -432.53556   -456.47821   -387.10241   -485.58101   -413.46513   -347.76288   -305.52017   -250.87247   -187.72078    -32.71384     -180.151   -306.84146   -263.24508   -263.44944   -299.07147   -400.43592   -324.73355   -207.50929   -155.87827   -215.65419    -42.26935   -448.63977   -310.41288   -453.72435   -457.37641   -307.51455   -323.49799   -538.71334   -470.63834   -389.08367   -509.31309   -584.10664   -275.58875     76.30176    -34.75076    -70.35649   -233.96798   -164.16006   -171.94094   -538.96301   -348.66263      0.16111     227.5689    227.47309    104.68942      22.5083   -127.45402   -291.45978   -448.91682   -348.86458    -391.5964   -352.25204   -290.87321    -68.45174     328.0044    644.38951   1077.86015    747.34829    706.47761    335.72068    123.93966   -244.46249   -303.34652    -320.3985    -381.0268   -406.11764   -357.82047   -446.67052   -505.00971   -489.72362   -545.56024   -346.35698   -409.47534   -482.60737    -521.4868   -467.16503    -355.2225   -369.46922     -466.552   -418.41114     -461.473    -317.6923   -317.37322   -156.31336   -345.00436   -404.10943   -336.93961   -516.18749   -338.76761   -491.28606   -430.61398   -439.33834    -488.7802    -455.7503    -485.9674   -513.87008   -552.07092   -476.94491   -475.52971     -442.001   -665.84768   -440.12505   -592.57043    -557.7643   -609.99644   -792.22587   -584.63929   -558.49449   -419.47424   -375.30213   -524.13395   -551.31546   -395.66097   -280.45412   -331.98391   -291.11718   -384.88295   -400.18604    -390.0522   -460.00482   -389.35754   -542.55238   -504.58137   -423.45488    -379.2815   -338.74571   -421.33238   -560.45152   -524.57992   -499.32099   -561.00178    -560.0069   -597.50122   -642.16878    -642.7976   -757.35708   -586.39203   -670.48874     -720.672   -635.66689   -575.20668   -577.63875   -525.04234   -435.83246   -437.25685   -289.25684   -409.19094   -561.11954   -450.99044    -463.8261   -193.61578    -547.6466   -473.62085   -478.31784          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -5.50   -479.16314   -230.62303    -278.5515   -703.08295   -409.41506   -265.41019   -309.63125   -117.19689   -399.68112   -358.86589    -507.5113   -661.31775     -9.01082   -583.03268    -513.9163   -319.58414   -534.71759   -329.33666   -182.52901   -419.81646   -421.17067   -321.32765   -309.30637   -479.55823   -228.80375    309.49542   1140.65298   1978.07885    1533.2394    755.20389   -147.10715   -185.79671   -494.34488   -218.16577   -611.22778   -398.09986   -415.13741   -401.85153   -614.84104   -428.77722   -476.42291   -445.85705   -493.45365   -420.00996   -319.66543   -448.88572   -405.98253   -393.77764   -318.80682   -379.89038   -304.26058   -382.68207   -347.37723   -299.43452   -328.39571   -304.07379    111.90661    848.78346   1429.90131   1433.82434    516.15083    -51.01596   -304.58618   -339.02827   -311.39912   -355.99428   -459.99604   -347.43243   -232.64257   -232.16297    209.51553    758.10231   1168.02481   1151.66459    830.43188    599.30626    555.95323    448.96798     269.9024   -158.60964   -212.51323   -215.82674   -392.58878   -375.64562   -445.08447    -429.0504   -424.16541   -338.59583   -447.53382   -208.39558   -149.28807    115.86169    -72.50207   -217.80197   -225.26748    -336.5989   -402.23747   -267.12985   -280.46182   -323.89608   -293.18988    -87.94081    -32.86062   -146.81208   -130.62969   -247.83073   -352.29384   -321.41776   -399.32656   -600.67038   -448.56147   -433.92188   -446.53874   -429.90833   -178.17162   -112.17523     -70.0873     -62.7217     22.42819     -3.15626   -127.36173   -313.70611   -253.53861    -260.7624   -172.53908     -4.04619    226.43758     85.81509     24.54689     -66.5021   -231.98602    -362.9623   -447.47542   -341.01907   -481.56675   -336.29714    -65.17619    403.47265    926.14815   1014.01452   1079.82289    667.82763    522.35339     89.11189   -410.80773   -109.71304   -427.90427   -234.53292   -382.02186   -451.11996   -380.43912   -411.20587   -734.12061   -633.31691   -523.22781    -621.9091    -453.8417   -305.76417    -388.4145   -532.13561    -396.5001   -481.54665   -418.13965   -365.65667   -426.14183   -164.51121   -393.26495   -325.34535   -255.80432   -438.38328   -580.77703    -499.3702   -462.49145   -519.07943    -561.2745   -501.59146   -402.34092     -353.517   -426.55263    -403.1792   -476.25617   -498.57483   -367.46496    -389.2061   -614.76842   -504.05479   -602.54865   -520.09572   -474.20762   -438.00942    -364.2945   -505.51209   -455.19674   -584.07196   -470.32124   -285.19164   -343.17817   -407.01123   -266.08909   -260.04008   -342.57947   -294.56584   -285.01806   -399.24813   -405.89132   -479.06386   -430.00179   -366.74639    -256.5353     -327.195   -254.98864   -231.66472   -223.59917   -548.13824   -563.56689   -558.24217   -592.31702   -570.56432   -554.82268    -716.9911   -500.49527    -532.9392   -575.31632   -468.75812   -355.06239   -376.88015   -516.64661   -351.76783   -166.73938   -407.07894   -383.10867   -151.28811   -406.18311   -389.55767   -487.50543   -227.35425   -253.95367   -420.16591   -379.08227   -473.03915    -356.1484   -401.01518    -420.2402   -562.41929   -362.60085   -620.62433   -528.16223   -608.27592   -672.44424   -747.21014   -654.97487   -750.50956    -556.8292   -373.23624   -434.27063   -546.61737   -633.25802   -451.29432   -464.42004   -539.59879   -515.91017   -351.27789   -279.51728    -73.40951   -339.75651    -205.1963   -301.68724   -324.70815   -526.37405   -426.50659   -437.93998    -481.4284          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -5.00   -469.47056    -97.49759   -398.18924   -856.57324   -349.37422   -400.22129   -190.37808   -216.48109   -414.26362   -516.52038   -354.73849   -470.02347    -263.5648   -521.09309   -375.00506   -348.14185   -544.79381    -345.1107   -131.17653   -288.47386   -489.11503   -223.70036   -229.76951   -525.82113   -278.47036    153.30628   1014.78605   2013.59375   1569.28968    715.01535   -135.08524   -264.88851   -332.83025   -100.30799   -316.77095   -362.99192   -544.17105   -422.02427   -252.13603   -379.58513   -372.65497   -259.77567   -202.15308   -358.63052   -307.33873   -467.42841   -476.14464   -293.17763   -468.37079   -373.51367    -194.5072    -350.6688   -283.02741   -340.08676   -274.40258    -76.07539    116.17469    802.93452   1479.12794   1311.48343    598.58263    -102.2309   -242.80567   -302.41029   -325.48055   -245.99516   -289.46335    -282.8031   -388.37935    -55.87879    138.24032    863.25501   1017.34045   1163.07482    745.70915     681.6974    598.47692    365.66809     72.52284   -279.15342   -365.05047   -259.29906   -322.78084   -387.89393   -471.22949   -465.04383   -273.78642   -354.96297   -259.31916   -305.89179    -126.5916      23.8326    106.55135     -5.95596   -258.34008   -343.63746   -482.58847   -455.52573   -396.26414   -238.29487   -192.58591   -174.14655    -99.28081    -285.1378   -338.60447   -384.33926   -447.61827   -527.79492   -415.84824   -475.79026   -579.46297   -380.99248   -440.76146   -477.75852   -299.43478   -289.52799    -86.42307    253.55714    -70.79455     18.29722   -164.81141   -210.26193   -401.10545   -310.76323   -158.31114    174.86919    254.26945    263.72622     13.19216     -20.2962   -227.62269   -381.70995   -425.74883   -449.84902   -467.50988   -409.53209    -85.19327    255.72817    885.99908   1066.34595   1316.04177    943.31118     482.6573     54.14663     46.99282   -376.79815   -431.93855   -343.07882   -313.40149   -422.35287   -357.86642   -409.03748    -493.6211   -469.36055   -552.24096   -457.40749   -173.15441   -406.80308   -313.02723   -315.53472   -273.70034   -412.47715   -324.53174    -371.2668   -345.45406    -343.3489   -219.16599   -260.98156   -245.25208   -401.65276   -454.29524   -410.71104   -499.27918   -564.58143   -472.51959   -436.41138    -462.9618   -369.01828   -488.07862   -511.04438   -363.94975   -526.93189   -409.54777   -498.50613   -455.68628   -560.95226   -354.79086   -389.69315   -582.30679    -483.9971   -455.37752   -464.86877   -351.09098   -388.48465   -575.68075    -372.2393   -195.25126   -284.45652   -254.66705   -232.33366   -174.86589   -187.18389   -381.47116    -357.9348   -425.15734   -253.12111   -468.86345   -357.71876   -110.32916   -334.79582   -262.78829   -327.87857   -248.73248   -379.05567    -338.7668     -457.109   -437.24554   -471.09429   -516.76564    -493.9985   -439.97456   -546.58158   -540.59649   -420.29123   -365.76596   -377.32017   -305.01208   -261.03397   -205.35094     -38.7382   -251.83655   -321.86749   -257.50785   -213.39703   -363.22894   -264.31703   -307.06611   -341.07397   -385.70125   -293.44012   -207.70294   -246.74517   -472.00809   -443.25255   -513.21139   -598.45779   -545.08861   -624.47863   -671.07651   -584.43817    -570.5562   -551.81752   -414.20681   -439.54184   -569.33386   -594.51459   -240.04686   -418.44398   -386.93071   -425.98085   -291.84443   -277.50613    -143.0883    -170.3614    -74.89818   -254.20876   -175.95674   -126.11785    -45.92092   -310.76892   -401.71219   -511.57837   -453.13207   -450.08302   -592.13652   -639.68831   -671.10885   -522.27733   -500.36157   -363.48557   -426.66239   -443.18454   -412.49454   -634.79528   -507.67142    -569.3871   -839.72251   -608.19742   -657.74734    -785.6246   -789.84191   -774.79179   -732.92895   -689.42391   -678.15242   -807.96101   -808.37753   -756.60607    -822.5744   -737.90622   -805.95763   -819.90572    -803.3836    -743.4969   -671.12847   -737.30704   -707.86699   -766.45656   -680.79808   -654.28426   -623.42523   -673.21358   -660.83879   -739.67992   -640.43188    -593.7056     -768.452   -711.27253   -684.22618   -666.27093   -778.09719    -740.9239   -842.67992   -764.95291   -634.67648   -801.40665   -843.89736   -773.53823          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -4.50   -453.42317     30.85109   -556.61158   -605.29241   -376.78223   -447.08727    -90.26081   -315.99037   -453.19801   -390.77649   -280.15927   -270.58774   -476.09001   -300.89294    -235.7212   -214.62701   -532.12659   -353.06449   -167.62293   -135.69767   -443.04236   -267.98947   -284.48809   -437.61705   -268.81619    132.38145    985.26165    1835.4743   1600.80951    551.93394   -259.66136   -327.17933   -253.45034   -408.83957   -291.97674   -291.22331   -256.32157   -286.02694   -422.31473   -267.20152   -103.04551   -365.70612   -262.47491   -320.28649   -124.87114   -471.49624   -387.91926    -460.2682   -459.21278    -462.4425   -269.53256   -467.06147   -428.33576   -319.80802   -354.00414    -290.6089    320.80932     1018.676   1392.93003   1135.49256     548.0698    -10.78897   -223.12655   -342.22345   -446.18172   -210.74884    -429.1794   -342.28546   -317.46874    -68.45686    189.59069    929.60306   1090.71823   1361.82904    755.89679    514.24762    346.35348    489.10424     86.11621   -133.39798   -241.76045    -341.2845   -492.92368   -438.66815   -372.99251   -388.86711   -432.49314   -315.43369   -288.52779   -232.71287   -321.44309     -9.38456      80.1313   -290.52672     -257.316    -359.3118   -314.94561   -445.01942   -571.92559   -223.49079   -267.95378    -114.6349    -40.68292   -171.34302   -170.85185   -330.36387   -335.44311   -348.56884   -405.78577   -389.01761   -368.56907   -457.14911   -601.47444   -482.80581   -341.00789   -175.20818    -50.04344     45.48089     85.11419     202.6986   -105.95183   -208.05071   -297.77584   -152.97712    -64.86063     152.6742    273.65334    289.82118     99.01389    -15.97988   -281.51427   -348.11121    -347.6221    -414.7754   -441.68615   -231.71849    108.68548    406.33907    819.30741   1331.64125   1266.18296    1107.2251     402.4545     61.22027    -316.5457   -403.93366   -406.60362   -419.62801   -485.54439   -380.44774   -314.49768   -443.91552   -422.49865   -444.12371   -398.26737   -523.39205   -395.22047   -288.28434   -307.22697    -272.7083   -362.18821   -240.79058   -331.20108   -414.27992    -195.4397   -146.73114   -197.37348   -146.47574   -240.88098   -327.46019   -378.80885   -303.20579   -381.61732   -363.14889   -346.73235   -379.83088   -385.96072   -394.48989   -202.18601   -459.40948   -334.62872   -507.18501   -423.44388   -532.51525    -473.2882   -621.34216   -470.95868   -498.22642   -437.59403   -445.49976   -377.09754   -302.07262   -285.90802   -327.36275   -203.80871   -160.85534   -232.18525    -52.78527      75.1033    -44.19733    -84.36662   -165.53782   -180.25453    -278.6362   -466.42265   -271.10354   -260.68637   -207.72841   -281.10752   -167.98548    -54.95072   -176.52797   -283.18327   -157.84854   -283.70737   -365.13689   -512.69965   -502.88331   -513.47094   -502.35775   -518.16849   -420.55917   -457.93083   -503.95119   -487.32472   -315.78072   -236.45703   -266.21898   -193.43105   -276.61971   -385.86547   -149.97146   -231.78087   -277.73552   -174.21588   -442.82527    -301.7729    -239.5692   -379.55446   -310.51622   -141.06395   -323.20591   -472.45473   -420.01806   -505.46807   -606.54542   -545.40171   -514.31871   -538.82706    -548.0145   -591.55602   -676.93873   -503.64584   -504.58542    -346.2055   -340.42961   -309.52441   -314.36712   -338.77775   -275.12452   -257.62834   -283.61621    -76.11483    -54.35245    -30.36773   -185.07813    -60.10906    -68.10109   -248.47377   -333.88384   -344.89933     -407.809   -400.79591   -436.41998   -464.97663   -577.42182   -638.84401   -580.30419   -485.59707   -452.39111   -347.84741    -320.0645   -440.70734   -497.98221   -474.76788   -614.69927   -612.05999   -512.81914   -647.03572     -672.373   -592.68971   -668.51533   -801.04716   -691.68514   -550.29574   -663.04633   -793.01221   -572.54873   -706.38149   -701.49612   -557.60466   -645.34235   -705.21393   -596.53675   -635.68625   -584.10497   -723.26535   -695.30468    -678.9608   -563.10442    -591.5334   -530.03375   -500.90805   -642.04257    -464.8019   -537.43391   -448.91489   -498.04873   -525.68355    -663.2519   -582.40885   -468.31375   -471.46067   -703.78144   -545.17789   -520.97244   -630.89831   -600.15316   -724.62886   -567.11943   -538.74848   -587.50579   -731.17076     -689.686   -503.06722   -681.61747   -438.58421   -448.79824   -647.20779   -598.97519   -493.51974   -533.09114   -595.18343   -507.00673   -669.42227   -830.29101    -737.3706   -756.64801   -641.62141   -584.24151   -620.36867   -614.90131   -385.06492   -475.80642    -509.2297   -355.22719   -478.76552   -672.66215   -569.66775   -307.60777   -420.29056   -338.92255   -411.39333   -403.43114   -339.50695   -538.74769   -465.01548   -457.47112   -347.07161   -771.02179   -584.58196   -314.04008   -735.36799   -549.87811   -728.41732    -605.8998   -662.69006    -657.3621    -750.0293   -645.92092   -778.32019   -910.50501   -633.33135   -415.02079   -936.48644   -897.50194   -759.85167   -551.96053   -875.63193   -770.06743   -821.55941  -1074.11084   -877.92484   -823.66017   -549.95979   -858.18412   -937.86118   -806.20161   -775.33054    -866.8689  -1216.48407  -1151.30663  -1048.01844   -920.02413   -821.92912   -813.41055   -839.46593   -764.83047   -979.45229    -849.2351   -894.04695   -997.23009   -729.40832   -768.83639   -820.58326   -523.48015   -847.21528   -876.07229  -1018.65689   -716.95237   -944.20507  -1036.45713   -936.67408   -722.85063  -1076.23015   -769.89379   -822.28721  -1056.00584   -859.56506   -857.70489  -1462.50443  -1163.63765  -1099.03442  -1137.20874  -1163.88848  -1479.54726  -1164.37349  -1091.90141  -1278.40038  -1158.64553   -1400.7592  -1114.89848  -1113.00271   -1452.4651  -1369.50447  -1366.29717  -1135.99116  -1093.46771   -1083.0822  -1410.86505  -1234.41007  -1532.71965  -1398.65084   -1127.6531  -1485.75221  -1477.20674  -1035.29995  -1512.41266   -1140.7597  -2080.84686  -1375.77038   -1472.4285  -1479.82917   -1445.2993  -1382.89164  -1226.37172  -1579.68551  -1228.11101  -1486.96356  -1446.79692  -1589.49009  -1720.00443  -1455.33174  -1468.53197  -1778.44467  -1431.68695  -1235.46956  -1811.10754  -2172.03607  -1584.30746  -2157.28143  -1850.17409   -1666.6394  -1446.07298  -1166.48368  -1286.07841  -1262.27461  -1423.91854  -1221.39911  -1549.01756  -1436.78553          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -4.00   -504.84036     30.72608   -675.92953   -369.41707   -422.53548   -393.35411    -183.7753    -247.4415    -425.6516   -309.25011   -222.19034    -84.44807   -407.26994   -246.10444    -158.2725   -212.29501   -538.00981   -366.59657   -357.68134   -313.67009   -219.74585   -312.30254   -326.31416   -270.22646   -177.45493    279.29074    1238.5888   1676.23759   1616.00511      395.562   -177.75522   -333.19407   -309.29631   -539.09534   -428.02468   -180.14235   -163.11917   -447.06305   -412.27708   -439.35366   -454.91565   -363.07362   -373.88738   -517.27295   -307.03213   -378.71499   -422.99317   -575.62403   -336.74811   -474.95331    -206.4462    -89.74236   -488.76225   -184.74294   -416.18451    -73.98197    221.68359   1142.64642   1713.59423   1317.67485    545.74387    113.26825   -272.82016   -284.94171   -513.91863    -451.9231    -282.1921    -48.39925   -288.44596   -115.68045    296.46527    925.38358    1399.7892   1130.50417    706.80727    438.60009    577.52226    302.14084    107.24267    -91.52032   -284.39387   -228.16619     -367.111   -259.44389   -392.00677   -390.18657   -371.38613    -317.0941   -379.64359   -338.00203   -151.51245   -218.87696    -12.86817   -240.96538   -282.40023   -331.27851   -399.96197   -185.13927    -213.8864   -488.90931   -372.25394     -59.9395   -147.22893   -148.34892   -144.12435   -266.70446   -376.20661   -377.62597   -435.94207   -359.24495     -488.158   -412.08831    -356.6617   -286.97311   -254.92854   -318.11731    -63.15209     74.58237      55.2568     29.24183   -165.01945   -253.22314   -198.93794    -30.64262     16.87986    300.30843    369.13862    192.65322    231.17965    135.54281   -280.16632   -181.90171   -257.82848   -317.30206   -302.19841   -316.15405    -70.53382    521.04232    922.84088   1030.71221   1253.31696   1081.98573    515.73918    176.19117   -161.34332   -363.06468   -478.00171   -341.20049   -306.93762   -369.65297   -417.00905      -513.62   -508.22982    -526.3207   -420.09232   -474.88163   -388.79088   -301.49992   -376.78748   -324.22725   -272.14036   -344.62691   -339.02509   -244.94895   -215.88874   -289.05245    -148.4843    -31.52176   -179.12359   -139.61489   -300.25915   -291.11592   -455.92084    -306.5203   -509.71692   -321.51692   -207.92285   -328.20783   -425.87409   -334.43534    -259.8873   -423.50553   -372.15724   -461.94052   -530.07093     -447.008   -431.15166   -425.10379   -395.25944   -425.87686    -359.7622   -319.80058   -334.79806   -277.01134   -254.58057    -20.67149    -123.1133    -47.94401     82.23399     -8.31148    143.87564   -126.01212   -162.30096   -125.69546   -170.16273   -193.07017   -296.27045   -217.85936   -112.88643     48.59955     -8.23724     25.07753    -94.77038   -412.09922   -373.37614   -459.22609   -549.57796   -422.71903   -429.71117   -488.54803   -449.51293   -362.62476   -345.77565   -333.82434   -395.61331   -403.83655   -220.27681   -195.54341   -182.03943   -191.34903   -182.27891   -169.31076   -248.49379   -189.39677   -280.84654   -398.54711    -124.3662   -156.93184   -212.47426   -192.48602   -258.81799   -265.10004   -263.69008    -94.05974   -362.24171   -612.97481    -530.7909   -593.26075   -517.80761   -582.05068   -417.23334   -537.89705   -401.85311   -482.40332   -342.66554   -373.86279   -307.18902   -297.79286   -275.00317    -100.8397   -108.14723    -87.24907    -22.60888     -7.61069     88.86966    127.93443     83.20351     38.80922    -39.17896    -83.99645   -431.29893   -514.20569    -456.2935    -414.0096   -461.03847   -525.95058   -504.61669   -460.22943   -482.95444   -564.45204   -470.66101   -433.23618   -380.99338   -294.60153   -422.03062   -483.92104     -456.592   -471.24618   -526.60514   -552.51718   -586.03928   -571.26604   -614.70957    -538.4244   -566.13261   -564.97532   -670.88084   -520.06808   -654.28225   -684.31708    -641.1168   -736.57492   -526.83443   -685.94431   -680.26453   -503.15688   -653.40714   -438.93167   -486.51137   -564.26109   -699.35516   -484.70938   -702.05539   -346.85231   -515.03182   -526.87132   -459.38528   -443.92359   -375.16332   -585.32249   -594.16654   -562.23184   -692.06741   -694.26497   -621.14514   -627.04118   -433.66815   -519.48786   -538.18512   -477.62745   -749.11187   -516.18268   -567.44594   -629.76568   -484.87236   -452.48821   -314.06538   -470.48554   -453.39275   -505.61287   -755.99519   -503.74488   -709.87888   -504.89076   -433.01594   -695.00413   -557.62594   -511.38606   -512.63632   -816.73342   -637.59932   -506.18556   -471.68085   -433.55258   -504.24198   -327.90449   -395.06153   -260.77507   -347.39981   -211.57988   -255.12186     42.55804   -208.95682   -563.16657   -117.99371   -135.82218   -146.90139   -377.00785   -418.02646   -590.17609   -365.09814    -526.0012   -740.74022   -446.52165     -504.929   -659.13637   -478.76322   -333.61285   -710.19529   -334.61946   -481.13903   -604.89239   -748.63103   -773.45449   -644.15718   -566.79322    -448.4499   -390.71602   -765.94649   -620.95642   -562.15302   -114.48201   -656.98065    -852.5002    -713.0353   -803.37615   -724.01349   -562.10119   -919.37667    -921.8574   -815.46706   -885.60574   -921.42247   -802.76091    -382.2132   -746.91875  -1116.35895    -852.0743   -629.27459   -814.52486   -613.65159   -682.18954   -750.18752   -944.62561   -600.26601   -781.60477   -645.71456   -663.76851   -738.82858  -1036.94542   -622.73326   -812.19633   -697.70525    -687.5698  -1021.19168  -1204.17742   -745.98205  -1010.79001   -879.89102    -846.1223   -778.22656   -1052.6596   -933.73247   -1034.8492  -1000.02095  -1160.02782   -770.16049  -1077.71102  -1140.31504   -913.64848  -1230.77697   -1144.0046  -1149.80962  -1175.04878  -1354.21994  -1158.63509  -1202.69067  -1240.97706  -1205.32847   -1318.6393   -1194.0833  -1429.28907   -1280.5178  -1047.50902  -1283.47484   -1374.8905  -1512.56471  -1203.00506  -1321.95891  -1041.11484  -1659.58536  -1526.89839   -1024.2462  -1443.87476  -1374.11996  -1497.74919  -1450.09896  -1613.79313  -1444.35248  -1425.54774  -2016.14472  -1636.07757  -1538.00348  -1587.81002   -1771.8734  -1227.40808  -1682.39014  -1599.16168  -1721.24296  -1374.26633  -1614.74575   -1796.1709   -1116.5725  -1465.89769  -1652.90624  -1568.84478   -1085.4151  -1070.33646  -2070.48705  -1261.69075   -1134.5549          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -3.50   -558.27829    -81.08185    -565.5065   -304.81377   -463.73709   -333.94633   -265.60494   -101.68413   -292.25269   -487.66596     -88.9744   -113.10097   -361.52027    -218.8043   -116.53148   -394.31771   -486.68331   -341.78088   -343.13769   -260.88125   -188.46241   -291.76448   -155.72166   -179.51411     80.29481    198.43163   1358.87409   1805.05752   1581.31863     425.2433      1.97972   -236.60611   -267.27889   -424.34017   -400.02202   -229.24577   -323.64212   -283.37365   -404.31205   -578.48213   -459.45692   -228.08487   -358.80238    -376.7749   -239.71291   -645.19707   -434.43364   -245.48779   -246.92624   -316.57985   -298.77328   -303.65945   -277.62471   -320.65097   -177.19321   -436.48508    137.53479    895.68357   1809.98108   1536.36626    643.23589     32.37181   -223.50494   -312.70894   -346.58284   -528.94496   -627.67146    -96.76282    -430.1182    -294.1064    332.48468    845.81673   1387.32271   1313.00232    807.19277    514.83372    639.29634    269.66301    126.98666   -231.07421   -403.23447   -524.04334   -295.15307   -375.62248   -440.55051   -349.84802   -315.45465    -256.7009   -317.12416   -301.96186    -28.23875      0.84057    -74.20125    -39.81086   -217.15797   -203.82264   -504.87945   -476.66322   -260.79455   -433.24815   -216.44588   -406.74925     15.48566    -49.74281   -261.00092   -166.02938   -441.08552   -558.11798   -367.59687   -341.79338   -440.23835   -342.92148   -319.14325   -218.82297   -272.91138   -347.23451      6.22549    238.57411     70.29123      24.4923   -138.47138   -191.94077   -357.91293    -91.48522    125.71532    132.56476    541.48237    467.09696      69.0198   -190.86706   -301.35279    -167.0341   -364.91982   -249.52759   -192.31308   -163.17884     55.58673    467.12606   1033.36145   1319.99763   1585.77566   1132.32822    774.13564    330.32056     31.61658   -165.64672   -232.08148   -453.73103   -329.29957   -412.91495   -380.53795   -413.82653   -516.56478     -399.546   -470.80297   -397.69685   -369.28887   -377.74289   -380.50026   -207.83579   -268.95563    -383.4781   -322.93904   -298.72533     -2.37082   -184.40747    -83.22858   -192.56469   -149.25093   -282.81115   -326.65136    -410.3206   -298.27219   -227.60314   -311.53926   -255.72078   -255.99353   -473.96457   -306.39491   -365.58748   -291.08153   -278.67934   -397.96871   -577.65004   -476.81251   -328.90895   -345.40996   -552.11542   -521.83086   -450.89922    -457.5131   -286.18641    -182.3261   -129.04607    -78.96497   -104.31457   -142.14932    -59.21055      4.27543     51.07025    214.29058    129.77328    -184.3121   -113.43833    -123.1259   -180.92592   -128.40162   -152.91089    -72.20435    152.84957    135.38406     41.11553   -159.99213   -303.26846   -417.89051   -233.41394   -451.22012   -423.91449   -455.71433   -422.22797   -371.17435   -473.58383     -470.634   -446.96476   -370.66221   -238.60627     -114.658   -146.16242   -128.46767   -171.58844   -150.55368    -85.87046   -194.38482   -180.36355   -187.29863   -189.93821   -166.15929   -106.46072   -150.16978   -156.34883    -146.3021   -288.12962   -179.53921   -196.41293   -412.91476   -440.38969   -538.92511   -626.41673    -681.0506   -429.27709   -529.69895   -591.80229   -313.66381   -353.58883     -384.559   -309.66228   -264.34967   -159.82749   -187.49018    -32.05666     -95.2073    -112.3373    107.06094     -29.4055    315.63764    120.10337     191.1144     115.9804     59.96734    -26.46612   -156.67679   -270.16122   -371.99982     22.65547   -371.24092   -460.35513   -395.36639   -455.86801   -269.04126   -311.89406   -296.08749   -355.25047   -430.79109   -487.84567   -386.31315   -235.78893   -380.01266   -589.07175   -380.85334   -490.48344   -468.49767   -450.54599   -467.44149   -572.18175   -714.91253    -607.9698   -501.76549   -614.09955   -623.01161   -523.36824   -557.13109   -470.78316   -557.44536   -633.70971   -656.72174   -526.18406   -598.02664   -411.21204   -407.48997   -487.52959   -377.29337   -342.10284   -410.18212    -500.5711   -374.05258   -416.58294   -322.42105   -243.03093   -452.52185   -466.52611   -393.15074   -431.42067   -517.24188   -496.99855   -551.22205   -421.90351   -706.50531   -452.20959   -428.57019   -506.38975   -240.00202   -506.31955   -385.22509   -494.35322   -331.87568   -337.88636   -499.58053   -205.55078   -327.56492   -432.76834   -415.26713   -477.09593   -485.44949   -584.88709   -477.73402    -570.3311   -657.66593   -416.42926   -586.40359   -504.24584   -352.22637   -318.13056   -423.83133   -363.76058   -306.90596   -380.49256   -165.24932    -99.13731   -138.57969     -84.7535   -235.97554     34.77652     51.31922   -315.56876   -303.24076   -440.73289    -310.8613   -442.41697   -341.48388   -418.36029   -525.82281   -226.26954   -168.61553   -381.14258   -296.00367    -356.9649   -357.03084   -425.50009   -326.60143   -245.44045   -477.43976   -344.81669   -378.22975   -173.19291   -636.99616   -348.22224   -447.75561   -450.57413   -392.56352   -713.47851    -454.4616   -628.14865   -602.79349   -742.37769   -505.45354   -399.89416   -453.73099   -758.30832   -375.21463   -318.75263    -529.6658   -729.80473   -536.50059   -649.98295   -716.45218   -624.34161   -911.11622   -470.66089   -799.97139   -725.29056   -953.66422   -806.78822   -898.75759   -468.75151   -648.08251   -803.55492    -649.1793   -454.08428   -631.11809   -515.30764   -604.68821    -848.7111   -612.22346   -766.69718   -982.20779   -741.30516   -671.75408   -983.96745  -1188.40187   -975.28278  -1185.67949   -993.09933  -1010.54068  -1059.48028  -1036.73203    -961.8629  -1346.99095  -1049.91625  -1090.32362  -1308.07876  -1260.27673  -1252.07062   -944.52134  -1325.26205  -1253.96046   -954.72679   -1157.2689  -1076.07773   -1129.1306  -1147.51118  -1393.69247  -1309.56155  -1373.20063  -1580.19082  -1220.72502  -1447.14641  -1499.37994  -1390.14842  -1524.13595  -1104.69147  -1396.91576  -1499.25867  -1526.64913  -1334.76208  -1603.58543  -1220.00999  -1338.60717  -1101.12544   -1316.1325  -1437.62134  -1408.39506  -1299.65308  -1669.30886  -1079.74307   -1286.4875  -1241.95193   -975.31143  -1750.62015  -1278.56892  -1305.12204  -1552.11424  -1308.12562  -1081.00816  -1347.18956  -1396.04128  -1446.93455  -1374.49636  -2176.96279  -1514.09468   -546.43312  -1743.66811          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -3.00   -599.67633   -176.86133   -465.76979   -296.10099   -506.87652    -233.5139   -284.99646   -131.60436    -246.3527   -607.46396     45.64676   -263.28841   -352.39243   -276.75451   -201.43634   -405.98711   -394.74521   -302.37215   -288.41268    -65.51851   -550.83211   -128.64999   -326.45132    -363.6921    -42.32477      249.673   1343.84894   2024.92512   1409.43898    617.51865     -111.804     -0.03697    -430.4281    -317.8317   -367.53178   -390.08812   -347.46466   -341.17129    -421.3909   -632.09034   -405.75131   -340.73332   -464.93526    -376.8877   -295.04155   -444.65702    -328.1979   -266.68535    -316.5769   -201.88565   -446.06994    -353.1914    -267.5695   -267.15002   -261.18788   -110.73688    341.38691   1113.74246   1779.77714   1377.03789    653.68527      16.0179   -298.53274    -276.1097   -277.01901   -267.05022   -363.59214   -450.55234   -191.45356    -36.01132    480.82272    687.12865   1486.62934   1260.46694    917.05193    770.19601    551.16572    210.41798    -53.13195   -304.11395   -292.98194   -404.12252   -296.48462   -357.48398   -368.87757      -72.803   -475.83076   -456.59167   -226.30166   -316.09662    -96.66317   -174.25919    -118.2774    -76.46708   -149.34526   -185.88928   -472.13894   -182.53627   -357.99342   -332.46666    -81.94232    -89.45918    -91.20666   -232.08518   -163.14461   -327.56736   -290.60535   -444.17995   -212.20128   -432.88315   -604.55958   -366.20158    -450.9724   -268.63513   -184.98913   -104.54861     72.64198    149.98924     73.55501     70.35462    -22.12134   -223.66123   -130.51103   -221.92094    106.24582    334.24267    385.93654    257.43929    340.21008    -14.23973    -110.5659    -296.7998   -316.48106   -338.99498   -362.13955   -346.55977    188.60712    524.21774   1222.38733   1562.15094   1689.35082   1144.91309    902.62325    484.47414    252.67182   -183.33591   -225.47515   -167.91495   -405.06414   -351.77913   -312.97371   -228.46172   -365.20143    -514.5088   -393.02603   -331.83626   -375.43837    -349.8862   -210.68598   -141.33548   -238.88089   -419.68328   -169.09369   -315.94781   -408.10187   -214.16971     70.43843     41.98339    122.16914   -159.47407   -347.84754   -432.78137   -327.36201   -381.52553     -373.808   -217.94493   -292.03349   -273.60879   -248.08461    -317.2621   -335.15256   -256.09932   -525.47417   -354.17266   -580.45512   -355.35478   -470.91171   -404.87969    -437.0994   -302.89825    -306.4526   -207.48471    -80.98328   -262.27475   -197.21501   -104.11766    -64.04522    -67.62086    172.92693     -4.56734    175.05696     38.07391     86.87954     68.63085    -50.27863    -28.73464     -67.4064    -35.48829     33.05913    158.76844    -89.53414   -221.48484   -263.81643   -240.62721   -277.88343   -288.61966   -568.48249   -322.06474   -466.60778   -487.82054   -289.56557   -271.42372   -473.37302   -293.64126   -367.01878   -257.30685    -85.01192    -92.12189     20.51278     -65.5328   -388.51747    104.05578   -201.50377   -182.55438    -160.6419   -204.73352   -135.33634    -86.22638     25.64866   -244.22218    -77.35887   -311.05517   -200.32617   -278.98059   -298.31798    -512.0364   -396.95967   -309.52792   -692.52512   -434.51656    -391.7904   -430.02157   -309.77666   -467.20213   -251.76929   -274.76736   -161.99315    -188.7704    -82.86695     66.02312   -135.39529     55.38618     76.63005    148.65382     -70.7025    341.74442    140.24038    212.94051     -33.9273    126.70609    -20.42078    -13.90931   -296.29758   -316.04905   -364.05809    -373.7969   -397.84584   -218.71575   -467.47192   -446.71449   -395.36039   -381.88676   -249.43499   -350.21304   -364.25696   -371.01162   -292.23141   -539.77417   -552.45034   -557.49772   -360.70275    -531.9573   -597.46108   -709.87773   -622.36545   -649.84101   -745.31842   -712.21474   -518.50228   -722.48929   -484.66155    -341.6519   -464.08601   -630.74291   -579.38724    -433.4082   -435.77183   -441.16181   -436.31263   -432.12026   -286.76414   -225.36679   -507.78414   -532.19189   -365.34706   -302.13484   -259.70382   -332.99211   -505.09646   -298.35485    -700.5334   -384.38509   -423.83598   -247.09381   -617.15918   -441.46574   -456.27399   -223.04789   -451.18093   -282.94582   -289.06823   -598.00514    -261.3078   -229.77904   -233.66371   -405.18239   -431.18266   -438.79497    -305.0306   -585.01169    -362.8788   -487.39182   -531.94383   -503.94616   -463.37172   -376.56318   -441.86581   -375.93145     -411.614   -605.24497   -200.35978   -207.95237   -377.75834   -286.59426   -265.38701   -387.93861   -296.55498   -300.98172   -151.47668   -333.84333   -136.15283   -313.35215    -84.93906    -35.19992    -63.05888   -333.22545   -130.22743   -345.98077   -311.50927   -172.89616      8.17845   -211.57718   -361.97963   -422.01753   -273.95306      -273.91   -249.08655   -484.40409   -316.23493   -371.58725   -390.91796   -272.25613   -453.82628    -294.1896   -349.66086   -415.49819   -483.31193   -364.13345   -524.55174   -275.16986   -646.59121   -506.54599   -606.30201   -746.30313   -473.86392   -467.23483    -509.6983   -528.92801    -610.9747   -475.53868   -563.56109   -230.66477   -399.06799   -567.88803   -305.45636   -444.19773   -669.63248   -378.68294    -517.3154   -684.58757   -709.94053   -421.90308    -277.2286   -654.22254   -334.87155   -611.77522   -486.86088   -609.03816   -485.27314   -647.91405   -636.24203   -600.05284   -581.30498   -622.05874   -839.84341   -672.12245   -738.97616   -477.25232   -847.00351    -590.9212   -856.84821   -956.30722  -1053.91952   -961.94694   -943.59472  -1023.62607   -956.07065  -1247.24862   -984.17523  -1392.90757  -1459.74858  -1040.17516  -1166.79808   -947.55078     -674.882  -1176.24624  -1210.24307  -1008.08988  -1798.31681  -1303.01524  -1225.93762  -1371.25734  -1523.68844  -1572.86856     -904.942  -1287.43513  -1089.89999   -1205.1941  -1204.07973   -967.13732  -1164.86597  -1115.88474   -939.18629  -1210.66788   -797.21547  -1066.57629   -1331.0194  -1285.49979  -1469.12857  -1622.34966  -1691.01836  -1059.47799  -1234.99825   -1358.6864  -1573.85516  -1294.36739  -1667.26522  -1327.51176  -1394.80138  -1299.53592  -1790.04636  -1472.49962  -1441.86958  -1153.98079  -1753.23997  -1783.23564   -1805.7201   -787.91845  -1483.97226  -1277.15696  -1125.71098          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -2.50   -522.28337   -249.38938   -406.34261   -307.89105   -546.74031   -132.61797   -299.34316   -276.34541   -333.54166   -454.17591   -162.38862   -234.99878   -356.25253   -277.57837   -282.86118   -356.43952   -279.90219   -206.31845    -380.4715    -306.2586   -375.30666   -225.78266    -310.2011   -392.85254   -192.53883    450.01639   1290.95438   1806.85312   1527.04657    717.50896    -81.77693   -193.29412   -452.36265   -249.19089   -373.20705    -346.2657   -346.41728   -275.33775   -310.28326   -519.98946   -442.57345   -191.47114   -397.69227   -307.85407   -365.60894   -393.07918   -397.46883   -220.40014   -107.53978   -409.82605   -495.56877   -287.87283   -365.21659   -281.10105   -388.54756    -98.76778    198.12354   1066.21209   1670.26384    1548.4754    776.87948     14.50433     97.89947    -403.3453    -322.4631   -356.94164   -338.79028   -269.34033   -392.01112    -84.74454    254.74697    959.57428   1475.20838   1372.47397    927.42747    512.45095    723.33608    484.70724     35.40872   -141.48918    -412.6683   -530.16418   -492.30301   -233.84695   -326.01565   -398.20223   -461.43501     -515.141   -182.06367   -147.44548   -117.90739    -44.82947     17.54119    -59.97213    -255.4393    -10.98037   -313.87443   -227.08584    -403.1945   -365.77804   -167.82212   -152.96746   -115.95918    105.88979    -192.8032   -270.73131   -315.82704   -277.95997   -425.51378   -320.63641   -402.35577    -510.8825   -535.23169   -232.29901   -145.82703    -265.5191     43.20886    483.51212    206.98445     137.8683     75.16146   -168.66887    -218.1995     58.86203    167.67989    151.00783    333.47632    420.01681    355.26434    -27.24924   -176.63225     -367.575   -314.90817   -258.93098   -284.83924   -135.40636    107.17695    709.74215   1360.26211   1696.69777   1661.72334   1288.27777    685.85225    352.87255   -132.10748   -189.26092   -271.64984    -37.31131   -282.95236   -241.03354   -221.54345   -647.12384   -450.30377   -497.52889   -307.64783   -337.75352   -269.80585    -279.3639   -274.18039   -227.94334   -119.58128    -132.4392   -291.32964   -172.05525   -291.78772   -161.72358    -82.65319   -196.41435   -121.32252   -217.08224   -322.12877   -175.91983   -335.28855   -237.47655   -372.14035    -244.9794   -378.44474   -167.38156    -314.5252   -164.84847   -157.11782   -293.75662   -273.65801   -359.27671   -288.82867   -240.78846   -215.66795   -420.96923   -445.26852   -362.70239   -313.26701   -173.13048    -164.5622   -181.80119   -151.58757    -95.67595    -149.4233     60.76322    253.73742    189.95477    163.55911     26.06453   -152.30825    -33.78934    -43.81498     37.30115   -136.36875   -117.66054     85.94227     41.87677   -149.77901       65.453   -330.45265   -287.75888   -278.05352   -342.12106   -447.92286   -300.40516   -339.23587     -501.005   -431.33701   -324.57721   -176.57065   -264.05502   -216.31785    -36.37137   -217.44894   -158.94293    -82.86721   -130.53929      4.17174   -112.63347    -78.74455     55.00281    -84.19232    -53.76101   -254.19325    -74.53917   -139.95303   -122.73232   -255.67516   -198.98032   -234.06664   -235.38141   -337.00039   -415.60631   -446.33145   -346.80577   -557.84601   -522.93711     -351.685   -469.51894   -374.16744   -419.94385   -171.78857   -223.78489   -300.06238   -360.05698    -33.15927    -23.01863     -75.1527    -48.57804    257.23373    251.06131    351.49681    304.31051    402.10846    445.71949    123.53248     38.83818   -109.53373   -151.11683   -253.16317    -47.84762   -439.85591   -415.25761   -324.31813   -290.62612   -327.32018   -315.63856   -182.46135   -259.98571   -215.80828   -232.97074   -269.36296   -212.88195   -234.85956   -320.06849   -429.61591   -260.86961   -528.17464   -583.15812   -605.79448   -354.70342   -523.52577   -566.39412   -499.70718   -593.59408   -605.26767   -407.26066   -334.88669   -509.67723    -620.8459   -407.63021   -484.57444   -664.20882   -740.84629   -402.65072   -316.31391   -319.67384   -572.69867    -258.5165   -435.53231   -244.72098   -311.47726   -336.63988   -253.23436   -273.71351   -380.90758   -393.01493   -359.80875   -439.86261   -289.95822    -354.2852   -484.96778   -651.26008   -650.08069   -482.69478   -589.09472    -213.1179   -429.56255   -185.77899   -381.08667   -399.14351   -204.84647   -322.35955   -443.94658   -371.46783    -466.5227   -344.25202   -279.17147    -439.7675   -173.96163    -97.78023   -365.43372   -400.07872   -509.60378   -181.87257   -485.69108    -579.4681   -292.47398   -368.56982   -270.13056   -430.84076   -176.94678   -103.27293     95.62462    -48.82132     23.02833     19.34632   -193.23022   -241.61018   -197.68625    -83.19505     83.83624   -188.89441   -131.11258   -118.54561     -194.774   -270.46226   -130.34135    -56.33747   -394.46386   -225.25076   -228.41355   -150.52578   -332.87045   -474.85742   -256.73638   -230.53438   -328.43842   -247.32643    -68.33395   -202.09033   -327.01747   -495.32871   -481.38016   -160.94572   -425.53661   -373.96768   -324.42893   -343.99272   -451.23496   -476.42854   -687.61293   -350.47947   -437.95868   -418.18792   -402.95533   -480.53405   -207.95982   -567.68746    -328.9461   -376.75854   -571.41782   -413.96539   -502.68328   -629.88576   -477.68977    -742.0861   -758.67854    -548.8146   -435.23108    -581.6512   -660.06372   -538.35489   -229.98969   -399.77453   -319.94574   -724.85219   -768.80767   -302.05143   -923.17676   -270.89584   -646.36789   -441.05447   -322.63227   -781.33292    -459.8553   -513.67139   -580.02635    -1058.246  -1182.20735  -1173.05449  -1414.17174  -1121.55429  -1011.97556    -953.0819  -1042.26706    -746.7904   -970.37231   -1240.6842    -944.8328   -800.50295  -1219.81932  -1086.50192  -1502.11497  -1036.87123   -947.22333  -1308.46193  -1143.42188  -1194.53969   -1321.1474  -1122.79555  -1256.29404  -1431.63852  -1094.68759   -1417.8801  -1376.83538  -1349.45219  -1100.68776  -1263.25487   -1643.0563   -1217.3918  -1209.15078  -1182.46832   -1608.3891  -1280.51112  -1279.90284  -1016.46844  -1796.74176  -1336.93607  -1344.10443  -1414.19185   -958.87744  -1146.79972   -932.55056  -1427.36752  -1525.41364  -1893.66397  -1306.30576  -1570.75136  -1406.42452  -1082.66027  -1600.60269  -1229.75016   -478.61625  -1390.10095  -1801.14245  -1310.84973   -912.89398          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -2.00   -444.78866   -317.27965   -372.69715   -357.93588   -418.82922    -206.4302   -318.90968   -312.08137   -386.11206   -355.46897   -351.66096   -163.31989   -370.82874   -197.57301    -367.9069   -357.94625    -509.9055   -241.03121   -421.27401   -283.91383    -317.4805   -363.55425   -232.41279   -252.72408    -93.62024    662.30194   1160.69938    1821.3938   1371.82233    528.68634   -100.09886   -361.44181   -401.71198   -305.55718   -433.02741   -327.81416   -371.82963   -360.28826   -514.47658    -420.0173   -354.22414   -215.14788   -428.22141   -273.02655   -641.97879   -379.70048   -509.37138   -456.45273    -287.2359    -348.8349   -380.37116   -453.88985   -357.28232   -410.64316   -405.96253   -421.21867    316.96084    954.27681    1413.2706   1321.06764    814.48637    120.68478   -178.41522   -234.40445   -292.68266   -242.54558   -360.40823   -138.53764   -250.56667   -182.07105    392.52991    941.98838      1404.61    1254.9769    736.80467    795.55967    581.59918    447.99609    123.20414   -223.48901   -144.63809   -138.29541   -173.91488   -570.74925   -253.14004   -274.97584   -328.31076   -388.98495   -357.53595   -367.70782   -224.12934     26.18126     -5.50407   -131.28707   -193.95793   -355.87651   -435.91745   -476.56399   -403.12506     -315.337   -302.93332   -103.81011    175.69412     99.94546   -234.06151   -297.81024   -409.32124    -372.4167   -296.18895   -377.49076   -607.11546    -432.1016   -229.60262   -347.48335   -118.04485    -187.6459    135.98726    217.96617     153.0507    234.78412     74.41128   -166.91522   -258.09116    -59.96457    134.68255    417.39833    449.38894    533.23116    281.54238     82.52653   -139.58623   -168.51041   -377.26124   -406.54517   -245.97727   -119.14882    131.98332    614.86932   1181.56483   1724.82851   1810.06477   1451.13914    923.04787    461.88762     58.66591     25.35059   -265.68842   -128.56517   -259.81332   -272.83595   -344.58214   -227.27545   -449.73546   -442.07181   -256.85315   -327.78036   -377.13517   -326.17748   -232.97731    -91.42632   -130.69507      -9.6596   -122.29532     -245.559   -396.62258     -75.7702    -45.02357     -27.9915    -95.36812   -172.68934   -336.99515   -243.49083   -381.99191   -222.59976   -379.74138   -270.26497   -164.33385   -144.40464   -212.23427   -242.51429   -266.58627   -272.55909   -295.12263   -425.94385   -382.45365   -354.58025   -203.65333   -483.07181   -442.25659   -468.13422   -195.15524    -248.6991    -39.84575   -147.51365    -52.43321    207.49422     47.54098    -61.40936    223.95847    218.54313    213.56929    -43.70851     98.08597   -174.09142   -208.73859    -37.77231   -176.08057     -59.8883   -238.47703    -78.63045   -129.54399    -70.60929    -72.43687     -158.598   -235.94919   -265.06708   -382.51998   -481.49962   -421.74763   -352.82687   -200.17189    -434.8983    -241.1004   -342.21875   -428.40156   -369.31699    -80.30578   -114.85333      91.1327   -228.68886     12.83711   -152.58759     -16.1067     23.16779     15.29464   -119.03199   -138.62943      82.5765       53.713    -38.27064    -31.17932    -68.72463   -252.61987   -150.73908   -104.89977   -574.14777   -335.14866   -483.66145   -428.73299   -364.01122   -399.00208   -480.26427   -384.18649   -414.05726   -252.37296   -215.61951   -110.16169    -69.93789   -114.71347    -12.04549      75.6782    360.51454    251.73915    286.44445    400.37568    518.55574    358.84175    278.69771    217.66588    214.42466     53.86032   -237.68354   -238.37816    -285.1788   -291.04648   -188.38936    -145.2645   -226.97491   -289.34896      -230.91    -91.43937   -299.00301    -84.91365   -158.57876    -112.1707   -336.89254   -190.46891   -275.08399   -243.08253   -519.19816   -417.30365   -451.51168     -355.357   -533.96172   -541.29321   -539.52212   -368.55969   -474.58501   -360.33137   -465.20819   -712.84751   -363.22614   -415.85023   -568.10967   -438.69147   -347.56915   -344.88628   -359.00629   -225.42065   -328.71685   -294.35726   -251.20872    -60.14746   -301.46896   -201.75927   -173.49243    -43.68394    -53.40005   -143.51709   -330.60593   -345.98932   -275.32496   -240.79218   -425.64846   -274.77122   -351.80605   -213.04602   -333.72287   -385.02997   -432.79711   -193.61958   -445.88594   -103.29871   -268.72388   -364.72782    -262.7559    -60.67112   -319.45011   -270.95471   -432.46043   -380.18275   -364.17936   -365.57279   -243.49088   -296.26806   -485.42971   -457.58845   -385.95591   -437.28652   -149.66929   -289.60985    -380.5703   -307.15679   -195.73661   -302.22005   -155.73214     51.69054   -137.23929     44.87245     61.03059    -95.22323    117.50241    -81.68732    254.07856    -65.30188    -79.81212   -122.54836    -65.11634    -74.32683   -191.76076   -298.80267   -250.19723   -334.69084    -370.8399   -239.30502   -342.64506   -280.24152   -250.02675   -271.81413    -393.5026   -260.17925   -213.46982   -134.89444   -291.50351   -230.29308   -316.92907   -227.16129   -332.62967   -403.60924   -239.64325    -417.7396   -572.32589   -407.56346   -263.51137   -269.50293   -317.94594   -631.41336   -285.44641   -423.02288   -186.46819   -537.81025   -334.70592   -309.29639   -341.60721   -517.99258    -247.6301   -625.46334   -352.85079   -384.29914   -272.02634   -434.84298   -431.55291   -509.74324   -315.08543   -527.58309   -453.88317   -341.08925    -578.1631   -669.14656   -745.95348   -504.54938   -597.58815   -411.15773   -504.29568   -666.70146    -572.8622   -708.95082   -593.92645   -783.17807   -579.24286   -821.66267   -899.27361   -791.26854   -672.57918   -981.75107  -1041.13544   -793.29486  -1095.38634   -706.62935  -1038.42239    -766.9092   -1160.7883  -1311.67499  -1023.52564  -1201.10985   -903.74546  -1055.04213   -962.40266  -1361.74142  -1415.33322  -1179.04241   -837.28372  -1369.84839  -1215.47335  -1229.01662   -811.27714   -1393.6616  -1382.76644  -1243.67126  -1184.21656  -1183.49842  -1270.40637   -962.26925  -1251.35989  -1306.03026  -1044.02383  -1199.63617  -1734.09114  -1374.43159  -1251.09672  -1177.79944  -1203.25292  -1372.44469  -1553.41449  -1433.81545   -884.18513  -1339.08272   -1690.8182  -1417.43819  -1605.52005  -1507.60667  -1184.43567   -1581.4686   -1421.9889   -972.72942  -1265.66327   -745.45435  -1025.43509    -261.0144  -1209.08237  -1525.72639          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -1.50   -391.80169   -380.30751   -333.67322   -402.14969   -292.57987   -306.68334   -344.63646   -242.69956   -386.68315     -446.128   -254.97928   -297.20624   -282.60185   -226.39313   -349.35687    -333.6828    -600.1357   -312.81442   -418.38733   -110.68052   -426.26901   -467.53737    -302.1198   -298.58458    -23.73733    150.20335   1491.94787   2046.18229   1390.89656    547.02281   -128.31609   -218.06733   -383.39674   -290.36377   -504.73512   -355.08233   -227.45822   -447.78969   -227.82623    -58.58099    -389.2966   -314.41499   -444.38954   -326.94705   -573.70547   -407.62324   -406.27031   -397.31524   -434.15376   -348.19517   -350.27828      -357.87    -347.9253   -376.50415   -196.62612   -129.87251    190.05342   1089.34065   1784.91308   1316.93496    774.20349     35.36802   -175.99198   -185.66276   -435.24769   -322.25753     -359.732   -307.86766   -193.12061   -107.81254    206.27996     945.3466   1508.94947   1233.87503    999.02033    617.25505    559.62131    669.76044    260.54793   -195.07063    -76.41342   -388.18638   -283.27636   -331.21268   -410.21626   -338.64474   -254.18386   -176.18497   -333.48184   -144.01344    -96.90999     -6.31989    118.12442    -32.92769   -217.69388   -346.32209   -498.20385    -309.1831   -308.43167   -496.11297   -230.91579    -34.18688   -181.81855    -95.74018    -46.09815   -270.15352   -312.96617   -510.21114   -473.75217   -375.66898   -396.28519   -299.43468   -421.85293   -361.45444   -232.83027   -181.63169    158.43893    421.84625    270.79709    119.05107      9.15413   -199.57838    -90.35669     -99.8994    150.44157    487.22336     426.0903    548.61963    296.45963    123.98842    -67.24939   -224.14611   -198.31918   -257.18251    -251.9057   -105.74581    160.00896    724.71124   1449.48333   1602.72296   1946.69658   1560.91519    904.73787    525.51603     65.61351   -259.42006   -204.64772   -348.73797   -228.08477   -322.41779   -359.71946   -357.47395   -337.81112   -382.53011   -413.88313   -384.98156   -306.84214   -239.23725   -194.50818   -146.75224   -143.47042   -211.39483   -186.79739    -89.76392   -112.46208      1.09309     50.13028    -10.83535     76.40304   -202.35043   -309.62386   -230.67253   -355.94632   -470.14908   -390.86338    -142.2115   -186.93024   -200.28016   -163.07104   -169.65827    -76.52936   -363.35076    -320.1956   -374.36622   -375.28706   -311.80128   -304.05535   -296.45041   -418.47867   -162.11004   -225.52692    -151.0316    -92.33374    -95.72303     -8.24312   -235.78564   -146.58977    107.46188    156.22237    223.33248     67.17054     42.87116     67.16447    -77.52716   -123.89105    -223.6171    -84.37188    -34.16975    -72.34056    170.72184     41.92923     11.18052   -209.97906   -175.09559   -249.85319   -310.27461   -358.67518      -149.12    -314.7037   -400.45601   -200.18418   -285.21802   -286.73326   -326.12101   -318.51319   -199.54641    -91.02298      88.6439     38.98531    -35.75282     18.23773   -163.10969    -55.01477    -39.35462    -47.53023   -178.06419   -109.36579   -153.89647     -3.94921   -202.78953    -60.09151    -68.64669   -186.35534   -257.12708   -289.52959   -331.24402     -479.191   -294.73611   -435.10218   -412.76112   -349.12828   -302.76545   -349.43727   -294.51567   -345.14999   -139.04356   -233.38673    -71.61721     -15.0551    124.86403      45.7319    195.50165    283.75625    294.38527    473.90771    495.04819    414.29383     441.2259    357.09449    266.22892    144.30596    -55.48431   -184.03313   -150.51075   -309.21739   -275.05165   -275.57844   -167.15611   -127.31083   -226.56112   -351.34501   -147.93113   -166.54745     59.12485    103.88402    -61.20722   -203.24902   -277.60805   -354.25067   -533.56476   -388.09976   -524.36761     -251.699   -371.72337   -532.73925   -484.94157   -356.10669   -465.39979   -413.62545   -466.14222   -518.94173   -396.18105   -307.70946   -506.51848   -336.16499   -286.13746   -411.78573   -428.01863   -265.33763   -293.19938   -199.98899   -237.23069   -116.09348   -312.69377   -303.51387    -16.27081    -66.04776    -340.6366   -181.78553   -129.63757   -252.31048   -226.94592   -346.24846   -219.23572   -321.85221   -305.31517   -267.73166   -430.22633   -295.22393   -216.45385   -394.67401   -124.60388   -367.23003     99.85707   -317.99011   -288.19816   -183.56181   -260.50509   -138.32306   -400.29789    -78.78577   -246.89562   -324.75575   -287.24882   -356.06032   -213.28197   -326.72155   -140.41666   -345.99939    -95.23033   -292.92708   -216.13923   -191.09182    -65.48345    -83.73451   -101.22848        41.44     72.40225     97.89502      67.2259    214.32801     33.70265    182.97527    -27.08171    -19.14435     80.28583    -19.43382    -87.59661   -121.00186   -123.46908   -101.06845   -418.98599   -224.16411   -143.37022   -193.50228   -184.58135   -181.78767    -140.5339   -109.13102   -357.31129   -233.36436   -202.24707    -293.1081   -198.35495   -411.04366   -368.04456   -152.66186   -450.40308   -437.33304   -364.59484   -478.02582   -379.97989   -433.33095   -255.75852   -371.98308   -384.20141   -396.74173   -462.54758   -475.93329   -159.30783   -502.89396   -190.77569   -268.58553    -462.0801   -737.87005   -253.83364   -168.76432   -219.46879   -492.37047   -203.95753   -242.12585   -215.83701   -266.23449   -443.32452   -424.45406     -502.089   -510.19264   -483.74679   -287.11631    -428.6581   -552.13213   -362.81375   -614.66312   -637.40072   -374.72985   -818.59048    -381.7929   -430.09804   -397.82079   -571.27309   -752.70364   -897.24947   -746.58357   -904.27528   -587.89443   -754.35996   -708.51874   -754.34706   -868.15346    -490.4979   -572.14609  -1011.09527  -1247.55679  -1083.74137   -907.32514  -1152.94664  -1041.08073   -952.48444  -1017.38702  -1216.25313  -1370.58094  -1520.16144  -1071.29811   -1056.9419   -919.88301  -1160.80421  -1197.27442  -1084.99558   -927.48921  -1043.87822  -1135.22599   -879.59588  -1019.37649   -875.86692  -1055.57295    -1701.076  -1072.86429    -890.1157  -1431.27025  -1239.25939  -1386.74675   -617.56695  -1242.50289   -948.33624   -833.57438  -1135.91832  -1231.79389  -1300.14679  -1529.49572  -1681.32729  -1228.17394  -1529.72644    -957.0332  -1653.01917  -1019.70684  -1284.53919   -283.92722   -2114.6877  -1698.57973  -1142.29013   -484.11492          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -1.00   -427.01356   -385.16092   -294.50626   -412.07509   -210.37673   -293.09968   -291.86265   -248.24531   -370.63737   -503.09516   -192.21711   -409.40875   -165.68375   -301.64971   -253.97201   -276.13428    -469.2938   -367.99963   -309.62576   -155.36192   -421.67968    -346.4513    -254.4748   -196.53492   -182.06978    137.94315   1410.74858   2042.34908   1492.03092     614.2787    -87.16051   -145.56784   -369.46554   -127.16599   -371.68282   -304.89359   -353.36006   -404.43327   -368.17239   -218.63386   -309.00873   -462.22867   -398.65156    -362.1008    -261.9957   -364.69438   -408.85062   -403.54336   -376.99092   -199.63017   -363.31656   -290.81456   -272.56376   -291.03335   -262.15892    -112.1173    400.77503    990.61884   1606.40709   1446.18365    566.10798    149.79635   -175.43405   -257.51188      -266.74   -446.59482   -280.67603     -217.904    -180.7942   -322.44486    139.18813    956.26171   1433.81318   1238.98612    616.51322    589.83638    642.10185    350.32678    201.02203   -186.43656    -83.51476   -454.85839   -243.67986   -298.74029   -379.10409   -219.12624   -266.82598   -341.12792   -282.77545    -209.0246    -51.76472    -12.27143    -55.72063      5.80521   -174.95009     44.93899   -496.32551   -428.44925   -361.24506   -536.64499   -188.70876    -57.55304     65.43812    115.51362   -295.64785   -259.35098   -170.60899   -402.27373   -478.04478   -439.03637   -394.89908   -412.40796   -495.69124   -409.88953    -135.3088   -135.34505     88.91287    243.16694    241.71353    222.71975    -12.36304   -127.88941   -147.53153     -22.9836    150.34925    426.40021    460.98503    551.11866    277.00605    284.16896      2.20281    -181.0514   -195.03618   -193.93051   -269.48643   -251.37982     75.09343    855.79079   1470.80263   1832.47418   1921.84204   1548.18887    965.22774    555.09429    124.95342   -177.80076   -166.60837   -313.52896   -273.36353    -342.5109   -321.94204   -385.66871   -396.61228   -251.04773   -282.65957   -323.62337   -187.70433   -239.10357   -232.52403    -85.70439   -237.33728    -43.33472    -93.29139   -114.76606    -88.39546     94.99126     64.17909     36.56944    -24.28877   -144.16321    -51.82177   -241.36889   -330.36206   -290.46127   -191.51595   -177.31426   -309.94826   -224.60069   -381.34369    -181.4224   -217.19156   -220.42731   -282.56629    -246.0646   -319.27247   -364.90251   -420.44973   -417.45444   -415.55991   -373.41418   -102.80547   -244.55349   -173.55714     81.78595     39.57655    -84.59576     27.54619    -11.84835      37.5154    250.18553    247.68431     80.74785      1.90035    -104.3706   -176.38977   -134.01097   -169.53341    -84.66423      35.7415      37.3936     151.6241    -32.25597   -192.16147    -68.81161    -11.00613   -275.80486   -358.69549   -370.86392   -364.14345   -503.89721   -319.87622   -337.30429   -313.18489   -211.75788   -228.75209   -101.65935   -151.22863    -57.73275     71.42533    -64.44382   -111.76787     -19.7716     10.75937    -96.02494    216.74756     118.8978      -2.7159     13.75638      28.9329     -8.94029   -172.38446   -144.41874   -137.53111    -142.7689   -308.77592   -256.14591   -398.53325   -401.92282   -461.77605     -410.981   -273.63546   -336.73326   -291.22907     -147.639   -165.75869    -82.94323   -221.30613    -89.40865     50.86765     85.30807    105.60428    395.04397    462.03299    255.18553    607.94306    648.86195    413.25344    392.43948    270.53862    111.47374    206.97658     61.09644      36.6771    -72.06986    -73.27198   -167.84268   -195.81214    -295.8215   -146.46221   -228.74377   -161.34986     82.36135   -145.95602   -139.06825   -178.09834     -8.84439   -267.55468    -47.73214   -245.20634   -427.44635   -371.20832    -431.0898   -434.77494   -533.85527   -406.24797   -322.65878   -453.27199   -353.08774   -328.75023   -505.63948    -547.7421    -472.0718   -377.22398   -473.15385   -339.07361   -278.24444    -246.0926   -210.02416   -115.59864   -310.51076   -329.09522   -326.97086    -349.4321    -62.15864    -69.86267   -227.93313   -156.92417   -179.05233   -262.24357    -351.1265   -331.42486   -219.16452   -396.24777   -229.20189   -234.83301   -382.15174   -391.76972   -198.41209   -361.46914   -268.11328   -184.65277   -156.43646   -245.22287   -278.40643    -280.5278   -251.37587   -285.57769   -199.38521   -107.43518   -235.42011   -117.48651    -220.6816   -278.27964     -352.634   -299.37035   -158.77616    -260.6996   -327.32009   -284.38593   -170.95189     41.65956   -161.59346   -119.89815     -80.7121   -129.61728     -60.5529     233.6669    101.13934    112.70716     75.13166    -44.87317    214.96262     78.26878       16.337     35.02843    -13.60578     -7.92043   -126.27864    -66.39919    -72.81267   -123.43056   -144.43295     80.15392    -282.4417   -196.86987   -106.25568   -255.04572    -94.47609    -138.3377   -109.26771    -102.4712     10.18416   -185.51492   -147.07963     -78.7432   -167.54036   -205.86461   -359.24545     -99.6917   -310.39565   -337.69484   -453.25662    -415.6475   -287.13167    -241.0419   -386.34994   -295.17444   -512.15197   -414.63075   -452.91578   -436.03361   -402.86483   -397.45718    -331.9227   -346.92474    -302.3846   -554.93093   -627.78768   -405.83419   -221.29145   -394.12136   -356.41924   -463.22496   -428.77314   -289.68203    -486.2177   -471.90825   -562.27048   -611.56732   -483.03947   -265.87515   -337.47542   -728.26015   -593.37136   -733.01464   -560.76176   -538.74989   -406.06803   -614.01303   -526.78508   -708.35293   -761.64644   -524.35328   -758.47772   -918.96446   -800.40641   -575.08081   -520.72348     -915.172  -1203.22152   -881.72701  -1067.68507  -1033.40019   -903.93984   -927.43829  -1059.30475  -1413.14529  -1038.96159  -1231.31061  -1388.79554   -909.30322  -1156.65992   -864.83078   -897.03781  -1250.96753  -1003.00894  -1147.99672  -1233.45555   -1269.6824  -1234.23734   -568.67591   -678.97098  -1089.74712  -1137.00435  -1229.03943  -1192.10554  -1177.33101  -1041.63514    -989.7216  -1267.60407  -1377.89035  -1174.41311    -709.1652   -779.13764  -1657.56643  -1085.35009  -1526.04397  -1221.16572   -915.45203  -1156.70537   -698.95001  -1273.95622  -1553.62387   -462.46428  -1331.65629   -963.53923   -1126.0519  -1114.73361  -1254.33187   -671.06196  -1414.75643          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -0.50   -466.83131   -390.53625   -258.13958   -413.50787     -146.046   -198.77016   -150.00298   -370.30392   -315.17458   -444.79839    -235.6484   -246.00464   -201.26903   -348.34384   -321.07019   -384.60392   -389.08577   -410.38464    -266.9537   -310.42204    -413.6836   -404.33803   -300.34162   -149.57288   -198.48236    290.75484    1283.9279   1911.69504    1466.5901    668.85969    -24.88416   -358.63656   -329.05045   -195.13283   -444.08613   -360.80307   -365.22568   -219.83634   -359.03237   -382.58909   -370.55774   -378.46629   -208.01329   -471.03201    -454.9773   -310.87929   -471.21184   -326.45751   -313.60525   -161.70715   -272.27959   -572.20055   -235.73402    -451.9251   -235.83382   -160.65352    107.65419    1043.7118   1655.56036   1542.98535    500.79592     63.33415   -131.53738   -381.95483   -261.94969   -448.73036   -168.93203   -207.00892   -270.11055     -37.8398    161.84975   1105.88276   1368.30035   1354.94693    966.71126    660.82217    738.08628    458.95052    323.09431     21.02151   -200.32143   -341.44196   -268.14944    -224.5401   -358.40799   -362.51239   -349.97307    -299.8322    -389.6512   -238.28294   -107.78173     88.05283       -0.366   -274.88936    -327.7457   -184.60781    -291.4135   -279.18762   -401.48212   -539.92468   -252.30388    -19.05798    144.99205     -7.20048   -154.58331    -197.7666   -382.49276   -294.06264   -416.28349   -355.32432   -419.68292   -388.10705   -379.82887   -230.95007   -241.78583    -44.93647     53.11749    265.85673    237.69118    237.71972    -30.02866    -65.60218   -160.10174    -47.36318    197.41227    407.90292     564.4914     580.7759     290.3364     78.79045    -98.14958   -146.31137   -328.90409   -231.20832   -146.31366     26.56446    191.33428    829.07007   1600.09268    1955.9685   1965.73404   1712.74671    925.70017    499.65813     29.29205    -88.14453    -262.3106     -79.1687     -204.983   -250.38918   -478.27084   -327.95575   -283.22853   -222.87597   -349.33028   -321.31676   -337.04206   -175.98025   -239.01399    -70.48471   -263.97974   -211.82802   -202.76185    -91.66976    -96.48398    -76.73516      54.3652     51.25567    -89.35723    -36.03923   -168.57169   -135.51803   -168.96156   -655.62898   -347.23778   -220.07095   -128.51443   -294.97801    -91.17267    -79.07526   -383.88232   -239.54785   -319.77824   -390.23671   -338.14969    -425.1686   -317.83543   -388.71763   -322.52791   -391.39975   -258.11615   -213.52479   -177.20403   -117.64118   -130.13498      31.3147    205.84999    219.31537     94.82739    242.96394    163.47744    156.66155    101.81003     72.02149   -222.04014   -101.74118     77.66463    -30.52062    -88.72488     22.54825    -23.41943     93.33336      3.49999   -126.10816    -188.1385   -331.55635   -287.04238   -231.24749     -203.195   -315.02126   -304.84272   -378.64252   -204.86138   -368.14031   -278.69611   -130.39766    -89.87398    -32.05606     40.66949    -59.67008     36.17608    117.93488    -25.95467    -53.76791    -59.99063   -212.37577    -52.27109      9.01515    -89.72896     44.67289      1.82533     -96.4098   -116.92106   -357.64013   -158.73809   -313.16397   -115.69016    -302.2644   -330.96366   -289.68698   -266.84387   -242.58122    -95.05391   -145.71504   -275.63133   -223.88083    -99.50868   -185.46898    -12.33744    194.13733     88.60031    250.53515    553.49232    261.30354    469.62439    419.72449    506.85508    701.06032     333.1287    246.78439     99.16797     78.93907    -20.57867    -70.31873   -206.46839     21.54005    -54.36882   -196.94509     19.15909   -140.96877   -128.09391    -60.25648     73.89042    -30.79791    -157.0843   -191.78683   -225.50455   -293.59489   -196.09172   -161.93355   -311.78899   -334.76737   -295.15005   -341.14248   -444.35349   -559.50035   -462.19097   -462.20335   -253.92336   -448.18655    -306.2033   -236.30908   -394.86534   -299.12733    -370.9979   -271.32883   -316.15341   -178.63779   -260.01343    -275.8032    -267.0697   -233.37656   -138.59596   -109.69055    -75.19796   -186.07153   -214.73281    -62.27401    -96.00015   -193.09469   -218.32333   -198.26405   -140.21688   -382.37778   -303.56601   -232.05462   -248.10108   -353.34633   -321.98737   -235.78558   -337.85722   -253.67389   -223.82073   -249.17879     -51.5252    -78.77241      4.04527   -207.85347   -119.87947    -85.83528   -204.48107    -58.06574   -109.18141   -128.53753   -340.77043   -194.91514    -266.8318   -152.14834   -260.50448   -219.97066   -183.71561   -131.54669     -96.1877   -144.53471    -82.41526   -102.82956     78.44899    -83.33687     79.94892    338.18226    236.04798     54.37868    248.76218      51.8105    -27.10527      6.66792    -12.76393   -167.82665     76.45198     43.35861   -135.42002     82.19746   -108.01154     51.84479   -160.37297    -207.5651    -71.95582     26.74321   -147.23386     37.66548    -67.99129    -50.03435   -167.33618   -274.93202   -114.52365   -290.83204   -243.93612   -173.41363   -282.97399   -176.74712   -435.53357   -244.42394   -221.98902   -285.93001   -372.70222   -395.85567    -232.2763    -300.3995   -229.38382   -541.57233   -534.06462   -386.99374   -302.32974   -456.76879   -318.51937   -228.35531   -309.66299   -255.52268   -169.60193   -583.00523   -381.81586   -398.01463   -397.76867   -527.38077   -570.95489   -432.66698    -27.86663   -402.95076   -244.60638      -301.44   -467.24646   -548.99285    -350.6626   -347.60322   -537.60244   -329.75853   -581.42959   -563.60594   -442.68597   -551.93421   -664.50597   -696.52765   -834.78561   -908.96689   -859.33038   -622.28965   -792.30249   -667.92734  -1094.88979   -893.37049   -994.29308   -678.99085   -990.88477  -1066.93111   -1371.2412   -797.68948   -827.39736  -1361.18251  -1362.13618   -983.33232  -1215.52857  -1016.94526  -1141.79269    -836.4314   -675.67588   -1040.3455  -1274.59214  -1254.38094  -1011.97573   -793.44988  -1160.72936  -1825.35152   -972.20138  -1057.74335   -896.35655  -1397.53422  -1247.45203   -799.20443   -1177.5333  -1104.53615   -985.72962  -1204.33452  -1117.36414  -1213.57465   -1497.4055   -992.24387  -1168.66482  -1115.85902  -1249.97237  -1027.22959  -1400.86774  -1692.58137   -798.25158   -989.57834   -612.25979    -499.4271  -1072.27242  -1387.11032   -748.18617   -980.63841  -1103.12352          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        0.00   -447.40195   -311.61957   -232.51177   -387.06526   -149.79598   -147.55959    -115.0136   -429.98048   -281.43593   -366.03273   -253.27722    -163.4135   -246.20456   -382.77134   -397.61504   -454.60815   -334.33282    -445.3264   -267.84338   -478.03616    -366.0693   -520.95438   -393.71064   -194.71504    -15.93854    375.90366   1426.32951   1860.37942   1259.52936     791.7941   -112.25903   -252.62098   -386.11912   -264.23523   -464.98021   -394.85501     -331.791   -434.05147   -424.49651   -314.92899   -473.79779   -304.68072   -301.21927   -259.20816   -457.36327   -358.95958   -356.82957   -295.75265   -408.34694   -226.12073   -375.29233   -284.06782    -286.7417   -236.18396    -522.4559   -114.44757    226.63779    800.53116   1791.36752    1219.2932    687.96623     80.35448   -371.07671   -302.16537   -264.45856   -339.50859    -340.0652   -262.70089   -272.44743    -68.76445    309.66337   1021.59537   1556.63922   1648.80077    878.19593    760.16007    673.11424    391.00697     253.1668    -61.62883   -412.62209   -432.34982   -197.50098   -331.10123   -347.83851   -285.08395   -306.53574   -287.57435   -274.75798   -398.29197    -83.55354      2.38137     90.73113    -17.89649   -174.87679   -239.30398   -257.80893   -362.34403   -203.07088   -399.60753    -96.37018     15.65878    189.63531     -21.1919     23.92282   -230.32468   -254.21202   -360.88975   -414.86259   -367.17873   -405.61795   -303.76118    -197.6119   -268.85216   -180.91281   -168.91531    157.72162    312.24259    327.36673    117.40484    -18.39462   -197.78261   -123.78845   -208.92153    286.71732    438.84975    667.06851    560.78934    558.94931    271.60827      20.0608   -115.58932    -223.0397   -252.48433   -284.70126   -153.98687    357.75634    749.52287   1586.74325   1988.81951   2081.40979   1697.00981    1123.3183     505.2431    144.84208   -205.50775   -251.23681    -19.25601    -122.9375    -298.7003   -222.09025   -463.35582   -305.48675   -283.27096   -465.23296   -318.68743    -197.2001    -97.92951   -243.51396    -131.6072    -49.01177   -106.17605    -77.72698   -349.93108   -155.08676    -39.02471    -19.05333     23.44542   -177.22493    -92.49781   -183.76064   -379.39298   -319.39049   -376.33216   -279.07164   -182.92394    -51.11724     -95.6429   -223.23616    -82.01338   -299.36134   -329.04501   -293.32531   -346.52039   -409.73419   -316.30273    -427.5102   -387.05672   -254.73755   -193.47059   -248.14128   -320.78101   -163.93443    -96.41856   -136.66159       3.9159    184.16104    106.91822    405.24403    197.24309    226.28548     -3.32287     -3.73361    -79.43895    -13.27465   -206.14009     61.53911    -23.95719    103.70443    -31.93091    117.67088    -48.01554    147.61855   -127.70789    -76.34619   -277.32595   -426.57719   -517.44482   -407.88531    -346.3463   -397.38049   -515.21372   -346.27171     -242.999   -272.50182    -79.79858      8.65417    -32.62559     46.36086    -47.56079    152.10792    136.62698     12.91024     38.46046    137.69301   -175.00232     71.20499    106.84851      146.323       4.5665    121.63458    -47.14927    -89.67045   -120.40729   -284.57606   -234.09525   -322.22086   -397.59795   -207.62972   -273.93912   -250.75538   -276.25266    -217.2096   -206.39736   -250.46011    -16.81734     43.28582      57.9495    170.51816    160.72765    118.30821    455.77931    497.86532    519.88468    456.27956    530.80361    724.03851    448.34966    296.05831    386.96976    123.75174    130.32905   -215.97191      0.03452   -324.74466   -272.47762    -72.18264   -216.65294    -99.51228   -177.05608   -161.66521     88.10863     43.92317    124.38359   -126.86674    -55.51509   -200.08461   -301.45589   -239.20462   -350.56456   -301.22368   -258.07186   -306.51277   -338.72328   -497.71794   -395.41434   -472.47089   -391.49698   -313.97699   -406.40766   -327.20533   -430.38555   -244.67149   -408.48894   -399.28466    -93.36263   -287.14799   -107.27072   -296.88056   -124.31311   -300.38892   -128.02221    -90.03623   -141.58584      80.3119     26.01565   -104.99658   -210.10649    -30.27823    -225.5569   -341.87422   -215.57863   -244.03378   -324.41983   -243.90662   -303.68168   -126.20613   -243.06868   -249.38273   -141.31215   -228.94328    -57.20513   -226.20262   -204.90859   -213.93988    -67.16661    -17.39501    -99.50261    -75.78303   -261.61271   -216.60542    -68.55085   -286.07538   -200.76959   -234.01506   -213.38963   -203.62937     -6.61223    -332.6287   -249.06688   -183.92303   -114.25714   -171.03098   -189.20941    104.16497    -15.84943    153.10646    185.01232    196.92026      133.764    167.87437     24.72541     72.97939    125.37401     77.53758     62.69598    -31.85927    157.17363    -73.35265    -87.51658     27.00922     36.60791    -74.38604   -110.23956      9.03185     74.82561     -93.4044    -58.76754   -293.50729   -130.00006   -282.79137     61.14706   -121.78703   -219.00616    -60.80735    -72.04203    -183.8997   -218.86005   -167.38339   -176.39975   -211.33819   -265.04853     -516.367   -128.98684   -239.54794   -464.38456   -612.51024   -361.88599   -283.02216    -449.1964   -100.54836   -386.11013    -99.93185   -202.06954   -425.49666   -128.43781   -485.85186   -521.72595   -241.79769   -372.94098     -59.8938   -487.96839   -270.23428   -291.48839   -280.09322    -33.18805   -298.43061   -394.60655   -284.77268    -91.99382   -185.10847   -442.20713    -483.6873   -598.08592   -814.68239   -275.29006   -428.38977   -823.75832   -770.28981   -514.63264   -712.49582   -516.15795   -593.07618   -569.09953   -645.41616   -732.00787   -536.20977   -727.04739   -771.37451   -702.89618   -757.09806   -880.65356   -947.72713   -964.61382   -827.52688  -1148.18466   -874.46397   -872.32323   -991.70518   -1158.5158  -1230.60926  -1306.07989   -965.50994  -1351.40729   -953.61548  -1052.50945   -939.30529  -1368.29369  -1021.92907  -1043.55739   -962.31175  -1152.14191   -971.17363  -1124.98534   -1322.4187  -1151.13016  -1113.12664  -1483.05452     -826.291  -1149.48051   -850.21209  -1375.75278  -1228.81744  -1228.43251   -1376.0129  -1186.76014  -1225.92848  -1305.42677  -1339.33654  -1512.73231  -1283.03995    -907.6853   -791.86572   -1383.1008  -1702.39478  -1246.91599   -856.61812   -297.67913   -863.29154    -888.1015   -1637.5175          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        0.50   -326.13648    -80.58932   -225.55258   -303.39982   -306.24162   -195.57466   -346.33747   -324.61794   -300.84961   -236.05841   -199.83612   -331.56031   -317.71208   -380.79346   -446.88088   -274.34311   -327.70902   -460.50164   -349.03791   -496.82227   -181.04019   -284.96119     -341.938   -280.09279    -27.07764    291.16855   1440.13877   1738.15783   1497.96798     672.0595   -125.95261   -426.23369   -467.20291   -300.10156   -333.46222   -367.59254   -425.61288   -395.52571    -332.2149   -508.05097   -443.04931   -403.53059   -349.55806   -171.25481   -350.95332   -406.74543   -359.37883   -374.97283   -315.80683   -375.28584   -284.74602   -352.87447   -412.99647   -259.24105   -239.96939   -145.82737    228.55839   1136.32881   1457.10816   1246.77265    736.62129    -57.72079   -188.04911   -391.97488   -251.67508   -338.86085   -384.65598    -260.4775   -373.48858   -286.71235     334.6925    989.77938   1432.22338   1504.58672    945.23599    720.70487    802.65692    555.49471    101.00293   -134.08115   -456.45373   -536.69504   -327.41301   -393.53869   -197.55667    -356.5445   -304.60134   -192.53034   -268.29949    -132.1331     64.71123     90.25304     -4.54058    -28.49204    -82.48989   -212.17721   -420.71781   -387.83509   -548.87937    -373.9227   -290.03112    -162.8911      2.50891     18.97348    -36.86101   -253.01439   -255.08988    -445.2807   -310.21956   -368.12887   -278.49483   -458.73207   -402.47876   -414.35272   -150.70485   -153.05158     89.82789    458.68056    375.55425    209.62439     17.48414    -29.43622   -159.82625     20.50646    191.91854    546.84512    683.58429    597.03416     368.5269    278.77237    -54.14672   -283.25973   -231.80253   -244.21551   -230.03936   -119.67668    237.58876    867.15627   1201.24267   1995.48129    2016.8025   1676.16026   1088.24475    506.01615    -61.13912    -85.97269   -168.16664   -266.07763   -279.45612   -317.53569   -343.02536   -244.01851   -265.45538   -364.22218   -260.64259   -439.06674    -61.03814   -395.51584   -202.54754     -180.831   -233.97993     -18.4971   -164.71066    -179.6696    -40.17755    103.27646      3.66857     -1.39187     50.04485   -168.38245   -262.02151   -466.55319    -269.7575    -162.3664   -232.04473   -227.64648     -0.62298   -164.22883    -89.01282   -267.60839   -228.73058   -331.36328   -257.46993   -265.26693   -307.15536   -351.10801   -412.94649   -339.22766   -319.01527   -282.45912   -134.11293   -228.49097    -76.47705    -84.78748    -50.84582    -83.31423    241.95524     39.15751    156.38497     130.1269    202.99507    170.84129    112.77673    -91.50037    -67.94689     65.98945   -115.58091    -11.94282     64.84668    233.00883      8.44212      4.74302    -57.33908   -188.60389    -35.38302   -308.54925   -346.73418    -391.9773   -412.38038   -502.87319   -314.68803    -417.0806   -396.43264   -251.17698   -211.38355   -104.98384    -85.69772      24.8927    -10.83498    -31.25986    153.51903     41.49845     55.86608    -70.14279   -315.29261     -8.99068     35.83993     64.01105    245.27831    171.14115     -96.5528   -107.24408   -122.16919   -168.75578    -216.6576   -258.25337   -470.73394   -200.37722   -410.28813   -402.07078   -378.81532   -238.69748   -195.90297   -243.39454    -28.70414   -106.22026   -162.18037    -16.15751    -15.70225    204.16083      200.037    295.26866     390.1564    587.63897    481.82565    523.27689    577.45804    622.86596    470.74798    321.95649    254.09452    166.20281    173.83294   -179.45791   -123.34324   -246.77772   -137.95671    -98.12512   -222.75221    -17.84667       0.2767     30.32334     22.98208     34.04915     17.42444   -170.47832   -168.44105     16.43979   -254.40278   -285.67624   -264.54907   -252.77255   -352.41949   -327.42305   -233.27358   -345.79149   -525.45664   -514.44395   -284.66188   -384.12856   -232.23232   -355.18596   -398.52158   -182.97053   -248.85529    -291.5755    -289.0645   -252.35861   -379.85069    -64.21474   -172.02393   -336.17021    -47.41826     31.74016    -29.73699      9.31295     57.04287     41.60483   -299.77916     -1.03722    -43.34035    -71.60808    -215.9478   -174.12212   -181.91317   -276.01362    -301.8754   -258.44829   -322.96722   -395.98707    -94.68422    -63.32412     31.97776    -77.80196    -110.1966     16.36389    -76.53945   -252.41464   -282.75081   -149.02275   -300.77008   -201.46497   -135.53223   -301.63408   -243.16378   -241.38513   -107.40503   -123.60442    -56.65543   -295.47228    -93.37714   -223.15306   -125.12698    -96.38991    -32.33043     34.51003    362.32924     13.92096      8.74113    180.75249     24.49565   -117.87024    258.62137    -98.95143    124.42744    -12.53151    152.83181    120.00578   -142.49699     73.36353      54.2289   -180.39234    -89.24835   -136.18551   -202.66827    -37.57978   -277.65528   -335.49497     -8.07477      7.45024   -202.46155   -242.77572   -356.93137   -293.44639   -104.77312   -157.67613   -153.82668    -95.43145    -96.16784   -457.64196   -152.91133   -313.45868   -140.16027    -171.9814   -411.25547   -328.40597   -377.96462   -113.91794   -530.16131   -436.43533   -230.28179   -276.49524   -183.87929   -375.76264   -320.34588   -208.95411   -188.46006    -402.0752   -201.31923    -349.7339   -189.94979   -272.96164    -43.21295   -435.49488   -352.80737    -30.17961    -154.8572    -464.3007   -201.47675   -324.86248   -377.34911   -220.31827   -512.32353   -402.74056   -361.78636   -590.86318   -509.92072   -357.95805   -729.76965   -621.96227   -452.68006   -655.85156   -703.92296   -996.22251   -629.92621   -796.38971   -461.45591    -727.3629   -622.01484   -820.34568   -922.29915   -546.47976   -873.09039   -936.88323   -763.86346   -908.77561   -662.69412    -697.5879  -1105.08832    -943.6582  -1249.80983  -1465.01033  -1015.90949   -963.05229  -1314.27142   -834.66339   -954.37677   -929.35997   -961.02535  -1147.92529   -743.93876   -918.63671   -895.48666  -1305.84355   -927.70891   -872.99845    -929.8533  -1337.81234    -987.5241  -1294.71811  -1419.41163  -1054.96192  -1079.65013  -1333.45527  -1392.63131  -1400.13063   -1246.4185  -1231.52514  -1154.46173  -1157.14383   -996.75459  -1099.15609   -455.91378  -1494.47512  -1543.99938    -214.2207   -627.08279   -118.46928  -1159.21595  -1163.60931   -974.46469          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        1.00   -205.40746    152.16924   -218.47652   -220.03114   -462.57446   -244.52851    -553.7376   -232.13544   -290.07787   -170.20621   -189.42334    -419.7909    -372.9987   -345.89062   -493.82319   -243.30251   -318.97504   -373.86592   -264.23968   -473.72943   -281.11641   -149.35699   -338.07547   -405.25794    -79.00089    173.67561   1247.64393   1802.68828   1653.26664    610.08628    -273.3988   -371.66497   -468.47403   -375.85695   -326.59375   -443.38853   -396.44283   -363.70547   -360.77503   -515.48931   -123.35108   -371.43667   -559.78628   -315.35537   -356.81412   -294.42058   -272.52633   -300.37583   -446.55681   -283.91079   -381.99743    -376.4836    -421.9319   -488.44024   -270.13146    -74.39553    193.55401    1130.7819   1456.77192   1208.25336    587.34386      48.6334     -77.7453   -389.88762   -285.94952   -250.24947   -365.83543   -334.31248   -259.02984     23.08067    546.30045   1206.71337    1520.0357    1482.8124    979.76877    773.07392    533.18278    337.68162     65.52652   -307.47696   -480.52841   -567.59281   -567.31212   -364.24694   -166.79432   -260.76976   -379.86996   -291.41565   -282.54632   -241.75966    -95.52758    -44.01168    176.84852    101.59757    -383.8294   -304.77887    -359.1719   -307.30372   -427.26093   -502.97483    -188.2147      -25.845    158.95246      -8.8442   -110.45199   -278.76392   -194.20566   -360.11999    -300.3519   -301.82978   -268.13521   -330.65019    -297.4163   -257.09405    -63.53904     30.54137    141.32702    175.10854    342.87973    282.34896     22.61494    -32.32851   -268.96835    -45.06549    210.13675    462.45576     479.3405    538.57304    435.28771    127.17597   -120.95531   -237.82379   -169.81812   -154.80916   -286.51339   -166.30068    214.63127    868.24435   1501.26688   1963.24252    2110.4758   1562.67442     965.4637    430.49546     -2.43725   -159.50993   -269.61843    -12.68375   -353.90708    -216.5034   -254.34831   -273.43506   -425.91802    -363.4577    -95.47884   -278.68408   -203.64683    -211.0451   -264.38178    -10.86242     -5.89429   -190.43014     -158.263    -65.54726   -126.24414     12.88456     35.33164     38.37267    -18.20434   -184.88953   -203.65388   -176.66524   -218.06533   -329.61523   -413.94314   -168.94981    -164.9544    -98.42518   -260.90737   -252.96514   -339.45551   -377.62989   -293.65426   -381.28418   -394.16904   -363.41741   -278.03155   -489.89629   -241.69772   -258.63603   -296.62251   -141.18765    -70.38159   -315.81264     58.83412    -47.11481      112.668    260.70234    288.63418    109.57202    223.13155     64.65176    120.56825     70.77797   -167.60192   -237.02439   -107.38856     -16.5841    111.50375     -99.5189     89.45022     95.08645    -69.52173    -59.52623   -157.37771   -208.65442   -279.98268   -331.79016   -401.10293   -307.48477   -219.45173   -314.50648   -275.54283   -133.72793   -124.80128    -21.57078    -69.22208    -63.86386    -48.67351      0.17177     14.88728     -1.71483    -67.59404   -143.34061      6.12002     99.34971    205.21616     11.95613     91.90655    -17.67558     61.67325   -116.18557     12.67797    -45.48971    -65.48983   -312.76272   -217.20002   -247.38617   -251.30973    -361.3382   -302.78872    -226.0408   -203.41677   -126.99438   -107.19191   -104.55125      18.3859     47.66418    -63.27441    117.30195    232.51826    438.96673    593.95983    409.40343    615.87156    642.11497    555.09939    676.12258    583.35754    470.93399    188.22049     36.28949     76.30188    108.10941   -230.87334    -65.78107   -160.03488    -197.6599   -204.60111      76.9031    -79.15571     31.62407    -75.42598     52.85613    -17.34872      22.2158     96.22722    -86.01079    -438.2117   -154.00616   -195.32471   -287.73862    -219.3743   -381.54726    -394.9421   -386.32794   -296.82623   -440.80394   -346.31678    -308.5616   -408.68769   -191.53377   -322.14825   -131.82815   -215.01074   -130.19286   -228.90115   -201.72007   -350.92454   -144.64168    -90.26623      38.6608   -238.23455    -62.21561    -47.63612    -79.72057    -51.55013   -200.15895   -101.80008     -86.8975    -91.63972   -206.60215   -222.11519   -183.46074   -146.12404   -248.55069   -157.31456   -117.64249   -119.13614    -183.6643   -134.59436   -218.18174    -168.0565    -36.98789   -352.07283   -166.94318    -85.57811   -145.81073   -141.81984      8.90045   -206.00777   -229.15869   -163.66606   -140.14446   -358.21237   -225.67785   -504.16057   -222.35301   -365.69433    -17.79627   -164.84729    -43.99902    -61.20153    -98.89092    -42.29513     78.01822    -93.94113    125.13374     11.19454    192.40972    121.35003     85.72004    -34.47428    116.60209    268.85021    379.68025    141.95046     41.96358    164.46821   -151.19363    -36.00373   -185.82949     34.80745    166.54905   -171.38167    294.91362   -312.47433   -209.14285     76.62789    -13.66449    -356.7854    102.73618   -224.97742   -280.74056   -156.48527     43.07537   -236.46368   -309.51573   -384.75977   -298.62972   -205.29453    -45.83979   -439.10726   -349.34984    -150.2408    -41.87047   -482.64249    -99.94254   -385.18097   -348.57908   -171.00595    -157.9971   -292.15791    -354.1589   -241.34534   -314.60485   -305.53688   -630.03647   -292.82432   -219.86856   -309.18041    -80.04269   -407.37282   -263.73501    -238.0829   -438.95452   -504.42071   -148.88792   -333.61656   -416.46849   -444.46868   -281.58128   -467.49561   -597.21313   -540.10219   -534.02465   -384.59576   -583.84822   -420.01849   -612.43782   -622.35502   -540.07513   -830.24402   -975.57519   -756.78404   -669.16569   -678.73333    -616.0816   -773.54685   -571.42685   -752.20402   -989.58031   -859.46207    -656.8685   -758.17342   -562.35507   -828.96881  -1353.73742   -946.00003  -1055.79098    -655.3091   -913.01202   -869.44823  -1220.07201  -1041.00034  -1230.05091   -986.73506   -920.14307  -1031.49444   -998.83377   -834.80764  -1167.89756  -1276.35335  -1092.01908  -1117.29547   -553.83232  -1034.30971  -1068.10051  -1012.23213   -975.42587   -818.23189  -1404.84472   -690.05111  -1021.48515   -624.42272   -839.55367   -652.75252  -1179.04869  -1222.38666   -872.54253  -1231.84851  -1401.53547  -1332.14617   -873.19698   -956.80895   -724.00211  -1184.34615  -1056.69114   -537.44638   -1117.8111   -586.16522          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        1.50   -113.99438     165.2168   -258.71845   -316.88427   -380.51916   -334.85083   -513.62525   -188.67716   -178.02918   -266.68031   -264.21242     -391.386   -405.21328   -277.18272   -538.97383   -322.10198   -304.58433   -248.20704   -182.86493   -448.68156   -373.82761   -159.04865   -344.38224   -380.57188    -30.98547    284.31465   1306.93976   1912.32161   1564.35879    450.82079   -255.86256   -331.19697   -391.50027   -323.40129   -417.49708   -158.14658   -176.74155   -203.02971   -154.97008   -396.11739   -274.46895   -319.70824   -358.67628   -333.79562   -416.15782   -384.98853   -346.54314   -248.43808   -515.31362   -269.09014   -411.80091   -424.80127   -363.33407   -451.86538   -211.36099   -211.84772    235.39452   1145.03145   1271.49628    1360.4295    560.26703     50.78624   -214.78057   -349.45382   -184.59016   -176.53963   -143.46276   -215.99356   -359.76082     -53.8156    304.56043    992.28944   1372.37988   1378.63418    910.62537    804.35446    544.71262     410.3534      6.12285   -275.59937   -420.83574   -488.75107    -346.5478   -624.94082   -267.19457   -201.29579   -329.58275   -323.31255   -351.90878   -137.72058   -112.58045     84.74622    -70.24014      0.05427    -464.7637   -346.55861   -368.66914   -412.35924   -465.06323   -313.32095     93.13362     98.91539    -43.21059      9.18272    -90.02762    -122.0327   -234.34608   -407.97313   -196.03507   -501.31428    -300.1362   -233.01754    -285.3896   -292.18426    -78.83289    -45.01336    -41.84389    393.72786     354.8123    208.69495    -23.88662    -61.62901   -176.89616    101.50616    150.46575    409.79894    695.82062    531.52114    429.64351     90.09785    -81.39874   -183.51951   -282.12009   -251.28917   -212.37641   -122.99123    187.95717    850.87087   1517.92075   1833.07035   1988.12145   1806.96747   1190.40809    493.36219    103.28161    -34.03227   -216.42347   -180.23977   -220.79486   -300.61669   -200.22118   -400.77944   -250.38385   -351.31435   -301.85524   -339.63146     -248.914   -174.23724   -111.03655    -93.43239     10.50265    -35.69638   -225.44453    -49.58455    -89.75519    -90.29706    -45.55484       1.3156    -31.33469   -124.47323   -219.61318   -313.44767   -236.24158   -297.75147   -294.28345   -177.11192   -259.04749   -173.31059   -119.48782    -122.4464   -185.00163   -168.06939   -258.83381   -284.23625   -438.92531   -604.02739   -256.07263   -377.23181   -429.98392   -327.97648    -174.7151   -277.31348   -125.52352    -27.03701   -256.00568    -83.45128     -11.5968    210.43927    190.71634      188.671    153.89274    104.39925    152.47234    -45.73454    -73.34234     21.20337     -6.26514     -33.5459    -50.48338    -99.59901     56.25863    261.32306    -32.92718    -74.07573    -99.08159    -348.3815   -244.17674    -272.0133   -335.93504   -478.59468   -425.54509   -239.63295   -315.14412   -289.20214    -287.7566    -26.10855    -94.16609     83.50121    -10.82591    120.37389    -32.22443     84.70626     -6.68482    -66.14463    166.00316     80.74473    149.02025    226.17679     84.13878    -19.87575     138.2591    -56.20856    -184.4391    -27.59458   -217.96386   -217.53461   -152.00574   -268.82886   -180.70503   -281.87297   -287.60555    -50.92329   -277.97166   -139.25881     59.36897   -110.19188    -49.34622    187.98878    109.35481    131.40246     240.8338    386.42853    401.45859    467.99927    442.66476    683.62887    783.61707    399.26105     455.3601    461.19116    360.07903    200.24592    -68.38853   -285.89256   -249.54146   -222.64508   -107.21214    -44.53823       9.5852    -18.98239     -6.55584     -7.44176    193.69913      25.1204   -108.09849     76.49141    -178.8045   -121.73913   -209.88182   -251.41001   -150.76999   -213.75627   -224.65486   -251.94883   -351.10953   -442.20592   -404.61936   -388.42964    -353.5906   -231.68852   -379.69116    -301.3073   -325.99319   -305.65249   -228.28775   -180.03316   -286.59716   -287.91866   -205.08259    -83.84152   -131.88784   -284.13124   -142.38137   -144.30409    -81.62356     -16.6399    101.98875    -19.14747   -127.39002   -327.66004   -170.56103    -26.48853   -289.90013   -200.00899   -148.56669    -98.96605   -188.94088    -47.57449    -313.1181   -288.15139    -162.2888    -62.51734   -268.53867   -137.26462   -173.83638   -150.59763    -68.46469   -156.46043   -111.99049    -86.98349   -243.87943    -56.97738   -197.05255   -109.44518   -215.46915   -153.24542   -212.87878    -84.24126   -150.55973   -139.34371     69.33481    -114.4612     56.83498   -127.86183    134.68308    160.06064    206.36149    146.49943    420.78511    111.24693    300.07322     157.7912     -6.77142    209.06497    201.98039     68.20508     95.09305    203.68384   -125.06159    125.79821   -226.33466   -189.12755    131.41773   -157.81919    -81.74553    -98.74621     -67.5086    -75.04129   -256.34329   -119.67372    105.47413   -135.79548   -104.20413    -58.57594    -136.7003   -146.73267   -119.30132   -235.21799    -34.66652    -110.4521   -424.07575      1.00152    -17.99412   -168.91891   -205.99869   -378.42921   -459.38006   -354.94511   -318.08331   -209.67899    -70.03773     -104.822   -458.50196   -101.08192   -204.73896   -265.09472    -161.9225    -93.33469   -228.50756   -324.88735    -242.0909   -297.49755   -357.51063    -81.20968     -237.508   -521.54955   -390.34498   -326.61549   -301.86428   -193.67507   -448.74143   -202.56264   -374.24114   -454.45728   -440.08141   -225.44515   -646.91863    -357.7098    -467.8925    -335.7294   -352.60646    -708.6776   -773.42654   -839.78662   -771.20366    -744.2291   -530.68931   -769.72667   -644.92824   -924.56681  -1062.63158   -683.87325  -1253.63322   -885.01336    -981.6477   -859.61759   -854.86706  -1301.58546  -1102.80764   -732.29054   -977.74572  -1209.84192   -948.81306   -824.39137  -1307.64727  -1127.50939   -960.56018   -884.50613  -1250.51985  -1267.54487   -970.34594  -1230.86527   -773.36212  -1020.53677   -827.77271   -888.51094  -1085.20639   -598.26889   -640.40849   -791.08503   -364.34022  -1220.76227   -985.72729  -1284.47985    -711.9889  -1106.32123  -1241.14652   -963.34109   -680.46533   -746.24607    -1335.786  -1585.52009   -918.13231   -1119.4956   -708.44298   -273.62458    -864.1511   -245.64294    -44.04196  -1291.16862   -731.38819          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        2.00    -46.90731     55.93358   -311.89085    -448.0342    -280.1975   -422.44389    -469.9188   -146.23284    -92.14323   -381.57571   -324.14862   -413.62855   -386.52832   -418.12882   -366.49767     -351.691   -240.35695   -280.76332   -389.00857    -470.7939   -282.79942     -356.738   -343.59518   -255.00059     52.12179    407.80537   1440.86655   1989.07728   1490.66731    464.68007     -7.76897   -230.14556   -443.92948   -345.83211   -455.99127   -305.62019   -553.15475   -242.73024    -501.4395   -299.63438   -357.06929   -273.50268   -383.00105   -148.05991   -353.60275   -435.62885   -374.48503   -315.69318    -446.9427    -215.4626    -280.8472   -529.16601   -320.50379   -371.03059   -341.84553    -145.8828    290.26264    868.06128   1499.83299   1566.19144    761.97184    -76.87061   -291.63672   -404.02156   -417.19628   -278.10632    -40.84677    -151.7007   -331.10688      178.138    389.62688    890.82938   1398.55693   1517.35232    790.55885    670.40705      661.481    526.96465    100.02182   -179.68751    -463.4751   -381.14984   -370.06366     -258.664   -221.56346   -291.59682   -225.16222   -280.24072   -237.11765   -217.87962    -77.71655    206.21951     -148.776    -93.40536   -126.97762    141.41145   -187.39075   -497.85052   -412.54786   -295.47828     58.70648    114.63086    160.42642     39.13929     -83.7233   -228.63875   -412.21368   -347.05846   -182.77791   -535.15769   -407.03097    -363.0728    -304.4146   -366.37096   -183.05886   -127.64541    218.14297    196.43034    276.10678    191.45929    -66.99429   -116.84005   -248.02146     43.98177    257.15009    510.48935    546.53661     530.7699    450.26322     -9.52822    -13.74827   -236.65794   -226.61037   -324.56552   -341.60264    -21.94768    356.35522    865.50788   1467.92805   1985.96567   1966.27261   1564.81829    950.10055    558.54255    119.95637   -165.68233   -133.00126   -175.52772   -258.16056   -213.56082   -230.02169   -298.76574   -421.68957    -400.8782    -207.4436   -232.11014   -256.49177   -162.34244    -98.01938   -232.66241   -163.09507    -17.23375    -99.84801   -195.08235   -171.90555    -58.94673    116.34448     -1.23879     79.45382    -31.58441    -113.3124    -259.3632   -224.74551   -307.29611   -302.28031   -252.85059   -273.91981   -214.03401   -177.71628     -21.2107    -339.9871   -252.39437    -228.4282    -259.1743   -349.65687    -157.3106   -483.17335   -384.80764   -231.88329   -274.28648   -246.83825    -84.33043    -78.13031    -66.97769     39.02706    -11.48666    181.77794     11.55607     168.6934    234.21983     57.45149    271.00001     37.35091   -150.01721     98.70749   -182.55726     -9.29644     12.43658     30.64434    -21.36385    -79.21997    132.49699     20.33753   -152.59728   -183.77864   -198.98111    -193.4562   -176.11034   -222.46905   -349.56669   -192.78257    -261.6977   -507.00637   -136.23274   -157.70221   -295.63996   -129.94574     31.27131     63.21082     -7.38151     77.10339     46.19465    105.48229     46.92967    -187.1933    -31.97841    147.41272    172.46489    342.43333     183.0902     49.87816    -12.16905     27.83704     -95.9674   -276.41236   -169.88265    -171.9334   -113.66122   -231.06132   -301.36111   -266.87305   -229.21402   -204.60293   -127.73266    -77.95706     58.28201    -39.49587     56.17255    209.71016     27.63031     91.45475    358.42316    544.85329    658.30872    766.27137    535.25802    842.99972    589.47069    697.04755    451.88229    149.53349    142.55945    -58.56819    233.77831   -220.57679   -170.45938   -219.37761   -152.37698    -82.74914    -91.41411    154.60814    222.17508     43.83011     -58.1618     86.86381     76.38081    -21.75332    -79.05605   -126.83464   -171.74124    -13.42409   -413.31339   -282.77447   -302.39712   -379.81214   -365.10533   -254.88999   -266.23104   -440.11851   -292.66085   -282.62536   -370.08955    -34.04567   -308.04013   -252.49533   -296.94455   -126.02627     27.69794   -134.56966   -101.17594    -91.31561    -36.78548   -279.11729    -98.23271     -78.3669   -149.76559     35.32979    -18.57309    -37.48182    -76.39996    -99.03794     23.08893   -161.49934   -441.06357   -172.18946   -333.13666   -150.75801    -248.6196    -56.84918    -95.43548    -65.92055   -198.29064      3.61687   -207.61499   -124.95917   -343.24299     71.48494   -225.62742   -135.90027   -103.10569   -191.46621    -64.95979    -69.57565    -86.39755    -17.35022   -190.72505   -152.58486   -186.01785   -172.32176    -82.78934     -36.8464    -51.72784     51.80231    -55.64798     25.78848    162.54101    -13.75281    205.29539    159.81105    330.65081    258.71145     320.7125     66.47371    288.29641     49.23279     59.43201     61.70201    102.67861   -147.86605    -43.93288   -117.02635   -108.32472    113.66183   -201.00339    -26.01841      30.8223    -41.66881   -213.69649     76.68726   -153.69065    -51.76485   -137.04124   -296.71391    -212.5562   -218.96638   -378.82907     48.34486    -44.08664   -301.72536   -178.83696   -269.67327   -301.42297    -617.6307    -412.0406   -183.88732   -152.06203   -232.27631   -362.76365   -161.45732   -477.76711   -357.17411   -177.94491   -425.27466   -188.99745   -256.96078     16.30029   -260.29345   -314.87699   -271.14495    -268.1882    -24.06878   -339.74686    -330.4702   -346.08772     74.52132   -318.39929   -547.87636   -156.05531   -148.08323   -151.43254     -383.702   -445.36194   -413.48579   -534.88476   -264.53056   -475.60393    -602.4274   -299.47919   -651.90436   -460.52547   -579.79522   -723.10938   -553.03886   -519.45477     -541.827    -566.4008    -683.9487   -560.96141   -856.15016   -724.84635   -839.24631   -969.64898   -770.88029   -648.19058  -1006.24532  -1044.52877   -876.43117  -1165.74229  -1292.68498  -1011.75177   -998.68952   -993.98898  -1102.47743   -960.44627   -973.23169   -998.97685   -869.24266  -1124.82781  -1125.75896   -692.24812  -1443.01049  -1175.65783  -1004.20804   -925.87612  -1216.80889      -756.43  -1095.40677  -1224.29919  -1263.65617  -1198.55682   -702.37503  -1071.94646  -1374.79723   -916.61618  -1539.48807   -420.94724   -785.10599   -650.60879  -1125.49482    -54.77518  -1356.16098    -941.0138  -1739.19356    -831.7863  -1616.16396  -1201.00961   -374.75747  -1183.92947  -1120.95194   -810.10059  -1523.96717          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        2.50     22.39098    -55.46605   -339.26654   -493.55007   -275.22395   -346.98492   -432.64119   -191.53453   -131.50542   -533.67003   -357.95746   -478.54867    -334.1141    -610.0821    -196.0843   -353.65415   -298.84284   -358.90265   -450.21873   -431.29892   -399.38004   -330.64312   -284.59885   -356.33552    -20.72631    304.79166   1448.24884   2008.45775   1492.68302    552.15298     78.11497   -197.69444   -323.64724   -329.75927   -144.52145   -431.77024    -249.1034   -501.60423     -410.368   -490.17204   -347.16133   -214.17285   -390.57177   -263.30624   -229.10549    -287.2925   -408.23059   -380.09436    -75.61996   -364.40966   -350.94148   -449.61243   -399.74673   -406.81363   -394.63342    -79.88464    234.60389    961.74595   1486.41299   1645.91276    829.85957    101.12261   -270.59087   -177.93247    -230.9592   -361.16862   -242.57421   -303.72888   -174.30499       31.085    330.22026   1146.58087   1393.20219   1516.43009    938.68401    489.59301    659.61901    489.49654     257.8882   -142.85842   -407.79268   -266.71927   -176.75119    -66.28482   -493.65314   -267.89073   -315.27085   -364.02855   -227.30648   -160.63152   -122.32566    -90.52788     97.00417    -24.93012   -306.68396    -324.4648   -292.99905   -346.58324   -481.91126   -389.54826   -214.77906   -232.94197     84.08348     -13.9137    -108.3686   -305.91625   -309.49001   -243.06351   -413.30087    -336.1389   -347.25344   -317.76542   -275.94587   -324.93783   -255.86327     -7.51141    355.90734    400.37983    186.89221    316.19085   -166.25017       -13.03   -316.48621     17.63315    254.63366    513.99374    503.25636    623.43862    415.67131    164.21829   -166.12506   -163.46166   -155.01983    -169.8035   -122.51336    -65.77949     97.50011    820.72779   1666.17531    1894.1991   2187.47203   1813.65409   1183.81647    627.47712     78.20125    -251.5145    -34.61149     -55.2592   -343.93414   -246.96615   -403.95782   -334.09992   -359.01516   -270.86041    -288.2778   -242.84933    -218.0632    -165.2954   -216.90461   -199.11633    -51.15191    -68.75478     -3.81467     -94.6126   -242.88775    -62.86725     10.38254    199.84504     -6.53576    -41.27522   -106.93677   -194.91673   -244.54505   -350.75921   -391.59702   -484.94225   -254.89818   -119.19073   -299.84572   -186.48658   -187.49718   -178.58884   -329.91211    -437.0509   -328.27332   -296.02346   -435.13148   -292.30799    -190.4483   -185.72804    -305.8073   -215.36146    -53.06198      2.15466     45.93054   -134.93086    125.43157     -8.87831     85.72227    255.44108     399.6195    318.48865      85.3424     -2.29103    120.35737    107.79391    -45.98385      -9.2142     90.58275     31.40736     34.22398     75.25181    -96.36315     23.83612   -110.46549    -222.3813   -214.62763   -216.95491   -228.86044   -363.17947   -332.80238   -420.39064   -254.06383    -97.88266   -160.39747    -77.18831     59.11657     29.08718     22.58161    167.28983     53.34868      3.71821     127.8924    -63.32698    -36.34366    -67.75383    112.32717     23.00144      206.687    228.41919    -51.92018    -50.58359   -222.56752     -42.4844   -238.66396    -79.57487   -295.68067   -413.63125   -134.55482    -114.5423   -375.46298   -304.61282   -234.29734   -204.99292   -111.03152    -58.84654     55.54669    281.28381    275.35862    286.19718    287.37435    390.30288    402.92327    473.70389    601.56209    615.55667    575.67913    651.48588    460.44052    244.32295    318.56197    313.18654    159.85916   -151.43658     41.45141   -121.54843     65.22252    -41.09883     94.65479    120.27483   -223.16186   -229.15216    102.39857     37.48518      8.34897     97.13466   -105.18893   -106.56133    -116.6247   -168.43772   -410.63405   -212.18848   -365.46476   -168.34655   -285.41142   -196.41582    -483.9322   -244.28694   -280.74767   -299.40393   -317.06449   -237.82175    -312.7963   -329.61461   -187.42887      -296.74   -144.15976    -59.21179   -212.85268   -305.52999   -145.22107    -65.60866   -126.47871   -174.99288     71.04724     79.58644    -88.26525    -93.20922   -191.55497     -26.8031   -121.72186    -36.55115   -250.71667   -224.59574   -115.69915   -147.01384   -282.57569   -251.68613   -199.01247    103.34349   -118.37672   -118.24845   -210.74963    -69.67366    164.58396    -51.17802   -107.92001   -251.48056     26.31379   -271.78862    -57.10281   -253.49435   -255.93385    -37.86539   -204.71224   -140.33191   -201.83635   -269.83895   -158.23071   -189.41162     35.39842   -183.01969    -85.93058     -69.7322    -54.40656    225.41102     98.85351    207.05863    188.77811    159.02144     26.45285    125.17827    118.79391    215.10254     129.6591    153.70828     218.7989      70.5849     52.74618     27.80546   -179.14574     70.63298     29.55762     59.12516   -220.09731      8.77384    -102.2103     67.59408     30.89646    -60.79076     51.27365   -309.57955    -30.35247   -326.90689    -230.6831   -134.93463    -92.97621   -219.88404     -62.7471   -198.42545   -291.76697   -264.09074     29.30209   -299.21545   -311.47917   -180.70584   -249.45383   -377.38894   -411.49707    -15.74759   -280.78422   -378.89042    -31.47649   -254.94433     -28.7521   -167.77508   -244.38719   -461.19908   -551.24921   -348.25381   -268.39669   -399.86229      -434.71   -165.92335   -209.70719   -495.82602   -289.32213    -74.09729   -439.39116   -345.77935   -337.42822   -485.43315   -354.10448   -341.96787   -469.30037   -227.10117   -523.08589   -413.55271   -283.52728   -419.62172   -623.88173   -842.55955   -516.55636   -612.39008   -572.45821   -774.99073   -783.27842    -662.4617   -784.46394   -921.80299   -688.23235    -958.6461   -840.86484   -1022.8945   -830.94056   -864.48621   -781.37141   -871.37936   -965.63246   -730.30373  -1122.89095   -956.57832  -1038.83559  -1152.23874  -1225.95385   -771.02499   -952.03599   -844.05243   -616.00298  -1073.59539  -1128.20941   -831.20285  -1100.99926    -786.4378   -981.05648  -1025.03682  -1033.13228   -675.06144   -539.02873   -972.94579  -1079.39998  -1071.47218  -1215.88151    -999.1846   -878.48268  -1343.11321   -610.90439  -1257.27586   -871.86192  -1440.23378  -1170.00319   -871.72054   -651.58852  -1529.55421  -1080.48431  -1235.87502    -99.60478    -863.3295   -763.47798  -1305.78486  -1219.83704          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        3.00     -10.1333   -162.96944   -249.63421   -388.24273   -343.45291   -209.79462   -401.32609   -238.32134   -189.16056   -586.00208   -284.89551    -414.0024   -193.89011   -421.73861   -231.44306   -308.44095   -544.18714    -458.3136   -461.13595    -351.2086   -486.08232   -318.23192   -141.59658   -415.62406     52.18898    402.53964   1339.99603   2018.09622   1447.63423    619.14811     33.89905   -355.69456   -553.17786   -395.66631   -435.72527   -333.86015   -194.74533   -274.81565   -150.99843   -568.98418   -549.18994   -392.37411   -366.56687   -374.25981   -302.24199   -483.98318   -308.07745   -352.80259   -311.38294   -375.69175   -392.15794   -291.13569   -493.34048   -501.87904   -133.81073     55.06527    302.75914   1033.26528   1585.37587   1530.53297    717.71837    207.85488   -243.31642     11.22164    -485.2851   -395.97003   -206.63047   -174.38948   -199.09651   -121.01015    449.53458   1031.49893   1653.37771   1384.45984    826.39141    697.80331    512.27016    484.48849    265.32394   -137.27368   -128.43797   -342.00446    -368.7276   -315.46074   -270.92523   -269.84232   -418.96924   -400.55093   -369.29271   -339.69973    -22.83436     -5.77594     25.20952    155.50552   -304.28512   -449.89399   -328.15802   -227.97683     -382.615    -256.0691   -192.02142     86.49498    171.70219     -36.5293   -260.71247   -128.50094    -16.02818   -293.59325   -240.28239   -316.58326   -394.74426   -332.86141   -254.75742   -396.07688   -343.65755   -114.36566     104.7192    296.72112     433.3531    108.45961      74.9243   -126.86085   -229.47306    -44.36476    -30.55722    537.57596     559.5045    500.86854    585.33191     12.57336   -131.12076   -172.15774   -231.60314   -151.46496    -86.09448   -105.24384     187.4264    833.63634   1319.69897   1855.23403   2130.28539   1676.88572   1184.65831    736.10382    170.27436    -98.04019   -165.59267    -42.10653   -397.72116    -38.80039   -448.57765   -265.21072   -328.49309   -215.98312   -285.95795   -255.57709   -167.51362   -326.03659    -40.84617     15.22169   -293.56446    -214.1457   -216.69708    -42.80918    -69.77398    -88.54804     56.05721    -40.55645   -121.33375      13.5325   -145.83917   -363.46548   -239.35644   -208.16379   -325.69679   -157.08046    -32.09245   -240.19667   -128.59451    -79.99641   -176.41713   -222.98056   -302.08865   -393.32043   -485.78322   -254.84818   -330.14335   -249.16622   -332.89851    -397.1907   -191.44188   -238.66883   -124.08919   -230.11771    -17.78597     22.22762    124.05684     298.9747    224.36115    204.52492     40.62111    192.92835    194.57047    -47.29791    -22.17055    -211.7885     96.51865    -51.07066    175.92668     31.84073    261.53469    138.04739   -111.24132   -135.26679    -50.34463   -285.17809    -275.7358   -350.13125   -224.30439   -248.90608    -147.1852   -304.36961   -166.90921    -287.7596   -195.16601   -294.01857    -25.91742       13.236     41.00673    181.78832      5.71178    105.90672    -87.30949     90.64372    -28.88436    214.86964     37.02267     83.51618    109.16511     68.65613     77.26629     -52.4968      4.71471   -134.34796   -279.72356   -177.45046   -310.36723   -242.22489    -312.9947   -193.01263   -289.70647   -281.09939   -302.14441   -200.29826     -12.9915    -90.99818    -95.18325    154.00536    -22.45615    385.02209    158.35105    565.15609    389.31612    553.64074    662.25591    628.80861    856.30731    504.59657    436.37864    352.25431    212.00907    126.12977     40.73701   -182.15346    -55.49389    -10.88818     -39.9515    -52.98637     84.20309     56.73266    -36.51726    125.65263     -6.37925     68.44891    -45.93906    -84.41271   -187.59764     20.54259   -228.33572   -214.11446   -294.89069   -163.70004   -239.10696   -425.30045   -524.67978   -306.13202   -358.75889   -486.41959   -489.71342   -284.76029   -252.55781   -213.85322   -142.10765   -178.05314   -104.91596    -252.5018   -269.23315    -196.0483   -213.86862   -204.37217   -181.91492    -24.90128    -66.68916    -25.48884   -101.49319    -39.96408   -164.17049   -113.28847    -225.2507    -51.11014   -121.13947   -150.53715   -218.56423     -92.7948   -401.69964   -317.92603   -241.32903   -251.25009   -239.34788   -115.91303    -201.7918    -13.16601     11.68374   -191.83039      -7.8996    -104.1179   -234.25013    -30.66515    -11.29359   -256.00154     54.34524    -16.14289   -181.51817    -299.2823     -6.88534   -143.21999   -271.24388   -139.94207     -84.5405    114.81864   -250.07959   -223.05474    -13.46928     62.49021     47.38225    187.81281    190.50275    127.04116    123.15776      94.1304    286.55796      7.47475    185.16473    265.03728     96.30594    127.88115    286.81425    142.00325     87.58743    178.25977   -174.39386     79.44841     23.10157    -37.45131     35.30139      59.2011   -185.12208   -420.25239   -213.23175    -82.32568     26.94325   -146.81602    -82.78101    -95.38472     21.36469   -193.42216   -300.01773   -406.59923      59.9623   -202.67496     -164.747    203.09926   -354.34236   -295.71072   -269.95573   -450.58508     51.26757     89.71498   -137.15657   -259.26097   -102.93063    -436.5512   -302.48176   -435.33221   -305.02185   -356.10118   -273.53915   -225.03467   -397.91867   -189.29772   -496.82417    -41.20698   -352.81809   -306.60407    -517.4369   -164.24561   -351.20975   -161.53801   -400.89083   -134.43098     -247.294   -139.23238   -701.82145   -405.30012   -584.36205   -545.09922   -414.09663   -485.82328   -479.43237    -358.2614   -641.49098   -686.08878   -722.40276   -521.82252   -748.57027   -574.09186   -635.45019   -685.07637   -928.20197   -695.73539    -845.4495   -777.66374   -828.08799   -881.43585   -846.84059  -1055.42041   -840.54371   -854.90424   -914.79945  -1059.07647  -1057.42997  -1063.45129  -1117.24879  -1053.78571  -1183.24536   -947.45879   -969.38746  -1023.73883   -946.50806   -833.08946   -570.95043   -912.70875   -666.91148  -1065.44623  -1033.54372  -1167.36764  -1014.17716  -1164.28977  -1103.48401   -971.79113  -1219.30975  -1187.09748   -772.50313  -1274.47856   -953.89109  -1134.76425   -1109.4517  -1036.90843  -1274.90493   -934.06055  -1079.05599  -1009.35003  -1122.11775   -950.98802  -1039.10671   -860.54249  -1351.72788  -1116.71533   -719.71385   -632.09869    136.45819          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        3.50    -84.39527   -270.50297   -155.83339   -279.01984   -391.53031    -95.84181   -389.69057   -359.49819   -344.14098   -453.35306   -126.50102   -294.48584    -81.87647   -306.16749   -274.32121   -302.18314   -423.62926   -374.08047    -479.8575   -238.54785   -493.86291   -343.55649    -50.98854   -323.00561    -42.89881    353.55354   1152.71821   2098.71166   1118.17411    709.86341     -2.32652   -447.54631   -554.86289   -261.31263   -419.61305   -223.49888   -305.37035   -141.88911    -289.8601   -391.81935   -325.88615   -400.66364   -588.83625   -419.26327     -92.9774   -448.95708   -187.88311   -347.52644   -502.92092    -257.1322   -423.67583   -594.49839   -209.07188   -487.52466   -113.45782   -142.96425    130.65386    854.36822   1541.15516   1487.39138    686.79545      46.6078   -251.84642   -250.09055   -281.28456   -347.84961    -67.94097   -115.30133   -221.96392    -95.21261    301.04111    906.12555   1607.68651   1212.71588    791.69347    575.32118    526.31965    869.34958     82.64453    -76.51493   -327.65799     -95.6403   -458.83094   -364.78058   -303.38983   -239.94763   -397.81478   -199.49856   -291.53386   -201.40648    -93.97745    118.64854    129.00074    112.95145    -55.50795   -390.65488   -160.53858   -320.71361   -410.40941   -338.07337    -35.44464    112.58776    140.04077    -51.22232     34.74601   -231.95599   -330.56947    -325.3651   -280.10415   -414.40992   -276.68479   -397.03546   -154.60339     -372.425   -336.86332     61.84917     -8.01049    515.07314    341.77942    317.63056     78.36232    -98.49164   -282.13779    -68.43563     304.7851    430.54436    725.36718    531.59084    268.98265     96.64998    -37.35138   -129.80677   -193.36389    -94.17309   -122.34098    -34.01922    225.71512    773.00776   1365.16288   1840.81016   1895.01917   1620.56713   1210.70237    447.44422    180.38876    -17.31912   -150.33241   -241.71671   -280.35826   -157.82904    -243.8349   -427.92485   -336.46968   -221.91222    -409.0484   -242.54304   -255.31841   -282.08332   -158.71754      1.17286     36.00319   -185.32657      0.45388   -213.34041     75.30061   -131.73159     48.56284    124.64563   -186.38873     -46.3395     -5.59003   -206.76616   -260.11625   -160.24592   -270.46898   -299.87977    -161.1196    -67.00988    -148.5181   -221.53609    -264.7366   -235.95676   -337.31029   -315.57748   -386.39738   -301.10419    -339.8426   -433.81571   -311.48418   -391.53642   -246.45834   -213.83965    -224.4352   -140.53687     22.73756     52.12243      6.91167     47.70663    272.95033    132.72385    145.96316    125.26085     29.58721     -4.56286     26.87373    -62.25834   -105.00852     68.13892    -12.78218    101.84931     118.8305    -28.86236    -74.61269    -12.16105   -216.27983   -195.32847   -226.83737   -148.53714   -283.24941   -397.17685   -244.36423   -307.54291   -178.48127    -71.83766    -89.76266   -126.45991   -125.40616    -60.42565     46.48425    153.58468    -10.91032     50.91384    -17.32209     76.36597      8.11655     49.49784     10.06123     23.80357     27.25243    109.08368     14.86445     74.04265    -31.15337   -209.19719   -165.78921   -179.33402   -223.29685   -340.34628   -480.24789   -405.41371    -251.1274   -266.30402    -94.59579    -78.22875   -253.85067      74.7211     48.89951    124.01121     92.44324    239.13326    311.06964    411.30692     393.8493    583.89154    620.29719    652.47827    605.84781    506.42408    354.44004    461.76197    369.81616     79.76609    -62.83041      6.54654   -126.43724   -144.35966     52.64074   -121.42781   -108.91511    128.77145     59.93612    103.16619     42.61415    111.18452    116.40246    -122.6463    174.51291     -92.9472   -154.61068     -76.0744   -271.66768   -272.58678   -249.15162   -318.94562   -314.61609   -403.65669    -349.9709    -288.2127   -378.99205   -317.22804   -376.47009      -258.11   -164.35239   -244.20694   -224.39684   -265.28512   -309.37357   -279.80256   -198.26804   -102.94131   -137.61688     27.41401     -94.3255     84.46089     70.51706    218.64732     -33.3736   -117.40945   -122.31334     63.96134     35.91949   -317.01105    -14.58586    -10.28555     45.58691     -69.3063    -98.00416     -191.441   -144.04084    -80.21639   -189.80145   -118.73226   -301.25619    -71.06374   -211.96523   -178.18131      48.3548     68.95382    -84.27512      -90.334   -105.49448    -179.6956     16.50821      66.3985    -46.95095     -88.4254   -246.15429   -119.33432     -3.06826    -84.38117    -53.60106   -243.92859    -19.69712     43.10416     206.5618    -13.06939   -169.18539    147.05855    297.42553     65.88154     125.9492     290.8688     54.13395    155.37747   -172.30889     -22.6874    153.74823    104.26555      8.63161   -195.69621    -71.80545   -148.11132   -135.29716   -128.69991    -64.43532     46.98119   -332.12004     69.34633    185.14956   -178.53437   -142.72646   -195.51596    -71.74065    269.60641     11.24196   -265.83724   -187.30695   -129.11069   -138.64573   -375.38662   -439.77266   -375.57343    -86.34572   -244.05361    -323.4289   -100.77091   -306.68186   -231.15278   -113.72992   -302.64263   -121.07221   -252.04053   -565.80368      5.03644   -106.13009   -240.59662   -189.80279   -433.69873    -17.08294    -279.5913   -128.93313   -160.70948   -105.89329   -150.93652   -469.02908   -359.48287   -345.56873   -130.36937   -305.92387   -288.49289   -142.24445   -438.24143   -250.63327   -308.25937   -206.17953   -281.40044   -328.32687   -416.66591   -618.57629   -637.56109   -476.11697   -307.12881   -614.56875    -668.8208   -598.21074   -727.36129   -498.47182   -812.24625   -635.61162   -900.75276   -645.99416   -811.97587   -885.32392   -768.60762   -540.82937   -732.21693   -756.76695  -1171.33124   -939.28649  -1251.77568   -942.61509    -961.6479   -829.96909   -942.73542   -937.85356  -1007.51035   -989.35936  -1118.54736   -891.05456   -973.95062   -904.13086  -1183.12748   -932.21102   -1065.7321   -1015.3595   -748.25078   -841.24275  -1473.34944   -896.96204  -1131.18455   -823.13171  -1201.08813   -709.84069  -1269.54648   -976.95077   -826.86394   -976.27512  -1135.53353   -533.03582   -767.05923  -1346.50384   -700.99062  -1064.94082   -877.33147   -605.05217  -1071.96197   -488.29504  -1353.61286   -536.65268  -1023.16698   -811.13621          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        4.00   -148.42225   -382.52533   -148.80798   -223.77611   -378.25641    -20.41665   -377.62435    -486.2098   -503.29033   -369.95156   -177.77456    -191.1626   -294.09727   -360.90202   -294.22933   -300.26436   -302.44627   -233.43328    -331.4415   -392.79411   -246.23507   -410.93951    -78.15562   -162.72047   -171.38547     372.5043   1207.98254   2130.81733   1416.92045    437.32126   -185.56433   -283.18499   -336.19503   -273.16573   -318.48987    -596.7611   -260.31246   -291.65593   -329.07429   -421.72274   -415.38299   -406.00556   -280.95554   -278.46741   -346.69893   -271.17586   -154.33671   -552.86066   -257.07954     -256.765   -369.16786   -354.88184   -252.58259   -334.13809   -355.27474   -300.49777    175.64616    967.36752    1772.7381   1088.83399    873.04328    167.35494   -250.65567   -229.50013   -373.92498   -248.78751   -353.56666   -217.93601    -135.1594    -96.47422    398.37626    937.51986   1392.10864   1555.04855    866.38304    761.09869    583.49358    589.28655     274.6238    143.43276    -131.7514   -280.70204   -510.35453   -408.73685    -374.5791   -302.35968   -372.23434   -278.88855   -142.87679    -41.38484     71.24867     58.32383     15.94342     98.63245   -207.87525   -272.99439   -437.62827   -371.16509   -400.84244    -324.1713    -40.92469     97.93166    197.77153    178.76399     71.02509   -170.33559   -326.03023   -254.00746   -185.60203   -302.13227   -397.86325   -206.67611   -520.03405   -295.13765   -267.58177   -155.34806    202.95714    300.60085    312.39835    197.59551    176.70763    -110.8023    -98.81866   -154.16888    290.18156    563.14063    656.66518    666.17643    333.20096    177.41731    -12.95984   -244.75229   -237.61558   -364.06782   -264.47406    -39.94589    201.29682    892.03836   1546.37401   1758.15563    1978.0759   1608.35444   1064.90201    641.30001    102.13102    -18.84682   -262.93856   -115.24753   -274.65013   -229.93231   -209.24563   -281.03584   -316.30321   -388.26743   -341.43302   -310.22541   -174.92819    -208.9533   -198.98086     20.40633      0.58406   -157.79503    -56.18879    -119.5967     -5.89542     -5.66105     55.54656     -1.39318     78.21838      54.3666      1.70657   -222.53843   -347.18226   -236.28331   -145.10439   -269.69888   -202.97326   -194.01153   -247.29985   -187.71803     33.46131   -197.56266   -327.57112   -358.91506    -300.7213   -299.89722   -347.87055   -210.27554   -296.11227   -180.09611   -245.22174   -222.87965    -28.32964     26.41558    -32.09417    -20.69938     39.90957    134.18749    139.14446    308.93159     207.7796    157.26215    -13.99309    -67.24229    -30.60243    -20.30707    -22.17858      5.15985    -17.95049     52.47457    -53.88215     -1.94058   -133.01018    -44.13373    -98.51495   -294.87732   -308.50352   -357.32326   -278.05386   -457.87789   -268.19496    -231.4981   -180.45237   -161.94583    -26.50762   -216.58917     26.73229    -57.39353    -20.14942    244.36915     60.38431     212.0439    131.93378    121.02007     98.58595     -8.44818      4.61385     19.86646    135.18469    -31.40941    -70.79769    -91.42783   -148.17132   -134.19566   -114.69986   -212.22943   -356.44135   -285.05586   -322.12374   -258.61416   -255.24508   -317.82092   -177.58541    -152.8484    -51.94266    -84.44866   -101.60257     14.38871      61.6917    131.54487    213.66176    234.50392    375.38391    659.85814    728.19686    743.41207    573.49763    631.93621    588.16472    433.21303    473.92746    175.40724    116.32055    -89.70354    -45.48293    -101.0764   -166.67286   -243.29503   -203.23046   -156.47827    133.51653     25.76769    161.54972     96.20843     57.41342     80.50378    -16.78915    -84.13798   -195.01066   -259.97938   -165.88408   -235.83196   -278.76831   -294.51294   -294.37684   -342.92571   -360.17425   -433.28158   -174.10449   -293.19099   -277.04288   -252.06429   -219.85301   -203.13179    -359.1247   -227.07911   -142.69507   -264.89622    -62.56672   -146.72581   -214.29316   -148.11504    -13.51643     59.60438    187.67504     13.74382     68.33709     60.29305     23.69609   -136.34076    -44.15246    -23.84277   -137.32843   -137.08574   -122.12811   -116.53402    -97.40361    -253.7769   -206.88563   -168.91646   -331.00673   -263.19208   -162.19046    -16.66859     82.89224    -99.99579     47.70224    149.75067    -81.38964    -27.80074   -110.30825   -101.56173   -169.17731   -140.12358   -202.81698     59.87895     -5.64685   -142.25232   -193.42263   -209.12675     12.57455   -340.34348   -211.00576    224.61109    105.77357     79.69581    147.51015    215.08922     214.5775    143.34488    368.41493    273.59453      93.1767     57.46135    239.05888     -30.1571     77.88659    -52.28786     -7.45144    -93.02204   -113.05163    124.01566    -38.31734     82.34345     26.71912    -68.25494    192.60049   -379.27795     77.28968    -64.71007    -90.37254    -44.75101   -107.59839     -361.693   -246.69864   -184.61493     98.46545   -290.82008   -365.91008   -317.27584     68.13325   -176.37827    -35.36589    -186.0737   -256.22693    -121.1162   -291.04518    -381.2114   -394.79793      6.08344   -244.37452      57.5242   -225.66835   -257.22832   -118.89435    -222.3381   -120.13907    -101.0578   -361.86801    124.77311    -77.96182     -8.90962   -394.40928   -334.86826   -329.51217   -381.15149     54.06811   -320.63689   -327.18254   -366.03397   -393.19667    -399.8507   -383.27537   -262.45535    -458.1573   -351.26148   -440.35326   -626.52026   -774.88939   -446.14577   -707.42675   -690.65503   -599.73586   -756.67936   -387.28171    -626.6104   -807.27084   -740.00048   -724.36047   -819.42885   -768.27772   -840.21743   -837.33324  -1139.30761  -1017.08582   -889.62367   -989.81815   -923.57636   -961.96078   -874.29705  -1002.90751  -1048.94095   -921.10304   -825.85394   -943.18272     -831.909   -881.14692  -1209.38185   -947.65839  -1267.44858   -924.51037   -759.48987  -1013.92862   -909.09981  -1153.67146  -1184.96973   -996.12476    -1223.293  -1007.79986   -704.09903   -888.10294   -685.17791   -1221.7442  -1057.64328   -821.17576  -1048.52889  -1096.26566    -885.6476  -1319.16358  -1341.75642   -894.08706   -800.42105   -862.71576  -1234.04014   -776.06202  -1142.88612   -683.23492   -791.24251   -735.65876   -697.25121   -797.69875          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        4.50   -142.09025   -513.77981   -220.03831   -157.52259   -389.86607      8.99908   -414.41069   -340.03324   -550.68601    -346.7841   -358.36303    -94.49017    -441.5073     -395.083   -327.21887   -349.82646   -320.68669    -42.12139   -147.07906   -467.27551   -274.86068    -515.8443   -198.49556   -261.13957    -218.5507    413.39282   1411.15006   1896.90571   1583.74274    484.13984    -92.61271   -103.05727   -426.79319   -275.49794    -378.8661   -512.75083   -169.75828   -333.54177   -376.05271    -361.1468   -541.49012   -251.75748   -216.84801   -328.35193   -352.68457   -414.10958   -117.05262   -398.63548    -258.3526   -356.77287   -211.83653    -486.1045   -322.40474   -336.01152   -252.88837   -154.79674    159.20941    909.21017   1606.90824   1469.27307    741.03086    105.52909   -170.04933   -145.47979   -262.08232   -354.60543   -235.50916   -250.97836   -226.74246   -282.40575    540.99463   1006.30734   1502.40097   1495.88492    980.60524    778.43565    503.05865    586.85271      274.484    -37.92098   -230.63897   -317.46796   -385.15478   -399.36018   -371.76387   -397.85364   -290.00466   -347.95089   -253.52776     -60.1027   -141.26602    233.00395    -11.41516    117.52303   -161.27847   -291.23395   -237.18793   -279.12903   -307.67914   -193.02338    -239.7164     38.98296     52.63008    -52.93509     51.85223    -49.74417   -251.67964   -303.74784   -223.93267   -339.49621   -380.83774    -372.0126   -341.15636   -334.24926   -234.43167     -76.5119    194.04534    233.79789    291.59003    290.36753      9.77685    -95.30623   -112.27176   -105.67271    196.14888    530.86575    656.85938     666.8428     442.1076    269.27426   -132.01535   -330.96645   -329.37388   -339.87711   -123.90387   -160.16572    285.04025    913.91916   1442.73658   1947.96501   1878.90269    1623.2326   1218.82534    586.48672      4.37573   -100.11825   -167.47157    -219.8732   -214.21658    -251.4324   -312.72217   -279.18927   -358.09188   -294.92925   -306.80988   -243.88415   -265.52124   -208.51401   -168.08241     55.98887   -104.92701   -119.42681    -40.64013    -86.89543    -34.97477    127.87872     40.50162    152.48516    141.78458     -53.7557    -18.64343   -133.94225    -320.6776   -185.98566   -290.71752   -253.35752   -238.67684     -162.012   -241.54408   -229.20745   -320.26594    -243.2481   -323.72515   -388.75297   -331.42674   -340.63475   -374.71397   -438.13027   -294.28695   -332.59491   -239.60091   -178.15659   -101.58439    -95.86683      94.7896     56.77575      63.9167     69.87338    113.77565    184.98212    178.59584    142.02906      -13.826    160.85768   -232.56555     35.31105    -68.87229      2.93302    148.89032     69.99269     71.38976     -0.36618   -223.45105   -108.64092   -374.49099   -128.46325     -202.183   -304.07494    -254.5921   -358.00328    -349.7322   -401.04345   -329.36073   -207.67651   -114.21689   -225.90656    -46.93001     20.34238     43.66388    130.60887     57.00962    186.05393    102.31476   -153.51032    -58.61105    -75.28433     -2.07766     24.81062     34.13518     39.00244    -36.87368    -42.93478      16.9846   -147.90658    -262.1042   -228.27943   -282.09206   -360.04164   -251.87035    -340.2266   -316.61314    -47.77473   -160.61065   -143.07809    -265.6015    -53.80707     12.13868     93.11231    126.44978    139.94487    193.33992     369.4693    462.42752    478.09627    627.18555    655.98644    763.37538    808.54629    514.14498    411.26007    180.82204     77.82764    107.00312    -17.30304    -124.8508    -41.74418   -143.67861    -44.85607      6.24081     -5.17478    133.23652      211.265    210.15562     56.07091    -24.06854    136.30938    -52.41345    -36.78057   -287.10694   -103.47999   -341.55121   -251.66066   -204.49764   -384.36111   -294.05007    -411.0432   -299.36106   -280.88393   -332.22759   -249.52055   -258.42822   -256.43873   -260.96894   -208.73905   -298.98392      -4.4261    -73.67814   -157.37051   -279.82497    -146.7063    -98.36006    -74.70932     96.10012    -48.85131    117.56433   -104.17404      89.1053    -56.61599   -212.21197      27.5509   -212.61231   -133.21698    -232.3454   -188.02484   -110.91092   -135.62893   -214.14314   -243.00866   -202.20211    -79.14073   -158.52768    -42.76894      8.01211   -181.15759    -51.83359     48.92339    -18.90716   -219.76274      8.99269   -125.55096     21.67872   -153.44391   -184.78917   -184.44336    -204.9358   -124.94781   -289.73459   -133.13231   -118.57198    -17.77832    -83.76403   -217.13522   -105.13948    -26.88902   -101.03146    102.08835     31.83273     343.2675     82.84284    259.88567     -33.9966    392.62224     40.81526     74.77324    352.44527    294.22342     97.77748    -39.35725    112.02713    124.80531    104.11363     83.77129    183.71117     86.91187    -56.64127   -173.03311    166.09078   -128.32156    -25.46439     26.04799    -45.70834   -165.15655    -178.5935    -37.95815    -169.7942     80.65556   -236.51654   -235.18837   -127.82686       0.2977   -303.60009   -101.74939   -133.38781    -31.35365   -335.69144    -30.06466   -212.57009   -394.47698   -312.92858     -102.204   -152.42424    -90.56233    -73.82642   -138.91837     39.77012   -264.68124   -120.24717   -208.81103   -241.52807   -459.57064   -147.59692   -221.53186   -225.80558   -443.49093   -331.93196   -192.02837    -87.18291   -127.76983   -294.73684   -301.04701   -186.80059   -264.96223   -276.04195   -386.13503   -297.99069   -506.31817   -675.94887   -588.95889   -658.57661   -357.65717   -570.59375   -407.59508   -571.21709   -583.17372   -656.19013   -736.01412   -762.49118   -806.91827   -422.00222   -556.04842   -914.24385   -683.95517  -1039.66836   -799.33108   -739.82301  -1082.71669   -866.06993   -769.12341   -913.26997  -1051.93075   -906.01648   -745.89009  -1020.93557    -903.6365   -885.43717  -1065.27642   -702.64415  -1038.47396  -1140.59464  -1297.34353    -799.0154   -892.86019   -917.66838   -913.10477   -863.19357    -883.2665   -979.10847   -805.24357  -1123.89136   -916.98242  -1151.96543  -1300.69233  -1040.22216  -1003.02181   -937.57106  -1117.82542  -1416.87372  -1183.98098   -780.36324   -833.91715  -1543.05024   -877.68783  -1179.98953   -828.72675   -820.61582   -715.26792   -706.88646  -1039.45883   -874.09582   -765.61963   -368.01173          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        5.00   -124.72187   -654.52578    -306.6978   -127.97158   -349.93956    -148.2411   -512.75375   -124.91324   -583.94425   -333.85182   -390.25722   -275.50713   -447.44252   -395.82697   -357.39181    -329.9054   -331.72925    -162.1365   -234.62214   -355.52199   -479.89699   -536.69077   -330.22956   -124.55108   -114.38908    220.87776   1502.79666   1717.94026   1421.14945    486.05731    184.27326   -332.10437   -514.03584   -317.23889   -409.22261   -460.19739   -424.72902   -151.31818   -331.25424   -266.72588   -276.28471   -228.21329   -350.34103   -309.49954       -329.0   -321.17312    -259.9491   -175.15648   -492.28211   -341.75864   -358.86255   -457.79013   -374.34306   -410.83002   -124.91341     33.17843    272.64167    906.84606   1602.08523   1448.25112    810.82353    100.08729   -250.26515   -114.30076   -251.67995    -376.3187   -402.93341   -203.38248   -162.31775    -37.41702    335.00428     968.9424   1499.11248   1263.05063    929.27461    601.53995    680.49282     529.2038    212.89773     19.73435   -232.10267   -399.51463   -307.50968    -357.5144   -252.01752   -307.63726   -323.56353   -286.47009   -167.04416   -126.18331      7.81305      9.48246     86.18314     76.80514   -173.50314   -302.48657   -113.69013   -369.74607   -406.75555   -232.53446   -178.32308    -86.46263    172.66203      4.68798   -120.70533   -214.79903   -241.03422   -369.49757   -375.91232   -440.01067   -315.07369   -323.73486   -460.11684   -260.00403   -243.03792      5.36784    102.41974    339.90559    310.60914    275.62007    169.84615     -31.7603   -117.30202      50.6622    278.33472    398.10168    592.13952     548.4147    472.07748    327.19929    -65.99101   -186.70459   -331.61027   -235.55378   -159.71041    -83.15751    287.87853    803.40623   1502.77933    1884.0766   1951.46885   1834.09815   1090.81757    446.28914    131.89841    -98.58621   -230.71977   -228.15535   -134.69279   -261.30983   -400.19024   -348.60247     -267.477   -218.04505   -332.06545    -182.1114   -341.06588   -290.07117   -296.52277     86.85737   -195.36296    -16.34418    -196.3386   -200.22335     -4.95548    -49.23618    -30.92216    174.76708      3.81975    -109.6627   -357.05761    -347.6514   -339.51783   -292.35113   -218.09767   -121.06388    -175.5026    -44.10355   -226.34381   -135.24502   -213.75553   -215.21207   -352.87037   -432.66003    -223.5222   -280.25265   -307.06413   -234.97858   -458.05852   -331.68048   -245.95877   -177.30388    -196.6969   -101.70487    -84.21927      -5.5823    -67.37726     65.85697    332.70479    193.14535    287.86237    114.12663     20.10263    -87.80938    145.06321   -107.80098   -124.35304    -42.47356    181.41052    192.09555     74.55838    -18.95648    -64.14074    -167.8745   -173.33838   -155.66746   -186.59153   -251.45361   -321.25441   -317.64114    -228.7259   -189.20298   -247.91366   -159.67352   -234.42183   -107.77876     24.84033   -141.95994    117.39222     92.59512     87.01813     93.86327    -23.67024   -160.76233    170.43066    189.94846     12.80033      5.36596     39.84975     26.36005    109.07845   -293.67282    -196.2219    -52.55527   -324.56481   -174.18479   -243.41079   -232.18711   -310.40835   -342.91033   -184.28405   -329.26673     40.33243   -163.65888   -107.78703    -57.80203     125.0438     14.91554    -33.87494    205.42659    285.10189    365.46967    457.92081     641.0475    700.99864     816.0014    574.64411    773.36588    537.61634    418.27077    363.73382    383.12747   -101.37109     75.61627   -152.79009   -110.07324    -77.21321    -39.96322    -16.02704   -110.32678     71.40821     31.60565    126.75916    121.05214   -285.91725     24.21615    -35.40988   -123.47584   -120.81585   -173.76654    -139.4876   -261.56445   -340.27533   -229.21169   -349.17637    -223.8931   -417.73195   -362.55058   -445.85852   -163.54632   -345.04174   -344.06421   -263.56849   -309.89396   -192.07916    -295.9912     -8.28474   -168.58484    -78.13662     63.23428      0.61971    -34.24335   -133.67369     16.26431    115.05824     16.37024      11.9148     30.29072     38.29579     -5.24114    -70.45661   -285.22488   -132.58433   -202.09322    -74.97619    -397.0875    -86.56222   -122.27024   -264.78065   -131.78204   -120.66789    -84.80539    -76.44308    -22.89199      19.8335    -36.51311   -107.31396     58.40757   -124.57196     -81.8178    -62.66434   -239.76024   -166.48382   -265.90618   -115.20507     15.42878   -171.73375    -179.3389    -68.18239   -169.61721    -91.26313   -155.68425     77.52417      4.22313     97.87676   -123.16932     -3.40683    -55.19758    141.13201    174.97597    374.93869     32.57448    234.96404     99.75632    127.59067     73.71202     87.44952    232.30937    -12.45188   -128.07177     31.43148     75.24648    -14.00217      -79.775    -17.04918     -11.1941   -303.83685    -13.60661    -55.56522   -148.48455     66.77316   -292.33084   -129.16476   -103.58335   -120.37283   -250.46493   -140.76427    -498.2626   -141.43675   -527.10969   -200.58593     66.87006   -161.40081    -77.01172   -484.14648   -454.19361   -312.79697    -34.41718    -172.0878     -225.397   -105.00792    197.18927   -291.58648   -321.77647   -103.64389     44.97541   -257.94612   -116.76538   -600.63352   -162.05663   -455.42297   -118.53019   -305.67608   -289.30385    -246.8529   -335.59557    -247.2384   -347.43855   -183.01494   -338.59938    -416.6069    -352.7187   -220.74708   -413.78626   -146.00952   -352.46181    -446.5626   -599.91575   -586.78783    -515.7772   -449.14494   -752.23659   -461.27246   -682.53408   -825.74473   -447.00006   -946.03106   -769.55659   -750.82498   -928.60621   -858.65431   -530.53839   -948.09409  -1010.79655   -932.54151   -893.77478   -787.20771   -872.72955  -1319.37305    -694.1438   -812.69829   -638.27348   -921.21606   -961.05205  -1111.85156   -823.98477  -1021.43496    -788.1702  -1129.33554   -794.50349  -1019.07814   -945.09568  -1222.26634   -724.86247   -613.00965  -1099.54728   -919.69805   -822.20948   -954.70546  -1134.03694  -1091.63372   -957.14602   -945.90137  -1010.90433   -990.86254    -995.4436    -915.0816   -604.77967  -1309.62966    -1191.353   -802.18719   -866.85985   -886.85247   -983.66918  -1264.35008   -929.50832   -908.76315    -775.6301  -1191.35154   -326.36078   -808.63283          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        5.50   -110.44992   -629.53738   -252.27844   -185.90803   -280.85329   -311.04716   -576.15694   -157.30047   -441.39078   -355.44457    -344.8805   -413.91516   -358.45039   -369.99655   -339.33337   -181.05213   -353.68409   -219.34882   -419.74527   -311.30681   -519.36486   -427.88654   -387.61036   -135.62971   -306.83857    386.41976   1244.82791   1863.48011   1324.38893    447.63294     20.10679    -270.4238   -330.76404   -476.95969    -374.3103   -543.60546   -377.81441    -98.57301    -417.8605     -350.272   -230.37998   -316.68888   -372.13033   -299.41288   -366.87426   -308.88482   -274.13625   -255.71344   -354.42351    -360.0352     -386.441   -412.66351   -361.78402   -321.55364   -312.75661    -188.7276     189.6698    962.38647   1458.50266   1493.34125    750.38248     62.42015   -143.90729   -236.41509   -331.89083   -277.32229   -316.47757   -202.86439    -36.94426    -13.21659     396.4082    967.38482   1513.24026   1314.19934    736.02076    785.53459    646.74427    528.93476    426.18684    -60.10163    -57.13541    -160.0895   -205.09089   -381.62717   -344.12247   -134.32797   -400.49584   -232.07634    -266.5293   -300.29529    -29.94765   -143.40342    265.17525     -56.8692    -74.99286   -167.99842   -328.53455   -152.44184   -206.73408   -260.16571    -92.14864    -207.1715    -75.03047     55.60217   -144.78188   -123.05999   -312.45328   -310.69606   -237.20556    -337.7461   -183.65724   -385.08464   -420.66559   -368.97876   -272.91983     81.91086    234.38608    295.75742    375.60034    176.05441     194.9535    -89.76342   -290.95178    -10.70297     97.57921    299.29001    580.42718    624.32317    412.84199    134.85021    -23.56155   -151.19514   -203.24368   -233.56229   -228.25431    109.27078    342.50465     640.9708    1407.4826     1826.038   1968.51385   1856.94658   1118.07877    786.11656    289.63851    174.99346   -124.75571   -275.99373   -211.74149   -292.12585   -251.43621   -206.32384   -323.30489   -388.66218   -402.79056   -266.54538   -264.89376   -280.55345      9.81178    -86.71284    -139.8088    -80.03167    -99.81088     -28.0884   -167.74181     52.76583    197.48692     24.01517    -10.88432     -74.9651   -186.15785   -341.08277    -462.8864   -294.95827    -179.2185   -131.62833   -156.56815   -231.71672   -175.74753    -43.47169     -122.775    -134.5939   -237.30351   -427.40417   -259.58168   -273.58227   -383.05316   -288.17544   -461.79735   -112.18364   -353.08435   -307.88409   -217.36928   -120.83615    113.42769   -123.45245   -140.92304    163.12804     190.4359    255.55316    158.38717     63.63927    176.04886    108.65236    -47.72973    -17.17797   -172.28853    -55.87647     79.05143    187.03362     91.85657    168.90093    -25.88416   -294.60362   -148.34446   -147.08384   -204.11608   -317.36123   -253.60741   -334.42806   -397.18319   -345.88046   -294.00664   -306.61295   -195.60894    -11.77239    -92.70855      9.82499    -91.12263     78.93952      56.9629    -19.78052     25.21418      0.50081     22.31666    222.64629       38.094    -56.71192     19.73842     80.05227     87.52762     13.00467   -226.62905   -203.10656    -96.03439    -81.49085   -333.18604   -163.49611   -417.56658   -267.25677   -280.50516   -137.54302   -273.93164   -223.95374     92.21135    -19.60411     15.72255    -43.50041    -52.09429    213.99152    147.41083    305.59274    456.98596    795.37377    887.79236    597.48631    735.78252    562.26299    484.78401    406.79606    210.96078    239.80556     232.4569    -48.65847    -83.28059   -150.15104   -114.94067   -356.01482      66.2158     51.67681    -51.36447    253.49794     -6.90991    -36.32876     20.92162    154.47051     -3.14502   -185.52168   -115.50922   -259.99792   -132.68051   -338.54184   -297.80293   -218.47594    -340.7819   -457.94131   -460.42294   -236.79288   -405.78215   -204.58174    -248.3199   -349.74437   -347.26188    -259.1186   -222.06042     -120.679   -279.49757    -72.93509   -151.67141     -33.7785    -70.73524     95.35815    -97.86588     64.09033     15.34718    103.26552     77.77756     -40.2724      9.85679   -220.10742    -92.57919     -59.1613   -123.55876    -99.21737   -124.15464   -113.97182   -161.41834   -128.45344   -173.62628   -184.60396    -39.43817     -7.28309     12.83334     -70.9471     -93.9439     26.78529    -53.11789    -53.66184      2.64725    -31.73976   -158.25909    -203.5594     51.98308   -186.16875   -187.61751    -93.23812   -167.27621     23.43915    -81.37183   -180.96376     -0.85801     12.02243    -44.97803    -16.67838    101.53775     99.80707     98.65497    239.09879    211.55052    190.88817     47.62268    211.56698    319.59573    176.08294    160.56721     17.36874    187.88228     -0.15407    -17.51784    -62.98335    211.05776   -196.32644     39.67826   -165.21178    -30.39331    -65.34191   -150.12638       1.4794      3.47229     79.64855     49.10891   -124.49423      -4.5377   -162.85768   -144.86004   -211.58273    -50.21273     88.44156    -258.5884   -221.93757   -229.00786   -163.34947    -20.01758   -345.25523    -54.78639    150.07818    -77.42875    -103.6371   -195.82256      12.4197    -50.28925   -153.37073   -205.27828     -5.10092   -155.04808   -175.95034   -169.17836     -14.5303    -81.47785   -110.28307   -123.08146   -231.94131   -368.73349   -475.31959    -93.43074   -179.69713   -304.04452    -393.0664   -230.55677   -271.01663   -269.75536   -127.88187   -271.87477   -459.18227    -65.90681   -394.88792   -338.34487   -354.61222   -401.51002   -460.43511   -590.50393   -557.08513   -755.81611   -748.39935   -581.97435   -848.64098    -854.3876   -937.45515   -631.90189   -716.59351   -613.93164    -699.4668   -817.55942   -778.35777   -719.21291   -860.67508   -877.91451  -1008.13359  -1046.75328   -962.18241  -1181.00417   -943.08801   -994.53609   -803.74529  -1029.78385   -903.18578   -864.18866   -921.77755   -795.01107      -512.61   -925.04432   -981.29645   -968.99554  -1116.66091  -1070.95734   -834.79652   -1193.0413   -1360.4239  -1177.81872  -1070.98393   -659.13724  -1270.76075   -805.46532  -1488.77262  -1242.86341  -1145.19426  -1148.79452   -846.54573  -1051.68012  -1117.79924  -1180.65506   -863.87765  -1136.05167   -647.00233    -910.3908  -1184.54586   -227.82484   -764.48806  -1094.15046   -924.65082   -385.95428          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        6.00   -285.50865   -420.29109   -186.85383   -260.06059   -260.73619   -161.84022   -522.12043    -318.4288   -284.72332   -352.39958   -389.32738   -346.46084   -189.13242    -324.8747   -245.81753   -373.05383   -440.41416   -169.17686   -542.19344    -331.6346   -341.33912   -297.44267   -310.79684   -210.51315   -335.75111    334.70539   1454.04386    1773.1651   1429.77126    409.93465    -69.23002   -139.03729   -185.61994   -349.71148   -369.27604   -379.49237   -626.84931   -275.90729   -306.41378   -346.92169   -370.53443    -480.1258   -420.60868   -324.45968   -262.42757   -372.06162   -305.95474    -360.6366   -263.87685   -340.20025   -332.41243   -132.82766    -308.7149   -295.44752   -277.35438   -173.54534    325.40988   1046.34751   1428.49025   1607.17535    759.12564     33.99988   -207.02688   -191.33015   -221.11698   -406.94578   -302.78823   -346.10764   -179.54625    -57.20543    257.57733     941.2789   1353.21926   1277.79584     822.2337    658.68025    502.10113    549.01065    483.53406    118.12666   -229.45013   -270.81882   -295.10247   -231.07452    -466.2773   -227.24113   -241.76915   -376.67136   -190.21165    -41.86058   -124.43904    340.99284      68.1759    -36.37925    -219.8327   -304.14701    -95.02768   -230.90526   -321.52659   -343.57695   -281.19752    -78.86605     69.19533    134.44937     65.82822   -409.13306   -275.52869   -144.52572   -118.92198   -322.18684   -316.82779   -387.89653   -273.12024   -213.92927   -191.95791    -34.87492    -28.71375     74.96661    223.75437    210.83349     36.98996     24.10088   -214.23362     11.31793    434.30509     205.6152     650.4419    467.57684    328.80083    131.23078    -76.35546   -161.04213   -201.43135    -93.39425   -216.86033   -230.12158    211.41709    767.85774   1348.08111   1845.08424   2066.93823   1730.55362    1210.9343    604.13837    220.09409    -130.6036   -114.84725   -219.90104   -159.76754   -162.66825   -200.64371   -249.49034   -416.00383   -214.00398   -290.81355   -255.42442   -172.88377   -198.18502     -7.71746   -173.70017   -102.84326    -29.87598   -288.43302   -156.95836   -163.84631   -216.59244    -29.80505     76.42085   -108.71824    -76.06092    -211.8793   -159.83446   -253.54914   -281.71639    -218.4852   -438.69018   -112.20963    -81.37002    -192.7298   -197.17345   -166.14541   -253.09913   -147.79807    -318.1278   -301.43199   -523.20372   -277.66346   -291.03622   -359.41234   -232.87146   -371.54142   -262.10957   -113.09535    -12.54225   -249.74221    -43.11765    141.24983    241.11962    191.16054    349.38662    274.50594    125.78554     59.97547     14.45785     48.24204      78.6815     -1.97588   -125.65998      5.30486     96.80647    -33.27194     34.07205     -57.8012     94.44091   -234.49887   -299.70147   -219.99598    -315.1914   -418.14972   -344.93923   -242.06577   -115.19092   -234.98952   -268.44669   -201.57034   -112.30295     45.73074   -243.61165    132.60175     36.87939    122.11032    146.67885    -52.71581     29.70185    155.18193     108.5081    -32.50256    -22.95004      1.32023    130.86914    -76.18666    -42.71348   -176.29521   -108.92921   -293.17073   -212.50913   -303.50259   -324.46854   -241.41618   -392.80927   -194.62932   -178.78501   -158.62868   -125.18565     10.86119   -120.63377    -66.75066     52.98602    103.09627    241.99325    225.83379    384.55454     431.8276    703.34287    658.45121     695.4562    688.99135    525.56176    521.86032    259.99636    393.57728     73.38146     20.93432    130.80367   -198.07779     -8.04929    -238.5175    -97.01028   -186.04705    -98.03034     -7.19483    119.68926    171.17121    -40.34125    114.61749    -18.32708   -162.05565    101.82245       6.1066   -231.88807   -240.54909   -282.92266   -188.79172   -263.25115   -282.51445   -220.00516   -318.51615   -373.71399   -513.07441   -531.27244   -206.48509   -524.27042   -337.05278    -59.52633   -250.78544   -207.62693    -36.60929   -197.91243    126.09034   -263.43014     32.06269     78.29383   -131.27777     -40.7748     57.41522    103.93231    -26.52174    123.92094    -69.47167   -101.46548    -58.01607   -186.83694   -162.87842   -220.84642    -117.3043   -238.15627   -106.54013   -128.86133      7.25099   -176.88132   -196.50909    -62.67608   -100.82208    -29.05265   -106.83788    -51.83624    -38.62772    -22.66414   -106.04221   -104.10714     -51.5962    -39.37593   -146.10163   -161.49365   -175.33626   -200.27084    -72.90433   -107.19763   -208.95788    -167.4327   -121.72078      -91.763    -31.73206      1.47941     80.40929    112.74154     186.7226     79.10503    237.98874     85.01256    253.82259    217.85992    260.43539    238.12233      51.9079    229.58795    -36.27937    300.45709    -88.54669    281.21678    -70.71992      52.7997     36.07405    -140.0821    -77.78722   -128.40607    -31.55395    -40.91474     -0.96235     -66.6015    -122.7924      1.64527   -103.23292     56.38325   -356.15051     -5.49072   -370.27037   -250.41729   -247.43029   -229.70317   -377.72867   -136.96379   -481.84012    205.29039    -98.70391   -203.26596   -399.30135   -128.01254   -431.58204   -462.26463   -286.61901   -239.42706   -318.68139   -170.55813   -232.22592   -238.13716    -34.65283   -142.15385   -321.02158   -341.62789   -237.86348   -244.58886   -201.71088   -218.72317   -293.59134   -334.08107   -188.45822   -498.48866   -236.63325   -375.69304   -164.49341   -172.34593   -259.00495   -621.90787   -369.91131   -335.98822   -486.34149   -568.22324     -596.186   -589.46623   -638.52976   -461.95396   -601.47521    -679.1675   -964.06968   -263.11899   -599.39738   -701.66426   -788.10984   -721.70567   -750.02451    -745.4564   -802.95859   -803.54126     -934.006   -804.41613   -862.74263   -671.11097   -976.19719   -846.11258  -1031.07814    -890.2701   -776.49563  -1047.96478   -989.59394   -792.72438   -956.03218   -1065.7717  -1063.26558   -927.04947   -1303.8853   -875.02115  -1019.48011   -1123.5284  -1004.28698  -1413.37073  -1381.32396   -933.48573   -995.20138  -1081.05022   -843.16346   -1031.0242   -759.59421   -706.06985  -1211.80622  -1028.93146   -782.43122   -985.93641  -1233.33999   -781.62772  -1248.12903   -792.75076   -965.05585   -953.86157  -1686.32901  -1503.37719  -1021.92356   -1225.0584   -801.91262   -289.06777  -1022.28917          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        6.50   -491.80573   -199.22576   -194.33935   -291.99786   -327.38706      3.43496   -458.62483   -344.62767   -242.29955   -346.61816    -469.3042   -328.63817   -189.46042   -193.48116    -76.84293   -548.24071   -435.22603   -181.14103   -334.98211   -434.75985   -200.31568   -302.62697   -279.04619    -88.96127   -182.75717    279.31151   1414.52789   1676.38616   1369.10347      489.485    -12.23163   -209.77232   -452.81543    -349.4767   -317.67463   -285.00371   -386.42305   -421.13232   -256.44627   -319.02742   -406.87538   -298.83208   -369.14721   -440.36789   -308.23342   -395.51502   -367.35557   -419.27239   -430.38882   -212.82181    -94.04426   -386.14035   -332.34105   -310.91095   -252.76084   -163.33265    311.89214    1036.1915   1768.59354   1643.53182    596.39177    115.84306   -218.61559    -130.6718   -143.92056   -132.03619   -298.57442   -361.78923   -114.98534    -79.74194    364.98299   1030.14592   1513.18452    1315.6953    818.24735    550.18407    537.02056    569.09653    161.57139    167.55235    -265.6639   -387.31445   -136.42881    -97.16271   -408.66368   -260.59614   -325.15216   -349.80831   -300.38482    -108.6371   -101.75668    111.39462    -66.91659    -78.36441     39.69889   -155.34921    -17.00616   -385.73295   -300.00888   -261.17932   -225.85959    102.31172    165.32984      41.1618    -18.99888   -230.57888   -251.08611    -385.8716   -322.51818   -317.86303   -387.61313   -250.14666   -276.96871   -398.40696   -325.63841    -45.29222    187.55681    349.89954    420.03189    217.35297     35.50622   -176.18469    -88.08101    -21.62163    233.97825    376.99987    757.43301    585.73545    361.09367    109.06917     59.84538   -193.01587    -259.8127    -301.3507   -225.25627    -47.99416    265.82849    882.70054   1473.21373    1965.7363   2055.81718   1857.04701   1077.78108     549.9799     38.32576    -32.71083   -139.61897   -196.73484   -218.96238   -120.88249   -304.48276   -258.24521   -332.05255   -234.85635   -217.34307   -221.70225   -337.24574   -168.49321     -58.7463   -171.68062    -82.73396    -23.49351    -88.86253   -131.02706   -151.12655     17.41742    109.73301     27.38738    -152.7319      5.86962   -119.39086   -105.90598    -85.54444   -208.56356   -305.15198     -344.993   -323.44586   -153.85719   -187.02128    -74.73273    -244.4378   -285.65836   -366.38404   -314.74055   -300.90275   -176.37052   -456.88935   -340.59959   -383.35707   -255.23598   -164.68182    -176.4892   -153.56111     24.50004    -35.23847     -42.8375    113.66384    266.51121    218.37226    111.06699    301.08082    185.25606    276.45221     47.59025    -99.13743    -137.0117    -96.69844   -113.52963    116.12981     29.62254    213.36146     43.17578    -70.35641    -87.66614   -160.67283   -110.84328   -298.34937   -260.85154   -414.66479   -267.55111   -267.40522   -197.03216   -372.13299   -149.76213   -161.54597   -180.18825     42.86473   -139.97304      37.6618    -26.30993    156.42938    111.87494     22.78579    -51.82816      8.25869     69.84765      16.8017     41.98309      76.0673     73.82742     81.86017    -75.56581   -103.19323   -203.60465   -268.94356   -254.66629   -272.77444   -174.39898   -204.19959   -372.77338   -399.79288   -112.65583   -167.97844   -182.98625     -14.4296    -56.42064    -18.62913     85.20509     154.9729     142.4714    232.26609    476.81537    486.77425    550.04015    619.14965    589.95148    558.68886    421.51877    520.03538    479.56616    327.87579    158.19751      77.1681    -43.93526     53.36253   -143.76424    -87.41836     42.02854    -61.93635    -18.10145     46.47695    100.78139     35.59491     96.31118     73.64852     18.69893     -31.4396    -10.31827    -63.22282   -308.14739   -216.39309     -239.439   -326.56843   -300.80675   -258.04368   -257.27868   -349.09573    -353.3561   -245.07058   -333.90401   -195.10473   -285.34111   -241.24768   -376.60865   -176.60256   -246.96308   -200.07116   -314.24038    -51.57591   -139.26817   -106.77898    -31.56732    -98.49747    -28.36881   -148.24001    -96.52338     96.29469     178.0558       0.0824    -81.00132   -126.27911   -288.20625     95.32753     53.41549    -16.75032   -262.35455   -315.99392   -254.19708   -153.21912   -195.14779   -142.86471   -176.51455    -61.74011   -121.27291     -2.83007   -149.74237     92.05934     32.16539    -30.40372    -43.43464   -159.56479    -19.94505    -73.68309     -7.73613    -75.51353    -63.85818   -137.70955   -193.71767   -264.86455   -274.62959   -377.58471     39.10655    -96.51287   -161.29665    249.27518    128.71799    304.67925    144.37268    129.40537    227.05234    131.77027    126.37142    207.66412    161.52072     76.87891     91.81833     193.6736     36.61728     18.90263     76.16287    157.57611    -92.74404     96.54524     13.47254       4.1909   -298.35222     -9.54376     74.92214    -54.61045    -31.23167   -174.86133      -1.5654    120.08181    -57.58131   -187.61859   -161.54538   -251.58404    -61.34187   -236.56417   -319.38259    -19.34241   -126.00962   -621.07494   -166.31524   -215.64083   -137.59266   -149.68022   -339.32404   -118.68029   -279.74136   -275.40911   -295.88391   -246.81307    -47.43486   -244.03526    -101.5332   -556.61544   -277.81604    -198.5031   -652.73584   -100.06016   -308.94558   -256.97688   -371.69752   -186.77059   -432.78221   -209.17388   -399.29832   -159.31169   -104.51697   -161.90292   -353.62454    -297.6341   -344.14822   -375.47671   -541.16362   -397.28289   -560.98084   -453.58365   -435.89438   -628.24179   -425.73984   -336.29057   -396.70978   -748.53893   -715.45021   -878.80443   -738.96667   -902.75836   -731.01319   -737.84291   -960.67267   -738.59592   -914.62447   -492.59868   -952.86627   -708.26761   -946.29824  -1030.54653   -838.21962   -898.83938  -1066.02625    -795.8238   -881.25879   -956.43786   -809.85617   -732.80168  -1027.85438  -1069.69975   -858.45594   -913.87111   -782.94048   -851.76653  -1181.68801  -1409.12582   -949.67668  -1157.96816   -921.29045   -616.86621   -1489.0353   -583.58897    -954.6808  -1158.33501   -999.61062      -961.23   -881.56856   -955.98945    -959.6408   -463.52138   -723.06836  -1093.50383   -737.36861   -666.03959  -1113.33896   -906.81839  -1105.20144  -1050.65437    -355.2656   -936.68549   -256.68247          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        7.00   -678.70607   -140.05958    -337.3042   -313.92651   -392.99585    -61.55531   -402.07681   -260.25503   -210.47479   -359.93372   -308.71887   -410.55787   -249.00796   -225.03717   -183.75222   -343.27601   -316.52931   -287.10238   -258.77704   -256.51918   -319.09219   -359.37723   -382.23337   -106.32062    -211.5128    320.42214   1421.50014   1839.77465   1587.13544     713.0132     10.65949   -338.11398   -420.37351    -146.5922   -304.62299   -306.38552   -392.62303   -417.79147   -281.52299   -213.37845   -224.41299   -236.71468   -213.41464   -451.48925   -321.41457   -300.03318   -334.27453   -339.20469   -309.10866   -422.70959   -441.68238    -281.3829   -393.60858    -441.8066   -318.88785   -290.66192     315.4154   1198.44512   1657.69512   1351.19779    709.51609     88.95134    -21.56206    -252.2529   -339.77275   -202.23418   -212.32675   -145.92838    -78.52908   -233.99633    332.95275   1267.44795   1526.68895   1467.80197    905.76061    621.95426    785.07171    609.34139    111.38168     91.46556   -385.25136   -474.28122   -297.36611   -275.86882   -304.73786   -453.75025   -432.87233   -271.82227   -348.16122   -210.41668     12.36882    156.64001    273.69408    128.67068   -102.56385   -147.47728   -376.25129   -282.01099   -258.37645   -253.18629    -102.2573     97.92989      2.06653     71.16447     23.02903   -156.95912   -336.25998   -329.52335   -232.72486   -354.05354   -178.13544    -396.8185   -305.75684   -234.74747   -311.49838   -103.51198    189.42865    271.71929    279.19456    199.30505     177.8431    -98.20837    -154.9248      3.21966    274.22555    507.28057    709.66766    552.14732    383.54045    139.34118   -141.55584   -231.01223   -210.18505   -254.86652   -196.76552   -222.26332    246.88825     894.4989   1386.56955   2000.24779   1852.82509   1748.08258   1150.74883    668.80686     306.1974    -55.99517    -68.65167   -152.34788   -131.74281   -252.95876   -175.63062   -365.77783   -313.08135   -440.16001   -229.52922   -293.59917    -116.1707   -159.52577    -82.28451   -155.63096     -8.21759   -118.83223   -135.47575     58.85109   -155.97344    -12.03793      6.73741     13.19488     -20.2104   -113.60796   -209.10061   -392.29903   -195.25599    -223.4399   -129.04838     -349.447   -140.62801   -181.76697   -226.79878   -178.82745   -177.73031   -157.84563   -431.25205    -220.9879   -366.44658    -311.9854   -420.58245   -371.64824   -365.55507   -325.03871   -269.99685   -156.96823    -67.92334   -168.83457   -195.37206   -126.01825     -15.4702     200.5477    190.24129      112.622    194.70182     211.6944     13.70236    -42.56743     84.59259   -111.90032     53.16672     198.2687     48.94016     54.67353     97.98423     72.12625    -22.58894    -68.89406   -131.46547   -311.66732   -389.10634   -289.06148   -257.94724   -314.89938   -426.87886   -163.61324   -258.57185   -261.74148   -178.81301   -112.16058   -192.07517   -119.60828    149.48102    -30.12411    151.99443     15.73901     60.14691    117.74406     58.78116     64.17998     247.7403    147.12062     73.28463     50.07754     35.59825     63.01669   -119.92408    -85.85799   -206.26429   -178.02128   -252.95885   -221.84581   -303.37866   -336.36965    -254.4941    -81.64631   -230.89671   -101.90974    -69.34642   -190.25034     97.53427    180.43821    170.92425    232.43283    304.13073    384.45874    220.31976    620.42875    741.10719    656.90261    542.22763    708.85788    487.33259    355.14441    283.06006    105.55815    -14.11484    -79.23068    -61.01116   -138.65453      -101.94   -202.49193     67.32984    -36.18288    -35.59237     86.81598     87.37053     28.58671      73.1296    168.22603    -85.29688   -137.37392    -61.32236   -215.14685   -310.48634   -265.59825   -203.31628   -118.02427   -333.44213    -492.4397   -283.96941    -322.4818   -328.97185   -402.29129    -269.3441   -285.54633    -188.7608   -122.01435    -65.57413   -260.84332   -271.64121   -150.25869   -144.23642    -71.93512    -11.29806    -10.37337    -48.55277    -13.74711   -179.29853    131.92872    -31.42145     26.56666     37.61719     59.15509    -92.14087    -161.8235    -52.25421   -164.62848     -98.8746   -204.66107   -250.54193   -114.72582    -44.10228   -214.26532    102.31695     -24.4697     48.36414   -285.12211      5.93625    111.89557   -123.31484      95.7569   -216.91421   -480.77046   -216.80477    -45.43375    -62.04776      4.11947   -273.88565   -191.73816   -138.45679   -135.02763    -278.1591    -93.41119     45.17422     68.53264   -137.01892     63.99986    -35.94777    147.41505    228.90624     329.4495    153.65167    370.28086    252.95301    255.09752     63.66866    209.81131      106.485      188.886    -46.21445     84.31984     97.98402     38.72657      -94.619    -34.01971    -90.36583     -9.71318     79.18769      57.0902     51.76523     51.01772   -124.13166    -36.51986     85.32832    -76.70794     32.98964   -139.04553   -155.28071    -249.9191   -405.66712   -234.50394   -167.00821   -113.92562   -206.24336   -218.34739   -280.09262     35.36131   -392.64027   -297.68482   -320.75794    -25.74615   -113.51719   -249.41866     -4.45548   -225.66592    -509.2205   -385.33281    -67.81386   -161.66069     76.58663   -407.30139   -183.76245   -386.33241    -12.96333     47.72526   -140.46254    -80.88655   -429.67266   -291.40553   -261.95987   -253.45949   -104.16744   -450.31287   -376.58858   -277.22731   -587.56551   -322.89215   -315.16273   -292.17658   -262.76115   -541.03362    -650.7587   -615.45184    -716.6432   -415.12106   -512.95954   -601.11918   -596.59526   -779.98508   -842.36059   -721.49752   -653.05551   -914.75355   -697.51277    -784.8041   -599.74666   -801.30425   -817.00543    -854.2178  -1148.86335   -820.28225   -885.02414   -743.29218   -850.27848   -977.95993   -933.81749   -954.28475  -1009.58807   -961.81517  -1021.42116   -863.46049    -861.1727   -905.69524     -925.332   -955.02789   -915.99906   -1086.3985   -951.25057    -938.9954   -787.82998   -638.19324   -602.96252   -804.86743    -901.9079   -716.90292   -860.35112  -1080.12971   -898.03244   -981.12731  -1015.91761  -1215.74631    -869.7716  -1298.78004   -649.89106   -571.77735   -536.37174  -1254.89773   -897.44236  -1005.31029  -1501.60522   -553.13378   -456.52909  -1441.22687          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        7.50    -661.8143   -130.48232   -475.73174   -317.21467   -368.11365   -236.01375   -335.82938   -249.18384   -188.13062   -367.39853   -131.25111    -419.8347   -303.56744    -483.8333   -325.87205   -256.76965   -230.87534   -533.39089   -250.17245   -278.49031   -441.97142   -179.63389   -430.39878    -84.51241    -28.96297    380.58443   1501.41138   1911.29364   1500.28651    530.09763   -131.49489   -297.10941   -363.80235   -285.40067    -403.2546   -352.16079   -409.11039   -392.49887   -424.40194   -273.82359   -351.72023   -263.26189   -315.09446   -534.86964   -453.04467   -151.02421     -523.691   -347.18848   -235.83033   -321.38221   -340.25427    -352.4947   -469.01062   -204.71428   -421.28891   -255.74746    216.61409     914.5962   1590.70104   1413.31933    775.46525    -27.66419   -138.51486   -325.66384   -164.16143   -220.47941   -191.39979   -141.19687   -242.92824    -26.33213    336.63862   1131.44904   1520.13138   1236.06335    883.44219    490.66172    603.73225    532.41302    291.01496    -69.85971   -224.55521   -115.46087   -210.62292   -370.54836   -401.94874   -296.42355   -215.90301   -255.95779   -241.95148   -222.20397    -28.50691    189.71739     219.3869    227.93104   -175.03475   -288.82506   -134.07695   -233.37449   -449.77231   -189.70543   -135.81923     56.48946     33.01393     25.50236    -55.70589   -327.23227   -238.50894   -332.13221    -446.4261   -360.82097   -552.65077   -349.36352   -491.41788   -346.08958   -258.09288   -141.59135     37.24074    315.97844    312.27889    210.07173     33.41881    -73.91097      5.90827    -90.64753     59.32437    546.15136    602.01478    517.31471    357.43308    109.29795     -5.50729   -250.41271   -243.80244   -304.43806   -208.74893     -38.2036    293.98791    792.07019   1332.84009   1807.62933   1911.43923   1736.48377   1268.67146     425.7466     194.7406     72.28406   -108.56483   -175.97119   -143.11118   -189.39324   -353.79981   -194.23726   -304.24656   -351.36704   -506.82244     -442.231   -229.48127    -243.1617   -263.05986   -283.94659   -219.27083   -180.81291   -101.28637     28.38827     -88.6613     91.39298    -65.32368     40.63676    158.21561    -40.92231   -202.47403   -180.45848   -317.72274   -266.64801   -335.59228   -192.21458   -153.80892    -27.97298    -54.52441   -202.62938   -204.45701   -430.72652   -367.93096   -346.20141   -357.03203   -365.22462   -324.31251   -332.75024   -385.20898   -295.99526   -371.06239   -138.96035   -161.28021    -63.51056   -105.85131   -139.66497    -26.40828     97.49952     101.7512    108.21793    253.62324    137.10795    180.14781       27.137   -152.73227   -193.36918    -83.90382    -80.26169    154.35946      98.4588    104.48913    -15.54208     105.9656    -88.53987   -118.15651   -264.85801   -273.87403    -386.3676    -350.0667   -203.71584   -315.33637   -219.86539   -144.06151   -247.52183   -310.11904     -8.63994   -238.20274    119.65737     76.81856     26.89696    115.49956    -103.1496    135.78447     -65.0252      29.3497     96.07981    -84.35926       121.92    -18.63363     70.65798     51.90606    118.26432     55.63318     -153.498   -137.28979   -460.63034    -89.62891   -346.50783    -195.4962   -261.04796   -220.45414   -204.19543   -225.34887   -114.59756     -3.00452    -46.71718      38.4844    127.38948     97.83604    152.08574    193.64527    495.00782    281.68046    668.30221    718.46854    587.33398    572.35313    582.08536    552.21123    598.94217    234.23265    197.15892   -136.83262    -29.64142   -145.19553    218.49853   -118.26573   -223.13159    109.78909     37.52324    183.23436     13.83519      7.10433     -0.40076    103.99867    -17.59456    -11.77927   -179.88582    -46.96267   -171.64895    -97.99697   -190.12168   -405.13863   -439.75051   -186.89448   -299.36615   -358.47997   -401.18323   -238.42488   -334.34544    -49.54129   -281.35874    -321.6322   -303.82393   -102.39417   -241.78156   -209.51536    -96.14257   -173.05958    -29.41696   -144.43966    -39.14587    -62.83595     33.08092    -16.44681    -29.20486    -92.10578      6.64688    -31.50077      -0.7369   -131.70678   -122.02914    -92.41837     -0.41727   -112.33511   -316.53062     -0.76892   -113.52423   -259.16176   -114.85455   -100.70845      42.7048   -152.35717     85.47121    -62.65522   -181.90838    -44.56639   -225.13927    -95.53645   -119.56225    -50.99981     42.61507   -174.01955   -153.75366    -99.83711   -231.88855   -363.14465    -29.44011   -191.46816   -259.18149    -43.14926      2.67665     87.55069     84.08479       9.6867     27.88277    193.14572    257.95423     66.05117    320.36196    279.47557    294.16443    177.67909    158.14045     113.9345    -70.52213    133.31868    108.37322    109.90501     29.33379    102.42845    -45.77984     49.13592    -72.82822   -185.97192   -156.67069    181.39329       4.2929     12.36519    -37.65432     58.28829    -36.34523     30.69269     67.58519    -105.5402   -129.40306   -209.12209   -106.96952   -256.73666   -156.01024   -181.03251   -146.03708   -338.50132    -12.57142   -206.17709   -141.19974   -147.26514   -268.09355    -281.2697   -209.37649    -208.8423   -151.39894   -336.30474   -178.06294   -145.55644   -127.05013    -45.99498   -212.95031   -154.27477   -317.91683   -293.53234     48.28952     -62.0968   -662.78089   -272.44826   -340.63324   -344.26625    -402.3364   -333.91173   -127.33772    -190.3993   -281.94818   -545.53143   -468.47936   -418.35559   -417.08871   -363.42715     -569.636   -458.72822   -436.89024   -501.45154   -442.72491   -519.43839   -665.36837   -760.07505    -837.5949   -852.53535   -747.87174   -614.25621    -822.1439   -652.45137   -920.01764   -755.62459   -647.26723   -567.60626   -971.85463  -1059.19164  -1024.24198   -806.85785   -1023.2801   -930.00284      -881.44   -949.61157  -1086.50417  -1007.44098    -846.6849  -1176.09377   -903.75575  -1080.30218  -1058.38812   -802.51737  -1196.70806   -862.83716  -1008.76687   -891.32491   -553.52668  -1105.73595  -1131.37037   -843.26646    -615.8642  -1556.43094   -708.28837  -1003.21836   -940.82841   -658.21029   -924.99508    -590.1906   -947.62987   -686.62274   -936.28475   -865.04794  -1036.26589  -1095.25694  -1061.91169   -481.33407   -188.50914   -682.94706   -180.80777   -818.70537   -338.61408          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        8.00   -657.58595   -126.68481   -427.57375   -328.88745   -326.11497   -347.67411   -321.28671   -308.49651    -159.9951   -379.37406   -134.22717   -390.83954   -345.86423   -484.58752   -358.28728   -273.38206   -195.61513   -380.50509   -315.55906   -406.92906   -422.38737   -115.00422   -257.17443    -89.19957    -47.62394    396.57721   1325.89629   1864.65053    1474.9573    508.04829    -81.45562   -198.77465   -302.25949    -404.2598   -436.30739   -472.25745   -175.12729    -89.31131   -499.24489   -387.57239   -135.30749   -352.77895   -402.71854   -276.60391   -559.14185   -452.08758   -373.67322   -563.96188   -184.54155   -117.68871   -361.13202   -380.87428   -554.11496    -376.1575   -485.95927   -333.05251    457.37506     720.4925   1539.56381    1323.8858    786.90711      3.35221    -75.24969   -122.37952   -334.83095    -313.2789   -210.48275   -261.03215   -246.82648   -106.99433    206.34915    1063.1814    1533.0179   1262.36711    850.04646    649.02053    672.56763    657.49639    164.41867     82.28521    -244.0514   -218.36088   -281.32145   -387.20469   -378.08385   -368.77967   -207.23701   -182.32194   -258.16403    -159.4312     75.15293    149.75582    196.92402     71.36941    -35.65572   -224.33774   -268.57727    -332.1476   -254.99083    -36.90978   -325.95083     56.44946    211.45803    100.63995     38.26081   -216.45011   -496.42114   -292.80654   -309.73614   -320.41701   -304.38443    -382.9777   -434.69439   -163.44462   -259.95209    -88.07241    176.98174    529.88601    261.25485    342.74544    -50.25494   -182.18788   -174.40625     31.43072    294.96217     402.6073    719.22451     553.1624    423.41318    167.61435    -50.25005    -47.08914   -171.37946   -458.40391   -243.17332    -38.60888    255.01322    742.09668   1571.72456   1933.85068   2084.69105   1776.35154   1151.19042    613.47875    231.64556    -63.32958     -84.5963   -195.09159   -286.65823   -172.60256   -509.01813   -418.49024   -335.75782   -326.77849   -134.17478   -320.98477   -273.55183   -166.68568    -286.6322   -219.07482   -110.43194   -246.95592   -189.02683    -61.81066   -154.44957     65.25383     90.20439    103.31737     26.21945   -103.47016   -133.94081   -380.62113   -148.37124   -253.89344   -240.72648   -257.15311   -328.68595   -201.61443     -7.02103   -159.15822   -185.66878   -280.75809   -420.90803   -282.38606   -200.46784    -341.6601   -317.69857   -365.28311   -249.63905   -348.36599   -228.56492   -111.45151   -251.12755     76.68696   -147.31945     55.24314    -34.61841    191.56112     158.9751    290.34715    134.91529    181.52436     38.23384      8.74184   -110.01483    -51.45755    -22.64522     91.55255    -22.18904     143.8718    -61.65172    -18.81898    -22.09167     12.48464   -164.89332   -284.94778   -289.87793   -379.33735   -316.27043   -345.56089   -332.78819   -362.17878   -321.40898   -145.79575   -240.03224    -26.27085   -146.46351    157.33277     52.11929    157.95581    105.20328    176.15522     13.25624    -30.91124    155.47844     -1.73425    219.86065     99.21461     -3.90101     158.0919     58.05328    -27.49532   -202.98475   -381.62318   -195.42525   -233.16183   -351.95369    -263.5211   -468.72116   -118.54746   -166.70437   -189.71986   -148.37007    -133.5487   -163.52143     -92.9137    -72.04085      -5.5374     58.16055    130.05217    249.77083    343.25741    346.55399    649.01365    805.85503    636.31312    596.18853    728.50565    525.93792    502.99787    198.99094    200.09106    152.48966    -20.55972     93.78883   -254.31677     44.38793     21.71405    -41.51849     78.90361     107.6597    111.31162      89.0247     70.46098     13.95655    -82.54642     -5.30245     -280.015     -4.87135     45.45964   -121.90147   -281.63203    -427.0977   -234.99412   -170.08047   -457.12927   -276.68706   -236.29208   -316.01167   -472.63375   -256.98369     -98.5161   -247.64237   -496.57796   -334.61201   -145.15584   -140.80596   -422.52937    -128.6562    -90.81329   -159.54967   -268.84049   -278.32931    -71.58237     47.70869     43.98116      -71.653     31.50287    -112.4739   -230.68072   -199.66637   -157.70528   -120.52815    -93.83065   -114.01155    -80.58302   -230.97279   -125.37615   -268.45568   -281.07335   -105.58782     -80.1348    -100.8461      9.29713   -129.33976      2.37127   -103.30495     -9.89222     -3.57589      87.7963    101.54433    -15.57491   -367.59528    -295.1624   -176.13321    -88.30406   -322.61566    -80.19817    -91.74996   -166.81274   -124.56758    125.25701    -55.80207   -181.32713     -63.4824    196.82505    180.31326    347.19071    117.40265     54.96434     77.99791    392.57284    335.99172    145.95721      95.6167    223.81521     -1.57889     56.05512     14.95119    106.01551      9.14506   -192.81674   -186.18299     12.52677   -290.89314     52.03963    -71.16813    114.80449    -55.08533     30.78854     69.40285    -77.36898    -96.71687   -136.08732   -146.68362     -34.6897    -83.45725    -63.40752      0.65726   -149.99677   -128.47874    -98.74393   -222.37378    -94.04532     47.08204   -174.83047   -273.66505   -351.72034   -200.67412   -285.86009   -243.55391   -304.19206   -262.54393   -224.04423    -17.16682   -189.50132   -540.88599   -227.76083   -286.19678   -202.65311   -201.63849   -215.16906    -370.9584   -216.90245     10.76902   -206.68853   -353.37517    -62.98638     11.07537   -247.20683   -241.87812   -473.90757    -92.78973   -361.73952   -460.82036   -383.44229   -431.08867   -284.73961   -581.48555   -305.55461   -740.68746   -757.67118   -362.78247   -486.80494   -403.90743   -694.85977   -896.18932  -1123.05725   -717.92746   -599.48837   -459.52648   -696.44328    -904.1111   -812.87107   -863.46121   -968.41582    -780.2866   -699.55504   -926.49153   -888.07845   -920.16268   -821.06972   -842.42666   -816.84033   -740.21662   -828.25192   -835.54738   -891.29202    -1007.696   -724.09243   -866.70647  -1258.39697   -816.89672   -899.19412  -1298.74691   -847.65391   -638.88455  -1034.77739   -978.67109  -1248.10956  -1073.86065  -1030.87349   -924.54562  -1427.53771  -1530.37073  -1194.84428   -1411.2442  -1040.39338   -514.08677   -753.06894  -1276.12187  -1446.25345   -1227.2235  -1352.65345   -107.65485   -352.88951  -1497.71807   -401.35445   -631.42437   -984.89903          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        8.50   -607.31772   -280.80785   -309.53316   -375.42239   -330.32513   -395.47021   -335.23948   -338.87286   -283.17287   -401.94612   -158.61739   -421.52433   -243.74589   -389.74332   -413.99791    -295.2632   -269.85406   -155.77969   -316.28109   -423.68512    -293.1332   -111.59364    -202.4185   -341.35233   -242.97142    458.58156   1290.13519   1916.04832   1512.63325    632.38555   -411.30941   -249.77242   -280.41023   -295.49504   -357.68744   -394.80423    -16.90002   -197.23481   -550.42215   -517.52244   -469.83804   -353.14085    -341.5948   -196.00223   -415.83691   -465.98386   -415.57864   -419.53867    -370.4928   -382.91106   -198.96743   -520.94693   -368.87764    -385.0038   -223.05349   -149.27303    328.71753   1069.88661   1466.67622   1371.31679    671.22911   -155.43184   -233.15131   -228.86109    -278.6791   -402.98478   -425.40721   -195.15039   -235.03019   -192.75611    392.17584   1012.23709   1491.09455   1388.61582    817.39731    564.37656    594.44012      593.878    265.21983     82.65348   -248.84426     -48.2174   -429.42059   -397.70589     -254.812   -277.28092   -314.15625   -389.91182   -204.76344     -170.089     -20.4234    134.77548    202.72222    150.60669     37.85315    -85.74324   -372.05994   -460.14028   -258.23181   -153.54636    -27.57867    -32.59088      37.8498      78.3502    -84.47584   -262.12158   -395.98682   -360.46314   -248.10165   -318.27508   -380.53552   -455.54663   -413.85599    -351.3542   -172.89351   -175.82211    139.91542    158.62866    293.06547    264.57687     29.23528   -146.86788   -167.17722    -30.94294    199.30374    408.79743    538.55345    600.39066    434.02882    155.67434      11.0136   -151.77754   -234.94752    -225.5604    -96.81242     65.56641    378.96232    842.38394    1402.4427   1909.37538   2016.10254   1761.18905   1185.76911    570.07174    155.18327   -163.88859   -184.01531   -267.30154   -150.84026   -193.69814   -269.48263   -274.76416   -295.13659   -222.06367   -306.54051   -176.65266   -248.69918   -319.18866   -321.75619   -113.55583    -75.85971   -142.38198   -216.84573     22.93371    -63.02064     13.26028   -125.56499     181.6514    118.44057     -93.7171   -207.79726   -340.46907   -297.46308   -367.60277   -270.20362    -118.9289   -308.46025     19.29705    -213.7779   -290.01041   -231.44166   -207.07727   -319.43383   -375.99285   -273.74375   -467.15247   -393.14614   -327.67378   -527.02728   -297.71671   -324.37938    -98.30928   -173.64671   -286.32734    -13.77866     23.11266    -32.89842    293.90106    211.69055    142.26711    331.92943    123.12439    115.68005    -95.49824   -100.23282    -65.55958   -127.17491    -81.61048     -0.03288    181.85534    -51.96138     58.31926   -148.93391    -31.84712   -238.03596   -217.24169   -235.72226   -294.90075   -238.68583   -181.52667   -184.23056   -217.01109   -448.56718   -328.30924   -114.68653   -185.55891   -270.57818    -74.41666      62.8674     23.08894     25.70509    135.01349     57.58439    -38.41859     18.96819     -3.88111     90.51112     29.58618    180.90235     14.09121     82.52938    -61.02339    112.37947   -271.31101   -158.71099   -173.97701   -322.23716   -253.45813   -122.52534   -266.75704   -198.16117   -124.23214   -214.36423    -171.1383     35.87714   -122.85058   -121.92958    110.51623     92.33341    305.35245    337.46149    510.75759    368.10847    587.62427    758.58155    660.18139    810.35938     598.4729    428.92193    405.94957    300.61232    219.65532      32.6886     -89.5285    -19.44055    -33.61626    -71.38748    -30.76951    -24.80738     22.35799   -122.04863    120.74907      4.04132    -12.37202     76.62053    -25.44019      2.01618      51.1619   -219.71629   -287.59404   -295.71496   -343.95076   -282.02928   -379.09857   -251.32857   -267.46772   -426.87443   -243.33462   -301.45574    -85.99529    -266.4182   -307.58028   -262.47305   -288.43014   -184.08831   -136.17983   -131.92028   -293.91395    -122.2214     16.67679     25.10755     40.53343    -66.95861    -23.46951     26.06044     56.68498    -14.60603   -121.82588      -102.28     38.74145   -112.77357   -126.39442   -230.88443   -137.12272   -217.85475   -174.17733   -147.68042   -211.35999     -146.133   -160.34874   -134.01517    -98.95898    -18.68001    -153.4405    -24.71065    129.37514    -23.83541     79.31545     -7.75814    -62.14285   -127.26235   -122.82828   -187.80973   -218.10131    -45.11251   -297.45727   -130.87335    -39.86033    -93.57943   -129.97199   -155.09965    -63.52682    107.85256    -55.32239     36.57272    142.61717    154.70649    168.34412     42.44597    348.64965    271.11396    368.31125    163.07133    339.48575     155.3053    159.96956    105.06381    136.67645    184.24586   -137.91351   -237.12756     95.20569   -307.66917    -16.35526    157.17061     -5.03859    -95.91466    319.82068   -139.99854   -154.47744    114.45836   -135.37837    -26.04994   -130.34804   -142.79901    -65.48259   -109.08889    -51.10392   -293.19292     -61.2408   -293.56007   -205.94994   -152.77071   -145.38911   -124.98892   -155.26458   -417.64348   -170.02466    -92.41295    -77.32636   -270.16618     -87.7805   -139.72237   -115.58175    -227.6955   -420.34187   -267.81187   -225.22453     -72.4211    -278.4099   -443.74417    -262.3543   -156.87725   -376.33666     29.36526   -343.53035   -446.82291   -260.42922   -230.91537   -220.91291   -167.46992   -302.94965   -341.68378    -304.8585   -265.81687    -450.0244   -561.52865   -591.05623   -788.10186   -492.41754   -302.04293   -646.67907   -611.98538   -681.49459   -709.35359    -712.9248   -676.05544   -503.56603   -700.76277   -558.08956   -786.17292   -889.54408   -737.23444    -712.1656   -824.92793   -798.05452  -1030.72075   -790.45704   -914.53299   -947.09157   -780.27641   -893.85472   -898.38996  -1088.62386   -942.52171   -981.74044   -871.23276   -987.63588   -872.43914   -955.28331   -799.85218   -964.60426  -1213.35994  -1009.94131  -1411.97178  -1028.43407   -464.34898   -927.89494  -1053.64325  -1023.90109   -646.81815  -1288.37904   -805.71141  -1036.77094   -756.03314  -1019.04973   -645.15942   -914.69438    -965.1692   -594.88744   -431.93442   -956.61627  -1190.31077   -920.38028   -632.75669   -730.19906   -913.13369   -669.39884    -524.8879  -1049.46439          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        9.00   -385.26966   -413.98196   -205.37017   -302.06691   -374.24084   -442.95604   -295.28436   -342.53124   -406.35282   -355.59717   -283.56302   -449.23653   -157.75573   -479.92667   -455.79335   -297.74801   -226.62961   -293.06404   -258.88751   -355.55614   -340.63876   -135.75251   -275.67098   -483.10953   -110.46584    431.88841   1417.14804   1835.35023   1511.52365    748.50733   -400.80712    -91.96025   -327.32816   -237.38699   -287.14435   -288.29934    -225.4336   -265.31603    -376.4001   -434.13097   -485.61213   -324.28018   -179.43301   -169.38191   -256.10451   -256.02898   -456.49372   -269.50432   -260.04851   -367.77486   -403.75831   -193.09005   -323.82339   -282.24811   -308.50673   -158.27659    325.45864    927.20935   1439.93047   1364.01927    694.32779    -32.46364    -69.44079   -309.97597   -338.52281   -346.52125   -331.88981   -271.09432    -242.4635    -38.39037    364.81914    885.99157   1516.62639   1258.75292     668.1157    734.82834    676.53133    531.44286    223.34559   -115.57136   -178.01304   -258.36243   -329.04659   -399.87192   -371.52477   -270.52292   -311.72547   -301.57294   -271.15176   -111.55186     166.5443     25.88699    148.85923    134.09459   -147.72221   -240.85109   -238.25475   -258.97655   -356.88591    -161.2262    -18.92129     25.34926    108.86051   -110.75402    -54.55836    -77.12538   -309.57878   -223.80241   -248.48667   -320.21058   -284.28916   -351.11033   -425.54573   -301.64045   -125.16778    -76.23637    173.80267    265.18646    291.61821    260.19403       0.8402   -148.95659   -136.31438     10.34778    174.48605    336.42603    685.58568    714.78578    468.34237    147.21836   -207.50352   -248.61876   -342.03297   -209.90465     -250.239   -251.83516    263.72942    912.44298   1549.54524     1926.534   1928.36985   1559.43435   1023.47471    671.26762    190.07521      9.94607     -3.56041   -297.84862   -220.59733   -314.47269   -227.17827   -244.52691   -286.86824    -337.4521   -245.07144   -143.42119   -152.84339    -89.82106   -275.60454    -33.35674    -81.27654   -175.71439    -193.9659   -140.03857   -195.49827    -25.36856     88.40005     94.95418    136.80129   -198.67851   -135.63754   -217.77643   -247.89893   -247.27201   -215.86833   -152.14238    -54.13738   -119.83382   -191.91528   -149.37841    -171.8926   -317.26121   -157.35283   -447.51923   -467.72667   -424.52234    -286.0587   -393.55675   -435.67649   -210.86845   -290.36548   -324.29185   -210.06899    -73.93681    -56.64486     -0.60816     93.44301    190.07486     71.84108      262.317    320.70833    141.15174     88.14323     15.65434    -62.00569    -57.53264   -122.38433     38.76529     90.90648    208.79038    147.97775      -42.112     84.56077    -109.7919   -153.45752    -148.6046   -318.06396   -265.15634   -436.65021    -338.6687    -290.5077   -334.81849   -265.24286   -288.96947   -318.88604    -109.1019      9.57017      2.19072    -99.57143     60.95177     93.73778    -23.23915    119.38862     71.81481    -68.15915      250.448      2.71134    168.20114     22.75073     40.70572     41.29868    147.57659    -23.47727   -186.22678   -273.68438   -185.14166   -139.05329   -408.06806   -144.20052   -334.53151   -258.57404   -158.58975   -196.85154    -210.4522   -352.63224     -4.26097     35.66917    192.00513     151.9006    354.01174    421.18237    502.49256     699.5126    576.45807     698.0364    655.95198    894.30087     625.9322    490.84838    369.24596    391.78285    181.60449     73.51695   -152.39932     -5.04463    -27.04695   -104.77945    -41.35603     10.73155    -48.92366    -52.37177    140.87171     62.89861      8.08711     27.89716    106.63721   -118.86606     18.47411      1.42462    -261.1078     -90.5652   -239.88051   -131.41898   -321.13218   -345.31415    -225.6933   -435.84906   -300.77407   -350.05931    -311.3913   -324.03378   -250.69212    -293.5371   -322.38838   -100.12328   -234.67464   -121.43178   -232.07566   -168.71738   -212.05754    -13.16875     -99.1325     129.4754    -20.76021    -83.30285     -8.19895     85.64749     10.85064    -27.78809    130.48973     14.16822   -188.74304   -188.69002    -90.97741   -153.99752   -200.47734   -243.58315   -227.27739     -335.576   -176.76465   -220.61187    -76.64476   -270.25735    -28.38211    -87.66933     99.96533    104.75615     21.80408   -152.38928   -190.27999    -17.64113     66.86335    -189.0903   -176.63688   -155.65691   -204.05654   -191.33005   -186.72571   -109.65838   -111.21502   -119.18918    -71.42727    -82.04923     56.92702     67.97035    102.10127    158.89383    277.21694    144.44232     62.65247    204.74243     13.68954     257.0939    168.44505    127.36845    243.69036      89.2546    308.50878     63.82339    128.84982    -104.9107     -10.5094   -209.61834     97.17457   -144.28032    -59.00225      6.33287    -14.01431    133.42279    -83.84128    -46.28624   -197.13604    -90.74998     62.22706    -27.52563   -206.14563   -131.38538   -240.37485    -87.93458     -191.563   -149.18203   -263.85158    -60.78292   -206.70701    -70.54385   -274.49225   -390.53271   -397.89231   -359.67067   -371.75644   -130.76657   -115.07402   -188.85605   -294.11124    -99.80946   -275.68138   -223.26824   -231.00802   -335.24406    -197.9388    -184.2814    -161.1999   -128.55519   -166.97827   -165.86194   -394.41048   -144.75283    -40.30575   -229.33588    -28.07238    -58.11271   -257.75264   -361.26186    -550.3598    -61.26744   -132.85285   -313.60741   -328.72162   -758.78671   -588.72787   -325.77306   -484.25794   -595.00403   -926.59297   -496.79198   -625.29262   -810.31633   -528.36207   -682.16959   -747.70991    -739.4468   -526.28849   -793.92178    -796.1448   -887.90654    -848.3342   -835.01134   -896.67002   -762.02181  -1136.08045   -813.62177  -1098.53996   -941.87504   -1033.5203   -801.40284   -829.41215   -903.84511    -775.9847   -919.03542   -1003.7324  -1030.50044  -1069.12012  -1279.46321     -634.481   -844.98002  -1094.27275  -1072.47833   -920.91362  -1126.93183  -1308.90084   -709.34559  -1335.26479   -509.31809    -997.8067  -1401.14472   -880.07625   -1253.2529  -1087.05834    -688.3397   -284.18025    -612.1364  -1031.14525    -770.3111   -621.87961   -885.43758   -747.34276   -562.04555   -771.94225  -1380.47683  -1010.35152          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        9.50   -195.72762   -460.34892   -283.06709   -207.05087   -408.32179    -391.8679   -208.60462    -332.9652   -385.80742   -222.20236   -359.01389   -385.78386   -229.92423   -561.30153   -390.62454   -302.38014   -109.74663   -405.92217   -184.30866   -215.04323   -301.97094   -296.53497   -425.05632   -399.98478   -258.56618    125.94098   1319.56542   1902.04934   1750.17997    684.90437   -359.21353    -344.0385   -312.15289   -372.82894   -373.64422    -212.7013   -427.74366   -184.87441   -402.77948   -422.25498   -460.30275   -355.99701   -221.70313   -310.42202   -357.50255   -242.82302   -389.26126   -129.08765   -217.79158   -355.62691   -226.86525   -236.44174   -307.70886   -324.30845   -190.30431    -15.49895    228.93139    946.91113   1482.15096   1425.21983     578.5538    -65.36208   -170.07846   -258.79422   -269.73383   -242.43397   -199.12806   -307.10643   -150.26994    -41.81721    485.05437   1243.45604   1502.68315   1328.39057    691.74752     451.8208    561.49164    469.18398    272.75694    -86.23209   -121.23337   -198.95251   -241.50957   -521.39373   -450.56015   -377.07572   -403.84328   -264.78546   -241.39641   -231.10201     -0.91028     303.7459    126.50517    105.99694   -247.72154   -213.16737    -274.9613   -285.96493   -332.86324   -320.69754   -149.65451      4.59042     -8.41651     -8.21593      93.7516   -200.57072   -387.57488    -345.4695   -358.45657   -358.16586   -407.68704   -439.80167    -186.0128   -283.33994   -201.99123    -50.46625    273.23517    329.73387    206.80517    169.55189    -57.38471   -129.99214   -226.55035    -25.73277    146.58448    512.73954     573.1002    478.26757    395.99829     35.42057    -75.65595   -246.50105   -224.08617    -134.4153   -227.76776   -150.19283    285.51432    983.23125   1364.09247   1983.91295   1959.00396   1814.63006   1167.17254    625.65218    295.72877   -167.97448   -197.49806   -173.43475   -117.17836   -107.92174   -333.73664   -278.15907   -354.08385   -400.72182    -172.3497   -101.93325   -189.88323   -290.54219    -104.6373    -96.46247     21.32829     -8.39236    -40.85655     75.56595    -56.59754    -38.30707       97.155     40.49991     -3.54411   -128.64082   -227.37705   -201.24615   -322.70151   -327.98363   -343.85103   -258.03669   -250.69127   -167.48161    -59.42069    -170.4685   -125.33563   -272.90832   -216.18042   -200.41966   -380.72973   -357.15093   -510.80869   -331.21548   -324.61721   -367.51803   -157.62808   -231.06502    -99.86994     -94.5393     19.41354    -64.21776     15.39049    123.19466     110.5803    224.23352    221.23472    174.32181       3.6357   -139.10835     29.94476     -80.9193    -69.92779     10.69921     45.08304    126.90317    141.57241    155.48261    -77.60192   -111.68856   -230.83344   -309.89365   -312.53772     -269.704   -344.15035   -238.96325   -327.31622    -95.46957   -250.99445   -270.48904   -126.39555     42.26951    -70.63378     -35.3486     18.31461    161.89106    146.82994     10.36185    220.49453     78.51202    209.78858     94.73102    -53.98175    150.63281    -39.59302    101.30061     71.79155     27.80254   -144.18262     79.89769   -140.37763   -183.98517   -256.76779   -182.80692   -353.51153   -235.85975    -279.5945    -50.16474    -222.7874     -25.9892     -81.8028    -46.56283     65.96594     87.65587    192.19847    234.88037     427.2844    389.09181    576.38795    626.75904    781.76874    781.78646    707.59897    530.71337    563.84661    409.12129    451.87564     68.41522    111.39153   -122.81225     -61.4107    -98.14984   -142.79775   -113.67642     52.16141    -54.50599     35.09812     21.76161     72.06375    285.30466     73.97887    126.97895      82.6305   -123.02553     -123.705   -130.54736   -105.11482   -393.51315    -46.55417    -300.8963   -484.10525   -201.00644   -209.66156   -132.87051   -305.72808   -414.81571   -302.33358   -345.68786   -130.77275   -255.04157   -216.01202   -174.93885   -127.10479   -104.76766   -116.03908   -131.56743   -109.75118    -74.01249    -24.45707    -82.48617      25.7399     98.07607    -54.88146    -29.90372    -15.26551    -31.94674   -132.36979   -128.21191   -142.73768    -55.83491   -217.34487   -147.56277   -105.46756    -102.6874    -93.81468   -158.90091   -200.39859    -40.08803   -157.24796    -98.37446     23.78134     55.40471   -132.58759     37.38291    158.56826      3.78035    -18.77318    -246.2305   -297.23363   -394.10938    -70.61983     26.31008    -31.37262     49.08691   -399.17487    -76.44964   -332.53112    -61.46173    -26.51025    -99.77998     74.68789     84.35817    103.39613    180.85394     293.6626     271.3734    178.80198    443.80215    186.47792    181.48641    149.88251     274.8883     73.06799    114.31538    108.57092      4.87013     23.44395     -102.293    159.19742   -135.20424    -14.34442    152.91641    -52.96485   -102.87228   -304.69449   -243.20265     245.1127   -271.99917   -128.73742   -238.18591     77.26114   -294.94748   -496.00511    -65.33366   -295.76649    -218.6038   -305.08356   -171.79046   -375.28511   -193.23294   -198.26796   -259.56742   -188.83498   -217.11175   -229.28324   -238.33537    -344.1222   -127.97188    -227.1095   -186.89577   -102.60891    -82.58813   -143.19517   -231.16782   -226.62907   -130.52183   -192.86895   -240.21302   -214.64003   -109.73498     76.23537   -278.29494   -117.37356   -141.82534   -160.28701   -355.98466   -159.52896   -160.15342   -580.67745    -152.0894   -426.34762   -460.49734   -462.22458   -508.37997   -384.47553   -448.64477   -331.40958   -634.15317     -425.313   -617.56583   -741.43045   -545.72254   -922.08321   -651.44136   -803.46273     -666.944   -827.17186   -774.66525   -968.93002   -958.36026  -1011.86654   -967.08675   -880.56873  -1062.39211   -573.83039   -902.59037  -1003.88394   -954.14223   -685.78884   -922.71181  -1215.68236   -755.66025   -824.48108   -879.66443  -1018.55887   -880.45901    -823.0669   -928.41882  -1049.85368  -1013.81012   -842.96611  -1017.18865   -997.98752  -1197.14156   -813.63522    -983.9802  -1002.71993  -1058.31933  -1049.87016  -1135.09387   -663.06741  -1088.85471  -1114.50414  -1027.39468   -975.73675  -1060.38561   -952.26015  -1432.14481   -652.48612   -964.48464   -1058.9616   -878.01041   -312.89267   -434.68248   -799.98696     57.95643          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       10.00   -109.28842   -353.90877   -390.00537   -192.99238   -387.42755   -368.33728   -223.94317   -357.30657   -380.84972   -187.50364   -435.38461   -273.03993   -263.86799   -339.25757   -304.88922   -389.27836   -293.14794   -513.43154   -353.83983   -425.75246   -160.24455    -212.9871   -271.79053    -181.4008   -205.27126     355.3702   1085.25095   1986.56888   1586.48198    482.57259   -302.28943   -179.85689   -328.22285   -450.29841    -353.7915   -121.44749   -299.80787   -349.23632   -361.30141    -369.9116   -344.95916    -349.1496    -298.6384   -362.55232   -355.47944   -306.95966   -314.81099   -316.89792    -243.0524   -407.67942   -328.11286   -409.75537   -439.00252   -415.17135   -296.61811   -137.64173    381.64481   1086.63534   2103.76547   1388.89527     486.4968     296.5307    -326.0705   -292.71538      -9.6386   -358.60567   -307.12606   -362.60871    -257.9602    -57.27072    324.70973   1058.95583   1394.39487   1196.06942    708.11733    647.41765    669.22578    416.86662    154.94645    -35.25762   -118.90398   -405.29798   -274.83878   -347.02166     -219.007   -412.64176   -299.96105   -311.86579   -188.19232   -243.33744     13.56636    140.82453    168.26621      120.858   -209.80568    -64.96523   -318.99534   -264.63871   -245.76179   -124.11682   -124.55906    -66.46026     24.30248      34.6745     16.20337   -230.76426   -333.83322   -240.85328   -393.03195    -321.8611   -446.47399   -462.38691   -415.46486   -330.88425   -249.06518    -91.39724    251.37671    311.16617    238.61545    232.10675      6.62605   -144.12906   -205.44766   -112.34004    195.01458    574.99321    612.35884    596.94935    356.81598    133.83801    -81.38199   -147.39218   -267.57737   -185.46008   -332.41003    -81.27045    299.75218     771.1289   1487.42258   2008.20266   2120.21285   1615.71024    1171.8608    530.96265     175.1275   -168.18342    -118.5017   -200.17977    -194.5387   -160.04744   -240.10508   -295.24253   -226.07088   -339.92829   -388.73098   -358.42206   -237.59781   -224.90415    -40.52427    -61.78883   -132.17015     46.10205    -24.49715      29.3916   -207.56806    -63.91274     21.78604     80.53758     32.95936   -154.42113    -27.80519   -337.84469   -320.63139   -316.23112     -52.8854   -342.94552   -365.52593    -276.8093   -132.36637   -300.40081    -177.6713   -173.64269   -327.72993   -279.85697   -449.18861   -526.12809    -413.1965   -231.67569   -187.80315   -164.94026   -330.80415    -288.2575     -7.76449    -64.31292     41.00899    -12.33043    -34.46042       94.017     256.0053    354.96191    252.37131    199.04469    145.66627    -72.33312    -72.75235     58.05319    -64.47764    -55.00906     -5.92031     70.65151    112.88062    -99.02134     51.07574   -133.39455   -188.62713   -336.25914   -200.51544   -316.15636   -222.54671   -163.00915   -339.40034   -230.36263   -328.19375   -322.78805   -142.82949     23.57119    -53.62124    111.25057     193.7342    -30.87598     44.87354    227.81001     93.80727     -46.8923    198.56614     72.64966    106.26802    134.34207   -195.71181     36.21762    109.02279     -3.44275   -105.14402    -23.95658   -107.90202   -352.16215   -149.52149   -207.98358   -265.39074    -280.6818   -386.61175   -169.08223     -3.62126   -132.36691   -142.81188    112.75787    -84.92601    176.69603    224.37537    210.91318    284.33493    409.19232    417.24652     686.3908    717.68494    644.64183    776.63594    724.29858    370.48297    489.89025    233.79703     200.7362    147.16439    115.57504    104.42651    -76.89456     -0.01975   -113.03138     51.69858     22.09124     96.64679     23.73183     19.89298    -26.57033    231.54833    180.15807      24.0929   -138.64438   -167.77818   -312.39795   -191.88673   -196.52736   -154.57757   -386.33767   -167.91491   -357.65477   -414.69093   -317.06254   -334.80568   -402.66734   -333.69069   -438.56568   -378.78595   -283.60643    -229.2571   -170.73936   -186.79883   -232.44716    -177.2195   -325.81993    -87.50983    -66.26923     62.97641    -68.13914     30.03338      95.4422     -1.70286     42.90296     37.29838     -23.3337   -104.45747   -100.62609    118.55123    -12.96469   -140.67032   -114.98844   -104.85707   -209.14872   -143.90711   -261.40187     25.80842    -76.08983   -111.60576    -59.08164      3.53314    -46.87803   -118.61974     14.26507   -122.84452      7.19207   -255.72882    -50.56675   -159.80888      5.11203   -138.80551   -185.81118   -127.51346   -263.70637   -151.47833   -165.89157    106.77252    -76.03655     42.65444     -70.2318    178.80942    -83.99979    105.12423    108.95911    290.61715    178.71445    -16.53475     74.74915    178.03562    166.01592     22.47022    191.15252     92.57613    209.44405    -67.00728    185.70908    -22.15834    -64.15217    -33.88986      21.4857     -54.0599     29.79409     93.23347    -30.60663   -160.27102    -20.37044    -22.47072    -96.79575      68.3732    -72.32441   -204.45711   -161.98796   -130.98334   -305.01955   -306.05754   -467.43245   -228.00636   -481.99418   -145.45818   -178.66466   -254.96853   -398.60403    -290.5408   -208.81441   -254.49351   -245.81242   -291.88731   -109.74482   -317.51255    -46.07345   -119.76685   -340.14842      2.60228     -415.852   -122.29656   -348.83996   -282.81632   -313.10448   -161.09743   -561.31618   -299.01387   -265.28791   -241.82642   -347.59609   -374.96515    -214.1169   -289.00484    -75.29328    -213.4737   -325.36187   -336.75157   -560.14711     -236.229    -121.6922   -554.67074   -409.61785   -518.24671   -325.96714   -736.64949   -551.86768   -703.50605     -716.372   -828.04417   -527.58957   -711.77362   -659.41797   -848.36247   -665.23992   -790.14004   -912.55908   -679.05984   -855.47227   -755.83929   -943.16594  -1137.88801   -780.84722  -1036.41352   -933.02656   -834.33031   -761.61994  -1141.32075   -835.04855   -778.07931   -642.98231   -781.82843   -569.60578   -999.60887  -1308.74371   -910.92076   -1070.5648   -904.74588  -1195.05738  -1088.67268  -1192.26215  -1233.15418   -893.37801   -843.00068  -1243.40387    -483.6067  -1453.68157   -1081.0819   -599.17136   -827.88695  -1120.90861   -953.43816   -452.32638  -1058.68749   -890.14618   -678.12322   -797.39421  -1060.66907   -333.92689  -1007.80469  -1269.94963  -1506.10901   -735.10791          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       10.50   -142.70509   -263.52051   -426.12232   -290.33789   -369.54997   -403.61785    -361.8301   -420.83917   -426.24504   -281.38204   -546.91708   -293.98119   -262.07895   -220.41607    -350.3719   -501.25686   -389.17362   -399.57552   -606.94489   -430.05398    -91.62145   -103.19576   -310.25527   -141.26867    -69.19081    483.94007   1315.72419   1888.29011   1517.31692    513.37891   -234.17528   -241.13805   -404.87512   -372.96571    -372.6342   -345.04565   -277.26203     -285.352   -308.29558   -341.76271   -446.23937   -472.68339   -399.90684   -245.29548   -351.80465   -341.15624   -485.84598   -281.88649   -398.75615   -693.84076   -198.65589   -239.21881   -276.21378    -334.9845   -231.44935   -193.45266     290.7888    785.06259    1509.3642   1409.50861    673.12586     57.03926   -201.47695   -328.01937   -277.48856   -196.77786   -352.01492   -184.22769   -381.38721    -60.19975    394.87643    880.64582   1398.84418   1329.28819    901.24528     622.6796     640.5088     433.6007    148.73821    -64.50946    -217.8358   -452.11874    -447.5109   -356.66705   -408.81533   -449.90269   -300.83774     -430.389    -96.92924    -294.0608    204.75629    132.98337    191.58366     29.68066   -151.77512   -228.90075   -266.19199   -380.01226   -276.55002   -257.35692   -142.39023    111.65659     46.77353     78.39388    -40.35863   -203.80019   -278.33114   -309.08055   -319.83335   -247.64599   -242.88076   -271.39775   -306.80967   -313.84792   -301.06921    -61.96634     81.63788      385.567    291.75522    306.64255      6.50982   -163.13317     -4.71843     91.08062    116.59407    433.32204     622.3019    460.76782    448.61923    166.03605   -204.21865   -215.52984   -176.88723   -276.75154   -187.03187   -110.01289    281.07382     956.2227   1388.59945   2077.76409   2017.78317   1581.71864   1070.59257     706.3579    304.79934    -47.26167   -143.02208   -250.63932    -80.40272    -69.54122   -154.44049   -381.47588    -283.9168   -328.76594   -477.02017   -300.64952   -251.63696   -220.26043   -157.79567   -177.75031    -76.47375   -119.11483   -162.23409    -122.0829    -36.45258    -67.98515    -64.06355     20.65093     97.84706    -72.18387    -32.79488   -202.59111    -346.5384   -275.64838   -273.96514   -112.75826   -117.68661   -188.21479   -277.19374   -205.66897   -169.02709   -153.45382   -260.71672    -350.6404   -445.34951   -363.95648   -396.48854   -273.18922   -373.80197   -466.64984   -211.01365   -225.83301    -97.99555    -61.52873    -15.47105     48.77447      41.7834     120.0611    275.82684    352.22526      90.0014     38.73999     -7.46609     93.82452      2.94009    -44.06329    -50.00351     26.73852      3.84952     44.05012     27.08941     30.86599      5.03728    -34.37787   -292.60413    -109.3495   -378.44858   -224.07263   -275.89199   -417.10724   -304.48627   -403.79468    -71.17908    -36.38164   -218.79896   -193.94518     57.93247     43.67674      9.79759    131.98568    187.55827    135.77445     34.22071     18.18577     29.55337    -81.44352     67.41615      94.3398     261.4258     46.97544     39.29749    -37.66446    -47.27505   -149.46258   -317.42825   -206.90257   -272.60323   -267.56574    -300.4493   -254.42078   -217.13356   -125.54377   -299.93482    -92.25432    -10.35581     33.70569     39.90359     68.25197    111.22051    121.36157    288.10125    321.68457    515.39486    811.89402    659.54061    764.32104     705.7134    759.11599    661.05495    517.23304     286.5568     61.97646    -78.14795    -26.61536   -222.15591       7.1091   -235.86102    -98.65485     28.58073     67.95233     92.20812    -19.28903     86.53933     19.20906     14.56003    105.26621     71.63928    -22.76379    -141.3175   -248.69641   -393.19847   -181.86541   -290.12839   -327.21714   -283.55108   -434.26915   -349.95821   -306.71981   -243.40508   -317.90766   -321.19801   -241.03843   -234.88171   -291.01562   -175.23484    -279.3927   -117.39687   -209.16831     -214.662   -224.84195   -107.36949     96.51534     -13.2615    -41.40093     -8.32881    187.82257    141.54463    -66.85699      -8.5428   -125.74054      95.8637   -213.85385    -30.46955   -106.88066   -194.56983      0.63922   -204.56088   -145.05119    -31.39042    -95.39875   -141.92964    -78.83814    -64.20477   -135.56405   -200.33039    -51.30984     -85.7688    -77.81879    -179.7228   -117.33912    -68.21683   -243.40396    -46.71639   -162.30389   -216.15056     13.78974    -90.45943   -187.86427   -167.74219   -103.62566   -166.28689   -124.65354    -57.38764    156.46239   -147.90137     99.71495    -64.97535    209.30467    165.93847    122.91518    107.17848    156.35477    -39.52284     411.2746    259.87824    156.67332    175.82774    188.00496    -23.85987     143.7591      5.89873     41.98272   -252.79214   -224.34578    -13.99046     33.21209     43.65929     -3.25892     -3.83787   -119.70801    -43.41933     18.29765   -233.97824    -55.40572    -63.12783   -258.58531    -64.18208   -152.95972    -24.84822   -344.91232   -239.82608    -36.24616    -23.61534   -490.30886     27.03985   -175.67789    -157.6744   -286.42269   -140.23817   -233.77153    -98.68625   -221.61147    -91.18733   -223.64482   -396.93989   -216.81676   -130.60309    -246.7803   -248.70279   -162.78615   -269.90633   -515.89743   -156.25409   -242.21952   -223.84833   -326.18475   -323.04196   -200.25984   -195.59796   -265.00463   -410.24625   -496.23506   -146.19202   -232.48084   -437.27537   -675.49202   -431.48261   -952.98635   -348.43357   -564.76188   -407.10338   -442.84754   -566.72601   -593.35219   -417.12026   -787.29487   -889.78411   -672.62296   -793.88552  -1082.40201   -698.38934   -777.86325   -842.31922    -781.4481   -870.15502   -823.75174   -601.09205   -723.95165  -1126.37183   -664.39458  -1145.08886   -918.56679  -1090.16187   -932.81594    -974.7616    -894.6217   -813.88375   -870.04698   -1131.8918   -854.26218   -954.26019   -776.44876    -778.6933    -837.5708  -1095.81101   -811.07145   -660.53455   -834.03155  -1057.61324   -985.30469  -1534.60079   -851.00661  -1019.50111   -588.40932   -629.23624   -763.18483   -849.28835  -1130.66661  -1161.91873   -763.70866  -1191.08617  -1231.16012  -1019.97245  -1567.55781   -901.57278   -412.13604    -313.5582     156.5863   -592.74291   -716.95547          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       11.00   -148.15374   -231.05878   -361.83049   -368.37124   -337.43217   -469.56626   -414.22431   -476.79047    -468.2908   -365.20694   -492.03658   -306.95544   -386.31535   -317.58031   -314.82808   -606.68272   -375.58126   -356.55994   -428.45354   -325.09297   -198.35141   -353.80528   -300.12986   -175.52222    -19.26219     462.8232   1350.61072   1674.88904   1555.10359    432.76688    -177.9613   -263.19705   -453.34171     -259.786   -413.86338   -307.36666    -338.7079   -300.02733   -238.75871   -413.04859   -447.53971   -328.81018    -491.7219   -379.47519   -430.00924   -359.30316   -525.60937   -270.73815   -387.18679   -421.37717    -423.8598   -419.18193   -276.37668   -267.75775    -194.9736   -183.23911    210.01541   1103.83742   1410.92733   1167.86092    618.13638    177.23856   -319.33854   -253.59956   -269.35467   -317.94561   -251.11889   -276.85919   -208.29287     31.37751    461.65165   1009.50599   1528.75223   1211.59929    858.78289    472.86818    462.70283    530.86253      59.2199    -106.2233   -352.74437   -321.45028   -414.59355   -380.19554   -386.84608   -276.22097   -357.55011   -199.14535    -172.6456   -191.47641   -120.35777     49.88587     98.39877    -46.48341    -245.5206   -282.65242   -327.24505   -276.44259   -403.57987   -296.55442   -119.23021    -18.48681     -74.6905   -106.90798    -14.61828   -168.30872   -355.31638   -268.65242   -391.96442   -348.84494   -229.62851   -205.38433   -223.53064   -295.42071   -259.79407    110.14889     10.62744    418.15663    242.95808    176.77074     50.71448      5.30138   -252.07629    -44.45512     247.6972    520.60086    645.82428    621.26395    372.55777    109.04683    -38.42674    -76.27157   -117.94258     -89.2028   -175.65402     25.96659    213.83566    799.44825   1543.82638    1895.0048   2002.96282    1509.0577   1118.89609    651.65766    122.50487     -0.45901    -55.02994   -122.32226   -221.12046   -211.15625   -294.17295   -252.24202   -205.37691   -322.63196    -182.5069   -236.92311   -403.49116   -391.81655   -207.11076   -145.61349    -154.4041   -177.80909    -211.4338   -144.87267   -163.92269    -13.23103   -258.84346    -107.3471     91.41738    -31.99318     16.81256   -146.68379   -220.11864   -422.28339   -207.93276   -135.51973   -219.11567   -227.03674    -34.86079   -159.66524   -385.33377   -277.67174   -229.57342    -521.6621   -353.04569    -385.9989   -379.31026   -466.47591   -374.69406   -373.42515   -426.83201   -143.81772   -265.20051   -127.78824   -118.59833    -73.74473     16.46557    284.72369      195.936    224.65704    177.92681    204.93603     29.24625     69.70659     37.59638   -198.45194      87.1171     -9.82999    173.75264      2.33955    224.93894    206.74229    -47.85923    -95.18717   -178.45019   -211.89832   -231.98665   -219.38509   -191.77047   -394.21389   -357.49891   -258.27888   -312.25379   -177.41392    -92.23891   -162.05382    -54.78237    -28.44918     -6.97072     43.92058     44.22518    318.46913    186.29835     46.23286    -111.3058    -405.3706      63.6411     -40.7756    -72.67568    151.58996     -75.4483   -163.98652    -42.52347    -58.61869   -274.85032   -134.25067   -429.33555   -234.63028   -217.13827   -291.30704    -216.2577   -243.91919   -312.92216   -115.01109    -69.23333    -60.17674     136.1354    118.29861    167.16824    154.05233    289.18808    139.24088    288.55485    569.41347    692.94526    742.96192    707.03328    596.64801    539.81961    407.25826    270.53837     83.22126     -4.64479    -22.82681     -77.9008     53.12867   -199.97114    -25.11116     44.50555    -32.72969    118.96414     -24.4793      60.7909    -43.00547     52.54069    -12.29386     37.91013   -183.56999    -107.6524   -155.46196   -225.39642    -168.4568   -256.96644    -178.6507   -262.17375   -369.67491   -328.33265   -284.78742   -443.93059   -204.54409   -453.67996   -404.42675   -237.24431   -149.38028   -274.81863   -254.20029    -53.93933   -166.91784   -141.67422    -36.24046     20.90721    -85.30301    -58.13785     71.15319   -101.05242     20.76338    102.22199      63.9484    -88.59162     -39.6061    -91.81077   -219.47703    -96.04466    -54.25204   -269.03495   -199.47279   -156.50512    -79.17886   -137.07107   -101.19693   -165.99195    -76.34136   -115.01648   -132.12957    -121.1307   -136.85389   -244.36954   -145.33759      3.70277    -80.40355    -66.52108   -193.92473   -127.46436   -239.11823    -11.17223       26.582   -123.60793   -105.97064    -31.23592    -53.77878    -18.65081    -74.79029    217.07039     45.13945     12.36052    137.69075     18.85877    130.65946    239.73843    303.82496    220.47216    172.77353    410.06894    172.59543     355.6901    145.59319     70.30412     63.17755     37.95601      77.9509     -24.2235     17.08801    -18.04975       -7.417    -62.07601   -164.79774    -63.06995    -79.38957     -2.40979     88.97571     47.10663     64.16265     50.35376   -109.44004   -103.84914     16.98043   -154.43975    -86.67342   -242.02643   -207.11344   -195.81602    -87.64956    -274.9655   -198.59996   -164.15934   -205.89864   -175.61823   -198.86679   -113.10959   -133.44685   -250.21183   -269.52445   -144.31665   -303.13076    -135.8929    -337.2551   -324.66658   -241.16319   -221.71292   -250.53973   -275.13722   -144.41168   -220.49248   -238.31106   -324.25367    -221.1614   -163.78026   -271.95673    -81.44455   -455.63954    -435.5722   -340.99912    -362.8075     -88.4945   -306.65907    -720.4883  -1073.27202   -590.31717   -438.20171   -407.41632   -527.96793   -604.14697   -545.31553   -884.26277   -638.75244   -654.23178   -732.38797   -683.08341   -367.64779   -496.37704    -945.9566   -970.59873  -1046.57359   -801.65617   -823.69225  -1028.40352   -706.34086   -800.23977   -761.15895   -898.09639   -464.79115   -732.24221    -912.8498   -988.76479  -1130.27162  -1152.18075   -900.27888   -810.95079   -762.56813   -817.35817   -936.04437  -1232.90879   -932.28294  -1037.52774   -748.55005   -924.65995  -1110.84192  -1006.16659   -714.87732   -882.17025   -771.84964   -1065.5842  -1015.21038   -771.97237  -1049.72392   -644.35624   -715.07331  -1284.83661  -1041.07402   -987.78724   -449.89423   -645.33458   -977.51146  -1290.27672   -828.34524   -803.81527   -873.74394   -731.48638   -788.55447   -941.16808          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       11.50    -185.4431    -246.8559   -276.15114    -481.3983    -317.6337   -481.49826   -262.23297   -540.53454   -333.27878   -426.80948   -433.50486    -85.58725   -629.30929   -327.63339   -215.84533   -577.09188   -372.76919   -424.72196   -327.39105    -298.7486   -310.08943    -429.0602    -45.73914    -172.9857   -141.53119    395.80554   1336.53624   1994.07085   1472.36991    179.29752       0.1294   -229.16693   -421.37244   -282.81933   -281.03151   -322.78306   -494.11309   -555.24838    -216.6153   -384.93136   -623.19765   -273.38952   -584.55965   -369.49915   -364.40131   -346.14734   -468.44328    -265.6328   -438.73753   -460.83926   -414.49866   -373.82618    -425.0335   -500.21774    -237.6347   -136.35241     200.2723   1124.37432   1456.90159   1276.10091    550.23518    -42.47274   -310.53412   -246.82259   -319.82902   -235.04346   -231.53675   -210.20462   -140.21625    -71.12641    344.18343    902.09799   1451.91088   1290.98031    869.22724    672.23892    679.50706    422.00954    110.29036   -418.06102   -391.10701   -467.05616   -423.76782   -286.22498   -362.06687     -374.603   -309.13105   -420.89235   -200.65138   -188.84639     75.20543    208.86886     -6.86844     53.06322    -10.62808   -343.98623   -261.63738   -134.53181   -266.20935   -283.95085    -37.62954     51.63798     57.44644   -148.85629   -181.78282   -107.14028   -332.69224   -407.08293   -374.21465    -423.0679   -325.42809   -371.83879   -254.82944   -327.92979   -285.68147    -136.1672    271.53668    180.56565    422.16158    135.61978      48.7153   -189.30299   -241.03501    -31.65169    123.74604    378.56335    634.68709    712.60845    318.53065     55.61299   -138.99751    -14.96249   -240.12361   -356.26958   -192.01352     -0.27741    106.42153    722.10439   1642.68033   1955.78911   2142.91599   1718.51449    800.85367    430.55978     50.36994   -278.36996   -201.62267    -288.9905   -255.30709    -97.94048    -214.8896   -266.38585   -188.90763    -389.7995   -205.36414   -341.58665   -217.78895   -250.55943   -130.50887    -89.71191    -47.22996   -175.79732   -188.11964    -97.29806    -126.0254    158.29873     16.10392     -3.84823    -55.12186   -134.91846   -154.15538   -171.17673   -203.30177   -206.72536   -345.99052   -119.54229   -307.74566     84.74557   -157.59376   -182.23908   -178.15176   -296.81156   -317.31183   -325.97982   -353.37836   -216.64607   -415.03056   -388.47731   -290.24783   -282.99686   -332.27921   -155.24221    -76.27501   -235.73445     43.90167      3.83427    262.27519    157.80581    156.86513    253.18405    269.97916      41.5599     56.56886    -22.71998    -16.91006    -49.46079    -77.15798    -69.04967     25.52391    182.88314    142.08757     -52.3214    -10.53343   -168.16466   -145.91363   -208.19872   -278.15612   -221.15558    -308.9253   -285.59463   -178.15375   -354.92218   -288.11819   -221.78007    -137.1822   -142.92945   -115.83458   -109.06806    158.18428     58.72069    129.21974     89.34942     -5.01581    -36.63126      8.68425    112.60161    211.99862     61.30451     37.04221    -60.03194     38.01261    -12.18296   -235.79787   -209.71711   -217.41899   -273.79789    -352.0429   -263.55022    -344.1811   -299.77373   -177.91677   -132.87837   -141.94321     -6.15583   -108.70157     47.28187    -39.79156    199.93045     56.45889    237.08826    263.21007    300.40867    494.61353    439.32376     684.5893    844.22191    624.64722    660.77331    737.35935    405.38738    301.24143    131.06931     26.50821    -67.67851    -46.97144   -159.25253     28.36984    -93.14344     84.08731     -9.46346     92.26939      45.9988    -30.14104     34.82051     295.1438     28.52358    131.49452    -88.64815    -99.43397   -129.26776   -105.48014   -272.02535   -314.00897   -328.10554   -103.59171   -396.25577   -339.64497   -313.33749   -346.04093   -311.81991   -274.71932   -140.36108   -332.04529   -276.95893    -275.8134    -252.9166   -172.86549   -393.94877   -119.19519   -127.38553    -63.73011   -179.62741    -55.23338   -124.67547    -12.51632    -31.97082    -59.09278     87.95044    -65.57325     -3.88433   -138.36012    -97.67493    -60.62111   -109.07894   -252.15353   -228.04076   -160.44964    -259.3801   -161.20637   -235.99496   -136.92639   -201.06254   -127.91238   -115.10375     -9.47977    -33.07154    -75.73466     82.56722    -68.78561    131.06496    -132.8336     -73.5702     96.11745     79.01328    -26.87998    -59.18434   -152.28055    -16.33254    -19.22362   -108.95672   -122.33173    -59.17055    130.84726     96.88956    -30.89261    -37.35506     44.62649     10.60615     95.26098    133.71256    147.06758    256.48549    231.87592    163.62575     295.5217        -10.7    310.41747     59.37662   -124.98234      95.3696     167.3928   -304.12644     22.42645   -189.54735    125.71928      156.217    -64.73712   -223.37847     47.10658    200.17167    -55.82231   -184.91846    -60.16693    -147.8086   -148.74897   -106.82043   -195.49561   -224.94942   -104.26231    -393.0257   -239.64723   -429.74551   -156.05499   -350.92879   -599.75871   -270.55112    -77.60196   -269.18343   -221.74589   -153.11257   -337.73729   -324.74913    -80.30003   -270.78714    -246.1748   -293.35583    -31.01131   -193.59147    -80.73032   -351.55435   -397.70204   -262.77355   -338.95728   -313.88433   -129.82257    -49.28909    -516.0318   -224.50128   -373.68631      0.07556   -214.86938   -497.69133   -254.34665   -422.11973   -739.83834   -279.72228   -516.10147   -600.96305   -474.07905   -585.60645   -333.09858   -603.58551   -389.94731   -665.19612   -540.04026   -669.70403   -805.92559   -727.34625   -622.53154    -687.0945   -750.41871   -887.23901   -709.12888   -829.38598    -860.2439   -742.48022   -677.38524   -849.79674   -478.42771   -814.59283  -1282.76476   -847.51291   -956.86227   -889.79715   -844.07599  -1087.66538   -949.63803  -1093.88689  -1023.54362   -871.62141  -1077.97043   -692.98011   -742.38163   -557.27868  -1024.45039   -695.96721   -767.21688   -602.68174   -811.62651   -657.64022  -1239.27756  -1093.71194  -1223.72246  -1197.45898   -929.75675  -1055.65656   -792.56444    -975.9309    -917.1724  -1352.04822   -902.09927  -1238.60575   -958.77881   -1381.6787   -417.63974   -659.57959   -465.30804    -840.9173  -1390.90726   -550.56983          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       12.00   -276.31719   -275.73117   -280.58912   -606.13815   -320.89976   -335.62247   -160.01238   -510.58944     -83.7217   -492.82869   -465.74204      3.52485   -383.94667   -318.01819   -278.16379   -352.37228   -335.73875   -324.67155    -206.8976    -275.7682   -352.47375   -421.97018   -129.24102    -28.46088   -190.66924    468.35262   1230.98166   1742.58838   1198.45418    166.92798   -209.40679   -143.06554   -502.52456   -312.61559   -211.97244   -298.66073   -403.62789   -490.03292   -280.00228   -328.36819   -323.16412    -249.8732   -264.35935   -488.01442   -319.53299   -345.02922   -453.79095   -354.72289   -269.04641   -360.72715   -341.06138   -341.30196   -268.78964   -299.06144   -325.85758   -271.41041    205.52674   1049.82537   1663.51765   1038.04633     549.5125   -218.21052   -103.68333   -251.45188   -222.23646   -235.79152   -316.97778   -276.31275   -188.43692   -103.34795    266.48806    942.87305   1412.56799   1302.03823    845.17024    653.65368    444.23625    411.36769   -221.92707   -619.10719   -394.19359   -373.81649   -244.02376    -330.6203   -177.20409   -373.47896   -362.48792   -253.29362   -197.03401   -251.38142    204.33862     36.26919    295.45733     34.46811   -142.15816   -273.51261   -224.64331   -299.21411     -145.681   -284.47187   -265.13331    -70.17895     73.75197    -61.83626   -124.79103   -238.16268   -356.86316    -410.2416   -390.68636   -270.73499   -332.61464   -455.20412   -175.58463   -282.26334   -183.18397    -48.26357    -16.76683     209.4064    310.79032     44.26881    141.88631   -113.18511    -90.46656     -2.10751    515.37004    453.32841    671.13192     591.0503    451.86582     59.73284     -17.7995   -120.02115   -247.14192   -349.58327   -275.78554   -113.35202    247.06369    894.33006   1491.79631   1980.62369   2269.14266   1692.17046   1128.78868    401.80559      0.03781   -132.42316   -127.86149   -175.19029   -241.78366   -274.32495   -186.81575   -227.72751   -455.15996   -283.28691   -389.78476   -335.13266    -314.0655   -143.52054    -68.30927   -168.05564     -76.7615   -152.53845   -203.62031    -53.79769    -39.68797    -51.59621    -61.09303    -49.94979     73.87816   -240.29672   -343.73174   -258.43675   -167.30051   -413.79736   -208.38651    -200.1778    -150.6842   -103.48776   -212.66041    122.41956   -338.24933   -408.53967   -113.44664    -251.8552   -298.75556   -157.70644   -373.56964   -350.97303   -324.59201   -282.88547   -260.85092   -131.61464    -96.45282   -238.44421   -141.73907    163.68005    -44.11416     43.98078     87.55223    341.99884    166.00282     87.18668     56.23464     -9.36434    142.45897    -35.30853    -69.52016    -21.51101    214.60721      54.1247    -34.51878    -34.58385    -52.37099   -158.51874   -238.76959   -236.18812    -181.3794   -270.57744   -404.24156   -310.94855   -347.78753   -240.45447    -99.76476   -280.01691    -197.7368   -282.32816     37.94783    -16.60263     45.18213    -53.15861    -10.55013    189.11218     154.7697     54.70683    -92.90229   -105.47548      7.37078      41.3275    111.14716    120.49658    101.50045   -159.93448   -164.89491     -2.51707   -153.90667   -337.56447   -344.65782   -140.67744   -298.42617   -271.39079   -282.68887   -116.80242     -80.3307   -128.86602    -54.09088   -161.89366     19.48873     -33.3989    156.38204     35.13394    228.85499    550.36132    507.94662    534.17801    575.39243    775.36345    745.73416     689.6974    596.01246    315.30355    390.03099    341.23888     76.47798     -117.801    -14.21603    -11.82671    -98.47602   -123.68724    -81.27725    -91.90606    -24.39479    158.56893    251.94907    127.53952      1.05697   -155.36471     35.46236   -134.02484   -287.52369   -440.37536   -304.18417   -388.35193   -518.54786   -207.66943    -288.1162    -360.3907   -548.79354   -376.44037   -374.68444   -376.15627   -341.69835   -422.30171   -257.90107   -267.54108   -210.93029     -137.869   -250.87112   -234.74922    -12.79551   -171.30763    -69.02959      6.18552   -121.80237    -26.93632     76.00647    -19.71393   -123.35467     40.74564     30.35495     -31.7914    -70.52272   -269.55728   -185.38231    -57.97886    -225.2542    -82.32628   -240.27231    -97.03644   -204.92194    -63.93932   -147.18223   -178.59408    102.43723    -33.09494      -45.214    -48.96778   -187.02377    139.09123     54.42807    -26.43351    -18.82009   -128.66978   -145.18906   -134.20598   -189.51096   -150.45051   -205.72911   -102.30386   -419.71679     -8.90771   -235.83953    -103.0452    -39.07788    -33.18247     186.8181    169.70753    -52.29762   -189.59653    221.69042    293.21215     67.84821     256.2185    209.89132     152.8817    152.80599     17.33253     39.11051      58.5353     94.40175    -127.2532     -63.3338   -108.67072   -130.40353    -133.0422     20.46321     71.01467    -32.50366    -47.71557      9.55754     109.9105      16.7896   -119.74414    -140.3482   -268.20488      9.01503   -112.27027     -75.6006     15.15636    -332.6792   -343.63748    -98.65111   -206.79481   -187.76596   -144.70305   -239.80976   -116.19833   -223.35273   -347.08981     53.62973   -140.18454   -123.20309   -328.37073   -233.46439   -144.60021    -297.0473     -340.903   -321.47801     -3.93006   -281.48996   -246.96896   -329.83906    -95.44262    -82.73453   -199.20335   -183.44197   -222.09131   -240.42486   -219.72731     -396.217   -340.90107   -583.36778     80.56542   -343.51177   -353.23168   -418.91373   -390.57797   -310.97326   -280.25359   -535.03195   -525.31328    -888.3019   -655.97585   -672.52537   -643.53687   -672.42827   -410.02412   -781.40017   -435.68646   -546.08132   -731.55683   -875.40112   -961.85742   -626.50211   -839.64727   -882.44327    -822.1118   -969.17198   -965.05048   -771.32999  -1229.99525   -902.13718   -878.51605  -1052.83762   -952.48636  -1036.93445   -916.98093   -940.89101  -1175.03892  -1051.17803  -1034.58768   -929.59928  -1058.32841   -987.48391  -1246.84392   -846.91601   -790.18235   -709.74729  -1100.14781   -822.22729   -976.52688   -828.03672  -1250.44048   -867.26509   -968.07155   -596.17385  -1275.25675   -997.92191  -1033.83921  -1406.36902   -632.76469   -783.62864   -964.22253   -979.55579   -944.15499    -37.41655   -709.45801   -251.69864  -1216.37254   -567.45868  -1119.64613          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       12.50    -349.9811   -245.40693   -322.86363   -622.08317   -306.25292   -137.83642   -138.49839   -445.82588   -127.77869   -496.69877   -499.91676   -262.83316   -215.14997   -264.30513   -441.99124   -393.50379   -269.41064   -276.22628   -359.45422   -341.34105   -394.01629   -261.86295   -278.03277   -177.47376   -208.58901    640.18836   1246.25451   1816.44686   1428.38338    230.74086   -286.80634   -151.18155   -367.14872   -256.74128   -400.45915   -340.98011   -327.42849   -329.27623   -289.56254   -494.74763   -417.08178   -280.65315   -392.30674   -393.01313    -429.2972   -243.61096   -437.07031   -591.95092   -285.61042   -305.56258   -292.39497   -274.81277   -447.21877   -210.91336   -376.13388   -205.60484    276.70772   1057.66481   1581.62542   1272.45367    654.84108   -196.00787    -287.0725   -163.13045   -408.24547    -311.0399   -290.36875   -339.59201   -228.41598   -168.82786    483.24563   1105.94222   1370.53429   1274.12829    682.87205     435.6139    630.33999     295.6414    -42.89648   -541.40969   -349.39233   -325.09448   -436.43969      0.09649   -278.52407   -424.65337   -292.67317   -375.59667   -291.34592    -97.80212    104.37022    280.59202    197.79103     28.80624      4.87858   -255.50384   -300.41193   -353.97333    -288.1253   -165.99976    -364.8018     51.13564     48.19866     99.43621   -159.90444   -183.30951   -256.94627   -430.48592     -380.003   -366.11454   -310.89818   -221.89843   -282.20544   -279.96044     -117.284   -195.76128    148.90919     363.0803    232.44877      186.797   -105.28181   -132.18379   -168.79798     67.27793    285.41461    418.12433    665.36012    553.06652    252.35536    205.85863   -114.08663   -123.07471   -320.20158   -134.76235   -234.59219   -141.11661    373.87145     850.8906   1578.90986   1961.25499   2057.56582   1576.67903   1103.47868    482.72177   -111.01109    -99.39806   -151.09806   -210.53701   -250.45961   -259.12858   -324.70804   -296.24532   -408.52378   -420.45361   -339.98796   -240.41695   -292.66743    -66.28265   -172.85734    -46.20773   -150.63293    -46.17928   -144.36945   -119.75405    -18.12958     27.92876   -124.11523     72.42984    124.25937   -100.84447   -113.72167    -75.60321   -267.72404   -149.26568   -260.29833   -225.80541   -212.92356   -302.73038   -246.15703   -128.66967   -119.86935   -249.20732   -276.25992   -203.15553   -370.01069     -367.776   -341.92292    -368.5116   -317.34067   -250.31969   -321.47911    -268.7161   -187.71219     24.40586    -48.12286    160.56099     41.84186     -1.84497    183.64771    176.17171    221.45502    104.49907    133.39601   -188.91146     40.22533     90.94927    -10.83197     60.91639      165.701    135.94294     128.4595     96.71987      52.7119   -180.49545    -95.93761   -150.33042   -133.10223   -280.41635   -375.37859    -397.1446   -319.26209    -246.9469    -79.53593   -242.38534   -134.96087    -80.69638     62.50961     -4.27604    -11.21981     57.95773    114.47817       86.011      6.53463     65.38635    -70.15643    -95.13756      5.11812     16.30241    -31.36722    109.28897     43.14468    -35.41602   -139.44899   -198.69771   -182.06285    -199.5796    -199.5956     -290.619   -295.07296   -316.46363   -202.03903   -284.49812   -302.34141   -201.84344    -82.19899   -238.70493     52.93847   -124.39677    186.39069    186.94441    169.12903    350.14668    714.16248     600.2113    606.00269     650.1193    645.90191    712.25479    477.06271    449.30304    287.97557    295.31006    135.72186      36.1264     20.90588     77.59695   -166.73317     -50.1201   -151.58664     23.83042    119.99308     91.35411    160.23119     54.99368    116.20976     -6.05879     37.37506      85.4989   -125.17545   -250.57595    -258.8018   -149.76398   -245.25656   -406.56834   -247.86562   -518.05811   -297.85521   -306.68227   -331.88735   -453.53267   -375.89417   -323.14201   -174.18348   -130.90619   -233.85578   -374.58892    -37.67965   -234.92536    -75.67083    -53.61753   -181.04766   -140.36285    -10.87979     -4.42631     38.89136    -27.38809     33.24416     -2.69299    -16.75862     57.51086   -103.92221   -122.04524     39.05811    -45.05882   -146.09342    -119.4508   -152.88295   -139.25675   -138.56235   -285.01597     57.36054     -91.7882    -73.58231   -168.14134    -64.34085   -271.14121    -27.99312     40.83866      0.77377    -27.64103     56.56662   -345.50079     25.22912   -227.08586   -292.35954   -248.44012    -86.66545   -159.17502   -142.15031   -180.97111   -259.28201   -100.64623    -66.67004    -53.96887     29.39777    185.71249      61.9509    348.19563    109.79743     415.3272    222.54048    173.17311     80.39515    209.33468      3.69235     43.36969     47.38306     37.35505    -89.15917    127.72561    209.40331     31.42771    -79.41404    201.72314    -43.04296     67.51052    -81.95025     -28.6797     -2.74514    -99.15679   -202.23761   -152.12216     15.55661    -54.20551   -231.41784   -119.56305   -168.33686    -36.33869   -120.15619    -51.25909   -214.51405   -382.01226     -326.549    -17.81969   -178.56037     -62.0446   -228.22921   -398.79598   -250.09244   -247.77562   -287.09435   -266.31195    -53.01787    -363.1149   -199.30445   -131.56897   -268.17221   -149.50255      7.76909      29.0947   -136.21788   -181.17727   -183.81771    -168.1549   -218.10344   -245.20352   -161.61022    -173.6715   -123.07112    -188.5796    167.48226   -436.61904   -331.96328   -368.96219   -667.02476   -451.21706   -446.44196   -186.81297    -74.34151   -225.43192   -501.90996   -403.81521   -488.07135   -571.04565   -469.92767   -880.34169   -707.18167   -516.72652   -696.99889    -643.2598   -515.10902   -827.83744   -839.99038  -1272.61867    -703.9118   -685.53616    -709.8092   -767.71388  -1123.15715  -1156.78923  -1033.84804  -1087.03741   -962.64399   -937.28688   -727.68535   -985.96155    -894.9275  -1074.69904  -1222.76777  -1121.44758   -811.86453   -843.49425  -1182.97953   -737.51938   -1070.5943   -903.49626   -574.36367  -1092.92145  -1330.44747  -1065.75497  -1148.07334   -859.06792  -1032.67258   -931.36412  -1110.63326   -902.35013  -1101.02213  -1379.94299   -1008.7062  -1150.62411   -919.57351   -241.27295  -1383.80564   -443.76186    -377.7796   -872.75133   -725.69242     174.2417     96.69993          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       13.00   -369.29778   -178.19519    -354.0726   -459.60263    -246.3716   -279.47003    -93.38927   -420.39882   -340.98212   -307.25691   -510.76784   -407.47161   -247.81054   -259.48751   -352.42786   -550.01069   -307.38386   -309.61579   -458.10133   -424.04328    -394.0347   -186.30249   -365.16268   -302.66954   -197.75476    564.40533   1337.06993    1883.6311    1442.5785    293.47951   -148.81724   -296.30368   -253.22653   -430.05896   -340.67084   -197.17517   -454.53056   -362.80352   -250.25008   -394.24328   -322.42324   -297.33964   -348.12486   -399.52045   -319.61092   -323.84661   -409.05712   -521.17936   -511.73322   -385.26224   -276.03486   -403.80397   -403.05519    -323.7902    -297.1066   -178.88916     230.3414   1042.07839   1484.22628   1373.91347     700.3247    -35.01853   -187.59769   -311.17599   -280.84398   -211.11738   -381.35147   -450.70996   -193.09003     71.06114    382.19231   1080.02497   1678.18443    1293.8027    952.71982    609.12183     397.8689    155.37569    -78.77633   -252.81111   -286.73734   -282.59418   -191.91533   -452.70215   -274.47279   -332.00262    -339.4874   -269.17163   -499.29125   -114.60339     29.26899     61.19188    150.95916     52.66626    -86.76527   -318.81273   -319.45994    -251.0552   -332.22674   -115.89482   -193.71845    107.36512    191.52168     80.83003     -66.7113   -305.13534   -237.80593   -353.69647   -342.29314   -226.56252   -390.08092   -474.91282   -257.94242   -418.91343   -188.94104     33.85964    178.02187    223.97677    335.26613    114.20612    109.41497    -25.49665    -204.2678   -132.45867     132.6419    350.35135    705.30206    411.63708    308.85083    -78.00617     97.03212   -165.45383   -179.37415   -273.09701   -219.90223    -23.86329    366.96331    707.40874   1453.40846   1803.33444   1956.33976   1620.48279     1025.044     351.1757    159.95917     15.75071   -120.69996   -245.83089   -212.28064   -280.46739    -215.1804   -278.56795    -207.6967   -332.06233    -232.9884   -403.17674   -166.20098   -176.94148   -159.64872   -145.13865   -215.93658     -99.6119   -189.61035    -89.69778    -74.31548    -48.66895    130.80353    -44.21202    -70.77051    -31.44167    -148.1184   -136.54018   -461.95801    -151.0024   -190.55292   -221.71354   -215.77449   -283.93651   -268.15585     -22.2053    -205.8377   -382.17132   -394.75236   -330.11458   -293.72888   -322.94078   -298.01204   -283.60083   -310.24102   -164.39293   -230.97165   -278.96379    -169.8871    -83.53443   -279.88057      23.6113     50.36923     82.92117     45.10809    268.31023    280.53848    135.36275     61.95883    100.05711     36.77409    -85.25977    -47.73998    114.44797    -23.07823      83.8402     36.32841     85.89932    -52.84886   -130.13234   -115.03263    -252.6687   -208.42414   -394.55455   -237.07143   -300.25172   -261.75736   -339.61314   -377.65116   -199.78601    -95.77972   -218.61619      55.0752   -142.77212     31.18202     136.5555    103.68173    -58.07734     -8.33408    -52.14432   -110.64789     15.45268    -78.13488     20.20633     38.30065     99.41542   -208.77205   -126.93843     -84.3966     25.00549   -192.44456   -181.47044   -258.14189   -304.38853   -347.89576   -343.10613   -301.68279   -214.40218     -63.6645   -101.92492     19.49829   -224.87932    -25.04326     37.22297     56.99683    276.09095    394.71168    250.65593    619.40371    440.10055    700.37164    593.78905    794.71736    556.58728    525.61313    393.95179    347.95569    229.04333    122.35556      6.77728     36.20391    -13.41902   -238.30142   -123.04167     97.75769    -17.48144     61.84123      4.05386     92.75191    115.38088     86.31604     54.61666     68.64101      -98.777    -94.29901   -153.91856   -163.43685   -321.84794    -180.5469   -298.95851    -289.4911   -234.53617   -435.38727   -373.39512   -380.67259   -384.73345   -322.26563   -249.31249   -255.09441   -273.38501   -133.69264   -284.30848   -238.65957    -96.90628   -100.10171   -264.77113   -109.72822    -55.08242    -26.92399   -205.18975    -90.62076     33.15068     88.25958     23.89807     -8.01012      8.98949   -166.67466   -108.63656    -70.50408   -198.16207   -135.91652    -92.20472   -107.29209   -180.42038   -241.36117    -95.25276   -186.34088   -141.69221     55.05598    -96.40694     -45.0467    -94.29693     -5.00541    -226.2301    -97.46451    -16.28315   -176.85597   -116.53223   -175.12117    -34.41119    -45.05221   -147.13896   -181.71291    -200.9133   -108.27516   -193.67457    -12.28071   -145.99303    -228.6619     16.18362    173.59182    103.42582     13.69594    132.59648    199.33899    276.50804    123.36818   -108.72769    236.11295    219.84845    116.78105      241.455    147.64052   -186.31096     72.96326    124.43247    -49.93756    140.32522    -71.93365    -97.49742     48.78199   -111.64485    -62.71258   -137.92701    -54.99553    -58.77963   -196.29278   -210.39891   -170.46232   -173.48108   -127.43527     -4.92079   -217.64892   -206.67715     16.75503   -440.32008    -216.5157    -76.70305   -311.92597   -113.80332    -171.3312   -245.46861   -329.58454   -181.18258    -97.68091    -139.5519   -195.87216   -260.94119   -258.99318   -362.55555   -122.93927   -268.19414   -244.99728   -162.17827   -326.82154   -572.01825    -176.1532   -192.98815   -190.98059   -419.30382   -357.83583   -210.98254     26.51388   -288.11961    -15.74404   -317.13922   -347.23358   -144.76495   -483.41807   -185.24702   -555.91256   -603.34618   -538.74074    -209.2829   -283.88789   -711.73438   -745.60798   -477.88762   -302.92912   -578.71297   -511.03951   -586.91371   -603.25942   -321.87578   -559.59085   -550.95179    -388.7751   -926.50154   -802.63885   -402.64717   -860.63017   -1134.2924   -886.80307  -1012.60474   -823.50413   -878.48447   -828.72397   -826.68614   -700.71516  -1027.60549   -760.95861   -973.33213   -928.93536   -830.28316  -1022.36341   -838.88789    -609.5684  -1053.02522   -819.96823   -695.47916   -782.17644  -1014.13967   -792.98529   -1182.7303  -1150.08285    -881.2863  -1120.52455   -978.50717    -867.3187   -966.89381   -733.65216   -883.80613  -1248.41167   -834.39922  -1209.90238  -1458.08264   -264.15205  -1222.50814  -1091.46958   -659.40568   -956.89583  -1402.77117  -1073.03536   -583.51573   -531.93331          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       13.50   -345.03667    -90.65678   -252.34388   -315.72363   -221.43758   -467.03327   -239.80777   -459.03214   -437.27719    -70.15364   -464.68839   -364.35015   -214.98041   -290.75125   -284.25434   -523.32114   -420.59758   -241.40318   -416.44406   -484.06178    -240.2992   -178.79834   -275.26538   -360.62795    -95.81826    385.66389   1442.83081   1819.31753    1393.0562    447.32143   -189.92175   -201.98943   -291.61935   -363.59905   -280.09019   -336.68036   -387.29864   -398.66382   -346.09176    -367.6651   -361.80252   -335.30697   -252.93399   -386.68556    -398.0873   -362.08257   -320.03005   -333.42846   -208.41254   -169.39376   -429.12257    -288.1835   -409.92603   -328.42908   -334.00055   -113.29606    277.80476   1194.82381   1346.27898   1320.13804    657.29937    172.00508   -307.50887   -414.80336   -213.66718   -330.63351   -217.48963   -373.00006    -163.2879   -205.89325    451.07175   1083.68681   1393.79868   1386.78071    724.06208    638.71946     385.4536    471.43466    -60.79437   -189.43154   -320.66998    -76.05774   -416.58069   -267.82295   -338.57665   -253.95372   -341.84871   -245.64805   -154.29052    -93.39792     -8.75369    229.04335    167.21104     45.82896    -60.61616   -355.67383   -223.71182   -275.63813   -157.28357   -188.04572   -155.37687    109.34661    122.34479    -40.80245   -128.14364   -149.19513   -340.24227   -264.38093   -275.48815   -265.12836   -289.67819   -346.13719   -244.01902   -299.37006     -97.8624   -156.70734    229.24863    314.51323    108.71235    235.26311     -18.4267    -34.56639   -125.29576     -5.94497    260.15218     570.2076    653.49036    591.16634    262.93165    239.91347     37.89361   -168.02931    -212.1793   -199.70777   -203.33496    -39.59099    482.69256    794.23689   1496.04333   1903.71051   1873.52837   1493.85956     946.2961    640.09336    154.51103   -198.86957   -179.65845   -125.94733   -226.52143    -72.10176   -398.68563   -274.89536   -130.30851   -385.46454   -250.99733   -293.86843   -363.36954   -114.31807    -61.13658    -68.49764   -205.05916   -175.13809   -224.61228    -78.61122    -34.10396     43.80887     96.47404    -11.74706   -133.51283      8.89208    -82.20699   -331.30573   -340.70602   -345.42208    -245.3582    -88.13504   -179.06479    -84.64847   -157.61464   -157.64394   -300.81623   -213.64867   -158.66298   -361.44856   -356.74101   -424.66437   -362.88663    -372.8462   -292.89408   -257.65865   -141.27513   -128.49023    -82.85483     -2.44111    -98.06503    -80.17803    251.23514    192.71759    342.55441    161.52898    127.47008    227.85456       4.5362    -99.92167    -102.9136   -146.41239    -87.29343      4.42709     -6.05129    -24.56772    238.72246       80.469     -96.7378    -55.32436   -162.70999   -125.16383    -295.9459   -279.02814   -542.31647   -208.22863   -348.15415     -312.154   -410.31675   -107.48015   -172.48972   -135.82906     65.89436    -70.00875      22.5665    234.46979     57.28571    126.16449       2.7891    -17.06099      29.1634     -53.9284     83.25216     -0.15694     55.81007    117.94437     124.6827    109.22961     -56.8693   -122.72509   -323.15414   -254.63323   -406.12916   -372.59336   -254.53575   -208.31505    -188.7209     -173.142   -263.26105    -99.19521   -154.26652    -102.1749     69.31226    -31.07747    274.41351     61.30778    240.00192    371.83935    168.18937    493.51997    567.42838    646.34945    827.45407    502.12224    355.86765    467.66217    196.19147    160.59354     85.27787     26.55976     -1.74839   -170.20487   -113.56832   -203.14119     -0.57358    -16.28883    147.55578    -52.13324   -106.70751     70.49169    121.61711     -4.95001    -32.13535    -16.55897    -135.9753   -132.03322   -179.51137   -255.75792   -174.11905   -168.26223   -328.19744   -200.04605   -277.02906   -438.03051   -354.71938   -393.92585   -490.87872     -1.38039   -322.78342    -24.06748   -208.95767   -311.78009   -242.91501   -114.45452   -138.55211    -75.05744   -136.69579     42.86785   -132.58077     -9.83878      7.15029    109.72544     16.24272    -11.35475    -43.48817    -38.14258      8.17329    -68.62073   -238.32918    -58.61545   -158.71337    -70.91941    -193.2687    -196.6896   -286.25748   -199.43667   -172.45678     56.57905   -165.81802      7.58505     54.67723     40.83373    -59.56559   -129.08305    -14.76989    -47.27183    -80.84289      3.98397    -11.43593    -67.56553    -166.8412   -305.47153   -265.01535   -179.02874   -148.73738    -60.24755   -160.60845    -21.80211    142.09963    -35.36124      5.07265    103.68023     37.61881    175.96897     182.9025      4.31234    118.61776      4.67534    164.18486     25.45104    123.57117     85.14175     13.67014    -22.64839     99.07465    163.20757     128.4014    -83.06652    -54.27747    -38.70019   -174.32657    -92.14635     44.18852    -75.13041     12.63474   -146.55454   -352.16695    -45.41839   -177.19608   -165.23914   -175.49501     -8.54848   -101.68138   -193.48537      9.83309   -144.61458   -307.56931   -363.94268   -253.54026    -70.72211   -211.64563   -258.49514   -200.98946   -388.09943   -406.93439   -237.35148   -300.56339   -287.91967   -321.32389    -359.7249    -375.8056   -229.21742    -87.47747     16.67726     63.26594   -518.45528    -49.82287   -395.48842   -298.93512   -338.50269   -292.00766   -318.72498   -204.79226   -245.76771   -211.94749   -336.25105     -355.049    -420.6218   -240.92758   -203.44254   -351.74137   -632.38658    -59.45618   -599.30841   -234.28377   -398.35087   -326.41377   -623.86749    -455.1422   -483.76506   -384.52492   -473.04845   -594.46091   -689.00883   -984.95163   -630.19505   -771.08085   -692.14085   -991.01434  -1018.67653    -856.1951   -985.54377      -753.15  -1161.04896   -900.25988   -915.40223  -1125.32163  -1034.92645   -886.67639  -1047.65048   -961.32106   -985.98838  -1176.24026  -1102.05033  -1070.78938  -1153.01206  -1343.50217   -813.81686   -960.74072  -1199.50058   -1023.3866   -839.01484   -813.25869  -1023.09114  -1087.78805  -1263.41985   -797.99905  -1204.55491   -797.22305  -1073.75299    -765.5635   -869.80242   -845.48678  -1008.90924   -472.27182  -1400.55934   -908.25681  -1079.50834    -650.0402   -895.16949   -925.04098   -559.02764  -1176.53238   -565.90878   -532.32905          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       14.00   -308.43151   -201.81753   -126.80367   -357.77543   -236.83845   -459.59987   -413.39599   -430.23976    -476.1872   -104.38248   -320.34104   -308.55666   -233.00871   -257.88465   -400.40755   -470.06432   -289.66366     -255.708   -307.08319   -542.75725   -460.09038   -197.07279   -266.14311   -253.51331    -104.3136    493.81443   1385.71452   1822.50392   1189.40089    407.50044   -206.64858   -417.58781   -390.62681   -329.10605   -451.51064   -414.04237    -336.2844   -342.27378   -573.25261   -398.66855   -242.77786   -411.62929   -413.60212   -326.08082   -390.46531   -312.97467   -489.48385   -468.05155   -391.91248   -232.21133   -397.46856   -277.46084    -357.1192    -380.4263   -274.14318   -110.64013     191.5743   1007.27007   1555.94501   1243.93871    402.25769    -62.28376   -272.44293   -442.09108   -289.98499   -559.32958    -339.4346   -240.45695   -346.10109     79.56194    577.32864    991.20137   1367.68055   1219.88424    630.91605    560.03616       3.4319    172.19595   -349.54707   -221.79513   -345.59921   -228.31451   -140.13512   -396.51943   -392.67044   -297.27063   -226.74688    -210.4267   -339.89274   -102.45435    -18.22204    262.49137    241.56985    127.52685   -192.07938   -128.62133   -324.00713   -332.47455   -663.25211   -241.53119      1.91716     42.78502      3.56511    -16.35327     46.86246   -260.87193   -355.32286   -275.69823   -231.44643   -381.27374    -297.7375   -265.14993   -374.48199    -191.8342    -182.8697    -37.33133    109.30709     316.3108    321.64732    222.41603    -57.37399    -76.52849   -398.95293    -21.84884    367.08576    494.39777    690.43086    729.82665    384.03794     78.89766   -161.33753   -101.19093   -365.09781   -345.09232   -198.64039      18.9184    217.92489    643.20037   1471.56882   1788.37291   1988.35947   1612.42134   1086.04911    477.21802    -142.5365    -43.23131   -159.32497   -199.33436   -200.95134   -319.87445   -383.42031   -406.82101   -370.29118    -396.4712   -253.48755   -333.59241   -202.92016   -210.71021   -117.99205     21.60658    -91.51838   -195.09556    -105.6443       33.612   -447.71892    177.31552    -93.03072    -16.08701    -21.92105   -203.12331    -31.14044   -277.63982   -314.14576   -424.21582   -478.99907   -207.26259   -186.83046    -83.26371     -134.018   -195.78564     -95.6388   -287.60675     -98.7918   -250.34219   -493.15816   -404.28666   -366.95999   -400.47044   -274.60748   -152.36518   -273.92474   -279.70282    -84.52279   -149.40515   -129.85993     25.10922    113.04377     48.13204    109.79814     84.82156    181.55238    146.61449     11.63538    -27.56838    122.39751     -7.74434   -179.24503    -20.19589    -18.97797    126.82303     88.71443    -27.22807    -11.16619    -27.13307   -282.89228   -242.79481   -151.14377   -236.21026   -352.93561   -114.79394   -320.85778   -251.56208   -349.32588    -53.86456    -96.00059   -231.56017    -38.81613     52.35058   -106.73415    -12.74926    111.46502    -84.75493     96.14142      73.8474      63.7501    -48.89118     62.39123     -115.477       49.194    -60.40224      2.51879    339.37234   -170.76484     -227.375   -174.03055   -283.20082   -240.78716   -443.66698   -348.30954   -440.06114   -285.23799    -37.34567   -331.62595   -122.13008   -180.32955   -110.99275    -93.06829    115.36008    -35.85226     28.99167    239.52208     306.7818    513.10681    573.45684    620.39564    456.32534     637.4501    540.33593    451.20344    304.07499    308.04609    209.68652    221.87359    -48.40709   -125.66584      6.18909    -99.00989     27.38266    -22.80106    -35.05821     97.14836     26.46578     65.32794    149.87398    -40.79106    -92.43531    -11.06871    -125.0582   -214.83142   -320.77895   -214.13456   -256.08735    -218.3252   -179.16377   -375.40409   -286.60548   -300.63419   -270.83213   -333.31321   -276.32941    -336.5729   -185.59686   -266.16582   -230.98317   -204.89841   -138.52995    -244.0448   -231.33173   -204.57363   -139.69761   -115.51655    -80.57543    130.52853    -142.2387    -11.85382     183.7414    -97.41911      -9.1059    -87.06871    -62.78355    -89.99693   -216.40143   -101.30408   -178.63744   -225.67405    -36.09828   -219.53026   -132.09711    -56.54844   -100.70354   -174.54027    -36.98294    -45.19721    -99.43015    -59.79915     24.77813     39.91143     33.84691     67.49492     -9.51876   -137.68658     -71.3094    103.72116    -61.43901   -239.56447   -177.68901   -174.13302   -167.68742   -172.80371    -36.07559    -74.32414     35.59407    -82.82337      1.80031     -16.4052   -162.17784     35.89773     61.07432    179.73334    175.35663    211.39557    116.60039    419.79331     94.44311     72.59117    197.62402    -66.73857      5.58244     14.54139     -36.4438    -47.78685    -26.04919   -114.77399   -183.13293     70.62524      86.4525     -53.6952    129.87795    -69.73207      8.69638     38.11139    -97.14219   -158.69467   -266.69477   -171.12698   -164.80026   -301.35685     -68.0599    -90.77735    -23.69182      40.5062   -162.16985   -133.41093   -168.05847   -174.88851    -286.9991   -367.14779    -184.3521   -324.65159   -160.25797    -229.4742   -301.95266    -44.66343   -134.40782   -268.19959   -310.26235   -259.88346   -103.40415   -273.38951      34.1879   -218.08456   -405.31026   -296.34301   -157.68763    -108.2249   -134.15627    -35.28693   -319.70975    -383.1808   -291.92816   -477.05732   -352.01296   -193.97795   -217.79053   -221.32997   -360.25465   -392.84011   -170.72569   -712.63379   -525.65114   -366.39512   -763.23921   -781.86328   -537.42602   -575.34439   -624.25477   -612.82759    -921.7081   -816.06429   -573.98553  -1011.05788   -846.07635   -845.84066  -1227.21565  -1113.01508   -859.43126   -831.07336   -904.24736   -870.79027  -1177.09259   -753.59933    -905.6387   -1029.1112   -996.61922   -769.13708  -1083.54553  -1034.50554   -995.76403   -949.28927  -1269.73461  -1219.91964  -1082.34644   -717.97508   -903.68548  -1275.65263  -1081.12452   -971.89736  -1290.44165  -1396.17912  -1114.56151   -929.12884     -872.574   -826.70881   -970.57875  -1019.79283   -596.56331  -1313.39433  -1560.93759   -974.73346   -829.19517   -924.25261   -617.88954   -844.70539   -540.20636   -261.80169   -609.09064   -488.96618   -236.22995   -312.62404          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       14.50   -197.45038   -346.68565   -192.09325   -434.94737   -286.59995   -343.14598   -482.47077   -336.39454   -466.73368   -295.99444   -226.07289   -287.32336    -237.7489   -274.27336   -451.37656   -469.07821   -242.26467   -563.11543    -203.6241   -134.92011   -520.51852   -180.91599   -231.67166   -222.85567   -114.43356    434.35817   1328.73221   1796.59089   1072.15087    434.02256   -216.44655   -283.20804   -523.12096   -423.09081   -255.73041   -466.77083   -126.92568   -256.72009   -340.48487   -368.56813   -204.74082   -356.43608   -327.08188   -316.36885    -260.8617   -227.77882   -414.41459   -543.34167   -260.26391   -288.91713     -374.416   -292.55379   -162.99854   -304.77179   -284.47673   -144.09605    368.36998    975.54631   1569.79368   1263.43073    419.33219   -137.90054   -263.65213   -177.95052    -469.3015   -333.54073   -449.89504   -344.35878   -557.18213   -113.22696    469.57074    976.23729   1592.07243   1204.24233    749.96729    553.78162    552.65849     52.15511     -3.82177   -358.76562   -423.25803   -449.27309   -287.43979   -395.91468   -542.51921   -290.54685    -245.5233   -322.60527   -364.15397   -590.66146    185.55699    133.75517     60.19553    119.47374   -136.45004   -204.90327   -336.86974   -294.25717    -332.6381   -230.66118     81.54683    -32.92362     42.82252    -59.57407   -274.23842   -117.85908   -249.44325   -348.76529   -453.35852   -541.40912   -290.88155   -104.78015   -407.40305    -255.6147   -232.33333     53.06925    127.03975    252.09015    371.23759    147.24209    -98.01454    -67.17772   -245.71421     -4.51447     137.2008    476.46934    449.54218      523.634    305.70385     98.77373    -72.50708    -79.38099   -233.80558   -227.37977   -177.75517    -66.74242    418.71844    971.05284   1423.79423   1846.71275    1444.0765   1544.71823    943.49401    509.60937    131.31924   -125.23177   -113.81254   -101.73879   -216.21752   -271.39751   -313.98949   -125.03675   -267.71213   -315.86882   -255.97059   -378.07929   -221.10968    -193.6274   -159.37273    -135.0151     50.23766    -91.08487    -59.93019   -120.56647    -54.12381     41.81322    121.48624     82.66924     94.30294     27.92738     28.32413   -120.69468   -354.05163   -267.12872   -310.26748   -148.70515   -220.05553   -161.87512   -257.28854   -279.63408   -322.26294   -215.78049   -248.81249   -226.16569   -267.44028   -329.27606   -150.03791   -166.66481   -349.32134   -355.64583   -253.04276    -201.2121   -348.16666    -68.19599   -192.18727     22.17277     116.4684    -47.77224    202.41799    176.36726    144.07916    -31.13167     -2.96635    176.86554     45.82637     90.58977    -58.66247     11.63326     20.55678   -151.24929     76.23609      73.5001     66.65287   -110.60064   -248.12976   -299.77509   -308.62601   -237.27378   -352.51312   -435.02291    -216.4703   -239.16309   -363.69073   -225.44302   -145.60453    -24.56006   -141.78155   -115.22737    -34.67441     156.7559    -25.63725      31.2805     21.04947    193.11693   -113.85743     62.34162     65.02718    107.39355     -2.82405     52.61948      7.36255    -49.44858   -122.69235   -109.89129     12.85651   -178.03436   -266.34815   -252.57755   -163.94395   -297.61969   -322.64206   -271.75778    -159.5264   -102.09934    -66.83738   -113.98166     21.98546    -79.47261    116.39616    198.28986    340.59964    106.60144     365.7433    597.22876     666.6782    727.65038    666.15386    438.52131      494.389    356.84095    320.02027      17.0116     131.4664       6.6523    -77.44886    122.89361      -5.3457   -127.77398    -21.45727      4.23598    -74.89316     61.78369     49.22063    168.59962    -49.29225    101.21235     68.61835    -59.19186     11.94809   -210.86324   -290.57182    -298.3368   -335.24685   -279.39431   -297.54408   -470.21592   -385.40529   -358.98782   -371.03302   -254.23296   -275.66699   -228.60369   -529.91016   -140.26778   -235.63723   -141.58473   -237.44718   -154.58687   -185.99063   -103.22678    -33.91177   -187.56062    -79.09469   -159.73749    159.14805     38.05796     13.71982     78.48353    -11.34947   -215.56017      1.08533    -74.11054   -116.74346     38.73436    -79.23663   -218.66259   -227.96408   -111.39456    -80.60762    -172.5115   -182.03877   -211.48524   -127.97063      -71.591    -87.05051   -157.91579    -49.51209    -88.69822   -204.11406     55.63262    -69.67043     95.94293    -89.50046   -183.97172   -180.77929   -242.91115   -310.06201   -150.86503    -22.34954    -96.39022   -152.53142     -8.65069    -143.3009    135.64585    -54.79662     76.66257    295.64038     76.41514    146.64211    146.37769    155.83485    209.55113    179.87916    271.59104    194.70987    -25.96865     30.36873      6.27594   -115.71471    -22.13031   -136.03985    -40.82838   -107.62129    -61.47629    -81.48911   -206.78951   -238.80483    118.55008   -152.76915   -296.56137     71.71307    145.89539    182.81461     -6.25834    -23.55873   -168.97902   -154.86717   -109.04639   -209.22274    -47.99581    -93.84169   -266.60067   -343.31965   -178.37243   -217.52778   -304.25558   -492.31281   -394.48759   -339.12331    -312.5811   -132.63906   -343.27622    -91.49668   -500.13261   -246.93247    -380.8672   -317.45875   -373.12097    -289.6354   -480.11723   -449.94415   -207.79024   -203.99514   -108.23612   -202.49654   -277.00873   -476.69158     -94.2222   -437.25951    -354.1693   -390.17768   -212.67478   -427.21132   -253.41688   -314.07552     41.97213    -14.55839   -272.32333   -627.06211    -657.8408   -530.61726   -672.18693   -625.54488   -887.98738   -843.31755   -729.22204   -792.67329   -573.38784   -509.46877  -1046.11205   -734.82367   -894.75272   -913.59768  -1036.08953   -757.25526   -890.45508  -1182.87852    -665.7622  -1125.00118   -749.93498   -978.85453  -1019.99746   -922.70543   -908.76617  -1096.09677  -1026.61016   -814.43262    -939.6955   -858.24387  -1340.78436  -1018.62238   -779.32485  -1345.18987  -1264.32778  -1073.00164   -875.25191  -1155.32779   -733.42586    -1064.984  -1126.97804   -802.96195   -1224.5119   -766.28416  -1000.38287   -841.84326   -891.06228   -603.49722   -722.20446   -897.99572   -483.10303   -997.23829    -734.7263   -497.90092   -551.96581   -377.10276   -542.63382   -591.64335   -315.72452    697.70826          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       15.00   -102.86722   -422.25781   -352.73865   -458.46336   -324.45996   -313.96914   -514.08551   -270.48324   -423.82921   -353.46338   -230.63587   -259.32475   -109.63172    -186.7521   -374.97016   -370.24096   -358.56398   -415.48677   -147.25681   -160.91627   -295.83049   -357.26979   -247.95096   -226.98507   -202.67005    398.90927   1402.19122   1774.76423   1348.87875    194.22287   -243.20939    -154.5325   -459.03441    -418.8331   -153.29721   -378.05862   -272.86853   -172.36447    -464.6239   -266.78823   -505.06255   -360.34482    -532.6022   -335.08067   -330.91753   -361.86179   -250.25106   -232.44719   -399.22091   -382.19729   -443.78189   -292.45288   -329.95088   -294.41095   -369.82983   -153.87173    305.07752    988.25605   1436.06216   1256.16936    492.53721     11.30761   -169.51014   -313.01331   -330.10015   -288.35944   -257.00022   -213.58443    -347.1092    -103.6013    445.17216    994.46948    1523.3866   1228.96421    720.01124    602.09423    356.53617    293.23899    -46.52048   -472.92431   -146.05089   -211.79971   -419.64851   -277.42868   -305.19475   -371.19434   -216.93732   -239.97228   -377.40107    -93.86134    -96.87662     58.66647     45.02968     67.59997   -142.46516   -198.57819   -315.37485   -277.31769   -451.52076   -139.82313   -116.51866     34.72019     59.40204    -26.42188   -171.64204   -209.31973   -314.65057   -323.87802   -302.52324   -345.00139   -247.81332   -216.22024   -177.48002   -426.67884   -153.32536   -197.23759     88.58984    352.87118    236.97386      65.6388      5.31341    -53.35959    -92.09507     96.75479    376.98994    516.58383    626.04079    375.99457    388.91726     130.0648    -49.61059   -282.39655   -130.50438   -222.06722    -66.88795    115.01623    210.56403    876.46004   1525.36733   1947.83546   2128.69565    1636.1651   1057.00501     528.5984      6.12447    -91.01547   -143.91795   -191.70618   -291.99965   -142.72816   -133.21468    -92.79448   -304.27489   -322.64603   -383.02469    -317.4648   -307.09886    -451.3193    -76.94736   -192.67669   -167.23775    -84.86214     54.87246    -298.8019    -24.79261     103.5289    117.60757     146.2808      4.11987     28.77751   -102.32512   -211.88177   -222.50845   -194.29142   -228.89365   -246.41543    -200.0852   -124.47339   -184.47373    -81.12511   -176.66929   -217.87587   -251.00483   -205.99868   -283.36777    -394.6383   -456.56747   -142.44176   -355.65986    -420.4686   -197.77258   -252.05025   -171.65909   -214.51926    -24.00227     -22.2032    -22.63705    214.15611    218.02581     160.8648    153.36574    268.39531     33.78533    -21.23076    -53.82829    -18.38184    -77.66893     35.96387    -67.46622    144.13046     157.9705    -39.12892    -42.79665   -146.80311   -258.42494   -231.42462    -187.6506   -410.43177   -516.44873   -287.96916   -407.08719   -300.24337   -244.56885   -243.50524      3.40691   -132.04664     45.79315    -34.36923    -10.55475     61.81268     89.66142    235.24504    119.75508      9.91081     30.91849    103.90319     14.92919    -50.91581    159.03259     96.45737     22.97577    -115.5891    -10.03301    -71.25191    -61.23884   -190.81675   -186.93655    -72.55578   -225.33003    -316.2192    -79.70876   -277.96652    -73.96974   -102.68339    -12.49158    -92.58189     98.88938    126.06565    -34.68122     22.49565    181.55648    484.81219    572.81997    608.50547    819.78239    630.90612    669.33909    592.96856    425.75585    315.64952     217.3258     71.81055     89.73225     94.59929   -118.62643    -44.53607     -58.3886   -140.16257   -154.19607     28.37047   -111.46477    141.38096     78.67892    127.06662    -61.21925     99.46972    138.94035    -17.05453    -21.94109    -82.58805   -103.33446   -191.27222   -191.50637   -379.50486    -263.0394   -278.46532   -118.89827   -147.42132   -236.53394   -327.21721    -257.1603   -377.09699   -172.92968   -250.02961   -215.90306    -58.22825   -204.71928   -197.61882   -111.38234    -93.30551   -143.19463     37.43359     14.97045    -69.55338    -87.78074     94.11615   -198.94715   -161.20883    -175.7067   -223.54526   -228.70855   -259.66493   -223.84339   -220.57709   -179.06698   -286.13296   -315.27404   -127.22445    -29.90723   -292.03759   -243.62353   -246.99326    -35.26015   -127.82416   -148.38167    -66.49855   -205.88273    -22.38215   -126.59256    -14.27147    -36.25088    -46.40775   -255.00328   -122.75326   -170.79441    -92.20296    -226.9689   -259.37541   -206.06429   -158.92798    -80.70518    -52.79767     28.07315   -108.97872    -193.0878     39.66604     36.59888    176.28756     67.55933    142.10873     38.58553     99.07698    214.73247    203.48052     39.50623     46.38736   -142.40946     62.06985     59.37697    -75.47214    -46.32911     59.04665    -73.77012     -8.50151   -144.14235    -76.81098   -104.00503    128.74629   -183.29948     45.38689      6.87791   -180.05409   -214.40386    141.87042     12.80014     -80.1668   -194.47065   -283.91316   -258.85532   -216.98053   -275.72557   -353.29873    -268.3962   -126.50466   -343.22296    -339.9239   -427.41653   -671.21268   -285.11654   -288.42437   -316.04086   -187.44167   -185.51873    -228.3108   -209.52174   -257.43651   -367.69124    -99.48798    -72.39655   -298.62353    -55.67947   -232.56885    -182.3713    -69.38705   -496.49031   -235.83611    -109.9097   -425.79679   -351.46159   -112.35267   -440.08646   -397.25052   -606.98883    -527.9274   -649.66297   -317.88861    -299.5946   -583.84791      -263.32   -748.73283   -553.36843   -616.00467   -780.83002   -492.95028   -770.17081   -783.89106   -770.32943   -851.00909   -639.74066    -801.6215   -925.42857   -654.75422   -841.80478   -710.02732   -705.06957   -882.99418   -892.71851  -1146.80905   -818.10421   -783.86915  -1104.55382  -1365.59276  -1021.14666   -989.37288   -620.99607   -918.82497   -960.78719   -1234.1536  -1046.47747  -1091.17088  -1072.98237   -869.46492   -923.45954  -1018.76765   -746.18356  -1078.79775   -935.93194   -958.76871   -715.95359  -1072.33734  -1224.61854   -885.34223  -1063.19983   -981.11938   -740.92458  -1094.37471   -777.39482   -1051.6655   -823.77955    -710.2658  -1212.99791   -560.77907  -1054.42722  -1407.76389  -1497.49975    -74.62185  -1772.26151   -357.99354   -773.26078          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       15.50    -97.59912   -290.83745   -476.61963   -367.94635   -342.14565   -561.41371   -557.20951   -215.89293   -419.46787   -312.11449   -198.63297    -156.2887    -52.16979    -92.69234   -430.83977   -188.13185   -304.58581   -122.27103   -160.68522   -329.49565   -267.28163   -596.06789   -359.46057   -172.85055   -313.05606    485.07363   1326.60322   1721.96731   1037.87201    212.52862      1.18618    -76.56608   -329.81763   -291.15679    -481.3718   -341.35555   -226.52121   -319.70244   -393.38896   -267.81146   -484.21081   -375.44407   -617.47788   -181.61011   -368.93186   -180.16329   -364.43112   -440.58819    -308.8743   -288.04508   -370.06639   -405.12849   -318.69449   -414.40631   -386.48693   -135.41862    152.40809   1154.67879   1467.90553   1315.05663    461.08188     48.47707   -290.63571   -211.33817   -213.90217   -203.53938   -222.20738   -205.09965    -389.6529    -75.02539    456.33032   1126.93554   1315.14144   1136.22288    650.89812    694.93539    313.95142    277.78408   -299.48986   -449.07382   -570.39094   -301.50362   -366.27576   -274.26533   -458.34647   -256.88328    -205.8121    -194.7693   -134.70525   -265.03778    -38.39755    236.42914    272.70316    143.48279   -123.60855   -268.45226   -336.03402   -409.03105   -285.75366    -207.7797   -180.14719    105.77963     23.04338    -96.21806     -56.0194   -213.38518    -94.09311   -334.02627   -163.41002   -346.68065   -348.38952   -355.17677   -251.50773   -313.22544   -251.73375    -92.45809     87.03026    172.40126    301.22648    181.33819     58.95786    -57.79657   -148.73262     36.75704    175.27135    321.06713    580.46433    554.56992    382.13971     87.58093    -35.38622   -321.61796    -341.4322   -142.70304   -338.42874     28.91061      81.1014    919.65843   1277.28643   1945.09538    1898.5834   1641.56576    836.56302    300.69777      1.74426     -3.54065   -206.51659   -235.14968   -190.70883   -214.94989    -234.5273   -314.03922   -271.90094   -228.33398   -353.75094   -285.72398   -241.56745   -363.35369    -34.23385    -131.2423   -114.87612    -126.0222   -109.17174   -161.05792   -173.10002       9.3752     68.78458    207.33325    -41.80994   -206.58559    -95.80286   -364.12211   -351.54677   -381.32547   -285.00057   -287.80706     -82.5745   -139.01263     -93.9682   -243.00038    -109.7996   -215.79931   -287.58026   -197.22548    -134.5685   -437.32765   -260.25688   -326.48444   -396.01484   -363.77257    -299.6133   -152.41679    -111.6465    -77.04477   -229.80499     91.58506     30.91837    152.97586     59.98618    260.26618    149.85884    219.20057    208.32847    -38.28528    -92.52785   -110.45102    -73.39575     85.36592     43.79846     24.89214     39.23572     82.58975    -86.18372     15.25422   -338.25484   -208.29712   -274.72721   -317.97945   -427.17804   -481.93358   -322.63541   -345.19536   -291.54788   -189.31196    -141.2612    -81.35965     34.74396    189.84047     96.79064    108.49379    153.56701     73.29964    149.30705    118.21589     67.49701   -173.73303    140.16889    166.30611    126.60109   -103.94881     54.64852    -97.21943     -156.283    -96.84556    -81.22308   -265.57907   -344.40012   -276.41545   -175.79748   -192.98553   -177.42747   -249.84386   -115.71395   -138.71216     81.06626    -27.07497     -34.0102     26.13514    118.42233    203.86052    193.35184    292.51014    578.00161    536.63539    641.65118    657.21574    534.36501    598.99619     479.4798    259.92849    259.63264     70.23612    106.40624    -51.72706    -49.07888   -170.36256    -75.11905     -13.6929     35.08137     23.32955     159.4374     -4.49696     53.76468       3.1136    -84.23678   -140.11773      9.65415    -78.95069    -21.95497   -161.64501    -288.0039   -210.48002   -160.04831   -272.89147   -419.26369   -278.01379   -317.06498   -280.85258   -369.48808   -198.67821   -285.94627   -354.57258   -200.27489   -334.90422   -331.65442   -178.45683     -92.3295   -287.47711   -159.85851   -106.53845   -184.09905    -82.82828    -21.41476     37.45499     19.27404      6.46672    -24.88651    -32.99483     -7.46281   -199.53259    -72.46015     -190.218   -251.65152     -0.97629    -58.32414   -227.37044   -328.09332   -270.78149   -344.08647   -237.84792   -239.21771    -24.63244   -134.20164    -117.8737   -193.81449     24.58053     29.47201    -46.07477    -17.53717     -26.1962   -117.07349      2.65942    -16.90794    -92.66064   -162.07963    -75.32949   -102.36167   -267.25793      3.47274    -135.6243   -133.54524     45.87001       2.9252   -195.21458    -49.72506    196.58838    138.88584    100.60672     82.86974    305.91225     84.34586    319.21501    -75.91233    146.11398    203.49156    109.28209   -142.37422    -54.24906    -52.44378     60.76377    -58.41847     106.6523    -67.64897   -244.25474     33.11503   -282.97285   -253.12455   -148.70242   -123.86459     20.82733     -92.2997     43.76513   -116.49923    -10.88571    -132.0913   -280.99903   -154.79747   -298.29406   -154.97667   -110.06474     -416.054   -303.76101   -381.73063   -223.35832   -204.54444    -138.4333   -176.34198   -192.28231   -243.91442    -89.35246   -370.31614   -189.79082   -153.64522   -329.85424   -303.20726   -126.75687   -493.67934   -456.24121   -337.98989   -175.81153   -622.73751   -380.89549   -163.90624   -526.93412   -333.46638   -469.70369   -149.69354   -321.13183   -370.44697   -361.18558   -310.29422   -511.01393   -441.44372   -211.48004   -397.46617    -213.7884   -650.14249   -283.80738     -556.679   -256.51085   -527.59413   -795.20608     -678.815   -690.58147   -593.52133   -599.41502   -863.38017   -940.78072   -911.20348   -426.00481   -728.14611   -824.65148  -1139.58294   -950.18045   -763.63625   -948.84459  -1113.21484   -691.91191   -918.40213  -1075.17614  -1297.09827   -945.00802   -784.85035  -1304.27211   -742.06892  -1110.91181  -1240.14084   -919.13521   -869.89275   -816.72244   -916.62931  -1130.46499   -966.77259   -999.22102  -1239.27251   -1323.5876  -1231.07064  -1042.14009  -1299.55283   -876.92504   -1273.4304  -1463.76563  -1210.91409   -969.80269  -1240.98842  -1358.66485   -749.61572   -999.76029  -1754.70415   -623.99747  -1752.58701   -1220.8617  -1004.53782   -809.61771   -1482.8063     34.30781   -544.88208   -572.02377  -1324.57951          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       16.00    -235.7063   -143.38228   -456.03214   -306.61989   -388.78561   -877.44904   -490.01231   -185.10367   -374.83311   -272.70735   -129.28163   -284.86106   -129.16306   -312.19795   -500.78775   -248.31317   -259.64322   -186.75503   -176.70746   -311.03163   -211.53451   -635.94067   -342.11328   -457.84238   -306.52437    629.19332   1278.28488   1760.24796   1138.96009    357.37276   -113.67328   -146.49302   -177.64113   -261.76726   -476.34086   -295.24965   -405.70772   -254.47448    -192.7288   -199.16255   -436.15245    -412.1431   -417.24043   -281.33454   -438.39113   -418.36062   -356.71159   -248.37251   -267.42257   -459.48082   -379.61076   -258.55028   -249.68621   -318.89382   -241.80962    -81.27969    251.09932     936.9862   1538.63725   1207.43881    596.91072   -114.97353   -343.32537   -273.82612   -311.30597   -134.22808   -281.13857   -378.06536   -279.22621   -150.95985    328.32753   1102.49373   1295.85508   1354.74558    717.00639    519.71009    412.87081   -199.64643    -708.6748   -878.34399   -618.27202    -431.1022    -82.67084   -137.61826   -193.02386   -325.64485   -295.69464   -295.94401   -225.58259       -18.56     76.23164    141.04253    169.35236     84.76018    -188.8134   -264.13269   -407.90655   -352.61585   -191.99974   -191.07952   -382.21446    -31.70179      32.0211     55.93232    -75.66703   -271.06599    -86.58099   -383.55637   -340.51126   -326.78889   -424.77773   -263.45082   -322.99763   -323.63012    -71.09307     -22.6974    143.13355    291.68782    492.43841     48.55941     43.83451   -137.01807   -212.67017   -118.72125     80.38236    444.76644    494.60315    497.12809    344.21937    146.30886    -164.9829   -197.78474   -177.62209   -275.47707    -38.46199    -57.25602    349.81058    887.05529   1362.34048   1985.31578     1926.629   1458.46254   1133.23281    202.49148     31.80924   -384.60835   -282.07031   -134.21862   -201.25501   -362.31041   -206.80101    -357.8599   -191.53138   -330.78738   -260.93259   -312.64407   -235.85125   -202.72372   -173.29811   -166.30923    -81.12786    -69.26315    -36.35791    -137.8549    -46.99957    -21.42069     52.98299    -35.78831    -21.95708   -116.46306   -172.57848   -281.46462    -309.3371    -425.6582   -364.04201   -207.62113   -191.76181    -93.67291   -190.88145    -199.7321   -126.30796   -259.74268   -174.80739   -364.31759   -345.27567   -382.88003   -235.98908   -382.53564   -446.87411   -189.14315   -291.44935   -127.59464   -133.36963   -108.40813     92.70865      6.96809    -17.06325    -12.80253     98.00559    111.50527    247.89909    181.03937      0.57717    -65.76175     -75.6077   -102.49044    -79.10408     35.19785    109.63447     53.70271       3.9802     24.05227      7.79483      36.7936    124.81681   -243.31385   -293.42497   -291.76757   -114.75924   -291.72421   -277.38726   -189.41484   -170.70398   -205.74732     -144.743   -275.97962     -2.14105    -61.95271   -248.08732    101.08221    -65.41289     87.94524     -3.00328    188.58501     58.38559     63.33595   -167.50456     27.70799    152.34118     11.59692     17.22315     40.59022    118.97584    -243.5568   -161.64459   -173.11225   -172.28452   -113.53416   -277.30878   -377.72422   -333.36786   -252.59985     -7.54426     63.19542   -163.47276     31.45858   -159.91901    193.04138    106.20308    289.69056    336.52586     222.9649    520.38247    383.71957    566.95532    589.43107    438.51949     470.7575    710.16137    451.19134    365.91438     95.47019    142.11978    -58.37834      2.26157      8.34529   -147.05538    -25.15543     20.76827    -40.40605     35.17618     59.25848    -96.67764    189.49184    162.86101     -8.36151     33.35334    -26.05422   -103.51939   -388.93692     18.39144   -272.87723   -277.04624    -147.4962    -53.02851   -241.80253   -322.76447   -276.50164   -186.40508   -401.08117   -260.51954   -276.75196    -88.72059   -333.90579   -256.86947   -403.40615   -292.34927   -135.00195   -177.72592    -51.71736    -38.00202    -90.89072    -78.03611     70.94706     38.14592    146.10803     146.9121     49.41296   -149.54448    -29.87664    -16.87827   -114.47785   -128.88421    -49.66533   -192.64927   -261.44078    -312.0097   -162.07739    -175.7786   -203.18859   -255.64936    -55.54079   -157.24638   -139.25697     46.59282   -182.12621     39.18789     -9.40491   -209.84576    -28.07763   -217.58941   -123.60449    -172.3382   -213.24028    -13.58648   -124.08725   -211.00023   -186.47644   -176.58491   -152.81474   -208.85196   -261.97473   -147.45374   -154.98754     82.50324     29.76414    287.34661    -59.94713     12.14687     54.50373    157.28744    205.91462    135.85757     74.53075   -111.13148    -98.31036     71.38617      94.4507    -45.23829      5.18964     -63.1576     -4.79135     32.66205   -143.71152    141.15798   -167.75369   -127.49402    -27.59187      17.7939    -81.50399   -105.15054   -117.53684   -439.97414    -56.33659   -339.30144   -177.00851     -87.3901   -330.27835    -183.1969   -296.00811    -418.3757   -365.53417   -278.34417   -381.57502    -433.9609   -174.62197   -607.39634   -333.70789   -303.51483   -342.41677   -353.11922   -324.09513   -292.02227    -16.09658   -303.51163   -328.86324   -289.74968   -274.45317   -270.72569   -375.24901   -369.53508   -386.91072    -355.3089    -52.47628   -239.21583   -703.06978   -597.26243   -305.04098   -537.02044   -426.33596   -276.91911   -410.72912   -384.46859   -406.86202   -297.69989   -213.80709   -526.58068   -276.89632   -576.36721   -744.61818   -452.70979   -736.79357   -555.34925   -552.37032   -894.56864   -836.24632   -651.88546   -896.41152   -642.57622  -1049.36545   -663.07651   -735.68828   -671.91606   -899.77353   -564.37931   -725.12852   -806.27822   -652.69989  -1135.04045  -1265.08398  -1013.27595   -912.55492   -895.24277  -1082.41863   -935.65339  -1156.83354  -1083.10846   -1113.7912  -1043.57029   -752.80876   -952.41483    -1491.425   -955.64891  -1010.21958   -540.77873   -366.11489  -1148.22809  -1266.43937  -1359.68519  -1006.86639  -1455.39211   -733.13747  -1422.95112  -1014.81166  -1127.68905   -651.72391   -1051.9157  -1040.82274    -587.4121   -915.53771  -1500.18168   -928.39604  -1383.03075  -1080.65561   -845.63605   -529.24685  -1182.33657  -1319.71435  -1005.43932          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       16.50   -371.37691    -92.67186   -418.99024   -348.59899   -421.64391   -688.53072   -397.56268   -321.80236   -299.41785   -232.50938   -198.41344   -537.98524   -222.62555   -551.72783    -379.8287   -286.52855   -347.17374   -225.50283     -273.003   -215.62054   -160.24803   -378.79132   -239.43292    -454.7155    -213.0904    670.30792   1361.85849   1740.20714   1308.30502    281.01563   -201.48139   -326.73477   -206.94933   -412.40509     -242.838   -236.02519   -381.16103   -199.92379   -236.79143   -437.49767   -376.54507    -469.0057    -471.9735     -293.665   -419.80262   -298.22536   -352.12272    -214.4934    -282.5492   -188.29775   -369.65471   -343.35111   -275.89694   -318.28277   -379.09313    -36.59387    284.55456   1160.38693   1220.04981    1074.8135    624.85358     68.19949   -219.77677   -322.99651   -346.44609   -326.74818   -398.92128   -407.81757    -244.6912     16.19323    438.21275   1034.09495   1435.33921   1339.11646    625.85622    377.26622   -160.97461   -998.13178  -2333.30851   -1696.3774   -1384.5436   -436.50128   -383.55684   -328.03158   -431.54021    -324.1478   -405.65307   -520.03428   -270.38734   -209.56731    173.78637     -16.7448    -51.16541     45.61216   -135.32094   -231.26325   -234.61253   -293.47661   -227.56064   -240.38989    -145.1396    102.18388    116.58344    163.32451    -88.31354   -238.62717   -415.96159   -389.54207   -364.24946   -330.38956   -357.77196   -349.32157   -329.74302   -372.69186    -207.2534   -142.28364     13.92394    257.26202     313.0283    122.32666    -51.24525   -121.08958    -73.51339   -172.32839    269.38464    420.48988     745.8265    633.76514    269.84684     68.39354   -147.67102   -172.06805   -170.06747   -301.40777    -269.8276    -71.50809    218.61593    951.98907   1475.62923   2033.75239   2003.09791   1656.72714   1119.18887    710.52963     67.02482     18.50866     -43.3414    -268.3544   -143.70019   -325.08456   -142.20006   -299.29911   -286.75997   -337.11748   -276.66504   -374.68117   -235.78335   -116.12051   -189.97947   -222.70963   -311.65984    -208.6032   -204.39705   -210.11357   -174.34043   -123.40218     19.49754     -1.03939    -33.36941   -247.31201   -422.58118   -466.42314   -413.27416   -356.80666   -180.66697   -264.21205   -198.77864    -53.05474   -121.00926   -197.57647   -443.76529   -290.97645   -457.48725   -344.62794    -193.1633   -320.38302   -331.36776   -255.63976    -379.8854    -78.88458   -260.48363   -113.86782   -250.49767   -105.47177   -115.40678    -40.73588     28.61976    150.92067    307.80464    163.17446      -1.1169      282.267     66.35291    -69.65734    -86.04287    114.86493     -1.51349     -1.85065    145.87001    -33.56932    103.18875     103.2436    -42.45856   -181.52503   -119.26379   -339.33687   -296.48558   -305.88718   -270.21524   -333.10988   -396.96323   -285.99214   -307.33318   -240.32988    -189.6104   -227.25136   -125.13956     86.42051      -24.582     47.61446     57.99142    -95.73639     64.04174      9.98409    -20.84691     37.47357    213.17682    147.11047    -15.13523   -114.51715      46.4761   -103.51358      7.13648     -155.097   -186.05184   -233.94297   -211.21893   -360.93266    -267.0768   -313.18847   -248.51029   -258.86176   -290.13848    -41.90117    -50.12522      7.33146     141.5596    -25.30996    112.62173     172.2823    222.43989    391.18825    460.72015    581.57582     627.2765    604.16516    664.59254    529.79933    422.88669     436.2762    243.57972    258.87682     26.65402    -68.42011    -88.56544   -176.74623   -191.62939   -114.92584   -114.84157      8.83677      82.9205    202.74889     64.64886     43.70046    -15.88903    -10.93743    203.15546   -129.89066     12.35519   -155.34001   -223.69495   -231.91674    -96.96295   -326.31346    -236.9181   -296.53788   -417.86167   -362.38764   -327.40003   -408.50407   -333.48161   -304.26111   -224.31007    -409.5165   -323.20077   -229.47592   -263.56962    -94.87666   -206.91215   -288.08736   -275.27177     -60.5708    -92.10855      1.93027     29.95675   -100.06181    -26.27763    -79.59508    -95.57526      0.04109    -72.18549   -202.33386   -331.32568    -85.08265   -211.68634   -122.72582   -247.07565   -246.20776   -240.72029   -216.79307   -195.38485    -21.87947   -134.36043    -27.80091    -60.95418    -347.6523    -26.98767   -232.02804   -160.47589   -126.39631   -119.89132     40.64296   -226.08803    -36.60969   -295.61173    -128.2896   -270.61011   -125.31476     -4.66242   -168.10548   -198.76785   -170.33099     34.66962    -26.46427    -72.10195    -64.87683    -80.91886     52.24664     64.73995    177.83467    196.02711     67.11566     83.85966    -30.31243    224.42219   -108.56943   -129.73053     -24.3225   -219.92808   -165.12378    -34.51473   -245.85313    -26.48347     54.90131   -127.91153   -173.08176   -277.35616    -98.26596   -206.94064    -85.95915    -79.11492    -39.04389    -66.34102   -229.88853    102.19536   -220.15044    -248.6065   -130.18535   -272.82992   -121.66581   -222.76523   -327.95347   -536.98687    -56.57344   -247.14109   -241.59401   -229.82659   -315.42979   -254.74525   -218.43247   -380.68583   -301.07985    -598.9713    -362.4378    -41.75059   -349.81084   -441.71542   -419.58966   -237.17001    -281.7591   -401.50444   -419.13689   -243.69339   -186.48875    -280.5598   -401.19946   -188.01969    -226.2356    -82.75214    -435.3345   -459.66099   -437.63352   -582.71942   -545.48349   -165.21992   -522.70365   -488.01109    -855.6769   -506.58881   -694.52624   -799.79251   -459.36806   -816.20193      -789.16   -761.12848   -906.10187   -908.07236   -927.50568   -628.85285  -1247.85845  -1079.17303   -767.71544   -830.96545   -861.87055   -1093.7218   -920.74435   -784.58617   -715.17561   -824.48851   -697.32669  -1194.16825  -1215.05006  -1018.51556  -1061.56228    -869.0296  -1385.41766   -1120.9876  -1021.04324   -886.19939  -1092.51557  -1350.06371  -1158.32908   -835.96715   -851.46383  -1819.83814  -1266.39115  -1267.44409    -718.0737  -1049.58622  -1205.36166  -1300.38354  -1332.39704   -1011.0419  -1277.88199  -1687.76076   -821.68829   -982.62738   -398.48151   -947.75253   -841.12615  -1016.04584  -1249.27124   -692.73269   -1208.4891   -979.25236  -1806.28629   -402.03879    -804.6308          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       17.00   -453.99611   -235.73317   -378.95747   -414.18501   -317.40232   -492.74815   -399.41043    -420.0889   -335.37603   -242.90759   -308.81451   -503.53831   -267.63178   -318.97448   -281.62884   -318.32611   -291.14649   -261.84812   -447.94544   -115.74655   -294.95217   -218.08552   -203.76894   -361.24734    -30.80531    480.02614   1537.76297   1780.81727   1362.42605    156.52324   -136.55384   -621.82656   -174.36997   -497.85053   -256.12609   -266.46785   -264.96536   -260.24363   -523.86553   -441.25343   -491.05607    -435.8385   -284.45603    -73.03742   -415.43493   -371.68397   -326.83575   -399.12616   -389.97956   -306.01609   -317.36807   -280.94691    -379.9558    -270.1802   -418.85979   -145.90543    263.79387    950.94401    1339.0184   1264.98354    507.90205    -54.14096    -365.5664   -305.08069   -261.14434   -363.73497    -98.88766   -207.56715   -103.60451    -44.09008    344.90728   1255.44136   1308.40636   1255.98701     720.1232    395.74732   -357.32849  -1739.99551  -2888.20943  -2553.00094  -1409.91601   -509.95904   -155.02792   -263.62477   -508.47698    -304.3766    -78.59166   -307.58417   -472.97134   -206.90521    107.15481     82.12271    105.12545    -19.09364   -203.51831   -249.93565    -292.6763   -289.49259    -270.4277   -240.85112    -51.49827     -6.79686     30.84756    -60.02844   -235.62073   -242.77179   -165.58649   -231.39043   -356.66081   -300.36701   -412.90327    -433.0457   -365.48221   -211.39772    -197.6025    -14.79533    167.58507     245.5837    231.40223   -112.80437     78.41883   -193.58693   -176.97193    -88.84533    243.71596    580.04148    647.66631     548.9885    255.12567    122.29738    -38.78718   -240.34431   -214.90098   -233.05161   -156.71041     91.71717    338.55877    968.39386   1407.76725   2094.05859   2213.88373   1641.53907     943.7456    493.93069    258.50116   -140.49296   -258.74185   -248.77127   -120.26777   -175.13496   -438.29563   -315.92569   -352.59556   -409.56793   -275.47416    -379.0264   -410.23555   -366.03818    -89.42357   -119.52585   -109.73545   -153.72941   -196.65616   -173.39018   -154.51529      -9.9018     -3.26161    -73.69002   -119.02337    -209.8775    -690.7343   -453.39694   -341.75975   -415.47899   -275.82267   -200.59698    -68.72785   -138.23679   -189.35411    -230.1398   -287.39352   -333.28302   -324.40303   -370.97684   -152.43494   -398.94877    -444.4595    -298.2553   -358.26924   -411.75664   -373.96711   -189.43448   -128.20441   -180.20409    -57.74593    -78.61929     91.04239        90.03    209.07513     254.2926    188.44531     69.70521     77.24795     73.04362   -167.85114    -60.92655   -173.92721     -7.54937     51.20394    120.64963      46.7286     88.58612     27.91145      -9.8807   -229.70465   -285.07669   -319.44759    -55.27475   -258.58156   -433.51221   -299.47765   -234.23484   -254.79331   -173.78716   -210.89919   -133.18996    -71.24569     96.99412      57.2307    -96.38393     60.73357     114.0024      43.3027      -53.718     84.46076    -37.78534     55.92558    -19.00084    121.80136    -28.01489     62.81814    111.18179   -311.76125     -3.34584   -124.77257   -336.46032   -206.04145   -169.24224   -411.85143   -301.35905   -239.52496   -167.01832    -81.22434   -227.56527    -56.91747    -154.0782     81.96887     48.39938     60.75108    186.98947    263.13685    474.53893    400.62496    703.14108    660.33465    602.56585     533.3424    710.38818    486.75828    391.32665     305.4517    282.96615     27.73349     20.89827    -61.81047    -13.38575    -83.19595   -211.59968   -236.57887   -106.40343     -0.84315    -24.89645     91.47895     34.80522     37.63184     16.98729   -155.92633   -252.27074    -157.9302    -316.2725   -174.28851   -310.97751   -205.07323    -305.5065   -188.23379   -307.56871   -288.33482   -316.87828   -328.41546   -270.82121   -353.56274   -308.08373   -355.40385   -226.42377     -297.509   -327.04498   -168.75952   -126.66782   -268.56242    -167.8417    -88.48304   -157.31589    -47.76493   -120.53556    -83.52103    179.08904    -70.65313      -96.208    -76.15611    -42.49411    -57.28306   -165.71118    -97.08477   -243.20709   -131.78911   -225.11743      -180.43    -238.1143   -151.29147    -229.3728   -160.95127   -148.35012   -290.82987   -104.21029   -116.94522     -82.4702   -202.52588   -265.42929   -162.78012   -200.15707   -111.36159   -175.12572   -239.51986   -261.33208   -236.27398   -117.12268   -426.86019   -410.54272   -187.02766   -377.87694   -225.34099   -192.74363   -254.03082   -515.95238     79.30096    -57.72115      9.09281    188.69625     19.06516   -243.19859   -134.83666    103.32977     62.94786     -81.5456     58.21358     23.64579    156.66177     92.02694   -137.63051   -270.08659     49.84902     53.33697   -206.98189   -324.23017   -268.19815    -90.49596    -65.15028   -250.21576   -210.03698   -278.68003    -88.06513   -194.25808   -196.68446   -406.05924   -130.35319   -315.33846    -78.52329   -433.01194   -274.85813   -225.33902   -165.43489   -321.96833   -303.17044   -319.66968   -236.14082   -677.80436   -506.63235   -529.71675   -507.49359    -315.0226   -242.32806    -314.4409   -338.03886    -522.6216   -295.85781   -275.04069   -495.73246     -2.80995   -345.83666   -500.02111   -537.90921   -373.36117   -224.29005    -249.4207   -342.76936   -626.12728   -608.90365   -492.41971   -424.65941   -431.75132   -670.80243   -467.18445   -558.49824    -554.1967   -507.36323   -509.89708   -348.87706   -573.37577   -652.43275  -1007.91821   -679.20033   -645.92246   -681.33681    -759.8525   -743.71006   -690.44979   -832.73029  -1029.45192   -964.90342   -782.46227   -805.45907   -956.01432  -1105.86215   -983.79077  -1089.60561  -1050.81344  -1151.20044   -896.37227  -1341.26277  -1370.54202  -1012.02067   -843.23395  -1269.88554  -1025.64856  -1441.02157  -1136.60053   -1281.2625  -1038.62313  -1510.35682  -1045.12642  -1176.30681  -1209.05703   -990.01971  -1216.54832    -987.4245  -1442.40465  -1538.15113  -1408.55955  -1227.38133  -1108.69125  -1374.25499  -1556.94342  -1031.18973  -1201.06457  -1364.51695   -1547.4286  -1643.95083   -875.81688  -1561.54058   -881.16458  -1261.36932   -959.67331   -451.29207  -1316.06353   -355.40676   -791.37584   -127.41727   -923.93826          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       17.50   -457.50178   -401.78144   -369.14658   -469.11384   -107.63203   -414.13158   -398.98423      -245.39   -310.76219   -302.94093   -321.17341   -394.80673   -266.56894   -118.70539   -140.96175    -362.4421    -94.12765   -358.79282   -587.56659    -131.0467   -294.47799   -275.96224   -258.64635   -366.21117    -55.60003    483.39519   1220.45018   1709.51347    938.91411    330.04389   -120.52213   -432.26802   -175.68388   -215.86607    -369.8474   -295.19795   -469.83415    -296.4462   -158.09787   -315.89836   -422.26198   -477.03384   -228.75605   -298.25414   -396.60155   -194.88836   -273.56407   -389.60868   -412.28809   -297.85378   -314.91056   -263.38324   -353.37781   -312.49977   -244.13212   -301.72949    256.21808   1131.06914   1197.01476   1127.61414    540.62467   -181.92321   -180.80092   -346.49807   -284.83905   -159.45984   -209.21086    -278.8255   -314.94445    -21.69339    379.65054   1291.24586   1455.91198   1297.22716     871.7887    275.87541   -296.38271  -1784.77802   -2508.0553  -2183.89568  -1297.39826   -603.29709   -329.30291   -208.13747    -256.0586    -285.2322   -204.90468    -223.5711   -299.34032   -227.62637    -91.40372    212.20915    110.00374    100.93337   -245.41891   -302.15356   -301.87253   -419.69653   -281.47556   -263.94347   -137.92726    104.79627    -77.42749    -56.25494   -173.89655   -147.14242    -235.0091   -312.17258   -208.00704   -488.87244   -367.59272   -378.47725   -318.40952   -430.61323   -143.17391   -140.90951    196.45787    219.52537    130.25944    107.53524    155.08565    -90.50581   -188.65098    -17.37499    132.14226    498.17385    897.58737    460.93931    247.43612     41.01092     26.13778     -214.017   -286.92398    -48.51356    -137.9352     18.45078    356.91313    830.93166   1509.38292   1921.61261   2076.34568   1653.83998   1196.35995    446.53272     21.08747   -326.80169   -144.05275   -273.48475   -246.15494   -227.14912   -303.70718   -361.66263   -300.53231    -301.7857    -289.1756   -348.68346   -301.68872   -233.46023   -154.32103    -64.35772     -167.476   -219.23204   -137.30426     22.27053    -21.88755     -82.7963    104.57277     227.6499    -78.21733   -166.04177   -157.92327   -225.02787   -187.05635   -175.49829     -259.838   -410.44623   -173.45909     -1.15869    -76.40245   -121.75845   -166.22172   -313.45526   -185.59089   -326.26581   -440.32881   -392.10361   -359.11669   -330.01862   -277.12072   -289.21341    -58.23107   -188.64947   -248.38647    -48.22226   -151.28344    -73.94984    -96.24356     94.53549    147.03475    140.15925    275.35006     82.65086     19.92399     81.39661    -44.63269     -4.20891    -30.41592    116.72568    -74.61637    119.30798     -46.7028     99.61923    -20.59151    -97.46301    -45.22217   -224.22111   -142.08483   -348.03689   -294.01466   -424.25004   -353.84107   -139.08782   -104.52903   -227.16556   -134.06332    -83.49551    -39.20509    -29.74568     51.93017      98.7738     31.49038    174.51522    -70.50814    140.62492    167.98512    -93.97935     55.16129    -93.72734    -68.21301     89.11537   -125.16193   -159.18659   -113.66956   -124.05102    -48.39488   -165.29797   -212.71327   -235.04328     -120.276   -351.17618   -524.15635   -261.62741    -77.34758   -117.54272    -61.85206     -6.10142    -68.49629     21.90212    -66.11021    374.46983    195.77435    259.98957    486.19457    412.82223    602.85998    809.32409    636.64517    709.74424    508.84314    567.33559    341.88852     48.58172     18.32011     92.41849   -195.47502    -81.31499   -298.14478   -208.84007     33.88766     60.69444      103.901     61.88048   -107.47776     78.48788    -24.80146    -30.79411    112.93574    -36.44168     37.45847   -137.32814   -106.18033    -251.4545   -204.36803    -88.68041   -314.31538   -460.06908   -400.12112   -150.20125   -424.07224   -266.38391   -406.71301   -320.29793   -420.21641   -212.68631   -178.03306   -177.70728   -241.91146   -119.21834   -205.68991    -111.4314    -22.68435   -120.20484   -126.03904     19.00889   -113.88077   -111.53183    -34.98791    -81.35592   -129.14529   -226.31474    -18.55263   -120.12085     -160.737   -373.49588    -58.32888   -190.09553   -205.99032   -239.55813   -231.17349   -291.83953   -209.27461   -248.90032    -166.9899   -107.48018    -37.62218   -101.27957   -192.21741   -161.09738    121.65506    -141.1051   -192.83222   -202.58116   -286.19574   -241.16944   -209.46077   -158.93338   -233.05989   -340.59908    -82.11339   -110.76883   -149.54984    -11.36759    -92.62735     12.99522    -24.61229   -148.05464    -177.3618     37.88168     34.47565     52.40649      5.39032     -66.9505    147.36532    133.85251    -86.30599    -24.79921    -52.22875   -118.01178   -291.28643     -3.22194   -251.92273   -268.77966   -303.90736   -216.26859    -71.15449   -218.30503   -399.36163   -135.33218    -45.55681   -104.83418   -105.69915   -120.21344   -294.45583   -202.67661    -100.3245   -227.61051   -299.89565   -346.79944   -268.31588   -220.98925   -349.96522   -597.23911   -554.38821   -378.98182   -427.52946    -719.4032   -244.97423   -365.27054   -511.21002   -300.96139   -345.03884   -389.14256   -309.41563   -409.79069   -414.89337   -247.34443   -503.43341   -522.78088   -420.08125   -428.07232   -714.35325   -325.66452   -677.55525    -238.6129   -481.88978   -568.59003    -574.0614   -530.83291   -456.42704   -230.70945   -386.15758   -472.02213   -603.15623   -599.79602   -451.60831   -776.21987   -531.81122   -716.94676   -759.27285   -978.27565   -682.86513    -513.3158   -995.64676   -843.44128   -942.27672   -751.37577   -951.96009  -1091.54794   -792.69645   -835.17622   -734.65666   -858.09618  -1483.91287   -848.37628   -770.88727  -1225.42574  -1191.34175  -1076.52824  -1240.30697  -1196.26161   -706.42965  -1397.53664   -696.71383  -1207.79637  -1145.35189  -1237.48705  -1404.80753   -985.59537  -1276.07531   -881.98288  -1402.78461  -1326.64708  -1228.77247  -1301.43873  -1087.80657  -1074.54127  -1538.47313  -1010.51244  -1341.92959  -1376.67686   -742.04406  -1645.14639  -1270.36318   -1167.7609  -1402.01595  -1844.34378  -1239.40398   -334.43786  -1313.40829  -1599.88765  -1623.17518  -1317.17745  -1572.23757   -968.52381   -932.21172  -1880.50411   -1319.5192   -869.20603          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       18.00   -435.64897   -485.53718   -413.05359   -536.72172     25.52504   -391.85282    -346.8012   -156.00478   -451.90535   -295.22269   -310.70074   -360.49432   -272.61379   -328.53177    -16.84459   -322.17623   -305.38552   -389.44511   -339.02281   -220.74637   -251.61511   -406.23747   -315.76822   -316.39485   -140.05083    671.51425    1108.2427   1537.60826   1017.22581    347.00061   -153.49886   -318.36505   -125.01986   -143.28077   -336.03731   -401.14496   -382.94509   -423.19265   -222.00026   -223.76158    -411.3585   -450.17034   -226.21375   -416.82072   -348.96176   -286.60558   -267.42009   -180.01949   -332.75337   -390.65165      -460.98   -433.80121   -390.03076   -382.14028   -295.14128   -193.59483     70.31816    996.72296   1657.82392    989.19715    277.84493    -99.69382   -180.73247   -230.33427   -273.83924   -409.16315   -203.27602    -289.9943   -300.51351     83.50368    341.32774    999.97871   1549.29172   1228.58501    724.89291    334.43158   -364.15002   -945.58967  -1416.65988  -1419.38405  -1088.41556   -516.34242   -231.10029   -203.91846   -381.51495   -447.97026   -387.97738   -333.26302   -329.37356   -144.06557     11.06484    172.36412    118.89685     88.12371    -37.98514     30.37956   -270.46616   -392.78615    -345.0876   -296.80143     51.97698    113.80637    -43.20926    -36.11276   -126.64772   -188.73412   -199.64953   -197.46612   -247.94569   -355.85767   -194.62461   -243.66145   -330.14122   -293.89026   -280.17278    -47.92952    168.89578    253.73491    218.48044    211.33999    -16.26312    -86.05949   -166.75132    129.97247    325.15772    582.44109    623.72588    472.45812    285.07316     93.83082     -37.1995   -268.45275   -213.89091   -252.17842   -241.82637     56.32197    330.73343    918.95099   1677.84287   1975.78999   1848.27749   1447.29051   1063.57037    491.61735    122.09102    -36.06541   -245.80106   -163.25164    -95.52204   -395.76534   -386.31074   -291.48479   -378.37253   -247.33729   -324.00185   -270.60116    -271.1171   -248.57061    -53.79565    -48.20681   -144.85783    -41.59668    -46.54751   -158.24662    -66.38219    -93.59156     18.16807    -58.31089    -22.29739    -270.0028   -320.94253   -264.94213   -334.40256   -354.19171   -355.26695   -105.52868   -319.21374   -276.01762    -29.73357    -173.9344   -385.50983    -326.8628    -244.6228   -595.07265   -334.85116   -447.94203   -380.60894    -281.7886   -425.99029   -369.13801   -399.36368     -193.216   -201.04119   -244.69167   -202.28455   -163.73404   -173.12722    232.50646    312.51021     92.96723    -14.31561    214.80943     67.33204    -34.49459       0.0104    -14.74584     41.14284     139.8379     39.08236     51.81402    119.95275    341.95879     153.7472     35.09471   -117.29897    -328.6578   -199.90899   -463.52646   -381.47003   -332.59303   -371.43969   -151.69139   -219.53321   -307.94506   -246.44717   -255.30821     59.34936    -92.13325     44.62908    120.86168     48.42322     29.68966      7.73567      -4.8027     40.09948    -13.32983     30.64638      0.73564    -25.28821    125.48112      0.46737    -15.24939   -107.89757   -182.73196   -117.56139   -384.16096   -205.91004   -314.48831   -239.15603   -236.78329   -280.31389   -291.18475   -195.19069    -48.94215     31.74633    -89.75955     15.02058     79.06554     63.33016     171.9034    227.24927    368.07088    202.07371    572.47191    404.23085    647.65185    684.05686    739.65718     544.3897    296.58681    269.57891    137.65581     46.67884    -92.69039      3.66668     35.23185   -141.74561    -42.78981    -97.05889    -93.97833   -107.51909    137.76693     33.26833    -39.80701   -127.57348     -0.38251    -71.49523   -179.94244    -117.1674   -154.73226   -229.75064   -464.44684   -431.29152   -376.30461    -328.4307     -170.246   -357.37239   -351.57595   -219.53578   -339.55315   -262.39405   -273.28649   -443.20276   -276.47574   -234.97138   -438.64548   -175.67344   -285.52723   -309.30717    -167.3105   -214.87978    -64.81037     35.72122    115.22223   -176.48633    -25.74258    -51.83998   -280.40228   -109.60684   -111.60623   -121.12439   -208.14521   -162.44314   -109.56458   -280.15809   -208.88709   -250.43182   -174.25355   -162.08328   -231.37647    -344.9878   -306.15145   -250.94927   -195.31153    -90.69787   -191.13292    -140.7723     -4.69445   -303.76023   -251.53988   -164.55345   -248.73312   -402.71852   -296.03042    -191.4951   -358.77298   -131.64064   -338.06656   -262.10905    -56.20037   -216.99807    -94.61353    -127.1242   -140.48753   -194.74524   -142.21894   -227.83295     87.52871    -69.14334     115.8878      1.18746    163.02212    104.08717      24.7879     -7.56012   -192.54309   -171.74277   -163.14342    -44.64621     76.38331    -65.75642   -243.90233   -124.07623   -141.34788    -92.05605   -245.56528   -130.02143   -352.22305   -104.33584   -197.63486   -112.47111   -472.32509   -128.81637   -327.53105    -153.8838   -296.52598   -368.97546   -272.39491   -306.90138   -286.98845   -403.97951    -527.3045   -370.91966   -563.06413   -461.26785   -317.23617   -552.59307   -388.04478   -479.73107   -366.86834   -439.58751   -309.38032   -391.02962   -436.42208   -554.70832   -359.38892   -367.34808   -495.54028   -367.77795   -353.27199   -468.21932   -493.66754   -576.79018   -469.23218   -522.46688   -347.74835   -411.31046   -521.67051   -420.14161   -367.82898   -641.84376   -632.77349   -488.89278   -829.45911    -534.3158   -615.53697   -579.97598   -590.23132   -604.11709   -709.09462   -726.71239   -884.95183    -708.0768  -1084.51115   -986.59586  -1138.18645   -848.45895   -627.41756   -958.99979   -920.36012  -1064.92223  -1144.48579  -1122.30253  -1063.99544   -969.36178  -1534.88179  -1292.58436  -1081.37044  -1254.33137  -1419.48755  -1400.48866  -1207.93269  -1303.21281  -1074.43146  -1461.86622  -1383.15712  -1211.91073  -1045.22798  -1228.58682   -982.73361  -1935.82575  -1308.22371  -1177.33156   -1315.1172  -1137.54131  -1559.26693  -1397.44049  -1351.42371  -1213.36325  -1517.97855  -1397.19126   -1479.6478  -1203.85428  -1276.76324  -1399.47924  -1781.45235  -1207.79737  -2055.78702  -1566.88564   -966.04243  -1335.01366  -1608.77031   -2428.3938  -2214.45017   -1688.5109   -847.44639   -2606.5713  -1337.99435          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       18.50   -405.23121   -465.81893   -498.97499    -553.4844   -171.47247   -344.82313   -295.65974   -168.82166   -626.63018   -302.30212   -328.67281   -352.09476   -411.16342   -457.69244   -149.74723   -294.08073   -406.32093   -221.83823   -300.71773   -327.71762    -322.2681   -470.49721   -386.46318   -161.59883   -164.34483    572.70868   1107.31932   1552.40037   1087.44027    268.81403    -97.07336   -316.59343   -257.13257   -346.40087   -269.40167   -298.48733   -438.01469   -430.49532   -355.71748   -354.18496   -256.84284   -419.09439   -366.30367   -501.29818   -360.40211   -402.75407   -362.05309   -248.92184   -229.18992   -273.46867   -505.87006   -245.00548   -398.38869   -346.96492   -532.95316    -173.3313    338.78392   1040.74445   1363.02494   1209.61899    351.48405    -44.25231   -339.44134   -300.67856   -301.85174   -289.68181   -349.10816   -236.22812    -294.7674    117.05838    443.01039   1149.41116   1365.59522   1194.75389    728.44642     552.4059    316.25463   -167.43435    -477.7572   -554.93882   -665.46141   -433.11947   -328.87025   -144.51421   -357.73041   -318.21295    -456.0085   -186.36999   -405.18777    -57.20179     208.5064    167.24497    231.70923     21.04441    -80.75062   -242.12472   -231.02773   -241.08162   -327.78468   -279.07253    -40.24549    -33.62376    -24.93672    -21.20864    -63.35173   -332.40083   -237.47763   -383.82018   -328.01107   -259.75486   -409.22446   -383.97858    -433.7093   -300.90778   -340.18257   -154.95746     78.98149    293.64507    303.19618    236.86259    -91.00054   -114.85735   -220.44096   -101.09326    306.82329    469.75894    635.70143    535.85782    168.71889     21.41092    -15.21757   -100.73287   -267.17615   -211.35777   -275.10624    175.42396    278.90666    935.75036   1471.60524   2077.35024   1926.91188   1480.11563    965.75337    526.83541     82.56055    -88.09768    -107.2388   -280.42116   -179.19156   -332.91943   -477.55035      -135.93   -305.09833   -259.76307   -261.18163   -263.16697    -98.47189   -152.02034    -103.2178    -47.09426    -97.86749    -59.98871      7.95048   -243.40654    -14.25177    -99.87333    -22.64088      2.15473     12.67841   -145.59657   -178.18254   -124.97152   -300.22252   -312.54909   -261.53209    -239.6693    -66.85677    -24.29413   -220.99182    -87.89232   -343.24839   -245.08013   -358.32838   -311.41666    -260.8368   -284.15555   -278.90335   -438.53458   -369.39803   -254.84815   -206.20703   -150.96777   -152.84052    -48.57477      29.1968   -140.03616    -67.95401     109.7457    182.85974    266.87402     78.09997    -19.48158    130.90958    -89.28616    -27.67056     61.30162     80.78499    -10.08225    139.42268    227.56513    210.48539    115.39872    -239.7544    -19.17972   -158.92009    -96.76505   -194.09596   -284.40987   -313.32217   -290.59198   -411.12366   -268.06407   -448.29936   -208.56696   -211.03457    -57.15898   -189.38239    -11.64601     77.16031    -30.25103     28.66492     98.14214     25.13697     99.79421    -71.12786     96.01516     56.58043     -73.8108     33.08591      65.7809      8.12043    -23.12292      1.70975    -65.89165   -211.65138   -355.75238   -274.76414   -304.80262   -252.28272   -266.64305    -362.0107   -157.22432   -151.57799   -112.27097    -22.21781    -92.46848    -99.80704     34.02663    147.67095     35.94172    349.11663    301.66613    323.31639    655.17274     516.4917     650.8761    646.00979    708.42224    555.57333    292.62289     380.7574     73.38601    162.27962      0.39073    -128.0958   -144.49185   -154.48435    -80.42936      61.0705       1.9003   -154.93629     -1.47851    -30.77753    -91.86351    111.26874     51.00641     69.98801     -1.50059   -219.27599   -210.89754   -353.28792   -133.49935   -261.91977   -250.85507   -505.59627   -440.36248   -239.56022   -300.56072   -533.03493   -257.64654   -347.07888   -331.92827   -375.80522   -217.25681   -169.40728   -308.38824   -113.00441   -300.22628    -148.0923   -193.38413   -108.80223   -284.35739    -42.44258      5.88962    177.28107    -78.01202      -1.6012      2.81946    -56.52931   -169.25413   -158.09978    -74.98561   -250.91401   -170.51939   -292.47917    -198.9454   -184.49209   -293.78635    -172.8864   -200.57894   -344.05529   -215.53471   -281.11714   -199.49689    -184.0386   -128.72329   -288.55767   -317.24331    -97.29644     -15.8893    -75.92915   -121.47037   -140.88978   -201.93996   -204.36558   -138.08355   -290.10873   -184.58757   -348.75092   -368.68275     12.27688    -249.3612    -95.61669   -323.98475   -146.21488    -41.53627   -142.25589   -132.30099     23.76462     -4.91944      46.6729    131.72433      5.83613    -45.83883     -6.56898   -125.08459   -233.13635   -210.56286      -206.73   -256.16193   -208.07265   -451.00204   -349.82352   -269.27353    -268.6092   -453.14204   -336.20589    -213.8392   -380.99421     -196.562     73.55305   -211.71506    -211.4436   -341.12132    -96.18917   -302.14163   -232.52895   -474.38566   -206.79332   -397.07356   -604.89481   -434.81123   -493.36882    -371.6473   -555.69337   -458.22606   -282.62916   -525.37026   -382.29689   -621.98349   -529.21433   -267.77183   -444.84269   -488.54272   -449.97319    -558.4081   -413.50332   -542.55526    -433.3447   -743.67655   -686.63298   -629.91993   -578.50854    -639.1576   -567.39114   -599.30298   -633.93463   -788.36918   -362.41938   -748.39094   -712.98834   -555.89994   -697.25687   -643.61008   -714.21955   -616.06109   -723.55214    -761.5055    -760.4623   -712.63008   -964.82402    -784.1637   -797.38106   -921.66014  -1266.31777  -1035.29357  -1194.10983  -1204.91394   -812.25214  -1087.89017   -1044.7033  -1036.37876  -1156.97716  -1315.96113  -1550.95269  -1196.46476  -1994.15719   -889.41057    -1426.323   -1438.9838  -1128.78729  -1063.16164  -1523.52931  -1314.53538  -1376.09899  -1000.32194  -1738.45785  -1424.11835  -1382.40209  -1549.60989  -1369.50884  -1452.50488  -1939.07577  -1806.91285  -1520.48551  -2072.35247  -1279.87851  -1320.25158  -1504.97843  -1524.41812  -1734.69553  -2190.80889  -1249.94538  -1812.16184  -1878.48284   -1910.0905  -2104.64988  -2266.38919   -2213.0463  -1839.29017  -1792.90255  -1746.78028   -701.71854  -3022.20279  -2917.23571  -2443.11755  -2396.98732  -2404.92699          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       19.00   -324.55064   -417.17139   -544.63837   -564.00693   -321.13948   -277.51649   -294.48981   -150.13454   -490.00568   -358.01715   -318.87405   -425.33418    -532.4289    -457.6317   -261.31268   -370.71614   -319.56463    -237.0793   -429.33649   -373.63714   -350.42793   -314.43981   -427.39201   -186.55414   -205.00318    546.14673   1232.61423   1564.82858    1209.6002    524.70673   -100.82591   -289.39626   -192.40343   -398.52577   -375.13235     -310.845   -519.33902   -254.84966   -302.24174   -387.55992   -679.21108   -429.64952   -510.70015   -349.46461   -443.41083   -368.54245   -359.29029   -324.19441   -332.18622   -478.88299   -522.17007   -296.36649   -361.72519   -341.02501   -216.43326   -101.09969    303.05565     822.6515   1303.10657    947.83834    693.16592      143.534    -356.1701   -237.33836   -295.17917   -369.67534   -395.16276   -156.25267   -339.85864    -72.62917    448.73752   1258.80665   1298.19552   1181.76289    756.05963    495.97289    526.54758    223.49606   -269.71014   -355.59794   -277.29271   -484.22593   -100.51125   -318.69347   -544.43287    -490.1905    -388.6921   -199.57012   -214.02756   -224.11011    -18.18522    175.08148    -31.00522    103.87349     56.77532     -196.108   -265.87855   -361.36435   -319.30164   -327.90814   -330.95621     19.81443     95.62279    -33.11652    -123.3137   -258.68141   -231.73267   -259.32733   -219.44123   -432.49402   -385.23894   -346.89446   -349.87178   -283.35066   -214.16033    -29.20482     28.41664    267.07292    333.07096    172.16782      6.13607    -60.18076   -115.99894    -60.39987    162.53003    439.29666    548.85047    620.26457    292.89758     70.58897   -164.91474    -138.4601   -113.00309   -169.38225   -139.47087     -9.70724    210.11604    945.21644   1395.00248   1798.87953    1960.1414   1713.20145    976.20814    409.38538    168.06324    -58.06135   -153.47224     -149.908   -294.90698   -176.94129   -185.81806   -321.47777   -163.95703   -307.70884   -351.20749   -333.29659   -392.40347   -239.27119    -76.31894    -71.44573     29.09894   -214.27007     -46.7922    -64.44892    -67.91538      14.0022    107.17934   -216.75132     33.52721   -141.57765   -108.34316   -168.81625   -201.61762    -272.2088   -257.93748   -154.25045   -174.07922    -121.2351   -206.70854   -266.91967   -201.27293   -248.68545   -390.55547   -174.16142   -436.25269   -403.44683   -273.42307   -376.63891   -314.54747   -280.22175   -229.43766   -149.40255    -71.26446   -120.10073    -38.76632    -10.59575    101.37543     34.37536    181.25237    174.62923    281.76032    113.54722     27.99929     -22.4013      0.18024    -56.04162   -185.11601     82.24188     70.46146    109.51768     90.93006     62.54011      5.95371   -115.35601   -163.55954   -204.71799   -219.53252   -335.30099   -313.24498    -290.2899   -352.80266   -297.56709   -232.82376    -85.60122   -102.25393   -163.61889    -261.0915    -53.16901     -0.33549     92.93826    157.70304      24.3609     -6.19094     81.64277    -51.37176     29.83165     56.88684     70.27169    -28.47576    -21.48322   -182.04182     56.71416   -184.60494     37.87846   -233.00001    -11.77246   -161.05881   -440.75352   -287.68673   -235.62658   -161.28088    -48.80375    -205.0043   -373.64057     40.40896   -237.65056    -69.76151    -94.95922    -73.93026    269.38129      64.5105    132.38218    367.03927    444.64752    548.74682    704.73259    762.78455    584.68236    655.94203    601.23623    381.56854    170.97602    -65.32512      7.55152     18.87127   -147.26779      5.94067   -225.67595    -99.22649     92.59995     87.24733   -103.63801    164.06427     30.76584     49.60875     11.99595     65.94042    -84.62176    -94.23022   -204.20683   -229.91676   -300.23902   -301.45946   -393.84099   -401.51162   -331.90179   -296.09095    -303.4509   -279.37655   -394.56868   -328.23619   -334.71311   -136.03837   -331.93858   -153.62531   -161.29161   -204.27274    -307.1331   -358.98103    -98.97933   -199.59723   -145.36678   -118.33905    -81.42227   -114.97389     -4.26838    -99.97473   -207.86275    -95.21057   -201.42297   -128.21346   -276.72238   -249.90214   -318.76958   -171.09897     -429.135   -231.48487   -225.61323   -173.96736   -314.44015   -280.70754   -231.06472   -307.65094   -167.00351   -192.90167     -94.3526   -300.86516   -256.13822   -178.52476   -198.38085   -221.25897   -213.23117   -415.47388   -197.12143   -396.69376   -190.57856    -119.0718   -264.73056   -310.61326   -348.96648   -284.48553   -252.75052   -122.91849   -120.68864    -52.27985   -125.31491     20.11747    -67.09859    -21.17985   -178.85424    -56.60911    -69.11226      4.29433     -15.0794   -220.48423     25.72303   -112.90977   -186.96661   -335.03181   -447.01123   -186.93597   -299.16641   -360.88322   -307.77216   -267.81213   -594.98415    -196.4535   -215.19073   -348.22192   -483.16812   -386.39704   -256.87805   -400.79791   -162.57618   -211.73914   -345.83517   -183.53233    -413.0935   -481.09896   -566.38077    -501.4274   -468.50886   -496.63778   -550.99422   -350.37813   -699.64889   -490.25887   -693.28733   -487.99679   -442.39686   -458.68658   -553.69609   -408.66993   -530.49954   -393.08671   -733.43548   -562.68779   -630.27399   -608.06951   -727.13626    -846.1488     -521.357   -548.03925   -550.54263   -662.21769   -630.18806   -811.30126   -391.16097    -734.9719   -585.64274   -695.83413   -712.26866   -480.39089   -724.02802   -736.29373   -546.04254    -790.4198   -703.33681   -1003.9715   -811.13802   -783.28653   -1052.3533   -982.43154   -839.22196   -1100.3001  -1268.50379  -1194.86998  -1015.61281  -1075.88609  -1181.25028  -1312.58148  -1334.62001  -1117.65879  -1382.30236  -1439.35781  -1626.38445  -1363.47367  -1461.84515  -1158.88797  -1720.22525  -1083.66919  -1069.75069  -1352.98126  -1412.89953  -1308.94914   -1593.9786  -1588.52837  -1441.34646  -1711.53896  -1463.38017  -1569.08183  -1540.96424  -1510.55041  -1814.96837  -1448.14053  -1958.76458  -1963.54244  -1309.96727   -1524.4926  -1769.39351   -1817.0071  -2099.16691   -2162.9684  -1769.07252  -1736.13748  -1972.66831  -2668.33179  -2150.43999  -2018.47684  -1861.87026  -2079.18923  -2332.54587  -2232.40299  -2427.18547  -2142.79072   -2131.9795  -2206.59017  -1992.45271          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       19.50   -233.31481   -357.77987   -563.15245    -565.5985   -342.92013   -261.75059   -279.99456   -147.95931   -289.00533    -424.4618   -229.94402   -477.40882   -478.67201   -426.00998   -325.18137   -236.45443   -366.43003   -458.23646   -487.31514   -378.98977   -330.12993   -346.93892   -364.34593    -95.54659    -310.1601    478.15998   1333.95257   1592.40806   1307.78667    358.06196   -278.63859   -276.46403   -344.42955    -305.3298    -415.2153   -316.69876   -567.36223    -149.8778   -430.58595   -419.55233   -250.65346   -409.71641   -367.87671   -288.09396   -355.22927   -389.04276   -424.24581   -362.34188   -348.76267   -352.80795   -317.08216   -438.19383   -210.48795   -513.89816    -85.70822   -258.66156    397.12913    940.98777   1264.44352    969.83399      342.002    210.94136   -387.00889   -428.37151   -344.87716   -263.45686   -228.39147   -399.41121   -361.06792   -237.45263    623.27909   1088.83028    1357.2991   1303.65731    789.69716    429.84123    257.83764     84.95025    101.04302   -324.59867   -297.77111    -205.8364   -427.99023   -389.55911   -382.39586   -321.40164   -287.97106   -532.89379   -133.09886    -33.90198    -54.25791     49.77404     39.38476    -52.79691    -51.69575   -306.42256   -345.64661   -449.51504   -257.53353   -127.95934   -166.73239       5.7175     58.48217   -135.49766    -56.81047   -356.61262   -357.87199   -306.23465    -382.1537   -260.88333   -326.60437    -337.4928   -356.88182   -467.33735    -311.4261     58.51382    117.77421    335.75716    225.27976      71.5961     26.06787    -79.95954   -152.67868     22.16626    204.21304    337.83403    650.83262     412.9155    377.04114    105.59464     -1.89621   -250.09024   -106.98441   -269.43017   -106.48948    -84.80132    230.55834    851.42309    1683.2764   1999.79051   2081.43557   1597.89831   1117.77519     532.3824     67.79043    -70.05548      1.17911   -228.52661   -374.60874   -300.83213   -342.15705   -292.37946   -288.05904   -412.51433   -401.34051   -141.32152   -145.89919    -32.61882    -92.94055   -189.61858   -162.45954   -169.67373   -127.60986   -250.19595     -78.1094    -60.03203    -56.64903    -189.3466     34.22149    175.14677    -27.91144   -106.26903   -207.95943   -332.04447    -188.8422     34.47397   -292.40201   -167.01006   -179.17883   -146.88075   -339.89302    -329.7965   -284.42256   -249.99914    -384.2936   -410.87897   -215.71982   -330.04498   -328.50137   -307.73944   -361.83154    -259.2236   -159.80377   -191.51335     11.47111    -26.21548    -46.73678     67.16165     158.6234    205.65747    242.95866    279.76421     -58.5283    -26.74628    -11.59237    -32.70234    -43.15943    -78.72461     29.96839     28.85349     49.88693      16.1682   -103.19196    -58.74648    -91.49683   -268.62726   -343.59673   -339.33763   -329.56801   -389.35623   -485.68969   -256.18561    -258.3699   -115.57041   -196.46081    -10.87984   -141.91323     57.45074    -12.49342     61.34002     23.90727     71.01661    -23.42994    -45.35407     40.01434      3.33636      9.92855    -65.69532      0.26362     18.28819    -63.44273    -20.87688    -23.68974     -88.9452   -273.34639   -176.91964   -215.53159   -417.48191    -77.61892   -159.48718   -412.41259   -268.80682   -158.39321    -32.69115   -103.72994    -75.54102     78.36861     67.19392     71.53402    138.23141    129.73152    320.65803    422.48987      456.449    595.16865    629.19991     695.4821    610.47632    352.44319    559.01495    200.55365    128.70562    -22.15596    -76.55633    -53.68316   -174.57751    -57.27746    -77.68341   -104.28229    -96.86948      6.85835   -104.97634    -45.44873    -35.11487    -86.46963     76.58896    -75.77596   -234.93372    -79.45838   -225.57198   -238.00996   -399.54832   -289.48984   -242.94826   -304.24637   -310.11315   -229.79589   -449.93205    -355.8443   -260.73885   -327.96588   -162.68942   -441.92266   -295.62984   -494.42374    -202.3179   -302.69015   -318.10666   -198.96916   -169.56171   -174.97396   -225.30244   -172.57285   -160.02192     -5.62455    -58.17377    -14.54112   -313.22519   -140.93813   -188.59473   -144.00033   -290.17508   -372.67953   -305.70804   -206.08606   -309.02718   -212.27116    -281.7951   -234.33415    -343.5804    -354.8804   -275.38748   -158.38275   -218.53601   -309.72174   -184.61557   -150.54775    -54.75417   -380.22749   -332.19488   -326.03092     -386.241   -290.26442   -393.03983   -250.97719   -399.02376   -396.26291   -504.40399   -470.61546   -437.79966   -363.76412   -219.40005   -399.19832   -237.94448    -81.09887   -150.33408   -157.66027     73.49773    -94.42835    -62.60519   -150.33525   -202.01967     47.36714   -123.31571    -259.1554     -37.7458   -295.98661   -307.61188    -313.1398   -373.79971   -299.44571   -220.90979   -353.90683   -301.95646   -284.66034   -167.76786    -42.35746   -304.94421   -478.93721   -525.33039   -509.80424   -356.56273   -435.12602   -492.53228   -306.86556   -631.31285   -320.37896   -611.67292   -542.38897   -517.16355   -510.10205   -508.41782   -377.40499   -632.41578   -709.45807   -565.06493   -467.06198   -632.01238   -566.14561   -560.44682   -547.90214   -629.57415   -684.67073   -781.99168   -970.36392   -671.54797   -652.10219   -629.63465   -451.28467   -999.85782    -605.8788    -535.5588   -670.55572   -887.58035   -834.41238   -879.93248   -664.21081   -847.75602   -783.86243   -719.86558  -1022.44476   -638.02294  -1015.25359   -888.69084   -592.28826   -850.96338     -683.261  -1038.26267  -1040.89758   -973.84506   -768.89305  -1197.40072  -1238.05166  -1332.77522  -1056.80967  -1006.49033  -1228.52794  -1541.85326  -1252.23424  -1326.38668  -1410.84594  -1406.95864   -1362.8305  -1281.45409   -989.15125  -1235.38214  -1352.37187  -1032.60228  -1675.32344  -1103.80618  -1640.37971  -1349.28324  -1780.37301  -1259.28715  -1800.48389  -1458.09964   -1538.4535   -1545.2636  -1464.76675  -1899.38066  -2016.61165  -1946.68743  -2024.83052  -1867.06348  -1739.72541  -1475.70227  -2188.01362  -1911.95503  -2031.88453  -1508.90558  -2168.33355   -1991.2079  -1705.61226  -2388.23396  -2170.88789  -1852.89158  -2328.21793  -2198.07485  -2549.68061  -1610.23181  -2474.94686  -2652.92625  -3021.87172  -2517.27855  -1729.00958  -2764.81333  -1733.39192          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       20.00   -139.52484   -320.97632   -485.08183   -539.59005   -288.39084   -261.66177   -195.94446   -198.28154   -241.53667   -461.77318     -95.7862   -480.03565   -393.95416   -335.71718    -319.1626    -21.26205    -423.3614   -568.47819   -522.38708   -354.12671   -376.86422   -369.98123   -533.33574   -151.71697   -350.41147    460.71367   1349.24868   1647.36347   1303.45687    217.49262   -318.04894   -197.09985   -392.64463   -211.96014   -197.43065    -361.0187   -364.18252   -235.82606   -281.90755   -294.35601   -387.98783   -427.48013   -352.14563   -254.81261   -370.19499   -475.24985   -245.98957   -158.61462   -308.28568   -374.23477   -486.46412   -362.66148   -265.93155   -377.29236   -219.12018   -189.54086    234.89116    862.82827   1165.66589   1203.49062    352.31297     81.92759   -248.80893   -233.22389   -331.79321   -220.17024   -229.61452   -297.24462    -279.2871    -21.96853    482.53837   1125.98264   1480.67625   1180.77975    624.88301    739.43439    527.30149    297.57865     -3.07548    -39.31692   -351.93661    -347.6956   -203.26126   -313.36561   -437.52899   -263.02146    -323.5564   -283.34896   -159.31775   -162.20353     40.14341    288.74076     66.16896     -57.8099   -174.83607   -208.61376   -239.07014   -389.88962   -245.06995   -237.38501   -186.67445    -19.26862     48.61357    -16.12748    -31.36621   -111.42834   -256.85864   -343.27002   -225.45664   -262.75994   -301.20036   -319.34392   -436.20664   -385.89839   -188.54081    -28.79626      111.145    303.54771    195.96881    143.86871      -84.717   -180.04607    -65.58625     29.61868    316.72822    558.66984    762.26525    548.29465    281.01472    110.57898    -50.21042   -226.56959   -319.52857   -111.85428   -184.76163    -35.98965     133.7365    735.13503   1673.98364   1931.32033   1808.83205   1650.23524   1032.00113    557.26487    257.39802     21.91596    -39.52779   -231.97048   -208.17987   -168.55531   -306.52234   -329.36281   -323.16572   -408.14499    -175.7357   -226.52584   -186.36482   -237.51346   -142.61105   -181.68287   -176.85421   -127.20331   -137.16135   -153.25934    -136.3612     47.32956    121.19451   -119.86077      6.16006    -47.29746   -161.31734   -336.75337   -213.07502    -318.1522   -229.29889   -169.08973   -157.80332   -112.22289   -124.92622   -260.24978    -163.4645   -193.27036   -389.91417   -407.26364   -271.39261   -273.50357   -402.50482   -373.65419   -285.11977   -167.66742    -284.2157   -211.73953   -137.84235     -64.9832    121.99285     24.61194      9.84771     96.43816    157.28454     230.4576    183.38774    202.49506    178.07449     113.6197     25.36905   -133.00626     -11.3935    -52.56677     37.08915     -3.79793    101.79952    116.00842     17.27049     11.98029   -181.58255   -138.70356   -360.87305    -190.0134   -335.63115     -297.942   -370.20062   -225.72272   -258.94869   -209.75552    -82.28826    -75.82878    -13.63761    -13.67723   -155.76217     38.75119    114.10703     68.21075     29.23966     98.25525     15.01116     34.70649    141.59541      -3.7196      5.62233     36.84053     23.51613    -18.41822   -213.21692   -158.63581   -317.08777   -244.65433   -262.30065   -315.76724    -98.11117   -326.07848   -489.20033   -273.16737   -229.84592   -272.09419    -92.76057   -172.27939    -46.69974     43.86211     98.02566    113.25654    188.57024    225.57633     402.2864      473.801    613.72625    568.90355    723.44633    468.13601    451.12328    268.90133    144.80319    115.38225    118.44558     83.69574    -71.13615   -271.82957   -303.59595    -187.4327    -22.51674   -191.17234    -34.61958    -44.35771     44.55993    154.63025    110.08508    -96.79455   -112.23087     -66.5494   -153.05081   -254.03212   -141.91483   -264.98546   -404.72618   -279.76223   -419.12636   -241.33151   -211.57589   -348.12468   -344.93657   -468.33106   -381.75518   -325.09181   -473.88251   -275.84813   -380.52866   -277.51483   -201.37852     -204.268   -330.74691   -276.97624   -127.32153    -85.36875     93.72073     -63.2935    -101.1046    -200.0278   -134.47112   -108.27372   -201.04728   -228.06604   -108.95436   -270.18049   -249.22229   -206.09983   -405.16141    -248.4053    -348.8108    -239.1039   -273.09866    -185.8391   -329.06929   -277.61677   -261.34249   -167.27336   -157.42464   -381.71193   -263.81692   -308.48389   -267.28822   -314.22333   -333.35865   -244.02689   -287.35034   -295.01245   -312.84339   -340.41603   -292.93393   -541.33851   -393.15555   -320.89394   -373.58439   -398.37177   -321.83587   -272.70489   -193.18218   -192.85719   -416.89157   -245.62971   -261.22861   -188.97397   -283.30394   -163.35095   -198.91621   -152.34308   -350.35092     27.51743   -359.12094   -261.09917   -241.83661   -294.50382   -467.27963   -448.27994   -588.16883   -287.35497   -429.42568   -394.98832    -514.3968   -448.37698   -480.95492   -274.05295   -415.89898   -629.20117   -587.44614   -535.79968    -623.6378   -739.59126     -611.566   -528.92225   -672.63992   -716.53395   -722.96143    -594.7346   -771.02413   -661.10946   -856.74831   -893.57464   -608.87789   -991.85436   -873.63326   -917.24424   -904.37534    -917.6518   -843.02996   -902.37204   -798.23795   -750.16518  -1105.62685   -699.42825  -1027.68957  -1062.11378          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan  -1161.45218  -1723.87002  -1911.92323  -1681.39304   -1538.2081  -1669.79182  -1671.34373  -1856.15198  -1885.97981   -1888.9598  -1739.32562  -1511.37608  -1981.97059   -2066.8543  -2139.33311  -1775.17988  -1977.05455  -1833.95177   -1761.9569  -1924.39334   -2614.3609  -2540.59136  -1965.10879  -2295.57409  -1967.68196     -2146.26  -2238.08383  -2988.25057  -2469.45497  -2538.24554  -2887.54901  -1014.62609  -4911.40308  -3218.55076  -3463.71949  -2920.33121          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       20.50   -204.83781   -298.55893   -433.58279   -449.46269   -212.68286   -202.83895   -198.22713   -214.01896   -267.85931   -402.59872   -169.49116   -403.31106   -457.54699    -310.8253   -302.81464   -112.08456   -297.40338   -510.23639   -525.23276   -434.53266   -330.37105   -485.33018   -367.95182   -197.32985   -322.38957    405.88443   1314.82991   1591.69216   1058.24485    239.63043   -217.20031   -332.73746   -419.86035   -346.85801     -9.24819   -408.75694   -315.78308   -386.59915   -430.57105   -212.14944   -311.73966   -411.07175   -403.89665   -447.56088   -395.05914   -434.71563   -341.39329   -727.65459   -333.20485   -453.56267   -351.66035   -330.03684    -298.5442    -423.0018   -463.70616   -109.74434    265.05374    740.78005   1455.38185   1015.03705     553.5431     -0.42894   -160.94325   -411.65763   -315.48591   -291.28322   -246.35636    -234.4272    -158.8793     -9.02626    490.91271   1155.74158   1355.60098   1160.01659    669.55747     750.3868    117.03604     86.41732      66.9175   -212.64455   -455.93619    -321.4195   -357.96642   -226.15372   -169.29559   -214.81583   -272.52556   -349.07032   -406.98913    -14.37828       -5.435    199.77032    145.17798   -100.57632   -147.25143   -312.10099   -207.75373   -282.16213    -295.5166   -349.98289   -173.65645     -9.21292    -54.85777    -71.09849   -150.79919    -291.5516    -305.0707   -200.99164   -425.77873   -316.80788   -355.66564   -328.40772   -332.06646   -181.74119   -266.20209    -18.06616    349.28171    146.61266    333.19963      96.5982    -93.21942    -56.53094   -206.33472    -43.14157    250.88557    456.55964    544.17063    434.60063    478.55177     37.12597   -177.64101   -169.07596   -212.55303   -268.35916   -384.88705   -107.96393    392.09766     958.1116   1525.87884   1957.12966    2067.6783   1568.90181   1114.60172    633.03658    158.90471    -88.09413   -167.95591    -202.2511   -109.26446   -292.96826   -351.04653   -285.70624   -196.33298   -331.98136   -152.16151   -151.24397   -261.87735   -206.32555   -244.49649    -84.42734   -173.55688    -273.7541    -51.42164   -160.36694    -95.09854     46.01081     49.98925   -133.68515    -40.88672   -197.98026   -179.39793     -138.228   -146.66586   -222.15051   -306.79081   -411.62969   -221.89564   -127.65739   -305.50448   -261.99339   -542.27375    -241.4176   -267.62383   -338.81774   -236.05919   -271.66786   -263.42022   -286.35751   -323.14019    -334.8753    -213.4002   -260.99465   -194.17194     -8.89767   -161.20867    -64.97902      16.0503      41.5125    205.16407     137.9264    139.80307    140.02011     54.08303     39.47097   -148.69632    -68.26076     30.19371     57.17864     50.89946    -18.91958     55.04618     -88.1916   -111.10258    -150.9479   -127.44425   -283.41947   -220.94674   -354.51076   -319.86982   -270.54624   -407.88232   -253.33039   -221.95352     -274.922   -265.65291   -141.34808    -32.83326      22.4571    102.12574    290.40149     70.44317     -72.1451     22.28874     78.19888     60.97735    131.92651     40.39889    -68.44257     -14.8821      45.8044     82.86723   -148.30828      59.1062   -400.50169   -146.37031   -278.71368    -400.7957   -446.06534   -338.27035   -299.03327   -181.18759   -272.09899    -198.6461    -27.61571   -131.41583   -104.23883   -104.61027    -38.12118     17.64396    157.05607    123.32684    346.87966    468.85382    562.13529    438.39242    396.91636    414.25166    472.93613    315.67402    373.12921    244.36567    145.62533     -95.8158    -12.50437    -44.59141   -213.01829   -162.53265     -8.44942    -73.03314     -88.8971     93.53198     58.82158    -285.6841     -1.51055    -22.15797     52.12312    -10.38552   -152.65142   -176.21453   -158.90373   -239.77026   -264.66378   -195.88842   -371.60884   -476.01175   -327.59276   -416.40311    -372.4834   -392.60909   -278.87903   -316.30462   -313.39304    -363.8768   -352.84899   -240.21946   -287.00809   -242.16344   -213.25415   -135.54994   -144.71351   -289.67203    -95.79632   -114.88546   -214.09116   -111.53836   -153.94676   -119.50446   -233.15016   -237.43996    -222.7179   -210.27953    -336.3498   -404.95255   -262.52099   -256.98808   -376.29318   -339.42297   -362.55359   -395.47436   -258.10956   -279.22978   -532.75245   -276.21968   -306.78343   -391.10031    -276.7048   -306.08935    -301.4868   -303.21897   -417.17369   -442.70444   -357.63508   -360.55053   -366.69773   -439.72064    -338.3466   -439.75443   -576.79337   -408.49473   -553.56547   -550.46235   -480.00085   -282.77238   -405.98204   -409.24665    -438.6146   -388.19921   -540.85997   -387.11007   -658.29837   -555.88615   -379.81425   -464.70693   -372.15009   -625.04087          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       21.00   -336.71864   -309.29902   -443.99262   -444.86954   -130.45387   -165.97041   -258.99773   -185.60558   -276.42675   -383.26396    -302.5021   -363.31251   -514.35391   -326.47387   -438.16326   -292.19037   -215.40829   -434.79795   -449.74062   -465.92825   -390.18652   -492.65069   -329.03018    -176.9054     -17.1587    336.72631   1261.37687   1493.20684   1052.53952     228.1213    -49.22977    -478.6062   -419.12483   -413.41283   -283.90123   -443.53027   -408.62167    -322.3325   -351.90145   -348.50487   -323.20355   -291.89943    -431.4141   -467.07375   -309.88314   -469.11162    -403.6166   -499.74896    -387.2288   -419.99912   -306.75773   -333.83277   -129.29693    -302.8985    -212.8689    -71.11758    242.50193     1049.944   1215.93997   1154.21185    268.86508   -192.32623   -241.81462   -315.05986    -341.3365   -324.51542   -418.13371   -237.00134   -212.44207    -30.67934    458.40371    937.34539   1554.82628   1233.44475    867.06865    493.70298     483.5638    201.25423    -46.63639   -211.54871   -334.72814   -180.47981   -219.80114   -211.08395    -369.9854   -371.35291    -337.8687   -316.39137   -235.91503     -9.58685    121.77959    214.69368    128.38668    230.90337   -216.90435   -242.20004   -273.43325   -347.76367   -310.10592   -321.69463   -209.90761    -14.38359    124.78762    -45.31204   -201.56934   -314.02124   -281.69838   -413.67977   -420.95981   -410.37978   -266.81126   -337.89804   -422.05744   -302.88684   -210.32661    -95.28727    146.98458    332.04573    179.55241    119.47757    -73.78782   -215.14438   -150.90187    179.85662    233.13658    613.47513    674.44269    461.58078     395.1383     81.29352   -195.93039   -143.18699   -225.73486   -315.47011   -162.63399     42.30696    287.07689    874.45283   1575.44592   2012.20313   1842.41911   1740.86603   1193.90207    458.10009    125.30547    -159.0686    -230.0625   -187.11586   -173.58266   -280.51796   -307.80234    -240.4019   -194.87421   -309.40779   -332.64137   -123.49697    -78.61349   -155.44505    -205.7792    112.61474   -170.35389    -60.32847   -127.41974    -74.38512   -211.24228    -48.65328    -42.29456    -57.40897   -115.15041    -57.19535   -221.16019   -404.84503    -342.6567   -233.44173   -260.27346   -164.06943   -116.31186   -214.97492    184.60368    -244.2707   -244.93013   -154.58919   -253.29554   -396.53571   -281.99128   -411.40463   -307.44099   -226.38041   -341.96647   -372.95092   -292.34558   -245.20388   -183.88231   -169.89444    -56.57831   -113.23941     87.57139    147.84537    120.96546    361.65524    199.72718     75.39278    118.05192     60.12304      3.82769    -62.49539    -82.11977     31.41192     74.36107    119.74113    170.76764    162.29555    -30.85521    -70.75779   -269.88222   -111.08882   -273.05825   -252.46153   -385.55766   -263.25206   -261.87568   -171.29521   -313.53057   -190.25057     -96.1742   -104.26852    -75.93703    -107.4897    -10.97128     67.34801    -14.38446     13.76274     53.64007     95.50054      2.72687      9.53319    187.41439      1.06743    141.36617   -145.30579    -65.06902    -24.74982      4.70637    -74.34925   -192.50061   -181.64919   -205.68209   -374.41575     -283.846   -282.01066   -261.83355   -322.57458   -305.18914   -147.64666   -173.70703   -114.39549   -123.01678     -61.2413    114.19418    143.23925     238.5375    198.22387    412.29553    421.18347    309.46828     644.7307    590.41736    508.10228     450.6572    419.02669    150.80937    202.36291    138.70869     -43.0669      0.37129   -101.21802   -231.02204   -382.32659    -161.6326    -73.58037    -43.45447     34.77342     52.80289   -165.47042     90.12939   -100.29513    -98.13498   -173.39065   -211.29222   -143.56605   -261.98373     -307.248   -228.29925   -445.52298   -351.47666   -347.23841   -455.32047   -320.41635   -333.33326   -397.91904   -430.46538   -287.22328   -179.99714   -306.79664   -298.61753   -363.02767   -291.40988   -284.66461   -357.97719   -191.17687   -136.27936   -193.41727   -371.64714   -194.42769   -203.03124   -218.15265   -153.68582    -339.8169    -257.7333    -268.8954   -283.68663    -316.8722   -347.34053   -396.31586   -226.42888   -353.65469   -458.47952   -469.29259   -501.60367   -441.52674   -383.67805   -406.84105   -414.03178     -652.461   -398.22592   -500.30291   -491.63251   -489.80996   -561.86693   -492.33605   -585.11206          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       21.50    -470.9339   -354.08983   -465.37028   -523.16672    -31.53556   -239.50973   -244.51764    -192.3311   -258.98751   -247.84108   -314.14001   -364.23953   -254.71987   -316.66606   -420.09423   -486.99413   -290.15209   -335.26161   -343.76598    -212.8404   -291.69265   -360.08084   -265.54352   -257.66191     24.44715    253.53574   1259.77166   1492.84552    1201.7375    283.24451    -25.91082   -268.67485   -431.32366   -441.58445   -435.59411    -381.7351   -360.11968   -377.25778   -308.71501   -281.40051   -401.48033   -199.85732   -407.42621   -407.56397   -256.11883   -455.90812   -381.39282   -328.84014   -324.30694   -264.61849   -349.44377   -365.59427   -272.63927    -385.3276   -350.21769   -117.21526    377.96179    899.43526   1254.15693   1027.18819    554.91124    -78.42514   -210.63296   -330.14364   -299.15211   -219.01586   -389.33644   -186.19674   -389.17236    -14.86485    523.78557    837.60692   1244.05379   1235.12176    682.97865    551.26983    618.10503    224.47254     19.78376   -299.79965   -263.87741   -294.77226   -410.66847   -336.02364   -277.63235   -351.48507   -276.41238   -341.27218   -232.01369   -119.52333     64.55207    173.51501    257.40412      93.2658   -217.98625   -444.76295    -377.0684   -319.82204   -221.03175   -221.98498   -227.63423     137.1361     39.72984    -23.66472      66.6636   -225.30667   -326.13976    -431.5355   -406.90922   -264.62126   -286.42865   -320.55235   -213.69266   -320.96214   -138.46742   -133.36704     32.11284    369.13254    243.02541    109.48262    -127.7692   -135.75718   -110.55921    134.39179    354.72971    581.06804    804.59608     618.8228    455.80266     206.8251      8.21164   -185.88392   -131.42156   -249.21773    -266.3857   -139.90829    401.97676    671.88066   1388.14538   1994.22595   1889.09731    1676.0098    943.82703    548.82592     94.77278   -170.15191    -82.51654    -172.4913   -306.61296   -273.53044   -292.14861   -322.39442   -225.44844   -387.30174   -246.98865   -497.58362   -340.38386    -91.79522   -150.76681   -210.92883   -232.97945   -108.18488   -192.94808   -144.47907    -88.30251    -74.91705     40.99956     73.36828   -153.88156   -181.71844   -142.25792   -123.92835   -262.51933   -216.47685   -117.14571    -274.9608    -236.2342   -140.10803   -280.03315   -134.99388   -300.53491   -291.09738   -329.89879   -447.70613   -345.78165   -264.30171   -299.14905   -254.34481   -371.17876   -404.43463   -280.63412   -342.03336   -217.12102    -80.01244      9.90417    -84.90599    -16.06206     97.43874    176.74446     288.7855      87.1848    126.67572     167.3788     44.19535    -41.11897   -109.24584     48.02671     -0.52385     60.39832     16.91826    141.23018    -67.86308   -181.59643     44.48731    -80.69854   -131.17669    -268.6401   -366.26464   -371.97601   -267.59079    -329.3643   -298.90864   -279.29773   -239.06395   -199.60352   -104.17691   -128.79296    -41.95038    -26.40606    -44.76209      99.8386     16.36549     28.85993     32.96469   -108.42712    -12.19662     73.06322     35.31145    125.83322    -92.36329      9.80662      -0.1891   -192.59833   -190.86842   -127.77211   -328.33828   -169.80067   -204.53687   -382.77708   -465.56092   -227.73793   -271.25074   -260.70119    -40.79092   -122.70437     -34.9925      48.0441    -50.87778   -129.84631     79.61201    116.99803    331.77441    502.45363     449.7371     486.9049    619.75974    514.11958    552.56022    371.60478     352.6633    162.05811      98.6617     13.78467    -75.85751   -189.31647   -227.42568    -118.6306   -241.91364   -228.49269    -117.7645    -13.58258   -122.23861   -104.42368    -95.56201    -78.67323    -22.60802    -52.86345   -197.83143   -234.05641   -251.82213   -159.27307   -368.64328   -359.30991   -356.99431   -388.43577   -262.59758   -433.25891   -355.62182   -327.26542   -318.15147   -396.90303   -525.15281   -555.96314   -401.31562   -312.21994    -320.6649   -214.17461   -400.96681     -341.992   -404.58666   -294.72888   -228.80409   -342.09794   -182.49979   -292.59259   -249.25307    -99.12964   -399.36364   -384.37504   -389.63186   -395.32304   -448.96809   -500.47233          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       22.00    -503.2854   -430.96022   -444.49335   -577.05178   -121.30512   -215.27815    -210.8969   -313.48695   -234.41423    -38.19573    -318.8925   -399.00838   -241.87661   -419.43085   -422.69916   -358.08262   -299.96901   -191.68652   -362.50378   -106.21849   -324.17617   -200.34189   -279.59604   -179.22841    -25.80903    462.54671   1304.70293   1533.90799     1243.432    225.02309   -129.24086   -180.16193    -230.4972   -327.67425   -376.57186    -424.4532   -407.77079   -353.89178   -307.52702   -249.35729   -393.47375   -260.63217    -376.9994   -374.50109    -441.2452   -420.67483   -423.10532   -220.79476   -339.84242   -338.10783   -476.67395   -316.05292   -554.78558    -310.2875   -539.09207   -199.91259    374.12402     972.9081   1274.12799    1207.6657    408.48813   -340.88307   -287.62464   -330.96416   -330.93667   -148.57118   -257.92199   -250.05986   -347.14693     77.88049    454.05684    986.24712   1324.55497   1208.40842    909.24888    735.10251    397.08444    128.77734    -74.86263   -205.12458   -385.69523   -202.10118     -16.9407   -271.24602   -211.66287   -213.04495   -460.92239   -260.35443   -359.88273     -25.7855    211.15588    196.91029    128.97704      2.09272     16.66942   -257.33795   -344.53526   -345.73672   -548.33424   -302.93731   -134.11283    -11.96751     91.21505      9.67732    -77.92084   -321.23803   -339.60056   -417.39521   -224.55532   -125.44355   -405.00094   -339.61158    -331.2789    -175.9249   -252.11691   -122.91096    177.09217    305.23389    174.52665    140.48099     15.36229   -154.92291    -68.64126    145.50708    361.48164    608.81573    712.02983    469.95284    337.34093     -0.75454     -48.8394   -205.60638   -204.31729   -221.20771   -158.99342    -58.06533    392.10549    811.56173   1286.01414   1748.97249   1851.57009   1373.93913    887.22452    437.86092    155.73853   -109.82651   -156.62176   -341.99926   -204.86914    -216.4602    -357.9271   -151.18289   -238.46512   -330.30166   -327.74361   -200.12449   -195.82851   -177.57373     55.91821    -87.32113      1.58924    -79.90289   -301.25139     63.30048   -157.34994    -15.77182     19.25936      7.02885    -26.14375     35.07808   -104.75916   -146.42003   -293.60841    -61.61469     -9.37415   -228.32937   -252.12329   -135.24957    -30.79499   -104.03897   -162.56481    -229.1272   -166.71188   -286.45097   -113.67583   -329.21158   -217.60354   -295.93453   -270.84163   -393.16474   -362.61192    -280.9035    -219.1555    -17.33312     -79.7605   -126.51935    -45.63941     65.50207    144.28199    253.01511    251.65503    245.13681    -25.13415    -37.20428   -112.30239    -96.90483   -267.51825    -28.56109     90.41503    147.12315     74.82667    -17.92959     -7.08288   -127.32815   -165.27289   -326.31199   -144.81832   -227.38416   -331.24489   -315.88638     -328.448   -252.37918   -265.18097   -354.87845    -32.09154    -56.83889    -117.6467    -58.19003     74.76203    -45.17638     95.97388    -30.04771     -10.6209    -19.88824     34.87282     10.03713    -19.21631    -37.12693    -56.05697    -35.61301     46.26191   -162.57809   -236.88151    -202.7858   -106.67195   -254.32365   -280.44833   -343.03835   -342.96011   -236.78175    -289.2209   -189.02499   -189.13536   -245.14985   -241.49234   -122.48144    -49.02391    126.51303    -58.63886      0.12265    187.30865    101.77206    256.08423     337.4628    250.41138    509.83911    452.29283     538.5864    401.23488    400.98199    150.74071     -8.61968     73.84001    -77.97738    -64.85405   -177.51142   -269.17993    -82.85763    -97.78967   -218.75625   -113.24053     -1.46799    -76.97707   -207.99608   -179.72419    -72.70985    -53.90094   -172.65656   -240.15963   -477.05112   -453.57015   -275.59766   -264.49438   -323.04822   -517.69079   -421.90845   -394.37928   -314.31703   -437.06117   -510.38891   -337.20079   -322.28196   -471.09165   -335.00369    -365.1008   -502.71633   -434.97628   -446.35829          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       22.50   -490.05054   -414.96879   -388.89986   -653.70045   -234.42348   -306.46737   -253.35831   -418.63751   -244.91416   -200.67978    -383.6369   -407.29782   -418.61541    -461.9998   -388.05701   -109.07615   -306.49769   -169.78573   -245.27184   -125.00486   -333.86335    -57.58954   -342.22842   -208.27872     -7.61277    630.93182   1215.95759   1468.88371   1232.00378    238.03946   -266.14004   -227.85381    -230.3773   -401.08976   -408.37432   -552.31182   -415.62189    -295.2816   -373.72119   -369.24363   -483.49797   -251.26933   -154.07596   -455.32754   -395.77891   -275.65647   -266.41921   -326.23759   -376.39934    -548.8615   -450.73273    -492.0239   -518.69915   -353.57523   -456.19699    -28.43707    109.09842   1129.62812   1133.85811   1062.00419    298.55884    -85.80212   -210.27998   -314.97803   -198.63061   -236.66824   -350.50672   -218.02638     -243.202    109.37289     420.4326   1011.33759   1722.83898   1384.92655     846.0561    591.05063    487.39145    178.46383    -37.87226   -151.51105   -390.33645   -210.45809   -163.70911   -331.03532   -218.80128   -407.09302   -177.79536   -300.17699   -171.63079    -53.87249    100.32877    272.93702     90.97102    -11.31897   -157.14566   -364.06814   -358.85161   -420.42583   -192.81607    -16.74442   -133.19407     116.9532     82.95786     39.28156    -36.90094   -265.21223   -466.48492   -419.36595   -602.02848   -270.51499   -400.54515   -374.43269   -303.22574   -240.69751    -255.9637    -55.20331     56.03135    322.33226    381.45647     28.75681    239.72287   -128.74031   -115.50635     97.30836    302.99854    659.39456    801.32529    775.80589    474.69908    134.98568    -81.74515   -154.32918   -359.26729   -231.15652   -144.04354     -26.4707     434.3609    850.10536   1460.95475   1812.72312   1698.97144   1378.44641    931.39086    217.14369     69.22058     -93.4701     11.08509   -136.41123    -97.01245   -217.62475   -330.31548   -277.01741   -203.83652   -401.97173   -316.82182   -425.15821     -195.386    -175.3405   -132.78979   -143.22321     49.61287   -127.30124   -187.97644    -45.15458    -44.67965    -12.27579    125.54933     86.63601     -94.4056    -51.50897   -184.46754   -384.10235   -488.69673   -337.26423   -142.62662   -224.30263    -93.66099    108.53519   -105.76992   -187.25461   -115.52474   -177.65251   -281.10396   -385.44042   -230.82927   -339.76192   -433.58688   -430.93986   -330.37105    -388.2617    -288.1749   -139.88862    -87.66191    -128.6188    -31.24705    -66.20277     38.77452    125.15434     80.85156     193.2633    199.06933    213.50967    143.71487    -10.71361   -212.77562    -15.40822    -87.83687      15.6374    -59.43912     108.6695     15.11664    -29.34561    -40.07261    -92.67302   -127.99991   -188.50526   -429.90609   -322.81796   -265.01232   -224.81587   -263.70789   -195.58237   -278.32056   -172.90272   -355.74258   -143.76278   -137.76479     -45.4232   -121.06814    -64.94473     31.17317      0.69429     23.59452     65.18211    -84.14777      7.01994     81.68867    -42.18208    -62.19307    100.58331   -185.27809    -59.25168    -47.13714   -211.15961    -320.6478   -231.82314   -403.11881   -301.57314   -347.44862   -396.93293   -330.93601   -260.33565   -267.59139   -168.08233   -133.16634    -37.82376   -164.71184     -23.4579    -71.51546     -8.99109    161.89513     89.07739    137.49445    292.00085    413.42313    361.40278    392.73427    442.41616    566.22344    284.51364    307.12475     82.66778      20.7407     -6.07311   -118.20973    -56.03633   -242.23955   -194.33544    -76.81469   -108.00218   -105.23644   -106.94833   -173.42313    -38.36832   -121.55196    -11.05695    -91.63684   -386.03626   -268.72736   -376.93349   -526.65637   -427.27583   -376.33752   -283.74887   -419.57305   -413.08815          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       23.00   -489.62176   -357.05669    -363.2809   -597.89214   -280.91942   -478.73232    -422.4749   -344.48695   -237.22914   -369.74945   -471.42762    -335.7558   -378.40319   -404.02138   -284.66305   -107.17285   -403.82459   -146.85604     -70.9683   -317.08025   -403.64132    -57.71589   -342.79033   -309.19602    -16.58867    580.18477   1144.01359   1333.02538   1423.78676    258.59878   -157.55948   -252.62258   -354.05299   -534.05735    -329.9587   -539.22515   -291.06652    -474.1317   -326.33291   -443.91196   -484.74727   -294.73468   -260.60813   -285.79951    -453.6102   -484.41343   -225.57864   -184.98661   -265.77293   -293.50196   -299.33965   -261.97663   -302.52142    -193.5051   -395.78886   -207.90949    156.74852    955.89049   1193.69214    950.98863    530.49537     17.19005   -246.87767   -181.72034   -405.77142   -347.10726   -339.36261   -337.21165   -179.50001    -72.39511    556.75477   1319.12711   1479.56557   1124.36235   1019.01086    800.89805    648.94488    166.89899    116.74774   -236.58869   -297.85074   -377.15498   -294.18671   -235.55715   -292.08807   -475.94396   -549.96861   -349.21925   -291.64602    -94.89324    111.13323    274.49657     275.1889    -55.83088    -281.4794   -191.36111   -356.09824   -110.32006   -176.50409   -301.18183   -179.29239   -157.12177    -17.44341   -195.45409   -117.39587   -209.44531   -219.39186   -259.64313   -493.00144    -374.5033   -306.07317   -261.56601   -310.74505   -167.83274     -236.015   -108.71191     206.9082    227.54844    220.48165      75.1277   -122.07813    -237.0213   -260.45712    -15.49431    320.46703    393.48165    614.76273    587.77158    363.96593     -9.48405   -139.95348   -234.25701   -405.20181   -271.70034    -160.5433     40.32743    300.90286    778.56765   1424.84203   1759.64419    1878.9617   1410.59917    974.39448    437.37607    184.24775    -52.49439   -130.01095   -236.19309   -230.41796   -108.00592   -342.59266   -285.36055   -286.53461   -382.99368   -359.14216   -164.03103   -226.64064   -224.87711     10.33755     -25.0548    -317.5748   -197.01209   -133.92296   -240.18863     12.39998    -152.8084    117.19112     27.19071     50.32116   -128.06755   -227.54837   -457.43162   -286.98346   -233.22135   -133.51118   -205.77192   -263.20369   -160.49677   -202.42798     -5.03168   -198.43896   -351.48587   -304.10932    -267.3886   -368.58773   -337.06112   -314.99308   -496.30648   -328.22159   -154.08134   -221.63978   -197.87372   -145.41589      8.75397     -4.45084     -1.03636      0.81858    -23.18903    221.94134     72.74224    138.25613    150.63206    126.80755      6.76825    -10.56444    -86.02733   -223.41957    -36.71921    -84.84699     92.14673    -16.40273     99.83426     56.71851    -51.56473    -31.10903   -211.75847   -255.53719    -282.1556    -193.0049   -366.88712   -384.00418   -306.80904   -181.37928   -134.71019    -214.6042   -113.19122   -191.96641   -156.35951      -1.0368     12.11621     28.59556      7.93689    -17.85337    -27.27513    -97.27455   -186.25304     22.88053     71.01243    -14.20303     84.15803     -56.4177   -122.02522   -115.74108   -226.39063   -283.83263   -218.68516   -177.69672   -388.09319   -312.89684   -393.76313   -323.42623   -313.14725   -256.91172   -207.32146   -213.73746   -146.94424   -253.25863   -351.18895     -0.65232     34.79571    -71.56403     415.6149    292.32952    452.64138    416.42898    226.55976    428.83626    401.57548    427.52155    284.02009    102.46723     10.73068    -85.79933   -137.99686   -179.69355   -337.17403   -361.76795    -258.1524   -229.83057   -215.00733   -204.75065   -168.25194   -248.05473   -325.78987   -295.13359          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       23.50   -522.96843    -254.0591    -321.1383   -406.92187   -331.72697   -601.11764   -585.65463   -312.02499   -204.29739    -451.2246   -479.93909    -262.4365   -205.83959   -145.12455   -277.14244   -225.44177   -436.44266     11.11572   -136.02548   -314.64763   -357.46021   -158.22121   -297.43281   -304.55225     106.5334     630.8835   1060.80641   1401.59795   1201.91624    452.97502    -14.44035    -81.19019   -412.20303   -412.62189    -296.5324   -504.82225   -384.05456   -388.98813   -476.79019    -534.5222   -279.65851    -378.2616   -326.65067   -303.63277   -454.76515   -323.36555   -285.02103   -256.27155   -377.29264   -312.24302   -313.06574   -318.68355   -358.75637   -401.98023   -249.01091   -165.76895    245.81669    844.55248   1261.27626    772.14255     378.2607    -12.71913   -369.02236   -478.80173   -384.12396   -183.78421   -406.24447   -275.43354   -224.94249    -95.91571    557.99226   1124.95711   1455.34561   1093.79055    748.75842    528.58124    240.25992    398.32932     82.85945   -224.92722    -207.4541   -470.60485   -286.17721   -168.94734   -175.57787   -261.87152   -174.76005   -286.11415   -256.17912   -154.82897     34.48848     377.1852    153.07947   -190.02709   -305.50819   -286.73805   -351.66718   -321.77743   -389.64206   -146.53018    -242.1252    -43.27638     -82.1379    -202.0119    -282.9076   -250.63959   -238.39073   -269.45383   -290.24512   -346.91253   -427.63829    -417.6331   -408.43099   -330.00011   -250.86336     36.18873    134.28388    241.85284    177.11654     68.74376    -29.72714     16.58794    -12.47434     -2.56756    260.11965    519.13348    719.99038     501.3011    349.91345    100.85462    -81.99646   -217.75398   -153.91361   -206.79779   -130.00798   -111.59028    346.34533    778.57651    1566.2549   1885.63857   2010.95081   1455.12526    994.45926    439.00813     96.39141   -134.54675   -154.56781   -149.27983   -223.58159   -147.75187   -240.63982   -286.84075   -400.93184   -392.31751   -339.55805   -343.65879   -226.38586   -146.59192   -170.13411    -55.92502   -108.99558   -183.12469    -258.4187    -22.57336   -161.33947     14.83073    -53.81812    -70.44196    -67.95595   -170.19541   -228.62609   -161.71629   -225.31911   -277.74966   -270.35487   -178.50326     27.80176     37.09115   -159.07333    109.82115   -321.26819   -322.72174   -286.06147    -316.7267   -291.24201   -210.16604   -421.66215   -287.52222   -282.43064   -266.95086   -392.21087   -222.59773    -135.4761    -325.2786     -93.1364   -114.39207    -28.72987    -37.33279    135.94865    236.40515     30.27119      51.7974     64.56298     25.41851    -50.56099    -97.90447    -54.91551    -15.22124     94.23621     47.06387    -34.89228    155.88983     12.42725    -78.87702   -234.51735   -239.49843   -155.88277   -355.77239   -294.60098   -255.66717   -269.82356   -240.36571   -255.36563   -252.91905    -328.1807   -184.73238    -46.53711   -111.80702    -19.82359     -35.1108      1.84567    -12.32794    -61.81824    -76.05312     97.63959   -136.86094    -70.83635     49.28806      89.4864     21.47424   -166.75613    -66.96629   -115.64574   -137.74532   -185.35504   -281.14343   -189.43677   -395.61491   -397.89279   -274.99922   -237.49307   -231.93344   -255.99369   -321.02581    -201.7319   -157.49475   -170.87989    -99.81875      2.78928     53.97707     29.07916    111.45473    112.11619    226.53229    215.69134    305.58942    288.09559    263.39156    210.85717    116.18839     44.34145    -95.46393   -139.84014   -249.39442   -269.32023          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       24.00   -516.20076   -208.10773   -298.28617   -207.50141   -418.73315   -576.38785   -410.73768   -465.50363   -232.08763   -531.93776   -434.10667   -564.42732   -168.98998    -49.57623   -327.42615   -395.15775   -346.71402    -90.45013   -202.26666   -220.62782    -422.8225   -207.73562   -412.67789   -380.49396     59.89112     603.1394   1173.97852   1453.97273   1097.51974    411.09345    -364.3872   -186.81516   -476.74036   -218.71306   -196.47202   -377.56745   -267.44034   -485.93331   -444.49861   -368.58023     -506.458   -269.34219   -259.53825   -373.03766   -195.07906    -541.4485   -409.70625   -173.23235   -346.43225   -227.73895   -307.58349   -263.62831   -343.54282   -396.34892   -350.32141     87.16651     272.6394    950.22434   1347.20901    925.91433    465.85919    -220.6545   -361.76828   -235.39517   -361.22757    -459.1656   -274.64021   -430.34689   -158.98417    148.56139    486.57987   1309.04878   1500.21562   1012.45008    732.56278     467.7924    486.00628     96.93421    -71.53615   -108.85026   -132.14073   -336.09393   -248.82868    -98.35098   -389.46545     -310.468   -284.45868   -261.51403   -204.82115   -381.00566    172.52178    171.21119    145.96753     43.18864   -136.09582   -188.12414   -174.00276   -367.12608   -328.65681   -102.59164    -12.38891    -11.51016     -36.8658   -134.30624    -70.75025   -315.80282   -328.49115   -279.03863   -317.28794   -327.14782   -332.55379   -433.82058   -447.85219   -446.75016   -222.35892     -24.3104    114.80088    259.40327    250.04157    144.53273    -77.41601    -14.31919   -129.56337     113.9589    195.38402    467.46299    643.57271    576.90924    316.86063    191.46101   -219.20487   -130.25201   -258.66868    -216.2439   -328.38838     34.39201    141.60153    775.43945   1525.36297   1911.81864   1927.70092   1515.30281   1018.13765    508.20251     97.70226   -138.47111    -97.76833     -177.913   -104.46165   -110.24576   -303.64797   -354.94556   -391.10567   -329.45227   -265.93445   -282.78008   -305.16219   -265.94017   -172.82146    -155.4095   -176.25806     18.37593   -161.60046   -173.26774   -114.02441   -136.24449    130.90696    106.98252    127.06924    -75.58639    -93.62678    -59.86165   -201.91548   -186.11554   -136.86381   -128.43269   -120.53155      7.60802     -126.471      13.0151   -388.02973    -240.0334    -43.22477    -357.3699   -238.54845   -405.03727   -317.37453   -346.42885   -369.63986   -256.32547   -296.31422   -240.00092   -187.11237   -226.18697    -22.18325    -14.92483     66.65109     85.51859    195.06194     83.02243    108.03769    240.72897     17.43276    -65.38756   -139.38736    -126.1493   -189.33545     -1.64369    -84.65418    -53.53013    -34.89889    -10.23134     -2.05466    -62.84462    -84.70452   -239.14259   -165.07404   -362.55966   -313.14449   -243.07458   -223.03318     -344.314   -285.88909   -204.39327   -178.66528   -266.82759    -68.93154   -105.34952   -120.41792    -24.11499    -45.82896     -0.39325    -54.78582     -16.3787    -46.54333      5.80179   -159.69625     98.50005     17.05475     95.29178    -14.26118    135.90478   -126.92819   -263.53644   -163.76171   -120.56241   -287.45186   -584.10603   -400.55134   -343.16553    -364.1412   -267.18424   -251.25498   -240.93596   -152.77549   -155.47959   -219.28269    -69.29953   -126.64068   -148.58514     18.01953   -113.50235    177.68726     30.19723    -24.86578   -180.49379          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       24.50   -541.32071   -229.48567   -366.87575   -247.95835   -443.09113   -437.74713   -317.38269   -560.12185    -347.1641   -472.61972   -394.00178   -767.54158   -247.07666   -138.57795   -366.46113   -425.33434   -291.11258   -331.06634   -328.16036   -257.43984   -319.68156     -233.884   -565.79133   -403.78969   -149.44511     626.5437   1212.24241   1479.12743   1077.62007    369.58731   -195.47338   -235.77526   -497.47287   -372.10771   -487.44697   -346.19124   -366.00608   -380.40158   -616.76161   -361.10271   -521.11308   -337.87172   -317.02067   -348.69912   -302.61111   -311.23965   -600.19478   -280.84782   -305.28966   -206.97914   -297.59895   -214.43244    -241.3327   -375.36824   -353.66999   -179.11177     91.07901   1019.70402   1380.03715     985.3951    476.59935   -137.52588   -387.83312   -157.65609   -309.92965   -338.83581   -347.33265   -208.56993    -248.7726   -105.28329    722.12094   1057.33467   1443.93508   1196.13452    734.16502    447.31388    112.83847    497.63731    -86.19987    -12.86631   -213.34574   -306.49588   -329.58815   -163.81428   -324.43892   -351.57982   -391.84761   -371.65379   -211.10107    -73.03856    290.52534    112.26918    233.85363     53.03752   -177.58522    -299.0723   -390.12202   -180.06401   -160.19226   -261.29728    -26.24201    -92.86804       77.794    -55.99704    -22.52779   -217.63914   -194.51565   -122.57645   -228.50388   -325.10983   -355.84226   -297.51062    -413.3532   -298.18048   -232.59859   -222.46283    231.16841    355.58398     204.3633     10.16643      70.6993   -135.94751   -269.32282     -72.0028    182.69169     633.4966    579.05126    399.46587    337.01123    142.72777     19.05634   -177.12251   -218.97247   -272.92286   -150.74631    -40.18402    149.77375    747.32439   1427.27499   1826.50947   2183.31933   1559.36523   1058.53928    478.61048    143.74164    -74.75088     -5.38086    -83.16301   -164.98828   -240.34991   -252.57267   -299.36963   -337.79241   -322.39292   -264.70754   -258.07516   -296.58343   -272.68232   -230.58875   -117.32085   -152.44123   -156.60511   -128.57998   -123.47246    -92.51598     -73.1294     20.55472     22.89547    -14.46721   -155.86905   -252.74119     -9.67935   -145.00201   -356.24656   -172.69103   -198.76476    -201.4633   -268.67807    -76.75965   -283.72159   -174.19167    -274.5295    -211.7076   -451.89032   -366.30395    -350.3615   -336.67453   -288.75142    -291.4059   -284.28236   -336.11331   -221.86892   -109.89152    -165.1283     -5.19213    -86.57791    -19.49136    153.01629    -47.89209    145.76637    157.29305     89.93667    139.14517    -47.69711     91.67134    -42.13698     69.70466     20.73392     -33.5384    157.99316     74.25541     17.34484      60.9517    -57.74723    -98.84676    -64.02445   -257.95049   -465.21815   -390.66232   -357.14835   -340.73975   -266.70673   -223.32971   -228.58576   -238.58893   -144.06569   -154.45305    -92.34314     -4.28679    -57.94216     35.17701   -125.28733   -148.44886    -75.55066    -15.25279     -73.5153    -11.19702    -34.10334      72.2421    -37.66957    -64.86414     21.75222   -177.38382   -245.49978   -175.03456   -481.35285   -265.01636     -292.015   -336.49721   -205.77798   -370.12999   -189.00063   -284.79136   -330.07374   -372.54679   -368.81851   -259.10759          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       25.00     -451.832   -318.01998   -496.14478    -270.2146   -326.85503   -243.43838   -313.20395    -432.6053   -490.60661   -346.90443   -370.79295   -452.52242   -325.41498   -172.69453   -386.76881   -382.30091   -385.16772   -341.06528   -325.58344   -327.96418   -228.17042   -202.48292      -305.29   -241.84645    -93.73671     606.8091   1042.84612   1302.51732   1133.21042    409.77122   -236.62994   -266.35168   -351.06206   -489.34846   -351.22556   -183.40232   -253.28305   -285.98183   -348.68115   -269.78966   -485.54042   -324.02952   -447.10906   -379.77962   -342.29253   -480.46535   -353.33214     -285.021   -513.60139   -198.19688   -379.72509   -240.94554   -165.40522   -373.54011   -180.70002   -132.90038    308.29719    839.67983   1137.10529    794.83883    612.71696    -71.33348   -394.18256   -214.36098   -380.96756   -350.16651   -218.39245   -182.95051   -123.91381   -129.96721    457.71579   1216.38758   1560.74482   1088.46347    798.28407    479.02436     451.4792     83.85155     37.72925   -255.64626   -331.34461   -265.33697   -288.74113   -177.05845   -386.73527   -299.73392   -201.67554   -256.71456   -219.53813    -56.90822     182.9147    327.89051    157.09829   -224.05969   -132.64717     -193.937   -347.38896   -299.56323   -210.46822   -192.41785     21.38005     -7.24641    -12.71844   -178.94605   -124.35653     -316.983    -235.6399   -401.58708   -421.11199    -382.2269   -323.87221   -245.72584   -331.05292   -481.85594   -301.11075     75.95961    210.62406    193.89329    318.63445     66.90362    -21.70604    -102.0056     -14.0645     27.61536    236.00132    683.35932    342.87435    521.66081    429.21906    187.85587    -84.13296   -249.62999    -97.62307     48.71761   -295.10548    -58.61228    393.14769    691.01259   1445.25805   1820.15748   1858.53157   1522.32953   1031.96075    486.57493    149.75995    -42.74865     10.48474   -303.61552   -177.03092   -254.31074   -234.73389   -290.27616   -296.70728   -390.70808   -454.56877   -289.32289    -219.6239   -230.33842   -129.11957   -214.69919    -70.12255    -167.0233   -171.66965   -109.50234     11.55617    -35.74224     28.64592     66.39038      6.68666    -53.52766    -57.78868   -179.00477   -310.15814    -180.9354   -197.22895   -271.33753   -247.19061   -120.86173   -210.49753   -274.01777   -126.65534   -234.28153   -248.07619   -376.67462   -282.05721   -347.42129   -296.05805   -269.99027   -283.75703   -282.09898   -269.91291   -236.00429    -32.91345   -103.85316     -5.42933    -70.33764    157.69189     -50.5369     20.57035    274.20207    229.89484    176.21121     -8.31011   -197.90388    -74.73033    -22.50315     63.73144     59.94423     81.35608    162.37024     11.99118     65.52967    -50.35241    -36.03124   -442.40449   -229.97373    -31.54118   -291.88369   -425.78079   -520.27748   -343.46452   -320.26438   -240.16222    -202.6888   -290.07149    -290.5429   -246.35191   -136.02986    -25.31857    -92.66908    -80.27954    108.04558    -37.86394   -180.25292    -83.08074    -31.51091   -105.71078   -128.38429      35.2644   -149.98619   -170.43345   -232.36897    -26.58803   -287.18506   -202.75038   -355.66552   -229.96847   -399.34554   -533.63341   -456.53653          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       25.50   -208.48811    -366.7266   -496.06621   -301.96733   -201.65261   -133.37656   -296.30278   -253.79308   -306.41733   -326.74773    -409.8205   -220.28853   -299.67945   -468.86961   -509.02919   -413.29357   -395.15158   -287.43196    -80.78718   -289.72021   -388.92968   -315.95176   -237.02917   -250.23358     -70.4961    504.89044   1238.99219    1305.7939   1124.76868    391.84088   -264.88243   -344.79004   -180.82378   -366.04624   -321.97646   -265.68189   -264.39161   -248.59267   -286.65139   -513.42884   -162.27415   -124.50347   -352.32066   -307.40804   -442.52953    -606.4572   -368.42441   -272.35998   -297.59089   -339.37024   -489.67062   -287.88669   -638.10721    -205.2713   -287.35536   -152.22382    349.53219   1043.16351   1041.43704    878.15289    263.58912    -50.36331   -185.34241   -324.32717   -278.13147   -319.52236    -334.9279   -302.19227    -374.4693     -64.7377    514.26512   1045.69758   1421.25547   1073.38517    760.28023    738.07043    251.67533   -110.17713   -242.57647   -137.04717   -162.55409   -192.79834    -291.0343     -186.089   -264.54471   -269.72796   -351.28225   -439.10741    -44.35769   -299.87284    152.12329    328.16108     180.4843   -112.80035   -302.04665    -94.98582    -321.4493   -349.32974   -254.60622   -257.89334   -160.07194    -28.44366   -152.47834    -15.38113    -99.33705   -373.96901   -278.96629   -344.59281   -441.74645   -351.57199   -283.66079   -385.68833   -250.93348   -215.06438   -179.08268    -33.13285     70.91653    114.38272    195.10605     79.71619    -47.38237   -166.79646     -114.955     25.36579    295.31909       474.14    533.19977    427.66139    466.00015     73.63898     12.58662   -157.93253   -217.74283   -214.51349    -62.24219   -108.34538     298.7736    893.20012   1251.19931    1825.4756   1720.13712   1630.35732    961.17337    484.98415    153.86146   -179.77618   -210.09819   -123.43549   -157.10222   -263.05583   -276.70998   -367.86444   -422.04869   -304.98746   -251.88291   -247.78115   -265.58752   -129.06796   -120.40104   -181.22021   -141.72641   -219.80623   -123.37846    -85.78527      0.12378     -18.3836     10.27613    -103.3074     27.87651    -59.69467   -208.16885   -237.71461   -229.64555   -415.34856    -95.29849     -126.244   -180.06872    -57.69286   -130.29027   -154.94853    -70.45458   -141.01134   -307.64606   -403.92907   -315.31497    -227.7409   -275.52416   -309.92786    -300.0027   -277.03382   -261.69661   -273.78157   -268.41993    -69.08483     24.81471   -128.68981     15.55178     32.24625    118.00351    228.64439     91.57899    167.19833     -7.26384      -3.1977    -24.51784    -79.64651     49.21255     17.00482     191.6238    -77.08784     38.09572    -72.10974   -183.69191    106.35314   -158.57821   -302.08722   -437.87088   -439.60665   -158.87544   -512.72123   -334.65477   -284.36247   -227.54034   -396.64803   -168.40959   -271.39931   -117.65713   -131.20351    -73.18214    -60.20356    -38.66283     97.18219     -62.2627    -46.14519   -161.26163    -70.88761   -117.85083    -99.94461   -264.64944   -202.66226     -88.7072   -321.01927   -511.14769          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       26.00     15.77075   -394.33756   -284.75289   -331.53943   -200.24993   -195.71383   -244.49057    -229.9747     -62.4232   -340.75343    -373.8484   -229.86121   -316.66658    -523.2747   -485.72464   -512.38138   -321.41079   -280.98614   -126.13688   -193.87721   -460.21702   -305.85941   -213.78509    -354.7024     51.85018    602.42763   1244.12638     1295.725   1027.65941    335.92357   -188.12302   -233.02913   -294.53818   -270.87149   -349.46227   -347.61614   -266.86528   -325.34259   -254.07947   -401.94641   -331.09766   -241.05864   -385.21431   -354.43003   -360.80534   -367.51747   -351.84962    -494.6803   -312.74352    -302.5242   -229.85727   -309.92309   -506.41713   -175.19542   -157.79896    -58.92459     308.9558    977.60235   1331.95677    675.53408    315.78601    -63.48305   -310.05172   -396.71219   -336.11546   -326.24143   -228.99871   -206.47142   -269.89482    -50.30403    519.49524   1225.06672   1392.47934   1232.19965    845.01309    520.44529    431.04672    157.27697    -144.0896   -315.58397   -236.89712   -218.49149   -331.40671   -417.62209   -288.27675   -459.35396   -272.93284   -220.73895    -79.74236    139.82564    221.50793    295.70871      2.12694    -16.87417   -121.57544   -327.66768   -475.91588   -252.78571   -205.96028    -285.3048    -95.57882    -19.78172    -49.12913     29.24089   -293.16844   -158.07575   -110.44769   -356.59666   -338.71608   -437.08756   -357.80382   -258.17959   -383.59312   -277.14712   -215.07942   -107.71357    185.00763    119.41366    174.30673      40.0778     29.28766   -142.35135   -132.65658     25.14086    378.80009    397.91785    415.12605    555.45262     235.7335      86.8304    -57.77207   -119.31732   -229.39594   -136.07754   -449.66241     15.19674    319.96584    751.62198   1301.98788   1735.95696   1914.92195   1634.22114   1177.14648    606.94706      93.5509     52.74617   -275.13692    -97.55607   -198.40117   -223.50884   -246.25027   -301.23978   -207.40721   -162.71336   -241.54241   -177.51397   -293.14263    -98.52876   -137.47211    -94.60201     -24.0658   -156.79538   -259.27603   -158.23228   -165.67725     81.58301   -108.26595     -36.0883     13.17494    -44.47411     25.26733   -249.61101   -222.84772   -444.18814   -287.39365   -251.50982    -79.06201   -150.56597   -112.05704     -24.3203    -58.50878   -223.38928   -353.15909    -104.7226   -333.45958   -358.84743   -264.35723   -262.72806   -385.04443   -252.81015   -414.37061   -175.27711   -181.46423    -19.40205   -266.66832     -173.314     59.84799    119.13649     17.20778     50.41708     245.1061     73.66084      5.63797    -28.79728    -90.57306   -108.96922   -112.10647    -30.73752    -43.60619     68.18322     70.82109    -75.69272    -67.87125   -197.73304    -28.72507   -269.44738   -368.17364   -282.20917   -287.33885   -162.94388   -343.28194   -352.22968   -266.01158   -382.47258   -421.65337    -161.1728   -224.99459    -260.4968   -125.65183   -157.71219   -105.48884    -75.03853    -15.63263   -263.03809   -193.00754   -314.04893          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       26.50     126.2402   -359.62829   -126.40085   -371.81211   -235.49413   -338.49079    -289.8439   -319.64055     -4.95507   -344.44613   -364.06953   -251.66673   -364.24652   -403.99874   -260.46863   -500.81182   -287.63736   -215.47176   -280.71997    -68.55996   -363.86561     -76.8981   -155.29847   -360.27583    -13.17164    588.27214   1236.18906   1330.96572    966.81666    150.16829    -232.6183    -90.66663    -298.8157   -233.06252   -452.28041   -372.14599   -296.29826   -535.73166    -375.7003   -370.29754   -524.73828   -358.60659   -340.11733   -391.29997   -433.82007   -335.40999   -299.38941   -282.56041   -366.61954   -255.27483   -411.55848   -333.11892   -277.70082   -415.01056   -195.07858   -173.30058    248.23608    738.05074   1196.43315    835.36603    417.27422    -52.50184   -437.77465   -245.92429   -316.29437   -123.35598   -322.61532   -271.42141   -172.99905     49.96134    721.71317   1094.40623   1249.51449   1042.05859     662.1262    477.66995    452.90981    214.24155     68.84362   -119.29254   -374.99205   -359.32179   -304.83545   -484.58485    -383.5043    -366.6495   -309.49852   -376.38082   -224.56389    -61.09953    138.49305    380.60101    106.44629     -4.54876   -181.06702   -248.15938   -299.40291   -348.00209   -307.90641   -153.16537   -231.54138    -54.44371     40.15408    -10.94162    -136.7836   -214.01252    -342.3336   -250.34418   -236.51379   -346.32696   -347.71073   -349.09318   -443.35242   -242.75687   -261.95846   -125.39516     -0.41154    214.92333      241.306    100.44108   -132.32483   -104.00244   -183.71516     48.67954     96.96434    441.43915    626.88101    540.54604    353.31941    -33.56566   -108.17354   -239.99046   -326.33972   -234.80184   -233.34559     36.39717    243.21348    778.80106   1209.76155   1607.57854   1814.54637   1458.20046   1174.78131    491.20877    145.98744      1.76551   -151.69555   -192.53266     -61.8182    -65.37194   -216.75898   -206.98295   -454.48711   -337.00831   -258.27387   -314.36626   -194.82142   -157.95891   -204.05849   -110.28339   -316.24249    -49.40939    -93.43875   -171.93818   -210.04797     108.6009    -90.11376    155.86729     66.80882   -121.95545   -225.09805    -286.7984   -287.51136   -228.89548   -222.59092   -318.75037   -263.62567   -237.25825   -182.11341    -171.4812   -254.75313   -165.12164     -208.636   -182.09349   -363.74767   -254.51473   -365.90722   -517.46094    -291.6828   -219.37089   -377.39617   -205.61369   -317.75817    -134.8364    -49.05442   -152.42727     -52.8878     12.13482    102.09649    125.06892    169.80201    103.72267    -20.95071     92.96732    -78.43649     -6.92528    -82.62321   -118.81291    -91.03535    -81.46949     27.05544    -93.90762    -98.11474   -180.80065   -278.39057   -283.57038   -188.43648   -293.58265    -367.8711   -316.65397     -497.579   -396.03834   -329.96172   -372.43497   -411.87348   -271.36662    -166.4207    -246.4804   -421.98332   -239.72879          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       27.00    -28.65061   -316.53076   -190.19515   -383.72302   -251.83452   -418.89288   -357.70792   -383.23128     68.35076   -229.60021   -447.65291   -115.60833   -325.63617    -446.6034   -109.31411   -430.69614   -155.67195   -137.06262   -297.73652   -411.26811   -320.68049   -175.88983   -194.90607   -216.46749    -72.90842    524.42906   1093.94247     1355.385   1079.53698    215.34837   -171.56878    -199.2487   -222.75037   -196.03218   -417.79132   -310.76608   -414.37331    -272.0522   -164.85234   -386.01427   -470.97752   -367.64545   -333.07924   -375.10306   -351.21317   -470.79038   -365.49973   -445.51381   -412.30254   -229.56024   -255.41529   -343.32626    -343.3478   -320.11176   -476.81767   -151.44243    230.97958    858.81855   1014.64459    918.94681    300.99101      -58.834   -342.32709   -493.93514   -365.12089   -258.25263   -394.80905   -310.30999    -57.75794    111.68224     672.4786    1125.2883   1416.78523   1028.84801    372.16668    537.71228    498.65698     342.1131     38.07137     35.52983   -139.56084   -494.03324   -368.83554   -511.76494   -265.50784   -296.01514   -134.70248   -296.33795   -193.06728     -176.362    -18.70489    216.85289    207.70969    -59.03427   -286.75955   -269.12243   -546.68313   -377.51041   -186.92506     -193.371     10.75286    -62.65732     14.24761   -262.74008    -69.62219   -223.94692   -309.94023   -242.91044   -233.74697   -309.07597     -369.797   -277.52321   -371.58967   -322.74982   -168.73179     -41.3708    193.75739    195.78211    247.43076    210.92671   -198.00536    -81.84419   -137.92018     64.97641     240.9971     399.3278    446.14929    637.13937    405.96069     55.26044    -95.76117      -96.335   -104.01472   -156.20626    -332.6892    -99.50247    129.81098    757.93085    1296.0567   1596.80855   1862.47408   1842.60676    904.98203    541.78292    297.37934   -125.84144    -31.45782   -200.24914   -260.11448   -253.69945   -331.03843   -282.37194   -290.22547   -232.75842   -285.41337   -391.85862    -323.4144    -238.6036    -76.10611    -187.3622   -103.79156   -156.21786   -164.12087   -249.44363   -173.48718    -57.39852      0.92028     51.01871     -28.5015    -21.00017     -229.016   -181.89443   -382.23381   -202.22933   -286.16596   -270.87772   -271.16112    -78.62779   -316.67948    -117.3121   -100.49562    -237.0273   -272.24861   -261.01421   -264.78382   -270.54143   -310.49213   -236.12723   -343.51147   -337.80442   -460.23045   -336.80658   -187.42135   -129.62706   -105.94739    -113.1017   -137.02226    123.65721     10.15771    176.99366    140.21608      202.451    111.21364   -142.32181    -98.51841   -257.90957   -104.21197   -179.12463     26.95617    -62.51335    -27.14554    -78.76742   -142.47357    -98.67997   -295.39687    -240.6331   -369.24828   -442.95683   -364.78599   -409.65366   -445.78203   -301.88144   -464.13677   -321.12505          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       27.50   -138.85407   -308.40116   -350.65418   -344.41536   -142.23772   -302.31265   -438.10493   -350.96599    114.97962   -192.40792   -410.62265     47.71662   -353.72397   -385.13389   -135.61575   -434.74584    -98.02337   -244.34173    -378.0846   -485.38884   -240.79997   -344.63456   -249.77633    -46.10832    -17.04814    534.92506    894.56352   1440.26413   1046.52235    282.52646    -79.79901   -200.51579   -195.03845   -434.58749   -285.70341   -256.55681   -409.39894   -303.29966   -293.23999    -440.5323   -371.27704   -364.28542   -374.27537    -430.9869   -366.86475   -545.94982   -427.51676   -253.35833   -389.28357   -383.37155    -358.4123   -517.35805   -327.22755   -420.02607   -439.46855   -114.67926     209.9335    990.43796   1076.39168    847.16576    454.24554      35.2197   -354.92985   -376.74533   -369.27144   -445.08394   -204.02489   -307.17771   -294.04082      61.6006    479.58973   1080.08568   1411.76081   1108.28465    645.57811    548.96068    415.54502    173.64052     54.22851     -0.73336    -353.9382    -497.3179    -328.3928   -346.14086   -289.92653   -338.44869   -434.49045   -239.35042   -354.69601    -62.60457    202.40656    222.27743     -30.8974    -62.17644   -228.18313   -371.93337   -236.03612   -360.78942    -308.8426   -134.35588   -302.06422     89.22921      2.75371     50.27089   -191.47919   -306.85627   -395.65786   -271.85615   -420.46676   -301.30043   -317.68251   -338.94388   -312.07171   -429.35913   -225.08929   -164.49807    241.15753     179.0334    135.42302      68.3707       22.744   -109.70565   -100.99838     37.19901    247.50908    379.12628    516.81155    595.34536    388.37882      98.6799   -121.79988   -196.43049   -204.31456   -151.18947   -203.56271     -9.80619    234.65823     573.4459   1406.66739   1715.25312   1812.10829   1504.71586     960.7443    593.06741     97.92171    -56.09704   -112.34347   -140.64325   -138.39352   -121.52231   -377.79799    -280.4256   -407.89003   -265.72379   -273.19636   -283.81176   -247.50498   -258.80311   -218.26692    -82.06362    -25.92384   -206.89318   -120.69423    -73.95703    -84.18949    -26.67276     55.50068     -3.07766   -102.87834    -34.02717   -163.05934   -139.70322   -126.64892   -306.09521   -178.38575   -185.43225   -181.30751   -104.69754   -199.55378    -122.0786   -170.12763   -283.42422   -330.16937   -260.59174   -461.39134   -303.03959   -366.36855   -393.48596   -357.32376   -402.27211   -264.78576   -201.54291   -285.46484   -209.79646     -86.6137    -14.95446     11.90109     58.60583      6.85245     15.75906    -28.79531     113.9049    -88.26335     74.48472   -106.67702   -237.58467   -230.61526    -187.1404     40.19692    147.90087    -35.09707     44.97724   -152.66724   -217.70967   -322.29696   -381.67463   -380.99081   -388.34454    -512.1082          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       28.00    -163.3232   -336.30291   -520.39798    -295.8734   -132.62089    -217.2459   -515.10378    -230.7458   -341.47523   -280.04139   -393.47207    -96.77213   -397.10202   -319.10997   -260.25681   -411.51104    -93.56909   -405.58661   -479.41343   -485.50636   -222.39449   -414.55249   -317.39836   -233.54304    -38.47365    376.24313    940.61006   1384.48581    879.93341      327.113    -55.83383   -281.58581   -414.16132   -519.84399    -246.4496    -373.0188   -511.27856   -341.78495   -439.32568   -486.42077   -482.18953   -320.87602   -252.92731   -339.14124   -299.86528   -332.97224   -421.17193   -320.20364    -127.4587   -466.11928    -563.3175   -531.87043   -507.31033   -386.63641   -262.67629     12.58272    259.56624   1160.47435    1252.8726   1013.17661    399.92414   -239.88985   -315.08044   -355.96006   -370.82957   -363.39497   -310.27877   -283.72334   -174.55994    109.10636    382.16959    1308.0286   1393.12269   1184.03577     456.9017    576.29097    350.97894    335.93728     -5.75619    -36.03626   -259.43545   -394.72009    -58.29337   -386.45787    -245.6336   -452.91958   -230.03211   -296.96238     -54.8423   -109.56935     147.2708    242.40446     91.01164     37.99119    -225.5837   -468.07908   -434.62905   -371.73838   -363.58794   -266.19634    -337.5465    -100.1082    -30.83405    -36.78469   -190.00667    -429.1559   -311.03127   -221.51731   -260.87025   -365.81789    -289.0941   -280.32063   -329.55426   -302.32553    -274.4688   -181.41345     93.88206    312.30471    190.12617    111.47065    -20.28892   -138.77442    -278.1066   -110.13357    240.48495    431.52599    574.23475    503.27656     263.0633    144.62067   -106.87731   -246.43746    -95.22854    -83.35392   -125.49111      2.05873      246.486    731.25197   1310.58129   1726.64003   1940.22635   1693.44029    884.25092     485.8544    232.48826     70.91541     -26.0437   -136.10473   -292.73527   -293.52681   -214.36756   -378.51949   -346.08067   -292.24037   -281.87065   -250.10435   -117.49938   -201.49578   -120.70123   -223.36354    -89.87932   -231.44416   -179.79668   -114.19855   -145.54083    -61.97631     48.22401    158.70871     10.76093    -15.30156    -113.5531     -3.72161   -213.60003   -217.26948   -287.91553   -519.17117    -295.6915   -164.40704    -72.85449    -16.50776   -143.83634   -112.02083   -251.02263   -441.38145   -387.31778   -297.43316   -315.12608    -279.3055    -207.8363   -354.85585   -257.29477   -311.43989   -214.35543   -126.66333   -168.60252    -22.30759    -12.71817    -64.91185     -92.4973    220.53852    115.94958    165.30861     14.87683    -52.17905   -189.32365   -143.98589   -117.77611   -168.13831    -14.40575   -168.20579    -16.04008   -114.40979   -184.72591   -277.69849          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       28.50   -233.82218   -455.89159   -630.77868   -299.38542   -305.03435    -188.0306   -534.26963   -192.59755   -619.03118   -361.99477   -431.94778   -323.95102   -257.78669   -415.43726   -448.73628   -371.96244   -185.54957   -496.07472   -466.21155   -429.12429    -258.1933   -307.15339   -319.31887   -183.84893      8.20716    370.10863    1098.9542   1517.08259     862.7107    228.39844   -148.57876   -294.18058   -479.73724   -232.44976   -377.99325   -138.99999   -578.23319   -359.75927   -408.84924   -542.37378   -414.74125   -466.38728   -422.32774   -367.21422   -298.72972   -400.01026   -346.89664   -374.76576   -431.66017   -319.21118   -365.56089   -413.47078   -361.57096    -388.4584   -236.94857   -152.63264    292.94721    873.07901   1304.77724   1019.78464    398.51608      4.49439   -357.94237   -118.24922   -309.46165   -298.72189   -387.45098   -343.71887   -369.52346     21.37254    570.73508   1048.86901   1430.44491   1067.75714    754.31032    430.69396    320.81833    329.96456    -14.87561   -267.92445   -240.17734   -401.19934   -372.76724   -112.71935   -398.50486   -525.77816   -420.03891   -569.41595    -125.0251   -155.81035     76.78487    300.65597    258.69509     34.08719    -25.57551   -297.93182   -401.44645   -396.73719   -448.99547   -498.28096   -295.75672     38.07634    -68.21748    -37.54465   -155.46367   -253.05733    -273.9353    -47.21168   -270.42155   -293.36588   -362.31303      -195.71   -322.78244   -321.98242   -257.86867    -45.94456     23.66021    212.96314    436.06906     91.06638     89.71022     -189.356    -49.85526     13.33826    373.22307     358.9621    730.02132     550.0061    407.29237    113.11614     65.41199   -178.48921   -162.10716   -292.53342   -237.14371     45.34058    223.84387    808.45993   1264.86888   1794.19041   2219.64098   1464.61186   1105.15227    526.38062     283.6826    -35.90253    -44.25576    -52.64273   -167.51247    -90.86524   -474.99847   -266.59679   -312.02069    -173.1133   -360.35418   -374.46677   -385.90086   -273.69678   -249.46275   -130.07731   -170.96105    -83.48316    -54.41579     -71.1227   -200.44684   -103.48868     13.67157     20.00477     14.72902     -1.73732   -110.19349   -232.44752   -159.50196   -275.55134   -234.52444   -131.68034    -353.7502   -173.56289   -155.56014    -152.8467   -217.50491    -350.2937    -240.4598   -281.66132   -357.93998   -338.50813   -485.74843   -397.31054   -342.09173   -354.02756   -327.62637   -292.00471   -205.96362   -185.80923   -161.97816    -80.22057    -43.03328     98.28244       1.0982     17.85801    106.27014     82.36071    -15.37132   -159.11408     -47.1802    -76.80647   -215.74073   -246.79649   -219.57927          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       29.00   -402.96833   -569.09362   -662.78306   -309.22726   -489.14642     -194.707   -307.22784   -218.55024   -616.29601    -218.8711   -376.80019   -355.81911   -158.70821   -525.73439   -428.92556    -328.1354   -398.25438   -516.59349    -412.5218   -289.35109   -337.69939   -487.73545   -286.53066   -217.20321    -57.45386     620.0777    955.75972   1372.15766    944.85502    215.44811    -67.96171   -251.74179   -533.89317   -402.66376   -396.84372   -205.91065   -486.54946   -285.12212   -267.35995    -612.0888   -348.66207    -442.7844   -369.18551   -339.87068   -469.80602    -321.4412   -409.05935    -300.8753   -420.87255   -235.66097   -197.46546    -381.4134   -284.72669   -395.73935   -278.72903    -92.82319    323.13543    808.92899   1263.41818    858.60074    386.16469   -110.03449   -336.26753   -357.46615   -197.70989   -418.55614   -270.59229   -281.65297   -185.92543    -28.44875    378.61461   1110.42999   1500.40406   1260.99729     772.2328    476.53411    351.01763    322.47469     -57.7996   -471.68351   -430.77507   -428.46303   -204.87343   -108.80258   -185.27203   -352.60432   -353.48766   -273.33268   -197.30811     38.69491    126.54143    100.99347    235.40721    219.78862    -62.78733   -325.76454   -564.40926   -506.92567   -505.76478   -397.69156   -163.87402   -202.35432    -72.23556   -132.81481   -240.61815   -381.98834   -262.16642   -372.85631   -484.06946   -422.92478     -256.006   -374.97585   -270.12946   -151.78522   -132.96446      6.55642    225.49963    199.65217     338.7555    102.87534    131.20583   -173.72733    -91.67064       65.866    313.60212    559.18914    536.58715    506.97515     361.2008     103.6112   -165.77755   -124.76901   -265.93906   -216.70937   -171.22337    -41.36019    270.67805     755.1668   1450.34249   1728.15969   1717.93222   1686.26266   1097.42342     638.4135    350.64406    -59.80517   -377.79567   -230.69173   -371.42529   -324.69635   -333.82463   -384.90587    -191.4279    -317.4829   -278.68689    -280.1314   -205.87537     -179.894   -161.59456    -107.9417   -219.85112     -2.50962    -149.6164    -48.01695     -151.741     21.82448    -43.04571    150.83017   -131.79401   -101.24142    -90.58096   -245.13355   -307.02735   -404.67268   -162.64374   -234.28873    -22.10909     89.65843   -112.40377   -172.72454   -208.55032   -297.07211   -320.87874   -289.43292    -300.5071    -382.1303   -340.22934   -336.56063    -419.0068   -309.38763   -345.93757   -242.90496   -172.95283   -125.43602   -156.74483    -34.39128   -126.77653   -156.19288     64.73403     -0.66675     17.35475    -65.75748   -101.79571   -333.14935          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       29.50   -672.43387   -605.53833    -642.2988   -284.97566   -544.84458   -218.23044   -280.29856   -187.02065   -684.23025    -24.11669   -435.22422   -362.82461   -225.06601   -494.67843   -389.30808   -188.59826   -510.43309   -334.50761   -300.79499   -158.77385   -188.11646    -408.5507   -294.77876   -255.02228     77.55323    539.30171    1030.4621   1282.28467   1024.45064    297.97284     -121.893   -360.68175   -575.02923   -535.97528   -362.39482   -307.98413   -423.45109   -365.66202   -313.46545   -506.62839   -353.47444   -326.89137   -464.76641   -391.76465    -451.0541   -557.77491    -484.8494   -289.19881   -419.13072   -213.90847   -239.42864   -220.99558   -334.49679   -183.17178   -417.37339    -94.78602    329.58202    583.12056   1108.94599    860.45481    364.81584    -95.91287   -113.14062   -392.80349    -259.2584   -309.71187   -197.17877   -376.79985   -218.61753     32.73025    559.83801    999.22795   1370.88606    954.34035    709.61966    327.12937     598.8807    245.34358     71.62292    -63.01502   -577.38964   -432.53171   -185.60702   -244.27679   -452.01371   -308.10832   -323.75281    -389.2105   -204.45802    -53.13815    145.64151    313.20513    238.20257      29.3561   -102.72701    -248.6841   -319.76237   -416.05817   -376.24563   -266.33991   -192.74388    -84.94556    -59.40154    -15.91922   -110.28947   -214.08355   -416.14111   -299.17612   -320.03272   -330.70105   -376.09322   -327.77518   -332.41367   -204.17579   -345.80575    -89.95547     51.16299    201.90319    160.25838    129.28952     34.77373   -168.00757   -126.94148      6.50227     233.3045    616.64908    412.52115    600.86989    325.91442    -26.35521   -134.35694   -150.43486   -183.32066   -167.98259   -185.02189    -83.21296    108.46236    625.34884   1272.75876   1766.23162   1883.51921   1521.53419    947.13106    752.42452     80.57885   -221.35391    -92.93328    -86.67372   -152.00792   -164.79379   -409.96172   -237.47262   -261.48497   -263.75507   -292.34015   -328.75974    -249.1747   -384.86627   -115.59192   -180.48184   -142.77911    -82.02934   -181.68181   -239.70335   -138.48991     96.33034       4.9494     55.12043    -46.77887   -144.37077   -207.63572   -211.15463   -247.62409   -362.68326   -296.30362   -388.48817   -115.93725   -237.53997   -210.96115   -188.36751   -151.20226    -297.0778   -171.67357   -313.22259   -355.79896   -479.71252      -282.34   -452.69182   -346.47881   -382.74241   -186.55771     -325.713   -325.22142   -247.02238   -100.87215   -104.92477    -48.02248     -48.8606   -252.69199   -298.37634          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       30.00   -661.09357   -512.01153   -564.60955   -294.60616   -365.74777   -278.55052    -416.6913   -200.08278   -481.54783   -186.96766   -468.59056   -345.50455   -254.43568   -439.43142   -345.85588    -193.9536   -424.81823   -249.10901   -342.19043   -178.99091   -232.83895   -345.40134   -236.61621   -280.50459     15.01957    410.12035   1076.99416   1313.69916    810.81193    469.24368   -282.81473    -360.8356    -389.9998   -479.06245   -411.58567   -250.36969   -385.34214   -274.44317   -528.07494   -336.09298   -466.23065   -471.93373   -335.41825   -370.21344    -459.1178   -520.62363   -411.22896   -443.69624   -440.37006   -287.63252   -432.55553   -363.06662   -417.07228   -394.96883   -451.94257    -13.09142    326.45572    916.98481   1002.35735    850.43534    262.37882    137.03961   -283.04447   -418.58864    -245.8541   -422.65139   -151.97874   -303.63557   -174.01307    112.48357    526.91022   1281.71845   1417.11282   1151.66633     691.0836    551.62197     354.8924     95.41302   -270.72638     -93.2382   -499.24761   -461.39425   -300.69048   -310.14013    -560.0803   -379.55468   -461.81008   -507.51269   -251.95036    -18.97336     96.29069    357.90792    342.34649    207.26839      1.79879   -187.77757   -208.85217   -322.96443   -353.16559   -351.11396   -240.48226    -45.15268   -254.40299    -42.08824   -134.19899   -116.54043   -318.57806   -358.01382   -242.73118   -411.80287   -475.83588   -329.83762   -392.82673   -383.36555    -143.4885     24.71857    209.20266    139.70756       -3.445    166.85493     -30.4571    -18.50923    -91.11234    -28.04011    263.97256    388.68426    532.27868    614.37646    387.80703    267.92969    -34.64725   -280.72806   -213.94497   -130.60084   -124.06555    -76.13616    150.96652    707.29079   1199.97077    1585.9141   1879.01221   1791.26732   1174.98336    640.75001    260.12271     16.03978    -59.41499   -188.10694   -298.35265   -204.83414   -229.75184   -219.40384   -356.03397   -251.18875   -363.85571   -347.75345   -318.85791   -139.48697    -141.3978   -170.25819   -139.28881   -357.69617   -149.02731    -38.60899   -152.86084    -21.95449     19.72802   -120.69568    -52.66565    -90.01844     29.15833   -177.80957    -275.1947   -182.33766   -232.64912   -412.54247   -230.83828    -118.6838   -180.61567   -227.85016   -205.38558   -229.27336   -249.00961   -270.30106   -241.67753   -221.86001   -424.68216   -204.15892   -372.29684   -409.00365   -456.42175   -224.71512   -222.94146    -246.3598   -203.16272   -435.65847          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       30.50   -541.92237   -424.57759   -503.47685   -330.14735   -160.45294   -378.98006   -495.69531    -263.1852   -204.40082    -453.6525   -374.11128   -337.55445   -272.49863   -327.35611   -284.85522   -284.84143    -366.6545   -258.90324   -331.84273   -355.78734   -398.75224   -259.07828   -288.72897   -104.30954     40.75557    606.94496   1191.74472   1444.40599    601.48354     365.2085   -283.29142   -302.01736   -239.33056   -438.92866   -293.92828    -380.5029   -571.66921   -269.18204   -410.83655   -174.02993   -565.05198    -563.5107   -403.31386   -340.33408   -473.51743   -468.69898   -522.02445   -284.65652   -393.79164   -282.30093   -406.74105   -322.11984   -353.54234   -347.07843   -360.91556    -85.29822    453.12916    866.22716   1215.24899    833.08752    428.11439     -68.6591   -321.66685    -320.3129   -565.44395   -336.16484   -490.80307   -107.21283   -110.82482     59.90381    417.61142   1142.94564   1212.67592    1297.1974     789.7276    456.41173    326.00349     81.52389    -62.91459   -216.27053   -328.37364   -278.01329   -268.68063   -325.96098    -441.3963   -274.88772   -131.48051   -370.14826   -293.94052     20.71712    262.84428     511.8372    388.52753    174.93094     82.28252   -251.95444   -199.85428   -175.86281   -454.07998   -335.11114    -95.61645   -140.28695    -24.07647   -104.91238   -207.48995   -175.33172    -378.0471   -323.34647   -445.20045   -364.80118   -315.78608   -244.57049   -290.62617   -395.38798   -105.68072    -84.12337    223.37968    221.25139     95.37078    301.87368    -34.25576    -44.77462   -185.43899   -112.75247    123.52635    470.60603    587.59552    519.84988    348.42941    -81.35868   -152.45913    -45.65193   -192.87496    -160.0915   -110.96867    161.17524    295.42991    738.71435   1306.02935   1613.90903   1792.41476   1541.74088   1280.24294     665.7292     107.2382    -88.13585   -155.13127    -132.8715   -326.20282   -242.87532   -162.65642   -297.51008   -244.29042   -359.80229   -327.45294   -239.58849   -250.72317   -200.85867    -80.95702   -236.39481   -226.34412   -196.59533    -20.78536   -157.23591   -201.80118   -139.94346    158.54261    -69.91021     55.70574   -142.65056   -244.54705   -266.82679   -306.14451   -312.85458   -259.21367   -263.99188   -239.94649   -231.62661   -232.75732    -80.51283    -271.3645    -229.8669   -256.59582   -299.57729   -191.96354   -462.30941   -316.01003    -381.7646   -316.03561   -442.81704   -274.10951   -403.38116          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       31.00   -399.80287   -365.48779   -467.68474   -367.73399     29.93275   -496.18942   -472.80056   -275.04733    -103.3287   -537.63402   -400.73286   -374.32247   -356.96599   -293.99458   -305.06253   -334.08996   -444.48467   -278.62782   -287.72804   -374.97673   -348.93144    -459.8482   -369.97531   -174.92319   -160.44506    250.66736   1199.38601   1347.03902    483.87546     60.45724   -237.70119   -350.50261    -246.5502   -304.93591   -331.77906   -476.82692   -443.41662   -372.40831   -333.33726   -415.06231   -528.86006   -366.97394   -753.93353   -234.84247   -297.08199   -595.32753   -436.37194   -206.77965   -267.43958   -443.39116   -491.20937   -362.91332   -342.88411   -282.94677    -329.8268    -82.96489    401.83147    891.95765   1346.02089    920.59718    256.17412   -242.86286   -219.35863   -241.28164   -309.71394    -317.4123   -490.97382    -296.6961   -235.26721    126.70597    590.46701   1228.85422   1354.27096   1203.81049    554.95363    429.86846    366.62923    213.00831   -187.71532   -103.02631   -326.84668   -381.80975   -357.35325   -303.63274   -537.16967    -247.3642   -456.25109   -366.59826   -213.03357   -147.13863     148.9294    258.11519    335.25872    362.37546       25.119   -168.65794    -80.93029   -304.36904   -298.55233   -121.82607   -199.61582   -230.31307    -70.20189    -61.71321   -186.07229   -179.78944   -324.10976   -287.17594   -392.20256   -297.06707   -315.28075   -358.83083   -297.19568   -326.31621   -240.62908    -28.47205      0.89874    282.97287    270.27545      264.957    -14.71149   -140.76194   -256.23757       2.6116    162.61835     343.8115    482.81034    404.06869    469.70175    138.64824    -20.52011   -177.15213   -259.81852   -204.76431   -271.57266    -61.59514    326.52546    562.86964   1216.66424   1723.62793   1893.45386   1720.12656   1015.39007    613.70381     211.8647    -69.23032   -111.89798   -127.63629   -140.60705   -168.44643   -370.69146   -279.91181   -338.87106   -229.54164    -314.0817   -333.36875   -410.94829    -176.0667   -147.42404   -199.50393   -218.29858   -114.52884   -240.78049   -199.95358   -136.01056   -191.80124     -16.7783   -144.74225      3.32723   -121.46705   -147.39302    -141.1734   -328.48171   -139.92154   -238.10125   -157.31137   -231.69367   -243.58062   -169.59847    -79.15962   -240.02842   -262.19461   -356.53292   -232.81749   -320.58615   -490.67804   -463.82967   -566.54174          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       31.50   -359.36349   -425.92012   -454.87239    -396.9022   -111.50372   -550.81039   -435.89377    -299.6844   -284.10981   -386.87084   -427.43232   -318.96312   -459.05468    -357.7181   -373.38356   -477.87837   -406.55025   -434.65679   -422.89401   -370.84613   -270.70633   -404.01195   -365.57871   -324.47607     -54.2494     300.0443   1182.86829   1196.22495    503.50831    106.04344   -324.34187   -262.75966   -120.65025   -309.34197   -313.59877   -356.37611   -466.44882   -422.44663   -369.29067    -352.5082   -250.74068   -440.61132   -474.96536    -43.17159   -397.07351   -620.13167   -517.16296   -461.15991    -430.7423   -201.96119   -512.18757   -385.41313   -413.64056   -218.00084   -307.89382    -87.09687    328.53377    902.47561    978.82164    719.91539    329.83126   -141.47352   -147.41656   -312.04023   -282.88389   -293.33428   -327.71319   -279.79325   -315.52361     87.06724    525.57574   1339.21649   1519.61565   1121.37488    817.84483    525.38859    160.50788     303.2707     -0.48385   -371.37503   -242.26856   -339.25254   -194.02303   -315.15547   -422.75512    -237.8922   -327.56327   -367.77776   -321.42934    -15.56075     97.02852    362.32031    109.68618    -33.99567    -70.89896   -122.17845   -267.56578   -128.61574   -127.71238   -328.43537   -233.57402    155.87285    -41.22557    -23.33934   -103.01355    -136.9364   -356.04215   -297.46978   -376.08533    -290.9875   -168.32704   -305.69401   -373.12506   -257.84813   -241.74045    -96.32196    143.48935    176.47646     97.11627    200.92453    138.01054     68.15374   -212.43506    154.32709     201.1162    413.42914    578.00614    507.96386    542.63163    145.18698     55.14096     -77.2095   -152.33421   -312.52201    -68.97939     -2.58943    227.96185    709.18422   1082.00935    1646.6976   1838.87202   1777.75507   1081.63426    639.15496    305.66334     -5.97728   -201.07624   -126.71365   -140.12584    -224.9198   -377.09976   -255.84291   -323.77103     -214.314   -393.85228   -341.97424    -240.1942    -203.9386   -137.95683    -68.04638   -122.71386   -134.70868    -73.76136   -258.75386    -73.71774   -167.14361   -156.63573     48.60659   -255.44835    -60.84342    142.71522   -174.01349   -202.60208   -233.42982   -228.05541   -371.18992   -312.86078   -279.12846   -264.29596   -239.60892   -107.38171   -246.81375   -331.20902   -483.58764          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       32.00    -411.5928   -553.94172   -457.94344   -472.75149   -282.92527   -496.32304   -368.62808   -311.15131   -426.46536   -300.26439   -281.48801   -269.13358   -457.80966   -375.89062   -447.56317    -391.7362   -164.79907    -524.1728   -473.35464   -470.45861   -260.03105   -206.40086   -412.39068   -286.99957     91.97021    634.15025     920.1538   1083.72105    687.53611    281.39587    -241.1833   -114.59385   -245.23563    -330.9647   -387.17733   -245.23496   -361.76926   -350.79718   -297.02081   -402.56881   -463.16912   -358.42363   -280.09149   -332.58206     -385.526   -497.04697   -458.87619   -369.88326   -200.00111    -281.4246   -384.76889   -360.61664   -453.62674   -353.05509   -244.04175      3.73876    224.35912    857.07646    929.27448    751.03202    273.88329   -159.21847   -289.10768   -262.94412   -457.60362   -391.59056    -385.2217   -309.04578   -327.16409    167.63186     822.9262   1035.56314   1241.63097   1040.44006     712.1741    142.18947    190.00813    355.41171    -57.18365    -49.45752   -208.24646   -232.98534   -397.11559   -239.09707   -369.68955   -292.97862   -442.05188   -266.96612   -191.18421    -23.66577    164.15074    341.39464    276.92386     21.66943   -118.39433   -159.49637   -257.20381   -133.23934   -157.02801   -291.51276   -198.24336   -132.44641    -28.42191    -79.55667   -148.42439   -208.17663   -282.55339   -328.72357   -286.33639   -316.36892   -390.69775   -297.24172   -278.20932   -270.84299   -273.27252   -106.46847      97.1606    306.67527    136.47014    165.08383     31.92022     45.46026   -175.65953    -19.04287    177.47612    483.93648    532.49047    512.09317    352.79249    100.21205     76.47846   -203.05256   -245.05741   -260.64398   -170.07782      7.20407    207.71468     609.7879   1065.24968   1864.82131   1779.28411   1578.04326   1312.20513    699.84335    476.51202     67.39876   -211.44735   -355.98783   -256.27329    -164.0251   -210.09973   -138.27583   -390.31363    -287.3852   -272.40418   -334.62753   -184.84626   -167.86708   -225.69288   -326.57232   -271.54954   -149.61934   -154.80904   -153.33902   -162.96759   -136.49192    -88.68995   -217.17426    -14.45498    -122.7055     -9.56952   -203.73498   -150.70616   -289.90022   -296.72296   -186.74227   -166.18948   -221.74252   -199.14671   -390.30324   -369.00044          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       32.50   -462.35913   -596.45513   -486.51912    -592.8734   -452.87852   -451.62947   -279.56684   -327.71961   -498.67838   -247.80127   -401.79964   -374.05479   -445.30313   -414.67927   -325.70164   -343.99972    -96.30515   -431.83267   -455.92278   -502.80222   -279.60351    -344.7625   -337.17556   -196.10308    -15.63963      534.335   1026.13768    967.50587    721.57343    -56.70682     22.36812    -42.26983   -375.06764   -317.22765   -352.32795   -412.00816   -346.68771   -461.65901   -304.92432   -259.34956   -354.58723   -492.44995   -476.30494   -164.38682   -335.58773   -474.47178     -619.279   -368.11225    -280.5255   -288.52099   -305.74892   -477.50611   -324.70865   -300.14639   -278.12717     -20.6988    248.63154    949.99341    969.83065   1028.26803    359.74264     40.27301   -244.08262   -239.29752   -327.55287   -338.36489   -501.46574   -377.56846   -116.88194      66.9749    558.00367   1082.19555   1403.72535    977.23249     819.4655    406.28905    437.18075    250.97012    -70.13058   -152.40504   -498.24632    -250.3548   -178.55664     -404.256   -425.70047   -455.78612   -372.49441   -439.54172    -60.42945   -103.44736    207.07178    261.37582   -107.28949      8.49171   -147.03624   -310.06447    -134.4277   -188.80932    -249.7385   -108.73994   -173.46429     32.43391     -2.76023   -291.10919   -189.01985   -157.73843   -313.10542   -333.01608   -268.16255   -355.73882   -302.99578   -375.94139   -305.65728   -273.39179   -219.39255      1.40664      86.7961    117.06264    254.88942     94.37544    -88.99241     -70.9839   -118.64366    -57.16079    234.90181    340.02737    591.85916    608.84118    342.13753    223.55086    -23.22284    -77.51782   -206.99964   -207.27287   -244.96414     40.49974    249.53449    735.70555   1178.06194   1689.51814   1937.14393   1790.21488   1131.27049    684.19314    307.20887   -102.55953   -156.11981   -149.39624   -253.39333   -257.97084   -210.72694   -401.93478   -218.87858    -462.1592   -207.42053   -233.17323   -360.18613   -251.46517    -268.6548   -224.75449   -106.21873   -237.27053    -46.85725   -247.80366   -298.70194   -150.57991      -51.412    -62.69609    -57.53607     77.39827   -199.28138   -193.65805   -183.05829   -409.39762   -355.68769   -249.76499   -448.19178   -347.18695          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       33.00   -488.11463   -549.86307   -512.15985   -341.80063   -429.73799   -416.76428    -272.3169   -307.55982    -535.4168    -89.68043   -462.34252   -481.03576    -493.9413   -435.69567   -166.56563   -255.00238   -112.69705    -410.0885   -374.82302   -618.31822     -294.736   -687.63874   -228.61908   -219.86043    -49.78446    649.29863   1000.96108    957.02647    744.09842    140.95834   -259.67724    -213.2403   -350.91913    -350.6545   -405.51847   -341.73588   -399.12411   -353.56073   -468.29918   -343.47677   -311.33828   -364.54404   -210.86346   -374.24571    -387.0382   -341.53789   -298.07029   -435.92301   -322.90971   -458.82492   -394.14502   -487.14834   -391.96347   -315.85647   -177.83719    118.62495    280.07698    702.25777    779.14035    670.57354    253.90799    -13.55987   -210.84242   -288.30136   -332.81782   -298.94846   -321.65033   -277.28531   -105.27528     -4.41178    456.72336    1052.4162   1413.64455     919.0227    715.45855    693.15826    140.02935    195.90244   -136.30547   -185.67305     -72.9619   -271.22536   -208.55766   -391.33497   -395.09361   -368.32324   -427.06189    -285.3893    -75.87465     -56.0015    133.52184    235.62313    197.46744   -108.11986   -173.07175   -108.57571   -192.40826   -227.27493    -53.31508   -149.60055    -23.58483   -161.52588   -106.58091   -249.25739   -118.25691   -201.84993   -196.95896   -216.01418    -333.1882    -263.7924    -299.2721   -269.40923   -470.63093   -338.54615   -118.90807     55.29992    -27.89354    235.94169    134.09178    212.64442    -34.61031     -94.4902   -139.49369    -73.73963    301.28488    500.29576    591.78809    447.53229    434.18383     42.41368     84.78505    -95.45859   -296.09462   -304.18534    -62.32742    -77.34049     124.8383    600.56379   1365.47482   1747.83807   1713.91632   1565.36981   1214.50304    803.37745    171.53571     60.20281   -167.20163   -168.74198   -262.49366   -239.96013   -229.91931   -331.78307   -265.73998   -369.96067   -309.12988   -370.11013   -408.87366   -296.83127   -241.88056    -260.2822   -126.67703   -259.15707   -134.40131   -202.57441   -212.89658   -235.63293    -138.6289    -39.82176    -53.51589    -71.08615   -193.54952   -253.57697   -260.61638   -388.02148          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       33.50    -531.5387   -612.96433   -549.82079   -161.84918   -299.52678   -409.44249   -346.44324   -241.60969   -499.79517   -179.06017   -239.64337    -435.9163   -497.29889   -440.89131    -36.38902   -249.30407   -252.41719   -349.50047    -432.9613   -573.10434    -198.7193   -438.59604   -207.69744   -164.81243   -142.80153    480.29823    807.31953     897.5157    686.88234     179.3496   -331.60532   -310.63632   -179.24229   -507.56715    -396.8631   -265.66903   -286.98302   -442.44696     -452.215   -571.12891   -424.88087   -386.67672   -465.21154   -357.99643    -317.2544   -284.29313    -28.95382   -340.26736   -336.70932   -419.37262   -347.33189   -396.65246   -299.45514   -369.84167   -280.08725    227.89225    238.17882    713.63296   1013.21892    522.23457    386.42392    -75.26226   -413.66266   -465.07539   -292.87462   -353.22652   -166.00117   -334.01967   -242.98289    196.95179    631.56504   1141.01704   1138.44312   1128.95692    613.46428    560.92514    354.87681    159.95731   -287.47367   -263.87908   -400.28951   -106.57965   -164.75302   -318.20321   -398.83846   -405.32452   -375.88406   -188.74719   -184.78738   -139.77972    211.07017    468.30326    212.48718     126.5832   -105.26456   -317.31054   -480.20935   -315.28294   -289.90296   -201.66879    -54.35512    -26.83278    -36.16937   -227.88483   -215.01836    -321.4904   -227.88498   -279.06513   -286.20593   -349.09275   -215.68184   -357.73766   -261.26959   -326.20169   -181.80947   -196.53361     75.10862     126.2406    391.81872    151.93994    156.81211   -107.54217   -107.16412       3.2578    109.28855     426.0687    389.34648    470.23396    368.10251    227.39645      0.96937   -230.79213   -178.51483   -191.66046   -307.97069   -169.01167    166.37633    508.33693    1029.1374   1645.50667   1788.51979   1551.70086   1123.85124    681.95097    235.56688    -45.69747   -131.04821   -146.25668    -172.3285   -361.04671   -211.46537   -388.21198   -314.06956   -294.52237   -262.67038    -344.7429   -329.75181   -282.89532   -223.62398   -135.94101    -207.9393   -163.17536   -286.88595   -304.12436   -230.41293   -134.75443    -74.96264    -43.23777    -80.66203    -69.01813   -314.57344          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       34.00   -552.69339   -830.74957   -454.91985   -150.64601   -203.46195   -382.20812    -422.5468   -207.40523   -524.88921   -210.31524    -89.67725   -370.91911   -380.59611   -347.97041   -125.78766    -400.4497   -362.17702   -224.95805    -554.6241    -343.4659   -155.62232   -402.50516    -86.67216   -209.77275    -88.62818    486.87694    774.89139    1143.2804    669.58646     95.43474     -92.9676   -389.78842    -87.84187   -178.74131   -428.60225   -284.13775   -390.18852   -346.23671   -285.26014   -457.99826   -268.90088   -348.19079   -398.13315   -519.45959   -369.20676    -492.6918   -442.60009   -420.67483   -350.42279   -347.03494   -336.62887   -276.70119   -290.71108   -378.25197    -90.23525    -65.11473    209.95502    806.12583     944.7943    713.00563    333.87151    -11.45419   -383.74568    -342.1665   -327.18345   -280.51457   -392.34577   -263.64027    -43.13776     63.44674    593.67591   1058.60343   1469.30136   1014.55128    831.25397    566.42658    352.56513     14.08432   -267.31875    -264.9114   -418.78689   -294.84066   -240.04592    -199.0946   -259.74514    -253.9225   -196.95444   -357.17463      -3.1542     51.01409    293.21262    134.75976    131.16607     86.61211   -195.40167   -313.83126   -420.14322   -269.52848   -343.46596   -237.00362   -262.10314   -188.81614    -59.89351   -183.45602    -222.2771   -272.02972   -115.78631   -265.36825   -310.05531   -376.29987    -470.5788   -387.46996   -261.64498   -263.08427   -246.74073   -165.66363   -188.45266    142.05539     78.42593     105.7279     67.38704   -230.42739   -200.53309     51.79288    148.02117    269.26233    498.23218    596.99798    420.16541    193.81028     -9.16354   -169.74708   -261.99407   -243.53844   -361.36884     -0.82004     93.04206    644.32587   1136.76713   1458.99679   1954.01728   1618.46366   1193.86571    746.74055    283.79371     56.73713   -309.56604    -51.93477   -140.14893   -335.69758   -210.54124   -230.60205   -377.21842   -366.67345   -339.27222   -311.44785   -351.07705   -310.42308   -213.67984   -194.35367   -176.20586   -209.67938   -334.60345   -330.21603    -202.5315   -194.31319   -123.77256   -180.32251          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       34.50   -551.35098   -986.99533   -347.80189   -308.06023   -361.37129   -349.29565   -559.37886   -265.82663   -511.48582   -183.49113   -183.32686   -378.16147    -360.1714   -203.28209   -152.74596   -409.73655    -475.9043    -94.53149   -545.47649   -278.79634    -116.2156   -515.69431   -239.15846   -343.91433   -216.95907    483.72677    872.11701   1014.56611    740.89796    121.98499   -109.13576   -332.59689   -348.08908   -179.92582   -489.92216   -339.90936   -516.56166   -316.00572    -306.2858   -520.66146   -256.64882   -311.57304   -242.20264   -425.67105   -474.40729   -374.78028   -409.39955   -575.10856   -261.46815   -216.68618   -441.90369   -236.70062   -368.61033   -289.93742   -366.03092     64.01188     147.1011     816.8596     784.1815    816.21096    389.74081      9.24386   -267.90255   -391.78502   -265.94956   -406.22764   -354.39515    -316.7611   -201.67038     -32.1253    501.99253    988.66274   1428.77768   1237.89792    819.38968    451.18186    463.45237    -75.97042    -58.81225    -199.2396   -280.40127   -409.83904   -337.50521   -180.18243   -279.35746   -197.09212   -362.66305   -342.88941     -32.1794      5.20708    131.03834    316.63577    160.35608     66.51451     -1.10549   -470.44453   -282.98962   -288.31176   -246.23544   -191.02073   -200.70652    -73.10682    -31.76959    -79.51188   -197.45163   -183.53828   -255.00726   -428.65923   -377.47766   -364.78607   -263.70398   -249.41455   -500.48358   -207.73378   -303.33127     20.51278      8.47265    104.22581    292.71042     67.67012     33.99865   -108.69054     -98.8943    -36.22405    133.84262    409.88524     485.4285    493.41716    394.49384    196.51814    -90.22723    -78.37832    -84.44164   -335.09218   -225.56237    -32.11178     98.94411    643.61804   1162.71485   1786.14212   1796.48063    1515.4447   1121.02104    665.70756    135.09541    152.03868   -141.17461   -208.96118   -192.49719   -313.05686    -490.3324   -363.83906   -242.98937   -289.25367   -372.15115    -337.0291   -489.59597   -353.58736    -266.4329   -208.96206   -178.86042    -90.34945   -105.06179   -250.19162   -256.79995   -513.72273          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       35.00   -504.10388   -717.87037   -273.70373   -411.22058   -469.06033   -320.85387   -620.79707   -379.34642   -478.68563    -288.7915   -371.74173    -407.0304   -377.79086   -194.39519   -146.90676   -455.99665   -589.78455    -45.25118   -444.88776   -306.71416    -90.87827   -327.05209   -181.88139   -303.29888     -3.68136    577.06965    848.43052    881.57814    759.66097    143.42472   -247.68633   -114.75455   -521.66216   -409.43573   -358.31644   -328.52652   -420.01914   -205.99384   -360.43402   -416.70053    -192.2403   -274.22127   -373.87863   -352.80901   -365.95673    -265.8279   -256.45913   -477.65832   -289.59193   -370.69432   -479.78578   -451.46966   -182.18817   -272.33163   -440.45385   -273.84664    422.31736    581.06899   1017.11929    674.33835    299.24419     89.47044   -371.18575     -354.075   -394.15837   -227.49544   -411.26517   -335.44083    -59.45166       19.141    543.79298   1189.43076   1159.25843   1246.90903     795.0226    551.92181    239.32057     66.84155   -307.85746    -79.54563   -326.60582     48.25548   -309.23313   -236.71032   -335.03538    -133.4547   -337.49258   -366.98244   -407.38293    -70.81795    259.24138    311.31016    379.24445    130.30096   -221.04022   -175.33854   -374.55361   -484.76731   -290.58357   -118.80762   -180.71776    -54.62551    -66.15648   -186.44077   -186.54256   -141.66782   -287.77069   -253.04144   -296.44661   -334.56327   -386.83787   -295.34529   -291.92843   -197.89774   -207.83732   -131.55205     65.63553    243.33286     214.2326    109.44808       9.9916   -157.91843   -167.58303     -5.51784     53.22473    393.62674     526.4161    559.37441    522.78252    197.71229    -39.57637   -130.91072    -53.52422   -370.75113   -192.28149    -20.13151     35.72562    572.45326    907.12455   1463.90587   1796.82869   1761.72022   1136.46948     815.6284    347.40832     38.64797    -42.49431    -74.03403    -382.6462   -221.51661   -392.90948   -259.85918   -320.54475   -305.52148   -452.27738   -305.70526   -301.27927   -147.66996    -237.3309   -277.13366   -202.84504   -281.86183   -309.94317          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       35.50   -470.85107   -484.25873   -243.02757   -413.61094   -411.51523   -398.75829   -472.55056   -414.61206   -468.32837   -339.98812    -479.9077   -302.83467    -384.0091   -244.94483   -240.22935   -555.86775    -494.3224   -270.84201   -381.03788   -125.09558   -129.58957   -117.76765    -94.72526   -176.16001        63.06    470.20063    745.22248    903.61539     574.3385    175.12296   -167.68977   -302.62127   -328.73281   -355.82869   -404.36075   -452.74546   -146.13572   -340.76033   -312.77997   -331.03982   -264.60248    -530.4202   -446.38111   -423.14574   -411.42886   -426.10339   -477.10921   -438.19936   -369.71634     -370.176   -445.12513   -385.75967   -379.40261   -150.14604   -140.68163   -116.07523    389.63028    620.01773    959.07584    616.35924    402.51529    -23.26537      71.8848   -311.54867   -275.85274   -560.35037   -266.17899    -355.2178   -289.46343      76.7942    595.45092   1152.88165   1270.48777    997.72323    824.15119    195.28128     -71.6909    109.59205   -142.85774    -170.6181   -175.53432   -354.22816   -269.81379   -192.04487   -299.47028   -150.93389   -424.44327   -134.18425   -205.92597    -71.92392    185.90348     216.3311    322.61261       102.04   -179.70894   -234.24013   -216.60943   -238.96483   -247.54764   -152.95469    -44.44016   -215.58103   -245.72847    -61.53489    -184.4598   -332.56233   -233.54925    -415.1283   -322.62559   -395.01929    -452.4178   -377.37449   -320.78522    -390.0242   -160.53401   -194.98751     69.98499     64.60757    221.64938     -7.14795    -10.87307   -110.95533    -80.87423   -135.19498     51.43591    340.85037    483.31198    570.27506    514.29197    197.63852     36.59566   -240.92414   -311.63369   -178.30794   -171.35985   -105.96215    186.48063    536.78348    940.52913   1440.59053   1807.78895   1761.41249   1185.71364    705.23504    351.17698    140.21063     -1.67542   -170.15831   -226.60188   -162.55332    -385.3493   -274.74714   -411.08224   -290.54592   -408.31453   -302.06674   -216.00962   -294.06499   -325.13104   -223.08637          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       36.00   -453.62456   -286.50262    -205.2554   -421.83725   -286.67709   -524.19386   -339.28048   -370.05976   -341.55457   -405.17634   -440.52198   -157.02423   -330.06422   -289.51366   -435.20549   -422.54604   -216.53105   -198.16497   -431.35331   -256.81567   -311.16245   -202.65437   -209.71266   -280.10369     87.63231    491.42838    943.34877    945.33267     585.5148    109.98915     65.83065   -334.54632   -278.84444   -360.75664   -376.58143   -495.91736   -276.53211   -488.60738    -402.1043   -243.30127   -401.86362   -477.88083   -478.57378   -413.00416   -402.08548   -306.95174   -443.08004   -443.58875   -494.81957   -299.81553   -510.65629   -283.60502    -477.0564   -182.88638   -239.28293     49.59479    175.54325     712.3899    857.32315    768.29517     289.1984    -73.97761   -378.23108   -484.41327   -356.76616   -402.10156    -246.8384   -257.12117   -151.60856    264.03217    503.42357    1122.4946   1207.76992    1153.1927    740.28955     402.0713    215.99033    128.14533    -32.69432   -192.93233   -400.50556   -236.36984   -158.43026   -116.89608   -400.58407    -48.23936   -339.47305   -238.20168   -109.24255     56.66574     166.5334    415.06874     330.0591    237.67738   -151.16602   -269.11944   -256.78474   -129.57363   -153.42043   -246.71031   -228.67347   -110.70136     -37.9197   -103.55337   -245.35012   -175.87045    -288.6435   -327.37887   -403.84115   -332.20501   -393.95476   -319.81507   -278.87549    -279.9532   -250.19567    -12.57462     60.96002    110.97232     80.13753    183.03025     50.45205    -65.05001   -120.82059     18.07374     78.50088    335.12924    481.97788    449.56452    260.80489    237.66757      23.5176   -171.94075   -162.12904   -276.97985   -190.61228   -107.34188    141.50057    388.84427    945.78353   1486.67821      1660.49   1717.20868   1488.79992    853.08588    196.22875     -8.81957   -183.57127   -108.65928   -238.50554   -200.39003   -223.38047   -280.30173   -322.12473   -405.29855   -370.57896    -327.4633   -378.71574   -514.25755          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       36.50   -455.51661   -240.10635   -120.19258   -392.50535   -217.80301   -601.95609   -247.73995   -237.87146     -330.482   -528.86406   -386.34982   -116.12208   -269.82238   -309.33966   -464.03593   -266.42482   -198.51855   -273.57616   -375.74402   -181.72113   -203.23178   -396.20151   -232.77599   -311.03803    -42.90136    255.57988    894.41001     961.8075    590.66053    239.18554   -207.73891   -202.99469   -372.04626   -401.82727   -308.72372   -549.98496   -503.49135   -384.50814   -442.73819   -305.67315    -343.9231   -380.84776   -439.06329   -449.63158   -436.15761   -316.73968   -512.92098    -213.2442   -460.75987   -370.76423   -403.66593   -397.94321   -319.89372   -309.50682   -281.26414     45.75713    320.32267    593.77784      918.645    602.40882    207.29969   -129.86768   -390.19435   -377.50583   -293.29749     55.33014    -84.65065   -196.21193   -333.72926    238.36517    448.07919     933.6494   1576.77352    1151.2993    654.08775    307.91432    320.48651    359.59901   -467.50866   -316.33349   -267.61259   -119.65153   -181.38091   -294.51193   -194.72671   -525.93103   -264.24056    -205.6174   -370.65421    -31.87999    196.04822    265.39987    382.25519    -33.47062   -306.26971    -81.33544   -151.60987    -175.7714    -30.65888   -340.58966   -159.43175     -4.84806   -105.50752   -122.92077   -221.45339   -165.62567   -355.54936   -443.90209   -337.67467   -140.53428    -314.1765   -327.29915   -305.14318   -260.56714   -299.09667   -106.10517    -90.33962    181.75898      75.3994    -20.49935     -10.8299   -195.90135    -24.46417     22.18022    225.27801    325.76299    551.86549     468.2749    346.16452    279.89654   -189.59925    -29.52179   -147.20874   -192.67807   -226.73772     27.79003    101.42284    335.20056    965.32875   1369.32875    1710.3714   1741.70569   1356.58158     802.8481    335.47114      2.90697   -102.09868   -227.94152   -228.53088   -297.65711   -299.95504   -304.90832   -341.51042   -405.07215   -376.13953          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       37.00   -529.01634   -190.83881    -18.82639   -325.31015   -178.58723   -478.16476   -272.77007   -165.76921   -492.25075   -551.29178   -341.06746   -233.77618   -312.82807   -321.40428   -438.53702   -453.87617   -328.18286   -528.32474   -261.05733   -233.38656   -327.97966   -456.10131   -376.57134    -172.6754   -106.47836    265.72141    878.50429    869.72928     549.4237    240.33121   -235.55778   -313.58243   -233.70078   -443.95871   -381.89691   -385.65827   -726.52345   -490.07133   -431.29769   -446.92368   -488.87065   -514.93792   -320.73196   -278.29128   -386.40075   -418.54846   -377.24743   -393.54159   -409.26294   -365.61264   -442.73302   -281.94788   -298.95101   -259.39119     -2.56187   -141.98997    342.29714    662.16614   1028.19595    830.86833    254.47736    -27.39455   -347.36622   -317.86414   -209.99976   -193.12293   -290.46586   -281.68869   -204.98575     84.13547    466.60593   1128.74613   1224.68676   1014.97469    590.56631    640.95875    -56.72674     77.97807    -72.02891   -129.97772   -186.88522    -406.1634   -218.60287   -335.94584   -218.18518   -377.30815   -332.74213   -358.13247   -367.08778   -226.27459    149.06505    414.59039    239.55416     20.72206   -198.87913   -327.54987    -189.9749   -346.51699   -216.93818   -271.88947   -162.41367    -93.07995   -133.00151   -176.40007   -248.19416   -251.02533   -320.50155   -418.45973   -279.23502   -496.74798   -320.27484   -357.56373    -295.0309   -310.58167   -303.61749   -196.25021     10.92204    231.84052     51.05025    217.30447    -48.98155    -25.93876    -102.0532     -0.01457    106.71152    391.96318    323.69747    665.76743    456.85315    326.78395     91.98938    -91.34354   -155.93464   -247.87687    -89.50661   -188.83893      59.7145    384.95291    835.63525   1381.49406   1595.81856   1774.69138   1261.04714    703.92958    371.61232      47.5702    -47.42011    -35.51223   -336.77855   -401.25293   -311.18226   -252.14849   -444.80569          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       37.50   -695.99145   -109.58261     31.56076   -274.17847   -142.47074   -355.70774   -319.84372   -134.94063   -436.99553   -557.69376   -333.83396   -357.60878   -244.53613   -494.71268   -507.68044    -528.8562    -411.7733   -336.22969   -294.79285   -289.38734   -450.72886   -387.94921   -387.35316   -206.14423    -91.84155    277.00424    824.84821    994.86169     675.5219    293.68753   -181.00187   -341.56474   -367.98616   -465.00858   -516.73446   -270.01397   -480.94121   -406.28419   -551.27999   -431.69478   -404.99919   -339.48662   -389.84685    -358.2134   -443.28787   -369.91602    -481.0467   -296.54481    -251.2846   -537.81552   -318.60189   -328.88828   -295.53468   -525.37735   -271.36881   -137.05511    -33.28527    776.30179   1107.11634    889.69197    234.27068    -73.45377   -408.13254   -323.72186   -309.31015   -268.87599   -278.62551   -232.99282      6.58675    150.44209    471.21029   1148.21817   1137.66734    990.99955    668.30511    270.19785    179.42261    155.27436   -117.04101   -102.86615   -194.15868   -218.21343   -318.99597    -265.9374    -319.2674   -385.41003   -304.93292   -268.68134     -343.355    -51.99362    321.64725    464.89489     339.7984      86.0842    -46.30355   -107.82236     -1.60502    -41.62372   -111.75507   -219.88265   -178.10915   -159.26338    -87.09662    -91.12968    -64.83122   -280.98468   -293.40007   -392.75853   -357.12101   -237.85866   -286.99174   -244.24668   -282.15835   -178.02337   -212.67512   -187.02507    -74.64648     49.32865      61.2681    118.31248    -57.25163     -69.7842   -176.60951     43.85263    177.13594    274.00305    399.90667    692.65151    437.56357    161.52727    178.10373   -198.41004   -278.12014   -193.38942   -160.94333   -217.30233     57.00784    361.94952    788.71016   1316.15195   1837.39277   1564.62437   1439.41261    910.97605     278.8726     65.32949   -239.74743    -275.3283   -302.54892   -436.58634   -414.43067          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       38.00   -765.86976   -136.07137    -93.13014   -184.53182   -101.71289   -445.67384   -370.94784   -115.87502   -429.94767   -528.74503   -265.62055   -381.85772    -105.7769   -423.98802   -628.06819   -387.99496   -424.72155   -111.86364   -252.95607    -190.5599   -396.49504   -346.04068   -366.84267    -224.8713   -171.48249    328.07563    649.71081    885.18437    602.81152    248.43721   -210.16574   -496.40174   -446.61022   -324.65523   -380.86907   -270.85552   -182.54258   -324.78082   -354.52297   -440.45883   -466.40235   -383.16306   -466.64661   -194.00019    -124.8277    -364.1326   -593.95965   -268.95979    -411.0807   -487.84865   -328.43069   -383.49438    -474.8313   -402.34142   -448.65026    -77.65337    412.59831    821.18146    860.62724    546.07268    345.88067    -139.4383   -364.65369   -378.99612   -330.48514   -304.16407   -220.44343   -253.74208   -201.03654    270.20365     541.2574   1118.13118   1197.63283   1156.94874    772.05118    381.05751    110.57427      27.7319     549.3781    107.15014   -236.01204   -109.18561   -357.90134   -318.42856   -482.97687   -350.51562   -242.98231   -352.02961    -32.08235      2.05178     40.58415    240.19386    345.30751    277.64807     34.66786   -246.14592    -34.08779     -5.79912   -139.13908   -539.69729   -293.03272   -102.11895    -82.90392    -91.57264   -134.33833   -206.15468   -319.10478   -190.40764   -359.80891   -362.60108   -289.32967   -333.80555   -474.88683   -417.48548   -152.06562   -133.11355    -95.66214     36.87092     38.57058    -63.85241    -10.91161    -75.52362   -269.29726   -228.32203     54.90372    398.41846    469.09463     478.0992    439.90285    185.24103    -31.01851    -88.90844   -279.61034   -250.21505   -339.72452   -141.21696      55.0937    462.72935    713.93006   1370.84624   1492.70186   1436.52527   1279.67073    675.52468    301.54906   -116.78185    -95.94537   -378.65066   -399.99979          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       38.50    -526.4864    -306.6817    -316.9975    -44.67389   -207.24711    -545.8548   -443.86201   -221.44947    -400.8236   -329.67741    -255.8443   -337.32561   -209.78519   -233.03248   -458.04087   -312.54217   -389.13283   -308.12944   -459.84201   -175.44113   -393.06215   -353.85825   -463.09871   -276.69944   -234.70292    334.77739      774.997    742.45161     494.6014    164.82706   -132.31358   -433.45794   -450.44209   -382.77445   -377.31134   -209.86679    -620.2254   -382.70644   -422.99537   -349.20825    -260.4484    -193.4311   -428.27671   -339.62299   -392.71041    -346.6391   -448.69792   -371.45764   -390.51475    -294.6561   -413.85396   -456.20447   -365.22699   -115.09719   -435.27944    -77.20228    125.06664    603.46949    806.29529    446.31214    287.00769   -145.67226   -357.11245   -290.02708   -306.42957   -325.30299   -286.20789    -283.5496   -129.96227    284.56309    212.72781   1091.49138   1334.19202   1018.98139    862.17934    543.09512    425.69772    170.26089     236.2878    320.55374     50.08115   -333.60031   -367.32513   -345.00705   -240.91928   -360.50169   -358.30698   -317.52829   -189.80963    -24.15016    142.21734    210.75256    260.05374        406.3    -15.50989    192.78024    -76.99189   -157.44599   -513.40633    -357.1517   -250.07435    -116.3353    -37.95137    -87.18063   -319.74735   -319.17938    -196.9892   -384.28046   -415.02829   -326.66109   -466.20452   -355.90866   -356.27655   -327.29267    -408.9771   -141.37093   -105.72955    -66.87231    216.37546     19.74366     20.90265    -92.09864    -18.64552    -67.69102    -31.25459     30.57095    378.64154    515.77746    627.46287    161.93073    -22.14806   -257.81354   -157.89814   -116.32176   -104.94879   -126.02023    140.62191    170.96826    874.89757     1188.817   1415.65227   1563.61217   1097.36667    759.48103    216.13965    -68.68943   -380.51044          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       39.00   -302.13181   -489.90012   -526.18284    -68.96725   -405.97624    -447.5403   -503.65269   -344.83451   -280.44506   -418.02299   -288.72001   -288.37475    -131.7868   -231.94432   -156.18386   -398.25917   -413.28675   -367.72935    -496.9455    -321.0981   -471.79228   -304.24234   -502.24577   -350.00463   -242.49947    342.33346    812.06896    734.21457    592.80004    132.71266   -223.25829   -446.79088    -319.9286   -520.85775   -385.70838   -488.73272   -356.23637   -489.36101    -403.2282   -317.44281   -398.65948   -356.52756   -257.15495   -357.32992   -325.09848   -505.97581   -365.73441   -225.20797   -438.02801   -465.89727   -409.31911   -436.04817   -306.88545   -366.56149   -137.22364    -74.35982    120.45898    785.95538     722.7134     781.8353    220.58719     -2.22861   -285.45463    -287.4267    -187.2597   -193.88838   -304.42418   -155.02018   -132.67833    252.98419     429.4822   1086.26815      942.361   1042.21997    779.63748    408.45585    360.31447    134.51847    226.78542    -41.11159   -124.61763   -224.34527     -394.022   -129.14827    -346.4146   -305.82265   -425.17997   -392.15091   -221.70251     11.02067    163.00475    384.06985    423.43057    479.38264    170.52404     12.05834   -167.34929   -418.46678    -422.0637   -307.21813   -218.92071   -109.25042   -109.10019   -216.97462   -193.69886   -200.46027   -298.86951   -262.47179   -216.10647   -342.33457   -348.47602   -420.14376   -340.80037   -242.01414   -244.65092    -76.67757   -118.07252    124.27762     36.61277      62.0778    -53.05026    -91.13902   -165.02868   -118.54446       51.581    280.09554    545.48163    464.38542    348.10238    174.16808    -73.67065   -196.60357   -300.27598   -221.99704   -242.47535   -274.93329    -60.72897    306.95902    502.85371   1164.85081   1408.97412   1205.83572    973.53417    149.33041   -274.16742          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       39.50   -206.18962   -580.91835   -542.06648   -248.70931   -570.15747   -338.47726   -463.17847   -327.83625   -178.86992   -479.69402   -317.70926   -305.66268    -50.28396   -218.02842    -13.20835   -456.14586   -381.41976   -231.33977   -510.05735   -313.88615   -419.45623   -303.76896   -454.28424   -389.91068    -255.2228    268.37591     745.5982    744.27901    446.63482    155.34387   -139.05712   -350.86978    -350.4607   -275.12926   -399.37075   -526.82404   -486.40245   -174.56407   -446.84117   -432.86502    -369.9746   -380.73419   -431.51717   -300.04217   -379.50186   -344.71371   -363.87503    -448.3615   -256.18932   -257.13253   -413.68131   -222.17129   -373.21569    -342.8569   -295.22123    -24.24102    219.42412    610.77107    939.40158    680.36982    481.30663     36.58635   -419.64191   -429.88956   -334.82504   -586.37075   -311.06454   -203.52415    -108.2515    284.61814    523.17692   1029.55916   1183.54217    892.68846    705.87874    416.39363    344.39662    169.09035    239.70074      89.3733   -394.21219   -465.25539   -571.27338   -434.25485   -322.02789   -171.25483   -314.34142   -184.77595    -94.78929    -24.32578    258.67095    467.82305    636.55079    307.46506    287.67923    -131.6729   -173.84145    -19.24956   -393.26909   -386.14683   -211.95756   -167.75994   -184.80984   -257.56427   -173.45756   -288.76921   -339.97893   -286.97937   -306.19442   -462.19284   -277.39966   -351.89317   -394.92466   -449.82276   -195.17929   -229.22223    -29.67335     98.19989    181.49176     22.20408    -24.81706   -188.25067      5.39423    -45.49521    132.37798    138.18353    519.04604    623.41458    539.85241     98.79201    108.22823   -178.75939   -304.45564   -200.27001   -256.97495   -208.64127   -105.87329    319.65029    544.67998   1125.93244   1054.20138    838.52257     133.0149          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       40.00    -232.0231   -378.09893   -496.77458   -400.79357   -661.57938   -363.99411   -402.60319   -329.40068     -287.841    -252.4174   -422.15159   -228.61717   -293.61588   -160.61034   -175.73088   -401.42384    -211.4241   -438.89082   -415.04625   -317.07839   -391.28408      6.98687   -360.30879   -429.62541   -119.12243    331.14967    804.08919    713.85189    480.29491    168.62224     60.39686   -523.54056   -394.45047   -415.34164   -453.43622   -430.74753   -392.34481   -303.30809   -465.19922    -322.8825   -359.73312    -199.7105   -418.64876   -355.58424   -403.70448   -268.54553   -440.09199    -461.5202   -434.00077   -487.28572   -394.55765   -175.58568   -360.46279   -213.94717    -183.7544   -210.56855    434.74163    441.58055    883.78864    564.59619    298.57559    -86.89647   -243.81716   -444.10381   -290.93672     -53.4532   -279.07753   -392.24165   -310.72663    248.80187    564.91245   1236.67465   1281.38686   1117.52982    629.77407    438.77836    297.26858    139.50203     85.13451   -166.14246   -138.03677   -249.16623   -497.57882   -235.57749   -372.26374   -206.53795   -240.12688   -207.96242   -170.18816     11.84487    181.16779    411.74406    637.87523    737.97935    223.21301    -50.75321   -335.74165   -274.59619   -347.32606   -289.97983   -124.49964   -256.43224    -44.69505    -60.87698    -23.37836   -351.16723   -336.64558    -362.6036    -385.1026   -406.51749   -298.14548   -299.53174   -352.53585   -277.01962   -138.19823   -200.97857   -109.68814    -14.17762    257.05431    151.45416     28.66364    -23.11989   -129.20144    -94.70793      64.5809    301.15431     379.2095    523.66678    430.02806    242.29774     17.79265   -203.48422   -109.95732   -186.27924   -299.37832   -252.81864   -170.06772    151.05339     516.9934    506.87547    350.79648          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       40.50   -210.59868   -177.89271   -355.38475   -465.41878   -610.19051   -252.63671   -359.01132   -365.63036   -418.09143   -278.80198   -453.03387   -215.32779   -378.64198     -226.652   -342.27464    -340.4558   -279.57558   -458.09078   -293.83377   -355.86643   -435.66029   -264.57011   -294.23941   -444.96062    -40.95057    328.09248    552.17583    834.56411    545.15206    262.51796    -13.41262   -380.92325   -348.80794   -277.55862   -311.96771   -470.01605   -621.53864   -408.58506   -213.90452   -218.48794   -422.49214   -260.47545    -354.8352   -335.04424   -427.85966   -488.17134    -149.7936   -300.64953   -362.60459   -272.80246    -354.9043   -443.79047   -395.08209   -363.47117   -242.15575   -179.63312    207.80154    478.76236    752.80056    292.09774    175.17888    -102.9368   -275.67874   -401.20007   -266.05225   -185.79858   -264.70666   -310.25979   -128.33241    208.89903    335.15418     1392.441   1212.39311   1013.64493    862.75532    445.27816    150.31319    100.99127    -65.78349    -142.3354    -258.2695   -600.20655    -431.0037   -334.97652   -274.61961   -277.31408   -407.17314   -281.42605   -170.95903    -34.53324    209.52974    355.77521    546.70735    189.74031    182.68677   -205.08604   -408.91411   -403.78788   -324.47131   -333.57725   -269.80785   -121.35804   -116.42371   -184.80762   -212.89544    -368.8605   -375.48764   -177.13777   -303.78073   -534.43528   -299.66815   -440.95708   -332.57505   -365.63756   -238.12422   -218.50321   -110.72434     50.54369     22.23036    179.31751    -66.32869    -36.31529   -131.53673    -72.40869    141.22882      7.74955    479.46314    515.81029    343.00066    335.64572    179.10364    -58.69651   -327.69611   -304.69627   -252.14153   -304.58222    -94.99033    -36.46738     74.48007          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       41.00   -179.06578     17.09834   -362.69266   -500.20268   -496.23493   -121.94926   -414.34443   -259.70402   -399.12987   -612.30517    -337.8354    -456.2102   -409.88741   -230.64642   -268.82063   -391.85859   -274.47985   -620.11124   -305.32523   -296.86322   -414.61163   -271.55232   -390.52775   -269.57807   -133.47062    311.64523      376.808    707.77324    514.17057    141.00611   -379.16544   -372.30887   -348.06816   -182.86578   -343.17386   -451.29934   -334.80015    -470.6322   -369.17438   -436.41851   -232.59931   -348.58209   -401.96092   -294.51931    -379.4105   -400.61369   -291.61883   -402.31086   -301.58038   -496.94589    -515.1996   -328.01941   -379.79944   -177.30136   -161.17563    -33.28139     244.1104    448.72141    457.23167    685.88271    524.48824     11.29884   -233.89605   -208.77904   -254.38097   -163.23871   -408.37507    -345.1829   -215.60434    157.28846    542.60616   1182.70073   1240.72244    847.23528    685.68922    401.63367    262.60921    246.16877    -32.94662   -232.42383    -38.46025   -228.70797   -409.84554   -221.15563   -152.39756   -485.75895   -324.52127    -253.0357   -207.06596   -187.99993     24.35028    222.82175    481.87057    170.95042     10.81977   -295.30592   -415.46997   -448.45682   -397.90211   -346.22585   -171.24739   -168.73682   -112.12803   -205.39148    -65.42935   -182.05573   -255.40655   -401.33005   -221.22218   -340.35443   -316.18039    -466.5153   -131.28992   -399.52713   -299.50821   -176.25359     11.54906    149.62338      36.7082     68.33955    -48.95847   -134.23649   -301.19197    -44.32762     41.21568    278.22884    441.57087    455.29155     381.6369    141.91313     76.94026       -169.1   -298.13212    -240.0788   -277.33474   -361.22825    -330.6258          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       41.50   -137.38049    -32.79509   -358.55174   -536.83912   -379.02071   -251.20315   -507.53303   -276.00375   -261.51251   -702.80401   -311.88996   -446.61201   -310.40355     -239.554   -321.31773   -414.30283   -219.60756   -429.04188   -281.49659   -349.00785   -378.65125   -272.18182   -409.25456   -114.65555    -52.27561    420.05121    412.04426    618.98817    428.36307    144.86799   -115.39726   -573.02719   -327.35733   -308.79519    -192.4934   -561.74095    -326.3166   -304.25942   -432.14755    -448.2646   -248.38754    -297.9172   -376.07285   -615.20881   -316.11172   -331.93603   -465.97029   -394.79266   -307.64393   -325.35034   -517.24356   -357.49546    -501.0673   -321.86631   -238.16055     -27.5061    331.48171    672.02197    763.05066    696.90754     305.0513    -92.47584   -205.20217   -186.01489   -333.30813   -390.97984   -345.55184   -211.42809    -79.63419     22.77036    567.11612   1015.87794   1110.98907   1083.57145    745.43232    335.94575    239.12887    -39.74551     41.27354   -271.27656   -128.32401   -433.74954   -300.94132   -328.62751   -427.96555   -251.11106   -458.52718   -146.49597   -277.39288    -24.67939    136.76629    337.50832    345.81209       278.18   -176.35966    -215.5667   -356.85013   -294.50179   -546.64947   -398.51625    -253.5849   -183.65028    -80.93909   -126.60418   -174.82237   -257.49614   -393.45408   -314.88917   -378.21832   -389.30819   -438.67805   -332.08433    -338.6786   -297.46121   -295.37699   -226.19687    -98.44217    190.02262     75.69024     21.48962     45.04492    -12.89731   -155.07088   -180.71367    -24.84066    186.97839    367.23458    418.19286    504.16362    327.87661    -50.39192   -138.22859   -245.43451   -232.31362   -433.40435          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       42.00   -155.25211   -102.26247   -271.28276    -476.3255   -359.00713   -372.82772   -553.79558   -413.59734    -155.7987   -613.82439   -373.66617   -310.98994   -160.21936   -348.13517   -503.06608   -290.98702   -342.80273   -436.63502   -202.01342   -462.23598   -473.90501   -356.47097   -452.04867     45.32072     89.70598    294.21678    482.45865    657.66919    541.50212      14.4432    -42.28213   -402.03418   -550.40968   -373.67247   -216.21513    -347.1125    -308.1053   -259.12114   -231.74358   -378.41264   -225.46917   -365.67868   -411.48879   -567.58223   -214.83539   -255.63004   -325.49163   -356.87745   -309.37615   -296.22074   -375.20901   -377.52797   -297.14868   -239.62542   -229.72624     -4.29079    235.94009    529.62885    934.74284    782.20711    120.88113   -147.61072   -211.83494   -458.54787   -299.53567   -445.87907   -357.87153   -310.28429   -312.55379    -15.08514    509.67041   1024.16776   1276.62156    920.05069    697.22261    437.67808    366.98072     394.2561     -32.6566   -135.83079   -357.20272   -381.97485   -186.56642   -351.50324   -307.51401   -430.09987   -298.55353   -162.83706   -338.64756     55.22263    202.97621    221.59263    287.80926    454.75265    -91.32313   -394.84078   -444.56964   -460.92623   -451.28189   -120.85079   -441.03958    -206.9737   -239.90772   -341.45352   -290.16035   -235.65336   -203.77663   -368.80685    -447.8555   -273.27415   -504.89683   -324.83065   -203.56561   -294.09172   -287.26973   -145.65466     -3.53916     97.56258    147.20972     41.29154     63.59754     -2.56517   -170.54571   -167.61481    -67.09031    203.88446    395.81282    465.85537    323.08974    224.72389    -43.67578    -90.38257   -330.50485   -402.07651          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       42.50   -257.27299   -113.20548   -296.90975   -309.71365   -380.39711   -371.61526   -422.38651   -515.93355   -300.94848    -461.6755   -438.75974   -273.84033   -216.54807   -368.63134    -357.3183   -342.36666   -391.90183    -469.1446   -305.98371   -529.04368   -254.40153   -312.07228   -452.18294    -93.81107    195.38715    353.26445    372.14339    714.33366    446.76276    -19.11994   -188.05689   -167.79123   -426.00493   -432.11705   -331.23509   -385.61296   -321.08428   -280.24559   -432.82033   -318.25284   -325.00609   -334.90197   -590.82163   -473.05641   -503.94685   -493.92973   -303.88565   -492.24664   -300.14452   -415.26917   -297.24027   -301.64923   -309.40146   -259.54538   -407.53522     14.37276    243.02892    642.95781    568.10475    778.84245    273.72327   -128.62771   -126.64336    -314.3654   -255.37151   -538.18156   -317.21027   -147.72257   -100.17758      1.22448    645.13074     919.7963    1081.6777    926.64272     831.0779    403.22871    186.30411    126.84328   -335.19352   -178.98062   -540.72144   -603.61057   -328.85441   -230.40795   -412.22298   -330.50158   -308.92411   -335.91296   -239.08823    -78.27094     23.56193     378.1551    366.64017    207.14956     72.08305   -339.16907   -395.37339   -458.23953   -449.24756   -398.98066   -283.12736   -133.74618   -121.90751   -141.77738   -254.82131   -162.41616   -261.81997   -523.75046    -125.9902   -342.02225   -357.37113   -314.88796   -271.91177   -355.44598   -249.53202   -276.91813    -34.89827      21.1504    177.55488      2.15372    -81.10184    -67.04893    -73.11567    -68.05639   -138.93018    222.52892    225.19049    426.34265    346.91651    151.09659   -151.16157   -380.21835          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       43.00   -293.47321    -65.46794    -236.6037   -112.31051   -347.34295   -234.05584   -338.35804     -387.958   -616.72261   -366.97878   -466.76584    -350.3144   -408.42882    -292.1569   -342.40573   -367.69173   -374.04067   -111.62954   -400.81266   -394.43629   -281.03807   -219.91208   -493.36672   -227.83626     72.66726     478.6838    569.74166    554.23858    353.16531     40.98299    -84.25414    -19.60346   -442.07001   -415.88626   -280.68397    -329.0861   -325.35784   -538.17548   -329.23666   -530.97466   -456.91119   -399.74469   -351.48919   -544.97582   -435.01901   -462.17323   -377.35295   -330.03842    -530.5149   -239.86308   -346.52347   -255.65768   -358.87402   -339.03439   -137.52145   -156.11314      261.163    678.03317    658.30618    486.45599    335.71629    -21.42941   -287.85283   -385.93007   -396.16358   -468.31937    -83.53803   -125.32922   -102.97396     96.09951    569.52753    1077.1442    1311.6151   1152.18744     764.6333    334.52118     52.61863    -63.48339   -283.46518   -308.40931   -248.77055   -605.62983   -321.22257   -484.06592    -297.9554   -153.75504   -330.83516   -352.26677   -288.67675   -129.59973    441.77053    301.20031     52.86836    193.01814     145.2662   -364.04876   -393.34553   -396.65149   -322.22214    -402.9898   -356.70593   -345.58672      2.41187   -105.71829   -142.09109    -424.4972   -331.93583   -373.89218   -345.41458   -180.68061   -392.28784   -336.15359   -447.64734    -414.9427   -261.45284   -202.40845    -32.25384    -49.13888     51.19758      5.98318     18.14276   -221.37077   -213.87217   -168.45749    -98.13794     78.23391     341.3433    281.46254    195.19645    -54.22869   -308.18923          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       43.50   -269.73358   -134.35824   -184.16651   -125.94843   -194.74209   -243.15897   -257.35283   -187.94305   -932.12502   -256.95936   -396.70783    -170.2547   -319.62139   -338.40459   -469.80715   -210.14796   -450.36273    -63.47573   -352.46805   -325.64319   -303.20136   -450.45759   -279.65585   -169.48822     -2.37288    373.36357    701.84029    603.10533    392.86265     25.03237    -94.87791   -301.29411   -253.23642   -408.67376   -219.43493   -329.32808   -538.66926   -354.18305   -520.79441   -459.51248   -369.27484   -502.62411   -373.25633   -392.86827    -571.2198   -333.38095   -419.35839   -284.49798   -494.72429   -143.93935    -279.0907    -372.4365   -249.37914   -255.71154   -422.99379    -18.83649    341.96105    479.93397    544.77445    633.91102    236.53888    146.03124    -183.9739    -337.0025   -388.77736   -424.92125   -430.86347   -243.46581   -468.72162      7.84736    675.76978    987.91106   1196.45446     878.6928    641.98563    379.15639    234.56981     66.26618     27.43225   -532.25581   -275.87552    -252.0978   -395.15079   -374.36729   -332.48634    -335.0663   -388.09887   -423.06334   -150.66636   -179.29994    275.00387    171.88882    345.95593      4.48178     -92.0885   -298.37783     -79.7681     -262.177   -448.09038   -348.15334   -240.10519   -301.38244   -161.60391    -35.12714   -234.06549   -234.27884    -302.6493   -236.53525   -221.14368   -250.24938   -298.32085   -396.07796   -376.83032   -146.78208   -263.40673   -211.69246   -146.23033     55.39265     21.71662    125.14244   -200.83318   -148.29834    -81.28132   -168.98999    -86.33642     87.33484    195.61426     267.3332     16.68908          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       44.00   -173.79752   -191.07047   -120.20708   -150.99157    -30.38679   -380.03306   -206.84106    -106.8135   -533.01774   -177.78705   -311.11783   -160.59818   -305.04705   -443.47334   -507.82126    -103.1915   -371.58106   -258.43169   -394.03879   -388.22135   -447.87031   -417.71945   -134.95098     -73.7007   -110.81953     91.76503    753.60083    633.10141    441.50085    -19.86623   -277.96805   -354.58324   -293.07299   -325.00889    -369.4895   -326.39587   -402.98417   -513.92183   -556.39166   -363.38809   -404.25673   -563.85678   -451.80663   -430.50401   -509.20099   -321.50495   -333.27806   -216.16788   -384.43732    -287.8003    -359.3655   -444.09435   -424.62299   -312.35992   -255.68827      4.86984    435.47507    373.98752    683.02073    579.93304    237.86656      -55.711   -146.57365   -335.18918   -254.36299   -271.93802    -323.6804   -159.82219   -199.04675     90.95221    676.64824   1050.38018   1272.62275   1183.28543    809.49601    391.33646    360.67299     11.00417   -318.96668   -209.90591   -446.44072   -333.11992    -393.0802   -283.22463   -373.56048   -320.23677   -182.12388   -251.98638   -273.44483    -40.61934    297.29858     81.77852    285.53438    197.34388     84.35906   -161.75472   -105.91279   -481.44789   -486.62712   -346.47198   -178.27066   -304.78548   -359.17722   -155.20813     -167.181   -253.68591   -425.88214   -205.58564   -290.75496   -227.25896   -397.21657    -235.9898    -411.3705   -380.25018    -57.40988   -195.75665   -209.56405    -31.54283    107.08461    -63.32294    -86.21225     45.34838   -197.93954    -236.4679   -223.37954     73.98606     -60.2258   -249.58151          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       44.50   -243.63091   -214.95068   -123.20228   -111.02816     15.82521   -406.45379   -134.60679   -201.04671   -305.94533    -246.9571   -290.49628   -499.64494   -344.05602   -536.14774   -435.75733     -178.082   -225.86888   -278.11021    -478.1371    -208.3963   -235.65906    -675.3925   -264.75506    -37.21404    -96.08907    189.91039    610.55002    717.35072    621.98767    135.41488   -317.33945   -345.95333   -310.03492   -349.49156   -386.02366    -425.3568   -286.21739   -645.68044   -437.84234   -513.43724   -400.50746   -423.30836   -331.19264   -518.27402   -460.36094   -371.04941   -273.03874   -445.67454   -362.90154   -471.61724    -383.7375   -323.02448   -494.10532   -267.48903    -362.0407    -49.08614    -19.07175    320.27688    458.65429    761.57828    220.83965   -176.61372   -306.96633   -392.56997   -119.76351   -220.73531   -198.25309   -315.50655   -117.79319    162.87579    324.08544   1011.31882   1062.21178   1073.88672    672.63225    353.68941     82.03586     86.93753   -178.63543   -363.65548     -170.713   -404.75571   -383.98342   -390.31155   -332.81997   -394.51451     -529.836   -360.89936     -90.5317    -42.17048    -54.74759    239.65102    262.30581     55.21925     62.39136   -102.96247   -283.16155    -440.8861   -188.45593   -329.38833   -391.75265   -149.26105    -115.3629   -280.93635    -127.6539    -186.5723   -382.30436   -343.62597   -352.05039   -259.64444     -375.036   -410.32585   -481.81926   -351.60105    -323.4568   -225.14864   -105.83425     67.66207     51.85194    119.94695   -119.43218   -123.01373   -236.45098   -391.11864   -124.12603   -176.19876          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       45.00   -340.01294   -212.19033   -240.27688    -36.30822   -233.70894    -385.4739   -133.51851    -280.8291   -357.52041   -384.10135    -369.3599   -657.42187    -348.6895   -626.79799   -288.06945   -246.36155   -283.84601   -303.31588   -546.53888   -211.12983   -131.18113   -315.14551   -356.82157   -120.59528     17.22782    406.53747    573.90324    636.36315    413.06895    198.50946   -182.32138   -312.33923   -158.51537   -378.63444   -133.81129   -552.88646   -273.80973   -364.01973   -377.29087   -317.49046   -500.26145   -382.21056   -235.45783   -448.93289   -271.49917   -332.06331   -440.91156   -315.32074   -417.63684   -375.81488   -338.99207   -316.20926   -260.47947    -340.0163   -445.47178   -149.45518    131.73954    422.52774    530.44683    587.65288    269.27511    -22.23572   -104.74285   -369.67684   -303.71597   -348.68733    -238.1607   -296.71694    -51.87857     89.13724    419.75132     1042.538   1209.04541   1311.76964   1049.02771    209.70209    199.98594    123.08857   -103.43575   -295.12849   -336.77016   -395.63298   -230.08194   -364.40174   -332.03585   -163.47427   -250.82922   -329.48156   -336.36192    -46.83935    150.11613      178.756    318.06464    389.93097    223.15981   -142.12487    -67.02881   -364.38905    -258.6751   -277.47831    -274.4711   -332.87661   -140.66722    -232.2843   -153.70326   -347.74316   -324.64739   -240.76317   -332.47401   -216.70796   -356.22995    -390.1947   -392.29304   -375.37643   -420.79375   -347.07949   -270.43649   -117.13577    -53.25783   -106.39753    -90.75236    -182.0311   -138.90676   -291.29539   -218.77606          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       45.50   -380.08289    -302.2626   -387.88615   -196.39784   -442.17245   -388.18211   -249.86023   -274.01759    -386.9383   -505.91396   -462.48804    -490.8849   -500.37437   -468.59498   -123.48561   -257.17949   -329.64144   -471.82185   -406.89849   -292.89228   -262.72876   -116.63692   -467.37324    -84.30689      -5.2884     536.4725    773.04035    640.32914    240.94367    180.46266    -341.9047   -345.71798   -332.30733   -529.46193    -342.3456   -594.91611   -224.21434   -421.44035   -412.69262   -398.67239    -440.8209   -263.99409   -390.41727    -344.1366   -417.60795   -366.65162   -364.81271   -302.38464   -472.85814   -235.28477   -306.85704   -498.52911    -339.6941    -167.7244   -417.08063   -166.36761    132.10679    454.39239    677.05475    399.14646    225.14244    -93.07039    -234.9803   -277.23366   -254.19907   -337.77339   -369.69874   -320.31664   -156.36354    196.53728    566.63117    850.46184   1171.59395     904.6284    835.25115    372.89523    287.79169     -1.47622    -19.90466   -186.13173   -320.74865   -345.75342   -456.13086   -321.90302   -261.49681   -409.52883   -293.17606   -322.83667   -199.29352    -75.25387     87.63844     318.3807    341.23024    243.44492      90.5682    -65.66896   -156.35763   -315.16733   -368.59935   -284.69383   -384.04169   -157.26754    -130.3192    -16.10744   -214.70986   -336.84535   -467.27403   -198.76326   -426.96395   -304.49415   -369.62297    -420.8508   -273.32471   -310.36357   -530.93777   -140.03754   -149.05615   -228.46436     16.31631   -144.28314      6.95885   -336.80715   -264.62568          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       46.00   -404.52694   -383.02373   -566.46281   -381.95188   -547.00896   -440.71535   -287.24502   -229.45432   -560.90495   -490.63845   -388.06463   -320.03521   -540.29503   -327.47344   -163.15401   -315.89963   -391.69677   -508.25344   -306.38598   -335.02439    -367.2704    -389.3979   -511.83427    -17.58702    145.00271    371.32304    570.21454     537.1738    318.62033    106.78373   -324.46304   -448.99645   -414.86678   -370.51704   -359.84234   -405.17889   -424.55231   -326.36785   -419.24118   -553.84997   -218.22034   -353.87819    -134.7139   -153.24166   -336.44446   -255.88017   -297.48362   -410.70334   -312.23731   -322.46415   -286.78953   -507.77987   -361.04452   -348.17974   -299.54879      49.2807     51.75685    368.12609    707.43915    216.42441     94.00178    116.15258   -251.17365   -242.18949   -328.46386   -317.71475   -391.62076   -319.52596    -37.69349    359.23646    507.21428    785.55447   1287.30652    942.13065    775.20397    405.58614    394.93238     79.23509     -9.55953   -308.50916   -126.80684   -228.74637   -244.71177    -281.3455   -262.47661   -290.27211   -280.46359   -255.44548   -225.71698      1.42423    114.17533       9.9338     327.0481    387.14466     126.8895    -15.06073    -45.55744   -210.35798   -285.78761   -310.37557   -337.25701   -205.91065   -116.16666    -98.58739   -142.82204   -181.29256   -306.32323   -377.85592   -352.05518   -462.64313   -439.33218   -244.14604   -452.90874   -354.07394   -367.98948   -170.17789   -217.82252   -119.30043   -123.14282    -48.19969    -93.88172   -225.47496          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       46.50   -378.78045   -454.47292   -493.55856   -408.53804   -416.01423   -418.79846   -246.63508   -244.81152   -745.91018   -527.78147   -254.52096   -387.46796   -354.21437   -385.43374    -128.1546    -357.5032    -426.1992   -455.68786    -272.7318   -393.79975   -392.46029   -659.89707   -438.45001    -271.9258     65.78844    216.94643    575.41828    514.19796    384.84949     70.06221   -153.17837   -331.45708   -406.84386   -467.91358   -409.31589   -282.55591   -258.00369   -446.39962   -452.63596   -662.58285   -293.77471    -152.1001   -283.52865   -448.74087   -393.33925   -334.67408   -167.78326   -346.77308   -404.78259   -273.81178   -277.52143   -316.95366   -355.62784   -397.97379   -216.50383   -133.49547    233.04876    306.66815    638.93893    562.48665     93.25621   -138.37121   -400.63202   -443.83094   -359.47159   -366.72649   -274.88409   -437.68468   -102.50679    255.65074    503.17538    846.83509   1152.16342   1222.72118    973.79117    433.73722    184.36985    141.09058     11.50609   -220.49019   -159.64565   -424.25938   -257.66823    -593.8791   -151.50241   -272.73239   -300.16534   -345.23719    -18.88421     75.87258    -17.16134    395.43473    454.59407    155.72174    107.77883     78.24226    -291.3285   -512.71153   -408.82295   -428.81212   -338.40773   -309.86344   -264.77337   -230.00812   -261.93516   -186.59588   -238.14242   -180.76899   -383.44045   -405.43292   -566.23195   -317.36031    -330.9236   -316.66107    -214.6755    -258.2503   -186.62694   -133.25121   -133.37849    -50.98121   -284.15674          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       47.00   -389.43593   -490.89776   -399.67352   -438.88053   -326.51654   -552.29157   -275.04163   -298.33363   -496.40375   -513.98133   -201.41032   -338.21359    -88.21662   -334.00121   -123.11839   -433.75354   -195.67502   -397.37203   -214.42303   -453.65192   -513.65998   -418.62633   -546.27889   -191.95931   -238.02405    172.00953    671.69488    544.34495    447.70149    109.17558    -82.26544   -199.33022   -471.52344   -237.76626   -278.87749    -489.7006   -225.96774   -277.63608   -395.15412   -262.83408   -412.77142   -277.91681   -217.76463   -322.93675   -282.13327   -436.32831   -214.36218   -287.26154   -241.55485   -406.27603   -306.80309   -421.01243   -385.70174   -489.28292   -356.68075   -105.18982    154.38581    352.10087    625.48795    466.36853    103.16835   -147.73249   -190.58529   -274.70976   -348.56848   -357.33452   -244.56177   -217.44884   -158.75803    -24.60073    346.87867   1067.37549   1121.21972    985.65517    829.29389    585.97964    -25.82771    -24.29667     24.48722    -180.0279   -130.41542   -340.69618   -385.54898   -457.49738   -339.73954   -486.82321   -272.22458   -385.82278    -98.14599    -80.43825    238.58603     94.46418    389.84893    460.22925    -92.62559   -159.80408   -129.76032   -381.45295   -323.81555   -454.89265   -228.67434   -254.30454   -188.10956   -122.41115   -160.07015   -219.37154   -464.62809   -274.05081   -410.63012   -378.95067   -407.25298   -357.39202   -423.52386   -329.00731   -214.91974   -251.14813   -206.35142   -122.96278   -185.93925   -330.23017          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       47.50   -490.81496   -468.64315    -292.1993   -402.58728   -322.67741   -623.10886   -344.03637   -300.16375   -287.70271   -294.95319   -194.19853   -344.80679   -223.75608     -93.7156   -200.05979    -523.4399    -99.86695   -259.63536   -407.98388   -345.35874   -439.95845   -206.43537   -367.34971   -148.07663   -134.22167    435.14078    721.43536    677.29066    347.08815    183.12863    -98.94334   -225.43917   -363.19242   -342.97539   -152.20184   -519.96332   -245.15391   -491.87691   -348.92576   -303.68074   -465.72105   -352.47592   -480.07803   -319.84563   -377.65917   -613.71111   -224.54794   -263.29842   -418.82223   -306.21527   -322.35704   -555.07307   -454.10864   -270.40435   -326.97087   -124.36317    294.47543    314.13846    536.54051    437.57591    187.82352   -118.63645   -352.88907   -208.85367   -203.58698   -272.82314   -436.26368   -382.84026   -259.87514     13.70412    770.08819   1041.93178   1254.36369   1016.61412    677.97773    202.38953    164.04077    185.50107    -59.28707    -230.5463    -310.9534   -416.82242   -487.10016   -415.54711   -535.79476   -373.16505   -464.99007   -288.23133   -375.64982     -68.9816    106.73928    216.78541    418.55067     217.7643    154.26495     86.71896   -321.47217    -277.0684   -384.22174   -392.53305   -313.05784   -230.26501    -98.05688   -238.71139   -185.35607   -154.18938   -346.35429   -259.42921    -383.3549   -362.05611   -437.81864   -398.73298   -341.71231   -514.74328   -327.98698    -301.7716   -205.05271   -367.88583          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       48.00   -642.34731   -522.41533   -248.53064   -344.97888   -326.45207   -522.31946   -402.08229   -314.16239   -189.23995   -254.46329    -97.51017   -466.51039   -242.73728   -213.27005   -201.34587   -456.08269   -241.31339   -121.50436     -460.113   -212.59901   -546.29332   -160.03465   -314.59804   -184.91649    -16.22857    441.98061    468.67122    527.26261    241.56604    -12.81459    -85.03878    -104.6953   -138.05089    -359.5023   -375.08336    -212.2352    -377.1452   -606.54761   -319.55205   -440.30706    -397.1183   -494.75742   -540.21989   -276.65257   -414.50153   -394.99155   -295.28647   -315.84515   -511.22506    -311.0296   -111.09786   -527.02211   -310.06716   -394.14626   -226.62516    -60.27345    202.86584    421.31378    453.45637    316.61279    161.03998   -174.87196    -449.4946   -164.31232   -370.58495   -347.32002   -333.46624   -239.93795    -251.6988     74.59984    491.43609    884.76329   1042.38882   1243.05369    651.71383    311.68262    133.45674     33.43092   -317.67426   -214.40911   -234.59177   -536.90064   -323.10494   -486.61579   -213.49911   -347.08029   -251.14318   -348.88092   -279.51152     97.01958    174.72361    365.87213    538.94711    365.17012     49.99916      -5.5667   -233.35952    -83.68246   -267.29582   -310.10009   -188.03177   -223.37216   -215.66033   -276.79827   -392.83892   -209.41855   -145.62219   -364.71764   -313.53049   -116.26065   -364.09362   -331.74391   -217.22801   -434.29398   -493.20326   -482.04064    -381.5738          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       48.50   -570.91517   -652.02128   -285.03161   -384.18113   -312.11456   -455.18303   -315.65329   -415.35049   -304.50181   -381.08521    -103.9985   -636.50076   -261.95719    -91.21668   -250.36809   -317.45776   -179.04035   -226.45548   -377.06917    -52.34663   -461.31071   -108.77533   -230.12543     -299.192   -164.13498     66.80097    417.63747    363.92771    147.96963   -206.15381   -137.63159     -170.777     -98.5487   -275.26175   -274.47028   -227.87382   -488.43153   -351.57973   -389.02982   -404.65261    -417.9655   -592.08066   -357.33372   -408.52075   -265.64011   -275.29215   -407.60903   -459.65611   -576.36926   -504.38027   -369.05511   -404.22851   -454.21615   -194.81172   -279.83899   -154.54925    127.30875    567.04425     607.7722    430.27807     -1.52617   -113.04985   -337.66118   -133.20649   -389.38579   -323.49793   -389.83222   -382.81766      3.25237      60.7299    371.87305   1045.99429   1077.79575    972.99685     879.2088    447.32078     90.64348   -120.81506     91.78127   -265.48145   -268.97037   -257.77907   -493.23688   -440.89875   -361.74306     -9.65605    -410.2593   -365.38557    -229.0575     47.12641    -48.32425    207.16231    429.54089    365.59446    237.76302   -164.82283    -364.9073   -293.28114   -309.54001   -337.38538   -306.19107   -369.96827   -180.43916   -427.06197   -297.71968    -72.96294    -292.6537   -368.96237   -362.75846   -211.24433    -308.1138   -532.21494   -387.65589   -410.37713   -532.68819   -415.99871          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       49.00    -487.6759   -680.54839   -315.32613   -556.04863   -284.21596   -450.77043   -347.05384   -490.47223   -460.81981   -295.89394   -202.98941    -638.4897   -278.83675    -201.0812   -203.40294   -350.83616   -116.06191   -280.92841   -300.31709   -282.14166   -191.21777     -90.4555    -47.36876   -302.43672   -122.03331     85.68984    369.53791    445.57393    130.74953   -192.27952   -258.92603   -259.16918   -116.54737   -368.98504   -316.48476   -258.08441   -290.80036   -374.98569   -445.29465   -723.91303   -488.33707   -298.69694   -427.96139   -376.71943   -302.69724   -464.36141   -297.65094    -431.4998   -435.87516   -420.57873   -289.21658   -316.56689   -141.41717   -357.07924   -135.03881   -176.02783     175.2151    311.14561    679.68688    449.58484     416.3203   -117.39876   -257.63064   -208.85169   -156.30503   -267.76196   -319.95847   -138.79678    -68.25169    320.41089    570.15247   1147.67219   1154.25661   1127.64854    967.30388    333.96502    176.97405    -84.40979    -66.82421   -233.59969   -220.62874   -238.20442    -447.2092   -349.72521    -482.1222   -275.41133   -467.65097   -314.03327    -100.7693    -25.66377    165.18048     40.25818    403.81067    228.32436     30.25433   -199.76112   -318.54137   -397.68106   -568.34746   -263.98136   -363.44145   -340.39737   -351.90289   -263.20872   -212.68353   -262.27813    -250.3256   -314.82222   -328.98345   -306.59435   -405.87522   -431.76183   -353.01068   -330.60909   -321.80486          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       49.50    -443.1569   -402.84293   -334.79863   -562.76333   -222.49495   -348.69319   -515.93882   -514.47427   -581.81988    -216.8022     -224.431   -443.68726     -336.236   -418.84746   -486.06861   -411.86858   -233.12786   -324.04727   -274.50612   -528.16251   -322.54312   -235.66402     -32.9343   -243.01055     24.27157    147.27925    402.49917     715.0317    110.06303    -44.67463   -208.50052   -391.89729   -357.16821   -394.71964   -235.80037    -348.5984   -371.08343   -426.79316   -373.24446   -596.18688   -262.19097   -307.61157   -399.33696   -568.06954   -568.55778   -413.82359   -309.90285   -376.91953   -252.21946   -310.80785   -516.25583   -395.78438   -275.80878   -242.39354   -499.22729   -151.32448    240.93145    354.46763    367.82428    526.60388    132.28294   -229.44398   -253.37375   -272.64594    -161.0919   -242.00151   -408.07382   -231.54227   -303.07742    277.71457    286.28811   1018.23637   1032.44109   1050.33908    339.82801    351.64536    115.03933     -97.3077   -218.58462   -218.16551    -424.6509   -426.95121   -295.76312   -354.04498   -351.50159   -250.18357   -391.85969   -284.63052   -216.64093   -108.82334    197.74854    270.81143    420.32332    298.03267    169.38064    104.08229   -165.27929   -248.90289   -292.48576   -366.83217    -320.4239   -451.70753   -139.42544    -218.5388   -207.64808   -251.55353   -434.70131    -344.5416   -377.23454   -349.04516   -529.53999   -486.79576   -405.64009   -413.72985          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       50.00   -437.21057   -214.00269   -371.10561   -396.39691   -205.80053   -201.11504   -586.73507   -515.59958   -468.99229   -282.02956   -124.42635    -267.2782   -423.48449   -474.26319   -694.74563   -376.41047    -292.0836   -594.55663   -235.77026   -408.26671   -457.59053   -285.18067   -237.68496   -152.91458     95.58575     41.69066    546.96583     539.5324    204.75168      19.8957     31.81203   -351.77956   -178.95536   -495.43454   -109.92406   -242.20984   -404.58444    -245.1685   -344.42022    -318.7281   -519.32641   -261.76715   -427.38767   -341.73064   -485.91014   -341.95307    -387.3796   -353.77124   -218.66583   -396.08462   -364.48756   -494.23756   -333.52648   -445.72449   -335.80043    153.26025      98.9986    368.60769    652.77553    460.53214    114.89608    109.87144   -262.88629   -319.08812     -203.969   -408.89661   -150.02967   -257.12169    -77.27637    140.88727    455.36556     1171.585    833.71319   1013.86684    878.74098    170.61431     98.32871    249.09651   -230.03626   -337.71499   -460.66822   -412.71928   -171.35529   -383.81557   -525.29884   -315.53804   -417.51092   -330.58072   -365.42884   -124.04826    240.72738    414.39031    371.85648    561.60595    527.40523    325.66024    -25.69239   -335.33423   -422.50628    -406.6458   -177.94935   -294.94713   -220.22224   -277.89155    -212.9466   -200.36805   -234.29059   -390.75198   -408.17117   -352.64225   -465.12486   -502.69253   -393.93495          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       50.50   -273.62225   -113.65884   -427.49146   -336.25669   -299.78047    -50.94239   -622.46903   -460.74526   -318.98669   -236.18999   -179.42916   -245.17634    -508.8404   -308.48052   -480.26817    -230.2401   -167.20585   -456.86976    -254.2923     -285.179   -190.17007   -430.63265   -434.01551      88.4491    -28.34501    160.05938     389.6716    416.00214    259.48907    145.00727    -51.21621   -310.40824   -392.48226    -334.7224   -284.34296    -287.4076   -374.99039   -345.44335   -270.88983   -387.70944   -465.57933    -387.6813   -120.17007   -472.47108   -457.71786   -407.28711   -355.84876   -542.77063   -233.08393   -429.36748   -403.46065   -306.06392   -356.11201   -286.61394   -107.97186   -109.04442     97.14955     199.3552    411.17605    530.83274     94.93568    -28.75568   -320.53023   -247.44343   -170.07531   -332.49104   -330.94646   -121.15516   -220.70177     44.48806    223.29461    972.95205   1265.02168     916.6427    524.05211    396.74068   -164.86377    -14.67139   -247.46452   -384.84831   -377.84886   -418.22403   -355.62575   -192.35688   -397.80011   -167.62699   -351.37364   -199.50284   -262.84664   -196.07763    143.72174    347.40584    445.71433    592.35142    522.72984    288.39329    -41.11941   -439.82861   -331.65212   -248.71407   -401.77138   -326.64373    -292.2696   -256.73931   -180.78984   -244.69246   -194.33376   -379.52167    -482.6909   -344.00378   -297.13042   -416.31325          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       51.00   -103.81849    -74.80952   -521.13629   -440.73485   -378.00196    -16.16113   -695.05487   -327.80837   -229.40677    -96.29022   -379.39662   -347.85335   -433.89031   -178.22329   -281.20463   -338.58602   -254.30788    -349.4834   -398.61961   -336.91645   -245.84396   -573.74851    -317.8558   -126.39847   -131.40048    254.67398    218.06704    460.06741     196.4012     82.26901     -63.8438   -132.05245   -360.31564   -270.35312   -366.44683   -476.13477   -502.49576   -507.43502   -463.82977   -563.37018   -352.51801   -553.51633   -156.73438   -411.95025   -218.93289    -327.0181    -334.0471   -406.30196   -256.39119   -486.57465   -448.77651   -249.26928   -223.74893   -367.45451   -226.16366    -177.1928     75.68443    282.09828    423.17459    461.53846     90.27099   -167.18183   -192.69967   -210.99822   -488.01392   -460.44447   -213.69234    -70.11665     50.17337    361.56279    839.08712    853.81886   1112.88689   1058.22454    738.08996    341.48954     66.41462      20.4463    -329.6628   -233.95574   -743.07018   -518.44157   -712.54266    -582.4073    -319.7381   -649.14803   -257.13887   -342.89446   -231.50281    -294.6407    -67.02352    294.88706    323.74467    644.85346    296.15295      82.8418     -9.81699   -172.41004   -235.91304   -329.59208   -394.80129   -214.39466    -15.08705   -244.30129   -146.29979   -206.01725   -272.94962   -335.60235   -361.05446   -345.76346   -629.50285          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       51.50     57.75871    -91.87378   -414.47474   -467.11922   -422.16069   -130.28061   -634.03214    -198.8471    -235.7978     -10.8759   -391.85358   -400.80468   -341.48357   -131.02447   -323.69504   -532.70683   -437.28297   -482.18545    -367.4511   -317.53419   -425.68365   -412.75607   -232.70672   -330.54807   -107.90515    258.31139     235.2293    434.32628    311.06411    100.11588     54.02349   -241.14024   -401.37842   -471.17581   -412.93704   -483.56529   -538.16733   -350.49937   -355.16122    -381.3852   -301.51387   -458.58994   -373.51213   -423.52523    -231.6181   -192.43661   -291.02672   -419.47537   -241.04712   -335.36132   -469.57318   -379.37624   -144.55076   -407.75059   -384.26106    -26.79871    -64.65545     143.7195    539.38157    581.58887    309.66496     23.43067     -144.205   -335.35515   -135.14522   -235.95137   -275.97445   -232.55072    -76.66403     -9.56488    551.07638    992.70181   1079.99468   1008.92271    667.21379    183.61214     22.70738   -125.94818     59.04658    -263.8242   -506.17529   -265.49282   -584.62337   -525.57519   -393.51171   -375.73363   -193.47342   -369.36076   -226.27868   -261.99719     30.87688    221.14432    524.73454    452.22578    265.46591     117.4644   -160.37971   -168.73914   -288.84972    -264.7681   -529.22323   -286.33145   -138.52149   -200.30764   -305.25564   -253.36797   -190.96859   -408.31521   -397.59555   -432.72269          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       52.00    136.57365   -174.28837   -272.05025   -353.05857   -431.27326   -361.02995   -530.26486   -286.74054   -376.86187   -102.10208   -380.91903   -140.42668    -311.4291   -281.18221   -273.51618   -441.93978   -458.12672   -431.47326   -174.89395   -228.45744   -410.41055   -222.81329   -211.17141   -182.18549   -198.58223     84.29229    541.34779     402.6603    230.58851    231.43511   -102.60922   -207.32493    -317.2221    -542.9356   -417.80453   -504.54647    -371.7906   -353.86274   -356.51178   -380.39066   -221.40264   -240.43859   -334.89576   -570.82498    -386.7406   -503.53631    -381.1662   -291.71217   -428.65697   -318.90797   -333.79514    -360.0443    -206.8636   -327.06654   -401.60169     55.37846     -22.1918    527.28628     609.6433    188.21926    331.47134   -115.38214   -277.42312   -129.21429   -165.67923   -329.76444   -551.32829   -276.05939   -236.95041    -75.09093    338.56304    816.40141   1169.20864   1063.09288    858.00278    437.57823      89.9241    -95.74027   -433.18596   -195.30743   -695.29506   -416.28699   -224.40538    -388.7074   -437.99722    -463.9328   -342.61009   -421.66315     52.19637   -125.64048    363.94154    473.95996    529.51076    337.41647    276.34671     98.41319   -110.05981   -210.38187   -441.00655    -333.3023   -298.57075   -358.36911   -394.67263   -199.62169    -209.8202   -301.22588   -322.62547   -239.45675    -473.6734          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       52.50    -65.49053   -354.68226    -82.03901   -215.42972    -446.6744   -454.60346    -554.9645   -322.57362   -469.32955   -287.28677   -440.05829    -46.70612   -317.75677   -455.25457   -227.66661    -395.6054   -413.99909   -364.00244    -57.20453   -206.43972    -332.1907   -327.99404   -194.33996   -344.61399   -120.50721     139.4641    347.52842    259.83477    452.94199     94.78551   -111.46165   -148.91673   -517.53034   -546.64691   -393.78198   -174.78351   -354.76983   -253.25834    -332.7043   -364.12517   -256.11345   -335.55777   -228.26543   -375.32025   -445.48804   -427.30482   -493.20268   -213.96347   -308.94645   -301.54523   -312.66706   -208.38654   -313.05207    -318.2364   -309.66991   -298.18121      9.50019    342.21818    440.79626    273.72602     63.60562    -77.40714   -110.60163   -218.07213   -351.37878   -283.20913   -217.14521   -298.83118    -182.6549     -28.8163    602.55399    952.33164    961.70908   1103.01064    887.87006    525.53495    215.65542    177.82157    -47.89703   -400.06124   -407.37359   -331.80339   -498.01654   -467.84977   -276.49701   -266.59139   -306.51342   -412.35747   -226.22956   -116.91114    163.77023     424.2397    323.41212     412.3246    398.94605     23.43035   -280.50276   -303.85657   -347.90745    -327.1387   -364.03365   -360.85696   -286.43076   -314.47692   -269.09875   -319.76638    -492.7317   -406.71515          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       53.00   -314.42922   -564.02246    -92.06491   -155.73351   -468.74242   -406.42915   -611.11021   -144.74065    -517.0569   -342.25346   -511.86958   -297.96373   -313.97648   -424.37148       1.4601   -455.08877   -393.65573    -318.6301   -395.41559   -201.70806   -327.42023   -143.40541   -256.18799   -262.67163    -55.86475     119.2934    589.63817    467.89395    334.74356      7.17534   -175.35951   -297.50288    -331.4257    -457.9354   -300.09101    -196.3479   -156.83225   -297.84695   -322.97384   -259.62532   -160.28834   -355.82177   -374.07682   -101.82659   -312.14949   -464.97539   -491.33142    -416.6839   -249.65593    -427.3683   -331.14657   -473.27251   -481.22154   -480.80424   -267.17812   -214.59806     34.33061     393.9088    524.63641    228.45569     187.6734   -176.29985   -164.53353   -393.66838   -413.21645   -241.16154   -264.41939   -295.26726   -202.92852     18.19569    462.07613     827.1414   1270.88751   1397.14344    788.09649    596.72522    175.63298    -35.98661   -137.90597    -73.14716   -202.67294   -495.12359   -393.76838   -390.47362    -220.0102   -373.53053   -317.15967    -550.7626   -105.57651   -360.01304     -50.4407    460.38547    497.94999     415.2893    159.65778    -57.65639   -119.37178   -373.95603   -394.70446   -438.52543   -366.30034   -259.03864   -281.58983   -297.40435   -161.53666   -388.40352   -368.25165          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       53.50   -491.76979   -658.26918   -179.62508   -257.23653   -480.09614   -414.30978   -569.21953   -123.36346   -395.92698   -300.62043   -435.15922   -450.54575   -177.67959   -415.80589     132.9279   -378.47691    -354.1519   -505.66858   -411.04774   -206.35128   -408.43746   -223.48228    -312.0898   -251.33523   -125.49772     72.75841    239.38732    280.95141    265.03457    -28.62421   -228.29792   -251.91521   -313.07494   -251.93562   -220.32762   -265.18562   -368.23181   -195.32802   -257.20359   -636.01418   -498.98189   -368.72075   -433.99998   -365.20109   -418.56295   -574.12452   -413.53138   -280.96603   -499.16726   -397.90777   -332.30508   -458.22379   -401.20564   -309.52507   -191.65506   -322.91082     52.21442    224.80397    459.03716    347.73334    159.39346    -55.21393   -236.02797   -323.09849   -360.79516   -356.33657   -300.66875    -70.51606   -218.80905     26.82118    454.89198    630.01374     926.0712    993.61682    782.54136    185.61061    130.59252     37.24118    -60.30885   -274.90243   -537.11314   -406.53871    -80.38277    -335.2186   -389.88626   -587.28288   -379.44694   -245.81478   -238.63433   -115.58392     59.34684    394.16329    282.89145    371.42986     281.7368    -47.40795   -211.35082   -379.63219   -244.28125   -364.55487   -376.78399   -408.96233   -316.36356   -291.60058   -166.78379   -415.82234          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       54.00   -505.13276   -623.23358    -240.1465    -256.9291   -403.24087   -472.25328   -386.04801    -261.0645   -319.13586   -361.57808    -330.9021   -406.28591    -37.64105   -585.31231   -121.85052   -235.58074   -400.47899   -539.24723   -238.13532   -196.99056   -408.12853   -198.40093   -218.83438   -422.10235    -35.89894    148.49666    409.95614    224.23291    245.04113   -125.02338   -171.57672    -97.07728   -396.40364    -177.8429   -490.15811   -380.07358   -363.04214   -362.90162   -421.50236   -502.27755   -476.38124   -546.65732   -293.43459   -404.99216   -542.60401   -534.97912   -383.34911    -73.65928   -360.11874   -429.23462   -334.93101   -338.77895   -260.86979   -244.84286   -226.64742   -126.46969    238.95063    265.19713     406.8719    181.80309    231.97179     -10.9898   -257.19685   -259.44053   -254.05414   -222.53497    -242.9858   -277.98123   -139.83327      7.09303    432.83518    822.17927   1053.97035   1158.07176    705.33584    275.45375    407.60057   -131.72884   -140.39599   -160.31414     66.46877   -674.71283   -569.49358   -365.40354   -347.45439   -194.94494   -275.95517   -190.87833   -209.86108   -142.51207     75.83066    352.47457    483.02077    370.02481    519.84147    231.76825   -251.86505   -499.54969   -472.06101   -362.12214    -325.2214   -326.51414   -195.41538   -200.31754   -468.47469          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       54.50   -414.31824   -503.89606   -334.10224   -155.63429   -375.51314   -500.77423   -189.12834   -386.91979   -357.53524   -489.65316   -345.92145   -366.55402   -228.44374   -588.88654    -62.30084    -79.03502   -524.35395   -411.58385    -257.9058   -164.33809   -364.01015   -270.12599   -222.18296   -473.53575    -60.79965    303.78672    308.04508    391.07153      30.4378    -54.69324    111.77512   -512.11987   -293.51085   -160.62396   -403.76194   -361.57147    -257.2205   -353.18532   -420.54346   -529.13888     -457.588   -529.90782   -379.30874    -535.1997   -416.41281   -506.69103   -385.34871   -272.27134   -339.59123   -459.38487   -278.01326   -425.66463   -353.32287   -283.74777   -191.93708    -52.24124     73.55128    382.20911    222.11186    266.87768    111.07691     -66.8413   -328.22481   -291.18282   -245.37259   -325.00746   -305.23699   -307.46513   -134.40938      3.92024     303.9215    763.23315    922.45024    875.58483    817.71807    519.89993    244.52223    -26.32978    -50.94558   -117.65733   -369.76245   -495.93304    -819.4912    -445.2789   -234.10801     53.58154   -232.49149   -268.19122   -365.90234     -9.38845    -80.63683    227.95708    503.64265    680.95832    258.33417     63.46189   -274.24857   -325.70347    -515.2992   -611.76762   -390.92816   -383.91866   -341.72168   -371.15365          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       55.00   -322.95316   -493.88457   -402.75721   -258.06236   -350.09978   -579.55525    -134.3674   -588.64137   -407.67502   -598.40646   -363.13619   -414.25788   -381.09653   -410.46118   -255.55933   -151.27281   -524.65997   -402.81937   -465.76985   -431.92052   -425.85554   -423.81231   -271.00164    -435.7967   -273.87692    166.98671    189.61786     281.5147    -16.44385       -8.367     34.33617   -427.90333   -335.27339   -286.00875   -395.73755   -343.41939   -431.61192   -355.25635   -523.05085   -524.64534   -409.16154     -87.7869   -520.79516   -510.48753   -534.71872   -399.28887   -267.86668   -240.57211   -448.69999   -402.56067    -256.5974   -394.75072   -326.31416   -380.66405     -243.005     41.79771    112.59537    350.18748    337.15038    432.99815     275.0002    -67.56677    -10.26662   -304.87023   -154.63107   -290.28109   -199.18862   -287.66443    -232.4965     -9.86596    460.44605    738.27444    821.00064    769.92728    724.94411    219.34933    172.60972    -71.49496   -111.75263   -247.28936   -305.81825   -420.33431   -518.55297   -294.75038   -268.60806   -351.76692    -340.0719   -296.52834   -227.49418   -168.91576    -59.55894    516.19352     581.4394    374.50341    535.78047    107.04945    -316.6212   -503.62674    -446.9287   -442.45407    -385.1911   -368.19287   -365.41698   -442.78176          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       55.50   -338.94085   -592.19909   -468.86485   -346.32959   -245.83672   -555.00094   -199.48284   -793.61602   -527.11645   -501.51155   -424.75062   -405.10017   -427.43503   -224.01487   -406.06388   -227.10813   -334.53282   -399.05681   -483.07461   -584.54287   -304.69854   -525.37077    -166.1841   -329.01282   -182.03458     81.11101    190.28302    428.24204     79.74044    -154.3618   -239.78336   -353.77463   -378.06596   -123.08536   -448.90771   -468.62603   -476.40835   -497.67239   -497.23651   -289.20199   -335.64533   -215.47108   -557.65509   -370.38539    -539.0118    -268.9643    -394.3002   -555.79056   -407.01838   -339.36074    -465.7225    -298.0266   -454.56357   -318.00895   -216.34838   -155.10636    157.00867    237.78165    569.90401    198.42279    184.85553   -132.81255   -273.36414   -490.00107   -214.35168   -196.01599   -328.44498   -248.93237   -127.67035     88.46111    506.25402    734.69353    965.56627    875.14165    717.93238    702.42771     139.3318      1.89014   -184.28512   -166.65934   -350.02227    -355.1879   -440.52735   -311.92293   -307.02587    -694.8275   -258.78822   -366.91784   -312.50392   -154.51539     -2.54488    168.97594    370.54369    277.38126    570.71425    130.62268   -181.08724   -484.43576   -394.87495    -396.6386   -539.54522    -437.3574    -362.5041          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       56.00   -379.76876   -497.27698   -468.50274   -326.24519   -192.91156   -576.44854   -249.75704   -502.39093   -499.07246   -402.31849   -391.95927   -399.92221   -472.06237   -164.97501    -506.5113   -403.36127   -202.06901   -459.64103   -308.74833   -493.61371   -166.15581   -366.69781   -281.62825   -240.56021    -58.30855    -15.02152     516.4238     75.79333     59.31751    -78.83507    -307.5854   -403.50815   -584.95792   -253.32724   -249.90939   -420.21643   -530.62299   -462.12377   -519.76869   -438.97224   -202.78441   -346.77268   -399.82823   -302.59394   -514.78421   -209.98628   -225.47306   -270.65231   -495.92194   -254.68981   -505.77448   -472.67059    -270.0827   -334.41714   -258.14366    -56.41656    -79.39664    438.73187    303.44449    339.82489    183.97275   -215.71556   -197.51567   -240.59093   -100.49704    -432.0882   -226.05728   -298.75271   -229.45043     123.3376    347.78814    719.92987   1161.21984    869.71684    667.09035    625.96812     80.54564   -269.21066     10.98708   -379.90154   -410.34468   -216.57989   -487.41784   -657.92907   -320.87449   -265.70241   -198.47703   -377.70957   -185.73556   -161.74676   -158.85571     79.00124    409.03742    337.96627    420.48238     85.25132   -223.72501    -257.9686    -269.5373   -604.45456   -389.06282   -537.91564          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       56.50   -296.28587   -357.49469    -290.2111   -196.54346   -254.75575    -590.3363   -271.21785   -295.16275   -388.40992   -177.46248   -357.24699   -453.23726   -459.39542   -118.88578    -459.8993   -500.66917    -93.95001   -425.39536   -187.90235   -557.43088   -346.77834   -398.55561   -281.80042   -315.80405    -149.5573    272.77544     242.4261    159.31803    122.71903    -74.85314   -293.23512   -231.39689   -456.59571   -261.12752   -549.58662   -431.85188   -556.16935   -367.95474   -587.76479   -394.23503   -411.23234   -441.81488   -535.59904   -294.08022   -541.94572   -367.34209   -376.06053   -318.54297   -342.70675   -422.00099   -323.98322   -457.26522   -157.96163   -478.63441   -398.84306   -269.80312    -76.02257    408.18745     167.9063    326.64915    173.99196    -181.3812   -195.17316   -302.86675   -312.14123   -189.99152   -429.14153   -250.55616    -194.1048      5.85425    467.38752    745.53649    836.99373    942.59323     854.9745    656.25666    270.56968   -228.20805    -34.39929   -302.30401    -125.0269   -386.56412   -505.34114   -402.77322   -292.31892   -364.92477   -310.32539   -161.85089   -260.93671   -185.15444    -85.87456    -53.70521    243.05873    355.06482    270.92593   -168.98257   -196.66087   -332.16769   -431.69006   -333.21843   -494.10022          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       57.00    -89.83153   -232.66527   -174.71136   -145.53276   -264.65972   -477.04246   -197.50043   -294.77196   -338.24885    -10.53062    -413.4479   -437.52227   -456.13828   -228.25969   -355.53138   -417.06209    -31.32548   -367.39261   -273.72567   -553.81921   -441.51281    -400.4537   -311.12397   -416.64222    -62.46134    324.83213    241.28624    398.22008    232.79049     44.13087   -294.82714   -545.75239   -253.46079   -277.47683   -569.77277   -510.62318   -175.60421   -360.75418   -228.46975   -427.86259   -439.32569   -469.47971     -515.402   -371.06321    -438.4926   -307.23319   -474.09229   -474.51126   -301.70934    -388.5207   -482.27073    -528.2931   -297.07331   -178.34379   -313.40861   -168.17938    -16.94769    192.19662     276.6343     612.7512     68.79104   -159.02669   -181.44328   -373.54405   -388.81006    -85.38705   -453.00726    -93.56331    -89.78997    -27.97582    488.68224    949.93927    635.75935   1059.26939    692.06408    609.23797     321.8299    -256.9356   -205.32149   -265.94151   -248.36808   -354.43676   -471.84489   -535.73295    -421.5429   -418.81813   -330.42698   -221.72655   -457.70268    -96.03473    -46.94885     86.96868    210.71798    632.01126    442.31703    103.83116   -156.98909   -231.93515   -370.58015   -315.38613   -421.29211          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       57.50    -93.23373   -179.50911   -173.45887   -198.36607     -192.315   -260.09157   -137.14401   -319.58376    -307.9888   -290.36907   -416.29659   -494.19193    -471.8951   -279.23631   -390.99776   -343.24048   -135.73044   -127.36869   -274.85853   -202.09561   -442.88064   -382.66589   -311.62192   -356.32703    -80.13308    189.29813    227.69235    258.80647    124.87287     56.88073    -65.44464   -408.92657   -438.57928   -303.49256   -341.93008   -486.87698   -288.84849   -400.97054   -257.86791   -507.31465   -275.55476   -428.42424   -446.69792   -390.69068   -224.83352   -341.17421   -410.44955   -332.41624   -318.72964   -359.30796   -431.93895   -297.13262   -529.95533   -447.29206   -300.25259   -335.15597    102.73856    290.77038    409.32756    268.49154    100.39397   -225.87618    -48.93071   -305.59222    -376.3779   -372.54139   -272.96452   -398.74821     -6.92903    119.58505    316.56972    857.83921   1098.01515   1014.17791    937.86931    465.75141    422.27395   -105.57191    -97.99804   -150.33573   -452.51766   -403.87735   -402.44318   -439.25562   -338.88479   -412.51832   -352.66455    -393.1786   -214.22568   -335.76231    -92.30071    169.38733    377.32564    396.65827    272.37557    214.17037     13.17313   -286.11748   -326.23452   -567.05808          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       58.00   -234.54161   -262.07098   -265.48844   -300.93507   -277.84422   -106.02853    -89.30981   -291.33269   -247.85928   -381.45802   -371.94474   -771.38821   -415.65551   -175.95768   -365.68265   -310.20536    -272.9482   -103.14058   -212.97568    -99.38202   -420.78419   -497.10853    -96.58172   -258.01472   -109.23988    166.82526    332.62893    265.26289    327.15587     -2.10364   -257.00197   -299.70899   -554.76135   -416.43191   -473.48701   -264.85163   -221.65946   -341.27315   -313.47389   -421.67085   -489.63538   -160.19498    -368.2749   -151.55191   -305.23449   -312.44401   -402.69491   -584.13522   -583.19566   -414.54897    -330.9653   -257.73943   -456.44414   -589.31667   -267.88899   -147.99765     81.80833    344.92148    446.33841    173.25925    106.89181    -67.18517   -192.08476   -421.00218   -317.32936   -329.45554   -202.21789   -196.62666   -197.87894    -25.25137    247.80775    688.61671   1105.17371    879.52428    894.53967    586.91157    208.35819    -61.98396     71.21265    -200.1515   -480.89759   -312.03788   -430.22809   -527.65426   -424.74297   -402.12912   -303.72851   -393.96722   -232.38542   -259.25529    -45.58581    227.45237     224.0078    353.66228     510.9933    140.78148   -181.99941    -198.8322   -452.47987          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       58.50   -345.75896   -347.48771   -239.88108   -335.71386   -365.73709   -102.87263   -187.44239   -286.61647    -193.9643   -334.40432   -341.91305   -659.55581   -298.72919   -173.61343   -323.49005   -210.48666   -362.93209   -187.05792   -376.72931   -256.34426   -228.04962   -248.86062   -342.01026   -314.57963   -119.89238    103.59531    407.35975    352.64249    423.38961    -78.58729   -157.07623   -335.48053   -460.52416   -437.81019   -302.21187   -216.91567   -550.26576   -278.75817   -252.34314   -410.42249   -480.78335   -298.40444   -325.48668   -578.96329   -537.55599    -294.2781   -507.67582   -288.76417   -467.91056   -578.95919   -341.66049   -143.07338   -339.89155   -145.65059   -497.71092    -94.91161    213.99415    303.73685    515.76928    190.30636    182.14382    226.20649    -239.2409    -91.96259   -209.15634    -408.5739    -94.45966   -397.85854   -217.69518     75.69555    593.78741    568.99184   1035.09102    971.63715    903.11798    599.81574    122.37101    -92.12441    -17.07846   -185.89805   -382.66915   -164.46809   -473.64635    -325.8432   -562.32451    -355.5482   -159.47628    -343.3632   -274.38223    -223.6751    -57.75543    150.83093    291.57733    546.03717    188.66317    -54.06218   -118.63138   -129.72471          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       59.00   -387.74379   -333.14436   -164.73836   -296.51385   -284.33077   -217.89153   -304.73662    -205.8822   -232.73601   -300.12906   -420.21422   -451.71757   -263.26762   -174.95368   -446.43186   -253.74174   -510.61821   -194.81302   -379.86547   -308.27174   -140.38764   -320.62555   -325.86738   -276.80499   -120.22098    147.66186    297.84178    132.95103    354.20197   -101.24084    -267.8135   -125.35922   -194.62419   -301.54395   -496.18213   -396.40283   -423.14713   -280.11336     29.56341    -433.1222   -369.35065   -580.21394    -64.48633    -399.4383   -390.39339     -367.918   -324.06412   -276.38756   -537.70309    -44.12768   -270.90237   -275.82587   -425.18882   -259.98307   -119.79767   -305.71489    118.53257     85.81739    434.82393    280.98013    110.83702     45.38797   -318.66131   -286.26895   -375.24675   -274.78975   -412.56508    -342.8239   -369.60726   -107.70532    281.33027    667.13505    724.31137    1182.1806     662.7574    614.74034    102.56741    243.41749   -153.12532   -359.34974    -190.3134   -116.78255   -357.15767   -235.66602   -385.10721   -426.36092    -285.3268   -201.05877   -137.32716   -207.22185   -181.60442    296.51642    144.62332    364.55071    207.42044    -69.11533    -66.84115   -345.74764          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       59.50   -323.11875   -383.65533   -198.52917   -172.22833   -210.32999   -269.06262   -343.20736   -213.17522   -396.30932   -273.95884   -512.59191   -377.24234   -469.19046   -117.04267   -511.04729   -318.79281   -378.95309    -99.21606   -455.77895   -358.32099   -260.63976   -468.59936   -454.22427    -342.7003   -254.96785    305.32513    101.14689    179.34363    -18.30055   -178.44565   -203.73633   -159.63621   -251.02791   -434.40226   -489.45366   -349.19763   -336.49719   -273.28832   -146.86225   -346.39803   -404.15205   -288.53716    -435.8971    -377.4293   -439.72226   -380.38883   -288.95495   -268.44063   -451.49849   -341.41494   -273.08651   -256.42557   -337.36221   -371.76136   -141.44912   -149.50395    244.42414    205.44724     524.3853    343.41608     65.49314   -130.68412   -216.57809   -268.70706    -308.8045   -331.99988   -255.72751   -236.27128   -220.17232    -51.76548    410.21189   1009.67325   1033.41577   1254.13386    717.45324    509.16596    408.46165    -87.42425   -158.18435   -269.63712   -262.06883   -150.67606   -230.85669   -378.74712   -310.78946   -314.86547    -284.8694   -517.89309   -350.49787   -190.60614    -11.60061     293.5136    415.58868    328.25544    299.76198     47.10963    -26.84772          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       60.00   -236.97793   -453.86748   -203.94897   -121.28393   -200.79466   -300.79559   -369.07032    -448.4472   -468.60184   -236.63493   -601.27084   -289.63376   -436.21522    -70.35592    -392.2317    -260.8948   -125.14225   -292.23434   -307.60103    -378.2841   -289.22926   -476.31398   -381.91624   -304.86228    -79.51585    -25.25908      24.3348    102.15643     49.62654   -148.53181   -210.52646   -202.67023   -357.21184   -434.71221   -601.67474   -252.30671   -352.27268   -513.36181   -550.08393   -377.78561   -677.39534   -316.70463   -552.02423    -245.7213    -456.3313   -270.12558   -275.09537    -89.05376   -406.02666   -391.31407   -338.21429   -303.75686   -353.80885   -279.62313   -293.23771      6.03741      7.96749    113.03841    313.31527     363.8195    134.94848    -163.6166     -99.6763   -323.02953   -164.31099    -90.80887   -246.27244   -320.68546   -276.06405    -53.77952    272.59906    663.27595     890.6416   1043.11599    859.81258    535.84573     339.4313    -10.34717    147.72216    182.19109   -216.08733   -298.67926   -482.84797   -623.21393   -328.86922   -262.07875   -214.86805   -425.62819   -346.86828   -126.71918     -6.57351    107.17398    254.05192    181.46785    370.09689    109.41466          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       60.50   -203.14324   -537.01859   -178.83618   -113.77103   -226.19814   -324.34678    -441.2191   -482.81255   -389.98371   -237.25282   -676.09552   -292.14282   -259.90828      9.09361   -340.12415   -257.15111   -112.00825   -445.85302   -297.98168   -289.17925   -250.25529   -414.41559   -325.46878   -176.91378     60.20983   -218.28159     15.00705    132.24766    186.01558   -135.15691    -231.4803   -264.99996   -374.80431   -401.61471    -354.9894   -306.77892   -523.51912   -529.67808   -406.71001   -257.30403   -449.48766   -242.54797   -428.38901   -336.76657   -604.02836   -457.32684   -549.51282   -398.78827   -319.06106   -405.26385     -463.834   -264.70649   -205.29743   -366.41353   -471.42984     103.5275    167.97019       99.924    320.14287    307.41096    163.25809    -63.53756   -214.40435   -372.92726     -1.23389    -511.0913   -248.97671      16.1631    -84.60691   -114.22251    348.94612    697.69447   1037.55981    915.73359      959.543    608.30231    311.88272    -23.01528    -82.40946    -36.67998   -114.70672   -388.74594   -416.18878   -314.12033   -430.60668   -394.20772   -362.61807   -540.07862   -364.28785     -5.85625    -92.16668    187.52885    329.59104    448.52904    211.70231      6.05801          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       61.00   -193.06007   -478.42803    -170.6408   -289.04103   -250.60056    -325.7278   -508.60657   -267.30768   -368.07407   -468.74579   -540.54309   -357.84781   -463.88725   -116.02659   -312.14506   -319.52778   -407.13567   -218.90261   -396.55783   -386.75034   -404.86739   -322.99776   -383.08656   -196.34634   -153.57227   -281.63111    332.40005     236.8687     47.76634    -151.3742   -102.63946   -274.48415   -320.07848   -232.81115   -345.80003   -577.98233    -295.5939   -492.87738   -444.68594   -299.97367    -84.03288   -615.84803   -293.09132   -500.45316   -519.84494   -332.62627   -407.51656   -477.41979   -479.00161   -410.16618   -536.17924   -536.54978   -403.56285    -332.1179   -316.77357    -91.78069     108.8516     32.20445    209.07573     130.9447    172.23804    -93.17237    -177.8839   -204.27053   -420.50743   -290.86465   -323.62809   -293.91469    -142.5076     19.38627    533.13493    553.26902   1105.68736    1065.6657   1057.84825    562.21777     459.7604   -106.64151   -215.92342   -251.82838   -304.33171   -484.69846   -403.55969   -414.83725   -452.59138    -354.2859   -445.95452   -465.58553   -212.14209   -196.30053    -39.92351    136.75394    351.05096    257.22724     58.05139          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       61.50    -224.9249   -422.99242   -270.83358   -435.68381   -303.73226   -299.36056   -446.26698   -198.38351   -300.54709    -473.2848   -396.73866   -295.18824   -647.08947   -281.32178   -223.26036   -352.98155   -512.41131   -189.59305   -375.32634   -462.04391   -401.09571   -393.73438   -325.16413   -211.35731   -115.13824   -191.68188    103.99938    201.92817    121.25592   -109.93614   -129.14272   -292.76451   -453.46147   -296.60884   -237.20013   -539.13129   -480.93334   -494.85783   -277.08281    -333.5532   -287.60829   -520.67639   -330.80166   -342.00405   -324.18841   -304.38602    -338.2343   -497.77483   -470.97691   -406.41002   -437.01607   -389.58127   -574.22936   -455.28423    -365.2257    -31.06886     128.1729    226.40569    125.52714    423.03478    207.04679   -175.65236   -258.88478   -295.01913   -456.56595   -361.40384   -389.88386   -306.35229    -97.51994      28.3365    240.13448    618.96041    885.05522   1058.07708    905.19393    559.85723     255.1742     -7.41803   -216.53928   -219.70965   -349.61322   -292.91772   -594.97105   -357.73906   -168.29364   -392.90427   -241.09739   -364.91927   -283.70134   -351.07906   -182.21734    154.59811    283.26143    405.16675   -104.81124          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       62.00    -298.0304    -400.8436   -366.97805   -497.55127   -388.57637   -250.84151   -339.11197   -225.57157   -277.75556   -371.64091   -261.57346   -395.09406   -488.20665   -387.41447   -219.53612   -354.67743   -457.60176   -499.06402    -418.7543   -473.15355   -304.81605   -783.44925   -312.59433   -292.06851    -66.39501   -104.42244   -201.45145     75.51363     25.41048    -77.53284   -172.14011   -288.93013   -431.84197   -328.99262   -109.78295   -521.80568   -561.53832   -380.82502   -528.10342   -497.03101   -246.65243   -481.96718   -351.60532   -314.27884   -371.34269   -432.46838   -299.01116   -290.22461   -593.21927   -452.93152   -619.81852   -271.96259   -309.21435   -249.53077   -284.28512    -50.76659     97.57002     59.66336    188.70437    254.36865     -12.8857    304.29675   -246.89014   -247.04612   -248.77241   -429.18598   -332.82585   -241.42708    273.37844   -108.85877    166.07097    595.08031    784.62781    802.61335    790.13417    570.67058    272.02781    -26.66487   -254.19808    -66.26504   -191.63213   -227.03177   -585.28918   -341.09678   -334.51742   -301.49939   -376.27417   -351.45802   -197.18985   -129.47934    -60.10032   -120.59149     226.6301    203.37505          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       62.50   -381.15063   -399.88377   -353.18044   -450.72061   -495.68507    -177.6664   -244.30332   -343.92927   -406.53071   -406.79973    -67.95993   -439.53449   -430.16364   -329.26511   -200.74335   -310.94796   -494.98515   -578.47627   -551.55348   -355.25892   -439.32655   -431.14532   -361.18229   -546.74945     -27.2828   -163.22091    -40.78653     56.30218     63.34329   -176.66751   -166.29432   -230.21374   -441.90661   -356.47686   -291.94956   -452.85989   -598.51892   -323.05664   -541.69632   -336.81207   -254.10668   -507.99467    -335.8272   -593.34443   -494.12617   -440.33443   -319.34166   -426.19104   -472.99408   -373.95716   -334.89862   -514.38801   -582.72186   -261.30711    -49.21606   -248.02655    -33.45676     60.63293    284.32679    111.64261    136.84044     -4.42478   -143.44545     -68.1146   -306.36018    -255.6427   -300.18874   -128.90454   -251.74079     51.41262    338.68242    673.16952   1042.66822   1243.40457     729.1477    486.29337     41.64162     16.72312    -56.35677    -92.67095   -270.87922    -425.9752    -328.6702   -339.65367   -200.84103   -333.86587   -396.69755   -404.59584    -283.4506   -204.01258     19.52364    -32.37037    -23.89477   -311.08458          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       63.00   -403.51228   -430.91657   -314.79391   -506.07621   -465.65023    -78.86731    -212.7455   -527.43906   -404.23455   -392.78218   -100.59354   -376.06686   -470.63849   -204.40593   -176.17026   -183.98987   -833.22004    -545.7372   -546.08948   -303.19779   -577.94753   -264.45113   -316.35989   -413.90709   -233.96904   -172.38846    194.30763     -0.58653    -98.63413    -20.44675   -370.09458   -311.89288   -436.03141    -165.8418   -335.21725   -287.32812   -519.54003   -475.93899   -451.28607   -320.41941   -479.38196   -403.81715   -252.78544   -480.62321   -386.43534   -506.25135   -481.41255   -418.90549   -456.26693      -617.01   -436.83136   -451.36046   -380.39617   -329.28753   -285.17875    -68.94246   -102.90594     52.71973    220.96662    245.93945    213.61244    -48.09903    -84.48618   -418.52384   -372.15276   -364.38489   -323.59116   -440.67355   -210.83211    -77.12418    313.73322    608.45465    885.37679   1084.14189    968.82486     612.1969    398.08262     84.12051   -278.19712    -308.5717   -310.62909   -442.75714   -441.19659   -196.31694   -276.99013   -305.32744   -267.98529   -152.60537    -243.5142   -408.09079   -142.25548     25.17961   -147.31129          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       63.50   -332.37155   -494.09825   -260.90747   -592.75723   -455.75231   -306.24923   -324.45502   -564.04476   -223.84926   -362.79123   -226.79993   -363.13463   -415.94083   -166.58088   -286.14798   -241.27912   -685.61575   -291.29576   -432.98468   -539.64175   -479.28699   -445.69179   -420.06987   -348.80813   -315.51101   -118.41854    255.88893    -57.14217     41.93266     -15.0163   -234.76488   -206.39954   -555.97219   -291.70914   -226.00569   -362.15846   -311.10222   -333.67452   -382.53311   -440.07213   -393.74739   -329.62261   -253.03493   -247.43118   -239.72692   -555.12566   -211.80237   -423.19155   -507.75899   -180.84046   -354.67323   -433.90883   -283.04905   -323.25022   -250.19717     47.14775    -26.21296    103.10832    178.49836    256.69049    120.67776      6.98506     -132.623   -219.94608   -281.36266   -577.75262   -415.21633   -171.90405   -217.78125     94.05413    272.23518    490.15441    895.36458   1010.75843    856.10302    724.67479    205.68687    256.62659   -157.18844    -515.1838   -357.58767    -236.5684   -363.20825   -326.93298   -269.35949   -360.29354   -359.35912   -348.34399    -332.6144   -180.13115    -261.4528   -187.58518   -178.50774          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       64.00   -237.50644   -404.84868   -228.84495   -595.85116   -492.09616   -367.22605   -476.08257   -484.82845    -57.30646   -321.92513   -263.32209   -214.12875   -401.51016   -257.26818   -503.97636   -369.24889   -257.08398    -173.9806   -385.40438   -467.23566   -448.63524   -453.34393   -487.13511   -179.32594   -406.45506   -135.42471     76.22859     38.00333       37.617   -114.77781   -226.70958   -375.62431   -271.72747   -517.83868   -380.33539   -388.92687   -490.61037   -279.69524   -301.56691   -503.09246   -315.52983   -230.16359   -294.33161   -322.10553   -629.16426   -380.87934   -460.38087   -238.25144   -282.37298   -420.55496   -194.17285     -251.944   -246.52266   -381.06177     -401.733   -348.63908   -200.81214    149.16143    -87.64056    182.01423     184.0025     -51.5554   -247.73667   -341.81468   -271.39621    -309.2247    -300.5318   -353.02759   -250.69499    155.39981     242.8145    487.11943    926.15299   1024.31072    831.64816    586.10861    359.37925    490.21987     69.39693   -255.39937   -262.30733   -412.15771    -534.1172   -262.98961   -292.76879   -443.10245   -545.39949   -341.28844     -356.926   -230.96805    -83.05757   -402.82355          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       64.50   -178.69027   -293.45553   -204.83756   -558.54058   -541.41899   -231.19117   -494.70113   -480.58429   -167.40153   -291.30787   -277.19282    -44.53131   -362.23824   -293.67958    -526.7941   -419.45823    -52.18864   -189.72578   -352.51286   -306.56238   -503.97305    -402.0821   -503.09726   -156.64229   -228.63002    158.13904    -51.44659    216.14663     67.64368   -102.17541   -159.45742   -350.85233   -279.54676   -521.97094   -369.29053   -370.00748   -499.14544   -198.08254   -354.94398    -452.2278   -245.53522   -280.08526   -383.19021   -374.15993   -399.20948    -484.8045   -234.48947   -344.65488   -347.06448   -454.68352      -89.737   -327.75137   -333.67905   -317.70106   -273.64236    -204.6457    -313.3322     86.39993     73.17516     340.3931    108.17021    312.64853   -300.60211   -291.04946   -223.39532   -225.42256   -257.09961   -152.22089   -110.04624   -101.83261    120.58721    491.83064   1095.20753   1034.94633     804.8806    710.52135    117.28063     92.24111    -46.66516   -104.26792   -162.23638   -498.65641   -285.98356   -501.45504   -396.71698    -469.6304   -662.62787   -351.75081   -473.22857   -377.52363   -199.17292   -436.41345          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       65.00   -144.81971   -184.22442   -181.30546   -458.32688   -393.50726   -161.69667   -368.47111   -511.83668   -350.80374   -319.56422   -258.12841   -179.27977    -330.0976   -259.05354   -328.27612   -586.45173    -33.61409   -261.20521   -356.32206   -352.27352   -389.96391      2.28496   -403.05904   -370.32099    -117.6562    118.79066     27.00535    331.65078      14.9528    -90.61668   -256.38792   -157.61363   -381.56749   -391.47861   -319.65557    -547.1294   -496.13344   -544.17696   -440.59247   -249.88033   -353.11362    -411.4701   -397.80716   -367.52042   -196.60606   -323.33573   -343.92793   -210.39794   -394.56178   -280.04756   -299.46258   -393.71365   -337.92892   -265.38709   -345.97124   -300.36576     -5.92922    138.25413    208.89663     21.40865     35.10002     -45.0624   -315.19836   -335.60214   -249.85958   -159.02597   -301.72321   -333.76496   -226.52727   -226.15606    289.16825    555.92772      713.112    867.03958    860.79472    385.63267    116.72931    252.37742   -127.73505   -181.41234    -235.3215   -362.15826   -382.80859   -404.63757   -306.62822   -359.76328    -418.9851   -387.65655   -326.60776   -376.46844   -416.56558          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       65.50   -111.97924   -159.85233   -126.34722   -329.65442    -298.0116   -287.44095   -221.01794    -486.3122   -296.91261   -384.69874   -231.59158   -491.60584   -394.31637   -276.91978   -430.91741   -457.48559   -217.37462   -444.22347   -237.73743   -441.45655   -433.67789    -84.84616   -281.98763   -181.63391    -24.46315     30.91983    162.44977    129.28092     -6.80344   -307.42632   -149.77709   -415.54867   -472.60185   -379.39669   -270.63469   -529.14426   -481.23292   -243.19194    -301.5356   -251.19784    -346.3696    -397.3808   -390.96983   -406.28179   -331.20564   -397.42617   -559.43429    -284.5628   -347.22415   -302.78545   -570.39707   -448.14866   -172.29975   -201.04368   -256.34718   -130.85792     12.11958    227.11626      3.03619    213.56921     77.71196    -37.10136   -138.72385   -329.50399   -301.54386   -359.83296   -381.47354    -36.63976   -381.41306     11.95133    355.00966    621.03861   1084.17907    990.43036   1018.18515    642.24997    303.33206    175.18869   -219.21328   -134.17722   -311.00524   -314.74284   -214.97749   -286.07946   -353.08056    -458.5588   -295.53752   -410.56405   -475.67987   -359.65605   -344.07816          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       66.00    -32.67418    -174.4052    -66.61216     -187.035   -298.20344   -316.23897   -258.07822   -432.29873   -154.83882   -482.32245   -258.94092   -551.77847   -321.37073   -216.89429   -600.67549   -374.95006   -272.52866   -442.41299   -108.80189   -731.86561   -414.43395   -347.46565   -271.79134   -320.87216    -49.98538     28.51209     -4.82298   -149.19555     -9.34738   -243.58848   -252.82339   -135.88053   -523.75082   -380.62097   -301.22645   -406.03738   -267.56091   -376.82378   -458.83315   -463.99388   -354.74708   -425.72358   -545.77522   -491.88677   -342.23688   -530.94117    -539.4057   -368.14595   -278.59211   -298.89548   -303.69942   -364.33976   -374.39154   -251.73902   -130.19512   -307.44559   -172.79874    133.56696    237.37394    100.00048   -177.59687     -1.80899   -387.03427   -279.12754   -502.63284   -395.52178   -333.71092   -108.11122   -285.69965    -16.17168    339.94727    268.33134    955.66511    790.84806   1018.25523    557.94754    444.35593    -15.38241    -191.1707   -171.48574   -395.88537   -364.22384   -440.90449    -302.1561   -499.88706   -447.67583   -369.33174   -344.02489   -338.02545   -457.61202          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       66.50     13.18638   -214.60229   -112.55409   -348.74613    -238.5851   -267.92807   -422.27518   -374.66313   -137.94827   -336.19019   -369.27473   -513.61233   -241.12506   -221.38599   -395.82305   -415.96072   -265.50166   -353.50316   -149.16005   -587.44415   -345.12087   -434.32366   -276.64938   -413.22873   -242.18441   -148.43854    -84.46163   -312.08276    -76.03625     -93.4323   -305.72956   -379.36057   -331.13825    -307.0917   -240.31669   -327.27475   -277.31579   -303.51055    -425.6029   -307.97416   -428.91607   -332.83015   -486.57071   -503.28995    -329.2243   -562.14342   -475.23014   -424.30229   -528.04192   -300.02525    -169.8979   -470.94176   -120.87454   -281.01172   -304.98875   -201.85678   -114.15635    120.85089     61.81712    135.77379     70.32924     -13.9032   -319.32452   -431.16886   -266.30054   -220.34051   -244.69957   -241.59013   -148.44061   -171.99428    162.22062    786.82224    858.44152    957.99306     884.3375    644.57064      450.067    288.37187    -42.06664   -117.33484    -253.9843   -397.81236   -286.26164   -556.79747   -539.70617   -296.60522   -400.06542   -312.44988   -277.50162   -383.78718          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       67.00    -82.46122    -362.8047   -250.34058   -512.34429    -79.23817   -292.77937   -598.25672   -249.64176   -223.29154    -78.01838    -420.4075   -411.20028   -357.08076    -404.2757   -224.54244   -462.43985   -393.25302   -403.23389   -115.23443   -398.47288   -249.96005   -269.36479   -224.58536    -208.8953   -315.24714   -178.84418   -112.31885    -51.17638    138.30282    -97.57574    -82.06887   -498.21451   -448.73522    -352.7319   -316.40599   -385.84086   -275.99447   -425.14637   -250.38944   -333.43536   -409.77211   -290.98972   -323.70836   -393.19672    -352.2096    -390.5771   -530.27093   -257.59226   -374.45322   -450.83433   -319.39753   -283.08975   -277.83545   -398.21228   -342.72873   -185.95222     141.0249      5.66755    228.05481    114.21461   -211.88824   -220.03314   -101.56985    -256.1258   -214.50714   -298.81279   -463.73596   -493.94254   -184.35564   -142.38766    230.00104    532.12128     529.8716    1090.4607    917.15785    669.67776    412.95958     61.36571   -280.71766   -237.73503   -162.71848   -159.86814    -314.8432   -244.82564   -340.56804   -453.51177   -398.17621   -380.40086   -476.96901          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       67.50   -341.04544    -450.2477   -325.98146   -582.86526     -86.0028   -540.95526   -607.43746   -126.48145   -289.32743   -197.24602   -310.41136   -332.82821   -380.22893   -415.22991   -419.19068   -444.94701   -504.49879   -505.63916   -232.32604   -353.29427   -210.69812   -243.69093     -286.778   -246.03855   -150.51797     16.61259    -13.17139    -42.50282    179.90803      6.34032    129.68444   -346.32089   -510.58773   -408.52022   -458.08451   -418.01871   -270.69279   -297.29129   -367.66581   -372.49212   -503.45265     -326.985   -244.08562    -415.0345   -252.45514   -426.00785   -339.56882   -375.60042   -205.97496   -517.85056   -332.77184   -234.55329   -328.84705   -413.38114   -211.29096   -161.61441       4.3752       7.3469    320.29265     81.27088     22.64109   -257.86794   -153.96024   -250.26634   -366.22776   -209.82348   -281.28175   -351.37327   -233.87842   -110.46926    176.26152    724.13059    851.98869   1073.20863    970.66916    813.43666    369.11952    273.37696     49.71038    -66.67966   -175.28844   -331.68882   -374.86571    -361.5649   -572.14234   -303.36101   -494.96067   -330.14895   -373.57327          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       68.00   -536.32861   -400.30382   -293.35603    -418.0715   -187.05266   -806.54423   -529.11803   -216.70796   -211.56141   -298.98896   -246.37443   -459.90769   -298.00273   -323.50467   -516.19095   -396.29807   -400.06418   -318.25751   -342.27757   -315.79024   -160.81849   -204.31416   -414.51643    -285.3277     -94.4631   -221.65001     50.21289   -150.22543    216.41761    -90.44499   -129.56036   -211.28836   -500.52475   -402.66467   -385.86043   -588.62003   -231.27255   -249.45705   -413.54796    -355.5781   -315.91045    -320.0554   -341.33525   -445.12436   -314.14558   -362.51499   -247.80541    -287.0968      4.44572   -341.18427   -246.69254   -316.11483   -456.81754   -299.99818   -223.15163   -186.79583   -112.46878     -38.4666    -11.74368    136.00425       18.398    -99.73291   -269.33711   -384.36056   -417.10522   -351.32236   -328.92558   -212.29446   -118.65681     12.01856     149.0433    395.14313    937.78152   1385.08419    845.71624    900.01724    279.98793    209.67491   -300.39118    -136.3391   -290.39511   -348.88864    -478.0071   -246.41996    -83.15761   -174.51446    -93.28506   -292.12546          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       68.50   -650.54553   -381.59155   -265.13497   -318.06539   -289.52415   -656.15827   -603.70622   -252.02701   -119.85159   -349.45183   -182.66224   -474.47868   -265.52857   -350.70788   -440.45001   -392.14715   -393.98759    -365.5385   -377.56034   -395.19425   -227.21767     -128.579   -455.97554   -101.46339   -128.62374   -131.07795    -93.88496   -182.48071      0.54751   -114.26658   -144.01951    -197.6163   -485.58107   -414.40447   -448.35023   -693.94196    -237.1839   -264.28854    -270.4245   -519.66636    -459.1737   -414.41906   -331.93119   -274.04746   -415.12599    -373.2195   -286.91068   -196.91639   -418.48531   -281.28554   -142.83617   -409.87319   -289.43077   -417.13258   -410.80071   -129.81896    -12.89829    184.99122    427.73118   -139.38752    249.23224     59.54265    -85.47462   -408.62577   -379.50018   -316.48158   -337.84792   -330.99789   -271.44777    -98.23537    123.33732    587.35392     607.1596    941.41553   1071.83915    751.54451    445.23582    186.35163     -0.76389    -71.96123    -84.43826   -420.14467   -486.45779   -200.91983   -312.91203   -371.85085   -306.70468   -357.17518          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       69.00   -625.39509   -502.10077   -284.11786   -340.65627      -233.54   -415.85131   -752.43173   -282.98983   -187.29613   -382.28892    -98.92165   -331.09836   -329.12465   -272.79383   -458.33029   -295.76059   -399.72056    -538.3659   -342.98888   -443.10344    -570.9876   -286.47611   -412.10378   -378.84518    -99.77048    -66.06832    -116.5046   -113.18028   -142.91877   -217.76997   -320.76779   -338.22586   -348.80367   -283.59396   -280.80364   -461.82927   -522.18483   -349.72249   -347.60084   -171.66543   -484.25179   -383.84468   -326.56663   -362.43912   -520.20894   -257.08575   -221.52192   -612.12455   -379.93884   -450.72903   -195.24676   -387.12101   -414.64928   -387.48637     -318.954   -329.10516   -163.01477     77.97194     25.18959     52.93888     58.37403     32.71966   -122.16097   -176.53275   -270.22314   -453.85369   -370.33341   -305.51662     17.20379    -71.82622       4.5253    584.36975     926.8047    891.54363    996.28308    834.13688    470.71953     188.8913    -168.3672     38.85863   -255.94145   -327.39446   -233.37268   -331.37402   -309.80628   -453.38644   -216.53602    -368.1557          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       69.50   -609.00835   -753.73919    -304.3705   -355.84839   -173.23677   -258.70029    -418.8203   -360.05212   -294.73359    -393.7278   -220.18179   -290.15896   -283.42365    -204.4417   -553.34339   -258.40904   -512.42806   -533.61206   -352.13869   -577.69405   -439.46209   -400.67948    -379.5202   -353.48154   -113.68893    300.49446   -305.50344   -130.47202   -152.81024    -15.01589   -386.45695   -195.55608   -226.64679   -229.26452   -230.21873   -374.68877   -372.96632   -486.43249   -380.30957   -175.87799    -89.52503   -272.43155   -169.90162   -387.33187   -282.07976   -276.50309   -218.56308    -477.9277   -426.62361    -428.6539   -330.58016   -375.46019   -453.15126   -325.16528   -250.46973   -253.74128   -151.65876    114.88957    212.46264     22.37269    188.87401     -6.36097   -425.33083   -259.89768   -148.65042   -319.86261   -248.29703    -88.40549   -389.35631   -106.34673     87.19691    613.74752    807.92758    981.80911    836.82883    992.60851    497.01653     93.52463    132.45842    -63.14547     90.76266   -208.38603   -332.34961   -184.94332   -382.53585   -219.28664   -510.51533          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       70.00   -577.09226   -666.27572   -328.40201   -297.89105   -169.86028    -218.1005   -131.58657   -415.86379   -389.49754   -431.94624   -327.84397   -204.34063    -92.21407   -259.33215   -435.42166   -466.18688   -572.00871   -370.40588    -401.4174   -661.62258   -282.31285   -371.12075   -525.34795   -124.27592   -177.15632   -147.94884    -87.95942   -134.99827   -181.40624    -69.60096   -343.75308   -266.76727   -476.31195   -280.65137   -460.59931   -236.91522   -527.19632   -208.98575   -544.97188   -223.86684   -428.93488   -382.60054    -362.6986   -276.13907   -417.62734   -355.24239   -219.94506   -297.51285   -337.20867    -348.7234   -268.96448   -320.08265   -224.83365   -425.19247   -166.28401   -198.09119    -52.59382    224.23021    -46.72143    207.21009    -21.37636   -125.46721    -66.56226   -232.57471   -276.57303   -265.05189   -299.82591   -349.30518   -245.79996    -43.68471    222.52096    527.52986     997.9939    789.14032   1008.77657    860.19059    655.02977     44.27416   -196.41091     39.78803   -239.60509   -174.44911   -272.67038   -337.33946    -425.0569   -320.42706   -327.18735          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       70.50   -282.04003   -488.78173   -364.49938    -288.6929   -135.53849   -228.81693   -189.76662   -385.00303   -423.61452   -424.77691   -308.10484   -136.06833    -33.56121    -345.4012   -321.38588   -522.87269   -220.28865   -175.41271   -330.53383   -445.18856    149.90554   -393.82008   -454.06114   -146.61067   -230.72894   -168.99594   -248.53286    -95.83634   -144.03689    104.04508   -142.35845   -265.04249   -209.76243   -414.05409   -491.50062   -300.71105   -312.67633   -215.44592   -409.05114   -563.63201   -290.57979   -241.48964   -362.58792   -506.17832   -321.83339   -345.53514   -323.56054    -338.5511   -247.91729   -449.87269   -344.08508    -349.6984   -501.25068   -474.58183   -265.77517   -116.45764   -126.60128   -103.25587     80.77871    343.51143      4.02353    -191.5072   -234.02847   -346.56679   -210.44126   -379.41157   -275.28532   -152.19219   -207.72772    -67.11567     360.9633    534.39578    730.16754    773.28609    808.63395    541.49282    358.21133    166.14125    -65.13645   -266.58113   -298.13538   -316.49275   -225.56785   -476.04565   -396.04034   -506.20943   -478.66714          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       71.00     -50.6558   -334.45904   -430.77113   -262.27594    -79.61574   -244.52293   -260.32992   -326.12921   -314.65147   -378.70001   -342.39231   -259.13326    -82.65495    -277.4383   -274.43999    -382.9538    -24.60152   -333.33492    -285.5905   -282.84567   -152.86769   -227.69041   -319.50883   -207.70822   -438.70126   -163.92048   -232.39505   -195.26203   -196.09885    105.01183    -214.1217   -491.04069   -290.03685   -416.92024   -299.45025    -314.5144   -446.95485    -279.0857   -323.16848   -483.12094   -377.59764    -260.3448    -522.5956   -411.32122   -285.35184   -152.07549   -292.05988   -334.03546   -235.68028    -456.4366   -416.07055   -332.39688    -464.3064   -212.34013   -442.81654     -27.6755      -43.552    185.64084    135.77611    214.46109    127.65829   -156.73071     169.4035   -412.22278   -456.83259   -353.38376   -377.91876   -393.89147      -90.064   -183.19007    188.39586     510.2851    814.35763    876.43189   1059.23411    716.30145    378.24183      50.5432    -29.49167   -297.36571   -429.09864   -213.88933   -333.79463   -172.28653   -347.66454   -430.18665          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       71.50    101.25548   -449.65682   -448.10064   -281.67861   -252.60336   -149.06568   -248.32675   -398.67354   -258.33169   -360.89638   -384.97369   -271.28232   -211.88601    -95.55972   -165.35137   -305.00093   -154.36053   -392.49961   -275.49117   -509.02053   -372.64663   -187.29414   -345.75416   -344.07328   -246.56024   -226.42126     21.28741    -109.0713    -114.0297     76.37113   -298.09552    -379.3118   -347.88312   -404.51862   -317.92939    -328.0931   -448.62129   -388.16054   -214.26484   -308.66065    -279.7529   -396.15605   -486.16308    -538.4146   -332.54493    -260.5081   -259.81947   -243.27206   -420.22989   -474.43532   -312.18075   -454.08625   -348.99993   -244.08434   -319.35689    -98.02488   -277.71799    -41.14333     41.22411      52.6306     28.79913     43.38523   -170.88935   -444.73319   -403.03946   -446.45259   -463.11238   -499.21908   -146.96023    -25.02375     32.33593    569.88704    760.22044   1146.56626    979.95619    685.67651     661.3168    248.44576     89.36862   -150.79288   -411.26456   -517.33205   -336.90361   -301.33021   -313.61694   -637.37987          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       72.00    -64.79826   -530.63647   -392.97504    -425.5786   -495.43147   -155.73721   -244.92348   -561.65667   -262.02181   -226.42324   -416.59389   -204.16148   -268.17468   -113.65531   -410.82822   -263.51085   -314.20781   -231.60311   -281.21415   -416.59006   -316.08868    -209.4509   -363.21746   -431.15979   -151.63406   -168.36827      62.0984     63.66048     76.48471   -158.93718   -356.04506   -198.15313   -369.79648   -399.07166    -390.8796   -363.30161   -284.52399   -535.33315   -287.48934    -351.6095   -238.68307   -484.85743   -403.50774   -414.01014   -446.21526   -231.05959   -442.28761   -319.35289     -498.708    -316.3776   -369.57719   -495.19576   -318.86573   -349.01707   -281.91627   -331.84637    -37.06043    -60.92572    222.07498    -64.18524   -125.08459     57.32041   -184.46634   -468.37406   -459.51675   -627.27596   -190.86304   -294.75576   -268.28673    -60.25179    268.06283    452.53049    960.75558    987.58288    982.54153    728.30072    444.67146    232.75604    183.68809   -199.01833   -312.91615   -587.67246   -274.68831    -377.0771   -475.20359   -693.24813          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       72.50   -176.12358   -550.56933   -378.88658   -582.00571   -497.07333   -358.49186   -298.41704    -353.2178   -253.28034   -160.21251   -443.67397   -153.74629   -314.38996   -291.30983    -492.7811   -272.08427   -499.43623   -354.67989   -242.33779   -344.57101   -206.00292   -494.70723   -319.18094   -336.57957   -149.92702   -242.86249      59.0966      6.58706   -133.22451   -227.12803   -282.70873   -428.86507   -320.34535   -312.33166     -228.495    -398.0661   -215.28668   -429.16039   -418.47544   -327.68751   -490.49299   -289.19089   -244.92765   -355.65699   -207.40692    -591.0951   -305.73518    -392.1356   -274.53314   -510.56121   -421.22449   -352.28617   -444.64474   -321.55833   -257.00139   -270.24586    162.29548    101.09477    139.64063    -72.84577     38.12716   -114.68275      26.5997   -371.47111   -276.37967   -427.30176   -356.33076   -366.04207   -280.14183     -60.7877    372.21309    737.94987    745.56064    793.44065    965.39694    617.24697    524.63384    295.80323    -90.99698    -99.13112   -210.64455   -305.92243   -468.58075   -400.22994   -268.66129          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       73.00    -231.8682   -463.87297   -447.20347   -634.07715   -449.71129   -453.85797   -398.88141   -150.84249   -479.91386   -403.10942   -561.78916   -401.02029   -418.96234   -281.99227   -467.54612   -366.92703   -505.52872    -479.2963   -304.96775   -598.44326    -168.8035   -303.59701   -167.63406   -240.83083     37.33932   -309.70178      4.47785   -201.85933    -84.13968   -101.47044   -293.62142   -206.75224   -276.76932   -306.20637    -546.5462   -508.19509   -538.96029   -266.57193   -399.58902   -241.20983   -369.58857   -312.26711   -195.70657   -317.49098   -207.14025   -343.24182   -417.04034   -407.50091   -285.13896   -407.31732   -148.91227   -385.77064   -342.76002   -437.79485   -328.13847   -201.05334   -113.06574    205.78182    138.97078    -79.35218    154.90903   -346.22214    -76.88534   -394.74043   -193.27212   -245.55125   -373.89566   -231.56723   -242.36953   -249.08772      -2.1556    667.41085    979.76215   1007.95322       799.09    782.28388    371.25443     10.06753   -136.09249   -193.30753   -383.02947   -120.24436   -340.35159   -319.36687   -404.37883          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       73.50   -175.97313   -387.69853    -471.0307    -607.4069   -477.63674   -393.19865   -495.23499   -159.73743   -728.99403   -557.65563    -722.2339   -682.74024   -384.26166   -251.46962    -429.0061   -359.70442   -470.33939    -474.5304   -420.80326   -443.60011   -312.38369   -231.77852   -342.22555   -255.82821   -171.09939   -364.46042    -57.32279   -200.51456     84.15861   -232.60767   -555.39088   -127.84256   -482.13045    -295.9344   -521.72776   -427.82989    -623.8689   -363.00839    -186.6106    -281.3252   -105.29162    -36.04921   -396.05897   -429.16771   -321.18417   -450.81566   -359.48073   -182.76544   -257.94746   -443.51392   -490.18294    -419.6868    -58.83552   -382.97978    -72.09585   -189.84985      91.6793   -129.50632   -141.18163    142.38706     94.39513    -10.19631   -172.36541   -215.61297   -436.50172    -266.7354   -332.99052   -600.35127   -235.84828   -116.06867    266.97959     428.0228    889.95836    783.26336    686.20453    742.79105    359.97983     85.83205     -8.14655     34.29232   -252.99766   -465.99728   -399.19514   -126.88234   -430.99766          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       74.00    -74.77436   -331.71299   -299.55637   -559.74049   -511.99877   -378.71216   -372.52125   -177.97183   -678.52463   -453.44273   -756.74951   -476.34077   -355.02861   -379.83038   -256.84129   -401.31502    -304.1179   -466.97513   -224.52018   -226.42365   -276.86304   -313.69583   -356.50938     -372.055   -279.10448   -126.82776   -175.74624   -277.68054   -222.73502    -68.06243   -451.87302   -356.50181   -580.87406   -215.90547    -476.9011    -225.5176   -451.50356   -477.06037   -234.08804   -389.70532   -507.23764   -355.00694   -350.59311    -573.4559   -426.61372   -558.93497   -401.23813   -236.99102   -224.94956   -382.32627   -424.70914   -255.79931   -216.59063   -421.63555   -281.52915    -123.3051    -49.98564    138.80405    149.41236    320.44683    -10.05049   -156.25841     -245.443   -293.53869   -297.51376   -272.74162   -112.94515   -209.55638   -342.81253   -107.33492    130.79299    278.93686     798.2766   1050.04196   1015.99759    601.31747    711.32181    158.35137   -193.12876   -128.17333   -339.59893   -251.37574   -357.04632   -464.89854          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       74.50      41.2433    -309.6117     15.19568   -482.30488   -456.81364   -497.89117   -164.47701   -276.39778   -466.17426   -307.75077   -510.22553    -342.1997   -470.69157   -580.54621   -337.32203   -580.15364   -205.14982   -403.11227   -197.15547   -235.44657   -315.03287    -326.2528   -547.27829   -256.51056     71.94064   -136.70018   -291.30582   -246.97654   -132.32825    -29.15097    -12.54662   -472.48971   -346.69441    -482.7168   -368.81674   -673.91136   -397.39957   -257.01683   -404.55467   -316.34315   -365.78665   -559.41371   -562.81256   -566.34775   -507.03193    -431.6537   -297.97131   -408.43548    -394.6913   -358.52735    -192.7283   -460.84995   -489.17295   -439.28252   -304.27536   -195.86583    -75.23185    -64.98925    146.16688      1.42974    -54.67283   -129.38933   -188.53522   -182.46113   -402.66274    -78.60808   -300.27155   -398.27644   -129.27379     16.72142     68.12738    469.46638    749.02723    938.02888   1113.38252     769.5446    537.39297     169.6132     49.27633   -146.03247   -154.27238   -356.71414   -459.81631   -308.89794          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       75.00   -285.50089   -306.68855   -146.84042   -332.68713   -392.03698   -616.08787    -71.79328   -501.98493   -387.47286   -190.80388   -328.52532   -328.44658   -469.48999   -540.60007   -489.98926   -256.07076   -226.42808   -319.22976   -476.26948   -326.95776   -471.05228   -315.31054   -557.18847   -278.52211   -276.13044   -248.30079   -217.63288   -143.23801    -295.6394   -221.39895    -61.86687   -424.69477    -376.4569   -295.28079   -217.32727   -380.59649   -396.14952   -415.68639   -424.71364   -413.37247   -399.96246   -503.37273    -75.02178   -501.98703   -395.53863   -481.79319    -356.8466   -178.16551   -256.87053   -306.02865   -158.80571   -512.49803   -593.08318   -197.05518   -374.03239    -86.58904    -13.91211     61.30068   -114.86167    -67.70702     50.20435    -70.60379    -88.19443    -189.4233   -541.99121   -160.89929   -668.27477   -288.84369   -279.58257      59.5088    115.19555    420.11956    583.05237   1107.72429   1248.57699    956.88244    571.98642     172.8537    266.55349   -226.93345   -260.31543   -378.39524   -284.38872   -561.16859          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       75.50   -573.78921   -386.81704   -491.56854   -191.99054    -372.0906   -627.42892   -131.78552     -708.217   -583.44531   -184.59304   -360.09422    -314.7292   -321.97421   -434.09141   -561.72414   -164.29456   -388.16692   -256.36801   -436.91286   -381.20088   -319.01859   -295.69261   -464.60967   -437.68602   -392.79962   -307.17881   -158.43143   -249.52899   -311.44278    -201.7933   -208.80352   -125.09794   -308.46514    -259.1322   -363.05393   -553.56869   -232.47812   -402.46554   -337.47626   -479.80525   -216.63185   -519.28213   -220.90422   -333.97501   -340.87547   -214.84445   -326.92772   -399.14291   -304.33945   -373.01849   -362.23243   -320.54194   -452.41848   -281.61611   -260.24516   -306.91502    -96.29131     74.15353     55.19038    153.43329    348.67741   -441.42329   -171.55941    -101.9195    -13.12358   -410.78177   -349.74742   -104.88336   -277.90336    -161.3834     76.40838    443.23816    697.02235    842.47478    1109.9253    807.64791    631.36914    209.69739   -169.97622   -170.15442   -273.88572   -143.34914   -314.76005   -337.85612          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       76.00   -753.84116   -410.02929   -758.31151   -180.77699   -415.33862   -530.27378    -212.4172   -770.23504   -770.81702   -285.98437   -395.20976   -401.00959   -251.18669   -349.54433   -559.28775   -159.64155   -346.12184   -330.82399   -387.80458   -369.18604   -150.56782   -300.57362   -415.57982   -329.37903   -410.62104   -172.22652   -312.59811     67.79681   -195.94778   -312.10173   -421.79749   -348.96725   -263.78388   -263.50394   -405.59401   -214.30323   -422.79484   -439.28662   -375.54121   -462.02627   -297.30938   -419.94265   -294.98648   -309.18861   -375.33979   -621.18755   -571.36276   -405.30749   -221.73661   -477.32389   -398.28207    -298.4916   -342.16524   -265.67632   -282.26824     -276.573     -207.469     74.46643    -68.11298     25.16579   -288.67876   -102.01883     -38.7655   -328.96162   -267.19189   -405.66136   -441.69446   -280.76029   -249.22355     -88.1822      258.096    618.10963     573.2816    685.48761   1016.10165    562.41006    467.93088    423.93572    107.50588    -80.15776    -82.45881   -290.04379   -395.37915          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       76.50   -592.95537    -334.0248   -506.19522   -155.80894   -590.73681   -425.52758   -275.07147   -712.36879   -619.20569   -381.33863   -479.19359    -439.7278   -343.04709   -347.59794   -332.55527   -221.61751   -228.11296   -432.99792   -271.07934   -424.90324   -204.07824   -162.20415   -466.37812   -196.46764   -325.31774   -214.80355    -483.5382    -29.54544     70.85456   -190.47789   -404.61634   -451.58203   -317.12739   -337.53897   -407.06773   -176.85312   -524.41856   -414.51802   -295.75577   -350.21216   -352.17508   -403.42382   -475.87833   -360.79584   -252.96859   -526.55076   -494.18124   -522.88445   -387.94592   -517.39218   -451.75505    -73.13789   -328.32885   -172.19024   -366.49585   -244.42261     -30.5027   -232.03634    -13.46549   -149.85171    -58.85826    162.08237   -204.95917     23.92905   -265.69561   -322.24471   -206.21691   -121.57691    -91.02796   -153.42707     -7.31677      691.914    659.58161    825.74181   1129.94687    896.58557    458.47334    263.69903    224.89789   -118.82859   -230.37192   -308.35865   -456.34082          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       77.00   -520.39163   -345.29163   -306.77993    -266.6209    -747.0445   -434.13809   -288.20983   -374.29434   -485.52503   -439.67903    -592.9038   -283.25935   -380.04171   -391.68112   -304.37354   -374.53948   -201.85909   -611.97914   -261.58281   -450.48555   -357.09169   -162.56361   -249.56305   -282.06569   -207.52963    -111.8796     59.31094   -187.59408   -187.72744    -81.59288    -490.4423    -385.2334   -324.49213   -393.15082   -396.87666   -267.02106   -360.66168   -289.51631    -406.1463   -255.97845   -200.47827   -324.54465   -516.07482   -276.64134   -297.18185   -442.89552   -314.31665   -193.97565   -355.64668   -312.16567    -87.88633    -266.9333   -295.05347   -186.59024   -501.66442   -324.35073    -85.09943    -125.6929   -246.35247     12.13434    179.74866    -49.87002    -68.54839   -382.96914    -462.0597   -249.81525   -201.71893     -23.7876   -173.29989   -172.62441    101.23124    449.34223    646.13507    933.37316    756.05743    769.95643    682.64167    388.63814    132.70271   -196.05903   -559.36179   -255.82696   -367.77688          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       77.50   -507.57969   -408.84398   -300.85899   -355.85111   -589.25525   -450.36505   -431.57486     -98.7073   -471.68427   -399.09773   -591.74133   -209.19334   -250.69111   -478.80177   -380.10366    -414.7731   -177.72906   -162.54847   -322.66142   -207.52052   -501.38192   -490.48162   -183.12511   -563.49031   -367.07886     44.02749      77.0018   -295.64581   -229.07094    -96.76668   -569.22241   -337.55935   -452.49037   -290.34228   -319.78232   -190.26364   -229.98844   -239.35139   -468.45935   -578.80812   -400.08778   -606.49559   -539.05971   -392.24124   -423.42871   -491.26374   -330.67568   -477.43066   -321.80301   -369.43786   -263.37911   -265.09897    -352.2268   -362.29196   -160.15009   -246.34582   -102.84225     28.19431    -87.03605    179.67938    189.80219    -10.09596    -120.8623   -363.54954   -135.47919   -138.92565    -260.0999   -282.73948    -392.6335   -252.56254     15.32842    474.34704    651.82384   1065.18603    773.16846    795.06308    555.34001      88.2469    115.15473    -187.3733   -133.80707   -520.37949   -362.17842          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       78.00   -393.90653    -471.1761   -434.48506   -377.71447   -322.48112   -435.39955   -562.11176   -287.82468   -444.85585   -152.28034   -557.85496   -233.69739    -179.4975   -549.57127   -335.87615   -357.33723   -418.51012     13.56793   -364.21317    -238.3268   -523.17409   -467.75922   -279.63061   -382.77619   -369.69921     -51.2524    115.72383     40.27157   -261.90151   -137.52855   -268.04298   -298.52565    -308.4803   -291.09216    -419.5409    -74.31105   -407.06959    -365.8019   -449.52849   -437.02155   -327.55466   -446.30026   -494.06221   -274.90251   -159.61435   -465.85813   -437.19521   -405.51274   -235.70971   -252.87669   -296.46542   -309.13502   -368.19737   -326.55606   -440.74931   -262.07699    -18.10161      33.2659    -68.17101    124.23218     -60.3785     52.81706   -256.62952   -127.95457   -214.41962   -324.97289   -216.73691   -363.12502    -508.8128   -138.27526    220.26881    539.48809    739.70397    890.83691   1137.12535    900.57843       531.62     39.66111    102.67586    -192.7496    -32.35845   -219.71107   -336.57697          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       78.50   -262.73528   -471.31591   -542.45048     -383.857   -155.10342   -407.25917   -433.09187   -400.18332   -414.51497    -60.01097   -476.62939   -217.14062    -44.00178   -423.19976   -295.35907   -237.41669   -405.71935   -424.93699   -481.96219   -431.71057    -227.8966   -244.07832   -677.63334   -400.39005     27.39385   -129.94314    -34.22185     61.51995   -341.16574   -348.25606    -87.33811   -453.36731   -119.80329   -482.05104   -458.72426   -175.37917    -270.9746   -364.12821   -425.08245   -170.92847    -261.5603   -332.54184   -262.16831   -315.54456   -329.18813   -226.49133   -582.45439   -285.69292   -374.96929   -337.84132   -273.91744   -216.42751    -292.2727   -389.89326   -474.59536     -59.9783   -156.97284    173.71585    -49.55259     67.94137    -95.48435    -83.34892   -126.87984    -138.6753   -188.02119    -165.7463   -152.79247   -162.48385    -188.1274    -19.20448     216.2545    417.04323    618.21507    966.22901    974.08551    677.65881    536.20764    159.94755     87.59795    -55.00652   -460.52048   -363.29334          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       79.00    -178.1685   -498.69316   -472.17243   -461.38113   -243.72117   -373.78585   -289.15227   -387.31971   -351.34077   -312.20865   -446.41496   -255.90909   -201.27775    -354.4176   -420.73863   -301.28618    -351.3076   -589.23354   -560.08998   -427.84608   -118.95553    -23.58755   -395.82683   -448.84036   -210.00894   -146.24634   -559.39202   -278.08308   -254.93087   -133.57867    -65.03348    -474.3535   -407.61922   -641.18182   -604.54779    -295.8767   -264.33333   -367.64147   -227.94414   -334.62999   -247.33147   -317.13993   -282.69104   -454.05421   -349.50431   -347.38328   -374.19983   -354.73517   -314.16159   -355.72978   -234.85158   -418.40872   -421.62729   -271.87164    -225.7674   -246.68776   -251.29033    139.17163   -234.07025   -132.90725   -146.07551    -72.27607   -231.45613    -57.63403   -396.58407    -354.6819   -345.05307   -119.80762   -155.50142   -214.69377    230.56688    295.50254    751.26965    925.40583    965.79702    889.53477    854.02271    257.89305     43.37685   -164.49279   -463.54778   -387.70498          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       79.50   -287.45559   -546.22093   -383.06944   -587.54894   -363.66208   -322.13032   -343.21509   -415.16967   -409.28651   -488.88273   -458.96937   -336.49369   -421.09735   -385.44096   -390.09492   -466.94895   -522.60765   -613.88951   -497.05947   -323.33903   -201.48963   -108.26404   -252.51096   -504.82992   -297.61016   -182.84377   -213.95282   -108.52139     -6.66797   -117.55321   -170.33301   -216.28552   -388.10576   -230.05874   -353.12714   -189.45105    -196.5413   -320.66811   -337.43744   -309.54301   -250.47897   -283.23852    -514.4869   -421.21706    -76.59216   -402.95028   -344.10448   -568.54598   -372.48154   -607.67075   -441.73791   -283.39358   -158.30192   -246.87306   -407.26165   -313.18924    -40.10245   -257.80238     27.09079    122.10527    -71.35651   -156.56859   -243.14867   -437.70722   -253.24515    -367.7095   -373.05421   -298.91964    -169.9551    -59.86276     63.56775     244.3065    746.23103    683.60916    989.37809    905.73545    551.73895    517.14106    304.25223    -49.42446   -235.37966   -442.89694          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       80.00   -395.76694   -561.04948   -338.79818   -706.67349   -389.11088   -261.42105   -443.32649   -502.40633   -475.61183   -418.41888   -523.51828   -346.06912   -269.28778   -402.03748   -199.59784   -337.79341   -562.49033   -440.59309   -481.44118   -213.27917   -124.49945   -210.58767   -202.32467    -417.5261   -333.18395   -252.78981    -108.7103    122.90113    -49.75427   -178.47136   -136.01417   -479.72397   -293.97578   -262.17154   -311.56584   -289.24958   -197.98236   -325.64691   -208.20984   -661.78623   -300.17047   -325.55231    -605.3566   -314.92448   -338.32772   -315.67558   -508.12164   -386.11135   -327.18671   -673.66207    -309.0745   -403.42719   -282.64654   -381.64163    -201.8155   -288.87429   -116.76377    -66.27384    100.03828     54.47494    -90.46562   -157.22759    -331.1781   -218.72018   -411.05132   -265.25456    -525.0909   -319.23204   -332.30377   -110.94878    -13.87346    270.46613    650.60389    772.84669    916.66506    783.16923    551.84522    220.96889      27.4361     76.35868   -252.47037   -374.27081          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
;

_pd2d_meas_2theta_phi_intensity_down_sigma
;
         241   4.00         4.20         4.40         4.60         4.80         5.00         5.20         5.40         5.60         5.80         6.00         6.20         6.40         6.60         6.80         7.00         7.20         7.40         7.60         7.80         8.00         8.20         8.40         8.60         8.80         9.00         9.20         9.40         9.60         9.80        10.00        10.20        10.40        10.60        10.80        11.00        11.20        11.40        11.60        11.80        12.00        12.20        12.40        12.60        12.80        13.00        13.20        13.40        13.60        13.80        14.00        14.20        14.40        14.60        14.80        15.00        15.20        15.40        15.60        15.80        16.00        16.20        16.40        16.60        16.80        17.00        17.20        17.40        17.60        17.80        18.00        18.20        18.40        18.60        18.80        19.00        19.20        19.40        19.60        19.80        20.00        20.20        20.40        20.60        20.80        21.00        21.20        21.40        21.60        21.80        22.00        22.20        22.40        22.60        22.80        23.00        23.20        23.40        23.60        23.80        24.00        24.20        24.40        24.60        24.80        25.00        25.20        25.40        25.60        25.80        26.00        26.20        26.40        26.60        26.80        27.00        27.20        27.40        27.60        27.80        28.00        28.20        28.40        28.60        28.80        29.00        29.20        29.40        29.60        29.80        30.00        30.20        30.40        30.60        30.80        31.00        31.20        31.40        31.60        31.80        32.00        32.20        32.40        32.60        32.80        33.00        33.20        33.40        33.60        33.80        34.00        34.20        34.40        34.60        34.80        35.00        35.20        35.40        35.60        35.80        36.00        36.20        36.40        36.60        36.80        37.00        37.20        37.40        37.60        37.80        38.00        38.20        38.40        38.60        38.80        39.00        39.20        39.40        39.60        39.80        40.00        40.20        40.40        40.60        40.80        41.00        41.20        41.40        41.60        41.80        42.00        42.20        42.40        42.60        42.80        43.00        43.20        43.40        43.60        43.80        44.00        44.20        44.40        44.60        44.80        45.00        45.20        45.40        45.60        45.80        46.00        46.20        46.40        46.60        46.80        47.00        47.20        47.40        47.60        47.80        48.00        48.20        48.40        48.60        48.80        49.00        49.20        49.40        49.60        49.80        50.00        50.20        50.40        50.60        50.80        51.00        51.20        51.40        51.60        51.80        52.00        52.20        52.40        52.60        52.80        53.00        53.20        53.40        53.60        53.80        54.00        54.20        54.40        54.60        54.80        55.00        55.20        55.40        55.60        55.80        56.00        56.20        56.40        56.60        56.80        57.00        57.20        57.40        57.60        57.80        58.00        58.20        58.40        58.60        58.80        59.00        59.20        59.40        59.60        59.80        60.00        60.20        60.40        60.60        60.80        61.00        61.20        61.40        61.60        61.80        62.00        62.20        62.40        62.60        62.80        63.00        63.20        63.40        63.60        63.80        64.00        64.20        64.40        64.60        64.80        65.00        65.20        65.40        65.60        65.80        66.00        66.20        66.40        66.60        66.80        67.00        67.20        67.40        67.60        67.80        68.00        68.20        68.40        68.60        68.80        69.00        69.20        69.40        69.60        69.80        70.00        70.20        70.40        70.60        70.80        71.00        71.20        71.40        71.60        71.80        72.00        72.20        72.40        72.60        72.80        73.00        73.20        73.40        73.60        73.80        74.00        74.20        74.40        74.60        74.80        75.00        75.20        75.40        75.60        75.80        76.00        76.20        76.40        76.60        76.80        77.00        77.20        77.40        77.60        77.80        78.00        78.20        78.40        78.60        78.80        79.00        79.20        79.40        79.60        79.80        80.00        80.20        80.40        80.60        80.80        81.00        81.20        81.40        81.60        81.80        82.00        82.20        82.40        82.60        82.80        83.00        83.20        83.40        83.60        83.80        84.00        84.20        84.40        84.60        84.80        85.00        85.20        85.40        85.60        85.80        86.00        86.20        86.40        86.60        86.80        87.00        87.20        87.40        87.60        87.80        88.00        88.20        88.40        88.60        88.80        89.00        89.20        89.40        89.60        89.80        90.00        90.20        90.40        90.60        90.80        91.00        91.20        91.40        91.60        91.80        92.00        92.20        92.40        92.60        92.80        93.00        93.20        93.40        93.60        93.80        94.00        94.20        94.40        94.60        94.80        95.00        95.20        95.40        95.60        95.80        96.00        96.20        96.40        96.60        96.80        97.00        97.20        97.40        97.60        97.80        98.00        98.20        98.40        98.60        98.80        99.00        99.20        99.40        99.60        99.80       100.00       100.20       100.40       100.60       100.80       101.00       101.20       101.40       101.60       101.80       102.00       102.20       102.40       102.60       102.80       103.00       103.20       103.40       103.60       103.80       104.00       104.20       104.40       104.60       104.80       105.00       105.20       105.40       105.60       105.80       106.00       106.20       106.40       106.60       106.80       107.00       107.20       107.40       107.60       107.80       108.00       108.20       108.40       108.60       108.80       109.00       109.20       109.40       109.60       109.80       110.00       110.20       110.40       110.60       110.80       111.00       111.20       111.40       111.60       111.80       112.00       112.20       112.40       112.60       112.80       113.00       113.20       113.40       113.60       113.80       114.00       114.20       114.40       114.60       114.80       115.00       115.20       115.40       115.60       115.80       116.00       116.20       116.40       116.60       116.80       117.00       117.20       117.40       117.60       117.80       118.00       118.20       118.40       118.60       118.80       119.00       119.20       119.40       119.60       119.80       120.00       120.20       120.40       120.60       120.80       121.00       121.20       121.40       121.60       121.80       122.00       122.20       122.40       122.60       122.80       123.00       123.20       123.40       123.60       123.80       124.00       124.20       124.40       124.60       124.80       125.00       125.20       125.40       125.60       125.80       126.00       126.20       126.40       126.60       126.80       127.00       127.20       127.40       127.60       127.80       128.00       128.20       128.40       128.60       128.80       129.00       129.20       129.40       129.60       129.80       130.00       130.20       130.40       130.60       130.80       131.00       131.20       131.40       131.60       131.80       132.00       132.20       132.40       132.60       132.80       133.00       133.20       133.40       133.60       133.80       134.00       134.20       134.40       134.60       134.80       135.00       135.20       135.40       135.60       135.80       136.00       136.20       136.40       136.60       136.80       137.00       137.20       137.40       137.60       137.80       138.00       138.20       138.40       138.60       138.80       139.00       139.20       139.40       139.60       139.80       140.00       140.20       140.40       140.60       140.80       141.00       141.20       141.40       141.60       141.80       142.00       142.20       142.40       142.60       142.80       143.00       143.20       143.40       143.60       143.80       144.00       144.20       144.40       144.60       144.80       145.00       145.20       145.40       145.60       145.80       146.00       146.20       146.40       146.60       146.80       147.00       147.20       147.40       147.60       147.80       148.00       148.20       148.40       148.60       148.80       149.00       149.20       149.40       149.60       149.80       150.00      
      -40.00     237.2916    222.90559    212.65555    217.79453    208.25199    207.71034    204.62591    189.20726     196.2075      191.181    182.15662    186.91469    183.46798    180.76988    182.02314    178.09141    172.34672          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -39.50    234.99514    225.90897    211.71155     212.5608    210.38648    208.84035    207.17419    191.82586    195.99659    188.13091    183.56609    187.40039    183.67067    181.74713    182.95553     173.7564    170.34776          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -39.00    233.80398     228.7762    214.72315    209.14905    213.64083    208.22969    204.20155    192.73882    196.39422    186.34727    181.52013    190.95958    182.29433    181.51769    180.58583    171.19381    166.34947    167.30016          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -38.50    233.87984    229.14869    217.53735    213.92003    218.01226    207.97251    199.22019    194.66694     196.0396    190.23092    181.15152    193.80269    183.56313     187.3827    176.23925    173.16359    165.47053    164.61066          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -38.00    232.78165    227.74871    219.64884    217.10616    220.32917    209.84862    195.34622    198.26788    192.02462    195.26062    182.89396    189.07132    182.77663    188.40795    175.35322    177.53719    166.12657    165.46672    167.83536          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -37.50    231.37783    226.02675    221.29551    218.67954    220.09428     212.2001    192.06582    201.08275    187.55124    191.19248    188.96499     187.7272    183.72916    187.99732    176.40542    179.22786    169.13076    176.81586    168.24998          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -37.00    230.89036    226.22907    222.16731    220.06643    218.34057    211.98969    188.17822     196.6518    185.24499    187.22681    190.99887    190.74869    184.03809    188.35879    173.18766    174.12741    170.05354    172.41278    170.00426          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -36.50    233.52875    227.71865    221.50448    220.26853     216.6537    208.80179    187.46522    191.62271     186.1263    187.96268    190.20752    190.31612    181.98656    185.72863    174.13481    169.79411    169.46182    162.98703    172.89022    170.41137          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -36.00    235.89537    225.32377    220.15954    222.14403    215.10063    204.31237     194.1774     188.4924    186.15928     193.0972    195.90557    185.82065    181.96205    181.47718     175.4813    175.50347     170.0877    165.89659    172.65638    169.89297          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -35.50    237.01398    224.31862    221.23556    220.79853    213.14964    200.55709    196.15169    191.47671    189.26609    192.86082    199.79279    189.72325    183.59915    177.36539    172.92101    173.17034    168.83813    170.28551    171.08734    166.51467    164.42876          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -35.00     236.5119    224.84589    223.44345    219.23856    210.10553    199.99493    194.87908    195.48041    192.47665    189.00124    192.70536    191.79266    183.72692    175.60504    173.63485     169.1063    170.80336    169.80496    170.34643    170.88474    157.53481          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -34.50    233.51418    225.29107    226.02907    219.18806     205.1501    198.41172    194.58751    198.99438    194.47995    187.70264    190.61231    187.75276     182.2544    177.77588    171.36311    170.37271    170.74863     172.8301    171.75186    172.01362    161.06423    159.55936          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -34.00    232.43214    223.76321    225.07836    221.70949    203.34345    195.82886    193.52885    201.11657    195.72559    189.69435     187.4024    183.46631    182.11354    179.73067    172.33671    172.51531    169.78213    174.35088    172.41893    167.97267    164.26238    161.74186          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -33.50    233.24563    223.70695    224.12486     223.5919    206.67983    197.25464    193.45635    199.58702    196.63486    186.95875    183.78415    182.96518    183.32048    174.52919    175.19113    175.33468    170.88029    173.33957     174.1222    167.21617    164.34526    163.24932    162.68211          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -33.00    237.44468    226.50006    222.46811    224.28832    205.95462     199.7462    193.77056    198.59893    197.69262    187.73092    184.45076    185.66012    182.41865    170.65889    175.58672    177.80934    166.69666    173.74562    170.99044    166.84039    166.18769    166.09721    163.01438    161.05324          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -32.50    240.65345    231.98456    219.85633    222.53505    202.53875    201.56795    194.49208    199.28696    197.15145    191.26659    187.30192    187.00891    183.57613    173.66599    173.33206    176.90621    163.89452    171.89893    169.94974    162.85996    168.71432    163.43729    161.40434     154.5324          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -32.00     241.2572    237.05953    218.99322    221.11626    200.86823    204.77274    195.79684    199.34658    191.35966    193.14514    190.86577    188.12901    183.17067    178.70808    172.39773    177.45791    169.88658    170.36428    171.04316    163.25004    170.80189    159.53418    158.73206    159.26876    162.08386          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -31.50    242.07136    239.59399    219.85686    222.00876    204.15899    210.33621    195.88506     200.2801    182.52183    196.86296    193.67918    186.21121    181.85054    181.56604    175.87212    178.99785    173.63496    169.34216    169.11247    163.06116    167.92927    157.65172    153.05568    165.15158    165.10808          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -31.00    243.26474    236.35834    220.83552    224.10195    205.67184     208.1585    195.46767    202.01755    182.54721     200.7851    192.22076    182.92121    183.45159    183.12399    177.13898     177.7862    173.83632    170.38484    167.85272    163.34328    165.93359    154.01757    163.55133    161.03187    170.12731     166.7499          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -30.50    245.89755    233.95119    221.35802    224.71728    205.35856    203.53088    197.42826     201.0329    191.61982    192.05595    185.31147    184.21653    180.06723    182.66756    173.48681     177.6126    174.15345    175.64124    167.31474    165.62942     166.8178      152.951     164.0694    159.06388    168.24415    168.93921    166.42658          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -30.00    249.91393    232.36081    221.25737    223.07499    203.93018    202.51366    200.28253    198.29686    198.01671    185.45571    183.84132      187.708    175.57267    182.72454    169.57364    180.87347    177.96048    180.46739    169.43342    169.85101    164.78608     160.4081    165.04154    157.09244    158.57317    166.69776    167.95857          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -29.50    247.32367    236.18817    224.68754    221.59323    204.25268    202.60551    202.93807    199.09248    198.08573    185.81456    189.78087    188.54759    175.20824    180.66523     171.0253    177.97324    178.19448    175.37367     171.2801    167.15514    160.24583     161.6739    168.12305    158.00874     157.9937    164.30463    170.51779     163.9061          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -29.00     243.9993    240.41602    226.21205    219.90804    205.33581    201.24502    200.62133    201.20384    195.78993    187.25406    191.37288    188.08615     177.0151    179.09007    177.38317    172.03799    174.51353    172.59514    169.43339    160.12253    159.24742    162.37226    168.22451    161.80388    163.36371    165.59796    170.85739     166.4654    165.80186          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -28.50    241.79344    241.31533    225.75327    217.91097    207.60786    198.59977    194.53811    202.22205    192.57614    188.37948    190.85537     182.7306    177.12743    177.97626    176.67467    174.63334     176.8079    172.45092    166.07724    163.33842    163.93953    163.09702    166.06337    166.52609    163.50707    168.96143    167.94946    169.32062    167.34319    156.47032          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -28.00    240.55876     238.2766    227.70015    213.91064    210.13285    201.00446    193.89907     203.0447    189.50515    189.65569    191.63545    178.56718    175.81022    174.95605    178.26305    180.95668    180.31611    168.94366    165.62491    169.19402    165.77308    166.47301    166.32912    168.56851    163.91663    167.76868    172.28801    172.43511    164.34707    160.83741    146.51571          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -27.50    239.14542    236.14966    228.64686    210.31745    213.19175    205.85643    194.98889    202.56488    187.93805    190.26616    189.70948    178.89528     178.2746    180.04565    179.69905    179.98276    180.84849     169.7647    166.36734    169.44164    169.15559    173.13758    163.29696    168.85748    167.71681    166.24062    179.88455    178.80007    163.79937    155.97536    150.44037          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -27.00    237.42545    235.33141    227.83366    210.94617    215.60227    206.23211    198.81398    200.23389    189.58593    190.19968    188.47391     181.3218    178.20597     186.5583     183.7623    177.73435    178.90508    171.17061    168.35334    169.35411    171.05191    169.69387    168.60411    169.97995    169.71268    170.33469    181.75937     179.8469    167.18982    154.98708    150.67356    145.28398          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -26.50    238.29063    236.53555    229.17287    214.24866    216.38038    202.81414    202.06307    200.08106    191.10626    192.11105    187.13958    183.96533    176.29785    178.49697    181.60186    174.06483    176.00979    173.80256    170.06554     170.8109    171.40764    166.86514    166.08902    165.25987    157.87111    171.04337    175.96146       173.58    166.78605     153.4961     146.2172    147.14621     143.9877          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -26.00    241.09994    238.52645    231.87796    216.67063    218.00073    198.82221    202.73525    200.56494    191.74237    193.96293    181.71209    184.27625    178.11992    176.42163    178.50138    176.68039    174.19962    172.21736     166.4239    170.78001    165.65661    162.02371     163.2763    164.34284    160.23959    171.09112    179.33474    178.20579    169.74315     158.8456    147.72089    148.99023    141.28904    144.39207          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -25.50    243.28537    239.54543    235.04611    215.15649    215.36022     199.8636    200.20066    196.66382    190.94621    191.14508    181.26731    183.74825    177.96643     179.1806    178.94443    182.39308    171.91825    171.73229    162.37285     168.4101    167.04856    163.70482    166.82388    164.28123    167.24951    170.74873    183.39447    179.87288    174.92971    161.64644    153.98109    150.48206    146.06195    146.91645     146.8049          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -25.00    244.45648     240.2089     231.6779    210.28076    213.02204    199.39722     195.4597    194.68805    194.17889    194.14437    184.66255    186.16326    177.46785    178.09855    179.42675    183.06559    172.98846     170.7834    165.19813    167.34855    164.36833    164.00462    166.96498    164.02608    168.77721    175.42438    187.60076    179.78627    176.28111    165.72005    156.24261    148.53649    145.96237    152.02563    143.02974    149.44679          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -24.50    245.09856    241.82592    229.47349    207.68444    210.79949    199.51575    198.52627    195.21863      198.465    200.23188    184.94139    184.50631    177.85879    174.85356    178.63605    179.99182    173.69377     171.2818    167.89978    164.56737    164.55985    164.68169    166.92463    162.01438    167.11419    174.94799    180.41724       174.21    179.32624    164.02836    152.37305    147.34518    146.56635    145.32084    146.24897     145.2324    142.47521          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -24.00    246.22719    244.52158    227.92065    212.89914    207.34315    202.44006    202.59098    195.23221    198.34121    197.25796    183.39654    174.59192    177.78908    171.44606    175.67475    180.19216     174.4314    175.97027    169.65191    165.43358    167.71823    165.76139    163.61964    156.64489    164.69799    170.17638    181.52506     181.8215    173.56466    159.62641    152.84079    150.82577    146.44857    149.86003    149.83545    144.40515    144.32176    141.20183          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -23.50    247.21572    245.45224    224.90352    218.40935    205.62628    206.98436    204.95617    196.99791    197.61105      192.993    184.06955    176.95748    181.01461    172.07015    173.70697    178.54648    173.47003    172.17368    168.53342      169.981    169.18379    168.36145    168.19305    159.39582    158.68357    171.19554    184.24962    186.02616    177.79695    165.46997    154.36605    150.62849    149.10139    148.99238    147.38643    146.12833    143.57532    140.49285    140.51188    140.84337          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -23.00     246.9553    245.16644    220.54544    221.89713    208.32164    206.32408    206.97174    199.04826    198.59764    189.00374    182.58307    182.58219    184.84019    177.18356    179.18094    176.82747    174.14081    165.41634    168.31186    168.91168    164.93739    167.60727    171.36813      159.692    162.50622    177.40571     185.9236    189.50653     177.2671    168.87516     153.7187    147.82051    149.74186    153.43254    146.51492    144.34145    144.27508    143.43036    140.11002     141.0127    139.97829          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -22.50    246.98158     244.6419    217.64779    217.66902    212.05827    204.64761    207.94102    201.69111    200.98989    186.90472     180.1237    181.08212    185.94833    182.85437    183.40828    176.97587     174.0611    167.07101    168.61835     169.5466    167.61796    166.12703    168.73835    162.33068    163.30098    176.73311    179.42906    188.59736    175.59917      171.998    155.58338    154.13275    151.87961    152.46525    145.64014    144.91892    143.15361     143.6245    141.11237    134.83114    136.45387    140.03631          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -22.00     247.6358    244.65102    218.68601    212.05316    215.08851    203.65808    205.03684    206.33317    201.27733     188.1274    186.63394    180.05736    188.54708     182.2061    176.52151     176.7246    173.86894    168.94291      166.227    171.38948    170.42286    167.07145     162.6415     160.4058     159.8983    176.00677    182.31952     190.2715     180.4047     167.7014    153.34692    151.50118    151.71238    152.68071    145.84488    146.46288    143.95581    141.19591    137.77812    139.34491    139.96301    139.82565    138.01098          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -21.50    250.67366    245.69983    220.54825    211.46406    217.16135    202.09265    204.13005    208.43678    200.40153    191.54118    191.59399    180.79252     188.0756    178.98329    172.95408    175.24958    173.95151    171.24056    163.91689    169.79744    168.81822    164.99209    161.98091    161.61346    163.38014     173.7086    184.33774    189.56683    183.34835    164.04646    153.44325    153.28023    147.87113    150.34031    149.67054    147.24402    144.65722    141.58024    143.22331    141.93089    141.20093    141.52318    143.58875    132.85003    133.99081          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -21.00    252.93444    247.81167    222.22697    217.12087    217.99692    200.57664     208.1701    204.21776    201.83448    192.74507    187.92832     182.3754     184.9497    178.26421    179.70719    175.03491    172.86382    170.52601    167.36097    166.20155    167.19203    164.77116    163.88977    164.50264    164.51649    173.29968    182.99441    190.97611    183.72941       163.42    154.54857    153.66936    149.36653    148.02774     149.8063    147.53291    142.15439    139.02095    145.75921    142.20642    140.96071    136.71977    138.52322    132.77027    135.63662    132.46807          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -20.50    254.37733    249.41148    222.96521    221.99981    216.80816    205.24874    210.04445    198.90344    202.22245    189.30299    186.90146    183.63686    181.22974     182.0816    184.53037    169.66225    172.98644    168.58817    169.64291    164.59806    163.83852    164.00267    164.33562    160.23892    161.71949    171.54485    184.00855    189.63744    181.10079    159.75468    156.20746    150.97906    149.04535    149.61208    144.90471    151.21755    143.65594     146.0335    144.43103    139.12111    141.38524    140.06075    139.02441    137.89195    135.79187    132.16351    132.62115    130.93661          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -20.00    255.13407    249.70911    222.56708    221.30542    215.03499    209.44109    210.59596    199.86238     201.0675    185.90512    186.72777    185.52297    181.27156    181.22195    180.28284    164.16211     171.8938    172.20837    168.84236    169.64624    165.75563    163.92367    165.67242    162.52853    162.67346    168.14064    183.60652    189.68666    180.31941    159.42336    151.95412    150.51742    149.13472    152.06316    152.69455    150.17419    144.19449    147.14198    143.51912    139.32319     142.6632    144.32249    141.93956    130.11191    136.62227     134.7141     136.4726    131.29419    134.93649    128.90006          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -19.50    256.22005    248.78507    219.88409     219.4063    213.52725    209.76298    211.70885    202.60786    202.07829    185.51186    186.80763    184.68741     185.5822    178.32089    177.19774    170.72226    168.08216    174.77654    170.30494    174.02148    167.72168    167.43153    158.09674    163.00281    170.67182    173.02203     185.4992    195.31433    178.63278    159.79266    156.11265     148.4587    149.78412    148.26713    151.01315    147.85499    146.93222    145.05991    142.87093    141.86271    145.74048    143.74586    140.64877    137.84799    137.32268    133.52216    133.94269     132.6978    132.93128     134.6761    131.74743          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -19.00    256.18668    247.85171    217.60265    217.78776     217.1701    207.52735    211.49701    201.71776    201.79318    188.26448    180.65303    179.36025    188.86134    174.89756    175.91606    175.64956    169.01606    173.77669    170.50764    170.05903    169.60432    169.04254    158.18767    162.37744    168.53492    173.56605    186.13688    193.96013    187.78033    163.43523     156.8582    154.80256    148.44942    150.21895    149.27641    146.70226    145.42522    145.02979    140.56706    143.00698    145.66337    142.37306    140.15588    141.67602    139.55334    133.27262    141.08174    133.07614    133.87046    137.52643     128.9787    125.14918    125.95431          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -18.50    255.21469    246.26375    217.45083    216.48003    221.87793    205.06792    209.33927    200.52324     197.9569    191.80188    177.74612    182.41884      188.272    172.17809     179.4538    174.05841    167.84674    168.57746    172.06685    169.64965    170.02267    168.95442    160.89385    166.54801    163.76621    175.46815    191.17502    189.99717    186.10976    161.47086    154.32337     152.3037    145.18143    152.30274    151.50399    145.56247    144.79109      143.517    145.43353    145.89774    142.93204    144.98177    138.88014    136.59475    140.74833    140.36646    135.62362    136.38778     139.7884    133.71881    134.67845    133.58696    134.96848    126.35525    131.55962          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -18.00    254.61358    245.89361    223.99321    214.53782    222.35945    205.65961    208.99786    202.82712    194.47807    194.26421    185.11086    188.70508     184.6712    177.25547    185.72734    174.24655     168.0371    164.65787    170.06978    169.09675    168.62498    166.93756    164.38774     164.6959    171.88815     177.1906    192.56037    195.05063    183.94478    163.39782    157.00492    152.98354    151.24767    154.91174    149.36454    144.96313    142.54128    146.60324    146.77736    140.80706    140.00829    144.00779    134.99045    142.28837    138.14908    139.48781    134.57333    138.05033    134.22345    133.13488    128.39726    131.35749    131.40463    134.63433    131.01329    129.32715    134.09011          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -17.50    254.35552     246.0896    229.08046    214.60154    217.86078      207.749    209.46255    206.16617    193.03619    193.21978    189.36043    189.06166    180.92117     180.0531    184.91615    175.07913    168.05231    170.70554    169.52646    168.88584    167.03698    162.13095      160.258    158.60658    168.40433    177.97228    188.45271    195.55035    182.11994    163.09789    156.51308    154.43823    150.93834    150.90541    149.43692    151.99813    144.45821    146.96285    146.73238    139.53837    145.04103    142.80416    139.92541    139.46408    138.52546    138.52565    136.58152    134.39198    135.17661    132.70438    131.82137    130.14908    135.36388    125.51329    131.35186    134.76321    135.87837    141.37661    141.08988          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -17.00    254.70439    245.67637    230.81146    215.50778     214.5434    210.33793    208.55817      207.577    192.93998    191.96403    185.94293    188.29801    180.88273    180.08296    181.61764    177.84135    169.55775    175.82772    172.71331    169.43707    165.42457    158.03123    161.90141    158.34613    166.46596    173.78143    184.43777    194.33523    180.70355    165.25991    151.37653     150.6962    155.56689    150.83184    150.21744     150.0615    149.65598    147.28047    145.31521    142.00649    146.24698    145.00138    138.46045     136.3334    133.27359     136.3634    139.24933    139.79321    134.74531    134.73293    132.77641    132.34073    133.66772    133.80127    131.09434    132.27532    142.89658    145.64542    149.42562    146.91403    138.95396    128.87436          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -16.50    256.18214    244.29113    231.40787    216.21948    216.03198    212.11739    208.40725    207.13881    192.30773    194.32578    182.98251    183.41802    181.84482    181.05689     178.5811    178.99596    171.42974    178.25165    170.33825    170.31951     163.7016    163.20069    164.45819     159.9867    167.82844    169.41425    186.95451    196.44544    182.34843    168.21565    152.20402    150.54595    155.46552    155.77015    150.87255    148.18822    147.39848    143.59063    142.66038    143.38212    145.28773    142.54337    142.48054    135.55371    136.02477    137.76869    139.22229    140.40798    135.73611    139.27487    130.46167    132.66861    132.66117    132.84264    135.29826    133.20419    139.13825    146.84559    153.38326    146.51666    135.46496    132.68197    130.20172    126.21957          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -16.00    258.73191    242.12193    233.40198    217.79659    218.17617    210.68392    209.67264    208.13934    190.98526    195.81904     182.9246    177.57036    187.01948     180.7647    177.21664    177.23022    172.15552    177.30625    166.82884    168.12544    168.24372    161.87034    165.25025    162.58975    165.57464    168.06651    191.90517    197.49656    181.67522    169.11265    157.20076    149.79749    151.62031    151.71095     147.7651    146.14707    148.64478    150.95227    141.99984    143.40517     142.4557    138.56797    141.95034     138.3109    140.52177    136.78436    140.64726    135.92756     132.8832    136.15226    133.17513    131.95297    133.01842    135.60556    135.94733    131.18282    142.15993    151.88002    153.42218    143.35582    145.13965    133.28326    127.71352    127.18033    123.70346    121.70656    125.59393          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -15.50    261.35592    240.06167    234.43086    218.00384    219.71747    207.93266    211.33288     211.0989    189.51775    192.59036    183.07333    184.81683    197.00342    183.07474    174.07273    176.26749    172.77799    175.95156    164.42304    168.07682    167.97548    159.11891    162.04489    161.85981    165.59936     171.5238    193.23496     199.5861     185.1081    166.66651    158.65625    155.42406    149.35876    151.16206    147.89529    140.94944    148.21115    149.97998    146.25825     142.8854    149.97533    139.44786    140.36084    139.46727    136.84624    136.71476    137.59208     139.0693    136.66581    134.17673    137.17955    133.89415    134.35211    126.92905    133.74797    134.20512    145.38325    151.60749    155.78138    154.18035    142.08772     132.8748    127.61317    126.11001    126.25704    125.10359    122.94964    124.43678    121.95657          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -15.00     263.4714    236.98499    234.21878     217.6313    219.96468    206.83866    213.21422    211.88449    191.56833    189.72425    184.55042    192.07543     193.6469    186.26383    178.70747    173.05205    173.84696    173.00586    168.55541    169.30852    168.37003    168.60438    160.82137     157.9033    165.32563    171.24029    192.63937    196.37886    186.17577    166.98995    160.59077    152.58798    152.75276    147.83366    147.87315    142.18484    146.13452    145.65612    145.16879    144.94801    145.18449    143.69564     134.9614    144.07698    142.39053    137.92604    139.51262    137.75976    137.04975    137.44615    136.24559    134.91976    136.77701    132.21672    129.91779    135.96054      147.734    153.97109    159.20441    153.48449    145.08486    133.70943     128.8237    129.38968    128.83336    125.52823    120.55162    125.31263    123.74162    124.01605    128.78947    131.96384          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -14.50    265.06106    232.30938    233.23389    218.79806    219.54558    210.19709    215.32926    206.88798    193.67547    186.33561    186.56653    190.69788    187.56155    186.49914    185.92287    174.08344    174.55707    173.04052    174.87204    169.15987    170.66832    164.70633    161.36842    160.24791     167.3509    169.71804    190.19035    196.89574    182.97794    171.07506     157.7184    153.85782    152.46391    150.35057    151.20507    145.25817    146.77179     145.7396    143.67802    144.11552     148.2726    140.69625    141.28985    143.42256     144.9411    139.11468    133.69908    135.09922     138.5499    135.47753    133.54377     140.3708    132.66797    133.46888     140.0157    139.85502    147.91408    157.30308    159.29641    157.23221    145.31238    133.39899    129.07245    128.11896    127.74097     126.0133    124.74422    125.50268    125.01558    131.88155    131.69224    127.40642    133.17776    133.07226    125.90792    126.47554          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -14.00    265.47044    227.62349    232.85535    220.98377    217.53004    214.02914    213.50059    203.36234    194.18999    183.22442    187.85716    190.60487    187.88585    186.92183    176.94647     175.1932     174.8449    172.99793    168.42184    170.12866    168.88226    158.46098     165.2613    165.47083    165.81328    170.54085     189.2654     198.5093    178.66422    173.08364    159.00595    158.41511     158.9442    154.75724    147.96454    144.71101    146.49451    146.69838    141.00774    145.09669    141.58168    138.59002      140.054    142.99071    139.23379    139.05505     142.1652    136.05769    137.99583     136.6586    133.71596    137.36993    138.19846    136.40343    137.06373    137.39026    146.70286     155.6832    159.44182    154.73446    147.30463     136.8753    130.32952     129.5625    124.96721    126.70462    125.68427    125.64437    123.32344    124.85724     135.2769    139.38837    140.17248    134.38112    125.20035     123.3854     128.2708    124.20695    123.63047          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -13.50    265.94105     228.9349    231.96856    220.01063    214.59517     217.3572    211.67686    205.31547    193.61081    186.96929     188.0084    189.80611    188.45368    186.06472    173.40439    174.33846    174.76075    176.60412    168.02421    170.44634    167.93984    161.43514    164.46501    165.12755    164.55876    178.30845    192.57302    197.73679    186.56442    168.95375    153.65277    156.85773    155.95103    155.60448    148.48098     143.1479    147.37473    147.30028    140.86424    143.40842    144.78157    140.57694    139.84341     145.7829    138.91952    141.09126    139.23454    137.74264    135.24751    133.66052    135.56824    137.58392    135.99541    137.55712    135.26731    139.30246    148.35753    156.93367    163.36858    156.39577    148.22372    138.65324    135.38577     128.9145    129.49375    130.25759    123.19069    125.97338    120.75581    126.84452    133.03638    141.58081    141.39015    137.88505    129.23261    129.82944    128.09252    127.93117     128.8617    126.28062    123.27828    118.32373    115.25928          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -13.00    265.58563    231.72097     230.6614    218.45853    214.10742    220.71203    213.48555    206.48993     194.2497    190.49308    185.74831    189.08638    189.06862    185.08982    178.17467    173.90189    177.25457    173.74085    170.47451    170.00277    168.58073    160.25266    158.66968    168.25816    163.16771    176.64166     194.6808    195.99169    187.87765    165.15085    153.23387    153.23016     155.6402    152.11466     148.4191    145.25266    146.90661    147.93503    147.01159    143.59339    144.26028    146.16586    135.08604    140.84376    139.31839    138.27637    137.41176    137.17066    134.68566    135.35932    136.32801    134.05617    135.99349    137.56029    134.26792    141.61262    147.43829    162.14704    166.27471    161.58994    147.53219    133.32921    130.35595    127.78576    125.32027    128.33223    129.35072    126.79222    133.48792    129.27207    134.29189    142.59818    141.59998    140.27464    131.65055    130.87965    130.85209    134.38184     131.8496    124.04494    123.40895     120.4996     120.3142    114.81404     116.4111     114.7018    111.72633          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -12.50    262.01085    234.36975    229.74567    218.15514    215.61204    220.78748    215.51054    207.49612    197.22532    189.03535      183.024    188.75231    190.34402    185.66794    181.05129    174.00641    177.93167    169.94672    170.93301    170.12396    163.96517    160.36951    164.68764    167.18664    167.24187    173.59755    193.09857    195.41426     183.3116    166.07041    158.53779    156.56252    153.35834     147.1997    151.51313    146.88291    150.38698    147.06083    145.63294    145.94709    142.51861    141.56858    140.21206    140.25604    142.24062    142.96811    136.07217    132.41287     137.4323    139.10655    138.77976    134.72601    136.99359    138.78647    132.73084    138.51159    144.77124    158.25348     163.4186    158.72058    149.86326    137.61025    131.50168    128.93159    129.10574    127.69063    129.93483    125.92517    124.54772    131.81101    138.55672    142.30118    141.77967    139.72747    134.24104    133.10836    135.35446    135.74019    130.79712    130.16346    126.67669    124.71229    119.96302    120.53458    117.83974    114.77554    113.27624     110.7697    112.47226    114.15614    116.33441          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -12.00    258.14292    234.23957    229.09402    220.85184    217.03197    220.12634    215.89228    207.44129    192.91825    188.24061    181.28686    188.77639    189.44557    185.57274    182.88305    175.37639    178.16947    173.05052    172.88151    172.02412    163.77611    161.09641    164.06163    163.32069    163.34485    173.73149    192.06353    199.72458    183.38046    171.18791     162.5679    155.84486    149.85128    150.01996    150.82543    146.99711    149.91656    147.67956    144.31213    146.49713     141.0125    142.02041    140.52836    139.04204    142.83683    139.10758    140.97457    137.97131    136.73512    137.32194    137.46255    133.89332    137.10851    138.23763     129.7025     134.0843    147.06685    162.53431    162.49764    162.38988    150.13468     136.7886    137.10734    133.68981    130.13496    133.38396    128.33551     127.0519    126.31291    129.98442    137.05728    143.51501    146.30614    143.49395    135.98777    135.43856    135.13068    135.40544     131.2381    130.82495    124.37232    123.56047     121.1424    119.77818    117.54761    117.35142    112.42341    114.69567    113.05188    113.37619    116.01702    117.78403     115.7832    113.32313    111.36715    109.66285          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -11.50    255.87167     233.8663    230.33901    223.51751    217.25663    220.37744    215.05625    207.84121     188.7736    192.15447    181.18214    193.02602     188.3243    184.71787    178.58422    177.76363    176.62588     175.1369    174.12724    171.50037    165.04628    167.62546    163.35141    160.88036    167.12737    175.36338    193.87519    201.01352    189.53958    174.00176     159.0799    155.60319    150.10123    151.64248    149.57873     146.4939    146.11456    144.90436    145.10648    144.82637    139.98705    140.96004     138.4773    133.62064    140.65892    138.37121    139.54118    135.54822    135.97812    138.17543    134.80225    134.86237    135.10503    139.13213    129.93592     140.7867    148.79136    162.92404    161.98087    163.94448    150.88915    135.31353      137.487    131.35649    124.83791    130.67774    128.70398    125.16218    125.82544     134.6209    139.32057    148.21513    149.83828    144.21924    141.03844    132.50184    136.42112    136.89357    134.81927    129.65639    127.13118    126.36913    122.09477    120.59571    115.68757    113.82725    114.59222    113.88871    115.73099    116.77175    118.55656    119.68494     119.2916    119.06217     114.2481    110.94098    112.23282    111.83665    107.83908    109.53604    113.91796          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -11.00    258.31939     234.0241    231.95415    225.10978    216.80247     220.1149    213.85227    208.67132    191.68042    195.68159    186.56748    193.15297    187.96764    179.34843    172.87775    179.00088    173.76778    176.65905    175.81542    170.82267    160.54558    166.60444    166.86182    160.34433    171.64409    178.52804     191.6512    197.52641    192.78409    175.00331    161.12851    154.88375    150.92575    154.29763    149.41735    147.17455    141.30154    144.57843    145.69697    143.75571    142.67729    140.92091    137.07816     137.7532    144.26571     143.7903    140.24801    136.39302    136.85308    133.34358    136.60261    137.15793    136.70158    134.56086    134.42679    137.22605    148.74767    160.60016    169.19113    165.72595    148.35377    141.82005    135.08695     131.3522    133.45233    128.99773    129.58088     127.6264    129.46594    129.95059    141.54336    149.00006     151.2033    141.06655    139.81347    134.60849    145.57012    137.85859    141.20288    132.15024    131.08096    128.51784    125.36311    123.38063    116.81322    119.54964    117.29001    113.99973    116.49139    117.03437     117.4475    120.14889    120.93014    118.59488    118.90111     114.6546    112.66973    112.00076    113.43718    112.82244     116.2289    115.29853    113.76427    112.41296    111.29106     109.8225    108.72256          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -10.50    261.71704    237.02858     232.8555    225.66601    217.51105    219.14178     212.2376    206.74311    195.00778    197.50105    190.75449    189.90901    186.58066     174.7609    173.26223    179.13832    170.81088    174.45134      176.226    172.00302    164.28815    162.35265    165.22684    161.07449    171.83944    177.34597    194.02372    196.98275    192.79401    176.64725    154.95742    153.02182    150.75566    150.02961    148.98716    150.30517     147.6367     148.3782    143.69738    142.21785    138.32895    144.05477    137.10107    138.40192    142.73004    142.48375    138.25492    136.54749    137.33197     135.3775    133.63328    139.52104    140.55718    133.42436    136.50179    136.63127    148.05072     161.4663    165.41917    168.06774    149.25802    143.45882    132.41101    129.02001    131.48292    126.97314     126.8252    130.98089    129.45757    131.69816     140.8325    145.06606    156.34679    145.46718    141.30301    137.63417    138.68158    138.94251    140.68277    136.02254    131.15254     131.6683     127.0024    125.04941    122.00901    118.86283    118.79676    117.29973    119.49327    115.74944    118.93647    124.26366    123.19316    117.19229    116.70128    116.29028    113.07719    113.95336    112.58638      112.157    112.40308    115.77832    116.56649    111.62347     111.4998    110.13742    111.39048    113.53453    111.76312    112.60959     114.3083    110.25147    107.03232          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
      -10.00    264.53754    240.16885    231.05399    225.38347    218.26171    218.79508    210.89281     201.8358    194.73757    198.86571     190.5094    188.71859    185.81119    178.66498    176.35107    178.22539    173.47115    172.28865    175.36835    172.05603    170.07579    163.32383    161.05988    161.63454    166.87015    179.44842    200.21164    198.15105    195.25295    172.83406    159.72817    154.09006    148.64983    149.88463    148.66316     149.7231    149.80484    151.22898    149.68575    142.86128    141.48711     145.4651     142.1884    141.77897     141.8023    142.10728    138.76153    138.72265    136.34527    139.08875    133.30747    134.24951    137.52076    135.92276    135.68018    137.73911    149.00023    166.37643    172.13236    168.73347    146.29159    138.66106    132.74179    131.11855    130.09917    134.27247     125.6828    126.43702    132.23039    128.91583    137.77505     149.5546    151.57858    147.57938    141.30084    138.67153    141.73734     143.6398    139.09664    135.45909    133.34754    131.08133    127.56827    126.08102    123.88364    117.25899    117.21731    117.50748    117.91667    120.10664    123.80725    123.50015    124.73449    119.85098    119.99996    117.90147    119.28428    116.02427    115.40391    112.51248    112.32821    116.25135    113.61774    116.24813    116.08662    112.06894    114.49082    115.19781    113.76528    113.13491    115.27143    111.48089    109.77378    114.06665    111.34178    115.14713     115.2296    113.02612    113.22445    115.07964    115.12728          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -9.50    265.59821    241.78324    230.23895    224.75152     218.3864     218.7409    213.25722    198.96898    193.13234    194.80144    190.07809    188.19162    186.94615     181.9848    181.16214    173.62592    175.68459    169.78318    173.31284    170.24486    172.30921    163.58063    161.43579    159.22074     166.6015    183.28759     194.2142    200.37936    201.01785     178.8492    167.24245    156.01998     153.0623    149.91431       149.48    151.21219    150.52568    149.30232    145.17575    142.81001    142.88924    136.95016    142.53865    141.57921    142.48043    136.91998     137.9332    136.94422    139.11034    137.89159    134.51403    135.83911    133.92421    133.13904    135.51362    139.40164    147.96115    163.45481    172.04972     165.6965    152.89382    140.16614    136.58752    129.86818     129.0326    129.24915    126.95976    125.18732    125.84795    129.30949    142.64881    150.20485    150.95826    150.70049    140.38492    140.71064    141.97255    143.13657    139.56956    131.44781    131.04687    131.10857      127.263    126.59576    123.59691    118.73478     116.4939    119.62293    116.01625     118.0924    123.89891    127.16968    124.49413    124.06983    120.53349    119.79329      117.609    120.42409    115.28559    114.31089      118.131    117.68277    119.20386    116.82635    118.00413    112.09254    109.95031    114.07073    118.12312    113.67438    114.08002     111.6639    113.11323    113.12248    114.38819    115.97347     114.7551    118.46241    110.87246    115.28726    111.94425    112.90295    111.19073    112.29413     113.9425      113.245    119.00083    118.35571    117.08713          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -9.00    266.77831    241.07356    230.30635    224.42786    217.67214    218.82474    216.03042    200.52182    194.22524    190.74148    189.15246    187.59485    189.42372    184.50578    179.80532     169.8935     175.8733     172.6346    172.80397    168.21379    168.65496    164.02667    167.57346    161.87176     163.5387    182.39136    192.29654    201.93136    200.95991    175.39665    165.28275    152.40857    151.98073    150.63026    146.64386    149.26165    149.05868    148.59368    151.87253    145.08847    141.92968    144.08404    142.36566    140.34211    142.59616    143.78475    135.01447    134.83277    137.82697    140.38968    138.60357    137.87896    136.90041    133.22895     130.1625     139.9003    148.04806    160.05007    172.56308    166.22916    151.74309    139.83872    137.46224    129.50101    136.12338    128.48465    125.93025    127.88243    125.24297    133.41885    141.52309    157.50538    153.11391    149.22096    142.41386    140.62633    142.18044    138.82112     135.2111     136.5231    128.86075    129.59417    128.34743    125.08247    121.17344    122.12382    120.37328    116.28869    121.23831    121.73348     125.6345     123.4573    124.86771    124.65465    123.00887     123.4085    117.16984    118.87146     116.8335    118.38501     118.5494    117.62525    118.72741    117.01113    115.77651    111.85641    114.98221     112.5995    114.82291    114.19104    116.09323    113.02439    109.26985     113.1057    114.44616    114.15635    117.10157    120.20999     116.1116    115.28387    112.60321    114.84957    112.90619    117.44906    113.69079    119.53715    117.86303    118.26913    118.93564    117.30191    114.14797    108.37581    112.31681    110.54889    112.78443    113.85582    116.46225    118.19598          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -8.50    265.73824    240.13208    229.93861    226.09997    216.54509     219.1724       216.09    203.88539    196.61003    190.37699    188.55626    188.33654    191.39028    185.10169    177.38714    171.42065    175.31488    174.85736    173.19236    168.53445    167.09103    167.73236    169.06101    162.17116    161.18906    181.33202    195.51466     203.3378    193.56392    178.92309    161.19038    153.93126    151.09328    150.14408    148.99795    148.45583    149.75441    149.41731    140.04762    145.20719    141.75623    140.58652    143.98845    144.00656    141.83593    138.62381    137.58978    137.38346    134.64333      138.675    138.19907    138.63966     137.1856    130.80522    135.39764    134.92639    149.64608    160.44486     172.7937    165.55812    151.38135    140.89223    132.71594     133.0784    129.98899    125.42957    123.96373    126.47666    131.72536    131.64959    142.99593    147.81512    157.87845     150.0564    142.81651    142.07804    143.80164    145.52945    137.61394    129.97401    134.24063    132.38737    130.38402    127.78793    126.04112    118.46017    119.74101    118.45208    120.45415    118.67639    121.48066    127.63459    125.56481    130.28189    121.59949    121.17477    122.48319    118.04592    121.46195    118.90434    116.75037    118.65196    122.09121    119.91252    114.88833    110.12379    115.07677    115.61158    116.00112    112.24377    114.12197    112.50734    113.56309    115.54703    114.86263    116.65832    117.95217    117.78088    116.11299    116.94976    115.18592    111.62655    115.32808    113.92244    117.44172    121.92287    122.97633    124.91428    120.59078    115.89225    114.05878    110.95086     116.4664    115.39057    112.89702    118.44142     118.6338    122.84409    127.61503    135.31883     133.0984     130.7999    125.62553    119.83718    112.62381    115.83486    115.98384    113.49183          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -8.00    262.17411    240.40204    228.83536    229.24807    218.11095     220.5412    215.95123    210.48224    197.67468    190.64942    190.95811    186.59749    191.50249    185.20395    178.59166    173.26564    174.78268     172.0215    168.98232     170.5544    171.64652     164.7754     169.6561     162.7912    163.45053    177.71513    197.11299    205.31886    193.56784    180.12204    162.06188    153.59561    150.49127    146.63939    148.12262    150.41782     150.4224    147.05185     150.1419    149.84942    143.11152    141.26535    141.89273    139.36168    142.35925    144.20733    138.83753    138.54088    142.92206    134.94075    140.11022    139.28477    135.00976    137.58247    135.41135    141.96087    146.10201    161.17042    172.31699    168.39826    153.97908    140.15134    135.68989    131.72493    133.45235    130.49437    128.73098    129.96534    129.63498    133.38243    142.33256     148.8618    154.16657    150.64562    144.21304    143.35548    142.93174    145.49776    142.24818    135.90969    134.02559    132.02955    131.87636    128.88783    126.91094    122.25582     117.7464    118.40157     117.7958    122.11407    124.48195    125.99764    128.26888    126.44041    121.40208    121.01568    118.78938    118.99866    118.07142    117.71833    120.28146    121.85927     121.9822    120.38265    116.93782    115.03723    109.59553    116.42499    116.91767    111.51175    116.68931     111.8005    112.05898    116.10011    115.46573    113.95401    114.29847    120.38707    120.09417    117.27243    117.36256    114.99208    115.38242    115.48356    117.32922    121.29019    124.29537    122.58771    119.75001    117.18233    114.96733    112.54576    110.25987    113.45614    116.93512    118.55684    118.28437    123.54242    130.46909     137.0314    135.77798    133.69311    126.71417    122.66628    113.90134     114.2443    116.24922    116.70763    114.52655    113.23355    111.55614    112.80718    110.59082    110.54621    107.96947    107.96154    109.71516    110.88109    112.39547    113.31517          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -7.50    258.07578    241.07852    227.84533    231.95166    220.72057    223.90158    216.45603    216.02008    198.36786    192.77381    193.79324    183.42301    188.50129    180.26686     180.5334    173.93133    171.86383    173.98437    166.10216    172.86121     169.9315    165.04879    167.28286    166.83736    164.15423    177.13562     194.5573    206.11179    196.13447    176.20433    160.75757    155.44399     151.5803    147.13666    150.07683    143.34778    148.69001    149.02457    150.85628    148.04331    144.08568    144.22095    143.14511     141.3113    139.87194    143.37685    139.07138    136.24233    139.83089    139.31486    136.33452    139.31203    139.40479    132.64142    134.10265    140.38015    148.65216    161.75787    170.79371    170.21104     153.3604    138.83207    132.20453    133.39321    132.83235    129.57647    127.16198    129.80495    131.45189    129.53075    138.41233    151.94569    156.19836    156.81802    145.72622    142.36607    144.56321    146.42653    140.48437    138.83708    136.12463    136.77852    131.08044    129.63562    127.45266    122.01496     117.3616    119.67448    120.34347    120.79284    126.37781    127.97486     127.8112    126.10408     125.7017     122.0754    120.61233    117.56629    118.20814    120.23821    120.98022    123.30366    118.47192    119.22726     120.3831    116.33542    114.82263    112.95092    115.53087    115.35755    119.38505    111.33233    117.71691    113.40904    115.66581    119.19406    120.80838    120.45224    120.14166    119.73891    119.54978    113.76888    116.43525    113.99566    116.89681    121.31271    124.45472    122.12131    123.11021    121.27581    118.18464    111.10476    114.88381    113.88253     116.2728    116.66292    121.03742    124.81559    134.11153    138.22943    139.30642     135.4935    128.40439      124.173    118.45732    115.79294    117.18362    119.52463    116.14817    116.23043    110.61333    114.28093    113.37227    112.41803    111.27652    111.53141    112.68184    111.13878     110.4228    116.05866    113.45364    115.03082    111.46389    112.44103    111.76304    115.62466    118.06547    117.31524    112.16373    113.46141    116.14883    117.59896    117.57313    115.22597          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -7.00    258.59669    241.52336    227.08461    230.30393    221.46932    227.60497    217.31763    209.98919    202.20702    194.84384    196.30179    185.37683    186.71856     175.1019    183.62886    169.82965    170.90461    174.95253    169.97312    171.99243    167.80972    167.80199     162.3611    165.04881    165.67838    180.76764    196.73399    203.43177    197.85005    175.73464    163.52681    160.26891    153.80765    148.80574    150.58055    147.63022    148.12438    149.93815    152.35955    145.37541    147.66505    142.66075    142.59572    141.28638     138.5714    142.45414    136.18912    136.24746    139.31476     140.5632    137.96692    135.56762    136.91508    132.93634    130.28634    137.19164    150.64061    162.83594    174.75799    166.14525     156.3019     139.8072    139.69996    132.78457    131.30099    129.56379     125.0471    134.71905    129.24739     135.7003    139.73691    155.57555    155.63284    152.97593    147.54256    145.21783    146.40835    146.46886    141.36539    141.98681    138.57172    138.12876     133.7578    130.85256    128.46525     123.0975    119.82221    117.96398    120.47667    120.99391    124.71588     127.2122    128.97914    129.77027    126.25388    123.82765    122.06899    122.63219    121.56567    123.92503    121.35706    120.99197    122.54939    121.50543    120.03783    116.68442    113.54194    114.52415    118.25674    116.14133    116.31781    114.74534    114.25453    115.01943    115.13516    116.84487    116.98946    122.03764    119.53594     117.6122    116.56378    118.22212    114.60894    118.80889    119.75267    126.31338    123.66869    125.41455    119.11438    119.36806    113.91733    114.86693    114.67605    111.70315    114.39085    118.49035    120.19273    126.56669    138.99308     144.4328    140.45669    137.08381    128.63175    125.24209    121.75181    120.15564     117.5148    115.70666    118.94361    113.90687     112.7359    114.18993    112.51405    112.42053    111.09998    111.67712    112.80626     114.8565     116.6459    116.44743    113.89382    115.71351    113.42788    112.06746    116.57004    116.95401    117.94527    119.97496    116.06139    114.51514    116.35894    119.33678    116.58913    119.24541    119.69432    117.80252    117.76949    120.13379    120.92026    118.05384    117.12699    116.14484    113.35946    113.85391     114.3581     113.1729    116.03577    113.15084    112.15329    111.18129    112.89197          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -6.50     261.9764    242.11431    227.09066     228.2722    220.53353    226.05546     215.7866    203.29806     208.6172    195.08445    198.32803    186.90293    187.61035    183.22494    182.61234     167.7842    172.52533    173.78344    170.14943    171.35715    165.55924    166.83695    163.81495     163.5137    167.00688    181.28149    196.07843    205.81739    196.88243     178.7702    160.86205    157.77477    155.44397    147.42866     147.4841    144.58145    148.83209    150.59294    152.69912    143.06072    146.00051    143.21516    142.07613    141.84751    143.75472    138.66459     136.2835    141.54303    137.33806    138.81106    141.26474    136.90401    132.65026    134.94386    136.14196    140.47058    150.45573     165.3154    173.03688    171.09259    153.12699    142.35948    135.12738    131.07019    131.67801    129.08143    127.50347    129.71076    131.60685    132.30133    144.71616    154.18766    160.30571     150.0675    147.59214    146.74297    147.21798    146.66286    140.60633    143.15745    139.49751    139.77557    136.50159    129.90861    120.09223    120.33452    117.23104    118.54922    118.48802    120.96267    125.96376     129.1599    130.49935    130.97711    129.05386    125.28456    125.25656    123.56162     121.6909    122.58577    125.24327      124.461    120.77965    119.96858    117.87045    116.90045    115.74158    117.10694    118.05497    114.99219    118.13097    115.60202    116.00378    112.85271    118.40936     115.1952    119.43674    118.47884    119.78033    120.31566    115.51483    119.03754    115.56374    116.75293    118.39481      126.371    126.19113    127.97169    121.35135    119.50984    113.40831    113.80331    113.71064    112.04917    112.09718    116.84708    120.93378    129.73703    135.70525    140.25988    144.84479    137.77052    130.68702    125.46722    120.43267     120.5745    115.38861    116.05642    116.66284    115.34244     112.9816    113.88842    115.76786    112.35843    112.92492    114.13416    112.70717    113.26091    115.68013    116.33394    115.33324    115.17228    114.79313    114.31843    116.87169    116.92219    118.25948    118.70563    116.60729    115.53925    116.61404    119.95607    117.73966    118.00294    116.83528    114.93273    121.55652    122.61211    123.97481     122.9249    117.27267     115.6615    112.90643    117.65371    114.30797     112.2012    113.21113    114.33858    110.36019    114.72407     118.5112    117.75819    119.18981    117.73746     117.3349    117.02012    115.57685    119.07152    119.35944    119.07876    116.63978    116.31529    115.96507    116.28748     113.4846    112.49407    118.94437    117.10052    115.23815    114.08077    114.27352          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -6.00    265.16566    242.91389    228.77594    225.21216    220.31998    224.37645    210.74399     202.7545    207.54928    194.78683    199.92288    185.12786    182.23226    188.00719    180.46375    172.62299    176.44651    175.80939    170.66661     172.0135    162.35639    162.97163    167.08395    163.37326    163.95715    175.66376    195.53769    209.49663     194.5389    181.80755     159.3548    153.57587    156.71369    152.39935    148.21608    146.49236     146.7306    148.12522    149.85375    145.49322    145.28852    142.47515    142.04769    144.76574    146.26381    136.41001    135.16727    140.95054     142.3458    140.29072    139.76174    135.53747    135.32658    135.68177    136.06513    139.78088    149.19414    165.66218    173.54548    172.47743    156.68467    144.93819    132.46528    132.31624    130.43793    129.67686    129.08447    129.22981    130.05284      132.896    141.02119    155.05576    154.58871    152.68823    147.67302    148.06196    147.16718    149.47155    141.30833    139.49802    139.95405    136.86328    134.42261    132.46778    124.98012    124.91332    118.36875    119.07134    121.92805    120.61762    126.99579    129.82581    129.93712    131.71735    133.14548    128.77805    127.11746    128.48565    127.33622     125.1261    125.99912    123.27767    125.98545    121.95181    117.77445    114.30157    115.19011    117.67606    117.28972    121.53678    117.26439    115.27721    115.58685    114.36546    118.25477    118.46269    119.24309    117.71706    124.79346    118.88762    114.77597     117.3449    114.94417    116.98169    117.78235    122.47929    125.91918      127.226     126.9381    118.39945    116.24214    116.20502    115.13034    115.67492    115.33782    118.47876    122.83998    127.37969    136.56784    143.88564    142.77878    137.06016    132.48789    122.46006    120.10421     124.7765    118.94899    116.66661     117.8562    114.82694     114.0488    113.65142    111.94703    113.04493    111.56276    112.16678    112.12726    115.95367    116.08327    118.18707    116.37527    115.22866    113.43327    113.54356    116.42482    118.23012    120.05636    116.10667    121.68929    120.96832    120.30461    117.10529    117.28064    117.25308    115.52226    117.68031    118.85782    121.29682    122.73934    121.62341     120.0616    115.37894    110.62316    117.74009    114.78188    117.03618     115.0052    118.17452    114.71514    115.55424    115.32334    115.23091    115.50993    117.67508     118.3947    115.30467    119.09839    117.67339      121.187    122.84701    120.31328    119.58584    117.74839    119.64075     118.2185    119.00304    117.95082    117.72124    117.52237    121.84604    118.87209     120.9164    116.53594    115.96683    114.29269    120.42455    116.79613    118.02736    116.23386    116.96271    118.47142    117.58452    119.83007    115.87962    117.44343    120.67187    122.70278    117.44571    121.12401    128.50287    125.42713    122.27569    121.77308    124.33216     124.4084    123.05737    122.11936          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -5.50    265.44236     243.3203    230.64924    222.08688    223.01821    221.80945    208.18309    202.66292    198.31622    190.46444    199.07999    184.85583    173.11858    184.53896    176.66876    176.80745    178.95985    179.03156    177.23915    168.88255    163.50749    160.67476    169.77656    170.20685    165.03567    175.40163    194.93059     206.3722    195.29463    178.16363    159.92644    152.73576    158.81733    148.95049     150.9179    148.44524    147.98131    151.26526     152.6152    146.53984    144.15047     139.7362     143.0812    140.57589    145.62523    139.01692    139.79384    138.18987    133.85571    140.63473    134.76256    138.87201    133.45563    136.11034    134.35947    138.86752    152.84617    165.08142     177.1351    173.15241    156.74415    143.53317    137.35991    133.88984     131.7367    129.26685    129.60974    131.55628     130.8605    132.36782    137.93982    152.16864    158.56847    152.93305    147.02642    144.30434    146.87129    145.93627    139.51552    137.09214    137.30419    136.65094    136.84862    131.37203    128.08216    122.19528     118.5514    116.53001    120.91721    122.80654    127.03112    131.29683     138.3163    143.06344    140.77032    137.24834    131.05485    133.42763    130.71967    131.56058    125.60562    124.84908    123.96246    122.63599    113.76608    115.69955    117.43698    113.93721    115.39602    118.21251    117.78166    115.38158    115.95083    113.57502    118.43775    118.67684    125.47444    121.67416    122.43225    122.60891    120.77673    116.36393    116.19183    117.87238    119.71506    125.48228    126.98912    123.74442    126.11743    121.12276    117.22656    114.71156    113.82365    113.58904    116.60785    118.83886    123.16589    130.34847    137.07881    143.23377    146.30497    140.97577    137.90044    126.32907    123.97832    124.73613    121.10567    119.04977    118.13661    115.35863    118.75183    114.88397    115.60482    113.99135    114.01428    114.15982    112.75722    113.13292    117.14668    116.09565    118.23325    115.95868    116.22398    120.64463    123.94328    121.45756    119.80574    121.04183    119.22539     119.8408    118.19413     116.6357    119.57113    118.91632    118.93588    120.88415     120.4303    121.62114    123.97349     119.3305    118.76252     118.4883    107.29505    115.89998    116.27959    114.49224    113.51901    116.18168    113.54105    112.17433    115.52215    117.21386    117.77887     118.9962    118.02792    116.90975    118.46702    126.28009    123.52376    120.72725     125.1092    119.26226    120.23086    123.15415    119.27783    121.82249    120.29878    117.80538    119.97804    121.98545    120.43252     121.7908    114.56068    122.73846    117.96617    118.89804     121.3006    116.72063    116.82511    118.84438    119.17404     122.6254    121.31464     118.4844    121.86687     123.2437    123.18717    124.05002    123.24971    127.57268    125.60501    126.12625    127.87566    124.22319    126.52283    127.35003    128.72596    128.74803    127.43605    130.00091    125.48749     128.3739    129.57461      125.066    123.58131     123.9773    122.68431    125.59009    126.93374    124.96459    124.00499    125.48916    123.51259    129.69923    128.44738     131.7176    129.37112    129.02238    128.86448    128.94413    135.14573    133.05028    135.30845    136.13854    128.86308    131.77587     134.5279    133.18182    134.66787    130.63993    133.15464    124.84002          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -5.00    264.92136    242.43435    230.64235    220.95927    225.91285    219.15406    212.39554    203.99483    194.90225    187.30218    196.72777    188.62068    182.36701     183.0127     174.7044    180.25412    178.75895    175.48639    177.03223    166.08549    167.29294    160.58207    166.31534    172.08527    171.16879    177.60961    194.37604    208.14788    194.10294    174.92277    161.18614    151.38304    150.54896    151.15737    151.26192    151.47926    149.28927    147.86944    147.33617    140.87778    142.04155    139.32205     141.7624    141.18319     147.7485    142.58708    140.79691    136.03624    138.85021    140.75247    143.44422    140.24974    134.05227    137.89037    136.86848    140.02403    151.66029    163.95805    178.38911    168.60399     156.8984    144.70601    138.48701    134.90351    132.90683    129.22012    129.71026    131.23873    131.18198    131.77516    142.95938    151.10706    157.08152    155.13628    149.59284    144.27165    147.51922    147.57447    138.10396    136.68238    134.92309     139.6307    136.13256    133.11509    125.48507    120.32928    122.09867    118.19115    119.58114    125.42026    127.31914    135.10834    147.54834    153.60744    154.66345    150.65228    143.81662    139.21761    136.05031    133.58824    129.23095    129.60821    123.37726    124.38533    118.68004    120.97759    119.06998     114.9787    115.12293    118.49092    119.41834    114.91422    116.46636    114.67252    115.40316    117.63496    118.92549    122.17354    123.88717    120.98502     117.2515    115.48474    116.22936    116.70893    120.17467    125.85172    128.75496    126.94829    125.56613    121.44693    116.64684    113.48495    114.44993    115.38508    114.62875    119.75129      121.609    130.86989    140.84851    144.99868    149.53471     143.6997    136.93317    128.08947     128.8326    128.56886    125.83216     122.8424    119.05668      114.246    115.38928    115.12426    117.56084    113.53867    114.54129    116.42918    113.80122    118.13521    115.52477    116.76098    115.43167    116.61067    112.99441    118.28251    117.00411    122.43635     123.0605    123.03099    119.03299    118.00077    119.76366    120.59968    122.00869    121.30676    121.67828    118.68488     121.8677    121.17997    124.94901    120.01812    119.12623    115.52824    115.99009    117.51942    116.62249     116.1571     113.3817     116.0815    115.85324    115.40451    116.02231    118.70835    119.08607    122.90165    120.12583    120.47035    126.26381    123.03478    123.81088    124.87789    125.31334     121.6231    123.01306    122.86471    120.81574    118.38552    122.65868    120.42223    123.69489    122.68806    121.85774     121.5317    122.14413    119.10106    117.18426    119.81204    117.44959    119.81975    122.10297    120.26115    117.51762    120.26462     124.5869    124.07608    125.13884    122.78428    126.76022    125.04214    128.29695    128.16365    124.73536    126.79575    126.33988    124.74416    131.01518    127.40552    132.92008     129.8613    128.77932    129.84741     125.1367    127.06922    130.15542    128.52075    128.92931    125.66912    121.86946    126.09575    124.16786    126.78249    124.29269    126.50807    124.46432    128.24387    127.15033    129.25094    129.73903    129.92804    130.42511    128.53489    130.17074    136.08672     132.8981    134.51705     135.3479    134.36026    138.10932     135.9074    133.39599    134.60965    132.40671    132.35519    131.31941    129.06845    133.82575    126.31983    127.96267    126.19009    126.50338    131.13379    131.83672    126.27666    130.54701    130.60681    129.65353    128.92191    132.57972    129.97209    126.84379    128.73481     130.5794    126.66097    125.58388    128.47501     125.0902    128.11449    125.42633    125.22973    125.10593     122.0995    128.65901    129.54272    125.10537     126.8603    124.95494    129.32086    130.16468    127.05318    125.41805    129.26595    125.39931    130.09134    129.20104    133.92221    128.09689     130.6409    128.90796    129.85816    130.48361    129.25757    131.54267    132.19387    130.87036    131.36486    132.79727    131.66203    138.68316    130.39302          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -4.50    264.37124    241.65931    229.70542    224.73132     223.5909     219.9044    216.00642    205.32589    198.00223    192.02787    194.47471    189.50573    190.92171    182.98787    177.70864    178.61354    175.44272    174.19649    174.07748    164.91566    167.89352    161.32449    169.30852    168.23156    177.14839    176.15796     198.0108    204.68087    202.37469    175.38328    162.38197    150.91044    157.40617    151.44408    145.20924    150.85643    144.98623      142.082    153.21919    145.90504    138.35356    143.72702    145.89833    145.16281    147.44609    139.02421     140.1334    138.92682    139.52377    139.80797    139.81075     139.1302    134.27287    137.55061    137.87285    139.60802    152.18236    169.00583    179.58299    172.83936    155.29268    144.55172    134.86079    135.86539    129.52822    127.76969    127.91039    126.78885    128.93766    134.54196    137.59391    153.17618    156.93259    157.14161    149.74033    144.38516    143.83116    143.34582    142.96442    134.62435    135.62383    139.00395    137.98224    134.12976    126.54661    122.66006    120.06472    119.59336    122.59607    121.89435    129.24453     137.7175    154.71586    162.37322    162.90884    156.33219    149.72948    141.38351    144.78784    142.47211    136.99283    131.00875    128.61575    122.21934    120.20413    118.63341    116.18831    116.71966    115.87612    115.59355    119.06084    116.19624    120.85842    116.02895    114.21792    117.30835    120.21744    120.76985    125.68129    121.96394    117.99105    116.43694    115.26916    117.93595    122.89819    127.10771     131.1735    128.02951    127.06689    121.84624    115.76541    115.43702    117.72711    112.30226    117.85892    119.87737    122.09186    130.79593    142.94218    148.17316    148.52196    145.82305    138.16006    131.31007    129.44902    126.98741    123.58433    121.85772    121.38506    113.57629    113.98158    115.46646    114.38795    112.56195    111.82538    112.17309     116.5974    115.52684    115.70745    117.05937    118.94761    116.97032    116.73742    117.19219     119.7463    120.17452    122.25898    121.32245    120.70311    119.70353    120.50475    117.48211    117.21628    121.98627    120.27432    119.27269    125.73427    122.72145    123.54071    126.94229     119.2447     120.1961    117.41179    116.86597    117.59395    118.63986    117.35606    116.08059    114.41763    118.80607    116.91935    117.43797    119.90614    118.62119    117.63314    123.29318    123.18746    125.48429    126.11803     128.2455    127.76667    121.05708    125.10894    122.98431    120.22472    124.26623    122.14291    118.26238     122.6246    123.99723    124.09115    125.36679    124.27695    121.85267    120.25473     122.5512     119.5851    119.63848    118.64072    120.15004    117.72641    122.33202    122.59749    125.19103    122.36646    127.83546    121.96879    125.88506    126.11058    126.74612     131.1499    122.91579    128.00933    121.69075    128.49755    130.53514    127.64751    129.47614    130.40765    132.12205    131.52142    128.66131    127.47646    130.09706    132.03173    123.72406    124.24877    125.99942    126.18331    127.86117    127.25491    129.06813    123.25709    130.64543    130.72393     131.2354    131.68025    131.79115    131.05836    127.33109    133.52202    135.20216    137.12111    136.32647    137.02459    134.18634    142.39661    137.46819    136.96474    139.14702    141.96511    145.66689    144.71019    138.50253    131.36977    129.22792    129.52999    127.67664    125.81355    131.37559    131.26208    129.65265    130.46463    131.59159    129.19922    128.44467     131.6044    132.73729    128.54957    129.67239    123.70744    123.42241    129.77869    126.21979    127.39452    126.02173    127.08822     126.2463    126.46047    125.97233    126.04945    128.20861    128.09712    126.37512    125.93987    128.70519    125.34849    129.07118    127.45041    125.83113     127.6991    127.65552    128.29985    132.75979    133.57649     130.3589    128.46854    129.90206    136.91117    132.81693    135.44716    135.34772    131.90888    134.92947    136.74654    131.42985    134.26354    134.53256    134.66809    135.58124    137.88375    137.38178    139.59828    141.68425    138.75039    138.97145    137.17867    144.89748    139.85819    146.09875      144.153    141.66591    141.78087    150.98486    146.82258    152.45066    153.47617    146.81136    146.01128    149.60887    155.87512     152.8019    154.07279    157.14374    156.72591    159.90227    158.14984    159.41172     161.0527    163.93137    161.62036    163.81138    163.48092    167.38809    169.16225    161.93105    164.06476      159.156     169.8754    170.00373    162.26926    170.41628    166.28845    173.03497    172.20816    165.68525     174.1615    172.90023    176.16214    169.42907    170.64406    172.77123    173.57512     170.0245    178.04172    177.31761    174.58437    178.38374    188.21957     187.0842    172.74998     182.3298    175.29737    174.99459    190.50979    193.85614    195.85252    183.14389    187.70105    190.92504     195.5611    207.60611    202.33588    200.48201    202.76068    202.57138    198.50978    192.30868    206.26083    204.38774    203.66189    209.92976    216.63166    201.83178     226.4957      214.948    225.50665    220.42805    222.93767    219.00471    213.90761    239.75286    221.11228    230.19568    225.29546    227.53145    219.61073    242.15454    231.34172    239.13043     247.5285    224.96144    239.90144     249.7061    244.00924     248.9176    243.17563    252.83689    237.01359    247.65926    258.83532    260.86468    259.81237    280.41322    258.18765    260.99719    271.15179    269.26812    275.25271    276.40512    280.36513    280.42411    276.25971    297.38707    286.68855    298.78977    297.23205    297.82832    292.34657    349.85036    338.35448    328.34866    319.43313    325.93801    312.89165    347.38129    375.13863    380.55124    385.31738    356.05624    386.31641    386.98395    447.14241     417.7648    459.03497    449.23292    422.69379    458.04749    508.21582    576.02088      564.479    554.71664    568.10817    551.32357    602.70091    827.52948    760.10338    806.66922    785.84866    854.52907     838.9709          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -4.00    262.29605    240.54358    228.77435    228.33523    220.04956     224.5156    214.00113    204.66759    199.76607    196.14246    191.97416    187.30944    188.69914       182.72    180.40697    176.97035    172.93021    176.59897    172.81012    161.23081    171.51678    163.15921    171.37695    161.46699    172.04424    181.39349    200.78544    204.19009    207.82977    176.71537    161.51853    148.43486    155.88564    150.24248     150.1394    147.14075    150.23731    149.89875    144.79234    140.91557    144.35909      142.195    145.80724    145.04021    141.84782    140.74644    140.73907    136.01532     140.1217    138.54089    139.00581    135.57476    135.26589     135.9803    137.57991    136.91701    151.08711    164.81966    181.70474      170.135    164.12405     142.2531    136.02057    130.33035    132.20669     124.9954    129.22105    128.06438    129.66513    135.88435    143.71999    153.60523    160.17519    157.71857    152.44358    144.07983    146.26882    147.78484    139.68239    136.41014    136.01303    135.16237     141.5207    129.89571    126.62023    123.44348    121.60677    119.01519    123.59973    123.64753    131.38925    141.45915     153.5917    161.91519    162.39766    155.76846    145.79525    142.30959    146.10279     146.7314    142.38292     136.7698    128.87195    123.67415    118.65211    119.83504    120.22926    116.89236    116.84764     117.0023    116.09637    118.59096    117.37507    115.34788    116.63824     118.6027    121.51577    121.83471    126.54604    124.01273     117.1626    118.67906    116.91798    116.82205    120.47826    125.56991    129.21474    128.25754    125.87757     124.6977    116.75158    116.99937     115.1225    117.43858    115.69648    120.07651    122.02067    135.49198    141.30091    148.51137    147.89183    144.88411    136.31631    127.10188     126.9146    125.59445    121.58178    119.95476      123.151    116.60619    116.79916    117.15122     116.0087    114.08324    113.63741    116.14832    115.85934    113.21731    116.55278    117.94386    116.32389    116.04652    118.03032    117.12371    122.90915    120.17735     120.7079    120.40022    121.60735    118.08904    118.65927    120.81298    117.02354    118.97492    122.07581    118.95895    123.88081    126.10232    124.25001    123.94487    122.18719    121.00559    119.71852    122.19218    121.16698     117.3929    115.74515    117.36886    112.87566    114.93349    115.63484    118.57049     120.6436    119.34458    117.79602    121.23829    124.03459    124.68499    127.73132    127.05331    123.93175    126.12978    127.92669    123.25417    123.71214    121.99047    124.01153    123.39009    126.11353    126.05216    130.00984    124.51429    123.58147    123.90598    120.58801    121.63738     122.0694    124.63488    120.69896    120.02827    118.95065    126.14865     122.4952    120.18785    125.93236      127.768    125.23391    126.49103    128.59138    127.88588    129.77484    129.58434    125.43549    125.60977    128.78169    129.48742    131.57924    129.43569    134.48779    129.96237    131.82553    130.35481    131.39158    132.32008    127.96043    130.26034     125.6901    127.60615    128.74229    126.97856    130.48238    126.87983    129.54794    133.58359    129.67076    130.23057    130.01602    132.58509      129.297    131.39871     132.5474    136.37427    136.54622    134.22403    138.79138    138.03883    142.58994    140.64682    138.23437    135.17516    143.91581    141.70691    138.65484    134.29288    133.31171    129.75094    129.57009    130.82507     128.9057    130.95946    133.13711    132.77263    130.39356    132.24325     128.4431    133.52518    132.30991    132.45779     130.0033    130.02758    129.24935    124.47589    131.39905    126.29162    127.47997    128.33758    130.33501    130.05256    129.88801    131.23624    126.65464    129.86435    126.26325    128.85353    125.23251    130.28036    131.53659    130.20665    129.55503    130.70371    129.14114    133.75663    131.07965    130.06327    132.70962    131.67061    131.44262     126.8917    132.62121     133.4506    136.66535    136.50099    135.98773    133.40862     135.8052    136.15329     138.2213    135.89292    135.56418    136.32238    137.60284    135.65105    143.09778    141.16426    146.58129    137.64858    136.77356    146.31022    148.03098    143.50341    146.80878    146.11245    144.43619    150.27173    149.78698    146.74809    148.96955    146.88917    147.86973    150.12302    148.42358    157.60624    153.86181    157.61836    158.28916    154.90624     163.4605     163.7547    160.24271     158.3381    161.21536    162.48491    169.72605    168.41909    165.57256    170.99081    160.32222    165.98145    170.38218    175.24315    168.98267    166.95992    165.97421    164.04451    169.14569    167.81698    172.44985    173.64958    176.04443    175.92611    175.77289    176.24873    181.75553    177.70595    176.67625    182.51462    184.69791    175.13608    186.23472    180.87752    190.41455    182.20266     188.1724     189.8595    185.09095     193.1007    194.20783    191.57914    186.37391    197.79139    197.50324    201.22425    200.95862    203.18693    202.44275    204.28923    198.13698    207.44868    200.95744     208.8506    212.57784    201.88632     210.8026    212.99914    213.42499    219.87961    230.33329     217.2319    220.45146    219.34399     223.9861    223.31365    233.20246    225.51066    237.79592     233.2621    239.18059    242.54677    230.25956    233.90235    235.78319    229.88986     236.9549    249.25292    244.71092    237.16409    247.52025     245.7196    243.80129    243.17191    249.94928    263.60541    269.34272    261.49012    269.54419    257.40916    265.64826    281.69142    262.27428    270.66304    279.11697    288.00807    285.82295    290.42372    302.37597    296.33471    313.08912    298.35775    296.38158    300.35984    324.90284    332.51064    315.17325     320.5788    331.80298    340.21924    358.47661    366.95071    367.61561    370.18237    408.23556    374.40111    447.15814    426.10595    464.43625     435.8738    419.06714    445.51841    490.62461    546.77908    550.62223    537.12584    557.97625    534.65754    687.94218    840.32247    847.82837    817.86691    870.89523      797.013    843.76749          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -3.50    260.03082    239.25164    229.00334    227.21435    219.03219    227.00982    211.86337    203.37845    199.66526    197.52936     193.0081    186.15765    186.49557     182.5794    183.37922    175.45971    178.22194    174.10246    169.26915    163.81575    172.25131    165.06802    167.20902    163.81638    167.96842    181.01389    197.90693     207.3857    204.64837     176.6779    165.43263    155.05573    155.47585    146.80086    152.24027    149.31141     151.6121    150.10911    145.48622    144.76805    143.72324    140.13935    140.95341    144.28248    140.99753    141.72849    137.14689    139.07061    139.93184    138.17967     139.4742    136.70498    136.80547    136.81311    138.53966    138.40667    148.42014    171.18439    180.84347    173.38633    160.44608    145.91811    136.86696    137.05325    137.09584    128.80366    130.11688    129.82179    130.30007    138.53443    143.00155    157.99928    158.29908    155.69352    149.60028    145.86105    145.07482    146.89471    139.23114    137.74464     139.8859    133.76468    133.90199    135.38915    123.76998    122.52571     123.0994    117.08991    125.26514    125.32446    131.67544     140.8278    149.29946    157.33935    157.38651    156.14539    150.63619    143.13455    151.65533    150.30519    141.75743    134.82681    126.54428    125.01314    117.06556    117.71793     116.7008    117.53465    117.86495    118.27018    117.33068    119.64294    116.46632     117.9111    117.42079    117.29075    124.23231    122.02483    122.07372    124.72664    117.15436    117.23805    114.56153     114.9527    122.47952    124.74488    129.10555    133.01412    124.85114    123.09889    117.85588    116.62227    115.67826    112.81648    116.47219    115.56476    119.60382    134.70547     144.8826    150.05075    152.08911    147.00839    137.21239    125.98878    123.03934    123.73231    122.54486    119.46107    121.78011     117.8883    120.49196    117.26624     116.5902     115.5243    111.89925    110.46827     112.9209    119.39536    117.49586    117.11447    118.04891    117.82632    115.65344    123.29014    119.27143    118.93093    121.63065    122.46464    118.75417    123.54661    123.13304    120.45769    120.76821    123.85946    122.93272    121.42324    124.29322    125.12408    123.93725    124.01081    122.85762    118.32855    123.06056    118.52591    119.77788    119.61805    118.32547    116.43621    117.90827    118.30262    117.90838    121.94517    119.55745    122.64823    118.50704    124.70911    128.56165    125.17304    129.12293    127.83771    125.82049    125.38511    125.78981    125.83923    123.13558    125.11448    120.62556    126.41727    124.59512     126.8098    129.23443    126.34601    128.15764    123.13647    124.07656    124.35294    123.67821    121.38906     118.9068    122.05455    119.00418    122.92409    127.08921    123.45494     125.7268    125.65264    125.53721    126.60993    126.66814    130.45308    127.54045    126.91171       129.55    129.53723    134.76852    136.35053      135.119     133.0466    135.50121    133.26783    138.12154    139.56411    136.26114    134.37758    132.65587    131.33353    128.99354    130.71268    131.85495    130.33195    130.57973    129.82058    127.64662    133.09632    133.11104    131.62404    129.82307    130.87105    130.63328     132.3937    134.69798    135.44826    139.52849     137.8312    140.12875    134.90203    139.88598    138.87916    141.36929    136.22374    138.39241    131.32888    135.26964     131.8172     131.4431    132.49538    128.94588    129.46552    129.38494    133.32848    135.10911    133.22829    130.78923    133.02073    130.26915    134.33832     131.1096    131.63422    130.78455    125.38474    128.19524     129.8733    128.62537    128.18527    131.07961    124.52785    123.38329    127.54171    125.70819    126.97221    127.98415    128.45246    131.24814    127.57488    126.21484    130.41488     130.2047    130.44809    130.24479    130.99375    128.04491    129.66794    131.20001    131.21449    131.47657    134.60351    131.84319    134.62017    134.44784    134.76663    133.03808    128.40299    131.59104    132.49143     135.8202    135.59509     134.0862    134.76051    131.47602    134.43768    138.25069    142.50691     138.7646    144.33292     139.8607    142.78097    143.95453    145.71293    148.34528    145.30059    146.49942    146.90838    149.47413    158.03592    154.69175    149.97528    148.55432    148.71087    145.62733    146.40652    153.97474    153.57278    154.46265    149.00857    158.02986    161.09174    160.10872     159.8315      167.708    161.40119    163.35576    164.02503    160.96711    172.50309    167.01065    170.12173    170.05562    168.38762    164.44069    176.71199    169.69015    170.44949    167.72002    168.62525    174.06266    169.74281    175.86086    177.35368    167.25954    178.36117    179.88043    176.74803    181.17917    177.40349    177.19469     176.8754    182.13539    178.32904    183.18788    184.53502     185.0527    181.77335    183.52056    183.76783    187.88287    186.78317    194.52544    193.48126    192.42943    194.06306    189.11779    201.73767    203.19111    195.46008    202.19357    208.68038    191.34527    197.02723    212.86522    209.76234    209.08272     207.8488    200.71713    211.66174    216.97512    214.96499    213.38061    218.05307    213.66619    213.49133    220.66697     234.6381    237.39615     224.2607    230.97196    236.94364    234.83001    235.58367     236.8809    226.34406    238.29157    231.90357    228.91945    250.18065    240.91644    243.12117    257.05609    246.23098     250.7842    248.15948    262.75797    258.43422    259.95789    266.83487    249.12542    240.12371    277.28875    260.66765    284.83965    264.31202    276.41245    278.75838    277.91477     305.3998    291.03282    282.87176     299.3361    290.33392    312.55759     319.1579    311.40552    331.31321    322.62366    315.93435    335.99277    309.25356     353.8572     368.0533    352.94289    375.16823    363.51462    358.71694    411.74752    404.04093    404.91996    444.67646    454.36738    447.15002    493.95572    554.08907    569.47471    531.03956    547.70792    553.89134    627.01337    858.82041     743.3329    826.93989    857.47131    766.46569    728.15856          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -3.00    257.94224     238.2287    229.37491     224.7222    221.16732     221.8773    211.48906     202.6255    198.62656    197.84768    194.48622    185.87533     184.3611    182.88237    181.92805    175.75924    178.91411    172.51947    167.21974    170.76413    165.43676     164.1194    169.71045    166.48847    168.82745    178.60747     195.0165    210.40397    198.75587    170.95144    157.47453    157.05079    146.53854    149.38419    149.37287    148.98371     145.9056    149.30737    147.28683    146.75421    143.05883    137.46628    137.93379    144.08951    140.62552    141.66786    138.37283     136.7186    143.45139    130.83498    138.80176    138.93838    136.16512    133.74813    134.60573    141.51496    153.13709    170.58527    186.35529    175.48752    157.71475     144.3104    137.15294    133.52305    130.11449    129.83373      125.686    130.41628     129.1815    135.88309     143.1136    151.61996     161.0361    157.89294    150.77688    145.67238    145.99998    144.78568    141.29362    136.06912    134.35671    136.16322     136.6077    132.41658    125.46852    119.09515    120.92828    122.00677    120.79958    122.15184    132.01801     136.5484    145.52944     156.8968    160.93139    160.62229    160.17819    152.71874    155.63746    145.53315     138.5078    131.75745    130.75695    124.81029    119.43533    120.31273    119.30191     116.6796    116.67144    123.50436    121.89281     117.9566    119.19495    118.18783    114.48363    117.63608    121.21735    121.97039    127.79648    125.26772    121.47616    117.04213    115.70036    117.09038    119.98386    125.58737    131.90438    128.42106    124.84974    119.39571    116.16382    117.42152    114.43263    118.79981    116.28177    118.13511    122.38547    133.40186    148.30899    151.09207    153.97359    147.87687    136.79714    131.56127    127.42467    121.72316     121.8748    122.77592    122.48542     116.6086     117.4173    114.49868    115.89167     115.0542    115.43338    117.14759    117.36944    117.98784    117.01352     119.1368     117.9542    119.57898    119.12704    116.87336     121.9462    118.86121    122.92374    123.31886    120.74181     125.3913    122.45829    122.42199    124.08096    122.79332    122.46888     123.9688    125.61134    123.70021    127.03473    122.14367    123.69983    118.49898    121.16212    119.63328    117.20303    116.32088    114.83525    119.20333    117.64558    115.93733    119.54041    118.42782    118.28776    124.71377    117.90169    127.81966    122.16553    125.52916    124.89528    130.49862    127.68755    126.55424    126.89712    124.04665    124.48579    123.58236    121.14992    121.11275    128.18527    125.67584    130.57641    125.44115    125.47357    118.93111    121.21455    122.72271    121.36167    124.04301    120.89632    121.83306     117.4204    120.92961     122.3821    123.20423    124.95715    125.90789    124.04671    127.42895    129.88739    128.48997    128.93434    126.06521    129.16493     132.5765    135.36978    136.83204    138.22329    142.08329    137.59806    142.05475    139.16125    140.24271    138.88095    134.30532     133.0119    132.69547    130.24947    131.51933    134.50152    129.81903    134.03616    132.75804    133.04114    135.16906    137.42669    138.14627    135.04332    135.94731    131.40289    131.69213    134.75814    140.95955     138.9053    142.45428    142.89222     144.8855    144.23791    142.80539    136.27109    138.06763    136.44806    131.44527    129.60641    130.58779    130.47618    128.25756    131.81617      127.409    131.06458    136.18452    134.18365    132.35844    133.71155    136.28057    130.34129    135.68773    133.86259    133.92542    131.67166     130.4991    132.13822    126.89671    131.91669    130.12499     134.6994    130.71646    131.48003    127.27657    128.24684    133.81134      131.171    130.39738     129.1955    127.79553    128.80828    132.10055     126.8982    130.24537    128.24279    128.44336    127.73856    130.73386     130.1498     130.1408    131.30227    134.31301    133.99506    133.00902     135.9098    132.26602    135.94988    134.12475    127.28772    126.67825    129.59602    135.86443    131.97467    138.12258    133.50938    129.85068    135.60279    141.46014    139.34747    142.24707    140.93654    139.79446    142.21829    145.22497    142.57449    147.26684    147.03555    144.49874    146.10859    149.05568    153.29707    150.94535    153.56243    147.77142    145.42596    155.13635    149.77027    153.63588    157.44068    156.86518    154.25254    156.68574    160.72425    160.68586    164.94551    164.13703    160.52336    163.38514    160.43571    170.22222    165.81544    162.93931    161.62752    169.52694    168.19392    169.62327    167.64471     170.5893    171.06748    172.76076    168.23149    169.36296    174.40747    181.11575    180.06201    171.26056      175.851    176.20206    182.81883    183.27215    180.47895    182.62354    177.29634    177.46212     182.3446    182.64984    188.30278     183.8579    184.64815    189.28988    186.18745     188.4686     192.4484    189.32563    191.19378    190.92378    194.51492     198.8501      198.583    193.38865     199.6297    200.05792    204.13231    196.20964    216.91148    205.79608     210.5909    212.89949    214.19549    210.05233    213.93778    221.29478    224.76496    220.55075    215.50352    224.24892    211.36248    216.39462     219.9719    215.23692    220.22349    215.54824     223.6481    237.11103    237.40071    231.16018    238.78749    232.88443    221.04746    239.82533    244.20908    251.42891    236.00776    251.29899    234.95521    251.19926    267.38563    259.27984    261.03537    262.81187    251.44894    252.68058    250.97991    281.40636    286.85456    283.48519    273.85147    248.84665    275.37119    289.17096    288.11676    270.56879    297.51159    283.47924    293.76492    303.96248     323.7081    310.41289     326.3807      315.628    301.20955    332.23443    383.28488    342.38631    385.83433    373.58699    365.83435    333.16242     417.6294    416.96925    413.80599    409.88068    429.29489    410.35956    481.62621    501.44067    570.17546    539.04307    518.14911    483.10408    638.23429    848.07472    842.89403    850.31315    809.56354    803.77066    759.06189          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -2.50    258.82673    239.82132    229.48546    222.98501    223.06877    217.56588    211.39257    202.27533    196.62696    194.19035    194.87269    186.91939    184.65378    184.34142    180.32449    176.38417    176.07666    171.91297    171.55024    169.22725    163.87683    164.32203    167.12544    167.98879    169.96676    175.79953    195.57685    208.29479    199.35399    174.78296    161.42594     158.5611    148.05333    150.97138    147.27211    146.96478     147.0942    149.08167    145.46346     142.0566    143.72742     139.5764    140.63409    141.66855    139.90215    138.64689    139.28712    138.98851    143.42456    141.07485    146.33977     136.6024    139.84809    135.95603    135.90545    139.25782    151.44435    168.27821    181.13388    171.35823    160.68357    145.49452    139.86829    136.56403    130.40359    128.87861     129.8609    127.95895    130.97616    130.94268     141.9614    155.14558    161.91492    160.77049    152.51729     145.4041    148.42835    146.12747    134.08954    136.05028    137.51889    134.93994    135.18022    131.08493    124.07189    121.99656    120.93945     121.9206     118.8595    123.15628    125.78859    137.56046    144.01764    155.94976    162.19724    168.08304    162.37315    154.86102    154.55333    146.73983    138.00346      129.524    128.42004     125.4222    120.26252    119.01787    119.82623    117.34093    119.66221    123.00831    117.05322    125.00064    115.53008    120.17967    115.97114    122.25848    123.44175    124.68081    123.78929    125.21397    122.25923    117.63065    119.88384    114.34704    120.87111    129.96257    132.58935    127.13689    126.06531    122.44124    117.19297    119.90798     113.8414    116.20767    119.90388    118.33255    125.23317    133.63512    143.31962    152.68797    153.95263    149.33911    139.16771    135.63497    129.01376    126.17517    123.27469    123.01459     123.0743    116.10899    119.79454    115.41914     116.7341    111.59475    113.14088    113.34033    112.61698    117.00893    116.33527    115.50562    116.40852    120.83383     116.7378    116.77142    120.52066    119.46761    125.91763    123.68881    125.89105    122.69379     126.5195    126.99214    123.35637    123.82914    123.44894    119.51908    125.91105    126.17636    124.05612     121.6808     117.7843    122.09941    120.49838    121.35053     115.8958    116.38419    118.10885    117.21874    118.03858    117.08326    118.83292    116.22985    120.55905     120.4183    121.88908    124.55891    126.51402    126.17679    127.74404    124.90064    126.76251    129.13684      123.754    121.22561    123.71971    121.05237    124.08768    120.80196    125.12397    124.18351     124.1333     120.4481    124.76455    119.19642    125.36989     124.4912    120.21656    120.33484    118.62338    120.44077    118.79948     122.6646    123.74346    122.97262    126.19651    129.76304    127.18346    127.69635    132.40918    129.50619    127.04468    130.15994    132.40487    129.30242    131.67744    131.80639    138.82839    140.72247    138.81045    140.69627     142.1361    136.13099    134.70129    136.71576    134.40837    135.14015    130.54466    131.53718     136.4688    132.41182    136.69241    144.89907    143.98704    143.30618    142.44132    139.38103    135.24648    132.30531    134.07716    131.48689    133.77945    138.71687     142.8477    137.74019     137.8574    139.18329    144.29919    143.96644    141.62679    140.38059    136.69094    133.28646    131.28636    134.19899    129.21465    130.13148    129.04229    127.16881    130.11859    132.45673     133.6251    129.80045    132.27335    132.37392    133.95566    133.74056    134.31406    133.84591    130.80975    129.54276    133.63775     130.6135    131.61871    129.02158    126.68415    126.68121    129.90044    130.43031    130.85933    128.30229    130.67993    128.13951    127.39269    128.92377    126.45693    131.32729    135.85165    129.86374    129.36705     129.0366    130.33404    126.92311    135.78859    133.40324    130.84371    129.58943    133.80695    129.67671    132.38185    133.61546    132.30286    136.50791    133.76809    131.92186    136.50277    137.62318    137.02425    131.29299     132.6801    134.14393    140.90958    141.68727    137.67199    141.52375    139.53929    143.35754    140.69146    146.68254    146.82337    147.86177    146.26379    143.09199    141.04263    152.21996     146.0258    147.18363    149.18594    148.93419    152.34546    145.90603    155.42108    157.17037    153.06515     160.0299    160.05805    159.84848    159.75287    160.64476    163.19748     163.5723    156.81875     165.6807    163.24199    166.80145      167.925    171.76818    169.07274    166.56189    164.52031    170.70168    168.26538    169.95265    170.50526    167.63647     172.1505    171.87147    176.35599    176.20086    172.86812    173.33083    181.04614    178.80488    179.56825    180.78856    183.44543    179.23411    182.01135    175.32864    190.69978    180.56927    177.66604    184.09112    183.78655    185.86866    188.54198    190.62157    199.75225    192.31671    181.00755    197.69033    194.35122    204.01686    198.27209    204.49098     196.2915     201.9809    194.68098    209.63527    211.71537    214.52887    210.43051    209.94853    200.39439    205.14657    216.34287    222.06396    217.14962    206.66219    222.31614    214.80547    219.03297    230.14896    239.90466    231.89535    225.67575    225.80657     222.2164    231.69935    230.10271    229.95479    225.43521    229.26646    234.48517    248.93369    250.95744    247.00435     239.5769    255.71754    243.60742    239.64936    253.80735    258.53726    251.03767    250.55403    249.55017    259.69641    275.63039    264.21861    268.95661    269.55359    275.00727    244.37601     277.4513    293.10332     281.7768    306.24207    271.98238    291.01391    289.77428    309.05874    322.47849    318.85085    342.82305    320.03029    317.50977    336.97422     360.5563    361.50721    342.35942    349.65528    388.56387    356.54614    427.18276    411.89855    414.66364    387.04772      423.283     440.6054    473.40966    522.87821    549.65169    545.77686     490.0519    531.57933     649.5738    795.87557    686.26248    762.67162    813.19652    672.25653    694.69373          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -2.00    259.88563    241.94612    229.11468    223.00349    223.49637     219.8023    211.19469    202.92994    195.10333    191.92638    195.53187    188.23946    187.08231    187.47743    180.16595    177.27538    177.61733    175.98953    171.23116    170.68951    164.47502    164.46979    164.28676    168.60854    167.50999    174.64859    194.12481    207.68133    195.78495    175.31968    162.62636    156.62477    151.20738    149.43068    146.46598    150.65148    152.93378    150.00368    150.18223    144.92321    144.32783    144.54666    143.84942     141.5995     145.8303    138.51291    140.75985    141.28063     143.3534    139.09054    139.30785    139.88697    135.03904    135.77734    139.28133    138.60937    152.84125    167.90546    176.37816    172.11667    156.09378    144.70991    138.84337    132.95012    133.51003    132.25605    131.60038    125.77528    137.35727    137.63492    142.23457    155.38505    162.25267    155.37561    150.54969    146.11468    144.16439     146.4307    136.48985    133.19703    134.44565    129.90453    132.62903    131.28168    124.47073    122.49044    119.96733    125.25497    123.51624    122.71718    128.93507    134.10644    145.13393    153.87743    159.61006    163.20455    164.24772    158.74073    158.23385    147.31883    142.44135    132.07005    132.31801    126.64341    124.56396    123.52293    117.97146    120.45535    118.74266    123.72901    118.60295    120.15279     118.6292    113.28131    114.69335    119.19974    124.04128    126.65554    126.71176    125.61806    120.24514    114.97473    117.62207    118.70669    121.49679    128.16172    133.59084    132.72294    127.49789    122.98942    116.13558    119.20164     116.3774      114.884    116.32035    119.80752    123.60961     135.2009    148.41441    153.62991    156.67673    151.76646    141.51019    135.24549    132.78193     127.5991    127.90609    126.26377    120.50256    118.23157     119.2728    118.58924    114.07278    111.88861    117.83402    109.56328    117.88957    118.86421    116.80737    120.61729    120.28382    116.51245    116.27903    120.80199     121.9812    120.32152    125.02075    125.91403    121.81069    123.48743    126.21483    125.12319     127.2349    124.71621     127.0057    126.07794    128.18111    127.08435    127.96618    126.71417    126.48023    120.80719    118.98974    113.90141    115.66616    111.30577    114.60948    121.14674    116.41103     120.1576    117.59206    122.20144    120.55623    120.86292    120.22164    126.12823    122.12993    124.65783    126.60829    128.68025    127.91313    126.30941    125.89031    122.44603    122.67252    124.46787    120.94481    123.62902    127.03626    127.17058    126.19573    124.61395    121.34167    123.11303    123.77652    121.40179    121.76468    120.91606    117.61008    120.24005    121.09802    122.10311    121.17559    129.57786    125.59821    128.77521    128.51398    130.43032    129.62545    130.41821    132.83803    126.62469    126.78682    125.66124    130.82749    135.75974    135.97147    136.92964    139.21889     131.8092     134.8072    137.03054    137.21288    137.47336    132.32262    133.86919    131.16446    129.12008    133.48344    134.46954    136.57469    139.61712    135.98641    135.92745    138.22513    139.03083    134.73657     133.0272    134.00263    133.85524    136.35998    139.97774    139.48347    141.38857     140.2043    139.89633    144.90996    143.23386    141.46966     138.1659    135.30036    131.20991    130.27019    133.82569     134.3657    130.29785    128.48258    132.27181    130.08961    130.09107    133.92252    134.70346    132.85299    136.08702    127.77296    133.52299    131.62907    133.64917    134.50665    134.73263    131.17389    128.35112    130.90718    131.56548    125.09691    129.09224    129.06346    124.40861    125.42118    128.75412    128.49115    124.89442    130.02535    129.35778    130.07158    130.97391    128.21766    128.22822    131.76727    131.44402    131.10775    133.67182    136.25421    132.79697    133.04105    128.80895    131.46145    132.75905    132.72583    135.67461    131.20562    134.45022    133.91802    131.44486    136.58068    128.49703    134.88895    131.19266    134.65052    134.22022     136.4201    143.60092    135.79952    137.17615    138.05458    141.61287    138.36082    147.53212     146.8598     147.1655    147.26936    141.48476    145.86511    149.07808     148.7507    145.53538    149.07169    150.54285    148.88677    149.94993    155.23371    153.27526    156.56836    153.35875    152.92142    155.73151    161.53618    162.27755    162.95962    165.47552    160.06814     164.6989    162.43132    165.41941    166.92292    168.18312    166.01092    166.92579    168.62144    166.58171    168.52772    168.62324    170.32261    166.96122    164.42699     175.4176    170.71647    177.45767    179.10183    179.38832    173.83312    171.41222    179.26427    177.04982    177.97101     179.3436    180.69236    178.42883    190.51695    188.60514    186.28102     183.0308    189.42199    181.89708    184.32178    192.68743    193.75614    194.03578     189.6872    189.93425     196.8102    196.84063    204.61461    190.47793    199.56433    198.36344     200.0461    199.86789    218.79762    209.28811    207.55734    206.52316    212.68689    199.78829    209.83781     222.3231    229.78299    218.81626    211.35636    212.97261    219.54874    231.38265    228.15913    222.86025    220.11624     226.1353     232.6757    234.53376    232.61554    232.74257    230.14893    237.01468    222.30316    234.61366    244.76083    249.75886    243.09504    233.75357    231.30863    245.95688    239.37197    256.27752    267.18362    249.07046    251.11405    243.28867    268.78241    255.24843    258.57871    277.72758    253.36856    253.21996    279.22567      290.604    292.06062     282.9495    288.84359    286.00674    293.61277    306.79707    317.84659    319.54606     320.2141    297.67396    312.16779    335.02155    362.04112    384.16501    364.46021    366.84802    349.73922    355.39545    383.20762    433.96337    427.38192    371.97945    427.53857    451.27135    419.08135     545.5905    513.21302    523.91838     512.7833    522.31739     563.5866    652.83993    773.17729    660.15477    677.53527    703.92254    766.04848          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -1.50    260.66986    243.95936    228.58428    223.02899    224.06191     222.9278    211.13003    204.60715    194.09198    194.42534    195.92612    188.30109     187.8833     186.8394    182.73947    177.05301     178.0238    178.52589      170.127    173.41858    166.34426     163.5675    168.82254    168.49946     166.9034    178.96966      196.664    205.14215    198.25875    177.84526    161.07919    154.87921    151.20598    149.60061     151.8638    149.50695    150.33641    150.17519    151.13542    143.64951    146.91901    139.27721     142.8991    138.54628    144.17457    137.93774    135.85709    140.08234    142.23448    134.38147    136.18727    139.03683    137.51256    136.93361    138.61382    139.69084    154.31514    168.67318    180.03428    170.23188    158.08591    144.96659    137.48727    133.11222    132.97965    126.02904    126.46885    127.67747    133.52057    134.57182    145.47059    155.78083    162.23807    157.19789    151.08085    147.50377    144.45503     147.5535    140.20886    137.58965    131.37766    129.31135    130.65398    129.93022     124.6342     123.4503    122.66852    121.55477    120.58296    123.46939    127.83807    136.99454    144.11757    154.24288    159.05907    157.54365    158.44709     158.3508    162.50332    154.48246    143.74935     138.9028    127.55675    125.12924     120.9337    121.03405     119.6661    119.08444    117.79854    119.95192    120.00751    119.87783    117.60497    120.34278     117.6225    122.91886    122.21511    125.97513    127.13414    124.21607    121.74708    119.57941    115.59951    118.39208    126.07471    127.22346    132.30297    133.98409    127.49424    123.54694    118.14447    114.31266    117.58044    116.15775    118.73123    120.44544     124.9482    134.97457    147.55006    156.06172    159.39052    151.37761    144.02749    138.46951    130.68537    126.40367    125.83025    125.62792      121.172     121.2607    118.29585    116.98425     113.7908    114.59071    115.68117    115.01787    114.67716    119.46268    117.75314    120.94742    118.45584    121.39865    119.81033    120.24591    122.92513     124.6303    124.52547    123.06456    126.02952    126.93385    128.03729    124.58023    128.32366    130.07529    128.86052    126.65919    127.56596    131.43888    125.34143     125.8446    123.13124    124.13504    123.41864    122.41505    119.33874    120.27526    114.78683    116.73179    118.98819     115.0391    120.10104    120.32089    123.66737    118.51685    121.23565    127.57791    125.62827    127.50395    127.33906    128.60777    126.33958    126.81633    124.50703    119.91916    123.70486    121.25454    123.48692    125.68905    124.92441    128.79382    127.76973    127.30174    125.45654    121.90378    122.19977    118.24245    122.90073    118.14958    122.58737    121.89197    120.00718      121.254    128.57633    126.35177    125.94945    123.17032    126.89666    130.70336    131.93336    128.17336     130.4252    129.51037    128.85279    127.30975    130.17357    131.37744    134.85454    133.70495      135.244    130.26932    135.75991    135.29977    133.17024    132.16402    132.87231    134.37153    129.72071     131.0752    130.13531    128.89216    132.08549    134.28695    131.61922    135.27838    135.71128    136.71204    136.08573    133.48559    135.29537    135.71675     135.6167    136.81436     143.4441     139.1242    142.00464    143.72918    146.03877    144.38686    141.51477    138.22663      135.324    134.56633       133.85    134.12311    130.51257    131.29804    134.24628    133.65312    131.81818     136.3657    137.28277    132.85561    132.12854    135.69876    129.07261    133.09474    132.90249    134.81574    135.60419    132.80523    132.44715    130.39125    131.91266    132.60237    131.43898    132.64603    131.77159    127.39836    132.06634    128.91023    129.76175    131.93414    128.78571    128.71096    126.06987    130.54166      130.841    131.00346    129.93124    129.28283    130.55724    134.12491    133.90224    134.13484    132.61151    131.41739     130.9943    131.03433    132.42538    132.89615    136.78954    133.84907    133.49511     135.9404    133.49085    134.71809    134.98073    130.19124    131.22189     135.0184    133.44712     137.6842    139.48085    141.30758    140.45626    140.71806    140.57346    141.86061     145.0245    147.24651    138.95431    141.98135    143.39346    145.75416    148.22651    148.35138    145.31945     147.0984    148.78816    152.43148    152.33093    157.76794    151.84401    159.67243    153.71754    158.05231    165.90297    158.88612    163.43293     160.3121    162.07528    162.70337    164.76356    171.26379    168.21958    165.74005    166.12557    164.07741    168.12954    171.40286    163.65074    171.75524    169.78887    165.16822    168.65108    174.05689    171.41916    175.89034    174.15114    173.50955    175.71363    173.74653    177.98058    179.99632    177.87137    180.04196    173.52566    178.81172    189.16702    181.17516     189.4278     182.7491     176.1475    186.56025    182.38151    183.72932    187.34826    194.54533    196.67409    191.19558    194.37559    189.63479    197.79161    200.91497    197.39835     201.7308       193.52    199.89436    202.76838    200.76817    212.36494    191.31609    220.58784    202.16278    222.07436    217.53762    211.92552    214.17923    209.61572    214.31477    219.28239    227.69705    224.67362    226.08039    226.42142    222.50817    220.88221    243.44349    233.11775     211.1538    228.37723       227.95     225.2177    246.48386    241.33741    240.08098    232.67381    232.35745    232.16318    232.94897    246.28251    245.24778    253.45909    251.25945    241.36121    244.16039    267.62062    253.53814    248.41567     271.0568    279.33677    267.94006    260.46659    267.41608    283.19652    283.02678    295.06263    259.09626    287.76947    308.45954     295.2082    301.07591    299.99009    319.68184    290.97938    344.23067    354.65852    334.58582     354.0844    354.18163    344.17753    335.41965    381.96455    408.33146    414.45998    424.36118    427.28493    445.77935    428.01103    492.17919    498.03838    448.97414    526.09323    503.32339    545.50452    738.55597    698.36891     824.3253    764.26952    756.98888     685.7669          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -1.00    260.08202    246.19117    230.06593    224.23447    223.43653    220.95281    211.61451    206.50207    193.63579    196.36229    195.45621     188.3117     188.2498    184.41865     187.1742    175.63722    176.79058    174.81184    172.19276    171.32708    164.52555    162.86665    167.90996    164.09507    167.57039    181.87178    195.76573    205.20789    201.54681    177.45416    158.49525    155.41259     153.2607    149.46439    154.42536    150.17638    152.45168    151.72517    150.09082    141.96208    143.70769    143.65477    143.58095    135.85121    144.46006    143.10802    137.17271    135.70288    138.26696     139.1077    138.45183    136.86598    133.10608    136.58402    134.65325    139.50243    153.25687    165.44718    178.87225    168.90124    157.21545    144.00157    137.64884    130.56507    128.62844    130.27064    128.39477    128.50981    130.21255    132.49408    142.82401    159.69821    162.87511    156.55869    151.71829    147.50351    145.72164      146.866    141.00327    135.63162    135.27225    132.87285     129.4406    129.17917    123.88023    121.77558    123.01742    118.87342    119.84297    125.48373    132.96632     138.9636    144.13714    149.25278    152.22799    152.43321     160.9725    159.70208    162.16578    158.24171    148.07321    140.30095    130.80873    126.45137     119.3344    119.24647    120.19907     119.9545    118.62864    121.93362    119.43295    122.54734    120.36499    117.37945     119.0777    119.00764    123.65803    124.37419    127.10352    123.92441    121.99366    119.76023    116.09839    116.31637    129.08352     131.8078    135.66413    132.07675     129.8998    122.06373    121.09246    117.62499    115.81907    117.41707    115.59163    121.28558    127.66134    140.41971    149.42137    158.16893    162.87071    156.67208    143.93185    136.84646    130.59914    125.08913    124.15073    125.59442    122.23858    119.27789    118.72875    118.29331    115.48719    113.65104    114.81957    113.30466    113.15657    116.96177    119.61944    119.62004    119.90113    117.54071    120.26276    125.03434    120.03192    122.21895    126.28415    129.91136    126.11327    129.08588    125.67668    127.25195    128.04594     125.5776    125.40677     125.4006    126.49671    129.62879    127.44683    127.04626    124.56823    122.01528    122.98834     121.8537    119.79497    117.25903    117.50135    118.19796     114.7585    118.17741    117.74288    119.86635    121.44837    122.66511    122.52268    125.50617    128.50306    127.55435    129.24017    128.58862    125.40224    124.78083    124.86703    126.42283    126.03616     123.5881    122.83895    117.61188    120.78349    128.59489    127.17343    126.75169    127.10914    121.25691     121.1292    122.81631    120.14631    122.75874    121.65854     120.5642    121.74508    125.43504    124.88323    124.50097    127.05361    129.36236     126.6751    128.45551    131.24396    131.49936    128.14179    128.11115    127.88788    130.93437    132.76813    131.53747    134.94952    132.33434    133.97686    134.21037    135.62792    132.62283    133.58723    130.22107    131.94324    129.66126     129.6236    129.76389    129.09973    131.90352    137.12585     134.5631    132.44214    133.71093    138.77029    138.03458    138.29287    134.95703    134.79509    140.62899    139.56718    138.38104     142.9936    141.08968    142.64215    144.39507     142.4488    143.31305     141.3289    139.06172    137.38702    133.57622    132.26243    133.51854    135.22743    135.42881    130.69362    133.91783    130.88389    134.53832    136.56193    137.39056    133.89906    135.09992    134.99808    133.04004    132.22337    129.52894     132.7216    133.33387    131.99655    129.81247      133.379    131.63555    131.57701    129.38763    129.67318    128.82017    132.57499    132.92991     130.0505    127.46293    127.46851    131.26737    129.62034    128.53899    126.53704    132.60722    130.19716    131.67239    130.31207    129.12151    134.06263    132.74523    131.62594    135.15414    133.48621    130.77734    136.27887    135.86634    129.98382    132.00567    129.17731    131.60953    133.73173    134.69274    132.57566    132.65237    132.62434    132.85858    134.15087    135.02148    137.04675    137.86234    141.63769    139.36622     139.9282    145.44366     147.4169    147.75836     141.9049    142.18437     141.4491    147.54463    149.19992    144.39628    149.85512    147.08505    146.30699    151.76511    152.63923    154.27031     153.2923    152.59634    155.01939    156.84158    162.25237     161.6074    162.99511    163.24727    159.93098    161.87832    164.69436    166.74143    166.79681    167.70392    170.02257    163.22226     168.3298    167.81419    173.36203    170.16837     168.0353    163.36422    171.96099    174.25724    178.31416     174.7703    170.77598    173.09781    176.98529    174.28253    173.26408    181.13686    172.15454    176.43165    178.95148    179.16784    184.28967    181.48382    187.58926    180.82061    177.54681    182.58142    181.96015    192.35997     185.4407    189.16855    193.24246    200.08233    197.99045    200.98598    194.43468    203.23522    192.06502    197.12271    201.77253    201.15146    205.50512    201.64652    214.07316    208.10964    207.99253    214.75868    221.35664    216.05752    219.05059    219.65434    220.19598      215.576    206.98477     230.0616    229.87015    224.02866       222.54    219.46804    217.94911      236.018    225.75508    215.15417    235.36905    226.59475    215.54854    228.87803    240.79684    250.51003     239.1192    234.37685    245.82925    242.01446    235.10221    248.68687    255.88539    242.76388     247.8268    242.29921    265.27348    247.71574    269.06466    259.54679    253.96072    256.63476    261.70503    282.31483    279.21685     294.9805    272.97557    290.47959    268.52448    318.15738    288.79107    306.79381    315.76069    307.26212    294.87784    317.35074    353.02832    331.14064    351.96465    347.93667     366.6439    351.46168    366.71101    408.23338    409.76779    420.27989    405.24753    386.84402    434.82201     480.1662    491.11111    520.84245    515.45705    486.68711     582.8849    678.74931    778.24827    703.69031    656.22126    552.20358    665.01182          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       -0.50    259.54101    248.41108    231.73276    225.74018    222.26292    215.34466     212.4708    208.57456    194.31764    196.47205    190.50452    189.75834    191.70324    180.57414    187.45669    174.77272    175.19003     174.0312     172.9117    171.45515    164.82389    165.42545    167.01961    164.75228     168.2933    183.45519    191.62762    205.87232    200.85294     174.9112    156.07775    154.85935    155.25706    147.01535    153.94852     149.7325    155.32327    152.17855    150.40074    144.89127     144.6537    142.54437    143.43991    142.72397    143.45557    139.69096    136.70698    140.10071    140.04213    135.33831     143.0249    141.93363    134.13449    136.12648    135.83913    141.93797    151.39322    165.91009    176.76897    173.97607    157.53063    142.70327    139.19419      134.634    132.79423    130.16049    128.37747    131.68269    132.30071    132.35869    140.42196    158.68891     158.9346    159.49535    152.91245    148.31508     148.3891    151.58735    144.00872    143.14799    138.69822    138.09454    134.23879    131.62834    124.35577    121.94529    122.37765    118.46952    123.60158    125.86891    130.80684     138.9902    144.14191    148.56325    153.60342    152.56119    158.46732      162.389    167.31061     162.1783    148.03099    138.22988    130.20715      123.536    122.13844     123.2552    123.02778    118.74911    120.83425    121.33004    122.48401    121.28917    120.00957    116.85894    117.42397    122.63943    126.37835    125.88532     125.8019    126.41768    121.49457    119.02118    117.55994    119.33697    126.53716    132.12149    130.54445    134.99868    127.19019    122.03183    119.89242    118.20024    115.42152    117.10821    116.13584    120.24712    123.70357    141.06764    152.41153    161.17116    158.73715    157.85704    145.77968    141.79462    132.82715    126.16836    122.86096    122.98573    125.32148    120.40205    117.23825    118.33242    116.13088    114.65383    112.87559    115.46311    116.00246    119.39675    118.79257    119.37278    116.08213    114.16819    118.37839    118.17222     120.6674     124.9581    128.03941    124.44968    122.68837    130.16451    128.60346    129.84426    125.90506     125.6485    126.49656    121.88197    127.55481    125.45341    126.31011     127.7962    129.67409    121.93903    127.94229    123.22278    121.18428    116.79675    117.11566    118.05556     116.8571     116.2253    119.12479    119.88787      122.241    120.53764    122.29449    123.75075    127.98106    125.42239    126.98707    129.44094    124.92988    125.14521     127.1444    122.95143    123.13117    120.27554     120.9751     121.2503    126.06747    128.58735    125.44732    125.12475    121.58062    122.32425    122.64225    122.71893    122.99477    122.66937    118.84532    121.77489    119.73098    126.88749    126.52439    123.83397    130.08648    124.52117    125.40245    127.84107    129.65364    127.40533    128.70273    130.52168    132.90003    132.36822    133.02113     133.5044    135.97789    132.29707     131.8806    132.60752    130.33294    134.30817    129.01767    128.05198    134.64277    133.94286    130.06443    135.34976    132.54998    130.44897    129.34712    131.17147    133.50948    135.51517    139.41285    138.81028    135.71707    140.81148    136.45795    136.36058    139.56115    141.30272    144.75499    142.10663    145.07398    141.32811    143.76869    144.96045     142.2378    136.21126    137.33498    134.77814    134.08232    132.79838    132.90933    133.28833    132.43338    131.83855    133.21846    137.19674    134.96782    135.37393    135.33921    137.83419    133.02284    131.84717    136.32927    135.49639     132.2443    132.05325    131.27376    126.80914    126.74601    132.72555    130.77898    133.02372    130.92822      127.901    132.43523    130.75131    128.95337    127.15837    127.98782     129.9202    129.74152    131.68542    134.24439    129.79236    130.61859    132.47293    130.16124    130.33239    131.45283    133.83927    135.62119    136.79903    134.46468    132.16322    135.26292    133.37389     131.5293     134.7538    130.77896    133.56111     132.4934    136.26421    135.94129    133.70792    131.27332    131.72351    138.12676    138.90521    141.63851    137.12583     141.1935     140.7723    143.72198    144.19373    142.93095    140.87946    142.45844    143.58359     144.0353    147.54808    148.51847    149.78363     149.3567    146.64423    146.89299    150.56774    151.39866    150.51143    154.31272    157.44501    150.43077    154.44785    162.04307    160.07871      163.194    160.34141    161.47176    161.66889    164.89933    158.41857    165.60723    164.11935    163.24116    163.99777     161.1114    173.45708    172.66738    170.11697    168.49444    168.54111    167.86688    169.17508     177.4937    174.22976    168.28219    169.85978    168.83606    174.90515    178.99652    178.92472    175.19118    178.21621    178.21223     172.1955    184.52904    181.42803    183.09245    178.84289    184.04634     181.1856    188.01705    192.81179    182.50084     192.0829    190.71865    198.98919     199.5927    195.94374    190.15557    197.75561    197.13055    199.54755    194.30678    195.69415    207.27069    202.56031    197.27682    202.09935    207.64018    207.15338    211.59417    216.20462    210.22726    213.11209    224.71818    216.07365    210.46224    226.30132     224.5671    217.66329    223.74701    206.77529    218.32656    230.88108    228.00785    227.16778    228.20331    212.51239    233.75508    228.02838    237.98353    253.68515    241.53094    231.17311    229.89822    241.33724     242.6796    245.45216    250.78568    247.29218     266.0575    242.07546    265.14918    262.28785    265.78034    246.63609    253.87999    249.56396    274.38847    261.00358    265.90786    272.60005    269.01741    287.74827    265.90903    262.26446    301.19404    311.36995     299.6813    298.57616     315.5089    297.35497    348.54811    344.40107    328.23422    364.48267    321.45001    359.57877    405.83557    378.30212    413.85549       412.32    405.43888    392.01516     443.5809    480.87275    506.97746    511.31805    452.82379    541.33014    536.86556     674.8886     680.4434    698.22992    744.42671    697.51736    679.72964          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        0.00    258.74869    248.65342    233.87741    227.69238    222.33828    212.84516    214.28158    209.65509    195.61721    196.43667     187.0828    191.06457    193.54669    178.12334    186.40658    174.90214    174.50384    174.84696    173.66389    171.73089    166.59723    168.65243    166.48047    169.77085    168.90743    183.75415    187.92521    204.98581    196.84593    176.50257    156.49215    150.85423    152.68507     149.0604    155.36589    146.44557    152.30068    149.10353    147.50495     144.4707    149.08448     144.3366    146.35754    142.21781    146.71344    143.72197    138.94816    132.67847    139.76472    139.01952    138.71517    139.49024    136.27517    132.81961    131.20912    139.65753    151.32675    164.83388     174.8961    170.75933    157.14933    139.35557     134.5719     132.0915    135.91337    127.99263    127.82004    130.60355    130.93031    134.09007    144.70135    157.00161    163.17439    158.54608    150.51748    148.83716    151.08269    149.00528    149.07445    148.45705    148.79057    141.89612    140.02791    129.84893    123.05937     121.2109    120.90199     120.8394    124.54567    125.19826    131.31222    138.40482    142.29891    152.40331    157.13788    160.06605    160.78558    164.67185    167.27623    161.60213    148.39131    136.25936    135.94316    126.29168    121.21634    120.85878    117.25134    119.67572    120.60596    122.43546     120.7667    121.12807    118.78629    117.43925    119.75054    120.67835    121.84562    124.90846     129.2286    124.46324    124.35664    119.92881    118.39315    117.69802    127.21758       128.22    135.27775    134.85719    129.59658    126.37506    118.41229    118.56293    116.75736    116.16856    117.92373    123.38436    124.88068    137.95068    151.07863    159.56004    161.34725    158.61301    145.38717     143.1762    134.91168    124.94491    123.79642    121.65469    119.36655    119.65853    119.69021    121.54576    116.49914    114.02063    116.09115     110.3263    117.15915    120.19725    119.18698    120.25294    118.28497    119.31632     117.6993     121.2409    125.20275    125.19348    125.99488    127.54564    125.57627    124.68305    129.42869    129.85953    127.50269    128.26628    124.88131    120.68727    123.56378     128.8755     125.5026    129.29289    128.60672    127.06429    125.04701    124.68741    125.64858    120.47143    118.86365    119.24758    114.33254     118.9074    116.33461    118.99004    122.93876    121.92643     117.9149    121.54188    128.57925    128.91554    128.43424    128.63712    128.78702    127.09872    123.46426     121.4282    124.48831     121.3301    121.67809    122.79295     123.7664    126.89735    130.25876    124.39652    126.11334    120.84416    119.27215    119.08715    118.22308     119.0793    121.92151    120.27659    119.78693    122.52478     122.8813    120.28988    125.56319    125.87187    126.86648    128.83729    126.57444    129.83201    130.64555    133.15982    131.53969    133.04652    134.97967    134.06965    133.91614    137.34971    134.61832    130.57483    135.45124     135.8141    134.35998    132.56454    134.92076    133.36078    130.37998     133.6111    132.97615    132.58791    134.55887    132.70867    132.44188    137.38475    138.80177    137.13973    137.57502    136.18943    132.69551     135.7363    134.95205     141.9919    142.95106    145.26841    143.93287    143.01546    144.88056    141.49346    141.53316    141.73892     139.1511    135.35582    135.62944    136.55663    137.77534    131.46702    130.68889    132.68576    133.24074    137.77869    135.17266     133.3679    134.49528    135.77652    131.15491    136.37963    134.96424    132.89578    132.84881    130.89568    130.86728    129.34146    132.72669    129.06202    130.22474     129.4289    130.24535     129.0737    130.44233    131.36102    129.72273    125.86279    127.49781    129.23634     128.3955    129.03246    136.69543    128.54126    130.77705    128.53349    131.41753    134.72449    131.18639    134.43092    131.95902     135.1764    134.14551    134.33404     135.5902    131.85832    136.51154    132.28955    134.66987    131.75699    131.14846    134.74838     134.8181    134.77806     134.6543     134.1122     140.2657    145.03068    139.82832    134.99189     139.0632    137.96822    136.68932    146.13952    143.30371    145.62667    140.76095    146.97506    143.51298    145.81483    150.00656    149.11041    147.62302    152.95683    148.49066    148.74846    150.44217    150.42415    152.38873    156.73017    155.68242    159.03399    158.27249    158.93525    162.74176    161.90479     164.0562    159.89751    161.18116    170.58419    161.47442    166.34592    163.21399    163.86267    160.79418    162.15251    165.83702    170.31694     168.6504    166.48924    167.63189    165.43019    173.60433    174.15031    172.93846    172.21755    166.51554     170.4323    175.21378    176.89144     177.0308    168.34048    177.75066     177.2027    181.62719    178.27165     182.4956    179.06313    185.03365     182.8324    185.19435    190.41457    189.32617    189.06201    184.56466    185.09238    191.61647    188.30993    183.87904    193.92913    198.36424    194.80405    186.00648     201.4913    195.68372    201.91361    206.40464    197.25134    206.47955    205.15204    212.80757    213.22829    208.99166    212.35123    213.27779    207.40276    212.15789     212.7835    226.12663    223.82638    217.16931    219.57895    205.00982    227.71152    219.77034    226.56246    233.60828    230.72461    225.77473    223.79255    231.23734    234.29579    243.46605    237.72581    234.48328    240.09951    232.11965    251.82828    245.91075     245.6412    227.35455    232.46903    262.67309    257.49944    269.48824    269.43559    256.20898    271.61667    260.33623    281.15017    274.59866     278.6496    278.97804    280.51813    292.78313    317.84282     297.1439    301.19335    295.97821    304.09993      313.419    313.91626    351.90032    375.52423    350.86567    376.72428    340.71642    344.17592    400.17603    382.43906    407.47392    384.54757    361.52905    404.00405    460.51223    492.99549     537.3238    523.60942    489.32002    512.58388    589.61771    618.72631    870.53604    623.91733    616.99429     614.6424    712.08198          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        0.50    257.50244    245.34822    236.90999    230.55581    225.14787    217.54616     218.5965    208.19919    198.66903     196.0035    188.30061    192.02534    190.37145    180.49809    182.19828    178.29613    177.69124    177.81836    176.71196    167.80483    170.11936    163.59599    165.49838     168.0206    169.55858    180.86228    194.98819    205.42777    202.60671    174.43175    161.02424    154.16553    154.73342    152.37636    153.42942    150.11942     151.2432    152.85835    146.97086    147.10206    151.40468     145.6496    142.54115    145.47742     140.4588    145.52857     139.0196    140.00833    137.90051    140.12511    136.66601    137.21089    134.11241    137.80281    136.89201    142.50035    151.72988    169.64378    177.24559    173.55766    157.52775    143.94324    135.55051    133.62074     131.9917    125.08804    130.80808    129.26984    125.93409    134.68505    142.17443    155.08792    159.11371    161.05037    148.49158    150.45799    152.10478    156.34139    154.67365    152.66313    158.16798    148.24735    140.81519    130.02278    125.54255    122.79971    122.20746    124.59979    120.91411    129.37963    134.50739    140.79923    149.06587    157.26702    165.03196    168.06298    165.36408    165.44052    165.98152    159.82616    148.23097    141.25028    131.26853    126.44004    123.48703    121.73393    117.90251    120.59095    119.93938    119.89626    121.15638    122.88597    117.04422    119.45132    118.44289    120.48197    122.51077    125.09669    128.15524    126.16295    122.54281    119.89607    117.07884    116.61894    125.58552    128.15178    133.60152    135.77293    129.37222    124.80046    119.99475    116.73847    117.51386    115.34991     117.4585    117.45313     125.9045    137.97765      147.845    159.88132    164.08235    158.63563     148.6145    142.82106    134.29601    129.68055    126.67049    126.19055    122.83289    119.48239     123.5153    116.37635     118.5592    115.00168    112.29835    115.66048     115.0576    117.40185    118.54503    119.11787    117.40115    116.29086    116.84525    119.88317    123.52917    124.74493     124.6101    126.22127    120.01549    125.91116    121.48433     122.7786    126.74049    126.80941    125.19547    122.50244    124.78185    126.59933    128.54322    128.85898    131.04108    127.07758    125.62081    121.96303     121.8399    119.23042    117.59124     116.7989    116.52183    115.35679    116.45285    120.12684    119.64004    120.81374    119.12806    123.99449    127.48656    127.07613    125.35098    129.35978    126.63058    127.24661     127.8059    126.06346     123.0971    122.40273    124.06131    122.93027    125.81546    125.92272    127.27688    128.03794    124.12893    122.08591    121.74831    121.53613    125.74389    120.86872    121.56606    124.02394     119.2743    123.56327    122.57165    127.64488    124.59568    126.82464    124.48305    131.39765    132.22124     128.3597    129.63573    131.29793    131.01708     129.9234    134.72735    133.71701    135.93411    134.65239    136.70597    131.57505    137.03301    139.22018    134.04056    131.46703     128.2456    131.94782    128.62114    130.87552     134.9792    135.58021    132.15323    132.09601    130.79857    135.39389     138.3129    134.85844    136.69238    137.47033    133.37215    135.12694    139.08862    138.13669    141.98272    147.50845    142.67824    143.47747    145.95201    144.83108    143.95066    139.62227    138.04365    135.62753    134.54101    134.14195    131.59885    134.13704    132.05895    133.01948    129.28543    137.52862    136.68146    134.89185    135.39311    136.70114    133.84608     134.4792    133.48859    133.01941    135.68106    132.61462    131.44123    130.56217    133.31814    130.54623    132.90971    128.51108    129.93171    127.56646    128.53193     130.6962    130.29608    127.59392    129.07937    128.27651    126.50642    130.18559    129.14657    131.91488    130.58099    132.12878    130.59428    135.20022    132.02033    131.77823     133.2754    134.53898    131.29005    132.68835    134.51194    130.47684    133.61352       131.83    134.27915    131.52414    134.80732    134.43869    132.94876    136.46238    132.88341    135.61274    137.60421     137.3128    140.57569    137.13491    141.60351      138.819    135.67438    148.77923    146.20553    144.94724    149.03398    143.96679    140.52359    141.92737    154.66009    141.61814    146.54124    147.72739    146.44987    150.64954    148.58142    154.88943    150.86115    152.86851    149.33843    154.80497    158.74777    161.66561    159.32224    161.51765    162.20429    167.70278    160.03444    162.72583    170.99869    163.72953     165.2588    165.63392    166.67126    159.06655    163.73186    160.72641    165.86529    166.64329    170.27363    170.57648    170.29422    174.39394    166.99035    168.02639    170.00877    172.79941    170.65194    174.37044    176.67205    175.65223    175.22931    173.38393     179.4349    181.36477     181.9338    180.03843    179.36768    180.92761    181.10656    180.07723    189.11892     187.8312    187.26347    185.57123    189.24815    190.85021    194.76842    200.65814    191.38384    189.01411    192.73496    192.24679    197.37564    200.28252     207.5109    202.41198    207.73878    209.55493    209.30176    215.03519    206.86989     211.2105     214.7505    216.88865    207.82589    209.14794    220.58215    217.56445    227.70314     218.1085     219.9535    224.26991    219.47882    227.84976    223.75606     218.3323     218.8829    245.15143    224.02008    229.16023    225.46559    230.90329    232.64571    230.57444    237.42745    233.57509    250.72216    240.61517    234.32034    236.53852    245.25719    239.14698      247.364    262.48323    259.17043    260.78535    266.73435    275.94258    286.05643    268.05089    256.64419    279.49543    270.62036    303.98516    265.11413    306.50057    307.49051    319.77755     269.3955    297.77074    334.25936    337.53492    330.21404    374.15239    340.65789    338.76092    380.95439    426.95056    432.93764    415.58053    388.25376    404.32554    397.45805    499.78241    475.03066    496.49591     445.5526    477.83748    574.92391    352.46089    640.22478    684.74638    619.11485    664.44465     663.0968          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        1.00    256.27323    242.05183    239.94294    233.45873    228.03855    222.14549     222.4183    207.03017    201.34816    196.61085    189.66834    191.19137    189.02477    181.92025     180.9579    178.45265    179.88237    177.17012    173.79797    167.93271    168.21252     161.9593    164.94591    168.00486    168.23735    175.68363    196.77782    209.10509    199.46627    175.96435    157.22361    154.31871     156.0032    152.25287    153.92682    150.04291     145.7485    148.71635    147.00167    145.99179    142.71911    144.10181    143.77203    144.17683    140.74558    143.05044    136.72145    138.50936    139.02678     139.2311    137.50098    139.44447    135.22985     135.0738    141.29499    139.82357    147.95019    170.34475    175.77684    175.25842    154.73904    142.90853    135.01661    131.62505    134.24098    126.66652    129.50365    131.00179     130.9385    136.57871    142.35655    157.12591    160.31097    160.69169    149.16616     147.2219    147.33295    150.68042    152.56953    153.44162    156.71907    150.60274    139.94716    132.51842    125.47847    120.43516    120.34466    121.13973    122.85652    125.37882    133.59072     144.9569    159.12909    171.83692    180.33547    173.85385    175.32756    163.74963    164.59747    154.67517    148.03893    137.77734    132.30141    127.06714    121.85276    121.70404    117.97739    118.16577    120.82333    122.08567    119.50024    119.41967    120.64612    115.95162    113.58632    120.32956     120.9945    125.51631    126.05809    122.82461    123.48197     122.5995     116.7945    115.97949    127.26649    129.96116    131.70855    135.04755    129.30527    125.69137    119.35236    118.41108    115.39893     116.9833    119.98408    119.69531     126.1905    139.20731     150.5746    160.78672    161.87972    155.74912    146.14264    140.67241    135.63523     129.5244    128.13621    126.01768     123.3812    123.69852     120.2258     117.0552    117.50003    116.64484    111.15385    115.44575    116.88519    115.11045    118.63868    120.42783    119.17065    118.38349    120.62248    118.11926    124.65733    124.15631    125.27546    126.68367    120.14397    124.39784    123.29328    123.21472    123.18432    120.20353    121.26847    122.20837    126.63633    125.23064    125.70912    126.51602    127.98688    123.13595    123.77748    120.31935     121.0256    117.45596    114.99682    117.08993    116.33454    118.69512    118.27097    118.51222    121.94669    124.80177    121.44611    128.47111    126.07802    127.76594    128.41633    126.54255    125.91754    128.43943    127.09809    123.60705    119.65667    122.77075    125.33673    121.46812    127.26946    123.83996    126.60475    126.99424    125.37073    121.64886    124.62493    123.40012    122.95557     123.1866     123.2053    118.78337    117.88835    120.58673    122.16062    122.56971    125.53944    127.02887    123.86542    128.87481    129.70936     129.1795    126.32629    128.63169    129.44852    127.42986    134.74108    133.34565    133.34802     133.3951    132.40238    131.51698    137.31564    132.53376    134.04303    134.87727    132.32265    130.50581    133.16176    132.89661    135.28234    135.28381    132.69101    131.18438    133.28909    135.39239    135.82673    134.20444    136.73521    133.42649    133.96351     137.6972    138.24983    137.38416    142.54044    143.47006    142.92331    144.10295    145.89664    146.64376    143.41297    143.22616    139.26594    132.64266    135.54998    135.19027    131.91272    130.91911    131.76313      133.552     131.9264    134.99855    135.10325    133.75158    133.09357    139.27547    138.42604    134.39989    134.70268    135.96775    139.58872    129.89192    130.67622    132.30057    130.35303    130.19521      131.438     130.5697    131.86745    125.91143    126.30949    130.24294    131.70275    130.84181    128.60835    124.91282    127.77868    129.56014    130.05776    132.10379    127.74743    132.51083    131.93694    130.26862    134.13655    131.86391    135.19593    132.83742    130.23229    132.62939    135.00894    134.26193    132.42681    131.87442      133.058    133.79906    136.52021    136.17737    136.26417    133.21444    129.59348    133.53803    137.27682    139.70242    136.11368    142.34745    141.19347    136.90133    142.29589    142.29505    144.29588    140.99947    143.54036    143.30873    143.00156    147.99268    147.82402     147.0346    148.80344    144.95712    152.42121    149.36997     150.9133     154.0591    154.36625    153.69065    156.72674    155.55427    153.38659     158.7541     160.8006    160.29283    161.13138    158.67761     165.0029    167.72775    161.14247    168.06314    162.33742    164.78991    160.49498    160.11182    161.01981    160.12301      164.484    169.48629    166.71921    166.77207    174.98803    175.50147    170.74269    172.86675    175.59988    171.51858    170.96887    177.52585    174.07856    175.77006    177.72705    180.10238    181.50443    180.89123    176.09711    181.47084    173.18833    180.98296    174.26704    180.71169    181.64692    188.56903    185.23398    186.56674    183.30647     189.3986     198.9244    196.06659    192.23858    185.15558    200.82347    202.10093    193.52049    213.73426    202.25421    202.87675    202.75793    203.90606    205.98838    205.41509    212.10676    212.26409    214.38625    208.32295     203.0797    225.49938    216.82422    219.89633    223.87381    209.73006    211.64676    226.80892    221.76539    226.87126    216.72221    220.79335    222.57703    240.90896     234.4187    237.96809     223.5048    225.87936    237.92386    226.01751    239.73872    239.42674    242.90917    241.45816    231.79987    233.95096    236.20397    268.08698      240.437    255.76515    230.02377    251.58656    248.12134    283.87148    271.28103    269.71719    280.99933    268.10952    281.46128     301.4749    297.50305    289.15749     321.7651    300.62767    294.56447    308.43149    339.49962     352.0633    313.23794    348.76777    317.43204    369.61665     369.0026    407.26529    420.67659    382.18983    359.23532    398.83216    417.31882    468.32031    503.06571    496.26924    493.13111    512.58964    553.91249    612.27625    623.30162       716.82     633.7144    563.35757    601.32018          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        1.50    256.14431    241.05208    238.30965    232.38126    227.50833     221.7763    221.62913    207.57018     202.5092     199.7028    191.32286    187.57632    190.01369    182.50364    182.18917    176.45119    181.24399     174.1995    169.94205    169.88243    165.35659    161.73095    166.38134    168.98455     165.0699    178.40531    196.86567    209.29975     196.9052    179.55922    156.03046    156.48203    158.09506     151.9719    153.25808    153.85587    148.90417    147.94676    147.87806    141.19445    145.93191    143.02148    141.69134    143.19321    139.30131    143.28628    139.14139    136.62087     135.8626    141.03879    140.73644    137.76563    137.11678    133.43957    134.58742    133.82267    150.40499    165.77692    178.19619    174.92451    154.70535    141.56765     135.9852    132.24078    133.56998    129.25869    126.69284    126.51773    131.45693    131.44115    142.66421    151.56854    160.92311    157.77088    147.98743    146.87957     148.5796    152.36616    149.75768    151.88661    152.50712    149.16833     138.3464    133.78822    126.61942     125.4689    123.38896    122.63229    119.12407    126.54303    133.31706    146.88984    166.54111    186.22491    184.67888     179.5356    170.85116    161.24035    162.81405    155.46085    145.62666    135.55818    131.30445    127.73851    121.82781    121.73451    121.98599    122.37007     120.2784    120.68245    119.32353    122.82385    123.51981    119.90937    118.16941    121.05888    124.46536    126.38239    128.17819    123.06362    122.95037    118.07385     119.7999    117.91431    121.96084    129.26356    134.66685    134.03348    129.96517    124.40481    119.28613    114.39255    120.01528    117.35689    119.95886    122.14507     126.9584    142.27414    150.48366    158.79047    160.30055    157.55555    144.48393    137.55955    132.16785    128.43327    127.68108    123.18295    124.87253    119.67531    122.32124    117.13766     115.9766    116.14999    114.26135    112.60327    118.13773    118.29382    118.92326    117.09583    118.71303    121.26701     115.3132    119.32656    122.72597    120.32913    128.19257    124.46523    120.25158    123.49343    122.69233    119.54161    120.65003    122.21175     122.6119    122.01886    125.89064     127.5467    124.85809    124.50475     123.7871    118.88025    122.66553    120.12765    112.80748     122.3892    119.45605    117.08494    116.14277    117.20782    120.16452    118.88872    120.37047    120.38447     122.3337    122.69932    125.93652     125.9819    127.63149    129.81158    125.56341    127.61903    129.37993    123.28434    121.84405    117.48801    121.57685      123.524    128.33859    126.35517    126.40438    130.10882    125.03157    124.28951     122.3132    120.65462    122.22991    122.02927    118.47206     124.2811    118.10927    118.80781    123.65255    123.82552     123.8696    122.20611    124.87694    127.05375     129.2978    128.67181    129.67753     130.8296    130.33205    129.61084    133.11775    131.52799    134.44343    133.38651    132.77516    129.11533    135.06166     136.9113      136.782    135.79694    137.71486    132.64959    128.57305     130.5565     132.6284    135.35598    131.91749    129.43046    131.33248    135.97083    135.70267    136.28419    135.89623    136.51213    133.56396    135.05952    138.16133    140.75562      145.713    147.66591    144.00785    145.69397    145.96138    144.56746     142.8296    143.00624    137.87306    134.47549    133.85671    131.18811    134.94005    133.88577    134.49858    135.47541    130.32694    135.91692    133.97352    134.35033    135.11512       133.95    134.78071      134.159    134.13984    135.02227    132.11085     135.1266    130.66769    128.11966    133.22937    131.53755    130.49498    128.16101    131.19986    126.11661    131.41552    129.65299     130.9516    129.39823    129.22002    127.74334    127.23131     125.5136    130.55419    131.38113    128.03845    126.32197    130.74195    130.43988    132.19565    129.93012     129.0185    135.97197    132.12591    133.13123    133.54059    131.22491    133.73459    134.91013    133.26453     134.6278    133.65776    135.16541    132.42281    133.19887    135.23359    131.79425    135.68407    136.57882    137.84561     139.3658    141.10762    141.78844    139.09625    143.96646    142.52673    145.49467    141.20493     140.9922    141.84742    145.97436    146.32531    150.53006      143.695    146.65883    147.67432    148.87544    151.31165    151.98307    154.40339    154.64491    153.05408    156.74293     158.2919    160.73574    160.26383    162.86043    159.53201    158.96843    165.51148     167.6688    160.57828     159.8957     163.4743    166.19317    158.04757    167.63294     167.8155    159.50993    161.83909    158.79977    163.76072     175.1306    167.09174    171.31844    170.48244    163.90333    170.57029    165.69973    173.27636     174.6604    178.23303     171.0009     176.4848    168.26678    171.25873     178.0004    179.49375    173.04782    183.51686    178.06144    182.06138      188.363    189.44289    183.78128    183.56587    190.63838    187.46704    198.03805    192.12098    207.07589    192.82028     195.7601    196.07682    195.98076    199.95582    207.30089    198.20744    201.39304    202.76196    205.97285    204.84692    220.98745    211.58638    212.82903    211.49056     199.8958    220.05155    218.99833    222.85883    218.24077    212.03881    218.11936    220.75975    217.23015    224.69478    221.79065    217.71621    218.54376    222.01788    234.67643    233.65375    230.07497    227.93852    243.79059    223.53252     236.1364    249.25967    239.54812    244.28238    225.83089    237.64657    225.53941    251.76002     267.3207    249.76139    254.84945    253.04059    258.10483    267.48892    275.22492    285.97494     259.3621    272.30423    266.77042    281.59743    316.27129    300.97777    300.62274    282.22762    299.52326    289.56958    296.84899     338.1921    325.52383    328.07828    353.32796    318.10014     345.1912    383.11909    390.30646    380.97036     385.1305    405.57529    401.60501    382.14762    509.32493    466.92572    494.82512    518.99288    511.29518    502.10153    601.28132    746.51669    546.20856    625.98205    760.02978    700.49849          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        2.00    257.11209    241.28959    235.16426    230.47619    226.69798    221.46309    220.88453    208.33078     203.0073    201.00992    193.19371    187.88864    190.97206    180.54206    181.13335    176.20336    176.74748     168.3674    172.97689    172.33095    164.08936    162.25598    169.19778    169.40929    164.06821    182.90456     196.8425    210.63323    196.19706    176.71915    158.21166     157.7694    157.96482    149.97635    148.81423    148.87834    149.70636    155.33142    146.46788    142.77284    145.22146    143.21821    141.41836    142.89423    142.24928    141.49241    139.54893    140.87601    139.22288    139.00047    142.36826    132.73764      135.471    136.19896    135.92781    141.61441    150.46851    164.08503    180.19368    177.34655     154.0302    142.12339    141.48634    135.26966     131.2536    125.01785    125.78435    132.16262    132.56582    134.12364    142.69599    154.75619    163.71355    161.97022     152.4854    146.68992    146.29526     153.3558    144.45066    143.19429    138.97886    143.18135    136.65192     131.6688     124.5778    123.55611    122.29698    125.43813    125.82463    128.71528    131.39153    146.39113    166.04281    181.94305    178.96646    171.01598    164.43302    157.85553    157.25333    152.08709    145.46625    137.59977    130.64074    128.24134     123.0549    122.67902    119.77259     121.2547     123.8656    124.31335      118.889    121.37969    119.96025    119.77877     117.4848    121.58121    123.44473     124.7794    130.41077    126.04076    121.35905    119.64076    118.62234    116.00379    125.62889    131.00577    131.37388    134.72979    131.83731    123.39857     121.3536    118.55429    119.08835    117.94728    117.64952    117.42749    126.66242    140.70998    151.56792    158.80005     161.4264    156.22797    144.30693    138.21644    130.18756    126.94306     125.4087    122.88477    122.13542    122.08043    120.38511    120.00896    117.44205     115.2989    115.50583    115.02621    119.29622    117.94911    119.18477    118.87391    118.31777     117.3886    119.83153    122.95834    126.09033    128.90733    125.12762     125.2937    124.07425    125.97885    123.49782    121.05879    122.43762    123.12379    119.15764     120.8515    125.88354    126.63737    128.54483    126.64379    123.89676    122.41807    120.01663    122.45948    119.06292    115.26342    114.85476    115.15753    116.22556    117.67719    119.86272    119.35134    119.57808    119.68282    121.72842    122.98505    123.37999    124.86827    125.92072    128.69807    127.16965    128.48355    124.42126    125.69168    124.18388    124.59614    124.31385    121.85328    123.49031    127.73102    123.60485    126.37974     128.5594    126.48148    126.27839    125.68935    119.65789    123.23201    125.79539     123.2856    116.54022    122.15396    123.05859    122.84879    123.41218    126.73328    124.77951     129.9983    129.43819    127.40525    130.28503     127.9371    129.11857     124.9268    128.10062    134.26227    133.43926     135.8262    136.61121     135.0394    139.32293    134.65635    134.89451    131.73285    133.20441    134.94011    131.64699    136.42007    131.69219    131.77399    135.70613    131.43266    129.12695    136.17055    139.15778    138.02861    136.87684    139.11875    134.91528    135.32346    137.97982    139.87009    140.96608    144.02231    146.73215    145.48233    152.20435    146.60977    143.16919     140.0203     136.4102    136.45193    135.47004    135.84601    134.21478    131.29536    134.00943    134.41686    130.37909    134.66339    137.02152     135.4489     135.2921    136.69771    130.90958    133.41671    133.46001    133.64035    131.30967    133.49037     130.4516    131.61321    131.87547    131.69632    132.16101    132.23603    127.58414    128.45248    132.58192    130.90174    128.54403    130.66754    129.45617    129.74097    124.83691    131.38471    135.24175    129.23413    130.88329    129.30205     129.0684    133.13902    132.28596    133.25046    132.22115    131.84202    133.38263    133.68366    134.88476     134.6741    131.95672    128.73585    131.35425    131.77485    136.23586    131.78482    135.36628    132.34163     131.4884    137.86875    135.15393    137.92447    137.80154    138.28411    139.26424    142.40016     138.2699    139.02224    143.68457    140.45271    138.72443    141.18807    144.90831     145.6865    147.89972    146.70194    148.83647    145.61143    145.31208    149.66299    149.07186    150.65979    151.10106    153.29911    150.01189    154.48502    159.74523    157.82944    160.40238    160.39825    161.67437     164.0743    162.64183    169.22485     161.1112    161.44879    158.05388    161.37182    163.25864    167.66395    166.81592    158.33377    165.27377    163.71449    166.05006    164.04201    170.14188     168.9951    169.11351    172.63895    174.71627    163.55708     174.4688    175.33965    176.09601    176.50883    172.84215    175.65233    178.02454    172.14766    171.47163     182.0182    174.65733    181.84161    182.96794    180.50425    193.44743    195.54776    186.67558    185.69239    192.69199    194.78198    193.47238    192.18562    198.84809    197.03692    191.82741    191.54689    195.02212    197.62186    207.10923    200.62005     204.5937    203.72849     213.5104    217.84997    215.81928    210.12756    207.68885    212.63938     209.9358    215.18708    214.30886    221.87692    214.78386    219.74932    215.32739    217.47734    222.63785    226.36679     223.4507    216.65566    216.58291     225.9118    238.76225    235.22724    223.77745    222.52706    230.93235    234.50893    249.40831    236.15786    236.60687    253.20762    244.89415    235.00766    233.75427     256.2932     262.1198     244.7761    245.74922    261.54075    264.02599    271.41368      262.667    256.75028     267.4411    272.59176    266.58508    299.93528    305.69754    297.12383    288.12865    286.79416    311.98536    308.96397    318.57442    361.65041    361.89932    340.42686    322.06755    343.81447    399.65064    378.25487    373.76965    363.71801    348.71922    379.89439    430.31601    437.08187    532.52023    432.23442    483.83949    468.05876    572.64725    618.81462    648.70903    692.96387    598.10297    588.72413    691.37143          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        2.50    258.27932    241.48824    233.01473    228.75815     226.7952    221.54934    221.10579    209.65396    201.74506    198.43639    195.46801    191.27723    192.32174    177.62212    179.13649    175.00878    174.28691    167.61411    173.11273    174.32628    171.03715     165.8185    164.27601    166.42615    172.80142    181.25614     196.6546    211.81236     199.2065    176.44248    159.31244    158.08139    155.62971    148.87651    146.17293    145.35002    144.33294    146.83203    146.97886    145.38783    142.51531    145.56354    141.44854    142.49744    146.11226    136.89894    135.29985    140.75383    136.67399    141.47656    137.29078    139.95891    135.17378    133.38807    133.97567    135.86485    147.57248    166.51891    177.36825    169.31828    159.74116    144.93675    138.15735    131.58949    131.66994    129.10019    129.63827    128.20257    127.89232    132.20795    146.31424    160.96137    161.17106    161.62233    152.34003    146.70845    148.37882    151.22113    145.33632    138.44966    139.10945    143.18502    135.11611    132.49096    127.22369     129.2901    125.57805    126.29959    124.17595    125.36296    133.37419    145.04273    160.24676    168.73835    168.01361    155.89171    152.99857    152.81712    159.25106    152.01285    143.40046    137.71147    133.43066    124.93196    122.38577    119.51754    120.38011    118.87852     121.7213     122.0438    119.75784    122.78637    120.43186    121.69213    119.35059    121.08281      121.308    127.90714     127.1827    127.78208    118.98791    123.53057    119.91638    120.34166    122.14751    130.17801    131.60575    135.18474    128.71838    120.31536    119.57474    117.56473    117.26853    115.31262    118.63119    122.99056    125.21883    140.39412    154.76691    162.46688    166.46122    158.80623    146.42851    139.67717    132.81424    128.88398    125.94438    127.69839    126.73415    118.94243    121.41045    121.08511    118.31922    118.78673    116.17792    116.63219    116.17464    114.65809    121.30839    119.81161    116.39918    121.29288    120.42922    119.91398    123.50855    126.29708    125.21704     125.8516    124.75357    127.68205    124.63989    121.88317    123.79964    125.01277    122.95279    125.81522     126.1867    127.37752    129.93779    126.21837    127.00038    120.02712    122.38171    118.05849    118.13616    117.00526    115.36249     113.9399    116.74056    121.05602    118.03837    119.20033     118.8043    123.31334    121.43649    124.41096    123.38914    127.57366    129.92659    128.58537    129.99538    128.90616    125.32165    123.69441    123.17332    126.40032    127.66978    122.38915    124.85141    126.60594     128.2557    127.57967    128.08362    124.65914    126.01668    125.97886    121.41645    119.30665    119.44291    121.47184    119.73856    125.11215    126.51482    120.27205    123.70981    124.75423    124.85648    126.19696    126.75975    130.26188    130.73699    128.44752    130.09127    127.91947    132.22843     132.8991    131.84388    129.82558    132.88114    131.96371    135.34073    131.17303    130.51931    132.66705    135.74169    131.74181    131.31603    138.55758    136.51357    133.22941    133.87689    136.03677     131.1245    134.02803    135.42531      135.712    134.17034    136.11671    136.06927    137.00851     139.4552    140.23523    140.54933    143.50222    147.21587    142.06001    145.54895    146.27563    144.71206    143.65118    138.85006    136.54011    134.14585    132.40293    133.02589    131.62011    135.29152    136.31065    130.14333    136.17158    136.34706    137.56403    140.35128    136.01631    137.45742    132.59422    134.50954    135.03618    131.76343    130.38113    130.52861    129.29642    131.79308    129.92639     127.8075    126.61435    134.97647    128.91376    131.62868    128.09491    130.38756    130.43792    132.59745    130.87178    126.88247    130.43692    134.43947    134.72913    127.52488    130.40693     131.4265    138.65589    126.86405    133.31135    133.74473    136.55937    135.30508    129.49388    136.26751    129.79114    129.10685    126.54655    131.08862    130.39086    135.80702    134.49145    134.24433    132.22039    133.50596    132.58426    139.70922    142.02623     137.5175     140.7155     137.8508    139.32514     135.0006    138.10886    145.67936    146.56141     141.0618    145.72679    144.44927    148.42859    151.12131     146.9095    142.25206    147.05686     142.7639    150.65143    146.80266    148.86112     148.5297    153.77239    154.55771    155.14888    159.40688    158.43498    160.19944    157.44306    162.37619    163.01006    156.84783    161.61194    166.67304    167.97386    164.31384    159.60316    163.47682    168.23964    164.81253    161.94278    163.78893    167.21496    162.27033    170.22982    172.18178    169.79635    165.27928    167.58664    170.76373    165.51413    181.00547    173.88878    172.94431    175.10709     174.7908    175.71399    175.66712    176.46901    185.95922    179.90963    176.25085    177.19327    181.54593     176.9362    186.87804    181.66401    185.68468    185.42655    187.38911    192.70899    189.63316    192.99074    193.53555    190.65085    195.01639    202.63694    203.96164    199.08476    194.43425    206.55354    206.85053     203.7576    208.36185    212.38762    209.59342    208.46035    214.26447    210.15481    211.91098    214.27129    215.66736    216.51319    216.62513    207.61263    215.27724     211.9587    232.68986    216.02283    216.18212    223.44335    216.90633    219.74917    231.96685    231.35566     218.7034    216.58546    222.82377    242.76893    237.86341    234.68878    244.25692    242.48804    231.27325    228.22014    246.56265    252.44523    257.31314    238.49597    239.98852    247.77341    266.62614      254.183    269.90869    263.23889     275.3912    272.53558    275.71567    313.60315    307.63635    291.85463     285.3092     294.2548    300.20403    298.45568    332.32161    331.92312    329.78429    351.00313     341.9321    318.42408    391.19666    394.40212    366.73307     406.2967    359.29517    384.94586     383.2003    459.97987    480.93186    411.07456    479.32042    482.24494    525.06586    691.59885    557.63097    487.24187    559.25844    743.51391    749.57786          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        3.00    258.87646    241.22966    235.90927    227.61941    227.73814    221.88421    221.30893    210.88389    200.92044    196.89624    195.13301    192.69948    192.53467    176.80666    175.13614    171.87146    174.14319     170.8073    173.65207    173.84679    172.12469    167.16228    164.48132    164.72996    164.38951    178.40925     194.9541    206.48693    204.14043     181.1477     162.7742     155.1916    154.64317    149.07082    152.53306    149.22092    148.65833    146.00121    146.86702     147.0358    146.19216    145.03028    139.20376    141.28505    138.82975    142.43369    142.72043    135.81103    136.22847    136.28113    135.49607    136.99243    132.83312    137.43078    135.38952    140.47663    150.86299    169.80421    177.92609    174.50742    158.08928    145.02792    136.95714    133.54125    133.87927    132.50649    130.74655    130.57558     131.4033    130.11728    143.14226    158.98194     163.9158    162.85998    155.54098     150.4086    144.19203    147.48816    140.16092    138.26133    137.11754    142.86942     140.4894    140.08871     128.2118    126.82783     123.0334    121.91166    124.22005    125.42164    134.57897    140.86167    147.72162    155.29636    148.96947    148.15178    146.44663    145.00988    153.31672    148.84901    148.58032    138.13603    132.53511    125.83481    122.64866    120.83622    121.90429    119.00407    116.65902    120.71445    119.49309    121.19237    121.74589    121.14237     116.2047    119.59132     124.2088    126.74993    124.89169     122.5536    118.19153    119.17582    117.61557    117.05468    128.60666    133.54411    129.75049    129.76401    129.57427    128.32221    118.64615    117.55292    117.36973    116.95952    118.22304    121.44261    128.57031    138.68387    147.70303    160.77057    162.82895     158.9991    146.11338    138.83639    132.28068    126.64109    124.84389    124.46946    124.15532    124.29594    123.56221    120.41571    117.50162     115.6183    115.53277    113.45479     116.5256    116.10719    119.25495    121.67324    118.41585    117.48307    118.85552    123.19996    119.54816    124.66073    123.42963    127.38856    125.31109    124.25332    124.23236    125.01633    128.84343    126.35546    123.86297    122.22617    129.01375    126.57991    123.02626    125.84617    122.66869    121.09641    118.89609    118.34482    116.72114    117.08296    117.81779    115.30289    118.00652    120.55447    119.97271     119.3642    121.88364    122.79046    121.43701    122.21855    125.01028    127.95301    126.40482    132.73888    124.99875     126.8215    125.37041    123.94094     124.5051    120.92156    122.93851    121.83576    122.95385    122.62349    129.23519    128.63052    124.89302    124.99734    125.20087    122.99311    123.35687    123.88834    120.07956    119.60237    121.50123    123.70339    124.12019    124.93056    124.46779    124.66592    121.28309    128.52303    129.43511    130.79563    128.14303    131.77249    130.22016    127.95447    129.70021    135.52776    134.42906    135.89542    136.58292     135.3996    136.21049    138.62003    136.49635    138.18312    134.17512    138.81876    134.84685    132.83006     133.3501    131.31892    133.55983    134.56459    130.99542    133.68203    136.20162    138.06115     132.5815    135.81175    132.79665    132.81199    138.43442    139.40761    142.43117    143.38325    147.50243    140.09542    148.60959    146.17235    145.00927    138.12528    137.70592     132.3091    134.03162    136.37975    133.74858     131.0658    132.35741    138.37226     135.0105    135.70603    138.27627    133.76617    134.87372    133.89298    135.74512    137.84966    134.96615    135.90116    137.78706     129.3738    132.58959    127.83683    128.65075    131.25732    134.50123    129.55782    131.29133    128.67609    127.46996    132.12399     129.8762    132.41863    129.01897    129.10974    123.02405    129.45229    126.57662    126.02883     129.6855    129.83416    130.15781    129.19288    130.01453    134.18717    131.89563    137.82109     135.7873    130.43521    133.96843    133.61676    133.46106    133.07549    132.32235    130.25051    133.75248    134.11024    127.80364    130.39135    134.35651    130.97158    131.93024    133.11976    134.95548    138.56056    135.54545    138.59896     140.1834    145.28158    140.20008    141.04708    141.03822    142.72977    145.56717    148.53109    149.72813    147.35037    142.03646    139.71398    146.76282     145.1746    153.09182    148.63259    149.27597    150.61585    153.14166    155.67042    155.82903    159.48692    160.15727    160.12691    157.87942     162.8382    157.61579    159.56223    161.09955    165.09447    159.53643    165.57565    161.15225    165.65152     162.2848    170.06601    170.07839     168.7367    169.24803    168.65852    164.78306    171.67012    175.88048    173.62527    166.29531    172.19726    175.94772    171.50154    168.80902    173.75764    175.82666    171.04785    174.87086    177.63516    174.96382    175.95584    177.64831    176.06939    182.36551    187.17811    188.18999    182.89206    190.27981    188.75761    184.11626    194.02942     187.8954    198.59805    198.53582    200.49792    190.31305    192.01534    201.18771    205.31601    204.65859    197.04306     198.9688    199.21593    209.41584    212.59168    206.32944    210.41395    206.75424    207.73871    212.67276    219.39532    219.58578    208.27101    208.92659    214.62158    209.26751    219.45776    226.48182      218.547      214.459    228.12655     223.2113    214.01246    226.99814    224.06605    221.04377     232.4298    226.03554    228.45344    240.22518    238.55166    235.59347    247.66273    235.71405     237.5537    251.55662    246.54661    260.00467    239.23508    256.89107    253.72147    241.18129    267.15785    270.48311    268.00633    275.88616    255.43369    264.55506    306.49175    277.33239    289.63027    266.76059    302.94731    308.88374    308.34607    330.29781    310.65081    360.87897    322.75251    336.78286    336.30996    367.61974    403.00903    392.24859     382.8351    362.40461    375.19499     426.6726    464.81765    500.90866    436.37092    492.92871    493.38403    576.51281    675.27783    678.28769    643.16538     592.1906    580.42662    645.69704          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        3.50    258.99786    241.10011    238.88206    226.49952    228.48188    223.51629    219.05895    209.23435       201.97    197.51872    192.37609    193.60648    191.95639    178.18749    176.55655    171.19151    176.50476     173.2199    179.32833    168.98939    166.03235    166.22866    166.18868    164.31244    164.90621    178.25336    196.64167    204.57532    194.97587     180.4377      166.847    153.20684    154.36728    150.63819    147.55432    148.87684    148.65092    142.76337    148.64798    145.63238    146.89922    139.88089    141.38896    146.61848    135.19062     143.2821    142.72518    137.31985    139.65007    139.62192     134.7781    133.74483    136.20426    137.93193    135.45845    141.12361    146.06229    168.36816    179.27065    173.18802    159.50588    139.52159    136.86466    136.01497    129.88726    130.90909     126.4374    131.73032    130.13669    132.95578    142.59146    158.68562    165.50678     158.7218    150.93885    147.80817    148.74236    149.34408     138.6154    136.23985    139.52839     142.8292    143.70647    139.02833    131.19809    129.92729    124.97434    126.01849    123.99898    126.06388    129.57368    136.75805    145.91523      148.747    149.16726    142.34872     140.2716    142.54465    149.28545    147.66472    142.16438    137.26964    130.04376    129.03234    118.15087    121.91964    121.02457     118.6523    118.19679    124.33737    119.87149    121.12867    122.40449    119.44297    119.82231    119.23219    124.31735     125.7746    127.25583    125.70816    120.69399    117.57149    117.61835    121.97933    124.83874    131.21278    134.10209    135.89693    128.97589    124.82614    121.88167    118.84971    118.25527    119.58746    115.42849    120.59303    125.80336    139.74723    149.42878    158.57064    165.54973    157.84948    149.50658    132.31027     133.3914    128.92977    124.50156    120.88249    123.67281    119.93433    122.02178    118.33306    116.23991    117.49481     115.3259    113.17828    117.52594    118.34862    118.71014    119.27087    119.16855    120.55429    120.39545    120.35339    121.97052    124.00473    125.26127    125.98568     125.8668    125.70244    124.29887    123.31488    127.27178    124.38259    123.19674    121.60147    126.40662    123.40317    126.66507    124.37978    124.09886    118.85497    118.67369    118.90355    118.97597    118.26362    115.00273    116.75451     115.8171    120.91989    120.65778    120.19465    120.44404    124.60494    122.52039    127.07907    126.20648    127.79882     128.2494    129.29757    129.76306    125.34431    125.01065    126.65546    126.02219    121.22279    122.71087     121.0739      127.168    127.03213    128.56047    122.74153    122.90124    126.69998    123.45663    123.57844    124.27366     121.9183    122.09769    119.50754    120.38727     123.5305    126.19659    125.70043    122.47495    126.28842    125.90143    127.31571    129.40337    130.43673     127.8922    129.06319    129.33338    127.08227    133.51341    134.69476    135.60297    141.38122    141.16087    138.72337    138.35766     139.8786    135.91151    137.63954    132.58381     132.5961    130.00761    131.62075    132.47424    133.86323    135.88952    133.76108     129.7604    135.38169    132.85696    136.32282    132.81707    136.09313    135.86167    137.42798     138.7338     142.8207    143.73158    143.60698    146.94694    145.23229    146.49163    144.89513    141.86363    143.84473    141.74053     138.3366    133.86775    135.07187     133.4656    133.13411     133.6216    136.25115    134.88343    135.34454    137.55481    133.09344    134.97768    139.82487     135.6099    134.17192    134.54854    131.78766    136.61941    131.10392    131.28712    127.80312      134.682    130.14879    131.00882    130.61019    132.96115    130.61563    129.38149    126.32984    131.08801    128.72057    131.59564    129.01537    129.84195    128.76071    130.04969    135.06814    131.79536    133.43261    129.11502    134.52685    132.54518    128.78831    131.21527    136.50734    132.57325    127.22981    133.08343    133.44614    133.95772    131.34554    134.88506    133.81907    133.62507    132.66001    140.29849     138.5276    131.46418     133.8507    135.59725    134.10654    137.20889    138.81243     138.0082    140.47519    139.40173    140.79499    141.81646     145.3147    143.40804    144.40031    144.06668    143.84302     148.2559    144.21371    146.21293    142.27233    141.55052    146.19514    160.34475    149.51635    151.16042    152.30444    154.56589    158.91344    159.37718    156.73392    158.22583    153.84804    155.41757    156.38825    155.76192    167.14457    161.39489     167.1998    163.52666    163.38009    161.76911    166.26704    163.66863    165.91952    160.07752    166.14103     163.2867    165.10421    166.83373    174.62505    174.44442    170.86127    172.79021    166.75856    175.01918    168.18525    164.15436    177.29346    172.50169    177.55948    183.56284    176.84939     184.1693    175.76147    184.82223    174.93107    183.14028    186.08204    185.98012    181.10263    188.78892    188.96909    182.10725    194.65879    190.80152    190.53278     194.7295    196.75813    195.07651     191.9948    193.25246    189.19887    198.98871    205.94216    194.98648    205.15797    209.19646    210.87593    212.42249    208.07739    211.89599    205.11299    215.85586    210.61566    213.10264    216.78621    217.87732    207.82987     207.4814    226.57453    228.44319     217.3058    215.28222      216.613    213.42559    226.76982    218.29421    233.35584     218.7709    228.58571    224.09107    238.91368    234.81458    237.92446    247.78714    229.26652    220.07666       235.83     252.5776    264.30152    242.18839    265.39665    255.09169    247.04354    253.47914    264.59729    259.82061    265.03283     257.2678    266.70289    263.39988    285.38708    289.97567    302.99197    297.20547    290.98684    304.65922    302.18094    321.22977    310.46413    326.18927    330.04147    324.06434    338.46298    374.52016    382.59081    405.70801    367.36436    411.48611    396.01864    415.41622     453.6534    499.10366    497.78771    471.69486     445.0496    542.15232    659.19287    619.86117    650.54771    674.54883    686.49201    560.68274          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        4.00    258.91607    240.70108    239.45863    228.02394    227.82482     226.2187    215.80687     207.5857    202.95325    196.49395    191.80984    188.46671    190.08265    183.66662    182.92616    171.39002    178.16855    176.34253    175.36018    166.50219    164.10442    162.52109    167.97358    165.01183    168.10336     178.8198    195.88564    208.73876    194.48598    172.52864    159.70985    153.72355    154.27857    153.16996    143.82887    153.77646    151.41701    149.86767    146.32016    143.59896    148.78839    143.20623    144.13015    146.82682    138.80018    137.48458    138.50494    139.71181     138.6129    139.66941    138.84295    135.12745    135.37412    135.44276    140.71227    141.66884    145.37411    169.69997    178.88271    167.00092     161.0714     142.9504    134.59317    132.35992    128.87062    127.90037    125.79923     128.7554    130.70399    130.02991    143.12479    155.58592    162.43026     161.9482     151.3679    148.51537     148.1981    147.51015    139.28788    139.93815    148.14028    147.21391    146.65891    139.98535    130.69286    127.56392    124.58595    119.69655    123.56608    126.58392    130.09149    136.79398    140.58312    144.52593    145.07312    147.24751    144.79447    141.50837    146.94472    144.29225    139.37807     136.2628     127.5269    127.35414     121.6541    122.74655    120.01847    120.72109    122.14994    120.58344    124.67928    119.02968    118.98986     119.2986    121.55126    120.41215    126.26702    124.25667    127.35274    122.37899    122.99594    119.54369    114.22653    119.07526    123.56401    131.32661    134.99469    136.21203    128.38308    124.07827    118.74252    120.66702    118.14071    117.96932    118.13841    119.39421    126.71738    140.86549    150.03926    157.41079    159.57524    155.39281    147.62239    132.29705    126.93136    124.67989     121.9872    121.48122    121.63952    120.21216    119.33851    117.98172     115.9268    117.20055     113.3505    116.42461    115.26566    116.86732    119.51581    119.36919    120.81426    119.38487    117.79515    122.09438    123.07931    123.55347    124.72459     126.0232     125.9509    124.22801     124.0906    127.98661    124.59783    123.53137    123.08877    121.11856    125.52229    124.90744    125.09332    127.55443    128.70283    122.29085    121.38185    121.08623     120.3829    119.46048    118.17861    117.10754    112.77535    119.39532    118.11789    119.86316    123.77203    123.59278    121.66224    124.12961    125.39203    126.23264    128.56169    129.53024    126.48571    125.50638    127.27508    121.19655    124.21724    123.00744    122.79272    125.10544    125.46684    125.72837    130.54106    124.00945    126.13523    124.50144    124.68012    123.39454     123.2826    124.73848    118.72594    123.13144    122.37675     125.2366    123.52428    122.41304    122.88639    127.80601    127.42965    126.68798    128.11685    131.60377    130.53708    134.06424    132.42306     129.9052    130.89324    135.45987    136.83101    138.86115    138.79736    140.37922    139.70526    136.99522    137.61994    135.80132    135.46421    133.48821    129.22805    131.50799    132.07067    131.18556    131.72854    132.39819    130.09079    133.72583    139.12775    132.77556    133.58346    134.67448    135.67619    132.67185    140.60502    141.37569    141.81848    143.45124    147.12946    142.14267    149.12511    148.22583    147.43953    141.10521    139.32081    138.91036    135.20454    132.51336    137.19874    133.15997    133.04409    131.72192    133.30919    135.51239    136.54576     134.9165    135.40873    135.31244    133.63691     134.2474    134.28838    131.47156    133.75953    132.97737    131.25036      129.159    133.63258    127.59723     129.3761    127.78301    131.06931    130.98655    133.18267    130.85853    132.17014    131.59232    127.36949    131.24399    128.12928    131.39219    129.11372    130.75112    124.76709     131.9437    128.77704    129.47328    134.13767    137.19346    132.22385    133.37506    130.40364    132.98854    132.22072    133.43696    130.01313    128.34121    133.65729     130.2725    134.97201    131.87291    128.33613    132.48139    136.35799    132.41875    135.82717    134.54684    135.54813    135.40955    141.96983    139.17215    145.73248    142.10166    145.78857    145.90182    143.71101    144.71224    140.23047    144.05507    145.95927    146.76882     146.8087    147.27771    143.14391    147.54388    153.65063    155.97587    153.80372    151.65031     153.5227    153.92204    159.55439    159.33613      156.718    159.92734    160.12847    163.22182    162.31345    161.83965    164.80739    165.13909    162.68529    158.89944    157.46606    170.90954    166.44739    161.47241    163.17948    157.58643    168.85722    162.79937    170.71207    172.14496     174.3337    170.61987    176.84281    168.98965    181.94378    172.34305    174.26303    171.47998    175.74985    165.32356    179.20362    173.85548    182.83954    180.31397    179.20285    185.51198    180.71199    184.18112    177.67306    182.45282    183.45592     186.0208    183.63908     192.4198    185.15525    192.54475    192.95332    195.07701    189.52092    194.13869    199.42571    197.92852    194.51451    192.70073    195.78648    195.61187    209.91469    208.40609    209.57142     213.4746    214.76562     207.3209    208.35895    225.68132    216.25287    212.23718    218.62957    211.83052    215.69985    217.44072     228.6408    226.07464    215.66533    220.06656    212.93052    231.51646    225.88105    224.54008    225.10427    223.66819    224.33978    229.00588    241.89158    244.52542    236.66781    239.89876    248.35674    245.10618    246.30707    252.39961     236.7813    253.13663    251.99746    244.21024    263.37965    260.65088    267.57103    254.92655    258.95334    267.48997    269.20575    286.37254    309.89537    280.32193    275.46194     286.4829    304.18461    316.75883    362.54512    333.84893    334.39746    336.84474    309.05404    342.32779    383.07139    387.75259    361.96599     407.5364    375.24167    377.00167    469.53678    494.83695    447.54793    444.98354    476.37374    448.26301     535.6185    712.36082    672.82631     677.4265    624.33271    645.29702     706.6508          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        4.50    260.38791    238.50116    238.04514    229.86156      227.203    226.41613    213.26213    206.26603    202.35515    192.26816    192.60939    182.25644    186.97138    185.24364    181.63281    175.17958    176.15952    180.25977    168.83685     165.8639     165.4013     162.8425    165.95244    167.31561    167.76271    179.27808    199.93102    205.78864    194.20954    171.86623    161.23292    151.24706     153.6797    152.46729    145.91632    152.27362    147.91457    149.87469    150.56012    146.64537    146.48338    142.76481    142.29406    143.13151    136.01398    142.14956    140.41022    140.37155    138.12038    142.12632    132.63348     137.0317    136.78286    134.54455    134.79322    144.58355    148.26296    171.85826    180.82308    171.15338    155.89598    143.00265    137.17372    133.85109    133.49673    130.40281    128.08579    128.61348    130.80139    136.38312    144.18587    156.84738    160.11228    160.52995    153.68761    146.62065     152.0071    147.02287    143.01042    137.63052    142.48624    146.24366    143.38638    139.08047    135.82703    125.17972    123.22094    125.42852     121.5539    125.25711    132.69418    133.87192    142.05939    145.84985    146.74112    148.18642     145.9207    142.28748    143.69005    142.63886     138.9741    132.89134    129.39007    128.13259    121.14743    119.94564    121.42293     121.7542      122.093    123.16496    122.54704    121.27898    116.97475    117.61698    119.62393    122.47166    126.72566    127.63893    128.69912    126.34005    121.27431    121.12977    119.72668     118.8003    122.41995    130.95678    133.15781    133.39445    132.04605     125.1239    119.93384    117.52501    120.07835    118.66073    117.85952    121.20706     126.0906    139.57113    147.62141    159.26673    159.26293    154.23646    147.37299    134.57531    131.46221    126.83441    123.30755    119.44621    122.97304    117.99094     120.1121    119.14985     116.1288    114.74063    117.62512    114.36097    115.96936    119.84886    120.11601    120.52308    118.83658    119.19477    121.73835    123.62818    120.89682    123.35727     126.0355    124.42419     128.0712    127.26176    129.25749    132.54914    130.63827    124.77031    126.40035    122.84074    125.40417    126.33995    126.75605    128.05859    127.43833    125.23397    122.37237    120.85598    117.90892    119.73518    119.05002      117.992    116.83963    118.73402    121.46794    118.69443    120.94907     120.8216    121.17054    123.87179    126.65579     127.9119    131.66323    130.60695     129.4828    127.06874    124.84405    125.26905     120.3576    124.01227    124.09259    123.88158    127.66021    128.38185     126.0494    124.58233    122.51052    126.43714    122.95233    121.10642    122.10511    120.66276    120.79228    123.57784    118.35771    123.65844     127.9471    122.70639    128.64205    124.95931    126.74741    127.58053    128.06896    129.53192    130.84417    132.11506    129.68683    129.63575    133.83194    132.85326    135.15292    138.56463     140.4402    136.88019    133.75617    135.09477    135.09061    134.26801     134.6713    133.56795    129.25755    129.72781    130.22277    131.66013    133.30495    133.90116    132.95951     134.6668    132.93284    130.78907    134.54298    137.57401    131.93255    135.47564    136.86603    141.95999    143.46537    144.48569    146.60129    146.04322    148.56951     146.7498     146.1749    141.38656    140.06377    135.27638    134.00115    134.14073    135.05952    132.13035    134.17002    134.99289    134.73079    136.12438    136.71522    137.66857    138.35024    137.97536     136.5379     136.1837    137.32023    133.66477     131.3636    133.64794    134.16854    132.28155    129.97349    133.11388    130.09787    128.37036    128.82339    127.61001    126.86243    128.01308    131.21203    132.13017    128.04418    126.85519    128.38117    130.45873    126.57723    133.45268    130.58426    130.23919    128.75298    132.38934    132.52357    131.34064    134.61965    137.02268    131.55836    131.11514     135.4133    133.04042    134.07689    133.43675    132.65682    131.73692    133.94957    131.56773    135.12131    130.02457    132.23487    135.77036    136.52007    136.53068    139.67999    137.03332    138.95789    142.10584    139.18518    142.20059     143.3486    145.53816    145.67358    145.63461     145.8811    146.54119    148.53258    146.92794    142.42032    146.69561    143.84792    151.65744    146.51348    155.44733    159.46784    157.68272    153.32426     157.9706    161.69808    158.29729    154.76589    158.22273    157.66015     157.1077    157.29261    161.25409    158.79145    162.79198    162.20261     162.1339    160.97759    170.48362    170.43217    165.61401    165.67186    165.02005    164.37299    167.73724    168.54322    176.83022    171.68193    169.21552     167.4943     170.6344    179.61264    182.44461     179.9331    173.15603    179.44245    175.54043    177.36465    172.64647    185.78596    179.22235    168.06496    175.88251    180.70978    176.25627    191.74377    186.46176    176.33943     186.8832    184.85882    196.64696     191.4472    196.84704    186.88857    189.16026    191.71513    196.76113     194.1348     198.3102    195.41887    204.76191    196.11398    194.15198    209.96806    205.24197    208.91952    215.70121    216.23365    216.76617    212.94693    213.87458    216.46745    223.72566    215.52943    215.11027    218.55705    220.19375    220.35653    223.44433     217.6332    211.51712    223.06406    223.47626    235.48144    226.03551     231.9842    229.27402    228.63166    229.15713    241.44112    234.83631    241.98786    235.82924    226.28192    234.57286    249.54436    247.00163    246.53062    252.38811    242.87317    248.47737    262.64808    260.92292    259.95577     265.0623    266.81343    263.40672    272.10003    302.65883    285.28587    282.31086    288.26143    293.44733    292.12234    288.68406    346.59186    326.11677    328.34831    308.55869    339.41991    351.97293    376.00997    371.30074    384.74959    393.13089    392.91031    383.90362    427.01235    469.24565    488.01386    487.02973    457.52605    437.28521     542.1273    714.31243    588.48319    587.29562    588.91193    647.54173    437.58465          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        5.00    262.20639    235.96652    236.42865    230.96786    226.58053    216.95301    211.14992     204.9125    201.86725    192.01528    193.83002    187.20304    182.01804    183.80446    179.44754    176.87802    177.64187    172.27923      177.172    168.00252    169.24328    166.05432    169.03911    166.71782    169.29253    180.32416    205.63781    203.77823    195.48352    173.18937    160.46286    150.24003    151.74657     152.3261    147.33981    151.14261    154.61049    148.84636    147.42488    145.73883    146.11448    142.19069    139.90687    140.64502    137.68737    141.83149    136.76236    136.47844    139.78079    140.68518    136.91995    138.12879    137.28189    133.44991    135.04774    137.82375    148.77829    167.77152    180.65577    172.93779    158.27661    146.45466    138.54355    133.29708     136.4762    127.34452    129.90902    126.18115      130.374    133.31101    144.21931    159.81063    163.51314    158.27317    150.78385    147.04892    147.93016    146.24658    137.74177    139.37761    143.53776    146.50326    145.49341    140.82485    131.17958    124.05711    124.52254    122.85592    124.83569    125.46907    133.74834    135.47884    140.67575    147.18883    151.03587    149.24798    147.46981    143.18145    142.29433    139.03723    134.54628    131.46722    126.67263    125.61607    123.45724    119.32398    116.76936    117.65627    120.71005    120.36395    120.20852    119.65286    119.64585    118.73023    116.98916     117.0975    123.78464    126.66858    126.58839    126.34976    124.14667    116.04652     119.2923    122.16505    121.41659    131.46575     133.2144    132.16748     130.5934     125.9768    116.40263    118.79804     116.1762    116.67074    117.12576    120.78996    127.83066     137.9539    149.84557    156.88369    160.85533    157.75222    146.36656    132.60541    131.68466    127.19093    123.36129     122.7732    122.35567    118.08759     122.1338    119.64456    117.25796    117.78415    115.27399    117.59629    119.15571    119.86305     119.0658    120.89583    123.20288    121.24901    115.90704    121.59907    122.76389    126.46747    124.72122    125.43735    127.16768    124.76528    128.12476    128.98088     129.8737    126.00025    124.41669    122.96453    125.19866     127.8857    128.55041    128.69981    123.47959    123.85451    119.26095    119.73881    122.87217    118.14859    115.71432    117.90531    118.92701    115.22549    120.41189    122.91859    120.73168    120.71904    122.99135    127.72479    127.03007    126.75913    130.44374    132.52523    126.72487    126.25126    126.69997    124.44699    124.18509    123.51904    122.01426    122.78292    125.21234    127.62103    129.23465    124.92065    125.37215    127.84874    118.85481    125.44811    121.89171    120.41126    124.64548    120.99908    118.29459    126.00608    126.68747    127.25229    126.44404    127.99408    128.99958    129.04746    131.42186    129.90467    130.64478    130.02406    129.73554    127.98574    129.28178    133.55364    131.74316    136.68719    132.16524    133.84581    131.94033    134.32106    137.51977    132.44148    133.79185    130.87339    128.13972    131.93405    133.91323    129.07672     133.4131    134.25316    131.84029    131.09305    134.46813    132.26177    134.95866    135.77223     137.7643    132.49612    141.87959    140.56993    141.78167    144.65313    147.41039    144.48439    146.28549     147.9281    145.63687     143.8474    136.10658    139.03362    137.50582    131.95613     139.2797    133.94662    134.60505    135.75095    132.43047    140.88888    139.79957    135.28994    137.89726     137.0669    138.22294    130.42777    137.05179     136.5821    135.40773    133.34015    133.06986    132.47428     131.9179    132.80187    130.47465    131.80519    133.19689    130.50578    129.38813    128.27801    132.29762    130.42859    131.16843     131.4764    128.38899    129.25197    131.01432    132.16172    129.11643    133.26438    130.46516    134.18888    133.25706    134.04398    135.22218    135.89468    132.54675    132.20725    132.27241    134.68483    133.18375    125.09408    130.98512    128.32852    128.78814    132.94854    130.29062    129.54677    132.88892    133.64223    132.91674    136.97829      137.516    136.89472    143.18068    141.03523    142.27026    147.90243     143.6621      144.509    142.44622    143.57976    144.48203    146.25653    139.78108    146.67286    149.35333     144.9032    150.29434    145.34435    150.68439    157.16166    154.75926    151.04995    156.59575    152.73206     156.3834    157.85951    160.29279    162.01205    159.86928    162.45831    159.64424    164.15721    154.67274    164.11095    165.58169    161.48137     162.4628     164.3005    163.20238    169.44533    160.84519     164.8223    161.59152    166.34853    169.37781    168.46653    168.39618    163.81841    167.30803    168.12834        182.3    166.72263    168.20493    168.68492    174.86849    170.80246    169.77726    169.36473    174.31099    175.10285    184.89172    177.31918    172.59158    189.28378    181.69241    179.05727     187.1411    179.20555    178.93781    184.09837    189.26844    200.55986    189.40981    193.96141    189.51803    192.17194    200.26744    201.60971    203.40293    196.83974    203.35186    198.87765    213.96837    212.09572    206.63579    212.91033    212.57281    213.89404    206.89058    220.03816     209.7785    219.08185    210.29616    217.40066    207.26765    217.91945    223.69991    212.42034    210.64153    214.29138     208.1929    232.47346    223.22382    224.38102    222.40697    229.37127    222.93383    236.04024    234.78355    227.47209    234.74282      253.214    238.96678    229.55638     249.6121    243.10288    261.97483     244.8881    243.10266    241.23932    245.03405    272.00053    266.39395     253.7374    273.49206    258.18083    275.92611     280.3505    303.44766    290.01073    309.72871    282.13955    269.93858    294.95192     342.8681    326.42703    321.10646    303.83184    327.16829    333.90258    416.26679    348.03198    330.00808    381.83443    355.48592    369.06968    424.32963    489.86958    427.98345    462.83313    475.67015    430.01039    588.49335    673.75925    689.30183    608.64482    724.87843    553.97773    564.88659          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        5.50    264.01315    236.17333     236.1679    231.13033    225.75633    207.42237     210.2627    206.10923    201.69243    198.06382    195.00116    191.74526    183.20755    185.41242    181.45551    175.32884    180.22211    169.38434    175.53979    169.10017    172.31931    171.40914    172.35899    165.20457    168.95803    184.36818    195.08723    207.38914    195.78075    174.30006    160.71833    159.74467    154.48826    151.45666    147.01689    146.38234    148.53393    148.87314    147.06682    142.01302    142.66159    141.03644    137.58344    140.81426    140.17582    142.40106    133.88131    137.72238    140.83117    138.61119    138.01522    136.61053    141.36988    135.98003    135.27143    136.96685     149.4029    166.53281    181.93567    174.74776    160.01286    143.45661    135.56031    134.18312    132.82482    131.21824    128.06306    132.04156    132.15093    135.66397    143.53915    158.66577    162.46564    157.74046    148.65242    146.10486    149.22381     152.2925    147.16472    146.99314    149.76206    142.78415    147.47738    141.68642    133.97629    126.16954    125.97855    125.92066      122.278    126.69247    132.14603     131.6299    143.16506    146.66254    151.81138    156.07337    150.41098    140.54259    140.27921    133.59461     134.2435    133.77861    127.02644    128.01051    121.67673    121.12073    118.94171    119.85783      119.588    122.22505    122.74708      118.724    119.06882    118.39008    114.70776    122.96378    124.03471    125.28862    127.45196    125.49999    121.23115    120.02411    117.67319    120.13776    124.45435    130.74092    138.36436    135.72005    132.11868    128.39527    119.23794    114.77326    114.21211    117.44509    119.36917    120.10936    124.97618    136.26727    150.45247     157.4534    160.00343    156.44551    146.12343    137.42107    130.10178    126.90719     123.0518    123.05184    123.08301    117.64854    121.38104    120.38663    117.02618    117.86528    117.65422    115.09862    116.88836    119.84639     119.9504    119.73893    121.44271    122.76859    118.79162    119.84444     121.2922     125.9191    127.83258    130.64447    128.08037     121.6852    124.72688    127.68034    126.20302    126.27423    126.88565    126.60818    126.64505    129.04375    130.10075    127.51753    126.25058    119.60976    121.48052    119.54682    119.18223    117.71012    115.35546    118.59488    113.80859    124.70479    121.96435    120.16282    124.28133    124.71709    119.42357    123.41262    123.93576    128.21576    129.22072    130.03695    133.26937    128.36924    128.35154    130.55236    123.24302     121.0817    126.72673    123.41661    128.53759    126.39905    127.56768    127.78608    125.89276    121.04286     122.7316    121.99097    124.40155    123.81276    122.77162     121.7785    118.93917    127.79275    127.15833    124.96991    129.09386    126.16186     127.1062    122.36295    132.42413    129.72492    129.30946    129.47045    130.14168    133.33969    130.34429    132.21931    132.09766    134.53227    134.32202    136.67151    133.02253    130.80061    134.81244    135.68209    126.24183     131.8214    130.18086    131.17348    130.74354     133.7895    134.64624    132.62727    131.38461    133.72325    133.59007    134.36267    137.73585    136.60904    138.86111    132.54336    142.12734    141.66684    141.07535    145.47749    145.71237     144.5091    146.30063    143.17099    141.51252    141.88988    138.72026    140.71779    134.61454    136.56354    136.37412     134.6947    130.07059    134.03514    136.36537    139.49072    135.09575    138.85271    136.24816    138.65873    136.46607    131.23073    134.99271    133.23042    128.96742    130.51685    133.35416    131.63275    134.47537    132.92438     132.5288    138.83166    130.19312    129.25666    128.69249    128.93795    126.98414    129.12858    126.34783     130.7562    127.86391    130.74997     130.0468    129.92757    131.63905    132.34113    127.74237    132.51962    133.78592    130.32345    131.04548    129.46895    134.41488    132.66823    131.96571    132.79671    130.86182    130.69118    132.09682    129.01958    130.51879    132.58928    126.47442    133.84458    138.69301    132.27031    132.48577    139.26149    137.63838    136.02325    138.72403    140.71076    141.69602    141.58912    138.57572    144.28441    144.90604    149.15328    144.14566    142.66259    147.99302    147.84519    149.45266    148.76311    149.73263    150.08985    153.19775    152.28028    152.37813    151.25759    151.93666    152.33194    155.88648    157.65429    159.74376    163.33446    156.96834    156.61547    161.16716    158.70291    160.31664    159.41041    165.12037    164.86757     161.4331    166.92164    162.40885    165.61586     161.5842    162.28498    161.30681    163.25104    168.18737    163.34084    165.45882    168.28552    169.01286    172.80807    169.08127    170.91866    174.31513    173.04823    176.17337    170.59979    171.40714    176.27102     174.3007    175.71323    181.87764    171.90629    182.77446    176.91033    184.84204    178.24396    185.36985    186.41993    188.51767    191.56656    189.25155    179.09894      196.672    196.26298       193.77    193.16517     196.0741    197.11367    196.96599    197.49235    200.63772    204.01705    209.47815    216.26864    207.89725    206.03759    212.14852    205.96227     204.2132      219.977    211.68018    214.50585    214.93536    212.81964    208.92151    212.46053    221.17461    215.30882    221.53528    208.45168    212.96537    230.12862    222.41556    231.50906    229.18411    232.23882    223.61298    231.46629    236.77391    233.43183    240.28605    232.92119     237.0324    229.60152    249.52593    240.53388    247.11518    248.94324    249.30469    243.87345    246.56268    255.88012     271.2566    254.00424    262.53121    251.25105    264.50173    284.06633    293.23094     272.9288     292.9671    298.32045    280.90049    275.72081    356.88674    339.61981    302.70795    330.11049    327.78624    348.68486      351.951    417.19969    378.14631    346.73303    406.63324    358.41279    423.98601     443.1286    465.58414    427.01664     465.8113    418.68543    536.76096    625.53965    525.22173    589.65397    578.58503    637.75462    591.40775          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        6.00    266.54212     239.5773    236.02854     231.4647    226.28199    208.30387    213.23122    208.50377    201.05931    201.98353    194.09639    193.38985    189.36346    187.52844    183.70825    177.03484    179.86656    174.82925    170.74605    172.09104    170.03201    162.69951    168.91415    162.67135    166.74776    182.60154    196.78214     207.4383    195.76038    175.42574    160.16922    158.47856     149.8796    148.70686     148.0497     144.7858    148.01219    149.76146    145.20382    144.56814    141.49115    140.35672    144.57062    141.87312     140.5658    139.17172    135.99014    141.72682    140.45518    137.92514    138.08431    139.67263    136.15656    138.07426    135.10846    141.12859    153.50394    170.60373    179.11664    174.87105    159.69411    142.84346    135.35607    134.54952    130.93758    130.19601    130.72574    131.22322    132.30089    135.34921    144.55055    157.84977    161.36012    160.03119    150.08074    147.30206    152.11617    155.44337    152.80221    152.20269    147.78022    149.86706    147.42619    142.87932    138.38801    127.11444    124.16936    125.84159    123.66016    128.25835    128.37365    132.36807    140.41182    145.97136    152.73082    153.83199    148.00441    141.01823     136.9177    135.31377    134.80508    131.68219    123.07281    124.83272    120.63463    123.84163    115.67011    121.78969    116.15327    121.68342    121.73973    120.50402    122.11423    115.80979    116.98991    120.23916    123.58329    124.92221    126.13381    123.30715    122.46553    118.80118     117.0359    124.36729    122.50515    132.58304    134.80007    134.32432    130.81908    126.61718    120.69295      117.162    116.03296    120.95795    121.54629    125.28332    125.57337    136.85957    150.11385    156.62604    163.21212    154.68921    148.98672    136.76054    127.57978    123.99775    121.45008     124.4297    120.64925    120.19699    119.78184    118.84414    119.91887    117.15994    117.47853    117.17854    116.21788    120.24745    118.05197     121.8375    120.57597    121.50721    118.70367    122.20451    123.23662    125.20115    124.05711    124.11264    128.00349    122.75866     125.6211    125.76695     127.4611    130.87052    129.44779    130.28426    129.77696    130.77993    129.42739    127.75282     122.1599    119.54088    118.46855    118.28127    118.00119    118.90543    120.32537    117.84719    115.55983     120.0577      121.428    121.96379    122.69855    124.06415    123.74011    123.61711     129.0847    128.66491    128.71655    129.51487    130.33652    125.40205    125.79244     128.8388    122.07227    122.68858    123.04362    124.83387    122.06402    127.25127    126.69916    126.30975    125.18726    122.33098    119.30822    124.04676     123.2081    121.41968    120.44055    122.16803    123.95091    130.22293    124.29366    124.97665    128.26977     128.2102    128.54632    130.03991    129.90869    131.68132    131.85166    130.12318    129.01547    132.35842    130.37538    132.26051    131.11501    136.42429     133.2245    132.90544    134.68733    133.09752    134.19142    134.44817    132.88689    133.46819    129.59968    130.52492    130.71899    132.13027    131.56098    129.09905     128.7048    132.22103    132.60051    136.54291    134.14532    135.82045    136.59211    135.29382    137.12137    145.09669    143.47399    146.85908    144.47487     147.5498    144.74468    151.18159    146.02126    143.13588    142.86184    138.38896      136.308    134.06511    133.37367    137.10389    140.24448    135.93672    134.61476    136.88561    136.87061     139.4048    138.06538    135.34163    136.01596     134.1463    136.99226    139.61174     130.9462     131.9745    133.09271    128.62475     131.2151    133.09787    132.64267    130.76298      131.632    130.70274    124.60149    128.94102    131.32453    133.15434    128.65206    131.05989    127.35862    138.61189    130.02134    129.51314    133.65733    126.91962    128.83138    130.39129    129.23923    131.80765    131.72597    135.91883    133.03903    127.16767    129.60162    131.92834    131.98505    132.17732    130.30606    130.65207    130.46067    132.72315    132.74561    130.38264    132.52165    131.82928    135.73462    141.10598    143.41952    133.01833    138.15729     137.7854     135.9991    142.57689    145.66878    141.83315    142.44098      143.625    143.97104    147.21136    148.93189    151.30284    151.57493    147.73788    147.58404    155.73525    156.80181    158.44661    153.24882    153.89936    156.33527    152.74268    157.62142    158.64246    159.28997    159.32817    159.75264    155.84677    158.75043    165.56003    163.49998    164.81733    158.02488    160.58111    159.88491    167.98991    166.28469    163.70962     161.5268    166.30812    161.33736    166.82062     172.7708    167.28856    171.26979    167.74918    164.72034      170.112    169.43551    171.51648    170.94348    178.31404     170.5664    169.10213    182.62542    180.77337    180.77315    183.95784     180.1879    174.65481    172.09702    183.62472    184.34226    192.00329    181.04378    187.79938    190.04224    192.55307    194.34761    190.05812    192.47833    186.87227    187.56406    204.05927    196.78199    200.30306    192.97219    199.39183    195.84758    194.86871    205.74856    207.05527    203.78474    207.31347    208.20805    198.43007    206.41376    211.71611    206.83606    223.91401    213.58512     213.2844    209.68548     216.0626    227.46626    215.20118    213.73932     217.6648    216.32645    232.80724    222.49535    220.23177    235.97212    216.89777     225.2435    217.86429    231.44357    235.92449    235.89466    238.51037    240.23626    225.28512    237.04212    242.41328    246.98658     246.7703    243.60701    239.17813    245.96969    266.62457      270.356    260.88988    264.79243     262.7506    274.15865    298.15657    277.63203    267.04899    297.78581    279.14037    306.61515     316.2755    354.46421    300.33487    358.20414    320.96051    338.51915    308.26892    376.30931    341.28694    365.21447    349.74931    354.75919    337.59757    447.59696    440.91569    475.11589    423.54376    461.90799    433.08358    660.02202    699.13094    531.76231    589.22137    572.73659    538.89066     641.3789          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        6.50    269.30927     243.1678    237.36277    230.43133    228.72634    211.33077    216.50224    210.62941    198.87782    202.17699    192.64131    194.29929    193.28437    186.68557    185.85206     178.5995    178.79922    178.23809    167.77046    176.47394    167.65294     162.0639    165.81619    158.08328    165.14494    180.50305     199.6415    205.27188    196.48227    175.16846    157.98591    151.73013    152.66713    153.98538    148.46342    147.17917    146.91576    146.63744    145.16848    144.03316    143.19718     140.4259    138.76217    141.97344    138.28594    138.50265    138.52047    137.01667    140.46627    140.87521    139.49571    139.23544    135.66876    137.51556    138.93109     140.1692    148.88635    168.57149    182.07128    176.25625    158.55042    146.19167    134.34849    132.81359    131.94994    131.46373    132.61388    131.76732    133.33924    136.12275    144.87903    154.47415    158.39622    160.58434    149.76963     150.6709    153.43941    153.99391    152.32202    152.74808    145.94693    147.41579    144.67351     142.6583    135.28946    128.61963    128.65296    125.45648    122.44239    126.93211    126.23498     133.9677    141.57846    146.03916    150.39889    149.49416    146.28287    141.65085    133.94579    131.77746    129.74717    131.25936    130.95744     127.3046    123.47992    121.56933    121.12522    117.95379    120.36381    120.66382    122.24783    118.38727    118.54069    118.90537    121.27718    119.49421    125.24482    126.32571    129.94002    126.51818     122.5803    119.32363    118.25173    122.16142     123.4364    131.87603    136.12513    135.51055    127.80383    125.27105    118.46401    115.12285    118.70518    119.98647     118.6309    124.09966    125.31722    135.58955    149.71816    158.54821    162.53981    159.64295    147.67879    137.21486    131.93641    127.24059    123.89025    122.94423     123.1381    121.22184    119.95651     118.3673    121.39999    116.90837    114.90133    118.71945    114.06466    118.47799    115.68001    120.78376    120.47762    119.41335     121.5763    125.07325    124.10733    122.91033    125.97528    127.42705    124.29846    124.36272    127.01994    128.63787    129.04055    128.14311    133.49161    130.17573    129.64167    128.62551    128.66148    125.12074    124.51855    123.55451    118.22764    122.09999    117.72053    116.15511     120.3779     118.7542    116.99243     119.1695    120.73592    120.64737    122.18646    124.85864    125.46271    122.75197    127.04787    127.13028    130.03074    130.42259    131.99864    128.84169    127.07645    124.30057    125.93071    124.46737    126.17604    126.51538    126.81863    127.21925    125.32199      125.051    125.65914    124.84506    124.30618     126.4273    120.63383    122.32053    122.16773    123.41455    126.09243    123.09158    126.96745    126.63108    128.34574    131.27344    130.38081    129.40801    133.05911    132.03252    131.75803    132.72349    131.82055    134.26128    133.05288    131.76729    133.21573    135.04741    132.02008    135.11409    132.41767     135.1519     132.9781    133.05423    133.02712     134.1606    129.20851    130.93457    128.61687    135.19551    129.38486    129.26525    131.08314    129.63546    135.44477    135.38639    135.61573    137.92474    139.31698    133.13309    137.08976    144.12588    144.25148    146.78428    146.94138     141.6208    144.96455    147.76561     147.3636    146.27234      142.856    138.37672    133.45842    135.16047    137.69885    133.71128    133.83419    134.40287    135.13319    134.93837    135.09126    134.57546    138.05132    136.74228    137.10736    133.59114    137.35484    135.51168    132.08154     135.1331    136.58598    130.42952    131.62087    133.59518    130.63758    132.10972    129.96968     130.4495    130.86286    128.06692     132.4306     132.4482     130.9807    134.75089    130.91547    130.32713    132.98504    130.61341     127.4557     129.2225    126.88973     130.1908    135.53584    134.42948    135.65746    132.60033    134.14316    128.21526    131.86061     135.3596    132.36776     128.2551    132.70239    126.48901    134.77825    132.94202    134.56836    132.45592    131.14268     134.6272    136.91041    137.09487    135.97091     134.4998    139.65449    135.64377    141.43963    140.15266     144.8372    144.23196    138.94843    141.22837    144.81981    147.22689    145.34254    148.78319    144.42339    145.91572    148.71293    151.44624    154.67941    156.00466    155.47463    151.25658    152.15454    155.17787    155.93172    155.88952    160.13391    155.62476    161.65425    156.36399    158.68697    164.41546    157.69849    165.83361    159.63464    160.29781    155.03411    163.57835    162.22101    168.94079    160.78181    167.07999    160.65356    167.41177    170.37906     168.7632    172.43391    168.70411    166.60254    169.67878     168.6492    168.45759    178.84386    174.05872    173.83683    173.05925    174.00853    181.37882    175.59525    175.02829    177.26783    173.73193    175.88922     175.5371    185.81741    189.29956    188.00196    182.96253    180.76065    184.30035    197.51309    190.48863    189.50768    184.80895    194.61121    190.16416    197.83581    205.91231    194.30172    195.81812    195.53843    198.14686    210.48194    203.57994    206.94852    208.62563    211.35111    203.30672    201.21079    219.94021    210.12199     206.9289    210.50364    212.06556    212.68408    226.63109    215.38218    221.22022    212.94079    216.80169    214.51899    225.64124     224.0173    221.02393    228.58943    223.25511    226.31938    229.97436    245.97168    225.96818    240.46913    232.36075    236.44293    228.37828    255.16886    249.00435    245.78435    240.83504    253.13002    246.26646    251.13886    258.89212    265.83938    258.76829     267.9994    268.69787     271.6752    289.99248    292.96509     282.3067    299.63209    291.04832    295.18199    301.53872    320.72965     326.3188    335.68101    341.07989     275.4945    344.24945    390.90393    378.46644    411.31562     353.7422    373.21466    374.87303    422.92189    502.58307    427.25176    416.09064    419.96978    486.14069     606.6912    531.47073    600.50191    636.58712     545.4889    580.67107          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        7.00    271.41564    242.89334    240.88767    229.28101    230.52024    213.84465    218.84226     211.8639    197.16316    199.60287    193.02099    196.29669    196.08668    184.66725    182.26009    177.94869    176.50177    179.98951    169.83483    174.67769    169.10541    164.77353    165.25769    161.58581    168.79734    179.11746    199.69107    205.84384    190.87466    174.70484    155.93012    150.78515    157.32701    149.48532     147.6202    147.57842    148.95704    145.43183      145.635    142.82805    147.76297    138.62585    144.79027     141.0204    140.26698    139.02528    139.36808    138.18069    136.88514    140.62314    142.18577    135.84256    136.91576    135.00814    136.78594    140.80198    149.04701    169.35491    181.55011    177.89657    161.15416    144.97704    137.72252    134.16419    133.38095    132.51953    131.05378    132.81673    133.80935    134.06069    147.90573    160.61285    164.86447    161.78483    157.70416    152.58106    154.67731    156.50502    146.29083    144.98413     149.4974    147.03543    147.09478    153.98872    142.68576    128.91093    126.77432    127.10063    123.72813     128.0655    130.52544    137.51632    139.44053     143.1197    145.88782    143.99656    143.26416    139.81032    134.61431    133.88364    130.11226     130.5007    127.90804    126.53122    120.26725     117.9793    117.82734    118.03654    122.26187     119.3342    124.22491    118.29972    120.84242    120.43913    117.84507    121.39988      125.857    126.58622     130.6318    125.44593    126.04814    119.11063     120.1905    122.22747    125.48915    134.20673    133.23371     132.8691    128.98216    128.90376    121.21224    117.91907    117.71889    120.88014     119.6067    122.63201    128.21556     137.7942    149.23431    158.67314      162.934    157.32341    147.06522    137.23373    130.50596    127.29632    123.02705    122.06277    125.32356    119.70075    118.74844    121.28515    113.03202    119.01038    117.01673    118.41145    116.48687    120.69611     118.3122    120.41587    120.46437    123.01914     121.6514    120.87141    125.94883    123.13245    127.24304    126.87301    127.18539    121.36126    125.23574     125.5091    125.72603    124.14715    124.02255    127.37634    123.72889    126.75582    127.89467     126.7084    124.94701    123.84065    125.16107    121.65762    118.74183    119.96139    121.47212    121.62461    117.39013    118.20339    116.47807    123.55602    123.27135    125.80023    121.77596    126.48391    126.40384    130.69251    129.75856    128.42896    129.98386    127.73258    127.10867     124.9485    124.75505    124.40131    126.20682    123.72107    121.38883    125.40107    129.19042    129.10542    127.75258     128.4313    124.68472    126.17221    122.94859    117.93711    123.48926    120.68837    126.01494    123.82289    123.97825    123.99725    124.49284    129.62041    132.12826    127.68991    131.98617    130.13611    131.71675    131.79319    129.64835    130.79615    129.83078    131.75394    132.26114    132.99235    134.61565    134.01579    130.22859    135.49652    135.35779    135.99622    135.49764    136.09978    135.76162     128.7472     132.7143    132.07707     130.9804     131.7104    129.44872    132.97787    135.84457    135.69221    138.68203    136.25806    136.19575    134.94884     138.3388    138.56561    143.39481    146.01109    143.30476    146.24263    143.94517    149.90842    146.98462    144.28035     141.5324    141.59167    136.49224    139.40582    137.31733    133.52409    136.91106    137.41347    135.22542    134.71947    137.50201    136.20442    134.91195    136.39064    137.43052     133.2022    136.91169    135.05965    134.11291    131.32151    135.62624    130.80283    129.35792    132.57003    131.84388    133.00312    130.81937    130.85334    125.55521    130.79319    131.71032    128.13942    131.49761    130.69082    129.19819    129.66446     130.7145    129.20058      128.362    134.02253    129.40257    132.80704    132.28683    132.61408    133.57638    136.51593    135.61324    134.57057    129.97686     134.2355    133.98032    128.46561    128.91518    132.89696    132.20809    131.03163    131.89958     130.3865    132.46846    132.87556     134.9343    136.28892    133.93772    139.55779    133.87584    139.24568     144.7777    144.03886    146.86529    144.81796    149.65249    144.30737    141.21397    140.91731    149.03208     146.8964    144.18927    144.46907    145.55134    147.46068    152.20819    154.37749    154.77757     154.8005    151.94701    156.64704    151.33331    161.84269    159.55105    159.18313    157.43066    156.92168    156.93723    166.23718    160.07802    157.99806    161.04519    156.48148    154.81402    167.03591    164.08678    166.29994    171.37995    170.11602    163.72518    162.94033    167.55467    167.84553    166.90323    168.64027    165.97969    168.50266     172.5258    179.24642    170.24465    168.94783    173.08847    174.81757    178.63406    175.26506    175.17277    175.31393    175.62084     175.0432    172.25619     175.3265    178.14588    173.83931    186.13403    180.44491    185.52989    190.83486    189.70168    188.83277    192.50816    186.42869    187.91646    192.37283    194.70696    211.32622    195.44252    197.81315    197.07782    200.58543    203.89542    204.06962     200.1092    196.34371    207.99698    206.51239    214.06877    218.37707    217.61741     216.6457    215.09497    216.19934    210.48637    217.89432    223.91572    222.86073    212.53284    206.51928    220.06816    216.93328       225.64    222.36594    219.78091    234.71764    222.87615    231.82695    238.58556     229.5137    234.27646    224.38591    232.70009    230.43151    256.04222    244.68321    240.24635    240.63753    244.66659    241.34372    258.53446    260.74769    261.51524    273.70161    257.71519    271.15613    264.59124    302.84318    311.09843    290.53258    286.41068    293.03262    294.83088    347.97801     321.5127    338.60578    336.67095    316.44511    295.02225    296.61107    380.94683    351.27912    389.17424     365.6949    373.39998    402.16798    429.09992    518.40988    451.69044    442.37597    472.69408    470.64543    557.69151    644.93944    649.24494    517.21149    677.56998    685.99422          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        7.50    271.47858    241.54634    243.96531    228.60591    229.94218     215.7079    220.54354    210.47406     199.6994    195.46257    193.21312    196.27087    189.93829    181.51386    177.92457    177.93797    174.86552    180.90567    174.29616    173.34731     168.5639    164.65985    165.08295    162.85421    168.30514    176.43496    197.32655    207.90877    194.67018    177.04258    158.32504    154.23398    153.82242    146.98964    148.79323    148.40276    146.67716    146.43314    148.17964    145.38322    147.07764    145.00254    144.47989     143.4227    139.06508    141.18515    141.64129    137.42008    138.34664    144.23246    140.55436    134.48526    138.50553     132.4077    136.05784    142.76923    151.28383    174.06997    180.86401    177.29652    160.22916    143.93572    141.66866    133.36225    130.95432    131.50411    130.17002     132.2661    131.56294    133.76653    146.05917    156.30495    162.17701    155.06362    150.48284     146.6434     151.4755    154.38033     148.8652    145.23632    142.95509    149.26739    152.80163    151.68096    146.42861    131.38916    126.60725    126.45747    123.16135    123.77995    128.22846    136.41778    139.67071    143.02942    142.83512    138.15566    140.66874    136.09819    129.93733    131.77845    131.11678    128.41149    128.18045    126.53859    121.83221    120.72593    121.07983    117.34435    118.41089    122.35731    123.00673    115.68329    117.12977    119.03532    120.46059    124.26563    125.80141    126.06937    125.21251    127.66142    123.78604    120.10064     120.0027    123.88055    122.73799     134.4358    130.26792    136.02713    135.79815    129.10215    121.91615    117.95188    118.34037    119.20924    118.77432    125.27323    126.94488    139.51414     152.4293    158.09759    161.39756     157.1656     144.4463    135.53158    126.31288    124.40126    123.12875    125.32149    123.60365    125.61896    119.16364    117.63957    120.08024    119.20235    117.03106    119.70094    112.32955    120.88207    116.81125    121.57658    121.56269    122.23669     121.3963    117.77569    122.67041    126.61253    129.31181    125.84512    126.01359    124.69352    120.39552    125.21387    120.78672    121.02226    126.27313    125.10026    123.88007    127.48368    125.19053    126.83814    124.58466    124.02223    125.03691    121.87117    122.77434    121.87945     117.1485     120.3876    118.02661    120.24955    120.50873     122.3669    119.09742    126.34511    126.59663    121.41972     124.5237    131.01552    131.62773    131.75841    128.83053    125.41409    123.87108    124.53918    127.20188    120.93608    123.50214    127.04802    128.64036    124.92178    127.51501    125.72634    126.52803    125.05917    128.73477    127.52794    122.52257    123.55717    124.95348    126.75626    120.15933     120.4109    122.80807    127.40093    127.45972    129.57152    128.49729    130.57097    130.14394    132.71589    129.25382    131.67945    129.82772    129.10683    127.52327     134.4294     131.8897    133.46484    134.67902    137.75125    133.18717    137.86263    134.61246    134.22476    135.44457    137.91638     135.3523    132.23518    133.62164    130.31889    133.03857    135.06852     129.9602    128.86041    131.35834    133.76096    135.14559    134.99922    135.90943    135.93074    135.44478    138.77191    142.35371    144.60984    143.32934    144.89262    145.28055    147.15541    149.27772    145.30523    140.24836    139.76458    136.01905    137.58487    135.75418    134.09319    137.63782    131.73815    137.50092    135.35374    136.14665    138.44244    135.42021    137.57266    141.19674    133.90281    132.43012    136.70421    134.41664    132.12224    132.19231    134.57151    132.48199    131.71808    134.52044    130.06255    132.34586    128.64958    130.72843    133.12441    130.59057    135.33117    132.67729    129.19283    128.42124    126.87634    130.06364    132.65416    130.95436    130.08907    131.12642    130.19777    134.29712    134.36896    132.75001    133.51447    131.55143    130.51158    129.74692    131.90776    130.39135    130.85108    132.02039    132.23082    131.98675    134.02821    132.53271    129.39068    135.12618    130.31865    135.54704      137.169    135.41896     138.0047    139.69337    136.72469    140.03508    144.16364    141.76508    140.27768    142.95056    149.04237     145.9041    143.24103    146.63992    145.57853    145.25352    139.13683    145.10719    144.70048    150.69461    153.95869    154.89567    156.85498    153.94879    153.18837    156.52786    160.57563    160.90052    158.94216     157.2918    158.70387    159.95382    163.76986    157.04287    159.97859    156.29709    162.34647    158.53819    154.10567     164.6587    167.13335    158.59124    160.63114    166.24832    162.37785     167.5906    165.85254    169.77449    167.19574    169.05154    166.57748    166.50464      173.718    170.30821    176.45029    171.35213    176.33592    174.43621    176.88869    175.73599    180.68721    176.17513    177.00015    177.01896    184.81811    177.94335    178.82746    177.79845    183.90095    181.72614    192.96147    190.82969    194.43348     189.6829    198.77409    183.82588    195.35375    193.93462    202.87127    197.21456    204.56703    199.15013    201.98012    207.98459    205.53789    204.66218    210.72231    210.16123    203.39302    202.79145    214.28213    205.19009    214.96942    205.61554    214.88949    208.86417    221.62208    214.12529    227.17818    218.83902    215.76753     217.8058    229.92218    225.88855    224.91796    230.34556    235.53786    225.72185     224.4875    241.67196    237.53755    235.30356    241.04639    233.91708    233.34905    256.84855    257.74105    255.70784    241.78212    233.89184    238.91755     253.3961    269.94331     275.1519    264.72076    277.63937    263.82871     265.4796    290.25566    277.40177    294.85328    290.61132    272.94532    309.77211    320.39431     320.4242    327.61839     298.5416    320.89905    332.61172    318.96451    394.00797    397.27523    366.01198    355.97069    377.16859     372.8232    424.73037     452.5956     407.9597    466.28464    445.18346    408.05967     541.8375    553.32868    523.49425    552.80369     665.8138    435.65119          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        8.00     271.8166    240.55058    243.18555    228.39066    229.86933    218.45141    217.48016    208.16073    202.20358    195.27381    191.37468    193.88454     185.3494    182.71757    181.96229    177.82062    174.42058    175.66338    173.56451    171.48712    167.66457    165.26857    163.28958     162.3254    168.20714    181.63646    198.26503    208.02087    195.71378    175.45625    161.26017    155.69102     152.3007    151.45948    148.72951    150.67779    150.37113    151.08838    145.65334    146.88433    146.23841    143.70595    147.96841    144.20362    139.21703    141.82578    139.13985    141.50335     140.7677    144.12451    139.67162    140.07346    141.99348    137.06716    131.44238    141.32245    152.27754    168.74525    181.00395    175.11979    160.19289    144.13923    142.75756    131.03741    132.89383    129.82139       129.81    130.88197    133.18054    134.62531    145.35679     156.3978    164.68896    158.31269    156.69936     148.9899    149.19912    154.02444    149.82257    145.46106    145.26905    146.41333    155.68159    154.13276    145.41449    134.99055    124.58081    127.50948    122.99723    123.64538    129.60367    135.50098    136.89832    139.46472    138.94643    136.70208    132.31668    135.11147    129.81413    131.44353    130.57818    125.73118    129.78621    129.39615    119.52355    118.52661    118.57819    118.94011    118.19567    120.17357    121.58329    118.12894    117.20829    118.89142    119.12735    121.80685    128.65785    130.29884     125.2944    125.35633    121.22679    124.60251    125.76891    121.86005    124.41731     133.4018    138.56882    135.38341    127.13976    124.93869    119.71979    119.98016     120.1905    116.07817    120.13239    123.34387    128.78081    139.90511    154.37108    159.54855    158.97949    154.96963     151.2589    137.20057    128.32591    125.93324    127.35959    124.74995    123.18814    122.78644    121.28714    119.35164    119.69099    117.70185    119.00007    120.30567    116.16616    118.29393    121.03151    121.40721      121.359    122.10772    121.90262    119.72063    123.47016     125.0652    124.44713    128.48964    126.41963     124.5763    122.09001     121.9933    122.89998    120.50392    123.60041    127.09139    120.59752    126.09605    123.52322    124.14316    124.11033    122.31155    123.32152    119.13228    119.83887    119.99929    117.47273    118.46537    119.25789    117.79331    121.54003    119.95562    125.31946    122.88899    125.44448    125.02566    124.73764    126.70275    127.81412    129.88428    130.03128    128.58451    123.58723    126.23396    125.24353    123.40091    126.97846    125.80687    124.64838    125.65211    129.21334     127.9011    128.66321    126.27061    124.80776    123.52109    124.26538    124.15573    123.29718    122.41768    125.17147     123.8328    125.08867    124.88176      127.563    127.91867     127.5914    129.83606    130.57764    132.36319    130.94994    132.22352    130.09606    131.29071    127.95547    131.08336    132.49693    137.87099    137.01937    135.79278     135.3343    135.68551    134.90042    135.74301    136.32385     134.7297    135.30339    128.69926    130.99212    129.56644    133.74913    134.66583    135.25244    130.41685    136.91245    136.53143    134.09547    137.02887    135.92401    135.39234    140.70538    137.55142    144.36856    146.79098    148.24647    147.19414     144.3252    149.94208    145.03894     146.4852    140.81569    139.26928     135.9559    134.14303    130.28649    133.97605     134.5516    131.32514    131.58235    131.81989     133.8781    136.59985    139.02776    134.83959    138.09228    137.14745    136.60007    132.39556    135.64894    130.68655    134.13041    134.20007    128.47892    131.14691    133.60558    128.65542    129.94899    133.87902    127.29546    133.44589      133.094    126.97325    130.39784    135.29621    127.98415    127.61737    125.80183    132.22589    128.84926    130.88412    129.77051    132.20647    133.45541    127.78051    130.37828    133.89146    132.82349    134.22578    132.47945    135.44083    134.05641      130.146    128.03329    127.93112    130.29019    131.13397     131.3347    131.75927    131.26081    133.09906    136.17347    132.84443    141.68812    138.32285    142.14761    139.36891    143.20856    141.27835    143.00117    143.89865    143.00086     143.9826    142.83899     140.0602    145.73602    142.34456    140.90199    146.82451    140.50268    144.91358    151.82572    150.60939    153.61283    148.70531    154.88476    155.69364    152.05786    163.04165    158.26429    161.17181    160.14493    163.84732    157.63356    164.11458    161.05673    157.78449    167.24941    161.84109    162.72279    155.14871    166.28937      157.936    166.02148    161.30568    160.79643    159.86095    164.45016    168.12861    169.14617    168.31102    167.27695    169.83868    167.63963    173.09379    171.40238    172.23835    166.52144    170.12971    180.78405     173.1295     177.5521    179.31947    184.30416    177.09712    172.80667    184.53628    180.99581    184.83191    179.12867      181.687    186.00523    189.23493    187.70522    187.40087    187.48776    186.09619    191.75568    192.74341    199.89652    199.55314    199.19098    192.23998    198.25073    204.74336    205.60092    202.95066    202.93656    200.26142    201.71922    207.94448    216.52657    213.99459    208.44104    209.26243    212.26929     215.0043    205.34297    224.06207     223.1103    218.90603    216.75189    214.00942    210.38014    210.96896    221.84169    214.88316     215.9543    228.65465    236.80691    227.85304    228.22758    243.81652    241.97022    232.54177       240.53    239.59214     255.1621    232.86016    243.64206    243.66518    242.74263     241.1083    257.13029    257.79539     259.9593    259.04114    258.96743    247.27091    274.24483    306.94114    311.23481    287.47616    297.79271    318.38952     334.3763    315.97217    330.22254    306.35222       311.47    345.03447     331.1873    325.78905    380.72492    359.87252    395.78063     392.5269    391.56997     405.1545    472.27698    475.93138    416.80893    503.98499     474.1973    486.31525    500.19973    521.39763    646.86684    583.79484    549.87213    598.01241          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        8.50    271.37992    241.90039     241.8504    227.88256    228.88606    224.00294    215.17467    206.32718    203.57458    198.20166     189.6783     192.3926      184.981     185.1821    183.56478    177.32303    174.73409    169.65338    171.96596    170.23901    168.12791    166.80346    167.88895    164.39222    169.50858    183.33604    200.19311    204.89686    194.00807    174.69789    156.99882    157.24559    152.81647    149.45922     149.3068    148.69829    152.65985    149.13589    147.72692     144.7232    148.84539     141.9078    147.43889    143.67485    140.16538    143.39368    141.31432    138.46611    138.20778    140.58548    137.25121     137.6353    139.87486    135.68031    133.96678    143.32465    152.79133    169.19225    180.91639    173.42313    157.21286    143.08752    137.11799    134.47857    136.43664    133.63189    128.14963    131.24061    133.68901    137.49251    143.92205     159.1273    166.86538    162.43233    154.58273    147.82784    151.87222    151.33927    154.49626     150.2498     152.4359    156.06385    156.68052    156.94651    145.87029    132.73715    125.41609    124.96848    121.89758    125.62741     129.2376    135.66099    138.05487    135.79423    130.54747    135.18201    128.20031    126.95621     129.1624    132.33139    128.17715     132.5302    129.90729    126.38793    124.66134    120.64654    118.19553    121.53093    119.23561    120.38667    119.37612    119.04584    121.19824    119.74641    118.61435    118.93216    125.32395    131.84297    127.94299    129.24793    124.43909    119.80392    120.34882     121.7539    124.47889     130.1934     136.1884    135.32855    132.79283    125.49096    124.48875    117.80114    117.79168    120.96298    119.71631    122.14773     131.7215    138.46882    149.09365    162.95821    159.52021    156.33927    146.98066    137.41263    131.45218    125.66127     127.0074    124.56217    121.32887    122.42518    121.01153    119.15872    121.16952    116.39426    116.41656    115.97964    117.76693     117.6933    119.81338    123.32831    121.89862    122.09118    118.78177    119.31048    121.97815    124.56095    125.19607    129.23873    127.52009    121.13857    119.79466    125.83466    126.16516    123.08878    125.56377    120.90935    120.06607    126.39157    124.92894    126.88577    125.41938    125.39402    119.95447    117.06366    118.99159    121.30149    118.04254    120.03657    123.18131    121.96367    121.13704    121.73852    123.09779    122.81029    124.05435    125.54376    126.20735    128.99961    130.29785    129.55359    133.18161    128.49036    125.98484    128.06382    125.36686    123.79394    124.96344    126.08976     125.6797    128.70385    124.99755    130.50079    122.19628    125.80575    124.92673    122.13466    120.84941    126.72874    122.66033    122.81697    121.68365    120.20309    128.44961     127.8206    127.42485    126.98522    128.70338    127.90562    131.54326    132.75802    133.27505    131.96877    129.38521    129.86686     130.0755    132.96477    132.96409    134.64503    133.55373    139.42924    136.60519    134.61463    133.25092    132.87379    133.09125    134.91397    132.46258    128.92736    132.50042    130.50167    130.72659    132.68237    133.20446    133.44798    133.97857    135.11927    135.57051    138.72977     137.1968    138.53187    139.27679    144.08009    143.24849    148.18516    146.08597      145.842    144.61974    146.77866    143.94295    141.93184    140.64666    139.51948    135.27138    133.01257    130.75074    133.06534    134.50865    134.95634    135.90616    135.77673    135.44757    136.33753     135.4062    136.66475    137.81856     135.6414    136.12372    133.44164    136.80324    132.65623    134.96963    133.51538    128.26236    130.59884    129.83761    133.82432    132.69747    131.46597    131.87933    132.86063    128.06273    130.49478     131.0626    132.15409    129.04028    127.95879    127.92832    130.78171    127.29904    129.40447    125.25389    131.85447    131.19667    132.58951    129.38543    135.32108    129.63812    131.32099    133.18597     137.4791    129.20188     126.8856    128.31102     128.2848    132.86757    131.68807    136.63717    134.29562    132.66123    132.61565    132.78835    136.30938    138.67273    140.35838    135.07191    139.08778    138.94067    141.07898    142.14201    142.84972    143.23707     143.9149    146.94273    147.52064    145.23539    146.46977    145.11257    146.51608    144.09593     148.9869    147.22332    150.51827    153.76058    149.98179    154.54559    151.16433    155.21578    155.01983    156.36135    161.28881    158.64802    155.51801    160.01263    164.49385    162.08573    159.59166    157.80681    161.23391    163.77171    156.32982    163.45377    167.42087    161.97944    162.21081    167.20453    159.73007     165.8807    166.35436    170.96665    166.67037    163.57459    172.04993    169.39677    175.38242     171.9623    172.24875    169.29741    169.99085    167.63018     176.8422    175.99082    175.57749    174.64401    174.83588    176.99096     180.7664    179.54452    184.28189    180.00637    183.27848    183.38422    188.79353    188.91995    190.72215    189.61538     195.7631    191.22795    188.65483    191.31131    194.10676    197.00309    198.92258    198.57225    199.40301    205.05866    214.33015    198.32891    207.29492    198.72346    196.44474     205.9664    214.12057    213.84765    216.66214    215.33588      215.079     211.5378    213.46633    220.27499    215.22686    219.99847    210.07209    213.78745    222.44577    233.97968    228.26237    225.35401    221.06274     220.9506    227.36233    238.38213     237.4303     235.7285    222.83833    229.95056    227.82826    262.73546    249.73676     243.6917    232.41812    237.26374    243.66977    258.59578    265.18725    261.46475    273.78371    273.64107    261.85447    267.25405    296.37197     294.2297    276.40273    316.31399    308.61565    300.92895    352.93629    310.35368    316.52096    283.85257    299.86275    322.93767    341.24789    390.43749    405.33213    364.36684    391.81906    343.45497    358.29284    461.46797    429.29449    427.60235    440.28533     461.7445    420.17031    562.13333    683.38756    664.22091    541.50157    660.92321    599.67463          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        9.00    267.87114    242.30628    240.78038      228.424     227.2563    229.19225    214.89987    203.45516    204.17387    198.12163    194.99441    191.49734    183.65549    186.28759     181.0238    177.54144    172.19547    173.42218    170.42108    170.11211     168.4005    169.83457    169.80829    167.34703    169.23724    181.71933    201.54702    199.56327    192.22595    175.05238    155.92796    156.43706    154.58808    144.18674    154.80083    145.66882    150.39527    149.36319    145.88373    147.33501    141.38744    145.69942    144.06344    145.68946    138.84996    142.07766    139.76661    139.65263    139.57694    140.47102    141.71734    136.81096    140.11491    134.05846    136.80697    142.92342    153.04604    169.97162    176.99193    176.74553    159.77544    138.46046    134.28988    135.43144    133.04747    134.90046    129.52971     130.0364    133.44322    136.50118    148.19084    162.50062    167.31506    159.74207    152.15612    150.98625      150.925    154.18239    150.30482    144.01349    150.91653    158.93324    161.13036    157.50658    148.26143    134.96634    125.33129    127.57114    127.33046    127.91049     133.7294    135.09999    136.04405    137.30449    130.21136    126.05705    127.17453    126.56471    127.64057    130.08968     126.4015    125.53916    129.18052    126.76184     121.7827    118.00374    119.82589    118.71572    118.06602    119.74098    120.15847    117.92786    115.52906    119.27758    121.75745    121.71039    124.98235    127.24432    124.78852    127.09248    122.74929    116.52455    122.90792    121.85447    128.42615    133.89097     133.3451    136.30848    134.30453    123.86105    122.61751    118.12506    117.88813    121.85939    116.98702     123.9353    130.46048     136.8223    155.70953    158.31119     162.2205    154.58738       146.62    137.98605    128.96196    131.04489    127.12115    126.55692    119.24206    122.27869    123.46978    120.53664    118.15634    119.48558    117.01554    115.41987    117.83028     117.8386    122.62835    118.94417    123.34724     119.4063    118.08368    119.59485    123.56859    125.84142    127.14439     125.4068    127.78327     126.4512    120.26753    123.55696    128.84814    124.55119    125.55649    128.23444    125.14906    126.24434    127.73651    122.27444    125.57902    124.79733    121.65704    124.18796    123.20423    120.09932    120.02355    118.64883    117.54044    120.02866    119.44104    120.14965    123.49367    123.92954    125.11645     124.8236    127.63737    130.15558    129.21674    132.17964    130.66745    131.23229    128.58013    127.08955     126.5612    124.64845    124.34028    126.18514    127.09195    123.59832    127.02757    128.20066    125.17607    125.05225    125.06461    118.90552    122.60543    121.21222    124.29513    120.82082    124.25344     120.0916    124.72056     124.4581    126.77184    127.53288    129.50212    130.62921     126.6862     129.1816    133.17348    131.88927    132.22467    133.80077    128.40209    131.28673    131.90093      136.316    135.25195    130.58432    134.92451     133.0373    138.40032    134.17562    135.66228     135.6353    130.99124    131.00004    131.49317    133.69552    133.66707    135.16281    137.00825    132.13357    134.83636    138.31092    139.17374     133.8759    133.51181    135.88117    140.24385     140.2083    148.96665    147.08414    144.52785    147.91897     144.5756    145.71909      146.425    140.95175    141.58574    136.76603    136.41358    132.68202    134.99473    128.75218    131.64022    130.94431    133.24983    132.70467     139.0252    139.44268    136.82772     136.9839    139.62987    134.53221    133.22526     135.2742    133.15968    133.94412    131.24519    130.90066    131.32307    131.59854    127.24724    131.02166    127.94227    128.80868    128.09759    128.01922    131.90283     131.9111    130.24253    126.63345    131.17227    132.33478    130.38768    130.47703    128.53842    131.33639    131.29026    129.41652    133.77374    133.86971    132.89367    134.61396    135.41612    132.76844    128.11076    130.47946    132.13069    130.46766    130.57544     128.6367    129.09813    129.06682    132.87451    130.57231    133.91415    134.82118    133.04994      135.342    138.39492    139.14954      142.247    138.85003    141.70199    137.20886    141.87413    139.85721    141.97369    142.87692    145.25935    145.46342    150.84553    150.37002    150.74803    153.40297    150.40821    151.60968    149.39582    155.82078    153.02057    153.12426    148.77326    150.51079     150.8931    159.54959     158.7785    157.83692    158.91934    157.42974    158.39535    156.76855    163.24522    160.62524    161.61304    156.26469    160.99082    159.87822    160.15294    167.45941    155.74441    165.57599    154.80698    161.57319    166.21328    171.31126    168.16885    165.46941    167.75571    168.95758    175.13236    171.69515    173.39387    170.52901    172.62234    168.21008    167.45196    173.92931    172.09261    179.01874    167.31276    179.80972    173.25174    181.46554    179.50257    180.41566    184.31143    183.82007    181.59382    192.08041    190.31646    191.36204    186.70775     188.6554     188.2237    196.04737    194.63801    199.80806    197.40675    201.38668    200.01343    195.12146    204.07338    205.15629     204.5896    204.81382    204.81065    208.60698    202.66197    212.80516    209.07573    215.79094    218.14073    205.00578    202.00855    229.94713    213.32003    212.28151    210.12764    213.77134    211.60924    232.87944    228.01396    214.89028    224.16797    222.58213    223.71705    226.60326    234.38204    237.38521    223.48947    222.00047    231.64361    227.96048    240.68173    246.95106    233.23617    254.96513    231.86165    240.50212    259.39235    262.63055    270.43428    264.07721    259.37263     271.4984    251.92491    282.16094     342.3704    318.67847    304.67496    268.68299    274.23384    315.45358    310.83045    312.80651    342.28192    335.70498    329.29029      339.823    324.27009    362.48042    383.18256    393.10386    387.01826    361.96727    469.67425    465.88796    467.34457    433.99643    430.39313    418.44561     546.3649    539.55353    566.22458    501.54363    547.61564    554.13437          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
        9.50    264.94289    242.53178    237.63332    229.14711    225.40926    224.74688    214.45013    201.92527      200.383    195.49645    200.06235    193.41766    182.54306    187.75527     178.7263    178.36588    169.71564    175.37574    170.93266    172.40945    167.16371    165.63075    170.02487    167.54944    168.48391    176.27425    201.15092    203.50047    195.70896    180.15418    162.31281    155.65443    155.80727    154.16569    149.89706    148.43689    148.85019    146.09255    148.66459    150.00842    146.90089    146.58792    145.77053    143.53052    141.18532    140.67115    140.07049    139.45688    140.83983     137.3995    138.90108    137.46094    134.20615    134.51682    134.45888    146.20229    150.17633    172.29823    179.58049    180.51371    155.53203    142.88096    141.32707    135.97501    135.59636    136.07119    128.26356    133.00086    135.44508    134.18948    144.01038    166.01134    167.24057    159.30083     148.6028    150.69037    152.83212    155.14246    155.63994    153.22962    153.05087    161.92045    164.84978    158.22117    145.40303    133.85978    128.62484    126.14318     125.2404    125.96928    132.21632     136.1453    134.07096    132.44045    130.78641    126.83276    125.76569    125.28281    126.77024    125.05958    129.59711    128.84874    128.60025    125.32121     121.0473    116.64845    119.82284     119.0371    119.72692    118.97699    119.86758    119.46284     119.2485    118.05368    118.94766    121.14032    127.73273    129.79849    128.68286    127.09087    124.14778    119.85412     116.3433    120.79156    127.78024    126.02859     136.0115     133.5361      131.854    125.29995     123.2909    119.01133    115.83416    119.73787    117.65184    122.31991    130.50869    142.44704     151.3394    159.72181    161.31572    157.23165    148.09792    139.83719    127.02462    127.56746    127.02405    124.78863    123.82375    124.33003    118.63298    120.41074    116.90861    118.49298    118.38039    116.85239    119.78395    117.77024    121.48027    122.62221    122.94361     121.2752    119.64156    119.23172     124.4031    126.34899    125.94314    130.90372    125.94603     125.7936    122.25287    125.64375    127.58437    128.28387    128.44589    126.95302    123.76569    123.80797    124.52201    123.66038    124.51661    120.48413    124.40562    119.53926    120.00747    121.77026    123.23007    120.41615    121.31532    122.24322    122.15573    125.72126     123.0241    125.03792    122.46438     126.9896    126.98477    129.04663    133.92901    133.97371    132.08043    130.58121     126.5702    125.51934    126.02696    125.07836    128.96969    126.27706    126.27017    127.72804    125.61534    126.90551    123.90673    125.57915    124.79642    119.42881    121.42035     122.8954    121.62359    119.72149    123.89277    122.51434    123.56925    122.93131     126.8541    127.51314    129.09436    127.97465    128.87088    134.07753    131.10848    134.15864    132.04871    133.60534    129.47318    129.06816    134.91954    135.46974    136.02151    134.42627    135.09553    130.68111    135.77731    133.41047     134.0971     133.9673    134.28071    130.69434    132.93958    132.85058    138.21383    137.55955    135.08463    136.29764    134.76608     136.2032    136.98048    135.47866    134.87107     134.2531    139.02785    142.20475    144.84427    143.70398    144.50461    145.81355    147.31481    144.54021    145.82267    143.89124      143.184    138.19856    137.53962    135.12997    133.47446     133.0801    134.27866    133.52813    134.14272    135.80046    136.56363    135.43867     135.3923    136.21529    134.19789    137.85052    134.72317    135.89694    136.39285    134.61077    136.32756    134.10128    130.02686    130.15035    129.54817    128.24341    129.08145    131.58792    130.67269     128.1119    133.73426    130.85823    129.79342    131.53023     131.6991    129.30096    132.00127    127.05593    131.39177    128.86327    131.15588    129.93078    132.37959    130.91966    133.54547    131.73659    131.79034    131.58633    129.41689    132.87813     134.1724    130.91638    129.64095    129.97407    128.02962    130.84039    129.06104    130.12535    131.16886    128.77167    136.63772    133.32016    137.88235    140.42308    139.17492     141.8363    140.43108    138.81801    143.42458    144.90009    143.20265     147.4176    148.32963    143.26519    151.78083    146.46911    146.82075    144.76414    144.13594      145.348    152.61226    152.67981    151.56137    156.94401    151.97242    152.35414    148.74864    158.50606    160.86779     156.4289    157.16986    159.32964    159.70448     158.3065    161.06118    162.76385     159.3078    160.74243    156.36149    155.80509    162.40754    160.31828    164.50161    160.45841    162.51254    164.40305    170.06404    166.32613    165.80667     162.7788    170.03974    164.20527    166.81945     169.2362    169.06531    169.73024    169.34541     169.7287    172.63312    179.45779    180.38275    181.52747     176.2003    175.44417    187.23404     184.3842    185.78468    183.50788    182.36933    183.20997    185.89348    187.49225    191.63717    188.36013    183.31992    188.64192    194.46364    192.12595    194.84863    198.66307    193.46865    193.82478    195.83623    188.49685     202.8648    208.87625    209.57041    202.22783     212.0422    205.44004    201.39738    208.20764    214.07954    210.86443    211.55363    219.75238    209.42964    213.67876    210.58699    216.54973    214.87567    216.77777    221.12775     211.6762    226.80536    220.58603    224.24581    226.11057    217.36144    220.80545    235.71843    236.47634    227.23725    229.67302     237.4473     233.7344    248.74434    255.69848    244.22743    238.31006    245.82077    245.36994    248.53409    266.42946    273.87235     246.4837    235.66135    265.62247    279.67651    284.12334    274.10128    299.21069    301.96451    294.42321    277.24083     349.6557    325.03524    312.71641    294.11839    330.70817    328.85372    352.18513    386.72566    374.28281    321.51421    404.50055    381.28402    374.41212    497.82216    478.17261    475.31346     459.8925    401.60859    481.91264    575.95969    634.07508    478.93783    361.03043    570.22609    588.86072          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       10.00    264.06672     244.6536    234.24282    229.61362     222.5053     220.9031    215.64822    204.05968    196.51449    196.15058    198.42481    195.64863    182.36769    185.42607    176.76966    176.42318    169.94448    177.03522    175.91376    174.26821    164.44851    163.35092    169.01366    166.13689    166.80015    185.17139    199.33147    205.01111    202.67895    183.62698    164.47947    157.69812    160.46143    154.53935    152.23365    150.24598    148.31847    147.97286    153.19291    148.52642    142.62407    144.16113    144.18909    142.88163    139.89436    142.43737    143.34212    138.45492    140.69441    140.28152    138.55033    137.06088    137.60616    135.20278    137.44492      138.311    153.63655    169.71752    182.80969    176.78599    158.35588    147.32063    137.22596    133.92696    137.99434    133.44848    134.41651    132.31353    132.27235    138.84636    146.36972    160.74433    167.48773    159.78554    152.52076    149.42805     151.5767    156.00202    159.93254     157.4064    158.00918    159.84444    165.94279    159.32501    145.25287    136.16844    128.51963    126.14954    125.21684     125.0279    130.64023    136.44061    133.93876     130.8607    127.92206    121.81636    125.83798    127.47089    126.96531     122.8195    127.49023    130.68605     126.0067    126.99195    126.57878    120.34199    117.64674    120.12867    118.16566    119.94643    122.82417    121.69693    118.79069    119.39126     118.1402    121.88602    125.15118    127.89442     129.2364    126.61582    125.40098    119.35781    118.81117    121.82504    126.45274    132.84188     136.7928    135.42406    131.72612    124.54918    123.39676    120.44198    121.57226    119.09371    119.87428    122.97345    128.94437    140.11861    148.31036    159.23487     159.5429    154.74839    143.07106    138.66404    126.24943     122.5473    124.13107    123.96818    120.87573      122.078    119.30478    117.86892    118.71309    116.38292    116.84508    119.63934    117.43327    117.06316    120.29899    121.44022    125.43601    122.22804     121.4718    123.44317    121.78627    123.67404    123.42338    127.09221    125.23466    126.45849    118.59604    130.45181    129.40394     128.5419    130.39719    126.99716    125.01748    125.25134    127.09066    126.65571    123.48315    123.96919    119.60719    115.68808    120.78233    127.04299    123.64965    124.84682       126.34    125.21937    123.78265    123.80746    125.22859    124.26879    128.23701    131.68531    121.91524    129.02966    131.49871    134.91697    132.56995    131.70454    126.06083    124.83773    127.40015    122.65575    124.37843    125.56007     129.1785    126.02215    129.12343    129.76198    125.66715     121.8415    122.42267    123.21017    125.18073    122.10335    121.44415    123.34238    120.66352    121.70264    121.90427    126.80175    125.94059     129.1572    129.31267    129.84625    130.29568    131.17367    133.04896     127.7634    133.06049    133.02847    136.18506    131.90106    131.16488    136.95932    135.93739    137.29447    139.89269     135.5757    135.68112    136.97099    134.69866    135.35942    134.80239    134.69182    130.22938    131.68511    134.52287    133.18116    135.82261    129.97523    135.42507    135.62116    134.89413    134.31081     135.9281    138.69014     136.1047    139.74219    141.93308    145.88884    146.01431    147.61151    147.55001    149.52915    150.47429    146.28129     143.6741    139.21574    138.06532    132.27457    133.41439    133.20177    133.75583    131.52836    136.32968     134.3231    136.18655    136.72593    135.56621    138.76333    138.48509    133.20609    134.01607    135.99878    136.03124    137.10127    134.58852     135.3538    131.57486    130.91284    131.22419    130.67196    133.34706    131.96707    129.35463    124.84863      132.174      134.213    130.12481     132.6306    130.84632    129.46251    131.51181    128.64292      133.273    130.23358    131.95175    131.99559    132.92377    132.34396    131.74941    133.13646    133.31527    135.77952    132.92098    132.54572     131.2163    132.37469    129.38626    130.25068    130.28498    127.03408    133.28238    133.39475    132.98111     134.3503    131.22953    134.10406    138.97064    138.17783    140.97657    140.67226    140.36336    138.71984    139.78786    144.40434    144.45314    145.75794    145.50251    141.26309    144.77635    140.96499    147.66031    147.01023    147.90842    144.82179    142.10164    152.10166    152.10184    154.54073    148.78759    152.15651    154.90942     160.2887      158.178    154.45034    156.41077    162.87466    165.69379     159.2138    160.02966    165.17334    162.05432    157.07999    158.86031    157.31711    163.71812      162.229    162.38027    157.67609    158.20709     164.4839    168.43912    165.00347    171.86628    164.29994    162.35283    161.46134     171.0666    173.23019    169.00587    160.26529    175.58784    164.82227    170.90057    168.58185    181.22396    182.30661     168.0668    176.90444    176.62525    177.26023    188.06841    184.29813    180.90637    180.72494    185.26996    187.13774    196.88262     188.0016    189.37553    187.45351    191.22929    184.68169    197.73191    196.94496    190.73342    201.36438     194.9316    197.98039    207.72586    208.78447    213.78964    206.78164    206.26011    198.94001    199.82002    214.19768    205.19566    199.62024    208.51919    223.36536    206.10826    212.70735    223.89589    227.48386    208.98343    202.82893    214.13047    211.30272    228.94558    229.95145    218.27411    224.55995    216.63576    232.26588    240.70453    234.47662    235.29525    236.55903    230.95875    234.13526    229.07858    248.15978    254.11527    248.99436    248.52973    240.68914    264.52614    247.34129    278.07133     251.7182     259.3895    265.06403    262.24196    275.88994    281.77108    296.80483    282.14262    306.12976    271.37799    345.25334    326.87326    322.44001    320.82935    335.46858    314.64952    351.70073    354.12811    392.68789     358.8339    374.18828    356.36649    353.92098    454.41488    451.23116    487.75976    475.17521    395.05764    422.85322    470.05319    410.82393    680.79215    594.31833     718.3382    624.17872          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       10.50     265.6138    247.16728    232.11883    229.97335    219.30392    222.21867    219.90976    206.13096    196.55349    199.10525    196.50568    193.18862    182.25938    186.07245    180.98416    173.59831    170.49045    176.99565    183.33163    177.22743    165.04171    162.41311     168.5859    164.62477    166.71862    184.14568     202.3987    205.12399    194.56259    173.63982    163.43819    156.05849    162.84544    152.30742    152.64505    146.46299    148.29392    148.84669    148.58142    148.76266    141.34923    143.09167     142.1484    145.37552    140.19407    142.18498    142.57118    137.90173    144.33004    143.15093    141.23639      137.264    135.11458    137.27414    141.12709     142.0614    153.26023    173.31127    179.55463    177.33269    153.04857    148.16325    139.01866    138.08054    135.02515    136.43269    133.52926    131.48361    133.98837    137.96789    144.47678     159.8522     165.0702    164.36469    153.22138    152.49656    153.54097      164.839    170.11375    166.58206     165.4389    162.63992    161.88283    154.33363    145.42697    136.83704     128.1916    124.62314      126.705    129.34231    132.85861    137.02579    133.12067    128.88186    126.42131    124.24376     124.1133    126.26112    127.99152    123.48545    129.01046    129.57671    128.37329    126.61883    124.22001    118.82488    120.31381    118.39659     119.6183      119.865     118.5295    119.42326    116.85598    120.25261    119.63373    120.72254    125.80255    130.54542    131.00446    128.49784    125.42461    120.57957     120.2984    121.45153    128.04709    129.06649    135.10485    133.76884    132.79256    127.44859    125.04743    124.84612    120.50776    119.00333    121.84009     124.3672    130.22015    139.56278    148.21266     159.7468    162.82102    155.78263    145.39854    135.41281    129.66614    123.62323    126.57697    123.03817    121.24102    121.42552    119.87029    117.93709     119.8957    118.80159    115.44786    116.64905    116.99417    117.07822    123.50768    119.22167    119.72379    120.16103    121.80739    122.98968    119.22739    126.85889    124.55415    127.13164    126.94149    126.44791    124.86501    127.17532    130.17282    127.96174    128.39168    127.54706    119.61382    124.24868    126.59934    126.12068    125.41399    123.65594    123.47562    118.75488    120.66483    122.69563    125.43503     133.3041    131.44983    130.19695    124.39776    123.40324    123.60723    124.35022     125.1368    127.83324    123.93598    129.01513    133.43136    132.05078    131.41712    128.87476     127.5323    123.75658    127.06972    125.65852     124.5687     127.0213    125.06627    126.01886    127.61471    126.45431    127.38243    123.39526    123.14846    121.52566     120.9017    124.65717    120.14768     123.1917    124.35993    126.62838    123.27842    125.53814    124.99933    128.09474    126.98648    127.77471    128.61488    128.18186      134.106    132.36526    134.41307    130.23044    134.02969    133.99604    138.60554    136.05683    137.90791    134.87473    136.65015    138.25561     138.1421    138.09153    134.54832    136.66985    133.73069    131.82179    126.17846    131.35528    134.30862    134.89019    137.68674     132.4758    132.83672    136.05023    133.35323    133.81749    135.50526     139.5231    137.48927    136.62349     145.4627    149.46196    147.19045    146.45994    147.18301    143.58745    147.52833    144.86683    141.69288    138.73057    135.22608    132.20563    132.53311    132.87652    135.85781    134.68364     138.9586    135.67152    137.23581    138.10824    137.49221    137.51318    139.23925     135.8938    133.85859     138.8827     134.0484    137.33959     132.2897    134.24745    133.37848     133.9353    131.58684    128.94668    128.88483    130.65204    129.52497    128.31253    134.04603    130.29285    128.62408    130.10686    130.07289    130.12933    130.02003    129.93594    130.82496    132.57851    133.61096    131.31688    128.08192    132.28545    132.45228    136.31085    131.75413    132.56291    131.34984    136.80949    130.33888     131.7011    133.05073    126.00303    132.52878    133.28372    133.35213    132.58337    130.40638    134.87414    132.88909    138.38664    135.06037     138.9813    138.28126    137.81637    135.72124    143.54256    144.35865    144.98717    147.53456    144.99993    143.14928    143.35968    145.68733    147.64833    145.05478    147.57922    143.37515    143.13401    142.25452    148.92175    145.17525    150.99691    152.60312    152.09382    155.25561    161.11657     155.4742    155.25091    159.89866    154.39096    156.22182    158.66643    158.67126    157.04589    161.02472    156.69378    158.65842    157.55258      165.045    162.65847    157.42718    160.83988    160.67841    155.33056    162.73471    166.96922    164.14133     168.2244    169.03975    165.46186    167.77647    173.85687    169.78803    169.12064    165.43225    168.02348    169.80816     179.8043    175.89658    171.37195    170.55594    171.08495     170.1893    175.09658    170.41063    175.77693    183.33414    184.18159    182.67877    184.16587    185.80775    193.71029    187.66968    184.23735    184.00122    189.54139    193.47212    199.04348    196.49315    193.77524    196.29586    195.82746    207.25851    207.96457    216.70559    197.97726    209.72997    208.68501    202.75765    200.41106    218.59555    199.94413    215.60389    206.27086    208.33671    226.62619    222.12783     211.7153    207.98994    217.52644    213.92094    218.17504     221.8037    232.08081    218.47044    224.13438    225.43962    220.41971    231.84724     224.5306     239.3968     215.4288    227.96843    227.98491    249.43619    238.79355    248.10875    253.78186    256.82006    230.76126    270.22161    248.33946    263.59211    249.89839    268.42674    246.04893    256.16071    281.25843     285.0324    288.45675    282.01923    269.79937    285.80549    322.33259    331.65463    302.48686    307.16467     308.2459    332.74785      354.052    387.62387    370.01606    375.53402    374.21754    365.36251    375.37019     460.3455    474.51686    473.65811    466.12915    445.69989    487.32716    575.10505    672.93012    729.86325    586.32665    586.89694    619.02031          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       11.00     266.4978    248.00939    231.51387    230.85882    220.92116    225.55856    222.45592    208.47802    197.50502    197.95293    195.16709    190.33432    182.94958    192.88601    184.60591    175.78026    174.66659    177.07024    175.94641    178.71521     165.7734    161.77888    167.06634    166.08933    168.58318    181.98808    203.82843    208.52376     193.3383    171.44519    161.22621     154.5005    154.46228    152.39901    148.37001    146.98293    148.78064    147.87451    150.61132    149.30632    146.59675    143.57088    146.49203    139.98298    145.14373    143.20263    142.25446     138.7869    138.84922    140.38524     143.1006    137.56438    139.84213    135.67921    139.18613    141.39492    148.57288     168.7756    181.11931    176.11837    160.80382    141.47376    138.52988    137.95613    135.18925    131.55485    131.96306    134.87547    137.13868    138.02327    147.07693    159.59889    167.41499    161.77706    151.57318    145.55512    156.16637     166.9342    180.43395      181.808    173.96269    162.46896    155.06587    147.41744    145.96617    132.03502    131.52436    122.60993    122.94173    126.29448    131.00738    136.70524    134.22874    131.19587    128.50949    122.84875    123.02127    125.57381    125.24025    123.63357     129.4049    127.85658    128.57469    129.05252    124.43841     118.7177    117.84528    120.84035    120.76222    118.67172    118.54258     120.6259    117.32416    120.90828    122.60174    126.70283    124.39479    128.85709    125.55735    123.74502    123.96175    121.31751    120.36641    122.89962     126.3128    131.66626    139.34757    133.88083    132.62307    126.90763    121.25518    120.81358    118.35024    122.91325    122.04431    124.55796    130.27012    138.73351    151.08146    159.78413    160.30194    151.79741    148.96232    136.89952    129.63283    124.61737    125.05151    125.85178     124.1744    122.67003    121.57564    117.75881    120.54995    117.92338    115.83903    117.78392     119.3827    118.51993    120.02764    122.04321    122.42452     122.0834    121.84009    118.94931    116.88529    126.09441    128.22259    127.08157     124.2813    123.68107    126.25791    123.83528     125.3491     123.5298    124.04442    128.57761    123.90979    125.62525    130.73877    129.84853    124.15921    125.98988    120.78108    124.06017    120.00854    124.33488    125.98507    127.72142    126.42443    124.76579    117.14487    120.26919    121.07047    123.94344    120.26203    126.82971    122.79177    130.20322    127.71147    130.74142    131.51545    129.84677     126.0605    122.01481    126.87344    124.88941    126.60971    128.65548    130.35714    127.72309    130.29214    127.26907    126.67111    126.99046    127.84997    125.13563     124.3912    120.11835    120.76496    122.57154    121.52161     125.0357    119.53084    128.59329    127.80119    128.42423    126.97508    130.12389    131.87877    129.35599    129.27053    135.93695    133.08495    132.35417    134.85909    135.02031    135.24357    139.23666    138.87153    135.93201    138.76432    137.16812    132.86857    136.13868    135.67793    135.46823    135.41036    133.55127    131.49243     136.0879    135.80664    133.83861    134.54371    132.65753    135.79696    136.64564    137.81545     136.4207    136.05026    138.89217    134.09098    138.90106    142.70756    147.39055    146.90482    146.20553    148.36615    142.06282    147.15184    145.58902     141.6114    137.05479    136.85946    134.37021     132.5386    134.25974     131.6103    133.67566    135.72457    135.44909    132.71071    141.18216    137.76559    134.36276    135.01146    135.75712    134.42638    135.22719    129.49251    131.28354    132.96636    131.17053    130.51031    127.17507    130.15985    130.70713     132.9511    133.49215    134.97296    130.67735    129.91561    127.50255    127.87335    130.61862    134.04917    129.23264    127.72841    131.62394    128.45718    132.99183    129.74325    132.08387    131.83221    131.64234    131.60564    130.47694    133.78925    133.99139    125.73481     132.6587    131.01402    132.40046    129.03588    128.60086    127.91188    128.77592    133.76813     131.5188    128.75352    135.08446    133.67553    132.47164    137.14532    141.87383    139.17702    140.32907    139.52664    136.75859    148.24773    145.54634    147.40915    145.37643    146.79195     147.0315    147.94229     153.5663    145.10512    149.32419    144.43981    144.76243    145.02179    148.43094    147.52191    146.04586    152.69722     152.4037    151.80507     159.1965     158.1233     159.5985    157.96775    159.91937    159.62382    161.32823     158.5126    154.79355    158.97528    157.02861    158.30547    151.36436    163.75849    162.52917    164.24337    164.12665    164.10066    165.72851    167.20321    164.27531    167.16265    167.27262    168.04193    162.62024      168.705    174.83989    172.16046    172.18259    176.25358    166.51837    177.17348    174.44018    172.68573    173.47449    176.22965    173.55011    177.90039    181.87878    183.32973    181.66437    185.32651     180.4375    183.19981    182.69943    187.95711    194.83844    182.30555    187.37826    183.44584    190.80401     199.6445    199.59375    196.70501    196.93045     201.6862    204.22973    200.92705    213.20859    208.34799    202.96797     199.4247    197.53247    201.11442    209.23867    199.79541    204.41598    203.12517     215.9659    215.74194    224.10235    210.43784    221.04655    214.87194    222.90044    208.34176    233.42813    213.74719    215.01895    229.38652    218.47393    216.27098    209.63465       243.43    233.58716    236.44083    230.06934    221.35196    233.17478     240.8546    244.21766    244.63717    250.94483    246.94558    252.47372    249.86893    254.27142    264.70897    268.56163     254.2899    274.64584    262.41572    277.46735    284.82347    297.14435    282.71955    289.24244    288.84301    294.54792    313.96516    330.28726    307.53312    344.09576    320.93819    363.93422    380.09542    378.26488    362.76744    351.85201    360.96473    382.24997    472.34031    439.36319    384.49458    441.86028    434.73913    481.70004    632.04789    635.97175     664.1307    521.02671    587.88713    607.56778          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       11.50    265.41072     247.4601    231.18067    230.02402    222.93488     225.9405    217.08297    210.93008    197.68056    193.51712    193.59997    187.69762    185.11786    189.29485    184.69588    178.15508    176.18973    177.90521     173.3745    171.70597    165.17915     166.9468    165.95248     165.6804    169.23499    181.88906    203.22385    205.25834    195.87021    173.96276    156.19127     152.3356    154.08203    150.53058    145.06376    150.07091    153.77186    146.93567    148.11561    149.29253    146.97092    143.02877     144.1586    143.02318    147.12266    143.04834    142.34013     142.1632    138.86891     140.2665    138.53014    139.50069    140.67167    141.02665    136.17125    143.17145    150.88274    171.28229    182.91176    178.13369     156.9304    144.77145     140.0026    134.49864    135.30547    130.38197    132.68673    131.54949    134.39613    136.73717    144.02241    162.03222    167.27377    162.75012    154.12333    152.74177    165.68144    176.50506    188.16074    186.38255    175.29106    160.37764    152.97639    148.49318    139.33051    133.98692    128.44161    125.82663    125.90943    126.44792    132.94937     134.1495    133.18267    130.35918    124.99944    125.90472    121.18792     123.8869    125.50968    124.18452    123.73272    127.20787    127.45302    122.86637    119.92745    117.29827    118.19114    119.69619    117.51874    120.54345    119.86682    119.59287    116.51397    118.70961    123.68282    122.78257    127.59617    127.54168    130.57865    126.50786    125.07225    123.43569    120.87411    123.45227    127.02673    133.27608    134.18403    134.58747    129.40276    124.18442    120.97623    118.88032    117.71121     120.2534    119.49848     123.7942    129.33426    138.38265    151.36335    159.15794    162.46383    156.95687    155.21491    141.86459    135.36207    128.36784    123.23212     125.9195    124.07617    120.29252    120.91463    117.20143    116.34488    118.65427    119.51911    116.99239    119.17749    119.47909    119.42896    120.19385    123.91586    121.23408    121.93525    119.02666    122.40811    125.73712    126.35689    126.81121    129.19547    124.23053    125.39069    120.91169    123.58233    123.84421     126.0208    126.22216    127.44562    126.35736    127.50401    129.63773    126.85245    124.90825    125.60399    122.17487     116.8581    121.80734    119.63567    121.25948    117.49681    120.90522    118.74175    120.81061    123.48163    125.69379     127.5515    124.88224    127.15965    129.33597    131.03326    131.02069    133.24844    128.79472    127.48499    123.10253      124.413     126.8455    126.24905    123.95445    129.15332    125.68504    127.65607    127.08562    124.67819    123.02347    121.60752    121.59167    119.49981    123.60741     121.2608    123.30703    122.60624    123.50524    125.10528    123.25873    125.20314    126.46552    127.60949    129.56785    130.03772    129.81515    131.32081    131.12983    128.24466    131.32749     134.0592    134.38188    131.29718    136.46397    133.28212    139.80997    138.96743    141.20453    138.52208    140.58811    139.85437    139.06892    138.64677    139.30232    132.17582    137.10685    133.32698     136.9451    133.69837     133.3944    136.25602    135.10139    135.94023    136.09611    135.89343    141.12832     136.5122    135.83404    143.05794    148.59997    149.51247    147.85945    146.52496    145.67675    138.07761    142.23538    142.74342    142.69827    137.52757    131.27838    133.24219    135.82862    132.69291    134.87288    134.62207    137.28922    130.28945    136.55651    137.00506    136.53084    136.15271    134.90377    135.41286    132.33963    131.77302    134.94044    132.84127    132.72719    133.79121    130.99489    131.46856    129.81265    128.34052    129.98406    132.98241    126.83281      126.563    129.99638    130.32071    129.85504    127.27544     129.4282    127.21007    130.95756    130.22098    131.65119    132.53645    133.96279    131.40279    127.99533    132.08425    129.46329    130.71644    136.19202    130.67744    129.05732    129.65711    126.59288    132.32419    128.92935    128.91878    125.99078    131.67813    130.48165    129.87542    133.40331    131.83458    131.77693    133.19974    137.31441    136.76524    139.09177    138.02236     139.9149    141.08246    140.45511    139.69334    144.46213    146.84337    143.44474    140.02358    148.53363    146.90569    142.56684    144.07034    147.83805    148.08195    148.39474    150.84038    149.76997    149.27098    150.15967    149.21287    154.25244     159.1919    158.18186    158.00664    154.04037    158.81304    155.43424     161.3327    159.76468    166.30981    159.66176    155.60158    156.10304    159.49937    160.55239    158.36217      156.032    165.93929    161.08913    158.21689    157.62033    166.00475    167.87896    168.04248    165.72389    167.70457    172.30032    169.96837    165.74674    165.06248    165.81654     165.1918    175.72253    172.72222    175.15408     176.9838    177.42109    179.97086    179.27038    176.82002    176.59636    182.81561    180.56684    187.34197     177.7587     185.7023    190.32482    190.53902    190.11463    187.82703    186.53818    193.83656    198.35008    197.60467    199.70609    200.51026    199.10399    196.45335    201.92777    204.68258     206.4978    197.99209    206.62504    199.38338    221.15912    221.37973    209.03351    213.71837    222.81027    212.79495    222.67654    224.49702    220.85377    219.16014      211.763    207.96566     220.1561    225.67362    222.25722    213.19779    222.18043    219.45084    226.35679    247.16343     228.1966    221.46017    227.27493    229.14211    232.53381    236.95774    256.85127    237.21098    239.49529    239.00179    247.11814     269.1527    257.46426    263.14408    268.10796    248.30265    267.70202    273.42041    288.29204     297.7905     294.4955    275.82143    276.90736     287.7751    306.69013     302.1232    322.86518    313.72679    331.03404    321.06392    367.50143    386.59372    393.64699    355.66906    354.03761    366.53808     383.2104    496.54308    479.09307    402.10263    481.90775    496.77885     503.0014     563.9135     541.4927    564.28651    561.86237    670.25821    512.84503          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       12.00    263.20428    247.30704    232.27601    229.72552    222.85514    221.52915    210.60852    212.09608    198.12873    193.42213      191.749    186.21021    186.06151    179.02745    183.42402    178.85031    176.82845    174.89226    172.80811     174.2184    164.19788    168.61288     164.8891    165.15217    168.87026    183.23686    202.58644     206.7521    198.29732    173.01891    162.35996    154.03861     154.6823      148.656    148.46578    141.77481    155.41831    144.41729    145.69256    152.40974    144.39297    142.24527    140.10914     144.4155    145.01169    143.15278    142.34421     138.0449    138.44532    139.82544    141.92123    139.84694    136.83777    136.77467    138.43824    141.74627    155.55916    174.19643    183.57493    176.83139    160.14466    141.60047    133.30111    136.21364    134.05559    130.39405    134.62386    130.74759    132.73296    140.10048    146.36944    159.34342    168.10933    164.03055    153.15774    156.13605    164.80934    181.58992    186.46956    184.18929    175.06427    155.95291    152.24369    145.79103    140.35051    136.06149    128.20425    125.30264    126.48351    129.38925     133.2565    136.41609    139.42638    132.17611    128.58051    129.09286    124.84979    123.09149    123.74953    121.25451    127.63878    125.59758    127.64477    127.30872    122.09439    121.61937     117.7998    123.76717    120.40963    119.79359    119.35947    120.37618    118.83679    114.40312    116.84757    122.33931    129.18168    129.01743    127.62626    125.35075     123.5229     120.4503    120.24076     120.1324    126.03957    136.06403     130.7055    135.68209    132.07998    124.87512     121.0801    122.46214    115.57192    118.71256    121.38911    126.72184    131.35512    141.77622    151.64149    160.02982    163.55979    159.16317    151.44483    140.60403    135.29782    126.32334    124.11451    123.84538    122.08133     120.1019    121.03045    119.98908    116.82135    115.70122    117.29538    115.65288    118.00296    118.48554    115.51329    119.15164    121.71437    121.69034    120.03007    120.73621    122.61997    122.55359     127.0778    123.81607    122.72663    127.18883    122.00549    122.47712    124.87186    125.92005    124.07177    125.89696    128.66983    129.96679    129.75782    130.54749    131.16823    126.81588    126.43483    122.74208    121.62019    123.39925    118.98486    117.68524    120.91477     120.3547    120.25774    120.14068    123.38368    122.78223     127.2166    128.26184    129.72532    124.82985    131.54538    133.15458    129.68722    127.93842     127.1406    124.27178    122.97848    127.76858    121.60263    125.09103    126.28087    124.64045    123.83388    127.06582     126.1946    128.21401    123.16501    122.27355    120.07143     121.8726    126.35145    124.82422    120.49619    119.79408    124.57861    123.43839    127.20898    124.83025    127.36953    127.26636    130.73626    124.71816    131.10624    131.71195    129.57607    128.08351    130.11651    133.07168    134.21493    136.26921      135.326    136.34409    138.00365    139.98833    138.06556    136.79717    137.37713     136.5693    138.13134    132.14606    133.10794    129.07145     134.5072    131.55166    133.61584    137.73529    136.81137    136.19204    137.95944    133.78128    137.52456    136.82772    137.07508    138.34321    139.93044    144.92475    144.52775    149.51614    147.64576    146.04286    146.24252    142.64335    140.92992    137.82342    137.31134    133.65163    132.23912    135.03468    133.65165    136.74139    134.81278    136.44772    134.04978    136.43981     135.8378    133.90412    132.91043    131.48278    138.68619    135.21676    135.13574    136.73239    135.38959    138.86721    130.62234    127.24119    132.72417    135.06223    134.72878    133.88929    132.67925    130.26616    128.39202    132.74305    135.86328    131.34111    128.41571     129.9324    127.44676     129.8852     128.8189    128.13647    128.11803    131.16608    129.17616    132.95931    131.57354    132.35491    130.06218    131.33531    131.65367    130.93574    130.59258    130.48619    128.04964     126.2969    126.16548    124.02692     132.8303    127.08636    131.36022    131.99668    133.13095    133.39416    137.20303    137.24668    138.73202    138.15137    137.98627    137.48748    142.08932    142.41302    142.22071    142.40538    144.99456    145.98414    144.78716    143.49151    148.97887    144.75233    147.26646    144.86557    142.90315    147.08358    148.65756     153.8345    150.18619    150.01327    152.80582    150.31222      159.904    160.88558    157.52503    158.84194    156.98331    155.75563    160.40959    157.28504    161.12387    153.78151    155.84379    154.46003    162.24599    162.78628    168.26847    162.14318    159.76657    154.47752    159.46422    166.90957    169.02272     162.5014    174.77432    160.81542    168.69639    174.86627    167.62549    166.99447     166.8248    169.44746    168.35984    167.80391    177.19616    179.92838    174.89739    182.62687    171.75077    180.78093    185.51532    180.93499    177.71647    178.11888    184.37703    183.77429    182.32554    189.92624    187.84774    193.79146    188.73817    191.61272    191.90499    192.62475    191.21127     197.4717    196.70749    189.90743    200.68408    208.37343    205.63929    210.21457    203.55432    203.70435    209.68251    196.28686    214.00964    215.43436    196.06818    206.40497    213.80801    206.54392    215.48049    223.52835    214.83656    222.15653    214.25672    211.71999    230.00134     220.8568     234.3482    214.59841    218.19758    226.14453    230.49726    233.68925    241.22093    239.34962    231.54849    229.78441    256.16773    236.07203    239.99742    238.58631    248.03496     231.9058    274.18427    262.25326    262.83828    267.54144    249.13119     265.9279    272.38108    308.04265     288.7714    300.59192    279.90169    301.63157    288.09007    286.77842    296.26157    306.87862    308.20364    291.05935    338.41439    388.53227    386.05447    400.22159    384.57549    401.53612    367.81037    408.86834     475.0869    425.45251    429.22874    484.52206    436.78694    594.15428     536.2992    517.56626    563.29366    636.85968    533.42537    568.58492          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       12.50    261.85002    247.14432    232.72276    229.29569    221.60487    217.44492    214.81659    214.06854    197.81629    198.30712    192.60368    191.25107    187.31995    182.51281    181.18899    176.32287    173.57445    173.49995    171.48536     179.6304    168.01744    165.14063    162.99181    166.46613      171.471     183.5966    202.70559     208.2883    198.29858     171.0491    161.60734    156.17983     154.9386    156.38568     149.8131    147.00203    151.54111    145.63163    146.08663    150.58757     148.1132    139.78217    144.84295     141.2968    141.83638      142.525    140.12107    144.15823    140.58988    142.24017    138.43377    140.52875    141.38635    141.42572    135.87965    140.64592     154.3482    170.94438    177.85099    176.53962    158.51762    144.67165    139.76839    135.22996    136.49937    129.99214    137.69755    131.38633    136.47315    139.17582     148.5117    161.76704    165.24572    166.83207    153.88634     158.0628    165.38989    177.66124    184.74591    176.65968    161.73635    151.50969    148.48068    144.35317    138.37279    136.53883     127.0687    127.69034    126.89108    126.80552    132.78734    137.24078    138.89726    134.42547    127.14419    123.67722    123.69101    121.27958    123.08222    125.36143    122.34601    126.61271    129.95794    125.39835    121.30171    118.81229    114.51855     114.8464    119.07683    119.14241    121.02585    122.16548    118.28119    117.29548    119.59195    124.35083    127.58177    129.38339    131.72454    127.23346    121.65278    120.11543    117.68654    122.30632    127.87215      133.793    132.68368    136.34592    129.69781    126.65313    123.59729    122.77644    118.61163    117.37067    120.52523    125.24822     128.5888     142.1617    148.81168    156.67594    160.94835     154.8059    149.62315    137.85042     129.7222    126.19738     124.3893    123.45874    119.15413    118.66474    121.39402    119.10213    117.63509    114.58998    119.27575    114.46583    116.40106    120.13489    117.97284    119.81467    120.46397    121.78018    122.70638    122.13129    121.00426    123.48142     129.5981    124.87746    124.80211    123.73567    125.49279    120.14852    126.27644    122.68131    124.63041    123.44462    126.18243    130.57058    128.26038    129.60343    130.21306    129.41219    127.78615    126.25725     120.8148    121.18599    119.90979     122.7539    119.83318    119.82653    123.20875    120.24402    124.18317    123.93527    127.66246    128.03668    123.62583    125.41715    129.31035    131.90799    131.27515    132.13814    127.61794    125.50361    123.94363     122.7115    127.38621    124.97278    125.48386    130.72433    123.09055    127.19016    126.90152    123.81416     122.7868     120.5998    118.46228    118.87657    120.65265     120.5856    121.51241    121.93911    126.35401    122.06199    124.71905     126.7008    128.22061    129.13725     131.8736    131.68819    130.87644    131.34063    132.87318      136.365    134.43953    135.77898    134.06035    137.35936    137.00155    142.94207    139.23561    139.35396    136.44461    134.90251    139.26617    134.46945     131.5187    133.15311    134.55722    129.92478    132.73911    134.40671     133.0111    137.15327    135.33846     133.5031    134.30049    137.21467    136.06477    137.54702    136.13149    138.51373     143.7176    149.22154    146.44482    146.34822    145.55065    146.62548    139.66555     144.1728    142.20154    139.90918    138.62483     135.2794     132.3722    137.31933     130.9103    136.09202    139.60386    138.53849    136.47601    134.69234    136.94339    136.43555     138.2796    136.07551    139.43514    134.27868     134.1801    133.06511    135.88016    134.52063    133.03115    130.91742    136.68773    137.18883    135.06115    133.37558    136.99916    131.43214    132.03913    131.92544    134.18138     128.0084    129.95708    133.95635    127.39573     130.1253    128.50698     128.6019    129.68722    129.78551    131.38945    132.78062    132.50298    130.39629    129.97371    131.80838    131.93136    127.23944    129.71021    132.79351    128.91835    129.14577    129.37064    130.23569    129.59965    130.16341    134.29615    133.04947    133.33684    131.17898    135.78012    135.75894    134.26758     132.5809    139.83726    136.76714    136.89775    143.78431    142.45769    142.38748    147.15313    144.37791    142.43319    144.20205    144.64598    145.08495    146.71597    146.80897    148.34678    154.44033    147.82374    150.75175    154.42532    150.41309    152.77565    155.66033    153.58589    158.32604    157.91443    158.96257     156.6368     157.2563    158.65757    163.77221     156.9047    158.49933    155.69674    158.22357    160.77256    158.74695    158.79596    165.92096    161.82845    158.39456    155.55069    163.80308    172.82942    165.27676     162.7365    162.82595    158.76265    168.36193    175.84229    163.23978    171.57512    167.83786    169.84977    177.38437    176.04331     175.0595    174.00172    173.48402    170.93003    177.30825    182.48981    180.67211     179.5044    184.37442    184.79207    179.89534    186.29524    184.83919    187.61814    188.54973    190.67029    190.16799    199.32574    195.77073    193.62461    200.87855    196.55195    200.35247    199.38751    195.54809    208.04786    200.63492    203.34255    206.03262    197.57178    213.17414    213.95927    216.10038    199.22554    215.55512    211.38897    212.32566    210.49995    217.82808    210.66489    207.39464    204.92963    216.53905    234.62161    219.75441    222.25193    223.41185    220.51689    224.87977    245.03907    241.39778    244.15107    239.04028    226.16406    228.26481    240.59201    235.80679    246.27745    250.79558    250.94171    252.52651    263.81119    262.73446    260.85118    260.80599    267.18749    263.51611    288.27508    294.97513    294.08954    295.34177    276.89004    284.60525    264.82069    314.80049    334.83962    347.75978    308.55709    312.84144    314.89276    369.15669    372.44202    351.25408    353.14706    376.02492    358.91843    403.29678    428.11857    468.27361    405.00724     488.1298    492.24478    500.28406    711.80431    653.14409    519.50116    553.94992    544.28221          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       13.00    263.98777    247.01288    232.23638    226.82767    221.85562    219.40656      219.828    214.97363    197.09014    200.14251    196.09466    193.99576      186.573    186.53691    179.40219    172.54606       174.15    175.55177    172.92951    173.97327    172.91517    165.54267    160.81362    167.52755    170.14336    182.04038    197.01537     208.6968    195.22288    173.02139    160.85482    161.84913    154.65908    158.59909    146.91111    148.34551    148.82351    151.56361    144.70979    147.18909    142.66879    140.20719    143.37619    143.81788      147.542    145.50818    142.94932    137.79178    138.01892    137.86068    141.32199    137.89281    141.01598    137.83897    137.42009    143.15695    153.75536    172.78812    180.81394    177.55507    161.73217    143.68371    136.87412    134.82268    135.10846    129.56522     132.1435    132.34683    133.39728    138.50297    146.65941    159.29583     170.9197    168.07068    159.25473    155.72965    163.69205      174.486    177.29728    170.36289    155.25368    144.53252    145.43741    142.10746     138.7896    132.56927    129.27705    125.04358    128.89681    129.43938    132.47356    135.85586    138.43717    134.09312    126.74175    121.17362    121.75318    124.35341    123.03984    122.47239    122.56994    127.44811    126.51484    124.93987    122.73654    119.67154    119.83478    118.81447    117.68974    117.97303    118.84174    119.99344     115.1347    115.84751    121.71815     125.4182    124.48554    130.74704    126.77467    126.13504    120.19808    120.04134    121.85214    125.31334     124.8414    134.64326    132.62335    136.14523    132.76194    125.42338    122.80689     119.0499    118.01178    119.69326    118.96884    123.70194    129.88305    141.40215    151.15369    154.10699    161.81764    157.67342    151.91872     140.2204    131.14381    127.95326    124.10447    125.15586    123.74071    125.36676    118.32742    120.48699    114.66322    117.91017    116.72059    118.35278    116.64377    117.13912    119.43749    118.95459    121.60083    120.10536    121.83147    120.46236    123.08246    123.08499    128.68523    125.98357     126.9631    126.73628    123.75641    121.96067    119.33006    122.57144     123.8487    123.37317    128.85882    129.74331    128.87866    129.96607    128.22898    128.71379    125.90189    124.48814    124.13644    117.70141    115.61213    116.86483    121.32718    117.50611    121.32748    121.10935    123.40047     122.6091    125.05634    126.13074    127.06396    125.52673    124.15759    130.10915     129.1855    130.30804    126.42431    127.25103    122.45401    123.83761    125.94336    123.52753    127.27586    125.71148    124.01816     129.1952     124.7782    126.76039    122.91953    122.14456    120.21027    118.86367    118.53269    123.38233    120.32386    122.45096    123.05002    122.41221    126.83907    126.47156    131.02923    125.37734    125.84697    130.26136    130.38001    134.43884    131.53034    132.11864    135.01109    135.40096    136.25174    134.50671    136.33575    137.48622    137.61051    137.38288     133.7544    131.62622    133.43839    133.31189    134.56935    135.91191    130.35197    131.46587    132.99705    131.84407    132.64212    132.53354    139.00723    133.58691    135.94642    137.07505    142.51381    135.00608    136.14078    136.91395    141.06269    146.71857    145.11978    147.30667    147.37334    145.47808    145.94284    144.20485    141.94381    138.28352    136.41892    137.46274    134.98991    134.46769    132.57216    134.17131    132.97687    133.75196    134.86847    135.52383    138.18384    136.87835    139.10663    133.37691    135.42784    131.51857     133.4955    131.72498    133.12521    130.46749    131.25765    130.84016    132.08182    134.42254    131.64878      130.941    131.68703    135.77946    125.91038    131.92282    131.97755    132.34531    130.49704    132.79835    127.89711    128.80702     131.1264    130.94174    127.95088    129.89742    128.60084     133.5562    132.45873    128.91211    132.11246     133.0744    131.03559    134.39268    131.62124    133.77964    130.08556    130.40234    122.88634    130.20235    128.73813    132.04946    128.29225    136.03672    133.82146    132.07043    136.52423    136.89833    138.02131    135.53584    134.62046    136.62378    137.66286    139.80602     140.8746    137.95683    138.60975    143.71899    137.15753    146.66816    138.87617    150.09201    147.86524    142.00339    147.23601     147.9213    148.23991    149.27655    155.44236    152.61601    152.55279    153.87524    156.62762    159.89484     161.7741    158.58643     156.4395    156.50774    154.84567     160.3581    162.52476    160.93242    160.20988     154.9094     155.0831    156.35559    158.55901    159.81704    162.97214    159.50841    154.94959    167.94629    165.69734    164.45885    163.02387    170.11114    156.97296    167.95198    168.14366    172.74333    171.51062    177.34309    172.41668    173.79294    177.12733    171.61474    175.91737    176.99961    170.42955    176.41926    182.43601    183.44946     181.4815    181.19675    178.77926    180.80264    192.04191     189.0898    187.59901    186.86318    185.97141    186.32384    198.97547    203.12217    196.01654        195.6    202.83352    195.03022    193.29193    211.82505    199.13657     200.4379    196.91892    206.08647    197.26323    214.46359    216.81066    213.09226     211.9599    213.80322     208.7153    212.80456    226.00649    218.07076    205.43511     205.9316    204.78659    214.31707    211.44815    219.43025    215.71509    212.76762    218.13848    222.46749    225.31888    229.45619    242.04619    220.74348    240.18178    246.77488    243.89954    242.16225    239.88379    237.32563    249.64458    247.08265     265.4102    258.84677    248.45571    259.91045    269.44145    268.30938    267.12966    284.75222    273.40342    291.83003    273.76675     312.3091    292.97525     301.5516    313.85978    330.95523    317.23469    313.69684    328.15297    355.66223    386.32252    399.10402    385.72787    379.81029    390.12647    413.22983    473.47382    482.00448    440.65553     547.5334    455.69473    647.93258    562.11809    653.38143    541.80156    461.24435    726.75148          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       13.50    268.42906    247.00777    231.41602     223.2848    218.81768    221.13833    221.34124    215.63099     198.3975    197.12513    195.65133    192.33045    185.61752    189.35991    178.36105    173.06832    178.00982    171.98663    176.78701    171.85639    171.48705    165.45437    169.04332    168.77602    172.41705    180.68046    203.66859    208.90432    194.92523    173.96397    159.58717    161.21958     155.1525    155.04093    147.85971    148.43966    149.35685    154.86443    147.43361    150.49054    147.88346    143.84943    140.48545    139.67031    143.90418    140.68203    142.91831    140.27634    139.02781    137.21653    143.74906    144.90732    139.96619    138.94224    137.16239    142.14332    153.66409    172.76857    179.68781    179.16799    159.70034    144.42614    137.40138    137.46341     133.0237    130.43704    136.04795    134.98726    129.55895     140.5242    147.28542    161.86127    168.06948    166.74874    155.22743    161.07684    168.53117    181.22997    178.17034    171.07354    155.23604    147.66023    142.94595    142.96651    138.86707    135.80555    125.44024    127.41252    126.40601    130.02643    133.65688    138.13059    138.70122    131.96981    127.91977    127.66691    121.53007    121.89856      124.354    123.44313    121.82237    127.52026    126.07653    125.39606    119.70426    120.41685    118.58484    114.50879      120.259    119.30392    119.49891    118.86656    118.07846    117.02558    120.78738    122.08349     128.8383    132.65769    128.65051    129.08021      121.975    116.66743     119.5354    122.89223    127.73249    135.45072    137.70098    135.69451    132.38912    128.15416    121.22835    120.62131    118.52176    115.46936    118.46413    123.00685     131.1552    138.51295    149.35477    157.12183    160.27394     154.3183    144.87691    142.39178    136.31528    129.66529    126.12051    127.38647    124.87788     128.2809     123.9238     120.1324    116.10207    119.78014    119.32592    119.68106    121.63016    118.39509    120.29309    120.22841    120.59573    118.08951    120.63525    121.49138    124.95505    121.55245     124.9292    128.43925    127.63055     123.5452    121.48866    124.46509    120.20757    125.01639    123.09378    124.66915    126.91505     126.3288    128.72269      127.688    130.48098    127.32917    125.25969    124.57238    122.66308    116.16991    121.06804    119.31555    117.84928    120.17024    119.42558    119.53176    119.90127    125.48584    122.17855     125.3071    124.39673    128.19033    127.49116    128.42681    130.02821    129.29082    127.57228    126.60208    122.74043    125.10139    122.97319    126.62012    126.38137    128.83915    126.72984    125.64478    129.50744     122.7825    123.65922    123.82091    118.96583    118.43115    125.51773    119.43873    120.74655    118.55034    122.11812    123.82436    124.38157    127.73608    128.83004    130.00429    130.03566     128.8287    126.16238    131.43116    130.68022    130.82696    131.49234    132.47372    130.95843    129.33616    135.82793    136.70525    138.34367    133.96127    133.17255    134.85287    134.44572    135.14722    135.07405    133.37809    133.28118      132.417    130.18208    130.16829     134.1977    135.31314    133.91418    133.80942    134.71863    137.31866    134.90845     136.3702    138.64926    142.71734    133.92637    145.76164    146.35809    145.65996    147.07742    147.93707    143.80098    145.26118      142.861    138.73655    137.86416    137.42047    130.33077    134.27374     134.3072    138.15171    137.95627     138.2764    138.32733    137.14302    140.54785    137.59396    136.42343    137.69757    137.87071    132.68227    134.06466    136.08195    131.64805     130.5646    127.77835    131.73466    129.10386    130.18228    129.06705    134.56235    130.84095    129.57703    130.48328    128.08943     128.5544    130.69001    125.94024    130.51938    132.62464    125.91505    131.37203    127.00331    132.41416    132.10063    128.61121    131.42706    131.36616     133.1189    131.31415    127.89201    130.34877    129.37886    131.34373     130.0468    128.20222    129.33521    128.86883     124.1781    128.25825    132.25687    133.58602    132.85482    133.15261    137.31016    134.71319    141.98842    136.45232    139.75585    140.25419    139.23774    139.82651    141.66129    141.36602    139.49621    139.78939    138.46011    141.58277    141.39223    143.58638    145.06813    143.63177    142.93951    146.60758    144.25595    152.55462    150.60463    148.80828    153.39055    150.87936    154.73198    158.16754    158.31939    158.90806    157.24139    158.56231    153.36267    162.20951    158.78773    157.66582    158.79453    159.52508    159.42623     152.2465    162.97405    160.53769    161.87817     156.2176    159.32525     155.6371    164.02219    163.01365    160.56326    167.97146    162.69504    159.05942    168.50873    170.37953    172.50074    163.74035    167.31774    163.77038    174.18146    179.89011    181.29132    177.01288    180.62541    168.02231    172.97308    186.25316     191.7247    185.33134    181.45253    180.18494    180.11848     182.0892    191.55513    188.71229    187.64272    186.08556    180.06972    195.10637    205.86497    194.76341    198.19927    197.98701    195.61322    193.30205    205.10854    198.85067     209.0607    203.50709    202.40889    205.43293    211.54556    213.11014    208.58213    197.19172    204.86371    207.89071    218.96993    211.61744    211.96771    203.06051    207.51824    213.84238    216.47599    230.01426    210.45564     222.3285     226.3133    222.82459    225.62858     227.4957    222.53094    228.95511    243.14355      234.959     228.8823    245.04909    230.60625    254.87738    245.18547    246.13932    246.85308    258.93882    269.29014    257.73591    263.10177    257.38431    264.79794     291.5903    287.99304     273.9086    282.14023    275.68133    269.50475    296.25731    308.88746    338.71539    312.34754    331.42419    341.49357    296.62808    358.45393    369.73338    380.38839    388.47114    382.68999    343.27114    474.95048    395.74186    418.91054    492.63944    488.00368    400.74179    606.07147    614.24352    549.62206    582.56725    541.04567    586.45432          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       14.00    272.77445    247.38864    231.13169    223.94556    216.95885    222.06017    223.20543     212.2711    200.83101    194.64627    191.35848    191.23564    187.01947    190.25674    179.60809    174.07084    176.68096    166.76543    173.17883    173.87198    172.51141    165.37715    167.06531    171.66985    172.09685    183.99994      204.155     208.0112    196.41412     173.8167    159.04948    154.59894    158.24542    152.61011    151.70708    148.97422    149.98592    148.62602    147.69867    150.65931    146.88831     145.3902     145.4741     142.1821    144.57235    140.95704    135.13559    141.08769    137.96954    139.78111    140.46042    140.99485    143.16527    143.64282    136.83688    135.72034    150.31467    171.15525    180.96076    179.22608    157.37053    141.64694    137.71942    139.66322    133.76363    134.43091      132.842    132.80077    135.59699    138.05163    148.97123    160.68276    166.80998    166.74704    158.30333    158.57566    174.16432    194.16345      195.226    184.23673    166.63306    150.86281    146.17551    145.34404    138.10905    132.45207    130.37439    125.73496    129.86702     127.3384    133.84054    131.96251    141.31621    134.20768    127.72804    121.98506    125.33974    122.85206    124.63629    124.85254     119.8483    127.48324    124.22337    125.52626    122.48194    123.76184    120.71736    116.25571    119.78095    120.74682    118.50968    118.31446    120.12764    119.36598     114.0223     122.8093    124.94166    129.85911    130.12183       127.33    119.93809    120.83176    119.71147    125.49867    125.61814    132.92614    136.48992     131.8113    130.47726    125.99574    122.23446    118.27822    120.35153    117.60886    119.34678    120.84995    128.79126    139.44116    152.65068    156.91474    161.06436    155.75976    147.45275    139.15046     129.6549    128.00419    126.05599    128.53544    126.84228    125.68948     126.2794    121.39514    121.03797    118.35842    118.79882    120.13781    119.39637    118.25163     117.4123    120.75277    118.02782    120.17238    120.11155    123.07768    122.86969    122.97817    124.70529    128.92283    128.73947    127.63815    123.34191    124.80158     121.5059    123.57073    125.50522    128.58198    127.34767    131.57514    127.95179    123.55302    125.85288    128.12659    122.47194    120.50646    121.49212    117.08041    118.78282    118.70238    121.24679    121.73691    118.89348    119.16726    122.86577    122.01688    123.32566    126.56613    126.15674    125.74146    126.70123    127.57055    132.42206       128.43    128.00669    123.87257    125.10311    122.31544    125.79239    126.10205    128.87677    127.71396    126.20034    123.01073    125.80166    123.09994    121.91902    119.74102    120.16777     119.5662    121.66309    120.97821    122.61317    122.26887    119.94449    123.82182     119.2251    126.22594    126.83211    126.13732      133.387    133.31283    127.40048    131.42394    128.64142    129.38071    129.25881    131.96334    134.90346    131.22222    135.09309     134.8216    133.22784     136.6734    131.06087    130.44892    134.70504    134.74828    133.42862    135.57089    133.10712     135.0445    128.77329    132.45747    132.90697    137.07136    136.94287    136.70077    133.79612    134.08234    138.86315    139.70107    138.20154    139.99468    139.15821    140.35234    144.06969    147.23516    149.07228    144.92718    142.40451    142.20474     141.2276    139.69648    136.92444    136.14735     135.2458    129.22483    132.97131    135.07389    136.95088    137.90604    136.47865    137.51134    138.48323    137.31336    136.71334    134.98521    136.14158    128.81891    127.58616    132.76819    133.89272    129.17504    130.64912     129.0872    126.75195     132.7183    131.63452    127.95975    131.12457    131.74582    130.49842    131.60145    129.45354    130.61404    130.01959    130.13932    129.45513     127.5655    132.32114    130.75364    129.61925    132.34039    130.69954    126.95902    133.89486    131.35697    128.01717    130.63088    130.82591    128.16376    128.89567    132.42395    124.82802    130.97906    128.26145    129.86119     125.6496    129.34588    129.86525     130.0084    132.33057    133.85802    134.33641    131.42723    138.34031    138.40939    140.28813    141.58183    139.01449    139.09759    136.91796    138.70437     141.8761    141.18829     139.8364    139.55096    144.98959    142.84203    145.07043    143.72142    146.31996    146.83271    147.08795    152.03879    146.43621    147.77645    150.38977    148.45857    155.64823    159.56662    165.54289    153.63883    158.84302    159.55968    164.62639    162.31074    163.76603    163.08633    157.43743    153.51784    160.63616    157.64052     162.7667    156.25215    162.54148    154.61401    162.31147     163.1359    164.71645    159.79963     167.5008    159.78976    165.23557    164.35857    166.50134    168.09738    167.09256    167.35796    165.71628    171.04319    167.05396    172.00011    170.97211    175.37114    174.54984    176.14253     175.1409    172.66389    185.06538    169.37047    178.35142    181.56996    183.70733    185.12373    183.04733    184.85575    191.63296    186.37597    191.13764    190.81395    185.46226    203.40164    197.10713    187.75365    197.02288    207.04047    194.34669    191.88776    203.57113    199.70036    202.76051    209.65004    209.10445    211.49961    212.95673    209.31348     203.1206     208.1089    224.63304    216.83224    217.65795    214.77175    210.77696    216.65907    219.33365    218.73715    232.14659    223.16166    218.58835    226.51813    230.97859    222.69145    236.29843    243.50601    240.35046    233.70332    244.91124    244.63198    248.69112    244.51781    244.03842    243.44436    253.07506    260.82053    256.66457     259.6757    263.02004    275.02165    278.80692    278.31591    282.79066    279.73461    296.74311    298.14017    312.50569    308.97016    329.71718    310.24659    326.43466    309.38716    318.48648    374.75122    355.01572     367.5159    354.58585    332.09721    404.78298    418.68419    436.57952    450.46221    433.27348    398.77531    462.31162    538.65594    585.30334     559.4009    509.50854    598.27962    502.82507          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       14.50    273.76018    247.86849     235.3379    224.89035    220.44358    222.07074    222.15666    207.29849     202.1581    195.42469    190.35894    189.07329    189.48586    187.23475    181.28069     176.3343    175.10805    175.93684    172.77138    173.27747    174.03255     166.1457    164.56199     167.5557    171.51198    179.67417    202.31744    207.87428    197.71261    175.74024    160.55488    153.90315     155.3278    149.31038    151.99306    145.74112    149.56665    149.45394    148.68872    147.73669    151.79228    146.27162    147.54301    142.12398    144.62929    143.27227    146.29656    144.42247    142.33857    138.60212    139.10839    142.15847    140.30326    141.81639    136.47746    141.12266    150.21503    168.84372    180.22436    175.28031    160.79451    143.47285    138.10168    137.33032    134.86119    130.33818    135.10018    133.19167    133.18564    140.16578     148.8079     165.7329    170.08325    163.20428    157.21316    160.97167    181.19316    208.63632    215.67342    204.76327    181.90112    158.57891    149.05207    141.19662    137.48961    132.58093    128.23302    122.19094    125.08036    131.69681     135.5397     140.4913    139.33562    134.26581    129.32318    126.02434    127.31261    123.66149    128.54805    125.64655    124.54046    125.27137    128.55737    125.22711      120.955    122.02539    119.15312    116.11422    119.38665    120.78919    121.77085    124.25161    118.68278    118.00367    118.50047    124.14092    127.04113    130.69898    126.60508    125.95377    123.32768    119.98882    123.56733      122.519    128.11683    131.70934    134.50005    130.56288    133.58441    126.39026    123.09709    118.50326    119.62109      117.114    117.56683    121.80255    129.46022    140.77556    148.68617    162.17802    154.84655    156.73958    147.65192    137.36608    131.01118    129.39418    123.47425    121.76856    124.34896    125.42633    124.03188    118.47982    119.09117    115.80474    116.64933    115.77173    116.14228    117.77178    117.52114    117.64019    118.94747    121.08176    120.77731    120.71303    124.50599    124.33829    124.75507    132.52902    132.92877    131.06679    129.93789    126.71702    124.67685    126.00536    122.10255    125.82358    129.39861    130.12508    127.67647    126.43775    126.43108    124.68412    123.10893    120.45272    124.47453    119.54465    120.90491     117.7143    119.41253    118.55532    120.06813    121.02548    117.38789    124.45119    127.61161    127.98891    126.53306    126.92482     126.5842    131.32289    129.69039     129.3759    129.36804    126.02294    123.82241     116.5355    124.32852    124.05399    129.94712    128.27154     125.5197    123.31314    125.35913    119.19545    126.49106    120.07349    121.85398    122.23433    121.96756    121.83687    119.67151    119.64856    123.41716    123.21826    125.51839    126.89301    125.13654    127.80274    124.75033    129.01413    129.77205    128.84001    131.26631    128.09512    129.86749    131.54474    129.51066     129.9387    135.35614    133.54549    135.48604    131.21415    133.70884     132.0098     131.3381    131.58557    133.40147     133.0262     134.5602    137.60862    135.63453    137.07211    134.68439    133.27849    137.22524    134.50385     132.9184    135.79885    135.49222    137.62104    135.01535    135.79909    136.80021    142.28398    144.60521    144.62731    145.03324    143.06509    139.29164     140.0587    139.04675     134.6422    134.28985    136.55009    135.30899    133.29526    134.03913    134.14613    138.76724     134.0392    135.41596    135.41074    132.63664    138.81288    137.37358    136.62663     135.7712    130.14837    130.00421    132.12285    129.95344    131.28485    134.21724    131.57299    128.68271    128.36959    130.41906    125.57605    129.46721    126.29548    129.38376    129.42364    129.14249    129.02413    126.34266    129.81537    125.94043    127.53824    129.94937    132.88782     129.2721    130.18765    131.19152    129.05583    131.34403    130.98185    132.07244    129.97515    131.10719     130.2974    129.60826    129.99302    133.60937    128.98016    129.94784    127.96562    124.93246    134.54305    128.78944    130.13026    130.50869    136.56725    131.29145     136.8341    135.66226    138.02001    136.84327    137.14598    140.25483    142.54758    136.77041    138.91316    139.56272    144.37114     137.6767    144.85228    145.12044    142.51941    144.20508     144.5817    140.03236    142.60334    149.57364    150.22204    149.35395     145.4998    152.16731    150.14761    158.48113    157.68685    154.27973    153.91927    151.63876    156.67759    156.42653    160.75136    160.59439    158.56182    155.77461    154.34804    155.94494    160.96361    161.90585    159.98293    162.80673     153.8106    158.17047    162.99362    153.44182    164.01542    172.69055    156.34854    161.36941    161.49961    167.09916    167.14178    168.74304    172.20522    168.03036    169.98424    172.86756    170.04022    178.37059    169.23412    164.03932    172.31279    183.53874    183.64083    180.14648    185.38662    178.51072    181.26586    184.48226    188.02161    183.33875    184.01377    182.55183    187.58016     198.8181    193.31964    197.13653    195.33475    197.05338     196.9237     201.8393    199.93867    203.76744    188.79958    201.78263     210.8034    203.90802    211.69042    216.73741    202.44565    210.54381    213.20442    193.33713    228.03007    228.79609    218.63658    221.43278    211.43039    214.69612    222.14109    219.13221    217.51386    219.59647     219.1475    211.80859     229.0757    225.70091     227.5674    246.67688     240.7885    220.53318    227.77687    255.75832     247.9435    246.42335    247.50213    241.69392    244.28493    260.20992    251.07046    247.46021    272.51803    257.79925    261.53068    282.07442    286.01916    283.46963    283.52992    302.37165    311.86085    321.34941    321.14246     326.2926    331.29205    296.17928    335.55306    343.51013    382.11917    378.49727    364.11189    377.38264    366.89336    331.47534    488.80897    446.82709    431.45193    446.69215    471.73808     427.8876    575.33822    576.76881    573.13786    581.06824    500.26476    535.24888          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       15.00    272.13257     248.2349    241.08052    225.24899    225.66851    221.00571    216.54283    205.49624    200.98975    195.72459    192.16662    187.08791    193.24971    184.70113    182.04185    177.96278    174.27059    178.88772    176.62861    172.32953    174.89609    166.43716    165.62203     163.4245    172.76371    179.12741    200.13105    211.81983    198.35307    175.49822    161.46072    154.85551    155.53165    154.62578    150.66056    149.18544    152.62232      146.611    147.67433      144.425    153.15522    145.54603    145.24596    141.76889    148.08885    144.57516    141.13634    143.55173    138.29767    141.98094    140.64369    138.08289    140.85032    141.77243    142.28946    140.97619    153.62627    167.33241    178.10807    173.41962    161.86613    140.99512    139.95613    139.72441    133.75832    128.05182    134.51504    133.64174    134.93936    141.46231    151.03673    161.28566    168.90929    165.69426    160.10099    163.26474    186.53978    221.53961    229.81681    222.56379    199.19062    170.42832    152.97896    148.15853    139.86532    136.29436    129.05849    128.74684    124.71662    130.79916    135.84072    137.34157    138.65008    137.46752    130.47253    126.07189    133.05105    129.59499    128.64896    125.51681    127.30488    125.08069    127.06322    125.27204    124.50948    118.96099    117.96452    117.51669    115.22137    117.21097     118.7243    119.47392    119.21886    118.36676     117.0169    121.27766    126.35663    127.20436    125.57894    124.95376    122.25383    117.07648    118.29023    119.85607    127.93257    132.50331    134.33534    131.20746    130.47612    126.74975    121.37779    123.18514    117.65937     118.0712    119.85489    121.71925    130.44907    140.21198     154.9839    160.14062    160.53017    160.98872    149.16599     139.1463    130.73467    128.48184    123.18914     121.3479    126.69644    121.66169    121.27572    117.82878    118.63193    113.23585    115.52861    111.48324    115.46458    122.99786    115.89005    117.71078      119.545    120.12557     117.5114    122.82147    120.95645    124.22876    127.93588    134.67013    138.60288    139.24532    134.59487    126.78377    124.95363    123.94935    126.12102    128.92043    129.58517    127.76709    128.14364    124.76124    127.92645     122.5554     123.5671    122.14743    120.13977    118.77428    120.64651    119.64551    118.48556    118.35207    121.85527     121.2952    123.29085    123.92042    122.86666    125.40334    125.10117    129.09997    126.07885    129.04162    130.71803    130.04059    126.74151    125.18652    123.17785    118.52322    123.12127    122.07254    126.62489    125.76745    128.88116    124.59959    123.94261    125.93434    123.65967     122.4998    122.63479    118.84245    117.58292    124.15379    120.95974    120.64847     122.6445    120.91201    121.97281    123.71789    126.50017    126.03637    128.23961    127.87766    129.46517    126.77305    131.36854    129.96959    130.01548    134.09551    131.84587    132.04991    134.40303    137.45375    132.70327    135.95377    132.64265     133.9068    129.04611    134.71617     132.7835    133.87328    133.02309    137.52837    135.75664    135.08901     135.9285    136.66273    133.40395    134.67813    132.76161    132.93855    135.86584    132.64284    136.18118    135.63903    140.46898    140.74063    145.38827    147.20793    143.59869    147.31682    144.17839    136.57293    138.00106     135.7419    135.72427    133.89646     135.3262    135.05328    133.14763    134.86889     130.8861    133.53648    134.60316    134.02327    130.31563    135.42085    134.61012    136.91682    131.67754    130.67306    132.32355    132.12423    132.28278    131.70008    132.72198    131.68113    128.40996    123.93078    128.52397    126.17205    128.90991    129.95536    126.11878    127.43702    128.66255    130.63965     128.2435    125.99602    129.24991    126.32486    128.57367    128.43704     127.6372    130.43219    130.48475    129.75455    129.14291     125.7013    133.28876    129.13324    129.45662    127.29053    131.11361    131.54042    135.24802    133.99497    130.11729     129.8659    131.30096    130.89371    129.88262    127.05196    129.53298    132.92121    135.85374    135.27466    137.73087    138.24961    137.37938    137.64797    135.50433    137.71826    143.93928    140.94625    141.18652    138.53561    142.10892    139.14394    145.38342    145.71587     139.0043    148.53669    146.56555    142.85683    149.73157    147.47714    146.55737    155.45568    148.70089    150.46819    150.17421     151.6767    158.65156    155.45838    154.17103    155.78325    154.72554    160.08183    161.39636    153.23634    157.30491     158.0093    161.49734    157.51415    160.79899    157.83609    160.95798    159.60475    163.63362    164.93981    165.48937    161.25207    165.19259    162.06501    161.18312    161.45869    167.02012    175.34904    171.97732    170.67371     167.6632    166.04898    173.76453    175.14652    172.01941    166.97479    175.96522     172.4902    183.12218    181.58887    179.99332    182.63891    177.55365     174.3127    182.82228    186.78604    186.33657    190.98914    186.07224    188.53852      193.752     197.4756    186.66028    187.00195    192.37884    192.20643    191.83352    196.68201     205.9878    201.61544    205.31665    205.84239    203.65944    204.25441    215.80971    213.74485    204.01735    200.46146    199.46252    212.47032    218.19596    200.87517    213.57134    215.63142    217.24306    215.30341     211.9276     219.7379    216.19904    225.61155    210.61231    228.46825     254.1739    233.13086    234.49231    234.21403    236.60853    243.56949    260.02795    272.23249    241.82837    239.86612    259.22501     257.4032    268.60055    267.19184    257.07977    259.81878    257.84414    277.17257    278.41769    290.30222    277.34445    271.10924    297.72144    299.03512    299.86335    320.79893    312.14346    331.50584    293.37654    331.02247    340.32825    379.39157    387.17005    364.21708    357.49826    371.67834    336.79042    502.97395    422.46261    385.15288    416.67557    419.28737    435.05837    619.00551    638.74964    610.99213    689.95156    458.37662    671.70916          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       15.50    271.24817    247.75137    245.46149    225.93344    228.24697    219.59872    211.84003    205.72281    199.15464    194.40725    192.83147    187.25769    193.42147    182.73802    185.47086    179.26306     175.3747    178.69738    176.15847    168.94277    170.25225    166.01359    169.61163    161.80215    169.56057     181.8378    201.00974    213.20207    193.98047    174.91722    163.56314    156.58993    155.06167    155.79166     152.1832    147.10348    147.92228    148.11351    148.76965    146.28947    149.46332    143.28013    146.81227     141.1171    143.70991     141.3194    143.69687    142.10257    140.19464    139.32583    135.66292    140.23779    139.72071    141.94605    138.12723      145.008    154.50675    172.25661    182.82409    173.95507    162.03153    146.83825    140.78802    135.76278    137.05464    134.01883    131.80583    137.60429    132.87441    139.89739    150.50235    160.22408    167.31043     168.6431     159.4764    160.51395    191.08553    221.64039    242.30548    234.66057    209.71355    178.42743    161.12824      146.576    142.42095    133.09839    129.96275    130.27714    126.16954    127.19602    131.50695    139.06886    143.89909    133.84187    131.61334    129.03311    127.42907    128.94485    131.53754    125.94872    127.50952    124.54876    125.14551    125.68018    122.76847    119.87142    124.31678    121.42543    117.84548    120.85049    116.82759    119.45376    117.37106    116.12914    113.77554    122.83323     126.0134    130.49258    129.05138    126.00561    120.85343    119.57676    121.83361    122.30117    125.44892    130.34401    136.38755    131.06306    127.77189    127.63936    122.83775    116.29786    121.77118    114.79072    120.13716    123.15264    131.92488    140.71338    153.55868    160.08783    163.46045    162.09867    149.93057    138.66438    129.15708    125.54388    124.50231     121.0739    120.92695     116.9001    117.95159    119.03946    116.86499    114.40551    114.52207    114.55993    119.09162    119.30271    119.58892    119.89699    117.82902    118.45166    118.58313    120.69339    124.07091     124.9521    123.81908    130.89601    135.22446    137.70499     134.8312    130.52596    129.12653    122.07141    124.99253    128.54433     129.5337     132.0566    128.66587    128.02379    123.05689    126.21878    129.06886    124.49589    122.12964    120.90421    119.50256    120.31528    119.15797    118.90712    121.90747    118.66565    124.33047    121.82624    125.36042    125.05474    125.71117    127.64387    129.92155    127.55198    130.53398    130.17155    127.27534     123.6583    126.56294    120.11385    120.84465     124.1445    127.54993    125.30824    128.37594    126.92544    120.43199     125.7276    122.76132    119.91542     119.5193    118.85716    119.22423    120.81659    121.33717    120.65647    119.92314    120.53792    121.79182     121.9693    128.92941     129.7692    131.32071    129.44374    129.53638    124.54557    127.20464    128.42292    132.22749    130.69798    137.11693     131.8421    134.77327    134.05364    135.45387    137.13583    134.19665    136.12436    133.64859    133.16055    133.22711    130.44943    131.52386    132.28595     135.3314    126.84234    133.98114    131.53986    134.68815     133.3555    135.35537    133.44684    135.70123    134.99821    137.29456    136.13986    142.33718    140.08558    139.97356    145.17717    145.30964    142.49022    141.97604    138.50053    136.52783    138.03553    133.52987    135.32749    133.11343    136.19207    127.87117    135.79854    133.91658    134.96463    139.35148    137.23061    131.87823    133.41628    133.40327    136.04004    135.07944    130.37112    133.52315    130.78158    133.58611    132.16804    126.39008    128.79901    129.57836    126.34027    129.92342    129.47687    128.10697    126.19628    123.20739    125.38115    126.65154    130.56739    129.81827    130.80475     128.9201    128.55593    126.11594    129.57736    124.72829    130.23266    129.90363    129.34042    127.17611    129.88408    128.32099    133.96239    129.79752    128.97441     126.7754    130.42425    128.95157    129.51374    127.38314    125.62551    131.55415    129.51584    126.49105    129.04353      130.466    129.97685     133.1736    137.26416     135.1954    138.77123    135.86297    136.04929    139.94929    134.52186     142.3052    141.84618    135.24375    134.41564    139.17022    140.70194    139.17481    143.55966    139.93948    144.49727    146.30646    146.77329     143.1613    148.16726     145.8121    147.78243    150.65113    158.74145    151.96993    152.70421    152.35022     156.3866    158.08195    154.95666    155.75368    151.04545    154.47536    158.25073    150.50512    160.00349    153.83821    156.41287    151.56624    158.86683    162.19069    161.10083     161.5461    160.84136      166.017    164.05482    165.88813     168.1661    163.17106    161.29919    171.54523    166.77957    173.08057     169.6816    167.63369    171.31579    174.56866    171.22249    174.30599    175.65868    173.49593    171.79243    183.69759    183.34637    181.48945     176.9523    183.61357    177.65312    188.39299    185.42976    182.68371    193.44228    183.33585    191.18586    187.87344    204.22148     194.5071    190.95652    199.25296    191.04762    204.94644    208.80233     207.8633    201.01662    199.99673    203.29387    197.50934    210.56034     213.4071    195.84815    203.10363    195.34068    217.86748     219.5173    213.85822    219.33144    215.44119    212.03451    205.21697    214.78264     212.3571    215.57025    214.13434    226.89352    212.98732    237.11432    231.05702    230.42402    239.29633    236.86451    228.18863    263.77697    253.68279    233.56599    241.77913     246.3845    253.56131    243.76522    261.20544    266.21228    238.00123    273.84815    265.81429    263.83151    298.31372     292.5566    280.13413    304.11272    296.82464    288.59507    331.37947    361.49582    302.28846    300.24372    319.56716    319.35458    327.41491    390.60834    375.79844    378.42528    391.02412    369.91977    358.10453    445.26019    502.63391     425.6073    479.68758     464.0705    510.88452    544.05726    645.82782    528.67649    640.61347     586.8574    578.88377          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       16.00    268.66222    246.82623    242.98328    227.15799    225.17421    219.51641    213.08769    206.39707    199.84586    194.76769     194.9354    189.88762    190.38391    184.58437    190.31351    180.83357    174.96237    174.00685    175.25118    169.47849    167.97553    169.68945    170.66115    158.05064    167.66596    186.01926    198.07891    209.90801    196.99798    174.89648    162.38268    159.09225    153.56057    157.97924    146.09881    147.81607    150.45835    149.33368     145.9176    146.91739    150.73394    147.88623    143.08719    140.92133    144.72735    140.82295    141.41734    146.19291     144.0617    139.44359    140.87577    145.09983    137.48931     142.3178     136.9693    140.60044    155.74606    168.89853    180.87417    174.79621    164.50724    147.77687    138.18915    135.87373    132.84852    131.16148    132.74812    135.63249    137.11592    138.78117    148.28899    159.76068    165.98294    168.31925    157.26136    165.41849    193.77987    230.10295    252.06564    244.53308    211.01609    179.05954    162.07433    149.76756    143.00026    132.09074    130.32028    127.36295    125.67321    131.57468    135.48299    140.54707    142.78225    135.98268    131.80041    129.92046    130.28707    130.78858    124.58026    128.18975    126.23472    125.46034    129.28209    125.53973    123.56662    120.50587    116.04664    119.48051    115.43401    119.83094     122.8789     117.2103    114.96281    117.57889    118.77719    120.62167    124.37703    128.91316    126.65905    126.86285    120.92949    118.73797    117.37474    122.36254    128.75678    134.42944      136.823    134.43664    129.38507    124.79009    121.65923    119.30642    117.14295    118.01135    119.56112    123.63485    129.38527    140.90072    150.25072    161.56914    161.87563    150.67091    149.16821    140.23905    131.49342    121.55839    121.79658    120.17455    117.35356    119.64287    115.85011    113.58163     119.1559    113.88666    114.07915    117.97636    116.42983    117.27671    118.87844     120.4326    120.30683    117.25093     117.9207    124.06895    123.00583     123.5223    128.06076    125.28249    129.30284    132.89273     132.6202    132.84452     133.6842    131.16028    129.00727    132.21685    134.14189    131.81047    131.04791    129.58938    127.91551    128.48296    129.68091    126.10744    122.94417    120.19099    120.87464    120.85647    120.66239    122.89112    120.93552    121.58811    127.98788    119.29039    125.77708    123.62122    125.61265    130.85204    130.25601    123.38113    130.59433    125.67914    126.77994    126.66813    123.74718    121.79669    119.79272    123.95968    127.11779    123.87176    126.74316    124.74982     121.8107    121.30156    125.76224    120.83812    123.00023    112.74833    122.51537    122.07788    118.49695    122.06564    118.34974    121.31325    126.75974    124.26436    128.07855    127.85753    126.06778    129.31055    127.95191     127.1092     124.8292    129.58727     128.8412    131.68641    130.38007    132.95522    128.52989    132.36843    133.43044    134.39246      134.029    136.59993    133.77844    132.25229    132.76698    131.52687    131.88991    131.73912    129.55917    130.43166    134.36799    135.32441    137.50444    135.74874    133.44817    133.61693    134.55568    132.69163    135.27139    135.72589    139.92221    137.77594    137.93111    139.51756    142.09645    141.96123    139.42987    138.49723    136.77783    137.14317    135.09644    135.66803    136.64026    134.64815    133.25001    136.41749     133.3601    135.38159    134.10638    133.34236    133.83987    134.38687    133.32981    133.05937    134.33076    134.46104    133.89396    128.71505    131.19691    129.82185    128.82552    126.94402    129.78526    124.37795    126.77727    127.13476    129.02857    129.07344     128.8933    127.40584    127.25025     128.3252    127.64066    128.05678    127.62622     126.3841    125.14462    124.95601    131.64559    127.52494    128.11484    126.62186     125.3245    130.69973    129.96112    128.55487    131.57316    129.31944    125.68836     126.3361    127.75993     128.5969    130.37712    126.69421    126.17267    129.23486    126.30521    131.87443    129.39801    133.44435    129.23242    130.22919    135.82775    136.40333    139.34808    138.04949    141.05901    137.48555    141.65465    139.62855    137.72975    143.15224    138.88692    138.28926    142.87795    145.71883    145.74626    146.49158    145.04525     141.6576    150.41543    150.64161    148.76418    148.82486     151.1027    153.46899    149.68989    160.86299    150.60069     158.3287    154.64342    156.50127    157.56548    154.83675    155.94949     159.5502    155.11871    156.93673    157.86981    158.48092    160.96016    162.89792    164.75607    157.91457    163.66726    162.35879    159.36379    162.05639    161.98078    159.54632    166.32467    161.39829    168.16252    169.67421    178.35436    163.15057    163.75757    171.33766    178.77575    166.61974    173.14235    168.40933    170.81441    177.91525    179.07433    180.00457    182.98535    185.47396    175.02064    177.85207    191.99933    190.55697    182.84556    189.57483    189.97113    185.53495    191.01422    187.64725    194.90506    194.69335    198.28553    192.55797    192.89857    211.55522    211.71495    207.43416    199.24817    198.16632    202.47328    209.32203    212.54676      196.895    206.16468    199.75125    200.96119    212.82599    209.77971    216.08906     211.2561    215.64562    212.67792    224.34835    235.21799    227.38134    224.85496     219.4945    224.98715    225.10054    240.06099    237.93717    221.08258    248.61786    227.28327    226.95743    259.58726    234.24493    247.26302    256.25029    239.87574    244.87594    268.03528    269.00527    257.79752    268.59277    274.06596    281.17149    295.36564    288.60314    291.34267    296.27491    295.20594    297.26785    309.36768    261.13371    347.27222    346.97198     360.0127    336.20897      370.741    388.13775    416.32554    374.03075    404.43626    350.47915    367.08807     416.2355    376.27104    460.96324    520.39625    481.90664    541.53693    607.10691    564.50168    534.17914    563.07824     638.2915    585.48129          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       16.50    264.95964    246.98716    240.03301    228.02162    223.11022      218.512    214.59117    215.13045    202.37154    197.62174    197.05219    193.42779    189.13704    186.41909    184.60097    182.37167    176.60362    174.17599    173.44609    173.18335    169.97281    169.11569    167.55098    160.76384      164.839    185.15536    201.33877    211.00918    197.52349    175.72382     166.2036    158.81941    153.77208    157.21482    149.03853    147.38389    150.69163     146.9096    150.30688    151.91448    145.05484    147.92774    141.85862    140.48584    143.79144    142.38486    142.26175    145.63762    137.94987    143.13842     140.8719    141.02398    137.46116    141.07848    138.08286    143.57441    155.01186    172.80369    180.20502    177.87201    160.43499    144.34562    138.50246    138.74609    137.05826    135.47684    134.08516    135.36982    138.12435    138.03419    151.79083    163.62442    168.33009     168.6008    159.26404    164.39566    198.55205    240.20511    258.43594    243.10205    212.79997    180.64953    166.53943     150.9636    142.92414    136.87664    130.48361    130.09723    129.38183     127.0562    136.47317    141.25307    144.08582    134.21689    131.12255     130.3013    130.43452    126.94953    127.16269    121.72752    126.03217    124.10609    124.88111    125.88355     119.7128    118.97645    114.78501     118.9993    117.18128    119.48207    116.74561    118.32278    117.87111    118.53317    117.81173     117.8649    122.70307    128.75827    125.66051    125.93874    124.58742     119.8859    120.01874    123.79744    127.98198    133.65244    133.37072    138.75751    129.95031    128.07583    118.82727    121.64541    119.37356    117.65825    120.39611    122.79912    130.64264    137.58807    149.82961    161.06087     163.1568    154.55688     148.7734    138.75658    130.70391     124.4549    120.73726    119.51256     116.7839    121.79588    120.57537    120.56808    114.10388    117.98236    118.16168    115.44377    114.74177    117.92219    119.30649    118.38125    120.95757     117.3526    121.21615    119.93987    122.04897    122.64368    123.16565    122.55618    124.59371    130.54648    136.29976    134.10205    133.22966    133.35596    128.27631    135.43457    130.54733    132.69261    131.12836    133.31002    132.65997    130.46392    128.37166     126.2751    123.86416    122.00834    121.57987    120.09226    122.62069    120.86445    122.02929    125.02178    124.60017    120.75069    125.23845    126.14163    126.05612    128.07657    129.56409    133.26577    129.68947    130.60037    126.35052    125.81986    123.41469    124.79777    121.30866    124.76634    128.31579    126.80613    124.04985    127.94208    123.60159    123.17951    121.10657    122.15307    120.96082    122.53221    119.25743    117.38873    115.77317    118.99768    122.59206    122.56297    122.50147    123.01809    121.99333    127.81431    128.25578    127.11013    128.47249    127.39001    126.83994    131.53693    129.09772    128.48189    129.37944    133.18024    129.77918     131.0534    132.39404    133.62678     132.3174    135.86231    136.39933    132.23378    131.00224    131.33337     128.8625    128.73592    127.64844     129.6011    131.50649    133.00669    132.91679    135.92749    130.01049    131.40544    131.61855    132.71096    135.39932    140.73185    134.60724    140.68044     137.7459    141.87323    144.63256    142.59192    143.96461    140.95909    133.12726     137.8349    135.75832    135.69076    139.04866    139.85259    133.85574      133.163    137.69391    134.62791    131.67269    131.17533     136.4877    130.48815    132.46153    134.71034    136.17862    130.26695    128.24088     127.1475    129.12821    127.41147    126.19371    128.21151    125.52286    126.28045    126.12129    127.03031    129.32954    128.78182    124.31172    128.53546    125.29215    124.69962    122.49402    126.16152     127.5666    125.33282    124.63559    127.13793    127.80501      126.584     128.6887    127.88755    130.54684    133.80025    132.88257    128.80505    128.86635     129.9305    125.92036    125.64712    127.00511    126.27442    127.61634    130.19044    130.56271    126.36085    129.91604    128.97856    135.22644    128.98066    133.60241    133.57044    133.59671     137.4976    135.62563    135.59968    137.36902    136.26158    144.44928    139.84034    136.36686    137.56948    138.93075    141.33798    135.88634    140.53583    143.45723     142.5929    145.07532    145.61472    148.64743    153.14422    150.98381    151.43455    147.25877    148.61779    150.96375    157.48287    156.10072    158.22629    155.19053    155.42143    157.46674    156.33045    157.30166    155.64046    159.18255    155.30324    155.55324     158.8953    155.01872    160.48157    162.37292    159.15318    160.00355    155.62389    160.56197    165.78275    168.35377    168.26955    162.51372    160.45551    176.30656    169.50268    171.13575    171.05365    163.55829    168.00375    176.54563    167.24008    172.16072    171.78261    178.83997    177.93331    181.25142    182.58433    182.17576    179.88053    177.85085    178.71969    190.25499    192.27375    187.36785    189.12791    188.15965    187.06828    194.72787    192.30074    194.60147    197.34137    188.05769    186.59211    200.30268    200.56105    201.52447    210.60812    202.30222    193.32945    216.92737    213.69375    208.32138    213.49941    202.71683    205.48946     214.9206    223.54986    218.73929    221.70827    210.90499    216.52352    215.41611     232.5199    226.00359    228.03541    230.06629    242.35205     222.1594    227.69918    238.75517    231.22639    233.11657    231.85091    237.95394    239.72506    247.48008    249.67378    251.93582    242.08021    247.54971    258.36007    259.60005     273.4789     274.8866    275.69867    279.57814    272.78335    300.35266    296.12487    309.64557    298.96668    302.12936     273.6804    370.88768    337.19444    338.59789    329.05387    304.16491    329.81714    371.25888    391.75878    371.14924    377.54199    405.27271    374.72742    387.24648    441.36601     479.1793    406.33099    442.55039    481.50041    518.06188    648.79623    685.96241    726.11118    490.60299    499.66601          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       17.00    261.22659    249.58046    237.68116    228.71652    224.44259     216.6304    212.96694    224.44649    206.16615    199.86842    198.06312    193.14888    190.43058    184.08172    180.01073    181.73398    180.01077    178.50288    171.21104    178.57763    175.45372    169.08424    166.30055    162.51161    168.93437    184.68588    203.57904    208.88944    193.32539     172.9786    168.75049    164.05968    151.99324    153.67187    155.26152    150.85155     153.1673     148.7982    149.96381    149.59537     147.2162    154.44728    142.83757    142.93913    147.31838    145.45512    142.55374    141.74907    137.32764    139.34502    140.67814    142.03061    141.52291    140.51587    141.74755    143.27651    152.30701    172.44874    180.98129    178.33733    156.91526    149.04233    137.97966    137.21872    135.86716    133.37392    135.82633    133.59385    140.88973    137.18592    151.47108    165.72333    170.47432    166.70751    161.71273    169.46658    200.97844    238.67222    251.64916    235.92242    202.93981    176.13774    168.72321    163.36519      152.039    139.71705    133.73359    128.29462    129.82225    130.76358    136.18267    141.43966    137.58739    136.59687    130.31389    127.88838      126.254    120.87202    124.49302    123.99133     122.0205    123.88208    123.72828    124.08772    123.58252    121.53639    118.07238    119.93233    119.57489    115.58436    119.93338    119.34921    118.04011    117.02612    119.26885    123.83978    124.07161    127.07697    130.76855    125.10519    124.74406    120.63827    118.14742    121.87903    128.06032    133.16121    133.41402    133.99835    129.49529    122.38546     120.6249    119.22764    118.79635    120.33826    120.20561    125.90611    130.34341    143.61256    152.80648    161.01379    161.44814    153.98315    147.38389    136.97897    129.92825    124.31109    123.05065    118.18072    117.70231    116.99968    120.99413    118.42257    118.07065    117.16604    116.15228    114.28477    118.23555    121.71171    120.14441    120.97238    118.78026    116.14005    119.50476    124.43954    121.73882    122.83925    126.49578    125.95146    122.59135    127.66057    130.80319    134.01435    131.72551    132.01555    127.20157    131.80655    133.74355    133.13377    136.16281    135.46848    135.11934    131.02939    127.55622    129.06888    122.77231    122.09589    123.78852    120.03109     121.2685    123.19411    121.98965    123.34811    121.03779    121.53425    123.49318    126.60118    124.91507    129.34182    130.31075    128.58888    127.57571    127.34279    125.56128    123.19399    123.87947    120.81037    122.81798    120.69558    124.43981    126.00774    126.03454    122.93988     125.1493    122.10931    123.83066    122.33192    120.32652     118.2257    116.99718    118.58592    118.58366    119.16799    120.93465    122.17506    123.47677    123.29456    121.79872    127.58502    129.91165    125.95829    128.41183    129.13864    126.30535    125.90421    127.65913    132.44032    133.14383    135.60098    137.84256    135.35192    132.41876    135.98033     137.0069    133.75452    136.88847    130.52151    133.17378    127.14093    130.68267    130.40388    124.87348    126.15225     125.8471    127.96033    130.38882    129.81041    129.24584    131.34713    128.42143    133.14154    133.18228    135.77555    134.24687    142.11628    142.75143    141.25674    142.85125    143.07852    138.77571    137.56545    135.01697    137.08843    131.79466    134.16411    137.16848    134.80435    133.63888    131.04061    135.52826    133.64233    130.26371    132.91811    131.26695    133.21844    134.02296    134.88222    129.49256    134.35221    129.62704    131.96644    126.99957    129.74759    127.12176    129.30672    130.06936    125.88775    125.59119    127.51643    121.58337    127.79629    125.90982     127.4434     126.9153    124.87619    125.87425    126.98089    127.63252    125.36323    125.46325    126.22761    129.83443    130.61891    128.87776    126.77864    125.85619    129.47817    127.66605    127.48458     124.7679    127.58857    128.33627    124.09901    126.24146    126.05472    126.24251    127.71488    123.53738    125.05865    128.19267    129.98637    129.27077    128.75984    133.31613    131.06725    134.36626    139.60167    138.54774    136.48238    135.56353    136.92964    138.39575    139.77818     143.3589     139.8425    141.21181     142.7449    139.82668     147.2521    139.74152    139.63136    146.34065    144.38309    147.79393    151.08923    151.96031    148.03797    149.66169    150.59752    152.33397    153.98679    154.65365    154.77112     154.2962    157.38764    154.87653     154.7539    160.93102    155.14682    156.88471    156.39222    156.10366    158.87789    157.16589    155.84913    163.47458    159.35455    157.78555    159.87839    165.40135      164.446    162.93104     168.0642    165.33564    175.23716    171.56903    172.77819    168.34079    168.14977    169.39307    169.17463    173.34711    168.21827    172.41726     171.0849    166.77008    177.96309    192.74887    186.13935    181.97926    180.70608     181.7538    184.29943    189.29432    186.15836    191.35365    191.23688    186.54136    187.41687    199.03151    194.68029    202.59858    197.49025    196.70338    197.55954    208.27149    205.37198    219.10744    202.26996     202.8649    205.78301    207.15001    218.84602     208.2833    213.13201     211.8804    209.03931    210.41978    225.07819    210.05575    226.91446    211.16218    226.97612    210.50449    224.57434    222.83851    230.63555    228.85799    223.98607    219.20495    231.14569    239.51458    248.76527    248.49555    239.88513    231.48165     252.9723    261.26364     262.6559    254.37854    258.69337    240.00894    229.80439    278.01615    275.73589    277.17591    275.37764    278.88001    264.42725    318.31531    297.80076    285.80574    304.69844    301.65742    308.39337    325.25235    349.70513    337.57584    364.68945    344.51746    343.20257     412.8743    431.16806    391.40252    402.02282    379.92463    365.75983    477.57768     437.1439    516.15724      433.806    471.29592    483.98298    537.22364    634.90064    713.36215    545.43765    520.31898    602.25672          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       17.50    262.92953    252.28078    237.17966    229.81612    227.63723    218.19248    212.15907    221.44642    209.32691    200.21526    196.55034    189.19839    191.34232    183.04869    187.04308    179.15642    183.69368    177.44876    173.06133    172.81235    175.37702    167.48723    166.46842    163.33687    178.12706    184.47395    200.35841    206.56181    184.02526    175.33179    165.11204    159.87573    155.09158    156.96324    152.66144    153.61338    153.09705    150.35151     148.9426    147.52341     149.5199    146.42355    141.23158    145.97876     146.5557    140.29657    139.96874    144.51418    143.52885     142.6106    141.16634     143.1252    142.98908    144.19767    139.72145     144.5399    154.44271     173.2168    178.49851    176.74603    156.86941     151.2403    141.55727    136.63611    138.22411    136.04864    135.35152    141.87943    135.58898    138.67557    151.16833     166.0914     168.5479    163.80326    161.14417    167.90626    193.41044    227.51725    231.39661       220.57    190.96437    174.83613    172.02737    169.59832     159.7572    143.62472    135.57795    129.54204    129.26164    130.28527    136.30177    141.46419    141.11407    139.13711    129.38323    125.81596    121.24089    121.43278    125.00177    120.36768     124.6822    122.86333    123.47937    124.02181    120.79777    115.70166    117.56865    116.98534    116.37025    118.29754     116.7479    116.85279    119.58931    113.83623    118.34793     120.8672     123.1139    128.97159    124.30393    126.24143    121.74947    121.25102    116.93057    122.47886    126.78604    135.19884    136.49515    134.54691    127.09758    123.82902    120.92698    117.66249    117.64537    118.37818    117.06862    122.39231    128.70516    141.61371    151.93554    159.19797    159.80306    158.61188     144.5692    138.48751    128.26497    124.69639    121.17217    121.84353     118.1592    119.45496    118.50888    119.04031    115.54401    117.34149    116.06837    114.54462     115.5705    117.80012    119.58709    120.07885     118.5663    119.26124    119.05728    121.24427    122.58056    126.26851    126.22212    127.17565    123.91042    127.99148     129.7017    128.54792    130.71389    129.61795    130.88392    130.79792    132.44948    131.59656    133.85975    139.47248    138.03358    134.53606    133.30658    131.11007    127.49948    121.75846    118.92541    120.93101    117.52661     119.8366    123.08543    117.32751    120.85093    122.58956      123.155    124.71296    126.27431    125.80835    127.83467    128.08916    128.48267    125.82628    123.97294    122.23732    122.19222    120.91433    122.87041    122.54474    125.98538    129.48723    125.64733    124.74663    124.41176     124.2677    119.87466    122.59804    118.25345     117.8691    116.65831    115.24434    113.80936    120.94149    118.29796    122.65491    119.71057    122.68719    119.63358    124.71785    126.12924     127.0217    125.86349    125.99833    125.47614    125.73819    125.99886    131.62954    136.56562    142.35961    141.66894     141.0051    140.68792    138.01045    134.72661    138.41091    133.21757    134.36384    129.33488    130.25877    128.71614    129.93499    127.63466    130.86017    124.38162    127.71524    131.50174    128.22872    131.82571     129.7421    129.30787    134.19577     130.8394    136.78242    135.33194    137.40722    141.53595    140.37899    143.23709    142.12565    137.50489    136.78767    140.81688    132.39202    134.23972    129.33028    130.95949    133.94711    125.78809    130.76569    131.89849     132.4817    134.96346    134.29159    137.77088    134.69356    134.25912    133.05401    130.78145    133.99282    132.08195    130.91961    128.24186     126.9862    122.73247    123.96947    131.15157    126.39537    122.96076    127.73157    126.33722    124.59478    127.21272    128.56555    125.33869    123.15152    125.82485    127.13162    128.20995    125.17143    124.56137    126.89208    130.40654    130.15048    127.37466    128.73582    128.31197    128.71143     128.0197    130.31541    128.69687    129.56386    128.87959     125.4292    128.16607     130.0742    120.22097    126.72706    126.40636    129.20016    126.97796    130.03158    127.24386    130.30756    133.58105    134.93036    130.94864    136.16591    134.36893      133.413    132.34581    136.33067    135.84996    137.97467    143.24641    143.18451    136.57676     139.6402     140.4935    139.39568    142.32157    142.33336     143.2652    144.11431    143.00908    143.47493    143.53282    147.16792    153.21221    149.43668    151.38126    152.44695    160.66443    155.09193    156.33161    150.73605    155.29182    151.62126    156.73154    158.53138    159.73676     157.7361    158.11105    161.59773    160.79853    160.05484    153.99575    157.24644    159.87142    159.89599    163.58013    166.26756     167.1668    160.84581    163.91465    166.92115    162.03103    177.72658    166.39472    173.93568    163.78586    169.19701      170.644    175.31057    175.76904     178.3613     181.0136    174.15391    178.43273    181.98538    182.71359    186.43223    184.83469    184.49817    185.80632    186.95409    182.96616     188.9643    191.91352     189.2202    184.50295    200.43938    196.88277    198.16955    201.17742    199.14686    199.35961    204.15714    202.64233    216.82517    206.13046    202.52181    202.81119    218.72449    215.89103    218.07135    207.00159    218.27945    218.46019    232.84511    218.71608    231.17102    221.43676    218.66493    230.28993    233.35843    219.88023    228.54083    242.71531    221.32946     228.0334    243.96513    235.63938    243.37577    251.93919    233.83275    239.47216    274.53078    260.93194    252.64691    258.98696    256.82705    262.24486    257.76593    275.78669    280.47659    276.75765    287.53839    282.13525    295.00936    307.63736     320.1423     310.7583    311.86947     322.7869    311.15709    365.14982    357.20543    331.62667    322.17814    345.75693    368.86546     373.9629    441.93999    397.54195    374.98327    422.38341    415.64963    453.78541    486.85498    506.60567    498.49556    516.18317    501.23047    690.55231    633.01682    708.62657    740.81855    763.08132    696.26619          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       18.00    266.59157    254.88818     237.1939    232.60325     229.8512      221.827    214.29603     218.9979     208.5385    200.93036    196.31331    191.05974    192.66659     183.6878    189.77047    172.96022    181.09567    175.63811    174.06956    173.08666    173.60185    171.11596    167.18778    164.58575    180.29047    178.42971    205.99285    208.05483    190.90294    175.66067    164.96983    154.33376    155.29909     157.0415    154.39151    155.43356    148.24502    151.05366     152.4408    148.11699    149.75176    148.50794    143.16169    147.56751    143.90884    147.55191    142.70532    143.77486    144.04975    142.06737    141.63596     138.7168    142.72324    142.61724     141.7057    143.57919    151.44327    171.28075    176.91852    174.64119    161.96823    151.97865    141.38473    137.50092    136.13433    134.79655    133.76492    136.01994     139.2311    138.72511    150.55969     164.3773    168.89553    165.70499    160.47455     163.6654    179.81166    202.19899     209.6973    196.01223    175.70204    170.54943     173.0294    172.20585    165.19506     146.0519    137.12045    126.80294     132.7241    130.11708    132.35356     139.0707    140.95089    138.45493    127.89246    127.31333    126.06309    124.71179    122.28237    122.37643    123.07028    125.12309     124.2522    123.89073    123.77771    118.25363    117.97978    119.84416    117.12394     116.3382    117.44955    117.55021     118.2162    119.85142    119.48748    122.11976    120.05521    127.50134    127.85913    126.51592    121.74411    120.84242    119.92675    119.57077    126.32501    128.96788     136.4375    137.12924    131.88447    121.39425    121.65042    119.11617     116.8916    120.74637     118.8158    124.41929    130.28685    141.94127    151.71303    158.68226    158.82391    152.35879    143.61601    135.84238    129.87506    124.15147    123.14686    117.45937    120.56743    116.98019    116.40465    119.42009    117.63923    114.09809    116.26504    113.71268    116.30202    117.96134    118.67036    119.45993    120.39056    116.88599    117.13491    122.52449    122.45035    126.30149    125.06413    127.39915    128.32806    126.26469    133.29518    133.21104    130.31004    132.01018    130.62022    128.00023    133.23428    134.32396    135.98579    137.62415    139.41605    135.84015    129.20597    127.13684    127.88572    126.02713    125.40956    122.19854    122.31631    120.14412    122.80916    122.53549     124.7219     123.1509    119.21832    121.93569    125.83083    125.40189    128.82604    126.40644    129.12865     124.6392    126.45269    124.63037    119.79878    121.00987    124.84374    119.57037    124.41716    123.51702    126.65798    122.71662     122.6588    123.27212    115.24158    120.56108    121.07146    121.55925    115.05921    115.54164    112.14667    119.57875    117.18357    115.57166    121.02083    122.97855    120.73479    119.15893    128.50037    129.73698    126.96473    124.78876    127.41124    128.42565    129.60833    129.16049    131.24832    134.90723    136.27134    138.04345    136.89459    140.67622    138.27425    137.07293    137.07426      136.244    129.62487    128.49109    125.15815    124.45005     125.7867     125.8432    124.95621    124.61004    126.67689    129.00791    128.21083    127.54101    127.90927    126.67749    133.42292    137.05178    139.16041    136.67966    138.83936    139.90006    139.17303    145.47216     139.6023    135.99449     137.1978    131.31081    137.29649    130.33679    130.06569    128.16344    130.07823    128.27932    129.94696    132.99804    135.23931    133.27707    136.93658    137.44043    130.28446    131.89953     129.3485    131.30868    130.95998    130.81747    125.59394    130.15104    127.77219    128.98093    127.42858    130.54099    131.21163    127.16454    126.10626    126.33012    129.69853    128.72072    129.29031    120.88304    122.54231    124.28711     121.7678    121.25452    123.30717    125.72358    127.12494    127.61622    128.04071    128.66104    127.86258    125.17153    128.39131    131.63487    127.39604    127.22792    129.59127    127.46768    127.31643    126.56328    127.99714    123.62509    124.54985    127.41293    126.43806    134.05363    127.57775    132.29448    130.26148    132.99489    130.77043    138.35246    133.63066    135.51666    136.12777    132.36408    136.78826    141.67351    137.65559    144.64853    136.97343    139.15588    139.76663    141.62723    138.12474    141.95609    143.36658    143.55705    144.16998    145.82179    149.04558    155.00943    147.64883    154.11998    155.02132    153.75071    154.60967    154.88726     156.9709    154.16633    156.87702    154.48108    156.15408    151.65662    160.06072    149.57711    157.10124    157.38017    158.81955     164.9141    166.82412    159.04634     158.5155     154.8996    166.12695     166.2256    163.21277     168.8866    169.77094     166.1373    167.72439    173.99916     173.4266    169.21165    167.44821    170.88774    170.11123    177.93986    179.83895    178.70248    185.00785     172.7519    182.47597    185.51068    181.22696    182.48264     189.2106    186.10478    185.74062    194.30679     195.0748    190.76224    188.66154    196.94272    193.73979    203.54611    204.61137    197.64642     199.9771    199.19049     201.8456    204.96714    211.42701    209.21574    208.04185    210.49921    206.31621    219.72498    215.62567    223.13113    212.70752    214.78986    210.22616    231.92073    227.73758    227.59486     228.0635     228.9437    221.63152    237.36264    228.10401    235.49113     230.6843     231.0634    227.76375    251.01408    262.06794    254.10917    250.82422    245.40043    234.61545    271.67026    281.50327    270.28606    261.74353    279.42506    267.49015    271.72336    285.13736    281.41584    292.62838    290.62387    283.18938    288.90928    330.15999    305.43311    338.48567    306.65993    306.96787    309.01929    356.18482    391.24789    364.66634    372.49128    350.42714    357.15797    430.56012      442.553     422.2741    425.02475    455.97927    443.77478    550.16661     586.2787    523.95484    531.91568    543.67709    561.16421    813.71451    912.88435    828.20053    785.31291     873.5397    765.65713          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       18.50    269.85615    257.90023    237.90703    234.40735    228.37224    223.49189    216.50466    214.81363      207.838    202.14825    199.20865    196.70817    194.41409    182.42309    187.42874    175.16453    179.90223    176.58744    175.95279    177.42224    171.83202    177.03499    167.68741    165.94727     176.3778    185.01322    203.80212    209.91685    194.50613    174.13158    166.98072    157.95862    154.11631    159.37199    156.98971    150.41947    148.31105    148.40218    156.87901    146.76406    148.04917    144.42166    143.95513    143.96582    143.27249    143.14221    141.28442    142.70775    141.69998    138.87825    139.99087    141.41993    142.15397    142.25157    144.06178    143.06644    155.45544    169.64578    182.09934    172.92069    158.31008    149.21862    140.09902    138.42227    128.25433    134.79975      136.896    138.75483    138.11077    145.01457    150.15311     166.6667    170.93123    164.80745    157.02799    159.46843    175.34044    187.93102    186.37831    176.84222    168.46051    165.82148    175.48506    175.18846    166.96632    151.18973    133.61356    128.77944    133.26382    132.83822    135.26354     140.4062    139.91792    137.73972    129.63314    124.87843    120.56703    122.03989    123.79888    119.82629    124.18334    126.22847    124.34341    123.87839    118.31127    120.77298    116.50322    120.86385    118.58484    113.50436    120.38939    119.88805    116.63311    121.42147    120.20392     121.1655    123.29603    128.50908    128.56709    125.66398    121.74792    119.67734    122.38383    121.54868     127.6072    133.16025    135.54672    136.80292    129.75749    125.00164    121.75386    119.44224      119.221    118.72638    120.74227     125.6043    123.46854    145.08889    150.82304    159.32598     160.6934    154.81283    144.53241    135.44005    132.44313    122.76307    119.40795    120.68113    118.10334    114.94216    120.07709     117.2602    115.84349    113.48507    117.38072    115.58602    114.52068    117.73057     120.4155    119.10549    118.69029    122.90263    116.76161    122.25312    122.95341    122.78713    124.71998    125.01611    130.51074    124.48757    130.39081    124.38768    127.78418    128.58927    129.69339    126.07631    131.06695    134.64611    135.30384    134.14211    133.85895    134.51668     125.9806    130.01893    126.71925    125.48074    121.54965    121.80435    123.13384    116.00635    120.50542    119.90979    122.27708    120.29171    126.40062    122.98364     125.4815    125.44273    128.71847    132.90681    127.20484    127.21603    127.20899    125.10054    120.35076    121.80756    119.05972    119.97189    119.77868    123.12883    125.71847    122.11714    121.97318     120.9822    117.73419    118.87041    117.45754    114.86214    118.34747    119.68084    117.26477     114.5965    122.12803    121.97379    120.05503    122.88351    120.55749      120.608    124.61715     126.7948    126.80928    127.86735    125.66024    127.09787    122.60628    127.02235    127.08366    130.29913    130.14406    133.08723    134.98012    136.01175    131.12624     131.4649    132.72168    131.43874    127.84217    128.81879     128.3513    126.36386    127.28653    125.66119    125.73968    128.64649    129.28809    128.62479    126.81023    123.86892    127.01404    125.38001    134.03727    134.79922     133.6344    141.57437    140.16176    140.54148    140.93176    142.57524    139.04517     138.5675    135.20994    131.31454    128.78822    130.43794    129.33511    127.22709    128.95335    125.90392    125.04208    129.43915    135.01687     134.6939    134.08236    130.95228    132.97696    133.64275    134.35211    135.10064    129.53003    130.30532    126.31373    128.17539    128.09927    127.00768    128.38709    130.49871    132.37434    120.20686     132.9734    128.53376     126.5285    125.10808    127.34188    119.23755    125.46537    124.92204    128.42671    124.13789    125.24966    125.01972    126.32852    125.74651     124.2807    129.42316    129.07697    127.31401    128.13043    130.07678    130.99242    128.56417    126.39805     123.4504    126.98732    127.42367    125.08302    126.76033    125.57341    121.06673     126.8756    128.30989    130.25593    130.66664    129.61184    130.88726    135.13857    134.73044    137.32963    135.46339    130.74837    135.91313    135.20416    138.47437    137.99309    143.26599    142.58366    143.03996    139.29685    141.87289     143.2811    140.83849    139.53719    145.81926    145.96971    147.98217    148.11898    146.80628    150.19411    152.48688    151.88799    147.56601    157.67252     155.4929    150.31589    153.53202    156.83164    157.63432      157.141     157.1804    158.59581    159.15188    156.04321     160.0632    160.99368    157.63974    159.18037    154.28199    157.54367    163.25177    165.83668    165.97503    167.11856    164.14587     168.8938    160.92106    160.93819    171.24126    173.10355    173.29014     174.1631    167.13947    176.12593    179.65273    175.48011     176.8303      178.576    173.88091    183.24011    179.61835    185.47943    193.75989    195.65908    181.62163    189.87554    197.66943     193.4438    195.68599     198.4808    190.40673      182.667    197.37918    200.16261    199.36098    202.52361    200.66713    194.90286    213.17665    212.54632    208.68019    198.20981    208.05828    215.57845    229.07873    224.46741    215.79504    217.99055    221.79192    223.65903    228.40684    223.73497    223.96605    222.17525    223.93572    223.54699    238.56139    258.57765    241.02266    244.39997    237.69099     247.8569    244.87782    255.10021    253.46737    247.23052    258.49742    264.56471    255.01256    294.44869    264.77202     281.0308    265.80948    253.25891    274.24794    290.68772    302.29508      313.733    290.56965    300.69002    302.03584    316.59198    333.70792    324.84065    333.24449    356.86862    356.18178    414.37867    399.83877    360.68086    368.89822    373.85353    387.31641    459.17532    454.16828    442.29201    500.49794    465.91546    437.55355    582.77519    589.40584     578.4867    559.47704    576.36672    549.58232    741.41012    897.68889    938.62469    897.27078    939.44683    862.97031          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       19.00    267.87867    261.56199    239.40665    234.70898    228.10174    221.81453    217.24592    211.20582    208.70516    202.32161     199.8689    198.27481    194.41304    182.13361    188.29318    176.94279    175.55672    175.72073    177.95805    180.28972    170.46575    175.49765     166.4218    166.59516    174.78267    186.67282    198.78107    207.93597    196.50129    176.83772    165.83201    159.48759    155.99299    156.45706    155.53253    154.60654    152.99649    150.95209    153.72345    144.93756    154.66534    146.86082     146.5016     144.0042    142.71571    143.88387    144.41305    141.72014     143.9219    141.47817    140.01584     141.2518    139.99318    142.81238    139.40502    145.45952    155.47696    166.29132    180.33875    175.01581    162.63872    150.81891     142.5418    132.57615    138.33846    134.58372     133.1349    134.94564    139.63659    148.12138    155.94245    163.96468    169.36313    167.45385    157.95358    162.06302    170.60364    176.25452    183.51288    172.86481     161.2507    166.15033     166.2978    170.69767    163.12173    148.33336    134.72542    132.13628    128.69026    132.26152    139.57427    139.22667     144.6727     138.1127    128.14288    128.36615    120.44227    119.68923    123.05538    119.28245     120.9801    126.70958     124.0315    122.36516    120.80332    121.18048    120.20395    116.93334    118.04755    120.06937    116.94862    119.12403    117.93299    119.07754    118.11853    122.26793     122.1437    125.05693    128.47122     125.6019    123.70235    121.25636    121.99008    117.41686    130.90431    129.38072    132.64635    133.63649    129.68886    124.79649    118.81957    119.82406    117.29568    117.94526    118.33084    122.08294    129.49469    138.28508    147.44996    161.00891    158.94753    155.58934    146.90749    131.67236    128.34625    121.08037    121.34173    120.32488    117.29304     116.7468    115.39372    114.96173    115.39024    115.26561    116.07471    116.10851    113.85183    118.84357    119.64483    119.91905     120.3464    120.00424    119.38417    118.34609    121.48595    124.97671    124.94696    129.96221     123.2235    118.91316    123.26091    122.28096    121.43586    128.66331    128.62168    129.39365    126.64699    131.73544    133.30989    134.12197     130.8207     132.0834    127.50481    122.24134    120.23046    121.55408    120.47568    119.27803    122.60274    118.69303    120.79092    121.25947    121.13537    122.14607     125.1961    122.16158    124.60624    127.63543    124.98382    129.16238    128.56539    126.09526      125.566    125.09273    123.85941    120.45915    119.97696      120.122    121.75775     125.0863     124.7513    120.90394    121.69583    121.43123     119.4924    119.63724     115.3034    118.26206    117.26906    116.19676    116.74655    115.62975    118.89396    120.60828     124.3477    121.88983    123.30219     123.8106    125.36999    125.57335     126.3097    124.29554    126.98346    127.55046    125.15723    127.96598    129.72651    128.52366    125.97539    131.73649    130.00106    130.23641    133.41734    134.52672    126.40242     127.0512     127.0953    124.81027    125.75789    125.23467    123.36308    128.06037    125.77384    124.25073    128.34065    126.46588    130.79903    130.11664    131.15995    128.71057    133.11811    132.53637    136.74777    135.72705    137.99692    143.18285    140.31659    143.29343    143.08578    139.14838     136.1104    132.52752     130.8422    131.28963    126.86766    128.11811    128.97737    130.47598    127.56377    128.61815    132.31563    129.52157    129.20046    131.42071     133.1007    133.28179    131.74048    130.53335     130.9909    132.16544    129.07146    123.29882    129.08021    131.57002    130.10193    127.44996     129.0655    124.73264    129.54074    126.33943    125.96496    127.15974    125.93172    124.62517    123.89044    126.30607    126.20709    122.60548    120.57427    123.94272    121.73668    124.50056    126.23692    126.72772    127.43474    122.71781    128.05219    129.36609     127.7287    130.85476    125.81116    130.95269    124.94948    129.51116    124.77019    126.63506    125.94952    122.44664    128.75899     127.8215    130.57603     130.6107    132.88466     129.9887    131.09774    133.91755    135.33531    133.69168    134.13626    133.56362    138.17151    142.78643    142.79956    141.91979    141.89143    139.66062    134.89385    141.80873    142.07295    142.91505    141.50973    140.38888    144.83551    145.56746    148.86379    151.12719      149.007    154.70453    153.65824    152.82931    156.86884     150.7497     156.0085    153.02303    157.14905    153.39885    157.69338    158.41747    156.60789    160.25452    154.86981    162.19958    163.86003    159.97763    163.98447    164.03257    158.01475    165.52315     166.0893    167.60355    171.37057    167.54899    167.33607    163.09531    166.67693    176.39722    176.82733    172.38868    176.23344     169.1019    175.23645    177.80299    176.00337    181.35485    173.45707    177.15267    186.39003    188.43699    186.37809    183.83902    188.55937     185.1634    185.21071    196.67076    197.92583    196.77638     198.8087    198.99912    198.87657    202.78878    204.12404     200.4624    205.38088    201.56769    206.11819    209.34336    215.72335     212.8973     215.2022    216.84169    214.67378    221.40027    218.97212    223.30596    223.14421    217.77938    212.13323    232.10207    228.83013    226.52733    217.22512    230.73002    229.79302    239.89522    248.90561    235.51831    249.73968    231.84882    253.73586    248.29639    259.44758    263.36808    249.88044    257.40923    252.82281    295.87602      278.728    274.88832    277.09461    268.98183    267.36133    296.58673    294.57605    304.71348    292.74369    316.29286    297.84313    319.81356    344.92345    349.43389     326.7118    335.78293    348.42129    344.73629    408.80816    395.06825    401.58451      394.098     385.3556    400.27378    449.65461    477.89888    478.91984    466.44175    448.00536     475.3232    611.20865    593.39666    600.51132    580.89159    605.94901    625.08819    863.65225     881.1275    853.95793    857.63066    964.64257    940.76209          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       19.50    263.47206    262.03884    240.38847    234.50991    228.55129    220.19735    218.44449    210.37912    209.53446    200.36494     202.8226    198.13192    192.77148    184.92609    189.06251    177.46433    177.19661    172.24319    174.77937    178.59754    170.34166    172.77809    166.32118    165.11744    174.54396    185.69223    199.43897     208.5716    195.80828    170.16756    160.70463    158.73479    152.18257    152.57862    154.00324    155.98198    156.50318    152.60226    146.86796    147.76407      150.964    150.67545    148.80757    148.73069     140.3626    140.43426     144.1524    144.58225    144.55471    139.78567    138.78092    136.96896    143.44514    138.48796    140.86259    150.23672    160.33281    168.59568    178.00649    172.17119    159.22613    148.59592    142.71806    138.95063    140.46758    136.28404     136.0585    141.51496    136.88508    147.30376    154.93872     163.1818    168.26195    166.63565    153.63209     158.7606    165.60366    174.08359    174.94936    173.64664    160.19881    154.53654    162.66587    164.45826    153.37431    141.06055    134.41696    130.58539    129.95415      132.488     137.5318    143.34112    142.67303    136.60329    132.25675    126.94607    122.87805    118.47783    119.42117    117.34585    123.70212    125.29242    124.87911    121.50384    121.15973    121.98026     116.1251    119.00175    119.55534    117.24096    114.37533    119.85517    116.96797    119.54436    118.76199    121.81435    125.65889    124.84153      128.564    125.66369    124.77189    119.92598    119.87521    122.83774    127.31822    133.19412    137.11806    135.69483    133.07491    128.18042    120.85185    119.69629    122.08888     117.9483    119.61061    123.38543    130.82464    138.27876    152.57748    159.57483    160.58953    154.62277    146.52447    132.20241    130.19701    123.41021    119.15884    118.61463    119.60602    118.51927    117.20353    117.40772    114.94782    115.82452    116.72727    113.04876    117.58594    117.83338    119.28987    118.17917    119.90685    119.36127     118.4927    117.58811    122.70811    122.91603    123.56111    126.01453    124.27394    122.85648    119.20791    124.78375    119.84498    121.45587    124.69201    128.83321    128.40738    130.72665    132.73217    133.75988    134.19117    129.64156    123.89893    120.08029     124.5563    119.87915    117.35826    119.36519    119.19049    117.68026    120.59865    120.13837    121.58112    122.97139    123.51028    121.30249    122.80339    125.18129     127.2561    127.57732    129.21313    127.76921    123.52821    123.50879    121.39477    120.89187    120.81697    122.68476    121.42998    122.49063    125.23281    124.33075    123.14976    121.39297    118.14202    117.94139    118.29756    116.86109    116.77284    116.34325    115.79621    117.54105    114.94465    120.17777    121.78877    116.78829    122.18254    125.21597    121.09572    124.80454    125.59066    122.93267    125.39415     127.3462    124.40925    126.29257     128.1627    129.18149    130.59996    129.66403    132.10462    133.39443    137.98438     133.5072    128.23397    132.27979    127.06706    127.48552    127.57609    125.36417    130.51664    122.99819    124.38321    127.96747      123.674    127.20806    126.91953    130.72094    127.91477    128.89288    130.58958     135.4363     136.1366    139.05678     140.8289    139.81873    140.17435    139.66783    143.54617    137.59214      131.749    132.76853    129.72497    133.98689    133.63283    129.74738    129.78436    128.91165    126.00069    127.35916    130.67928    131.17495    132.39846     129.5393    131.44055    127.90067    131.56618    129.57386    127.09591    128.05234    131.47302    125.73484    129.25418    128.28434    127.41116     124.8036    127.21669    122.47576    127.26033    124.03181    125.47927    123.51828    122.19818    122.01144    122.16196    124.31527    123.76868    120.14675    124.18759    124.15883    119.70484     126.7872    124.28844    128.31496    123.41862    127.38815    125.75637    132.72903    125.86143    128.50758    130.64078    128.24883    123.62953     126.2554     124.4324    124.57834    124.80029    127.49292    130.73349     127.1819     129.3669    132.42275    128.90827    130.41338    130.03859    132.82599    137.91637    135.50543    134.24772     134.5364    138.78488    138.00318     144.6712    138.65305    142.42984    143.49657    139.11422    137.20357    140.09339    140.66128    142.76952    142.05395    144.02272    142.28585    148.19322    150.33637    151.42188    151.78619    151.59544    149.56018    155.71164    157.31916    156.18318    158.13072    153.38302    152.36981    155.16643    156.70596     159.3599    157.48168    159.51955    157.09774    160.37485    162.92242    161.90242    159.04831    167.12769    159.59484    162.42721    174.50987     169.9193    168.82888    168.06975    164.17891    175.47259    170.25802    179.02101    182.03778    178.33159    176.06735    178.33216    177.15064    179.82952    173.81411    175.60476    178.87238    173.94294    182.50711     185.6986     184.4394    183.61009    182.58604    187.29054    195.90029    193.84811    191.98966    195.17642    192.43732    192.23993    204.07029    212.20111    201.84627    204.94744    208.90529    200.57055    213.54287    210.05042    216.50579    211.32015    210.26439    220.26148    220.44634    221.76516    229.21294    214.68779    229.61214     226.8623    225.54457     234.3135    236.28869    222.21218    237.75734    232.81058    244.50954    239.11442    241.25163    243.68117    248.07044    248.57891    253.62082    269.91726    263.02921    271.46661    259.91794    264.66785    289.35764    271.83501    270.55328    294.21597    278.44615    278.45384    284.20866    336.49214    306.39605    306.44455    310.77591    304.33606    332.93988    339.62313     358.9878    344.74431    350.30735    372.42835    383.57143    387.55071    394.26457    420.69622    404.53062    406.23902    401.86358    484.94644    473.75213    475.52011    502.87918    489.90395    453.79852    617.68082    588.50915    653.36026    593.97251     641.3929    698.16015    859.01104    943.63624    948.19083    1036.6624   1004.94983          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       20.00    260.81272     261.7016    241.09019    234.60192     227.7162    223.41435    221.13643    209.86948    213.24754    199.84253    206.72728    195.96725    192.12481     185.2591    191.70072    179.57194     183.2444    174.32055    172.22692    179.07691    169.25182    171.00293    168.74183    166.11848     173.5653    184.44807    204.74327    210.31345    195.18082    176.67841    157.49108    157.55077    155.48538    148.93455     153.0703    152.27922    150.67726    146.47067     151.5153    147.64156     148.6904    148.97819    149.82518    143.10457    143.88338    144.38014    143.65862    139.60932    139.86743    140.75086    142.06323    135.83406    138.96339    139.23113    141.03716    141.76383    154.65487    166.99108    177.89368    169.89689    158.76889    142.30355    146.27566    138.20871     137.1847    135.85468    137.16218    133.64631    142.42097    145.49042    154.92943    168.53976    168.96923     163.2321    154.23704    158.78067    165.87522    167.72308    175.65375    171.24334    163.34422    157.50846    154.48213    153.86541     145.4109    137.22213    131.56416    128.37942    126.78326    134.40547    139.84831    137.63013    140.33442    138.11216    132.97844     126.9178    124.14747    119.80234    123.34426    119.93743    124.16229    125.85993    127.25657    121.51631    118.40916    117.36238    116.82877    117.41471    117.54352    118.54143    116.91934    116.42053     120.0488     119.7058    120.16716    117.91241    124.22229    129.02349    128.93261    124.54985    122.34731    122.12611    119.02858     121.3032    124.36853    132.75396    136.38971    133.28247    128.53125     127.1329    120.61442       119.13    120.10072     121.8036     122.6809    122.61757     129.8106    138.29135    153.10134     159.4427    158.99982    155.44924    144.84815    135.77966    127.06866     124.4354    122.43869    121.40013    119.45796    116.79742    113.37147    116.82893     117.1107    115.80291    112.69262    114.37421    114.82977    114.10011    119.37376    120.01119    120.05917    119.29796    119.28003    121.10475    118.36133    123.29045    126.48567    126.26697    122.93437    123.14612    120.36567    123.62808    126.07937    127.03569    125.52452    127.37876    130.66936    131.68939    133.48775    133.51588    131.59876    129.46586    122.94012    121.34729    121.26708    118.19235    115.60764    120.69937    117.10757    118.00564    115.05159    118.02014    120.80773    121.38327    121.05677     122.5622    118.35086     125.6559    125.01044    128.11026    127.67698    125.15422    124.97399     120.8648    120.96783    119.89311    120.58119    120.05552     122.2195     118.6907    122.73828    122.40974     119.3136    119.75358    119.55591     117.6299    116.82241    116.33184    116.55413    117.21544    117.33893    117.55315    116.60696    121.58288    117.47505    119.15643    122.25391    121.59503    124.67251    125.37019    125.40498    124.67068    125.49767    124.99774    126.65956    123.57955    127.49564    127.84692    126.71867    131.12311    130.17356    128.07675    133.50001     133.1133    135.11931    129.69105    128.18114    125.27074    120.17134    124.74787    124.53464    125.04013     125.6077    125.61236    122.77168    124.81448    126.90323    125.34243    125.65181     129.0765    128.18496    132.15791    133.04385    137.59774    138.04327    137.53429     139.2708    135.83567    136.01135    136.14845    134.99098    129.26654    128.51465    127.85817    127.67926    130.05894    124.80199    128.09723    127.66698    125.56909    127.39557    127.78945    133.85301    131.78138    129.18486    128.66476    131.37856    127.11901    130.38256    125.83385    128.80077    125.54733    125.63203    126.14288    126.27361    124.36931    127.23952    127.27026    121.31752    125.93421    124.37548    118.97492     125.6985    123.04492    121.88714    122.38669    123.08383    124.53745    118.89612    123.97093    123.64618    125.74605    124.59432     125.9427      125.284     129.4163     124.2406    126.23186    125.46956     124.4138     125.0242    126.62967    120.90284     124.2792     123.2777    125.93806    121.79168    125.37136    125.23139    125.68215    128.39815    133.23219    128.69814    129.51788    131.86988       131.09       141.36    136.64112    138.97988    138.66159    136.02154    132.23639    138.33458    138.14829    145.12331     138.5572    137.43954    138.24455    146.55309     141.4544    146.71399    146.26932    140.33838    146.97899    142.05422    150.35854    152.91122     147.7493     145.8043     150.8533    158.22002    152.61664    156.76672    150.17097    153.26247    153.58566    152.71299    156.78222    151.56977    149.98998    155.55698    146.03442    155.75646    154.23124    156.58693    158.99256     158.0982    156.66904    167.09026    162.71133    163.29371    162.33568     166.9246    170.00825    171.31056    170.54818    176.19369    166.78647    172.98834    165.89451    168.09362     173.2261    173.81573    178.91624    174.52257    179.29294    174.14567    188.37001    176.88857     181.0572    180.44177    178.53977    171.54245    196.05313      188.694    187.23658    192.19419    190.60948    188.02055    198.46097          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan    289.06149    295.71328    317.31163    305.88555    315.60469    308.03043    344.10356    345.99484    344.27425    358.30816    348.31846    339.27569    362.03774    418.43706    418.52997    428.59186    402.96234    385.10892    419.19854    487.73622    530.79535    500.05277    481.48034    498.47294    509.66877    633.88316    646.36473     664.6346    640.93305    610.73933    863.39963    756.11172   1188.73698     1128.829   1096.53242    983.78273          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       20.50     264.3071    261.55019     241.5431    232.60235    225.58916    229.73547    220.78614    211.15829    214.39378    202.11466    202.24794    193.59333    189.83972    188.30456    194.49323    181.19206    181.24248    176.18988    173.74453    175.49583    170.57774    171.09636     170.4963    166.16915    173.67181    177.89892    202.30956    209.84768     192.2502    177.45208    157.50885    156.15346    154.88848    155.64529    151.74423    154.56148    147.43166    144.09487    148.59596    148.68972    146.01803    144.87962    144.94557    145.30694    144.98498    146.85893    146.74497    142.78057    138.10838    138.06592    136.98903    139.54324    140.85722    137.61226    144.96933    144.01759    155.47251    171.27025    177.28019    170.65396    162.13462    145.45867    142.95152    142.44104    135.19696    135.09043    136.27012    136.65059    136.59015    140.64169    156.13107    163.24545    172.37359    164.21703    156.79369    160.24646    165.03432    169.30065    173.94019    168.58332    163.33778    156.45617    151.12899    146.10349    141.51615    133.49652    128.85834    127.54151    130.36172    134.73702    133.75492    140.77935    140.80148     133.1831    130.09016    125.97367    127.17472    123.79223    117.31999    121.14064    123.11131    124.18198     126.5186    123.43477    118.34108    119.05201    119.67073    119.47339    120.72834    119.05972    119.66596    114.26994    116.07225    117.09162    117.38985    121.57658    123.04066    126.46574    124.86823    125.81142    118.85055    117.69312    116.11286    117.26258      124.205    128.97495    134.85558    134.04583    129.00431    123.47595    124.82504    116.62122    118.62816    119.33979    118.29521    121.13184    132.10134    140.83533      152.762    161.32416     162.3145    154.70182    145.05219    135.17417    125.39169     124.4662     120.4199    119.96126    118.30915    119.15978    113.14202    115.42396    115.89116    117.54194    118.75324    115.73324    116.88785    117.53438    118.80895    117.28903     121.0008    121.83233    117.95603    116.45163    117.51799    121.90511    123.67872    125.42891     125.7159    126.19733    119.92025     126.1484    128.67155    129.31646    129.47802    132.93336    130.48994    129.25909    130.99347    137.29219    131.50266    129.26633     127.0986    123.81999     117.9443    119.85312    117.60071    116.94465    118.15427    115.22992    119.57929    120.71427    118.03232    119.68679    122.36815    121.54277    123.53717    123.58141    126.77115    128.22123    124.52142    126.77148    125.47249    119.95791    120.53312    119.41212    124.33723    121.22748    120.64128    123.29071    122.29799    124.16054    121.97533    120.19297    119.13082    118.24882    116.08928    117.96194    116.73363    116.85088    119.66007    117.24534    116.55438    118.53133    117.72231    120.54424    122.65245    123.69248     124.0928    124.18261    128.55662     127.4317    123.83625     124.2943    124.17282    125.02809    127.52119     124.7759    127.34866    124.39783    131.42995    128.13127     128.3574    128.65693    128.28947    128.92848    128.80869    127.11122    123.79996    125.19257    122.08578    123.60722    123.24213    122.89937    122.16645    125.94831    125.41543    127.42338    128.05864    130.76342    131.25411     129.5987     132.2171    135.29528    137.45462    134.43578    139.20851     134.1175    137.44464    135.53254    133.54263     131.1571    129.84449    124.53719    125.93792     128.1098    127.52371    124.19701    126.65709     128.5535    125.98612     129.6977    129.73972    125.20649    125.82651    128.23431    127.14923     129.2252    128.84641    123.64101    127.70599    122.43467    121.61507    127.10216    125.60991    126.38461    121.74709    122.00495    121.68326    125.39526    123.77518    124.65808    120.83414    122.24289    120.59395    118.61784    117.86243    120.58644    122.41294    123.42354    124.99941    120.32067    126.10177    123.25011    127.38847    128.48312    126.25422    123.95797    126.36444    126.44579    127.43005    124.08761    125.82799    122.35087    127.02335    126.41876    123.96998    127.93508    128.07382    127.76562     131.0839    128.44626    127.54835    130.03199    130.58438    127.86931    136.99256    134.52031    134.75372    135.77928     137.4751    133.25828    137.35832     135.5656    134.06006    136.17603     134.4999    137.10383    137.01519    144.18353    142.43352    140.35056    140.74493    138.56542     145.8375    144.39775    143.23315    141.10847    143.32486     143.7876    150.13313    145.89654    144.01984    141.71865    144.89432          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       21.00    269.20618     262.6885    241.67477    229.59785    224.66195    234.04721    219.97752     215.2702    210.09873    202.15103    196.98608    191.25752    188.20465    196.27962    191.59847     183.4837    177.75953    176.26352    176.44473    173.61475    175.09721     172.4769    170.83032    165.36521    172.03433    180.00055    195.41795    204.83347    196.60197    174.60968    159.71191     158.3575    155.04353    156.06241    153.08792    156.32051    145.94922    147.63333    151.14332     152.2878    148.18445    142.03438    147.39343    142.23101    141.33827    145.84746    145.84352    145.50118    144.48677    142.91658    136.08455    136.57462    143.75735    138.84513    143.57634    143.51433    154.61325    166.60478    176.73054    172.08686    159.18344    145.14263    141.10907    140.92704    135.27246    135.99673    134.31783    141.07098    140.07592    144.01372    159.58954    161.68278    171.64041    163.45418    161.20248    154.88866    164.51963    165.61564    168.24479    171.59535    165.67703    155.10553    149.84461     145.3678    137.91045    133.22175    129.00159    129.75416    129.22951    133.16131    138.75287    140.79174    137.89735    138.93382    133.86245    126.28076    122.68168    124.14775    122.38207    120.40687    123.53775    125.74966    125.85855    124.53484    120.17443    119.63943    118.68239    119.34158    117.84767    122.01319    118.76421    113.51368    119.81619    117.68326    119.21047    120.45666    121.60385    128.85855    125.29462    125.78772    123.16478    120.17141     119.6011    121.60244    125.93155    129.21336    139.03303    131.56609    129.13491    124.90132    121.61903    119.70889     115.8889    121.21041    117.65707    121.22323    127.88009    136.10165    149.35079    158.49237    158.16949    156.88343    142.09727    136.32051      125.785     122.8323    122.69437    121.21911    121.17999    117.86046    118.53791    117.10471    115.25877     114.3335    117.27886    113.37891    117.59843    114.30436     119.2752    116.51405    119.04999     116.5761    117.93389    118.48008    117.42053    120.60235    122.10116     123.9739    123.78252    123.55714    123.55161    121.22704    125.49421    125.67701    130.11445    128.09916    133.39682    129.34518    128.48216    134.92355    128.50729    127.16142     123.9403    123.73468    114.87323     115.2371    115.74338    117.01607    116.77367    120.26455    117.00411    113.17587    117.12457    121.01099    123.39176    119.69345    123.57919    121.67953    126.34734    126.15322     126.0238    123.61804    122.58409    120.03998    118.62904    123.04336     119.4467    117.95321    122.82281    123.02296    121.18202    121.39978    122.79288    119.35193    116.44617    119.63545    113.05257    117.35421     116.7092    117.71038    117.44643    119.77936    117.03346    117.48421    119.13609     118.9038    120.00322    124.99617    123.35756    122.47476    124.77202     126.8824     124.6276    125.92469    122.86383    123.66132    122.66108      123.695     127.8046    124.72711    127.83835    128.80752    128.46724    130.18388    128.76912    126.81549    128.43161    125.60437     125.3109    121.64198    123.94968    123.17611    123.33217     123.7327    125.60889    121.31612    123.85507    128.61014    124.82402     127.3042    131.22948    127.32159    135.34705    131.98666    134.34724    138.09155    137.21222    133.74631    134.20688    133.09138    132.50724    128.68921    126.93897    122.78715    124.37253    124.40148    124.16005    124.66812     125.0762    122.54597    126.51505    129.72054    128.84353    127.49313     127.4713    129.90536     126.3718    125.38473    124.56456    123.91629    126.19994    123.19719    122.26591    119.84618     122.4665    120.14334    125.05791    118.88223    120.89383    117.99922    121.58086    123.69417    118.84776    123.14802    123.01831    118.66283    123.17565    121.72154    123.17528    120.92118    123.70548    121.31438    125.68865    123.43638    126.31087     121.5991    125.22193    121.08211    119.86124    129.19462    122.74652    124.10465    120.34841     119.5645    122.28934    126.48422    128.10093     121.0557    118.81705    121.41966    123.27036    126.94606    126.39707    128.71943    126.43515    125.34027    124.89706    129.97676    125.44252     125.1359    126.17432          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       21.50    273.05824    263.75965    241.29179    227.22553    224.36798    230.69291    219.88097    218.05316    206.12235    198.18689    199.22851    188.29322    190.93109    195.50597    190.64383    187.83314    177.49136    176.21912    177.12717    174.39848     174.5113    172.66605    168.72816     168.7887    171.14346    182.53995    195.99053    205.81822    197.90403    173.98522    159.23439    160.16078    158.19193    157.58206    153.43514    155.92097     148.9481    148.63125     149.1767    150.71418    150.77831    154.25926    142.90864    147.06361    145.06472    144.80199    144.30508    141.58481    146.16888    142.06863    136.71218    140.07317     139.1297    137.37853    143.30188    152.09924    156.30505     173.4486    179.49291     173.3305    158.07668     147.1392    137.85099    138.22867    138.24108    137.01023    134.18174    133.28403     140.9939    146.07093    157.69792    162.58487    171.19603    164.58028    155.58549    155.90242    164.66658    167.79829    163.89648     168.7892     162.5555    156.72859     154.2214     150.6472    139.17334    130.28216    128.94695     129.7346    130.84117    131.70209    137.74136    138.50829    139.43297    134.74438    130.19764    124.57455    124.21855    123.41115    120.52858    121.44139    121.20282    126.69517     122.7561    124.17059    118.22527    114.10696    119.17769    119.49775    119.65834    117.32882    117.03306    118.47342     114.3199    115.25911    117.37314    121.67551    122.35053    129.34179    126.01349    122.26389    120.58211    119.97556    121.96888    121.19332    130.17199    132.99267    139.89507     131.9704    131.23853    125.57669    119.86204    118.40194    115.89458    118.74507    118.36998    120.91946    128.68121    136.10718    149.36441    154.37512    158.97154    157.76878    145.51999    141.80918    131.03141    123.22289    122.31518    120.79974    118.92058    115.35752    118.63883    113.00547    114.10863    113.72102    113.41444    117.34587    115.64258     116.8429    115.66632    114.41507     115.6356    117.18636    117.53703    117.98449    120.09902    119.31917    124.33746    123.76932    124.13924    123.49155    127.20172    128.34712    127.83352     132.6803      126.872    130.37099    129.36717    128.94936    126.49361    128.99738    130.39853    125.48085    126.56661    124.82632    121.91508    117.17351     118.1791    116.83624    117.65315    119.13414    118.04725    114.61177    117.49368    120.03177    118.90238    120.36961    120.46743    124.57129     123.7396     126.7931     124.4587    125.34433    122.84464    120.33379    118.04819    118.39733    119.51703    119.75847    119.85022    122.38312    121.99689    122.75467    122.85804    120.97496      120.967    119.10223    115.01256    115.25025     113.3819    113.66546    115.57622    114.44305     114.0285    116.28284    116.45143    122.01405    118.19344    118.33072    124.59325    123.88196    122.33587    125.30977    125.23788    122.46588    124.02846    123.01886     124.9812    129.06354    125.14285    130.87399    126.80047    126.42098    126.28624    126.31141    128.91301    129.95234    129.94665    127.18402    124.48753    122.79243    122.76744    122.93036     125.5421     125.4678    121.05678    121.19054    123.30442    123.10933    124.97412    124.93783     128.5237    129.63148    132.96027     135.0806     136.2368    138.71202    138.24189    136.21312    134.52787    133.16338    132.11014    128.24934    127.66291    126.96659      121.236    126.06155     126.0138    123.30695    125.82523    123.04411    123.21628    127.96471    126.50239    129.11479    127.74938    125.98809    127.27346    123.86007    124.49163    125.87173    120.38417     123.2712    121.46349    117.61061    122.13572    120.37068    116.28949    122.26191    122.25982    114.81566    119.83338    120.85861    120.71192    122.31357     119.1775    116.44867    118.81398    120.96002    118.50295    120.56011    118.96489    116.96371    120.78562    120.50764    120.51627    122.24139    120.95205    116.49502    119.84182    118.95692    122.31103    117.42446    114.81676          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       22.00    274.30192    264.43307    241.96366    229.83831    223.54105    227.41338    218.74331    215.78624    207.19708    194.53953    201.34062    193.45982    191.55979    194.22494    187.24194    189.30397    177.01511    179.24139    174.28595    173.71838    172.77619    167.96418     173.7461    168.38007    171.32382    185.41292    197.89461      205.568     195.2404    177.17445    159.92279    158.36537    160.22038    153.13239    150.35718    157.99375    150.71371    150.52266    143.32812    147.33018    150.50097    152.87085     144.0034     145.8423    146.76216     143.0148    143.84195    140.47749    139.50243    141.52145    142.78575    139.90214    142.25096    139.99153    145.20702    145.60549    160.09809    172.34698     176.9506    170.75224    159.94624    148.28202    142.02139    137.92319    137.79058    135.81556    136.75547    136.35627    140.58489    141.56495    151.57585    163.72146    172.62732    165.25348    157.67626    159.52103    165.67667     172.5064    165.65916    162.34088    161.29706    157.84851    157.33684     156.0363    145.12462    141.13588    127.80929    125.00589    124.92936    132.59187    137.80914    139.51122    141.15032    136.68151     131.4708    127.35525    126.53403    122.48535    120.56849    121.96327    122.74616    121.11737     122.9812    120.66372    121.84006    116.26963     115.9329    116.52505    118.68528     117.5091     117.1882    119.17378    113.87996    118.63949    118.53958    122.20876    122.52042    128.01727    129.70462    122.49911    119.94162    119.69996     121.3747    121.53258    127.65802    132.30234     132.0955    133.41075    130.82387    127.25512    121.41608    117.12384    114.52303    116.39892    121.65792    121.82177     127.6574    137.93485    147.82429    155.56521    158.30346    152.56737    148.77762    139.26793    133.90896    127.99393    122.75419    119.59947    117.00097    116.63767    120.18412    111.87955    110.90463    111.72741    113.58504    115.01484    115.04701     114.1234    115.34988    116.88278    119.99608    119.32903    114.32256    120.77806    115.58617    120.51072    122.07768    120.37518     125.9433     131.9814    137.58785    143.51539    137.92895    139.81662     134.7773    130.83365    128.98673    128.01343    129.43156    123.04461    127.86551    127.15164    125.87751    124.03581    120.63012    115.14473    115.23182    114.87764    115.23195    118.63806    114.43935    115.12037    116.51015    115.65598    119.17273    119.17588    122.27341    122.11077    122.95349    124.72504    125.20109    122.38163    124.06156    120.14992    118.43416    118.49599    118.96765    120.67329    118.18616     122.4943    122.69828    119.37103    119.04707    120.01535    118.72699    112.09369    117.12633    115.93763    116.04394    116.14093    112.94775    115.95001     117.7643    113.07973    115.21179    120.24869    118.40888    119.34696    121.05754    121.91593    124.72261    126.66666    124.94057    120.34393    120.69329     120.1863    124.36757    127.59056    126.88385    131.11296    129.50811    128.74137     134.5215     127.8112    128.71438    127.71712    124.86595    124.58623    121.01948    121.45866    118.59949    119.77396    123.30176    121.04735    120.58998    123.98287    120.51942    124.82422     126.5185     128.4131     127.2347     125.5425    129.40684    131.65048    134.06148    137.23105    137.10744    133.87682    130.75512    133.39517    132.38853    125.56885    125.34485    126.15884     122.9352    124.02684    121.06677    123.00399    126.99049    127.55164    126.00178    125.74165    126.68346    128.26657    124.64246    125.27958    123.28425    119.30899     122.6175    121.53551    119.22717    129.48307    116.49847    116.82434    119.99472    114.67943    118.96029    117.45963    117.77117    115.92095    115.00692    115.05762    115.12901    112.10704    116.22769    112.65566    110.64263    111.26646          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       22.50    274.56167    265.29433    244.64953    232.79415    221.48539    226.18552    217.76851    216.15964    213.24239    199.65194    200.51695    195.13802    191.08202     189.7893    182.27672    188.67722    176.82305    180.27478    178.16665    173.90877    170.53423    175.18992    175.26985      166.409     173.8542     189.0544    199.08353      204.242    187.18561    172.39769    163.31099    157.16047    161.07899    157.35433    152.63709    159.74572    150.63374    153.60918    144.96427    151.45211    148.88982    151.72648    146.40846    146.67423    146.46241    142.37529    142.66747    145.79169    145.48269    144.23718    143.14727    137.85744    140.28787    141.20698    142.24904    147.35546    158.72927    175.18121    177.42947    171.71357    157.35241    146.64782     140.3305     137.0939    140.04233    140.55462    135.29749    138.75794    138.77067    142.56474       156.11     168.8457     169.8723    163.64331    160.57731    163.47899    171.70379    174.35244    167.94208    160.21247    160.51943    153.68584    161.57804    161.03725    153.49363    138.19906     132.2398    129.07827    131.08834    130.97626    133.96467    139.27251    137.27302    136.74178    125.30013    125.45005    124.81303    120.56956     118.4057    119.10614    122.28657    122.43418    123.86904    122.58687    118.70326    118.96809    116.61678      114.441    119.79162    116.46465    121.09956    119.70388     118.3111    113.99505    119.40397    121.07103     125.8364    127.97827    128.06727    118.15124    121.19436    119.00548    120.23482    121.55878    127.97181    135.44981    132.85512    137.94093    127.40752    123.53626    122.33387    118.12448    118.64037    113.55758    120.47725    122.04076    129.10509    139.55323    147.74016    154.34051    151.97173    148.19979    142.35671    135.03841    133.57596    129.96891    122.19282    121.77882    118.62152    115.26637    118.27622    116.76703    112.26628    110.21412    113.64717    115.61633    114.31372      115.729     117.7884    115.50448    120.60415    115.90006    117.45605    119.51845    117.39009    120.49228    118.43231    123.74399    130.14705    136.43486    150.68999    157.49139    144.21208     134.7946     128.1232    127.98899    126.48034    126.13798    128.68356    120.61767    127.05035    127.86864    123.42352    122.12829    117.46579    118.22908    114.36036    117.83538     114.3672    117.82384    117.07744    115.15536     115.5373    121.00298     119.6436    121.66925    122.05771     120.7471    121.66515    124.02823    126.34627    128.85108    123.63933    121.73997    122.04729    114.56682    121.62113    120.42044    123.01198    121.86754    121.77866    120.28012    118.07488    120.44623    116.85924    115.68881    115.19161    113.73947    112.33475    114.89136    114.12843    112.84875    115.11571    116.16664    114.11646    115.25226    117.48031    119.66688    122.43495    123.37591    123.76576    119.60182    122.03367    120.91009    122.97135    123.42569     122.8221    121.84482    126.54168    123.68367    129.19538    124.47571    127.05301    128.07274    124.01284    125.79423    123.36338    121.86518    120.84583    120.86842    117.34586    119.27325     122.2988     122.5109    120.74445    121.84335    120.03484    125.46122    124.33558    126.72306    126.12972    128.64832    128.39807    131.11497    130.97982    133.71617    136.11177    133.16031    131.58086    130.37093    129.24897    127.15164    127.88639     121.6857    124.04676    121.09486     121.4481    122.93513    117.88913    120.22171    123.29504    120.39597    124.84617    122.80227    122.67033    120.34676     119.7965    120.13955    123.00984    116.91843    113.69953    112.03198    111.47395    108.54679    110.75408    113.76299          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       23.00    274.86384    266.36029    249.30988    233.87361    219.57157    228.17042    220.71008    217.86888     216.3352    204.52237    200.18863    191.68921    190.86442    187.24883    180.75797    187.60362    177.23916    181.20276    184.34086    171.69198    174.08681    177.26724    168.76438    164.25455    172.99047    183.22053    200.48193    205.56919    193.94619    174.63712    163.54792    154.48668     158.4602    158.70248    155.67922    157.34263    157.57848    147.98773    147.76044    147.98495    153.30114    151.64983    150.28025    145.13394    139.65673    146.80103    142.64604    145.10842    141.63705     140.0381    143.83252    139.26715    136.90427    138.04292    142.82807    145.30139    156.88159    175.05899    175.76869    169.08932    155.99862     146.1495    144.58621    139.14896    138.84123    134.82286     132.3652    139.31453    140.81159    141.10362    160.04632    172.45259    171.87539    165.37449    165.28729    164.89893     174.0658    171.61461     168.2722    160.35452    154.96094    157.03172    164.27025    162.95143    155.44542    142.41345    134.26353    128.44022    129.71763    129.71229    138.97327    140.99568    141.33237    138.08782    132.83041     127.0377    122.82967    121.75872    120.76975    120.89323    122.77331    127.34269    124.65935    121.31422    116.81257    116.35203    114.47754    117.60954    114.06363    116.38492    118.77626    118.12928     119.9222    114.81372    119.97907    124.32964    125.54867    125.09559    124.23333    122.71568     119.6614    121.03131    119.92285    122.73487    126.59139    128.00325    133.05627    129.72726    128.18386    123.82453     121.3624    118.11893    120.62817    112.67108    116.80882    123.62962    130.69855    134.67712    145.99322    154.13427    153.59877    151.04481    141.93009    133.20033     131.5175    125.01748    122.75489    119.33807    119.35595    118.64873    116.95463    119.18577    113.98339    112.54315    113.91417      114.286    113.59952    114.43115    116.75298    120.04309     115.2069    118.98113       117.64     114.6308    116.17602    120.07101    120.16764    123.87865    127.22618    135.61974    143.39728    139.96796    133.30684    122.41216    126.36957    122.75308    125.17826    126.20026    125.71041    126.65733    122.79084    127.07604    122.81435    123.88998    122.41772    123.96062    118.86099    115.99019    115.94896    120.06199    117.89832     113.3204    123.02015    117.02339    115.93185    118.93685     121.9803    122.54081    125.08244    123.94789    125.47254    122.96127    123.52563    119.83464    119.49747    115.74018    118.44316    117.93013    119.09643    123.59176    121.71727    119.29331    115.87715    118.17801    118.44824    115.97393    116.89966      114.179    113.48022     112.9133    113.63413    114.83612    117.85532     112.5577    116.00627    115.80506    117.84854    119.68378    121.29462    121.65245    122.78332    121.69141    121.25932    118.51713    121.75445    122.49716    122.92172    124.07029    120.81562     125.9865    121.69367    124.38546    125.16881    119.24152    123.67996    124.58294    123.25267    120.66248    118.92795    120.17578     117.9215    119.99275    116.84108     120.1878    120.49074    124.10812    123.42492    120.82239    122.68162    124.33051    122.60688    124.51677    125.54591     127.3115    132.77861    135.12477    133.02002    132.61563    131.54578    128.06082    128.83196    127.21487    119.89745    121.41357      122.133    119.76087    118.86103    118.50179    118.06025    117.18488     116.1279     118.0666     113.0712    114.85468    112.03871          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       23.50    274.63967    266.62788    253.59972    234.87314    221.41063     229.4611    226.06381    216.59512     213.7663    204.20507    199.66242    189.90003    189.09126    188.91319    183.23292    187.38683    180.27987    181.10387    179.08383    170.42298    175.69928    173.39404    168.90655    164.95129    172.53128    174.69393    197.30374    204.17945       196.41    180.13649    163.84473    163.42236    155.89066    153.84607    151.91874    152.01233    147.78127     149.2732    152.16657     150.3786    141.74423     148.9379    151.65116    145.01019    140.69832    141.70696    141.25944    142.60694     148.0972    142.20918    144.58646    140.56793    141.91762    143.83035    143.34978    150.09961    159.80292    172.14443    176.49701    172.47459    156.11581    146.06762    143.89225    139.15669    141.47238    132.25336    138.59007    140.20273    138.75424    146.45269      158.418     169.2672    165.04321    170.75934    159.23855    162.00517    176.63131    170.46968    167.14284     160.0023    158.04629    160.51684    164.21209    164.45634    158.74267     146.6814    131.04251    128.01914    130.80655     138.5247    136.13355    142.40055    142.33495    132.34007    135.81865    128.18062    124.10548    123.92611    120.67485     121.5451    121.40229    126.05214    124.27488    124.00132    119.34443    118.02191    117.52989    115.84895     120.8573    118.68783    119.03993    116.23361    116.13844    114.02154    118.37739    121.17283    124.47259     126.5214    125.44011    123.46685    119.59894    119.48123    120.40746    119.42866    125.18629    132.98789    135.24437     129.2502    129.11454    123.64857    118.83004    121.32158    115.81755    116.47364    115.41968    119.69307     130.2248    138.09873    148.72847    156.19124    155.87596    149.27499    142.75304    132.69714    127.35088    123.01891    122.61653    118.84008    119.75301    120.06281    120.31412    119.96665    120.33076    114.83269    112.83001    114.42468     118.4354    113.34456    118.04886    120.32675     116.3151    115.69819    117.80466    117.41464    118.64726    117.45143    118.47059    120.38798    122.09704    127.52039    126.72395    126.50492    126.67949    125.52138     118.9294    124.72382    123.92297    124.40892    126.61173    126.23663     121.7371     122.1628    122.62907    117.61083    119.11792    117.65141    117.53253     113.7588     112.9873    113.08253    116.71444    116.47754    117.16252    115.36559    117.63768    117.44116    120.56738    121.93576     122.1484    123.46458    122.94815    125.83199    121.35779    120.97578    119.06319    118.62647    115.36261    119.23604     120.9083    120.47249    121.81924    119.59145    119.25892    118.17566    115.98415    112.79254    113.60741    112.95798    115.30208    109.20667    110.86792    112.08406    113.99233    115.32356    114.41474    112.69795    117.79951    120.65078    120.17091    123.40576    119.66958    121.01751    120.27461     120.4479    123.09251    120.66573     120.7389    124.67251     120.9871    122.69099    124.19652    122.16646     122.6188    121.28643    123.45002    118.56241    117.60484    120.31433    118.37083    113.91519    119.02615    117.49377    117.81654    120.45755    119.98267    117.42879    118.99355    120.41622    119.86647    118.90737    123.09063    121.50313    126.38509    124.91654    127.80413    129.23432    129.32235    129.98967    126.67306    127.60683     119.6287     121.3881     118.6169    115.69903    109.40122          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       24.00    273.74762    263.19725    255.93685    236.99643    220.78541    224.19793    222.34542    216.00207    211.70735    203.55583     193.8458    194.52277    190.18475    190.00666    187.67456    186.90811     178.1051    179.90005    175.35344    174.14098    170.40526    176.25742    170.55734    164.49845    177.08133    176.92364    199.91308    202.94995    197.15248     182.6091    163.35167    158.18505    155.74567    149.11692    154.96154    152.07001    148.74235    151.30462    151.76797    147.73655    156.62849    145.45981    152.69335    147.72271    143.55065     146.0255    141.83173    143.45676    145.23629    143.15226    141.33604    140.92802    144.38889    144.30041    140.43043    146.86626    153.62692    171.75821    182.29001     169.1606       154.26    147.69448    142.49246     134.4561    137.15821    135.60647    136.69802    137.63952    142.39923    145.42608    156.56135    169.58308    174.11901    169.98158    158.17778    160.91456    168.16989    171.34297    172.06175    169.01347    158.51316    159.90722    157.86625    155.83181    154.71841    143.57112    134.39757    127.97725    126.89872    135.00073    139.31462    143.91978    144.99409      137.647     132.0396     127.8082    124.96872    121.17638    121.08223    121.93759    122.81926    122.96559    125.67232    122.76183    119.31421    117.80408    118.15532    116.06393    116.76574    119.55093    119.10503    118.32994    114.32328     119.3421    116.25131    117.82272    123.28897    126.26088    128.20251    122.04648    116.45312    116.39119     120.8778     120.4746    125.71852    129.84749    134.89328    129.88681    126.34105    124.08759    120.38401    115.58462    117.16944    116.56375    117.82462    122.66522    129.18276     135.5983    148.60069    151.89057    156.75315    149.36769    142.70101    128.21547     123.2805    119.75796     119.6588    114.81733    120.15365    121.70115    121.50942    118.27927    116.48738    117.21618    112.30206     112.1707    115.93733    115.09016     118.8657    118.46716    115.42163    114.33402    118.35506    116.02293    119.92466    119.27929    120.72779    118.18254    122.59373    120.48864    122.86032     124.4943     124.2655    122.34749    120.89367    125.45987    123.82878    121.42354     125.4835    122.49969     124.6778    119.51174    119.51119    116.69526    117.02314    115.18061    116.45918    113.33856    111.01742    114.76657     114.4149    115.34555    115.61724    117.03262    115.25433    116.78368     120.3815    119.19948    119.83271    121.99763    121.73354    124.12721    121.42959    120.94112    118.18005    117.22093    113.96169    114.42784    117.66759    120.35589    121.82119    120.66607    117.90673    116.01887    115.61241     115.6959    112.57272    111.69573    109.52849    113.56228    110.20488    112.26985    112.86766    113.21219    115.89404    114.34336    114.90434    122.03149    119.88141    121.16945    124.36049    120.82089      119.273    120.21327    119.95434    117.78766    121.56681     119.6103    122.57692    121.25371    123.34621    127.18298    124.80861    122.13074    120.74942    116.25148    117.61383    118.60353    118.57855    114.98335    114.30832    114.75186    111.75485    116.36568    117.72129    114.81988    115.56067    117.39235    114.12916    119.55844    117.27288    120.32859    117.96485    123.84241    116.14601    117.78804          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       24.50     271.3246    259.73457     254.7333    236.74323    218.94076     218.0959    219.61852    215.83519    211.55249    208.31385    189.83844    197.95576    194.81662     188.2147    192.80923    187.39285    176.58255    180.15488    172.44906    178.44901    170.39004    174.90646    176.37154    170.66504    173.46546    182.86101    205.88734    205.85722    196.25456    178.98999    163.42351    158.98311    157.54626    152.41197    151.53029    153.81611    152.19304    154.66566    152.32714     149.1148    153.13243    147.44204    153.36261    148.84914    144.46107    146.86569    145.02518    142.30229    145.65837    145.35936    143.73977    140.15839    142.53487    142.73201    141.83085    144.99426    157.21803    171.37489    179.02148    171.97315     157.3265    146.29368    139.77645    139.46187    137.00167    139.00712    134.70618    135.36321     138.5363    145.41527    155.05533     168.5942    177.15756    164.17759    156.49307    161.48506     164.1278    173.61283    175.51537    174.04495    163.84024    159.39666    153.98325    152.78522    150.42023    148.52188    132.69947    129.50255    131.66094    136.58036     135.9379     147.8281     144.2556     141.4921    133.45961    133.17852    127.25189    121.12462    123.63284    120.71457     124.1705    121.08246    122.76788     121.4704    119.16113    120.28002    116.19594    110.25189    120.01423    118.17959    117.50296    116.89639    118.39266    120.21905      115.791    120.90429    121.42502    126.45157    127.53929     121.8983    119.66068    116.35863    117.96174    121.98757      124.836    133.47157    130.23982    132.63075    127.02314    120.43852    119.67443    117.76206     114.6072    114.62817    116.48556    118.99842    125.90042    137.18582    149.61545     154.6917     157.9036    149.77829    141.27353    134.70572     127.0682     119.6103    122.41638     116.2676     118.6658    119.53181    119.41878    116.48729    115.76398    111.95245    108.44137    111.83686    112.57546    113.48705    116.96154    117.10855    113.94696    114.68836    116.01049    117.25893    121.02584    118.42925    121.32017    120.44004    118.63125    122.44332    120.04782    120.63641    119.25991     120.1171     119.7289    120.29031    126.02138    122.67517    122.90398    121.55942    123.84567    118.27642    119.51767    116.34673    116.56942    119.66061    113.56926    113.27331    111.21862    114.36476    113.03485    113.43443    115.74747    114.81639     116.3468    115.75377    118.49665    124.49494    122.18218    121.81885    123.04026     119.0692    121.10941    119.42296    116.55568    116.21565    114.15547    114.02354    117.62458    119.74537    120.48688    118.97993    116.40631    116.80595    112.56339    116.57875    113.99739    109.86901    111.52042    111.96713     107.1103    108.25384    113.33233    112.50151    112.29801     114.9882    115.67327    120.76625    120.43641    117.85092    119.41404    120.12432    117.46853    116.61094    119.38827    119.62686     118.6364    121.09398    124.12464    122.96325    123.95821    124.74214    125.99635    121.32804    120.95999    117.94225    115.12593    115.50923     112.0553    112.49159    113.69329    110.74223    109.18918    109.72212    112.92783    110.50988    110.12904          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       25.00    269.63586    256.39017    252.01553    234.81045    220.39231    221.00245    219.97812    216.39578    212.58981    214.81702    193.35881     195.9923    197.61858    184.75702    195.06399    185.59292    182.48479    180.57093    170.76582     181.4594    175.08796    172.00864    178.54699    168.35385     173.1985    186.68298    207.82095    206.20262    196.00357    172.02405     167.1014    159.01242    153.84878    155.58198    153.38348    155.58003    152.11809    157.23655    153.11834    146.98194    150.99844    151.62104    151.29258     152.6424    143.37274    146.59038    143.68897    145.46256    143.55566    143.82855    140.88978      142.791    142.20249    140.78541    137.91363    143.58292    154.94746    173.45963    179.55822    170.24218    158.06826    148.18421    141.84984    139.44212    135.46094    134.66195    133.42221    137.22189    139.75745    144.61392    156.06086    173.81797    173.56095    162.80048    155.26569     155.6721    166.02395    178.72319    178.77654    172.86705     168.8165    161.09572    153.70129    151.55641    146.23981    137.91733    136.31361    128.50958    130.05852    132.45568    142.68413    148.20787    149.64321    141.35782    137.57855    133.93715    129.56406    123.52012    118.82726    119.00484     121.3727    123.52515    125.51848    123.44781    119.26777    117.47576    115.49787     116.6247    115.50116    115.84268    117.24122    116.21078    115.78947    117.65034    118.72924    119.65708     124.0327     125.8071    124.62895    122.64535    119.13793     115.3616    114.58651    120.78067    124.11127    126.30271    133.13657    132.98592     126.9881    124.40521    120.30249    116.84023    114.41761    115.42572    119.14206    115.55171    126.68514    135.40775    146.96856    153.01499     153.5569    150.89396    137.28824    131.49829    124.65195    116.63532     117.4894    118.76667    115.84605     114.7456     115.1827    114.33199    115.00682    111.66992    112.61584    111.44461    116.73408    113.96154    118.05971    116.34305    119.11018    115.04469    112.74721    115.58428    116.79098    119.46299    121.58285     120.0178     121.2484    116.15186    119.83792    120.91633    118.93226    120.18209    120.62992     121.5489    117.12014    124.19169    125.31363    122.73169    123.01422    121.66512    115.22344    120.18725    116.54858    115.63977    111.89331    110.76884    110.73843    113.73783    112.76331     110.9806    109.92406     115.6845    115.80786    115.86354    118.18386    119.21111    121.29406    121.26571    122.99206    123.12046    117.46884    118.72296    113.67928    114.54003    116.37281    114.92402    115.41782    117.93227    120.43959    119.23591    118.56152    115.75401    116.05716    113.66623     113.4914    111.20335    114.13869    114.64456    110.62228    104.96564    110.73884    115.75053    114.08951     113.3786    115.28061    115.99318    117.57839    117.41365    118.93832    116.97455    118.23978    119.02437    117.68226    119.23261    118.12072    118.82496    120.87818    122.70127    122.68847    121.24178    122.56631    117.72664    117.80942    116.87394    111.60408    109.17883    108.98958     104.5387          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       25.50    270.49175    258.47355    250.57478    235.00783    221.33515     223.8698    218.95587    216.75686    209.83302    213.12258    195.56334    194.49665    198.93854    188.55674    191.38381    184.84125    183.16611    177.03296     171.9723    181.23293    174.00829    171.02586    175.08606    164.70423    175.54529    181.53889    202.73963    205.46639    194.37615    171.84341    168.25394    159.91055    156.77938    148.30556    152.94039    154.13308    153.82597    154.65282    152.51562    150.14134    146.14622    148.35931    146.14006    150.08146    145.86387    144.47994    137.62432    142.45925    141.08077    143.62349     146.5832    144.26008    141.68558    137.34086    140.54166    143.19822    160.47304    171.49075    178.07205    174.42486    155.01725    142.36834    140.16245    137.45601    133.27633    138.84945    136.39836    140.60005    138.32202    143.80123    157.69072    166.51323     172.5455     166.4178    158.59491    161.76123    168.31458    181.33964    186.22429    177.07969    173.05847    162.74394    155.35519    152.54276    145.43117    138.39067    131.95906    124.62959    130.78104    133.52087    138.96429    147.89954    151.73097    145.61814    135.98232    134.50829     127.1357     124.6234    122.55841    123.26037    121.02783    125.09005      122.335    125.63328    120.83917    117.72778    115.71312    117.16668    114.76401    122.31679    118.39244    118.75247    115.34245    114.88733    113.45405    118.32984    122.51137    122.36712    128.00667    123.02632    118.98751    116.86347    114.23098    125.35533    122.24048    127.83908    134.21242    131.88822    127.62564    118.84561    117.15299    116.23444     116.8557    113.69524    115.75692    121.89541    123.81374    133.57904    148.32895    154.42255    153.01304    147.26621    139.55956    130.82946     123.6832    118.69769    117.40725    116.85385      116.899    113.18245    116.28675    113.15531     113.6523     112.7865     111.0873    111.78199    108.76783    113.48773    113.18966    115.22468    113.02157    116.26287    114.41143    113.53999    119.26417    119.64683    116.90277    120.53577    118.25635     118.1659    115.79653    117.46223    119.16558    121.80829    116.88021    119.40683    118.01216     123.1573    122.43448    121.97778     121.5167    120.06498    116.88344    117.73237     115.1616    113.08838    113.84011    112.61496    111.25049    112.67089     111.6157    114.74197    113.59836    113.67799    113.30187     114.7658    113.23321    118.01355    119.05767    122.26901    117.91696    120.75007    113.78936     117.6605    117.57691    110.76562    115.40839    114.66106    113.35015    116.35654    118.28001    117.45771    114.25236    116.59536     114.7988    114.08757    110.37536     113.8492    111.82971    108.41633    107.40606    112.77813    110.38647    115.56106     111.9785    112.21313    114.22281    113.04948    116.75697    113.35433    118.01891    115.26895    115.64488    114.70535    114.59841      117.242    117.80403    117.13109    116.38646    117.44497    112.26638    111.59125    109.80861          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       26.00    271.65407    260.29598    253.04287    239.22624    224.00068    225.48163    214.88918    216.15021    208.80533    210.18323    199.27563     189.5088    193.17285     191.4496    188.40155    182.36623    182.73877    174.43282    177.46479    176.68619    172.61039    171.99611    168.08714    170.91415    176.49315    181.43108    197.29768    203.44774     192.2162     173.9023    168.11857    160.75642    161.96064    155.04556    155.51918    151.86855     155.9131    152.79537    153.64928    155.72493    149.78697     149.8699    148.20881    147.45622     149.6976     145.9153    145.93418    143.50675    144.31991    143.51364    145.49958    143.28914    144.13422    143.40634    138.91305     144.6851    159.74856     171.7177    178.95667    174.59874      157.362      145.646    140.86242    137.34059     136.9237    142.46504      141.059    137.09641    141.64022    147.02366     156.3357     168.7446    173.64454    165.56955    154.47505    157.74532    173.32117    182.63061    186.55863    177.67658    171.76659    162.72843    162.74085    154.16159    146.58042    138.48449    130.48088    130.26344     132.3977    130.92839    138.24771    148.17294    149.34878    141.14733    137.71542    130.23834      127.942    119.39397    123.27304    122.64796    122.92384     122.9396    121.38728    121.04904    117.72715    116.63515     117.3367    117.75322    116.73335    115.69398    117.88522    117.14504     118.7286    116.36665    118.85821    117.79109    120.20467    126.46508    123.53836    122.02747    122.46207    117.59379    117.36462    117.09509    126.17226    126.09474    133.53897    131.52906    128.73885    121.33183    116.37312    116.12871    114.68603    117.25603     118.5587     118.0948    124.34493    134.70311    146.73335     152.7517    154.49599    150.70592    143.76714    129.60751    125.56517    120.99003    121.04482    118.32102    117.43942    112.77957    112.84254    114.25581    113.78937    111.37755    113.16653    112.35494    113.64272    113.72768    115.41081    117.67151    115.85178    114.64212     113.3885    116.02449    118.67191    119.50512    118.55933    123.68512    121.31438    119.42454    114.39498    117.70389     115.6127    117.45668    118.37412    119.94584    119.03728    116.71637    121.54705    120.05429    124.55538    120.29708    120.30457     115.5163    116.74299    116.18606    114.77486    116.14065    113.57367    113.11536     112.1377    111.57377    115.43936    119.00801    112.74158    114.40109    114.78108    115.15064    119.24842    120.38984    119.50395    119.78112    119.28094    115.03407    113.61065    115.59079    112.84746    114.23855    115.88104    113.02575     117.0226    117.66391    115.58073    119.69201    112.42311    110.62284    107.87593    106.98647    110.04625    109.85419     110.0364    106.35193     107.9471    111.47578    111.10067    113.58961    114.34904    111.18639    111.26657    112.76547    114.80006    113.77215    113.97149    108.38105    111.76484    101.29332          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       26.50     272.3419    262.92522    254.92398    240.99148    227.08266    225.93607     217.0177    214.92803    211.82411     210.1069    204.33001    186.75442    188.23804    188.38309    186.60424    186.57262    182.38103    174.29363     179.9674    176.23721    174.94728    175.20339    168.13033    173.29748    172.54846     179.5335    195.56413    200.21078    193.25922    174.31365    169.84036    158.59162    157.42568     156.0973    159.14144     154.1056    158.84282    159.66884    153.63622    149.64345    153.46097    148.70502    149.15678    146.50924    145.61909    143.93026    139.50515    143.44626    148.39509    143.27504    147.55807    144.09418    143.69463    139.59583    141.97213    144.97807    157.33762    172.89694    177.45481    173.30774    158.35547    145.47265      141.961    140.55763    139.19775    139.59668    138.92101    135.80034    141.62867    146.00702     159.2898    172.85609    171.28188    167.05002    159.05424    157.21842    167.70396    181.59077     182.1297     178.4641    168.32242    166.35584    163.27962     156.0736    149.32387    136.48449    133.48547    129.77864    130.85902    134.53129    147.01007    146.33112    149.45052    145.47388    140.32974    127.94036    126.63428    123.69048    120.21055    120.32355     123.0361    125.67677    125.38376    122.51071    119.61422    118.47861    119.10135     115.0067    115.67474    115.91449    119.63656    116.12127    118.34165    116.29017    114.48775    118.78746    119.01084     123.9161     123.2912     119.8796     118.8781    115.69433    119.72271    114.30738    125.11301    129.65822    132.04924    131.57294    127.49615    122.42552    117.11293    114.62988     116.6944    114.56229    119.66606    121.46303     124.1291    133.16078    144.93699    152.56922    155.11657    150.00698    140.73184    132.83651    126.33293    121.03578    123.85832     119.6015    118.00647    114.52297    113.31497    113.84808    114.57073      112.611    113.54516    111.93711    115.77717    109.81468    114.42205    115.58717    112.65074    115.97028     114.3137    115.89434    114.45067    115.37263    117.00124    121.70183    119.91287    120.38667    117.64231    117.15498    118.15214    116.57495    114.61295    118.32058    120.51557    114.93027     122.1032     120.3639    121.36775    118.86312    118.31005    114.36993    111.32601    110.34118    115.74329    112.94343    112.91805    110.86434    111.46661     108.8596    110.19846    117.08666    113.84581    114.43556    115.82396    113.00885    118.32789    120.13926     116.8827    118.23318    116.30283    117.52884    113.34394    110.26287    111.72979    113.10378    113.67248    112.54238    114.81937    113.89779    113.54891    110.76703     109.3768    113.02937    107.35198    107.37718     106.0596    108.74283    106.93966    105.91558    105.92656    108.77684    107.61965    106.07761    105.97199    106.92492    106.45723    104.46108          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       27.00    271.87215    261.21984    253.65256     239.6144    230.05672    226.79535    218.94849    214.29015    214.72515    207.87415    199.63914    190.41633    189.15456    186.32712    187.03941    189.44204    178.27374    175.01214    184.05367    171.43014    174.70754    170.98714    170.58996    171.25616    171.35308    178.10834    195.65588    205.22193    199.10719    174.96751    166.89023    160.86979    155.16146    155.94539    160.00451    156.19839    154.58855    149.13507    158.07456    149.11734    151.32751     148.6467    153.48836    146.97356    147.60364    145.21822    145.72356    148.41127    142.00318    137.55187    143.46278    144.78059    145.77459    143.89183    136.96901     144.8783    153.58452    170.33538     178.6791    172.75158    160.58331    141.44624    137.98401    135.04378    136.74992    136.47488    138.62966    137.41277    141.00605    146.70471    161.29054    170.26088    175.03232    167.73218    162.45134    157.82313    163.10853    174.25486    175.39591    167.53844    162.03652    166.49443     161.9023    156.71837    146.74829    137.19697    131.97787    132.74185    129.61878    132.77955    137.19308    149.31653    153.39268    151.88945    144.24395    132.83991    127.69764    126.96412    125.37578    123.07287    123.56179    122.52043    124.51937    122.40924    117.68851    119.87795    117.56949    116.62253    113.86568    115.62307    115.58226    118.42292    115.62698    117.24749     117.3609    116.88503    123.57541    122.40311    124.09619    122.39869     123.2754    118.64168    117.94451    118.50179    123.35593    127.92714    131.36581    127.71606     127.9259    121.94372    119.17853    115.68305    115.15924    111.50716    119.39877    117.92034    122.83564    130.58966    145.66064    152.22707    155.86067    154.92004    143.22537    134.53685     123.8517    121.05149    120.79204    121.38909    116.34468    118.76724    111.83163    113.52603     109.9971    110.29616    110.84272    111.06118    110.02935     111.3397    113.28005      117.248    119.94555    113.99003    116.09382    114.44993     113.9003    115.97881    116.54667    120.63348    122.23792    120.65762    118.83962    117.81223    117.69782    117.44925    118.23571    116.66916    116.97772    117.07181    117.94658    120.53758    121.33807    119.22079    118.43494    118.08822    113.81543    112.40047    113.22554    113.14366     109.0808    107.32534    112.06287     108.9021     113.0408    115.00816    118.26758    113.78533    113.85268    116.98642    115.88273    118.85604    118.98668    120.64371     117.7976    111.53859    107.73488     111.1325    112.22296    109.85067    112.08989    114.50729     112.4786    114.98949    111.00978    108.15959    107.93317    106.69795    106.31264    104.00123    107.73304    104.99142    104.12578    102.79292    106.35744     97.45821          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       27.50    273.66599    262.28649    252.71761     236.9362    231.83097    225.45259    217.76941    215.31877    216.82392    204.14663    200.51672    193.07346    184.54766    190.81186    190.80323    186.91155    180.32119    177.71532    179.97328    172.30351    174.44738     167.3352    173.59453    170.64352      177.259    180.21787    196.26265    206.23539    196.84595    177.19297     166.5692    165.86795    155.52599    156.38387    157.81896    151.26963    154.19479    151.26193    154.25938    149.84319    153.16681    147.33559     148.4028    149.28938    145.07502     146.8507    145.34439    143.14588    143.58693    143.08118    147.00986    145.15629    145.44973    145.31057    139.67931    146.13906    153.05065    169.09312    174.40879    173.36822    160.79115    142.42456    142.86062    139.99715    139.59605    140.72062    140.23267    139.38273    141.28495    144.39124    162.05757    169.61084    173.24733    168.19038    161.42901    156.32071    158.94902    165.08548    168.03142    161.78179    158.60642    161.77039    162.13286    154.87549    144.20609    137.52786    129.05207     129.7136    132.79191    133.91807    142.38857    145.95876    157.64099    158.08334    150.20325    142.60294    135.62042    129.96747    127.48513    123.16699    125.26288    124.91888    125.21216    119.47737    118.48634    120.53083    117.91366     112.7786    114.96279      116.843    112.67774    120.14372    115.06533     116.4307    119.96362    118.79935    123.68543    123.39107    124.68247    121.87933    123.31499    114.34455    114.80374    121.32129    122.35194     128.2855    129.87068     132.3357    127.85583    120.40656    120.65308    111.59541    114.42546      112.517    115.39568     118.6074    122.72481    132.70693    140.40124    153.66044    152.32554    148.30432    143.03106    128.14265    128.10696    120.81737    120.20128    118.67259     117.3536    116.72609    117.70043    114.91374    116.23806     111.1296    109.00526    110.10617    112.56672    110.47319    110.89539    114.72281    117.29191    116.29152    113.96497    115.93094    116.26854     114.8935    116.03391    117.76322    120.85043    120.93748    121.71756    125.72252     126.2276    120.61895    119.13864    120.86419      121.751    118.33439    119.76731    118.63367     125.0504    121.14758    117.81458    117.69662    118.21816    110.96062    111.01591    110.88535    109.70216    107.59469    108.55393    106.77181    109.72955    110.00528    111.47869     116.4369    113.96742    115.50561    115.13373    119.15106     118.8413    118.20674    116.15765    112.63093    111.24914    112.60637    111.03882    107.59311    110.87803    110.23909    118.18331    110.46528    110.46907    109.70179    109.26099    103.58363    103.57871     97.71715     93.62767          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       28.00    277.74952    266.32727    253.78148    236.02596    232.41871    224.89972    215.56163      218.337    219.24728    206.58111    203.18499    194.35144    186.79708    189.38061    189.76765    189.28903    185.37878    181.88367    174.84611    172.93774    174.03806    171.72991    174.50406    168.62983    180.25873    185.03629    196.65415    204.34135    192.67438    175.48238    165.91317    162.67738    153.29755    154.50384    157.17832    154.73472    152.28117    155.23912    152.78509    151.93228    149.43886    146.64261    146.48689    148.64504     149.5289    150.87074    144.50953    137.21486    142.24284    146.50764    143.36503    138.50286    144.82086     141.3038    140.26067    149.48044    160.29753    173.68343    178.02879    172.61267    156.59125    150.23859    139.43969    137.73937     138.1803     139.9416    140.67215    142.72015    143.93651     144.7755    162.47216    176.80788    172.19233    168.17563    159.21443    157.62908    154.10232    155.57849    158.61311    155.10036    154.94669    158.46173    165.06263    150.65486    145.91349    140.55336    131.66899    129.67796    127.53777    133.53662    137.52879    147.04471    153.01631    152.53504    152.87438    148.23838    143.66999    132.61053    131.59195    121.11204     122.9916     126.6162    125.54269    123.01334    118.01393    118.16758    116.01915     115.9655    115.17336    112.97744    116.14478    113.42657    115.24027    115.94967    114.94794    117.88059    123.49228    122.55597    123.29004    123.84461    119.11584    114.66634    118.53273    118.61687    124.08876    126.00285    131.80636    129.16485    128.78531    121.03384    121.61523    111.49116    112.16702    117.94989    115.59349    119.51103    123.68163    136.87977    143.46892    150.49453    157.31132     150.5459    142.95077    130.73265    125.40175    119.47371    121.50685    118.75401    115.50051    116.52312    115.43886    111.74309    110.88126    110.12608    112.13424    112.37281    112.54422    113.50444    111.76628    112.64709    113.61366    115.17658    112.63137    116.58558    115.75737    115.40972    119.34827    119.07353    124.58836    124.78104     128.1867    126.88402     126.7441     125.3634    122.21327    122.56644    122.19727    120.32039    124.29592     119.7291    124.26071    122.54501    119.40387    117.60892    114.84088    112.60815    107.02701    109.72304    109.28682    107.27101    107.71955    109.30152    108.56842    108.49333    111.15659     112.3002     111.5431     112.5702     116.1667    113.92734    115.20627    115.35527    113.41863      111.337    111.39115    109.17043    108.09406    111.46659    106.99123    109.51965    108.33202    105.49668    103.21268     97.94307          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       28.50    281.04652    268.16327    257.16316    238.86018    233.49602    226.19356    214.60008    219.25116    221.86298    213.01444    202.63503    193.90009    192.34913    188.80076     189.6615    187.36032    186.53754    184.11023     175.5248    174.09629    174.85865    174.66349     171.7805    168.90602    177.99004    185.74223    198.58789      207.773     189.0847    172.31972    161.95521    164.33591    153.84799    154.67334    153.36224    154.63226    154.22219    154.53522    153.35475     156.1778    149.65797    148.20561    143.74094    150.63593    148.13826    145.71933    144.81937    146.05284    150.51073    140.11428    142.10634    143.88402    145.18096    140.41149    140.43877    147.74186     158.4901    171.71787    178.41395    176.45946    159.12444     151.8775    148.01226     140.9135    136.28276    136.17913    139.64537    139.65082    143.52633    148.72656    159.16222    169.22977    173.10441    169.92827    160.46841    155.31776    157.02063     159.6104    158.44115    156.24305    157.02495    164.09796    168.14978    158.65364    149.49594    141.31407    131.74596    130.84559    128.99102    131.86765    138.31617    142.79867    147.67242     149.4615    145.30655    144.89625    146.06022    135.20104    130.48693    126.93366    122.48157    124.63678     121.9772    123.07202    117.34135    118.37222    116.92679    112.62497    117.80986    116.13273    116.69864    112.82471    114.55308    113.85655    116.84034    118.56032    122.11492    124.97639     123.2138    122.82321    117.89568    117.28225     116.2604    121.06276    120.41173    130.79438    131.98443    128.33968    126.49076    121.36263    118.91659    114.73469    115.75093    113.94668     114.2355    119.59056    125.00592    133.50332    146.55272     150.4541    158.42385    150.72656    144.01595    134.75023    124.08162    122.35208    117.70108    119.59321    119.18831    119.16574    119.97665    112.83572    114.17268    111.15833    110.31375    111.63086     112.3694    111.60977     112.7425    112.38764    109.09885    112.12422     114.7143    113.03861    115.20148    117.34874    116.38246    121.68862    126.16315    128.07912      136.286    137.85086    132.19299    128.59697    125.57124    120.90061    119.16905    121.15954     119.6165    123.39462     117.3918    119.79639    116.78139    113.51696    109.93453    106.78336     107.6736    109.75819     106.2179    103.05447    108.77272    106.08616    109.99921    106.84892    110.53979    110.70637     115.3735    115.74739    113.47445    112.92542    110.34202     114.4538    111.84399    108.27877    109.19423    104.89509    103.39637    103.00961    100.83388          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       29.00    281.13498    266.78875    257.51626    243.11972    237.45673    227.49668    218.91382    214.30038    218.66932    215.01237    205.04084    195.58814    191.63125    189.02478    187.73352    187.10953     188.0837    187.29486    176.56601    175.50821    177.79413    169.02797    172.54508     172.9389    180.00206    180.90688    200.28846    209.02584    194.63723    181.08118    165.09473    163.83158    161.27657    152.82946     152.1139    156.66389    154.72976    151.99403    152.99553    154.91277    148.66506    150.42781    145.13645     145.2369    146.72371    148.92668    145.82964    146.03174    148.64299    140.74621    139.43525    140.76586    141.42644    143.55167    144.51242    144.86907    159.82319    168.57174    176.95618    171.35882    159.84603    149.13676    140.01917    138.46773    140.91185    135.62102    140.02697    141.65209    144.95938    148.49789    156.24593    174.11126     175.4292     169.7757    160.01695    156.36121    153.62325    158.89033    156.11753     159.2605     159.6505    165.18672    171.34549    166.07085    145.92559    133.83888    130.04851    132.34495     131.8235    135.87752    141.26422    141.56231    143.80184    144.60454    141.40517    137.00445    136.60045     133.2277    129.39117    127.32623    127.17485    124.08412    124.96131    121.77488    118.78503    117.00232    115.41297    113.71063    114.06482    115.70206    116.86588    114.44099    113.81632     115.5225    116.16689    118.27515    122.78503    120.26127    127.04156    118.10007    118.62767    115.44939    116.45756    118.58807    120.77732    131.05506    127.35807    132.30949    126.91605    120.73983    117.88557    114.72905    115.12156    108.95459    117.35827    119.17496    125.78556    137.09541    146.71907    150.92896    151.06788    154.41581    142.37976    131.10441    125.85093    120.34205     120.1352    116.45511    121.85039    125.61855    122.29702    119.28992    116.32772    112.41517    108.75383    108.52796     111.2441    112.74162    109.24012    112.65924    111.32246    110.98311    113.33197    113.32918     115.5358    117.58568    115.41088    117.21566    122.03221    123.28281    126.67223    128.82675    126.70089    123.84607      118.149    117.12942    119.52403    121.93143    122.00454     121.2602     118.3899     118.2459    117.80061    112.57226    108.42999    105.37046    107.71735    105.27208    105.53816    107.98187    105.98363    106.99202    107.11635    107.87899    107.58068    109.75245    111.18519    110.54671    109.81386    113.67142    110.38714    110.56854    102.41044    103.25804          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       29.50    277.96031    261.45046    257.92372    248.35302    243.59137    228.06861     220.4307    209.69911    216.40098    212.72633     209.0005    198.22992    187.00558    185.51593    184.30905     187.4542    184.60512    184.47957    177.40432    173.69267    175.96256    170.69217    173.34658    174.41043    176.92626    183.53283    195.97366    200.81367    196.93605    180.29814     160.1861    162.59733    165.98202     150.6725    153.03422    154.20525    154.08744    151.10192    155.09987    153.47071    147.66098    145.11764     147.8762    144.92461    145.72346    148.80162    144.77132    145.81282    141.72811    142.55223     144.5173      147.476    142.84018    142.92106     139.1628    145.83136    160.85167    169.10225    176.30379    171.44329    156.96599    149.05901    143.35163     141.8204     140.7047    140.30703    139.44844    141.84026    141.51858    149.16624     161.3986    171.17325    175.52632    165.89375    156.06968    150.92482    156.51823    163.04337    161.93997    160.10914    161.82251     168.6864    173.99193    167.50656    148.82846    139.93296    130.38565    131.80488    134.06681    135.21282    141.17936     144.8564     144.1069    141.95228    135.33357    134.54418    129.70209    131.61532    130.57557    129.29889    127.23923     125.8205    123.61422    122.45181    118.89725    115.94079    110.49465    113.25353    118.18912    116.48873    115.89175    117.61092     113.8185    115.64594    116.06871     119.0113    121.61046    120.62628    121.28638    116.26974      119.251    115.26119    114.07693    118.81636    121.00164    128.38485    126.98709    126.76116    128.02809    122.65611    116.10407     116.2462    114.61206    112.51295    115.78727    120.38447    123.53734    134.45682    146.29291     155.5617     153.1982    147.82088    140.48023    130.29037    123.46138    116.37715    117.87507    118.42048    121.94208    123.64417    122.24614    120.98173    117.55295    116.22943      109.208    110.17382    112.58035    110.56185    113.09083    112.72381    114.42464    110.24875    112.52998    115.33509     112.0669    111.93989     112.8685    114.40737    117.95672    119.67097    121.66482    117.76429    116.56451    118.68723    115.04272    119.11808     119.8891    121.66646    119.48073    120.06457    117.81135    114.72904    115.91365    113.19448    111.37522    109.89224    106.07945    104.87366    103.25386    106.88859    106.45097    107.31769    106.17592    107.35042    105.49918    107.11048    108.42246    108.48366      103.839    102.96249          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       30.00    274.96387    258.79336    256.91257    252.08193    237.35489    228.54368    217.96946    211.04974    214.99217    211.51234    210.24359    197.08647    190.74101    187.49208    186.04767    189.04549    184.20463    184.03024    178.28704    172.35303    175.39279    172.16552     173.7329    173.90344    175.01524    188.98823    193.69757    199.17395     191.9136    178.93807    160.56144    160.01195    165.11049    150.84366    151.00727    153.89878    149.16064    152.61023    153.53388    153.80695    154.51389    147.41655    146.14753    143.98221      149.893    150.02456    148.05123    152.90979    141.43601    146.05318    140.43405    142.41177    144.98087    142.77631    147.52927    146.08833    154.09112    169.12577    175.35607    169.18177     159.3424    145.70121    145.35768    139.49428    139.58364    136.19866    139.69987    142.03897    145.06045    150.12976    159.02769    175.81472    174.27041    168.99094     160.5329    156.63852    159.89663    158.71245    160.06478     161.8401    164.10179     164.4797    168.84784    161.84429    153.04029    142.37155    137.52562    130.43958    130.94306    135.47662    147.29879    146.27914    146.98422    141.56452    139.92694    133.07615    129.62956    131.56888    137.78366     136.2034    134.16158    125.53647    121.27962    123.91319    121.15546    117.94289    116.88088    112.97478    118.51696    115.50729    117.07982    113.17932    114.14104     112.4018    117.14086    116.19894    123.18474    121.88316    119.87977    118.83558    115.65622    117.09618    115.63494    116.30917    123.69548    127.54499    129.76968    127.95212    126.72637    120.82853     121.5618    118.68451    112.32728    117.30488    112.15948    115.82828    123.53842    132.89975     142.6348     149.9275    154.35143    148.39083    139.38256     133.9794    123.15671    119.81334    116.66143    120.33551    118.21187    120.02396     119.1345    115.70435    118.45931    112.92671    113.14721    108.50589    110.33119    109.88385    111.94137    109.49224    114.79711    105.82794    114.25487    109.08678    111.97061     112.4508    113.71129    117.28554    114.73859    114.98492    118.82717    119.23488    115.63059     116.5018    114.76965     115.3995    119.60689    122.51023    121.29011    120.21187    118.36338    110.79809    111.69027    112.83618    113.78634    106.36235    107.04109    105.21687     101.3774    102.16497    100.88776    103.29426    102.28321     100.8087    101.84453     96.56393          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       30.50    274.30559    257.25626    255.38147    252.65045    229.21184    228.02892    215.99633    214.79962    212.73814    212.07761     209.4853     193.5926    195.28853    188.55826    188.95103    191.24759     186.2752    186.40651    178.93784    170.93048    175.49439    168.59522    170.55394    176.12793    182.48707    197.13015    198.81797    199.77083    189.63486    177.10885    168.11761    158.37739    160.19613    154.53085     151.6868    156.32832    151.09311     156.6333    152.12805    151.93837     152.1434    150.94813    147.11184    142.80298     149.1407    147.37503    146.32226    148.31731    142.00962     147.2233    144.48636    143.49617    144.70245    143.67721    149.89192    150.09184    157.09381    167.64058    178.77805    175.21538    160.51198    146.35748     141.4971    138.73332    136.81857    135.29714    143.25924     143.0577    145.54185    152.52327    160.43965    173.06756     176.7351    172.16736    160.49767    156.74682     157.8887    160.00096    157.13157    159.37055    158.04063    160.79648    165.72711    160.89109     150.9717    138.24902     132.8693    130.67473    137.32898    138.33037    146.22673    147.41843     148.3639    145.43106    140.12119    135.51522    127.94947    136.29603    140.42514      139.469    133.75016    128.02661    125.30004    124.79271    120.18579    115.84496    113.24959    116.63182    117.67524    115.22133     114.8018    115.45969    113.79445    112.34483    113.07793    117.03613    120.00967    123.70372    119.12769    120.33837    115.95524    115.88556    114.32827    114.87749    122.75855    128.55892    130.52734    130.45742    120.93288     121.1397    118.96558    117.92591      113.069    112.05392     114.4699    117.04372    123.68727    132.01268    143.55649    149.52757    151.88015    146.97418     140.5546    131.26193    123.45123    118.54044    117.63884    118.40131     118.1251    117.61552    118.05981      116.229    111.73102    109.91794    110.91662    106.22777    107.59812     111.9529     110.1374    110.61949     112.5921    109.19654    109.56754    110.87177    112.01727    115.14846    113.66311    115.35121    113.93357    115.33241    118.33475     114.3357    116.63469    116.58604    118.25823    119.59008    119.22023    124.35398    121.45902    115.64703    114.43885    110.16298    106.72203    109.53686     106.7002    107.99782    103.54917    106.14007    101.02042     98.76003     98.99554     97.52151          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       31.00    274.61002    256.15714    253.65144    248.61209    224.67342    226.76147    217.82752    217.39959    210.02047    211.82201    207.78326    195.77675    194.66099    189.36772    189.06459     189.5745    185.55034    185.15697    180.27492    175.23693    175.91438    171.04541    174.31932    170.97931    181.83145    191.99543    201.35417    200.74577    190.67876    176.68934    167.48576    158.89395    161.89701    154.76849    155.18669     161.8305    152.31563    156.21154    155.00492    153.64531    154.80493    147.07841    150.91695     144.2406    148.67394    145.87494    148.11398    146.64658    143.60418    142.82236    144.82029    140.59346    146.36593    147.00598    146.68861    152.20561    160.62016    171.41898    179.67133    169.62518    160.43191    148.46376    143.64061     139.8783    137.35991    139.22559    142.68373    139.11121     144.8608     149.9209    166.79613    173.24888     174.0604    168.82841      158.249    155.91037    159.62755    161.98303    160.78133      158.036    156.90891    160.11584    159.30149    153.75333    150.44541    139.67914     134.7312    136.06328    133.30783    138.06178     141.2152    148.62619    148.04363    143.04325    137.23287    137.62245     129.7543     135.7812    138.91067    138.92975    128.93959     131.9686     123.9914    124.66505    119.77455    117.59142    117.66182    118.05918    114.84249    115.75547    116.36411    117.79333    114.84477    114.29211    113.51207     120.4554    121.75957    119.61577    121.05756    120.75665     119.1298    112.43764    117.15527    119.83838    119.92684     125.1812    128.00882    127.19137    124.15656     122.1137    117.07846    113.41959    116.79316     111.9381    111.90681    114.50751    126.27486    131.73881    141.48362    152.07158    154.18053     150.6221    137.45915    129.79433    126.41709    118.14713    114.35609    121.44974    124.50046    117.33285    120.79545    117.25925    113.05992    110.93905    106.68557    108.66542    108.01844    110.64351    109.74639     109.9638    111.05584    110.87324     108.6531     110.0379    112.98979    113.30766    114.78699    115.64423    115.11709    113.01271    115.72706    116.54141    117.77958    114.03581     117.1576    119.65781    119.56047    119.63915    120.68001    116.01212    111.96922     112.6305    108.43531    103.91515    105.51216    102.24161    100.68026     97.94545          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       31.50     275.4362    255.43522    255.33051     246.0059    230.68712    225.85525    222.57168    216.36206    209.88989    206.41649    207.44462     198.6546    192.20926    192.44754    187.38647    188.37614    181.98616    181.44919    184.05765    181.02583    178.98638     169.2549    172.68266    170.10237    179.67936    195.03728    200.58585    197.17083    190.21754    179.72812    165.46353    165.55705    157.82873    163.07982    152.74156    152.06988    156.52722    156.29499    151.99699    149.28935     152.4521    148.72631    146.10875    147.18902    149.37528    145.84363    143.50533    147.23019    143.88971     146.7308    143.59186     142.7188    141.28627     143.3184    146.78284    151.91075    159.70909    169.96943    180.07541    169.01039    158.65872    148.97175    142.49909    140.30876    136.09399    139.40092    139.45466     140.5003    147.32732    150.90488     167.8705    177.35853    178.03208    170.52188    159.49744    156.03429    160.10986    157.38367    157.69314    155.08225    157.80007    159.01467     156.4797    155.64007    148.48885      138.656    133.95998    130.53233     134.0291     141.2785    143.72912    147.60446    151.70995    148.11368    141.17854    137.39358    135.53588     134.3495    134.14661    130.11831    128.21076    123.96766    122.33833    122.52449    120.97816    117.24302    118.73497    116.16341     115.2633    115.84939    113.21786    114.33175    114.07863    116.71297    113.90748    112.87425    120.11924    123.55028    122.82463    121.32926     116.5367    113.47401    113.99636    117.54112     119.2638    125.84628    129.90642    131.28873    127.35074     120.8605    119.11582    113.21012     111.5536    114.86246    113.56683    116.42178    121.71375    130.76862    140.34486    152.75334    152.58509     150.4573    138.48897    128.87159    124.29642    115.93606    118.76608    120.43095    121.14546    122.33778    121.38508    117.42401    116.68846    113.51962    110.25999    106.72127     106.0876    107.79482    107.38536    108.64344    109.89524    110.01284    111.54652     106.2518    106.90947    112.18616    113.55105    114.54127    114.64008    116.19308    111.25569    113.90472    111.47709    113.43225    113.37127    113.62983    115.84208     117.1672    117.57476    115.75762     112.8878    107.26265    102.06105    100.58529          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       32.00    275.66008    254.94185     256.5951     246.7731    234.01058    226.83272    221.92937     215.5893    211.18855    202.85479      207.804    199.27457    192.82186    192.94218    183.05246    187.47825    184.40452     181.2586    187.47589    176.41624    173.83136    169.88315    174.36557    175.30613    181.90093    190.10199    200.37722    192.42022    187.37651    176.79793    167.70338    167.02542     160.0144    164.74339    152.95418    150.54304    155.19471    152.02749    154.89108    150.77088    153.59989    148.23634    148.13949    144.38832    147.12508    150.85456    145.77651    143.70396    143.99428    138.85512    144.14726    141.79414    143.42434    143.47434    144.02489    152.09756    157.76195    171.28487    174.72822    170.43244    160.86743    148.16698    141.88539    140.96247      141.797    138.19467     136.0856    140.80798    144.40411    152.47921    166.30946    172.37277     171.9869    168.21868    160.66669    152.83745    161.05968    165.38294    161.65106    158.10115    155.47782    154.79741    154.84365    148.88562    148.03855    142.17388    134.39061    129.43589    136.06614    138.82125    144.15657    153.71488     156.2249    145.68581    138.57386    137.47379    137.87776    131.81806    130.50462    126.72066    126.29376     121.8409    124.17318    124.09845    116.47284    117.87129    116.03381     116.9124    117.53832    113.20421    113.71091    114.82491    114.57407    113.65443    116.83164    113.00929    122.26768    120.78669    121.51484    117.08178    115.90761     115.0534    112.83391    116.19157    121.84437    126.51559    129.59188    126.84034    126.21193    120.49625    118.72146    115.40104    113.07787    115.25082    112.72505    117.13856    122.63652    128.27972    141.65553    150.83341    153.97549    149.83903    143.32373    132.29552    122.08502    121.47983    119.32619    120.73801    120.25412    123.99261    124.78842    121.41309    115.56378     109.5406    108.91743    108.13258    106.85776    104.80467    108.82889    108.60872    109.04675    110.52317    111.61038    109.65768    108.13057    109.79677    114.39586    113.59869    115.53161    112.04685     116.6738    110.75025     114.2401    111.37855    114.18027    113.69937    113.68268    109.48908    110.98969    109.68979    100.84739          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       32.50    275.28712    255.29928    256.77432    250.80102    234.99036    228.06667     219.3577    214.81854     213.0626    207.15493    206.45637    202.93095    193.69391    192.57257    185.10228    190.43088    188.75176    183.27856    184.46258     178.1525    173.20294    172.70579    172.24459    177.42249    175.76969    189.44173    203.32592    192.96278    183.52192    176.94984     166.1209    159.15306    161.29521    158.78236     156.5796     151.2536    149.91672    150.26417    153.37049    153.78865    149.05921    150.89004    149.18616    150.66385    149.27396    148.31055    146.17866    146.54461    148.09918     144.7449    146.49238    141.80803    142.52078     143.8009    141.32423    149.86249    158.09265    172.07544    172.32363     170.0356    161.41499     144.5427    138.20875    138.10931    134.04087    139.79983    136.57388    140.03828    143.52085    153.95353    165.50424    174.32566    179.24271    171.20286    163.69531    158.13243    165.24098    168.54255    173.72923     166.9319    158.94329    152.67783    160.04842    154.22864    150.10037    142.95417    138.20689    132.00637    132.30339    135.43635    146.32555    151.52457    151.24418    144.08961    139.01632    135.75813    136.48647    136.00089    129.41396    126.73724    123.22167    122.21713    121.10283    121.60856    115.85542    117.78101    113.12779    117.11444    115.39315    118.50517    116.32198    116.83174    113.84667    113.65637    115.49628    115.18065    118.79766    120.82466    121.93803    120.12452    119.27271    115.17658    114.79905    114.36761    120.49604    125.21858    127.13663    129.71172    123.60992    121.34628    114.07384    114.17435     111.0453      116.351    115.77121    116.27111    121.94206    132.51659    141.98696    153.28371    156.96254    152.44324    141.87439     135.9099    124.49509    120.54834    118.93357    117.97238    119.41993    120.11357    119.08335     116.9403    115.36703    114.35863    110.44914    107.24112     108.6531    107.22402    107.40597    107.27304    108.67935    108.61723    109.31885    106.17546    108.35332    109.28674    110.10833    110.89117    112.58631    110.72163    111.35101    116.72663    106.27731    109.87827    106.87459    106.51581    105.88159       98.249          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       33.00    275.10807    257.52975    255.82006    248.08796     234.9403    227.48629    219.86798    210.93108    213.35391    214.09682    204.92462    207.78483    193.67038     193.5558    189.51305    189.63228    182.81308    187.07347    178.28407     181.6675    173.60285     171.3989    171.70547    176.11059     175.4968     189.6145    201.07611    192.50783    185.31508     174.2495    169.57346    163.36349    156.56354    160.25329    155.42961    154.45181    149.38521    153.35959    154.21123    157.59463    149.36043    150.93965    145.92682    151.12054    148.20088    147.13348    144.44092     144.0938     152.0834    150.37572    142.09188    144.88419    146.89466    143.93493    142.23667    147.75446    162.27177    170.32153    172.00732    170.32082    160.71481    148.98838    144.97696    138.12066    137.64586      136.591    141.18421    138.50571    142.95417    151.17834    165.46133    180.18002    180.72032    167.83012    157.83938    158.96071    164.53892    178.85542    181.73233     171.2623    160.69029     155.9018    159.80686    156.39026    151.11693    143.50404    135.99328    132.27586    134.35547    140.13076    147.36183    152.12274    151.27637    143.57053    138.16705    133.91127    130.03645    133.95045    124.51489    126.70131    119.19329    126.09194    121.35744    121.95769    119.02655    116.47719    114.50541    115.15482    117.99577    117.75534    116.86091    114.10679    115.84482    112.62008    112.32126    118.61804    117.90654    120.30038    124.80228    120.91998    117.01461    113.96689    116.72241     115.4368    117.30095    127.75502    129.18826    126.92812    123.93944     119.4645    115.38032    109.68937    111.65773    112.04615    112.75848    116.35936    121.23583    131.58507    139.74628    151.42791    156.04815    157.16221    147.64782    133.11226    127.17147    119.91163    120.56884    114.17349    114.82537    115.57093    114.90558    112.83972    107.23914    109.28584    109.44335    109.59447    106.97163    105.82273    107.19072    104.95357    106.78754    107.92137    106.20883    106.44532    107.57244     108.7708    108.05475    110.22037    109.41754    110.95106    108.64177    108.50074    102.90315     99.48608          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       33.50    277.39125    261.36523    255.40378     247.6528    237.04824    224.68631     221.0327    205.69322    212.80832    213.42785    204.36654    211.64005    198.02357    193.08777    190.51916    188.10705    180.89773    187.49824    178.18804    181.13782     171.4394    173.78158    170.29163    172.80766    179.51474     186.9427     199.5415    192.96005    187.50271    172.89397    174.13012    161.93424    155.60675    163.12071    156.05977     153.8103    149.59817    154.35283    154.03083    157.81411    152.11779    154.88224    155.03783    144.85419    145.43316    147.96345    143.77822    150.20662    144.49475    148.23584    145.35574     146.3307    144.45857    143.50137    146.58779    152.98894    164.41884    166.22071    171.87048    163.93937    158.30069    146.47046    141.46461    139.84989    139.81203    142.22523    135.88992    140.77276      145.577    149.95548    167.86092    178.76331     177.2784    170.30793    161.76365     158.9092    168.41121    177.76586    183.38264    176.88375    168.93117    163.38588    163.51925    159.27996    149.71692    146.12343    136.03749    136.19517     135.2356    140.25996    146.39957    148.55527    146.60186    145.56618      140.045    131.97281    128.36465    126.96581    123.54161    119.25695    123.55859    125.74291    122.02717    121.19197    114.07361    118.09816     116.9734    116.95395    115.17019    119.80412    114.17041    116.92227    112.85132    114.57142    113.57746    115.17665    119.21469     120.2751     121.9547    117.38971    117.50124     115.3213    110.52204      116.892    118.95853    123.30331    129.10493    125.46246     125.2927    121.72513    117.83306    112.54564    109.38213    112.41623    112.47512    115.27459    121.84896    130.66697    140.34666    146.58345    156.76775     149.6625     143.5637    137.67768    127.45541    119.92366      113.822    113.67104    115.43549    109.17604    113.20737    112.93679     109.6175    106.31129    108.00116    106.14149    104.62107    108.76674    105.86818    105.20223    107.13103    107.52601    108.55395    104.59239    108.01291    106.96262    107.20931    109.88151    110.00359    106.59016    101.99246          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       34.00    280.15385    264.52434    256.95983     250.4592    240.42773    225.68676    222.23139    208.59177    213.28585    212.61461     201.8063    210.92298    200.08544    191.12285    189.92782    187.24951    184.35836    183.09607    179.93801    178.20092     170.1252    178.50192    165.83291    173.63973    178.16555    194.16172    195.70282     201.8252    190.92119    171.36953    167.08366    165.31514    156.01121    160.19443    157.55102    156.53165    158.77717     157.2242    149.64456    155.51112    151.45053    154.85563    151.28606    147.10801    150.47279    150.94328    149.82173    150.16423    146.10045    147.15776    148.19651    141.24597     146.5038    148.52951    147.54811    148.84231    160.10782     176.6379    176.25175    168.98244    155.54475     146.1756    143.66166    142.53801    139.13871    140.97853    138.89265    136.64258    143.83161    153.35668     164.8169    180.51369    179.89407    169.03639    161.09865     159.3543    166.84177     172.6419    178.01646    171.53412     171.2525    165.71868    164.93411    161.17552    153.52427    147.44837     135.0375    138.41452    135.91072     137.2296    143.78812    149.85178    147.84369    148.43911    138.19402     137.1928    131.66414    122.35043     122.6226    121.39529    120.67446    121.96592    124.40458    122.71101    119.25735    114.71917    115.10692    118.56876    118.58264    119.20709    117.93537    116.81856    114.95997    113.46318    115.33643    118.60345    118.36889    121.10508    120.92166    121.69835    116.68714    114.81475    116.62916    116.14475    118.60169    120.91859    130.66414    128.84515    122.46871    119.05904    117.50489     113.2596    113.53475    110.91709    115.00417    114.87565    121.12302    128.65766    140.62455    145.09276    151.06664    149.35082    141.61401    132.02949    126.39671    116.41566    116.62614    112.07503    110.99543    110.00847    112.63989    109.89565    108.42067    106.35786     103.2134    103.07933    105.43987    105.55423    103.07255    103.83232    103.97891    102.75236    106.69552    107.63956    104.09831     102.8733    103.32566     97.95513          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       34.50    283.40073    265.70534    258.89341     252.1555    241.10278    228.97249    222.08394    215.15296    215.20878     212.0783    203.70026    206.47135    199.86649    189.68101    188.35144    182.36682    187.86574    184.67069     181.9655    181.17619    170.94185    177.85349    171.04363    175.68145    178.52774    193.11083    196.96472    204.52694    191.17408    173.76244    162.23878    163.49836    155.76658    157.31538    157.51138    153.37144    154.98605    155.32195    154.34622    152.94898    150.94467    151.65699    149.20248    148.85551    149.00658     145.6251    145.67653    145.07255    147.42206    145.25001    145.92611      138.079      143.754    144.56846    142.65407    151.66464    159.46314    169.06024    177.06477    169.04859    157.86678    146.98271     144.7948     141.5523    138.25745    142.20467    141.27509    137.12425    142.14788    150.99304    167.49054    173.11752    182.94468    179.78603    164.08646    158.24234    164.14515     171.4746    167.71834    168.27893    165.06633    172.05172    167.63069    165.36468    160.23716    148.58309     139.8882    134.14623    137.44941    138.73997    141.99638    151.87173    147.30956    149.03702    139.49087    138.87418    126.11885    125.23161    124.98857    126.21402    122.46975    125.19853    124.86673    125.93992    120.58524    117.81799    114.10852    115.88257    115.27255    115.85865    118.71999    119.62722    113.68649    114.42981    114.81196    117.20339    119.16367       121.95     120.1773    118.59542    114.78029    115.47939    112.15809    112.56599    118.79249    125.99529    126.84833    126.68838    123.78003    117.28316    116.50156     115.5366    114.27391    111.28276    117.10421    112.80645    120.35274    126.83271    139.72788    147.38741    149.88385    148.14306    141.72082    131.50222     124.1465    114.35414    115.50433    109.57208    108.57474    108.94951    109.70728    108.52807    106.88828    104.58956     103.7218    103.51527    105.83251    104.86467    104.89991    104.07017    104.95028    102.89294    102.98744     99.68555     93.75734     97.22937          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       35.00    284.08402     264.8673    260.05848    250.75731    239.33852    232.49893    221.67815    216.68278    213.22124    210.63485    207.54727    200.72687    199.36888    191.12285    186.88008    186.42991    190.04688    186.70109    181.09327    182.58576    169.77793    173.88226    175.71235    177.68313    175.95535    190.90225    195.43586    200.12721    190.33032    175.48932    161.69159    165.83662    158.16672     154.9373    159.52874    153.36445    157.04309    154.12528    147.67745    152.22796    146.30956    151.96976    154.46579    146.12614    150.26066    150.02448    149.70421    154.39929    145.38985    150.49608    146.26875    144.04305    142.56787    147.04279    146.07095    150.81304    160.56684    168.59243    178.38375    165.34421    157.14411     147.2801    140.49983    141.59093    140.25499    139.20887    141.01227    139.11858    143.87938     151.2537    168.26643    175.90778    175.25543    178.61198    167.24939    157.16924    164.24775      169.782    167.31234    162.35195    161.63267    166.63461    171.43761     172.9623    165.95499    151.62413    143.92871    138.53846    137.37751    140.59586    143.54975    152.44404    153.16516    145.82062    142.15102    137.41005    128.32378    123.83035    123.86289    123.20148    124.97563    124.96285     121.9377    126.46175    120.82892    114.19704    117.79783     117.3856    116.13975    119.91151    123.59235    116.37921    113.92814    116.83156    115.39198    116.22962    118.44709    121.72013    120.82163    118.27618    116.63364    116.46159    110.16063    112.23984    117.35767    123.61848    127.14677    125.52456    124.79524    119.06718    116.16855    116.01952    110.50475    114.09579     112.1058    115.03676    118.63894    130.09933     134.8992    144.88702     151.4641    144.97414    140.38223    131.46653    123.46263     116.3177    112.31845    111.94184    108.47981    108.10291    108.26748    106.88914    105.96155    106.56111    103.75861    103.06071    101.03878    101.84577    102.41995    103.00181    103.09472     99.95651     92.59681          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       35.50    283.50338    265.36932     260.2451     246.1896    238.49286    228.74315    221.71729    219.04662     211.9879    212.69029    208.95898    201.59687    197.97762    190.17665     186.6077    191.06289    188.29427    185.10829    180.36927    181.36319    172.96956    169.49913    174.98367    175.24401    174.02388    191.85788    196.50598    206.85001    182.40934    175.99466    162.07137    161.72292    162.42429     162.7288    159.29992    155.55373    154.72485    152.87224     151.7452    152.81702    150.03942     153.0674    154.48649    150.65584    141.63653    149.36197    147.31896    152.86433    148.29093    148.90347    146.16028    143.76899    144.82893    149.91045    141.21639     150.7309    157.32099    169.50562     178.8853    174.92382    158.06743     146.9272    141.14532    139.14089    140.53857    140.82356    137.15425    140.53568    145.12043    150.54277    166.94315    180.46063    179.02486    173.01081    165.54689    163.00867    162.72735    170.58269    170.44976    162.62006     161.4406    167.71195    173.17707    173.09062    176.50559    159.96819    146.35062    135.32033    136.84612    142.86048    150.48074    146.75974     149.6503    148.28289    139.78278    132.05569    126.48943    127.00809    126.56539    124.62525    128.20656    125.64595    123.78248    126.43429    118.17089    114.34534    114.67855    118.49037     118.9204    119.43308    120.23058    121.26499    118.26035    115.86281    113.97394    113.64352    114.18408    116.22015    119.42801    119.27693    113.69685    116.04914    111.46254     116.0044    118.13049    122.17695    129.21585    125.78362    120.86019    118.92738    115.40994    113.76947    111.57759    111.93961    111.42813    112.14274    120.11568    127.10986    135.29026    143.74018     147.8953    149.24061    136.10573     134.3853    121.74846    118.53348     117.2257    108.74067    109.33532    104.03158    106.57666    105.89776    102.34611    101.85701    103.37547    100.31122     95.53257      99.2683    100.03487     96.86566          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       36.00    281.59493    267.22398    257.75714    247.16911    240.07194    228.39623    220.16761    223.11171    213.43971    213.44887    205.99048    205.01111    198.60116     190.5553    189.10956    190.85823    184.02785    183.40206    185.20461    181.67417    179.64193    170.81424    178.97969    170.31503     179.9959    192.56102    194.43705    202.44686    194.85339    171.11437    163.87292    157.91368       160.43    162.83419    160.68269    154.88477     156.7899    153.71555    152.85157    153.04158    152.63928    153.22426    151.60807    151.91386    152.76342    148.73668    148.00227    150.80233    142.67062    149.00636    149.45861    145.67948    149.41092    137.99864    150.71823    151.95194    160.02323    170.43691    174.14481    166.72351    156.22203    149.15293    142.98981    141.64048    140.21593    144.46752    137.13393    139.88788     144.9817    150.61369    166.62175    176.53841    179.30633    177.36717    171.24884    163.63036     165.3336    170.10285    171.32488    166.25325    174.31222    172.69645    173.40119    184.20172    177.27118    165.55504    152.88442     139.4119    134.89314    140.83615    149.91835    152.47164     154.9924    147.06336    139.94614    136.66126    125.69605    125.02955    124.34505    127.47718    129.56832    130.70871    126.02981     123.8892    121.26447    116.34636    118.25093    113.47137    120.94455    118.02291     120.0654    117.24692    114.73612    112.70124    114.81366    116.98608    119.67441    119.54066    120.95611    119.99822    120.54316    111.88303    116.32544     114.8476    120.15382    123.55792    125.87174    124.60914      124.205    122.69938    119.03702    112.96802     112.3521    110.95527    110.97804    110.63413    117.32779    126.04783    134.24153    144.16939    147.73056     146.3597    142.38019    132.23902    125.47802    116.75153    115.48106    112.51264    111.40828    106.47385    102.93892    102.45941    106.57535    105.07562    100.27783     99.86582     95.88608     92.01344          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       36.50    279.18293    269.40748    254.90967    249.97532    242.86901    230.47734    219.01564    223.61038    211.89287    212.50658    201.65917    208.18814    202.19008    195.35137    191.36296    191.30555    180.93422    180.24375    180.34252      182.018    184.18333    170.14607    181.84271    175.52387    179.36838    185.68527    197.50473    198.61459    192.77435    174.52332    160.48144    162.70385    158.52537    162.73467     159.9772    159.16834    162.67668    153.34058    151.80191    152.56772     153.6405    153.08749    156.85534    153.76275      152.915    146.55723    147.34741    149.27421    144.75039    148.31172      146.543    147.23873    146.51551    147.68559    143.33884    147.42268    156.97638    167.57482    177.10342    173.89705    159.51097    151.67185    145.16956    143.52717    136.98237    143.60561    136.06505    139.58573    144.38727     149.0479    167.61357    175.65649    183.27889    176.72222    167.27635    165.15811    165.99417    170.60187    174.57144    170.09466    173.70635    184.98466    183.18513    187.22159    177.41987    171.42767    155.36675    144.97363    140.09191    142.66794    151.06255    158.41694    155.79326    153.93222    143.10081    134.04055    132.31623    126.99397    123.51988    129.01588    130.25488    126.18834    128.56482    122.21858     121.8277    117.66721    116.51898    116.42576    119.51941    119.48741    118.69008    123.09122    113.31479     113.6079    114.33713    110.87415    118.83419    119.46883    120.27356    117.04843    115.63619    114.00758    112.62231    115.48962    112.16292    125.71251    127.47272    126.45433    122.40303     116.8602    113.39068      111.901    111.99086    111.25073    110.29331    114.53499    117.74741    122.21422    136.93187    142.34271    146.58326    148.89943    143.46878    135.85816    122.44842    119.02616    114.37973    107.91126    108.68254    105.48108    106.29582    102.72167     98.01146      97.6816     92.32073          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       37.00     277.7947    270.02927    254.17621    254.62339    244.44155    234.30386    221.11743    223.27438    207.55545    210.96391    204.52966    210.81029    203.14773    200.80871     190.4644    191.45868     182.5897    175.25431    177.30197    182.68754      183.444    172.26606    177.76173    176.80145    180.18938    190.88436    193.45301     199.9561    187.39399    174.41114    165.24096    159.34175    158.18956    160.13795    161.34109    161.13377    163.85003    160.91957    152.11181    153.03315    150.81462    154.44393    148.71119     152.1456    151.85829    148.54778    147.41139    145.52367    146.64458    147.76032    145.83516    146.97308    147.40674    144.92872    144.97827    149.84193    160.22877    172.14768    174.19977    167.88862     162.5885    147.71796       139.91    141.51198    141.46975    138.99451    142.85265    141.04041    144.06459    152.10212    165.52188    173.59817    180.31453    178.40731    168.36869    165.73235    168.88507    175.62102    179.49481    176.22879     178.9084    189.42511    193.26726     189.1923    178.27957    162.79245    153.54721    141.50888    137.92458    141.23936    147.35594    156.59708     162.6212    156.54542     145.9073     139.7104    133.62881    136.44808    129.61753    129.01856    125.66866    127.12364    123.99173    122.56515    120.47926    120.29163    117.21758     113.8052    116.64468    119.48066    120.71457    118.04656    119.86843    118.06223    120.84824     110.1629    118.44007    119.97902    123.08844    116.68186    116.21484    113.78156    110.90943    113.48249    118.78148    122.96135    124.55598      126.939    121.51249    122.11141    113.92487    112.29335    108.68704    114.23469      108.184    111.29389    116.05238     123.5856    134.09084    142.60923    147.54469    149.58604    144.97872    136.79644    127.92829    118.97068    117.23884    109.57115     110.3571    107.96312    103.74271     99.12035     92.99782          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       37.50    276.05651    267.55239    255.03205    250.08838    244.83894    236.15228    222.20486    221.21749    212.16414    211.81264    210.01694    207.92843    202.43363    199.75702    192.68915     189.3206    188.95685    178.83875    180.33005    184.63706    182.88657    178.52539    176.68991    174.04276    178.89645    192.65404    192.66006    199.93865    189.89901    178.26619    165.32777    161.78797    164.17627    158.86981    161.98119    158.63619    159.65261    157.53021    147.62309    147.57711    153.94684    153.86697    153.05796    150.41452    154.63866    150.10218    148.02246    151.52869    147.61038    147.34724     145.1149    144.09658    149.95248    149.08195    145.84229    149.51342    156.06576    173.12404    175.23694    174.00184     160.3527    148.84913    142.11657     142.9073    134.77769      140.882    143.57874    142.60058    147.67477    155.01803     161.9648    176.13088    179.84687    175.33075    166.63873    164.00322    167.52836    175.67596    178.38975    183.87377    187.57958     191.1935    197.88466    193.62052    175.57075    160.62614    150.46966    142.76891    140.56403    143.43033    150.73249    157.97933    164.36131    158.10283    156.64583    147.07342    148.10301    149.26919    137.89341    130.22664    128.53452    123.69453    123.57581    124.27813     118.8873    119.22273    119.36944    119.30957    116.80077    119.60256    122.60406     120.7937    117.39747    116.74846    117.23411    117.56522    119.71569    122.55397    121.88129    119.45309    114.75628    111.23463    112.87426    111.87884     119.2163    119.02554     125.1628     125.0955    119.86908    115.96545    115.03111    110.21604    112.16251    109.87422    108.97954    108.95863     115.1008    123.62608    130.96654    142.95076    151.80997    152.54959    146.82813     141.8602    129.94846    120.58643    113.19676    110.94792    106.93585    102.19068     93.40005          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       38.00     278.5605    263.91573    253.80215    246.86597     244.6897    238.97723    223.23621    221.95898    216.17915    213.74375    209.08333    205.63572    203.36103    195.97375    196.38622    190.23983    189.29029     185.0154    180.62405    180.65167    178.27502    186.03694    175.08797    177.40152     177.9186    190.51466    192.63044    199.32672     186.6222    183.45318    167.06448    165.62649    164.02743     159.5664    154.55103    158.24943    153.85376    157.25877    153.02876     154.2255    156.02533    146.72139    149.41003    152.03868    153.79025    151.07817    144.12558    147.53069    146.29756    145.28324    144.35142    144.83455    146.41541    141.41481    145.00541    151.21265    156.38671    168.16412    171.58616    170.89347    162.46722    148.41935    144.46997    143.01446    135.63469    141.28987    141.58753    143.56215    151.19286    154.02065    161.87425    176.78911    181.23332    176.03045    170.03638    164.08287    172.53331    182.24879    187.32398    194.42746    195.39942    196.64214    197.02827    197.88008    182.52907    161.03678    141.18571    141.89016    140.74363    140.37633    147.81215    157.33817    163.35177    167.06112    157.25483    158.18869    161.79027    158.54011    150.24809    129.10161    127.03631    125.48253    123.96656    120.93353    120.00193    119.39381    119.28813    118.38583    116.95583     118.0339    119.02833    120.01014    119.74563    117.34401    118.22572    116.72145    118.95468     116.7673    116.19382    117.45928    113.86221    111.28773    112.93912    111.00626    111.26924    123.22558    126.68661    127.01208    122.43369    118.18783    116.18946      112.218    109.82184    108.01699    110.75965    108.54141    114.76082    123.44851    133.56182    139.28668    149.16949    153.94978    146.20343    138.93983    128.76344    122.11637    111.76656     104.2224     97.35154          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       38.50    283.77104    262.08709    252.78285     245.6417    243.34592    242.84879    223.48784     222.2378    217.99995     212.2946    209.40249    207.43039    203.66973    195.45914    196.04971     198.0753    185.57825    182.66431    179.72172    175.63241    174.73614      182.495    170.23793    179.50337    178.73492    188.09787    198.83092    204.91612    186.47821    179.88975    170.19283     160.5334    161.14793    160.44717    156.85803    159.68461    156.75259    159.88178    153.95984    155.12063    153.47278    153.34687    149.19358    150.04797    153.80628    150.11606    150.11932    145.39173    148.69008    151.30752    148.12413    148.45734    148.81879    149.47513    150.50708    148.88233    159.95899     162.9335    172.44368    164.96153    158.87796    152.24436    144.88065    136.92485    139.72506    143.12211    138.12682    140.35226     148.2097    152.81998    162.75718    171.34746    179.80464    176.88979    167.37346    165.46609    169.93529    181.99003    201.07693    205.56313    209.45276    202.52007    193.93821    192.06198    177.35519    160.27266    149.56439    134.89969    135.93588    138.00871    154.31459    154.69175     165.6948    164.12948    160.65977    154.07656     159.5651    161.52976    148.76499    137.49296    128.42812     122.9401     120.9689    122.03011    120.31718    120.50542    117.51328    120.03823    119.54437    119.64216    119.56758    116.88194    117.71134    116.06126    115.14044    115.26975    116.88285    117.51535    118.42448    117.71538     116.1275    115.65448    112.27739    113.68305     114.2526    116.93482    122.57363    126.35651    123.95429    117.25388    112.71463      111.792    107.58836    105.03327    107.75647    113.57094    111.87483    120.78299    134.42627    142.31744    144.37435     148.6672    143.13109    133.16649    124.80469    111.22626     98.95941          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       39.00    287.45725    263.48662    252.38179    248.44535    241.50975    242.95444    227.50256    216.35031    217.89765    210.78058    211.93058    209.01985    202.47734    197.83298    194.70059    193.56869    186.46111     183.0137    180.96845    179.92789     183.2922     174.3817     169.0173    179.10202    174.51578    187.73592    195.33611    198.56444    194.26841     179.8346    172.30312    161.63658    157.39486     165.6016     160.8141    162.73085    159.38848    160.46601    153.80356    154.01421    154.71382    155.94647    153.41472     150.8111     150.6754    156.05503    150.28073    147.15018    151.24328    148.03166    145.30511    148.11756    145.25238    149.11832    149.88434    152.43298    160.12745    170.35265    169.44158    165.67519    159.06671    150.52308    141.49505    143.07588    137.48241    140.20017    140.71277    141.54855    143.87089    154.13785    165.22511    175.71493    176.75788    170.71879    169.62413    165.49915    166.15434    177.47004    198.08736    215.56913    212.43538    201.29247    191.98802    189.04405    177.93754    160.60381    145.37216     136.4912     132.8613    140.87588    150.03794     161.4427     167.2541    171.27967    170.45881    159.46392     158.3537     154.0817    145.51407     135.1088    130.80525    126.97431    122.06558    124.95704    121.58088    119.69688    117.78683    119.15629    117.36859    120.94843    121.10491    119.56844    116.41733    116.80924    115.70085    118.04718    116.15595    119.97451    116.51146    115.10429     115.7087    112.24407    112.50432    111.45559    113.07394    117.13096    121.68715    123.54195    122.20275    118.03989    114.69928     114.3017     110.1128    105.60814    106.07824    112.27595    114.12456    117.89189    129.73348    137.37002    141.95425    139.69034    134.17139    119.59584    102.23902          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       39.50    288.53954    269.24187    253.54692    250.67254    240.05087    241.12814    229.18299    208.72186    216.78368    211.97784    206.78881     206.5205    201.66208    200.73245    196.23306    187.91086    189.16624    184.28572     183.5231    181.61916    178.51936    175.74516    174.68035    177.99889    182.07481    192.04881    195.66686    192.39927    191.38957    181.92117    172.09286    162.66933    158.24695    162.04682    164.11559    162.89045    161.93939    158.02821    155.51132    156.03556     150.2357    149.33192    153.36817    153.54407    152.24185     153.7663    149.22109    147.73092    149.27097    147.39312    144.42832    149.42603    146.82826    146.49722    151.58787    151.01197    155.54553    168.69525    172.49069    165.71034    156.79577    148.28568    147.18774    141.55219    135.69231    140.12439     137.4998    138.50176    146.33146    152.59686    169.39701    177.84249    179.01438    170.68224    168.76412    162.75599     164.5866    175.20371    189.01127    208.48451    209.74413    204.17749    191.00135    186.74359    176.07856    163.20209    148.75627    142.34012     138.4692    138.58184    149.48625    163.20333    175.94818    184.08991    181.51682    177.07451     156.9963     145.5663     136.3297    130.64481    124.41899    125.24142    129.84478    122.91663    121.68486    119.51411    117.90701      118.853    117.09868    118.23122    122.24173    117.78215    114.44985    117.01361    116.74198    118.01911     118.9957    122.43648    118.21616     116.7397    114.41688    112.11486    111.53505    112.26081    110.54276    117.54156    122.05459    122.09821     120.7929    116.19816    113.71752    110.91352    110.39857     106.4949    103.39351    105.14832    110.40639    113.74067    124.41379    132.47199    134.00736    127.47683    107.46259          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       40.00    287.88119     270.5593    251.66171    251.66693    238.90704    240.70814    229.83812    209.58743    213.76775    214.58788    206.93853     203.0258    201.76843    204.99628    195.76452    186.58584    190.64268    180.54672    181.69949    181.98552    172.87095    177.64783    179.13324    173.45123     180.5006    189.17369    198.39808    192.62752    188.39803    181.04723    170.33563    163.14561    164.14369    159.28669    162.46353    162.52879    161.47172      155.626    158.25746    158.70953    150.38174    152.06816    152.89341     151.9229    150.07347    150.38493    146.00159    150.57495    147.00082    149.98823    146.81261    146.96376    145.92019    148.48688    151.75813    152.44508    158.65669    164.38184    168.43663    165.41089    155.51371    151.06358    146.37953    143.74825    140.91173    140.93353    139.09404    142.46224    143.14882     153.1703    165.48205    175.88831    176.29255    172.91657    164.91826    161.79321    162.60827    171.47594    183.31339    196.90506    196.19175    189.59053    186.52726    183.80914    173.54485    158.14505    145.39646    140.21188    133.02534    140.94178    148.70091    165.98109    181.86524    197.89561     198.8083    188.48451    165.60097    142.11481    134.49547    129.10538    126.51792    129.46207    126.79528     124.1478    121.65455    120.42397    115.17928    120.08057    119.15134    120.83489    115.46408    119.83592    116.43425    115.89047    113.70635    116.98909    116.96816    118.73779     119.3307       116.99    117.45778    108.70246    112.96642    111.61354    115.69223    118.53367    119.60704    123.20377    122.11958    116.46525    117.46852    109.62352    105.92721    105.14122    105.26208    104.71812    107.18699    118.05516    121.40068     121.6863     113.9388          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       40.50    290.01898     268.1453    251.45451    249.95086    239.83849    239.60578    229.39858    216.62439    212.53062    215.51738    209.83365    201.04071    200.51109    203.67223    195.39012     191.5533    189.70107    181.65456     181.4189    181.53272    177.27165    179.89882      180.564    172.88856    181.47585    183.40954    190.90249    193.44087    187.40051    179.10007    170.93608    171.70479    166.29445    158.40426    163.11662    160.58713    159.70671    156.00154    155.27384    154.47242    150.84562    149.54781    147.91198    149.15969    149.97839    148.31091    149.73028    146.19288     150.8278     147.9528    148.03513    146.98096    143.98985    139.96032     150.6024    149.63942    161.94269    165.71382    176.02975    164.25385    155.20366    156.35517    145.83741    145.66278    137.58098    135.74259    137.03096    141.08684    144.07625    155.57866    163.26215    180.33933    179.30543    172.78591    167.93162    160.13148    163.92642    171.33408     181.2478     182.5633    186.66086    182.69254     178.7209    182.68582    180.47123    163.37624    150.70513    143.43723    139.78123    144.06559    145.17845    159.98843    174.72666    190.32928    196.51246    184.83439    168.23879     148.6006     132.3416    127.21683    128.09292    125.98947    126.56416    124.63313    123.17659    117.60132    112.61624    118.84198    118.03722    120.87852    120.67807    118.67352     119.5919    118.06489    115.14283    114.85733    116.29461    118.64436    120.54246    118.32051    115.93505    110.39851     111.7904    108.63769    114.85395    115.42403    120.14727    123.40319    120.08866    118.44331    110.30397     107.6629    106.61513     104.7777    102.57602    104.60267    104.06906    108.98754    107.21741          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       41.00    292.43512    262.05322     252.0885    248.30338     245.5137    239.16792    232.74328    225.00169    213.61008    210.51189     208.5867    198.93859    201.44729    203.30242    197.24558    193.23939    187.94569    179.75087    182.09208    177.25947    182.42023    183.82761    178.98126    177.48379    178.16445    186.36168    189.04079    195.40079    185.04435    184.31638    172.66907    168.82173    162.17866    154.70217    159.25925    162.40821    161.10751    156.11059    155.49829    156.66474     161.7977    150.55672    151.33074    149.72402    150.84561    147.76866    150.85329    152.41126    152.67366    144.00624    146.86779    143.61009    142.58489    146.86937    147.74232    154.31163    162.93365    162.89547    172.44116    165.17997    156.80716    154.04877    142.72766    138.69697    144.49971    140.32426    139.00074    144.95596    144.57709    152.76628    168.27145    174.55112    183.93264    175.50309    162.58383    160.02082    166.56189    174.43329    181.68864    183.33295    188.36903     184.1559    181.34496    184.33653    184.24535    169.53435    150.94762    138.74127    136.68922    141.64279    148.48582    154.25305    161.06935      172.296    178.38776    170.53568    159.47108     146.9863    132.84643    126.18878    123.98785    124.18984    124.90105    119.64146    122.73447    120.72221    115.79203    116.07277    117.68094    118.44472    116.85029    120.25242     120.8976    116.59575    115.78682    115.69447    114.37989    115.72686    118.83226    119.59579    115.57744    111.98447    108.70509    111.13503      112.854    118.09747    123.02075    119.94049    120.85683    116.00981    109.04896    108.57942    106.40682    103.92559    101.55156    101.24748     99.11601          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       41.50    295.13038    262.42475    252.63378    244.78226    250.62806    237.79166    240.20168    227.68711    212.06502     208.9861    207.48961    202.60859    203.78763    206.07798    197.56952    190.35832    189.47256    179.53824    184.74498    178.93997    182.64371    184.44427    177.18644    180.86524    178.82668    186.67734     189.5031    190.79107    187.44323    181.26951    174.20292    161.23851     160.2096    159.82332    162.16504    161.12449    158.70428    154.89344    157.58743     156.5043    159.32999    151.27834    151.51188    153.42019    154.53585    149.37182    148.56973     149.5544    151.17553    147.17731    140.93122    142.22132    145.19301    148.94501    147.77165    155.21307    161.50805    169.66391    169.01296    168.25816    159.04998    149.62622    145.70528    143.65311     143.7096    142.83172    140.07522    139.07124    145.07624    153.16238    164.93364    174.91682    182.23936    175.28977     167.3239     164.5826    167.58027    173.09066    183.53092    185.62328    183.39307    187.19381    181.96837    182.29449    179.04617    165.87548    154.99606    142.27611    139.17322    140.27263    144.94101    151.87731    155.92025    161.92715    161.70776    158.71129    155.48795    147.68116    140.29956    128.32597    122.85687    126.26108    121.82523    120.39954     122.1647    117.48173    118.20668    116.35042     116.5713    119.31252    113.72577    119.50654    119.70531    117.24312    114.37557    115.20293    113.06364    115.96804    121.06064    115.97861    115.08107    114.89253    109.92751    110.24473    111.58326    115.12306    115.34363    123.53168    120.37215    117.02862    108.17748    108.13791    102.55107    101.52107    100.35837          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       42.00    297.60721    267.14506    252.81784     243.7878    248.50625    234.92211    242.63669    226.55804    209.67745      211.686    205.23059    202.35227    205.64663    203.96964    197.72512    188.16872     186.6273    186.66698    184.96684    178.39732    175.38829    180.82032    179.42856    178.01552    181.68997    185.89734    195.44279    193.96782    188.07723    171.29961    173.44218    164.79344    162.15746    160.18275    159.85714    157.09725    157.06622    155.98598    154.76892    156.07507    161.12152    151.42224    147.44991    149.82007      148.961     156.7296    154.48498    153.44513    151.18897     151.3244    147.13208      145.131    145.74662    144.85055    148.25616    153.23264    162.55571     165.7091    172.62407    165.01436    158.02721    150.58708    146.67619    141.05519      143.914    143.03534    143.68545    140.15651    146.98189    156.22372    163.85069     176.4574    179.81081    173.05546    164.99699    160.68655    166.17806    172.39687    177.62326    182.74969    181.11541    185.21807    184.22917     178.2882    172.73895    158.67393    147.87982    141.74468    135.66093    139.72758    148.14261    155.61826    153.21832    157.03477    156.26657    154.82095    147.15789     140.7636    135.83577    133.62521    120.94198    123.71331    124.42475    122.14242    124.90373    120.37117    115.43338    118.25174    116.52234    114.70454    116.88442      120.123    118.06093    116.24911    115.41065    112.47019      114.297    116.78743    117.90587    117.78461    113.22993    111.61157     112.8022    108.06116    111.90836    114.72426    121.80601    118.59885    119.14796    112.56461    108.07041    101.94093    100.09102     92.64331          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       42.50    295.15245    272.93912    254.51964    245.24611     241.7337    232.35145    230.97275    226.27874    214.00084    211.88955    204.65426    199.93781    204.49019    194.64865    197.05537    187.84441    185.55097    187.19152    180.25059    172.90274    178.44224    181.84136    183.42573    177.17008     189.0979    188.56819    193.84519     191.3857    185.80867    174.60472    172.52828    166.36724    165.51656    161.77037    159.50093     156.9288    156.63987    158.53461    162.85135    157.11989    157.23411    153.13453    153.23076    149.84992    150.77422    156.43424    153.37881    145.12428    149.05951    147.18853    150.42542    141.87185    145.85654    147.67943    150.57872    156.66367    160.68852    168.54718    172.28072    163.92014    159.11816     144.8214    146.18832     138.5886    143.18125    143.55232    140.59302    143.67391    144.47045    152.92374    159.37477    172.75231     178.2984    177.23943    166.10936    163.12129    163.05372    170.18117    168.87625    173.06572    175.85396    177.77612    177.44737    172.85387    159.45377    149.18021    144.04549    136.21548     133.4458    144.49532    149.59117    157.97309    159.46692    156.37309    157.58739    157.15724    146.00406    136.01927    125.01991    129.37466    128.52242     122.4793    122.29925    120.78154    124.35778    119.92999     117.0038    117.25985    116.15974     115.9485    116.10959    117.39471    115.57276    114.88679     115.4639    118.62298    114.44585    114.08176    118.68931    118.55454    117.38955    113.37306    109.18715    108.58435    111.08054     115.1529    116.87975     119.0197    115.69429    112.15059    106.22393     95.38908          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       43.00    292.38365    279.80924    258.77043    246.91149    237.33347    233.99811    225.45617    226.79912    218.83747    209.70644    205.87684    201.15336    203.01414     199.0371    202.37567    194.45905    191.82507      183.496    181.38435    178.76126    184.07262    177.11447    182.91814    175.24858    188.63499    186.55102    194.73983    196.83647     186.0282    173.52711     171.9824    169.76347    167.63625     164.2142    158.96862    156.32904    158.62926    157.40552    156.58341     158.7575    152.71318    158.37153    155.29731    154.80973    152.96313    152.06939    153.16426    151.07848    155.10478    152.40512    151.63926    148.18739    147.63021    145.25935     151.6085    153.15164    157.75353    170.76835    171.74979    169.05108    158.94281    149.22897    140.25746     142.0101    142.28805    142.87316    145.57779    137.66253     141.2162    150.21183    163.76966    175.18983    182.03046    178.30652    168.25717    156.82764     158.0227    156.52068     167.8122    169.03545    165.41158    168.10468    174.17324    168.27841    159.59189    140.86942    143.97297    138.15701    136.16881    142.22185    154.10818    158.55267    163.51732    160.87844    156.51141    152.44546    141.33615    137.54653    129.71052     123.2617    126.13441    126.72442    122.33879    120.58922    119.28993     117.8123    120.48977    120.42046    117.48528    117.88007    116.22215    118.49664    115.83881    117.33125    116.11453    118.49888    117.35066    117.91056    117.19298    117.60553    115.00336    113.06929    111.66615    108.86038    111.72315    111.16977    116.06206    116.20899    114.67927    106.86576     93.04865          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       43.50    289.68152     276.9046    262.81338    250.42746    238.47395    236.06748    228.07137    227.83601    219.94308    209.99312     207.1362    201.42753    198.82753    198.49131    199.93556    197.79773    193.16513    186.39021     181.1684    184.36205    184.89465    172.87708    179.56092    178.56581    183.97978    188.62869    195.57356     196.8069    186.72971    176.35384    174.26758    166.46647    160.84561    165.93835    164.98569    158.51348    164.05497    159.77865    159.55681    157.32503    158.07325    156.22332    153.91686    147.74409    150.48484    154.73755    151.39426    148.27662       153.56     149.7337     148.0202    146.83636    147.66982    149.15496    149.93214    153.67023    160.57297    167.11209    170.27022    166.98309       157.76    150.77234    141.63146    139.28365    143.08037    145.82839    141.76981    141.12925    147.54002    149.69591    162.44522    174.13171    179.20173    174.46626    163.16417    163.62111    158.19236    159.50255    159.37089    160.70802    162.11626    162.68271    167.72786    159.23309    152.77982     144.7029    137.32775    137.10187    135.38612     142.1184    148.25759    153.99211     163.3407    159.02898    152.90025    145.19644    138.33345    133.02394    129.73703    125.38852     124.0322    126.85118    125.07163    124.20098    119.79402    115.92798    118.34052    119.00492    117.97065    120.86158    114.24006    114.51561    116.38531    119.12303    114.89806    115.02044    117.65204    118.03593    118.75041    116.75258    116.64727    113.28425    110.70754    110.11144    107.86439    111.10104    112.94002    113.72259    102.31017          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       44.00    288.18025    274.43744    266.30733     249.6011    240.18664    237.08898    232.09597    229.29552    220.69042    209.38909    205.96626     201.2025    198.62202    196.82948    194.84463    193.86674    188.64406    185.88046    180.00829     183.7146     185.4717    177.52052    175.39996    179.34064    181.96306     188.0491    191.71207    184.29716    185.66138    177.98999    172.62036    162.96424    161.27079    169.11614    163.64394     162.1134    161.10069     162.1205    160.68418    156.18877    159.95289    156.60099      159.076     152.0499    149.24475    148.49078    156.03053    148.14244    152.03334    147.07664    152.60334    152.77245    150.35773    149.41085     150.7388     152.1053    160.62765    166.05534    167.98642    171.72532    158.03542    149.33251    150.65872    141.69298    142.04003    138.69639    142.74921    146.87244    149.58174    153.54235    165.04068    173.73039    176.04375    174.73847    167.67823    164.38971    157.73644    158.58805    155.10494    157.94075     160.5275    163.58624    164.83422    161.87971    155.53119     146.3249    138.72572     134.1442    136.94082    138.79784    151.75359    157.41142    161.11454    157.51331     154.1945    143.46354    136.95651    132.49193    130.48352    127.74813    127.06332    127.50422    125.27469    122.60133    119.84685    119.71964    117.30386    120.99275    121.00539    117.97421    117.99939    114.75245    115.62588    115.46056    113.21693    113.21989    117.46098    118.23041    117.15395    117.53906    111.22529    109.39162    110.52421    108.28994    110.45631    108.57541    108.19212     99.66893          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       44.50    292.10473      273.573    266.45205    247.71238    240.30298    239.79055    232.27599    230.69996    220.91495    208.07138     206.5369    201.39324    203.00737    195.18641    195.08553    191.38016    187.13383    185.18529     180.5743    184.34053    183.08343    174.02735    178.93159    176.62324    187.88391    185.61978    192.33992    189.88556    189.98938    179.30122    169.14471    164.85314    159.38015    163.35062    161.43512     162.5269    157.46239    158.84813    161.55724    158.46587    157.27286    157.45192    156.24144    152.39019    150.54851    150.62965    153.41281    148.87882    149.05244    145.51173    153.35331    150.70638    146.24549    143.08958    146.92319    149.43189    161.80459    168.94866    174.70989    167.29749    159.01437    153.65208    138.43775    144.25372    140.05124    144.85715    142.68131    145.68591    145.43729    159.91791    162.70602    171.67276    176.02928    175.11197    168.61992    160.76428    158.60975    159.43689    157.64389    158.09207    162.12158    162.74502    164.65108     163.8369    151.70798    148.01249    144.27021    139.94682    136.43528    144.52258    143.34069    154.29421    153.76924     153.5304    152.67902    148.25265    137.13117    129.32481    130.71975    130.05809    131.64066    125.82157    123.74129    123.93881    119.93205    120.35059    117.30489      115.313    118.25427    116.19952      113.887    121.49736    117.30092    117.42995    115.83125    116.99221    116.39985    117.85993    116.74304     115.4198    110.24227    110.40504    107.40673    106.89403    105.15509     100.8749          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       45.00    295.66124    273.84241    265.44574    247.68588    238.22541    240.26226    227.42919    229.81505    221.67077    207.94024    209.72519    200.34215    204.35911    191.24787    195.59408    193.96449     188.4435    187.72602    180.56017    180.67561    185.98211    175.18292    183.56477    181.33138    192.72141    188.43327    196.00616    199.57194    193.98701    173.88616    169.24418    169.49006    163.59575    168.56179    164.82207    163.78134     158.5792    159.38956    158.97991    162.04412    157.82956    159.30942    156.89408    154.76873    149.50531    153.64842    147.85468    154.84068    152.78709    152.69988    150.43267    154.35535    148.70617    146.78741    149.62864    150.62855    158.76862     168.4883    168.86252     168.0852    159.77242    150.98284    146.38164     146.5613    142.21986    137.67609    142.51207    142.49717    147.19438    152.58748    165.17207    177.03475    179.80793     178.3061    170.30208    160.43904    162.19356    157.88306    162.48666    159.16908    163.81834    169.55234    169.99636    163.09875    150.15304    142.90249    146.04331    141.41286    138.56894    135.05467    145.79471    154.26111     152.9622    159.81252    155.39698    149.60337    140.91077      134.631    130.76951    129.90034    129.54274    126.10489    123.87949    124.33919    123.33329    114.69281    116.42991    117.74719    116.40232    114.81911    116.87144    114.52535    117.97384    109.34844    115.73419    114.58466    118.37246    114.59332    118.96942    112.22124    115.78625    108.22504    105.56743    102.33826      94.2901          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       45.50    297.36143    271.79823    264.31455    252.20572    237.21873    238.39156     223.4046    226.81054    225.24135    209.28642    212.51028    199.81736    203.90613    193.32002    198.58823    194.03491    186.47713    190.36447    186.49994    178.12623    180.74848    174.03532    180.29961    180.46601    191.75603    191.42329    202.37897    194.71231    193.08577    179.43893    172.02625    170.07713    164.49824    162.68331    164.36737    166.96746    152.46611    157.36032    154.34813    158.85624    158.57156    159.00695    155.92585    154.48239    150.93473    146.29407    148.15086    146.09849    149.94525    149.79378    150.85646    152.46214    154.86031    148.82912    147.63365    154.69589    163.14554    159.87763    169.62777     166.4556    161.13757    150.78035    147.47783     142.3857    138.70967    145.98886    142.76941    143.90633     147.9051    152.36876    165.74801    172.00544    178.99307    171.94966    165.73271    158.38839    157.58248    164.74053    158.77199    160.84364    168.14815      170.353    175.03862    168.79234    152.84887    145.33591    141.70474    141.11564    140.21262    141.99626    142.40991    153.36386     158.2998    157.98306    161.72574    157.96229    153.26297    145.71682    133.36621    132.09455    129.83101    125.16237    124.60259     125.7881    120.81755    117.88835    120.66427    116.36324    115.58186    115.23718    115.52863    121.28188    118.70568    115.83381    117.36703    115.05479    117.93139     118.0209    114.54277    119.88543     111.0692    109.69653     98.63834          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       46.00    297.51652    271.12798    263.48354    255.24427    238.97199    233.74367    222.18327    225.32963    229.53606    213.13312    212.32995    205.25002    200.04508    197.62612    198.42702    192.11902    184.90145    189.86242     191.0435    179.45732    181.21604    176.81749    175.51984    176.03006    185.16689    190.26114    198.25917    188.92109    186.75355    178.96559    173.41391    174.00464    165.96833    163.00976    166.79415    169.82062    161.94156     150.6423    157.27166    156.13637    158.99727    158.59468    153.88196    156.59053    157.55964    151.19691    149.55115    150.90634    153.11022    147.65893    152.06515    150.44271    150.88578    148.63758    153.07839    152.53818    160.59096    169.77961    168.96537    169.54385    162.73683    150.15221    146.10848    143.85519    143.45449    142.30286    144.62279    144.18769    145.02494    153.26705    164.19607    173.16249    180.90514    172.65711    161.34521    159.58411    156.18928    161.37435     162.6966    163.93666     167.1316    174.85397    183.57826    179.20856    161.30697    148.75609     141.9346    138.93166    134.60808    141.46469    145.22484    151.07105    154.71627    156.93287     155.8149    159.23826    153.61989    147.77513     137.2915    131.86259    128.93387    123.26782    122.94581    125.84973    121.67697    119.09759    121.46656    117.68334    116.45776    122.14849    115.01866    115.31965    116.49231    118.66156    116.57081    111.61142    114.54229    112.06161    118.40814    116.37442    114.70964    100.73391          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       46.50    299.26604    271.83097    264.72597    255.03464    240.42409    232.29906    223.56167     222.8657    231.62816    214.69046    208.56239    211.47794    200.29025    200.33249    201.21969    192.84557    182.73143    186.99667     186.5631    182.60416    176.14605    182.85449    177.17253    176.85116    185.46175    186.79861    199.36449    197.85302    184.74965    175.25634    171.69328    170.61501     168.3275    166.51942    168.11134    165.68925    164.32075    157.60147    157.46674     157.5182    164.08163    153.54658    155.34239    152.92203    158.55574    158.59474    151.12794    151.45899    151.60484     148.2684    149.52672    155.38135    148.73212    148.16174    151.48092    152.58698    156.62456    164.44192     164.3275    163.14137    158.65593    147.52305      144.444    145.08158    144.61343    136.56458    142.92771    140.42848    147.23653    155.73472    163.14359    176.98118    176.39062    172.28982    169.04551    161.17155    160.22002    163.51244    159.80251    165.68889    173.79156    178.41893    185.06161    182.71115    169.15263    151.98188    140.77704    139.71225    136.04574    140.16099    150.34838    151.75387    156.16246    156.45225    159.78874       154.73    152.01157    142.47455     139.0872     133.7028    127.32978    127.27978    124.21188    120.03761    121.39541    124.18328    118.09372    119.78548    116.77778    118.47688    122.82479    117.86159    116.16775    116.47556    117.32275     116.7776    111.00107    114.83412    115.54846    108.84469    101.43196          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       47.00    301.63446    274.08233    265.94938    253.66473    242.34767    232.52944    227.57835    215.99453    226.40003    214.90156    205.40155    207.25099    204.73305    201.93381    199.82344    194.75129    187.86548    187.75283    188.97988    187.36496    171.72125    178.58761    178.77978     176.4538    178.93829    184.42062    199.98177    199.24929    185.63181    168.80751    169.51787     164.2402    168.71343    166.87056    162.92166    162.74585    158.16359    153.49268    158.04602    157.21178    161.72057    160.35044    155.95335    159.33281    157.02516    160.81577    152.60686    148.25793    151.58249    146.23927    153.70405    147.39975    146.02167    151.20129    153.91978    157.35673    159.28948    164.61318    168.15904    164.26973    161.10247    150.29811    151.16514    142.78792    142.07628    139.71965    139.07669    145.03393    148.93963    154.71156    161.14897    177.22056    177.90214    177.77944    170.07002    165.50865    160.51956    160.85588    165.38177     167.0488    171.54539    173.33729     179.0208    176.90276    166.20815    156.72421    140.20133    143.58958    135.23296    142.35197    147.36143     154.2118    157.03294    161.68778    162.16872    156.97631    153.35365    143.14415    136.63397    131.64536     127.6946    128.13444    121.84888    121.61979    123.44311    120.54918    121.09136    119.33144    117.65642    115.10736     114.6868    119.11132    118.56605    115.37251    111.92028    117.11938    115.77044    111.52832    107.66793     94.98681          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       47.50    303.79445     277.4441    267.13284    254.87996     243.6148    231.30524    230.47805    216.71524    223.74753    220.06507    205.38538    203.39259    205.59213    202.20573    193.93046    198.30524    194.66695    190.58873    187.85083    189.58292    175.61279    175.06392    176.83067    182.36819    177.61973    190.18725    197.09218    201.21639    191.47877    177.61506     172.7234    169.37602    168.85398    165.26175    156.47704    161.41287    163.00719     165.3344    154.14113    155.16842    159.67638    160.46823    158.13725    154.83619    152.06266    157.10002    150.58917    150.93231    148.66906    147.52643    150.97047    145.21139    146.75567    147.09143    155.55564    161.25089    163.49135    164.22331    165.75071    164.38448    155.95933     152.6439    150.49165    147.22555    144.96301    147.63773     141.7791    146.17279    141.55468    153.12546    168.89485    172.44693    175.77379    174.37648    169.94853    164.28179    161.83687    164.13071    168.49164    169.37213    172.71318    172.09117    167.67066    168.71826    158.52049    150.82403    147.16141    137.67693    137.89684    142.91354    146.23824     150.9455    161.76247    157.55879    165.34395    155.85586    153.76002    147.08086    137.75014    128.86305    129.15278    124.38665    125.40967    122.09213    122.13198    119.23882    120.55155    116.22068    117.85304    119.09634    118.00222    114.87246    112.52653    115.05842    114.09606    115.93262    111.59041    105.74372          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       48.00    306.07467    280.68356    267.02596    256.08435     245.0101    231.20444    232.00676    221.82499    223.45619    220.55738    209.22858    203.77564    202.06446    195.53477    192.18218    197.93286    194.83645    192.52439     187.5149    192.08644    180.99245    174.71822    175.42285    181.42538    180.50228    189.56602    192.46755    195.88372    194.51307    178.70771    174.00367    165.43871    166.88391    168.89493    163.16603    162.23602    166.58766    163.33945    157.06464    160.36071    158.94988     155.9125    159.45253     161.3145     150.9457    156.79214    156.98224    155.13906    154.29372    153.34673    146.62603    151.78717    147.54096    153.01015    152.46586    151.71609    165.02867    164.55355    164.89942    163.52711    161.43772    150.80372    146.13973    143.50019    141.01254    145.42734    140.36097    139.95923     144.2884    148.97468    163.20074    173.56237    177.07524     174.0307    173.66856    165.98589    164.94024    169.63337    172.51779    167.34326    170.06594    169.37745    172.80022    165.40263    155.99972    148.46732    137.80859    138.27404      138.259    140.55247    145.59304    156.40066    161.19987    167.19611    166.61778     158.9247    153.83486    146.00307    142.00122    130.84484    128.15078    128.27053    124.42448    123.82514    121.85872    119.84633     121.8254     118.0969    116.54746    116.52829    115.78618    113.04056    113.21495    115.90675    109.84675    114.07448    104.73615          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       48.50    308.65112    283.80087    265.81188    255.45957     245.5645    232.83257     229.1615    222.09654    223.81745    215.81385     214.0911    208.45817    202.32396    193.71681    192.76167    194.02719    191.77318    189.28804     188.7071    190.62009    182.38215    176.60187    172.38614    185.61587    180.72559    194.87652    194.08883    193.39434     188.4861     173.3704    169.70899    165.12602    166.64999    166.68666    166.33689    161.33361    163.33655    160.11423    158.19952    156.62633    157.53309    156.75906      154.959    156.89829    152.50901    149.21328    159.22781    153.26702    152.94043    153.10308    147.20288    149.01111    154.58401     152.5358    147.41728    155.77946    158.73609    167.09926    163.89698    165.17356    156.74121    151.45031    146.58525    142.97325    146.17749    142.68295    147.86529    145.95566     149.3064    152.36779    163.94023    172.73816    180.26785    173.60843    175.77595    168.52299    168.09118     174.0845      178.496    176.90017      175.854    174.00318    170.82059    166.41511    159.96105    150.65977    140.01003    135.62011    135.99611     140.6832    145.63708    158.10129     155.4856    167.00968    168.05523    159.64563    153.67256    150.62525    140.57555    135.01012    131.69723    128.65886    127.53219    127.96436    123.78544       122.75    121.42225    119.90943    117.65441    117.55987    119.18384    116.59212     117.3334     112.9263    110.96669     104.7438          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       49.00    311.33001    286.15399    263.20606      253.188    245.44157    235.54712    227.06179     223.4966    225.28181     215.6196    215.46148    212.15876    199.14347    196.79721    192.39809    195.04608    191.54166    188.05419    188.33482    187.93586    184.24476    180.91095    168.92169    182.63605    181.45156    192.10171    187.47361    197.31422    186.37645     177.8858    169.75618      168.947    167.84289    168.38318    162.84503    159.84419    162.45142    160.61116    158.00182    166.19717     156.8423    155.53239    151.90309    154.15052    153.08194    152.67277     152.4505    153.63726    150.70245    146.50685     144.5209    147.62478    148.35079    148.40067    148.76376     156.4836    167.50444    171.50784    166.61014    166.14181    157.23011    152.46233    143.52915    145.67481    139.03007    141.89483    142.55556    146.97188    143.04924    152.74228    161.18992    173.98667    177.04547     177.3097    176.89601    169.11903    175.60207    183.51013    187.31318    187.16769    184.62397    186.64478    182.72668     167.2723    162.12182    149.71798    144.07729    135.62608    136.75692    141.25815    150.73894    158.60797    162.32174    166.85146    166.33683    160.99346     153.4031     144.7947    141.16008    134.60366    129.58875    128.68019    127.56085    126.45086    123.29627    120.20885    119.79196     116.5685    121.98964    115.25212    118.04828    115.36802    113.53537    114.02148    102.87291          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       49.50    314.33971    288.67962    261.30664    252.74012    246.91134    235.49296    226.64819    225.14361    226.16203    219.37632    215.54029    211.15375    192.54638    196.90442    193.81208    195.58152    192.24116    193.61665    188.02706      188.243    187.71604    182.10269    169.54744    175.83952    181.94556    186.13223    192.20522    195.99043    182.54913    183.00513    174.57703    171.49574    165.61825    167.15551    165.86027    161.49647    163.66481    162.61273     162.6427    167.92082    153.38157    159.57782    149.41823    152.63528    153.51952    156.76207    157.24297    152.12217    154.64721    154.71794    153.74244    148.85981    151.26433    144.31802    148.15456    157.10114    161.94078    169.06449    174.22198    165.44522    162.15408    150.28393    145.94327      144.946     145.1487    143.20202    142.67809    144.30638    144.52811    151.89352    165.52709    173.44029    179.50188     173.8276    169.49547    170.50143    180.30403    186.42546    195.54959    196.09185    203.58379    197.02985    188.76971    179.62762    166.36926    156.14494    144.47077    143.96271    139.32491    141.50429    146.22742    157.29462    159.31667     166.2937    166.76369    158.96714    150.25288    146.57471    141.07072    134.29586    128.94337    131.71505    125.34576    126.46593    125.47493     123.5002    121.62266    119.57102    119.17793    114.89045    118.08929    116.92327    114.56492    104.47271          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       50.00    317.67884    292.03722    263.21968    254.21428     247.8138    234.54204    229.41507    222.31243    226.43435    216.58057    214.50117    207.50348    197.08085     195.5534     195.2194    193.33507    190.56318    196.08208    191.56762    186.12054    187.85489    184.21433    177.15205    177.72727    183.37468    186.76202    193.24857    193.88962     189.1254    186.84659    181.08501     168.2011     167.6645    167.80678    172.22491    157.27268    166.41106    168.28035    161.09999    164.30615    160.49882    157.47885    156.42447    167.42515    154.47592    156.18105    158.72261    152.08161    150.27077    157.98673    151.06882    149.09962    146.59314    150.94034    151.34801    154.79049    164.38991    169.71896    173.59753     168.1365    156.78849    147.69353     148.6717    148.17262    139.57625    147.64721    140.87591    149.88696     145.8517    158.48272     158.7116    179.93683    178.19387    178.73887    172.18518    172.30247    187.56088    195.45626    200.36351    205.09154    205.22868    208.54128    206.02914    201.98551    179.48882    162.26733    143.29802    142.17998    138.41559     140.3245    148.58032    157.27954    164.98014      174.497    171.50387    169.52522    160.46946    148.25086    142.23851     135.1558    131.02524    126.25563    129.89204    128.04705    123.11663    124.08307    123.91899    121.65933    121.12758     116.7125    115.96754     115.1662    104.90746          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       50.50    319.38317    296.22309    266.63367    255.23606    245.22995    234.98365    233.72367    221.06086    226.72113    210.94077    214.54585    211.46441    204.04267    197.82942    195.03996    190.31275    187.57543     194.1942    195.66525    185.86241    187.68863    184.89477    182.88633    176.11069    183.22848    188.08622    190.44115    200.10946    193.36508    187.42848    176.25814    165.77198    166.19492    169.02017    168.11501    161.86545    164.61039    162.46863    163.83815    156.57212    164.28256    159.18109    158.96154    156.37929    152.53709     155.1726    154.38311    155.13771    152.97071    157.14647    154.83551    155.19983    152.83617    154.10343    153.04629    152.30072    166.06243    167.99492    164.71823    172.03161    163.05506     153.9282    147.91283    146.37927    150.00304    143.88532    142.42317    145.28072    150.90919     157.0205    165.28833    173.80024    179.80117    175.20984    176.05397    178.70717    183.93545    198.42996     213.3173    217.06975    211.96349     212.0021    214.31096    208.41822    188.22575    165.42572    149.21873    138.11676     142.2116    148.28286    147.08809    155.54855    160.69222    170.82508    169.45848    169.22508    155.55312    149.24459    143.29271    135.69806    133.41046    130.02678    127.77286    128.06442    128.09491    124.84067    120.19513    118.74868    119.93361    117.98462    114.33814    104.34927          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       51.00    320.27199    298.12187    268.46847    254.05398     241.3956    236.65076    238.99011    222.16374    226.67333    211.74484    213.83214    214.78991    207.36239    197.57074     195.2877     189.1224    187.49558    191.04126    191.55452    189.83212    190.00273    190.36046    183.14595    175.58902    183.68744    189.14163    191.57785    190.85571    187.86859    180.19685    172.41895    171.58119    170.39928    168.71811    163.13087    170.59638    163.16151    163.18413    164.36629    161.42841    164.37368    162.78465    153.08931    157.59309    150.32162    159.70651    159.44045    152.91399     153.0016    154.70124    154.92107    156.34442    145.79874    146.58841    154.68954    160.17828    161.43357    164.69035    163.91829    165.73878    162.66539    150.17599    148.07727    144.18712    147.85154      142.615    141.41485    145.00722    149.45502    157.40843    169.54463    175.06184    173.66031    185.22707    177.50228    177.54848    186.68144    201.49586    221.86744    233.37278    221.07138    210.58742    209.74828    199.47903    182.02751    166.43682    148.94506    144.06537    142.57751    147.31681    147.60773    152.57582    159.30685    163.80893    166.73156    163.18678    154.51369    148.49351    145.68451    139.07902    128.58744    127.27011    126.66345    131.38461    124.65865    123.44902    120.63912    118.75748    118.19931    114.73438    107.41613          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       51.50    320.33937    298.33504    271.87371    253.22327    238.60903    239.35779    236.62862    221.52425    224.56384    216.55726    213.72649    210.97014    208.71285    198.73288      199.028    190.65435    191.59473    190.75027      188.436    188.64798    180.61886    186.90446    184.36014     181.0156     183.0921    189.50256    192.49226    193.73513    187.01536    175.23579    170.86988    170.37458    173.67758    164.12647    160.36921    166.96355    162.57189    162.76268    164.58753    158.44295    164.27854    162.23052    154.78066    160.70229    153.35883    156.90871    158.56232    152.87341    148.78871    149.55302     153.0692    158.44589    159.09455    149.44688     154.1648    155.50741    159.83855    163.38114    168.27195    166.24819    162.64862     159.0492    143.98724    142.88827    146.73078    141.59521    141.54895    145.46121    144.72391    154.75007    167.73989     179.0029    177.61788    175.78847    179.96694    178.45476    187.83591    209.17446     229.2673    237.80554    231.57605    209.74651    195.66803    184.26148    173.49428    160.32663    150.38472    142.52497    139.05884    141.35524    147.24678     152.5523    156.31239    159.65861    159.77457    157.30101    154.75854    146.42318    137.88374    138.27554    138.77668    129.44314    126.50406    129.37143    127.91009    126.43342    120.85615    124.17291    115.79785    108.77885          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       52.00    319.79088    298.58327     274.7407    254.63303    239.82099      243.123    233.17904    226.10739      221.024    217.41395    215.32784    210.23866    210.55085     199.8574     198.2856    191.64272    194.55815    185.64614    183.93967    178.57508     179.9519    186.57119      187.135     181.8566    180.17801    193.15652    193.27376    194.06637    187.14195    178.41404    180.64936    173.33467    171.59121    158.70844    163.29893    161.95026    165.86712     161.2811    159.12787    160.52874    161.94964    159.52103    159.06075    160.63315    155.42675    154.55767    156.04181     154.3144     153.0332     155.8349    155.15377    151.65733    155.70004     153.8206    149.75617    156.79491    155.89045    170.03414    169.46296    172.68483    163.63875    154.29723    146.64179    146.40952    144.88744    142.87361    148.32664     146.9197    140.66083    160.11702    163.37901    175.88792    182.97815    182.58073    176.76459    187.09891    191.16028    206.37966    227.03812    249.76041    240.64047    217.72701    191.96072    174.85122    160.50028    149.62428    142.83934    137.03374    140.44245    142.38712    145.10051    155.63632    158.93082    158.54744    155.14599    153.67862    148.79319    146.00936    144.34664    138.77173    136.35794    127.96538    127.57553    125.82174    127.02842    124.48087    122.59073    115.09951    107.14021          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       52.50    317.93446    298.60346    279.62873    255.92276    243.18746    244.60572    233.59622    230.05507    217.28256    216.45038    217.17015    206.40421    209.88894    203.58981    193.95244    191.30539     190.1496    187.32672    181.66462    175.20264    182.70469    182.99598    181.38323    183.60621    183.41985    186.21904    192.93234    191.26583    187.90654    180.33424    178.14068    171.54007    169.72922    169.24613    163.23463    160.90852    163.43632    162.71862     157.2188    162.78704    155.61641    159.95297    161.86287    162.77394    159.99935    154.03393    152.91152    154.72807    151.95366    159.91154    154.22724    154.10946    154.33316    155.55614    154.90716    152.82574    162.66714    162.85275    167.46603    159.82469    161.49948    156.08561     145.4459    146.99146    146.16394    145.37061    149.98396    143.59154    143.77787     156.6288    162.05444    174.33372    177.03971     184.6561    179.80667     175.4227     186.8666    201.18281    220.72737    241.33559    238.29058    223.15991    197.63637    173.33187    155.77406    143.80979    141.23631    139.74832    137.19774    134.98557    145.35792    156.16885    157.47461    157.02109    157.39123    151.51871    146.42455    145.71146    144.62717     137.7174    133.45814    132.31571    130.18909    127.20126    122.53517    122.71475     113.6788    109.45601          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       53.00    315.79402    298.41872    281.71832    257.47262    246.29722    244.24627    235.41079    229.57351    215.66196    220.44744    214.54978    206.47058    205.41729    209.35734    190.05501     193.3467    180.80591    191.24472    181.27022    181.81621      181.378    184.36637    178.51657    182.13997    188.15617    186.50677    191.34029    197.98596    185.32665    185.48071    171.07729     168.5939    170.53195    167.75206    167.72672    158.65137    171.56011    161.01685    159.91681    162.50819      157.534    158.55407    163.07842    158.62287     158.0649    157.99403    151.49124    158.06918    152.85145    155.88843    157.29328    151.77748    154.01038     153.8904    154.31927     158.7537    164.12737    163.33731    164.10189    171.75771    160.82395    156.89732    149.13922    142.43962    149.77907    145.80507    145.69602    142.23061    147.74929    154.63178    166.04078     174.1459    176.02991    179.16929    176.23554    174.87073    178.25952    194.85344      210.652    227.94364     230.2243    223.50559    199.64088    176.87497    155.74824    146.49721    135.34346    140.09112      136.738    143.04277     146.6771    146.39019    154.78806    156.58611    162.55236    155.24816    150.46115    146.56136    137.83007    137.90386    132.25674    129.88144    131.04546     128.6728     124.9519     117.0453    114.60084          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       53.50    314.88797    297.57742    281.80316     262.9186    248.64266    246.83526    233.51749     227.4793    219.57618    222.71241    212.16424    210.31211    202.87527      210.508    191.00328    196.44543    181.72596    190.61674    180.83132    184.55353    181.90789      184.634    181.70585    182.10823    190.63344    193.75029    194.46735    194.76842    185.45386    185.48065    174.82328    174.69844     170.6905    168.61957    167.42725    154.27946     167.6329     157.9268     158.5313    160.00975    163.28186    165.63118    159.53458    162.49717    162.30475    160.62969    158.06835    154.33062     153.3368    157.52911     158.6023    149.39794    156.95345    154.66533    152.91433    162.81823    161.02345    164.05046    165.72208    167.96933    160.60357    156.25394    154.34634    147.95703    146.50155    146.00461    142.37793    146.40466    142.97688    154.75985    158.77396    167.50859     177.5801    179.25436    173.89812    176.87799    176.61118     182.0248    200.00425      206.921    219.00922    219.49349      200.205    179.56444     159.3625    148.39788     141.1035    139.46015    139.13577    141.79792    143.38335    149.22245    154.79586    164.54948    171.92078    174.37975    164.83979    152.62861    138.51289    135.78948    130.53851    132.28543    126.72187    126.25452    125.14674    119.63031          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       54.00    314.53947    296.63276    281.24789    268.86706    248.46746    246.36565    230.38146    227.21187    221.53807    216.05288    206.90921    208.25663    205.13787     209.0282    196.99622    194.40766    186.73636    187.60498    179.32936    183.44931    185.51328    184.08734    179.75617    180.36403    188.91241    190.49208    193.67915    189.17014    184.34357    182.63232    173.57827    179.28385      174.008     166.8452    170.18826    165.78615    164.47275    160.49653    161.03208     156.5985    162.36809    161.32891    156.48573    160.14661    156.16996    161.25005    157.74074     154.2451    151.17905    159.52771    156.63744    155.10581    149.54835    149.55683    158.08773    158.53217    162.07832    166.50706    165.67723    167.72297    160.64307    155.37683    148.70814    147.79551    153.52635    144.13687    143.62888    142.22488     148.3815    152.26592    164.02715    170.32837    178.82024    178.28223    174.78626    173.68666    168.11204    178.51905    185.63195    201.71135    203.97163    206.99031    197.11521    176.64494    165.08932    147.33219    143.76714    140.65507    136.83354    140.46815    149.20212    149.11666    155.78059    168.32118    182.99536       190.32    187.66321    174.01147     148.3692    141.97772    129.78988    125.00023    129.13888    126.87866    120.73291          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       54.50    312.97526    296.94294    282.21125     275.3138    247.38983    241.37451    228.06344    227.42954    222.77243    216.03033    205.46175    207.26635    206.69635    203.93453    197.62623     194.6504    191.16251    189.27572    181.50099    184.12736    187.03837    182.69895    184.23135    181.15582    186.84086     191.9664    189.52488    186.42679    191.15985     181.7479    172.20668    174.40789    171.68279    171.43628    172.09231    167.07553     162.3926    162.85458    155.42638    158.94862    160.17307    162.93314    162.34819    160.17057    158.46532    160.75061    162.66277    151.52472    147.95319    156.70396    155.66907    152.54701    144.91128    154.64063    152.81096    155.36582    163.49488    173.84741    167.80336    166.52177    160.05693    155.14175    146.55983    146.73442    144.23478    147.28074    145.72293    150.25701    150.65093    156.48785    162.72181    173.13046    177.40329     172.5261    172.87204    172.36502     164.0343    170.84269    185.22994    187.69352    195.29958    197.81564    191.03901    178.68783    161.36374     153.4005    144.80975    139.50241    139.89493    141.33866    143.82005    150.96086     167.3763     171.0708    186.99881    196.21473    192.50853    179.78242    163.67599    143.54255     133.9601    129.05715    123.98758    120.08013          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       55.00    311.80968    298.96856    283.88726    278.23401    247.19312    234.30664    225.68297       227.09    225.40363    217.39934    208.49046    209.71733    207.14779    197.16214     198.9089     198.6234    188.61022    191.13939    179.92048    184.90443    182.89354    181.75092    184.13208    181.35855    185.33987    192.79736    194.47381    184.64119    189.22749    182.69573    175.19412    178.54703    168.39605    168.83377    171.20889    167.63168    166.58358    160.82331    160.83577    158.51738    161.22917    162.54625    157.25016      159.262    159.33725    159.61669    163.12495    153.97125    151.77118    157.99122    150.87536    152.94187    154.39569    151.24051    155.27132    154.36911    162.63203    164.34584    172.10903    165.78224    164.37917    154.63932    144.31583    146.55426    145.40957    143.49105     147.9555     145.9743    151.60053    154.96815    164.08399    173.98972    175.04171    177.45573    178.96034    174.31463     168.4004    169.96434    185.13409    185.19416    184.18448    186.17336    184.00229    172.89907    163.60721    149.36129    143.96129    141.58186    140.12342    146.49098    143.77949     152.8473    159.44575    165.42816    181.65546    192.35571    190.00635    181.80827    157.37751    151.53729    135.63425    128.96362    121.70373    112.41791          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       55.50    312.96361    302.69405    283.23006    278.67248    248.60469    238.94122    230.53869    226.44808    223.34842    215.25165    215.38426     210.3266    203.78864    193.89138    199.77902    200.51371    189.40239    188.50307    185.87509    186.06611    182.93145    179.44773    178.38476    182.17768    191.48355    192.53588    195.51787    187.22191    184.50684    181.04495    177.40315    177.18325    170.36315     160.1363    170.53085    168.46064    169.40617     164.1681    163.73779     165.6572    162.83311    162.62728    163.26254    161.16515    159.38369    167.48084    164.60992    167.32029    159.87286    154.75638    159.97191    150.27533    154.38103    151.70763    155.13767    161.31982    160.92234    169.87298    168.33667    170.19767    161.99137    152.21531    143.24105    146.65464    142.10618    146.12879    144.27551    143.65122     149.0486    152.72894    164.99509    171.84855    174.36039    178.50092     174.0618    173.48736     168.9972     181.8785    183.86546    192.78322    191.65817    184.86424    174.03838     168.4112    154.67791    151.12098    141.64544    142.49756    137.31025     144.6431    146.04429    155.81295    156.70506    157.79913     171.4352    173.99517    176.54972     167.5334    156.33582    146.17815    133.89483    125.52424    113.37218          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       56.00    314.44989    304.64847    280.96531    278.41472     254.0203    244.94773     234.4632     226.2829    221.43057    214.83442    215.90973    209.25201    200.94296    199.26558     201.9738     198.9495    191.33168    184.28375    187.82462    181.90179    183.07245    182.38204    179.85149    180.73042    191.37277    191.45771    197.38504    194.89939    178.45808    182.08693    176.06725    172.46824     172.7659    170.75534    172.48266    166.10197    170.39094      170.546    167.44841    164.24623    166.09732    158.61733    160.66265    157.21129    156.35169    162.37184    161.38581    152.61936    163.48104    155.89247    154.80707    154.32121     152.0418    154.87289    159.29988    158.05262      163.412    171.66367    169.76789     173.0502    162.88328    158.96068    153.96132    146.88268    152.17524    145.08866    152.04522    145.40867    152.20874    153.16959    166.78413    172.26615    180.11917    178.97759    169.47709    173.44109    171.92155    180.63298    191.86857    199.98898    199.95104    188.17623     174.0013     164.3998    153.81485     147.6011    141.36018    137.29958    133.42359    138.10802    148.11421    151.60066    160.41141    160.38117    165.33011    165.60092    157.71474    148.33587    143.41498    135.06154    127.29211    124.92999          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       56.50    315.20599     304.6923    281.21747    276.52595    258.79201     251.9312    236.65968    225.03971    222.61334      216.516    210.46789    207.24154    203.07858    204.97837    202.70229    198.70746    190.22538    186.80286    186.35965    183.21987    182.13432    185.64127    178.69485    183.91012    183.56414    188.14977    196.15328    197.58969    185.20194    184.83055    176.84533    173.02133    174.01073     174.6712    168.15768    171.62505    172.75461    172.86448    173.20233    164.46479    166.45651    159.82773    155.24236    159.41261    155.72955    162.21965    156.41864    157.49403    161.82378    158.81242    152.76013    153.79685    152.18057    153.48058     154.7199    159.16823    163.79674    166.75365    170.87744    170.59156    163.42937    155.74731    152.12715     149.1358    148.87881    151.90529    148.06108    147.78333    153.37518    156.18281    167.19724    169.25644    179.32154    180.17099    170.92836    174.41817    175.47772    181.78938    193.79056    202.75128    197.71671    184.77389    179.78398    164.05313    149.09138    144.03201     141.3589    138.58536    135.99344    143.34371    143.52427    148.99155    156.21384    156.28134    153.17735    155.11096    151.76333    140.93398    138.20651    126.47642    120.99158          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       57.00    315.23971    303.58557    283.04085    275.99583    260.67371    257.55633    235.91689    223.24698    221.16722    217.33015    209.05739    205.89016    207.64168    201.43549    197.56569      201.495    188.81775    185.79622    185.17793    183.37309    178.35662    185.60684    182.03325    183.93873    183.02604    186.89468    193.77679    193.78773    189.38761    180.92708    174.05472    174.45982    173.37524    170.65502    168.50834    175.75878    172.43443    170.55381    170.94561    164.90373    161.78288    154.30559    161.65092    157.31908    159.95397    159.33318     153.2569    157.36455    158.90854    156.95232    159.19731    156.40448    154.82398    151.94112    154.03019    159.53552    162.44905    164.29403    171.78992    163.75263    167.39472    157.62882    155.97885    147.58402    144.80437    142.38202    142.73165    147.94285    143.59783    150.49456    163.80662    178.86572    176.02897    184.59181    173.39695    173.39666      176.142    188.99847    195.86232    201.45815    195.07013     187.4458    177.39704    163.19319    151.14341    143.54216    136.59223    137.60793    137.71587    140.96872    142.97186    149.46177    152.21349    153.95741    153.38972    149.80174    139.59642    133.85975    134.18273     123.5264     112.7342          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       57.50    317.54973    301.58607    283.94204    277.56267    260.41338    260.87449    234.36786    221.83758    220.26785    211.58103     209.0161    204.91606     207.3524    197.31844    199.21681    202.53499    189.17716    182.60044    183.80567    187.44819    179.54088    182.81268      188.205    182.82336    183.99645    186.23607    196.55453    185.83829    185.28627     188.1303     176.8998    171.21388     173.1705    171.25769    166.22081      174.002    164.36399    164.14913    166.83767    166.68294    164.80925     156.1075    165.64794    160.50712    156.36702    165.89105    165.84611    164.47729    153.01789    151.15762    156.32001     157.8575    152.98032    153.38165    150.20088    164.59102      165.206    168.44334    170.24387    169.80889    167.00819    160.30309    154.12305    149.37198    148.34391    152.26691    145.75456    140.02578    155.12755    155.66293    167.47381    176.24573    179.76248     183.4103    179.65972    172.50928     173.3345    183.21968     187.0084    196.45103    186.91927    177.12792    172.46004    163.27172    152.75218      146.859    132.08183    137.53119    139.56129    133.25068    145.32453    152.01522    150.17565    154.78942    153.91279    149.76688    139.34248    132.30491    124.57968    114.11433          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       58.00    321.26052    299.60682    288.07587    276.26999     261.7506    258.46848    236.90511    223.65096    223.03247    206.52174      207.629     202.8042    208.25485    204.15602    199.41394     199.9972    193.32053    185.79318    189.11536    186.55361    183.11933    186.40194    191.18505    182.94454    189.28426    192.55531    201.31612     192.2655    186.17625    190.72577    179.73347    168.14012    170.36644    171.78021    167.86867    168.46889    168.62419    169.44445    173.80815    164.88021    170.57292    158.39277      161.654    160.98826    158.96561    158.36577    160.46918    162.20579    159.69425    151.65208    154.15725    156.59962    153.85741    153.37629    157.96215    155.26831    164.42791    173.17175     172.8292     170.6956    164.19425    156.33781    151.17677    150.43178    153.04456    145.95205    146.55985    149.18848    148.18661    155.97944    169.37617    172.78559    177.48746    179.68104    179.20579    176.65423    174.36838    188.78217    180.98302    186.36004    183.06714    178.60047    167.94279    163.00751    151.46795    144.28207    144.55678     141.4718     137.1693    139.57657    146.36798    151.45119    150.85911    154.10313     153.6807    146.42537    140.18474     131.7084    119.37541          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       58.50     325.4089    299.83169    293.74926    272.39118    263.93784    253.97412    242.92188    228.24678     220.8599    205.58526    205.60561    204.94928    210.10194    210.06668    199.39891    197.33391    193.43991    190.42411    195.67931    175.93798    183.18606    180.13163    184.74784    185.81624    188.55687    191.70423    200.89233    192.98257       186.57    185.13502     178.5383    170.37431    166.79263    171.24729    168.47493    165.97335    173.41492    171.29868    171.06744    165.66727    168.81026    166.68915    160.77809    159.43359    158.14884    158.62385    159.22226    156.61584      159.671    161.94525    165.58136    157.37396    157.72237    156.21139    157.81495    158.26255    158.39615    166.10878    166.89051    171.08461    159.05293    155.52269    153.29563    148.80758    147.44188     145.5345    146.31004    148.69405    149.97555    157.35003    161.54266      169.552    181.55708    181.28855    176.93637    176.28367     181.8048    180.06733    178.37933    179.64123    179.60188    176.34042    167.53351    159.22273    154.36702    145.19365    141.47302    142.15113     141.0461    138.61681    145.62156    146.91118    153.04307    155.57911     148.8819    147.09061    135.50289    126.11581          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       59.00    327.54432    300.97481    297.46148    266.17661    265.27134    252.96906    248.18698    228.91283    218.88796    205.32341    206.37922    208.69734    203.77675    208.29209    198.77638    193.31197    192.18248    190.40521    193.57988    184.76459    181.39377    182.71884    181.97441    187.03401     187.2345    183.98324    194.34155     197.9059    186.24816    185.75233    174.71875    183.99809    171.60312    170.50173    169.57332    165.09861    169.78714    167.70588    167.80388    168.91389    164.97126    162.26587    160.78315    166.17693     156.2062    152.18182    159.17294    157.89587    165.65824    162.17301    157.25147    158.01745    161.67633    158.61566    161.76157    158.53151    161.32695    165.12855    170.43853    170.27624    164.65987    160.94128    156.06448    154.99396    152.57634    149.96928    152.10702    151.85587    155.72071    155.87069     169.3134     177.0111    177.14606    184.38735    181.66489    170.67331    177.71844    170.15045     174.1244    174.10757    175.50953     170.1796    162.64284    164.07015    151.98843    145.36743    142.57796    140.52145    141.15499    140.72392    142.84309    147.40664    157.00658    152.46304    148.41303    142.09824    131.47625    123.25892          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       59.50    324.44802    301.88257    293.26486    272.76456    264.36242    254.89248    247.15269    230.29029    219.76219    207.72469    210.63882    209.27384    200.79566    203.95469    197.02081    193.64428    191.95478    190.63315    195.82188    187.92937     186.6351    184.50552    183.47981    189.33473    186.24148    182.36791    194.86385    194.46804    191.74493     182.9556     177.7766    181.48167    173.95605    171.50114     171.4308    162.50967    163.22607    162.54646    165.81478    166.93915    162.72076    163.43609    160.62561    168.08546    166.67338    159.27842    158.35819     160.2825    159.55847    159.89369    155.39756    156.14341    159.38506    158.89584    158.61021    155.21373    166.07345    162.55391    170.34285    163.97244    165.92246     161.6068    157.27963     151.3218    153.18013     156.6999    153.47455     148.3063    147.27949    150.45378    164.56318    176.79103    176.84596     181.0511    177.03737    170.11514    170.48895    172.02932    174.81447    170.51225    166.16773    165.21988    160.59285    155.25344    145.56275    143.13812    139.70528    135.28105    141.36001    145.66443    144.26968     149.3241    149.37316     147.9063     149.8195    140.44824    127.36135          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       60.00    320.47572    301.05578    292.64716     280.3592    263.39537     256.6976    244.33025     236.6293    220.48212    210.72348    216.28082    204.21281    200.71556    198.69941    196.18361    196.81164    190.20454     196.0959    189.05157    189.29987    189.54823    179.71477    179.53714    186.36255     182.5847    186.62319     192.9655    189.92823      191.209    186.48213    182.28928    176.87474    173.56031    172.00568    169.53438    164.32459    171.44774    163.65118    164.54737    163.47723    166.64052    163.31843    158.27186    166.24592    163.08224    156.33348     154.5985    162.12793     164.0622    153.27142    154.38649     153.8233    159.99329    161.20018    160.65103    159.98884    171.85886    175.92084    165.50456    163.68311    159.02969     159.3839    153.06787    150.63762    150.40818    150.09143     157.5341    149.87365    155.69753    158.37278    164.01824    170.52687    178.34733    186.53345    183.56308    178.74771    168.96483    169.03623    178.99381    173.53598    168.50418     163.0835    154.67429    153.58669    146.65981    143.00035    143.42801    139.11093    138.15714    142.78446    145.04375    149.23627    153.67655     152.4783    140.35939    134.55359          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       60.50    315.23758    298.58923     295.0465     284.7151    261.18793     256.9884    241.72801    240.92289    221.25589    211.50603    220.26952    203.08601    201.16038    193.45879    195.36123    199.43018    190.15732    201.04954    187.11898    190.66659    189.05285    178.10819    180.14089    180.34269     179.5223    194.59204    190.44893    190.23175    192.30281    186.08259    185.02572    181.98967    175.12407    173.09433    164.22353    168.11553    170.00921     163.9874    172.01099    170.15145    162.54642    167.93352    169.40561    166.83258    162.47959    164.44174    159.59033    157.03527    164.91444    158.48869    154.08267    152.59101    153.88094    157.33704    163.02704     162.1621     166.2517    169.09029    170.27209    170.07768    164.30988     158.1122     155.1986    153.55619    153.97321    154.96331    154.72974    150.97197    151.15815    162.92449    169.13394    170.93732    186.61803    182.14498    181.53074    168.42731     167.1855    167.51313    171.34447    174.90059    166.10281     164.4074    154.99974    149.14573    146.26826    141.87618     139.9703    138.52801    139.52809    139.53224    143.35177    151.05479     154.8603    152.49076    146.33661    126.29949          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       61.00    314.70407    297.31073    292.88701    282.85317    261.32208    252.44563    238.92399    238.59074    225.83678    215.20587    215.80182    205.43672    201.52961    194.51175    193.61474    203.92178     198.3583    196.23943    193.22508    194.58206    191.15886    187.89412    185.07199     188.4393    185.22219    187.36045     184.4213    191.84065    190.24679    181.72954    175.79763    179.58805    176.82298    173.42207    170.45002    164.64774    163.51827    169.06332    168.11161    167.56456    163.59466    168.92178    166.81784    163.73304    159.79466    166.45022    160.44205    158.98315    159.54132    155.41101    157.00456    160.63929    156.05366    158.93184    159.36171    163.95866    164.91692    169.43901    171.50184    168.11043    165.26246    160.62581    154.22013    150.37375    151.58382    153.51842     155.0327     150.8773     153.1347    156.42411    165.06143    180.23573    182.44023    179.27237    178.94954    171.77099    162.26884    163.72299     162.6211    173.89241    172.46006    168.50993    158.80108    154.95043    150.42358    139.80726    136.23896    134.34904    140.47779    133.86227    142.21101    151.43387    150.52437     148.1325    137.70466          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       61.50    312.75369    297.36029    288.67224    281.73464    262.85829    250.72254    237.48575    237.00703    231.19697    222.32463    209.71451    205.98196    201.70216    198.61394    196.52174    205.69196    205.50275    195.23464    194.76682    196.94323     188.2357     190.4502    187.68335    186.57266    185.60192    191.37171    189.37037    192.73881    200.14235    185.86237    179.06021    176.10018    173.49567    179.52034     173.2892    173.44072    163.93168    168.82997    163.56639    170.74128    170.02967    163.75462    162.40723    159.70785    165.37416    164.36788    164.84485    165.69174    157.05001    152.62044    157.06317      158.334    157.23298    154.70174    156.12905    162.18028    161.46283    169.20507    178.01723    173.86547    166.79175    158.90626     157.8339    153.62025    146.33758    150.94004    145.43259    149.15557    154.20602    159.00587     163.6161    174.63296    183.23046    184.37747    179.72004    169.64603    163.51208    161.17397    165.34871    165.93367    174.28299     169.5198     156.7055    154.59737     155.8723    142.96226    136.24165    141.15025    134.11766    141.35009    143.07641    146.59392    151.84217    142.81271     127.3405          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       62.00    309.41088    298.62865    281.22675    279.77738    264.66561    254.25133    243.42369    237.37978    231.03248    224.89937     209.0291    205.16858    203.57593    199.09035    200.11899    197.48492    203.23827    196.32247    194.48519    192.73959    183.13704    193.47731    186.62154    184.36334    184.91127    196.41895    196.26545    191.14808    194.20881    183.68747    179.99938    176.19495     174.5396     176.5292    176.27864     176.4318    172.19653    166.38271    169.88136    167.65518    165.44529    164.46923    165.64908    161.17683    164.42032    166.83015    170.68585    160.49059    159.75717    152.82614    156.25949    161.24274    152.48196    161.27349    159.32159    161.32739    169.25625     169.3562    171.47992    167.25866    172.77176    164.18153    159.18285    152.82693    143.00471     157.8246    152.36724    151.53571    155.84432    161.37054    167.92592    176.68069    181.25755    183.47659    180.19349    169.03686    168.94467    159.81812    160.72029    162.72961    165.15004    162.54761    156.62863    155.99001     152.9351     146.8928    138.63789    138.65497    137.63526     136.8711    140.40962    145.64161    145.79467     137.2335          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       62.50    307.35544     302.0953    277.84353    274.02069    264.98204    262.07023    249.40502    239.97229    228.47288     225.0973    206.89762    202.26863    205.75967    197.87248    199.32684    194.91665      199.894    191.55595    195.77593    188.32113    190.63034    188.86312    184.43419    195.81472    183.15084    186.90597    195.82763    188.65872    185.97155    181.59955    179.82208    181.16297    174.53214    173.26792    173.76874    172.53158    178.12914    172.43122    167.65548    164.34768     166.2296    162.09559    165.89771    163.57278    166.30581    168.10763    165.50166    158.52476    160.99252    164.58044    150.88618    156.12965    155.90707    147.81326    158.33823    156.39766    165.56769    169.14839    169.06205    173.08655    169.56766    160.67539    155.18918    154.78665    152.22498    148.32185    144.54659    148.76584    156.44139     159.2408    168.04182    175.34373    182.52373     185.4794    180.20704      179.742      166.685    166.29978    157.55808    158.85145    157.02794    157.10592    156.32114    150.07647    149.40602    143.81681    144.33477     139.8696      142.229    135.88618    143.22624    145.08957    142.09654    124.12859          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       63.00    309.16832    305.48392    276.34748    268.49679    264.77129    272.19378    251.68418    244.13249    231.00529    227.69124    211.96806    206.30842    203.91603    197.33796    188.99413    196.84617    198.96359    188.41954    190.87687    185.81448    193.75839    189.38337     185.9727    195.19946    185.85301    185.52211       193.47    189.59433    189.94793    189.61812     184.9539    183.47359    171.48877    164.77543    172.72026    174.24667    175.48689    174.72925    167.31542    163.33786    165.77818    165.09594    160.71628    164.17554    156.28027     169.0752    162.00827    164.49853    160.37462    159.60261    161.95563    160.08109    156.55651    153.15758    161.04531    160.18308    168.76758    166.41809    172.08068    175.37316    168.69986    163.63421    156.04769     156.3841    156.16173    157.88093    149.23533    152.61014    152.05396    160.71729    168.37411    176.55348    189.91383    180.59283    186.36804    182.61079    175.81062    166.13923    164.50721    153.29454    161.00826    156.14921    156.13366     148.1545    144.15191    143.69712    145.61288    143.14506    143.42615     138.3416    143.11259    141.67326    130.29864          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       63.50    313.63376    308.69577    277.28841    267.60339    263.81298    268.03126    249.66031    245.33471    235.20961    229.97022    219.61985     211.3991     202.7026    194.99777    188.85798    200.10052    196.07006     193.3039    184.91269    192.53718    191.10012    196.87652    182.73119    192.68846    193.26897    192.27515    191.41615    190.14053    197.33249    182.23149    180.72025    178.85856    181.93047    175.65073    176.86001    177.87778    168.77511    172.20465    176.10576    167.45522    166.96113    163.42397    165.30374    163.43495    166.18182    160.46762    162.96744    163.91747    161.90879    164.99052    164.01292    161.93436    153.15949    153.44281     154.7972    164.10638      166.828     169.3535     172.0282    173.07546    167.92821    158.90952    160.84826    161.80132     157.4876    159.36819    148.96808    152.71563    151.25816    152.56849     166.9873    175.08208    185.22006    186.40548    180.85225    181.39277    168.38684     165.2002    155.33677    152.29331    155.38466    156.69318    151.53127    150.36179    144.38074    144.18449    141.38799    138.11365    139.74211     138.7318    138.69919    140.52183    130.32193          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       64.00    316.45182    307.07607    281.02312    271.38424    264.81328    264.53324     250.0767    242.63687    236.72144    227.33766    221.07532    209.68769      201.436      194.471    196.84145    200.31025    192.07736    194.65464    183.77006    188.19566    190.30852    193.25545    187.25207    188.04304    196.09414    199.55629    192.14533    190.93777    199.15188    183.77202    184.06144     182.0497    179.12713    174.01506    174.20531    176.92107    168.92818    170.49075    175.52688    171.47351    170.84577    162.07641    169.47785    164.23848    172.21638    159.50832    164.99042    161.82646    161.12306    158.29794    161.04226    163.44302     157.8834    161.37031    156.89061    161.99743    173.13462    163.79674    173.24945    170.46084    166.12762    164.35486    161.94719     158.3398    154.54516     157.0513    154.70865    156.18773    151.85327    158.29816    166.16375    174.09357    185.60881    182.39076    179.40152    179.67303    175.14567     163.9293    167.47597    159.74864    160.12697    155.94042    146.02161    151.37641    141.57745    144.13449    146.10667    135.54983    137.86488    137.09115    141.13345    134.06048          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       64.50    317.32734    304.64645     284.4065    274.47859     268.4465     261.4664    252.24366    245.65228    233.88368    220.00343    219.98596    209.28659    202.18002    195.29947    197.23452     194.6306    196.65374    196.02452    186.81841    185.62263    194.65284    191.95109    187.25747     191.7609    193.85722    194.48869    196.34283    188.97074    185.19694    181.33488    185.21887    173.83781    172.03059    174.72588    174.61019    174.67302    181.18386    173.25158    176.38471    171.83769    172.78935    169.70665    172.67277    167.79098    170.43467    169.38549    162.81641    166.20334     160.0566    161.56181    155.72681    162.39654    160.37109    168.86536    163.54469    156.26054    167.02198    173.52088    168.95125    168.55485    165.59738     166.3879    160.50313    157.44776    151.71067    154.49137     151.0212    160.95585    152.76614    157.93518    165.16606    171.15087    184.96395    186.57155    182.31116    178.95625    168.99254     162.0379    159.18577    158.75142    149.84587    155.63366    158.12981    150.63136    144.33831    143.85696    146.38347    138.23806    143.48787    136.80857       135.71    127.78551          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       65.00    316.66955    302.07578    284.39699    277.49052    267.30074    257.43333    255.16994    249.58543    231.75874    216.06455    221.15711    214.12376    205.81765    195.72441    195.64837    192.98474    199.98334    197.07074    192.04264     192.6494     193.2987    187.30713    190.05133     194.0182    193.62726    192.08299    197.63129    194.88772    186.88344     180.9473    187.35959    165.16962     168.8517    173.59541    177.81423    168.06692    171.31246    171.86588    172.90725    171.08136    172.08377    176.39512    168.83482    164.29163    163.49737    165.08306    163.09261    160.87876    165.59573    161.22036    159.79581    167.23396    156.84078    160.39809    165.40845    168.25193    170.81212    169.64887    169.35809      172.973    165.45772    166.46248    167.00992    153.04533    158.26813    155.27203    155.01292     157.6403     154.0659    164.21099     169.7756    176.71027    176.70721    184.94484    177.04401    176.96231    169.15828    165.89257    161.07618    157.31185    160.56126     156.8605    152.41148    151.96813    148.02644    144.53198    138.41963    135.58201    136.74523    137.53653    134.27631          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       65.50    315.10919    303.32728    283.09668    280.17408    262.86401    253.93027    256.68007    249.86638    235.87419    223.06475    223.19856    217.53899    205.44881    201.32727    198.53069    195.77227    199.84033    198.04668    193.00786     194.6838    190.11941    192.36151       183.71    184.54111    187.86136    187.60344    196.46293    198.35956    187.04354    179.19835    173.73765    173.02318    175.73547    173.94305    181.66538    175.99305    172.01237    174.01839    175.79447    170.27052     175.8782    172.64089    170.34431    170.75618    163.92281    167.13142     171.1266    164.84571    162.55491    162.28368    159.29486    157.54624    156.09104    161.39497    166.28236     166.8149    171.38487    170.31068    168.52395     171.8434    170.14685    160.35575    163.24342    157.67789    154.58533     158.5505    152.35196    158.29327    161.86775    156.78124    174.67097    181.38993    181.42525     182.8371    179.01679    177.06101    168.71257    167.59857    162.18665    153.85048    154.43266    154.28738    156.43825     154.1856    148.78162    145.88662    140.50082    136.63268    136.85968    132.95105    120.07499          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       66.00      314.712    304.43193    284.54646    282.09464    262.21053    257.56101    252.62782    247.13709    242.61632    228.03501    221.51846    215.06059    205.09417    214.97854    199.61036    198.01121    196.09364    198.22606    192.53505    197.95313    187.77315    195.69264    182.73121    186.30213    189.66888    201.36963    191.69788    197.66033    193.68757     186.8699    170.77667    176.15833    175.51564    176.58098    172.73174    178.80257    175.34637    173.90709    174.81285    174.87178    173.47097    177.79586    175.80094    173.10245    163.83059    164.59937     167.3219    164.03268     163.9426    164.23083    156.66382    161.91331    163.22695    161.89127    166.35319    161.95695    170.03468    173.92209    175.73056    159.86861    172.16323    166.04799    158.14397    159.33659    159.84589     151.8692    153.17554     155.2643    162.69446    165.26085    165.96967    174.41875    185.00668    183.48678    184.44345    177.47206    169.03147    160.97205    160.30842    157.78888    156.50116    154.92466    157.57335    153.11755     144.3008    147.84955    140.25083    146.94356    136.45272    128.94482          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       66.50    315.80086    304.65808    288.16841    280.68955    264.84559     261.1515    247.19625    243.50573    242.52946     230.4694    219.37101    213.71271    208.18714    217.07988    197.65611    198.59341    196.34909    199.96423    191.06877    196.45442    192.08092    197.56039    185.48949    190.06974     190.7402    197.86246    195.08524    205.31991    198.37988    191.20417    181.43144     181.2911     180.4198     176.3069    176.45374     178.4146    173.61674    168.54647    175.40867    178.17391    175.54224    174.64948    176.63385    169.71244    167.51998    166.34136    166.03399    165.00622    168.15971    161.75336    153.95685    167.10418    161.00195    161.73772    160.98334    166.11629    161.49489     178.3958    180.73048    176.69855    167.92344    159.13947    159.91571    159.38929    154.02689    157.38203    154.10938    155.19205    160.84746    162.16644    170.78458    183.83554    183.71452    187.56782    187.13842    179.56924    172.85821    165.03295       156.55    154.01532    152.68729    152.44891    150.71631    151.31744     144.4882    146.78784    139.56359    137.09299    131.53698    119.84925          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       67.00    317.21083    302.56369    290.14746    279.87486    267.14274    261.46505    248.22914    240.26863    236.20169     230.0177    221.44274    216.39112    209.77843     209.3899      197.203    204.06814    199.01138    199.79203    189.18033    193.52069    193.87568     195.5781    180.62875    179.62484    190.27862    194.31394    191.16817    199.58092    190.65595    189.95687    188.82358    182.17402    178.55617     178.4577    170.10143    174.36336    171.76357    170.71695     171.5026    174.81538    172.67725    168.62679    174.25197    169.64252    170.55763    173.96251    168.26218    161.54862    164.27543    164.21177    164.98423    160.86294     160.7039    164.85812    161.17072    159.92169    179.91382    169.51367    176.90206    173.77425    169.25622    165.51169    158.56291     160.4543    157.56621     153.2885    152.76883    153.31232    155.42322    162.02615    172.85253     180.0741     184.1292    190.88384    195.94288      181.665    174.47961    167.37572    164.85946    154.28892    152.22338    151.01006    151.77566    148.65178    147.02644    148.86429    139.97195    137.06745    138.76386          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       67.50    319.59981    301.25035    289.73456    279.57633       269.11    259.25078    250.08023    237.85949    232.73385    229.12883     224.8178    216.91011    210.87497    209.57389    203.18882    213.99143    199.05449    197.70386    189.07676    195.00835    192.82492     192.6545     184.8297    186.47955    191.71821    184.33574    187.16142    195.66641    191.39429    189.88994    185.24815    183.68277    177.74485    178.24877    166.60317    170.28677    172.12244    172.91691    174.15988    167.11005    176.15569    171.30788    174.98321    161.22693    172.49699     166.4931     166.9445    163.09231    166.66055    159.60703    160.97861    160.29746    160.19732    161.73178    160.07759     167.4208    167.74767    173.00788    178.96654    174.31876    176.69702     173.0613    160.56503    161.12929    158.64687    145.81991    155.22987    156.80622    156.56452    159.66213     169.3415    176.34852    192.18067    190.53052    188.43188    183.57354       175.33    164.12035    165.91721    160.81474    153.82153    155.51842     149.9817    148.94205    146.40052    144.72667    141.88079    135.20482    124.63597          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       68.00    322.02594    299.86915    288.73196    278.91797    271.67762    258.84741    249.88605    240.60059    233.57374    225.59341    223.51392     217.9763     211.7085    208.63817    206.17588    212.85576    202.29659    189.21573    192.45809    193.12044    194.42907     191.8213    190.63892    188.12958    191.89498    191.74458    195.70946    192.08835    193.76122    192.93715    186.90384    183.42995    180.37422    175.90965    171.63906    169.50516     177.1975    171.85877    172.66284    168.86143    180.27524    175.95293     167.4679    164.66729    168.31919    167.59333    168.42695    169.76449    171.98976    167.90245    160.66186    163.54154    156.57823    165.00848    162.97611    167.99136    170.70392    174.65368     174.6415    175.50059    175.52236    172.99744    159.48716    156.90879    159.63238    154.65014    157.41063    152.17724    151.27766    164.32205    165.28388    179.21076    186.77491    194.83868    186.71296    185.76163    173.25443    174.98308    162.64749     156.6892    161.33138    149.29874    146.88825    151.42632    146.07512    143.55619     138.2337    133.67559          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       68.50    324.72979    303.10263    287.93281    280.48845    272.91655    258.96853    247.65597    241.37157    233.07048    225.60621    220.63377    216.71836    212.19849    210.71385    213.42874    211.77029    205.58982    196.16863    190.50796    186.54358    193.83509    194.63963    198.71664    180.51268     187.9631     195.1482    195.31217    188.30559    193.70534     185.9026    188.08209    175.56557     181.1747    175.73305    179.36988    182.98606    181.55846    171.99635    171.75075    174.92669    181.13876    167.95522    171.61908    173.65198    167.77263     169.7698    167.46315    171.15619    171.10218    164.35419    164.31626    161.32608    163.11612    163.87626    160.27937    167.12695    163.24554    174.80499    175.83735    169.79556    181.98997    173.52096    163.02411      158.333    158.73583    151.59429    159.51717    156.26782    157.92871    163.43955    167.02457    178.77717    185.73935    186.52132    187.67941    183.02515    176.10741    165.06249    160.08455    153.86852    154.57433    162.94268    154.38805    155.33955    144.31036    140.19713    137.03083    122.95588          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       69.00    329.66753    306.49986    287.28099    283.03435    271.33006    256.11363    245.71061    243.88073    232.06296    231.74085    222.52391    211.21716    212.50735    217.49212    213.17475    205.72902    206.83059      206.433    194.85735     186.8844    187.74709    194.31147    196.65389    184.52815    191.36271    191.81109    197.63381    184.65261    192.16016    181.36784    186.64291    193.74185    181.80806     177.7492    179.02318    177.88796    178.49349    172.43118    169.16985    172.47399    174.71853     168.1073    169.23249     169.1733    168.64889    169.95034    170.56959    164.48137    166.33356    168.20649    169.56467    170.36211    165.39849    164.45457    161.48752    162.46508    170.37232    167.37616    173.40336    174.02225    169.20985    170.47764    163.71373    158.74455    165.24015    161.05608    158.09769    158.93183    162.43299    162.95595    172.88378    183.30582    190.23989    186.13334    193.21208    188.38377    176.24064    170.23178    161.11764    160.17559    158.00575    154.16994    154.66313    156.57145    146.69179    139.94517    137.22852    123.49974          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       69.50    335.88931    309.58427    288.12205    282.63743    269.64002    254.33672     245.6475    249.31474    233.19561    232.71773    226.60032    206.91512    211.86362    213.68715    200.98333    200.86507    206.64825    206.93431    204.86425    188.72631    183.51067     189.3939    190.88167    187.39961    192.11574    195.29049    199.09783    191.17094    192.85199    184.06119    187.45516    182.54472    176.33676    181.52048    181.61114    175.46683    179.43538    177.02878     172.8875    173.43324    177.65591    167.28602    161.88313    161.06671    168.21267    169.01749    167.26832    168.62798    168.45239    165.95416    171.42065    162.29746    170.21388    166.44129     165.8295    162.23126    169.68523    174.27743    176.10163     174.8605    175.30074    171.26495    169.80204    163.13085     163.1996    157.68405    158.93068    159.30387     161.3277    168.93167    172.30749    180.01253    182.68134    191.39574    189.39416    180.28977    179.37006    168.04788    163.83955    161.46562    150.70508    153.91378     154.9807    152.96039     145.3411    141.43195     138.8065          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       70.00    342.46649    311.54906    290.29532    279.54596    268.39907     255.0761     246.9687    246.12498    233.84161    230.09687    226.36149    210.41089    207.84955    210.05705    200.30269    206.75948     207.4055    206.91044    202.71188      201.316    183.51166    187.16079    190.10962    186.45761    193.44465    186.90119    189.25299    198.54298    194.00984    191.08765    192.33703    180.06863    182.90762    183.16816    177.93204    179.74193    176.12718    170.37474    176.86076    172.40905    164.40541     173.3742    170.22413    169.75782    169.67747    170.78959    174.11132    170.64539     167.0814    164.99262    168.30735    170.75307    167.61238    171.46398    165.27656     165.9168    173.51509    169.34733    178.27484    175.60807    180.98437    168.99163    169.53543    159.86973    156.60787    157.45819    156.82549    157.38871    166.46058    168.26639    174.28111    177.11868    192.16351    192.67913    187.76718    183.95661     176.8689    174.71287     162.9784    152.43937    155.98696    160.35695    151.27932    146.34858    147.70984    147.90921    131.83419          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       70.50    336.25498    313.62914     293.7052    277.35917    267.67702    256.40905    249.59927    240.29582     235.7125    225.43198    220.10561    215.11246    206.10594    209.19649      202.986    209.80492    205.71258    208.92313     202.6355    200.13287    196.51932    185.53079     188.7474    184.34822    189.70257    185.40047    190.40972     198.9043    192.33421    186.53101    191.95703    184.50824    182.77985    173.50329    172.35536    179.14616    180.62565    181.43037    172.71943    174.84837    168.27879    170.51252    172.11385    162.56534    175.53116    167.53709    169.92096    166.69913    158.48987    173.70624    172.45316    166.06254    166.98441    169.10857    170.39577    166.26325    176.76169    181.44603    178.85059    175.50746    168.34175    173.23792    169.07042    158.37655    159.96063    162.08316    162.72706    166.03749    166.67589    161.37669    179.95526     179.6992    186.33525    192.08528    200.55886    183.59001     181.2958    163.96567    163.85715    150.18804    155.36094    155.56234    148.08237    148.92462    144.96133    142.21356     122.7819          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       71.00    330.57108    315.49383    301.54195    276.05367    267.63875    256.01547     252.4674    239.50687    233.52576    224.01835    217.87248    215.85435    206.31142    209.08073    206.99252    206.73511    203.66606    206.78748    203.09526    200.40521    200.14414    202.86901    186.93876    184.74276     188.8237    186.80395    189.35025    197.05451    190.83948    185.49793     187.1014    179.94443    177.23749    187.78838    170.96176    178.02692    180.82444    183.83318    175.39151    171.30096    177.82405    169.75074    174.91442    173.56433    175.40523    168.50512    168.69725    168.73993    167.15343    164.65493    166.42245    165.94143    169.87042    163.01027    171.06893    165.63683    177.46053    174.97841    185.75955    184.66323     176.2152    172.70412    164.41768    160.18832     162.6887    157.40351     164.4725    159.16961    159.52742    157.00552    169.38124    181.56401    193.22765    190.54465     191.5128    188.49286    181.36872    164.47673    159.00833    158.61706    152.38807    153.47797    145.82634    144.21743    147.45801    135.79307          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       71.50    326.10151    312.82337    307.99011    277.13223    269.53052     257.4881     255.9267    240.89554    234.45511    225.06179    220.81039    215.35155    212.26236    209.36802    210.48695     204.8689     210.9218     200.4891    204.10678    197.58191    199.04188    194.86834    188.94731    190.39365    189.57891    184.32726    187.09963    195.91933    185.64274    185.06028    189.63746    177.47523    175.17654    187.78403    177.05538    180.65882    172.81901    184.82286    181.57931    175.74049     179.8192    172.70916    175.60687    171.57331     173.3258    166.49017    165.54142    166.26561    171.60176    167.99453    165.73717    164.38319    169.25384    169.29406    169.21649    171.25069    166.59056    179.89676    178.89553    178.10526    174.08389    175.50334    170.59271    160.51647    159.95439     161.5899    161.65153    164.16341    161.49659    165.89786    175.99907    176.98043    189.05913    195.79966    193.25463    186.78058    179.15567    168.81377    169.84471    159.84109    158.71751    157.85838    148.52447    144.44091    138.61234    134.04218          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       72.00    325.57505    310.55683     311.0904     281.9215    271.66909    257.11825    260.01321    240.49479    234.85168    226.23534    221.55282    216.06473    217.42328    208.99102    205.71998    204.89488    213.96853    193.16978    207.98234    198.48863    193.90929    191.19117     191.7465     196.1616     192.8477    197.97536    191.57853     189.5748    181.92968    179.98429    189.20103    176.36335    180.85094    181.97213    188.21243    180.90168    175.44782    178.66962    182.24952    181.54529    183.95432     171.0082    177.70305    179.19386    172.27374    170.15337    166.90868    170.28951    168.73134    167.11111    173.17534     160.3147    173.62886    170.56057    162.21347    171.05567    167.87031    176.08085    181.83253     184.8586    179.45609    179.14142    173.81038    168.02319    167.15172    162.07155    154.58539    153.93248    162.58482    172.19824    176.90841    184.00188    192.54517    191.34163    192.36768    183.56812    182.43111    167.53295    165.09805    160.37967    156.80332    156.73482    150.95829    148.88566    143.46466    132.71779          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       72.50    325.26956    309.98275    305.55085    285.60027     274.8828    254.09745    261.92293     240.3086    230.74734    225.88098    217.52172    215.78138    213.50873    207.91206    204.45266    206.46813    215.41058    196.46728    206.58557    201.81177    192.86772    201.60482    195.26604    199.23024    202.37403    201.87693    191.42527    188.49236    184.25534    191.27094    184.56622    188.89326    175.81604    181.60126    180.96785    174.70362    179.54422    185.98904    181.43222     176.7359    179.57672    170.51127    174.43014    178.15408    174.67056    178.61314    173.20592    166.61817    160.66868    168.74363     171.0685    170.43716     170.6709    168.11206    172.35268    178.59034    174.17034    184.27764    187.84025    181.45964    181.31984    180.10378    174.03056    170.25399    167.04643    164.43128    166.31113    172.39858    158.67121    162.75453    165.94749    185.90237    189.46054    193.95775    193.75712    186.67751    176.26807    173.79631    165.08176    161.07483    156.14952    158.25131    156.30698    148.98275     138.7012          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       73.00    325.28511    313.37468    299.99554    286.78612    278.13257    253.49885    256.62572    240.38822      231.925    223.55542     214.4069    214.98113    211.34345    209.12027    207.84208    204.01123    206.66819    200.91739    199.89789    198.64495    193.85365    202.89773    196.52348    195.96909     193.4723    203.49229    193.98268     191.6687    189.08355    186.21422    186.60129    187.10714    178.99086    178.81312    180.67466    179.30114    182.93063    170.64946    180.25924    177.67482     177.4612    175.88816    176.11155    170.20195    177.35191    178.56991    175.43143    173.43275     168.5372    164.65859    161.40681    172.65755    171.64944    165.34376     170.7267    168.56999    172.02162    178.21708    186.67824    183.85726     183.2137     179.1262    167.32813    171.03761    169.81292    163.60054    166.51347    164.51508    167.94608    165.08678    170.11292    179.68288    192.95247     191.8768    196.03782      190.603    178.01084    166.10924    163.17829     158.4503    156.49025    152.03472     154.2943    149.39566    138.84862          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       73.50    328.57268    315.39118    298.04806    288.09233     280.8258    263.16805     251.7704    237.36164    236.61161    224.90528    222.72173     215.2023    214.36067    207.87555    209.24074     204.1057    199.81213    204.22142    190.10522    201.32474    194.65316    199.58765    194.70361    189.29743    189.44998    197.04618    198.29479     203.5372    191.94275    183.27187    188.68759    179.36512    173.08599    178.48146    184.43953     179.8779    177.28527    166.64473    181.03297    173.75048    174.62867    176.08493     178.6761    171.96856    172.80825    174.39208    171.96317    170.77028    173.07058     170.2792    167.61991    169.43792    150.41941    170.09209    169.78586    160.34868    169.88168    175.65466    179.15254    180.58267    180.66946    178.14848     172.9352    169.99828    164.23173    163.02997    159.37224    165.89396    165.89391    172.99054    179.13986    184.16625    189.83395    193.33832    187.47738    182.87105    183.17969    174.00927    167.73691    157.38425    164.60407    160.33756    154.83327    147.90342    134.85799          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       74.00    333.26914    315.97559      299.587    290.74226    281.38214    273.07944    254.23633    234.80003    237.46935    227.02651     229.5985    218.60551    215.85215    206.95942    210.55909    206.74806    199.13955    204.65769    194.42181    195.92213    194.68393    199.70632    197.23984    199.79716    194.83101    200.32865    199.26413    204.96299    198.09746    185.41511     184.0049    179.63306     188.7977    178.67722     181.7969    176.15889     176.5805    182.27388     176.2612    178.78494    177.72333    176.95151     182.6842    173.36352     179.5546    175.13441    176.61602    173.54082    169.70892    164.63476    173.07724     168.9251    171.44963    165.98953    163.46474    167.50073    181.87217    171.54898    184.44926    181.06437     177.8439    175.39385     176.9895    173.07214    161.69991    165.78617    172.21089    163.92166    169.04156    166.38219    178.69968    183.35023    200.07786     201.2807     198.1432    184.50912    182.95903    169.22065    162.06823    159.10404    158.05138    154.60656    156.29208    148.26193          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       74.50    338.70246    316.38108     301.9544    295.17335    281.89866    275.88084    259.64638    240.22603     235.9133    226.01416    228.32213    220.10135    217.41302    208.68921    205.57933    208.90142    199.78853    204.67817    194.08616    194.08849    196.12636    198.56899    204.04576     202.8681    201.30132    197.95814    196.66281    201.88136    199.48531     190.9257     187.4896    190.80315    178.55657    182.19197    181.43966    178.37919    182.76292    173.84635    172.36923    177.39342    175.75263    175.11969    174.14886    175.40758    176.58894    173.89542    168.08594    174.45185    167.41794    163.85395    171.07831    175.56849    174.91461     171.0172    166.18026    173.43303    176.58154    177.36014    177.79732       178.25    177.24677    179.05441    169.02236    168.43629    172.69887    164.98218    158.64762    168.17065    167.61079    166.60389     174.7811    186.50758    191.43772    204.32374    200.46832    198.50338    186.36694    177.34738    164.34274     161.3007    159.05162    153.93197    161.22313    141.41204          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       75.00    333.78545     317.3767    301.89615    298.59246    283.83693    274.66946    261.49663    250.96076    236.28721    224.61476     227.2543    218.15595    217.74747    209.23648    203.85994    200.50573    201.42914    209.10828    194.15594    197.90865    196.54974    196.59097    195.52087    203.65122    200.22913    196.83935    198.43019    204.03324     198.3315    192.91009    193.66391    194.73898    178.65616    179.14996    181.83948    192.49449     178.6206    186.48778    175.67335    175.64484    175.52238    176.83643    175.46375    170.61317    175.19122    177.03869    174.58888    176.17746    177.47917    172.75375    168.97001    166.34645    172.02904     171.6561    175.07216    171.62172    170.31196     174.7871    183.42601    176.25049    178.06443    180.33462    173.05149    169.60654    171.96045    168.13926    167.56828    164.40271    158.60167    171.61131    171.55305    190.41411    190.96625    202.15075    206.36759    194.37581    189.66069      180.684    169.86892    164.25696    161.62863    157.35729    152.44086    143.46587          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       75.50    328.11906    313.65111    301.82422    297.87875     286.8551    270.71882    257.27674    258.98419    239.01182    226.49185    225.15301    215.65318    210.69202    208.19025    203.27943    196.72548    201.69975    211.84065    199.55904    200.77722    193.85607     194.3265    190.29106    206.56322    193.18676     203.8202    200.51279    207.35339    193.34834    197.50092    196.00496    189.85798    189.25727    191.25649    183.35106    182.30529    176.64894     180.7536    172.79219    175.53002    177.20598    181.00745    175.18349    178.23891    177.95307    178.69343    169.55556    172.52715    175.76305    172.19649    171.09787    176.57162    166.90918    170.50775    175.99427    175.95311    183.45365    180.57411    179.67251    179.76191    180.55277    172.95094     165.6119    172.30873    164.49971     165.6466    165.53215     167.0259    168.05673    166.69863    170.17847    186.20037    197.05293    200.05487     199.9118    201.17638    190.23308    182.47925    170.09087    164.95285     164.8606    161.18375    149.54412     133.2902          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       76.00    323.77861    306.67711    301.61279    293.58531    288.54629    265.84026    253.07787    262.67774    241.04411    230.82958    224.55003    214.19104    207.33612    208.74829    202.52562    202.73331    201.44893    208.42023    200.12675    193.50969    194.36476    203.04996    195.39666    193.84797    194.49833    207.37543    206.45033    197.95039    193.22965    192.55832     191.8283     198.2491    193.34457    184.51565    187.60699    185.36258    185.16899    176.85083      176.583    173.03039    180.00967    172.59166    172.31325     174.5377    174.13591    181.31474    178.11056    170.17695    168.37281    175.96904     176.5929    175.11566    168.19435    172.74459    169.45378    168.94776    182.19902    182.81726    181.26034    186.18015    181.63563    183.70858    175.54672    165.85829    167.99977    169.02106    168.89437     165.7013     163.7729    169.71056    176.77304    189.85935    194.96806    200.88042    201.28873    206.72112    194.22669    179.55789    169.74947    160.61815    165.65655    165.77868    153.68714          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       76.50    326.16089    300.19385    301.00073    289.79928    284.63442     262.5525    257.24762    265.22253    246.16053    232.16866    232.56293     215.8216    212.32249    206.30193    201.96681     207.2531    200.82637    205.34611    196.37268    196.81376    203.50994    201.86063    200.65909     194.7865    205.75573    201.25141    206.82076    193.68115    198.65137    197.81005    189.01254    197.08523    193.92799    176.37078    186.28375    178.24091     181.8551    185.18017    182.79228    180.97646    176.67998    174.90284    177.76143    174.53209     177.3929    168.14194    174.21954    176.32363    169.67481    176.99322    169.46723    178.04157    167.08721    172.60485    171.86207    173.99328    174.63799    176.06162    174.45115    183.56647    182.56278    171.62524    174.95039    169.13411    167.68371    162.62763    167.30684    162.18397    162.19077    173.46295    172.10712    187.64224    195.23711    196.74518     200.0055    190.10236    197.49625    179.77751    178.79161    167.88609      162.128     165.9172    150.02298          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       77.00    328.43004    300.15265    300.72602    293.61649      280.626    265.50122    260.97327    260.98913    250.61654    229.43602    239.95617    218.25438    219.49931    200.14092    202.39353    207.15453     201.2996      201.725    194.55666    201.97947    197.10821    196.40459    192.55775    197.82811    205.54689    195.81464    198.11351    196.18727    198.37549    198.58327     190.2682    194.59667    185.93672    182.40361     189.5669    191.94446    175.09063    185.38027    177.69495    183.94421    171.77547    177.30703    176.02925    173.01072    174.71382    177.81698    170.94166    176.39707    174.14426     172.8607    168.43122     172.6174    174.40115    174.24064    174.54025    172.69718    176.28643    185.36719    179.85012    179.32406    187.78655    180.58906    180.57037    172.89866    169.60694    162.49171    170.83739    164.68554    159.67694    170.31329    167.00243    182.71979    188.95628     194.5813    193.63764    194.26284    198.60032    180.78738     169.5553    167.45218    172.37364    161.46251    138.27083          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       77.50    329.89905    301.76582    300.11916     298.3189    278.66692    269.18746    263.68315    255.50241    243.45256    230.06159    244.43346    224.44347    227.45937    207.73916    205.20428    210.36529    203.10549    196.72062    194.77289    194.45536    192.15591    199.53921    195.39404    194.74668    199.15199    192.76899    199.13106    199.61734    196.60308    197.97557    190.63194    189.90374    188.83531    186.67504    180.96751    189.82586    185.36485    187.15151    183.78393    189.41988     179.4292    175.96345    184.12843       171.03    171.58805    176.69352    177.85864    173.54541    168.81926    172.13146    177.92929    171.21687    176.04315    168.89268    178.14528    174.47498    187.90585    186.34145    181.79126    184.53434    188.17192    181.67266    178.88904    182.11768    167.97172    163.49041    167.96211    173.94529    171.89914    171.40489    179.95838    178.58258    200.55002    197.42758     194.7326    195.56624    192.40831    182.09005    168.42812    173.42353    165.05621    158.13497    141.53445          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       78.00    327.77384    302.75861    298.64001    298.95262    278.30909    271.19903    266.37253    252.61222    238.10903     233.9243    243.62814    234.78331    226.73626    224.50456    218.90047    216.19555    202.49681    195.62442    203.32789    189.98806    198.00096    196.86491    199.04381    187.59998    192.56803    191.63962      205.624    198.83681     204.9318    193.60948    193.51743    191.80003    183.90922    185.99871    189.01855    185.12244    188.29014    184.61091    181.66023    182.02479    181.57117    189.12952    184.89248    189.76183     183.1826    175.69824    176.13006    181.17874    174.82995    170.67874    168.01898    171.52669    173.94407    171.90944      173.358    172.90322     170.9177    172.44372     186.6348     176.1531    180.96207    179.76162    177.68553    175.88156    171.01515    170.71142     170.3901    167.84634    170.44192    177.73024    176.66551    187.05153    202.10645    199.05697    194.59588    193.52413    187.32873    188.23967    173.02107    166.83298    165.63805    152.61568    138.50925          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       78.50    325.68766    305.57293     296.7996    291.69738    278.73739    271.87199    264.13823    252.18682    243.59944    238.50725    238.00757    234.57606    223.42111    218.34851    227.42735    219.44744     206.8563    205.84033    205.06703    191.71119    193.48006    192.73245    187.87642    188.61478    193.17748     196.2836     210.4643    194.42482    204.65261    181.47271    185.93276    195.54188    182.71816    183.79549    185.08865    186.45971    192.76136    179.87162    179.33925    185.25565    177.90164    177.31509    182.76487    181.16271    191.21841    181.68058    179.54119    179.79565    173.55719    167.93588     170.8511    170.03427    176.01439    171.45801    180.19784    172.11134    174.36652    181.88928    183.03019    185.18631    183.09103     177.9672    173.44593    176.24937    173.70336    174.74524    173.98814    166.29396    171.96641    177.17629    181.05997    190.67122    195.89686    206.27286    204.19561    204.51612    196.67475     190.3921    172.15192    165.05511    166.28717    156.25845          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       79.00    323.28084    309.71782    298.45795     283.7686     279.1864    271.06153    259.39519    250.47419    246.39479    247.69577    236.90731    228.64436    226.83259    210.87964    220.59092     221.1441    214.79167    211.04192    204.02496    200.40516    197.80693    194.04798    191.11861    194.02037    192.93583    201.45217    202.07228    193.17347    193.71436    199.23427    182.38986    191.06713    182.04056    189.94442    190.86168    184.53499    180.66761    188.77043    185.67426    181.71099    178.30412    183.73938    184.73415    179.53619    181.09677    180.61793    177.48508     180.3677      181.429    173.92565    177.68362    175.81218    171.06376    173.26394    167.92678    175.31265    183.05941    174.41947     178.4851    187.53202    176.05575    180.48597    178.14619    175.03782    173.99945    171.38797    171.73038    177.27858    175.76584    169.75444    180.69121    186.01385    193.41703    203.17117    202.77978    202.29362    194.47905    184.37455    176.66559    170.84205    169.77881    155.70757          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       79.50    319.37548    313.80743    300.77647    284.82934    279.90879     268.6283    255.91752     246.6061    242.86224     251.3163    239.36596    229.44858    231.01638    218.98385     219.1597     223.9462     215.7954    213.81729    205.04399    202.81683    205.49716    203.33624    194.37607    191.33617    189.57188    201.20265    192.47985    194.89947     195.7964    195.09822    187.85151     182.3021    189.27576    189.47903    191.18782    181.53422    172.18591    205.78854    187.17018    188.64182    183.33067    181.61219    191.49827    183.45024    172.58369     180.6002    178.96375    173.48583     181.8529    188.23274    175.62936    178.68158    180.18875      174.257    170.52761    176.46881    173.06991    177.40237    177.59364    189.03177    179.17539    180.74777    180.09924     176.6545    170.79689    167.37752    172.38537    168.67482    167.79192     167.5967    186.11422    187.37193    195.60734    196.38441    197.27556    206.43464    197.43434    189.34743    177.05677    173.87478    171.17595    152.54078          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
       80.00    314.89655    311.93284    302.23633    288.96778    280.63817      266.383    256.06991    247.37919    241.99261    242.81396    234.71539    233.14403    223.94949    221.61584    222.29351    219.09454    218.90914    216.95356     206.6919      203.339    204.49564     204.2015    188.68998    193.82666    191.33261    191.85556    191.22701    194.88224    195.69085    192.21096    192.82049    188.66766     191.6543    189.69636    183.16058    188.26607    182.91685    190.79336    188.52857    181.67729    187.36786    183.28625    185.36939    182.66168    181.35918    177.79748     180.8068    175.74051    178.19937    176.75559    183.89576    181.11285    172.31092     177.5136    168.42765    178.26887    183.30864    185.88452    189.94301     190.9711     183.4484    182.33483    180.17363    179.39993     176.2402    177.15156    171.94513    170.21035    175.51771    167.71268    172.67298    182.76808     199.9072    208.69916    205.10645    198.59024    194.57679    183.17698    176.32628    168.93547    162.25132    150.94693          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan          nan
;
    """
    return s_out

def template_for_crystal():
    s_out = """
data_Fe3O4            
_cell_angle_alpha 90.0                    
_cell_angle_beta 90.0
_cell_angle_gamma 90.0
_cell_length_a 8.56212
_cell_length_b 8.56212
_cell_length_c 8.56212
_space_group_it_coordinate_system_code 2  
_space_group_name_H-M_ref    "F d -3 m"

loop_                                     
_atom_site_label
_atom_site_type_symbol
_atom_site_fract_x
_atom_site_fract_y
_atom_site_fract_z
_atom_site_adp_type
_atom_site_B_iso_or_equiv
_atom_site_occupancy
 Fe3A Fe3+ 0.12500 0.12500 0.12500 Uiso 0.0  1.0 
 Fe3B Fe3+ 0.50000 0.50000 0.50000 Uiso 0.0  1.0 
   O1  O2- 0.25521 0.25521 0.25521 Uiso 0.0  1.0 


loop_
_atom_site_scat_label
_atom_site_scat_lande
Fe3A 2.000
Fe3B 2.000

loop_     
_atom_site_susceptibility_label
_atom_site_susceptibility_chi_type
_atom_site_susceptibility_chi_11
_atom_site_susceptibility_chi_22
_atom_site_susceptibility_chi_33
_atom_site_susceptibility_chi_12
_atom_site_susceptibility_chi_13
_atom_site_susceptibility_chi_23
 Fe3A Cani -3.468(92) -3.468  -3.468   0.000   0.000   0.000
 Fe3B Cani  3.041      3.041   3.041   0.000   0.000   0.000    
    """
    return s_out


l_argv = sys.argv

flag = False
f_rcif = os.path.join(os.getcwd(), "main.rcif")
if len(l_argv) >= 2:
    f_rcif = os.path.abspath(l_argv[1])
flag = os.path.isfile(f_rcif)

if flag:
    if len(l_argv) >= 3:
        f_out = os.path.abspath(l_argv[2])
    else:
        f_out = f_rcif
    STR_FROM_CIF = str(pycifstar.to_global(f_rcif))
    print("Calculations are started \n")
    _obj = RhoChi.from_cif(STR_FROM_CIF)
    _obj.apply_constraint()
    d_res = _obj.refine()
    print(d_res)
    print("Calculations are finished.")
    _obj.save_to_file(f_out)
    print(f"The results are saved in file \n'{f_out:}'")
else:
    print("Input file are not defined\n")
    
    print("""
If you would like to create template for: 
1. single-crystal diffraction experiment        --- type: 1
2. 1d powder diffraction experiment             --- type: 2
3. 2d powder diffraction experiment             --- type: 3
4. 2d powder diffraction with one-axial texture --- type: 4
          """)
    s_answ = (input()).strip()
    s_cry = template_for_crystal()
    flag = True
    if s_answ == "1":
        s_exp = template_for_single()
    elif s_answ == "2":
        s_exp = template_for_pd()
    elif s_answ == "3":
        s_exp = template_for_pd2d()
    elif s_answ == "4":
        s_exp = template_for_pd2dt()
    else:
        flag = False
        print("It's not so difficult to type '1' or '2' or '3' or '4'. ")
    if flag:
        with open(f_rcif, "w") as fid:
            fid.write(f"global_ \n {s_cry:}\n\n{s_exp:}")
        print(f"The template are saved in file \n'{f_rcif:}'")
print("That's all")